/**
 * Combinatorical Function Prototypes for
 * Efficient Greedy Lottery Wheel Generator
 *
 * Author: Tyler McHenry <tyler@nerdland.net>
 *
 * For a more thorough description of the design and implementation of this
 * generator than can be provided in the in-line comments (with pretty diagrams
 * and everything!) please see "The Lottery Problem", a series of articles on
 * Nerdland: http://nerdland.net/the-lottery-problem/
 *
 * Licensing:
 * 
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * See the comments in combinatorics.cc for more information.
 *
 * Last Modified: 11 July 2009
 *
 **/

#ifndef COMBINATORICS_H
#define COMBINATORICS_H

#include <vector>
#include <stdint.h>
#include <sys/types.h>

void generateAllTickets(size_t ticket_size, size_t range,
                        std::vector<uint64_t>& outvector);

void generateAllCombos(uint64_t base_value, size_t combo_size,
                       std::vector<uint64_t>& outvector);

size_t computeStartingCoverage(size_t range, size_t ticket_size,
                               size_t match_size);

size_t comb(size_t n, size_t k);

#endif
