/**
 * Precomputed Bit Counts for
 * Efficient Greedy Lottery Wheel Generator
 *
 * Author: Tyler McHenry <tyler@nerdland.net>
 *
 * For a more thorough description of the design and implementation of this
 * generator than can be provided in the in-line comments (with pretty diagrams
 * and everything!) please see "The Lottery Problem", a series of articles on
 * Nerdland: http://nerdland.net/the-lottery-problem/
 *
 * Licensing:
 * 
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This is a generated file which provides a number of lookup tables used by the
 * functions in combinatorics.cc for faster calculations. These are:
 *
 * BIT_COUNT_16 : Maps a number to a count of bits set to 1 in the number's
 *                binary representation.
 *
 * BIT_COUNT_16_REVERSE : Maps a count of bits to an array of numbers with that
 *                        many bits set to 1 in their binary representation.
 *
 * BIT_COUNT_16_REVERSE_IDX : Maps a count of bits to the number of elements in
 *                            the array at that location in the
 *                            BIT_COUNT_16_REVERSE lookup table. 
 *
 * BIT_MASKS_16 : Maps a number to an array of bitmasks that each include a
 *                single bit that is set to 1 in the binary representation
 *                of the number.
 *
 * BIT_MASKS_16_IDX : Maps a number to the number of elementsin the array at that
 *                    location in the BIT_MASKS_16 lookup table.
 *
 * BITS_SET_16 : Maps a number to an array of the indices of the bits that are
 *               set to 1 in its binary representation, where the
 *               least-significant bit is index 0.
 *
 * BITS_SET_16_IDX : Maps a number to the number of elementsin the array at that
 *                   location in the BITS_SET_16 lookup table.
 *
 * Last Modified: 11 July 2009
 *
 **/

#include <sys/types.h>

const uint8_t BIT_COUNT_16[] = {
  0, 
  1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 
  3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
  3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
  4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 1, 2, 2, 3, 2, 
  3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
  3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
  4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 1, 2, 2, 3, 2, 
  3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
  3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
  5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
  13, 13, 14, 13, 14, 14, 15, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
  4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 3, 3, 4, 3, 
  4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 2, 
  3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
  5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
  13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
  4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 
  5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 
  4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
  6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
  14, 14, 15, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
  6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
  7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
  6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
  6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 4, 
  5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
  7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
  7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
  9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
  13, 13, 14, 13, 14, 14, 15, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
  8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
  8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 
  7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
  9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
  13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
  8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
  9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
  8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
  10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
  14, 14, 15, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
  10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
  11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
  10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
  10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 8, 
  9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
  11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
  11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
  13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
  12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
  13, 13, 14, 13, 14, 14, 15, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
  12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
  11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
  13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
  12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
  14, 14, 15, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 12, 
  13, 13, 14, 13, 14, 14, 15, 13, 14, 14, 15, 14, 15, 15, 16};

const uint16_t BIT_COUNT_16_REVERSE[17][12870] = {
    { 0, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 3, 5, 6, 9, 10, 12, 17, 18, 20, 24, 33, 34, 36, 40, 48, 65, 66, 68, 72, 80, 96, 
    129, 130, 132, 136, 144, 160, 192, 257, 258, 260, 264, 272, 288, 320, 384, 513, 514, 516, 520, 528, 
    544, 576, 640, 768, 1025, 1026, 1028, 1032, 1040, 1056, 1088, 1152, 1280, 1536, 2049, 2050, 2052, 2056, 2064, 2080, 
    2112, 2176, 2304, 2560, 3072, 4097, 4098, 4100, 4104, 4112, 4128, 4160, 4224, 4352, 4608, 5120, 6144, 8193, 8194, 8196, 
    8200, 8208, 8224, 8256, 8320, 8448, 8704, 9216, 10240, 12288, 16385, 16386, 16388, 16392, 16400, 16416, 16448, 16512, 16640, 16896, 
    17408, 18432, 20480, 24576, 32769, 32770, 32772, 32776, 32784, 32800, 32832, 32896, 33024, 33280, 33792, 34816, 36864, 40960, 49152, },
  { 7, 11, 13, 14, 19, 21, 22, 25, 26, 28, 35, 37, 38, 41, 42, 44, 49, 50, 52, 56, 67, 
    69, 70, 73, 74, 76, 81, 82, 84, 88, 97, 98, 100, 104, 112, 131, 133, 134, 137, 138, 140, 
    145, 146, 148, 152, 161, 162, 164, 168, 176, 193, 194, 196, 200, 208, 224, 259, 261, 262, 265, 266, 
    268, 273, 274, 276, 280, 289, 290, 292, 296, 304, 321, 322, 324, 328, 336, 352, 385, 386, 388, 392, 
    400, 416, 448, 515, 517, 518, 521, 522, 524, 529, 530, 532, 536, 545, 546, 548, 552, 560, 577, 578, 
    580, 584, 592, 608, 641, 642, 644, 648, 656, 672, 704, 769, 770, 772, 776, 784, 800, 832, 896, 1027, 
    1029, 1030, 1033, 1034, 1036, 1041, 1042, 1044, 1048, 1057, 1058, 1060, 1064, 1072, 1089, 1090, 1092, 1096, 1104, 1120, 
    1153, 1154, 1156, 1160, 1168, 1184, 1216, 1281, 1282, 1284, 1288, 1296, 1312, 1344, 1408, 1537, 1538, 1540, 1544, 1552, 
    1568, 1600, 1664, 1792, 2051, 2053, 2054, 2057, 2058, 2060, 2065, 2066, 2068, 2072, 2081, 2082, 2084, 2088, 2096, 2113, 
    2114, 2116, 2120, 2128, 2144, 2177, 2178, 2180, 2184, 2192, 2208, 2240, 2305, 2306, 2308, 2312, 2320, 2336, 2368, 2432, 
    2561, 2562, 2564, 2568, 2576, 2592, 2624, 2688, 2816, 3073, 3074, 3076, 3080, 3088, 3104, 3136, 3200, 3328, 3584, 4099, 
    4101, 4102, 4105, 4106, 4108, 4113, 4114, 4116, 4120, 4129, 4130, 4132, 4136, 4144, 4161, 4162, 4164, 4168, 4176, 4192, 
    4225, 4226, 4228, 4232, 4240, 4256, 4288, 4353, 4354, 4356, 4360, 4368, 4384, 4416, 4480, 4609, 4610, 4612, 4616, 4624, 
    4640, 4672, 4736, 4864, 5121, 5122, 5124, 5128, 5136, 5152, 5184, 5248, 5376, 5632, 6145, 6146, 6148, 6152, 6160, 6176, 
    6208, 6272, 6400, 6656, 7168, 8195, 8197, 8198, 8201, 8202, 8204, 8209, 8210, 8212, 8216, 8225, 8226, 8228, 8232, 8240, 
    8257, 8258, 8260, 8264, 8272, 8288, 8321, 8322, 8324, 8328, 8336, 8352, 8384, 8449, 8450, 8452, 8456, 8464, 8480, 8512, 
    8576, 8705, 8706, 8708, 8712, 8720, 8736, 8768, 8832, 8960, 9217, 9218, 9220, 9224, 9232, 9248, 9280, 9344, 9472, 9728, 
    10241, 10242, 10244, 10248, 10256, 10272, 10304, 10368, 10496, 10752, 11264, 12289, 12290, 12292, 12296, 12304, 12320, 12352, 12416, 12544, 
    12800, 13312, 14336, 16387, 16389, 16390, 16393, 16394, 16396, 16401, 16402, 16404, 16408, 16417, 16418, 16420, 16424, 16432, 16449, 16450, 
    16452, 16456, 16464, 16480, 16513, 16514, 16516, 16520, 16528, 16544, 16576, 16641, 16642, 16644, 16648, 16656, 16672, 16704, 16768, 16897, 
    16898, 16900, 16904, 16912, 16928, 16960, 17024, 17152, 17409, 17410, 17412, 17416, 17424, 17440, 17472, 17536, 17664, 17920, 18433, 18434, 
    18436, 18440, 18448, 18464, 18496, 18560, 18688, 18944, 19456, 20481, 20482, 20484, 20488, 20496, 20512, 20544, 20608, 20736, 20992, 21504, 
    22528, 24577, 24578, 24580, 24584, 24592, 24608, 24640, 24704, 24832, 25088, 25600, 26624, 28672, 32771, 32773, 32774, 32777, 32778, 32780, 
    32785, 32786, 32788, 32792, 32801, 32802, 32804, 32808, 32816, 32833, 32834, 32836, 32840, 32848, 32864, 32897, 32898, 32900, 32904, 32912, 
    32928, 32960, 33025, 33026, 33028, 33032, 33040, 33056, 33088, 33152, 33281, 33282, 33284, 33288, 33296, 33312, 33344, 33408, 33536, 33793, 
    33794, 33796, 33800, 33808, 33824, 33856, 33920, 34048, 34304, 34817, 34818, 34820, 34824, 34832, 34848, 34880, 34944, 35072, 35328, 35840, 
    36865, 36866, 36868, 36872, 36880, 36896, 36928, 36992, 37120, 37376, 37888, 38912, 40961, 40962, 40964, 40968, 40976, 40992, 41024, 41088, 
    41216, 41472, 41984, 43008, 45056, 49153, 49154, 49156, 49160, 49168, 49184, 49216, 49280, 49408, 49664, 50176, 51200, 53248, 57344, },
  { 15, 23, 27, 29, 30, 39, 43, 45, 46, 51, 53, 54, 57, 58, 60, 71, 75, 77, 78, 83, 85, 
    86, 89, 90, 92, 99, 101, 102, 105, 106, 108, 113, 114, 116, 120, 135, 139, 141, 142, 147, 149, 
    150, 153, 154, 156, 163, 165, 166, 169, 170, 172, 177, 178, 180, 184, 195, 197, 198, 201, 202, 204, 
    209, 210, 212, 216, 225, 226, 228, 232, 240, 263, 267, 269, 270, 275, 277, 278, 281, 282, 284, 291, 
    293, 294, 297, 298, 300, 305, 306, 308, 312, 323, 325, 326, 329, 330, 332, 337, 338, 340, 344, 353, 
    354, 356, 360, 368, 387, 389, 390, 393, 394, 396, 401, 402, 404, 408, 417, 418, 420, 424, 432, 449, 
    450, 452, 456, 464, 480, 519, 523, 525, 526, 531, 533, 534, 537, 538, 540, 547, 549, 550, 553, 554, 
    556, 561, 562, 564, 568, 579, 581, 582, 585, 586, 588, 593, 594, 596, 600, 609, 610, 612, 616, 624, 
    643, 645, 646, 649, 650, 652, 657, 658, 660, 664, 673, 674, 676, 680, 688, 705, 706, 708, 712, 720, 
    736, 771, 773, 774, 777, 778, 780, 785, 786, 788, 792, 801, 802, 804, 808, 816, 833, 834, 836, 840, 
    848, 864, 897, 898, 900, 904, 912, 928, 960, 1031, 1035, 1037, 1038, 1043, 1045, 1046, 1049, 1050, 1052, 1059, 
    1061, 1062, 1065, 1066, 1068, 1073, 1074, 1076, 1080, 1091, 1093, 1094, 1097, 1098, 1100, 1105, 1106, 1108, 1112, 1121, 
    1122, 1124, 1128, 1136, 1155, 1157, 1158, 1161, 1162, 1164, 1169, 1170, 1172, 1176, 1185, 1186, 1188, 1192, 1200, 1217, 
    1218, 1220, 1224, 1232, 1248, 1283, 1285, 1286, 1289, 1290, 1292, 1297, 1298, 1300, 1304, 1313, 1314, 1316, 1320, 1328, 
    1345, 1346, 1348, 1352, 1360, 1376, 1409, 1410, 1412, 1416, 1424, 1440, 1472, 1539, 1541, 1542, 1545, 1546, 1548, 1553, 
    1554, 1556, 1560, 1569, 1570, 1572, 1576, 1584, 1601, 1602, 1604, 1608, 1616, 1632, 1665, 1666, 1668, 1672, 1680, 1696, 
    1728, 1793, 1794, 1796, 1800, 1808, 1824, 1856, 1920, 2055, 2059, 2061, 2062, 2067, 2069, 2070, 2073, 2074, 2076, 2083, 
    2085, 2086, 2089, 2090, 2092, 2097, 2098, 2100, 2104, 2115, 2117, 2118, 2121, 2122, 2124, 2129, 2130, 2132, 2136, 2145, 
    2146, 2148, 2152, 2160, 2179, 2181, 2182, 2185, 2186, 2188, 2193, 2194, 2196, 2200, 2209, 2210, 2212, 2216, 2224, 2241, 
    2242, 2244, 2248, 2256, 2272, 2307, 2309, 2310, 2313, 2314, 2316, 2321, 2322, 2324, 2328, 2337, 2338, 2340, 2344, 2352, 
    2369, 2370, 2372, 2376, 2384, 2400, 2433, 2434, 2436, 2440, 2448, 2464, 2496, 2563, 2565, 2566, 2569, 2570, 2572, 2577, 
    2578, 2580, 2584, 2593, 2594, 2596, 2600, 2608, 2625, 2626, 2628, 2632, 2640, 2656, 2689, 2690, 2692, 2696, 2704, 2720, 
    2752, 2817, 2818, 2820, 2824, 2832, 2848, 2880, 2944, 3075, 3077, 3078, 3081, 3082, 3084, 3089, 3090, 3092, 3096, 3105, 
    3106, 3108, 3112, 3120, 3137, 3138, 3140, 3144, 3152, 3168, 3201, 3202, 3204, 3208, 3216, 3232, 3264, 3329, 3330, 3332, 
    3336, 3344, 3360, 3392, 3456, 3585, 3586, 3588, 3592, 3600, 3616, 3648, 3712, 3840, 4103, 4107, 4109, 4110, 4115, 4117, 
    4118, 4121, 4122, 4124, 4131, 4133, 4134, 4137, 4138, 4140, 4145, 4146, 4148, 4152, 4163, 4165, 4166, 4169, 4170, 4172, 
    4177, 4178, 4180, 4184, 4193, 4194, 4196, 4200, 4208, 4227, 4229, 4230, 4233, 4234, 4236, 4241, 4242, 4244, 4248, 4257, 
    4258, 4260, 4264, 4272, 4289, 4290, 4292, 4296, 4304, 4320, 4355, 4357, 4358, 4361, 4362, 4364, 4369, 4370, 4372, 4376, 
    4385, 4386, 4388, 4392, 4400, 4417, 4418, 4420, 4424, 4432, 4448, 4481, 4482, 4484, 4488, 4496, 4512, 4544, 4611, 4613, 
    4614, 4617, 4618, 4620, 4625, 4626, 4628, 4632, 4641, 4642, 4644, 4648, 4656, 4673, 4674, 4676, 4680, 4688, 4704, 4737, 
    4738, 4740, 4744, 4752, 4768, 4800, 4865, 4866, 4868, 4872, 4880, 4896, 4928, 4992, 5123, 5125, 5126, 5129, 5130, 5132, 
    5137, 5138, 5140, 5144, 5153, 5154, 5156, 5160, 5168, 5185, 5186, 5188, 5192, 5200, 5216, 5249, 5250, 5252, 5256, 5264, 
    5280, 5312, 5377, 5378, 5380, 5384, 5392, 5408, 5440, 5504, 5633, 5634, 5636, 5640, 5648, 5664, 5696, 5760, 5888, 6147, 
    6149, 6150, 6153, 6154, 6156, 6161, 6162, 6164, 6168, 6177, 6178, 6180, 6184, 6192, 6209, 6210, 6212, 6216, 6224, 6240, 
    6273, 6274, 6276, 6280, 6288, 6304, 6336, 6401, 6402, 6404, 6408, 6416, 6432, 6464, 6528, 6657, 6658, 6660, 6664, 6672, 
    6688, 6720, 6784, 6912, 7169, 7170, 7172, 7176, 7184, 7200, 7232, 7296, 7424, 7680, 8199, 8203, 8205, 8206, 8211, 8213, 
    8214, 8217, 8218, 8220, 8227, 8229, 8230, 8233, 8234, 8236, 8241, 8242, 8244, 8248, 8259, 8261, 8262, 8265, 8266, 8268, 
    8273, 8274, 8276, 8280, 8289, 8290, 8292, 8296, 8304, 8323, 8325, 8326, 8329, 8330, 8332, 8337, 8338, 8340, 8344, 8353, 
    8354, 8356, 8360, 8368, 8385, 8386, 8388, 8392, 8400, 8416, 8451, 8453, 8454, 8457, 8458, 8460, 8465, 8466, 8468, 8472, 
    8481, 8482, 8484, 8488, 8496, 8513, 8514, 8516, 8520, 8528, 8544, 8577, 8578, 8580, 8584, 8592, 8608, 8640, 8707, 8709, 
    8710, 8713, 8714, 8716, 8721, 8722, 8724, 8728, 8737, 8738, 8740, 8744, 8752, 8769, 8770, 8772, 8776, 8784, 8800, 8833, 
    8834, 8836, 8840, 8848, 8864, 8896, 8961, 8962, 8964, 8968, 8976, 8992, 9024, 9088, 9219, 9221, 9222, 9225, 9226, 9228, 
    9233, 9234, 9236, 9240, 9249, 9250, 9252, 9256, 9264, 9281, 9282, 9284, 9288, 9296, 9312, 9345, 9346, 9348, 9352, 9360, 
    9376, 9408, 9473, 9474, 9476, 9480, 9488, 9504, 9536, 9600, 9729, 9730, 9732, 9736, 9744, 9760, 9792, 9856, 9984, 10243, 
    10245, 10246, 10249, 10250, 10252, 10257, 10258, 10260, 10264, 10273, 10274, 10276, 10280, 10288, 10305, 10306, 10308, 10312, 10320, 10336, 
    10369, 10370, 10372, 10376, 10384, 10400, 10432, 10497, 10498, 10500, 10504, 10512, 10528, 10560, 10624, 10753, 10754, 10756, 10760, 10768, 
    10784, 10816, 10880, 11008, 11265, 11266, 11268, 11272, 11280, 11296, 11328, 11392, 11520, 11776, 12291, 12293, 12294, 12297, 12298, 12300, 
    12305, 12306, 12308, 12312, 12321, 12322, 12324, 12328, 12336, 12353, 12354, 12356, 12360, 12368, 12384, 12417, 12418, 12420, 12424, 12432, 
    12448, 12480, 12545, 12546, 12548, 12552, 12560, 12576, 12608, 12672, 12801, 12802, 12804, 12808, 12816, 12832, 12864, 12928, 13056, 13313, 
    13314, 13316, 13320, 13328, 13344, 13376, 13440, 13568, 13824, 14337, 14338, 14340, 14344, 14352, 14368, 14400, 14464, 14592, 14848, 15360, 
    16391, 16395, 16397, 16398, 16403, 16405, 16406, 16409, 16410, 16412, 16419, 16421, 16422, 16425, 16426, 16428, 16433, 16434, 16436, 16440, 
    16451, 16453, 16454, 16457, 16458, 16460, 16465, 16466, 16468, 16472, 16481, 16482, 16484, 16488, 16496, 16515, 16517, 16518, 16521, 16522, 
    16524, 16529, 16530, 16532, 16536, 16545, 16546, 16548, 16552, 16560, 16577, 16578, 16580, 16584, 16592, 16608, 16643, 16645, 16646, 16649, 
    16650, 16652, 16657, 16658, 16660, 16664, 16673, 16674, 16676, 16680, 16688, 16705, 16706, 16708, 16712, 16720, 16736, 16769, 16770, 16772, 
    16776, 16784, 16800, 16832, 16899, 16901, 16902, 16905, 16906, 16908, 16913, 16914, 16916, 16920, 16929, 16930, 16932, 16936, 16944, 16961, 
    16962, 16964, 16968, 16976, 16992, 17025, 17026, 17028, 17032, 17040, 17056, 17088, 17153, 17154, 17156, 17160, 17168, 17184, 17216, 17280, 
    17411, 17413, 17414, 17417, 17418, 17420, 17425, 17426, 17428, 17432, 17441, 17442, 17444, 17448, 17456, 17473, 17474, 17476, 17480, 17488, 
    17504, 17537, 17538, 17540, 17544, 17552, 17568, 17600, 17665, 17666, 17668, 17672, 17680, 17696, 17728, 17792, 17921, 17922, 17924, 17928, 
    17936, 17952, 17984, 18048, 18176, 18435, 18437, 18438, 18441, 18442, 18444, 18449, 18450, 18452, 18456, 18465, 18466, 18468, 18472, 18480, 
    18497, 18498, 18500, 18504, 18512, 18528, 18561, 18562, 18564, 18568, 18576, 18592, 18624, 18689, 18690, 18692, 18696, 18704, 18720, 18752, 
    18816, 18945, 18946, 18948, 18952, 18960, 18976, 19008, 19072, 19200, 19457, 19458, 19460, 19464, 19472, 19488, 19520, 19584, 19712, 19968, 
    20483, 20485, 20486, 20489, 20490, 20492, 20497, 20498, 20500, 20504, 20513, 20514, 20516, 20520, 20528, 20545, 20546, 20548, 20552, 20560, 
    20576, 20609, 20610, 20612, 20616, 20624, 20640, 20672, 20737, 20738, 20740, 20744, 20752, 20768, 20800, 20864, 20993, 20994, 20996, 21000, 
    21008, 21024, 21056, 21120, 21248, 21505, 21506, 21508, 21512, 21520, 21536, 21568, 21632, 21760, 22016, 22529, 22530, 22532, 22536, 22544, 
    22560, 22592, 22656, 22784, 23040, 23552, 24579, 24581, 24582, 24585, 24586, 24588, 24593, 24594, 24596, 24600, 24609, 24610, 24612, 24616, 
    24624, 24641, 24642, 24644, 24648, 24656, 24672, 24705, 24706, 24708, 24712, 24720, 24736, 24768, 24833, 24834, 24836, 24840, 24848, 24864, 
    24896, 24960, 25089, 25090, 25092, 25096, 25104, 25120, 25152, 25216, 25344, 25601, 25602, 25604, 25608, 25616, 25632, 25664, 25728, 25856, 
    26112, 26625, 26626, 26628, 26632, 26640, 26656, 26688, 26752, 26880, 27136, 27648, 28673, 28674, 28676, 28680, 28688, 28704, 28736, 28800, 
    28928, 29184, 29696, 30720, 32775, 32779, 32781, 32782, 32787, 32789, 32790, 32793, 32794, 32796, 32803, 32805, 32806, 32809, 32810, 32812, 
    32817, 32818, 32820, 32824, 32835, 32837, 32838, 32841, 32842, 32844, 32849, 32850, 32852, 32856, 32865, 32866, 32868, 32872, 32880, 32899, 
    32901, 32902, 32905, 32906, 32908, 32913, 32914, 32916, 32920, 32929, 32930, 32932, 32936, 32944, 32961, 32962, 32964, 32968, 32976, 32992, 
    33027, 33029, 33030, 33033, 33034, 33036, 33041, 33042, 33044, 33048, 33057, 33058, 33060, 33064, 33072, 33089, 33090, 33092, 33096, 33104, 
    33120, 33153, 33154, 33156, 33160, 33168, 33184, 33216, 33283, 33285, 33286, 33289, 33290, 33292, 33297, 33298, 33300, 33304, 33313, 33314, 
    33316, 33320, 33328, 33345, 33346, 33348, 33352, 33360, 33376, 33409, 33410, 33412, 33416, 33424, 33440, 33472, 33537, 33538, 33540, 33544, 
    33552, 33568, 33600, 33664, 33795, 33797, 33798, 33801, 33802, 33804, 33809, 33810, 33812, 33816, 33825, 33826, 33828, 33832, 33840, 33857, 
    33858, 33860, 33864, 33872, 33888, 33921, 33922, 33924, 33928, 33936, 33952, 33984, 34049, 34050, 34052, 34056, 34064, 34080, 34112, 34176, 
    34305, 34306, 34308, 34312, 34320, 34336, 34368, 34432, 34560, 34819, 34821, 34822, 34825, 34826, 34828, 34833, 34834, 34836, 34840, 34849, 
    34850, 34852, 34856, 34864, 34881, 34882, 34884, 34888, 34896, 34912, 34945, 34946, 34948, 34952, 34960, 34976, 35008, 35073, 35074, 35076, 
    35080, 35088, 35104, 35136, 35200, 35329, 35330, 35332, 35336, 35344, 35360, 35392, 35456, 35584, 35841, 35842, 35844, 35848, 35856, 35872, 
    35904, 35968, 36096, 36352, 36867, 36869, 36870, 36873, 36874, 36876, 36881, 36882, 36884, 36888, 36897, 36898, 36900, 36904, 36912, 36929, 
    36930, 36932, 36936, 36944, 36960, 36993, 36994, 36996, 37000, 37008, 37024, 37056, 37121, 37122, 37124, 37128, 37136, 37152, 37184, 37248, 
    37377, 37378, 37380, 37384, 37392, 37408, 37440, 37504, 37632, 37889, 37890, 37892, 37896, 37904, 37920, 37952, 38016, 38144, 38400, 38913, 
    38914, 38916, 38920, 38928, 38944, 38976, 39040, 39168, 39424, 39936, 40963, 40965, 40966, 40969, 40970, 40972, 40977, 40978, 40980, 40984, 
    40993, 40994, 40996, 41000, 41008, 41025, 41026, 41028, 41032, 41040, 41056, 41089, 41090, 41092, 41096, 41104, 41120, 41152, 41217, 41218, 
    41220, 41224, 41232, 41248, 41280, 41344, 41473, 41474, 41476, 41480, 41488, 41504, 41536, 41600, 41728, 41985, 41986, 41988, 41992, 42000, 
    42016, 42048, 42112, 42240, 42496, 43009, 43010, 43012, 43016, 43024, 43040, 43072, 43136, 43264, 43520, 44032, 45057, 45058, 45060, 45064, 
    45072, 45088, 45120, 45184, 45312, 45568, 46080, 47104, 49155, 49157, 49158, 49161, 49162, 49164, 49169, 49170, 49172, 49176, 49185, 49186, 
    49188, 49192, 49200, 49217, 49218, 49220, 49224, 49232, 49248, 49281, 49282, 49284, 49288, 49296, 49312, 49344, 49409, 49410, 49412, 49416, 
    49424, 49440, 49472, 49536, 49665, 49666, 49668, 49672, 49680, 49696, 49728, 49792, 49920, 50177, 50178, 50180, 50184, 50192, 50208, 50240, 
    50304, 50432, 50688, 51201, 51202, 51204, 51208, 51216, 51232, 51264, 51328, 51456, 51712, 52224, 53249, 53250, 53252, 53256, 53264, 53280, 
    53312, 53376, 53504, 53760, 54272, 55296, 57345, 57346, 57348, 57352, 57360, 57376, 57408, 57472, 57600, 57856, 58368, 59392, 61440, },
  { 31, 47, 55, 59, 61, 62, 79, 87, 91, 93, 94, 103, 107, 109, 110, 115, 117, 118, 121, 122, 124, 
    143, 151, 155, 157, 158, 167, 171, 173, 174, 179, 181, 182, 185, 186, 188, 199, 203, 205, 206, 211, 
    213, 214, 217, 218, 220, 227, 229, 230, 233, 234, 236, 241, 242, 244, 248, 271, 279, 283, 285, 286, 
    295, 299, 301, 302, 307, 309, 310, 313, 314, 316, 327, 331, 333, 334, 339, 341, 342, 345, 346, 348, 
    355, 357, 358, 361, 362, 364, 369, 370, 372, 376, 391, 395, 397, 398, 403, 405, 406, 409, 410, 412, 
    419, 421, 422, 425, 426, 428, 433, 434, 436, 440, 451, 453, 454, 457, 458, 460, 465, 466, 468, 472, 
    481, 482, 484, 488, 496, 527, 535, 539, 541, 542, 551, 555, 557, 558, 563, 565, 566, 569, 570, 572, 
    583, 587, 589, 590, 595, 597, 598, 601, 602, 604, 611, 613, 614, 617, 618, 620, 625, 626, 628, 632, 
    647, 651, 653, 654, 659, 661, 662, 665, 666, 668, 675, 677, 678, 681, 682, 684, 689, 690, 692, 696, 
    707, 709, 710, 713, 714, 716, 721, 722, 724, 728, 737, 738, 740, 744, 752, 775, 779, 781, 782, 787, 
    789, 790, 793, 794, 796, 803, 805, 806, 809, 810, 812, 817, 818, 820, 824, 835, 837, 838, 841, 842, 
    844, 849, 850, 852, 856, 865, 866, 868, 872, 880, 899, 901, 902, 905, 906, 908, 913, 914, 916, 920, 
    929, 930, 932, 936, 944, 961, 962, 964, 968, 976, 992, 1039, 1047, 1051, 1053, 1054, 1063, 1067, 1069, 1070, 
    1075, 1077, 1078, 1081, 1082, 1084, 1095, 1099, 1101, 1102, 1107, 1109, 1110, 1113, 1114, 1116, 1123, 1125, 1126, 1129, 
    1130, 1132, 1137, 1138, 1140, 1144, 1159, 1163, 1165, 1166, 1171, 1173, 1174, 1177, 1178, 1180, 1187, 1189, 1190, 1193, 
    1194, 1196, 1201, 1202, 1204, 1208, 1219, 1221, 1222, 1225, 1226, 1228, 1233, 1234, 1236, 1240, 1249, 1250, 1252, 1256, 
    1264, 1287, 1291, 1293, 1294, 1299, 1301, 1302, 1305, 1306, 1308, 1315, 1317, 1318, 1321, 1322, 1324, 1329, 1330, 1332, 
    1336, 1347, 1349, 1350, 1353, 1354, 1356, 1361, 1362, 1364, 1368, 1377, 1378, 1380, 1384, 1392, 1411, 1413, 1414, 1417, 
    1418, 1420, 1425, 1426, 1428, 1432, 1441, 1442, 1444, 1448, 1456, 1473, 1474, 1476, 1480, 1488, 1504, 1543, 1547, 1549, 
    1550, 1555, 1557, 1558, 1561, 1562, 1564, 1571, 1573, 1574, 1577, 1578, 1580, 1585, 1586, 1588, 1592, 1603, 1605, 1606, 
    1609, 1610, 1612, 1617, 1618, 1620, 1624, 1633, 1634, 1636, 1640, 1648, 1667, 1669, 1670, 1673, 1674, 1676, 1681, 1682, 
    1684, 1688, 1697, 1698, 1700, 1704, 1712, 1729, 1730, 1732, 1736, 1744, 1760, 1795, 1797, 1798, 1801, 1802, 1804, 1809, 
    1810, 1812, 1816, 1825, 1826, 1828, 1832, 1840, 1857, 1858, 1860, 1864, 1872, 1888, 1921, 1922, 1924, 1928, 1936, 1952, 
    1984, 2063, 2071, 2075, 2077, 2078, 2087, 2091, 2093, 2094, 2099, 2101, 2102, 2105, 2106, 2108, 2119, 2123, 2125, 2126, 
    2131, 2133, 2134, 2137, 2138, 2140, 2147, 2149, 2150, 2153, 2154, 2156, 2161, 2162, 2164, 2168, 2183, 2187, 2189, 2190, 
    2195, 2197, 2198, 2201, 2202, 2204, 2211, 2213, 2214, 2217, 2218, 2220, 2225, 2226, 2228, 2232, 2243, 2245, 2246, 2249, 
    2250, 2252, 2257, 2258, 2260, 2264, 2273, 2274, 2276, 2280, 2288, 2311, 2315, 2317, 2318, 2323, 2325, 2326, 2329, 2330, 
    2332, 2339, 2341, 2342, 2345, 2346, 2348, 2353, 2354, 2356, 2360, 2371, 2373, 2374, 2377, 2378, 2380, 2385, 2386, 2388, 
    2392, 2401, 2402, 2404, 2408, 2416, 2435, 2437, 2438, 2441, 2442, 2444, 2449, 2450, 2452, 2456, 2465, 2466, 2468, 2472, 
    2480, 2497, 2498, 2500, 2504, 2512, 2528, 2567, 2571, 2573, 2574, 2579, 2581, 2582, 2585, 2586, 2588, 2595, 2597, 2598, 
    2601, 2602, 2604, 2609, 2610, 2612, 2616, 2627, 2629, 2630, 2633, 2634, 2636, 2641, 2642, 2644, 2648, 2657, 2658, 2660, 
    2664, 2672, 2691, 2693, 2694, 2697, 2698, 2700, 2705, 2706, 2708, 2712, 2721, 2722, 2724, 2728, 2736, 2753, 2754, 2756, 
    2760, 2768, 2784, 2819, 2821, 2822, 2825, 2826, 2828, 2833, 2834, 2836, 2840, 2849, 2850, 2852, 2856, 2864, 2881, 2882, 
    2884, 2888, 2896, 2912, 2945, 2946, 2948, 2952, 2960, 2976, 3008, 3079, 3083, 3085, 3086, 3091, 3093, 3094, 3097, 3098, 
    3100, 3107, 3109, 3110, 3113, 3114, 3116, 3121, 3122, 3124, 3128, 3139, 3141, 3142, 3145, 3146, 3148, 3153, 3154, 3156, 
    3160, 3169, 3170, 3172, 3176, 3184, 3203, 3205, 3206, 3209, 3210, 3212, 3217, 3218, 3220, 3224, 3233, 3234, 3236, 3240, 
    3248, 3265, 3266, 3268, 3272, 3280, 3296, 3331, 3333, 3334, 3337, 3338, 3340, 3345, 3346, 3348, 3352, 3361, 3362, 3364, 
    3368, 3376, 3393, 3394, 3396, 3400, 3408, 3424, 3457, 3458, 3460, 3464, 3472, 3488, 3520, 3587, 3589, 3590, 3593, 3594, 
    3596, 3601, 3602, 3604, 3608, 3617, 3618, 3620, 3624, 3632, 3649, 3650, 3652, 3656, 3664, 3680, 3713, 3714, 3716, 3720, 
    3728, 3744, 3776, 3841, 3842, 3844, 3848, 3856, 3872, 3904, 3968, 4111, 4119, 4123, 4125, 4126, 4135, 4139, 4141, 4142, 
    4147, 4149, 4150, 4153, 4154, 4156, 4167, 4171, 4173, 4174, 4179, 4181, 4182, 4185, 4186, 4188, 4195, 4197, 4198, 4201, 
    4202, 4204, 4209, 4210, 4212, 4216, 4231, 4235, 4237, 4238, 4243, 4245, 4246, 4249, 4250, 4252, 4259, 4261, 4262, 4265, 
    4266, 4268, 4273, 4274, 4276, 4280, 4291, 4293, 4294, 4297, 4298, 4300, 4305, 4306, 4308, 4312, 4321, 4322, 4324, 4328, 
    4336, 4359, 4363, 4365, 4366, 4371, 4373, 4374, 4377, 4378, 4380, 4387, 4389, 4390, 4393, 4394, 4396, 4401, 4402, 4404, 
    4408, 4419, 4421, 4422, 4425, 4426, 4428, 4433, 4434, 4436, 4440, 4449, 4450, 4452, 4456, 4464, 4483, 4485, 4486, 4489, 
    4490, 4492, 4497, 4498, 4500, 4504, 4513, 4514, 4516, 4520, 4528, 4545, 4546, 4548, 4552, 4560, 4576, 4615, 4619, 4621, 
    4622, 4627, 4629, 4630, 4633, 4634, 4636, 4643, 4645, 4646, 4649, 4650, 4652, 4657, 4658, 4660, 4664, 4675, 4677, 4678, 
    4681, 4682, 4684, 4689, 4690, 4692, 4696, 4705, 4706, 4708, 4712, 4720, 4739, 4741, 4742, 4745, 4746, 4748, 4753, 4754, 
    4756, 4760, 4769, 4770, 4772, 4776, 4784, 4801, 4802, 4804, 4808, 4816, 4832, 4867, 4869, 4870, 4873, 4874, 4876, 4881, 
    4882, 4884, 4888, 4897, 4898, 4900, 4904, 4912, 4929, 4930, 4932, 4936, 4944, 4960, 4993, 4994, 4996, 5000, 5008, 5024, 
    5056, 5127, 5131, 5133, 5134, 5139, 5141, 5142, 5145, 5146, 5148, 5155, 5157, 5158, 5161, 5162, 5164, 5169, 5170, 5172, 
    5176, 5187, 5189, 5190, 5193, 5194, 5196, 5201, 5202, 5204, 5208, 5217, 5218, 5220, 5224, 5232, 5251, 5253, 5254, 5257, 
    5258, 5260, 5265, 5266, 5268, 5272, 5281, 5282, 5284, 5288, 5296, 5313, 5314, 5316, 5320, 5328, 5344, 5379, 5381, 5382, 
    5385, 5386, 5388, 5393, 5394, 5396, 5400, 5409, 5410, 5412, 5416, 5424, 5441, 5442, 5444, 5448, 5456, 5472, 5505, 5506, 
    5508, 5512, 5520, 5536, 5568, 5635, 5637, 5638, 5641, 5642, 5644, 5649, 5650, 5652, 5656, 5665, 5666, 5668, 5672, 5680, 
    5697, 5698, 5700, 5704, 5712, 5728, 5761, 5762, 5764, 5768, 5776, 5792, 5824, 5889, 5890, 5892, 5896, 5904, 5920, 5952, 
    6016, 6151, 6155, 6157, 6158, 6163, 6165, 6166, 6169, 6170, 6172, 6179, 6181, 6182, 6185, 6186, 6188, 6193, 6194, 6196, 
    6200, 6211, 6213, 6214, 6217, 6218, 6220, 6225, 6226, 6228, 6232, 6241, 6242, 6244, 6248, 6256, 6275, 6277, 6278, 6281, 
    6282, 6284, 6289, 6290, 6292, 6296, 6305, 6306, 6308, 6312, 6320, 6337, 6338, 6340, 6344, 6352, 6368, 6403, 6405, 6406, 
    6409, 6410, 6412, 6417, 6418, 6420, 6424, 6433, 6434, 6436, 6440, 6448, 6465, 6466, 6468, 6472, 6480, 6496, 6529, 6530, 
    6532, 6536, 6544, 6560, 6592, 6659, 6661, 6662, 6665, 6666, 6668, 6673, 6674, 6676, 6680, 6689, 6690, 6692, 6696, 6704, 
    6721, 6722, 6724, 6728, 6736, 6752, 6785, 6786, 6788, 6792, 6800, 6816, 6848, 6913, 6914, 6916, 6920, 6928, 6944, 6976, 
    7040, 7171, 7173, 7174, 7177, 7178, 7180, 7185, 7186, 7188, 7192, 7201, 7202, 7204, 7208, 7216, 7233, 7234, 7236, 7240, 
    7248, 7264, 7297, 7298, 7300, 7304, 7312, 7328, 7360, 7425, 7426, 7428, 7432, 7440, 7456, 7488, 7552, 7681, 7682, 7684, 
    7688, 7696, 7712, 7744, 7808, 7936, 8207, 8215, 8219, 8221, 8222, 8231, 8235, 8237, 8238, 8243, 8245, 8246, 8249, 8250, 
    8252, 8263, 8267, 8269, 8270, 8275, 8277, 8278, 8281, 8282, 8284, 8291, 8293, 8294, 8297, 8298, 8300, 8305, 8306, 8308, 
    8312, 8327, 8331, 8333, 8334, 8339, 8341, 8342, 8345, 8346, 8348, 8355, 8357, 8358, 8361, 8362, 8364, 8369, 8370, 8372, 
    8376, 8387, 8389, 8390, 8393, 8394, 8396, 8401, 8402, 8404, 8408, 8417, 8418, 8420, 8424, 8432, 8455, 8459, 8461, 8462, 
    8467, 8469, 8470, 8473, 8474, 8476, 8483, 8485, 8486, 8489, 8490, 8492, 8497, 8498, 8500, 8504, 8515, 8517, 8518, 8521, 
    8522, 8524, 8529, 8530, 8532, 8536, 8545, 8546, 8548, 8552, 8560, 8579, 8581, 8582, 8585, 8586, 8588, 8593, 8594, 8596, 
    8600, 8609, 8610, 8612, 8616, 8624, 8641, 8642, 8644, 8648, 8656, 8672, 8711, 8715, 8717, 8718, 8723, 8725, 8726, 8729, 
    8730, 8732, 8739, 8741, 8742, 8745, 8746, 8748, 8753, 8754, 8756, 8760, 8771, 8773, 8774, 8777, 8778, 8780, 8785, 8786, 
    8788, 8792, 8801, 8802, 8804, 8808, 8816, 8835, 8837, 8838, 8841, 8842, 8844, 8849, 8850, 8852, 8856, 8865, 8866, 8868, 
    8872, 8880, 8897, 8898, 8900, 8904, 8912, 8928, 8963, 8965, 8966, 8969, 8970, 8972, 8977, 8978, 8980, 8984, 8993, 8994, 
    8996, 9000, 9008, 9025, 9026, 9028, 9032, 9040, 9056, 9089, 9090, 9092, 9096, 9104, 9120, 9152, 9223, 9227, 9229, 9230, 
    9235, 9237, 9238, 9241, 9242, 9244, 9251, 9253, 9254, 9257, 9258, 9260, 9265, 9266, 9268, 9272, 9283, 9285, 9286, 9289, 
    9290, 9292, 9297, 9298, 9300, 9304, 9313, 9314, 9316, 9320, 9328, 9347, 9349, 9350, 9353, 9354, 9356, 9361, 9362, 9364, 
    9368, 9377, 9378, 9380, 9384, 9392, 9409, 9410, 9412, 9416, 9424, 9440, 9475, 9477, 9478, 9481, 9482, 9484, 9489, 9490, 
    9492, 9496, 9505, 9506, 9508, 9512, 9520, 9537, 9538, 9540, 9544, 9552, 9568, 9601, 9602, 9604, 9608, 9616, 9632, 9664, 
    9731, 9733, 9734, 9737, 9738, 9740, 9745, 9746, 9748, 9752, 9761, 9762, 9764, 9768, 9776, 9793, 9794, 9796, 9800, 9808, 
    9824, 9857, 9858, 9860, 9864, 9872, 9888, 9920, 9985, 9986, 9988, 9992, 10000, 10016, 10048, 10112, 10247, 10251, 10253, 10254, 
    10259, 10261, 10262, 10265, 10266, 10268, 10275, 10277, 10278, 10281, 10282, 10284, 10289, 10290, 10292, 10296, 10307, 10309, 10310, 10313, 
    10314, 10316, 10321, 10322, 10324, 10328, 10337, 10338, 10340, 10344, 10352, 10371, 10373, 10374, 10377, 10378, 10380, 10385, 10386, 10388, 
    10392, 10401, 10402, 10404, 10408, 10416, 10433, 10434, 10436, 10440, 10448, 10464, 10499, 10501, 10502, 10505, 10506, 10508, 10513, 10514, 
    10516, 10520, 10529, 10530, 10532, 10536, 10544, 10561, 10562, 10564, 10568, 10576, 10592, 10625, 10626, 10628, 10632, 10640, 10656, 10688, 
    10755, 10757, 10758, 10761, 10762, 10764, 10769, 10770, 10772, 10776, 10785, 10786, 10788, 10792, 10800, 10817, 10818, 10820, 10824, 10832, 
    10848, 10881, 10882, 10884, 10888, 10896, 10912, 10944, 11009, 11010, 11012, 11016, 11024, 11040, 11072, 11136, 11267, 11269, 11270, 11273, 
    11274, 11276, 11281, 11282, 11284, 11288, 11297, 11298, 11300, 11304, 11312, 11329, 11330, 11332, 11336, 11344, 11360, 11393, 11394, 11396, 
    11400, 11408, 11424, 11456, 11521, 11522, 11524, 11528, 11536, 11552, 11584, 11648, 11777, 11778, 11780, 11784, 11792, 11808, 11840, 11904, 
    12032, 12295, 12299, 12301, 12302, 12307, 12309, 12310, 12313, 12314, 12316, 12323, 12325, 12326, 12329, 12330, 12332, 12337, 12338, 12340, 
    12344, 12355, 12357, 12358, 12361, 12362, 12364, 12369, 12370, 12372, 12376, 12385, 12386, 12388, 12392, 12400, 12419, 12421, 12422, 12425, 
    12426, 12428, 12433, 12434, 12436, 12440, 12449, 12450, 12452, 12456, 12464, 12481, 12482, 12484, 12488, 12496, 12512, 12547, 12549, 12550, 
    12553, 12554, 12556, 12561, 12562, 12564, 12568, 12577, 12578, 12580, 12584, 12592, 12609, 12610, 12612, 12616, 12624, 12640, 12673, 12674, 
    12676, 12680, 12688, 12704, 12736, 12803, 12805, 12806, 12809, 12810, 12812, 12817, 12818, 12820, 12824, 12833, 12834, 12836, 12840, 12848, 
    12865, 12866, 12868, 12872, 12880, 12896, 12929, 12930, 12932, 12936, 12944, 12960, 12992, 13057, 13058, 13060, 13064, 13072, 13088, 13120, 
    13184, 13315, 13317, 13318, 13321, 13322, 13324, 13329, 13330, 13332, 13336, 13345, 13346, 13348, 13352, 13360, 13377, 13378, 13380, 13384, 
    13392, 13408, 13441, 13442, 13444, 13448, 13456, 13472, 13504, 13569, 13570, 13572, 13576, 13584, 13600, 13632, 13696, 13825, 13826, 13828, 
    13832, 13840, 13856, 13888, 13952, 14080, 14339, 14341, 14342, 14345, 14346, 14348, 14353, 14354, 14356, 14360, 14369, 14370, 14372, 14376, 
    14384, 14401, 14402, 14404, 14408, 14416, 14432, 14465, 14466, 14468, 14472, 14480, 14496, 14528, 14593, 14594, 14596, 14600, 14608, 14624, 
    14656, 14720, 14849, 14850, 14852, 14856, 14864, 14880, 14912, 14976, 15104, 15361, 15362, 15364, 15368, 15376, 15392, 15424, 15488, 15616, 
    15872, 16399, 16407, 16411, 16413, 16414, 16423, 16427, 16429, 16430, 16435, 16437, 16438, 16441, 16442, 16444, 16455, 16459, 16461, 16462, 
    16467, 16469, 16470, 16473, 16474, 16476, 16483, 16485, 16486, 16489, 16490, 16492, 16497, 16498, 16500, 16504, 16519, 16523, 16525, 16526, 
    16531, 16533, 16534, 16537, 16538, 16540, 16547, 16549, 16550, 16553, 16554, 16556, 16561, 16562, 16564, 16568, 16579, 16581, 16582, 16585, 
    16586, 16588, 16593, 16594, 16596, 16600, 16609, 16610, 16612, 16616, 16624, 16647, 16651, 16653, 16654, 16659, 16661, 16662, 16665, 16666, 
    16668, 16675, 16677, 16678, 16681, 16682, 16684, 16689, 16690, 16692, 16696, 16707, 16709, 16710, 16713, 16714, 16716, 16721, 16722, 16724, 
    16728, 16737, 16738, 16740, 16744, 16752, 16771, 16773, 16774, 16777, 16778, 16780, 16785, 16786, 16788, 16792, 16801, 16802, 16804, 16808, 
    16816, 16833, 16834, 16836, 16840, 16848, 16864, 16903, 16907, 16909, 16910, 16915, 16917, 16918, 16921, 16922, 16924, 16931, 16933, 16934, 
    16937, 16938, 16940, 16945, 16946, 16948, 16952, 16963, 16965, 16966, 16969, 16970, 16972, 16977, 16978, 16980, 16984, 16993, 16994, 16996, 
    17000, 17008, 17027, 17029, 17030, 17033, 17034, 17036, 17041, 17042, 17044, 17048, 17057, 17058, 17060, 17064, 17072, 17089, 17090, 17092, 
    17096, 17104, 17120, 17155, 17157, 17158, 17161, 17162, 17164, 17169, 17170, 17172, 17176, 17185, 17186, 17188, 17192, 17200, 17217, 17218, 
    17220, 17224, 17232, 17248, 17281, 17282, 17284, 17288, 17296, 17312, 17344, 17415, 17419, 17421, 17422, 17427, 17429, 17430, 17433, 17434, 
    17436, 17443, 17445, 17446, 17449, 17450, 17452, 17457, 17458, 17460, 17464, 17475, 17477, 17478, 17481, 17482, 17484, 17489, 17490, 17492, 
    17496, 17505, 17506, 17508, 17512, 17520, 17539, 17541, 17542, 17545, 17546, 17548, 17553, 17554, 17556, 17560, 17569, 17570, 17572, 17576, 
    17584, 17601, 17602, 17604, 17608, 17616, 17632, 17667, 17669, 17670, 17673, 17674, 17676, 17681, 17682, 17684, 17688, 17697, 17698, 17700, 
    17704, 17712, 17729, 17730, 17732, 17736, 17744, 17760, 17793, 17794, 17796, 17800, 17808, 17824, 17856, 17923, 17925, 17926, 17929, 17930, 
    17932, 17937, 17938, 17940, 17944, 17953, 17954, 17956, 17960, 17968, 17985, 17986, 17988, 17992, 18000, 18016, 18049, 18050, 18052, 18056, 
    18064, 18080, 18112, 18177, 18178, 18180, 18184, 18192, 18208, 18240, 18304, 18439, 18443, 18445, 18446, 18451, 18453, 18454, 18457, 18458, 
    18460, 18467, 18469, 18470, 18473, 18474, 18476, 18481, 18482, 18484, 18488, 18499, 18501, 18502, 18505, 18506, 18508, 18513, 18514, 18516, 
    18520, 18529, 18530, 18532, 18536, 18544, 18563, 18565, 18566, 18569, 18570, 18572, 18577, 18578, 18580, 18584, 18593, 18594, 18596, 18600, 
    18608, 18625, 18626, 18628, 18632, 18640, 18656, 18691, 18693, 18694, 18697, 18698, 18700, 18705, 18706, 18708, 18712, 18721, 18722, 18724, 
    18728, 18736, 18753, 18754, 18756, 18760, 18768, 18784, 18817, 18818, 18820, 18824, 18832, 18848, 18880, 18947, 18949, 18950, 18953, 18954, 
    18956, 18961, 18962, 18964, 18968, 18977, 18978, 18980, 18984, 18992, 19009, 19010, 19012, 19016, 19024, 19040, 19073, 19074, 19076, 19080, 
    19088, 19104, 19136, 19201, 19202, 19204, 19208, 19216, 19232, 19264, 19328, 19459, 19461, 19462, 19465, 19466, 19468, 19473, 19474, 19476, 
    19480, 19489, 19490, 19492, 19496, 19504, 19521, 19522, 19524, 19528, 19536, 19552, 19585, 19586, 19588, 19592, 19600, 19616, 19648, 19713, 
    19714, 19716, 19720, 19728, 19744, 19776, 19840, 19969, 19970, 19972, 19976, 19984, 20000, 20032, 20096, 20224, 20487, 20491, 20493, 20494, 
    20499, 20501, 20502, 20505, 20506, 20508, 20515, 20517, 20518, 20521, 20522, 20524, 20529, 20530, 20532, 20536, 20547, 20549, 20550, 20553, 
    20554, 20556, 20561, 20562, 20564, 20568, 20577, 20578, 20580, 20584, 20592, 20611, 20613, 20614, 20617, 20618, 20620, 20625, 20626, 20628, 
    20632, 20641, 20642, 20644, 20648, 20656, 20673, 20674, 20676, 20680, 20688, 20704, 20739, 20741, 20742, 20745, 20746, 20748, 20753, 20754, 
    20756, 20760, 20769, 20770, 20772, 20776, 20784, 20801, 20802, 20804, 20808, 20816, 20832, 20865, 20866, 20868, 20872, 20880, 20896, 20928, 
    20995, 20997, 20998, 21001, 21002, 21004, 21009, 21010, 21012, 21016, 21025, 21026, 21028, 21032, 21040, 21057, 21058, 21060, 21064, 21072, 
    21088, 21121, 21122, 21124, 21128, 21136, 21152, 21184, 21249, 21250, 21252, 21256, 21264, 21280, 21312, 21376, 21507, 21509, 21510, 21513, 
    21514, 21516, 21521, 21522, 21524, 21528, 21537, 21538, 21540, 21544, 21552, 21569, 21570, 21572, 21576, 21584, 21600, 21633, 21634, 21636, 
    21640, 21648, 21664, 21696, 21761, 21762, 21764, 21768, 21776, 21792, 21824, 21888, 22017, 22018, 22020, 22024, 22032, 22048, 22080, 22144, 
    22272, 22531, 22533, 22534, 22537, 22538, 22540, 22545, 22546, 22548, 22552, 22561, 22562, 22564, 22568, 22576, 22593, 22594, 22596, 22600, 
    22608, 22624, 22657, 22658, 22660, 22664, 22672, 22688, 22720, 22785, 22786, 22788, 22792, 22800, 22816, 22848, 22912, 23041, 23042, 23044, 
    23048, 23056, 23072, 23104, 23168, 23296, 23553, 23554, 23556, 23560, 23568, 23584, 23616, 23680, 23808, 24064, 24583, 24587, 24589, 24590, 
    24595, 24597, 24598, 24601, 24602, 24604, 24611, 24613, 24614, 24617, 24618, 24620, 24625, 24626, 24628, 24632, 24643, 24645, 24646, 24649, 
    24650, 24652, 24657, 24658, 24660, 24664, 24673, 24674, 24676, 24680, 24688, 24707, 24709, 24710, 24713, 24714, 24716, 24721, 24722, 24724, 
    24728, 24737, 24738, 24740, 24744, 24752, 24769, 24770, 24772, 24776, 24784, 24800, 24835, 24837, 24838, 24841, 24842, 24844, 24849, 24850, 
    24852, 24856, 24865, 24866, 24868, 24872, 24880, 24897, 24898, 24900, 24904, 24912, 24928, 24961, 24962, 24964, 24968, 24976, 24992, 25024, 
    25091, 25093, 25094, 25097, 25098, 25100, 25105, 25106, 25108, 25112, 25121, 25122, 25124, 25128, 25136, 25153, 25154, 25156, 25160, 25168, 
    25184, 25217, 25218, 25220, 25224, 25232, 25248, 25280, 25345, 25346, 25348, 25352, 25360, 25376, 25408, 25472, 25603, 25605, 25606, 25609, 
    25610, 25612, 25617, 25618, 25620, 25624, 25633, 25634, 25636, 25640, 25648, 25665, 25666, 25668, 25672, 25680, 25696, 25729, 25730, 25732, 
    25736, 25744, 25760, 25792, 25857, 25858, 25860, 25864, 25872, 25888, 25920, 25984, 26113, 26114, 26116, 26120, 26128, 26144, 26176, 26240, 
    26368, 26627, 26629, 26630, 26633, 26634, 26636, 26641, 26642, 26644, 26648, 26657, 26658, 26660, 26664, 26672, 26689, 26690, 26692, 26696, 
    26704, 26720, 26753, 26754, 26756, 26760, 26768, 26784, 26816, 26881, 26882, 26884, 26888, 26896, 26912, 26944, 27008, 27137, 27138, 27140, 
    27144, 27152, 27168, 27200, 27264, 27392, 27649, 27650, 27652, 27656, 27664, 27680, 27712, 27776, 27904, 28160, 28675, 28677, 28678, 28681, 
    28682, 28684, 28689, 28690, 28692, 28696, 28705, 28706, 28708, 28712, 28720, 28737, 28738, 28740, 28744, 28752, 28768, 28801, 28802, 28804, 
    28808, 28816, 28832, 28864, 28929, 28930, 28932, 28936, 28944, 28960, 28992, 29056, 29185, 29186, 29188, 29192, 29200, 29216, 29248, 29312, 
    29440, 29697, 29698, 29700, 29704, 29712, 29728, 29760, 29824, 29952, 30208, 30721, 30722, 30724, 30728, 30736, 30752, 30784, 30848, 30976, 
    31232, 31744, 32783, 32791, 32795, 32797, 32798, 32807, 32811, 32813, 32814, 32819, 32821, 32822, 32825, 32826, 32828, 32839, 32843, 32845, 
    32846, 32851, 32853, 32854, 32857, 32858, 32860, 32867, 32869, 32870, 32873, 32874, 32876, 32881, 32882, 32884, 32888, 32903, 32907, 32909, 
    32910, 32915, 32917, 32918, 32921, 32922, 32924, 32931, 32933, 32934, 32937, 32938, 32940, 32945, 32946, 32948, 32952, 32963, 32965, 32966, 
    32969, 32970, 32972, 32977, 32978, 32980, 32984, 32993, 32994, 32996, 33000, 33008, 33031, 33035, 33037, 33038, 33043, 33045, 33046, 33049, 
    33050, 33052, 33059, 33061, 33062, 33065, 33066, 33068, 33073, 33074, 33076, 33080, 33091, 33093, 33094, 33097, 33098, 33100, 33105, 33106, 
    33108, 33112, 33121, 33122, 33124, 33128, 33136, 33155, 33157, 33158, 33161, 33162, 33164, 33169, 33170, 33172, 33176, 33185, 33186, 33188, 
    33192, 33200, 33217, 33218, 33220, 33224, 33232, 33248, 33287, 33291, 33293, 33294, 33299, 33301, 33302, 33305, 33306, 33308, 33315, 33317, 
    33318, 33321, 33322, 33324, 33329, 33330, 33332, 33336, 33347, 33349, 33350, 33353, 33354, 33356, 33361, 33362, 33364, 33368, 33377, 33378, 
    33380, 33384, 33392, 33411, 33413, 33414, 33417, 33418, 33420, 33425, 33426, 33428, 33432, 33441, 33442, 33444, 33448, 33456, 33473, 33474, 
    33476, 33480, 33488, 33504, 33539, 33541, 33542, 33545, 33546, 33548, 33553, 33554, 33556, 33560, 33569, 33570, 33572, 33576, 33584, 33601, 
    33602, 33604, 33608, 33616, 33632, 33665, 33666, 33668, 33672, 33680, 33696, 33728, 33799, 33803, 33805, 33806, 33811, 33813, 33814, 33817, 
    33818, 33820, 33827, 33829, 33830, 33833, 33834, 33836, 33841, 33842, 33844, 33848, 33859, 33861, 33862, 33865, 33866, 33868, 33873, 33874, 
    33876, 33880, 33889, 33890, 33892, 33896, 33904, 33923, 33925, 33926, 33929, 33930, 33932, 33937, 33938, 33940, 33944, 33953, 33954, 33956, 
    33960, 33968, 33985, 33986, 33988, 33992, 34000, 34016, 34051, 34053, 34054, 34057, 34058, 34060, 34065, 34066, 34068, 34072, 34081, 34082, 
    34084, 34088, 34096, 34113, 34114, 34116, 34120, 34128, 34144, 34177, 34178, 34180, 34184, 34192, 34208, 34240, 34307, 34309, 34310, 34313, 
    34314, 34316, 34321, 34322, 34324, 34328, 34337, 34338, 34340, 34344, 34352, 34369, 34370, 34372, 34376, 34384, 34400, 34433, 34434, 34436, 
    34440, 34448, 34464, 34496, 34561, 34562, 34564, 34568, 34576, 34592, 34624, 34688, 34823, 34827, 34829, 34830, 34835, 34837, 34838, 34841, 
    34842, 34844, 34851, 34853, 34854, 34857, 34858, 34860, 34865, 34866, 34868, 34872, 34883, 34885, 34886, 34889, 34890, 34892, 34897, 34898, 
    34900, 34904, 34913, 34914, 34916, 34920, 34928, 34947, 34949, 34950, 34953, 34954, 34956, 34961, 34962, 34964, 34968, 34977, 34978, 34980, 
    34984, 34992, 35009, 35010, 35012, 35016, 35024, 35040, 35075, 35077, 35078, 35081, 35082, 35084, 35089, 35090, 35092, 35096, 35105, 35106, 
    35108, 35112, 35120, 35137, 35138, 35140, 35144, 35152, 35168, 35201, 35202, 35204, 35208, 35216, 35232, 35264, 35331, 35333, 35334, 35337, 
    35338, 35340, 35345, 35346, 35348, 35352, 35361, 35362, 35364, 35368, 35376, 35393, 35394, 35396, 35400, 35408, 35424, 35457, 35458, 35460, 
    35464, 35472, 35488, 35520, 35585, 35586, 35588, 35592, 35600, 35616, 35648, 35712, 35843, 35845, 35846, 35849, 35850, 35852, 35857, 35858, 
    35860, 35864, 35873, 35874, 35876, 35880, 35888, 35905, 35906, 35908, 35912, 35920, 35936, 35969, 35970, 35972, 35976, 35984, 36000, 36032, 
    36097, 36098, 36100, 36104, 36112, 36128, 36160, 36224, 36353, 36354, 36356, 36360, 36368, 36384, 36416, 36480, 36608, 36871, 36875, 36877, 
    36878, 36883, 36885, 36886, 36889, 36890, 36892, 36899, 36901, 36902, 36905, 36906, 36908, 36913, 36914, 36916, 36920, 36931, 36933, 36934, 
    36937, 36938, 36940, 36945, 36946, 36948, 36952, 36961, 36962, 36964, 36968, 36976, 36995, 36997, 36998, 37001, 37002, 37004, 37009, 37010, 
    37012, 37016, 37025, 37026, 37028, 37032, 37040, 37057, 37058, 37060, 37064, 37072, 37088, 37123, 37125, 37126, 37129, 37130, 37132, 37137, 
    37138, 37140, 37144, 37153, 37154, 37156, 37160, 37168, 37185, 37186, 37188, 37192, 37200, 37216, 37249, 37250, 37252, 37256, 37264, 37280, 
    37312, 37379, 37381, 37382, 37385, 37386, 37388, 37393, 37394, 37396, 37400, 37409, 37410, 37412, 37416, 37424, 37441, 37442, 37444, 37448, 
    37456, 37472, 37505, 37506, 37508, 37512, 37520, 37536, 37568, 37633, 37634, 37636, 37640, 37648, 37664, 37696, 37760, 37891, 37893, 37894, 
    37897, 37898, 37900, 37905, 37906, 37908, 37912, 37921, 37922, 37924, 37928, 37936, 37953, 37954, 37956, 37960, 37968, 37984, 38017, 38018, 
    38020, 38024, 38032, 38048, 38080, 38145, 38146, 38148, 38152, 38160, 38176, 38208, 38272, 38401, 38402, 38404, 38408, 38416, 38432, 38464, 
    38528, 38656, 38915, 38917, 38918, 38921, 38922, 38924, 38929, 38930, 38932, 38936, 38945, 38946, 38948, 38952, 38960, 38977, 38978, 38980, 
    38984, 38992, 39008, 39041, 39042, 39044, 39048, 39056, 39072, 39104, 39169, 39170, 39172, 39176, 39184, 39200, 39232, 39296, 39425, 39426, 
    39428, 39432, 39440, 39456, 39488, 39552, 39680, 39937, 39938, 39940, 39944, 39952, 39968, 40000, 40064, 40192, 40448, 40967, 40971, 40973, 
    40974, 40979, 40981, 40982, 40985, 40986, 40988, 40995, 40997, 40998, 41001, 41002, 41004, 41009, 41010, 41012, 41016, 41027, 41029, 41030, 
    41033, 41034, 41036, 41041, 41042, 41044, 41048, 41057, 41058, 41060, 41064, 41072, 41091, 41093, 41094, 41097, 41098, 41100, 41105, 41106, 
    41108, 41112, 41121, 41122, 41124, 41128, 41136, 41153, 41154, 41156, 41160, 41168, 41184, 41219, 41221, 41222, 41225, 41226, 41228, 41233, 
    41234, 41236, 41240, 41249, 41250, 41252, 41256, 41264, 41281, 41282, 41284, 41288, 41296, 41312, 41345, 41346, 41348, 41352, 41360, 41376, 
    41408, 41475, 41477, 41478, 41481, 41482, 41484, 41489, 41490, 41492, 41496, 41505, 41506, 41508, 41512, 41520, 41537, 41538, 41540, 41544, 
    41552, 41568, 41601, 41602, 41604, 41608, 41616, 41632, 41664, 41729, 41730, 41732, 41736, 41744, 41760, 41792, 41856, 41987, 41989, 41990, 
    41993, 41994, 41996, 42001, 42002, 42004, 42008, 42017, 42018, 42020, 42024, 42032, 42049, 42050, 42052, 42056, 42064, 42080, 42113, 42114, 
    42116, 42120, 42128, 42144, 42176, 42241, 42242, 42244, 42248, 42256, 42272, 42304, 42368, 42497, 42498, 42500, 42504, 42512, 42528, 42560, 
    42624, 42752, 43011, 43013, 43014, 43017, 43018, 43020, 43025, 43026, 43028, 43032, 43041, 43042, 43044, 43048, 43056, 43073, 43074, 43076, 
    43080, 43088, 43104, 43137, 43138, 43140, 43144, 43152, 43168, 43200, 43265, 43266, 43268, 43272, 43280, 43296, 43328, 43392, 43521, 43522, 
    43524, 43528, 43536, 43552, 43584, 43648, 43776, 44033, 44034, 44036, 44040, 44048, 44064, 44096, 44160, 44288, 44544, 45059, 45061, 45062, 
    45065, 45066, 45068, 45073, 45074, 45076, 45080, 45089, 45090, 45092, 45096, 45104, 45121, 45122, 45124, 45128, 45136, 45152, 45185, 45186, 
    45188, 45192, 45200, 45216, 45248, 45313, 45314, 45316, 45320, 45328, 45344, 45376, 45440, 45569, 45570, 45572, 45576, 45584, 45600, 45632, 
    45696, 45824, 46081, 46082, 46084, 46088, 46096, 46112, 46144, 46208, 46336, 46592, 47105, 47106, 47108, 47112, 47120, 47136, 47168, 47232, 
    47360, 47616, 48128, 49159, 49163, 49165, 49166, 49171, 49173, 49174, 49177, 49178, 49180, 49187, 49189, 49190, 49193, 49194, 49196, 49201, 
    49202, 49204, 49208, 49219, 49221, 49222, 49225, 49226, 49228, 49233, 49234, 49236, 49240, 49249, 49250, 49252, 49256, 49264, 49283, 49285, 
    49286, 49289, 49290, 49292, 49297, 49298, 49300, 49304, 49313, 49314, 49316, 49320, 49328, 49345, 49346, 49348, 49352, 49360, 49376, 49411, 
    49413, 49414, 49417, 49418, 49420, 49425, 49426, 49428, 49432, 49441, 49442, 49444, 49448, 49456, 49473, 49474, 49476, 49480, 49488, 49504, 
    49537, 49538, 49540, 49544, 49552, 49568, 49600, 49667, 49669, 49670, 49673, 49674, 49676, 49681, 49682, 49684, 49688, 49697, 49698, 49700, 
    49704, 49712, 49729, 49730, 49732, 49736, 49744, 49760, 49793, 49794, 49796, 49800, 49808, 49824, 49856, 49921, 49922, 49924, 49928, 49936, 
    49952, 49984, 50048, 50179, 50181, 50182, 50185, 50186, 50188, 50193, 50194, 50196, 50200, 50209, 50210, 50212, 50216, 50224, 50241, 50242, 
    50244, 50248, 50256, 50272, 50305, 50306, 50308, 50312, 50320, 50336, 50368, 50433, 50434, 50436, 50440, 50448, 50464, 50496, 50560, 50689, 
    50690, 50692, 50696, 50704, 50720, 50752, 50816, 50944, 51203, 51205, 51206, 51209, 51210, 51212, 51217, 51218, 51220, 51224, 51233, 51234, 
    51236, 51240, 51248, 51265, 51266, 51268, 51272, 51280, 51296, 51329, 51330, 51332, 51336, 51344, 51360, 51392, 51457, 51458, 51460, 51464, 
    51472, 51488, 51520, 51584, 51713, 51714, 51716, 51720, 51728, 51744, 51776, 51840, 51968, 52225, 52226, 52228, 52232, 52240, 52256, 52288, 
    52352, 52480, 52736, 53251, 53253, 53254, 53257, 53258, 53260, 53265, 53266, 53268, 53272, 53281, 53282, 53284, 53288, 53296, 53313, 53314, 
    53316, 53320, 53328, 53344, 53377, 53378, 53380, 53384, 53392, 53408, 53440, 53505, 53506, 53508, 53512, 53520, 53536, 53568, 53632, 53761, 
    53762, 53764, 53768, 53776, 53792, 53824, 53888, 54016, 54273, 54274, 54276, 54280, 54288, 54304, 54336, 54400, 54528, 54784, 55297, 55298, 
    55300, 55304, 55312, 55328, 55360, 55424, 55552, 55808, 56320, 57347, 57349, 57350, 57353, 57354, 57356, 57361, 57362, 57364, 57368, 57377, 
    57378, 57380, 57384, 57392, 57409, 57410, 57412, 57416, 57424, 57440, 57473, 57474, 57476, 57480, 57488, 57504, 57536, 57601, 57602, 57604, 
    57608, 57616, 57632, 57664, 57728, 57857, 57858, 57860, 57864, 57872, 57888, 57920, 57984, 58112, 58369, 58370, 58372, 58376, 58384, 58400, 
    58432, 58496, 58624, 58880, 59393, 59394, 59396, 59400, 59408, 59424, 59456, 59520, 59648, 59904, 60416, 61441, 61442, 61444, 61448, 61456, 
    61472, 61504, 61568, 61696, 61952, 62464, 63488, },
  { 63, 95, 111, 119, 123, 125, 126, 159, 175, 183, 187, 189, 190, 207, 215, 219, 221, 222, 231, 235, 237, 
    238, 243, 245, 246, 249, 250, 252, 287, 303, 311, 315, 317, 318, 335, 343, 347, 349, 350, 359, 363, 
    365, 366, 371, 373, 374, 377, 378, 380, 399, 407, 411, 413, 414, 423, 427, 429, 430, 435, 437, 438, 
    441, 442, 444, 455, 459, 461, 462, 467, 469, 470, 473, 474, 476, 483, 485, 486, 489, 490, 492, 497, 
    498, 500, 504, 543, 559, 567, 571, 573, 574, 591, 599, 603, 605, 606, 615, 619, 621, 622, 627, 629, 
    630, 633, 634, 636, 655, 663, 667, 669, 670, 679, 683, 685, 686, 691, 693, 694, 697, 698, 700, 711, 
    715, 717, 718, 723, 725, 726, 729, 730, 732, 739, 741, 742, 745, 746, 748, 753, 754, 756, 760, 783, 
    791, 795, 797, 798, 807, 811, 813, 814, 819, 821, 822, 825, 826, 828, 839, 843, 845, 846, 851, 853, 
    854, 857, 858, 860, 867, 869, 870, 873, 874, 876, 881, 882, 884, 888, 903, 907, 909, 910, 915, 917, 
    918, 921, 922, 924, 931, 933, 934, 937, 938, 940, 945, 946, 948, 952, 963, 965, 966, 969, 970, 972, 
    977, 978, 980, 984, 993, 994, 996, 1000, 1008, 1055, 1071, 1079, 1083, 1085, 1086, 1103, 1111, 1115, 1117, 1118, 
    1127, 1131, 1133, 1134, 1139, 1141, 1142, 1145, 1146, 1148, 1167, 1175, 1179, 1181, 1182, 1191, 1195, 1197, 1198, 1203, 
    1205, 1206, 1209, 1210, 1212, 1223, 1227, 1229, 1230, 1235, 1237, 1238, 1241, 1242, 1244, 1251, 1253, 1254, 1257, 1258, 
    1260, 1265, 1266, 1268, 1272, 1295, 1303, 1307, 1309, 1310, 1319, 1323, 1325, 1326, 1331, 1333, 1334, 1337, 1338, 1340, 
    1351, 1355, 1357, 1358, 1363, 1365, 1366, 1369, 1370, 1372, 1379, 1381, 1382, 1385, 1386, 1388, 1393, 1394, 1396, 1400, 
    1415, 1419, 1421, 1422, 1427, 1429, 1430, 1433, 1434, 1436, 1443, 1445, 1446, 1449, 1450, 1452, 1457, 1458, 1460, 1464, 
    1475, 1477, 1478, 1481, 1482, 1484, 1489, 1490, 1492, 1496, 1505, 1506, 1508, 1512, 1520, 1551, 1559, 1563, 1565, 1566, 
    1575, 1579, 1581, 1582, 1587, 1589, 1590, 1593, 1594, 1596, 1607, 1611, 1613, 1614, 1619, 1621, 1622, 1625, 1626, 1628, 
    1635, 1637, 1638, 1641, 1642, 1644, 1649, 1650, 1652, 1656, 1671, 1675, 1677, 1678, 1683, 1685, 1686, 1689, 1690, 1692, 
    1699, 1701, 1702, 1705, 1706, 1708, 1713, 1714, 1716, 1720, 1731, 1733, 1734, 1737, 1738, 1740, 1745, 1746, 1748, 1752, 
    1761, 1762, 1764, 1768, 1776, 1799, 1803, 1805, 1806, 1811, 1813, 1814, 1817, 1818, 1820, 1827, 1829, 1830, 1833, 1834, 
    1836, 1841, 1842, 1844, 1848, 1859, 1861, 1862, 1865, 1866, 1868, 1873, 1874, 1876, 1880, 1889, 1890, 1892, 1896, 1904, 
    1923, 1925, 1926, 1929, 1930, 1932, 1937, 1938, 1940, 1944, 1953, 1954, 1956, 1960, 1968, 1985, 1986, 1988, 1992, 2000, 
    2016, 2079, 2095, 2103, 2107, 2109, 2110, 2127, 2135, 2139, 2141, 2142, 2151, 2155, 2157, 2158, 2163, 2165, 2166, 2169, 
    2170, 2172, 2191, 2199, 2203, 2205, 2206, 2215, 2219, 2221, 2222, 2227, 2229, 2230, 2233, 2234, 2236, 2247, 2251, 2253, 
    2254, 2259, 2261, 2262, 2265, 2266, 2268, 2275, 2277, 2278, 2281, 2282, 2284, 2289, 2290, 2292, 2296, 2319, 2327, 2331, 
    2333, 2334, 2343, 2347, 2349, 2350, 2355, 2357, 2358, 2361, 2362, 2364, 2375, 2379, 2381, 2382, 2387, 2389, 2390, 2393, 
    2394, 2396, 2403, 2405, 2406, 2409, 2410, 2412, 2417, 2418, 2420, 2424, 2439, 2443, 2445, 2446, 2451, 2453, 2454, 2457, 
    2458, 2460, 2467, 2469, 2470, 2473, 2474, 2476, 2481, 2482, 2484, 2488, 2499, 2501, 2502, 2505, 2506, 2508, 2513, 2514, 
    2516, 2520, 2529, 2530, 2532, 2536, 2544, 2575, 2583, 2587, 2589, 2590, 2599, 2603, 2605, 2606, 2611, 2613, 2614, 2617, 
    2618, 2620, 2631, 2635, 2637, 2638, 2643, 2645, 2646, 2649, 2650, 2652, 2659, 2661, 2662, 2665, 2666, 2668, 2673, 2674, 
    2676, 2680, 2695, 2699, 2701, 2702, 2707, 2709, 2710, 2713, 2714, 2716, 2723, 2725, 2726, 2729, 2730, 2732, 2737, 2738, 
    2740, 2744, 2755, 2757, 2758, 2761, 2762, 2764, 2769, 2770, 2772, 2776, 2785, 2786, 2788, 2792, 2800, 2823, 2827, 2829, 
    2830, 2835, 2837, 2838, 2841, 2842, 2844, 2851, 2853, 2854, 2857, 2858, 2860, 2865, 2866, 2868, 2872, 2883, 2885, 2886, 
    2889, 2890, 2892, 2897, 2898, 2900, 2904, 2913, 2914, 2916, 2920, 2928, 2947, 2949, 2950, 2953, 2954, 2956, 2961, 2962, 
    2964, 2968, 2977, 2978, 2980, 2984, 2992, 3009, 3010, 3012, 3016, 3024, 3040, 3087, 3095, 3099, 3101, 3102, 3111, 3115, 
    3117, 3118, 3123, 3125, 3126, 3129, 3130, 3132, 3143, 3147, 3149, 3150, 3155, 3157, 3158, 3161, 3162, 3164, 3171, 3173, 
    3174, 3177, 3178, 3180, 3185, 3186, 3188, 3192, 3207, 3211, 3213, 3214, 3219, 3221, 3222, 3225, 3226, 3228, 3235, 3237, 
    3238, 3241, 3242, 3244, 3249, 3250, 3252, 3256, 3267, 3269, 3270, 3273, 3274, 3276, 3281, 3282, 3284, 3288, 3297, 3298, 
    3300, 3304, 3312, 3335, 3339, 3341, 3342, 3347, 3349, 3350, 3353, 3354, 3356, 3363, 3365, 3366, 3369, 3370, 3372, 3377, 
    3378, 3380, 3384, 3395, 3397, 3398, 3401, 3402, 3404, 3409, 3410, 3412, 3416, 3425, 3426, 3428, 3432, 3440, 3459, 3461, 
    3462, 3465, 3466, 3468, 3473, 3474, 3476, 3480, 3489, 3490, 3492, 3496, 3504, 3521, 3522, 3524, 3528, 3536, 3552, 3591, 
    3595, 3597, 3598, 3603, 3605, 3606, 3609, 3610, 3612, 3619, 3621, 3622, 3625, 3626, 3628, 3633, 3634, 3636, 3640, 3651, 
    3653, 3654, 3657, 3658, 3660, 3665, 3666, 3668, 3672, 3681, 3682, 3684, 3688, 3696, 3715, 3717, 3718, 3721, 3722, 3724, 
    3729, 3730, 3732, 3736, 3745, 3746, 3748, 3752, 3760, 3777, 3778, 3780, 3784, 3792, 3808, 3843, 3845, 3846, 3849, 3850, 
    3852, 3857, 3858, 3860, 3864, 3873, 3874, 3876, 3880, 3888, 3905, 3906, 3908, 3912, 3920, 3936, 3969, 3970, 3972, 3976, 
    3984, 4000, 4032, 4127, 4143, 4151, 4155, 4157, 4158, 4175, 4183, 4187, 4189, 4190, 4199, 4203, 4205, 4206, 4211, 4213, 
    4214, 4217, 4218, 4220, 4239, 4247, 4251, 4253, 4254, 4263, 4267, 4269, 4270, 4275, 4277, 4278, 4281, 4282, 4284, 4295, 
    4299, 4301, 4302, 4307, 4309, 4310, 4313, 4314, 4316, 4323, 4325, 4326, 4329, 4330, 4332, 4337, 4338, 4340, 4344, 4367, 
    4375, 4379, 4381, 4382, 4391, 4395, 4397, 4398, 4403, 4405, 4406, 4409, 4410, 4412, 4423, 4427, 4429, 4430, 4435, 4437, 
    4438, 4441, 4442, 4444, 4451, 4453, 4454, 4457, 4458, 4460, 4465, 4466, 4468, 4472, 4487, 4491, 4493, 4494, 4499, 4501, 
    4502, 4505, 4506, 4508, 4515, 4517, 4518, 4521, 4522, 4524, 4529, 4530, 4532, 4536, 4547, 4549, 4550, 4553, 4554, 4556, 
    4561, 4562, 4564, 4568, 4577, 4578, 4580, 4584, 4592, 4623, 4631, 4635, 4637, 4638, 4647, 4651, 4653, 4654, 4659, 4661, 
    4662, 4665, 4666, 4668, 4679, 4683, 4685, 4686, 4691, 4693, 4694, 4697, 4698, 4700, 4707, 4709, 4710, 4713, 4714, 4716, 
    4721, 4722, 4724, 4728, 4743, 4747, 4749, 4750, 4755, 4757, 4758, 4761, 4762, 4764, 4771, 4773, 4774, 4777, 4778, 4780, 
    4785, 4786, 4788, 4792, 4803, 4805, 4806, 4809, 4810, 4812, 4817, 4818, 4820, 4824, 4833, 4834, 4836, 4840, 4848, 4871, 
    4875, 4877, 4878, 4883, 4885, 4886, 4889, 4890, 4892, 4899, 4901, 4902, 4905, 4906, 4908, 4913, 4914, 4916, 4920, 4931, 
    4933, 4934, 4937, 4938, 4940, 4945, 4946, 4948, 4952, 4961, 4962, 4964, 4968, 4976, 4995, 4997, 4998, 5001, 5002, 5004, 
    5009, 5010, 5012, 5016, 5025, 5026, 5028, 5032, 5040, 5057, 5058, 5060, 5064, 5072, 5088, 5135, 5143, 5147, 5149, 5150, 
    5159, 5163, 5165, 5166, 5171, 5173, 5174, 5177, 5178, 5180, 5191, 5195, 5197, 5198, 5203, 5205, 5206, 5209, 5210, 5212, 
    5219, 5221, 5222, 5225, 5226, 5228, 5233, 5234, 5236, 5240, 5255, 5259, 5261, 5262, 5267, 5269, 5270, 5273, 5274, 5276, 
    5283, 5285, 5286, 5289, 5290, 5292, 5297, 5298, 5300, 5304, 5315, 5317, 5318, 5321, 5322, 5324, 5329, 5330, 5332, 5336, 
    5345, 5346, 5348, 5352, 5360, 5383, 5387, 5389, 5390, 5395, 5397, 5398, 5401, 5402, 5404, 5411, 5413, 5414, 5417, 5418, 
    5420, 5425, 5426, 5428, 5432, 5443, 5445, 5446, 5449, 5450, 5452, 5457, 5458, 5460, 5464, 5473, 5474, 5476, 5480, 5488, 
    5507, 5509, 5510, 5513, 5514, 5516, 5521, 5522, 5524, 5528, 5537, 5538, 5540, 5544, 5552, 5569, 5570, 5572, 5576, 5584, 
    5600, 5639, 5643, 5645, 5646, 5651, 5653, 5654, 5657, 5658, 5660, 5667, 5669, 5670, 5673, 5674, 5676, 5681, 5682, 5684, 
    5688, 5699, 5701, 5702, 5705, 5706, 5708, 5713, 5714, 5716, 5720, 5729, 5730, 5732, 5736, 5744, 5763, 5765, 5766, 5769, 
    5770, 5772, 5777, 5778, 5780, 5784, 5793, 5794, 5796, 5800, 5808, 5825, 5826, 5828, 5832, 5840, 5856, 5891, 5893, 5894, 
    5897, 5898, 5900, 5905, 5906, 5908, 5912, 5921, 5922, 5924, 5928, 5936, 5953, 5954, 5956, 5960, 5968, 5984, 6017, 6018, 
    6020, 6024, 6032, 6048, 6080, 6159, 6167, 6171, 6173, 6174, 6183, 6187, 6189, 6190, 6195, 6197, 6198, 6201, 6202, 6204, 
    6215, 6219, 6221, 6222, 6227, 6229, 6230, 6233, 6234, 6236, 6243, 6245, 6246, 6249, 6250, 6252, 6257, 6258, 6260, 6264, 
    6279, 6283, 6285, 6286, 6291, 6293, 6294, 6297, 6298, 6300, 6307, 6309, 6310, 6313, 6314, 6316, 6321, 6322, 6324, 6328, 
    6339, 6341, 6342, 6345, 6346, 6348, 6353, 6354, 6356, 6360, 6369, 6370, 6372, 6376, 6384, 6407, 6411, 6413, 6414, 6419, 
    6421, 6422, 6425, 6426, 6428, 6435, 6437, 6438, 6441, 6442, 6444, 6449, 6450, 6452, 6456, 6467, 6469, 6470, 6473, 6474, 
    6476, 6481, 6482, 6484, 6488, 6497, 6498, 6500, 6504, 6512, 6531, 6533, 6534, 6537, 6538, 6540, 6545, 6546, 6548, 6552, 
    6561, 6562, 6564, 6568, 6576, 6593, 6594, 6596, 6600, 6608, 6624, 6663, 6667, 6669, 6670, 6675, 6677, 6678, 6681, 6682, 
    6684, 6691, 6693, 6694, 6697, 6698, 6700, 6705, 6706, 6708, 6712, 6723, 6725, 6726, 6729, 6730, 6732, 6737, 6738, 6740, 
    6744, 6753, 6754, 6756, 6760, 6768, 6787, 6789, 6790, 6793, 6794, 6796, 6801, 6802, 6804, 6808, 6817, 6818, 6820, 6824, 
    6832, 6849, 6850, 6852, 6856, 6864, 6880, 6915, 6917, 6918, 6921, 6922, 6924, 6929, 6930, 6932, 6936, 6945, 6946, 6948, 
    6952, 6960, 6977, 6978, 6980, 6984, 6992, 7008, 7041, 7042, 7044, 7048, 7056, 7072, 7104, 7175, 7179, 7181, 7182, 7187, 
    7189, 7190, 7193, 7194, 7196, 7203, 7205, 7206, 7209, 7210, 7212, 7217, 7218, 7220, 7224, 7235, 7237, 7238, 7241, 7242, 
    7244, 7249, 7250, 7252, 7256, 7265, 7266, 7268, 7272, 7280, 7299, 7301, 7302, 7305, 7306, 7308, 7313, 7314, 7316, 7320, 
    7329, 7330, 7332, 7336, 7344, 7361, 7362, 7364, 7368, 7376, 7392, 7427, 7429, 7430, 7433, 7434, 7436, 7441, 7442, 7444, 
    7448, 7457, 7458, 7460, 7464, 7472, 7489, 7490, 7492, 7496, 7504, 7520, 7553, 7554, 7556, 7560, 7568, 7584, 7616, 7683, 
    7685, 7686, 7689, 7690, 7692, 7697, 7698, 7700, 7704, 7713, 7714, 7716, 7720, 7728, 7745, 7746, 7748, 7752, 7760, 7776, 
    7809, 7810, 7812, 7816, 7824, 7840, 7872, 7937, 7938, 7940, 7944, 7952, 7968, 8000, 8064, 8223, 8239, 8247, 8251, 8253, 
    8254, 8271, 8279, 8283, 8285, 8286, 8295, 8299, 8301, 8302, 8307, 8309, 8310, 8313, 8314, 8316, 8335, 8343, 8347, 8349, 
    8350, 8359, 8363, 8365, 8366, 8371, 8373, 8374, 8377, 8378, 8380, 8391, 8395, 8397, 8398, 8403, 8405, 8406, 8409, 8410, 
    8412, 8419, 8421, 8422, 8425, 8426, 8428, 8433, 8434, 8436, 8440, 8463, 8471, 8475, 8477, 8478, 8487, 8491, 8493, 8494, 
    8499, 8501, 8502, 8505, 8506, 8508, 8519, 8523, 8525, 8526, 8531, 8533, 8534, 8537, 8538, 8540, 8547, 8549, 8550, 8553, 
    8554, 8556, 8561, 8562, 8564, 8568, 8583, 8587, 8589, 8590, 8595, 8597, 8598, 8601, 8602, 8604, 8611, 8613, 8614, 8617, 
    8618, 8620, 8625, 8626, 8628, 8632, 8643, 8645, 8646, 8649, 8650, 8652, 8657, 8658, 8660, 8664, 8673, 8674, 8676, 8680, 
    8688, 8719, 8727, 8731, 8733, 8734, 8743, 8747, 8749, 8750, 8755, 8757, 8758, 8761, 8762, 8764, 8775, 8779, 8781, 8782, 
    8787, 8789, 8790, 8793, 8794, 8796, 8803, 8805, 8806, 8809, 8810, 8812, 8817, 8818, 8820, 8824, 8839, 8843, 8845, 8846, 
    8851, 8853, 8854, 8857, 8858, 8860, 8867, 8869, 8870, 8873, 8874, 8876, 8881, 8882, 8884, 8888, 8899, 8901, 8902, 8905, 
    8906, 8908, 8913, 8914, 8916, 8920, 8929, 8930, 8932, 8936, 8944, 8967, 8971, 8973, 8974, 8979, 8981, 8982, 8985, 8986, 
    8988, 8995, 8997, 8998, 9001, 9002, 9004, 9009, 9010, 9012, 9016, 9027, 9029, 9030, 9033, 9034, 9036, 9041, 9042, 9044, 
    9048, 9057, 9058, 9060, 9064, 9072, 9091, 9093, 9094, 9097, 9098, 9100, 9105, 9106, 9108, 9112, 9121, 9122, 9124, 9128, 
    9136, 9153, 9154, 9156, 9160, 9168, 9184, 9231, 9239, 9243, 9245, 9246, 9255, 9259, 9261, 9262, 9267, 9269, 9270, 9273, 
    9274, 9276, 9287, 9291, 9293, 9294, 9299, 9301, 9302, 9305, 9306, 9308, 9315, 9317, 9318, 9321, 9322, 9324, 9329, 9330, 
    9332, 9336, 9351, 9355, 9357, 9358, 9363, 9365, 9366, 9369, 9370, 9372, 9379, 9381, 9382, 9385, 9386, 9388, 9393, 9394, 
    9396, 9400, 9411, 9413, 9414, 9417, 9418, 9420, 9425, 9426, 9428, 9432, 9441, 9442, 9444, 9448, 9456, 9479, 9483, 9485, 
    9486, 9491, 9493, 9494, 9497, 9498, 9500, 9507, 9509, 9510, 9513, 9514, 9516, 9521, 9522, 9524, 9528, 9539, 9541, 9542, 
    9545, 9546, 9548, 9553, 9554, 9556, 9560, 9569, 9570, 9572, 9576, 9584, 9603, 9605, 9606, 9609, 9610, 9612, 9617, 9618, 
    9620, 9624, 9633, 9634, 9636, 9640, 9648, 9665, 9666, 9668, 9672, 9680, 9696, 9735, 9739, 9741, 9742, 9747, 9749, 9750, 
    9753, 9754, 9756, 9763, 9765, 9766, 9769, 9770, 9772, 9777, 9778, 9780, 9784, 9795, 9797, 9798, 9801, 9802, 9804, 9809, 
    9810, 9812, 9816, 9825, 9826, 9828, 9832, 9840, 9859, 9861, 9862, 9865, 9866, 9868, 9873, 9874, 9876, 9880, 9889, 9890, 
    9892, 9896, 9904, 9921, 9922, 9924, 9928, 9936, 9952, 9987, 9989, 9990, 9993, 9994, 9996, 10001, 10002, 10004, 10008, 10017, 
    10018, 10020, 10024, 10032, 10049, 10050, 10052, 10056, 10064, 10080, 10113, 10114, 10116, 10120, 10128, 10144, 10176, 10255, 10263, 10267, 
    10269, 10270, 10279, 10283, 10285, 10286, 10291, 10293, 10294, 10297, 10298, 10300, 10311, 10315, 10317, 10318, 10323, 10325, 10326, 10329, 
    10330, 10332, 10339, 10341, 10342, 10345, 10346, 10348, 10353, 10354, 10356, 10360, 10375, 10379, 10381, 10382, 10387, 10389, 10390, 10393, 
    10394, 10396, 10403, 10405, 10406, 10409, 10410, 10412, 10417, 10418, 10420, 10424, 10435, 10437, 10438, 10441, 10442, 10444, 10449, 10450, 
    10452, 10456, 10465, 10466, 10468, 10472, 10480, 10503, 10507, 10509, 10510, 10515, 10517, 10518, 10521, 10522, 10524, 10531, 10533, 10534, 
    10537, 10538, 10540, 10545, 10546, 10548, 10552, 10563, 10565, 10566, 10569, 10570, 10572, 10577, 10578, 10580, 10584, 10593, 10594, 10596, 
    10600, 10608, 10627, 10629, 10630, 10633, 10634, 10636, 10641, 10642, 10644, 10648, 10657, 10658, 10660, 10664, 10672, 10689, 10690, 10692, 
    10696, 10704, 10720, 10759, 10763, 10765, 10766, 10771, 10773, 10774, 10777, 10778, 10780, 10787, 10789, 10790, 10793, 10794, 10796, 10801, 
    10802, 10804, 10808, 10819, 10821, 10822, 10825, 10826, 10828, 10833, 10834, 10836, 10840, 10849, 10850, 10852, 10856, 10864, 10883, 10885, 
    10886, 10889, 10890, 10892, 10897, 10898, 10900, 10904, 10913, 10914, 10916, 10920, 10928, 10945, 10946, 10948, 10952, 10960, 10976, 11011, 
    11013, 11014, 11017, 11018, 11020, 11025, 11026, 11028, 11032, 11041, 11042, 11044, 11048, 11056, 11073, 11074, 11076, 11080, 11088, 11104, 
    11137, 11138, 11140, 11144, 11152, 11168, 11200, 11271, 11275, 11277, 11278, 11283, 11285, 11286, 11289, 11290, 11292, 11299, 11301, 11302, 
    11305, 11306, 11308, 11313, 11314, 11316, 11320, 11331, 11333, 11334, 11337, 11338, 11340, 11345, 11346, 11348, 11352, 11361, 11362, 11364, 
    11368, 11376, 11395, 11397, 11398, 11401, 11402, 11404, 11409, 11410, 11412, 11416, 11425, 11426, 11428, 11432, 11440, 11457, 11458, 11460, 
    11464, 11472, 11488, 11523, 11525, 11526, 11529, 11530, 11532, 11537, 11538, 11540, 11544, 11553, 11554, 11556, 11560, 11568, 11585, 11586, 
    11588, 11592, 11600, 11616, 11649, 11650, 11652, 11656, 11664, 11680, 11712, 11779, 11781, 11782, 11785, 11786, 11788, 11793, 11794, 11796, 
    11800, 11809, 11810, 11812, 11816, 11824, 11841, 11842, 11844, 11848, 11856, 11872, 11905, 11906, 11908, 11912, 11920, 11936, 11968, 12033, 
    12034, 12036, 12040, 12048, 12064, 12096, 12160, 12303, 12311, 12315, 12317, 12318, 12327, 12331, 12333, 12334, 12339, 12341, 12342, 12345, 
    12346, 12348, 12359, 12363, 12365, 12366, 12371, 12373, 12374, 12377, 12378, 12380, 12387, 12389, 12390, 12393, 12394, 12396, 12401, 12402, 
    12404, 12408, 12423, 12427, 12429, 12430, 12435, 12437, 12438, 12441, 12442, 12444, 12451, 12453, 12454, 12457, 12458, 12460, 12465, 12466, 
    12468, 12472, 12483, 12485, 12486, 12489, 12490, 12492, 12497, 12498, 12500, 12504, 12513, 12514, 12516, 12520, 12528, 12551, 12555, 12557, 
    12558, 12563, 12565, 12566, 12569, 12570, 12572, 12579, 12581, 12582, 12585, 12586, 12588, 12593, 12594, 12596, 12600, 12611, 12613, 12614, 
    12617, 12618, 12620, 12625, 12626, 12628, 12632, 12641, 12642, 12644, 12648, 12656, 12675, 12677, 12678, 12681, 12682, 12684, 12689, 12690, 
    12692, 12696, 12705, 12706, 12708, 12712, 12720, 12737, 12738, 12740, 12744, 12752, 12768, 12807, 12811, 12813, 12814, 12819, 12821, 12822, 
    12825, 12826, 12828, 12835, 12837, 12838, 12841, 12842, 12844, 12849, 12850, 12852, 12856, 12867, 12869, 12870, 12873, 12874, 12876, 12881, 
    12882, 12884, 12888, 12897, 12898, 12900, 12904, 12912, 12931, 12933, 12934, 12937, 12938, 12940, 12945, 12946, 12948, 12952, 12961, 12962, 
    12964, 12968, 12976, 12993, 12994, 12996, 13000, 13008, 13024, 13059, 13061, 13062, 13065, 13066, 13068, 13073, 13074, 13076, 13080, 13089, 
    13090, 13092, 13096, 13104, 13121, 13122, 13124, 13128, 13136, 13152, 13185, 13186, 13188, 13192, 13200, 13216, 13248, 13319, 13323, 13325, 
    13326, 13331, 13333, 13334, 13337, 13338, 13340, 13347, 13349, 13350, 13353, 13354, 13356, 13361, 13362, 13364, 13368, 13379, 13381, 13382, 
    13385, 13386, 13388, 13393, 13394, 13396, 13400, 13409, 13410, 13412, 13416, 13424, 13443, 13445, 13446, 13449, 13450, 13452, 13457, 13458, 
    13460, 13464, 13473, 13474, 13476, 13480, 13488, 13505, 13506, 13508, 13512, 13520, 13536, 13571, 13573, 13574, 13577, 13578, 13580, 13585, 
    13586, 13588, 13592, 13601, 13602, 13604, 13608, 13616, 13633, 13634, 13636, 13640, 13648, 13664, 13697, 13698, 13700, 13704, 13712, 13728, 
    13760, 13827, 13829, 13830, 13833, 13834, 13836, 13841, 13842, 13844, 13848, 13857, 13858, 13860, 13864, 13872, 13889, 13890, 13892, 13896, 
    13904, 13920, 13953, 13954, 13956, 13960, 13968, 13984, 14016, 14081, 14082, 14084, 14088, 14096, 14112, 14144, 14208, 14343, 14347, 14349, 
    14350, 14355, 14357, 14358, 14361, 14362, 14364, 14371, 14373, 14374, 14377, 14378, 14380, 14385, 14386, 14388, 14392, 14403, 14405, 14406, 
    14409, 14410, 14412, 14417, 14418, 14420, 14424, 14433, 14434, 14436, 14440, 14448, 14467, 14469, 14470, 14473, 14474, 14476, 14481, 14482, 
    14484, 14488, 14497, 14498, 14500, 14504, 14512, 14529, 14530, 14532, 14536, 14544, 14560, 14595, 14597, 14598, 14601, 14602, 14604, 14609, 
    14610, 14612, 14616, 14625, 14626, 14628, 14632, 14640, 14657, 14658, 14660, 14664, 14672, 14688, 14721, 14722, 14724, 14728, 14736, 14752, 
    14784, 14851, 14853, 14854, 14857, 14858, 14860, 14865, 14866, 14868, 14872, 14881, 14882, 14884, 14888, 14896, 14913, 14914, 14916, 14920, 
    14928, 14944, 14977, 14978, 14980, 14984, 14992, 15008, 15040, 15105, 15106, 15108, 15112, 15120, 15136, 15168, 15232, 15363, 15365, 15366, 
    15369, 15370, 15372, 15377, 15378, 15380, 15384, 15393, 15394, 15396, 15400, 15408, 15425, 15426, 15428, 15432, 15440, 15456, 15489, 15490, 
    15492, 15496, 15504, 15520, 15552, 15617, 15618, 15620, 15624, 15632, 15648, 15680, 15744, 15873, 15874, 15876, 15880, 15888, 15904, 15936, 
    16000, 16128, 16415, 16431, 16439, 16443, 16445, 16446, 16463, 16471, 16475, 16477, 16478, 16487, 16491, 16493, 16494, 16499, 16501, 16502, 
    16505, 16506, 16508, 16527, 16535, 16539, 16541, 16542, 16551, 16555, 16557, 16558, 16563, 16565, 16566, 16569, 16570, 16572, 16583, 16587, 
    16589, 16590, 16595, 16597, 16598, 16601, 16602, 16604, 16611, 16613, 16614, 16617, 16618, 16620, 16625, 16626, 16628, 16632, 16655, 16663, 
    16667, 16669, 16670, 16679, 16683, 16685, 16686, 16691, 16693, 16694, 16697, 16698, 16700, 16711, 16715, 16717, 16718, 16723, 16725, 16726, 
    16729, 16730, 16732, 16739, 16741, 16742, 16745, 16746, 16748, 16753, 16754, 16756, 16760, 16775, 16779, 16781, 16782, 16787, 16789, 16790, 
    16793, 16794, 16796, 16803, 16805, 16806, 16809, 16810, 16812, 16817, 16818, 16820, 16824, 16835, 16837, 16838, 16841, 16842, 16844, 16849, 
    16850, 16852, 16856, 16865, 16866, 16868, 16872, 16880, 16911, 16919, 16923, 16925, 16926, 16935, 16939, 16941, 16942, 16947, 16949, 16950, 
    16953, 16954, 16956, 16967, 16971, 16973, 16974, 16979, 16981, 16982, 16985, 16986, 16988, 16995, 16997, 16998, 17001, 17002, 17004, 17009, 
    17010, 17012, 17016, 17031, 17035, 17037, 17038, 17043, 17045, 17046, 17049, 17050, 17052, 17059, 17061, 17062, 17065, 17066, 17068, 17073, 
    17074, 17076, 17080, 17091, 17093, 17094, 17097, 17098, 17100, 17105, 17106, 17108, 17112, 17121, 17122, 17124, 17128, 17136, 17159, 17163, 
    17165, 17166, 17171, 17173, 17174, 17177, 17178, 17180, 17187, 17189, 17190, 17193, 17194, 17196, 17201, 17202, 17204, 17208, 17219, 17221, 
    17222, 17225, 17226, 17228, 17233, 17234, 17236, 17240, 17249, 17250, 17252, 17256, 17264, 17283, 17285, 17286, 17289, 17290, 17292, 17297, 
    17298, 17300, 17304, 17313, 17314, 17316, 17320, 17328, 17345, 17346, 17348, 17352, 17360, 17376, 17423, 17431, 17435, 17437, 17438, 17447, 
    17451, 17453, 17454, 17459, 17461, 17462, 17465, 17466, 17468, 17479, 17483, 17485, 17486, 17491, 17493, 17494, 17497, 17498, 17500, 17507, 
    17509, 17510, 17513, 17514, 17516, 17521, 17522, 17524, 17528, 17543, 17547, 17549, 17550, 17555, 17557, 17558, 17561, 17562, 17564, 17571, 
    17573, 17574, 17577, 17578, 17580, 17585, 17586, 17588, 17592, 17603, 17605, 17606, 17609, 17610, 17612, 17617, 17618, 17620, 17624, 17633, 
    17634, 17636, 17640, 17648, 17671, 17675, 17677, 17678, 17683, 17685, 17686, 17689, 17690, 17692, 17699, 17701, 17702, 17705, 17706, 17708, 
    17713, 17714, 17716, 17720, 17731, 17733, 17734, 17737, 17738, 17740, 17745, 17746, 17748, 17752, 17761, 17762, 17764, 17768, 17776, 17795, 
    17797, 17798, 17801, 17802, 17804, 17809, 17810, 17812, 17816, 17825, 17826, 17828, 17832, 17840, 17857, 17858, 17860, 17864, 17872, 17888, 
    17927, 17931, 17933, 17934, 17939, 17941, 17942, 17945, 17946, 17948, 17955, 17957, 17958, 17961, 17962, 17964, 17969, 17970, 17972, 17976, 
    17987, 17989, 17990, 17993, 17994, 17996, 18001, 18002, 18004, 18008, 18017, 18018, 18020, 18024, 18032, 18051, 18053, 18054, 18057, 18058, 
    18060, 18065, 18066, 18068, 18072, 18081, 18082, 18084, 18088, 18096, 18113, 18114, 18116, 18120, 18128, 18144, 18179, 18181, 18182, 18185, 
    18186, 18188, 18193, 18194, 18196, 18200, 18209, 18210, 18212, 18216, 18224, 18241, 18242, 18244, 18248, 18256, 18272, 18305, 18306, 18308, 
    18312, 18320, 18336, 18368, 18447, 18455, 18459, 18461, 18462, 18471, 18475, 18477, 18478, 18483, 18485, 18486, 18489, 18490, 18492, 18503, 
    18507, 18509, 18510, 18515, 18517, 18518, 18521, 18522, 18524, 18531, 18533, 18534, 18537, 18538, 18540, 18545, 18546, 18548, 18552, 18567, 
    18571, 18573, 18574, 18579, 18581, 18582, 18585, 18586, 18588, 18595, 18597, 18598, 18601, 18602, 18604, 18609, 18610, 18612, 18616, 18627, 
    18629, 18630, 18633, 18634, 18636, 18641, 18642, 18644, 18648, 18657, 18658, 18660, 18664, 18672, 18695, 18699, 18701, 18702, 18707, 18709, 
    18710, 18713, 18714, 18716, 18723, 18725, 18726, 18729, 18730, 18732, 18737, 18738, 18740, 18744, 18755, 18757, 18758, 18761, 18762, 18764, 
    18769, 18770, 18772, 18776, 18785, 18786, 18788, 18792, 18800, 18819, 18821, 18822, 18825, 18826, 18828, 18833, 18834, 18836, 18840, 18849, 
    18850, 18852, 18856, 18864, 18881, 18882, 18884, 18888, 18896, 18912, 18951, 18955, 18957, 18958, 18963, 18965, 18966, 18969, 18970, 18972, 
    18979, 18981, 18982, 18985, 18986, 18988, 18993, 18994, 18996, 19000, 19011, 19013, 19014, 19017, 19018, 19020, 19025, 19026, 19028, 19032, 
    19041, 19042, 19044, 19048, 19056, 19075, 19077, 19078, 19081, 19082, 19084, 19089, 19090, 19092, 19096, 19105, 19106, 19108, 19112, 19120, 
    19137, 19138, 19140, 19144, 19152, 19168, 19203, 19205, 19206, 19209, 19210, 19212, 19217, 19218, 19220, 19224, 19233, 19234, 19236, 19240, 
    19248, 19265, 19266, 19268, 19272, 19280, 19296, 19329, 19330, 19332, 19336, 19344, 19360, 19392, 19463, 19467, 19469, 19470, 19475, 19477, 
    19478, 19481, 19482, 19484, 19491, 19493, 19494, 19497, 19498, 19500, 19505, 19506, 19508, 19512, 19523, 19525, 19526, 19529, 19530, 19532, 
    19537, 19538, 19540, 19544, 19553, 19554, 19556, 19560, 19568, 19587, 19589, 19590, 19593, 19594, 19596, 19601, 19602, 19604, 19608, 19617, 
    19618, 19620, 19624, 19632, 19649, 19650, 19652, 19656, 19664, 19680, 19715, 19717, 19718, 19721, 19722, 19724, 19729, 19730, 19732, 19736, 
    19745, 19746, 19748, 19752, 19760, 19777, 19778, 19780, 19784, 19792, 19808, 19841, 19842, 19844, 19848, 19856, 19872, 19904, 19971, 19973, 
    19974, 19977, 19978, 19980, 19985, 19986, 19988, 19992, 20001, 20002, 20004, 20008, 20016, 20033, 20034, 20036, 20040, 20048, 20064, 20097, 
    20098, 20100, 20104, 20112, 20128, 20160, 20225, 20226, 20228, 20232, 20240, 20256, 20288, 20352, 20495, 20503, 20507, 20509, 20510, 20519, 
    20523, 20525, 20526, 20531, 20533, 20534, 20537, 20538, 20540, 20551, 20555, 20557, 20558, 20563, 20565, 20566, 20569, 20570, 20572, 20579, 
    20581, 20582, 20585, 20586, 20588, 20593, 20594, 20596, 20600, 20615, 20619, 20621, 20622, 20627, 20629, 20630, 20633, 20634, 20636, 20643, 
    20645, 20646, 20649, 20650, 20652, 20657, 20658, 20660, 20664, 20675, 20677, 20678, 20681, 20682, 20684, 20689, 20690, 20692, 20696, 20705, 
    20706, 20708, 20712, 20720, 20743, 20747, 20749, 20750, 20755, 20757, 20758, 20761, 20762, 20764, 20771, 20773, 20774, 20777, 20778, 20780, 
    20785, 20786, 20788, 20792, 20803, 20805, 20806, 20809, 20810, 20812, 20817, 20818, 20820, 20824, 20833, 20834, 20836, 20840, 20848, 20867, 
    20869, 20870, 20873, 20874, 20876, 20881, 20882, 20884, 20888, 20897, 20898, 20900, 20904, 20912, 20929, 20930, 20932, 20936, 20944, 20960, 
    20999, 21003, 21005, 21006, 21011, 21013, 21014, 21017, 21018, 21020, 21027, 21029, 21030, 21033, 21034, 21036, 21041, 21042, 21044, 21048, 
    21059, 21061, 21062, 21065, 21066, 21068, 21073, 21074, 21076, 21080, 21089, 21090, 21092, 21096, 21104, 21123, 21125, 21126, 21129, 21130, 
    21132, 21137, 21138, 21140, 21144, 21153, 21154, 21156, 21160, 21168, 21185, 21186, 21188, 21192, 21200, 21216, 21251, 21253, 21254, 21257, 
    21258, 21260, 21265, 21266, 21268, 21272, 21281, 21282, 21284, 21288, 21296, 21313, 21314, 21316, 21320, 21328, 21344, 21377, 21378, 21380, 
    21384, 21392, 21408, 21440, 21511, 21515, 21517, 21518, 21523, 21525, 21526, 21529, 21530, 21532, 21539, 21541, 21542, 21545, 21546, 21548, 
    21553, 21554, 21556, 21560, 21571, 21573, 21574, 21577, 21578, 21580, 21585, 21586, 21588, 21592, 21601, 21602, 21604, 21608, 21616, 21635, 
    21637, 21638, 21641, 21642, 21644, 21649, 21650, 21652, 21656, 21665, 21666, 21668, 21672, 21680, 21697, 21698, 21700, 21704, 21712, 21728, 
    21763, 21765, 21766, 21769, 21770, 21772, 21777, 21778, 21780, 21784, 21793, 21794, 21796, 21800, 21808, 21825, 21826, 21828, 21832, 21840, 
    21856, 21889, 21890, 21892, 21896, 21904, 21920, 21952, 22019, 22021, 22022, 22025, 22026, 22028, 22033, 22034, 22036, 22040, 22049, 22050, 
    22052, 22056, 22064, 22081, 22082, 22084, 22088, 22096, 22112, 22145, 22146, 22148, 22152, 22160, 22176, 22208, 22273, 22274, 22276, 22280, 
    22288, 22304, 22336, 22400, 22535, 22539, 22541, 22542, 22547, 22549, 22550, 22553, 22554, 22556, 22563, 22565, 22566, 22569, 22570, 22572, 
    22577, 22578, 22580, 22584, 22595, 22597, 22598, 22601, 22602, 22604, 22609, 22610, 22612, 22616, 22625, 22626, 22628, 22632, 22640, 22659, 
    22661, 22662, 22665, 22666, 22668, 22673, 22674, 22676, 22680, 22689, 22690, 22692, 22696, 22704, 22721, 22722, 22724, 22728, 22736, 22752, 
    22787, 22789, 22790, 22793, 22794, 22796, 22801, 22802, 22804, 22808, 22817, 22818, 22820, 22824, 22832, 22849, 22850, 22852, 22856, 22864, 
    22880, 22913, 22914, 22916, 22920, 22928, 22944, 22976, 23043, 23045, 23046, 23049, 23050, 23052, 23057, 23058, 23060, 23064, 23073, 23074, 
    23076, 23080, 23088, 23105, 23106, 23108, 23112, 23120, 23136, 23169, 23170, 23172, 23176, 23184, 23200, 23232, 23297, 23298, 23300, 23304, 
    23312, 23328, 23360, 23424, 23555, 23557, 23558, 23561, 23562, 23564, 23569, 23570, 23572, 23576, 23585, 23586, 23588, 23592, 23600, 23617, 
    23618, 23620, 23624, 23632, 23648, 23681, 23682, 23684, 23688, 23696, 23712, 23744, 23809, 23810, 23812, 23816, 23824, 23840, 23872, 23936, 
    24065, 24066, 24068, 24072, 24080, 24096, 24128, 24192, 24320, 24591, 24599, 24603, 24605, 24606, 24615, 24619, 24621, 24622, 24627, 24629, 
    24630, 24633, 24634, 24636, 24647, 24651, 24653, 24654, 24659, 24661, 24662, 24665, 24666, 24668, 24675, 24677, 24678, 24681, 24682, 24684, 
    24689, 24690, 24692, 24696, 24711, 24715, 24717, 24718, 24723, 24725, 24726, 24729, 24730, 24732, 24739, 24741, 24742, 24745, 24746, 24748, 
    24753, 24754, 24756, 24760, 24771, 24773, 24774, 24777, 24778, 24780, 24785, 24786, 24788, 24792, 24801, 24802, 24804, 24808, 24816, 24839, 
    24843, 24845, 24846, 24851, 24853, 24854, 24857, 24858, 24860, 24867, 24869, 24870, 24873, 24874, 24876, 24881, 24882, 24884, 24888, 24899, 
    24901, 24902, 24905, 24906, 24908, 24913, 24914, 24916, 24920, 24929, 24930, 24932, 24936, 24944, 24963, 24965, 24966, 24969, 24970, 24972, 
    24977, 24978, 24980, 24984, 24993, 24994, 24996, 25000, 25008, 25025, 25026, 25028, 25032, 25040, 25056, 25095, 25099, 25101, 25102, 25107, 
    25109, 25110, 25113, 25114, 25116, 25123, 25125, 25126, 25129, 25130, 25132, 25137, 25138, 25140, 25144, 25155, 25157, 25158, 25161, 25162, 
    25164, 25169, 25170, 25172, 25176, 25185, 25186, 25188, 25192, 25200, 25219, 25221, 25222, 25225, 25226, 25228, 25233, 25234, 25236, 25240, 
    25249, 25250, 25252, 25256, 25264, 25281, 25282, 25284, 25288, 25296, 25312, 25347, 25349, 25350, 25353, 25354, 25356, 25361, 25362, 25364, 
    25368, 25377, 25378, 25380, 25384, 25392, 25409, 25410, 25412, 25416, 25424, 25440, 25473, 25474, 25476, 25480, 25488, 25504, 25536, 25607, 
    25611, 25613, 25614, 25619, 25621, 25622, 25625, 25626, 25628, 25635, 25637, 25638, 25641, 25642, 25644, 25649, 25650, 25652, 25656, 25667, 
    25669, 25670, 25673, 25674, 25676, 25681, 25682, 25684, 25688, 25697, 25698, 25700, 25704, 25712, 25731, 25733, 25734, 25737, 25738, 25740, 
    25745, 25746, 25748, 25752, 25761, 25762, 25764, 25768, 25776, 25793, 25794, 25796, 25800, 25808, 25824, 25859, 25861, 25862, 25865, 25866, 
    25868, 25873, 25874, 25876, 25880, 25889, 25890, 25892, 25896, 25904, 25921, 25922, 25924, 25928, 25936, 25952, 25985, 25986, 25988, 25992, 
    26000, 26016, 26048, 26115, 26117, 26118, 26121, 26122, 26124, 26129, 26130, 26132, 26136, 26145, 26146, 26148, 26152, 26160, 26177, 26178, 
    26180, 26184, 26192, 26208, 26241, 26242, 26244, 26248, 26256, 26272, 26304, 26369, 26370, 26372, 26376, 26384, 26400, 26432, 26496, 26631, 
    26635, 26637, 26638, 26643, 26645, 26646, 26649, 26650, 26652, 26659, 26661, 26662, 26665, 26666, 26668, 26673, 26674, 26676, 26680, 26691, 
    26693, 26694, 26697, 26698, 26700, 26705, 26706, 26708, 26712, 26721, 26722, 26724, 26728, 26736, 26755, 26757, 26758, 26761, 26762, 26764, 
    26769, 26770, 26772, 26776, 26785, 26786, 26788, 26792, 26800, 26817, 26818, 26820, 26824, 26832, 26848, 26883, 26885, 26886, 26889, 26890, 
    26892, 26897, 26898, 26900, 26904, 26913, 26914, 26916, 26920, 26928, 26945, 26946, 26948, 26952, 26960, 26976, 27009, 27010, 27012, 27016, 
    27024, 27040, 27072, 27139, 27141, 27142, 27145, 27146, 27148, 27153, 27154, 27156, 27160, 27169, 27170, 27172, 27176, 27184, 27201, 27202, 
    27204, 27208, 27216, 27232, 27265, 27266, 27268, 27272, 27280, 27296, 27328, 27393, 27394, 27396, 27400, 27408, 27424, 27456, 27520, 27651, 
    27653, 27654, 27657, 27658, 27660, 27665, 27666, 27668, 27672, 27681, 27682, 27684, 27688, 27696, 27713, 27714, 27716, 27720, 27728, 27744, 
    27777, 27778, 27780, 27784, 27792, 27808, 27840, 27905, 27906, 27908, 27912, 27920, 27936, 27968, 28032, 28161, 28162, 28164, 28168, 28176, 
    28192, 28224, 28288, 28416, 28679, 28683, 28685, 28686, 28691, 28693, 28694, 28697, 28698, 28700, 28707, 28709, 28710, 28713, 28714, 28716, 
    28721, 28722, 28724, 28728, 28739, 28741, 28742, 28745, 28746, 28748, 28753, 28754, 28756, 28760, 28769, 28770, 28772, 28776, 28784, 28803, 
    28805, 28806, 28809, 28810, 28812, 28817, 28818, 28820, 28824, 28833, 28834, 28836, 28840, 28848, 28865, 28866, 28868, 28872, 28880, 28896, 
    28931, 28933, 28934, 28937, 28938, 28940, 28945, 28946, 28948, 28952, 28961, 28962, 28964, 28968, 28976, 28993, 28994, 28996, 29000, 29008, 
    29024, 29057, 29058, 29060, 29064, 29072, 29088, 29120, 29187, 29189, 29190, 29193, 29194, 29196, 29201, 29202, 29204, 29208, 29217, 29218, 
    29220, 29224, 29232, 29249, 29250, 29252, 29256, 29264, 29280, 29313, 29314, 29316, 29320, 29328, 29344, 29376, 29441, 29442, 29444, 29448, 
    29456, 29472, 29504, 29568, 29699, 29701, 29702, 29705, 29706, 29708, 29713, 29714, 29716, 29720, 29729, 29730, 29732, 29736, 29744, 29761, 
    29762, 29764, 29768, 29776, 29792, 29825, 29826, 29828, 29832, 29840, 29856, 29888, 29953, 29954, 29956, 29960, 29968, 29984, 30016, 30080, 
    30209, 30210, 30212, 30216, 30224, 30240, 30272, 30336, 30464, 30723, 30725, 30726, 30729, 30730, 30732, 30737, 30738, 30740, 30744, 30753, 
    30754, 30756, 30760, 30768, 30785, 30786, 30788, 30792, 30800, 30816, 30849, 30850, 30852, 30856, 30864, 30880, 30912, 30977, 30978, 30980, 
    30984, 30992, 31008, 31040, 31104, 31233, 31234, 31236, 31240, 31248, 31264, 31296, 31360, 31488, 31745, 31746, 31748, 31752, 31760, 31776, 
    31808, 31872, 32000, 32256, 32799, 32815, 32823, 32827, 32829, 32830, 32847, 32855, 32859, 32861, 32862, 32871, 32875, 32877, 32878, 32883, 
    32885, 32886, 32889, 32890, 32892, 32911, 32919, 32923, 32925, 32926, 32935, 32939, 32941, 32942, 32947, 32949, 32950, 32953, 32954, 32956, 
    32967, 32971, 32973, 32974, 32979, 32981, 32982, 32985, 32986, 32988, 32995, 32997, 32998, 33001, 33002, 33004, 33009, 33010, 33012, 33016, 
    33039, 33047, 33051, 33053, 33054, 33063, 33067, 33069, 33070, 33075, 33077, 33078, 33081, 33082, 33084, 33095, 33099, 33101, 33102, 33107, 
    33109, 33110, 33113, 33114, 33116, 33123, 33125, 33126, 33129, 33130, 33132, 33137, 33138, 33140, 33144, 33159, 33163, 33165, 33166, 33171, 
    33173, 33174, 33177, 33178, 33180, 33187, 33189, 33190, 33193, 33194, 33196, 33201, 33202, 33204, 33208, 33219, 33221, 33222, 33225, 33226, 
    33228, 33233, 33234, 33236, 33240, 33249, 33250, 33252, 33256, 33264, 33295, 33303, 33307, 33309, 33310, 33319, 33323, 33325, 33326, 33331, 
    33333, 33334, 33337, 33338, 33340, 33351, 33355, 33357, 33358, 33363, 33365, 33366, 33369, 33370, 33372, 33379, 33381, 33382, 33385, 33386, 
    33388, 33393, 33394, 33396, 33400, 33415, 33419, 33421, 33422, 33427, 33429, 33430, 33433, 33434, 33436, 33443, 33445, 33446, 33449, 33450, 
    33452, 33457, 33458, 33460, 33464, 33475, 33477, 33478, 33481, 33482, 33484, 33489, 33490, 33492, 33496, 33505, 33506, 33508, 33512, 33520, 
    33543, 33547, 33549, 33550, 33555, 33557, 33558, 33561, 33562, 33564, 33571, 33573, 33574, 33577, 33578, 33580, 33585, 33586, 33588, 33592, 
    33603, 33605, 33606, 33609, 33610, 33612, 33617, 33618, 33620, 33624, 33633, 33634, 33636, 33640, 33648, 33667, 33669, 33670, 33673, 33674, 
    33676, 33681, 33682, 33684, 33688, 33697, 33698, 33700, 33704, 33712, 33729, 33730, 33732, 33736, 33744, 33760, 33807, 33815, 33819, 33821, 
    33822, 33831, 33835, 33837, 33838, 33843, 33845, 33846, 33849, 33850, 33852, 33863, 33867, 33869, 33870, 33875, 33877, 33878, 33881, 33882, 
    33884, 33891, 33893, 33894, 33897, 33898, 33900, 33905, 33906, 33908, 33912, 33927, 33931, 33933, 33934, 33939, 33941, 33942, 33945, 33946, 
    33948, 33955, 33957, 33958, 33961, 33962, 33964, 33969, 33970, 33972, 33976, 33987, 33989, 33990, 33993, 33994, 33996, 34001, 34002, 34004, 
    34008, 34017, 34018, 34020, 34024, 34032, 34055, 34059, 34061, 34062, 34067, 34069, 34070, 34073, 34074, 34076, 34083, 34085, 34086, 34089, 
    34090, 34092, 34097, 34098, 34100, 34104, 34115, 34117, 34118, 34121, 34122, 34124, 34129, 34130, 34132, 34136, 34145, 34146, 34148, 34152, 
    34160, 34179, 34181, 34182, 34185, 34186, 34188, 34193, 34194, 34196, 34200, 34209, 34210, 34212, 34216, 34224, 34241, 34242, 34244, 34248, 
    34256, 34272, 34311, 34315, 34317, 34318, 34323, 34325, 34326, 34329, 34330, 34332, 34339, 34341, 34342, 34345, 34346, 34348, 34353, 34354, 
    34356, 34360, 34371, 34373, 34374, 34377, 34378, 34380, 34385, 34386, 34388, 34392, 34401, 34402, 34404, 34408, 34416, 34435, 34437, 34438, 
    34441, 34442, 34444, 34449, 34450, 34452, 34456, 34465, 34466, 34468, 34472, 34480, 34497, 34498, 34500, 34504, 34512, 34528, 34563, 34565, 
    34566, 34569, 34570, 34572, 34577, 34578, 34580, 34584, 34593, 34594, 34596, 34600, 34608, 34625, 34626, 34628, 34632, 34640, 34656, 34689, 
    34690, 34692, 34696, 34704, 34720, 34752, 34831, 34839, 34843, 34845, 34846, 34855, 34859, 34861, 34862, 34867, 34869, 34870, 34873, 34874, 
    34876, 34887, 34891, 34893, 34894, 34899, 34901, 34902, 34905, 34906, 34908, 34915, 34917, 34918, 34921, 34922, 34924, 34929, 34930, 34932, 
    34936, 34951, 34955, 34957, 34958, 34963, 34965, 34966, 34969, 34970, 34972, 34979, 34981, 34982, 34985, 34986, 34988, 34993, 34994, 34996, 
    35000, 35011, 35013, 35014, 35017, 35018, 35020, 35025, 35026, 35028, 35032, 35041, 35042, 35044, 35048, 35056, 35079, 35083, 35085, 35086, 
    35091, 35093, 35094, 35097, 35098, 35100, 35107, 35109, 35110, 35113, 35114, 35116, 35121, 35122, 35124, 35128, 35139, 35141, 35142, 35145, 
    35146, 35148, 35153, 35154, 35156, 35160, 35169, 35170, 35172, 35176, 35184, 35203, 35205, 35206, 35209, 35210, 35212, 35217, 35218, 35220, 
    35224, 35233, 35234, 35236, 35240, 35248, 35265, 35266, 35268, 35272, 35280, 35296, 35335, 35339, 35341, 35342, 35347, 35349, 35350, 35353, 
    35354, 35356, 35363, 35365, 35366, 35369, 35370, 35372, 35377, 35378, 35380, 35384, 35395, 35397, 35398, 35401, 35402, 35404, 35409, 35410, 
    35412, 35416, 35425, 35426, 35428, 35432, 35440, 35459, 35461, 35462, 35465, 35466, 35468, 35473, 35474, 35476, 35480, 35489, 35490, 35492, 
    35496, 35504, 35521, 35522, 35524, 35528, 35536, 35552, 35587, 35589, 35590, 35593, 35594, 35596, 35601, 35602, 35604, 35608, 35617, 35618, 
    35620, 35624, 35632, 35649, 35650, 35652, 35656, 35664, 35680, 35713, 35714, 35716, 35720, 35728, 35744, 35776, 35847, 35851, 35853, 35854, 
    35859, 35861, 35862, 35865, 35866, 35868, 35875, 35877, 35878, 35881, 35882, 35884, 35889, 35890, 35892, 35896, 35907, 35909, 35910, 35913, 
    35914, 35916, 35921, 35922, 35924, 35928, 35937, 35938, 35940, 35944, 35952, 35971, 35973, 35974, 35977, 35978, 35980, 35985, 35986, 35988, 
    35992, 36001, 36002, 36004, 36008, 36016, 36033, 36034, 36036, 36040, 36048, 36064, 36099, 36101, 36102, 36105, 36106, 36108, 36113, 36114, 
    36116, 36120, 36129, 36130, 36132, 36136, 36144, 36161, 36162, 36164, 36168, 36176, 36192, 36225, 36226, 36228, 36232, 36240, 36256, 36288, 
    36355, 36357, 36358, 36361, 36362, 36364, 36369, 36370, 36372, 36376, 36385, 36386, 36388, 36392, 36400, 36417, 36418, 36420, 36424, 36432, 
    36448, 36481, 36482, 36484, 36488, 36496, 36512, 36544, 36609, 36610, 36612, 36616, 36624, 36640, 36672, 36736, 36879, 36887, 36891, 36893, 
    36894, 36903, 36907, 36909, 36910, 36915, 36917, 36918, 36921, 36922, 36924, 36935, 36939, 36941, 36942, 36947, 36949, 36950, 36953, 36954, 
    36956, 36963, 36965, 36966, 36969, 36970, 36972, 36977, 36978, 36980, 36984, 36999, 37003, 37005, 37006, 37011, 37013, 37014, 37017, 37018, 
    37020, 37027, 37029, 37030, 37033, 37034, 37036, 37041, 37042, 37044, 37048, 37059, 37061, 37062, 37065, 37066, 37068, 37073, 37074, 37076, 
    37080, 37089, 37090, 37092, 37096, 37104, 37127, 37131, 37133, 37134, 37139, 37141, 37142, 37145, 37146, 37148, 37155, 37157, 37158, 37161, 
    37162, 37164, 37169, 37170, 37172, 37176, 37187, 37189, 37190, 37193, 37194, 37196, 37201, 37202, 37204, 37208, 37217, 37218, 37220, 37224, 
    37232, 37251, 37253, 37254, 37257, 37258, 37260, 37265, 37266, 37268, 37272, 37281, 37282, 37284, 37288, 37296, 37313, 37314, 37316, 37320, 
    37328, 37344, 37383, 37387, 37389, 37390, 37395, 37397, 37398, 37401, 37402, 37404, 37411, 37413, 37414, 37417, 37418, 37420, 37425, 37426, 
    37428, 37432, 37443, 37445, 37446, 37449, 37450, 37452, 37457, 37458, 37460, 37464, 37473, 37474, 37476, 37480, 37488, 37507, 37509, 37510, 
    37513, 37514, 37516, 37521, 37522, 37524, 37528, 37537, 37538, 37540, 37544, 37552, 37569, 37570, 37572, 37576, 37584, 37600, 37635, 37637, 
    37638, 37641, 37642, 37644, 37649, 37650, 37652, 37656, 37665, 37666, 37668, 37672, 37680, 37697, 37698, 37700, 37704, 37712, 37728, 37761, 
    37762, 37764, 37768, 37776, 37792, 37824, 37895, 37899, 37901, 37902, 37907, 37909, 37910, 37913, 37914, 37916, 37923, 37925, 37926, 37929, 
    37930, 37932, 37937, 37938, 37940, 37944, 37955, 37957, 37958, 37961, 37962, 37964, 37969, 37970, 37972, 37976, 37985, 37986, 37988, 37992, 
    38000, 38019, 38021, 38022, 38025, 38026, 38028, 38033, 38034, 38036, 38040, 38049, 38050, 38052, 38056, 38064, 38081, 38082, 38084, 38088, 
    38096, 38112, 38147, 38149, 38150, 38153, 38154, 38156, 38161, 38162, 38164, 38168, 38177, 38178, 38180, 38184, 38192, 38209, 38210, 38212, 
    38216, 38224, 38240, 38273, 38274, 38276, 38280, 38288, 38304, 38336, 38403, 38405, 38406, 38409, 38410, 38412, 38417, 38418, 38420, 38424, 
    38433, 38434, 38436, 38440, 38448, 38465, 38466, 38468, 38472, 38480, 38496, 38529, 38530, 38532, 38536, 38544, 38560, 38592, 38657, 38658, 
    38660, 38664, 38672, 38688, 38720, 38784, 38919, 38923, 38925, 38926, 38931, 38933, 38934, 38937, 38938, 38940, 38947, 38949, 38950, 38953, 
    38954, 38956, 38961, 38962, 38964, 38968, 38979, 38981, 38982, 38985, 38986, 38988, 38993, 38994, 38996, 39000, 39009, 39010, 39012, 39016, 
    39024, 39043, 39045, 39046, 39049, 39050, 39052, 39057, 39058, 39060, 39064, 39073, 39074, 39076, 39080, 39088, 39105, 39106, 39108, 39112, 
    39120, 39136, 39171, 39173, 39174, 39177, 39178, 39180, 39185, 39186, 39188, 39192, 39201, 39202, 39204, 39208, 39216, 39233, 39234, 39236, 
    39240, 39248, 39264, 39297, 39298, 39300, 39304, 39312, 39328, 39360, 39427, 39429, 39430, 39433, 39434, 39436, 39441, 39442, 39444, 39448, 
    39457, 39458, 39460, 39464, 39472, 39489, 39490, 39492, 39496, 39504, 39520, 39553, 39554, 39556, 39560, 39568, 39584, 39616, 39681, 39682, 
    39684, 39688, 39696, 39712, 39744, 39808, 39939, 39941, 39942, 39945, 39946, 39948, 39953, 39954, 39956, 39960, 39969, 39970, 39972, 39976, 
    39984, 40001, 40002, 40004, 40008, 40016, 40032, 40065, 40066, 40068, 40072, 40080, 40096, 40128, 40193, 40194, 40196, 40200, 40208, 40224, 
    40256, 40320, 40449, 40450, 40452, 40456, 40464, 40480, 40512, 40576, 40704, 40975, 40983, 40987, 40989, 40990, 40999, 41003, 41005, 41006, 
    41011, 41013, 41014, 41017, 41018, 41020, 41031, 41035, 41037, 41038, 41043, 41045, 41046, 41049, 41050, 41052, 41059, 41061, 41062, 41065, 
    41066, 41068, 41073, 41074, 41076, 41080, 41095, 41099, 41101, 41102, 41107, 41109, 41110, 41113, 41114, 41116, 41123, 41125, 41126, 41129, 
    41130, 41132, 41137, 41138, 41140, 41144, 41155, 41157, 41158, 41161, 41162, 41164, 41169, 41170, 41172, 41176, 41185, 41186, 41188, 41192, 
    41200, 41223, 41227, 41229, 41230, 41235, 41237, 41238, 41241, 41242, 41244, 41251, 41253, 41254, 41257, 41258, 41260, 41265, 41266, 41268, 
    41272, 41283, 41285, 41286, 41289, 41290, 41292, 41297, 41298, 41300, 41304, 41313, 41314, 41316, 41320, 41328, 41347, 41349, 41350, 41353, 
    41354, 41356, 41361, 41362, 41364, 41368, 41377, 41378, 41380, 41384, 41392, 41409, 41410, 41412, 41416, 41424, 41440, 41479, 41483, 41485, 
    41486, 41491, 41493, 41494, 41497, 41498, 41500, 41507, 41509, 41510, 41513, 41514, 41516, 41521, 41522, 41524, 41528, 41539, 41541, 41542, 
    41545, 41546, 41548, 41553, 41554, 41556, 41560, 41569, 41570, 41572, 41576, 41584, 41603, 41605, 41606, 41609, 41610, 41612, 41617, 41618, 
    41620, 41624, 41633, 41634, 41636, 41640, 41648, 41665, 41666, 41668, 41672, 41680, 41696, 41731, 41733, 41734, 41737, 41738, 41740, 41745, 
    41746, 41748, 41752, 41761, 41762, 41764, 41768, 41776, 41793, 41794, 41796, 41800, 41808, 41824, 41857, 41858, 41860, 41864, 41872, 41888, 
    41920, 41991, 41995, 41997, 41998, 42003, 42005, 42006, 42009, 42010, 42012, 42019, 42021, 42022, 42025, 42026, 42028, 42033, 42034, 42036, 
    42040, 42051, 42053, 42054, 42057, 42058, 42060, 42065, 42066, 42068, 42072, 42081, 42082, 42084, 42088, 42096, 42115, 42117, 42118, 42121, 
    42122, 42124, 42129, 42130, 42132, 42136, 42145, 42146, 42148, 42152, 42160, 42177, 42178, 42180, 42184, 42192, 42208, 42243, 42245, 42246, 
    42249, 42250, 42252, 42257, 42258, 42260, 42264, 42273, 42274, 42276, 42280, 42288, 42305, 42306, 42308, 42312, 42320, 42336, 42369, 42370, 
    42372, 42376, 42384, 42400, 42432, 42499, 42501, 42502, 42505, 42506, 42508, 42513, 42514, 42516, 42520, 42529, 42530, 42532, 42536, 42544, 
    42561, 42562, 42564, 42568, 42576, 42592, 42625, 42626, 42628, 42632, 42640, 42656, 42688, 42753, 42754, 42756, 42760, 42768, 42784, 42816, 
    42880, 43015, 43019, 43021, 43022, 43027, 43029, 43030, 43033, 43034, 43036, 43043, 43045, 43046, 43049, 43050, 43052, 43057, 43058, 43060, 
    43064, 43075, 43077, 43078, 43081, 43082, 43084, 43089, 43090, 43092, 43096, 43105, 43106, 43108, 43112, 43120, 43139, 43141, 43142, 43145, 
    43146, 43148, 43153, 43154, 43156, 43160, 43169, 43170, 43172, 43176, 43184, 43201, 43202, 43204, 43208, 43216, 43232, 43267, 43269, 43270, 
    43273, 43274, 43276, 43281, 43282, 43284, 43288, 43297, 43298, 43300, 43304, 43312, 43329, 43330, 43332, 43336, 43344, 43360, 43393, 43394, 
    43396, 43400, 43408, 43424, 43456, 43523, 43525, 43526, 43529, 43530, 43532, 43537, 43538, 43540, 43544, 43553, 43554, 43556, 43560, 43568, 
    43585, 43586, 43588, 43592, 43600, 43616, 43649, 43650, 43652, 43656, 43664, 43680, 43712, 43777, 43778, 43780, 43784, 43792, 43808, 43840, 
    43904, 44035, 44037, 44038, 44041, 44042, 44044, 44049, 44050, 44052, 44056, 44065, 44066, 44068, 44072, 44080, 44097, 44098, 44100, 44104, 
    44112, 44128, 44161, 44162, 44164, 44168, 44176, 44192, 44224, 44289, 44290, 44292, 44296, 44304, 44320, 44352, 44416, 44545, 44546, 44548, 
    44552, 44560, 44576, 44608, 44672, 44800, 45063, 45067, 45069, 45070, 45075, 45077, 45078, 45081, 45082, 45084, 45091, 45093, 45094, 45097, 
    45098, 45100, 45105, 45106, 45108, 45112, 45123, 45125, 45126, 45129, 45130, 45132, 45137, 45138, 45140, 45144, 45153, 45154, 45156, 45160, 
    45168, 45187, 45189, 45190, 45193, 45194, 45196, 45201, 45202, 45204, 45208, 45217, 45218, 45220, 45224, 45232, 45249, 45250, 45252, 45256, 
    45264, 45280, 45315, 45317, 45318, 45321, 45322, 45324, 45329, 45330, 45332, 45336, 45345, 45346, 45348, 45352, 45360, 45377, 45378, 45380, 
    45384, 45392, 45408, 45441, 45442, 45444, 45448, 45456, 45472, 45504, 45571, 45573, 45574, 45577, 45578, 45580, 45585, 45586, 45588, 45592, 
    45601, 45602, 45604, 45608, 45616, 45633, 45634, 45636, 45640, 45648, 45664, 45697, 45698, 45700, 45704, 45712, 45728, 45760, 45825, 45826, 
    45828, 45832, 45840, 45856, 45888, 45952, 46083, 46085, 46086, 46089, 46090, 46092, 46097, 46098, 46100, 46104, 46113, 46114, 46116, 46120, 
    46128, 46145, 46146, 46148, 46152, 46160, 46176, 46209, 46210, 46212, 46216, 46224, 46240, 46272, 46337, 46338, 46340, 46344, 46352, 46368, 
    46400, 46464, 46593, 46594, 46596, 46600, 46608, 46624, 46656, 46720, 46848, 47107, 47109, 47110, 47113, 47114, 47116, 47121, 47122, 47124, 
    47128, 47137, 47138, 47140, 47144, 47152, 47169, 47170, 47172, 47176, 47184, 47200, 47233, 47234, 47236, 47240, 47248, 47264, 47296, 47361, 
    47362, 47364, 47368, 47376, 47392, 47424, 47488, 47617, 47618, 47620, 47624, 47632, 47648, 47680, 47744, 47872, 48129, 48130, 48132, 48136, 
    48144, 48160, 48192, 48256, 48384, 48640, 49167, 49175, 49179, 49181, 49182, 49191, 49195, 49197, 49198, 49203, 49205, 49206, 49209, 49210, 
    49212, 49223, 49227, 49229, 49230, 49235, 49237, 49238, 49241, 49242, 49244, 49251, 49253, 49254, 49257, 49258, 49260, 49265, 49266, 49268, 
    49272, 49287, 49291, 49293, 49294, 49299, 49301, 49302, 49305, 49306, 49308, 49315, 49317, 49318, 49321, 49322, 49324, 49329, 49330, 49332, 
    49336, 49347, 49349, 49350, 49353, 49354, 49356, 49361, 49362, 49364, 49368, 49377, 49378, 49380, 49384, 49392, 49415, 49419, 49421, 49422, 
    49427, 49429, 49430, 49433, 49434, 49436, 49443, 49445, 49446, 49449, 49450, 49452, 49457, 49458, 49460, 49464, 49475, 49477, 49478, 49481, 
    49482, 49484, 49489, 49490, 49492, 49496, 49505, 49506, 49508, 49512, 49520, 49539, 49541, 49542, 49545, 49546, 49548, 49553, 49554, 49556, 
    49560, 49569, 49570, 49572, 49576, 49584, 49601, 49602, 49604, 49608, 49616, 49632, 49671, 49675, 49677, 49678, 49683, 49685, 49686, 49689, 
    49690, 49692, 49699, 49701, 49702, 49705, 49706, 49708, 49713, 49714, 49716, 49720, 49731, 49733, 49734, 49737, 49738, 49740, 49745, 49746, 
    49748, 49752, 49761, 49762, 49764, 49768, 49776, 49795, 49797, 49798, 49801, 49802, 49804, 49809, 49810, 49812, 49816, 49825, 49826, 49828, 
    49832, 49840, 49857, 49858, 49860, 49864, 49872, 49888, 49923, 49925, 49926, 49929, 49930, 49932, 49937, 49938, 49940, 49944, 49953, 49954, 
    49956, 49960, 49968, 49985, 49986, 49988, 49992, 50000, 50016, 50049, 50050, 50052, 50056, 50064, 50080, 50112, 50183, 50187, 50189, 50190, 
    50195, 50197, 50198, 50201, 50202, 50204, 50211, 50213, 50214, 50217, 50218, 50220, 50225, 50226, 50228, 50232, 50243, 50245, 50246, 50249, 
    50250, 50252, 50257, 50258, 50260, 50264, 50273, 50274, 50276, 50280, 50288, 50307, 50309, 50310, 50313, 50314, 50316, 50321, 50322, 50324, 
    50328, 50337, 50338, 50340, 50344, 50352, 50369, 50370, 50372, 50376, 50384, 50400, 50435, 50437, 50438, 50441, 50442, 50444, 50449, 50450, 
    50452, 50456, 50465, 50466, 50468, 50472, 50480, 50497, 50498, 50500, 50504, 50512, 50528, 50561, 50562, 50564, 50568, 50576, 50592, 50624, 
    50691, 50693, 50694, 50697, 50698, 50700, 50705, 50706, 50708, 50712, 50721, 50722, 50724, 50728, 50736, 50753, 50754, 50756, 50760, 50768, 
    50784, 50817, 50818, 50820, 50824, 50832, 50848, 50880, 50945, 50946, 50948, 50952, 50960, 50976, 51008, 51072, 51207, 51211, 51213, 51214, 
    51219, 51221, 51222, 51225, 51226, 51228, 51235, 51237, 51238, 51241, 51242, 51244, 51249, 51250, 51252, 51256, 51267, 51269, 51270, 51273, 
    51274, 51276, 51281, 51282, 51284, 51288, 51297, 51298, 51300, 51304, 51312, 51331, 51333, 51334, 51337, 51338, 51340, 51345, 51346, 51348, 
    51352, 51361, 51362, 51364, 51368, 51376, 51393, 51394, 51396, 51400, 51408, 51424, 51459, 51461, 51462, 51465, 51466, 51468, 51473, 51474, 
    51476, 51480, 51489, 51490, 51492, 51496, 51504, 51521, 51522, 51524, 51528, 51536, 51552, 51585, 51586, 51588, 51592, 51600, 51616, 51648, 
    51715, 51717, 51718, 51721, 51722, 51724, 51729, 51730, 51732, 51736, 51745, 51746, 51748, 51752, 51760, 51777, 51778, 51780, 51784, 51792, 
    51808, 51841, 51842, 51844, 51848, 51856, 51872, 51904, 51969, 51970, 51972, 51976, 51984, 52000, 52032, 52096, 52227, 52229, 52230, 52233, 
    52234, 52236, 52241, 52242, 52244, 52248, 52257, 52258, 52260, 52264, 52272, 52289, 52290, 52292, 52296, 52304, 52320, 52353, 52354, 52356, 
    52360, 52368, 52384, 52416, 52481, 52482, 52484, 52488, 52496, 52512, 52544, 52608, 52737, 52738, 52740, 52744, 52752, 52768, 52800, 52864, 
    52992, 53255, 53259, 53261, 53262, 53267, 53269, 53270, 53273, 53274, 53276, 53283, 53285, 53286, 53289, 53290, 53292, 53297, 53298, 53300, 
    53304, 53315, 53317, 53318, 53321, 53322, 53324, 53329, 53330, 53332, 53336, 53345, 53346, 53348, 53352, 53360, 53379, 53381, 53382, 53385, 
    53386, 53388, 53393, 53394, 53396, 53400, 53409, 53410, 53412, 53416, 53424, 53441, 53442, 53444, 53448, 53456, 53472, 53507, 53509, 53510, 
    53513, 53514, 53516, 53521, 53522, 53524, 53528, 53537, 53538, 53540, 53544, 53552, 53569, 53570, 53572, 53576, 53584, 53600, 53633, 53634, 
    53636, 53640, 53648, 53664, 53696, 53763, 53765, 53766, 53769, 53770, 53772, 53777, 53778, 53780, 53784, 53793, 53794, 53796, 53800, 53808, 
    53825, 53826, 53828, 53832, 53840, 53856, 53889, 53890, 53892, 53896, 53904, 53920, 53952, 54017, 54018, 54020, 54024, 54032, 54048, 54080, 
    54144, 54275, 54277, 54278, 54281, 54282, 54284, 54289, 54290, 54292, 54296, 54305, 54306, 54308, 54312, 54320, 54337, 54338, 54340, 54344, 
    54352, 54368, 54401, 54402, 54404, 54408, 54416, 54432, 54464, 54529, 54530, 54532, 54536, 54544, 54560, 54592, 54656, 54785, 54786, 54788, 
    54792, 54800, 54816, 54848, 54912, 55040, 55299, 55301, 55302, 55305, 55306, 55308, 55313, 55314, 55316, 55320, 55329, 55330, 55332, 55336, 
    55344, 55361, 55362, 55364, 55368, 55376, 55392, 55425, 55426, 55428, 55432, 55440, 55456, 55488, 55553, 55554, 55556, 55560, 55568, 55584, 
    55616, 55680, 55809, 55810, 55812, 55816, 55824, 55840, 55872, 55936, 56064, 56321, 56322, 56324, 56328, 56336, 56352, 56384, 56448, 56576, 
    56832, 57351, 57355, 57357, 57358, 57363, 57365, 57366, 57369, 57370, 57372, 57379, 57381, 57382, 57385, 57386, 57388, 57393, 57394, 57396, 
    57400, 57411, 57413, 57414, 57417, 57418, 57420, 57425, 57426, 57428, 57432, 57441, 57442, 57444, 57448, 57456, 57475, 57477, 57478, 57481, 
    57482, 57484, 57489, 57490, 57492, 57496, 57505, 57506, 57508, 57512, 57520, 57537, 57538, 57540, 57544, 57552, 57568, 57603, 57605, 57606, 
    57609, 57610, 57612, 57617, 57618, 57620, 57624, 57633, 57634, 57636, 57640, 57648, 57665, 57666, 57668, 57672, 57680, 57696, 57729, 57730, 
    57732, 57736, 57744, 57760, 57792, 57859, 57861, 57862, 57865, 57866, 57868, 57873, 57874, 57876, 57880, 57889, 57890, 57892, 57896, 57904, 
    57921, 57922, 57924, 57928, 57936, 57952, 57985, 57986, 57988, 57992, 58000, 58016, 58048, 58113, 58114, 58116, 58120, 58128, 58144, 58176, 
    58240, 58371, 58373, 58374, 58377, 58378, 58380, 58385, 58386, 58388, 58392, 58401, 58402, 58404, 58408, 58416, 58433, 58434, 58436, 58440, 
    58448, 58464, 58497, 58498, 58500, 58504, 58512, 58528, 58560, 58625, 58626, 58628, 58632, 58640, 58656, 58688, 58752, 58881, 58882, 58884, 
    58888, 58896, 58912, 58944, 59008, 59136, 59395, 59397, 59398, 59401, 59402, 59404, 59409, 59410, 59412, 59416, 59425, 59426, 59428, 59432, 
    59440, 59457, 59458, 59460, 59464, 59472, 59488, 59521, 59522, 59524, 59528, 59536, 59552, 59584, 59649, 59650, 59652, 59656, 59664, 59680, 
    59712, 59776, 59905, 59906, 59908, 59912, 59920, 59936, 59968, 60032, 60160, 60417, 60418, 60420, 60424, 60432, 60448, 60480, 60544, 60672, 
    60928, 61443, 61445, 61446, 61449, 61450, 61452, 61457, 61458, 61460, 61464, 61473, 61474, 61476, 61480, 61488, 61505, 61506, 61508, 61512, 
    61520, 61536, 61569, 61570, 61572, 61576, 61584, 61600, 61632, 61697, 61698, 61700, 61704, 61712, 61728, 61760, 61824, 61953, 61954, 61956, 
    61960, 61968, 61984, 62016, 62080, 62208, 62465, 62466, 62468, 62472, 62480, 62496, 62528, 62592, 62720, 62976, 63489, 63490, 63492, 63496, 
    63504, 63520, 63552, 63616, 63744, 64000, 64512, },
  { 127, 191, 223, 239, 247, 251, 253, 254, 319, 351, 367, 375, 379, 381, 382, 415, 431, 439, 443, 445, 446, 
    463, 471, 475, 477, 478, 487, 491, 493, 494, 499, 501, 502, 505, 506, 508, 575, 607, 623, 631, 635, 
    637, 638, 671, 687, 695, 699, 701, 702, 719, 727, 731, 733, 734, 743, 747, 749, 750, 755, 757, 758, 
    761, 762, 764, 799, 815, 823, 827, 829, 830, 847, 855, 859, 861, 862, 871, 875, 877, 878, 883, 885, 
    886, 889, 890, 892, 911, 919, 923, 925, 926, 935, 939, 941, 942, 947, 949, 950, 953, 954, 956, 967, 
    971, 973, 974, 979, 981, 982, 985, 986, 988, 995, 997, 998, 1001, 1002, 1004, 1009, 1010, 1012, 1016, 1087, 
    1119, 1135, 1143, 1147, 1149, 1150, 1183, 1199, 1207, 1211, 1213, 1214, 1231, 1239, 1243, 1245, 1246, 1255, 1259, 1261, 
    1262, 1267, 1269, 1270, 1273, 1274, 1276, 1311, 1327, 1335, 1339, 1341, 1342, 1359, 1367, 1371, 1373, 1374, 1383, 1387, 
    1389, 1390, 1395, 1397, 1398, 1401, 1402, 1404, 1423, 1431, 1435, 1437, 1438, 1447, 1451, 1453, 1454, 1459, 1461, 1462, 
    1465, 1466, 1468, 1479, 1483, 1485, 1486, 1491, 1493, 1494, 1497, 1498, 1500, 1507, 1509, 1510, 1513, 1514, 1516, 1521, 
    1522, 1524, 1528, 1567, 1583, 1591, 1595, 1597, 1598, 1615, 1623, 1627, 1629, 1630, 1639, 1643, 1645, 1646, 1651, 1653, 
    1654, 1657, 1658, 1660, 1679, 1687, 1691, 1693, 1694, 1703, 1707, 1709, 1710, 1715, 1717, 1718, 1721, 1722, 1724, 1735, 
    1739, 1741, 1742, 1747, 1749, 1750, 1753, 1754, 1756, 1763, 1765, 1766, 1769, 1770, 1772, 1777, 1778, 1780, 1784, 1807, 
    1815, 1819, 1821, 1822, 1831, 1835, 1837, 1838, 1843, 1845, 1846, 1849, 1850, 1852, 1863, 1867, 1869, 1870, 1875, 1877, 
    1878, 1881, 1882, 1884, 1891, 1893, 1894, 1897, 1898, 1900, 1905, 1906, 1908, 1912, 1927, 1931, 1933, 1934, 1939, 1941, 
    1942, 1945, 1946, 1948, 1955, 1957, 1958, 1961, 1962, 1964, 1969, 1970, 1972, 1976, 1987, 1989, 1990, 1993, 1994, 1996, 
    2001, 2002, 2004, 2008, 2017, 2018, 2020, 2024, 2032, 2111, 2143, 2159, 2167, 2171, 2173, 2174, 2207, 2223, 2231, 2235, 
    2237, 2238, 2255, 2263, 2267, 2269, 2270, 2279, 2283, 2285, 2286, 2291, 2293, 2294, 2297, 2298, 2300, 2335, 2351, 2359, 
    2363, 2365, 2366, 2383, 2391, 2395, 2397, 2398, 2407, 2411, 2413, 2414, 2419, 2421, 2422, 2425, 2426, 2428, 2447, 2455, 
    2459, 2461, 2462, 2471, 2475, 2477, 2478, 2483, 2485, 2486, 2489, 2490, 2492, 2503, 2507, 2509, 2510, 2515, 2517, 2518, 
    2521, 2522, 2524, 2531, 2533, 2534, 2537, 2538, 2540, 2545, 2546, 2548, 2552, 2591, 2607, 2615, 2619, 2621, 2622, 2639, 
    2647, 2651, 2653, 2654, 2663, 2667, 2669, 2670, 2675, 2677, 2678, 2681, 2682, 2684, 2703, 2711, 2715, 2717, 2718, 2727, 
    2731, 2733, 2734, 2739, 2741, 2742, 2745, 2746, 2748, 2759, 2763, 2765, 2766, 2771, 2773, 2774, 2777, 2778, 2780, 2787, 
    2789, 2790, 2793, 2794, 2796, 2801, 2802, 2804, 2808, 2831, 2839, 2843, 2845, 2846, 2855, 2859, 2861, 2862, 2867, 2869, 
    2870, 2873, 2874, 2876, 2887, 2891, 2893, 2894, 2899, 2901, 2902, 2905, 2906, 2908, 2915, 2917, 2918, 2921, 2922, 2924, 
    2929, 2930, 2932, 2936, 2951, 2955, 2957, 2958, 2963, 2965, 2966, 2969, 2970, 2972, 2979, 2981, 2982, 2985, 2986, 2988, 
    2993, 2994, 2996, 3000, 3011, 3013, 3014, 3017, 3018, 3020, 3025, 3026, 3028, 3032, 3041, 3042, 3044, 3048, 3056, 3103, 
    3119, 3127, 3131, 3133, 3134, 3151, 3159, 3163, 3165, 3166, 3175, 3179, 3181, 3182, 3187, 3189, 3190, 3193, 3194, 3196, 
    3215, 3223, 3227, 3229, 3230, 3239, 3243, 3245, 3246, 3251, 3253, 3254, 3257, 3258, 3260, 3271, 3275, 3277, 3278, 3283, 
    3285, 3286, 3289, 3290, 3292, 3299, 3301, 3302, 3305, 3306, 3308, 3313, 3314, 3316, 3320, 3343, 3351, 3355, 3357, 3358, 
    3367, 3371, 3373, 3374, 3379, 3381, 3382, 3385, 3386, 3388, 3399, 3403, 3405, 3406, 3411, 3413, 3414, 3417, 3418, 3420, 
    3427, 3429, 3430, 3433, 3434, 3436, 3441, 3442, 3444, 3448, 3463, 3467, 3469, 3470, 3475, 3477, 3478, 3481, 3482, 3484, 
    3491, 3493, 3494, 3497, 3498, 3500, 3505, 3506, 3508, 3512, 3523, 3525, 3526, 3529, 3530, 3532, 3537, 3538, 3540, 3544, 
    3553, 3554, 3556, 3560, 3568, 3599, 3607, 3611, 3613, 3614, 3623, 3627, 3629, 3630, 3635, 3637, 3638, 3641, 3642, 3644, 
    3655, 3659, 3661, 3662, 3667, 3669, 3670, 3673, 3674, 3676, 3683, 3685, 3686, 3689, 3690, 3692, 3697, 3698, 3700, 3704, 
    3719, 3723, 3725, 3726, 3731, 3733, 3734, 3737, 3738, 3740, 3747, 3749, 3750, 3753, 3754, 3756, 3761, 3762, 3764, 3768, 
    3779, 3781, 3782, 3785, 3786, 3788, 3793, 3794, 3796, 3800, 3809, 3810, 3812, 3816, 3824, 3847, 3851, 3853, 3854, 3859, 
    3861, 3862, 3865, 3866, 3868, 3875, 3877, 3878, 3881, 3882, 3884, 3889, 3890, 3892, 3896, 3907, 3909, 3910, 3913, 3914, 
    3916, 3921, 3922, 3924, 3928, 3937, 3938, 3940, 3944, 3952, 3971, 3973, 3974, 3977, 3978, 3980, 3985, 3986, 3988, 3992, 
    4001, 4002, 4004, 4008, 4016, 4033, 4034, 4036, 4040, 4048, 4064, 4159, 4191, 4207, 4215, 4219, 4221, 4222, 4255, 4271, 
    4279, 4283, 4285, 4286, 4303, 4311, 4315, 4317, 4318, 4327, 4331, 4333, 4334, 4339, 4341, 4342, 4345, 4346, 4348, 4383, 
    4399, 4407, 4411, 4413, 4414, 4431, 4439, 4443, 4445, 4446, 4455, 4459, 4461, 4462, 4467, 4469, 4470, 4473, 4474, 4476, 
    4495, 4503, 4507, 4509, 4510, 4519, 4523, 4525, 4526, 4531, 4533, 4534, 4537, 4538, 4540, 4551, 4555, 4557, 4558, 4563, 
    4565, 4566, 4569, 4570, 4572, 4579, 4581, 4582, 4585, 4586, 4588, 4593, 4594, 4596, 4600, 4639, 4655, 4663, 4667, 4669, 
    4670, 4687, 4695, 4699, 4701, 4702, 4711, 4715, 4717, 4718, 4723, 4725, 4726, 4729, 4730, 4732, 4751, 4759, 4763, 4765, 
    4766, 4775, 4779, 4781, 4782, 4787, 4789, 4790, 4793, 4794, 4796, 4807, 4811, 4813, 4814, 4819, 4821, 4822, 4825, 4826, 
    4828, 4835, 4837, 4838, 4841, 4842, 4844, 4849, 4850, 4852, 4856, 4879, 4887, 4891, 4893, 4894, 4903, 4907, 4909, 4910, 
    4915, 4917, 4918, 4921, 4922, 4924, 4935, 4939, 4941, 4942, 4947, 4949, 4950, 4953, 4954, 4956, 4963, 4965, 4966, 4969, 
    4970, 4972, 4977, 4978, 4980, 4984, 4999, 5003, 5005, 5006, 5011, 5013, 5014, 5017, 5018, 5020, 5027, 5029, 5030, 5033, 
    5034, 5036, 5041, 5042, 5044, 5048, 5059, 5061, 5062, 5065, 5066, 5068, 5073, 5074, 5076, 5080, 5089, 5090, 5092, 5096, 
    5104, 5151, 5167, 5175, 5179, 5181, 5182, 5199, 5207, 5211, 5213, 5214, 5223, 5227, 5229, 5230, 5235, 5237, 5238, 5241, 
    5242, 5244, 5263, 5271, 5275, 5277, 5278, 5287, 5291, 5293, 5294, 5299, 5301, 5302, 5305, 5306, 5308, 5319, 5323, 5325, 
    5326, 5331, 5333, 5334, 5337, 5338, 5340, 5347, 5349, 5350, 5353, 5354, 5356, 5361, 5362, 5364, 5368, 5391, 5399, 5403, 
    5405, 5406, 5415, 5419, 5421, 5422, 5427, 5429, 5430, 5433, 5434, 5436, 5447, 5451, 5453, 5454, 5459, 5461, 5462, 5465, 
    5466, 5468, 5475, 5477, 5478, 5481, 5482, 5484, 5489, 5490, 5492, 5496, 5511, 5515, 5517, 5518, 5523, 5525, 5526, 5529, 
    5530, 5532, 5539, 5541, 5542, 5545, 5546, 5548, 5553, 5554, 5556, 5560, 5571, 5573, 5574, 5577, 5578, 5580, 5585, 5586, 
    5588, 5592, 5601, 5602, 5604, 5608, 5616, 5647, 5655, 5659, 5661, 5662, 5671, 5675, 5677, 5678, 5683, 5685, 5686, 5689, 
    5690, 5692, 5703, 5707, 5709, 5710, 5715, 5717, 5718, 5721, 5722, 5724, 5731, 5733, 5734, 5737, 5738, 5740, 5745, 5746, 
    5748, 5752, 5767, 5771, 5773, 5774, 5779, 5781, 5782, 5785, 5786, 5788, 5795, 5797, 5798, 5801, 5802, 5804, 5809, 5810, 
    5812, 5816, 5827, 5829, 5830, 5833, 5834, 5836, 5841, 5842, 5844, 5848, 5857, 5858, 5860, 5864, 5872, 5895, 5899, 5901, 
    5902, 5907, 5909, 5910, 5913, 5914, 5916, 5923, 5925, 5926, 5929, 5930, 5932, 5937, 5938, 5940, 5944, 5955, 5957, 5958, 
    5961, 5962, 5964, 5969, 5970, 5972, 5976, 5985, 5986, 5988, 5992, 6000, 6019, 6021, 6022, 6025, 6026, 6028, 6033, 6034, 
    6036, 6040, 6049, 6050, 6052, 6056, 6064, 6081, 6082, 6084, 6088, 6096, 6112, 6175, 6191, 6199, 6203, 6205, 6206, 6223, 
    6231, 6235, 6237, 6238, 6247, 6251, 6253, 6254, 6259, 6261, 6262, 6265, 6266, 6268, 6287, 6295, 6299, 6301, 6302, 6311, 
    6315, 6317, 6318, 6323, 6325, 6326, 6329, 6330, 6332, 6343, 6347, 6349, 6350, 6355, 6357, 6358, 6361, 6362, 6364, 6371, 
    6373, 6374, 6377, 6378, 6380, 6385, 6386, 6388, 6392, 6415, 6423, 6427, 6429, 6430, 6439, 6443, 6445, 6446, 6451, 6453, 
    6454, 6457, 6458, 6460, 6471, 6475, 6477, 6478, 6483, 6485, 6486, 6489, 6490, 6492, 6499, 6501, 6502, 6505, 6506, 6508, 
    6513, 6514, 6516, 6520, 6535, 6539, 6541, 6542, 6547, 6549, 6550, 6553, 6554, 6556, 6563, 6565, 6566, 6569, 6570, 6572, 
    6577, 6578, 6580, 6584, 6595, 6597, 6598, 6601, 6602, 6604, 6609, 6610, 6612, 6616, 6625, 6626, 6628, 6632, 6640, 6671, 
    6679, 6683, 6685, 6686, 6695, 6699, 6701, 6702, 6707, 6709, 6710, 6713, 6714, 6716, 6727, 6731, 6733, 6734, 6739, 6741, 
    6742, 6745, 6746, 6748, 6755, 6757, 6758, 6761, 6762, 6764, 6769, 6770, 6772, 6776, 6791, 6795, 6797, 6798, 6803, 6805, 
    6806, 6809, 6810, 6812, 6819, 6821, 6822, 6825, 6826, 6828, 6833, 6834, 6836, 6840, 6851, 6853, 6854, 6857, 6858, 6860, 
    6865, 6866, 6868, 6872, 6881, 6882, 6884, 6888, 6896, 6919, 6923, 6925, 6926, 6931, 6933, 6934, 6937, 6938, 6940, 6947, 
    6949, 6950, 6953, 6954, 6956, 6961, 6962, 6964, 6968, 6979, 6981, 6982, 6985, 6986, 6988, 6993, 6994, 6996, 7000, 7009, 
    7010, 7012, 7016, 7024, 7043, 7045, 7046, 7049, 7050, 7052, 7057, 7058, 7060, 7064, 7073, 7074, 7076, 7080, 7088, 7105, 
    7106, 7108, 7112, 7120, 7136, 7183, 7191, 7195, 7197, 7198, 7207, 7211, 7213, 7214, 7219, 7221, 7222, 7225, 7226, 7228, 
    7239, 7243, 7245, 7246, 7251, 7253, 7254, 7257, 7258, 7260, 7267, 7269, 7270, 7273, 7274, 7276, 7281, 7282, 7284, 7288, 
    7303, 7307, 7309, 7310, 7315, 7317, 7318, 7321, 7322, 7324, 7331, 7333, 7334, 7337, 7338, 7340, 7345, 7346, 7348, 7352, 
    7363, 7365, 7366, 7369, 7370, 7372, 7377, 7378, 7380, 7384, 7393, 7394, 7396, 7400, 7408, 7431, 7435, 7437, 7438, 7443, 
    7445, 7446, 7449, 7450, 7452, 7459, 7461, 7462, 7465, 7466, 7468, 7473, 7474, 7476, 7480, 7491, 7493, 7494, 7497, 7498, 
    7500, 7505, 7506, 7508, 7512, 7521, 7522, 7524, 7528, 7536, 7555, 7557, 7558, 7561, 7562, 7564, 7569, 7570, 7572, 7576, 
    7585, 7586, 7588, 7592, 7600, 7617, 7618, 7620, 7624, 7632, 7648, 7687, 7691, 7693, 7694, 7699, 7701, 7702, 7705, 7706, 
    7708, 7715, 7717, 7718, 7721, 7722, 7724, 7729, 7730, 7732, 7736, 7747, 7749, 7750, 7753, 7754, 7756, 7761, 7762, 7764, 
    7768, 7777, 7778, 7780, 7784, 7792, 7811, 7813, 7814, 7817, 7818, 7820, 7825, 7826, 7828, 7832, 7841, 7842, 7844, 7848, 
    7856, 7873, 7874, 7876, 7880, 7888, 7904, 7939, 7941, 7942, 7945, 7946, 7948, 7953, 7954, 7956, 7960, 7969, 7970, 7972, 
    7976, 7984, 8001, 8002, 8004, 8008, 8016, 8032, 8065, 8066, 8068, 8072, 8080, 8096, 8128, 8255, 8287, 8303, 8311, 8315, 
    8317, 8318, 8351, 8367, 8375, 8379, 8381, 8382, 8399, 8407, 8411, 8413, 8414, 8423, 8427, 8429, 8430, 8435, 8437, 8438, 
    8441, 8442, 8444, 8479, 8495, 8503, 8507, 8509, 8510, 8527, 8535, 8539, 8541, 8542, 8551, 8555, 8557, 8558, 8563, 8565, 
    8566, 8569, 8570, 8572, 8591, 8599, 8603, 8605, 8606, 8615, 8619, 8621, 8622, 8627, 8629, 8630, 8633, 8634, 8636, 8647, 
    8651, 8653, 8654, 8659, 8661, 8662, 8665, 8666, 8668, 8675, 8677, 8678, 8681, 8682, 8684, 8689, 8690, 8692, 8696, 8735, 
    8751, 8759, 8763, 8765, 8766, 8783, 8791, 8795, 8797, 8798, 8807, 8811, 8813, 8814, 8819, 8821, 8822, 8825, 8826, 8828, 
    8847, 8855, 8859, 8861, 8862, 8871, 8875, 8877, 8878, 8883, 8885, 8886, 8889, 8890, 8892, 8903, 8907, 8909, 8910, 8915, 
    8917, 8918, 8921, 8922, 8924, 8931, 8933, 8934, 8937, 8938, 8940, 8945, 8946, 8948, 8952, 8975, 8983, 8987, 8989, 8990, 
    8999, 9003, 9005, 9006, 9011, 9013, 9014, 9017, 9018, 9020, 9031, 9035, 9037, 9038, 9043, 9045, 9046, 9049, 9050, 9052, 
    9059, 9061, 9062, 9065, 9066, 9068, 9073, 9074, 9076, 9080, 9095, 9099, 9101, 9102, 9107, 9109, 9110, 9113, 9114, 9116, 
    9123, 9125, 9126, 9129, 9130, 9132, 9137, 9138, 9140, 9144, 9155, 9157, 9158, 9161, 9162, 9164, 9169, 9170, 9172, 9176, 
    9185, 9186, 9188, 9192, 9200, 9247, 9263, 9271, 9275, 9277, 9278, 9295, 9303, 9307, 9309, 9310, 9319, 9323, 9325, 9326, 
    9331, 9333, 9334, 9337, 9338, 9340, 9359, 9367, 9371, 9373, 9374, 9383, 9387, 9389, 9390, 9395, 9397, 9398, 9401, 9402, 
    9404, 9415, 9419, 9421, 9422, 9427, 9429, 9430, 9433, 9434, 9436, 9443, 9445, 9446, 9449, 9450, 9452, 9457, 9458, 9460, 
    9464, 9487, 9495, 9499, 9501, 9502, 9511, 9515, 9517, 9518, 9523, 9525, 9526, 9529, 9530, 9532, 9543, 9547, 9549, 9550, 
    9555, 9557, 9558, 9561, 9562, 9564, 9571, 9573, 9574, 9577, 9578, 9580, 9585, 9586, 9588, 9592, 9607, 9611, 9613, 9614, 
    9619, 9621, 9622, 9625, 9626, 9628, 9635, 9637, 9638, 9641, 9642, 9644, 9649, 9650, 9652, 9656, 9667, 9669, 9670, 9673, 
    9674, 9676, 9681, 9682, 9684, 9688, 9697, 9698, 9700, 9704, 9712, 9743, 9751, 9755, 9757, 9758, 9767, 9771, 9773, 9774, 
    9779, 9781, 9782, 9785, 9786, 9788, 9799, 9803, 9805, 9806, 9811, 9813, 9814, 9817, 9818, 9820, 9827, 9829, 9830, 9833, 
    9834, 9836, 9841, 9842, 9844, 9848, 9863, 9867, 9869, 9870, 9875, 9877, 9878, 9881, 9882, 9884, 9891, 9893, 9894, 9897, 
    9898, 9900, 9905, 9906, 9908, 9912, 9923, 9925, 9926, 9929, 9930, 9932, 9937, 9938, 9940, 9944, 9953, 9954, 9956, 9960, 
    9968, 9991, 9995, 9997, 9998, 10003, 10005, 10006, 10009, 10010, 10012, 10019, 10021, 10022, 10025, 10026, 10028, 10033, 10034, 10036, 
    10040, 10051, 10053, 10054, 10057, 10058, 10060, 10065, 10066, 10068, 10072, 10081, 10082, 10084, 10088, 10096, 10115, 10117, 10118, 10121, 
    10122, 10124, 10129, 10130, 10132, 10136, 10145, 10146, 10148, 10152, 10160, 10177, 10178, 10180, 10184, 10192, 10208, 10271, 10287, 10295, 
    10299, 10301, 10302, 10319, 10327, 10331, 10333, 10334, 10343, 10347, 10349, 10350, 10355, 10357, 10358, 10361, 10362, 10364, 10383, 10391, 
    10395, 10397, 10398, 10407, 10411, 10413, 10414, 10419, 10421, 10422, 10425, 10426, 10428, 10439, 10443, 10445, 10446, 10451, 10453, 10454, 
    10457, 10458, 10460, 10467, 10469, 10470, 10473, 10474, 10476, 10481, 10482, 10484, 10488, 10511, 10519, 10523, 10525, 10526, 10535, 10539, 
    10541, 10542, 10547, 10549, 10550, 10553, 10554, 10556, 10567, 10571, 10573, 10574, 10579, 10581, 10582, 10585, 10586, 10588, 10595, 10597, 
    10598, 10601, 10602, 10604, 10609, 10610, 10612, 10616, 10631, 10635, 10637, 10638, 10643, 10645, 10646, 10649, 10650, 10652, 10659, 10661, 
    10662, 10665, 10666, 10668, 10673, 10674, 10676, 10680, 10691, 10693, 10694, 10697, 10698, 10700, 10705, 10706, 10708, 10712, 10721, 10722, 
    10724, 10728, 10736, 10767, 10775, 10779, 10781, 10782, 10791, 10795, 10797, 10798, 10803, 10805, 10806, 10809, 10810, 10812, 10823, 10827, 
    10829, 10830, 10835, 10837, 10838, 10841, 10842, 10844, 10851, 10853, 10854, 10857, 10858, 10860, 10865, 10866, 10868, 10872, 10887, 10891, 
    10893, 10894, 10899, 10901, 10902, 10905, 10906, 10908, 10915, 10917, 10918, 10921, 10922, 10924, 10929, 10930, 10932, 10936, 10947, 10949, 
    10950, 10953, 10954, 10956, 10961, 10962, 10964, 10968, 10977, 10978, 10980, 10984, 10992, 11015, 11019, 11021, 11022, 11027, 11029, 11030, 
    11033, 11034, 11036, 11043, 11045, 11046, 11049, 11050, 11052, 11057, 11058, 11060, 11064, 11075, 11077, 11078, 11081, 11082, 11084, 11089, 
    11090, 11092, 11096, 11105, 11106, 11108, 11112, 11120, 11139, 11141, 11142, 11145, 11146, 11148, 11153, 11154, 11156, 11160, 11169, 11170, 
    11172, 11176, 11184, 11201, 11202, 11204, 11208, 11216, 11232, 11279, 11287, 11291, 11293, 11294, 11303, 11307, 11309, 11310, 11315, 11317, 
    11318, 11321, 11322, 11324, 11335, 11339, 11341, 11342, 11347, 11349, 11350, 11353, 11354, 11356, 11363, 11365, 11366, 11369, 11370, 11372, 
    11377, 11378, 11380, 11384, 11399, 11403, 11405, 11406, 11411, 11413, 11414, 11417, 11418, 11420, 11427, 11429, 11430, 11433, 11434, 11436, 
    11441, 11442, 11444, 11448, 11459, 11461, 11462, 11465, 11466, 11468, 11473, 11474, 11476, 11480, 11489, 11490, 11492, 11496, 11504, 11527, 
    11531, 11533, 11534, 11539, 11541, 11542, 11545, 11546, 11548, 11555, 11557, 11558, 11561, 11562, 11564, 11569, 11570, 11572, 11576, 11587, 
    11589, 11590, 11593, 11594, 11596, 11601, 11602, 11604, 11608, 11617, 11618, 11620, 11624, 11632, 11651, 11653, 11654, 11657, 11658, 11660, 
    11665, 11666, 11668, 11672, 11681, 11682, 11684, 11688, 11696, 11713, 11714, 11716, 11720, 11728, 11744, 11783, 11787, 11789, 11790, 11795, 
    11797, 11798, 11801, 11802, 11804, 11811, 11813, 11814, 11817, 11818, 11820, 11825, 11826, 11828, 11832, 11843, 11845, 11846, 11849, 11850, 
    11852, 11857, 11858, 11860, 11864, 11873, 11874, 11876, 11880, 11888, 11907, 11909, 11910, 11913, 11914, 11916, 11921, 11922, 11924, 11928, 
    11937, 11938, 11940, 11944, 11952, 11969, 11970, 11972, 11976, 11984, 12000, 12035, 12037, 12038, 12041, 12042, 12044, 12049, 12050, 12052, 
    12056, 12065, 12066, 12068, 12072, 12080, 12097, 12098, 12100, 12104, 12112, 12128, 12161, 12162, 12164, 12168, 12176, 12192, 12224, 12319, 
    12335, 12343, 12347, 12349, 12350, 12367, 12375, 12379, 12381, 12382, 12391, 12395, 12397, 12398, 12403, 12405, 12406, 12409, 12410, 12412, 
    12431, 12439, 12443, 12445, 12446, 12455, 12459, 12461, 12462, 12467, 12469, 12470, 12473, 12474, 12476, 12487, 12491, 12493, 12494, 12499, 
    12501, 12502, 12505, 12506, 12508, 12515, 12517, 12518, 12521, 12522, 12524, 12529, 12530, 12532, 12536, 12559, 12567, 12571, 12573, 12574, 
    12583, 12587, 12589, 12590, 12595, 12597, 12598, 12601, 12602, 12604, 12615, 12619, 12621, 12622, 12627, 12629, 12630, 12633, 12634, 12636, 
    12643, 12645, 12646, 12649, 12650, 12652, 12657, 12658, 12660, 12664, 12679, 12683, 12685, 12686, 12691, 12693, 12694, 12697, 12698, 12700, 
    12707, 12709, 12710, 12713, 12714, 12716, 12721, 12722, 12724, 12728, 12739, 12741, 12742, 12745, 12746, 12748, 12753, 12754, 12756, 12760, 
    12769, 12770, 12772, 12776, 12784, 12815, 12823, 12827, 12829, 12830, 12839, 12843, 12845, 12846, 12851, 12853, 12854, 12857, 12858, 12860, 
    12871, 12875, 12877, 12878, 12883, 12885, 12886, 12889, 12890, 12892, 12899, 12901, 12902, 12905, 12906, 12908, 12913, 12914, 12916, 12920, 
    12935, 12939, 12941, 12942, 12947, 12949, 12950, 12953, 12954, 12956, 12963, 12965, 12966, 12969, 12970, 12972, 12977, 12978, 12980, 12984, 
    12995, 12997, 12998, 13001, 13002, 13004, 13009, 13010, 13012, 13016, 13025, 13026, 13028, 13032, 13040, 13063, 13067, 13069, 13070, 13075, 
    13077, 13078, 13081, 13082, 13084, 13091, 13093, 13094, 13097, 13098, 13100, 13105, 13106, 13108, 13112, 13123, 13125, 13126, 13129, 13130, 
    13132, 13137, 13138, 13140, 13144, 13153, 13154, 13156, 13160, 13168, 13187, 13189, 13190, 13193, 13194, 13196, 13201, 13202, 13204, 13208, 
    13217, 13218, 13220, 13224, 13232, 13249, 13250, 13252, 13256, 13264, 13280, 13327, 13335, 13339, 13341, 13342, 13351, 13355, 13357, 13358, 
    13363, 13365, 13366, 13369, 13370, 13372, 13383, 13387, 13389, 13390, 13395, 13397, 13398, 13401, 13402, 13404, 13411, 13413, 13414, 13417, 
    13418, 13420, 13425, 13426, 13428, 13432, 13447, 13451, 13453, 13454, 13459, 13461, 13462, 13465, 13466, 13468, 13475, 13477, 13478, 13481, 
    13482, 13484, 13489, 13490, 13492, 13496, 13507, 13509, 13510, 13513, 13514, 13516, 13521, 13522, 13524, 13528, 13537, 13538, 13540, 13544, 
    13552, 13575, 13579, 13581, 13582, 13587, 13589, 13590, 13593, 13594, 13596, 13603, 13605, 13606, 13609, 13610, 13612, 13617, 13618, 13620, 
    13624, 13635, 13637, 13638, 13641, 13642, 13644, 13649, 13650, 13652, 13656, 13665, 13666, 13668, 13672, 13680, 13699, 13701, 13702, 13705, 
    13706, 13708, 13713, 13714, 13716, 13720, 13729, 13730, 13732, 13736, 13744, 13761, 13762, 13764, 13768, 13776, 13792, 13831, 13835, 13837, 
    13838, 13843, 13845, 13846, 13849, 13850, 13852, 13859, 13861, 13862, 13865, 13866, 13868, 13873, 13874, 13876, 13880, 13891, 13893, 13894, 
    13897, 13898, 13900, 13905, 13906, 13908, 13912, 13921, 13922, 13924, 13928, 13936, 13955, 13957, 13958, 13961, 13962, 13964, 13969, 13970, 
    13972, 13976, 13985, 13986, 13988, 13992, 14000, 14017, 14018, 14020, 14024, 14032, 14048, 14083, 14085, 14086, 14089, 14090, 14092, 14097, 
    14098, 14100, 14104, 14113, 14114, 14116, 14120, 14128, 14145, 14146, 14148, 14152, 14160, 14176, 14209, 14210, 14212, 14216, 14224, 14240, 
    14272, 14351, 14359, 14363, 14365, 14366, 14375, 14379, 14381, 14382, 14387, 14389, 14390, 14393, 14394, 14396, 14407, 14411, 14413, 14414, 
    14419, 14421, 14422, 14425, 14426, 14428, 14435, 14437, 14438, 14441, 14442, 14444, 14449, 14450, 14452, 14456, 14471, 14475, 14477, 14478, 
    14483, 14485, 14486, 14489, 14490, 14492, 14499, 14501, 14502, 14505, 14506, 14508, 14513, 14514, 14516, 14520, 14531, 14533, 14534, 14537, 
    14538, 14540, 14545, 14546, 14548, 14552, 14561, 14562, 14564, 14568, 14576, 14599, 14603, 14605, 14606, 14611, 14613, 14614, 14617, 14618, 
    14620, 14627, 14629, 14630, 14633, 14634, 14636, 14641, 14642, 14644, 14648, 14659, 14661, 14662, 14665, 14666, 14668, 14673, 14674, 14676, 
    14680, 14689, 14690, 14692, 14696, 14704, 14723, 14725, 14726, 14729, 14730, 14732, 14737, 14738, 14740, 14744, 14753, 14754, 14756, 14760, 
    14768, 14785, 14786, 14788, 14792, 14800, 14816, 14855, 14859, 14861, 14862, 14867, 14869, 14870, 14873, 14874, 14876, 14883, 14885, 14886, 
    14889, 14890, 14892, 14897, 14898, 14900, 14904, 14915, 14917, 14918, 14921, 14922, 14924, 14929, 14930, 14932, 14936, 14945, 14946, 14948, 
    14952, 14960, 14979, 14981, 14982, 14985, 14986, 14988, 14993, 14994, 14996, 15000, 15009, 15010, 15012, 15016, 15024, 15041, 15042, 15044, 
    15048, 15056, 15072, 15107, 15109, 15110, 15113, 15114, 15116, 15121, 15122, 15124, 15128, 15137, 15138, 15140, 15144, 15152, 15169, 15170, 
    15172, 15176, 15184, 15200, 15233, 15234, 15236, 15240, 15248, 15264, 15296, 15367, 15371, 15373, 15374, 15379, 15381, 15382, 15385, 15386, 
    15388, 15395, 15397, 15398, 15401, 15402, 15404, 15409, 15410, 15412, 15416, 15427, 15429, 15430, 15433, 15434, 15436, 15441, 15442, 15444, 
    15448, 15457, 15458, 15460, 15464, 15472, 15491, 15493, 15494, 15497, 15498, 15500, 15505, 15506, 15508, 15512, 15521, 15522, 15524, 15528, 
    15536, 15553, 15554, 15556, 15560, 15568, 15584, 15619, 15621, 15622, 15625, 15626, 15628, 15633, 15634, 15636, 15640, 15649, 15650, 15652, 
    15656, 15664, 15681, 15682, 15684, 15688, 15696, 15712, 15745, 15746, 15748, 15752, 15760, 15776, 15808, 15875, 15877, 15878, 15881, 15882, 
    15884, 15889, 15890, 15892, 15896, 15905, 15906, 15908, 15912, 15920, 15937, 15938, 15940, 15944, 15952, 15968, 16001, 16002, 16004, 16008, 
    16016, 16032, 16064, 16129, 16130, 16132, 16136, 16144, 16160, 16192, 16256, 16447, 16479, 16495, 16503, 16507, 16509, 16510, 16543, 16559, 
    16567, 16571, 16573, 16574, 16591, 16599, 16603, 16605, 16606, 16615, 16619, 16621, 16622, 16627, 16629, 16630, 16633, 16634, 16636, 16671, 
    16687, 16695, 16699, 16701, 16702, 16719, 16727, 16731, 16733, 16734, 16743, 16747, 16749, 16750, 16755, 16757, 16758, 16761, 16762, 16764, 
    16783, 16791, 16795, 16797, 16798, 16807, 16811, 16813, 16814, 16819, 16821, 16822, 16825, 16826, 16828, 16839, 16843, 16845, 16846, 16851, 
    16853, 16854, 16857, 16858, 16860, 16867, 16869, 16870, 16873, 16874, 16876, 16881, 16882, 16884, 16888, 16927, 16943, 16951, 16955, 16957, 
    16958, 16975, 16983, 16987, 16989, 16990, 16999, 17003, 17005, 17006, 17011, 17013, 17014, 17017, 17018, 17020, 17039, 17047, 17051, 17053, 
    17054, 17063, 17067, 17069, 17070, 17075, 17077, 17078, 17081, 17082, 17084, 17095, 17099, 17101, 17102, 17107, 17109, 17110, 17113, 17114, 
    17116, 17123, 17125, 17126, 17129, 17130, 17132, 17137, 17138, 17140, 17144, 17167, 17175, 17179, 17181, 17182, 17191, 17195, 17197, 17198, 
    17203, 17205, 17206, 17209, 17210, 17212, 17223, 17227, 17229, 17230, 17235, 17237, 17238, 17241, 17242, 17244, 17251, 17253, 17254, 17257, 
    17258, 17260, 17265, 17266, 17268, 17272, 17287, 17291, 17293, 17294, 17299, 17301, 17302, 17305, 17306, 17308, 17315, 17317, 17318, 17321, 
    17322, 17324, 17329, 17330, 17332, 17336, 17347, 17349, 17350, 17353, 17354, 17356, 17361, 17362, 17364, 17368, 17377, 17378, 17380, 17384, 
    17392, 17439, 17455, 17463, 17467, 17469, 17470, 17487, 17495, 17499, 17501, 17502, 17511, 17515, 17517, 17518, 17523, 17525, 17526, 17529, 
    17530, 17532, 17551, 17559, 17563, 17565, 17566, 17575, 17579, 17581, 17582, 17587, 17589, 17590, 17593, 17594, 17596, 17607, 17611, 17613, 
    17614, 17619, 17621, 17622, 17625, 17626, 17628, 17635, 17637, 17638, 17641, 17642, 17644, 17649, 17650, 17652, 17656, 17679, 17687, 17691, 
    17693, 17694, 17703, 17707, 17709, 17710, 17715, 17717, 17718, 17721, 17722, 17724, 17735, 17739, 17741, 17742, 17747, 17749, 17750, 17753, 
    17754, 17756, 17763, 17765, 17766, 17769, 17770, 17772, 17777, 17778, 17780, 17784, 17799, 17803, 17805, 17806, 17811, 17813, 17814, 17817, 
    17818, 17820, 17827, 17829, 17830, 17833, 17834, 17836, 17841, 17842, 17844, 17848, 17859, 17861, 17862, 17865, 17866, 17868, 17873, 17874, 
    17876, 17880, 17889, 17890, 17892, 17896, 17904, 17935, 17943, 17947, 17949, 17950, 17959, 17963, 17965, 17966, 17971, 17973, 17974, 17977, 
    17978, 17980, 17991, 17995, 17997, 17998, 18003, 18005, 18006, 18009, 18010, 18012, 18019, 18021, 18022, 18025, 18026, 18028, 18033, 18034, 
    18036, 18040, 18055, 18059, 18061, 18062, 18067, 18069, 18070, 18073, 18074, 18076, 18083, 18085, 18086, 18089, 18090, 18092, 18097, 18098, 
    18100, 18104, 18115, 18117, 18118, 18121, 18122, 18124, 18129, 18130, 18132, 18136, 18145, 18146, 18148, 18152, 18160, 18183, 18187, 18189, 
    18190, 18195, 18197, 18198, 18201, 18202, 18204, 18211, 18213, 18214, 18217, 18218, 18220, 18225, 18226, 18228, 18232, 18243, 18245, 18246, 
    18249, 18250, 18252, 18257, 18258, 18260, 18264, 18273, 18274, 18276, 18280, 18288, 18307, 18309, 18310, 18313, 18314, 18316, 18321, 18322, 
    18324, 18328, 18337, 18338, 18340, 18344, 18352, 18369, 18370, 18372, 18376, 18384, 18400, 18463, 18479, 18487, 18491, 18493, 18494, 18511, 
    18519, 18523, 18525, 18526, 18535, 18539, 18541, 18542, 18547, 18549, 18550, 18553, 18554, 18556, 18575, 18583, 18587, 18589, 18590, 18599, 
    18603, 18605, 18606, 18611, 18613, 18614, 18617, 18618, 18620, 18631, 18635, 18637, 18638, 18643, 18645, 18646, 18649, 18650, 18652, 18659, 
    18661, 18662, 18665, 18666, 18668, 18673, 18674, 18676, 18680, 18703, 18711, 18715, 18717, 18718, 18727, 18731, 18733, 18734, 18739, 18741, 
    18742, 18745, 18746, 18748, 18759, 18763, 18765, 18766, 18771, 18773, 18774, 18777, 18778, 18780, 18787, 18789, 18790, 18793, 18794, 18796, 
    18801, 18802, 18804, 18808, 18823, 18827, 18829, 18830, 18835, 18837, 18838, 18841, 18842, 18844, 18851, 18853, 18854, 18857, 18858, 18860, 
    18865, 18866, 18868, 18872, 18883, 18885, 18886, 18889, 18890, 18892, 18897, 18898, 18900, 18904, 18913, 18914, 18916, 18920, 18928, 18959, 
    18967, 18971, 18973, 18974, 18983, 18987, 18989, 18990, 18995, 18997, 18998, 19001, 19002, 19004, 19015, 19019, 19021, 19022, 19027, 19029, 
    19030, 19033, 19034, 19036, 19043, 19045, 19046, 19049, 19050, 19052, 19057, 19058, 19060, 19064, 19079, 19083, 19085, 19086, 19091, 19093, 
    19094, 19097, 19098, 19100, 19107, 19109, 19110, 19113, 19114, 19116, 19121, 19122, 19124, 19128, 19139, 19141, 19142, 19145, 19146, 19148, 
    19153, 19154, 19156, 19160, 19169, 19170, 19172, 19176, 19184, 19207, 19211, 19213, 19214, 19219, 19221, 19222, 19225, 19226, 19228, 19235, 
    19237, 19238, 19241, 19242, 19244, 19249, 19250, 19252, 19256, 19267, 19269, 19270, 19273, 19274, 19276, 19281, 19282, 19284, 19288, 19297, 
    19298, 19300, 19304, 19312, 19331, 19333, 19334, 19337, 19338, 19340, 19345, 19346, 19348, 19352, 19361, 19362, 19364, 19368, 19376, 19393, 
    19394, 19396, 19400, 19408, 19424, 19471, 19479, 19483, 19485, 19486, 19495, 19499, 19501, 19502, 19507, 19509, 19510, 19513, 19514, 19516, 
    19527, 19531, 19533, 19534, 19539, 19541, 19542, 19545, 19546, 19548, 19555, 19557, 19558, 19561, 19562, 19564, 19569, 19570, 19572, 19576, 
    19591, 19595, 19597, 19598, 19603, 19605, 19606, 19609, 19610, 19612, 19619, 19621, 19622, 19625, 19626, 19628, 19633, 19634, 19636, 19640, 
    19651, 19653, 19654, 19657, 19658, 19660, 19665, 19666, 19668, 19672, 19681, 19682, 19684, 19688, 19696, 19719, 19723, 19725, 19726, 19731, 
    19733, 19734, 19737, 19738, 19740, 19747, 19749, 19750, 19753, 19754, 19756, 19761, 19762, 19764, 19768, 19779, 19781, 19782, 19785, 19786, 
    19788, 19793, 19794, 19796, 19800, 19809, 19810, 19812, 19816, 19824, 19843, 19845, 19846, 19849, 19850, 19852, 19857, 19858, 19860, 19864, 
    19873, 19874, 19876, 19880, 19888, 19905, 19906, 19908, 19912, 19920, 19936, 19975, 19979, 19981, 19982, 19987, 19989, 19990, 19993, 19994, 
    19996, 20003, 20005, 20006, 20009, 20010, 20012, 20017, 20018, 20020, 20024, 20035, 20037, 20038, 20041, 20042, 20044, 20049, 20050, 20052, 
    20056, 20065, 20066, 20068, 20072, 20080, 20099, 20101, 20102, 20105, 20106, 20108, 20113, 20114, 20116, 20120, 20129, 20130, 20132, 20136, 
    20144, 20161, 20162, 20164, 20168, 20176, 20192, 20227, 20229, 20230, 20233, 20234, 20236, 20241, 20242, 20244, 20248, 20257, 20258, 20260, 
    20264, 20272, 20289, 20290, 20292, 20296, 20304, 20320, 20353, 20354, 20356, 20360, 20368, 20384, 20416, 20511, 20527, 20535, 20539, 20541, 
    20542, 20559, 20567, 20571, 20573, 20574, 20583, 20587, 20589, 20590, 20595, 20597, 20598, 20601, 20602, 20604, 20623, 20631, 20635, 20637, 
    20638, 20647, 20651, 20653, 20654, 20659, 20661, 20662, 20665, 20666, 20668, 20679, 20683, 20685, 20686, 20691, 20693, 20694, 20697, 20698, 
    20700, 20707, 20709, 20710, 20713, 20714, 20716, 20721, 20722, 20724, 20728, 20751, 20759, 20763, 20765, 20766, 20775, 20779, 20781, 20782, 
    20787, 20789, 20790, 20793, 20794, 20796, 20807, 20811, 20813, 20814, 20819, 20821, 20822, 20825, 20826, 20828, 20835, 20837, 20838, 20841, 
    20842, 20844, 20849, 20850, 20852, 20856, 20871, 20875, 20877, 20878, 20883, 20885, 20886, 20889, 20890, 20892, 20899, 20901, 20902, 20905, 
    20906, 20908, 20913, 20914, 20916, 20920, 20931, 20933, 20934, 20937, 20938, 20940, 20945, 20946, 20948, 20952, 20961, 20962, 20964, 20968, 
    20976, 21007, 21015, 21019, 21021, 21022, 21031, 21035, 21037, 21038, 21043, 21045, 21046, 21049, 21050, 21052, 21063, 21067, 21069, 21070, 
    21075, 21077, 21078, 21081, 21082, 21084, 21091, 21093, 21094, 21097, 21098, 21100, 21105, 21106, 21108, 21112, 21127, 21131, 21133, 21134, 
    21139, 21141, 21142, 21145, 21146, 21148, 21155, 21157, 21158, 21161, 21162, 21164, 21169, 21170, 21172, 21176, 21187, 21189, 21190, 21193, 
    21194, 21196, 21201, 21202, 21204, 21208, 21217, 21218, 21220, 21224, 21232, 21255, 21259, 21261, 21262, 21267, 21269, 21270, 21273, 21274, 
    21276, 21283, 21285, 21286, 21289, 21290, 21292, 21297, 21298, 21300, 21304, 21315, 21317, 21318, 21321, 21322, 21324, 21329, 21330, 21332, 
    21336, 21345, 21346, 21348, 21352, 21360, 21379, 21381, 21382, 21385, 21386, 21388, 21393, 21394, 21396, 21400, 21409, 21410, 21412, 21416, 
    21424, 21441, 21442, 21444, 21448, 21456, 21472, 21519, 21527, 21531, 21533, 21534, 21543, 21547, 21549, 21550, 21555, 21557, 21558, 21561, 
    21562, 21564, 21575, 21579, 21581, 21582, 21587, 21589, 21590, 21593, 21594, 21596, 21603, 21605, 21606, 21609, 21610, 21612, 21617, 21618, 
    21620, 21624, 21639, 21643, 21645, 21646, 21651, 21653, 21654, 21657, 21658, 21660, 21667, 21669, 21670, 21673, 21674, 21676, 21681, 21682, 
    21684, 21688, 21699, 21701, 21702, 21705, 21706, 21708, 21713, 21714, 21716, 21720, 21729, 21730, 21732, 21736, 21744, 21767, 21771, 21773, 
    21774, 21779, 21781, 21782, 21785, 21786, 21788, 21795, 21797, 21798, 21801, 21802, 21804, 21809, 21810, 21812, 21816, 21827, 21829, 21830, 
    21833, 21834, 21836, 21841, 21842, 21844, 21848, 21857, 21858, 21860, 21864, 21872, 21891, 21893, 21894, 21897, 21898, 21900, 21905, 21906, 
    21908, 21912, 21921, 21922, 21924, 21928, 21936, 21953, 21954, 21956, 21960, 21968, 21984, 22023, 22027, 22029, 22030, 22035, 22037, 22038, 
    22041, 22042, 22044, 22051, 22053, 22054, 22057, 22058, 22060, 22065, 22066, 22068, 22072, 22083, 22085, 22086, 22089, 22090, 22092, 22097, 
    22098, 22100, 22104, 22113, 22114, 22116, 22120, 22128, 22147, 22149, 22150, 22153, 22154, 22156, 22161, 22162, 22164, 22168, 22177, 22178, 
    22180, 22184, 22192, 22209, 22210, 22212, 22216, 22224, 22240, 22275, 22277, 22278, 22281, 22282, 22284, 22289, 22290, 22292, 22296, 22305, 
    22306, 22308, 22312, 22320, 22337, 22338, 22340, 22344, 22352, 22368, 22401, 22402, 22404, 22408, 22416, 22432, 22464, 22543, 22551, 22555, 
    22557, 22558, 22567, 22571, 22573, 22574, 22579, 22581, 22582, 22585, 22586, 22588, 22599, 22603, 22605, 22606, 22611, 22613, 22614, 22617, 
    22618, 22620, 22627, 22629, 22630, 22633, 22634, 22636, 22641, 22642, 22644, 22648, 22663, 22667, 22669, 22670, 22675, 22677, 22678, 22681, 
    22682, 22684, 22691, 22693, 22694, 22697, 22698, 22700, 22705, 22706, 22708, 22712, 22723, 22725, 22726, 22729, 22730, 22732, 22737, 22738, 
    22740, 22744, 22753, 22754, 22756, 22760, 22768, 22791, 22795, 22797, 22798, 22803, 22805, 22806, 22809, 22810, 22812, 22819, 22821, 22822, 
    22825, 22826, 22828, 22833, 22834, 22836, 22840, 22851, 22853, 22854, 22857, 22858, 22860, 22865, 22866, 22868, 22872, 22881, 22882, 22884, 
    22888, 22896, 22915, 22917, 22918, 22921, 22922, 22924, 22929, 22930, 22932, 22936, 22945, 22946, 22948, 22952, 22960, 22977, 22978, 22980, 
    22984, 22992, 23008, 23047, 23051, 23053, 23054, 23059, 23061, 23062, 23065, 23066, 23068, 23075, 23077, 23078, 23081, 23082, 23084, 23089, 
    23090, 23092, 23096, 23107, 23109, 23110, 23113, 23114, 23116, 23121, 23122, 23124, 23128, 23137, 23138, 23140, 23144, 23152, 23171, 23173, 
    23174, 23177, 23178, 23180, 23185, 23186, 23188, 23192, 23201, 23202, 23204, 23208, 23216, 23233, 23234, 23236, 23240, 23248, 23264, 23299, 
    23301, 23302, 23305, 23306, 23308, 23313, 23314, 23316, 23320, 23329, 23330, 23332, 23336, 23344, 23361, 23362, 23364, 23368, 23376, 23392, 
    23425, 23426, 23428, 23432, 23440, 23456, 23488, 23559, 23563, 23565, 23566, 23571, 23573, 23574, 23577, 23578, 23580, 23587, 23589, 23590, 
    23593, 23594, 23596, 23601, 23602, 23604, 23608, 23619, 23621, 23622, 23625, 23626, 23628, 23633, 23634, 23636, 23640, 23649, 23650, 23652, 
    23656, 23664, 23683, 23685, 23686, 23689, 23690, 23692, 23697, 23698, 23700, 23704, 23713, 23714, 23716, 23720, 23728, 23745, 23746, 23748, 
    23752, 23760, 23776, 23811, 23813, 23814, 23817, 23818, 23820, 23825, 23826, 23828, 23832, 23841, 23842, 23844, 23848, 23856, 23873, 23874, 
    23876, 23880, 23888, 23904, 23937, 23938, 23940, 23944, 23952, 23968, 24000, 24067, 24069, 24070, 24073, 24074, 24076, 24081, 24082, 24084, 
    24088, 24097, 24098, 24100, 24104, 24112, 24129, 24130, 24132, 24136, 24144, 24160, 24193, 24194, 24196, 24200, 24208, 24224, 24256, 24321, 
    24322, 24324, 24328, 24336, 24352, 24384, 24448, 24607, 24623, 24631, 24635, 24637, 24638, 24655, 24663, 24667, 24669, 24670, 24679, 24683, 
    24685, 24686, 24691, 24693, 24694, 24697, 24698, 24700, 24719, 24727, 24731, 24733, 24734, 24743, 24747, 24749, 24750, 24755, 24757, 24758, 
    24761, 24762, 24764, 24775, 24779, 24781, 24782, 24787, 24789, 24790, 24793, 24794, 24796, 24803, 24805, 24806, 24809, 24810, 24812, 24817, 
    24818, 24820, 24824, 24847, 24855, 24859, 24861, 24862, 24871, 24875, 24877, 24878, 24883, 24885, 24886, 24889, 24890, 24892, 24903, 24907, 
    24909, 24910, 24915, 24917, 24918, 24921, 24922, 24924, 24931, 24933, 24934, 24937, 24938, 24940, 24945, 24946, 24948, 24952, 24967, 24971, 
    24973, 24974, 24979, 24981, 24982, 24985, 24986, 24988, 24995, 24997, 24998, 25001, 25002, 25004, 25009, 25010, 25012, 25016, 25027, 25029, 
    25030, 25033, 25034, 25036, 25041, 25042, 25044, 25048, 25057, 25058, 25060, 25064, 25072, 25103, 25111, 25115, 25117, 25118, 25127, 25131, 
    25133, 25134, 25139, 25141, 25142, 25145, 25146, 25148, 25159, 25163, 25165, 25166, 25171, 25173, 25174, 25177, 25178, 25180, 25187, 25189, 
    25190, 25193, 25194, 25196, 25201, 25202, 25204, 25208, 25223, 25227, 25229, 25230, 25235, 25237, 25238, 25241, 25242, 25244, 25251, 25253, 
    25254, 25257, 25258, 25260, 25265, 25266, 25268, 25272, 25283, 25285, 25286, 25289, 25290, 25292, 25297, 25298, 25300, 25304, 25313, 25314, 
    25316, 25320, 25328, 25351, 25355, 25357, 25358, 25363, 25365, 25366, 25369, 25370, 25372, 25379, 25381, 25382, 25385, 25386, 25388, 25393, 
    25394, 25396, 25400, 25411, 25413, 25414, 25417, 25418, 25420, 25425, 25426, 25428, 25432, 25441, 25442, 25444, 25448, 25456, 25475, 25477, 
    25478, 25481, 25482, 25484, 25489, 25490, 25492, 25496, 25505, 25506, 25508, 25512, 25520, 25537, 25538, 25540, 25544, 25552, 25568, 25615, 
    25623, 25627, 25629, 25630, 25639, 25643, 25645, 25646, 25651, 25653, 25654, 25657, 25658, 25660, 25671, 25675, 25677, 25678, 25683, 25685, 
    25686, 25689, 25690, 25692, 25699, 25701, 25702, 25705, 25706, 25708, 25713, 25714, 25716, 25720, 25735, 25739, 25741, 25742, 25747, 25749, 
    25750, 25753, 25754, 25756, 25763, 25765, 25766, 25769, 25770, 25772, 25777, 25778, 25780, 25784, 25795, 25797, 25798, 25801, 25802, 25804, 
    25809, 25810, 25812, 25816, 25825, 25826, 25828, 25832, 25840, 25863, 25867, 25869, 25870, 25875, 25877, 25878, 25881, 25882, 25884, 25891, 
    25893, 25894, 25897, 25898, 25900, 25905, 25906, 25908, 25912, 25923, 25925, 25926, 25929, 25930, 25932, 25937, 25938, 25940, 25944, 25953, 
    25954, 25956, 25960, 25968, 25987, 25989, 25990, 25993, 25994, 25996, 26001, 26002, 26004, 26008, 26017, 26018, 26020, 26024, 26032, 26049, 
    26050, 26052, 26056, 26064, 26080, 26119, 26123, 26125, 26126, 26131, 26133, 26134, 26137, 26138, 26140, 26147, 26149, 26150, 26153, 26154, 
    26156, 26161, 26162, 26164, 26168, 26179, 26181, 26182, 26185, 26186, 26188, 26193, 26194, 26196, 26200, 26209, 26210, 26212, 26216, 26224, 
    26243, 26245, 26246, 26249, 26250, 26252, 26257, 26258, 26260, 26264, 26273, 26274, 26276, 26280, 26288, 26305, 26306, 26308, 26312, 26320, 
    26336, 26371, 26373, 26374, 26377, 26378, 26380, 26385, 26386, 26388, 26392, 26401, 26402, 26404, 26408, 26416, 26433, 26434, 26436, 26440, 
    26448, 26464, 26497, 26498, 26500, 26504, 26512, 26528, 26560, 26639, 26647, 26651, 26653, 26654, 26663, 26667, 26669, 26670, 26675, 26677, 
    26678, 26681, 26682, 26684, 26695, 26699, 26701, 26702, 26707, 26709, 26710, 26713, 26714, 26716, 26723, 26725, 26726, 26729, 26730, 26732, 
    26737, 26738, 26740, 26744, 26759, 26763, 26765, 26766, 26771, 26773, 26774, 26777, 26778, 26780, 26787, 26789, 26790, 26793, 26794, 26796, 
    26801, 26802, 26804, 26808, 26819, 26821, 26822, 26825, 26826, 26828, 26833, 26834, 26836, 26840, 26849, 26850, 26852, 26856, 26864, 26887, 
    26891, 26893, 26894, 26899, 26901, 26902, 26905, 26906, 26908, 26915, 26917, 26918, 26921, 26922, 26924, 26929, 26930, 26932, 26936, 26947, 
    26949, 26950, 26953, 26954, 26956, 26961, 26962, 26964, 26968, 26977, 26978, 26980, 26984, 26992, 27011, 27013, 27014, 27017, 27018, 27020, 
    27025, 27026, 27028, 27032, 27041, 27042, 27044, 27048, 27056, 27073, 27074, 27076, 27080, 27088, 27104, 27143, 27147, 27149, 27150, 27155, 
    27157, 27158, 27161, 27162, 27164, 27171, 27173, 27174, 27177, 27178, 27180, 27185, 27186, 27188, 27192, 27203, 27205, 27206, 27209, 27210, 
    27212, 27217, 27218, 27220, 27224, 27233, 27234, 27236, 27240, 27248, 27267, 27269, 27270, 27273, 27274, 27276, 27281, 27282, 27284, 27288, 
    27297, 27298, 27300, 27304, 27312, 27329, 27330, 27332, 27336, 27344, 27360, 27395, 27397, 27398, 27401, 27402, 27404, 27409, 27410, 27412, 
    27416, 27425, 27426, 27428, 27432, 27440, 27457, 27458, 27460, 27464, 27472, 27488, 27521, 27522, 27524, 27528, 27536, 27552, 27584, 27655, 
    27659, 27661, 27662, 27667, 27669, 27670, 27673, 27674, 27676, 27683, 27685, 27686, 27689, 27690, 27692, 27697, 27698, 27700, 27704, 27715, 
    27717, 27718, 27721, 27722, 27724, 27729, 27730, 27732, 27736, 27745, 27746, 27748, 27752, 27760, 27779, 27781, 27782, 27785, 27786, 27788, 
    27793, 27794, 27796, 27800, 27809, 27810, 27812, 27816, 27824, 27841, 27842, 27844, 27848, 27856, 27872, 27907, 27909, 27910, 27913, 27914, 
    27916, 27921, 27922, 27924, 27928, 27937, 27938, 27940, 27944, 27952, 27969, 27970, 27972, 27976, 27984, 28000, 28033, 28034, 28036, 28040, 
    28048, 28064, 28096, 28163, 28165, 28166, 28169, 28170, 28172, 28177, 28178, 28180, 28184, 28193, 28194, 28196, 28200, 28208, 28225, 28226, 
    28228, 28232, 28240, 28256, 28289, 28290, 28292, 28296, 28304, 28320, 28352, 28417, 28418, 28420, 28424, 28432, 28448, 28480, 28544, 28687, 
    28695, 28699, 28701, 28702, 28711, 28715, 28717, 28718, 28723, 28725, 28726, 28729, 28730, 28732, 28743, 28747, 28749, 28750, 28755, 28757, 
    28758, 28761, 28762, 28764, 28771, 28773, 28774, 28777, 28778, 28780, 28785, 28786, 28788, 28792, 28807, 28811, 28813, 28814, 28819, 28821, 
    28822, 28825, 28826, 28828, 28835, 28837, 28838, 28841, 28842, 28844, 28849, 28850, 28852, 28856, 28867, 28869, 28870, 28873, 28874, 28876, 
    28881, 28882, 28884, 28888, 28897, 28898, 28900, 28904, 28912, 28935, 28939, 28941, 28942, 28947, 28949, 28950, 28953, 28954, 28956, 28963, 
    28965, 28966, 28969, 28970, 28972, 28977, 28978, 28980, 28984, 28995, 28997, 28998, 29001, 29002, 29004, 29009, 29010, 29012, 29016, 29025, 
    29026, 29028, 29032, 29040, 29059, 29061, 29062, 29065, 29066, 29068, 29073, 29074, 29076, 29080, 29089, 29090, 29092, 29096, 29104, 29121, 
    29122, 29124, 29128, 29136, 29152, 29191, 29195, 29197, 29198, 29203, 29205, 29206, 29209, 29210, 29212, 29219, 29221, 29222, 29225, 29226, 
    29228, 29233, 29234, 29236, 29240, 29251, 29253, 29254, 29257, 29258, 29260, 29265, 29266, 29268, 29272, 29281, 29282, 29284, 29288, 29296, 
    29315, 29317, 29318, 29321, 29322, 29324, 29329, 29330, 29332, 29336, 29345, 29346, 29348, 29352, 29360, 29377, 29378, 29380, 29384, 29392, 
    29408, 29443, 29445, 29446, 29449, 29450, 29452, 29457, 29458, 29460, 29464, 29473, 29474, 29476, 29480, 29488, 29505, 29506, 29508, 29512, 
    29520, 29536, 29569, 29570, 29572, 29576, 29584, 29600, 29632, 29703, 29707, 29709, 29710, 29715, 29717, 29718, 29721, 29722, 29724, 29731, 
    29733, 29734, 29737, 29738, 29740, 29745, 29746, 29748, 29752, 29763, 29765, 29766, 29769, 29770, 29772, 29777, 29778, 29780, 29784, 29793, 
    29794, 29796, 29800, 29808, 29827, 29829, 29830, 29833, 29834, 29836, 29841, 29842, 29844, 29848, 29857, 29858, 29860, 29864, 29872, 29889, 
    29890, 29892, 29896, 29904, 29920, 29955, 29957, 29958, 29961, 29962, 29964, 29969, 29970, 29972, 29976, 29985, 29986, 29988, 29992, 30000, 
    30017, 30018, 30020, 30024, 30032, 30048, 30081, 30082, 30084, 30088, 30096, 30112, 30144, 30211, 30213, 30214, 30217, 30218, 30220, 30225, 
    30226, 30228, 30232, 30241, 30242, 30244, 30248, 30256, 30273, 30274, 30276, 30280, 30288, 30304, 30337, 30338, 30340, 30344, 30352, 30368, 
    30400, 30465, 30466, 30468, 30472, 30480, 30496, 30528, 30592, 30727, 30731, 30733, 30734, 30739, 30741, 30742, 30745, 30746, 30748, 30755, 
    30757, 30758, 30761, 30762, 30764, 30769, 30770, 30772, 30776, 30787, 30789, 30790, 30793, 30794, 30796, 30801, 30802, 30804, 30808, 30817, 
    30818, 30820, 30824, 30832, 30851, 30853, 30854, 30857, 30858, 30860, 30865, 30866, 30868, 30872, 30881, 30882, 30884, 30888, 30896, 30913, 
    30914, 30916, 30920, 30928, 30944, 30979, 30981, 30982, 30985, 30986, 30988, 30993, 30994, 30996, 31000, 31009, 31010, 31012, 31016, 31024, 
    31041, 31042, 31044, 31048, 31056, 31072, 31105, 31106, 31108, 31112, 31120, 31136, 31168, 31235, 31237, 31238, 31241, 31242, 31244, 31249, 
    31250, 31252, 31256, 31265, 31266, 31268, 31272, 31280, 31297, 31298, 31300, 31304, 31312, 31328, 31361, 31362, 31364, 31368, 31376, 31392, 
    31424, 31489, 31490, 31492, 31496, 31504, 31520, 31552, 31616, 31747, 31749, 31750, 31753, 31754, 31756, 31761, 31762, 31764, 31768, 31777, 
    31778, 31780, 31784, 31792, 31809, 31810, 31812, 31816, 31824, 31840, 31873, 31874, 31876, 31880, 31888, 31904, 31936, 32001, 32002, 32004, 
    32008, 32016, 32032, 32064, 32128, 32257, 32258, 32260, 32264, 32272, 32288, 32320, 32384, 32512, 32831, 32863, 32879, 32887, 32891, 32893, 
    32894, 32927, 32943, 32951, 32955, 32957, 32958, 32975, 32983, 32987, 32989, 32990, 32999, 33003, 33005, 33006, 33011, 33013, 33014, 33017, 
    33018, 33020, 33055, 33071, 33079, 33083, 33085, 33086, 33103, 33111, 33115, 33117, 33118, 33127, 33131, 33133, 33134, 33139, 33141, 33142, 
    33145, 33146, 33148, 33167, 33175, 33179, 33181, 33182, 33191, 33195, 33197, 33198, 33203, 33205, 33206, 33209, 33210, 33212, 33223, 33227, 
    33229, 33230, 33235, 33237, 33238, 33241, 33242, 33244, 33251, 33253, 33254, 33257, 33258, 33260, 33265, 33266, 33268, 33272, 33311, 33327, 
    33335, 33339, 33341, 33342, 33359, 33367, 33371, 33373, 33374, 33383, 33387, 33389, 33390, 33395, 33397, 33398, 33401, 33402, 33404, 33423, 
    33431, 33435, 33437, 33438, 33447, 33451, 33453, 33454, 33459, 33461, 33462, 33465, 33466, 33468, 33479, 33483, 33485, 33486, 33491, 33493, 
    33494, 33497, 33498, 33500, 33507, 33509, 33510, 33513, 33514, 33516, 33521, 33522, 33524, 33528, 33551, 33559, 33563, 33565, 33566, 33575, 
    33579, 33581, 33582, 33587, 33589, 33590, 33593, 33594, 33596, 33607, 33611, 33613, 33614, 33619, 33621, 33622, 33625, 33626, 33628, 33635, 
    33637, 33638, 33641, 33642, 33644, 33649, 33650, 33652, 33656, 33671, 33675, 33677, 33678, 33683, 33685, 33686, 33689, 33690, 33692, 33699, 
    33701, 33702, 33705, 33706, 33708, 33713, 33714, 33716, 33720, 33731, 33733, 33734, 33737, 33738, 33740, 33745, 33746, 33748, 33752, 33761, 
    33762, 33764, 33768, 33776, 33823, 33839, 33847, 33851, 33853, 33854, 33871, 33879, 33883, 33885, 33886, 33895, 33899, 33901, 33902, 33907, 
    33909, 33910, 33913, 33914, 33916, 33935, 33943, 33947, 33949, 33950, 33959, 33963, 33965, 33966, 33971, 33973, 33974, 33977, 33978, 33980, 
    33991, 33995, 33997, 33998, 34003, 34005, 34006, 34009, 34010, 34012, 34019, 34021, 34022, 34025, 34026, 34028, 34033, 34034, 34036, 34040, 
    34063, 34071, 34075, 34077, 34078, 34087, 34091, 34093, 34094, 34099, 34101, 34102, 34105, 34106, 34108, 34119, 34123, 34125, 34126, 34131, 
    34133, 34134, 34137, 34138, 34140, 34147, 34149, 34150, 34153, 34154, 34156, 34161, 34162, 34164, 34168, 34183, 34187, 34189, 34190, 34195, 
    34197, 34198, 34201, 34202, 34204, 34211, 34213, 34214, 34217, 34218, 34220, 34225, 34226, 34228, 34232, 34243, 34245, 34246, 34249, 34250, 
    34252, 34257, 34258, 34260, 34264, 34273, 34274, 34276, 34280, 34288, 34319, 34327, 34331, 34333, 34334, 34343, 34347, 34349, 34350, 34355, 
    34357, 34358, 34361, 34362, 34364, 34375, 34379, 34381, 34382, 34387, 34389, 34390, 34393, 34394, 34396, 34403, 34405, 34406, 34409, 34410, 
    34412, 34417, 34418, 34420, 34424, 34439, 34443, 34445, 34446, 34451, 34453, 34454, 34457, 34458, 34460, 34467, 34469, 34470, 34473, 34474, 
    34476, 34481, 34482, 34484, 34488, 34499, 34501, 34502, 34505, 34506, 34508, 34513, 34514, 34516, 34520, 34529, 34530, 34532, 34536, 34544, 
    34567, 34571, 34573, 34574, 34579, 34581, 34582, 34585, 34586, 34588, 34595, 34597, 34598, 34601, 34602, 34604, 34609, 34610, 34612, 34616, 
    34627, 34629, 34630, 34633, 34634, 34636, 34641, 34642, 34644, 34648, 34657, 34658, 34660, 34664, 34672, 34691, 34693, 34694, 34697, 34698, 
    34700, 34705, 34706, 34708, 34712, 34721, 34722, 34724, 34728, 34736, 34753, 34754, 34756, 34760, 34768, 34784, 34847, 34863, 34871, 34875, 
    34877, 34878, 34895, 34903, 34907, 34909, 34910, 34919, 34923, 34925, 34926, 34931, 34933, 34934, 34937, 34938, 34940, 34959, 34967, 34971, 
    34973, 34974, 34983, 34987, 34989, 34990, 34995, 34997, 34998, 35001, 35002, 35004, 35015, 35019, 35021, 35022, 35027, 35029, 35030, 35033, 
    35034, 35036, 35043, 35045, 35046, 35049, 35050, 35052, 35057, 35058, 35060, 35064, 35087, 35095, 35099, 35101, 35102, 35111, 35115, 35117, 
    35118, 35123, 35125, 35126, 35129, 35130, 35132, 35143, 35147, 35149, 35150, 35155, 35157, 35158, 35161, 35162, 35164, 35171, 35173, 35174, 
    35177, 35178, 35180, 35185, 35186, 35188, 35192, 35207, 35211, 35213, 35214, 35219, 35221, 35222, 35225, 35226, 35228, 35235, 35237, 35238, 
    35241, 35242, 35244, 35249, 35250, 35252, 35256, 35267, 35269, 35270, 35273, 35274, 35276, 35281, 35282, 35284, 35288, 35297, 35298, 35300, 
    35304, 35312, 35343, 35351, 35355, 35357, 35358, 35367, 35371, 35373, 35374, 35379, 35381, 35382, 35385, 35386, 35388, 35399, 35403, 35405, 
    35406, 35411, 35413, 35414, 35417, 35418, 35420, 35427, 35429, 35430, 35433, 35434, 35436, 35441, 35442, 35444, 35448, 35463, 35467, 35469, 
    35470, 35475, 35477, 35478, 35481, 35482, 35484, 35491, 35493, 35494, 35497, 35498, 35500, 35505, 35506, 35508, 35512, 35523, 35525, 35526, 
    35529, 35530, 35532, 35537, 35538, 35540, 35544, 35553, 35554, 35556, 35560, 35568, 35591, 35595, 35597, 35598, 35603, 35605, 35606, 35609, 
    35610, 35612, 35619, 35621, 35622, 35625, 35626, 35628, 35633, 35634, 35636, 35640, 35651, 35653, 35654, 35657, 35658, 35660, 35665, 35666, 
    35668, 35672, 35681, 35682, 35684, 35688, 35696, 35715, 35717, 35718, 35721, 35722, 35724, 35729, 35730, 35732, 35736, 35745, 35746, 35748, 
    35752, 35760, 35777, 35778, 35780, 35784, 35792, 35808, 35855, 35863, 35867, 35869, 35870, 35879, 35883, 35885, 35886, 35891, 35893, 35894, 
    35897, 35898, 35900, 35911, 35915, 35917, 35918, 35923, 35925, 35926, 35929, 35930, 35932, 35939, 35941, 35942, 35945, 35946, 35948, 35953, 
    35954, 35956, 35960, 35975, 35979, 35981, 35982, 35987, 35989, 35990, 35993, 35994, 35996, 36003, 36005, 36006, 36009, 36010, 36012, 36017, 
    36018, 36020, 36024, 36035, 36037, 36038, 36041, 36042, 36044, 36049, 36050, 36052, 36056, 36065, 36066, 36068, 36072, 36080, 36103, 36107, 
    36109, 36110, 36115, 36117, 36118, 36121, 36122, 36124, 36131, 36133, 36134, 36137, 36138, 36140, 36145, 36146, 36148, 36152, 36163, 36165, 
    36166, 36169, 36170, 36172, 36177, 36178, 36180, 36184, 36193, 36194, 36196, 36200, 36208, 36227, 36229, 36230, 36233, 36234, 36236, 36241, 
    36242, 36244, 36248, 36257, 36258, 36260, 36264, 36272, 36289, 36290, 36292, 36296, 36304, 36320, 36359, 36363, 36365, 36366, 36371, 36373, 
    36374, 36377, 36378, 36380, 36387, 36389, 36390, 36393, 36394, 36396, 36401, 36402, 36404, 36408, 36419, 36421, 36422, 36425, 36426, 36428, 
    36433, 36434, 36436, 36440, 36449, 36450, 36452, 36456, 36464, 36483, 36485, 36486, 36489, 36490, 36492, 36497, 36498, 36500, 36504, 36513, 
    36514, 36516, 36520, 36528, 36545, 36546, 36548, 36552, 36560, 36576, 36611, 36613, 36614, 36617, 36618, 36620, 36625, 36626, 36628, 36632, 
    36641, 36642, 36644, 36648, 36656, 36673, 36674, 36676, 36680, 36688, 36704, 36737, 36738, 36740, 36744, 36752, 36768, 36800, 36895, 36911, 
    36919, 36923, 36925, 36926, 36943, 36951, 36955, 36957, 36958, 36967, 36971, 36973, 36974, 36979, 36981, 36982, 36985, 36986, 36988, 37007, 
    37015, 37019, 37021, 37022, 37031, 37035, 37037, 37038, 37043, 37045, 37046, 37049, 37050, 37052, 37063, 37067, 37069, 37070, 37075, 37077, 
    37078, 37081, 37082, 37084, 37091, 37093, 37094, 37097, 37098, 37100, 37105, 37106, 37108, 37112, 37135, 37143, 37147, 37149, 37150, 37159, 
    37163, 37165, 37166, 37171, 37173, 37174, 37177, 37178, 37180, 37191, 37195, 37197, 37198, 37203, 37205, 37206, 37209, 37210, 37212, 37219, 
    37221, 37222, 37225, 37226, 37228, 37233, 37234, 37236, 37240, 37255, 37259, 37261, 37262, 37267, 37269, 37270, 37273, 37274, 37276, 37283, 
    37285, 37286, 37289, 37290, 37292, 37297, 37298, 37300, 37304, 37315, 37317, 37318, 37321, 37322, 37324, 37329, 37330, 37332, 37336, 37345, 
    37346, 37348, 37352, 37360, 37391, 37399, 37403, 37405, 37406, 37415, 37419, 37421, 37422, 37427, 37429, 37430, 37433, 37434, 37436, 37447, 
    37451, 37453, 37454, 37459, 37461, 37462, 37465, 37466, 37468, 37475, 37477, 37478, 37481, 37482, 37484, 37489, 37490, 37492, 37496, 37511, 
    37515, 37517, 37518, 37523, 37525, 37526, 37529, 37530, 37532, 37539, 37541, 37542, 37545, 37546, 37548, 37553, 37554, 37556, 37560, 37571, 
    37573, 37574, 37577, 37578, 37580, 37585, 37586, 37588, 37592, 37601, 37602, 37604, 37608, 37616, 37639, 37643, 37645, 37646, 37651, 37653, 
    37654, 37657, 37658, 37660, 37667, 37669, 37670, 37673, 37674, 37676, 37681, 37682, 37684, 37688, 37699, 37701, 37702, 37705, 37706, 37708, 
    37713, 37714, 37716, 37720, 37729, 37730, 37732, 37736, 37744, 37763, 37765, 37766, 37769, 37770, 37772, 37777, 37778, 37780, 37784, 37793, 
    37794, 37796, 37800, 37808, 37825, 37826, 37828, 37832, 37840, 37856, 37903, 37911, 37915, 37917, 37918, 37927, 37931, 37933, 37934, 37939, 
    37941, 37942, 37945, 37946, 37948, 37959, 37963, 37965, 37966, 37971, 37973, 37974, 37977, 37978, 37980, 37987, 37989, 37990, 37993, 37994, 
    37996, 38001, 38002, 38004, 38008, 38023, 38027, 38029, 38030, 38035, 38037, 38038, 38041, 38042, 38044, 38051, 38053, 38054, 38057, 38058, 
    38060, 38065, 38066, 38068, 38072, 38083, 38085, 38086, 38089, 38090, 38092, 38097, 38098, 38100, 38104, 38113, 38114, 38116, 38120, 38128, 
    38151, 38155, 38157, 38158, 38163, 38165, 38166, 38169, 38170, 38172, 38179, 38181, 38182, 38185, 38186, 38188, 38193, 38194, 38196, 38200, 
    38211, 38213, 38214, 38217, 38218, 38220, 38225, 38226, 38228, 38232, 38241, 38242, 38244, 38248, 38256, 38275, 38277, 38278, 38281, 38282, 
    38284, 38289, 38290, 38292, 38296, 38305, 38306, 38308, 38312, 38320, 38337, 38338, 38340, 38344, 38352, 38368, 38407, 38411, 38413, 38414, 
    38419, 38421, 38422, 38425, 38426, 38428, 38435, 38437, 38438, 38441, 38442, 38444, 38449, 38450, 38452, 38456, 38467, 38469, 38470, 38473, 
    38474, 38476, 38481, 38482, 38484, 38488, 38497, 38498, 38500, 38504, 38512, 38531, 38533, 38534, 38537, 38538, 38540, 38545, 38546, 38548, 
    38552, 38561, 38562, 38564, 38568, 38576, 38593, 38594, 38596, 38600, 38608, 38624, 38659, 38661, 38662, 38665, 38666, 38668, 38673, 38674, 
    38676, 38680, 38689, 38690, 38692, 38696, 38704, 38721, 38722, 38724, 38728, 38736, 38752, 38785, 38786, 38788, 38792, 38800, 38816, 38848, 
    38927, 38935, 38939, 38941, 38942, 38951, 38955, 38957, 38958, 38963, 38965, 38966, 38969, 38970, 38972, 38983, 38987, 38989, 38990, 38995, 
    38997, 38998, 39001, 39002, 39004, 39011, 39013, 39014, 39017, 39018, 39020, 39025, 39026, 39028, 39032, 39047, 39051, 39053, 39054, 39059, 
    39061, 39062, 39065, 39066, 39068, 39075, 39077, 39078, 39081, 39082, 39084, 39089, 39090, 39092, 39096, 39107, 39109, 39110, 39113, 39114, 
    39116, 39121, 39122, 39124, 39128, 39137, 39138, 39140, 39144, 39152, 39175, 39179, 39181, 39182, 39187, 39189, 39190, 39193, 39194, 39196, 
    39203, 39205, 39206, 39209, 39210, 39212, 39217, 39218, 39220, 39224, 39235, 39237, 39238, 39241, 39242, 39244, 39249, 39250, 39252, 39256, 
    39265, 39266, 39268, 39272, 39280, 39299, 39301, 39302, 39305, 39306, 39308, 39313, 39314, 39316, 39320, 39329, 39330, 39332, 39336, 39344, 
    39361, 39362, 39364, 39368, 39376, 39392, 39431, 39435, 39437, 39438, 39443, 39445, 39446, 39449, 39450, 39452, 39459, 39461, 39462, 39465, 
    39466, 39468, 39473, 39474, 39476, 39480, 39491, 39493, 39494, 39497, 39498, 39500, 39505, 39506, 39508, 39512, 39521, 39522, 39524, 39528, 
    39536, 39555, 39557, 39558, 39561, 39562, 39564, 39569, 39570, 39572, 39576, 39585, 39586, 39588, 39592, 39600, 39617, 39618, 39620, 39624, 
    39632, 39648, 39683, 39685, 39686, 39689, 39690, 39692, 39697, 39698, 39700, 39704, 39713, 39714, 39716, 39720, 39728, 39745, 39746, 39748, 
    39752, 39760, 39776, 39809, 39810, 39812, 39816, 39824, 39840, 39872, 39943, 39947, 39949, 39950, 39955, 39957, 39958, 39961, 39962, 39964, 
    39971, 39973, 39974, 39977, 39978, 39980, 39985, 39986, 39988, 39992, 40003, 40005, 40006, 40009, 40010, 40012, 40017, 40018, 40020, 40024, 
    40033, 40034, 40036, 40040, 40048, 40067, 40069, 40070, 40073, 40074, 40076, 40081, 40082, 40084, 40088, 40097, 40098, 40100, 40104, 40112, 
    40129, 40130, 40132, 40136, 40144, 40160, 40195, 40197, 40198, 40201, 40202, 40204, 40209, 40210, 40212, 40216, 40225, 40226, 40228, 40232, 
    40240, 40257, 40258, 40260, 40264, 40272, 40288, 40321, 40322, 40324, 40328, 40336, 40352, 40384, 40451, 40453, 40454, 40457, 40458, 40460, 
    40465, 40466, 40468, 40472, 40481, 40482, 40484, 40488, 40496, 40513, 40514, 40516, 40520, 40528, 40544, 40577, 40578, 40580, 40584, 40592, 
    40608, 40640, 40705, 40706, 40708, 40712, 40720, 40736, 40768, 40832, 40991, 41007, 41015, 41019, 41021, 41022, 41039, 41047, 41051, 41053, 
    41054, 41063, 41067, 41069, 41070, 41075, 41077, 41078, 41081, 41082, 41084, 41103, 41111, 41115, 41117, 41118, 41127, 41131, 41133, 41134, 
    41139, 41141, 41142, 41145, 41146, 41148, 41159, 41163, 41165, 41166, 41171, 41173, 41174, 41177, 41178, 41180, 41187, 41189, 41190, 41193, 
    41194, 41196, 41201, 41202, 41204, 41208, 41231, 41239, 41243, 41245, 41246, 41255, 41259, 41261, 41262, 41267, 41269, 41270, 41273, 41274, 
    41276, 41287, 41291, 41293, 41294, 41299, 41301, 41302, 41305, 41306, 41308, 41315, 41317, 41318, 41321, 41322, 41324, 41329, 41330, 41332, 
    41336, 41351, 41355, 41357, 41358, 41363, 41365, 41366, 41369, 41370, 41372, 41379, 41381, 41382, 41385, 41386, 41388, 41393, 41394, 41396, 
    41400, 41411, 41413, 41414, 41417, 41418, 41420, 41425, 41426, 41428, 41432, 41441, 41442, 41444, 41448, 41456, 41487, 41495, 41499, 41501, 
    41502, 41511, 41515, 41517, 41518, 41523, 41525, 41526, 41529, 41530, 41532, 41543, 41547, 41549, 41550, 41555, 41557, 41558, 41561, 41562, 
    41564, 41571, 41573, 41574, 41577, 41578, 41580, 41585, 41586, 41588, 41592, 41607, 41611, 41613, 41614, 41619, 41621, 41622, 41625, 41626, 
    41628, 41635, 41637, 41638, 41641, 41642, 41644, 41649, 41650, 41652, 41656, 41667, 41669, 41670, 41673, 41674, 41676, 41681, 41682, 41684, 
    41688, 41697, 41698, 41700, 41704, 41712, 41735, 41739, 41741, 41742, 41747, 41749, 41750, 41753, 41754, 41756, 41763, 41765, 41766, 41769, 
    41770, 41772, 41777, 41778, 41780, 41784, 41795, 41797, 41798, 41801, 41802, 41804, 41809, 41810, 41812, 41816, 41825, 41826, 41828, 41832, 
    41840, 41859, 41861, 41862, 41865, 41866, 41868, 41873, 41874, 41876, 41880, 41889, 41890, 41892, 41896, 41904, 41921, 41922, 41924, 41928, 
    41936, 41952, 41999, 42007, 42011, 42013, 42014, 42023, 42027, 42029, 42030, 42035, 42037, 42038, 42041, 42042, 42044, 42055, 42059, 42061, 
    42062, 42067, 42069, 42070, 42073, 42074, 42076, 42083, 42085, 42086, 42089, 42090, 42092, 42097, 42098, 42100, 42104, 42119, 42123, 42125, 
    42126, 42131, 42133, 42134, 42137, 42138, 42140, 42147, 42149, 42150, 42153, 42154, 42156, 42161, 42162, 42164, 42168, 42179, 42181, 42182, 
    42185, 42186, 42188, 42193, 42194, 42196, 42200, 42209, 42210, 42212, 42216, 42224, 42247, 42251, 42253, 42254, 42259, 42261, 42262, 42265, 
    42266, 42268, 42275, 42277, 42278, 42281, 42282, 42284, 42289, 42290, 42292, 42296, 42307, 42309, 42310, 42313, 42314, 42316, 42321, 42322, 
    42324, 42328, 42337, 42338, 42340, 42344, 42352, 42371, 42373, 42374, 42377, 42378, 42380, 42385, 42386, 42388, 42392, 42401, 42402, 42404, 
    42408, 42416, 42433, 42434, 42436, 42440, 42448, 42464, 42503, 42507, 42509, 42510, 42515, 42517, 42518, 42521, 42522, 42524, 42531, 42533, 
    42534, 42537, 42538, 42540, 42545, 42546, 42548, 42552, 42563, 42565, 42566, 42569, 42570, 42572, 42577, 42578, 42580, 42584, 42593, 42594, 
    42596, 42600, 42608, 42627, 42629, 42630, 42633, 42634, 42636, 42641, 42642, 42644, 42648, 42657, 42658, 42660, 42664, 42672, 42689, 42690, 
    42692, 42696, 42704, 42720, 42755, 42757, 42758, 42761, 42762, 42764, 42769, 42770, 42772, 42776, 42785, 42786, 42788, 42792, 42800, 42817, 
    42818, 42820, 42824, 42832, 42848, 42881, 42882, 42884, 42888, 42896, 42912, 42944, 43023, 43031, 43035, 43037, 43038, 43047, 43051, 43053, 
    43054, 43059, 43061, 43062, 43065, 43066, 43068, 43079, 43083, 43085, 43086, 43091, 43093, 43094, 43097, 43098, 43100, 43107, 43109, 43110, 
    43113, 43114, 43116, 43121, 43122, 43124, 43128, 43143, 43147, 43149, 43150, 43155, 43157, 43158, 43161, 43162, 43164, 43171, 43173, 43174, 
    43177, 43178, 43180, 43185, 43186, 43188, 43192, 43203, 43205, 43206, 43209, 43210, 43212, 43217, 43218, 43220, 43224, 43233, 43234, 43236, 
    43240, 43248, 43271, 43275, 43277, 43278, 43283, 43285, 43286, 43289, 43290, 43292, 43299, 43301, 43302, 43305, 43306, 43308, 43313, 43314, 
    43316, 43320, 43331, 43333, 43334, 43337, 43338, 43340, 43345, 43346, 43348, 43352, 43361, 43362, 43364, 43368, 43376, 43395, 43397, 43398, 
    43401, 43402, 43404, 43409, 43410, 43412, 43416, 43425, 43426, 43428, 43432, 43440, 43457, 43458, 43460, 43464, 43472, 43488, 43527, 43531, 
    43533, 43534, 43539, 43541, 43542, 43545, 43546, 43548, 43555, 43557, 43558, 43561, 43562, 43564, 43569, 43570, 43572, 43576, 43587, 43589, 
    43590, 43593, 43594, 43596, 43601, 43602, 43604, 43608, 43617, 43618, 43620, 43624, 43632, 43651, 43653, 43654, 43657, 43658, 43660, 43665, 
    43666, 43668, 43672, 43681, 43682, 43684, 43688, 43696, 43713, 43714, 43716, 43720, 43728, 43744, 43779, 43781, 43782, 43785, 43786, 43788, 
    43793, 43794, 43796, 43800, 43809, 43810, 43812, 43816, 43824, 43841, 43842, 43844, 43848, 43856, 43872, 43905, 43906, 43908, 43912, 43920, 
    43936, 43968, 44039, 44043, 44045, 44046, 44051, 44053, 44054, 44057, 44058, 44060, 44067, 44069, 44070, 44073, 44074, 44076, 44081, 44082, 
    44084, 44088, 44099, 44101, 44102, 44105, 44106, 44108, 44113, 44114, 44116, 44120, 44129, 44130, 44132, 44136, 44144, 44163, 44165, 44166, 
    44169, 44170, 44172, 44177, 44178, 44180, 44184, 44193, 44194, 44196, 44200, 44208, 44225, 44226, 44228, 44232, 44240, 44256, 44291, 44293, 
    44294, 44297, 44298, 44300, 44305, 44306, 44308, 44312, 44321, 44322, 44324, 44328, 44336, 44353, 44354, 44356, 44360, 44368, 44384, 44417, 
    44418, 44420, 44424, 44432, 44448, 44480, 44547, 44549, 44550, 44553, 44554, 44556, 44561, 44562, 44564, 44568, 44577, 44578, 44580, 44584, 
    44592, 44609, 44610, 44612, 44616, 44624, 44640, 44673, 44674, 44676, 44680, 44688, 44704, 44736, 44801, 44802, 44804, 44808, 44816, 44832, 
    44864, 44928, 45071, 45079, 45083, 45085, 45086, 45095, 45099, 45101, 45102, 45107, 45109, 45110, 45113, 45114, 45116, 45127, 45131, 45133, 
    45134, 45139, 45141, 45142, 45145, 45146, 45148, 45155, 45157, 45158, 45161, 45162, 45164, 45169, 45170, 45172, 45176, 45191, 45195, 45197, 
    45198, 45203, 45205, 45206, 45209, 45210, 45212, 45219, 45221, 45222, 45225, 45226, 45228, 45233, 45234, 45236, 45240, 45251, 45253, 45254, 
    45257, 45258, 45260, 45265, 45266, 45268, 45272, 45281, 45282, 45284, 45288, 45296, 45319, 45323, 45325, 45326, 45331, 45333, 45334, 45337, 
    45338, 45340, 45347, 45349, 45350, 45353, 45354, 45356, 45361, 45362, 45364, 45368, 45379, 45381, 45382, 45385, 45386, 45388, 45393, 45394, 
    45396, 45400, 45409, 45410, 45412, 45416, 45424, 45443, 45445, 45446, 45449, 45450, 45452, 45457, 45458, 45460, 45464, 45473, 45474, 45476, 
    45480, 45488, 45505, 45506, 45508, 45512, 45520, 45536, 45575, 45579, 45581, 45582, 45587, 45589, 45590, 45593, 45594, 45596, 45603, 45605, 
    45606, 45609, 45610, 45612, 45617, 45618, 45620, 45624, 45635, 45637, 45638, 45641, 45642, 45644, 45649, 45650, 45652, 45656, 45665, 45666, 
    45668, 45672, 45680, 45699, 45701, 45702, 45705, 45706, 45708, 45713, 45714, 45716, 45720, 45729, 45730, 45732, 45736, 45744, 45761, 45762, 
    45764, 45768, 45776, 45792, 45827, 45829, 45830, 45833, 45834, 45836, 45841, 45842, 45844, 45848, 45857, 45858, 45860, 45864, 45872, 45889, 
    45890, 45892, 45896, 45904, 45920, 45953, 45954, 45956, 45960, 45968, 45984, 46016, 46087, 46091, 46093, 46094, 46099, 46101, 46102, 46105, 
    46106, 46108, 46115, 46117, 46118, 46121, 46122, 46124, 46129, 46130, 46132, 46136, 46147, 46149, 46150, 46153, 46154, 46156, 46161, 46162, 
    46164, 46168, 46177, 46178, 46180, 46184, 46192, 46211, 46213, 46214, 46217, 46218, 46220, 46225, 46226, 46228, 46232, 46241, 46242, 46244, 
    46248, 46256, 46273, 46274, 46276, 46280, 46288, 46304, 46339, 46341, 46342, 46345, 46346, 46348, 46353, 46354, 46356, 46360, 46369, 46370, 
    46372, 46376, 46384, 46401, 46402, 46404, 46408, 46416, 46432, 46465, 46466, 46468, 46472, 46480, 46496, 46528, 46595, 46597, 46598, 46601, 
    46602, 46604, 46609, 46610, 46612, 46616, 46625, 46626, 46628, 46632, 46640, 46657, 46658, 46660, 46664, 46672, 46688, 46721, 46722, 46724, 
    46728, 46736, 46752, 46784, 46849, 46850, 46852, 46856, 46864, 46880, 46912, 46976, 47111, 47115, 47117, 47118, 47123, 47125, 47126, 47129, 
    47130, 47132, 47139, 47141, 47142, 47145, 47146, 47148, 47153, 47154, 47156, 47160, 47171, 47173, 47174, 47177, 47178, 47180, 47185, 47186, 
    47188, 47192, 47201, 47202, 47204, 47208, 47216, 47235, 47237, 47238, 47241, 47242, 47244, 47249, 47250, 47252, 47256, 47265, 47266, 47268, 
    47272, 47280, 47297, 47298, 47300, 47304, 47312, 47328, 47363, 47365, 47366, 47369, 47370, 47372, 47377, 47378, 47380, 47384, 47393, 47394, 
    47396, 47400, 47408, 47425, 47426, 47428, 47432, 47440, 47456, 47489, 47490, 47492, 47496, 47504, 47520, 47552, 47619, 47621, 47622, 47625, 
    47626, 47628, 47633, 47634, 47636, 47640, 47649, 47650, 47652, 47656, 47664, 47681, 47682, 47684, 47688, 47696, 47712, 47745, 47746, 47748, 
    47752, 47760, 47776, 47808, 47873, 47874, 47876, 47880, 47888, 47904, 47936, 48000, 48131, 48133, 48134, 48137, 48138, 48140, 48145, 48146, 
    48148, 48152, 48161, 48162, 48164, 48168, 48176, 48193, 48194, 48196, 48200, 48208, 48224, 48257, 48258, 48260, 48264, 48272, 48288, 48320, 
    48385, 48386, 48388, 48392, 48400, 48416, 48448, 48512, 48641, 48642, 48644, 48648, 48656, 48672, 48704, 48768, 48896, 49183, 49199, 49207, 
    49211, 49213, 49214, 49231, 49239, 49243, 49245, 49246, 49255, 49259, 49261, 49262, 49267, 49269, 49270, 49273, 49274, 49276, 49295, 49303, 
    49307, 49309, 49310, 49319, 49323, 49325, 49326, 49331, 49333, 49334, 49337, 49338, 49340, 49351, 49355, 49357, 49358, 49363, 49365, 49366, 
    49369, 49370, 49372, 49379, 49381, 49382, 49385, 49386, 49388, 49393, 49394, 49396, 49400, 49423, 49431, 49435, 49437, 49438, 49447, 49451, 
    49453, 49454, 49459, 49461, 49462, 49465, 49466, 49468, 49479, 49483, 49485, 49486, 49491, 49493, 49494, 49497, 49498, 49500, 49507, 49509, 
    49510, 49513, 49514, 49516, 49521, 49522, 49524, 49528, 49543, 49547, 49549, 49550, 49555, 49557, 49558, 49561, 49562, 49564, 49571, 49573, 
    49574, 49577, 49578, 49580, 49585, 49586, 49588, 49592, 49603, 49605, 49606, 49609, 49610, 49612, 49617, 49618, 49620, 49624, 49633, 49634, 
    49636, 49640, 49648, 49679, 49687, 49691, 49693, 49694, 49703, 49707, 49709, 49710, 49715, 49717, 49718, 49721, 49722, 49724, 49735, 49739, 
    49741, 49742, 49747, 49749, 49750, 49753, 49754, 49756, 49763, 49765, 49766, 49769, 49770, 49772, 49777, 49778, 49780, 49784, 49799, 49803, 
    49805, 49806, 49811, 49813, 49814, 49817, 49818, 49820, 49827, 49829, 49830, 49833, 49834, 49836, 49841, 49842, 49844, 49848, 49859, 49861, 
    49862, 49865, 49866, 49868, 49873, 49874, 49876, 49880, 49889, 49890, 49892, 49896, 49904, 49927, 49931, 49933, 49934, 49939, 49941, 49942, 
    49945, 49946, 49948, 49955, 49957, 49958, 49961, 49962, 49964, 49969, 49970, 49972, 49976, 49987, 49989, 49990, 49993, 49994, 49996, 50001, 
    50002, 50004, 50008, 50017, 50018, 50020, 50024, 50032, 50051, 50053, 50054, 50057, 50058, 50060, 50065, 50066, 50068, 50072, 50081, 50082, 
    50084, 50088, 50096, 50113, 50114, 50116, 50120, 50128, 50144, 50191, 50199, 50203, 50205, 50206, 50215, 50219, 50221, 50222, 50227, 50229, 
    50230, 50233, 50234, 50236, 50247, 50251, 50253, 50254, 50259, 50261, 50262, 50265, 50266, 50268, 50275, 50277, 50278, 50281, 50282, 50284, 
    50289, 50290, 50292, 50296, 50311, 50315, 50317, 50318, 50323, 50325, 50326, 50329, 50330, 50332, 50339, 50341, 50342, 50345, 50346, 50348, 
    50353, 50354, 50356, 50360, 50371, 50373, 50374, 50377, 50378, 50380, 50385, 50386, 50388, 50392, 50401, 50402, 50404, 50408, 50416, 50439, 
    50443, 50445, 50446, 50451, 50453, 50454, 50457, 50458, 50460, 50467, 50469, 50470, 50473, 50474, 50476, 50481, 50482, 50484, 50488, 50499, 
    50501, 50502, 50505, 50506, 50508, 50513, 50514, 50516, 50520, 50529, 50530, 50532, 50536, 50544, 50563, 50565, 50566, 50569, 50570, 50572, 
    50577, 50578, 50580, 50584, 50593, 50594, 50596, 50600, 50608, 50625, 50626, 50628, 50632, 50640, 50656, 50695, 50699, 50701, 50702, 50707, 
    50709, 50710, 50713, 50714, 50716, 50723, 50725, 50726, 50729, 50730, 50732, 50737, 50738, 50740, 50744, 50755, 50757, 50758, 50761, 50762, 
    50764, 50769, 50770, 50772, 50776, 50785, 50786, 50788, 50792, 50800, 50819, 50821, 50822, 50825, 50826, 50828, 50833, 50834, 50836, 50840, 
    50849, 50850, 50852, 50856, 50864, 50881, 50882, 50884, 50888, 50896, 50912, 50947, 50949, 50950, 50953, 50954, 50956, 50961, 50962, 50964, 
    50968, 50977, 50978, 50980, 50984, 50992, 51009, 51010, 51012, 51016, 51024, 51040, 51073, 51074, 51076, 51080, 51088, 51104, 51136, 51215, 
    51223, 51227, 51229, 51230, 51239, 51243, 51245, 51246, 51251, 51253, 51254, 51257, 51258, 51260, 51271, 51275, 51277, 51278, 51283, 51285, 
    51286, 51289, 51290, 51292, 51299, 51301, 51302, 51305, 51306, 51308, 51313, 51314, 51316, 51320, 51335, 51339, 51341, 51342, 51347, 51349, 
    51350, 51353, 51354, 51356, 51363, 51365, 51366, 51369, 51370, 51372, 51377, 51378, 51380, 51384, 51395, 51397, 51398, 51401, 51402, 51404, 
    51409, 51410, 51412, 51416, 51425, 51426, 51428, 51432, 51440, 51463, 51467, 51469, 51470, 51475, 51477, 51478, 51481, 51482, 51484, 51491, 
    51493, 51494, 51497, 51498, 51500, 51505, 51506, 51508, 51512, 51523, 51525, 51526, 51529, 51530, 51532, 51537, 51538, 51540, 51544, 51553, 
    51554, 51556, 51560, 51568, 51587, 51589, 51590, 51593, 51594, 51596, 51601, 51602, 51604, 51608, 51617, 51618, 51620, 51624, 51632, 51649, 
    51650, 51652, 51656, 51664, 51680, 51719, 51723, 51725, 51726, 51731, 51733, 51734, 51737, 51738, 51740, 51747, 51749, 51750, 51753, 51754, 
    51756, 51761, 51762, 51764, 51768, 51779, 51781, 51782, 51785, 51786, 51788, 51793, 51794, 51796, 51800, 51809, 51810, 51812, 51816, 51824, 
    51843, 51845, 51846, 51849, 51850, 51852, 51857, 51858, 51860, 51864, 51873, 51874, 51876, 51880, 51888, 51905, 51906, 51908, 51912, 51920, 
    51936, 51971, 51973, 51974, 51977, 51978, 51980, 51985, 51986, 51988, 51992, 52001, 52002, 52004, 52008, 52016, 52033, 52034, 52036, 52040, 
    52048, 52064, 52097, 52098, 52100, 52104, 52112, 52128, 52160, 52231, 52235, 52237, 52238, 52243, 52245, 52246, 52249, 52250, 52252, 52259, 
    52261, 52262, 52265, 52266, 52268, 52273, 52274, 52276, 52280, 52291, 52293, 52294, 52297, 52298, 52300, 52305, 52306, 52308, 52312, 52321, 
    52322, 52324, 52328, 52336, 52355, 52357, 52358, 52361, 52362, 52364, 52369, 52370, 52372, 52376, 52385, 52386, 52388, 52392, 52400, 52417, 
    52418, 52420, 52424, 52432, 52448, 52483, 52485, 52486, 52489, 52490, 52492, 52497, 52498, 52500, 52504, 52513, 52514, 52516, 52520, 52528, 
    52545, 52546, 52548, 52552, 52560, 52576, 52609, 52610, 52612, 52616, 52624, 52640, 52672, 52739, 52741, 52742, 52745, 52746, 52748, 52753, 
    52754, 52756, 52760, 52769, 52770, 52772, 52776, 52784, 52801, 52802, 52804, 52808, 52816, 52832, 52865, 52866, 52868, 52872, 52880, 52896, 
    52928, 52993, 52994, 52996, 53000, 53008, 53024, 53056, 53120, 53263, 53271, 53275, 53277, 53278, 53287, 53291, 53293, 53294, 53299, 53301, 
    53302, 53305, 53306, 53308, 53319, 53323, 53325, 53326, 53331, 53333, 53334, 53337, 53338, 53340, 53347, 53349, 53350, 53353, 53354, 53356, 
    53361, 53362, 53364, 53368, 53383, 53387, 53389, 53390, 53395, 53397, 53398, 53401, 53402, 53404, 53411, 53413, 53414, 53417, 53418, 53420, 
    53425, 53426, 53428, 53432, 53443, 53445, 53446, 53449, 53450, 53452, 53457, 53458, 53460, 53464, 53473, 53474, 53476, 53480, 53488, 53511, 
    53515, 53517, 53518, 53523, 53525, 53526, 53529, 53530, 53532, 53539, 53541, 53542, 53545, 53546, 53548, 53553, 53554, 53556, 53560, 53571, 
    53573, 53574, 53577, 53578, 53580, 53585, 53586, 53588, 53592, 53601, 53602, 53604, 53608, 53616, 53635, 53637, 53638, 53641, 53642, 53644, 
    53649, 53650, 53652, 53656, 53665, 53666, 53668, 53672, 53680, 53697, 53698, 53700, 53704, 53712, 53728, 53767, 53771, 53773, 53774, 53779, 
    53781, 53782, 53785, 53786, 53788, 53795, 53797, 53798, 53801, 53802, 53804, 53809, 53810, 53812, 53816, 53827, 53829, 53830, 53833, 53834, 
    53836, 53841, 53842, 53844, 53848, 53857, 53858, 53860, 53864, 53872, 53891, 53893, 53894, 53897, 53898, 53900, 53905, 53906, 53908, 53912, 
    53921, 53922, 53924, 53928, 53936, 53953, 53954, 53956, 53960, 53968, 53984, 54019, 54021, 54022, 54025, 54026, 54028, 54033, 54034, 54036, 
    54040, 54049, 54050, 54052, 54056, 54064, 54081, 54082, 54084, 54088, 54096, 54112, 54145, 54146, 54148, 54152, 54160, 54176, 54208, 54279, 
    54283, 54285, 54286, 54291, 54293, 54294, 54297, 54298, 54300, 54307, 54309, 54310, 54313, 54314, 54316, 54321, 54322, 54324, 54328, 54339, 
    54341, 54342, 54345, 54346, 54348, 54353, 54354, 54356, 54360, 54369, 54370, 54372, 54376, 54384, 54403, 54405, 54406, 54409, 54410, 54412, 
    54417, 54418, 54420, 54424, 54433, 54434, 54436, 54440, 54448, 54465, 54466, 54468, 54472, 54480, 54496, 54531, 54533, 54534, 54537, 54538, 
    54540, 54545, 54546, 54548, 54552, 54561, 54562, 54564, 54568, 54576, 54593, 54594, 54596, 54600, 54608, 54624, 54657, 54658, 54660, 54664, 
    54672, 54688, 54720, 54787, 54789, 54790, 54793, 54794, 54796, 54801, 54802, 54804, 54808, 54817, 54818, 54820, 54824, 54832, 54849, 54850, 
    54852, 54856, 54864, 54880, 54913, 54914, 54916, 54920, 54928, 54944, 54976, 55041, 55042, 55044, 55048, 55056, 55072, 55104, 55168, 55303, 
    55307, 55309, 55310, 55315, 55317, 55318, 55321, 55322, 55324, 55331, 55333, 55334, 55337, 55338, 55340, 55345, 55346, 55348, 55352, 55363, 
    55365, 55366, 55369, 55370, 55372, 55377, 55378, 55380, 55384, 55393, 55394, 55396, 55400, 55408, 55427, 55429, 55430, 55433, 55434, 55436, 
    55441, 55442, 55444, 55448, 55457, 55458, 55460, 55464, 55472, 55489, 55490, 55492, 55496, 55504, 55520, 55555, 55557, 55558, 55561, 55562, 
    55564, 55569, 55570, 55572, 55576, 55585, 55586, 55588, 55592, 55600, 55617, 55618, 55620, 55624, 55632, 55648, 55681, 55682, 55684, 55688, 
    55696, 55712, 55744, 55811, 55813, 55814, 55817, 55818, 55820, 55825, 55826, 55828, 55832, 55841, 55842, 55844, 55848, 55856, 55873, 55874, 
    55876, 55880, 55888, 55904, 55937, 55938, 55940, 55944, 55952, 55968, 56000, 56065, 56066, 56068, 56072, 56080, 56096, 56128, 56192, 56323, 
    56325, 56326, 56329, 56330, 56332, 56337, 56338, 56340, 56344, 56353, 56354, 56356, 56360, 56368, 56385, 56386, 56388, 56392, 56400, 56416, 
    56449, 56450, 56452, 56456, 56464, 56480, 56512, 56577, 56578, 56580, 56584, 56592, 56608, 56640, 56704, 56833, 56834, 56836, 56840, 56848, 
    56864, 56896, 56960, 57088, 57359, 57367, 57371, 57373, 57374, 57383, 57387, 57389, 57390, 57395, 57397, 57398, 57401, 57402, 57404, 57415, 
    57419, 57421, 57422, 57427, 57429, 57430, 57433, 57434, 57436, 57443, 57445, 57446, 57449, 57450, 57452, 57457, 57458, 57460, 57464, 57479, 
    57483, 57485, 57486, 57491, 57493, 57494, 57497, 57498, 57500, 57507, 57509, 57510, 57513, 57514, 57516, 57521, 57522, 57524, 57528, 57539, 
    57541, 57542, 57545, 57546, 57548, 57553, 57554, 57556, 57560, 57569, 57570, 57572, 57576, 57584, 57607, 57611, 57613, 57614, 57619, 57621, 
    57622, 57625, 57626, 57628, 57635, 57637, 57638, 57641, 57642, 57644, 57649, 57650, 57652, 57656, 57667, 57669, 57670, 57673, 57674, 57676, 
    57681, 57682, 57684, 57688, 57697, 57698, 57700, 57704, 57712, 57731, 57733, 57734, 57737, 57738, 57740, 57745, 57746, 57748, 57752, 57761, 
    57762, 57764, 57768, 57776, 57793, 57794, 57796, 57800, 57808, 57824, 57863, 57867, 57869, 57870, 57875, 57877, 57878, 57881, 57882, 57884, 
    57891, 57893, 57894, 57897, 57898, 57900, 57905, 57906, 57908, 57912, 57923, 57925, 57926, 57929, 57930, 57932, 57937, 57938, 57940, 57944, 
    57953, 57954, 57956, 57960, 57968, 57987, 57989, 57990, 57993, 57994, 57996, 58001, 58002, 58004, 58008, 58017, 58018, 58020, 58024, 58032, 
    58049, 58050, 58052, 58056, 58064, 58080, 58115, 58117, 58118, 58121, 58122, 58124, 58129, 58130, 58132, 58136, 58145, 58146, 58148, 58152, 
    58160, 58177, 58178, 58180, 58184, 58192, 58208, 58241, 58242, 58244, 58248, 58256, 58272, 58304, 58375, 58379, 58381, 58382, 58387, 58389, 
    58390, 58393, 58394, 58396, 58403, 58405, 58406, 58409, 58410, 58412, 58417, 58418, 58420, 58424, 58435, 58437, 58438, 58441, 58442, 58444, 
    58449, 58450, 58452, 58456, 58465, 58466, 58468, 58472, 58480, 58499, 58501, 58502, 58505, 58506, 58508, 58513, 58514, 58516, 58520, 58529, 
    58530, 58532, 58536, 58544, 58561, 58562, 58564, 58568, 58576, 58592, 58627, 58629, 58630, 58633, 58634, 58636, 58641, 58642, 58644, 58648, 
    58657, 58658, 58660, 58664, 58672, 58689, 58690, 58692, 58696, 58704, 58720, 58753, 58754, 58756, 58760, 58768, 58784, 58816, 58883, 58885, 
    58886, 58889, 58890, 58892, 58897, 58898, 58900, 58904, 58913, 58914, 58916, 58920, 58928, 58945, 58946, 58948, 58952, 58960, 58976, 59009, 
    59010, 59012, 59016, 59024, 59040, 59072, 59137, 59138, 59140, 59144, 59152, 59168, 59200, 59264, 59399, 59403, 59405, 59406, 59411, 59413, 
    59414, 59417, 59418, 59420, 59427, 59429, 59430, 59433, 59434, 59436, 59441, 59442, 59444, 59448, 59459, 59461, 59462, 59465, 59466, 59468, 
    59473, 59474, 59476, 59480, 59489, 59490, 59492, 59496, 59504, 59523, 59525, 59526, 59529, 59530, 59532, 59537, 59538, 59540, 59544, 59553, 
    59554, 59556, 59560, 59568, 59585, 59586, 59588, 59592, 59600, 59616, 59651, 59653, 59654, 59657, 59658, 59660, 59665, 59666, 59668, 59672, 
    59681, 59682, 59684, 59688, 59696, 59713, 59714, 59716, 59720, 59728, 59744, 59777, 59778, 59780, 59784, 59792, 59808, 59840, 59907, 59909, 
    59910, 59913, 59914, 59916, 59921, 59922, 59924, 59928, 59937, 59938, 59940, 59944, 59952, 59969, 59970, 59972, 59976, 59984, 60000, 60033, 
    60034, 60036, 60040, 60048, 60064, 60096, 60161, 60162, 60164, 60168, 60176, 60192, 60224, 60288, 60419, 60421, 60422, 60425, 60426, 60428, 
    60433, 60434, 60436, 60440, 60449, 60450, 60452, 60456, 60464, 60481, 60482, 60484, 60488, 60496, 60512, 60545, 60546, 60548, 60552, 60560, 
    60576, 60608, 60673, 60674, 60676, 60680, 60688, 60704, 60736, 60800, 60929, 60930, 60932, 60936, 60944, 60960, 60992, 61056, 61184, 61447, 
    61451, 61453, 61454, 61459, 61461, 61462, 61465, 61466, 61468, 61475, 61477, 61478, 61481, 61482, 61484, 61489, 61490, 61492, 61496, 61507, 
    61509, 61510, 61513, 61514, 61516, 61521, 61522, 61524, 61528, 61537, 61538, 61540, 61544, 61552, 61571, 61573, 61574, 61577, 61578, 61580, 
    61585, 61586, 61588, 61592, 61601, 61602, 61604, 61608, 61616, 61633, 61634, 61636, 61640, 61648, 61664, 61699, 61701, 61702, 61705, 61706, 
    61708, 61713, 61714, 61716, 61720, 61729, 61730, 61732, 61736, 61744, 61761, 61762, 61764, 61768, 61776, 61792, 61825, 61826, 61828, 61832, 
    61840, 61856, 61888, 61955, 61957, 61958, 61961, 61962, 61964, 61969, 61970, 61972, 61976, 61985, 61986, 61988, 61992, 62000, 62017, 62018, 
    62020, 62024, 62032, 62048, 62081, 62082, 62084, 62088, 62096, 62112, 62144, 62209, 62210, 62212, 62216, 62224, 62240, 62272, 62336, 62467, 
    62469, 62470, 62473, 62474, 62476, 62481, 62482, 62484, 62488, 62497, 62498, 62500, 62504, 62512, 62529, 62530, 62532, 62536, 62544, 62560, 
    62593, 62594, 62596, 62600, 62608, 62624, 62656, 62721, 62722, 62724, 62728, 62736, 62752, 62784, 62848, 62977, 62978, 62980, 62984, 62992, 
    63008, 63040, 63104, 63232, 63491, 63493, 63494, 63497, 63498, 63500, 63505, 63506, 63508, 63512, 63521, 63522, 63524, 63528, 63536, 63553, 
    63554, 63556, 63560, 63568, 63584, 63617, 63618, 63620, 63624, 63632, 63648, 63680, 63745, 63746, 63748, 63752, 63760, 63776, 63808, 63872, 
    64001, 64002, 64004, 64008, 64016, 64032, 64064, 64128, 64256, 64513, 64514, 64516, 64520, 64528, 64544, 64576, 64640, 64768, 65024, },
  { 255, 383, 447, 479, 495, 503, 507, 509, 510, 639, 703, 735, 751, 759, 763, 765, 766, 831, 863, 879, 887, 
    891, 893, 894, 927, 943, 951, 955, 957, 958, 975, 983, 987, 989, 990, 999, 1003, 1005, 1006, 1011, 1013, 
    1014, 1017, 1018, 1020, 1151, 1215, 1247, 1263, 1271, 1275, 1277, 1278, 1343, 1375, 1391, 1399, 1403, 1405, 1406, 1439, 
    1455, 1463, 1467, 1469, 1470, 1487, 1495, 1499, 1501, 1502, 1511, 1515, 1517, 1518, 1523, 1525, 1526, 1529, 1530, 1532, 
    1599, 1631, 1647, 1655, 1659, 1661, 1662, 1695, 1711, 1719, 1723, 1725, 1726, 1743, 1751, 1755, 1757, 1758, 1767, 1771, 
    1773, 1774, 1779, 1781, 1782, 1785, 1786, 1788, 1823, 1839, 1847, 1851, 1853, 1854, 1871, 1879, 1883, 1885, 1886, 1895, 
    1899, 1901, 1902, 1907, 1909, 1910, 1913, 1914, 1916, 1935, 1943, 1947, 1949, 1950, 1959, 1963, 1965, 1966, 1971, 1973, 
    1974, 1977, 1978, 1980, 1991, 1995, 1997, 1998, 2003, 2005, 2006, 2009, 2010, 2012, 2019, 2021, 2022, 2025, 2026, 2028, 
    2033, 2034, 2036, 2040, 2175, 2239, 2271, 2287, 2295, 2299, 2301, 2302, 2367, 2399, 2415, 2423, 2427, 2429, 2430, 2463, 
    2479, 2487, 2491, 2493, 2494, 2511, 2519, 2523, 2525, 2526, 2535, 2539, 2541, 2542, 2547, 2549, 2550, 2553, 2554, 2556, 
    2623, 2655, 2671, 2679, 2683, 2685, 2686, 2719, 2735, 2743, 2747, 2749, 2750, 2767, 2775, 2779, 2781, 2782, 2791, 2795, 
    2797, 2798, 2803, 2805, 2806, 2809, 2810, 2812, 2847, 2863, 2871, 2875, 2877, 2878, 2895, 2903, 2907, 2909, 2910, 2919, 
    2923, 2925, 2926, 2931, 2933, 2934, 2937, 2938, 2940, 2959, 2967, 2971, 2973, 2974, 2983, 2987, 2989, 2990, 2995, 2997, 
    2998, 3001, 3002, 3004, 3015, 3019, 3021, 3022, 3027, 3029, 3030, 3033, 3034, 3036, 3043, 3045, 3046, 3049, 3050, 3052, 
    3057, 3058, 3060, 3064, 3135, 3167, 3183, 3191, 3195, 3197, 3198, 3231, 3247, 3255, 3259, 3261, 3262, 3279, 3287, 3291, 
    3293, 3294, 3303, 3307, 3309, 3310, 3315, 3317, 3318, 3321, 3322, 3324, 3359, 3375, 3383, 3387, 3389, 3390, 3407, 3415, 
    3419, 3421, 3422, 3431, 3435, 3437, 3438, 3443, 3445, 3446, 3449, 3450, 3452, 3471, 3479, 3483, 3485, 3486, 3495, 3499, 
    3501, 3502, 3507, 3509, 3510, 3513, 3514, 3516, 3527, 3531, 3533, 3534, 3539, 3541, 3542, 3545, 3546, 3548, 3555, 3557, 
    3558, 3561, 3562, 3564, 3569, 3570, 3572, 3576, 3615, 3631, 3639, 3643, 3645, 3646, 3663, 3671, 3675, 3677, 3678, 3687, 
    3691, 3693, 3694, 3699, 3701, 3702, 3705, 3706, 3708, 3727, 3735, 3739, 3741, 3742, 3751, 3755, 3757, 3758, 3763, 3765, 
    3766, 3769, 3770, 3772, 3783, 3787, 3789, 3790, 3795, 3797, 3798, 3801, 3802, 3804, 3811, 3813, 3814, 3817, 3818, 3820, 
    3825, 3826, 3828, 3832, 3855, 3863, 3867, 3869, 3870, 3879, 3883, 3885, 3886, 3891, 3893, 3894, 3897, 3898, 3900, 3911, 
    3915, 3917, 3918, 3923, 3925, 3926, 3929, 3930, 3932, 3939, 3941, 3942, 3945, 3946, 3948, 3953, 3954, 3956, 3960, 3975, 
    3979, 3981, 3982, 3987, 3989, 3990, 3993, 3994, 3996, 4003, 4005, 4006, 4009, 4010, 4012, 4017, 4018, 4020, 4024, 4035, 
    4037, 4038, 4041, 4042, 4044, 4049, 4050, 4052, 4056, 4065, 4066, 4068, 4072, 4080, 4223, 4287, 4319, 4335, 4343, 4347, 
    4349, 4350, 4415, 4447, 4463, 4471, 4475, 4477, 4478, 4511, 4527, 4535, 4539, 4541, 4542, 4559, 4567, 4571, 4573, 4574, 
    4583, 4587, 4589, 4590, 4595, 4597, 4598, 4601, 4602, 4604, 4671, 4703, 4719, 4727, 4731, 4733, 4734, 4767, 4783, 4791, 
    4795, 4797, 4798, 4815, 4823, 4827, 4829, 4830, 4839, 4843, 4845, 4846, 4851, 4853, 4854, 4857, 4858, 4860, 4895, 4911, 
    4919, 4923, 4925, 4926, 4943, 4951, 4955, 4957, 4958, 4967, 4971, 4973, 4974, 4979, 4981, 4982, 4985, 4986, 4988, 5007, 
    5015, 5019, 5021, 5022, 5031, 5035, 5037, 5038, 5043, 5045, 5046, 5049, 5050, 5052, 5063, 5067, 5069, 5070, 5075, 5077, 
    5078, 5081, 5082, 5084, 5091, 5093, 5094, 5097, 5098, 5100, 5105, 5106, 5108, 5112, 5183, 5215, 5231, 5239, 5243, 5245, 
    5246, 5279, 5295, 5303, 5307, 5309, 5310, 5327, 5335, 5339, 5341, 5342, 5351, 5355, 5357, 5358, 5363, 5365, 5366, 5369, 
    5370, 5372, 5407, 5423, 5431, 5435, 5437, 5438, 5455, 5463, 5467, 5469, 5470, 5479, 5483, 5485, 5486, 5491, 5493, 5494, 
    5497, 5498, 5500, 5519, 5527, 5531, 5533, 5534, 5543, 5547, 5549, 5550, 5555, 5557, 5558, 5561, 5562, 5564, 5575, 5579, 
    5581, 5582, 5587, 5589, 5590, 5593, 5594, 5596, 5603, 5605, 5606, 5609, 5610, 5612, 5617, 5618, 5620, 5624, 5663, 5679, 
    5687, 5691, 5693, 5694, 5711, 5719, 5723, 5725, 5726, 5735, 5739, 5741, 5742, 5747, 5749, 5750, 5753, 5754, 5756, 5775, 
    5783, 5787, 5789, 5790, 5799, 5803, 5805, 5806, 5811, 5813, 5814, 5817, 5818, 5820, 5831, 5835, 5837, 5838, 5843, 5845, 
    5846, 5849, 5850, 5852, 5859, 5861, 5862, 5865, 5866, 5868, 5873, 5874, 5876, 5880, 5903, 5911, 5915, 5917, 5918, 5927, 
    5931, 5933, 5934, 5939, 5941, 5942, 5945, 5946, 5948, 5959, 5963, 5965, 5966, 5971, 5973, 5974, 5977, 5978, 5980, 5987, 
    5989, 5990, 5993, 5994, 5996, 6001, 6002, 6004, 6008, 6023, 6027, 6029, 6030, 6035, 6037, 6038, 6041, 6042, 6044, 6051, 
    6053, 6054, 6057, 6058, 6060, 6065, 6066, 6068, 6072, 6083, 6085, 6086, 6089, 6090, 6092, 6097, 6098, 6100, 6104, 6113, 
    6114, 6116, 6120, 6128, 6207, 6239, 6255, 6263, 6267, 6269, 6270, 6303, 6319, 6327, 6331, 6333, 6334, 6351, 6359, 6363, 
    6365, 6366, 6375, 6379, 6381, 6382, 6387, 6389, 6390, 6393, 6394, 6396, 6431, 6447, 6455, 6459, 6461, 6462, 6479, 6487, 
    6491, 6493, 6494, 6503, 6507, 6509, 6510, 6515, 6517, 6518, 6521, 6522, 6524, 6543, 6551, 6555, 6557, 6558, 6567, 6571, 
    6573, 6574, 6579, 6581, 6582, 6585, 6586, 6588, 6599, 6603, 6605, 6606, 6611, 6613, 6614, 6617, 6618, 6620, 6627, 6629, 
    6630, 6633, 6634, 6636, 6641, 6642, 6644, 6648, 6687, 6703, 6711, 6715, 6717, 6718, 6735, 6743, 6747, 6749, 6750, 6759, 
    6763, 6765, 6766, 6771, 6773, 6774, 6777, 6778, 6780, 6799, 6807, 6811, 6813, 6814, 6823, 6827, 6829, 6830, 6835, 6837, 
    6838, 6841, 6842, 6844, 6855, 6859, 6861, 6862, 6867, 6869, 6870, 6873, 6874, 6876, 6883, 6885, 6886, 6889, 6890, 6892, 
    6897, 6898, 6900, 6904, 6927, 6935, 6939, 6941, 6942, 6951, 6955, 6957, 6958, 6963, 6965, 6966, 6969, 6970, 6972, 6983, 
    6987, 6989, 6990, 6995, 6997, 6998, 7001, 7002, 7004, 7011, 7013, 7014, 7017, 7018, 7020, 7025, 7026, 7028, 7032, 7047, 
    7051, 7053, 7054, 7059, 7061, 7062, 7065, 7066, 7068, 7075, 7077, 7078, 7081, 7082, 7084, 7089, 7090, 7092, 7096, 7107, 
    7109, 7110, 7113, 7114, 7116, 7121, 7122, 7124, 7128, 7137, 7138, 7140, 7144, 7152, 7199, 7215, 7223, 7227, 7229, 7230, 
    7247, 7255, 7259, 7261, 7262, 7271, 7275, 7277, 7278, 7283, 7285, 7286, 7289, 7290, 7292, 7311, 7319, 7323, 7325, 7326, 
    7335, 7339, 7341, 7342, 7347, 7349, 7350, 7353, 7354, 7356, 7367, 7371, 7373, 7374, 7379, 7381, 7382, 7385, 7386, 7388, 
    7395, 7397, 7398, 7401, 7402, 7404, 7409, 7410, 7412, 7416, 7439, 7447, 7451, 7453, 7454, 7463, 7467, 7469, 7470, 7475, 
    7477, 7478, 7481, 7482, 7484, 7495, 7499, 7501, 7502, 7507, 7509, 7510, 7513, 7514, 7516, 7523, 7525, 7526, 7529, 7530, 
    7532, 7537, 7538, 7540, 7544, 7559, 7563, 7565, 7566, 7571, 7573, 7574, 7577, 7578, 7580, 7587, 7589, 7590, 7593, 7594, 
    7596, 7601, 7602, 7604, 7608, 7619, 7621, 7622, 7625, 7626, 7628, 7633, 7634, 7636, 7640, 7649, 7650, 7652, 7656, 7664, 
    7695, 7703, 7707, 7709, 7710, 7719, 7723, 7725, 7726, 7731, 7733, 7734, 7737, 7738, 7740, 7751, 7755, 7757, 7758, 7763, 
    7765, 7766, 7769, 7770, 7772, 7779, 7781, 7782, 7785, 7786, 7788, 7793, 7794, 7796, 7800, 7815, 7819, 7821, 7822, 7827, 
    7829, 7830, 7833, 7834, 7836, 7843, 7845, 7846, 7849, 7850, 7852, 7857, 7858, 7860, 7864, 7875, 7877, 7878, 7881, 7882, 
    7884, 7889, 7890, 7892, 7896, 7905, 7906, 7908, 7912, 7920, 7943, 7947, 7949, 7950, 7955, 7957, 7958, 7961, 7962, 7964, 
    7971, 7973, 7974, 7977, 7978, 7980, 7985, 7986, 7988, 7992, 8003, 8005, 8006, 8009, 8010, 8012, 8017, 8018, 8020, 8024, 
    8033, 8034, 8036, 8040, 8048, 8067, 8069, 8070, 8073, 8074, 8076, 8081, 8082, 8084, 8088, 8097, 8098, 8100, 8104, 8112, 
    8129, 8130, 8132, 8136, 8144, 8160, 8319, 8383, 8415, 8431, 8439, 8443, 8445, 8446, 8511, 8543, 8559, 8567, 8571, 8573, 
    8574, 8607, 8623, 8631, 8635, 8637, 8638, 8655, 8663, 8667, 8669, 8670, 8679, 8683, 8685, 8686, 8691, 8693, 8694, 8697, 
    8698, 8700, 8767, 8799, 8815, 8823, 8827, 8829, 8830, 8863, 8879, 8887, 8891, 8893, 8894, 8911, 8919, 8923, 8925, 8926, 
    8935, 8939, 8941, 8942, 8947, 8949, 8950, 8953, 8954, 8956, 8991, 9007, 9015, 9019, 9021, 9022, 9039, 9047, 9051, 9053, 
    9054, 9063, 9067, 9069, 9070, 9075, 9077, 9078, 9081, 9082, 9084, 9103, 9111, 9115, 9117, 9118, 9127, 9131, 9133, 9134, 
    9139, 9141, 9142, 9145, 9146, 9148, 9159, 9163, 9165, 9166, 9171, 9173, 9174, 9177, 9178, 9180, 9187, 9189, 9190, 9193, 
    9194, 9196, 9201, 9202, 9204, 9208, 9279, 9311, 9327, 9335, 9339, 9341, 9342, 9375, 9391, 9399, 9403, 9405, 9406, 9423, 
    9431, 9435, 9437, 9438, 9447, 9451, 9453, 9454, 9459, 9461, 9462, 9465, 9466, 9468, 9503, 9519, 9527, 9531, 9533, 9534, 
    9551, 9559, 9563, 9565, 9566, 9575, 9579, 9581, 9582, 9587, 9589, 9590, 9593, 9594, 9596, 9615, 9623, 9627, 9629, 9630, 
    9639, 9643, 9645, 9646, 9651, 9653, 9654, 9657, 9658, 9660, 9671, 9675, 9677, 9678, 9683, 9685, 9686, 9689, 9690, 9692, 
    9699, 9701, 9702, 9705, 9706, 9708, 9713, 9714, 9716, 9720, 9759, 9775, 9783, 9787, 9789, 9790, 9807, 9815, 9819, 9821, 
    9822, 9831, 9835, 9837, 9838, 9843, 9845, 9846, 9849, 9850, 9852, 9871, 9879, 9883, 9885, 9886, 9895, 9899, 9901, 9902, 
    9907, 9909, 9910, 9913, 9914, 9916, 9927, 9931, 9933, 9934, 9939, 9941, 9942, 9945, 9946, 9948, 9955, 9957, 9958, 9961, 
    9962, 9964, 9969, 9970, 9972, 9976, 9999, 10007, 10011, 10013, 10014, 10023, 10027, 10029, 10030, 10035, 10037, 10038, 10041, 10042, 
    10044, 10055, 10059, 10061, 10062, 10067, 10069, 10070, 10073, 10074, 10076, 10083, 10085, 10086, 10089, 10090, 10092, 10097, 10098, 10100, 
    10104, 10119, 10123, 10125, 10126, 10131, 10133, 10134, 10137, 10138, 10140, 10147, 10149, 10150, 10153, 10154, 10156, 10161, 10162, 10164, 
    10168, 10179, 10181, 10182, 10185, 10186, 10188, 10193, 10194, 10196, 10200, 10209, 10210, 10212, 10216, 10224, 10303, 10335, 10351, 10359, 
    10363, 10365, 10366, 10399, 10415, 10423, 10427, 10429, 10430, 10447, 10455, 10459, 10461, 10462, 10471, 10475, 10477, 10478, 10483, 10485, 
    10486, 10489, 10490, 10492, 10527, 10543, 10551, 10555, 10557, 10558, 10575, 10583, 10587, 10589, 10590, 10599, 10603, 10605, 10606, 10611, 
    10613, 10614, 10617, 10618, 10620, 10639, 10647, 10651, 10653, 10654, 10663, 10667, 10669, 10670, 10675, 10677, 10678, 10681, 10682, 10684, 
    10695, 10699, 10701, 10702, 10707, 10709, 10710, 10713, 10714, 10716, 10723, 10725, 10726, 10729, 10730, 10732, 10737, 10738, 10740, 10744, 
    10783, 10799, 10807, 10811, 10813, 10814, 10831, 10839, 10843, 10845, 10846, 10855, 10859, 10861, 10862, 10867, 10869, 10870, 10873, 10874, 
    10876, 10895, 10903, 10907, 10909, 10910, 10919, 10923, 10925, 10926, 10931, 10933, 10934, 10937, 10938, 10940, 10951, 10955, 10957, 10958, 
    10963, 10965, 10966, 10969, 10970, 10972, 10979, 10981, 10982, 10985, 10986, 10988, 10993, 10994, 10996, 11000, 11023, 11031, 11035, 11037, 
    11038, 11047, 11051, 11053, 11054, 11059, 11061, 11062, 11065, 11066, 11068, 11079, 11083, 11085, 11086, 11091, 11093, 11094, 11097, 11098, 
    11100, 11107, 11109, 11110, 11113, 11114, 11116, 11121, 11122, 11124, 11128, 11143, 11147, 11149, 11150, 11155, 11157, 11158, 11161, 11162, 
    11164, 11171, 11173, 11174, 11177, 11178, 11180, 11185, 11186, 11188, 11192, 11203, 11205, 11206, 11209, 11210, 11212, 11217, 11218, 11220, 
    11224, 11233, 11234, 11236, 11240, 11248, 11295, 11311, 11319, 11323, 11325, 11326, 11343, 11351, 11355, 11357, 11358, 11367, 11371, 11373, 
    11374, 11379, 11381, 11382, 11385, 11386, 11388, 11407, 11415, 11419, 11421, 11422, 11431, 11435, 11437, 11438, 11443, 11445, 11446, 11449, 
    11450, 11452, 11463, 11467, 11469, 11470, 11475, 11477, 11478, 11481, 11482, 11484, 11491, 11493, 11494, 11497, 11498, 11500, 11505, 11506, 
    11508, 11512, 11535, 11543, 11547, 11549, 11550, 11559, 11563, 11565, 11566, 11571, 11573, 11574, 11577, 11578, 11580, 11591, 11595, 11597, 
    11598, 11603, 11605, 11606, 11609, 11610, 11612, 11619, 11621, 11622, 11625, 11626, 11628, 11633, 11634, 11636, 11640, 11655, 11659, 11661, 
    11662, 11667, 11669, 11670, 11673, 11674, 11676, 11683, 11685, 11686, 11689, 11690, 11692, 11697, 11698, 11700, 11704, 11715, 11717, 11718, 
    11721, 11722, 11724, 11729, 11730, 11732, 11736, 11745, 11746, 11748, 11752, 11760, 11791, 11799, 11803, 11805, 11806, 11815, 11819, 11821, 
    11822, 11827, 11829, 11830, 11833, 11834, 11836, 11847, 11851, 11853, 11854, 11859, 11861, 11862, 11865, 11866, 11868, 11875, 11877, 11878, 
    11881, 11882, 11884, 11889, 11890, 11892, 11896, 11911, 11915, 11917, 11918, 11923, 11925, 11926, 11929, 11930, 11932, 11939, 11941, 11942, 
    11945, 11946, 11948, 11953, 11954, 11956, 11960, 11971, 11973, 11974, 11977, 11978, 11980, 11985, 11986, 11988, 11992, 12001, 12002, 12004, 
    12008, 12016, 12039, 12043, 12045, 12046, 12051, 12053, 12054, 12057, 12058, 12060, 12067, 12069, 12070, 12073, 12074, 12076, 12081, 12082, 
    12084, 12088, 12099, 12101, 12102, 12105, 12106, 12108, 12113, 12114, 12116, 12120, 12129, 12130, 12132, 12136, 12144, 12163, 12165, 12166, 
    12169, 12170, 12172, 12177, 12178, 12180, 12184, 12193, 12194, 12196, 12200, 12208, 12225, 12226, 12228, 12232, 12240, 12256, 12351, 12383, 
    12399, 12407, 12411, 12413, 12414, 12447, 12463, 12471, 12475, 12477, 12478, 12495, 12503, 12507, 12509, 12510, 12519, 12523, 12525, 12526, 
    12531, 12533, 12534, 12537, 12538, 12540, 12575, 12591, 12599, 12603, 12605, 12606, 12623, 12631, 12635, 12637, 12638, 12647, 12651, 12653, 
    12654, 12659, 12661, 12662, 12665, 12666, 12668, 12687, 12695, 12699, 12701, 12702, 12711, 12715, 12717, 12718, 12723, 12725, 12726, 12729, 
    12730, 12732, 12743, 12747, 12749, 12750, 12755, 12757, 12758, 12761, 12762, 12764, 12771, 12773, 12774, 12777, 12778, 12780, 12785, 12786, 
    12788, 12792, 12831, 12847, 12855, 12859, 12861, 12862, 12879, 12887, 12891, 12893, 12894, 12903, 12907, 12909, 12910, 12915, 12917, 12918, 
    12921, 12922, 12924, 12943, 12951, 12955, 12957, 12958, 12967, 12971, 12973, 12974, 12979, 12981, 12982, 12985, 12986, 12988, 12999, 13003, 
    13005, 13006, 13011, 13013, 13014, 13017, 13018, 13020, 13027, 13029, 13030, 13033, 13034, 13036, 13041, 13042, 13044, 13048, 13071, 13079, 
    13083, 13085, 13086, 13095, 13099, 13101, 13102, 13107, 13109, 13110, 13113, 13114, 13116, 13127, 13131, 13133, 13134, 13139, 13141, 13142, 
    13145, 13146, 13148, 13155, 13157, 13158, 13161, 13162, 13164, 13169, 13170, 13172, 13176, 13191, 13195, 13197, 13198, 13203, 13205, 13206, 
    13209, 13210, 13212, 13219, 13221, 13222, 13225, 13226, 13228, 13233, 13234, 13236, 13240, 13251, 13253, 13254, 13257, 13258, 13260, 13265, 
    13266, 13268, 13272, 13281, 13282, 13284, 13288, 13296, 13343, 13359, 13367, 13371, 13373, 13374, 13391, 13399, 13403, 13405, 13406, 13415, 
    13419, 13421, 13422, 13427, 13429, 13430, 13433, 13434, 13436, 13455, 13463, 13467, 13469, 13470, 13479, 13483, 13485, 13486, 13491, 13493, 
    13494, 13497, 13498, 13500, 13511, 13515, 13517, 13518, 13523, 13525, 13526, 13529, 13530, 13532, 13539, 13541, 13542, 13545, 13546, 13548, 
    13553, 13554, 13556, 13560, 13583, 13591, 13595, 13597, 13598, 13607, 13611, 13613, 13614, 13619, 13621, 13622, 13625, 13626, 13628, 13639, 
    13643, 13645, 13646, 13651, 13653, 13654, 13657, 13658, 13660, 13667, 13669, 13670, 13673, 13674, 13676, 13681, 13682, 13684, 13688, 13703, 
    13707, 13709, 13710, 13715, 13717, 13718, 13721, 13722, 13724, 13731, 13733, 13734, 13737, 13738, 13740, 13745, 13746, 13748, 13752, 13763, 
    13765, 13766, 13769, 13770, 13772, 13777, 13778, 13780, 13784, 13793, 13794, 13796, 13800, 13808, 13839, 13847, 13851, 13853, 13854, 13863, 
    13867, 13869, 13870, 13875, 13877, 13878, 13881, 13882, 13884, 13895, 13899, 13901, 13902, 13907, 13909, 13910, 13913, 13914, 13916, 13923, 
    13925, 13926, 13929, 13930, 13932, 13937, 13938, 13940, 13944, 13959, 13963, 13965, 13966, 13971, 13973, 13974, 13977, 13978, 13980, 13987, 
    13989, 13990, 13993, 13994, 13996, 14001, 14002, 14004, 14008, 14019, 14021, 14022, 14025, 14026, 14028, 14033, 14034, 14036, 14040, 14049, 
    14050, 14052, 14056, 14064, 14087, 14091, 14093, 14094, 14099, 14101, 14102, 14105, 14106, 14108, 14115, 14117, 14118, 14121, 14122, 14124, 
    14129, 14130, 14132, 14136, 14147, 14149, 14150, 14153, 14154, 14156, 14161, 14162, 14164, 14168, 14177, 14178, 14180, 14184, 14192, 14211, 
    14213, 14214, 14217, 14218, 14220, 14225, 14226, 14228, 14232, 14241, 14242, 14244, 14248, 14256, 14273, 14274, 14276, 14280, 14288, 14304, 
    14367, 14383, 14391, 14395, 14397, 14398, 14415, 14423, 14427, 14429, 14430, 14439, 14443, 14445, 14446, 14451, 14453, 14454, 14457, 14458, 
    14460, 14479, 14487, 14491, 14493, 14494, 14503, 14507, 14509, 14510, 14515, 14517, 14518, 14521, 14522, 14524, 14535, 14539, 14541, 14542, 
    14547, 14549, 14550, 14553, 14554, 14556, 14563, 14565, 14566, 14569, 14570, 14572, 14577, 14578, 14580, 14584, 14607, 14615, 14619, 14621, 
    14622, 14631, 14635, 14637, 14638, 14643, 14645, 14646, 14649, 14650, 14652, 14663, 14667, 14669, 14670, 14675, 14677, 14678, 14681, 14682, 
    14684, 14691, 14693, 14694, 14697, 14698, 14700, 14705, 14706, 14708, 14712, 14727, 14731, 14733, 14734, 14739, 14741, 14742, 14745, 14746, 
    14748, 14755, 14757, 14758, 14761, 14762, 14764, 14769, 14770, 14772, 14776, 14787, 14789, 14790, 14793, 14794, 14796, 14801, 14802, 14804, 
    14808, 14817, 14818, 14820, 14824, 14832, 14863, 14871, 14875, 14877, 14878, 14887, 14891, 14893, 14894, 14899, 14901, 14902, 14905, 14906, 
    14908, 14919, 14923, 14925, 14926, 14931, 14933, 14934, 14937, 14938, 14940, 14947, 14949, 14950, 14953, 14954, 14956, 14961, 14962, 14964, 
    14968, 14983, 14987, 14989, 14990, 14995, 14997, 14998, 15001, 15002, 15004, 15011, 15013, 15014, 15017, 15018, 15020, 15025, 15026, 15028, 
    15032, 15043, 15045, 15046, 15049, 15050, 15052, 15057, 15058, 15060, 15064, 15073, 15074, 15076, 15080, 15088, 15111, 15115, 15117, 15118, 
    15123, 15125, 15126, 15129, 15130, 15132, 15139, 15141, 15142, 15145, 15146, 15148, 15153, 15154, 15156, 15160, 15171, 15173, 15174, 15177, 
    15178, 15180, 15185, 15186, 15188, 15192, 15201, 15202, 15204, 15208, 15216, 15235, 15237, 15238, 15241, 15242, 15244, 15249, 15250, 15252, 
    15256, 15265, 15266, 15268, 15272, 15280, 15297, 15298, 15300, 15304, 15312, 15328, 15375, 15383, 15387, 15389, 15390, 15399, 15403, 15405, 
    15406, 15411, 15413, 15414, 15417, 15418, 15420, 15431, 15435, 15437, 15438, 15443, 15445, 15446, 15449, 15450, 15452, 15459, 15461, 15462, 
    15465, 15466, 15468, 15473, 15474, 15476, 15480, 15495, 15499, 15501, 15502, 15507, 15509, 15510, 15513, 15514, 15516, 15523, 15525, 15526, 
    15529, 15530, 15532, 15537, 15538, 15540, 15544, 15555, 15557, 15558, 15561, 15562, 15564, 15569, 15570, 15572, 15576, 15585, 15586, 15588, 
    15592, 15600, 15623, 15627, 15629, 15630, 15635, 15637, 15638, 15641, 15642, 15644, 15651, 15653, 15654, 15657, 15658, 15660, 15665, 15666, 
    15668, 15672, 15683, 15685, 15686, 15689, 15690, 15692, 15697, 15698, 15700, 15704, 15713, 15714, 15716, 15720, 15728, 15747, 15749, 15750, 
    15753, 15754, 15756, 15761, 15762, 15764, 15768, 15777, 15778, 15780, 15784, 15792, 15809, 15810, 15812, 15816, 15824, 15840, 15879, 15883, 
    15885, 15886, 15891, 15893, 15894, 15897, 15898, 15900, 15907, 15909, 15910, 15913, 15914, 15916, 15921, 15922, 15924, 15928, 15939, 15941, 
    15942, 15945, 15946, 15948, 15953, 15954, 15956, 15960, 15969, 15970, 15972, 15976, 15984, 16003, 16005, 16006, 16009, 16010, 16012, 16017, 
    16018, 16020, 16024, 16033, 16034, 16036, 16040, 16048, 16065, 16066, 16068, 16072, 16080, 16096, 16131, 16133, 16134, 16137, 16138, 16140, 
    16145, 16146, 16148, 16152, 16161, 16162, 16164, 16168, 16176, 16193, 16194, 16196, 16200, 16208, 16224, 16257, 16258, 16260, 16264, 16272, 
    16288, 16320, 16511, 16575, 16607, 16623, 16631, 16635, 16637, 16638, 16703, 16735, 16751, 16759, 16763, 16765, 16766, 16799, 16815, 16823, 
    16827, 16829, 16830, 16847, 16855, 16859, 16861, 16862, 16871, 16875, 16877, 16878, 16883, 16885, 16886, 16889, 16890, 16892, 16959, 16991, 
    17007, 17015, 17019, 17021, 17022, 17055, 17071, 17079, 17083, 17085, 17086, 17103, 17111, 17115, 17117, 17118, 17127, 17131, 17133, 17134, 
    17139, 17141, 17142, 17145, 17146, 17148, 17183, 17199, 17207, 17211, 17213, 17214, 17231, 17239, 17243, 17245, 17246, 17255, 17259, 17261, 
    17262, 17267, 17269, 17270, 17273, 17274, 17276, 17295, 17303, 17307, 17309, 17310, 17319, 17323, 17325, 17326, 17331, 17333, 17334, 17337, 
    17338, 17340, 17351, 17355, 17357, 17358, 17363, 17365, 17366, 17369, 17370, 17372, 17379, 17381, 17382, 17385, 17386, 17388, 17393, 17394, 
    17396, 17400, 17471, 17503, 17519, 17527, 17531, 17533, 17534, 17567, 17583, 17591, 17595, 17597, 17598, 17615, 17623, 17627, 17629, 17630, 
    17639, 17643, 17645, 17646, 17651, 17653, 17654, 17657, 17658, 17660, 17695, 17711, 17719, 17723, 17725, 17726, 17743, 17751, 17755, 17757, 
    17758, 17767, 17771, 17773, 17774, 17779, 17781, 17782, 17785, 17786, 17788, 17807, 17815, 17819, 17821, 17822, 17831, 17835, 17837, 17838, 
    17843, 17845, 17846, 17849, 17850, 17852, 17863, 17867, 17869, 17870, 17875, 17877, 17878, 17881, 17882, 17884, 17891, 17893, 17894, 17897, 
    17898, 17900, 17905, 17906, 17908, 17912, 17951, 17967, 17975, 17979, 17981, 17982, 17999, 18007, 18011, 18013, 18014, 18023, 18027, 18029, 
    18030, 18035, 18037, 18038, 18041, 18042, 18044, 18063, 18071, 18075, 18077, 18078, 18087, 18091, 18093, 18094, 18099, 18101, 18102, 18105, 
    18106, 18108, 18119, 18123, 18125, 18126, 18131, 18133, 18134, 18137, 18138, 18140, 18147, 18149, 18150, 18153, 18154, 18156, 18161, 18162, 
    18164, 18168, 18191, 18199, 18203, 18205, 18206, 18215, 18219, 18221, 18222, 18227, 18229, 18230, 18233, 18234, 18236, 18247, 18251, 18253, 
    18254, 18259, 18261, 18262, 18265, 18266, 18268, 18275, 18277, 18278, 18281, 18282, 18284, 18289, 18290, 18292, 18296, 18311, 18315, 18317, 
    18318, 18323, 18325, 18326, 18329, 18330, 18332, 18339, 18341, 18342, 18345, 18346, 18348, 18353, 18354, 18356, 18360, 18371, 18373, 18374, 
    18377, 18378, 18380, 18385, 18386, 18388, 18392, 18401, 18402, 18404, 18408, 18416, 18495, 18527, 18543, 18551, 18555, 18557, 18558, 18591, 
    18607, 18615, 18619, 18621, 18622, 18639, 18647, 18651, 18653, 18654, 18663, 18667, 18669, 18670, 18675, 18677, 18678, 18681, 18682, 18684, 
    18719, 18735, 18743, 18747, 18749, 18750, 18767, 18775, 18779, 18781, 18782, 18791, 18795, 18797, 18798, 18803, 18805, 18806, 18809, 18810, 
    18812, 18831, 18839, 18843, 18845, 18846, 18855, 18859, 18861, 18862, 18867, 18869, 18870, 18873, 18874, 18876, 18887, 18891, 18893, 18894, 
    18899, 18901, 18902, 18905, 18906, 18908, 18915, 18917, 18918, 18921, 18922, 18924, 18929, 18930, 18932, 18936, 18975, 18991, 18999, 19003, 
    19005, 19006, 19023, 19031, 19035, 19037, 19038, 19047, 19051, 19053, 19054, 19059, 19061, 19062, 19065, 19066, 19068, 19087, 19095, 19099, 
    19101, 19102, 19111, 19115, 19117, 19118, 19123, 19125, 19126, 19129, 19130, 19132, 19143, 19147, 19149, 19150, 19155, 19157, 19158, 19161, 
    19162, 19164, 19171, 19173, 19174, 19177, 19178, 19180, 19185, 19186, 19188, 19192, 19215, 19223, 19227, 19229, 19230, 19239, 19243, 19245, 
    19246, 19251, 19253, 19254, 19257, 19258, 19260, 19271, 19275, 19277, 19278, 19283, 19285, 19286, 19289, 19290, 19292, 19299, 19301, 19302, 
    19305, 19306, 19308, 19313, 19314, 19316, 19320, 19335, 19339, 19341, 19342, 19347, 19349, 19350, 19353, 19354, 19356, 19363, 19365, 19366, 
    19369, 19370, 19372, 19377, 19378, 19380, 19384, 19395, 19397, 19398, 19401, 19402, 19404, 19409, 19410, 19412, 19416, 19425, 19426, 19428, 
    19432, 19440, 19487, 19503, 19511, 19515, 19517, 19518, 19535, 19543, 19547, 19549, 19550, 19559, 19563, 19565, 19566, 19571, 19573, 19574, 
    19577, 19578, 19580, 19599, 19607, 19611, 19613, 19614, 19623, 19627, 19629, 19630, 19635, 19637, 19638, 19641, 19642, 19644, 19655, 19659, 
    19661, 19662, 19667, 19669, 19670, 19673, 19674, 19676, 19683, 19685, 19686, 19689, 19690, 19692, 19697, 19698, 19700, 19704, 19727, 19735, 
    19739, 19741, 19742, 19751, 19755, 19757, 19758, 19763, 19765, 19766, 19769, 19770, 19772, 19783, 19787, 19789, 19790, 19795, 19797, 19798, 
    19801, 19802, 19804, 19811, 19813, 19814, 19817, 19818, 19820, 19825, 19826, 19828, 19832, 19847, 19851, 19853, 19854, 19859, 19861, 19862, 
    19865, 19866, 19868, 19875, 19877, 19878, 19881, 19882, 19884, 19889, 19890, 19892, 19896, 19907, 19909, 19910, 19913, 19914, 19916, 19921, 
    19922, 19924, 19928, 19937, 19938, 19940, 19944, 19952, 19983, 19991, 19995, 19997, 19998, 20007, 20011, 20013, 20014, 20019, 20021, 20022, 
    20025, 20026, 20028, 20039, 20043, 20045, 20046, 20051, 20053, 20054, 20057, 20058, 20060, 20067, 20069, 20070, 20073, 20074, 20076, 20081, 
    20082, 20084, 20088, 20103, 20107, 20109, 20110, 20115, 20117, 20118, 20121, 20122, 20124, 20131, 20133, 20134, 20137, 20138, 20140, 20145, 
    20146, 20148, 20152, 20163, 20165, 20166, 20169, 20170, 20172, 20177, 20178, 20180, 20184, 20193, 20194, 20196, 20200, 20208, 20231, 20235, 
    20237, 20238, 20243, 20245, 20246, 20249, 20250, 20252, 20259, 20261, 20262, 20265, 20266, 20268, 20273, 20274, 20276, 20280, 20291, 20293, 
    20294, 20297, 20298, 20300, 20305, 20306, 20308, 20312, 20321, 20322, 20324, 20328, 20336, 20355, 20357, 20358, 20361, 20362, 20364, 20369, 
    20370, 20372, 20376, 20385, 20386, 20388, 20392, 20400, 20417, 20418, 20420, 20424, 20432, 20448, 20543, 20575, 20591, 20599, 20603, 20605, 
    20606, 20639, 20655, 20663, 20667, 20669, 20670, 20687, 20695, 20699, 20701, 20702, 20711, 20715, 20717, 20718, 20723, 20725, 20726, 20729, 
    20730, 20732, 20767, 20783, 20791, 20795, 20797, 20798, 20815, 20823, 20827, 20829, 20830, 20839, 20843, 20845, 20846, 20851, 20853, 20854, 
    20857, 20858, 20860, 20879, 20887, 20891, 20893, 20894, 20903, 20907, 20909, 20910, 20915, 20917, 20918, 20921, 20922, 20924, 20935, 20939, 
    20941, 20942, 20947, 20949, 20950, 20953, 20954, 20956, 20963, 20965, 20966, 20969, 20970, 20972, 20977, 20978, 20980, 20984, 21023, 21039, 
    21047, 21051, 21053, 21054, 21071, 21079, 21083, 21085, 21086, 21095, 21099, 21101, 21102, 21107, 21109, 21110, 21113, 21114, 21116, 21135, 
    21143, 21147, 21149, 21150, 21159, 21163, 21165, 21166, 21171, 21173, 21174, 21177, 21178, 21180, 21191, 21195, 21197, 21198, 21203, 21205, 
    21206, 21209, 21210, 21212, 21219, 21221, 21222, 21225, 21226, 21228, 21233, 21234, 21236, 21240, 21263, 21271, 21275, 21277, 21278, 21287, 
    21291, 21293, 21294, 21299, 21301, 21302, 21305, 21306, 21308, 21319, 21323, 21325, 21326, 21331, 21333, 21334, 21337, 21338, 21340, 21347, 
    21349, 21350, 21353, 21354, 21356, 21361, 21362, 21364, 21368, 21383, 21387, 21389, 21390, 21395, 21397, 21398, 21401, 21402, 21404, 21411, 
    21413, 21414, 21417, 21418, 21420, 21425, 21426, 21428, 21432, 21443, 21445, 21446, 21449, 21450, 21452, 21457, 21458, 21460, 21464, 21473, 
    21474, 21476, 21480, 21488, 21535, 21551, 21559, 21563, 21565, 21566, 21583, 21591, 21595, 21597, 21598, 21607, 21611, 21613, 21614, 21619, 
    21621, 21622, 21625, 21626, 21628, 21647, 21655, 21659, 21661, 21662, 21671, 21675, 21677, 21678, 21683, 21685, 21686, 21689, 21690, 21692, 
    21703, 21707, 21709, 21710, 21715, 21717, 21718, 21721, 21722, 21724, 21731, 21733, 21734, 21737, 21738, 21740, 21745, 21746, 21748, 21752, 
    21775, 21783, 21787, 21789, 21790, 21799, 21803, 21805, 21806, 21811, 21813, 21814, 21817, 21818, 21820, 21831, 21835, 21837, 21838, 21843, 
    21845, 21846, 21849, 21850, 21852, 21859, 21861, 21862, 21865, 21866, 21868, 21873, 21874, 21876, 21880, 21895, 21899, 21901, 21902, 21907, 
    21909, 21910, 21913, 21914, 21916, 21923, 21925, 21926, 21929, 21930, 21932, 21937, 21938, 21940, 21944, 21955, 21957, 21958, 21961, 21962, 
    21964, 21969, 21970, 21972, 21976, 21985, 21986, 21988, 21992, 22000, 22031, 22039, 22043, 22045, 22046, 22055, 22059, 22061, 22062, 22067, 
    22069, 22070, 22073, 22074, 22076, 22087, 22091, 22093, 22094, 22099, 22101, 22102, 22105, 22106, 22108, 22115, 22117, 22118, 22121, 22122, 
    22124, 22129, 22130, 22132, 22136, 22151, 22155, 22157, 22158, 22163, 22165, 22166, 22169, 22170, 22172, 22179, 22181, 22182, 22185, 22186, 
    22188, 22193, 22194, 22196, 22200, 22211, 22213, 22214, 22217, 22218, 22220, 22225, 22226, 22228, 22232, 22241, 22242, 22244, 22248, 22256, 
    22279, 22283, 22285, 22286, 22291, 22293, 22294, 22297, 22298, 22300, 22307, 22309, 22310, 22313, 22314, 22316, 22321, 22322, 22324, 22328, 
    22339, 22341, 22342, 22345, 22346, 22348, 22353, 22354, 22356, 22360, 22369, 22370, 22372, 22376, 22384, 22403, 22405, 22406, 22409, 22410, 
    22412, 22417, 22418, 22420, 22424, 22433, 22434, 22436, 22440, 22448, 22465, 22466, 22468, 22472, 22480, 22496, 22559, 22575, 22583, 22587, 
    22589, 22590, 22607, 22615, 22619, 22621, 22622, 22631, 22635, 22637, 22638, 22643, 22645, 22646, 22649, 22650, 22652, 22671, 22679, 22683, 
    22685, 22686, 22695, 22699, 22701, 22702, 22707, 22709, 22710, 22713, 22714, 22716, 22727, 22731, 22733, 22734, 22739, 22741, 22742, 22745, 
    22746, 22748, 22755, 22757, 22758, 22761, 22762, 22764, 22769, 22770, 22772, 22776, 22799, 22807, 22811, 22813, 22814, 22823, 22827, 22829, 
    22830, 22835, 22837, 22838, 22841, 22842, 22844, 22855, 22859, 22861, 22862, 22867, 22869, 22870, 22873, 22874, 22876, 22883, 22885, 22886, 
    22889, 22890, 22892, 22897, 22898, 22900, 22904, 22919, 22923, 22925, 22926, 22931, 22933, 22934, 22937, 22938, 22940, 22947, 22949, 22950, 
    22953, 22954, 22956, 22961, 22962, 22964, 22968, 22979, 22981, 22982, 22985, 22986, 22988, 22993, 22994, 22996, 23000, 23009, 23010, 23012, 
    23016, 23024, 23055, 23063, 23067, 23069, 23070, 23079, 23083, 23085, 23086, 23091, 23093, 23094, 23097, 23098, 23100, 23111, 23115, 23117, 
    23118, 23123, 23125, 23126, 23129, 23130, 23132, 23139, 23141, 23142, 23145, 23146, 23148, 23153, 23154, 23156, 23160, 23175, 23179, 23181, 
    23182, 23187, 23189, 23190, 23193, 23194, 23196, 23203, 23205, 23206, 23209, 23210, 23212, 23217, 23218, 23220, 23224, 23235, 23237, 23238, 
    23241, 23242, 23244, 23249, 23250, 23252, 23256, 23265, 23266, 23268, 23272, 23280, 23303, 23307, 23309, 23310, 23315, 23317, 23318, 23321, 
    23322, 23324, 23331, 23333, 23334, 23337, 23338, 23340, 23345, 23346, 23348, 23352, 23363, 23365, 23366, 23369, 23370, 23372, 23377, 23378, 
    23380, 23384, 23393, 23394, 23396, 23400, 23408, 23427, 23429, 23430, 23433, 23434, 23436, 23441, 23442, 23444, 23448, 23457, 23458, 23460, 
    23464, 23472, 23489, 23490, 23492, 23496, 23504, 23520, 23567, 23575, 23579, 23581, 23582, 23591, 23595, 23597, 23598, 23603, 23605, 23606, 
    23609, 23610, 23612, 23623, 23627, 23629, 23630, 23635, 23637, 23638, 23641, 23642, 23644, 23651, 23653, 23654, 23657, 23658, 23660, 23665, 
    23666, 23668, 23672, 23687, 23691, 23693, 23694, 23699, 23701, 23702, 23705, 23706, 23708, 23715, 23717, 23718, 23721, 23722, 23724, 23729, 
    23730, 23732, 23736, 23747, 23749, 23750, 23753, 23754, 23756, 23761, 23762, 23764, 23768, 23777, 23778, 23780, 23784, 23792, 23815, 23819, 
    23821, 23822, 23827, 23829, 23830, 23833, 23834, 23836, 23843, 23845, 23846, 23849, 23850, 23852, 23857, 23858, 23860, 23864, 23875, 23877, 
    23878, 23881, 23882, 23884, 23889, 23890, 23892, 23896, 23905, 23906, 23908, 23912, 23920, 23939, 23941, 23942, 23945, 23946, 23948, 23953, 
    23954, 23956, 23960, 23969, 23970, 23972, 23976, 23984, 24001, 24002, 24004, 24008, 24016, 24032, 24071, 24075, 24077, 24078, 24083, 24085, 
    24086, 24089, 24090, 24092, 24099, 24101, 24102, 24105, 24106, 24108, 24113, 24114, 24116, 24120, 24131, 24133, 24134, 24137, 24138, 24140, 
    24145, 24146, 24148, 24152, 24161, 24162, 24164, 24168, 24176, 24195, 24197, 24198, 24201, 24202, 24204, 24209, 24210, 24212, 24216, 24225, 
    24226, 24228, 24232, 24240, 24257, 24258, 24260, 24264, 24272, 24288, 24323, 24325, 24326, 24329, 24330, 24332, 24337, 24338, 24340, 24344, 
    24353, 24354, 24356, 24360, 24368, 24385, 24386, 24388, 24392, 24400, 24416, 24449, 24450, 24452, 24456, 24464, 24480, 24512, 24639, 24671, 
    24687, 24695, 24699, 24701, 24702, 24735, 24751, 24759, 24763, 24765, 24766, 24783, 24791, 24795, 24797, 24798, 24807, 24811, 24813, 24814, 
    24819, 24821, 24822, 24825, 24826, 24828, 24863, 24879, 24887, 24891, 24893, 24894, 24911, 24919, 24923, 24925, 24926, 24935, 24939, 24941, 
    24942, 24947, 24949, 24950, 24953, 24954, 24956, 24975, 24983, 24987, 24989, 24990, 24999, 25003, 25005, 25006, 25011, 25013, 25014, 25017, 
    25018, 25020, 25031, 25035, 25037, 25038, 25043, 25045, 25046, 25049, 25050, 25052, 25059, 25061, 25062, 25065, 25066, 25068, 25073, 25074, 
    25076, 25080, 25119, 25135, 25143, 25147, 25149, 25150, 25167, 25175, 25179, 25181, 25182, 25191, 25195, 25197, 25198, 25203, 25205, 25206, 
    25209, 25210, 25212, 25231, 25239, 25243, 25245, 25246, 25255, 25259, 25261, 25262, 25267, 25269, 25270, 25273, 25274, 25276, 25287, 25291, 
    25293, 25294, 25299, 25301, 25302, 25305, 25306, 25308, 25315, 25317, 25318, 25321, 25322, 25324, 25329, 25330, 25332, 25336, 25359, 25367, 
    25371, 25373, 25374, 25383, 25387, 25389, 25390, 25395, 25397, 25398, 25401, 25402, 25404, 25415, 25419, 25421, 25422, 25427, 25429, 25430, 
    25433, 25434, 25436, 25443, 25445, 25446, 25449, 25450, 25452, 25457, 25458, 25460, 25464, 25479, 25483, 25485, 25486, 25491, 25493, 25494, 
    25497, 25498, 25500, 25507, 25509, 25510, 25513, 25514, 25516, 25521, 25522, 25524, 25528, 25539, 25541, 25542, 25545, 25546, 25548, 25553, 
    25554, 25556, 25560, 25569, 25570, 25572, 25576, 25584, 25631, 25647, 25655, 25659, 25661, 25662, 25679, 25687, 25691, 25693, 25694, 25703, 
    25707, 25709, 25710, 25715, 25717, 25718, 25721, 25722, 25724, 25743, 25751, 25755, 25757, 25758, 25767, 25771, 25773, 25774, 25779, 25781, 
    25782, 25785, 25786, 25788, 25799, 25803, 25805, 25806, 25811, 25813, 25814, 25817, 25818, 25820, 25827, 25829, 25830, 25833, 25834, 25836, 
    25841, 25842, 25844, 25848, 25871, 25879, 25883, 25885, 25886, 25895, 25899, 25901, 25902, 25907, 25909, 25910, 25913, 25914, 25916, 25927, 
    25931, 25933, 25934, 25939, 25941, 25942, 25945, 25946, 25948, 25955, 25957, 25958, 25961, 25962, 25964, 25969, 25970, 25972, 25976, 25991, 
    25995, 25997, 25998, 26003, 26005, 26006, 26009, 26010, 26012, 26019, 26021, 26022, 26025, 26026, 26028, 26033, 26034, 26036, 26040, 26051, 
    26053, 26054, 26057, 26058, 26060, 26065, 26066, 26068, 26072, 26081, 26082, 26084, 26088, 26096, 26127, 26135, 26139, 26141, 26142, 26151, 
    26155, 26157, 26158, 26163, 26165, 26166, 26169, 26170, 26172, 26183, 26187, 26189, 26190, 26195, 26197, 26198, 26201, 26202, 26204, 26211, 
    26213, 26214, 26217, 26218, 26220, 26225, 26226, 26228, 26232, 26247, 26251, 26253, 26254, 26259, 26261, 26262, 26265, 26266, 26268, 26275, 
    26277, 26278, 26281, 26282, 26284, 26289, 26290, 26292, 26296, 26307, 26309, 26310, 26313, 26314, 26316, 26321, 26322, 26324, 26328, 26337, 
    26338, 26340, 26344, 26352, 26375, 26379, 26381, 26382, 26387, 26389, 26390, 26393, 26394, 26396, 26403, 26405, 26406, 26409, 26410, 26412, 
    26417, 26418, 26420, 26424, 26435, 26437, 26438, 26441, 26442, 26444, 26449, 26450, 26452, 26456, 26465, 26466, 26468, 26472, 26480, 26499, 
    26501, 26502, 26505, 26506, 26508, 26513, 26514, 26516, 26520, 26529, 26530, 26532, 26536, 26544, 26561, 26562, 26564, 26568, 26576, 26592, 
    26655, 26671, 26679, 26683, 26685, 26686, 26703, 26711, 26715, 26717, 26718, 26727, 26731, 26733, 26734, 26739, 26741, 26742, 26745, 26746, 
    26748, 26767, 26775, 26779, 26781, 26782, 26791, 26795, 26797, 26798, 26803, 26805, 26806, 26809, 26810, 26812, 26823, 26827, 26829, 26830, 
    26835, 26837, 26838, 26841, 26842, 26844, 26851, 26853, 26854, 26857, 26858, 26860, 26865, 26866, 26868, 26872, 26895, 26903, 26907, 26909, 
    26910, 26919, 26923, 26925, 26926, 26931, 26933, 26934, 26937, 26938, 26940, 26951, 26955, 26957, 26958, 26963, 26965, 26966, 26969, 26970, 
    26972, 26979, 26981, 26982, 26985, 26986, 26988, 26993, 26994, 26996, 27000, 27015, 27019, 27021, 27022, 27027, 27029, 27030, 27033, 27034, 
    27036, 27043, 27045, 27046, 27049, 27050, 27052, 27057, 27058, 27060, 27064, 27075, 27077, 27078, 27081, 27082, 27084, 27089, 27090, 27092, 
    27096, 27105, 27106, 27108, 27112, 27120, 27151, 27159, 27163, 27165, 27166, 27175, 27179, 27181, 27182, 27187, 27189, 27190, 27193, 27194, 
    27196, 27207, 27211, 27213, 27214, 27219, 27221, 27222, 27225, 27226, 27228, 27235, 27237, 27238, 27241, 27242, 27244, 27249, 27250, 27252, 
    27256, 27271, 27275, 27277, 27278, 27283, 27285, 27286, 27289, 27290, 27292, 27299, 27301, 27302, 27305, 27306, 27308, 27313, 27314, 27316, 
    27320, 27331, 27333, 27334, 27337, 27338, 27340, 27345, 27346, 27348, 27352, 27361, 27362, 27364, 27368, 27376, 27399, 27403, 27405, 27406, 
    27411, 27413, 27414, 27417, 27418, 27420, 27427, 27429, 27430, 27433, 27434, 27436, 27441, 27442, 27444, 27448, 27459, 27461, 27462, 27465, 
    27466, 27468, 27473, 27474, 27476, 27480, 27489, 27490, 27492, 27496, 27504, 27523, 27525, 27526, 27529, 27530, 27532, 27537, 27538, 27540, 
    27544, 27553, 27554, 27556, 27560, 27568, 27585, 27586, 27588, 27592, 27600, 27616, 27663, 27671, 27675, 27677, 27678, 27687, 27691, 27693, 
    27694, 27699, 27701, 27702, 27705, 27706, 27708, 27719, 27723, 27725, 27726, 27731, 27733, 27734, 27737, 27738, 27740, 27747, 27749, 27750, 
    27753, 27754, 27756, 27761, 27762, 27764, 27768, 27783, 27787, 27789, 27790, 27795, 27797, 27798, 27801, 27802, 27804, 27811, 27813, 27814, 
    27817, 27818, 27820, 27825, 27826, 27828, 27832, 27843, 27845, 27846, 27849, 27850, 27852, 27857, 27858, 27860, 27864, 27873, 27874, 27876, 
    27880, 27888, 27911, 27915, 27917, 27918, 27923, 27925, 27926, 27929, 27930, 27932, 27939, 27941, 27942, 27945, 27946, 27948, 27953, 27954, 
    27956, 27960, 27971, 27973, 27974, 27977, 27978, 27980, 27985, 27986, 27988, 27992, 28001, 28002, 28004, 28008, 28016, 28035, 28037, 28038, 
    28041, 28042, 28044, 28049, 28050, 28052, 28056, 28065, 28066, 28068, 28072, 28080, 28097, 28098, 28100, 28104, 28112, 28128, 28167, 28171, 
    28173, 28174, 28179, 28181, 28182, 28185, 28186, 28188, 28195, 28197, 28198, 28201, 28202, 28204, 28209, 28210, 28212, 28216, 28227, 28229, 
    28230, 28233, 28234, 28236, 28241, 28242, 28244, 28248, 28257, 28258, 28260, 28264, 28272, 28291, 28293, 28294, 28297, 28298, 28300, 28305, 
    28306, 28308, 28312, 28321, 28322, 28324, 28328, 28336, 28353, 28354, 28356, 28360, 28368, 28384, 28419, 28421, 28422, 28425, 28426, 28428, 
    28433, 28434, 28436, 28440, 28449, 28450, 28452, 28456, 28464, 28481, 28482, 28484, 28488, 28496, 28512, 28545, 28546, 28548, 28552, 28560, 
    28576, 28608, 28703, 28719, 28727, 28731, 28733, 28734, 28751, 28759, 28763, 28765, 28766, 28775, 28779, 28781, 28782, 28787, 28789, 28790, 
    28793, 28794, 28796, 28815, 28823, 28827, 28829, 28830, 28839, 28843, 28845, 28846, 28851, 28853, 28854, 28857, 28858, 28860, 28871, 28875, 
    28877, 28878, 28883, 28885, 28886, 28889, 28890, 28892, 28899, 28901, 28902, 28905, 28906, 28908, 28913, 28914, 28916, 28920, 28943, 28951, 
    28955, 28957, 28958, 28967, 28971, 28973, 28974, 28979, 28981, 28982, 28985, 28986, 28988, 28999, 29003, 29005, 29006, 29011, 29013, 29014, 
    29017, 29018, 29020, 29027, 29029, 29030, 29033, 29034, 29036, 29041, 29042, 29044, 29048, 29063, 29067, 29069, 29070, 29075, 29077, 29078, 
    29081, 29082, 29084, 29091, 29093, 29094, 29097, 29098, 29100, 29105, 29106, 29108, 29112, 29123, 29125, 29126, 29129, 29130, 29132, 29137, 
    29138, 29140, 29144, 29153, 29154, 29156, 29160, 29168, 29199, 29207, 29211, 29213, 29214, 29223, 29227, 29229, 29230, 29235, 29237, 29238, 
    29241, 29242, 29244, 29255, 29259, 29261, 29262, 29267, 29269, 29270, 29273, 29274, 29276, 29283, 29285, 29286, 29289, 29290, 29292, 29297, 
    29298, 29300, 29304, 29319, 29323, 29325, 29326, 29331, 29333, 29334, 29337, 29338, 29340, 29347, 29349, 29350, 29353, 29354, 29356, 29361, 
    29362, 29364, 29368, 29379, 29381, 29382, 29385, 29386, 29388, 29393, 29394, 29396, 29400, 29409, 29410, 29412, 29416, 29424, 29447, 29451, 
    29453, 29454, 29459, 29461, 29462, 29465, 29466, 29468, 29475, 29477, 29478, 29481, 29482, 29484, 29489, 29490, 29492, 29496, 29507, 29509, 
    29510, 29513, 29514, 29516, 29521, 29522, 29524, 29528, 29537, 29538, 29540, 29544, 29552, 29571, 29573, 29574, 29577, 29578, 29580, 29585, 
    29586, 29588, 29592, 29601, 29602, 29604, 29608, 29616, 29633, 29634, 29636, 29640, 29648, 29664, 29711, 29719, 29723, 29725, 29726, 29735, 
    29739, 29741, 29742, 29747, 29749, 29750, 29753, 29754, 29756, 29767, 29771, 29773, 29774, 29779, 29781, 29782, 29785, 29786, 29788, 29795, 
    29797, 29798, 29801, 29802, 29804, 29809, 29810, 29812, 29816, 29831, 29835, 29837, 29838, 29843, 29845, 29846, 29849, 29850, 29852, 29859, 
    29861, 29862, 29865, 29866, 29868, 29873, 29874, 29876, 29880, 29891, 29893, 29894, 29897, 29898, 29900, 29905, 29906, 29908, 29912, 29921, 
    29922, 29924, 29928, 29936, 29959, 29963, 29965, 29966, 29971, 29973, 29974, 29977, 29978, 29980, 29987, 29989, 29990, 29993, 29994, 29996, 
    30001, 30002, 30004, 30008, 30019, 30021, 30022, 30025, 30026, 30028, 30033, 30034, 30036, 30040, 30049, 30050, 30052, 30056, 30064, 30083, 
    30085, 30086, 30089, 30090, 30092, 30097, 30098, 30100, 30104, 30113, 30114, 30116, 30120, 30128, 30145, 30146, 30148, 30152, 30160, 30176, 
    30215, 30219, 30221, 30222, 30227, 30229, 30230, 30233, 30234, 30236, 30243, 30245, 30246, 30249, 30250, 30252, 30257, 30258, 30260, 30264, 
    30275, 30277, 30278, 30281, 30282, 30284, 30289, 30290, 30292, 30296, 30305, 30306, 30308, 30312, 30320, 30339, 30341, 30342, 30345, 30346, 
    30348, 30353, 30354, 30356, 30360, 30369, 30370, 30372, 30376, 30384, 30401, 30402, 30404, 30408, 30416, 30432, 30467, 30469, 30470, 30473, 
    30474, 30476, 30481, 30482, 30484, 30488, 30497, 30498, 30500, 30504, 30512, 30529, 30530, 30532, 30536, 30544, 30560, 30593, 30594, 30596, 
    30600, 30608, 30624, 30656, 30735, 30743, 30747, 30749, 30750, 30759, 30763, 30765, 30766, 30771, 30773, 30774, 30777, 30778, 30780, 30791, 
    30795, 30797, 30798, 30803, 30805, 30806, 30809, 30810, 30812, 30819, 30821, 30822, 30825, 30826, 30828, 30833, 30834, 30836, 30840, 30855, 
    30859, 30861, 30862, 30867, 30869, 30870, 30873, 30874, 30876, 30883, 30885, 30886, 30889, 30890, 30892, 30897, 30898, 30900, 30904, 30915, 
    30917, 30918, 30921, 30922, 30924, 30929, 30930, 30932, 30936, 30945, 30946, 30948, 30952, 30960, 30983, 30987, 30989, 30990, 30995, 30997, 
    30998, 31001, 31002, 31004, 31011, 31013, 31014, 31017, 31018, 31020, 31025, 31026, 31028, 31032, 31043, 31045, 31046, 31049, 31050, 31052, 
    31057, 31058, 31060, 31064, 31073, 31074, 31076, 31080, 31088, 31107, 31109, 31110, 31113, 31114, 31116, 31121, 31122, 31124, 31128, 31137, 
    31138, 31140, 31144, 31152, 31169, 31170, 31172, 31176, 31184, 31200, 31239, 31243, 31245, 31246, 31251, 31253, 31254, 31257, 31258, 31260, 
    31267, 31269, 31270, 31273, 31274, 31276, 31281, 31282, 31284, 31288, 31299, 31301, 31302, 31305, 31306, 31308, 31313, 31314, 31316, 31320, 
    31329, 31330, 31332, 31336, 31344, 31363, 31365, 31366, 31369, 31370, 31372, 31377, 31378, 31380, 31384, 31393, 31394, 31396, 31400, 31408, 
    31425, 31426, 31428, 31432, 31440, 31456, 31491, 31493, 31494, 31497, 31498, 31500, 31505, 31506, 31508, 31512, 31521, 31522, 31524, 31528, 
    31536, 31553, 31554, 31556, 31560, 31568, 31584, 31617, 31618, 31620, 31624, 31632, 31648, 31680, 31751, 31755, 31757, 31758, 31763, 31765, 
    31766, 31769, 31770, 31772, 31779, 31781, 31782, 31785, 31786, 31788, 31793, 31794, 31796, 31800, 31811, 31813, 31814, 31817, 31818, 31820, 
    31825, 31826, 31828, 31832, 31841, 31842, 31844, 31848, 31856, 31875, 31877, 31878, 31881, 31882, 31884, 31889, 31890, 31892, 31896, 31905, 
    31906, 31908, 31912, 31920, 31937, 31938, 31940, 31944, 31952, 31968, 32003, 32005, 32006, 32009, 32010, 32012, 32017, 32018, 32020, 32024, 
    32033, 32034, 32036, 32040, 32048, 32065, 32066, 32068, 32072, 32080, 32096, 32129, 32130, 32132, 32136, 32144, 32160, 32192, 32259, 32261, 
    32262, 32265, 32266, 32268, 32273, 32274, 32276, 32280, 32289, 32290, 32292, 32296, 32304, 32321, 32322, 32324, 32328, 32336, 32352, 32385, 
    32386, 32388, 32392, 32400, 32416, 32448, 32513, 32514, 32516, 32520, 32528, 32544, 32576, 32640, 32895, 32959, 32991, 33007, 33015, 33019, 
    33021, 33022, 33087, 33119, 33135, 33143, 33147, 33149, 33150, 33183, 33199, 33207, 33211, 33213, 33214, 33231, 33239, 33243, 33245, 33246, 
    33255, 33259, 33261, 33262, 33267, 33269, 33270, 33273, 33274, 33276, 33343, 33375, 33391, 33399, 33403, 33405, 33406, 33439, 33455, 33463, 
    33467, 33469, 33470, 33487, 33495, 33499, 33501, 33502, 33511, 33515, 33517, 33518, 33523, 33525, 33526, 33529, 33530, 33532, 33567, 33583, 
    33591, 33595, 33597, 33598, 33615, 33623, 33627, 33629, 33630, 33639, 33643, 33645, 33646, 33651, 33653, 33654, 33657, 33658, 33660, 33679, 
    33687, 33691, 33693, 33694, 33703, 33707, 33709, 33710, 33715, 33717, 33718, 33721, 33722, 33724, 33735, 33739, 33741, 33742, 33747, 33749, 
    33750, 33753, 33754, 33756, 33763, 33765, 33766, 33769, 33770, 33772, 33777, 33778, 33780, 33784, 33855, 33887, 33903, 33911, 33915, 33917, 
    33918, 33951, 33967, 33975, 33979, 33981, 33982, 33999, 34007, 34011, 34013, 34014, 34023, 34027, 34029, 34030, 34035, 34037, 34038, 34041, 
    34042, 34044, 34079, 34095, 34103, 34107, 34109, 34110, 34127, 34135, 34139, 34141, 34142, 34151, 34155, 34157, 34158, 34163, 34165, 34166, 
    34169, 34170, 34172, 34191, 34199, 34203, 34205, 34206, 34215, 34219, 34221, 34222, 34227, 34229, 34230, 34233, 34234, 34236, 34247, 34251, 
    34253, 34254, 34259, 34261, 34262, 34265, 34266, 34268, 34275, 34277, 34278, 34281, 34282, 34284, 34289, 34290, 34292, 34296, 34335, 34351, 
    34359, 34363, 34365, 34366, 34383, 34391, 34395, 34397, 34398, 34407, 34411, 34413, 34414, 34419, 34421, 34422, 34425, 34426, 34428, 34447, 
    34455, 34459, 34461, 34462, 34471, 34475, 34477, 34478, 34483, 34485, 34486, 34489, 34490, 34492, 34503, 34507, 34509, 34510, 34515, 34517, 
    34518, 34521, 34522, 34524, 34531, 34533, 34534, 34537, 34538, 34540, 34545, 34546, 34548, 34552, 34575, 34583, 34587, 34589, 34590, 34599, 
    34603, 34605, 34606, 34611, 34613, 34614, 34617, 34618, 34620, 34631, 34635, 34637, 34638, 34643, 34645, 34646, 34649, 34650, 34652, 34659, 
    34661, 34662, 34665, 34666, 34668, 34673, 34674, 34676, 34680, 34695, 34699, 34701, 34702, 34707, 34709, 34710, 34713, 34714, 34716, 34723, 
    34725, 34726, 34729, 34730, 34732, 34737, 34738, 34740, 34744, 34755, 34757, 34758, 34761, 34762, 34764, 34769, 34770, 34772, 34776, 34785, 
    34786, 34788, 34792, 34800, 34879, 34911, 34927, 34935, 34939, 34941, 34942, 34975, 34991, 34999, 35003, 35005, 35006, 35023, 35031, 35035, 
    35037, 35038, 35047, 35051, 35053, 35054, 35059, 35061, 35062, 35065, 35066, 35068, 35103, 35119, 35127, 35131, 35133, 35134, 35151, 35159, 
    35163, 35165, 35166, 35175, 35179, 35181, 35182, 35187, 35189, 35190, 35193, 35194, 35196, 35215, 35223, 35227, 35229, 35230, 35239, 35243, 
    35245, 35246, 35251, 35253, 35254, 35257, 35258, 35260, 35271, 35275, 35277, 35278, 35283, 35285, 35286, 35289, 35290, 35292, 35299, 35301, 
    35302, 35305, 35306, 35308, 35313, 35314, 35316, 35320, 35359, 35375, 35383, 35387, 35389, 35390, 35407, 35415, 35419, 35421, 35422, 35431, 
    35435, 35437, 35438, 35443, 35445, 35446, 35449, 35450, 35452, 35471, 35479, 35483, 35485, 35486, 35495, 35499, 35501, 35502, 35507, 35509, 
    35510, 35513, 35514, 35516, 35527, 35531, 35533, 35534, 35539, 35541, 35542, 35545, 35546, 35548, 35555, 35557, 35558, 35561, 35562, 35564, 
    35569, 35570, 35572, 35576, 35599, 35607, 35611, 35613, 35614, 35623, 35627, 35629, 35630, 35635, 35637, 35638, 35641, 35642, 35644, 35655, 
    35659, 35661, 35662, 35667, 35669, 35670, 35673, 35674, 35676, 35683, 35685, 35686, 35689, 35690, 35692, 35697, 35698, 35700, 35704, 35719, 
    35723, 35725, 35726, 35731, 35733, 35734, 35737, 35738, 35740, 35747, 35749, 35750, 35753, 35754, 35756, 35761, 35762, 35764, 35768, 35779, 
    35781, 35782, 35785, 35786, 35788, 35793, 35794, 35796, 35800, 35809, 35810, 35812, 35816, 35824, 35871, 35887, 35895, 35899, 35901, 35902, 
    35919, 35927, 35931, 35933, 35934, 35943, 35947, 35949, 35950, 35955, 35957, 35958, 35961, 35962, 35964, 35983, 35991, 35995, 35997, 35998, 
    36007, 36011, 36013, 36014, 36019, 36021, 36022, 36025, 36026, 36028, 36039, 36043, 36045, 36046, 36051, 36053, 36054, 36057, 36058, 36060, 
    36067, 36069, 36070, 36073, 36074, 36076, 36081, 36082, 36084, 36088, 36111, 36119, 36123, 36125, 36126, 36135, 36139, 36141, 36142, 36147, 
    36149, 36150, 36153, 36154, 36156, 36167, 36171, 36173, 36174, 36179, 36181, 36182, 36185, 36186, 36188, 36195, 36197, 36198, 36201, 36202, 
    36204, 36209, 36210, 36212, 36216, 36231, 36235, 36237, 36238, 36243, 36245, 36246, 36249, 36250, 36252, 36259, 36261, 36262, 36265, 36266, 
    36268, 36273, 36274, 36276, 36280, 36291, 36293, 36294, 36297, 36298, 36300, 36305, 36306, 36308, 36312, 36321, 36322, 36324, 36328, 36336, 
    36367, 36375, 36379, 36381, 36382, 36391, 36395, 36397, 36398, 36403, 36405, 36406, 36409, 36410, 36412, 36423, 36427, 36429, 36430, 36435, 
    36437, 36438, 36441, 36442, 36444, 36451, 36453, 36454, 36457, 36458, 36460, 36465, 36466, 36468, 36472, 36487, 36491, 36493, 36494, 36499, 
    36501, 36502, 36505, 36506, 36508, 36515, 36517, 36518, 36521, 36522, 36524, 36529, 36530, 36532, 36536, 36547, 36549, 36550, 36553, 36554, 
    36556, 36561, 36562, 36564, 36568, 36577, 36578, 36580, 36584, 36592, 36615, 36619, 36621, 36622, 36627, 36629, 36630, 36633, 36634, 36636, 
    36643, 36645, 36646, 36649, 36650, 36652, 36657, 36658, 36660, 36664, 36675, 36677, 36678, 36681, 36682, 36684, 36689, 36690, 36692, 36696, 
    36705, 36706, 36708, 36712, 36720, 36739, 36741, 36742, 36745, 36746, 36748, 36753, 36754, 36756, 36760, 36769, 36770, 36772, 36776, 36784, 
    36801, 36802, 36804, 36808, 36816, 36832, 36927, 36959, 36975, 36983, 36987, 36989, 36990, 37023, 37039, 37047, 37051, 37053, 37054, 37071, 
    37079, 37083, 37085, 37086, 37095, 37099, 37101, 37102, 37107, 37109, 37110, 37113, 37114, 37116, 37151, 37167, 37175, 37179, 37181, 37182, 
    37199, 37207, 37211, 37213, 37214, 37223, 37227, 37229, 37230, 37235, 37237, 37238, 37241, 37242, 37244, 37263, 37271, 37275, 37277, 37278, 
    37287, 37291, 37293, 37294, 37299, 37301, 37302, 37305, 37306, 37308, 37319, 37323, 37325, 37326, 37331, 37333, 37334, 37337, 37338, 37340, 
    37347, 37349, 37350, 37353, 37354, 37356, 37361, 37362, 37364, 37368, 37407, 37423, 37431, 37435, 37437, 37438, 37455, 37463, 37467, 37469, 
    37470, 37479, 37483, 37485, 37486, 37491, 37493, 37494, 37497, 37498, 37500, 37519, 37527, 37531, 37533, 37534, 37543, 37547, 37549, 37550, 
    37555, 37557, 37558, 37561, 37562, 37564, 37575, 37579, 37581, 37582, 37587, 37589, 37590, 37593, 37594, 37596, 37603, 37605, 37606, 37609, 
    37610, 37612, 37617, 37618, 37620, 37624, 37647, 37655, 37659, 37661, 37662, 37671, 37675, 37677, 37678, 37683, 37685, 37686, 37689, 37690, 
    37692, 37703, 37707, 37709, 37710, 37715, 37717, 37718, 37721, 37722, 37724, 37731, 37733, 37734, 37737, 37738, 37740, 37745, 37746, 37748, 
    37752, 37767, 37771, 37773, 37774, 37779, 37781, 37782, 37785, 37786, 37788, 37795, 37797, 37798, 37801, 37802, 37804, 37809, 37810, 37812, 
    37816, 37827, 37829, 37830, 37833, 37834, 37836, 37841, 37842, 37844, 37848, 37857, 37858, 37860, 37864, 37872, 37919, 37935, 37943, 37947, 
    37949, 37950, 37967, 37975, 37979, 37981, 37982, 37991, 37995, 37997, 37998, 38003, 38005, 38006, 38009, 38010, 38012, 38031, 38039, 38043, 
    38045, 38046, 38055, 38059, 38061, 38062, 38067, 38069, 38070, 38073, 38074, 38076, 38087, 38091, 38093, 38094, 38099, 38101, 38102, 38105, 
    38106, 38108, 38115, 38117, 38118, 38121, 38122, 38124, 38129, 38130, 38132, 38136, 38159, 38167, 38171, 38173, 38174, 38183, 38187, 38189, 
    38190, 38195, 38197, 38198, 38201, 38202, 38204, 38215, 38219, 38221, 38222, 38227, 38229, 38230, 38233, 38234, 38236, 38243, 38245, 38246, 
    38249, 38250, 38252, 38257, 38258, 38260, 38264, 38279, 38283, 38285, 38286, 38291, 38293, 38294, 38297, 38298, 38300, 38307, 38309, 38310, 
    38313, 38314, 38316, 38321, 38322, 38324, 38328, 38339, 38341, 38342, 38345, 38346, 38348, 38353, 38354, 38356, 38360, 38369, 38370, 38372, 
    38376, 38384, 38415, 38423, 38427, 38429, 38430, 38439, 38443, 38445, 38446, 38451, 38453, 38454, 38457, 38458, 38460, 38471, 38475, 38477, 
    38478, 38483, 38485, 38486, 38489, 38490, 38492, 38499, 38501, 38502, 38505, 38506, 38508, 38513, 38514, 38516, 38520, 38535, 38539, 38541, 
    38542, 38547, 38549, 38550, 38553, 38554, 38556, 38563, 38565, 38566, 38569, 38570, 38572, 38577, 38578, 38580, 38584, 38595, 38597, 38598, 
    38601, 38602, 38604, 38609, 38610, 38612, 38616, 38625, 38626, 38628, 38632, 38640, 38663, 38667, 38669, 38670, 38675, 38677, 38678, 38681, 
    38682, 38684, 38691, 38693, 38694, 38697, 38698, 38700, 38705, 38706, 38708, 38712, 38723, 38725, 38726, 38729, 38730, 38732, 38737, 38738, 
    38740, 38744, 38753, 38754, 38756, 38760, 38768, 38787, 38789, 38790, 38793, 38794, 38796, 38801, 38802, 38804, 38808, 38817, 38818, 38820, 
    38824, 38832, 38849, 38850, 38852, 38856, 38864, 38880, 38943, 38959, 38967, 38971, 38973, 38974, 38991, 38999, 39003, 39005, 39006, 39015, 
    39019, 39021, 39022, 39027, 39029, 39030, 39033, 39034, 39036, 39055, 39063, 39067, 39069, 39070, 39079, 39083, 39085, 39086, 39091, 39093, 
    39094, 39097, 39098, 39100, 39111, 39115, 39117, 39118, 39123, 39125, 39126, 39129, 39130, 39132, 39139, 39141, 39142, 39145, 39146, 39148, 
    39153, 39154, 39156, 39160, 39183, 39191, 39195, 39197, 39198, 39207, 39211, 39213, 39214, 39219, 39221, 39222, 39225, 39226, 39228, 39239, 
    39243, 39245, 39246, 39251, 39253, 39254, 39257, 39258, 39260, 39267, 39269, 39270, 39273, 39274, 39276, 39281, 39282, 39284, 39288, 39303, 
    39307, 39309, 39310, 39315, 39317, 39318, 39321, 39322, 39324, 39331, 39333, 39334, 39337, 39338, 39340, 39345, 39346, 39348, 39352, 39363, 
    39365, 39366, 39369, 39370, 39372, 39377, 39378, 39380, 39384, 39393, 39394, 39396, 39400, 39408, 39439, 39447, 39451, 39453, 39454, 39463, 
    39467, 39469, 39470, 39475, 39477, 39478, 39481, 39482, 39484, 39495, 39499, 39501, 39502, 39507, 39509, 39510, 39513, 39514, 39516, 39523, 
    39525, 39526, 39529, 39530, 39532, 39537, 39538, 39540, 39544, 39559, 39563, 39565, 39566, 39571, 39573, 39574, 39577, 39578, 39580, 39587, 
    39589, 39590, 39593, 39594, 39596, 39601, 39602, 39604, 39608, 39619, 39621, 39622, 39625, 39626, 39628, 39633, 39634, 39636, 39640, 39649, 
    39650, 39652, 39656, 39664, 39687, 39691, 39693, 39694, 39699, 39701, 39702, 39705, 39706, 39708, 39715, 39717, 39718, 39721, 39722, 39724, 
    39729, 39730, 39732, 39736, 39747, 39749, 39750, 39753, 39754, 39756, 39761, 39762, 39764, 39768, 39777, 39778, 39780, 39784, 39792, 39811, 
    39813, 39814, 39817, 39818, 39820, 39825, 39826, 39828, 39832, 39841, 39842, 39844, 39848, 39856, 39873, 39874, 39876, 39880, 39888, 39904, 
    39951, 39959, 39963, 39965, 39966, 39975, 39979, 39981, 39982, 39987, 39989, 39990, 39993, 39994, 39996, 40007, 40011, 40013, 40014, 40019, 
    40021, 40022, 40025, 40026, 40028, 40035, 40037, 40038, 40041, 40042, 40044, 40049, 40050, 40052, 40056, 40071, 40075, 40077, 40078, 40083, 
    40085, 40086, 40089, 40090, 40092, 40099, 40101, 40102, 40105, 40106, 40108, 40113, 40114, 40116, 40120, 40131, 40133, 40134, 40137, 40138, 
    40140, 40145, 40146, 40148, 40152, 40161, 40162, 40164, 40168, 40176, 40199, 40203, 40205, 40206, 40211, 40213, 40214, 40217, 40218, 40220, 
    40227, 40229, 40230, 40233, 40234, 40236, 40241, 40242, 40244, 40248, 40259, 40261, 40262, 40265, 40266, 40268, 40273, 40274, 40276, 40280, 
    40289, 40290, 40292, 40296, 40304, 40323, 40325, 40326, 40329, 40330, 40332, 40337, 40338, 40340, 40344, 40353, 40354, 40356, 40360, 40368, 
    40385, 40386, 40388, 40392, 40400, 40416, 40455, 40459, 40461, 40462, 40467, 40469, 40470, 40473, 40474, 40476, 40483, 40485, 40486, 40489, 
    40490, 40492, 40497, 40498, 40500, 40504, 40515, 40517, 40518, 40521, 40522, 40524, 40529, 40530, 40532, 40536, 40545, 40546, 40548, 40552, 
    40560, 40579, 40581, 40582, 40585, 40586, 40588, 40593, 40594, 40596, 40600, 40609, 40610, 40612, 40616, 40624, 40641, 40642, 40644, 40648, 
    40656, 40672, 40707, 40709, 40710, 40713, 40714, 40716, 40721, 40722, 40724, 40728, 40737, 40738, 40740, 40744, 40752, 40769, 40770, 40772, 
    40776, 40784, 40800, 40833, 40834, 40836, 40840, 40848, 40864, 40896, 41023, 41055, 41071, 41079, 41083, 41085, 41086, 41119, 41135, 41143, 
    41147, 41149, 41150, 41167, 41175, 41179, 41181, 41182, 41191, 41195, 41197, 41198, 41203, 41205, 41206, 41209, 41210, 41212, 41247, 41263, 
    41271, 41275, 41277, 41278, 41295, 41303, 41307, 41309, 41310, 41319, 41323, 41325, 41326, 41331, 41333, 41334, 41337, 41338, 41340, 41359, 
    41367, 41371, 41373, 41374, 41383, 41387, 41389, 41390, 41395, 41397, 41398, 41401, 41402, 41404, 41415, 41419, 41421, 41422, 41427, 41429, 
    41430, 41433, 41434, 41436, 41443, 41445, 41446, 41449, 41450, 41452, 41457, 41458, 41460, 41464, 41503, 41519, 41527, 41531, 41533, 41534, 
    41551, 41559, 41563, 41565, 41566, 41575, 41579, 41581, 41582, 41587, 41589, 41590, 41593, 41594, 41596, 41615, 41623, 41627, 41629, 41630, 
    41639, 41643, 41645, 41646, 41651, 41653, 41654, 41657, 41658, 41660, 41671, 41675, 41677, 41678, 41683, 41685, 41686, 41689, 41690, 41692, 
    41699, 41701, 41702, 41705, 41706, 41708, 41713, 41714, 41716, 41720, 41743, 41751, 41755, 41757, 41758, 41767, 41771, 41773, 41774, 41779, 
    41781, 41782, 41785, 41786, 41788, 41799, 41803, 41805, 41806, 41811, 41813, 41814, 41817, 41818, 41820, 41827, 41829, 41830, 41833, 41834, 
    41836, 41841, 41842, 41844, 41848, 41863, 41867, 41869, 41870, 41875, 41877, 41878, 41881, 41882, 41884, 41891, 41893, 41894, 41897, 41898, 
    41900, 41905, 41906, 41908, 41912, 41923, 41925, 41926, 41929, 41930, 41932, 41937, 41938, 41940, 41944, 41953, 41954, 41956, 41960, 41968, 
    42015, 42031, 42039, 42043, 42045, 42046, 42063, 42071, 42075, 42077, 42078, 42087, 42091, 42093, 42094, 42099, 42101, 42102, 42105, 42106, 
    42108, 42127, 42135, 42139, 42141, 42142, 42151, 42155, 42157, 42158, 42163, 42165, 42166, 42169, 42170, 42172, 42183, 42187, 42189, 42190, 
    42195, 42197, 42198, 42201, 42202, 42204, 42211, 42213, 42214, 42217, 42218, 42220, 42225, 42226, 42228, 42232, 42255, 42263, 42267, 42269, 
    42270, 42279, 42283, 42285, 42286, 42291, 42293, 42294, 42297, 42298, 42300, 42311, 42315, 42317, 42318, 42323, 42325, 42326, 42329, 42330, 
    42332, 42339, 42341, 42342, 42345, 42346, 42348, 42353, 42354, 42356, 42360, 42375, 42379, 42381, 42382, 42387, 42389, 42390, 42393, 42394, 
    42396, 42403, 42405, 42406, 42409, 42410, 42412, 42417, 42418, 42420, 42424, 42435, 42437, 42438, 42441, 42442, 42444, 42449, 42450, 42452, 
    42456, 42465, 42466, 42468, 42472, 42480, 42511, 42519, 42523, 42525, 42526, 42535, 42539, 42541, 42542, 42547, 42549, 42550, 42553, 42554, 
    42556, 42567, 42571, 42573, 42574, 42579, 42581, 42582, 42585, 42586, 42588, 42595, 42597, 42598, 42601, 42602, 42604, 42609, 42610, 42612, 
    42616, 42631, 42635, 42637, 42638, 42643, 42645, 42646, 42649, 42650, 42652, 42659, 42661, 42662, 42665, 42666, 42668, 42673, 42674, 42676, 
    42680, 42691, 42693, 42694, 42697, 42698, 42700, 42705, 42706, 42708, 42712, 42721, 42722, 42724, 42728, 42736, 42759, 42763, 42765, 42766, 
    42771, 42773, 42774, 42777, 42778, 42780, 42787, 42789, 42790, 42793, 42794, 42796, 42801, 42802, 42804, 42808, 42819, 42821, 42822, 42825, 
    42826, 42828, 42833, 42834, 42836, 42840, 42849, 42850, 42852, 42856, 42864, 42883, 42885, 42886, 42889, 42890, 42892, 42897, 42898, 42900, 
    42904, 42913, 42914, 42916, 42920, 42928, 42945, 42946, 42948, 42952, 42960, 42976, 43039, 43055, 43063, 43067, 43069, 43070, 43087, 43095, 
    43099, 43101, 43102, 43111, 43115, 43117, 43118, 43123, 43125, 43126, 43129, 43130, 43132, 43151, 43159, 43163, 43165, 43166, 43175, 43179, 
    43181, 43182, 43187, 43189, 43190, 43193, 43194, 43196, 43207, 43211, 43213, 43214, 43219, 43221, 43222, 43225, 43226, 43228, 43235, 43237, 
    43238, 43241, 43242, 43244, 43249, 43250, 43252, 43256, 43279, 43287, 43291, 43293, 43294, 43303, 43307, 43309, 43310, 43315, 43317, 43318, 
    43321, 43322, 43324, 43335, 43339, 43341, 43342, 43347, 43349, 43350, 43353, 43354, 43356, 43363, 43365, 43366, 43369, 43370, 43372, 43377, 
    43378, 43380, 43384, 43399, 43403, 43405, 43406, 43411, 43413, 43414, 43417, 43418, 43420, 43427, 43429, 43430, 43433, 43434, 43436, 43441, 
    43442, 43444, 43448, 43459, 43461, 43462, 43465, 43466, 43468, 43473, 43474, 43476, 43480, 43489, 43490, 43492, 43496, 43504, 43535, 43543, 
    43547, 43549, 43550, 43559, 43563, 43565, 43566, 43571, 43573, 43574, 43577, 43578, 43580, 43591, 43595, 43597, 43598, 43603, 43605, 43606, 
    43609, 43610, 43612, 43619, 43621, 43622, 43625, 43626, 43628, 43633, 43634, 43636, 43640, 43655, 43659, 43661, 43662, 43667, 43669, 43670, 
    43673, 43674, 43676, 43683, 43685, 43686, 43689, 43690, 43692, 43697, 43698, 43700, 43704, 43715, 43717, 43718, 43721, 43722, 43724, 43729, 
    43730, 43732, 43736, 43745, 43746, 43748, 43752, 43760, 43783, 43787, 43789, 43790, 43795, 43797, 43798, 43801, 43802, 43804, 43811, 43813, 
    43814, 43817, 43818, 43820, 43825, 43826, 43828, 43832, 43843, 43845, 43846, 43849, 43850, 43852, 43857, 43858, 43860, 43864, 43873, 43874, 
    43876, 43880, 43888, 43907, 43909, 43910, 43913, 43914, 43916, 43921, 43922, 43924, 43928, 43937, 43938, 43940, 43944, 43952, 43969, 43970, 
    43972, 43976, 43984, 44000, 44047, 44055, 44059, 44061, 44062, 44071, 44075, 44077, 44078, 44083, 44085, 44086, 44089, 44090, 44092, 44103, 
    44107, 44109, 44110, 44115, 44117, 44118, 44121, 44122, 44124, 44131, 44133, 44134, 44137, 44138, 44140, 44145, 44146, 44148, 44152, 44167, 
    44171, 44173, 44174, 44179, 44181, 44182, 44185, 44186, 44188, 44195, 44197, 44198, 44201, 44202, 44204, 44209, 44210, 44212, 44216, 44227, 
    44229, 44230, 44233, 44234, 44236, 44241, 44242, 44244, 44248, 44257, 44258, 44260, 44264, 44272, 44295, 44299, 44301, 44302, 44307, 44309, 
    44310, 44313, 44314, 44316, 44323, 44325, 44326, 44329, 44330, 44332, 44337, 44338, 44340, 44344, 44355, 44357, 44358, 44361, 44362, 44364, 
    44369, 44370, 44372, 44376, 44385, 44386, 44388, 44392, 44400, 44419, 44421, 44422, 44425, 44426, 44428, 44433, 44434, 44436, 44440, 44449, 
    44450, 44452, 44456, 44464, 44481, 44482, 44484, 44488, 44496, 44512, 44551, 44555, 44557, 44558, 44563, 44565, 44566, 44569, 44570, 44572, 
    44579, 44581, 44582, 44585, 44586, 44588, 44593, 44594, 44596, 44600, 44611, 44613, 44614, 44617, 44618, 44620, 44625, 44626, 44628, 44632, 
    44641, 44642, 44644, 44648, 44656, 44675, 44677, 44678, 44681, 44682, 44684, 44689, 44690, 44692, 44696, 44705, 44706, 44708, 44712, 44720, 
    44737, 44738, 44740, 44744, 44752, 44768, 44803, 44805, 44806, 44809, 44810, 44812, 44817, 44818, 44820, 44824, 44833, 44834, 44836, 44840, 
    44848, 44865, 44866, 44868, 44872, 44880, 44896, 44929, 44930, 44932, 44936, 44944, 44960, 44992, 45087, 45103, 45111, 45115, 45117, 45118, 
    45135, 45143, 45147, 45149, 45150, 45159, 45163, 45165, 45166, 45171, 45173, 45174, 45177, 45178, 45180, 45199, 45207, 45211, 45213, 45214, 
    45223, 45227, 45229, 45230, 45235, 45237, 45238, 45241, 45242, 45244, 45255, 45259, 45261, 45262, 45267, 45269, 45270, 45273, 45274, 45276, 
    45283, 45285, 45286, 45289, 45290, 45292, 45297, 45298, 45300, 45304, 45327, 45335, 45339, 45341, 45342, 45351, 45355, 45357, 45358, 45363, 
    45365, 45366, 45369, 45370, 45372, 45383, 45387, 45389, 45390, 45395, 45397, 45398, 45401, 45402, 45404, 45411, 45413, 45414, 45417, 45418, 
    45420, 45425, 45426, 45428, 45432, 45447, 45451, 45453, 45454, 45459, 45461, 45462, 45465, 45466, 45468, 45475, 45477, 45478, 45481, 45482, 
    45484, 45489, 45490, 45492, 45496, 45507, 45509, 45510, 45513, 45514, 45516, 45521, 45522, 45524, 45528, 45537, 45538, 45540, 45544, 45552, 
    45583, 45591, 45595, 45597, 45598, 45607, 45611, 45613, 45614, 45619, 45621, 45622, 45625, 45626, 45628, 45639, 45643, 45645, 45646, 45651, 
    45653, 45654, 45657, 45658, 45660, 45667, 45669, 45670, 45673, 45674, 45676, 45681, 45682, 45684, 45688, 45703, 45707, 45709, 45710, 45715, 
    45717, 45718, 45721, 45722, 45724, 45731, 45733, 45734, 45737, 45738, 45740, 45745, 45746, 45748, 45752, 45763, 45765, 45766, 45769, 45770, 
    45772, 45777, 45778, 45780, 45784, 45793, 45794, 45796, 45800, 45808, 45831, 45835, 45837, 45838, 45843, 45845, 45846, 45849, 45850, 45852, 
    45859, 45861, 45862, 45865, 45866, 45868, 45873, 45874, 45876, 45880, 45891, 45893, 45894, 45897, 45898, 45900, 45905, 45906, 45908, 45912, 
    45921, 45922, 45924, 45928, 45936, 45955, 45957, 45958, 45961, 45962, 45964, 45969, 45970, 45972, 45976, 45985, 45986, 45988, 45992, 46000, 
    46017, 46018, 46020, 46024, 46032, 46048, 46095, 46103, 46107, 46109, 46110, 46119, 46123, 46125, 46126, 46131, 46133, 46134, 46137, 46138, 
    46140, 46151, 46155, 46157, 46158, 46163, 46165, 46166, 46169, 46170, 46172, 46179, 46181, 46182, 46185, 46186, 46188, 46193, 46194, 46196, 
    46200, 46215, 46219, 46221, 46222, 46227, 46229, 46230, 46233, 46234, 46236, 46243, 46245, 46246, 46249, 46250, 46252, 46257, 46258, 46260, 
    46264, 46275, 46277, 46278, 46281, 46282, 46284, 46289, 46290, 46292, 46296, 46305, 46306, 46308, 46312, 46320, 46343, 46347, 46349, 46350, 
    46355, 46357, 46358, 46361, 46362, 46364, 46371, 46373, 46374, 46377, 46378, 46380, 46385, 46386, 46388, 46392, 46403, 46405, 46406, 46409, 
    46410, 46412, 46417, 46418, 46420, 46424, 46433, 46434, 46436, 46440, 46448, 46467, 46469, 46470, 46473, 46474, 46476, 46481, 46482, 46484, 
    46488, 46497, 46498, 46500, 46504, 46512, 46529, 46530, 46532, 46536, 46544, 46560, 46599, 46603, 46605, 46606, 46611, 46613, 46614, 46617, 
    46618, 46620, 46627, 46629, 46630, 46633, 46634, 46636, 46641, 46642, 46644, 46648, 46659, 46661, 46662, 46665, 46666, 46668, 46673, 46674, 
    46676, 46680, 46689, 46690, 46692, 46696, 46704, 46723, 46725, 46726, 46729, 46730, 46732, 46737, 46738, 46740, 46744, 46753, 46754, 46756, 
    46760, 46768, 46785, 46786, 46788, 46792, 46800, 46816, 46851, 46853, 46854, 46857, 46858, 46860, 46865, 46866, 46868, 46872, 46881, 46882, 
    46884, 46888, 46896, 46913, 46914, 46916, 46920, 46928, 46944, 46977, 46978, 46980, 46984, 46992, 47008, 47040, 47119, 47127, 47131, 47133, 
    47134, 47143, 47147, 47149, 47150, 47155, 47157, 47158, 47161, 47162, 47164, 47175, 47179, 47181, 47182, 47187, 47189, 47190, 47193, 47194, 
    47196, 47203, 47205, 47206, 47209, 47210, 47212, 47217, 47218, 47220, 47224, 47239, 47243, 47245, 47246, 47251, 47253, 47254, 47257, 47258, 
    47260, 47267, 47269, 47270, 47273, 47274, 47276, 47281, 47282, 47284, 47288, 47299, 47301, 47302, 47305, 47306, 47308, 47313, 47314, 47316, 
    47320, 47329, 47330, 47332, 47336, 47344, 47367, 47371, 47373, 47374, 47379, 47381, 47382, 47385, 47386, 47388, 47395, 47397, 47398, 47401, 
    47402, 47404, 47409, 47410, 47412, 47416, 47427, 47429, 47430, 47433, 47434, 47436, 47441, 47442, 47444, 47448, 47457, 47458, 47460, 47464, 
    47472, 47491, 47493, 47494, 47497, 47498, 47500, 47505, 47506, 47508, 47512, 47521, 47522, 47524, 47528, 47536, 47553, 47554, 47556, 47560, 
    47568, 47584, 47623, 47627, 47629, 47630, 47635, 47637, 47638, 47641, 47642, 47644, 47651, 47653, 47654, 47657, 47658, 47660, 47665, 47666, 
    47668, 47672, 47683, 47685, 47686, 47689, 47690, 47692, 47697, 47698, 47700, 47704, 47713, 47714, 47716, 47720, 47728, 47747, 47749, 47750, 
    47753, 47754, 47756, 47761, 47762, 47764, 47768, 47777, 47778, 47780, 47784, 47792, 47809, 47810, 47812, 47816, 47824, 47840, 47875, 47877, 
    47878, 47881, 47882, 47884, 47889, 47890, 47892, 47896, 47905, 47906, 47908, 47912, 47920, 47937, 47938, 47940, 47944, 47952, 47968, 48001, 
    48002, 48004, 48008, 48016, 48032, 48064, 48135, 48139, 48141, 48142, 48147, 48149, 48150, 48153, 48154, 48156, 48163, 48165, 48166, 48169, 
    48170, 48172, 48177, 48178, 48180, 48184, 48195, 48197, 48198, 48201, 48202, 48204, 48209, 48210, 48212, 48216, 48225, 48226, 48228, 48232, 
    48240, 48259, 48261, 48262, 48265, 48266, 48268, 48273, 48274, 48276, 48280, 48289, 48290, 48292, 48296, 48304, 48321, 48322, 48324, 48328, 
    48336, 48352, 48387, 48389, 48390, 48393, 48394, 48396, 48401, 48402, 48404, 48408, 48417, 48418, 48420, 48424, 48432, 48449, 48450, 48452, 
    48456, 48464, 48480, 48513, 48514, 48516, 48520, 48528, 48544, 48576, 48643, 48645, 48646, 48649, 48650, 48652, 48657, 48658, 48660, 48664, 
    48673, 48674, 48676, 48680, 48688, 48705, 48706, 48708, 48712, 48720, 48736, 48769, 48770, 48772, 48776, 48784, 48800, 48832, 48897, 48898, 
    48900, 48904, 48912, 48928, 48960, 49024, 49215, 49247, 49263, 49271, 49275, 49277, 49278, 49311, 49327, 49335, 49339, 49341, 49342, 49359, 
    49367, 49371, 49373, 49374, 49383, 49387, 49389, 49390, 49395, 49397, 49398, 49401, 49402, 49404, 49439, 49455, 49463, 49467, 49469, 49470, 
    49487, 49495, 49499, 49501, 49502, 49511, 49515, 49517, 49518, 49523, 49525, 49526, 49529, 49530, 49532, 49551, 49559, 49563, 49565, 49566, 
    49575, 49579, 49581, 49582, 49587, 49589, 49590, 49593, 49594, 49596, 49607, 49611, 49613, 49614, 49619, 49621, 49622, 49625, 49626, 49628, 
    49635, 49637, 49638, 49641, 49642, 49644, 49649, 49650, 49652, 49656, 49695, 49711, 49719, 49723, 49725, 49726, 49743, 49751, 49755, 49757, 
    49758, 49767, 49771, 49773, 49774, 49779, 49781, 49782, 49785, 49786, 49788, 49807, 49815, 49819, 49821, 49822, 49831, 49835, 49837, 49838, 
    49843, 49845, 49846, 49849, 49850, 49852, 49863, 49867, 49869, 49870, 49875, 49877, 49878, 49881, 49882, 49884, 49891, 49893, 49894, 49897, 
    49898, 49900, 49905, 49906, 49908, 49912, 49935, 49943, 49947, 49949, 49950, 49959, 49963, 49965, 49966, 49971, 49973, 49974, 49977, 49978, 
    49980, 49991, 49995, 49997, 49998, 50003, 50005, 50006, 50009, 50010, 50012, 50019, 50021, 50022, 50025, 50026, 50028, 50033, 50034, 50036, 
    50040, 50055, 50059, 50061, 50062, 50067, 50069, 50070, 50073, 50074, 50076, 50083, 50085, 50086, 50089, 50090, 50092, 50097, 50098, 50100, 
    50104, 50115, 50117, 50118, 50121, 50122, 50124, 50129, 50130, 50132, 50136, 50145, 50146, 50148, 50152, 50160, 50207, 50223, 50231, 50235, 
    50237, 50238, 50255, 50263, 50267, 50269, 50270, 50279, 50283, 50285, 50286, 50291, 50293, 50294, 50297, 50298, 50300, 50319, 50327, 50331, 
    50333, 50334, 50343, 50347, 50349, 50350, 50355, 50357, 50358, 50361, 50362, 50364, 50375, 50379, 50381, 50382, 50387, 50389, 50390, 50393, 
    50394, 50396, 50403, 50405, 50406, 50409, 50410, 50412, 50417, 50418, 50420, 50424, 50447, 50455, 50459, 50461, 50462, 50471, 50475, 50477, 
    50478, 50483, 50485, 50486, 50489, 50490, 50492, 50503, 50507, 50509, 50510, 50515, 50517, 50518, 50521, 50522, 50524, 50531, 50533, 50534, 
    50537, 50538, 50540, 50545, 50546, 50548, 50552, 50567, 50571, 50573, 50574, 50579, 50581, 50582, 50585, 50586, 50588, 50595, 50597, 50598, 
    50601, 50602, 50604, 50609, 50610, 50612, 50616, 50627, 50629, 50630, 50633, 50634, 50636, 50641, 50642, 50644, 50648, 50657, 50658, 50660, 
    50664, 50672, 50703, 50711, 50715, 50717, 50718, 50727, 50731, 50733, 50734, 50739, 50741, 50742, 50745, 50746, 50748, 50759, 50763, 50765, 
    50766, 50771, 50773, 50774, 50777, 50778, 50780, 50787, 50789, 50790, 50793, 50794, 50796, 50801, 50802, 50804, 50808, 50823, 50827, 50829, 
    50830, 50835, 50837, 50838, 50841, 50842, 50844, 50851, 50853, 50854, 50857, 50858, 50860, 50865, 50866, 50868, 50872, 50883, 50885, 50886, 
    50889, 50890, 50892, 50897, 50898, 50900, 50904, 50913, 50914, 50916, 50920, 50928, 50951, 50955, 50957, 50958, 50963, 50965, 50966, 50969, 
    50970, 50972, 50979, 50981, 50982, 50985, 50986, 50988, 50993, 50994, 50996, 51000, 51011, 51013, 51014, 51017, 51018, 51020, 51025, 51026, 
    51028, 51032, 51041, 51042, 51044, 51048, 51056, 51075, 51077, 51078, 51081, 51082, 51084, 51089, 51090, 51092, 51096, 51105, 51106, 51108, 
    51112, 51120, 51137, 51138, 51140, 51144, 51152, 51168, 51231, 51247, 51255, 51259, 51261, 51262, 51279, 51287, 51291, 51293, 51294, 51303, 
    51307, 51309, 51310, 51315, 51317, 51318, 51321, 51322, 51324, 51343, 51351, 51355, 51357, 51358, 51367, 51371, 51373, 51374, 51379, 51381, 
    51382, 51385, 51386, 51388, 51399, 51403, 51405, 51406, 51411, 51413, 51414, 51417, 51418, 51420, 51427, 51429, 51430, 51433, 51434, 51436, 
    51441, 51442, 51444, 51448, 51471, 51479, 51483, 51485, 51486, 51495, 51499, 51501, 51502, 51507, 51509, 51510, 51513, 51514, 51516, 51527, 
    51531, 51533, 51534, 51539, 51541, 51542, 51545, 51546, 51548, 51555, 51557, 51558, 51561, 51562, 51564, 51569, 51570, 51572, 51576, 51591, 
    51595, 51597, 51598, 51603, 51605, 51606, 51609, 51610, 51612, 51619, 51621, 51622, 51625, 51626, 51628, 51633, 51634, 51636, 51640, 51651, 
    51653, 51654, 51657, 51658, 51660, 51665, 51666, 51668, 51672, 51681, 51682, 51684, 51688, 51696, 51727, 51735, 51739, 51741, 51742, 51751, 
    51755, 51757, 51758, 51763, 51765, 51766, 51769, 51770, 51772, 51783, 51787, 51789, 51790, 51795, 51797, 51798, 51801, 51802, 51804, 51811, 
    51813, 51814, 51817, 51818, 51820, 51825, 51826, 51828, 51832, 51847, 51851, 51853, 51854, 51859, 51861, 51862, 51865, 51866, 51868, 51875, 
    51877, 51878, 51881, 51882, 51884, 51889, 51890, 51892, 51896, 51907, 51909, 51910, 51913, 51914, 51916, 51921, 51922, 51924, 51928, 51937, 
    51938, 51940, 51944, 51952, 51975, 51979, 51981, 51982, 51987, 51989, 51990, 51993, 51994, 51996, 52003, 52005, 52006, 52009, 52010, 52012, 
    52017, 52018, 52020, 52024, 52035, 52037, 52038, 52041, 52042, 52044, 52049, 52050, 52052, 52056, 52065, 52066, 52068, 52072, 52080, 52099, 
    52101, 52102, 52105, 52106, 52108, 52113, 52114, 52116, 52120, 52129, 52130, 52132, 52136, 52144, 52161, 52162, 52164, 52168, 52176, 52192, 
    52239, 52247, 52251, 52253, 52254, 52263, 52267, 52269, 52270, 52275, 52277, 52278, 52281, 52282, 52284, 52295, 52299, 52301, 52302, 52307, 
    52309, 52310, 52313, 52314, 52316, 52323, 52325, 52326, 52329, 52330, 52332, 52337, 52338, 52340, 52344, 52359, 52363, 52365, 52366, 52371, 
    52373, 52374, 52377, 52378, 52380, 52387, 52389, 52390, 52393, 52394, 52396, 52401, 52402, 52404, 52408, 52419, 52421, 52422, 52425, 52426, 
    52428, 52433, 52434, 52436, 52440, 52449, 52450, 52452, 52456, 52464, 52487, 52491, 52493, 52494, 52499, 52501, 52502, 52505, 52506, 52508, 
    52515, 52517, 52518, 52521, 52522, 52524, 52529, 52530, 52532, 52536, 52547, 52549, 52550, 52553, 52554, 52556, 52561, 52562, 52564, 52568, 
    52577, 52578, 52580, 52584, 52592, 52611, 52613, 52614, 52617, 52618, 52620, 52625, 52626, 52628, 52632, 52641, 52642, 52644, 52648, 52656, 
    52673, 52674, 52676, 52680, 52688, 52704, 52743, 52747, 52749, 52750, 52755, 52757, 52758, 52761, 52762, 52764, 52771, 52773, 52774, 52777, 
    52778, 52780, 52785, 52786, 52788, 52792, 52803, 52805, 52806, 52809, 52810, 52812, 52817, 52818, 52820, 52824, 52833, 52834, 52836, 52840, 
    52848, 52867, 52869, 52870, 52873, 52874, 52876, 52881, 52882, 52884, 52888, 52897, 52898, 52900, 52904, 52912, 52929, 52930, 52932, 52936, 
    52944, 52960, 52995, 52997, 52998, 53001, 53002, 53004, 53009, 53010, 53012, 53016, 53025, 53026, 53028, 53032, 53040, 53057, 53058, 53060, 
    53064, 53072, 53088, 53121, 53122, 53124, 53128, 53136, 53152, 53184, 53279, 53295, 53303, 53307, 53309, 53310, 53327, 53335, 53339, 53341, 
    53342, 53351, 53355, 53357, 53358, 53363, 53365, 53366, 53369, 53370, 53372, 53391, 53399, 53403, 53405, 53406, 53415, 53419, 53421, 53422, 
    53427, 53429, 53430, 53433, 53434, 53436, 53447, 53451, 53453, 53454, 53459, 53461, 53462, 53465, 53466, 53468, 53475, 53477, 53478, 53481, 
    53482, 53484, 53489, 53490, 53492, 53496, 53519, 53527, 53531, 53533, 53534, 53543, 53547, 53549, 53550, 53555, 53557, 53558, 53561, 53562, 
    53564, 53575, 53579, 53581, 53582, 53587, 53589, 53590, 53593, 53594, 53596, 53603, 53605, 53606, 53609, 53610, 53612, 53617, 53618, 53620, 
    53624, 53639, 53643, 53645, 53646, 53651, 53653, 53654, 53657, 53658, 53660, 53667, 53669, 53670, 53673, 53674, 53676, 53681, 53682, 53684, 
    53688, 53699, 53701, 53702, 53705, 53706, 53708, 53713, 53714, 53716, 53720, 53729, 53730, 53732, 53736, 53744, 53775, 53783, 53787, 53789, 
    53790, 53799, 53803, 53805, 53806, 53811, 53813, 53814, 53817, 53818, 53820, 53831, 53835, 53837, 53838, 53843, 53845, 53846, 53849, 53850, 
    53852, 53859, 53861, 53862, 53865, 53866, 53868, 53873, 53874, 53876, 53880, 53895, 53899, 53901, 53902, 53907, 53909, 53910, 53913, 53914, 
    53916, 53923, 53925, 53926, 53929, 53930, 53932, 53937, 53938, 53940, 53944, 53955, 53957, 53958, 53961, 53962, 53964, 53969, 53970, 53972, 
    53976, 53985, 53986, 53988, 53992, 54000, 54023, 54027, 54029, 54030, 54035, 54037, 54038, 54041, 54042, 54044, 54051, 54053, 54054, 54057, 
    54058, 54060, 54065, 54066, 54068, 54072, 54083, 54085, 54086, 54089, 54090, 54092, 54097, 54098, 54100, 54104, 54113, 54114, 54116, 54120, 
    54128, 54147, 54149, 54150, 54153, 54154, 54156, 54161, 54162, 54164, 54168, 54177, 54178, 54180, 54184, 54192, 54209, 54210, 54212, 54216, 
    54224, 54240, 54287, 54295, 54299, 54301, 54302, 54311, 54315, 54317, 54318, 54323, 54325, 54326, 54329, 54330, 54332, 54343, 54347, 54349, 
    54350, 54355, 54357, 54358, 54361, 54362, 54364, 54371, 54373, 54374, 54377, 54378, 54380, 54385, 54386, 54388, 54392, 54407, 54411, 54413, 
    54414, 54419, 54421, 54422, 54425, 54426, 54428, 54435, 54437, 54438, 54441, 54442, 54444, 54449, 54450, 54452, 54456, 54467, 54469, 54470, 
    54473, 54474, 54476, 54481, 54482, 54484, 54488, 54497, 54498, 54500, 54504, 54512, 54535, 54539, 54541, 54542, 54547, 54549, 54550, 54553, 
    54554, 54556, 54563, 54565, 54566, 54569, 54570, 54572, 54577, 54578, 54580, 54584, 54595, 54597, 54598, 54601, 54602, 54604, 54609, 54610, 
    54612, 54616, 54625, 54626, 54628, 54632, 54640, 54659, 54661, 54662, 54665, 54666, 54668, 54673, 54674, 54676, 54680, 54689, 54690, 54692, 
    54696, 54704, 54721, 54722, 54724, 54728, 54736, 54752, 54791, 54795, 54797, 54798, 54803, 54805, 54806, 54809, 54810, 54812, 54819, 54821, 
    54822, 54825, 54826, 54828, 54833, 54834, 54836, 54840, 54851, 54853, 54854, 54857, 54858, 54860, 54865, 54866, 54868, 54872, 54881, 54882, 
    54884, 54888, 54896, 54915, 54917, 54918, 54921, 54922, 54924, 54929, 54930, 54932, 54936, 54945, 54946, 54948, 54952, 54960, 54977, 54978, 
    54980, 54984, 54992, 55008, 55043, 55045, 55046, 55049, 55050, 55052, 55057, 55058, 55060, 55064, 55073, 55074, 55076, 55080, 55088, 55105, 
    55106, 55108, 55112, 55120, 55136, 55169, 55170, 55172, 55176, 55184, 55200, 55232, 55311, 55319, 55323, 55325, 55326, 55335, 55339, 55341, 
    55342, 55347, 55349, 55350, 55353, 55354, 55356, 55367, 55371, 55373, 55374, 55379, 55381, 55382, 55385, 55386, 55388, 55395, 55397, 55398, 
    55401, 55402, 55404, 55409, 55410, 55412, 55416, 55431, 55435, 55437, 55438, 55443, 55445, 55446, 55449, 55450, 55452, 55459, 55461, 55462, 
    55465, 55466, 55468, 55473, 55474, 55476, 55480, 55491, 55493, 55494, 55497, 55498, 55500, 55505, 55506, 55508, 55512, 55521, 55522, 55524, 
    55528, 55536, 55559, 55563, 55565, 55566, 55571, 55573, 55574, 55577, 55578, 55580, 55587, 55589, 55590, 55593, 55594, 55596, 55601, 55602, 
    55604, 55608, 55619, 55621, 55622, 55625, 55626, 55628, 55633, 55634, 55636, 55640, 55649, 55650, 55652, 55656, 55664, 55683, 55685, 55686, 
    55689, 55690, 55692, 55697, 55698, 55700, 55704, 55713, 55714, 55716, 55720, 55728, 55745, 55746, 55748, 55752, 55760, 55776, 55815, 55819, 
    55821, 55822, 55827, 55829, 55830, 55833, 55834, 55836, 55843, 55845, 55846, 55849, 55850, 55852, 55857, 55858, 55860, 55864, 55875, 55877, 
    55878, 55881, 55882, 55884, 55889, 55890, 55892, 55896, 55905, 55906, 55908, 55912, 55920, 55939, 55941, 55942, 55945, 55946, 55948, 55953, 
    55954, 55956, 55960, 55969, 55970, 55972, 55976, 55984, 56001, 56002, 56004, 56008, 56016, 56032, 56067, 56069, 56070, 56073, 56074, 56076, 
    56081, 56082, 56084, 56088, 56097, 56098, 56100, 56104, 56112, 56129, 56130, 56132, 56136, 56144, 56160, 56193, 56194, 56196, 56200, 56208, 
    56224, 56256, 56327, 56331, 56333, 56334, 56339, 56341, 56342, 56345, 56346, 56348, 56355, 56357, 56358, 56361, 56362, 56364, 56369, 56370, 
    56372, 56376, 56387, 56389, 56390, 56393, 56394, 56396, 56401, 56402, 56404, 56408, 56417, 56418, 56420, 56424, 56432, 56451, 56453, 56454, 
    56457, 56458, 56460, 56465, 56466, 56468, 56472, 56481, 56482, 56484, 56488, 56496, 56513, 56514, 56516, 56520, 56528, 56544, 56579, 56581, 
    56582, 56585, 56586, 56588, 56593, 56594, 56596, 56600, 56609, 56610, 56612, 56616, 56624, 56641, 56642, 56644, 56648, 56656, 56672, 56705, 
    56706, 56708, 56712, 56720, 56736, 56768, 56835, 56837, 56838, 56841, 56842, 56844, 56849, 56850, 56852, 56856, 56865, 56866, 56868, 56872, 
    56880, 56897, 56898, 56900, 56904, 56912, 56928, 56961, 56962, 56964, 56968, 56976, 56992, 57024, 57089, 57090, 57092, 57096, 57104, 57120, 
    57152, 57216, 57375, 57391, 57399, 57403, 57405, 57406, 57423, 57431, 57435, 57437, 57438, 57447, 57451, 57453, 57454, 57459, 57461, 57462, 
    57465, 57466, 57468, 57487, 57495, 57499, 57501, 57502, 57511, 57515, 57517, 57518, 57523, 57525, 57526, 57529, 57530, 57532, 57543, 57547, 
    57549, 57550, 57555, 57557, 57558, 57561, 57562, 57564, 57571, 57573, 57574, 57577, 57578, 57580, 57585, 57586, 57588, 57592, 57615, 57623, 
    57627, 57629, 57630, 57639, 57643, 57645, 57646, 57651, 57653, 57654, 57657, 57658, 57660, 57671, 57675, 57677, 57678, 57683, 57685, 57686, 
    57689, 57690, 57692, 57699, 57701, 57702, 57705, 57706, 57708, 57713, 57714, 57716, 57720, 57735, 57739, 57741, 57742, 57747, 57749, 57750, 
    57753, 57754, 57756, 57763, 57765, 57766, 57769, 57770, 57772, 57777, 57778, 57780, 57784, 57795, 57797, 57798, 57801, 57802, 57804, 57809, 
    57810, 57812, 57816, 57825, 57826, 57828, 57832, 57840, 57871, 57879, 57883, 57885, 57886, 57895, 57899, 57901, 57902, 57907, 57909, 57910, 
    57913, 57914, 57916, 57927, 57931, 57933, 57934, 57939, 57941, 57942, 57945, 57946, 57948, 57955, 57957, 57958, 57961, 57962, 57964, 57969, 
    57970, 57972, 57976, 57991, 57995, 57997, 57998, 58003, 58005, 58006, 58009, 58010, 58012, 58019, 58021, 58022, 58025, 58026, 58028, 58033, 
    58034, 58036, 58040, 58051, 58053, 58054, 58057, 58058, 58060, 58065, 58066, 58068, 58072, 58081, 58082, 58084, 58088, 58096, 58119, 58123, 
    58125, 58126, 58131, 58133, 58134, 58137, 58138, 58140, 58147, 58149, 58150, 58153, 58154, 58156, 58161, 58162, 58164, 58168, 58179, 58181, 
    58182, 58185, 58186, 58188, 58193, 58194, 58196, 58200, 58209, 58210, 58212, 58216, 58224, 58243, 58245, 58246, 58249, 58250, 58252, 58257, 
    58258, 58260, 58264, 58273, 58274, 58276, 58280, 58288, 58305, 58306, 58308, 58312, 58320, 58336, 58383, 58391, 58395, 58397, 58398, 58407, 
    58411, 58413, 58414, 58419, 58421, 58422, 58425, 58426, 58428, 58439, 58443, 58445, 58446, 58451, 58453, 58454, 58457, 58458, 58460, 58467, 
    58469, 58470, 58473, 58474, 58476, 58481, 58482, 58484, 58488, 58503, 58507, 58509, 58510, 58515, 58517, 58518, 58521, 58522, 58524, 58531, 
    58533, 58534, 58537, 58538, 58540, 58545, 58546, 58548, 58552, 58563, 58565, 58566, 58569, 58570, 58572, 58577, 58578, 58580, 58584, 58593, 
    58594, 58596, 58600, 58608, 58631, 58635, 58637, 58638, 58643, 58645, 58646, 58649, 58650, 58652, 58659, 58661, 58662, 58665, 58666, 58668, 
    58673, 58674, 58676, 58680, 58691, 58693, 58694, 58697, 58698, 58700, 58705, 58706, 58708, 58712, 58721, 58722, 58724, 58728, 58736, 58755, 
    58757, 58758, 58761, 58762, 58764, 58769, 58770, 58772, 58776, 58785, 58786, 58788, 58792, 58800, 58817, 58818, 58820, 58824, 58832, 58848, 
    58887, 58891, 58893, 58894, 58899, 58901, 58902, 58905, 58906, 58908, 58915, 58917, 58918, 58921, 58922, 58924, 58929, 58930, 58932, 58936, 
    58947, 58949, 58950, 58953, 58954, 58956, 58961, 58962, 58964, 58968, 58977, 58978, 58980, 58984, 58992, 59011, 59013, 59014, 59017, 59018, 
    59020, 59025, 59026, 59028, 59032, 59041, 59042, 59044, 59048, 59056, 59073, 59074, 59076, 59080, 59088, 59104, 59139, 59141, 59142, 59145, 
    59146, 59148, 59153, 59154, 59156, 59160, 59169, 59170, 59172, 59176, 59184, 59201, 59202, 59204, 59208, 59216, 59232, 59265, 59266, 59268, 
    59272, 59280, 59296, 59328, 59407, 59415, 59419, 59421, 59422, 59431, 59435, 59437, 59438, 59443, 59445, 59446, 59449, 59450, 59452, 59463, 
    59467, 59469, 59470, 59475, 59477, 59478, 59481, 59482, 59484, 59491, 59493, 59494, 59497, 59498, 59500, 59505, 59506, 59508, 59512, 59527, 
    59531, 59533, 59534, 59539, 59541, 59542, 59545, 59546, 59548, 59555, 59557, 59558, 59561, 59562, 59564, 59569, 59570, 59572, 59576, 59587, 
    59589, 59590, 59593, 59594, 59596, 59601, 59602, 59604, 59608, 59617, 59618, 59620, 59624, 59632, 59655, 59659, 59661, 59662, 59667, 59669, 
    59670, 59673, 59674, 59676, 59683, 59685, 59686, 59689, 59690, 59692, 59697, 59698, 59700, 59704, 59715, 59717, 59718, 59721, 59722, 59724, 
    59729, 59730, 59732, 59736, 59745, 59746, 59748, 59752, 59760, 59779, 59781, 59782, 59785, 59786, 59788, 59793, 59794, 59796, 59800, 59809, 
    59810, 59812, 59816, 59824, 59841, 59842, 59844, 59848, 59856, 59872, 59911, 59915, 59917, 59918, 59923, 59925, 59926, 59929, 59930, 59932, 
    59939, 59941, 59942, 59945, 59946, 59948, 59953, 59954, 59956, 59960, 59971, 59973, 59974, 59977, 59978, 59980, 59985, 59986, 59988, 59992, 
    60001, 60002, 60004, 60008, 60016, 60035, 60037, 60038, 60041, 60042, 60044, 60049, 60050, 60052, 60056, 60065, 60066, 60068, 60072, 60080, 
    60097, 60098, 60100, 60104, 60112, 60128, 60163, 60165, 60166, 60169, 60170, 60172, 60177, 60178, 60180, 60184, 60193, 60194, 60196, 60200, 
    60208, 60225, 60226, 60228, 60232, 60240, 60256, 60289, 60290, 60292, 60296, 60304, 60320, 60352, 60423, 60427, 60429, 60430, 60435, 60437, 
    60438, 60441, 60442, 60444, 60451, 60453, 60454, 60457, 60458, 60460, 60465, 60466, 60468, 60472, 60483, 60485, 60486, 60489, 60490, 60492, 
    60497, 60498, 60500, 60504, 60513, 60514, 60516, 60520, 60528, 60547, 60549, 60550, 60553, 60554, 60556, 60561, 60562, 60564, 60568, 60577, 
    60578, 60580, 60584, 60592, 60609, 60610, 60612, 60616, 60624, 60640, 60675, 60677, 60678, 60681, 60682, 60684, 60689, 60690, 60692, 60696, 
    60705, 60706, 60708, 60712, 60720, 60737, 60738, 60740, 60744, 60752, 60768, 60801, 60802, 60804, 60808, 60816, 60832, 60864, 60931, 60933, 
    60934, 60937, 60938, 60940, 60945, 60946, 60948, 60952, 60961, 60962, 60964, 60968, 60976, 60993, 60994, 60996, 61000, 61008, 61024, 61057, 
    61058, 61060, 61064, 61072, 61088, 61120, 61185, 61186, 61188, 61192, 61200, 61216, 61248, 61312, 61455, 61463, 61467, 61469, 61470, 61479, 
    61483, 61485, 61486, 61491, 61493, 61494, 61497, 61498, 61500, 61511, 61515, 61517, 61518, 61523, 61525, 61526, 61529, 61530, 61532, 61539, 
    61541, 61542, 61545, 61546, 61548, 61553, 61554, 61556, 61560, 61575, 61579, 61581, 61582, 61587, 61589, 61590, 61593, 61594, 61596, 61603, 
    61605, 61606, 61609, 61610, 61612, 61617, 61618, 61620, 61624, 61635, 61637, 61638, 61641, 61642, 61644, 61649, 61650, 61652, 61656, 61665, 
    61666, 61668, 61672, 61680, 61703, 61707, 61709, 61710, 61715, 61717, 61718, 61721, 61722, 61724, 61731, 61733, 61734, 61737, 61738, 61740, 
    61745, 61746, 61748, 61752, 61763, 61765, 61766, 61769, 61770, 61772, 61777, 61778, 61780, 61784, 61793, 61794, 61796, 61800, 61808, 61827, 
    61829, 61830, 61833, 61834, 61836, 61841, 61842, 61844, 61848, 61857, 61858, 61860, 61864, 61872, 61889, 61890, 61892, 61896, 61904, 61920, 
    61959, 61963, 61965, 61966, 61971, 61973, 61974, 61977, 61978, 61980, 61987, 61989, 61990, 61993, 61994, 61996, 62001, 62002, 62004, 62008, 
    62019, 62021, 62022, 62025, 62026, 62028, 62033, 62034, 62036, 62040, 62049, 62050, 62052, 62056, 62064, 62083, 62085, 62086, 62089, 62090, 
    62092, 62097, 62098, 62100, 62104, 62113, 62114, 62116, 62120, 62128, 62145, 62146, 62148, 62152, 62160, 62176, 62211, 62213, 62214, 62217, 
    62218, 62220, 62225, 62226, 62228, 62232, 62241, 62242, 62244, 62248, 62256, 62273, 62274, 62276, 62280, 62288, 62304, 62337, 62338, 62340, 
    62344, 62352, 62368, 62400, 62471, 62475, 62477, 62478, 62483, 62485, 62486, 62489, 62490, 62492, 62499, 62501, 62502, 62505, 62506, 62508, 
    62513, 62514, 62516, 62520, 62531, 62533, 62534, 62537, 62538, 62540, 62545, 62546, 62548, 62552, 62561, 62562, 62564, 62568, 62576, 62595, 
    62597, 62598, 62601, 62602, 62604, 62609, 62610, 62612, 62616, 62625, 62626, 62628, 62632, 62640, 62657, 62658, 62660, 62664, 62672, 62688, 
    62723, 62725, 62726, 62729, 62730, 62732, 62737, 62738, 62740, 62744, 62753, 62754, 62756, 62760, 62768, 62785, 62786, 62788, 62792, 62800, 
    62816, 62849, 62850, 62852, 62856, 62864, 62880, 62912, 62979, 62981, 62982, 62985, 62986, 62988, 62993, 62994, 62996, 63000, 63009, 63010, 
    63012, 63016, 63024, 63041, 63042, 63044, 63048, 63056, 63072, 63105, 63106, 63108, 63112, 63120, 63136, 63168, 63233, 63234, 63236, 63240, 
    63248, 63264, 63296, 63360, 63495, 63499, 63501, 63502, 63507, 63509, 63510, 63513, 63514, 63516, 63523, 63525, 63526, 63529, 63530, 63532, 
    63537, 63538, 63540, 63544, 63555, 63557, 63558, 63561, 63562, 63564, 63569, 63570, 63572, 63576, 63585, 63586, 63588, 63592, 63600, 63619, 
    63621, 63622, 63625, 63626, 63628, 63633, 63634, 63636, 63640, 63649, 63650, 63652, 63656, 63664, 63681, 63682, 63684, 63688, 63696, 63712, 
    63747, 63749, 63750, 63753, 63754, 63756, 63761, 63762, 63764, 63768, 63777, 63778, 63780, 63784, 63792, 63809, 63810, 63812, 63816, 63824, 
    63840, 63873, 63874, 63876, 63880, 63888, 63904, 63936, 64003, 64005, 64006, 64009, 64010, 64012, 64017, 64018, 64020, 64024, 64033, 64034, 
    64036, 64040, 64048, 64065, 64066, 64068, 64072, 64080, 64096, 64129, 64130, 64132, 64136, 64144, 64160, 64192, 64257, 64258, 64260, 64264, 
    64272, 64288, 64320, 64384, 64515, 64517, 64518, 64521, 64522, 64524, 64529, 64530, 64532, 64536, 64545, 64546, 64548, 64552, 64560, 64577, 
    64578, 64580, 64584, 64592, 64608, 64641, 64642, 64644, 64648, 64656, 64672, 64704, 64769, 64770, 64772, 64776, 64784, 64800, 64832, 64896, 
    65025, 65026, 65028, 65032, 65040, 65056, 65088, 65152, 65280, },
  { 511, 767, 895, 959, 991, 1007, 1015, 1019, 1021, 1022, 1279, 1407, 1471, 1503, 1519, 1527, 1531, 1533, 1534, 1663, 1727, 
    1759, 1775, 1783, 1787, 1789, 1790, 1855, 1887, 1903, 1911, 1915, 1917, 1918, 1951, 1967, 1975, 1979, 1981, 1982, 1999, 
    2007, 2011, 2013, 2014, 2023, 2027, 2029, 2030, 2035, 2037, 2038, 2041, 2042, 2044, 2303, 2431, 2495, 2527, 2543, 2551, 
    2555, 2557, 2558, 2687, 2751, 2783, 2799, 2807, 2811, 2813, 2814, 2879, 2911, 2927, 2935, 2939, 2941, 2942, 2975, 2991, 
    2999, 3003, 3005, 3006, 3023, 3031, 3035, 3037, 3038, 3047, 3051, 3053, 3054, 3059, 3061, 3062, 3065, 3066, 3068, 3199, 
    3263, 3295, 3311, 3319, 3323, 3325, 3326, 3391, 3423, 3439, 3447, 3451, 3453, 3454, 3487, 3503, 3511, 3515, 3517, 3518, 
    3535, 3543, 3547, 3549, 3550, 3559, 3563, 3565, 3566, 3571, 3573, 3574, 3577, 3578, 3580, 3647, 3679, 3695, 3703, 3707, 
    3709, 3710, 3743, 3759, 3767, 3771, 3773, 3774, 3791, 3799, 3803, 3805, 3806, 3815, 3819, 3821, 3822, 3827, 3829, 3830, 
    3833, 3834, 3836, 3871, 3887, 3895, 3899, 3901, 3902, 3919, 3927, 3931, 3933, 3934, 3943, 3947, 3949, 3950, 3955, 3957, 
    3958, 3961, 3962, 3964, 3983, 3991, 3995, 3997, 3998, 4007, 4011, 4013, 4014, 4019, 4021, 4022, 4025, 4026, 4028, 4039, 
    4043, 4045, 4046, 4051, 4053, 4054, 4057, 4058, 4060, 4067, 4069, 4070, 4073, 4074, 4076, 4081, 4082, 4084, 4088, 4351, 
    4479, 4543, 4575, 4591, 4599, 4603, 4605, 4606, 4735, 4799, 4831, 4847, 4855, 4859, 4861, 4862, 4927, 4959, 4975, 4983, 
    4987, 4989, 4990, 5023, 5039, 5047, 5051, 5053, 5054, 5071, 5079, 5083, 5085, 5086, 5095, 5099, 5101, 5102, 5107, 5109, 
    5110, 5113, 5114, 5116, 5247, 5311, 5343, 5359, 5367, 5371, 5373, 5374, 5439, 5471, 5487, 5495, 5499, 5501, 5502, 5535, 
    5551, 5559, 5563, 5565, 5566, 5583, 5591, 5595, 5597, 5598, 5607, 5611, 5613, 5614, 5619, 5621, 5622, 5625, 5626, 5628, 
    5695, 5727, 5743, 5751, 5755, 5757, 5758, 5791, 5807, 5815, 5819, 5821, 5822, 5839, 5847, 5851, 5853, 5854, 5863, 5867, 
    5869, 5870, 5875, 5877, 5878, 5881, 5882, 5884, 5919, 5935, 5943, 5947, 5949, 5950, 5967, 5975, 5979, 5981, 5982, 5991, 
    5995, 5997, 5998, 6003, 6005, 6006, 6009, 6010, 6012, 6031, 6039, 6043, 6045, 6046, 6055, 6059, 6061, 6062, 6067, 6069, 
    6070, 6073, 6074, 6076, 6087, 6091, 6093, 6094, 6099, 6101, 6102, 6105, 6106, 6108, 6115, 6117, 6118, 6121, 6122, 6124, 
    6129, 6130, 6132, 6136, 6271, 6335, 6367, 6383, 6391, 6395, 6397, 6398, 6463, 6495, 6511, 6519, 6523, 6525, 6526, 6559, 
    6575, 6583, 6587, 6589, 6590, 6607, 6615, 6619, 6621, 6622, 6631, 6635, 6637, 6638, 6643, 6645, 6646, 6649, 6650, 6652, 
    6719, 6751, 6767, 6775, 6779, 6781, 6782, 6815, 6831, 6839, 6843, 6845, 6846, 6863, 6871, 6875, 6877, 6878, 6887, 6891, 
    6893, 6894, 6899, 6901, 6902, 6905, 6906, 6908, 6943, 6959, 6967, 6971, 6973, 6974, 6991, 6999, 7003, 7005, 7006, 7015, 
    7019, 7021, 7022, 7027, 7029, 7030, 7033, 7034, 7036, 7055, 7063, 7067, 7069, 7070, 7079, 7083, 7085, 7086, 7091, 7093, 
    7094, 7097, 7098, 7100, 7111, 7115, 7117, 7118, 7123, 7125, 7126, 7129, 7130, 7132, 7139, 7141, 7142, 7145, 7146, 7148, 
    7153, 7154, 7156, 7160, 7231, 7263, 7279, 7287, 7291, 7293, 7294, 7327, 7343, 7351, 7355, 7357, 7358, 7375, 7383, 7387, 
    7389, 7390, 7399, 7403, 7405, 7406, 7411, 7413, 7414, 7417, 7418, 7420, 7455, 7471, 7479, 7483, 7485, 7486, 7503, 7511, 
    7515, 7517, 7518, 7527, 7531, 7533, 7534, 7539, 7541, 7542, 7545, 7546, 7548, 7567, 7575, 7579, 7581, 7582, 7591, 7595, 
    7597, 7598, 7603, 7605, 7606, 7609, 7610, 7612, 7623, 7627, 7629, 7630, 7635, 7637, 7638, 7641, 7642, 7644, 7651, 7653, 
    7654, 7657, 7658, 7660, 7665, 7666, 7668, 7672, 7711, 7727, 7735, 7739, 7741, 7742, 7759, 7767, 7771, 7773, 7774, 7783, 
    7787, 7789, 7790, 7795, 7797, 7798, 7801, 7802, 7804, 7823, 7831, 7835, 7837, 7838, 7847, 7851, 7853, 7854, 7859, 7861, 
    7862, 7865, 7866, 7868, 7879, 7883, 7885, 7886, 7891, 7893, 7894, 7897, 7898, 7900, 7907, 7909, 7910, 7913, 7914, 7916, 
    7921, 7922, 7924, 7928, 7951, 7959, 7963, 7965, 7966, 7975, 7979, 7981, 7982, 7987, 7989, 7990, 7993, 7994, 7996, 8007, 
    8011, 8013, 8014, 8019, 8021, 8022, 8025, 8026, 8028, 8035, 8037, 8038, 8041, 8042, 8044, 8049, 8050, 8052, 8056, 8071, 
    8075, 8077, 8078, 8083, 8085, 8086, 8089, 8090, 8092, 8099, 8101, 8102, 8105, 8106, 8108, 8113, 8114, 8116, 8120, 8131, 
    8133, 8134, 8137, 8138, 8140, 8145, 8146, 8148, 8152, 8161, 8162, 8164, 8168, 8176, 8447, 8575, 8639, 8671, 8687, 8695, 
    8699, 8701, 8702, 8831, 8895, 8927, 8943, 8951, 8955, 8957, 8958, 9023, 9055, 9071, 9079, 9083, 9085, 9086, 9119, 9135, 
    9143, 9147, 9149, 9150, 9167, 9175, 9179, 9181, 9182, 9191, 9195, 9197, 9198, 9203, 9205, 9206, 9209, 9210, 9212, 9343, 
    9407, 9439, 9455, 9463, 9467, 9469, 9470, 9535, 9567, 9583, 9591, 9595, 9597, 9598, 9631, 9647, 9655, 9659, 9661, 9662, 
    9679, 9687, 9691, 9693, 9694, 9703, 9707, 9709, 9710, 9715, 9717, 9718, 9721, 9722, 9724, 9791, 9823, 9839, 9847, 9851, 
    9853, 9854, 9887, 9903, 9911, 9915, 9917, 9918, 9935, 9943, 9947, 9949, 9950, 9959, 9963, 9965, 9966, 9971, 9973, 9974, 
    9977, 9978, 9980, 10015, 10031, 10039, 10043, 10045, 10046, 10063, 10071, 10075, 10077, 10078, 10087, 10091, 10093, 10094, 10099, 10101, 
    10102, 10105, 10106, 10108, 10127, 10135, 10139, 10141, 10142, 10151, 10155, 10157, 10158, 10163, 10165, 10166, 10169, 10170, 10172, 10183, 
    10187, 10189, 10190, 10195, 10197, 10198, 10201, 10202, 10204, 10211, 10213, 10214, 10217, 10218, 10220, 10225, 10226, 10228, 10232, 10367, 
    10431, 10463, 10479, 10487, 10491, 10493, 10494, 10559, 10591, 10607, 10615, 10619, 10621, 10622, 10655, 10671, 10679, 10683, 10685, 10686, 
    10703, 10711, 10715, 10717, 10718, 10727, 10731, 10733, 10734, 10739, 10741, 10742, 10745, 10746, 10748, 10815, 10847, 10863, 10871, 10875, 
    10877, 10878, 10911, 10927, 10935, 10939, 10941, 10942, 10959, 10967, 10971, 10973, 10974, 10983, 10987, 10989, 10990, 10995, 10997, 10998, 
    11001, 11002, 11004, 11039, 11055, 11063, 11067, 11069, 11070, 11087, 11095, 11099, 11101, 11102, 11111, 11115, 11117, 11118, 11123, 11125, 
    11126, 11129, 11130, 11132, 11151, 11159, 11163, 11165, 11166, 11175, 11179, 11181, 11182, 11187, 11189, 11190, 11193, 11194, 11196, 11207, 
    11211, 11213, 11214, 11219, 11221, 11222, 11225, 11226, 11228, 11235, 11237, 11238, 11241, 11242, 11244, 11249, 11250, 11252, 11256, 11327, 
    11359, 11375, 11383, 11387, 11389, 11390, 11423, 11439, 11447, 11451, 11453, 11454, 11471, 11479, 11483, 11485, 11486, 11495, 11499, 11501, 
    11502, 11507, 11509, 11510, 11513, 11514, 11516, 11551, 11567, 11575, 11579, 11581, 11582, 11599, 11607, 11611, 11613, 11614, 11623, 11627, 
    11629, 11630, 11635, 11637, 11638, 11641, 11642, 11644, 11663, 11671, 11675, 11677, 11678, 11687, 11691, 11693, 11694, 11699, 11701, 11702, 
    11705, 11706, 11708, 11719, 11723, 11725, 11726, 11731, 11733, 11734, 11737, 11738, 11740, 11747, 11749, 11750, 11753, 11754, 11756, 11761, 
    11762, 11764, 11768, 11807, 11823, 11831, 11835, 11837, 11838, 11855, 11863, 11867, 11869, 11870, 11879, 11883, 11885, 11886, 11891, 11893, 
    11894, 11897, 11898, 11900, 11919, 11927, 11931, 11933, 11934, 11943, 11947, 11949, 11950, 11955, 11957, 11958, 11961, 11962, 11964, 11975, 
    11979, 11981, 11982, 11987, 11989, 11990, 11993, 11994, 11996, 12003, 12005, 12006, 12009, 12010, 12012, 12017, 12018, 12020, 12024, 12047, 
    12055, 12059, 12061, 12062, 12071, 12075, 12077, 12078, 12083, 12085, 12086, 12089, 12090, 12092, 12103, 12107, 12109, 12110, 12115, 12117, 
    12118, 12121, 12122, 12124, 12131, 12133, 12134, 12137, 12138, 12140, 12145, 12146, 12148, 12152, 12167, 12171, 12173, 12174, 12179, 12181, 
    12182, 12185, 12186, 12188, 12195, 12197, 12198, 12201, 12202, 12204, 12209, 12210, 12212, 12216, 12227, 12229, 12230, 12233, 12234, 12236, 
    12241, 12242, 12244, 12248, 12257, 12258, 12260, 12264, 12272, 12415, 12479, 12511, 12527, 12535, 12539, 12541, 12542, 12607, 12639, 12655, 
    12663, 12667, 12669, 12670, 12703, 12719, 12727, 12731, 12733, 12734, 12751, 12759, 12763, 12765, 12766, 12775, 12779, 12781, 12782, 12787, 
    12789, 12790, 12793, 12794, 12796, 12863, 12895, 12911, 12919, 12923, 12925, 12926, 12959, 12975, 12983, 12987, 12989, 12990, 13007, 13015, 
    13019, 13021, 13022, 13031, 13035, 13037, 13038, 13043, 13045, 13046, 13049, 13050, 13052, 13087, 13103, 13111, 13115, 13117, 13118, 13135, 
    13143, 13147, 13149, 13150, 13159, 13163, 13165, 13166, 13171, 13173, 13174, 13177, 13178, 13180, 13199, 13207, 13211, 13213, 13214, 13223, 
    13227, 13229, 13230, 13235, 13237, 13238, 13241, 13242, 13244, 13255, 13259, 13261, 13262, 13267, 13269, 13270, 13273, 13274, 13276, 13283, 
    13285, 13286, 13289, 13290, 13292, 13297, 13298, 13300, 13304, 13375, 13407, 13423, 13431, 13435, 13437, 13438, 13471, 13487, 13495, 13499, 
    13501, 13502, 13519, 13527, 13531, 13533, 13534, 13543, 13547, 13549, 13550, 13555, 13557, 13558, 13561, 13562, 13564, 13599, 13615, 13623, 
    13627, 13629, 13630, 13647, 13655, 13659, 13661, 13662, 13671, 13675, 13677, 13678, 13683, 13685, 13686, 13689, 13690, 13692, 13711, 13719, 
    13723, 13725, 13726, 13735, 13739, 13741, 13742, 13747, 13749, 13750, 13753, 13754, 13756, 13767, 13771, 13773, 13774, 13779, 13781, 13782, 
    13785, 13786, 13788, 13795, 13797, 13798, 13801, 13802, 13804, 13809, 13810, 13812, 13816, 13855, 13871, 13879, 13883, 13885, 13886, 13903, 
    13911, 13915, 13917, 13918, 13927, 13931, 13933, 13934, 13939, 13941, 13942, 13945, 13946, 13948, 13967, 13975, 13979, 13981, 13982, 13991, 
    13995, 13997, 13998, 14003, 14005, 14006, 14009, 14010, 14012, 14023, 14027, 14029, 14030, 14035, 14037, 14038, 14041, 14042, 14044, 14051, 
    14053, 14054, 14057, 14058, 14060, 14065, 14066, 14068, 14072, 14095, 14103, 14107, 14109, 14110, 14119, 14123, 14125, 14126, 14131, 14133, 
    14134, 14137, 14138, 14140, 14151, 14155, 14157, 14158, 14163, 14165, 14166, 14169, 14170, 14172, 14179, 14181, 14182, 14185, 14186, 14188, 
    14193, 14194, 14196, 14200, 14215, 14219, 14221, 14222, 14227, 14229, 14230, 14233, 14234, 14236, 14243, 14245, 14246, 14249, 14250, 14252, 
    14257, 14258, 14260, 14264, 14275, 14277, 14278, 14281, 14282, 14284, 14289, 14290, 14292, 14296, 14305, 14306, 14308, 14312, 14320, 14399, 
    14431, 14447, 14455, 14459, 14461, 14462, 14495, 14511, 14519, 14523, 14525, 14526, 14543, 14551, 14555, 14557, 14558, 14567, 14571, 14573, 
    14574, 14579, 14581, 14582, 14585, 14586, 14588, 14623, 14639, 14647, 14651, 14653, 14654, 14671, 14679, 14683, 14685, 14686, 14695, 14699, 
    14701, 14702, 14707, 14709, 14710, 14713, 14714, 14716, 14735, 14743, 14747, 14749, 14750, 14759, 14763, 14765, 14766, 14771, 14773, 14774, 
    14777, 14778, 14780, 14791, 14795, 14797, 14798, 14803, 14805, 14806, 14809, 14810, 14812, 14819, 14821, 14822, 14825, 14826, 14828, 14833, 
    14834, 14836, 14840, 14879, 14895, 14903, 14907, 14909, 14910, 14927, 14935, 14939, 14941, 14942, 14951, 14955, 14957, 14958, 14963, 14965, 
    14966, 14969, 14970, 14972, 14991, 14999, 15003, 15005, 15006, 15015, 15019, 15021, 15022, 15027, 15029, 15030, 15033, 15034, 15036, 15047, 
    15051, 15053, 15054, 15059, 15061, 15062, 15065, 15066, 15068, 15075, 15077, 15078, 15081, 15082, 15084, 15089, 15090, 15092, 15096, 15119, 
    15127, 15131, 15133, 15134, 15143, 15147, 15149, 15150, 15155, 15157, 15158, 15161, 15162, 15164, 15175, 15179, 15181, 15182, 15187, 15189, 
    15190, 15193, 15194, 15196, 15203, 15205, 15206, 15209, 15210, 15212, 15217, 15218, 15220, 15224, 15239, 15243, 15245, 15246, 15251, 15253, 
    15254, 15257, 15258, 15260, 15267, 15269, 15270, 15273, 15274, 15276, 15281, 15282, 15284, 15288, 15299, 15301, 15302, 15305, 15306, 15308, 
    15313, 15314, 15316, 15320, 15329, 15330, 15332, 15336, 15344, 15391, 15407, 15415, 15419, 15421, 15422, 15439, 15447, 15451, 15453, 15454, 
    15463, 15467, 15469, 15470, 15475, 15477, 15478, 15481, 15482, 15484, 15503, 15511, 15515, 15517, 15518, 15527, 15531, 15533, 15534, 15539, 
    15541, 15542, 15545, 15546, 15548, 15559, 15563, 15565, 15566, 15571, 15573, 15574, 15577, 15578, 15580, 15587, 15589, 15590, 15593, 15594, 
    15596, 15601, 15602, 15604, 15608, 15631, 15639, 15643, 15645, 15646, 15655, 15659, 15661, 15662, 15667, 15669, 15670, 15673, 15674, 15676, 
    15687, 15691, 15693, 15694, 15699, 15701, 15702, 15705, 15706, 15708, 15715, 15717, 15718, 15721, 15722, 15724, 15729, 15730, 15732, 15736, 
    15751, 15755, 15757, 15758, 15763, 15765, 15766, 15769, 15770, 15772, 15779, 15781, 15782, 15785, 15786, 15788, 15793, 15794, 15796, 15800, 
    15811, 15813, 15814, 15817, 15818, 15820, 15825, 15826, 15828, 15832, 15841, 15842, 15844, 15848, 15856, 15887, 15895, 15899, 15901, 15902, 
    15911, 15915, 15917, 15918, 15923, 15925, 15926, 15929, 15930, 15932, 15943, 15947, 15949, 15950, 15955, 15957, 15958, 15961, 15962, 15964, 
    15971, 15973, 15974, 15977, 15978, 15980, 15985, 15986, 15988, 15992, 16007, 16011, 16013, 16014, 16019, 16021, 16022, 16025, 16026, 16028, 
    16035, 16037, 16038, 16041, 16042, 16044, 16049, 16050, 16052, 16056, 16067, 16069, 16070, 16073, 16074, 16076, 16081, 16082, 16084, 16088, 
    16097, 16098, 16100, 16104, 16112, 16135, 16139, 16141, 16142, 16147, 16149, 16150, 16153, 16154, 16156, 16163, 16165, 16166, 16169, 16170, 
    16172, 16177, 16178, 16180, 16184, 16195, 16197, 16198, 16201, 16202, 16204, 16209, 16210, 16212, 16216, 16225, 16226, 16228, 16232, 16240, 
    16259, 16261, 16262, 16265, 16266, 16268, 16273, 16274, 16276, 16280, 16289, 16290, 16292, 16296, 16304, 16321, 16322, 16324, 16328, 16336, 
    16352, 16639, 16767, 16831, 16863, 16879, 16887, 16891, 16893, 16894, 17023, 17087, 17119, 17135, 17143, 17147, 17149, 17150, 17215, 17247, 
    17263, 17271, 17275, 17277, 17278, 17311, 17327, 17335, 17339, 17341, 17342, 17359, 17367, 17371, 17373, 17374, 17383, 17387, 17389, 17390, 
    17395, 17397, 17398, 17401, 17402, 17404, 17535, 17599, 17631, 17647, 17655, 17659, 17661, 17662, 17727, 17759, 17775, 17783, 17787, 17789, 
    17790, 17823, 17839, 17847, 17851, 17853, 17854, 17871, 17879, 17883, 17885, 17886, 17895, 17899, 17901, 17902, 17907, 17909, 17910, 17913, 
    17914, 17916, 17983, 18015, 18031, 18039, 18043, 18045, 18046, 18079, 18095, 18103, 18107, 18109, 18110, 18127, 18135, 18139, 18141, 18142, 
    18151, 18155, 18157, 18158, 18163, 18165, 18166, 18169, 18170, 18172, 18207, 18223, 18231, 18235, 18237, 18238, 18255, 18263, 18267, 18269, 
    18270, 18279, 18283, 18285, 18286, 18291, 18293, 18294, 18297, 18298, 18300, 18319, 18327, 18331, 18333, 18334, 18343, 18347, 18349, 18350, 
    18355, 18357, 18358, 18361, 18362, 18364, 18375, 18379, 18381, 18382, 18387, 18389, 18390, 18393, 18394, 18396, 18403, 18405, 18406, 18409, 
    18410, 18412, 18417, 18418, 18420, 18424, 18559, 18623, 18655, 18671, 18679, 18683, 18685, 18686, 18751, 18783, 18799, 18807, 18811, 18813, 
    18814, 18847, 18863, 18871, 18875, 18877, 18878, 18895, 18903, 18907, 18909, 18910, 18919, 18923, 18925, 18926, 18931, 18933, 18934, 18937, 
    18938, 18940, 19007, 19039, 19055, 19063, 19067, 19069, 19070, 19103, 19119, 19127, 19131, 19133, 19134, 19151, 19159, 19163, 19165, 19166, 
    19175, 19179, 19181, 19182, 19187, 19189, 19190, 19193, 19194, 19196, 19231, 19247, 19255, 19259, 19261, 19262, 19279, 19287, 19291, 19293, 
    19294, 19303, 19307, 19309, 19310, 19315, 19317, 19318, 19321, 19322, 19324, 19343, 19351, 19355, 19357, 19358, 19367, 19371, 19373, 19374, 
    19379, 19381, 19382, 19385, 19386, 19388, 19399, 19403, 19405, 19406, 19411, 19413, 19414, 19417, 19418, 19420, 19427, 19429, 19430, 19433, 
    19434, 19436, 19441, 19442, 19444, 19448, 19519, 19551, 19567, 19575, 19579, 19581, 19582, 19615, 19631, 19639, 19643, 19645, 19646, 19663, 
    19671, 19675, 19677, 19678, 19687, 19691, 19693, 19694, 19699, 19701, 19702, 19705, 19706, 19708, 19743, 19759, 19767, 19771, 19773, 19774, 
    19791, 19799, 19803, 19805, 19806, 19815, 19819, 19821, 19822, 19827, 19829, 19830, 19833, 19834, 19836, 19855, 19863, 19867, 19869, 19870, 
    19879, 19883, 19885, 19886, 19891, 19893, 19894, 19897, 19898, 19900, 19911, 19915, 19917, 19918, 19923, 19925, 19926, 19929, 19930, 19932, 
    19939, 19941, 19942, 19945, 19946, 19948, 19953, 19954, 19956, 19960, 19999, 20015, 20023, 20027, 20029, 20030, 20047, 20055, 20059, 20061, 
    20062, 20071, 20075, 20077, 20078, 20083, 20085, 20086, 20089, 20090, 20092, 20111, 20119, 20123, 20125, 20126, 20135, 20139, 20141, 20142, 
    20147, 20149, 20150, 20153, 20154, 20156, 20167, 20171, 20173, 20174, 20179, 20181, 20182, 20185, 20186, 20188, 20195, 20197, 20198, 20201, 
    20202, 20204, 20209, 20210, 20212, 20216, 20239, 20247, 20251, 20253, 20254, 20263, 20267, 20269, 20270, 20275, 20277, 20278, 20281, 20282, 
    20284, 20295, 20299, 20301, 20302, 20307, 20309, 20310, 20313, 20314, 20316, 20323, 20325, 20326, 20329, 20330, 20332, 20337, 20338, 20340, 
    20344, 20359, 20363, 20365, 20366, 20371, 20373, 20374, 20377, 20378, 20380, 20387, 20389, 20390, 20393, 20394, 20396, 20401, 20402, 20404, 
    20408, 20419, 20421, 20422, 20425, 20426, 20428, 20433, 20434, 20436, 20440, 20449, 20450, 20452, 20456, 20464, 20607, 20671, 20703, 20719, 
    20727, 20731, 20733, 20734, 20799, 20831, 20847, 20855, 20859, 20861, 20862, 20895, 20911, 20919, 20923, 20925, 20926, 20943, 20951, 20955, 
    20957, 20958, 20967, 20971, 20973, 20974, 20979, 20981, 20982, 20985, 20986, 20988, 21055, 21087, 21103, 21111, 21115, 21117, 21118, 21151, 
    21167, 21175, 21179, 21181, 21182, 21199, 21207, 21211, 21213, 21214, 21223, 21227, 21229, 21230, 21235, 21237, 21238, 21241, 21242, 21244, 
    21279, 21295, 21303, 21307, 21309, 21310, 21327, 21335, 21339, 21341, 21342, 21351, 21355, 21357, 21358, 21363, 21365, 21366, 21369, 21370, 
    21372, 21391, 21399, 21403, 21405, 21406, 21415, 21419, 21421, 21422, 21427, 21429, 21430, 21433, 21434, 21436, 21447, 21451, 21453, 21454, 
    21459, 21461, 21462, 21465, 21466, 21468, 21475, 21477, 21478, 21481, 21482, 21484, 21489, 21490, 21492, 21496, 21567, 21599, 21615, 21623, 
    21627, 21629, 21630, 21663, 21679, 21687, 21691, 21693, 21694, 21711, 21719, 21723, 21725, 21726, 21735, 21739, 21741, 21742, 21747, 21749, 
    21750, 21753, 21754, 21756, 21791, 21807, 21815, 21819, 21821, 21822, 21839, 21847, 21851, 21853, 21854, 21863, 21867, 21869, 21870, 21875, 
    21877, 21878, 21881, 21882, 21884, 21903, 21911, 21915, 21917, 21918, 21927, 21931, 21933, 21934, 21939, 21941, 21942, 21945, 21946, 21948, 
    21959, 21963, 21965, 21966, 21971, 21973, 21974, 21977, 21978, 21980, 21987, 21989, 21990, 21993, 21994, 21996, 22001, 22002, 22004, 22008, 
    22047, 22063, 22071, 22075, 22077, 22078, 22095, 22103, 22107, 22109, 22110, 22119, 22123, 22125, 22126, 22131, 22133, 22134, 22137, 22138, 
    22140, 22159, 22167, 22171, 22173, 22174, 22183, 22187, 22189, 22190, 22195, 22197, 22198, 22201, 22202, 22204, 22215, 22219, 22221, 22222, 
    22227, 22229, 22230, 22233, 22234, 22236, 22243, 22245, 22246, 22249, 22250, 22252, 22257, 22258, 22260, 22264, 22287, 22295, 22299, 22301, 
    22302, 22311, 22315, 22317, 22318, 22323, 22325, 22326, 22329, 22330, 22332, 22343, 22347, 22349, 22350, 22355, 22357, 22358, 22361, 22362, 
    22364, 22371, 22373, 22374, 22377, 22378, 22380, 22385, 22386, 22388, 22392, 22407, 22411, 22413, 22414, 22419, 22421, 22422, 22425, 22426, 
    22428, 22435, 22437, 22438, 22441, 22442, 22444, 22449, 22450, 22452, 22456, 22467, 22469, 22470, 22473, 22474, 22476, 22481, 22482, 22484, 
    22488, 22497, 22498, 22500, 22504, 22512, 22591, 22623, 22639, 22647, 22651, 22653, 22654, 22687, 22703, 22711, 22715, 22717, 22718, 22735, 
    22743, 22747, 22749, 22750, 22759, 22763, 22765, 22766, 22771, 22773, 22774, 22777, 22778, 22780, 22815, 22831, 22839, 22843, 22845, 22846, 
    22863, 22871, 22875, 22877, 22878, 22887, 22891, 22893, 22894, 22899, 22901, 22902, 22905, 22906, 22908, 22927, 22935, 22939, 22941, 22942, 
    22951, 22955, 22957, 22958, 22963, 22965, 22966, 22969, 22970, 22972, 22983, 22987, 22989, 22990, 22995, 22997, 22998, 23001, 23002, 23004, 
    23011, 23013, 23014, 23017, 23018, 23020, 23025, 23026, 23028, 23032, 23071, 23087, 23095, 23099, 23101, 23102, 23119, 23127, 23131, 23133, 
    23134, 23143, 23147, 23149, 23150, 23155, 23157, 23158, 23161, 23162, 23164, 23183, 23191, 23195, 23197, 23198, 23207, 23211, 23213, 23214, 
    23219, 23221, 23222, 23225, 23226, 23228, 23239, 23243, 23245, 23246, 23251, 23253, 23254, 23257, 23258, 23260, 23267, 23269, 23270, 23273, 
    23274, 23276, 23281, 23282, 23284, 23288, 23311, 23319, 23323, 23325, 23326, 23335, 23339, 23341, 23342, 23347, 23349, 23350, 23353, 23354, 
    23356, 23367, 23371, 23373, 23374, 23379, 23381, 23382, 23385, 23386, 23388, 23395, 23397, 23398, 23401, 23402, 23404, 23409, 23410, 23412, 
    23416, 23431, 23435, 23437, 23438, 23443, 23445, 23446, 23449, 23450, 23452, 23459, 23461, 23462, 23465, 23466, 23468, 23473, 23474, 23476, 
    23480, 23491, 23493, 23494, 23497, 23498, 23500, 23505, 23506, 23508, 23512, 23521, 23522, 23524, 23528, 23536, 23583, 23599, 23607, 23611, 
    23613, 23614, 23631, 23639, 23643, 23645, 23646, 23655, 23659, 23661, 23662, 23667, 23669, 23670, 23673, 23674, 23676, 23695, 23703, 23707, 
    23709, 23710, 23719, 23723, 23725, 23726, 23731, 23733, 23734, 23737, 23738, 23740, 23751, 23755, 23757, 23758, 23763, 23765, 23766, 23769, 
    23770, 23772, 23779, 23781, 23782, 23785, 23786, 23788, 23793, 23794, 23796, 23800, 23823, 23831, 23835, 23837, 23838, 23847, 23851, 23853, 
    23854, 23859, 23861, 23862, 23865, 23866, 23868, 23879, 23883, 23885, 23886, 23891, 23893, 23894, 23897, 23898, 23900, 23907, 23909, 23910, 
    23913, 23914, 23916, 23921, 23922, 23924, 23928, 23943, 23947, 23949, 23950, 23955, 23957, 23958, 23961, 23962, 23964, 23971, 23973, 23974, 
    23977, 23978, 23980, 23985, 23986, 23988, 23992, 24003, 24005, 24006, 24009, 24010, 24012, 24017, 24018, 24020, 24024, 24033, 24034, 24036, 
    24040, 24048, 24079, 24087, 24091, 24093, 24094, 24103, 24107, 24109, 24110, 24115, 24117, 24118, 24121, 24122, 24124, 24135, 24139, 24141, 
    24142, 24147, 24149, 24150, 24153, 24154, 24156, 24163, 24165, 24166, 24169, 24170, 24172, 24177, 24178, 24180, 24184, 24199, 24203, 24205, 
    24206, 24211, 24213, 24214, 24217, 24218, 24220, 24227, 24229, 24230, 24233, 24234, 24236, 24241, 24242, 24244, 24248, 24259, 24261, 24262, 
    24265, 24266, 24268, 24273, 24274, 24276, 24280, 24289, 24290, 24292, 24296, 24304, 24327, 24331, 24333, 24334, 24339, 24341, 24342, 24345, 
    24346, 24348, 24355, 24357, 24358, 24361, 24362, 24364, 24369, 24370, 24372, 24376, 24387, 24389, 24390, 24393, 24394, 24396, 24401, 24402, 
    24404, 24408, 24417, 24418, 24420, 24424, 24432, 24451, 24453, 24454, 24457, 24458, 24460, 24465, 24466, 24468, 24472, 24481, 24482, 24484, 
    24488, 24496, 24513, 24514, 24516, 24520, 24528, 24544, 24703, 24767, 24799, 24815, 24823, 24827, 24829, 24830, 24895, 24927, 24943, 24951, 
    24955, 24957, 24958, 24991, 25007, 25015, 25019, 25021, 25022, 25039, 25047, 25051, 25053, 25054, 25063, 25067, 25069, 25070, 25075, 25077, 
    25078, 25081, 25082, 25084, 25151, 25183, 25199, 25207, 25211, 25213, 25214, 25247, 25263, 25271, 25275, 25277, 25278, 25295, 25303, 25307, 
    25309, 25310, 25319, 25323, 25325, 25326, 25331, 25333, 25334, 25337, 25338, 25340, 25375, 25391, 25399, 25403, 25405, 25406, 25423, 25431, 
    25435, 25437, 25438, 25447, 25451, 25453, 25454, 25459, 25461, 25462, 25465, 25466, 25468, 25487, 25495, 25499, 25501, 25502, 25511, 25515, 
    25517, 25518, 25523, 25525, 25526, 25529, 25530, 25532, 25543, 25547, 25549, 25550, 25555, 25557, 25558, 25561, 25562, 25564, 25571, 25573, 
    25574, 25577, 25578, 25580, 25585, 25586, 25588, 25592, 25663, 25695, 25711, 25719, 25723, 25725, 25726, 25759, 25775, 25783, 25787, 25789, 
    25790, 25807, 25815, 25819, 25821, 25822, 25831, 25835, 25837, 25838, 25843, 25845, 25846, 25849, 25850, 25852, 25887, 25903, 25911, 25915, 
    25917, 25918, 25935, 25943, 25947, 25949, 25950, 25959, 25963, 25965, 25966, 25971, 25973, 25974, 25977, 25978, 25980, 25999, 26007, 26011, 
    26013, 26014, 26023, 26027, 26029, 26030, 26035, 26037, 26038, 26041, 26042, 26044, 26055, 26059, 26061, 26062, 26067, 26069, 26070, 26073, 
    26074, 26076, 26083, 26085, 26086, 26089, 26090, 26092, 26097, 26098, 26100, 26104, 26143, 26159, 26167, 26171, 26173, 26174, 26191, 26199, 
    26203, 26205, 26206, 26215, 26219, 26221, 26222, 26227, 26229, 26230, 26233, 26234, 26236, 26255, 26263, 26267, 26269, 26270, 26279, 26283, 
    26285, 26286, 26291, 26293, 26294, 26297, 26298, 26300, 26311, 26315, 26317, 26318, 26323, 26325, 26326, 26329, 26330, 26332, 26339, 26341, 
    26342, 26345, 26346, 26348, 26353, 26354, 26356, 26360, 26383, 26391, 26395, 26397, 26398, 26407, 26411, 26413, 26414, 26419, 26421, 26422, 
    26425, 26426, 26428, 26439, 26443, 26445, 26446, 26451, 26453, 26454, 26457, 26458, 26460, 26467, 26469, 26470, 26473, 26474, 26476, 26481, 
    26482, 26484, 26488, 26503, 26507, 26509, 26510, 26515, 26517, 26518, 26521, 26522, 26524, 26531, 26533, 26534, 26537, 26538, 26540, 26545, 
    26546, 26548, 26552, 26563, 26565, 26566, 26569, 26570, 26572, 26577, 26578, 26580, 26584, 26593, 26594, 26596, 26600, 26608, 26687, 26719, 
    26735, 26743, 26747, 26749, 26750, 26783, 26799, 26807, 26811, 26813, 26814, 26831, 26839, 26843, 26845, 26846, 26855, 26859, 26861, 26862, 
    26867, 26869, 26870, 26873, 26874, 26876, 26911, 26927, 26935, 26939, 26941, 26942, 26959, 26967, 26971, 26973, 26974, 26983, 26987, 26989, 
    26990, 26995, 26997, 26998, 27001, 27002, 27004, 27023, 27031, 27035, 27037, 27038, 27047, 27051, 27053, 27054, 27059, 27061, 27062, 27065, 
    27066, 27068, 27079, 27083, 27085, 27086, 27091, 27093, 27094, 27097, 27098, 27100, 27107, 27109, 27110, 27113, 27114, 27116, 27121, 27122, 
    27124, 27128, 27167, 27183, 27191, 27195, 27197, 27198, 27215, 27223, 27227, 27229, 27230, 27239, 27243, 27245, 27246, 27251, 27253, 27254, 
    27257, 27258, 27260, 27279, 27287, 27291, 27293, 27294, 27303, 27307, 27309, 27310, 27315, 27317, 27318, 27321, 27322, 27324, 27335, 27339, 
    27341, 27342, 27347, 27349, 27350, 27353, 27354, 27356, 27363, 27365, 27366, 27369, 27370, 27372, 27377, 27378, 27380, 27384, 27407, 27415, 
    27419, 27421, 27422, 27431, 27435, 27437, 27438, 27443, 27445, 27446, 27449, 27450, 27452, 27463, 27467, 27469, 27470, 27475, 27477, 27478, 
    27481, 27482, 27484, 27491, 27493, 27494, 27497, 27498, 27500, 27505, 27506, 27508, 27512, 27527, 27531, 27533, 27534, 27539, 27541, 27542, 
    27545, 27546, 27548, 27555, 27557, 27558, 27561, 27562, 27564, 27569, 27570, 27572, 27576, 27587, 27589, 27590, 27593, 27594, 27596, 27601, 
    27602, 27604, 27608, 27617, 27618, 27620, 27624, 27632, 27679, 27695, 27703, 27707, 27709, 27710, 27727, 27735, 27739, 27741, 27742, 27751, 
    27755, 27757, 27758, 27763, 27765, 27766, 27769, 27770, 27772, 27791, 27799, 27803, 27805, 27806, 27815, 27819, 27821, 27822, 27827, 27829, 
    27830, 27833, 27834, 27836, 27847, 27851, 27853, 27854, 27859, 27861, 27862, 27865, 27866, 27868, 27875, 27877, 27878, 27881, 27882, 27884, 
    27889, 27890, 27892, 27896, 27919, 27927, 27931, 27933, 27934, 27943, 27947, 27949, 27950, 27955, 27957, 27958, 27961, 27962, 27964, 27975, 
    27979, 27981, 27982, 27987, 27989, 27990, 27993, 27994, 27996, 28003, 28005, 28006, 28009, 28010, 28012, 28017, 28018, 28020, 28024, 28039, 
    28043, 28045, 28046, 28051, 28053, 28054, 28057, 28058, 28060, 28067, 28069, 28070, 28073, 28074, 28076, 28081, 28082, 28084, 28088, 28099, 
    28101, 28102, 28105, 28106, 28108, 28113, 28114, 28116, 28120, 28129, 28130, 28132, 28136, 28144, 28175, 28183, 28187, 28189, 28190, 28199, 
    28203, 28205, 28206, 28211, 28213, 28214, 28217, 28218, 28220, 28231, 28235, 28237, 28238, 28243, 28245, 28246, 28249, 28250, 28252, 28259, 
    28261, 28262, 28265, 28266, 28268, 28273, 28274, 28276, 28280, 28295, 28299, 28301, 28302, 28307, 28309, 28310, 28313, 28314, 28316, 28323, 
    28325, 28326, 28329, 28330, 28332, 28337, 28338, 28340, 28344, 28355, 28357, 28358, 28361, 28362, 28364, 28369, 28370, 28372, 28376, 28385, 
    28386, 28388, 28392, 28400, 28423, 28427, 28429, 28430, 28435, 28437, 28438, 28441, 28442, 28444, 28451, 28453, 28454, 28457, 28458, 28460, 
    28465, 28466, 28468, 28472, 28483, 28485, 28486, 28489, 28490, 28492, 28497, 28498, 28500, 28504, 28513, 28514, 28516, 28520, 28528, 28547, 
    28549, 28550, 28553, 28554, 28556, 28561, 28562, 28564, 28568, 28577, 28578, 28580, 28584, 28592, 28609, 28610, 28612, 28616, 28624, 28640, 
    28735, 28767, 28783, 28791, 28795, 28797, 28798, 28831, 28847, 28855, 28859, 28861, 28862, 28879, 28887, 28891, 28893, 28894, 28903, 28907, 
    28909, 28910, 28915, 28917, 28918, 28921, 28922, 28924, 28959, 28975, 28983, 28987, 28989, 28990, 29007, 29015, 29019, 29021, 29022, 29031, 
    29035, 29037, 29038, 29043, 29045, 29046, 29049, 29050, 29052, 29071, 29079, 29083, 29085, 29086, 29095, 29099, 29101, 29102, 29107, 29109, 
    29110, 29113, 29114, 29116, 29127, 29131, 29133, 29134, 29139, 29141, 29142, 29145, 29146, 29148, 29155, 29157, 29158, 29161, 29162, 29164, 
    29169, 29170, 29172, 29176, 29215, 29231, 29239, 29243, 29245, 29246, 29263, 29271, 29275, 29277, 29278, 29287, 29291, 29293, 29294, 29299, 
    29301, 29302, 29305, 29306, 29308, 29327, 29335, 29339, 29341, 29342, 29351, 29355, 29357, 29358, 29363, 29365, 29366, 29369, 29370, 29372, 
    29383, 29387, 29389, 29390, 29395, 29397, 29398, 29401, 29402, 29404, 29411, 29413, 29414, 29417, 29418, 29420, 29425, 29426, 29428, 29432, 
    29455, 29463, 29467, 29469, 29470, 29479, 29483, 29485, 29486, 29491, 29493, 29494, 29497, 29498, 29500, 29511, 29515, 29517, 29518, 29523, 
    29525, 29526, 29529, 29530, 29532, 29539, 29541, 29542, 29545, 29546, 29548, 29553, 29554, 29556, 29560, 29575, 29579, 29581, 29582, 29587, 
    29589, 29590, 29593, 29594, 29596, 29603, 29605, 29606, 29609, 29610, 29612, 29617, 29618, 29620, 29624, 29635, 29637, 29638, 29641, 29642, 
    29644, 29649, 29650, 29652, 29656, 29665, 29666, 29668, 29672, 29680, 29727, 29743, 29751, 29755, 29757, 29758, 29775, 29783, 29787, 29789, 
    29790, 29799, 29803, 29805, 29806, 29811, 29813, 29814, 29817, 29818, 29820, 29839, 29847, 29851, 29853, 29854, 29863, 29867, 29869, 29870, 
    29875, 29877, 29878, 29881, 29882, 29884, 29895, 29899, 29901, 29902, 29907, 29909, 29910, 29913, 29914, 29916, 29923, 29925, 29926, 29929, 
    29930, 29932, 29937, 29938, 29940, 29944, 29967, 29975, 29979, 29981, 29982, 29991, 29995, 29997, 29998, 30003, 30005, 30006, 30009, 30010, 
    30012, 30023, 30027, 30029, 30030, 30035, 30037, 30038, 30041, 30042, 30044, 30051, 30053, 30054, 30057, 30058, 30060, 30065, 30066, 30068, 
    30072, 30087, 30091, 30093, 30094, 30099, 30101, 30102, 30105, 30106, 30108, 30115, 30117, 30118, 30121, 30122, 30124, 30129, 30130, 30132, 
    30136, 30147, 30149, 30150, 30153, 30154, 30156, 30161, 30162, 30164, 30168, 30177, 30178, 30180, 30184, 30192, 30223, 30231, 30235, 30237, 
    30238, 30247, 30251, 30253, 30254, 30259, 30261, 30262, 30265, 30266, 30268, 30279, 30283, 30285, 30286, 30291, 30293, 30294, 30297, 30298, 
    30300, 30307, 30309, 30310, 30313, 30314, 30316, 30321, 30322, 30324, 30328, 30343, 30347, 30349, 30350, 30355, 30357, 30358, 30361, 30362, 
    30364, 30371, 30373, 30374, 30377, 30378, 30380, 30385, 30386, 30388, 30392, 30403, 30405, 30406, 30409, 30410, 30412, 30417, 30418, 30420, 
    30424, 30433, 30434, 30436, 30440, 30448, 30471, 30475, 30477, 30478, 30483, 30485, 30486, 30489, 30490, 30492, 30499, 30501, 30502, 30505, 
    30506, 30508, 30513, 30514, 30516, 30520, 30531, 30533, 30534, 30537, 30538, 30540, 30545, 30546, 30548, 30552, 30561, 30562, 30564, 30568, 
    30576, 30595, 30597, 30598, 30601, 30602, 30604, 30609, 30610, 30612, 30616, 30625, 30626, 30628, 30632, 30640, 30657, 30658, 30660, 30664, 
    30672, 30688, 30751, 30767, 30775, 30779, 30781, 30782, 30799, 30807, 30811, 30813, 30814, 30823, 30827, 30829, 30830, 30835, 30837, 30838, 
    30841, 30842, 30844, 30863, 30871, 30875, 30877, 30878, 30887, 30891, 30893, 30894, 30899, 30901, 30902, 30905, 30906, 30908, 30919, 30923, 
    30925, 30926, 30931, 30933, 30934, 30937, 30938, 30940, 30947, 30949, 30950, 30953, 30954, 30956, 30961, 30962, 30964, 30968, 30991, 30999, 
    31003, 31005, 31006, 31015, 31019, 31021, 31022, 31027, 31029, 31030, 31033, 31034, 31036, 31047, 31051, 31053, 31054, 31059, 31061, 31062, 
    31065, 31066, 31068, 31075, 31077, 31078, 31081, 31082, 31084, 31089, 31090, 31092, 31096, 31111, 31115, 31117, 31118, 31123, 31125, 31126, 
    31129, 31130, 31132, 31139, 31141, 31142, 31145, 31146, 31148, 31153, 31154, 31156, 31160, 31171, 31173, 31174, 31177, 31178, 31180, 31185, 
    31186, 31188, 31192, 31201, 31202, 31204, 31208, 31216, 31247, 31255, 31259, 31261, 31262, 31271, 31275, 31277, 31278, 31283, 31285, 31286, 
    31289, 31290, 31292, 31303, 31307, 31309, 31310, 31315, 31317, 31318, 31321, 31322, 31324, 31331, 31333, 31334, 31337, 31338, 31340, 31345, 
    31346, 31348, 31352, 31367, 31371, 31373, 31374, 31379, 31381, 31382, 31385, 31386, 31388, 31395, 31397, 31398, 31401, 31402, 31404, 31409, 
    31410, 31412, 31416, 31427, 31429, 31430, 31433, 31434, 31436, 31441, 31442, 31444, 31448, 31457, 31458, 31460, 31464, 31472, 31495, 31499, 
    31501, 31502, 31507, 31509, 31510, 31513, 31514, 31516, 31523, 31525, 31526, 31529, 31530, 31532, 31537, 31538, 31540, 31544, 31555, 31557, 
    31558, 31561, 31562, 31564, 31569, 31570, 31572, 31576, 31585, 31586, 31588, 31592, 31600, 31619, 31621, 31622, 31625, 31626, 31628, 31633, 
    31634, 31636, 31640, 31649, 31650, 31652, 31656, 31664, 31681, 31682, 31684, 31688, 31696, 31712, 31759, 31767, 31771, 31773, 31774, 31783, 
    31787, 31789, 31790, 31795, 31797, 31798, 31801, 31802, 31804, 31815, 31819, 31821, 31822, 31827, 31829, 31830, 31833, 31834, 31836, 31843, 
    31845, 31846, 31849, 31850, 31852, 31857, 31858, 31860, 31864, 31879, 31883, 31885, 31886, 31891, 31893, 31894, 31897, 31898, 31900, 31907, 
    31909, 31910, 31913, 31914, 31916, 31921, 31922, 31924, 31928, 31939, 31941, 31942, 31945, 31946, 31948, 31953, 31954, 31956, 31960, 31969, 
    31970, 31972, 31976, 31984, 32007, 32011, 32013, 32014, 32019, 32021, 32022, 32025, 32026, 32028, 32035, 32037, 32038, 32041, 32042, 32044, 
    32049, 32050, 32052, 32056, 32067, 32069, 32070, 32073, 32074, 32076, 32081, 32082, 32084, 32088, 32097, 32098, 32100, 32104, 32112, 32131, 
    32133, 32134, 32137, 32138, 32140, 32145, 32146, 32148, 32152, 32161, 32162, 32164, 32168, 32176, 32193, 32194, 32196, 32200, 32208, 32224, 
    32263, 32267, 32269, 32270, 32275, 32277, 32278, 32281, 32282, 32284, 32291, 32293, 32294, 32297, 32298, 32300, 32305, 32306, 32308, 32312, 
    32323, 32325, 32326, 32329, 32330, 32332, 32337, 32338, 32340, 32344, 32353, 32354, 32356, 32360, 32368, 32387, 32389, 32390, 32393, 32394, 
    32396, 32401, 32402, 32404, 32408, 32417, 32418, 32420, 32424, 32432, 32449, 32450, 32452, 32456, 32464, 32480, 32515, 32517, 32518, 32521, 
    32522, 32524, 32529, 32530, 32532, 32536, 32545, 32546, 32548, 32552, 32560, 32577, 32578, 32580, 32584, 32592, 32608, 32641, 32642, 32644, 
    32648, 32656, 32672, 32704, 33023, 33151, 33215, 33247, 33263, 33271, 33275, 33277, 33278, 33407, 33471, 33503, 33519, 33527, 33531, 33533, 
    33534, 33599, 33631, 33647, 33655, 33659, 33661, 33662, 33695, 33711, 33719, 33723, 33725, 33726, 33743, 33751, 33755, 33757, 33758, 33767, 
    33771, 33773, 33774, 33779, 33781, 33782, 33785, 33786, 33788, 33919, 33983, 34015, 34031, 34039, 34043, 34045, 34046, 34111, 34143, 34159, 
    34167, 34171, 34173, 34174, 34207, 34223, 34231, 34235, 34237, 34238, 34255, 34263, 34267, 34269, 34270, 34279, 34283, 34285, 34286, 34291, 
    34293, 34294, 34297, 34298, 34300, 34367, 34399, 34415, 34423, 34427, 34429, 34430, 34463, 34479, 34487, 34491, 34493, 34494, 34511, 34519, 
    34523, 34525, 34526, 34535, 34539, 34541, 34542, 34547, 34549, 34550, 34553, 34554, 34556, 34591, 34607, 34615, 34619, 34621, 34622, 34639, 
    34647, 34651, 34653, 34654, 34663, 34667, 34669, 34670, 34675, 34677, 34678, 34681, 34682, 34684, 34703, 34711, 34715, 34717, 34718, 34727, 
    34731, 34733, 34734, 34739, 34741, 34742, 34745, 34746, 34748, 34759, 34763, 34765, 34766, 34771, 34773, 34774, 34777, 34778, 34780, 34787, 
    34789, 34790, 34793, 34794, 34796, 34801, 34802, 34804, 34808, 34943, 35007, 35039, 35055, 35063, 35067, 35069, 35070, 35135, 35167, 35183, 
    35191, 35195, 35197, 35198, 35231, 35247, 35255, 35259, 35261, 35262, 35279, 35287, 35291, 35293, 35294, 35303, 35307, 35309, 35310, 35315, 
    35317, 35318, 35321, 35322, 35324, 35391, 35423, 35439, 35447, 35451, 35453, 35454, 35487, 35503, 35511, 35515, 35517, 35518, 35535, 35543, 
    35547, 35549, 35550, 35559, 35563, 35565, 35566, 35571, 35573, 35574, 35577, 35578, 35580, 35615, 35631, 35639, 35643, 35645, 35646, 35663, 
    35671, 35675, 35677, 35678, 35687, 35691, 35693, 35694, 35699, 35701, 35702, 35705, 35706, 35708, 35727, 35735, 35739, 35741, 35742, 35751, 
    35755, 35757, 35758, 35763, 35765, 35766, 35769, 35770, 35772, 35783, 35787, 35789, 35790, 35795, 35797, 35798, 35801, 35802, 35804, 35811, 
    35813, 35814, 35817, 35818, 35820, 35825, 35826, 35828, 35832, 35903, 35935, 35951, 35959, 35963, 35965, 35966, 35999, 36015, 36023, 36027, 
    36029, 36030, 36047, 36055, 36059, 36061, 36062, 36071, 36075, 36077, 36078, 36083, 36085, 36086, 36089, 36090, 36092, 36127, 36143, 36151, 
    36155, 36157, 36158, 36175, 36183, 36187, 36189, 36190, 36199, 36203, 36205, 36206, 36211, 36213, 36214, 36217, 36218, 36220, 36239, 36247, 
    36251, 36253, 36254, 36263, 36267, 36269, 36270, 36275, 36277, 36278, 36281, 36282, 36284, 36295, 36299, 36301, 36302, 36307, 36309, 36310, 
    36313, 36314, 36316, 36323, 36325, 36326, 36329, 36330, 36332, 36337, 36338, 36340, 36344, 36383, 36399, 36407, 36411, 36413, 36414, 36431, 
    36439, 36443, 36445, 36446, 36455, 36459, 36461, 36462, 36467, 36469, 36470, 36473, 36474, 36476, 36495, 36503, 36507, 36509, 36510, 36519, 
    36523, 36525, 36526, 36531, 36533, 36534, 36537, 36538, 36540, 36551, 36555, 36557, 36558, 36563, 36565, 36566, 36569, 36570, 36572, 36579, 
    36581, 36582, 36585, 36586, 36588, 36593, 36594, 36596, 36600, 36623, 36631, 36635, 36637, 36638, 36647, 36651, 36653, 36654, 36659, 36661, 
    36662, 36665, 36666, 36668, 36679, 36683, 36685, 36686, 36691, 36693, 36694, 36697, 36698, 36700, 36707, 36709, 36710, 36713, 36714, 36716, 
    36721, 36722, 36724, 36728, 36743, 36747, 36749, 36750, 36755, 36757, 36758, 36761, 36762, 36764, 36771, 36773, 36774, 36777, 36778, 36780, 
    36785, 36786, 36788, 36792, 36803, 36805, 36806, 36809, 36810, 36812, 36817, 36818, 36820, 36824, 36833, 36834, 36836, 36840, 36848, 36991, 
    37055, 37087, 37103, 37111, 37115, 37117, 37118, 37183, 37215, 37231, 37239, 37243, 37245, 37246, 37279, 37295, 37303, 37307, 37309, 37310, 
    37327, 37335, 37339, 37341, 37342, 37351, 37355, 37357, 37358, 37363, 37365, 37366, 37369, 37370, 37372, 37439, 37471, 37487, 37495, 37499, 
    37501, 37502, 37535, 37551, 37559, 37563, 37565, 37566, 37583, 37591, 37595, 37597, 37598, 37607, 37611, 37613, 37614, 37619, 37621, 37622, 
    37625, 37626, 37628, 37663, 37679, 37687, 37691, 37693, 37694, 37711, 37719, 37723, 37725, 37726, 37735, 37739, 37741, 37742, 37747, 37749, 
    37750, 37753, 37754, 37756, 37775, 37783, 37787, 37789, 37790, 37799, 37803, 37805, 37806, 37811, 37813, 37814, 37817, 37818, 37820, 37831, 
    37835, 37837, 37838, 37843, 37845, 37846, 37849, 37850, 37852, 37859, 37861, 37862, 37865, 37866, 37868, 37873, 37874, 37876, 37880, 37951, 
    37983, 37999, 38007, 38011, 38013, 38014, 38047, 38063, 38071, 38075, 38077, 38078, 38095, 38103, 38107, 38109, 38110, 38119, 38123, 38125, 
    38126, 38131, 38133, 38134, 38137, 38138, 38140, 38175, 38191, 38199, 38203, 38205, 38206, 38223, 38231, 38235, 38237, 38238, 38247, 38251, 
    38253, 38254, 38259, 38261, 38262, 38265, 38266, 38268, 38287, 38295, 38299, 38301, 38302, 38311, 38315, 38317, 38318, 38323, 38325, 38326, 
    38329, 38330, 38332, 38343, 38347, 38349, 38350, 38355, 38357, 38358, 38361, 38362, 38364, 38371, 38373, 38374, 38377, 38378, 38380, 38385, 
    38386, 38388, 38392, 38431, 38447, 38455, 38459, 38461, 38462, 38479, 38487, 38491, 38493, 38494, 38503, 38507, 38509, 38510, 38515, 38517, 
    38518, 38521, 38522, 38524, 38543, 38551, 38555, 38557, 38558, 38567, 38571, 38573, 38574, 38579, 38581, 38582, 38585, 38586, 38588, 38599, 
    38603, 38605, 38606, 38611, 38613, 38614, 38617, 38618, 38620, 38627, 38629, 38630, 38633, 38634, 38636, 38641, 38642, 38644, 38648, 38671, 
    38679, 38683, 38685, 38686, 38695, 38699, 38701, 38702, 38707, 38709, 38710, 38713, 38714, 38716, 38727, 38731, 38733, 38734, 38739, 38741, 
    38742, 38745, 38746, 38748, 38755, 38757, 38758, 38761, 38762, 38764, 38769, 38770, 38772, 38776, 38791, 38795, 38797, 38798, 38803, 38805, 
    38806, 38809, 38810, 38812, 38819, 38821, 38822, 38825, 38826, 38828, 38833, 38834, 38836, 38840, 38851, 38853, 38854, 38857, 38858, 38860, 
    38865, 38866, 38868, 38872, 38881, 38882, 38884, 38888, 38896, 38975, 39007, 39023, 39031, 39035, 39037, 39038, 39071, 39087, 39095, 39099, 
    39101, 39102, 39119, 39127, 39131, 39133, 39134, 39143, 39147, 39149, 39150, 39155, 39157, 39158, 39161, 39162, 39164, 39199, 39215, 39223, 
    39227, 39229, 39230, 39247, 39255, 39259, 39261, 39262, 39271, 39275, 39277, 39278, 39283, 39285, 39286, 39289, 39290, 39292, 39311, 39319, 
    39323, 39325, 39326, 39335, 39339, 39341, 39342, 39347, 39349, 39350, 39353, 39354, 39356, 39367, 39371, 39373, 39374, 39379, 39381, 39382, 
    39385, 39386, 39388, 39395, 39397, 39398, 39401, 39402, 39404, 39409, 39410, 39412, 39416, 39455, 39471, 39479, 39483, 39485, 39486, 39503, 
    39511, 39515, 39517, 39518, 39527, 39531, 39533, 39534, 39539, 39541, 39542, 39545, 39546, 39548, 39567, 39575, 39579, 39581, 39582, 39591, 
    39595, 39597, 39598, 39603, 39605, 39606, 39609, 39610, 39612, 39623, 39627, 39629, 39630, 39635, 39637, 39638, 39641, 39642, 39644, 39651, 
    39653, 39654, 39657, 39658, 39660, 39665, 39666, 39668, 39672, 39695, 39703, 39707, 39709, 39710, 39719, 39723, 39725, 39726, 39731, 39733, 
    39734, 39737, 39738, 39740, 39751, 39755, 39757, 39758, 39763, 39765, 39766, 39769, 39770, 39772, 39779, 39781, 39782, 39785, 39786, 39788, 
    39793, 39794, 39796, 39800, 39815, 39819, 39821, 39822, 39827, 39829, 39830, 39833, 39834, 39836, 39843, 39845, 39846, 39849, 39850, 39852, 
    39857, 39858, 39860, 39864, 39875, 39877, 39878, 39881, 39882, 39884, 39889, 39890, 39892, 39896, 39905, 39906, 39908, 39912, 39920, 39967, 
    39983, 39991, 39995, 39997, 39998, 40015, 40023, 40027, 40029, 40030, 40039, 40043, 40045, 40046, 40051, 40053, 40054, 40057, 40058, 40060, 
    40079, 40087, 40091, 40093, 40094, 40103, 40107, 40109, 40110, 40115, 40117, 40118, 40121, 40122, 40124, 40135, 40139, 40141, 40142, 40147, 
    40149, 40150, 40153, 40154, 40156, 40163, 40165, 40166, 40169, 40170, 40172, 40177, 40178, 40180, 40184, 40207, 40215, 40219, 40221, 40222, 
    40231, 40235, 40237, 40238, 40243, 40245, 40246, 40249, 40250, 40252, 40263, 40267, 40269, 40270, 40275, 40277, 40278, 40281, 40282, 40284, 
    40291, 40293, 40294, 40297, 40298, 40300, 40305, 40306, 40308, 40312, 40327, 40331, 40333, 40334, 40339, 40341, 40342, 40345, 40346, 40348, 
    40355, 40357, 40358, 40361, 40362, 40364, 40369, 40370, 40372, 40376, 40387, 40389, 40390, 40393, 40394, 40396, 40401, 40402, 40404, 40408, 
    40417, 40418, 40420, 40424, 40432, 40463, 40471, 40475, 40477, 40478, 40487, 40491, 40493, 40494, 40499, 40501, 40502, 40505, 40506, 40508, 
    40519, 40523, 40525, 40526, 40531, 40533, 40534, 40537, 40538, 40540, 40547, 40549, 40550, 40553, 40554, 40556, 40561, 40562, 40564, 40568, 
    40583, 40587, 40589, 40590, 40595, 40597, 40598, 40601, 40602, 40604, 40611, 40613, 40614, 40617, 40618, 40620, 40625, 40626, 40628, 40632, 
    40643, 40645, 40646, 40649, 40650, 40652, 40657, 40658, 40660, 40664, 40673, 40674, 40676, 40680, 40688, 40711, 40715, 40717, 40718, 40723, 
    40725, 40726, 40729, 40730, 40732, 40739, 40741, 40742, 40745, 40746, 40748, 40753, 40754, 40756, 40760, 40771, 40773, 40774, 40777, 40778, 
    40780, 40785, 40786, 40788, 40792, 40801, 40802, 40804, 40808, 40816, 40835, 40837, 40838, 40841, 40842, 40844, 40849, 40850, 40852, 40856, 
    40865, 40866, 40868, 40872, 40880, 40897, 40898, 40900, 40904, 40912, 40928, 41087, 41151, 41183, 41199, 41207, 41211, 41213, 41214, 41279, 
    41311, 41327, 41335, 41339, 41341, 41342, 41375, 41391, 41399, 41403, 41405, 41406, 41423, 41431, 41435, 41437, 41438, 41447, 41451, 41453, 
    41454, 41459, 41461, 41462, 41465, 41466, 41468, 41535, 41567, 41583, 41591, 41595, 41597, 41598, 41631, 41647, 41655, 41659, 41661, 41662, 
    41679, 41687, 41691, 41693, 41694, 41703, 41707, 41709, 41710, 41715, 41717, 41718, 41721, 41722, 41724, 41759, 41775, 41783, 41787, 41789, 
    41790, 41807, 41815, 41819, 41821, 41822, 41831, 41835, 41837, 41838, 41843, 41845, 41846, 41849, 41850, 41852, 41871, 41879, 41883, 41885, 
    41886, 41895, 41899, 41901, 41902, 41907, 41909, 41910, 41913, 41914, 41916, 41927, 41931, 41933, 41934, 41939, 41941, 41942, 41945, 41946, 
    41948, 41955, 41957, 41958, 41961, 41962, 41964, 41969, 41970, 41972, 41976, 42047, 42079, 42095, 42103, 42107, 42109, 42110, 42143, 42159, 
    42167, 42171, 42173, 42174, 42191, 42199, 42203, 42205, 42206, 42215, 42219, 42221, 42222, 42227, 42229, 42230, 42233, 42234, 42236, 42271, 
    42287, 42295, 42299, 42301, 42302, 42319, 42327, 42331, 42333, 42334, 42343, 42347, 42349, 42350, 42355, 42357, 42358, 42361, 42362, 42364, 
    42383, 42391, 42395, 42397, 42398, 42407, 42411, 42413, 42414, 42419, 42421, 42422, 42425, 42426, 42428, 42439, 42443, 42445, 42446, 42451, 
    42453, 42454, 42457, 42458, 42460, 42467, 42469, 42470, 42473, 42474, 42476, 42481, 42482, 42484, 42488, 42527, 42543, 42551, 42555, 42557, 
    42558, 42575, 42583, 42587, 42589, 42590, 42599, 42603, 42605, 42606, 42611, 42613, 42614, 42617, 42618, 42620, 42639, 42647, 42651, 42653, 
    42654, 42663, 42667, 42669, 42670, 42675, 42677, 42678, 42681, 42682, 42684, 42695, 42699, 42701, 42702, 42707, 42709, 42710, 42713, 42714, 
    42716, 42723, 42725, 42726, 42729, 42730, 42732, 42737, 42738, 42740, 42744, 42767, 42775, 42779, 42781, 42782, 42791, 42795, 42797, 42798, 
    42803, 42805, 42806, 42809, 42810, 42812, 42823, 42827, 42829, 42830, 42835, 42837, 42838, 42841, 42842, 42844, 42851, 42853, 42854, 42857, 
    42858, 42860, 42865, 42866, 42868, 42872, 42887, 42891, 42893, 42894, 42899, 42901, 42902, 42905, 42906, 42908, 42915, 42917, 42918, 42921, 
    42922, 42924, 42929, 42930, 42932, 42936, 42947, 42949, 42950, 42953, 42954, 42956, 42961, 42962, 42964, 42968, 42977, 42978, 42980, 42984, 
    42992, 43071, 43103, 43119, 43127, 43131, 43133, 43134, 43167, 43183, 43191, 43195, 43197, 43198, 43215, 43223, 43227, 43229, 43230, 43239, 
    43243, 43245, 43246, 43251, 43253, 43254, 43257, 43258, 43260, 43295, 43311, 43319, 43323, 43325, 43326, 43343, 43351, 43355, 43357, 43358, 
    43367, 43371, 43373, 43374, 43379, 43381, 43382, 43385, 43386, 43388, 43407, 43415, 43419, 43421, 43422, 43431, 43435, 43437, 43438, 43443, 
    43445, 43446, 43449, 43450, 43452, 43463, 43467, 43469, 43470, 43475, 43477, 43478, 43481, 43482, 43484, 43491, 43493, 43494, 43497, 43498, 
    43500, 43505, 43506, 43508, 43512, 43551, 43567, 43575, 43579, 43581, 43582, 43599, 43607, 43611, 43613, 43614, 43623, 43627, 43629, 43630, 
    43635, 43637, 43638, 43641, 43642, 43644, 43663, 43671, 43675, 43677, 43678, 43687, 43691, 43693, 43694, 43699, 43701, 43702, 43705, 43706, 
    43708, 43719, 43723, 43725, 43726, 43731, 43733, 43734, 43737, 43738, 43740, 43747, 43749, 43750, 43753, 43754, 43756, 43761, 43762, 43764, 
    43768, 43791, 43799, 43803, 43805, 43806, 43815, 43819, 43821, 43822, 43827, 43829, 43830, 43833, 43834, 43836, 43847, 43851, 43853, 43854, 
    43859, 43861, 43862, 43865, 43866, 43868, 43875, 43877, 43878, 43881, 43882, 43884, 43889, 43890, 43892, 43896, 43911, 43915, 43917, 43918, 
    43923, 43925, 43926, 43929, 43930, 43932, 43939, 43941, 43942, 43945, 43946, 43948, 43953, 43954, 43956, 43960, 43971, 43973, 43974, 43977, 
    43978, 43980, 43985, 43986, 43988, 43992, 44001, 44002, 44004, 44008, 44016, 44063, 44079, 44087, 44091, 44093, 44094, 44111, 44119, 44123, 
    44125, 44126, 44135, 44139, 44141, 44142, 44147, 44149, 44150, 44153, 44154, 44156, 44175, 44183, 44187, 44189, 44190, 44199, 44203, 44205, 
    44206, 44211, 44213, 44214, 44217, 44218, 44220, 44231, 44235, 44237, 44238, 44243, 44245, 44246, 44249, 44250, 44252, 44259, 44261, 44262, 
    44265, 44266, 44268, 44273, 44274, 44276, 44280, 44303, 44311, 44315, 44317, 44318, 44327, 44331, 44333, 44334, 44339, 44341, 44342, 44345, 
    44346, 44348, 44359, 44363, 44365, 44366, 44371, 44373, 44374, 44377, 44378, 44380, 44387, 44389, 44390, 44393, 44394, 44396, 44401, 44402, 
    44404, 44408, 44423, 44427, 44429, 44430, 44435, 44437, 44438, 44441, 44442, 44444, 44451, 44453, 44454, 44457, 44458, 44460, 44465, 44466, 
    44468, 44472, 44483, 44485, 44486, 44489, 44490, 44492, 44497, 44498, 44500, 44504, 44513, 44514, 44516, 44520, 44528, 44559, 44567, 44571, 
    44573, 44574, 44583, 44587, 44589, 44590, 44595, 44597, 44598, 44601, 44602, 44604, 44615, 44619, 44621, 44622, 44627, 44629, 44630, 44633, 
    44634, 44636, 44643, 44645, 44646, 44649, 44650, 44652, 44657, 44658, 44660, 44664, 44679, 44683, 44685, 44686, 44691, 44693, 44694, 44697, 
    44698, 44700, 44707, 44709, 44710, 44713, 44714, 44716, 44721, 44722, 44724, 44728, 44739, 44741, 44742, 44745, 44746, 44748, 44753, 44754, 
    44756, 44760, 44769, 44770, 44772, 44776, 44784, 44807, 44811, 44813, 44814, 44819, 44821, 44822, 44825, 44826, 44828, 44835, 44837, 44838, 
    44841, 44842, 44844, 44849, 44850, 44852, 44856, 44867, 44869, 44870, 44873, 44874, 44876, 44881, 44882, 44884, 44888, 44897, 44898, 44900, 
    44904, 44912, 44931, 44933, 44934, 44937, 44938, 44940, 44945, 44946, 44948, 44952, 44961, 44962, 44964, 44968, 44976, 44993, 44994, 44996, 
    45000, 45008, 45024, 45119, 45151, 45167, 45175, 45179, 45181, 45182, 45215, 45231, 45239, 45243, 45245, 45246, 45263, 45271, 45275, 45277, 
    45278, 45287, 45291, 45293, 45294, 45299, 45301, 45302, 45305, 45306, 45308, 45343, 45359, 45367, 45371, 45373, 45374, 45391, 45399, 45403, 
    45405, 45406, 45415, 45419, 45421, 45422, 45427, 45429, 45430, 45433, 45434, 45436, 45455, 45463, 45467, 45469, 45470, 45479, 45483, 45485, 
    45486, 45491, 45493, 45494, 45497, 45498, 45500, 45511, 45515, 45517, 45518, 45523, 45525, 45526, 45529, 45530, 45532, 45539, 45541, 45542, 
    45545, 45546, 45548, 45553, 45554, 45556, 45560, 45599, 45615, 45623, 45627, 45629, 45630, 45647, 45655, 45659, 45661, 45662, 45671, 45675, 
    45677, 45678, 45683, 45685, 45686, 45689, 45690, 45692, 45711, 45719, 45723, 45725, 45726, 45735, 45739, 45741, 45742, 45747, 45749, 45750, 
    45753, 45754, 45756, 45767, 45771, 45773, 45774, 45779, 45781, 45782, 45785, 45786, 45788, 45795, 45797, 45798, 45801, 45802, 45804, 45809, 
    45810, 45812, 45816, 45839, 45847, 45851, 45853, 45854, 45863, 45867, 45869, 45870, 45875, 45877, 45878, 45881, 45882, 45884, 45895, 45899, 
    45901, 45902, 45907, 45909, 45910, 45913, 45914, 45916, 45923, 45925, 45926, 45929, 45930, 45932, 45937, 45938, 45940, 45944, 45959, 45963, 
    45965, 45966, 45971, 45973, 45974, 45977, 45978, 45980, 45987, 45989, 45990, 45993, 45994, 45996, 46001, 46002, 46004, 46008, 46019, 46021, 
    46022, 46025, 46026, 46028, 46033, 46034, 46036, 46040, 46049, 46050, 46052, 46056, 46064, 46111, 46127, 46135, 46139, 46141, 46142, 46159, 
    46167, 46171, 46173, 46174, 46183, 46187, 46189, 46190, 46195, 46197, 46198, 46201, 46202, 46204, 46223, 46231, 46235, 46237, 46238, 46247, 
    46251, 46253, 46254, 46259, 46261, 46262, 46265, 46266, 46268, 46279, 46283, 46285, 46286, 46291, 46293, 46294, 46297, 46298, 46300, 46307, 
    46309, 46310, 46313, 46314, 46316, 46321, 46322, 46324, 46328, 46351, 46359, 46363, 46365, 46366, 46375, 46379, 46381, 46382, 46387, 46389, 
    46390, 46393, 46394, 46396, 46407, 46411, 46413, 46414, 46419, 46421, 46422, 46425, 46426, 46428, 46435, 46437, 46438, 46441, 46442, 46444, 
    46449, 46450, 46452, 46456, 46471, 46475, 46477, 46478, 46483, 46485, 46486, 46489, 46490, 46492, 46499, 46501, 46502, 46505, 46506, 46508, 
    46513, 46514, 46516, 46520, 46531, 46533, 46534, 46537, 46538, 46540, 46545, 46546, 46548, 46552, 46561, 46562, 46564, 46568, 46576, 46607, 
    46615, 46619, 46621, 46622, 46631, 46635, 46637, 46638, 46643, 46645, 46646, 46649, 46650, 46652, 46663, 46667, 46669, 46670, 46675, 46677, 
    46678, 46681, 46682, 46684, 46691, 46693, 46694, 46697, 46698, 46700, 46705, 46706, 46708, 46712, 46727, 46731, 46733, 46734, 46739, 46741, 
    46742, 46745, 46746, 46748, 46755, 46757, 46758, 46761, 46762, 46764, 46769, 46770, 46772, 46776, 46787, 46789, 46790, 46793, 46794, 46796, 
    46801, 46802, 46804, 46808, 46817, 46818, 46820, 46824, 46832, 46855, 46859, 46861, 46862, 46867, 46869, 46870, 46873, 46874, 46876, 46883, 
    46885, 46886, 46889, 46890, 46892, 46897, 46898, 46900, 46904, 46915, 46917, 46918, 46921, 46922, 46924, 46929, 46930, 46932, 46936, 46945, 
    46946, 46948, 46952, 46960, 46979, 46981, 46982, 46985, 46986, 46988, 46993, 46994, 46996, 47000, 47009, 47010, 47012, 47016, 47024, 47041, 
    47042, 47044, 47048, 47056, 47072, 47135, 47151, 47159, 47163, 47165, 47166, 47183, 47191, 47195, 47197, 47198, 47207, 47211, 47213, 47214, 
    47219, 47221, 47222, 47225, 47226, 47228, 47247, 47255, 47259, 47261, 47262, 47271, 47275, 47277, 47278, 47283, 47285, 47286, 47289, 47290, 
    47292, 47303, 47307, 47309, 47310, 47315, 47317, 47318, 47321, 47322, 47324, 47331, 47333, 47334, 47337, 47338, 47340, 47345, 47346, 47348, 
    47352, 47375, 47383, 47387, 47389, 47390, 47399, 47403, 47405, 47406, 47411, 47413, 47414, 47417, 47418, 47420, 47431, 47435, 47437, 47438, 
    47443, 47445, 47446, 47449, 47450, 47452, 47459, 47461, 47462, 47465, 47466, 47468, 47473, 47474, 47476, 47480, 47495, 47499, 47501, 47502, 
    47507, 47509, 47510, 47513, 47514, 47516, 47523, 47525, 47526, 47529, 47530, 47532, 47537, 47538, 47540, 47544, 47555, 47557, 47558, 47561, 
    47562, 47564, 47569, 47570, 47572, 47576, 47585, 47586, 47588, 47592, 47600, 47631, 47639, 47643, 47645, 47646, 47655, 47659, 47661, 47662, 
    47667, 47669, 47670, 47673, 47674, 47676, 47687, 47691, 47693, 47694, 47699, 47701, 47702, 47705, 47706, 47708, 47715, 47717, 47718, 47721, 
    47722, 47724, 47729, 47730, 47732, 47736, 47751, 47755, 47757, 47758, 47763, 47765, 47766, 47769, 47770, 47772, 47779, 47781, 47782, 47785, 
    47786, 47788, 47793, 47794, 47796, 47800, 47811, 47813, 47814, 47817, 47818, 47820, 47825, 47826, 47828, 47832, 47841, 47842, 47844, 47848, 
    47856, 47879, 47883, 47885, 47886, 47891, 47893, 47894, 47897, 47898, 47900, 47907, 47909, 47910, 47913, 47914, 47916, 47921, 47922, 47924, 
    47928, 47939, 47941, 47942, 47945, 47946, 47948, 47953, 47954, 47956, 47960, 47969, 47970, 47972, 47976, 47984, 48003, 48005, 48006, 48009, 
    48010, 48012, 48017, 48018, 48020, 48024, 48033, 48034, 48036, 48040, 48048, 48065, 48066, 48068, 48072, 48080, 48096, 48143, 48151, 48155, 
    48157, 48158, 48167, 48171, 48173, 48174, 48179, 48181, 48182, 48185, 48186, 48188, 48199, 48203, 48205, 48206, 48211, 48213, 48214, 48217, 
    48218, 48220, 48227, 48229, 48230, 48233, 48234, 48236, 48241, 48242, 48244, 48248, 48263, 48267, 48269, 48270, 48275, 48277, 48278, 48281, 
    48282, 48284, 48291, 48293, 48294, 48297, 48298, 48300, 48305, 48306, 48308, 48312, 48323, 48325, 48326, 48329, 48330, 48332, 48337, 48338, 
    48340, 48344, 48353, 48354, 48356, 48360, 48368, 48391, 48395, 48397, 48398, 48403, 48405, 48406, 48409, 48410, 48412, 48419, 48421, 48422, 
    48425, 48426, 48428, 48433, 48434, 48436, 48440, 48451, 48453, 48454, 48457, 48458, 48460, 48465, 48466, 48468, 48472, 48481, 48482, 48484, 
    48488, 48496, 48515, 48517, 48518, 48521, 48522, 48524, 48529, 48530, 48532, 48536, 48545, 48546, 48548, 48552, 48560, 48577, 48578, 48580, 
    48584, 48592, 48608, 48647, 48651, 48653, 48654, 48659, 48661, 48662, 48665, 48666, 48668, 48675, 48677, 48678, 48681, 48682, 48684, 48689, 
    48690, 48692, 48696, 48707, 48709, 48710, 48713, 48714, 48716, 48721, 48722, 48724, 48728, 48737, 48738, 48740, 48744, 48752, 48771, 48773, 
    48774, 48777, 48778, 48780, 48785, 48786, 48788, 48792, 48801, 48802, 48804, 48808, 48816, 48833, 48834, 48836, 48840, 48848, 48864, 48899, 
    48901, 48902, 48905, 48906, 48908, 48913, 48914, 48916, 48920, 48929, 48930, 48932, 48936, 48944, 48961, 48962, 48964, 48968, 48976, 48992, 
    49025, 49026, 49028, 49032, 49040, 49056, 49088, 49279, 49343, 49375, 49391, 49399, 49403, 49405, 49406, 49471, 49503, 49519, 49527, 49531, 
    49533, 49534, 49567, 49583, 49591, 49595, 49597, 49598, 49615, 49623, 49627, 49629, 49630, 49639, 49643, 49645, 49646, 49651, 49653, 49654, 
    49657, 49658, 49660, 49727, 49759, 49775, 49783, 49787, 49789, 49790, 49823, 49839, 49847, 49851, 49853, 49854, 49871, 49879, 49883, 49885, 
    49886, 49895, 49899, 49901, 49902, 49907, 49909, 49910, 49913, 49914, 49916, 49951, 49967, 49975, 49979, 49981, 49982, 49999, 50007, 50011, 
    50013, 50014, 50023, 50027, 50029, 50030, 50035, 50037, 50038, 50041, 50042, 50044, 50063, 50071, 50075, 50077, 50078, 50087, 50091, 50093, 
    50094, 50099, 50101, 50102, 50105, 50106, 50108, 50119, 50123, 50125, 50126, 50131, 50133, 50134, 50137, 50138, 50140, 50147, 50149, 50150, 
    50153, 50154, 50156, 50161, 50162, 50164, 50168, 50239, 50271, 50287, 50295, 50299, 50301, 50302, 50335, 50351, 50359, 50363, 50365, 50366, 
    50383, 50391, 50395, 50397, 50398, 50407, 50411, 50413, 50414, 50419, 50421, 50422, 50425, 50426, 50428, 50463, 50479, 50487, 50491, 50493, 
    50494, 50511, 50519, 50523, 50525, 50526, 50535, 50539, 50541, 50542, 50547, 50549, 50550, 50553, 50554, 50556, 50575, 50583, 50587, 50589, 
    50590, 50599, 50603, 50605, 50606, 50611, 50613, 50614, 50617, 50618, 50620, 50631, 50635, 50637, 50638, 50643, 50645, 50646, 50649, 50650, 
    50652, 50659, 50661, 50662, 50665, 50666, 50668, 50673, 50674, 50676, 50680, 50719, 50735, 50743, 50747, 50749, 50750, 50767, 50775, 50779, 
    50781, 50782, 50791, 50795, 50797, 50798, 50803, 50805, 50806, 50809, 50810, 50812, 50831, 50839, 50843, 50845, 50846, 50855, 50859, 50861, 
    50862, 50867, 50869, 50870, 50873, 50874, 50876, 50887, 50891, 50893, 50894, 50899, 50901, 50902, 50905, 50906, 50908, 50915, 50917, 50918, 
    50921, 50922, 50924, 50929, 50930, 50932, 50936, 50959, 50967, 50971, 50973, 50974, 50983, 50987, 50989, 50990, 50995, 50997, 50998, 51001, 
    51002, 51004, 51015, 51019, 51021, 51022, 51027, 51029, 51030, 51033, 51034, 51036, 51043, 51045, 51046, 51049, 51050, 51052, 51057, 51058, 
    51060, 51064, 51079, 51083, 51085, 51086, 51091, 51093, 51094, 51097, 51098, 51100, 51107, 51109, 51110, 51113, 51114, 51116, 51121, 51122, 
    51124, 51128, 51139, 51141, 51142, 51145, 51146, 51148, 51153, 51154, 51156, 51160, 51169, 51170, 51172, 51176, 51184, 51263, 51295, 51311, 
    51319, 51323, 51325, 51326, 51359, 51375, 51383, 51387, 51389, 51390, 51407, 51415, 51419, 51421, 51422, 51431, 51435, 51437, 51438, 51443, 
    51445, 51446, 51449, 51450, 51452, 51487, 51503, 51511, 51515, 51517, 51518, 51535, 51543, 51547, 51549, 51550, 51559, 51563, 51565, 51566, 
    51571, 51573, 51574, 51577, 51578, 51580, 51599, 51607, 51611, 51613, 51614, 51623, 51627, 51629, 51630, 51635, 51637, 51638, 51641, 51642, 
    51644, 51655, 51659, 51661, 51662, 51667, 51669, 51670, 51673, 51674, 51676, 51683, 51685, 51686, 51689, 51690, 51692, 51697, 51698, 51700, 
    51704, 51743, 51759, 51767, 51771, 51773, 51774, 51791, 51799, 51803, 51805, 51806, 51815, 51819, 51821, 51822, 51827, 51829, 51830, 51833, 
    51834, 51836, 51855, 51863, 51867, 51869, 51870, 51879, 51883, 51885, 51886, 51891, 51893, 51894, 51897, 51898, 51900, 51911, 51915, 51917, 
    51918, 51923, 51925, 51926, 51929, 51930, 51932, 51939, 51941, 51942, 51945, 51946, 51948, 51953, 51954, 51956, 51960, 51983, 51991, 51995, 
    51997, 51998, 52007, 52011, 52013, 52014, 52019, 52021, 52022, 52025, 52026, 52028, 52039, 52043, 52045, 52046, 52051, 52053, 52054, 52057, 
    52058, 52060, 52067, 52069, 52070, 52073, 52074, 52076, 52081, 52082, 52084, 52088, 52103, 52107, 52109, 52110, 52115, 52117, 52118, 52121, 
    52122, 52124, 52131, 52133, 52134, 52137, 52138, 52140, 52145, 52146, 52148, 52152, 52163, 52165, 52166, 52169, 52170, 52172, 52177, 52178, 
    52180, 52184, 52193, 52194, 52196, 52200, 52208, 52255, 52271, 52279, 52283, 52285, 52286, 52303, 52311, 52315, 52317, 52318, 52327, 52331, 
    52333, 52334, 52339, 52341, 52342, 52345, 52346, 52348, 52367, 52375, 52379, 52381, 52382, 52391, 52395, 52397, 52398, 52403, 52405, 52406, 
    52409, 52410, 52412, 52423, 52427, 52429, 52430, 52435, 52437, 52438, 52441, 52442, 52444, 52451, 52453, 52454, 52457, 52458, 52460, 52465, 
    52466, 52468, 52472, 52495, 52503, 52507, 52509, 52510, 52519, 52523, 52525, 52526, 52531, 52533, 52534, 52537, 52538, 52540, 52551, 52555, 
    52557, 52558, 52563, 52565, 52566, 52569, 52570, 52572, 52579, 52581, 52582, 52585, 52586, 52588, 52593, 52594, 52596, 52600, 52615, 52619, 
    52621, 52622, 52627, 52629, 52630, 52633, 52634, 52636, 52643, 52645, 52646, 52649, 52650, 52652, 52657, 52658, 52660, 52664, 52675, 52677, 
    52678, 52681, 52682, 52684, 52689, 52690, 52692, 52696, 52705, 52706, 52708, 52712, 52720, 52751, 52759, 52763, 52765, 52766, 52775, 52779, 
    52781, 52782, 52787, 52789, 52790, 52793, 52794, 52796, 52807, 52811, 52813, 52814, 52819, 52821, 52822, 52825, 52826, 52828, 52835, 52837, 
    52838, 52841, 52842, 52844, 52849, 52850, 52852, 52856, 52871, 52875, 52877, 52878, 52883, 52885, 52886, 52889, 52890, 52892, 52899, 52901, 
    52902, 52905, 52906, 52908, 52913, 52914, 52916, 52920, 52931, 52933, 52934, 52937, 52938, 52940, 52945, 52946, 52948, 52952, 52961, 52962, 
    52964, 52968, 52976, 52999, 53003, 53005, 53006, 53011, 53013, 53014, 53017, 53018, 53020, 53027, 53029, 53030, 53033, 53034, 53036, 53041, 
    53042, 53044, 53048, 53059, 53061, 53062, 53065, 53066, 53068, 53073, 53074, 53076, 53080, 53089, 53090, 53092, 53096, 53104, 53123, 53125, 
    53126, 53129, 53130, 53132, 53137, 53138, 53140, 53144, 53153, 53154, 53156, 53160, 53168, 53185, 53186, 53188, 53192, 53200, 53216, 53311, 
    53343, 53359, 53367, 53371, 53373, 53374, 53407, 53423, 53431, 53435, 53437, 53438, 53455, 53463, 53467, 53469, 53470, 53479, 53483, 53485, 
    53486, 53491, 53493, 53494, 53497, 53498, 53500, 53535, 53551, 53559, 53563, 53565, 53566, 53583, 53591, 53595, 53597, 53598, 53607, 53611, 
    53613, 53614, 53619, 53621, 53622, 53625, 53626, 53628, 53647, 53655, 53659, 53661, 53662, 53671, 53675, 53677, 53678, 53683, 53685, 53686, 
    53689, 53690, 53692, 53703, 53707, 53709, 53710, 53715, 53717, 53718, 53721, 53722, 53724, 53731, 53733, 53734, 53737, 53738, 53740, 53745, 
    53746, 53748, 53752, 53791, 53807, 53815, 53819, 53821, 53822, 53839, 53847, 53851, 53853, 53854, 53863, 53867, 53869, 53870, 53875, 53877, 
    53878, 53881, 53882, 53884, 53903, 53911, 53915, 53917, 53918, 53927, 53931, 53933, 53934, 53939, 53941, 53942, 53945, 53946, 53948, 53959, 
    53963, 53965, 53966, 53971, 53973, 53974, 53977, 53978, 53980, 53987, 53989, 53990, 53993, 53994, 53996, 54001, 54002, 54004, 54008, 54031, 
    54039, 54043, 54045, 54046, 54055, 54059, 54061, 54062, 54067, 54069, 54070, 54073, 54074, 54076, 54087, 54091, 54093, 54094, 54099, 54101, 
    54102, 54105, 54106, 54108, 54115, 54117, 54118, 54121, 54122, 54124, 54129, 54130, 54132, 54136, 54151, 54155, 54157, 54158, 54163, 54165, 
    54166, 54169, 54170, 54172, 54179, 54181, 54182, 54185, 54186, 54188, 54193, 54194, 54196, 54200, 54211, 54213, 54214, 54217, 54218, 54220, 
    54225, 54226, 54228, 54232, 54241, 54242, 54244, 54248, 54256, 54303, 54319, 54327, 54331, 54333, 54334, 54351, 54359, 54363, 54365, 54366, 
    54375, 54379, 54381, 54382, 54387, 54389, 54390, 54393, 54394, 54396, 54415, 54423, 54427, 54429, 54430, 54439, 54443, 54445, 54446, 54451, 
    54453, 54454, 54457, 54458, 54460, 54471, 54475, 54477, 54478, 54483, 54485, 54486, 54489, 54490, 54492, 54499, 54501, 54502, 54505, 54506, 
    54508, 54513, 54514, 54516, 54520, 54543, 54551, 54555, 54557, 54558, 54567, 54571, 54573, 54574, 54579, 54581, 54582, 54585, 54586, 54588, 
    54599, 54603, 54605, 54606, 54611, 54613, 54614, 54617, 54618, 54620, 54627, 54629, 54630, 54633, 54634, 54636, 54641, 54642, 54644, 54648, 
    54663, 54667, 54669, 54670, 54675, 54677, 54678, 54681, 54682, 54684, 54691, 54693, 54694, 54697, 54698, 54700, 54705, 54706, 54708, 54712, 
    54723, 54725, 54726, 54729, 54730, 54732, 54737, 54738, 54740, 54744, 54753, 54754, 54756, 54760, 54768, 54799, 54807, 54811, 54813, 54814, 
    54823, 54827, 54829, 54830, 54835, 54837, 54838, 54841, 54842, 54844, 54855, 54859, 54861, 54862, 54867, 54869, 54870, 54873, 54874, 54876, 
    54883, 54885, 54886, 54889, 54890, 54892, 54897, 54898, 54900, 54904, 54919, 54923, 54925, 54926, 54931, 54933, 54934, 54937, 54938, 54940, 
    54947, 54949, 54950, 54953, 54954, 54956, 54961, 54962, 54964, 54968, 54979, 54981, 54982, 54985, 54986, 54988, 54993, 54994, 54996, 55000, 
    55009, 55010, 55012, 55016, 55024, 55047, 55051, 55053, 55054, 55059, 55061, 55062, 55065, 55066, 55068, 55075, 55077, 55078, 55081, 55082, 
    55084, 55089, 55090, 55092, 55096, 55107, 55109, 55110, 55113, 55114, 55116, 55121, 55122, 55124, 55128, 55137, 55138, 55140, 55144, 55152, 
    55171, 55173, 55174, 55177, 55178, 55180, 55185, 55186, 55188, 55192, 55201, 55202, 55204, 55208, 55216, 55233, 55234, 55236, 55240, 55248, 
    55264, 55327, 55343, 55351, 55355, 55357, 55358, 55375, 55383, 55387, 55389, 55390, 55399, 55403, 55405, 55406, 55411, 55413, 55414, 55417, 
    55418, 55420, 55439, 55447, 55451, 55453, 55454, 55463, 55467, 55469, 55470, 55475, 55477, 55478, 55481, 55482, 55484, 55495, 55499, 55501, 
    55502, 55507, 55509, 55510, 55513, 55514, 55516, 55523, 55525, 55526, 55529, 55530, 55532, 55537, 55538, 55540, 55544, 55567, 55575, 55579, 
    55581, 55582, 55591, 55595, 55597, 55598, 55603, 55605, 55606, 55609, 55610, 55612, 55623, 55627, 55629, 55630, 55635, 55637, 55638, 55641, 
    55642, 55644, 55651, 55653, 55654, 55657, 55658, 55660, 55665, 55666, 55668, 55672, 55687, 55691, 55693, 55694, 55699, 55701, 55702, 55705, 
    55706, 55708, 55715, 55717, 55718, 55721, 55722, 55724, 55729, 55730, 55732, 55736, 55747, 55749, 55750, 55753, 55754, 55756, 55761, 55762, 
    55764, 55768, 55777, 55778, 55780, 55784, 55792, 55823, 55831, 55835, 55837, 55838, 55847, 55851, 55853, 55854, 55859, 55861, 55862, 55865, 
    55866, 55868, 55879, 55883, 55885, 55886, 55891, 55893, 55894, 55897, 55898, 55900, 55907, 55909, 55910, 55913, 55914, 55916, 55921, 55922, 
    55924, 55928, 55943, 55947, 55949, 55950, 55955, 55957, 55958, 55961, 55962, 55964, 55971, 55973, 55974, 55977, 55978, 55980, 55985, 55986, 
    55988, 55992, 56003, 56005, 56006, 56009, 56010, 56012, 56017, 56018, 56020, 56024, 56033, 56034, 56036, 56040, 56048, 56071, 56075, 56077, 
    56078, 56083, 56085, 56086, 56089, 56090, 56092, 56099, 56101, 56102, 56105, 56106, 56108, 56113, 56114, 56116, 56120, 56131, 56133, 56134, 
    56137, 56138, 56140, 56145, 56146, 56148, 56152, 56161, 56162, 56164, 56168, 56176, 56195, 56197, 56198, 56201, 56202, 56204, 56209, 56210, 
    56212, 56216, 56225, 56226, 56228, 56232, 56240, 56257, 56258, 56260, 56264, 56272, 56288, 56335, 56343, 56347, 56349, 56350, 56359, 56363, 
    56365, 56366, 56371, 56373, 56374, 56377, 56378, 56380, 56391, 56395, 56397, 56398, 56403, 56405, 56406, 56409, 56410, 56412, 56419, 56421, 
    56422, 56425, 56426, 56428, 56433, 56434, 56436, 56440, 56455, 56459, 56461, 56462, 56467, 56469, 56470, 56473, 56474, 56476, 56483, 56485, 
    56486, 56489, 56490, 56492, 56497, 56498, 56500, 56504, 56515, 56517, 56518, 56521, 56522, 56524, 56529, 56530, 56532, 56536, 56545, 56546, 
    56548, 56552, 56560, 56583, 56587, 56589, 56590, 56595, 56597, 56598, 56601, 56602, 56604, 56611, 56613, 56614, 56617, 56618, 56620, 56625, 
    56626, 56628, 56632, 56643, 56645, 56646, 56649, 56650, 56652, 56657, 56658, 56660, 56664, 56673, 56674, 56676, 56680, 56688, 56707, 56709, 
    56710, 56713, 56714, 56716, 56721, 56722, 56724, 56728, 56737, 56738, 56740, 56744, 56752, 56769, 56770, 56772, 56776, 56784, 56800, 56839, 
    56843, 56845, 56846, 56851, 56853, 56854, 56857, 56858, 56860, 56867, 56869, 56870, 56873, 56874, 56876, 56881, 56882, 56884, 56888, 56899, 
    56901, 56902, 56905, 56906, 56908, 56913, 56914, 56916, 56920, 56929, 56930, 56932, 56936, 56944, 56963, 56965, 56966, 56969, 56970, 56972, 
    56977, 56978, 56980, 56984, 56993, 56994, 56996, 57000, 57008, 57025, 57026, 57028, 57032, 57040, 57056, 57091, 57093, 57094, 57097, 57098, 
    57100, 57105, 57106, 57108, 57112, 57121, 57122, 57124, 57128, 57136, 57153, 57154, 57156, 57160, 57168, 57184, 57217, 57218, 57220, 57224, 
    57232, 57248, 57280, 57407, 57439, 57455, 57463, 57467, 57469, 57470, 57503, 57519, 57527, 57531, 57533, 57534, 57551, 57559, 57563, 57565, 
    57566, 57575, 57579, 57581, 57582, 57587, 57589, 57590, 57593, 57594, 57596, 57631, 57647, 57655, 57659, 57661, 57662, 57679, 57687, 57691, 
    57693, 57694, 57703, 57707, 57709, 57710, 57715, 57717, 57718, 57721, 57722, 57724, 57743, 57751, 57755, 57757, 57758, 57767, 57771, 57773, 
    57774, 57779, 57781, 57782, 57785, 57786, 57788, 57799, 57803, 57805, 57806, 57811, 57813, 57814, 57817, 57818, 57820, 57827, 57829, 57830, 
    57833, 57834, 57836, 57841, 57842, 57844, 57848, 57887, 57903, 57911, 57915, 57917, 57918, 57935, 57943, 57947, 57949, 57950, 57959, 57963, 
    57965, 57966, 57971, 57973, 57974, 57977, 57978, 57980, 57999, 58007, 58011, 58013, 58014, 58023, 58027, 58029, 58030, 58035, 58037, 58038, 
    58041, 58042, 58044, 58055, 58059, 58061, 58062, 58067, 58069, 58070, 58073, 58074, 58076, 58083, 58085, 58086, 58089, 58090, 58092, 58097, 
    58098, 58100, 58104, 58127, 58135, 58139, 58141, 58142, 58151, 58155, 58157, 58158, 58163, 58165, 58166, 58169, 58170, 58172, 58183, 58187, 
    58189, 58190, 58195, 58197, 58198, 58201, 58202, 58204, 58211, 58213, 58214, 58217, 58218, 58220, 58225, 58226, 58228, 58232, 58247, 58251, 
    58253, 58254, 58259, 58261, 58262, 58265, 58266, 58268, 58275, 58277, 58278, 58281, 58282, 58284, 58289, 58290, 58292, 58296, 58307, 58309, 
    58310, 58313, 58314, 58316, 58321, 58322, 58324, 58328, 58337, 58338, 58340, 58344, 58352, 58399, 58415, 58423, 58427, 58429, 58430, 58447, 
    58455, 58459, 58461, 58462, 58471, 58475, 58477, 58478, 58483, 58485, 58486, 58489, 58490, 58492, 58511, 58519, 58523, 58525, 58526, 58535, 
    58539, 58541, 58542, 58547, 58549, 58550, 58553, 58554, 58556, 58567, 58571, 58573, 58574, 58579, 58581, 58582, 58585, 58586, 58588, 58595, 
    58597, 58598, 58601, 58602, 58604, 58609, 58610, 58612, 58616, 58639, 58647, 58651, 58653, 58654, 58663, 58667, 58669, 58670, 58675, 58677, 
    58678, 58681, 58682, 58684, 58695, 58699, 58701, 58702, 58707, 58709, 58710, 58713, 58714, 58716, 58723, 58725, 58726, 58729, 58730, 58732, 
    58737, 58738, 58740, 58744, 58759, 58763, 58765, 58766, 58771, 58773, 58774, 58777, 58778, 58780, 58787, 58789, 58790, 58793, 58794, 58796, 
    58801, 58802, 58804, 58808, 58819, 58821, 58822, 58825, 58826, 58828, 58833, 58834, 58836, 58840, 58849, 58850, 58852, 58856, 58864, 58895, 
    58903, 58907, 58909, 58910, 58919, 58923, 58925, 58926, 58931, 58933, 58934, 58937, 58938, 58940, 58951, 58955, 58957, 58958, 58963, 58965, 
    58966, 58969, 58970, 58972, 58979, 58981, 58982, 58985, 58986, 58988, 58993, 58994, 58996, 59000, 59015, 59019, 59021, 59022, 59027, 59029, 
    59030, 59033, 59034, 59036, 59043, 59045, 59046, 59049, 59050, 59052, 59057, 59058, 59060, 59064, 59075, 59077, 59078, 59081, 59082, 59084, 
    59089, 59090, 59092, 59096, 59105, 59106, 59108, 59112, 59120, 59143, 59147, 59149, 59150, 59155, 59157, 59158, 59161, 59162, 59164, 59171, 
    59173, 59174, 59177, 59178, 59180, 59185, 59186, 59188, 59192, 59203, 59205, 59206, 59209, 59210, 59212, 59217, 59218, 59220, 59224, 59233, 
    59234, 59236, 59240, 59248, 59267, 59269, 59270, 59273, 59274, 59276, 59281, 59282, 59284, 59288, 59297, 59298, 59300, 59304, 59312, 59329, 
    59330, 59332, 59336, 59344, 59360, 59423, 59439, 59447, 59451, 59453, 59454, 59471, 59479, 59483, 59485, 59486, 59495, 59499, 59501, 59502, 
    59507, 59509, 59510, 59513, 59514, 59516, 59535, 59543, 59547, 59549, 59550, 59559, 59563, 59565, 59566, 59571, 59573, 59574, 59577, 59578, 
    59580, 59591, 59595, 59597, 59598, 59603, 59605, 59606, 59609, 59610, 59612, 59619, 59621, 59622, 59625, 59626, 59628, 59633, 59634, 59636, 
    59640, 59663, 59671, 59675, 59677, 59678, 59687, 59691, 59693, 59694, 59699, 59701, 59702, 59705, 59706, 59708, 59719, 59723, 59725, 59726, 
    59731, 59733, 59734, 59737, 59738, 59740, 59747, 59749, 59750, 59753, 59754, 59756, 59761, 59762, 59764, 59768, 59783, 59787, 59789, 59790, 
    59795, 59797, 59798, 59801, 59802, 59804, 59811, 59813, 59814, 59817, 59818, 59820, 59825, 59826, 59828, 59832, 59843, 59845, 59846, 59849, 
    59850, 59852, 59857, 59858, 59860, 59864, 59873, 59874, 59876, 59880, 59888, 59919, 59927, 59931, 59933, 59934, 59943, 59947, 59949, 59950, 
    59955, 59957, 59958, 59961, 59962, 59964, 59975, 59979, 59981, 59982, 59987, 59989, 59990, 59993, 59994, 59996, 60003, 60005, 60006, 60009, 
    60010, 60012, 60017, 60018, 60020, 60024, 60039, 60043, 60045, 60046, 60051, 60053, 60054, 60057, 60058, 60060, 60067, 60069, 60070, 60073, 
    60074, 60076, 60081, 60082, 60084, 60088, 60099, 60101, 60102, 60105, 60106, 60108, 60113, 60114, 60116, 60120, 60129, 60130, 60132, 60136, 
    60144, 60167, 60171, 60173, 60174, 60179, 60181, 60182, 60185, 60186, 60188, 60195, 60197, 60198, 60201, 60202, 60204, 60209, 60210, 60212, 
    60216, 60227, 60229, 60230, 60233, 60234, 60236, 60241, 60242, 60244, 60248, 60257, 60258, 60260, 60264, 60272, 60291, 60293, 60294, 60297, 
    60298, 60300, 60305, 60306, 60308, 60312, 60321, 60322, 60324, 60328, 60336, 60353, 60354, 60356, 60360, 60368, 60384, 60431, 60439, 60443, 
    60445, 60446, 60455, 60459, 60461, 60462, 60467, 60469, 60470, 60473, 60474, 60476, 60487, 60491, 60493, 60494, 60499, 60501, 60502, 60505, 
    60506, 60508, 60515, 60517, 60518, 60521, 60522, 60524, 60529, 60530, 60532, 60536, 60551, 60555, 60557, 60558, 60563, 60565, 60566, 60569, 
    60570, 60572, 60579, 60581, 60582, 60585, 60586, 60588, 60593, 60594, 60596, 60600, 60611, 60613, 60614, 60617, 60618, 60620, 60625, 60626, 
    60628, 60632, 60641, 60642, 60644, 60648, 60656, 60679, 60683, 60685, 60686, 60691, 60693, 60694, 60697, 60698, 60700, 60707, 60709, 60710, 
    60713, 60714, 60716, 60721, 60722, 60724, 60728, 60739, 60741, 60742, 60745, 60746, 60748, 60753, 60754, 60756, 60760, 60769, 60770, 60772, 
    60776, 60784, 60803, 60805, 60806, 60809, 60810, 60812, 60817, 60818, 60820, 60824, 60833, 60834, 60836, 60840, 60848, 60865, 60866, 60868, 
    60872, 60880, 60896, 60935, 60939, 60941, 60942, 60947, 60949, 60950, 60953, 60954, 60956, 60963, 60965, 60966, 60969, 60970, 60972, 60977, 
    60978, 60980, 60984, 60995, 60997, 60998, 61001, 61002, 61004, 61009, 61010, 61012, 61016, 61025, 61026, 61028, 61032, 61040, 61059, 61061, 
    61062, 61065, 61066, 61068, 61073, 61074, 61076, 61080, 61089, 61090, 61092, 61096, 61104, 61121, 61122, 61124, 61128, 61136, 61152, 61187, 
    61189, 61190, 61193, 61194, 61196, 61201, 61202, 61204, 61208, 61217, 61218, 61220, 61224, 61232, 61249, 61250, 61252, 61256, 61264, 61280, 
    61313, 61314, 61316, 61320, 61328, 61344, 61376, 61471, 61487, 61495, 61499, 61501, 61502, 61519, 61527, 61531, 61533, 61534, 61543, 61547, 
    61549, 61550, 61555, 61557, 61558, 61561, 61562, 61564, 61583, 61591, 61595, 61597, 61598, 61607, 61611, 61613, 61614, 61619, 61621, 61622, 
    61625, 61626, 61628, 61639, 61643, 61645, 61646, 61651, 61653, 61654, 61657, 61658, 61660, 61667, 61669, 61670, 61673, 61674, 61676, 61681, 
    61682, 61684, 61688, 61711, 61719, 61723, 61725, 61726, 61735, 61739, 61741, 61742, 61747, 61749, 61750, 61753, 61754, 61756, 61767, 61771, 
    61773, 61774, 61779, 61781, 61782, 61785, 61786, 61788, 61795, 61797, 61798, 61801, 61802, 61804, 61809, 61810, 61812, 61816, 61831, 61835, 
    61837, 61838, 61843, 61845, 61846, 61849, 61850, 61852, 61859, 61861, 61862, 61865, 61866, 61868, 61873, 61874, 61876, 61880, 61891, 61893, 
    61894, 61897, 61898, 61900, 61905, 61906, 61908, 61912, 61921, 61922, 61924, 61928, 61936, 61967, 61975, 61979, 61981, 61982, 61991, 61995, 
    61997, 61998, 62003, 62005, 62006, 62009, 62010, 62012, 62023, 62027, 62029, 62030, 62035, 62037, 62038, 62041, 62042, 62044, 62051, 62053, 
    62054, 62057, 62058, 62060, 62065, 62066, 62068, 62072, 62087, 62091, 62093, 62094, 62099, 62101, 62102, 62105, 62106, 62108, 62115, 62117, 
    62118, 62121, 62122, 62124, 62129, 62130, 62132, 62136, 62147, 62149, 62150, 62153, 62154, 62156, 62161, 62162, 62164, 62168, 62177, 62178, 
    62180, 62184, 62192, 62215, 62219, 62221, 62222, 62227, 62229, 62230, 62233, 62234, 62236, 62243, 62245, 62246, 62249, 62250, 62252, 62257, 
    62258, 62260, 62264, 62275, 62277, 62278, 62281, 62282, 62284, 62289, 62290, 62292, 62296, 62305, 62306, 62308, 62312, 62320, 62339, 62341, 
    62342, 62345, 62346, 62348, 62353, 62354, 62356, 62360, 62369, 62370, 62372, 62376, 62384, 62401, 62402, 62404, 62408, 62416, 62432, 62479, 
    62487, 62491, 62493, 62494, 62503, 62507, 62509, 62510, 62515, 62517, 62518, 62521, 62522, 62524, 62535, 62539, 62541, 62542, 62547, 62549, 
    62550, 62553, 62554, 62556, 62563, 62565, 62566, 62569, 62570, 62572, 62577, 62578, 62580, 62584, 62599, 62603, 62605, 62606, 62611, 62613, 
    62614, 62617, 62618, 62620, 62627, 62629, 62630, 62633, 62634, 62636, 62641, 62642, 62644, 62648, 62659, 62661, 62662, 62665, 62666, 62668, 
    62673, 62674, 62676, 62680, 62689, 62690, 62692, 62696, 62704, 62727, 62731, 62733, 62734, 62739, 62741, 62742, 62745, 62746, 62748, 62755, 
    62757, 62758, 62761, 62762, 62764, 62769, 62770, 62772, 62776, 62787, 62789, 62790, 62793, 62794, 62796, 62801, 62802, 62804, 62808, 62817, 
    62818, 62820, 62824, 62832, 62851, 62853, 62854, 62857, 62858, 62860, 62865, 62866, 62868, 62872, 62881, 62882, 62884, 62888, 62896, 62913, 
    62914, 62916, 62920, 62928, 62944, 62983, 62987, 62989, 62990, 62995, 62997, 62998, 63001, 63002, 63004, 63011, 63013, 63014, 63017, 63018, 
    63020, 63025, 63026, 63028, 63032, 63043, 63045, 63046, 63049, 63050, 63052, 63057, 63058, 63060, 63064, 63073, 63074, 63076, 63080, 63088, 
    63107, 63109, 63110, 63113, 63114, 63116, 63121, 63122, 63124, 63128, 63137, 63138, 63140, 63144, 63152, 63169, 63170, 63172, 63176, 63184, 
    63200, 63235, 63237, 63238, 63241, 63242, 63244, 63249, 63250, 63252, 63256, 63265, 63266, 63268, 63272, 63280, 63297, 63298, 63300, 63304, 
    63312, 63328, 63361, 63362, 63364, 63368, 63376, 63392, 63424, 63503, 63511, 63515, 63517, 63518, 63527, 63531, 63533, 63534, 63539, 63541, 
    63542, 63545, 63546, 63548, 63559, 63563, 63565, 63566, 63571, 63573, 63574, 63577, 63578, 63580, 63587, 63589, 63590, 63593, 63594, 63596, 
    63601, 63602, 63604, 63608, 63623, 63627, 63629, 63630, 63635, 63637, 63638, 63641, 63642, 63644, 63651, 63653, 63654, 63657, 63658, 63660, 
    63665, 63666, 63668, 63672, 63683, 63685, 63686, 63689, 63690, 63692, 63697, 63698, 63700, 63704, 63713, 63714, 63716, 63720, 63728, 63751, 
    63755, 63757, 63758, 63763, 63765, 63766, 63769, 63770, 63772, 63779, 63781, 63782, 63785, 63786, 63788, 63793, 63794, 63796, 63800, 63811, 
    63813, 63814, 63817, 63818, 63820, 63825, 63826, 63828, 63832, 63841, 63842, 63844, 63848, 63856, 63875, 63877, 63878, 63881, 63882, 63884, 
    63889, 63890, 63892, 63896, 63905, 63906, 63908, 63912, 63920, 63937, 63938, 63940, 63944, 63952, 63968, 64007, 64011, 64013, 64014, 64019, 
    64021, 64022, 64025, 64026, 64028, 64035, 64037, 64038, 64041, 64042, 64044, 64049, 64050, 64052, 64056, 64067, 64069, 64070, 64073, 64074, 
    64076, 64081, 64082, 64084, 64088, 64097, 64098, 64100, 64104, 64112, 64131, 64133, 64134, 64137, 64138, 64140, 64145, 64146, 64148, 64152, 
    64161, 64162, 64164, 64168, 64176, 64193, 64194, 64196, 64200, 64208, 64224, 64259, 64261, 64262, 64265, 64266, 64268, 64273, 64274, 64276, 
    64280, 64289, 64290, 64292, 64296, 64304, 64321, 64322, 64324, 64328, 64336, 64352, 64385, 64386, 64388, 64392, 64400, 64416, 64448, 64519, 
    64523, 64525, 64526, 64531, 64533, 64534, 64537, 64538, 64540, 64547, 64549, 64550, 64553, 64554, 64556, 64561, 64562, 64564, 64568, 64579, 
    64581, 64582, 64585, 64586, 64588, 64593, 64594, 64596, 64600, 64609, 64610, 64612, 64616, 64624, 64643, 64645, 64646, 64649, 64650, 64652, 
    64657, 64658, 64660, 64664, 64673, 64674, 64676, 64680, 64688, 64705, 64706, 64708, 64712, 64720, 64736, 64771, 64773, 64774, 64777, 64778, 
    64780, 64785, 64786, 64788, 64792, 64801, 64802, 64804, 64808, 64816, 64833, 64834, 64836, 64840, 64848, 64864, 64897, 64898, 64900, 64904, 
    64912, 64928, 64960, 65027, 65029, 65030, 65033, 65034, 65036, 65041, 65042, 65044, 65048, 65057, 65058, 65060, 65064, 65072, 65089, 65090, 
    65092, 65096, 65104, 65120, 65153, 65154, 65156, 65160, 65168, 65184, 65216, 65281, 65282, 65284, 65288, 65296, 65312, 65344, 65408, },
  { 1023, 1535, 1791, 1919, 1983, 2015, 2031, 2039, 2043, 2045, 2046, 2559, 2815, 2943, 3007, 3039, 3055, 3063, 3067, 3069, 3070, 
    3327, 3455, 3519, 3551, 3567, 3575, 3579, 3581, 3582, 3711, 3775, 3807, 3823, 3831, 3835, 3837, 3838, 3903, 3935, 3951, 
    3959, 3963, 3965, 3966, 3999, 4015, 4023, 4027, 4029, 4030, 4047, 4055, 4059, 4061, 4062, 4071, 4075, 4077, 4078, 4083, 
    4085, 4086, 4089, 4090, 4092, 4607, 4863, 4991, 5055, 5087, 5103, 5111, 5115, 5117, 5118, 5375, 5503, 5567, 5599, 5615, 
    5623, 5627, 5629, 5630, 5759, 5823, 5855, 5871, 5879, 5883, 5885, 5886, 5951, 5983, 5999, 6007, 6011, 6013, 6014, 6047, 
    6063, 6071, 6075, 6077, 6078, 6095, 6103, 6107, 6109, 6110, 6119, 6123, 6125, 6126, 6131, 6133, 6134, 6137, 6138, 6140, 
    6399, 6527, 6591, 6623, 6639, 6647, 6651, 6653, 6654, 6783, 6847, 6879, 6895, 6903, 6907, 6909, 6910, 6975, 7007, 7023, 
    7031, 7035, 7037, 7038, 7071, 7087, 7095, 7099, 7101, 7102, 7119, 7127, 7131, 7133, 7134, 7143, 7147, 7149, 7150, 7155, 
    7157, 7158, 7161, 7162, 7164, 7295, 7359, 7391, 7407, 7415, 7419, 7421, 7422, 7487, 7519, 7535, 7543, 7547, 7549, 7550, 
    7583, 7599, 7607, 7611, 7613, 7614, 7631, 7639, 7643, 7645, 7646, 7655, 7659, 7661, 7662, 7667, 7669, 7670, 7673, 7674, 
    7676, 7743, 7775, 7791, 7799, 7803, 7805, 7806, 7839, 7855, 7863, 7867, 7869, 7870, 7887, 7895, 7899, 7901, 7902, 7911, 
    7915, 7917, 7918, 7923, 7925, 7926, 7929, 7930, 7932, 7967, 7983, 7991, 7995, 7997, 7998, 8015, 8023, 8027, 8029, 8030, 
    8039, 8043, 8045, 8046, 8051, 8053, 8054, 8057, 8058, 8060, 8079, 8087, 8091, 8093, 8094, 8103, 8107, 8109, 8110, 8115, 
    8117, 8118, 8121, 8122, 8124, 8135, 8139, 8141, 8142, 8147, 8149, 8150, 8153, 8154, 8156, 8163, 8165, 8166, 8169, 8170, 
    8172, 8177, 8178, 8180, 8184, 8703, 8959, 9087, 9151, 9183, 9199, 9207, 9211, 9213, 9214, 9471, 9599, 9663, 9695, 9711, 
    9719, 9723, 9725, 9726, 9855, 9919, 9951, 9967, 9975, 9979, 9981, 9982, 10047, 10079, 10095, 10103, 10107, 10109, 10110, 10143, 
    10159, 10167, 10171, 10173, 10174, 10191, 10199, 10203, 10205, 10206, 10215, 10219, 10221, 10222, 10227, 10229, 10230, 10233, 10234, 10236, 
    10495, 10623, 10687, 10719, 10735, 10743, 10747, 10749, 10750, 10879, 10943, 10975, 10991, 10999, 11003, 11005, 11006, 11071, 11103, 11119, 
    11127, 11131, 11133, 11134, 11167, 11183, 11191, 11195, 11197, 11198, 11215, 11223, 11227, 11229, 11230, 11239, 11243, 11245, 11246, 11251, 
    11253, 11254, 11257, 11258, 11260, 11391, 11455, 11487, 11503, 11511, 11515, 11517, 11518, 11583, 11615, 11631, 11639, 11643, 11645, 11646, 
    11679, 11695, 11703, 11707, 11709, 11710, 11727, 11735, 11739, 11741, 11742, 11751, 11755, 11757, 11758, 11763, 11765, 11766, 11769, 11770, 
    11772, 11839, 11871, 11887, 11895, 11899, 11901, 11902, 11935, 11951, 11959, 11963, 11965, 11966, 11983, 11991, 11995, 11997, 11998, 12007, 
    12011, 12013, 12014, 12019, 12021, 12022, 12025, 12026, 12028, 12063, 12079, 12087, 12091, 12093, 12094, 12111, 12119, 12123, 12125, 12126, 
    12135, 12139, 12141, 12142, 12147, 12149, 12150, 12153, 12154, 12156, 12175, 12183, 12187, 12189, 12190, 12199, 12203, 12205, 12206, 12211, 
    12213, 12214, 12217, 12218, 12220, 12231, 12235, 12237, 12238, 12243, 12245, 12246, 12249, 12250, 12252, 12259, 12261, 12262, 12265, 12266, 
    12268, 12273, 12274, 12276, 12280, 12543, 12671, 12735, 12767, 12783, 12791, 12795, 12797, 12798, 12927, 12991, 13023, 13039, 13047, 13051, 
    13053, 13054, 13119, 13151, 13167, 13175, 13179, 13181, 13182, 13215, 13231, 13239, 13243, 13245, 13246, 13263, 13271, 13275, 13277, 13278, 
    13287, 13291, 13293, 13294, 13299, 13301, 13302, 13305, 13306, 13308, 13439, 13503, 13535, 13551, 13559, 13563, 13565, 13566, 13631, 13663, 
    13679, 13687, 13691, 13693, 13694, 13727, 13743, 13751, 13755, 13757, 13758, 13775, 13783, 13787, 13789, 13790, 13799, 13803, 13805, 13806, 
    13811, 13813, 13814, 13817, 13818, 13820, 13887, 13919, 13935, 13943, 13947, 13949, 13950, 13983, 13999, 14007, 14011, 14013, 14014, 14031, 
    14039, 14043, 14045, 14046, 14055, 14059, 14061, 14062, 14067, 14069, 14070, 14073, 14074, 14076, 14111, 14127, 14135, 14139, 14141, 14142, 
    14159, 14167, 14171, 14173, 14174, 14183, 14187, 14189, 14190, 14195, 14197, 14198, 14201, 14202, 14204, 14223, 14231, 14235, 14237, 14238, 
    14247, 14251, 14253, 14254, 14259, 14261, 14262, 14265, 14266, 14268, 14279, 14283, 14285, 14286, 14291, 14293, 14294, 14297, 14298, 14300, 
    14307, 14309, 14310, 14313, 14314, 14316, 14321, 14322, 14324, 14328, 14463, 14527, 14559, 14575, 14583, 14587, 14589, 14590, 14655, 14687, 
    14703, 14711, 14715, 14717, 14718, 14751, 14767, 14775, 14779, 14781, 14782, 14799, 14807, 14811, 14813, 14814, 14823, 14827, 14829, 14830, 
    14835, 14837, 14838, 14841, 14842, 14844, 14911, 14943, 14959, 14967, 14971, 14973, 14974, 15007, 15023, 15031, 15035, 15037, 15038, 15055, 
    15063, 15067, 15069, 15070, 15079, 15083, 15085, 15086, 15091, 15093, 15094, 15097, 15098, 15100, 15135, 15151, 15159, 15163, 15165, 15166, 
    15183, 15191, 15195, 15197, 15198, 15207, 15211, 15213, 15214, 15219, 15221, 15222, 15225, 15226, 15228, 15247, 15255, 15259, 15261, 15262, 
    15271, 15275, 15277, 15278, 15283, 15285, 15286, 15289, 15290, 15292, 15303, 15307, 15309, 15310, 15315, 15317, 15318, 15321, 15322, 15324, 
    15331, 15333, 15334, 15337, 15338, 15340, 15345, 15346, 15348, 15352, 15423, 15455, 15471, 15479, 15483, 15485, 15486, 15519, 15535, 15543, 
    15547, 15549, 15550, 15567, 15575, 15579, 15581, 15582, 15591, 15595, 15597, 15598, 15603, 15605, 15606, 15609, 15610, 15612, 15647, 15663, 
    15671, 15675, 15677, 15678, 15695, 15703, 15707, 15709, 15710, 15719, 15723, 15725, 15726, 15731, 15733, 15734, 15737, 15738, 15740, 15759, 
    15767, 15771, 15773, 15774, 15783, 15787, 15789, 15790, 15795, 15797, 15798, 15801, 15802, 15804, 15815, 15819, 15821, 15822, 15827, 15829, 
    15830, 15833, 15834, 15836, 15843, 15845, 15846, 15849, 15850, 15852, 15857, 15858, 15860, 15864, 15903, 15919, 15927, 15931, 15933, 15934, 
    15951, 15959, 15963, 15965, 15966, 15975, 15979, 15981, 15982, 15987, 15989, 15990, 15993, 15994, 15996, 16015, 16023, 16027, 16029, 16030, 
    16039, 16043, 16045, 16046, 16051, 16053, 16054, 16057, 16058, 16060, 16071, 16075, 16077, 16078, 16083, 16085, 16086, 16089, 16090, 16092, 
    16099, 16101, 16102, 16105, 16106, 16108, 16113, 16114, 16116, 16120, 16143, 16151, 16155, 16157, 16158, 16167, 16171, 16173, 16174, 16179, 
    16181, 16182, 16185, 16186, 16188, 16199, 16203, 16205, 16206, 16211, 16213, 16214, 16217, 16218, 16220, 16227, 16229, 16230, 16233, 16234, 
    16236, 16241, 16242, 16244, 16248, 16263, 16267, 16269, 16270, 16275, 16277, 16278, 16281, 16282, 16284, 16291, 16293, 16294, 16297, 16298, 
    16300, 16305, 16306, 16308, 16312, 16323, 16325, 16326, 16329, 16330, 16332, 16337, 16338, 16340, 16344, 16353, 16354, 16356, 16360, 16368, 
    16895, 17151, 17279, 17343, 17375, 17391, 17399, 17403, 17405, 17406, 17663, 17791, 17855, 17887, 17903, 17911, 17915, 17917, 17918, 18047, 
    18111, 18143, 18159, 18167, 18171, 18173, 18174, 18239, 18271, 18287, 18295, 18299, 18301, 18302, 18335, 18351, 18359, 18363, 18365, 18366, 
    18383, 18391, 18395, 18397, 18398, 18407, 18411, 18413, 18414, 18419, 18421, 18422, 18425, 18426, 18428, 18687, 18815, 18879, 18911, 18927, 
    18935, 18939, 18941, 18942, 19071, 19135, 19167, 19183, 19191, 19195, 19197, 19198, 19263, 19295, 19311, 19319, 19323, 19325, 19326, 19359, 
    19375, 19383, 19387, 19389, 19390, 19407, 19415, 19419, 19421, 19422, 19431, 19435, 19437, 19438, 19443, 19445, 19446, 19449, 19450, 19452, 
    19583, 19647, 19679, 19695, 19703, 19707, 19709, 19710, 19775, 19807, 19823, 19831, 19835, 19837, 19838, 19871, 19887, 19895, 19899, 19901, 
    19902, 19919, 19927, 19931, 19933, 19934, 19943, 19947, 19949, 19950, 19955, 19957, 19958, 19961, 19962, 19964, 20031, 20063, 20079, 20087, 
    20091, 20093, 20094, 20127, 20143, 20151, 20155, 20157, 20158, 20175, 20183, 20187, 20189, 20190, 20199, 20203, 20205, 20206, 20211, 20213, 
    20214, 20217, 20218, 20220, 20255, 20271, 20279, 20283, 20285, 20286, 20303, 20311, 20315, 20317, 20318, 20327, 20331, 20333, 20334, 20339, 
    20341, 20342, 20345, 20346, 20348, 20367, 20375, 20379, 20381, 20382, 20391, 20395, 20397, 20398, 20403, 20405, 20406, 20409, 20410, 20412, 
    20423, 20427, 20429, 20430, 20435, 20437, 20438, 20441, 20442, 20444, 20451, 20453, 20454, 20457, 20458, 20460, 20465, 20466, 20468, 20472, 
    20735, 20863, 20927, 20959, 20975, 20983, 20987, 20989, 20990, 21119, 21183, 21215, 21231, 21239, 21243, 21245, 21246, 21311, 21343, 21359, 
    21367, 21371, 21373, 21374, 21407, 21423, 21431, 21435, 21437, 21438, 21455, 21463, 21467, 21469, 21470, 21479, 21483, 21485, 21486, 21491, 
    21493, 21494, 21497, 21498, 21500, 21631, 21695, 21727, 21743, 21751, 21755, 21757, 21758, 21823, 21855, 21871, 21879, 21883, 21885, 21886, 
    21919, 21935, 21943, 21947, 21949, 21950, 21967, 21975, 21979, 21981, 21982, 21991, 21995, 21997, 21998, 22003, 22005, 22006, 22009, 22010, 
    22012, 22079, 22111, 22127, 22135, 22139, 22141, 22142, 22175, 22191, 22199, 22203, 22205, 22206, 22223, 22231, 22235, 22237, 22238, 22247, 
    22251, 22253, 22254, 22259, 22261, 22262, 22265, 22266, 22268, 22303, 22319, 22327, 22331, 22333, 22334, 22351, 22359, 22363, 22365, 22366, 
    22375, 22379, 22381, 22382, 22387, 22389, 22390, 22393, 22394, 22396, 22415, 22423, 22427, 22429, 22430, 22439, 22443, 22445, 22446, 22451, 
    22453, 22454, 22457, 22458, 22460, 22471, 22475, 22477, 22478, 22483, 22485, 22486, 22489, 22490, 22492, 22499, 22501, 22502, 22505, 22506, 
    22508, 22513, 22514, 22516, 22520, 22655, 22719, 22751, 22767, 22775, 22779, 22781, 22782, 22847, 22879, 22895, 22903, 22907, 22909, 22910, 
    22943, 22959, 22967, 22971, 22973, 22974, 22991, 22999, 23003, 23005, 23006, 23015, 23019, 23021, 23022, 23027, 23029, 23030, 23033, 23034, 
    23036, 23103, 23135, 23151, 23159, 23163, 23165, 23166, 23199, 23215, 23223, 23227, 23229, 23230, 23247, 23255, 23259, 23261, 23262, 23271, 
    23275, 23277, 23278, 23283, 23285, 23286, 23289, 23290, 23292, 23327, 23343, 23351, 23355, 23357, 23358, 23375, 23383, 23387, 23389, 23390, 
    23399, 23403, 23405, 23406, 23411, 23413, 23414, 23417, 23418, 23420, 23439, 23447, 23451, 23453, 23454, 23463, 23467, 23469, 23470, 23475, 
    23477, 23478, 23481, 23482, 23484, 23495, 23499, 23501, 23502, 23507, 23509, 23510, 23513, 23514, 23516, 23523, 23525, 23526, 23529, 23530, 
    23532, 23537, 23538, 23540, 23544, 23615, 23647, 23663, 23671, 23675, 23677, 23678, 23711, 23727, 23735, 23739, 23741, 23742, 23759, 23767, 
    23771, 23773, 23774, 23783, 23787, 23789, 23790, 23795, 23797, 23798, 23801, 23802, 23804, 23839, 23855, 23863, 23867, 23869, 23870, 23887, 
    23895, 23899, 23901, 23902, 23911, 23915, 23917, 23918, 23923, 23925, 23926, 23929, 23930, 23932, 23951, 23959, 23963, 23965, 23966, 23975, 
    23979, 23981, 23982, 23987, 23989, 23990, 23993, 23994, 23996, 24007, 24011, 24013, 24014, 24019, 24021, 24022, 24025, 24026, 24028, 24035, 
    24037, 24038, 24041, 24042, 24044, 24049, 24050, 24052, 24056, 24095, 24111, 24119, 24123, 24125, 24126, 24143, 24151, 24155, 24157, 24158, 
    24167, 24171, 24173, 24174, 24179, 24181, 24182, 24185, 24186, 24188, 24207, 24215, 24219, 24221, 24222, 24231, 24235, 24237, 24238, 24243, 
    24245, 24246, 24249, 24250, 24252, 24263, 24267, 24269, 24270, 24275, 24277, 24278, 24281, 24282, 24284, 24291, 24293, 24294, 24297, 24298, 
    24300, 24305, 24306, 24308, 24312, 24335, 24343, 24347, 24349, 24350, 24359, 24363, 24365, 24366, 24371, 24373, 24374, 24377, 24378, 24380, 
    24391, 24395, 24397, 24398, 24403, 24405, 24406, 24409, 24410, 24412, 24419, 24421, 24422, 24425, 24426, 24428, 24433, 24434, 24436, 24440, 
    24455, 24459, 24461, 24462, 24467, 24469, 24470, 24473, 24474, 24476, 24483, 24485, 24486, 24489, 24490, 24492, 24497, 24498, 24500, 24504, 
    24515, 24517, 24518, 24521, 24522, 24524, 24529, 24530, 24532, 24536, 24545, 24546, 24548, 24552, 24560, 24831, 24959, 25023, 25055, 25071, 
    25079, 25083, 25085, 25086, 25215, 25279, 25311, 25327, 25335, 25339, 25341, 25342, 25407, 25439, 25455, 25463, 25467, 25469, 25470, 25503, 
    25519, 25527, 25531, 25533, 25534, 25551, 25559, 25563, 25565, 25566, 25575, 25579, 25581, 25582, 25587, 25589, 25590, 25593, 25594, 25596, 
    25727, 25791, 25823, 25839, 25847, 25851, 25853, 25854, 25919, 25951, 25967, 25975, 25979, 25981, 25982, 26015, 26031, 26039, 26043, 26045, 
    26046, 26063, 26071, 26075, 26077, 26078, 26087, 26091, 26093, 26094, 26099, 26101, 26102, 26105, 26106, 26108, 26175, 26207, 26223, 26231, 
    26235, 26237, 26238, 26271, 26287, 26295, 26299, 26301, 26302, 26319, 26327, 26331, 26333, 26334, 26343, 26347, 26349, 26350, 26355, 26357, 
    26358, 26361, 26362, 26364, 26399, 26415, 26423, 26427, 26429, 26430, 26447, 26455, 26459, 26461, 26462, 26471, 26475, 26477, 26478, 26483, 
    26485, 26486, 26489, 26490, 26492, 26511, 26519, 26523, 26525, 26526, 26535, 26539, 26541, 26542, 26547, 26549, 26550, 26553, 26554, 26556, 
    26567, 26571, 26573, 26574, 26579, 26581, 26582, 26585, 26586, 26588, 26595, 26597, 26598, 26601, 26602, 26604, 26609, 26610, 26612, 26616, 
    26751, 26815, 26847, 26863, 26871, 26875, 26877, 26878, 26943, 26975, 26991, 26999, 27003, 27005, 27006, 27039, 27055, 27063, 27067, 27069, 
    27070, 27087, 27095, 27099, 27101, 27102, 27111, 27115, 27117, 27118, 27123, 27125, 27126, 27129, 27130, 27132, 27199, 27231, 27247, 27255, 
    27259, 27261, 27262, 27295, 27311, 27319, 27323, 27325, 27326, 27343, 27351, 27355, 27357, 27358, 27367, 27371, 27373, 27374, 27379, 27381, 
    27382, 27385, 27386, 27388, 27423, 27439, 27447, 27451, 27453, 27454, 27471, 27479, 27483, 27485, 27486, 27495, 27499, 27501, 27502, 27507, 
    27509, 27510, 27513, 27514, 27516, 27535, 27543, 27547, 27549, 27550, 27559, 27563, 27565, 27566, 27571, 27573, 27574, 27577, 27578, 27580, 
    27591, 27595, 27597, 27598, 27603, 27605, 27606, 27609, 27610, 27612, 27619, 27621, 27622, 27625, 27626, 27628, 27633, 27634, 27636, 27640, 
    27711, 27743, 27759, 27767, 27771, 27773, 27774, 27807, 27823, 27831, 27835, 27837, 27838, 27855, 27863, 27867, 27869, 27870, 27879, 27883, 
    27885, 27886, 27891, 27893, 27894, 27897, 27898, 27900, 27935, 27951, 27959, 27963, 27965, 27966, 27983, 27991, 27995, 27997, 27998, 28007, 
    28011, 28013, 28014, 28019, 28021, 28022, 28025, 28026, 28028, 28047, 28055, 28059, 28061, 28062, 28071, 28075, 28077, 28078, 28083, 28085, 
    28086, 28089, 28090, 28092, 28103, 28107, 28109, 28110, 28115, 28117, 28118, 28121, 28122, 28124, 28131, 28133, 28134, 28137, 28138, 28140, 
    28145, 28146, 28148, 28152, 28191, 28207, 28215, 28219, 28221, 28222, 28239, 28247, 28251, 28253, 28254, 28263, 28267, 28269, 28270, 28275, 
    28277, 28278, 28281, 28282, 28284, 28303, 28311, 28315, 28317, 28318, 28327, 28331, 28333, 28334, 28339, 28341, 28342, 28345, 28346, 28348, 
    28359, 28363, 28365, 28366, 28371, 28373, 28374, 28377, 28378, 28380, 28387, 28389, 28390, 28393, 28394, 28396, 28401, 28402, 28404, 28408, 
    28431, 28439, 28443, 28445, 28446, 28455, 28459, 28461, 28462, 28467, 28469, 28470, 28473, 28474, 28476, 28487, 28491, 28493, 28494, 28499, 
    28501, 28502, 28505, 28506, 28508, 28515, 28517, 28518, 28521, 28522, 28524, 28529, 28530, 28532, 28536, 28551, 28555, 28557, 28558, 28563, 
    28565, 28566, 28569, 28570, 28572, 28579, 28581, 28582, 28585, 28586, 28588, 28593, 28594, 28596, 28600, 28611, 28613, 28614, 28617, 28618, 
    28620, 28625, 28626, 28628, 28632, 28641, 28642, 28644, 28648, 28656, 28799, 28863, 28895, 28911, 28919, 28923, 28925, 28926, 28991, 29023, 
    29039, 29047, 29051, 29053, 29054, 29087, 29103, 29111, 29115, 29117, 29118, 29135, 29143, 29147, 29149, 29150, 29159, 29163, 29165, 29166, 
    29171, 29173, 29174, 29177, 29178, 29180, 29247, 29279, 29295, 29303, 29307, 29309, 29310, 29343, 29359, 29367, 29371, 29373, 29374, 29391, 
    29399, 29403, 29405, 29406, 29415, 29419, 29421, 29422, 29427, 29429, 29430, 29433, 29434, 29436, 29471, 29487, 29495, 29499, 29501, 29502, 
    29519, 29527, 29531, 29533, 29534, 29543, 29547, 29549, 29550, 29555, 29557, 29558, 29561, 29562, 29564, 29583, 29591, 29595, 29597, 29598, 
    29607, 29611, 29613, 29614, 29619, 29621, 29622, 29625, 29626, 29628, 29639, 29643, 29645, 29646, 29651, 29653, 29654, 29657, 29658, 29660, 
    29667, 29669, 29670, 29673, 29674, 29676, 29681, 29682, 29684, 29688, 29759, 29791, 29807, 29815, 29819, 29821, 29822, 29855, 29871, 29879, 
    29883, 29885, 29886, 29903, 29911, 29915, 29917, 29918, 29927, 29931, 29933, 29934, 29939, 29941, 29942, 29945, 29946, 29948, 29983, 29999, 
    30007, 30011, 30013, 30014, 30031, 30039, 30043, 30045, 30046, 30055, 30059, 30061, 30062, 30067, 30069, 30070, 30073, 30074, 30076, 30095, 
    30103, 30107, 30109, 30110, 30119, 30123, 30125, 30126, 30131, 30133, 30134, 30137, 30138, 30140, 30151, 30155, 30157, 30158, 30163, 30165, 
    30166, 30169, 30170, 30172, 30179, 30181, 30182, 30185, 30186, 30188, 30193, 30194, 30196, 30200, 30239, 30255, 30263, 30267, 30269, 30270, 
    30287, 30295, 30299, 30301, 30302, 30311, 30315, 30317, 30318, 30323, 30325, 30326, 30329, 30330, 30332, 30351, 30359, 30363, 30365, 30366, 
    30375, 30379, 30381, 30382, 30387, 30389, 30390, 30393, 30394, 30396, 30407, 30411, 30413, 30414, 30419, 30421, 30422, 30425, 30426, 30428, 
    30435, 30437, 30438, 30441, 30442, 30444, 30449, 30450, 30452, 30456, 30479, 30487, 30491, 30493, 30494, 30503, 30507, 30509, 30510, 30515, 
    30517, 30518, 30521, 30522, 30524, 30535, 30539, 30541, 30542, 30547, 30549, 30550, 30553, 30554, 30556, 30563, 30565, 30566, 30569, 30570, 
    30572, 30577, 30578, 30580, 30584, 30599, 30603, 30605, 30606, 30611, 30613, 30614, 30617, 30618, 30620, 30627, 30629, 30630, 30633, 30634, 
    30636, 30641, 30642, 30644, 30648, 30659, 30661, 30662, 30665, 30666, 30668, 30673, 30674, 30676, 30680, 30689, 30690, 30692, 30696, 30704, 
    30783, 30815, 30831, 30839, 30843, 30845, 30846, 30879, 30895, 30903, 30907, 30909, 30910, 30927, 30935, 30939, 30941, 30942, 30951, 30955, 
    30957, 30958, 30963, 30965, 30966, 30969, 30970, 30972, 31007, 31023, 31031, 31035, 31037, 31038, 31055, 31063, 31067, 31069, 31070, 31079, 
    31083, 31085, 31086, 31091, 31093, 31094, 31097, 31098, 31100, 31119, 31127, 31131, 31133, 31134, 31143, 31147, 31149, 31150, 31155, 31157, 
    31158, 31161, 31162, 31164, 31175, 31179, 31181, 31182, 31187, 31189, 31190, 31193, 31194, 31196, 31203, 31205, 31206, 31209, 31210, 31212, 
    31217, 31218, 31220, 31224, 31263, 31279, 31287, 31291, 31293, 31294, 31311, 31319, 31323, 31325, 31326, 31335, 31339, 31341, 31342, 31347, 
    31349, 31350, 31353, 31354, 31356, 31375, 31383, 31387, 31389, 31390, 31399, 31403, 31405, 31406, 31411, 31413, 31414, 31417, 31418, 31420, 
    31431, 31435, 31437, 31438, 31443, 31445, 31446, 31449, 31450, 31452, 31459, 31461, 31462, 31465, 31466, 31468, 31473, 31474, 31476, 31480, 
    31503, 31511, 31515, 31517, 31518, 31527, 31531, 31533, 31534, 31539, 31541, 31542, 31545, 31546, 31548, 31559, 31563, 31565, 31566, 31571, 
    31573, 31574, 31577, 31578, 31580, 31587, 31589, 31590, 31593, 31594, 31596, 31601, 31602, 31604, 31608, 31623, 31627, 31629, 31630, 31635, 
    31637, 31638, 31641, 31642, 31644, 31651, 31653, 31654, 31657, 31658, 31660, 31665, 31666, 31668, 31672, 31683, 31685, 31686, 31689, 31690, 
    31692, 31697, 31698, 31700, 31704, 31713, 31714, 31716, 31720, 31728, 31775, 31791, 31799, 31803, 31805, 31806, 31823, 31831, 31835, 31837, 
    31838, 31847, 31851, 31853, 31854, 31859, 31861, 31862, 31865, 31866, 31868, 31887, 31895, 31899, 31901, 31902, 31911, 31915, 31917, 31918, 
    31923, 31925, 31926, 31929, 31930, 31932, 31943, 31947, 31949, 31950, 31955, 31957, 31958, 31961, 31962, 31964, 31971, 31973, 31974, 31977, 
    31978, 31980, 31985, 31986, 31988, 31992, 32015, 32023, 32027, 32029, 32030, 32039, 32043, 32045, 32046, 32051, 32053, 32054, 32057, 32058, 
    32060, 32071, 32075, 32077, 32078, 32083, 32085, 32086, 32089, 32090, 32092, 32099, 32101, 32102, 32105, 32106, 32108, 32113, 32114, 32116, 
    32120, 32135, 32139, 32141, 32142, 32147, 32149, 32150, 32153, 32154, 32156, 32163, 32165, 32166, 32169, 32170, 32172, 32177, 32178, 32180, 
    32184, 32195, 32197, 32198, 32201, 32202, 32204, 32209, 32210, 32212, 32216, 32225, 32226, 32228, 32232, 32240, 32271, 32279, 32283, 32285, 
    32286, 32295, 32299, 32301, 32302, 32307, 32309, 32310, 32313, 32314, 32316, 32327, 32331, 32333, 32334, 32339, 32341, 32342, 32345, 32346, 
    32348, 32355, 32357, 32358, 32361, 32362, 32364, 32369, 32370, 32372, 32376, 32391, 32395, 32397, 32398, 32403, 32405, 32406, 32409, 32410, 
    32412, 32419, 32421, 32422, 32425, 32426, 32428, 32433, 32434, 32436, 32440, 32451, 32453, 32454, 32457, 32458, 32460, 32465, 32466, 32468, 
    32472, 32481, 32482, 32484, 32488, 32496, 32519, 32523, 32525, 32526, 32531, 32533, 32534, 32537, 32538, 32540, 32547, 32549, 32550, 32553, 
    32554, 32556, 32561, 32562, 32564, 32568, 32579, 32581, 32582, 32585, 32586, 32588, 32593, 32594, 32596, 32600, 32609, 32610, 32612, 32616, 
    32624, 32643, 32645, 32646, 32649, 32650, 32652, 32657, 32658, 32660, 32664, 32673, 32674, 32676, 32680, 32688, 32705, 32706, 32708, 32712, 
    32720, 32736, 33279, 33535, 33663, 33727, 33759, 33775, 33783, 33787, 33789, 33790, 34047, 34175, 34239, 34271, 34287, 34295, 34299, 34301, 
    34302, 34431, 34495, 34527, 34543, 34551, 34555, 34557, 34558, 34623, 34655, 34671, 34679, 34683, 34685, 34686, 34719, 34735, 34743, 34747, 
    34749, 34750, 34767, 34775, 34779, 34781, 34782, 34791, 34795, 34797, 34798, 34803, 34805, 34806, 34809, 34810, 34812, 35071, 35199, 35263, 
    35295, 35311, 35319, 35323, 35325, 35326, 35455, 35519, 35551, 35567, 35575, 35579, 35581, 35582, 35647, 35679, 35695, 35703, 35707, 35709, 
    35710, 35743, 35759, 35767, 35771, 35773, 35774, 35791, 35799, 35803, 35805, 35806, 35815, 35819, 35821, 35822, 35827, 35829, 35830, 35833, 
    35834, 35836, 35967, 36031, 36063, 36079, 36087, 36091, 36093, 36094, 36159, 36191, 36207, 36215, 36219, 36221, 36222, 36255, 36271, 36279, 
    36283, 36285, 36286, 36303, 36311, 36315, 36317, 36318, 36327, 36331, 36333, 36334, 36339, 36341, 36342, 36345, 36346, 36348, 36415, 36447, 
    36463, 36471, 36475, 36477, 36478, 36511, 36527, 36535, 36539, 36541, 36542, 36559, 36567, 36571, 36573, 36574, 36583, 36587, 36589, 36590, 
    36595, 36597, 36598, 36601, 36602, 36604, 36639, 36655, 36663, 36667, 36669, 36670, 36687, 36695, 36699, 36701, 36702, 36711, 36715, 36717, 
    36718, 36723, 36725, 36726, 36729, 36730, 36732, 36751, 36759, 36763, 36765, 36766, 36775, 36779, 36781, 36782, 36787, 36789, 36790, 36793, 
    36794, 36796, 36807, 36811, 36813, 36814, 36819, 36821, 36822, 36825, 36826, 36828, 36835, 36837, 36838, 36841, 36842, 36844, 36849, 36850, 
    36852, 36856, 37119, 37247, 37311, 37343, 37359, 37367, 37371, 37373, 37374, 37503, 37567, 37599, 37615, 37623, 37627, 37629, 37630, 37695, 
    37727, 37743, 37751, 37755, 37757, 37758, 37791, 37807, 37815, 37819, 37821, 37822, 37839, 37847, 37851, 37853, 37854, 37863, 37867, 37869, 
    37870, 37875, 37877, 37878, 37881, 37882, 37884, 38015, 38079, 38111, 38127, 38135, 38139, 38141, 38142, 38207, 38239, 38255, 38263, 38267, 
    38269, 38270, 38303, 38319, 38327, 38331, 38333, 38334, 38351, 38359, 38363, 38365, 38366, 38375, 38379, 38381, 38382, 38387, 38389, 38390, 
    38393, 38394, 38396, 38463, 38495, 38511, 38519, 38523, 38525, 38526, 38559, 38575, 38583, 38587, 38589, 38590, 38607, 38615, 38619, 38621, 
    38622, 38631, 38635, 38637, 38638, 38643, 38645, 38646, 38649, 38650, 38652, 38687, 38703, 38711, 38715, 38717, 38718, 38735, 38743, 38747, 
    38749, 38750, 38759, 38763, 38765, 38766, 38771, 38773, 38774, 38777, 38778, 38780, 38799, 38807, 38811, 38813, 38814, 38823, 38827, 38829, 
    38830, 38835, 38837, 38838, 38841, 38842, 38844, 38855, 38859, 38861, 38862, 38867, 38869, 38870, 38873, 38874, 38876, 38883, 38885, 38886, 
    38889, 38890, 38892, 38897, 38898, 38900, 38904, 39039, 39103, 39135, 39151, 39159, 39163, 39165, 39166, 39231, 39263, 39279, 39287, 39291, 
    39293, 39294, 39327, 39343, 39351, 39355, 39357, 39358, 39375, 39383, 39387, 39389, 39390, 39399, 39403, 39405, 39406, 39411, 39413, 39414, 
    39417, 39418, 39420, 39487, 39519, 39535, 39543, 39547, 39549, 39550, 39583, 39599, 39607, 39611, 39613, 39614, 39631, 39639, 39643, 39645, 
    39646, 39655, 39659, 39661, 39662, 39667, 39669, 39670, 39673, 39674, 39676, 39711, 39727, 39735, 39739, 39741, 39742, 39759, 39767, 39771, 
    39773, 39774, 39783, 39787, 39789, 39790, 39795, 39797, 39798, 39801, 39802, 39804, 39823, 39831, 39835, 39837, 39838, 39847, 39851, 39853, 
    39854, 39859, 39861, 39862, 39865, 39866, 39868, 39879, 39883, 39885, 39886, 39891, 39893, 39894, 39897, 39898, 39900, 39907, 39909, 39910, 
    39913, 39914, 39916, 39921, 39922, 39924, 39928, 39999, 40031, 40047, 40055, 40059, 40061, 40062, 40095, 40111, 40119, 40123, 40125, 40126, 
    40143, 40151, 40155, 40157, 40158, 40167, 40171, 40173, 40174, 40179, 40181, 40182, 40185, 40186, 40188, 40223, 40239, 40247, 40251, 40253, 
    40254, 40271, 40279, 40283, 40285, 40286, 40295, 40299, 40301, 40302, 40307, 40309, 40310, 40313, 40314, 40316, 40335, 40343, 40347, 40349, 
    40350, 40359, 40363, 40365, 40366, 40371, 40373, 40374, 40377, 40378, 40380, 40391, 40395, 40397, 40398, 40403, 40405, 40406, 40409, 40410, 
    40412, 40419, 40421, 40422, 40425, 40426, 40428, 40433, 40434, 40436, 40440, 40479, 40495, 40503, 40507, 40509, 40510, 40527, 40535, 40539, 
    40541, 40542, 40551, 40555, 40557, 40558, 40563, 40565, 40566, 40569, 40570, 40572, 40591, 40599, 40603, 40605, 40606, 40615, 40619, 40621, 
    40622, 40627, 40629, 40630, 40633, 40634, 40636, 40647, 40651, 40653, 40654, 40659, 40661, 40662, 40665, 40666, 40668, 40675, 40677, 40678, 
    40681, 40682, 40684, 40689, 40690, 40692, 40696, 40719, 40727, 40731, 40733, 40734, 40743, 40747, 40749, 40750, 40755, 40757, 40758, 40761, 
    40762, 40764, 40775, 40779, 40781, 40782, 40787, 40789, 40790, 40793, 40794, 40796, 40803, 40805, 40806, 40809, 40810, 40812, 40817, 40818, 
    40820, 40824, 40839, 40843, 40845, 40846, 40851, 40853, 40854, 40857, 40858, 40860, 40867, 40869, 40870, 40873, 40874, 40876, 40881, 40882, 
    40884, 40888, 40899, 40901, 40902, 40905, 40906, 40908, 40913, 40914, 40916, 40920, 40929, 40930, 40932, 40936, 40944, 41215, 41343, 41407, 
    41439, 41455, 41463, 41467, 41469, 41470, 41599, 41663, 41695, 41711, 41719, 41723, 41725, 41726, 41791, 41823, 41839, 41847, 41851, 41853, 
    41854, 41887, 41903, 41911, 41915, 41917, 41918, 41935, 41943, 41947, 41949, 41950, 41959, 41963, 41965, 41966, 41971, 41973, 41974, 41977, 
    41978, 41980, 42111, 42175, 42207, 42223, 42231, 42235, 42237, 42238, 42303, 42335, 42351, 42359, 42363, 42365, 42366, 42399, 42415, 42423, 
    42427, 42429, 42430, 42447, 42455, 42459, 42461, 42462, 42471, 42475, 42477, 42478, 42483, 42485, 42486, 42489, 42490, 42492, 42559, 42591, 
    42607, 42615, 42619, 42621, 42622, 42655, 42671, 42679, 42683, 42685, 42686, 42703, 42711, 42715, 42717, 42718, 42727, 42731, 42733, 42734, 
    42739, 42741, 42742, 42745, 42746, 42748, 42783, 42799, 42807, 42811, 42813, 42814, 42831, 42839, 42843, 42845, 42846, 42855, 42859, 42861, 
    42862, 42867, 42869, 42870, 42873, 42874, 42876, 42895, 42903, 42907, 42909, 42910, 42919, 42923, 42925, 42926, 42931, 42933, 42934, 42937, 
    42938, 42940, 42951, 42955, 42957, 42958, 42963, 42965, 42966, 42969, 42970, 42972, 42979, 42981, 42982, 42985, 42986, 42988, 42993, 42994, 
    42996, 43000, 43135, 43199, 43231, 43247, 43255, 43259, 43261, 43262, 43327, 43359, 43375, 43383, 43387, 43389, 43390, 43423, 43439, 43447, 
    43451, 43453, 43454, 43471, 43479, 43483, 43485, 43486, 43495, 43499, 43501, 43502, 43507, 43509, 43510, 43513, 43514, 43516, 43583, 43615, 
    43631, 43639, 43643, 43645, 43646, 43679, 43695, 43703, 43707, 43709, 43710, 43727, 43735, 43739, 43741, 43742, 43751, 43755, 43757, 43758, 
    43763, 43765, 43766, 43769, 43770, 43772, 43807, 43823, 43831, 43835, 43837, 43838, 43855, 43863, 43867, 43869, 43870, 43879, 43883, 43885, 
    43886, 43891, 43893, 43894, 43897, 43898, 43900, 43919, 43927, 43931, 43933, 43934, 43943, 43947, 43949, 43950, 43955, 43957, 43958, 43961, 
    43962, 43964, 43975, 43979, 43981, 43982, 43987, 43989, 43990, 43993, 43994, 43996, 44003, 44005, 44006, 44009, 44010, 44012, 44017, 44018, 
    44020, 44024, 44095, 44127, 44143, 44151, 44155, 44157, 44158, 44191, 44207, 44215, 44219, 44221, 44222, 44239, 44247, 44251, 44253, 44254, 
    44263, 44267, 44269, 44270, 44275, 44277, 44278, 44281, 44282, 44284, 44319, 44335, 44343, 44347, 44349, 44350, 44367, 44375, 44379, 44381, 
    44382, 44391, 44395, 44397, 44398, 44403, 44405, 44406, 44409, 44410, 44412, 44431, 44439, 44443, 44445, 44446, 44455, 44459, 44461, 44462, 
    44467, 44469, 44470, 44473, 44474, 44476, 44487, 44491, 44493, 44494, 44499, 44501, 44502, 44505, 44506, 44508, 44515, 44517, 44518, 44521, 
    44522, 44524, 44529, 44530, 44532, 44536, 44575, 44591, 44599, 44603, 44605, 44606, 44623, 44631, 44635, 44637, 44638, 44647, 44651, 44653, 
    44654, 44659, 44661, 44662, 44665, 44666, 44668, 44687, 44695, 44699, 44701, 44702, 44711, 44715, 44717, 44718, 44723, 44725, 44726, 44729, 
    44730, 44732, 44743, 44747, 44749, 44750, 44755, 44757, 44758, 44761, 44762, 44764, 44771, 44773, 44774, 44777, 44778, 44780, 44785, 44786, 
    44788, 44792, 44815, 44823, 44827, 44829, 44830, 44839, 44843, 44845, 44846, 44851, 44853, 44854, 44857, 44858, 44860, 44871, 44875, 44877, 
    44878, 44883, 44885, 44886, 44889, 44890, 44892, 44899, 44901, 44902, 44905, 44906, 44908, 44913, 44914, 44916, 44920, 44935, 44939, 44941, 
    44942, 44947, 44949, 44950, 44953, 44954, 44956, 44963, 44965, 44966, 44969, 44970, 44972, 44977, 44978, 44980, 44984, 44995, 44997, 44998, 
    45001, 45002, 45004, 45009, 45010, 45012, 45016, 45025, 45026, 45028, 45032, 45040, 45183, 45247, 45279, 45295, 45303, 45307, 45309, 45310, 
    45375, 45407, 45423, 45431, 45435, 45437, 45438, 45471, 45487, 45495, 45499, 45501, 45502, 45519, 45527, 45531, 45533, 45534, 45543, 45547, 
    45549, 45550, 45555, 45557, 45558, 45561, 45562, 45564, 45631, 45663, 45679, 45687, 45691, 45693, 45694, 45727, 45743, 45751, 45755, 45757, 
    45758, 45775, 45783, 45787, 45789, 45790, 45799, 45803, 45805, 45806, 45811, 45813, 45814, 45817, 45818, 45820, 45855, 45871, 45879, 45883, 
    45885, 45886, 45903, 45911, 45915, 45917, 45918, 45927, 45931, 45933, 45934, 45939, 45941, 45942, 45945, 45946, 45948, 45967, 45975, 45979, 
    45981, 45982, 45991, 45995, 45997, 45998, 46003, 46005, 46006, 46009, 46010, 46012, 46023, 46027, 46029, 46030, 46035, 46037, 46038, 46041, 
    46042, 46044, 46051, 46053, 46054, 46057, 46058, 46060, 46065, 46066, 46068, 46072, 46143, 46175, 46191, 46199, 46203, 46205, 46206, 46239, 
    46255, 46263, 46267, 46269, 46270, 46287, 46295, 46299, 46301, 46302, 46311, 46315, 46317, 46318, 46323, 46325, 46326, 46329, 46330, 46332, 
    46367, 46383, 46391, 46395, 46397, 46398, 46415, 46423, 46427, 46429, 46430, 46439, 46443, 46445, 46446, 46451, 46453, 46454, 46457, 46458, 
    46460, 46479, 46487, 46491, 46493, 46494, 46503, 46507, 46509, 46510, 46515, 46517, 46518, 46521, 46522, 46524, 46535, 46539, 46541, 46542, 
    46547, 46549, 46550, 46553, 46554, 46556, 46563, 46565, 46566, 46569, 46570, 46572, 46577, 46578, 46580, 46584, 46623, 46639, 46647, 46651, 
    46653, 46654, 46671, 46679, 46683, 46685, 46686, 46695, 46699, 46701, 46702, 46707, 46709, 46710, 46713, 46714, 46716, 46735, 46743, 46747, 
    46749, 46750, 46759, 46763, 46765, 46766, 46771, 46773, 46774, 46777, 46778, 46780, 46791, 46795, 46797, 46798, 46803, 46805, 46806, 46809, 
    46810, 46812, 46819, 46821, 46822, 46825, 46826, 46828, 46833, 46834, 46836, 46840, 46863, 46871, 46875, 46877, 46878, 46887, 46891, 46893, 
    46894, 46899, 46901, 46902, 46905, 46906, 46908, 46919, 46923, 46925, 46926, 46931, 46933, 46934, 46937, 46938, 46940, 46947, 46949, 46950, 
    46953, 46954, 46956, 46961, 46962, 46964, 46968, 46983, 46987, 46989, 46990, 46995, 46997, 46998, 47001, 47002, 47004, 47011, 47013, 47014, 
    47017, 47018, 47020, 47025, 47026, 47028, 47032, 47043, 47045, 47046, 47049, 47050, 47052, 47057, 47058, 47060, 47064, 47073, 47074, 47076, 
    47080, 47088, 47167, 47199, 47215, 47223, 47227, 47229, 47230, 47263, 47279, 47287, 47291, 47293, 47294, 47311, 47319, 47323, 47325, 47326, 
    47335, 47339, 47341, 47342, 47347, 47349, 47350, 47353, 47354, 47356, 47391, 47407, 47415, 47419, 47421, 47422, 47439, 47447, 47451, 47453, 
    47454, 47463, 47467, 47469, 47470, 47475, 47477, 47478, 47481, 47482, 47484, 47503, 47511, 47515, 47517, 47518, 47527, 47531, 47533, 47534, 
    47539, 47541, 47542, 47545, 47546, 47548, 47559, 47563, 47565, 47566, 47571, 47573, 47574, 47577, 47578, 47580, 47587, 47589, 47590, 47593, 
    47594, 47596, 47601, 47602, 47604, 47608, 47647, 47663, 47671, 47675, 47677, 47678, 47695, 47703, 47707, 47709, 47710, 47719, 47723, 47725, 
    47726, 47731, 47733, 47734, 47737, 47738, 47740, 47759, 47767, 47771, 47773, 47774, 47783, 47787, 47789, 47790, 47795, 47797, 47798, 47801, 
    47802, 47804, 47815, 47819, 47821, 47822, 47827, 47829, 47830, 47833, 47834, 47836, 47843, 47845, 47846, 47849, 47850, 47852, 47857, 47858, 
    47860, 47864, 47887, 47895, 47899, 47901, 47902, 47911, 47915, 47917, 47918, 47923, 47925, 47926, 47929, 47930, 47932, 47943, 47947, 47949, 
    47950, 47955, 47957, 47958, 47961, 47962, 47964, 47971, 47973, 47974, 47977, 47978, 47980, 47985, 47986, 47988, 47992, 48007, 48011, 48013, 
    48014, 48019, 48021, 48022, 48025, 48026, 48028, 48035, 48037, 48038, 48041, 48042, 48044, 48049, 48050, 48052, 48056, 48067, 48069, 48070, 
    48073, 48074, 48076, 48081, 48082, 48084, 48088, 48097, 48098, 48100, 48104, 48112, 48159, 48175, 48183, 48187, 48189, 48190, 48207, 48215, 
    48219, 48221, 48222, 48231, 48235, 48237, 48238, 48243, 48245, 48246, 48249, 48250, 48252, 48271, 48279, 48283, 48285, 48286, 48295, 48299, 
    48301, 48302, 48307, 48309, 48310, 48313, 48314, 48316, 48327, 48331, 48333, 48334, 48339, 48341, 48342, 48345, 48346, 48348, 48355, 48357, 
    48358, 48361, 48362, 48364, 48369, 48370, 48372, 48376, 48399, 48407, 48411, 48413, 48414, 48423, 48427, 48429, 48430, 48435, 48437, 48438, 
    48441, 48442, 48444, 48455, 48459, 48461, 48462, 48467, 48469, 48470, 48473, 48474, 48476, 48483, 48485, 48486, 48489, 48490, 48492, 48497, 
    48498, 48500, 48504, 48519, 48523, 48525, 48526, 48531, 48533, 48534, 48537, 48538, 48540, 48547, 48549, 48550, 48553, 48554, 48556, 48561, 
    48562, 48564, 48568, 48579, 48581, 48582, 48585, 48586, 48588, 48593, 48594, 48596, 48600, 48609, 48610, 48612, 48616, 48624, 48655, 48663, 
    48667, 48669, 48670, 48679, 48683, 48685, 48686, 48691, 48693, 48694, 48697, 48698, 48700, 48711, 48715, 48717, 48718, 48723, 48725, 48726, 
    48729, 48730, 48732, 48739, 48741, 48742, 48745, 48746, 48748, 48753, 48754, 48756, 48760, 48775, 48779, 48781, 48782, 48787, 48789, 48790, 
    48793, 48794, 48796, 48803, 48805, 48806, 48809, 48810, 48812, 48817, 48818, 48820, 48824, 48835, 48837, 48838, 48841, 48842, 48844, 48849, 
    48850, 48852, 48856, 48865, 48866, 48868, 48872, 48880, 48903, 48907, 48909, 48910, 48915, 48917, 48918, 48921, 48922, 48924, 48931, 48933, 
    48934, 48937, 48938, 48940, 48945, 48946, 48948, 48952, 48963, 48965, 48966, 48969, 48970, 48972, 48977, 48978, 48980, 48984, 48993, 48994, 
    48996, 49000, 49008, 49027, 49029, 49030, 49033, 49034, 49036, 49041, 49042, 49044, 49048, 49057, 49058, 49060, 49064, 49072, 49089, 49090, 
    49092, 49096, 49104, 49120, 49407, 49535, 49599, 49631, 49647, 49655, 49659, 49661, 49662, 49791, 49855, 49887, 49903, 49911, 49915, 49917, 
    49918, 49983, 50015, 50031, 50039, 50043, 50045, 50046, 50079, 50095, 50103, 50107, 50109, 50110, 50127, 50135, 50139, 50141, 50142, 50151, 
    50155, 50157, 50158, 50163, 50165, 50166, 50169, 50170, 50172, 50303, 50367, 50399, 50415, 50423, 50427, 50429, 50430, 50495, 50527, 50543, 
    50551, 50555, 50557, 50558, 50591, 50607, 50615, 50619, 50621, 50622, 50639, 50647, 50651, 50653, 50654, 50663, 50667, 50669, 50670, 50675, 
    50677, 50678, 50681, 50682, 50684, 50751, 50783, 50799, 50807, 50811, 50813, 50814, 50847, 50863, 50871, 50875, 50877, 50878, 50895, 50903, 
    50907, 50909, 50910, 50919, 50923, 50925, 50926, 50931, 50933, 50934, 50937, 50938, 50940, 50975, 50991, 50999, 51003, 51005, 51006, 51023, 
    51031, 51035, 51037, 51038, 51047, 51051, 51053, 51054, 51059, 51061, 51062, 51065, 51066, 51068, 51087, 51095, 51099, 51101, 51102, 51111, 
    51115, 51117, 51118, 51123, 51125, 51126, 51129, 51130, 51132, 51143, 51147, 51149, 51150, 51155, 51157, 51158, 51161, 51162, 51164, 51171, 
    51173, 51174, 51177, 51178, 51180, 51185, 51186, 51188, 51192, 51327, 51391, 51423, 51439, 51447, 51451, 51453, 51454, 51519, 51551, 51567, 
    51575, 51579, 51581, 51582, 51615, 51631, 51639, 51643, 51645, 51646, 51663, 51671, 51675, 51677, 51678, 51687, 51691, 51693, 51694, 51699, 
    51701, 51702, 51705, 51706, 51708, 51775, 51807, 51823, 51831, 51835, 51837, 51838, 51871, 51887, 51895, 51899, 51901, 51902, 51919, 51927, 
    51931, 51933, 51934, 51943, 51947, 51949, 51950, 51955, 51957, 51958, 51961, 51962, 51964, 51999, 52015, 52023, 52027, 52029, 52030, 52047, 
    52055, 52059, 52061, 52062, 52071, 52075, 52077, 52078, 52083, 52085, 52086, 52089, 52090, 52092, 52111, 52119, 52123, 52125, 52126, 52135, 
    52139, 52141, 52142, 52147, 52149, 52150, 52153, 52154, 52156, 52167, 52171, 52173, 52174, 52179, 52181, 52182, 52185, 52186, 52188, 52195, 
    52197, 52198, 52201, 52202, 52204, 52209, 52210, 52212, 52216, 52287, 52319, 52335, 52343, 52347, 52349, 52350, 52383, 52399, 52407, 52411, 
    52413, 52414, 52431, 52439, 52443, 52445, 52446, 52455, 52459, 52461, 52462, 52467, 52469, 52470, 52473, 52474, 52476, 52511, 52527, 52535, 
    52539, 52541, 52542, 52559, 52567, 52571, 52573, 52574, 52583, 52587, 52589, 52590, 52595, 52597, 52598, 52601, 52602, 52604, 52623, 52631, 
    52635, 52637, 52638, 52647, 52651, 52653, 52654, 52659, 52661, 52662, 52665, 52666, 52668, 52679, 52683, 52685, 52686, 52691, 52693, 52694, 
    52697, 52698, 52700, 52707, 52709, 52710, 52713, 52714, 52716, 52721, 52722, 52724, 52728, 52767, 52783, 52791, 52795, 52797, 52798, 52815, 
    52823, 52827, 52829, 52830, 52839, 52843, 52845, 52846, 52851, 52853, 52854, 52857, 52858, 52860, 52879, 52887, 52891, 52893, 52894, 52903, 
    52907, 52909, 52910, 52915, 52917, 52918, 52921, 52922, 52924, 52935, 52939, 52941, 52942, 52947, 52949, 52950, 52953, 52954, 52956, 52963, 
    52965, 52966, 52969, 52970, 52972, 52977, 52978, 52980, 52984, 53007, 53015, 53019, 53021, 53022, 53031, 53035, 53037, 53038, 53043, 53045, 
    53046, 53049, 53050, 53052, 53063, 53067, 53069, 53070, 53075, 53077, 53078, 53081, 53082, 53084, 53091, 53093, 53094, 53097, 53098, 53100, 
    53105, 53106, 53108, 53112, 53127, 53131, 53133, 53134, 53139, 53141, 53142, 53145, 53146, 53148, 53155, 53157, 53158, 53161, 53162, 53164, 
    53169, 53170, 53172, 53176, 53187, 53189, 53190, 53193, 53194, 53196, 53201, 53202, 53204, 53208, 53217, 53218, 53220, 53224, 53232, 53375, 
    53439, 53471, 53487, 53495, 53499, 53501, 53502, 53567, 53599, 53615, 53623, 53627, 53629, 53630, 53663, 53679, 53687, 53691, 53693, 53694, 
    53711, 53719, 53723, 53725, 53726, 53735, 53739, 53741, 53742, 53747, 53749, 53750, 53753, 53754, 53756, 53823, 53855, 53871, 53879, 53883, 
    53885, 53886, 53919, 53935, 53943, 53947, 53949, 53950, 53967, 53975, 53979, 53981, 53982, 53991, 53995, 53997, 53998, 54003, 54005, 54006, 
    54009, 54010, 54012, 54047, 54063, 54071, 54075, 54077, 54078, 54095, 54103, 54107, 54109, 54110, 54119, 54123, 54125, 54126, 54131, 54133, 
    54134, 54137, 54138, 54140, 54159, 54167, 54171, 54173, 54174, 54183, 54187, 54189, 54190, 54195, 54197, 54198, 54201, 54202, 54204, 54215, 
    54219, 54221, 54222, 54227, 54229, 54230, 54233, 54234, 54236, 54243, 54245, 54246, 54249, 54250, 54252, 54257, 54258, 54260, 54264, 54335, 
    54367, 54383, 54391, 54395, 54397, 54398, 54431, 54447, 54455, 54459, 54461, 54462, 54479, 54487, 54491, 54493, 54494, 54503, 54507, 54509, 
    54510, 54515, 54517, 54518, 54521, 54522, 54524, 54559, 54575, 54583, 54587, 54589, 54590, 54607, 54615, 54619, 54621, 54622, 54631, 54635, 
    54637, 54638, 54643, 54645, 54646, 54649, 54650, 54652, 54671, 54679, 54683, 54685, 54686, 54695, 54699, 54701, 54702, 54707, 54709, 54710, 
    54713, 54714, 54716, 54727, 54731, 54733, 54734, 54739, 54741, 54742, 54745, 54746, 54748, 54755, 54757, 54758, 54761, 54762, 54764, 54769, 
    54770, 54772, 54776, 54815, 54831, 54839, 54843, 54845, 54846, 54863, 54871, 54875, 54877, 54878, 54887, 54891, 54893, 54894, 54899, 54901, 
    54902, 54905, 54906, 54908, 54927, 54935, 54939, 54941, 54942, 54951, 54955, 54957, 54958, 54963, 54965, 54966, 54969, 54970, 54972, 54983, 
    54987, 54989, 54990, 54995, 54997, 54998, 55001, 55002, 55004, 55011, 55013, 55014, 55017, 55018, 55020, 55025, 55026, 55028, 55032, 55055, 
    55063, 55067, 55069, 55070, 55079, 55083, 55085, 55086, 55091, 55093, 55094, 55097, 55098, 55100, 55111, 55115, 55117, 55118, 55123, 55125, 
    55126, 55129, 55130, 55132, 55139, 55141, 55142, 55145, 55146, 55148, 55153, 55154, 55156, 55160, 55175, 55179, 55181, 55182, 55187, 55189, 
    55190, 55193, 55194, 55196, 55203, 55205, 55206, 55209, 55210, 55212, 55217, 55218, 55220, 55224, 55235, 55237, 55238, 55241, 55242, 55244, 
    55249, 55250, 55252, 55256, 55265, 55266, 55268, 55272, 55280, 55359, 55391, 55407, 55415, 55419, 55421, 55422, 55455, 55471, 55479, 55483, 
    55485, 55486, 55503, 55511, 55515, 55517, 55518, 55527, 55531, 55533, 55534, 55539, 55541, 55542, 55545, 55546, 55548, 55583, 55599, 55607, 
    55611, 55613, 55614, 55631, 55639, 55643, 55645, 55646, 55655, 55659, 55661, 55662, 55667, 55669, 55670, 55673, 55674, 55676, 55695, 55703, 
    55707, 55709, 55710, 55719, 55723, 55725, 55726, 55731, 55733, 55734, 55737, 55738, 55740, 55751, 55755, 55757, 55758, 55763, 55765, 55766, 
    55769, 55770, 55772, 55779, 55781, 55782, 55785, 55786, 55788, 55793, 55794, 55796, 55800, 55839, 55855, 55863, 55867, 55869, 55870, 55887, 
    55895, 55899, 55901, 55902, 55911, 55915, 55917, 55918, 55923, 55925, 55926, 55929, 55930, 55932, 55951, 55959, 55963, 55965, 55966, 55975, 
    55979, 55981, 55982, 55987, 55989, 55990, 55993, 55994, 55996, 56007, 56011, 56013, 56014, 56019, 56021, 56022, 56025, 56026, 56028, 56035, 
    56037, 56038, 56041, 56042, 56044, 56049, 56050, 56052, 56056, 56079, 56087, 56091, 56093, 56094, 56103, 56107, 56109, 56110, 56115, 56117, 
    56118, 56121, 56122, 56124, 56135, 56139, 56141, 56142, 56147, 56149, 56150, 56153, 56154, 56156, 56163, 56165, 56166, 56169, 56170, 56172, 
    56177, 56178, 56180, 56184, 56199, 56203, 56205, 56206, 56211, 56213, 56214, 56217, 56218, 56220, 56227, 56229, 56230, 56233, 56234, 56236, 
    56241, 56242, 56244, 56248, 56259, 56261, 56262, 56265, 56266, 56268, 56273, 56274, 56276, 56280, 56289, 56290, 56292, 56296, 56304, 56351, 
    56367, 56375, 56379, 56381, 56382, 56399, 56407, 56411, 56413, 56414, 56423, 56427, 56429, 56430, 56435, 56437, 56438, 56441, 56442, 56444, 
    56463, 56471, 56475, 56477, 56478, 56487, 56491, 56493, 56494, 56499, 56501, 56502, 56505, 56506, 56508, 56519, 56523, 56525, 56526, 56531, 
    56533, 56534, 56537, 56538, 56540, 56547, 56549, 56550, 56553, 56554, 56556, 56561, 56562, 56564, 56568, 56591, 56599, 56603, 56605, 56606, 
    56615, 56619, 56621, 56622, 56627, 56629, 56630, 56633, 56634, 56636, 56647, 56651, 56653, 56654, 56659, 56661, 56662, 56665, 56666, 56668, 
    56675, 56677, 56678, 56681, 56682, 56684, 56689, 56690, 56692, 56696, 56711, 56715, 56717, 56718, 56723, 56725, 56726, 56729, 56730, 56732, 
    56739, 56741, 56742, 56745, 56746, 56748, 56753, 56754, 56756, 56760, 56771, 56773, 56774, 56777, 56778, 56780, 56785, 56786, 56788, 56792, 
    56801, 56802, 56804, 56808, 56816, 56847, 56855, 56859, 56861, 56862, 56871, 56875, 56877, 56878, 56883, 56885, 56886, 56889, 56890, 56892, 
    56903, 56907, 56909, 56910, 56915, 56917, 56918, 56921, 56922, 56924, 56931, 56933, 56934, 56937, 56938, 56940, 56945, 56946, 56948, 56952, 
    56967, 56971, 56973, 56974, 56979, 56981, 56982, 56985, 56986, 56988, 56995, 56997, 56998, 57001, 57002, 57004, 57009, 57010, 57012, 57016, 
    57027, 57029, 57030, 57033, 57034, 57036, 57041, 57042, 57044, 57048, 57057, 57058, 57060, 57064, 57072, 57095, 57099, 57101, 57102, 57107, 
    57109, 57110, 57113, 57114, 57116, 57123, 57125, 57126, 57129, 57130, 57132, 57137, 57138, 57140, 57144, 57155, 57157, 57158, 57161, 57162, 
    57164, 57169, 57170, 57172, 57176, 57185, 57186, 57188, 57192, 57200, 57219, 57221, 57222, 57225, 57226, 57228, 57233, 57234, 57236, 57240, 
    57249, 57250, 57252, 57256, 57264, 57281, 57282, 57284, 57288, 57296, 57312, 57471, 57535, 57567, 57583, 57591, 57595, 57597, 57598, 57663, 
    57695, 57711, 57719, 57723, 57725, 57726, 57759, 57775, 57783, 57787, 57789, 57790, 57807, 57815, 57819, 57821, 57822, 57831, 57835, 57837, 
    57838, 57843, 57845, 57846, 57849, 57850, 57852, 57919, 57951, 57967, 57975, 57979, 57981, 57982, 58015, 58031, 58039, 58043, 58045, 58046, 
    58063, 58071, 58075, 58077, 58078, 58087, 58091, 58093, 58094, 58099, 58101, 58102, 58105, 58106, 58108, 58143, 58159, 58167, 58171, 58173, 
    58174, 58191, 58199, 58203, 58205, 58206, 58215, 58219, 58221, 58222, 58227, 58229, 58230, 58233, 58234, 58236, 58255, 58263, 58267, 58269, 
    58270, 58279, 58283, 58285, 58286, 58291, 58293, 58294, 58297, 58298, 58300, 58311, 58315, 58317, 58318, 58323, 58325, 58326, 58329, 58330, 
    58332, 58339, 58341, 58342, 58345, 58346, 58348, 58353, 58354, 58356, 58360, 58431, 58463, 58479, 58487, 58491, 58493, 58494, 58527, 58543, 
    58551, 58555, 58557, 58558, 58575, 58583, 58587, 58589, 58590, 58599, 58603, 58605, 58606, 58611, 58613, 58614, 58617, 58618, 58620, 58655, 
    58671, 58679, 58683, 58685, 58686, 58703, 58711, 58715, 58717, 58718, 58727, 58731, 58733, 58734, 58739, 58741, 58742, 58745, 58746, 58748, 
    58767, 58775, 58779, 58781, 58782, 58791, 58795, 58797, 58798, 58803, 58805, 58806, 58809, 58810, 58812, 58823, 58827, 58829, 58830, 58835, 
    58837, 58838, 58841, 58842, 58844, 58851, 58853, 58854, 58857, 58858, 58860, 58865, 58866, 58868, 58872, 58911, 58927, 58935, 58939, 58941, 
    58942, 58959, 58967, 58971, 58973, 58974, 58983, 58987, 58989, 58990, 58995, 58997, 58998, 59001, 59002, 59004, 59023, 59031, 59035, 59037, 
    59038, 59047, 59051, 59053, 59054, 59059, 59061, 59062, 59065, 59066, 59068, 59079, 59083, 59085, 59086, 59091, 59093, 59094, 59097, 59098, 
    59100, 59107, 59109, 59110, 59113, 59114, 59116, 59121, 59122, 59124, 59128, 59151, 59159, 59163, 59165, 59166, 59175, 59179, 59181, 59182, 
    59187, 59189, 59190, 59193, 59194, 59196, 59207, 59211, 59213, 59214, 59219, 59221, 59222, 59225, 59226, 59228, 59235, 59237, 59238, 59241, 
    59242, 59244, 59249, 59250, 59252, 59256, 59271, 59275, 59277, 59278, 59283, 59285, 59286, 59289, 59290, 59292, 59299, 59301, 59302, 59305, 
    59306, 59308, 59313, 59314, 59316, 59320, 59331, 59333, 59334, 59337, 59338, 59340, 59345, 59346, 59348, 59352, 59361, 59362, 59364, 59368, 
    59376, 59455, 59487, 59503, 59511, 59515, 59517, 59518, 59551, 59567, 59575, 59579, 59581, 59582, 59599, 59607, 59611, 59613, 59614, 59623, 
    59627, 59629, 59630, 59635, 59637, 59638, 59641, 59642, 59644, 59679, 59695, 59703, 59707, 59709, 59710, 59727, 59735, 59739, 59741, 59742, 
    59751, 59755, 59757, 59758, 59763, 59765, 59766, 59769, 59770, 59772, 59791, 59799, 59803, 59805, 59806, 59815, 59819, 59821, 59822, 59827, 
    59829, 59830, 59833, 59834, 59836, 59847, 59851, 59853, 59854, 59859, 59861, 59862, 59865, 59866, 59868, 59875, 59877, 59878, 59881, 59882, 
    59884, 59889, 59890, 59892, 59896, 59935, 59951, 59959, 59963, 59965, 59966, 59983, 59991, 59995, 59997, 59998, 60007, 60011, 60013, 60014, 
    60019, 60021, 60022, 60025, 60026, 60028, 60047, 60055, 60059, 60061, 60062, 60071, 60075, 60077, 60078, 60083, 60085, 60086, 60089, 60090, 
    60092, 60103, 60107, 60109, 60110, 60115, 60117, 60118, 60121, 60122, 60124, 60131, 60133, 60134, 60137, 60138, 60140, 60145, 60146, 60148, 
    60152, 60175, 60183, 60187, 60189, 60190, 60199, 60203, 60205, 60206, 60211, 60213, 60214, 60217, 60218, 60220, 60231, 60235, 60237, 60238, 
    60243, 60245, 60246, 60249, 60250, 60252, 60259, 60261, 60262, 60265, 60266, 60268, 60273, 60274, 60276, 60280, 60295, 60299, 60301, 60302, 
    60307, 60309, 60310, 60313, 60314, 60316, 60323, 60325, 60326, 60329, 60330, 60332, 60337, 60338, 60340, 60344, 60355, 60357, 60358, 60361, 
    60362, 60364, 60369, 60370, 60372, 60376, 60385, 60386, 60388, 60392, 60400, 60447, 60463, 60471, 60475, 60477, 60478, 60495, 60503, 60507, 
    60509, 60510, 60519, 60523, 60525, 60526, 60531, 60533, 60534, 60537, 60538, 60540, 60559, 60567, 60571, 60573, 60574, 60583, 60587, 60589, 
    60590, 60595, 60597, 60598, 60601, 60602, 60604, 60615, 60619, 60621, 60622, 60627, 60629, 60630, 60633, 60634, 60636, 60643, 60645, 60646, 
    60649, 60650, 60652, 60657, 60658, 60660, 60664, 60687, 60695, 60699, 60701, 60702, 60711, 60715, 60717, 60718, 60723, 60725, 60726, 60729, 
    60730, 60732, 60743, 60747, 60749, 60750, 60755, 60757, 60758, 60761, 60762, 60764, 60771, 60773, 60774, 60777, 60778, 60780, 60785, 60786, 
    60788, 60792, 60807, 60811, 60813, 60814, 60819, 60821, 60822, 60825, 60826, 60828, 60835, 60837, 60838, 60841, 60842, 60844, 60849, 60850, 
    60852, 60856, 60867, 60869, 60870, 60873, 60874, 60876, 60881, 60882, 60884, 60888, 60897, 60898, 60900, 60904, 60912, 60943, 60951, 60955, 
    60957, 60958, 60967, 60971, 60973, 60974, 60979, 60981, 60982, 60985, 60986, 60988, 60999, 61003, 61005, 61006, 61011, 61013, 61014, 61017, 
    61018, 61020, 61027, 61029, 61030, 61033, 61034, 61036, 61041, 61042, 61044, 61048, 61063, 61067, 61069, 61070, 61075, 61077, 61078, 61081, 
    61082, 61084, 61091, 61093, 61094, 61097, 61098, 61100, 61105, 61106, 61108, 61112, 61123, 61125, 61126, 61129, 61130, 61132, 61137, 61138, 
    61140, 61144, 61153, 61154, 61156, 61160, 61168, 61191, 61195, 61197, 61198, 61203, 61205, 61206, 61209, 61210, 61212, 61219, 61221, 61222, 
    61225, 61226, 61228, 61233, 61234, 61236, 61240, 61251, 61253, 61254, 61257, 61258, 61260, 61265, 61266, 61268, 61272, 61281, 61282, 61284, 
    61288, 61296, 61315, 61317, 61318, 61321, 61322, 61324, 61329, 61330, 61332, 61336, 61345, 61346, 61348, 61352, 61360, 61377, 61378, 61380, 
    61384, 61392, 61408, 61503, 61535, 61551, 61559, 61563, 61565, 61566, 61599, 61615, 61623, 61627, 61629, 61630, 61647, 61655, 61659, 61661, 
    61662, 61671, 61675, 61677, 61678, 61683, 61685, 61686, 61689, 61690, 61692, 61727, 61743, 61751, 61755, 61757, 61758, 61775, 61783, 61787, 
    61789, 61790, 61799, 61803, 61805, 61806, 61811, 61813, 61814, 61817, 61818, 61820, 61839, 61847, 61851, 61853, 61854, 61863, 61867, 61869, 
    61870, 61875, 61877, 61878, 61881, 61882, 61884, 61895, 61899, 61901, 61902, 61907, 61909, 61910, 61913, 61914, 61916, 61923, 61925, 61926, 
    61929, 61930, 61932, 61937, 61938, 61940, 61944, 61983, 61999, 62007, 62011, 62013, 62014, 62031, 62039, 62043, 62045, 62046, 62055, 62059, 
    62061, 62062, 62067, 62069, 62070, 62073, 62074, 62076, 62095, 62103, 62107, 62109, 62110, 62119, 62123, 62125, 62126, 62131, 62133, 62134, 
    62137, 62138, 62140, 62151, 62155, 62157, 62158, 62163, 62165, 62166, 62169, 62170, 62172, 62179, 62181, 62182, 62185, 62186, 62188, 62193, 
    62194, 62196, 62200, 62223, 62231, 62235, 62237, 62238, 62247, 62251, 62253, 62254, 62259, 62261, 62262, 62265, 62266, 62268, 62279, 62283, 
    62285, 62286, 62291, 62293, 62294, 62297, 62298, 62300, 62307, 62309, 62310, 62313, 62314, 62316, 62321, 62322, 62324, 62328, 62343, 62347, 
    62349, 62350, 62355, 62357, 62358, 62361, 62362, 62364, 62371, 62373, 62374, 62377, 62378, 62380, 62385, 62386, 62388, 62392, 62403, 62405, 
    62406, 62409, 62410, 62412, 62417, 62418, 62420, 62424, 62433, 62434, 62436, 62440, 62448, 62495, 62511, 62519, 62523, 62525, 62526, 62543, 
    62551, 62555, 62557, 62558, 62567, 62571, 62573, 62574, 62579, 62581, 62582, 62585, 62586, 62588, 62607, 62615, 62619, 62621, 62622, 62631, 
    62635, 62637, 62638, 62643, 62645, 62646, 62649, 62650, 62652, 62663, 62667, 62669, 62670, 62675, 62677, 62678, 62681, 62682, 62684, 62691, 
    62693, 62694, 62697, 62698, 62700, 62705, 62706, 62708, 62712, 62735, 62743, 62747, 62749, 62750, 62759, 62763, 62765, 62766, 62771, 62773, 
    62774, 62777, 62778, 62780, 62791, 62795, 62797, 62798, 62803, 62805, 62806, 62809, 62810, 62812, 62819, 62821, 62822, 62825, 62826, 62828, 
    62833, 62834, 62836, 62840, 62855, 62859, 62861, 62862, 62867, 62869, 62870, 62873, 62874, 62876, 62883, 62885, 62886, 62889, 62890, 62892, 
    62897, 62898, 62900, 62904, 62915, 62917, 62918, 62921, 62922, 62924, 62929, 62930, 62932, 62936, 62945, 62946, 62948, 62952, 62960, 62991, 
    62999, 63003, 63005, 63006, 63015, 63019, 63021, 63022, 63027, 63029, 63030, 63033, 63034, 63036, 63047, 63051, 63053, 63054, 63059, 63061, 
    63062, 63065, 63066, 63068, 63075, 63077, 63078, 63081, 63082, 63084, 63089, 63090, 63092, 63096, 63111, 63115, 63117, 63118, 63123, 63125, 
    63126, 63129, 63130, 63132, 63139, 63141, 63142, 63145, 63146, 63148, 63153, 63154, 63156, 63160, 63171, 63173, 63174, 63177, 63178, 63180, 
    63185, 63186, 63188, 63192, 63201, 63202, 63204, 63208, 63216, 63239, 63243, 63245, 63246, 63251, 63253, 63254, 63257, 63258, 63260, 63267, 
    63269, 63270, 63273, 63274, 63276, 63281, 63282, 63284, 63288, 63299, 63301, 63302, 63305, 63306, 63308, 63313, 63314, 63316, 63320, 63329, 
    63330, 63332, 63336, 63344, 63363, 63365, 63366, 63369, 63370, 63372, 63377, 63378, 63380, 63384, 63393, 63394, 63396, 63400, 63408, 63425, 
    63426, 63428, 63432, 63440, 63456, 63519, 63535, 63543, 63547, 63549, 63550, 63567, 63575, 63579, 63581, 63582, 63591, 63595, 63597, 63598, 
    63603, 63605, 63606, 63609, 63610, 63612, 63631, 63639, 63643, 63645, 63646, 63655, 63659, 63661, 63662, 63667, 63669, 63670, 63673, 63674, 
    63676, 63687, 63691, 63693, 63694, 63699, 63701, 63702, 63705, 63706, 63708, 63715, 63717, 63718, 63721, 63722, 63724, 63729, 63730, 63732, 
    63736, 63759, 63767, 63771, 63773, 63774, 63783, 63787, 63789, 63790, 63795, 63797, 63798, 63801, 63802, 63804, 63815, 63819, 63821, 63822, 
    63827, 63829, 63830, 63833, 63834, 63836, 63843, 63845, 63846, 63849, 63850, 63852, 63857, 63858, 63860, 63864, 63879, 63883, 63885, 63886, 
    63891, 63893, 63894, 63897, 63898, 63900, 63907, 63909, 63910, 63913, 63914, 63916, 63921, 63922, 63924, 63928, 63939, 63941, 63942, 63945, 
    63946, 63948, 63953, 63954, 63956, 63960, 63969, 63970, 63972, 63976, 63984, 64015, 64023, 64027, 64029, 64030, 64039, 64043, 64045, 64046, 
    64051, 64053, 64054, 64057, 64058, 64060, 64071, 64075, 64077, 64078, 64083, 64085, 64086, 64089, 64090, 64092, 64099, 64101, 64102, 64105, 
    64106, 64108, 64113, 64114, 64116, 64120, 64135, 64139, 64141, 64142, 64147, 64149, 64150, 64153, 64154, 64156, 64163, 64165, 64166, 64169, 
    64170, 64172, 64177, 64178, 64180, 64184, 64195, 64197, 64198, 64201, 64202, 64204, 64209, 64210, 64212, 64216, 64225, 64226, 64228, 64232, 
    64240, 64263, 64267, 64269, 64270, 64275, 64277, 64278, 64281, 64282, 64284, 64291, 64293, 64294, 64297, 64298, 64300, 64305, 64306, 64308, 
    64312, 64323, 64325, 64326, 64329, 64330, 64332, 64337, 64338, 64340, 64344, 64353, 64354, 64356, 64360, 64368, 64387, 64389, 64390, 64393, 
    64394, 64396, 64401, 64402, 64404, 64408, 64417, 64418, 64420, 64424, 64432, 64449, 64450, 64452, 64456, 64464, 64480, 64527, 64535, 64539, 
    64541, 64542, 64551, 64555, 64557, 64558, 64563, 64565, 64566, 64569, 64570, 64572, 64583, 64587, 64589, 64590, 64595, 64597, 64598, 64601, 
    64602, 64604, 64611, 64613, 64614, 64617, 64618, 64620, 64625, 64626, 64628, 64632, 64647, 64651, 64653, 64654, 64659, 64661, 64662, 64665, 
    64666, 64668, 64675, 64677, 64678, 64681, 64682, 64684, 64689, 64690, 64692, 64696, 64707, 64709, 64710, 64713, 64714, 64716, 64721, 64722, 
    64724, 64728, 64737, 64738, 64740, 64744, 64752, 64775, 64779, 64781, 64782, 64787, 64789, 64790, 64793, 64794, 64796, 64803, 64805, 64806, 
    64809, 64810, 64812, 64817, 64818, 64820, 64824, 64835, 64837, 64838, 64841, 64842, 64844, 64849, 64850, 64852, 64856, 64865, 64866, 64868, 
    64872, 64880, 64899, 64901, 64902, 64905, 64906, 64908, 64913, 64914, 64916, 64920, 64929, 64930, 64932, 64936, 64944, 64961, 64962, 64964, 
    64968, 64976, 64992, 65031, 65035, 65037, 65038, 65043, 65045, 65046, 65049, 65050, 65052, 65059, 65061, 65062, 65065, 65066, 65068, 65073, 
    65074, 65076, 65080, 65091, 65093, 65094, 65097, 65098, 65100, 65105, 65106, 65108, 65112, 65121, 65122, 65124, 65128, 65136, 65155, 65157, 
    65158, 65161, 65162, 65164, 65169, 65170, 65172, 65176, 65185, 65186, 65188, 65192, 65200, 65217, 65218, 65220, 65224, 65232, 65248, 65283, 
    65285, 65286, 65289, 65290, 65292, 65297, 65298, 65300, 65304, 65313, 65314, 65316, 65320, 65328, 65345, 65346, 65348, 65352, 65360, 65376, 
    65409, 65410, 65412, 65416, 65424, 65440, 65472, },
  { 2047, 3071, 3583, 3839, 3967, 4031, 4063, 4079, 4087, 4091, 4093, 4094, 5119, 5631, 5887, 6015, 6079, 6111, 6127, 6135, 6139, 
    6141, 6142, 6655, 6911, 7039, 7103, 7135, 7151, 7159, 7163, 7165, 7166, 7423, 7551, 7615, 7647, 7663, 7671, 7675, 7677, 
    7678, 7807, 7871, 7903, 7919, 7927, 7931, 7933, 7934, 7999, 8031, 8047, 8055, 8059, 8061, 8062, 8095, 8111, 8119, 8123, 
    8125, 8126, 8143, 8151, 8155, 8157, 8158, 8167, 8171, 8173, 8174, 8179, 8181, 8182, 8185, 8186, 8188, 9215, 9727, 9983, 
    10111, 10175, 10207, 10223, 10231, 10235, 10237, 10238, 10751, 11007, 11135, 11199, 11231, 11247, 11255, 11259, 11261, 11262, 11519, 11647, 
    11711, 11743, 11759, 11767, 11771, 11773, 11774, 11903, 11967, 11999, 12015, 12023, 12027, 12029, 12030, 12095, 12127, 12143, 12151, 12155, 
    12157, 12158, 12191, 12207, 12215, 12219, 12221, 12222, 12239, 12247, 12251, 12253, 12254, 12263, 12267, 12269, 12270, 12275, 12277, 12278, 
    12281, 12282, 12284, 12799, 13055, 13183, 13247, 13279, 13295, 13303, 13307, 13309, 13310, 13567, 13695, 13759, 13791, 13807, 13815, 13819, 
    13821, 13822, 13951, 14015, 14047, 14063, 14071, 14075, 14077, 14078, 14143, 14175, 14191, 14199, 14203, 14205, 14206, 14239, 14255, 14263, 
    14267, 14269, 14270, 14287, 14295, 14299, 14301, 14302, 14311, 14315, 14317, 14318, 14323, 14325, 14326, 14329, 14330, 14332, 14591, 14719, 
    14783, 14815, 14831, 14839, 14843, 14845, 14846, 14975, 15039, 15071, 15087, 15095, 15099, 15101, 15102, 15167, 15199, 15215, 15223, 15227, 
    15229, 15230, 15263, 15279, 15287, 15291, 15293, 15294, 15311, 15319, 15323, 15325, 15326, 15335, 15339, 15341, 15342, 15347, 15349, 15350, 
    15353, 15354, 15356, 15487, 15551, 15583, 15599, 15607, 15611, 15613, 15614, 15679, 15711, 15727, 15735, 15739, 15741, 15742, 15775, 15791, 
    15799, 15803, 15805, 15806, 15823, 15831, 15835, 15837, 15838, 15847, 15851, 15853, 15854, 15859, 15861, 15862, 15865, 15866, 15868, 15935, 
    15967, 15983, 15991, 15995, 15997, 15998, 16031, 16047, 16055, 16059, 16061, 16062, 16079, 16087, 16091, 16093, 16094, 16103, 16107, 16109, 
    16110, 16115, 16117, 16118, 16121, 16122, 16124, 16159, 16175, 16183, 16187, 16189, 16190, 16207, 16215, 16219, 16221, 16222, 16231, 16235, 
    16237, 16238, 16243, 16245, 16246, 16249, 16250, 16252, 16271, 16279, 16283, 16285, 16286, 16295, 16299, 16301, 16302, 16307, 16309, 16310, 
    16313, 16314, 16316, 16327, 16331, 16333, 16334, 16339, 16341, 16342, 16345, 16346, 16348, 16355, 16357, 16358, 16361, 16362, 16364, 16369, 
    16370, 16372, 16376, 17407, 17919, 18175, 18303, 18367, 18399, 18415, 18423, 18427, 18429, 18430, 18943, 19199, 19327, 19391, 19423, 19439, 
    19447, 19451, 19453, 19454, 19711, 19839, 19903, 19935, 19951, 19959, 19963, 19965, 19966, 20095, 20159, 20191, 20207, 20215, 20219, 20221, 
    20222, 20287, 20319, 20335, 20343, 20347, 20349, 20350, 20383, 20399, 20407, 20411, 20413, 20414, 20431, 20439, 20443, 20445, 20446, 20455, 
    20459, 20461, 20462, 20467, 20469, 20470, 20473, 20474, 20476, 20991, 21247, 21375, 21439, 21471, 21487, 21495, 21499, 21501, 21502, 21759, 
    21887, 21951, 21983, 21999, 22007, 22011, 22013, 22014, 22143, 22207, 22239, 22255, 22263, 22267, 22269, 22270, 22335, 22367, 22383, 22391, 
    22395, 22397, 22398, 22431, 22447, 22455, 22459, 22461, 22462, 22479, 22487, 22491, 22493, 22494, 22503, 22507, 22509, 22510, 22515, 22517, 
    22518, 22521, 22522, 22524, 22783, 22911, 22975, 23007, 23023, 23031, 23035, 23037, 23038, 23167, 23231, 23263, 23279, 23287, 23291, 23293, 
    23294, 23359, 23391, 23407, 23415, 23419, 23421, 23422, 23455, 23471, 23479, 23483, 23485, 23486, 23503, 23511, 23515, 23517, 23518, 23527, 
    23531, 23533, 23534, 23539, 23541, 23542, 23545, 23546, 23548, 23679, 23743, 23775, 23791, 23799, 23803, 23805, 23806, 23871, 23903, 23919, 
    23927, 23931, 23933, 23934, 23967, 23983, 23991, 23995, 23997, 23998, 24015, 24023, 24027, 24029, 24030, 24039, 24043, 24045, 24046, 24051, 
    24053, 24054, 24057, 24058, 24060, 24127, 24159, 24175, 24183, 24187, 24189, 24190, 24223, 24239, 24247, 24251, 24253, 24254, 24271, 24279, 
    24283, 24285, 24286, 24295, 24299, 24301, 24302, 24307, 24309, 24310, 24313, 24314, 24316, 24351, 24367, 24375, 24379, 24381, 24382, 24399, 
    24407, 24411, 24413, 24414, 24423, 24427, 24429, 24430, 24435, 24437, 24438, 24441, 24442, 24444, 24463, 24471, 24475, 24477, 24478, 24487, 
    24491, 24493, 24494, 24499, 24501, 24502, 24505, 24506, 24508, 24519, 24523, 24525, 24526, 24531, 24533, 24534, 24537, 24538, 24540, 24547, 
    24549, 24550, 24553, 24554, 24556, 24561, 24562, 24564, 24568, 25087, 25343, 25471, 25535, 25567, 25583, 25591, 25595, 25597, 25598, 25855, 
    25983, 26047, 26079, 26095, 26103, 26107, 26109, 26110, 26239, 26303, 26335, 26351, 26359, 26363, 26365, 26366, 26431, 26463, 26479, 26487, 
    26491, 26493, 26494, 26527, 26543, 26551, 26555, 26557, 26558, 26575, 26583, 26587, 26589, 26590, 26599, 26603, 26605, 26606, 26611, 26613, 
    26614, 26617, 26618, 26620, 26879, 27007, 27071, 27103, 27119, 27127, 27131, 27133, 27134, 27263, 27327, 27359, 27375, 27383, 27387, 27389, 
    27390, 27455, 27487, 27503, 27511, 27515, 27517, 27518, 27551, 27567, 27575, 27579, 27581, 27582, 27599, 27607, 27611, 27613, 27614, 27623, 
    27627, 27629, 27630, 27635, 27637, 27638, 27641, 27642, 27644, 27775, 27839, 27871, 27887, 27895, 27899, 27901, 27902, 27967, 27999, 28015, 
    28023, 28027, 28029, 28030, 28063, 28079, 28087, 28091, 28093, 28094, 28111, 28119, 28123, 28125, 28126, 28135, 28139, 28141, 28142, 28147, 
    28149, 28150, 28153, 28154, 28156, 28223, 28255, 28271, 28279, 28283, 28285, 28286, 28319, 28335, 28343, 28347, 28349, 28350, 28367, 28375, 
    28379, 28381, 28382, 28391, 28395, 28397, 28398, 28403, 28405, 28406, 28409, 28410, 28412, 28447, 28463, 28471, 28475, 28477, 28478, 28495, 
    28503, 28507, 28509, 28510, 28519, 28523, 28525, 28526, 28531, 28533, 28534, 28537, 28538, 28540, 28559, 28567, 28571, 28573, 28574, 28583, 
    28587, 28589, 28590, 28595, 28597, 28598, 28601, 28602, 28604, 28615, 28619, 28621, 28622, 28627, 28629, 28630, 28633, 28634, 28636, 28643, 
    28645, 28646, 28649, 28650, 28652, 28657, 28658, 28660, 28664, 28927, 29055, 29119, 29151, 29167, 29175, 29179, 29181, 29182, 29311, 29375, 
    29407, 29423, 29431, 29435, 29437, 29438, 29503, 29535, 29551, 29559, 29563, 29565, 29566, 29599, 29615, 29623, 29627, 29629, 29630, 29647, 
    29655, 29659, 29661, 29662, 29671, 29675, 29677, 29678, 29683, 29685, 29686, 29689, 29690, 29692, 29823, 29887, 29919, 29935, 29943, 29947, 
    29949, 29950, 30015, 30047, 30063, 30071, 30075, 30077, 30078, 30111, 30127, 30135, 30139, 30141, 30142, 30159, 30167, 30171, 30173, 30174, 
    30183, 30187, 30189, 30190, 30195, 30197, 30198, 30201, 30202, 30204, 30271, 30303, 30319, 30327, 30331, 30333, 30334, 30367, 30383, 30391, 
    30395, 30397, 30398, 30415, 30423, 30427, 30429, 30430, 30439, 30443, 30445, 30446, 30451, 30453, 30454, 30457, 30458, 30460, 30495, 30511, 
    30519, 30523, 30525, 30526, 30543, 30551, 30555, 30557, 30558, 30567, 30571, 30573, 30574, 30579, 30581, 30582, 30585, 30586, 30588, 30607, 
    30615, 30619, 30621, 30622, 30631, 30635, 30637, 30638, 30643, 30645, 30646, 30649, 30650, 30652, 30663, 30667, 30669, 30670, 30675, 30677, 
    30678, 30681, 30682, 30684, 30691, 30693, 30694, 30697, 30698, 30700, 30705, 30706, 30708, 30712, 30847, 30911, 30943, 30959, 30967, 30971, 
    30973, 30974, 31039, 31071, 31087, 31095, 31099, 31101, 31102, 31135, 31151, 31159, 31163, 31165, 31166, 31183, 31191, 31195, 31197, 31198, 
    31207, 31211, 31213, 31214, 31219, 31221, 31222, 31225, 31226, 31228, 31295, 31327, 31343, 31351, 31355, 31357, 31358, 31391, 31407, 31415, 
    31419, 31421, 31422, 31439, 31447, 31451, 31453, 31454, 31463, 31467, 31469, 31470, 31475, 31477, 31478, 31481, 31482, 31484, 31519, 31535, 
    31543, 31547, 31549, 31550, 31567, 31575, 31579, 31581, 31582, 31591, 31595, 31597, 31598, 31603, 31605, 31606, 31609, 31610, 31612, 31631, 
    31639, 31643, 31645, 31646, 31655, 31659, 31661, 31662, 31667, 31669, 31670, 31673, 31674, 31676, 31687, 31691, 31693, 31694, 31699, 31701, 
    31702, 31705, 31706, 31708, 31715, 31717, 31718, 31721, 31722, 31724, 31729, 31730, 31732, 31736, 31807, 31839, 31855, 31863, 31867, 31869, 
    31870, 31903, 31919, 31927, 31931, 31933, 31934, 31951, 31959, 31963, 31965, 31966, 31975, 31979, 31981, 31982, 31987, 31989, 31990, 31993, 
    31994, 31996, 32031, 32047, 32055, 32059, 32061, 32062, 32079, 32087, 32091, 32093, 32094, 32103, 32107, 32109, 32110, 32115, 32117, 32118, 
    32121, 32122, 32124, 32143, 32151, 32155, 32157, 32158, 32167, 32171, 32173, 32174, 32179, 32181, 32182, 32185, 32186, 32188, 32199, 32203, 
    32205, 32206, 32211, 32213, 32214, 32217, 32218, 32220, 32227, 32229, 32230, 32233, 32234, 32236, 32241, 32242, 32244, 32248, 32287, 32303, 
    32311, 32315, 32317, 32318, 32335, 32343, 32347, 32349, 32350, 32359, 32363, 32365, 32366, 32371, 32373, 32374, 32377, 32378, 32380, 32399, 
    32407, 32411, 32413, 32414, 32423, 32427, 32429, 32430, 32435, 32437, 32438, 32441, 32442, 32444, 32455, 32459, 32461, 32462, 32467, 32469, 
    32470, 32473, 32474, 32476, 32483, 32485, 32486, 32489, 32490, 32492, 32497, 32498, 32500, 32504, 32527, 32535, 32539, 32541, 32542, 32551, 
    32555, 32557, 32558, 32563, 32565, 32566, 32569, 32570, 32572, 32583, 32587, 32589, 32590, 32595, 32597, 32598, 32601, 32602, 32604, 32611, 
    32613, 32614, 32617, 32618, 32620, 32625, 32626, 32628, 32632, 32647, 32651, 32653, 32654, 32659, 32661, 32662, 32665, 32666, 32668, 32675, 
    32677, 32678, 32681, 32682, 32684, 32689, 32690, 32692, 32696, 32707, 32709, 32710, 32713, 32714, 32716, 32721, 32722, 32724, 32728, 32737, 
    32738, 32740, 32744, 32752, 33791, 34303, 34559, 34687, 34751, 34783, 34799, 34807, 34811, 34813, 34814, 35327, 35583, 35711, 35775, 35807, 
    35823, 35831, 35835, 35837, 35838, 36095, 36223, 36287, 36319, 36335, 36343, 36347, 36349, 36350, 36479, 36543, 36575, 36591, 36599, 36603, 
    36605, 36606, 36671, 36703, 36719, 36727, 36731, 36733, 36734, 36767, 36783, 36791, 36795, 36797, 36798, 36815, 36823, 36827, 36829, 36830, 
    36839, 36843, 36845, 36846, 36851, 36853, 36854, 36857, 36858, 36860, 37375, 37631, 37759, 37823, 37855, 37871, 37879, 37883, 37885, 37886, 
    38143, 38271, 38335, 38367, 38383, 38391, 38395, 38397, 38398, 38527, 38591, 38623, 38639, 38647, 38651, 38653, 38654, 38719, 38751, 38767, 
    38775, 38779, 38781, 38782, 38815, 38831, 38839, 38843, 38845, 38846, 38863, 38871, 38875, 38877, 38878, 38887, 38891, 38893, 38894, 38899, 
    38901, 38902, 38905, 38906, 38908, 39167, 39295, 39359, 39391, 39407, 39415, 39419, 39421, 39422, 39551, 39615, 39647, 39663, 39671, 39675, 
    39677, 39678, 39743, 39775, 39791, 39799, 39803, 39805, 39806, 39839, 39855, 39863, 39867, 39869, 39870, 39887, 39895, 39899, 39901, 39902, 
    39911, 39915, 39917, 39918, 39923, 39925, 39926, 39929, 39930, 39932, 40063, 40127, 40159, 40175, 40183, 40187, 40189, 40190, 40255, 40287, 
    40303, 40311, 40315, 40317, 40318, 40351, 40367, 40375, 40379, 40381, 40382, 40399, 40407, 40411, 40413, 40414, 40423, 40427, 40429, 40430, 
    40435, 40437, 40438, 40441, 40442, 40444, 40511, 40543, 40559, 40567, 40571, 40573, 40574, 40607, 40623, 40631, 40635, 40637, 40638, 40655, 
    40663, 40667, 40669, 40670, 40679, 40683, 40685, 40686, 40691, 40693, 40694, 40697, 40698, 40700, 40735, 40751, 40759, 40763, 40765, 40766, 
    40783, 40791, 40795, 40797, 40798, 40807, 40811, 40813, 40814, 40819, 40821, 40822, 40825, 40826, 40828, 40847, 40855, 40859, 40861, 40862, 
    40871, 40875, 40877, 40878, 40883, 40885, 40886, 40889, 40890, 40892, 40903, 40907, 40909, 40910, 40915, 40917, 40918, 40921, 40922, 40924, 
    40931, 40933, 40934, 40937, 40938, 40940, 40945, 40946, 40948, 40952, 41471, 41727, 41855, 41919, 41951, 41967, 41975, 41979, 41981, 41982, 
    42239, 42367, 42431, 42463, 42479, 42487, 42491, 42493, 42494, 42623, 42687, 42719, 42735, 42743, 42747, 42749, 42750, 42815, 42847, 42863, 
    42871, 42875, 42877, 42878, 42911, 42927, 42935, 42939, 42941, 42942, 42959, 42967, 42971, 42973, 42974, 42983, 42987, 42989, 42990, 42995, 
    42997, 42998, 43001, 43002, 43004, 43263, 43391, 43455, 43487, 43503, 43511, 43515, 43517, 43518, 43647, 43711, 43743, 43759, 43767, 43771, 
    43773, 43774, 43839, 43871, 43887, 43895, 43899, 43901, 43902, 43935, 43951, 43959, 43963, 43965, 43966, 43983, 43991, 43995, 43997, 43998, 
    44007, 44011, 44013, 44014, 44019, 44021, 44022, 44025, 44026, 44028, 44159, 44223, 44255, 44271, 44279, 44283, 44285, 44286, 44351, 44383, 
    44399, 44407, 44411, 44413, 44414, 44447, 44463, 44471, 44475, 44477, 44478, 44495, 44503, 44507, 44509, 44510, 44519, 44523, 44525, 44526, 
    44531, 44533, 44534, 44537, 44538, 44540, 44607, 44639, 44655, 44663, 44667, 44669, 44670, 44703, 44719, 44727, 44731, 44733, 44734, 44751, 
    44759, 44763, 44765, 44766, 44775, 44779, 44781, 44782, 44787, 44789, 44790, 44793, 44794, 44796, 44831, 44847, 44855, 44859, 44861, 44862, 
    44879, 44887, 44891, 44893, 44894, 44903, 44907, 44909, 44910, 44915, 44917, 44918, 44921, 44922, 44924, 44943, 44951, 44955, 44957, 44958, 
    44967, 44971, 44973, 44974, 44979, 44981, 44982, 44985, 44986, 44988, 44999, 45003, 45005, 45006, 45011, 45013, 45014, 45017, 45018, 45020, 
    45027, 45029, 45030, 45033, 45034, 45036, 45041, 45042, 45044, 45048, 45311, 45439, 45503, 45535, 45551, 45559, 45563, 45565, 45566, 45695, 
    45759, 45791, 45807, 45815, 45819, 45821, 45822, 45887, 45919, 45935, 45943, 45947, 45949, 45950, 45983, 45999, 46007, 46011, 46013, 46014, 
    46031, 46039, 46043, 46045, 46046, 46055, 46059, 46061, 46062, 46067, 46069, 46070, 46073, 46074, 46076, 46207, 46271, 46303, 46319, 46327, 
    46331, 46333, 46334, 46399, 46431, 46447, 46455, 46459, 46461, 46462, 46495, 46511, 46519, 46523, 46525, 46526, 46543, 46551, 46555, 46557, 
    46558, 46567, 46571, 46573, 46574, 46579, 46581, 46582, 46585, 46586, 46588, 46655, 46687, 46703, 46711, 46715, 46717, 46718, 46751, 46767, 
    46775, 46779, 46781, 46782, 46799, 46807, 46811, 46813, 46814, 46823, 46827, 46829, 46830, 46835, 46837, 46838, 46841, 46842, 46844, 46879, 
    46895, 46903, 46907, 46909, 46910, 46927, 46935, 46939, 46941, 46942, 46951, 46955, 46957, 46958, 46963, 46965, 46966, 46969, 46970, 46972, 
    46991, 46999, 47003, 47005, 47006, 47015, 47019, 47021, 47022, 47027, 47029, 47030, 47033, 47034, 47036, 47047, 47051, 47053, 47054, 47059, 
    47061, 47062, 47065, 47066, 47068, 47075, 47077, 47078, 47081, 47082, 47084, 47089, 47090, 47092, 47096, 47231, 47295, 47327, 47343, 47351, 
    47355, 47357, 47358, 47423, 47455, 47471, 47479, 47483, 47485, 47486, 47519, 47535, 47543, 47547, 47549, 47550, 47567, 47575, 47579, 47581, 
    47582, 47591, 47595, 47597, 47598, 47603, 47605, 47606, 47609, 47610, 47612, 47679, 47711, 47727, 47735, 47739, 47741, 47742, 47775, 47791, 
    47799, 47803, 47805, 47806, 47823, 47831, 47835, 47837, 47838, 47847, 47851, 47853, 47854, 47859, 47861, 47862, 47865, 47866, 47868, 47903, 
    47919, 47927, 47931, 47933, 47934, 47951, 47959, 47963, 47965, 47966, 47975, 47979, 47981, 47982, 47987, 47989, 47990, 47993, 47994, 47996, 
    48015, 48023, 48027, 48029, 48030, 48039, 48043, 48045, 48046, 48051, 48053, 48054, 48057, 48058, 48060, 48071, 48075, 48077, 48078, 48083, 
    48085, 48086, 48089, 48090, 48092, 48099, 48101, 48102, 48105, 48106, 48108, 48113, 48114, 48116, 48120, 48191, 48223, 48239, 48247, 48251, 
    48253, 48254, 48287, 48303, 48311, 48315, 48317, 48318, 48335, 48343, 48347, 48349, 48350, 48359, 48363, 48365, 48366, 48371, 48373, 48374, 
    48377, 48378, 48380, 48415, 48431, 48439, 48443, 48445, 48446, 48463, 48471, 48475, 48477, 48478, 48487, 48491, 48493, 48494, 48499, 48501, 
    48502, 48505, 48506, 48508, 48527, 48535, 48539, 48541, 48542, 48551, 48555, 48557, 48558, 48563, 48565, 48566, 48569, 48570, 48572, 48583, 
    48587, 48589, 48590, 48595, 48597, 48598, 48601, 48602, 48604, 48611, 48613, 48614, 48617, 48618, 48620, 48625, 48626, 48628, 48632, 48671, 
    48687, 48695, 48699, 48701, 48702, 48719, 48727, 48731, 48733, 48734, 48743, 48747, 48749, 48750, 48755, 48757, 48758, 48761, 48762, 48764, 
    48783, 48791, 48795, 48797, 48798, 48807, 48811, 48813, 48814, 48819, 48821, 48822, 48825, 48826, 48828, 48839, 48843, 48845, 48846, 48851, 
    48853, 48854, 48857, 48858, 48860, 48867, 48869, 48870, 48873, 48874, 48876, 48881, 48882, 48884, 48888, 48911, 48919, 48923, 48925, 48926, 
    48935, 48939, 48941, 48942, 48947, 48949, 48950, 48953, 48954, 48956, 48967, 48971, 48973, 48974, 48979, 48981, 48982, 48985, 48986, 48988, 
    48995, 48997, 48998, 49001, 49002, 49004, 49009, 49010, 49012, 49016, 49031, 49035, 49037, 49038, 49043, 49045, 49046, 49049, 49050, 49052, 
    49059, 49061, 49062, 49065, 49066, 49068, 49073, 49074, 49076, 49080, 49091, 49093, 49094, 49097, 49098, 49100, 49105, 49106, 49108, 49112, 
    49121, 49122, 49124, 49128, 49136, 49663, 49919, 50047, 50111, 50143, 50159, 50167, 50171, 50173, 50174, 50431, 50559, 50623, 50655, 50671, 
    50679, 50683, 50685, 50686, 50815, 50879, 50911, 50927, 50935, 50939, 50941, 50942, 51007, 51039, 51055, 51063, 51067, 51069, 51070, 51103, 
    51119, 51127, 51131, 51133, 51134, 51151, 51159, 51163, 51165, 51166, 51175, 51179, 51181, 51182, 51187, 51189, 51190, 51193, 51194, 51196, 
    51455, 51583, 51647, 51679, 51695, 51703, 51707, 51709, 51710, 51839, 51903, 51935, 51951, 51959, 51963, 51965, 51966, 52031, 52063, 52079, 
    52087, 52091, 52093, 52094, 52127, 52143, 52151, 52155, 52157, 52158, 52175, 52183, 52187, 52189, 52190, 52199, 52203, 52205, 52206, 52211, 
    52213, 52214, 52217, 52218, 52220, 52351, 52415, 52447, 52463, 52471, 52475, 52477, 52478, 52543, 52575, 52591, 52599, 52603, 52605, 52606, 
    52639, 52655, 52663, 52667, 52669, 52670, 52687, 52695, 52699, 52701, 52702, 52711, 52715, 52717, 52718, 52723, 52725, 52726, 52729, 52730, 
    52732, 52799, 52831, 52847, 52855, 52859, 52861, 52862, 52895, 52911, 52919, 52923, 52925, 52926, 52943, 52951, 52955, 52957, 52958, 52967, 
    52971, 52973, 52974, 52979, 52981, 52982, 52985, 52986, 52988, 53023, 53039, 53047, 53051, 53053, 53054, 53071, 53079, 53083, 53085, 53086, 
    53095, 53099, 53101, 53102, 53107, 53109, 53110, 53113, 53114, 53116, 53135, 53143, 53147, 53149, 53150, 53159, 53163, 53165, 53166, 53171, 
    53173, 53174, 53177, 53178, 53180, 53191, 53195, 53197, 53198, 53203, 53205, 53206, 53209, 53210, 53212, 53219, 53221, 53222, 53225, 53226, 
    53228, 53233, 53234, 53236, 53240, 53503, 53631, 53695, 53727, 53743, 53751, 53755, 53757, 53758, 53887, 53951, 53983, 53999, 54007, 54011, 
    54013, 54014, 54079, 54111, 54127, 54135, 54139, 54141, 54142, 54175, 54191, 54199, 54203, 54205, 54206, 54223, 54231, 54235, 54237, 54238, 
    54247, 54251, 54253, 54254, 54259, 54261, 54262, 54265, 54266, 54268, 54399, 54463, 54495, 54511, 54519, 54523, 54525, 54526, 54591, 54623, 
    54639, 54647, 54651, 54653, 54654, 54687, 54703, 54711, 54715, 54717, 54718, 54735, 54743, 54747, 54749, 54750, 54759, 54763, 54765, 54766, 
    54771, 54773, 54774, 54777, 54778, 54780, 54847, 54879, 54895, 54903, 54907, 54909, 54910, 54943, 54959, 54967, 54971, 54973, 54974, 54991, 
    54999, 55003, 55005, 55006, 55015, 55019, 55021, 55022, 55027, 55029, 55030, 55033, 55034, 55036, 55071, 55087, 55095, 55099, 55101, 55102, 
    55119, 55127, 55131, 55133, 55134, 55143, 55147, 55149, 55150, 55155, 55157, 55158, 55161, 55162, 55164, 55183, 55191, 55195, 55197, 55198, 
    55207, 55211, 55213, 55214, 55219, 55221, 55222, 55225, 55226, 55228, 55239, 55243, 55245, 55246, 55251, 55253, 55254, 55257, 55258, 55260, 
    55267, 55269, 55270, 55273, 55274, 55276, 55281, 55282, 55284, 55288, 55423, 55487, 55519, 55535, 55543, 55547, 55549, 55550, 55615, 55647, 
    55663, 55671, 55675, 55677, 55678, 55711, 55727, 55735, 55739, 55741, 55742, 55759, 55767, 55771, 55773, 55774, 55783, 55787, 55789, 55790, 
    55795, 55797, 55798, 55801, 55802, 55804, 55871, 55903, 55919, 55927, 55931, 55933, 55934, 55967, 55983, 55991, 55995, 55997, 55998, 56015, 
    56023, 56027, 56029, 56030, 56039, 56043, 56045, 56046, 56051, 56053, 56054, 56057, 56058, 56060, 56095, 56111, 56119, 56123, 56125, 56126, 
    56143, 56151, 56155, 56157, 56158, 56167, 56171, 56173, 56174, 56179, 56181, 56182, 56185, 56186, 56188, 56207, 56215, 56219, 56221, 56222, 
    56231, 56235, 56237, 56238, 56243, 56245, 56246, 56249, 56250, 56252, 56263, 56267, 56269, 56270, 56275, 56277, 56278, 56281, 56282, 56284, 
    56291, 56293, 56294, 56297, 56298, 56300, 56305, 56306, 56308, 56312, 56383, 56415, 56431, 56439, 56443, 56445, 56446, 56479, 56495, 56503, 
    56507, 56509, 56510, 56527, 56535, 56539, 56541, 56542, 56551, 56555, 56557, 56558, 56563, 56565, 56566, 56569, 56570, 56572, 56607, 56623, 
    56631, 56635, 56637, 56638, 56655, 56663, 56667, 56669, 56670, 56679, 56683, 56685, 56686, 56691, 56693, 56694, 56697, 56698, 56700, 56719, 
    56727, 56731, 56733, 56734, 56743, 56747, 56749, 56750, 56755, 56757, 56758, 56761, 56762, 56764, 56775, 56779, 56781, 56782, 56787, 56789, 
    56790, 56793, 56794, 56796, 56803, 56805, 56806, 56809, 56810, 56812, 56817, 56818, 56820, 56824, 56863, 56879, 56887, 56891, 56893, 56894, 
    56911, 56919, 56923, 56925, 56926, 56935, 56939, 56941, 56942, 56947, 56949, 56950, 56953, 56954, 56956, 56975, 56983, 56987, 56989, 56990, 
    56999, 57003, 57005, 57006, 57011, 57013, 57014, 57017, 57018, 57020, 57031, 57035, 57037, 57038, 57043, 57045, 57046, 57049, 57050, 57052, 
    57059, 57061, 57062, 57065, 57066, 57068, 57073, 57074, 57076, 57080, 57103, 57111, 57115, 57117, 57118, 57127, 57131, 57133, 57134, 57139, 
    57141, 57142, 57145, 57146, 57148, 57159, 57163, 57165, 57166, 57171, 57173, 57174, 57177, 57178, 57180, 57187, 57189, 57190, 57193, 57194, 
    57196, 57201, 57202, 57204, 57208, 57223, 57227, 57229, 57230, 57235, 57237, 57238, 57241, 57242, 57244, 57251, 57253, 57254, 57257, 57258, 
    57260, 57265, 57266, 57268, 57272, 57283, 57285, 57286, 57289, 57290, 57292, 57297, 57298, 57300, 57304, 57313, 57314, 57316, 57320, 57328, 
    57599, 57727, 57791, 57823, 57839, 57847, 57851, 57853, 57854, 57983, 58047, 58079, 58095, 58103, 58107, 58109, 58110, 58175, 58207, 58223, 
    58231, 58235, 58237, 58238, 58271, 58287, 58295, 58299, 58301, 58302, 58319, 58327, 58331, 58333, 58334, 58343, 58347, 58349, 58350, 58355, 
    58357, 58358, 58361, 58362, 58364, 58495, 58559, 58591, 58607, 58615, 58619, 58621, 58622, 58687, 58719, 58735, 58743, 58747, 58749, 58750, 
    58783, 58799, 58807, 58811, 58813, 58814, 58831, 58839, 58843, 58845, 58846, 58855, 58859, 58861, 58862, 58867, 58869, 58870, 58873, 58874, 
    58876, 58943, 58975, 58991, 58999, 59003, 59005, 59006, 59039, 59055, 59063, 59067, 59069, 59070, 59087, 59095, 59099, 59101, 59102, 59111, 
    59115, 59117, 59118, 59123, 59125, 59126, 59129, 59130, 59132, 59167, 59183, 59191, 59195, 59197, 59198, 59215, 59223, 59227, 59229, 59230, 
    59239, 59243, 59245, 59246, 59251, 59253, 59254, 59257, 59258, 59260, 59279, 59287, 59291, 59293, 59294, 59303, 59307, 59309, 59310, 59315, 
    59317, 59318, 59321, 59322, 59324, 59335, 59339, 59341, 59342, 59347, 59349, 59350, 59353, 59354, 59356, 59363, 59365, 59366, 59369, 59370, 
    59372, 59377, 59378, 59380, 59384, 59519, 59583, 59615, 59631, 59639, 59643, 59645, 59646, 59711, 59743, 59759, 59767, 59771, 59773, 59774, 
    59807, 59823, 59831, 59835, 59837, 59838, 59855, 59863, 59867, 59869, 59870, 59879, 59883, 59885, 59886, 59891, 59893, 59894, 59897, 59898, 
    59900, 59967, 59999, 60015, 60023, 60027, 60029, 60030, 60063, 60079, 60087, 60091, 60093, 60094, 60111, 60119, 60123, 60125, 60126, 60135, 
    60139, 60141, 60142, 60147, 60149, 60150, 60153, 60154, 60156, 60191, 60207, 60215, 60219, 60221, 60222, 60239, 60247, 60251, 60253, 60254, 
    60263, 60267, 60269, 60270, 60275, 60277, 60278, 60281, 60282, 60284, 60303, 60311, 60315, 60317, 60318, 60327, 60331, 60333, 60334, 60339, 
    60341, 60342, 60345, 60346, 60348, 60359, 60363, 60365, 60366, 60371, 60373, 60374, 60377, 60378, 60380, 60387, 60389, 60390, 60393, 60394, 
    60396, 60401, 60402, 60404, 60408, 60479, 60511, 60527, 60535, 60539, 60541, 60542, 60575, 60591, 60599, 60603, 60605, 60606, 60623, 60631, 
    60635, 60637, 60638, 60647, 60651, 60653, 60654, 60659, 60661, 60662, 60665, 60666, 60668, 60703, 60719, 60727, 60731, 60733, 60734, 60751, 
    60759, 60763, 60765, 60766, 60775, 60779, 60781, 60782, 60787, 60789, 60790, 60793, 60794, 60796, 60815, 60823, 60827, 60829, 60830, 60839, 
    60843, 60845, 60846, 60851, 60853, 60854, 60857, 60858, 60860, 60871, 60875, 60877, 60878, 60883, 60885, 60886, 60889, 60890, 60892, 60899, 
    60901, 60902, 60905, 60906, 60908, 60913, 60914, 60916, 60920, 60959, 60975, 60983, 60987, 60989, 60990, 61007, 61015, 61019, 61021, 61022, 
    61031, 61035, 61037, 61038, 61043, 61045, 61046, 61049, 61050, 61052, 61071, 61079, 61083, 61085, 61086, 61095, 61099, 61101, 61102, 61107, 
    61109, 61110, 61113, 61114, 61116, 61127, 61131, 61133, 61134, 61139, 61141, 61142, 61145, 61146, 61148, 61155, 61157, 61158, 61161, 61162, 
    61164, 61169, 61170, 61172, 61176, 61199, 61207, 61211, 61213, 61214, 61223, 61227, 61229, 61230, 61235, 61237, 61238, 61241, 61242, 61244, 
    61255, 61259, 61261, 61262, 61267, 61269, 61270, 61273, 61274, 61276, 61283, 61285, 61286, 61289, 61290, 61292, 61297, 61298, 61300, 61304, 
    61319, 61323, 61325, 61326, 61331, 61333, 61334, 61337, 61338, 61340, 61347, 61349, 61350, 61353, 61354, 61356, 61361, 61362, 61364, 61368, 
    61379, 61381, 61382, 61385, 61386, 61388, 61393, 61394, 61396, 61400, 61409, 61410, 61412, 61416, 61424, 61567, 61631, 61663, 61679, 61687, 
    61691, 61693, 61694, 61759, 61791, 61807, 61815, 61819, 61821, 61822, 61855, 61871, 61879, 61883, 61885, 61886, 61903, 61911, 61915, 61917, 
    61918, 61927, 61931, 61933, 61934, 61939, 61941, 61942, 61945, 61946, 61948, 62015, 62047, 62063, 62071, 62075, 62077, 62078, 62111, 62127, 
    62135, 62139, 62141, 62142, 62159, 62167, 62171, 62173, 62174, 62183, 62187, 62189, 62190, 62195, 62197, 62198, 62201, 62202, 62204, 62239, 
    62255, 62263, 62267, 62269, 62270, 62287, 62295, 62299, 62301, 62302, 62311, 62315, 62317, 62318, 62323, 62325, 62326, 62329, 62330, 62332, 
    62351, 62359, 62363, 62365, 62366, 62375, 62379, 62381, 62382, 62387, 62389, 62390, 62393, 62394, 62396, 62407, 62411, 62413, 62414, 62419, 
    62421, 62422, 62425, 62426, 62428, 62435, 62437, 62438, 62441, 62442, 62444, 62449, 62450, 62452, 62456, 62527, 62559, 62575, 62583, 62587, 
    62589, 62590, 62623, 62639, 62647, 62651, 62653, 62654, 62671, 62679, 62683, 62685, 62686, 62695, 62699, 62701, 62702, 62707, 62709, 62710, 
    62713, 62714, 62716, 62751, 62767, 62775, 62779, 62781, 62782, 62799, 62807, 62811, 62813, 62814, 62823, 62827, 62829, 62830, 62835, 62837, 
    62838, 62841, 62842, 62844, 62863, 62871, 62875, 62877, 62878, 62887, 62891, 62893, 62894, 62899, 62901, 62902, 62905, 62906, 62908, 62919, 
    62923, 62925, 62926, 62931, 62933, 62934, 62937, 62938, 62940, 62947, 62949, 62950, 62953, 62954, 62956, 62961, 62962, 62964, 62968, 63007, 
    63023, 63031, 63035, 63037, 63038, 63055, 63063, 63067, 63069, 63070, 63079, 63083, 63085, 63086, 63091, 63093, 63094, 63097, 63098, 63100, 
    63119, 63127, 63131, 63133, 63134, 63143, 63147, 63149, 63150, 63155, 63157, 63158, 63161, 63162, 63164, 63175, 63179, 63181, 63182, 63187, 
    63189, 63190, 63193, 63194, 63196, 63203, 63205, 63206, 63209, 63210, 63212, 63217, 63218, 63220, 63224, 63247, 63255, 63259, 63261, 63262, 
    63271, 63275, 63277, 63278, 63283, 63285, 63286, 63289, 63290, 63292, 63303, 63307, 63309, 63310, 63315, 63317, 63318, 63321, 63322, 63324, 
    63331, 63333, 63334, 63337, 63338, 63340, 63345, 63346, 63348, 63352, 63367, 63371, 63373, 63374, 63379, 63381, 63382, 63385, 63386, 63388, 
    63395, 63397, 63398, 63401, 63402, 63404, 63409, 63410, 63412, 63416, 63427, 63429, 63430, 63433, 63434, 63436, 63441, 63442, 63444, 63448, 
    63457, 63458, 63460, 63464, 63472, 63551, 63583, 63599, 63607, 63611, 63613, 63614, 63647, 63663, 63671, 63675, 63677, 63678, 63695, 63703, 
    63707, 63709, 63710, 63719, 63723, 63725, 63726, 63731, 63733, 63734, 63737, 63738, 63740, 63775, 63791, 63799, 63803, 63805, 63806, 63823, 
    63831, 63835, 63837, 63838, 63847, 63851, 63853, 63854, 63859, 63861, 63862, 63865, 63866, 63868, 63887, 63895, 63899, 63901, 63902, 63911, 
    63915, 63917, 63918, 63923, 63925, 63926, 63929, 63930, 63932, 63943, 63947, 63949, 63950, 63955, 63957, 63958, 63961, 63962, 63964, 63971, 
    63973, 63974, 63977, 63978, 63980, 63985, 63986, 63988, 63992, 64031, 64047, 64055, 64059, 64061, 64062, 64079, 64087, 64091, 64093, 64094, 
    64103, 64107, 64109, 64110, 64115, 64117, 64118, 64121, 64122, 64124, 64143, 64151, 64155, 64157, 64158, 64167, 64171, 64173, 64174, 64179, 
    64181, 64182, 64185, 64186, 64188, 64199, 64203, 64205, 64206, 64211, 64213, 64214, 64217, 64218, 64220, 64227, 64229, 64230, 64233, 64234, 
    64236, 64241, 64242, 64244, 64248, 64271, 64279, 64283, 64285, 64286, 64295, 64299, 64301, 64302, 64307, 64309, 64310, 64313, 64314, 64316, 
    64327, 64331, 64333, 64334, 64339, 64341, 64342, 64345, 64346, 64348, 64355, 64357, 64358, 64361, 64362, 64364, 64369, 64370, 64372, 64376, 
    64391, 64395, 64397, 64398, 64403, 64405, 64406, 64409, 64410, 64412, 64419, 64421, 64422, 64425, 64426, 64428, 64433, 64434, 64436, 64440, 
    64451, 64453, 64454, 64457, 64458, 64460, 64465, 64466, 64468, 64472, 64481, 64482, 64484, 64488, 64496, 64543, 64559, 64567, 64571, 64573, 
    64574, 64591, 64599, 64603, 64605, 64606, 64615, 64619, 64621, 64622, 64627, 64629, 64630, 64633, 64634, 64636, 64655, 64663, 64667, 64669, 
    64670, 64679, 64683, 64685, 64686, 64691, 64693, 64694, 64697, 64698, 64700, 64711, 64715, 64717, 64718, 64723, 64725, 64726, 64729, 64730, 
    64732, 64739, 64741, 64742, 64745, 64746, 64748, 64753, 64754, 64756, 64760, 64783, 64791, 64795, 64797, 64798, 64807, 64811, 64813, 64814, 
    64819, 64821, 64822, 64825, 64826, 64828, 64839, 64843, 64845, 64846, 64851, 64853, 64854, 64857, 64858, 64860, 64867, 64869, 64870, 64873, 
    64874, 64876, 64881, 64882, 64884, 64888, 64903, 64907, 64909, 64910, 64915, 64917, 64918, 64921, 64922, 64924, 64931, 64933, 64934, 64937, 
    64938, 64940, 64945, 64946, 64948, 64952, 64963, 64965, 64966, 64969, 64970, 64972, 64977, 64978, 64980, 64984, 64993, 64994, 64996, 65000, 
    65008, 65039, 65047, 65051, 65053, 65054, 65063, 65067, 65069, 65070, 65075, 65077, 65078, 65081, 65082, 65084, 65095, 65099, 65101, 65102, 
    65107, 65109, 65110, 65113, 65114, 65116, 65123, 65125, 65126, 65129, 65130, 65132, 65137, 65138, 65140, 65144, 65159, 65163, 65165, 65166, 
    65171, 65173, 65174, 65177, 65178, 65180, 65187, 65189, 65190, 65193, 65194, 65196, 65201, 65202, 65204, 65208, 65219, 65221, 65222, 65225, 
    65226, 65228, 65233, 65234, 65236, 65240, 65249, 65250, 65252, 65256, 65264, 65287, 65291, 65293, 65294, 65299, 65301, 65302, 65305, 65306, 
    65308, 65315, 65317, 65318, 65321, 65322, 65324, 65329, 65330, 65332, 65336, 65347, 65349, 65350, 65353, 65354, 65356, 65361, 65362, 65364, 
    65368, 65377, 65378, 65380, 65384, 65392, 65411, 65413, 65414, 65417, 65418, 65420, 65425, 65426, 65428, 65432, 65441, 65442, 65444, 65448, 
    65456, 65473, 65474, 65476, 65480, 65488, 65504, },
  { 4095, 6143, 7167, 7679, 7935, 8063, 8127, 8159, 8175, 8183, 8187, 8189, 8190, 10239, 11263, 11775, 12031, 12159, 12223, 12255, 12271, 
    12279, 12283, 12285, 12286, 13311, 13823, 14079, 14207, 14271, 14303, 14319, 14327, 14331, 14333, 14334, 14847, 15103, 15231, 15295, 15327, 
    15343, 15351, 15355, 15357, 15358, 15615, 15743, 15807, 15839, 15855, 15863, 15867, 15869, 15870, 15999, 16063, 16095, 16111, 16119, 16123, 
    16125, 16126, 16191, 16223, 16239, 16247, 16251, 16253, 16254, 16287, 16303, 16311, 16315, 16317, 16318, 16335, 16343, 16347, 16349, 16350, 
    16359, 16363, 16365, 16366, 16371, 16373, 16374, 16377, 16378, 16380, 18431, 19455, 19967, 20223, 20351, 20415, 20447, 20463, 20471, 20475, 
    20477, 20478, 21503, 22015, 22271, 22399, 22463, 22495, 22511, 22519, 22523, 22525, 22526, 23039, 23295, 23423, 23487, 23519, 23535, 23543, 
    23547, 23549, 23550, 23807, 23935, 23999, 24031, 24047, 24055, 24059, 24061, 24062, 24191, 24255, 24287, 24303, 24311, 24315, 24317, 24318, 
    24383, 24415, 24431, 24439, 24443, 24445, 24446, 24479, 24495, 24503, 24507, 24509, 24510, 24527, 24535, 24539, 24541, 24542, 24551, 24555, 
    24557, 24558, 24563, 24565, 24566, 24569, 24570, 24572, 25599, 26111, 26367, 26495, 26559, 26591, 26607, 26615, 26619, 26621, 26622, 27135, 
    27391, 27519, 27583, 27615, 27631, 27639, 27643, 27645, 27646, 27903, 28031, 28095, 28127, 28143, 28151, 28155, 28157, 28158, 28287, 28351, 
    28383, 28399, 28407, 28411, 28413, 28414, 28479, 28511, 28527, 28535, 28539, 28541, 28542, 28575, 28591, 28599, 28603, 28605, 28606, 28623, 
    28631, 28635, 28637, 28638, 28647, 28651, 28653, 28654, 28659, 28661, 28662, 28665, 28666, 28668, 29183, 29439, 29567, 29631, 29663, 29679, 
    29687, 29691, 29693, 29694, 29951, 30079, 30143, 30175, 30191, 30199, 30203, 30205, 30206, 30335, 30399, 30431, 30447, 30455, 30459, 30461, 
    30462, 30527, 30559, 30575, 30583, 30587, 30589, 30590, 30623, 30639, 30647, 30651, 30653, 30654, 30671, 30679, 30683, 30685, 30686, 30695, 
    30699, 30701, 30702, 30707, 30709, 30710, 30713, 30714, 30716, 30975, 31103, 31167, 31199, 31215, 31223, 31227, 31229, 31230, 31359, 31423, 
    31455, 31471, 31479, 31483, 31485, 31486, 31551, 31583, 31599, 31607, 31611, 31613, 31614, 31647, 31663, 31671, 31675, 31677, 31678, 31695, 
    31703, 31707, 31709, 31710, 31719, 31723, 31725, 31726, 31731, 31733, 31734, 31737, 31738, 31740, 31871, 31935, 31967, 31983, 31991, 31995, 
    31997, 31998, 32063, 32095, 32111, 32119, 32123, 32125, 32126, 32159, 32175, 32183, 32187, 32189, 32190, 32207, 32215, 32219, 32221, 32222, 
    32231, 32235, 32237, 32238, 32243, 32245, 32246, 32249, 32250, 32252, 32319, 32351, 32367, 32375, 32379, 32381, 32382, 32415, 32431, 32439, 
    32443, 32445, 32446, 32463, 32471, 32475, 32477, 32478, 32487, 32491, 32493, 32494, 32499, 32501, 32502, 32505, 32506, 32508, 32543, 32559, 
    32567, 32571, 32573, 32574, 32591, 32599, 32603, 32605, 32606, 32615, 32619, 32621, 32622, 32627, 32629, 32630, 32633, 32634, 32636, 32655, 
    32663, 32667, 32669, 32670, 32679, 32683, 32685, 32686, 32691, 32693, 32694, 32697, 32698, 32700, 32711, 32715, 32717, 32718, 32723, 32725, 
    32726, 32729, 32730, 32732, 32739, 32741, 32742, 32745, 32746, 32748, 32753, 32754, 32756, 32760, 34815, 35839, 36351, 36607, 36735, 36799, 
    36831, 36847, 36855, 36859, 36861, 36862, 37887, 38399, 38655, 38783, 38847, 38879, 38895, 38903, 38907, 38909, 38910, 39423, 39679, 39807, 
    39871, 39903, 39919, 39927, 39931, 39933, 39934, 40191, 40319, 40383, 40415, 40431, 40439, 40443, 40445, 40446, 40575, 40639, 40671, 40687, 
    40695, 40699, 40701, 40702, 40767, 40799, 40815, 40823, 40827, 40829, 40830, 40863, 40879, 40887, 40891, 40893, 40894, 40911, 40919, 40923, 
    40925, 40926, 40935, 40939, 40941, 40942, 40947, 40949, 40950, 40953, 40954, 40956, 41983, 42495, 42751, 42879, 42943, 42975, 42991, 42999, 
    43003, 43005, 43006, 43519, 43775, 43903, 43967, 43999, 44015, 44023, 44027, 44029, 44030, 44287, 44415, 44479, 44511, 44527, 44535, 44539, 
    44541, 44542, 44671, 44735, 44767, 44783, 44791, 44795, 44797, 44798, 44863, 44895, 44911, 44919, 44923, 44925, 44926, 44959, 44975, 44983, 
    44987, 44989, 44990, 45007, 45015, 45019, 45021, 45022, 45031, 45035, 45037, 45038, 45043, 45045, 45046, 45049, 45050, 45052, 45567, 45823, 
    45951, 46015, 46047, 46063, 46071, 46075, 46077, 46078, 46335, 46463, 46527, 46559, 46575, 46583, 46587, 46589, 46590, 46719, 46783, 46815, 
    46831, 46839, 46843, 46845, 46846, 46911, 46943, 46959, 46967, 46971, 46973, 46974, 47007, 47023, 47031, 47035, 47037, 47038, 47055, 47063, 
    47067, 47069, 47070, 47079, 47083, 47085, 47086, 47091, 47093, 47094, 47097, 47098, 47100, 47359, 47487, 47551, 47583, 47599, 47607, 47611, 
    47613, 47614, 47743, 47807, 47839, 47855, 47863, 47867, 47869, 47870, 47935, 47967, 47983, 47991, 47995, 47997, 47998, 48031, 48047, 48055, 
    48059, 48061, 48062, 48079, 48087, 48091, 48093, 48094, 48103, 48107, 48109, 48110, 48115, 48117, 48118, 48121, 48122, 48124, 48255, 48319, 
    48351, 48367, 48375, 48379, 48381, 48382, 48447, 48479, 48495, 48503, 48507, 48509, 48510, 48543, 48559, 48567, 48571, 48573, 48574, 48591, 
    48599, 48603, 48605, 48606, 48615, 48619, 48621, 48622, 48627, 48629, 48630, 48633, 48634, 48636, 48703, 48735, 48751, 48759, 48763, 48765, 
    48766, 48799, 48815, 48823, 48827, 48829, 48830, 48847, 48855, 48859, 48861, 48862, 48871, 48875, 48877, 48878, 48883, 48885, 48886, 48889, 
    48890, 48892, 48927, 48943, 48951, 48955, 48957, 48958, 48975, 48983, 48987, 48989, 48990, 48999, 49003, 49005, 49006, 49011, 49013, 49014, 
    49017, 49018, 49020, 49039, 49047, 49051, 49053, 49054, 49063, 49067, 49069, 49070, 49075, 49077, 49078, 49081, 49082, 49084, 49095, 49099, 
    49101, 49102, 49107, 49109, 49110, 49113, 49114, 49116, 49123, 49125, 49126, 49129, 49130, 49132, 49137, 49138, 49140, 49144, 50175, 50687, 
    50943, 51071, 51135, 51167, 51183, 51191, 51195, 51197, 51198, 51711, 51967, 52095, 52159, 52191, 52207, 52215, 52219, 52221, 52222, 52479, 
    52607, 52671, 52703, 52719, 52727, 52731, 52733, 52734, 52863, 52927, 52959, 52975, 52983, 52987, 52989, 52990, 53055, 53087, 53103, 53111, 
    53115, 53117, 53118, 53151, 53167, 53175, 53179, 53181, 53182, 53199, 53207, 53211, 53213, 53214, 53223, 53227, 53229, 53230, 53235, 53237, 
    53238, 53241, 53242, 53244, 53759, 54015, 54143, 54207, 54239, 54255, 54263, 54267, 54269, 54270, 54527, 54655, 54719, 54751, 54767, 54775, 
    54779, 54781, 54782, 54911, 54975, 55007, 55023, 55031, 55035, 55037, 55038, 55103, 55135, 55151, 55159, 55163, 55165, 55166, 55199, 55215, 
    55223, 55227, 55229, 55230, 55247, 55255, 55259, 55261, 55262, 55271, 55275, 55277, 55278, 55283, 55285, 55286, 55289, 55290, 55292, 55551, 
    55679, 55743, 55775, 55791, 55799, 55803, 55805, 55806, 55935, 55999, 56031, 56047, 56055, 56059, 56061, 56062, 56127, 56159, 56175, 56183, 
    56187, 56189, 56190, 56223, 56239, 56247, 56251, 56253, 56254, 56271, 56279, 56283, 56285, 56286, 56295, 56299, 56301, 56302, 56307, 56309, 
    56310, 56313, 56314, 56316, 56447, 56511, 56543, 56559, 56567, 56571, 56573, 56574, 56639, 56671, 56687, 56695, 56699, 56701, 56702, 56735, 
    56751, 56759, 56763, 56765, 56766, 56783, 56791, 56795, 56797, 56798, 56807, 56811, 56813, 56814, 56819, 56821, 56822, 56825, 56826, 56828, 
    56895, 56927, 56943, 56951, 56955, 56957, 56958, 56991, 57007, 57015, 57019, 57021, 57022, 57039, 57047, 57051, 57053, 57054, 57063, 57067, 
    57069, 57070, 57075, 57077, 57078, 57081, 57082, 57084, 57119, 57135, 57143, 57147, 57149, 57150, 57167, 57175, 57179, 57181, 57182, 57191, 
    57195, 57197, 57198, 57203, 57205, 57206, 57209, 57210, 57212, 57231, 57239, 57243, 57245, 57246, 57255, 57259, 57261, 57262, 57267, 57269, 
    57270, 57273, 57274, 57276, 57287, 57291, 57293, 57294, 57299, 57301, 57302, 57305, 57306, 57308, 57315, 57317, 57318, 57321, 57322, 57324, 
    57329, 57330, 57332, 57336, 57855, 58111, 58239, 58303, 58335, 58351, 58359, 58363, 58365, 58366, 58623, 58751, 58815, 58847, 58863, 58871, 
    58875, 58877, 58878, 59007, 59071, 59103, 59119, 59127, 59131, 59133, 59134, 59199, 59231, 59247, 59255, 59259, 59261, 59262, 59295, 59311, 
    59319, 59323, 59325, 59326, 59343, 59351, 59355, 59357, 59358, 59367, 59371, 59373, 59374, 59379, 59381, 59382, 59385, 59386, 59388, 59647, 
    59775, 59839, 59871, 59887, 59895, 59899, 59901, 59902, 60031, 60095, 60127, 60143, 60151, 60155, 60157, 60158, 60223, 60255, 60271, 60279, 
    60283, 60285, 60286, 60319, 60335, 60343, 60347, 60349, 60350, 60367, 60375, 60379, 60381, 60382, 60391, 60395, 60397, 60398, 60403, 60405, 
    60406, 60409, 60410, 60412, 60543, 60607, 60639, 60655, 60663, 60667, 60669, 60670, 60735, 60767, 60783, 60791, 60795, 60797, 60798, 60831, 
    60847, 60855, 60859, 60861, 60862, 60879, 60887, 60891, 60893, 60894, 60903, 60907, 60909, 60910, 60915, 60917, 60918, 60921, 60922, 60924, 
    60991, 61023, 61039, 61047, 61051, 61053, 61054, 61087, 61103, 61111, 61115, 61117, 61118, 61135, 61143, 61147, 61149, 61150, 61159, 61163, 
    61165, 61166, 61171, 61173, 61174, 61177, 61178, 61180, 61215, 61231, 61239, 61243, 61245, 61246, 61263, 61271, 61275, 61277, 61278, 61287, 
    61291, 61293, 61294, 61299, 61301, 61302, 61305, 61306, 61308, 61327, 61335, 61339, 61341, 61342, 61351, 61355, 61357, 61358, 61363, 61365, 
    61366, 61369, 61370, 61372, 61383, 61387, 61389, 61390, 61395, 61397, 61398, 61401, 61402, 61404, 61411, 61413, 61414, 61417, 61418, 61420, 
    61425, 61426, 61428, 61432, 61695, 61823, 61887, 61919, 61935, 61943, 61947, 61949, 61950, 62079, 62143, 62175, 62191, 62199, 62203, 62205, 
    62206, 62271, 62303, 62319, 62327, 62331, 62333, 62334, 62367, 62383, 62391, 62395, 62397, 62398, 62415, 62423, 62427, 62429, 62430, 62439, 
    62443, 62445, 62446, 62451, 62453, 62454, 62457, 62458, 62460, 62591, 62655, 62687, 62703, 62711, 62715, 62717, 62718, 62783, 62815, 62831, 
    62839, 62843, 62845, 62846, 62879, 62895, 62903, 62907, 62909, 62910, 62927, 62935, 62939, 62941, 62942, 62951, 62955, 62957, 62958, 62963, 
    62965, 62966, 62969, 62970, 62972, 63039, 63071, 63087, 63095, 63099, 63101, 63102, 63135, 63151, 63159, 63163, 63165, 63166, 63183, 63191, 
    63195, 63197, 63198, 63207, 63211, 63213, 63214, 63219, 63221, 63222, 63225, 63226, 63228, 63263, 63279, 63287, 63291, 63293, 63294, 63311, 
    63319, 63323, 63325, 63326, 63335, 63339, 63341, 63342, 63347, 63349, 63350, 63353, 63354, 63356, 63375, 63383, 63387, 63389, 63390, 63399, 
    63403, 63405, 63406, 63411, 63413, 63414, 63417, 63418, 63420, 63431, 63435, 63437, 63438, 63443, 63445, 63446, 63449, 63450, 63452, 63459, 
    63461, 63462, 63465, 63466, 63468, 63473, 63474, 63476, 63480, 63615, 63679, 63711, 63727, 63735, 63739, 63741, 63742, 63807, 63839, 63855, 
    63863, 63867, 63869, 63870, 63903, 63919, 63927, 63931, 63933, 63934, 63951, 63959, 63963, 63965, 63966, 63975, 63979, 63981, 63982, 63987, 
    63989, 63990, 63993, 63994, 63996, 64063, 64095, 64111, 64119, 64123, 64125, 64126, 64159, 64175, 64183, 64187, 64189, 64190, 64207, 64215, 
    64219, 64221, 64222, 64231, 64235, 64237, 64238, 64243, 64245, 64246, 64249, 64250, 64252, 64287, 64303, 64311, 64315, 64317, 64318, 64335, 
    64343, 64347, 64349, 64350, 64359, 64363, 64365, 64366, 64371, 64373, 64374, 64377, 64378, 64380, 64399, 64407, 64411, 64413, 64414, 64423, 
    64427, 64429, 64430, 64435, 64437, 64438, 64441, 64442, 64444, 64455, 64459, 64461, 64462, 64467, 64469, 64470, 64473, 64474, 64476, 64483, 
    64485, 64486, 64489, 64490, 64492, 64497, 64498, 64500, 64504, 64575, 64607, 64623, 64631, 64635, 64637, 64638, 64671, 64687, 64695, 64699, 
    64701, 64702, 64719, 64727, 64731, 64733, 64734, 64743, 64747, 64749, 64750, 64755, 64757, 64758, 64761, 64762, 64764, 64799, 64815, 64823, 
    64827, 64829, 64830, 64847, 64855, 64859, 64861, 64862, 64871, 64875, 64877, 64878, 64883, 64885, 64886, 64889, 64890, 64892, 64911, 64919, 
    64923, 64925, 64926, 64935, 64939, 64941, 64942, 64947, 64949, 64950, 64953, 64954, 64956, 64967, 64971, 64973, 64974, 64979, 64981, 64982, 
    64985, 64986, 64988, 64995, 64997, 64998, 65001, 65002, 65004, 65009, 65010, 65012, 65016, 65055, 65071, 65079, 65083, 65085, 65086, 65103, 
    65111, 65115, 65117, 65118, 65127, 65131, 65133, 65134, 65139, 65141, 65142, 65145, 65146, 65148, 65167, 65175, 65179, 65181, 65182, 65191, 
    65195, 65197, 65198, 65203, 65205, 65206, 65209, 65210, 65212, 65223, 65227, 65229, 65230, 65235, 65237, 65238, 65241, 65242, 65244, 65251, 
    65253, 65254, 65257, 65258, 65260, 65265, 65266, 65268, 65272, 65295, 65303, 65307, 65309, 65310, 65319, 65323, 65325, 65326, 65331, 65333, 
    65334, 65337, 65338, 65340, 65351, 65355, 65357, 65358, 65363, 65365, 65366, 65369, 65370, 65372, 65379, 65381, 65382, 65385, 65386, 65388, 
    65393, 65394, 65396, 65400, 65415, 65419, 65421, 65422, 65427, 65429, 65430, 65433, 65434, 65436, 65443, 65445, 65446, 65449, 65450, 65452, 
    65457, 65458, 65460, 65464, 65475, 65477, 65478, 65481, 65482, 65484, 65489, 65490, 65492, 65496, 65505, 65506, 65508, 65512, 65520, },
  { 8191, 12287, 14335, 15359, 15871, 16127, 16255, 16319, 16351, 16367, 16375, 16379, 16381, 16382, 20479, 22527, 23551, 24063, 24319, 24447, 24511, 
    24543, 24559, 24567, 24571, 24573, 24574, 26623, 27647, 28159, 28415, 28543, 28607, 28639, 28655, 28663, 28667, 28669, 28670, 29695, 30207, 
    30463, 30591, 30655, 30687, 30703, 30711, 30715, 30717, 30718, 31231, 31487, 31615, 31679, 31711, 31727, 31735, 31739, 31741, 31742, 31999, 
    32127, 32191, 32223, 32239, 32247, 32251, 32253, 32254, 32383, 32447, 32479, 32495, 32503, 32507, 32509, 32510, 32575, 32607, 32623, 32631, 
    32635, 32637, 32638, 32671, 32687, 32695, 32699, 32701, 32702, 32719, 32727, 32731, 32733, 32734, 32743, 32747, 32749, 32750, 32755, 32757, 
    32758, 32761, 32762, 32764, 36863, 38911, 39935, 40447, 40703, 40831, 40895, 40927, 40943, 40951, 40955, 40957, 40958, 43007, 44031, 44543, 
    44799, 44927, 44991, 45023, 45039, 45047, 45051, 45053, 45054, 46079, 46591, 46847, 46975, 47039, 47071, 47087, 47095, 47099, 47101, 47102, 
    47615, 47871, 47999, 48063, 48095, 48111, 48119, 48123, 48125, 48126, 48383, 48511, 48575, 48607, 48623, 48631, 48635, 48637, 48638, 48767, 
    48831, 48863, 48879, 48887, 48891, 48893, 48894, 48959, 48991, 49007, 49015, 49019, 49021, 49022, 49055, 49071, 49079, 49083, 49085, 49086, 
    49103, 49111, 49115, 49117, 49118, 49127, 49131, 49133, 49134, 49139, 49141, 49142, 49145, 49146, 49148, 51199, 52223, 52735, 52991, 53119, 
    53183, 53215, 53231, 53239, 53243, 53245, 53246, 54271, 54783, 55039, 55167, 55231, 55263, 55279, 55287, 55291, 55293, 55294, 55807, 56063, 
    56191, 56255, 56287, 56303, 56311, 56315, 56317, 56318, 56575, 56703, 56767, 56799, 56815, 56823, 56827, 56829, 56830, 56959, 57023, 57055, 
    57071, 57079, 57083, 57085, 57086, 57151, 57183, 57199, 57207, 57211, 57213, 57214, 57247, 57263, 57271, 57275, 57277, 57278, 57295, 57303, 
    57307, 57309, 57310, 57319, 57323, 57325, 57326, 57331, 57333, 57334, 57337, 57338, 57340, 58367, 58879, 59135, 59263, 59327, 59359, 59375, 
    59383, 59387, 59389, 59390, 59903, 60159, 60287, 60351, 60383, 60399, 60407, 60411, 60413, 60414, 60671, 60799, 60863, 60895, 60911, 60919, 
    60923, 60925, 60926, 61055, 61119, 61151, 61167, 61175, 61179, 61181, 61182, 61247, 61279, 61295, 61303, 61307, 61309, 61310, 61343, 61359, 
    61367, 61371, 61373, 61374, 61391, 61399, 61403, 61405, 61406, 61415, 61419, 61421, 61422, 61427, 61429, 61430, 61433, 61434, 61436, 61951, 
    62207, 62335, 62399, 62431, 62447, 62455, 62459, 62461, 62462, 62719, 62847, 62911, 62943, 62959, 62967, 62971, 62973, 62974, 63103, 63167, 
    63199, 63215, 63223, 63227, 63229, 63230, 63295, 63327, 63343, 63351, 63355, 63357, 63358, 63391, 63407, 63415, 63419, 63421, 63422, 63439, 
    63447, 63451, 63453, 63454, 63463, 63467, 63469, 63470, 63475, 63477, 63478, 63481, 63482, 63484, 63743, 63871, 63935, 63967, 63983, 63991, 
    63995, 63997, 63998, 64127, 64191, 64223, 64239, 64247, 64251, 64253, 64254, 64319, 64351, 64367, 64375, 64379, 64381, 64382, 64415, 64431, 
    64439, 64443, 64445, 64446, 64463, 64471, 64475, 64477, 64478, 64487, 64491, 64493, 64494, 64499, 64501, 64502, 64505, 64506, 64508, 64639, 
    64703, 64735, 64751, 64759, 64763, 64765, 64766, 64831, 64863, 64879, 64887, 64891, 64893, 64894, 64927, 64943, 64951, 64955, 64957, 64958, 
    64975, 64983, 64987, 64989, 64990, 64999, 65003, 65005, 65006, 65011, 65013, 65014, 65017, 65018, 65020, 65087, 65119, 65135, 65143, 65147, 
    65149, 65150, 65183, 65199, 65207, 65211, 65213, 65214, 65231, 65239, 65243, 65245, 65246, 65255, 65259, 65261, 65262, 65267, 65269, 65270, 
    65273, 65274, 65276, 65311, 65327, 65335, 65339, 65341, 65342, 65359, 65367, 65371, 65373, 65374, 65383, 65387, 65389, 65390, 65395, 65397, 
    65398, 65401, 65402, 65404, 65423, 65431, 65435, 65437, 65438, 65447, 65451, 65453, 65454, 65459, 65461, 65462, 65465, 65466, 65468, 65479, 
    65483, 65485, 65486, 65491, 65493, 65494, 65497, 65498, 65500, 65507, 65509, 65510, 65513, 65514, 65516, 65521, 65522, 65524, 65528, },
  { 16383, 24575, 28671, 30719, 31743, 32255, 32511, 32639, 32703, 32735, 32751, 32759, 32763, 32765, 32766, 40959, 45055, 47103, 48127, 48639, 48895, 
    49023, 49087, 49119, 49135, 49143, 49147, 49149, 49150, 53247, 55295, 56319, 56831, 57087, 57215, 57279, 57311, 57327, 57335, 57339, 57341, 
    57342, 59391, 60415, 60927, 61183, 61311, 61375, 61407, 61423, 61431, 61435, 61437, 61438, 62463, 62975, 63231, 63359, 63423, 63455, 63471, 
    63479, 63483, 63485, 63486, 63999, 64255, 64383, 64447, 64479, 64495, 64503, 64507, 64509, 64510, 64767, 64895, 64959, 64991, 65007, 65015, 
    65019, 65021, 65022, 65151, 65215, 65247, 65263, 65271, 65275, 65277, 65278, 65343, 65375, 65391, 65399, 65403, 65405, 65406, 65439, 65455, 
    65463, 65467, 65469, 65470, 65487, 65495, 65499, 65501, 65502, 65511, 65515, 65517, 65518, 65523, 65525, 65526, 65529, 65530, 65532, },
  { 32767, 49151, 57343, 61439, 63487, 64511, 65023, 65279, 65407, 65471, 65503, 65519, 65527, 65531, 65533, 65534, },
  { 65535, },
};

const uint16_t BIT_COUNT_16_REVERSE_IDX[17] = {1, 16, 120, 560, 1820, 4368, 8008, 11440, 12870, 11440, 8008, 4368, 1820, 560, 120, 16, 1, };

const size_t BIT_MASKS_16[65536][17] = {
    { },
  { 1, },
  { 2, },
  { 1, 2, },
  { 4, },
  { 1, 4, },
  { 2, 4, },
  { 1, 2, 4, },
  { 8, },
  { 1, 8, },
  { 2, 8, },
  { 1, 2, 8, },
  { 4, 8, },
  { 1, 4, 8, },
  { 2, 4, 8, },
  { 1, 2, 4, 8, },
  { 16, },
  { 1, 16, },
  { 2, 16, },
  { 1, 2, 16, },
  { 4, 16, },
  { 1, 4, 16, },
  { 2, 4, 16, },
  { 1, 2, 4, 16, },
  { 8, 16, },
  { 1, 8, 16, },
  { 2, 8, 16, },
  { 1, 2, 8, 16, },
  { 4, 8, 16, },
  { 1, 4, 8, 16, },
  { 2, 4, 8, 16, },
  { 1, 2, 4, 8, 16, },
  { 32, },
  { 1, 32, },
  { 2, 32, },
  { 1, 2, 32, },
  { 4, 32, },
  { 1, 4, 32, },
  { 2, 4, 32, },
  { 1, 2, 4, 32, },
  { 8, 32, },
  { 1, 8, 32, },
  { 2, 8, 32, },
  { 1, 2, 8, 32, },
  { 4, 8, 32, },
  { 1, 4, 8, 32, },
  { 2, 4, 8, 32, },
  { 1, 2, 4, 8, 32, },
  { 16, 32, },
  { 1, 16, 32, },
  { 2, 16, 32, },
  { 1, 2, 16, 32, },
  { 4, 16, 32, },
  { 1, 4, 16, 32, },
  { 2, 4, 16, 32, },
  { 1, 2, 4, 16, 32, },
  { 8, 16, 32, },
  { 1, 8, 16, 32, },
  { 2, 8, 16, 32, },
  { 1, 2, 8, 16, 32, },
  { 4, 8, 16, 32, },
  { 1, 4, 8, 16, 32, },
  { 2, 4, 8, 16, 32, },
  { 1, 2, 4, 8, 16, 32, },
  { 64, },
  { 1, 64, },
  { 2, 64, },
  { 1, 2, 64, },
  { 4, 64, },
  { 1, 4, 64, },
  { 2, 4, 64, },
  { 1, 2, 4, 64, },
  { 8, 64, },
  { 1, 8, 64, },
  { 2, 8, 64, },
  { 1, 2, 8, 64, },
  { 4, 8, 64, },
  { 1, 4, 8, 64, },
  { 2, 4, 8, 64, },
  { 1, 2, 4, 8, 64, },
  { 16, 64, },
  { 1, 16, 64, },
  { 2, 16, 64, },
  { 1, 2, 16, 64, },
  { 4, 16, 64, },
  { 1, 4, 16, 64, },
  { 2, 4, 16, 64, },
  { 1, 2, 4, 16, 64, },
  { 8, 16, 64, },
  { 1, 8, 16, 64, },
  { 2, 8, 16, 64, },
  { 1, 2, 8, 16, 64, },
  { 4, 8, 16, 64, },
  { 1, 4, 8, 16, 64, },
  { 2, 4, 8, 16, 64, },
  { 1, 2, 4, 8, 16, 64, },
  { 32, 64, },
  { 1, 32, 64, },
  { 2, 32, 64, },
  { 1, 2, 32, 64, },
  { 4, 32, 64, },
  { 1, 4, 32, 64, },
  { 2, 4, 32, 64, },
  { 1, 2, 4, 32, 64, },
  { 8, 32, 64, },
  { 1, 8, 32, 64, },
  { 2, 8, 32, 64, },
  { 1, 2, 8, 32, 64, },
  { 4, 8, 32, 64, },
  { 1, 4, 8, 32, 64, },
  { 2, 4, 8, 32, 64, },
  { 1, 2, 4, 8, 32, 64, },
  { 16, 32, 64, },
  { 1, 16, 32, 64, },
  { 2, 16, 32, 64, },
  { 1, 2, 16, 32, 64, },
  { 4, 16, 32, 64, },
  { 1, 4, 16, 32, 64, },
  { 2, 4, 16, 32, 64, },
  { 1, 2, 4, 16, 32, 64, },
  { 8, 16, 32, 64, },
  { 1, 8, 16, 32, 64, },
  { 2, 8, 16, 32, 64, },
  { 1, 2, 8, 16, 32, 64, },
  { 4, 8, 16, 32, 64, },
  { 1, 4, 8, 16, 32, 64, },
  { 2, 4, 8, 16, 32, 64, },
  { 1, 2, 4, 8, 16, 32, 64, },
  { 128, },
  { 1, 128, },
  { 2, 128, },
  { 1, 2, 128, },
  { 4, 128, },
  { 1, 4, 128, },
  { 2, 4, 128, },
  { 1, 2, 4, 128, },
  { 8, 128, },
  { 1, 8, 128, },
  { 2, 8, 128, },
  { 1, 2, 8, 128, },
  { 4, 8, 128, },
  { 1, 4, 8, 128, },
  { 2, 4, 8, 128, },
  { 1, 2, 4, 8, 128, },
  { 16, 128, },
  { 1, 16, 128, },
  { 2, 16, 128, },
  { 1, 2, 16, 128, },
  { 4, 16, 128, },
  { 1, 4, 16, 128, },
  { 2, 4, 16, 128, },
  { 1, 2, 4, 16, 128, },
  { 8, 16, 128, },
  { 1, 8, 16, 128, },
  { 2, 8, 16, 128, },
  { 1, 2, 8, 16, 128, },
  { 4, 8, 16, 128, },
  { 1, 4, 8, 16, 128, },
  { 2, 4, 8, 16, 128, },
  { 1, 2, 4, 8, 16, 128, },
  { 32, 128, },
  { 1, 32, 128, },
  { 2, 32, 128, },
  { 1, 2, 32, 128, },
  { 4, 32, 128, },
  { 1, 4, 32, 128, },
  { 2, 4, 32, 128, },
  { 1, 2, 4, 32, 128, },
  { 8, 32, 128, },
  { 1, 8, 32, 128, },
  { 2, 8, 32, 128, },
  { 1, 2, 8, 32, 128, },
  { 4, 8, 32, 128, },
  { 1, 4, 8, 32, 128, },
  { 2, 4, 8, 32, 128, },
  { 1, 2, 4, 8, 32, 128, },
  { 16, 32, 128, },
  { 1, 16, 32, 128, },
  { 2, 16, 32, 128, },
  { 1, 2, 16, 32, 128, },
  { 4, 16, 32, 128, },
  { 1, 4, 16, 32, 128, },
  { 2, 4, 16, 32, 128, },
  { 1, 2, 4, 16, 32, 128, },
  { 8, 16, 32, 128, },
  { 1, 8, 16, 32, 128, },
  { 2, 8, 16, 32, 128, },
  { 1, 2, 8, 16, 32, 128, },
  { 4, 8, 16, 32, 128, },
  { 1, 4, 8, 16, 32, 128, },
  { 2, 4, 8, 16, 32, 128, },
  { 1, 2, 4, 8, 16, 32, 128, },
  { 64, 128, },
  { 1, 64, 128, },
  { 2, 64, 128, },
  { 1, 2, 64, 128, },
  { 4, 64, 128, },
  { 1, 4, 64, 128, },
  { 2, 4, 64, 128, },
  { 1, 2, 4, 64, 128, },
  { 8, 64, 128, },
  { 1, 8, 64, 128, },
  { 2, 8, 64, 128, },
  { 1, 2, 8, 64, 128, },
  { 4, 8, 64, 128, },
  { 1, 4, 8, 64, 128, },
  { 2, 4, 8, 64, 128, },
  { 1, 2, 4, 8, 64, 128, },
  { 16, 64, 128, },
  { 1, 16, 64, 128, },
  { 2, 16, 64, 128, },
  { 1, 2, 16, 64, 128, },
  { 4, 16, 64, 128, },
  { 1, 4, 16, 64, 128, },
  { 2, 4, 16, 64, 128, },
  { 1, 2, 4, 16, 64, 128, },
  { 8, 16, 64, 128, },
  { 1, 8, 16, 64, 128, },
  { 2, 8, 16, 64, 128, },
  { 1, 2, 8, 16, 64, 128, },
  { 4, 8, 16, 64, 128, },
  { 1, 4, 8, 16, 64, 128, },
  { 2, 4, 8, 16, 64, 128, },
  { 1, 2, 4, 8, 16, 64, 128, },
  { 32, 64, 128, },
  { 1, 32, 64, 128, },
  { 2, 32, 64, 128, },
  { 1, 2, 32, 64, 128, },
  { 4, 32, 64, 128, },
  { 1, 4, 32, 64, 128, },
  { 2, 4, 32, 64, 128, },
  { 1, 2, 4, 32, 64, 128, },
  { 8, 32, 64, 128, },
  { 1, 8, 32, 64, 128, },
  { 2, 8, 32, 64, 128, },
  { 1, 2, 8, 32, 64, 128, },
  { 4, 8, 32, 64, 128, },
  { 1, 4, 8, 32, 64, 128, },
  { 2, 4, 8, 32, 64, 128, },
  { 1, 2, 4, 8, 32, 64, 128, },
  { 16, 32, 64, 128, },
  { 1, 16, 32, 64, 128, },
  { 2, 16, 32, 64, 128, },
  { 1, 2, 16, 32, 64, 128, },
  { 4, 16, 32, 64, 128, },
  { 1, 4, 16, 32, 64, 128, },
  { 2, 4, 16, 32, 64, 128, },
  { 1, 2, 4, 16, 32, 64, 128, },
  { 8, 16, 32, 64, 128, },
  { 1, 8, 16, 32, 64, 128, },
  { 2, 8, 16, 32, 64, 128, },
  { 1, 2, 8, 16, 32, 64, 128, },
  { 4, 8, 16, 32, 64, 128, },
  { 1, 4, 8, 16, 32, 64, 128, },
  { 2, 4, 8, 16, 32, 64, 128, },
  { 1, 2, 4, 8, 16, 32, 64, 128, },
  { 256, },
  { 1, 256, },
  { 2, 256, },
  { 1, 2, 256, },
  { 4, 256, },
  { 1, 4, 256, },
  { 2, 4, 256, },
  { 1, 2, 4, 256, },
  { 8, 256, },
  { 1, 8, 256, },
  { 2, 8, 256, },
  { 1, 2, 8, 256, },
  { 4, 8, 256, },
  { 1, 4, 8, 256, },
  { 2, 4, 8, 256, },
  { 1, 2, 4, 8, 256, },
  { 16, 256, },
  { 1, 16, 256, },
  { 2, 16, 256, },
  { 1, 2, 16, 256, },
  { 4, 16, 256, },
  { 1, 4, 16, 256, },
  { 2, 4, 16, 256, },
  { 1, 2, 4, 16, 256, },
  { 8, 16, 256, },
  { 1, 8, 16, 256, },
  { 2, 8, 16, 256, },
  { 1, 2, 8, 16, 256, },
  { 4, 8, 16, 256, },
  { 1, 4, 8, 16, 256, },
  { 2, 4, 8, 16, 256, },
  { 1, 2, 4, 8, 16, 256, },
  { 32, 256, },
  { 1, 32, 256, },
  { 2, 32, 256, },
  { 1, 2, 32, 256, },
  { 4, 32, 256, },
  { 1, 4, 32, 256, },
  { 2, 4, 32, 256, },
  { 1, 2, 4, 32, 256, },
  { 8, 32, 256, },
  { 1, 8, 32, 256, },
  { 2, 8, 32, 256, },
  { 1, 2, 8, 32, 256, },
  { 4, 8, 32, 256, },
  { 1, 4, 8, 32, 256, },
  { 2, 4, 8, 32, 256, },
  { 1, 2, 4, 8, 32, 256, },
  { 16, 32, 256, },
  { 1, 16, 32, 256, },
  { 2, 16, 32, 256, },
  { 1, 2, 16, 32, 256, },
  { 4, 16, 32, 256, },
  { 1, 4, 16, 32, 256, },
  { 2, 4, 16, 32, 256, },
  { 1, 2, 4, 16, 32, 256, },
  { 8, 16, 32, 256, },
  { 1, 8, 16, 32, 256, },
  { 2, 8, 16, 32, 256, },
  { 1, 2, 8, 16, 32, 256, },
  { 4, 8, 16, 32, 256, },
  { 1, 4, 8, 16, 32, 256, },
  { 2, 4, 8, 16, 32, 256, },
  { 1, 2, 4, 8, 16, 32, 256, },
  { 64, 256, },
  { 1, 64, 256, },
  { 2, 64, 256, },
  { 1, 2, 64, 256, },
  { 4, 64, 256, },
  { 1, 4, 64, 256, },
  { 2, 4, 64, 256, },
  { 1, 2, 4, 64, 256, },
  { 8, 64, 256, },
  { 1, 8, 64, 256, },
  { 2, 8, 64, 256, },
  { 1, 2, 8, 64, 256, },
  { 4, 8, 64, 256, },
  { 1, 4, 8, 64, 256, },
  { 2, 4, 8, 64, 256, },
  { 1, 2, 4, 8, 64, 256, },
  { 16, 64, 256, },
  { 1, 16, 64, 256, },
  { 2, 16, 64, 256, },
  { 1, 2, 16, 64, 256, },
  { 4, 16, 64, 256, },
  { 1, 4, 16, 64, 256, },
  { 2, 4, 16, 64, 256, },
  { 1, 2, 4, 16, 64, 256, },
  { 8, 16, 64, 256, },
  { 1, 8, 16, 64, 256, },
  { 2, 8, 16, 64, 256, },
  { 1, 2, 8, 16, 64, 256, },
  { 4, 8, 16, 64, 256, },
  { 1, 4, 8, 16, 64, 256, },
  { 2, 4, 8, 16, 64, 256, },
  { 1, 2, 4, 8, 16, 64, 256, },
  { 32, 64, 256, },
  { 1, 32, 64, 256, },
  { 2, 32, 64, 256, },
  { 1, 2, 32, 64, 256, },
  { 4, 32, 64, 256, },
  { 1, 4, 32, 64, 256, },
  { 2, 4, 32, 64, 256, },
  { 1, 2, 4, 32, 64, 256, },
  { 8, 32, 64, 256, },
  { 1, 8, 32, 64, 256, },
  { 2, 8, 32, 64, 256, },
  { 1, 2, 8, 32, 64, 256, },
  { 4, 8, 32, 64, 256, },
  { 1, 4, 8, 32, 64, 256, },
  { 2, 4, 8, 32, 64, 256, },
  { 1, 2, 4, 8, 32, 64, 256, },
  { 16, 32, 64, 256, },
  { 1, 16, 32, 64, 256, },
  { 2, 16, 32, 64, 256, },
  { 1, 2, 16, 32, 64, 256, },
  { 4, 16, 32, 64, 256, },
  { 1, 4, 16, 32, 64, 256, },
  { 2, 4, 16, 32, 64, 256, },
  { 1, 2, 4, 16, 32, 64, 256, },
  { 8, 16, 32, 64, 256, },
  { 1, 8, 16, 32, 64, 256, },
  { 2, 8, 16, 32, 64, 256, },
  { 1, 2, 8, 16, 32, 64, 256, },
  { 4, 8, 16, 32, 64, 256, },
  { 1, 4, 8, 16, 32, 64, 256, },
  { 2, 4, 8, 16, 32, 64, 256, },
  { 1, 2, 4, 8, 16, 32, 64, 256, },
  { 128, 256, },
  { 1, 128, 256, },
  { 2, 128, 256, },
  { 1, 2, 128, 256, },
  { 4, 128, 256, },
  { 1, 4, 128, 256, },
  { 2, 4, 128, 256, },
  { 1, 2, 4, 128, 256, },
  { 8, 128, 256, },
  { 1, 8, 128, 256, },
  { 2, 8, 128, 256, },
  { 1, 2, 8, 128, 256, },
  { 4, 8, 128, 256, },
  { 1, 4, 8, 128, 256, },
  { 2, 4, 8, 128, 256, },
  { 1, 2, 4, 8, 128, 256, },
  { 16, 128, 256, },
  { 1, 16, 128, 256, },
  { 2, 16, 128, 256, },
  { 1, 2, 16, 128, 256, },
  { 4, 16, 128, 256, },
  { 1, 4, 16, 128, 256, },
  { 2, 4, 16, 128, 256, },
  { 1, 2, 4, 16, 128, 256, },
  { 8, 16, 128, 256, },
  { 1, 8, 16, 128, 256, },
  { 2, 8, 16, 128, 256, },
  { 1, 2, 8, 16, 128, 256, },
  { 4, 8, 16, 128, 256, },
  { 1, 4, 8, 16, 128, 256, },
  { 2, 4, 8, 16, 128, 256, },
  { 1, 2, 4, 8, 16, 128, 256, },
  { 32, 128, 256, },
  { 1, 32, 128, 256, },
  { 2, 32, 128, 256, },
  { 1, 2, 32, 128, 256, },
  { 4, 32, 128, 256, },
  { 1, 4, 32, 128, 256, },
  { 2, 4, 32, 128, 256, },
  { 1, 2, 4, 32, 128, 256, },
  { 8, 32, 128, 256, },
  { 1, 8, 32, 128, 256, },
  { 2, 8, 32, 128, 256, },
  { 1, 2, 8, 32, 128, 256, },
  { 4, 8, 32, 128, 256, },
  { 1, 4, 8, 32, 128, 256, },
  { 2, 4, 8, 32, 128, 256, },
  { 1, 2, 4, 8, 32, 128, 256, },
  { 16, 32, 128, 256, },
  { 1, 16, 32, 128, 256, },
  { 2, 16, 32, 128, 256, },
  { 1, 2, 16, 32, 128, 256, },
  { 4, 16, 32, 128, 256, },
  { 1, 4, 16, 32, 128, 256, },
  { 2, 4, 16, 32, 128, 256, },
  { 1, 2, 4, 16, 32, 128, 256, },
  { 8, 16, 32, 128, 256, },
  { 1, 8, 16, 32, 128, 256, },
  { 2, 8, 16, 32, 128, 256, },
  { 1, 2, 8, 16, 32, 128, 256, },
  { 4, 8, 16, 32, 128, 256, },
  { 1, 4, 8, 16, 32, 128, 256, },
  { 2, 4, 8, 16, 32, 128, 256, },
  { 1, 2, 4, 8, 16, 32, 128, 256, },
  { 64, 128, 256, },
  { 1, 64, 128, 256, },
  { 2, 64, 128, 256, },
  { 1, 2, 64, 128, 256, },
  { 4, 64, 128, 256, },
  { 1, 4, 64, 128, 256, },
  { 2, 4, 64, 128, 256, },
  { 1, 2, 4, 64, 128, 256, },
  { 8, 64, 128, 256, },
  { 1, 8, 64, 128, 256, },
  { 2, 8, 64, 128, 256, },
  { 1, 2, 8, 64, 128, 256, },
  { 4, 8, 64, 128, 256, },
  { 1, 4, 8, 64, 128, 256, },
  { 2, 4, 8, 64, 128, 256, },
  { 1, 2, 4, 8, 64, 128, 256, },
  { 16, 64, 128, 256, },
  { 1, 16, 64, 128, 256, },
  { 2, 16, 64, 128, 256, },
  { 1, 2, 16, 64, 128, 256, },
  { 4, 16, 64, 128, 256, },
  { 1, 4, 16, 64, 128, 256, },
  { 2, 4, 16, 64, 128, 256, },
  { 1, 2, 4, 16, 64, 128, 256, },
  { 8, 16, 64, 128, 256, },
  { 1, 8, 16, 64, 128, 256, },
  { 2, 8, 16, 64, 128, 256, },
  { 1, 2, 8, 16, 64, 128, 256, },
  { 4, 8, 16, 64, 128, 256, },
  { 1, 4, 8, 16, 64, 128, 256, },
  { 2, 4, 8, 16, 64, 128, 256, },
  { 1, 2, 4, 8, 16, 64, 128, 256, },
  { 32, 64, 128, 256, },
  { 1, 32, 64, 128, 256, },
  { 2, 32, 64, 128, 256, },
  { 1, 2, 32, 64, 128, 256, },
  { 4, 32, 64, 128, 256, },
  { 1, 4, 32, 64, 128, 256, },
  { 2, 4, 32, 64, 128, 256, },
  { 1, 2, 4, 32, 64, 128, 256, },
  { 8, 32, 64, 128, 256, },
  { 1, 8, 32, 64, 128, 256, },
  { 2, 8, 32, 64, 128, 256, },
  { 1, 2, 8, 32, 64, 128, 256, },
  { 4, 8, 32, 64, 128, 256, },
  { 1, 4, 8, 32, 64, 128, 256, },
  { 2, 4, 8, 32, 64, 128, 256, },
  { 1, 2, 4, 8, 32, 64, 128, 256, },
  { 16, 32, 64, 128, 256, },
  { 1, 16, 32, 64, 128, 256, },
  { 2, 16, 32, 64, 128, 256, },
  { 1, 2, 16, 32, 64, 128, 256, },
  { 4, 16, 32, 64, 128, 256, },
  { 1, 4, 16, 32, 64, 128, 256, },
  { 2, 4, 16, 32, 64, 128, 256, },
  { 1, 2, 4, 16, 32, 64, 128, 256, },
  { 8, 16, 32, 64, 128, 256, },
  { 1, 8, 16, 32, 64, 128, 256, },
  { 2, 8, 16, 32, 64, 128, 256, },
  { 1, 2, 8, 16, 32, 64, 128, 256, },
  { 4, 8, 16, 32, 64, 128, 256, },
  { 1, 4, 8, 16, 32, 64, 128, 256, },
  { 2, 4, 8, 16, 32, 64, 128, 256, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, },
  { 512, },
  { 1, 512, },
  { 2, 512, },
  { 1, 2, 512, },
  { 4, 512, },
  { 1, 4, 512, },
  { 2, 4, 512, },
  { 1, 2, 4, 512, },
  { 8, 512, },
  { 1, 8, 512, },
  { 2, 8, 512, },
  { 1, 2, 8, 512, },
  { 4, 8, 512, },
  { 1, 4, 8, 512, },
  { 2, 4, 8, 512, },
  { 1, 2, 4, 8, 512, },
  { 16, 512, },
  { 1, 16, 512, },
  { 2, 16, 512, },
  { 1, 2, 16, 512, },
  { 4, 16, 512, },
  { 1, 4, 16, 512, },
  { 2, 4, 16, 512, },
  { 1, 2, 4, 16, 512, },
  { 8, 16, 512, },
  { 1, 8, 16, 512, },
  { 2, 8, 16, 512, },
  { 1, 2, 8, 16, 512, },
  { 4, 8, 16, 512, },
  { 1, 4, 8, 16, 512, },
  { 2, 4, 8, 16, 512, },
  { 1, 2, 4, 8, 16, 512, },
  { 32, 512, },
  { 1, 32, 512, },
  { 2, 32, 512, },
  { 1, 2, 32, 512, },
  { 4, 32, 512, },
  { 1, 4, 32, 512, },
  { 2, 4, 32, 512, },
  { 1, 2, 4, 32, 512, },
  { 8, 32, 512, },
  { 1, 8, 32, 512, },
  { 2, 8, 32, 512, },
  { 1, 2, 8, 32, 512, },
  { 4, 8, 32, 512, },
  { 1, 4, 8, 32, 512, },
  { 2, 4, 8, 32, 512, },
  { 1, 2, 4, 8, 32, 512, },
  { 16, 32, 512, },
  { 1, 16, 32, 512, },
  { 2, 16, 32, 512, },
  { 1, 2, 16, 32, 512, },
  { 4, 16, 32, 512, },
  { 1, 4, 16, 32, 512, },
  { 2, 4, 16, 32, 512, },
  { 1, 2, 4, 16, 32, 512, },
  { 8, 16, 32, 512, },
  { 1, 8, 16, 32, 512, },
  { 2, 8, 16, 32, 512, },
  { 1, 2, 8, 16, 32, 512, },
  { 4, 8, 16, 32, 512, },
  { 1, 4, 8, 16, 32, 512, },
  { 2, 4, 8, 16, 32, 512, },
  { 1, 2, 4, 8, 16, 32, 512, },
  { 64, 512, },
  { 1, 64, 512, },
  { 2, 64, 512, },
  { 1, 2, 64, 512, },
  { 4, 64, 512, },
  { 1, 4, 64, 512, },
  { 2, 4, 64, 512, },
  { 1, 2, 4, 64, 512, },
  { 8, 64, 512, },
  { 1, 8, 64, 512, },
  { 2, 8, 64, 512, },
  { 1, 2, 8, 64, 512, },
  { 4, 8, 64, 512, },
  { 1, 4, 8, 64, 512, },
  { 2, 4, 8, 64, 512, },
  { 1, 2, 4, 8, 64, 512, },
  { 16, 64, 512, },
  { 1, 16, 64, 512, },
  { 2, 16, 64, 512, },
  { 1, 2, 16, 64, 512, },
  { 4, 16, 64, 512, },
  { 1, 4, 16, 64, 512, },
  { 2, 4, 16, 64, 512, },
  { 1, 2, 4, 16, 64, 512, },
  { 8, 16, 64, 512, },
  { 1, 8, 16, 64, 512, },
  { 2, 8, 16, 64, 512, },
  { 1, 2, 8, 16, 64, 512, },
  { 4, 8, 16, 64, 512, },
  { 1, 4, 8, 16, 64, 512, },
  { 2, 4, 8, 16, 64, 512, },
  { 1, 2, 4, 8, 16, 64, 512, },
  { 32, 64, 512, },
  { 1, 32, 64, 512, },
  { 2, 32, 64, 512, },
  { 1, 2, 32, 64, 512, },
  { 4, 32, 64, 512, },
  { 1, 4, 32, 64, 512, },
  { 2, 4, 32, 64, 512, },
  { 1, 2, 4, 32, 64, 512, },
  { 8, 32, 64, 512, },
  { 1, 8, 32, 64, 512, },
  { 2, 8, 32, 64, 512, },
  { 1, 2, 8, 32, 64, 512, },
  { 4, 8, 32, 64, 512, },
  { 1, 4, 8, 32, 64, 512, },
  { 2, 4, 8, 32, 64, 512, },
  { 1, 2, 4, 8, 32, 64, 512, },
  { 16, 32, 64, 512, },
  { 1, 16, 32, 64, 512, },
  { 2, 16, 32, 64, 512, },
  { 1, 2, 16, 32, 64, 512, },
  { 4, 16, 32, 64, 512, },
  { 1, 4, 16, 32, 64, 512, },
  { 2, 4, 16, 32, 64, 512, },
  { 1, 2, 4, 16, 32, 64, 512, },
  { 8, 16, 32, 64, 512, },
  { 1, 8, 16, 32, 64, 512, },
  { 2, 8, 16, 32, 64, 512, },
  { 1, 2, 8, 16, 32, 64, 512, },
  { 4, 8, 16, 32, 64, 512, },
  { 1, 4, 8, 16, 32, 64, 512, },
  { 2, 4, 8, 16, 32, 64, 512, },
  { 1, 2, 4, 8, 16, 32, 64, 512, },
  { 128, 512, },
  { 1, 128, 512, },
  { 2, 128, 512, },
  { 1, 2, 128, 512, },
  { 4, 128, 512, },
  { 1, 4, 128, 512, },
  { 2, 4, 128, 512, },
  { 1, 2, 4, 128, 512, },
  { 8, 128, 512, },
  { 1, 8, 128, 512, },
  { 2, 8, 128, 512, },
  { 1, 2, 8, 128, 512, },
  { 4, 8, 128, 512, },
  { 1, 4, 8, 128, 512, },
  { 2, 4, 8, 128, 512, },
  { 1, 2, 4, 8, 128, 512, },
  { 16, 128, 512, },
  { 1, 16, 128, 512, },
  { 2, 16, 128, 512, },
  { 1, 2, 16, 128, 512, },
  { 4, 16, 128, 512, },
  { 1, 4, 16, 128, 512, },
  { 2, 4, 16, 128, 512, },
  { 1, 2, 4, 16, 128, 512, },
  { 8, 16, 128, 512, },
  { 1, 8, 16, 128, 512, },
  { 2, 8, 16, 128, 512, },
  { 1, 2, 8, 16, 128, 512, },
  { 4, 8, 16, 128, 512, },
  { 1, 4, 8, 16, 128, 512, },
  { 2, 4, 8, 16, 128, 512, },
  { 1, 2, 4, 8, 16, 128, 512, },
  { 32, 128, 512, },
  { 1, 32, 128, 512, },
  { 2, 32, 128, 512, },
  { 1, 2, 32, 128, 512, },
  { 4, 32, 128, 512, },
  { 1, 4, 32, 128, 512, },
  { 2, 4, 32, 128, 512, },
  { 1, 2, 4, 32, 128, 512, },
  { 8, 32, 128, 512, },
  { 1, 8, 32, 128, 512, },
  { 2, 8, 32, 128, 512, },
  { 1, 2, 8, 32, 128, 512, },
  { 4, 8, 32, 128, 512, },
  { 1, 4, 8, 32, 128, 512, },
  { 2, 4, 8, 32, 128, 512, },
  { 1, 2, 4, 8, 32, 128, 512, },
  { 16, 32, 128, 512, },
  { 1, 16, 32, 128, 512, },
  { 2, 16, 32, 128, 512, },
  { 1, 2, 16, 32, 128, 512, },
  { 4, 16, 32, 128, 512, },
  { 1, 4, 16, 32, 128, 512, },
  { 2, 4, 16, 32, 128, 512, },
  { 1, 2, 4, 16, 32, 128, 512, },
  { 8, 16, 32, 128, 512, },
  { 1, 8, 16, 32, 128, 512, },
  { 2, 8, 16, 32, 128, 512, },
  { 1, 2, 8, 16, 32, 128, 512, },
  { 4, 8, 16, 32, 128, 512, },
  { 1, 4, 8, 16, 32, 128, 512, },
  { 2, 4, 8, 16, 32, 128, 512, },
  { 1, 2, 4, 8, 16, 32, 128, 512, },
  { 64, 128, 512, },
  { 1, 64, 128, 512, },
  { 2, 64, 128, 512, },
  { 1, 2, 64, 128, 512, },
  { 4, 64, 128, 512, },
  { 1, 4, 64, 128, 512, },
  { 2, 4, 64, 128, 512, },
  { 1, 2, 4, 64, 128, 512, },
  { 8, 64, 128, 512, },
  { 1, 8, 64, 128, 512, },
  { 2, 8, 64, 128, 512, },
  { 1, 2, 8, 64, 128, 512, },
  { 4, 8, 64, 128, 512, },
  { 1, 4, 8, 64, 128, 512, },
  { 2, 4, 8, 64, 128, 512, },
  { 1, 2, 4, 8, 64, 128, 512, },
  { 16, 64, 128, 512, },
  { 1, 16, 64, 128, 512, },
  { 2, 16, 64, 128, 512, },
  { 1, 2, 16, 64, 128, 512, },
  { 4, 16, 64, 128, 512, },
  { 1, 4, 16, 64, 128, 512, },
  { 2, 4, 16, 64, 128, 512, },
  { 1, 2, 4, 16, 64, 128, 512, },
  { 8, 16, 64, 128, 512, },
  { 1, 8, 16, 64, 128, 512, },
  { 2, 8, 16, 64, 128, 512, },
  { 1, 2, 8, 16, 64, 128, 512, },
  { 4, 8, 16, 64, 128, 512, },
  { 1, 4, 8, 16, 64, 128, 512, },
  { 2, 4, 8, 16, 64, 128, 512, },
  { 1, 2, 4, 8, 16, 64, 128, 512, },
  { 32, 64, 128, 512, },
  { 1, 32, 64, 128, 512, },
  { 2, 32, 64, 128, 512, },
  { 1, 2, 32, 64, 128, 512, },
  { 4, 32, 64, 128, 512, },
  { 1, 4, 32, 64, 128, 512, },
  { 2, 4, 32, 64, 128, 512, },
  { 1, 2, 4, 32, 64, 128, 512, },
  { 8, 32, 64, 128, 512, },
  { 1, 8, 32, 64, 128, 512, },
  { 2, 8, 32, 64, 128, 512, },
  { 1, 2, 8, 32, 64, 128, 512, },
  { 4, 8, 32, 64, 128, 512, },
  { 1, 4, 8, 32, 64, 128, 512, },
  { 2, 4, 8, 32, 64, 128, 512, },
  { 1, 2, 4, 8, 32, 64, 128, 512, },
  { 16, 32, 64, 128, 512, },
  { 1, 16, 32, 64, 128, 512, },
  { 2, 16, 32, 64, 128, 512, },
  { 1, 2, 16, 32, 64, 128, 512, },
  { 4, 16, 32, 64, 128, 512, },
  { 1, 4, 16, 32, 64, 128, 512, },
  { 2, 4, 16, 32, 64, 128, 512, },
  { 1, 2, 4, 16, 32, 64, 128, 512, },
  { 8, 16, 32, 64, 128, 512, },
  { 1, 8, 16, 32, 64, 128, 512, },
  { 2, 8, 16, 32, 64, 128, 512, },
  { 1, 2, 8, 16, 32, 64, 128, 512, },
  { 4, 8, 16, 32, 64, 128, 512, },
  { 1, 4, 8, 16, 32, 64, 128, 512, },
  { 2, 4, 8, 16, 32, 64, 128, 512, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, },
  { 256, 512, },
  { 1, 256, 512, },
  { 2, 256, 512, },
  { 1, 2, 256, 512, },
  { 4, 256, 512, },
  { 1, 4, 256, 512, },
  { 2, 4, 256, 512, },
  { 1, 2, 4, 256, 512, },
  { 8, 256, 512, },
  { 1, 8, 256, 512, },
  { 2, 8, 256, 512, },
  { 1, 2, 8, 256, 512, },
  { 4, 8, 256, 512, },
  { 1, 4, 8, 256, 512, },
  { 2, 4, 8, 256, 512, },
  { 1, 2, 4, 8, 256, 512, },
  { 16, 256, 512, },
  { 1, 16, 256, 512, },
  { 2, 16, 256, 512, },
  { 1, 2, 16, 256, 512, },
  { 4, 16, 256, 512, },
  { 1, 4, 16, 256, 512, },
  { 2, 4, 16, 256, 512, },
  { 1, 2, 4, 16, 256, 512, },
  { 8, 16, 256, 512, },
  { 1, 8, 16, 256, 512, },
  { 2, 8, 16, 256, 512, },
  { 1, 2, 8, 16, 256, 512, },
  { 4, 8, 16, 256, 512, },
  { 1, 4, 8, 16, 256, 512, },
  { 2, 4, 8, 16, 256, 512, },
  { 1, 2, 4, 8, 16, 256, 512, },
  { 32, 256, 512, },
  { 1, 32, 256, 512, },
  { 2, 32, 256, 512, },
  { 1, 2, 32, 256, 512, },
  { 4, 32, 256, 512, },
  { 1, 4, 32, 256, 512, },
  { 2, 4, 32, 256, 512, },
  { 1, 2, 4, 32, 256, 512, },
  { 8, 32, 256, 512, },
  { 1, 8, 32, 256, 512, },
  { 2, 8, 32, 256, 512, },
  { 1, 2, 8, 32, 256, 512, },
  { 4, 8, 32, 256, 512, },
  { 1, 4, 8, 32, 256, 512, },
  { 2, 4, 8, 32, 256, 512, },
  { 1, 2, 4, 8, 32, 256, 512, },
  { 16, 32, 256, 512, },
  { 1, 16, 32, 256, 512, },
  { 2, 16, 32, 256, 512, },
  { 1, 2, 16, 32, 256, 512, },
  { 4, 16, 32, 256, 512, },
  { 1, 4, 16, 32, 256, 512, },
  { 2, 4, 16, 32, 256, 512, },
  { 1, 2, 4, 16, 32, 256, 512, },
  { 8, 16, 32, 256, 512, },
  { 1, 8, 16, 32, 256, 512, },
  { 2, 8, 16, 32, 256, 512, },
  { 1, 2, 8, 16, 32, 256, 512, },
  { 4, 8, 16, 32, 256, 512, },
  { 1, 4, 8, 16, 32, 256, 512, },
  { 2, 4, 8, 16, 32, 256, 512, },
  { 1, 2, 4, 8, 16, 32, 256, 512, },
  { 64, 256, 512, },
  { 1, 64, 256, 512, },
  { 2, 64, 256, 512, },
  { 1, 2, 64, 256, 512, },
  { 4, 64, 256, 512, },
  { 1, 4, 64, 256, 512, },
  { 2, 4, 64, 256, 512, },
  { 1, 2, 4, 64, 256, 512, },
  { 8, 64, 256, 512, },
  { 1, 8, 64, 256, 512, },
  { 2, 8, 64, 256, 512, },
  { 1, 2, 8, 64, 256, 512, },
  { 4, 8, 64, 256, 512, },
  { 1, 4, 8, 64, 256, 512, },
  { 2, 4, 8, 64, 256, 512, },
  { 1, 2, 4, 8, 64, 256, 512, },
  { 16, 64, 256, 512, },
  { 1, 16, 64, 256, 512, },
  { 2, 16, 64, 256, 512, },
  { 1, 2, 16, 64, 256, 512, },
  { 4, 16, 64, 256, 512, },
  { 1, 4, 16, 64, 256, 512, },
  { 2, 4, 16, 64, 256, 512, },
  { 1, 2, 4, 16, 64, 256, 512, },
  { 8, 16, 64, 256, 512, },
  { 1, 8, 16, 64, 256, 512, },
  { 2, 8, 16, 64, 256, 512, },
  { 1, 2, 8, 16, 64, 256, 512, },
  { 4, 8, 16, 64, 256, 512, },
  { 1, 4, 8, 16, 64, 256, 512, },
  { 2, 4, 8, 16, 64, 256, 512, },
  { 1, 2, 4, 8, 16, 64, 256, 512, },
  { 32, 64, 256, 512, },
  { 1, 32, 64, 256, 512, },
  { 2, 32, 64, 256, 512, },
  { 1, 2, 32, 64, 256, 512, },
  { 4, 32, 64, 256, 512, },
  { 1, 4, 32, 64, 256, 512, },
  { 2, 4, 32, 64, 256, 512, },
  { 1, 2, 4, 32, 64, 256, 512, },
  { 8, 32, 64, 256, 512, },
  { 1, 8, 32, 64, 256, 512, },
  { 2, 8, 32, 64, 256, 512, },
  { 1, 2, 8, 32, 64, 256, 512, },
  { 4, 8, 32, 64, 256, 512, },
  { 1, 4, 8, 32, 64, 256, 512, },
  { 2, 4, 8, 32, 64, 256, 512, },
  { 1, 2, 4, 8, 32, 64, 256, 512, },
  { 16, 32, 64, 256, 512, },
  { 1, 16, 32, 64, 256, 512, },
  { 2, 16, 32, 64, 256, 512, },
  { 1, 2, 16, 32, 64, 256, 512, },
  { 4, 16, 32, 64, 256, 512, },
  { 1, 4, 16, 32, 64, 256, 512, },
  { 2, 4, 16, 32, 64, 256, 512, },
  { 1, 2, 4, 16, 32, 64, 256, 512, },
  { 8, 16, 32, 64, 256, 512, },
  { 1, 8, 16, 32, 64, 256, 512, },
  { 2, 8, 16, 32, 64, 256, 512, },
  { 1, 2, 8, 16, 32, 64, 256, 512, },
  { 4, 8, 16, 32, 64, 256, 512, },
  { 1, 4, 8, 16, 32, 64, 256, 512, },
  { 2, 4, 8, 16, 32, 64, 256, 512, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, },
  { 128, 256, 512, },
  { 1, 128, 256, 512, },
  { 2, 128, 256, 512, },
  { 1, 2, 128, 256, 512, },
  { 4, 128, 256, 512, },
  { 1, 4, 128, 256, 512, },
  { 2, 4, 128, 256, 512, },
  { 1, 2, 4, 128, 256, 512, },
  { 8, 128, 256, 512, },
  { 1, 8, 128, 256, 512, },
  { 2, 8, 128, 256, 512, },
  { 1, 2, 8, 128, 256, 512, },
  { 4, 8, 128, 256, 512, },
  { 1, 4, 8, 128, 256, 512, },
  { 2, 4, 8, 128, 256, 512, },
  { 1, 2, 4, 8, 128, 256, 512, },
  { 16, 128, 256, 512, },
  { 1, 16, 128, 256, 512, },
  { 2, 16, 128, 256, 512, },
  { 1, 2, 16, 128, 256, 512, },
  { 4, 16, 128, 256, 512, },
  { 1, 4, 16, 128, 256, 512, },
  { 2, 4, 16, 128, 256, 512, },
  { 1, 2, 4, 16, 128, 256, 512, },
  { 8, 16, 128, 256, 512, },
  { 1, 8, 16, 128, 256, 512, },
  { 2, 8, 16, 128, 256, 512, },
  { 1, 2, 8, 16, 128, 256, 512, },
  { 4, 8, 16, 128, 256, 512, },
  { 1, 4, 8, 16, 128, 256, 512, },
  { 2, 4, 8, 16, 128, 256, 512, },
  { 1, 2, 4, 8, 16, 128, 256, 512, },
  { 32, 128, 256, 512, },
  { 1, 32, 128, 256, 512, },
  { 2, 32, 128, 256, 512, },
  { 1, 2, 32, 128, 256, 512, },
  { 4, 32, 128, 256, 512, },
  { 1, 4, 32, 128, 256, 512, },
  { 2, 4, 32, 128, 256, 512, },
  { 1, 2, 4, 32, 128, 256, 512, },
  { 8, 32, 128, 256, 512, },
  { 1, 8, 32, 128, 256, 512, },
  { 2, 8, 32, 128, 256, 512, },
  { 1, 2, 8, 32, 128, 256, 512, },
  { 4, 8, 32, 128, 256, 512, },
  { 1, 4, 8, 32, 128, 256, 512, },
  { 2, 4, 8, 32, 128, 256, 512, },
  { 1, 2, 4, 8, 32, 128, 256, 512, },
  { 16, 32, 128, 256, 512, },
  { 1, 16, 32, 128, 256, 512, },
  { 2, 16, 32, 128, 256, 512, },
  { 1, 2, 16, 32, 128, 256, 512, },
  { 4, 16, 32, 128, 256, 512, },
  { 1, 4, 16, 32, 128, 256, 512, },
  { 2, 4, 16, 32, 128, 256, 512, },
  { 1, 2, 4, 16, 32, 128, 256, 512, },
  { 8, 16, 32, 128, 256, 512, },
  { 1, 8, 16, 32, 128, 256, 512, },
  { 2, 8, 16, 32, 128, 256, 512, },
  { 1, 2, 8, 16, 32, 128, 256, 512, },
  { 4, 8, 16, 32, 128, 256, 512, },
  { 1, 4, 8, 16, 32, 128, 256, 512, },
  { 2, 4, 8, 16, 32, 128, 256, 512, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, },
  { 64, 128, 256, 512, },
  { 1, 64, 128, 256, 512, },
  { 2, 64, 128, 256, 512, },
  { 1, 2, 64, 128, 256, 512, },
  { 4, 64, 128, 256, 512, },
  { 1, 4, 64, 128, 256, 512, },
  { 2, 4, 64, 128, 256, 512, },
  { 1, 2, 4, 64, 128, 256, 512, },
  { 8, 64, 128, 256, 512, },
  { 1, 8, 64, 128, 256, 512, },
  { 2, 8, 64, 128, 256, 512, },
  { 1, 2, 8, 64, 128, 256, 512, },
  { 4, 8, 64, 128, 256, 512, },
  { 1, 4, 8, 64, 128, 256, 512, },
  { 2, 4, 8, 64, 128, 256, 512, },
  { 1, 2, 4, 8, 64, 128, 256, 512, },
  { 16, 64, 128, 256, 512, },
  { 1, 16, 64, 128, 256, 512, },
  { 2, 16, 64, 128, 256, 512, },
  { 1, 2, 16, 64, 128, 256, 512, },
  { 4, 16, 64, 128, 256, 512, },
  { 1, 4, 16, 64, 128, 256, 512, },
  { 2, 4, 16, 64, 128, 256, 512, },
  { 1, 2, 4, 16, 64, 128, 256, 512, },
  { 8, 16, 64, 128, 256, 512, },
  { 1, 8, 16, 64, 128, 256, 512, },
  { 2, 8, 16, 64, 128, 256, 512, },
  { 1, 2, 8, 16, 64, 128, 256, 512, },
  { 4, 8, 16, 64, 128, 256, 512, },
  { 1, 4, 8, 16, 64, 128, 256, 512, },
  { 2, 4, 8, 16, 64, 128, 256, 512, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, },
  { 32, 64, 128, 256, 512, },
  { 1, 32, 64, 128, 256, 512, },
  { 2, 32, 64, 128, 256, 512, },
  { 1, 2, 32, 64, 128, 256, 512, },
  { 4, 32, 64, 128, 256, 512, },
  { 1, 4, 32, 64, 128, 256, 512, },
  { 2, 4, 32, 64, 128, 256, 512, },
  { 1, 2, 4, 32, 64, 128, 256, 512, },
  { 8, 32, 64, 128, 256, 512, },
  { 1, 8, 32, 64, 128, 256, 512, },
  { 2, 8, 32, 64, 128, 256, 512, },
  { 1, 2, 8, 32, 64, 128, 256, 512, },
  { 4, 8, 32, 64, 128, 256, 512, },
  { 1, 4, 8, 32, 64, 128, 256, 512, },
  { 2, 4, 8, 32, 64, 128, 256, 512, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, },
  { 16, 32, 64, 128, 256, 512, },
  { 1, 16, 32, 64, 128, 256, 512, },
  { 2, 16, 32, 64, 128, 256, 512, },
  { 1, 2, 16, 32, 64, 128, 256, 512, },
  { 4, 16, 32, 64, 128, 256, 512, },
  { 1, 4, 16, 32, 64, 128, 256, 512, },
  { 2, 4, 16, 32, 64, 128, 256, 512, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, },
  { 8, 16, 32, 64, 128, 256, 512, },
  { 1, 8, 16, 32, 64, 128, 256, 512, },
  { 2, 8, 16, 32, 64, 128, 256, 512, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, },
  { 4, 8, 16, 32, 64, 128, 256, 512, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, },
  { 1024, },
  { 1, 1024, },
  { 2, 1024, },
  { 1, 2, 1024, },
  { 4, 1024, },
  { 1, 4, 1024, },
  { 2, 4, 1024, },
  { 1, 2, 4, 1024, },
  { 8, 1024, },
  { 1, 8, 1024, },
  { 2, 8, 1024, },
  { 1, 2, 8, 1024, },
  { 4, 8, 1024, },
  { 1, 4, 8, 1024, },
  { 2, 4, 8, 1024, },
  { 1, 2, 4, 8, 1024, },
  { 16, 1024, },
  { 1, 16, 1024, },
  { 2, 16, 1024, },
  { 1, 2, 16, 1024, },
  { 4, 16, 1024, },
  { 1, 4, 16, 1024, },
  { 2, 4, 16, 1024, },
  { 1, 2, 4, 16, 1024, },
  { 8, 16, 1024, },
  { 1, 8, 16, 1024, },
  { 2, 8, 16, 1024, },
  { 1, 2, 8, 16, 1024, },
  { 4, 8, 16, 1024, },
  { 1, 4, 8, 16, 1024, },
  { 2, 4, 8, 16, 1024, },
  { 1, 2, 4, 8, 16, 1024, },
  { 32, 1024, },
  { 1, 32, 1024, },
  { 2, 32, 1024, },
  { 1, 2, 32, 1024, },
  { 4, 32, 1024, },
  { 1, 4, 32, 1024, },
  { 2, 4, 32, 1024, },
  { 1, 2, 4, 32, 1024, },
  { 8, 32, 1024, },
  { 1, 8, 32, 1024, },
  { 2, 8, 32, 1024, },
  { 1, 2, 8, 32, 1024, },
  { 4, 8, 32, 1024, },
  { 1, 4, 8, 32, 1024, },
  { 2, 4, 8, 32, 1024, },
  { 1, 2, 4, 8, 32, 1024, },
  { 16, 32, 1024, },
  { 1, 16, 32, 1024, },
  { 2, 16, 32, 1024, },
  { 1, 2, 16, 32, 1024, },
  { 4, 16, 32, 1024, },
  { 1, 4, 16, 32, 1024, },
  { 2, 4, 16, 32, 1024, },
  { 1, 2, 4, 16, 32, 1024, },
  { 8, 16, 32, 1024, },
  { 1, 8, 16, 32, 1024, },
  { 2, 8, 16, 32, 1024, },
  { 1, 2, 8, 16, 32, 1024, },
  { 4, 8, 16, 32, 1024, },
  { 1, 4, 8, 16, 32, 1024, },
  { 2, 4, 8, 16, 32, 1024, },
  { 1, 2, 4, 8, 16, 32, 1024, },
  { 64, 1024, },
  { 1, 64, 1024, },
  { 2, 64, 1024, },
  { 1, 2, 64, 1024, },
  { 4, 64, 1024, },
  { 1, 4, 64, 1024, },
  { 2, 4, 64, 1024, },
  { 1, 2, 4, 64, 1024, },
  { 8, 64, 1024, },
  { 1, 8, 64, 1024, },
  { 2, 8, 64, 1024, },
  { 1, 2, 8, 64, 1024, },
  { 4, 8, 64, 1024, },
  { 1, 4, 8, 64, 1024, },
  { 2, 4, 8, 64, 1024, },
  { 1, 2, 4, 8, 64, 1024, },
  { 16, 64, 1024, },
  { 1, 16, 64, 1024, },
  { 2, 16, 64, 1024, },
  { 1, 2, 16, 64, 1024, },
  { 4, 16, 64, 1024, },
  { 1, 4, 16, 64, 1024, },
  { 2, 4, 16, 64, 1024, },
  { 1, 2, 4, 16, 64, 1024, },
  { 8, 16, 64, 1024, },
  { 1, 8, 16, 64, 1024, },
  { 2, 8, 16, 64, 1024, },
  { 1, 2, 8, 16, 64, 1024, },
  { 4, 8, 16, 64, 1024, },
  { 1, 4, 8, 16, 64, 1024, },
  { 2, 4, 8, 16, 64, 1024, },
  { 1, 2, 4, 8, 16, 64, 1024, },
  { 32, 64, 1024, },
  { 1, 32, 64, 1024, },
  { 2, 32, 64, 1024, },
  { 1, 2, 32, 64, 1024, },
  { 4, 32, 64, 1024, },
  { 1, 4, 32, 64, 1024, },
  { 2, 4, 32, 64, 1024, },
  { 1, 2, 4, 32, 64, 1024, },
  { 8, 32, 64, 1024, },
  { 1, 8, 32, 64, 1024, },
  { 2, 8, 32, 64, 1024, },
  { 1, 2, 8, 32, 64, 1024, },
  { 4, 8, 32, 64, 1024, },
  { 1, 4, 8, 32, 64, 1024, },
  { 2, 4, 8, 32, 64, 1024, },
  { 1, 2, 4, 8, 32, 64, 1024, },
  { 16, 32, 64, 1024, },
  { 1, 16, 32, 64, 1024, },
  { 2, 16, 32, 64, 1024, },
  { 1, 2, 16, 32, 64, 1024, },
  { 4, 16, 32, 64, 1024, },
  { 1, 4, 16, 32, 64, 1024, },
  { 2, 4, 16, 32, 64, 1024, },
  { 1, 2, 4, 16, 32, 64, 1024, },
  { 8, 16, 32, 64, 1024, },
  { 1, 8, 16, 32, 64, 1024, },
  { 2, 8, 16, 32, 64, 1024, },
  { 1, 2, 8, 16, 32, 64, 1024, },
  { 4, 8, 16, 32, 64, 1024, },
  { 1, 4, 8, 16, 32, 64, 1024, },
  { 2, 4, 8, 16, 32, 64, 1024, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, },
  { 128, 1024, },
  { 1, 128, 1024, },
  { 2, 128, 1024, },
  { 1, 2, 128, 1024, },
  { 4, 128, 1024, },
  { 1, 4, 128, 1024, },
  { 2, 4, 128, 1024, },
  { 1, 2, 4, 128, 1024, },
  { 8, 128, 1024, },
  { 1, 8, 128, 1024, },
  { 2, 8, 128, 1024, },
  { 1, 2, 8, 128, 1024, },
  { 4, 8, 128, 1024, },
  { 1, 4, 8, 128, 1024, },
  { 2, 4, 8, 128, 1024, },
  { 1, 2, 4, 8, 128, 1024, },
  { 16, 128, 1024, },
  { 1, 16, 128, 1024, },
  { 2, 16, 128, 1024, },
  { 1, 2, 16, 128, 1024, },
  { 4, 16, 128, 1024, },
  { 1, 4, 16, 128, 1024, },
  { 2, 4, 16, 128, 1024, },
  { 1, 2, 4, 16, 128, 1024, },
  { 8, 16, 128, 1024, },
  { 1, 8, 16, 128, 1024, },
  { 2, 8, 16, 128, 1024, },
  { 1, 2, 8, 16, 128, 1024, },
  { 4, 8, 16, 128, 1024, },
  { 1, 4, 8, 16, 128, 1024, },
  { 2, 4, 8, 16, 128, 1024, },
  { 1, 2, 4, 8, 16, 128, 1024, },
  { 32, 128, 1024, },
  { 1, 32, 128, 1024, },
  { 2, 32, 128, 1024, },
  { 1, 2, 32, 128, 1024, },
  { 4, 32, 128, 1024, },
  { 1, 4, 32, 128, 1024, },
  { 2, 4, 32, 128, 1024, },
  { 1, 2, 4, 32, 128, 1024, },
  { 8, 32, 128, 1024, },
  { 1, 8, 32, 128, 1024, },
  { 2, 8, 32, 128, 1024, },
  { 1, 2, 8, 32, 128, 1024, },
  { 4, 8, 32, 128, 1024, },
  { 1, 4, 8, 32, 128, 1024, },
  { 2, 4, 8, 32, 128, 1024, },
  { 1, 2, 4, 8, 32, 128, 1024, },
  { 16, 32, 128, 1024, },
  { 1, 16, 32, 128, 1024, },
  { 2, 16, 32, 128, 1024, },
  { 1, 2, 16, 32, 128, 1024, },
  { 4, 16, 32, 128, 1024, },
  { 1, 4, 16, 32, 128, 1024, },
  { 2, 4, 16, 32, 128, 1024, },
  { 1, 2, 4, 16, 32, 128, 1024, },
  { 8, 16, 32, 128, 1024, },
  { 1, 8, 16, 32, 128, 1024, },
  { 2, 8, 16, 32, 128, 1024, },
  { 1, 2, 8, 16, 32, 128, 1024, },
  { 4, 8, 16, 32, 128, 1024, },
  { 1, 4, 8, 16, 32, 128, 1024, },
  { 2, 4, 8, 16, 32, 128, 1024, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, },
  { 64, 128, 1024, },
  { 1, 64, 128, 1024, },
  { 2, 64, 128, 1024, },
  { 1, 2, 64, 128, 1024, },
  { 4, 64, 128, 1024, },
  { 1, 4, 64, 128, 1024, },
  { 2, 4, 64, 128, 1024, },
  { 1, 2, 4, 64, 128, 1024, },
  { 8, 64, 128, 1024, },
  { 1, 8, 64, 128, 1024, },
  { 2, 8, 64, 128, 1024, },
  { 1, 2, 8, 64, 128, 1024, },
  { 4, 8, 64, 128, 1024, },
  { 1, 4, 8, 64, 128, 1024, },
  { 2, 4, 8, 64, 128, 1024, },
  { 1, 2, 4, 8, 64, 128, 1024, },
  { 16, 64, 128, 1024, },
  { 1, 16, 64, 128, 1024, },
  { 2, 16, 64, 128, 1024, },
  { 1, 2, 16, 64, 128, 1024, },
  { 4, 16, 64, 128, 1024, },
  { 1, 4, 16, 64, 128, 1024, },
  { 2, 4, 16, 64, 128, 1024, },
  { 1, 2, 4, 16, 64, 128, 1024, },
  { 8, 16, 64, 128, 1024, },
  { 1, 8, 16, 64, 128, 1024, },
  { 2, 8, 16, 64, 128, 1024, },
  { 1, 2, 8, 16, 64, 128, 1024, },
  { 4, 8, 16, 64, 128, 1024, },
  { 1, 4, 8, 16, 64, 128, 1024, },
  { 2, 4, 8, 16, 64, 128, 1024, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, },
  { 32, 64, 128, 1024, },
  { 1, 32, 64, 128, 1024, },
  { 2, 32, 64, 128, 1024, },
  { 1, 2, 32, 64, 128, 1024, },
  { 4, 32, 64, 128, 1024, },
  { 1, 4, 32, 64, 128, 1024, },
  { 2, 4, 32, 64, 128, 1024, },
  { 1, 2, 4, 32, 64, 128, 1024, },
  { 8, 32, 64, 128, 1024, },
  { 1, 8, 32, 64, 128, 1024, },
  { 2, 8, 32, 64, 128, 1024, },
  { 1, 2, 8, 32, 64, 128, 1024, },
  { 4, 8, 32, 64, 128, 1024, },
  { 1, 4, 8, 32, 64, 128, 1024, },
  { 2, 4, 8, 32, 64, 128, 1024, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, },
  { 16, 32, 64, 128, 1024, },
  { 1, 16, 32, 64, 128, 1024, },
  { 2, 16, 32, 64, 128, 1024, },
  { 1, 2, 16, 32, 64, 128, 1024, },
  { 4, 16, 32, 64, 128, 1024, },
  { 1, 4, 16, 32, 64, 128, 1024, },
  { 2, 4, 16, 32, 64, 128, 1024, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, },
  { 8, 16, 32, 64, 128, 1024, },
  { 1, 8, 16, 32, 64, 128, 1024, },
  { 2, 8, 16, 32, 64, 128, 1024, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, },
  { 4, 8, 16, 32, 64, 128, 1024, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, },
  { 256, 1024, },
  { 1, 256, 1024, },
  { 2, 256, 1024, },
  { 1, 2, 256, 1024, },
  { 4, 256, 1024, },
  { 1, 4, 256, 1024, },
  { 2, 4, 256, 1024, },
  { 1, 2, 4, 256, 1024, },
  { 8, 256, 1024, },
  { 1, 8, 256, 1024, },
  { 2, 8, 256, 1024, },
  { 1, 2, 8, 256, 1024, },
  { 4, 8, 256, 1024, },
  { 1, 4, 8, 256, 1024, },
  { 2, 4, 8, 256, 1024, },
  { 1, 2, 4, 8, 256, 1024, },
  { 16, 256, 1024, },
  { 1, 16, 256, 1024, },
  { 2, 16, 256, 1024, },
  { 1, 2, 16, 256, 1024, },
  { 4, 16, 256, 1024, },
  { 1, 4, 16, 256, 1024, },
  { 2, 4, 16, 256, 1024, },
  { 1, 2, 4, 16, 256, 1024, },
  { 8, 16, 256, 1024, },
  { 1, 8, 16, 256, 1024, },
  { 2, 8, 16, 256, 1024, },
  { 1, 2, 8, 16, 256, 1024, },
  { 4, 8, 16, 256, 1024, },
  { 1, 4, 8, 16, 256, 1024, },
  { 2, 4, 8, 16, 256, 1024, },
  { 1, 2, 4, 8, 16, 256, 1024, },
  { 32, 256, 1024, },
  { 1, 32, 256, 1024, },
  { 2, 32, 256, 1024, },
  { 1, 2, 32, 256, 1024, },
  { 4, 32, 256, 1024, },
  { 1, 4, 32, 256, 1024, },
  { 2, 4, 32, 256, 1024, },
  { 1, 2, 4, 32, 256, 1024, },
  { 8, 32, 256, 1024, },
  { 1, 8, 32, 256, 1024, },
  { 2, 8, 32, 256, 1024, },
  { 1, 2, 8, 32, 256, 1024, },
  { 4, 8, 32, 256, 1024, },
  { 1, 4, 8, 32, 256, 1024, },
  { 2, 4, 8, 32, 256, 1024, },
  { 1, 2, 4, 8, 32, 256, 1024, },
  { 16, 32, 256, 1024, },
  { 1, 16, 32, 256, 1024, },
  { 2, 16, 32, 256, 1024, },
  { 1, 2, 16, 32, 256, 1024, },
  { 4, 16, 32, 256, 1024, },
  { 1, 4, 16, 32, 256, 1024, },
  { 2, 4, 16, 32, 256, 1024, },
  { 1, 2, 4, 16, 32, 256, 1024, },
  { 8, 16, 32, 256, 1024, },
  { 1, 8, 16, 32, 256, 1024, },
  { 2, 8, 16, 32, 256, 1024, },
  { 1, 2, 8, 16, 32, 256, 1024, },
  { 4, 8, 16, 32, 256, 1024, },
  { 1, 4, 8, 16, 32, 256, 1024, },
  { 2, 4, 8, 16, 32, 256, 1024, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, },
  { 64, 256, 1024, },
  { 1, 64, 256, 1024, },
  { 2, 64, 256, 1024, },
  { 1, 2, 64, 256, 1024, },
  { 4, 64, 256, 1024, },
  { 1, 4, 64, 256, 1024, },
  { 2, 4, 64, 256, 1024, },
  { 1, 2, 4, 64, 256, 1024, },
  { 8, 64, 256, 1024, },
  { 1, 8, 64, 256, 1024, },
  { 2, 8, 64, 256, 1024, },
  { 1, 2, 8, 64, 256, 1024, },
  { 4, 8, 64, 256, 1024, },
  { 1, 4, 8, 64, 256, 1024, },
  { 2, 4, 8, 64, 256, 1024, },
  { 1, 2, 4, 8, 64, 256, 1024, },
  { 16, 64, 256, 1024, },
  { 1, 16, 64, 256, 1024, },
  { 2, 16, 64, 256, 1024, },
  { 1, 2, 16, 64, 256, 1024, },
  { 4, 16, 64, 256, 1024, },
  { 1, 4, 16, 64, 256, 1024, },
  { 2, 4, 16, 64, 256, 1024, },
  { 1, 2, 4, 16, 64, 256, 1024, },
  { 8, 16, 64, 256, 1024, },
  { 1, 8, 16, 64, 256, 1024, },
  { 2, 8, 16, 64, 256, 1024, },
  { 1, 2, 8, 16, 64, 256, 1024, },
  { 4, 8, 16, 64, 256, 1024, },
  { 1, 4, 8, 16, 64, 256, 1024, },
  { 2, 4, 8, 16, 64, 256, 1024, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, },
  { 32, 64, 256, 1024, },
  { 1, 32, 64, 256, 1024, },
  { 2, 32, 64, 256, 1024, },
  { 1, 2, 32, 64, 256, 1024, },
  { 4, 32, 64, 256, 1024, },
  { 1, 4, 32, 64, 256, 1024, },
  { 2, 4, 32, 64, 256, 1024, },
  { 1, 2, 4, 32, 64, 256, 1024, },
  { 8, 32, 64, 256, 1024, },
  { 1, 8, 32, 64, 256, 1024, },
  { 2, 8, 32, 64, 256, 1024, },
  { 1, 2, 8, 32, 64, 256, 1024, },
  { 4, 8, 32, 64, 256, 1024, },
  { 1, 4, 8, 32, 64, 256, 1024, },
  { 2, 4, 8, 32, 64, 256, 1024, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, },
  { 16, 32, 64, 256, 1024, },
  { 1, 16, 32, 64, 256, 1024, },
  { 2, 16, 32, 64, 256, 1024, },
  { 1, 2, 16, 32, 64, 256, 1024, },
  { 4, 16, 32, 64, 256, 1024, },
  { 1, 4, 16, 32, 64, 256, 1024, },
  { 2, 4, 16, 32, 64, 256, 1024, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, },
  { 8, 16, 32, 64, 256, 1024, },
  { 1, 8, 16, 32, 64, 256, 1024, },
  { 2, 8, 16, 32, 64, 256, 1024, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, },
  { 4, 8, 16, 32, 64, 256, 1024, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, },
  { 128, 256, 1024, },
  { 1, 128, 256, 1024, },
  { 2, 128, 256, 1024, },
  { 1, 2, 128, 256, 1024, },
  { 4, 128, 256, 1024, },
  { 1, 4, 128, 256, 1024, },
  { 2, 4, 128, 256, 1024, },
  { 1, 2, 4, 128, 256, 1024, },
  { 8, 128, 256, 1024, },
  { 1, 8, 128, 256, 1024, },
  { 2, 8, 128, 256, 1024, },
  { 1, 2, 8, 128, 256, 1024, },
  { 4, 8, 128, 256, 1024, },
  { 1, 4, 8, 128, 256, 1024, },
  { 2, 4, 8, 128, 256, 1024, },
  { 1, 2, 4, 8, 128, 256, 1024, },
  { 16, 128, 256, 1024, },
  { 1, 16, 128, 256, 1024, },
  { 2, 16, 128, 256, 1024, },
  { 1, 2, 16, 128, 256, 1024, },
  { 4, 16, 128, 256, 1024, },
  { 1, 4, 16, 128, 256, 1024, },
  { 2, 4, 16, 128, 256, 1024, },
  { 1, 2, 4, 16, 128, 256, 1024, },
  { 8, 16, 128, 256, 1024, },
  { 1, 8, 16, 128, 256, 1024, },
  { 2, 8, 16, 128, 256, 1024, },
  { 1, 2, 8, 16, 128, 256, 1024, },
  { 4, 8, 16, 128, 256, 1024, },
  { 1, 4, 8, 16, 128, 256, 1024, },
  { 2, 4, 8, 16, 128, 256, 1024, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, },
  { 32, 128, 256, 1024, },
  { 1, 32, 128, 256, 1024, },
  { 2, 32, 128, 256, 1024, },
  { 1, 2, 32, 128, 256, 1024, },
  { 4, 32, 128, 256, 1024, },
  { 1, 4, 32, 128, 256, 1024, },
  { 2, 4, 32, 128, 256, 1024, },
  { 1, 2, 4, 32, 128, 256, 1024, },
  { 8, 32, 128, 256, 1024, },
  { 1, 8, 32, 128, 256, 1024, },
  { 2, 8, 32, 128, 256, 1024, },
  { 1, 2, 8, 32, 128, 256, 1024, },
  { 4, 8, 32, 128, 256, 1024, },
  { 1, 4, 8, 32, 128, 256, 1024, },
  { 2, 4, 8, 32, 128, 256, 1024, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, },
  { 16, 32, 128, 256, 1024, },
  { 1, 16, 32, 128, 256, 1024, },
  { 2, 16, 32, 128, 256, 1024, },
  { 1, 2, 16, 32, 128, 256, 1024, },
  { 4, 16, 32, 128, 256, 1024, },
  { 1, 4, 16, 32, 128, 256, 1024, },
  { 2, 4, 16, 32, 128, 256, 1024, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, },
  { 8, 16, 32, 128, 256, 1024, },
  { 1, 8, 16, 32, 128, 256, 1024, },
  { 2, 8, 16, 32, 128, 256, 1024, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, },
  { 4, 8, 16, 32, 128, 256, 1024, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, },
  { 64, 128, 256, 1024, },
  { 1, 64, 128, 256, 1024, },
  { 2, 64, 128, 256, 1024, },
  { 1, 2, 64, 128, 256, 1024, },
  { 4, 64, 128, 256, 1024, },
  { 1, 4, 64, 128, 256, 1024, },
  { 2, 4, 64, 128, 256, 1024, },
  { 1, 2, 4, 64, 128, 256, 1024, },
  { 8, 64, 128, 256, 1024, },
  { 1, 8, 64, 128, 256, 1024, },
  { 2, 8, 64, 128, 256, 1024, },
  { 1, 2, 8, 64, 128, 256, 1024, },
  { 4, 8, 64, 128, 256, 1024, },
  { 1, 4, 8, 64, 128, 256, 1024, },
  { 2, 4, 8, 64, 128, 256, 1024, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, },
  { 16, 64, 128, 256, 1024, },
  { 1, 16, 64, 128, 256, 1024, },
  { 2, 16, 64, 128, 256, 1024, },
  { 1, 2, 16, 64, 128, 256, 1024, },
  { 4, 16, 64, 128, 256, 1024, },
  { 1, 4, 16, 64, 128, 256, 1024, },
  { 2, 4, 16, 64, 128, 256, 1024, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, },
  { 8, 16, 64, 128, 256, 1024, },
  { 1, 8, 16, 64, 128, 256, 1024, },
  { 2, 8, 16, 64, 128, 256, 1024, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, },
  { 4, 8, 16, 64, 128, 256, 1024, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, },
  { 32, 64, 128, 256, 1024, },
  { 1, 32, 64, 128, 256, 1024, },
  { 2, 32, 64, 128, 256, 1024, },
  { 1, 2, 32, 64, 128, 256, 1024, },
  { 4, 32, 64, 128, 256, 1024, },
  { 1, 4, 32, 64, 128, 256, 1024, },
  { 2, 4, 32, 64, 128, 256, 1024, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, },
  { 8, 32, 64, 128, 256, 1024, },
  { 1, 8, 32, 64, 128, 256, 1024, },
  { 2, 8, 32, 64, 128, 256, 1024, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, },
  { 4, 8, 32, 64, 128, 256, 1024, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, },
  { 16, 32, 64, 128, 256, 1024, },
  { 1, 16, 32, 64, 128, 256, 1024, },
  { 2, 16, 32, 64, 128, 256, 1024, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, },
  { 4, 16, 32, 64, 128, 256, 1024, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, },
  { 8, 16, 32, 64, 128, 256, 1024, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, },
  { 512, 1024, },
  { 1, 512, 1024, },
  { 2, 512, 1024, },
  { 1, 2, 512, 1024, },
  { 4, 512, 1024, },
  { 1, 4, 512, 1024, },
  { 2, 4, 512, 1024, },
  { 1, 2, 4, 512, 1024, },
  { 8, 512, 1024, },
  { 1, 8, 512, 1024, },
  { 2, 8, 512, 1024, },
  { 1, 2, 8, 512, 1024, },
  { 4, 8, 512, 1024, },
  { 1, 4, 8, 512, 1024, },
  { 2, 4, 8, 512, 1024, },
  { 1, 2, 4, 8, 512, 1024, },
  { 16, 512, 1024, },
  { 1, 16, 512, 1024, },
  { 2, 16, 512, 1024, },
  { 1, 2, 16, 512, 1024, },
  { 4, 16, 512, 1024, },
  { 1, 4, 16, 512, 1024, },
  { 2, 4, 16, 512, 1024, },
  { 1, 2, 4, 16, 512, 1024, },
  { 8, 16, 512, 1024, },
  { 1, 8, 16, 512, 1024, },
  { 2, 8, 16, 512, 1024, },
  { 1, 2, 8, 16, 512, 1024, },
  { 4, 8, 16, 512, 1024, },
  { 1, 4, 8, 16, 512, 1024, },
  { 2, 4, 8, 16, 512, 1024, },
  { 1, 2, 4, 8, 16, 512, 1024, },
  { 32, 512, 1024, },
  { 1, 32, 512, 1024, },
  { 2, 32, 512, 1024, },
  { 1, 2, 32, 512, 1024, },
  { 4, 32, 512, 1024, },
  { 1, 4, 32, 512, 1024, },
  { 2, 4, 32, 512, 1024, },
  { 1, 2, 4, 32, 512, 1024, },
  { 8, 32, 512, 1024, },
  { 1, 8, 32, 512, 1024, },
  { 2, 8, 32, 512, 1024, },
  { 1, 2, 8, 32, 512, 1024, },
  { 4, 8, 32, 512, 1024, },
  { 1, 4, 8, 32, 512, 1024, },
  { 2, 4, 8, 32, 512, 1024, },
  { 1, 2, 4, 8, 32, 512, 1024, },
  { 16, 32, 512, 1024, },
  { 1, 16, 32, 512, 1024, },
  { 2, 16, 32, 512, 1024, },
  { 1, 2, 16, 32, 512, 1024, },
  { 4, 16, 32, 512, 1024, },
  { 1, 4, 16, 32, 512, 1024, },
  { 2, 4, 16, 32, 512, 1024, },
  { 1, 2, 4, 16, 32, 512, 1024, },
  { 8, 16, 32, 512, 1024, },
  { 1, 8, 16, 32, 512, 1024, },
  { 2, 8, 16, 32, 512, 1024, },
  { 1, 2, 8, 16, 32, 512, 1024, },
  { 4, 8, 16, 32, 512, 1024, },
  { 1, 4, 8, 16, 32, 512, 1024, },
  { 2, 4, 8, 16, 32, 512, 1024, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, },
  { 64, 512, 1024, },
  { 1, 64, 512, 1024, },
  { 2, 64, 512, 1024, },
  { 1, 2, 64, 512, 1024, },
  { 4, 64, 512, 1024, },
  { 1, 4, 64, 512, 1024, },
  { 2, 4, 64, 512, 1024, },
  { 1, 2, 4, 64, 512, 1024, },
  { 8, 64, 512, 1024, },
  { 1, 8, 64, 512, 1024, },
  { 2, 8, 64, 512, 1024, },
  { 1, 2, 8, 64, 512, 1024, },
  { 4, 8, 64, 512, 1024, },
  { 1, 4, 8, 64, 512, 1024, },
  { 2, 4, 8, 64, 512, 1024, },
  { 1, 2, 4, 8, 64, 512, 1024, },
  { 16, 64, 512, 1024, },
  { 1, 16, 64, 512, 1024, },
  { 2, 16, 64, 512, 1024, },
  { 1, 2, 16, 64, 512, 1024, },
  { 4, 16, 64, 512, 1024, },
  { 1, 4, 16, 64, 512, 1024, },
  { 2, 4, 16, 64, 512, 1024, },
  { 1, 2, 4, 16, 64, 512, 1024, },
  { 8, 16, 64, 512, 1024, },
  { 1, 8, 16, 64, 512, 1024, },
  { 2, 8, 16, 64, 512, 1024, },
  { 1, 2, 8, 16, 64, 512, 1024, },
  { 4, 8, 16, 64, 512, 1024, },
  { 1, 4, 8, 16, 64, 512, 1024, },
  { 2, 4, 8, 16, 64, 512, 1024, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, },
  { 32, 64, 512, 1024, },
  { 1, 32, 64, 512, 1024, },
  { 2, 32, 64, 512, 1024, },
  { 1, 2, 32, 64, 512, 1024, },
  { 4, 32, 64, 512, 1024, },
  { 1, 4, 32, 64, 512, 1024, },
  { 2, 4, 32, 64, 512, 1024, },
  { 1, 2, 4, 32, 64, 512, 1024, },
  { 8, 32, 64, 512, 1024, },
  { 1, 8, 32, 64, 512, 1024, },
  { 2, 8, 32, 64, 512, 1024, },
  { 1, 2, 8, 32, 64, 512, 1024, },
  { 4, 8, 32, 64, 512, 1024, },
  { 1, 4, 8, 32, 64, 512, 1024, },
  { 2, 4, 8, 32, 64, 512, 1024, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, },
  { 16, 32, 64, 512, 1024, },
  { 1, 16, 32, 64, 512, 1024, },
  { 2, 16, 32, 64, 512, 1024, },
  { 1, 2, 16, 32, 64, 512, 1024, },
  { 4, 16, 32, 64, 512, 1024, },
  { 1, 4, 16, 32, 64, 512, 1024, },
  { 2, 4, 16, 32, 64, 512, 1024, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, },
  { 8, 16, 32, 64, 512, 1024, },
  { 1, 8, 16, 32, 64, 512, 1024, },
  { 2, 8, 16, 32, 64, 512, 1024, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, },
  { 4, 8, 16, 32, 64, 512, 1024, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, },
  { 128, 512, 1024, },
  { 1, 128, 512, 1024, },
  { 2, 128, 512, 1024, },
  { 1, 2, 128, 512, 1024, },
  { 4, 128, 512, 1024, },
  { 1, 4, 128, 512, 1024, },
  { 2, 4, 128, 512, 1024, },
  { 1, 2, 4, 128, 512, 1024, },
  { 8, 128, 512, 1024, },
  { 1, 8, 128, 512, 1024, },
  { 2, 8, 128, 512, 1024, },
  { 1, 2, 8, 128, 512, 1024, },
  { 4, 8, 128, 512, 1024, },
  { 1, 4, 8, 128, 512, 1024, },
  { 2, 4, 8, 128, 512, 1024, },
  { 1, 2, 4, 8, 128, 512, 1024, },
  { 16, 128, 512, 1024, },
  { 1, 16, 128, 512, 1024, },
  { 2, 16, 128, 512, 1024, },
  { 1, 2, 16, 128, 512, 1024, },
  { 4, 16, 128, 512, 1024, },
  { 1, 4, 16, 128, 512, 1024, },
  { 2, 4, 16, 128, 512, 1024, },
  { 1, 2, 4, 16, 128, 512, 1024, },
  { 8, 16, 128, 512, 1024, },
  { 1, 8, 16, 128, 512, 1024, },
  { 2, 8, 16, 128, 512, 1024, },
  { 1, 2, 8, 16, 128, 512, 1024, },
  { 4, 8, 16, 128, 512, 1024, },
  { 1, 4, 8, 16, 128, 512, 1024, },
  { 2, 4, 8, 16, 128, 512, 1024, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, },
  { 32, 128, 512, 1024, },
  { 1, 32, 128, 512, 1024, },
  { 2, 32, 128, 512, 1024, },
  { 1, 2, 32, 128, 512, 1024, },
  { 4, 32, 128, 512, 1024, },
  { 1, 4, 32, 128, 512, 1024, },
  { 2, 4, 32, 128, 512, 1024, },
  { 1, 2, 4, 32, 128, 512, 1024, },
  { 8, 32, 128, 512, 1024, },
  { 1, 8, 32, 128, 512, 1024, },
  { 2, 8, 32, 128, 512, 1024, },
  { 1, 2, 8, 32, 128, 512, 1024, },
  { 4, 8, 32, 128, 512, 1024, },
  { 1, 4, 8, 32, 128, 512, 1024, },
  { 2, 4, 8, 32, 128, 512, 1024, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, },
  { 16, 32, 128, 512, 1024, },
  { 1, 16, 32, 128, 512, 1024, },
  { 2, 16, 32, 128, 512, 1024, },
  { 1, 2, 16, 32, 128, 512, 1024, },
  { 4, 16, 32, 128, 512, 1024, },
  { 1, 4, 16, 32, 128, 512, 1024, },
  { 2, 4, 16, 32, 128, 512, 1024, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, },
  { 8, 16, 32, 128, 512, 1024, },
  { 1, 8, 16, 32, 128, 512, 1024, },
  { 2, 8, 16, 32, 128, 512, 1024, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, },
  { 4, 8, 16, 32, 128, 512, 1024, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, },
  { 64, 128, 512, 1024, },
  { 1, 64, 128, 512, 1024, },
  { 2, 64, 128, 512, 1024, },
  { 1, 2, 64, 128, 512, 1024, },
  { 4, 64, 128, 512, 1024, },
  { 1, 4, 64, 128, 512, 1024, },
  { 2, 4, 64, 128, 512, 1024, },
  { 1, 2, 4, 64, 128, 512, 1024, },
  { 8, 64, 128, 512, 1024, },
  { 1, 8, 64, 128, 512, 1024, },
  { 2, 8, 64, 128, 512, 1024, },
  { 1, 2, 8, 64, 128, 512, 1024, },
  { 4, 8, 64, 128, 512, 1024, },
  { 1, 4, 8, 64, 128, 512, 1024, },
  { 2, 4, 8, 64, 128, 512, 1024, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, },
  { 16, 64, 128, 512, 1024, },
  { 1, 16, 64, 128, 512, 1024, },
  { 2, 16, 64, 128, 512, 1024, },
  { 1, 2, 16, 64, 128, 512, 1024, },
  { 4, 16, 64, 128, 512, 1024, },
  { 1, 4, 16, 64, 128, 512, 1024, },
  { 2, 4, 16, 64, 128, 512, 1024, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, },
  { 8, 16, 64, 128, 512, 1024, },
  { 1, 8, 16, 64, 128, 512, 1024, },
  { 2, 8, 16, 64, 128, 512, 1024, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, },
  { 4, 8, 16, 64, 128, 512, 1024, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, },
  { 32, 64, 128, 512, 1024, },
  { 1, 32, 64, 128, 512, 1024, },
  { 2, 32, 64, 128, 512, 1024, },
  { 1, 2, 32, 64, 128, 512, 1024, },
  { 4, 32, 64, 128, 512, 1024, },
  { 1, 4, 32, 64, 128, 512, 1024, },
  { 2, 4, 32, 64, 128, 512, 1024, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, },
  { 8, 32, 64, 128, 512, 1024, },
  { 1, 8, 32, 64, 128, 512, 1024, },
  { 2, 8, 32, 64, 128, 512, 1024, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, },
  { 4, 8, 32, 64, 128, 512, 1024, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, },
  { 16, 32, 64, 128, 512, 1024, },
  { 1, 16, 32, 64, 128, 512, 1024, },
  { 2, 16, 32, 64, 128, 512, 1024, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, },
  { 4, 16, 32, 64, 128, 512, 1024, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, },
  { 8, 16, 32, 64, 128, 512, 1024, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, },
  { 256, 512, 1024, },
  { 1, 256, 512, 1024, },
  { 2, 256, 512, 1024, },
  { 1, 2, 256, 512, 1024, },
  { 4, 256, 512, 1024, },
  { 1, 4, 256, 512, 1024, },
  { 2, 4, 256, 512, 1024, },
  { 1, 2, 4, 256, 512, 1024, },
  { 8, 256, 512, 1024, },
  { 1, 8, 256, 512, 1024, },
  { 2, 8, 256, 512, 1024, },
  { 1, 2, 8, 256, 512, 1024, },
  { 4, 8, 256, 512, 1024, },
  { 1, 4, 8, 256, 512, 1024, },
  { 2, 4, 8, 256, 512, 1024, },
  { 1, 2, 4, 8, 256, 512, 1024, },
  { 16, 256, 512, 1024, },
  { 1, 16, 256, 512, 1024, },
  { 2, 16, 256, 512, 1024, },
  { 1, 2, 16, 256, 512, 1024, },
  { 4, 16, 256, 512, 1024, },
  { 1, 4, 16, 256, 512, 1024, },
  { 2, 4, 16, 256, 512, 1024, },
  { 1, 2, 4, 16, 256, 512, 1024, },
  { 8, 16, 256, 512, 1024, },
  { 1, 8, 16, 256, 512, 1024, },
  { 2, 8, 16, 256, 512, 1024, },
  { 1, 2, 8, 16, 256, 512, 1024, },
  { 4, 8, 16, 256, 512, 1024, },
  { 1, 4, 8, 16, 256, 512, 1024, },
  { 2, 4, 8, 16, 256, 512, 1024, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, },
  { 32, 256, 512, 1024, },
  { 1, 32, 256, 512, 1024, },
  { 2, 32, 256, 512, 1024, },
  { 1, 2, 32, 256, 512, 1024, },
  { 4, 32, 256, 512, 1024, },
  { 1, 4, 32, 256, 512, 1024, },
  { 2, 4, 32, 256, 512, 1024, },
  { 1, 2, 4, 32, 256, 512, 1024, },
  { 8, 32, 256, 512, 1024, },
  { 1, 8, 32, 256, 512, 1024, },
  { 2, 8, 32, 256, 512, 1024, },
  { 1, 2, 8, 32, 256, 512, 1024, },
  { 4, 8, 32, 256, 512, 1024, },
  { 1, 4, 8, 32, 256, 512, 1024, },
  { 2, 4, 8, 32, 256, 512, 1024, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, },
  { 16, 32, 256, 512, 1024, },
  { 1, 16, 32, 256, 512, 1024, },
  { 2, 16, 32, 256, 512, 1024, },
  { 1, 2, 16, 32, 256, 512, 1024, },
  { 4, 16, 32, 256, 512, 1024, },
  { 1, 4, 16, 32, 256, 512, 1024, },
  { 2, 4, 16, 32, 256, 512, 1024, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, },
  { 8, 16, 32, 256, 512, 1024, },
  { 1, 8, 16, 32, 256, 512, 1024, },
  { 2, 8, 16, 32, 256, 512, 1024, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, },
  { 4, 8, 16, 32, 256, 512, 1024, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, },
  { 64, 256, 512, 1024, },
  { 1, 64, 256, 512, 1024, },
  { 2, 64, 256, 512, 1024, },
  { 1, 2, 64, 256, 512, 1024, },
  { 4, 64, 256, 512, 1024, },
  { 1, 4, 64, 256, 512, 1024, },
  { 2, 4, 64, 256, 512, 1024, },
  { 1, 2, 4, 64, 256, 512, 1024, },
  { 8, 64, 256, 512, 1024, },
  { 1, 8, 64, 256, 512, 1024, },
  { 2, 8, 64, 256, 512, 1024, },
  { 1, 2, 8, 64, 256, 512, 1024, },
  { 4, 8, 64, 256, 512, 1024, },
  { 1, 4, 8, 64, 256, 512, 1024, },
  { 2, 4, 8, 64, 256, 512, 1024, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, },
  { 16, 64, 256, 512, 1024, },
  { 1, 16, 64, 256, 512, 1024, },
  { 2, 16, 64, 256, 512, 1024, },
  { 1, 2, 16, 64, 256, 512, 1024, },
  { 4, 16, 64, 256, 512, 1024, },
  { 1, 4, 16, 64, 256, 512, 1024, },
  { 2, 4, 16, 64, 256, 512, 1024, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, },
  { 8, 16, 64, 256, 512, 1024, },
  { 1, 8, 16, 64, 256, 512, 1024, },
  { 2, 8, 16, 64, 256, 512, 1024, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, },
  { 4, 8, 16, 64, 256, 512, 1024, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, },
  { 32, 64, 256, 512, 1024, },
  { 1, 32, 64, 256, 512, 1024, },
  { 2, 32, 64, 256, 512, 1024, },
  { 1, 2, 32, 64, 256, 512, 1024, },
  { 4, 32, 64, 256, 512, 1024, },
  { 1, 4, 32, 64, 256, 512, 1024, },
  { 2, 4, 32, 64, 256, 512, 1024, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, },
  { 8, 32, 64, 256, 512, 1024, },
  { 1, 8, 32, 64, 256, 512, 1024, },
  { 2, 8, 32, 64, 256, 512, 1024, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, },
  { 4, 8, 32, 64, 256, 512, 1024, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, },
  { 16, 32, 64, 256, 512, 1024, },
  { 1, 16, 32, 64, 256, 512, 1024, },
  { 2, 16, 32, 64, 256, 512, 1024, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, },
  { 4, 16, 32, 64, 256, 512, 1024, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, },
  { 8, 16, 32, 64, 256, 512, 1024, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, },
  { 128, 256, 512, 1024, },
  { 1, 128, 256, 512, 1024, },
  { 2, 128, 256, 512, 1024, },
  { 1, 2, 128, 256, 512, 1024, },
  { 4, 128, 256, 512, 1024, },
  { 1, 4, 128, 256, 512, 1024, },
  { 2, 4, 128, 256, 512, 1024, },
  { 1, 2, 4, 128, 256, 512, 1024, },
  { 8, 128, 256, 512, 1024, },
  { 1, 8, 128, 256, 512, 1024, },
  { 2, 8, 128, 256, 512, 1024, },
  { 1, 2, 8, 128, 256, 512, 1024, },
  { 4, 8, 128, 256, 512, 1024, },
  { 1, 4, 8, 128, 256, 512, 1024, },
  { 2, 4, 8, 128, 256, 512, 1024, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, },
  { 16, 128, 256, 512, 1024, },
  { 1, 16, 128, 256, 512, 1024, },
  { 2, 16, 128, 256, 512, 1024, },
  { 1, 2, 16, 128, 256, 512, 1024, },
  { 4, 16, 128, 256, 512, 1024, },
  { 1, 4, 16, 128, 256, 512, 1024, },
  { 2, 4, 16, 128, 256, 512, 1024, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, },
  { 8, 16, 128, 256, 512, 1024, },
  { 1, 8, 16, 128, 256, 512, 1024, },
  { 2, 8, 16, 128, 256, 512, 1024, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, },
  { 4, 8, 16, 128, 256, 512, 1024, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, },
  { 32, 128, 256, 512, 1024, },
  { 1, 32, 128, 256, 512, 1024, },
  { 2, 32, 128, 256, 512, 1024, },
  { 1, 2, 32, 128, 256, 512, 1024, },
  { 4, 32, 128, 256, 512, 1024, },
  { 1, 4, 32, 128, 256, 512, 1024, },
  { 2, 4, 32, 128, 256, 512, 1024, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, },
  { 8, 32, 128, 256, 512, 1024, },
  { 1, 8, 32, 128, 256, 512, 1024, },
  { 2, 8, 32, 128, 256, 512, 1024, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, },
  { 4, 8, 32, 128, 256, 512, 1024, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, },
  { 16, 32, 128, 256, 512, 1024, },
  { 1, 16, 32, 128, 256, 512, 1024, },
  { 2, 16, 32, 128, 256, 512, 1024, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, },
  { 4, 16, 32, 128, 256, 512, 1024, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, },
  { 8, 16, 32, 128, 256, 512, 1024, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, },
  { 64, 128, 256, 512, 1024, },
  { 1, 64, 128, 256, 512, 1024, },
  { 2, 64, 128, 256, 512, 1024, },
  { 1, 2, 64, 128, 256, 512, 1024, },
  { 4, 64, 128, 256, 512, 1024, },
  { 1, 4, 64, 128, 256, 512, 1024, },
  { 2, 4, 64, 128, 256, 512, 1024, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, },
  { 8, 64, 128, 256, 512, 1024, },
  { 1, 8, 64, 128, 256, 512, 1024, },
  { 2, 8, 64, 128, 256, 512, 1024, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, },
  { 4, 8, 64, 128, 256, 512, 1024, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, },
  { 16, 64, 128, 256, 512, 1024, },
  { 1, 16, 64, 128, 256, 512, 1024, },
  { 2, 16, 64, 128, 256, 512, 1024, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, },
  { 4, 16, 64, 128, 256, 512, 1024, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, },
  { 8, 16, 64, 128, 256, 512, 1024, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, },
  { 32, 64, 128, 256, 512, 1024, },
  { 1, 32, 64, 128, 256, 512, 1024, },
  { 2, 32, 64, 128, 256, 512, 1024, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, },
  { 4, 32, 64, 128, 256, 512, 1024, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, },
  { 8, 32, 64, 128, 256, 512, 1024, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, },
  { 16, 32, 64, 128, 256, 512, 1024, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, },
  { 2048, },
  { 1, 2048, },
  { 2, 2048, },
  { 1, 2, 2048, },
  { 4, 2048, },
  { 1, 4, 2048, },
  { 2, 4, 2048, },
  { 1, 2, 4, 2048, },
  { 8, 2048, },
  { 1, 8, 2048, },
  { 2, 8, 2048, },
  { 1, 2, 8, 2048, },
  { 4, 8, 2048, },
  { 1, 4, 8, 2048, },
  { 2, 4, 8, 2048, },
  { 1, 2, 4, 8, 2048, },
  { 16, 2048, },
  { 1, 16, 2048, },
  { 2, 16, 2048, },
  { 1, 2, 16, 2048, },
  { 4, 16, 2048, },
  { 1, 4, 16, 2048, },
  { 2, 4, 16, 2048, },
  { 1, 2, 4, 16, 2048, },
  { 8, 16, 2048, },
  { 1, 8, 16, 2048, },
  { 2, 8, 16, 2048, },
  { 1, 2, 8, 16, 2048, },
  { 4, 8, 16, 2048, },
  { 1, 4, 8, 16, 2048, },
  { 2, 4, 8, 16, 2048, },
  { 1, 2, 4, 8, 16, 2048, },
  { 32, 2048, },
  { 1, 32, 2048, },
  { 2, 32, 2048, },
  { 1, 2, 32, 2048, },
  { 4, 32, 2048, },
  { 1, 4, 32, 2048, },
  { 2, 4, 32, 2048, },
  { 1, 2, 4, 32, 2048, },
  { 8, 32, 2048, },
  { 1, 8, 32, 2048, },
  { 2, 8, 32, 2048, },
  { 1, 2, 8, 32, 2048, },
  { 4, 8, 32, 2048, },
  { 1, 4, 8, 32, 2048, },
  { 2, 4, 8, 32, 2048, },
  { 1, 2, 4, 8, 32, 2048, },
  { 16, 32, 2048, },
  { 1, 16, 32, 2048, },
  { 2, 16, 32, 2048, },
  { 1, 2, 16, 32, 2048, },
  { 4, 16, 32, 2048, },
  { 1, 4, 16, 32, 2048, },
  { 2, 4, 16, 32, 2048, },
  { 1, 2, 4, 16, 32, 2048, },
  { 8, 16, 32, 2048, },
  { 1, 8, 16, 32, 2048, },
  { 2, 8, 16, 32, 2048, },
  { 1, 2, 8, 16, 32, 2048, },
  { 4, 8, 16, 32, 2048, },
  { 1, 4, 8, 16, 32, 2048, },
  { 2, 4, 8, 16, 32, 2048, },
  { 1, 2, 4, 8, 16, 32, 2048, },
  { 64, 2048, },
  { 1, 64, 2048, },
  { 2, 64, 2048, },
  { 1, 2, 64, 2048, },
  { 4, 64, 2048, },
  { 1, 4, 64, 2048, },
  { 2, 4, 64, 2048, },
  { 1, 2, 4, 64, 2048, },
  { 8, 64, 2048, },
  { 1, 8, 64, 2048, },
  { 2, 8, 64, 2048, },
  { 1, 2, 8, 64, 2048, },
  { 4, 8, 64, 2048, },
  { 1, 4, 8, 64, 2048, },
  { 2, 4, 8, 64, 2048, },
  { 1, 2, 4, 8, 64, 2048, },
  { 16, 64, 2048, },
  { 1, 16, 64, 2048, },
  { 2, 16, 64, 2048, },
  { 1, 2, 16, 64, 2048, },
  { 4, 16, 64, 2048, },
  { 1, 4, 16, 64, 2048, },
  { 2, 4, 16, 64, 2048, },
  { 1, 2, 4, 16, 64, 2048, },
  { 8, 16, 64, 2048, },
  { 1, 8, 16, 64, 2048, },
  { 2, 8, 16, 64, 2048, },
  { 1, 2, 8, 16, 64, 2048, },
  { 4, 8, 16, 64, 2048, },
  { 1, 4, 8, 16, 64, 2048, },
  { 2, 4, 8, 16, 64, 2048, },
  { 1, 2, 4, 8, 16, 64, 2048, },
  { 32, 64, 2048, },
  { 1, 32, 64, 2048, },
  { 2, 32, 64, 2048, },
  { 1, 2, 32, 64, 2048, },
  { 4, 32, 64, 2048, },
  { 1, 4, 32, 64, 2048, },
  { 2, 4, 32, 64, 2048, },
  { 1, 2, 4, 32, 64, 2048, },
  { 8, 32, 64, 2048, },
  { 1, 8, 32, 64, 2048, },
  { 2, 8, 32, 64, 2048, },
  { 1, 2, 8, 32, 64, 2048, },
  { 4, 8, 32, 64, 2048, },
  { 1, 4, 8, 32, 64, 2048, },
  { 2, 4, 8, 32, 64, 2048, },
  { 1, 2, 4, 8, 32, 64, 2048, },
  { 16, 32, 64, 2048, },
  { 1, 16, 32, 64, 2048, },
  { 2, 16, 32, 64, 2048, },
  { 1, 2, 16, 32, 64, 2048, },
  { 4, 16, 32, 64, 2048, },
  { 1, 4, 16, 32, 64, 2048, },
  { 2, 4, 16, 32, 64, 2048, },
  { 1, 2, 4, 16, 32, 64, 2048, },
  { 8, 16, 32, 64, 2048, },
  { 1, 8, 16, 32, 64, 2048, },
  { 2, 8, 16, 32, 64, 2048, },
  { 1, 2, 8, 16, 32, 64, 2048, },
  { 4, 8, 16, 32, 64, 2048, },
  { 1, 4, 8, 16, 32, 64, 2048, },
  { 2, 4, 8, 16, 32, 64, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, },
  { 128, 2048, },
  { 1, 128, 2048, },
  { 2, 128, 2048, },
  { 1, 2, 128, 2048, },
  { 4, 128, 2048, },
  { 1, 4, 128, 2048, },
  { 2, 4, 128, 2048, },
  { 1, 2, 4, 128, 2048, },
  { 8, 128, 2048, },
  { 1, 8, 128, 2048, },
  { 2, 8, 128, 2048, },
  { 1, 2, 8, 128, 2048, },
  { 4, 8, 128, 2048, },
  { 1, 4, 8, 128, 2048, },
  { 2, 4, 8, 128, 2048, },
  { 1, 2, 4, 8, 128, 2048, },
  { 16, 128, 2048, },
  { 1, 16, 128, 2048, },
  { 2, 16, 128, 2048, },
  { 1, 2, 16, 128, 2048, },
  { 4, 16, 128, 2048, },
  { 1, 4, 16, 128, 2048, },
  { 2, 4, 16, 128, 2048, },
  { 1, 2, 4, 16, 128, 2048, },
  { 8, 16, 128, 2048, },
  { 1, 8, 16, 128, 2048, },
  { 2, 8, 16, 128, 2048, },
  { 1, 2, 8, 16, 128, 2048, },
  { 4, 8, 16, 128, 2048, },
  { 1, 4, 8, 16, 128, 2048, },
  { 2, 4, 8, 16, 128, 2048, },
  { 1, 2, 4, 8, 16, 128, 2048, },
  { 32, 128, 2048, },
  { 1, 32, 128, 2048, },
  { 2, 32, 128, 2048, },
  { 1, 2, 32, 128, 2048, },
  { 4, 32, 128, 2048, },
  { 1, 4, 32, 128, 2048, },
  { 2, 4, 32, 128, 2048, },
  { 1, 2, 4, 32, 128, 2048, },
  { 8, 32, 128, 2048, },
  { 1, 8, 32, 128, 2048, },
  { 2, 8, 32, 128, 2048, },
  { 1, 2, 8, 32, 128, 2048, },
  { 4, 8, 32, 128, 2048, },
  { 1, 4, 8, 32, 128, 2048, },
  { 2, 4, 8, 32, 128, 2048, },
  { 1, 2, 4, 8, 32, 128, 2048, },
  { 16, 32, 128, 2048, },
  { 1, 16, 32, 128, 2048, },
  { 2, 16, 32, 128, 2048, },
  { 1, 2, 16, 32, 128, 2048, },
  { 4, 16, 32, 128, 2048, },
  { 1, 4, 16, 32, 128, 2048, },
  { 2, 4, 16, 32, 128, 2048, },
  { 1, 2, 4, 16, 32, 128, 2048, },
  { 8, 16, 32, 128, 2048, },
  { 1, 8, 16, 32, 128, 2048, },
  { 2, 8, 16, 32, 128, 2048, },
  { 1, 2, 8, 16, 32, 128, 2048, },
  { 4, 8, 16, 32, 128, 2048, },
  { 1, 4, 8, 16, 32, 128, 2048, },
  { 2, 4, 8, 16, 32, 128, 2048, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, },
  { 64, 128, 2048, },
  { 1, 64, 128, 2048, },
  { 2, 64, 128, 2048, },
  { 1, 2, 64, 128, 2048, },
  { 4, 64, 128, 2048, },
  { 1, 4, 64, 128, 2048, },
  { 2, 4, 64, 128, 2048, },
  { 1, 2, 4, 64, 128, 2048, },
  { 8, 64, 128, 2048, },
  { 1, 8, 64, 128, 2048, },
  { 2, 8, 64, 128, 2048, },
  { 1, 2, 8, 64, 128, 2048, },
  { 4, 8, 64, 128, 2048, },
  { 1, 4, 8, 64, 128, 2048, },
  { 2, 4, 8, 64, 128, 2048, },
  { 1, 2, 4, 8, 64, 128, 2048, },
  { 16, 64, 128, 2048, },
  { 1, 16, 64, 128, 2048, },
  { 2, 16, 64, 128, 2048, },
  { 1, 2, 16, 64, 128, 2048, },
  { 4, 16, 64, 128, 2048, },
  { 1, 4, 16, 64, 128, 2048, },
  { 2, 4, 16, 64, 128, 2048, },
  { 1, 2, 4, 16, 64, 128, 2048, },
  { 8, 16, 64, 128, 2048, },
  { 1, 8, 16, 64, 128, 2048, },
  { 2, 8, 16, 64, 128, 2048, },
  { 1, 2, 8, 16, 64, 128, 2048, },
  { 4, 8, 16, 64, 128, 2048, },
  { 1, 4, 8, 16, 64, 128, 2048, },
  { 2, 4, 8, 16, 64, 128, 2048, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, },
  { 32, 64, 128, 2048, },
  { 1, 32, 64, 128, 2048, },
  { 2, 32, 64, 128, 2048, },
  { 1, 2, 32, 64, 128, 2048, },
  { 4, 32, 64, 128, 2048, },
  { 1, 4, 32, 64, 128, 2048, },
  { 2, 4, 32, 64, 128, 2048, },
  { 1, 2, 4, 32, 64, 128, 2048, },
  { 8, 32, 64, 128, 2048, },
  { 1, 8, 32, 64, 128, 2048, },
  { 2, 8, 32, 64, 128, 2048, },
  { 1, 2, 8, 32, 64, 128, 2048, },
  { 4, 8, 32, 64, 128, 2048, },
  { 1, 4, 8, 32, 64, 128, 2048, },
  { 2, 4, 8, 32, 64, 128, 2048, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, },
  { 16, 32, 64, 128, 2048, },
  { 1, 16, 32, 64, 128, 2048, },
  { 2, 16, 32, 64, 128, 2048, },
  { 1, 2, 16, 32, 64, 128, 2048, },
  { 4, 16, 32, 64, 128, 2048, },
  { 1, 4, 16, 32, 64, 128, 2048, },
  { 2, 4, 16, 32, 64, 128, 2048, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, },
  { 8, 16, 32, 64, 128, 2048, },
  { 1, 8, 16, 32, 64, 128, 2048, },
  { 2, 8, 16, 32, 64, 128, 2048, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, },
  { 4, 8, 16, 32, 64, 128, 2048, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, },
  { 256, 2048, },
  { 1, 256, 2048, },
  { 2, 256, 2048, },
  { 1, 2, 256, 2048, },
  { 4, 256, 2048, },
  { 1, 4, 256, 2048, },
  { 2, 4, 256, 2048, },
  { 1, 2, 4, 256, 2048, },
  { 8, 256, 2048, },
  { 1, 8, 256, 2048, },
  { 2, 8, 256, 2048, },
  { 1, 2, 8, 256, 2048, },
  { 4, 8, 256, 2048, },
  { 1, 4, 8, 256, 2048, },
  { 2, 4, 8, 256, 2048, },
  { 1, 2, 4, 8, 256, 2048, },
  { 16, 256, 2048, },
  { 1, 16, 256, 2048, },
  { 2, 16, 256, 2048, },
  { 1, 2, 16, 256, 2048, },
  { 4, 16, 256, 2048, },
  { 1, 4, 16, 256, 2048, },
  { 2, 4, 16, 256, 2048, },
  { 1, 2, 4, 16, 256, 2048, },
  { 8, 16, 256, 2048, },
  { 1, 8, 16, 256, 2048, },
  { 2, 8, 16, 256, 2048, },
  { 1, 2, 8, 16, 256, 2048, },
  { 4, 8, 16, 256, 2048, },
  { 1, 4, 8, 16, 256, 2048, },
  { 2, 4, 8, 16, 256, 2048, },
  { 1, 2, 4, 8, 16, 256, 2048, },
  { 32, 256, 2048, },
  { 1, 32, 256, 2048, },
  { 2, 32, 256, 2048, },
  { 1, 2, 32, 256, 2048, },
  { 4, 32, 256, 2048, },
  { 1, 4, 32, 256, 2048, },
  { 2, 4, 32, 256, 2048, },
  { 1, 2, 4, 32, 256, 2048, },
  { 8, 32, 256, 2048, },
  { 1, 8, 32, 256, 2048, },
  { 2, 8, 32, 256, 2048, },
  { 1, 2, 8, 32, 256, 2048, },
  { 4, 8, 32, 256, 2048, },
  { 1, 4, 8, 32, 256, 2048, },
  { 2, 4, 8, 32, 256, 2048, },
  { 1, 2, 4, 8, 32, 256, 2048, },
  { 16, 32, 256, 2048, },
  { 1, 16, 32, 256, 2048, },
  { 2, 16, 32, 256, 2048, },
  { 1, 2, 16, 32, 256, 2048, },
  { 4, 16, 32, 256, 2048, },
  { 1, 4, 16, 32, 256, 2048, },
  { 2, 4, 16, 32, 256, 2048, },
  { 1, 2, 4, 16, 32, 256, 2048, },
  { 8, 16, 32, 256, 2048, },
  { 1, 8, 16, 32, 256, 2048, },
  { 2, 8, 16, 32, 256, 2048, },
  { 1, 2, 8, 16, 32, 256, 2048, },
  { 4, 8, 16, 32, 256, 2048, },
  { 1, 4, 8, 16, 32, 256, 2048, },
  { 2, 4, 8, 16, 32, 256, 2048, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, },
  { 64, 256, 2048, },
  { 1, 64, 256, 2048, },
  { 2, 64, 256, 2048, },
  { 1, 2, 64, 256, 2048, },
  { 4, 64, 256, 2048, },
  { 1, 4, 64, 256, 2048, },
  { 2, 4, 64, 256, 2048, },
  { 1, 2, 4, 64, 256, 2048, },
  { 8, 64, 256, 2048, },
  { 1, 8, 64, 256, 2048, },
  { 2, 8, 64, 256, 2048, },
  { 1, 2, 8, 64, 256, 2048, },
  { 4, 8, 64, 256, 2048, },
  { 1, 4, 8, 64, 256, 2048, },
  { 2, 4, 8, 64, 256, 2048, },
  { 1, 2, 4, 8, 64, 256, 2048, },
  { 16, 64, 256, 2048, },
  { 1, 16, 64, 256, 2048, },
  { 2, 16, 64, 256, 2048, },
  { 1, 2, 16, 64, 256, 2048, },
  { 4, 16, 64, 256, 2048, },
  { 1, 4, 16, 64, 256, 2048, },
  { 2, 4, 16, 64, 256, 2048, },
  { 1, 2, 4, 16, 64, 256, 2048, },
  { 8, 16, 64, 256, 2048, },
  { 1, 8, 16, 64, 256, 2048, },
  { 2, 8, 16, 64, 256, 2048, },
  { 1, 2, 8, 16, 64, 256, 2048, },
  { 4, 8, 16, 64, 256, 2048, },
  { 1, 4, 8, 16, 64, 256, 2048, },
  { 2, 4, 8, 16, 64, 256, 2048, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, },
  { 32, 64, 256, 2048, },
  { 1, 32, 64, 256, 2048, },
  { 2, 32, 64, 256, 2048, },
  { 1, 2, 32, 64, 256, 2048, },
  { 4, 32, 64, 256, 2048, },
  { 1, 4, 32, 64, 256, 2048, },
  { 2, 4, 32, 64, 256, 2048, },
  { 1, 2, 4, 32, 64, 256, 2048, },
  { 8, 32, 64, 256, 2048, },
  { 1, 8, 32, 64, 256, 2048, },
  { 2, 8, 32, 64, 256, 2048, },
  { 1, 2, 8, 32, 64, 256, 2048, },
  { 4, 8, 32, 64, 256, 2048, },
  { 1, 4, 8, 32, 64, 256, 2048, },
  { 2, 4, 8, 32, 64, 256, 2048, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, },
  { 16, 32, 64, 256, 2048, },
  { 1, 16, 32, 64, 256, 2048, },
  { 2, 16, 32, 64, 256, 2048, },
  { 1, 2, 16, 32, 64, 256, 2048, },
  { 4, 16, 32, 64, 256, 2048, },
  { 1, 4, 16, 32, 64, 256, 2048, },
  { 2, 4, 16, 32, 64, 256, 2048, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, },
  { 8, 16, 32, 64, 256, 2048, },
  { 1, 8, 16, 32, 64, 256, 2048, },
  { 2, 8, 16, 32, 64, 256, 2048, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, },
  { 4, 8, 16, 32, 64, 256, 2048, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, },
  { 128, 256, 2048, },
  { 1, 128, 256, 2048, },
  { 2, 128, 256, 2048, },
  { 1, 2, 128, 256, 2048, },
  { 4, 128, 256, 2048, },
  { 1, 4, 128, 256, 2048, },
  { 2, 4, 128, 256, 2048, },
  { 1, 2, 4, 128, 256, 2048, },
  { 8, 128, 256, 2048, },
  { 1, 8, 128, 256, 2048, },
  { 2, 8, 128, 256, 2048, },
  { 1, 2, 8, 128, 256, 2048, },
  { 4, 8, 128, 256, 2048, },
  { 1, 4, 8, 128, 256, 2048, },
  { 2, 4, 8, 128, 256, 2048, },
  { 1, 2, 4, 8, 128, 256, 2048, },
  { 16, 128, 256, 2048, },
  { 1, 16, 128, 256, 2048, },
  { 2, 16, 128, 256, 2048, },
  { 1, 2, 16, 128, 256, 2048, },
  { 4, 16, 128, 256, 2048, },
  { 1, 4, 16, 128, 256, 2048, },
  { 2, 4, 16, 128, 256, 2048, },
  { 1, 2, 4, 16, 128, 256, 2048, },
  { 8, 16, 128, 256, 2048, },
  { 1, 8, 16, 128, 256, 2048, },
  { 2, 8, 16, 128, 256, 2048, },
  { 1, 2, 8, 16, 128, 256, 2048, },
  { 4, 8, 16, 128, 256, 2048, },
  { 1, 4, 8, 16, 128, 256, 2048, },
  { 2, 4, 8, 16, 128, 256, 2048, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, },
  { 32, 128, 256, 2048, },
  { 1, 32, 128, 256, 2048, },
  { 2, 32, 128, 256, 2048, },
  { 1, 2, 32, 128, 256, 2048, },
  { 4, 32, 128, 256, 2048, },
  { 1, 4, 32, 128, 256, 2048, },
  { 2, 4, 32, 128, 256, 2048, },
  { 1, 2, 4, 32, 128, 256, 2048, },
  { 8, 32, 128, 256, 2048, },
  { 1, 8, 32, 128, 256, 2048, },
  { 2, 8, 32, 128, 256, 2048, },
  { 1, 2, 8, 32, 128, 256, 2048, },
  { 4, 8, 32, 128, 256, 2048, },
  { 1, 4, 8, 32, 128, 256, 2048, },
  { 2, 4, 8, 32, 128, 256, 2048, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, },
  { 16, 32, 128, 256, 2048, },
  { 1, 16, 32, 128, 256, 2048, },
  { 2, 16, 32, 128, 256, 2048, },
  { 1, 2, 16, 32, 128, 256, 2048, },
  { 4, 16, 32, 128, 256, 2048, },
  { 1, 4, 16, 32, 128, 256, 2048, },
  { 2, 4, 16, 32, 128, 256, 2048, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, },
  { 8, 16, 32, 128, 256, 2048, },
  { 1, 8, 16, 32, 128, 256, 2048, },
  { 2, 8, 16, 32, 128, 256, 2048, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, },
  { 4, 8, 16, 32, 128, 256, 2048, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, },
  { 64, 128, 256, 2048, },
  { 1, 64, 128, 256, 2048, },
  { 2, 64, 128, 256, 2048, },
  { 1, 2, 64, 128, 256, 2048, },
  { 4, 64, 128, 256, 2048, },
  { 1, 4, 64, 128, 256, 2048, },
  { 2, 4, 64, 128, 256, 2048, },
  { 1, 2, 4, 64, 128, 256, 2048, },
  { 8, 64, 128, 256, 2048, },
  { 1, 8, 64, 128, 256, 2048, },
  { 2, 8, 64, 128, 256, 2048, },
  { 1, 2, 8, 64, 128, 256, 2048, },
  { 4, 8, 64, 128, 256, 2048, },
  { 1, 4, 8, 64, 128, 256, 2048, },
  { 2, 4, 8, 64, 128, 256, 2048, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, },
  { 16, 64, 128, 256, 2048, },
  { 1, 16, 64, 128, 256, 2048, },
  { 2, 16, 64, 128, 256, 2048, },
  { 1, 2, 16, 64, 128, 256, 2048, },
  { 4, 16, 64, 128, 256, 2048, },
  { 1, 4, 16, 64, 128, 256, 2048, },
  { 2, 4, 16, 64, 128, 256, 2048, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, },
  { 8, 16, 64, 128, 256, 2048, },
  { 1, 8, 16, 64, 128, 256, 2048, },
  { 2, 8, 16, 64, 128, 256, 2048, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, },
  { 4, 8, 16, 64, 128, 256, 2048, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, },
  { 32, 64, 128, 256, 2048, },
  { 1, 32, 64, 128, 256, 2048, },
  { 2, 32, 64, 128, 256, 2048, },
  { 1, 2, 32, 64, 128, 256, 2048, },
  { 4, 32, 64, 128, 256, 2048, },
  { 1, 4, 32, 64, 128, 256, 2048, },
  { 2, 4, 32, 64, 128, 256, 2048, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, },
  { 8, 32, 64, 128, 256, 2048, },
  { 1, 8, 32, 64, 128, 256, 2048, },
  { 2, 8, 32, 64, 128, 256, 2048, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, },
  { 4, 8, 32, 64, 128, 256, 2048, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, },
  { 16, 32, 64, 128, 256, 2048, },
  { 1, 16, 32, 64, 128, 256, 2048, },
  { 2, 16, 32, 64, 128, 256, 2048, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, },
  { 4, 16, 32, 64, 128, 256, 2048, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, },
  { 8, 16, 32, 64, 128, 256, 2048, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, },
  { 512, 2048, },
  { 1, 512, 2048, },
  { 2, 512, 2048, },
  { 1, 2, 512, 2048, },
  { 4, 512, 2048, },
  { 1, 4, 512, 2048, },
  { 2, 4, 512, 2048, },
  { 1, 2, 4, 512, 2048, },
  { 8, 512, 2048, },
  { 1, 8, 512, 2048, },
  { 2, 8, 512, 2048, },
  { 1, 2, 8, 512, 2048, },
  { 4, 8, 512, 2048, },
  { 1, 4, 8, 512, 2048, },
  { 2, 4, 8, 512, 2048, },
  { 1, 2, 4, 8, 512, 2048, },
  { 16, 512, 2048, },
  { 1, 16, 512, 2048, },
  { 2, 16, 512, 2048, },
  { 1, 2, 16, 512, 2048, },
  { 4, 16, 512, 2048, },
  { 1, 4, 16, 512, 2048, },
  { 2, 4, 16, 512, 2048, },
  { 1, 2, 4, 16, 512, 2048, },
  { 8, 16, 512, 2048, },
  { 1, 8, 16, 512, 2048, },
  { 2, 8, 16, 512, 2048, },
  { 1, 2, 8, 16, 512, 2048, },
  { 4, 8, 16, 512, 2048, },
  { 1, 4, 8, 16, 512, 2048, },
  { 2, 4, 8, 16, 512, 2048, },
  { 1, 2, 4, 8, 16, 512, 2048, },
  { 32, 512, 2048, },
  { 1, 32, 512, 2048, },
  { 2, 32, 512, 2048, },
  { 1, 2, 32, 512, 2048, },
  { 4, 32, 512, 2048, },
  { 1, 4, 32, 512, 2048, },
  { 2, 4, 32, 512, 2048, },
  { 1, 2, 4, 32, 512, 2048, },
  { 8, 32, 512, 2048, },
  { 1, 8, 32, 512, 2048, },
  { 2, 8, 32, 512, 2048, },
  { 1, 2, 8, 32, 512, 2048, },
  { 4, 8, 32, 512, 2048, },
  { 1, 4, 8, 32, 512, 2048, },
  { 2, 4, 8, 32, 512, 2048, },
  { 1, 2, 4, 8, 32, 512, 2048, },
  { 16, 32, 512, 2048, },
  { 1, 16, 32, 512, 2048, },
  { 2, 16, 32, 512, 2048, },
  { 1, 2, 16, 32, 512, 2048, },
  { 4, 16, 32, 512, 2048, },
  { 1, 4, 16, 32, 512, 2048, },
  { 2, 4, 16, 32, 512, 2048, },
  { 1, 2, 4, 16, 32, 512, 2048, },
  { 8, 16, 32, 512, 2048, },
  { 1, 8, 16, 32, 512, 2048, },
  { 2, 8, 16, 32, 512, 2048, },
  { 1, 2, 8, 16, 32, 512, 2048, },
  { 4, 8, 16, 32, 512, 2048, },
  { 1, 4, 8, 16, 32, 512, 2048, },
  { 2, 4, 8, 16, 32, 512, 2048, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, },
  { 64, 512, 2048, },
  { 1, 64, 512, 2048, },
  { 2, 64, 512, 2048, },
  { 1, 2, 64, 512, 2048, },
  { 4, 64, 512, 2048, },
  { 1, 4, 64, 512, 2048, },
  { 2, 4, 64, 512, 2048, },
  { 1, 2, 4, 64, 512, 2048, },
  { 8, 64, 512, 2048, },
  { 1, 8, 64, 512, 2048, },
  { 2, 8, 64, 512, 2048, },
  { 1, 2, 8, 64, 512, 2048, },
  { 4, 8, 64, 512, 2048, },
  { 1, 4, 8, 64, 512, 2048, },
  { 2, 4, 8, 64, 512, 2048, },
  { 1, 2, 4, 8, 64, 512, 2048, },
  { 16, 64, 512, 2048, },
  { 1, 16, 64, 512, 2048, },
  { 2, 16, 64, 512, 2048, },
  { 1, 2, 16, 64, 512, 2048, },
  { 4, 16, 64, 512, 2048, },
  { 1, 4, 16, 64, 512, 2048, },
  { 2, 4, 16, 64, 512, 2048, },
  { 1, 2, 4, 16, 64, 512, 2048, },
  { 8, 16, 64, 512, 2048, },
  { 1, 8, 16, 64, 512, 2048, },
  { 2, 8, 16, 64, 512, 2048, },
  { 1, 2, 8, 16, 64, 512, 2048, },
  { 4, 8, 16, 64, 512, 2048, },
  { 1, 4, 8, 16, 64, 512, 2048, },
  { 2, 4, 8, 16, 64, 512, 2048, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, },
  { 32, 64, 512, 2048, },
  { 1, 32, 64, 512, 2048, },
  { 2, 32, 64, 512, 2048, },
  { 1, 2, 32, 64, 512, 2048, },
  { 4, 32, 64, 512, 2048, },
  { 1, 4, 32, 64, 512, 2048, },
  { 2, 4, 32, 64, 512, 2048, },
  { 1, 2, 4, 32, 64, 512, 2048, },
  { 8, 32, 64, 512, 2048, },
  { 1, 8, 32, 64, 512, 2048, },
  { 2, 8, 32, 64, 512, 2048, },
  { 1, 2, 8, 32, 64, 512, 2048, },
  { 4, 8, 32, 64, 512, 2048, },
  { 1, 4, 8, 32, 64, 512, 2048, },
  { 2, 4, 8, 32, 64, 512, 2048, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, },
  { 16, 32, 64, 512, 2048, },
  { 1, 16, 32, 64, 512, 2048, },
  { 2, 16, 32, 64, 512, 2048, },
  { 1, 2, 16, 32, 64, 512, 2048, },
  { 4, 16, 32, 64, 512, 2048, },
  { 1, 4, 16, 32, 64, 512, 2048, },
  { 2, 4, 16, 32, 64, 512, 2048, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, },
  { 8, 16, 32, 64, 512, 2048, },
  { 1, 8, 16, 32, 64, 512, 2048, },
  { 2, 8, 16, 32, 64, 512, 2048, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, },
  { 4, 8, 16, 32, 64, 512, 2048, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, },
  { 128, 512, 2048, },
  { 1, 128, 512, 2048, },
  { 2, 128, 512, 2048, },
  { 1, 2, 128, 512, 2048, },
  { 4, 128, 512, 2048, },
  { 1, 4, 128, 512, 2048, },
  { 2, 4, 128, 512, 2048, },
  { 1, 2, 4, 128, 512, 2048, },
  { 8, 128, 512, 2048, },
  { 1, 8, 128, 512, 2048, },
  { 2, 8, 128, 512, 2048, },
  { 1, 2, 8, 128, 512, 2048, },
  { 4, 8, 128, 512, 2048, },
  { 1, 4, 8, 128, 512, 2048, },
  { 2, 4, 8, 128, 512, 2048, },
  { 1, 2, 4, 8, 128, 512, 2048, },
  { 16, 128, 512, 2048, },
  { 1, 16, 128, 512, 2048, },
  { 2, 16, 128, 512, 2048, },
  { 1, 2, 16, 128, 512, 2048, },
  { 4, 16, 128, 512, 2048, },
  { 1, 4, 16, 128, 512, 2048, },
  { 2, 4, 16, 128, 512, 2048, },
  { 1, 2, 4, 16, 128, 512, 2048, },
  { 8, 16, 128, 512, 2048, },
  { 1, 8, 16, 128, 512, 2048, },
  { 2, 8, 16, 128, 512, 2048, },
  { 1, 2, 8, 16, 128, 512, 2048, },
  { 4, 8, 16, 128, 512, 2048, },
  { 1, 4, 8, 16, 128, 512, 2048, },
  { 2, 4, 8, 16, 128, 512, 2048, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, },
  { 32, 128, 512, 2048, },
  { 1, 32, 128, 512, 2048, },
  { 2, 32, 128, 512, 2048, },
  { 1, 2, 32, 128, 512, 2048, },
  { 4, 32, 128, 512, 2048, },
  { 1, 4, 32, 128, 512, 2048, },
  { 2, 4, 32, 128, 512, 2048, },
  { 1, 2, 4, 32, 128, 512, 2048, },
  { 8, 32, 128, 512, 2048, },
  { 1, 8, 32, 128, 512, 2048, },
  { 2, 8, 32, 128, 512, 2048, },
  { 1, 2, 8, 32, 128, 512, 2048, },
  { 4, 8, 32, 128, 512, 2048, },
  { 1, 4, 8, 32, 128, 512, 2048, },
  { 2, 4, 8, 32, 128, 512, 2048, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, },
  { 16, 32, 128, 512, 2048, },
  { 1, 16, 32, 128, 512, 2048, },
  { 2, 16, 32, 128, 512, 2048, },
  { 1, 2, 16, 32, 128, 512, 2048, },
  { 4, 16, 32, 128, 512, 2048, },
  { 1, 4, 16, 32, 128, 512, 2048, },
  { 2, 4, 16, 32, 128, 512, 2048, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, },
  { 8, 16, 32, 128, 512, 2048, },
  { 1, 8, 16, 32, 128, 512, 2048, },
  { 2, 8, 16, 32, 128, 512, 2048, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, },
  { 4, 8, 16, 32, 128, 512, 2048, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, },
  { 64, 128, 512, 2048, },
  { 1, 64, 128, 512, 2048, },
  { 2, 64, 128, 512, 2048, },
  { 1, 2, 64, 128, 512, 2048, },
  { 4, 64, 128, 512, 2048, },
  { 1, 4, 64, 128, 512, 2048, },
  { 2, 4, 64, 128, 512, 2048, },
  { 1, 2, 4, 64, 128, 512, 2048, },
  { 8, 64, 128, 512, 2048, },
  { 1, 8, 64, 128, 512, 2048, },
  { 2, 8, 64, 128, 512, 2048, },
  { 1, 2, 8, 64, 128, 512, 2048, },
  { 4, 8, 64, 128, 512, 2048, },
  { 1, 4, 8, 64, 128, 512, 2048, },
  { 2, 4, 8, 64, 128, 512, 2048, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, },
  { 16, 64, 128, 512, 2048, },
  { 1, 16, 64, 128, 512, 2048, },
  { 2, 16, 64, 128, 512, 2048, },
  { 1, 2, 16, 64, 128, 512, 2048, },
  { 4, 16, 64, 128, 512, 2048, },
  { 1, 4, 16, 64, 128, 512, 2048, },
  { 2, 4, 16, 64, 128, 512, 2048, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, },
  { 8, 16, 64, 128, 512, 2048, },
  { 1, 8, 16, 64, 128, 512, 2048, },
  { 2, 8, 16, 64, 128, 512, 2048, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, },
  { 4, 8, 16, 64, 128, 512, 2048, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, },
  { 32, 64, 128, 512, 2048, },
  { 1, 32, 64, 128, 512, 2048, },
  { 2, 32, 64, 128, 512, 2048, },
  { 1, 2, 32, 64, 128, 512, 2048, },
  { 4, 32, 64, 128, 512, 2048, },
  { 1, 4, 32, 64, 128, 512, 2048, },
  { 2, 4, 32, 64, 128, 512, 2048, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, },
  { 8, 32, 64, 128, 512, 2048, },
  { 1, 8, 32, 64, 128, 512, 2048, },
  { 2, 8, 32, 64, 128, 512, 2048, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, },
  { 4, 8, 32, 64, 128, 512, 2048, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, },
  { 16, 32, 64, 128, 512, 2048, },
  { 1, 16, 32, 64, 128, 512, 2048, },
  { 2, 16, 32, 64, 128, 512, 2048, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, },
  { 4, 16, 32, 64, 128, 512, 2048, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, },
  { 8, 16, 32, 64, 128, 512, 2048, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, },
  { 256, 512, 2048, },
  { 1, 256, 512, 2048, },
  { 2, 256, 512, 2048, },
  { 1, 2, 256, 512, 2048, },
  { 4, 256, 512, 2048, },
  { 1, 4, 256, 512, 2048, },
  { 2, 4, 256, 512, 2048, },
  { 1, 2, 4, 256, 512, 2048, },
  { 8, 256, 512, 2048, },
  { 1, 8, 256, 512, 2048, },
  { 2, 8, 256, 512, 2048, },
  { 1, 2, 8, 256, 512, 2048, },
  { 4, 8, 256, 512, 2048, },
  { 1, 4, 8, 256, 512, 2048, },
  { 2, 4, 8, 256, 512, 2048, },
  { 1, 2, 4, 8, 256, 512, 2048, },
  { 16, 256, 512, 2048, },
  { 1, 16, 256, 512, 2048, },
  { 2, 16, 256, 512, 2048, },
  { 1, 2, 16, 256, 512, 2048, },
  { 4, 16, 256, 512, 2048, },
  { 1, 4, 16, 256, 512, 2048, },
  { 2, 4, 16, 256, 512, 2048, },
  { 1, 2, 4, 16, 256, 512, 2048, },
  { 8, 16, 256, 512, 2048, },
  { 1, 8, 16, 256, 512, 2048, },
  { 2, 8, 16, 256, 512, 2048, },
  { 1, 2, 8, 16, 256, 512, 2048, },
  { 4, 8, 16, 256, 512, 2048, },
  { 1, 4, 8, 16, 256, 512, 2048, },
  { 2, 4, 8, 16, 256, 512, 2048, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, },
  { 32, 256, 512, 2048, },
  { 1, 32, 256, 512, 2048, },
  { 2, 32, 256, 512, 2048, },
  { 1, 2, 32, 256, 512, 2048, },
  { 4, 32, 256, 512, 2048, },
  { 1, 4, 32, 256, 512, 2048, },
  { 2, 4, 32, 256, 512, 2048, },
  { 1, 2, 4, 32, 256, 512, 2048, },
  { 8, 32, 256, 512, 2048, },
  { 1, 8, 32, 256, 512, 2048, },
  { 2, 8, 32, 256, 512, 2048, },
  { 1, 2, 8, 32, 256, 512, 2048, },
  { 4, 8, 32, 256, 512, 2048, },
  { 1, 4, 8, 32, 256, 512, 2048, },
  { 2, 4, 8, 32, 256, 512, 2048, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, },
  { 16, 32, 256, 512, 2048, },
  { 1, 16, 32, 256, 512, 2048, },
  { 2, 16, 32, 256, 512, 2048, },
  { 1, 2, 16, 32, 256, 512, 2048, },
  { 4, 16, 32, 256, 512, 2048, },
  { 1, 4, 16, 32, 256, 512, 2048, },
  { 2, 4, 16, 32, 256, 512, 2048, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, },
  { 8, 16, 32, 256, 512, 2048, },
  { 1, 8, 16, 32, 256, 512, 2048, },
  { 2, 8, 16, 32, 256, 512, 2048, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, },
  { 4, 8, 16, 32, 256, 512, 2048, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, },
  { 64, 256, 512, 2048, },
  { 1, 64, 256, 512, 2048, },
  { 2, 64, 256, 512, 2048, },
  { 1, 2, 64, 256, 512, 2048, },
  { 4, 64, 256, 512, 2048, },
  { 1, 4, 64, 256, 512, 2048, },
  { 2, 4, 64, 256, 512, 2048, },
  { 1, 2, 4, 64, 256, 512, 2048, },
  { 8, 64, 256, 512, 2048, },
  { 1, 8, 64, 256, 512, 2048, },
  { 2, 8, 64, 256, 512, 2048, },
  { 1, 2, 8, 64, 256, 512, 2048, },
  { 4, 8, 64, 256, 512, 2048, },
  { 1, 4, 8, 64, 256, 512, 2048, },
  { 2, 4, 8, 64, 256, 512, 2048, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, },
  { 16, 64, 256, 512, 2048, },
  { 1, 16, 64, 256, 512, 2048, },
  { 2, 16, 64, 256, 512, 2048, },
  { 1, 2, 16, 64, 256, 512, 2048, },
  { 4, 16, 64, 256, 512, 2048, },
  { 1, 4, 16, 64, 256, 512, 2048, },
  { 2, 4, 16, 64, 256, 512, 2048, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, },
  { 8, 16, 64, 256, 512, 2048, },
  { 1, 8, 16, 64, 256, 512, 2048, },
  { 2, 8, 16, 64, 256, 512, 2048, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, },
  { 4, 8, 16, 64, 256, 512, 2048, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, },
  { 32, 64, 256, 512, 2048, },
  { 1, 32, 64, 256, 512, 2048, },
  { 2, 32, 64, 256, 512, 2048, },
  { 1, 2, 32, 64, 256, 512, 2048, },
  { 4, 32, 64, 256, 512, 2048, },
  { 1, 4, 32, 64, 256, 512, 2048, },
  { 2, 4, 32, 64, 256, 512, 2048, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, },
  { 8, 32, 64, 256, 512, 2048, },
  { 1, 8, 32, 64, 256, 512, 2048, },
  { 2, 8, 32, 64, 256, 512, 2048, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, },
  { 4, 8, 32, 64, 256, 512, 2048, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, },
  { 16, 32, 64, 256, 512, 2048, },
  { 1, 16, 32, 64, 256, 512, 2048, },
  { 2, 16, 32, 64, 256, 512, 2048, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, },
  { 4, 16, 32, 64, 256, 512, 2048, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, },
  { 8, 16, 32, 64, 256, 512, 2048, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, },
  { 128, 256, 512, 2048, },
  { 1, 128, 256, 512, 2048, },
  { 2, 128, 256, 512, 2048, },
  { 1, 2, 128, 256, 512, 2048, },
  { 4, 128, 256, 512, 2048, },
  { 1, 4, 128, 256, 512, 2048, },
  { 2, 4, 128, 256, 512, 2048, },
  { 1, 2, 4, 128, 256, 512, 2048, },
  { 8, 128, 256, 512, 2048, },
  { 1, 8, 128, 256, 512, 2048, },
  { 2, 8, 128, 256, 512, 2048, },
  { 1, 2, 8, 128, 256, 512, 2048, },
  { 4, 8, 128, 256, 512, 2048, },
  { 1, 4, 8, 128, 256, 512, 2048, },
  { 2, 4, 8, 128, 256, 512, 2048, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, },
  { 16, 128, 256, 512, 2048, },
  { 1, 16, 128, 256, 512, 2048, },
  { 2, 16, 128, 256, 512, 2048, },
  { 1, 2, 16, 128, 256, 512, 2048, },
  { 4, 16, 128, 256, 512, 2048, },
  { 1, 4, 16, 128, 256, 512, 2048, },
  { 2, 4, 16, 128, 256, 512, 2048, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, },
  { 8, 16, 128, 256, 512, 2048, },
  { 1, 8, 16, 128, 256, 512, 2048, },
  { 2, 8, 16, 128, 256, 512, 2048, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, },
  { 4, 8, 16, 128, 256, 512, 2048, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, },
  { 32, 128, 256, 512, 2048, },
  { 1, 32, 128, 256, 512, 2048, },
  { 2, 32, 128, 256, 512, 2048, },
  { 1, 2, 32, 128, 256, 512, 2048, },
  { 4, 32, 128, 256, 512, 2048, },
  { 1, 4, 32, 128, 256, 512, 2048, },
  { 2, 4, 32, 128, 256, 512, 2048, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, },
  { 8, 32, 128, 256, 512, 2048, },
  { 1, 8, 32, 128, 256, 512, 2048, },
  { 2, 8, 32, 128, 256, 512, 2048, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, },
  { 4, 8, 32, 128, 256, 512, 2048, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, },
  { 16, 32, 128, 256, 512, 2048, },
  { 1, 16, 32, 128, 256, 512, 2048, },
  { 2, 16, 32, 128, 256, 512, 2048, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, },
  { 4, 16, 32, 128, 256, 512, 2048, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, },
  { 8, 16, 32, 128, 256, 512, 2048, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, },
  { 64, 128, 256, 512, 2048, },
  { 1, 64, 128, 256, 512, 2048, },
  { 2, 64, 128, 256, 512, 2048, },
  { 1, 2, 64, 128, 256, 512, 2048, },
  { 4, 64, 128, 256, 512, 2048, },
  { 1, 4, 64, 128, 256, 512, 2048, },
  { 2, 4, 64, 128, 256, 512, 2048, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, },
  { 8, 64, 128, 256, 512, 2048, },
  { 1, 8, 64, 128, 256, 512, 2048, },
  { 2, 8, 64, 128, 256, 512, 2048, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, },
  { 4, 8, 64, 128, 256, 512, 2048, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, },
  { 16, 64, 128, 256, 512, 2048, },
  { 1, 16, 64, 128, 256, 512, 2048, },
  { 2, 16, 64, 128, 256, 512, 2048, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, },
  { 4, 16, 64, 128, 256, 512, 2048, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, },
  { 8, 16, 64, 128, 256, 512, 2048, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, },
  { 32, 64, 128, 256, 512, 2048, },
  { 1, 32, 64, 128, 256, 512, 2048, },
  { 2, 32, 64, 128, 256, 512, 2048, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, },
  { 4, 32, 64, 128, 256, 512, 2048, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, },
  { 8, 32, 64, 128, 256, 512, 2048, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, },
  { 16, 32, 64, 128, 256, 512, 2048, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, },
  { 1024, 2048, },
  { 1, 1024, 2048, },
  { 2, 1024, 2048, },
  { 1, 2, 1024, 2048, },
  { 4, 1024, 2048, },
  { 1, 4, 1024, 2048, },
  { 2, 4, 1024, 2048, },
  { 1, 2, 4, 1024, 2048, },
  { 8, 1024, 2048, },
  { 1, 8, 1024, 2048, },
  { 2, 8, 1024, 2048, },
  { 1, 2, 8, 1024, 2048, },
  { 4, 8, 1024, 2048, },
  { 1, 4, 8, 1024, 2048, },
  { 2, 4, 8, 1024, 2048, },
  { 1, 2, 4, 8, 1024, 2048, },
  { 16, 1024, 2048, },
  { 1, 16, 1024, 2048, },
  { 2, 16, 1024, 2048, },
  { 1, 2, 16, 1024, 2048, },
  { 4, 16, 1024, 2048, },
  { 1, 4, 16, 1024, 2048, },
  { 2, 4, 16, 1024, 2048, },
  { 1, 2, 4, 16, 1024, 2048, },
  { 8, 16, 1024, 2048, },
  { 1, 8, 16, 1024, 2048, },
  { 2, 8, 16, 1024, 2048, },
  { 1, 2, 8, 16, 1024, 2048, },
  { 4, 8, 16, 1024, 2048, },
  { 1, 4, 8, 16, 1024, 2048, },
  { 2, 4, 8, 16, 1024, 2048, },
  { 1, 2, 4, 8, 16, 1024, 2048, },
  { 32, 1024, 2048, },
  { 1, 32, 1024, 2048, },
  { 2, 32, 1024, 2048, },
  { 1, 2, 32, 1024, 2048, },
  { 4, 32, 1024, 2048, },
  { 1, 4, 32, 1024, 2048, },
  { 2, 4, 32, 1024, 2048, },
  { 1, 2, 4, 32, 1024, 2048, },
  { 8, 32, 1024, 2048, },
  { 1, 8, 32, 1024, 2048, },
  { 2, 8, 32, 1024, 2048, },
  { 1, 2, 8, 32, 1024, 2048, },
  { 4, 8, 32, 1024, 2048, },
  { 1, 4, 8, 32, 1024, 2048, },
  { 2, 4, 8, 32, 1024, 2048, },
  { 1, 2, 4, 8, 32, 1024, 2048, },
  { 16, 32, 1024, 2048, },
  { 1, 16, 32, 1024, 2048, },
  { 2, 16, 32, 1024, 2048, },
  { 1, 2, 16, 32, 1024, 2048, },
  { 4, 16, 32, 1024, 2048, },
  { 1, 4, 16, 32, 1024, 2048, },
  { 2, 4, 16, 32, 1024, 2048, },
  { 1, 2, 4, 16, 32, 1024, 2048, },
  { 8, 16, 32, 1024, 2048, },
  { 1, 8, 16, 32, 1024, 2048, },
  { 2, 8, 16, 32, 1024, 2048, },
  { 1, 2, 8, 16, 32, 1024, 2048, },
  { 4, 8, 16, 32, 1024, 2048, },
  { 1, 4, 8, 16, 32, 1024, 2048, },
  { 2, 4, 8, 16, 32, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, },
  { 64, 1024, 2048, },
  { 1, 64, 1024, 2048, },
  { 2, 64, 1024, 2048, },
  { 1, 2, 64, 1024, 2048, },
  { 4, 64, 1024, 2048, },
  { 1, 4, 64, 1024, 2048, },
  { 2, 4, 64, 1024, 2048, },
  { 1, 2, 4, 64, 1024, 2048, },
  { 8, 64, 1024, 2048, },
  { 1, 8, 64, 1024, 2048, },
  { 2, 8, 64, 1024, 2048, },
  { 1, 2, 8, 64, 1024, 2048, },
  { 4, 8, 64, 1024, 2048, },
  { 1, 4, 8, 64, 1024, 2048, },
  { 2, 4, 8, 64, 1024, 2048, },
  { 1, 2, 4, 8, 64, 1024, 2048, },
  { 16, 64, 1024, 2048, },
  { 1, 16, 64, 1024, 2048, },
  { 2, 16, 64, 1024, 2048, },
  { 1, 2, 16, 64, 1024, 2048, },
  { 4, 16, 64, 1024, 2048, },
  { 1, 4, 16, 64, 1024, 2048, },
  { 2, 4, 16, 64, 1024, 2048, },
  { 1, 2, 4, 16, 64, 1024, 2048, },
  { 8, 16, 64, 1024, 2048, },
  { 1, 8, 16, 64, 1024, 2048, },
  { 2, 8, 16, 64, 1024, 2048, },
  { 1, 2, 8, 16, 64, 1024, 2048, },
  { 4, 8, 16, 64, 1024, 2048, },
  { 1, 4, 8, 16, 64, 1024, 2048, },
  { 2, 4, 8, 16, 64, 1024, 2048, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, },
  { 32, 64, 1024, 2048, },
  { 1, 32, 64, 1024, 2048, },
  { 2, 32, 64, 1024, 2048, },
  { 1, 2, 32, 64, 1024, 2048, },
  { 4, 32, 64, 1024, 2048, },
  { 1, 4, 32, 64, 1024, 2048, },
  { 2, 4, 32, 64, 1024, 2048, },
  { 1, 2, 4, 32, 64, 1024, 2048, },
  { 8, 32, 64, 1024, 2048, },
  { 1, 8, 32, 64, 1024, 2048, },
  { 2, 8, 32, 64, 1024, 2048, },
  { 1, 2, 8, 32, 64, 1024, 2048, },
  { 4, 8, 32, 64, 1024, 2048, },
  { 1, 4, 8, 32, 64, 1024, 2048, },
  { 2, 4, 8, 32, 64, 1024, 2048, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, },
  { 16, 32, 64, 1024, 2048, },
  { 1, 16, 32, 64, 1024, 2048, },
  { 2, 16, 32, 64, 1024, 2048, },
  { 1, 2, 16, 32, 64, 1024, 2048, },
  { 4, 16, 32, 64, 1024, 2048, },
  { 1, 4, 16, 32, 64, 1024, 2048, },
  { 2, 4, 16, 32, 64, 1024, 2048, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, },
  { 8, 16, 32, 64, 1024, 2048, },
  { 1, 8, 16, 32, 64, 1024, 2048, },
  { 2, 8, 16, 32, 64, 1024, 2048, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, },
  { 4, 8, 16, 32, 64, 1024, 2048, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, },
  { 128, 1024, 2048, },
  { 1, 128, 1024, 2048, },
  { 2, 128, 1024, 2048, },
  { 1, 2, 128, 1024, 2048, },
  { 4, 128, 1024, 2048, },
  { 1, 4, 128, 1024, 2048, },
  { 2, 4, 128, 1024, 2048, },
  { 1, 2, 4, 128, 1024, 2048, },
  { 8, 128, 1024, 2048, },
  { 1, 8, 128, 1024, 2048, },
  { 2, 8, 128, 1024, 2048, },
  { 1, 2, 8, 128, 1024, 2048, },
  { 4, 8, 128, 1024, 2048, },
  { 1, 4, 8, 128, 1024, 2048, },
  { 2, 4, 8, 128, 1024, 2048, },
  { 1, 2, 4, 8, 128, 1024, 2048, },
  { 16, 128, 1024, 2048, },
  { 1, 16, 128, 1024, 2048, },
  { 2, 16, 128, 1024, 2048, },
  { 1, 2, 16, 128, 1024, 2048, },
  { 4, 16, 128, 1024, 2048, },
  { 1, 4, 16, 128, 1024, 2048, },
  { 2, 4, 16, 128, 1024, 2048, },
  { 1, 2, 4, 16, 128, 1024, 2048, },
  { 8, 16, 128, 1024, 2048, },
  { 1, 8, 16, 128, 1024, 2048, },
  { 2, 8, 16, 128, 1024, 2048, },
  { 1, 2, 8, 16, 128, 1024, 2048, },
  { 4, 8, 16, 128, 1024, 2048, },
  { 1, 4, 8, 16, 128, 1024, 2048, },
  { 2, 4, 8, 16, 128, 1024, 2048, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, },
  { 32, 128, 1024, 2048, },
  { 1, 32, 128, 1024, 2048, },
  { 2, 32, 128, 1024, 2048, },
  { 1, 2, 32, 128, 1024, 2048, },
  { 4, 32, 128, 1024, 2048, },
  { 1, 4, 32, 128, 1024, 2048, },
  { 2, 4, 32, 128, 1024, 2048, },
  { 1, 2, 4, 32, 128, 1024, 2048, },
  { 8, 32, 128, 1024, 2048, },
  { 1, 8, 32, 128, 1024, 2048, },
  { 2, 8, 32, 128, 1024, 2048, },
  { 1, 2, 8, 32, 128, 1024, 2048, },
  { 4, 8, 32, 128, 1024, 2048, },
  { 1, 4, 8, 32, 128, 1024, 2048, },
  { 2, 4, 8, 32, 128, 1024, 2048, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, },
  { 16, 32, 128, 1024, 2048, },
  { 1, 16, 32, 128, 1024, 2048, },
  { 2, 16, 32, 128, 1024, 2048, },
  { 1, 2, 16, 32, 128, 1024, 2048, },
  { 4, 16, 32, 128, 1024, 2048, },
  { 1, 4, 16, 32, 128, 1024, 2048, },
  { 2, 4, 16, 32, 128, 1024, 2048, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, },
  { 8, 16, 32, 128, 1024, 2048, },
  { 1, 8, 16, 32, 128, 1024, 2048, },
  { 2, 8, 16, 32, 128, 1024, 2048, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, },
  { 4, 8, 16, 32, 128, 1024, 2048, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, },
  { 64, 128, 1024, 2048, },
  { 1, 64, 128, 1024, 2048, },
  { 2, 64, 128, 1024, 2048, },
  { 1, 2, 64, 128, 1024, 2048, },
  { 4, 64, 128, 1024, 2048, },
  { 1, 4, 64, 128, 1024, 2048, },
  { 2, 4, 64, 128, 1024, 2048, },
  { 1, 2, 4, 64, 128, 1024, 2048, },
  { 8, 64, 128, 1024, 2048, },
  { 1, 8, 64, 128, 1024, 2048, },
  { 2, 8, 64, 128, 1024, 2048, },
  { 1, 2, 8, 64, 128, 1024, 2048, },
  { 4, 8, 64, 128, 1024, 2048, },
  { 1, 4, 8, 64, 128, 1024, 2048, },
  { 2, 4, 8, 64, 128, 1024, 2048, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, },
  { 16, 64, 128, 1024, 2048, },
  { 1, 16, 64, 128, 1024, 2048, },
  { 2, 16, 64, 128, 1024, 2048, },
  { 1, 2, 16, 64, 128, 1024, 2048, },
  { 4, 16, 64, 128, 1024, 2048, },
  { 1, 4, 16, 64, 128, 1024, 2048, },
  { 2, 4, 16, 64, 128, 1024, 2048, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, },
  { 8, 16, 64, 128, 1024, 2048, },
  { 1, 8, 16, 64, 128, 1024, 2048, },
  { 2, 8, 16, 64, 128, 1024, 2048, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, },
  { 4, 8, 16, 64, 128, 1024, 2048, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, },
  { 32, 64, 128, 1024, 2048, },
  { 1, 32, 64, 128, 1024, 2048, },
  { 2, 32, 64, 128, 1024, 2048, },
  { 1, 2, 32, 64, 128, 1024, 2048, },
  { 4, 32, 64, 128, 1024, 2048, },
  { 1, 4, 32, 64, 128, 1024, 2048, },
  { 2, 4, 32, 64, 128, 1024, 2048, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, },
  { 8, 32, 64, 128, 1024, 2048, },
  { 1, 8, 32, 64, 128, 1024, 2048, },
  { 2, 8, 32, 64, 128, 1024, 2048, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, },
  { 4, 8, 32, 64, 128, 1024, 2048, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, },
  { 16, 32, 64, 128, 1024, 2048, },
  { 1, 16, 32, 64, 128, 1024, 2048, },
  { 2, 16, 32, 64, 128, 1024, 2048, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, },
  { 4, 16, 32, 64, 128, 1024, 2048, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, },
  { 8, 16, 32, 64, 128, 1024, 2048, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, },
  { 256, 1024, 2048, },
  { 1, 256, 1024, 2048, },
  { 2, 256, 1024, 2048, },
  { 1, 2, 256, 1024, 2048, },
  { 4, 256, 1024, 2048, },
  { 1, 4, 256, 1024, 2048, },
  { 2, 4, 256, 1024, 2048, },
  { 1, 2, 4, 256, 1024, 2048, },
  { 8, 256, 1024, 2048, },
  { 1, 8, 256, 1024, 2048, },
  { 2, 8, 256, 1024, 2048, },
  { 1, 2, 8, 256, 1024, 2048, },
  { 4, 8, 256, 1024, 2048, },
  { 1, 4, 8, 256, 1024, 2048, },
  { 2, 4, 8, 256, 1024, 2048, },
  { 1, 2, 4, 8, 256, 1024, 2048, },
  { 16, 256, 1024, 2048, },
  { 1, 16, 256, 1024, 2048, },
  { 2, 16, 256, 1024, 2048, },
  { 1, 2, 16, 256, 1024, 2048, },
  { 4, 16, 256, 1024, 2048, },
  { 1, 4, 16, 256, 1024, 2048, },
  { 2, 4, 16, 256, 1024, 2048, },
  { 1, 2, 4, 16, 256, 1024, 2048, },
  { 8, 16, 256, 1024, 2048, },
  { 1, 8, 16, 256, 1024, 2048, },
  { 2, 8, 16, 256, 1024, 2048, },
  { 1, 2, 8, 16, 256, 1024, 2048, },
  { 4, 8, 16, 256, 1024, 2048, },
  { 1, 4, 8, 16, 256, 1024, 2048, },
  { 2, 4, 8, 16, 256, 1024, 2048, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, },
  { 32, 256, 1024, 2048, },
  { 1, 32, 256, 1024, 2048, },
  { 2, 32, 256, 1024, 2048, },
  { 1, 2, 32, 256, 1024, 2048, },
  { 4, 32, 256, 1024, 2048, },
  { 1, 4, 32, 256, 1024, 2048, },
  { 2, 4, 32, 256, 1024, 2048, },
  { 1, 2, 4, 32, 256, 1024, 2048, },
  { 8, 32, 256, 1024, 2048, },
  { 1, 8, 32, 256, 1024, 2048, },
  { 2, 8, 32, 256, 1024, 2048, },
  { 1, 2, 8, 32, 256, 1024, 2048, },
  { 4, 8, 32, 256, 1024, 2048, },
  { 1, 4, 8, 32, 256, 1024, 2048, },
  { 2, 4, 8, 32, 256, 1024, 2048, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, },
  { 16, 32, 256, 1024, 2048, },
  { 1, 16, 32, 256, 1024, 2048, },
  { 2, 16, 32, 256, 1024, 2048, },
  { 1, 2, 16, 32, 256, 1024, 2048, },
  { 4, 16, 32, 256, 1024, 2048, },
  { 1, 4, 16, 32, 256, 1024, 2048, },
  { 2, 4, 16, 32, 256, 1024, 2048, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, },
  { 8, 16, 32, 256, 1024, 2048, },
  { 1, 8, 16, 32, 256, 1024, 2048, },
  { 2, 8, 16, 32, 256, 1024, 2048, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, },
  { 4, 8, 16, 32, 256, 1024, 2048, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, },
  { 64, 256, 1024, 2048, },
  { 1, 64, 256, 1024, 2048, },
  { 2, 64, 256, 1024, 2048, },
  { 1, 2, 64, 256, 1024, 2048, },
  { 4, 64, 256, 1024, 2048, },
  { 1, 4, 64, 256, 1024, 2048, },
  { 2, 4, 64, 256, 1024, 2048, },
  { 1, 2, 4, 64, 256, 1024, 2048, },
  { 8, 64, 256, 1024, 2048, },
  { 1, 8, 64, 256, 1024, 2048, },
  { 2, 8, 64, 256, 1024, 2048, },
  { 1, 2, 8, 64, 256, 1024, 2048, },
  { 4, 8, 64, 256, 1024, 2048, },
  { 1, 4, 8, 64, 256, 1024, 2048, },
  { 2, 4, 8, 64, 256, 1024, 2048, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, },
  { 16, 64, 256, 1024, 2048, },
  { 1, 16, 64, 256, 1024, 2048, },
  { 2, 16, 64, 256, 1024, 2048, },
  { 1, 2, 16, 64, 256, 1024, 2048, },
  { 4, 16, 64, 256, 1024, 2048, },
  { 1, 4, 16, 64, 256, 1024, 2048, },
  { 2, 4, 16, 64, 256, 1024, 2048, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, },
  { 8, 16, 64, 256, 1024, 2048, },
  { 1, 8, 16, 64, 256, 1024, 2048, },
  { 2, 8, 16, 64, 256, 1024, 2048, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, },
  { 4, 8, 16, 64, 256, 1024, 2048, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, },
  { 32, 64, 256, 1024, 2048, },
  { 1, 32, 64, 256, 1024, 2048, },
  { 2, 32, 64, 256, 1024, 2048, },
  { 1, 2, 32, 64, 256, 1024, 2048, },
  { 4, 32, 64, 256, 1024, 2048, },
  { 1, 4, 32, 64, 256, 1024, 2048, },
  { 2, 4, 32, 64, 256, 1024, 2048, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, },
  { 8, 32, 64, 256, 1024, 2048, },
  { 1, 8, 32, 64, 256, 1024, 2048, },
  { 2, 8, 32, 64, 256, 1024, 2048, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, },
  { 4, 8, 32, 64, 256, 1024, 2048, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, },
  { 16, 32, 64, 256, 1024, 2048, },
  { 1, 16, 32, 64, 256, 1024, 2048, },
  { 2, 16, 32, 64, 256, 1024, 2048, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, },
  { 4, 16, 32, 64, 256, 1024, 2048, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, },
  { 8, 16, 32, 64, 256, 1024, 2048, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, },
  { 128, 256, 1024, 2048, },
  { 1, 128, 256, 1024, 2048, },
  { 2, 128, 256, 1024, 2048, },
  { 1, 2, 128, 256, 1024, 2048, },
  { 4, 128, 256, 1024, 2048, },
  { 1, 4, 128, 256, 1024, 2048, },
  { 2, 4, 128, 256, 1024, 2048, },
  { 1, 2, 4, 128, 256, 1024, 2048, },
  { 8, 128, 256, 1024, 2048, },
  { 1, 8, 128, 256, 1024, 2048, },
  { 2, 8, 128, 256, 1024, 2048, },
  { 1, 2, 8, 128, 256, 1024, 2048, },
  { 4, 8, 128, 256, 1024, 2048, },
  { 1, 4, 8, 128, 256, 1024, 2048, },
  { 2, 4, 8, 128, 256, 1024, 2048, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, },
  { 16, 128, 256, 1024, 2048, },
  { 1, 16, 128, 256, 1024, 2048, },
  { 2, 16, 128, 256, 1024, 2048, },
  { 1, 2, 16, 128, 256, 1024, 2048, },
  { 4, 16, 128, 256, 1024, 2048, },
  { 1, 4, 16, 128, 256, 1024, 2048, },
  { 2, 4, 16, 128, 256, 1024, 2048, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, },
  { 8, 16, 128, 256, 1024, 2048, },
  { 1, 8, 16, 128, 256, 1024, 2048, },
  { 2, 8, 16, 128, 256, 1024, 2048, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, },
  { 4, 8, 16, 128, 256, 1024, 2048, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, },
  { 32, 128, 256, 1024, 2048, },
  { 1, 32, 128, 256, 1024, 2048, },
  { 2, 32, 128, 256, 1024, 2048, },
  { 1, 2, 32, 128, 256, 1024, 2048, },
  { 4, 32, 128, 256, 1024, 2048, },
  { 1, 4, 32, 128, 256, 1024, 2048, },
  { 2, 4, 32, 128, 256, 1024, 2048, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, },
  { 8, 32, 128, 256, 1024, 2048, },
  { 1, 8, 32, 128, 256, 1024, 2048, },
  { 2, 8, 32, 128, 256, 1024, 2048, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, },
  { 4, 8, 32, 128, 256, 1024, 2048, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, },
  { 16, 32, 128, 256, 1024, 2048, },
  { 1, 16, 32, 128, 256, 1024, 2048, },
  { 2, 16, 32, 128, 256, 1024, 2048, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, },
  { 4, 16, 32, 128, 256, 1024, 2048, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, },
  { 8, 16, 32, 128, 256, 1024, 2048, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, },
  { 64, 128, 256, 1024, 2048, },
  { 1, 64, 128, 256, 1024, 2048, },
  { 2, 64, 128, 256, 1024, 2048, },
  { 1, 2, 64, 128, 256, 1024, 2048, },
  { 4, 64, 128, 256, 1024, 2048, },
  { 1, 4, 64, 128, 256, 1024, 2048, },
  { 2, 4, 64, 128, 256, 1024, 2048, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, },
  { 8, 64, 128, 256, 1024, 2048, },
  { 1, 8, 64, 128, 256, 1024, 2048, },
  { 2, 8, 64, 128, 256, 1024, 2048, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, },
  { 4, 8, 64, 128, 256, 1024, 2048, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, },
  { 16, 64, 128, 256, 1024, 2048, },
  { 1, 16, 64, 128, 256, 1024, 2048, },
  { 2, 16, 64, 128, 256, 1024, 2048, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, },
  { 4, 16, 64, 128, 256, 1024, 2048, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, },
  { 8, 16, 64, 128, 256, 1024, 2048, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, },
  { 32, 64, 128, 256, 1024, 2048, },
  { 1, 32, 64, 128, 256, 1024, 2048, },
  { 2, 32, 64, 128, 256, 1024, 2048, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, },
  { 4, 32, 64, 128, 256, 1024, 2048, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, },
  { 8, 32, 64, 128, 256, 1024, 2048, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, },
  { 16, 32, 64, 128, 256, 1024, 2048, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, },
  { 512, 1024, 2048, },
  { 1, 512, 1024, 2048, },
  { 2, 512, 1024, 2048, },
  { 1, 2, 512, 1024, 2048, },
  { 4, 512, 1024, 2048, },
  { 1, 4, 512, 1024, 2048, },
  { 2, 4, 512, 1024, 2048, },
  { 1, 2, 4, 512, 1024, 2048, },
  { 8, 512, 1024, 2048, },
  { 1, 8, 512, 1024, 2048, },
  { 2, 8, 512, 1024, 2048, },
  { 1, 2, 8, 512, 1024, 2048, },
  { 4, 8, 512, 1024, 2048, },
  { 1, 4, 8, 512, 1024, 2048, },
  { 2, 4, 8, 512, 1024, 2048, },
  { 1, 2, 4, 8, 512, 1024, 2048, },
  { 16, 512, 1024, 2048, },
  { 1, 16, 512, 1024, 2048, },
  { 2, 16, 512, 1024, 2048, },
  { 1, 2, 16, 512, 1024, 2048, },
  { 4, 16, 512, 1024, 2048, },
  { 1, 4, 16, 512, 1024, 2048, },
  { 2, 4, 16, 512, 1024, 2048, },
  { 1, 2, 4, 16, 512, 1024, 2048, },
  { 8, 16, 512, 1024, 2048, },
  { 1, 8, 16, 512, 1024, 2048, },
  { 2, 8, 16, 512, 1024, 2048, },
  { 1, 2, 8, 16, 512, 1024, 2048, },
  { 4, 8, 16, 512, 1024, 2048, },
  { 1, 4, 8, 16, 512, 1024, 2048, },
  { 2, 4, 8, 16, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, },
  { 32, 512, 1024, 2048, },
  { 1, 32, 512, 1024, 2048, },
  { 2, 32, 512, 1024, 2048, },
  { 1, 2, 32, 512, 1024, 2048, },
  { 4, 32, 512, 1024, 2048, },
  { 1, 4, 32, 512, 1024, 2048, },
  { 2, 4, 32, 512, 1024, 2048, },
  { 1, 2, 4, 32, 512, 1024, 2048, },
  { 8, 32, 512, 1024, 2048, },
  { 1, 8, 32, 512, 1024, 2048, },
  { 2, 8, 32, 512, 1024, 2048, },
  { 1, 2, 8, 32, 512, 1024, 2048, },
  { 4, 8, 32, 512, 1024, 2048, },
  { 1, 4, 8, 32, 512, 1024, 2048, },
  { 2, 4, 8, 32, 512, 1024, 2048, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, },
  { 16, 32, 512, 1024, 2048, },
  { 1, 16, 32, 512, 1024, 2048, },
  { 2, 16, 32, 512, 1024, 2048, },
  { 1, 2, 16, 32, 512, 1024, 2048, },
  { 4, 16, 32, 512, 1024, 2048, },
  { 1, 4, 16, 32, 512, 1024, 2048, },
  { 2, 4, 16, 32, 512, 1024, 2048, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, },
  { 8, 16, 32, 512, 1024, 2048, },
  { 1, 8, 16, 32, 512, 1024, 2048, },
  { 2, 8, 16, 32, 512, 1024, 2048, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, },
  { 4, 8, 16, 32, 512, 1024, 2048, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, },
  { 64, 512, 1024, 2048, },
  { 1, 64, 512, 1024, 2048, },
  { 2, 64, 512, 1024, 2048, },
  { 1, 2, 64, 512, 1024, 2048, },
  { 4, 64, 512, 1024, 2048, },
  { 1, 4, 64, 512, 1024, 2048, },
  { 2, 4, 64, 512, 1024, 2048, },
  { 1, 2, 4, 64, 512, 1024, 2048, },
  { 8, 64, 512, 1024, 2048, },
  { 1, 8, 64, 512, 1024, 2048, },
  { 2, 8, 64, 512, 1024, 2048, },
  { 1, 2, 8, 64, 512, 1024, 2048, },
  { 4, 8, 64, 512, 1024, 2048, },
  { 1, 4, 8, 64, 512, 1024, 2048, },
  { 2, 4, 8, 64, 512, 1024, 2048, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, },
  { 16, 64, 512, 1024, 2048, },
  { 1, 16, 64, 512, 1024, 2048, },
  { 2, 16, 64, 512, 1024, 2048, },
  { 1, 2, 16, 64, 512, 1024, 2048, },
  { 4, 16, 64, 512, 1024, 2048, },
  { 1, 4, 16, 64, 512, 1024, 2048, },
  { 2, 4, 16, 64, 512, 1024, 2048, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, },
  { 8, 16, 64, 512, 1024, 2048, },
  { 1, 8, 16, 64, 512, 1024, 2048, },
  { 2, 8, 16, 64, 512, 1024, 2048, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, },
  { 4, 8, 16, 64, 512, 1024, 2048, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, },
  { 32, 64, 512, 1024, 2048, },
  { 1, 32, 64, 512, 1024, 2048, },
  { 2, 32, 64, 512, 1024, 2048, },
  { 1, 2, 32, 64, 512, 1024, 2048, },
  { 4, 32, 64, 512, 1024, 2048, },
  { 1, 4, 32, 64, 512, 1024, 2048, },
  { 2, 4, 32, 64, 512, 1024, 2048, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, },
  { 8, 32, 64, 512, 1024, 2048, },
  { 1, 8, 32, 64, 512, 1024, 2048, },
  { 2, 8, 32, 64, 512, 1024, 2048, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, },
  { 4, 8, 32, 64, 512, 1024, 2048, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, },
  { 16, 32, 64, 512, 1024, 2048, },
  { 1, 16, 32, 64, 512, 1024, 2048, },
  { 2, 16, 32, 64, 512, 1024, 2048, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, },
  { 4, 16, 32, 64, 512, 1024, 2048, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, },
  { 8, 16, 32, 64, 512, 1024, 2048, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, },
  { 128, 512, 1024, 2048, },
  { 1, 128, 512, 1024, 2048, },
  { 2, 128, 512, 1024, 2048, },
  { 1, 2, 128, 512, 1024, 2048, },
  { 4, 128, 512, 1024, 2048, },
  { 1, 4, 128, 512, 1024, 2048, },
  { 2, 4, 128, 512, 1024, 2048, },
  { 1, 2, 4, 128, 512, 1024, 2048, },
  { 8, 128, 512, 1024, 2048, },
  { 1, 8, 128, 512, 1024, 2048, },
  { 2, 8, 128, 512, 1024, 2048, },
  { 1, 2, 8, 128, 512, 1024, 2048, },
  { 4, 8, 128, 512, 1024, 2048, },
  { 1, 4, 8, 128, 512, 1024, 2048, },
  { 2, 4, 8, 128, 512, 1024, 2048, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, },
  { 16, 128, 512, 1024, 2048, },
  { 1, 16, 128, 512, 1024, 2048, },
  { 2, 16, 128, 512, 1024, 2048, },
  { 1, 2, 16, 128, 512, 1024, 2048, },
  { 4, 16, 128, 512, 1024, 2048, },
  { 1, 4, 16, 128, 512, 1024, 2048, },
  { 2, 4, 16, 128, 512, 1024, 2048, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, },
  { 8, 16, 128, 512, 1024, 2048, },
  { 1, 8, 16, 128, 512, 1024, 2048, },
  { 2, 8, 16, 128, 512, 1024, 2048, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, },
  { 4, 8, 16, 128, 512, 1024, 2048, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, },
  { 32, 128, 512, 1024, 2048, },
  { 1, 32, 128, 512, 1024, 2048, },
  { 2, 32, 128, 512, 1024, 2048, },
  { 1, 2, 32, 128, 512, 1024, 2048, },
  { 4, 32, 128, 512, 1024, 2048, },
  { 1, 4, 32, 128, 512, 1024, 2048, },
  { 2, 4, 32, 128, 512, 1024, 2048, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, },
  { 8, 32, 128, 512, 1024, 2048, },
  { 1, 8, 32, 128, 512, 1024, 2048, },
  { 2, 8, 32, 128, 512, 1024, 2048, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, },
  { 4, 8, 32, 128, 512, 1024, 2048, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, },
  { 16, 32, 128, 512, 1024, 2048, },
  { 1, 16, 32, 128, 512, 1024, 2048, },
  { 2, 16, 32, 128, 512, 1024, 2048, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, },
  { 4, 16, 32, 128, 512, 1024, 2048, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, },
  { 8, 16, 32, 128, 512, 1024, 2048, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, },
  { 64, 128, 512, 1024, 2048, },
  { 1, 64, 128, 512, 1024, 2048, },
  { 2, 64, 128, 512, 1024, 2048, },
  { 1, 2, 64, 128, 512, 1024, 2048, },
  { 4, 64, 128, 512, 1024, 2048, },
  { 1, 4, 64, 128, 512, 1024, 2048, },
  { 2, 4, 64, 128, 512, 1024, 2048, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, },
  { 8, 64, 128, 512, 1024, 2048, },
  { 1, 8, 64, 128, 512, 1024, 2048, },
  { 2, 8, 64, 128, 512, 1024, 2048, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, },
  { 4, 8, 64, 128, 512, 1024, 2048, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, },
  { 16, 64, 128, 512, 1024, 2048, },
  { 1, 16, 64, 128, 512, 1024, 2048, },
  { 2, 16, 64, 128, 512, 1024, 2048, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, },
  { 4, 16, 64, 128, 512, 1024, 2048, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, },
  { 8, 16, 64, 128, 512, 1024, 2048, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, },
  { 32, 64, 128, 512, 1024, 2048, },
  { 1, 32, 64, 128, 512, 1024, 2048, },
  { 2, 32, 64, 128, 512, 1024, 2048, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, },
  { 4, 32, 64, 128, 512, 1024, 2048, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, },
  { 8, 32, 64, 128, 512, 1024, 2048, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, },
  { 16, 32, 64, 128, 512, 1024, 2048, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, },
  { 256, 512, 1024, 2048, },
  { 1, 256, 512, 1024, 2048, },
  { 2, 256, 512, 1024, 2048, },
  { 1, 2, 256, 512, 1024, 2048, },
  { 4, 256, 512, 1024, 2048, },
  { 1, 4, 256, 512, 1024, 2048, },
  { 2, 4, 256, 512, 1024, 2048, },
  { 1, 2, 4, 256, 512, 1024, 2048, },
  { 8, 256, 512, 1024, 2048, },
  { 1, 8, 256, 512, 1024, 2048, },
  { 2, 8, 256, 512, 1024, 2048, },
  { 1, 2, 8, 256, 512, 1024, 2048, },
  { 4, 8, 256, 512, 1024, 2048, },
  { 1, 4, 8, 256, 512, 1024, 2048, },
  { 2, 4, 8, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, },
  { 16, 256, 512, 1024, 2048, },
  { 1, 16, 256, 512, 1024, 2048, },
  { 2, 16, 256, 512, 1024, 2048, },
  { 1, 2, 16, 256, 512, 1024, 2048, },
  { 4, 16, 256, 512, 1024, 2048, },
  { 1, 4, 16, 256, 512, 1024, 2048, },
  { 2, 4, 16, 256, 512, 1024, 2048, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, },
  { 8, 16, 256, 512, 1024, 2048, },
  { 1, 8, 16, 256, 512, 1024, 2048, },
  { 2, 8, 16, 256, 512, 1024, 2048, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, },
  { 4, 8, 16, 256, 512, 1024, 2048, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, },
  { 32, 256, 512, 1024, 2048, },
  { 1, 32, 256, 512, 1024, 2048, },
  { 2, 32, 256, 512, 1024, 2048, },
  { 1, 2, 32, 256, 512, 1024, 2048, },
  { 4, 32, 256, 512, 1024, 2048, },
  { 1, 4, 32, 256, 512, 1024, 2048, },
  { 2, 4, 32, 256, 512, 1024, 2048, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, },
  { 8, 32, 256, 512, 1024, 2048, },
  { 1, 8, 32, 256, 512, 1024, 2048, },
  { 2, 8, 32, 256, 512, 1024, 2048, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, },
  { 4, 8, 32, 256, 512, 1024, 2048, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, },
  { 16, 32, 256, 512, 1024, 2048, },
  { 1, 16, 32, 256, 512, 1024, 2048, },
  { 2, 16, 32, 256, 512, 1024, 2048, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, },
  { 4, 16, 32, 256, 512, 1024, 2048, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, },
  { 8, 16, 32, 256, 512, 1024, 2048, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, },
  { 64, 256, 512, 1024, 2048, },
  { 1, 64, 256, 512, 1024, 2048, },
  { 2, 64, 256, 512, 1024, 2048, },
  { 1, 2, 64, 256, 512, 1024, 2048, },
  { 4, 64, 256, 512, 1024, 2048, },
  { 1, 4, 64, 256, 512, 1024, 2048, },
  { 2, 4, 64, 256, 512, 1024, 2048, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, },
  { 8, 64, 256, 512, 1024, 2048, },
  { 1, 8, 64, 256, 512, 1024, 2048, },
  { 2, 8, 64, 256, 512, 1024, 2048, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, },
  { 4, 8, 64, 256, 512, 1024, 2048, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, },
  { 16, 64, 256, 512, 1024, 2048, },
  { 1, 16, 64, 256, 512, 1024, 2048, },
  { 2, 16, 64, 256, 512, 1024, 2048, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, },
  { 4, 16, 64, 256, 512, 1024, 2048, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, },
  { 8, 16, 64, 256, 512, 1024, 2048, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, },
  { 32, 64, 256, 512, 1024, 2048, },
  { 1, 32, 64, 256, 512, 1024, 2048, },
  { 2, 32, 64, 256, 512, 1024, 2048, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, },
  { 4, 32, 64, 256, 512, 1024, 2048, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, },
  { 8, 32, 64, 256, 512, 1024, 2048, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, },
  { 16, 32, 64, 256, 512, 1024, 2048, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, },
  { 128, 256, 512, 1024, 2048, },
  { 1, 128, 256, 512, 1024, 2048, },
  { 2, 128, 256, 512, 1024, 2048, },
  { 1, 2, 128, 256, 512, 1024, 2048, },
  { 4, 128, 256, 512, 1024, 2048, },
  { 1, 4, 128, 256, 512, 1024, 2048, },
  { 2, 4, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, },
  { 8, 128, 256, 512, 1024, 2048, },
  { 1, 8, 128, 256, 512, 1024, 2048, },
  { 2, 8, 128, 256, 512, 1024, 2048, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, },
  { 4, 8, 128, 256, 512, 1024, 2048, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, },
  { 16, 128, 256, 512, 1024, 2048, },
  { 1, 16, 128, 256, 512, 1024, 2048, },
  { 2, 16, 128, 256, 512, 1024, 2048, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, },
  { 4, 16, 128, 256, 512, 1024, 2048, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, },
  { 8, 16, 128, 256, 512, 1024, 2048, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, },
  { 32, 128, 256, 512, 1024, 2048, },
  { 1, 32, 128, 256, 512, 1024, 2048, },
  { 2, 32, 128, 256, 512, 1024, 2048, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, },
  { 4, 32, 128, 256, 512, 1024, 2048, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, },
  { 8, 32, 128, 256, 512, 1024, 2048, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, },
  { 16, 32, 128, 256, 512, 1024, 2048, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, },
  { 64, 128, 256, 512, 1024, 2048, },
  { 1, 64, 128, 256, 512, 1024, 2048, },
  { 2, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, },
  { 4, 64, 128, 256, 512, 1024, 2048, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, },
  { 8, 64, 128, 256, 512, 1024, 2048, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, },
  { 16, 64, 128, 256, 512, 1024, 2048, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, },
  { 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, },
  { 4096, },
  { 1, 4096, },
  { 2, 4096, },
  { 1, 2, 4096, },
  { 4, 4096, },
  { 1, 4, 4096, },
  { 2, 4, 4096, },
  { 1, 2, 4, 4096, },
  { 8, 4096, },
  { 1, 8, 4096, },
  { 2, 8, 4096, },
  { 1, 2, 8, 4096, },
  { 4, 8, 4096, },
  { 1, 4, 8, 4096, },
  { 2, 4, 8, 4096, },
  { 1, 2, 4, 8, 4096, },
  { 16, 4096, },
  { 1, 16, 4096, },
  { 2, 16, 4096, },
  { 1, 2, 16, 4096, },
  { 4, 16, 4096, },
  { 1, 4, 16, 4096, },
  { 2, 4, 16, 4096, },
  { 1, 2, 4, 16, 4096, },
  { 8, 16, 4096, },
  { 1, 8, 16, 4096, },
  { 2, 8, 16, 4096, },
  { 1, 2, 8, 16, 4096, },
  { 4, 8, 16, 4096, },
  { 1, 4, 8, 16, 4096, },
  { 2, 4, 8, 16, 4096, },
  { 1, 2, 4, 8, 16, 4096, },
  { 32, 4096, },
  { 1, 32, 4096, },
  { 2, 32, 4096, },
  { 1, 2, 32, 4096, },
  { 4, 32, 4096, },
  { 1, 4, 32, 4096, },
  { 2, 4, 32, 4096, },
  { 1, 2, 4, 32, 4096, },
  { 8, 32, 4096, },
  { 1, 8, 32, 4096, },
  { 2, 8, 32, 4096, },
  { 1, 2, 8, 32, 4096, },
  { 4, 8, 32, 4096, },
  { 1, 4, 8, 32, 4096, },
  { 2, 4, 8, 32, 4096, },
  { 1, 2, 4, 8, 32, 4096, },
  { 16, 32, 4096, },
  { 1, 16, 32, 4096, },
  { 2, 16, 32, 4096, },
  { 1, 2, 16, 32, 4096, },
  { 4, 16, 32, 4096, },
  { 1, 4, 16, 32, 4096, },
  { 2, 4, 16, 32, 4096, },
  { 1, 2, 4, 16, 32, 4096, },
  { 8, 16, 32, 4096, },
  { 1, 8, 16, 32, 4096, },
  { 2, 8, 16, 32, 4096, },
  { 1, 2, 8, 16, 32, 4096, },
  { 4, 8, 16, 32, 4096, },
  { 1, 4, 8, 16, 32, 4096, },
  { 2, 4, 8, 16, 32, 4096, },
  { 1, 2, 4, 8, 16, 32, 4096, },
  { 64, 4096, },
  { 1, 64, 4096, },
  { 2, 64, 4096, },
  { 1, 2, 64, 4096, },
  { 4, 64, 4096, },
  { 1, 4, 64, 4096, },
  { 2, 4, 64, 4096, },
  { 1, 2, 4, 64, 4096, },
  { 8, 64, 4096, },
  { 1, 8, 64, 4096, },
  { 2, 8, 64, 4096, },
  { 1, 2, 8, 64, 4096, },
  { 4, 8, 64, 4096, },
  { 1, 4, 8, 64, 4096, },
  { 2, 4, 8, 64, 4096, },
  { 1, 2, 4, 8, 64, 4096, },
  { 16, 64, 4096, },
  { 1, 16, 64, 4096, },
  { 2, 16, 64, 4096, },
  { 1, 2, 16, 64, 4096, },
  { 4, 16, 64, 4096, },
  { 1, 4, 16, 64, 4096, },
  { 2, 4, 16, 64, 4096, },
  { 1, 2, 4, 16, 64, 4096, },
  { 8, 16, 64, 4096, },
  { 1, 8, 16, 64, 4096, },
  { 2, 8, 16, 64, 4096, },
  { 1, 2, 8, 16, 64, 4096, },
  { 4, 8, 16, 64, 4096, },
  { 1, 4, 8, 16, 64, 4096, },
  { 2, 4, 8, 16, 64, 4096, },
  { 1, 2, 4, 8, 16, 64, 4096, },
  { 32, 64, 4096, },
  { 1, 32, 64, 4096, },
  { 2, 32, 64, 4096, },
  { 1, 2, 32, 64, 4096, },
  { 4, 32, 64, 4096, },
  { 1, 4, 32, 64, 4096, },
  { 2, 4, 32, 64, 4096, },
  { 1, 2, 4, 32, 64, 4096, },
  { 8, 32, 64, 4096, },
  { 1, 8, 32, 64, 4096, },
  { 2, 8, 32, 64, 4096, },
  { 1, 2, 8, 32, 64, 4096, },
  { 4, 8, 32, 64, 4096, },
  { 1, 4, 8, 32, 64, 4096, },
  { 2, 4, 8, 32, 64, 4096, },
  { 1, 2, 4, 8, 32, 64, 4096, },
  { 16, 32, 64, 4096, },
  { 1, 16, 32, 64, 4096, },
  { 2, 16, 32, 64, 4096, },
  { 1, 2, 16, 32, 64, 4096, },
  { 4, 16, 32, 64, 4096, },
  { 1, 4, 16, 32, 64, 4096, },
  { 2, 4, 16, 32, 64, 4096, },
  { 1, 2, 4, 16, 32, 64, 4096, },
  { 8, 16, 32, 64, 4096, },
  { 1, 8, 16, 32, 64, 4096, },
  { 2, 8, 16, 32, 64, 4096, },
  { 1, 2, 8, 16, 32, 64, 4096, },
  { 4, 8, 16, 32, 64, 4096, },
  { 1, 4, 8, 16, 32, 64, 4096, },
  { 2, 4, 8, 16, 32, 64, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 4096, },
  { 128, 4096, },
  { 1, 128, 4096, },
  { 2, 128, 4096, },
  { 1, 2, 128, 4096, },
  { 4, 128, 4096, },
  { 1, 4, 128, 4096, },
  { 2, 4, 128, 4096, },
  { 1, 2, 4, 128, 4096, },
  { 8, 128, 4096, },
  { 1, 8, 128, 4096, },
  { 2, 8, 128, 4096, },
  { 1, 2, 8, 128, 4096, },
  { 4, 8, 128, 4096, },
  { 1, 4, 8, 128, 4096, },
  { 2, 4, 8, 128, 4096, },
  { 1, 2, 4, 8, 128, 4096, },
  { 16, 128, 4096, },
  { 1, 16, 128, 4096, },
  { 2, 16, 128, 4096, },
  { 1, 2, 16, 128, 4096, },
  { 4, 16, 128, 4096, },
  { 1, 4, 16, 128, 4096, },
  { 2, 4, 16, 128, 4096, },
  { 1, 2, 4, 16, 128, 4096, },
  { 8, 16, 128, 4096, },
  { 1, 8, 16, 128, 4096, },
  { 2, 8, 16, 128, 4096, },
  { 1, 2, 8, 16, 128, 4096, },
  { 4, 8, 16, 128, 4096, },
  { 1, 4, 8, 16, 128, 4096, },
  { 2, 4, 8, 16, 128, 4096, },
  { 1, 2, 4, 8, 16, 128, 4096, },
  { 32, 128, 4096, },
  { 1, 32, 128, 4096, },
  { 2, 32, 128, 4096, },
  { 1, 2, 32, 128, 4096, },
  { 4, 32, 128, 4096, },
  { 1, 4, 32, 128, 4096, },
  { 2, 4, 32, 128, 4096, },
  { 1, 2, 4, 32, 128, 4096, },
  { 8, 32, 128, 4096, },
  { 1, 8, 32, 128, 4096, },
  { 2, 8, 32, 128, 4096, },
  { 1, 2, 8, 32, 128, 4096, },
  { 4, 8, 32, 128, 4096, },
  { 1, 4, 8, 32, 128, 4096, },
  { 2, 4, 8, 32, 128, 4096, },
  { 1, 2, 4, 8, 32, 128, 4096, },
  { 16, 32, 128, 4096, },
  { 1, 16, 32, 128, 4096, },
  { 2, 16, 32, 128, 4096, },
  { 1, 2, 16, 32, 128, 4096, },
  { 4, 16, 32, 128, 4096, },
  { 1, 4, 16, 32, 128, 4096, },
  { 2, 4, 16, 32, 128, 4096, },
  { 1, 2, 4, 16, 32, 128, 4096, },
  { 8, 16, 32, 128, 4096, },
  { 1, 8, 16, 32, 128, 4096, },
  { 2, 8, 16, 32, 128, 4096, },
  { 1, 2, 8, 16, 32, 128, 4096, },
  { 4, 8, 16, 32, 128, 4096, },
  { 1, 4, 8, 16, 32, 128, 4096, },
  { 2, 4, 8, 16, 32, 128, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 4096, },
  { 64, 128, 4096, },
  { 1, 64, 128, 4096, },
  { 2, 64, 128, 4096, },
  { 1, 2, 64, 128, 4096, },
  { 4, 64, 128, 4096, },
  { 1, 4, 64, 128, 4096, },
  { 2, 4, 64, 128, 4096, },
  { 1, 2, 4, 64, 128, 4096, },
  { 8, 64, 128, 4096, },
  { 1, 8, 64, 128, 4096, },
  { 2, 8, 64, 128, 4096, },
  { 1, 2, 8, 64, 128, 4096, },
  { 4, 8, 64, 128, 4096, },
  { 1, 4, 8, 64, 128, 4096, },
  { 2, 4, 8, 64, 128, 4096, },
  { 1, 2, 4, 8, 64, 128, 4096, },
  { 16, 64, 128, 4096, },
  { 1, 16, 64, 128, 4096, },
  { 2, 16, 64, 128, 4096, },
  { 1, 2, 16, 64, 128, 4096, },
  { 4, 16, 64, 128, 4096, },
  { 1, 4, 16, 64, 128, 4096, },
  { 2, 4, 16, 64, 128, 4096, },
  { 1, 2, 4, 16, 64, 128, 4096, },
  { 8, 16, 64, 128, 4096, },
  { 1, 8, 16, 64, 128, 4096, },
  { 2, 8, 16, 64, 128, 4096, },
  { 1, 2, 8, 16, 64, 128, 4096, },
  { 4, 8, 16, 64, 128, 4096, },
  { 1, 4, 8, 16, 64, 128, 4096, },
  { 2, 4, 8, 16, 64, 128, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 4096, },
  { 32, 64, 128, 4096, },
  { 1, 32, 64, 128, 4096, },
  { 2, 32, 64, 128, 4096, },
  { 1, 2, 32, 64, 128, 4096, },
  { 4, 32, 64, 128, 4096, },
  { 1, 4, 32, 64, 128, 4096, },
  { 2, 4, 32, 64, 128, 4096, },
  { 1, 2, 4, 32, 64, 128, 4096, },
  { 8, 32, 64, 128, 4096, },
  { 1, 8, 32, 64, 128, 4096, },
  { 2, 8, 32, 64, 128, 4096, },
  { 1, 2, 8, 32, 64, 128, 4096, },
  { 4, 8, 32, 64, 128, 4096, },
  { 1, 4, 8, 32, 64, 128, 4096, },
  { 2, 4, 8, 32, 64, 128, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 4096, },
  { 16, 32, 64, 128, 4096, },
  { 1, 16, 32, 64, 128, 4096, },
  { 2, 16, 32, 64, 128, 4096, },
  { 1, 2, 16, 32, 64, 128, 4096, },
  { 4, 16, 32, 64, 128, 4096, },
  { 1, 4, 16, 32, 64, 128, 4096, },
  { 2, 4, 16, 32, 64, 128, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 4096, },
  { 8, 16, 32, 64, 128, 4096, },
  { 1, 8, 16, 32, 64, 128, 4096, },
  { 2, 8, 16, 32, 64, 128, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 4096, },
  { 4, 8, 16, 32, 64, 128, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 4096, },
  { 256, 4096, },
  { 1, 256, 4096, },
  { 2, 256, 4096, },
  { 1, 2, 256, 4096, },
  { 4, 256, 4096, },
  { 1, 4, 256, 4096, },
  { 2, 4, 256, 4096, },
  { 1, 2, 4, 256, 4096, },
  { 8, 256, 4096, },
  { 1, 8, 256, 4096, },
  { 2, 8, 256, 4096, },
  { 1, 2, 8, 256, 4096, },
  { 4, 8, 256, 4096, },
  { 1, 4, 8, 256, 4096, },
  { 2, 4, 8, 256, 4096, },
  { 1, 2, 4, 8, 256, 4096, },
  { 16, 256, 4096, },
  { 1, 16, 256, 4096, },
  { 2, 16, 256, 4096, },
  { 1, 2, 16, 256, 4096, },
  { 4, 16, 256, 4096, },
  { 1, 4, 16, 256, 4096, },
  { 2, 4, 16, 256, 4096, },
  { 1, 2, 4, 16, 256, 4096, },
  { 8, 16, 256, 4096, },
  { 1, 8, 16, 256, 4096, },
  { 2, 8, 16, 256, 4096, },
  { 1, 2, 8, 16, 256, 4096, },
  { 4, 8, 16, 256, 4096, },
  { 1, 4, 8, 16, 256, 4096, },
  { 2, 4, 8, 16, 256, 4096, },
  { 1, 2, 4, 8, 16, 256, 4096, },
  { 32, 256, 4096, },
  { 1, 32, 256, 4096, },
  { 2, 32, 256, 4096, },
  { 1, 2, 32, 256, 4096, },
  { 4, 32, 256, 4096, },
  { 1, 4, 32, 256, 4096, },
  { 2, 4, 32, 256, 4096, },
  { 1, 2, 4, 32, 256, 4096, },
  { 8, 32, 256, 4096, },
  { 1, 8, 32, 256, 4096, },
  { 2, 8, 32, 256, 4096, },
  { 1, 2, 8, 32, 256, 4096, },
  { 4, 8, 32, 256, 4096, },
  { 1, 4, 8, 32, 256, 4096, },
  { 2, 4, 8, 32, 256, 4096, },
  { 1, 2, 4, 8, 32, 256, 4096, },
  { 16, 32, 256, 4096, },
  { 1, 16, 32, 256, 4096, },
  { 2, 16, 32, 256, 4096, },
  { 1, 2, 16, 32, 256, 4096, },
  { 4, 16, 32, 256, 4096, },
  { 1, 4, 16, 32, 256, 4096, },
  { 2, 4, 16, 32, 256, 4096, },
  { 1, 2, 4, 16, 32, 256, 4096, },
  { 8, 16, 32, 256, 4096, },
  { 1, 8, 16, 32, 256, 4096, },
  { 2, 8, 16, 32, 256, 4096, },
  { 1, 2, 8, 16, 32, 256, 4096, },
  { 4, 8, 16, 32, 256, 4096, },
  { 1, 4, 8, 16, 32, 256, 4096, },
  { 2, 4, 8, 16, 32, 256, 4096, },
  { 1, 2, 4, 8, 16, 32, 256, 4096, },
  { 64, 256, 4096, },
  { 1, 64, 256, 4096, },
  { 2, 64, 256, 4096, },
  { 1, 2, 64, 256, 4096, },
  { 4, 64, 256, 4096, },
  { 1, 4, 64, 256, 4096, },
  { 2, 4, 64, 256, 4096, },
  { 1, 2, 4, 64, 256, 4096, },
  { 8, 64, 256, 4096, },
  { 1, 8, 64, 256, 4096, },
  { 2, 8, 64, 256, 4096, },
  { 1, 2, 8, 64, 256, 4096, },
  { 4, 8, 64, 256, 4096, },
  { 1, 4, 8, 64, 256, 4096, },
  { 2, 4, 8, 64, 256, 4096, },
  { 1, 2, 4, 8, 64, 256, 4096, },
  { 16, 64, 256, 4096, },
  { 1, 16, 64, 256, 4096, },
  { 2, 16, 64, 256, 4096, },
  { 1, 2, 16, 64, 256, 4096, },
  { 4, 16, 64, 256, 4096, },
  { 1, 4, 16, 64, 256, 4096, },
  { 2, 4, 16, 64, 256, 4096, },
  { 1, 2, 4, 16, 64, 256, 4096, },
  { 8, 16, 64, 256, 4096, },
  { 1, 8, 16, 64, 256, 4096, },
  { 2, 8, 16, 64, 256, 4096, },
  { 1, 2, 8, 16, 64, 256, 4096, },
  { 4, 8, 16, 64, 256, 4096, },
  { 1, 4, 8, 16, 64, 256, 4096, },
  { 2, 4, 8, 16, 64, 256, 4096, },
  { 1, 2, 4, 8, 16, 64, 256, 4096, },
  { 32, 64, 256, 4096, },
  { 1, 32, 64, 256, 4096, },
  { 2, 32, 64, 256, 4096, },
  { 1, 2, 32, 64, 256, 4096, },
  { 4, 32, 64, 256, 4096, },
  { 1, 4, 32, 64, 256, 4096, },
  { 2, 4, 32, 64, 256, 4096, },
  { 1, 2, 4, 32, 64, 256, 4096, },
  { 8, 32, 64, 256, 4096, },
  { 1, 8, 32, 64, 256, 4096, },
  { 2, 8, 32, 64, 256, 4096, },
  { 1, 2, 8, 32, 64, 256, 4096, },
  { 4, 8, 32, 64, 256, 4096, },
  { 1, 4, 8, 32, 64, 256, 4096, },
  { 2, 4, 8, 32, 64, 256, 4096, },
  { 1, 2, 4, 8, 32, 64, 256, 4096, },
  { 16, 32, 64, 256, 4096, },
  { 1, 16, 32, 64, 256, 4096, },
  { 2, 16, 32, 64, 256, 4096, },
  { 1, 2, 16, 32, 64, 256, 4096, },
  { 4, 16, 32, 64, 256, 4096, },
  { 1, 4, 16, 32, 64, 256, 4096, },
  { 2, 4, 16, 32, 64, 256, 4096, },
  { 1, 2, 4, 16, 32, 64, 256, 4096, },
  { 8, 16, 32, 64, 256, 4096, },
  { 1, 8, 16, 32, 64, 256, 4096, },
  { 2, 8, 16, 32, 64, 256, 4096, },
  { 1, 2, 8, 16, 32, 64, 256, 4096, },
  { 4, 8, 16, 32, 64, 256, 4096, },
  { 1, 4, 8, 16, 32, 64, 256, 4096, },
  { 2, 4, 8, 16, 32, 64, 256, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 4096, },
  { 128, 256, 4096, },
  { 1, 128, 256, 4096, },
  { 2, 128, 256, 4096, },
  { 1, 2, 128, 256, 4096, },
  { 4, 128, 256, 4096, },
  { 1, 4, 128, 256, 4096, },
  { 2, 4, 128, 256, 4096, },
  { 1, 2, 4, 128, 256, 4096, },
  { 8, 128, 256, 4096, },
  { 1, 8, 128, 256, 4096, },
  { 2, 8, 128, 256, 4096, },
  { 1, 2, 8, 128, 256, 4096, },
  { 4, 8, 128, 256, 4096, },
  { 1, 4, 8, 128, 256, 4096, },
  { 2, 4, 8, 128, 256, 4096, },
  { 1, 2, 4, 8, 128, 256, 4096, },
  { 16, 128, 256, 4096, },
  { 1, 16, 128, 256, 4096, },
  { 2, 16, 128, 256, 4096, },
  { 1, 2, 16, 128, 256, 4096, },
  { 4, 16, 128, 256, 4096, },
  { 1, 4, 16, 128, 256, 4096, },
  { 2, 4, 16, 128, 256, 4096, },
  { 1, 2, 4, 16, 128, 256, 4096, },
  { 8, 16, 128, 256, 4096, },
  { 1, 8, 16, 128, 256, 4096, },
  { 2, 8, 16, 128, 256, 4096, },
  { 1, 2, 8, 16, 128, 256, 4096, },
  { 4, 8, 16, 128, 256, 4096, },
  { 1, 4, 8, 16, 128, 256, 4096, },
  { 2, 4, 8, 16, 128, 256, 4096, },
  { 1, 2, 4, 8, 16, 128, 256, 4096, },
  { 32, 128, 256, 4096, },
  { 1, 32, 128, 256, 4096, },
  { 2, 32, 128, 256, 4096, },
  { 1, 2, 32, 128, 256, 4096, },
  { 4, 32, 128, 256, 4096, },
  { 1, 4, 32, 128, 256, 4096, },
  { 2, 4, 32, 128, 256, 4096, },
  { 1, 2, 4, 32, 128, 256, 4096, },
  { 8, 32, 128, 256, 4096, },
  { 1, 8, 32, 128, 256, 4096, },
  { 2, 8, 32, 128, 256, 4096, },
  { 1, 2, 8, 32, 128, 256, 4096, },
  { 4, 8, 32, 128, 256, 4096, },
  { 1, 4, 8, 32, 128, 256, 4096, },
  { 2, 4, 8, 32, 128, 256, 4096, },
  { 1, 2, 4, 8, 32, 128, 256, 4096, },
  { 16, 32, 128, 256, 4096, },
  { 1, 16, 32, 128, 256, 4096, },
  { 2, 16, 32, 128, 256, 4096, },
  { 1, 2, 16, 32, 128, 256, 4096, },
  { 4, 16, 32, 128, 256, 4096, },
  { 1, 4, 16, 32, 128, 256, 4096, },
  { 2, 4, 16, 32, 128, 256, 4096, },
  { 1, 2, 4, 16, 32, 128, 256, 4096, },
  { 8, 16, 32, 128, 256, 4096, },
  { 1, 8, 16, 32, 128, 256, 4096, },
  { 2, 8, 16, 32, 128, 256, 4096, },
  { 1, 2, 8, 16, 32, 128, 256, 4096, },
  { 4, 8, 16, 32, 128, 256, 4096, },
  { 1, 4, 8, 16, 32, 128, 256, 4096, },
  { 2, 4, 8, 16, 32, 128, 256, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 4096, },
  { 64, 128, 256, 4096, },
  { 1, 64, 128, 256, 4096, },
  { 2, 64, 128, 256, 4096, },
  { 1, 2, 64, 128, 256, 4096, },
  { 4, 64, 128, 256, 4096, },
  { 1, 4, 64, 128, 256, 4096, },
  { 2, 4, 64, 128, 256, 4096, },
  { 1, 2, 4, 64, 128, 256, 4096, },
  { 8, 64, 128, 256, 4096, },
  { 1, 8, 64, 128, 256, 4096, },
  { 2, 8, 64, 128, 256, 4096, },
  { 1, 2, 8, 64, 128, 256, 4096, },
  { 4, 8, 64, 128, 256, 4096, },
  { 1, 4, 8, 64, 128, 256, 4096, },
  { 2, 4, 8, 64, 128, 256, 4096, },
  { 1, 2, 4, 8, 64, 128, 256, 4096, },
  { 16, 64, 128, 256, 4096, },
  { 1, 16, 64, 128, 256, 4096, },
  { 2, 16, 64, 128, 256, 4096, },
  { 1, 2, 16, 64, 128, 256, 4096, },
  { 4, 16, 64, 128, 256, 4096, },
  { 1, 4, 16, 64, 128, 256, 4096, },
  { 2, 4, 16, 64, 128, 256, 4096, },
  { 1, 2, 4, 16, 64, 128, 256, 4096, },
  { 8, 16, 64, 128, 256, 4096, },
  { 1, 8, 16, 64, 128, 256, 4096, },
  { 2, 8, 16, 64, 128, 256, 4096, },
  { 1, 2, 8, 16, 64, 128, 256, 4096, },
  { 4, 8, 16, 64, 128, 256, 4096, },
  { 1, 4, 8, 16, 64, 128, 256, 4096, },
  { 2, 4, 8, 16, 64, 128, 256, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 4096, },
  { 32, 64, 128, 256, 4096, },
  { 1, 32, 64, 128, 256, 4096, },
  { 2, 32, 64, 128, 256, 4096, },
  { 1, 2, 32, 64, 128, 256, 4096, },
  { 4, 32, 64, 128, 256, 4096, },
  { 1, 4, 32, 64, 128, 256, 4096, },
  { 2, 4, 32, 64, 128, 256, 4096, },
  { 1, 2, 4, 32, 64, 128, 256, 4096, },
  { 8, 32, 64, 128, 256, 4096, },
  { 1, 8, 32, 64, 128, 256, 4096, },
  { 2, 8, 32, 64, 128, 256, 4096, },
  { 1, 2, 8, 32, 64, 128, 256, 4096, },
  { 4, 8, 32, 64, 128, 256, 4096, },
  { 1, 4, 8, 32, 64, 128, 256, 4096, },
  { 2, 4, 8, 32, 64, 128, 256, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 4096, },
  { 16, 32, 64, 128, 256, 4096, },
  { 1, 16, 32, 64, 128, 256, 4096, },
  { 2, 16, 32, 64, 128, 256, 4096, },
  { 1, 2, 16, 32, 64, 128, 256, 4096, },
  { 4, 16, 32, 64, 128, 256, 4096, },
  { 1, 4, 16, 32, 64, 128, 256, 4096, },
  { 2, 4, 16, 32, 64, 128, 256, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 4096, },
  { 8, 16, 32, 64, 128, 256, 4096, },
  { 1, 8, 16, 32, 64, 128, 256, 4096, },
  { 2, 8, 16, 32, 64, 128, 256, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 4096, },
  { 4, 8, 16, 32, 64, 128, 256, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 4096, },
  { 512, 4096, },
  { 1, 512, 4096, },
  { 2, 512, 4096, },
  { 1, 2, 512, 4096, },
  { 4, 512, 4096, },
  { 1, 4, 512, 4096, },
  { 2, 4, 512, 4096, },
  { 1, 2, 4, 512, 4096, },
  { 8, 512, 4096, },
  { 1, 8, 512, 4096, },
  { 2, 8, 512, 4096, },
  { 1, 2, 8, 512, 4096, },
  { 4, 8, 512, 4096, },
  { 1, 4, 8, 512, 4096, },
  { 2, 4, 8, 512, 4096, },
  { 1, 2, 4, 8, 512, 4096, },
  { 16, 512, 4096, },
  { 1, 16, 512, 4096, },
  { 2, 16, 512, 4096, },
  { 1, 2, 16, 512, 4096, },
  { 4, 16, 512, 4096, },
  { 1, 4, 16, 512, 4096, },
  { 2, 4, 16, 512, 4096, },
  { 1, 2, 4, 16, 512, 4096, },
  { 8, 16, 512, 4096, },
  { 1, 8, 16, 512, 4096, },
  { 2, 8, 16, 512, 4096, },
  { 1, 2, 8, 16, 512, 4096, },
  { 4, 8, 16, 512, 4096, },
  { 1, 4, 8, 16, 512, 4096, },
  { 2, 4, 8, 16, 512, 4096, },
  { 1, 2, 4, 8, 16, 512, 4096, },
  { 32, 512, 4096, },
  { 1, 32, 512, 4096, },
  { 2, 32, 512, 4096, },
  { 1, 2, 32, 512, 4096, },
  { 4, 32, 512, 4096, },
  { 1, 4, 32, 512, 4096, },
  { 2, 4, 32, 512, 4096, },
  { 1, 2, 4, 32, 512, 4096, },
  { 8, 32, 512, 4096, },
  { 1, 8, 32, 512, 4096, },
  { 2, 8, 32, 512, 4096, },
  { 1, 2, 8, 32, 512, 4096, },
  { 4, 8, 32, 512, 4096, },
  { 1, 4, 8, 32, 512, 4096, },
  { 2, 4, 8, 32, 512, 4096, },
  { 1, 2, 4, 8, 32, 512, 4096, },
  { 16, 32, 512, 4096, },
  { 1, 16, 32, 512, 4096, },
  { 2, 16, 32, 512, 4096, },
  { 1, 2, 16, 32, 512, 4096, },
  { 4, 16, 32, 512, 4096, },
  { 1, 4, 16, 32, 512, 4096, },
  { 2, 4, 16, 32, 512, 4096, },
  { 1, 2, 4, 16, 32, 512, 4096, },
  { 8, 16, 32, 512, 4096, },
  { 1, 8, 16, 32, 512, 4096, },
  { 2, 8, 16, 32, 512, 4096, },
  { 1, 2, 8, 16, 32, 512, 4096, },
  { 4, 8, 16, 32, 512, 4096, },
  { 1, 4, 8, 16, 32, 512, 4096, },
  { 2, 4, 8, 16, 32, 512, 4096, },
  { 1, 2, 4, 8, 16, 32, 512, 4096, },
  { 64, 512, 4096, },
  { 1, 64, 512, 4096, },
  { 2, 64, 512, 4096, },
  { 1, 2, 64, 512, 4096, },
  { 4, 64, 512, 4096, },
  { 1, 4, 64, 512, 4096, },
  { 2, 4, 64, 512, 4096, },
  { 1, 2, 4, 64, 512, 4096, },
  { 8, 64, 512, 4096, },
  { 1, 8, 64, 512, 4096, },
  { 2, 8, 64, 512, 4096, },
  { 1, 2, 8, 64, 512, 4096, },
  { 4, 8, 64, 512, 4096, },
  { 1, 4, 8, 64, 512, 4096, },
  { 2, 4, 8, 64, 512, 4096, },
  { 1, 2, 4, 8, 64, 512, 4096, },
  { 16, 64, 512, 4096, },
  { 1, 16, 64, 512, 4096, },
  { 2, 16, 64, 512, 4096, },
  { 1, 2, 16, 64, 512, 4096, },
  { 4, 16, 64, 512, 4096, },
  { 1, 4, 16, 64, 512, 4096, },
  { 2, 4, 16, 64, 512, 4096, },
  { 1, 2, 4, 16, 64, 512, 4096, },
  { 8, 16, 64, 512, 4096, },
  { 1, 8, 16, 64, 512, 4096, },
  { 2, 8, 16, 64, 512, 4096, },
  { 1, 2, 8, 16, 64, 512, 4096, },
  { 4, 8, 16, 64, 512, 4096, },
  { 1, 4, 8, 16, 64, 512, 4096, },
  { 2, 4, 8, 16, 64, 512, 4096, },
  { 1, 2, 4, 8, 16, 64, 512, 4096, },
  { 32, 64, 512, 4096, },
  { 1, 32, 64, 512, 4096, },
  { 2, 32, 64, 512, 4096, },
  { 1, 2, 32, 64, 512, 4096, },
  { 4, 32, 64, 512, 4096, },
  { 1, 4, 32, 64, 512, 4096, },
  { 2, 4, 32, 64, 512, 4096, },
  { 1, 2, 4, 32, 64, 512, 4096, },
  { 8, 32, 64, 512, 4096, },
  { 1, 8, 32, 64, 512, 4096, },
  { 2, 8, 32, 64, 512, 4096, },
  { 1, 2, 8, 32, 64, 512, 4096, },
  { 4, 8, 32, 64, 512, 4096, },
  { 1, 4, 8, 32, 64, 512, 4096, },
  { 2, 4, 8, 32, 64, 512, 4096, },
  { 1, 2, 4, 8, 32, 64, 512, 4096, },
  { 16, 32, 64, 512, 4096, },
  { 1, 16, 32, 64, 512, 4096, },
  { 2, 16, 32, 64, 512, 4096, },
  { 1, 2, 16, 32, 64, 512, 4096, },
  { 4, 16, 32, 64, 512, 4096, },
  { 1, 4, 16, 32, 64, 512, 4096, },
  { 2, 4, 16, 32, 64, 512, 4096, },
  { 1, 2, 4, 16, 32, 64, 512, 4096, },
  { 8, 16, 32, 64, 512, 4096, },
  { 1, 8, 16, 32, 64, 512, 4096, },
  { 2, 8, 16, 32, 64, 512, 4096, },
  { 1, 2, 8, 16, 32, 64, 512, 4096, },
  { 4, 8, 16, 32, 64, 512, 4096, },
  { 1, 4, 8, 16, 32, 64, 512, 4096, },
  { 2, 4, 8, 16, 32, 64, 512, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 4096, },
  { 128, 512, 4096, },
  { 1, 128, 512, 4096, },
  { 2, 128, 512, 4096, },
  { 1, 2, 128, 512, 4096, },
  { 4, 128, 512, 4096, },
  { 1, 4, 128, 512, 4096, },
  { 2, 4, 128, 512, 4096, },
  { 1, 2, 4, 128, 512, 4096, },
  { 8, 128, 512, 4096, },
  { 1, 8, 128, 512, 4096, },
  { 2, 8, 128, 512, 4096, },
  { 1, 2, 8, 128, 512, 4096, },
  { 4, 8, 128, 512, 4096, },
  { 1, 4, 8, 128, 512, 4096, },
  { 2, 4, 8, 128, 512, 4096, },
  { 1, 2, 4, 8, 128, 512, 4096, },
  { 16, 128, 512, 4096, },
  { 1, 16, 128, 512, 4096, },
  { 2, 16, 128, 512, 4096, },
  { 1, 2, 16, 128, 512, 4096, },
  { 4, 16, 128, 512, 4096, },
  { 1, 4, 16, 128, 512, 4096, },
  { 2, 4, 16, 128, 512, 4096, },
  { 1, 2, 4, 16, 128, 512, 4096, },
  { 8, 16, 128, 512, 4096, },
  { 1, 8, 16, 128, 512, 4096, },
  { 2, 8, 16, 128, 512, 4096, },
  { 1, 2, 8, 16, 128, 512, 4096, },
  { 4, 8, 16, 128, 512, 4096, },
  { 1, 4, 8, 16, 128, 512, 4096, },
  { 2, 4, 8, 16, 128, 512, 4096, },
  { 1, 2, 4, 8, 16, 128, 512, 4096, },
  { 32, 128, 512, 4096, },
  { 1, 32, 128, 512, 4096, },
  { 2, 32, 128, 512, 4096, },
  { 1, 2, 32, 128, 512, 4096, },
  { 4, 32, 128, 512, 4096, },
  { 1, 4, 32, 128, 512, 4096, },
  { 2, 4, 32, 128, 512, 4096, },
  { 1, 2, 4, 32, 128, 512, 4096, },
  { 8, 32, 128, 512, 4096, },
  { 1, 8, 32, 128, 512, 4096, },
  { 2, 8, 32, 128, 512, 4096, },
  { 1, 2, 8, 32, 128, 512, 4096, },
  { 4, 8, 32, 128, 512, 4096, },
  { 1, 4, 8, 32, 128, 512, 4096, },
  { 2, 4, 8, 32, 128, 512, 4096, },
  { 1, 2, 4, 8, 32, 128, 512, 4096, },
  { 16, 32, 128, 512, 4096, },
  { 1, 16, 32, 128, 512, 4096, },
  { 2, 16, 32, 128, 512, 4096, },
  { 1, 2, 16, 32, 128, 512, 4096, },
  { 4, 16, 32, 128, 512, 4096, },
  { 1, 4, 16, 32, 128, 512, 4096, },
  { 2, 4, 16, 32, 128, 512, 4096, },
  { 1, 2, 4, 16, 32, 128, 512, 4096, },
  { 8, 16, 32, 128, 512, 4096, },
  { 1, 8, 16, 32, 128, 512, 4096, },
  { 2, 8, 16, 32, 128, 512, 4096, },
  { 1, 2, 8, 16, 32, 128, 512, 4096, },
  { 4, 8, 16, 32, 128, 512, 4096, },
  { 1, 4, 8, 16, 32, 128, 512, 4096, },
  { 2, 4, 8, 16, 32, 128, 512, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 4096, },
  { 64, 128, 512, 4096, },
  { 1, 64, 128, 512, 4096, },
  { 2, 64, 128, 512, 4096, },
  { 1, 2, 64, 128, 512, 4096, },
  { 4, 64, 128, 512, 4096, },
  { 1, 4, 64, 128, 512, 4096, },
  { 2, 4, 64, 128, 512, 4096, },
  { 1, 2, 4, 64, 128, 512, 4096, },
  { 8, 64, 128, 512, 4096, },
  { 1, 8, 64, 128, 512, 4096, },
  { 2, 8, 64, 128, 512, 4096, },
  { 1, 2, 8, 64, 128, 512, 4096, },
  { 4, 8, 64, 128, 512, 4096, },
  { 1, 4, 8, 64, 128, 512, 4096, },
  { 2, 4, 8, 64, 128, 512, 4096, },
  { 1, 2, 4, 8, 64, 128, 512, 4096, },
  { 16, 64, 128, 512, 4096, },
  { 1, 16, 64, 128, 512, 4096, },
  { 2, 16, 64, 128, 512, 4096, },
  { 1, 2, 16, 64, 128, 512, 4096, },
  { 4, 16, 64, 128, 512, 4096, },
  { 1, 4, 16, 64, 128, 512, 4096, },
  { 2, 4, 16, 64, 128, 512, 4096, },
  { 1, 2, 4, 16, 64, 128, 512, 4096, },
  { 8, 16, 64, 128, 512, 4096, },
  { 1, 8, 16, 64, 128, 512, 4096, },
  { 2, 8, 16, 64, 128, 512, 4096, },
  { 1, 2, 8, 16, 64, 128, 512, 4096, },
  { 4, 8, 16, 64, 128, 512, 4096, },
  { 1, 4, 8, 16, 64, 128, 512, 4096, },
  { 2, 4, 8, 16, 64, 128, 512, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 4096, },
  { 32, 64, 128, 512, 4096, },
  { 1, 32, 64, 128, 512, 4096, },
  { 2, 32, 64, 128, 512, 4096, },
  { 1, 2, 32, 64, 128, 512, 4096, },
  { 4, 32, 64, 128, 512, 4096, },
  { 1, 4, 32, 64, 128, 512, 4096, },
  { 2, 4, 32, 64, 128, 512, 4096, },
  { 1, 2, 4, 32, 64, 128, 512, 4096, },
  { 8, 32, 64, 128, 512, 4096, },
  { 1, 8, 32, 64, 128, 512, 4096, },
  { 2, 8, 32, 64, 128, 512, 4096, },
  { 1, 2, 8, 32, 64, 128, 512, 4096, },
  { 4, 8, 32, 64, 128, 512, 4096, },
  { 1, 4, 8, 32, 64, 128, 512, 4096, },
  { 2, 4, 8, 32, 64, 128, 512, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 4096, },
  { 16, 32, 64, 128, 512, 4096, },
  { 1, 16, 32, 64, 128, 512, 4096, },
  { 2, 16, 32, 64, 128, 512, 4096, },
  { 1, 2, 16, 32, 64, 128, 512, 4096, },
  { 4, 16, 32, 64, 128, 512, 4096, },
  { 1, 4, 16, 32, 64, 128, 512, 4096, },
  { 2, 4, 16, 32, 64, 128, 512, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 4096, },
  { 8, 16, 32, 64, 128, 512, 4096, },
  { 1, 8, 16, 32, 64, 128, 512, 4096, },
  { 2, 8, 16, 32, 64, 128, 512, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 4096, },
  { 4, 8, 16, 32, 64, 128, 512, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 4096, },
  { 256, 512, 4096, },
  { 1, 256, 512, 4096, },
  { 2, 256, 512, 4096, },
  { 1, 2, 256, 512, 4096, },
  { 4, 256, 512, 4096, },
  { 1, 4, 256, 512, 4096, },
  { 2, 4, 256, 512, 4096, },
  { 1, 2, 4, 256, 512, 4096, },
  { 8, 256, 512, 4096, },
  { 1, 8, 256, 512, 4096, },
  { 2, 8, 256, 512, 4096, },
  { 1, 2, 8, 256, 512, 4096, },
  { 4, 8, 256, 512, 4096, },
  { 1, 4, 8, 256, 512, 4096, },
  { 2, 4, 8, 256, 512, 4096, },
  { 1, 2, 4, 8, 256, 512, 4096, },
  { 16, 256, 512, 4096, },
  { 1, 16, 256, 512, 4096, },
  { 2, 16, 256, 512, 4096, },
  { 1, 2, 16, 256, 512, 4096, },
  { 4, 16, 256, 512, 4096, },
  { 1, 4, 16, 256, 512, 4096, },
  { 2, 4, 16, 256, 512, 4096, },
  { 1, 2, 4, 16, 256, 512, 4096, },
  { 8, 16, 256, 512, 4096, },
  { 1, 8, 16, 256, 512, 4096, },
  { 2, 8, 16, 256, 512, 4096, },
  { 1, 2, 8, 16, 256, 512, 4096, },
  { 4, 8, 16, 256, 512, 4096, },
  { 1, 4, 8, 16, 256, 512, 4096, },
  { 2, 4, 8, 16, 256, 512, 4096, },
  { 1, 2, 4, 8, 16, 256, 512, 4096, },
  { 32, 256, 512, 4096, },
  { 1, 32, 256, 512, 4096, },
  { 2, 32, 256, 512, 4096, },
  { 1, 2, 32, 256, 512, 4096, },
  { 4, 32, 256, 512, 4096, },
  { 1, 4, 32, 256, 512, 4096, },
  { 2, 4, 32, 256, 512, 4096, },
  { 1, 2, 4, 32, 256, 512, 4096, },
  { 8, 32, 256, 512, 4096, },
  { 1, 8, 32, 256, 512, 4096, },
  { 2, 8, 32, 256, 512, 4096, },
  { 1, 2, 8, 32, 256, 512, 4096, },
  { 4, 8, 32, 256, 512, 4096, },
  { 1, 4, 8, 32, 256, 512, 4096, },
  { 2, 4, 8, 32, 256, 512, 4096, },
  { 1, 2, 4, 8, 32, 256, 512, 4096, },
  { 16, 32, 256, 512, 4096, },
  { 1, 16, 32, 256, 512, 4096, },
  { 2, 16, 32, 256, 512, 4096, },
  { 1, 2, 16, 32, 256, 512, 4096, },
  { 4, 16, 32, 256, 512, 4096, },
  { 1, 4, 16, 32, 256, 512, 4096, },
  { 2, 4, 16, 32, 256, 512, 4096, },
  { 1, 2, 4, 16, 32, 256, 512, 4096, },
  { 8, 16, 32, 256, 512, 4096, },
  { 1, 8, 16, 32, 256, 512, 4096, },
  { 2, 8, 16, 32, 256, 512, 4096, },
  { 1, 2, 8, 16, 32, 256, 512, 4096, },
  { 4, 8, 16, 32, 256, 512, 4096, },
  { 1, 4, 8, 16, 32, 256, 512, 4096, },
  { 2, 4, 8, 16, 32, 256, 512, 4096, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 4096, },
  { 64, 256, 512, 4096, },
  { 1, 64, 256, 512, 4096, },
  { 2, 64, 256, 512, 4096, },
  { 1, 2, 64, 256, 512, 4096, },
  { 4, 64, 256, 512, 4096, },
  { 1, 4, 64, 256, 512, 4096, },
  { 2, 4, 64, 256, 512, 4096, },
  { 1, 2, 4, 64, 256, 512, 4096, },
  { 8, 64, 256, 512, 4096, },
  { 1, 8, 64, 256, 512, 4096, },
  { 2, 8, 64, 256, 512, 4096, },
  { 1, 2, 8, 64, 256, 512, 4096, },
  { 4, 8, 64, 256, 512, 4096, },
  { 1, 4, 8, 64, 256, 512, 4096, },
  { 2, 4, 8, 64, 256, 512, 4096, },
  { 1, 2, 4, 8, 64, 256, 512, 4096, },
  { 16, 64, 256, 512, 4096, },
  { 1, 16, 64, 256, 512, 4096, },
  { 2, 16, 64, 256, 512, 4096, },
  { 1, 2, 16, 64, 256, 512, 4096, },
  { 4, 16, 64, 256, 512, 4096, },
  { 1, 4, 16, 64, 256, 512, 4096, },
  { 2, 4, 16, 64, 256, 512, 4096, },
  { 1, 2, 4, 16, 64, 256, 512, 4096, },
  { 8, 16, 64, 256, 512, 4096, },
  { 1, 8, 16, 64, 256, 512, 4096, },
  { 2, 8, 16, 64, 256, 512, 4096, },
  { 1, 2, 8, 16, 64, 256, 512, 4096, },
  { 4, 8, 16, 64, 256, 512, 4096, },
  { 1, 4, 8, 16, 64, 256, 512, 4096, },
  { 2, 4, 8, 16, 64, 256, 512, 4096, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 4096, },
  { 32, 64, 256, 512, 4096, },
  { 1, 32, 64, 256, 512, 4096, },
  { 2, 32, 64, 256, 512, 4096, },
  { 1, 2, 32, 64, 256, 512, 4096, },
  { 4, 32, 64, 256, 512, 4096, },
  { 1, 4, 32, 64, 256, 512, 4096, },
  { 2, 4, 32, 64, 256, 512, 4096, },
  { 1, 2, 4, 32, 64, 256, 512, 4096, },
  { 8, 32, 64, 256, 512, 4096, },
  { 1, 8, 32, 64, 256, 512, 4096, },
  { 2, 8, 32, 64, 256, 512, 4096, },
  { 1, 2, 8, 32, 64, 256, 512, 4096, },
  { 4, 8, 32, 64, 256, 512, 4096, },
  { 1, 4, 8, 32, 64, 256, 512, 4096, },
  { 2, 4, 8, 32, 64, 256, 512, 4096, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 4096, },
  { 16, 32, 64, 256, 512, 4096, },
  { 1, 16, 32, 64, 256, 512, 4096, },
  { 2, 16, 32, 64, 256, 512, 4096, },
  { 1, 2, 16, 32, 64, 256, 512, 4096, },
  { 4, 16, 32, 64, 256, 512, 4096, },
  { 1, 4, 16, 32, 64, 256, 512, 4096, },
  { 2, 4, 16, 32, 64, 256, 512, 4096, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 4096, },
  { 8, 16, 32, 64, 256, 512, 4096, },
  { 1, 8, 16, 32, 64, 256, 512, 4096, },
  { 2, 8, 16, 32, 64, 256, 512, 4096, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 4096, },
  { 4, 8, 16, 32, 64, 256, 512, 4096, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 4096, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 4096, },
  { 128, 256, 512, 4096, },
  { 1, 128, 256, 512, 4096, },
  { 2, 128, 256, 512, 4096, },
  { 1, 2, 128, 256, 512, 4096, },
  { 4, 128, 256, 512, 4096, },
  { 1, 4, 128, 256, 512, 4096, },
  { 2, 4, 128, 256, 512, 4096, },
  { 1, 2, 4, 128, 256, 512, 4096, },
  { 8, 128, 256, 512, 4096, },
  { 1, 8, 128, 256, 512, 4096, },
  { 2, 8, 128, 256, 512, 4096, },
  { 1, 2, 8, 128, 256, 512, 4096, },
  { 4, 8, 128, 256, 512, 4096, },
  { 1, 4, 8, 128, 256, 512, 4096, },
  { 2, 4, 8, 128, 256, 512, 4096, },
  { 1, 2, 4, 8, 128, 256, 512, 4096, },
  { 16, 128, 256, 512, 4096, },
  { 1, 16, 128, 256, 512, 4096, },
  { 2, 16, 128, 256, 512, 4096, },
  { 1, 2, 16, 128, 256, 512, 4096, },
  { 4, 16, 128, 256, 512, 4096, },
  { 1, 4, 16, 128, 256, 512, 4096, },
  { 2, 4, 16, 128, 256, 512, 4096, },
  { 1, 2, 4, 16, 128, 256, 512, 4096, },
  { 8, 16, 128, 256, 512, 4096, },
  { 1, 8, 16, 128, 256, 512, 4096, },
  { 2, 8, 16, 128, 256, 512, 4096, },
  { 1, 2, 8, 16, 128, 256, 512, 4096, },
  { 4, 8, 16, 128, 256, 512, 4096, },
  { 1, 4, 8, 16, 128, 256, 512, 4096, },
  { 2, 4, 8, 16, 128, 256, 512, 4096, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 4096, },
  { 32, 128, 256, 512, 4096, },
  { 1, 32, 128, 256, 512, 4096, },
  { 2, 32, 128, 256, 512, 4096, },
  { 1, 2, 32, 128, 256, 512, 4096, },
  { 4, 32, 128, 256, 512, 4096, },
  { 1, 4, 32, 128, 256, 512, 4096, },
  { 2, 4, 32, 128, 256, 512, 4096, },
  { 1, 2, 4, 32, 128, 256, 512, 4096, },
  { 8, 32, 128, 256, 512, 4096, },
  { 1, 8, 32, 128, 256, 512, 4096, },
  { 2, 8, 32, 128, 256, 512, 4096, },
  { 1, 2, 8, 32, 128, 256, 512, 4096, },
  { 4, 8, 32, 128, 256, 512, 4096, },
  { 1, 4, 8, 32, 128, 256, 512, 4096, },
  { 2, 4, 8, 32, 128, 256, 512, 4096, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 4096, },
  { 16, 32, 128, 256, 512, 4096, },
  { 1, 16, 32, 128, 256, 512, 4096, },
  { 2, 16, 32, 128, 256, 512, 4096, },
  { 1, 2, 16, 32, 128, 256, 512, 4096, },
  { 4, 16, 32, 128, 256, 512, 4096, },
  { 1, 4, 16, 32, 128, 256, 512, 4096, },
  { 2, 4, 16, 32, 128, 256, 512, 4096, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 4096, },
  { 8, 16, 32, 128, 256, 512, 4096, },
  { 1, 8, 16, 32, 128, 256, 512, 4096, },
  { 2, 8, 16, 32, 128, 256, 512, 4096, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 4096, },
  { 4, 8, 16, 32, 128, 256, 512, 4096, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 4096, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 4096, },
  { 64, 128, 256, 512, 4096, },
  { 1, 64, 128, 256, 512, 4096, },
  { 2, 64, 128, 256, 512, 4096, },
  { 1, 2, 64, 128, 256, 512, 4096, },
  { 4, 64, 128, 256, 512, 4096, },
  { 1, 4, 64, 128, 256, 512, 4096, },
  { 2, 4, 64, 128, 256, 512, 4096, },
  { 1, 2, 4, 64, 128, 256, 512, 4096, },
  { 8, 64, 128, 256, 512, 4096, },
  { 1, 8, 64, 128, 256, 512, 4096, },
  { 2, 8, 64, 128, 256, 512, 4096, },
  { 1, 2, 8, 64, 128, 256, 512, 4096, },
  { 4, 8, 64, 128, 256, 512, 4096, },
  { 1, 4, 8, 64, 128, 256, 512, 4096, },
  { 2, 4, 8, 64, 128, 256, 512, 4096, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 4096, },
  { 16, 64, 128, 256, 512, 4096, },
  { 1, 16, 64, 128, 256, 512, 4096, },
  { 2, 16, 64, 128, 256, 512, 4096, },
  { 1, 2, 16, 64, 128, 256, 512, 4096, },
  { 4, 16, 64, 128, 256, 512, 4096, },
  { 1, 4, 16, 64, 128, 256, 512, 4096, },
  { 2, 4, 16, 64, 128, 256, 512, 4096, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 4096, },
  { 8, 16, 64, 128, 256, 512, 4096, },
  { 1, 8, 16, 64, 128, 256, 512, 4096, },
  { 2, 8, 16, 64, 128, 256, 512, 4096, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 4096, },
  { 4, 8, 16, 64, 128, 256, 512, 4096, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 4096, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 4096, },
  { 32, 64, 128, 256, 512, 4096, },
  { 1, 32, 64, 128, 256, 512, 4096, },
  { 2, 32, 64, 128, 256, 512, 4096, },
  { 1, 2, 32, 64, 128, 256, 512, 4096, },
  { 4, 32, 64, 128, 256, 512, 4096, },
  { 1, 4, 32, 64, 128, 256, 512, 4096, },
  { 2, 4, 32, 64, 128, 256, 512, 4096, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 4096, },
  { 8, 32, 64, 128, 256, 512, 4096, },
  { 1, 8, 32, 64, 128, 256, 512, 4096, },
  { 2, 8, 32, 64, 128, 256, 512, 4096, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 4096, },
  { 4, 8, 32, 64, 128, 256, 512, 4096, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 4096, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 4096, },
  { 16, 32, 64, 128, 256, 512, 4096, },
  { 1, 16, 32, 64, 128, 256, 512, 4096, },
  { 2, 16, 32, 64, 128, 256, 512, 4096, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 4096, },
  { 4, 16, 32, 64, 128, 256, 512, 4096, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 4096, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 4096, },
  { 8, 16, 32, 64, 128, 256, 512, 4096, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 4096, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 4096, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, },
  { 1024, 4096, },
  { 1, 1024, 4096, },
  { 2, 1024, 4096, },
  { 1, 2, 1024, 4096, },
  { 4, 1024, 4096, },
  { 1, 4, 1024, 4096, },
  { 2, 4, 1024, 4096, },
  { 1, 2, 4, 1024, 4096, },
  { 8, 1024, 4096, },
  { 1, 8, 1024, 4096, },
  { 2, 8, 1024, 4096, },
  { 1, 2, 8, 1024, 4096, },
  { 4, 8, 1024, 4096, },
  { 1, 4, 8, 1024, 4096, },
  { 2, 4, 8, 1024, 4096, },
  { 1, 2, 4, 8, 1024, 4096, },
  { 16, 1024, 4096, },
  { 1, 16, 1024, 4096, },
  { 2, 16, 1024, 4096, },
  { 1, 2, 16, 1024, 4096, },
  { 4, 16, 1024, 4096, },
  { 1, 4, 16, 1024, 4096, },
  { 2, 4, 16, 1024, 4096, },
  { 1, 2, 4, 16, 1024, 4096, },
  { 8, 16, 1024, 4096, },
  { 1, 8, 16, 1024, 4096, },
  { 2, 8, 16, 1024, 4096, },
  { 1, 2, 8, 16, 1024, 4096, },
  { 4, 8, 16, 1024, 4096, },
  { 1, 4, 8, 16, 1024, 4096, },
  { 2, 4, 8, 16, 1024, 4096, },
  { 1, 2, 4, 8, 16, 1024, 4096, },
  { 32, 1024, 4096, },
  { 1, 32, 1024, 4096, },
  { 2, 32, 1024, 4096, },
  { 1, 2, 32, 1024, 4096, },
  { 4, 32, 1024, 4096, },
  { 1, 4, 32, 1024, 4096, },
  { 2, 4, 32, 1024, 4096, },
  { 1, 2, 4, 32, 1024, 4096, },
  { 8, 32, 1024, 4096, },
  { 1, 8, 32, 1024, 4096, },
  { 2, 8, 32, 1024, 4096, },
  { 1, 2, 8, 32, 1024, 4096, },
  { 4, 8, 32, 1024, 4096, },
  { 1, 4, 8, 32, 1024, 4096, },
  { 2, 4, 8, 32, 1024, 4096, },
  { 1, 2, 4, 8, 32, 1024, 4096, },
  { 16, 32, 1024, 4096, },
  { 1, 16, 32, 1024, 4096, },
  { 2, 16, 32, 1024, 4096, },
  { 1, 2, 16, 32, 1024, 4096, },
  { 4, 16, 32, 1024, 4096, },
  { 1, 4, 16, 32, 1024, 4096, },
  { 2, 4, 16, 32, 1024, 4096, },
  { 1, 2, 4, 16, 32, 1024, 4096, },
  { 8, 16, 32, 1024, 4096, },
  { 1, 8, 16, 32, 1024, 4096, },
  { 2, 8, 16, 32, 1024, 4096, },
  { 1, 2, 8, 16, 32, 1024, 4096, },
  { 4, 8, 16, 32, 1024, 4096, },
  { 1, 4, 8, 16, 32, 1024, 4096, },
  { 2, 4, 8, 16, 32, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 1024, 4096, },
  { 64, 1024, 4096, },
  { 1, 64, 1024, 4096, },
  { 2, 64, 1024, 4096, },
  { 1, 2, 64, 1024, 4096, },
  { 4, 64, 1024, 4096, },
  { 1, 4, 64, 1024, 4096, },
  { 2, 4, 64, 1024, 4096, },
  { 1, 2, 4, 64, 1024, 4096, },
  { 8, 64, 1024, 4096, },
  { 1, 8, 64, 1024, 4096, },
  { 2, 8, 64, 1024, 4096, },
  { 1, 2, 8, 64, 1024, 4096, },
  { 4, 8, 64, 1024, 4096, },
  { 1, 4, 8, 64, 1024, 4096, },
  { 2, 4, 8, 64, 1024, 4096, },
  { 1, 2, 4, 8, 64, 1024, 4096, },
  { 16, 64, 1024, 4096, },
  { 1, 16, 64, 1024, 4096, },
  { 2, 16, 64, 1024, 4096, },
  { 1, 2, 16, 64, 1024, 4096, },
  { 4, 16, 64, 1024, 4096, },
  { 1, 4, 16, 64, 1024, 4096, },
  { 2, 4, 16, 64, 1024, 4096, },
  { 1, 2, 4, 16, 64, 1024, 4096, },
  { 8, 16, 64, 1024, 4096, },
  { 1, 8, 16, 64, 1024, 4096, },
  { 2, 8, 16, 64, 1024, 4096, },
  { 1, 2, 8, 16, 64, 1024, 4096, },
  { 4, 8, 16, 64, 1024, 4096, },
  { 1, 4, 8, 16, 64, 1024, 4096, },
  { 2, 4, 8, 16, 64, 1024, 4096, },
  { 1, 2, 4, 8, 16, 64, 1024, 4096, },
  { 32, 64, 1024, 4096, },
  { 1, 32, 64, 1024, 4096, },
  { 2, 32, 64, 1024, 4096, },
  { 1, 2, 32, 64, 1024, 4096, },
  { 4, 32, 64, 1024, 4096, },
  { 1, 4, 32, 64, 1024, 4096, },
  { 2, 4, 32, 64, 1024, 4096, },
  { 1, 2, 4, 32, 64, 1024, 4096, },
  { 8, 32, 64, 1024, 4096, },
  { 1, 8, 32, 64, 1024, 4096, },
  { 2, 8, 32, 64, 1024, 4096, },
  { 1, 2, 8, 32, 64, 1024, 4096, },
  { 4, 8, 32, 64, 1024, 4096, },
  { 1, 4, 8, 32, 64, 1024, 4096, },
  { 2, 4, 8, 32, 64, 1024, 4096, },
  { 1, 2, 4, 8, 32, 64, 1024, 4096, },
  { 16, 32, 64, 1024, 4096, },
  { 1, 16, 32, 64, 1024, 4096, },
  { 2, 16, 32, 64, 1024, 4096, },
  { 1, 2, 16, 32, 64, 1024, 4096, },
  { 4, 16, 32, 64, 1024, 4096, },
  { 1, 4, 16, 32, 64, 1024, 4096, },
  { 2, 4, 16, 32, 64, 1024, 4096, },
  { 1, 2, 4, 16, 32, 64, 1024, 4096, },
  { 8, 16, 32, 64, 1024, 4096, },
  { 1, 8, 16, 32, 64, 1024, 4096, },
  { 2, 8, 16, 32, 64, 1024, 4096, },
  { 1, 2, 8, 16, 32, 64, 1024, 4096, },
  { 4, 8, 16, 32, 64, 1024, 4096, },
  { 1, 4, 8, 16, 32, 64, 1024, 4096, },
  { 2, 4, 8, 16, 32, 64, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 4096, },
  { 128, 1024, 4096, },
  { 1, 128, 1024, 4096, },
  { 2, 128, 1024, 4096, },
  { 1, 2, 128, 1024, 4096, },
  { 4, 128, 1024, 4096, },
  { 1, 4, 128, 1024, 4096, },
  { 2, 4, 128, 1024, 4096, },
  { 1, 2, 4, 128, 1024, 4096, },
  { 8, 128, 1024, 4096, },
  { 1, 8, 128, 1024, 4096, },
  { 2, 8, 128, 1024, 4096, },
  { 1, 2, 8, 128, 1024, 4096, },
  { 4, 8, 128, 1024, 4096, },
  { 1, 4, 8, 128, 1024, 4096, },
  { 2, 4, 8, 128, 1024, 4096, },
  { 1, 2, 4, 8, 128, 1024, 4096, },
  { 16, 128, 1024, 4096, },
  { 1, 16, 128, 1024, 4096, },
  { 2, 16, 128, 1024, 4096, },
  { 1, 2, 16, 128, 1024, 4096, },
  { 4, 16, 128, 1024, 4096, },
  { 1, 4, 16, 128, 1024, 4096, },
  { 2, 4, 16, 128, 1024, 4096, },
  { 1, 2, 4, 16, 128, 1024, 4096, },
  { 8, 16, 128, 1024, 4096, },
  { 1, 8, 16, 128, 1024, 4096, },
  { 2, 8, 16, 128, 1024, 4096, },
  { 1, 2, 8, 16, 128, 1024, 4096, },
  { 4, 8, 16, 128, 1024, 4096, },
  { 1, 4, 8, 16, 128, 1024, 4096, },
  { 2, 4, 8, 16, 128, 1024, 4096, },
  { 1, 2, 4, 8, 16, 128, 1024, 4096, },
  { 32, 128, 1024, 4096, },
  { 1, 32, 128, 1024, 4096, },
  { 2, 32, 128, 1024, 4096, },
  { 1, 2, 32, 128, 1024, 4096, },
  { 4, 32, 128, 1024, 4096, },
  { 1, 4, 32, 128, 1024, 4096, },
  { 2, 4, 32, 128, 1024, 4096, },
  { 1, 2, 4, 32, 128, 1024, 4096, },
  { 8, 32, 128, 1024, 4096, },
  { 1, 8, 32, 128, 1024, 4096, },
  { 2, 8, 32, 128, 1024, 4096, },
  { 1, 2, 8, 32, 128, 1024, 4096, },
  { 4, 8, 32, 128, 1024, 4096, },
  { 1, 4, 8, 32, 128, 1024, 4096, },
  { 2, 4, 8, 32, 128, 1024, 4096, },
  { 1, 2, 4, 8, 32, 128, 1024, 4096, },
  { 16, 32, 128, 1024, 4096, },
  { 1, 16, 32, 128, 1024, 4096, },
  { 2, 16, 32, 128, 1024, 4096, },
  { 1, 2, 16, 32, 128, 1024, 4096, },
  { 4, 16, 32, 128, 1024, 4096, },
  { 1, 4, 16, 32, 128, 1024, 4096, },
  { 2, 4, 16, 32, 128, 1024, 4096, },
  { 1, 2, 4, 16, 32, 128, 1024, 4096, },
  { 8, 16, 32, 128, 1024, 4096, },
  { 1, 8, 16, 32, 128, 1024, 4096, },
  { 2, 8, 16, 32, 128, 1024, 4096, },
  { 1, 2, 8, 16, 32, 128, 1024, 4096, },
  { 4, 8, 16, 32, 128, 1024, 4096, },
  { 1, 4, 8, 16, 32, 128, 1024, 4096, },
  { 2, 4, 8, 16, 32, 128, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 4096, },
  { 64, 128, 1024, 4096, },
  { 1, 64, 128, 1024, 4096, },
  { 2, 64, 128, 1024, 4096, },
  { 1, 2, 64, 128, 1024, 4096, },
  { 4, 64, 128, 1024, 4096, },
  { 1, 4, 64, 128, 1024, 4096, },
  { 2, 4, 64, 128, 1024, 4096, },
  { 1, 2, 4, 64, 128, 1024, 4096, },
  { 8, 64, 128, 1024, 4096, },
  { 1, 8, 64, 128, 1024, 4096, },
  { 2, 8, 64, 128, 1024, 4096, },
  { 1, 2, 8, 64, 128, 1024, 4096, },
  { 4, 8, 64, 128, 1024, 4096, },
  { 1, 4, 8, 64, 128, 1024, 4096, },
  { 2, 4, 8, 64, 128, 1024, 4096, },
  { 1, 2, 4, 8, 64, 128, 1024, 4096, },
  { 16, 64, 128, 1024, 4096, },
  { 1, 16, 64, 128, 1024, 4096, },
  { 2, 16, 64, 128, 1024, 4096, },
  { 1, 2, 16, 64, 128, 1024, 4096, },
  { 4, 16, 64, 128, 1024, 4096, },
  { 1, 4, 16, 64, 128, 1024, 4096, },
  { 2, 4, 16, 64, 128, 1024, 4096, },
  { 1, 2, 4, 16, 64, 128, 1024, 4096, },
  { 8, 16, 64, 128, 1024, 4096, },
  { 1, 8, 16, 64, 128, 1024, 4096, },
  { 2, 8, 16, 64, 128, 1024, 4096, },
  { 1, 2, 8, 16, 64, 128, 1024, 4096, },
  { 4, 8, 16, 64, 128, 1024, 4096, },
  { 1, 4, 8, 16, 64, 128, 1024, 4096, },
  { 2, 4, 8, 16, 64, 128, 1024, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 4096, },
  { 32, 64, 128, 1024, 4096, },
  { 1, 32, 64, 128, 1024, 4096, },
  { 2, 32, 64, 128, 1024, 4096, },
  { 1, 2, 32, 64, 128, 1024, 4096, },
  { 4, 32, 64, 128, 1024, 4096, },
  { 1, 4, 32, 64, 128, 1024, 4096, },
  { 2, 4, 32, 64, 128, 1024, 4096, },
  { 1, 2, 4, 32, 64, 128, 1024, 4096, },
  { 8, 32, 64, 128, 1024, 4096, },
  { 1, 8, 32, 64, 128, 1024, 4096, },
  { 2, 8, 32, 64, 128, 1024, 4096, },
  { 1, 2, 8, 32, 64, 128, 1024, 4096, },
  { 4, 8, 32, 64, 128, 1024, 4096, },
  { 1, 4, 8, 32, 64, 128, 1024, 4096, },
  { 2, 4, 8, 32, 64, 128, 1024, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 4096, },
  { 16, 32, 64, 128, 1024, 4096, },
  { 1, 16, 32, 64, 128, 1024, 4096, },
  { 2, 16, 32, 64, 128, 1024, 4096, },
  { 1, 2, 16, 32, 64, 128, 1024, 4096, },
  { 4, 16, 32, 64, 128, 1024, 4096, },
  { 1, 4, 16, 32, 64, 128, 1024, 4096, },
  { 2, 4, 16, 32, 64, 128, 1024, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 4096, },
  { 8, 16, 32, 64, 128, 1024, 4096, },
  { 1, 8, 16, 32, 64, 128, 1024, 4096, },
  { 2, 8, 16, 32, 64, 128, 1024, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 4096, },
  { 4, 8, 16, 32, 64, 128, 1024, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 4096, },
  { 256, 1024, 4096, },
  { 1, 256, 1024, 4096, },
  { 2, 256, 1024, 4096, },
  { 1, 2, 256, 1024, 4096, },
  { 4, 256, 1024, 4096, },
  { 1, 4, 256, 1024, 4096, },
  { 2, 4, 256, 1024, 4096, },
  { 1, 2, 4, 256, 1024, 4096, },
  { 8, 256, 1024, 4096, },
  { 1, 8, 256, 1024, 4096, },
  { 2, 8, 256, 1024, 4096, },
  { 1, 2, 8, 256, 1024, 4096, },
  { 4, 8, 256, 1024, 4096, },
  { 1, 4, 8, 256, 1024, 4096, },
  { 2, 4, 8, 256, 1024, 4096, },
  { 1, 2, 4, 8, 256, 1024, 4096, },
  { 16, 256, 1024, 4096, },
  { 1, 16, 256, 1024, 4096, },
  { 2, 16, 256, 1024, 4096, },
  { 1, 2, 16, 256, 1024, 4096, },
  { 4, 16, 256, 1024, 4096, },
  { 1, 4, 16, 256, 1024, 4096, },
  { 2, 4, 16, 256, 1024, 4096, },
  { 1, 2, 4, 16, 256, 1024, 4096, },
  { 8, 16, 256, 1024, 4096, },
  { 1, 8, 16, 256, 1024, 4096, },
  { 2, 8, 16, 256, 1024, 4096, },
  { 1, 2, 8, 16, 256, 1024, 4096, },
  { 4, 8, 16, 256, 1024, 4096, },
  { 1, 4, 8, 16, 256, 1024, 4096, },
  { 2, 4, 8, 16, 256, 1024, 4096, },
  { 1, 2, 4, 8, 16, 256, 1024, 4096, },
  { 32, 256, 1024, 4096, },
  { 1, 32, 256, 1024, 4096, },
  { 2, 32, 256, 1024, 4096, },
  { 1, 2, 32, 256, 1024, 4096, },
  { 4, 32, 256, 1024, 4096, },
  { 1, 4, 32, 256, 1024, 4096, },
  { 2, 4, 32, 256, 1024, 4096, },
  { 1, 2, 4, 32, 256, 1024, 4096, },
  { 8, 32, 256, 1024, 4096, },
  { 1, 8, 32, 256, 1024, 4096, },
  { 2, 8, 32, 256, 1024, 4096, },
  { 1, 2, 8, 32, 256, 1024, 4096, },
  { 4, 8, 32, 256, 1024, 4096, },
  { 1, 4, 8, 32, 256, 1024, 4096, },
  { 2, 4, 8, 32, 256, 1024, 4096, },
  { 1, 2, 4, 8, 32, 256, 1024, 4096, },
  { 16, 32, 256, 1024, 4096, },
  { 1, 16, 32, 256, 1024, 4096, },
  { 2, 16, 32, 256, 1024, 4096, },
  { 1, 2, 16, 32, 256, 1024, 4096, },
  { 4, 16, 32, 256, 1024, 4096, },
  { 1, 4, 16, 32, 256, 1024, 4096, },
  { 2, 4, 16, 32, 256, 1024, 4096, },
  { 1, 2, 4, 16, 32, 256, 1024, 4096, },
  { 8, 16, 32, 256, 1024, 4096, },
  { 1, 8, 16, 32, 256, 1024, 4096, },
  { 2, 8, 16, 32, 256, 1024, 4096, },
  { 1, 2, 8, 16, 32, 256, 1024, 4096, },
  { 4, 8, 16, 32, 256, 1024, 4096, },
  { 1, 4, 8, 16, 32, 256, 1024, 4096, },
  { 2, 4, 8, 16, 32, 256, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 4096, },
  { 64, 256, 1024, 4096, },
  { 1, 64, 256, 1024, 4096, },
  { 2, 64, 256, 1024, 4096, },
  { 1, 2, 64, 256, 1024, 4096, },
  { 4, 64, 256, 1024, 4096, },
  { 1, 4, 64, 256, 1024, 4096, },
  { 2, 4, 64, 256, 1024, 4096, },
  { 1, 2, 4, 64, 256, 1024, 4096, },
  { 8, 64, 256, 1024, 4096, },
  { 1, 8, 64, 256, 1024, 4096, },
  { 2, 8, 64, 256, 1024, 4096, },
  { 1, 2, 8, 64, 256, 1024, 4096, },
  { 4, 8, 64, 256, 1024, 4096, },
  { 1, 4, 8, 64, 256, 1024, 4096, },
  { 2, 4, 8, 64, 256, 1024, 4096, },
  { 1, 2, 4, 8, 64, 256, 1024, 4096, },
  { 16, 64, 256, 1024, 4096, },
  { 1, 16, 64, 256, 1024, 4096, },
  { 2, 16, 64, 256, 1024, 4096, },
  { 1, 2, 16, 64, 256, 1024, 4096, },
  { 4, 16, 64, 256, 1024, 4096, },
  { 1, 4, 16, 64, 256, 1024, 4096, },
  { 2, 4, 16, 64, 256, 1024, 4096, },
  { 1, 2, 4, 16, 64, 256, 1024, 4096, },
  { 8, 16, 64, 256, 1024, 4096, },
  { 1, 8, 16, 64, 256, 1024, 4096, },
  { 2, 8, 16, 64, 256, 1024, 4096, },
  { 1, 2, 8, 16, 64, 256, 1024, 4096, },
  { 4, 8, 16, 64, 256, 1024, 4096, },
  { 1, 4, 8, 16, 64, 256, 1024, 4096, },
  { 2, 4, 8, 16, 64, 256, 1024, 4096, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 4096, },
  { 32, 64, 256, 1024, 4096, },
  { 1, 32, 64, 256, 1024, 4096, },
  { 2, 32, 64, 256, 1024, 4096, },
  { 1, 2, 32, 64, 256, 1024, 4096, },
  { 4, 32, 64, 256, 1024, 4096, },
  { 1, 4, 32, 64, 256, 1024, 4096, },
  { 2, 4, 32, 64, 256, 1024, 4096, },
  { 1, 2, 4, 32, 64, 256, 1024, 4096, },
  { 8, 32, 64, 256, 1024, 4096, },
  { 1, 8, 32, 64, 256, 1024, 4096, },
  { 2, 8, 32, 64, 256, 1024, 4096, },
  { 1, 2, 8, 32, 64, 256, 1024, 4096, },
  { 4, 8, 32, 64, 256, 1024, 4096, },
  { 1, 4, 8, 32, 64, 256, 1024, 4096, },
  { 2, 4, 8, 32, 64, 256, 1024, 4096, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 4096, },
  { 16, 32, 64, 256, 1024, 4096, },
  { 1, 16, 32, 64, 256, 1024, 4096, },
  { 2, 16, 32, 64, 256, 1024, 4096, },
  { 1, 2, 16, 32, 64, 256, 1024, 4096, },
  { 4, 16, 32, 64, 256, 1024, 4096, },
  { 1, 4, 16, 32, 64, 256, 1024, 4096, },
  { 2, 4, 16, 32, 64, 256, 1024, 4096, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 4096, },
  { 8, 16, 32, 64, 256, 1024, 4096, },
  { 1, 8, 16, 32, 64, 256, 1024, 4096, },
  { 2, 8, 16, 32, 64, 256, 1024, 4096, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 4096, },
  { 4, 8, 16, 32, 64, 256, 1024, 4096, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 4096, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 4096, },
  { 128, 256, 1024, 4096, },
  { 1, 128, 256, 1024, 4096, },
  { 2, 128, 256, 1024, 4096, },
  { 1, 2, 128, 256, 1024, 4096, },
  { 4, 128, 256, 1024, 4096, },
  { 1, 4, 128, 256, 1024, 4096, },
  { 2, 4, 128, 256, 1024, 4096, },
  { 1, 2, 4, 128, 256, 1024, 4096, },
  { 8, 128, 256, 1024, 4096, },
  { 1, 8, 128, 256, 1024, 4096, },
  { 2, 8, 128, 256, 1024, 4096, },
  { 1, 2, 8, 128, 256, 1024, 4096, },
  { 4, 8, 128, 256, 1024, 4096, },
  { 1, 4, 8, 128, 256, 1024, 4096, },
  { 2, 4, 8, 128, 256, 1024, 4096, },
  { 1, 2, 4, 8, 128, 256, 1024, 4096, },
  { 16, 128, 256, 1024, 4096, },
  { 1, 16, 128, 256, 1024, 4096, },
  { 2, 16, 128, 256, 1024, 4096, },
  { 1, 2, 16, 128, 256, 1024, 4096, },
  { 4, 16, 128, 256, 1024, 4096, },
  { 1, 4, 16, 128, 256, 1024, 4096, },
  { 2, 4, 16, 128, 256, 1024, 4096, },
  { 1, 2, 4, 16, 128, 256, 1024, 4096, },
  { 8, 16, 128, 256, 1024, 4096, },
  { 1, 8, 16, 128, 256, 1024, 4096, },
  { 2, 8, 16, 128, 256, 1024, 4096, },
  { 1, 2, 8, 16, 128, 256, 1024, 4096, },
  { 4, 8, 16, 128, 256, 1024, 4096, },
  { 1, 4, 8, 16, 128, 256, 1024, 4096, },
  { 2, 4, 8, 16, 128, 256, 1024, 4096, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 4096, },
  { 32, 128, 256, 1024, 4096, },
  { 1, 32, 128, 256, 1024, 4096, },
  { 2, 32, 128, 256, 1024, 4096, },
  { 1, 2, 32, 128, 256, 1024, 4096, },
  { 4, 32, 128, 256, 1024, 4096, },
  { 1, 4, 32, 128, 256, 1024, 4096, },
  { 2, 4, 32, 128, 256, 1024, 4096, },
  { 1, 2, 4, 32, 128, 256, 1024, 4096, },
  { 8, 32, 128, 256, 1024, 4096, },
  { 1, 8, 32, 128, 256, 1024, 4096, },
  { 2, 8, 32, 128, 256, 1024, 4096, },
  { 1, 2, 8, 32, 128, 256, 1024, 4096, },
  { 4, 8, 32, 128, 256, 1024, 4096, },
  { 1, 4, 8, 32, 128, 256, 1024, 4096, },
  { 2, 4, 8, 32, 128, 256, 1024, 4096, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 4096, },
  { 16, 32, 128, 256, 1024, 4096, },
  { 1, 16, 32, 128, 256, 1024, 4096, },
  { 2, 16, 32, 128, 256, 1024, 4096, },
  { 1, 2, 16, 32, 128, 256, 1024, 4096, },
  { 4, 16, 32, 128, 256, 1024, 4096, },
  { 1, 4, 16, 32, 128, 256, 1024, 4096, },
  { 2, 4, 16, 32, 128, 256, 1024, 4096, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 4096, },
  { 8, 16, 32, 128, 256, 1024, 4096, },
  { 1, 8, 16, 32, 128, 256, 1024, 4096, },
  { 2, 8, 16, 32, 128, 256, 1024, 4096, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 4096, },
  { 4, 8, 16, 32, 128, 256, 1024, 4096, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 4096, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 4096, },
  { 64, 128, 256, 1024, 4096, },
  { 1, 64, 128, 256, 1024, 4096, },
  { 2, 64, 128, 256, 1024, 4096, },
  { 1, 2, 64, 128, 256, 1024, 4096, },
  { 4, 64, 128, 256, 1024, 4096, },
  { 1, 4, 64, 128, 256, 1024, 4096, },
  { 2, 4, 64, 128, 256, 1024, 4096, },
  { 1, 2, 4, 64, 128, 256, 1024, 4096, },
  { 8, 64, 128, 256, 1024, 4096, },
  { 1, 8, 64, 128, 256, 1024, 4096, },
  { 2, 8, 64, 128, 256, 1024, 4096, },
  { 1, 2, 8, 64, 128, 256, 1024, 4096, },
  { 4, 8, 64, 128, 256, 1024, 4096, },
  { 1, 4, 8, 64, 128, 256, 1024, 4096, },
  { 2, 4, 8, 64, 128, 256, 1024, 4096, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 4096, },
  { 16, 64, 128, 256, 1024, 4096, },
  { 1, 16, 64, 128, 256, 1024, 4096, },
  { 2, 16, 64, 128, 256, 1024, 4096, },
  { 1, 2, 16, 64, 128, 256, 1024, 4096, },
  { 4, 16, 64, 128, 256, 1024, 4096, },
  { 1, 4, 16, 64, 128, 256, 1024, 4096, },
  { 2, 4, 16, 64, 128, 256, 1024, 4096, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 4096, },
  { 8, 16, 64, 128, 256, 1024, 4096, },
  { 1, 8, 16, 64, 128, 256, 1024, 4096, },
  { 2, 8, 16, 64, 128, 256, 1024, 4096, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 4096, },
  { 4, 8, 16, 64, 128, 256, 1024, 4096, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 4096, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 4096, },
  { 32, 64, 128, 256, 1024, 4096, },
  { 1, 32, 64, 128, 256, 1024, 4096, },
  { 2, 32, 64, 128, 256, 1024, 4096, },
  { 1, 2, 32, 64, 128, 256, 1024, 4096, },
  { 4, 32, 64, 128, 256, 1024, 4096, },
  { 1, 4, 32, 64, 128, 256, 1024, 4096, },
  { 2, 4, 32, 64, 128, 256, 1024, 4096, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 4096, },
  { 8, 32, 64, 128, 256, 1024, 4096, },
  { 1, 8, 32, 64, 128, 256, 1024, 4096, },
  { 2, 8, 32, 64, 128, 256, 1024, 4096, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 4096, },
  { 4, 8, 32, 64, 128, 256, 1024, 4096, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 4096, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 4096, },
  { 16, 32, 64, 128, 256, 1024, 4096, },
  { 1, 16, 32, 64, 128, 256, 1024, 4096, },
  { 2, 16, 32, 64, 128, 256, 1024, 4096, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 4096, },
  { 4, 16, 32, 64, 128, 256, 1024, 4096, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 4096, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 4096, },
  { 8, 16, 32, 64, 128, 256, 1024, 4096, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 4096, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 4096, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, },
  { 512, 1024, 4096, },
  { 1, 512, 1024, 4096, },
  { 2, 512, 1024, 4096, },
  { 1, 2, 512, 1024, 4096, },
  { 4, 512, 1024, 4096, },
  { 1, 4, 512, 1024, 4096, },
  { 2, 4, 512, 1024, 4096, },
  { 1, 2, 4, 512, 1024, 4096, },
  { 8, 512, 1024, 4096, },
  { 1, 8, 512, 1024, 4096, },
  { 2, 8, 512, 1024, 4096, },
  { 1, 2, 8, 512, 1024, 4096, },
  { 4, 8, 512, 1024, 4096, },
  { 1, 4, 8, 512, 1024, 4096, },
  { 2, 4, 8, 512, 1024, 4096, },
  { 1, 2, 4, 8, 512, 1024, 4096, },
  { 16, 512, 1024, 4096, },
  { 1, 16, 512, 1024, 4096, },
  { 2, 16, 512, 1024, 4096, },
  { 1, 2, 16, 512, 1024, 4096, },
  { 4, 16, 512, 1024, 4096, },
  { 1, 4, 16, 512, 1024, 4096, },
  { 2, 4, 16, 512, 1024, 4096, },
  { 1, 2, 4, 16, 512, 1024, 4096, },
  { 8, 16, 512, 1024, 4096, },
  { 1, 8, 16, 512, 1024, 4096, },
  { 2, 8, 16, 512, 1024, 4096, },
  { 1, 2, 8, 16, 512, 1024, 4096, },
  { 4, 8, 16, 512, 1024, 4096, },
  { 1, 4, 8, 16, 512, 1024, 4096, },
  { 2, 4, 8, 16, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 512, 1024, 4096, },
  { 32, 512, 1024, 4096, },
  { 1, 32, 512, 1024, 4096, },
  { 2, 32, 512, 1024, 4096, },
  { 1, 2, 32, 512, 1024, 4096, },
  { 4, 32, 512, 1024, 4096, },
  { 1, 4, 32, 512, 1024, 4096, },
  { 2, 4, 32, 512, 1024, 4096, },
  { 1, 2, 4, 32, 512, 1024, 4096, },
  { 8, 32, 512, 1024, 4096, },
  { 1, 8, 32, 512, 1024, 4096, },
  { 2, 8, 32, 512, 1024, 4096, },
  { 1, 2, 8, 32, 512, 1024, 4096, },
  { 4, 8, 32, 512, 1024, 4096, },
  { 1, 4, 8, 32, 512, 1024, 4096, },
  { 2, 4, 8, 32, 512, 1024, 4096, },
  { 1, 2, 4, 8, 32, 512, 1024, 4096, },
  { 16, 32, 512, 1024, 4096, },
  { 1, 16, 32, 512, 1024, 4096, },
  { 2, 16, 32, 512, 1024, 4096, },
  { 1, 2, 16, 32, 512, 1024, 4096, },
  { 4, 16, 32, 512, 1024, 4096, },
  { 1, 4, 16, 32, 512, 1024, 4096, },
  { 2, 4, 16, 32, 512, 1024, 4096, },
  { 1, 2, 4, 16, 32, 512, 1024, 4096, },
  { 8, 16, 32, 512, 1024, 4096, },
  { 1, 8, 16, 32, 512, 1024, 4096, },
  { 2, 8, 16, 32, 512, 1024, 4096, },
  { 1, 2, 8, 16, 32, 512, 1024, 4096, },
  { 4, 8, 16, 32, 512, 1024, 4096, },
  { 1, 4, 8, 16, 32, 512, 1024, 4096, },
  { 2, 4, 8, 16, 32, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 4096, },
  { 64, 512, 1024, 4096, },
  { 1, 64, 512, 1024, 4096, },
  { 2, 64, 512, 1024, 4096, },
  { 1, 2, 64, 512, 1024, 4096, },
  { 4, 64, 512, 1024, 4096, },
  { 1, 4, 64, 512, 1024, 4096, },
  { 2, 4, 64, 512, 1024, 4096, },
  { 1, 2, 4, 64, 512, 1024, 4096, },
  { 8, 64, 512, 1024, 4096, },
  { 1, 8, 64, 512, 1024, 4096, },
  { 2, 8, 64, 512, 1024, 4096, },
  { 1, 2, 8, 64, 512, 1024, 4096, },
  { 4, 8, 64, 512, 1024, 4096, },
  { 1, 4, 8, 64, 512, 1024, 4096, },
  { 2, 4, 8, 64, 512, 1024, 4096, },
  { 1, 2, 4, 8, 64, 512, 1024, 4096, },
  { 16, 64, 512, 1024, 4096, },
  { 1, 16, 64, 512, 1024, 4096, },
  { 2, 16, 64, 512, 1024, 4096, },
  { 1, 2, 16, 64, 512, 1024, 4096, },
  { 4, 16, 64, 512, 1024, 4096, },
  { 1, 4, 16, 64, 512, 1024, 4096, },
  { 2, 4, 16, 64, 512, 1024, 4096, },
  { 1, 2, 4, 16, 64, 512, 1024, 4096, },
  { 8, 16, 64, 512, 1024, 4096, },
  { 1, 8, 16, 64, 512, 1024, 4096, },
  { 2, 8, 16, 64, 512, 1024, 4096, },
  { 1, 2, 8, 16, 64, 512, 1024, 4096, },
  { 4, 8, 16, 64, 512, 1024, 4096, },
  { 1, 4, 8, 16, 64, 512, 1024, 4096, },
  { 2, 4, 8, 16, 64, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 4096, },
  { 32, 64, 512, 1024, 4096, },
  { 1, 32, 64, 512, 1024, 4096, },
  { 2, 32, 64, 512, 1024, 4096, },
  { 1, 2, 32, 64, 512, 1024, 4096, },
  { 4, 32, 64, 512, 1024, 4096, },
  { 1, 4, 32, 64, 512, 1024, 4096, },
  { 2, 4, 32, 64, 512, 1024, 4096, },
  { 1, 2, 4, 32, 64, 512, 1024, 4096, },
  { 8, 32, 64, 512, 1024, 4096, },
  { 1, 8, 32, 64, 512, 1024, 4096, },
  { 2, 8, 32, 64, 512, 1024, 4096, },
  { 1, 2, 8, 32, 64, 512, 1024, 4096, },
  { 4, 8, 32, 64, 512, 1024, 4096, },
  { 1, 4, 8, 32, 64, 512, 1024, 4096, },
  { 2, 4, 8, 32, 64, 512, 1024, 4096, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 4096, },
  { 16, 32, 64, 512, 1024, 4096, },
  { 1, 16, 32, 64, 512, 1024, 4096, },
  { 2, 16, 32, 64, 512, 1024, 4096, },
  { 1, 2, 16, 32, 64, 512, 1024, 4096, },
  { 4, 16, 32, 64, 512, 1024, 4096, },
  { 1, 4, 16, 32, 64, 512, 1024, 4096, },
  { 2, 4, 16, 32, 64, 512, 1024, 4096, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 4096, },
  { 8, 16, 32, 64, 512, 1024, 4096, },
  { 1, 8, 16, 32, 64, 512, 1024, 4096, },
  { 2, 8, 16, 32, 64, 512, 1024, 4096, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 4096, },
  { 4, 8, 16, 32, 64, 512, 1024, 4096, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 4096, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 4096, },
  { 128, 512, 1024, 4096, },
  { 1, 128, 512, 1024, 4096, },
  { 2, 128, 512, 1024, 4096, },
  { 1, 2, 128, 512, 1024, 4096, },
  { 4, 128, 512, 1024, 4096, },
  { 1, 4, 128, 512, 1024, 4096, },
  { 2, 4, 128, 512, 1024, 4096, },
  { 1, 2, 4, 128, 512, 1024, 4096, },
  { 8, 128, 512, 1024, 4096, },
  { 1, 8, 128, 512, 1024, 4096, },
  { 2, 8, 128, 512, 1024, 4096, },
  { 1, 2, 8, 128, 512, 1024, 4096, },
  { 4, 8, 128, 512, 1024, 4096, },
  { 1, 4, 8, 128, 512, 1024, 4096, },
  { 2, 4, 8, 128, 512, 1024, 4096, },
  { 1, 2, 4, 8, 128, 512, 1024, 4096, },
  { 16, 128, 512, 1024, 4096, },
  { 1, 16, 128, 512, 1024, 4096, },
  { 2, 16, 128, 512, 1024, 4096, },
  { 1, 2, 16, 128, 512, 1024, 4096, },
  { 4, 16, 128, 512, 1024, 4096, },
  { 1, 4, 16, 128, 512, 1024, 4096, },
  { 2, 4, 16, 128, 512, 1024, 4096, },
  { 1, 2, 4, 16, 128, 512, 1024, 4096, },
  { 8, 16, 128, 512, 1024, 4096, },
  { 1, 8, 16, 128, 512, 1024, 4096, },
  { 2, 8, 16, 128, 512, 1024, 4096, },
  { 1, 2, 8, 16, 128, 512, 1024, 4096, },
  { 4, 8, 16, 128, 512, 1024, 4096, },
  { 1, 4, 8, 16, 128, 512, 1024, 4096, },
  { 2, 4, 8, 16, 128, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 4096, },
  { 32, 128, 512, 1024, 4096, },
  { 1, 32, 128, 512, 1024, 4096, },
  { 2, 32, 128, 512, 1024, 4096, },
  { 1, 2, 32, 128, 512, 1024, 4096, },
  { 4, 32, 128, 512, 1024, 4096, },
  { 1, 4, 32, 128, 512, 1024, 4096, },
  { 2, 4, 32, 128, 512, 1024, 4096, },
  { 1, 2, 4, 32, 128, 512, 1024, 4096, },
  { 8, 32, 128, 512, 1024, 4096, },
  { 1, 8, 32, 128, 512, 1024, 4096, },
  { 2, 8, 32, 128, 512, 1024, 4096, },
  { 1, 2, 8, 32, 128, 512, 1024, 4096, },
  { 4, 8, 32, 128, 512, 1024, 4096, },
  { 1, 4, 8, 32, 128, 512, 1024, 4096, },
  { 2, 4, 8, 32, 128, 512, 1024, 4096, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 4096, },
  { 16, 32, 128, 512, 1024, 4096, },
  { 1, 16, 32, 128, 512, 1024, 4096, },
  { 2, 16, 32, 128, 512, 1024, 4096, },
  { 1, 2, 16, 32, 128, 512, 1024, 4096, },
  { 4, 16, 32, 128, 512, 1024, 4096, },
  { 1, 4, 16, 32, 128, 512, 1024, 4096, },
  { 2, 4, 16, 32, 128, 512, 1024, 4096, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 4096, },
  { 8, 16, 32, 128, 512, 1024, 4096, },
  { 1, 8, 16, 32, 128, 512, 1024, 4096, },
  { 2, 8, 16, 32, 128, 512, 1024, 4096, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 4096, },
  { 4, 8, 16, 32, 128, 512, 1024, 4096, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 4096, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 4096, },
  { 64, 128, 512, 1024, 4096, },
  { 1, 64, 128, 512, 1024, 4096, },
  { 2, 64, 128, 512, 1024, 4096, },
  { 1, 2, 64, 128, 512, 1024, 4096, },
  { 4, 64, 128, 512, 1024, 4096, },
  { 1, 4, 64, 128, 512, 1024, 4096, },
  { 2, 4, 64, 128, 512, 1024, 4096, },
  { 1, 2, 4, 64, 128, 512, 1024, 4096, },
  { 8, 64, 128, 512, 1024, 4096, },
  { 1, 8, 64, 128, 512, 1024, 4096, },
  { 2, 8, 64, 128, 512, 1024, 4096, },
  { 1, 2, 8, 64, 128, 512, 1024, 4096, },
  { 4, 8, 64, 128, 512, 1024, 4096, },
  { 1, 4, 8, 64, 128, 512, 1024, 4096, },
  { 2, 4, 8, 64, 128, 512, 1024, 4096, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 4096, },
  { 16, 64, 128, 512, 1024, 4096, },
  { 1, 16, 64, 128, 512, 1024, 4096, },
  { 2, 16, 64, 128, 512, 1024, 4096, },
  { 1, 2, 16, 64, 128, 512, 1024, 4096, },
  { 4, 16, 64, 128, 512, 1024, 4096, },
  { 1, 4, 16, 64, 128, 512, 1024, 4096, },
  { 2, 4, 16, 64, 128, 512, 1024, 4096, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 4096, },
  { 8, 16, 64, 128, 512, 1024, 4096, },
  { 1, 8, 16, 64, 128, 512, 1024, 4096, },
  { 2, 8, 16, 64, 128, 512, 1024, 4096, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 4096, },
  { 4, 8, 16, 64, 128, 512, 1024, 4096, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 4096, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 4096, },
  { 32, 64, 128, 512, 1024, 4096, },
  { 1, 32, 64, 128, 512, 1024, 4096, },
  { 2, 32, 64, 128, 512, 1024, 4096, },
  { 1, 2, 32, 64, 128, 512, 1024, 4096, },
  { 4, 32, 64, 128, 512, 1024, 4096, },
  { 1, 4, 32, 64, 128, 512, 1024, 4096, },
  { 2, 4, 32, 64, 128, 512, 1024, 4096, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 4096, },
  { 8, 32, 64, 128, 512, 1024, 4096, },
  { 1, 8, 32, 64, 128, 512, 1024, 4096, },
  { 2, 8, 32, 64, 128, 512, 1024, 4096, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 4096, },
  { 4, 8, 32, 64, 128, 512, 1024, 4096, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 4096, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 4096, },
  { 16, 32, 64, 128, 512, 1024, 4096, },
  { 1, 16, 32, 64, 128, 512, 1024, 4096, },
  { 2, 16, 32, 64, 128, 512, 1024, 4096, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 4096, },
  { 4, 16, 32, 64, 128, 512, 1024, 4096, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 4096, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 4096, },
  { 8, 16, 32, 64, 128, 512, 1024, 4096, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 4096, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 4096, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, },
  { 256, 512, 1024, 4096, },
  { 1, 256, 512, 1024, 4096, },
  { 2, 256, 512, 1024, 4096, },
  { 1, 2, 256, 512, 1024, 4096, },
  { 4, 256, 512, 1024, 4096, },
  { 1, 4, 256, 512, 1024, 4096, },
  { 2, 4, 256, 512, 1024, 4096, },
  { 1, 2, 4, 256, 512, 1024, 4096, },
  { 8, 256, 512, 1024, 4096, },
  { 1, 8, 256, 512, 1024, 4096, },
  { 2, 8, 256, 512, 1024, 4096, },
  { 1, 2, 8, 256, 512, 1024, 4096, },
  { 4, 8, 256, 512, 1024, 4096, },
  { 1, 4, 8, 256, 512, 1024, 4096, },
  { 2, 4, 8, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 256, 512, 1024, 4096, },
  { 16, 256, 512, 1024, 4096, },
  { 1, 16, 256, 512, 1024, 4096, },
  { 2, 16, 256, 512, 1024, 4096, },
  { 1, 2, 16, 256, 512, 1024, 4096, },
  { 4, 16, 256, 512, 1024, 4096, },
  { 1, 4, 16, 256, 512, 1024, 4096, },
  { 2, 4, 16, 256, 512, 1024, 4096, },
  { 1, 2, 4, 16, 256, 512, 1024, 4096, },
  { 8, 16, 256, 512, 1024, 4096, },
  { 1, 8, 16, 256, 512, 1024, 4096, },
  { 2, 8, 16, 256, 512, 1024, 4096, },
  { 1, 2, 8, 16, 256, 512, 1024, 4096, },
  { 4, 8, 16, 256, 512, 1024, 4096, },
  { 1, 4, 8, 16, 256, 512, 1024, 4096, },
  { 2, 4, 8, 16, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 4096, },
  { 32, 256, 512, 1024, 4096, },
  { 1, 32, 256, 512, 1024, 4096, },
  { 2, 32, 256, 512, 1024, 4096, },
  { 1, 2, 32, 256, 512, 1024, 4096, },
  { 4, 32, 256, 512, 1024, 4096, },
  { 1, 4, 32, 256, 512, 1024, 4096, },
  { 2, 4, 32, 256, 512, 1024, 4096, },
  { 1, 2, 4, 32, 256, 512, 1024, 4096, },
  { 8, 32, 256, 512, 1024, 4096, },
  { 1, 8, 32, 256, 512, 1024, 4096, },
  { 2, 8, 32, 256, 512, 1024, 4096, },
  { 1, 2, 8, 32, 256, 512, 1024, 4096, },
  { 4, 8, 32, 256, 512, 1024, 4096, },
  { 1, 4, 8, 32, 256, 512, 1024, 4096, },
  { 2, 4, 8, 32, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 4096, },
  { 16, 32, 256, 512, 1024, 4096, },
  { 1, 16, 32, 256, 512, 1024, 4096, },
  { 2, 16, 32, 256, 512, 1024, 4096, },
  { 1, 2, 16, 32, 256, 512, 1024, 4096, },
  { 4, 16, 32, 256, 512, 1024, 4096, },
  { 1, 4, 16, 32, 256, 512, 1024, 4096, },
  { 2, 4, 16, 32, 256, 512, 1024, 4096, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 4096, },
  { 8, 16, 32, 256, 512, 1024, 4096, },
  { 1, 8, 16, 32, 256, 512, 1024, 4096, },
  { 2, 8, 16, 32, 256, 512, 1024, 4096, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 4096, },
  { 4, 8, 16, 32, 256, 512, 1024, 4096, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 4096, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 4096, },
  { 64, 256, 512, 1024, 4096, },
  { 1, 64, 256, 512, 1024, 4096, },
  { 2, 64, 256, 512, 1024, 4096, },
  { 1, 2, 64, 256, 512, 1024, 4096, },
  { 4, 64, 256, 512, 1024, 4096, },
  { 1, 4, 64, 256, 512, 1024, 4096, },
  { 2, 4, 64, 256, 512, 1024, 4096, },
  { 1, 2, 4, 64, 256, 512, 1024, 4096, },
  { 8, 64, 256, 512, 1024, 4096, },
  { 1, 8, 64, 256, 512, 1024, 4096, },
  { 2, 8, 64, 256, 512, 1024, 4096, },
  { 1, 2, 8, 64, 256, 512, 1024, 4096, },
  { 4, 8, 64, 256, 512, 1024, 4096, },
  { 1, 4, 8, 64, 256, 512, 1024, 4096, },
  { 2, 4, 8, 64, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 4096, },
  { 16, 64, 256, 512, 1024, 4096, },
  { 1, 16, 64, 256, 512, 1024, 4096, },
  { 2, 16, 64, 256, 512, 1024, 4096, },
  { 1, 2, 16, 64, 256, 512, 1024, 4096, },
  { 4, 16, 64, 256, 512, 1024, 4096, },
  { 1, 4, 16, 64, 256, 512, 1024, 4096, },
  { 2, 4, 16, 64, 256, 512, 1024, 4096, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 4096, },
  { 8, 16, 64, 256, 512, 1024, 4096, },
  { 1, 8, 16, 64, 256, 512, 1024, 4096, },
  { 2, 8, 16, 64, 256, 512, 1024, 4096, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 4096, },
  { 4, 8, 16, 64, 256, 512, 1024, 4096, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 4096, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 4096, },
  { 32, 64, 256, 512, 1024, 4096, },
  { 1, 32, 64, 256, 512, 1024, 4096, },
  { 2, 32, 64, 256, 512, 1024, 4096, },
  { 1, 2, 32, 64, 256, 512, 1024, 4096, },
  { 4, 32, 64, 256, 512, 1024, 4096, },
  { 1, 4, 32, 64, 256, 512, 1024, 4096, },
  { 2, 4, 32, 64, 256, 512, 1024, 4096, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 4096, },
  { 8, 32, 64, 256, 512, 1024, 4096, },
  { 1, 8, 32, 64, 256, 512, 1024, 4096, },
  { 2, 8, 32, 64, 256, 512, 1024, 4096, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 4096, },
  { 4, 8, 32, 64, 256, 512, 1024, 4096, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 4096, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 4096, },
  { 16, 32, 64, 256, 512, 1024, 4096, },
  { 1, 16, 32, 64, 256, 512, 1024, 4096, },
  { 2, 16, 32, 64, 256, 512, 1024, 4096, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 4096, },
  { 4, 16, 32, 64, 256, 512, 1024, 4096, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 4096, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 4096, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 4096, },
  { 8, 16, 32, 64, 256, 512, 1024, 4096, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 4096, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 4096, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 4096, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 4096, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, },
  { 128, 256, 512, 1024, 4096, },
  { 1, 128, 256, 512, 1024, 4096, },
  { 2, 128, 256, 512, 1024, 4096, },
  { 1, 2, 128, 256, 512, 1024, 4096, },
  { 4, 128, 256, 512, 1024, 4096, },
  { 1, 4, 128, 256, 512, 1024, 4096, },
  { 2, 4, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 128, 256, 512, 1024, 4096, },
  { 8, 128, 256, 512, 1024, 4096, },
  { 1, 8, 128, 256, 512, 1024, 4096, },
  { 2, 8, 128, 256, 512, 1024, 4096, },
  { 1, 2, 8, 128, 256, 512, 1024, 4096, },
  { 4, 8, 128, 256, 512, 1024, 4096, },
  { 1, 4, 8, 128, 256, 512, 1024, 4096, },
  { 2, 4, 8, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 4096, },
  { 16, 128, 256, 512, 1024, 4096, },
  { 1, 16, 128, 256, 512, 1024, 4096, },
  { 2, 16, 128, 256, 512, 1024, 4096, },
  { 1, 2, 16, 128, 256, 512, 1024, 4096, },
  { 4, 16, 128, 256, 512, 1024, 4096, },
  { 1, 4, 16, 128, 256, 512, 1024, 4096, },
  { 2, 4, 16, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 4096, },
  { 8, 16, 128, 256, 512, 1024, 4096, },
  { 1, 8, 16, 128, 256, 512, 1024, 4096, },
  { 2, 8, 16, 128, 256, 512, 1024, 4096, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 4096, },
  { 4, 8, 16, 128, 256, 512, 1024, 4096, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 4096, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 4096, },
  { 32, 128, 256, 512, 1024, 4096, },
  { 1, 32, 128, 256, 512, 1024, 4096, },
  { 2, 32, 128, 256, 512, 1024, 4096, },
  { 1, 2, 32, 128, 256, 512, 1024, 4096, },
  { 4, 32, 128, 256, 512, 1024, 4096, },
  { 1, 4, 32, 128, 256, 512, 1024, 4096, },
  { 2, 4, 32, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 4096, },
  { 8, 32, 128, 256, 512, 1024, 4096, },
  { 1, 8, 32, 128, 256, 512, 1024, 4096, },
  { 2, 8, 32, 128, 256, 512, 1024, 4096, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 4096, },
  { 4, 8, 32, 128, 256, 512, 1024, 4096, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 4096, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 4096, },
  { 16, 32, 128, 256, 512, 1024, 4096, },
  { 1, 16, 32, 128, 256, 512, 1024, 4096, },
  { 2, 16, 32, 128, 256, 512, 1024, 4096, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 4096, },
  { 4, 16, 32, 128, 256, 512, 1024, 4096, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 4096, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 4096, },
  { 8, 16, 32, 128, 256, 512, 1024, 4096, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 4096, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 4096, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 4096, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 4096, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, },
  { 64, 128, 256, 512, 1024, 4096, },
  { 1, 64, 128, 256, 512, 1024, 4096, },
  { 2, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 64, 128, 256, 512, 1024, 4096, },
  { 4, 64, 128, 256, 512, 1024, 4096, },
  { 1, 4, 64, 128, 256, 512, 1024, 4096, },
  { 2, 4, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 4096, },
  { 8, 64, 128, 256, 512, 1024, 4096, },
  { 1, 8, 64, 128, 256, 512, 1024, 4096, },
  { 2, 8, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 4096, },
  { 4, 8, 64, 128, 256, 512, 1024, 4096, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 4096, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 4096, },
  { 16, 64, 128, 256, 512, 1024, 4096, },
  { 1, 16, 64, 128, 256, 512, 1024, 4096, },
  { 2, 16, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 4096, },
  { 4, 16, 64, 128, 256, 512, 1024, 4096, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 4096, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 4096, },
  { 8, 16, 64, 128, 256, 512, 1024, 4096, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 4096, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 4096, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 4096, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, },
  { 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 4096, },
  { 4, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 4096, },
  { 8, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 4096, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, },
  { 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, },
  { 2048, 4096, },
  { 1, 2048, 4096, },
  { 2, 2048, 4096, },
  { 1, 2, 2048, 4096, },
  { 4, 2048, 4096, },
  { 1, 4, 2048, 4096, },
  { 2, 4, 2048, 4096, },
  { 1, 2, 4, 2048, 4096, },
  { 8, 2048, 4096, },
  { 1, 8, 2048, 4096, },
  { 2, 8, 2048, 4096, },
  { 1, 2, 8, 2048, 4096, },
  { 4, 8, 2048, 4096, },
  { 1, 4, 8, 2048, 4096, },
  { 2, 4, 8, 2048, 4096, },
  { 1, 2, 4, 8, 2048, 4096, },
  { 16, 2048, 4096, },
  { 1, 16, 2048, 4096, },
  { 2, 16, 2048, 4096, },
  { 1, 2, 16, 2048, 4096, },
  { 4, 16, 2048, 4096, },
  { 1, 4, 16, 2048, 4096, },
  { 2, 4, 16, 2048, 4096, },
  { 1, 2, 4, 16, 2048, 4096, },
  { 8, 16, 2048, 4096, },
  { 1, 8, 16, 2048, 4096, },
  { 2, 8, 16, 2048, 4096, },
  { 1, 2, 8, 16, 2048, 4096, },
  { 4, 8, 16, 2048, 4096, },
  { 1, 4, 8, 16, 2048, 4096, },
  { 2, 4, 8, 16, 2048, 4096, },
  { 1, 2, 4, 8, 16, 2048, 4096, },
  { 32, 2048, 4096, },
  { 1, 32, 2048, 4096, },
  { 2, 32, 2048, 4096, },
  { 1, 2, 32, 2048, 4096, },
  { 4, 32, 2048, 4096, },
  { 1, 4, 32, 2048, 4096, },
  { 2, 4, 32, 2048, 4096, },
  { 1, 2, 4, 32, 2048, 4096, },
  { 8, 32, 2048, 4096, },
  { 1, 8, 32, 2048, 4096, },
  { 2, 8, 32, 2048, 4096, },
  { 1, 2, 8, 32, 2048, 4096, },
  { 4, 8, 32, 2048, 4096, },
  { 1, 4, 8, 32, 2048, 4096, },
  { 2, 4, 8, 32, 2048, 4096, },
  { 1, 2, 4, 8, 32, 2048, 4096, },
  { 16, 32, 2048, 4096, },
  { 1, 16, 32, 2048, 4096, },
  { 2, 16, 32, 2048, 4096, },
  { 1, 2, 16, 32, 2048, 4096, },
  { 4, 16, 32, 2048, 4096, },
  { 1, 4, 16, 32, 2048, 4096, },
  { 2, 4, 16, 32, 2048, 4096, },
  { 1, 2, 4, 16, 32, 2048, 4096, },
  { 8, 16, 32, 2048, 4096, },
  { 1, 8, 16, 32, 2048, 4096, },
  { 2, 8, 16, 32, 2048, 4096, },
  { 1, 2, 8, 16, 32, 2048, 4096, },
  { 4, 8, 16, 32, 2048, 4096, },
  { 1, 4, 8, 16, 32, 2048, 4096, },
  { 2, 4, 8, 16, 32, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 2048, 4096, },
  { 64, 2048, 4096, },
  { 1, 64, 2048, 4096, },
  { 2, 64, 2048, 4096, },
  { 1, 2, 64, 2048, 4096, },
  { 4, 64, 2048, 4096, },
  { 1, 4, 64, 2048, 4096, },
  { 2, 4, 64, 2048, 4096, },
  { 1, 2, 4, 64, 2048, 4096, },
  { 8, 64, 2048, 4096, },
  { 1, 8, 64, 2048, 4096, },
  { 2, 8, 64, 2048, 4096, },
  { 1, 2, 8, 64, 2048, 4096, },
  { 4, 8, 64, 2048, 4096, },
  { 1, 4, 8, 64, 2048, 4096, },
  { 2, 4, 8, 64, 2048, 4096, },
  { 1, 2, 4, 8, 64, 2048, 4096, },
  { 16, 64, 2048, 4096, },
  { 1, 16, 64, 2048, 4096, },
  { 2, 16, 64, 2048, 4096, },
  { 1, 2, 16, 64, 2048, 4096, },
  { 4, 16, 64, 2048, 4096, },
  { 1, 4, 16, 64, 2048, 4096, },
  { 2, 4, 16, 64, 2048, 4096, },
  { 1, 2, 4, 16, 64, 2048, 4096, },
  { 8, 16, 64, 2048, 4096, },
  { 1, 8, 16, 64, 2048, 4096, },
  { 2, 8, 16, 64, 2048, 4096, },
  { 1, 2, 8, 16, 64, 2048, 4096, },
  { 4, 8, 16, 64, 2048, 4096, },
  { 1, 4, 8, 16, 64, 2048, 4096, },
  { 2, 4, 8, 16, 64, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 2048, 4096, },
  { 32, 64, 2048, 4096, },
  { 1, 32, 64, 2048, 4096, },
  { 2, 32, 64, 2048, 4096, },
  { 1, 2, 32, 64, 2048, 4096, },
  { 4, 32, 64, 2048, 4096, },
  { 1, 4, 32, 64, 2048, 4096, },
  { 2, 4, 32, 64, 2048, 4096, },
  { 1, 2, 4, 32, 64, 2048, 4096, },
  { 8, 32, 64, 2048, 4096, },
  { 1, 8, 32, 64, 2048, 4096, },
  { 2, 8, 32, 64, 2048, 4096, },
  { 1, 2, 8, 32, 64, 2048, 4096, },
  { 4, 8, 32, 64, 2048, 4096, },
  { 1, 4, 8, 32, 64, 2048, 4096, },
  { 2, 4, 8, 32, 64, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 2048, 4096, },
  { 16, 32, 64, 2048, 4096, },
  { 1, 16, 32, 64, 2048, 4096, },
  { 2, 16, 32, 64, 2048, 4096, },
  { 1, 2, 16, 32, 64, 2048, 4096, },
  { 4, 16, 32, 64, 2048, 4096, },
  { 1, 4, 16, 32, 64, 2048, 4096, },
  { 2, 4, 16, 32, 64, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 2048, 4096, },
  { 8, 16, 32, 64, 2048, 4096, },
  { 1, 8, 16, 32, 64, 2048, 4096, },
  { 2, 8, 16, 32, 64, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 2048, 4096, },
  { 4, 8, 16, 32, 64, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 4096, },
  { 128, 2048, 4096, },
  { 1, 128, 2048, 4096, },
  { 2, 128, 2048, 4096, },
  { 1, 2, 128, 2048, 4096, },
  { 4, 128, 2048, 4096, },
  { 1, 4, 128, 2048, 4096, },
  { 2, 4, 128, 2048, 4096, },
  { 1, 2, 4, 128, 2048, 4096, },
  { 8, 128, 2048, 4096, },
  { 1, 8, 128, 2048, 4096, },
  { 2, 8, 128, 2048, 4096, },
  { 1, 2, 8, 128, 2048, 4096, },
  { 4, 8, 128, 2048, 4096, },
  { 1, 4, 8, 128, 2048, 4096, },
  { 2, 4, 8, 128, 2048, 4096, },
  { 1, 2, 4, 8, 128, 2048, 4096, },
  { 16, 128, 2048, 4096, },
  { 1, 16, 128, 2048, 4096, },
  { 2, 16, 128, 2048, 4096, },
  { 1, 2, 16, 128, 2048, 4096, },
  { 4, 16, 128, 2048, 4096, },
  { 1, 4, 16, 128, 2048, 4096, },
  { 2, 4, 16, 128, 2048, 4096, },
  { 1, 2, 4, 16, 128, 2048, 4096, },
  { 8, 16, 128, 2048, 4096, },
  { 1, 8, 16, 128, 2048, 4096, },
  { 2, 8, 16, 128, 2048, 4096, },
  { 1, 2, 8, 16, 128, 2048, 4096, },
  { 4, 8, 16, 128, 2048, 4096, },
  { 1, 4, 8, 16, 128, 2048, 4096, },
  { 2, 4, 8, 16, 128, 2048, 4096, },
  { 1, 2, 4, 8, 16, 128, 2048, 4096, },
  { 32, 128, 2048, 4096, },
  { 1, 32, 128, 2048, 4096, },
  { 2, 32, 128, 2048, 4096, },
  { 1, 2, 32, 128, 2048, 4096, },
  { 4, 32, 128, 2048, 4096, },
  { 1, 4, 32, 128, 2048, 4096, },
  { 2, 4, 32, 128, 2048, 4096, },
  { 1, 2, 4, 32, 128, 2048, 4096, },
  { 8, 32, 128, 2048, 4096, },
  { 1, 8, 32, 128, 2048, 4096, },
  { 2, 8, 32, 128, 2048, 4096, },
  { 1, 2, 8, 32, 128, 2048, 4096, },
  { 4, 8, 32, 128, 2048, 4096, },
  { 1, 4, 8, 32, 128, 2048, 4096, },
  { 2, 4, 8, 32, 128, 2048, 4096, },
  { 1, 2, 4, 8, 32, 128, 2048, 4096, },
  { 16, 32, 128, 2048, 4096, },
  { 1, 16, 32, 128, 2048, 4096, },
  { 2, 16, 32, 128, 2048, 4096, },
  { 1, 2, 16, 32, 128, 2048, 4096, },
  { 4, 16, 32, 128, 2048, 4096, },
  { 1, 4, 16, 32, 128, 2048, 4096, },
  { 2, 4, 16, 32, 128, 2048, 4096, },
  { 1, 2, 4, 16, 32, 128, 2048, 4096, },
  { 8, 16, 32, 128, 2048, 4096, },
  { 1, 8, 16, 32, 128, 2048, 4096, },
  { 2, 8, 16, 32, 128, 2048, 4096, },
  { 1, 2, 8, 16, 32, 128, 2048, 4096, },
  { 4, 8, 16, 32, 128, 2048, 4096, },
  { 1, 4, 8, 16, 32, 128, 2048, 4096, },
  { 2, 4, 8, 16, 32, 128, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 4096, },
  { 64, 128, 2048, 4096, },
  { 1, 64, 128, 2048, 4096, },
  { 2, 64, 128, 2048, 4096, },
  { 1, 2, 64, 128, 2048, 4096, },
  { 4, 64, 128, 2048, 4096, },
  { 1, 4, 64, 128, 2048, 4096, },
  { 2, 4, 64, 128, 2048, 4096, },
  { 1, 2, 4, 64, 128, 2048, 4096, },
  { 8, 64, 128, 2048, 4096, },
  { 1, 8, 64, 128, 2048, 4096, },
  { 2, 8, 64, 128, 2048, 4096, },
  { 1, 2, 8, 64, 128, 2048, 4096, },
  { 4, 8, 64, 128, 2048, 4096, },
  { 1, 4, 8, 64, 128, 2048, 4096, },
  { 2, 4, 8, 64, 128, 2048, 4096, },
  { 1, 2, 4, 8, 64, 128, 2048, 4096, },
  { 16, 64, 128, 2048, 4096, },
  { 1, 16, 64, 128, 2048, 4096, },
  { 2, 16, 64, 128, 2048, 4096, },
  { 1, 2, 16, 64, 128, 2048, 4096, },
  { 4, 16, 64, 128, 2048, 4096, },
  { 1, 4, 16, 64, 128, 2048, 4096, },
  { 2, 4, 16, 64, 128, 2048, 4096, },
  { 1, 2, 4, 16, 64, 128, 2048, 4096, },
  { 8, 16, 64, 128, 2048, 4096, },
  { 1, 8, 16, 64, 128, 2048, 4096, },
  { 2, 8, 16, 64, 128, 2048, 4096, },
  { 1, 2, 8, 16, 64, 128, 2048, 4096, },
  { 4, 8, 16, 64, 128, 2048, 4096, },
  { 1, 4, 8, 16, 64, 128, 2048, 4096, },
  { 2, 4, 8, 16, 64, 128, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 4096, },
  { 32, 64, 128, 2048, 4096, },
  { 1, 32, 64, 128, 2048, 4096, },
  { 2, 32, 64, 128, 2048, 4096, },
  { 1, 2, 32, 64, 128, 2048, 4096, },
  { 4, 32, 64, 128, 2048, 4096, },
  { 1, 4, 32, 64, 128, 2048, 4096, },
  { 2, 4, 32, 64, 128, 2048, 4096, },
  { 1, 2, 4, 32, 64, 128, 2048, 4096, },
  { 8, 32, 64, 128, 2048, 4096, },
  { 1, 8, 32, 64, 128, 2048, 4096, },
  { 2, 8, 32, 64, 128, 2048, 4096, },
  { 1, 2, 8, 32, 64, 128, 2048, 4096, },
  { 4, 8, 32, 64, 128, 2048, 4096, },
  { 1, 4, 8, 32, 64, 128, 2048, 4096, },
  { 2, 4, 8, 32, 64, 128, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 4096, },
  { 16, 32, 64, 128, 2048, 4096, },
  { 1, 16, 32, 64, 128, 2048, 4096, },
  { 2, 16, 32, 64, 128, 2048, 4096, },
  { 1, 2, 16, 32, 64, 128, 2048, 4096, },
  { 4, 16, 32, 64, 128, 2048, 4096, },
  { 1, 4, 16, 32, 64, 128, 2048, 4096, },
  { 2, 4, 16, 32, 64, 128, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 4096, },
  { 8, 16, 32, 64, 128, 2048, 4096, },
  { 1, 8, 16, 32, 64, 128, 2048, 4096, },
  { 2, 8, 16, 32, 64, 128, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 4096, },
  { 4, 8, 16, 32, 64, 128, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 4096, },
  { 256, 2048, 4096, },
  { 1, 256, 2048, 4096, },
  { 2, 256, 2048, 4096, },
  { 1, 2, 256, 2048, 4096, },
  { 4, 256, 2048, 4096, },
  { 1, 4, 256, 2048, 4096, },
  { 2, 4, 256, 2048, 4096, },
  { 1, 2, 4, 256, 2048, 4096, },
  { 8, 256, 2048, 4096, },
  { 1, 8, 256, 2048, 4096, },
  { 2, 8, 256, 2048, 4096, },
  { 1, 2, 8, 256, 2048, 4096, },
  { 4, 8, 256, 2048, 4096, },
  { 1, 4, 8, 256, 2048, 4096, },
  { 2, 4, 8, 256, 2048, 4096, },
  { 1, 2, 4, 8, 256, 2048, 4096, },
  { 16, 256, 2048, 4096, },
  { 1, 16, 256, 2048, 4096, },
  { 2, 16, 256, 2048, 4096, },
  { 1, 2, 16, 256, 2048, 4096, },
  { 4, 16, 256, 2048, 4096, },
  { 1, 4, 16, 256, 2048, 4096, },
  { 2, 4, 16, 256, 2048, 4096, },
  { 1, 2, 4, 16, 256, 2048, 4096, },
  { 8, 16, 256, 2048, 4096, },
  { 1, 8, 16, 256, 2048, 4096, },
  { 2, 8, 16, 256, 2048, 4096, },
  { 1, 2, 8, 16, 256, 2048, 4096, },
  { 4, 8, 16, 256, 2048, 4096, },
  { 1, 4, 8, 16, 256, 2048, 4096, },
  { 2, 4, 8, 16, 256, 2048, 4096, },
  { 1, 2, 4, 8, 16, 256, 2048, 4096, },
  { 32, 256, 2048, 4096, },
  { 1, 32, 256, 2048, 4096, },
  { 2, 32, 256, 2048, 4096, },
  { 1, 2, 32, 256, 2048, 4096, },
  { 4, 32, 256, 2048, 4096, },
  { 1, 4, 32, 256, 2048, 4096, },
  { 2, 4, 32, 256, 2048, 4096, },
  { 1, 2, 4, 32, 256, 2048, 4096, },
  { 8, 32, 256, 2048, 4096, },
  { 1, 8, 32, 256, 2048, 4096, },
  { 2, 8, 32, 256, 2048, 4096, },
  { 1, 2, 8, 32, 256, 2048, 4096, },
  { 4, 8, 32, 256, 2048, 4096, },
  { 1, 4, 8, 32, 256, 2048, 4096, },
  { 2, 4, 8, 32, 256, 2048, 4096, },
  { 1, 2, 4, 8, 32, 256, 2048, 4096, },
  { 16, 32, 256, 2048, 4096, },
  { 1, 16, 32, 256, 2048, 4096, },
  { 2, 16, 32, 256, 2048, 4096, },
  { 1, 2, 16, 32, 256, 2048, 4096, },
  { 4, 16, 32, 256, 2048, 4096, },
  { 1, 4, 16, 32, 256, 2048, 4096, },
  { 2, 4, 16, 32, 256, 2048, 4096, },
  { 1, 2, 4, 16, 32, 256, 2048, 4096, },
  { 8, 16, 32, 256, 2048, 4096, },
  { 1, 8, 16, 32, 256, 2048, 4096, },
  { 2, 8, 16, 32, 256, 2048, 4096, },
  { 1, 2, 8, 16, 32, 256, 2048, 4096, },
  { 4, 8, 16, 32, 256, 2048, 4096, },
  { 1, 4, 8, 16, 32, 256, 2048, 4096, },
  { 2, 4, 8, 16, 32, 256, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 4096, },
  { 64, 256, 2048, 4096, },
  { 1, 64, 256, 2048, 4096, },
  { 2, 64, 256, 2048, 4096, },
  { 1, 2, 64, 256, 2048, 4096, },
  { 4, 64, 256, 2048, 4096, },
  { 1, 4, 64, 256, 2048, 4096, },
  { 2, 4, 64, 256, 2048, 4096, },
  { 1, 2, 4, 64, 256, 2048, 4096, },
  { 8, 64, 256, 2048, 4096, },
  { 1, 8, 64, 256, 2048, 4096, },
  { 2, 8, 64, 256, 2048, 4096, },
  { 1, 2, 8, 64, 256, 2048, 4096, },
  { 4, 8, 64, 256, 2048, 4096, },
  { 1, 4, 8, 64, 256, 2048, 4096, },
  { 2, 4, 8, 64, 256, 2048, 4096, },
  { 1, 2, 4, 8, 64, 256, 2048, 4096, },
  { 16, 64, 256, 2048, 4096, },
  { 1, 16, 64, 256, 2048, 4096, },
  { 2, 16, 64, 256, 2048, 4096, },
  { 1, 2, 16, 64, 256, 2048, 4096, },
  { 4, 16, 64, 256, 2048, 4096, },
  { 1, 4, 16, 64, 256, 2048, 4096, },
  { 2, 4, 16, 64, 256, 2048, 4096, },
  { 1, 2, 4, 16, 64, 256, 2048, 4096, },
  { 8, 16, 64, 256, 2048, 4096, },
  { 1, 8, 16, 64, 256, 2048, 4096, },
  { 2, 8, 16, 64, 256, 2048, 4096, },
  { 1, 2, 8, 16, 64, 256, 2048, 4096, },
  { 4, 8, 16, 64, 256, 2048, 4096, },
  { 1, 4, 8, 16, 64, 256, 2048, 4096, },
  { 2, 4, 8, 16, 64, 256, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 4096, },
  { 32, 64, 256, 2048, 4096, },
  { 1, 32, 64, 256, 2048, 4096, },
  { 2, 32, 64, 256, 2048, 4096, },
  { 1, 2, 32, 64, 256, 2048, 4096, },
  { 4, 32, 64, 256, 2048, 4096, },
  { 1, 4, 32, 64, 256, 2048, 4096, },
  { 2, 4, 32, 64, 256, 2048, 4096, },
  { 1, 2, 4, 32, 64, 256, 2048, 4096, },
  { 8, 32, 64, 256, 2048, 4096, },
  { 1, 8, 32, 64, 256, 2048, 4096, },
  { 2, 8, 32, 64, 256, 2048, 4096, },
  { 1, 2, 8, 32, 64, 256, 2048, 4096, },
  { 4, 8, 32, 64, 256, 2048, 4096, },
  { 1, 4, 8, 32, 64, 256, 2048, 4096, },
  { 2, 4, 8, 32, 64, 256, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 4096, },
  { 16, 32, 64, 256, 2048, 4096, },
  { 1, 16, 32, 64, 256, 2048, 4096, },
  { 2, 16, 32, 64, 256, 2048, 4096, },
  { 1, 2, 16, 32, 64, 256, 2048, 4096, },
  { 4, 16, 32, 64, 256, 2048, 4096, },
  { 1, 4, 16, 32, 64, 256, 2048, 4096, },
  { 2, 4, 16, 32, 64, 256, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 4096, },
  { 8, 16, 32, 64, 256, 2048, 4096, },
  { 1, 8, 16, 32, 64, 256, 2048, 4096, },
  { 2, 8, 16, 32, 64, 256, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 4096, },
  { 4, 8, 16, 32, 64, 256, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 4096, },
  { 128, 256, 2048, 4096, },
  { 1, 128, 256, 2048, 4096, },
  { 2, 128, 256, 2048, 4096, },
  { 1, 2, 128, 256, 2048, 4096, },
  { 4, 128, 256, 2048, 4096, },
  { 1, 4, 128, 256, 2048, 4096, },
  { 2, 4, 128, 256, 2048, 4096, },
  { 1, 2, 4, 128, 256, 2048, 4096, },
  { 8, 128, 256, 2048, 4096, },
  { 1, 8, 128, 256, 2048, 4096, },
  { 2, 8, 128, 256, 2048, 4096, },
  { 1, 2, 8, 128, 256, 2048, 4096, },
  { 4, 8, 128, 256, 2048, 4096, },
  { 1, 4, 8, 128, 256, 2048, 4096, },
  { 2, 4, 8, 128, 256, 2048, 4096, },
  { 1, 2, 4, 8, 128, 256, 2048, 4096, },
  { 16, 128, 256, 2048, 4096, },
  { 1, 16, 128, 256, 2048, 4096, },
  { 2, 16, 128, 256, 2048, 4096, },
  { 1, 2, 16, 128, 256, 2048, 4096, },
  { 4, 16, 128, 256, 2048, 4096, },
  { 1, 4, 16, 128, 256, 2048, 4096, },
  { 2, 4, 16, 128, 256, 2048, 4096, },
  { 1, 2, 4, 16, 128, 256, 2048, 4096, },
  { 8, 16, 128, 256, 2048, 4096, },
  { 1, 8, 16, 128, 256, 2048, 4096, },
  { 2, 8, 16, 128, 256, 2048, 4096, },
  { 1, 2, 8, 16, 128, 256, 2048, 4096, },
  { 4, 8, 16, 128, 256, 2048, 4096, },
  { 1, 4, 8, 16, 128, 256, 2048, 4096, },
  { 2, 4, 8, 16, 128, 256, 2048, 4096, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 4096, },
  { 32, 128, 256, 2048, 4096, },
  { 1, 32, 128, 256, 2048, 4096, },
  { 2, 32, 128, 256, 2048, 4096, },
  { 1, 2, 32, 128, 256, 2048, 4096, },
  { 4, 32, 128, 256, 2048, 4096, },
  { 1, 4, 32, 128, 256, 2048, 4096, },
  { 2, 4, 32, 128, 256, 2048, 4096, },
  { 1, 2, 4, 32, 128, 256, 2048, 4096, },
  { 8, 32, 128, 256, 2048, 4096, },
  { 1, 8, 32, 128, 256, 2048, 4096, },
  { 2, 8, 32, 128, 256, 2048, 4096, },
  { 1, 2, 8, 32, 128, 256, 2048, 4096, },
  { 4, 8, 32, 128, 256, 2048, 4096, },
  { 1, 4, 8, 32, 128, 256, 2048, 4096, },
  { 2, 4, 8, 32, 128, 256, 2048, 4096, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 4096, },
  { 16, 32, 128, 256, 2048, 4096, },
  { 1, 16, 32, 128, 256, 2048, 4096, },
  { 2, 16, 32, 128, 256, 2048, 4096, },
  { 1, 2, 16, 32, 128, 256, 2048, 4096, },
  { 4, 16, 32, 128, 256, 2048, 4096, },
  { 1, 4, 16, 32, 128, 256, 2048, 4096, },
  { 2, 4, 16, 32, 128, 256, 2048, 4096, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 4096, },
  { 8, 16, 32, 128, 256, 2048, 4096, },
  { 1, 8, 16, 32, 128, 256, 2048, 4096, },
  { 2, 8, 16, 32, 128, 256, 2048, 4096, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 4096, },
  { 4, 8, 16, 32, 128, 256, 2048, 4096, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 4096, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 4096, },
  { 64, 128, 256, 2048, 4096, },
  { 1, 64, 128, 256, 2048, 4096, },
  { 2, 64, 128, 256, 2048, 4096, },
  { 1, 2, 64, 128, 256, 2048, 4096, },
  { 4, 64, 128, 256, 2048, 4096, },
  { 1, 4, 64, 128, 256, 2048, 4096, },
  { 2, 4, 64, 128, 256, 2048, 4096, },
  { 1, 2, 4, 64, 128, 256, 2048, 4096, },
  { 8, 64, 128, 256, 2048, 4096, },
  { 1, 8, 64, 128, 256, 2048, 4096, },
  { 2, 8, 64, 128, 256, 2048, 4096, },
  { 1, 2, 8, 64, 128, 256, 2048, 4096, },
  { 4, 8, 64, 128, 256, 2048, 4096, },
  { 1, 4, 8, 64, 128, 256, 2048, 4096, },
  { 2, 4, 8, 64, 128, 256, 2048, 4096, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 4096, },
  { 16, 64, 128, 256, 2048, 4096, },
  { 1, 16, 64, 128, 256, 2048, 4096, },
  { 2, 16, 64, 128, 256, 2048, 4096, },
  { 1, 2, 16, 64, 128, 256, 2048, 4096, },
  { 4, 16, 64, 128, 256, 2048, 4096, },
  { 1, 4, 16, 64, 128, 256, 2048, 4096, },
  { 2, 4, 16, 64, 128, 256, 2048, 4096, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 4096, },
  { 8, 16, 64, 128, 256, 2048, 4096, },
  { 1, 8, 16, 64, 128, 256, 2048, 4096, },
  { 2, 8, 16, 64, 128, 256, 2048, 4096, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 4096, },
  { 4, 8, 16, 64, 128, 256, 2048, 4096, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 4096, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 4096, },
  { 32, 64, 128, 256, 2048, 4096, },
  { 1, 32, 64, 128, 256, 2048, 4096, },
  { 2, 32, 64, 128, 256, 2048, 4096, },
  { 1, 2, 32, 64, 128, 256, 2048, 4096, },
  { 4, 32, 64, 128, 256, 2048, 4096, },
  { 1, 4, 32, 64, 128, 256, 2048, 4096, },
  { 2, 4, 32, 64, 128, 256, 2048, 4096, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 4096, },
  { 8, 32, 64, 128, 256, 2048, 4096, },
  { 1, 8, 32, 64, 128, 256, 2048, 4096, },
  { 2, 8, 32, 64, 128, 256, 2048, 4096, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 4096, },
  { 4, 8, 32, 64, 128, 256, 2048, 4096, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 4096, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 4096, },
  { 16, 32, 64, 128, 256, 2048, 4096, },
  { 1, 16, 32, 64, 128, 256, 2048, 4096, },
  { 2, 16, 32, 64, 128, 256, 2048, 4096, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 4096, },
  { 4, 16, 32, 64, 128, 256, 2048, 4096, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 4096, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 4096, },
  { 8, 16, 32, 64, 128, 256, 2048, 4096, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 4096, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 4096, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, },
  { 512, 2048, 4096, },
  { 1, 512, 2048, 4096, },
  { 2, 512, 2048, 4096, },
  { 1, 2, 512, 2048, 4096, },
  { 4, 512, 2048, 4096, },
  { 1, 4, 512, 2048, 4096, },
  { 2, 4, 512, 2048, 4096, },
  { 1, 2, 4, 512, 2048, 4096, },
  { 8, 512, 2048, 4096, },
  { 1, 8, 512, 2048, 4096, },
  { 2, 8, 512, 2048, 4096, },
  { 1, 2, 8, 512, 2048, 4096, },
  { 4, 8, 512, 2048, 4096, },
  { 1, 4, 8, 512, 2048, 4096, },
  { 2, 4, 8, 512, 2048, 4096, },
  { 1, 2, 4, 8, 512, 2048, 4096, },
  { 16, 512, 2048, 4096, },
  { 1, 16, 512, 2048, 4096, },
  { 2, 16, 512, 2048, 4096, },
  { 1, 2, 16, 512, 2048, 4096, },
  { 4, 16, 512, 2048, 4096, },
  { 1, 4, 16, 512, 2048, 4096, },
  { 2, 4, 16, 512, 2048, 4096, },
  { 1, 2, 4, 16, 512, 2048, 4096, },
  { 8, 16, 512, 2048, 4096, },
  { 1, 8, 16, 512, 2048, 4096, },
  { 2, 8, 16, 512, 2048, 4096, },
  { 1, 2, 8, 16, 512, 2048, 4096, },
  { 4, 8, 16, 512, 2048, 4096, },
  { 1, 4, 8, 16, 512, 2048, 4096, },
  { 2, 4, 8, 16, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 512, 2048, 4096, },
  { 32, 512, 2048, 4096, },
  { 1, 32, 512, 2048, 4096, },
  { 2, 32, 512, 2048, 4096, },
  { 1, 2, 32, 512, 2048, 4096, },
  { 4, 32, 512, 2048, 4096, },
  { 1, 4, 32, 512, 2048, 4096, },
  { 2, 4, 32, 512, 2048, 4096, },
  { 1, 2, 4, 32, 512, 2048, 4096, },
  { 8, 32, 512, 2048, 4096, },
  { 1, 8, 32, 512, 2048, 4096, },
  { 2, 8, 32, 512, 2048, 4096, },
  { 1, 2, 8, 32, 512, 2048, 4096, },
  { 4, 8, 32, 512, 2048, 4096, },
  { 1, 4, 8, 32, 512, 2048, 4096, },
  { 2, 4, 8, 32, 512, 2048, 4096, },
  { 1, 2, 4, 8, 32, 512, 2048, 4096, },
  { 16, 32, 512, 2048, 4096, },
  { 1, 16, 32, 512, 2048, 4096, },
  { 2, 16, 32, 512, 2048, 4096, },
  { 1, 2, 16, 32, 512, 2048, 4096, },
  { 4, 16, 32, 512, 2048, 4096, },
  { 1, 4, 16, 32, 512, 2048, 4096, },
  { 2, 4, 16, 32, 512, 2048, 4096, },
  { 1, 2, 4, 16, 32, 512, 2048, 4096, },
  { 8, 16, 32, 512, 2048, 4096, },
  { 1, 8, 16, 32, 512, 2048, 4096, },
  { 2, 8, 16, 32, 512, 2048, 4096, },
  { 1, 2, 8, 16, 32, 512, 2048, 4096, },
  { 4, 8, 16, 32, 512, 2048, 4096, },
  { 1, 4, 8, 16, 32, 512, 2048, 4096, },
  { 2, 4, 8, 16, 32, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 4096, },
  { 64, 512, 2048, 4096, },
  { 1, 64, 512, 2048, 4096, },
  { 2, 64, 512, 2048, 4096, },
  { 1, 2, 64, 512, 2048, 4096, },
  { 4, 64, 512, 2048, 4096, },
  { 1, 4, 64, 512, 2048, 4096, },
  { 2, 4, 64, 512, 2048, 4096, },
  { 1, 2, 4, 64, 512, 2048, 4096, },
  { 8, 64, 512, 2048, 4096, },
  { 1, 8, 64, 512, 2048, 4096, },
  { 2, 8, 64, 512, 2048, 4096, },
  { 1, 2, 8, 64, 512, 2048, 4096, },
  { 4, 8, 64, 512, 2048, 4096, },
  { 1, 4, 8, 64, 512, 2048, 4096, },
  { 2, 4, 8, 64, 512, 2048, 4096, },
  { 1, 2, 4, 8, 64, 512, 2048, 4096, },
  { 16, 64, 512, 2048, 4096, },
  { 1, 16, 64, 512, 2048, 4096, },
  { 2, 16, 64, 512, 2048, 4096, },
  { 1, 2, 16, 64, 512, 2048, 4096, },
  { 4, 16, 64, 512, 2048, 4096, },
  { 1, 4, 16, 64, 512, 2048, 4096, },
  { 2, 4, 16, 64, 512, 2048, 4096, },
  { 1, 2, 4, 16, 64, 512, 2048, 4096, },
  { 8, 16, 64, 512, 2048, 4096, },
  { 1, 8, 16, 64, 512, 2048, 4096, },
  { 2, 8, 16, 64, 512, 2048, 4096, },
  { 1, 2, 8, 16, 64, 512, 2048, 4096, },
  { 4, 8, 16, 64, 512, 2048, 4096, },
  { 1, 4, 8, 16, 64, 512, 2048, 4096, },
  { 2, 4, 8, 16, 64, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 4096, },
  { 32, 64, 512, 2048, 4096, },
  { 1, 32, 64, 512, 2048, 4096, },
  { 2, 32, 64, 512, 2048, 4096, },
  { 1, 2, 32, 64, 512, 2048, 4096, },
  { 4, 32, 64, 512, 2048, 4096, },
  { 1, 4, 32, 64, 512, 2048, 4096, },
  { 2, 4, 32, 64, 512, 2048, 4096, },
  { 1, 2, 4, 32, 64, 512, 2048, 4096, },
  { 8, 32, 64, 512, 2048, 4096, },
  { 1, 8, 32, 64, 512, 2048, 4096, },
  { 2, 8, 32, 64, 512, 2048, 4096, },
  { 1, 2, 8, 32, 64, 512, 2048, 4096, },
  { 4, 8, 32, 64, 512, 2048, 4096, },
  { 1, 4, 8, 32, 64, 512, 2048, 4096, },
  { 2, 4, 8, 32, 64, 512, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 4096, },
  { 16, 32, 64, 512, 2048, 4096, },
  { 1, 16, 32, 64, 512, 2048, 4096, },
  { 2, 16, 32, 64, 512, 2048, 4096, },
  { 1, 2, 16, 32, 64, 512, 2048, 4096, },
  { 4, 16, 32, 64, 512, 2048, 4096, },
  { 1, 4, 16, 32, 64, 512, 2048, 4096, },
  { 2, 4, 16, 32, 64, 512, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 4096, },
  { 8, 16, 32, 64, 512, 2048, 4096, },
  { 1, 8, 16, 32, 64, 512, 2048, 4096, },
  { 2, 8, 16, 32, 64, 512, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 4096, },
  { 4, 8, 16, 32, 64, 512, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 4096, },
  { 128, 512, 2048, 4096, },
  { 1, 128, 512, 2048, 4096, },
  { 2, 128, 512, 2048, 4096, },
  { 1, 2, 128, 512, 2048, 4096, },
  { 4, 128, 512, 2048, 4096, },
  { 1, 4, 128, 512, 2048, 4096, },
  { 2, 4, 128, 512, 2048, 4096, },
  { 1, 2, 4, 128, 512, 2048, 4096, },
  { 8, 128, 512, 2048, 4096, },
  { 1, 8, 128, 512, 2048, 4096, },
  { 2, 8, 128, 512, 2048, 4096, },
  { 1, 2, 8, 128, 512, 2048, 4096, },
  { 4, 8, 128, 512, 2048, 4096, },
  { 1, 4, 8, 128, 512, 2048, 4096, },
  { 2, 4, 8, 128, 512, 2048, 4096, },
  { 1, 2, 4, 8, 128, 512, 2048, 4096, },
  { 16, 128, 512, 2048, 4096, },
  { 1, 16, 128, 512, 2048, 4096, },
  { 2, 16, 128, 512, 2048, 4096, },
  { 1, 2, 16, 128, 512, 2048, 4096, },
  { 4, 16, 128, 512, 2048, 4096, },
  { 1, 4, 16, 128, 512, 2048, 4096, },
  { 2, 4, 16, 128, 512, 2048, 4096, },
  { 1, 2, 4, 16, 128, 512, 2048, 4096, },
  { 8, 16, 128, 512, 2048, 4096, },
  { 1, 8, 16, 128, 512, 2048, 4096, },
  { 2, 8, 16, 128, 512, 2048, 4096, },
  { 1, 2, 8, 16, 128, 512, 2048, 4096, },
  { 4, 8, 16, 128, 512, 2048, 4096, },
  { 1, 4, 8, 16, 128, 512, 2048, 4096, },
  { 2, 4, 8, 16, 128, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 4096, },
  { 32, 128, 512, 2048, 4096, },
  { 1, 32, 128, 512, 2048, 4096, },
  { 2, 32, 128, 512, 2048, 4096, },
  { 1, 2, 32, 128, 512, 2048, 4096, },
  { 4, 32, 128, 512, 2048, 4096, },
  { 1, 4, 32, 128, 512, 2048, 4096, },
  { 2, 4, 32, 128, 512, 2048, 4096, },
  { 1, 2, 4, 32, 128, 512, 2048, 4096, },
  { 8, 32, 128, 512, 2048, 4096, },
  { 1, 8, 32, 128, 512, 2048, 4096, },
  { 2, 8, 32, 128, 512, 2048, 4096, },
  { 1, 2, 8, 32, 128, 512, 2048, 4096, },
  { 4, 8, 32, 128, 512, 2048, 4096, },
  { 1, 4, 8, 32, 128, 512, 2048, 4096, },
  { 2, 4, 8, 32, 128, 512, 2048, 4096, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 4096, },
  { 16, 32, 128, 512, 2048, 4096, },
  { 1, 16, 32, 128, 512, 2048, 4096, },
  { 2, 16, 32, 128, 512, 2048, 4096, },
  { 1, 2, 16, 32, 128, 512, 2048, 4096, },
  { 4, 16, 32, 128, 512, 2048, 4096, },
  { 1, 4, 16, 32, 128, 512, 2048, 4096, },
  { 2, 4, 16, 32, 128, 512, 2048, 4096, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 4096, },
  { 8, 16, 32, 128, 512, 2048, 4096, },
  { 1, 8, 16, 32, 128, 512, 2048, 4096, },
  { 2, 8, 16, 32, 128, 512, 2048, 4096, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 4096, },
  { 4, 8, 16, 32, 128, 512, 2048, 4096, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 4096, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 4096, },
  { 64, 128, 512, 2048, 4096, },
  { 1, 64, 128, 512, 2048, 4096, },
  { 2, 64, 128, 512, 2048, 4096, },
  { 1, 2, 64, 128, 512, 2048, 4096, },
  { 4, 64, 128, 512, 2048, 4096, },
  { 1, 4, 64, 128, 512, 2048, 4096, },
  { 2, 4, 64, 128, 512, 2048, 4096, },
  { 1, 2, 4, 64, 128, 512, 2048, 4096, },
  { 8, 64, 128, 512, 2048, 4096, },
  { 1, 8, 64, 128, 512, 2048, 4096, },
  { 2, 8, 64, 128, 512, 2048, 4096, },
  { 1, 2, 8, 64, 128, 512, 2048, 4096, },
  { 4, 8, 64, 128, 512, 2048, 4096, },
  { 1, 4, 8, 64, 128, 512, 2048, 4096, },
  { 2, 4, 8, 64, 128, 512, 2048, 4096, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 4096, },
  { 16, 64, 128, 512, 2048, 4096, },
  { 1, 16, 64, 128, 512, 2048, 4096, },
  { 2, 16, 64, 128, 512, 2048, 4096, },
  { 1, 2, 16, 64, 128, 512, 2048, 4096, },
  { 4, 16, 64, 128, 512, 2048, 4096, },
  { 1, 4, 16, 64, 128, 512, 2048, 4096, },
  { 2, 4, 16, 64, 128, 512, 2048, 4096, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 4096, },
  { 8, 16, 64, 128, 512, 2048, 4096, },
  { 1, 8, 16, 64, 128, 512, 2048, 4096, },
  { 2, 8, 16, 64, 128, 512, 2048, 4096, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 4096, },
  { 4, 8, 16, 64, 128, 512, 2048, 4096, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 4096, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 4096, },
  { 32, 64, 128, 512, 2048, 4096, },
  { 1, 32, 64, 128, 512, 2048, 4096, },
  { 2, 32, 64, 128, 512, 2048, 4096, },
  { 1, 2, 32, 64, 128, 512, 2048, 4096, },
  { 4, 32, 64, 128, 512, 2048, 4096, },
  { 1, 4, 32, 64, 128, 512, 2048, 4096, },
  { 2, 4, 32, 64, 128, 512, 2048, 4096, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 4096, },
  { 8, 32, 64, 128, 512, 2048, 4096, },
  { 1, 8, 32, 64, 128, 512, 2048, 4096, },
  { 2, 8, 32, 64, 128, 512, 2048, 4096, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 4096, },
  { 4, 8, 32, 64, 128, 512, 2048, 4096, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 4096, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 4096, },
  { 16, 32, 64, 128, 512, 2048, 4096, },
  { 1, 16, 32, 64, 128, 512, 2048, 4096, },
  { 2, 16, 32, 64, 128, 512, 2048, 4096, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 4096, },
  { 4, 16, 32, 64, 128, 512, 2048, 4096, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 4096, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 4096, },
  { 8, 16, 32, 64, 128, 512, 2048, 4096, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 4096, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 4096, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, },
  { 256, 512, 2048, 4096, },
  { 1, 256, 512, 2048, 4096, },
  { 2, 256, 512, 2048, 4096, },
  { 1, 2, 256, 512, 2048, 4096, },
  { 4, 256, 512, 2048, 4096, },
  { 1, 4, 256, 512, 2048, 4096, },
  { 2, 4, 256, 512, 2048, 4096, },
  { 1, 2, 4, 256, 512, 2048, 4096, },
  { 8, 256, 512, 2048, 4096, },
  { 1, 8, 256, 512, 2048, 4096, },
  { 2, 8, 256, 512, 2048, 4096, },
  { 1, 2, 8, 256, 512, 2048, 4096, },
  { 4, 8, 256, 512, 2048, 4096, },
  { 1, 4, 8, 256, 512, 2048, 4096, },
  { 2, 4, 8, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 256, 512, 2048, 4096, },
  { 16, 256, 512, 2048, 4096, },
  { 1, 16, 256, 512, 2048, 4096, },
  { 2, 16, 256, 512, 2048, 4096, },
  { 1, 2, 16, 256, 512, 2048, 4096, },
  { 4, 16, 256, 512, 2048, 4096, },
  { 1, 4, 16, 256, 512, 2048, 4096, },
  { 2, 4, 16, 256, 512, 2048, 4096, },
  { 1, 2, 4, 16, 256, 512, 2048, 4096, },
  { 8, 16, 256, 512, 2048, 4096, },
  { 1, 8, 16, 256, 512, 2048, 4096, },
  { 2, 8, 16, 256, 512, 2048, 4096, },
  { 1, 2, 8, 16, 256, 512, 2048, 4096, },
  { 4, 8, 16, 256, 512, 2048, 4096, },
  { 1, 4, 8, 16, 256, 512, 2048, 4096, },
  { 2, 4, 8, 16, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 4096, },
  { 32, 256, 512, 2048, 4096, },
  { 1, 32, 256, 512, 2048, 4096, },
  { 2, 32, 256, 512, 2048, 4096, },
  { 1, 2, 32, 256, 512, 2048, 4096, },
  { 4, 32, 256, 512, 2048, 4096, },
  { 1, 4, 32, 256, 512, 2048, 4096, },
  { 2, 4, 32, 256, 512, 2048, 4096, },
  { 1, 2, 4, 32, 256, 512, 2048, 4096, },
  { 8, 32, 256, 512, 2048, 4096, },
  { 1, 8, 32, 256, 512, 2048, 4096, },
  { 2, 8, 32, 256, 512, 2048, 4096, },
  { 1, 2, 8, 32, 256, 512, 2048, 4096, },
  { 4, 8, 32, 256, 512, 2048, 4096, },
  { 1, 4, 8, 32, 256, 512, 2048, 4096, },
  { 2, 4, 8, 32, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 4096, },
  { 16, 32, 256, 512, 2048, 4096, },
  { 1, 16, 32, 256, 512, 2048, 4096, },
  { 2, 16, 32, 256, 512, 2048, 4096, },
  { 1, 2, 16, 32, 256, 512, 2048, 4096, },
  { 4, 16, 32, 256, 512, 2048, 4096, },
  { 1, 4, 16, 32, 256, 512, 2048, 4096, },
  { 2, 4, 16, 32, 256, 512, 2048, 4096, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 4096, },
  { 8, 16, 32, 256, 512, 2048, 4096, },
  { 1, 8, 16, 32, 256, 512, 2048, 4096, },
  { 2, 8, 16, 32, 256, 512, 2048, 4096, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 4096, },
  { 4, 8, 16, 32, 256, 512, 2048, 4096, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 4096, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 4096, },
  { 64, 256, 512, 2048, 4096, },
  { 1, 64, 256, 512, 2048, 4096, },
  { 2, 64, 256, 512, 2048, 4096, },
  { 1, 2, 64, 256, 512, 2048, 4096, },
  { 4, 64, 256, 512, 2048, 4096, },
  { 1, 4, 64, 256, 512, 2048, 4096, },
  { 2, 4, 64, 256, 512, 2048, 4096, },
  { 1, 2, 4, 64, 256, 512, 2048, 4096, },
  { 8, 64, 256, 512, 2048, 4096, },
  { 1, 8, 64, 256, 512, 2048, 4096, },
  { 2, 8, 64, 256, 512, 2048, 4096, },
  { 1, 2, 8, 64, 256, 512, 2048, 4096, },
  { 4, 8, 64, 256, 512, 2048, 4096, },
  { 1, 4, 8, 64, 256, 512, 2048, 4096, },
  { 2, 4, 8, 64, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 4096, },
  { 16, 64, 256, 512, 2048, 4096, },
  { 1, 16, 64, 256, 512, 2048, 4096, },
  { 2, 16, 64, 256, 512, 2048, 4096, },
  { 1, 2, 16, 64, 256, 512, 2048, 4096, },
  { 4, 16, 64, 256, 512, 2048, 4096, },
  { 1, 4, 16, 64, 256, 512, 2048, 4096, },
  { 2, 4, 16, 64, 256, 512, 2048, 4096, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 4096, },
  { 8, 16, 64, 256, 512, 2048, 4096, },
  { 1, 8, 16, 64, 256, 512, 2048, 4096, },
  { 2, 8, 16, 64, 256, 512, 2048, 4096, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 4096, },
  { 4, 8, 16, 64, 256, 512, 2048, 4096, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 4096, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 4096, },
  { 32, 64, 256, 512, 2048, 4096, },
  { 1, 32, 64, 256, 512, 2048, 4096, },
  { 2, 32, 64, 256, 512, 2048, 4096, },
  { 1, 2, 32, 64, 256, 512, 2048, 4096, },
  { 4, 32, 64, 256, 512, 2048, 4096, },
  { 1, 4, 32, 64, 256, 512, 2048, 4096, },
  { 2, 4, 32, 64, 256, 512, 2048, 4096, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 4096, },
  { 8, 32, 64, 256, 512, 2048, 4096, },
  { 1, 8, 32, 64, 256, 512, 2048, 4096, },
  { 2, 8, 32, 64, 256, 512, 2048, 4096, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 4096, },
  { 4, 8, 32, 64, 256, 512, 2048, 4096, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 4096, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 4096, },
  { 16, 32, 64, 256, 512, 2048, 4096, },
  { 1, 16, 32, 64, 256, 512, 2048, 4096, },
  { 2, 16, 32, 64, 256, 512, 2048, 4096, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 4096, },
  { 4, 16, 32, 64, 256, 512, 2048, 4096, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 4096, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 4096, },
  { 8, 16, 32, 64, 256, 512, 2048, 4096, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 4096, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 4096, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, },
  { 128, 256, 512, 2048, 4096, },
  { 1, 128, 256, 512, 2048, 4096, },
  { 2, 128, 256, 512, 2048, 4096, },
  { 1, 2, 128, 256, 512, 2048, 4096, },
  { 4, 128, 256, 512, 2048, 4096, },
  { 1, 4, 128, 256, 512, 2048, 4096, },
  { 2, 4, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 128, 256, 512, 2048, 4096, },
  { 8, 128, 256, 512, 2048, 4096, },
  { 1, 8, 128, 256, 512, 2048, 4096, },
  { 2, 8, 128, 256, 512, 2048, 4096, },
  { 1, 2, 8, 128, 256, 512, 2048, 4096, },
  { 4, 8, 128, 256, 512, 2048, 4096, },
  { 1, 4, 8, 128, 256, 512, 2048, 4096, },
  { 2, 4, 8, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 4096, },
  { 16, 128, 256, 512, 2048, 4096, },
  { 1, 16, 128, 256, 512, 2048, 4096, },
  { 2, 16, 128, 256, 512, 2048, 4096, },
  { 1, 2, 16, 128, 256, 512, 2048, 4096, },
  { 4, 16, 128, 256, 512, 2048, 4096, },
  { 1, 4, 16, 128, 256, 512, 2048, 4096, },
  { 2, 4, 16, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 4096, },
  { 8, 16, 128, 256, 512, 2048, 4096, },
  { 1, 8, 16, 128, 256, 512, 2048, 4096, },
  { 2, 8, 16, 128, 256, 512, 2048, 4096, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 4096, },
  { 4, 8, 16, 128, 256, 512, 2048, 4096, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 4096, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 4096, },
  { 32, 128, 256, 512, 2048, 4096, },
  { 1, 32, 128, 256, 512, 2048, 4096, },
  { 2, 32, 128, 256, 512, 2048, 4096, },
  { 1, 2, 32, 128, 256, 512, 2048, 4096, },
  { 4, 32, 128, 256, 512, 2048, 4096, },
  { 1, 4, 32, 128, 256, 512, 2048, 4096, },
  { 2, 4, 32, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 4096, },
  { 8, 32, 128, 256, 512, 2048, 4096, },
  { 1, 8, 32, 128, 256, 512, 2048, 4096, },
  { 2, 8, 32, 128, 256, 512, 2048, 4096, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 4096, },
  { 4, 8, 32, 128, 256, 512, 2048, 4096, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 4096, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 4096, },
  { 16, 32, 128, 256, 512, 2048, 4096, },
  { 1, 16, 32, 128, 256, 512, 2048, 4096, },
  { 2, 16, 32, 128, 256, 512, 2048, 4096, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 4096, },
  { 4, 16, 32, 128, 256, 512, 2048, 4096, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 4096, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 4096, },
  { 8, 16, 32, 128, 256, 512, 2048, 4096, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 4096, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 4096, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 4096, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 4096, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, },
  { 64, 128, 256, 512, 2048, 4096, },
  { 1, 64, 128, 256, 512, 2048, 4096, },
  { 2, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 64, 128, 256, 512, 2048, 4096, },
  { 4, 64, 128, 256, 512, 2048, 4096, },
  { 1, 4, 64, 128, 256, 512, 2048, 4096, },
  { 2, 4, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 4096, },
  { 8, 64, 128, 256, 512, 2048, 4096, },
  { 1, 8, 64, 128, 256, 512, 2048, 4096, },
  { 2, 8, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 4096, },
  { 4, 8, 64, 128, 256, 512, 2048, 4096, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 4096, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 4096, },
  { 16, 64, 128, 256, 512, 2048, 4096, },
  { 1, 16, 64, 128, 256, 512, 2048, 4096, },
  { 2, 16, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 4096, },
  { 4, 16, 64, 128, 256, 512, 2048, 4096, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 4096, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 4096, },
  { 8, 16, 64, 128, 256, 512, 2048, 4096, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 4096, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 4096, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 4096, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, },
  { 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 32, 64, 128, 256, 512, 2048, 4096, },
  { 2, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 4096, },
  { 4, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 4096, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 4096, },
  { 8, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 4096, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 4096, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, },
  { 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, },
  { 1024, 2048, 4096, },
  { 1, 1024, 2048, 4096, },
  { 2, 1024, 2048, 4096, },
  { 1, 2, 1024, 2048, 4096, },
  { 4, 1024, 2048, 4096, },
  { 1, 4, 1024, 2048, 4096, },
  { 2, 4, 1024, 2048, 4096, },
  { 1, 2, 4, 1024, 2048, 4096, },
  { 8, 1024, 2048, 4096, },
  { 1, 8, 1024, 2048, 4096, },
  { 2, 8, 1024, 2048, 4096, },
  { 1, 2, 8, 1024, 2048, 4096, },
  { 4, 8, 1024, 2048, 4096, },
  { 1, 4, 8, 1024, 2048, 4096, },
  { 2, 4, 8, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 1024, 2048, 4096, },
  { 16, 1024, 2048, 4096, },
  { 1, 16, 1024, 2048, 4096, },
  { 2, 16, 1024, 2048, 4096, },
  { 1, 2, 16, 1024, 2048, 4096, },
  { 4, 16, 1024, 2048, 4096, },
  { 1, 4, 16, 1024, 2048, 4096, },
  { 2, 4, 16, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 1024, 2048, 4096, },
  { 8, 16, 1024, 2048, 4096, },
  { 1, 8, 16, 1024, 2048, 4096, },
  { 2, 8, 16, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 1024, 2048, 4096, },
  { 4, 8, 16, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 1024, 2048, 4096, },
  { 32, 1024, 2048, 4096, },
  { 1, 32, 1024, 2048, 4096, },
  { 2, 32, 1024, 2048, 4096, },
  { 1, 2, 32, 1024, 2048, 4096, },
  { 4, 32, 1024, 2048, 4096, },
  { 1, 4, 32, 1024, 2048, 4096, },
  { 2, 4, 32, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 1024, 2048, 4096, },
  { 8, 32, 1024, 2048, 4096, },
  { 1, 8, 32, 1024, 2048, 4096, },
  { 2, 8, 32, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 1024, 2048, 4096, },
  { 4, 8, 32, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 1024, 2048, 4096, },
  { 16, 32, 1024, 2048, 4096, },
  { 1, 16, 32, 1024, 2048, 4096, },
  { 2, 16, 32, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 1024, 2048, 4096, },
  { 4, 16, 32, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 1024, 2048, 4096, },
  { 8, 16, 32, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 4096, },
  { 64, 1024, 2048, 4096, },
  { 1, 64, 1024, 2048, 4096, },
  { 2, 64, 1024, 2048, 4096, },
  { 1, 2, 64, 1024, 2048, 4096, },
  { 4, 64, 1024, 2048, 4096, },
  { 1, 4, 64, 1024, 2048, 4096, },
  { 2, 4, 64, 1024, 2048, 4096, },
  { 1, 2, 4, 64, 1024, 2048, 4096, },
  { 8, 64, 1024, 2048, 4096, },
  { 1, 8, 64, 1024, 2048, 4096, },
  { 2, 8, 64, 1024, 2048, 4096, },
  { 1, 2, 8, 64, 1024, 2048, 4096, },
  { 4, 8, 64, 1024, 2048, 4096, },
  { 1, 4, 8, 64, 1024, 2048, 4096, },
  { 2, 4, 8, 64, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 64, 1024, 2048, 4096, },
  { 16, 64, 1024, 2048, 4096, },
  { 1, 16, 64, 1024, 2048, 4096, },
  { 2, 16, 64, 1024, 2048, 4096, },
  { 1, 2, 16, 64, 1024, 2048, 4096, },
  { 4, 16, 64, 1024, 2048, 4096, },
  { 1, 4, 16, 64, 1024, 2048, 4096, },
  { 2, 4, 16, 64, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 64, 1024, 2048, 4096, },
  { 8, 16, 64, 1024, 2048, 4096, },
  { 1, 8, 16, 64, 1024, 2048, 4096, },
  { 2, 8, 16, 64, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 64, 1024, 2048, 4096, },
  { 4, 8, 16, 64, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 64, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 64, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 4096, },
  { 32, 64, 1024, 2048, 4096, },
  { 1, 32, 64, 1024, 2048, 4096, },
  { 2, 32, 64, 1024, 2048, 4096, },
  { 1, 2, 32, 64, 1024, 2048, 4096, },
  { 4, 32, 64, 1024, 2048, 4096, },
  { 1, 4, 32, 64, 1024, 2048, 4096, },
  { 2, 4, 32, 64, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 64, 1024, 2048, 4096, },
  { 8, 32, 64, 1024, 2048, 4096, },
  { 1, 8, 32, 64, 1024, 2048, 4096, },
  { 2, 8, 32, 64, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 64, 1024, 2048, 4096, },
  { 4, 8, 32, 64, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 64, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 64, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 4096, },
  { 16, 32, 64, 1024, 2048, 4096, },
  { 1, 16, 32, 64, 1024, 2048, 4096, },
  { 2, 16, 32, 64, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 64, 1024, 2048, 4096, },
  { 4, 16, 32, 64, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 64, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 64, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 4096, },
  { 8, 16, 32, 64, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 64, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 64, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 64, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, },
  { 128, 1024, 2048, 4096, },
  { 1, 128, 1024, 2048, 4096, },
  { 2, 128, 1024, 2048, 4096, },
  { 1, 2, 128, 1024, 2048, 4096, },
  { 4, 128, 1024, 2048, 4096, },
  { 1, 4, 128, 1024, 2048, 4096, },
  { 2, 4, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 128, 1024, 2048, 4096, },
  { 8, 128, 1024, 2048, 4096, },
  { 1, 8, 128, 1024, 2048, 4096, },
  { 2, 8, 128, 1024, 2048, 4096, },
  { 1, 2, 8, 128, 1024, 2048, 4096, },
  { 4, 8, 128, 1024, 2048, 4096, },
  { 1, 4, 8, 128, 1024, 2048, 4096, },
  { 2, 4, 8, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 128, 1024, 2048, 4096, },
  { 16, 128, 1024, 2048, 4096, },
  { 1, 16, 128, 1024, 2048, 4096, },
  { 2, 16, 128, 1024, 2048, 4096, },
  { 1, 2, 16, 128, 1024, 2048, 4096, },
  { 4, 16, 128, 1024, 2048, 4096, },
  { 1, 4, 16, 128, 1024, 2048, 4096, },
  { 2, 4, 16, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 128, 1024, 2048, 4096, },
  { 8, 16, 128, 1024, 2048, 4096, },
  { 1, 8, 16, 128, 1024, 2048, 4096, },
  { 2, 8, 16, 128, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 128, 1024, 2048, 4096, },
  { 4, 8, 16, 128, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 128, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 4096, },
  { 32, 128, 1024, 2048, 4096, },
  { 1, 32, 128, 1024, 2048, 4096, },
  { 2, 32, 128, 1024, 2048, 4096, },
  { 1, 2, 32, 128, 1024, 2048, 4096, },
  { 4, 32, 128, 1024, 2048, 4096, },
  { 1, 4, 32, 128, 1024, 2048, 4096, },
  { 2, 4, 32, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 128, 1024, 2048, 4096, },
  { 8, 32, 128, 1024, 2048, 4096, },
  { 1, 8, 32, 128, 1024, 2048, 4096, },
  { 2, 8, 32, 128, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 128, 1024, 2048, 4096, },
  { 4, 8, 32, 128, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 128, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 4096, },
  { 16, 32, 128, 1024, 2048, 4096, },
  { 1, 16, 32, 128, 1024, 2048, 4096, },
  { 2, 16, 32, 128, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 128, 1024, 2048, 4096, },
  { 4, 16, 32, 128, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 128, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 4096, },
  { 8, 16, 32, 128, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 128, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 128, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 128, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, },
  { 64, 128, 1024, 2048, 4096, },
  { 1, 64, 128, 1024, 2048, 4096, },
  { 2, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 64, 128, 1024, 2048, 4096, },
  { 4, 64, 128, 1024, 2048, 4096, },
  { 1, 4, 64, 128, 1024, 2048, 4096, },
  { 2, 4, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 64, 128, 1024, 2048, 4096, },
  { 8, 64, 128, 1024, 2048, 4096, },
  { 1, 8, 64, 128, 1024, 2048, 4096, },
  { 2, 8, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 8, 64, 128, 1024, 2048, 4096, },
  { 4, 8, 64, 128, 1024, 2048, 4096, },
  { 1, 4, 8, 64, 128, 1024, 2048, 4096, },
  { 2, 4, 8, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 4096, },
  { 16, 64, 128, 1024, 2048, 4096, },
  { 1, 16, 64, 128, 1024, 2048, 4096, },
  { 2, 16, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 16, 64, 128, 1024, 2048, 4096, },
  { 4, 16, 64, 128, 1024, 2048, 4096, },
  { 1, 4, 16, 64, 128, 1024, 2048, 4096, },
  { 2, 4, 16, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 4096, },
  { 8, 16, 64, 128, 1024, 2048, 4096, },
  { 1, 8, 16, 64, 128, 1024, 2048, 4096, },
  { 2, 8, 16, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 4096, },
  { 4, 8, 16, 64, 128, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, },
  { 32, 64, 128, 1024, 2048, 4096, },
  { 1, 32, 64, 128, 1024, 2048, 4096, },
  { 2, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 32, 64, 128, 1024, 2048, 4096, },
  { 4, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 4, 32, 64, 128, 1024, 2048, 4096, },
  { 2, 4, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 4096, },
  { 8, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 8, 32, 64, 128, 1024, 2048, 4096, },
  { 2, 8, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 4096, },
  { 4, 8, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, },
  { 16, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 2, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 4, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 8, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, },
  { 256, 1024, 2048, 4096, },
  { 1, 256, 1024, 2048, 4096, },
  { 2, 256, 1024, 2048, 4096, },
  { 1, 2, 256, 1024, 2048, 4096, },
  { 4, 256, 1024, 2048, 4096, },
  { 1, 4, 256, 1024, 2048, 4096, },
  { 2, 4, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 256, 1024, 2048, 4096, },
  { 8, 256, 1024, 2048, 4096, },
  { 1, 8, 256, 1024, 2048, 4096, },
  { 2, 8, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 256, 1024, 2048, 4096, },
  { 4, 8, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 256, 1024, 2048, 4096, },
  { 16, 256, 1024, 2048, 4096, },
  { 1, 16, 256, 1024, 2048, 4096, },
  { 2, 16, 256, 1024, 2048, 4096, },
  { 1, 2, 16, 256, 1024, 2048, 4096, },
  { 4, 16, 256, 1024, 2048, 4096, },
  { 1, 4, 16, 256, 1024, 2048, 4096, },
  { 2, 4, 16, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 256, 1024, 2048, 4096, },
  { 8, 16, 256, 1024, 2048, 4096, },
  { 1, 8, 16, 256, 1024, 2048, 4096, },
  { 2, 8, 16, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 256, 1024, 2048, 4096, },
  { 4, 8, 16, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 4096, },
  { 32, 256, 1024, 2048, 4096, },
  { 1, 32, 256, 1024, 2048, 4096, },
  { 2, 32, 256, 1024, 2048, 4096, },
  { 1, 2, 32, 256, 1024, 2048, 4096, },
  { 4, 32, 256, 1024, 2048, 4096, },
  { 1, 4, 32, 256, 1024, 2048, 4096, },
  { 2, 4, 32, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 256, 1024, 2048, 4096, },
  { 8, 32, 256, 1024, 2048, 4096, },
  { 1, 8, 32, 256, 1024, 2048, 4096, },
  { 2, 8, 32, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 256, 1024, 2048, 4096, },
  { 4, 8, 32, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 4096, },
  { 16, 32, 256, 1024, 2048, 4096, },
  { 1, 16, 32, 256, 1024, 2048, 4096, },
  { 2, 16, 32, 256, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 256, 1024, 2048, 4096, },
  { 4, 16, 32, 256, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 256, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 4096, },
  { 8, 16, 32, 256, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 256, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, },
  { 64, 256, 1024, 2048, 4096, },
  { 1, 64, 256, 1024, 2048, 4096, },
  { 2, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 64, 256, 1024, 2048, 4096, },
  { 4, 64, 256, 1024, 2048, 4096, },
  { 1, 4, 64, 256, 1024, 2048, 4096, },
  { 2, 4, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 64, 256, 1024, 2048, 4096, },
  { 8, 64, 256, 1024, 2048, 4096, },
  { 1, 8, 64, 256, 1024, 2048, 4096, },
  { 2, 8, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 64, 256, 1024, 2048, 4096, },
  { 4, 8, 64, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 64, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 4096, },
  { 16, 64, 256, 1024, 2048, 4096, },
  { 1, 16, 64, 256, 1024, 2048, 4096, },
  { 2, 16, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 16, 64, 256, 1024, 2048, 4096, },
  { 4, 16, 64, 256, 1024, 2048, 4096, },
  { 1, 4, 16, 64, 256, 1024, 2048, 4096, },
  { 2, 4, 16, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 4096, },
  { 8, 16, 64, 256, 1024, 2048, 4096, },
  { 1, 8, 16, 64, 256, 1024, 2048, 4096, },
  { 2, 8, 16, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 4096, },
  { 4, 8, 16, 64, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, },
  { 32, 64, 256, 1024, 2048, 4096, },
  { 1, 32, 64, 256, 1024, 2048, 4096, },
  { 2, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 32, 64, 256, 1024, 2048, 4096, },
  { 4, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 4, 32, 64, 256, 1024, 2048, 4096, },
  { 2, 4, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 4096, },
  { 8, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 8, 32, 64, 256, 1024, 2048, 4096, },
  { 2, 8, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 4096, },
  { 4, 8, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, },
  { 16, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 2, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 4, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 8, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, },
  { 128, 256, 1024, 2048, 4096, },
  { 1, 128, 256, 1024, 2048, 4096, },
  { 2, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 128, 256, 1024, 2048, 4096, },
  { 4, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 128, 256, 1024, 2048, 4096, },
  { 8, 128, 256, 1024, 2048, 4096, },
  { 1, 8, 128, 256, 1024, 2048, 4096, },
  { 2, 8, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 128, 256, 1024, 2048, 4096, },
  { 4, 8, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 4096, },
  { 16, 128, 256, 1024, 2048, 4096, },
  { 1, 16, 128, 256, 1024, 2048, 4096, },
  { 2, 16, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 16, 128, 256, 1024, 2048, 4096, },
  { 4, 16, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 16, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 16, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 4096, },
  { 8, 16, 128, 256, 1024, 2048, 4096, },
  { 1, 8, 16, 128, 256, 1024, 2048, 4096, },
  { 2, 8, 16, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 4096, },
  { 4, 8, 16, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, },
  { 32, 128, 256, 1024, 2048, 4096, },
  { 1, 32, 128, 256, 1024, 2048, 4096, },
  { 2, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 32, 128, 256, 1024, 2048, 4096, },
  { 4, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 32, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 4096, },
  { 8, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 8, 32, 128, 256, 1024, 2048, 4096, },
  { 2, 8, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 4096, },
  { 4, 8, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, },
  { 16, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 2, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 4, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 8, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, },
  { 64, 128, 256, 1024, 2048, 4096, },
  { 1, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 64, 128, 256, 1024, 2048, 4096, },
  { 4, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 4096, },
  { 8, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 8, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 8, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 4096, },
  { 4, 8, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, },
  { 16, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 4, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 8, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, },
  { 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 4, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 8, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, },
  { 512, 1024, 2048, 4096, },
  { 1, 512, 1024, 2048, 4096, },
  { 2, 512, 1024, 2048, 4096, },
  { 1, 2, 512, 1024, 2048, 4096, },
  { 4, 512, 1024, 2048, 4096, },
  { 1, 4, 512, 1024, 2048, 4096, },
  { 2, 4, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 512, 1024, 2048, 4096, },
  { 8, 512, 1024, 2048, 4096, },
  { 1, 8, 512, 1024, 2048, 4096, },
  { 2, 8, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 512, 1024, 2048, 4096, },
  { 4, 8, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 512, 1024, 2048, 4096, },
  { 16, 512, 1024, 2048, 4096, },
  { 1, 16, 512, 1024, 2048, 4096, },
  { 2, 16, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 512, 1024, 2048, 4096, },
  { 4, 16, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 512, 1024, 2048, 4096, },
  { 8, 16, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 4096, },
  { 32, 512, 1024, 2048, 4096, },
  { 1, 32, 512, 1024, 2048, 4096, },
  { 2, 32, 512, 1024, 2048, 4096, },
  { 1, 2, 32, 512, 1024, 2048, 4096, },
  { 4, 32, 512, 1024, 2048, 4096, },
  { 1, 4, 32, 512, 1024, 2048, 4096, },
  { 2, 4, 32, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 512, 1024, 2048, 4096, },
  { 8, 32, 512, 1024, 2048, 4096, },
  { 1, 8, 32, 512, 1024, 2048, 4096, },
  { 2, 8, 32, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 512, 1024, 2048, 4096, },
  { 4, 8, 32, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 4096, },
  { 16, 32, 512, 1024, 2048, 4096, },
  { 1, 16, 32, 512, 1024, 2048, 4096, },
  { 2, 16, 32, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 512, 1024, 2048, 4096, },
  { 4, 16, 32, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 4096, },
  { 8, 16, 32, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, },
  { 64, 512, 1024, 2048, 4096, },
  { 1, 64, 512, 1024, 2048, 4096, },
  { 2, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 64, 512, 1024, 2048, 4096, },
  { 4, 64, 512, 1024, 2048, 4096, },
  { 1, 4, 64, 512, 1024, 2048, 4096, },
  { 2, 4, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 64, 512, 1024, 2048, 4096, },
  { 8, 64, 512, 1024, 2048, 4096, },
  { 1, 8, 64, 512, 1024, 2048, 4096, },
  { 2, 8, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 64, 512, 1024, 2048, 4096, },
  { 4, 8, 64, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 64, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 4096, },
  { 16, 64, 512, 1024, 2048, 4096, },
  { 1, 16, 64, 512, 1024, 2048, 4096, },
  { 2, 16, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 64, 512, 1024, 2048, 4096, },
  { 4, 16, 64, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 64, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 4096, },
  { 8, 16, 64, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 64, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 64, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, },
  { 32, 64, 512, 1024, 2048, 4096, },
  { 1, 32, 64, 512, 1024, 2048, 4096, },
  { 2, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 32, 64, 512, 1024, 2048, 4096, },
  { 4, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 4, 32, 64, 512, 1024, 2048, 4096, },
  { 2, 4, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 4096, },
  { 8, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 8, 32, 64, 512, 1024, 2048, 4096, },
  { 2, 8, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 4096, },
  { 4, 8, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, },
  { 16, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 2, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 4, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 8, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, },
  { 128, 512, 1024, 2048, 4096, },
  { 1, 128, 512, 1024, 2048, 4096, },
  { 2, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 128, 512, 1024, 2048, 4096, },
  { 4, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 128, 512, 1024, 2048, 4096, },
  { 8, 128, 512, 1024, 2048, 4096, },
  { 1, 8, 128, 512, 1024, 2048, 4096, },
  { 2, 8, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 128, 512, 1024, 2048, 4096, },
  { 4, 8, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 4096, },
  { 16, 128, 512, 1024, 2048, 4096, },
  { 1, 16, 128, 512, 1024, 2048, 4096, },
  { 2, 16, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 128, 512, 1024, 2048, 4096, },
  { 4, 16, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 4096, },
  { 8, 16, 128, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 128, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, },
  { 32, 128, 512, 1024, 2048, 4096, },
  { 1, 32, 128, 512, 1024, 2048, 4096, },
  { 2, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 32, 128, 512, 1024, 2048, 4096, },
  { 4, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 32, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 4096, },
  { 8, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 8, 32, 128, 512, 1024, 2048, 4096, },
  { 2, 8, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 4096, },
  { 4, 8, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, },
  { 16, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 2, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 4, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 8, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, },
  { 64, 128, 512, 1024, 2048, 4096, },
  { 1, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 64, 128, 512, 1024, 2048, 4096, },
  { 4, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 4096, },
  { 8, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 8, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 8, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 4096, },
  { 4, 8, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, },
  { 16, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 4, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 8, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, },
  { 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 4, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 8, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, },
  { 256, 512, 1024, 2048, 4096, },
  { 1, 256, 512, 1024, 2048, 4096, },
  { 2, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 256, 512, 1024, 2048, 4096, },
  { 4, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 256, 512, 1024, 2048, 4096, },
  { 8, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 4096, },
  { 16, 256, 512, 1024, 2048, 4096, },
  { 1, 16, 256, 512, 1024, 2048, 4096, },
  { 2, 16, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 256, 512, 1024, 2048, 4096, },
  { 4, 16, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 4096, },
  { 8, 16, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, },
  { 32, 256, 512, 1024, 2048, 4096, },
  { 1, 32, 256, 512, 1024, 2048, 4096, },
  { 2, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 32, 256, 512, 1024, 2048, 4096, },
  { 4, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 32, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 4096, },
  { 8, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 32, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, },
  { 16, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 2, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 4, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 8, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, },
  { 64, 256, 512, 1024, 2048, 4096, },
  { 1, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 64, 256, 512, 1024, 2048, 4096, },
  { 4, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 4096, },
  { 8, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, },
  { 16, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 4, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 8, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, },
  { 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 4, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 8, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, },
  { 128, 256, 512, 1024, 2048, 4096, },
  { 1, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 4096, },
  { 8, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, },
  { 16, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 8, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, },
  { 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 8, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, },
  { 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 8, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, },
  { 8192, },
  { 1, 8192, },
  { 2, 8192, },
  { 1, 2, 8192, },
  { 4, 8192, },
  { 1, 4, 8192, },
  { 2, 4, 8192, },
  { 1, 2, 4, 8192, },
  { 8, 8192, },
  { 1, 8, 8192, },
  { 2, 8, 8192, },
  { 1, 2, 8, 8192, },
  { 4, 8, 8192, },
  { 1, 4, 8, 8192, },
  { 2, 4, 8, 8192, },
  { 1, 2, 4, 8, 8192, },
  { 16, 8192, },
  { 1, 16, 8192, },
  { 2, 16, 8192, },
  { 1, 2, 16, 8192, },
  { 4, 16, 8192, },
  { 1, 4, 16, 8192, },
  { 2, 4, 16, 8192, },
  { 1, 2, 4, 16, 8192, },
  { 8, 16, 8192, },
  { 1, 8, 16, 8192, },
  { 2, 8, 16, 8192, },
  { 1, 2, 8, 16, 8192, },
  { 4, 8, 16, 8192, },
  { 1, 4, 8, 16, 8192, },
  { 2, 4, 8, 16, 8192, },
  { 1, 2, 4, 8, 16, 8192, },
  { 32, 8192, },
  { 1, 32, 8192, },
  { 2, 32, 8192, },
  { 1, 2, 32, 8192, },
  { 4, 32, 8192, },
  { 1, 4, 32, 8192, },
  { 2, 4, 32, 8192, },
  { 1, 2, 4, 32, 8192, },
  { 8, 32, 8192, },
  { 1, 8, 32, 8192, },
  { 2, 8, 32, 8192, },
  { 1, 2, 8, 32, 8192, },
  { 4, 8, 32, 8192, },
  { 1, 4, 8, 32, 8192, },
  { 2, 4, 8, 32, 8192, },
  { 1, 2, 4, 8, 32, 8192, },
  { 16, 32, 8192, },
  { 1, 16, 32, 8192, },
  { 2, 16, 32, 8192, },
  { 1, 2, 16, 32, 8192, },
  { 4, 16, 32, 8192, },
  { 1, 4, 16, 32, 8192, },
  { 2, 4, 16, 32, 8192, },
  { 1, 2, 4, 16, 32, 8192, },
  { 8, 16, 32, 8192, },
  { 1, 8, 16, 32, 8192, },
  { 2, 8, 16, 32, 8192, },
  { 1, 2, 8, 16, 32, 8192, },
  { 4, 8, 16, 32, 8192, },
  { 1, 4, 8, 16, 32, 8192, },
  { 2, 4, 8, 16, 32, 8192, },
  { 1, 2, 4, 8, 16, 32, 8192, },
  { 64, 8192, },
  { 1, 64, 8192, },
  { 2, 64, 8192, },
  { 1, 2, 64, 8192, },
  { 4, 64, 8192, },
  { 1, 4, 64, 8192, },
  { 2, 4, 64, 8192, },
  { 1, 2, 4, 64, 8192, },
  { 8, 64, 8192, },
  { 1, 8, 64, 8192, },
  { 2, 8, 64, 8192, },
  { 1, 2, 8, 64, 8192, },
  { 4, 8, 64, 8192, },
  { 1, 4, 8, 64, 8192, },
  { 2, 4, 8, 64, 8192, },
  { 1, 2, 4, 8, 64, 8192, },
  { 16, 64, 8192, },
  { 1, 16, 64, 8192, },
  { 2, 16, 64, 8192, },
  { 1, 2, 16, 64, 8192, },
  { 4, 16, 64, 8192, },
  { 1, 4, 16, 64, 8192, },
  { 2, 4, 16, 64, 8192, },
  { 1, 2, 4, 16, 64, 8192, },
  { 8, 16, 64, 8192, },
  { 1, 8, 16, 64, 8192, },
  { 2, 8, 16, 64, 8192, },
  { 1, 2, 8, 16, 64, 8192, },
  { 4, 8, 16, 64, 8192, },
  { 1, 4, 8, 16, 64, 8192, },
  { 2, 4, 8, 16, 64, 8192, },
  { 1, 2, 4, 8, 16, 64, 8192, },
  { 32, 64, 8192, },
  { 1, 32, 64, 8192, },
  { 2, 32, 64, 8192, },
  { 1, 2, 32, 64, 8192, },
  { 4, 32, 64, 8192, },
  { 1, 4, 32, 64, 8192, },
  { 2, 4, 32, 64, 8192, },
  { 1, 2, 4, 32, 64, 8192, },
  { 8, 32, 64, 8192, },
  { 1, 8, 32, 64, 8192, },
  { 2, 8, 32, 64, 8192, },
  { 1, 2, 8, 32, 64, 8192, },
  { 4, 8, 32, 64, 8192, },
  { 1, 4, 8, 32, 64, 8192, },
  { 2, 4, 8, 32, 64, 8192, },
  { 1, 2, 4, 8, 32, 64, 8192, },
  { 16, 32, 64, 8192, },
  { 1, 16, 32, 64, 8192, },
  { 2, 16, 32, 64, 8192, },
  { 1, 2, 16, 32, 64, 8192, },
  { 4, 16, 32, 64, 8192, },
  { 1, 4, 16, 32, 64, 8192, },
  { 2, 4, 16, 32, 64, 8192, },
  { 1, 2, 4, 16, 32, 64, 8192, },
  { 8, 16, 32, 64, 8192, },
  { 1, 8, 16, 32, 64, 8192, },
  { 2, 8, 16, 32, 64, 8192, },
  { 1, 2, 8, 16, 32, 64, 8192, },
  { 4, 8, 16, 32, 64, 8192, },
  { 1, 4, 8, 16, 32, 64, 8192, },
  { 2, 4, 8, 16, 32, 64, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 8192, },
  { 128, 8192, },
  { 1, 128, 8192, },
  { 2, 128, 8192, },
  { 1, 2, 128, 8192, },
  { 4, 128, 8192, },
  { 1, 4, 128, 8192, },
  { 2, 4, 128, 8192, },
  { 1, 2, 4, 128, 8192, },
  { 8, 128, 8192, },
  { 1, 8, 128, 8192, },
  { 2, 8, 128, 8192, },
  { 1, 2, 8, 128, 8192, },
  { 4, 8, 128, 8192, },
  { 1, 4, 8, 128, 8192, },
  { 2, 4, 8, 128, 8192, },
  { 1, 2, 4, 8, 128, 8192, },
  { 16, 128, 8192, },
  { 1, 16, 128, 8192, },
  { 2, 16, 128, 8192, },
  { 1, 2, 16, 128, 8192, },
  { 4, 16, 128, 8192, },
  { 1, 4, 16, 128, 8192, },
  { 2, 4, 16, 128, 8192, },
  { 1, 2, 4, 16, 128, 8192, },
  { 8, 16, 128, 8192, },
  { 1, 8, 16, 128, 8192, },
  { 2, 8, 16, 128, 8192, },
  { 1, 2, 8, 16, 128, 8192, },
  { 4, 8, 16, 128, 8192, },
  { 1, 4, 8, 16, 128, 8192, },
  { 2, 4, 8, 16, 128, 8192, },
  { 1, 2, 4, 8, 16, 128, 8192, },
  { 32, 128, 8192, },
  { 1, 32, 128, 8192, },
  { 2, 32, 128, 8192, },
  { 1, 2, 32, 128, 8192, },
  { 4, 32, 128, 8192, },
  { 1, 4, 32, 128, 8192, },
  { 2, 4, 32, 128, 8192, },
  { 1, 2, 4, 32, 128, 8192, },
  { 8, 32, 128, 8192, },
  { 1, 8, 32, 128, 8192, },
  { 2, 8, 32, 128, 8192, },
  { 1, 2, 8, 32, 128, 8192, },
  { 4, 8, 32, 128, 8192, },
  { 1, 4, 8, 32, 128, 8192, },
  { 2, 4, 8, 32, 128, 8192, },
  { 1, 2, 4, 8, 32, 128, 8192, },
  { 16, 32, 128, 8192, },
  { 1, 16, 32, 128, 8192, },
  { 2, 16, 32, 128, 8192, },
  { 1, 2, 16, 32, 128, 8192, },
  { 4, 16, 32, 128, 8192, },
  { 1, 4, 16, 32, 128, 8192, },
  { 2, 4, 16, 32, 128, 8192, },
  { 1, 2, 4, 16, 32, 128, 8192, },
  { 8, 16, 32, 128, 8192, },
  { 1, 8, 16, 32, 128, 8192, },
  { 2, 8, 16, 32, 128, 8192, },
  { 1, 2, 8, 16, 32, 128, 8192, },
  { 4, 8, 16, 32, 128, 8192, },
  { 1, 4, 8, 16, 32, 128, 8192, },
  { 2, 4, 8, 16, 32, 128, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 8192, },
  { 64, 128, 8192, },
  { 1, 64, 128, 8192, },
  { 2, 64, 128, 8192, },
  { 1, 2, 64, 128, 8192, },
  { 4, 64, 128, 8192, },
  { 1, 4, 64, 128, 8192, },
  { 2, 4, 64, 128, 8192, },
  { 1, 2, 4, 64, 128, 8192, },
  { 8, 64, 128, 8192, },
  { 1, 8, 64, 128, 8192, },
  { 2, 8, 64, 128, 8192, },
  { 1, 2, 8, 64, 128, 8192, },
  { 4, 8, 64, 128, 8192, },
  { 1, 4, 8, 64, 128, 8192, },
  { 2, 4, 8, 64, 128, 8192, },
  { 1, 2, 4, 8, 64, 128, 8192, },
  { 16, 64, 128, 8192, },
  { 1, 16, 64, 128, 8192, },
  { 2, 16, 64, 128, 8192, },
  { 1, 2, 16, 64, 128, 8192, },
  { 4, 16, 64, 128, 8192, },
  { 1, 4, 16, 64, 128, 8192, },
  { 2, 4, 16, 64, 128, 8192, },
  { 1, 2, 4, 16, 64, 128, 8192, },
  { 8, 16, 64, 128, 8192, },
  { 1, 8, 16, 64, 128, 8192, },
  { 2, 8, 16, 64, 128, 8192, },
  { 1, 2, 8, 16, 64, 128, 8192, },
  { 4, 8, 16, 64, 128, 8192, },
  { 1, 4, 8, 16, 64, 128, 8192, },
  { 2, 4, 8, 16, 64, 128, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 8192, },
  { 32, 64, 128, 8192, },
  { 1, 32, 64, 128, 8192, },
  { 2, 32, 64, 128, 8192, },
  { 1, 2, 32, 64, 128, 8192, },
  { 4, 32, 64, 128, 8192, },
  { 1, 4, 32, 64, 128, 8192, },
  { 2, 4, 32, 64, 128, 8192, },
  { 1, 2, 4, 32, 64, 128, 8192, },
  { 8, 32, 64, 128, 8192, },
  { 1, 8, 32, 64, 128, 8192, },
  { 2, 8, 32, 64, 128, 8192, },
  { 1, 2, 8, 32, 64, 128, 8192, },
  { 4, 8, 32, 64, 128, 8192, },
  { 1, 4, 8, 32, 64, 128, 8192, },
  { 2, 4, 8, 32, 64, 128, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 8192, },
  { 16, 32, 64, 128, 8192, },
  { 1, 16, 32, 64, 128, 8192, },
  { 2, 16, 32, 64, 128, 8192, },
  { 1, 2, 16, 32, 64, 128, 8192, },
  { 4, 16, 32, 64, 128, 8192, },
  { 1, 4, 16, 32, 64, 128, 8192, },
  { 2, 4, 16, 32, 64, 128, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 8192, },
  { 8, 16, 32, 64, 128, 8192, },
  { 1, 8, 16, 32, 64, 128, 8192, },
  { 2, 8, 16, 32, 64, 128, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 8192, },
  { 4, 8, 16, 32, 64, 128, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 8192, },
  { 256, 8192, },
  { 1, 256, 8192, },
  { 2, 256, 8192, },
  { 1, 2, 256, 8192, },
  { 4, 256, 8192, },
  { 1, 4, 256, 8192, },
  { 2, 4, 256, 8192, },
  { 1, 2, 4, 256, 8192, },
  { 8, 256, 8192, },
  { 1, 8, 256, 8192, },
  { 2, 8, 256, 8192, },
  { 1, 2, 8, 256, 8192, },
  { 4, 8, 256, 8192, },
  { 1, 4, 8, 256, 8192, },
  { 2, 4, 8, 256, 8192, },
  { 1, 2, 4, 8, 256, 8192, },
  { 16, 256, 8192, },
  { 1, 16, 256, 8192, },
  { 2, 16, 256, 8192, },
  { 1, 2, 16, 256, 8192, },
  { 4, 16, 256, 8192, },
  { 1, 4, 16, 256, 8192, },
  { 2, 4, 16, 256, 8192, },
  { 1, 2, 4, 16, 256, 8192, },
  { 8, 16, 256, 8192, },
  { 1, 8, 16, 256, 8192, },
  { 2, 8, 16, 256, 8192, },
  { 1, 2, 8, 16, 256, 8192, },
  { 4, 8, 16, 256, 8192, },
  { 1, 4, 8, 16, 256, 8192, },
  { 2, 4, 8, 16, 256, 8192, },
  { 1, 2, 4, 8, 16, 256, 8192, },
  { 32, 256, 8192, },
  { 1, 32, 256, 8192, },
  { 2, 32, 256, 8192, },
  { 1, 2, 32, 256, 8192, },
  { 4, 32, 256, 8192, },
  { 1, 4, 32, 256, 8192, },
  { 2, 4, 32, 256, 8192, },
  { 1, 2, 4, 32, 256, 8192, },
  { 8, 32, 256, 8192, },
  { 1, 8, 32, 256, 8192, },
  { 2, 8, 32, 256, 8192, },
  { 1, 2, 8, 32, 256, 8192, },
  { 4, 8, 32, 256, 8192, },
  { 1, 4, 8, 32, 256, 8192, },
  { 2, 4, 8, 32, 256, 8192, },
  { 1, 2, 4, 8, 32, 256, 8192, },
  { 16, 32, 256, 8192, },
  { 1, 16, 32, 256, 8192, },
  { 2, 16, 32, 256, 8192, },
  { 1, 2, 16, 32, 256, 8192, },
  { 4, 16, 32, 256, 8192, },
  { 1, 4, 16, 32, 256, 8192, },
  { 2, 4, 16, 32, 256, 8192, },
  { 1, 2, 4, 16, 32, 256, 8192, },
  { 8, 16, 32, 256, 8192, },
  { 1, 8, 16, 32, 256, 8192, },
  { 2, 8, 16, 32, 256, 8192, },
  { 1, 2, 8, 16, 32, 256, 8192, },
  { 4, 8, 16, 32, 256, 8192, },
  { 1, 4, 8, 16, 32, 256, 8192, },
  { 2, 4, 8, 16, 32, 256, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 8192, },
  { 64, 256, 8192, },
  { 1, 64, 256, 8192, },
  { 2, 64, 256, 8192, },
  { 1, 2, 64, 256, 8192, },
  { 4, 64, 256, 8192, },
  { 1, 4, 64, 256, 8192, },
  { 2, 4, 64, 256, 8192, },
  { 1, 2, 4, 64, 256, 8192, },
  { 8, 64, 256, 8192, },
  { 1, 8, 64, 256, 8192, },
  { 2, 8, 64, 256, 8192, },
  { 1, 2, 8, 64, 256, 8192, },
  { 4, 8, 64, 256, 8192, },
  { 1, 4, 8, 64, 256, 8192, },
  { 2, 4, 8, 64, 256, 8192, },
  { 1, 2, 4, 8, 64, 256, 8192, },
  { 16, 64, 256, 8192, },
  { 1, 16, 64, 256, 8192, },
  { 2, 16, 64, 256, 8192, },
  { 1, 2, 16, 64, 256, 8192, },
  { 4, 16, 64, 256, 8192, },
  { 1, 4, 16, 64, 256, 8192, },
  { 2, 4, 16, 64, 256, 8192, },
  { 1, 2, 4, 16, 64, 256, 8192, },
  { 8, 16, 64, 256, 8192, },
  { 1, 8, 16, 64, 256, 8192, },
  { 2, 8, 16, 64, 256, 8192, },
  { 1, 2, 8, 16, 64, 256, 8192, },
  { 4, 8, 16, 64, 256, 8192, },
  { 1, 4, 8, 16, 64, 256, 8192, },
  { 2, 4, 8, 16, 64, 256, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 8192, },
  { 32, 64, 256, 8192, },
  { 1, 32, 64, 256, 8192, },
  { 2, 32, 64, 256, 8192, },
  { 1, 2, 32, 64, 256, 8192, },
  { 4, 32, 64, 256, 8192, },
  { 1, 4, 32, 64, 256, 8192, },
  { 2, 4, 32, 64, 256, 8192, },
  { 1, 2, 4, 32, 64, 256, 8192, },
  { 8, 32, 64, 256, 8192, },
  { 1, 8, 32, 64, 256, 8192, },
  { 2, 8, 32, 64, 256, 8192, },
  { 1, 2, 8, 32, 64, 256, 8192, },
  { 4, 8, 32, 64, 256, 8192, },
  { 1, 4, 8, 32, 64, 256, 8192, },
  { 2, 4, 8, 32, 64, 256, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 8192, },
  { 16, 32, 64, 256, 8192, },
  { 1, 16, 32, 64, 256, 8192, },
  { 2, 16, 32, 64, 256, 8192, },
  { 1, 2, 16, 32, 64, 256, 8192, },
  { 4, 16, 32, 64, 256, 8192, },
  { 1, 4, 16, 32, 64, 256, 8192, },
  { 2, 4, 16, 32, 64, 256, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 8192, },
  { 8, 16, 32, 64, 256, 8192, },
  { 1, 8, 16, 32, 64, 256, 8192, },
  { 2, 8, 16, 32, 64, 256, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 8192, },
  { 4, 8, 16, 32, 64, 256, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 8192, },
  { 128, 256, 8192, },
  { 1, 128, 256, 8192, },
  { 2, 128, 256, 8192, },
  { 1, 2, 128, 256, 8192, },
  { 4, 128, 256, 8192, },
  { 1, 4, 128, 256, 8192, },
  { 2, 4, 128, 256, 8192, },
  { 1, 2, 4, 128, 256, 8192, },
  { 8, 128, 256, 8192, },
  { 1, 8, 128, 256, 8192, },
  { 2, 8, 128, 256, 8192, },
  { 1, 2, 8, 128, 256, 8192, },
  { 4, 8, 128, 256, 8192, },
  { 1, 4, 8, 128, 256, 8192, },
  { 2, 4, 8, 128, 256, 8192, },
  { 1, 2, 4, 8, 128, 256, 8192, },
  { 16, 128, 256, 8192, },
  { 1, 16, 128, 256, 8192, },
  { 2, 16, 128, 256, 8192, },
  { 1, 2, 16, 128, 256, 8192, },
  { 4, 16, 128, 256, 8192, },
  { 1, 4, 16, 128, 256, 8192, },
  { 2, 4, 16, 128, 256, 8192, },
  { 1, 2, 4, 16, 128, 256, 8192, },
  { 8, 16, 128, 256, 8192, },
  { 1, 8, 16, 128, 256, 8192, },
  { 2, 8, 16, 128, 256, 8192, },
  { 1, 2, 8, 16, 128, 256, 8192, },
  { 4, 8, 16, 128, 256, 8192, },
  { 1, 4, 8, 16, 128, 256, 8192, },
  { 2, 4, 8, 16, 128, 256, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 8192, },
  { 32, 128, 256, 8192, },
  { 1, 32, 128, 256, 8192, },
  { 2, 32, 128, 256, 8192, },
  { 1, 2, 32, 128, 256, 8192, },
  { 4, 32, 128, 256, 8192, },
  { 1, 4, 32, 128, 256, 8192, },
  { 2, 4, 32, 128, 256, 8192, },
  { 1, 2, 4, 32, 128, 256, 8192, },
  { 8, 32, 128, 256, 8192, },
  { 1, 8, 32, 128, 256, 8192, },
  { 2, 8, 32, 128, 256, 8192, },
  { 1, 2, 8, 32, 128, 256, 8192, },
  { 4, 8, 32, 128, 256, 8192, },
  { 1, 4, 8, 32, 128, 256, 8192, },
  { 2, 4, 8, 32, 128, 256, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 8192, },
  { 16, 32, 128, 256, 8192, },
  { 1, 16, 32, 128, 256, 8192, },
  { 2, 16, 32, 128, 256, 8192, },
  { 1, 2, 16, 32, 128, 256, 8192, },
  { 4, 16, 32, 128, 256, 8192, },
  { 1, 4, 16, 32, 128, 256, 8192, },
  { 2, 4, 16, 32, 128, 256, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 8192, },
  { 8, 16, 32, 128, 256, 8192, },
  { 1, 8, 16, 32, 128, 256, 8192, },
  { 2, 8, 16, 32, 128, 256, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 8192, },
  { 4, 8, 16, 32, 128, 256, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 8192, },
  { 64, 128, 256, 8192, },
  { 1, 64, 128, 256, 8192, },
  { 2, 64, 128, 256, 8192, },
  { 1, 2, 64, 128, 256, 8192, },
  { 4, 64, 128, 256, 8192, },
  { 1, 4, 64, 128, 256, 8192, },
  { 2, 4, 64, 128, 256, 8192, },
  { 1, 2, 4, 64, 128, 256, 8192, },
  { 8, 64, 128, 256, 8192, },
  { 1, 8, 64, 128, 256, 8192, },
  { 2, 8, 64, 128, 256, 8192, },
  { 1, 2, 8, 64, 128, 256, 8192, },
  { 4, 8, 64, 128, 256, 8192, },
  { 1, 4, 8, 64, 128, 256, 8192, },
  { 2, 4, 8, 64, 128, 256, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 8192, },
  { 16, 64, 128, 256, 8192, },
  { 1, 16, 64, 128, 256, 8192, },
  { 2, 16, 64, 128, 256, 8192, },
  { 1, 2, 16, 64, 128, 256, 8192, },
  { 4, 16, 64, 128, 256, 8192, },
  { 1, 4, 16, 64, 128, 256, 8192, },
  { 2, 4, 16, 64, 128, 256, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 8192, },
  { 8, 16, 64, 128, 256, 8192, },
  { 1, 8, 16, 64, 128, 256, 8192, },
  { 2, 8, 16, 64, 128, 256, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 8192, },
  { 4, 8, 16, 64, 128, 256, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 8192, },
  { 32, 64, 128, 256, 8192, },
  { 1, 32, 64, 128, 256, 8192, },
  { 2, 32, 64, 128, 256, 8192, },
  { 1, 2, 32, 64, 128, 256, 8192, },
  { 4, 32, 64, 128, 256, 8192, },
  { 1, 4, 32, 64, 128, 256, 8192, },
  { 2, 4, 32, 64, 128, 256, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 8192, },
  { 8, 32, 64, 128, 256, 8192, },
  { 1, 8, 32, 64, 128, 256, 8192, },
  { 2, 8, 32, 64, 128, 256, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 8192, },
  { 4, 8, 32, 64, 128, 256, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 8192, },
  { 16, 32, 64, 128, 256, 8192, },
  { 1, 16, 32, 64, 128, 256, 8192, },
  { 2, 16, 32, 64, 128, 256, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 8192, },
  { 4, 16, 32, 64, 128, 256, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 8192, },
  { 8, 16, 32, 64, 128, 256, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 8192, },
  { 512, 8192, },
  { 1, 512, 8192, },
  { 2, 512, 8192, },
  { 1, 2, 512, 8192, },
  { 4, 512, 8192, },
  { 1, 4, 512, 8192, },
  { 2, 4, 512, 8192, },
  { 1, 2, 4, 512, 8192, },
  { 8, 512, 8192, },
  { 1, 8, 512, 8192, },
  { 2, 8, 512, 8192, },
  { 1, 2, 8, 512, 8192, },
  { 4, 8, 512, 8192, },
  { 1, 4, 8, 512, 8192, },
  { 2, 4, 8, 512, 8192, },
  { 1, 2, 4, 8, 512, 8192, },
  { 16, 512, 8192, },
  { 1, 16, 512, 8192, },
  { 2, 16, 512, 8192, },
  { 1, 2, 16, 512, 8192, },
  { 4, 16, 512, 8192, },
  { 1, 4, 16, 512, 8192, },
  { 2, 4, 16, 512, 8192, },
  { 1, 2, 4, 16, 512, 8192, },
  { 8, 16, 512, 8192, },
  { 1, 8, 16, 512, 8192, },
  { 2, 8, 16, 512, 8192, },
  { 1, 2, 8, 16, 512, 8192, },
  { 4, 8, 16, 512, 8192, },
  { 1, 4, 8, 16, 512, 8192, },
  { 2, 4, 8, 16, 512, 8192, },
  { 1, 2, 4, 8, 16, 512, 8192, },
  { 32, 512, 8192, },
  { 1, 32, 512, 8192, },
  { 2, 32, 512, 8192, },
  { 1, 2, 32, 512, 8192, },
  { 4, 32, 512, 8192, },
  { 1, 4, 32, 512, 8192, },
  { 2, 4, 32, 512, 8192, },
  { 1, 2, 4, 32, 512, 8192, },
  { 8, 32, 512, 8192, },
  { 1, 8, 32, 512, 8192, },
  { 2, 8, 32, 512, 8192, },
  { 1, 2, 8, 32, 512, 8192, },
  { 4, 8, 32, 512, 8192, },
  { 1, 4, 8, 32, 512, 8192, },
  { 2, 4, 8, 32, 512, 8192, },
  { 1, 2, 4, 8, 32, 512, 8192, },
  { 16, 32, 512, 8192, },
  { 1, 16, 32, 512, 8192, },
  { 2, 16, 32, 512, 8192, },
  { 1, 2, 16, 32, 512, 8192, },
  { 4, 16, 32, 512, 8192, },
  { 1, 4, 16, 32, 512, 8192, },
  { 2, 4, 16, 32, 512, 8192, },
  { 1, 2, 4, 16, 32, 512, 8192, },
  { 8, 16, 32, 512, 8192, },
  { 1, 8, 16, 32, 512, 8192, },
  { 2, 8, 16, 32, 512, 8192, },
  { 1, 2, 8, 16, 32, 512, 8192, },
  { 4, 8, 16, 32, 512, 8192, },
  { 1, 4, 8, 16, 32, 512, 8192, },
  { 2, 4, 8, 16, 32, 512, 8192, },
  { 1, 2, 4, 8, 16, 32, 512, 8192, },
  { 64, 512, 8192, },
  { 1, 64, 512, 8192, },
  { 2, 64, 512, 8192, },
  { 1, 2, 64, 512, 8192, },
  { 4, 64, 512, 8192, },
  { 1, 4, 64, 512, 8192, },
  { 2, 4, 64, 512, 8192, },
  { 1, 2, 4, 64, 512, 8192, },
  { 8, 64, 512, 8192, },
  { 1, 8, 64, 512, 8192, },
  { 2, 8, 64, 512, 8192, },
  { 1, 2, 8, 64, 512, 8192, },
  { 4, 8, 64, 512, 8192, },
  { 1, 4, 8, 64, 512, 8192, },
  { 2, 4, 8, 64, 512, 8192, },
  { 1, 2, 4, 8, 64, 512, 8192, },
  { 16, 64, 512, 8192, },
  { 1, 16, 64, 512, 8192, },
  { 2, 16, 64, 512, 8192, },
  { 1, 2, 16, 64, 512, 8192, },
  { 4, 16, 64, 512, 8192, },
  { 1, 4, 16, 64, 512, 8192, },
  { 2, 4, 16, 64, 512, 8192, },
  { 1, 2, 4, 16, 64, 512, 8192, },
  { 8, 16, 64, 512, 8192, },
  { 1, 8, 16, 64, 512, 8192, },
  { 2, 8, 16, 64, 512, 8192, },
  { 1, 2, 8, 16, 64, 512, 8192, },
  { 4, 8, 16, 64, 512, 8192, },
  { 1, 4, 8, 16, 64, 512, 8192, },
  { 2, 4, 8, 16, 64, 512, 8192, },
  { 1, 2, 4, 8, 16, 64, 512, 8192, },
  { 32, 64, 512, 8192, },
  { 1, 32, 64, 512, 8192, },
  { 2, 32, 64, 512, 8192, },
  { 1, 2, 32, 64, 512, 8192, },
  { 4, 32, 64, 512, 8192, },
  { 1, 4, 32, 64, 512, 8192, },
  { 2, 4, 32, 64, 512, 8192, },
  { 1, 2, 4, 32, 64, 512, 8192, },
  { 8, 32, 64, 512, 8192, },
  { 1, 8, 32, 64, 512, 8192, },
  { 2, 8, 32, 64, 512, 8192, },
  { 1, 2, 8, 32, 64, 512, 8192, },
  { 4, 8, 32, 64, 512, 8192, },
  { 1, 4, 8, 32, 64, 512, 8192, },
  { 2, 4, 8, 32, 64, 512, 8192, },
  { 1, 2, 4, 8, 32, 64, 512, 8192, },
  { 16, 32, 64, 512, 8192, },
  { 1, 16, 32, 64, 512, 8192, },
  { 2, 16, 32, 64, 512, 8192, },
  { 1, 2, 16, 32, 64, 512, 8192, },
  { 4, 16, 32, 64, 512, 8192, },
  { 1, 4, 16, 32, 64, 512, 8192, },
  { 2, 4, 16, 32, 64, 512, 8192, },
  { 1, 2, 4, 16, 32, 64, 512, 8192, },
  { 8, 16, 32, 64, 512, 8192, },
  { 1, 8, 16, 32, 64, 512, 8192, },
  { 2, 8, 16, 32, 64, 512, 8192, },
  { 1, 2, 8, 16, 32, 64, 512, 8192, },
  { 4, 8, 16, 32, 64, 512, 8192, },
  { 1, 4, 8, 16, 32, 64, 512, 8192, },
  { 2, 4, 8, 16, 32, 64, 512, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 8192, },
  { 128, 512, 8192, },
  { 1, 128, 512, 8192, },
  { 2, 128, 512, 8192, },
  { 1, 2, 128, 512, 8192, },
  { 4, 128, 512, 8192, },
  { 1, 4, 128, 512, 8192, },
  { 2, 4, 128, 512, 8192, },
  { 1, 2, 4, 128, 512, 8192, },
  { 8, 128, 512, 8192, },
  { 1, 8, 128, 512, 8192, },
  { 2, 8, 128, 512, 8192, },
  { 1, 2, 8, 128, 512, 8192, },
  { 4, 8, 128, 512, 8192, },
  { 1, 4, 8, 128, 512, 8192, },
  { 2, 4, 8, 128, 512, 8192, },
  { 1, 2, 4, 8, 128, 512, 8192, },
  { 16, 128, 512, 8192, },
  { 1, 16, 128, 512, 8192, },
  { 2, 16, 128, 512, 8192, },
  { 1, 2, 16, 128, 512, 8192, },
  { 4, 16, 128, 512, 8192, },
  { 1, 4, 16, 128, 512, 8192, },
  { 2, 4, 16, 128, 512, 8192, },
  { 1, 2, 4, 16, 128, 512, 8192, },
  { 8, 16, 128, 512, 8192, },
  { 1, 8, 16, 128, 512, 8192, },
  { 2, 8, 16, 128, 512, 8192, },
  { 1, 2, 8, 16, 128, 512, 8192, },
  { 4, 8, 16, 128, 512, 8192, },
  { 1, 4, 8, 16, 128, 512, 8192, },
  { 2, 4, 8, 16, 128, 512, 8192, },
  { 1, 2, 4, 8, 16, 128, 512, 8192, },
  { 32, 128, 512, 8192, },
  { 1, 32, 128, 512, 8192, },
  { 2, 32, 128, 512, 8192, },
  { 1, 2, 32, 128, 512, 8192, },
  { 4, 32, 128, 512, 8192, },
  { 1, 4, 32, 128, 512, 8192, },
  { 2, 4, 32, 128, 512, 8192, },
  { 1, 2, 4, 32, 128, 512, 8192, },
  { 8, 32, 128, 512, 8192, },
  { 1, 8, 32, 128, 512, 8192, },
  { 2, 8, 32, 128, 512, 8192, },
  { 1, 2, 8, 32, 128, 512, 8192, },
  { 4, 8, 32, 128, 512, 8192, },
  { 1, 4, 8, 32, 128, 512, 8192, },
  { 2, 4, 8, 32, 128, 512, 8192, },
  { 1, 2, 4, 8, 32, 128, 512, 8192, },
  { 16, 32, 128, 512, 8192, },
  { 1, 16, 32, 128, 512, 8192, },
  { 2, 16, 32, 128, 512, 8192, },
  { 1, 2, 16, 32, 128, 512, 8192, },
  { 4, 16, 32, 128, 512, 8192, },
  { 1, 4, 16, 32, 128, 512, 8192, },
  { 2, 4, 16, 32, 128, 512, 8192, },
  { 1, 2, 4, 16, 32, 128, 512, 8192, },
  { 8, 16, 32, 128, 512, 8192, },
  { 1, 8, 16, 32, 128, 512, 8192, },
  { 2, 8, 16, 32, 128, 512, 8192, },
  { 1, 2, 8, 16, 32, 128, 512, 8192, },
  { 4, 8, 16, 32, 128, 512, 8192, },
  { 1, 4, 8, 16, 32, 128, 512, 8192, },
  { 2, 4, 8, 16, 32, 128, 512, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 8192, },
  { 64, 128, 512, 8192, },
  { 1, 64, 128, 512, 8192, },
  { 2, 64, 128, 512, 8192, },
  { 1, 2, 64, 128, 512, 8192, },
  { 4, 64, 128, 512, 8192, },
  { 1, 4, 64, 128, 512, 8192, },
  { 2, 4, 64, 128, 512, 8192, },
  { 1, 2, 4, 64, 128, 512, 8192, },
  { 8, 64, 128, 512, 8192, },
  { 1, 8, 64, 128, 512, 8192, },
  { 2, 8, 64, 128, 512, 8192, },
  { 1, 2, 8, 64, 128, 512, 8192, },
  { 4, 8, 64, 128, 512, 8192, },
  { 1, 4, 8, 64, 128, 512, 8192, },
  { 2, 4, 8, 64, 128, 512, 8192, },
  { 1, 2, 4, 8, 64, 128, 512, 8192, },
  { 16, 64, 128, 512, 8192, },
  { 1, 16, 64, 128, 512, 8192, },
  { 2, 16, 64, 128, 512, 8192, },
  { 1, 2, 16, 64, 128, 512, 8192, },
  { 4, 16, 64, 128, 512, 8192, },
  { 1, 4, 16, 64, 128, 512, 8192, },
  { 2, 4, 16, 64, 128, 512, 8192, },
  { 1, 2, 4, 16, 64, 128, 512, 8192, },
  { 8, 16, 64, 128, 512, 8192, },
  { 1, 8, 16, 64, 128, 512, 8192, },
  { 2, 8, 16, 64, 128, 512, 8192, },
  { 1, 2, 8, 16, 64, 128, 512, 8192, },
  { 4, 8, 16, 64, 128, 512, 8192, },
  { 1, 4, 8, 16, 64, 128, 512, 8192, },
  { 2, 4, 8, 16, 64, 128, 512, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 8192, },
  { 32, 64, 128, 512, 8192, },
  { 1, 32, 64, 128, 512, 8192, },
  { 2, 32, 64, 128, 512, 8192, },
  { 1, 2, 32, 64, 128, 512, 8192, },
  { 4, 32, 64, 128, 512, 8192, },
  { 1, 4, 32, 64, 128, 512, 8192, },
  { 2, 4, 32, 64, 128, 512, 8192, },
  { 1, 2, 4, 32, 64, 128, 512, 8192, },
  { 8, 32, 64, 128, 512, 8192, },
  { 1, 8, 32, 64, 128, 512, 8192, },
  { 2, 8, 32, 64, 128, 512, 8192, },
  { 1, 2, 8, 32, 64, 128, 512, 8192, },
  { 4, 8, 32, 64, 128, 512, 8192, },
  { 1, 4, 8, 32, 64, 128, 512, 8192, },
  { 2, 4, 8, 32, 64, 128, 512, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 8192, },
  { 16, 32, 64, 128, 512, 8192, },
  { 1, 16, 32, 64, 128, 512, 8192, },
  { 2, 16, 32, 64, 128, 512, 8192, },
  { 1, 2, 16, 32, 64, 128, 512, 8192, },
  { 4, 16, 32, 64, 128, 512, 8192, },
  { 1, 4, 16, 32, 64, 128, 512, 8192, },
  { 2, 4, 16, 32, 64, 128, 512, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 8192, },
  { 8, 16, 32, 64, 128, 512, 8192, },
  { 1, 8, 16, 32, 64, 128, 512, 8192, },
  { 2, 8, 16, 32, 64, 128, 512, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 8192, },
  { 4, 8, 16, 32, 64, 128, 512, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 8192, },
  { 256, 512, 8192, },
  { 1, 256, 512, 8192, },
  { 2, 256, 512, 8192, },
  { 1, 2, 256, 512, 8192, },
  { 4, 256, 512, 8192, },
  { 1, 4, 256, 512, 8192, },
  { 2, 4, 256, 512, 8192, },
  { 1, 2, 4, 256, 512, 8192, },
  { 8, 256, 512, 8192, },
  { 1, 8, 256, 512, 8192, },
  { 2, 8, 256, 512, 8192, },
  { 1, 2, 8, 256, 512, 8192, },
  { 4, 8, 256, 512, 8192, },
  { 1, 4, 8, 256, 512, 8192, },
  { 2, 4, 8, 256, 512, 8192, },
  { 1, 2, 4, 8, 256, 512, 8192, },
  { 16, 256, 512, 8192, },
  { 1, 16, 256, 512, 8192, },
  { 2, 16, 256, 512, 8192, },
  { 1, 2, 16, 256, 512, 8192, },
  { 4, 16, 256, 512, 8192, },
  { 1, 4, 16, 256, 512, 8192, },
  { 2, 4, 16, 256, 512, 8192, },
  { 1, 2, 4, 16, 256, 512, 8192, },
  { 8, 16, 256, 512, 8192, },
  { 1, 8, 16, 256, 512, 8192, },
  { 2, 8, 16, 256, 512, 8192, },
  { 1, 2, 8, 16, 256, 512, 8192, },
  { 4, 8, 16, 256, 512, 8192, },
  { 1, 4, 8, 16, 256, 512, 8192, },
  { 2, 4, 8, 16, 256, 512, 8192, },
  { 1, 2, 4, 8, 16, 256, 512, 8192, },
  { 32, 256, 512, 8192, },
  { 1, 32, 256, 512, 8192, },
  { 2, 32, 256, 512, 8192, },
  { 1, 2, 32, 256, 512, 8192, },
  { 4, 32, 256, 512, 8192, },
  { 1, 4, 32, 256, 512, 8192, },
  { 2, 4, 32, 256, 512, 8192, },
  { 1, 2, 4, 32, 256, 512, 8192, },
  { 8, 32, 256, 512, 8192, },
  { 1, 8, 32, 256, 512, 8192, },
  { 2, 8, 32, 256, 512, 8192, },
  { 1, 2, 8, 32, 256, 512, 8192, },
  { 4, 8, 32, 256, 512, 8192, },
  { 1, 4, 8, 32, 256, 512, 8192, },
  { 2, 4, 8, 32, 256, 512, 8192, },
  { 1, 2, 4, 8, 32, 256, 512, 8192, },
  { 16, 32, 256, 512, 8192, },
  { 1, 16, 32, 256, 512, 8192, },
  { 2, 16, 32, 256, 512, 8192, },
  { 1, 2, 16, 32, 256, 512, 8192, },
  { 4, 16, 32, 256, 512, 8192, },
  { 1, 4, 16, 32, 256, 512, 8192, },
  { 2, 4, 16, 32, 256, 512, 8192, },
  { 1, 2, 4, 16, 32, 256, 512, 8192, },
  { 8, 16, 32, 256, 512, 8192, },
  { 1, 8, 16, 32, 256, 512, 8192, },
  { 2, 8, 16, 32, 256, 512, 8192, },
  { 1, 2, 8, 16, 32, 256, 512, 8192, },
  { 4, 8, 16, 32, 256, 512, 8192, },
  { 1, 4, 8, 16, 32, 256, 512, 8192, },
  { 2, 4, 8, 16, 32, 256, 512, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 8192, },
  { 64, 256, 512, 8192, },
  { 1, 64, 256, 512, 8192, },
  { 2, 64, 256, 512, 8192, },
  { 1, 2, 64, 256, 512, 8192, },
  { 4, 64, 256, 512, 8192, },
  { 1, 4, 64, 256, 512, 8192, },
  { 2, 4, 64, 256, 512, 8192, },
  { 1, 2, 4, 64, 256, 512, 8192, },
  { 8, 64, 256, 512, 8192, },
  { 1, 8, 64, 256, 512, 8192, },
  { 2, 8, 64, 256, 512, 8192, },
  { 1, 2, 8, 64, 256, 512, 8192, },
  { 4, 8, 64, 256, 512, 8192, },
  { 1, 4, 8, 64, 256, 512, 8192, },
  { 2, 4, 8, 64, 256, 512, 8192, },
  { 1, 2, 4, 8, 64, 256, 512, 8192, },
  { 16, 64, 256, 512, 8192, },
  { 1, 16, 64, 256, 512, 8192, },
  { 2, 16, 64, 256, 512, 8192, },
  { 1, 2, 16, 64, 256, 512, 8192, },
  { 4, 16, 64, 256, 512, 8192, },
  { 1, 4, 16, 64, 256, 512, 8192, },
  { 2, 4, 16, 64, 256, 512, 8192, },
  { 1, 2, 4, 16, 64, 256, 512, 8192, },
  { 8, 16, 64, 256, 512, 8192, },
  { 1, 8, 16, 64, 256, 512, 8192, },
  { 2, 8, 16, 64, 256, 512, 8192, },
  { 1, 2, 8, 16, 64, 256, 512, 8192, },
  { 4, 8, 16, 64, 256, 512, 8192, },
  { 1, 4, 8, 16, 64, 256, 512, 8192, },
  { 2, 4, 8, 16, 64, 256, 512, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 8192, },
  { 32, 64, 256, 512, 8192, },
  { 1, 32, 64, 256, 512, 8192, },
  { 2, 32, 64, 256, 512, 8192, },
  { 1, 2, 32, 64, 256, 512, 8192, },
  { 4, 32, 64, 256, 512, 8192, },
  { 1, 4, 32, 64, 256, 512, 8192, },
  { 2, 4, 32, 64, 256, 512, 8192, },
  { 1, 2, 4, 32, 64, 256, 512, 8192, },
  { 8, 32, 64, 256, 512, 8192, },
  { 1, 8, 32, 64, 256, 512, 8192, },
  { 2, 8, 32, 64, 256, 512, 8192, },
  { 1, 2, 8, 32, 64, 256, 512, 8192, },
  { 4, 8, 32, 64, 256, 512, 8192, },
  { 1, 4, 8, 32, 64, 256, 512, 8192, },
  { 2, 4, 8, 32, 64, 256, 512, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 8192, },
  { 16, 32, 64, 256, 512, 8192, },
  { 1, 16, 32, 64, 256, 512, 8192, },
  { 2, 16, 32, 64, 256, 512, 8192, },
  { 1, 2, 16, 32, 64, 256, 512, 8192, },
  { 4, 16, 32, 64, 256, 512, 8192, },
  { 1, 4, 16, 32, 64, 256, 512, 8192, },
  { 2, 4, 16, 32, 64, 256, 512, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 8192, },
  { 8, 16, 32, 64, 256, 512, 8192, },
  { 1, 8, 16, 32, 64, 256, 512, 8192, },
  { 2, 8, 16, 32, 64, 256, 512, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 8192, },
  { 4, 8, 16, 32, 64, 256, 512, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 8192, },
  { 128, 256, 512, 8192, },
  { 1, 128, 256, 512, 8192, },
  { 2, 128, 256, 512, 8192, },
  { 1, 2, 128, 256, 512, 8192, },
  { 4, 128, 256, 512, 8192, },
  { 1, 4, 128, 256, 512, 8192, },
  { 2, 4, 128, 256, 512, 8192, },
  { 1, 2, 4, 128, 256, 512, 8192, },
  { 8, 128, 256, 512, 8192, },
  { 1, 8, 128, 256, 512, 8192, },
  { 2, 8, 128, 256, 512, 8192, },
  { 1, 2, 8, 128, 256, 512, 8192, },
  { 4, 8, 128, 256, 512, 8192, },
  { 1, 4, 8, 128, 256, 512, 8192, },
  { 2, 4, 8, 128, 256, 512, 8192, },
  { 1, 2, 4, 8, 128, 256, 512, 8192, },
  { 16, 128, 256, 512, 8192, },
  { 1, 16, 128, 256, 512, 8192, },
  { 2, 16, 128, 256, 512, 8192, },
  { 1, 2, 16, 128, 256, 512, 8192, },
  { 4, 16, 128, 256, 512, 8192, },
  { 1, 4, 16, 128, 256, 512, 8192, },
  { 2, 4, 16, 128, 256, 512, 8192, },
  { 1, 2, 4, 16, 128, 256, 512, 8192, },
  { 8, 16, 128, 256, 512, 8192, },
  { 1, 8, 16, 128, 256, 512, 8192, },
  { 2, 8, 16, 128, 256, 512, 8192, },
  { 1, 2, 8, 16, 128, 256, 512, 8192, },
  { 4, 8, 16, 128, 256, 512, 8192, },
  { 1, 4, 8, 16, 128, 256, 512, 8192, },
  { 2, 4, 8, 16, 128, 256, 512, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 8192, },
  { 32, 128, 256, 512, 8192, },
  { 1, 32, 128, 256, 512, 8192, },
  { 2, 32, 128, 256, 512, 8192, },
  { 1, 2, 32, 128, 256, 512, 8192, },
  { 4, 32, 128, 256, 512, 8192, },
  { 1, 4, 32, 128, 256, 512, 8192, },
  { 2, 4, 32, 128, 256, 512, 8192, },
  { 1, 2, 4, 32, 128, 256, 512, 8192, },
  { 8, 32, 128, 256, 512, 8192, },
  { 1, 8, 32, 128, 256, 512, 8192, },
  { 2, 8, 32, 128, 256, 512, 8192, },
  { 1, 2, 8, 32, 128, 256, 512, 8192, },
  { 4, 8, 32, 128, 256, 512, 8192, },
  { 1, 4, 8, 32, 128, 256, 512, 8192, },
  { 2, 4, 8, 32, 128, 256, 512, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 8192, },
  { 16, 32, 128, 256, 512, 8192, },
  { 1, 16, 32, 128, 256, 512, 8192, },
  { 2, 16, 32, 128, 256, 512, 8192, },
  { 1, 2, 16, 32, 128, 256, 512, 8192, },
  { 4, 16, 32, 128, 256, 512, 8192, },
  { 1, 4, 16, 32, 128, 256, 512, 8192, },
  { 2, 4, 16, 32, 128, 256, 512, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 8192, },
  { 8, 16, 32, 128, 256, 512, 8192, },
  { 1, 8, 16, 32, 128, 256, 512, 8192, },
  { 2, 8, 16, 32, 128, 256, 512, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 8192, },
  { 4, 8, 16, 32, 128, 256, 512, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 8192, },
  { 64, 128, 256, 512, 8192, },
  { 1, 64, 128, 256, 512, 8192, },
  { 2, 64, 128, 256, 512, 8192, },
  { 1, 2, 64, 128, 256, 512, 8192, },
  { 4, 64, 128, 256, 512, 8192, },
  { 1, 4, 64, 128, 256, 512, 8192, },
  { 2, 4, 64, 128, 256, 512, 8192, },
  { 1, 2, 4, 64, 128, 256, 512, 8192, },
  { 8, 64, 128, 256, 512, 8192, },
  { 1, 8, 64, 128, 256, 512, 8192, },
  { 2, 8, 64, 128, 256, 512, 8192, },
  { 1, 2, 8, 64, 128, 256, 512, 8192, },
  { 4, 8, 64, 128, 256, 512, 8192, },
  { 1, 4, 8, 64, 128, 256, 512, 8192, },
  { 2, 4, 8, 64, 128, 256, 512, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 8192, },
  { 16, 64, 128, 256, 512, 8192, },
  { 1, 16, 64, 128, 256, 512, 8192, },
  { 2, 16, 64, 128, 256, 512, 8192, },
  { 1, 2, 16, 64, 128, 256, 512, 8192, },
  { 4, 16, 64, 128, 256, 512, 8192, },
  { 1, 4, 16, 64, 128, 256, 512, 8192, },
  { 2, 4, 16, 64, 128, 256, 512, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 8192, },
  { 8, 16, 64, 128, 256, 512, 8192, },
  { 1, 8, 16, 64, 128, 256, 512, 8192, },
  { 2, 8, 16, 64, 128, 256, 512, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 8192, },
  { 4, 8, 16, 64, 128, 256, 512, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 8192, },
  { 32, 64, 128, 256, 512, 8192, },
  { 1, 32, 64, 128, 256, 512, 8192, },
  { 2, 32, 64, 128, 256, 512, 8192, },
  { 1, 2, 32, 64, 128, 256, 512, 8192, },
  { 4, 32, 64, 128, 256, 512, 8192, },
  { 1, 4, 32, 64, 128, 256, 512, 8192, },
  { 2, 4, 32, 64, 128, 256, 512, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 8192, },
  { 8, 32, 64, 128, 256, 512, 8192, },
  { 1, 8, 32, 64, 128, 256, 512, 8192, },
  { 2, 8, 32, 64, 128, 256, 512, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 8192, },
  { 4, 8, 32, 64, 128, 256, 512, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 8192, },
  { 16, 32, 64, 128, 256, 512, 8192, },
  { 1, 16, 32, 64, 128, 256, 512, 8192, },
  { 2, 16, 32, 64, 128, 256, 512, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 8192, },
  { 4, 16, 32, 64, 128, 256, 512, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 8192, },
  { 8, 16, 32, 64, 128, 256, 512, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 8192, },
  { 1024, 8192, },
  { 1, 1024, 8192, },
  { 2, 1024, 8192, },
  { 1, 2, 1024, 8192, },
  { 4, 1024, 8192, },
  { 1, 4, 1024, 8192, },
  { 2, 4, 1024, 8192, },
  { 1, 2, 4, 1024, 8192, },
  { 8, 1024, 8192, },
  { 1, 8, 1024, 8192, },
  { 2, 8, 1024, 8192, },
  { 1, 2, 8, 1024, 8192, },
  { 4, 8, 1024, 8192, },
  { 1, 4, 8, 1024, 8192, },
  { 2, 4, 8, 1024, 8192, },
  { 1, 2, 4, 8, 1024, 8192, },
  { 16, 1024, 8192, },
  { 1, 16, 1024, 8192, },
  { 2, 16, 1024, 8192, },
  { 1, 2, 16, 1024, 8192, },
  { 4, 16, 1024, 8192, },
  { 1, 4, 16, 1024, 8192, },
  { 2, 4, 16, 1024, 8192, },
  { 1, 2, 4, 16, 1024, 8192, },
  { 8, 16, 1024, 8192, },
  { 1, 8, 16, 1024, 8192, },
  { 2, 8, 16, 1024, 8192, },
  { 1, 2, 8, 16, 1024, 8192, },
  { 4, 8, 16, 1024, 8192, },
  { 1, 4, 8, 16, 1024, 8192, },
  { 2, 4, 8, 16, 1024, 8192, },
  { 1, 2, 4, 8, 16, 1024, 8192, },
  { 32, 1024, 8192, },
  { 1, 32, 1024, 8192, },
  { 2, 32, 1024, 8192, },
  { 1, 2, 32, 1024, 8192, },
  { 4, 32, 1024, 8192, },
  { 1, 4, 32, 1024, 8192, },
  { 2, 4, 32, 1024, 8192, },
  { 1, 2, 4, 32, 1024, 8192, },
  { 8, 32, 1024, 8192, },
  { 1, 8, 32, 1024, 8192, },
  { 2, 8, 32, 1024, 8192, },
  { 1, 2, 8, 32, 1024, 8192, },
  { 4, 8, 32, 1024, 8192, },
  { 1, 4, 8, 32, 1024, 8192, },
  { 2, 4, 8, 32, 1024, 8192, },
  { 1, 2, 4, 8, 32, 1024, 8192, },
  { 16, 32, 1024, 8192, },
  { 1, 16, 32, 1024, 8192, },
  { 2, 16, 32, 1024, 8192, },
  { 1, 2, 16, 32, 1024, 8192, },
  { 4, 16, 32, 1024, 8192, },
  { 1, 4, 16, 32, 1024, 8192, },
  { 2, 4, 16, 32, 1024, 8192, },
  { 1, 2, 4, 16, 32, 1024, 8192, },
  { 8, 16, 32, 1024, 8192, },
  { 1, 8, 16, 32, 1024, 8192, },
  { 2, 8, 16, 32, 1024, 8192, },
  { 1, 2, 8, 16, 32, 1024, 8192, },
  { 4, 8, 16, 32, 1024, 8192, },
  { 1, 4, 8, 16, 32, 1024, 8192, },
  { 2, 4, 8, 16, 32, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 1024, 8192, },
  { 64, 1024, 8192, },
  { 1, 64, 1024, 8192, },
  { 2, 64, 1024, 8192, },
  { 1, 2, 64, 1024, 8192, },
  { 4, 64, 1024, 8192, },
  { 1, 4, 64, 1024, 8192, },
  { 2, 4, 64, 1024, 8192, },
  { 1, 2, 4, 64, 1024, 8192, },
  { 8, 64, 1024, 8192, },
  { 1, 8, 64, 1024, 8192, },
  { 2, 8, 64, 1024, 8192, },
  { 1, 2, 8, 64, 1024, 8192, },
  { 4, 8, 64, 1024, 8192, },
  { 1, 4, 8, 64, 1024, 8192, },
  { 2, 4, 8, 64, 1024, 8192, },
  { 1, 2, 4, 8, 64, 1024, 8192, },
  { 16, 64, 1024, 8192, },
  { 1, 16, 64, 1024, 8192, },
  { 2, 16, 64, 1024, 8192, },
  { 1, 2, 16, 64, 1024, 8192, },
  { 4, 16, 64, 1024, 8192, },
  { 1, 4, 16, 64, 1024, 8192, },
  { 2, 4, 16, 64, 1024, 8192, },
  { 1, 2, 4, 16, 64, 1024, 8192, },
  { 8, 16, 64, 1024, 8192, },
  { 1, 8, 16, 64, 1024, 8192, },
  { 2, 8, 16, 64, 1024, 8192, },
  { 1, 2, 8, 16, 64, 1024, 8192, },
  { 4, 8, 16, 64, 1024, 8192, },
  { 1, 4, 8, 16, 64, 1024, 8192, },
  { 2, 4, 8, 16, 64, 1024, 8192, },
  { 1, 2, 4, 8, 16, 64, 1024, 8192, },
  { 32, 64, 1024, 8192, },
  { 1, 32, 64, 1024, 8192, },
  { 2, 32, 64, 1024, 8192, },
  { 1, 2, 32, 64, 1024, 8192, },
  { 4, 32, 64, 1024, 8192, },
  { 1, 4, 32, 64, 1024, 8192, },
  { 2, 4, 32, 64, 1024, 8192, },
  { 1, 2, 4, 32, 64, 1024, 8192, },
  { 8, 32, 64, 1024, 8192, },
  { 1, 8, 32, 64, 1024, 8192, },
  { 2, 8, 32, 64, 1024, 8192, },
  { 1, 2, 8, 32, 64, 1024, 8192, },
  { 4, 8, 32, 64, 1024, 8192, },
  { 1, 4, 8, 32, 64, 1024, 8192, },
  { 2, 4, 8, 32, 64, 1024, 8192, },
  { 1, 2, 4, 8, 32, 64, 1024, 8192, },
  { 16, 32, 64, 1024, 8192, },
  { 1, 16, 32, 64, 1024, 8192, },
  { 2, 16, 32, 64, 1024, 8192, },
  { 1, 2, 16, 32, 64, 1024, 8192, },
  { 4, 16, 32, 64, 1024, 8192, },
  { 1, 4, 16, 32, 64, 1024, 8192, },
  { 2, 4, 16, 32, 64, 1024, 8192, },
  { 1, 2, 4, 16, 32, 64, 1024, 8192, },
  { 8, 16, 32, 64, 1024, 8192, },
  { 1, 8, 16, 32, 64, 1024, 8192, },
  { 2, 8, 16, 32, 64, 1024, 8192, },
  { 1, 2, 8, 16, 32, 64, 1024, 8192, },
  { 4, 8, 16, 32, 64, 1024, 8192, },
  { 1, 4, 8, 16, 32, 64, 1024, 8192, },
  { 2, 4, 8, 16, 32, 64, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 8192, },
  { 128, 1024, 8192, },
  { 1, 128, 1024, 8192, },
  { 2, 128, 1024, 8192, },
  { 1, 2, 128, 1024, 8192, },
  { 4, 128, 1024, 8192, },
  { 1, 4, 128, 1024, 8192, },
  { 2, 4, 128, 1024, 8192, },
  { 1, 2, 4, 128, 1024, 8192, },
  { 8, 128, 1024, 8192, },
  { 1, 8, 128, 1024, 8192, },
  { 2, 8, 128, 1024, 8192, },
  { 1, 2, 8, 128, 1024, 8192, },
  { 4, 8, 128, 1024, 8192, },
  { 1, 4, 8, 128, 1024, 8192, },
  { 2, 4, 8, 128, 1024, 8192, },
  { 1, 2, 4, 8, 128, 1024, 8192, },
  { 16, 128, 1024, 8192, },
  { 1, 16, 128, 1024, 8192, },
  { 2, 16, 128, 1024, 8192, },
  { 1, 2, 16, 128, 1024, 8192, },
  { 4, 16, 128, 1024, 8192, },
  { 1, 4, 16, 128, 1024, 8192, },
  { 2, 4, 16, 128, 1024, 8192, },
  { 1, 2, 4, 16, 128, 1024, 8192, },
  { 8, 16, 128, 1024, 8192, },
  { 1, 8, 16, 128, 1024, 8192, },
  { 2, 8, 16, 128, 1024, 8192, },
  { 1, 2, 8, 16, 128, 1024, 8192, },
  { 4, 8, 16, 128, 1024, 8192, },
  { 1, 4, 8, 16, 128, 1024, 8192, },
  { 2, 4, 8, 16, 128, 1024, 8192, },
  { 1, 2, 4, 8, 16, 128, 1024, 8192, },
  { 32, 128, 1024, 8192, },
  { 1, 32, 128, 1024, 8192, },
  { 2, 32, 128, 1024, 8192, },
  { 1, 2, 32, 128, 1024, 8192, },
  { 4, 32, 128, 1024, 8192, },
  { 1, 4, 32, 128, 1024, 8192, },
  { 2, 4, 32, 128, 1024, 8192, },
  { 1, 2, 4, 32, 128, 1024, 8192, },
  { 8, 32, 128, 1024, 8192, },
  { 1, 8, 32, 128, 1024, 8192, },
  { 2, 8, 32, 128, 1024, 8192, },
  { 1, 2, 8, 32, 128, 1024, 8192, },
  { 4, 8, 32, 128, 1024, 8192, },
  { 1, 4, 8, 32, 128, 1024, 8192, },
  { 2, 4, 8, 32, 128, 1024, 8192, },
  { 1, 2, 4, 8, 32, 128, 1024, 8192, },
  { 16, 32, 128, 1024, 8192, },
  { 1, 16, 32, 128, 1024, 8192, },
  { 2, 16, 32, 128, 1024, 8192, },
  { 1, 2, 16, 32, 128, 1024, 8192, },
  { 4, 16, 32, 128, 1024, 8192, },
  { 1, 4, 16, 32, 128, 1024, 8192, },
  { 2, 4, 16, 32, 128, 1024, 8192, },
  { 1, 2, 4, 16, 32, 128, 1024, 8192, },
  { 8, 16, 32, 128, 1024, 8192, },
  { 1, 8, 16, 32, 128, 1024, 8192, },
  { 2, 8, 16, 32, 128, 1024, 8192, },
  { 1, 2, 8, 16, 32, 128, 1024, 8192, },
  { 4, 8, 16, 32, 128, 1024, 8192, },
  { 1, 4, 8, 16, 32, 128, 1024, 8192, },
  { 2, 4, 8, 16, 32, 128, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 8192, },
  { 64, 128, 1024, 8192, },
  { 1, 64, 128, 1024, 8192, },
  { 2, 64, 128, 1024, 8192, },
  { 1, 2, 64, 128, 1024, 8192, },
  { 4, 64, 128, 1024, 8192, },
  { 1, 4, 64, 128, 1024, 8192, },
  { 2, 4, 64, 128, 1024, 8192, },
  { 1, 2, 4, 64, 128, 1024, 8192, },
  { 8, 64, 128, 1024, 8192, },
  { 1, 8, 64, 128, 1024, 8192, },
  { 2, 8, 64, 128, 1024, 8192, },
  { 1, 2, 8, 64, 128, 1024, 8192, },
  { 4, 8, 64, 128, 1024, 8192, },
  { 1, 4, 8, 64, 128, 1024, 8192, },
  { 2, 4, 8, 64, 128, 1024, 8192, },
  { 1, 2, 4, 8, 64, 128, 1024, 8192, },
  { 16, 64, 128, 1024, 8192, },
  { 1, 16, 64, 128, 1024, 8192, },
  { 2, 16, 64, 128, 1024, 8192, },
  { 1, 2, 16, 64, 128, 1024, 8192, },
  { 4, 16, 64, 128, 1024, 8192, },
  { 1, 4, 16, 64, 128, 1024, 8192, },
  { 2, 4, 16, 64, 128, 1024, 8192, },
  { 1, 2, 4, 16, 64, 128, 1024, 8192, },
  { 8, 16, 64, 128, 1024, 8192, },
  { 1, 8, 16, 64, 128, 1024, 8192, },
  { 2, 8, 16, 64, 128, 1024, 8192, },
  { 1, 2, 8, 16, 64, 128, 1024, 8192, },
  { 4, 8, 16, 64, 128, 1024, 8192, },
  { 1, 4, 8, 16, 64, 128, 1024, 8192, },
  { 2, 4, 8, 16, 64, 128, 1024, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 8192, },
  { 32, 64, 128, 1024, 8192, },
  { 1, 32, 64, 128, 1024, 8192, },
  { 2, 32, 64, 128, 1024, 8192, },
  { 1, 2, 32, 64, 128, 1024, 8192, },
  { 4, 32, 64, 128, 1024, 8192, },
  { 1, 4, 32, 64, 128, 1024, 8192, },
  { 2, 4, 32, 64, 128, 1024, 8192, },
  { 1, 2, 4, 32, 64, 128, 1024, 8192, },
  { 8, 32, 64, 128, 1024, 8192, },
  { 1, 8, 32, 64, 128, 1024, 8192, },
  { 2, 8, 32, 64, 128, 1024, 8192, },
  { 1, 2, 8, 32, 64, 128, 1024, 8192, },
  { 4, 8, 32, 64, 128, 1024, 8192, },
  { 1, 4, 8, 32, 64, 128, 1024, 8192, },
  { 2, 4, 8, 32, 64, 128, 1024, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 8192, },
  { 16, 32, 64, 128, 1024, 8192, },
  { 1, 16, 32, 64, 128, 1024, 8192, },
  { 2, 16, 32, 64, 128, 1024, 8192, },
  { 1, 2, 16, 32, 64, 128, 1024, 8192, },
  { 4, 16, 32, 64, 128, 1024, 8192, },
  { 1, 4, 16, 32, 64, 128, 1024, 8192, },
  { 2, 4, 16, 32, 64, 128, 1024, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 8192, },
  { 8, 16, 32, 64, 128, 1024, 8192, },
  { 1, 8, 16, 32, 64, 128, 1024, 8192, },
  { 2, 8, 16, 32, 64, 128, 1024, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 8192, },
  { 4, 8, 16, 32, 64, 128, 1024, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 8192, },
  { 256, 1024, 8192, },
  { 1, 256, 1024, 8192, },
  { 2, 256, 1024, 8192, },
  { 1, 2, 256, 1024, 8192, },
  { 4, 256, 1024, 8192, },
  { 1, 4, 256, 1024, 8192, },
  { 2, 4, 256, 1024, 8192, },
  { 1, 2, 4, 256, 1024, 8192, },
  { 8, 256, 1024, 8192, },
  { 1, 8, 256, 1024, 8192, },
  { 2, 8, 256, 1024, 8192, },
  { 1, 2, 8, 256, 1024, 8192, },
  { 4, 8, 256, 1024, 8192, },
  { 1, 4, 8, 256, 1024, 8192, },
  { 2, 4, 8, 256, 1024, 8192, },
  { 1, 2, 4, 8, 256, 1024, 8192, },
  { 16, 256, 1024, 8192, },
  { 1, 16, 256, 1024, 8192, },
  { 2, 16, 256, 1024, 8192, },
  { 1, 2, 16, 256, 1024, 8192, },
  { 4, 16, 256, 1024, 8192, },
  { 1, 4, 16, 256, 1024, 8192, },
  { 2, 4, 16, 256, 1024, 8192, },
  { 1, 2, 4, 16, 256, 1024, 8192, },
  { 8, 16, 256, 1024, 8192, },
  { 1, 8, 16, 256, 1024, 8192, },
  { 2, 8, 16, 256, 1024, 8192, },
  { 1, 2, 8, 16, 256, 1024, 8192, },
  { 4, 8, 16, 256, 1024, 8192, },
  { 1, 4, 8, 16, 256, 1024, 8192, },
  { 2, 4, 8, 16, 256, 1024, 8192, },
  { 1, 2, 4, 8, 16, 256, 1024, 8192, },
  { 32, 256, 1024, 8192, },
  { 1, 32, 256, 1024, 8192, },
  { 2, 32, 256, 1024, 8192, },
  { 1, 2, 32, 256, 1024, 8192, },
  { 4, 32, 256, 1024, 8192, },
  { 1, 4, 32, 256, 1024, 8192, },
  { 2, 4, 32, 256, 1024, 8192, },
  { 1, 2, 4, 32, 256, 1024, 8192, },
  { 8, 32, 256, 1024, 8192, },
  { 1, 8, 32, 256, 1024, 8192, },
  { 2, 8, 32, 256, 1024, 8192, },
  { 1, 2, 8, 32, 256, 1024, 8192, },
  { 4, 8, 32, 256, 1024, 8192, },
  { 1, 4, 8, 32, 256, 1024, 8192, },
  { 2, 4, 8, 32, 256, 1024, 8192, },
  { 1, 2, 4, 8, 32, 256, 1024, 8192, },
  { 16, 32, 256, 1024, 8192, },
  { 1, 16, 32, 256, 1024, 8192, },
  { 2, 16, 32, 256, 1024, 8192, },
  { 1, 2, 16, 32, 256, 1024, 8192, },
  { 4, 16, 32, 256, 1024, 8192, },
  { 1, 4, 16, 32, 256, 1024, 8192, },
  { 2, 4, 16, 32, 256, 1024, 8192, },
  { 1, 2, 4, 16, 32, 256, 1024, 8192, },
  { 8, 16, 32, 256, 1024, 8192, },
  { 1, 8, 16, 32, 256, 1024, 8192, },
  { 2, 8, 16, 32, 256, 1024, 8192, },
  { 1, 2, 8, 16, 32, 256, 1024, 8192, },
  { 4, 8, 16, 32, 256, 1024, 8192, },
  { 1, 4, 8, 16, 32, 256, 1024, 8192, },
  { 2, 4, 8, 16, 32, 256, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 8192, },
  { 64, 256, 1024, 8192, },
  { 1, 64, 256, 1024, 8192, },
  { 2, 64, 256, 1024, 8192, },
  { 1, 2, 64, 256, 1024, 8192, },
  { 4, 64, 256, 1024, 8192, },
  { 1, 4, 64, 256, 1024, 8192, },
  { 2, 4, 64, 256, 1024, 8192, },
  { 1, 2, 4, 64, 256, 1024, 8192, },
  { 8, 64, 256, 1024, 8192, },
  { 1, 8, 64, 256, 1024, 8192, },
  { 2, 8, 64, 256, 1024, 8192, },
  { 1, 2, 8, 64, 256, 1024, 8192, },
  { 4, 8, 64, 256, 1024, 8192, },
  { 1, 4, 8, 64, 256, 1024, 8192, },
  { 2, 4, 8, 64, 256, 1024, 8192, },
  { 1, 2, 4, 8, 64, 256, 1024, 8192, },
  { 16, 64, 256, 1024, 8192, },
  { 1, 16, 64, 256, 1024, 8192, },
  { 2, 16, 64, 256, 1024, 8192, },
  { 1, 2, 16, 64, 256, 1024, 8192, },
  { 4, 16, 64, 256, 1024, 8192, },
  { 1, 4, 16, 64, 256, 1024, 8192, },
  { 2, 4, 16, 64, 256, 1024, 8192, },
  { 1, 2, 4, 16, 64, 256, 1024, 8192, },
  { 8, 16, 64, 256, 1024, 8192, },
  { 1, 8, 16, 64, 256, 1024, 8192, },
  { 2, 8, 16, 64, 256, 1024, 8192, },
  { 1, 2, 8, 16, 64, 256, 1024, 8192, },
  { 4, 8, 16, 64, 256, 1024, 8192, },
  { 1, 4, 8, 16, 64, 256, 1024, 8192, },
  { 2, 4, 8, 16, 64, 256, 1024, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 8192, },
  { 32, 64, 256, 1024, 8192, },
  { 1, 32, 64, 256, 1024, 8192, },
  { 2, 32, 64, 256, 1024, 8192, },
  { 1, 2, 32, 64, 256, 1024, 8192, },
  { 4, 32, 64, 256, 1024, 8192, },
  { 1, 4, 32, 64, 256, 1024, 8192, },
  { 2, 4, 32, 64, 256, 1024, 8192, },
  { 1, 2, 4, 32, 64, 256, 1024, 8192, },
  { 8, 32, 64, 256, 1024, 8192, },
  { 1, 8, 32, 64, 256, 1024, 8192, },
  { 2, 8, 32, 64, 256, 1024, 8192, },
  { 1, 2, 8, 32, 64, 256, 1024, 8192, },
  { 4, 8, 32, 64, 256, 1024, 8192, },
  { 1, 4, 8, 32, 64, 256, 1024, 8192, },
  { 2, 4, 8, 32, 64, 256, 1024, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 8192, },
  { 16, 32, 64, 256, 1024, 8192, },
  { 1, 16, 32, 64, 256, 1024, 8192, },
  { 2, 16, 32, 64, 256, 1024, 8192, },
  { 1, 2, 16, 32, 64, 256, 1024, 8192, },
  { 4, 16, 32, 64, 256, 1024, 8192, },
  { 1, 4, 16, 32, 64, 256, 1024, 8192, },
  { 2, 4, 16, 32, 64, 256, 1024, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 8192, },
  { 8, 16, 32, 64, 256, 1024, 8192, },
  { 1, 8, 16, 32, 64, 256, 1024, 8192, },
  { 2, 8, 16, 32, 64, 256, 1024, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 8192, },
  { 4, 8, 16, 32, 64, 256, 1024, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 8192, },
  { 128, 256, 1024, 8192, },
  { 1, 128, 256, 1024, 8192, },
  { 2, 128, 256, 1024, 8192, },
  { 1, 2, 128, 256, 1024, 8192, },
  { 4, 128, 256, 1024, 8192, },
  { 1, 4, 128, 256, 1024, 8192, },
  { 2, 4, 128, 256, 1024, 8192, },
  { 1, 2, 4, 128, 256, 1024, 8192, },
  { 8, 128, 256, 1024, 8192, },
  { 1, 8, 128, 256, 1024, 8192, },
  { 2, 8, 128, 256, 1024, 8192, },
  { 1, 2, 8, 128, 256, 1024, 8192, },
  { 4, 8, 128, 256, 1024, 8192, },
  { 1, 4, 8, 128, 256, 1024, 8192, },
  { 2, 4, 8, 128, 256, 1024, 8192, },
  { 1, 2, 4, 8, 128, 256, 1024, 8192, },
  { 16, 128, 256, 1024, 8192, },
  { 1, 16, 128, 256, 1024, 8192, },
  { 2, 16, 128, 256, 1024, 8192, },
  { 1, 2, 16, 128, 256, 1024, 8192, },
  { 4, 16, 128, 256, 1024, 8192, },
  { 1, 4, 16, 128, 256, 1024, 8192, },
  { 2, 4, 16, 128, 256, 1024, 8192, },
  { 1, 2, 4, 16, 128, 256, 1024, 8192, },
  { 8, 16, 128, 256, 1024, 8192, },
  { 1, 8, 16, 128, 256, 1024, 8192, },
  { 2, 8, 16, 128, 256, 1024, 8192, },
  { 1, 2, 8, 16, 128, 256, 1024, 8192, },
  { 4, 8, 16, 128, 256, 1024, 8192, },
  { 1, 4, 8, 16, 128, 256, 1024, 8192, },
  { 2, 4, 8, 16, 128, 256, 1024, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 8192, },
  { 32, 128, 256, 1024, 8192, },
  { 1, 32, 128, 256, 1024, 8192, },
  { 2, 32, 128, 256, 1024, 8192, },
  { 1, 2, 32, 128, 256, 1024, 8192, },
  { 4, 32, 128, 256, 1024, 8192, },
  { 1, 4, 32, 128, 256, 1024, 8192, },
  { 2, 4, 32, 128, 256, 1024, 8192, },
  { 1, 2, 4, 32, 128, 256, 1024, 8192, },
  { 8, 32, 128, 256, 1024, 8192, },
  { 1, 8, 32, 128, 256, 1024, 8192, },
  { 2, 8, 32, 128, 256, 1024, 8192, },
  { 1, 2, 8, 32, 128, 256, 1024, 8192, },
  { 4, 8, 32, 128, 256, 1024, 8192, },
  { 1, 4, 8, 32, 128, 256, 1024, 8192, },
  { 2, 4, 8, 32, 128, 256, 1024, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 8192, },
  { 16, 32, 128, 256, 1024, 8192, },
  { 1, 16, 32, 128, 256, 1024, 8192, },
  { 2, 16, 32, 128, 256, 1024, 8192, },
  { 1, 2, 16, 32, 128, 256, 1024, 8192, },
  { 4, 16, 32, 128, 256, 1024, 8192, },
  { 1, 4, 16, 32, 128, 256, 1024, 8192, },
  { 2, 4, 16, 32, 128, 256, 1024, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 8192, },
  { 8, 16, 32, 128, 256, 1024, 8192, },
  { 1, 8, 16, 32, 128, 256, 1024, 8192, },
  { 2, 8, 16, 32, 128, 256, 1024, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 8192, },
  { 4, 8, 16, 32, 128, 256, 1024, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 8192, },
  { 64, 128, 256, 1024, 8192, },
  { 1, 64, 128, 256, 1024, 8192, },
  { 2, 64, 128, 256, 1024, 8192, },
  { 1, 2, 64, 128, 256, 1024, 8192, },
  { 4, 64, 128, 256, 1024, 8192, },
  { 1, 4, 64, 128, 256, 1024, 8192, },
  { 2, 4, 64, 128, 256, 1024, 8192, },
  { 1, 2, 4, 64, 128, 256, 1024, 8192, },
  { 8, 64, 128, 256, 1024, 8192, },
  { 1, 8, 64, 128, 256, 1024, 8192, },
  { 2, 8, 64, 128, 256, 1024, 8192, },
  { 1, 2, 8, 64, 128, 256, 1024, 8192, },
  { 4, 8, 64, 128, 256, 1024, 8192, },
  { 1, 4, 8, 64, 128, 256, 1024, 8192, },
  { 2, 4, 8, 64, 128, 256, 1024, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 8192, },
  { 16, 64, 128, 256, 1024, 8192, },
  { 1, 16, 64, 128, 256, 1024, 8192, },
  { 2, 16, 64, 128, 256, 1024, 8192, },
  { 1, 2, 16, 64, 128, 256, 1024, 8192, },
  { 4, 16, 64, 128, 256, 1024, 8192, },
  { 1, 4, 16, 64, 128, 256, 1024, 8192, },
  { 2, 4, 16, 64, 128, 256, 1024, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 8192, },
  { 8, 16, 64, 128, 256, 1024, 8192, },
  { 1, 8, 16, 64, 128, 256, 1024, 8192, },
  { 2, 8, 16, 64, 128, 256, 1024, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 8192, },
  { 4, 8, 16, 64, 128, 256, 1024, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 8192, },
  { 32, 64, 128, 256, 1024, 8192, },
  { 1, 32, 64, 128, 256, 1024, 8192, },
  { 2, 32, 64, 128, 256, 1024, 8192, },
  { 1, 2, 32, 64, 128, 256, 1024, 8192, },
  { 4, 32, 64, 128, 256, 1024, 8192, },
  { 1, 4, 32, 64, 128, 256, 1024, 8192, },
  { 2, 4, 32, 64, 128, 256, 1024, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 8192, },
  { 8, 32, 64, 128, 256, 1024, 8192, },
  { 1, 8, 32, 64, 128, 256, 1024, 8192, },
  { 2, 8, 32, 64, 128, 256, 1024, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 8192, },
  { 4, 8, 32, 64, 128, 256, 1024, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 8192, },
  { 16, 32, 64, 128, 256, 1024, 8192, },
  { 1, 16, 32, 64, 128, 256, 1024, 8192, },
  { 2, 16, 32, 64, 128, 256, 1024, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 8192, },
  { 4, 16, 32, 64, 128, 256, 1024, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 8192, },
  { 8, 16, 32, 64, 128, 256, 1024, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, },
  { 512, 1024, 8192, },
  { 1, 512, 1024, 8192, },
  { 2, 512, 1024, 8192, },
  { 1, 2, 512, 1024, 8192, },
  { 4, 512, 1024, 8192, },
  { 1, 4, 512, 1024, 8192, },
  { 2, 4, 512, 1024, 8192, },
  { 1, 2, 4, 512, 1024, 8192, },
  { 8, 512, 1024, 8192, },
  { 1, 8, 512, 1024, 8192, },
  { 2, 8, 512, 1024, 8192, },
  { 1, 2, 8, 512, 1024, 8192, },
  { 4, 8, 512, 1024, 8192, },
  { 1, 4, 8, 512, 1024, 8192, },
  { 2, 4, 8, 512, 1024, 8192, },
  { 1, 2, 4, 8, 512, 1024, 8192, },
  { 16, 512, 1024, 8192, },
  { 1, 16, 512, 1024, 8192, },
  { 2, 16, 512, 1024, 8192, },
  { 1, 2, 16, 512, 1024, 8192, },
  { 4, 16, 512, 1024, 8192, },
  { 1, 4, 16, 512, 1024, 8192, },
  { 2, 4, 16, 512, 1024, 8192, },
  { 1, 2, 4, 16, 512, 1024, 8192, },
  { 8, 16, 512, 1024, 8192, },
  { 1, 8, 16, 512, 1024, 8192, },
  { 2, 8, 16, 512, 1024, 8192, },
  { 1, 2, 8, 16, 512, 1024, 8192, },
  { 4, 8, 16, 512, 1024, 8192, },
  { 1, 4, 8, 16, 512, 1024, 8192, },
  { 2, 4, 8, 16, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 512, 1024, 8192, },
  { 32, 512, 1024, 8192, },
  { 1, 32, 512, 1024, 8192, },
  { 2, 32, 512, 1024, 8192, },
  { 1, 2, 32, 512, 1024, 8192, },
  { 4, 32, 512, 1024, 8192, },
  { 1, 4, 32, 512, 1024, 8192, },
  { 2, 4, 32, 512, 1024, 8192, },
  { 1, 2, 4, 32, 512, 1024, 8192, },
  { 8, 32, 512, 1024, 8192, },
  { 1, 8, 32, 512, 1024, 8192, },
  { 2, 8, 32, 512, 1024, 8192, },
  { 1, 2, 8, 32, 512, 1024, 8192, },
  { 4, 8, 32, 512, 1024, 8192, },
  { 1, 4, 8, 32, 512, 1024, 8192, },
  { 2, 4, 8, 32, 512, 1024, 8192, },
  { 1, 2, 4, 8, 32, 512, 1024, 8192, },
  { 16, 32, 512, 1024, 8192, },
  { 1, 16, 32, 512, 1024, 8192, },
  { 2, 16, 32, 512, 1024, 8192, },
  { 1, 2, 16, 32, 512, 1024, 8192, },
  { 4, 16, 32, 512, 1024, 8192, },
  { 1, 4, 16, 32, 512, 1024, 8192, },
  { 2, 4, 16, 32, 512, 1024, 8192, },
  { 1, 2, 4, 16, 32, 512, 1024, 8192, },
  { 8, 16, 32, 512, 1024, 8192, },
  { 1, 8, 16, 32, 512, 1024, 8192, },
  { 2, 8, 16, 32, 512, 1024, 8192, },
  { 1, 2, 8, 16, 32, 512, 1024, 8192, },
  { 4, 8, 16, 32, 512, 1024, 8192, },
  { 1, 4, 8, 16, 32, 512, 1024, 8192, },
  { 2, 4, 8, 16, 32, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 8192, },
  { 64, 512, 1024, 8192, },
  { 1, 64, 512, 1024, 8192, },
  { 2, 64, 512, 1024, 8192, },
  { 1, 2, 64, 512, 1024, 8192, },
  { 4, 64, 512, 1024, 8192, },
  { 1, 4, 64, 512, 1024, 8192, },
  { 2, 4, 64, 512, 1024, 8192, },
  { 1, 2, 4, 64, 512, 1024, 8192, },
  { 8, 64, 512, 1024, 8192, },
  { 1, 8, 64, 512, 1024, 8192, },
  { 2, 8, 64, 512, 1024, 8192, },
  { 1, 2, 8, 64, 512, 1024, 8192, },
  { 4, 8, 64, 512, 1024, 8192, },
  { 1, 4, 8, 64, 512, 1024, 8192, },
  { 2, 4, 8, 64, 512, 1024, 8192, },
  { 1, 2, 4, 8, 64, 512, 1024, 8192, },
  { 16, 64, 512, 1024, 8192, },
  { 1, 16, 64, 512, 1024, 8192, },
  { 2, 16, 64, 512, 1024, 8192, },
  { 1, 2, 16, 64, 512, 1024, 8192, },
  { 4, 16, 64, 512, 1024, 8192, },
  { 1, 4, 16, 64, 512, 1024, 8192, },
  { 2, 4, 16, 64, 512, 1024, 8192, },
  { 1, 2, 4, 16, 64, 512, 1024, 8192, },
  { 8, 16, 64, 512, 1024, 8192, },
  { 1, 8, 16, 64, 512, 1024, 8192, },
  { 2, 8, 16, 64, 512, 1024, 8192, },
  { 1, 2, 8, 16, 64, 512, 1024, 8192, },
  { 4, 8, 16, 64, 512, 1024, 8192, },
  { 1, 4, 8, 16, 64, 512, 1024, 8192, },
  { 2, 4, 8, 16, 64, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 8192, },
  { 32, 64, 512, 1024, 8192, },
  { 1, 32, 64, 512, 1024, 8192, },
  { 2, 32, 64, 512, 1024, 8192, },
  { 1, 2, 32, 64, 512, 1024, 8192, },
  { 4, 32, 64, 512, 1024, 8192, },
  { 1, 4, 32, 64, 512, 1024, 8192, },
  { 2, 4, 32, 64, 512, 1024, 8192, },
  { 1, 2, 4, 32, 64, 512, 1024, 8192, },
  { 8, 32, 64, 512, 1024, 8192, },
  { 1, 8, 32, 64, 512, 1024, 8192, },
  { 2, 8, 32, 64, 512, 1024, 8192, },
  { 1, 2, 8, 32, 64, 512, 1024, 8192, },
  { 4, 8, 32, 64, 512, 1024, 8192, },
  { 1, 4, 8, 32, 64, 512, 1024, 8192, },
  { 2, 4, 8, 32, 64, 512, 1024, 8192, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 8192, },
  { 16, 32, 64, 512, 1024, 8192, },
  { 1, 16, 32, 64, 512, 1024, 8192, },
  { 2, 16, 32, 64, 512, 1024, 8192, },
  { 1, 2, 16, 32, 64, 512, 1024, 8192, },
  { 4, 16, 32, 64, 512, 1024, 8192, },
  { 1, 4, 16, 32, 64, 512, 1024, 8192, },
  { 2, 4, 16, 32, 64, 512, 1024, 8192, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 8192, },
  { 8, 16, 32, 64, 512, 1024, 8192, },
  { 1, 8, 16, 32, 64, 512, 1024, 8192, },
  { 2, 8, 16, 32, 64, 512, 1024, 8192, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 8192, },
  { 4, 8, 16, 32, 64, 512, 1024, 8192, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 8192, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 8192, },
  { 128, 512, 1024, 8192, },
  { 1, 128, 512, 1024, 8192, },
  { 2, 128, 512, 1024, 8192, },
  { 1, 2, 128, 512, 1024, 8192, },
  { 4, 128, 512, 1024, 8192, },
  { 1, 4, 128, 512, 1024, 8192, },
  { 2, 4, 128, 512, 1024, 8192, },
  { 1, 2, 4, 128, 512, 1024, 8192, },
  { 8, 128, 512, 1024, 8192, },
  { 1, 8, 128, 512, 1024, 8192, },
  { 2, 8, 128, 512, 1024, 8192, },
  { 1, 2, 8, 128, 512, 1024, 8192, },
  { 4, 8, 128, 512, 1024, 8192, },
  { 1, 4, 8, 128, 512, 1024, 8192, },
  { 2, 4, 8, 128, 512, 1024, 8192, },
  { 1, 2, 4, 8, 128, 512, 1024, 8192, },
  { 16, 128, 512, 1024, 8192, },
  { 1, 16, 128, 512, 1024, 8192, },
  { 2, 16, 128, 512, 1024, 8192, },
  { 1, 2, 16, 128, 512, 1024, 8192, },
  { 4, 16, 128, 512, 1024, 8192, },
  { 1, 4, 16, 128, 512, 1024, 8192, },
  { 2, 4, 16, 128, 512, 1024, 8192, },
  { 1, 2, 4, 16, 128, 512, 1024, 8192, },
  { 8, 16, 128, 512, 1024, 8192, },
  { 1, 8, 16, 128, 512, 1024, 8192, },
  { 2, 8, 16, 128, 512, 1024, 8192, },
  { 1, 2, 8, 16, 128, 512, 1024, 8192, },
  { 4, 8, 16, 128, 512, 1024, 8192, },
  { 1, 4, 8, 16, 128, 512, 1024, 8192, },
  { 2, 4, 8, 16, 128, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 8192, },
  { 32, 128, 512, 1024, 8192, },
  { 1, 32, 128, 512, 1024, 8192, },
  { 2, 32, 128, 512, 1024, 8192, },
  { 1, 2, 32, 128, 512, 1024, 8192, },
  { 4, 32, 128, 512, 1024, 8192, },
  { 1, 4, 32, 128, 512, 1024, 8192, },
  { 2, 4, 32, 128, 512, 1024, 8192, },
  { 1, 2, 4, 32, 128, 512, 1024, 8192, },
  { 8, 32, 128, 512, 1024, 8192, },
  { 1, 8, 32, 128, 512, 1024, 8192, },
  { 2, 8, 32, 128, 512, 1024, 8192, },
  { 1, 2, 8, 32, 128, 512, 1024, 8192, },
  { 4, 8, 32, 128, 512, 1024, 8192, },
  { 1, 4, 8, 32, 128, 512, 1024, 8192, },
  { 2, 4, 8, 32, 128, 512, 1024, 8192, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 8192, },
  { 16, 32, 128, 512, 1024, 8192, },
  { 1, 16, 32, 128, 512, 1024, 8192, },
  { 2, 16, 32, 128, 512, 1024, 8192, },
  { 1, 2, 16, 32, 128, 512, 1024, 8192, },
  { 4, 16, 32, 128, 512, 1024, 8192, },
  { 1, 4, 16, 32, 128, 512, 1024, 8192, },
  { 2, 4, 16, 32, 128, 512, 1024, 8192, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 8192, },
  { 8, 16, 32, 128, 512, 1024, 8192, },
  { 1, 8, 16, 32, 128, 512, 1024, 8192, },
  { 2, 8, 16, 32, 128, 512, 1024, 8192, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 8192, },
  { 4, 8, 16, 32, 128, 512, 1024, 8192, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 8192, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 8192, },
  { 64, 128, 512, 1024, 8192, },
  { 1, 64, 128, 512, 1024, 8192, },
  { 2, 64, 128, 512, 1024, 8192, },
  { 1, 2, 64, 128, 512, 1024, 8192, },
  { 4, 64, 128, 512, 1024, 8192, },
  { 1, 4, 64, 128, 512, 1024, 8192, },
  { 2, 4, 64, 128, 512, 1024, 8192, },
  { 1, 2, 4, 64, 128, 512, 1024, 8192, },
  { 8, 64, 128, 512, 1024, 8192, },
  { 1, 8, 64, 128, 512, 1024, 8192, },
  { 2, 8, 64, 128, 512, 1024, 8192, },
  { 1, 2, 8, 64, 128, 512, 1024, 8192, },
  { 4, 8, 64, 128, 512, 1024, 8192, },
  { 1, 4, 8, 64, 128, 512, 1024, 8192, },
  { 2, 4, 8, 64, 128, 512, 1024, 8192, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 8192, },
  { 16, 64, 128, 512, 1024, 8192, },
  { 1, 16, 64, 128, 512, 1024, 8192, },
  { 2, 16, 64, 128, 512, 1024, 8192, },
  { 1, 2, 16, 64, 128, 512, 1024, 8192, },
  { 4, 16, 64, 128, 512, 1024, 8192, },
  { 1, 4, 16, 64, 128, 512, 1024, 8192, },
  { 2, 4, 16, 64, 128, 512, 1024, 8192, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 8192, },
  { 8, 16, 64, 128, 512, 1024, 8192, },
  { 1, 8, 16, 64, 128, 512, 1024, 8192, },
  { 2, 8, 16, 64, 128, 512, 1024, 8192, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 8192, },
  { 4, 8, 16, 64, 128, 512, 1024, 8192, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 8192, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 8192, },
  { 32, 64, 128, 512, 1024, 8192, },
  { 1, 32, 64, 128, 512, 1024, 8192, },
  { 2, 32, 64, 128, 512, 1024, 8192, },
  { 1, 2, 32, 64, 128, 512, 1024, 8192, },
  { 4, 32, 64, 128, 512, 1024, 8192, },
  { 1, 4, 32, 64, 128, 512, 1024, 8192, },
  { 2, 4, 32, 64, 128, 512, 1024, 8192, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 8192, },
  { 8, 32, 64, 128, 512, 1024, 8192, },
  { 1, 8, 32, 64, 128, 512, 1024, 8192, },
  { 2, 8, 32, 64, 128, 512, 1024, 8192, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 8192, },
  { 4, 8, 32, 64, 128, 512, 1024, 8192, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 8192, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 8192, },
  { 16, 32, 64, 128, 512, 1024, 8192, },
  { 1, 16, 32, 64, 128, 512, 1024, 8192, },
  { 2, 16, 32, 64, 128, 512, 1024, 8192, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 8192, },
  { 4, 16, 32, 64, 128, 512, 1024, 8192, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 8192, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 8192, },
  { 8, 16, 32, 64, 128, 512, 1024, 8192, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 8192, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 8192, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, },
  { 256, 512, 1024, 8192, },
  { 1, 256, 512, 1024, 8192, },
  { 2, 256, 512, 1024, 8192, },
  { 1, 2, 256, 512, 1024, 8192, },
  { 4, 256, 512, 1024, 8192, },
  { 1, 4, 256, 512, 1024, 8192, },
  { 2, 4, 256, 512, 1024, 8192, },
  { 1, 2, 4, 256, 512, 1024, 8192, },
  { 8, 256, 512, 1024, 8192, },
  { 1, 8, 256, 512, 1024, 8192, },
  { 2, 8, 256, 512, 1024, 8192, },
  { 1, 2, 8, 256, 512, 1024, 8192, },
  { 4, 8, 256, 512, 1024, 8192, },
  { 1, 4, 8, 256, 512, 1024, 8192, },
  { 2, 4, 8, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 256, 512, 1024, 8192, },
  { 16, 256, 512, 1024, 8192, },
  { 1, 16, 256, 512, 1024, 8192, },
  { 2, 16, 256, 512, 1024, 8192, },
  { 1, 2, 16, 256, 512, 1024, 8192, },
  { 4, 16, 256, 512, 1024, 8192, },
  { 1, 4, 16, 256, 512, 1024, 8192, },
  { 2, 4, 16, 256, 512, 1024, 8192, },
  { 1, 2, 4, 16, 256, 512, 1024, 8192, },
  { 8, 16, 256, 512, 1024, 8192, },
  { 1, 8, 16, 256, 512, 1024, 8192, },
  { 2, 8, 16, 256, 512, 1024, 8192, },
  { 1, 2, 8, 16, 256, 512, 1024, 8192, },
  { 4, 8, 16, 256, 512, 1024, 8192, },
  { 1, 4, 8, 16, 256, 512, 1024, 8192, },
  { 2, 4, 8, 16, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 8192, },
  { 32, 256, 512, 1024, 8192, },
  { 1, 32, 256, 512, 1024, 8192, },
  { 2, 32, 256, 512, 1024, 8192, },
  { 1, 2, 32, 256, 512, 1024, 8192, },
  { 4, 32, 256, 512, 1024, 8192, },
  { 1, 4, 32, 256, 512, 1024, 8192, },
  { 2, 4, 32, 256, 512, 1024, 8192, },
  { 1, 2, 4, 32, 256, 512, 1024, 8192, },
  { 8, 32, 256, 512, 1024, 8192, },
  { 1, 8, 32, 256, 512, 1024, 8192, },
  { 2, 8, 32, 256, 512, 1024, 8192, },
  { 1, 2, 8, 32, 256, 512, 1024, 8192, },
  { 4, 8, 32, 256, 512, 1024, 8192, },
  { 1, 4, 8, 32, 256, 512, 1024, 8192, },
  { 2, 4, 8, 32, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 8192, },
  { 16, 32, 256, 512, 1024, 8192, },
  { 1, 16, 32, 256, 512, 1024, 8192, },
  { 2, 16, 32, 256, 512, 1024, 8192, },
  { 1, 2, 16, 32, 256, 512, 1024, 8192, },
  { 4, 16, 32, 256, 512, 1024, 8192, },
  { 1, 4, 16, 32, 256, 512, 1024, 8192, },
  { 2, 4, 16, 32, 256, 512, 1024, 8192, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 8192, },
  { 8, 16, 32, 256, 512, 1024, 8192, },
  { 1, 8, 16, 32, 256, 512, 1024, 8192, },
  { 2, 8, 16, 32, 256, 512, 1024, 8192, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 8192, },
  { 4, 8, 16, 32, 256, 512, 1024, 8192, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 8192, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 8192, },
  { 64, 256, 512, 1024, 8192, },
  { 1, 64, 256, 512, 1024, 8192, },
  { 2, 64, 256, 512, 1024, 8192, },
  { 1, 2, 64, 256, 512, 1024, 8192, },
  { 4, 64, 256, 512, 1024, 8192, },
  { 1, 4, 64, 256, 512, 1024, 8192, },
  { 2, 4, 64, 256, 512, 1024, 8192, },
  { 1, 2, 4, 64, 256, 512, 1024, 8192, },
  { 8, 64, 256, 512, 1024, 8192, },
  { 1, 8, 64, 256, 512, 1024, 8192, },
  { 2, 8, 64, 256, 512, 1024, 8192, },
  { 1, 2, 8, 64, 256, 512, 1024, 8192, },
  { 4, 8, 64, 256, 512, 1024, 8192, },
  { 1, 4, 8, 64, 256, 512, 1024, 8192, },
  { 2, 4, 8, 64, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 8192, },
  { 16, 64, 256, 512, 1024, 8192, },
  { 1, 16, 64, 256, 512, 1024, 8192, },
  { 2, 16, 64, 256, 512, 1024, 8192, },
  { 1, 2, 16, 64, 256, 512, 1024, 8192, },
  { 4, 16, 64, 256, 512, 1024, 8192, },
  { 1, 4, 16, 64, 256, 512, 1024, 8192, },
  { 2, 4, 16, 64, 256, 512, 1024, 8192, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 8192, },
  { 8, 16, 64, 256, 512, 1024, 8192, },
  { 1, 8, 16, 64, 256, 512, 1024, 8192, },
  { 2, 8, 16, 64, 256, 512, 1024, 8192, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 8192, },
  { 4, 8, 16, 64, 256, 512, 1024, 8192, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 8192, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 8192, },
  { 32, 64, 256, 512, 1024, 8192, },
  { 1, 32, 64, 256, 512, 1024, 8192, },
  { 2, 32, 64, 256, 512, 1024, 8192, },
  { 1, 2, 32, 64, 256, 512, 1024, 8192, },
  { 4, 32, 64, 256, 512, 1024, 8192, },
  { 1, 4, 32, 64, 256, 512, 1024, 8192, },
  { 2, 4, 32, 64, 256, 512, 1024, 8192, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 8192, },
  { 8, 32, 64, 256, 512, 1024, 8192, },
  { 1, 8, 32, 64, 256, 512, 1024, 8192, },
  { 2, 8, 32, 64, 256, 512, 1024, 8192, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 8192, },
  { 4, 8, 32, 64, 256, 512, 1024, 8192, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 8192, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 8192, },
  { 16, 32, 64, 256, 512, 1024, 8192, },
  { 1, 16, 32, 64, 256, 512, 1024, 8192, },
  { 2, 16, 32, 64, 256, 512, 1024, 8192, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 8192, },
  { 4, 16, 32, 64, 256, 512, 1024, 8192, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 8192, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 8192, },
  { 8, 16, 32, 64, 256, 512, 1024, 8192, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 8192, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 8192, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, },
  { 128, 256, 512, 1024, 8192, },
  { 1, 128, 256, 512, 1024, 8192, },
  { 2, 128, 256, 512, 1024, 8192, },
  { 1, 2, 128, 256, 512, 1024, 8192, },
  { 4, 128, 256, 512, 1024, 8192, },
  { 1, 4, 128, 256, 512, 1024, 8192, },
  { 2, 4, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 128, 256, 512, 1024, 8192, },
  { 8, 128, 256, 512, 1024, 8192, },
  { 1, 8, 128, 256, 512, 1024, 8192, },
  { 2, 8, 128, 256, 512, 1024, 8192, },
  { 1, 2, 8, 128, 256, 512, 1024, 8192, },
  { 4, 8, 128, 256, 512, 1024, 8192, },
  { 1, 4, 8, 128, 256, 512, 1024, 8192, },
  { 2, 4, 8, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 8192, },
  { 16, 128, 256, 512, 1024, 8192, },
  { 1, 16, 128, 256, 512, 1024, 8192, },
  { 2, 16, 128, 256, 512, 1024, 8192, },
  { 1, 2, 16, 128, 256, 512, 1024, 8192, },
  { 4, 16, 128, 256, 512, 1024, 8192, },
  { 1, 4, 16, 128, 256, 512, 1024, 8192, },
  { 2, 4, 16, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 8192, },
  { 8, 16, 128, 256, 512, 1024, 8192, },
  { 1, 8, 16, 128, 256, 512, 1024, 8192, },
  { 2, 8, 16, 128, 256, 512, 1024, 8192, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 8192, },
  { 4, 8, 16, 128, 256, 512, 1024, 8192, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 8192, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 8192, },
  { 32, 128, 256, 512, 1024, 8192, },
  { 1, 32, 128, 256, 512, 1024, 8192, },
  { 2, 32, 128, 256, 512, 1024, 8192, },
  { 1, 2, 32, 128, 256, 512, 1024, 8192, },
  { 4, 32, 128, 256, 512, 1024, 8192, },
  { 1, 4, 32, 128, 256, 512, 1024, 8192, },
  { 2, 4, 32, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 8192, },
  { 8, 32, 128, 256, 512, 1024, 8192, },
  { 1, 8, 32, 128, 256, 512, 1024, 8192, },
  { 2, 8, 32, 128, 256, 512, 1024, 8192, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 8192, },
  { 4, 8, 32, 128, 256, 512, 1024, 8192, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 8192, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 8192, },
  { 16, 32, 128, 256, 512, 1024, 8192, },
  { 1, 16, 32, 128, 256, 512, 1024, 8192, },
  { 2, 16, 32, 128, 256, 512, 1024, 8192, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 8192, },
  { 4, 16, 32, 128, 256, 512, 1024, 8192, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 8192, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 8192, },
  { 8, 16, 32, 128, 256, 512, 1024, 8192, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 8192, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 8192, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, },
  { 64, 128, 256, 512, 1024, 8192, },
  { 1, 64, 128, 256, 512, 1024, 8192, },
  { 2, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 64, 128, 256, 512, 1024, 8192, },
  { 4, 64, 128, 256, 512, 1024, 8192, },
  { 1, 4, 64, 128, 256, 512, 1024, 8192, },
  { 2, 4, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 8192, },
  { 8, 64, 128, 256, 512, 1024, 8192, },
  { 1, 8, 64, 128, 256, 512, 1024, 8192, },
  { 2, 8, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 8192, },
  { 4, 8, 64, 128, 256, 512, 1024, 8192, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 8192, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 8192, },
  { 16, 64, 128, 256, 512, 1024, 8192, },
  { 1, 16, 64, 128, 256, 512, 1024, 8192, },
  { 2, 16, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 8192, },
  { 4, 16, 64, 128, 256, 512, 1024, 8192, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 8192, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 8192, },
  { 8, 16, 64, 128, 256, 512, 1024, 8192, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 8192, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 8192, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, },
  { 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 8192, },
  { 4, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 8192, },
  { 8, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 8192, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, },
  { 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, },
  { 2048, 8192, },
  { 1, 2048, 8192, },
  { 2, 2048, 8192, },
  { 1, 2, 2048, 8192, },
  { 4, 2048, 8192, },
  { 1, 4, 2048, 8192, },
  { 2, 4, 2048, 8192, },
  { 1, 2, 4, 2048, 8192, },
  { 8, 2048, 8192, },
  { 1, 8, 2048, 8192, },
  { 2, 8, 2048, 8192, },
  { 1, 2, 8, 2048, 8192, },
  { 4, 8, 2048, 8192, },
  { 1, 4, 8, 2048, 8192, },
  { 2, 4, 8, 2048, 8192, },
  { 1, 2, 4, 8, 2048, 8192, },
  { 16, 2048, 8192, },
  { 1, 16, 2048, 8192, },
  { 2, 16, 2048, 8192, },
  { 1, 2, 16, 2048, 8192, },
  { 4, 16, 2048, 8192, },
  { 1, 4, 16, 2048, 8192, },
  { 2, 4, 16, 2048, 8192, },
  { 1, 2, 4, 16, 2048, 8192, },
  { 8, 16, 2048, 8192, },
  { 1, 8, 16, 2048, 8192, },
  { 2, 8, 16, 2048, 8192, },
  { 1, 2, 8, 16, 2048, 8192, },
  { 4, 8, 16, 2048, 8192, },
  { 1, 4, 8, 16, 2048, 8192, },
  { 2, 4, 8, 16, 2048, 8192, },
  { 1, 2, 4, 8, 16, 2048, 8192, },
  { 32, 2048, 8192, },
  { 1, 32, 2048, 8192, },
  { 2, 32, 2048, 8192, },
  { 1, 2, 32, 2048, 8192, },
  { 4, 32, 2048, 8192, },
  { 1, 4, 32, 2048, 8192, },
  { 2, 4, 32, 2048, 8192, },
  { 1, 2, 4, 32, 2048, 8192, },
  { 8, 32, 2048, 8192, },
  { 1, 8, 32, 2048, 8192, },
  { 2, 8, 32, 2048, 8192, },
  { 1, 2, 8, 32, 2048, 8192, },
  { 4, 8, 32, 2048, 8192, },
  { 1, 4, 8, 32, 2048, 8192, },
  { 2, 4, 8, 32, 2048, 8192, },
  { 1, 2, 4, 8, 32, 2048, 8192, },
  { 16, 32, 2048, 8192, },
  { 1, 16, 32, 2048, 8192, },
  { 2, 16, 32, 2048, 8192, },
  { 1, 2, 16, 32, 2048, 8192, },
  { 4, 16, 32, 2048, 8192, },
  { 1, 4, 16, 32, 2048, 8192, },
  { 2, 4, 16, 32, 2048, 8192, },
  { 1, 2, 4, 16, 32, 2048, 8192, },
  { 8, 16, 32, 2048, 8192, },
  { 1, 8, 16, 32, 2048, 8192, },
  { 2, 8, 16, 32, 2048, 8192, },
  { 1, 2, 8, 16, 32, 2048, 8192, },
  { 4, 8, 16, 32, 2048, 8192, },
  { 1, 4, 8, 16, 32, 2048, 8192, },
  { 2, 4, 8, 16, 32, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 2048, 8192, },
  { 64, 2048, 8192, },
  { 1, 64, 2048, 8192, },
  { 2, 64, 2048, 8192, },
  { 1, 2, 64, 2048, 8192, },
  { 4, 64, 2048, 8192, },
  { 1, 4, 64, 2048, 8192, },
  { 2, 4, 64, 2048, 8192, },
  { 1, 2, 4, 64, 2048, 8192, },
  { 8, 64, 2048, 8192, },
  { 1, 8, 64, 2048, 8192, },
  { 2, 8, 64, 2048, 8192, },
  { 1, 2, 8, 64, 2048, 8192, },
  { 4, 8, 64, 2048, 8192, },
  { 1, 4, 8, 64, 2048, 8192, },
  { 2, 4, 8, 64, 2048, 8192, },
  { 1, 2, 4, 8, 64, 2048, 8192, },
  { 16, 64, 2048, 8192, },
  { 1, 16, 64, 2048, 8192, },
  { 2, 16, 64, 2048, 8192, },
  { 1, 2, 16, 64, 2048, 8192, },
  { 4, 16, 64, 2048, 8192, },
  { 1, 4, 16, 64, 2048, 8192, },
  { 2, 4, 16, 64, 2048, 8192, },
  { 1, 2, 4, 16, 64, 2048, 8192, },
  { 8, 16, 64, 2048, 8192, },
  { 1, 8, 16, 64, 2048, 8192, },
  { 2, 8, 16, 64, 2048, 8192, },
  { 1, 2, 8, 16, 64, 2048, 8192, },
  { 4, 8, 16, 64, 2048, 8192, },
  { 1, 4, 8, 16, 64, 2048, 8192, },
  { 2, 4, 8, 16, 64, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 2048, 8192, },
  { 32, 64, 2048, 8192, },
  { 1, 32, 64, 2048, 8192, },
  { 2, 32, 64, 2048, 8192, },
  { 1, 2, 32, 64, 2048, 8192, },
  { 4, 32, 64, 2048, 8192, },
  { 1, 4, 32, 64, 2048, 8192, },
  { 2, 4, 32, 64, 2048, 8192, },
  { 1, 2, 4, 32, 64, 2048, 8192, },
  { 8, 32, 64, 2048, 8192, },
  { 1, 8, 32, 64, 2048, 8192, },
  { 2, 8, 32, 64, 2048, 8192, },
  { 1, 2, 8, 32, 64, 2048, 8192, },
  { 4, 8, 32, 64, 2048, 8192, },
  { 1, 4, 8, 32, 64, 2048, 8192, },
  { 2, 4, 8, 32, 64, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 2048, 8192, },
  { 16, 32, 64, 2048, 8192, },
  { 1, 16, 32, 64, 2048, 8192, },
  { 2, 16, 32, 64, 2048, 8192, },
  { 1, 2, 16, 32, 64, 2048, 8192, },
  { 4, 16, 32, 64, 2048, 8192, },
  { 1, 4, 16, 32, 64, 2048, 8192, },
  { 2, 4, 16, 32, 64, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 2048, 8192, },
  { 8, 16, 32, 64, 2048, 8192, },
  { 1, 8, 16, 32, 64, 2048, 8192, },
  { 2, 8, 16, 32, 64, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 2048, 8192, },
  { 4, 8, 16, 32, 64, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 8192, },
  { 128, 2048, 8192, },
  { 1, 128, 2048, 8192, },
  { 2, 128, 2048, 8192, },
  { 1, 2, 128, 2048, 8192, },
  { 4, 128, 2048, 8192, },
  { 1, 4, 128, 2048, 8192, },
  { 2, 4, 128, 2048, 8192, },
  { 1, 2, 4, 128, 2048, 8192, },
  { 8, 128, 2048, 8192, },
  { 1, 8, 128, 2048, 8192, },
  { 2, 8, 128, 2048, 8192, },
  { 1, 2, 8, 128, 2048, 8192, },
  { 4, 8, 128, 2048, 8192, },
  { 1, 4, 8, 128, 2048, 8192, },
  { 2, 4, 8, 128, 2048, 8192, },
  { 1, 2, 4, 8, 128, 2048, 8192, },
  { 16, 128, 2048, 8192, },
  { 1, 16, 128, 2048, 8192, },
  { 2, 16, 128, 2048, 8192, },
  { 1, 2, 16, 128, 2048, 8192, },
  { 4, 16, 128, 2048, 8192, },
  { 1, 4, 16, 128, 2048, 8192, },
  { 2, 4, 16, 128, 2048, 8192, },
  { 1, 2, 4, 16, 128, 2048, 8192, },
  { 8, 16, 128, 2048, 8192, },
  { 1, 8, 16, 128, 2048, 8192, },
  { 2, 8, 16, 128, 2048, 8192, },
  { 1, 2, 8, 16, 128, 2048, 8192, },
  { 4, 8, 16, 128, 2048, 8192, },
  { 1, 4, 8, 16, 128, 2048, 8192, },
  { 2, 4, 8, 16, 128, 2048, 8192, },
  { 1, 2, 4, 8, 16, 128, 2048, 8192, },
  { 32, 128, 2048, 8192, },
  { 1, 32, 128, 2048, 8192, },
  { 2, 32, 128, 2048, 8192, },
  { 1, 2, 32, 128, 2048, 8192, },
  { 4, 32, 128, 2048, 8192, },
  { 1, 4, 32, 128, 2048, 8192, },
  { 2, 4, 32, 128, 2048, 8192, },
  { 1, 2, 4, 32, 128, 2048, 8192, },
  { 8, 32, 128, 2048, 8192, },
  { 1, 8, 32, 128, 2048, 8192, },
  { 2, 8, 32, 128, 2048, 8192, },
  { 1, 2, 8, 32, 128, 2048, 8192, },
  { 4, 8, 32, 128, 2048, 8192, },
  { 1, 4, 8, 32, 128, 2048, 8192, },
  { 2, 4, 8, 32, 128, 2048, 8192, },
  { 1, 2, 4, 8, 32, 128, 2048, 8192, },
  { 16, 32, 128, 2048, 8192, },
  { 1, 16, 32, 128, 2048, 8192, },
  { 2, 16, 32, 128, 2048, 8192, },
  { 1, 2, 16, 32, 128, 2048, 8192, },
  { 4, 16, 32, 128, 2048, 8192, },
  { 1, 4, 16, 32, 128, 2048, 8192, },
  { 2, 4, 16, 32, 128, 2048, 8192, },
  { 1, 2, 4, 16, 32, 128, 2048, 8192, },
  { 8, 16, 32, 128, 2048, 8192, },
  { 1, 8, 16, 32, 128, 2048, 8192, },
  { 2, 8, 16, 32, 128, 2048, 8192, },
  { 1, 2, 8, 16, 32, 128, 2048, 8192, },
  { 4, 8, 16, 32, 128, 2048, 8192, },
  { 1, 4, 8, 16, 32, 128, 2048, 8192, },
  { 2, 4, 8, 16, 32, 128, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 8192, },
  { 64, 128, 2048, 8192, },
  { 1, 64, 128, 2048, 8192, },
  { 2, 64, 128, 2048, 8192, },
  { 1, 2, 64, 128, 2048, 8192, },
  { 4, 64, 128, 2048, 8192, },
  { 1, 4, 64, 128, 2048, 8192, },
  { 2, 4, 64, 128, 2048, 8192, },
  { 1, 2, 4, 64, 128, 2048, 8192, },
  { 8, 64, 128, 2048, 8192, },
  { 1, 8, 64, 128, 2048, 8192, },
  { 2, 8, 64, 128, 2048, 8192, },
  { 1, 2, 8, 64, 128, 2048, 8192, },
  { 4, 8, 64, 128, 2048, 8192, },
  { 1, 4, 8, 64, 128, 2048, 8192, },
  { 2, 4, 8, 64, 128, 2048, 8192, },
  { 1, 2, 4, 8, 64, 128, 2048, 8192, },
  { 16, 64, 128, 2048, 8192, },
  { 1, 16, 64, 128, 2048, 8192, },
  { 2, 16, 64, 128, 2048, 8192, },
  { 1, 2, 16, 64, 128, 2048, 8192, },
  { 4, 16, 64, 128, 2048, 8192, },
  { 1, 4, 16, 64, 128, 2048, 8192, },
  { 2, 4, 16, 64, 128, 2048, 8192, },
  { 1, 2, 4, 16, 64, 128, 2048, 8192, },
  { 8, 16, 64, 128, 2048, 8192, },
  { 1, 8, 16, 64, 128, 2048, 8192, },
  { 2, 8, 16, 64, 128, 2048, 8192, },
  { 1, 2, 8, 16, 64, 128, 2048, 8192, },
  { 4, 8, 16, 64, 128, 2048, 8192, },
  { 1, 4, 8, 16, 64, 128, 2048, 8192, },
  { 2, 4, 8, 16, 64, 128, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 8192, },
  { 32, 64, 128, 2048, 8192, },
  { 1, 32, 64, 128, 2048, 8192, },
  { 2, 32, 64, 128, 2048, 8192, },
  { 1, 2, 32, 64, 128, 2048, 8192, },
  { 4, 32, 64, 128, 2048, 8192, },
  { 1, 4, 32, 64, 128, 2048, 8192, },
  { 2, 4, 32, 64, 128, 2048, 8192, },
  { 1, 2, 4, 32, 64, 128, 2048, 8192, },
  { 8, 32, 64, 128, 2048, 8192, },
  { 1, 8, 32, 64, 128, 2048, 8192, },
  { 2, 8, 32, 64, 128, 2048, 8192, },
  { 1, 2, 8, 32, 64, 128, 2048, 8192, },
  { 4, 8, 32, 64, 128, 2048, 8192, },
  { 1, 4, 8, 32, 64, 128, 2048, 8192, },
  { 2, 4, 8, 32, 64, 128, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 8192, },
  { 16, 32, 64, 128, 2048, 8192, },
  { 1, 16, 32, 64, 128, 2048, 8192, },
  { 2, 16, 32, 64, 128, 2048, 8192, },
  { 1, 2, 16, 32, 64, 128, 2048, 8192, },
  { 4, 16, 32, 64, 128, 2048, 8192, },
  { 1, 4, 16, 32, 64, 128, 2048, 8192, },
  { 2, 4, 16, 32, 64, 128, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 8192, },
  { 8, 16, 32, 64, 128, 2048, 8192, },
  { 1, 8, 16, 32, 64, 128, 2048, 8192, },
  { 2, 8, 16, 32, 64, 128, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 8192, },
  { 4, 8, 16, 32, 64, 128, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 8192, },
  { 256, 2048, 8192, },
  { 1, 256, 2048, 8192, },
  { 2, 256, 2048, 8192, },
  { 1, 2, 256, 2048, 8192, },
  { 4, 256, 2048, 8192, },
  { 1, 4, 256, 2048, 8192, },
  { 2, 4, 256, 2048, 8192, },
  { 1, 2, 4, 256, 2048, 8192, },
  { 8, 256, 2048, 8192, },
  { 1, 8, 256, 2048, 8192, },
  { 2, 8, 256, 2048, 8192, },
  { 1, 2, 8, 256, 2048, 8192, },
  { 4, 8, 256, 2048, 8192, },
  { 1, 4, 8, 256, 2048, 8192, },
  { 2, 4, 8, 256, 2048, 8192, },
  { 1, 2, 4, 8, 256, 2048, 8192, },
  { 16, 256, 2048, 8192, },
  { 1, 16, 256, 2048, 8192, },
  { 2, 16, 256, 2048, 8192, },
  { 1, 2, 16, 256, 2048, 8192, },
  { 4, 16, 256, 2048, 8192, },
  { 1, 4, 16, 256, 2048, 8192, },
  { 2, 4, 16, 256, 2048, 8192, },
  { 1, 2, 4, 16, 256, 2048, 8192, },
  { 8, 16, 256, 2048, 8192, },
  { 1, 8, 16, 256, 2048, 8192, },
  { 2, 8, 16, 256, 2048, 8192, },
  { 1, 2, 8, 16, 256, 2048, 8192, },
  { 4, 8, 16, 256, 2048, 8192, },
  { 1, 4, 8, 16, 256, 2048, 8192, },
  { 2, 4, 8, 16, 256, 2048, 8192, },
  { 1, 2, 4, 8, 16, 256, 2048, 8192, },
  { 32, 256, 2048, 8192, },
  { 1, 32, 256, 2048, 8192, },
  { 2, 32, 256, 2048, 8192, },
  { 1, 2, 32, 256, 2048, 8192, },
  { 4, 32, 256, 2048, 8192, },
  { 1, 4, 32, 256, 2048, 8192, },
  { 2, 4, 32, 256, 2048, 8192, },
  { 1, 2, 4, 32, 256, 2048, 8192, },
  { 8, 32, 256, 2048, 8192, },
  { 1, 8, 32, 256, 2048, 8192, },
  { 2, 8, 32, 256, 2048, 8192, },
  { 1, 2, 8, 32, 256, 2048, 8192, },
  { 4, 8, 32, 256, 2048, 8192, },
  { 1, 4, 8, 32, 256, 2048, 8192, },
  { 2, 4, 8, 32, 256, 2048, 8192, },
  { 1, 2, 4, 8, 32, 256, 2048, 8192, },
  { 16, 32, 256, 2048, 8192, },
  { 1, 16, 32, 256, 2048, 8192, },
  { 2, 16, 32, 256, 2048, 8192, },
  { 1, 2, 16, 32, 256, 2048, 8192, },
  { 4, 16, 32, 256, 2048, 8192, },
  { 1, 4, 16, 32, 256, 2048, 8192, },
  { 2, 4, 16, 32, 256, 2048, 8192, },
  { 1, 2, 4, 16, 32, 256, 2048, 8192, },
  { 8, 16, 32, 256, 2048, 8192, },
  { 1, 8, 16, 32, 256, 2048, 8192, },
  { 2, 8, 16, 32, 256, 2048, 8192, },
  { 1, 2, 8, 16, 32, 256, 2048, 8192, },
  { 4, 8, 16, 32, 256, 2048, 8192, },
  { 1, 4, 8, 16, 32, 256, 2048, 8192, },
  { 2, 4, 8, 16, 32, 256, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 8192, },
  { 64, 256, 2048, 8192, },
  { 1, 64, 256, 2048, 8192, },
  { 2, 64, 256, 2048, 8192, },
  { 1, 2, 64, 256, 2048, 8192, },
  { 4, 64, 256, 2048, 8192, },
  { 1, 4, 64, 256, 2048, 8192, },
  { 2, 4, 64, 256, 2048, 8192, },
  { 1, 2, 4, 64, 256, 2048, 8192, },
  { 8, 64, 256, 2048, 8192, },
  { 1, 8, 64, 256, 2048, 8192, },
  { 2, 8, 64, 256, 2048, 8192, },
  { 1, 2, 8, 64, 256, 2048, 8192, },
  { 4, 8, 64, 256, 2048, 8192, },
  { 1, 4, 8, 64, 256, 2048, 8192, },
  { 2, 4, 8, 64, 256, 2048, 8192, },
  { 1, 2, 4, 8, 64, 256, 2048, 8192, },
  { 16, 64, 256, 2048, 8192, },
  { 1, 16, 64, 256, 2048, 8192, },
  { 2, 16, 64, 256, 2048, 8192, },
  { 1, 2, 16, 64, 256, 2048, 8192, },
  { 4, 16, 64, 256, 2048, 8192, },
  { 1, 4, 16, 64, 256, 2048, 8192, },
  { 2, 4, 16, 64, 256, 2048, 8192, },
  { 1, 2, 4, 16, 64, 256, 2048, 8192, },
  { 8, 16, 64, 256, 2048, 8192, },
  { 1, 8, 16, 64, 256, 2048, 8192, },
  { 2, 8, 16, 64, 256, 2048, 8192, },
  { 1, 2, 8, 16, 64, 256, 2048, 8192, },
  { 4, 8, 16, 64, 256, 2048, 8192, },
  { 1, 4, 8, 16, 64, 256, 2048, 8192, },
  { 2, 4, 8, 16, 64, 256, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 8192, },
  { 32, 64, 256, 2048, 8192, },
  { 1, 32, 64, 256, 2048, 8192, },
  { 2, 32, 64, 256, 2048, 8192, },
  { 1, 2, 32, 64, 256, 2048, 8192, },
  { 4, 32, 64, 256, 2048, 8192, },
  { 1, 4, 32, 64, 256, 2048, 8192, },
  { 2, 4, 32, 64, 256, 2048, 8192, },
  { 1, 2, 4, 32, 64, 256, 2048, 8192, },
  { 8, 32, 64, 256, 2048, 8192, },
  { 1, 8, 32, 64, 256, 2048, 8192, },
  { 2, 8, 32, 64, 256, 2048, 8192, },
  { 1, 2, 8, 32, 64, 256, 2048, 8192, },
  { 4, 8, 32, 64, 256, 2048, 8192, },
  { 1, 4, 8, 32, 64, 256, 2048, 8192, },
  { 2, 4, 8, 32, 64, 256, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 8192, },
  { 16, 32, 64, 256, 2048, 8192, },
  { 1, 16, 32, 64, 256, 2048, 8192, },
  { 2, 16, 32, 64, 256, 2048, 8192, },
  { 1, 2, 16, 32, 64, 256, 2048, 8192, },
  { 4, 16, 32, 64, 256, 2048, 8192, },
  { 1, 4, 16, 32, 64, 256, 2048, 8192, },
  { 2, 4, 16, 32, 64, 256, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 8192, },
  { 8, 16, 32, 64, 256, 2048, 8192, },
  { 1, 8, 16, 32, 64, 256, 2048, 8192, },
  { 2, 8, 16, 32, 64, 256, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 8192, },
  { 4, 8, 16, 32, 64, 256, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 8192, },
  { 128, 256, 2048, 8192, },
  { 1, 128, 256, 2048, 8192, },
  { 2, 128, 256, 2048, 8192, },
  { 1, 2, 128, 256, 2048, 8192, },
  { 4, 128, 256, 2048, 8192, },
  { 1, 4, 128, 256, 2048, 8192, },
  { 2, 4, 128, 256, 2048, 8192, },
  { 1, 2, 4, 128, 256, 2048, 8192, },
  { 8, 128, 256, 2048, 8192, },
  { 1, 8, 128, 256, 2048, 8192, },
  { 2, 8, 128, 256, 2048, 8192, },
  { 1, 2, 8, 128, 256, 2048, 8192, },
  { 4, 8, 128, 256, 2048, 8192, },
  { 1, 4, 8, 128, 256, 2048, 8192, },
  { 2, 4, 8, 128, 256, 2048, 8192, },
  { 1, 2, 4, 8, 128, 256, 2048, 8192, },
  { 16, 128, 256, 2048, 8192, },
  { 1, 16, 128, 256, 2048, 8192, },
  { 2, 16, 128, 256, 2048, 8192, },
  { 1, 2, 16, 128, 256, 2048, 8192, },
  { 4, 16, 128, 256, 2048, 8192, },
  { 1, 4, 16, 128, 256, 2048, 8192, },
  { 2, 4, 16, 128, 256, 2048, 8192, },
  { 1, 2, 4, 16, 128, 256, 2048, 8192, },
  { 8, 16, 128, 256, 2048, 8192, },
  { 1, 8, 16, 128, 256, 2048, 8192, },
  { 2, 8, 16, 128, 256, 2048, 8192, },
  { 1, 2, 8, 16, 128, 256, 2048, 8192, },
  { 4, 8, 16, 128, 256, 2048, 8192, },
  { 1, 4, 8, 16, 128, 256, 2048, 8192, },
  { 2, 4, 8, 16, 128, 256, 2048, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 8192, },
  { 32, 128, 256, 2048, 8192, },
  { 1, 32, 128, 256, 2048, 8192, },
  { 2, 32, 128, 256, 2048, 8192, },
  { 1, 2, 32, 128, 256, 2048, 8192, },
  { 4, 32, 128, 256, 2048, 8192, },
  { 1, 4, 32, 128, 256, 2048, 8192, },
  { 2, 4, 32, 128, 256, 2048, 8192, },
  { 1, 2, 4, 32, 128, 256, 2048, 8192, },
  { 8, 32, 128, 256, 2048, 8192, },
  { 1, 8, 32, 128, 256, 2048, 8192, },
  { 2, 8, 32, 128, 256, 2048, 8192, },
  { 1, 2, 8, 32, 128, 256, 2048, 8192, },
  { 4, 8, 32, 128, 256, 2048, 8192, },
  { 1, 4, 8, 32, 128, 256, 2048, 8192, },
  { 2, 4, 8, 32, 128, 256, 2048, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 8192, },
  { 16, 32, 128, 256, 2048, 8192, },
  { 1, 16, 32, 128, 256, 2048, 8192, },
  { 2, 16, 32, 128, 256, 2048, 8192, },
  { 1, 2, 16, 32, 128, 256, 2048, 8192, },
  { 4, 16, 32, 128, 256, 2048, 8192, },
  { 1, 4, 16, 32, 128, 256, 2048, 8192, },
  { 2, 4, 16, 32, 128, 256, 2048, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 8192, },
  { 8, 16, 32, 128, 256, 2048, 8192, },
  { 1, 8, 16, 32, 128, 256, 2048, 8192, },
  { 2, 8, 16, 32, 128, 256, 2048, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 8192, },
  { 4, 8, 16, 32, 128, 256, 2048, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 8192, },
  { 64, 128, 256, 2048, 8192, },
  { 1, 64, 128, 256, 2048, 8192, },
  { 2, 64, 128, 256, 2048, 8192, },
  { 1, 2, 64, 128, 256, 2048, 8192, },
  { 4, 64, 128, 256, 2048, 8192, },
  { 1, 4, 64, 128, 256, 2048, 8192, },
  { 2, 4, 64, 128, 256, 2048, 8192, },
  { 1, 2, 4, 64, 128, 256, 2048, 8192, },
  { 8, 64, 128, 256, 2048, 8192, },
  { 1, 8, 64, 128, 256, 2048, 8192, },
  { 2, 8, 64, 128, 256, 2048, 8192, },
  { 1, 2, 8, 64, 128, 256, 2048, 8192, },
  { 4, 8, 64, 128, 256, 2048, 8192, },
  { 1, 4, 8, 64, 128, 256, 2048, 8192, },
  { 2, 4, 8, 64, 128, 256, 2048, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 8192, },
  { 16, 64, 128, 256, 2048, 8192, },
  { 1, 16, 64, 128, 256, 2048, 8192, },
  { 2, 16, 64, 128, 256, 2048, 8192, },
  { 1, 2, 16, 64, 128, 256, 2048, 8192, },
  { 4, 16, 64, 128, 256, 2048, 8192, },
  { 1, 4, 16, 64, 128, 256, 2048, 8192, },
  { 2, 4, 16, 64, 128, 256, 2048, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 8192, },
  { 8, 16, 64, 128, 256, 2048, 8192, },
  { 1, 8, 16, 64, 128, 256, 2048, 8192, },
  { 2, 8, 16, 64, 128, 256, 2048, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 8192, },
  { 4, 8, 16, 64, 128, 256, 2048, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 8192, },
  { 32, 64, 128, 256, 2048, 8192, },
  { 1, 32, 64, 128, 256, 2048, 8192, },
  { 2, 32, 64, 128, 256, 2048, 8192, },
  { 1, 2, 32, 64, 128, 256, 2048, 8192, },
  { 4, 32, 64, 128, 256, 2048, 8192, },
  { 1, 4, 32, 64, 128, 256, 2048, 8192, },
  { 2, 4, 32, 64, 128, 256, 2048, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 8192, },
  { 8, 32, 64, 128, 256, 2048, 8192, },
  { 1, 8, 32, 64, 128, 256, 2048, 8192, },
  { 2, 8, 32, 64, 128, 256, 2048, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 8192, },
  { 4, 8, 32, 64, 128, 256, 2048, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 8192, },
  { 16, 32, 64, 128, 256, 2048, 8192, },
  { 1, 16, 32, 64, 128, 256, 2048, 8192, },
  { 2, 16, 32, 64, 128, 256, 2048, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 8192, },
  { 4, 16, 32, 64, 128, 256, 2048, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 8192, },
  { 8, 16, 32, 64, 128, 256, 2048, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, },
  { 512, 2048, 8192, },
  { 1, 512, 2048, 8192, },
  { 2, 512, 2048, 8192, },
  { 1, 2, 512, 2048, 8192, },
  { 4, 512, 2048, 8192, },
  { 1, 4, 512, 2048, 8192, },
  { 2, 4, 512, 2048, 8192, },
  { 1, 2, 4, 512, 2048, 8192, },
  { 8, 512, 2048, 8192, },
  { 1, 8, 512, 2048, 8192, },
  { 2, 8, 512, 2048, 8192, },
  { 1, 2, 8, 512, 2048, 8192, },
  { 4, 8, 512, 2048, 8192, },
  { 1, 4, 8, 512, 2048, 8192, },
  { 2, 4, 8, 512, 2048, 8192, },
  { 1, 2, 4, 8, 512, 2048, 8192, },
  { 16, 512, 2048, 8192, },
  { 1, 16, 512, 2048, 8192, },
  { 2, 16, 512, 2048, 8192, },
  { 1, 2, 16, 512, 2048, 8192, },
  { 4, 16, 512, 2048, 8192, },
  { 1, 4, 16, 512, 2048, 8192, },
  { 2, 4, 16, 512, 2048, 8192, },
  { 1, 2, 4, 16, 512, 2048, 8192, },
  { 8, 16, 512, 2048, 8192, },
  { 1, 8, 16, 512, 2048, 8192, },
  { 2, 8, 16, 512, 2048, 8192, },
  { 1, 2, 8, 16, 512, 2048, 8192, },
  { 4, 8, 16, 512, 2048, 8192, },
  { 1, 4, 8, 16, 512, 2048, 8192, },
  { 2, 4, 8, 16, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 512, 2048, 8192, },
  { 32, 512, 2048, 8192, },
  { 1, 32, 512, 2048, 8192, },
  { 2, 32, 512, 2048, 8192, },
  { 1, 2, 32, 512, 2048, 8192, },
  { 4, 32, 512, 2048, 8192, },
  { 1, 4, 32, 512, 2048, 8192, },
  { 2, 4, 32, 512, 2048, 8192, },
  { 1, 2, 4, 32, 512, 2048, 8192, },
  { 8, 32, 512, 2048, 8192, },
  { 1, 8, 32, 512, 2048, 8192, },
  { 2, 8, 32, 512, 2048, 8192, },
  { 1, 2, 8, 32, 512, 2048, 8192, },
  { 4, 8, 32, 512, 2048, 8192, },
  { 1, 4, 8, 32, 512, 2048, 8192, },
  { 2, 4, 8, 32, 512, 2048, 8192, },
  { 1, 2, 4, 8, 32, 512, 2048, 8192, },
  { 16, 32, 512, 2048, 8192, },
  { 1, 16, 32, 512, 2048, 8192, },
  { 2, 16, 32, 512, 2048, 8192, },
  { 1, 2, 16, 32, 512, 2048, 8192, },
  { 4, 16, 32, 512, 2048, 8192, },
  { 1, 4, 16, 32, 512, 2048, 8192, },
  { 2, 4, 16, 32, 512, 2048, 8192, },
  { 1, 2, 4, 16, 32, 512, 2048, 8192, },
  { 8, 16, 32, 512, 2048, 8192, },
  { 1, 8, 16, 32, 512, 2048, 8192, },
  { 2, 8, 16, 32, 512, 2048, 8192, },
  { 1, 2, 8, 16, 32, 512, 2048, 8192, },
  { 4, 8, 16, 32, 512, 2048, 8192, },
  { 1, 4, 8, 16, 32, 512, 2048, 8192, },
  { 2, 4, 8, 16, 32, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 8192, },
  { 64, 512, 2048, 8192, },
  { 1, 64, 512, 2048, 8192, },
  { 2, 64, 512, 2048, 8192, },
  { 1, 2, 64, 512, 2048, 8192, },
  { 4, 64, 512, 2048, 8192, },
  { 1, 4, 64, 512, 2048, 8192, },
  { 2, 4, 64, 512, 2048, 8192, },
  { 1, 2, 4, 64, 512, 2048, 8192, },
  { 8, 64, 512, 2048, 8192, },
  { 1, 8, 64, 512, 2048, 8192, },
  { 2, 8, 64, 512, 2048, 8192, },
  { 1, 2, 8, 64, 512, 2048, 8192, },
  { 4, 8, 64, 512, 2048, 8192, },
  { 1, 4, 8, 64, 512, 2048, 8192, },
  { 2, 4, 8, 64, 512, 2048, 8192, },
  { 1, 2, 4, 8, 64, 512, 2048, 8192, },
  { 16, 64, 512, 2048, 8192, },
  { 1, 16, 64, 512, 2048, 8192, },
  { 2, 16, 64, 512, 2048, 8192, },
  { 1, 2, 16, 64, 512, 2048, 8192, },
  { 4, 16, 64, 512, 2048, 8192, },
  { 1, 4, 16, 64, 512, 2048, 8192, },
  { 2, 4, 16, 64, 512, 2048, 8192, },
  { 1, 2, 4, 16, 64, 512, 2048, 8192, },
  { 8, 16, 64, 512, 2048, 8192, },
  { 1, 8, 16, 64, 512, 2048, 8192, },
  { 2, 8, 16, 64, 512, 2048, 8192, },
  { 1, 2, 8, 16, 64, 512, 2048, 8192, },
  { 4, 8, 16, 64, 512, 2048, 8192, },
  { 1, 4, 8, 16, 64, 512, 2048, 8192, },
  { 2, 4, 8, 16, 64, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 8192, },
  { 32, 64, 512, 2048, 8192, },
  { 1, 32, 64, 512, 2048, 8192, },
  { 2, 32, 64, 512, 2048, 8192, },
  { 1, 2, 32, 64, 512, 2048, 8192, },
  { 4, 32, 64, 512, 2048, 8192, },
  { 1, 4, 32, 64, 512, 2048, 8192, },
  { 2, 4, 32, 64, 512, 2048, 8192, },
  { 1, 2, 4, 32, 64, 512, 2048, 8192, },
  { 8, 32, 64, 512, 2048, 8192, },
  { 1, 8, 32, 64, 512, 2048, 8192, },
  { 2, 8, 32, 64, 512, 2048, 8192, },
  { 1, 2, 8, 32, 64, 512, 2048, 8192, },
  { 4, 8, 32, 64, 512, 2048, 8192, },
  { 1, 4, 8, 32, 64, 512, 2048, 8192, },
  { 2, 4, 8, 32, 64, 512, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 8192, },
  { 16, 32, 64, 512, 2048, 8192, },
  { 1, 16, 32, 64, 512, 2048, 8192, },
  { 2, 16, 32, 64, 512, 2048, 8192, },
  { 1, 2, 16, 32, 64, 512, 2048, 8192, },
  { 4, 16, 32, 64, 512, 2048, 8192, },
  { 1, 4, 16, 32, 64, 512, 2048, 8192, },
  { 2, 4, 16, 32, 64, 512, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 8192, },
  { 8, 16, 32, 64, 512, 2048, 8192, },
  { 1, 8, 16, 32, 64, 512, 2048, 8192, },
  { 2, 8, 16, 32, 64, 512, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 8192, },
  { 4, 8, 16, 32, 64, 512, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 8192, },
  { 128, 512, 2048, 8192, },
  { 1, 128, 512, 2048, 8192, },
  { 2, 128, 512, 2048, 8192, },
  { 1, 2, 128, 512, 2048, 8192, },
  { 4, 128, 512, 2048, 8192, },
  { 1, 4, 128, 512, 2048, 8192, },
  { 2, 4, 128, 512, 2048, 8192, },
  { 1, 2, 4, 128, 512, 2048, 8192, },
  { 8, 128, 512, 2048, 8192, },
  { 1, 8, 128, 512, 2048, 8192, },
  { 2, 8, 128, 512, 2048, 8192, },
  { 1, 2, 8, 128, 512, 2048, 8192, },
  { 4, 8, 128, 512, 2048, 8192, },
  { 1, 4, 8, 128, 512, 2048, 8192, },
  { 2, 4, 8, 128, 512, 2048, 8192, },
  { 1, 2, 4, 8, 128, 512, 2048, 8192, },
  { 16, 128, 512, 2048, 8192, },
  { 1, 16, 128, 512, 2048, 8192, },
  { 2, 16, 128, 512, 2048, 8192, },
  { 1, 2, 16, 128, 512, 2048, 8192, },
  { 4, 16, 128, 512, 2048, 8192, },
  { 1, 4, 16, 128, 512, 2048, 8192, },
  { 2, 4, 16, 128, 512, 2048, 8192, },
  { 1, 2, 4, 16, 128, 512, 2048, 8192, },
  { 8, 16, 128, 512, 2048, 8192, },
  { 1, 8, 16, 128, 512, 2048, 8192, },
  { 2, 8, 16, 128, 512, 2048, 8192, },
  { 1, 2, 8, 16, 128, 512, 2048, 8192, },
  { 4, 8, 16, 128, 512, 2048, 8192, },
  { 1, 4, 8, 16, 128, 512, 2048, 8192, },
  { 2, 4, 8, 16, 128, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 8192, },
  { 32, 128, 512, 2048, 8192, },
  { 1, 32, 128, 512, 2048, 8192, },
  { 2, 32, 128, 512, 2048, 8192, },
  { 1, 2, 32, 128, 512, 2048, 8192, },
  { 4, 32, 128, 512, 2048, 8192, },
  { 1, 4, 32, 128, 512, 2048, 8192, },
  { 2, 4, 32, 128, 512, 2048, 8192, },
  { 1, 2, 4, 32, 128, 512, 2048, 8192, },
  { 8, 32, 128, 512, 2048, 8192, },
  { 1, 8, 32, 128, 512, 2048, 8192, },
  { 2, 8, 32, 128, 512, 2048, 8192, },
  { 1, 2, 8, 32, 128, 512, 2048, 8192, },
  { 4, 8, 32, 128, 512, 2048, 8192, },
  { 1, 4, 8, 32, 128, 512, 2048, 8192, },
  { 2, 4, 8, 32, 128, 512, 2048, 8192, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 8192, },
  { 16, 32, 128, 512, 2048, 8192, },
  { 1, 16, 32, 128, 512, 2048, 8192, },
  { 2, 16, 32, 128, 512, 2048, 8192, },
  { 1, 2, 16, 32, 128, 512, 2048, 8192, },
  { 4, 16, 32, 128, 512, 2048, 8192, },
  { 1, 4, 16, 32, 128, 512, 2048, 8192, },
  { 2, 4, 16, 32, 128, 512, 2048, 8192, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 8192, },
  { 8, 16, 32, 128, 512, 2048, 8192, },
  { 1, 8, 16, 32, 128, 512, 2048, 8192, },
  { 2, 8, 16, 32, 128, 512, 2048, 8192, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 8192, },
  { 4, 8, 16, 32, 128, 512, 2048, 8192, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 8192, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 8192, },
  { 64, 128, 512, 2048, 8192, },
  { 1, 64, 128, 512, 2048, 8192, },
  { 2, 64, 128, 512, 2048, 8192, },
  { 1, 2, 64, 128, 512, 2048, 8192, },
  { 4, 64, 128, 512, 2048, 8192, },
  { 1, 4, 64, 128, 512, 2048, 8192, },
  { 2, 4, 64, 128, 512, 2048, 8192, },
  { 1, 2, 4, 64, 128, 512, 2048, 8192, },
  { 8, 64, 128, 512, 2048, 8192, },
  { 1, 8, 64, 128, 512, 2048, 8192, },
  { 2, 8, 64, 128, 512, 2048, 8192, },
  { 1, 2, 8, 64, 128, 512, 2048, 8192, },
  { 4, 8, 64, 128, 512, 2048, 8192, },
  { 1, 4, 8, 64, 128, 512, 2048, 8192, },
  { 2, 4, 8, 64, 128, 512, 2048, 8192, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 8192, },
  { 16, 64, 128, 512, 2048, 8192, },
  { 1, 16, 64, 128, 512, 2048, 8192, },
  { 2, 16, 64, 128, 512, 2048, 8192, },
  { 1, 2, 16, 64, 128, 512, 2048, 8192, },
  { 4, 16, 64, 128, 512, 2048, 8192, },
  { 1, 4, 16, 64, 128, 512, 2048, 8192, },
  { 2, 4, 16, 64, 128, 512, 2048, 8192, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 8192, },
  { 8, 16, 64, 128, 512, 2048, 8192, },
  { 1, 8, 16, 64, 128, 512, 2048, 8192, },
  { 2, 8, 16, 64, 128, 512, 2048, 8192, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 8192, },
  { 4, 8, 16, 64, 128, 512, 2048, 8192, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 8192, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 8192, },
  { 32, 64, 128, 512, 2048, 8192, },
  { 1, 32, 64, 128, 512, 2048, 8192, },
  { 2, 32, 64, 128, 512, 2048, 8192, },
  { 1, 2, 32, 64, 128, 512, 2048, 8192, },
  { 4, 32, 64, 128, 512, 2048, 8192, },
  { 1, 4, 32, 64, 128, 512, 2048, 8192, },
  { 2, 4, 32, 64, 128, 512, 2048, 8192, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 8192, },
  { 8, 32, 64, 128, 512, 2048, 8192, },
  { 1, 8, 32, 64, 128, 512, 2048, 8192, },
  { 2, 8, 32, 64, 128, 512, 2048, 8192, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 8192, },
  { 4, 8, 32, 64, 128, 512, 2048, 8192, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 8192, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 8192, },
  { 16, 32, 64, 128, 512, 2048, 8192, },
  { 1, 16, 32, 64, 128, 512, 2048, 8192, },
  { 2, 16, 32, 64, 128, 512, 2048, 8192, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 8192, },
  { 4, 16, 32, 64, 128, 512, 2048, 8192, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 8192, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 8192, },
  { 8, 16, 32, 64, 128, 512, 2048, 8192, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 8192, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 8192, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, },
  { 256, 512, 2048, 8192, },
  { 1, 256, 512, 2048, 8192, },
  { 2, 256, 512, 2048, 8192, },
  { 1, 2, 256, 512, 2048, 8192, },
  { 4, 256, 512, 2048, 8192, },
  { 1, 4, 256, 512, 2048, 8192, },
  { 2, 4, 256, 512, 2048, 8192, },
  { 1, 2, 4, 256, 512, 2048, 8192, },
  { 8, 256, 512, 2048, 8192, },
  { 1, 8, 256, 512, 2048, 8192, },
  { 2, 8, 256, 512, 2048, 8192, },
  { 1, 2, 8, 256, 512, 2048, 8192, },
  { 4, 8, 256, 512, 2048, 8192, },
  { 1, 4, 8, 256, 512, 2048, 8192, },
  { 2, 4, 8, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 256, 512, 2048, 8192, },
  { 16, 256, 512, 2048, 8192, },
  { 1, 16, 256, 512, 2048, 8192, },
  { 2, 16, 256, 512, 2048, 8192, },
  { 1, 2, 16, 256, 512, 2048, 8192, },
  { 4, 16, 256, 512, 2048, 8192, },
  { 1, 4, 16, 256, 512, 2048, 8192, },
  { 2, 4, 16, 256, 512, 2048, 8192, },
  { 1, 2, 4, 16, 256, 512, 2048, 8192, },
  { 8, 16, 256, 512, 2048, 8192, },
  { 1, 8, 16, 256, 512, 2048, 8192, },
  { 2, 8, 16, 256, 512, 2048, 8192, },
  { 1, 2, 8, 16, 256, 512, 2048, 8192, },
  { 4, 8, 16, 256, 512, 2048, 8192, },
  { 1, 4, 8, 16, 256, 512, 2048, 8192, },
  { 2, 4, 8, 16, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 8192, },
  { 32, 256, 512, 2048, 8192, },
  { 1, 32, 256, 512, 2048, 8192, },
  { 2, 32, 256, 512, 2048, 8192, },
  { 1, 2, 32, 256, 512, 2048, 8192, },
  { 4, 32, 256, 512, 2048, 8192, },
  { 1, 4, 32, 256, 512, 2048, 8192, },
  { 2, 4, 32, 256, 512, 2048, 8192, },
  { 1, 2, 4, 32, 256, 512, 2048, 8192, },
  { 8, 32, 256, 512, 2048, 8192, },
  { 1, 8, 32, 256, 512, 2048, 8192, },
  { 2, 8, 32, 256, 512, 2048, 8192, },
  { 1, 2, 8, 32, 256, 512, 2048, 8192, },
  { 4, 8, 32, 256, 512, 2048, 8192, },
  { 1, 4, 8, 32, 256, 512, 2048, 8192, },
  { 2, 4, 8, 32, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 8192, },
  { 16, 32, 256, 512, 2048, 8192, },
  { 1, 16, 32, 256, 512, 2048, 8192, },
  { 2, 16, 32, 256, 512, 2048, 8192, },
  { 1, 2, 16, 32, 256, 512, 2048, 8192, },
  { 4, 16, 32, 256, 512, 2048, 8192, },
  { 1, 4, 16, 32, 256, 512, 2048, 8192, },
  { 2, 4, 16, 32, 256, 512, 2048, 8192, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 8192, },
  { 8, 16, 32, 256, 512, 2048, 8192, },
  { 1, 8, 16, 32, 256, 512, 2048, 8192, },
  { 2, 8, 16, 32, 256, 512, 2048, 8192, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 8192, },
  { 4, 8, 16, 32, 256, 512, 2048, 8192, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 8192, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 8192, },
  { 64, 256, 512, 2048, 8192, },
  { 1, 64, 256, 512, 2048, 8192, },
  { 2, 64, 256, 512, 2048, 8192, },
  { 1, 2, 64, 256, 512, 2048, 8192, },
  { 4, 64, 256, 512, 2048, 8192, },
  { 1, 4, 64, 256, 512, 2048, 8192, },
  { 2, 4, 64, 256, 512, 2048, 8192, },
  { 1, 2, 4, 64, 256, 512, 2048, 8192, },
  { 8, 64, 256, 512, 2048, 8192, },
  { 1, 8, 64, 256, 512, 2048, 8192, },
  { 2, 8, 64, 256, 512, 2048, 8192, },
  { 1, 2, 8, 64, 256, 512, 2048, 8192, },
  { 4, 8, 64, 256, 512, 2048, 8192, },
  { 1, 4, 8, 64, 256, 512, 2048, 8192, },
  { 2, 4, 8, 64, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 8192, },
  { 16, 64, 256, 512, 2048, 8192, },
  { 1, 16, 64, 256, 512, 2048, 8192, },
  { 2, 16, 64, 256, 512, 2048, 8192, },
  { 1, 2, 16, 64, 256, 512, 2048, 8192, },
  { 4, 16, 64, 256, 512, 2048, 8192, },
  { 1, 4, 16, 64, 256, 512, 2048, 8192, },
  { 2, 4, 16, 64, 256, 512, 2048, 8192, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 8192, },
  { 8, 16, 64, 256, 512, 2048, 8192, },
  { 1, 8, 16, 64, 256, 512, 2048, 8192, },
  { 2, 8, 16, 64, 256, 512, 2048, 8192, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 8192, },
  { 4, 8, 16, 64, 256, 512, 2048, 8192, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 8192, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 8192, },
  { 32, 64, 256, 512, 2048, 8192, },
  { 1, 32, 64, 256, 512, 2048, 8192, },
  { 2, 32, 64, 256, 512, 2048, 8192, },
  { 1, 2, 32, 64, 256, 512, 2048, 8192, },
  { 4, 32, 64, 256, 512, 2048, 8192, },
  { 1, 4, 32, 64, 256, 512, 2048, 8192, },
  { 2, 4, 32, 64, 256, 512, 2048, 8192, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 8192, },
  { 8, 32, 64, 256, 512, 2048, 8192, },
  { 1, 8, 32, 64, 256, 512, 2048, 8192, },
  { 2, 8, 32, 64, 256, 512, 2048, 8192, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 8192, },
  { 4, 8, 32, 64, 256, 512, 2048, 8192, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 8192, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 8192, },
  { 16, 32, 64, 256, 512, 2048, 8192, },
  { 1, 16, 32, 64, 256, 512, 2048, 8192, },
  { 2, 16, 32, 64, 256, 512, 2048, 8192, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 8192, },
  { 4, 16, 32, 64, 256, 512, 2048, 8192, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 8192, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 8192, },
  { 8, 16, 32, 64, 256, 512, 2048, 8192, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 8192, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 8192, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, },
  { 128, 256, 512, 2048, 8192, },
  { 1, 128, 256, 512, 2048, 8192, },
  { 2, 128, 256, 512, 2048, 8192, },
  { 1, 2, 128, 256, 512, 2048, 8192, },
  { 4, 128, 256, 512, 2048, 8192, },
  { 1, 4, 128, 256, 512, 2048, 8192, },
  { 2, 4, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 128, 256, 512, 2048, 8192, },
  { 8, 128, 256, 512, 2048, 8192, },
  { 1, 8, 128, 256, 512, 2048, 8192, },
  { 2, 8, 128, 256, 512, 2048, 8192, },
  { 1, 2, 8, 128, 256, 512, 2048, 8192, },
  { 4, 8, 128, 256, 512, 2048, 8192, },
  { 1, 4, 8, 128, 256, 512, 2048, 8192, },
  { 2, 4, 8, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 8192, },
  { 16, 128, 256, 512, 2048, 8192, },
  { 1, 16, 128, 256, 512, 2048, 8192, },
  { 2, 16, 128, 256, 512, 2048, 8192, },
  { 1, 2, 16, 128, 256, 512, 2048, 8192, },
  { 4, 16, 128, 256, 512, 2048, 8192, },
  { 1, 4, 16, 128, 256, 512, 2048, 8192, },
  { 2, 4, 16, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 8192, },
  { 8, 16, 128, 256, 512, 2048, 8192, },
  { 1, 8, 16, 128, 256, 512, 2048, 8192, },
  { 2, 8, 16, 128, 256, 512, 2048, 8192, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 8192, },
  { 4, 8, 16, 128, 256, 512, 2048, 8192, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 8192, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 8192, },
  { 32, 128, 256, 512, 2048, 8192, },
  { 1, 32, 128, 256, 512, 2048, 8192, },
  { 2, 32, 128, 256, 512, 2048, 8192, },
  { 1, 2, 32, 128, 256, 512, 2048, 8192, },
  { 4, 32, 128, 256, 512, 2048, 8192, },
  { 1, 4, 32, 128, 256, 512, 2048, 8192, },
  { 2, 4, 32, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 8192, },
  { 8, 32, 128, 256, 512, 2048, 8192, },
  { 1, 8, 32, 128, 256, 512, 2048, 8192, },
  { 2, 8, 32, 128, 256, 512, 2048, 8192, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 8192, },
  { 4, 8, 32, 128, 256, 512, 2048, 8192, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 8192, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 8192, },
  { 16, 32, 128, 256, 512, 2048, 8192, },
  { 1, 16, 32, 128, 256, 512, 2048, 8192, },
  { 2, 16, 32, 128, 256, 512, 2048, 8192, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 8192, },
  { 4, 16, 32, 128, 256, 512, 2048, 8192, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 8192, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 8192, },
  { 8, 16, 32, 128, 256, 512, 2048, 8192, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 8192, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 8192, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, },
  { 64, 128, 256, 512, 2048, 8192, },
  { 1, 64, 128, 256, 512, 2048, 8192, },
  { 2, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 64, 128, 256, 512, 2048, 8192, },
  { 4, 64, 128, 256, 512, 2048, 8192, },
  { 1, 4, 64, 128, 256, 512, 2048, 8192, },
  { 2, 4, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 8192, },
  { 8, 64, 128, 256, 512, 2048, 8192, },
  { 1, 8, 64, 128, 256, 512, 2048, 8192, },
  { 2, 8, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 8192, },
  { 4, 8, 64, 128, 256, 512, 2048, 8192, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 8192, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 8192, },
  { 16, 64, 128, 256, 512, 2048, 8192, },
  { 1, 16, 64, 128, 256, 512, 2048, 8192, },
  { 2, 16, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 8192, },
  { 4, 16, 64, 128, 256, 512, 2048, 8192, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 8192, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 8192, },
  { 8, 16, 64, 128, 256, 512, 2048, 8192, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 8192, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 8192, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, },
  { 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 32, 64, 128, 256, 512, 2048, 8192, },
  { 2, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 8192, },
  { 4, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 8192, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 8192, },
  { 8, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 8192, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 8192, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, },
  { 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, },
  { 1024, 2048, 8192, },
  { 1, 1024, 2048, 8192, },
  { 2, 1024, 2048, 8192, },
  { 1, 2, 1024, 2048, 8192, },
  { 4, 1024, 2048, 8192, },
  { 1, 4, 1024, 2048, 8192, },
  { 2, 4, 1024, 2048, 8192, },
  { 1, 2, 4, 1024, 2048, 8192, },
  { 8, 1024, 2048, 8192, },
  { 1, 8, 1024, 2048, 8192, },
  { 2, 8, 1024, 2048, 8192, },
  { 1, 2, 8, 1024, 2048, 8192, },
  { 4, 8, 1024, 2048, 8192, },
  { 1, 4, 8, 1024, 2048, 8192, },
  { 2, 4, 8, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 1024, 2048, 8192, },
  { 16, 1024, 2048, 8192, },
  { 1, 16, 1024, 2048, 8192, },
  { 2, 16, 1024, 2048, 8192, },
  { 1, 2, 16, 1024, 2048, 8192, },
  { 4, 16, 1024, 2048, 8192, },
  { 1, 4, 16, 1024, 2048, 8192, },
  { 2, 4, 16, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 1024, 2048, 8192, },
  { 8, 16, 1024, 2048, 8192, },
  { 1, 8, 16, 1024, 2048, 8192, },
  { 2, 8, 16, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 1024, 2048, 8192, },
  { 4, 8, 16, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 1024, 2048, 8192, },
  { 32, 1024, 2048, 8192, },
  { 1, 32, 1024, 2048, 8192, },
  { 2, 32, 1024, 2048, 8192, },
  { 1, 2, 32, 1024, 2048, 8192, },
  { 4, 32, 1024, 2048, 8192, },
  { 1, 4, 32, 1024, 2048, 8192, },
  { 2, 4, 32, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 1024, 2048, 8192, },
  { 8, 32, 1024, 2048, 8192, },
  { 1, 8, 32, 1024, 2048, 8192, },
  { 2, 8, 32, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 1024, 2048, 8192, },
  { 4, 8, 32, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 1024, 2048, 8192, },
  { 16, 32, 1024, 2048, 8192, },
  { 1, 16, 32, 1024, 2048, 8192, },
  { 2, 16, 32, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 1024, 2048, 8192, },
  { 4, 16, 32, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 1024, 2048, 8192, },
  { 8, 16, 32, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 8192, },
  { 64, 1024, 2048, 8192, },
  { 1, 64, 1024, 2048, 8192, },
  { 2, 64, 1024, 2048, 8192, },
  { 1, 2, 64, 1024, 2048, 8192, },
  { 4, 64, 1024, 2048, 8192, },
  { 1, 4, 64, 1024, 2048, 8192, },
  { 2, 4, 64, 1024, 2048, 8192, },
  { 1, 2, 4, 64, 1024, 2048, 8192, },
  { 8, 64, 1024, 2048, 8192, },
  { 1, 8, 64, 1024, 2048, 8192, },
  { 2, 8, 64, 1024, 2048, 8192, },
  { 1, 2, 8, 64, 1024, 2048, 8192, },
  { 4, 8, 64, 1024, 2048, 8192, },
  { 1, 4, 8, 64, 1024, 2048, 8192, },
  { 2, 4, 8, 64, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 64, 1024, 2048, 8192, },
  { 16, 64, 1024, 2048, 8192, },
  { 1, 16, 64, 1024, 2048, 8192, },
  { 2, 16, 64, 1024, 2048, 8192, },
  { 1, 2, 16, 64, 1024, 2048, 8192, },
  { 4, 16, 64, 1024, 2048, 8192, },
  { 1, 4, 16, 64, 1024, 2048, 8192, },
  { 2, 4, 16, 64, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 64, 1024, 2048, 8192, },
  { 8, 16, 64, 1024, 2048, 8192, },
  { 1, 8, 16, 64, 1024, 2048, 8192, },
  { 2, 8, 16, 64, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 64, 1024, 2048, 8192, },
  { 4, 8, 16, 64, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 64, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 64, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 8192, },
  { 32, 64, 1024, 2048, 8192, },
  { 1, 32, 64, 1024, 2048, 8192, },
  { 2, 32, 64, 1024, 2048, 8192, },
  { 1, 2, 32, 64, 1024, 2048, 8192, },
  { 4, 32, 64, 1024, 2048, 8192, },
  { 1, 4, 32, 64, 1024, 2048, 8192, },
  { 2, 4, 32, 64, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 64, 1024, 2048, 8192, },
  { 8, 32, 64, 1024, 2048, 8192, },
  { 1, 8, 32, 64, 1024, 2048, 8192, },
  { 2, 8, 32, 64, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 64, 1024, 2048, 8192, },
  { 4, 8, 32, 64, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 64, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 64, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 8192, },
  { 16, 32, 64, 1024, 2048, 8192, },
  { 1, 16, 32, 64, 1024, 2048, 8192, },
  { 2, 16, 32, 64, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 64, 1024, 2048, 8192, },
  { 4, 16, 32, 64, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 64, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 64, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 8192, },
  { 8, 16, 32, 64, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 64, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 64, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 64, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 8192, },
  { 128, 1024, 2048, 8192, },
  { 1, 128, 1024, 2048, 8192, },
  { 2, 128, 1024, 2048, 8192, },
  { 1, 2, 128, 1024, 2048, 8192, },
  { 4, 128, 1024, 2048, 8192, },
  { 1, 4, 128, 1024, 2048, 8192, },
  { 2, 4, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 128, 1024, 2048, 8192, },
  { 8, 128, 1024, 2048, 8192, },
  { 1, 8, 128, 1024, 2048, 8192, },
  { 2, 8, 128, 1024, 2048, 8192, },
  { 1, 2, 8, 128, 1024, 2048, 8192, },
  { 4, 8, 128, 1024, 2048, 8192, },
  { 1, 4, 8, 128, 1024, 2048, 8192, },
  { 2, 4, 8, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 128, 1024, 2048, 8192, },
  { 16, 128, 1024, 2048, 8192, },
  { 1, 16, 128, 1024, 2048, 8192, },
  { 2, 16, 128, 1024, 2048, 8192, },
  { 1, 2, 16, 128, 1024, 2048, 8192, },
  { 4, 16, 128, 1024, 2048, 8192, },
  { 1, 4, 16, 128, 1024, 2048, 8192, },
  { 2, 4, 16, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 128, 1024, 2048, 8192, },
  { 8, 16, 128, 1024, 2048, 8192, },
  { 1, 8, 16, 128, 1024, 2048, 8192, },
  { 2, 8, 16, 128, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 128, 1024, 2048, 8192, },
  { 4, 8, 16, 128, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 128, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 8192, },
  { 32, 128, 1024, 2048, 8192, },
  { 1, 32, 128, 1024, 2048, 8192, },
  { 2, 32, 128, 1024, 2048, 8192, },
  { 1, 2, 32, 128, 1024, 2048, 8192, },
  { 4, 32, 128, 1024, 2048, 8192, },
  { 1, 4, 32, 128, 1024, 2048, 8192, },
  { 2, 4, 32, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 128, 1024, 2048, 8192, },
  { 8, 32, 128, 1024, 2048, 8192, },
  { 1, 8, 32, 128, 1024, 2048, 8192, },
  { 2, 8, 32, 128, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 128, 1024, 2048, 8192, },
  { 4, 8, 32, 128, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 128, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 8192, },
  { 16, 32, 128, 1024, 2048, 8192, },
  { 1, 16, 32, 128, 1024, 2048, 8192, },
  { 2, 16, 32, 128, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 128, 1024, 2048, 8192, },
  { 4, 16, 32, 128, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 128, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 8192, },
  { 8, 16, 32, 128, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 128, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 128, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 128, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 8192, },
  { 64, 128, 1024, 2048, 8192, },
  { 1, 64, 128, 1024, 2048, 8192, },
  { 2, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 64, 128, 1024, 2048, 8192, },
  { 4, 64, 128, 1024, 2048, 8192, },
  { 1, 4, 64, 128, 1024, 2048, 8192, },
  { 2, 4, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 64, 128, 1024, 2048, 8192, },
  { 8, 64, 128, 1024, 2048, 8192, },
  { 1, 8, 64, 128, 1024, 2048, 8192, },
  { 2, 8, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 8, 64, 128, 1024, 2048, 8192, },
  { 4, 8, 64, 128, 1024, 2048, 8192, },
  { 1, 4, 8, 64, 128, 1024, 2048, 8192, },
  { 2, 4, 8, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 8192, },
  { 16, 64, 128, 1024, 2048, 8192, },
  { 1, 16, 64, 128, 1024, 2048, 8192, },
  { 2, 16, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 16, 64, 128, 1024, 2048, 8192, },
  { 4, 16, 64, 128, 1024, 2048, 8192, },
  { 1, 4, 16, 64, 128, 1024, 2048, 8192, },
  { 2, 4, 16, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 8192, },
  { 8, 16, 64, 128, 1024, 2048, 8192, },
  { 1, 8, 16, 64, 128, 1024, 2048, 8192, },
  { 2, 8, 16, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 8192, },
  { 4, 8, 16, 64, 128, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 8192, },
  { 32, 64, 128, 1024, 2048, 8192, },
  { 1, 32, 64, 128, 1024, 2048, 8192, },
  { 2, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 32, 64, 128, 1024, 2048, 8192, },
  { 4, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 4, 32, 64, 128, 1024, 2048, 8192, },
  { 2, 4, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 8192, },
  { 8, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 8, 32, 64, 128, 1024, 2048, 8192, },
  { 2, 8, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 8192, },
  { 4, 8, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 8192, },
  { 16, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 2, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 4, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 8, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, },
  { 256, 1024, 2048, 8192, },
  { 1, 256, 1024, 2048, 8192, },
  { 2, 256, 1024, 2048, 8192, },
  { 1, 2, 256, 1024, 2048, 8192, },
  { 4, 256, 1024, 2048, 8192, },
  { 1, 4, 256, 1024, 2048, 8192, },
  { 2, 4, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 256, 1024, 2048, 8192, },
  { 8, 256, 1024, 2048, 8192, },
  { 1, 8, 256, 1024, 2048, 8192, },
  { 2, 8, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 256, 1024, 2048, 8192, },
  { 4, 8, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 256, 1024, 2048, 8192, },
  { 16, 256, 1024, 2048, 8192, },
  { 1, 16, 256, 1024, 2048, 8192, },
  { 2, 16, 256, 1024, 2048, 8192, },
  { 1, 2, 16, 256, 1024, 2048, 8192, },
  { 4, 16, 256, 1024, 2048, 8192, },
  { 1, 4, 16, 256, 1024, 2048, 8192, },
  { 2, 4, 16, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 256, 1024, 2048, 8192, },
  { 8, 16, 256, 1024, 2048, 8192, },
  { 1, 8, 16, 256, 1024, 2048, 8192, },
  { 2, 8, 16, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 256, 1024, 2048, 8192, },
  { 4, 8, 16, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 8192, },
  { 32, 256, 1024, 2048, 8192, },
  { 1, 32, 256, 1024, 2048, 8192, },
  { 2, 32, 256, 1024, 2048, 8192, },
  { 1, 2, 32, 256, 1024, 2048, 8192, },
  { 4, 32, 256, 1024, 2048, 8192, },
  { 1, 4, 32, 256, 1024, 2048, 8192, },
  { 2, 4, 32, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 256, 1024, 2048, 8192, },
  { 8, 32, 256, 1024, 2048, 8192, },
  { 1, 8, 32, 256, 1024, 2048, 8192, },
  { 2, 8, 32, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 256, 1024, 2048, 8192, },
  { 4, 8, 32, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 8192, },
  { 16, 32, 256, 1024, 2048, 8192, },
  { 1, 16, 32, 256, 1024, 2048, 8192, },
  { 2, 16, 32, 256, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 256, 1024, 2048, 8192, },
  { 4, 16, 32, 256, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 256, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 8192, },
  { 8, 16, 32, 256, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 256, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 8192, },
  { 64, 256, 1024, 2048, 8192, },
  { 1, 64, 256, 1024, 2048, 8192, },
  { 2, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 64, 256, 1024, 2048, 8192, },
  { 4, 64, 256, 1024, 2048, 8192, },
  { 1, 4, 64, 256, 1024, 2048, 8192, },
  { 2, 4, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 64, 256, 1024, 2048, 8192, },
  { 8, 64, 256, 1024, 2048, 8192, },
  { 1, 8, 64, 256, 1024, 2048, 8192, },
  { 2, 8, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 64, 256, 1024, 2048, 8192, },
  { 4, 8, 64, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 64, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 8192, },
  { 16, 64, 256, 1024, 2048, 8192, },
  { 1, 16, 64, 256, 1024, 2048, 8192, },
  { 2, 16, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 16, 64, 256, 1024, 2048, 8192, },
  { 4, 16, 64, 256, 1024, 2048, 8192, },
  { 1, 4, 16, 64, 256, 1024, 2048, 8192, },
  { 2, 4, 16, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 8192, },
  { 8, 16, 64, 256, 1024, 2048, 8192, },
  { 1, 8, 16, 64, 256, 1024, 2048, 8192, },
  { 2, 8, 16, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 8192, },
  { 4, 8, 16, 64, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 8192, },
  { 32, 64, 256, 1024, 2048, 8192, },
  { 1, 32, 64, 256, 1024, 2048, 8192, },
  { 2, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 32, 64, 256, 1024, 2048, 8192, },
  { 4, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 4, 32, 64, 256, 1024, 2048, 8192, },
  { 2, 4, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 8192, },
  { 8, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 8, 32, 64, 256, 1024, 2048, 8192, },
  { 2, 8, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 8192, },
  { 4, 8, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 8192, },
  { 16, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 2, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 4, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 8, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, },
  { 128, 256, 1024, 2048, 8192, },
  { 1, 128, 256, 1024, 2048, 8192, },
  { 2, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 128, 256, 1024, 2048, 8192, },
  { 4, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 128, 256, 1024, 2048, 8192, },
  { 8, 128, 256, 1024, 2048, 8192, },
  { 1, 8, 128, 256, 1024, 2048, 8192, },
  { 2, 8, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 128, 256, 1024, 2048, 8192, },
  { 4, 8, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 8192, },
  { 16, 128, 256, 1024, 2048, 8192, },
  { 1, 16, 128, 256, 1024, 2048, 8192, },
  { 2, 16, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 16, 128, 256, 1024, 2048, 8192, },
  { 4, 16, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 16, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 16, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 8192, },
  { 8, 16, 128, 256, 1024, 2048, 8192, },
  { 1, 8, 16, 128, 256, 1024, 2048, 8192, },
  { 2, 8, 16, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 8192, },
  { 4, 8, 16, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 8192, },
  { 32, 128, 256, 1024, 2048, 8192, },
  { 1, 32, 128, 256, 1024, 2048, 8192, },
  { 2, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 32, 128, 256, 1024, 2048, 8192, },
  { 4, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 32, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 8192, },
  { 8, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 8, 32, 128, 256, 1024, 2048, 8192, },
  { 2, 8, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 8192, },
  { 4, 8, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 8192, },
  { 16, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 2, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 4, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 8, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, },
  { 64, 128, 256, 1024, 2048, 8192, },
  { 1, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 64, 128, 256, 1024, 2048, 8192, },
  { 4, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 8192, },
  { 8, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 8, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 8, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 8192, },
  { 4, 8, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 8192, },
  { 16, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 4, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 8, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, },
  { 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 4, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 8, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, },
  { 512, 1024, 2048, 8192, },
  { 1, 512, 1024, 2048, 8192, },
  { 2, 512, 1024, 2048, 8192, },
  { 1, 2, 512, 1024, 2048, 8192, },
  { 4, 512, 1024, 2048, 8192, },
  { 1, 4, 512, 1024, 2048, 8192, },
  { 2, 4, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 512, 1024, 2048, 8192, },
  { 8, 512, 1024, 2048, 8192, },
  { 1, 8, 512, 1024, 2048, 8192, },
  { 2, 8, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 512, 1024, 2048, 8192, },
  { 4, 8, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 512, 1024, 2048, 8192, },
  { 16, 512, 1024, 2048, 8192, },
  { 1, 16, 512, 1024, 2048, 8192, },
  { 2, 16, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 512, 1024, 2048, 8192, },
  { 4, 16, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 512, 1024, 2048, 8192, },
  { 8, 16, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 8192, },
  { 32, 512, 1024, 2048, 8192, },
  { 1, 32, 512, 1024, 2048, 8192, },
  { 2, 32, 512, 1024, 2048, 8192, },
  { 1, 2, 32, 512, 1024, 2048, 8192, },
  { 4, 32, 512, 1024, 2048, 8192, },
  { 1, 4, 32, 512, 1024, 2048, 8192, },
  { 2, 4, 32, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 512, 1024, 2048, 8192, },
  { 8, 32, 512, 1024, 2048, 8192, },
  { 1, 8, 32, 512, 1024, 2048, 8192, },
  { 2, 8, 32, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 512, 1024, 2048, 8192, },
  { 4, 8, 32, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 8192, },
  { 16, 32, 512, 1024, 2048, 8192, },
  { 1, 16, 32, 512, 1024, 2048, 8192, },
  { 2, 16, 32, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 512, 1024, 2048, 8192, },
  { 4, 16, 32, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 8192, },
  { 8, 16, 32, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 8192, },
  { 64, 512, 1024, 2048, 8192, },
  { 1, 64, 512, 1024, 2048, 8192, },
  { 2, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 64, 512, 1024, 2048, 8192, },
  { 4, 64, 512, 1024, 2048, 8192, },
  { 1, 4, 64, 512, 1024, 2048, 8192, },
  { 2, 4, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 64, 512, 1024, 2048, 8192, },
  { 8, 64, 512, 1024, 2048, 8192, },
  { 1, 8, 64, 512, 1024, 2048, 8192, },
  { 2, 8, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 64, 512, 1024, 2048, 8192, },
  { 4, 8, 64, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 64, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 8192, },
  { 16, 64, 512, 1024, 2048, 8192, },
  { 1, 16, 64, 512, 1024, 2048, 8192, },
  { 2, 16, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 64, 512, 1024, 2048, 8192, },
  { 4, 16, 64, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 64, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 8192, },
  { 8, 16, 64, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 64, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 64, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 8192, },
  { 32, 64, 512, 1024, 2048, 8192, },
  { 1, 32, 64, 512, 1024, 2048, 8192, },
  { 2, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 32, 64, 512, 1024, 2048, 8192, },
  { 4, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 4, 32, 64, 512, 1024, 2048, 8192, },
  { 2, 4, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 8192, },
  { 8, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 8, 32, 64, 512, 1024, 2048, 8192, },
  { 2, 8, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 8192, },
  { 4, 8, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 8192, },
  { 16, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 2, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 4, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 8, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, },
  { 128, 512, 1024, 2048, 8192, },
  { 1, 128, 512, 1024, 2048, 8192, },
  { 2, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 128, 512, 1024, 2048, 8192, },
  { 4, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 128, 512, 1024, 2048, 8192, },
  { 8, 128, 512, 1024, 2048, 8192, },
  { 1, 8, 128, 512, 1024, 2048, 8192, },
  { 2, 8, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 128, 512, 1024, 2048, 8192, },
  { 4, 8, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 8192, },
  { 16, 128, 512, 1024, 2048, 8192, },
  { 1, 16, 128, 512, 1024, 2048, 8192, },
  { 2, 16, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 128, 512, 1024, 2048, 8192, },
  { 4, 16, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 8192, },
  { 8, 16, 128, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 128, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 8192, },
  { 32, 128, 512, 1024, 2048, 8192, },
  { 1, 32, 128, 512, 1024, 2048, 8192, },
  { 2, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 32, 128, 512, 1024, 2048, 8192, },
  { 4, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 32, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 8192, },
  { 8, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 8, 32, 128, 512, 1024, 2048, 8192, },
  { 2, 8, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 8192, },
  { 4, 8, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 8192, },
  { 16, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 2, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 4, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 8, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, },
  { 64, 128, 512, 1024, 2048, 8192, },
  { 1, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 64, 128, 512, 1024, 2048, 8192, },
  { 4, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 8192, },
  { 8, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 8, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 8, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 8192, },
  { 4, 8, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 8192, },
  { 16, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 4, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 8, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, },
  { 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 4, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 8, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, },
  { 256, 512, 1024, 2048, 8192, },
  { 1, 256, 512, 1024, 2048, 8192, },
  { 2, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 256, 512, 1024, 2048, 8192, },
  { 4, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 256, 512, 1024, 2048, 8192, },
  { 8, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 8192, },
  { 16, 256, 512, 1024, 2048, 8192, },
  { 1, 16, 256, 512, 1024, 2048, 8192, },
  { 2, 16, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 256, 512, 1024, 2048, 8192, },
  { 4, 16, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 8192, },
  { 8, 16, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 8192, },
  { 32, 256, 512, 1024, 2048, 8192, },
  { 1, 32, 256, 512, 1024, 2048, 8192, },
  { 2, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 32, 256, 512, 1024, 2048, 8192, },
  { 4, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 32, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 8192, },
  { 8, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 32, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 8192, },
  { 16, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 2, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 4, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 8, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, },
  { 64, 256, 512, 1024, 2048, 8192, },
  { 1, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 64, 256, 512, 1024, 2048, 8192, },
  { 4, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 8192, },
  { 8, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 8192, },
  { 16, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 4, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 8, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, },
  { 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 4, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 8, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, },
  { 128, 256, 512, 1024, 2048, 8192, },
  { 1, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 8192, },
  { 8, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 8192, },
  { 16, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 8, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, },
  { 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 8, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, },
  { 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 8, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, },
  { 4096, 8192, },
  { 1, 4096, 8192, },
  { 2, 4096, 8192, },
  { 1, 2, 4096, 8192, },
  { 4, 4096, 8192, },
  { 1, 4, 4096, 8192, },
  { 2, 4, 4096, 8192, },
  { 1, 2, 4, 4096, 8192, },
  { 8, 4096, 8192, },
  { 1, 8, 4096, 8192, },
  { 2, 8, 4096, 8192, },
  { 1, 2, 8, 4096, 8192, },
  { 4, 8, 4096, 8192, },
  { 1, 4, 8, 4096, 8192, },
  { 2, 4, 8, 4096, 8192, },
  { 1, 2, 4, 8, 4096, 8192, },
  { 16, 4096, 8192, },
  { 1, 16, 4096, 8192, },
  { 2, 16, 4096, 8192, },
  { 1, 2, 16, 4096, 8192, },
  { 4, 16, 4096, 8192, },
  { 1, 4, 16, 4096, 8192, },
  { 2, 4, 16, 4096, 8192, },
  { 1, 2, 4, 16, 4096, 8192, },
  { 8, 16, 4096, 8192, },
  { 1, 8, 16, 4096, 8192, },
  { 2, 8, 16, 4096, 8192, },
  { 1, 2, 8, 16, 4096, 8192, },
  { 4, 8, 16, 4096, 8192, },
  { 1, 4, 8, 16, 4096, 8192, },
  { 2, 4, 8, 16, 4096, 8192, },
  { 1, 2, 4, 8, 16, 4096, 8192, },
  { 32, 4096, 8192, },
  { 1, 32, 4096, 8192, },
  { 2, 32, 4096, 8192, },
  { 1, 2, 32, 4096, 8192, },
  { 4, 32, 4096, 8192, },
  { 1, 4, 32, 4096, 8192, },
  { 2, 4, 32, 4096, 8192, },
  { 1, 2, 4, 32, 4096, 8192, },
  { 8, 32, 4096, 8192, },
  { 1, 8, 32, 4096, 8192, },
  { 2, 8, 32, 4096, 8192, },
  { 1, 2, 8, 32, 4096, 8192, },
  { 4, 8, 32, 4096, 8192, },
  { 1, 4, 8, 32, 4096, 8192, },
  { 2, 4, 8, 32, 4096, 8192, },
  { 1, 2, 4, 8, 32, 4096, 8192, },
  { 16, 32, 4096, 8192, },
  { 1, 16, 32, 4096, 8192, },
  { 2, 16, 32, 4096, 8192, },
  { 1, 2, 16, 32, 4096, 8192, },
  { 4, 16, 32, 4096, 8192, },
  { 1, 4, 16, 32, 4096, 8192, },
  { 2, 4, 16, 32, 4096, 8192, },
  { 1, 2, 4, 16, 32, 4096, 8192, },
  { 8, 16, 32, 4096, 8192, },
  { 1, 8, 16, 32, 4096, 8192, },
  { 2, 8, 16, 32, 4096, 8192, },
  { 1, 2, 8, 16, 32, 4096, 8192, },
  { 4, 8, 16, 32, 4096, 8192, },
  { 1, 4, 8, 16, 32, 4096, 8192, },
  { 2, 4, 8, 16, 32, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 4096, 8192, },
  { 64, 4096, 8192, },
  { 1, 64, 4096, 8192, },
  { 2, 64, 4096, 8192, },
  { 1, 2, 64, 4096, 8192, },
  { 4, 64, 4096, 8192, },
  { 1, 4, 64, 4096, 8192, },
  { 2, 4, 64, 4096, 8192, },
  { 1, 2, 4, 64, 4096, 8192, },
  { 8, 64, 4096, 8192, },
  { 1, 8, 64, 4096, 8192, },
  { 2, 8, 64, 4096, 8192, },
  { 1, 2, 8, 64, 4096, 8192, },
  { 4, 8, 64, 4096, 8192, },
  { 1, 4, 8, 64, 4096, 8192, },
  { 2, 4, 8, 64, 4096, 8192, },
  { 1, 2, 4, 8, 64, 4096, 8192, },
  { 16, 64, 4096, 8192, },
  { 1, 16, 64, 4096, 8192, },
  { 2, 16, 64, 4096, 8192, },
  { 1, 2, 16, 64, 4096, 8192, },
  { 4, 16, 64, 4096, 8192, },
  { 1, 4, 16, 64, 4096, 8192, },
  { 2, 4, 16, 64, 4096, 8192, },
  { 1, 2, 4, 16, 64, 4096, 8192, },
  { 8, 16, 64, 4096, 8192, },
  { 1, 8, 16, 64, 4096, 8192, },
  { 2, 8, 16, 64, 4096, 8192, },
  { 1, 2, 8, 16, 64, 4096, 8192, },
  { 4, 8, 16, 64, 4096, 8192, },
  { 1, 4, 8, 16, 64, 4096, 8192, },
  { 2, 4, 8, 16, 64, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 4096, 8192, },
  { 32, 64, 4096, 8192, },
  { 1, 32, 64, 4096, 8192, },
  { 2, 32, 64, 4096, 8192, },
  { 1, 2, 32, 64, 4096, 8192, },
  { 4, 32, 64, 4096, 8192, },
  { 1, 4, 32, 64, 4096, 8192, },
  { 2, 4, 32, 64, 4096, 8192, },
  { 1, 2, 4, 32, 64, 4096, 8192, },
  { 8, 32, 64, 4096, 8192, },
  { 1, 8, 32, 64, 4096, 8192, },
  { 2, 8, 32, 64, 4096, 8192, },
  { 1, 2, 8, 32, 64, 4096, 8192, },
  { 4, 8, 32, 64, 4096, 8192, },
  { 1, 4, 8, 32, 64, 4096, 8192, },
  { 2, 4, 8, 32, 64, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 4096, 8192, },
  { 16, 32, 64, 4096, 8192, },
  { 1, 16, 32, 64, 4096, 8192, },
  { 2, 16, 32, 64, 4096, 8192, },
  { 1, 2, 16, 32, 64, 4096, 8192, },
  { 4, 16, 32, 64, 4096, 8192, },
  { 1, 4, 16, 32, 64, 4096, 8192, },
  { 2, 4, 16, 32, 64, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 4096, 8192, },
  { 8, 16, 32, 64, 4096, 8192, },
  { 1, 8, 16, 32, 64, 4096, 8192, },
  { 2, 8, 16, 32, 64, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 4096, 8192, },
  { 4, 8, 16, 32, 64, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 4096, 8192, },
  { 128, 4096, 8192, },
  { 1, 128, 4096, 8192, },
  { 2, 128, 4096, 8192, },
  { 1, 2, 128, 4096, 8192, },
  { 4, 128, 4096, 8192, },
  { 1, 4, 128, 4096, 8192, },
  { 2, 4, 128, 4096, 8192, },
  { 1, 2, 4, 128, 4096, 8192, },
  { 8, 128, 4096, 8192, },
  { 1, 8, 128, 4096, 8192, },
  { 2, 8, 128, 4096, 8192, },
  { 1, 2, 8, 128, 4096, 8192, },
  { 4, 8, 128, 4096, 8192, },
  { 1, 4, 8, 128, 4096, 8192, },
  { 2, 4, 8, 128, 4096, 8192, },
  { 1, 2, 4, 8, 128, 4096, 8192, },
  { 16, 128, 4096, 8192, },
  { 1, 16, 128, 4096, 8192, },
  { 2, 16, 128, 4096, 8192, },
  { 1, 2, 16, 128, 4096, 8192, },
  { 4, 16, 128, 4096, 8192, },
  { 1, 4, 16, 128, 4096, 8192, },
  { 2, 4, 16, 128, 4096, 8192, },
  { 1, 2, 4, 16, 128, 4096, 8192, },
  { 8, 16, 128, 4096, 8192, },
  { 1, 8, 16, 128, 4096, 8192, },
  { 2, 8, 16, 128, 4096, 8192, },
  { 1, 2, 8, 16, 128, 4096, 8192, },
  { 4, 8, 16, 128, 4096, 8192, },
  { 1, 4, 8, 16, 128, 4096, 8192, },
  { 2, 4, 8, 16, 128, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 4096, 8192, },
  { 32, 128, 4096, 8192, },
  { 1, 32, 128, 4096, 8192, },
  { 2, 32, 128, 4096, 8192, },
  { 1, 2, 32, 128, 4096, 8192, },
  { 4, 32, 128, 4096, 8192, },
  { 1, 4, 32, 128, 4096, 8192, },
  { 2, 4, 32, 128, 4096, 8192, },
  { 1, 2, 4, 32, 128, 4096, 8192, },
  { 8, 32, 128, 4096, 8192, },
  { 1, 8, 32, 128, 4096, 8192, },
  { 2, 8, 32, 128, 4096, 8192, },
  { 1, 2, 8, 32, 128, 4096, 8192, },
  { 4, 8, 32, 128, 4096, 8192, },
  { 1, 4, 8, 32, 128, 4096, 8192, },
  { 2, 4, 8, 32, 128, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 4096, 8192, },
  { 16, 32, 128, 4096, 8192, },
  { 1, 16, 32, 128, 4096, 8192, },
  { 2, 16, 32, 128, 4096, 8192, },
  { 1, 2, 16, 32, 128, 4096, 8192, },
  { 4, 16, 32, 128, 4096, 8192, },
  { 1, 4, 16, 32, 128, 4096, 8192, },
  { 2, 4, 16, 32, 128, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 4096, 8192, },
  { 8, 16, 32, 128, 4096, 8192, },
  { 1, 8, 16, 32, 128, 4096, 8192, },
  { 2, 8, 16, 32, 128, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 4096, 8192, },
  { 4, 8, 16, 32, 128, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 4096, 8192, },
  { 64, 128, 4096, 8192, },
  { 1, 64, 128, 4096, 8192, },
  { 2, 64, 128, 4096, 8192, },
  { 1, 2, 64, 128, 4096, 8192, },
  { 4, 64, 128, 4096, 8192, },
  { 1, 4, 64, 128, 4096, 8192, },
  { 2, 4, 64, 128, 4096, 8192, },
  { 1, 2, 4, 64, 128, 4096, 8192, },
  { 8, 64, 128, 4096, 8192, },
  { 1, 8, 64, 128, 4096, 8192, },
  { 2, 8, 64, 128, 4096, 8192, },
  { 1, 2, 8, 64, 128, 4096, 8192, },
  { 4, 8, 64, 128, 4096, 8192, },
  { 1, 4, 8, 64, 128, 4096, 8192, },
  { 2, 4, 8, 64, 128, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 4096, 8192, },
  { 16, 64, 128, 4096, 8192, },
  { 1, 16, 64, 128, 4096, 8192, },
  { 2, 16, 64, 128, 4096, 8192, },
  { 1, 2, 16, 64, 128, 4096, 8192, },
  { 4, 16, 64, 128, 4096, 8192, },
  { 1, 4, 16, 64, 128, 4096, 8192, },
  { 2, 4, 16, 64, 128, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 4096, 8192, },
  { 8, 16, 64, 128, 4096, 8192, },
  { 1, 8, 16, 64, 128, 4096, 8192, },
  { 2, 8, 16, 64, 128, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 4096, 8192, },
  { 4, 8, 16, 64, 128, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 4096, 8192, },
  { 32, 64, 128, 4096, 8192, },
  { 1, 32, 64, 128, 4096, 8192, },
  { 2, 32, 64, 128, 4096, 8192, },
  { 1, 2, 32, 64, 128, 4096, 8192, },
  { 4, 32, 64, 128, 4096, 8192, },
  { 1, 4, 32, 64, 128, 4096, 8192, },
  { 2, 4, 32, 64, 128, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 4096, 8192, },
  { 8, 32, 64, 128, 4096, 8192, },
  { 1, 8, 32, 64, 128, 4096, 8192, },
  { 2, 8, 32, 64, 128, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 4096, 8192, },
  { 4, 8, 32, 64, 128, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 4096, 8192, },
  { 16, 32, 64, 128, 4096, 8192, },
  { 1, 16, 32, 64, 128, 4096, 8192, },
  { 2, 16, 32, 64, 128, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 4096, 8192, },
  { 4, 16, 32, 64, 128, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 4096, 8192, },
  { 8, 16, 32, 64, 128, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 4096, 8192, },
  { 256, 4096, 8192, },
  { 1, 256, 4096, 8192, },
  { 2, 256, 4096, 8192, },
  { 1, 2, 256, 4096, 8192, },
  { 4, 256, 4096, 8192, },
  { 1, 4, 256, 4096, 8192, },
  { 2, 4, 256, 4096, 8192, },
  { 1, 2, 4, 256, 4096, 8192, },
  { 8, 256, 4096, 8192, },
  { 1, 8, 256, 4096, 8192, },
  { 2, 8, 256, 4096, 8192, },
  { 1, 2, 8, 256, 4096, 8192, },
  { 4, 8, 256, 4096, 8192, },
  { 1, 4, 8, 256, 4096, 8192, },
  { 2, 4, 8, 256, 4096, 8192, },
  { 1, 2, 4, 8, 256, 4096, 8192, },
  { 16, 256, 4096, 8192, },
  { 1, 16, 256, 4096, 8192, },
  { 2, 16, 256, 4096, 8192, },
  { 1, 2, 16, 256, 4096, 8192, },
  { 4, 16, 256, 4096, 8192, },
  { 1, 4, 16, 256, 4096, 8192, },
  { 2, 4, 16, 256, 4096, 8192, },
  { 1, 2, 4, 16, 256, 4096, 8192, },
  { 8, 16, 256, 4096, 8192, },
  { 1, 8, 16, 256, 4096, 8192, },
  { 2, 8, 16, 256, 4096, 8192, },
  { 1, 2, 8, 16, 256, 4096, 8192, },
  { 4, 8, 16, 256, 4096, 8192, },
  { 1, 4, 8, 16, 256, 4096, 8192, },
  { 2, 4, 8, 16, 256, 4096, 8192, },
  { 1, 2, 4, 8, 16, 256, 4096, 8192, },
  { 32, 256, 4096, 8192, },
  { 1, 32, 256, 4096, 8192, },
  { 2, 32, 256, 4096, 8192, },
  { 1, 2, 32, 256, 4096, 8192, },
  { 4, 32, 256, 4096, 8192, },
  { 1, 4, 32, 256, 4096, 8192, },
  { 2, 4, 32, 256, 4096, 8192, },
  { 1, 2, 4, 32, 256, 4096, 8192, },
  { 8, 32, 256, 4096, 8192, },
  { 1, 8, 32, 256, 4096, 8192, },
  { 2, 8, 32, 256, 4096, 8192, },
  { 1, 2, 8, 32, 256, 4096, 8192, },
  { 4, 8, 32, 256, 4096, 8192, },
  { 1, 4, 8, 32, 256, 4096, 8192, },
  { 2, 4, 8, 32, 256, 4096, 8192, },
  { 1, 2, 4, 8, 32, 256, 4096, 8192, },
  { 16, 32, 256, 4096, 8192, },
  { 1, 16, 32, 256, 4096, 8192, },
  { 2, 16, 32, 256, 4096, 8192, },
  { 1, 2, 16, 32, 256, 4096, 8192, },
  { 4, 16, 32, 256, 4096, 8192, },
  { 1, 4, 16, 32, 256, 4096, 8192, },
  { 2, 4, 16, 32, 256, 4096, 8192, },
  { 1, 2, 4, 16, 32, 256, 4096, 8192, },
  { 8, 16, 32, 256, 4096, 8192, },
  { 1, 8, 16, 32, 256, 4096, 8192, },
  { 2, 8, 16, 32, 256, 4096, 8192, },
  { 1, 2, 8, 16, 32, 256, 4096, 8192, },
  { 4, 8, 16, 32, 256, 4096, 8192, },
  { 1, 4, 8, 16, 32, 256, 4096, 8192, },
  { 2, 4, 8, 16, 32, 256, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 4096, 8192, },
  { 64, 256, 4096, 8192, },
  { 1, 64, 256, 4096, 8192, },
  { 2, 64, 256, 4096, 8192, },
  { 1, 2, 64, 256, 4096, 8192, },
  { 4, 64, 256, 4096, 8192, },
  { 1, 4, 64, 256, 4096, 8192, },
  { 2, 4, 64, 256, 4096, 8192, },
  { 1, 2, 4, 64, 256, 4096, 8192, },
  { 8, 64, 256, 4096, 8192, },
  { 1, 8, 64, 256, 4096, 8192, },
  { 2, 8, 64, 256, 4096, 8192, },
  { 1, 2, 8, 64, 256, 4096, 8192, },
  { 4, 8, 64, 256, 4096, 8192, },
  { 1, 4, 8, 64, 256, 4096, 8192, },
  { 2, 4, 8, 64, 256, 4096, 8192, },
  { 1, 2, 4, 8, 64, 256, 4096, 8192, },
  { 16, 64, 256, 4096, 8192, },
  { 1, 16, 64, 256, 4096, 8192, },
  { 2, 16, 64, 256, 4096, 8192, },
  { 1, 2, 16, 64, 256, 4096, 8192, },
  { 4, 16, 64, 256, 4096, 8192, },
  { 1, 4, 16, 64, 256, 4096, 8192, },
  { 2, 4, 16, 64, 256, 4096, 8192, },
  { 1, 2, 4, 16, 64, 256, 4096, 8192, },
  { 8, 16, 64, 256, 4096, 8192, },
  { 1, 8, 16, 64, 256, 4096, 8192, },
  { 2, 8, 16, 64, 256, 4096, 8192, },
  { 1, 2, 8, 16, 64, 256, 4096, 8192, },
  { 4, 8, 16, 64, 256, 4096, 8192, },
  { 1, 4, 8, 16, 64, 256, 4096, 8192, },
  { 2, 4, 8, 16, 64, 256, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 4096, 8192, },
  { 32, 64, 256, 4096, 8192, },
  { 1, 32, 64, 256, 4096, 8192, },
  { 2, 32, 64, 256, 4096, 8192, },
  { 1, 2, 32, 64, 256, 4096, 8192, },
  { 4, 32, 64, 256, 4096, 8192, },
  { 1, 4, 32, 64, 256, 4096, 8192, },
  { 2, 4, 32, 64, 256, 4096, 8192, },
  { 1, 2, 4, 32, 64, 256, 4096, 8192, },
  { 8, 32, 64, 256, 4096, 8192, },
  { 1, 8, 32, 64, 256, 4096, 8192, },
  { 2, 8, 32, 64, 256, 4096, 8192, },
  { 1, 2, 8, 32, 64, 256, 4096, 8192, },
  { 4, 8, 32, 64, 256, 4096, 8192, },
  { 1, 4, 8, 32, 64, 256, 4096, 8192, },
  { 2, 4, 8, 32, 64, 256, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 4096, 8192, },
  { 16, 32, 64, 256, 4096, 8192, },
  { 1, 16, 32, 64, 256, 4096, 8192, },
  { 2, 16, 32, 64, 256, 4096, 8192, },
  { 1, 2, 16, 32, 64, 256, 4096, 8192, },
  { 4, 16, 32, 64, 256, 4096, 8192, },
  { 1, 4, 16, 32, 64, 256, 4096, 8192, },
  { 2, 4, 16, 32, 64, 256, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 4096, 8192, },
  { 8, 16, 32, 64, 256, 4096, 8192, },
  { 1, 8, 16, 32, 64, 256, 4096, 8192, },
  { 2, 8, 16, 32, 64, 256, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 4096, 8192, },
  { 4, 8, 16, 32, 64, 256, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 4096, 8192, },
  { 128, 256, 4096, 8192, },
  { 1, 128, 256, 4096, 8192, },
  { 2, 128, 256, 4096, 8192, },
  { 1, 2, 128, 256, 4096, 8192, },
  { 4, 128, 256, 4096, 8192, },
  { 1, 4, 128, 256, 4096, 8192, },
  { 2, 4, 128, 256, 4096, 8192, },
  { 1, 2, 4, 128, 256, 4096, 8192, },
  { 8, 128, 256, 4096, 8192, },
  { 1, 8, 128, 256, 4096, 8192, },
  { 2, 8, 128, 256, 4096, 8192, },
  { 1, 2, 8, 128, 256, 4096, 8192, },
  { 4, 8, 128, 256, 4096, 8192, },
  { 1, 4, 8, 128, 256, 4096, 8192, },
  { 2, 4, 8, 128, 256, 4096, 8192, },
  { 1, 2, 4, 8, 128, 256, 4096, 8192, },
  { 16, 128, 256, 4096, 8192, },
  { 1, 16, 128, 256, 4096, 8192, },
  { 2, 16, 128, 256, 4096, 8192, },
  { 1, 2, 16, 128, 256, 4096, 8192, },
  { 4, 16, 128, 256, 4096, 8192, },
  { 1, 4, 16, 128, 256, 4096, 8192, },
  { 2, 4, 16, 128, 256, 4096, 8192, },
  { 1, 2, 4, 16, 128, 256, 4096, 8192, },
  { 8, 16, 128, 256, 4096, 8192, },
  { 1, 8, 16, 128, 256, 4096, 8192, },
  { 2, 8, 16, 128, 256, 4096, 8192, },
  { 1, 2, 8, 16, 128, 256, 4096, 8192, },
  { 4, 8, 16, 128, 256, 4096, 8192, },
  { 1, 4, 8, 16, 128, 256, 4096, 8192, },
  { 2, 4, 8, 16, 128, 256, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 4096, 8192, },
  { 32, 128, 256, 4096, 8192, },
  { 1, 32, 128, 256, 4096, 8192, },
  { 2, 32, 128, 256, 4096, 8192, },
  { 1, 2, 32, 128, 256, 4096, 8192, },
  { 4, 32, 128, 256, 4096, 8192, },
  { 1, 4, 32, 128, 256, 4096, 8192, },
  { 2, 4, 32, 128, 256, 4096, 8192, },
  { 1, 2, 4, 32, 128, 256, 4096, 8192, },
  { 8, 32, 128, 256, 4096, 8192, },
  { 1, 8, 32, 128, 256, 4096, 8192, },
  { 2, 8, 32, 128, 256, 4096, 8192, },
  { 1, 2, 8, 32, 128, 256, 4096, 8192, },
  { 4, 8, 32, 128, 256, 4096, 8192, },
  { 1, 4, 8, 32, 128, 256, 4096, 8192, },
  { 2, 4, 8, 32, 128, 256, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 4096, 8192, },
  { 16, 32, 128, 256, 4096, 8192, },
  { 1, 16, 32, 128, 256, 4096, 8192, },
  { 2, 16, 32, 128, 256, 4096, 8192, },
  { 1, 2, 16, 32, 128, 256, 4096, 8192, },
  { 4, 16, 32, 128, 256, 4096, 8192, },
  { 1, 4, 16, 32, 128, 256, 4096, 8192, },
  { 2, 4, 16, 32, 128, 256, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 4096, 8192, },
  { 8, 16, 32, 128, 256, 4096, 8192, },
  { 1, 8, 16, 32, 128, 256, 4096, 8192, },
  { 2, 8, 16, 32, 128, 256, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 4096, 8192, },
  { 4, 8, 16, 32, 128, 256, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 4096, 8192, },
  { 64, 128, 256, 4096, 8192, },
  { 1, 64, 128, 256, 4096, 8192, },
  { 2, 64, 128, 256, 4096, 8192, },
  { 1, 2, 64, 128, 256, 4096, 8192, },
  { 4, 64, 128, 256, 4096, 8192, },
  { 1, 4, 64, 128, 256, 4096, 8192, },
  { 2, 4, 64, 128, 256, 4096, 8192, },
  { 1, 2, 4, 64, 128, 256, 4096, 8192, },
  { 8, 64, 128, 256, 4096, 8192, },
  { 1, 8, 64, 128, 256, 4096, 8192, },
  { 2, 8, 64, 128, 256, 4096, 8192, },
  { 1, 2, 8, 64, 128, 256, 4096, 8192, },
  { 4, 8, 64, 128, 256, 4096, 8192, },
  { 1, 4, 8, 64, 128, 256, 4096, 8192, },
  { 2, 4, 8, 64, 128, 256, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 4096, 8192, },
  { 16, 64, 128, 256, 4096, 8192, },
  { 1, 16, 64, 128, 256, 4096, 8192, },
  { 2, 16, 64, 128, 256, 4096, 8192, },
  { 1, 2, 16, 64, 128, 256, 4096, 8192, },
  { 4, 16, 64, 128, 256, 4096, 8192, },
  { 1, 4, 16, 64, 128, 256, 4096, 8192, },
  { 2, 4, 16, 64, 128, 256, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 4096, 8192, },
  { 8, 16, 64, 128, 256, 4096, 8192, },
  { 1, 8, 16, 64, 128, 256, 4096, 8192, },
  { 2, 8, 16, 64, 128, 256, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 4096, 8192, },
  { 4, 8, 16, 64, 128, 256, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 4096, 8192, },
  { 32, 64, 128, 256, 4096, 8192, },
  { 1, 32, 64, 128, 256, 4096, 8192, },
  { 2, 32, 64, 128, 256, 4096, 8192, },
  { 1, 2, 32, 64, 128, 256, 4096, 8192, },
  { 4, 32, 64, 128, 256, 4096, 8192, },
  { 1, 4, 32, 64, 128, 256, 4096, 8192, },
  { 2, 4, 32, 64, 128, 256, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 4096, 8192, },
  { 8, 32, 64, 128, 256, 4096, 8192, },
  { 1, 8, 32, 64, 128, 256, 4096, 8192, },
  { 2, 8, 32, 64, 128, 256, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 4096, 8192, },
  { 4, 8, 32, 64, 128, 256, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 4096, 8192, },
  { 16, 32, 64, 128, 256, 4096, 8192, },
  { 1, 16, 32, 64, 128, 256, 4096, 8192, },
  { 2, 16, 32, 64, 128, 256, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 4096, 8192, },
  { 4, 16, 32, 64, 128, 256, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 4096, 8192, },
  { 8, 16, 32, 64, 128, 256, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, },
  { 512, 4096, 8192, },
  { 1, 512, 4096, 8192, },
  { 2, 512, 4096, 8192, },
  { 1, 2, 512, 4096, 8192, },
  { 4, 512, 4096, 8192, },
  { 1, 4, 512, 4096, 8192, },
  { 2, 4, 512, 4096, 8192, },
  { 1, 2, 4, 512, 4096, 8192, },
  { 8, 512, 4096, 8192, },
  { 1, 8, 512, 4096, 8192, },
  { 2, 8, 512, 4096, 8192, },
  { 1, 2, 8, 512, 4096, 8192, },
  { 4, 8, 512, 4096, 8192, },
  { 1, 4, 8, 512, 4096, 8192, },
  { 2, 4, 8, 512, 4096, 8192, },
  { 1, 2, 4, 8, 512, 4096, 8192, },
  { 16, 512, 4096, 8192, },
  { 1, 16, 512, 4096, 8192, },
  { 2, 16, 512, 4096, 8192, },
  { 1, 2, 16, 512, 4096, 8192, },
  { 4, 16, 512, 4096, 8192, },
  { 1, 4, 16, 512, 4096, 8192, },
  { 2, 4, 16, 512, 4096, 8192, },
  { 1, 2, 4, 16, 512, 4096, 8192, },
  { 8, 16, 512, 4096, 8192, },
  { 1, 8, 16, 512, 4096, 8192, },
  { 2, 8, 16, 512, 4096, 8192, },
  { 1, 2, 8, 16, 512, 4096, 8192, },
  { 4, 8, 16, 512, 4096, 8192, },
  { 1, 4, 8, 16, 512, 4096, 8192, },
  { 2, 4, 8, 16, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 512, 4096, 8192, },
  { 32, 512, 4096, 8192, },
  { 1, 32, 512, 4096, 8192, },
  { 2, 32, 512, 4096, 8192, },
  { 1, 2, 32, 512, 4096, 8192, },
  { 4, 32, 512, 4096, 8192, },
  { 1, 4, 32, 512, 4096, 8192, },
  { 2, 4, 32, 512, 4096, 8192, },
  { 1, 2, 4, 32, 512, 4096, 8192, },
  { 8, 32, 512, 4096, 8192, },
  { 1, 8, 32, 512, 4096, 8192, },
  { 2, 8, 32, 512, 4096, 8192, },
  { 1, 2, 8, 32, 512, 4096, 8192, },
  { 4, 8, 32, 512, 4096, 8192, },
  { 1, 4, 8, 32, 512, 4096, 8192, },
  { 2, 4, 8, 32, 512, 4096, 8192, },
  { 1, 2, 4, 8, 32, 512, 4096, 8192, },
  { 16, 32, 512, 4096, 8192, },
  { 1, 16, 32, 512, 4096, 8192, },
  { 2, 16, 32, 512, 4096, 8192, },
  { 1, 2, 16, 32, 512, 4096, 8192, },
  { 4, 16, 32, 512, 4096, 8192, },
  { 1, 4, 16, 32, 512, 4096, 8192, },
  { 2, 4, 16, 32, 512, 4096, 8192, },
  { 1, 2, 4, 16, 32, 512, 4096, 8192, },
  { 8, 16, 32, 512, 4096, 8192, },
  { 1, 8, 16, 32, 512, 4096, 8192, },
  { 2, 8, 16, 32, 512, 4096, 8192, },
  { 1, 2, 8, 16, 32, 512, 4096, 8192, },
  { 4, 8, 16, 32, 512, 4096, 8192, },
  { 1, 4, 8, 16, 32, 512, 4096, 8192, },
  { 2, 4, 8, 16, 32, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 512, 4096, 8192, },
  { 64, 512, 4096, 8192, },
  { 1, 64, 512, 4096, 8192, },
  { 2, 64, 512, 4096, 8192, },
  { 1, 2, 64, 512, 4096, 8192, },
  { 4, 64, 512, 4096, 8192, },
  { 1, 4, 64, 512, 4096, 8192, },
  { 2, 4, 64, 512, 4096, 8192, },
  { 1, 2, 4, 64, 512, 4096, 8192, },
  { 8, 64, 512, 4096, 8192, },
  { 1, 8, 64, 512, 4096, 8192, },
  { 2, 8, 64, 512, 4096, 8192, },
  { 1, 2, 8, 64, 512, 4096, 8192, },
  { 4, 8, 64, 512, 4096, 8192, },
  { 1, 4, 8, 64, 512, 4096, 8192, },
  { 2, 4, 8, 64, 512, 4096, 8192, },
  { 1, 2, 4, 8, 64, 512, 4096, 8192, },
  { 16, 64, 512, 4096, 8192, },
  { 1, 16, 64, 512, 4096, 8192, },
  { 2, 16, 64, 512, 4096, 8192, },
  { 1, 2, 16, 64, 512, 4096, 8192, },
  { 4, 16, 64, 512, 4096, 8192, },
  { 1, 4, 16, 64, 512, 4096, 8192, },
  { 2, 4, 16, 64, 512, 4096, 8192, },
  { 1, 2, 4, 16, 64, 512, 4096, 8192, },
  { 8, 16, 64, 512, 4096, 8192, },
  { 1, 8, 16, 64, 512, 4096, 8192, },
  { 2, 8, 16, 64, 512, 4096, 8192, },
  { 1, 2, 8, 16, 64, 512, 4096, 8192, },
  { 4, 8, 16, 64, 512, 4096, 8192, },
  { 1, 4, 8, 16, 64, 512, 4096, 8192, },
  { 2, 4, 8, 16, 64, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 512, 4096, 8192, },
  { 32, 64, 512, 4096, 8192, },
  { 1, 32, 64, 512, 4096, 8192, },
  { 2, 32, 64, 512, 4096, 8192, },
  { 1, 2, 32, 64, 512, 4096, 8192, },
  { 4, 32, 64, 512, 4096, 8192, },
  { 1, 4, 32, 64, 512, 4096, 8192, },
  { 2, 4, 32, 64, 512, 4096, 8192, },
  { 1, 2, 4, 32, 64, 512, 4096, 8192, },
  { 8, 32, 64, 512, 4096, 8192, },
  { 1, 8, 32, 64, 512, 4096, 8192, },
  { 2, 8, 32, 64, 512, 4096, 8192, },
  { 1, 2, 8, 32, 64, 512, 4096, 8192, },
  { 4, 8, 32, 64, 512, 4096, 8192, },
  { 1, 4, 8, 32, 64, 512, 4096, 8192, },
  { 2, 4, 8, 32, 64, 512, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 512, 4096, 8192, },
  { 16, 32, 64, 512, 4096, 8192, },
  { 1, 16, 32, 64, 512, 4096, 8192, },
  { 2, 16, 32, 64, 512, 4096, 8192, },
  { 1, 2, 16, 32, 64, 512, 4096, 8192, },
  { 4, 16, 32, 64, 512, 4096, 8192, },
  { 1, 4, 16, 32, 64, 512, 4096, 8192, },
  { 2, 4, 16, 32, 64, 512, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 512, 4096, 8192, },
  { 8, 16, 32, 64, 512, 4096, 8192, },
  { 1, 8, 16, 32, 64, 512, 4096, 8192, },
  { 2, 8, 16, 32, 64, 512, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 512, 4096, 8192, },
  { 4, 8, 16, 32, 64, 512, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 512, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 4096, 8192, },
  { 128, 512, 4096, 8192, },
  { 1, 128, 512, 4096, 8192, },
  { 2, 128, 512, 4096, 8192, },
  { 1, 2, 128, 512, 4096, 8192, },
  { 4, 128, 512, 4096, 8192, },
  { 1, 4, 128, 512, 4096, 8192, },
  { 2, 4, 128, 512, 4096, 8192, },
  { 1, 2, 4, 128, 512, 4096, 8192, },
  { 8, 128, 512, 4096, 8192, },
  { 1, 8, 128, 512, 4096, 8192, },
  { 2, 8, 128, 512, 4096, 8192, },
  { 1, 2, 8, 128, 512, 4096, 8192, },
  { 4, 8, 128, 512, 4096, 8192, },
  { 1, 4, 8, 128, 512, 4096, 8192, },
  { 2, 4, 8, 128, 512, 4096, 8192, },
  { 1, 2, 4, 8, 128, 512, 4096, 8192, },
  { 16, 128, 512, 4096, 8192, },
  { 1, 16, 128, 512, 4096, 8192, },
  { 2, 16, 128, 512, 4096, 8192, },
  { 1, 2, 16, 128, 512, 4096, 8192, },
  { 4, 16, 128, 512, 4096, 8192, },
  { 1, 4, 16, 128, 512, 4096, 8192, },
  { 2, 4, 16, 128, 512, 4096, 8192, },
  { 1, 2, 4, 16, 128, 512, 4096, 8192, },
  { 8, 16, 128, 512, 4096, 8192, },
  { 1, 8, 16, 128, 512, 4096, 8192, },
  { 2, 8, 16, 128, 512, 4096, 8192, },
  { 1, 2, 8, 16, 128, 512, 4096, 8192, },
  { 4, 8, 16, 128, 512, 4096, 8192, },
  { 1, 4, 8, 16, 128, 512, 4096, 8192, },
  { 2, 4, 8, 16, 128, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 512, 4096, 8192, },
  { 32, 128, 512, 4096, 8192, },
  { 1, 32, 128, 512, 4096, 8192, },
  { 2, 32, 128, 512, 4096, 8192, },
  { 1, 2, 32, 128, 512, 4096, 8192, },
  { 4, 32, 128, 512, 4096, 8192, },
  { 1, 4, 32, 128, 512, 4096, 8192, },
  { 2, 4, 32, 128, 512, 4096, 8192, },
  { 1, 2, 4, 32, 128, 512, 4096, 8192, },
  { 8, 32, 128, 512, 4096, 8192, },
  { 1, 8, 32, 128, 512, 4096, 8192, },
  { 2, 8, 32, 128, 512, 4096, 8192, },
  { 1, 2, 8, 32, 128, 512, 4096, 8192, },
  { 4, 8, 32, 128, 512, 4096, 8192, },
  { 1, 4, 8, 32, 128, 512, 4096, 8192, },
  { 2, 4, 8, 32, 128, 512, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 512, 4096, 8192, },
  { 16, 32, 128, 512, 4096, 8192, },
  { 1, 16, 32, 128, 512, 4096, 8192, },
  { 2, 16, 32, 128, 512, 4096, 8192, },
  { 1, 2, 16, 32, 128, 512, 4096, 8192, },
  { 4, 16, 32, 128, 512, 4096, 8192, },
  { 1, 4, 16, 32, 128, 512, 4096, 8192, },
  { 2, 4, 16, 32, 128, 512, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 512, 4096, 8192, },
  { 8, 16, 32, 128, 512, 4096, 8192, },
  { 1, 8, 16, 32, 128, 512, 4096, 8192, },
  { 2, 8, 16, 32, 128, 512, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 512, 4096, 8192, },
  { 4, 8, 16, 32, 128, 512, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 512, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 4096, 8192, },
  { 64, 128, 512, 4096, 8192, },
  { 1, 64, 128, 512, 4096, 8192, },
  { 2, 64, 128, 512, 4096, 8192, },
  { 1, 2, 64, 128, 512, 4096, 8192, },
  { 4, 64, 128, 512, 4096, 8192, },
  { 1, 4, 64, 128, 512, 4096, 8192, },
  { 2, 4, 64, 128, 512, 4096, 8192, },
  { 1, 2, 4, 64, 128, 512, 4096, 8192, },
  { 8, 64, 128, 512, 4096, 8192, },
  { 1, 8, 64, 128, 512, 4096, 8192, },
  { 2, 8, 64, 128, 512, 4096, 8192, },
  { 1, 2, 8, 64, 128, 512, 4096, 8192, },
  { 4, 8, 64, 128, 512, 4096, 8192, },
  { 1, 4, 8, 64, 128, 512, 4096, 8192, },
  { 2, 4, 8, 64, 128, 512, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 512, 4096, 8192, },
  { 16, 64, 128, 512, 4096, 8192, },
  { 1, 16, 64, 128, 512, 4096, 8192, },
  { 2, 16, 64, 128, 512, 4096, 8192, },
  { 1, 2, 16, 64, 128, 512, 4096, 8192, },
  { 4, 16, 64, 128, 512, 4096, 8192, },
  { 1, 4, 16, 64, 128, 512, 4096, 8192, },
  { 2, 4, 16, 64, 128, 512, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 512, 4096, 8192, },
  { 8, 16, 64, 128, 512, 4096, 8192, },
  { 1, 8, 16, 64, 128, 512, 4096, 8192, },
  { 2, 8, 16, 64, 128, 512, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 512, 4096, 8192, },
  { 4, 8, 16, 64, 128, 512, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 512, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 4096, 8192, },
  { 32, 64, 128, 512, 4096, 8192, },
  { 1, 32, 64, 128, 512, 4096, 8192, },
  { 2, 32, 64, 128, 512, 4096, 8192, },
  { 1, 2, 32, 64, 128, 512, 4096, 8192, },
  { 4, 32, 64, 128, 512, 4096, 8192, },
  { 1, 4, 32, 64, 128, 512, 4096, 8192, },
  { 2, 4, 32, 64, 128, 512, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 512, 4096, 8192, },
  { 8, 32, 64, 128, 512, 4096, 8192, },
  { 1, 8, 32, 64, 128, 512, 4096, 8192, },
  { 2, 8, 32, 64, 128, 512, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 512, 4096, 8192, },
  { 4, 8, 32, 64, 128, 512, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 512, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 512, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 4096, 8192, },
  { 16, 32, 64, 128, 512, 4096, 8192, },
  { 1, 16, 32, 64, 128, 512, 4096, 8192, },
  { 2, 16, 32, 64, 128, 512, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 512, 4096, 8192, },
  { 4, 16, 32, 64, 128, 512, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 512, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 512, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 4096, 8192, },
  { 8, 16, 32, 64, 128, 512, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 512, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 512, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 512, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, },
  { 256, 512, 4096, 8192, },
  { 1, 256, 512, 4096, 8192, },
  { 2, 256, 512, 4096, 8192, },
  { 1, 2, 256, 512, 4096, 8192, },
  { 4, 256, 512, 4096, 8192, },
  { 1, 4, 256, 512, 4096, 8192, },
  { 2, 4, 256, 512, 4096, 8192, },
  { 1, 2, 4, 256, 512, 4096, 8192, },
  { 8, 256, 512, 4096, 8192, },
  { 1, 8, 256, 512, 4096, 8192, },
  { 2, 8, 256, 512, 4096, 8192, },
  { 1, 2, 8, 256, 512, 4096, 8192, },
  { 4, 8, 256, 512, 4096, 8192, },
  { 1, 4, 8, 256, 512, 4096, 8192, },
  { 2, 4, 8, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 256, 512, 4096, 8192, },
  { 16, 256, 512, 4096, 8192, },
  { 1, 16, 256, 512, 4096, 8192, },
  { 2, 16, 256, 512, 4096, 8192, },
  { 1, 2, 16, 256, 512, 4096, 8192, },
  { 4, 16, 256, 512, 4096, 8192, },
  { 1, 4, 16, 256, 512, 4096, 8192, },
  { 2, 4, 16, 256, 512, 4096, 8192, },
  { 1, 2, 4, 16, 256, 512, 4096, 8192, },
  { 8, 16, 256, 512, 4096, 8192, },
  { 1, 8, 16, 256, 512, 4096, 8192, },
  { 2, 8, 16, 256, 512, 4096, 8192, },
  { 1, 2, 8, 16, 256, 512, 4096, 8192, },
  { 4, 8, 16, 256, 512, 4096, 8192, },
  { 1, 4, 8, 16, 256, 512, 4096, 8192, },
  { 2, 4, 8, 16, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 256, 512, 4096, 8192, },
  { 32, 256, 512, 4096, 8192, },
  { 1, 32, 256, 512, 4096, 8192, },
  { 2, 32, 256, 512, 4096, 8192, },
  { 1, 2, 32, 256, 512, 4096, 8192, },
  { 4, 32, 256, 512, 4096, 8192, },
  { 1, 4, 32, 256, 512, 4096, 8192, },
  { 2, 4, 32, 256, 512, 4096, 8192, },
  { 1, 2, 4, 32, 256, 512, 4096, 8192, },
  { 8, 32, 256, 512, 4096, 8192, },
  { 1, 8, 32, 256, 512, 4096, 8192, },
  { 2, 8, 32, 256, 512, 4096, 8192, },
  { 1, 2, 8, 32, 256, 512, 4096, 8192, },
  { 4, 8, 32, 256, 512, 4096, 8192, },
  { 1, 4, 8, 32, 256, 512, 4096, 8192, },
  { 2, 4, 8, 32, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 32, 256, 512, 4096, 8192, },
  { 16, 32, 256, 512, 4096, 8192, },
  { 1, 16, 32, 256, 512, 4096, 8192, },
  { 2, 16, 32, 256, 512, 4096, 8192, },
  { 1, 2, 16, 32, 256, 512, 4096, 8192, },
  { 4, 16, 32, 256, 512, 4096, 8192, },
  { 1, 4, 16, 32, 256, 512, 4096, 8192, },
  { 2, 4, 16, 32, 256, 512, 4096, 8192, },
  { 1, 2, 4, 16, 32, 256, 512, 4096, 8192, },
  { 8, 16, 32, 256, 512, 4096, 8192, },
  { 1, 8, 16, 32, 256, 512, 4096, 8192, },
  { 2, 8, 16, 32, 256, 512, 4096, 8192, },
  { 1, 2, 8, 16, 32, 256, 512, 4096, 8192, },
  { 4, 8, 16, 32, 256, 512, 4096, 8192, },
  { 1, 4, 8, 16, 32, 256, 512, 4096, 8192, },
  { 2, 4, 8, 16, 32, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 4096, 8192, },
  { 64, 256, 512, 4096, 8192, },
  { 1, 64, 256, 512, 4096, 8192, },
  { 2, 64, 256, 512, 4096, 8192, },
  { 1, 2, 64, 256, 512, 4096, 8192, },
  { 4, 64, 256, 512, 4096, 8192, },
  { 1, 4, 64, 256, 512, 4096, 8192, },
  { 2, 4, 64, 256, 512, 4096, 8192, },
  { 1, 2, 4, 64, 256, 512, 4096, 8192, },
  { 8, 64, 256, 512, 4096, 8192, },
  { 1, 8, 64, 256, 512, 4096, 8192, },
  { 2, 8, 64, 256, 512, 4096, 8192, },
  { 1, 2, 8, 64, 256, 512, 4096, 8192, },
  { 4, 8, 64, 256, 512, 4096, 8192, },
  { 1, 4, 8, 64, 256, 512, 4096, 8192, },
  { 2, 4, 8, 64, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 64, 256, 512, 4096, 8192, },
  { 16, 64, 256, 512, 4096, 8192, },
  { 1, 16, 64, 256, 512, 4096, 8192, },
  { 2, 16, 64, 256, 512, 4096, 8192, },
  { 1, 2, 16, 64, 256, 512, 4096, 8192, },
  { 4, 16, 64, 256, 512, 4096, 8192, },
  { 1, 4, 16, 64, 256, 512, 4096, 8192, },
  { 2, 4, 16, 64, 256, 512, 4096, 8192, },
  { 1, 2, 4, 16, 64, 256, 512, 4096, 8192, },
  { 8, 16, 64, 256, 512, 4096, 8192, },
  { 1, 8, 16, 64, 256, 512, 4096, 8192, },
  { 2, 8, 16, 64, 256, 512, 4096, 8192, },
  { 1, 2, 8, 16, 64, 256, 512, 4096, 8192, },
  { 4, 8, 16, 64, 256, 512, 4096, 8192, },
  { 1, 4, 8, 16, 64, 256, 512, 4096, 8192, },
  { 2, 4, 8, 16, 64, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 4096, 8192, },
  { 32, 64, 256, 512, 4096, 8192, },
  { 1, 32, 64, 256, 512, 4096, 8192, },
  { 2, 32, 64, 256, 512, 4096, 8192, },
  { 1, 2, 32, 64, 256, 512, 4096, 8192, },
  { 4, 32, 64, 256, 512, 4096, 8192, },
  { 1, 4, 32, 64, 256, 512, 4096, 8192, },
  { 2, 4, 32, 64, 256, 512, 4096, 8192, },
  { 1, 2, 4, 32, 64, 256, 512, 4096, 8192, },
  { 8, 32, 64, 256, 512, 4096, 8192, },
  { 1, 8, 32, 64, 256, 512, 4096, 8192, },
  { 2, 8, 32, 64, 256, 512, 4096, 8192, },
  { 1, 2, 8, 32, 64, 256, 512, 4096, 8192, },
  { 4, 8, 32, 64, 256, 512, 4096, 8192, },
  { 1, 4, 8, 32, 64, 256, 512, 4096, 8192, },
  { 2, 4, 8, 32, 64, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 4096, 8192, },
  { 16, 32, 64, 256, 512, 4096, 8192, },
  { 1, 16, 32, 64, 256, 512, 4096, 8192, },
  { 2, 16, 32, 64, 256, 512, 4096, 8192, },
  { 1, 2, 16, 32, 64, 256, 512, 4096, 8192, },
  { 4, 16, 32, 64, 256, 512, 4096, 8192, },
  { 1, 4, 16, 32, 64, 256, 512, 4096, 8192, },
  { 2, 4, 16, 32, 64, 256, 512, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 4096, 8192, },
  { 8, 16, 32, 64, 256, 512, 4096, 8192, },
  { 1, 8, 16, 32, 64, 256, 512, 4096, 8192, },
  { 2, 8, 16, 32, 64, 256, 512, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 4096, 8192, },
  { 4, 8, 16, 32, 64, 256, 512, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, },
  { 128, 256, 512, 4096, 8192, },
  { 1, 128, 256, 512, 4096, 8192, },
  { 2, 128, 256, 512, 4096, 8192, },
  { 1, 2, 128, 256, 512, 4096, 8192, },
  { 4, 128, 256, 512, 4096, 8192, },
  { 1, 4, 128, 256, 512, 4096, 8192, },
  { 2, 4, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 128, 256, 512, 4096, 8192, },
  { 8, 128, 256, 512, 4096, 8192, },
  { 1, 8, 128, 256, 512, 4096, 8192, },
  { 2, 8, 128, 256, 512, 4096, 8192, },
  { 1, 2, 8, 128, 256, 512, 4096, 8192, },
  { 4, 8, 128, 256, 512, 4096, 8192, },
  { 1, 4, 8, 128, 256, 512, 4096, 8192, },
  { 2, 4, 8, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 128, 256, 512, 4096, 8192, },
  { 16, 128, 256, 512, 4096, 8192, },
  { 1, 16, 128, 256, 512, 4096, 8192, },
  { 2, 16, 128, 256, 512, 4096, 8192, },
  { 1, 2, 16, 128, 256, 512, 4096, 8192, },
  { 4, 16, 128, 256, 512, 4096, 8192, },
  { 1, 4, 16, 128, 256, 512, 4096, 8192, },
  { 2, 4, 16, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 16, 128, 256, 512, 4096, 8192, },
  { 8, 16, 128, 256, 512, 4096, 8192, },
  { 1, 8, 16, 128, 256, 512, 4096, 8192, },
  { 2, 8, 16, 128, 256, 512, 4096, 8192, },
  { 1, 2, 8, 16, 128, 256, 512, 4096, 8192, },
  { 4, 8, 16, 128, 256, 512, 4096, 8192, },
  { 1, 4, 8, 16, 128, 256, 512, 4096, 8192, },
  { 2, 4, 8, 16, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 4096, 8192, },
  { 32, 128, 256, 512, 4096, 8192, },
  { 1, 32, 128, 256, 512, 4096, 8192, },
  { 2, 32, 128, 256, 512, 4096, 8192, },
  { 1, 2, 32, 128, 256, 512, 4096, 8192, },
  { 4, 32, 128, 256, 512, 4096, 8192, },
  { 1, 4, 32, 128, 256, 512, 4096, 8192, },
  { 2, 4, 32, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 32, 128, 256, 512, 4096, 8192, },
  { 8, 32, 128, 256, 512, 4096, 8192, },
  { 1, 8, 32, 128, 256, 512, 4096, 8192, },
  { 2, 8, 32, 128, 256, 512, 4096, 8192, },
  { 1, 2, 8, 32, 128, 256, 512, 4096, 8192, },
  { 4, 8, 32, 128, 256, 512, 4096, 8192, },
  { 1, 4, 8, 32, 128, 256, 512, 4096, 8192, },
  { 2, 4, 8, 32, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 4096, 8192, },
  { 16, 32, 128, 256, 512, 4096, 8192, },
  { 1, 16, 32, 128, 256, 512, 4096, 8192, },
  { 2, 16, 32, 128, 256, 512, 4096, 8192, },
  { 1, 2, 16, 32, 128, 256, 512, 4096, 8192, },
  { 4, 16, 32, 128, 256, 512, 4096, 8192, },
  { 1, 4, 16, 32, 128, 256, 512, 4096, 8192, },
  { 2, 4, 16, 32, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 4096, 8192, },
  { 8, 16, 32, 128, 256, 512, 4096, 8192, },
  { 1, 8, 16, 32, 128, 256, 512, 4096, 8192, },
  { 2, 8, 16, 32, 128, 256, 512, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 4096, 8192, },
  { 4, 8, 16, 32, 128, 256, 512, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, },
  { 64, 128, 256, 512, 4096, 8192, },
  { 1, 64, 128, 256, 512, 4096, 8192, },
  { 2, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 64, 128, 256, 512, 4096, 8192, },
  { 4, 64, 128, 256, 512, 4096, 8192, },
  { 1, 4, 64, 128, 256, 512, 4096, 8192, },
  { 2, 4, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 64, 128, 256, 512, 4096, 8192, },
  { 8, 64, 128, 256, 512, 4096, 8192, },
  { 1, 8, 64, 128, 256, 512, 4096, 8192, },
  { 2, 8, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 8, 64, 128, 256, 512, 4096, 8192, },
  { 4, 8, 64, 128, 256, 512, 4096, 8192, },
  { 1, 4, 8, 64, 128, 256, 512, 4096, 8192, },
  { 2, 4, 8, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 4096, 8192, },
  { 16, 64, 128, 256, 512, 4096, 8192, },
  { 1, 16, 64, 128, 256, 512, 4096, 8192, },
  { 2, 16, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 16, 64, 128, 256, 512, 4096, 8192, },
  { 4, 16, 64, 128, 256, 512, 4096, 8192, },
  { 1, 4, 16, 64, 128, 256, 512, 4096, 8192, },
  { 2, 4, 16, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 4096, 8192, },
  { 8, 16, 64, 128, 256, 512, 4096, 8192, },
  { 1, 8, 16, 64, 128, 256, 512, 4096, 8192, },
  { 2, 8, 16, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 4096, 8192, },
  { 4, 8, 16, 64, 128, 256, 512, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, },
  { 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 32, 64, 128, 256, 512, 4096, 8192, },
  { 2, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 32, 64, 128, 256, 512, 4096, 8192, },
  { 4, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 4, 32, 64, 128, 256, 512, 4096, 8192, },
  { 2, 4, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 4096, 8192, },
  { 8, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 8, 32, 64, 128, 256, 512, 4096, 8192, },
  { 2, 8, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 4096, 8192, },
  { 4, 8, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, },
  { 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 2, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 4, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 8, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, },
  { 1024, 4096, 8192, },
  { 1, 1024, 4096, 8192, },
  { 2, 1024, 4096, 8192, },
  { 1, 2, 1024, 4096, 8192, },
  { 4, 1024, 4096, 8192, },
  { 1, 4, 1024, 4096, 8192, },
  { 2, 4, 1024, 4096, 8192, },
  { 1, 2, 4, 1024, 4096, 8192, },
  { 8, 1024, 4096, 8192, },
  { 1, 8, 1024, 4096, 8192, },
  { 2, 8, 1024, 4096, 8192, },
  { 1, 2, 8, 1024, 4096, 8192, },
  { 4, 8, 1024, 4096, 8192, },
  { 1, 4, 8, 1024, 4096, 8192, },
  { 2, 4, 8, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 1024, 4096, 8192, },
  { 16, 1024, 4096, 8192, },
  { 1, 16, 1024, 4096, 8192, },
  { 2, 16, 1024, 4096, 8192, },
  { 1, 2, 16, 1024, 4096, 8192, },
  { 4, 16, 1024, 4096, 8192, },
  { 1, 4, 16, 1024, 4096, 8192, },
  { 2, 4, 16, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 1024, 4096, 8192, },
  { 8, 16, 1024, 4096, 8192, },
  { 1, 8, 16, 1024, 4096, 8192, },
  { 2, 8, 16, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 1024, 4096, 8192, },
  { 4, 8, 16, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 1024, 4096, 8192, },
  { 32, 1024, 4096, 8192, },
  { 1, 32, 1024, 4096, 8192, },
  { 2, 32, 1024, 4096, 8192, },
  { 1, 2, 32, 1024, 4096, 8192, },
  { 4, 32, 1024, 4096, 8192, },
  { 1, 4, 32, 1024, 4096, 8192, },
  { 2, 4, 32, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 1024, 4096, 8192, },
  { 8, 32, 1024, 4096, 8192, },
  { 1, 8, 32, 1024, 4096, 8192, },
  { 2, 8, 32, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 1024, 4096, 8192, },
  { 4, 8, 32, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 1024, 4096, 8192, },
  { 16, 32, 1024, 4096, 8192, },
  { 1, 16, 32, 1024, 4096, 8192, },
  { 2, 16, 32, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 1024, 4096, 8192, },
  { 4, 16, 32, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 1024, 4096, 8192, },
  { 8, 16, 32, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 1024, 4096, 8192, },
  { 64, 1024, 4096, 8192, },
  { 1, 64, 1024, 4096, 8192, },
  { 2, 64, 1024, 4096, 8192, },
  { 1, 2, 64, 1024, 4096, 8192, },
  { 4, 64, 1024, 4096, 8192, },
  { 1, 4, 64, 1024, 4096, 8192, },
  { 2, 4, 64, 1024, 4096, 8192, },
  { 1, 2, 4, 64, 1024, 4096, 8192, },
  { 8, 64, 1024, 4096, 8192, },
  { 1, 8, 64, 1024, 4096, 8192, },
  { 2, 8, 64, 1024, 4096, 8192, },
  { 1, 2, 8, 64, 1024, 4096, 8192, },
  { 4, 8, 64, 1024, 4096, 8192, },
  { 1, 4, 8, 64, 1024, 4096, 8192, },
  { 2, 4, 8, 64, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 64, 1024, 4096, 8192, },
  { 16, 64, 1024, 4096, 8192, },
  { 1, 16, 64, 1024, 4096, 8192, },
  { 2, 16, 64, 1024, 4096, 8192, },
  { 1, 2, 16, 64, 1024, 4096, 8192, },
  { 4, 16, 64, 1024, 4096, 8192, },
  { 1, 4, 16, 64, 1024, 4096, 8192, },
  { 2, 4, 16, 64, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 64, 1024, 4096, 8192, },
  { 8, 16, 64, 1024, 4096, 8192, },
  { 1, 8, 16, 64, 1024, 4096, 8192, },
  { 2, 8, 16, 64, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 64, 1024, 4096, 8192, },
  { 4, 8, 16, 64, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 64, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 64, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 1024, 4096, 8192, },
  { 32, 64, 1024, 4096, 8192, },
  { 1, 32, 64, 1024, 4096, 8192, },
  { 2, 32, 64, 1024, 4096, 8192, },
  { 1, 2, 32, 64, 1024, 4096, 8192, },
  { 4, 32, 64, 1024, 4096, 8192, },
  { 1, 4, 32, 64, 1024, 4096, 8192, },
  { 2, 4, 32, 64, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 64, 1024, 4096, 8192, },
  { 8, 32, 64, 1024, 4096, 8192, },
  { 1, 8, 32, 64, 1024, 4096, 8192, },
  { 2, 8, 32, 64, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 64, 1024, 4096, 8192, },
  { 4, 8, 32, 64, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 64, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 64, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 1024, 4096, 8192, },
  { 16, 32, 64, 1024, 4096, 8192, },
  { 1, 16, 32, 64, 1024, 4096, 8192, },
  { 2, 16, 32, 64, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 64, 1024, 4096, 8192, },
  { 4, 16, 32, 64, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 64, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 64, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 1024, 4096, 8192, },
  { 8, 16, 32, 64, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 64, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 64, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 64, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 4096, 8192, },
  { 128, 1024, 4096, 8192, },
  { 1, 128, 1024, 4096, 8192, },
  { 2, 128, 1024, 4096, 8192, },
  { 1, 2, 128, 1024, 4096, 8192, },
  { 4, 128, 1024, 4096, 8192, },
  { 1, 4, 128, 1024, 4096, 8192, },
  { 2, 4, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 128, 1024, 4096, 8192, },
  { 8, 128, 1024, 4096, 8192, },
  { 1, 8, 128, 1024, 4096, 8192, },
  { 2, 8, 128, 1024, 4096, 8192, },
  { 1, 2, 8, 128, 1024, 4096, 8192, },
  { 4, 8, 128, 1024, 4096, 8192, },
  { 1, 4, 8, 128, 1024, 4096, 8192, },
  { 2, 4, 8, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 128, 1024, 4096, 8192, },
  { 16, 128, 1024, 4096, 8192, },
  { 1, 16, 128, 1024, 4096, 8192, },
  { 2, 16, 128, 1024, 4096, 8192, },
  { 1, 2, 16, 128, 1024, 4096, 8192, },
  { 4, 16, 128, 1024, 4096, 8192, },
  { 1, 4, 16, 128, 1024, 4096, 8192, },
  { 2, 4, 16, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 128, 1024, 4096, 8192, },
  { 8, 16, 128, 1024, 4096, 8192, },
  { 1, 8, 16, 128, 1024, 4096, 8192, },
  { 2, 8, 16, 128, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 128, 1024, 4096, 8192, },
  { 4, 8, 16, 128, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 128, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 1024, 4096, 8192, },
  { 32, 128, 1024, 4096, 8192, },
  { 1, 32, 128, 1024, 4096, 8192, },
  { 2, 32, 128, 1024, 4096, 8192, },
  { 1, 2, 32, 128, 1024, 4096, 8192, },
  { 4, 32, 128, 1024, 4096, 8192, },
  { 1, 4, 32, 128, 1024, 4096, 8192, },
  { 2, 4, 32, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 128, 1024, 4096, 8192, },
  { 8, 32, 128, 1024, 4096, 8192, },
  { 1, 8, 32, 128, 1024, 4096, 8192, },
  { 2, 8, 32, 128, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 128, 1024, 4096, 8192, },
  { 4, 8, 32, 128, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 128, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 1024, 4096, 8192, },
  { 16, 32, 128, 1024, 4096, 8192, },
  { 1, 16, 32, 128, 1024, 4096, 8192, },
  { 2, 16, 32, 128, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 128, 1024, 4096, 8192, },
  { 4, 16, 32, 128, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 128, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 1024, 4096, 8192, },
  { 8, 16, 32, 128, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 128, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 128, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 128, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 4096, 8192, },
  { 64, 128, 1024, 4096, 8192, },
  { 1, 64, 128, 1024, 4096, 8192, },
  { 2, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 64, 128, 1024, 4096, 8192, },
  { 4, 64, 128, 1024, 4096, 8192, },
  { 1, 4, 64, 128, 1024, 4096, 8192, },
  { 2, 4, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 64, 128, 1024, 4096, 8192, },
  { 8, 64, 128, 1024, 4096, 8192, },
  { 1, 8, 64, 128, 1024, 4096, 8192, },
  { 2, 8, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 8, 64, 128, 1024, 4096, 8192, },
  { 4, 8, 64, 128, 1024, 4096, 8192, },
  { 1, 4, 8, 64, 128, 1024, 4096, 8192, },
  { 2, 4, 8, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 1024, 4096, 8192, },
  { 16, 64, 128, 1024, 4096, 8192, },
  { 1, 16, 64, 128, 1024, 4096, 8192, },
  { 2, 16, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 16, 64, 128, 1024, 4096, 8192, },
  { 4, 16, 64, 128, 1024, 4096, 8192, },
  { 1, 4, 16, 64, 128, 1024, 4096, 8192, },
  { 2, 4, 16, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 1024, 4096, 8192, },
  { 8, 16, 64, 128, 1024, 4096, 8192, },
  { 1, 8, 16, 64, 128, 1024, 4096, 8192, },
  { 2, 8, 16, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 1024, 4096, 8192, },
  { 4, 8, 16, 64, 128, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 4096, 8192, },
  { 32, 64, 128, 1024, 4096, 8192, },
  { 1, 32, 64, 128, 1024, 4096, 8192, },
  { 2, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 32, 64, 128, 1024, 4096, 8192, },
  { 4, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 4, 32, 64, 128, 1024, 4096, 8192, },
  { 2, 4, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 1024, 4096, 8192, },
  { 8, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 8, 32, 64, 128, 1024, 4096, 8192, },
  { 2, 8, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 1024, 4096, 8192, },
  { 4, 8, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 4096, 8192, },
  { 16, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 2, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 4, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 8, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, },
  { 256, 1024, 4096, 8192, },
  { 1, 256, 1024, 4096, 8192, },
  { 2, 256, 1024, 4096, 8192, },
  { 1, 2, 256, 1024, 4096, 8192, },
  { 4, 256, 1024, 4096, 8192, },
  { 1, 4, 256, 1024, 4096, 8192, },
  { 2, 4, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 256, 1024, 4096, 8192, },
  { 8, 256, 1024, 4096, 8192, },
  { 1, 8, 256, 1024, 4096, 8192, },
  { 2, 8, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 256, 1024, 4096, 8192, },
  { 4, 8, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 256, 1024, 4096, 8192, },
  { 16, 256, 1024, 4096, 8192, },
  { 1, 16, 256, 1024, 4096, 8192, },
  { 2, 16, 256, 1024, 4096, 8192, },
  { 1, 2, 16, 256, 1024, 4096, 8192, },
  { 4, 16, 256, 1024, 4096, 8192, },
  { 1, 4, 16, 256, 1024, 4096, 8192, },
  { 2, 4, 16, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 256, 1024, 4096, 8192, },
  { 8, 16, 256, 1024, 4096, 8192, },
  { 1, 8, 16, 256, 1024, 4096, 8192, },
  { 2, 8, 16, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 256, 1024, 4096, 8192, },
  { 4, 8, 16, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 256, 1024, 4096, 8192, },
  { 32, 256, 1024, 4096, 8192, },
  { 1, 32, 256, 1024, 4096, 8192, },
  { 2, 32, 256, 1024, 4096, 8192, },
  { 1, 2, 32, 256, 1024, 4096, 8192, },
  { 4, 32, 256, 1024, 4096, 8192, },
  { 1, 4, 32, 256, 1024, 4096, 8192, },
  { 2, 4, 32, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 256, 1024, 4096, 8192, },
  { 8, 32, 256, 1024, 4096, 8192, },
  { 1, 8, 32, 256, 1024, 4096, 8192, },
  { 2, 8, 32, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 256, 1024, 4096, 8192, },
  { 4, 8, 32, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 256, 1024, 4096, 8192, },
  { 16, 32, 256, 1024, 4096, 8192, },
  { 1, 16, 32, 256, 1024, 4096, 8192, },
  { 2, 16, 32, 256, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 256, 1024, 4096, 8192, },
  { 4, 16, 32, 256, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 256, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 256, 1024, 4096, 8192, },
  { 8, 16, 32, 256, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 256, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 256, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 4096, 8192, },
  { 64, 256, 1024, 4096, 8192, },
  { 1, 64, 256, 1024, 4096, 8192, },
  { 2, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 64, 256, 1024, 4096, 8192, },
  { 4, 64, 256, 1024, 4096, 8192, },
  { 1, 4, 64, 256, 1024, 4096, 8192, },
  { 2, 4, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 64, 256, 1024, 4096, 8192, },
  { 8, 64, 256, 1024, 4096, 8192, },
  { 1, 8, 64, 256, 1024, 4096, 8192, },
  { 2, 8, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 64, 256, 1024, 4096, 8192, },
  { 4, 8, 64, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 64, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 64, 256, 1024, 4096, 8192, },
  { 16, 64, 256, 1024, 4096, 8192, },
  { 1, 16, 64, 256, 1024, 4096, 8192, },
  { 2, 16, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 16, 64, 256, 1024, 4096, 8192, },
  { 4, 16, 64, 256, 1024, 4096, 8192, },
  { 1, 4, 16, 64, 256, 1024, 4096, 8192, },
  { 2, 4, 16, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 64, 256, 1024, 4096, 8192, },
  { 8, 16, 64, 256, 1024, 4096, 8192, },
  { 1, 8, 16, 64, 256, 1024, 4096, 8192, },
  { 2, 8, 16, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 64, 256, 1024, 4096, 8192, },
  { 4, 8, 16, 64, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 64, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 4096, 8192, },
  { 32, 64, 256, 1024, 4096, 8192, },
  { 1, 32, 64, 256, 1024, 4096, 8192, },
  { 2, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 32, 64, 256, 1024, 4096, 8192, },
  { 4, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 4, 32, 64, 256, 1024, 4096, 8192, },
  { 2, 4, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 64, 256, 1024, 4096, 8192, },
  { 8, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 8, 32, 64, 256, 1024, 4096, 8192, },
  { 2, 8, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 64, 256, 1024, 4096, 8192, },
  { 4, 8, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 64, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 4096, 8192, },
  { 16, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 2, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 4, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 8, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, },
  { 128, 256, 1024, 4096, 8192, },
  { 1, 128, 256, 1024, 4096, 8192, },
  { 2, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 128, 256, 1024, 4096, 8192, },
  { 4, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 128, 256, 1024, 4096, 8192, },
  { 8, 128, 256, 1024, 4096, 8192, },
  { 1, 8, 128, 256, 1024, 4096, 8192, },
  { 2, 8, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 128, 256, 1024, 4096, 8192, },
  { 4, 8, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 128, 256, 1024, 4096, 8192, },
  { 16, 128, 256, 1024, 4096, 8192, },
  { 1, 16, 128, 256, 1024, 4096, 8192, },
  { 2, 16, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 16, 128, 256, 1024, 4096, 8192, },
  { 4, 16, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 16, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 16, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 128, 256, 1024, 4096, 8192, },
  { 8, 16, 128, 256, 1024, 4096, 8192, },
  { 1, 8, 16, 128, 256, 1024, 4096, 8192, },
  { 2, 8, 16, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 128, 256, 1024, 4096, 8192, },
  { 4, 8, 16, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 4096, 8192, },
  { 32, 128, 256, 1024, 4096, 8192, },
  { 1, 32, 128, 256, 1024, 4096, 8192, },
  { 2, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 32, 128, 256, 1024, 4096, 8192, },
  { 4, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 32, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 128, 256, 1024, 4096, 8192, },
  { 8, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 8, 32, 128, 256, 1024, 4096, 8192, },
  { 2, 8, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 128, 256, 1024, 4096, 8192, },
  { 4, 8, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 4096, 8192, },
  { 16, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 2, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 4, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 8, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, },
  { 64, 128, 256, 1024, 4096, 8192, },
  { 1, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 64, 128, 256, 1024, 4096, 8192, },
  { 4, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 64, 128, 256, 1024, 4096, 8192, },
  { 8, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 8, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 8, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 64, 128, 256, 1024, 4096, 8192, },
  { 4, 8, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 4096, 8192, },
  { 16, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 4, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 8, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 8, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 8, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, },
  { 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 4, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 8, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 8, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 8, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, },
  { 512, 1024, 4096, 8192, },
  { 1, 512, 1024, 4096, 8192, },
  { 2, 512, 1024, 4096, 8192, },
  { 1, 2, 512, 1024, 4096, 8192, },
  { 4, 512, 1024, 4096, 8192, },
  { 1, 4, 512, 1024, 4096, 8192, },
  { 2, 4, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 512, 1024, 4096, 8192, },
  { 8, 512, 1024, 4096, 8192, },
  { 1, 8, 512, 1024, 4096, 8192, },
  { 2, 8, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 512, 1024, 4096, 8192, },
  { 4, 8, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 512, 1024, 4096, 8192, },
  { 16, 512, 1024, 4096, 8192, },
  { 1, 16, 512, 1024, 4096, 8192, },
  { 2, 16, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 512, 1024, 4096, 8192, },
  { 4, 16, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 512, 1024, 4096, 8192, },
  { 8, 16, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 512, 1024, 4096, 8192, },
  { 32, 512, 1024, 4096, 8192, },
  { 1, 32, 512, 1024, 4096, 8192, },
  { 2, 32, 512, 1024, 4096, 8192, },
  { 1, 2, 32, 512, 1024, 4096, 8192, },
  { 4, 32, 512, 1024, 4096, 8192, },
  { 1, 4, 32, 512, 1024, 4096, 8192, },
  { 2, 4, 32, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 512, 1024, 4096, 8192, },
  { 8, 32, 512, 1024, 4096, 8192, },
  { 1, 8, 32, 512, 1024, 4096, 8192, },
  { 2, 8, 32, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 512, 1024, 4096, 8192, },
  { 4, 8, 32, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 512, 1024, 4096, 8192, },
  { 16, 32, 512, 1024, 4096, 8192, },
  { 1, 16, 32, 512, 1024, 4096, 8192, },
  { 2, 16, 32, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 512, 1024, 4096, 8192, },
  { 4, 16, 32, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 512, 1024, 4096, 8192, },
  { 8, 16, 32, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 4096, 8192, },
  { 64, 512, 1024, 4096, 8192, },
  { 1, 64, 512, 1024, 4096, 8192, },
  { 2, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 64, 512, 1024, 4096, 8192, },
  { 4, 64, 512, 1024, 4096, 8192, },
  { 1, 4, 64, 512, 1024, 4096, 8192, },
  { 2, 4, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 64, 512, 1024, 4096, 8192, },
  { 8, 64, 512, 1024, 4096, 8192, },
  { 1, 8, 64, 512, 1024, 4096, 8192, },
  { 2, 8, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 64, 512, 1024, 4096, 8192, },
  { 4, 8, 64, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 64, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 64, 512, 1024, 4096, 8192, },
  { 16, 64, 512, 1024, 4096, 8192, },
  { 1, 16, 64, 512, 1024, 4096, 8192, },
  { 2, 16, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 64, 512, 1024, 4096, 8192, },
  { 4, 16, 64, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 64, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 64, 512, 1024, 4096, 8192, },
  { 8, 16, 64, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 64, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 64, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 64, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 64, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 4096, 8192, },
  { 32, 64, 512, 1024, 4096, 8192, },
  { 1, 32, 64, 512, 1024, 4096, 8192, },
  { 2, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 32, 64, 512, 1024, 4096, 8192, },
  { 4, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 4, 32, 64, 512, 1024, 4096, 8192, },
  { 2, 4, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 64, 512, 1024, 4096, 8192, },
  { 8, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 8, 32, 64, 512, 1024, 4096, 8192, },
  { 2, 8, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 64, 512, 1024, 4096, 8192, },
  { 4, 8, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 64, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 4096, 8192, },
  { 16, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 2, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 4, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 8, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, },
  { 128, 512, 1024, 4096, 8192, },
  { 1, 128, 512, 1024, 4096, 8192, },
  { 2, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 128, 512, 1024, 4096, 8192, },
  { 4, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 128, 512, 1024, 4096, 8192, },
  { 8, 128, 512, 1024, 4096, 8192, },
  { 1, 8, 128, 512, 1024, 4096, 8192, },
  { 2, 8, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 128, 512, 1024, 4096, 8192, },
  { 4, 8, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 128, 512, 1024, 4096, 8192, },
  { 16, 128, 512, 1024, 4096, 8192, },
  { 1, 16, 128, 512, 1024, 4096, 8192, },
  { 2, 16, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 128, 512, 1024, 4096, 8192, },
  { 4, 16, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 128, 512, 1024, 4096, 8192, },
  { 8, 16, 128, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 128, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 128, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 4096, 8192, },
  { 32, 128, 512, 1024, 4096, 8192, },
  { 1, 32, 128, 512, 1024, 4096, 8192, },
  { 2, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 32, 128, 512, 1024, 4096, 8192, },
  { 4, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 32, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 128, 512, 1024, 4096, 8192, },
  { 8, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 8, 32, 128, 512, 1024, 4096, 8192, },
  { 2, 8, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 128, 512, 1024, 4096, 8192, },
  { 4, 8, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 4096, 8192, },
  { 16, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 2, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 4, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 8, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, },
  { 64, 128, 512, 1024, 4096, 8192, },
  { 1, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 64, 128, 512, 1024, 4096, 8192, },
  { 4, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 64, 128, 512, 1024, 4096, 8192, },
  { 8, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 8, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 8, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 64, 128, 512, 1024, 4096, 8192, },
  { 4, 8, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 4096, 8192, },
  { 16, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 4, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 8, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, },
  { 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 4, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 8, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 8, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 8, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, },
  { 256, 512, 1024, 4096, 8192, },
  { 1, 256, 512, 1024, 4096, 8192, },
  { 2, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 256, 512, 1024, 4096, 8192, },
  { 4, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 256, 512, 1024, 4096, 8192, },
  { 8, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 256, 512, 1024, 4096, 8192, },
  { 16, 256, 512, 1024, 4096, 8192, },
  { 1, 16, 256, 512, 1024, 4096, 8192, },
  { 2, 16, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 256, 512, 1024, 4096, 8192, },
  { 4, 16, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 256, 512, 1024, 4096, 8192, },
  { 8, 16, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 4096, 8192, },
  { 32, 256, 512, 1024, 4096, 8192, },
  { 1, 32, 256, 512, 1024, 4096, 8192, },
  { 2, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 32, 256, 512, 1024, 4096, 8192, },
  { 4, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 32, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 256, 512, 1024, 4096, 8192, },
  { 8, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 32, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 4096, 8192, },
  { 16, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 2, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 4, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 8, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, },
  { 64, 256, 512, 1024, 4096, 8192, },
  { 1, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 64, 256, 512, 1024, 4096, 8192, },
  { 4, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 64, 256, 512, 1024, 4096, 8192, },
  { 8, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 64, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 4096, 8192, },
  { 16, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 4, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 8, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, },
  { 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 4, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 8, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, },
  { 128, 256, 512, 1024, 4096, 8192, },
  { 1, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 128, 256, 512, 1024, 4096, 8192, },
  { 8, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 4096, 8192, },
  { 16, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 8, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, },
  { 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 8, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, },
  { 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 8, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, },
  { 2048, 4096, 8192, },
  { 1, 2048, 4096, 8192, },
  { 2, 2048, 4096, 8192, },
  { 1, 2, 2048, 4096, 8192, },
  { 4, 2048, 4096, 8192, },
  { 1, 4, 2048, 4096, 8192, },
  { 2, 4, 2048, 4096, 8192, },
  { 1, 2, 4, 2048, 4096, 8192, },
  { 8, 2048, 4096, 8192, },
  { 1, 8, 2048, 4096, 8192, },
  { 2, 8, 2048, 4096, 8192, },
  { 1, 2, 8, 2048, 4096, 8192, },
  { 4, 8, 2048, 4096, 8192, },
  { 1, 4, 8, 2048, 4096, 8192, },
  { 2, 4, 8, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 2048, 4096, 8192, },
  { 16, 2048, 4096, 8192, },
  { 1, 16, 2048, 4096, 8192, },
  { 2, 16, 2048, 4096, 8192, },
  { 1, 2, 16, 2048, 4096, 8192, },
  { 4, 16, 2048, 4096, 8192, },
  { 1, 4, 16, 2048, 4096, 8192, },
  { 2, 4, 16, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 2048, 4096, 8192, },
  { 8, 16, 2048, 4096, 8192, },
  { 1, 8, 16, 2048, 4096, 8192, },
  { 2, 8, 16, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 2048, 4096, 8192, },
  { 4, 8, 16, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 2048, 4096, 8192, },
  { 32, 2048, 4096, 8192, },
  { 1, 32, 2048, 4096, 8192, },
  { 2, 32, 2048, 4096, 8192, },
  { 1, 2, 32, 2048, 4096, 8192, },
  { 4, 32, 2048, 4096, 8192, },
  { 1, 4, 32, 2048, 4096, 8192, },
  { 2, 4, 32, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 2048, 4096, 8192, },
  { 8, 32, 2048, 4096, 8192, },
  { 1, 8, 32, 2048, 4096, 8192, },
  { 2, 8, 32, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 2048, 4096, 8192, },
  { 4, 8, 32, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 2048, 4096, 8192, },
  { 16, 32, 2048, 4096, 8192, },
  { 1, 16, 32, 2048, 4096, 8192, },
  { 2, 16, 32, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 2048, 4096, 8192, },
  { 4, 16, 32, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 2048, 4096, 8192, },
  { 8, 16, 32, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 2048, 4096, 8192, },
  { 64, 2048, 4096, 8192, },
  { 1, 64, 2048, 4096, 8192, },
  { 2, 64, 2048, 4096, 8192, },
  { 1, 2, 64, 2048, 4096, 8192, },
  { 4, 64, 2048, 4096, 8192, },
  { 1, 4, 64, 2048, 4096, 8192, },
  { 2, 4, 64, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 2048, 4096, 8192, },
  { 8, 64, 2048, 4096, 8192, },
  { 1, 8, 64, 2048, 4096, 8192, },
  { 2, 8, 64, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 2048, 4096, 8192, },
  { 4, 8, 64, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 2048, 4096, 8192, },
  { 16, 64, 2048, 4096, 8192, },
  { 1, 16, 64, 2048, 4096, 8192, },
  { 2, 16, 64, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 2048, 4096, 8192, },
  { 4, 16, 64, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 2048, 4096, 8192, },
  { 8, 16, 64, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 2048, 4096, 8192, },
  { 32, 64, 2048, 4096, 8192, },
  { 1, 32, 64, 2048, 4096, 8192, },
  { 2, 32, 64, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 2048, 4096, 8192, },
  { 4, 32, 64, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 2048, 4096, 8192, },
  { 8, 32, 64, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 2048, 4096, 8192, },
  { 16, 32, 64, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 4096, 8192, },
  { 128, 2048, 4096, 8192, },
  { 1, 128, 2048, 4096, 8192, },
  { 2, 128, 2048, 4096, 8192, },
  { 1, 2, 128, 2048, 4096, 8192, },
  { 4, 128, 2048, 4096, 8192, },
  { 1, 4, 128, 2048, 4096, 8192, },
  { 2, 4, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 128, 2048, 4096, 8192, },
  { 8, 128, 2048, 4096, 8192, },
  { 1, 8, 128, 2048, 4096, 8192, },
  { 2, 8, 128, 2048, 4096, 8192, },
  { 1, 2, 8, 128, 2048, 4096, 8192, },
  { 4, 8, 128, 2048, 4096, 8192, },
  { 1, 4, 8, 128, 2048, 4096, 8192, },
  { 2, 4, 8, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 128, 2048, 4096, 8192, },
  { 16, 128, 2048, 4096, 8192, },
  { 1, 16, 128, 2048, 4096, 8192, },
  { 2, 16, 128, 2048, 4096, 8192, },
  { 1, 2, 16, 128, 2048, 4096, 8192, },
  { 4, 16, 128, 2048, 4096, 8192, },
  { 1, 4, 16, 128, 2048, 4096, 8192, },
  { 2, 4, 16, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 128, 2048, 4096, 8192, },
  { 8, 16, 128, 2048, 4096, 8192, },
  { 1, 8, 16, 128, 2048, 4096, 8192, },
  { 2, 8, 16, 128, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 128, 2048, 4096, 8192, },
  { 4, 8, 16, 128, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 128, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 2048, 4096, 8192, },
  { 32, 128, 2048, 4096, 8192, },
  { 1, 32, 128, 2048, 4096, 8192, },
  { 2, 32, 128, 2048, 4096, 8192, },
  { 1, 2, 32, 128, 2048, 4096, 8192, },
  { 4, 32, 128, 2048, 4096, 8192, },
  { 1, 4, 32, 128, 2048, 4096, 8192, },
  { 2, 4, 32, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 128, 2048, 4096, 8192, },
  { 8, 32, 128, 2048, 4096, 8192, },
  { 1, 8, 32, 128, 2048, 4096, 8192, },
  { 2, 8, 32, 128, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 128, 2048, 4096, 8192, },
  { 4, 8, 32, 128, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 128, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 2048, 4096, 8192, },
  { 16, 32, 128, 2048, 4096, 8192, },
  { 1, 16, 32, 128, 2048, 4096, 8192, },
  { 2, 16, 32, 128, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 128, 2048, 4096, 8192, },
  { 4, 16, 32, 128, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 128, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 2048, 4096, 8192, },
  { 8, 16, 32, 128, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 128, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 128, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 128, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 4096, 8192, },
  { 64, 128, 2048, 4096, 8192, },
  { 1, 64, 128, 2048, 4096, 8192, },
  { 2, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 64, 128, 2048, 4096, 8192, },
  { 4, 64, 128, 2048, 4096, 8192, },
  { 1, 4, 64, 128, 2048, 4096, 8192, },
  { 2, 4, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 128, 2048, 4096, 8192, },
  { 8, 64, 128, 2048, 4096, 8192, },
  { 1, 8, 64, 128, 2048, 4096, 8192, },
  { 2, 8, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 128, 2048, 4096, 8192, },
  { 4, 8, 64, 128, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 128, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 2048, 4096, 8192, },
  { 16, 64, 128, 2048, 4096, 8192, },
  { 1, 16, 64, 128, 2048, 4096, 8192, },
  { 2, 16, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 128, 2048, 4096, 8192, },
  { 4, 16, 64, 128, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 128, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 2048, 4096, 8192, },
  { 8, 16, 64, 128, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 128, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 128, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 4096, 8192, },
  { 32, 64, 128, 2048, 4096, 8192, },
  { 1, 32, 64, 128, 2048, 4096, 8192, },
  { 2, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 128, 2048, 4096, 8192, },
  { 4, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 128, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 2048, 4096, 8192, },
  { 8, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 128, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 4096, 8192, },
  { 16, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, },
  { 256, 2048, 4096, 8192, },
  { 1, 256, 2048, 4096, 8192, },
  { 2, 256, 2048, 4096, 8192, },
  { 1, 2, 256, 2048, 4096, 8192, },
  { 4, 256, 2048, 4096, 8192, },
  { 1, 4, 256, 2048, 4096, 8192, },
  { 2, 4, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 256, 2048, 4096, 8192, },
  { 8, 256, 2048, 4096, 8192, },
  { 1, 8, 256, 2048, 4096, 8192, },
  { 2, 8, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 256, 2048, 4096, 8192, },
  { 4, 8, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 256, 2048, 4096, 8192, },
  { 16, 256, 2048, 4096, 8192, },
  { 1, 16, 256, 2048, 4096, 8192, },
  { 2, 16, 256, 2048, 4096, 8192, },
  { 1, 2, 16, 256, 2048, 4096, 8192, },
  { 4, 16, 256, 2048, 4096, 8192, },
  { 1, 4, 16, 256, 2048, 4096, 8192, },
  { 2, 4, 16, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 256, 2048, 4096, 8192, },
  { 8, 16, 256, 2048, 4096, 8192, },
  { 1, 8, 16, 256, 2048, 4096, 8192, },
  { 2, 8, 16, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 256, 2048, 4096, 8192, },
  { 4, 8, 16, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 256, 2048, 4096, 8192, },
  { 32, 256, 2048, 4096, 8192, },
  { 1, 32, 256, 2048, 4096, 8192, },
  { 2, 32, 256, 2048, 4096, 8192, },
  { 1, 2, 32, 256, 2048, 4096, 8192, },
  { 4, 32, 256, 2048, 4096, 8192, },
  { 1, 4, 32, 256, 2048, 4096, 8192, },
  { 2, 4, 32, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 256, 2048, 4096, 8192, },
  { 8, 32, 256, 2048, 4096, 8192, },
  { 1, 8, 32, 256, 2048, 4096, 8192, },
  { 2, 8, 32, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 256, 2048, 4096, 8192, },
  { 4, 8, 32, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 256, 2048, 4096, 8192, },
  { 16, 32, 256, 2048, 4096, 8192, },
  { 1, 16, 32, 256, 2048, 4096, 8192, },
  { 2, 16, 32, 256, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 256, 2048, 4096, 8192, },
  { 4, 16, 32, 256, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 256, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 256, 2048, 4096, 8192, },
  { 8, 16, 32, 256, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 256, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 256, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 4096, 8192, },
  { 64, 256, 2048, 4096, 8192, },
  { 1, 64, 256, 2048, 4096, 8192, },
  { 2, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 64, 256, 2048, 4096, 8192, },
  { 4, 64, 256, 2048, 4096, 8192, },
  { 1, 4, 64, 256, 2048, 4096, 8192, },
  { 2, 4, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 256, 2048, 4096, 8192, },
  { 8, 64, 256, 2048, 4096, 8192, },
  { 1, 8, 64, 256, 2048, 4096, 8192, },
  { 2, 8, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 256, 2048, 4096, 8192, },
  { 4, 8, 64, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 256, 2048, 4096, 8192, },
  { 16, 64, 256, 2048, 4096, 8192, },
  { 1, 16, 64, 256, 2048, 4096, 8192, },
  { 2, 16, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 256, 2048, 4096, 8192, },
  { 4, 16, 64, 256, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 256, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 256, 2048, 4096, 8192, },
  { 8, 16, 64, 256, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 256, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 256, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 4096, 8192, },
  { 32, 64, 256, 2048, 4096, 8192, },
  { 1, 32, 64, 256, 2048, 4096, 8192, },
  { 2, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 256, 2048, 4096, 8192, },
  { 4, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 256, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 256, 2048, 4096, 8192, },
  { 8, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 256, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 256, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 4096, 8192, },
  { 16, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, },
  { 128, 256, 2048, 4096, 8192, },
  { 1, 128, 256, 2048, 4096, 8192, },
  { 2, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 128, 256, 2048, 4096, 8192, },
  { 4, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 128, 256, 2048, 4096, 8192, },
  { 8, 128, 256, 2048, 4096, 8192, },
  { 1, 8, 128, 256, 2048, 4096, 8192, },
  { 2, 8, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 128, 256, 2048, 4096, 8192, },
  { 4, 8, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 128, 256, 2048, 4096, 8192, },
  { 16, 128, 256, 2048, 4096, 8192, },
  { 1, 16, 128, 256, 2048, 4096, 8192, },
  { 2, 16, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 16, 128, 256, 2048, 4096, 8192, },
  { 4, 16, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 16, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 16, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 128, 256, 2048, 4096, 8192, },
  { 8, 16, 128, 256, 2048, 4096, 8192, },
  { 1, 8, 16, 128, 256, 2048, 4096, 8192, },
  { 2, 8, 16, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 128, 256, 2048, 4096, 8192, },
  { 4, 8, 16, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 4096, 8192, },
  { 32, 128, 256, 2048, 4096, 8192, },
  { 1, 32, 128, 256, 2048, 4096, 8192, },
  { 2, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 32, 128, 256, 2048, 4096, 8192, },
  { 4, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 32, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 128, 256, 2048, 4096, 8192, },
  { 8, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 8, 32, 128, 256, 2048, 4096, 8192, },
  { 2, 8, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 128, 256, 2048, 4096, 8192, },
  { 4, 8, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 4096, 8192, },
  { 16, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 2, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 4, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 8, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, },
  { 64, 128, 256, 2048, 4096, 8192, },
  { 1, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 64, 128, 256, 2048, 4096, 8192, },
  { 4, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 128, 256, 2048, 4096, 8192, },
  { 8, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 8, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 8, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 128, 256, 2048, 4096, 8192, },
  { 4, 8, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 4096, 8192, },
  { 16, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 4, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 8, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, },
  { 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 4, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 8, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, },
  { 512, 2048, 4096, 8192, },
  { 1, 512, 2048, 4096, 8192, },
  { 2, 512, 2048, 4096, 8192, },
  { 1, 2, 512, 2048, 4096, 8192, },
  { 4, 512, 2048, 4096, 8192, },
  { 1, 4, 512, 2048, 4096, 8192, },
  { 2, 4, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 512, 2048, 4096, 8192, },
  { 8, 512, 2048, 4096, 8192, },
  { 1, 8, 512, 2048, 4096, 8192, },
  { 2, 8, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 512, 2048, 4096, 8192, },
  { 4, 8, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 512, 2048, 4096, 8192, },
  { 16, 512, 2048, 4096, 8192, },
  { 1, 16, 512, 2048, 4096, 8192, },
  { 2, 16, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 512, 2048, 4096, 8192, },
  { 4, 16, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 512, 2048, 4096, 8192, },
  { 8, 16, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 512, 2048, 4096, 8192, },
  { 32, 512, 2048, 4096, 8192, },
  { 1, 32, 512, 2048, 4096, 8192, },
  { 2, 32, 512, 2048, 4096, 8192, },
  { 1, 2, 32, 512, 2048, 4096, 8192, },
  { 4, 32, 512, 2048, 4096, 8192, },
  { 1, 4, 32, 512, 2048, 4096, 8192, },
  { 2, 4, 32, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 512, 2048, 4096, 8192, },
  { 8, 32, 512, 2048, 4096, 8192, },
  { 1, 8, 32, 512, 2048, 4096, 8192, },
  { 2, 8, 32, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 512, 2048, 4096, 8192, },
  { 4, 8, 32, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 512, 2048, 4096, 8192, },
  { 16, 32, 512, 2048, 4096, 8192, },
  { 1, 16, 32, 512, 2048, 4096, 8192, },
  { 2, 16, 32, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 512, 2048, 4096, 8192, },
  { 4, 16, 32, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 512, 2048, 4096, 8192, },
  { 8, 16, 32, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 4096, 8192, },
  { 64, 512, 2048, 4096, 8192, },
  { 1, 64, 512, 2048, 4096, 8192, },
  { 2, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 64, 512, 2048, 4096, 8192, },
  { 4, 64, 512, 2048, 4096, 8192, },
  { 1, 4, 64, 512, 2048, 4096, 8192, },
  { 2, 4, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 512, 2048, 4096, 8192, },
  { 8, 64, 512, 2048, 4096, 8192, },
  { 1, 8, 64, 512, 2048, 4096, 8192, },
  { 2, 8, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 512, 2048, 4096, 8192, },
  { 4, 8, 64, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 512, 2048, 4096, 8192, },
  { 16, 64, 512, 2048, 4096, 8192, },
  { 1, 16, 64, 512, 2048, 4096, 8192, },
  { 2, 16, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 512, 2048, 4096, 8192, },
  { 4, 16, 64, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 512, 2048, 4096, 8192, },
  { 8, 16, 64, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 4096, 8192, },
  { 32, 64, 512, 2048, 4096, 8192, },
  { 1, 32, 64, 512, 2048, 4096, 8192, },
  { 2, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 512, 2048, 4096, 8192, },
  { 4, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 512, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 512, 2048, 4096, 8192, },
  { 8, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 512, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 512, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 4096, 8192, },
  { 16, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, },
  { 128, 512, 2048, 4096, 8192, },
  { 1, 128, 512, 2048, 4096, 8192, },
  { 2, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 128, 512, 2048, 4096, 8192, },
  { 4, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 128, 512, 2048, 4096, 8192, },
  { 8, 128, 512, 2048, 4096, 8192, },
  { 1, 8, 128, 512, 2048, 4096, 8192, },
  { 2, 8, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 128, 512, 2048, 4096, 8192, },
  { 4, 8, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 128, 512, 2048, 4096, 8192, },
  { 16, 128, 512, 2048, 4096, 8192, },
  { 1, 16, 128, 512, 2048, 4096, 8192, },
  { 2, 16, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 128, 512, 2048, 4096, 8192, },
  { 4, 16, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 128, 512, 2048, 4096, 8192, },
  { 8, 16, 128, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 128, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 128, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 4096, 8192, },
  { 32, 128, 512, 2048, 4096, 8192, },
  { 1, 32, 128, 512, 2048, 4096, 8192, },
  { 2, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 32, 128, 512, 2048, 4096, 8192, },
  { 4, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 32, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 128, 512, 2048, 4096, 8192, },
  { 8, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 8, 32, 128, 512, 2048, 4096, 8192, },
  { 2, 8, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 128, 512, 2048, 4096, 8192, },
  { 4, 8, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 4096, 8192, },
  { 16, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 2, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 4, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 8, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, },
  { 64, 128, 512, 2048, 4096, 8192, },
  { 1, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 64, 128, 512, 2048, 4096, 8192, },
  { 4, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 128, 512, 2048, 4096, 8192, },
  { 8, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 8, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 8, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 128, 512, 2048, 4096, 8192, },
  { 4, 8, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 4096, 8192, },
  { 16, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 4, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 8, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, },
  { 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 4, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 8, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, },
  { 256, 512, 2048, 4096, 8192, },
  { 1, 256, 512, 2048, 4096, 8192, },
  { 2, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 256, 512, 2048, 4096, 8192, },
  { 4, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 256, 512, 2048, 4096, 8192, },
  { 8, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 256, 512, 2048, 4096, 8192, },
  { 16, 256, 512, 2048, 4096, 8192, },
  { 1, 16, 256, 512, 2048, 4096, 8192, },
  { 2, 16, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 256, 512, 2048, 4096, 8192, },
  { 4, 16, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 256, 512, 2048, 4096, 8192, },
  { 8, 16, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 4096, 8192, },
  { 32, 256, 512, 2048, 4096, 8192, },
  { 1, 32, 256, 512, 2048, 4096, 8192, },
  { 2, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 32, 256, 512, 2048, 4096, 8192, },
  { 4, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 32, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 256, 512, 2048, 4096, 8192, },
  { 8, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 32, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 4096, 8192, },
  { 16, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 2, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 4, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 8, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, },
  { 64, 256, 512, 2048, 4096, 8192, },
  { 1, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 64, 256, 512, 2048, 4096, 8192, },
  { 4, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 256, 512, 2048, 4096, 8192, },
  { 8, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 4096, 8192, },
  { 16, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 4, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 8, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, },
  { 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 4, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 8, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, },
  { 128, 256, 512, 2048, 4096, 8192, },
  { 1, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 128, 256, 512, 2048, 4096, 8192, },
  { 8, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 4096, 8192, },
  { 16, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 8, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, },
  { 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 8, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, },
  { 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 8, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, },
  { 1024, 2048, 4096, 8192, },
  { 1, 1024, 2048, 4096, 8192, },
  { 2, 1024, 2048, 4096, 8192, },
  { 1, 2, 1024, 2048, 4096, 8192, },
  { 4, 1024, 2048, 4096, 8192, },
  { 1, 4, 1024, 2048, 4096, 8192, },
  { 2, 4, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 1024, 2048, 4096, 8192, },
  { 8, 1024, 2048, 4096, 8192, },
  { 1, 8, 1024, 2048, 4096, 8192, },
  { 2, 8, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 1024, 2048, 4096, 8192, },
  { 4, 8, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 1024, 2048, 4096, 8192, },
  { 16, 1024, 2048, 4096, 8192, },
  { 1, 16, 1024, 2048, 4096, 8192, },
  { 2, 16, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 1024, 2048, 4096, 8192, },
  { 4, 16, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 1024, 2048, 4096, 8192, },
  { 8, 16, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 1024, 2048, 4096, 8192, },
  { 32, 1024, 2048, 4096, 8192, },
  { 1, 32, 1024, 2048, 4096, 8192, },
  { 2, 32, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 1024, 2048, 4096, 8192, },
  { 4, 32, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 1024, 2048, 4096, 8192, },
  { 8, 32, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 1024, 2048, 4096, 8192, },
  { 16, 32, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 4096, 8192, },
  { 64, 1024, 2048, 4096, 8192, },
  { 1, 64, 1024, 2048, 4096, 8192, },
  { 2, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 64, 1024, 2048, 4096, 8192, },
  { 4, 64, 1024, 2048, 4096, 8192, },
  { 1, 4, 64, 1024, 2048, 4096, 8192, },
  { 2, 4, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 1024, 2048, 4096, 8192, },
  { 8, 64, 1024, 2048, 4096, 8192, },
  { 1, 8, 64, 1024, 2048, 4096, 8192, },
  { 2, 8, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 1024, 2048, 4096, 8192, },
  { 4, 8, 64, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 1024, 2048, 4096, 8192, },
  { 16, 64, 1024, 2048, 4096, 8192, },
  { 1, 16, 64, 1024, 2048, 4096, 8192, },
  { 2, 16, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 1024, 2048, 4096, 8192, },
  { 4, 16, 64, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 1024, 2048, 4096, 8192, },
  { 8, 16, 64, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 4096, 8192, },
  { 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 32, 64, 1024, 2048, 4096, 8192, },
  { 2, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 1024, 2048, 4096, 8192, },
  { 4, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 1024, 2048, 4096, 8192, },
  { 8, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 4096, 8192, },
  { 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, },
  { 128, 1024, 2048, 4096, 8192, },
  { 1, 128, 1024, 2048, 4096, 8192, },
  { 2, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 128, 1024, 2048, 4096, 8192, },
  { 4, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 128, 1024, 2048, 4096, 8192, },
  { 8, 128, 1024, 2048, 4096, 8192, },
  { 1, 8, 128, 1024, 2048, 4096, 8192, },
  { 2, 8, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 128, 1024, 2048, 4096, 8192, },
  { 4, 8, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 128, 1024, 2048, 4096, 8192, },
  { 16, 128, 1024, 2048, 4096, 8192, },
  { 1, 16, 128, 1024, 2048, 4096, 8192, },
  { 2, 16, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 128, 1024, 2048, 4096, 8192, },
  { 4, 16, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 128, 1024, 2048, 4096, 8192, },
  { 8, 16, 128, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 128, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 128, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 4096, 8192, },
  { 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 32, 128, 1024, 2048, 4096, 8192, },
  { 2, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 128, 1024, 2048, 4096, 8192, },
  { 4, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 128, 1024, 2048, 4096, 8192, },
  { 8, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 128, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 128, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 4096, 8192, },
  { 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, },
  { 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 64, 128, 1024, 2048, 4096, 8192, },
  { 4, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 128, 1024, 2048, 4096, 8192, },
  { 8, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 8, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 8, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 128, 1024, 2048, 4096, 8192, },
  { 4, 8, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 4096, 8192, },
  { 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 4, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 8, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, },
  { 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 4, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 8, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, },
  { 256, 1024, 2048, 4096, 8192, },
  { 1, 256, 1024, 2048, 4096, 8192, },
  { 2, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 256, 1024, 2048, 4096, 8192, },
  { 4, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 256, 1024, 2048, 4096, 8192, },
  { 8, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 256, 1024, 2048, 4096, 8192, },
  { 16, 256, 1024, 2048, 4096, 8192, },
  { 1, 16, 256, 1024, 2048, 4096, 8192, },
  { 2, 16, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 256, 1024, 2048, 4096, 8192, },
  { 4, 16, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 256, 1024, 2048, 4096, 8192, },
  { 8, 16, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 4096, 8192, },
  { 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 32, 256, 1024, 2048, 4096, 8192, },
  { 2, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 256, 1024, 2048, 4096, 8192, },
  { 4, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 256, 1024, 2048, 4096, 8192, },
  { 8, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 4096, 8192, },
  { 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, },
  { 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 64, 256, 1024, 2048, 4096, 8192, },
  { 4, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 256, 1024, 2048, 4096, 8192, },
  { 8, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 4096, 8192, },
  { 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 4, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 8, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, },
  { 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 4, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 8, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, },
  { 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 128, 256, 1024, 2048, 4096, 8192, },
  { 8, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 4096, 8192, },
  { 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 8, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, },
  { 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 8, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, },
  { 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 8, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, },
  { 512, 1024, 2048, 4096, 8192, },
  { 1, 512, 1024, 2048, 4096, 8192, },
  { 2, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 512, 1024, 2048, 4096, 8192, },
  { 4, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 512, 1024, 2048, 4096, 8192, },
  { 8, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 512, 1024, 2048, 4096, 8192, },
  { 16, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 4096, 8192, },
  { 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 32, 512, 1024, 2048, 4096, 8192, },
  { 2, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 512, 1024, 2048, 4096, 8192, },
  { 4, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 512, 1024, 2048, 4096, 8192, },
  { 8, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 4096, 8192, },
  { 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, },
  { 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 64, 512, 1024, 2048, 4096, 8192, },
  { 4, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 512, 1024, 2048, 4096, 8192, },
  { 8, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 4096, 8192, },
  { 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, },
  { 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 4, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 8, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, },
  { 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 128, 512, 1024, 2048, 4096, 8192, },
  { 8, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 4096, 8192, },
  { 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, },
  { 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 8, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, },
  { 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 8, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, },
  { 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 4096, 8192, },
  { 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, },
  { 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, },
  { 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, },
  { 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, },
  { 16384, },
  { 1, 16384, },
  { 2, 16384, },
  { 1, 2, 16384, },
  { 4, 16384, },
  { 1, 4, 16384, },
  { 2, 4, 16384, },
  { 1, 2, 4, 16384, },
  { 8, 16384, },
  { 1, 8, 16384, },
  { 2, 8, 16384, },
  { 1, 2, 8, 16384, },
  { 4, 8, 16384, },
  { 1, 4, 8, 16384, },
  { 2, 4, 8, 16384, },
  { 1, 2, 4, 8, 16384, },
  { 16, 16384, },
  { 1, 16, 16384, },
  { 2, 16, 16384, },
  { 1, 2, 16, 16384, },
  { 4, 16, 16384, },
  { 1, 4, 16, 16384, },
  { 2, 4, 16, 16384, },
  { 1, 2, 4, 16, 16384, },
  { 8, 16, 16384, },
  { 1, 8, 16, 16384, },
  { 2, 8, 16, 16384, },
  { 1, 2, 8, 16, 16384, },
  { 4, 8, 16, 16384, },
  { 1, 4, 8, 16, 16384, },
  { 2, 4, 8, 16, 16384, },
  { 1, 2, 4, 8, 16, 16384, },
  { 32, 16384, },
  { 1, 32, 16384, },
  { 2, 32, 16384, },
  { 1, 2, 32, 16384, },
  { 4, 32, 16384, },
  { 1, 4, 32, 16384, },
  { 2, 4, 32, 16384, },
  { 1, 2, 4, 32, 16384, },
  { 8, 32, 16384, },
  { 1, 8, 32, 16384, },
  { 2, 8, 32, 16384, },
  { 1, 2, 8, 32, 16384, },
  { 4, 8, 32, 16384, },
  { 1, 4, 8, 32, 16384, },
  { 2, 4, 8, 32, 16384, },
  { 1, 2, 4, 8, 32, 16384, },
  { 16, 32, 16384, },
  { 1, 16, 32, 16384, },
  { 2, 16, 32, 16384, },
  { 1, 2, 16, 32, 16384, },
  { 4, 16, 32, 16384, },
  { 1, 4, 16, 32, 16384, },
  { 2, 4, 16, 32, 16384, },
  { 1, 2, 4, 16, 32, 16384, },
  { 8, 16, 32, 16384, },
  { 1, 8, 16, 32, 16384, },
  { 2, 8, 16, 32, 16384, },
  { 1, 2, 8, 16, 32, 16384, },
  { 4, 8, 16, 32, 16384, },
  { 1, 4, 8, 16, 32, 16384, },
  { 2, 4, 8, 16, 32, 16384, },
  { 1, 2, 4, 8, 16, 32, 16384, },
  { 64, 16384, },
  { 1, 64, 16384, },
  { 2, 64, 16384, },
  { 1, 2, 64, 16384, },
  { 4, 64, 16384, },
  { 1, 4, 64, 16384, },
  { 2, 4, 64, 16384, },
  { 1, 2, 4, 64, 16384, },
  { 8, 64, 16384, },
  { 1, 8, 64, 16384, },
  { 2, 8, 64, 16384, },
  { 1, 2, 8, 64, 16384, },
  { 4, 8, 64, 16384, },
  { 1, 4, 8, 64, 16384, },
  { 2, 4, 8, 64, 16384, },
  { 1, 2, 4, 8, 64, 16384, },
  { 16, 64, 16384, },
  { 1, 16, 64, 16384, },
  { 2, 16, 64, 16384, },
  { 1, 2, 16, 64, 16384, },
  { 4, 16, 64, 16384, },
  { 1, 4, 16, 64, 16384, },
  { 2, 4, 16, 64, 16384, },
  { 1, 2, 4, 16, 64, 16384, },
  { 8, 16, 64, 16384, },
  { 1, 8, 16, 64, 16384, },
  { 2, 8, 16, 64, 16384, },
  { 1, 2, 8, 16, 64, 16384, },
  { 4, 8, 16, 64, 16384, },
  { 1, 4, 8, 16, 64, 16384, },
  { 2, 4, 8, 16, 64, 16384, },
  { 1, 2, 4, 8, 16, 64, 16384, },
  { 32, 64, 16384, },
  { 1, 32, 64, 16384, },
  { 2, 32, 64, 16384, },
  { 1, 2, 32, 64, 16384, },
  { 4, 32, 64, 16384, },
  { 1, 4, 32, 64, 16384, },
  { 2, 4, 32, 64, 16384, },
  { 1, 2, 4, 32, 64, 16384, },
  { 8, 32, 64, 16384, },
  { 1, 8, 32, 64, 16384, },
  { 2, 8, 32, 64, 16384, },
  { 1, 2, 8, 32, 64, 16384, },
  { 4, 8, 32, 64, 16384, },
  { 1, 4, 8, 32, 64, 16384, },
  { 2, 4, 8, 32, 64, 16384, },
  { 1, 2, 4, 8, 32, 64, 16384, },
  { 16, 32, 64, 16384, },
  { 1, 16, 32, 64, 16384, },
  { 2, 16, 32, 64, 16384, },
  { 1, 2, 16, 32, 64, 16384, },
  { 4, 16, 32, 64, 16384, },
  { 1, 4, 16, 32, 64, 16384, },
  { 2, 4, 16, 32, 64, 16384, },
  { 1, 2, 4, 16, 32, 64, 16384, },
  { 8, 16, 32, 64, 16384, },
  { 1, 8, 16, 32, 64, 16384, },
  { 2, 8, 16, 32, 64, 16384, },
  { 1, 2, 8, 16, 32, 64, 16384, },
  { 4, 8, 16, 32, 64, 16384, },
  { 1, 4, 8, 16, 32, 64, 16384, },
  { 2, 4, 8, 16, 32, 64, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 16384, },
  { 128, 16384, },
  { 1, 128, 16384, },
  { 2, 128, 16384, },
  { 1, 2, 128, 16384, },
  { 4, 128, 16384, },
  { 1, 4, 128, 16384, },
  { 2, 4, 128, 16384, },
  { 1, 2, 4, 128, 16384, },
  { 8, 128, 16384, },
  { 1, 8, 128, 16384, },
  { 2, 8, 128, 16384, },
  { 1, 2, 8, 128, 16384, },
  { 4, 8, 128, 16384, },
  { 1, 4, 8, 128, 16384, },
  { 2, 4, 8, 128, 16384, },
  { 1, 2, 4, 8, 128, 16384, },
  { 16, 128, 16384, },
  { 1, 16, 128, 16384, },
  { 2, 16, 128, 16384, },
  { 1, 2, 16, 128, 16384, },
  { 4, 16, 128, 16384, },
  { 1, 4, 16, 128, 16384, },
  { 2, 4, 16, 128, 16384, },
  { 1, 2, 4, 16, 128, 16384, },
  { 8, 16, 128, 16384, },
  { 1, 8, 16, 128, 16384, },
  { 2, 8, 16, 128, 16384, },
  { 1, 2, 8, 16, 128, 16384, },
  { 4, 8, 16, 128, 16384, },
  { 1, 4, 8, 16, 128, 16384, },
  { 2, 4, 8, 16, 128, 16384, },
  { 1, 2, 4, 8, 16, 128, 16384, },
  { 32, 128, 16384, },
  { 1, 32, 128, 16384, },
  { 2, 32, 128, 16384, },
  { 1, 2, 32, 128, 16384, },
  { 4, 32, 128, 16384, },
  { 1, 4, 32, 128, 16384, },
  { 2, 4, 32, 128, 16384, },
  { 1, 2, 4, 32, 128, 16384, },
  { 8, 32, 128, 16384, },
  { 1, 8, 32, 128, 16384, },
  { 2, 8, 32, 128, 16384, },
  { 1, 2, 8, 32, 128, 16384, },
  { 4, 8, 32, 128, 16384, },
  { 1, 4, 8, 32, 128, 16384, },
  { 2, 4, 8, 32, 128, 16384, },
  { 1, 2, 4, 8, 32, 128, 16384, },
  { 16, 32, 128, 16384, },
  { 1, 16, 32, 128, 16384, },
  { 2, 16, 32, 128, 16384, },
  { 1, 2, 16, 32, 128, 16384, },
  { 4, 16, 32, 128, 16384, },
  { 1, 4, 16, 32, 128, 16384, },
  { 2, 4, 16, 32, 128, 16384, },
  { 1, 2, 4, 16, 32, 128, 16384, },
  { 8, 16, 32, 128, 16384, },
  { 1, 8, 16, 32, 128, 16384, },
  { 2, 8, 16, 32, 128, 16384, },
  { 1, 2, 8, 16, 32, 128, 16384, },
  { 4, 8, 16, 32, 128, 16384, },
  { 1, 4, 8, 16, 32, 128, 16384, },
  { 2, 4, 8, 16, 32, 128, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 16384, },
  { 64, 128, 16384, },
  { 1, 64, 128, 16384, },
  { 2, 64, 128, 16384, },
  { 1, 2, 64, 128, 16384, },
  { 4, 64, 128, 16384, },
  { 1, 4, 64, 128, 16384, },
  { 2, 4, 64, 128, 16384, },
  { 1, 2, 4, 64, 128, 16384, },
  { 8, 64, 128, 16384, },
  { 1, 8, 64, 128, 16384, },
  { 2, 8, 64, 128, 16384, },
  { 1, 2, 8, 64, 128, 16384, },
  { 4, 8, 64, 128, 16384, },
  { 1, 4, 8, 64, 128, 16384, },
  { 2, 4, 8, 64, 128, 16384, },
  { 1, 2, 4, 8, 64, 128, 16384, },
  { 16, 64, 128, 16384, },
  { 1, 16, 64, 128, 16384, },
  { 2, 16, 64, 128, 16384, },
  { 1, 2, 16, 64, 128, 16384, },
  { 4, 16, 64, 128, 16384, },
  { 1, 4, 16, 64, 128, 16384, },
  { 2, 4, 16, 64, 128, 16384, },
  { 1, 2, 4, 16, 64, 128, 16384, },
  { 8, 16, 64, 128, 16384, },
  { 1, 8, 16, 64, 128, 16384, },
  { 2, 8, 16, 64, 128, 16384, },
  { 1, 2, 8, 16, 64, 128, 16384, },
  { 4, 8, 16, 64, 128, 16384, },
  { 1, 4, 8, 16, 64, 128, 16384, },
  { 2, 4, 8, 16, 64, 128, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 16384, },
  { 32, 64, 128, 16384, },
  { 1, 32, 64, 128, 16384, },
  { 2, 32, 64, 128, 16384, },
  { 1, 2, 32, 64, 128, 16384, },
  { 4, 32, 64, 128, 16384, },
  { 1, 4, 32, 64, 128, 16384, },
  { 2, 4, 32, 64, 128, 16384, },
  { 1, 2, 4, 32, 64, 128, 16384, },
  { 8, 32, 64, 128, 16384, },
  { 1, 8, 32, 64, 128, 16384, },
  { 2, 8, 32, 64, 128, 16384, },
  { 1, 2, 8, 32, 64, 128, 16384, },
  { 4, 8, 32, 64, 128, 16384, },
  { 1, 4, 8, 32, 64, 128, 16384, },
  { 2, 4, 8, 32, 64, 128, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 16384, },
  { 16, 32, 64, 128, 16384, },
  { 1, 16, 32, 64, 128, 16384, },
  { 2, 16, 32, 64, 128, 16384, },
  { 1, 2, 16, 32, 64, 128, 16384, },
  { 4, 16, 32, 64, 128, 16384, },
  { 1, 4, 16, 32, 64, 128, 16384, },
  { 2, 4, 16, 32, 64, 128, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 16384, },
  { 8, 16, 32, 64, 128, 16384, },
  { 1, 8, 16, 32, 64, 128, 16384, },
  { 2, 8, 16, 32, 64, 128, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 16384, },
  { 4, 8, 16, 32, 64, 128, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 16384, },
  { 256, 16384, },
  { 1, 256, 16384, },
  { 2, 256, 16384, },
  { 1, 2, 256, 16384, },
  { 4, 256, 16384, },
  { 1, 4, 256, 16384, },
  { 2, 4, 256, 16384, },
  { 1, 2, 4, 256, 16384, },
  { 8, 256, 16384, },
  { 1, 8, 256, 16384, },
  { 2, 8, 256, 16384, },
  { 1, 2, 8, 256, 16384, },
  { 4, 8, 256, 16384, },
  { 1, 4, 8, 256, 16384, },
  { 2, 4, 8, 256, 16384, },
  { 1, 2, 4, 8, 256, 16384, },
  { 16, 256, 16384, },
  { 1, 16, 256, 16384, },
  { 2, 16, 256, 16384, },
  { 1, 2, 16, 256, 16384, },
  { 4, 16, 256, 16384, },
  { 1, 4, 16, 256, 16384, },
  { 2, 4, 16, 256, 16384, },
  { 1, 2, 4, 16, 256, 16384, },
  { 8, 16, 256, 16384, },
  { 1, 8, 16, 256, 16384, },
  { 2, 8, 16, 256, 16384, },
  { 1, 2, 8, 16, 256, 16384, },
  { 4, 8, 16, 256, 16384, },
  { 1, 4, 8, 16, 256, 16384, },
  { 2, 4, 8, 16, 256, 16384, },
  { 1, 2, 4, 8, 16, 256, 16384, },
  { 32, 256, 16384, },
  { 1, 32, 256, 16384, },
  { 2, 32, 256, 16384, },
  { 1, 2, 32, 256, 16384, },
  { 4, 32, 256, 16384, },
  { 1, 4, 32, 256, 16384, },
  { 2, 4, 32, 256, 16384, },
  { 1, 2, 4, 32, 256, 16384, },
  { 8, 32, 256, 16384, },
  { 1, 8, 32, 256, 16384, },
  { 2, 8, 32, 256, 16384, },
  { 1, 2, 8, 32, 256, 16384, },
  { 4, 8, 32, 256, 16384, },
  { 1, 4, 8, 32, 256, 16384, },
  { 2, 4, 8, 32, 256, 16384, },
  { 1, 2, 4, 8, 32, 256, 16384, },
  { 16, 32, 256, 16384, },
  { 1, 16, 32, 256, 16384, },
  { 2, 16, 32, 256, 16384, },
  { 1, 2, 16, 32, 256, 16384, },
  { 4, 16, 32, 256, 16384, },
  { 1, 4, 16, 32, 256, 16384, },
  { 2, 4, 16, 32, 256, 16384, },
  { 1, 2, 4, 16, 32, 256, 16384, },
  { 8, 16, 32, 256, 16384, },
  { 1, 8, 16, 32, 256, 16384, },
  { 2, 8, 16, 32, 256, 16384, },
  { 1, 2, 8, 16, 32, 256, 16384, },
  { 4, 8, 16, 32, 256, 16384, },
  { 1, 4, 8, 16, 32, 256, 16384, },
  { 2, 4, 8, 16, 32, 256, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 16384, },
  { 64, 256, 16384, },
  { 1, 64, 256, 16384, },
  { 2, 64, 256, 16384, },
  { 1, 2, 64, 256, 16384, },
  { 4, 64, 256, 16384, },
  { 1, 4, 64, 256, 16384, },
  { 2, 4, 64, 256, 16384, },
  { 1, 2, 4, 64, 256, 16384, },
  { 8, 64, 256, 16384, },
  { 1, 8, 64, 256, 16384, },
  { 2, 8, 64, 256, 16384, },
  { 1, 2, 8, 64, 256, 16384, },
  { 4, 8, 64, 256, 16384, },
  { 1, 4, 8, 64, 256, 16384, },
  { 2, 4, 8, 64, 256, 16384, },
  { 1, 2, 4, 8, 64, 256, 16384, },
  { 16, 64, 256, 16384, },
  { 1, 16, 64, 256, 16384, },
  { 2, 16, 64, 256, 16384, },
  { 1, 2, 16, 64, 256, 16384, },
  { 4, 16, 64, 256, 16384, },
  { 1, 4, 16, 64, 256, 16384, },
  { 2, 4, 16, 64, 256, 16384, },
  { 1, 2, 4, 16, 64, 256, 16384, },
  { 8, 16, 64, 256, 16384, },
  { 1, 8, 16, 64, 256, 16384, },
  { 2, 8, 16, 64, 256, 16384, },
  { 1, 2, 8, 16, 64, 256, 16384, },
  { 4, 8, 16, 64, 256, 16384, },
  { 1, 4, 8, 16, 64, 256, 16384, },
  { 2, 4, 8, 16, 64, 256, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 16384, },
  { 32, 64, 256, 16384, },
  { 1, 32, 64, 256, 16384, },
  { 2, 32, 64, 256, 16384, },
  { 1, 2, 32, 64, 256, 16384, },
  { 4, 32, 64, 256, 16384, },
  { 1, 4, 32, 64, 256, 16384, },
  { 2, 4, 32, 64, 256, 16384, },
  { 1, 2, 4, 32, 64, 256, 16384, },
  { 8, 32, 64, 256, 16384, },
  { 1, 8, 32, 64, 256, 16384, },
  { 2, 8, 32, 64, 256, 16384, },
  { 1, 2, 8, 32, 64, 256, 16384, },
  { 4, 8, 32, 64, 256, 16384, },
  { 1, 4, 8, 32, 64, 256, 16384, },
  { 2, 4, 8, 32, 64, 256, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 16384, },
  { 16, 32, 64, 256, 16384, },
  { 1, 16, 32, 64, 256, 16384, },
  { 2, 16, 32, 64, 256, 16384, },
  { 1, 2, 16, 32, 64, 256, 16384, },
  { 4, 16, 32, 64, 256, 16384, },
  { 1, 4, 16, 32, 64, 256, 16384, },
  { 2, 4, 16, 32, 64, 256, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 16384, },
  { 8, 16, 32, 64, 256, 16384, },
  { 1, 8, 16, 32, 64, 256, 16384, },
  { 2, 8, 16, 32, 64, 256, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 16384, },
  { 4, 8, 16, 32, 64, 256, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 16384, },
  { 128, 256, 16384, },
  { 1, 128, 256, 16384, },
  { 2, 128, 256, 16384, },
  { 1, 2, 128, 256, 16384, },
  { 4, 128, 256, 16384, },
  { 1, 4, 128, 256, 16384, },
  { 2, 4, 128, 256, 16384, },
  { 1, 2, 4, 128, 256, 16384, },
  { 8, 128, 256, 16384, },
  { 1, 8, 128, 256, 16384, },
  { 2, 8, 128, 256, 16384, },
  { 1, 2, 8, 128, 256, 16384, },
  { 4, 8, 128, 256, 16384, },
  { 1, 4, 8, 128, 256, 16384, },
  { 2, 4, 8, 128, 256, 16384, },
  { 1, 2, 4, 8, 128, 256, 16384, },
  { 16, 128, 256, 16384, },
  { 1, 16, 128, 256, 16384, },
  { 2, 16, 128, 256, 16384, },
  { 1, 2, 16, 128, 256, 16384, },
  { 4, 16, 128, 256, 16384, },
  { 1, 4, 16, 128, 256, 16384, },
  { 2, 4, 16, 128, 256, 16384, },
  { 1, 2, 4, 16, 128, 256, 16384, },
  { 8, 16, 128, 256, 16384, },
  { 1, 8, 16, 128, 256, 16384, },
  { 2, 8, 16, 128, 256, 16384, },
  { 1, 2, 8, 16, 128, 256, 16384, },
  { 4, 8, 16, 128, 256, 16384, },
  { 1, 4, 8, 16, 128, 256, 16384, },
  { 2, 4, 8, 16, 128, 256, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 16384, },
  { 32, 128, 256, 16384, },
  { 1, 32, 128, 256, 16384, },
  { 2, 32, 128, 256, 16384, },
  { 1, 2, 32, 128, 256, 16384, },
  { 4, 32, 128, 256, 16384, },
  { 1, 4, 32, 128, 256, 16384, },
  { 2, 4, 32, 128, 256, 16384, },
  { 1, 2, 4, 32, 128, 256, 16384, },
  { 8, 32, 128, 256, 16384, },
  { 1, 8, 32, 128, 256, 16384, },
  { 2, 8, 32, 128, 256, 16384, },
  { 1, 2, 8, 32, 128, 256, 16384, },
  { 4, 8, 32, 128, 256, 16384, },
  { 1, 4, 8, 32, 128, 256, 16384, },
  { 2, 4, 8, 32, 128, 256, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 16384, },
  { 16, 32, 128, 256, 16384, },
  { 1, 16, 32, 128, 256, 16384, },
  { 2, 16, 32, 128, 256, 16384, },
  { 1, 2, 16, 32, 128, 256, 16384, },
  { 4, 16, 32, 128, 256, 16384, },
  { 1, 4, 16, 32, 128, 256, 16384, },
  { 2, 4, 16, 32, 128, 256, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 16384, },
  { 8, 16, 32, 128, 256, 16384, },
  { 1, 8, 16, 32, 128, 256, 16384, },
  { 2, 8, 16, 32, 128, 256, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 16384, },
  { 4, 8, 16, 32, 128, 256, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 16384, },
  { 64, 128, 256, 16384, },
  { 1, 64, 128, 256, 16384, },
  { 2, 64, 128, 256, 16384, },
  { 1, 2, 64, 128, 256, 16384, },
  { 4, 64, 128, 256, 16384, },
  { 1, 4, 64, 128, 256, 16384, },
  { 2, 4, 64, 128, 256, 16384, },
  { 1, 2, 4, 64, 128, 256, 16384, },
  { 8, 64, 128, 256, 16384, },
  { 1, 8, 64, 128, 256, 16384, },
  { 2, 8, 64, 128, 256, 16384, },
  { 1, 2, 8, 64, 128, 256, 16384, },
  { 4, 8, 64, 128, 256, 16384, },
  { 1, 4, 8, 64, 128, 256, 16384, },
  { 2, 4, 8, 64, 128, 256, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 16384, },
  { 16, 64, 128, 256, 16384, },
  { 1, 16, 64, 128, 256, 16384, },
  { 2, 16, 64, 128, 256, 16384, },
  { 1, 2, 16, 64, 128, 256, 16384, },
  { 4, 16, 64, 128, 256, 16384, },
  { 1, 4, 16, 64, 128, 256, 16384, },
  { 2, 4, 16, 64, 128, 256, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 16384, },
  { 8, 16, 64, 128, 256, 16384, },
  { 1, 8, 16, 64, 128, 256, 16384, },
  { 2, 8, 16, 64, 128, 256, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 16384, },
  { 4, 8, 16, 64, 128, 256, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 16384, },
  { 32, 64, 128, 256, 16384, },
  { 1, 32, 64, 128, 256, 16384, },
  { 2, 32, 64, 128, 256, 16384, },
  { 1, 2, 32, 64, 128, 256, 16384, },
  { 4, 32, 64, 128, 256, 16384, },
  { 1, 4, 32, 64, 128, 256, 16384, },
  { 2, 4, 32, 64, 128, 256, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 16384, },
  { 8, 32, 64, 128, 256, 16384, },
  { 1, 8, 32, 64, 128, 256, 16384, },
  { 2, 8, 32, 64, 128, 256, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 16384, },
  { 4, 8, 32, 64, 128, 256, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 16384, },
  { 16, 32, 64, 128, 256, 16384, },
  { 1, 16, 32, 64, 128, 256, 16384, },
  { 2, 16, 32, 64, 128, 256, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 16384, },
  { 4, 16, 32, 64, 128, 256, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 16384, },
  { 8, 16, 32, 64, 128, 256, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 16384, },
  { 512, 16384, },
  { 1, 512, 16384, },
  { 2, 512, 16384, },
  { 1, 2, 512, 16384, },
  { 4, 512, 16384, },
  { 1, 4, 512, 16384, },
  { 2, 4, 512, 16384, },
  { 1, 2, 4, 512, 16384, },
  { 8, 512, 16384, },
  { 1, 8, 512, 16384, },
  { 2, 8, 512, 16384, },
  { 1, 2, 8, 512, 16384, },
  { 4, 8, 512, 16384, },
  { 1, 4, 8, 512, 16384, },
  { 2, 4, 8, 512, 16384, },
  { 1, 2, 4, 8, 512, 16384, },
  { 16, 512, 16384, },
  { 1, 16, 512, 16384, },
  { 2, 16, 512, 16384, },
  { 1, 2, 16, 512, 16384, },
  { 4, 16, 512, 16384, },
  { 1, 4, 16, 512, 16384, },
  { 2, 4, 16, 512, 16384, },
  { 1, 2, 4, 16, 512, 16384, },
  { 8, 16, 512, 16384, },
  { 1, 8, 16, 512, 16384, },
  { 2, 8, 16, 512, 16384, },
  { 1, 2, 8, 16, 512, 16384, },
  { 4, 8, 16, 512, 16384, },
  { 1, 4, 8, 16, 512, 16384, },
  { 2, 4, 8, 16, 512, 16384, },
  { 1, 2, 4, 8, 16, 512, 16384, },
  { 32, 512, 16384, },
  { 1, 32, 512, 16384, },
  { 2, 32, 512, 16384, },
  { 1, 2, 32, 512, 16384, },
  { 4, 32, 512, 16384, },
  { 1, 4, 32, 512, 16384, },
  { 2, 4, 32, 512, 16384, },
  { 1, 2, 4, 32, 512, 16384, },
  { 8, 32, 512, 16384, },
  { 1, 8, 32, 512, 16384, },
  { 2, 8, 32, 512, 16384, },
  { 1, 2, 8, 32, 512, 16384, },
  { 4, 8, 32, 512, 16384, },
  { 1, 4, 8, 32, 512, 16384, },
  { 2, 4, 8, 32, 512, 16384, },
  { 1, 2, 4, 8, 32, 512, 16384, },
  { 16, 32, 512, 16384, },
  { 1, 16, 32, 512, 16384, },
  { 2, 16, 32, 512, 16384, },
  { 1, 2, 16, 32, 512, 16384, },
  { 4, 16, 32, 512, 16384, },
  { 1, 4, 16, 32, 512, 16384, },
  { 2, 4, 16, 32, 512, 16384, },
  { 1, 2, 4, 16, 32, 512, 16384, },
  { 8, 16, 32, 512, 16384, },
  { 1, 8, 16, 32, 512, 16384, },
  { 2, 8, 16, 32, 512, 16384, },
  { 1, 2, 8, 16, 32, 512, 16384, },
  { 4, 8, 16, 32, 512, 16384, },
  { 1, 4, 8, 16, 32, 512, 16384, },
  { 2, 4, 8, 16, 32, 512, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 16384, },
  { 64, 512, 16384, },
  { 1, 64, 512, 16384, },
  { 2, 64, 512, 16384, },
  { 1, 2, 64, 512, 16384, },
  { 4, 64, 512, 16384, },
  { 1, 4, 64, 512, 16384, },
  { 2, 4, 64, 512, 16384, },
  { 1, 2, 4, 64, 512, 16384, },
  { 8, 64, 512, 16384, },
  { 1, 8, 64, 512, 16384, },
  { 2, 8, 64, 512, 16384, },
  { 1, 2, 8, 64, 512, 16384, },
  { 4, 8, 64, 512, 16384, },
  { 1, 4, 8, 64, 512, 16384, },
  { 2, 4, 8, 64, 512, 16384, },
  { 1, 2, 4, 8, 64, 512, 16384, },
  { 16, 64, 512, 16384, },
  { 1, 16, 64, 512, 16384, },
  { 2, 16, 64, 512, 16384, },
  { 1, 2, 16, 64, 512, 16384, },
  { 4, 16, 64, 512, 16384, },
  { 1, 4, 16, 64, 512, 16384, },
  { 2, 4, 16, 64, 512, 16384, },
  { 1, 2, 4, 16, 64, 512, 16384, },
  { 8, 16, 64, 512, 16384, },
  { 1, 8, 16, 64, 512, 16384, },
  { 2, 8, 16, 64, 512, 16384, },
  { 1, 2, 8, 16, 64, 512, 16384, },
  { 4, 8, 16, 64, 512, 16384, },
  { 1, 4, 8, 16, 64, 512, 16384, },
  { 2, 4, 8, 16, 64, 512, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 16384, },
  { 32, 64, 512, 16384, },
  { 1, 32, 64, 512, 16384, },
  { 2, 32, 64, 512, 16384, },
  { 1, 2, 32, 64, 512, 16384, },
  { 4, 32, 64, 512, 16384, },
  { 1, 4, 32, 64, 512, 16384, },
  { 2, 4, 32, 64, 512, 16384, },
  { 1, 2, 4, 32, 64, 512, 16384, },
  { 8, 32, 64, 512, 16384, },
  { 1, 8, 32, 64, 512, 16384, },
  { 2, 8, 32, 64, 512, 16384, },
  { 1, 2, 8, 32, 64, 512, 16384, },
  { 4, 8, 32, 64, 512, 16384, },
  { 1, 4, 8, 32, 64, 512, 16384, },
  { 2, 4, 8, 32, 64, 512, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 16384, },
  { 16, 32, 64, 512, 16384, },
  { 1, 16, 32, 64, 512, 16384, },
  { 2, 16, 32, 64, 512, 16384, },
  { 1, 2, 16, 32, 64, 512, 16384, },
  { 4, 16, 32, 64, 512, 16384, },
  { 1, 4, 16, 32, 64, 512, 16384, },
  { 2, 4, 16, 32, 64, 512, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 16384, },
  { 8, 16, 32, 64, 512, 16384, },
  { 1, 8, 16, 32, 64, 512, 16384, },
  { 2, 8, 16, 32, 64, 512, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 16384, },
  { 4, 8, 16, 32, 64, 512, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 16384, },
  { 128, 512, 16384, },
  { 1, 128, 512, 16384, },
  { 2, 128, 512, 16384, },
  { 1, 2, 128, 512, 16384, },
  { 4, 128, 512, 16384, },
  { 1, 4, 128, 512, 16384, },
  { 2, 4, 128, 512, 16384, },
  { 1, 2, 4, 128, 512, 16384, },
  { 8, 128, 512, 16384, },
  { 1, 8, 128, 512, 16384, },
  { 2, 8, 128, 512, 16384, },
  { 1, 2, 8, 128, 512, 16384, },
  { 4, 8, 128, 512, 16384, },
  { 1, 4, 8, 128, 512, 16384, },
  { 2, 4, 8, 128, 512, 16384, },
  { 1, 2, 4, 8, 128, 512, 16384, },
  { 16, 128, 512, 16384, },
  { 1, 16, 128, 512, 16384, },
  { 2, 16, 128, 512, 16384, },
  { 1, 2, 16, 128, 512, 16384, },
  { 4, 16, 128, 512, 16384, },
  { 1, 4, 16, 128, 512, 16384, },
  { 2, 4, 16, 128, 512, 16384, },
  { 1, 2, 4, 16, 128, 512, 16384, },
  { 8, 16, 128, 512, 16384, },
  { 1, 8, 16, 128, 512, 16384, },
  { 2, 8, 16, 128, 512, 16384, },
  { 1, 2, 8, 16, 128, 512, 16384, },
  { 4, 8, 16, 128, 512, 16384, },
  { 1, 4, 8, 16, 128, 512, 16384, },
  { 2, 4, 8, 16, 128, 512, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 16384, },
  { 32, 128, 512, 16384, },
  { 1, 32, 128, 512, 16384, },
  { 2, 32, 128, 512, 16384, },
  { 1, 2, 32, 128, 512, 16384, },
  { 4, 32, 128, 512, 16384, },
  { 1, 4, 32, 128, 512, 16384, },
  { 2, 4, 32, 128, 512, 16384, },
  { 1, 2, 4, 32, 128, 512, 16384, },
  { 8, 32, 128, 512, 16384, },
  { 1, 8, 32, 128, 512, 16384, },
  { 2, 8, 32, 128, 512, 16384, },
  { 1, 2, 8, 32, 128, 512, 16384, },
  { 4, 8, 32, 128, 512, 16384, },
  { 1, 4, 8, 32, 128, 512, 16384, },
  { 2, 4, 8, 32, 128, 512, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 16384, },
  { 16, 32, 128, 512, 16384, },
  { 1, 16, 32, 128, 512, 16384, },
  { 2, 16, 32, 128, 512, 16384, },
  { 1, 2, 16, 32, 128, 512, 16384, },
  { 4, 16, 32, 128, 512, 16384, },
  { 1, 4, 16, 32, 128, 512, 16384, },
  { 2, 4, 16, 32, 128, 512, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 16384, },
  { 8, 16, 32, 128, 512, 16384, },
  { 1, 8, 16, 32, 128, 512, 16384, },
  { 2, 8, 16, 32, 128, 512, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 16384, },
  { 4, 8, 16, 32, 128, 512, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 16384, },
  { 64, 128, 512, 16384, },
  { 1, 64, 128, 512, 16384, },
  { 2, 64, 128, 512, 16384, },
  { 1, 2, 64, 128, 512, 16384, },
  { 4, 64, 128, 512, 16384, },
  { 1, 4, 64, 128, 512, 16384, },
  { 2, 4, 64, 128, 512, 16384, },
  { 1, 2, 4, 64, 128, 512, 16384, },
  { 8, 64, 128, 512, 16384, },
  { 1, 8, 64, 128, 512, 16384, },
  { 2, 8, 64, 128, 512, 16384, },
  { 1, 2, 8, 64, 128, 512, 16384, },
  { 4, 8, 64, 128, 512, 16384, },
  { 1, 4, 8, 64, 128, 512, 16384, },
  { 2, 4, 8, 64, 128, 512, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 16384, },
  { 16, 64, 128, 512, 16384, },
  { 1, 16, 64, 128, 512, 16384, },
  { 2, 16, 64, 128, 512, 16384, },
  { 1, 2, 16, 64, 128, 512, 16384, },
  { 4, 16, 64, 128, 512, 16384, },
  { 1, 4, 16, 64, 128, 512, 16384, },
  { 2, 4, 16, 64, 128, 512, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 16384, },
  { 8, 16, 64, 128, 512, 16384, },
  { 1, 8, 16, 64, 128, 512, 16384, },
  { 2, 8, 16, 64, 128, 512, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 16384, },
  { 4, 8, 16, 64, 128, 512, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 16384, },
  { 32, 64, 128, 512, 16384, },
  { 1, 32, 64, 128, 512, 16384, },
  { 2, 32, 64, 128, 512, 16384, },
  { 1, 2, 32, 64, 128, 512, 16384, },
  { 4, 32, 64, 128, 512, 16384, },
  { 1, 4, 32, 64, 128, 512, 16384, },
  { 2, 4, 32, 64, 128, 512, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 16384, },
  { 8, 32, 64, 128, 512, 16384, },
  { 1, 8, 32, 64, 128, 512, 16384, },
  { 2, 8, 32, 64, 128, 512, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 16384, },
  { 4, 8, 32, 64, 128, 512, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 16384, },
  { 16, 32, 64, 128, 512, 16384, },
  { 1, 16, 32, 64, 128, 512, 16384, },
  { 2, 16, 32, 64, 128, 512, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 16384, },
  { 4, 16, 32, 64, 128, 512, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 16384, },
  { 8, 16, 32, 64, 128, 512, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 16384, },
  { 256, 512, 16384, },
  { 1, 256, 512, 16384, },
  { 2, 256, 512, 16384, },
  { 1, 2, 256, 512, 16384, },
  { 4, 256, 512, 16384, },
  { 1, 4, 256, 512, 16384, },
  { 2, 4, 256, 512, 16384, },
  { 1, 2, 4, 256, 512, 16384, },
  { 8, 256, 512, 16384, },
  { 1, 8, 256, 512, 16384, },
  { 2, 8, 256, 512, 16384, },
  { 1, 2, 8, 256, 512, 16384, },
  { 4, 8, 256, 512, 16384, },
  { 1, 4, 8, 256, 512, 16384, },
  { 2, 4, 8, 256, 512, 16384, },
  { 1, 2, 4, 8, 256, 512, 16384, },
  { 16, 256, 512, 16384, },
  { 1, 16, 256, 512, 16384, },
  { 2, 16, 256, 512, 16384, },
  { 1, 2, 16, 256, 512, 16384, },
  { 4, 16, 256, 512, 16384, },
  { 1, 4, 16, 256, 512, 16384, },
  { 2, 4, 16, 256, 512, 16384, },
  { 1, 2, 4, 16, 256, 512, 16384, },
  { 8, 16, 256, 512, 16384, },
  { 1, 8, 16, 256, 512, 16384, },
  { 2, 8, 16, 256, 512, 16384, },
  { 1, 2, 8, 16, 256, 512, 16384, },
  { 4, 8, 16, 256, 512, 16384, },
  { 1, 4, 8, 16, 256, 512, 16384, },
  { 2, 4, 8, 16, 256, 512, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 16384, },
  { 32, 256, 512, 16384, },
  { 1, 32, 256, 512, 16384, },
  { 2, 32, 256, 512, 16384, },
  { 1, 2, 32, 256, 512, 16384, },
  { 4, 32, 256, 512, 16384, },
  { 1, 4, 32, 256, 512, 16384, },
  { 2, 4, 32, 256, 512, 16384, },
  { 1, 2, 4, 32, 256, 512, 16384, },
  { 8, 32, 256, 512, 16384, },
  { 1, 8, 32, 256, 512, 16384, },
  { 2, 8, 32, 256, 512, 16384, },
  { 1, 2, 8, 32, 256, 512, 16384, },
  { 4, 8, 32, 256, 512, 16384, },
  { 1, 4, 8, 32, 256, 512, 16384, },
  { 2, 4, 8, 32, 256, 512, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 16384, },
  { 16, 32, 256, 512, 16384, },
  { 1, 16, 32, 256, 512, 16384, },
  { 2, 16, 32, 256, 512, 16384, },
  { 1, 2, 16, 32, 256, 512, 16384, },
  { 4, 16, 32, 256, 512, 16384, },
  { 1, 4, 16, 32, 256, 512, 16384, },
  { 2, 4, 16, 32, 256, 512, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 16384, },
  { 8, 16, 32, 256, 512, 16384, },
  { 1, 8, 16, 32, 256, 512, 16384, },
  { 2, 8, 16, 32, 256, 512, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 16384, },
  { 4, 8, 16, 32, 256, 512, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 16384, },
  { 64, 256, 512, 16384, },
  { 1, 64, 256, 512, 16384, },
  { 2, 64, 256, 512, 16384, },
  { 1, 2, 64, 256, 512, 16384, },
  { 4, 64, 256, 512, 16384, },
  { 1, 4, 64, 256, 512, 16384, },
  { 2, 4, 64, 256, 512, 16384, },
  { 1, 2, 4, 64, 256, 512, 16384, },
  { 8, 64, 256, 512, 16384, },
  { 1, 8, 64, 256, 512, 16384, },
  { 2, 8, 64, 256, 512, 16384, },
  { 1, 2, 8, 64, 256, 512, 16384, },
  { 4, 8, 64, 256, 512, 16384, },
  { 1, 4, 8, 64, 256, 512, 16384, },
  { 2, 4, 8, 64, 256, 512, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 16384, },
  { 16, 64, 256, 512, 16384, },
  { 1, 16, 64, 256, 512, 16384, },
  { 2, 16, 64, 256, 512, 16384, },
  { 1, 2, 16, 64, 256, 512, 16384, },
  { 4, 16, 64, 256, 512, 16384, },
  { 1, 4, 16, 64, 256, 512, 16384, },
  { 2, 4, 16, 64, 256, 512, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 16384, },
  { 8, 16, 64, 256, 512, 16384, },
  { 1, 8, 16, 64, 256, 512, 16384, },
  { 2, 8, 16, 64, 256, 512, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 16384, },
  { 4, 8, 16, 64, 256, 512, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 16384, },
  { 32, 64, 256, 512, 16384, },
  { 1, 32, 64, 256, 512, 16384, },
  { 2, 32, 64, 256, 512, 16384, },
  { 1, 2, 32, 64, 256, 512, 16384, },
  { 4, 32, 64, 256, 512, 16384, },
  { 1, 4, 32, 64, 256, 512, 16384, },
  { 2, 4, 32, 64, 256, 512, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 16384, },
  { 8, 32, 64, 256, 512, 16384, },
  { 1, 8, 32, 64, 256, 512, 16384, },
  { 2, 8, 32, 64, 256, 512, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 16384, },
  { 4, 8, 32, 64, 256, 512, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 16384, },
  { 16, 32, 64, 256, 512, 16384, },
  { 1, 16, 32, 64, 256, 512, 16384, },
  { 2, 16, 32, 64, 256, 512, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 16384, },
  { 4, 16, 32, 64, 256, 512, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 16384, },
  { 8, 16, 32, 64, 256, 512, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 16384, },
  { 128, 256, 512, 16384, },
  { 1, 128, 256, 512, 16384, },
  { 2, 128, 256, 512, 16384, },
  { 1, 2, 128, 256, 512, 16384, },
  { 4, 128, 256, 512, 16384, },
  { 1, 4, 128, 256, 512, 16384, },
  { 2, 4, 128, 256, 512, 16384, },
  { 1, 2, 4, 128, 256, 512, 16384, },
  { 8, 128, 256, 512, 16384, },
  { 1, 8, 128, 256, 512, 16384, },
  { 2, 8, 128, 256, 512, 16384, },
  { 1, 2, 8, 128, 256, 512, 16384, },
  { 4, 8, 128, 256, 512, 16384, },
  { 1, 4, 8, 128, 256, 512, 16384, },
  { 2, 4, 8, 128, 256, 512, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 16384, },
  { 16, 128, 256, 512, 16384, },
  { 1, 16, 128, 256, 512, 16384, },
  { 2, 16, 128, 256, 512, 16384, },
  { 1, 2, 16, 128, 256, 512, 16384, },
  { 4, 16, 128, 256, 512, 16384, },
  { 1, 4, 16, 128, 256, 512, 16384, },
  { 2, 4, 16, 128, 256, 512, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 16384, },
  { 8, 16, 128, 256, 512, 16384, },
  { 1, 8, 16, 128, 256, 512, 16384, },
  { 2, 8, 16, 128, 256, 512, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 16384, },
  { 4, 8, 16, 128, 256, 512, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 16384, },
  { 32, 128, 256, 512, 16384, },
  { 1, 32, 128, 256, 512, 16384, },
  { 2, 32, 128, 256, 512, 16384, },
  { 1, 2, 32, 128, 256, 512, 16384, },
  { 4, 32, 128, 256, 512, 16384, },
  { 1, 4, 32, 128, 256, 512, 16384, },
  { 2, 4, 32, 128, 256, 512, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 16384, },
  { 8, 32, 128, 256, 512, 16384, },
  { 1, 8, 32, 128, 256, 512, 16384, },
  { 2, 8, 32, 128, 256, 512, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 16384, },
  { 4, 8, 32, 128, 256, 512, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 16384, },
  { 16, 32, 128, 256, 512, 16384, },
  { 1, 16, 32, 128, 256, 512, 16384, },
  { 2, 16, 32, 128, 256, 512, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 16384, },
  { 4, 16, 32, 128, 256, 512, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 16384, },
  { 8, 16, 32, 128, 256, 512, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 16384, },
  { 64, 128, 256, 512, 16384, },
  { 1, 64, 128, 256, 512, 16384, },
  { 2, 64, 128, 256, 512, 16384, },
  { 1, 2, 64, 128, 256, 512, 16384, },
  { 4, 64, 128, 256, 512, 16384, },
  { 1, 4, 64, 128, 256, 512, 16384, },
  { 2, 4, 64, 128, 256, 512, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 16384, },
  { 8, 64, 128, 256, 512, 16384, },
  { 1, 8, 64, 128, 256, 512, 16384, },
  { 2, 8, 64, 128, 256, 512, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 16384, },
  { 4, 8, 64, 128, 256, 512, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 16384, },
  { 16, 64, 128, 256, 512, 16384, },
  { 1, 16, 64, 128, 256, 512, 16384, },
  { 2, 16, 64, 128, 256, 512, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 16384, },
  { 4, 16, 64, 128, 256, 512, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 16384, },
  { 8, 16, 64, 128, 256, 512, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 16384, },
  { 32, 64, 128, 256, 512, 16384, },
  { 1, 32, 64, 128, 256, 512, 16384, },
  { 2, 32, 64, 128, 256, 512, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 16384, },
  { 4, 32, 64, 128, 256, 512, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 16384, },
  { 8, 32, 64, 128, 256, 512, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 16384, },
  { 16, 32, 64, 128, 256, 512, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 16384, },
  { 1024, 16384, },
  { 1, 1024, 16384, },
  { 2, 1024, 16384, },
  { 1, 2, 1024, 16384, },
  { 4, 1024, 16384, },
  { 1, 4, 1024, 16384, },
  { 2, 4, 1024, 16384, },
  { 1, 2, 4, 1024, 16384, },
  { 8, 1024, 16384, },
  { 1, 8, 1024, 16384, },
  { 2, 8, 1024, 16384, },
  { 1, 2, 8, 1024, 16384, },
  { 4, 8, 1024, 16384, },
  { 1, 4, 8, 1024, 16384, },
  { 2, 4, 8, 1024, 16384, },
  { 1, 2, 4, 8, 1024, 16384, },
  { 16, 1024, 16384, },
  { 1, 16, 1024, 16384, },
  { 2, 16, 1024, 16384, },
  { 1, 2, 16, 1024, 16384, },
  { 4, 16, 1024, 16384, },
  { 1, 4, 16, 1024, 16384, },
  { 2, 4, 16, 1024, 16384, },
  { 1, 2, 4, 16, 1024, 16384, },
  { 8, 16, 1024, 16384, },
  { 1, 8, 16, 1024, 16384, },
  { 2, 8, 16, 1024, 16384, },
  { 1, 2, 8, 16, 1024, 16384, },
  { 4, 8, 16, 1024, 16384, },
  { 1, 4, 8, 16, 1024, 16384, },
  { 2, 4, 8, 16, 1024, 16384, },
  { 1, 2, 4, 8, 16, 1024, 16384, },
  { 32, 1024, 16384, },
  { 1, 32, 1024, 16384, },
  { 2, 32, 1024, 16384, },
  { 1, 2, 32, 1024, 16384, },
  { 4, 32, 1024, 16384, },
  { 1, 4, 32, 1024, 16384, },
  { 2, 4, 32, 1024, 16384, },
  { 1, 2, 4, 32, 1024, 16384, },
  { 8, 32, 1024, 16384, },
  { 1, 8, 32, 1024, 16384, },
  { 2, 8, 32, 1024, 16384, },
  { 1, 2, 8, 32, 1024, 16384, },
  { 4, 8, 32, 1024, 16384, },
  { 1, 4, 8, 32, 1024, 16384, },
  { 2, 4, 8, 32, 1024, 16384, },
  { 1, 2, 4, 8, 32, 1024, 16384, },
  { 16, 32, 1024, 16384, },
  { 1, 16, 32, 1024, 16384, },
  { 2, 16, 32, 1024, 16384, },
  { 1, 2, 16, 32, 1024, 16384, },
  { 4, 16, 32, 1024, 16384, },
  { 1, 4, 16, 32, 1024, 16384, },
  { 2, 4, 16, 32, 1024, 16384, },
  { 1, 2, 4, 16, 32, 1024, 16384, },
  { 8, 16, 32, 1024, 16384, },
  { 1, 8, 16, 32, 1024, 16384, },
  { 2, 8, 16, 32, 1024, 16384, },
  { 1, 2, 8, 16, 32, 1024, 16384, },
  { 4, 8, 16, 32, 1024, 16384, },
  { 1, 4, 8, 16, 32, 1024, 16384, },
  { 2, 4, 8, 16, 32, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 1024, 16384, },
  { 64, 1024, 16384, },
  { 1, 64, 1024, 16384, },
  { 2, 64, 1024, 16384, },
  { 1, 2, 64, 1024, 16384, },
  { 4, 64, 1024, 16384, },
  { 1, 4, 64, 1024, 16384, },
  { 2, 4, 64, 1024, 16384, },
  { 1, 2, 4, 64, 1024, 16384, },
  { 8, 64, 1024, 16384, },
  { 1, 8, 64, 1024, 16384, },
  { 2, 8, 64, 1024, 16384, },
  { 1, 2, 8, 64, 1024, 16384, },
  { 4, 8, 64, 1024, 16384, },
  { 1, 4, 8, 64, 1024, 16384, },
  { 2, 4, 8, 64, 1024, 16384, },
  { 1, 2, 4, 8, 64, 1024, 16384, },
  { 16, 64, 1024, 16384, },
  { 1, 16, 64, 1024, 16384, },
  { 2, 16, 64, 1024, 16384, },
  { 1, 2, 16, 64, 1024, 16384, },
  { 4, 16, 64, 1024, 16384, },
  { 1, 4, 16, 64, 1024, 16384, },
  { 2, 4, 16, 64, 1024, 16384, },
  { 1, 2, 4, 16, 64, 1024, 16384, },
  { 8, 16, 64, 1024, 16384, },
  { 1, 8, 16, 64, 1024, 16384, },
  { 2, 8, 16, 64, 1024, 16384, },
  { 1, 2, 8, 16, 64, 1024, 16384, },
  { 4, 8, 16, 64, 1024, 16384, },
  { 1, 4, 8, 16, 64, 1024, 16384, },
  { 2, 4, 8, 16, 64, 1024, 16384, },
  { 1, 2, 4, 8, 16, 64, 1024, 16384, },
  { 32, 64, 1024, 16384, },
  { 1, 32, 64, 1024, 16384, },
  { 2, 32, 64, 1024, 16384, },
  { 1, 2, 32, 64, 1024, 16384, },
  { 4, 32, 64, 1024, 16384, },
  { 1, 4, 32, 64, 1024, 16384, },
  { 2, 4, 32, 64, 1024, 16384, },
  { 1, 2, 4, 32, 64, 1024, 16384, },
  { 8, 32, 64, 1024, 16384, },
  { 1, 8, 32, 64, 1024, 16384, },
  { 2, 8, 32, 64, 1024, 16384, },
  { 1, 2, 8, 32, 64, 1024, 16384, },
  { 4, 8, 32, 64, 1024, 16384, },
  { 1, 4, 8, 32, 64, 1024, 16384, },
  { 2, 4, 8, 32, 64, 1024, 16384, },
  { 1, 2, 4, 8, 32, 64, 1024, 16384, },
  { 16, 32, 64, 1024, 16384, },
  { 1, 16, 32, 64, 1024, 16384, },
  { 2, 16, 32, 64, 1024, 16384, },
  { 1, 2, 16, 32, 64, 1024, 16384, },
  { 4, 16, 32, 64, 1024, 16384, },
  { 1, 4, 16, 32, 64, 1024, 16384, },
  { 2, 4, 16, 32, 64, 1024, 16384, },
  { 1, 2, 4, 16, 32, 64, 1024, 16384, },
  { 8, 16, 32, 64, 1024, 16384, },
  { 1, 8, 16, 32, 64, 1024, 16384, },
  { 2, 8, 16, 32, 64, 1024, 16384, },
  { 1, 2, 8, 16, 32, 64, 1024, 16384, },
  { 4, 8, 16, 32, 64, 1024, 16384, },
  { 1, 4, 8, 16, 32, 64, 1024, 16384, },
  { 2, 4, 8, 16, 32, 64, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 16384, },
  { 128, 1024, 16384, },
  { 1, 128, 1024, 16384, },
  { 2, 128, 1024, 16384, },
  { 1, 2, 128, 1024, 16384, },
  { 4, 128, 1024, 16384, },
  { 1, 4, 128, 1024, 16384, },
  { 2, 4, 128, 1024, 16384, },
  { 1, 2, 4, 128, 1024, 16384, },
  { 8, 128, 1024, 16384, },
  { 1, 8, 128, 1024, 16384, },
  { 2, 8, 128, 1024, 16384, },
  { 1, 2, 8, 128, 1024, 16384, },
  { 4, 8, 128, 1024, 16384, },
  { 1, 4, 8, 128, 1024, 16384, },
  { 2, 4, 8, 128, 1024, 16384, },
  { 1, 2, 4, 8, 128, 1024, 16384, },
  { 16, 128, 1024, 16384, },
  { 1, 16, 128, 1024, 16384, },
  { 2, 16, 128, 1024, 16384, },
  { 1, 2, 16, 128, 1024, 16384, },
  { 4, 16, 128, 1024, 16384, },
  { 1, 4, 16, 128, 1024, 16384, },
  { 2, 4, 16, 128, 1024, 16384, },
  { 1, 2, 4, 16, 128, 1024, 16384, },
  { 8, 16, 128, 1024, 16384, },
  { 1, 8, 16, 128, 1024, 16384, },
  { 2, 8, 16, 128, 1024, 16384, },
  { 1, 2, 8, 16, 128, 1024, 16384, },
  { 4, 8, 16, 128, 1024, 16384, },
  { 1, 4, 8, 16, 128, 1024, 16384, },
  { 2, 4, 8, 16, 128, 1024, 16384, },
  { 1, 2, 4, 8, 16, 128, 1024, 16384, },
  { 32, 128, 1024, 16384, },
  { 1, 32, 128, 1024, 16384, },
  { 2, 32, 128, 1024, 16384, },
  { 1, 2, 32, 128, 1024, 16384, },
  { 4, 32, 128, 1024, 16384, },
  { 1, 4, 32, 128, 1024, 16384, },
  { 2, 4, 32, 128, 1024, 16384, },
  { 1, 2, 4, 32, 128, 1024, 16384, },
  { 8, 32, 128, 1024, 16384, },
  { 1, 8, 32, 128, 1024, 16384, },
  { 2, 8, 32, 128, 1024, 16384, },
  { 1, 2, 8, 32, 128, 1024, 16384, },
  { 4, 8, 32, 128, 1024, 16384, },
  { 1, 4, 8, 32, 128, 1024, 16384, },
  { 2, 4, 8, 32, 128, 1024, 16384, },
  { 1, 2, 4, 8, 32, 128, 1024, 16384, },
  { 16, 32, 128, 1024, 16384, },
  { 1, 16, 32, 128, 1024, 16384, },
  { 2, 16, 32, 128, 1024, 16384, },
  { 1, 2, 16, 32, 128, 1024, 16384, },
  { 4, 16, 32, 128, 1024, 16384, },
  { 1, 4, 16, 32, 128, 1024, 16384, },
  { 2, 4, 16, 32, 128, 1024, 16384, },
  { 1, 2, 4, 16, 32, 128, 1024, 16384, },
  { 8, 16, 32, 128, 1024, 16384, },
  { 1, 8, 16, 32, 128, 1024, 16384, },
  { 2, 8, 16, 32, 128, 1024, 16384, },
  { 1, 2, 8, 16, 32, 128, 1024, 16384, },
  { 4, 8, 16, 32, 128, 1024, 16384, },
  { 1, 4, 8, 16, 32, 128, 1024, 16384, },
  { 2, 4, 8, 16, 32, 128, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 16384, },
  { 64, 128, 1024, 16384, },
  { 1, 64, 128, 1024, 16384, },
  { 2, 64, 128, 1024, 16384, },
  { 1, 2, 64, 128, 1024, 16384, },
  { 4, 64, 128, 1024, 16384, },
  { 1, 4, 64, 128, 1024, 16384, },
  { 2, 4, 64, 128, 1024, 16384, },
  { 1, 2, 4, 64, 128, 1024, 16384, },
  { 8, 64, 128, 1024, 16384, },
  { 1, 8, 64, 128, 1024, 16384, },
  { 2, 8, 64, 128, 1024, 16384, },
  { 1, 2, 8, 64, 128, 1024, 16384, },
  { 4, 8, 64, 128, 1024, 16384, },
  { 1, 4, 8, 64, 128, 1024, 16384, },
  { 2, 4, 8, 64, 128, 1024, 16384, },
  { 1, 2, 4, 8, 64, 128, 1024, 16384, },
  { 16, 64, 128, 1024, 16384, },
  { 1, 16, 64, 128, 1024, 16384, },
  { 2, 16, 64, 128, 1024, 16384, },
  { 1, 2, 16, 64, 128, 1024, 16384, },
  { 4, 16, 64, 128, 1024, 16384, },
  { 1, 4, 16, 64, 128, 1024, 16384, },
  { 2, 4, 16, 64, 128, 1024, 16384, },
  { 1, 2, 4, 16, 64, 128, 1024, 16384, },
  { 8, 16, 64, 128, 1024, 16384, },
  { 1, 8, 16, 64, 128, 1024, 16384, },
  { 2, 8, 16, 64, 128, 1024, 16384, },
  { 1, 2, 8, 16, 64, 128, 1024, 16384, },
  { 4, 8, 16, 64, 128, 1024, 16384, },
  { 1, 4, 8, 16, 64, 128, 1024, 16384, },
  { 2, 4, 8, 16, 64, 128, 1024, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 16384, },
  { 32, 64, 128, 1024, 16384, },
  { 1, 32, 64, 128, 1024, 16384, },
  { 2, 32, 64, 128, 1024, 16384, },
  { 1, 2, 32, 64, 128, 1024, 16384, },
  { 4, 32, 64, 128, 1024, 16384, },
  { 1, 4, 32, 64, 128, 1024, 16384, },
  { 2, 4, 32, 64, 128, 1024, 16384, },
  { 1, 2, 4, 32, 64, 128, 1024, 16384, },
  { 8, 32, 64, 128, 1024, 16384, },
  { 1, 8, 32, 64, 128, 1024, 16384, },
  { 2, 8, 32, 64, 128, 1024, 16384, },
  { 1, 2, 8, 32, 64, 128, 1024, 16384, },
  { 4, 8, 32, 64, 128, 1024, 16384, },
  { 1, 4, 8, 32, 64, 128, 1024, 16384, },
  { 2, 4, 8, 32, 64, 128, 1024, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 16384, },
  { 16, 32, 64, 128, 1024, 16384, },
  { 1, 16, 32, 64, 128, 1024, 16384, },
  { 2, 16, 32, 64, 128, 1024, 16384, },
  { 1, 2, 16, 32, 64, 128, 1024, 16384, },
  { 4, 16, 32, 64, 128, 1024, 16384, },
  { 1, 4, 16, 32, 64, 128, 1024, 16384, },
  { 2, 4, 16, 32, 64, 128, 1024, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 16384, },
  { 8, 16, 32, 64, 128, 1024, 16384, },
  { 1, 8, 16, 32, 64, 128, 1024, 16384, },
  { 2, 8, 16, 32, 64, 128, 1024, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 16384, },
  { 4, 8, 16, 32, 64, 128, 1024, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 16384, },
  { 256, 1024, 16384, },
  { 1, 256, 1024, 16384, },
  { 2, 256, 1024, 16384, },
  { 1, 2, 256, 1024, 16384, },
  { 4, 256, 1024, 16384, },
  { 1, 4, 256, 1024, 16384, },
  { 2, 4, 256, 1024, 16384, },
  { 1, 2, 4, 256, 1024, 16384, },
  { 8, 256, 1024, 16384, },
  { 1, 8, 256, 1024, 16384, },
  { 2, 8, 256, 1024, 16384, },
  { 1, 2, 8, 256, 1024, 16384, },
  { 4, 8, 256, 1024, 16384, },
  { 1, 4, 8, 256, 1024, 16384, },
  { 2, 4, 8, 256, 1024, 16384, },
  { 1, 2, 4, 8, 256, 1024, 16384, },
  { 16, 256, 1024, 16384, },
  { 1, 16, 256, 1024, 16384, },
  { 2, 16, 256, 1024, 16384, },
  { 1, 2, 16, 256, 1024, 16384, },
  { 4, 16, 256, 1024, 16384, },
  { 1, 4, 16, 256, 1024, 16384, },
  { 2, 4, 16, 256, 1024, 16384, },
  { 1, 2, 4, 16, 256, 1024, 16384, },
  { 8, 16, 256, 1024, 16384, },
  { 1, 8, 16, 256, 1024, 16384, },
  { 2, 8, 16, 256, 1024, 16384, },
  { 1, 2, 8, 16, 256, 1024, 16384, },
  { 4, 8, 16, 256, 1024, 16384, },
  { 1, 4, 8, 16, 256, 1024, 16384, },
  { 2, 4, 8, 16, 256, 1024, 16384, },
  { 1, 2, 4, 8, 16, 256, 1024, 16384, },
  { 32, 256, 1024, 16384, },
  { 1, 32, 256, 1024, 16384, },
  { 2, 32, 256, 1024, 16384, },
  { 1, 2, 32, 256, 1024, 16384, },
  { 4, 32, 256, 1024, 16384, },
  { 1, 4, 32, 256, 1024, 16384, },
  { 2, 4, 32, 256, 1024, 16384, },
  { 1, 2, 4, 32, 256, 1024, 16384, },
  { 8, 32, 256, 1024, 16384, },
  { 1, 8, 32, 256, 1024, 16384, },
  { 2, 8, 32, 256, 1024, 16384, },
  { 1, 2, 8, 32, 256, 1024, 16384, },
  { 4, 8, 32, 256, 1024, 16384, },
  { 1, 4, 8, 32, 256, 1024, 16384, },
  { 2, 4, 8, 32, 256, 1024, 16384, },
  { 1, 2, 4, 8, 32, 256, 1024, 16384, },
  { 16, 32, 256, 1024, 16384, },
  { 1, 16, 32, 256, 1024, 16384, },
  { 2, 16, 32, 256, 1024, 16384, },
  { 1, 2, 16, 32, 256, 1024, 16384, },
  { 4, 16, 32, 256, 1024, 16384, },
  { 1, 4, 16, 32, 256, 1024, 16384, },
  { 2, 4, 16, 32, 256, 1024, 16384, },
  { 1, 2, 4, 16, 32, 256, 1024, 16384, },
  { 8, 16, 32, 256, 1024, 16384, },
  { 1, 8, 16, 32, 256, 1024, 16384, },
  { 2, 8, 16, 32, 256, 1024, 16384, },
  { 1, 2, 8, 16, 32, 256, 1024, 16384, },
  { 4, 8, 16, 32, 256, 1024, 16384, },
  { 1, 4, 8, 16, 32, 256, 1024, 16384, },
  { 2, 4, 8, 16, 32, 256, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 16384, },
  { 64, 256, 1024, 16384, },
  { 1, 64, 256, 1024, 16384, },
  { 2, 64, 256, 1024, 16384, },
  { 1, 2, 64, 256, 1024, 16384, },
  { 4, 64, 256, 1024, 16384, },
  { 1, 4, 64, 256, 1024, 16384, },
  { 2, 4, 64, 256, 1024, 16384, },
  { 1, 2, 4, 64, 256, 1024, 16384, },
  { 8, 64, 256, 1024, 16384, },
  { 1, 8, 64, 256, 1024, 16384, },
  { 2, 8, 64, 256, 1024, 16384, },
  { 1, 2, 8, 64, 256, 1024, 16384, },
  { 4, 8, 64, 256, 1024, 16384, },
  { 1, 4, 8, 64, 256, 1024, 16384, },
  { 2, 4, 8, 64, 256, 1024, 16384, },
  { 1, 2, 4, 8, 64, 256, 1024, 16384, },
  { 16, 64, 256, 1024, 16384, },
  { 1, 16, 64, 256, 1024, 16384, },
  { 2, 16, 64, 256, 1024, 16384, },
  { 1, 2, 16, 64, 256, 1024, 16384, },
  { 4, 16, 64, 256, 1024, 16384, },
  { 1, 4, 16, 64, 256, 1024, 16384, },
  { 2, 4, 16, 64, 256, 1024, 16384, },
  { 1, 2, 4, 16, 64, 256, 1024, 16384, },
  { 8, 16, 64, 256, 1024, 16384, },
  { 1, 8, 16, 64, 256, 1024, 16384, },
  { 2, 8, 16, 64, 256, 1024, 16384, },
  { 1, 2, 8, 16, 64, 256, 1024, 16384, },
  { 4, 8, 16, 64, 256, 1024, 16384, },
  { 1, 4, 8, 16, 64, 256, 1024, 16384, },
  { 2, 4, 8, 16, 64, 256, 1024, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 16384, },
  { 32, 64, 256, 1024, 16384, },
  { 1, 32, 64, 256, 1024, 16384, },
  { 2, 32, 64, 256, 1024, 16384, },
  { 1, 2, 32, 64, 256, 1024, 16384, },
  { 4, 32, 64, 256, 1024, 16384, },
  { 1, 4, 32, 64, 256, 1024, 16384, },
  { 2, 4, 32, 64, 256, 1024, 16384, },
  { 1, 2, 4, 32, 64, 256, 1024, 16384, },
  { 8, 32, 64, 256, 1024, 16384, },
  { 1, 8, 32, 64, 256, 1024, 16384, },
  { 2, 8, 32, 64, 256, 1024, 16384, },
  { 1, 2, 8, 32, 64, 256, 1024, 16384, },
  { 4, 8, 32, 64, 256, 1024, 16384, },
  { 1, 4, 8, 32, 64, 256, 1024, 16384, },
  { 2, 4, 8, 32, 64, 256, 1024, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 16384, },
  { 16, 32, 64, 256, 1024, 16384, },
  { 1, 16, 32, 64, 256, 1024, 16384, },
  { 2, 16, 32, 64, 256, 1024, 16384, },
  { 1, 2, 16, 32, 64, 256, 1024, 16384, },
  { 4, 16, 32, 64, 256, 1024, 16384, },
  { 1, 4, 16, 32, 64, 256, 1024, 16384, },
  { 2, 4, 16, 32, 64, 256, 1024, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 16384, },
  { 8, 16, 32, 64, 256, 1024, 16384, },
  { 1, 8, 16, 32, 64, 256, 1024, 16384, },
  { 2, 8, 16, 32, 64, 256, 1024, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 16384, },
  { 4, 8, 16, 32, 64, 256, 1024, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 16384, },
  { 128, 256, 1024, 16384, },
  { 1, 128, 256, 1024, 16384, },
  { 2, 128, 256, 1024, 16384, },
  { 1, 2, 128, 256, 1024, 16384, },
  { 4, 128, 256, 1024, 16384, },
  { 1, 4, 128, 256, 1024, 16384, },
  { 2, 4, 128, 256, 1024, 16384, },
  { 1, 2, 4, 128, 256, 1024, 16384, },
  { 8, 128, 256, 1024, 16384, },
  { 1, 8, 128, 256, 1024, 16384, },
  { 2, 8, 128, 256, 1024, 16384, },
  { 1, 2, 8, 128, 256, 1024, 16384, },
  { 4, 8, 128, 256, 1024, 16384, },
  { 1, 4, 8, 128, 256, 1024, 16384, },
  { 2, 4, 8, 128, 256, 1024, 16384, },
  { 1, 2, 4, 8, 128, 256, 1024, 16384, },
  { 16, 128, 256, 1024, 16384, },
  { 1, 16, 128, 256, 1024, 16384, },
  { 2, 16, 128, 256, 1024, 16384, },
  { 1, 2, 16, 128, 256, 1024, 16384, },
  { 4, 16, 128, 256, 1024, 16384, },
  { 1, 4, 16, 128, 256, 1024, 16384, },
  { 2, 4, 16, 128, 256, 1024, 16384, },
  { 1, 2, 4, 16, 128, 256, 1024, 16384, },
  { 8, 16, 128, 256, 1024, 16384, },
  { 1, 8, 16, 128, 256, 1024, 16384, },
  { 2, 8, 16, 128, 256, 1024, 16384, },
  { 1, 2, 8, 16, 128, 256, 1024, 16384, },
  { 4, 8, 16, 128, 256, 1024, 16384, },
  { 1, 4, 8, 16, 128, 256, 1024, 16384, },
  { 2, 4, 8, 16, 128, 256, 1024, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 16384, },
  { 32, 128, 256, 1024, 16384, },
  { 1, 32, 128, 256, 1024, 16384, },
  { 2, 32, 128, 256, 1024, 16384, },
  { 1, 2, 32, 128, 256, 1024, 16384, },
  { 4, 32, 128, 256, 1024, 16384, },
  { 1, 4, 32, 128, 256, 1024, 16384, },
  { 2, 4, 32, 128, 256, 1024, 16384, },
  { 1, 2, 4, 32, 128, 256, 1024, 16384, },
  { 8, 32, 128, 256, 1024, 16384, },
  { 1, 8, 32, 128, 256, 1024, 16384, },
  { 2, 8, 32, 128, 256, 1024, 16384, },
  { 1, 2, 8, 32, 128, 256, 1024, 16384, },
  { 4, 8, 32, 128, 256, 1024, 16384, },
  { 1, 4, 8, 32, 128, 256, 1024, 16384, },
  { 2, 4, 8, 32, 128, 256, 1024, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 16384, },
  { 16, 32, 128, 256, 1024, 16384, },
  { 1, 16, 32, 128, 256, 1024, 16384, },
  { 2, 16, 32, 128, 256, 1024, 16384, },
  { 1, 2, 16, 32, 128, 256, 1024, 16384, },
  { 4, 16, 32, 128, 256, 1024, 16384, },
  { 1, 4, 16, 32, 128, 256, 1024, 16384, },
  { 2, 4, 16, 32, 128, 256, 1024, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 16384, },
  { 8, 16, 32, 128, 256, 1024, 16384, },
  { 1, 8, 16, 32, 128, 256, 1024, 16384, },
  { 2, 8, 16, 32, 128, 256, 1024, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 16384, },
  { 4, 8, 16, 32, 128, 256, 1024, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 16384, },
  { 64, 128, 256, 1024, 16384, },
  { 1, 64, 128, 256, 1024, 16384, },
  { 2, 64, 128, 256, 1024, 16384, },
  { 1, 2, 64, 128, 256, 1024, 16384, },
  { 4, 64, 128, 256, 1024, 16384, },
  { 1, 4, 64, 128, 256, 1024, 16384, },
  { 2, 4, 64, 128, 256, 1024, 16384, },
  { 1, 2, 4, 64, 128, 256, 1024, 16384, },
  { 8, 64, 128, 256, 1024, 16384, },
  { 1, 8, 64, 128, 256, 1024, 16384, },
  { 2, 8, 64, 128, 256, 1024, 16384, },
  { 1, 2, 8, 64, 128, 256, 1024, 16384, },
  { 4, 8, 64, 128, 256, 1024, 16384, },
  { 1, 4, 8, 64, 128, 256, 1024, 16384, },
  { 2, 4, 8, 64, 128, 256, 1024, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 16384, },
  { 16, 64, 128, 256, 1024, 16384, },
  { 1, 16, 64, 128, 256, 1024, 16384, },
  { 2, 16, 64, 128, 256, 1024, 16384, },
  { 1, 2, 16, 64, 128, 256, 1024, 16384, },
  { 4, 16, 64, 128, 256, 1024, 16384, },
  { 1, 4, 16, 64, 128, 256, 1024, 16384, },
  { 2, 4, 16, 64, 128, 256, 1024, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 16384, },
  { 8, 16, 64, 128, 256, 1024, 16384, },
  { 1, 8, 16, 64, 128, 256, 1024, 16384, },
  { 2, 8, 16, 64, 128, 256, 1024, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 16384, },
  { 4, 8, 16, 64, 128, 256, 1024, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 16384, },
  { 32, 64, 128, 256, 1024, 16384, },
  { 1, 32, 64, 128, 256, 1024, 16384, },
  { 2, 32, 64, 128, 256, 1024, 16384, },
  { 1, 2, 32, 64, 128, 256, 1024, 16384, },
  { 4, 32, 64, 128, 256, 1024, 16384, },
  { 1, 4, 32, 64, 128, 256, 1024, 16384, },
  { 2, 4, 32, 64, 128, 256, 1024, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 16384, },
  { 8, 32, 64, 128, 256, 1024, 16384, },
  { 1, 8, 32, 64, 128, 256, 1024, 16384, },
  { 2, 8, 32, 64, 128, 256, 1024, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 16384, },
  { 4, 8, 32, 64, 128, 256, 1024, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 16384, },
  { 16, 32, 64, 128, 256, 1024, 16384, },
  { 1, 16, 32, 64, 128, 256, 1024, 16384, },
  { 2, 16, 32, 64, 128, 256, 1024, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 16384, },
  { 4, 16, 32, 64, 128, 256, 1024, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 16384, },
  { 8, 16, 32, 64, 128, 256, 1024, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 16384, },
  { 512, 1024, 16384, },
  { 1, 512, 1024, 16384, },
  { 2, 512, 1024, 16384, },
  { 1, 2, 512, 1024, 16384, },
  { 4, 512, 1024, 16384, },
  { 1, 4, 512, 1024, 16384, },
  { 2, 4, 512, 1024, 16384, },
  { 1, 2, 4, 512, 1024, 16384, },
  { 8, 512, 1024, 16384, },
  { 1, 8, 512, 1024, 16384, },
  { 2, 8, 512, 1024, 16384, },
  { 1, 2, 8, 512, 1024, 16384, },
  { 4, 8, 512, 1024, 16384, },
  { 1, 4, 8, 512, 1024, 16384, },
  { 2, 4, 8, 512, 1024, 16384, },
  { 1, 2, 4, 8, 512, 1024, 16384, },
  { 16, 512, 1024, 16384, },
  { 1, 16, 512, 1024, 16384, },
  { 2, 16, 512, 1024, 16384, },
  { 1, 2, 16, 512, 1024, 16384, },
  { 4, 16, 512, 1024, 16384, },
  { 1, 4, 16, 512, 1024, 16384, },
  { 2, 4, 16, 512, 1024, 16384, },
  { 1, 2, 4, 16, 512, 1024, 16384, },
  { 8, 16, 512, 1024, 16384, },
  { 1, 8, 16, 512, 1024, 16384, },
  { 2, 8, 16, 512, 1024, 16384, },
  { 1, 2, 8, 16, 512, 1024, 16384, },
  { 4, 8, 16, 512, 1024, 16384, },
  { 1, 4, 8, 16, 512, 1024, 16384, },
  { 2, 4, 8, 16, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 512, 1024, 16384, },
  { 32, 512, 1024, 16384, },
  { 1, 32, 512, 1024, 16384, },
  { 2, 32, 512, 1024, 16384, },
  { 1, 2, 32, 512, 1024, 16384, },
  { 4, 32, 512, 1024, 16384, },
  { 1, 4, 32, 512, 1024, 16384, },
  { 2, 4, 32, 512, 1024, 16384, },
  { 1, 2, 4, 32, 512, 1024, 16384, },
  { 8, 32, 512, 1024, 16384, },
  { 1, 8, 32, 512, 1024, 16384, },
  { 2, 8, 32, 512, 1024, 16384, },
  { 1, 2, 8, 32, 512, 1024, 16384, },
  { 4, 8, 32, 512, 1024, 16384, },
  { 1, 4, 8, 32, 512, 1024, 16384, },
  { 2, 4, 8, 32, 512, 1024, 16384, },
  { 1, 2, 4, 8, 32, 512, 1024, 16384, },
  { 16, 32, 512, 1024, 16384, },
  { 1, 16, 32, 512, 1024, 16384, },
  { 2, 16, 32, 512, 1024, 16384, },
  { 1, 2, 16, 32, 512, 1024, 16384, },
  { 4, 16, 32, 512, 1024, 16384, },
  { 1, 4, 16, 32, 512, 1024, 16384, },
  { 2, 4, 16, 32, 512, 1024, 16384, },
  { 1, 2, 4, 16, 32, 512, 1024, 16384, },
  { 8, 16, 32, 512, 1024, 16384, },
  { 1, 8, 16, 32, 512, 1024, 16384, },
  { 2, 8, 16, 32, 512, 1024, 16384, },
  { 1, 2, 8, 16, 32, 512, 1024, 16384, },
  { 4, 8, 16, 32, 512, 1024, 16384, },
  { 1, 4, 8, 16, 32, 512, 1024, 16384, },
  { 2, 4, 8, 16, 32, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 16384, },
  { 64, 512, 1024, 16384, },
  { 1, 64, 512, 1024, 16384, },
  { 2, 64, 512, 1024, 16384, },
  { 1, 2, 64, 512, 1024, 16384, },
  { 4, 64, 512, 1024, 16384, },
  { 1, 4, 64, 512, 1024, 16384, },
  { 2, 4, 64, 512, 1024, 16384, },
  { 1, 2, 4, 64, 512, 1024, 16384, },
  { 8, 64, 512, 1024, 16384, },
  { 1, 8, 64, 512, 1024, 16384, },
  { 2, 8, 64, 512, 1024, 16384, },
  { 1, 2, 8, 64, 512, 1024, 16384, },
  { 4, 8, 64, 512, 1024, 16384, },
  { 1, 4, 8, 64, 512, 1024, 16384, },
  { 2, 4, 8, 64, 512, 1024, 16384, },
  { 1, 2, 4, 8, 64, 512, 1024, 16384, },
  { 16, 64, 512, 1024, 16384, },
  { 1, 16, 64, 512, 1024, 16384, },
  { 2, 16, 64, 512, 1024, 16384, },
  { 1, 2, 16, 64, 512, 1024, 16384, },
  { 4, 16, 64, 512, 1024, 16384, },
  { 1, 4, 16, 64, 512, 1024, 16384, },
  { 2, 4, 16, 64, 512, 1024, 16384, },
  { 1, 2, 4, 16, 64, 512, 1024, 16384, },
  { 8, 16, 64, 512, 1024, 16384, },
  { 1, 8, 16, 64, 512, 1024, 16384, },
  { 2, 8, 16, 64, 512, 1024, 16384, },
  { 1, 2, 8, 16, 64, 512, 1024, 16384, },
  { 4, 8, 16, 64, 512, 1024, 16384, },
  { 1, 4, 8, 16, 64, 512, 1024, 16384, },
  { 2, 4, 8, 16, 64, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 16384, },
  { 32, 64, 512, 1024, 16384, },
  { 1, 32, 64, 512, 1024, 16384, },
  { 2, 32, 64, 512, 1024, 16384, },
  { 1, 2, 32, 64, 512, 1024, 16384, },
  { 4, 32, 64, 512, 1024, 16384, },
  { 1, 4, 32, 64, 512, 1024, 16384, },
  { 2, 4, 32, 64, 512, 1024, 16384, },
  { 1, 2, 4, 32, 64, 512, 1024, 16384, },
  { 8, 32, 64, 512, 1024, 16384, },
  { 1, 8, 32, 64, 512, 1024, 16384, },
  { 2, 8, 32, 64, 512, 1024, 16384, },
  { 1, 2, 8, 32, 64, 512, 1024, 16384, },
  { 4, 8, 32, 64, 512, 1024, 16384, },
  { 1, 4, 8, 32, 64, 512, 1024, 16384, },
  { 2, 4, 8, 32, 64, 512, 1024, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 16384, },
  { 16, 32, 64, 512, 1024, 16384, },
  { 1, 16, 32, 64, 512, 1024, 16384, },
  { 2, 16, 32, 64, 512, 1024, 16384, },
  { 1, 2, 16, 32, 64, 512, 1024, 16384, },
  { 4, 16, 32, 64, 512, 1024, 16384, },
  { 1, 4, 16, 32, 64, 512, 1024, 16384, },
  { 2, 4, 16, 32, 64, 512, 1024, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 16384, },
  { 8, 16, 32, 64, 512, 1024, 16384, },
  { 1, 8, 16, 32, 64, 512, 1024, 16384, },
  { 2, 8, 16, 32, 64, 512, 1024, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 16384, },
  { 4, 8, 16, 32, 64, 512, 1024, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 16384, },
  { 128, 512, 1024, 16384, },
  { 1, 128, 512, 1024, 16384, },
  { 2, 128, 512, 1024, 16384, },
  { 1, 2, 128, 512, 1024, 16384, },
  { 4, 128, 512, 1024, 16384, },
  { 1, 4, 128, 512, 1024, 16384, },
  { 2, 4, 128, 512, 1024, 16384, },
  { 1, 2, 4, 128, 512, 1024, 16384, },
  { 8, 128, 512, 1024, 16384, },
  { 1, 8, 128, 512, 1024, 16384, },
  { 2, 8, 128, 512, 1024, 16384, },
  { 1, 2, 8, 128, 512, 1024, 16384, },
  { 4, 8, 128, 512, 1024, 16384, },
  { 1, 4, 8, 128, 512, 1024, 16384, },
  { 2, 4, 8, 128, 512, 1024, 16384, },
  { 1, 2, 4, 8, 128, 512, 1024, 16384, },
  { 16, 128, 512, 1024, 16384, },
  { 1, 16, 128, 512, 1024, 16384, },
  { 2, 16, 128, 512, 1024, 16384, },
  { 1, 2, 16, 128, 512, 1024, 16384, },
  { 4, 16, 128, 512, 1024, 16384, },
  { 1, 4, 16, 128, 512, 1024, 16384, },
  { 2, 4, 16, 128, 512, 1024, 16384, },
  { 1, 2, 4, 16, 128, 512, 1024, 16384, },
  { 8, 16, 128, 512, 1024, 16384, },
  { 1, 8, 16, 128, 512, 1024, 16384, },
  { 2, 8, 16, 128, 512, 1024, 16384, },
  { 1, 2, 8, 16, 128, 512, 1024, 16384, },
  { 4, 8, 16, 128, 512, 1024, 16384, },
  { 1, 4, 8, 16, 128, 512, 1024, 16384, },
  { 2, 4, 8, 16, 128, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 16384, },
  { 32, 128, 512, 1024, 16384, },
  { 1, 32, 128, 512, 1024, 16384, },
  { 2, 32, 128, 512, 1024, 16384, },
  { 1, 2, 32, 128, 512, 1024, 16384, },
  { 4, 32, 128, 512, 1024, 16384, },
  { 1, 4, 32, 128, 512, 1024, 16384, },
  { 2, 4, 32, 128, 512, 1024, 16384, },
  { 1, 2, 4, 32, 128, 512, 1024, 16384, },
  { 8, 32, 128, 512, 1024, 16384, },
  { 1, 8, 32, 128, 512, 1024, 16384, },
  { 2, 8, 32, 128, 512, 1024, 16384, },
  { 1, 2, 8, 32, 128, 512, 1024, 16384, },
  { 4, 8, 32, 128, 512, 1024, 16384, },
  { 1, 4, 8, 32, 128, 512, 1024, 16384, },
  { 2, 4, 8, 32, 128, 512, 1024, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 16384, },
  { 16, 32, 128, 512, 1024, 16384, },
  { 1, 16, 32, 128, 512, 1024, 16384, },
  { 2, 16, 32, 128, 512, 1024, 16384, },
  { 1, 2, 16, 32, 128, 512, 1024, 16384, },
  { 4, 16, 32, 128, 512, 1024, 16384, },
  { 1, 4, 16, 32, 128, 512, 1024, 16384, },
  { 2, 4, 16, 32, 128, 512, 1024, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 16384, },
  { 8, 16, 32, 128, 512, 1024, 16384, },
  { 1, 8, 16, 32, 128, 512, 1024, 16384, },
  { 2, 8, 16, 32, 128, 512, 1024, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 16384, },
  { 4, 8, 16, 32, 128, 512, 1024, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 16384, },
  { 64, 128, 512, 1024, 16384, },
  { 1, 64, 128, 512, 1024, 16384, },
  { 2, 64, 128, 512, 1024, 16384, },
  { 1, 2, 64, 128, 512, 1024, 16384, },
  { 4, 64, 128, 512, 1024, 16384, },
  { 1, 4, 64, 128, 512, 1024, 16384, },
  { 2, 4, 64, 128, 512, 1024, 16384, },
  { 1, 2, 4, 64, 128, 512, 1024, 16384, },
  { 8, 64, 128, 512, 1024, 16384, },
  { 1, 8, 64, 128, 512, 1024, 16384, },
  { 2, 8, 64, 128, 512, 1024, 16384, },
  { 1, 2, 8, 64, 128, 512, 1024, 16384, },
  { 4, 8, 64, 128, 512, 1024, 16384, },
  { 1, 4, 8, 64, 128, 512, 1024, 16384, },
  { 2, 4, 8, 64, 128, 512, 1024, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 16384, },
  { 16, 64, 128, 512, 1024, 16384, },
  { 1, 16, 64, 128, 512, 1024, 16384, },
  { 2, 16, 64, 128, 512, 1024, 16384, },
  { 1, 2, 16, 64, 128, 512, 1024, 16384, },
  { 4, 16, 64, 128, 512, 1024, 16384, },
  { 1, 4, 16, 64, 128, 512, 1024, 16384, },
  { 2, 4, 16, 64, 128, 512, 1024, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 16384, },
  { 8, 16, 64, 128, 512, 1024, 16384, },
  { 1, 8, 16, 64, 128, 512, 1024, 16384, },
  { 2, 8, 16, 64, 128, 512, 1024, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 16384, },
  { 4, 8, 16, 64, 128, 512, 1024, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 16384, },
  { 32, 64, 128, 512, 1024, 16384, },
  { 1, 32, 64, 128, 512, 1024, 16384, },
  { 2, 32, 64, 128, 512, 1024, 16384, },
  { 1, 2, 32, 64, 128, 512, 1024, 16384, },
  { 4, 32, 64, 128, 512, 1024, 16384, },
  { 1, 4, 32, 64, 128, 512, 1024, 16384, },
  { 2, 4, 32, 64, 128, 512, 1024, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 16384, },
  { 8, 32, 64, 128, 512, 1024, 16384, },
  { 1, 8, 32, 64, 128, 512, 1024, 16384, },
  { 2, 8, 32, 64, 128, 512, 1024, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 16384, },
  { 4, 8, 32, 64, 128, 512, 1024, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 16384, },
  { 16, 32, 64, 128, 512, 1024, 16384, },
  { 1, 16, 32, 64, 128, 512, 1024, 16384, },
  { 2, 16, 32, 64, 128, 512, 1024, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 16384, },
  { 4, 16, 32, 64, 128, 512, 1024, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 16384, },
  { 8, 16, 32, 64, 128, 512, 1024, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 16384, },
  { 256, 512, 1024, 16384, },
  { 1, 256, 512, 1024, 16384, },
  { 2, 256, 512, 1024, 16384, },
  { 1, 2, 256, 512, 1024, 16384, },
  { 4, 256, 512, 1024, 16384, },
  { 1, 4, 256, 512, 1024, 16384, },
  { 2, 4, 256, 512, 1024, 16384, },
  { 1, 2, 4, 256, 512, 1024, 16384, },
  { 8, 256, 512, 1024, 16384, },
  { 1, 8, 256, 512, 1024, 16384, },
  { 2, 8, 256, 512, 1024, 16384, },
  { 1, 2, 8, 256, 512, 1024, 16384, },
  { 4, 8, 256, 512, 1024, 16384, },
  { 1, 4, 8, 256, 512, 1024, 16384, },
  { 2, 4, 8, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 256, 512, 1024, 16384, },
  { 16, 256, 512, 1024, 16384, },
  { 1, 16, 256, 512, 1024, 16384, },
  { 2, 16, 256, 512, 1024, 16384, },
  { 1, 2, 16, 256, 512, 1024, 16384, },
  { 4, 16, 256, 512, 1024, 16384, },
  { 1, 4, 16, 256, 512, 1024, 16384, },
  { 2, 4, 16, 256, 512, 1024, 16384, },
  { 1, 2, 4, 16, 256, 512, 1024, 16384, },
  { 8, 16, 256, 512, 1024, 16384, },
  { 1, 8, 16, 256, 512, 1024, 16384, },
  { 2, 8, 16, 256, 512, 1024, 16384, },
  { 1, 2, 8, 16, 256, 512, 1024, 16384, },
  { 4, 8, 16, 256, 512, 1024, 16384, },
  { 1, 4, 8, 16, 256, 512, 1024, 16384, },
  { 2, 4, 8, 16, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 16384, },
  { 32, 256, 512, 1024, 16384, },
  { 1, 32, 256, 512, 1024, 16384, },
  { 2, 32, 256, 512, 1024, 16384, },
  { 1, 2, 32, 256, 512, 1024, 16384, },
  { 4, 32, 256, 512, 1024, 16384, },
  { 1, 4, 32, 256, 512, 1024, 16384, },
  { 2, 4, 32, 256, 512, 1024, 16384, },
  { 1, 2, 4, 32, 256, 512, 1024, 16384, },
  { 8, 32, 256, 512, 1024, 16384, },
  { 1, 8, 32, 256, 512, 1024, 16384, },
  { 2, 8, 32, 256, 512, 1024, 16384, },
  { 1, 2, 8, 32, 256, 512, 1024, 16384, },
  { 4, 8, 32, 256, 512, 1024, 16384, },
  { 1, 4, 8, 32, 256, 512, 1024, 16384, },
  { 2, 4, 8, 32, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 16384, },
  { 16, 32, 256, 512, 1024, 16384, },
  { 1, 16, 32, 256, 512, 1024, 16384, },
  { 2, 16, 32, 256, 512, 1024, 16384, },
  { 1, 2, 16, 32, 256, 512, 1024, 16384, },
  { 4, 16, 32, 256, 512, 1024, 16384, },
  { 1, 4, 16, 32, 256, 512, 1024, 16384, },
  { 2, 4, 16, 32, 256, 512, 1024, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 16384, },
  { 8, 16, 32, 256, 512, 1024, 16384, },
  { 1, 8, 16, 32, 256, 512, 1024, 16384, },
  { 2, 8, 16, 32, 256, 512, 1024, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 16384, },
  { 4, 8, 16, 32, 256, 512, 1024, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 16384, },
  { 64, 256, 512, 1024, 16384, },
  { 1, 64, 256, 512, 1024, 16384, },
  { 2, 64, 256, 512, 1024, 16384, },
  { 1, 2, 64, 256, 512, 1024, 16384, },
  { 4, 64, 256, 512, 1024, 16384, },
  { 1, 4, 64, 256, 512, 1024, 16384, },
  { 2, 4, 64, 256, 512, 1024, 16384, },
  { 1, 2, 4, 64, 256, 512, 1024, 16384, },
  { 8, 64, 256, 512, 1024, 16384, },
  { 1, 8, 64, 256, 512, 1024, 16384, },
  { 2, 8, 64, 256, 512, 1024, 16384, },
  { 1, 2, 8, 64, 256, 512, 1024, 16384, },
  { 4, 8, 64, 256, 512, 1024, 16384, },
  { 1, 4, 8, 64, 256, 512, 1024, 16384, },
  { 2, 4, 8, 64, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 16384, },
  { 16, 64, 256, 512, 1024, 16384, },
  { 1, 16, 64, 256, 512, 1024, 16384, },
  { 2, 16, 64, 256, 512, 1024, 16384, },
  { 1, 2, 16, 64, 256, 512, 1024, 16384, },
  { 4, 16, 64, 256, 512, 1024, 16384, },
  { 1, 4, 16, 64, 256, 512, 1024, 16384, },
  { 2, 4, 16, 64, 256, 512, 1024, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 16384, },
  { 8, 16, 64, 256, 512, 1024, 16384, },
  { 1, 8, 16, 64, 256, 512, 1024, 16384, },
  { 2, 8, 16, 64, 256, 512, 1024, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 16384, },
  { 4, 8, 16, 64, 256, 512, 1024, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 16384, },
  { 32, 64, 256, 512, 1024, 16384, },
  { 1, 32, 64, 256, 512, 1024, 16384, },
  { 2, 32, 64, 256, 512, 1024, 16384, },
  { 1, 2, 32, 64, 256, 512, 1024, 16384, },
  { 4, 32, 64, 256, 512, 1024, 16384, },
  { 1, 4, 32, 64, 256, 512, 1024, 16384, },
  { 2, 4, 32, 64, 256, 512, 1024, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 16384, },
  { 8, 32, 64, 256, 512, 1024, 16384, },
  { 1, 8, 32, 64, 256, 512, 1024, 16384, },
  { 2, 8, 32, 64, 256, 512, 1024, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 16384, },
  { 4, 8, 32, 64, 256, 512, 1024, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 16384, },
  { 16, 32, 64, 256, 512, 1024, 16384, },
  { 1, 16, 32, 64, 256, 512, 1024, 16384, },
  { 2, 16, 32, 64, 256, 512, 1024, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 16384, },
  { 4, 16, 32, 64, 256, 512, 1024, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 16384, },
  { 8, 16, 32, 64, 256, 512, 1024, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 16384, },
  { 128, 256, 512, 1024, 16384, },
  { 1, 128, 256, 512, 1024, 16384, },
  { 2, 128, 256, 512, 1024, 16384, },
  { 1, 2, 128, 256, 512, 1024, 16384, },
  { 4, 128, 256, 512, 1024, 16384, },
  { 1, 4, 128, 256, 512, 1024, 16384, },
  { 2, 4, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 128, 256, 512, 1024, 16384, },
  { 8, 128, 256, 512, 1024, 16384, },
  { 1, 8, 128, 256, 512, 1024, 16384, },
  { 2, 8, 128, 256, 512, 1024, 16384, },
  { 1, 2, 8, 128, 256, 512, 1024, 16384, },
  { 4, 8, 128, 256, 512, 1024, 16384, },
  { 1, 4, 8, 128, 256, 512, 1024, 16384, },
  { 2, 4, 8, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 16384, },
  { 16, 128, 256, 512, 1024, 16384, },
  { 1, 16, 128, 256, 512, 1024, 16384, },
  { 2, 16, 128, 256, 512, 1024, 16384, },
  { 1, 2, 16, 128, 256, 512, 1024, 16384, },
  { 4, 16, 128, 256, 512, 1024, 16384, },
  { 1, 4, 16, 128, 256, 512, 1024, 16384, },
  { 2, 4, 16, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 16384, },
  { 8, 16, 128, 256, 512, 1024, 16384, },
  { 1, 8, 16, 128, 256, 512, 1024, 16384, },
  { 2, 8, 16, 128, 256, 512, 1024, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 16384, },
  { 4, 8, 16, 128, 256, 512, 1024, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 16384, },
  { 32, 128, 256, 512, 1024, 16384, },
  { 1, 32, 128, 256, 512, 1024, 16384, },
  { 2, 32, 128, 256, 512, 1024, 16384, },
  { 1, 2, 32, 128, 256, 512, 1024, 16384, },
  { 4, 32, 128, 256, 512, 1024, 16384, },
  { 1, 4, 32, 128, 256, 512, 1024, 16384, },
  { 2, 4, 32, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 16384, },
  { 8, 32, 128, 256, 512, 1024, 16384, },
  { 1, 8, 32, 128, 256, 512, 1024, 16384, },
  { 2, 8, 32, 128, 256, 512, 1024, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 16384, },
  { 4, 8, 32, 128, 256, 512, 1024, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 16384, },
  { 16, 32, 128, 256, 512, 1024, 16384, },
  { 1, 16, 32, 128, 256, 512, 1024, 16384, },
  { 2, 16, 32, 128, 256, 512, 1024, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 16384, },
  { 4, 16, 32, 128, 256, 512, 1024, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 16384, },
  { 8, 16, 32, 128, 256, 512, 1024, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 16384, },
  { 64, 128, 256, 512, 1024, 16384, },
  { 1, 64, 128, 256, 512, 1024, 16384, },
  { 2, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 64, 128, 256, 512, 1024, 16384, },
  { 4, 64, 128, 256, 512, 1024, 16384, },
  { 1, 4, 64, 128, 256, 512, 1024, 16384, },
  { 2, 4, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 16384, },
  { 8, 64, 128, 256, 512, 1024, 16384, },
  { 1, 8, 64, 128, 256, 512, 1024, 16384, },
  { 2, 8, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 16384, },
  { 4, 8, 64, 128, 256, 512, 1024, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 16384, },
  { 16, 64, 128, 256, 512, 1024, 16384, },
  { 1, 16, 64, 128, 256, 512, 1024, 16384, },
  { 2, 16, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 16384, },
  { 4, 16, 64, 128, 256, 512, 1024, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 16384, },
  { 8, 16, 64, 128, 256, 512, 1024, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 16384, },
  { 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 16384, },
  { 4, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 16384, },
  { 8, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 16384, },
  { 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, },
  { 2048, 16384, },
  { 1, 2048, 16384, },
  { 2, 2048, 16384, },
  { 1, 2, 2048, 16384, },
  { 4, 2048, 16384, },
  { 1, 4, 2048, 16384, },
  { 2, 4, 2048, 16384, },
  { 1, 2, 4, 2048, 16384, },
  { 8, 2048, 16384, },
  { 1, 8, 2048, 16384, },
  { 2, 8, 2048, 16384, },
  { 1, 2, 8, 2048, 16384, },
  { 4, 8, 2048, 16384, },
  { 1, 4, 8, 2048, 16384, },
  { 2, 4, 8, 2048, 16384, },
  { 1, 2, 4, 8, 2048, 16384, },
  { 16, 2048, 16384, },
  { 1, 16, 2048, 16384, },
  { 2, 16, 2048, 16384, },
  { 1, 2, 16, 2048, 16384, },
  { 4, 16, 2048, 16384, },
  { 1, 4, 16, 2048, 16384, },
  { 2, 4, 16, 2048, 16384, },
  { 1, 2, 4, 16, 2048, 16384, },
  { 8, 16, 2048, 16384, },
  { 1, 8, 16, 2048, 16384, },
  { 2, 8, 16, 2048, 16384, },
  { 1, 2, 8, 16, 2048, 16384, },
  { 4, 8, 16, 2048, 16384, },
  { 1, 4, 8, 16, 2048, 16384, },
  { 2, 4, 8, 16, 2048, 16384, },
  { 1, 2, 4, 8, 16, 2048, 16384, },
  { 32, 2048, 16384, },
  { 1, 32, 2048, 16384, },
  { 2, 32, 2048, 16384, },
  { 1, 2, 32, 2048, 16384, },
  { 4, 32, 2048, 16384, },
  { 1, 4, 32, 2048, 16384, },
  { 2, 4, 32, 2048, 16384, },
  { 1, 2, 4, 32, 2048, 16384, },
  { 8, 32, 2048, 16384, },
  { 1, 8, 32, 2048, 16384, },
  { 2, 8, 32, 2048, 16384, },
  { 1, 2, 8, 32, 2048, 16384, },
  { 4, 8, 32, 2048, 16384, },
  { 1, 4, 8, 32, 2048, 16384, },
  { 2, 4, 8, 32, 2048, 16384, },
  { 1, 2, 4, 8, 32, 2048, 16384, },
  { 16, 32, 2048, 16384, },
  { 1, 16, 32, 2048, 16384, },
  { 2, 16, 32, 2048, 16384, },
  { 1, 2, 16, 32, 2048, 16384, },
  { 4, 16, 32, 2048, 16384, },
  { 1, 4, 16, 32, 2048, 16384, },
  { 2, 4, 16, 32, 2048, 16384, },
  { 1, 2, 4, 16, 32, 2048, 16384, },
  { 8, 16, 32, 2048, 16384, },
  { 1, 8, 16, 32, 2048, 16384, },
  { 2, 8, 16, 32, 2048, 16384, },
  { 1, 2, 8, 16, 32, 2048, 16384, },
  { 4, 8, 16, 32, 2048, 16384, },
  { 1, 4, 8, 16, 32, 2048, 16384, },
  { 2, 4, 8, 16, 32, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 2048, 16384, },
  { 64, 2048, 16384, },
  { 1, 64, 2048, 16384, },
  { 2, 64, 2048, 16384, },
  { 1, 2, 64, 2048, 16384, },
  { 4, 64, 2048, 16384, },
  { 1, 4, 64, 2048, 16384, },
  { 2, 4, 64, 2048, 16384, },
  { 1, 2, 4, 64, 2048, 16384, },
  { 8, 64, 2048, 16384, },
  { 1, 8, 64, 2048, 16384, },
  { 2, 8, 64, 2048, 16384, },
  { 1, 2, 8, 64, 2048, 16384, },
  { 4, 8, 64, 2048, 16384, },
  { 1, 4, 8, 64, 2048, 16384, },
  { 2, 4, 8, 64, 2048, 16384, },
  { 1, 2, 4, 8, 64, 2048, 16384, },
  { 16, 64, 2048, 16384, },
  { 1, 16, 64, 2048, 16384, },
  { 2, 16, 64, 2048, 16384, },
  { 1, 2, 16, 64, 2048, 16384, },
  { 4, 16, 64, 2048, 16384, },
  { 1, 4, 16, 64, 2048, 16384, },
  { 2, 4, 16, 64, 2048, 16384, },
  { 1, 2, 4, 16, 64, 2048, 16384, },
  { 8, 16, 64, 2048, 16384, },
  { 1, 8, 16, 64, 2048, 16384, },
  { 2, 8, 16, 64, 2048, 16384, },
  { 1, 2, 8, 16, 64, 2048, 16384, },
  { 4, 8, 16, 64, 2048, 16384, },
  { 1, 4, 8, 16, 64, 2048, 16384, },
  { 2, 4, 8, 16, 64, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 2048, 16384, },
  { 32, 64, 2048, 16384, },
  { 1, 32, 64, 2048, 16384, },
  { 2, 32, 64, 2048, 16384, },
  { 1, 2, 32, 64, 2048, 16384, },
  { 4, 32, 64, 2048, 16384, },
  { 1, 4, 32, 64, 2048, 16384, },
  { 2, 4, 32, 64, 2048, 16384, },
  { 1, 2, 4, 32, 64, 2048, 16384, },
  { 8, 32, 64, 2048, 16384, },
  { 1, 8, 32, 64, 2048, 16384, },
  { 2, 8, 32, 64, 2048, 16384, },
  { 1, 2, 8, 32, 64, 2048, 16384, },
  { 4, 8, 32, 64, 2048, 16384, },
  { 1, 4, 8, 32, 64, 2048, 16384, },
  { 2, 4, 8, 32, 64, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 2048, 16384, },
  { 16, 32, 64, 2048, 16384, },
  { 1, 16, 32, 64, 2048, 16384, },
  { 2, 16, 32, 64, 2048, 16384, },
  { 1, 2, 16, 32, 64, 2048, 16384, },
  { 4, 16, 32, 64, 2048, 16384, },
  { 1, 4, 16, 32, 64, 2048, 16384, },
  { 2, 4, 16, 32, 64, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 2048, 16384, },
  { 8, 16, 32, 64, 2048, 16384, },
  { 1, 8, 16, 32, 64, 2048, 16384, },
  { 2, 8, 16, 32, 64, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 2048, 16384, },
  { 4, 8, 16, 32, 64, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 16384, },
  { 128, 2048, 16384, },
  { 1, 128, 2048, 16384, },
  { 2, 128, 2048, 16384, },
  { 1, 2, 128, 2048, 16384, },
  { 4, 128, 2048, 16384, },
  { 1, 4, 128, 2048, 16384, },
  { 2, 4, 128, 2048, 16384, },
  { 1, 2, 4, 128, 2048, 16384, },
  { 8, 128, 2048, 16384, },
  { 1, 8, 128, 2048, 16384, },
  { 2, 8, 128, 2048, 16384, },
  { 1, 2, 8, 128, 2048, 16384, },
  { 4, 8, 128, 2048, 16384, },
  { 1, 4, 8, 128, 2048, 16384, },
  { 2, 4, 8, 128, 2048, 16384, },
  { 1, 2, 4, 8, 128, 2048, 16384, },
  { 16, 128, 2048, 16384, },
  { 1, 16, 128, 2048, 16384, },
  { 2, 16, 128, 2048, 16384, },
  { 1, 2, 16, 128, 2048, 16384, },
  { 4, 16, 128, 2048, 16384, },
  { 1, 4, 16, 128, 2048, 16384, },
  { 2, 4, 16, 128, 2048, 16384, },
  { 1, 2, 4, 16, 128, 2048, 16384, },
  { 8, 16, 128, 2048, 16384, },
  { 1, 8, 16, 128, 2048, 16384, },
  { 2, 8, 16, 128, 2048, 16384, },
  { 1, 2, 8, 16, 128, 2048, 16384, },
  { 4, 8, 16, 128, 2048, 16384, },
  { 1, 4, 8, 16, 128, 2048, 16384, },
  { 2, 4, 8, 16, 128, 2048, 16384, },
  { 1, 2, 4, 8, 16, 128, 2048, 16384, },
  { 32, 128, 2048, 16384, },
  { 1, 32, 128, 2048, 16384, },
  { 2, 32, 128, 2048, 16384, },
  { 1, 2, 32, 128, 2048, 16384, },
  { 4, 32, 128, 2048, 16384, },
  { 1, 4, 32, 128, 2048, 16384, },
  { 2, 4, 32, 128, 2048, 16384, },
  { 1, 2, 4, 32, 128, 2048, 16384, },
  { 8, 32, 128, 2048, 16384, },
  { 1, 8, 32, 128, 2048, 16384, },
  { 2, 8, 32, 128, 2048, 16384, },
  { 1, 2, 8, 32, 128, 2048, 16384, },
  { 4, 8, 32, 128, 2048, 16384, },
  { 1, 4, 8, 32, 128, 2048, 16384, },
  { 2, 4, 8, 32, 128, 2048, 16384, },
  { 1, 2, 4, 8, 32, 128, 2048, 16384, },
  { 16, 32, 128, 2048, 16384, },
  { 1, 16, 32, 128, 2048, 16384, },
  { 2, 16, 32, 128, 2048, 16384, },
  { 1, 2, 16, 32, 128, 2048, 16384, },
  { 4, 16, 32, 128, 2048, 16384, },
  { 1, 4, 16, 32, 128, 2048, 16384, },
  { 2, 4, 16, 32, 128, 2048, 16384, },
  { 1, 2, 4, 16, 32, 128, 2048, 16384, },
  { 8, 16, 32, 128, 2048, 16384, },
  { 1, 8, 16, 32, 128, 2048, 16384, },
  { 2, 8, 16, 32, 128, 2048, 16384, },
  { 1, 2, 8, 16, 32, 128, 2048, 16384, },
  { 4, 8, 16, 32, 128, 2048, 16384, },
  { 1, 4, 8, 16, 32, 128, 2048, 16384, },
  { 2, 4, 8, 16, 32, 128, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 16384, },
  { 64, 128, 2048, 16384, },
  { 1, 64, 128, 2048, 16384, },
  { 2, 64, 128, 2048, 16384, },
  { 1, 2, 64, 128, 2048, 16384, },
  { 4, 64, 128, 2048, 16384, },
  { 1, 4, 64, 128, 2048, 16384, },
  { 2, 4, 64, 128, 2048, 16384, },
  { 1, 2, 4, 64, 128, 2048, 16384, },
  { 8, 64, 128, 2048, 16384, },
  { 1, 8, 64, 128, 2048, 16384, },
  { 2, 8, 64, 128, 2048, 16384, },
  { 1, 2, 8, 64, 128, 2048, 16384, },
  { 4, 8, 64, 128, 2048, 16384, },
  { 1, 4, 8, 64, 128, 2048, 16384, },
  { 2, 4, 8, 64, 128, 2048, 16384, },
  { 1, 2, 4, 8, 64, 128, 2048, 16384, },
  { 16, 64, 128, 2048, 16384, },
  { 1, 16, 64, 128, 2048, 16384, },
  { 2, 16, 64, 128, 2048, 16384, },
  { 1, 2, 16, 64, 128, 2048, 16384, },
  { 4, 16, 64, 128, 2048, 16384, },
  { 1, 4, 16, 64, 128, 2048, 16384, },
  { 2, 4, 16, 64, 128, 2048, 16384, },
  { 1, 2, 4, 16, 64, 128, 2048, 16384, },
  { 8, 16, 64, 128, 2048, 16384, },
  { 1, 8, 16, 64, 128, 2048, 16384, },
  { 2, 8, 16, 64, 128, 2048, 16384, },
  { 1, 2, 8, 16, 64, 128, 2048, 16384, },
  { 4, 8, 16, 64, 128, 2048, 16384, },
  { 1, 4, 8, 16, 64, 128, 2048, 16384, },
  { 2, 4, 8, 16, 64, 128, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 16384, },
  { 32, 64, 128, 2048, 16384, },
  { 1, 32, 64, 128, 2048, 16384, },
  { 2, 32, 64, 128, 2048, 16384, },
  { 1, 2, 32, 64, 128, 2048, 16384, },
  { 4, 32, 64, 128, 2048, 16384, },
  { 1, 4, 32, 64, 128, 2048, 16384, },
  { 2, 4, 32, 64, 128, 2048, 16384, },
  { 1, 2, 4, 32, 64, 128, 2048, 16384, },
  { 8, 32, 64, 128, 2048, 16384, },
  { 1, 8, 32, 64, 128, 2048, 16384, },
  { 2, 8, 32, 64, 128, 2048, 16384, },
  { 1, 2, 8, 32, 64, 128, 2048, 16384, },
  { 4, 8, 32, 64, 128, 2048, 16384, },
  { 1, 4, 8, 32, 64, 128, 2048, 16384, },
  { 2, 4, 8, 32, 64, 128, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 16384, },
  { 16, 32, 64, 128, 2048, 16384, },
  { 1, 16, 32, 64, 128, 2048, 16384, },
  { 2, 16, 32, 64, 128, 2048, 16384, },
  { 1, 2, 16, 32, 64, 128, 2048, 16384, },
  { 4, 16, 32, 64, 128, 2048, 16384, },
  { 1, 4, 16, 32, 64, 128, 2048, 16384, },
  { 2, 4, 16, 32, 64, 128, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 16384, },
  { 8, 16, 32, 64, 128, 2048, 16384, },
  { 1, 8, 16, 32, 64, 128, 2048, 16384, },
  { 2, 8, 16, 32, 64, 128, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 16384, },
  { 4, 8, 16, 32, 64, 128, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 16384, },
  { 256, 2048, 16384, },
  { 1, 256, 2048, 16384, },
  { 2, 256, 2048, 16384, },
  { 1, 2, 256, 2048, 16384, },
  { 4, 256, 2048, 16384, },
  { 1, 4, 256, 2048, 16384, },
  { 2, 4, 256, 2048, 16384, },
  { 1, 2, 4, 256, 2048, 16384, },
  { 8, 256, 2048, 16384, },
  { 1, 8, 256, 2048, 16384, },
  { 2, 8, 256, 2048, 16384, },
  { 1, 2, 8, 256, 2048, 16384, },
  { 4, 8, 256, 2048, 16384, },
  { 1, 4, 8, 256, 2048, 16384, },
  { 2, 4, 8, 256, 2048, 16384, },
  { 1, 2, 4, 8, 256, 2048, 16384, },
  { 16, 256, 2048, 16384, },
  { 1, 16, 256, 2048, 16384, },
  { 2, 16, 256, 2048, 16384, },
  { 1, 2, 16, 256, 2048, 16384, },
  { 4, 16, 256, 2048, 16384, },
  { 1, 4, 16, 256, 2048, 16384, },
  { 2, 4, 16, 256, 2048, 16384, },
  { 1, 2, 4, 16, 256, 2048, 16384, },
  { 8, 16, 256, 2048, 16384, },
  { 1, 8, 16, 256, 2048, 16384, },
  { 2, 8, 16, 256, 2048, 16384, },
  { 1, 2, 8, 16, 256, 2048, 16384, },
  { 4, 8, 16, 256, 2048, 16384, },
  { 1, 4, 8, 16, 256, 2048, 16384, },
  { 2, 4, 8, 16, 256, 2048, 16384, },
  { 1, 2, 4, 8, 16, 256, 2048, 16384, },
  { 32, 256, 2048, 16384, },
  { 1, 32, 256, 2048, 16384, },
  { 2, 32, 256, 2048, 16384, },
  { 1, 2, 32, 256, 2048, 16384, },
  { 4, 32, 256, 2048, 16384, },
  { 1, 4, 32, 256, 2048, 16384, },
  { 2, 4, 32, 256, 2048, 16384, },
  { 1, 2, 4, 32, 256, 2048, 16384, },
  { 8, 32, 256, 2048, 16384, },
  { 1, 8, 32, 256, 2048, 16384, },
  { 2, 8, 32, 256, 2048, 16384, },
  { 1, 2, 8, 32, 256, 2048, 16384, },
  { 4, 8, 32, 256, 2048, 16384, },
  { 1, 4, 8, 32, 256, 2048, 16384, },
  { 2, 4, 8, 32, 256, 2048, 16384, },
  { 1, 2, 4, 8, 32, 256, 2048, 16384, },
  { 16, 32, 256, 2048, 16384, },
  { 1, 16, 32, 256, 2048, 16384, },
  { 2, 16, 32, 256, 2048, 16384, },
  { 1, 2, 16, 32, 256, 2048, 16384, },
  { 4, 16, 32, 256, 2048, 16384, },
  { 1, 4, 16, 32, 256, 2048, 16384, },
  { 2, 4, 16, 32, 256, 2048, 16384, },
  { 1, 2, 4, 16, 32, 256, 2048, 16384, },
  { 8, 16, 32, 256, 2048, 16384, },
  { 1, 8, 16, 32, 256, 2048, 16384, },
  { 2, 8, 16, 32, 256, 2048, 16384, },
  { 1, 2, 8, 16, 32, 256, 2048, 16384, },
  { 4, 8, 16, 32, 256, 2048, 16384, },
  { 1, 4, 8, 16, 32, 256, 2048, 16384, },
  { 2, 4, 8, 16, 32, 256, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 16384, },
  { 64, 256, 2048, 16384, },
  { 1, 64, 256, 2048, 16384, },
  { 2, 64, 256, 2048, 16384, },
  { 1, 2, 64, 256, 2048, 16384, },
  { 4, 64, 256, 2048, 16384, },
  { 1, 4, 64, 256, 2048, 16384, },
  { 2, 4, 64, 256, 2048, 16384, },
  { 1, 2, 4, 64, 256, 2048, 16384, },
  { 8, 64, 256, 2048, 16384, },
  { 1, 8, 64, 256, 2048, 16384, },
  { 2, 8, 64, 256, 2048, 16384, },
  { 1, 2, 8, 64, 256, 2048, 16384, },
  { 4, 8, 64, 256, 2048, 16384, },
  { 1, 4, 8, 64, 256, 2048, 16384, },
  { 2, 4, 8, 64, 256, 2048, 16384, },
  { 1, 2, 4, 8, 64, 256, 2048, 16384, },
  { 16, 64, 256, 2048, 16384, },
  { 1, 16, 64, 256, 2048, 16384, },
  { 2, 16, 64, 256, 2048, 16384, },
  { 1, 2, 16, 64, 256, 2048, 16384, },
  { 4, 16, 64, 256, 2048, 16384, },
  { 1, 4, 16, 64, 256, 2048, 16384, },
  { 2, 4, 16, 64, 256, 2048, 16384, },
  { 1, 2, 4, 16, 64, 256, 2048, 16384, },
  { 8, 16, 64, 256, 2048, 16384, },
  { 1, 8, 16, 64, 256, 2048, 16384, },
  { 2, 8, 16, 64, 256, 2048, 16384, },
  { 1, 2, 8, 16, 64, 256, 2048, 16384, },
  { 4, 8, 16, 64, 256, 2048, 16384, },
  { 1, 4, 8, 16, 64, 256, 2048, 16384, },
  { 2, 4, 8, 16, 64, 256, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 16384, },
  { 32, 64, 256, 2048, 16384, },
  { 1, 32, 64, 256, 2048, 16384, },
  { 2, 32, 64, 256, 2048, 16384, },
  { 1, 2, 32, 64, 256, 2048, 16384, },
  { 4, 32, 64, 256, 2048, 16384, },
  { 1, 4, 32, 64, 256, 2048, 16384, },
  { 2, 4, 32, 64, 256, 2048, 16384, },
  { 1, 2, 4, 32, 64, 256, 2048, 16384, },
  { 8, 32, 64, 256, 2048, 16384, },
  { 1, 8, 32, 64, 256, 2048, 16384, },
  { 2, 8, 32, 64, 256, 2048, 16384, },
  { 1, 2, 8, 32, 64, 256, 2048, 16384, },
  { 4, 8, 32, 64, 256, 2048, 16384, },
  { 1, 4, 8, 32, 64, 256, 2048, 16384, },
  { 2, 4, 8, 32, 64, 256, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 16384, },
  { 16, 32, 64, 256, 2048, 16384, },
  { 1, 16, 32, 64, 256, 2048, 16384, },
  { 2, 16, 32, 64, 256, 2048, 16384, },
  { 1, 2, 16, 32, 64, 256, 2048, 16384, },
  { 4, 16, 32, 64, 256, 2048, 16384, },
  { 1, 4, 16, 32, 64, 256, 2048, 16384, },
  { 2, 4, 16, 32, 64, 256, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 16384, },
  { 8, 16, 32, 64, 256, 2048, 16384, },
  { 1, 8, 16, 32, 64, 256, 2048, 16384, },
  { 2, 8, 16, 32, 64, 256, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 16384, },
  { 4, 8, 16, 32, 64, 256, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 16384, },
  { 128, 256, 2048, 16384, },
  { 1, 128, 256, 2048, 16384, },
  { 2, 128, 256, 2048, 16384, },
  { 1, 2, 128, 256, 2048, 16384, },
  { 4, 128, 256, 2048, 16384, },
  { 1, 4, 128, 256, 2048, 16384, },
  { 2, 4, 128, 256, 2048, 16384, },
  { 1, 2, 4, 128, 256, 2048, 16384, },
  { 8, 128, 256, 2048, 16384, },
  { 1, 8, 128, 256, 2048, 16384, },
  { 2, 8, 128, 256, 2048, 16384, },
  { 1, 2, 8, 128, 256, 2048, 16384, },
  { 4, 8, 128, 256, 2048, 16384, },
  { 1, 4, 8, 128, 256, 2048, 16384, },
  { 2, 4, 8, 128, 256, 2048, 16384, },
  { 1, 2, 4, 8, 128, 256, 2048, 16384, },
  { 16, 128, 256, 2048, 16384, },
  { 1, 16, 128, 256, 2048, 16384, },
  { 2, 16, 128, 256, 2048, 16384, },
  { 1, 2, 16, 128, 256, 2048, 16384, },
  { 4, 16, 128, 256, 2048, 16384, },
  { 1, 4, 16, 128, 256, 2048, 16384, },
  { 2, 4, 16, 128, 256, 2048, 16384, },
  { 1, 2, 4, 16, 128, 256, 2048, 16384, },
  { 8, 16, 128, 256, 2048, 16384, },
  { 1, 8, 16, 128, 256, 2048, 16384, },
  { 2, 8, 16, 128, 256, 2048, 16384, },
  { 1, 2, 8, 16, 128, 256, 2048, 16384, },
  { 4, 8, 16, 128, 256, 2048, 16384, },
  { 1, 4, 8, 16, 128, 256, 2048, 16384, },
  { 2, 4, 8, 16, 128, 256, 2048, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 16384, },
  { 32, 128, 256, 2048, 16384, },
  { 1, 32, 128, 256, 2048, 16384, },
  { 2, 32, 128, 256, 2048, 16384, },
  { 1, 2, 32, 128, 256, 2048, 16384, },
  { 4, 32, 128, 256, 2048, 16384, },
  { 1, 4, 32, 128, 256, 2048, 16384, },
  { 2, 4, 32, 128, 256, 2048, 16384, },
  { 1, 2, 4, 32, 128, 256, 2048, 16384, },
  { 8, 32, 128, 256, 2048, 16384, },
  { 1, 8, 32, 128, 256, 2048, 16384, },
  { 2, 8, 32, 128, 256, 2048, 16384, },
  { 1, 2, 8, 32, 128, 256, 2048, 16384, },
  { 4, 8, 32, 128, 256, 2048, 16384, },
  { 1, 4, 8, 32, 128, 256, 2048, 16384, },
  { 2, 4, 8, 32, 128, 256, 2048, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 16384, },
  { 16, 32, 128, 256, 2048, 16384, },
  { 1, 16, 32, 128, 256, 2048, 16384, },
  { 2, 16, 32, 128, 256, 2048, 16384, },
  { 1, 2, 16, 32, 128, 256, 2048, 16384, },
  { 4, 16, 32, 128, 256, 2048, 16384, },
  { 1, 4, 16, 32, 128, 256, 2048, 16384, },
  { 2, 4, 16, 32, 128, 256, 2048, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 16384, },
  { 8, 16, 32, 128, 256, 2048, 16384, },
  { 1, 8, 16, 32, 128, 256, 2048, 16384, },
  { 2, 8, 16, 32, 128, 256, 2048, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 16384, },
  { 4, 8, 16, 32, 128, 256, 2048, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 16384, },
  { 64, 128, 256, 2048, 16384, },
  { 1, 64, 128, 256, 2048, 16384, },
  { 2, 64, 128, 256, 2048, 16384, },
  { 1, 2, 64, 128, 256, 2048, 16384, },
  { 4, 64, 128, 256, 2048, 16384, },
  { 1, 4, 64, 128, 256, 2048, 16384, },
  { 2, 4, 64, 128, 256, 2048, 16384, },
  { 1, 2, 4, 64, 128, 256, 2048, 16384, },
  { 8, 64, 128, 256, 2048, 16384, },
  { 1, 8, 64, 128, 256, 2048, 16384, },
  { 2, 8, 64, 128, 256, 2048, 16384, },
  { 1, 2, 8, 64, 128, 256, 2048, 16384, },
  { 4, 8, 64, 128, 256, 2048, 16384, },
  { 1, 4, 8, 64, 128, 256, 2048, 16384, },
  { 2, 4, 8, 64, 128, 256, 2048, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 16384, },
  { 16, 64, 128, 256, 2048, 16384, },
  { 1, 16, 64, 128, 256, 2048, 16384, },
  { 2, 16, 64, 128, 256, 2048, 16384, },
  { 1, 2, 16, 64, 128, 256, 2048, 16384, },
  { 4, 16, 64, 128, 256, 2048, 16384, },
  { 1, 4, 16, 64, 128, 256, 2048, 16384, },
  { 2, 4, 16, 64, 128, 256, 2048, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 16384, },
  { 8, 16, 64, 128, 256, 2048, 16384, },
  { 1, 8, 16, 64, 128, 256, 2048, 16384, },
  { 2, 8, 16, 64, 128, 256, 2048, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 16384, },
  { 4, 8, 16, 64, 128, 256, 2048, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 16384, },
  { 32, 64, 128, 256, 2048, 16384, },
  { 1, 32, 64, 128, 256, 2048, 16384, },
  { 2, 32, 64, 128, 256, 2048, 16384, },
  { 1, 2, 32, 64, 128, 256, 2048, 16384, },
  { 4, 32, 64, 128, 256, 2048, 16384, },
  { 1, 4, 32, 64, 128, 256, 2048, 16384, },
  { 2, 4, 32, 64, 128, 256, 2048, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 16384, },
  { 8, 32, 64, 128, 256, 2048, 16384, },
  { 1, 8, 32, 64, 128, 256, 2048, 16384, },
  { 2, 8, 32, 64, 128, 256, 2048, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 16384, },
  { 4, 8, 32, 64, 128, 256, 2048, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 16384, },
  { 16, 32, 64, 128, 256, 2048, 16384, },
  { 1, 16, 32, 64, 128, 256, 2048, 16384, },
  { 2, 16, 32, 64, 128, 256, 2048, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 16384, },
  { 4, 16, 32, 64, 128, 256, 2048, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 16384, },
  { 8, 16, 32, 64, 128, 256, 2048, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 16384, },
  { 512, 2048, 16384, },
  { 1, 512, 2048, 16384, },
  { 2, 512, 2048, 16384, },
  { 1, 2, 512, 2048, 16384, },
  { 4, 512, 2048, 16384, },
  { 1, 4, 512, 2048, 16384, },
  { 2, 4, 512, 2048, 16384, },
  { 1, 2, 4, 512, 2048, 16384, },
  { 8, 512, 2048, 16384, },
  { 1, 8, 512, 2048, 16384, },
  { 2, 8, 512, 2048, 16384, },
  { 1, 2, 8, 512, 2048, 16384, },
  { 4, 8, 512, 2048, 16384, },
  { 1, 4, 8, 512, 2048, 16384, },
  { 2, 4, 8, 512, 2048, 16384, },
  { 1, 2, 4, 8, 512, 2048, 16384, },
  { 16, 512, 2048, 16384, },
  { 1, 16, 512, 2048, 16384, },
  { 2, 16, 512, 2048, 16384, },
  { 1, 2, 16, 512, 2048, 16384, },
  { 4, 16, 512, 2048, 16384, },
  { 1, 4, 16, 512, 2048, 16384, },
  { 2, 4, 16, 512, 2048, 16384, },
  { 1, 2, 4, 16, 512, 2048, 16384, },
  { 8, 16, 512, 2048, 16384, },
  { 1, 8, 16, 512, 2048, 16384, },
  { 2, 8, 16, 512, 2048, 16384, },
  { 1, 2, 8, 16, 512, 2048, 16384, },
  { 4, 8, 16, 512, 2048, 16384, },
  { 1, 4, 8, 16, 512, 2048, 16384, },
  { 2, 4, 8, 16, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 512, 2048, 16384, },
  { 32, 512, 2048, 16384, },
  { 1, 32, 512, 2048, 16384, },
  { 2, 32, 512, 2048, 16384, },
  { 1, 2, 32, 512, 2048, 16384, },
  { 4, 32, 512, 2048, 16384, },
  { 1, 4, 32, 512, 2048, 16384, },
  { 2, 4, 32, 512, 2048, 16384, },
  { 1, 2, 4, 32, 512, 2048, 16384, },
  { 8, 32, 512, 2048, 16384, },
  { 1, 8, 32, 512, 2048, 16384, },
  { 2, 8, 32, 512, 2048, 16384, },
  { 1, 2, 8, 32, 512, 2048, 16384, },
  { 4, 8, 32, 512, 2048, 16384, },
  { 1, 4, 8, 32, 512, 2048, 16384, },
  { 2, 4, 8, 32, 512, 2048, 16384, },
  { 1, 2, 4, 8, 32, 512, 2048, 16384, },
  { 16, 32, 512, 2048, 16384, },
  { 1, 16, 32, 512, 2048, 16384, },
  { 2, 16, 32, 512, 2048, 16384, },
  { 1, 2, 16, 32, 512, 2048, 16384, },
  { 4, 16, 32, 512, 2048, 16384, },
  { 1, 4, 16, 32, 512, 2048, 16384, },
  { 2, 4, 16, 32, 512, 2048, 16384, },
  { 1, 2, 4, 16, 32, 512, 2048, 16384, },
  { 8, 16, 32, 512, 2048, 16384, },
  { 1, 8, 16, 32, 512, 2048, 16384, },
  { 2, 8, 16, 32, 512, 2048, 16384, },
  { 1, 2, 8, 16, 32, 512, 2048, 16384, },
  { 4, 8, 16, 32, 512, 2048, 16384, },
  { 1, 4, 8, 16, 32, 512, 2048, 16384, },
  { 2, 4, 8, 16, 32, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 16384, },
  { 64, 512, 2048, 16384, },
  { 1, 64, 512, 2048, 16384, },
  { 2, 64, 512, 2048, 16384, },
  { 1, 2, 64, 512, 2048, 16384, },
  { 4, 64, 512, 2048, 16384, },
  { 1, 4, 64, 512, 2048, 16384, },
  { 2, 4, 64, 512, 2048, 16384, },
  { 1, 2, 4, 64, 512, 2048, 16384, },
  { 8, 64, 512, 2048, 16384, },
  { 1, 8, 64, 512, 2048, 16384, },
  { 2, 8, 64, 512, 2048, 16384, },
  { 1, 2, 8, 64, 512, 2048, 16384, },
  { 4, 8, 64, 512, 2048, 16384, },
  { 1, 4, 8, 64, 512, 2048, 16384, },
  { 2, 4, 8, 64, 512, 2048, 16384, },
  { 1, 2, 4, 8, 64, 512, 2048, 16384, },
  { 16, 64, 512, 2048, 16384, },
  { 1, 16, 64, 512, 2048, 16384, },
  { 2, 16, 64, 512, 2048, 16384, },
  { 1, 2, 16, 64, 512, 2048, 16384, },
  { 4, 16, 64, 512, 2048, 16384, },
  { 1, 4, 16, 64, 512, 2048, 16384, },
  { 2, 4, 16, 64, 512, 2048, 16384, },
  { 1, 2, 4, 16, 64, 512, 2048, 16384, },
  { 8, 16, 64, 512, 2048, 16384, },
  { 1, 8, 16, 64, 512, 2048, 16384, },
  { 2, 8, 16, 64, 512, 2048, 16384, },
  { 1, 2, 8, 16, 64, 512, 2048, 16384, },
  { 4, 8, 16, 64, 512, 2048, 16384, },
  { 1, 4, 8, 16, 64, 512, 2048, 16384, },
  { 2, 4, 8, 16, 64, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 16384, },
  { 32, 64, 512, 2048, 16384, },
  { 1, 32, 64, 512, 2048, 16384, },
  { 2, 32, 64, 512, 2048, 16384, },
  { 1, 2, 32, 64, 512, 2048, 16384, },
  { 4, 32, 64, 512, 2048, 16384, },
  { 1, 4, 32, 64, 512, 2048, 16384, },
  { 2, 4, 32, 64, 512, 2048, 16384, },
  { 1, 2, 4, 32, 64, 512, 2048, 16384, },
  { 8, 32, 64, 512, 2048, 16384, },
  { 1, 8, 32, 64, 512, 2048, 16384, },
  { 2, 8, 32, 64, 512, 2048, 16384, },
  { 1, 2, 8, 32, 64, 512, 2048, 16384, },
  { 4, 8, 32, 64, 512, 2048, 16384, },
  { 1, 4, 8, 32, 64, 512, 2048, 16384, },
  { 2, 4, 8, 32, 64, 512, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 16384, },
  { 16, 32, 64, 512, 2048, 16384, },
  { 1, 16, 32, 64, 512, 2048, 16384, },
  { 2, 16, 32, 64, 512, 2048, 16384, },
  { 1, 2, 16, 32, 64, 512, 2048, 16384, },
  { 4, 16, 32, 64, 512, 2048, 16384, },
  { 1, 4, 16, 32, 64, 512, 2048, 16384, },
  { 2, 4, 16, 32, 64, 512, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 16384, },
  { 8, 16, 32, 64, 512, 2048, 16384, },
  { 1, 8, 16, 32, 64, 512, 2048, 16384, },
  { 2, 8, 16, 32, 64, 512, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 16384, },
  { 4, 8, 16, 32, 64, 512, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 16384, },
  { 128, 512, 2048, 16384, },
  { 1, 128, 512, 2048, 16384, },
  { 2, 128, 512, 2048, 16384, },
  { 1, 2, 128, 512, 2048, 16384, },
  { 4, 128, 512, 2048, 16384, },
  { 1, 4, 128, 512, 2048, 16384, },
  { 2, 4, 128, 512, 2048, 16384, },
  { 1, 2, 4, 128, 512, 2048, 16384, },
  { 8, 128, 512, 2048, 16384, },
  { 1, 8, 128, 512, 2048, 16384, },
  { 2, 8, 128, 512, 2048, 16384, },
  { 1, 2, 8, 128, 512, 2048, 16384, },
  { 4, 8, 128, 512, 2048, 16384, },
  { 1, 4, 8, 128, 512, 2048, 16384, },
  { 2, 4, 8, 128, 512, 2048, 16384, },
  { 1, 2, 4, 8, 128, 512, 2048, 16384, },
  { 16, 128, 512, 2048, 16384, },
  { 1, 16, 128, 512, 2048, 16384, },
  { 2, 16, 128, 512, 2048, 16384, },
  { 1, 2, 16, 128, 512, 2048, 16384, },
  { 4, 16, 128, 512, 2048, 16384, },
  { 1, 4, 16, 128, 512, 2048, 16384, },
  { 2, 4, 16, 128, 512, 2048, 16384, },
  { 1, 2, 4, 16, 128, 512, 2048, 16384, },
  { 8, 16, 128, 512, 2048, 16384, },
  { 1, 8, 16, 128, 512, 2048, 16384, },
  { 2, 8, 16, 128, 512, 2048, 16384, },
  { 1, 2, 8, 16, 128, 512, 2048, 16384, },
  { 4, 8, 16, 128, 512, 2048, 16384, },
  { 1, 4, 8, 16, 128, 512, 2048, 16384, },
  { 2, 4, 8, 16, 128, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 16384, },
  { 32, 128, 512, 2048, 16384, },
  { 1, 32, 128, 512, 2048, 16384, },
  { 2, 32, 128, 512, 2048, 16384, },
  { 1, 2, 32, 128, 512, 2048, 16384, },
  { 4, 32, 128, 512, 2048, 16384, },
  { 1, 4, 32, 128, 512, 2048, 16384, },
  { 2, 4, 32, 128, 512, 2048, 16384, },
  { 1, 2, 4, 32, 128, 512, 2048, 16384, },
  { 8, 32, 128, 512, 2048, 16384, },
  { 1, 8, 32, 128, 512, 2048, 16384, },
  { 2, 8, 32, 128, 512, 2048, 16384, },
  { 1, 2, 8, 32, 128, 512, 2048, 16384, },
  { 4, 8, 32, 128, 512, 2048, 16384, },
  { 1, 4, 8, 32, 128, 512, 2048, 16384, },
  { 2, 4, 8, 32, 128, 512, 2048, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 16384, },
  { 16, 32, 128, 512, 2048, 16384, },
  { 1, 16, 32, 128, 512, 2048, 16384, },
  { 2, 16, 32, 128, 512, 2048, 16384, },
  { 1, 2, 16, 32, 128, 512, 2048, 16384, },
  { 4, 16, 32, 128, 512, 2048, 16384, },
  { 1, 4, 16, 32, 128, 512, 2048, 16384, },
  { 2, 4, 16, 32, 128, 512, 2048, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 16384, },
  { 8, 16, 32, 128, 512, 2048, 16384, },
  { 1, 8, 16, 32, 128, 512, 2048, 16384, },
  { 2, 8, 16, 32, 128, 512, 2048, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 16384, },
  { 4, 8, 16, 32, 128, 512, 2048, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 16384, },
  { 64, 128, 512, 2048, 16384, },
  { 1, 64, 128, 512, 2048, 16384, },
  { 2, 64, 128, 512, 2048, 16384, },
  { 1, 2, 64, 128, 512, 2048, 16384, },
  { 4, 64, 128, 512, 2048, 16384, },
  { 1, 4, 64, 128, 512, 2048, 16384, },
  { 2, 4, 64, 128, 512, 2048, 16384, },
  { 1, 2, 4, 64, 128, 512, 2048, 16384, },
  { 8, 64, 128, 512, 2048, 16384, },
  { 1, 8, 64, 128, 512, 2048, 16384, },
  { 2, 8, 64, 128, 512, 2048, 16384, },
  { 1, 2, 8, 64, 128, 512, 2048, 16384, },
  { 4, 8, 64, 128, 512, 2048, 16384, },
  { 1, 4, 8, 64, 128, 512, 2048, 16384, },
  { 2, 4, 8, 64, 128, 512, 2048, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 16384, },
  { 16, 64, 128, 512, 2048, 16384, },
  { 1, 16, 64, 128, 512, 2048, 16384, },
  { 2, 16, 64, 128, 512, 2048, 16384, },
  { 1, 2, 16, 64, 128, 512, 2048, 16384, },
  { 4, 16, 64, 128, 512, 2048, 16384, },
  { 1, 4, 16, 64, 128, 512, 2048, 16384, },
  { 2, 4, 16, 64, 128, 512, 2048, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 16384, },
  { 8, 16, 64, 128, 512, 2048, 16384, },
  { 1, 8, 16, 64, 128, 512, 2048, 16384, },
  { 2, 8, 16, 64, 128, 512, 2048, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 16384, },
  { 4, 8, 16, 64, 128, 512, 2048, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 16384, },
  { 32, 64, 128, 512, 2048, 16384, },
  { 1, 32, 64, 128, 512, 2048, 16384, },
  { 2, 32, 64, 128, 512, 2048, 16384, },
  { 1, 2, 32, 64, 128, 512, 2048, 16384, },
  { 4, 32, 64, 128, 512, 2048, 16384, },
  { 1, 4, 32, 64, 128, 512, 2048, 16384, },
  { 2, 4, 32, 64, 128, 512, 2048, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 16384, },
  { 8, 32, 64, 128, 512, 2048, 16384, },
  { 1, 8, 32, 64, 128, 512, 2048, 16384, },
  { 2, 8, 32, 64, 128, 512, 2048, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 16384, },
  { 4, 8, 32, 64, 128, 512, 2048, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 16384, },
  { 16, 32, 64, 128, 512, 2048, 16384, },
  { 1, 16, 32, 64, 128, 512, 2048, 16384, },
  { 2, 16, 32, 64, 128, 512, 2048, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 16384, },
  { 4, 16, 32, 64, 128, 512, 2048, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 16384, },
  { 8, 16, 32, 64, 128, 512, 2048, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 16384, },
  { 256, 512, 2048, 16384, },
  { 1, 256, 512, 2048, 16384, },
  { 2, 256, 512, 2048, 16384, },
  { 1, 2, 256, 512, 2048, 16384, },
  { 4, 256, 512, 2048, 16384, },
  { 1, 4, 256, 512, 2048, 16384, },
  { 2, 4, 256, 512, 2048, 16384, },
  { 1, 2, 4, 256, 512, 2048, 16384, },
  { 8, 256, 512, 2048, 16384, },
  { 1, 8, 256, 512, 2048, 16384, },
  { 2, 8, 256, 512, 2048, 16384, },
  { 1, 2, 8, 256, 512, 2048, 16384, },
  { 4, 8, 256, 512, 2048, 16384, },
  { 1, 4, 8, 256, 512, 2048, 16384, },
  { 2, 4, 8, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 256, 512, 2048, 16384, },
  { 16, 256, 512, 2048, 16384, },
  { 1, 16, 256, 512, 2048, 16384, },
  { 2, 16, 256, 512, 2048, 16384, },
  { 1, 2, 16, 256, 512, 2048, 16384, },
  { 4, 16, 256, 512, 2048, 16384, },
  { 1, 4, 16, 256, 512, 2048, 16384, },
  { 2, 4, 16, 256, 512, 2048, 16384, },
  { 1, 2, 4, 16, 256, 512, 2048, 16384, },
  { 8, 16, 256, 512, 2048, 16384, },
  { 1, 8, 16, 256, 512, 2048, 16384, },
  { 2, 8, 16, 256, 512, 2048, 16384, },
  { 1, 2, 8, 16, 256, 512, 2048, 16384, },
  { 4, 8, 16, 256, 512, 2048, 16384, },
  { 1, 4, 8, 16, 256, 512, 2048, 16384, },
  { 2, 4, 8, 16, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 16384, },
  { 32, 256, 512, 2048, 16384, },
  { 1, 32, 256, 512, 2048, 16384, },
  { 2, 32, 256, 512, 2048, 16384, },
  { 1, 2, 32, 256, 512, 2048, 16384, },
  { 4, 32, 256, 512, 2048, 16384, },
  { 1, 4, 32, 256, 512, 2048, 16384, },
  { 2, 4, 32, 256, 512, 2048, 16384, },
  { 1, 2, 4, 32, 256, 512, 2048, 16384, },
  { 8, 32, 256, 512, 2048, 16384, },
  { 1, 8, 32, 256, 512, 2048, 16384, },
  { 2, 8, 32, 256, 512, 2048, 16384, },
  { 1, 2, 8, 32, 256, 512, 2048, 16384, },
  { 4, 8, 32, 256, 512, 2048, 16384, },
  { 1, 4, 8, 32, 256, 512, 2048, 16384, },
  { 2, 4, 8, 32, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 16384, },
  { 16, 32, 256, 512, 2048, 16384, },
  { 1, 16, 32, 256, 512, 2048, 16384, },
  { 2, 16, 32, 256, 512, 2048, 16384, },
  { 1, 2, 16, 32, 256, 512, 2048, 16384, },
  { 4, 16, 32, 256, 512, 2048, 16384, },
  { 1, 4, 16, 32, 256, 512, 2048, 16384, },
  { 2, 4, 16, 32, 256, 512, 2048, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 16384, },
  { 8, 16, 32, 256, 512, 2048, 16384, },
  { 1, 8, 16, 32, 256, 512, 2048, 16384, },
  { 2, 8, 16, 32, 256, 512, 2048, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 16384, },
  { 4, 8, 16, 32, 256, 512, 2048, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 16384, },
  { 64, 256, 512, 2048, 16384, },
  { 1, 64, 256, 512, 2048, 16384, },
  { 2, 64, 256, 512, 2048, 16384, },
  { 1, 2, 64, 256, 512, 2048, 16384, },
  { 4, 64, 256, 512, 2048, 16384, },
  { 1, 4, 64, 256, 512, 2048, 16384, },
  { 2, 4, 64, 256, 512, 2048, 16384, },
  { 1, 2, 4, 64, 256, 512, 2048, 16384, },
  { 8, 64, 256, 512, 2048, 16384, },
  { 1, 8, 64, 256, 512, 2048, 16384, },
  { 2, 8, 64, 256, 512, 2048, 16384, },
  { 1, 2, 8, 64, 256, 512, 2048, 16384, },
  { 4, 8, 64, 256, 512, 2048, 16384, },
  { 1, 4, 8, 64, 256, 512, 2048, 16384, },
  { 2, 4, 8, 64, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 16384, },
  { 16, 64, 256, 512, 2048, 16384, },
  { 1, 16, 64, 256, 512, 2048, 16384, },
  { 2, 16, 64, 256, 512, 2048, 16384, },
  { 1, 2, 16, 64, 256, 512, 2048, 16384, },
  { 4, 16, 64, 256, 512, 2048, 16384, },
  { 1, 4, 16, 64, 256, 512, 2048, 16384, },
  { 2, 4, 16, 64, 256, 512, 2048, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 16384, },
  { 8, 16, 64, 256, 512, 2048, 16384, },
  { 1, 8, 16, 64, 256, 512, 2048, 16384, },
  { 2, 8, 16, 64, 256, 512, 2048, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 16384, },
  { 4, 8, 16, 64, 256, 512, 2048, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 16384, },
  { 32, 64, 256, 512, 2048, 16384, },
  { 1, 32, 64, 256, 512, 2048, 16384, },
  { 2, 32, 64, 256, 512, 2048, 16384, },
  { 1, 2, 32, 64, 256, 512, 2048, 16384, },
  { 4, 32, 64, 256, 512, 2048, 16384, },
  { 1, 4, 32, 64, 256, 512, 2048, 16384, },
  { 2, 4, 32, 64, 256, 512, 2048, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 16384, },
  { 8, 32, 64, 256, 512, 2048, 16384, },
  { 1, 8, 32, 64, 256, 512, 2048, 16384, },
  { 2, 8, 32, 64, 256, 512, 2048, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 16384, },
  { 4, 8, 32, 64, 256, 512, 2048, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 16384, },
  { 16, 32, 64, 256, 512, 2048, 16384, },
  { 1, 16, 32, 64, 256, 512, 2048, 16384, },
  { 2, 16, 32, 64, 256, 512, 2048, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 16384, },
  { 4, 16, 32, 64, 256, 512, 2048, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 16384, },
  { 8, 16, 32, 64, 256, 512, 2048, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 16384, },
  { 128, 256, 512, 2048, 16384, },
  { 1, 128, 256, 512, 2048, 16384, },
  { 2, 128, 256, 512, 2048, 16384, },
  { 1, 2, 128, 256, 512, 2048, 16384, },
  { 4, 128, 256, 512, 2048, 16384, },
  { 1, 4, 128, 256, 512, 2048, 16384, },
  { 2, 4, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 128, 256, 512, 2048, 16384, },
  { 8, 128, 256, 512, 2048, 16384, },
  { 1, 8, 128, 256, 512, 2048, 16384, },
  { 2, 8, 128, 256, 512, 2048, 16384, },
  { 1, 2, 8, 128, 256, 512, 2048, 16384, },
  { 4, 8, 128, 256, 512, 2048, 16384, },
  { 1, 4, 8, 128, 256, 512, 2048, 16384, },
  { 2, 4, 8, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 16384, },
  { 16, 128, 256, 512, 2048, 16384, },
  { 1, 16, 128, 256, 512, 2048, 16384, },
  { 2, 16, 128, 256, 512, 2048, 16384, },
  { 1, 2, 16, 128, 256, 512, 2048, 16384, },
  { 4, 16, 128, 256, 512, 2048, 16384, },
  { 1, 4, 16, 128, 256, 512, 2048, 16384, },
  { 2, 4, 16, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 16384, },
  { 8, 16, 128, 256, 512, 2048, 16384, },
  { 1, 8, 16, 128, 256, 512, 2048, 16384, },
  { 2, 8, 16, 128, 256, 512, 2048, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 16384, },
  { 4, 8, 16, 128, 256, 512, 2048, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 16384, },
  { 32, 128, 256, 512, 2048, 16384, },
  { 1, 32, 128, 256, 512, 2048, 16384, },
  { 2, 32, 128, 256, 512, 2048, 16384, },
  { 1, 2, 32, 128, 256, 512, 2048, 16384, },
  { 4, 32, 128, 256, 512, 2048, 16384, },
  { 1, 4, 32, 128, 256, 512, 2048, 16384, },
  { 2, 4, 32, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 16384, },
  { 8, 32, 128, 256, 512, 2048, 16384, },
  { 1, 8, 32, 128, 256, 512, 2048, 16384, },
  { 2, 8, 32, 128, 256, 512, 2048, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 16384, },
  { 4, 8, 32, 128, 256, 512, 2048, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 16384, },
  { 16, 32, 128, 256, 512, 2048, 16384, },
  { 1, 16, 32, 128, 256, 512, 2048, 16384, },
  { 2, 16, 32, 128, 256, 512, 2048, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 16384, },
  { 4, 16, 32, 128, 256, 512, 2048, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 16384, },
  { 8, 16, 32, 128, 256, 512, 2048, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 16384, },
  { 64, 128, 256, 512, 2048, 16384, },
  { 1, 64, 128, 256, 512, 2048, 16384, },
  { 2, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 64, 128, 256, 512, 2048, 16384, },
  { 4, 64, 128, 256, 512, 2048, 16384, },
  { 1, 4, 64, 128, 256, 512, 2048, 16384, },
  { 2, 4, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 16384, },
  { 8, 64, 128, 256, 512, 2048, 16384, },
  { 1, 8, 64, 128, 256, 512, 2048, 16384, },
  { 2, 8, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 16384, },
  { 4, 8, 64, 128, 256, 512, 2048, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 16384, },
  { 16, 64, 128, 256, 512, 2048, 16384, },
  { 1, 16, 64, 128, 256, 512, 2048, 16384, },
  { 2, 16, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 16384, },
  { 4, 16, 64, 128, 256, 512, 2048, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 16384, },
  { 8, 16, 64, 128, 256, 512, 2048, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 16384, },
  { 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 32, 64, 128, 256, 512, 2048, 16384, },
  { 2, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 16384, },
  { 4, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 16384, },
  { 8, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 16384, },
  { 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, },
  { 1024, 2048, 16384, },
  { 1, 1024, 2048, 16384, },
  { 2, 1024, 2048, 16384, },
  { 1, 2, 1024, 2048, 16384, },
  { 4, 1024, 2048, 16384, },
  { 1, 4, 1024, 2048, 16384, },
  { 2, 4, 1024, 2048, 16384, },
  { 1, 2, 4, 1024, 2048, 16384, },
  { 8, 1024, 2048, 16384, },
  { 1, 8, 1024, 2048, 16384, },
  { 2, 8, 1024, 2048, 16384, },
  { 1, 2, 8, 1024, 2048, 16384, },
  { 4, 8, 1024, 2048, 16384, },
  { 1, 4, 8, 1024, 2048, 16384, },
  { 2, 4, 8, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 1024, 2048, 16384, },
  { 16, 1024, 2048, 16384, },
  { 1, 16, 1024, 2048, 16384, },
  { 2, 16, 1024, 2048, 16384, },
  { 1, 2, 16, 1024, 2048, 16384, },
  { 4, 16, 1024, 2048, 16384, },
  { 1, 4, 16, 1024, 2048, 16384, },
  { 2, 4, 16, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 1024, 2048, 16384, },
  { 8, 16, 1024, 2048, 16384, },
  { 1, 8, 16, 1024, 2048, 16384, },
  { 2, 8, 16, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 1024, 2048, 16384, },
  { 4, 8, 16, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 1024, 2048, 16384, },
  { 32, 1024, 2048, 16384, },
  { 1, 32, 1024, 2048, 16384, },
  { 2, 32, 1024, 2048, 16384, },
  { 1, 2, 32, 1024, 2048, 16384, },
  { 4, 32, 1024, 2048, 16384, },
  { 1, 4, 32, 1024, 2048, 16384, },
  { 2, 4, 32, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 1024, 2048, 16384, },
  { 8, 32, 1024, 2048, 16384, },
  { 1, 8, 32, 1024, 2048, 16384, },
  { 2, 8, 32, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 1024, 2048, 16384, },
  { 4, 8, 32, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 1024, 2048, 16384, },
  { 16, 32, 1024, 2048, 16384, },
  { 1, 16, 32, 1024, 2048, 16384, },
  { 2, 16, 32, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 1024, 2048, 16384, },
  { 4, 16, 32, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 1024, 2048, 16384, },
  { 8, 16, 32, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 16384, },
  { 64, 1024, 2048, 16384, },
  { 1, 64, 1024, 2048, 16384, },
  { 2, 64, 1024, 2048, 16384, },
  { 1, 2, 64, 1024, 2048, 16384, },
  { 4, 64, 1024, 2048, 16384, },
  { 1, 4, 64, 1024, 2048, 16384, },
  { 2, 4, 64, 1024, 2048, 16384, },
  { 1, 2, 4, 64, 1024, 2048, 16384, },
  { 8, 64, 1024, 2048, 16384, },
  { 1, 8, 64, 1024, 2048, 16384, },
  { 2, 8, 64, 1024, 2048, 16384, },
  { 1, 2, 8, 64, 1024, 2048, 16384, },
  { 4, 8, 64, 1024, 2048, 16384, },
  { 1, 4, 8, 64, 1024, 2048, 16384, },
  { 2, 4, 8, 64, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 64, 1024, 2048, 16384, },
  { 16, 64, 1024, 2048, 16384, },
  { 1, 16, 64, 1024, 2048, 16384, },
  { 2, 16, 64, 1024, 2048, 16384, },
  { 1, 2, 16, 64, 1024, 2048, 16384, },
  { 4, 16, 64, 1024, 2048, 16384, },
  { 1, 4, 16, 64, 1024, 2048, 16384, },
  { 2, 4, 16, 64, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 64, 1024, 2048, 16384, },
  { 8, 16, 64, 1024, 2048, 16384, },
  { 1, 8, 16, 64, 1024, 2048, 16384, },
  { 2, 8, 16, 64, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 64, 1024, 2048, 16384, },
  { 4, 8, 16, 64, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 64, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 64, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 16384, },
  { 32, 64, 1024, 2048, 16384, },
  { 1, 32, 64, 1024, 2048, 16384, },
  { 2, 32, 64, 1024, 2048, 16384, },
  { 1, 2, 32, 64, 1024, 2048, 16384, },
  { 4, 32, 64, 1024, 2048, 16384, },
  { 1, 4, 32, 64, 1024, 2048, 16384, },
  { 2, 4, 32, 64, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 64, 1024, 2048, 16384, },
  { 8, 32, 64, 1024, 2048, 16384, },
  { 1, 8, 32, 64, 1024, 2048, 16384, },
  { 2, 8, 32, 64, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 64, 1024, 2048, 16384, },
  { 4, 8, 32, 64, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 64, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 64, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 16384, },
  { 16, 32, 64, 1024, 2048, 16384, },
  { 1, 16, 32, 64, 1024, 2048, 16384, },
  { 2, 16, 32, 64, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 64, 1024, 2048, 16384, },
  { 4, 16, 32, 64, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 64, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 64, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 16384, },
  { 8, 16, 32, 64, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 64, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 64, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 64, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 16384, },
  { 128, 1024, 2048, 16384, },
  { 1, 128, 1024, 2048, 16384, },
  { 2, 128, 1024, 2048, 16384, },
  { 1, 2, 128, 1024, 2048, 16384, },
  { 4, 128, 1024, 2048, 16384, },
  { 1, 4, 128, 1024, 2048, 16384, },
  { 2, 4, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 128, 1024, 2048, 16384, },
  { 8, 128, 1024, 2048, 16384, },
  { 1, 8, 128, 1024, 2048, 16384, },
  { 2, 8, 128, 1024, 2048, 16384, },
  { 1, 2, 8, 128, 1024, 2048, 16384, },
  { 4, 8, 128, 1024, 2048, 16384, },
  { 1, 4, 8, 128, 1024, 2048, 16384, },
  { 2, 4, 8, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 128, 1024, 2048, 16384, },
  { 16, 128, 1024, 2048, 16384, },
  { 1, 16, 128, 1024, 2048, 16384, },
  { 2, 16, 128, 1024, 2048, 16384, },
  { 1, 2, 16, 128, 1024, 2048, 16384, },
  { 4, 16, 128, 1024, 2048, 16384, },
  { 1, 4, 16, 128, 1024, 2048, 16384, },
  { 2, 4, 16, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 128, 1024, 2048, 16384, },
  { 8, 16, 128, 1024, 2048, 16384, },
  { 1, 8, 16, 128, 1024, 2048, 16384, },
  { 2, 8, 16, 128, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 128, 1024, 2048, 16384, },
  { 4, 8, 16, 128, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 128, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 16384, },
  { 32, 128, 1024, 2048, 16384, },
  { 1, 32, 128, 1024, 2048, 16384, },
  { 2, 32, 128, 1024, 2048, 16384, },
  { 1, 2, 32, 128, 1024, 2048, 16384, },
  { 4, 32, 128, 1024, 2048, 16384, },
  { 1, 4, 32, 128, 1024, 2048, 16384, },
  { 2, 4, 32, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 128, 1024, 2048, 16384, },
  { 8, 32, 128, 1024, 2048, 16384, },
  { 1, 8, 32, 128, 1024, 2048, 16384, },
  { 2, 8, 32, 128, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 128, 1024, 2048, 16384, },
  { 4, 8, 32, 128, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 128, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 16384, },
  { 16, 32, 128, 1024, 2048, 16384, },
  { 1, 16, 32, 128, 1024, 2048, 16384, },
  { 2, 16, 32, 128, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 128, 1024, 2048, 16384, },
  { 4, 16, 32, 128, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 128, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 16384, },
  { 8, 16, 32, 128, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 128, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 128, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 128, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 16384, },
  { 64, 128, 1024, 2048, 16384, },
  { 1, 64, 128, 1024, 2048, 16384, },
  { 2, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 64, 128, 1024, 2048, 16384, },
  { 4, 64, 128, 1024, 2048, 16384, },
  { 1, 4, 64, 128, 1024, 2048, 16384, },
  { 2, 4, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 64, 128, 1024, 2048, 16384, },
  { 8, 64, 128, 1024, 2048, 16384, },
  { 1, 8, 64, 128, 1024, 2048, 16384, },
  { 2, 8, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 8, 64, 128, 1024, 2048, 16384, },
  { 4, 8, 64, 128, 1024, 2048, 16384, },
  { 1, 4, 8, 64, 128, 1024, 2048, 16384, },
  { 2, 4, 8, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 16384, },
  { 16, 64, 128, 1024, 2048, 16384, },
  { 1, 16, 64, 128, 1024, 2048, 16384, },
  { 2, 16, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 16, 64, 128, 1024, 2048, 16384, },
  { 4, 16, 64, 128, 1024, 2048, 16384, },
  { 1, 4, 16, 64, 128, 1024, 2048, 16384, },
  { 2, 4, 16, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 16384, },
  { 8, 16, 64, 128, 1024, 2048, 16384, },
  { 1, 8, 16, 64, 128, 1024, 2048, 16384, },
  { 2, 8, 16, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 16384, },
  { 4, 8, 16, 64, 128, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 16384, },
  { 32, 64, 128, 1024, 2048, 16384, },
  { 1, 32, 64, 128, 1024, 2048, 16384, },
  { 2, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 32, 64, 128, 1024, 2048, 16384, },
  { 4, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 4, 32, 64, 128, 1024, 2048, 16384, },
  { 2, 4, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 16384, },
  { 8, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 8, 32, 64, 128, 1024, 2048, 16384, },
  { 2, 8, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 16384, },
  { 4, 8, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 16384, },
  { 16, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 2, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 4, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 8, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 16384, },
  { 256, 1024, 2048, 16384, },
  { 1, 256, 1024, 2048, 16384, },
  { 2, 256, 1024, 2048, 16384, },
  { 1, 2, 256, 1024, 2048, 16384, },
  { 4, 256, 1024, 2048, 16384, },
  { 1, 4, 256, 1024, 2048, 16384, },
  { 2, 4, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 256, 1024, 2048, 16384, },
  { 8, 256, 1024, 2048, 16384, },
  { 1, 8, 256, 1024, 2048, 16384, },
  { 2, 8, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 256, 1024, 2048, 16384, },
  { 4, 8, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 256, 1024, 2048, 16384, },
  { 16, 256, 1024, 2048, 16384, },
  { 1, 16, 256, 1024, 2048, 16384, },
  { 2, 16, 256, 1024, 2048, 16384, },
  { 1, 2, 16, 256, 1024, 2048, 16384, },
  { 4, 16, 256, 1024, 2048, 16384, },
  { 1, 4, 16, 256, 1024, 2048, 16384, },
  { 2, 4, 16, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 256, 1024, 2048, 16384, },
  { 8, 16, 256, 1024, 2048, 16384, },
  { 1, 8, 16, 256, 1024, 2048, 16384, },
  { 2, 8, 16, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 256, 1024, 2048, 16384, },
  { 4, 8, 16, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 16384, },
  { 32, 256, 1024, 2048, 16384, },
  { 1, 32, 256, 1024, 2048, 16384, },
  { 2, 32, 256, 1024, 2048, 16384, },
  { 1, 2, 32, 256, 1024, 2048, 16384, },
  { 4, 32, 256, 1024, 2048, 16384, },
  { 1, 4, 32, 256, 1024, 2048, 16384, },
  { 2, 4, 32, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 256, 1024, 2048, 16384, },
  { 8, 32, 256, 1024, 2048, 16384, },
  { 1, 8, 32, 256, 1024, 2048, 16384, },
  { 2, 8, 32, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 256, 1024, 2048, 16384, },
  { 4, 8, 32, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 16384, },
  { 16, 32, 256, 1024, 2048, 16384, },
  { 1, 16, 32, 256, 1024, 2048, 16384, },
  { 2, 16, 32, 256, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 256, 1024, 2048, 16384, },
  { 4, 16, 32, 256, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 256, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 16384, },
  { 8, 16, 32, 256, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 256, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 16384, },
  { 64, 256, 1024, 2048, 16384, },
  { 1, 64, 256, 1024, 2048, 16384, },
  { 2, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 64, 256, 1024, 2048, 16384, },
  { 4, 64, 256, 1024, 2048, 16384, },
  { 1, 4, 64, 256, 1024, 2048, 16384, },
  { 2, 4, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 64, 256, 1024, 2048, 16384, },
  { 8, 64, 256, 1024, 2048, 16384, },
  { 1, 8, 64, 256, 1024, 2048, 16384, },
  { 2, 8, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 64, 256, 1024, 2048, 16384, },
  { 4, 8, 64, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 64, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 16384, },
  { 16, 64, 256, 1024, 2048, 16384, },
  { 1, 16, 64, 256, 1024, 2048, 16384, },
  { 2, 16, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 16, 64, 256, 1024, 2048, 16384, },
  { 4, 16, 64, 256, 1024, 2048, 16384, },
  { 1, 4, 16, 64, 256, 1024, 2048, 16384, },
  { 2, 4, 16, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 16384, },
  { 8, 16, 64, 256, 1024, 2048, 16384, },
  { 1, 8, 16, 64, 256, 1024, 2048, 16384, },
  { 2, 8, 16, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 16384, },
  { 4, 8, 16, 64, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 16384, },
  { 32, 64, 256, 1024, 2048, 16384, },
  { 1, 32, 64, 256, 1024, 2048, 16384, },
  { 2, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 32, 64, 256, 1024, 2048, 16384, },
  { 4, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 4, 32, 64, 256, 1024, 2048, 16384, },
  { 2, 4, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 16384, },
  { 8, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 8, 32, 64, 256, 1024, 2048, 16384, },
  { 2, 8, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 16384, },
  { 4, 8, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 16384, },
  { 16, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 2, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 4, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 8, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 16384, },
  { 128, 256, 1024, 2048, 16384, },
  { 1, 128, 256, 1024, 2048, 16384, },
  { 2, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 128, 256, 1024, 2048, 16384, },
  { 4, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 128, 256, 1024, 2048, 16384, },
  { 8, 128, 256, 1024, 2048, 16384, },
  { 1, 8, 128, 256, 1024, 2048, 16384, },
  { 2, 8, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 128, 256, 1024, 2048, 16384, },
  { 4, 8, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 16384, },
  { 16, 128, 256, 1024, 2048, 16384, },
  { 1, 16, 128, 256, 1024, 2048, 16384, },
  { 2, 16, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 16, 128, 256, 1024, 2048, 16384, },
  { 4, 16, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 16, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 16, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 16384, },
  { 8, 16, 128, 256, 1024, 2048, 16384, },
  { 1, 8, 16, 128, 256, 1024, 2048, 16384, },
  { 2, 8, 16, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 16384, },
  { 4, 8, 16, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 16384, },
  { 32, 128, 256, 1024, 2048, 16384, },
  { 1, 32, 128, 256, 1024, 2048, 16384, },
  { 2, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 32, 128, 256, 1024, 2048, 16384, },
  { 4, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 32, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 16384, },
  { 8, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 8, 32, 128, 256, 1024, 2048, 16384, },
  { 2, 8, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 16384, },
  { 4, 8, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 16384, },
  { 16, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 2, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 4, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 8, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 16384, },
  { 64, 128, 256, 1024, 2048, 16384, },
  { 1, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 64, 128, 256, 1024, 2048, 16384, },
  { 4, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 16384, },
  { 8, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 8, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 8, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 16384, },
  { 4, 8, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 16384, },
  { 16, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 4, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 8, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 16384, },
  { 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 4, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 8, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, },
  { 512, 1024, 2048, 16384, },
  { 1, 512, 1024, 2048, 16384, },
  { 2, 512, 1024, 2048, 16384, },
  { 1, 2, 512, 1024, 2048, 16384, },
  { 4, 512, 1024, 2048, 16384, },
  { 1, 4, 512, 1024, 2048, 16384, },
  { 2, 4, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 512, 1024, 2048, 16384, },
  { 8, 512, 1024, 2048, 16384, },
  { 1, 8, 512, 1024, 2048, 16384, },
  { 2, 8, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 512, 1024, 2048, 16384, },
  { 4, 8, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 512, 1024, 2048, 16384, },
  { 16, 512, 1024, 2048, 16384, },
  { 1, 16, 512, 1024, 2048, 16384, },
  { 2, 16, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 512, 1024, 2048, 16384, },
  { 4, 16, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 512, 1024, 2048, 16384, },
  { 8, 16, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 16384, },
  { 32, 512, 1024, 2048, 16384, },
  { 1, 32, 512, 1024, 2048, 16384, },
  { 2, 32, 512, 1024, 2048, 16384, },
  { 1, 2, 32, 512, 1024, 2048, 16384, },
  { 4, 32, 512, 1024, 2048, 16384, },
  { 1, 4, 32, 512, 1024, 2048, 16384, },
  { 2, 4, 32, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 512, 1024, 2048, 16384, },
  { 8, 32, 512, 1024, 2048, 16384, },
  { 1, 8, 32, 512, 1024, 2048, 16384, },
  { 2, 8, 32, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 512, 1024, 2048, 16384, },
  { 4, 8, 32, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 16384, },
  { 16, 32, 512, 1024, 2048, 16384, },
  { 1, 16, 32, 512, 1024, 2048, 16384, },
  { 2, 16, 32, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 512, 1024, 2048, 16384, },
  { 4, 16, 32, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 16384, },
  { 8, 16, 32, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 16384, },
  { 64, 512, 1024, 2048, 16384, },
  { 1, 64, 512, 1024, 2048, 16384, },
  { 2, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 64, 512, 1024, 2048, 16384, },
  { 4, 64, 512, 1024, 2048, 16384, },
  { 1, 4, 64, 512, 1024, 2048, 16384, },
  { 2, 4, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 64, 512, 1024, 2048, 16384, },
  { 8, 64, 512, 1024, 2048, 16384, },
  { 1, 8, 64, 512, 1024, 2048, 16384, },
  { 2, 8, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 64, 512, 1024, 2048, 16384, },
  { 4, 8, 64, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 64, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 16384, },
  { 16, 64, 512, 1024, 2048, 16384, },
  { 1, 16, 64, 512, 1024, 2048, 16384, },
  { 2, 16, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 64, 512, 1024, 2048, 16384, },
  { 4, 16, 64, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 64, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 16384, },
  { 8, 16, 64, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 64, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 64, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 16384, },
  { 32, 64, 512, 1024, 2048, 16384, },
  { 1, 32, 64, 512, 1024, 2048, 16384, },
  { 2, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 32, 64, 512, 1024, 2048, 16384, },
  { 4, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 4, 32, 64, 512, 1024, 2048, 16384, },
  { 2, 4, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 16384, },
  { 8, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 8, 32, 64, 512, 1024, 2048, 16384, },
  { 2, 8, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 16384, },
  { 4, 8, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 16384, },
  { 16, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 2, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 4, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 8, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 16384, },
  { 128, 512, 1024, 2048, 16384, },
  { 1, 128, 512, 1024, 2048, 16384, },
  { 2, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 128, 512, 1024, 2048, 16384, },
  { 4, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 128, 512, 1024, 2048, 16384, },
  { 8, 128, 512, 1024, 2048, 16384, },
  { 1, 8, 128, 512, 1024, 2048, 16384, },
  { 2, 8, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 128, 512, 1024, 2048, 16384, },
  { 4, 8, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 16384, },
  { 16, 128, 512, 1024, 2048, 16384, },
  { 1, 16, 128, 512, 1024, 2048, 16384, },
  { 2, 16, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 128, 512, 1024, 2048, 16384, },
  { 4, 16, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 16384, },
  { 8, 16, 128, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 128, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 16384, },
  { 32, 128, 512, 1024, 2048, 16384, },
  { 1, 32, 128, 512, 1024, 2048, 16384, },
  { 2, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 32, 128, 512, 1024, 2048, 16384, },
  { 4, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 32, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 16384, },
  { 8, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 8, 32, 128, 512, 1024, 2048, 16384, },
  { 2, 8, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 16384, },
  { 4, 8, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 16384, },
  { 16, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 2, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 4, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 8, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 16384, },
  { 64, 128, 512, 1024, 2048, 16384, },
  { 1, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 64, 128, 512, 1024, 2048, 16384, },
  { 4, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 16384, },
  { 8, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 8, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 8, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 16384, },
  { 4, 8, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 16384, },
  { 16, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 4, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 8, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 16384, },
  { 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 4, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 8, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, },
  { 256, 512, 1024, 2048, 16384, },
  { 1, 256, 512, 1024, 2048, 16384, },
  { 2, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 256, 512, 1024, 2048, 16384, },
  { 4, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 256, 512, 1024, 2048, 16384, },
  { 8, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 16384, },
  { 16, 256, 512, 1024, 2048, 16384, },
  { 1, 16, 256, 512, 1024, 2048, 16384, },
  { 2, 16, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 256, 512, 1024, 2048, 16384, },
  { 4, 16, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 16384, },
  { 8, 16, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 16384, },
  { 32, 256, 512, 1024, 2048, 16384, },
  { 1, 32, 256, 512, 1024, 2048, 16384, },
  { 2, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 32, 256, 512, 1024, 2048, 16384, },
  { 4, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 32, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 16384, },
  { 8, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 32, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 16384, },
  { 16, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 2, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 4, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 8, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 16384, },
  { 64, 256, 512, 1024, 2048, 16384, },
  { 1, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 64, 256, 512, 1024, 2048, 16384, },
  { 4, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 16384, },
  { 8, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 16384, },
  { 16, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 4, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 8, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 16384, },
  { 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 4, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 8, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, },
  { 128, 256, 512, 1024, 2048, 16384, },
  { 1, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 16384, },
  { 8, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 16384, },
  { 16, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 8, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 16384, },
  { 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 8, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, },
  { 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 8, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, },
  { 4096, 16384, },
  { 1, 4096, 16384, },
  { 2, 4096, 16384, },
  { 1, 2, 4096, 16384, },
  { 4, 4096, 16384, },
  { 1, 4, 4096, 16384, },
  { 2, 4, 4096, 16384, },
  { 1, 2, 4, 4096, 16384, },
  { 8, 4096, 16384, },
  { 1, 8, 4096, 16384, },
  { 2, 8, 4096, 16384, },
  { 1, 2, 8, 4096, 16384, },
  { 4, 8, 4096, 16384, },
  { 1, 4, 8, 4096, 16384, },
  { 2, 4, 8, 4096, 16384, },
  { 1, 2, 4, 8, 4096, 16384, },
  { 16, 4096, 16384, },
  { 1, 16, 4096, 16384, },
  { 2, 16, 4096, 16384, },
  { 1, 2, 16, 4096, 16384, },
  { 4, 16, 4096, 16384, },
  { 1, 4, 16, 4096, 16384, },
  { 2, 4, 16, 4096, 16384, },
  { 1, 2, 4, 16, 4096, 16384, },
  { 8, 16, 4096, 16384, },
  { 1, 8, 16, 4096, 16384, },
  { 2, 8, 16, 4096, 16384, },
  { 1, 2, 8, 16, 4096, 16384, },
  { 4, 8, 16, 4096, 16384, },
  { 1, 4, 8, 16, 4096, 16384, },
  { 2, 4, 8, 16, 4096, 16384, },
  { 1, 2, 4, 8, 16, 4096, 16384, },
  { 32, 4096, 16384, },
  { 1, 32, 4096, 16384, },
  { 2, 32, 4096, 16384, },
  { 1, 2, 32, 4096, 16384, },
  { 4, 32, 4096, 16384, },
  { 1, 4, 32, 4096, 16384, },
  { 2, 4, 32, 4096, 16384, },
  { 1, 2, 4, 32, 4096, 16384, },
  { 8, 32, 4096, 16384, },
  { 1, 8, 32, 4096, 16384, },
  { 2, 8, 32, 4096, 16384, },
  { 1, 2, 8, 32, 4096, 16384, },
  { 4, 8, 32, 4096, 16384, },
  { 1, 4, 8, 32, 4096, 16384, },
  { 2, 4, 8, 32, 4096, 16384, },
  { 1, 2, 4, 8, 32, 4096, 16384, },
  { 16, 32, 4096, 16384, },
  { 1, 16, 32, 4096, 16384, },
  { 2, 16, 32, 4096, 16384, },
  { 1, 2, 16, 32, 4096, 16384, },
  { 4, 16, 32, 4096, 16384, },
  { 1, 4, 16, 32, 4096, 16384, },
  { 2, 4, 16, 32, 4096, 16384, },
  { 1, 2, 4, 16, 32, 4096, 16384, },
  { 8, 16, 32, 4096, 16384, },
  { 1, 8, 16, 32, 4096, 16384, },
  { 2, 8, 16, 32, 4096, 16384, },
  { 1, 2, 8, 16, 32, 4096, 16384, },
  { 4, 8, 16, 32, 4096, 16384, },
  { 1, 4, 8, 16, 32, 4096, 16384, },
  { 2, 4, 8, 16, 32, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 4096, 16384, },
  { 64, 4096, 16384, },
  { 1, 64, 4096, 16384, },
  { 2, 64, 4096, 16384, },
  { 1, 2, 64, 4096, 16384, },
  { 4, 64, 4096, 16384, },
  { 1, 4, 64, 4096, 16384, },
  { 2, 4, 64, 4096, 16384, },
  { 1, 2, 4, 64, 4096, 16384, },
  { 8, 64, 4096, 16384, },
  { 1, 8, 64, 4096, 16384, },
  { 2, 8, 64, 4096, 16384, },
  { 1, 2, 8, 64, 4096, 16384, },
  { 4, 8, 64, 4096, 16384, },
  { 1, 4, 8, 64, 4096, 16384, },
  { 2, 4, 8, 64, 4096, 16384, },
  { 1, 2, 4, 8, 64, 4096, 16384, },
  { 16, 64, 4096, 16384, },
  { 1, 16, 64, 4096, 16384, },
  { 2, 16, 64, 4096, 16384, },
  { 1, 2, 16, 64, 4096, 16384, },
  { 4, 16, 64, 4096, 16384, },
  { 1, 4, 16, 64, 4096, 16384, },
  { 2, 4, 16, 64, 4096, 16384, },
  { 1, 2, 4, 16, 64, 4096, 16384, },
  { 8, 16, 64, 4096, 16384, },
  { 1, 8, 16, 64, 4096, 16384, },
  { 2, 8, 16, 64, 4096, 16384, },
  { 1, 2, 8, 16, 64, 4096, 16384, },
  { 4, 8, 16, 64, 4096, 16384, },
  { 1, 4, 8, 16, 64, 4096, 16384, },
  { 2, 4, 8, 16, 64, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 4096, 16384, },
  { 32, 64, 4096, 16384, },
  { 1, 32, 64, 4096, 16384, },
  { 2, 32, 64, 4096, 16384, },
  { 1, 2, 32, 64, 4096, 16384, },
  { 4, 32, 64, 4096, 16384, },
  { 1, 4, 32, 64, 4096, 16384, },
  { 2, 4, 32, 64, 4096, 16384, },
  { 1, 2, 4, 32, 64, 4096, 16384, },
  { 8, 32, 64, 4096, 16384, },
  { 1, 8, 32, 64, 4096, 16384, },
  { 2, 8, 32, 64, 4096, 16384, },
  { 1, 2, 8, 32, 64, 4096, 16384, },
  { 4, 8, 32, 64, 4096, 16384, },
  { 1, 4, 8, 32, 64, 4096, 16384, },
  { 2, 4, 8, 32, 64, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 4096, 16384, },
  { 16, 32, 64, 4096, 16384, },
  { 1, 16, 32, 64, 4096, 16384, },
  { 2, 16, 32, 64, 4096, 16384, },
  { 1, 2, 16, 32, 64, 4096, 16384, },
  { 4, 16, 32, 64, 4096, 16384, },
  { 1, 4, 16, 32, 64, 4096, 16384, },
  { 2, 4, 16, 32, 64, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 4096, 16384, },
  { 8, 16, 32, 64, 4096, 16384, },
  { 1, 8, 16, 32, 64, 4096, 16384, },
  { 2, 8, 16, 32, 64, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 4096, 16384, },
  { 4, 8, 16, 32, 64, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 4096, 16384, },
  { 128, 4096, 16384, },
  { 1, 128, 4096, 16384, },
  { 2, 128, 4096, 16384, },
  { 1, 2, 128, 4096, 16384, },
  { 4, 128, 4096, 16384, },
  { 1, 4, 128, 4096, 16384, },
  { 2, 4, 128, 4096, 16384, },
  { 1, 2, 4, 128, 4096, 16384, },
  { 8, 128, 4096, 16384, },
  { 1, 8, 128, 4096, 16384, },
  { 2, 8, 128, 4096, 16384, },
  { 1, 2, 8, 128, 4096, 16384, },
  { 4, 8, 128, 4096, 16384, },
  { 1, 4, 8, 128, 4096, 16384, },
  { 2, 4, 8, 128, 4096, 16384, },
  { 1, 2, 4, 8, 128, 4096, 16384, },
  { 16, 128, 4096, 16384, },
  { 1, 16, 128, 4096, 16384, },
  { 2, 16, 128, 4096, 16384, },
  { 1, 2, 16, 128, 4096, 16384, },
  { 4, 16, 128, 4096, 16384, },
  { 1, 4, 16, 128, 4096, 16384, },
  { 2, 4, 16, 128, 4096, 16384, },
  { 1, 2, 4, 16, 128, 4096, 16384, },
  { 8, 16, 128, 4096, 16384, },
  { 1, 8, 16, 128, 4096, 16384, },
  { 2, 8, 16, 128, 4096, 16384, },
  { 1, 2, 8, 16, 128, 4096, 16384, },
  { 4, 8, 16, 128, 4096, 16384, },
  { 1, 4, 8, 16, 128, 4096, 16384, },
  { 2, 4, 8, 16, 128, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 4096, 16384, },
  { 32, 128, 4096, 16384, },
  { 1, 32, 128, 4096, 16384, },
  { 2, 32, 128, 4096, 16384, },
  { 1, 2, 32, 128, 4096, 16384, },
  { 4, 32, 128, 4096, 16384, },
  { 1, 4, 32, 128, 4096, 16384, },
  { 2, 4, 32, 128, 4096, 16384, },
  { 1, 2, 4, 32, 128, 4096, 16384, },
  { 8, 32, 128, 4096, 16384, },
  { 1, 8, 32, 128, 4096, 16384, },
  { 2, 8, 32, 128, 4096, 16384, },
  { 1, 2, 8, 32, 128, 4096, 16384, },
  { 4, 8, 32, 128, 4096, 16384, },
  { 1, 4, 8, 32, 128, 4096, 16384, },
  { 2, 4, 8, 32, 128, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 4096, 16384, },
  { 16, 32, 128, 4096, 16384, },
  { 1, 16, 32, 128, 4096, 16384, },
  { 2, 16, 32, 128, 4096, 16384, },
  { 1, 2, 16, 32, 128, 4096, 16384, },
  { 4, 16, 32, 128, 4096, 16384, },
  { 1, 4, 16, 32, 128, 4096, 16384, },
  { 2, 4, 16, 32, 128, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 4096, 16384, },
  { 8, 16, 32, 128, 4096, 16384, },
  { 1, 8, 16, 32, 128, 4096, 16384, },
  { 2, 8, 16, 32, 128, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 4096, 16384, },
  { 4, 8, 16, 32, 128, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 4096, 16384, },
  { 64, 128, 4096, 16384, },
  { 1, 64, 128, 4096, 16384, },
  { 2, 64, 128, 4096, 16384, },
  { 1, 2, 64, 128, 4096, 16384, },
  { 4, 64, 128, 4096, 16384, },
  { 1, 4, 64, 128, 4096, 16384, },
  { 2, 4, 64, 128, 4096, 16384, },
  { 1, 2, 4, 64, 128, 4096, 16384, },
  { 8, 64, 128, 4096, 16384, },
  { 1, 8, 64, 128, 4096, 16384, },
  { 2, 8, 64, 128, 4096, 16384, },
  { 1, 2, 8, 64, 128, 4096, 16384, },
  { 4, 8, 64, 128, 4096, 16384, },
  { 1, 4, 8, 64, 128, 4096, 16384, },
  { 2, 4, 8, 64, 128, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 4096, 16384, },
  { 16, 64, 128, 4096, 16384, },
  { 1, 16, 64, 128, 4096, 16384, },
  { 2, 16, 64, 128, 4096, 16384, },
  { 1, 2, 16, 64, 128, 4096, 16384, },
  { 4, 16, 64, 128, 4096, 16384, },
  { 1, 4, 16, 64, 128, 4096, 16384, },
  { 2, 4, 16, 64, 128, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 4096, 16384, },
  { 8, 16, 64, 128, 4096, 16384, },
  { 1, 8, 16, 64, 128, 4096, 16384, },
  { 2, 8, 16, 64, 128, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 4096, 16384, },
  { 4, 8, 16, 64, 128, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 4096, 16384, },
  { 32, 64, 128, 4096, 16384, },
  { 1, 32, 64, 128, 4096, 16384, },
  { 2, 32, 64, 128, 4096, 16384, },
  { 1, 2, 32, 64, 128, 4096, 16384, },
  { 4, 32, 64, 128, 4096, 16384, },
  { 1, 4, 32, 64, 128, 4096, 16384, },
  { 2, 4, 32, 64, 128, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 4096, 16384, },
  { 8, 32, 64, 128, 4096, 16384, },
  { 1, 8, 32, 64, 128, 4096, 16384, },
  { 2, 8, 32, 64, 128, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 4096, 16384, },
  { 4, 8, 32, 64, 128, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 4096, 16384, },
  { 16, 32, 64, 128, 4096, 16384, },
  { 1, 16, 32, 64, 128, 4096, 16384, },
  { 2, 16, 32, 64, 128, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 4096, 16384, },
  { 4, 16, 32, 64, 128, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 4096, 16384, },
  { 8, 16, 32, 64, 128, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 4096, 16384, },
  { 256, 4096, 16384, },
  { 1, 256, 4096, 16384, },
  { 2, 256, 4096, 16384, },
  { 1, 2, 256, 4096, 16384, },
  { 4, 256, 4096, 16384, },
  { 1, 4, 256, 4096, 16384, },
  { 2, 4, 256, 4096, 16384, },
  { 1, 2, 4, 256, 4096, 16384, },
  { 8, 256, 4096, 16384, },
  { 1, 8, 256, 4096, 16384, },
  { 2, 8, 256, 4096, 16384, },
  { 1, 2, 8, 256, 4096, 16384, },
  { 4, 8, 256, 4096, 16384, },
  { 1, 4, 8, 256, 4096, 16384, },
  { 2, 4, 8, 256, 4096, 16384, },
  { 1, 2, 4, 8, 256, 4096, 16384, },
  { 16, 256, 4096, 16384, },
  { 1, 16, 256, 4096, 16384, },
  { 2, 16, 256, 4096, 16384, },
  { 1, 2, 16, 256, 4096, 16384, },
  { 4, 16, 256, 4096, 16384, },
  { 1, 4, 16, 256, 4096, 16384, },
  { 2, 4, 16, 256, 4096, 16384, },
  { 1, 2, 4, 16, 256, 4096, 16384, },
  { 8, 16, 256, 4096, 16384, },
  { 1, 8, 16, 256, 4096, 16384, },
  { 2, 8, 16, 256, 4096, 16384, },
  { 1, 2, 8, 16, 256, 4096, 16384, },
  { 4, 8, 16, 256, 4096, 16384, },
  { 1, 4, 8, 16, 256, 4096, 16384, },
  { 2, 4, 8, 16, 256, 4096, 16384, },
  { 1, 2, 4, 8, 16, 256, 4096, 16384, },
  { 32, 256, 4096, 16384, },
  { 1, 32, 256, 4096, 16384, },
  { 2, 32, 256, 4096, 16384, },
  { 1, 2, 32, 256, 4096, 16384, },
  { 4, 32, 256, 4096, 16384, },
  { 1, 4, 32, 256, 4096, 16384, },
  { 2, 4, 32, 256, 4096, 16384, },
  { 1, 2, 4, 32, 256, 4096, 16384, },
  { 8, 32, 256, 4096, 16384, },
  { 1, 8, 32, 256, 4096, 16384, },
  { 2, 8, 32, 256, 4096, 16384, },
  { 1, 2, 8, 32, 256, 4096, 16384, },
  { 4, 8, 32, 256, 4096, 16384, },
  { 1, 4, 8, 32, 256, 4096, 16384, },
  { 2, 4, 8, 32, 256, 4096, 16384, },
  { 1, 2, 4, 8, 32, 256, 4096, 16384, },
  { 16, 32, 256, 4096, 16384, },
  { 1, 16, 32, 256, 4096, 16384, },
  { 2, 16, 32, 256, 4096, 16384, },
  { 1, 2, 16, 32, 256, 4096, 16384, },
  { 4, 16, 32, 256, 4096, 16384, },
  { 1, 4, 16, 32, 256, 4096, 16384, },
  { 2, 4, 16, 32, 256, 4096, 16384, },
  { 1, 2, 4, 16, 32, 256, 4096, 16384, },
  { 8, 16, 32, 256, 4096, 16384, },
  { 1, 8, 16, 32, 256, 4096, 16384, },
  { 2, 8, 16, 32, 256, 4096, 16384, },
  { 1, 2, 8, 16, 32, 256, 4096, 16384, },
  { 4, 8, 16, 32, 256, 4096, 16384, },
  { 1, 4, 8, 16, 32, 256, 4096, 16384, },
  { 2, 4, 8, 16, 32, 256, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 4096, 16384, },
  { 64, 256, 4096, 16384, },
  { 1, 64, 256, 4096, 16384, },
  { 2, 64, 256, 4096, 16384, },
  { 1, 2, 64, 256, 4096, 16384, },
  { 4, 64, 256, 4096, 16384, },
  { 1, 4, 64, 256, 4096, 16384, },
  { 2, 4, 64, 256, 4096, 16384, },
  { 1, 2, 4, 64, 256, 4096, 16384, },
  { 8, 64, 256, 4096, 16384, },
  { 1, 8, 64, 256, 4096, 16384, },
  { 2, 8, 64, 256, 4096, 16384, },
  { 1, 2, 8, 64, 256, 4096, 16384, },
  { 4, 8, 64, 256, 4096, 16384, },
  { 1, 4, 8, 64, 256, 4096, 16384, },
  { 2, 4, 8, 64, 256, 4096, 16384, },
  { 1, 2, 4, 8, 64, 256, 4096, 16384, },
  { 16, 64, 256, 4096, 16384, },
  { 1, 16, 64, 256, 4096, 16384, },
  { 2, 16, 64, 256, 4096, 16384, },
  { 1, 2, 16, 64, 256, 4096, 16384, },
  { 4, 16, 64, 256, 4096, 16384, },
  { 1, 4, 16, 64, 256, 4096, 16384, },
  { 2, 4, 16, 64, 256, 4096, 16384, },
  { 1, 2, 4, 16, 64, 256, 4096, 16384, },
  { 8, 16, 64, 256, 4096, 16384, },
  { 1, 8, 16, 64, 256, 4096, 16384, },
  { 2, 8, 16, 64, 256, 4096, 16384, },
  { 1, 2, 8, 16, 64, 256, 4096, 16384, },
  { 4, 8, 16, 64, 256, 4096, 16384, },
  { 1, 4, 8, 16, 64, 256, 4096, 16384, },
  { 2, 4, 8, 16, 64, 256, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 4096, 16384, },
  { 32, 64, 256, 4096, 16384, },
  { 1, 32, 64, 256, 4096, 16384, },
  { 2, 32, 64, 256, 4096, 16384, },
  { 1, 2, 32, 64, 256, 4096, 16384, },
  { 4, 32, 64, 256, 4096, 16384, },
  { 1, 4, 32, 64, 256, 4096, 16384, },
  { 2, 4, 32, 64, 256, 4096, 16384, },
  { 1, 2, 4, 32, 64, 256, 4096, 16384, },
  { 8, 32, 64, 256, 4096, 16384, },
  { 1, 8, 32, 64, 256, 4096, 16384, },
  { 2, 8, 32, 64, 256, 4096, 16384, },
  { 1, 2, 8, 32, 64, 256, 4096, 16384, },
  { 4, 8, 32, 64, 256, 4096, 16384, },
  { 1, 4, 8, 32, 64, 256, 4096, 16384, },
  { 2, 4, 8, 32, 64, 256, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 4096, 16384, },
  { 16, 32, 64, 256, 4096, 16384, },
  { 1, 16, 32, 64, 256, 4096, 16384, },
  { 2, 16, 32, 64, 256, 4096, 16384, },
  { 1, 2, 16, 32, 64, 256, 4096, 16384, },
  { 4, 16, 32, 64, 256, 4096, 16384, },
  { 1, 4, 16, 32, 64, 256, 4096, 16384, },
  { 2, 4, 16, 32, 64, 256, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 4096, 16384, },
  { 8, 16, 32, 64, 256, 4096, 16384, },
  { 1, 8, 16, 32, 64, 256, 4096, 16384, },
  { 2, 8, 16, 32, 64, 256, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 4096, 16384, },
  { 4, 8, 16, 32, 64, 256, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 4096, 16384, },
  { 128, 256, 4096, 16384, },
  { 1, 128, 256, 4096, 16384, },
  { 2, 128, 256, 4096, 16384, },
  { 1, 2, 128, 256, 4096, 16384, },
  { 4, 128, 256, 4096, 16384, },
  { 1, 4, 128, 256, 4096, 16384, },
  { 2, 4, 128, 256, 4096, 16384, },
  { 1, 2, 4, 128, 256, 4096, 16384, },
  { 8, 128, 256, 4096, 16384, },
  { 1, 8, 128, 256, 4096, 16384, },
  { 2, 8, 128, 256, 4096, 16384, },
  { 1, 2, 8, 128, 256, 4096, 16384, },
  { 4, 8, 128, 256, 4096, 16384, },
  { 1, 4, 8, 128, 256, 4096, 16384, },
  { 2, 4, 8, 128, 256, 4096, 16384, },
  { 1, 2, 4, 8, 128, 256, 4096, 16384, },
  { 16, 128, 256, 4096, 16384, },
  { 1, 16, 128, 256, 4096, 16384, },
  { 2, 16, 128, 256, 4096, 16384, },
  { 1, 2, 16, 128, 256, 4096, 16384, },
  { 4, 16, 128, 256, 4096, 16384, },
  { 1, 4, 16, 128, 256, 4096, 16384, },
  { 2, 4, 16, 128, 256, 4096, 16384, },
  { 1, 2, 4, 16, 128, 256, 4096, 16384, },
  { 8, 16, 128, 256, 4096, 16384, },
  { 1, 8, 16, 128, 256, 4096, 16384, },
  { 2, 8, 16, 128, 256, 4096, 16384, },
  { 1, 2, 8, 16, 128, 256, 4096, 16384, },
  { 4, 8, 16, 128, 256, 4096, 16384, },
  { 1, 4, 8, 16, 128, 256, 4096, 16384, },
  { 2, 4, 8, 16, 128, 256, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 4096, 16384, },
  { 32, 128, 256, 4096, 16384, },
  { 1, 32, 128, 256, 4096, 16384, },
  { 2, 32, 128, 256, 4096, 16384, },
  { 1, 2, 32, 128, 256, 4096, 16384, },
  { 4, 32, 128, 256, 4096, 16384, },
  { 1, 4, 32, 128, 256, 4096, 16384, },
  { 2, 4, 32, 128, 256, 4096, 16384, },
  { 1, 2, 4, 32, 128, 256, 4096, 16384, },
  { 8, 32, 128, 256, 4096, 16384, },
  { 1, 8, 32, 128, 256, 4096, 16384, },
  { 2, 8, 32, 128, 256, 4096, 16384, },
  { 1, 2, 8, 32, 128, 256, 4096, 16384, },
  { 4, 8, 32, 128, 256, 4096, 16384, },
  { 1, 4, 8, 32, 128, 256, 4096, 16384, },
  { 2, 4, 8, 32, 128, 256, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 4096, 16384, },
  { 16, 32, 128, 256, 4096, 16384, },
  { 1, 16, 32, 128, 256, 4096, 16384, },
  { 2, 16, 32, 128, 256, 4096, 16384, },
  { 1, 2, 16, 32, 128, 256, 4096, 16384, },
  { 4, 16, 32, 128, 256, 4096, 16384, },
  { 1, 4, 16, 32, 128, 256, 4096, 16384, },
  { 2, 4, 16, 32, 128, 256, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 4096, 16384, },
  { 8, 16, 32, 128, 256, 4096, 16384, },
  { 1, 8, 16, 32, 128, 256, 4096, 16384, },
  { 2, 8, 16, 32, 128, 256, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 4096, 16384, },
  { 4, 8, 16, 32, 128, 256, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 4096, 16384, },
  { 64, 128, 256, 4096, 16384, },
  { 1, 64, 128, 256, 4096, 16384, },
  { 2, 64, 128, 256, 4096, 16384, },
  { 1, 2, 64, 128, 256, 4096, 16384, },
  { 4, 64, 128, 256, 4096, 16384, },
  { 1, 4, 64, 128, 256, 4096, 16384, },
  { 2, 4, 64, 128, 256, 4096, 16384, },
  { 1, 2, 4, 64, 128, 256, 4096, 16384, },
  { 8, 64, 128, 256, 4096, 16384, },
  { 1, 8, 64, 128, 256, 4096, 16384, },
  { 2, 8, 64, 128, 256, 4096, 16384, },
  { 1, 2, 8, 64, 128, 256, 4096, 16384, },
  { 4, 8, 64, 128, 256, 4096, 16384, },
  { 1, 4, 8, 64, 128, 256, 4096, 16384, },
  { 2, 4, 8, 64, 128, 256, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 4096, 16384, },
  { 16, 64, 128, 256, 4096, 16384, },
  { 1, 16, 64, 128, 256, 4096, 16384, },
  { 2, 16, 64, 128, 256, 4096, 16384, },
  { 1, 2, 16, 64, 128, 256, 4096, 16384, },
  { 4, 16, 64, 128, 256, 4096, 16384, },
  { 1, 4, 16, 64, 128, 256, 4096, 16384, },
  { 2, 4, 16, 64, 128, 256, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 4096, 16384, },
  { 8, 16, 64, 128, 256, 4096, 16384, },
  { 1, 8, 16, 64, 128, 256, 4096, 16384, },
  { 2, 8, 16, 64, 128, 256, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 4096, 16384, },
  { 4, 8, 16, 64, 128, 256, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 4096, 16384, },
  { 32, 64, 128, 256, 4096, 16384, },
  { 1, 32, 64, 128, 256, 4096, 16384, },
  { 2, 32, 64, 128, 256, 4096, 16384, },
  { 1, 2, 32, 64, 128, 256, 4096, 16384, },
  { 4, 32, 64, 128, 256, 4096, 16384, },
  { 1, 4, 32, 64, 128, 256, 4096, 16384, },
  { 2, 4, 32, 64, 128, 256, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 4096, 16384, },
  { 8, 32, 64, 128, 256, 4096, 16384, },
  { 1, 8, 32, 64, 128, 256, 4096, 16384, },
  { 2, 8, 32, 64, 128, 256, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 4096, 16384, },
  { 4, 8, 32, 64, 128, 256, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 4096, 16384, },
  { 16, 32, 64, 128, 256, 4096, 16384, },
  { 1, 16, 32, 64, 128, 256, 4096, 16384, },
  { 2, 16, 32, 64, 128, 256, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 4096, 16384, },
  { 4, 16, 32, 64, 128, 256, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 4096, 16384, },
  { 8, 16, 32, 64, 128, 256, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 4096, 16384, },
  { 512, 4096, 16384, },
  { 1, 512, 4096, 16384, },
  { 2, 512, 4096, 16384, },
  { 1, 2, 512, 4096, 16384, },
  { 4, 512, 4096, 16384, },
  { 1, 4, 512, 4096, 16384, },
  { 2, 4, 512, 4096, 16384, },
  { 1, 2, 4, 512, 4096, 16384, },
  { 8, 512, 4096, 16384, },
  { 1, 8, 512, 4096, 16384, },
  { 2, 8, 512, 4096, 16384, },
  { 1, 2, 8, 512, 4096, 16384, },
  { 4, 8, 512, 4096, 16384, },
  { 1, 4, 8, 512, 4096, 16384, },
  { 2, 4, 8, 512, 4096, 16384, },
  { 1, 2, 4, 8, 512, 4096, 16384, },
  { 16, 512, 4096, 16384, },
  { 1, 16, 512, 4096, 16384, },
  { 2, 16, 512, 4096, 16384, },
  { 1, 2, 16, 512, 4096, 16384, },
  { 4, 16, 512, 4096, 16384, },
  { 1, 4, 16, 512, 4096, 16384, },
  { 2, 4, 16, 512, 4096, 16384, },
  { 1, 2, 4, 16, 512, 4096, 16384, },
  { 8, 16, 512, 4096, 16384, },
  { 1, 8, 16, 512, 4096, 16384, },
  { 2, 8, 16, 512, 4096, 16384, },
  { 1, 2, 8, 16, 512, 4096, 16384, },
  { 4, 8, 16, 512, 4096, 16384, },
  { 1, 4, 8, 16, 512, 4096, 16384, },
  { 2, 4, 8, 16, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 512, 4096, 16384, },
  { 32, 512, 4096, 16384, },
  { 1, 32, 512, 4096, 16384, },
  { 2, 32, 512, 4096, 16384, },
  { 1, 2, 32, 512, 4096, 16384, },
  { 4, 32, 512, 4096, 16384, },
  { 1, 4, 32, 512, 4096, 16384, },
  { 2, 4, 32, 512, 4096, 16384, },
  { 1, 2, 4, 32, 512, 4096, 16384, },
  { 8, 32, 512, 4096, 16384, },
  { 1, 8, 32, 512, 4096, 16384, },
  { 2, 8, 32, 512, 4096, 16384, },
  { 1, 2, 8, 32, 512, 4096, 16384, },
  { 4, 8, 32, 512, 4096, 16384, },
  { 1, 4, 8, 32, 512, 4096, 16384, },
  { 2, 4, 8, 32, 512, 4096, 16384, },
  { 1, 2, 4, 8, 32, 512, 4096, 16384, },
  { 16, 32, 512, 4096, 16384, },
  { 1, 16, 32, 512, 4096, 16384, },
  { 2, 16, 32, 512, 4096, 16384, },
  { 1, 2, 16, 32, 512, 4096, 16384, },
  { 4, 16, 32, 512, 4096, 16384, },
  { 1, 4, 16, 32, 512, 4096, 16384, },
  { 2, 4, 16, 32, 512, 4096, 16384, },
  { 1, 2, 4, 16, 32, 512, 4096, 16384, },
  { 8, 16, 32, 512, 4096, 16384, },
  { 1, 8, 16, 32, 512, 4096, 16384, },
  { 2, 8, 16, 32, 512, 4096, 16384, },
  { 1, 2, 8, 16, 32, 512, 4096, 16384, },
  { 4, 8, 16, 32, 512, 4096, 16384, },
  { 1, 4, 8, 16, 32, 512, 4096, 16384, },
  { 2, 4, 8, 16, 32, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 4096, 16384, },
  { 64, 512, 4096, 16384, },
  { 1, 64, 512, 4096, 16384, },
  { 2, 64, 512, 4096, 16384, },
  { 1, 2, 64, 512, 4096, 16384, },
  { 4, 64, 512, 4096, 16384, },
  { 1, 4, 64, 512, 4096, 16384, },
  { 2, 4, 64, 512, 4096, 16384, },
  { 1, 2, 4, 64, 512, 4096, 16384, },
  { 8, 64, 512, 4096, 16384, },
  { 1, 8, 64, 512, 4096, 16384, },
  { 2, 8, 64, 512, 4096, 16384, },
  { 1, 2, 8, 64, 512, 4096, 16384, },
  { 4, 8, 64, 512, 4096, 16384, },
  { 1, 4, 8, 64, 512, 4096, 16384, },
  { 2, 4, 8, 64, 512, 4096, 16384, },
  { 1, 2, 4, 8, 64, 512, 4096, 16384, },
  { 16, 64, 512, 4096, 16384, },
  { 1, 16, 64, 512, 4096, 16384, },
  { 2, 16, 64, 512, 4096, 16384, },
  { 1, 2, 16, 64, 512, 4096, 16384, },
  { 4, 16, 64, 512, 4096, 16384, },
  { 1, 4, 16, 64, 512, 4096, 16384, },
  { 2, 4, 16, 64, 512, 4096, 16384, },
  { 1, 2, 4, 16, 64, 512, 4096, 16384, },
  { 8, 16, 64, 512, 4096, 16384, },
  { 1, 8, 16, 64, 512, 4096, 16384, },
  { 2, 8, 16, 64, 512, 4096, 16384, },
  { 1, 2, 8, 16, 64, 512, 4096, 16384, },
  { 4, 8, 16, 64, 512, 4096, 16384, },
  { 1, 4, 8, 16, 64, 512, 4096, 16384, },
  { 2, 4, 8, 16, 64, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 4096, 16384, },
  { 32, 64, 512, 4096, 16384, },
  { 1, 32, 64, 512, 4096, 16384, },
  { 2, 32, 64, 512, 4096, 16384, },
  { 1, 2, 32, 64, 512, 4096, 16384, },
  { 4, 32, 64, 512, 4096, 16384, },
  { 1, 4, 32, 64, 512, 4096, 16384, },
  { 2, 4, 32, 64, 512, 4096, 16384, },
  { 1, 2, 4, 32, 64, 512, 4096, 16384, },
  { 8, 32, 64, 512, 4096, 16384, },
  { 1, 8, 32, 64, 512, 4096, 16384, },
  { 2, 8, 32, 64, 512, 4096, 16384, },
  { 1, 2, 8, 32, 64, 512, 4096, 16384, },
  { 4, 8, 32, 64, 512, 4096, 16384, },
  { 1, 4, 8, 32, 64, 512, 4096, 16384, },
  { 2, 4, 8, 32, 64, 512, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 4096, 16384, },
  { 16, 32, 64, 512, 4096, 16384, },
  { 1, 16, 32, 64, 512, 4096, 16384, },
  { 2, 16, 32, 64, 512, 4096, 16384, },
  { 1, 2, 16, 32, 64, 512, 4096, 16384, },
  { 4, 16, 32, 64, 512, 4096, 16384, },
  { 1, 4, 16, 32, 64, 512, 4096, 16384, },
  { 2, 4, 16, 32, 64, 512, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 4096, 16384, },
  { 8, 16, 32, 64, 512, 4096, 16384, },
  { 1, 8, 16, 32, 64, 512, 4096, 16384, },
  { 2, 8, 16, 32, 64, 512, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 4096, 16384, },
  { 4, 8, 16, 32, 64, 512, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 4096, 16384, },
  { 128, 512, 4096, 16384, },
  { 1, 128, 512, 4096, 16384, },
  { 2, 128, 512, 4096, 16384, },
  { 1, 2, 128, 512, 4096, 16384, },
  { 4, 128, 512, 4096, 16384, },
  { 1, 4, 128, 512, 4096, 16384, },
  { 2, 4, 128, 512, 4096, 16384, },
  { 1, 2, 4, 128, 512, 4096, 16384, },
  { 8, 128, 512, 4096, 16384, },
  { 1, 8, 128, 512, 4096, 16384, },
  { 2, 8, 128, 512, 4096, 16384, },
  { 1, 2, 8, 128, 512, 4096, 16384, },
  { 4, 8, 128, 512, 4096, 16384, },
  { 1, 4, 8, 128, 512, 4096, 16384, },
  { 2, 4, 8, 128, 512, 4096, 16384, },
  { 1, 2, 4, 8, 128, 512, 4096, 16384, },
  { 16, 128, 512, 4096, 16384, },
  { 1, 16, 128, 512, 4096, 16384, },
  { 2, 16, 128, 512, 4096, 16384, },
  { 1, 2, 16, 128, 512, 4096, 16384, },
  { 4, 16, 128, 512, 4096, 16384, },
  { 1, 4, 16, 128, 512, 4096, 16384, },
  { 2, 4, 16, 128, 512, 4096, 16384, },
  { 1, 2, 4, 16, 128, 512, 4096, 16384, },
  { 8, 16, 128, 512, 4096, 16384, },
  { 1, 8, 16, 128, 512, 4096, 16384, },
  { 2, 8, 16, 128, 512, 4096, 16384, },
  { 1, 2, 8, 16, 128, 512, 4096, 16384, },
  { 4, 8, 16, 128, 512, 4096, 16384, },
  { 1, 4, 8, 16, 128, 512, 4096, 16384, },
  { 2, 4, 8, 16, 128, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 4096, 16384, },
  { 32, 128, 512, 4096, 16384, },
  { 1, 32, 128, 512, 4096, 16384, },
  { 2, 32, 128, 512, 4096, 16384, },
  { 1, 2, 32, 128, 512, 4096, 16384, },
  { 4, 32, 128, 512, 4096, 16384, },
  { 1, 4, 32, 128, 512, 4096, 16384, },
  { 2, 4, 32, 128, 512, 4096, 16384, },
  { 1, 2, 4, 32, 128, 512, 4096, 16384, },
  { 8, 32, 128, 512, 4096, 16384, },
  { 1, 8, 32, 128, 512, 4096, 16384, },
  { 2, 8, 32, 128, 512, 4096, 16384, },
  { 1, 2, 8, 32, 128, 512, 4096, 16384, },
  { 4, 8, 32, 128, 512, 4096, 16384, },
  { 1, 4, 8, 32, 128, 512, 4096, 16384, },
  { 2, 4, 8, 32, 128, 512, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 4096, 16384, },
  { 16, 32, 128, 512, 4096, 16384, },
  { 1, 16, 32, 128, 512, 4096, 16384, },
  { 2, 16, 32, 128, 512, 4096, 16384, },
  { 1, 2, 16, 32, 128, 512, 4096, 16384, },
  { 4, 16, 32, 128, 512, 4096, 16384, },
  { 1, 4, 16, 32, 128, 512, 4096, 16384, },
  { 2, 4, 16, 32, 128, 512, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 4096, 16384, },
  { 8, 16, 32, 128, 512, 4096, 16384, },
  { 1, 8, 16, 32, 128, 512, 4096, 16384, },
  { 2, 8, 16, 32, 128, 512, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 4096, 16384, },
  { 4, 8, 16, 32, 128, 512, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 4096, 16384, },
  { 64, 128, 512, 4096, 16384, },
  { 1, 64, 128, 512, 4096, 16384, },
  { 2, 64, 128, 512, 4096, 16384, },
  { 1, 2, 64, 128, 512, 4096, 16384, },
  { 4, 64, 128, 512, 4096, 16384, },
  { 1, 4, 64, 128, 512, 4096, 16384, },
  { 2, 4, 64, 128, 512, 4096, 16384, },
  { 1, 2, 4, 64, 128, 512, 4096, 16384, },
  { 8, 64, 128, 512, 4096, 16384, },
  { 1, 8, 64, 128, 512, 4096, 16384, },
  { 2, 8, 64, 128, 512, 4096, 16384, },
  { 1, 2, 8, 64, 128, 512, 4096, 16384, },
  { 4, 8, 64, 128, 512, 4096, 16384, },
  { 1, 4, 8, 64, 128, 512, 4096, 16384, },
  { 2, 4, 8, 64, 128, 512, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 4096, 16384, },
  { 16, 64, 128, 512, 4096, 16384, },
  { 1, 16, 64, 128, 512, 4096, 16384, },
  { 2, 16, 64, 128, 512, 4096, 16384, },
  { 1, 2, 16, 64, 128, 512, 4096, 16384, },
  { 4, 16, 64, 128, 512, 4096, 16384, },
  { 1, 4, 16, 64, 128, 512, 4096, 16384, },
  { 2, 4, 16, 64, 128, 512, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 4096, 16384, },
  { 8, 16, 64, 128, 512, 4096, 16384, },
  { 1, 8, 16, 64, 128, 512, 4096, 16384, },
  { 2, 8, 16, 64, 128, 512, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 4096, 16384, },
  { 4, 8, 16, 64, 128, 512, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 4096, 16384, },
  { 32, 64, 128, 512, 4096, 16384, },
  { 1, 32, 64, 128, 512, 4096, 16384, },
  { 2, 32, 64, 128, 512, 4096, 16384, },
  { 1, 2, 32, 64, 128, 512, 4096, 16384, },
  { 4, 32, 64, 128, 512, 4096, 16384, },
  { 1, 4, 32, 64, 128, 512, 4096, 16384, },
  { 2, 4, 32, 64, 128, 512, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 4096, 16384, },
  { 8, 32, 64, 128, 512, 4096, 16384, },
  { 1, 8, 32, 64, 128, 512, 4096, 16384, },
  { 2, 8, 32, 64, 128, 512, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 4096, 16384, },
  { 4, 8, 32, 64, 128, 512, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 4096, 16384, },
  { 16, 32, 64, 128, 512, 4096, 16384, },
  { 1, 16, 32, 64, 128, 512, 4096, 16384, },
  { 2, 16, 32, 64, 128, 512, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 4096, 16384, },
  { 4, 16, 32, 64, 128, 512, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 4096, 16384, },
  { 8, 16, 32, 64, 128, 512, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 4096, 16384, },
  { 256, 512, 4096, 16384, },
  { 1, 256, 512, 4096, 16384, },
  { 2, 256, 512, 4096, 16384, },
  { 1, 2, 256, 512, 4096, 16384, },
  { 4, 256, 512, 4096, 16384, },
  { 1, 4, 256, 512, 4096, 16384, },
  { 2, 4, 256, 512, 4096, 16384, },
  { 1, 2, 4, 256, 512, 4096, 16384, },
  { 8, 256, 512, 4096, 16384, },
  { 1, 8, 256, 512, 4096, 16384, },
  { 2, 8, 256, 512, 4096, 16384, },
  { 1, 2, 8, 256, 512, 4096, 16384, },
  { 4, 8, 256, 512, 4096, 16384, },
  { 1, 4, 8, 256, 512, 4096, 16384, },
  { 2, 4, 8, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 256, 512, 4096, 16384, },
  { 16, 256, 512, 4096, 16384, },
  { 1, 16, 256, 512, 4096, 16384, },
  { 2, 16, 256, 512, 4096, 16384, },
  { 1, 2, 16, 256, 512, 4096, 16384, },
  { 4, 16, 256, 512, 4096, 16384, },
  { 1, 4, 16, 256, 512, 4096, 16384, },
  { 2, 4, 16, 256, 512, 4096, 16384, },
  { 1, 2, 4, 16, 256, 512, 4096, 16384, },
  { 8, 16, 256, 512, 4096, 16384, },
  { 1, 8, 16, 256, 512, 4096, 16384, },
  { 2, 8, 16, 256, 512, 4096, 16384, },
  { 1, 2, 8, 16, 256, 512, 4096, 16384, },
  { 4, 8, 16, 256, 512, 4096, 16384, },
  { 1, 4, 8, 16, 256, 512, 4096, 16384, },
  { 2, 4, 8, 16, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 4096, 16384, },
  { 32, 256, 512, 4096, 16384, },
  { 1, 32, 256, 512, 4096, 16384, },
  { 2, 32, 256, 512, 4096, 16384, },
  { 1, 2, 32, 256, 512, 4096, 16384, },
  { 4, 32, 256, 512, 4096, 16384, },
  { 1, 4, 32, 256, 512, 4096, 16384, },
  { 2, 4, 32, 256, 512, 4096, 16384, },
  { 1, 2, 4, 32, 256, 512, 4096, 16384, },
  { 8, 32, 256, 512, 4096, 16384, },
  { 1, 8, 32, 256, 512, 4096, 16384, },
  { 2, 8, 32, 256, 512, 4096, 16384, },
  { 1, 2, 8, 32, 256, 512, 4096, 16384, },
  { 4, 8, 32, 256, 512, 4096, 16384, },
  { 1, 4, 8, 32, 256, 512, 4096, 16384, },
  { 2, 4, 8, 32, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 4096, 16384, },
  { 16, 32, 256, 512, 4096, 16384, },
  { 1, 16, 32, 256, 512, 4096, 16384, },
  { 2, 16, 32, 256, 512, 4096, 16384, },
  { 1, 2, 16, 32, 256, 512, 4096, 16384, },
  { 4, 16, 32, 256, 512, 4096, 16384, },
  { 1, 4, 16, 32, 256, 512, 4096, 16384, },
  { 2, 4, 16, 32, 256, 512, 4096, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 4096, 16384, },
  { 8, 16, 32, 256, 512, 4096, 16384, },
  { 1, 8, 16, 32, 256, 512, 4096, 16384, },
  { 2, 8, 16, 32, 256, 512, 4096, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 4096, 16384, },
  { 4, 8, 16, 32, 256, 512, 4096, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 4096, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 4096, 16384, },
  { 64, 256, 512, 4096, 16384, },
  { 1, 64, 256, 512, 4096, 16384, },
  { 2, 64, 256, 512, 4096, 16384, },
  { 1, 2, 64, 256, 512, 4096, 16384, },
  { 4, 64, 256, 512, 4096, 16384, },
  { 1, 4, 64, 256, 512, 4096, 16384, },
  { 2, 4, 64, 256, 512, 4096, 16384, },
  { 1, 2, 4, 64, 256, 512, 4096, 16384, },
  { 8, 64, 256, 512, 4096, 16384, },
  { 1, 8, 64, 256, 512, 4096, 16384, },
  { 2, 8, 64, 256, 512, 4096, 16384, },
  { 1, 2, 8, 64, 256, 512, 4096, 16384, },
  { 4, 8, 64, 256, 512, 4096, 16384, },
  { 1, 4, 8, 64, 256, 512, 4096, 16384, },
  { 2, 4, 8, 64, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 4096, 16384, },
  { 16, 64, 256, 512, 4096, 16384, },
  { 1, 16, 64, 256, 512, 4096, 16384, },
  { 2, 16, 64, 256, 512, 4096, 16384, },
  { 1, 2, 16, 64, 256, 512, 4096, 16384, },
  { 4, 16, 64, 256, 512, 4096, 16384, },
  { 1, 4, 16, 64, 256, 512, 4096, 16384, },
  { 2, 4, 16, 64, 256, 512, 4096, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 4096, 16384, },
  { 8, 16, 64, 256, 512, 4096, 16384, },
  { 1, 8, 16, 64, 256, 512, 4096, 16384, },
  { 2, 8, 16, 64, 256, 512, 4096, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 4096, 16384, },
  { 4, 8, 16, 64, 256, 512, 4096, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 4096, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 4096, 16384, },
  { 32, 64, 256, 512, 4096, 16384, },
  { 1, 32, 64, 256, 512, 4096, 16384, },
  { 2, 32, 64, 256, 512, 4096, 16384, },
  { 1, 2, 32, 64, 256, 512, 4096, 16384, },
  { 4, 32, 64, 256, 512, 4096, 16384, },
  { 1, 4, 32, 64, 256, 512, 4096, 16384, },
  { 2, 4, 32, 64, 256, 512, 4096, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 4096, 16384, },
  { 8, 32, 64, 256, 512, 4096, 16384, },
  { 1, 8, 32, 64, 256, 512, 4096, 16384, },
  { 2, 8, 32, 64, 256, 512, 4096, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 4096, 16384, },
  { 4, 8, 32, 64, 256, 512, 4096, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 4096, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 4096, 16384, },
  { 16, 32, 64, 256, 512, 4096, 16384, },
  { 1, 16, 32, 64, 256, 512, 4096, 16384, },
  { 2, 16, 32, 64, 256, 512, 4096, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 4096, 16384, },
  { 4, 16, 32, 64, 256, 512, 4096, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 4096, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 4096, 16384, },
  { 8, 16, 32, 64, 256, 512, 4096, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 4096, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 4096, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 4096, 16384, },
  { 128, 256, 512, 4096, 16384, },
  { 1, 128, 256, 512, 4096, 16384, },
  { 2, 128, 256, 512, 4096, 16384, },
  { 1, 2, 128, 256, 512, 4096, 16384, },
  { 4, 128, 256, 512, 4096, 16384, },
  { 1, 4, 128, 256, 512, 4096, 16384, },
  { 2, 4, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 128, 256, 512, 4096, 16384, },
  { 8, 128, 256, 512, 4096, 16384, },
  { 1, 8, 128, 256, 512, 4096, 16384, },
  { 2, 8, 128, 256, 512, 4096, 16384, },
  { 1, 2, 8, 128, 256, 512, 4096, 16384, },
  { 4, 8, 128, 256, 512, 4096, 16384, },
  { 1, 4, 8, 128, 256, 512, 4096, 16384, },
  { 2, 4, 8, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 4096, 16384, },
  { 16, 128, 256, 512, 4096, 16384, },
  { 1, 16, 128, 256, 512, 4096, 16384, },
  { 2, 16, 128, 256, 512, 4096, 16384, },
  { 1, 2, 16, 128, 256, 512, 4096, 16384, },
  { 4, 16, 128, 256, 512, 4096, 16384, },
  { 1, 4, 16, 128, 256, 512, 4096, 16384, },
  { 2, 4, 16, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 4096, 16384, },
  { 8, 16, 128, 256, 512, 4096, 16384, },
  { 1, 8, 16, 128, 256, 512, 4096, 16384, },
  { 2, 8, 16, 128, 256, 512, 4096, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 4096, 16384, },
  { 4, 8, 16, 128, 256, 512, 4096, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 4096, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 4096, 16384, },
  { 32, 128, 256, 512, 4096, 16384, },
  { 1, 32, 128, 256, 512, 4096, 16384, },
  { 2, 32, 128, 256, 512, 4096, 16384, },
  { 1, 2, 32, 128, 256, 512, 4096, 16384, },
  { 4, 32, 128, 256, 512, 4096, 16384, },
  { 1, 4, 32, 128, 256, 512, 4096, 16384, },
  { 2, 4, 32, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 4096, 16384, },
  { 8, 32, 128, 256, 512, 4096, 16384, },
  { 1, 8, 32, 128, 256, 512, 4096, 16384, },
  { 2, 8, 32, 128, 256, 512, 4096, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 4096, 16384, },
  { 4, 8, 32, 128, 256, 512, 4096, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 4096, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 4096, 16384, },
  { 16, 32, 128, 256, 512, 4096, 16384, },
  { 1, 16, 32, 128, 256, 512, 4096, 16384, },
  { 2, 16, 32, 128, 256, 512, 4096, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 4096, 16384, },
  { 4, 16, 32, 128, 256, 512, 4096, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 4096, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 4096, 16384, },
  { 8, 16, 32, 128, 256, 512, 4096, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 4096, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 4096, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 4096, 16384, },
  { 64, 128, 256, 512, 4096, 16384, },
  { 1, 64, 128, 256, 512, 4096, 16384, },
  { 2, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 64, 128, 256, 512, 4096, 16384, },
  { 4, 64, 128, 256, 512, 4096, 16384, },
  { 1, 4, 64, 128, 256, 512, 4096, 16384, },
  { 2, 4, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 4096, 16384, },
  { 8, 64, 128, 256, 512, 4096, 16384, },
  { 1, 8, 64, 128, 256, 512, 4096, 16384, },
  { 2, 8, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 4096, 16384, },
  { 4, 8, 64, 128, 256, 512, 4096, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 4096, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 4096, 16384, },
  { 16, 64, 128, 256, 512, 4096, 16384, },
  { 1, 16, 64, 128, 256, 512, 4096, 16384, },
  { 2, 16, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 4096, 16384, },
  { 4, 16, 64, 128, 256, 512, 4096, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 4096, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 4096, 16384, },
  { 8, 16, 64, 128, 256, 512, 4096, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 4096, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 4096, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 4096, 16384, },
  { 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 32, 64, 128, 256, 512, 4096, 16384, },
  { 2, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 4096, 16384, },
  { 4, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 4096, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 4096, 16384, },
  { 8, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 4096, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 4096, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 4096, 16384, },
  { 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, },
  { 1024, 4096, 16384, },
  { 1, 1024, 4096, 16384, },
  { 2, 1024, 4096, 16384, },
  { 1, 2, 1024, 4096, 16384, },
  { 4, 1024, 4096, 16384, },
  { 1, 4, 1024, 4096, 16384, },
  { 2, 4, 1024, 4096, 16384, },
  { 1, 2, 4, 1024, 4096, 16384, },
  { 8, 1024, 4096, 16384, },
  { 1, 8, 1024, 4096, 16384, },
  { 2, 8, 1024, 4096, 16384, },
  { 1, 2, 8, 1024, 4096, 16384, },
  { 4, 8, 1024, 4096, 16384, },
  { 1, 4, 8, 1024, 4096, 16384, },
  { 2, 4, 8, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 1024, 4096, 16384, },
  { 16, 1024, 4096, 16384, },
  { 1, 16, 1024, 4096, 16384, },
  { 2, 16, 1024, 4096, 16384, },
  { 1, 2, 16, 1024, 4096, 16384, },
  { 4, 16, 1024, 4096, 16384, },
  { 1, 4, 16, 1024, 4096, 16384, },
  { 2, 4, 16, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 1024, 4096, 16384, },
  { 8, 16, 1024, 4096, 16384, },
  { 1, 8, 16, 1024, 4096, 16384, },
  { 2, 8, 16, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 1024, 4096, 16384, },
  { 4, 8, 16, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 1024, 4096, 16384, },
  { 32, 1024, 4096, 16384, },
  { 1, 32, 1024, 4096, 16384, },
  { 2, 32, 1024, 4096, 16384, },
  { 1, 2, 32, 1024, 4096, 16384, },
  { 4, 32, 1024, 4096, 16384, },
  { 1, 4, 32, 1024, 4096, 16384, },
  { 2, 4, 32, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 1024, 4096, 16384, },
  { 8, 32, 1024, 4096, 16384, },
  { 1, 8, 32, 1024, 4096, 16384, },
  { 2, 8, 32, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 1024, 4096, 16384, },
  { 4, 8, 32, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 1024, 4096, 16384, },
  { 16, 32, 1024, 4096, 16384, },
  { 1, 16, 32, 1024, 4096, 16384, },
  { 2, 16, 32, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 1024, 4096, 16384, },
  { 4, 16, 32, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 1024, 4096, 16384, },
  { 8, 16, 32, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 1024, 4096, 16384, },
  { 64, 1024, 4096, 16384, },
  { 1, 64, 1024, 4096, 16384, },
  { 2, 64, 1024, 4096, 16384, },
  { 1, 2, 64, 1024, 4096, 16384, },
  { 4, 64, 1024, 4096, 16384, },
  { 1, 4, 64, 1024, 4096, 16384, },
  { 2, 4, 64, 1024, 4096, 16384, },
  { 1, 2, 4, 64, 1024, 4096, 16384, },
  { 8, 64, 1024, 4096, 16384, },
  { 1, 8, 64, 1024, 4096, 16384, },
  { 2, 8, 64, 1024, 4096, 16384, },
  { 1, 2, 8, 64, 1024, 4096, 16384, },
  { 4, 8, 64, 1024, 4096, 16384, },
  { 1, 4, 8, 64, 1024, 4096, 16384, },
  { 2, 4, 8, 64, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 64, 1024, 4096, 16384, },
  { 16, 64, 1024, 4096, 16384, },
  { 1, 16, 64, 1024, 4096, 16384, },
  { 2, 16, 64, 1024, 4096, 16384, },
  { 1, 2, 16, 64, 1024, 4096, 16384, },
  { 4, 16, 64, 1024, 4096, 16384, },
  { 1, 4, 16, 64, 1024, 4096, 16384, },
  { 2, 4, 16, 64, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 64, 1024, 4096, 16384, },
  { 8, 16, 64, 1024, 4096, 16384, },
  { 1, 8, 16, 64, 1024, 4096, 16384, },
  { 2, 8, 16, 64, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 64, 1024, 4096, 16384, },
  { 4, 8, 16, 64, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 64, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 64, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 1024, 4096, 16384, },
  { 32, 64, 1024, 4096, 16384, },
  { 1, 32, 64, 1024, 4096, 16384, },
  { 2, 32, 64, 1024, 4096, 16384, },
  { 1, 2, 32, 64, 1024, 4096, 16384, },
  { 4, 32, 64, 1024, 4096, 16384, },
  { 1, 4, 32, 64, 1024, 4096, 16384, },
  { 2, 4, 32, 64, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 64, 1024, 4096, 16384, },
  { 8, 32, 64, 1024, 4096, 16384, },
  { 1, 8, 32, 64, 1024, 4096, 16384, },
  { 2, 8, 32, 64, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 64, 1024, 4096, 16384, },
  { 4, 8, 32, 64, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 64, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 64, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 1024, 4096, 16384, },
  { 16, 32, 64, 1024, 4096, 16384, },
  { 1, 16, 32, 64, 1024, 4096, 16384, },
  { 2, 16, 32, 64, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 64, 1024, 4096, 16384, },
  { 4, 16, 32, 64, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 64, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 64, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 1024, 4096, 16384, },
  { 8, 16, 32, 64, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 64, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 64, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 64, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 4096, 16384, },
  { 128, 1024, 4096, 16384, },
  { 1, 128, 1024, 4096, 16384, },
  { 2, 128, 1024, 4096, 16384, },
  { 1, 2, 128, 1024, 4096, 16384, },
  { 4, 128, 1024, 4096, 16384, },
  { 1, 4, 128, 1024, 4096, 16384, },
  { 2, 4, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 128, 1024, 4096, 16384, },
  { 8, 128, 1024, 4096, 16384, },
  { 1, 8, 128, 1024, 4096, 16384, },
  { 2, 8, 128, 1024, 4096, 16384, },
  { 1, 2, 8, 128, 1024, 4096, 16384, },
  { 4, 8, 128, 1024, 4096, 16384, },
  { 1, 4, 8, 128, 1024, 4096, 16384, },
  { 2, 4, 8, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 128, 1024, 4096, 16384, },
  { 16, 128, 1024, 4096, 16384, },
  { 1, 16, 128, 1024, 4096, 16384, },
  { 2, 16, 128, 1024, 4096, 16384, },
  { 1, 2, 16, 128, 1024, 4096, 16384, },
  { 4, 16, 128, 1024, 4096, 16384, },
  { 1, 4, 16, 128, 1024, 4096, 16384, },
  { 2, 4, 16, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 128, 1024, 4096, 16384, },
  { 8, 16, 128, 1024, 4096, 16384, },
  { 1, 8, 16, 128, 1024, 4096, 16384, },
  { 2, 8, 16, 128, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 128, 1024, 4096, 16384, },
  { 4, 8, 16, 128, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 128, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 1024, 4096, 16384, },
  { 32, 128, 1024, 4096, 16384, },
  { 1, 32, 128, 1024, 4096, 16384, },
  { 2, 32, 128, 1024, 4096, 16384, },
  { 1, 2, 32, 128, 1024, 4096, 16384, },
  { 4, 32, 128, 1024, 4096, 16384, },
  { 1, 4, 32, 128, 1024, 4096, 16384, },
  { 2, 4, 32, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 128, 1024, 4096, 16384, },
  { 8, 32, 128, 1024, 4096, 16384, },
  { 1, 8, 32, 128, 1024, 4096, 16384, },
  { 2, 8, 32, 128, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 128, 1024, 4096, 16384, },
  { 4, 8, 32, 128, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 128, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 1024, 4096, 16384, },
  { 16, 32, 128, 1024, 4096, 16384, },
  { 1, 16, 32, 128, 1024, 4096, 16384, },
  { 2, 16, 32, 128, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 128, 1024, 4096, 16384, },
  { 4, 16, 32, 128, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 128, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 1024, 4096, 16384, },
  { 8, 16, 32, 128, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 128, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 128, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 128, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 4096, 16384, },
  { 64, 128, 1024, 4096, 16384, },
  { 1, 64, 128, 1024, 4096, 16384, },
  { 2, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 64, 128, 1024, 4096, 16384, },
  { 4, 64, 128, 1024, 4096, 16384, },
  { 1, 4, 64, 128, 1024, 4096, 16384, },
  { 2, 4, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 64, 128, 1024, 4096, 16384, },
  { 8, 64, 128, 1024, 4096, 16384, },
  { 1, 8, 64, 128, 1024, 4096, 16384, },
  { 2, 8, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 8, 64, 128, 1024, 4096, 16384, },
  { 4, 8, 64, 128, 1024, 4096, 16384, },
  { 1, 4, 8, 64, 128, 1024, 4096, 16384, },
  { 2, 4, 8, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 1024, 4096, 16384, },
  { 16, 64, 128, 1024, 4096, 16384, },
  { 1, 16, 64, 128, 1024, 4096, 16384, },
  { 2, 16, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 16, 64, 128, 1024, 4096, 16384, },
  { 4, 16, 64, 128, 1024, 4096, 16384, },
  { 1, 4, 16, 64, 128, 1024, 4096, 16384, },
  { 2, 4, 16, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 1024, 4096, 16384, },
  { 8, 16, 64, 128, 1024, 4096, 16384, },
  { 1, 8, 16, 64, 128, 1024, 4096, 16384, },
  { 2, 8, 16, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 1024, 4096, 16384, },
  { 4, 8, 16, 64, 128, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 4096, 16384, },
  { 32, 64, 128, 1024, 4096, 16384, },
  { 1, 32, 64, 128, 1024, 4096, 16384, },
  { 2, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 32, 64, 128, 1024, 4096, 16384, },
  { 4, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 4, 32, 64, 128, 1024, 4096, 16384, },
  { 2, 4, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 1024, 4096, 16384, },
  { 8, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 8, 32, 64, 128, 1024, 4096, 16384, },
  { 2, 8, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 1024, 4096, 16384, },
  { 4, 8, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 4096, 16384, },
  { 16, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 2, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 4, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 8, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 16384, },
  { 256, 1024, 4096, 16384, },
  { 1, 256, 1024, 4096, 16384, },
  { 2, 256, 1024, 4096, 16384, },
  { 1, 2, 256, 1024, 4096, 16384, },
  { 4, 256, 1024, 4096, 16384, },
  { 1, 4, 256, 1024, 4096, 16384, },
  { 2, 4, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 256, 1024, 4096, 16384, },
  { 8, 256, 1024, 4096, 16384, },
  { 1, 8, 256, 1024, 4096, 16384, },
  { 2, 8, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 256, 1024, 4096, 16384, },
  { 4, 8, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 256, 1024, 4096, 16384, },
  { 16, 256, 1024, 4096, 16384, },
  { 1, 16, 256, 1024, 4096, 16384, },
  { 2, 16, 256, 1024, 4096, 16384, },
  { 1, 2, 16, 256, 1024, 4096, 16384, },
  { 4, 16, 256, 1024, 4096, 16384, },
  { 1, 4, 16, 256, 1024, 4096, 16384, },
  { 2, 4, 16, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 256, 1024, 4096, 16384, },
  { 8, 16, 256, 1024, 4096, 16384, },
  { 1, 8, 16, 256, 1024, 4096, 16384, },
  { 2, 8, 16, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 256, 1024, 4096, 16384, },
  { 4, 8, 16, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 256, 1024, 4096, 16384, },
  { 32, 256, 1024, 4096, 16384, },
  { 1, 32, 256, 1024, 4096, 16384, },
  { 2, 32, 256, 1024, 4096, 16384, },
  { 1, 2, 32, 256, 1024, 4096, 16384, },
  { 4, 32, 256, 1024, 4096, 16384, },
  { 1, 4, 32, 256, 1024, 4096, 16384, },
  { 2, 4, 32, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 256, 1024, 4096, 16384, },
  { 8, 32, 256, 1024, 4096, 16384, },
  { 1, 8, 32, 256, 1024, 4096, 16384, },
  { 2, 8, 32, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 256, 1024, 4096, 16384, },
  { 4, 8, 32, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 256, 1024, 4096, 16384, },
  { 16, 32, 256, 1024, 4096, 16384, },
  { 1, 16, 32, 256, 1024, 4096, 16384, },
  { 2, 16, 32, 256, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 256, 1024, 4096, 16384, },
  { 4, 16, 32, 256, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 256, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 256, 1024, 4096, 16384, },
  { 8, 16, 32, 256, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 256, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 256, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 4096, 16384, },
  { 64, 256, 1024, 4096, 16384, },
  { 1, 64, 256, 1024, 4096, 16384, },
  { 2, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 64, 256, 1024, 4096, 16384, },
  { 4, 64, 256, 1024, 4096, 16384, },
  { 1, 4, 64, 256, 1024, 4096, 16384, },
  { 2, 4, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 64, 256, 1024, 4096, 16384, },
  { 8, 64, 256, 1024, 4096, 16384, },
  { 1, 8, 64, 256, 1024, 4096, 16384, },
  { 2, 8, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 64, 256, 1024, 4096, 16384, },
  { 4, 8, 64, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 64, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 64, 256, 1024, 4096, 16384, },
  { 16, 64, 256, 1024, 4096, 16384, },
  { 1, 16, 64, 256, 1024, 4096, 16384, },
  { 2, 16, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 16, 64, 256, 1024, 4096, 16384, },
  { 4, 16, 64, 256, 1024, 4096, 16384, },
  { 1, 4, 16, 64, 256, 1024, 4096, 16384, },
  { 2, 4, 16, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 64, 256, 1024, 4096, 16384, },
  { 8, 16, 64, 256, 1024, 4096, 16384, },
  { 1, 8, 16, 64, 256, 1024, 4096, 16384, },
  { 2, 8, 16, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 64, 256, 1024, 4096, 16384, },
  { 4, 8, 16, 64, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 64, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 4096, 16384, },
  { 32, 64, 256, 1024, 4096, 16384, },
  { 1, 32, 64, 256, 1024, 4096, 16384, },
  { 2, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 32, 64, 256, 1024, 4096, 16384, },
  { 4, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 4, 32, 64, 256, 1024, 4096, 16384, },
  { 2, 4, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 64, 256, 1024, 4096, 16384, },
  { 8, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 8, 32, 64, 256, 1024, 4096, 16384, },
  { 2, 8, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 64, 256, 1024, 4096, 16384, },
  { 4, 8, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 64, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 4096, 16384, },
  { 16, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 2, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 4, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 8, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 16384, },
  { 128, 256, 1024, 4096, 16384, },
  { 1, 128, 256, 1024, 4096, 16384, },
  { 2, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 128, 256, 1024, 4096, 16384, },
  { 4, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 128, 256, 1024, 4096, 16384, },
  { 8, 128, 256, 1024, 4096, 16384, },
  { 1, 8, 128, 256, 1024, 4096, 16384, },
  { 2, 8, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 128, 256, 1024, 4096, 16384, },
  { 4, 8, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 128, 256, 1024, 4096, 16384, },
  { 16, 128, 256, 1024, 4096, 16384, },
  { 1, 16, 128, 256, 1024, 4096, 16384, },
  { 2, 16, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 16, 128, 256, 1024, 4096, 16384, },
  { 4, 16, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 16, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 16, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 128, 256, 1024, 4096, 16384, },
  { 8, 16, 128, 256, 1024, 4096, 16384, },
  { 1, 8, 16, 128, 256, 1024, 4096, 16384, },
  { 2, 8, 16, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 128, 256, 1024, 4096, 16384, },
  { 4, 8, 16, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 4096, 16384, },
  { 32, 128, 256, 1024, 4096, 16384, },
  { 1, 32, 128, 256, 1024, 4096, 16384, },
  { 2, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 32, 128, 256, 1024, 4096, 16384, },
  { 4, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 32, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 128, 256, 1024, 4096, 16384, },
  { 8, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 8, 32, 128, 256, 1024, 4096, 16384, },
  { 2, 8, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 128, 256, 1024, 4096, 16384, },
  { 4, 8, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 4096, 16384, },
  { 16, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 2, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 4, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 8, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 16384, },
  { 64, 128, 256, 1024, 4096, 16384, },
  { 1, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 64, 128, 256, 1024, 4096, 16384, },
  { 4, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 64, 128, 256, 1024, 4096, 16384, },
  { 8, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 8, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 8, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 64, 128, 256, 1024, 4096, 16384, },
  { 4, 8, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 4096, 16384, },
  { 16, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 4, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 8, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 8, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 8, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 4, 8, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 16384, },
  { 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 4, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 8, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 8, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 8, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 4, 8, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 4, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, },
  { 512, 1024, 4096, 16384, },
  { 1, 512, 1024, 4096, 16384, },
  { 2, 512, 1024, 4096, 16384, },
  { 1, 2, 512, 1024, 4096, 16384, },
  { 4, 512, 1024, 4096, 16384, },
  { 1, 4, 512, 1024, 4096, 16384, },
  { 2, 4, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 512, 1024, 4096, 16384, },
  { 8, 512, 1024, 4096, 16384, },
  { 1, 8, 512, 1024, 4096, 16384, },
  { 2, 8, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 512, 1024, 4096, 16384, },
  { 4, 8, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 512, 1024, 4096, 16384, },
  { 16, 512, 1024, 4096, 16384, },
  { 1, 16, 512, 1024, 4096, 16384, },
  { 2, 16, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 512, 1024, 4096, 16384, },
  { 4, 16, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 512, 1024, 4096, 16384, },
  { 8, 16, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 512, 1024, 4096, 16384, },
  { 32, 512, 1024, 4096, 16384, },
  { 1, 32, 512, 1024, 4096, 16384, },
  { 2, 32, 512, 1024, 4096, 16384, },
  { 1, 2, 32, 512, 1024, 4096, 16384, },
  { 4, 32, 512, 1024, 4096, 16384, },
  { 1, 4, 32, 512, 1024, 4096, 16384, },
  { 2, 4, 32, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 512, 1024, 4096, 16384, },
  { 8, 32, 512, 1024, 4096, 16384, },
  { 1, 8, 32, 512, 1024, 4096, 16384, },
  { 2, 8, 32, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 512, 1024, 4096, 16384, },
  { 4, 8, 32, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 512, 1024, 4096, 16384, },
  { 16, 32, 512, 1024, 4096, 16384, },
  { 1, 16, 32, 512, 1024, 4096, 16384, },
  { 2, 16, 32, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 512, 1024, 4096, 16384, },
  { 4, 16, 32, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 512, 1024, 4096, 16384, },
  { 8, 16, 32, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 4096, 16384, },
  { 64, 512, 1024, 4096, 16384, },
  { 1, 64, 512, 1024, 4096, 16384, },
  { 2, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 64, 512, 1024, 4096, 16384, },
  { 4, 64, 512, 1024, 4096, 16384, },
  { 1, 4, 64, 512, 1024, 4096, 16384, },
  { 2, 4, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 64, 512, 1024, 4096, 16384, },
  { 8, 64, 512, 1024, 4096, 16384, },
  { 1, 8, 64, 512, 1024, 4096, 16384, },
  { 2, 8, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 64, 512, 1024, 4096, 16384, },
  { 4, 8, 64, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 64, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 64, 512, 1024, 4096, 16384, },
  { 16, 64, 512, 1024, 4096, 16384, },
  { 1, 16, 64, 512, 1024, 4096, 16384, },
  { 2, 16, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 64, 512, 1024, 4096, 16384, },
  { 4, 16, 64, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 64, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 64, 512, 1024, 4096, 16384, },
  { 8, 16, 64, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 64, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 64, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 64, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 64, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 4096, 16384, },
  { 32, 64, 512, 1024, 4096, 16384, },
  { 1, 32, 64, 512, 1024, 4096, 16384, },
  { 2, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 32, 64, 512, 1024, 4096, 16384, },
  { 4, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 4, 32, 64, 512, 1024, 4096, 16384, },
  { 2, 4, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 64, 512, 1024, 4096, 16384, },
  { 8, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 8, 32, 64, 512, 1024, 4096, 16384, },
  { 2, 8, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 64, 512, 1024, 4096, 16384, },
  { 4, 8, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 64, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 4096, 16384, },
  { 16, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 2, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 4, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 8, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 16384, },
  { 128, 512, 1024, 4096, 16384, },
  { 1, 128, 512, 1024, 4096, 16384, },
  { 2, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 128, 512, 1024, 4096, 16384, },
  { 4, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 128, 512, 1024, 4096, 16384, },
  { 8, 128, 512, 1024, 4096, 16384, },
  { 1, 8, 128, 512, 1024, 4096, 16384, },
  { 2, 8, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 128, 512, 1024, 4096, 16384, },
  { 4, 8, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 128, 512, 1024, 4096, 16384, },
  { 16, 128, 512, 1024, 4096, 16384, },
  { 1, 16, 128, 512, 1024, 4096, 16384, },
  { 2, 16, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 128, 512, 1024, 4096, 16384, },
  { 4, 16, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 128, 512, 1024, 4096, 16384, },
  { 8, 16, 128, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 128, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 128, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 4096, 16384, },
  { 32, 128, 512, 1024, 4096, 16384, },
  { 1, 32, 128, 512, 1024, 4096, 16384, },
  { 2, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 32, 128, 512, 1024, 4096, 16384, },
  { 4, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 32, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 128, 512, 1024, 4096, 16384, },
  { 8, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 8, 32, 128, 512, 1024, 4096, 16384, },
  { 2, 8, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 128, 512, 1024, 4096, 16384, },
  { 4, 8, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 4096, 16384, },
  { 16, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 2, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 4, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 8, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 16384, },
  { 64, 128, 512, 1024, 4096, 16384, },
  { 1, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 64, 128, 512, 1024, 4096, 16384, },
  { 4, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 64, 128, 512, 1024, 4096, 16384, },
  { 8, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 8, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 8, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 64, 128, 512, 1024, 4096, 16384, },
  { 4, 8, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 4096, 16384, },
  { 16, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 4, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 8, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 16384, },
  { 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 4, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 8, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 8, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 8, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 4, 8, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 4, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, },
  { 256, 512, 1024, 4096, 16384, },
  { 1, 256, 512, 1024, 4096, 16384, },
  { 2, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 256, 512, 1024, 4096, 16384, },
  { 4, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 256, 512, 1024, 4096, 16384, },
  { 8, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 256, 512, 1024, 4096, 16384, },
  { 16, 256, 512, 1024, 4096, 16384, },
  { 1, 16, 256, 512, 1024, 4096, 16384, },
  { 2, 16, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 256, 512, 1024, 4096, 16384, },
  { 4, 16, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 256, 512, 1024, 4096, 16384, },
  { 8, 16, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 4096, 16384, },
  { 32, 256, 512, 1024, 4096, 16384, },
  { 1, 32, 256, 512, 1024, 4096, 16384, },
  { 2, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 32, 256, 512, 1024, 4096, 16384, },
  { 4, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 32, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 256, 512, 1024, 4096, 16384, },
  { 8, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 32, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 4096, 16384, },
  { 16, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 2, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 4, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 8, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 16384, },
  { 64, 256, 512, 1024, 4096, 16384, },
  { 1, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 64, 256, 512, 1024, 4096, 16384, },
  { 4, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 64, 256, 512, 1024, 4096, 16384, },
  { 8, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 64, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 4096, 16384, },
  { 16, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 4, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 8, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 16384, },
  { 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 4, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 8, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 4, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, },
  { 128, 256, 512, 1024, 4096, 16384, },
  { 1, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 128, 256, 512, 1024, 4096, 16384, },
  { 8, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 4096, 16384, },
  { 16, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 8, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 16384, },
  { 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 8, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, },
  { 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 8, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, },
  { 2048, 4096, 16384, },
  { 1, 2048, 4096, 16384, },
  { 2, 2048, 4096, 16384, },
  { 1, 2, 2048, 4096, 16384, },
  { 4, 2048, 4096, 16384, },
  { 1, 4, 2048, 4096, 16384, },
  { 2, 4, 2048, 4096, 16384, },
  { 1, 2, 4, 2048, 4096, 16384, },
  { 8, 2048, 4096, 16384, },
  { 1, 8, 2048, 4096, 16384, },
  { 2, 8, 2048, 4096, 16384, },
  { 1, 2, 8, 2048, 4096, 16384, },
  { 4, 8, 2048, 4096, 16384, },
  { 1, 4, 8, 2048, 4096, 16384, },
  { 2, 4, 8, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 2048, 4096, 16384, },
  { 16, 2048, 4096, 16384, },
  { 1, 16, 2048, 4096, 16384, },
  { 2, 16, 2048, 4096, 16384, },
  { 1, 2, 16, 2048, 4096, 16384, },
  { 4, 16, 2048, 4096, 16384, },
  { 1, 4, 16, 2048, 4096, 16384, },
  { 2, 4, 16, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 2048, 4096, 16384, },
  { 8, 16, 2048, 4096, 16384, },
  { 1, 8, 16, 2048, 4096, 16384, },
  { 2, 8, 16, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 2048, 4096, 16384, },
  { 4, 8, 16, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 2048, 4096, 16384, },
  { 32, 2048, 4096, 16384, },
  { 1, 32, 2048, 4096, 16384, },
  { 2, 32, 2048, 4096, 16384, },
  { 1, 2, 32, 2048, 4096, 16384, },
  { 4, 32, 2048, 4096, 16384, },
  { 1, 4, 32, 2048, 4096, 16384, },
  { 2, 4, 32, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 2048, 4096, 16384, },
  { 8, 32, 2048, 4096, 16384, },
  { 1, 8, 32, 2048, 4096, 16384, },
  { 2, 8, 32, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 2048, 4096, 16384, },
  { 4, 8, 32, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 2048, 4096, 16384, },
  { 16, 32, 2048, 4096, 16384, },
  { 1, 16, 32, 2048, 4096, 16384, },
  { 2, 16, 32, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 2048, 4096, 16384, },
  { 4, 16, 32, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 2048, 4096, 16384, },
  { 8, 16, 32, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 2048, 4096, 16384, },
  { 64, 2048, 4096, 16384, },
  { 1, 64, 2048, 4096, 16384, },
  { 2, 64, 2048, 4096, 16384, },
  { 1, 2, 64, 2048, 4096, 16384, },
  { 4, 64, 2048, 4096, 16384, },
  { 1, 4, 64, 2048, 4096, 16384, },
  { 2, 4, 64, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 2048, 4096, 16384, },
  { 8, 64, 2048, 4096, 16384, },
  { 1, 8, 64, 2048, 4096, 16384, },
  { 2, 8, 64, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 2048, 4096, 16384, },
  { 4, 8, 64, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 2048, 4096, 16384, },
  { 16, 64, 2048, 4096, 16384, },
  { 1, 16, 64, 2048, 4096, 16384, },
  { 2, 16, 64, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 2048, 4096, 16384, },
  { 4, 16, 64, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 2048, 4096, 16384, },
  { 8, 16, 64, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 2048, 4096, 16384, },
  { 32, 64, 2048, 4096, 16384, },
  { 1, 32, 64, 2048, 4096, 16384, },
  { 2, 32, 64, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 2048, 4096, 16384, },
  { 4, 32, 64, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 2048, 4096, 16384, },
  { 8, 32, 64, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 2048, 4096, 16384, },
  { 16, 32, 64, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 4096, 16384, },
  { 128, 2048, 4096, 16384, },
  { 1, 128, 2048, 4096, 16384, },
  { 2, 128, 2048, 4096, 16384, },
  { 1, 2, 128, 2048, 4096, 16384, },
  { 4, 128, 2048, 4096, 16384, },
  { 1, 4, 128, 2048, 4096, 16384, },
  { 2, 4, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 128, 2048, 4096, 16384, },
  { 8, 128, 2048, 4096, 16384, },
  { 1, 8, 128, 2048, 4096, 16384, },
  { 2, 8, 128, 2048, 4096, 16384, },
  { 1, 2, 8, 128, 2048, 4096, 16384, },
  { 4, 8, 128, 2048, 4096, 16384, },
  { 1, 4, 8, 128, 2048, 4096, 16384, },
  { 2, 4, 8, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 128, 2048, 4096, 16384, },
  { 16, 128, 2048, 4096, 16384, },
  { 1, 16, 128, 2048, 4096, 16384, },
  { 2, 16, 128, 2048, 4096, 16384, },
  { 1, 2, 16, 128, 2048, 4096, 16384, },
  { 4, 16, 128, 2048, 4096, 16384, },
  { 1, 4, 16, 128, 2048, 4096, 16384, },
  { 2, 4, 16, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 128, 2048, 4096, 16384, },
  { 8, 16, 128, 2048, 4096, 16384, },
  { 1, 8, 16, 128, 2048, 4096, 16384, },
  { 2, 8, 16, 128, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 128, 2048, 4096, 16384, },
  { 4, 8, 16, 128, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 128, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 2048, 4096, 16384, },
  { 32, 128, 2048, 4096, 16384, },
  { 1, 32, 128, 2048, 4096, 16384, },
  { 2, 32, 128, 2048, 4096, 16384, },
  { 1, 2, 32, 128, 2048, 4096, 16384, },
  { 4, 32, 128, 2048, 4096, 16384, },
  { 1, 4, 32, 128, 2048, 4096, 16384, },
  { 2, 4, 32, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 128, 2048, 4096, 16384, },
  { 8, 32, 128, 2048, 4096, 16384, },
  { 1, 8, 32, 128, 2048, 4096, 16384, },
  { 2, 8, 32, 128, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 128, 2048, 4096, 16384, },
  { 4, 8, 32, 128, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 128, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 2048, 4096, 16384, },
  { 16, 32, 128, 2048, 4096, 16384, },
  { 1, 16, 32, 128, 2048, 4096, 16384, },
  { 2, 16, 32, 128, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 128, 2048, 4096, 16384, },
  { 4, 16, 32, 128, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 128, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 2048, 4096, 16384, },
  { 8, 16, 32, 128, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 128, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 128, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 128, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 4096, 16384, },
  { 64, 128, 2048, 4096, 16384, },
  { 1, 64, 128, 2048, 4096, 16384, },
  { 2, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 64, 128, 2048, 4096, 16384, },
  { 4, 64, 128, 2048, 4096, 16384, },
  { 1, 4, 64, 128, 2048, 4096, 16384, },
  { 2, 4, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 128, 2048, 4096, 16384, },
  { 8, 64, 128, 2048, 4096, 16384, },
  { 1, 8, 64, 128, 2048, 4096, 16384, },
  { 2, 8, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 128, 2048, 4096, 16384, },
  { 4, 8, 64, 128, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 128, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 2048, 4096, 16384, },
  { 16, 64, 128, 2048, 4096, 16384, },
  { 1, 16, 64, 128, 2048, 4096, 16384, },
  { 2, 16, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 128, 2048, 4096, 16384, },
  { 4, 16, 64, 128, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 128, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 2048, 4096, 16384, },
  { 8, 16, 64, 128, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 128, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 128, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 4096, 16384, },
  { 32, 64, 128, 2048, 4096, 16384, },
  { 1, 32, 64, 128, 2048, 4096, 16384, },
  { 2, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 128, 2048, 4096, 16384, },
  { 4, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 128, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 2048, 4096, 16384, },
  { 8, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 128, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 4096, 16384, },
  { 16, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 16384, },
  { 256, 2048, 4096, 16384, },
  { 1, 256, 2048, 4096, 16384, },
  { 2, 256, 2048, 4096, 16384, },
  { 1, 2, 256, 2048, 4096, 16384, },
  { 4, 256, 2048, 4096, 16384, },
  { 1, 4, 256, 2048, 4096, 16384, },
  { 2, 4, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 256, 2048, 4096, 16384, },
  { 8, 256, 2048, 4096, 16384, },
  { 1, 8, 256, 2048, 4096, 16384, },
  { 2, 8, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 256, 2048, 4096, 16384, },
  { 4, 8, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 256, 2048, 4096, 16384, },
  { 16, 256, 2048, 4096, 16384, },
  { 1, 16, 256, 2048, 4096, 16384, },
  { 2, 16, 256, 2048, 4096, 16384, },
  { 1, 2, 16, 256, 2048, 4096, 16384, },
  { 4, 16, 256, 2048, 4096, 16384, },
  { 1, 4, 16, 256, 2048, 4096, 16384, },
  { 2, 4, 16, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 256, 2048, 4096, 16384, },
  { 8, 16, 256, 2048, 4096, 16384, },
  { 1, 8, 16, 256, 2048, 4096, 16384, },
  { 2, 8, 16, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 256, 2048, 4096, 16384, },
  { 4, 8, 16, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 256, 2048, 4096, 16384, },
  { 32, 256, 2048, 4096, 16384, },
  { 1, 32, 256, 2048, 4096, 16384, },
  { 2, 32, 256, 2048, 4096, 16384, },
  { 1, 2, 32, 256, 2048, 4096, 16384, },
  { 4, 32, 256, 2048, 4096, 16384, },
  { 1, 4, 32, 256, 2048, 4096, 16384, },
  { 2, 4, 32, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 256, 2048, 4096, 16384, },
  { 8, 32, 256, 2048, 4096, 16384, },
  { 1, 8, 32, 256, 2048, 4096, 16384, },
  { 2, 8, 32, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 256, 2048, 4096, 16384, },
  { 4, 8, 32, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 256, 2048, 4096, 16384, },
  { 16, 32, 256, 2048, 4096, 16384, },
  { 1, 16, 32, 256, 2048, 4096, 16384, },
  { 2, 16, 32, 256, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 256, 2048, 4096, 16384, },
  { 4, 16, 32, 256, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 256, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 256, 2048, 4096, 16384, },
  { 8, 16, 32, 256, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 256, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 256, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 4096, 16384, },
  { 64, 256, 2048, 4096, 16384, },
  { 1, 64, 256, 2048, 4096, 16384, },
  { 2, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 64, 256, 2048, 4096, 16384, },
  { 4, 64, 256, 2048, 4096, 16384, },
  { 1, 4, 64, 256, 2048, 4096, 16384, },
  { 2, 4, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 256, 2048, 4096, 16384, },
  { 8, 64, 256, 2048, 4096, 16384, },
  { 1, 8, 64, 256, 2048, 4096, 16384, },
  { 2, 8, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 256, 2048, 4096, 16384, },
  { 4, 8, 64, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 256, 2048, 4096, 16384, },
  { 16, 64, 256, 2048, 4096, 16384, },
  { 1, 16, 64, 256, 2048, 4096, 16384, },
  { 2, 16, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 256, 2048, 4096, 16384, },
  { 4, 16, 64, 256, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 256, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 256, 2048, 4096, 16384, },
  { 8, 16, 64, 256, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 256, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 256, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 4096, 16384, },
  { 32, 64, 256, 2048, 4096, 16384, },
  { 1, 32, 64, 256, 2048, 4096, 16384, },
  { 2, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 256, 2048, 4096, 16384, },
  { 4, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 256, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 256, 2048, 4096, 16384, },
  { 8, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 256, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 256, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 4096, 16384, },
  { 16, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 16384, },
  { 128, 256, 2048, 4096, 16384, },
  { 1, 128, 256, 2048, 4096, 16384, },
  { 2, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 128, 256, 2048, 4096, 16384, },
  { 4, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 128, 256, 2048, 4096, 16384, },
  { 8, 128, 256, 2048, 4096, 16384, },
  { 1, 8, 128, 256, 2048, 4096, 16384, },
  { 2, 8, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 128, 256, 2048, 4096, 16384, },
  { 4, 8, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 128, 256, 2048, 4096, 16384, },
  { 16, 128, 256, 2048, 4096, 16384, },
  { 1, 16, 128, 256, 2048, 4096, 16384, },
  { 2, 16, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 16, 128, 256, 2048, 4096, 16384, },
  { 4, 16, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 16, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 16, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 128, 256, 2048, 4096, 16384, },
  { 8, 16, 128, 256, 2048, 4096, 16384, },
  { 1, 8, 16, 128, 256, 2048, 4096, 16384, },
  { 2, 8, 16, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 128, 256, 2048, 4096, 16384, },
  { 4, 8, 16, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 4096, 16384, },
  { 32, 128, 256, 2048, 4096, 16384, },
  { 1, 32, 128, 256, 2048, 4096, 16384, },
  { 2, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 32, 128, 256, 2048, 4096, 16384, },
  { 4, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 32, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 128, 256, 2048, 4096, 16384, },
  { 8, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 8, 32, 128, 256, 2048, 4096, 16384, },
  { 2, 8, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 128, 256, 2048, 4096, 16384, },
  { 4, 8, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 4096, 16384, },
  { 16, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 2, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 4, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 8, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 16384, },
  { 64, 128, 256, 2048, 4096, 16384, },
  { 1, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 64, 128, 256, 2048, 4096, 16384, },
  { 4, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 128, 256, 2048, 4096, 16384, },
  { 8, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 8, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 8, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 128, 256, 2048, 4096, 16384, },
  { 4, 8, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 4096, 16384, },
  { 16, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 4, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 8, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 16384, },
  { 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 4, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 8, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, },
  { 512, 2048, 4096, 16384, },
  { 1, 512, 2048, 4096, 16384, },
  { 2, 512, 2048, 4096, 16384, },
  { 1, 2, 512, 2048, 4096, 16384, },
  { 4, 512, 2048, 4096, 16384, },
  { 1, 4, 512, 2048, 4096, 16384, },
  { 2, 4, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 512, 2048, 4096, 16384, },
  { 8, 512, 2048, 4096, 16384, },
  { 1, 8, 512, 2048, 4096, 16384, },
  { 2, 8, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 512, 2048, 4096, 16384, },
  { 4, 8, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 512, 2048, 4096, 16384, },
  { 16, 512, 2048, 4096, 16384, },
  { 1, 16, 512, 2048, 4096, 16384, },
  { 2, 16, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 512, 2048, 4096, 16384, },
  { 4, 16, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 512, 2048, 4096, 16384, },
  { 8, 16, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 512, 2048, 4096, 16384, },
  { 32, 512, 2048, 4096, 16384, },
  { 1, 32, 512, 2048, 4096, 16384, },
  { 2, 32, 512, 2048, 4096, 16384, },
  { 1, 2, 32, 512, 2048, 4096, 16384, },
  { 4, 32, 512, 2048, 4096, 16384, },
  { 1, 4, 32, 512, 2048, 4096, 16384, },
  { 2, 4, 32, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 512, 2048, 4096, 16384, },
  { 8, 32, 512, 2048, 4096, 16384, },
  { 1, 8, 32, 512, 2048, 4096, 16384, },
  { 2, 8, 32, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 512, 2048, 4096, 16384, },
  { 4, 8, 32, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 512, 2048, 4096, 16384, },
  { 16, 32, 512, 2048, 4096, 16384, },
  { 1, 16, 32, 512, 2048, 4096, 16384, },
  { 2, 16, 32, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 512, 2048, 4096, 16384, },
  { 4, 16, 32, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 512, 2048, 4096, 16384, },
  { 8, 16, 32, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 4096, 16384, },
  { 64, 512, 2048, 4096, 16384, },
  { 1, 64, 512, 2048, 4096, 16384, },
  { 2, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 64, 512, 2048, 4096, 16384, },
  { 4, 64, 512, 2048, 4096, 16384, },
  { 1, 4, 64, 512, 2048, 4096, 16384, },
  { 2, 4, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 512, 2048, 4096, 16384, },
  { 8, 64, 512, 2048, 4096, 16384, },
  { 1, 8, 64, 512, 2048, 4096, 16384, },
  { 2, 8, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 512, 2048, 4096, 16384, },
  { 4, 8, 64, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 512, 2048, 4096, 16384, },
  { 16, 64, 512, 2048, 4096, 16384, },
  { 1, 16, 64, 512, 2048, 4096, 16384, },
  { 2, 16, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 512, 2048, 4096, 16384, },
  { 4, 16, 64, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 512, 2048, 4096, 16384, },
  { 8, 16, 64, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 4096, 16384, },
  { 32, 64, 512, 2048, 4096, 16384, },
  { 1, 32, 64, 512, 2048, 4096, 16384, },
  { 2, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 512, 2048, 4096, 16384, },
  { 4, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 512, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 512, 2048, 4096, 16384, },
  { 8, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 512, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 512, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 4096, 16384, },
  { 16, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 16384, },
  { 128, 512, 2048, 4096, 16384, },
  { 1, 128, 512, 2048, 4096, 16384, },
  { 2, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 128, 512, 2048, 4096, 16384, },
  { 4, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 128, 512, 2048, 4096, 16384, },
  { 8, 128, 512, 2048, 4096, 16384, },
  { 1, 8, 128, 512, 2048, 4096, 16384, },
  { 2, 8, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 128, 512, 2048, 4096, 16384, },
  { 4, 8, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 128, 512, 2048, 4096, 16384, },
  { 16, 128, 512, 2048, 4096, 16384, },
  { 1, 16, 128, 512, 2048, 4096, 16384, },
  { 2, 16, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 128, 512, 2048, 4096, 16384, },
  { 4, 16, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 128, 512, 2048, 4096, 16384, },
  { 8, 16, 128, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 128, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 128, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 4096, 16384, },
  { 32, 128, 512, 2048, 4096, 16384, },
  { 1, 32, 128, 512, 2048, 4096, 16384, },
  { 2, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 32, 128, 512, 2048, 4096, 16384, },
  { 4, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 32, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 128, 512, 2048, 4096, 16384, },
  { 8, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 8, 32, 128, 512, 2048, 4096, 16384, },
  { 2, 8, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 128, 512, 2048, 4096, 16384, },
  { 4, 8, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 4096, 16384, },
  { 16, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 2, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 4, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 8, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 16384, },
  { 64, 128, 512, 2048, 4096, 16384, },
  { 1, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 64, 128, 512, 2048, 4096, 16384, },
  { 4, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 128, 512, 2048, 4096, 16384, },
  { 8, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 8, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 8, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 128, 512, 2048, 4096, 16384, },
  { 4, 8, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 4096, 16384, },
  { 16, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 4, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 8, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 16384, },
  { 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 4, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 8, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, },
  { 256, 512, 2048, 4096, 16384, },
  { 1, 256, 512, 2048, 4096, 16384, },
  { 2, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 256, 512, 2048, 4096, 16384, },
  { 4, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 256, 512, 2048, 4096, 16384, },
  { 8, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 256, 512, 2048, 4096, 16384, },
  { 16, 256, 512, 2048, 4096, 16384, },
  { 1, 16, 256, 512, 2048, 4096, 16384, },
  { 2, 16, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 256, 512, 2048, 4096, 16384, },
  { 4, 16, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 256, 512, 2048, 4096, 16384, },
  { 8, 16, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 4096, 16384, },
  { 32, 256, 512, 2048, 4096, 16384, },
  { 1, 32, 256, 512, 2048, 4096, 16384, },
  { 2, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 32, 256, 512, 2048, 4096, 16384, },
  { 4, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 32, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 256, 512, 2048, 4096, 16384, },
  { 8, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 32, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 4096, 16384, },
  { 16, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 2, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 4, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 8, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 16384, },
  { 64, 256, 512, 2048, 4096, 16384, },
  { 1, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 64, 256, 512, 2048, 4096, 16384, },
  { 4, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 256, 512, 2048, 4096, 16384, },
  { 8, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 4096, 16384, },
  { 16, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 4, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 8, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 16384, },
  { 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 4, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 8, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, },
  { 128, 256, 512, 2048, 4096, 16384, },
  { 1, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 128, 256, 512, 2048, 4096, 16384, },
  { 8, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 4096, 16384, },
  { 16, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 8, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 16384, },
  { 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 8, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, },
  { 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 8, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, },
  { 1024, 2048, 4096, 16384, },
  { 1, 1024, 2048, 4096, 16384, },
  { 2, 1024, 2048, 4096, 16384, },
  { 1, 2, 1024, 2048, 4096, 16384, },
  { 4, 1024, 2048, 4096, 16384, },
  { 1, 4, 1024, 2048, 4096, 16384, },
  { 2, 4, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 1024, 2048, 4096, 16384, },
  { 8, 1024, 2048, 4096, 16384, },
  { 1, 8, 1024, 2048, 4096, 16384, },
  { 2, 8, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 1024, 2048, 4096, 16384, },
  { 4, 8, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 1024, 2048, 4096, 16384, },
  { 16, 1024, 2048, 4096, 16384, },
  { 1, 16, 1024, 2048, 4096, 16384, },
  { 2, 16, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 1024, 2048, 4096, 16384, },
  { 4, 16, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 1024, 2048, 4096, 16384, },
  { 8, 16, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 1024, 2048, 4096, 16384, },
  { 32, 1024, 2048, 4096, 16384, },
  { 1, 32, 1024, 2048, 4096, 16384, },
  { 2, 32, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 1024, 2048, 4096, 16384, },
  { 4, 32, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 1024, 2048, 4096, 16384, },
  { 8, 32, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 1024, 2048, 4096, 16384, },
  { 16, 32, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 4096, 16384, },
  { 64, 1024, 2048, 4096, 16384, },
  { 1, 64, 1024, 2048, 4096, 16384, },
  { 2, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 64, 1024, 2048, 4096, 16384, },
  { 4, 64, 1024, 2048, 4096, 16384, },
  { 1, 4, 64, 1024, 2048, 4096, 16384, },
  { 2, 4, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 1024, 2048, 4096, 16384, },
  { 8, 64, 1024, 2048, 4096, 16384, },
  { 1, 8, 64, 1024, 2048, 4096, 16384, },
  { 2, 8, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 1024, 2048, 4096, 16384, },
  { 4, 8, 64, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 1024, 2048, 4096, 16384, },
  { 16, 64, 1024, 2048, 4096, 16384, },
  { 1, 16, 64, 1024, 2048, 4096, 16384, },
  { 2, 16, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 1024, 2048, 4096, 16384, },
  { 4, 16, 64, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 1024, 2048, 4096, 16384, },
  { 8, 16, 64, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 4096, 16384, },
  { 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 32, 64, 1024, 2048, 4096, 16384, },
  { 2, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 1024, 2048, 4096, 16384, },
  { 4, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 1024, 2048, 4096, 16384, },
  { 8, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 4096, 16384, },
  { 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 16384, },
  { 128, 1024, 2048, 4096, 16384, },
  { 1, 128, 1024, 2048, 4096, 16384, },
  { 2, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 128, 1024, 2048, 4096, 16384, },
  { 4, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 128, 1024, 2048, 4096, 16384, },
  { 8, 128, 1024, 2048, 4096, 16384, },
  { 1, 8, 128, 1024, 2048, 4096, 16384, },
  { 2, 8, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 128, 1024, 2048, 4096, 16384, },
  { 4, 8, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 128, 1024, 2048, 4096, 16384, },
  { 16, 128, 1024, 2048, 4096, 16384, },
  { 1, 16, 128, 1024, 2048, 4096, 16384, },
  { 2, 16, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 128, 1024, 2048, 4096, 16384, },
  { 4, 16, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 128, 1024, 2048, 4096, 16384, },
  { 8, 16, 128, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 128, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 128, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 4096, 16384, },
  { 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 32, 128, 1024, 2048, 4096, 16384, },
  { 2, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 128, 1024, 2048, 4096, 16384, },
  { 4, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 128, 1024, 2048, 4096, 16384, },
  { 8, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 128, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 128, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 4096, 16384, },
  { 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 16384, },
  { 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 64, 128, 1024, 2048, 4096, 16384, },
  { 4, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 128, 1024, 2048, 4096, 16384, },
  { 8, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 8, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 8, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 128, 1024, 2048, 4096, 16384, },
  { 4, 8, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 4096, 16384, },
  { 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 4, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 8, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 16384, },
  { 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 4, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 8, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, },
  { 256, 1024, 2048, 4096, 16384, },
  { 1, 256, 1024, 2048, 4096, 16384, },
  { 2, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 256, 1024, 2048, 4096, 16384, },
  { 4, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 256, 1024, 2048, 4096, 16384, },
  { 8, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 256, 1024, 2048, 4096, 16384, },
  { 16, 256, 1024, 2048, 4096, 16384, },
  { 1, 16, 256, 1024, 2048, 4096, 16384, },
  { 2, 16, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 256, 1024, 2048, 4096, 16384, },
  { 4, 16, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 256, 1024, 2048, 4096, 16384, },
  { 8, 16, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 4096, 16384, },
  { 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 32, 256, 1024, 2048, 4096, 16384, },
  { 2, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 256, 1024, 2048, 4096, 16384, },
  { 4, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 256, 1024, 2048, 4096, 16384, },
  { 8, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 4096, 16384, },
  { 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 16384, },
  { 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 64, 256, 1024, 2048, 4096, 16384, },
  { 4, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 256, 1024, 2048, 4096, 16384, },
  { 8, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 4096, 16384, },
  { 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 4, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 8, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 16384, },
  { 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 4, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 8, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, },
  { 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 128, 256, 1024, 2048, 4096, 16384, },
  { 8, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 4096, 16384, },
  { 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 8, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 16384, },
  { 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 8, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, },
  { 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 8, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, },
  { 512, 1024, 2048, 4096, 16384, },
  { 1, 512, 1024, 2048, 4096, 16384, },
  { 2, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 512, 1024, 2048, 4096, 16384, },
  { 4, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 512, 1024, 2048, 4096, 16384, },
  { 8, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 512, 1024, 2048, 4096, 16384, },
  { 16, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 4096, 16384, },
  { 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 32, 512, 1024, 2048, 4096, 16384, },
  { 2, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 512, 1024, 2048, 4096, 16384, },
  { 4, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 512, 1024, 2048, 4096, 16384, },
  { 8, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 4096, 16384, },
  { 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 16384, },
  { 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 64, 512, 1024, 2048, 4096, 16384, },
  { 4, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 512, 1024, 2048, 4096, 16384, },
  { 8, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 4096, 16384, },
  { 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 16384, },
  { 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 4, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 8, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, },
  { 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 128, 512, 1024, 2048, 4096, 16384, },
  { 8, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 4096, 16384, },
  { 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 16384, },
  { 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 8, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, },
  { 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 8, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, },
  { 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 4096, 16384, },
  { 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 16384, },
  { 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, },
  { 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, },
  { 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, },
  { 8192, 16384, },
  { 1, 8192, 16384, },
  { 2, 8192, 16384, },
  { 1, 2, 8192, 16384, },
  { 4, 8192, 16384, },
  { 1, 4, 8192, 16384, },
  { 2, 4, 8192, 16384, },
  { 1, 2, 4, 8192, 16384, },
  { 8, 8192, 16384, },
  { 1, 8, 8192, 16384, },
  { 2, 8, 8192, 16384, },
  { 1, 2, 8, 8192, 16384, },
  { 4, 8, 8192, 16384, },
  { 1, 4, 8, 8192, 16384, },
  { 2, 4, 8, 8192, 16384, },
  { 1, 2, 4, 8, 8192, 16384, },
  { 16, 8192, 16384, },
  { 1, 16, 8192, 16384, },
  { 2, 16, 8192, 16384, },
  { 1, 2, 16, 8192, 16384, },
  { 4, 16, 8192, 16384, },
  { 1, 4, 16, 8192, 16384, },
  { 2, 4, 16, 8192, 16384, },
  { 1, 2, 4, 16, 8192, 16384, },
  { 8, 16, 8192, 16384, },
  { 1, 8, 16, 8192, 16384, },
  { 2, 8, 16, 8192, 16384, },
  { 1, 2, 8, 16, 8192, 16384, },
  { 4, 8, 16, 8192, 16384, },
  { 1, 4, 8, 16, 8192, 16384, },
  { 2, 4, 8, 16, 8192, 16384, },
  { 1, 2, 4, 8, 16, 8192, 16384, },
  { 32, 8192, 16384, },
  { 1, 32, 8192, 16384, },
  { 2, 32, 8192, 16384, },
  { 1, 2, 32, 8192, 16384, },
  { 4, 32, 8192, 16384, },
  { 1, 4, 32, 8192, 16384, },
  { 2, 4, 32, 8192, 16384, },
  { 1, 2, 4, 32, 8192, 16384, },
  { 8, 32, 8192, 16384, },
  { 1, 8, 32, 8192, 16384, },
  { 2, 8, 32, 8192, 16384, },
  { 1, 2, 8, 32, 8192, 16384, },
  { 4, 8, 32, 8192, 16384, },
  { 1, 4, 8, 32, 8192, 16384, },
  { 2, 4, 8, 32, 8192, 16384, },
  { 1, 2, 4, 8, 32, 8192, 16384, },
  { 16, 32, 8192, 16384, },
  { 1, 16, 32, 8192, 16384, },
  { 2, 16, 32, 8192, 16384, },
  { 1, 2, 16, 32, 8192, 16384, },
  { 4, 16, 32, 8192, 16384, },
  { 1, 4, 16, 32, 8192, 16384, },
  { 2, 4, 16, 32, 8192, 16384, },
  { 1, 2, 4, 16, 32, 8192, 16384, },
  { 8, 16, 32, 8192, 16384, },
  { 1, 8, 16, 32, 8192, 16384, },
  { 2, 8, 16, 32, 8192, 16384, },
  { 1, 2, 8, 16, 32, 8192, 16384, },
  { 4, 8, 16, 32, 8192, 16384, },
  { 1, 4, 8, 16, 32, 8192, 16384, },
  { 2, 4, 8, 16, 32, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 8192, 16384, },
  { 64, 8192, 16384, },
  { 1, 64, 8192, 16384, },
  { 2, 64, 8192, 16384, },
  { 1, 2, 64, 8192, 16384, },
  { 4, 64, 8192, 16384, },
  { 1, 4, 64, 8192, 16384, },
  { 2, 4, 64, 8192, 16384, },
  { 1, 2, 4, 64, 8192, 16384, },
  { 8, 64, 8192, 16384, },
  { 1, 8, 64, 8192, 16384, },
  { 2, 8, 64, 8192, 16384, },
  { 1, 2, 8, 64, 8192, 16384, },
  { 4, 8, 64, 8192, 16384, },
  { 1, 4, 8, 64, 8192, 16384, },
  { 2, 4, 8, 64, 8192, 16384, },
  { 1, 2, 4, 8, 64, 8192, 16384, },
  { 16, 64, 8192, 16384, },
  { 1, 16, 64, 8192, 16384, },
  { 2, 16, 64, 8192, 16384, },
  { 1, 2, 16, 64, 8192, 16384, },
  { 4, 16, 64, 8192, 16384, },
  { 1, 4, 16, 64, 8192, 16384, },
  { 2, 4, 16, 64, 8192, 16384, },
  { 1, 2, 4, 16, 64, 8192, 16384, },
  { 8, 16, 64, 8192, 16384, },
  { 1, 8, 16, 64, 8192, 16384, },
  { 2, 8, 16, 64, 8192, 16384, },
  { 1, 2, 8, 16, 64, 8192, 16384, },
  { 4, 8, 16, 64, 8192, 16384, },
  { 1, 4, 8, 16, 64, 8192, 16384, },
  { 2, 4, 8, 16, 64, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 8192, 16384, },
  { 32, 64, 8192, 16384, },
  { 1, 32, 64, 8192, 16384, },
  { 2, 32, 64, 8192, 16384, },
  { 1, 2, 32, 64, 8192, 16384, },
  { 4, 32, 64, 8192, 16384, },
  { 1, 4, 32, 64, 8192, 16384, },
  { 2, 4, 32, 64, 8192, 16384, },
  { 1, 2, 4, 32, 64, 8192, 16384, },
  { 8, 32, 64, 8192, 16384, },
  { 1, 8, 32, 64, 8192, 16384, },
  { 2, 8, 32, 64, 8192, 16384, },
  { 1, 2, 8, 32, 64, 8192, 16384, },
  { 4, 8, 32, 64, 8192, 16384, },
  { 1, 4, 8, 32, 64, 8192, 16384, },
  { 2, 4, 8, 32, 64, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 8192, 16384, },
  { 16, 32, 64, 8192, 16384, },
  { 1, 16, 32, 64, 8192, 16384, },
  { 2, 16, 32, 64, 8192, 16384, },
  { 1, 2, 16, 32, 64, 8192, 16384, },
  { 4, 16, 32, 64, 8192, 16384, },
  { 1, 4, 16, 32, 64, 8192, 16384, },
  { 2, 4, 16, 32, 64, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 8192, 16384, },
  { 8, 16, 32, 64, 8192, 16384, },
  { 1, 8, 16, 32, 64, 8192, 16384, },
  { 2, 8, 16, 32, 64, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 8192, 16384, },
  { 4, 8, 16, 32, 64, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 8192, 16384, },
  { 128, 8192, 16384, },
  { 1, 128, 8192, 16384, },
  { 2, 128, 8192, 16384, },
  { 1, 2, 128, 8192, 16384, },
  { 4, 128, 8192, 16384, },
  { 1, 4, 128, 8192, 16384, },
  { 2, 4, 128, 8192, 16384, },
  { 1, 2, 4, 128, 8192, 16384, },
  { 8, 128, 8192, 16384, },
  { 1, 8, 128, 8192, 16384, },
  { 2, 8, 128, 8192, 16384, },
  { 1, 2, 8, 128, 8192, 16384, },
  { 4, 8, 128, 8192, 16384, },
  { 1, 4, 8, 128, 8192, 16384, },
  { 2, 4, 8, 128, 8192, 16384, },
  { 1, 2, 4, 8, 128, 8192, 16384, },
  { 16, 128, 8192, 16384, },
  { 1, 16, 128, 8192, 16384, },
  { 2, 16, 128, 8192, 16384, },
  { 1, 2, 16, 128, 8192, 16384, },
  { 4, 16, 128, 8192, 16384, },
  { 1, 4, 16, 128, 8192, 16384, },
  { 2, 4, 16, 128, 8192, 16384, },
  { 1, 2, 4, 16, 128, 8192, 16384, },
  { 8, 16, 128, 8192, 16384, },
  { 1, 8, 16, 128, 8192, 16384, },
  { 2, 8, 16, 128, 8192, 16384, },
  { 1, 2, 8, 16, 128, 8192, 16384, },
  { 4, 8, 16, 128, 8192, 16384, },
  { 1, 4, 8, 16, 128, 8192, 16384, },
  { 2, 4, 8, 16, 128, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 8192, 16384, },
  { 32, 128, 8192, 16384, },
  { 1, 32, 128, 8192, 16384, },
  { 2, 32, 128, 8192, 16384, },
  { 1, 2, 32, 128, 8192, 16384, },
  { 4, 32, 128, 8192, 16384, },
  { 1, 4, 32, 128, 8192, 16384, },
  { 2, 4, 32, 128, 8192, 16384, },
  { 1, 2, 4, 32, 128, 8192, 16384, },
  { 8, 32, 128, 8192, 16384, },
  { 1, 8, 32, 128, 8192, 16384, },
  { 2, 8, 32, 128, 8192, 16384, },
  { 1, 2, 8, 32, 128, 8192, 16384, },
  { 4, 8, 32, 128, 8192, 16384, },
  { 1, 4, 8, 32, 128, 8192, 16384, },
  { 2, 4, 8, 32, 128, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 8192, 16384, },
  { 16, 32, 128, 8192, 16384, },
  { 1, 16, 32, 128, 8192, 16384, },
  { 2, 16, 32, 128, 8192, 16384, },
  { 1, 2, 16, 32, 128, 8192, 16384, },
  { 4, 16, 32, 128, 8192, 16384, },
  { 1, 4, 16, 32, 128, 8192, 16384, },
  { 2, 4, 16, 32, 128, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 8192, 16384, },
  { 8, 16, 32, 128, 8192, 16384, },
  { 1, 8, 16, 32, 128, 8192, 16384, },
  { 2, 8, 16, 32, 128, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 8192, 16384, },
  { 4, 8, 16, 32, 128, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 8192, 16384, },
  { 64, 128, 8192, 16384, },
  { 1, 64, 128, 8192, 16384, },
  { 2, 64, 128, 8192, 16384, },
  { 1, 2, 64, 128, 8192, 16384, },
  { 4, 64, 128, 8192, 16384, },
  { 1, 4, 64, 128, 8192, 16384, },
  { 2, 4, 64, 128, 8192, 16384, },
  { 1, 2, 4, 64, 128, 8192, 16384, },
  { 8, 64, 128, 8192, 16384, },
  { 1, 8, 64, 128, 8192, 16384, },
  { 2, 8, 64, 128, 8192, 16384, },
  { 1, 2, 8, 64, 128, 8192, 16384, },
  { 4, 8, 64, 128, 8192, 16384, },
  { 1, 4, 8, 64, 128, 8192, 16384, },
  { 2, 4, 8, 64, 128, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 8192, 16384, },
  { 16, 64, 128, 8192, 16384, },
  { 1, 16, 64, 128, 8192, 16384, },
  { 2, 16, 64, 128, 8192, 16384, },
  { 1, 2, 16, 64, 128, 8192, 16384, },
  { 4, 16, 64, 128, 8192, 16384, },
  { 1, 4, 16, 64, 128, 8192, 16384, },
  { 2, 4, 16, 64, 128, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 8192, 16384, },
  { 8, 16, 64, 128, 8192, 16384, },
  { 1, 8, 16, 64, 128, 8192, 16384, },
  { 2, 8, 16, 64, 128, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 8192, 16384, },
  { 4, 8, 16, 64, 128, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 8192, 16384, },
  { 32, 64, 128, 8192, 16384, },
  { 1, 32, 64, 128, 8192, 16384, },
  { 2, 32, 64, 128, 8192, 16384, },
  { 1, 2, 32, 64, 128, 8192, 16384, },
  { 4, 32, 64, 128, 8192, 16384, },
  { 1, 4, 32, 64, 128, 8192, 16384, },
  { 2, 4, 32, 64, 128, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 8192, 16384, },
  { 8, 32, 64, 128, 8192, 16384, },
  { 1, 8, 32, 64, 128, 8192, 16384, },
  { 2, 8, 32, 64, 128, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 8192, 16384, },
  { 4, 8, 32, 64, 128, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 8192, 16384, },
  { 16, 32, 64, 128, 8192, 16384, },
  { 1, 16, 32, 64, 128, 8192, 16384, },
  { 2, 16, 32, 64, 128, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 8192, 16384, },
  { 4, 16, 32, 64, 128, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 8192, 16384, },
  { 8, 16, 32, 64, 128, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 8192, 16384, },
  { 256, 8192, 16384, },
  { 1, 256, 8192, 16384, },
  { 2, 256, 8192, 16384, },
  { 1, 2, 256, 8192, 16384, },
  { 4, 256, 8192, 16384, },
  { 1, 4, 256, 8192, 16384, },
  { 2, 4, 256, 8192, 16384, },
  { 1, 2, 4, 256, 8192, 16384, },
  { 8, 256, 8192, 16384, },
  { 1, 8, 256, 8192, 16384, },
  { 2, 8, 256, 8192, 16384, },
  { 1, 2, 8, 256, 8192, 16384, },
  { 4, 8, 256, 8192, 16384, },
  { 1, 4, 8, 256, 8192, 16384, },
  { 2, 4, 8, 256, 8192, 16384, },
  { 1, 2, 4, 8, 256, 8192, 16384, },
  { 16, 256, 8192, 16384, },
  { 1, 16, 256, 8192, 16384, },
  { 2, 16, 256, 8192, 16384, },
  { 1, 2, 16, 256, 8192, 16384, },
  { 4, 16, 256, 8192, 16384, },
  { 1, 4, 16, 256, 8192, 16384, },
  { 2, 4, 16, 256, 8192, 16384, },
  { 1, 2, 4, 16, 256, 8192, 16384, },
  { 8, 16, 256, 8192, 16384, },
  { 1, 8, 16, 256, 8192, 16384, },
  { 2, 8, 16, 256, 8192, 16384, },
  { 1, 2, 8, 16, 256, 8192, 16384, },
  { 4, 8, 16, 256, 8192, 16384, },
  { 1, 4, 8, 16, 256, 8192, 16384, },
  { 2, 4, 8, 16, 256, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 8192, 16384, },
  { 32, 256, 8192, 16384, },
  { 1, 32, 256, 8192, 16384, },
  { 2, 32, 256, 8192, 16384, },
  { 1, 2, 32, 256, 8192, 16384, },
  { 4, 32, 256, 8192, 16384, },
  { 1, 4, 32, 256, 8192, 16384, },
  { 2, 4, 32, 256, 8192, 16384, },
  { 1, 2, 4, 32, 256, 8192, 16384, },
  { 8, 32, 256, 8192, 16384, },
  { 1, 8, 32, 256, 8192, 16384, },
  { 2, 8, 32, 256, 8192, 16384, },
  { 1, 2, 8, 32, 256, 8192, 16384, },
  { 4, 8, 32, 256, 8192, 16384, },
  { 1, 4, 8, 32, 256, 8192, 16384, },
  { 2, 4, 8, 32, 256, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 8192, 16384, },
  { 16, 32, 256, 8192, 16384, },
  { 1, 16, 32, 256, 8192, 16384, },
  { 2, 16, 32, 256, 8192, 16384, },
  { 1, 2, 16, 32, 256, 8192, 16384, },
  { 4, 16, 32, 256, 8192, 16384, },
  { 1, 4, 16, 32, 256, 8192, 16384, },
  { 2, 4, 16, 32, 256, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 8192, 16384, },
  { 8, 16, 32, 256, 8192, 16384, },
  { 1, 8, 16, 32, 256, 8192, 16384, },
  { 2, 8, 16, 32, 256, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 8192, 16384, },
  { 4, 8, 16, 32, 256, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 8192, 16384, },
  { 64, 256, 8192, 16384, },
  { 1, 64, 256, 8192, 16384, },
  { 2, 64, 256, 8192, 16384, },
  { 1, 2, 64, 256, 8192, 16384, },
  { 4, 64, 256, 8192, 16384, },
  { 1, 4, 64, 256, 8192, 16384, },
  { 2, 4, 64, 256, 8192, 16384, },
  { 1, 2, 4, 64, 256, 8192, 16384, },
  { 8, 64, 256, 8192, 16384, },
  { 1, 8, 64, 256, 8192, 16384, },
  { 2, 8, 64, 256, 8192, 16384, },
  { 1, 2, 8, 64, 256, 8192, 16384, },
  { 4, 8, 64, 256, 8192, 16384, },
  { 1, 4, 8, 64, 256, 8192, 16384, },
  { 2, 4, 8, 64, 256, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 8192, 16384, },
  { 16, 64, 256, 8192, 16384, },
  { 1, 16, 64, 256, 8192, 16384, },
  { 2, 16, 64, 256, 8192, 16384, },
  { 1, 2, 16, 64, 256, 8192, 16384, },
  { 4, 16, 64, 256, 8192, 16384, },
  { 1, 4, 16, 64, 256, 8192, 16384, },
  { 2, 4, 16, 64, 256, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 8192, 16384, },
  { 8, 16, 64, 256, 8192, 16384, },
  { 1, 8, 16, 64, 256, 8192, 16384, },
  { 2, 8, 16, 64, 256, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 8192, 16384, },
  { 4, 8, 16, 64, 256, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 8192, 16384, },
  { 32, 64, 256, 8192, 16384, },
  { 1, 32, 64, 256, 8192, 16384, },
  { 2, 32, 64, 256, 8192, 16384, },
  { 1, 2, 32, 64, 256, 8192, 16384, },
  { 4, 32, 64, 256, 8192, 16384, },
  { 1, 4, 32, 64, 256, 8192, 16384, },
  { 2, 4, 32, 64, 256, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 8192, 16384, },
  { 8, 32, 64, 256, 8192, 16384, },
  { 1, 8, 32, 64, 256, 8192, 16384, },
  { 2, 8, 32, 64, 256, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 8192, 16384, },
  { 4, 8, 32, 64, 256, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 8192, 16384, },
  { 16, 32, 64, 256, 8192, 16384, },
  { 1, 16, 32, 64, 256, 8192, 16384, },
  { 2, 16, 32, 64, 256, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 8192, 16384, },
  { 4, 16, 32, 64, 256, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 8192, 16384, },
  { 8, 16, 32, 64, 256, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 8192, 16384, },
  { 128, 256, 8192, 16384, },
  { 1, 128, 256, 8192, 16384, },
  { 2, 128, 256, 8192, 16384, },
  { 1, 2, 128, 256, 8192, 16384, },
  { 4, 128, 256, 8192, 16384, },
  { 1, 4, 128, 256, 8192, 16384, },
  { 2, 4, 128, 256, 8192, 16384, },
  { 1, 2, 4, 128, 256, 8192, 16384, },
  { 8, 128, 256, 8192, 16384, },
  { 1, 8, 128, 256, 8192, 16384, },
  { 2, 8, 128, 256, 8192, 16384, },
  { 1, 2, 8, 128, 256, 8192, 16384, },
  { 4, 8, 128, 256, 8192, 16384, },
  { 1, 4, 8, 128, 256, 8192, 16384, },
  { 2, 4, 8, 128, 256, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 8192, 16384, },
  { 16, 128, 256, 8192, 16384, },
  { 1, 16, 128, 256, 8192, 16384, },
  { 2, 16, 128, 256, 8192, 16384, },
  { 1, 2, 16, 128, 256, 8192, 16384, },
  { 4, 16, 128, 256, 8192, 16384, },
  { 1, 4, 16, 128, 256, 8192, 16384, },
  { 2, 4, 16, 128, 256, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 8192, 16384, },
  { 8, 16, 128, 256, 8192, 16384, },
  { 1, 8, 16, 128, 256, 8192, 16384, },
  { 2, 8, 16, 128, 256, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 8192, 16384, },
  { 4, 8, 16, 128, 256, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 8192, 16384, },
  { 32, 128, 256, 8192, 16384, },
  { 1, 32, 128, 256, 8192, 16384, },
  { 2, 32, 128, 256, 8192, 16384, },
  { 1, 2, 32, 128, 256, 8192, 16384, },
  { 4, 32, 128, 256, 8192, 16384, },
  { 1, 4, 32, 128, 256, 8192, 16384, },
  { 2, 4, 32, 128, 256, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 8192, 16384, },
  { 8, 32, 128, 256, 8192, 16384, },
  { 1, 8, 32, 128, 256, 8192, 16384, },
  { 2, 8, 32, 128, 256, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 8192, 16384, },
  { 4, 8, 32, 128, 256, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 8192, 16384, },
  { 16, 32, 128, 256, 8192, 16384, },
  { 1, 16, 32, 128, 256, 8192, 16384, },
  { 2, 16, 32, 128, 256, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 8192, 16384, },
  { 4, 16, 32, 128, 256, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 8192, 16384, },
  { 8, 16, 32, 128, 256, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 8192, 16384, },
  { 64, 128, 256, 8192, 16384, },
  { 1, 64, 128, 256, 8192, 16384, },
  { 2, 64, 128, 256, 8192, 16384, },
  { 1, 2, 64, 128, 256, 8192, 16384, },
  { 4, 64, 128, 256, 8192, 16384, },
  { 1, 4, 64, 128, 256, 8192, 16384, },
  { 2, 4, 64, 128, 256, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 8192, 16384, },
  { 8, 64, 128, 256, 8192, 16384, },
  { 1, 8, 64, 128, 256, 8192, 16384, },
  { 2, 8, 64, 128, 256, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 8192, 16384, },
  { 4, 8, 64, 128, 256, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 8192, 16384, },
  { 16, 64, 128, 256, 8192, 16384, },
  { 1, 16, 64, 128, 256, 8192, 16384, },
  { 2, 16, 64, 128, 256, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 8192, 16384, },
  { 4, 16, 64, 128, 256, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 8192, 16384, },
  { 8, 16, 64, 128, 256, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 8192, 16384, },
  { 32, 64, 128, 256, 8192, 16384, },
  { 1, 32, 64, 128, 256, 8192, 16384, },
  { 2, 32, 64, 128, 256, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 8192, 16384, },
  { 4, 32, 64, 128, 256, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 8192, 16384, },
  { 8, 32, 64, 128, 256, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 8192, 16384, },
  { 16, 32, 64, 128, 256, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 8192, 16384, },
  { 512, 8192, 16384, },
  { 1, 512, 8192, 16384, },
  { 2, 512, 8192, 16384, },
  { 1, 2, 512, 8192, 16384, },
  { 4, 512, 8192, 16384, },
  { 1, 4, 512, 8192, 16384, },
  { 2, 4, 512, 8192, 16384, },
  { 1, 2, 4, 512, 8192, 16384, },
  { 8, 512, 8192, 16384, },
  { 1, 8, 512, 8192, 16384, },
  { 2, 8, 512, 8192, 16384, },
  { 1, 2, 8, 512, 8192, 16384, },
  { 4, 8, 512, 8192, 16384, },
  { 1, 4, 8, 512, 8192, 16384, },
  { 2, 4, 8, 512, 8192, 16384, },
  { 1, 2, 4, 8, 512, 8192, 16384, },
  { 16, 512, 8192, 16384, },
  { 1, 16, 512, 8192, 16384, },
  { 2, 16, 512, 8192, 16384, },
  { 1, 2, 16, 512, 8192, 16384, },
  { 4, 16, 512, 8192, 16384, },
  { 1, 4, 16, 512, 8192, 16384, },
  { 2, 4, 16, 512, 8192, 16384, },
  { 1, 2, 4, 16, 512, 8192, 16384, },
  { 8, 16, 512, 8192, 16384, },
  { 1, 8, 16, 512, 8192, 16384, },
  { 2, 8, 16, 512, 8192, 16384, },
  { 1, 2, 8, 16, 512, 8192, 16384, },
  { 4, 8, 16, 512, 8192, 16384, },
  { 1, 4, 8, 16, 512, 8192, 16384, },
  { 2, 4, 8, 16, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 512, 8192, 16384, },
  { 32, 512, 8192, 16384, },
  { 1, 32, 512, 8192, 16384, },
  { 2, 32, 512, 8192, 16384, },
  { 1, 2, 32, 512, 8192, 16384, },
  { 4, 32, 512, 8192, 16384, },
  { 1, 4, 32, 512, 8192, 16384, },
  { 2, 4, 32, 512, 8192, 16384, },
  { 1, 2, 4, 32, 512, 8192, 16384, },
  { 8, 32, 512, 8192, 16384, },
  { 1, 8, 32, 512, 8192, 16384, },
  { 2, 8, 32, 512, 8192, 16384, },
  { 1, 2, 8, 32, 512, 8192, 16384, },
  { 4, 8, 32, 512, 8192, 16384, },
  { 1, 4, 8, 32, 512, 8192, 16384, },
  { 2, 4, 8, 32, 512, 8192, 16384, },
  { 1, 2, 4, 8, 32, 512, 8192, 16384, },
  { 16, 32, 512, 8192, 16384, },
  { 1, 16, 32, 512, 8192, 16384, },
  { 2, 16, 32, 512, 8192, 16384, },
  { 1, 2, 16, 32, 512, 8192, 16384, },
  { 4, 16, 32, 512, 8192, 16384, },
  { 1, 4, 16, 32, 512, 8192, 16384, },
  { 2, 4, 16, 32, 512, 8192, 16384, },
  { 1, 2, 4, 16, 32, 512, 8192, 16384, },
  { 8, 16, 32, 512, 8192, 16384, },
  { 1, 8, 16, 32, 512, 8192, 16384, },
  { 2, 8, 16, 32, 512, 8192, 16384, },
  { 1, 2, 8, 16, 32, 512, 8192, 16384, },
  { 4, 8, 16, 32, 512, 8192, 16384, },
  { 1, 4, 8, 16, 32, 512, 8192, 16384, },
  { 2, 4, 8, 16, 32, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 8192, 16384, },
  { 64, 512, 8192, 16384, },
  { 1, 64, 512, 8192, 16384, },
  { 2, 64, 512, 8192, 16384, },
  { 1, 2, 64, 512, 8192, 16384, },
  { 4, 64, 512, 8192, 16384, },
  { 1, 4, 64, 512, 8192, 16384, },
  { 2, 4, 64, 512, 8192, 16384, },
  { 1, 2, 4, 64, 512, 8192, 16384, },
  { 8, 64, 512, 8192, 16384, },
  { 1, 8, 64, 512, 8192, 16384, },
  { 2, 8, 64, 512, 8192, 16384, },
  { 1, 2, 8, 64, 512, 8192, 16384, },
  { 4, 8, 64, 512, 8192, 16384, },
  { 1, 4, 8, 64, 512, 8192, 16384, },
  { 2, 4, 8, 64, 512, 8192, 16384, },
  { 1, 2, 4, 8, 64, 512, 8192, 16384, },
  { 16, 64, 512, 8192, 16384, },
  { 1, 16, 64, 512, 8192, 16384, },
  { 2, 16, 64, 512, 8192, 16384, },
  { 1, 2, 16, 64, 512, 8192, 16384, },
  { 4, 16, 64, 512, 8192, 16384, },
  { 1, 4, 16, 64, 512, 8192, 16384, },
  { 2, 4, 16, 64, 512, 8192, 16384, },
  { 1, 2, 4, 16, 64, 512, 8192, 16384, },
  { 8, 16, 64, 512, 8192, 16384, },
  { 1, 8, 16, 64, 512, 8192, 16384, },
  { 2, 8, 16, 64, 512, 8192, 16384, },
  { 1, 2, 8, 16, 64, 512, 8192, 16384, },
  { 4, 8, 16, 64, 512, 8192, 16384, },
  { 1, 4, 8, 16, 64, 512, 8192, 16384, },
  { 2, 4, 8, 16, 64, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 8192, 16384, },
  { 32, 64, 512, 8192, 16384, },
  { 1, 32, 64, 512, 8192, 16384, },
  { 2, 32, 64, 512, 8192, 16384, },
  { 1, 2, 32, 64, 512, 8192, 16384, },
  { 4, 32, 64, 512, 8192, 16384, },
  { 1, 4, 32, 64, 512, 8192, 16384, },
  { 2, 4, 32, 64, 512, 8192, 16384, },
  { 1, 2, 4, 32, 64, 512, 8192, 16384, },
  { 8, 32, 64, 512, 8192, 16384, },
  { 1, 8, 32, 64, 512, 8192, 16384, },
  { 2, 8, 32, 64, 512, 8192, 16384, },
  { 1, 2, 8, 32, 64, 512, 8192, 16384, },
  { 4, 8, 32, 64, 512, 8192, 16384, },
  { 1, 4, 8, 32, 64, 512, 8192, 16384, },
  { 2, 4, 8, 32, 64, 512, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 8192, 16384, },
  { 16, 32, 64, 512, 8192, 16384, },
  { 1, 16, 32, 64, 512, 8192, 16384, },
  { 2, 16, 32, 64, 512, 8192, 16384, },
  { 1, 2, 16, 32, 64, 512, 8192, 16384, },
  { 4, 16, 32, 64, 512, 8192, 16384, },
  { 1, 4, 16, 32, 64, 512, 8192, 16384, },
  { 2, 4, 16, 32, 64, 512, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 8192, 16384, },
  { 8, 16, 32, 64, 512, 8192, 16384, },
  { 1, 8, 16, 32, 64, 512, 8192, 16384, },
  { 2, 8, 16, 32, 64, 512, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 8192, 16384, },
  { 4, 8, 16, 32, 64, 512, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 8192, 16384, },
  { 128, 512, 8192, 16384, },
  { 1, 128, 512, 8192, 16384, },
  { 2, 128, 512, 8192, 16384, },
  { 1, 2, 128, 512, 8192, 16384, },
  { 4, 128, 512, 8192, 16384, },
  { 1, 4, 128, 512, 8192, 16384, },
  { 2, 4, 128, 512, 8192, 16384, },
  { 1, 2, 4, 128, 512, 8192, 16384, },
  { 8, 128, 512, 8192, 16384, },
  { 1, 8, 128, 512, 8192, 16384, },
  { 2, 8, 128, 512, 8192, 16384, },
  { 1, 2, 8, 128, 512, 8192, 16384, },
  { 4, 8, 128, 512, 8192, 16384, },
  { 1, 4, 8, 128, 512, 8192, 16384, },
  { 2, 4, 8, 128, 512, 8192, 16384, },
  { 1, 2, 4, 8, 128, 512, 8192, 16384, },
  { 16, 128, 512, 8192, 16384, },
  { 1, 16, 128, 512, 8192, 16384, },
  { 2, 16, 128, 512, 8192, 16384, },
  { 1, 2, 16, 128, 512, 8192, 16384, },
  { 4, 16, 128, 512, 8192, 16384, },
  { 1, 4, 16, 128, 512, 8192, 16384, },
  { 2, 4, 16, 128, 512, 8192, 16384, },
  { 1, 2, 4, 16, 128, 512, 8192, 16384, },
  { 8, 16, 128, 512, 8192, 16384, },
  { 1, 8, 16, 128, 512, 8192, 16384, },
  { 2, 8, 16, 128, 512, 8192, 16384, },
  { 1, 2, 8, 16, 128, 512, 8192, 16384, },
  { 4, 8, 16, 128, 512, 8192, 16384, },
  { 1, 4, 8, 16, 128, 512, 8192, 16384, },
  { 2, 4, 8, 16, 128, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 8192, 16384, },
  { 32, 128, 512, 8192, 16384, },
  { 1, 32, 128, 512, 8192, 16384, },
  { 2, 32, 128, 512, 8192, 16384, },
  { 1, 2, 32, 128, 512, 8192, 16384, },
  { 4, 32, 128, 512, 8192, 16384, },
  { 1, 4, 32, 128, 512, 8192, 16384, },
  { 2, 4, 32, 128, 512, 8192, 16384, },
  { 1, 2, 4, 32, 128, 512, 8192, 16384, },
  { 8, 32, 128, 512, 8192, 16384, },
  { 1, 8, 32, 128, 512, 8192, 16384, },
  { 2, 8, 32, 128, 512, 8192, 16384, },
  { 1, 2, 8, 32, 128, 512, 8192, 16384, },
  { 4, 8, 32, 128, 512, 8192, 16384, },
  { 1, 4, 8, 32, 128, 512, 8192, 16384, },
  { 2, 4, 8, 32, 128, 512, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 8192, 16384, },
  { 16, 32, 128, 512, 8192, 16384, },
  { 1, 16, 32, 128, 512, 8192, 16384, },
  { 2, 16, 32, 128, 512, 8192, 16384, },
  { 1, 2, 16, 32, 128, 512, 8192, 16384, },
  { 4, 16, 32, 128, 512, 8192, 16384, },
  { 1, 4, 16, 32, 128, 512, 8192, 16384, },
  { 2, 4, 16, 32, 128, 512, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 8192, 16384, },
  { 8, 16, 32, 128, 512, 8192, 16384, },
  { 1, 8, 16, 32, 128, 512, 8192, 16384, },
  { 2, 8, 16, 32, 128, 512, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 8192, 16384, },
  { 4, 8, 16, 32, 128, 512, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 8192, 16384, },
  { 64, 128, 512, 8192, 16384, },
  { 1, 64, 128, 512, 8192, 16384, },
  { 2, 64, 128, 512, 8192, 16384, },
  { 1, 2, 64, 128, 512, 8192, 16384, },
  { 4, 64, 128, 512, 8192, 16384, },
  { 1, 4, 64, 128, 512, 8192, 16384, },
  { 2, 4, 64, 128, 512, 8192, 16384, },
  { 1, 2, 4, 64, 128, 512, 8192, 16384, },
  { 8, 64, 128, 512, 8192, 16384, },
  { 1, 8, 64, 128, 512, 8192, 16384, },
  { 2, 8, 64, 128, 512, 8192, 16384, },
  { 1, 2, 8, 64, 128, 512, 8192, 16384, },
  { 4, 8, 64, 128, 512, 8192, 16384, },
  { 1, 4, 8, 64, 128, 512, 8192, 16384, },
  { 2, 4, 8, 64, 128, 512, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 8192, 16384, },
  { 16, 64, 128, 512, 8192, 16384, },
  { 1, 16, 64, 128, 512, 8192, 16384, },
  { 2, 16, 64, 128, 512, 8192, 16384, },
  { 1, 2, 16, 64, 128, 512, 8192, 16384, },
  { 4, 16, 64, 128, 512, 8192, 16384, },
  { 1, 4, 16, 64, 128, 512, 8192, 16384, },
  { 2, 4, 16, 64, 128, 512, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 8192, 16384, },
  { 8, 16, 64, 128, 512, 8192, 16384, },
  { 1, 8, 16, 64, 128, 512, 8192, 16384, },
  { 2, 8, 16, 64, 128, 512, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 8192, 16384, },
  { 4, 8, 16, 64, 128, 512, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 8192, 16384, },
  { 32, 64, 128, 512, 8192, 16384, },
  { 1, 32, 64, 128, 512, 8192, 16384, },
  { 2, 32, 64, 128, 512, 8192, 16384, },
  { 1, 2, 32, 64, 128, 512, 8192, 16384, },
  { 4, 32, 64, 128, 512, 8192, 16384, },
  { 1, 4, 32, 64, 128, 512, 8192, 16384, },
  { 2, 4, 32, 64, 128, 512, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 8192, 16384, },
  { 8, 32, 64, 128, 512, 8192, 16384, },
  { 1, 8, 32, 64, 128, 512, 8192, 16384, },
  { 2, 8, 32, 64, 128, 512, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 8192, 16384, },
  { 4, 8, 32, 64, 128, 512, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 8192, 16384, },
  { 16, 32, 64, 128, 512, 8192, 16384, },
  { 1, 16, 32, 64, 128, 512, 8192, 16384, },
  { 2, 16, 32, 64, 128, 512, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 8192, 16384, },
  { 4, 16, 32, 64, 128, 512, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 8192, 16384, },
  { 8, 16, 32, 64, 128, 512, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 8192, 16384, },
  { 256, 512, 8192, 16384, },
  { 1, 256, 512, 8192, 16384, },
  { 2, 256, 512, 8192, 16384, },
  { 1, 2, 256, 512, 8192, 16384, },
  { 4, 256, 512, 8192, 16384, },
  { 1, 4, 256, 512, 8192, 16384, },
  { 2, 4, 256, 512, 8192, 16384, },
  { 1, 2, 4, 256, 512, 8192, 16384, },
  { 8, 256, 512, 8192, 16384, },
  { 1, 8, 256, 512, 8192, 16384, },
  { 2, 8, 256, 512, 8192, 16384, },
  { 1, 2, 8, 256, 512, 8192, 16384, },
  { 4, 8, 256, 512, 8192, 16384, },
  { 1, 4, 8, 256, 512, 8192, 16384, },
  { 2, 4, 8, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 256, 512, 8192, 16384, },
  { 16, 256, 512, 8192, 16384, },
  { 1, 16, 256, 512, 8192, 16384, },
  { 2, 16, 256, 512, 8192, 16384, },
  { 1, 2, 16, 256, 512, 8192, 16384, },
  { 4, 16, 256, 512, 8192, 16384, },
  { 1, 4, 16, 256, 512, 8192, 16384, },
  { 2, 4, 16, 256, 512, 8192, 16384, },
  { 1, 2, 4, 16, 256, 512, 8192, 16384, },
  { 8, 16, 256, 512, 8192, 16384, },
  { 1, 8, 16, 256, 512, 8192, 16384, },
  { 2, 8, 16, 256, 512, 8192, 16384, },
  { 1, 2, 8, 16, 256, 512, 8192, 16384, },
  { 4, 8, 16, 256, 512, 8192, 16384, },
  { 1, 4, 8, 16, 256, 512, 8192, 16384, },
  { 2, 4, 8, 16, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 8192, 16384, },
  { 32, 256, 512, 8192, 16384, },
  { 1, 32, 256, 512, 8192, 16384, },
  { 2, 32, 256, 512, 8192, 16384, },
  { 1, 2, 32, 256, 512, 8192, 16384, },
  { 4, 32, 256, 512, 8192, 16384, },
  { 1, 4, 32, 256, 512, 8192, 16384, },
  { 2, 4, 32, 256, 512, 8192, 16384, },
  { 1, 2, 4, 32, 256, 512, 8192, 16384, },
  { 8, 32, 256, 512, 8192, 16384, },
  { 1, 8, 32, 256, 512, 8192, 16384, },
  { 2, 8, 32, 256, 512, 8192, 16384, },
  { 1, 2, 8, 32, 256, 512, 8192, 16384, },
  { 4, 8, 32, 256, 512, 8192, 16384, },
  { 1, 4, 8, 32, 256, 512, 8192, 16384, },
  { 2, 4, 8, 32, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 8192, 16384, },
  { 16, 32, 256, 512, 8192, 16384, },
  { 1, 16, 32, 256, 512, 8192, 16384, },
  { 2, 16, 32, 256, 512, 8192, 16384, },
  { 1, 2, 16, 32, 256, 512, 8192, 16384, },
  { 4, 16, 32, 256, 512, 8192, 16384, },
  { 1, 4, 16, 32, 256, 512, 8192, 16384, },
  { 2, 4, 16, 32, 256, 512, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 8192, 16384, },
  { 8, 16, 32, 256, 512, 8192, 16384, },
  { 1, 8, 16, 32, 256, 512, 8192, 16384, },
  { 2, 8, 16, 32, 256, 512, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 8192, 16384, },
  { 4, 8, 16, 32, 256, 512, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 8192, 16384, },
  { 64, 256, 512, 8192, 16384, },
  { 1, 64, 256, 512, 8192, 16384, },
  { 2, 64, 256, 512, 8192, 16384, },
  { 1, 2, 64, 256, 512, 8192, 16384, },
  { 4, 64, 256, 512, 8192, 16384, },
  { 1, 4, 64, 256, 512, 8192, 16384, },
  { 2, 4, 64, 256, 512, 8192, 16384, },
  { 1, 2, 4, 64, 256, 512, 8192, 16384, },
  { 8, 64, 256, 512, 8192, 16384, },
  { 1, 8, 64, 256, 512, 8192, 16384, },
  { 2, 8, 64, 256, 512, 8192, 16384, },
  { 1, 2, 8, 64, 256, 512, 8192, 16384, },
  { 4, 8, 64, 256, 512, 8192, 16384, },
  { 1, 4, 8, 64, 256, 512, 8192, 16384, },
  { 2, 4, 8, 64, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 8192, 16384, },
  { 16, 64, 256, 512, 8192, 16384, },
  { 1, 16, 64, 256, 512, 8192, 16384, },
  { 2, 16, 64, 256, 512, 8192, 16384, },
  { 1, 2, 16, 64, 256, 512, 8192, 16384, },
  { 4, 16, 64, 256, 512, 8192, 16384, },
  { 1, 4, 16, 64, 256, 512, 8192, 16384, },
  { 2, 4, 16, 64, 256, 512, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 8192, 16384, },
  { 8, 16, 64, 256, 512, 8192, 16384, },
  { 1, 8, 16, 64, 256, 512, 8192, 16384, },
  { 2, 8, 16, 64, 256, 512, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 8192, 16384, },
  { 4, 8, 16, 64, 256, 512, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 8192, 16384, },
  { 32, 64, 256, 512, 8192, 16384, },
  { 1, 32, 64, 256, 512, 8192, 16384, },
  { 2, 32, 64, 256, 512, 8192, 16384, },
  { 1, 2, 32, 64, 256, 512, 8192, 16384, },
  { 4, 32, 64, 256, 512, 8192, 16384, },
  { 1, 4, 32, 64, 256, 512, 8192, 16384, },
  { 2, 4, 32, 64, 256, 512, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 8192, 16384, },
  { 8, 32, 64, 256, 512, 8192, 16384, },
  { 1, 8, 32, 64, 256, 512, 8192, 16384, },
  { 2, 8, 32, 64, 256, 512, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 8192, 16384, },
  { 4, 8, 32, 64, 256, 512, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 8192, 16384, },
  { 16, 32, 64, 256, 512, 8192, 16384, },
  { 1, 16, 32, 64, 256, 512, 8192, 16384, },
  { 2, 16, 32, 64, 256, 512, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 8192, 16384, },
  { 4, 16, 32, 64, 256, 512, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 8192, 16384, },
  { 8, 16, 32, 64, 256, 512, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 8192, 16384, },
  { 128, 256, 512, 8192, 16384, },
  { 1, 128, 256, 512, 8192, 16384, },
  { 2, 128, 256, 512, 8192, 16384, },
  { 1, 2, 128, 256, 512, 8192, 16384, },
  { 4, 128, 256, 512, 8192, 16384, },
  { 1, 4, 128, 256, 512, 8192, 16384, },
  { 2, 4, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 128, 256, 512, 8192, 16384, },
  { 8, 128, 256, 512, 8192, 16384, },
  { 1, 8, 128, 256, 512, 8192, 16384, },
  { 2, 8, 128, 256, 512, 8192, 16384, },
  { 1, 2, 8, 128, 256, 512, 8192, 16384, },
  { 4, 8, 128, 256, 512, 8192, 16384, },
  { 1, 4, 8, 128, 256, 512, 8192, 16384, },
  { 2, 4, 8, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 8192, 16384, },
  { 16, 128, 256, 512, 8192, 16384, },
  { 1, 16, 128, 256, 512, 8192, 16384, },
  { 2, 16, 128, 256, 512, 8192, 16384, },
  { 1, 2, 16, 128, 256, 512, 8192, 16384, },
  { 4, 16, 128, 256, 512, 8192, 16384, },
  { 1, 4, 16, 128, 256, 512, 8192, 16384, },
  { 2, 4, 16, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 8192, 16384, },
  { 8, 16, 128, 256, 512, 8192, 16384, },
  { 1, 8, 16, 128, 256, 512, 8192, 16384, },
  { 2, 8, 16, 128, 256, 512, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 8192, 16384, },
  { 4, 8, 16, 128, 256, 512, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 8192, 16384, },
  { 32, 128, 256, 512, 8192, 16384, },
  { 1, 32, 128, 256, 512, 8192, 16384, },
  { 2, 32, 128, 256, 512, 8192, 16384, },
  { 1, 2, 32, 128, 256, 512, 8192, 16384, },
  { 4, 32, 128, 256, 512, 8192, 16384, },
  { 1, 4, 32, 128, 256, 512, 8192, 16384, },
  { 2, 4, 32, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 8192, 16384, },
  { 8, 32, 128, 256, 512, 8192, 16384, },
  { 1, 8, 32, 128, 256, 512, 8192, 16384, },
  { 2, 8, 32, 128, 256, 512, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 8192, 16384, },
  { 4, 8, 32, 128, 256, 512, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 8192, 16384, },
  { 16, 32, 128, 256, 512, 8192, 16384, },
  { 1, 16, 32, 128, 256, 512, 8192, 16384, },
  { 2, 16, 32, 128, 256, 512, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 8192, 16384, },
  { 4, 16, 32, 128, 256, 512, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 8192, 16384, },
  { 8, 16, 32, 128, 256, 512, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 8192, 16384, },
  { 64, 128, 256, 512, 8192, 16384, },
  { 1, 64, 128, 256, 512, 8192, 16384, },
  { 2, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 64, 128, 256, 512, 8192, 16384, },
  { 4, 64, 128, 256, 512, 8192, 16384, },
  { 1, 4, 64, 128, 256, 512, 8192, 16384, },
  { 2, 4, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 8192, 16384, },
  { 8, 64, 128, 256, 512, 8192, 16384, },
  { 1, 8, 64, 128, 256, 512, 8192, 16384, },
  { 2, 8, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 8192, 16384, },
  { 4, 8, 64, 128, 256, 512, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 8192, 16384, },
  { 16, 64, 128, 256, 512, 8192, 16384, },
  { 1, 16, 64, 128, 256, 512, 8192, 16384, },
  { 2, 16, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 8192, 16384, },
  { 4, 16, 64, 128, 256, 512, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 8192, 16384, },
  { 8, 16, 64, 128, 256, 512, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 8192, 16384, },
  { 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 32, 64, 128, 256, 512, 8192, 16384, },
  { 2, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 8192, 16384, },
  { 4, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 8192, 16384, },
  { 8, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 8192, 16384, },
  { 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, },
  { 1024, 8192, 16384, },
  { 1, 1024, 8192, 16384, },
  { 2, 1024, 8192, 16384, },
  { 1, 2, 1024, 8192, 16384, },
  { 4, 1024, 8192, 16384, },
  { 1, 4, 1024, 8192, 16384, },
  { 2, 4, 1024, 8192, 16384, },
  { 1, 2, 4, 1024, 8192, 16384, },
  { 8, 1024, 8192, 16384, },
  { 1, 8, 1024, 8192, 16384, },
  { 2, 8, 1024, 8192, 16384, },
  { 1, 2, 8, 1024, 8192, 16384, },
  { 4, 8, 1024, 8192, 16384, },
  { 1, 4, 8, 1024, 8192, 16384, },
  { 2, 4, 8, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 1024, 8192, 16384, },
  { 16, 1024, 8192, 16384, },
  { 1, 16, 1024, 8192, 16384, },
  { 2, 16, 1024, 8192, 16384, },
  { 1, 2, 16, 1024, 8192, 16384, },
  { 4, 16, 1024, 8192, 16384, },
  { 1, 4, 16, 1024, 8192, 16384, },
  { 2, 4, 16, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 1024, 8192, 16384, },
  { 8, 16, 1024, 8192, 16384, },
  { 1, 8, 16, 1024, 8192, 16384, },
  { 2, 8, 16, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 1024, 8192, 16384, },
  { 4, 8, 16, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 1024, 8192, 16384, },
  { 32, 1024, 8192, 16384, },
  { 1, 32, 1024, 8192, 16384, },
  { 2, 32, 1024, 8192, 16384, },
  { 1, 2, 32, 1024, 8192, 16384, },
  { 4, 32, 1024, 8192, 16384, },
  { 1, 4, 32, 1024, 8192, 16384, },
  { 2, 4, 32, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 1024, 8192, 16384, },
  { 8, 32, 1024, 8192, 16384, },
  { 1, 8, 32, 1024, 8192, 16384, },
  { 2, 8, 32, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 1024, 8192, 16384, },
  { 4, 8, 32, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 1024, 8192, 16384, },
  { 16, 32, 1024, 8192, 16384, },
  { 1, 16, 32, 1024, 8192, 16384, },
  { 2, 16, 32, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 1024, 8192, 16384, },
  { 4, 16, 32, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 1024, 8192, 16384, },
  { 8, 16, 32, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 1024, 8192, 16384, },
  { 64, 1024, 8192, 16384, },
  { 1, 64, 1024, 8192, 16384, },
  { 2, 64, 1024, 8192, 16384, },
  { 1, 2, 64, 1024, 8192, 16384, },
  { 4, 64, 1024, 8192, 16384, },
  { 1, 4, 64, 1024, 8192, 16384, },
  { 2, 4, 64, 1024, 8192, 16384, },
  { 1, 2, 4, 64, 1024, 8192, 16384, },
  { 8, 64, 1024, 8192, 16384, },
  { 1, 8, 64, 1024, 8192, 16384, },
  { 2, 8, 64, 1024, 8192, 16384, },
  { 1, 2, 8, 64, 1024, 8192, 16384, },
  { 4, 8, 64, 1024, 8192, 16384, },
  { 1, 4, 8, 64, 1024, 8192, 16384, },
  { 2, 4, 8, 64, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 64, 1024, 8192, 16384, },
  { 16, 64, 1024, 8192, 16384, },
  { 1, 16, 64, 1024, 8192, 16384, },
  { 2, 16, 64, 1024, 8192, 16384, },
  { 1, 2, 16, 64, 1024, 8192, 16384, },
  { 4, 16, 64, 1024, 8192, 16384, },
  { 1, 4, 16, 64, 1024, 8192, 16384, },
  { 2, 4, 16, 64, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 64, 1024, 8192, 16384, },
  { 8, 16, 64, 1024, 8192, 16384, },
  { 1, 8, 16, 64, 1024, 8192, 16384, },
  { 2, 8, 16, 64, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 64, 1024, 8192, 16384, },
  { 4, 8, 16, 64, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 64, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 64, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 1024, 8192, 16384, },
  { 32, 64, 1024, 8192, 16384, },
  { 1, 32, 64, 1024, 8192, 16384, },
  { 2, 32, 64, 1024, 8192, 16384, },
  { 1, 2, 32, 64, 1024, 8192, 16384, },
  { 4, 32, 64, 1024, 8192, 16384, },
  { 1, 4, 32, 64, 1024, 8192, 16384, },
  { 2, 4, 32, 64, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 64, 1024, 8192, 16384, },
  { 8, 32, 64, 1024, 8192, 16384, },
  { 1, 8, 32, 64, 1024, 8192, 16384, },
  { 2, 8, 32, 64, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 64, 1024, 8192, 16384, },
  { 4, 8, 32, 64, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 64, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 64, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 1024, 8192, 16384, },
  { 16, 32, 64, 1024, 8192, 16384, },
  { 1, 16, 32, 64, 1024, 8192, 16384, },
  { 2, 16, 32, 64, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 64, 1024, 8192, 16384, },
  { 4, 16, 32, 64, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 64, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 64, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 1024, 8192, 16384, },
  { 8, 16, 32, 64, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 64, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 64, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 64, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 8192, 16384, },
  { 128, 1024, 8192, 16384, },
  { 1, 128, 1024, 8192, 16384, },
  { 2, 128, 1024, 8192, 16384, },
  { 1, 2, 128, 1024, 8192, 16384, },
  { 4, 128, 1024, 8192, 16384, },
  { 1, 4, 128, 1024, 8192, 16384, },
  { 2, 4, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 128, 1024, 8192, 16384, },
  { 8, 128, 1024, 8192, 16384, },
  { 1, 8, 128, 1024, 8192, 16384, },
  { 2, 8, 128, 1024, 8192, 16384, },
  { 1, 2, 8, 128, 1024, 8192, 16384, },
  { 4, 8, 128, 1024, 8192, 16384, },
  { 1, 4, 8, 128, 1024, 8192, 16384, },
  { 2, 4, 8, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 128, 1024, 8192, 16384, },
  { 16, 128, 1024, 8192, 16384, },
  { 1, 16, 128, 1024, 8192, 16384, },
  { 2, 16, 128, 1024, 8192, 16384, },
  { 1, 2, 16, 128, 1024, 8192, 16384, },
  { 4, 16, 128, 1024, 8192, 16384, },
  { 1, 4, 16, 128, 1024, 8192, 16384, },
  { 2, 4, 16, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 128, 1024, 8192, 16384, },
  { 8, 16, 128, 1024, 8192, 16384, },
  { 1, 8, 16, 128, 1024, 8192, 16384, },
  { 2, 8, 16, 128, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 128, 1024, 8192, 16384, },
  { 4, 8, 16, 128, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 128, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 1024, 8192, 16384, },
  { 32, 128, 1024, 8192, 16384, },
  { 1, 32, 128, 1024, 8192, 16384, },
  { 2, 32, 128, 1024, 8192, 16384, },
  { 1, 2, 32, 128, 1024, 8192, 16384, },
  { 4, 32, 128, 1024, 8192, 16384, },
  { 1, 4, 32, 128, 1024, 8192, 16384, },
  { 2, 4, 32, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 128, 1024, 8192, 16384, },
  { 8, 32, 128, 1024, 8192, 16384, },
  { 1, 8, 32, 128, 1024, 8192, 16384, },
  { 2, 8, 32, 128, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 128, 1024, 8192, 16384, },
  { 4, 8, 32, 128, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 128, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 1024, 8192, 16384, },
  { 16, 32, 128, 1024, 8192, 16384, },
  { 1, 16, 32, 128, 1024, 8192, 16384, },
  { 2, 16, 32, 128, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 128, 1024, 8192, 16384, },
  { 4, 16, 32, 128, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 128, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 1024, 8192, 16384, },
  { 8, 16, 32, 128, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 128, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 128, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 128, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 8192, 16384, },
  { 64, 128, 1024, 8192, 16384, },
  { 1, 64, 128, 1024, 8192, 16384, },
  { 2, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 64, 128, 1024, 8192, 16384, },
  { 4, 64, 128, 1024, 8192, 16384, },
  { 1, 4, 64, 128, 1024, 8192, 16384, },
  { 2, 4, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 64, 128, 1024, 8192, 16384, },
  { 8, 64, 128, 1024, 8192, 16384, },
  { 1, 8, 64, 128, 1024, 8192, 16384, },
  { 2, 8, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 8, 64, 128, 1024, 8192, 16384, },
  { 4, 8, 64, 128, 1024, 8192, 16384, },
  { 1, 4, 8, 64, 128, 1024, 8192, 16384, },
  { 2, 4, 8, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 1024, 8192, 16384, },
  { 16, 64, 128, 1024, 8192, 16384, },
  { 1, 16, 64, 128, 1024, 8192, 16384, },
  { 2, 16, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 16, 64, 128, 1024, 8192, 16384, },
  { 4, 16, 64, 128, 1024, 8192, 16384, },
  { 1, 4, 16, 64, 128, 1024, 8192, 16384, },
  { 2, 4, 16, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 1024, 8192, 16384, },
  { 8, 16, 64, 128, 1024, 8192, 16384, },
  { 1, 8, 16, 64, 128, 1024, 8192, 16384, },
  { 2, 8, 16, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 1024, 8192, 16384, },
  { 4, 8, 16, 64, 128, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 8192, 16384, },
  { 32, 64, 128, 1024, 8192, 16384, },
  { 1, 32, 64, 128, 1024, 8192, 16384, },
  { 2, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 32, 64, 128, 1024, 8192, 16384, },
  { 4, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 4, 32, 64, 128, 1024, 8192, 16384, },
  { 2, 4, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 1024, 8192, 16384, },
  { 8, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 8, 32, 64, 128, 1024, 8192, 16384, },
  { 2, 8, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 1024, 8192, 16384, },
  { 4, 8, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 8192, 16384, },
  { 16, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 2, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 4, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 8, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 8192, 16384, },
  { 256, 1024, 8192, 16384, },
  { 1, 256, 1024, 8192, 16384, },
  { 2, 256, 1024, 8192, 16384, },
  { 1, 2, 256, 1024, 8192, 16384, },
  { 4, 256, 1024, 8192, 16384, },
  { 1, 4, 256, 1024, 8192, 16384, },
  { 2, 4, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 256, 1024, 8192, 16384, },
  { 8, 256, 1024, 8192, 16384, },
  { 1, 8, 256, 1024, 8192, 16384, },
  { 2, 8, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 256, 1024, 8192, 16384, },
  { 4, 8, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 256, 1024, 8192, 16384, },
  { 16, 256, 1024, 8192, 16384, },
  { 1, 16, 256, 1024, 8192, 16384, },
  { 2, 16, 256, 1024, 8192, 16384, },
  { 1, 2, 16, 256, 1024, 8192, 16384, },
  { 4, 16, 256, 1024, 8192, 16384, },
  { 1, 4, 16, 256, 1024, 8192, 16384, },
  { 2, 4, 16, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 256, 1024, 8192, 16384, },
  { 8, 16, 256, 1024, 8192, 16384, },
  { 1, 8, 16, 256, 1024, 8192, 16384, },
  { 2, 8, 16, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 256, 1024, 8192, 16384, },
  { 4, 8, 16, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 1024, 8192, 16384, },
  { 32, 256, 1024, 8192, 16384, },
  { 1, 32, 256, 1024, 8192, 16384, },
  { 2, 32, 256, 1024, 8192, 16384, },
  { 1, 2, 32, 256, 1024, 8192, 16384, },
  { 4, 32, 256, 1024, 8192, 16384, },
  { 1, 4, 32, 256, 1024, 8192, 16384, },
  { 2, 4, 32, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 256, 1024, 8192, 16384, },
  { 8, 32, 256, 1024, 8192, 16384, },
  { 1, 8, 32, 256, 1024, 8192, 16384, },
  { 2, 8, 32, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 256, 1024, 8192, 16384, },
  { 4, 8, 32, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 1024, 8192, 16384, },
  { 16, 32, 256, 1024, 8192, 16384, },
  { 1, 16, 32, 256, 1024, 8192, 16384, },
  { 2, 16, 32, 256, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 256, 1024, 8192, 16384, },
  { 4, 16, 32, 256, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 256, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 1024, 8192, 16384, },
  { 8, 16, 32, 256, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 256, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 8192, 16384, },
  { 64, 256, 1024, 8192, 16384, },
  { 1, 64, 256, 1024, 8192, 16384, },
  { 2, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 64, 256, 1024, 8192, 16384, },
  { 4, 64, 256, 1024, 8192, 16384, },
  { 1, 4, 64, 256, 1024, 8192, 16384, },
  { 2, 4, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 64, 256, 1024, 8192, 16384, },
  { 8, 64, 256, 1024, 8192, 16384, },
  { 1, 8, 64, 256, 1024, 8192, 16384, },
  { 2, 8, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 64, 256, 1024, 8192, 16384, },
  { 4, 8, 64, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 64, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 1024, 8192, 16384, },
  { 16, 64, 256, 1024, 8192, 16384, },
  { 1, 16, 64, 256, 1024, 8192, 16384, },
  { 2, 16, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 16, 64, 256, 1024, 8192, 16384, },
  { 4, 16, 64, 256, 1024, 8192, 16384, },
  { 1, 4, 16, 64, 256, 1024, 8192, 16384, },
  { 2, 4, 16, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 1024, 8192, 16384, },
  { 8, 16, 64, 256, 1024, 8192, 16384, },
  { 1, 8, 16, 64, 256, 1024, 8192, 16384, },
  { 2, 8, 16, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 1024, 8192, 16384, },
  { 4, 8, 16, 64, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 8192, 16384, },
  { 32, 64, 256, 1024, 8192, 16384, },
  { 1, 32, 64, 256, 1024, 8192, 16384, },
  { 2, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 32, 64, 256, 1024, 8192, 16384, },
  { 4, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 4, 32, 64, 256, 1024, 8192, 16384, },
  { 2, 4, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 1024, 8192, 16384, },
  { 8, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 8, 32, 64, 256, 1024, 8192, 16384, },
  { 2, 8, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 1024, 8192, 16384, },
  { 4, 8, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 8192, 16384, },
  { 16, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 2, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 4, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 8, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 8192, 16384, },
  { 128, 256, 1024, 8192, 16384, },
  { 1, 128, 256, 1024, 8192, 16384, },
  { 2, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 128, 256, 1024, 8192, 16384, },
  { 4, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 128, 256, 1024, 8192, 16384, },
  { 8, 128, 256, 1024, 8192, 16384, },
  { 1, 8, 128, 256, 1024, 8192, 16384, },
  { 2, 8, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 128, 256, 1024, 8192, 16384, },
  { 4, 8, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 1024, 8192, 16384, },
  { 16, 128, 256, 1024, 8192, 16384, },
  { 1, 16, 128, 256, 1024, 8192, 16384, },
  { 2, 16, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 16, 128, 256, 1024, 8192, 16384, },
  { 4, 16, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 16, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 16, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 1024, 8192, 16384, },
  { 8, 16, 128, 256, 1024, 8192, 16384, },
  { 1, 8, 16, 128, 256, 1024, 8192, 16384, },
  { 2, 8, 16, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 1024, 8192, 16384, },
  { 4, 8, 16, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 8192, 16384, },
  { 32, 128, 256, 1024, 8192, 16384, },
  { 1, 32, 128, 256, 1024, 8192, 16384, },
  { 2, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 32, 128, 256, 1024, 8192, 16384, },
  { 4, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 32, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 1024, 8192, 16384, },
  { 8, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 8, 32, 128, 256, 1024, 8192, 16384, },
  { 2, 8, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 1024, 8192, 16384, },
  { 4, 8, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 8192, 16384, },
  { 16, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 2, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 4, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 8, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 8192, 16384, },
  { 64, 128, 256, 1024, 8192, 16384, },
  { 1, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 64, 128, 256, 1024, 8192, 16384, },
  { 4, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 1024, 8192, 16384, },
  { 8, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 8, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 8, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 1024, 8192, 16384, },
  { 4, 8, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 8192, 16384, },
  { 16, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 4, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 8, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 8192, 16384, },
  { 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 4, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 8, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, },
  { 512, 1024, 8192, 16384, },
  { 1, 512, 1024, 8192, 16384, },
  { 2, 512, 1024, 8192, 16384, },
  { 1, 2, 512, 1024, 8192, 16384, },
  { 4, 512, 1024, 8192, 16384, },
  { 1, 4, 512, 1024, 8192, 16384, },
  { 2, 4, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 512, 1024, 8192, 16384, },
  { 8, 512, 1024, 8192, 16384, },
  { 1, 8, 512, 1024, 8192, 16384, },
  { 2, 8, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 512, 1024, 8192, 16384, },
  { 4, 8, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 512, 1024, 8192, 16384, },
  { 16, 512, 1024, 8192, 16384, },
  { 1, 16, 512, 1024, 8192, 16384, },
  { 2, 16, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 512, 1024, 8192, 16384, },
  { 4, 16, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 512, 1024, 8192, 16384, },
  { 8, 16, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 512, 1024, 8192, 16384, },
  { 32, 512, 1024, 8192, 16384, },
  { 1, 32, 512, 1024, 8192, 16384, },
  { 2, 32, 512, 1024, 8192, 16384, },
  { 1, 2, 32, 512, 1024, 8192, 16384, },
  { 4, 32, 512, 1024, 8192, 16384, },
  { 1, 4, 32, 512, 1024, 8192, 16384, },
  { 2, 4, 32, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 512, 1024, 8192, 16384, },
  { 8, 32, 512, 1024, 8192, 16384, },
  { 1, 8, 32, 512, 1024, 8192, 16384, },
  { 2, 8, 32, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 512, 1024, 8192, 16384, },
  { 4, 8, 32, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 512, 1024, 8192, 16384, },
  { 16, 32, 512, 1024, 8192, 16384, },
  { 1, 16, 32, 512, 1024, 8192, 16384, },
  { 2, 16, 32, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 512, 1024, 8192, 16384, },
  { 4, 16, 32, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 512, 1024, 8192, 16384, },
  { 8, 16, 32, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 8192, 16384, },
  { 64, 512, 1024, 8192, 16384, },
  { 1, 64, 512, 1024, 8192, 16384, },
  { 2, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 64, 512, 1024, 8192, 16384, },
  { 4, 64, 512, 1024, 8192, 16384, },
  { 1, 4, 64, 512, 1024, 8192, 16384, },
  { 2, 4, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 64, 512, 1024, 8192, 16384, },
  { 8, 64, 512, 1024, 8192, 16384, },
  { 1, 8, 64, 512, 1024, 8192, 16384, },
  { 2, 8, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 64, 512, 1024, 8192, 16384, },
  { 4, 8, 64, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 64, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 64, 512, 1024, 8192, 16384, },
  { 16, 64, 512, 1024, 8192, 16384, },
  { 1, 16, 64, 512, 1024, 8192, 16384, },
  { 2, 16, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 64, 512, 1024, 8192, 16384, },
  { 4, 16, 64, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 64, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 64, 512, 1024, 8192, 16384, },
  { 8, 16, 64, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 64, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 64, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 64, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 64, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 8192, 16384, },
  { 32, 64, 512, 1024, 8192, 16384, },
  { 1, 32, 64, 512, 1024, 8192, 16384, },
  { 2, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 32, 64, 512, 1024, 8192, 16384, },
  { 4, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 4, 32, 64, 512, 1024, 8192, 16384, },
  { 2, 4, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 64, 512, 1024, 8192, 16384, },
  { 8, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 8, 32, 64, 512, 1024, 8192, 16384, },
  { 2, 8, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 64, 512, 1024, 8192, 16384, },
  { 4, 8, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 64, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 8192, 16384, },
  { 16, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 2, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 4, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 8, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 8192, 16384, },
  { 128, 512, 1024, 8192, 16384, },
  { 1, 128, 512, 1024, 8192, 16384, },
  { 2, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 128, 512, 1024, 8192, 16384, },
  { 4, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 128, 512, 1024, 8192, 16384, },
  { 8, 128, 512, 1024, 8192, 16384, },
  { 1, 8, 128, 512, 1024, 8192, 16384, },
  { 2, 8, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 128, 512, 1024, 8192, 16384, },
  { 4, 8, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 128, 512, 1024, 8192, 16384, },
  { 16, 128, 512, 1024, 8192, 16384, },
  { 1, 16, 128, 512, 1024, 8192, 16384, },
  { 2, 16, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 128, 512, 1024, 8192, 16384, },
  { 4, 16, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 128, 512, 1024, 8192, 16384, },
  { 8, 16, 128, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 128, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 128, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 8192, 16384, },
  { 32, 128, 512, 1024, 8192, 16384, },
  { 1, 32, 128, 512, 1024, 8192, 16384, },
  { 2, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 32, 128, 512, 1024, 8192, 16384, },
  { 4, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 32, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 128, 512, 1024, 8192, 16384, },
  { 8, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 8, 32, 128, 512, 1024, 8192, 16384, },
  { 2, 8, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 128, 512, 1024, 8192, 16384, },
  { 4, 8, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 8192, 16384, },
  { 16, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 2, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 4, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 8, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 8192, 16384, },
  { 64, 128, 512, 1024, 8192, 16384, },
  { 1, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 64, 128, 512, 1024, 8192, 16384, },
  { 4, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 64, 128, 512, 1024, 8192, 16384, },
  { 8, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 8, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 8, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 64, 128, 512, 1024, 8192, 16384, },
  { 4, 8, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 8192, 16384, },
  { 16, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 4, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 8, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 8192, 16384, },
  { 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 4, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 8, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 8, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 8, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 4, 8, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 4, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, },
  { 256, 512, 1024, 8192, 16384, },
  { 1, 256, 512, 1024, 8192, 16384, },
  { 2, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 256, 512, 1024, 8192, 16384, },
  { 4, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 256, 512, 1024, 8192, 16384, },
  { 8, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 256, 512, 1024, 8192, 16384, },
  { 16, 256, 512, 1024, 8192, 16384, },
  { 1, 16, 256, 512, 1024, 8192, 16384, },
  { 2, 16, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 256, 512, 1024, 8192, 16384, },
  { 4, 16, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 256, 512, 1024, 8192, 16384, },
  { 8, 16, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 8192, 16384, },
  { 32, 256, 512, 1024, 8192, 16384, },
  { 1, 32, 256, 512, 1024, 8192, 16384, },
  { 2, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 32, 256, 512, 1024, 8192, 16384, },
  { 4, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 32, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 256, 512, 1024, 8192, 16384, },
  { 8, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 32, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 8192, 16384, },
  { 16, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 2, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 4, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 8, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 8192, 16384, },
  { 64, 256, 512, 1024, 8192, 16384, },
  { 1, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 64, 256, 512, 1024, 8192, 16384, },
  { 4, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 64, 256, 512, 1024, 8192, 16384, },
  { 8, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 64, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 8192, 16384, },
  { 16, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 4, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 8, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 8192, 16384, },
  { 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 4, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 8, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 4, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, },
  { 128, 256, 512, 1024, 8192, 16384, },
  { 1, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 128, 256, 512, 1024, 8192, 16384, },
  { 8, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 8192, 16384, },
  { 16, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 8, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 8192, 16384, },
  { 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 8, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, },
  { 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 8, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, },
  { 2048, 8192, 16384, },
  { 1, 2048, 8192, 16384, },
  { 2, 2048, 8192, 16384, },
  { 1, 2, 2048, 8192, 16384, },
  { 4, 2048, 8192, 16384, },
  { 1, 4, 2048, 8192, 16384, },
  { 2, 4, 2048, 8192, 16384, },
  { 1, 2, 4, 2048, 8192, 16384, },
  { 8, 2048, 8192, 16384, },
  { 1, 8, 2048, 8192, 16384, },
  { 2, 8, 2048, 8192, 16384, },
  { 1, 2, 8, 2048, 8192, 16384, },
  { 4, 8, 2048, 8192, 16384, },
  { 1, 4, 8, 2048, 8192, 16384, },
  { 2, 4, 8, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 2048, 8192, 16384, },
  { 16, 2048, 8192, 16384, },
  { 1, 16, 2048, 8192, 16384, },
  { 2, 16, 2048, 8192, 16384, },
  { 1, 2, 16, 2048, 8192, 16384, },
  { 4, 16, 2048, 8192, 16384, },
  { 1, 4, 16, 2048, 8192, 16384, },
  { 2, 4, 16, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 2048, 8192, 16384, },
  { 8, 16, 2048, 8192, 16384, },
  { 1, 8, 16, 2048, 8192, 16384, },
  { 2, 8, 16, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 2048, 8192, 16384, },
  { 4, 8, 16, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 2048, 8192, 16384, },
  { 32, 2048, 8192, 16384, },
  { 1, 32, 2048, 8192, 16384, },
  { 2, 32, 2048, 8192, 16384, },
  { 1, 2, 32, 2048, 8192, 16384, },
  { 4, 32, 2048, 8192, 16384, },
  { 1, 4, 32, 2048, 8192, 16384, },
  { 2, 4, 32, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 2048, 8192, 16384, },
  { 8, 32, 2048, 8192, 16384, },
  { 1, 8, 32, 2048, 8192, 16384, },
  { 2, 8, 32, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 2048, 8192, 16384, },
  { 4, 8, 32, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 2048, 8192, 16384, },
  { 16, 32, 2048, 8192, 16384, },
  { 1, 16, 32, 2048, 8192, 16384, },
  { 2, 16, 32, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 2048, 8192, 16384, },
  { 4, 16, 32, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 2048, 8192, 16384, },
  { 8, 16, 32, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 2048, 8192, 16384, },
  { 64, 2048, 8192, 16384, },
  { 1, 64, 2048, 8192, 16384, },
  { 2, 64, 2048, 8192, 16384, },
  { 1, 2, 64, 2048, 8192, 16384, },
  { 4, 64, 2048, 8192, 16384, },
  { 1, 4, 64, 2048, 8192, 16384, },
  { 2, 4, 64, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 2048, 8192, 16384, },
  { 8, 64, 2048, 8192, 16384, },
  { 1, 8, 64, 2048, 8192, 16384, },
  { 2, 8, 64, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 2048, 8192, 16384, },
  { 4, 8, 64, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 2048, 8192, 16384, },
  { 16, 64, 2048, 8192, 16384, },
  { 1, 16, 64, 2048, 8192, 16384, },
  { 2, 16, 64, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 2048, 8192, 16384, },
  { 4, 16, 64, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 2048, 8192, 16384, },
  { 8, 16, 64, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 2048, 8192, 16384, },
  { 32, 64, 2048, 8192, 16384, },
  { 1, 32, 64, 2048, 8192, 16384, },
  { 2, 32, 64, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 2048, 8192, 16384, },
  { 4, 32, 64, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 2048, 8192, 16384, },
  { 8, 32, 64, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 2048, 8192, 16384, },
  { 16, 32, 64, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 8192, 16384, },
  { 128, 2048, 8192, 16384, },
  { 1, 128, 2048, 8192, 16384, },
  { 2, 128, 2048, 8192, 16384, },
  { 1, 2, 128, 2048, 8192, 16384, },
  { 4, 128, 2048, 8192, 16384, },
  { 1, 4, 128, 2048, 8192, 16384, },
  { 2, 4, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 128, 2048, 8192, 16384, },
  { 8, 128, 2048, 8192, 16384, },
  { 1, 8, 128, 2048, 8192, 16384, },
  { 2, 8, 128, 2048, 8192, 16384, },
  { 1, 2, 8, 128, 2048, 8192, 16384, },
  { 4, 8, 128, 2048, 8192, 16384, },
  { 1, 4, 8, 128, 2048, 8192, 16384, },
  { 2, 4, 8, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 128, 2048, 8192, 16384, },
  { 16, 128, 2048, 8192, 16384, },
  { 1, 16, 128, 2048, 8192, 16384, },
  { 2, 16, 128, 2048, 8192, 16384, },
  { 1, 2, 16, 128, 2048, 8192, 16384, },
  { 4, 16, 128, 2048, 8192, 16384, },
  { 1, 4, 16, 128, 2048, 8192, 16384, },
  { 2, 4, 16, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 128, 2048, 8192, 16384, },
  { 8, 16, 128, 2048, 8192, 16384, },
  { 1, 8, 16, 128, 2048, 8192, 16384, },
  { 2, 8, 16, 128, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 128, 2048, 8192, 16384, },
  { 4, 8, 16, 128, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 128, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 2048, 8192, 16384, },
  { 32, 128, 2048, 8192, 16384, },
  { 1, 32, 128, 2048, 8192, 16384, },
  { 2, 32, 128, 2048, 8192, 16384, },
  { 1, 2, 32, 128, 2048, 8192, 16384, },
  { 4, 32, 128, 2048, 8192, 16384, },
  { 1, 4, 32, 128, 2048, 8192, 16384, },
  { 2, 4, 32, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 128, 2048, 8192, 16384, },
  { 8, 32, 128, 2048, 8192, 16384, },
  { 1, 8, 32, 128, 2048, 8192, 16384, },
  { 2, 8, 32, 128, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 128, 2048, 8192, 16384, },
  { 4, 8, 32, 128, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 128, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 2048, 8192, 16384, },
  { 16, 32, 128, 2048, 8192, 16384, },
  { 1, 16, 32, 128, 2048, 8192, 16384, },
  { 2, 16, 32, 128, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 128, 2048, 8192, 16384, },
  { 4, 16, 32, 128, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 128, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 2048, 8192, 16384, },
  { 8, 16, 32, 128, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 128, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 128, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 128, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 8192, 16384, },
  { 64, 128, 2048, 8192, 16384, },
  { 1, 64, 128, 2048, 8192, 16384, },
  { 2, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 64, 128, 2048, 8192, 16384, },
  { 4, 64, 128, 2048, 8192, 16384, },
  { 1, 4, 64, 128, 2048, 8192, 16384, },
  { 2, 4, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 128, 2048, 8192, 16384, },
  { 8, 64, 128, 2048, 8192, 16384, },
  { 1, 8, 64, 128, 2048, 8192, 16384, },
  { 2, 8, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 128, 2048, 8192, 16384, },
  { 4, 8, 64, 128, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 128, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 2048, 8192, 16384, },
  { 16, 64, 128, 2048, 8192, 16384, },
  { 1, 16, 64, 128, 2048, 8192, 16384, },
  { 2, 16, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 128, 2048, 8192, 16384, },
  { 4, 16, 64, 128, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 128, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 2048, 8192, 16384, },
  { 8, 16, 64, 128, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 128, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 128, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 8192, 16384, },
  { 32, 64, 128, 2048, 8192, 16384, },
  { 1, 32, 64, 128, 2048, 8192, 16384, },
  { 2, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 128, 2048, 8192, 16384, },
  { 4, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 128, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 2048, 8192, 16384, },
  { 8, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 128, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 8192, 16384, },
  { 16, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 8192, 16384, },
  { 256, 2048, 8192, 16384, },
  { 1, 256, 2048, 8192, 16384, },
  { 2, 256, 2048, 8192, 16384, },
  { 1, 2, 256, 2048, 8192, 16384, },
  { 4, 256, 2048, 8192, 16384, },
  { 1, 4, 256, 2048, 8192, 16384, },
  { 2, 4, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 256, 2048, 8192, 16384, },
  { 8, 256, 2048, 8192, 16384, },
  { 1, 8, 256, 2048, 8192, 16384, },
  { 2, 8, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 256, 2048, 8192, 16384, },
  { 4, 8, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 256, 2048, 8192, 16384, },
  { 16, 256, 2048, 8192, 16384, },
  { 1, 16, 256, 2048, 8192, 16384, },
  { 2, 16, 256, 2048, 8192, 16384, },
  { 1, 2, 16, 256, 2048, 8192, 16384, },
  { 4, 16, 256, 2048, 8192, 16384, },
  { 1, 4, 16, 256, 2048, 8192, 16384, },
  { 2, 4, 16, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 256, 2048, 8192, 16384, },
  { 8, 16, 256, 2048, 8192, 16384, },
  { 1, 8, 16, 256, 2048, 8192, 16384, },
  { 2, 8, 16, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 256, 2048, 8192, 16384, },
  { 4, 8, 16, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 2048, 8192, 16384, },
  { 32, 256, 2048, 8192, 16384, },
  { 1, 32, 256, 2048, 8192, 16384, },
  { 2, 32, 256, 2048, 8192, 16384, },
  { 1, 2, 32, 256, 2048, 8192, 16384, },
  { 4, 32, 256, 2048, 8192, 16384, },
  { 1, 4, 32, 256, 2048, 8192, 16384, },
  { 2, 4, 32, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 256, 2048, 8192, 16384, },
  { 8, 32, 256, 2048, 8192, 16384, },
  { 1, 8, 32, 256, 2048, 8192, 16384, },
  { 2, 8, 32, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 256, 2048, 8192, 16384, },
  { 4, 8, 32, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 2048, 8192, 16384, },
  { 16, 32, 256, 2048, 8192, 16384, },
  { 1, 16, 32, 256, 2048, 8192, 16384, },
  { 2, 16, 32, 256, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 256, 2048, 8192, 16384, },
  { 4, 16, 32, 256, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 256, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 2048, 8192, 16384, },
  { 8, 16, 32, 256, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 256, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 8192, 16384, },
  { 64, 256, 2048, 8192, 16384, },
  { 1, 64, 256, 2048, 8192, 16384, },
  { 2, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 64, 256, 2048, 8192, 16384, },
  { 4, 64, 256, 2048, 8192, 16384, },
  { 1, 4, 64, 256, 2048, 8192, 16384, },
  { 2, 4, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 256, 2048, 8192, 16384, },
  { 8, 64, 256, 2048, 8192, 16384, },
  { 1, 8, 64, 256, 2048, 8192, 16384, },
  { 2, 8, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 256, 2048, 8192, 16384, },
  { 4, 8, 64, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 2048, 8192, 16384, },
  { 16, 64, 256, 2048, 8192, 16384, },
  { 1, 16, 64, 256, 2048, 8192, 16384, },
  { 2, 16, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 256, 2048, 8192, 16384, },
  { 4, 16, 64, 256, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 256, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 2048, 8192, 16384, },
  { 8, 16, 64, 256, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 256, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 8192, 16384, },
  { 32, 64, 256, 2048, 8192, 16384, },
  { 1, 32, 64, 256, 2048, 8192, 16384, },
  { 2, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 256, 2048, 8192, 16384, },
  { 4, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 256, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 2048, 8192, 16384, },
  { 8, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 256, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 8192, 16384, },
  { 16, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 8192, 16384, },
  { 128, 256, 2048, 8192, 16384, },
  { 1, 128, 256, 2048, 8192, 16384, },
  { 2, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 128, 256, 2048, 8192, 16384, },
  { 4, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 128, 256, 2048, 8192, 16384, },
  { 8, 128, 256, 2048, 8192, 16384, },
  { 1, 8, 128, 256, 2048, 8192, 16384, },
  { 2, 8, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 128, 256, 2048, 8192, 16384, },
  { 4, 8, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 2048, 8192, 16384, },
  { 16, 128, 256, 2048, 8192, 16384, },
  { 1, 16, 128, 256, 2048, 8192, 16384, },
  { 2, 16, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 16, 128, 256, 2048, 8192, 16384, },
  { 4, 16, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 16, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 16, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 2048, 8192, 16384, },
  { 8, 16, 128, 256, 2048, 8192, 16384, },
  { 1, 8, 16, 128, 256, 2048, 8192, 16384, },
  { 2, 8, 16, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 2048, 8192, 16384, },
  { 4, 8, 16, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 8192, 16384, },
  { 32, 128, 256, 2048, 8192, 16384, },
  { 1, 32, 128, 256, 2048, 8192, 16384, },
  { 2, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 32, 128, 256, 2048, 8192, 16384, },
  { 4, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 32, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 2048, 8192, 16384, },
  { 8, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 8, 32, 128, 256, 2048, 8192, 16384, },
  { 2, 8, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 2048, 8192, 16384, },
  { 4, 8, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 8192, 16384, },
  { 16, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 2, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 4, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 8, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 8192, 16384, },
  { 64, 128, 256, 2048, 8192, 16384, },
  { 1, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 64, 128, 256, 2048, 8192, 16384, },
  { 4, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 2048, 8192, 16384, },
  { 8, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 8, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 8, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 2048, 8192, 16384, },
  { 4, 8, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 8192, 16384, },
  { 16, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 4, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 8, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 8192, 16384, },
  { 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 4, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 8, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, },
  { 512, 2048, 8192, 16384, },
  { 1, 512, 2048, 8192, 16384, },
  { 2, 512, 2048, 8192, 16384, },
  { 1, 2, 512, 2048, 8192, 16384, },
  { 4, 512, 2048, 8192, 16384, },
  { 1, 4, 512, 2048, 8192, 16384, },
  { 2, 4, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 512, 2048, 8192, 16384, },
  { 8, 512, 2048, 8192, 16384, },
  { 1, 8, 512, 2048, 8192, 16384, },
  { 2, 8, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 512, 2048, 8192, 16384, },
  { 4, 8, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 512, 2048, 8192, 16384, },
  { 16, 512, 2048, 8192, 16384, },
  { 1, 16, 512, 2048, 8192, 16384, },
  { 2, 16, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 512, 2048, 8192, 16384, },
  { 4, 16, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 512, 2048, 8192, 16384, },
  { 8, 16, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 512, 2048, 8192, 16384, },
  { 32, 512, 2048, 8192, 16384, },
  { 1, 32, 512, 2048, 8192, 16384, },
  { 2, 32, 512, 2048, 8192, 16384, },
  { 1, 2, 32, 512, 2048, 8192, 16384, },
  { 4, 32, 512, 2048, 8192, 16384, },
  { 1, 4, 32, 512, 2048, 8192, 16384, },
  { 2, 4, 32, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 512, 2048, 8192, 16384, },
  { 8, 32, 512, 2048, 8192, 16384, },
  { 1, 8, 32, 512, 2048, 8192, 16384, },
  { 2, 8, 32, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 512, 2048, 8192, 16384, },
  { 4, 8, 32, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 512, 2048, 8192, 16384, },
  { 16, 32, 512, 2048, 8192, 16384, },
  { 1, 16, 32, 512, 2048, 8192, 16384, },
  { 2, 16, 32, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 512, 2048, 8192, 16384, },
  { 4, 16, 32, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 512, 2048, 8192, 16384, },
  { 8, 16, 32, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 8192, 16384, },
  { 64, 512, 2048, 8192, 16384, },
  { 1, 64, 512, 2048, 8192, 16384, },
  { 2, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 64, 512, 2048, 8192, 16384, },
  { 4, 64, 512, 2048, 8192, 16384, },
  { 1, 4, 64, 512, 2048, 8192, 16384, },
  { 2, 4, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 512, 2048, 8192, 16384, },
  { 8, 64, 512, 2048, 8192, 16384, },
  { 1, 8, 64, 512, 2048, 8192, 16384, },
  { 2, 8, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 512, 2048, 8192, 16384, },
  { 4, 8, 64, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 512, 2048, 8192, 16384, },
  { 16, 64, 512, 2048, 8192, 16384, },
  { 1, 16, 64, 512, 2048, 8192, 16384, },
  { 2, 16, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 512, 2048, 8192, 16384, },
  { 4, 16, 64, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 512, 2048, 8192, 16384, },
  { 8, 16, 64, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 8192, 16384, },
  { 32, 64, 512, 2048, 8192, 16384, },
  { 1, 32, 64, 512, 2048, 8192, 16384, },
  { 2, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 512, 2048, 8192, 16384, },
  { 4, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 512, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 512, 2048, 8192, 16384, },
  { 8, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 512, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 512, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 8192, 16384, },
  { 16, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 8192, 16384, },
  { 128, 512, 2048, 8192, 16384, },
  { 1, 128, 512, 2048, 8192, 16384, },
  { 2, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 128, 512, 2048, 8192, 16384, },
  { 4, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 128, 512, 2048, 8192, 16384, },
  { 8, 128, 512, 2048, 8192, 16384, },
  { 1, 8, 128, 512, 2048, 8192, 16384, },
  { 2, 8, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 128, 512, 2048, 8192, 16384, },
  { 4, 8, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 128, 512, 2048, 8192, 16384, },
  { 16, 128, 512, 2048, 8192, 16384, },
  { 1, 16, 128, 512, 2048, 8192, 16384, },
  { 2, 16, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 128, 512, 2048, 8192, 16384, },
  { 4, 16, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 128, 512, 2048, 8192, 16384, },
  { 8, 16, 128, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 128, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 128, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 8192, 16384, },
  { 32, 128, 512, 2048, 8192, 16384, },
  { 1, 32, 128, 512, 2048, 8192, 16384, },
  { 2, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 32, 128, 512, 2048, 8192, 16384, },
  { 4, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 32, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 128, 512, 2048, 8192, 16384, },
  { 8, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 8, 32, 128, 512, 2048, 8192, 16384, },
  { 2, 8, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 128, 512, 2048, 8192, 16384, },
  { 4, 8, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 8192, 16384, },
  { 16, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 2, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 4, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 8, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 8192, 16384, },
  { 64, 128, 512, 2048, 8192, 16384, },
  { 1, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 64, 128, 512, 2048, 8192, 16384, },
  { 4, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 128, 512, 2048, 8192, 16384, },
  { 8, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 8, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 8, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 128, 512, 2048, 8192, 16384, },
  { 4, 8, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 8192, 16384, },
  { 16, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 4, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 8, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 8192, 16384, },
  { 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 4, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 8, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, },
  { 256, 512, 2048, 8192, 16384, },
  { 1, 256, 512, 2048, 8192, 16384, },
  { 2, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 256, 512, 2048, 8192, 16384, },
  { 4, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 256, 512, 2048, 8192, 16384, },
  { 8, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 256, 512, 2048, 8192, 16384, },
  { 16, 256, 512, 2048, 8192, 16384, },
  { 1, 16, 256, 512, 2048, 8192, 16384, },
  { 2, 16, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 256, 512, 2048, 8192, 16384, },
  { 4, 16, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 256, 512, 2048, 8192, 16384, },
  { 8, 16, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 8192, 16384, },
  { 32, 256, 512, 2048, 8192, 16384, },
  { 1, 32, 256, 512, 2048, 8192, 16384, },
  { 2, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 32, 256, 512, 2048, 8192, 16384, },
  { 4, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 32, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 256, 512, 2048, 8192, 16384, },
  { 8, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 32, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 8192, 16384, },
  { 16, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 2, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 4, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 8, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 8192, 16384, },
  { 64, 256, 512, 2048, 8192, 16384, },
  { 1, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 64, 256, 512, 2048, 8192, 16384, },
  { 4, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 256, 512, 2048, 8192, 16384, },
  { 8, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 8192, 16384, },
  { 16, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 4, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 8, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 8192, 16384, },
  { 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 4, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 8, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, },
  { 128, 256, 512, 2048, 8192, 16384, },
  { 1, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 128, 256, 512, 2048, 8192, 16384, },
  { 8, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 8192, 16384, },
  { 16, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 8, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 8192, 16384, },
  { 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 8, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, },
  { 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 8, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, },
  { 1024, 2048, 8192, 16384, },
  { 1, 1024, 2048, 8192, 16384, },
  { 2, 1024, 2048, 8192, 16384, },
  { 1, 2, 1024, 2048, 8192, 16384, },
  { 4, 1024, 2048, 8192, 16384, },
  { 1, 4, 1024, 2048, 8192, 16384, },
  { 2, 4, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 1024, 2048, 8192, 16384, },
  { 8, 1024, 2048, 8192, 16384, },
  { 1, 8, 1024, 2048, 8192, 16384, },
  { 2, 8, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 1024, 2048, 8192, 16384, },
  { 4, 8, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 1024, 2048, 8192, 16384, },
  { 16, 1024, 2048, 8192, 16384, },
  { 1, 16, 1024, 2048, 8192, 16384, },
  { 2, 16, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 1024, 2048, 8192, 16384, },
  { 4, 16, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 1024, 2048, 8192, 16384, },
  { 8, 16, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 1024, 2048, 8192, 16384, },
  { 32, 1024, 2048, 8192, 16384, },
  { 1, 32, 1024, 2048, 8192, 16384, },
  { 2, 32, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 1024, 2048, 8192, 16384, },
  { 4, 32, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 1024, 2048, 8192, 16384, },
  { 8, 32, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 1024, 2048, 8192, 16384, },
  { 16, 32, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 8192, 16384, },
  { 64, 1024, 2048, 8192, 16384, },
  { 1, 64, 1024, 2048, 8192, 16384, },
  { 2, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 64, 1024, 2048, 8192, 16384, },
  { 4, 64, 1024, 2048, 8192, 16384, },
  { 1, 4, 64, 1024, 2048, 8192, 16384, },
  { 2, 4, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 1024, 2048, 8192, 16384, },
  { 8, 64, 1024, 2048, 8192, 16384, },
  { 1, 8, 64, 1024, 2048, 8192, 16384, },
  { 2, 8, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 1024, 2048, 8192, 16384, },
  { 4, 8, 64, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 1024, 2048, 8192, 16384, },
  { 16, 64, 1024, 2048, 8192, 16384, },
  { 1, 16, 64, 1024, 2048, 8192, 16384, },
  { 2, 16, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 1024, 2048, 8192, 16384, },
  { 4, 16, 64, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 1024, 2048, 8192, 16384, },
  { 8, 16, 64, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 8192, 16384, },
  { 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 32, 64, 1024, 2048, 8192, 16384, },
  { 2, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 1024, 2048, 8192, 16384, },
  { 4, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 1024, 2048, 8192, 16384, },
  { 8, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 8192, 16384, },
  { 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 8192, 16384, },
  { 128, 1024, 2048, 8192, 16384, },
  { 1, 128, 1024, 2048, 8192, 16384, },
  { 2, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 128, 1024, 2048, 8192, 16384, },
  { 4, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 128, 1024, 2048, 8192, 16384, },
  { 8, 128, 1024, 2048, 8192, 16384, },
  { 1, 8, 128, 1024, 2048, 8192, 16384, },
  { 2, 8, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 128, 1024, 2048, 8192, 16384, },
  { 4, 8, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 128, 1024, 2048, 8192, 16384, },
  { 16, 128, 1024, 2048, 8192, 16384, },
  { 1, 16, 128, 1024, 2048, 8192, 16384, },
  { 2, 16, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 128, 1024, 2048, 8192, 16384, },
  { 4, 16, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 128, 1024, 2048, 8192, 16384, },
  { 8, 16, 128, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 128, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 128, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 8192, 16384, },
  { 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 32, 128, 1024, 2048, 8192, 16384, },
  { 2, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 128, 1024, 2048, 8192, 16384, },
  { 4, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 128, 1024, 2048, 8192, 16384, },
  { 8, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 128, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 128, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 8192, 16384, },
  { 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 8192, 16384, },
  { 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 64, 128, 1024, 2048, 8192, 16384, },
  { 4, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 128, 1024, 2048, 8192, 16384, },
  { 8, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 8, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 8, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 128, 1024, 2048, 8192, 16384, },
  { 4, 8, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 8192, 16384, },
  { 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 4, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 8, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 8192, 16384, },
  { 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 4, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 8, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, },
  { 256, 1024, 2048, 8192, 16384, },
  { 1, 256, 1024, 2048, 8192, 16384, },
  { 2, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 256, 1024, 2048, 8192, 16384, },
  { 4, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 256, 1024, 2048, 8192, 16384, },
  { 8, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 256, 1024, 2048, 8192, 16384, },
  { 16, 256, 1024, 2048, 8192, 16384, },
  { 1, 16, 256, 1024, 2048, 8192, 16384, },
  { 2, 16, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 256, 1024, 2048, 8192, 16384, },
  { 4, 16, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 256, 1024, 2048, 8192, 16384, },
  { 8, 16, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 8192, 16384, },
  { 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 32, 256, 1024, 2048, 8192, 16384, },
  { 2, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 256, 1024, 2048, 8192, 16384, },
  { 4, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 256, 1024, 2048, 8192, 16384, },
  { 8, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 8192, 16384, },
  { 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 8192, 16384, },
  { 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 64, 256, 1024, 2048, 8192, 16384, },
  { 4, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 256, 1024, 2048, 8192, 16384, },
  { 8, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 8192, 16384, },
  { 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 4, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 8, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 8192, 16384, },
  { 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 4, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 8, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, },
  { 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 128, 256, 1024, 2048, 8192, 16384, },
  { 8, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 8192, 16384, },
  { 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 8, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 8192, 16384, },
  { 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 8, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, },
  { 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 8, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, },
  { 512, 1024, 2048, 8192, 16384, },
  { 1, 512, 1024, 2048, 8192, 16384, },
  { 2, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 512, 1024, 2048, 8192, 16384, },
  { 4, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 512, 1024, 2048, 8192, 16384, },
  { 8, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 512, 1024, 2048, 8192, 16384, },
  { 16, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 8192, 16384, },
  { 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 32, 512, 1024, 2048, 8192, 16384, },
  { 2, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 512, 1024, 2048, 8192, 16384, },
  { 4, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 512, 1024, 2048, 8192, 16384, },
  { 8, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 8192, 16384, },
  { 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 8192, 16384, },
  { 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 64, 512, 1024, 2048, 8192, 16384, },
  { 4, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 512, 1024, 2048, 8192, 16384, },
  { 8, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 8192, 16384, },
  { 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 8192, 16384, },
  { 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 4, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 8, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, },
  { 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 128, 512, 1024, 2048, 8192, 16384, },
  { 8, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 8192, 16384, },
  { 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 8192, 16384, },
  { 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 8, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, },
  { 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 8, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, },
  { 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 8192, 16384, },
  { 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 8192, 16384, },
  { 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, },
  { 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, },
  { 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, },
  { 4096, 8192, 16384, },
  { 1, 4096, 8192, 16384, },
  { 2, 4096, 8192, 16384, },
  { 1, 2, 4096, 8192, 16384, },
  { 4, 4096, 8192, 16384, },
  { 1, 4, 4096, 8192, 16384, },
  { 2, 4, 4096, 8192, 16384, },
  { 1, 2, 4, 4096, 8192, 16384, },
  { 8, 4096, 8192, 16384, },
  { 1, 8, 4096, 8192, 16384, },
  { 2, 8, 4096, 8192, 16384, },
  { 1, 2, 8, 4096, 8192, 16384, },
  { 4, 8, 4096, 8192, 16384, },
  { 1, 4, 8, 4096, 8192, 16384, },
  { 2, 4, 8, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 4096, 8192, 16384, },
  { 16, 4096, 8192, 16384, },
  { 1, 16, 4096, 8192, 16384, },
  { 2, 16, 4096, 8192, 16384, },
  { 1, 2, 16, 4096, 8192, 16384, },
  { 4, 16, 4096, 8192, 16384, },
  { 1, 4, 16, 4096, 8192, 16384, },
  { 2, 4, 16, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 4096, 8192, 16384, },
  { 8, 16, 4096, 8192, 16384, },
  { 1, 8, 16, 4096, 8192, 16384, },
  { 2, 8, 16, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 4096, 8192, 16384, },
  { 4, 8, 16, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 4096, 8192, 16384, },
  { 32, 4096, 8192, 16384, },
  { 1, 32, 4096, 8192, 16384, },
  { 2, 32, 4096, 8192, 16384, },
  { 1, 2, 32, 4096, 8192, 16384, },
  { 4, 32, 4096, 8192, 16384, },
  { 1, 4, 32, 4096, 8192, 16384, },
  { 2, 4, 32, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 4096, 8192, 16384, },
  { 8, 32, 4096, 8192, 16384, },
  { 1, 8, 32, 4096, 8192, 16384, },
  { 2, 8, 32, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 4096, 8192, 16384, },
  { 4, 8, 32, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 4096, 8192, 16384, },
  { 16, 32, 4096, 8192, 16384, },
  { 1, 16, 32, 4096, 8192, 16384, },
  { 2, 16, 32, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 4096, 8192, 16384, },
  { 4, 16, 32, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 4096, 8192, 16384, },
  { 8, 16, 32, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 4096, 8192, 16384, },
  { 64, 4096, 8192, 16384, },
  { 1, 64, 4096, 8192, 16384, },
  { 2, 64, 4096, 8192, 16384, },
  { 1, 2, 64, 4096, 8192, 16384, },
  { 4, 64, 4096, 8192, 16384, },
  { 1, 4, 64, 4096, 8192, 16384, },
  { 2, 4, 64, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 4096, 8192, 16384, },
  { 8, 64, 4096, 8192, 16384, },
  { 1, 8, 64, 4096, 8192, 16384, },
  { 2, 8, 64, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 4096, 8192, 16384, },
  { 4, 8, 64, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 4096, 8192, 16384, },
  { 16, 64, 4096, 8192, 16384, },
  { 1, 16, 64, 4096, 8192, 16384, },
  { 2, 16, 64, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 4096, 8192, 16384, },
  { 4, 16, 64, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 4096, 8192, 16384, },
  { 8, 16, 64, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 4096, 8192, 16384, },
  { 32, 64, 4096, 8192, 16384, },
  { 1, 32, 64, 4096, 8192, 16384, },
  { 2, 32, 64, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 4096, 8192, 16384, },
  { 4, 32, 64, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 4096, 8192, 16384, },
  { 8, 32, 64, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 4096, 8192, 16384, },
  { 16, 32, 64, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 4096, 8192, 16384, },
  { 128, 4096, 8192, 16384, },
  { 1, 128, 4096, 8192, 16384, },
  { 2, 128, 4096, 8192, 16384, },
  { 1, 2, 128, 4096, 8192, 16384, },
  { 4, 128, 4096, 8192, 16384, },
  { 1, 4, 128, 4096, 8192, 16384, },
  { 2, 4, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 4096, 8192, 16384, },
  { 8, 128, 4096, 8192, 16384, },
  { 1, 8, 128, 4096, 8192, 16384, },
  { 2, 8, 128, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 4096, 8192, 16384, },
  { 4, 8, 128, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 4096, 8192, 16384, },
  { 16, 128, 4096, 8192, 16384, },
  { 1, 16, 128, 4096, 8192, 16384, },
  { 2, 16, 128, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 4096, 8192, 16384, },
  { 4, 16, 128, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 4096, 8192, 16384, },
  { 8, 16, 128, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 4096, 8192, 16384, },
  { 32, 128, 4096, 8192, 16384, },
  { 1, 32, 128, 4096, 8192, 16384, },
  { 2, 32, 128, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 4096, 8192, 16384, },
  { 4, 32, 128, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 4096, 8192, 16384, },
  { 8, 32, 128, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 4096, 8192, 16384, },
  { 16, 32, 128, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 4096, 8192, 16384, },
  { 64, 128, 4096, 8192, 16384, },
  { 1, 64, 128, 4096, 8192, 16384, },
  { 2, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 4096, 8192, 16384, },
  { 4, 64, 128, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 4096, 8192, 16384, },
  { 8, 64, 128, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 4096, 8192, 16384, },
  { 16, 64, 128, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 4096, 8192, 16384, },
  { 32, 64, 128, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 4096, 8192, 16384, },
  { 256, 4096, 8192, 16384, },
  { 1, 256, 4096, 8192, 16384, },
  { 2, 256, 4096, 8192, 16384, },
  { 1, 2, 256, 4096, 8192, 16384, },
  { 4, 256, 4096, 8192, 16384, },
  { 1, 4, 256, 4096, 8192, 16384, },
  { 2, 4, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 256, 4096, 8192, 16384, },
  { 8, 256, 4096, 8192, 16384, },
  { 1, 8, 256, 4096, 8192, 16384, },
  { 2, 8, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 256, 4096, 8192, 16384, },
  { 4, 8, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 256, 4096, 8192, 16384, },
  { 16, 256, 4096, 8192, 16384, },
  { 1, 16, 256, 4096, 8192, 16384, },
  { 2, 16, 256, 4096, 8192, 16384, },
  { 1, 2, 16, 256, 4096, 8192, 16384, },
  { 4, 16, 256, 4096, 8192, 16384, },
  { 1, 4, 16, 256, 4096, 8192, 16384, },
  { 2, 4, 16, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 256, 4096, 8192, 16384, },
  { 8, 16, 256, 4096, 8192, 16384, },
  { 1, 8, 16, 256, 4096, 8192, 16384, },
  { 2, 8, 16, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 256, 4096, 8192, 16384, },
  { 4, 8, 16, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 4096, 8192, 16384, },
  { 32, 256, 4096, 8192, 16384, },
  { 1, 32, 256, 4096, 8192, 16384, },
  { 2, 32, 256, 4096, 8192, 16384, },
  { 1, 2, 32, 256, 4096, 8192, 16384, },
  { 4, 32, 256, 4096, 8192, 16384, },
  { 1, 4, 32, 256, 4096, 8192, 16384, },
  { 2, 4, 32, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 256, 4096, 8192, 16384, },
  { 8, 32, 256, 4096, 8192, 16384, },
  { 1, 8, 32, 256, 4096, 8192, 16384, },
  { 2, 8, 32, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 256, 4096, 8192, 16384, },
  { 4, 8, 32, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 4096, 8192, 16384, },
  { 16, 32, 256, 4096, 8192, 16384, },
  { 1, 16, 32, 256, 4096, 8192, 16384, },
  { 2, 16, 32, 256, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 256, 4096, 8192, 16384, },
  { 4, 16, 32, 256, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 256, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 4096, 8192, 16384, },
  { 8, 16, 32, 256, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 256, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 4096, 8192, 16384, },
  { 64, 256, 4096, 8192, 16384, },
  { 1, 64, 256, 4096, 8192, 16384, },
  { 2, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 64, 256, 4096, 8192, 16384, },
  { 4, 64, 256, 4096, 8192, 16384, },
  { 1, 4, 64, 256, 4096, 8192, 16384, },
  { 2, 4, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 256, 4096, 8192, 16384, },
  { 8, 64, 256, 4096, 8192, 16384, },
  { 1, 8, 64, 256, 4096, 8192, 16384, },
  { 2, 8, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 256, 4096, 8192, 16384, },
  { 4, 8, 64, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 4096, 8192, 16384, },
  { 16, 64, 256, 4096, 8192, 16384, },
  { 1, 16, 64, 256, 4096, 8192, 16384, },
  { 2, 16, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 256, 4096, 8192, 16384, },
  { 4, 16, 64, 256, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 256, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 4096, 8192, 16384, },
  { 8, 16, 64, 256, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 256, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 4096, 8192, 16384, },
  { 32, 64, 256, 4096, 8192, 16384, },
  { 1, 32, 64, 256, 4096, 8192, 16384, },
  { 2, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 256, 4096, 8192, 16384, },
  { 4, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 256, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 4096, 8192, 16384, },
  { 8, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 256, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 4096, 8192, 16384, },
  { 16, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 4096, 8192, 16384, },
  { 128, 256, 4096, 8192, 16384, },
  { 1, 128, 256, 4096, 8192, 16384, },
  { 2, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 128, 256, 4096, 8192, 16384, },
  { 4, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 256, 4096, 8192, 16384, },
  { 8, 128, 256, 4096, 8192, 16384, },
  { 1, 8, 128, 256, 4096, 8192, 16384, },
  { 2, 8, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 256, 4096, 8192, 16384, },
  { 4, 8, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 4096, 8192, 16384, },
  { 16, 128, 256, 4096, 8192, 16384, },
  { 1, 16, 128, 256, 4096, 8192, 16384, },
  { 2, 16, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 256, 4096, 8192, 16384, },
  { 4, 16, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 4096, 8192, 16384, },
  { 8, 16, 128, 256, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 256, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 4096, 8192, 16384, },
  { 32, 128, 256, 4096, 8192, 16384, },
  { 1, 32, 128, 256, 4096, 8192, 16384, },
  { 2, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 256, 4096, 8192, 16384, },
  { 4, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 4096, 8192, 16384, },
  { 8, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 256, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 4096, 8192, 16384, },
  { 16, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 4096, 8192, 16384, },
  { 64, 128, 256, 4096, 8192, 16384, },
  { 1, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 256, 4096, 8192, 16384, },
  { 4, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 4096, 8192, 16384, },
  { 8, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 4096, 8192, 16384, },
  { 16, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 4096, 8192, 16384, },
  { 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, },
  { 512, 4096, 8192, 16384, },
  { 1, 512, 4096, 8192, 16384, },
  { 2, 512, 4096, 8192, 16384, },
  { 1, 2, 512, 4096, 8192, 16384, },
  { 4, 512, 4096, 8192, 16384, },
  { 1, 4, 512, 4096, 8192, 16384, },
  { 2, 4, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 512, 4096, 8192, 16384, },
  { 8, 512, 4096, 8192, 16384, },
  { 1, 8, 512, 4096, 8192, 16384, },
  { 2, 8, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 512, 4096, 8192, 16384, },
  { 4, 8, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 512, 4096, 8192, 16384, },
  { 16, 512, 4096, 8192, 16384, },
  { 1, 16, 512, 4096, 8192, 16384, },
  { 2, 16, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 512, 4096, 8192, 16384, },
  { 4, 16, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 512, 4096, 8192, 16384, },
  { 8, 16, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 512, 4096, 8192, 16384, },
  { 32, 512, 4096, 8192, 16384, },
  { 1, 32, 512, 4096, 8192, 16384, },
  { 2, 32, 512, 4096, 8192, 16384, },
  { 1, 2, 32, 512, 4096, 8192, 16384, },
  { 4, 32, 512, 4096, 8192, 16384, },
  { 1, 4, 32, 512, 4096, 8192, 16384, },
  { 2, 4, 32, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 512, 4096, 8192, 16384, },
  { 8, 32, 512, 4096, 8192, 16384, },
  { 1, 8, 32, 512, 4096, 8192, 16384, },
  { 2, 8, 32, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 512, 4096, 8192, 16384, },
  { 4, 8, 32, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 512, 4096, 8192, 16384, },
  { 16, 32, 512, 4096, 8192, 16384, },
  { 1, 16, 32, 512, 4096, 8192, 16384, },
  { 2, 16, 32, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 512, 4096, 8192, 16384, },
  { 4, 16, 32, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 512, 4096, 8192, 16384, },
  { 8, 16, 32, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 4096, 8192, 16384, },
  { 64, 512, 4096, 8192, 16384, },
  { 1, 64, 512, 4096, 8192, 16384, },
  { 2, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 64, 512, 4096, 8192, 16384, },
  { 4, 64, 512, 4096, 8192, 16384, },
  { 1, 4, 64, 512, 4096, 8192, 16384, },
  { 2, 4, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 512, 4096, 8192, 16384, },
  { 8, 64, 512, 4096, 8192, 16384, },
  { 1, 8, 64, 512, 4096, 8192, 16384, },
  { 2, 8, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 512, 4096, 8192, 16384, },
  { 4, 8, 64, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 512, 4096, 8192, 16384, },
  { 16, 64, 512, 4096, 8192, 16384, },
  { 1, 16, 64, 512, 4096, 8192, 16384, },
  { 2, 16, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 512, 4096, 8192, 16384, },
  { 4, 16, 64, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 512, 4096, 8192, 16384, },
  { 8, 16, 64, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 4096, 8192, 16384, },
  { 32, 64, 512, 4096, 8192, 16384, },
  { 1, 32, 64, 512, 4096, 8192, 16384, },
  { 2, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 512, 4096, 8192, 16384, },
  { 4, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 512, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 512, 4096, 8192, 16384, },
  { 8, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 512, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 512, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 4096, 8192, 16384, },
  { 16, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 4096, 8192, 16384, },
  { 128, 512, 4096, 8192, 16384, },
  { 1, 128, 512, 4096, 8192, 16384, },
  { 2, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 128, 512, 4096, 8192, 16384, },
  { 4, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 512, 4096, 8192, 16384, },
  { 8, 128, 512, 4096, 8192, 16384, },
  { 1, 8, 128, 512, 4096, 8192, 16384, },
  { 2, 8, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 512, 4096, 8192, 16384, },
  { 4, 8, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 512, 4096, 8192, 16384, },
  { 16, 128, 512, 4096, 8192, 16384, },
  { 1, 16, 128, 512, 4096, 8192, 16384, },
  { 2, 16, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 512, 4096, 8192, 16384, },
  { 4, 16, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 512, 4096, 8192, 16384, },
  { 8, 16, 128, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 4096, 8192, 16384, },
  { 32, 128, 512, 4096, 8192, 16384, },
  { 1, 32, 128, 512, 4096, 8192, 16384, },
  { 2, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 512, 4096, 8192, 16384, },
  { 4, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 512, 4096, 8192, 16384, },
  { 8, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 512, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 512, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 4096, 8192, 16384, },
  { 16, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 4096, 8192, 16384, },
  { 64, 128, 512, 4096, 8192, 16384, },
  { 1, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 512, 4096, 8192, 16384, },
  { 4, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 512, 4096, 8192, 16384, },
  { 8, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 512, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 4096, 8192, 16384, },
  { 16, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 4096, 8192, 16384, },
  { 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, },
  { 256, 512, 4096, 8192, 16384, },
  { 1, 256, 512, 4096, 8192, 16384, },
  { 2, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 256, 512, 4096, 8192, 16384, },
  { 4, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 256, 512, 4096, 8192, 16384, },
  { 8, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 256, 512, 4096, 8192, 16384, },
  { 16, 256, 512, 4096, 8192, 16384, },
  { 1, 16, 256, 512, 4096, 8192, 16384, },
  { 2, 16, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 256, 512, 4096, 8192, 16384, },
  { 4, 16, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 256, 512, 4096, 8192, 16384, },
  { 8, 16, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 4096, 8192, 16384, },
  { 32, 256, 512, 4096, 8192, 16384, },
  { 1, 32, 256, 512, 4096, 8192, 16384, },
  { 2, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 32, 256, 512, 4096, 8192, 16384, },
  { 4, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 32, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 256, 512, 4096, 8192, 16384, },
  { 8, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 32, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 4096, 8192, 16384, },
  { 16, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 2, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 4, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 8, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 4096, 8192, 16384, },
  { 64, 256, 512, 4096, 8192, 16384, },
  { 1, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 64, 256, 512, 4096, 8192, 16384, },
  { 4, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 256, 512, 4096, 8192, 16384, },
  { 8, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 4096, 8192, 16384, },
  { 16, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 4, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 8, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 4096, 8192, 16384, },
  { 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 4, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 8, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, },
  { 128, 256, 512, 4096, 8192, 16384, },
  { 1, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 256, 512, 4096, 8192, 16384, },
  { 8, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 4096, 8192, 16384, },
  { 16, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 8, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 4096, 8192, 16384, },
  { 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 8, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, },
  { 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 8, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, },
  { 1024, 4096, 8192, 16384, },
  { 1, 1024, 4096, 8192, 16384, },
  { 2, 1024, 4096, 8192, 16384, },
  { 1, 2, 1024, 4096, 8192, 16384, },
  { 4, 1024, 4096, 8192, 16384, },
  { 1, 4, 1024, 4096, 8192, 16384, },
  { 2, 4, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 1024, 4096, 8192, 16384, },
  { 8, 1024, 4096, 8192, 16384, },
  { 1, 8, 1024, 4096, 8192, 16384, },
  { 2, 8, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 1024, 4096, 8192, 16384, },
  { 4, 8, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 1024, 4096, 8192, 16384, },
  { 16, 1024, 4096, 8192, 16384, },
  { 1, 16, 1024, 4096, 8192, 16384, },
  { 2, 16, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 1024, 4096, 8192, 16384, },
  { 4, 16, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 1024, 4096, 8192, 16384, },
  { 8, 16, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 1024, 4096, 8192, 16384, },
  { 32, 1024, 4096, 8192, 16384, },
  { 1, 32, 1024, 4096, 8192, 16384, },
  { 2, 32, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 1024, 4096, 8192, 16384, },
  { 4, 32, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 1024, 4096, 8192, 16384, },
  { 8, 32, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 1024, 4096, 8192, 16384, },
  { 16, 32, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 1024, 4096, 8192, 16384, },
  { 64, 1024, 4096, 8192, 16384, },
  { 1, 64, 1024, 4096, 8192, 16384, },
  { 2, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 64, 1024, 4096, 8192, 16384, },
  { 4, 64, 1024, 4096, 8192, 16384, },
  { 1, 4, 64, 1024, 4096, 8192, 16384, },
  { 2, 4, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 1024, 4096, 8192, 16384, },
  { 8, 64, 1024, 4096, 8192, 16384, },
  { 1, 8, 64, 1024, 4096, 8192, 16384, },
  { 2, 8, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 1024, 4096, 8192, 16384, },
  { 4, 8, 64, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 1024, 4096, 8192, 16384, },
  { 16, 64, 1024, 4096, 8192, 16384, },
  { 1, 16, 64, 1024, 4096, 8192, 16384, },
  { 2, 16, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 1024, 4096, 8192, 16384, },
  { 4, 16, 64, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 1024, 4096, 8192, 16384, },
  { 8, 16, 64, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 1024, 4096, 8192, 16384, },
  { 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 32, 64, 1024, 4096, 8192, 16384, },
  { 2, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 1024, 4096, 8192, 16384, },
  { 4, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 1024, 4096, 8192, 16384, },
  { 8, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 1024, 4096, 8192, 16384, },
  { 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 4096, 8192, 16384, },
  { 128, 1024, 4096, 8192, 16384, },
  { 1, 128, 1024, 4096, 8192, 16384, },
  { 2, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 128, 1024, 4096, 8192, 16384, },
  { 4, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 1024, 4096, 8192, 16384, },
  { 8, 128, 1024, 4096, 8192, 16384, },
  { 1, 8, 128, 1024, 4096, 8192, 16384, },
  { 2, 8, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 1024, 4096, 8192, 16384, },
  { 4, 8, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 1024, 4096, 8192, 16384, },
  { 16, 128, 1024, 4096, 8192, 16384, },
  { 1, 16, 128, 1024, 4096, 8192, 16384, },
  { 2, 16, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 1024, 4096, 8192, 16384, },
  { 4, 16, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 1024, 4096, 8192, 16384, },
  { 8, 16, 128, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 1024, 4096, 8192, 16384, },
  { 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 32, 128, 1024, 4096, 8192, 16384, },
  { 2, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 1024, 4096, 8192, 16384, },
  { 4, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 1024, 4096, 8192, 16384, },
  { 8, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 1024, 4096, 8192, 16384, },
  { 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 4096, 8192, 16384, },
  { 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 1024, 4096, 8192, 16384, },
  { 4, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 1024, 4096, 8192, 16384, },
  { 8, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 1024, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 1024, 4096, 8192, 16384, },
  { 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 4096, 8192, 16384, },
  { 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, },
  { 256, 1024, 4096, 8192, 16384, },
  { 1, 256, 1024, 4096, 8192, 16384, },
  { 2, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 256, 1024, 4096, 8192, 16384, },
  { 4, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 256, 1024, 4096, 8192, 16384, },
  { 8, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 256, 1024, 4096, 8192, 16384, },
  { 16, 256, 1024, 4096, 8192, 16384, },
  { 1, 16, 256, 1024, 4096, 8192, 16384, },
  { 2, 16, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 256, 1024, 4096, 8192, 16384, },
  { 4, 16, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 256, 1024, 4096, 8192, 16384, },
  { 8, 16, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 1024, 4096, 8192, 16384, },
  { 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 32, 256, 1024, 4096, 8192, 16384, },
  { 2, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 256, 1024, 4096, 8192, 16384, },
  { 4, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 256, 1024, 4096, 8192, 16384, },
  { 8, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 1024, 4096, 8192, 16384, },
  { 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 4096, 8192, 16384, },
  { 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 64, 256, 1024, 4096, 8192, 16384, },
  { 4, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 256, 1024, 4096, 8192, 16384, },
  { 8, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 1024, 4096, 8192, 16384, },
  { 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 4, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 8, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 4096, 8192, 16384, },
  { 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 4, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 8, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, },
  { 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 256, 1024, 4096, 8192, 16384, },
  { 8, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 1024, 4096, 8192, 16384, },
  { 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 8, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 4096, 8192, 16384, },
  { 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 8, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, },
  { 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 8, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, },
  { 512, 1024, 4096, 8192, 16384, },
  { 1, 512, 1024, 4096, 8192, 16384, },
  { 2, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 512, 1024, 4096, 8192, 16384, },
  { 4, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 512, 1024, 4096, 8192, 16384, },
  { 8, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 512, 1024, 4096, 8192, 16384, },
  { 16, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 512, 1024, 4096, 8192, 16384, },
  { 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 32, 512, 1024, 4096, 8192, 16384, },
  { 2, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 512, 1024, 4096, 8192, 16384, },
  { 4, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 512, 1024, 4096, 8192, 16384, },
  { 8, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 512, 1024, 4096, 8192, 16384, },
  { 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 4096, 8192, 16384, },
  { 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 64, 512, 1024, 4096, 8192, 16384, },
  { 4, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 512, 1024, 4096, 8192, 16384, },
  { 8, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 512, 1024, 4096, 8192, 16384, },
  { 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 4096, 8192, 16384, },
  { 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 4, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 8, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, },
  { 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 512, 1024, 4096, 8192, 16384, },
  { 8, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 512, 1024, 4096, 8192, 16384, },
  { 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 4096, 8192, 16384, },
  { 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 8, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, },
  { 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 8, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, },
  { 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 256, 512, 1024, 4096, 8192, 16384, },
  { 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 4096, 8192, 16384, },
  { 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, },
  { 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, },
  { 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, },
  { 2048, 4096, 8192, 16384, },
  { 1, 2048, 4096, 8192, 16384, },
  { 2, 2048, 4096, 8192, 16384, },
  { 1, 2, 2048, 4096, 8192, 16384, },
  { 4, 2048, 4096, 8192, 16384, },
  { 1, 4, 2048, 4096, 8192, 16384, },
  { 2, 4, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 2048, 4096, 8192, 16384, },
  { 8, 2048, 4096, 8192, 16384, },
  { 1, 8, 2048, 4096, 8192, 16384, },
  { 2, 8, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 2048, 4096, 8192, 16384, },
  { 4, 8, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 2048, 4096, 8192, 16384, },
  { 16, 2048, 4096, 8192, 16384, },
  { 1, 16, 2048, 4096, 8192, 16384, },
  { 2, 16, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 2048, 4096, 8192, 16384, },
  { 4, 16, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 2048, 4096, 8192, 16384, },
  { 8, 16, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 2048, 4096, 8192, 16384, },
  { 32, 2048, 4096, 8192, 16384, },
  { 1, 32, 2048, 4096, 8192, 16384, },
  { 2, 32, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 2048, 4096, 8192, 16384, },
  { 4, 32, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 2048, 4096, 8192, 16384, },
  { 8, 32, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 2048, 4096, 8192, 16384, },
  { 16, 32, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 2048, 4096, 8192, 16384, },
  { 64, 2048, 4096, 8192, 16384, },
  { 1, 64, 2048, 4096, 8192, 16384, },
  { 2, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 2048, 4096, 8192, 16384, },
  { 4, 64, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 2048, 4096, 8192, 16384, },
  { 8, 64, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 2048, 4096, 8192, 16384, },
  { 16, 64, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 2048, 4096, 8192, 16384, },
  { 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 4096, 8192, 16384, },
  { 128, 2048, 4096, 8192, 16384, },
  { 1, 128, 2048, 4096, 8192, 16384, },
  { 2, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 128, 2048, 4096, 8192, 16384, },
  { 4, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 2048, 4096, 8192, 16384, },
  { 8, 128, 2048, 4096, 8192, 16384, },
  { 1, 8, 128, 2048, 4096, 8192, 16384, },
  { 2, 8, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 2048, 4096, 8192, 16384, },
  { 4, 8, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 2048, 4096, 8192, 16384, },
  { 16, 128, 2048, 4096, 8192, 16384, },
  { 1, 16, 128, 2048, 4096, 8192, 16384, },
  { 2, 16, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 2048, 4096, 8192, 16384, },
  { 4, 16, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 2048, 4096, 8192, 16384, },
  { 8, 16, 128, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 2048, 4096, 8192, 16384, },
  { 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 32, 128, 2048, 4096, 8192, 16384, },
  { 2, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 2048, 4096, 8192, 16384, },
  { 4, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 2048, 4096, 8192, 16384, },
  { 8, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 2048, 4096, 8192, 16384, },
  { 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 4096, 8192, 16384, },
  { 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 2048, 4096, 8192, 16384, },
  { 4, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 2048, 4096, 8192, 16384, },
  { 8, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 2048, 4096, 8192, 16384, },
  { 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 4096, 8192, 16384, },
  { 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, },
  { 256, 2048, 4096, 8192, 16384, },
  { 1, 256, 2048, 4096, 8192, 16384, },
  { 2, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 256, 2048, 4096, 8192, 16384, },
  { 4, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 256, 2048, 4096, 8192, 16384, },
  { 8, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 256, 2048, 4096, 8192, 16384, },
  { 16, 256, 2048, 4096, 8192, 16384, },
  { 1, 16, 256, 2048, 4096, 8192, 16384, },
  { 2, 16, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 256, 2048, 4096, 8192, 16384, },
  { 4, 16, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 256, 2048, 4096, 8192, 16384, },
  { 8, 16, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 2048, 4096, 8192, 16384, },
  { 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 32, 256, 2048, 4096, 8192, 16384, },
  { 2, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 256, 2048, 4096, 8192, 16384, },
  { 4, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 256, 2048, 4096, 8192, 16384, },
  { 8, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 2048, 4096, 8192, 16384, },
  { 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 4096, 8192, 16384, },
  { 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 256, 2048, 4096, 8192, 16384, },
  { 4, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 256, 2048, 4096, 8192, 16384, },
  { 8, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 2048, 4096, 8192, 16384, },
  { 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 4096, 8192, 16384, },
  { 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, },
  { 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 256, 2048, 4096, 8192, 16384, },
  { 8, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 2048, 4096, 8192, 16384, },
  { 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 8, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 4096, 8192, 16384, },
  { 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 8, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, },
  { 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 8, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, },
  { 512, 2048, 4096, 8192, 16384, },
  { 1, 512, 2048, 4096, 8192, 16384, },
  { 2, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 512, 2048, 4096, 8192, 16384, },
  { 4, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 512, 2048, 4096, 8192, 16384, },
  { 8, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 512, 2048, 4096, 8192, 16384, },
  { 16, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 512, 2048, 4096, 8192, 16384, },
  { 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 32, 512, 2048, 4096, 8192, 16384, },
  { 2, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 512, 2048, 4096, 8192, 16384, },
  { 4, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 512, 2048, 4096, 8192, 16384, },
  { 8, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 512, 2048, 4096, 8192, 16384, },
  { 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 4096, 8192, 16384, },
  { 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 512, 2048, 4096, 8192, 16384, },
  { 4, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 512, 2048, 4096, 8192, 16384, },
  { 8, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 512, 2048, 4096, 8192, 16384, },
  { 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 4096, 8192, 16384, },
  { 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, },
  { 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 512, 2048, 4096, 8192, 16384, },
  { 8, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 512, 2048, 4096, 8192, 16384, },
  { 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 4096, 8192, 16384, },
  { 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 8, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, },
  { 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 8, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, },
  { 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 256, 512, 2048, 4096, 8192, 16384, },
  { 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 4096, 8192, 16384, },
  { 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, },
  { 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, },
  { 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, },
  { 1024, 2048, 4096, 8192, 16384, },
  { 1, 1024, 2048, 4096, 8192, 16384, },
  { 2, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 1024, 2048, 4096, 8192, 16384, },
  { 4, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 1024, 2048, 4096, 8192, 16384, },
  { 8, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 1024, 2048, 4096, 8192, 16384, },
  { 16, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 1024, 2048, 4096, 8192, 16384, },
  { 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, },
  { 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 1024, 2048, 4096, 8192, 16384, },
  { 4, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 1024, 2048, 4096, 8192, 16384, },
  { 8, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 1024, 2048, 4096, 8192, 16384, },
  { 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, },
  { 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, },
  { 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 1024, 2048, 4096, 8192, 16384, },
  { 8, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 1024, 2048, 4096, 8192, 16384, },
  { 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, },
  { 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, },
  { 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 8, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, },
  { 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 256, 1024, 2048, 4096, 8192, 16384, },
  { 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, },
  { 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, },
  { 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, },
  { 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, },
  { 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, },
  { 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, },
  { 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, },
  { 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, },
  { 32768, },
  { 1, 32768, },
  { 2, 32768, },
  { 1, 2, 32768, },
  { 4, 32768, },
  { 1, 4, 32768, },
  { 2, 4, 32768, },
  { 1, 2, 4, 32768, },
  { 8, 32768, },
  { 1, 8, 32768, },
  { 2, 8, 32768, },
  { 1, 2, 8, 32768, },
  { 4, 8, 32768, },
  { 1, 4, 8, 32768, },
  { 2, 4, 8, 32768, },
  { 1, 2, 4, 8, 32768, },
  { 16, 32768, },
  { 1, 16, 32768, },
  { 2, 16, 32768, },
  { 1, 2, 16, 32768, },
  { 4, 16, 32768, },
  { 1, 4, 16, 32768, },
  { 2, 4, 16, 32768, },
  { 1, 2, 4, 16, 32768, },
  { 8, 16, 32768, },
  { 1, 8, 16, 32768, },
  { 2, 8, 16, 32768, },
  { 1, 2, 8, 16, 32768, },
  { 4, 8, 16, 32768, },
  { 1, 4, 8, 16, 32768, },
  { 2, 4, 8, 16, 32768, },
  { 1, 2, 4, 8, 16, 32768, },
  { 32, 32768, },
  { 1, 32, 32768, },
  { 2, 32, 32768, },
  { 1, 2, 32, 32768, },
  { 4, 32, 32768, },
  { 1, 4, 32, 32768, },
  { 2, 4, 32, 32768, },
  { 1, 2, 4, 32, 32768, },
  { 8, 32, 32768, },
  { 1, 8, 32, 32768, },
  { 2, 8, 32, 32768, },
  { 1, 2, 8, 32, 32768, },
  { 4, 8, 32, 32768, },
  { 1, 4, 8, 32, 32768, },
  { 2, 4, 8, 32, 32768, },
  { 1, 2, 4, 8, 32, 32768, },
  { 16, 32, 32768, },
  { 1, 16, 32, 32768, },
  { 2, 16, 32, 32768, },
  { 1, 2, 16, 32, 32768, },
  { 4, 16, 32, 32768, },
  { 1, 4, 16, 32, 32768, },
  { 2, 4, 16, 32, 32768, },
  { 1, 2, 4, 16, 32, 32768, },
  { 8, 16, 32, 32768, },
  { 1, 8, 16, 32, 32768, },
  { 2, 8, 16, 32, 32768, },
  { 1, 2, 8, 16, 32, 32768, },
  { 4, 8, 16, 32, 32768, },
  { 1, 4, 8, 16, 32, 32768, },
  { 2, 4, 8, 16, 32, 32768, },
  { 1, 2, 4, 8, 16, 32, 32768, },
  { 64, 32768, },
  { 1, 64, 32768, },
  { 2, 64, 32768, },
  { 1, 2, 64, 32768, },
  { 4, 64, 32768, },
  { 1, 4, 64, 32768, },
  { 2, 4, 64, 32768, },
  { 1, 2, 4, 64, 32768, },
  { 8, 64, 32768, },
  { 1, 8, 64, 32768, },
  { 2, 8, 64, 32768, },
  { 1, 2, 8, 64, 32768, },
  { 4, 8, 64, 32768, },
  { 1, 4, 8, 64, 32768, },
  { 2, 4, 8, 64, 32768, },
  { 1, 2, 4, 8, 64, 32768, },
  { 16, 64, 32768, },
  { 1, 16, 64, 32768, },
  { 2, 16, 64, 32768, },
  { 1, 2, 16, 64, 32768, },
  { 4, 16, 64, 32768, },
  { 1, 4, 16, 64, 32768, },
  { 2, 4, 16, 64, 32768, },
  { 1, 2, 4, 16, 64, 32768, },
  { 8, 16, 64, 32768, },
  { 1, 8, 16, 64, 32768, },
  { 2, 8, 16, 64, 32768, },
  { 1, 2, 8, 16, 64, 32768, },
  { 4, 8, 16, 64, 32768, },
  { 1, 4, 8, 16, 64, 32768, },
  { 2, 4, 8, 16, 64, 32768, },
  { 1, 2, 4, 8, 16, 64, 32768, },
  { 32, 64, 32768, },
  { 1, 32, 64, 32768, },
  { 2, 32, 64, 32768, },
  { 1, 2, 32, 64, 32768, },
  { 4, 32, 64, 32768, },
  { 1, 4, 32, 64, 32768, },
  { 2, 4, 32, 64, 32768, },
  { 1, 2, 4, 32, 64, 32768, },
  { 8, 32, 64, 32768, },
  { 1, 8, 32, 64, 32768, },
  { 2, 8, 32, 64, 32768, },
  { 1, 2, 8, 32, 64, 32768, },
  { 4, 8, 32, 64, 32768, },
  { 1, 4, 8, 32, 64, 32768, },
  { 2, 4, 8, 32, 64, 32768, },
  { 1, 2, 4, 8, 32, 64, 32768, },
  { 16, 32, 64, 32768, },
  { 1, 16, 32, 64, 32768, },
  { 2, 16, 32, 64, 32768, },
  { 1, 2, 16, 32, 64, 32768, },
  { 4, 16, 32, 64, 32768, },
  { 1, 4, 16, 32, 64, 32768, },
  { 2, 4, 16, 32, 64, 32768, },
  { 1, 2, 4, 16, 32, 64, 32768, },
  { 8, 16, 32, 64, 32768, },
  { 1, 8, 16, 32, 64, 32768, },
  { 2, 8, 16, 32, 64, 32768, },
  { 1, 2, 8, 16, 32, 64, 32768, },
  { 4, 8, 16, 32, 64, 32768, },
  { 1, 4, 8, 16, 32, 64, 32768, },
  { 2, 4, 8, 16, 32, 64, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 32768, },
  { 128, 32768, },
  { 1, 128, 32768, },
  { 2, 128, 32768, },
  { 1, 2, 128, 32768, },
  { 4, 128, 32768, },
  { 1, 4, 128, 32768, },
  { 2, 4, 128, 32768, },
  { 1, 2, 4, 128, 32768, },
  { 8, 128, 32768, },
  { 1, 8, 128, 32768, },
  { 2, 8, 128, 32768, },
  { 1, 2, 8, 128, 32768, },
  { 4, 8, 128, 32768, },
  { 1, 4, 8, 128, 32768, },
  { 2, 4, 8, 128, 32768, },
  { 1, 2, 4, 8, 128, 32768, },
  { 16, 128, 32768, },
  { 1, 16, 128, 32768, },
  { 2, 16, 128, 32768, },
  { 1, 2, 16, 128, 32768, },
  { 4, 16, 128, 32768, },
  { 1, 4, 16, 128, 32768, },
  { 2, 4, 16, 128, 32768, },
  { 1, 2, 4, 16, 128, 32768, },
  { 8, 16, 128, 32768, },
  { 1, 8, 16, 128, 32768, },
  { 2, 8, 16, 128, 32768, },
  { 1, 2, 8, 16, 128, 32768, },
  { 4, 8, 16, 128, 32768, },
  { 1, 4, 8, 16, 128, 32768, },
  { 2, 4, 8, 16, 128, 32768, },
  { 1, 2, 4, 8, 16, 128, 32768, },
  { 32, 128, 32768, },
  { 1, 32, 128, 32768, },
  { 2, 32, 128, 32768, },
  { 1, 2, 32, 128, 32768, },
  { 4, 32, 128, 32768, },
  { 1, 4, 32, 128, 32768, },
  { 2, 4, 32, 128, 32768, },
  { 1, 2, 4, 32, 128, 32768, },
  { 8, 32, 128, 32768, },
  { 1, 8, 32, 128, 32768, },
  { 2, 8, 32, 128, 32768, },
  { 1, 2, 8, 32, 128, 32768, },
  { 4, 8, 32, 128, 32768, },
  { 1, 4, 8, 32, 128, 32768, },
  { 2, 4, 8, 32, 128, 32768, },
  { 1, 2, 4, 8, 32, 128, 32768, },
  { 16, 32, 128, 32768, },
  { 1, 16, 32, 128, 32768, },
  { 2, 16, 32, 128, 32768, },
  { 1, 2, 16, 32, 128, 32768, },
  { 4, 16, 32, 128, 32768, },
  { 1, 4, 16, 32, 128, 32768, },
  { 2, 4, 16, 32, 128, 32768, },
  { 1, 2, 4, 16, 32, 128, 32768, },
  { 8, 16, 32, 128, 32768, },
  { 1, 8, 16, 32, 128, 32768, },
  { 2, 8, 16, 32, 128, 32768, },
  { 1, 2, 8, 16, 32, 128, 32768, },
  { 4, 8, 16, 32, 128, 32768, },
  { 1, 4, 8, 16, 32, 128, 32768, },
  { 2, 4, 8, 16, 32, 128, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 32768, },
  { 64, 128, 32768, },
  { 1, 64, 128, 32768, },
  { 2, 64, 128, 32768, },
  { 1, 2, 64, 128, 32768, },
  { 4, 64, 128, 32768, },
  { 1, 4, 64, 128, 32768, },
  { 2, 4, 64, 128, 32768, },
  { 1, 2, 4, 64, 128, 32768, },
  { 8, 64, 128, 32768, },
  { 1, 8, 64, 128, 32768, },
  { 2, 8, 64, 128, 32768, },
  { 1, 2, 8, 64, 128, 32768, },
  { 4, 8, 64, 128, 32768, },
  { 1, 4, 8, 64, 128, 32768, },
  { 2, 4, 8, 64, 128, 32768, },
  { 1, 2, 4, 8, 64, 128, 32768, },
  { 16, 64, 128, 32768, },
  { 1, 16, 64, 128, 32768, },
  { 2, 16, 64, 128, 32768, },
  { 1, 2, 16, 64, 128, 32768, },
  { 4, 16, 64, 128, 32768, },
  { 1, 4, 16, 64, 128, 32768, },
  { 2, 4, 16, 64, 128, 32768, },
  { 1, 2, 4, 16, 64, 128, 32768, },
  { 8, 16, 64, 128, 32768, },
  { 1, 8, 16, 64, 128, 32768, },
  { 2, 8, 16, 64, 128, 32768, },
  { 1, 2, 8, 16, 64, 128, 32768, },
  { 4, 8, 16, 64, 128, 32768, },
  { 1, 4, 8, 16, 64, 128, 32768, },
  { 2, 4, 8, 16, 64, 128, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 32768, },
  { 32, 64, 128, 32768, },
  { 1, 32, 64, 128, 32768, },
  { 2, 32, 64, 128, 32768, },
  { 1, 2, 32, 64, 128, 32768, },
  { 4, 32, 64, 128, 32768, },
  { 1, 4, 32, 64, 128, 32768, },
  { 2, 4, 32, 64, 128, 32768, },
  { 1, 2, 4, 32, 64, 128, 32768, },
  { 8, 32, 64, 128, 32768, },
  { 1, 8, 32, 64, 128, 32768, },
  { 2, 8, 32, 64, 128, 32768, },
  { 1, 2, 8, 32, 64, 128, 32768, },
  { 4, 8, 32, 64, 128, 32768, },
  { 1, 4, 8, 32, 64, 128, 32768, },
  { 2, 4, 8, 32, 64, 128, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 32768, },
  { 16, 32, 64, 128, 32768, },
  { 1, 16, 32, 64, 128, 32768, },
  { 2, 16, 32, 64, 128, 32768, },
  { 1, 2, 16, 32, 64, 128, 32768, },
  { 4, 16, 32, 64, 128, 32768, },
  { 1, 4, 16, 32, 64, 128, 32768, },
  { 2, 4, 16, 32, 64, 128, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 32768, },
  { 8, 16, 32, 64, 128, 32768, },
  { 1, 8, 16, 32, 64, 128, 32768, },
  { 2, 8, 16, 32, 64, 128, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 32768, },
  { 4, 8, 16, 32, 64, 128, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 32768, },
  { 256, 32768, },
  { 1, 256, 32768, },
  { 2, 256, 32768, },
  { 1, 2, 256, 32768, },
  { 4, 256, 32768, },
  { 1, 4, 256, 32768, },
  { 2, 4, 256, 32768, },
  { 1, 2, 4, 256, 32768, },
  { 8, 256, 32768, },
  { 1, 8, 256, 32768, },
  { 2, 8, 256, 32768, },
  { 1, 2, 8, 256, 32768, },
  { 4, 8, 256, 32768, },
  { 1, 4, 8, 256, 32768, },
  { 2, 4, 8, 256, 32768, },
  { 1, 2, 4, 8, 256, 32768, },
  { 16, 256, 32768, },
  { 1, 16, 256, 32768, },
  { 2, 16, 256, 32768, },
  { 1, 2, 16, 256, 32768, },
  { 4, 16, 256, 32768, },
  { 1, 4, 16, 256, 32768, },
  { 2, 4, 16, 256, 32768, },
  { 1, 2, 4, 16, 256, 32768, },
  { 8, 16, 256, 32768, },
  { 1, 8, 16, 256, 32768, },
  { 2, 8, 16, 256, 32768, },
  { 1, 2, 8, 16, 256, 32768, },
  { 4, 8, 16, 256, 32768, },
  { 1, 4, 8, 16, 256, 32768, },
  { 2, 4, 8, 16, 256, 32768, },
  { 1, 2, 4, 8, 16, 256, 32768, },
  { 32, 256, 32768, },
  { 1, 32, 256, 32768, },
  { 2, 32, 256, 32768, },
  { 1, 2, 32, 256, 32768, },
  { 4, 32, 256, 32768, },
  { 1, 4, 32, 256, 32768, },
  { 2, 4, 32, 256, 32768, },
  { 1, 2, 4, 32, 256, 32768, },
  { 8, 32, 256, 32768, },
  { 1, 8, 32, 256, 32768, },
  { 2, 8, 32, 256, 32768, },
  { 1, 2, 8, 32, 256, 32768, },
  { 4, 8, 32, 256, 32768, },
  { 1, 4, 8, 32, 256, 32768, },
  { 2, 4, 8, 32, 256, 32768, },
  { 1, 2, 4, 8, 32, 256, 32768, },
  { 16, 32, 256, 32768, },
  { 1, 16, 32, 256, 32768, },
  { 2, 16, 32, 256, 32768, },
  { 1, 2, 16, 32, 256, 32768, },
  { 4, 16, 32, 256, 32768, },
  { 1, 4, 16, 32, 256, 32768, },
  { 2, 4, 16, 32, 256, 32768, },
  { 1, 2, 4, 16, 32, 256, 32768, },
  { 8, 16, 32, 256, 32768, },
  { 1, 8, 16, 32, 256, 32768, },
  { 2, 8, 16, 32, 256, 32768, },
  { 1, 2, 8, 16, 32, 256, 32768, },
  { 4, 8, 16, 32, 256, 32768, },
  { 1, 4, 8, 16, 32, 256, 32768, },
  { 2, 4, 8, 16, 32, 256, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 32768, },
  { 64, 256, 32768, },
  { 1, 64, 256, 32768, },
  { 2, 64, 256, 32768, },
  { 1, 2, 64, 256, 32768, },
  { 4, 64, 256, 32768, },
  { 1, 4, 64, 256, 32768, },
  { 2, 4, 64, 256, 32768, },
  { 1, 2, 4, 64, 256, 32768, },
  { 8, 64, 256, 32768, },
  { 1, 8, 64, 256, 32768, },
  { 2, 8, 64, 256, 32768, },
  { 1, 2, 8, 64, 256, 32768, },
  { 4, 8, 64, 256, 32768, },
  { 1, 4, 8, 64, 256, 32768, },
  { 2, 4, 8, 64, 256, 32768, },
  { 1, 2, 4, 8, 64, 256, 32768, },
  { 16, 64, 256, 32768, },
  { 1, 16, 64, 256, 32768, },
  { 2, 16, 64, 256, 32768, },
  { 1, 2, 16, 64, 256, 32768, },
  { 4, 16, 64, 256, 32768, },
  { 1, 4, 16, 64, 256, 32768, },
  { 2, 4, 16, 64, 256, 32768, },
  { 1, 2, 4, 16, 64, 256, 32768, },
  { 8, 16, 64, 256, 32768, },
  { 1, 8, 16, 64, 256, 32768, },
  { 2, 8, 16, 64, 256, 32768, },
  { 1, 2, 8, 16, 64, 256, 32768, },
  { 4, 8, 16, 64, 256, 32768, },
  { 1, 4, 8, 16, 64, 256, 32768, },
  { 2, 4, 8, 16, 64, 256, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 32768, },
  { 32, 64, 256, 32768, },
  { 1, 32, 64, 256, 32768, },
  { 2, 32, 64, 256, 32768, },
  { 1, 2, 32, 64, 256, 32768, },
  { 4, 32, 64, 256, 32768, },
  { 1, 4, 32, 64, 256, 32768, },
  { 2, 4, 32, 64, 256, 32768, },
  { 1, 2, 4, 32, 64, 256, 32768, },
  { 8, 32, 64, 256, 32768, },
  { 1, 8, 32, 64, 256, 32768, },
  { 2, 8, 32, 64, 256, 32768, },
  { 1, 2, 8, 32, 64, 256, 32768, },
  { 4, 8, 32, 64, 256, 32768, },
  { 1, 4, 8, 32, 64, 256, 32768, },
  { 2, 4, 8, 32, 64, 256, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 32768, },
  { 16, 32, 64, 256, 32768, },
  { 1, 16, 32, 64, 256, 32768, },
  { 2, 16, 32, 64, 256, 32768, },
  { 1, 2, 16, 32, 64, 256, 32768, },
  { 4, 16, 32, 64, 256, 32768, },
  { 1, 4, 16, 32, 64, 256, 32768, },
  { 2, 4, 16, 32, 64, 256, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 32768, },
  { 8, 16, 32, 64, 256, 32768, },
  { 1, 8, 16, 32, 64, 256, 32768, },
  { 2, 8, 16, 32, 64, 256, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 32768, },
  { 4, 8, 16, 32, 64, 256, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 32768, },
  { 128, 256, 32768, },
  { 1, 128, 256, 32768, },
  { 2, 128, 256, 32768, },
  { 1, 2, 128, 256, 32768, },
  { 4, 128, 256, 32768, },
  { 1, 4, 128, 256, 32768, },
  { 2, 4, 128, 256, 32768, },
  { 1, 2, 4, 128, 256, 32768, },
  { 8, 128, 256, 32768, },
  { 1, 8, 128, 256, 32768, },
  { 2, 8, 128, 256, 32768, },
  { 1, 2, 8, 128, 256, 32768, },
  { 4, 8, 128, 256, 32768, },
  { 1, 4, 8, 128, 256, 32768, },
  { 2, 4, 8, 128, 256, 32768, },
  { 1, 2, 4, 8, 128, 256, 32768, },
  { 16, 128, 256, 32768, },
  { 1, 16, 128, 256, 32768, },
  { 2, 16, 128, 256, 32768, },
  { 1, 2, 16, 128, 256, 32768, },
  { 4, 16, 128, 256, 32768, },
  { 1, 4, 16, 128, 256, 32768, },
  { 2, 4, 16, 128, 256, 32768, },
  { 1, 2, 4, 16, 128, 256, 32768, },
  { 8, 16, 128, 256, 32768, },
  { 1, 8, 16, 128, 256, 32768, },
  { 2, 8, 16, 128, 256, 32768, },
  { 1, 2, 8, 16, 128, 256, 32768, },
  { 4, 8, 16, 128, 256, 32768, },
  { 1, 4, 8, 16, 128, 256, 32768, },
  { 2, 4, 8, 16, 128, 256, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 32768, },
  { 32, 128, 256, 32768, },
  { 1, 32, 128, 256, 32768, },
  { 2, 32, 128, 256, 32768, },
  { 1, 2, 32, 128, 256, 32768, },
  { 4, 32, 128, 256, 32768, },
  { 1, 4, 32, 128, 256, 32768, },
  { 2, 4, 32, 128, 256, 32768, },
  { 1, 2, 4, 32, 128, 256, 32768, },
  { 8, 32, 128, 256, 32768, },
  { 1, 8, 32, 128, 256, 32768, },
  { 2, 8, 32, 128, 256, 32768, },
  { 1, 2, 8, 32, 128, 256, 32768, },
  { 4, 8, 32, 128, 256, 32768, },
  { 1, 4, 8, 32, 128, 256, 32768, },
  { 2, 4, 8, 32, 128, 256, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 32768, },
  { 16, 32, 128, 256, 32768, },
  { 1, 16, 32, 128, 256, 32768, },
  { 2, 16, 32, 128, 256, 32768, },
  { 1, 2, 16, 32, 128, 256, 32768, },
  { 4, 16, 32, 128, 256, 32768, },
  { 1, 4, 16, 32, 128, 256, 32768, },
  { 2, 4, 16, 32, 128, 256, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 32768, },
  { 8, 16, 32, 128, 256, 32768, },
  { 1, 8, 16, 32, 128, 256, 32768, },
  { 2, 8, 16, 32, 128, 256, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 32768, },
  { 4, 8, 16, 32, 128, 256, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 32768, },
  { 64, 128, 256, 32768, },
  { 1, 64, 128, 256, 32768, },
  { 2, 64, 128, 256, 32768, },
  { 1, 2, 64, 128, 256, 32768, },
  { 4, 64, 128, 256, 32768, },
  { 1, 4, 64, 128, 256, 32768, },
  { 2, 4, 64, 128, 256, 32768, },
  { 1, 2, 4, 64, 128, 256, 32768, },
  { 8, 64, 128, 256, 32768, },
  { 1, 8, 64, 128, 256, 32768, },
  { 2, 8, 64, 128, 256, 32768, },
  { 1, 2, 8, 64, 128, 256, 32768, },
  { 4, 8, 64, 128, 256, 32768, },
  { 1, 4, 8, 64, 128, 256, 32768, },
  { 2, 4, 8, 64, 128, 256, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 32768, },
  { 16, 64, 128, 256, 32768, },
  { 1, 16, 64, 128, 256, 32768, },
  { 2, 16, 64, 128, 256, 32768, },
  { 1, 2, 16, 64, 128, 256, 32768, },
  { 4, 16, 64, 128, 256, 32768, },
  { 1, 4, 16, 64, 128, 256, 32768, },
  { 2, 4, 16, 64, 128, 256, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 32768, },
  { 8, 16, 64, 128, 256, 32768, },
  { 1, 8, 16, 64, 128, 256, 32768, },
  { 2, 8, 16, 64, 128, 256, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 32768, },
  { 4, 8, 16, 64, 128, 256, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 32768, },
  { 32, 64, 128, 256, 32768, },
  { 1, 32, 64, 128, 256, 32768, },
  { 2, 32, 64, 128, 256, 32768, },
  { 1, 2, 32, 64, 128, 256, 32768, },
  { 4, 32, 64, 128, 256, 32768, },
  { 1, 4, 32, 64, 128, 256, 32768, },
  { 2, 4, 32, 64, 128, 256, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 32768, },
  { 8, 32, 64, 128, 256, 32768, },
  { 1, 8, 32, 64, 128, 256, 32768, },
  { 2, 8, 32, 64, 128, 256, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 32768, },
  { 4, 8, 32, 64, 128, 256, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 32768, },
  { 16, 32, 64, 128, 256, 32768, },
  { 1, 16, 32, 64, 128, 256, 32768, },
  { 2, 16, 32, 64, 128, 256, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 32768, },
  { 4, 16, 32, 64, 128, 256, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 32768, },
  { 8, 16, 32, 64, 128, 256, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 32768, },
  { 512, 32768, },
  { 1, 512, 32768, },
  { 2, 512, 32768, },
  { 1, 2, 512, 32768, },
  { 4, 512, 32768, },
  { 1, 4, 512, 32768, },
  { 2, 4, 512, 32768, },
  { 1, 2, 4, 512, 32768, },
  { 8, 512, 32768, },
  { 1, 8, 512, 32768, },
  { 2, 8, 512, 32768, },
  { 1, 2, 8, 512, 32768, },
  { 4, 8, 512, 32768, },
  { 1, 4, 8, 512, 32768, },
  { 2, 4, 8, 512, 32768, },
  { 1, 2, 4, 8, 512, 32768, },
  { 16, 512, 32768, },
  { 1, 16, 512, 32768, },
  { 2, 16, 512, 32768, },
  { 1, 2, 16, 512, 32768, },
  { 4, 16, 512, 32768, },
  { 1, 4, 16, 512, 32768, },
  { 2, 4, 16, 512, 32768, },
  { 1, 2, 4, 16, 512, 32768, },
  { 8, 16, 512, 32768, },
  { 1, 8, 16, 512, 32768, },
  { 2, 8, 16, 512, 32768, },
  { 1, 2, 8, 16, 512, 32768, },
  { 4, 8, 16, 512, 32768, },
  { 1, 4, 8, 16, 512, 32768, },
  { 2, 4, 8, 16, 512, 32768, },
  { 1, 2, 4, 8, 16, 512, 32768, },
  { 32, 512, 32768, },
  { 1, 32, 512, 32768, },
  { 2, 32, 512, 32768, },
  { 1, 2, 32, 512, 32768, },
  { 4, 32, 512, 32768, },
  { 1, 4, 32, 512, 32768, },
  { 2, 4, 32, 512, 32768, },
  { 1, 2, 4, 32, 512, 32768, },
  { 8, 32, 512, 32768, },
  { 1, 8, 32, 512, 32768, },
  { 2, 8, 32, 512, 32768, },
  { 1, 2, 8, 32, 512, 32768, },
  { 4, 8, 32, 512, 32768, },
  { 1, 4, 8, 32, 512, 32768, },
  { 2, 4, 8, 32, 512, 32768, },
  { 1, 2, 4, 8, 32, 512, 32768, },
  { 16, 32, 512, 32768, },
  { 1, 16, 32, 512, 32768, },
  { 2, 16, 32, 512, 32768, },
  { 1, 2, 16, 32, 512, 32768, },
  { 4, 16, 32, 512, 32768, },
  { 1, 4, 16, 32, 512, 32768, },
  { 2, 4, 16, 32, 512, 32768, },
  { 1, 2, 4, 16, 32, 512, 32768, },
  { 8, 16, 32, 512, 32768, },
  { 1, 8, 16, 32, 512, 32768, },
  { 2, 8, 16, 32, 512, 32768, },
  { 1, 2, 8, 16, 32, 512, 32768, },
  { 4, 8, 16, 32, 512, 32768, },
  { 1, 4, 8, 16, 32, 512, 32768, },
  { 2, 4, 8, 16, 32, 512, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 32768, },
  { 64, 512, 32768, },
  { 1, 64, 512, 32768, },
  { 2, 64, 512, 32768, },
  { 1, 2, 64, 512, 32768, },
  { 4, 64, 512, 32768, },
  { 1, 4, 64, 512, 32768, },
  { 2, 4, 64, 512, 32768, },
  { 1, 2, 4, 64, 512, 32768, },
  { 8, 64, 512, 32768, },
  { 1, 8, 64, 512, 32768, },
  { 2, 8, 64, 512, 32768, },
  { 1, 2, 8, 64, 512, 32768, },
  { 4, 8, 64, 512, 32768, },
  { 1, 4, 8, 64, 512, 32768, },
  { 2, 4, 8, 64, 512, 32768, },
  { 1, 2, 4, 8, 64, 512, 32768, },
  { 16, 64, 512, 32768, },
  { 1, 16, 64, 512, 32768, },
  { 2, 16, 64, 512, 32768, },
  { 1, 2, 16, 64, 512, 32768, },
  { 4, 16, 64, 512, 32768, },
  { 1, 4, 16, 64, 512, 32768, },
  { 2, 4, 16, 64, 512, 32768, },
  { 1, 2, 4, 16, 64, 512, 32768, },
  { 8, 16, 64, 512, 32768, },
  { 1, 8, 16, 64, 512, 32768, },
  { 2, 8, 16, 64, 512, 32768, },
  { 1, 2, 8, 16, 64, 512, 32768, },
  { 4, 8, 16, 64, 512, 32768, },
  { 1, 4, 8, 16, 64, 512, 32768, },
  { 2, 4, 8, 16, 64, 512, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 32768, },
  { 32, 64, 512, 32768, },
  { 1, 32, 64, 512, 32768, },
  { 2, 32, 64, 512, 32768, },
  { 1, 2, 32, 64, 512, 32768, },
  { 4, 32, 64, 512, 32768, },
  { 1, 4, 32, 64, 512, 32768, },
  { 2, 4, 32, 64, 512, 32768, },
  { 1, 2, 4, 32, 64, 512, 32768, },
  { 8, 32, 64, 512, 32768, },
  { 1, 8, 32, 64, 512, 32768, },
  { 2, 8, 32, 64, 512, 32768, },
  { 1, 2, 8, 32, 64, 512, 32768, },
  { 4, 8, 32, 64, 512, 32768, },
  { 1, 4, 8, 32, 64, 512, 32768, },
  { 2, 4, 8, 32, 64, 512, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 32768, },
  { 16, 32, 64, 512, 32768, },
  { 1, 16, 32, 64, 512, 32768, },
  { 2, 16, 32, 64, 512, 32768, },
  { 1, 2, 16, 32, 64, 512, 32768, },
  { 4, 16, 32, 64, 512, 32768, },
  { 1, 4, 16, 32, 64, 512, 32768, },
  { 2, 4, 16, 32, 64, 512, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 32768, },
  { 8, 16, 32, 64, 512, 32768, },
  { 1, 8, 16, 32, 64, 512, 32768, },
  { 2, 8, 16, 32, 64, 512, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 32768, },
  { 4, 8, 16, 32, 64, 512, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 32768, },
  { 128, 512, 32768, },
  { 1, 128, 512, 32768, },
  { 2, 128, 512, 32768, },
  { 1, 2, 128, 512, 32768, },
  { 4, 128, 512, 32768, },
  { 1, 4, 128, 512, 32768, },
  { 2, 4, 128, 512, 32768, },
  { 1, 2, 4, 128, 512, 32768, },
  { 8, 128, 512, 32768, },
  { 1, 8, 128, 512, 32768, },
  { 2, 8, 128, 512, 32768, },
  { 1, 2, 8, 128, 512, 32768, },
  { 4, 8, 128, 512, 32768, },
  { 1, 4, 8, 128, 512, 32768, },
  { 2, 4, 8, 128, 512, 32768, },
  { 1, 2, 4, 8, 128, 512, 32768, },
  { 16, 128, 512, 32768, },
  { 1, 16, 128, 512, 32768, },
  { 2, 16, 128, 512, 32768, },
  { 1, 2, 16, 128, 512, 32768, },
  { 4, 16, 128, 512, 32768, },
  { 1, 4, 16, 128, 512, 32768, },
  { 2, 4, 16, 128, 512, 32768, },
  { 1, 2, 4, 16, 128, 512, 32768, },
  { 8, 16, 128, 512, 32768, },
  { 1, 8, 16, 128, 512, 32768, },
  { 2, 8, 16, 128, 512, 32768, },
  { 1, 2, 8, 16, 128, 512, 32768, },
  { 4, 8, 16, 128, 512, 32768, },
  { 1, 4, 8, 16, 128, 512, 32768, },
  { 2, 4, 8, 16, 128, 512, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 32768, },
  { 32, 128, 512, 32768, },
  { 1, 32, 128, 512, 32768, },
  { 2, 32, 128, 512, 32768, },
  { 1, 2, 32, 128, 512, 32768, },
  { 4, 32, 128, 512, 32768, },
  { 1, 4, 32, 128, 512, 32768, },
  { 2, 4, 32, 128, 512, 32768, },
  { 1, 2, 4, 32, 128, 512, 32768, },
  { 8, 32, 128, 512, 32768, },
  { 1, 8, 32, 128, 512, 32768, },
  { 2, 8, 32, 128, 512, 32768, },
  { 1, 2, 8, 32, 128, 512, 32768, },
  { 4, 8, 32, 128, 512, 32768, },
  { 1, 4, 8, 32, 128, 512, 32768, },
  { 2, 4, 8, 32, 128, 512, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 32768, },
  { 16, 32, 128, 512, 32768, },
  { 1, 16, 32, 128, 512, 32768, },
  { 2, 16, 32, 128, 512, 32768, },
  { 1, 2, 16, 32, 128, 512, 32768, },
  { 4, 16, 32, 128, 512, 32768, },
  { 1, 4, 16, 32, 128, 512, 32768, },
  { 2, 4, 16, 32, 128, 512, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 32768, },
  { 8, 16, 32, 128, 512, 32768, },
  { 1, 8, 16, 32, 128, 512, 32768, },
  { 2, 8, 16, 32, 128, 512, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 32768, },
  { 4, 8, 16, 32, 128, 512, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 32768, },
  { 64, 128, 512, 32768, },
  { 1, 64, 128, 512, 32768, },
  { 2, 64, 128, 512, 32768, },
  { 1, 2, 64, 128, 512, 32768, },
  { 4, 64, 128, 512, 32768, },
  { 1, 4, 64, 128, 512, 32768, },
  { 2, 4, 64, 128, 512, 32768, },
  { 1, 2, 4, 64, 128, 512, 32768, },
  { 8, 64, 128, 512, 32768, },
  { 1, 8, 64, 128, 512, 32768, },
  { 2, 8, 64, 128, 512, 32768, },
  { 1, 2, 8, 64, 128, 512, 32768, },
  { 4, 8, 64, 128, 512, 32768, },
  { 1, 4, 8, 64, 128, 512, 32768, },
  { 2, 4, 8, 64, 128, 512, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 32768, },
  { 16, 64, 128, 512, 32768, },
  { 1, 16, 64, 128, 512, 32768, },
  { 2, 16, 64, 128, 512, 32768, },
  { 1, 2, 16, 64, 128, 512, 32768, },
  { 4, 16, 64, 128, 512, 32768, },
  { 1, 4, 16, 64, 128, 512, 32768, },
  { 2, 4, 16, 64, 128, 512, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 32768, },
  { 8, 16, 64, 128, 512, 32768, },
  { 1, 8, 16, 64, 128, 512, 32768, },
  { 2, 8, 16, 64, 128, 512, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 32768, },
  { 4, 8, 16, 64, 128, 512, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 32768, },
  { 32, 64, 128, 512, 32768, },
  { 1, 32, 64, 128, 512, 32768, },
  { 2, 32, 64, 128, 512, 32768, },
  { 1, 2, 32, 64, 128, 512, 32768, },
  { 4, 32, 64, 128, 512, 32768, },
  { 1, 4, 32, 64, 128, 512, 32768, },
  { 2, 4, 32, 64, 128, 512, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 32768, },
  { 8, 32, 64, 128, 512, 32768, },
  { 1, 8, 32, 64, 128, 512, 32768, },
  { 2, 8, 32, 64, 128, 512, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 32768, },
  { 4, 8, 32, 64, 128, 512, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 32768, },
  { 16, 32, 64, 128, 512, 32768, },
  { 1, 16, 32, 64, 128, 512, 32768, },
  { 2, 16, 32, 64, 128, 512, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 32768, },
  { 4, 16, 32, 64, 128, 512, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 32768, },
  { 8, 16, 32, 64, 128, 512, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 32768, },
  { 256, 512, 32768, },
  { 1, 256, 512, 32768, },
  { 2, 256, 512, 32768, },
  { 1, 2, 256, 512, 32768, },
  { 4, 256, 512, 32768, },
  { 1, 4, 256, 512, 32768, },
  { 2, 4, 256, 512, 32768, },
  { 1, 2, 4, 256, 512, 32768, },
  { 8, 256, 512, 32768, },
  { 1, 8, 256, 512, 32768, },
  { 2, 8, 256, 512, 32768, },
  { 1, 2, 8, 256, 512, 32768, },
  { 4, 8, 256, 512, 32768, },
  { 1, 4, 8, 256, 512, 32768, },
  { 2, 4, 8, 256, 512, 32768, },
  { 1, 2, 4, 8, 256, 512, 32768, },
  { 16, 256, 512, 32768, },
  { 1, 16, 256, 512, 32768, },
  { 2, 16, 256, 512, 32768, },
  { 1, 2, 16, 256, 512, 32768, },
  { 4, 16, 256, 512, 32768, },
  { 1, 4, 16, 256, 512, 32768, },
  { 2, 4, 16, 256, 512, 32768, },
  { 1, 2, 4, 16, 256, 512, 32768, },
  { 8, 16, 256, 512, 32768, },
  { 1, 8, 16, 256, 512, 32768, },
  { 2, 8, 16, 256, 512, 32768, },
  { 1, 2, 8, 16, 256, 512, 32768, },
  { 4, 8, 16, 256, 512, 32768, },
  { 1, 4, 8, 16, 256, 512, 32768, },
  { 2, 4, 8, 16, 256, 512, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 32768, },
  { 32, 256, 512, 32768, },
  { 1, 32, 256, 512, 32768, },
  { 2, 32, 256, 512, 32768, },
  { 1, 2, 32, 256, 512, 32768, },
  { 4, 32, 256, 512, 32768, },
  { 1, 4, 32, 256, 512, 32768, },
  { 2, 4, 32, 256, 512, 32768, },
  { 1, 2, 4, 32, 256, 512, 32768, },
  { 8, 32, 256, 512, 32768, },
  { 1, 8, 32, 256, 512, 32768, },
  { 2, 8, 32, 256, 512, 32768, },
  { 1, 2, 8, 32, 256, 512, 32768, },
  { 4, 8, 32, 256, 512, 32768, },
  { 1, 4, 8, 32, 256, 512, 32768, },
  { 2, 4, 8, 32, 256, 512, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 32768, },
  { 16, 32, 256, 512, 32768, },
  { 1, 16, 32, 256, 512, 32768, },
  { 2, 16, 32, 256, 512, 32768, },
  { 1, 2, 16, 32, 256, 512, 32768, },
  { 4, 16, 32, 256, 512, 32768, },
  { 1, 4, 16, 32, 256, 512, 32768, },
  { 2, 4, 16, 32, 256, 512, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 32768, },
  { 8, 16, 32, 256, 512, 32768, },
  { 1, 8, 16, 32, 256, 512, 32768, },
  { 2, 8, 16, 32, 256, 512, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 32768, },
  { 4, 8, 16, 32, 256, 512, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 32768, },
  { 64, 256, 512, 32768, },
  { 1, 64, 256, 512, 32768, },
  { 2, 64, 256, 512, 32768, },
  { 1, 2, 64, 256, 512, 32768, },
  { 4, 64, 256, 512, 32768, },
  { 1, 4, 64, 256, 512, 32768, },
  { 2, 4, 64, 256, 512, 32768, },
  { 1, 2, 4, 64, 256, 512, 32768, },
  { 8, 64, 256, 512, 32768, },
  { 1, 8, 64, 256, 512, 32768, },
  { 2, 8, 64, 256, 512, 32768, },
  { 1, 2, 8, 64, 256, 512, 32768, },
  { 4, 8, 64, 256, 512, 32768, },
  { 1, 4, 8, 64, 256, 512, 32768, },
  { 2, 4, 8, 64, 256, 512, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 32768, },
  { 16, 64, 256, 512, 32768, },
  { 1, 16, 64, 256, 512, 32768, },
  { 2, 16, 64, 256, 512, 32768, },
  { 1, 2, 16, 64, 256, 512, 32768, },
  { 4, 16, 64, 256, 512, 32768, },
  { 1, 4, 16, 64, 256, 512, 32768, },
  { 2, 4, 16, 64, 256, 512, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 32768, },
  { 8, 16, 64, 256, 512, 32768, },
  { 1, 8, 16, 64, 256, 512, 32768, },
  { 2, 8, 16, 64, 256, 512, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 32768, },
  { 4, 8, 16, 64, 256, 512, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 32768, },
  { 32, 64, 256, 512, 32768, },
  { 1, 32, 64, 256, 512, 32768, },
  { 2, 32, 64, 256, 512, 32768, },
  { 1, 2, 32, 64, 256, 512, 32768, },
  { 4, 32, 64, 256, 512, 32768, },
  { 1, 4, 32, 64, 256, 512, 32768, },
  { 2, 4, 32, 64, 256, 512, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 32768, },
  { 8, 32, 64, 256, 512, 32768, },
  { 1, 8, 32, 64, 256, 512, 32768, },
  { 2, 8, 32, 64, 256, 512, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 32768, },
  { 4, 8, 32, 64, 256, 512, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 32768, },
  { 16, 32, 64, 256, 512, 32768, },
  { 1, 16, 32, 64, 256, 512, 32768, },
  { 2, 16, 32, 64, 256, 512, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 32768, },
  { 4, 16, 32, 64, 256, 512, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 32768, },
  { 8, 16, 32, 64, 256, 512, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 32768, },
  { 128, 256, 512, 32768, },
  { 1, 128, 256, 512, 32768, },
  { 2, 128, 256, 512, 32768, },
  { 1, 2, 128, 256, 512, 32768, },
  { 4, 128, 256, 512, 32768, },
  { 1, 4, 128, 256, 512, 32768, },
  { 2, 4, 128, 256, 512, 32768, },
  { 1, 2, 4, 128, 256, 512, 32768, },
  { 8, 128, 256, 512, 32768, },
  { 1, 8, 128, 256, 512, 32768, },
  { 2, 8, 128, 256, 512, 32768, },
  { 1, 2, 8, 128, 256, 512, 32768, },
  { 4, 8, 128, 256, 512, 32768, },
  { 1, 4, 8, 128, 256, 512, 32768, },
  { 2, 4, 8, 128, 256, 512, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 32768, },
  { 16, 128, 256, 512, 32768, },
  { 1, 16, 128, 256, 512, 32768, },
  { 2, 16, 128, 256, 512, 32768, },
  { 1, 2, 16, 128, 256, 512, 32768, },
  { 4, 16, 128, 256, 512, 32768, },
  { 1, 4, 16, 128, 256, 512, 32768, },
  { 2, 4, 16, 128, 256, 512, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 32768, },
  { 8, 16, 128, 256, 512, 32768, },
  { 1, 8, 16, 128, 256, 512, 32768, },
  { 2, 8, 16, 128, 256, 512, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 32768, },
  { 4, 8, 16, 128, 256, 512, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 32768, },
  { 32, 128, 256, 512, 32768, },
  { 1, 32, 128, 256, 512, 32768, },
  { 2, 32, 128, 256, 512, 32768, },
  { 1, 2, 32, 128, 256, 512, 32768, },
  { 4, 32, 128, 256, 512, 32768, },
  { 1, 4, 32, 128, 256, 512, 32768, },
  { 2, 4, 32, 128, 256, 512, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 32768, },
  { 8, 32, 128, 256, 512, 32768, },
  { 1, 8, 32, 128, 256, 512, 32768, },
  { 2, 8, 32, 128, 256, 512, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 32768, },
  { 4, 8, 32, 128, 256, 512, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 32768, },
  { 16, 32, 128, 256, 512, 32768, },
  { 1, 16, 32, 128, 256, 512, 32768, },
  { 2, 16, 32, 128, 256, 512, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 32768, },
  { 4, 16, 32, 128, 256, 512, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 32768, },
  { 8, 16, 32, 128, 256, 512, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 32768, },
  { 64, 128, 256, 512, 32768, },
  { 1, 64, 128, 256, 512, 32768, },
  { 2, 64, 128, 256, 512, 32768, },
  { 1, 2, 64, 128, 256, 512, 32768, },
  { 4, 64, 128, 256, 512, 32768, },
  { 1, 4, 64, 128, 256, 512, 32768, },
  { 2, 4, 64, 128, 256, 512, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 32768, },
  { 8, 64, 128, 256, 512, 32768, },
  { 1, 8, 64, 128, 256, 512, 32768, },
  { 2, 8, 64, 128, 256, 512, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 32768, },
  { 4, 8, 64, 128, 256, 512, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 32768, },
  { 16, 64, 128, 256, 512, 32768, },
  { 1, 16, 64, 128, 256, 512, 32768, },
  { 2, 16, 64, 128, 256, 512, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 32768, },
  { 4, 16, 64, 128, 256, 512, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 32768, },
  { 8, 16, 64, 128, 256, 512, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 32768, },
  { 32, 64, 128, 256, 512, 32768, },
  { 1, 32, 64, 128, 256, 512, 32768, },
  { 2, 32, 64, 128, 256, 512, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 32768, },
  { 4, 32, 64, 128, 256, 512, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 32768, },
  { 8, 32, 64, 128, 256, 512, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 32768, },
  { 16, 32, 64, 128, 256, 512, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 32768, },
  { 1024, 32768, },
  { 1, 1024, 32768, },
  { 2, 1024, 32768, },
  { 1, 2, 1024, 32768, },
  { 4, 1024, 32768, },
  { 1, 4, 1024, 32768, },
  { 2, 4, 1024, 32768, },
  { 1, 2, 4, 1024, 32768, },
  { 8, 1024, 32768, },
  { 1, 8, 1024, 32768, },
  { 2, 8, 1024, 32768, },
  { 1, 2, 8, 1024, 32768, },
  { 4, 8, 1024, 32768, },
  { 1, 4, 8, 1024, 32768, },
  { 2, 4, 8, 1024, 32768, },
  { 1, 2, 4, 8, 1024, 32768, },
  { 16, 1024, 32768, },
  { 1, 16, 1024, 32768, },
  { 2, 16, 1024, 32768, },
  { 1, 2, 16, 1024, 32768, },
  { 4, 16, 1024, 32768, },
  { 1, 4, 16, 1024, 32768, },
  { 2, 4, 16, 1024, 32768, },
  { 1, 2, 4, 16, 1024, 32768, },
  { 8, 16, 1024, 32768, },
  { 1, 8, 16, 1024, 32768, },
  { 2, 8, 16, 1024, 32768, },
  { 1, 2, 8, 16, 1024, 32768, },
  { 4, 8, 16, 1024, 32768, },
  { 1, 4, 8, 16, 1024, 32768, },
  { 2, 4, 8, 16, 1024, 32768, },
  { 1, 2, 4, 8, 16, 1024, 32768, },
  { 32, 1024, 32768, },
  { 1, 32, 1024, 32768, },
  { 2, 32, 1024, 32768, },
  { 1, 2, 32, 1024, 32768, },
  { 4, 32, 1024, 32768, },
  { 1, 4, 32, 1024, 32768, },
  { 2, 4, 32, 1024, 32768, },
  { 1, 2, 4, 32, 1024, 32768, },
  { 8, 32, 1024, 32768, },
  { 1, 8, 32, 1024, 32768, },
  { 2, 8, 32, 1024, 32768, },
  { 1, 2, 8, 32, 1024, 32768, },
  { 4, 8, 32, 1024, 32768, },
  { 1, 4, 8, 32, 1024, 32768, },
  { 2, 4, 8, 32, 1024, 32768, },
  { 1, 2, 4, 8, 32, 1024, 32768, },
  { 16, 32, 1024, 32768, },
  { 1, 16, 32, 1024, 32768, },
  { 2, 16, 32, 1024, 32768, },
  { 1, 2, 16, 32, 1024, 32768, },
  { 4, 16, 32, 1024, 32768, },
  { 1, 4, 16, 32, 1024, 32768, },
  { 2, 4, 16, 32, 1024, 32768, },
  { 1, 2, 4, 16, 32, 1024, 32768, },
  { 8, 16, 32, 1024, 32768, },
  { 1, 8, 16, 32, 1024, 32768, },
  { 2, 8, 16, 32, 1024, 32768, },
  { 1, 2, 8, 16, 32, 1024, 32768, },
  { 4, 8, 16, 32, 1024, 32768, },
  { 1, 4, 8, 16, 32, 1024, 32768, },
  { 2, 4, 8, 16, 32, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 32768, },
  { 64, 1024, 32768, },
  { 1, 64, 1024, 32768, },
  { 2, 64, 1024, 32768, },
  { 1, 2, 64, 1024, 32768, },
  { 4, 64, 1024, 32768, },
  { 1, 4, 64, 1024, 32768, },
  { 2, 4, 64, 1024, 32768, },
  { 1, 2, 4, 64, 1024, 32768, },
  { 8, 64, 1024, 32768, },
  { 1, 8, 64, 1024, 32768, },
  { 2, 8, 64, 1024, 32768, },
  { 1, 2, 8, 64, 1024, 32768, },
  { 4, 8, 64, 1024, 32768, },
  { 1, 4, 8, 64, 1024, 32768, },
  { 2, 4, 8, 64, 1024, 32768, },
  { 1, 2, 4, 8, 64, 1024, 32768, },
  { 16, 64, 1024, 32768, },
  { 1, 16, 64, 1024, 32768, },
  { 2, 16, 64, 1024, 32768, },
  { 1, 2, 16, 64, 1024, 32768, },
  { 4, 16, 64, 1024, 32768, },
  { 1, 4, 16, 64, 1024, 32768, },
  { 2, 4, 16, 64, 1024, 32768, },
  { 1, 2, 4, 16, 64, 1024, 32768, },
  { 8, 16, 64, 1024, 32768, },
  { 1, 8, 16, 64, 1024, 32768, },
  { 2, 8, 16, 64, 1024, 32768, },
  { 1, 2, 8, 16, 64, 1024, 32768, },
  { 4, 8, 16, 64, 1024, 32768, },
  { 1, 4, 8, 16, 64, 1024, 32768, },
  { 2, 4, 8, 16, 64, 1024, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 32768, },
  { 32, 64, 1024, 32768, },
  { 1, 32, 64, 1024, 32768, },
  { 2, 32, 64, 1024, 32768, },
  { 1, 2, 32, 64, 1024, 32768, },
  { 4, 32, 64, 1024, 32768, },
  { 1, 4, 32, 64, 1024, 32768, },
  { 2, 4, 32, 64, 1024, 32768, },
  { 1, 2, 4, 32, 64, 1024, 32768, },
  { 8, 32, 64, 1024, 32768, },
  { 1, 8, 32, 64, 1024, 32768, },
  { 2, 8, 32, 64, 1024, 32768, },
  { 1, 2, 8, 32, 64, 1024, 32768, },
  { 4, 8, 32, 64, 1024, 32768, },
  { 1, 4, 8, 32, 64, 1024, 32768, },
  { 2, 4, 8, 32, 64, 1024, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 32768, },
  { 16, 32, 64, 1024, 32768, },
  { 1, 16, 32, 64, 1024, 32768, },
  { 2, 16, 32, 64, 1024, 32768, },
  { 1, 2, 16, 32, 64, 1024, 32768, },
  { 4, 16, 32, 64, 1024, 32768, },
  { 1, 4, 16, 32, 64, 1024, 32768, },
  { 2, 4, 16, 32, 64, 1024, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 32768, },
  { 8, 16, 32, 64, 1024, 32768, },
  { 1, 8, 16, 32, 64, 1024, 32768, },
  { 2, 8, 16, 32, 64, 1024, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 32768, },
  { 4, 8, 16, 32, 64, 1024, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 32768, },
  { 128, 1024, 32768, },
  { 1, 128, 1024, 32768, },
  { 2, 128, 1024, 32768, },
  { 1, 2, 128, 1024, 32768, },
  { 4, 128, 1024, 32768, },
  { 1, 4, 128, 1024, 32768, },
  { 2, 4, 128, 1024, 32768, },
  { 1, 2, 4, 128, 1024, 32768, },
  { 8, 128, 1024, 32768, },
  { 1, 8, 128, 1024, 32768, },
  { 2, 8, 128, 1024, 32768, },
  { 1, 2, 8, 128, 1024, 32768, },
  { 4, 8, 128, 1024, 32768, },
  { 1, 4, 8, 128, 1024, 32768, },
  { 2, 4, 8, 128, 1024, 32768, },
  { 1, 2, 4, 8, 128, 1024, 32768, },
  { 16, 128, 1024, 32768, },
  { 1, 16, 128, 1024, 32768, },
  { 2, 16, 128, 1024, 32768, },
  { 1, 2, 16, 128, 1024, 32768, },
  { 4, 16, 128, 1024, 32768, },
  { 1, 4, 16, 128, 1024, 32768, },
  { 2, 4, 16, 128, 1024, 32768, },
  { 1, 2, 4, 16, 128, 1024, 32768, },
  { 8, 16, 128, 1024, 32768, },
  { 1, 8, 16, 128, 1024, 32768, },
  { 2, 8, 16, 128, 1024, 32768, },
  { 1, 2, 8, 16, 128, 1024, 32768, },
  { 4, 8, 16, 128, 1024, 32768, },
  { 1, 4, 8, 16, 128, 1024, 32768, },
  { 2, 4, 8, 16, 128, 1024, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 32768, },
  { 32, 128, 1024, 32768, },
  { 1, 32, 128, 1024, 32768, },
  { 2, 32, 128, 1024, 32768, },
  { 1, 2, 32, 128, 1024, 32768, },
  { 4, 32, 128, 1024, 32768, },
  { 1, 4, 32, 128, 1024, 32768, },
  { 2, 4, 32, 128, 1024, 32768, },
  { 1, 2, 4, 32, 128, 1024, 32768, },
  { 8, 32, 128, 1024, 32768, },
  { 1, 8, 32, 128, 1024, 32768, },
  { 2, 8, 32, 128, 1024, 32768, },
  { 1, 2, 8, 32, 128, 1024, 32768, },
  { 4, 8, 32, 128, 1024, 32768, },
  { 1, 4, 8, 32, 128, 1024, 32768, },
  { 2, 4, 8, 32, 128, 1024, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 32768, },
  { 16, 32, 128, 1024, 32768, },
  { 1, 16, 32, 128, 1024, 32768, },
  { 2, 16, 32, 128, 1024, 32768, },
  { 1, 2, 16, 32, 128, 1024, 32768, },
  { 4, 16, 32, 128, 1024, 32768, },
  { 1, 4, 16, 32, 128, 1024, 32768, },
  { 2, 4, 16, 32, 128, 1024, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 32768, },
  { 8, 16, 32, 128, 1024, 32768, },
  { 1, 8, 16, 32, 128, 1024, 32768, },
  { 2, 8, 16, 32, 128, 1024, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 32768, },
  { 4, 8, 16, 32, 128, 1024, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 32768, },
  { 64, 128, 1024, 32768, },
  { 1, 64, 128, 1024, 32768, },
  { 2, 64, 128, 1024, 32768, },
  { 1, 2, 64, 128, 1024, 32768, },
  { 4, 64, 128, 1024, 32768, },
  { 1, 4, 64, 128, 1024, 32768, },
  { 2, 4, 64, 128, 1024, 32768, },
  { 1, 2, 4, 64, 128, 1024, 32768, },
  { 8, 64, 128, 1024, 32768, },
  { 1, 8, 64, 128, 1024, 32768, },
  { 2, 8, 64, 128, 1024, 32768, },
  { 1, 2, 8, 64, 128, 1024, 32768, },
  { 4, 8, 64, 128, 1024, 32768, },
  { 1, 4, 8, 64, 128, 1024, 32768, },
  { 2, 4, 8, 64, 128, 1024, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 32768, },
  { 16, 64, 128, 1024, 32768, },
  { 1, 16, 64, 128, 1024, 32768, },
  { 2, 16, 64, 128, 1024, 32768, },
  { 1, 2, 16, 64, 128, 1024, 32768, },
  { 4, 16, 64, 128, 1024, 32768, },
  { 1, 4, 16, 64, 128, 1024, 32768, },
  { 2, 4, 16, 64, 128, 1024, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 32768, },
  { 8, 16, 64, 128, 1024, 32768, },
  { 1, 8, 16, 64, 128, 1024, 32768, },
  { 2, 8, 16, 64, 128, 1024, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 32768, },
  { 4, 8, 16, 64, 128, 1024, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 32768, },
  { 32, 64, 128, 1024, 32768, },
  { 1, 32, 64, 128, 1024, 32768, },
  { 2, 32, 64, 128, 1024, 32768, },
  { 1, 2, 32, 64, 128, 1024, 32768, },
  { 4, 32, 64, 128, 1024, 32768, },
  { 1, 4, 32, 64, 128, 1024, 32768, },
  { 2, 4, 32, 64, 128, 1024, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 32768, },
  { 8, 32, 64, 128, 1024, 32768, },
  { 1, 8, 32, 64, 128, 1024, 32768, },
  { 2, 8, 32, 64, 128, 1024, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 32768, },
  { 4, 8, 32, 64, 128, 1024, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 32768, },
  { 16, 32, 64, 128, 1024, 32768, },
  { 1, 16, 32, 64, 128, 1024, 32768, },
  { 2, 16, 32, 64, 128, 1024, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 32768, },
  { 4, 16, 32, 64, 128, 1024, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 32768, },
  { 8, 16, 32, 64, 128, 1024, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 32768, },
  { 256, 1024, 32768, },
  { 1, 256, 1024, 32768, },
  { 2, 256, 1024, 32768, },
  { 1, 2, 256, 1024, 32768, },
  { 4, 256, 1024, 32768, },
  { 1, 4, 256, 1024, 32768, },
  { 2, 4, 256, 1024, 32768, },
  { 1, 2, 4, 256, 1024, 32768, },
  { 8, 256, 1024, 32768, },
  { 1, 8, 256, 1024, 32768, },
  { 2, 8, 256, 1024, 32768, },
  { 1, 2, 8, 256, 1024, 32768, },
  { 4, 8, 256, 1024, 32768, },
  { 1, 4, 8, 256, 1024, 32768, },
  { 2, 4, 8, 256, 1024, 32768, },
  { 1, 2, 4, 8, 256, 1024, 32768, },
  { 16, 256, 1024, 32768, },
  { 1, 16, 256, 1024, 32768, },
  { 2, 16, 256, 1024, 32768, },
  { 1, 2, 16, 256, 1024, 32768, },
  { 4, 16, 256, 1024, 32768, },
  { 1, 4, 16, 256, 1024, 32768, },
  { 2, 4, 16, 256, 1024, 32768, },
  { 1, 2, 4, 16, 256, 1024, 32768, },
  { 8, 16, 256, 1024, 32768, },
  { 1, 8, 16, 256, 1024, 32768, },
  { 2, 8, 16, 256, 1024, 32768, },
  { 1, 2, 8, 16, 256, 1024, 32768, },
  { 4, 8, 16, 256, 1024, 32768, },
  { 1, 4, 8, 16, 256, 1024, 32768, },
  { 2, 4, 8, 16, 256, 1024, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 32768, },
  { 32, 256, 1024, 32768, },
  { 1, 32, 256, 1024, 32768, },
  { 2, 32, 256, 1024, 32768, },
  { 1, 2, 32, 256, 1024, 32768, },
  { 4, 32, 256, 1024, 32768, },
  { 1, 4, 32, 256, 1024, 32768, },
  { 2, 4, 32, 256, 1024, 32768, },
  { 1, 2, 4, 32, 256, 1024, 32768, },
  { 8, 32, 256, 1024, 32768, },
  { 1, 8, 32, 256, 1024, 32768, },
  { 2, 8, 32, 256, 1024, 32768, },
  { 1, 2, 8, 32, 256, 1024, 32768, },
  { 4, 8, 32, 256, 1024, 32768, },
  { 1, 4, 8, 32, 256, 1024, 32768, },
  { 2, 4, 8, 32, 256, 1024, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 32768, },
  { 16, 32, 256, 1024, 32768, },
  { 1, 16, 32, 256, 1024, 32768, },
  { 2, 16, 32, 256, 1024, 32768, },
  { 1, 2, 16, 32, 256, 1024, 32768, },
  { 4, 16, 32, 256, 1024, 32768, },
  { 1, 4, 16, 32, 256, 1024, 32768, },
  { 2, 4, 16, 32, 256, 1024, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 32768, },
  { 8, 16, 32, 256, 1024, 32768, },
  { 1, 8, 16, 32, 256, 1024, 32768, },
  { 2, 8, 16, 32, 256, 1024, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 32768, },
  { 4, 8, 16, 32, 256, 1024, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 32768, },
  { 64, 256, 1024, 32768, },
  { 1, 64, 256, 1024, 32768, },
  { 2, 64, 256, 1024, 32768, },
  { 1, 2, 64, 256, 1024, 32768, },
  { 4, 64, 256, 1024, 32768, },
  { 1, 4, 64, 256, 1024, 32768, },
  { 2, 4, 64, 256, 1024, 32768, },
  { 1, 2, 4, 64, 256, 1024, 32768, },
  { 8, 64, 256, 1024, 32768, },
  { 1, 8, 64, 256, 1024, 32768, },
  { 2, 8, 64, 256, 1024, 32768, },
  { 1, 2, 8, 64, 256, 1024, 32768, },
  { 4, 8, 64, 256, 1024, 32768, },
  { 1, 4, 8, 64, 256, 1024, 32768, },
  { 2, 4, 8, 64, 256, 1024, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 32768, },
  { 16, 64, 256, 1024, 32768, },
  { 1, 16, 64, 256, 1024, 32768, },
  { 2, 16, 64, 256, 1024, 32768, },
  { 1, 2, 16, 64, 256, 1024, 32768, },
  { 4, 16, 64, 256, 1024, 32768, },
  { 1, 4, 16, 64, 256, 1024, 32768, },
  { 2, 4, 16, 64, 256, 1024, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 32768, },
  { 8, 16, 64, 256, 1024, 32768, },
  { 1, 8, 16, 64, 256, 1024, 32768, },
  { 2, 8, 16, 64, 256, 1024, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 32768, },
  { 4, 8, 16, 64, 256, 1024, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 32768, },
  { 32, 64, 256, 1024, 32768, },
  { 1, 32, 64, 256, 1024, 32768, },
  { 2, 32, 64, 256, 1024, 32768, },
  { 1, 2, 32, 64, 256, 1024, 32768, },
  { 4, 32, 64, 256, 1024, 32768, },
  { 1, 4, 32, 64, 256, 1024, 32768, },
  { 2, 4, 32, 64, 256, 1024, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 32768, },
  { 8, 32, 64, 256, 1024, 32768, },
  { 1, 8, 32, 64, 256, 1024, 32768, },
  { 2, 8, 32, 64, 256, 1024, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 32768, },
  { 4, 8, 32, 64, 256, 1024, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 32768, },
  { 16, 32, 64, 256, 1024, 32768, },
  { 1, 16, 32, 64, 256, 1024, 32768, },
  { 2, 16, 32, 64, 256, 1024, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 32768, },
  { 4, 16, 32, 64, 256, 1024, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 32768, },
  { 8, 16, 32, 64, 256, 1024, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 32768, },
  { 128, 256, 1024, 32768, },
  { 1, 128, 256, 1024, 32768, },
  { 2, 128, 256, 1024, 32768, },
  { 1, 2, 128, 256, 1024, 32768, },
  { 4, 128, 256, 1024, 32768, },
  { 1, 4, 128, 256, 1024, 32768, },
  { 2, 4, 128, 256, 1024, 32768, },
  { 1, 2, 4, 128, 256, 1024, 32768, },
  { 8, 128, 256, 1024, 32768, },
  { 1, 8, 128, 256, 1024, 32768, },
  { 2, 8, 128, 256, 1024, 32768, },
  { 1, 2, 8, 128, 256, 1024, 32768, },
  { 4, 8, 128, 256, 1024, 32768, },
  { 1, 4, 8, 128, 256, 1024, 32768, },
  { 2, 4, 8, 128, 256, 1024, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 32768, },
  { 16, 128, 256, 1024, 32768, },
  { 1, 16, 128, 256, 1024, 32768, },
  { 2, 16, 128, 256, 1024, 32768, },
  { 1, 2, 16, 128, 256, 1024, 32768, },
  { 4, 16, 128, 256, 1024, 32768, },
  { 1, 4, 16, 128, 256, 1024, 32768, },
  { 2, 4, 16, 128, 256, 1024, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 32768, },
  { 8, 16, 128, 256, 1024, 32768, },
  { 1, 8, 16, 128, 256, 1024, 32768, },
  { 2, 8, 16, 128, 256, 1024, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 32768, },
  { 4, 8, 16, 128, 256, 1024, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 32768, },
  { 32, 128, 256, 1024, 32768, },
  { 1, 32, 128, 256, 1024, 32768, },
  { 2, 32, 128, 256, 1024, 32768, },
  { 1, 2, 32, 128, 256, 1024, 32768, },
  { 4, 32, 128, 256, 1024, 32768, },
  { 1, 4, 32, 128, 256, 1024, 32768, },
  { 2, 4, 32, 128, 256, 1024, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 32768, },
  { 8, 32, 128, 256, 1024, 32768, },
  { 1, 8, 32, 128, 256, 1024, 32768, },
  { 2, 8, 32, 128, 256, 1024, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 32768, },
  { 4, 8, 32, 128, 256, 1024, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 32768, },
  { 16, 32, 128, 256, 1024, 32768, },
  { 1, 16, 32, 128, 256, 1024, 32768, },
  { 2, 16, 32, 128, 256, 1024, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 32768, },
  { 4, 16, 32, 128, 256, 1024, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 32768, },
  { 8, 16, 32, 128, 256, 1024, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 32768, },
  { 64, 128, 256, 1024, 32768, },
  { 1, 64, 128, 256, 1024, 32768, },
  { 2, 64, 128, 256, 1024, 32768, },
  { 1, 2, 64, 128, 256, 1024, 32768, },
  { 4, 64, 128, 256, 1024, 32768, },
  { 1, 4, 64, 128, 256, 1024, 32768, },
  { 2, 4, 64, 128, 256, 1024, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 32768, },
  { 8, 64, 128, 256, 1024, 32768, },
  { 1, 8, 64, 128, 256, 1024, 32768, },
  { 2, 8, 64, 128, 256, 1024, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 32768, },
  { 4, 8, 64, 128, 256, 1024, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 32768, },
  { 16, 64, 128, 256, 1024, 32768, },
  { 1, 16, 64, 128, 256, 1024, 32768, },
  { 2, 16, 64, 128, 256, 1024, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 32768, },
  { 4, 16, 64, 128, 256, 1024, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 32768, },
  { 8, 16, 64, 128, 256, 1024, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 32768, },
  { 32, 64, 128, 256, 1024, 32768, },
  { 1, 32, 64, 128, 256, 1024, 32768, },
  { 2, 32, 64, 128, 256, 1024, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 32768, },
  { 4, 32, 64, 128, 256, 1024, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 32768, },
  { 8, 32, 64, 128, 256, 1024, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 32768, },
  { 16, 32, 64, 128, 256, 1024, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 32768, },
  { 512, 1024, 32768, },
  { 1, 512, 1024, 32768, },
  { 2, 512, 1024, 32768, },
  { 1, 2, 512, 1024, 32768, },
  { 4, 512, 1024, 32768, },
  { 1, 4, 512, 1024, 32768, },
  { 2, 4, 512, 1024, 32768, },
  { 1, 2, 4, 512, 1024, 32768, },
  { 8, 512, 1024, 32768, },
  { 1, 8, 512, 1024, 32768, },
  { 2, 8, 512, 1024, 32768, },
  { 1, 2, 8, 512, 1024, 32768, },
  { 4, 8, 512, 1024, 32768, },
  { 1, 4, 8, 512, 1024, 32768, },
  { 2, 4, 8, 512, 1024, 32768, },
  { 1, 2, 4, 8, 512, 1024, 32768, },
  { 16, 512, 1024, 32768, },
  { 1, 16, 512, 1024, 32768, },
  { 2, 16, 512, 1024, 32768, },
  { 1, 2, 16, 512, 1024, 32768, },
  { 4, 16, 512, 1024, 32768, },
  { 1, 4, 16, 512, 1024, 32768, },
  { 2, 4, 16, 512, 1024, 32768, },
  { 1, 2, 4, 16, 512, 1024, 32768, },
  { 8, 16, 512, 1024, 32768, },
  { 1, 8, 16, 512, 1024, 32768, },
  { 2, 8, 16, 512, 1024, 32768, },
  { 1, 2, 8, 16, 512, 1024, 32768, },
  { 4, 8, 16, 512, 1024, 32768, },
  { 1, 4, 8, 16, 512, 1024, 32768, },
  { 2, 4, 8, 16, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 32768, },
  { 32, 512, 1024, 32768, },
  { 1, 32, 512, 1024, 32768, },
  { 2, 32, 512, 1024, 32768, },
  { 1, 2, 32, 512, 1024, 32768, },
  { 4, 32, 512, 1024, 32768, },
  { 1, 4, 32, 512, 1024, 32768, },
  { 2, 4, 32, 512, 1024, 32768, },
  { 1, 2, 4, 32, 512, 1024, 32768, },
  { 8, 32, 512, 1024, 32768, },
  { 1, 8, 32, 512, 1024, 32768, },
  { 2, 8, 32, 512, 1024, 32768, },
  { 1, 2, 8, 32, 512, 1024, 32768, },
  { 4, 8, 32, 512, 1024, 32768, },
  { 1, 4, 8, 32, 512, 1024, 32768, },
  { 2, 4, 8, 32, 512, 1024, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 32768, },
  { 16, 32, 512, 1024, 32768, },
  { 1, 16, 32, 512, 1024, 32768, },
  { 2, 16, 32, 512, 1024, 32768, },
  { 1, 2, 16, 32, 512, 1024, 32768, },
  { 4, 16, 32, 512, 1024, 32768, },
  { 1, 4, 16, 32, 512, 1024, 32768, },
  { 2, 4, 16, 32, 512, 1024, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 32768, },
  { 8, 16, 32, 512, 1024, 32768, },
  { 1, 8, 16, 32, 512, 1024, 32768, },
  { 2, 8, 16, 32, 512, 1024, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 32768, },
  { 4, 8, 16, 32, 512, 1024, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 32768, },
  { 64, 512, 1024, 32768, },
  { 1, 64, 512, 1024, 32768, },
  { 2, 64, 512, 1024, 32768, },
  { 1, 2, 64, 512, 1024, 32768, },
  { 4, 64, 512, 1024, 32768, },
  { 1, 4, 64, 512, 1024, 32768, },
  { 2, 4, 64, 512, 1024, 32768, },
  { 1, 2, 4, 64, 512, 1024, 32768, },
  { 8, 64, 512, 1024, 32768, },
  { 1, 8, 64, 512, 1024, 32768, },
  { 2, 8, 64, 512, 1024, 32768, },
  { 1, 2, 8, 64, 512, 1024, 32768, },
  { 4, 8, 64, 512, 1024, 32768, },
  { 1, 4, 8, 64, 512, 1024, 32768, },
  { 2, 4, 8, 64, 512, 1024, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 32768, },
  { 16, 64, 512, 1024, 32768, },
  { 1, 16, 64, 512, 1024, 32768, },
  { 2, 16, 64, 512, 1024, 32768, },
  { 1, 2, 16, 64, 512, 1024, 32768, },
  { 4, 16, 64, 512, 1024, 32768, },
  { 1, 4, 16, 64, 512, 1024, 32768, },
  { 2, 4, 16, 64, 512, 1024, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 32768, },
  { 8, 16, 64, 512, 1024, 32768, },
  { 1, 8, 16, 64, 512, 1024, 32768, },
  { 2, 8, 16, 64, 512, 1024, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 32768, },
  { 4, 8, 16, 64, 512, 1024, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 32768, },
  { 32, 64, 512, 1024, 32768, },
  { 1, 32, 64, 512, 1024, 32768, },
  { 2, 32, 64, 512, 1024, 32768, },
  { 1, 2, 32, 64, 512, 1024, 32768, },
  { 4, 32, 64, 512, 1024, 32768, },
  { 1, 4, 32, 64, 512, 1024, 32768, },
  { 2, 4, 32, 64, 512, 1024, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 32768, },
  { 8, 32, 64, 512, 1024, 32768, },
  { 1, 8, 32, 64, 512, 1024, 32768, },
  { 2, 8, 32, 64, 512, 1024, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 32768, },
  { 4, 8, 32, 64, 512, 1024, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 32768, },
  { 16, 32, 64, 512, 1024, 32768, },
  { 1, 16, 32, 64, 512, 1024, 32768, },
  { 2, 16, 32, 64, 512, 1024, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 32768, },
  { 4, 16, 32, 64, 512, 1024, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 32768, },
  { 8, 16, 32, 64, 512, 1024, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 32768, },
  { 128, 512, 1024, 32768, },
  { 1, 128, 512, 1024, 32768, },
  { 2, 128, 512, 1024, 32768, },
  { 1, 2, 128, 512, 1024, 32768, },
  { 4, 128, 512, 1024, 32768, },
  { 1, 4, 128, 512, 1024, 32768, },
  { 2, 4, 128, 512, 1024, 32768, },
  { 1, 2, 4, 128, 512, 1024, 32768, },
  { 8, 128, 512, 1024, 32768, },
  { 1, 8, 128, 512, 1024, 32768, },
  { 2, 8, 128, 512, 1024, 32768, },
  { 1, 2, 8, 128, 512, 1024, 32768, },
  { 4, 8, 128, 512, 1024, 32768, },
  { 1, 4, 8, 128, 512, 1024, 32768, },
  { 2, 4, 8, 128, 512, 1024, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 32768, },
  { 16, 128, 512, 1024, 32768, },
  { 1, 16, 128, 512, 1024, 32768, },
  { 2, 16, 128, 512, 1024, 32768, },
  { 1, 2, 16, 128, 512, 1024, 32768, },
  { 4, 16, 128, 512, 1024, 32768, },
  { 1, 4, 16, 128, 512, 1024, 32768, },
  { 2, 4, 16, 128, 512, 1024, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 32768, },
  { 8, 16, 128, 512, 1024, 32768, },
  { 1, 8, 16, 128, 512, 1024, 32768, },
  { 2, 8, 16, 128, 512, 1024, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 32768, },
  { 4, 8, 16, 128, 512, 1024, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 32768, },
  { 32, 128, 512, 1024, 32768, },
  { 1, 32, 128, 512, 1024, 32768, },
  { 2, 32, 128, 512, 1024, 32768, },
  { 1, 2, 32, 128, 512, 1024, 32768, },
  { 4, 32, 128, 512, 1024, 32768, },
  { 1, 4, 32, 128, 512, 1024, 32768, },
  { 2, 4, 32, 128, 512, 1024, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 32768, },
  { 8, 32, 128, 512, 1024, 32768, },
  { 1, 8, 32, 128, 512, 1024, 32768, },
  { 2, 8, 32, 128, 512, 1024, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 32768, },
  { 4, 8, 32, 128, 512, 1024, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 32768, },
  { 16, 32, 128, 512, 1024, 32768, },
  { 1, 16, 32, 128, 512, 1024, 32768, },
  { 2, 16, 32, 128, 512, 1024, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 32768, },
  { 4, 16, 32, 128, 512, 1024, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 32768, },
  { 8, 16, 32, 128, 512, 1024, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 32768, },
  { 64, 128, 512, 1024, 32768, },
  { 1, 64, 128, 512, 1024, 32768, },
  { 2, 64, 128, 512, 1024, 32768, },
  { 1, 2, 64, 128, 512, 1024, 32768, },
  { 4, 64, 128, 512, 1024, 32768, },
  { 1, 4, 64, 128, 512, 1024, 32768, },
  { 2, 4, 64, 128, 512, 1024, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 32768, },
  { 8, 64, 128, 512, 1024, 32768, },
  { 1, 8, 64, 128, 512, 1024, 32768, },
  { 2, 8, 64, 128, 512, 1024, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 32768, },
  { 4, 8, 64, 128, 512, 1024, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 32768, },
  { 16, 64, 128, 512, 1024, 32768, },
  { 1, 16, 64, 128, 512, 1024, 32768, },
  { 2, 16, 64, 128, 512, 1024, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 32768, },
  { 4, 16, 64, 128, 512, 1024, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 32768, },
  { 8, 16, 64, 128, 512, 1024, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 32768, },
  { 32, 64, 128, 512, 1024, 32768, },
  { 1, 32, 64, 128, 512, 1024, 32768, },
  { 2, 32, 64, 128, 512, 1024, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 32768, },
  { 4, 32, 64, 128, 512, 1024, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 32768, },
  { 8, 32, 64, 128, 512, 1024, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 32768, },
  { 16, 32, 64, 128, 512, 1024, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 32768, },
  { 256, 512, 1024, 32768, },
  { 1, 256, 512, 1024, 32768, },
  { 2, 256, 512, 1024, 32768, },
  { 1, 2, 256, 512, 1024, 32768, },
  { 4, 256, 512, 1024, 32768, },
  { 1, 4, 256, 512, 1024, 32768, },
  { 2, 4, 256, 512, 1024, 32768, },
  { 1, 2, 4, 256, 512, 1024, 32768, },
  { 8, 256, 512, 1024, 32768, },
  { 1, 8, 256, 512, 1024, 32768, },
  { 2, 8, 256, 512, 1024, 32768, },
  { 1, 2, 8, 256, 512, 1024, 32768, },
  { 4, 8, 256, 512, 1024, 32768, },
  { 1, 4, 8, 256, 512, 1024, 32768, },
  { 2, 4, 8, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 32768, },
  { 16, 256, 512, 1024, 32768, },
  { 1, 16, 256, 512, 1024, 32768, },
  { 2, 16, 256, 512, 1024, 32768, },
  { 1, 2, 16, 256, 512, 1024, 32768, },
  { 4, 16, 256, 512, 1024, 32768, },
  { 1, 4, 16, 256, 512, 1024, 32768, },
  { 2, 4, 16, 256, 512, 1024, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 32768, },
  { 8, 16, 256, 512, 1024, 32768, },
  { 1, 8, 16, 256, 512, 1024, 32768, },
  { 2, 8, 16, 256, 512, 1024, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 32768, },
  { 4, 8, 16, 256, 512, 1024, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 32768, },
  { 32, 256, 512, 1024, 32768, },
  { 1, 32, 256, 512, 1024, 32768, },
  { 2, 32, 256, 512, 1024, 32768, },
  { 1, 2, 32, 256, 512, 1024, 32768, },
  { 4, 32, 256, 512, 1024, 32768, },
  { 1, 4, 32, 256, 512, 1024, 32768, },
  { 2, 4, 32, 256, 512, 1024, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 32768, },
  { 8, 32, 256, 512, 1024, 32768, },
  { 1, 8, 32, 256, 512, 1024, 32768, },
  { 2, 8, 32, 256, 512, 1024, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 32768, },
  { 4, 8, 32, 256, 512, 1024, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 32768, },
  { 16, 32, 256, 512, 1024, 32768, },
  { 1, 16, 32, 256, 512, 1024, 32768, },
  { 2, 16, 32, 256, 512, 1024, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 32768, },
  { 4, 16, 32, 256, 512, 1024, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 32768, },
  { 8, 16, 32, 256, 512, 1024, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 32768, },
  { 64, 256, 512, 1024, 32768, },
  { 1, 64, 256, 512, 1024, 32768, },
  { 2, 64, 256, 512, 1024, 32768, },
  { 1, 2, 64, 256, 512, 1024, 32768, },
  { 4, 64, 256, 512, 1024, 32768, },
  { 1, 4, 64, 256, 512, 1024, 32768, },
  { 2, 4, 64, 256, 512, 1024, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 32768, },
  { 8, 64, 256, 512, 1024, 32768, },
  { 1, 8, 64, 256, 512, 1024, 32768, },
  { 2, 8, 64, 256, 512, 1024, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 32768, },
  { 4, 8, 64, 256, 512, 1024, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 32768, },
  { 16, 64, 256, 512, 1024, 32768, },
  { 1, 16, 64, 256, 512, 1024, 32768, },
  { 2, 16, 64, 256, 512, 1024, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 32768, },
  { 4, 16, 64, 256, 512, 1024, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 32768, },
  { 8, 16, 64, 256, 512, 1024, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 32768, },
  { 32, 64, 256, 512, 1024, 32768, },
  { 1, 32, 64, 256, 512, 1024, 32768, },
  { 2, 32, 64, 256, 512, 1024, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 32768, },
  { 4, 32, 64, 256, 512, 1024, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 32768, },
  { 8, 32, 64, 256, 512, 1024, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 32768, },
  { 16, 32, 64, 256, 512, 1024, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 32768, },
  { 128, 256, 512, 1024, 32768, },
  { 1, 128, 256, 512, 1024, 32768, },
  { 2, 128, 256, 512, 1024, 32768, },
  { 1, 2, 128, 256, 512, 1024, 32768, },
  { 4, 128, 256, 512, 1024, 32768, },
  { 1, 4, 128, 256, 512, 1024, 32768, },
  { 2, 4, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 32768, },
  { 8, 128, 256, 512, 1024, 32768, },
  { 1, 8, 128, 256, 512, 1024, 32768, },
  { 2, 8, 128, 256, 512, 1024, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 32768, },
  { 4, 8, 128, 256, 512, 1024, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 32768, },
  { 16, 128, 256, 512, 1024, 32768, },
  { 1, 16, 128, 256, 512, 1024, 32768, },
  { 2, 16, 128, 256, 512, 1024, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 32768, },
  { 4, 16, 128, 256, 512, 1024, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 32768, },
  { 8, 16, 128, 256, 512, 1024, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 32768, },
  { 32, 128, 256, 512, 1024, 32768, },
  { 1, 32, 128, 256, 512, 1024, 32768, },
  { 2, 32, 128, 256, 512, 1024, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 32768, },
  { 4, 32, 128, 256, 512, 1024, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 32768, },
  { 8, 32, 128, 256, 512, 1024, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 32768, },
  { 16, 32, 128, 256, 512, 1024, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 32768, },
  { 64, 128, 256, 512, 1024, 32768, },
  { 1, 64, 128, 256, 512, 1024, 32768, },
  { 2, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 32768, },
  { 4, 64, 128, 256, 512, 1024, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 32768, },
  { 8, 64, 128, 256, 512, 1024, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 32768, },
  { 16, 64, 128, 256, 512, 1024, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 32768, },
  { 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 32768, },
  { 2048, 32768, },
  { 1, 2048, 32768, },
  { 2, 2048, 32768, },
  { 1, 2, 2048, 32768, },
  { 4, 2048, 32768, },
  { 1, 4, 2048, 32768, },
  { 2, 4, 2048, 32768, },
  { 1, 2, 4, 2048, 32768, },
  { 8, 2048, 32768, },
  { 1, 8, 2048, 32768, },
  { 2, 8, 2048, 32768, },
  { 1, 2, 8, 2048, 32768, },
  { 4, 8, 2048, 32768, },
  { 1, 4, 8, 2048, 32768, },
  { 2, 4, 8, 2048, 32768, },
  { 1, 2, 4, 8, 2048, 32768, },
  { 16, 2048, 32768, },
  { 1, 16, 2048, 32768, },
  { 2, 16, 2048, 32768, },
  { 1, 2, 16, 2048, 32768, },
  { 4, 16, 2048, 32768, },
  { 1, 4, 16, 2048, 32768, },
  { 2, 4, 16, 2048, 32768, },
  { 1, 2, 4, 16, 2048, 32768, },
  { 8, 16, 2048, 32768, },
  { 1, 8, 16, 2048, 32768, },
  { 2, 8, 16, 2048, 32768, },
  { 1, 2, 8, 16, 2048, 32768, },
  { 4, 8, 16, 2048, 32768, },
  { 1, 4, 8, 16, 2048, 32768, },
  { 2, 4, 8, 16, 2048, 32768, },
  { 1, 2, 4, 8, 16, 2048, 32768, },
  { 32, 2048, 32768, },
  { 1, 32, 2048, 32768, },
  { 2, 32, 2048, 32768, },
  { 1, 2, 32, 2048, 32768, },
  { 4, 32, 2048, 32768, },
  { 1, 4, 32, 2048, 32768, },
  { 2, 4, 32, 2048, 32768, },
  { 1, 2, 4, 32, 2048, 32768, },
  { 8, 32, 2048, 32768, },
  { 1, 8, 32, 2048, 32768, },
  { 2, 8, 32, 2048, 32768, },
  { 1, 2, 8, 32, 2048, 32768, },
  { 4, 8, 32, 2048, 32768, },
  { 1, 4, 8, 32, 2048, 32768, },
  { 2, 4, 8, 32, 2048, 32768, },
  { 1, 2, 4, 8, 32, 2048, 32768, },
  { 16, 32, 2048, 32768, },
  { 1, 16, 32, 2048, 32768, },
  { 2, 16, 32, 2048, 32768, },
  { 1, 2, 16, 32, 2048, 32768, },
  { 4, 16, 32, 2048, 32768, },
  { 1, 4, 16, 32, 2048, 32768, },
  { 2, 4, 16, 32, 2048, 32768, },
  { 1, 2, 4, 16, 32, 2048, 32768, },
  { 8, 16, 32, 2048, 32768, },
  { 1, 8, 16, 32, 2048, 32768, },
  { 2, 8, 16, 32, 2048, 32768, },
  { 1, 2, 8, 16, 32, 2048, 32768, },
  { 4, 8, 16, 32, 2048, 32768, },
  { 1, 4, 8, 16, 32, 2048, 32768, },
  { 2, 4, 8, 16, 32, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 2048, 32768, },
  { 64, 2048, 32768, },
  { 1, 64, 2048, 32768, },
  { 2, 64, 2048, 32768, },
  { 1, 2, 64, 2048, 32768, },
  { 4, 64, 2048, 32768, },
  { 1, 4, 64, 2048, 32768, },
  { 2, 4, 64, 2048, 32768, },
  { 1, 2, 4, 64, 2048, 32768, },
  { 8, 64, 2048, 32768, },
  { 1, 8, 64, 2048, 32768, },
  { 2, 8, 64, 2048, 32768, },
  { 1, 2, 8, 64, 2048, 32768, },
  { 4, 8, 64, 2048, 32768, },
  { 1, 4, 8, 64, 2048, 32768, },
  { 2, 4, 8, 64, 2048, 32768, },
  { 1, 2, 4, 8, 64, 2048, 32768, },
  { 16, 64, 2048, 32768, },
  { 1, 16, 64, 2048, 32768, },
  { 2, 16, 64, 2048, 32768, },
  { 1, 2, 16, 64, 2048, 32768, },
  { 4, 16, 64, 2048, 32768, },
  { 1, 4, 16, 64, 2048, 32768, },
  { 2, 4, 16, 64, 2048, 32768, },
  { 1, 2, 4, 16, 64, 2048, 32768, },
  { 8, 16, 64, 2048, 32768, },
  { 1, 8, 16, 64, 2048, 32768, },
  { 2, 8, 16, 64, 2048, 32768, },
  { 1, 2, 8, 16, 64, 2048, 32768, },
  { 4, 8, 16, 64, 2048, 32768, },
  { 1, 4, 8, 16, 64, 2048, 32768, },
  { 2, 4, 8, 16, 64, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 2048, 32768, },
  { 32, 64, 2048, 32768, },
  { 1, 32, 64, 2048, 32768, },
  { 2, 32, 64, 2048, 32768, },
  { 1, 2, 32, 64, 2048, 32768, },
  { 4, 32, 64, 2048, 32768, },
  { 1, 4, 32, 64, 2048, 32768, },
  { 2, 4, 32, 64, 2048, 32768, },
  { 1, 2, 4, 32, 64, 2048, 32768, },
  { 8, 32, 64, 2048, 32768, },
  { 1, 8, 32, 64, 2048, 32768, },
  { 2, 8, 32, 64, 2048, 32768, },
  { 1, 2, 8, 32, 64, 2048, 32768, },
  { 4, 8, 32, 64, 2048, 32768, },
  { 1, 4, 8, 32, 64, 2048, 32768, },
  { 2, 4, 8, 32, 64, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 2048, 32768, },
  { 16, 32, 64, 2048, 32768, },
  { 1, 16, 32, 64, 2048, 32768, },
  { 2, 16, 32, 64, 2048, 32768, },
  { 1, 2, 16, 32, 64, 2048, 32768, },
  { 4, 16, 32, 64, 2048, 32768, },
  { 1, 4, 16, 32, 64, 2048, 32768, },
  { 2, 4, 16, 32, 64, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 2048, 32768, },
  { 8, 16, 32, 64, 2048, 32768, },
  { 1, 8, 16, 32, 64, 2048, 32768, },
  { 2, 8, 16, 32, 64, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 2048, 32768, },
  { 4, 8, 16, 32, 64, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 32768, },
  { 128, 2048, 32768, },
  { 1, 128, 2048, 32768, },
  { 2, 128, 2048, 32768, },
  { 1, 2, 128, 2048, 32768, },
  { 4, 128, 2048, 32768, },
  { 1, 4, 128, 2048, 32768, },
  { 2, 4, 128, 2048, 32768, },
  { 1, 2, 4, 128, 2048, 32768, },
  { 8, 128, 2048, 32768, },
  { 1, 8, 128, 2048, 32768, },
  { 2, 8, 128, 2048, 32768, },
  { 1, 2, 8, 128, 2048, 32768, },
  { 4, 8, 128, 2048, 32768, },
  { 1, 4, 8, 128, 2048, 32768, },
  { 2, 4, 8, 128, 2048, 32768, },
  { 1, 2, 4, 8, 128, 2048, 32768, },
  { 16, 128, 2048, 32768, },
  { 1, 16, 128, 2048, 32768, },
  { 2, 16, 128, 2048, 32768, },
  { 1, 2, 16, 128, 2048, 32768, },
  { 4, 16, 128, 2048, 32768, },
  { 1, 4, 16, 128, 2048, 32768, },
  { 2, 4, 16, 128, 2048, 32768, },
  { 1, 2, 4, 16, 128, 2048, 32768, },
  { 8, 16, 128, 2048, 32768, },
  { 1, 8, 16, 128, 2048, 32768, },
  { 2, 8, 16, 128, 2048, 32768, },
  { 1, 2, 8, 16, 128, 2048, 32768, },
  { 4, 8, 16, 128, 2048, 32768, },
  { 1, 4, 8, 16, 128, 2048, 32768, },
  { 2, 4, 8, 16, 128, 2048, 32768, },
  { 1, 2, 4, 8, 16, 128, 2048, 32768, },
  { 32, 128, 2048, 32768, },
  { 1, 32, 128, 2048, 32768, },
  { 2, 32, 128, 2048, 32768, },
  { 1, 2, 32, 128, 2048, 32768, },
  { 4, 32, 128, 2048, 32768, },
  { 1, 4, 32, 128, 2048, 32768, },
  { 2, 4, 32, 128, 2048, 32768, },
  { 1, 2, 4, 32, 128, 2048, 32768, },
  { 8, 32, 128, 2048, 32768, },
  { 1, 8, 32, 128, 2048, 32768, },
  { 2, 8, 32, 128, 2048, 32768, },
  { 1, 2, 8, 32, 128, 2048, 32768, },
  { 4, 8, 32, 128, 2048, 32768, },
  { 1, 4, 8, 32, 128, 2048, 32768, },
  { 2, 4, 8, 32, 128, 2048, 32768, },
  { 1, 2, 4, 8, 32, 128, 2048, 32768, },
  { 16, 32, 128, 2048, 32768, },
  { 1, 16, 32, 128, 2048, 32768, },
  { 2, 16, 32, 128, 2048, 32768, },
  { 1, 2, 16, 32, 128, 2048, 32768, },
  { 4, 16, 32, 128, 2048, 32768, },
  { 1, 4, 16, 32, 128, 2048, 32768, },
  { 2, 4, 16, 32, 128, 2048, 32768, },
  { 1, 2, 4, 16, 32, 128, 2048, 32768, },
  { 8, 16, 32, 128, 2048, 32768, },
  { 1, 8, 16, 32, 128, 2048, 32768, },
  { 2, 8, 16, 32, 128, 2048, 32768, },
  { 1, 2, 8, 16, 32, 128, 2048, 32768, },
  { 4, 8, 16, 32, 128, 2048, 32768, },
  { 1, 4, 8, 16, 32, 128, 2048, 32768, },
  { 2, 4, 8, 16, 32, 128, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 32768, },
  { 64, 128, 2048, 32768, },
  { 1, 64, 128, 2048, 32768, },
  { 2, 64, 128, 2048, 32768, },
  { 1, 2, 64, 128, 2048, 32768, },
  { 4, 64, 128, 2048, 32768, },
  { 1, 4, 64, 128, 2048, 32768, },
  { 2, 4, 64, 128, 2048, 32768, },
  { 1, 2, 4, 64, 128, 2048, 32768, },
  { 8, 64, 128, 2048, 32768, },
  { 1, 8, 64, 128, 2048, 32768, },
  { 2, 8, 64, 128, 2048, 32768, },
  { 1, 2, 8, 64, 128, 2048, 32768, },
  { 4, 8, 64, 128, 2048, 32768, },
  { 1, 4, 8, 64, 128, 2048, 32768, },
  { 2, 4, 8, 64, 128, 2048, 32768, },
  { 1, 2, 4, 8, 64, 128, 2048, 32768, },
  { 16, 64, 128, 2048, 32768, },
  { 1, 16, 64, 128, 2048, 32768, },
  { 2, 16, 64, 128, 2048, 32768, },
  { 1, 2, 16, 64, 128, 2048, 32768, },
  { 4, 16, 64, 128, 2048, 32768, },
  { 1, 4, 16, 64, 128, 2048, 32768, },
  { 2, 4, 16, 64, 128, 2048, 32768, },
  { 1, 2, 4, 16, 64, 128, 2048, 32768, },
  { 8, 16, 64, 128, 2048, 32768, },
  { 1, 8, 16, 64, 128, 2048, 32768, },
  { 2, 8, 16, 64, 128, 2048, 32768, },
  { 1, 2, 8, 16, 64, 128, 2048, 32768, },
  { 4, 8, 16, 64, 128, 2048, 32768, },
  { 1, 4, 8, 16, 64, 128, 2048, 32768, },
  { 2, 4, 8, 16, 64, 128, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 32768, },
  { 32, 64, 128, 2048, 32768, },
  { 1, 32, 64, 128, 2048, 32768, },
  { 2, 32, 64, 128, 2048, 32768, },
  { 1, 2, 32, 64, 128, 2048, 32768, },
  { 4, 32, 64, 128, 2048, 32768, },
  { 1, 4, 32, 64, 128, 2048, 32768, },
  { 2, 4, 32, 64, 128, 2048, 32768, },
  { 1, 2, 4, 32, 64, 128, 2048, 32768, },
  { 8, 32, 64, 128, 2048, 32768, },
  { 1, 8, 32, 64, 128, 2048, 32768, },
  { 2, 8, 32, 64, 128, 2048, 32768, },
  { 1, 2, 8, 32, 64, 128, 2048, 32768, },
  { 4, 8, 32, 64, 128, 2048, 32768, },
  { 1, 4, 8, 32, 64, 128, 2048, 32768, },
  { 2, 4, 8, 32, 64, 128, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 32768, },
  { 16, 32, 64, 128, 2048, 32768, },
  { 1, 16, 32, 64, 128, 2048, 32768, },
  { 2, 16, 32, 64, 128, 2048, 32768, },
  { 1, 2, 16, 32, 64, 128, 2048, 32768, },
  { 4, 16, 32, 64, 128, 2048, 32768, },
  { 1, 4, 16, 32, 64, 128, 2048, 32768, },
  { 2, 4, 16, 32, 64, 128, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 32768, },
  { 8, 16, 32, 64, 128, 2048, 32768, },
  { 1, 8, 16, 32, 64, 128, 2048, 32768, },
  { 2, 8, 16, 32, 64, 128, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 32768, },
  { 4, 8, 16, 32, 64, 128, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 32768, },
  { 256, 2048, 32768, },
  { 1, 256, 2048, 32768, },
  { 2, 256, 2048, 32768, },
  { 1, 2, 256, 2048, 32768, },
  { 4, 256, 2048, 32768, },
  { 1, 4, 256, 2048, 32768, },
  { 2, 4, 256, 2048, 32768, },
  { 1, 2, 4, 256, 2048, 32768, },
  { 8, 256, 2048, 32768, },
  { 1, 8, 256, 2048, 32768, },
  { 2, 8, 256, 2048, 32768, },
  { 1, 2, 8, 256, 2048, 32768, },
  { 4, 8, 256, 2048, 32768, },
  { 1, 4, 8, 256, 2048, 32768, },
  { 2, 4, 8, 256, 2048, 32768, },
  { 1, 2, 4, 8, 256, 2048, 32768, },
  { 16, 256, 2048, 32768, },
  { 1, 16, 256, 2048, 32768, },
  { 2, 16, 256, 2048, 32768, },
  { 1, 2, 16, 256, 2048, 32768, },
  { 4, 16, 256, 2048, 32768, },
  { 1, 4, 16, 256, 2048, 32768, },
  { 2, 4, 16, 256, 2048, 32768, },
  { 1, 2, 4, 16, 256, 2048, 32768, },
  { 8, 16, 256, 2048, 32768, },
  { 1, 8, 16, 256, 2048, 32768, },
  { 2, 8, 16, 256, 2048, 32768, },
  { 1, 2, 8, 16, 256, 2048, 32768, },
  { 4, 8, 16, 256, 2048, 32768, },
  { 1, 4, 8, 16, 256, 2048, 32768, },
  { 2, 4, 8, 16, 256, 2048, 32768, },
  { 1, 2, 4, 8, 16, 256, 2048, 32768, },
  { 32, 256, 2048, 32768, },
  { 1, 32, 256, 2048, 32768, },
  { 2, 32, 256, 2048, 32768, },
  { 1, 2, 32, 256, 2048, 32768, },
  { 4, 32, 256, 2048, 32768, },
  { 1, 4, 32, 256, 2048, 32768, },
  { 2, 4, 32, 256, 2048, 32768, },
  { 1, 2, 4, 32, 256, 2048, 32768, },
  { 8, 32, 256, 2048, 32768, },
  { 1, 8, 32, 256, 2048, 32768, },
  { 2, 8, 32, 256, 2048, 32768, },
  { 1, 2, 8, 32, 256, 2048, 32768, },
  { 4, 8, 32, 256, 2048, 32768, },
  { 1, 4, 8, 32, 256, 2048, 32768, },
  { 2, 4, 8, 32, 256, 2048, 32768, },
  { 1, 2, 4, 8, 32, 256, 2048, 32768, },
  { 16, 32, 256, 2048, 32768, },
  { 1, 16, 32, 256, 2048, 32768, },
  { 2, 16, 32, 256, 2048, 32768, },
  { 1, 2, 16, 32, 256, 2048, 32768, },
  { 4, 16, 32, 256, 2048, 32768, },
  { 1, 4, 16, 32, 256, 2048, 32768, },
  { 2, 4, 16, 32, 256, 2048, 32768, },
  { 1, 2, 4, 16, 32, 256, 2048, 32768, },
  { 8, 16, 32, 256, 2048, 32768, },
  { 1, 8, 16, 32, 256, 2048, 32768, },
  { 2, 8, 16, 32, 256, 2048, 32768, },
  { 1, 2, 8, 16, 32, 256, 2048, 32768, },
  { 4, 8, 16, 32, 256, 2048, 32768, },
  { 1, 4, 8, 16, 32, 256, 2048, 32768, },
  { 2, 4, 8, 16, 32, 256, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 32768, },
  { 64, 256, 2048, 32768, },
  { 1, 64, 256, 2048, 32768, },
  { 2, 64, 256, 2048, 32768, },
  { 1, 2, 64, 256, 2048, 32768, },
  { 4, 64, 256, 2048, 32768, },
  { 1, 4, 64, 256, 2048, 32768, },
  { 2, 4, 64, 256, 2048, 32768, },
  { 1, 2, 4, 64, 256, 2048, 32768, },
  { 8, 64, 256, 2048, 32768, },
  { 1, 8, 64, 256, 2048, 32768, },
  { 2, 8, 64, 256, 2048, 32768, },
  { 1, 2, 8, 64, 256, 2048, 32768, },
  { 4, 8, 64, 256, 2048, 32768, },
  { 1, 4, 8, 64, 256, 2048, 32768, },
  { 2, 4, 8, 64, 256, 2048, 32768, },
  { 1, 2, 4, 8, 64, 256, 2048, 32768, },
  { 16, 64, 256, 2048, 32768, },
  { 1, 16, 64, 256, 2048, 32768, },
  { 2, 16, 64, 256, 2048, 32768, },
  { 1, 2, 16, 64, 256, 2048, 32768, },
  { 4, 16, 64, 256, 2048, 32768, },
  { 1, 4, 16, 64, 256, 2048, 32768, },
  { 2, 4, 16, 64, 256, 2048, 32768, },
  { 1, 2, 4, 16, 64, 256, 2048, 32768, },
  { 8, 16, 64, 256, 2048, 32768, },
  { 1, 8, 16, 64, 256, 2048, 32768, },
  { 2, 8, 16, 64, 256, 2048, 32768, },
  { 1, 2, 8, 16, 64, 256, 2048, 32768, },
  { 4, 8, 16, 64, 256, 2048, 32768, },
  { 1, 4, 8, 16, 64, 256, 2048, 32768, },
  { 2, 4, 8, 16, 64, 256, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 32768, },
  { 32, 64, 256, 2048, 32768, },
  { 1, 32, 64, 256, 2048, 32768, },
  { 2, 32, 64, 256, 2048, 32768, },
  { 1, 2, 32, 64, 256, 2048, 32768, },
  { 4, 32, 64, 256, 2048, 32768, },
  { 1, 4, 32, 64, 256, 2048, 32768, },
  { 2, 4, 32, 64, 256, 2048, 32768, },
  { 1, 2, 4, 32, 64, 256, 2048, 32768, },
  { 8, 32, 64, 256, 2048, 32768, },
  { 1, 8, 32, 64, 256, 2048, 32768, },
  { 2, 8, 32, 64, 256, 2048, 32768, },
  { 1, 2, 8, 32, 64, 256, 2048, 32768, },
  { 4, 8, 32, 64, 256, 2048, 32768, },
  { 1, 4, 8, 32, 64, 256, 2048, 32768, },
  { 2, 4, 8, 32, 64, 256, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 32768, },
  { 16, 32, 64, 256, 2048, 32768, },
  { 1, 16, 32, 64, 256, 2048, 32768, },
  { 2, 16, 32, 64, 256, 2048, 32768, },
  { 1, 2, 16, 32, 64, 256, 2048, 32768, },
  { 4, 16, 32, 64, 256, 2048, 32768, },
  { 1, 4, 16, 32, 64, 256, 2048, 32768, },
  { 2, 4, 16, 32, 64, 256, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 32768, },
  { 8, 16, 32, 64, 256, 2048, 32768, },
  { 1, 8, 16, 32, 64, 256, 2048, 32768, },
  { 2, 8, 16, 32, 64, 256, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 32768, },
  { 4, 8, 16, 32, 64, 256, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 32768, },
  { 128, 256, 2048, 32768, },
  { 1, 128, 256, 2048, 32768, },
  { 2, 128, 256, 2048, 32768, },
  { 1, 2, 128, 256, 2048, 32768, },
  { 4, 128, 256, 2048, 32768, },
  { 1, 4, 128, 256, 2048, 32768, },
  { 2, 4, 128, 256, 2048, 32768, },
  { 1, 2, 4, 128, 256, 2048, 32768, },
  { 8, 128, 256, 2048, 32768, },
  { 1, 8, 128, 256, 2048, 32768, },
  { 2, 8, 128, 256, 2048, 32768, },
  { 1, 2, 8, 128, 256, 2048, 32768, },
  { 4, 8, 128, 256, 2048, 32768, },
  { 1, 4, 8, 128, 256, 2048, 32768, },
  { 2, 4, 8, 128, 256, 2048, 32768, },
  { 1, 2, 4, 8, 128, 256, 2048, 32768, },
  { 16, 128, 256, 2048, 32768, },
  { 1, 16, 128, 256, 2048, 32768, },
  { 2, 16, 128, 256, 2048, 32768, },
  { 1, 2, 16, 128, 256, 2048, 32768, },
  { 4, 16, 128, 256, 2048, 32768, },
  { 1, 4, 16, 128, 256, 2048, 32768, },
  { 2, 4, 16, 128, 256, 2048, 32768, },
  { 1, 2, 4, 16, 128, 256, 2048, 32768, },
  { 8, 16, 128, 256, 2048, 32768, },
  { 1, 8, 16, 128, 256, 2048, 32768, },
  { 2, 8, 16, 128, 256, 2048, 32768, },
  { 1, 2, 8, 16, 128, 256, 2048, 32768, },
  { 4, 8, 16, 128, 256, 2048, 32768, },
  { 1, 4, 8, 16, 128, 256, 2048, 32768, },
  { 2, 4, 8, 16, 128, 256, 2048, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 32768, },
  { 32, 128, 256, 2048, 32768, },
  { 1, 32, 128, 256, 2048, 32768, },
  { 2, 32, 128, 256, 2048, 32768, },
  { 1, 2, 32, 128, 256, 2048, 32768, },
  { 4, 32, 128, 256, 2048, 32768, },
  { 1, 4, 32, 128, 256, 2048, 32768, },
  { 2, 4, 32, 128, 256, 2048, 32768, },
  { 1, 2, 4, 32, 128, 256, 2048, 32768, },
  { 8, 32, 128, 256, 2048, 32768, },
  { 1, 8, 32, 128, 256, 2048, 32768, },
  { 2, 8, 32, 128, 256, 2048, 32768, },
  { 1, 2, 8, 32, 128, 256, 2048, 32768, },
  { 4, 8, 32, 128, 256, 2048, 32768, },
  { 1, 4, 8, 32, 128, 256, 2048, 32768, },
  { 2, 4, 8, 32, 128, 256, 2048, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 32768, },
  { 16, 32, 128, 256, 2048, 32768, },
  { 1, 16, 32, 128, 256, 2048, 32768, },
  { 2, 16, 32, 128, 256, 2048, 32768, },
  { 1, 2, 16, 32, 128, 256, 2048, 32768, },
  { 4, 16, 32, 128, 256, 2048, 32768, },
  { 1, 4, 16, 32, 128, 256, 2048, 32768, },
  { 2, 4, 16, 32, 128, 256, 2048, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 32768, },
  { 8, 16, 32, 128, 256, 2048, 32768, },
  { 1, 8, 16, 32, 128, 256, 2048, 32768, },
  { 2, 8, 16, 32, 128, 256, 2048, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 32768, },
  { 4, 8, 16, 32, 128, 256, 2048, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 32768, },
  { 64, 128, 256, 2048, 32768, },
  { 1, 64, 128, 256, 2048, 32768, },
  { 2, 64, 128, 256, 2048, 32768, },
  { 1, 2, 64, 128, 256, 2048, 32768, },
  { 4, 64, 128, 256, 2048, 32768, },
  { 1, 4, 64, 128, 256, 2048, 32768, },
  { 2, 4, 64, 128, 256, 2048, 32768, },
  { 1, 2, 4, 64, 128, 256, 2048, 32768, },
  { 8, 64, 128, 256, 2048, 32768, },
  { 1, 8, 64, 128, 256, 2048, 32768, },
  { 2, 8, 64, 128, 256, 2048, 32768, },
  { 1, 2, 8, 64, 128, 256, 2048, 32768, },
  { 4, 8, 64, 128, 256, 2048, 32768, },
  { 1, 4, 8, 64, 128, 256, 2048, 32768, },
  { 2, 4, 8, 64, 128, 256, 2048, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 32768, },
  { 16, 64, 128, 256, 2048, 32768, },
  { 1, 16, 64, 128, 256, 2048, 32768, },
  { 2, 16, 64, 128, 256, 2048, 32768, },
  { 1, 2, 16, 64, 128, 256, 2048, 32768, },
  { 4, 16, 64, 128, 256, 2048, 32768, },
  { 1, 4, 16, 64, 128, 256, 2048, 32768, },
  { 2, 4, 16, 64, 128, 256, 2048, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 32768, },
  { 8, 16, 64, 128, 256, 2048, 32768, },
  { 1, 8, 16, 64, 128, 256, 2048, 32768, },
  { 2, 8, 16, 64, 128, 256, 2048, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 32768, },
  { 4, 8, 16, 64, 128, 256, 2048, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 32768, },
  { 32, 64, 128, 256, 2048, 32768, },
  { 1, 32, 64, 128, 256, 2048, 32768, },
  { 2, 32, 64, 128, 256, 2048, 32768, },
  { 1, 2, 32, 64, 128, 256, 2048, 32768, },
  { 4, 32, 64, 128, 256, 2048, 32768, },
  { 1, 4, 32, 64, 128, 256, 2048, 32768, },
  { 2, 4, 32, 64, 128, 256, 2048, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 32768, },
  { 8, 32, 64, 128, 256, 2048, 32768, },
  { 1, 8, 32, 64, 128, 256, 2048, 32768, },
  { 2, 8, 32, 64, 128, 256, 2048, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 32768, },
  { 4, 8, 32, 64, 128, 256, 2048, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 32768, },
  { 16, 32, 64, 128, 256, 2048, 32768, },
  { 1, 16, 32, 64, 128, 256, 2048, 32768, },
  { 2, 16, 32, 64, 128, 256, 2048, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 32768, },
  { 4, 16, 32, 64, 128, 256, 2048, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 32768, },
  { 8, 16, 32, 64, 128, 256, 2048, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 32768, },
  { 512, 2048, 32768, },
  { 1, 512, 2048, 32768, },
  { 2, 512, 2048, 32768, },
  { 1, 2, 512, 2048, 32768, },
  { 4, 512, 2048, 32768, },
  { 1, 4, 512, 2048, 32768, },
  { 2, 4, 512, 2048, 32768, },
  { 1, 2, 4, 512, 2048, 32768, },
  { 8, 512, 2048, 32768, },
  { 1, 8, 512, 2048, 32768, },
  { 2, 8, 512, 2048, 32768, },
  { 1, 2, 8, 512, 2048, 32768, },
  { 4, 8, 512, 2048, 32768, },
  { 1, 4, 8, 512, 2048, 32768, },
  { 2, 4, 8, 512, 2048, 32768, },
  { 1, 2, 4, 8, 512, 2048, 32768, },
  { 16, 512, 2048, 32768, },
  { 1, 16, 512, 2048, 32768, },
  { 2, 16, 512, 2048, 32768, },
  { 1, 2, 16, 512, 2048, 32768, },
  { 4, 16, 512, 2048, 32768, },
  { 1, 4, 16, 512, 2048, 32768, },
  { 2, 4, 16, 512, 2048, 32768, },
  { 1, 2, 4, 16, 512, 2048, 32768, },
  { 8, 16, 512, 2048, 32768, },
  { 1, 8, 16, 512, 2048, 32768, },
  { 2, 8, 16, 512, 2048, 32768, },
  { 1, 2, 8, 16, 512, 2048, 32768, },
  { 4, 8, 16, 512, 2048, 32768, },
  { 1, 4, 8, 16, 512, 2048, 32768, },
  { 2, 4, 8, 16, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 512, 2048, 32768, },
  { 32, 512, 2048, 32768, },
  { 1, 32, 512, 2048, 32768, },
  { 2, 32, 512, 2048, 32768, },
  { 1, 2, 32, 512, 2048, 32768, },
  { 4, 32, 512, 2048, 32768, },
  { 1, 4, 32, 512, 2048, 32768, },
  { 2, 4, 32, 512, 2048, 32768, },
  { 1, 2, 4, 32, 512, 2048, 32768, },
  { 8, 32, 512, 2048, 32768, },
  { 1, 8, 32, 512, 2048, 32768, },
  { 2, 8, 32, 512, 2048, 32768, },
  { 1, 2, 8, 32, 512, 2048, 32768, },
  { 4, 8, 32, 512, 2048, 32768, },
  { 1, 4, 8, 32, 512, 2048, 32768, },
  { 2, 4, 8, 32, 512, 2048, 32768, },
  { 1, 2, 4, 8, 32, 512, 2048, 32768, },
  { 16, 32, 512, 2048, 32768, },
  { 1, 16, 32, 512, 2048, 32768, },
  { 2, 16, 32, 512, 2048, 32768, },
  { 1, 2, 16, 32, 512, 2048, 32768, },
  { 4, 16, 32, 512, 2048, 32768, },
  { 1, 4, 16, 32, 512, 2048, 32768, },
  { 2, 4, 16, 32, 512, 2048, 32768, },
  { 1, 2, 4, 16, 32, 512, 2048, 32768, },
  { 8, 16, 32, 512, 2048, 32768, },
  { 1, 8, 16, 32, 512, 2048, 32768, },
  { 2, 8, 16, 32, 512, 2048, 32768, },
  { 1, 2, 8, 16, 32, 512, 2048, 32768, },
  { 4, 8, 16, 32, 512, 2048, 32768, },
  { 1, 4, 8, 16, 32, 512, 2048, 32768, },
  { 2, 4, 8, 16, 32, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 32768, },
  { 64, 512, 2048, 32768, },
  { 1, 64, 512, 2048, 32768, },
  { 2, 64, 512, 2048, 32768, },
  { 1, 2, 64, 512, 2048, 32768, },
  { 4, 64, 512, 2048, 32768, },
  { 1, 4, 64, 512, 2048, 32768, },
  { 2, 4, 64, 512, 2048, 32768, },
  { 1, 2, 4, 64, 512, 2048, 32768, },
  { 8, 64, 512, 2048, 32768, },
  { 1, 8, 64, 512, 2048, 32768, },
  { 2, 8, 64, 512, 2048, 32768, },
  { 1, 2, 8, 64, 512, 2048, 32768, },
  { 4, 8, 64, 512, 2048, 32768, },
  { 1, 4, 8, 64, 512, 2048, 32768, },
  { 2, 4, 8, 64, 512, 2048, 32768, },
  { 1, 2, 4, 8, 64, 512, 2048, 32768, },
  { 16, 64, 512, 2048, 32768, },
  { 1, 16, 64, 512, 2048, 32768, },
  { 2, 16, 64, 512, 2048, 32768, },
  { 1, 2, 16, 64, 512, 2048, 32768, },
  { 4, 16, 64, 512, 2048, 32768, },
  { 1, 4, 16, 64, 512, 2048, 32768, },
  { 2, 4, 16, 64, 512, 2048, 32768, },
  { 1, 2, 4, 16, 64, 512, 2048, 32768, },
  { 8, 16, 64, 512, 2048, 32768, },
  { 1, 8, 16, 64, 512, 2048, 32768, },
  { 2, 8, 16, 64, 512, 2048, 32768, },
  { 1, 2, 8, 16, 64, 512, 2048, 32768, },
  { 4, 8, 16, 64, 512, 2048, 32768, },
  { 1, 4, 8, 16, 64, 512, 2048, 32768, },
  { 2, 4, 8, 16, 64, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 32768, },
  { 32, 64, 512, 2048, 32768, },
  { 1, 32, 64, 512, 2048, 32768, },
  { 2, 32, 64, 512, 2048, 32768, },
  { 1, 2, 32, 64, 512, 2048, 32768, },
  { 4, 32, 64, 512, 2048, 32768, },
  { 1, 4, 32, 64, 512, 2048, 32768, },
  { 2, 4, 32, 64, 512, 2048, 32768, },
  { 1, 2, 4, 32, 64, 512, 2048, 32768, },
  { 8, 32, 64, 512, 2048, 32768, },
  { 1, 8, 32, 64, 512, 2048, 32768, },
  { 2, 8, 32, 64, 512, 2048, 32768, },
  { 1, 2, 8, 32, 64, 512, 2048, 32768, },
  { 4, 8, 32, 64, 512, 2048, 32768, },
  { 1, 4, 8, 32, 64, 512, 2048, 32768, },
  { 2, 4, 8, 32, 64, 512, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 32768, },
  { 16, 32, 64, 512, 2048, 32768, },
  { 1, 16, 32, 64, 512, 2048, 32768, },
  { 2, 16, 32, 64, 512, 2048, 32768, },
  { 1, 2, 16, 32, 64, 512, 2048, 32768, },
  { 4, 16, 32, 64, 512, 2048, 32768, },
  { 1, 4, 16, 32, 64, 512, 2048, 32768, },
  { 2, 4, 16, 32, 64, 512, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 32768, },
  { 8, 16, 32, 64, 512, 2048, 32768, },
  { 1, 8, 16, 32, 64, 512, 2048, 32768, },
  { 2, 8, 16, 32, 64, 512, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 32768, },
  { 4, 8, 16, 32, 64, 512, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 32768, },
  { 128, 512, 2048, 32768, },
  { 1, 128, 512, 2048, 32768, },
  { 2, 128, 512, 2048, 32768, },
  { 1, 2, 128, 512, 2048, 32768, },
  { 4, 128, 512, 2048, 32768, },
  { 1, 4, 128, 512, 2048, 32768, },
  { 2, 4, 128, 512, 2048, 32768, },
  { 1, 2, 4, 128, 512, 2048, 32768, },
  { 8, 128, 512, 2048, 32768, },
  { 1, 8, 128, 512, 2048, 32768, },
  { 2, 8, 128, 512, 2048, 32768, },
  { 1, 2, 8, 128, 512, 2048, 32768, },
  { 4, 8, 128, 512, 2048, 32768, },
  { 1, 4, 8, 128, 512, 2048, 32768, },
  { 2, 4, 8, 128, 512, 2048, 32768, },
  { 1, 2, 4, 8, 128, 512, 2048, 32768, },
  { 16, 128, 512, 2048, 32768, },
  { 1, 16, 128, 512, 2048, 32768, },
  { 2, 16, 128, 512, 2048, 32768, },
  { 1, 2, 16, 128, 512, 2048, 32768, },
  { 4, 16, 128, 512, 2048, 32768, },
  { 1, 4, 16, 128, 512, 2048, 32768, },
  { 2, 4, 16, 128, 512, 2048, 32768, },
  { 1, 2, 4, 16, 128, 512, 2048, 32768, },
  { 8, 16, 128, 512, 2048, 32768, },
  { 1, 8, 16, 128, 512, 2048, 32768, },
  { 2, 8, 16, 128, 512, 2048, 32768, },
  { 1, 2, 8, 16, 128, 512, 2048, 32768, },
  { 4, 8, 16, 128, 512, 2048, 32768, },
  { 1, 4, 8, 16, 128, 512, 2048, 32768, },
  { 2, 4, 8, 16, 128, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 32768, },
  { 32, 128, 512, 2048, 32768, },
  { 1, 32, 128, 512, 2048, 32768, },
  { 2, 32, 128, 512, 2048, 32768, },
  { 1, 2, 32, 128, 512, 2048, 32768, },
  { 4, 32, 128, 512, 2048, 32768, },
  { 1, 4, 32, 128, 512, 2048, 32768, },
  { 2, 4, 32, 128, 512, 2048, 32768, },
  { 1, 2, 4, 32, 128, 512, 2048, 32768, },
  { 8, 32, 128, 512, 2048, 32768, },
  { 1, 8, 32, 128, 512, 2048, 32768, },
  { 2, 8, 32, 128, 512, 2048, 32768, },
  { 1, 2, 8, 32, 128, 512, 2048, 32768, },
  { 4, 8, 32, 128, 512, 2048, 32768, },
  { 1, 4, 8, 32, 128, 512, 2048, 32768, },
  { 2, 4, 8, 32, 128, 512, 2048, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 32768, },
  { 16, 32, 128, 512, 2048, 32768, },
  { 1, 16, 32, 128, 512, 2048, 32768, },
  { 2, 16, 32, 128, 512, 2048, 32768, },
  { 1, 2, 16, 32, 128, 512, 2048, 32768, },
  { 4, 16, 32, 128, 512, 2048, 32768, },
  { 1, 4, 16, 32, 128, 512, 2048, 32768, },
  { 2, 4, 16, 32, 128, 512, 2048, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 32768, },
  { 8, 16, 32, 128, 512, 2048, 32768, },
  { 1, 8, 16, 32, 128, 512, 2048, 32768, },
  { 2, 8, 16, 32, 128, 512, 2048, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 32768, },
  { 4, 8, 16, 32, 128, 512, 2048, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 32768, },
  { 64, 128, 512, 2048, 32768, },
  { 1, 64, 128, 512, 2048, 32768, },
  { 2, 64, 128, 512, 2048, 32768, },
  { 1, 2, 64, 128, 512, 2048, 32768, },
  { 4, 64, 128, 512, 2048, 32768, },
  { 1, 4, 64, 128, 512, 2048, 32768, },
  { 2, 4, 64, 128, 512, 2048, 32768, },
  { 1, 2, 4, 64, 128, 512, 2048, 32768, },
  { 8, 64, 128, 512, 2048, 32768, },
  { 1, 8, 64, 128, 512, 2048, 32768, },
  { 2, 8, 64, 128, 512, 2048, 32768, },
  { 1, 2, 8, 64, 128, 512, 2048, 32768, },
  { 4, 8, 64, 128, 512, 2048, 32768, },
  { 1, 4, 8, 64, 128, 512, 2048, 32768, },
  { 2, 4, 8, 64, 128, 512, 2048, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 32768, },
  { 16, 64, 128, 512, 2048, 32768, },
  { 1, 16, 64, 128, 512, 2048, 32768, },
  { 2, 16, 64, 128, 512, 2048, 32768, },
  { 1, 2, 16, 64, 128, 512, 2048, 32768, },
  { 4, 16, 64, 128, 512, 2048, 32768, },
  { 1, 4, 16, 64, 128, 512, 2048, 32768, },
  { 2, 4, 16, 64, 128, 512, 2048, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 32768, },
  { 8, 16, 64, 128, 512, 2048, 32768, },
  { 1, 8, 16, 64, 128, 512, 2048, 32768, },
  { 2, 8, 16, 64, 128, 512, 2048, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 32768, },
  { 4, 8, 16, 64, 128, 512, 2048, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 32768, },
  { 32, 64, 128, 512, 2048, 32768, },
  { 1, 32, 64, 128, 512, 2048, 32768, },
  { 2, 32, 64, 128, 512, 2048, 32768, },
  { 1, 2, 32, 64, 128, 512, 2048, 32768, },
  { 4, 32, 64, 128, 512, 2048, 32768, },
  { 1, 4, 32, 64, 128, 512, 2048, 32768, },
  { 2, 4, 32, 64, 128, 512, 2048, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 32768, },
  { 8, 32, 64, 128, 512, 2048, 32768, },
  { 1, 8, 32, 64, 128, 512, 2048, 32768, },
  { 2, 8, 32, 64, 128, 512, 2048, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 32768, },
  { 4, 8, 32, 64, 128, 512, 2048, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 32768, },
  { 16, 32, 64, 128, 512, 2048, 32768, },
  { 1, 16, 32, 64, 128, 512, 2048, 32768, },
  { 2, 16, 32, 64, 128, 512, 2048, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 32768, },
  { 4, 16, 32, 64, 128, 512, 2048, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 32768, },
  { 8, 16, 32, 64, 128, 512, 2048, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 32768, },
  { 256, 512, 2048, 32768, },
  { 1, 256, 512, 2048, 32768, },
  { 2, 256, 512, 2048, 32768, },
  { 1, 2, 256, 512, 2048, 32768, },
  { 4, 256, 512, 2048, 32768, },
  { 1, 4, 256, 512, 2048, 32768, },
  { 2, 4, 256, 512, 2048, 32768, },
  { 1, 2, 4, 256, 512, 2048, 32768, },
  { 8, 256, 512, 2048, 32768, },
  { 1, 8, 256, 512, 2048, 32768, },
  { 2, 8, 256, 512, 2048, 32768, },
  { 1, 2, 8, 256, 512, 2048, 32768, },
  { 4, 8, 256, 512, 2048, 32768, },
  { 1, 4, 8, 256, 512, 2048, 32768, },
  { 2, 4, 8, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 256, 512, 2048, 32768, },
  { 16, 256, 512, 2048, 32768, },
  { 1, 16, 256, 512, 2048, 32768, },
  { 2, 16, 256, 512, 2048, 32768, },
  { 1, 2, 16, 256, 512, 2048, 32768, },
  { 4, 16, 256, 512, 2048, 32768, },
  { 1, 4, 16, 256, 512, 2048, 32768, },
  { 2, 4, 16, 256, 512, 2048, 32768, },
  { 1, 2, 4, 16, 256, 512, 2048, 32768, },
  { 8, 16, 256, 512, 2048, 32768, },
  { 1, 8, 16, 256, 512, 2048, 32768, },
  { 2, 8, 16, 256, 512, 2048, 32768, },
  { 1, 2, 8, 16, 256, 512, 2048, 32768, },
  { 4, 8, 16, 256, 512, 2048, 32768, },
  { 1, 4, 8, 16, 256, 512, 2048, 32768, },
  { 2, 4, 8, 16, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 32768, },
  { 32, 256, 512, 2048, 32768, },
  { 1, 32, 256, 512, 2048, 32768, },
  { 2, 32, 256, 512, 2048, 32768, },
  { 1, 2, 32, 256, 512, 2048, 32768, },
  { 4, 32, 256, 512, 2048, 32768, },
  { 1, 4, 32, 256, 512, 2048, 32768, },
  { 2, 4, 32, 256, 512, 2048, 32768, },
  { 1, 2, 4, 32, 256, 512, 2048, 32768, },
  { 8, 32, 256, 512, 2048, 32768, },
  { 1, 8, 32, 256, 512, 2048, 32768, },
  { 2, 8, 32, 256, 512, 2048, 32768, },
  { 1, 2, 8, 32, 256, 512, 2048, 32768, },
  { 4, 8, 32, 256, 512, 2048, 32768, },
  { 1, 4, 8, 32, 256, 512, 2048, 32768, },
  { 2, 4, 8, 32, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 32768, },
  { 16, 32, 256, 512, 2048, 32768, },
  { 1, 16, 32, 256, 512, 2048, 32768, },
  { 2, 16, 32, 256, 512, 2048, 32768, },
  { 1, 2, 16, 32, 256, 512, 2048, 32768, },
  { 4, 16, 32, 256, 512, 2048, 32768, },
  { 1, 4, 16, 32, 256, 512, 2048, 32768, },
  { 2, 4, 16, 32, 256, 512, 2048, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 32768, },
  { 8, 16, 32, 256, 512, 2048, 32768, },
  { 1, 8, 16, 32, 256, 512, 2048, 32768, },
  { 2, 8, 16, 32, 256, 512, 2048, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 32768, },
  { 4, 8, 16, 32, 256, 512, 2048, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 32768, },
  { 64, 256, 512, 2048, 32768, },
  { 1, 64, 256, 512, 2048, 32768, },
  { 2, 64, 256, 512, 2048, 32768, },
  { 1, 2, 64, 256, 512, 2048, 32768, },
  { 4, 64, 256, 512, 2048, 32768, },
  { 1, 4, 64, 256, 512, 2048, 32768, },
  { 2, 4, 64, 256, 512, 2048, 32768, },
  { 1, 2, 4, 64, 256, 512, 2048, 32768, },
  { 8, 64, 256, 512, 2048, 32768, },
  { 1, 8, 64, 256, 512, 2048, 32768, },
  { 2, 8, 64, 256, 512, 2048, 32768, },
  { 1, 2, 8, 64, 256, 512, 2048, 32768, },
  { 4, 8, 64, 256, 512, 2048, 32768, },
  { 1, 4, 8, 64, 256, 512, 2048, 32768, },
  { 2, 4, 8, 64, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 32768, },
  { 16, 64, 256, 512, 2048, 32768, },
  { 1, 16, 64, 256, 512, 2048, 32768, },
  { 2, 16, 64, 256, 512, 2048, 32768, },
  { 1, 2, 16, 64, 256, 512, 2048, 32768, },
  { 4, 16, 64, 256, 512, 2048, 32768, },
  { 1, 4, 16, 64, 256, 512, 2048, 32768, },
  { 2, 4, 16, 64, 256, 512, 2048, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 32768, },
  { 8, 16, 64, 256, 512, 2048, 32768, },
  { 1, 8, 16, 64, 256, 512, 2048, 32768, },
  { 2, 8, 16, 64, 256, 512, 2048, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 32768, },
  { 4, 8, 16, 64, 256, 512, 2048, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 32768, },
  { 32, 64, 256, 512, 2048, 32768, },
  { 1, 32, 64, 256, 512, 2048, 32768, },
  { 2, 32, 64, 256, 512, 2048, 32768, },
  { 1, 2, 32, 64, 256, 512, 2048, 32768, },
  { 4, 32, 64, 256, 512, 2048, 32768, },
  { 1, 4, 32, 64, 256, 512, 2048, 32768, },
  { 2, 4, 32, 64, 256, 512, 2048, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 32768, },
  { 8, 32, 64, 256, 512, 2048, 32768, },
  { 1, 8, 32, 64, 256, 512, 2048, 32768, },
  { 2, 8, 32, 64, 256, 512, 2048, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 32768, },
  { 4, 8, 32, 64, 256, 512, 2048, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 32768, },
  { 16, 32, 64, 256, 512, 2048, 32768, },
  { 1, 16, 32, 64, 256, 512, 2048, 32768, },
  { 2, 16, 32, 64, 256, 512, 2048, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 32768, },
  { 4, 16, 32, 64, 256, 512, 2048, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 32768, },
  { 8, 16, 32, 64, 256, 512, 2048, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 32768, },
  { 128, 256, 512, 2048, 32768, },
  { 1, 128, 256, 512, 2048, 32768, },
  { 2, 128, 256, 512, 2048, 32768, },
  { 1, 2, 128, 256, 512, 2048, 32768, },
  { 4, 128, 256, 512, 2048, 32768, },
  { 1, 4, 128, 256, 512, 2048, 32768, },
  { 2, 4, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 128, 256, 512, 2048, 32768, },
  { 8, 128, 256, 512, 2048, 32768, },
  { 1, 8, 128, 256, 512, 2048, 32768, },
  { 2, 8, 128, 256, 512, 2048, 32768, },
  { 1, 2, 8, 128, 256, 512, 2048, 32768, },
  { 4, 8, 128, 256, 512, 2048, 32768, },
  { 1, 4, 8, 128, 256, 512, 2048, 32768, },
  { 2, 4, 8, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 32768, },
  { 16, 128, 256, 512, 2048, 32768, },
  { 1, 16, 128, 256, 512, 2048, 32768, },
  { 2, 16, 128, 256, 512, 2048, 32768, },
  { 1, 2, 16, 128, 256, 512, 2048, 32768, },
  { 4, 16, 128, 256, 512, 2048, 32768, },
  { 1, 4, 16, 128, 256, 512, 2048, 32768, },
  { 2, 4, 16, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 32768, },
  { 8, 16, 128, 256, 512, 2048, 32768, },
  { 1, 8, 16, 128, 256, 512, 2048, 32768, },
  { 2, 8, 16, 128, 256, 512, 2048, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 32768, },
  { 4, 8, 16, 128, 256, 512, 2048, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 32768, },
  { 32, 128, 256, 512, 2048, 32768, },
  { 1, 32, 128, 256, 512, 2048, 32768, },
  { 2, 32, 128, 256, 512, 2048, 32768, },
  { 1, 2, 32, 128, 256, 512, 2048, 32768, },
  { 4, 32, 128, 256, 512, 2048, 32768, },
  { 1, 4, 32, 128, 256, 512, 2048, 32768, },
  { 2, 4, 32, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 32768, },
  { 8, 32, 128, 256, 512, 2048, 32768, },
  { 1, 8, 32, 128, 256, 512, 2048, 32768, },
  { 2, 8, 32, 128, 256, 512, 2048, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 32768, },
  { 4, 8, 32, 128, 256, 512, 2048, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 32768, },
  { 16, 32, 128, 256, 512, 2048, 32768, },
  { 1, 16, 32, 128, 256, 512, 2048, 32768, },
  { 2, 16, 32, 128, 256, 512, 2048, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 32768, },
  { 4, 16, 32, 128, 256, 512, 2048, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 32768, },
  { 8, 16, 32, 128, 256, 512, 2048, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 32768, },
  { 64, 128, 256, 512, 2048, 32768, },
  { 1, 64, 128, 256, 512, 2048, 32768, },
  { 2, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 64, 128, 256, 512, 2048, 32768, },
  { 4, 64, 128, 256, 512, 2048, 32768, },
  { 1, 4, 64, 128, 256, 512, 2048, 32768, },
  { 2, 4, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 32768, },
  { 8, 64, 128, 256, 512, 2048, 32768, },
  { 1, 8, 64, 128, 256, 512, 2048, 32768, },
  { 2, 8, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 32768, },
  { 4, 8, 64, 128, 256, 512, 2048, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 32768, },
  { 16, 64, 128, 256, 512, 2048, 32768, },
  { 1, 16, 64, 128, 256, 512, 2048, 32768, },
  { 2, 16, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 32768, },
  { 4, 16, 64, 128, 256, 512, 2048, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 32768, },
  { 8, 16, 64, 128, 256, 512, 2048, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 32768, },
  { 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 32, 64, 128, 256, 512, 2048, 32768, },
  { 2, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 32768, },
  { 4, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 32768, },
  { 8, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 32768, },
  { 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 32768, },
  { 1024, 2048, 32768, },
  { 1, 1024, 2048, 32768, },
  { 2, 1024, 2048, 32768, },
  { 1, 2, 1024, 2048, 32768, },
  { 4, 1024, 2048, 32768, },
  { 1, 4, 1024, 2048, 32768, },
  { 2, 4, 1024, 2048, 32768, },
  { 1, 2, 4, 1024, 2048, 32768, },
  { 8, 1024, 2048, 32768, },
  { 1, 8, 1024, 2048, 32768, },
  { 2, 8, 1024, 2048, 32768, },
  { 1, 2, 8, 1024, 2048, 32768, },
  { 4, 8, 1024, 2048, 32768, },
  { 1, 4, 8, 1024, 2048, 32768, },
  { 2, 4, 8, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 1024, 2048, 32768, },
  { 16, 1024, 2048, 32768, },
  { 1, 16, 1024, 2048, 32768, },
  { 2, 16, 1024, 2048, 32768, },
  { 1, 2, 16, 1024, 2048, 32768, },
  { 4, 16, 1024, 2048, 32768, },
  { 1, 4, 16, 1024, 2048, 32768, },
  { 2, 4, 16, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 1024, 2048, 32768, },
  { 8, 16, 1024, 2048, 32768, },
  { 1, 8, 16, 1024, 2048, 32768, },
  { 2, 8, 16, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 1024, 2048, 32768, },
  { 4, 8, 16, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 1024, 2048, 32768, },
  { 32, 1024, 2048, 32768, },
  { 1, 32, 1024, 2048, 32768, },
  { 2, 32, 1024, 2048, 32768, },
  { 1, 2, 32, 1024, 2048, 32768, },
  { 4, 32, 1024, 2048, 32768, },
  { 1, 4, 32, 1024, 2048, 32768, },
  { 2, 4, 32, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 1024, 2048, 32768, },
  { 8, 32, 1024, 2048, 32768, },
  { 1, 8, 32, 1024, 2048, 32768, },
  { 2, 8, 32, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 1024, 2048, 32768, },
  { 4, 8, 32, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 1024, 2048, 32768, },
  { 16, 32, 1024, 2048, 32768, },
  { 1, 16, 32, 1024, 2048, 32768, },
  { 2, 16, 32, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 1024, 2048, 32768, },
  { 4, 16, 32, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 1024, 2048, 32768, },
  { 8, 16, 32, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 32768, },
  { 64, 1024, 2048, 32768, },
  { 1, 64, 1024, 2048, 32768, },
  { 2, 64, 1024, 2048, 32768, },
  { 1, 2, 64, 1024, 2048, 32768, },
  { 4, 64, 1024, 2048, 32768, },
  { 1, 4, 64, 1024, 2048, 32768, },
  { 2, 4, 64, 1024, 2048, 32768, },
  { 1, 2, 4, 64, 1024, 2048, 32768, },
  { 8, 64, 1024, 2048, 32768, },
  { 1, 8, 64, 1024, 2048, 32768, },
  { 2, 8, 64, 1024, 2048, 32768, },
  { 1, 2, 8, 64, 1024, 2048, 32768, },
  { 4, 8, 64, 1024, 2048, 32768, },
  { 1, 4, 8, 64, 1024, 2048, 32768, },
  { 2, 4, 8, 64, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 64, 1024, 2048, 32768, },
  { 16, 64, 1024, 2048, 32768, },
  { 1, 16, 64, 1024, 2048, 32768, },
  { 2, 16, 64, 1024, 2048, 32768, },
  { 1, 2, 16, 64, 1024, 2048, 32768, },
  { 4, 16, 64, 1024, 2048, 32768, },
  { 1, 4, 16, 64, 1024, 2048, 32768, },
  { 2, 4, 16, 64, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 64, 1024, 2048, 32768, },
  { 8, 16, 64, 1024, 2048, 32768, },
  { 1, 8, 16, 64, 1024, 2048, 32768, },
  { 2, 8, 16, 64, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 64, 1024, 2048, 32768, },
  { 4, 8, 16, 64, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 64, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 64, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 32768, },
  { 32, 64, 1024, 2048, 32768, },
  { 1, 32, 64, 1024, 2048, 32768, },
  { 2, 32, 64, 1024, 2048, 32768, },
  { 1, 2, 32, 64, 1024, 2048, 32768, },
  { 4, 32, 64, 1024, 2048, 32768, },
  { 1, 4, 32, 64, 1024, 2048, 32768, },
  { 2, 4, 32, 64, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 64, 1024, 2048, 32768, },
  { 8, 32, 64, 1024, 2048, 32768, },
  { 1, 8, 32, 64, 1024, 2048, 32768, },
  { 2, 8, 32, 64, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 64, 1024, 2048, 32768, },
  { 4, 8, 32, 64, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 64, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 64, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 32768, },
  { 16, 32, 64, 1024, 2048, 32768, },
  { 1, 16, 32, 64, 1024, 2048, 32768, },
  { 2, 16, 32, 64, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 64, 1024, 2048, 32768, },
  { 4, 16, 32, 64, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 64, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 64, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 32768, },
  { 8, 16, 32, 64, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 64, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 64, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 64, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 32768, },
  { 128, 1024, 2048, 32768, },
  { 1, 128, 1024, 2048, 32768, },
  { 2, 128, 1024, 2048, 32768, },
  { 1, 2, 128, 1024, 2048, 32768, },
  { 4, 128, 1024, 2048, 32768, },
  { 1, 4, 128, 1024, 2048, 32768, },
  { 2, 4, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 128, 1024, 2048, 32768, },
  { 8, 128, 1024, 2048, 32768, },
  { 1, 8, 128, 1024, 2048, 32768, },
  { 2, 8, 128, 1024, 2048, 32768, },
  { 1, 2, 8, 128, 1024, 2048, 32768, },
  { 4, 8, 128, 1024, 2048, 32768, },
  { 1, 4, 8, 128, 1024, 2048, 32768, },
  { 2, 4, 8, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 128, 1024, 2048, 32768, },
  { 16, 128, 1024, 2048, 32768, },
  { 1, 16, 128, 1024, 2048, 32768, },
  { 2, 16, 128, 1024, 2048, 32768, },
  { 1, 2, 16, 128, 1024, 2048, 32768, },
  { 4, 16, 128, 1024, 2048, 32768, },
  { 1, 4, 16, 128, 1024, 2048, 32768, },
  { 2, 4, 16, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 128, 1024, 2048, 32768, },
  { 8, 16, 128, 1024, 2048, 32768, },
  { 1, 8, 16, 128, 1024, 2048, 32768, },
  { 2, 8, 16, 128, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 128, 1024, 2048, 32768, },
  { 4, 8, 16, 128, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 128, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 32768, },
  { 32, 128, 1024, 2048, 32768, },
  { 1, 32, 128, 1024, 2048, 32768, },
  { 2, 32, 128, 1024, 2048, 32768, },
  { 1, 2, 32, 128, 1024, 2048, 32768, },
  { 4, 32, 128, 1024, 2048, 32768, },
  { 1, 4, 32, 128, 1024, 2048, 32768, },
  { 2, 4, 32, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 128, 1024, 2048, 32768, },
  { 8, 32, 128, 1024, 2048, 32768, },
  { 1, 8, 32, 128, 1024, 2048, 32768, },
  { 2, 8, 32, 128, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 128, 1024, 2048, 32768, },
  { 4, 8, 32, 128, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 128, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 32768, },
  { 16, 32, 128, 1024, 2048, 32768, },
  { 1, 16, 32, 128, 1024, 2048, 32768, },
  { 2, 16, 32, 128, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 128, 1024, 2048, 32768, },
  { 4, 16, 32, 128, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 128, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 32768, },
  { 8, 16, 32, 128, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 128, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 128, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 128, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 32768, },
  { 64, 128, 1024, 2048, 32768, },
  { 1, 64, 128, 1024, 2048, 32768, },
  { 2, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 64, 128, 1024, 2048, 32768, },
  { 4, 64, 128, 1024, 2048, 32768, },
  { 1, 4, 64, 128, 1024, 2048, 32768, },
  { 2, 4, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 64, 128, 1024, 2048, 32768, },
  { 8, 64, 128, 1024, 2048, 32768, },
  { 1, 8, 64, 128, 1024, 2048, 32768, },
  { 2, 8, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 8, 64, 128, 1024, 2048, 32768, },
  { 4, 8, 64, 128, 1024, 2048, 32768, },
  { 1, 4, 8, 64, 128, 1024, 2048, 32768, },
  { 2, 4, 8, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 32768, },
  { 16, 64, 128, 1024, 2048, 32768, },
  { 1, 16, 64, 128, 1024, 2048, 32768, },
  { 2, 16, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 16, 64, 128, 1024, 2048, 32768, },
  { 4, 16, 64, 128, 1024, 2048, 32768, },
  { 1, 4, 16, 64, 128, 1024, 2048, 32768, },
  { 2, 4, 16, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 32768, },
  { 8, 16, 64, 128, 1024, 2048, 32768, },
  { 1, 8, 16, 64, 128, 1024, 2048, 32768, },
  { 2, 8, 16, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 32768, },
  { 4, 8, 16, 64, 128, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 32768, },
  { 32, 64, 128, 1024, 2048, 32768, },
  { 1, 32, 64, 128, 1024, 2048, 32768, },
  { 2, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 32, 64, 128, 1024, 2048, 32768, },
  { 4, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 4, 32, 64, 128, 1024, 2048, 32768, },
  { 2, 4, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 32768, },
  { 8, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 8, 32, 64, 128, 1024, 2048, 32768, },
  { 2, 8, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 32768, },
  { 4, 8, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 32768, },
  { 16, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 2, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 4, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 8, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 32768, },
  { 256, 1024, 2048, 32768, },
  { 1, 256, 1024, 2048, 32768, },
  { 2, 256, 1024, 2048, 32768, },
  { 1, 2, 256, 1024, 2048, 32768, },
  { 4, 256, 1024, 2048, 32768, },
  { 1, 4, 256, 1024, 2048, 32768, },
  { 2, 4, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 256, 1024, 2048, 32768, },
  { 8, 256, 1024, 2048, 32768, },
  { 1, 8, 256, 1024, 2048, 32768, },
  { 2, 8, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 256, 1024, 2048, 32768, },
  { 4, 8, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 256, 1024, 2048, 32768, },
  { 16, 256, 1024, 2048, 32768, },
  { 1, 16, 256, 1024, 2048, 32768, },
  { 2, 16, 256, 1024, 2048, 32768, },
  { 1, 2, 16, 256, 1024, 2048, 32768, },
  { 4, 16, 256, 1024, 2048, 32768, },
  { 1, 4, 16, 256, 1024, 2048, 32768, },
  { 2, 4, 16, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 256, 1024, 2048, 32768, },
  { 8, 16, 256, 1024, 2048, 32768, },
  { 1, 8, 16, 256, 1024, 2048, 32768, },
  { 2, 8, 16, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 256, 1024, 2048, 32768, },
  { 4, 8, 16, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 32768, },
  { 32, 256, 1024, 2048, 32768, },
  { 1, 32, 256, 1024, 2048, 32768, },
  { 2, 32, 256, 1024, 2048, 32768, },
  { 1, 2, 32, 256, 1024, 2048, 32768, },
  { 4, 32, 256, 1024, 2048, 32768, },
  { 1, 4, 32, 256, 1024, 2048, 32768, },
  { 2, 4, 32, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 256, 1024, 2048, 32768, },
  { 8, 32, 256, 1024, 2048, 32768, },
  { 1, 8, 32, 256, 1024, 2048, 32768, },
  { 2, 8, 32, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 256, 1024, 2048, 32768, },
  { 4, 8, 32, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 32768, },
  { 16, 32, 256, 1024, 2048, 32768, },
  { 1, 16, 32, 256, 1024, 2048, 32768, },
  { 2, 16, 32, 256, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 256, 1024, 2048, 32768, },
  { 4, 16, 32, 256, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 256, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 32768, },
  { 8, 16, 32, 256, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 256, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 32768, },
  { 64, 256, 1024, 2048, 32768, },
  { 1, 64, 256, 1024, 2048, 32768, },
  { 2, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 64, 256, 1024, 2048, 32768, },
  { 4, 64, 256, 1024, 2048, 32768, },
  { 1, 4, 64, 256, 1024, 2048, 32768, },
  { 2, 4, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 64, 256, 1024, 2048, 32768, },
  { 8, 64, 256, 1024, 2048, 32768, },
  { 1, 8, 64, 256, 1024, 2048, 32768, },
  { 2, 8, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 64, 256, 1024, 2048, 32768, },
  { 4, 8, 64, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 64, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 32768, },
  { 16, 64, 256, 1024, 2048, 32768, },
  { 1, 16, 64, 256, 1024, 2048, 32768, },
  { 2, 16, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 16, 64, 256, 1024, 2048, 32768, },
  { 4, 16, 64, 256, 1024, 2048, 32768, },
  { 1, 4, 16, 64, 256, 1024, 2048, 32768, },
  { 2, 4, 16, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 32768, },
  { 8, 16, 64, 256, 1024, 2048, 32768, },
  { 1, 8, 16, 64, 256, 1024, 2048, 32768, },
  { 2, 8, 16, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 32768, },
  { 4, 8, 16, 64, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 32768, },
  { 32, 64, 256, 1024, 2048, 32768, },
  { 1, 32, 64, 256, 1024, 2048, 32768, },
  { 2, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 32, 64, 256, 1024, 2048, 32768, },
  { 4, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 4, 32, 64, 256, 1024, 2048, 32768, },
  { 2, 4, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 32768, },
  { 8, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 8, 32, 64, 256, 1024, 2048, 32768, },
  { 2, 8, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 32768, },
  { 4, 8, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 32768, },
  { 16, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 2, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 4, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 8, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 32768, },
  { 128, 256, 1024, 2048, 32768, },
  { 1, 128, 256, 1024, 2048, 32768, },
  { 2, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 128, 256, 1024, 2048, 32768, },
  { 4, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 128, 256, 1024, 2048, 32768, },
  { 8, 128, 256, 1024, 2048, 32768, },
  { 1, 8, 128, 256, 1024, 2048, 32768, },
  { 2, 8, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 128, 256, 1024, 2048, 32768, },
  { 4, 8, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 32768, },
  { 16, 128, 256, 1024, 2048, 32768, },
  { 1, 16, 128, 256, 1024, 2048, 32768, },
  { 2, 16, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 16, 128, 256, 1024, 2048, 32768, },
  { 4, 16, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 16, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 16, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 32768, },
  { 8, 16, 128, 256, 1024, 2048, 32768, },
  { 1, 8, 16, 128, 256, 1024, 2048, 32768, },
  { 2, 8, 16, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 32768, },
  { 4, 8, 16, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 32768, },
  { 32, 128, 256, 1024, 2048, 32768, },
  { 1, 32, 128, 256, 1024, 2048, 32768, },
  { 2, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 32, 128, 256, 1024, 2048, 32768, },
  { 4, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 32, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 32768, },
  { 8, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 8, 32, 128, 256, 1024, 2048, 32768, },
  { 2, 8, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 32768, },
  { 4, 8, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 32768, },
  { 16, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 2, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 4, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 8, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 32768, },
  { 64, 128, 256, 1024, 2048, 32768, },
  { 1, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 64, 128, 256, 1024, 2048, 32768, },
  { 4, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 32768, },
  { 8, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 8, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 8, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 32768, },
  { 4, 8, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 32768, },
  { 16, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 4, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 8, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 32768, },
  { 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 4, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 8, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 32768, },
  { 512, 1024, 2048, 32768, },
  { 1, 512, 1024, 2048, 32768, },
  { 2, 512, 1024, 2048, 32768, },
  { 1, 2, 512, 1024, 2048, 32768, },
  { 4, 512, 1024, 2048, 32768, },
  { 1, 4, 512, 1024, 2048, 32768, },
  { 2, 4, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 512, 1024, 2048, 32768, },
  { 8, 512, 1024, 2048, 32768, },
  { 1, 8, 512, 1024, 2048, 32768, },
  { 2, 8, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 512, 1024, 2048, 32768, },
  { 4, 8, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 512, 1024, 2048, 32768, },
  { 16, 512, 1024, 2048, 32768, },
  { 1, 16, 512, 1024, 2048, 32768, },
  { 2, 16, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 512, 1024, 2048, 32768, },
  { 4, 16, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 512, 1024, 2048, 32768, },
  { 8, 16, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 32768, },
  { 32, 512, 1024, 2048, 32768, },
  { 1, 32, 512, 1024, 2048, 32768, },
  { 2, 32, 512, 1024, 2048, 32768, },
  { 1, 2, 32, 512, 1024, 2048, 32768, },
  { 4, 32, 512, 1024, 2048, 32768, },
  { 1, 4, 32, 512, 1024, 2048, 32768, },
  { 2, 4, 32, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 512, 1024, 2048, 32768, },
  { 8, 32, 512, 1024, 2048, 32768, },
  { 1, 8, 32, 512, 1024, 2048, 32768, },
  { 2, 8, 32, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 512, 1024, 2048, 32768, },
  { 4, 8, 32, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 32768, },
  { 16, 32, 512, 1024, 2048, 32768, },
  { 1, 16, 32, 512, 1024, 2048, 32768, },
  { 2, 16, 32, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 512, 1024, 2048, 32768, },
  { 4, 16, 32, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 32768, },
  { 8, 16, 32, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 32768, },
  { 64, 512, 1024, 2048, 32768, },
  { 1, 64, 512, 1024, 2048, 32768, },
  { 2, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 64, 512, 1024, 2048, 32768, },
  { 4, 64, 512, 1024, 2048, 32768, },
  { 1, 4, 64, 512, 1024, 2048, 32768, },
  { 2, 4, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 64, 512, 1024, 2048, 32768, },
  { 8, 64, 512, 1024, 2048, 32768, },
  { 1, 8, 64, 512, 1024, 2048, 32768, },
  { 2, 8, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 64, 512, 1024, 2048, 32768, },
  { 4, 8, 64, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 64, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 32768, },
  { 16, 64, 512, 1024, 2048, 32768, },
  { 1, 16, 64, 512, 1024, 2048, 32768, },
  { 2, 16, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 64, 512, 1024, 2048, 32768, },
  { 4, 16, 64, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 64, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 32768, },
  { 8, 16, 64, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 64, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 64, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 32768, },
  { 32, 64, 512, 1024, 2048, 32768, },
  { 1, 32, 64, 512, 1024, 2048, 32768, },
  { 2, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 32, 64, 512, 1024, 2048, 32768, },
  { 4, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 4, 32, 64, 512, 1024, 2048, 32768, },
  { 2, 4, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 32768, },
  { 8, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 8, 32, 64, 512, 1024, 2048, 32768, },
  { 2, 8, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 32768, },
  { 4, 8, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 32768, },
  { 16, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 2, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 4, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 8, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 32768, },
  { 128, 512, 1024, 2048, 32768, },
  { 1, 128, 512, 1024, 2048, 32768, },
  { 2, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 128, 512, 1024, 2048, 32768, },
  { 4, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 128, 512, 1024, 2048, 32768, },
  { 8, 128, 512, 1024, 2048, 32768, },
  { 1, 8, 128, 512, 1024, 2048, 32768, },
  { 2, 8, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 128, 512, 1024, 2048, 32768, },
  { 4, 8, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 32768, },
  { 16, 128, 512, 1024, 2048, 32768, },
  { 1, 16, 128, 512, 1024, 2048, 32768, },
  { 2, 16, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 128, 512, 1024, 2048, 32768, },
  { 4, 16, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 32768, },
  { 8, 16, 128, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 128, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 32768, },
  { 32, 128, 512, 1024, 2048, 32768, },
  { 1, 32, 128, 512, 1024, 2048, 32768, },
  { 2, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 32, 128, 512, 1024, 2048, 32768, },
  { 4, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 32, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 32768, },
  { 8, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 8, 32, 128, 512, 1024, 2048, 32768, },
  { 2, 8, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 32768, },
  { 4, 8, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 32768, },
  { 16, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 2, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 4, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 8, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 32768, },
  { 64, 128, 512, 1024, 2048, 32768, },
  { 1, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 64, 128, 512, 1024, 2048, 32768, },
  { 4, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 32768, },
  { 8, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 8, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 8, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 32768, },
  { 4, 8, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 32768, },
  { 16, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 4, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 8, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 32768, },
  { 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 4, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 8, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 32768, },
  { 256, 512, 1024, 2048, 32768, },
  { 1, 256, 512, 1024, 2048, 32768, },
  { 2, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 256, 512, 1024, 2048, 32768, },
  { 4, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 256, 512, 1024, 2048, 32768, },
  { 8, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 32768, },
  { 16, 256, 512, 1024, 2048, 32768, },
  { 1, 16, 256, 512, 1024, 2048, 32768, },
  { 2, 16, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 256, 512, 1024, 2048, 32768, },
  { 4, 16, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 32768, },
  { 8, 16, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 32768, },
  { 32, 256, 512, 1024, 2048, 32768, },
  { 1, 32, 256, 512, 1024, 2048, 32768, },
  { 2, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 32, 256, 512, 1024, 2048, 32768, },
  { 4, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 32, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 32768, },
  { 8, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 32, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 32768, },
  { 16, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 2, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 4, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 8, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 32768, },
  { 64, 256, 512, 1024, 2048, 32768, },
  { 1, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 64, 256, 512, 1024, 2048, 32768, },
  { 4, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 32768, },
  { 8, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 32768, },
  { 16, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 4, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 8, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 32768, },
  { 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 4, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 8, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 32768, },
  { 128, 256, 512, 1024, 2048, 32768, },
  { 1, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 32768, },
  { 8, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 32768, },
  { 16, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 8, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 32768, },
  { 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 8, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 32768, },
  { 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 8, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 32768, },
  { 4096, 32768, },
  { 1, 4096, 32768, },
  { 2, 4096, 32768, },
  { 1, 2, 4096, 32768, },
  { 4, 4096, 32768, },
  { 1, 4, 4096, 32768, },
  { 2, 4, 4096, 32768, },
  { 1, 2, 4, 4096, 32768, },
  { 8, 4096, 32768, },
  { 1, 8, 4096, 32768, },
  { 2, 8, 4096, 32768, },
  { 1, 2, 8, 4096, 32768, },
  { 4, 8, 4096, 32768, },
  { 1, 4, 8, 4096, 32768, },
  { 2, 4, 8, 4096, 32768, },
  { 1, 2, 4, 8, 4096, 32768, },
  { 16, 4096, 32768, },
  { 1, 16, 4096, 32768, },
  { 2, 16, 4096, 32768, },
  { 1, 2, 16, 4096, 32768, },
  { 4, 16, 4096, 32768, },
  { 1, 4, 16, 4096, 32768, },
  { 2, 4, 16, 4096, 32768, },
  { 1, 2, 4, 16, 4096, 32768, },
  { 8, 16, 4096, 32768, },
  { 1, 8, 16, 4096, 32768, },
  { 2, 8, 16, 4096, 32768, },
  { 1, 2, 8, 16, 4096, 32768, },
  { 4, 8, 16, 4096, 32768, },
  { 1, 4, 8, 16, 4096, 32768, },
  { 2, 4, 8, 16, 4096, 32768, },
  { 1, 2, 4, 8, 16, 4096, 32768, },
  { 32, 4096, 32768, },
  { 1, 32, 4096, 32768, },
  { 2, 32, 4096, 32768, },
  { 1, 2, 32, 4096, 32768, },
  { 4, 32, 4096, 32768, },
  { 1, 4, 32, 4096, 32768, },
  { 2, 4, 32, 4096, 32768, },
  { 1, 2, 4, 32, 4096, 32768, },
  { 8, 32, 4096, 32768, },
  { 1, 8, 32, 4096, 32768, },
  { 2, 8, 32, 4096, 32768, },
  { 1, 2, 8, 32, 4096, 32768, },
  { 4, 8, 32, 4096, 32768, },
  { 1, 4, 8, 32, 4096, 32768, },
  { 2, 4, 8, 32, 4096, 32768, },
  { 1, 2, 4, 8, 32, 4096, 32768, },
  { 16, 32, 4096, 32768, },
  { 1, 16, 32, 4096, 32768, },
  { 2, 16, 32, 4096, 32768, },
  { 1, 2, 16, 32, 4096, 32768, },
  { 4, 16, 32, 4096, 32768, },
  { 1, 4, 16, 32, 4096, 32768, },
  { 2, 4, 16, 32, 4096, 32768, },
  { 1, 2, 4, 16, 32, 4096, 32768, },
  { 8, 16, 32, 4096, 32768, },
  { 1, 8, 16, 32, 4096, 32768, },
  { 2, 8, 16, 32, 4096, 32768, },
  { 1, 2, 8, 16, 32, 4096, 32768, },
  { 4, 8, 16, 32, 4096, 32768, },
  { 1, 4, 8, 16, 32, 4096, 32768, },
  { 2, 4, 8, 16, 32, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 4096, 32768, },
  { 64, 4096, 32768, },
  { 1, 64, 4096, 32768, },
  { 2, 64, 4096, 32768, },
  { 1, 2, 64, 4096, 32768, },
  { 4, 64, 4096, 32768, },
  { 1, 4, 64, 4096, 32768, },
  { 2, 4, 64, 4096, 32768, },
  { 1, 2, 4, 64, 4096, 32768, },
  { 8, 64, 4096, 32768, },
  { 1, 8, 64, 4096, 32768, },
  { 2, 8, 64, 4096, 32768, },
  { 1, 2, 8, 64, 4096, 32768, },
  { 4, 8, 64, 4096, 32768, },
  { 1, 4, 8, 64, 4096, 32768, },
  { 2, 4, 8, 64, 4096, 32768, },
  { 1, 2, 4, 8, 64, 4096, 32768, },
  { 16, 64, 4096, 32768, },
  { 1, 16, 64, 4096, 32768, },
  { 2, 16, 64, 4096, 32768, },
  { 1, 2, 16, 64, 4096, 32768, },
  { 4, 16, 64, 4096, 32768, },
  { 1, 4, 16, 64, 4096, 32768, },
  { 2, 4, 16, 64, 4096, 32768, },
  { 1, 2, 4, 16, 64, 4096, 32768, },
  { 8, 16, 64, 4096, 32768, },
  { 1, 8, 16, 64, 4096, 32768, },
  { 2, 8, 16, 64, 4096, 32768, },
  { 1, 2, 8, 16, 64, 4096, 32768, },
  { 4, 8, 16, 64, 4096, 32768, },
  { 1, 4, 8, 16, 64, 4096, 32768, },
  { 2, 4, 8, 16, 64, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 4096, 32768, },
  { 32, 64, 4096, 32768, },
  { 1, 32, 64, 4096, 32768, },
  { 2, 32, 64, 4096, 32768, },
  { 1, 2, 32, 64, 4096, 32768, },
  { 4, 32, 64, 4096, 32768, },
  { 1, 4, 32, 64, 4096, 32768, },
  { 2, 4, 32, 64, 4096, 32768, },
  { 1, 2, 4, 32, 64, 4096, 32768, },
  { 8, 32, 64, 4096, 32768, },
  { 1, 8, 32, 64, 4096, 32768, },
  { 2, 8, 32, 64, 4096, 32768, },
  { 1, 2, 8, 32, 64, 4096, 32768, },
  { 4, 8, 32, 64, 4096, 32768, },
  { 1, 4, 8, 32, 64, 4096, 32768, },
  { 2, 4, 8, 32, 64, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 4096, 32768, },
  { 16, 32, 64, 4096, 32768, },
  { 1, 16, 32, 64, 4096, 32768, },
  { 2, 16, 32, 64, 4096, 32768, },
  { 1, 2, 16, 32, 64, 4096, 32768, },
  { 4, 16, 32, 64, 4096, 32768, },
  { 1, 4, 16, 32, 64, 4096, 32768, },
  { 2, 4, 16, 32, 64, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 4096, 32768, },
  { 8, 16, 32, 64, 4096, 32768, },
  { 1, 8, 16, 32, 64, 4096, 32768, },
  { 2, 8, 16, 32, 64, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 4096, 32768, },
  { 4, 8, 16, 32, 64, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 4096, 32768, },
  { 128, 4096, 32768, },
  { 1, 128, 4096, 32768, },
  { 2, 128, 4096, 32768, },
  { 1, 2, 128, 4096, 32768, },
  { 4, 128, 4096, 32768, },
  { 1, 4, 128, 4096, 32768, },
  { 2, 4, 128, 4096, 32768, },
  { 1, 2, 4, 128, 4096, 32768, },
  { 8, 128, 4096, 32768, },
  { 1, 8, 128, 4096, 32768, },
  { 2, 8, 128, 4096, 32768, },
  { 1, 2, 8, 128, 4096, 32768, },
  { 4, 8, 128, 4096, 32768, },
  { 1, 4, 8, 128, 4096, 32768, },
  { 2, 4, 8, 128, 4096, 32768, },
  { 1, 2, 4, 8, 128, 4096, 32768, },
  { 16, 128, 4096, 32768, },
  { 1, 16, 128, 4096, 32768, },
  { 2, 16, 128, 4096, 32768, },
  { 1, 2, 16, 128, 4096, 32768, },
  { 4, 16, 128, 4096, 32768, },
  { 1, 4, 16, 128, 4096, 32768, },
  { 2, 4, 16, 128, 4096, 32768, },
  { 1, 2, 4, 16, 128, 4096, 32768, },
  { 8, 16, 128, 4096, 32768, },
  { 1, 8, 16, 128, 4096, 32768, },
  { 2, 8, 16, 128, 4096, 32768, },
  { 1, 2, 8, 16, 128, 4096, 32768, },
  { 4, 8, 16, 128, 4096, 32768, },
  { 1, 4, 8, 16, 128, 4096, 32768, },
  { 2, 4, 8, 16, 128, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 4096, 32768, },
  { 32, 128, 4096, 32768, },
  { 1, 32, 128, 4096, 32768, },
  { 2, 32, 128, 4096, 32768, },
  { 1, 2, 32, 128, 4096, 32768, },
  { 4, 32, 128, 4096, 32768, },
  { 1, 4, 32, 128, 4096, 32768, },
  { 2, 4, 32, 128, 4096, 32768, },
  { 1, 2, 4, 32, 128, 4096, 32768, },
  { 8, 32, 128, 4096, 32768, },
  { 1, 8, 32, 128, 4096, 32768, },
  { 2, 8, 32, 128, 4096, 32768, },
  { 1, 2, 8, 32, 128, 4096, 32768, },
  { 4, 8, 32, 128, 4096, 32768, },
  { 1, 4, 8, 32, 128, 4096, 32768, },
  { 2, 4, 8, 32, 128, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 4096, 32768, },
  { 16, 32, 128, 4096, 32768, },
  { 1, 16, 32, 128, 4096, 32768, },
  { 2, 16, 32, 128, 4096, 32768, },
  { 1, 2, 16, 32, 128, 4096, 32768, },
  { 4, 16, 32, 128, 4096, 32768, },
  { 1, 4, 16, 32, 128, 4096, 32768, },
  { 2, 4, 16, 32, 128, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 4096, 32768, },
  { 8, 16, 32, 128, 4096, 32768, },
  { 1, 8, 16, 32, 128, 4096, 32768, },
  { 2, 8, 16, 32, 128, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 4096, 32768, },
  { 4, 8, 16, 32, 128, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 4096, 32768, },
  { 64, 128, 4096, 32768, },
  { 1, 64, 128, 4096, 32768, },
  { 2, 64, 128, 4096, 32768, },
  { 1, 2, 64, 128, 4096, 32768, },
  { 4, 64, 128, 4096, 32768, },
  { 1, 4, 64, 128, 4096, 32768, },
  { 2, 4, 64, 128, 4096, 32768, },
  { 1, 2, 4, 64, 128, 4096, 32768, },
  { 8, 64, 128, 4096, 32768, },
  { 1, 8, 64, 128, 4096, 32768, },
  { 2, 8, 64, 128, 4096, 32768, },
  { 1, 2, 8, 64, 128, 4096, 32768, },
  { 4, 8, 64, 128, 4096, 32768, },
  { 1, 4, 8, 64, 128, 4096, 32768, },
  { 2, 4, 8, 64, 128, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 4096, 32768, },
  { 16, 64, 128, 4096, 32768, },
  { 1, 16, 64, 128, 4096, 32768, },
  { 2, 16, 64, 128, 4096, 32768, },
  { 1, 2, 16, 64, 128, 4096, 32768, },
  { 4, 16, 64, 128, 4096, 32768, },
  { 1, 4, 16, 64, 128, 4096, 32768, },
  { 2, 4, 16, 64, 128, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 4096, 32768, },
  { 8, 16, 64, 128, 4096, 32768, },
  { 1, 8, 16, 64, 128, 4096, 32768, },
  { 2, 8, 16, 64, 128, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 4096, 32768, },
  { 4, 8, 16, 64, 128, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 4096, 32768, },
  { 32, 64, 128, 4096, 32768, },
  { 1, 32, 64, 128, 4096, 32768, },
  { 2, 32, 64, 128, 4096, 32768, },
  { 1, 2, 32, 64, 128, 4096, 32768, },
  { 4, 32, 64, 128, 4096, 32768, },
  { 1, 4, 32, 64, 128, 4096, 32768, },
  { 2, 4, 32, 64, 128, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 4096, 32768, },
  { 8, 32, 64, 128, 4096, 32768, },
  { 1, 8, 32, 64, 128, 4096, 32768, },
  { 2, 8, 32, 64, 128, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 4096, 32768, },
  { 4, 8, 32, 64, 128, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 4096, 32768, },
  { 16, 32, 64, 128, 4096, 32768, },
  { 1, 16, 32, 64, 128, 4096, 32768, },
  { 2, 16, 32, 64, 128, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 4096, 32768, },
  { 4, 16, 32, 64, 128, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 4096, 32768, },
  { 8, 16, 32, 64, 128, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 4096, 32768, },
  { 256, 4096, 32768, },
  { 1, 256, 4096, 32768, },
  { 2, 256, 4096, 32768, },
  { 1, 2, 256, 4096, 32768, },
  { 4, 256, 4096, 32768, },
  { 1, 4, 256, 4096, 32768, },
  { 2, 4, 256, 4096, 32768, },
  { 1, 2, 4, 256, 4096, 32768, },
  { 8, 256, 4096, 32768, },
  { 1, 8, 256, 4096, 32768, },
  { 2, 8, 256, 4096, 32768, },
  { 1, 2, 8, 256, 4096, 32768, },
  { 4, 8, 256, 4096, 32768, },
  { 1, 4, 8, 256, 4096, 32768, },
  { 2, 4, 8, 256, 4096, 32768, },
  { 1, 2, 4, 8, 256, 4096, 32768, },
  { 16, 256, 4096, 32768, },
  { 1, 16, 256, 4096, 32768, },
  { 2, 16, 256, 4096, 32768, },
  { 1, 2, 16, 256, 4096, 32768, },
  { 4, 16, 256, 4096, 32768, },
  { 1, 4, 16, 256, 4096, 32768, },
  { 2, 4, 16, 256, 4096, 32768, },
  { 1, 2, 4, 16, 256, 4096, 32768, },
  { 8, 16, 256, 4096, 32768, },
  { 1, 8, 16, 256, 4096, 32768, },
  { 2, 8, 16, 256, 4096, 32768, },
  { 1, 2, 8, 16, 256, 4096, 32768, },
  { 4, 8, 16, 256, 4096, 32768, },
  { 1, 4, 8, 16, 256, 4096, 32768, },
  { 2, 4, 8, 16, 256, 4096, 32768, },
  { 1, 2, 4, 8, 16, 256, 4096, 32768, },
  { 32, 256, 4096, 32768, },
  { 1, 32, 256, 4096, 32768, },
  { 2, 32, 256, 4096, 32768, },
  { 1, 2, 32, 256, 4096, 32768, },
  { 4, 32, 256, 4096, 32768, },
  { 1, 4, 32, 256, 4096, 32768, },
  { 2, 4, 32, 256, 4096, 32768, },
  { 1, 2, 4, 32, 256, 4096, 32768, },
  { 8, 32, 256, 4096, 32768, },
  { 1, 8, 32, 256, 4096, 32768, },
  { 2, 8, 32, 256, 4096, 32768, },
  { 1, 2, 8, 32, 256, 4096, 32768, },
  { 4, 8, 32, 256, 4096, 32768, },
  { 1, 4, 8, 32, 256, 4096, 32768, },
  { 2, 4, 8, 32, 256, 4096, 32768, },
  { 1, 2, 4, 8, 32, 256, 4096, 32768, },
  { 16, 32, 256, 4096, 32768, },
  { 1, 16, 32, 256, 4096, 32768, },
  { 2, 16, 32, 256, 4096, 32768, },
  { 1, 2, 16, 32, 256, 4096, 32768, },
  { 4, 16, 32, 256, 4096, 32768, },
  { 1, 4, 16, 32, 256, 4096, 32768, },
  { 2, 4, 16, 32, 256, 4096, 32768, },
  { 1, 2, 4, 16, 32, 256, 4096, 32768, },
  { 8, 16, 32, 256, 4096, 32768, },
  { 1, 8, 16, 32, 256, 4096, 32768, },
  { 2, 8, 16, 32, 256, 4096, 32768, },
  { 1, 2, 8, 16, 32, 256, 4096, 32768, },
  { 4, 8, 16, 32, 256, 4096, 32768, },
  { 1, 4, 8, 16, 32, 256, 4096, 32768, },
  { 2, 4, 8, 16, 32, 256, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 4096, 32768, },
  { 64, 256, 4096, 32768, },
  { 1, 64, 256, 4096, 32768, },
  { 2, 64, 256, 4096, 32768, },
  { 1, 2, 64, 256, 4096, 32768, },
  { 4, 64, 256, 4096, 32768, },
  { 1, 4, 64, 256, 4096, 32768, },
  { 2, 4, 64, 256, 4096, 32768, },
  { 1, 2, 4, 64, 256, 4096, 32768, },
  { 8, 64, 256, 4096, 32768, },
  { 1, 8, 64, 256, 4096, 32768, },
  { 2, 8, 64, 256, 4096, 32768, },
  { 1, 2, 8, 64, 256, 4096, 32768, },
  { 4, 8, 64, 256, 4096, 32768, },
  { 1, 4, 8, 64, 256, 4096, 32768, },
  { 2, 4, 8, 64, 256, 4096, 32768, },
  { 1, 2, 4, 8, 64, 256, 4096, 32768, },
  { 16, 64, 256, 4096, 32768, },
  { 1, 16, 64, 256, 4096, 32768, },
  { 2, 16, 64, 256, 4096, 32768, },
  { 1, 2, 16, 64, 256, 4096, 32768, },
  { 4, 16, 64, 256, 4096, 32768, },
  { 1, 4, 16, 64, 256, 4096, 32768, },
  { 2, 4, 16, 64, 256, 4096, 32768, },
  { 1, 2, 4, 16, 64, 256, 4096, 32768, },
  { 8, 16, 64, 256, 4096, 32768, },
  { 1, 8, 16, 64, 256, 4096, 32768, },
  { 2, 8, 16, 64, 256, 4096, 32768, },
  { 1, 2, 8, 16, 64, 256, 4096, 32768, },
  { 4, 8, 16, 64, 256, 4096, 32768, },
  { 1, 4, 8, 16, 64, 256, 4096, 32768, },
  { 2, 4, 8, 16, 64, 256, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 4096, 32768, },
  { 32, 64, 256, 4096, 32768, },
  { 1, 32, 64, 256, 4096, 32768, },
  { 2, 32, 64, 256, 4096, 32768, },
  { 1, 2, 32, 64, 256, 4096, 32768, },
  { 4, 32, 64, 256, 4096, 32768, },
  { 1, 4, 32, 64, 256, 4096, 32768, },
  { 2, 4, 32, 64, 256, 4096, 32768, },
  { 1, 2, 4, 32, 64, 256, 4096, 32768, },
  { 8, 32, 64, 256, 4096, 32768, },
  { 1, 8, 32, 64, 256, 4096, 32768, },
  { 2, 8, 32, 64, 256, 4096, 32768, },
  { 1, 2, 8, 32, 64, 256, 4096, 32768, },
  { 4, 8, 32, 64, 256, 4096, 32768, },
  { 1, 4, 8, 32, 64, 256, 4096, 32768, },
  { 2, 4, 8, 32, 64, 256, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 4096, 32768, },
  { 16, 32, 64, 256, 4096, 32768, },
  { 1, 16, 32, 64, 256, 4096, 32768, },
  { 2, 16, 32, 64, 256, 4096, 32768, },
  { 1, 2, 16, 32, 64, 256, 4096, 32768, },
  { 4, 16, 32, 64, 256, 4096, 32768, },
  { 1, 4, 16, 32, 64, 256, 4096, 32768, },
  { 2, 4, 16, 32, 64, 256, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 4096, 32768, },
  { 8, 16, 32, 64, 256, 4096, 32768, },
  { 1, 8, 16, 32, 64, 256, 4096, 32768, },
  { 2, 8, 16, 32, 64, 256, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 4096, 32768, },
  { 4, 8, 16, 32, 64, 256, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 4096, 32768, },
  { 128, 256, 4096, 32768, },
  { 1, 128, 256, 4096, 32768, },
  { 2, 128, 256, 4096, 32768, },
  { 1, 2, 128, 256, 4096, 32768, },
  { 4, 128, 256, 4096, 32768, },
  { 1, 4, 128, 256, 4096, 32768, },
  { 2, 4, 128, 256, 4096, 32768, },
  { 1, 2, 4, 128, 256, 4096, 32768, },
  { 8, 128, 256, 4096, 32768, },
  { 1, 8, 128, 256, 4096, 32768, },
  { 2, 8, 128, 256, 4096, 32768, },
  { 1, 2, 8, 128, 256, 4096, 32768, },
  { 4, 8, 128, 256, 4096, 32768, },
  { 1, 4, 8, 128, 256, 4096, 32768, },
  { 2, 4, 8, 128, 256, 4096, 32768, },
  { 1, 2, 4, 8, 128, 256, 4096, 32768, },
  { 16, 128, 256, 4096, 32768, },
  { 1, 16, 128, 256, 4096, 32768, },
  { 2, 16, 128, 256, 4096, 32768, },
  { 1, 2, 16, 128, 256, 4096, 32768, },
  { 4, 16, 128, 256, 4096, 32768, },
  { 1, 4, 16, 128, 256, 4096, 32768, },
  { 2, 4, 16, 128, 256, 4096, 32768, },
  { 1, 2, 4, 16, 128, 256, 4096, 32768, },
  { 8, 16, 128, 256, 4096, 32768, },
  { 1, 8, 16, 128, 256, 4096, 32768, },
  { 2, 8, 16, 128, 256, 4096, 32768, },
  { 1, 2, 8, 16, 128, 256, 4096, 32768, },
  { 4, 8, 16, 128, 256, 4096, 32768, },
  { 1, 4, 8, 16, 128, 256, 4096, 32768, },
  { 2, 4, 8, 16, 128, 256, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 4096, 32768, },
  { 32, 128, 256, 4096, 32768, },
  { 1, 32, 128, 256, 4096, 32768, },
  { 2, 32, 128, 256, 4096, 32768, },
  { 1, 2, 32, 128, 256, 4096, 32768, },
  { 4, 32, 128, 256, 4096, 32768, },
  { 1, 4, 32, 128, 256, 4096, 32768, },
  { 2, 4, 32, 128, 256, 4096, 32768, },
  { 1, 2, 4, 32, 128, 256, 4096, 32768, },
  { 8, 32, 128, 256, 4096, 32768, },
  { 1, 8, 32, 128, 256, 4096, 32768, },
  { 2, 8, 32, 128, 256, 4096, 32768, },
  { 1, 2, 8, 32, 128, 256, 4096, 32768, },
  { 4, 8, 32, 128, 256, 4096, 32768, },
  { 1, 4, 8, 32, 128, 256, 4096, 32768, },
  { 2, 4, 8, 32, 128, 256, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 4096, 32768, },
  { 16, 32, 128, 256, 4096, 32768, },
  { 1, 16, 32, 128, 256, 4096, 32768, },
  { 2, 16, 32, 128, 256, 4096, 32768, },
  { 1, 2, 16, 32, 128, 256, 4096, 32768, },
  { 4, 16, 32, 128, 256, 4096, 32768, },
  { 1, 4, 16, 32, 128, 256, 4096, 32768, },
  { 2, 4, 16, 32, 128, 256, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 4096, 32768, },
  { 8, 16, 32, 128, 256, 4096, 32768, },
  { 1, 8, 16, 32, 128, 256, 4096, 32768, },
  { 2, 8, 16, 32, 128, 256, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 4096, 32768, },
  { 4, 8, 16, 32, 128, 256, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 4096, 32768, },
  { 64, 128, 256, 4096, 32768, },
  { 1, 64, 128, 256, 4096, 32768, },
  { 2, 64, 128, 256, 4096, 32768, },
  { 1, 2, 64, 128, 256, 4096, 32768, },
  { 4, 64, 128, 256, 4096, 32768, },
  { 1, 4, 64, 128, 256, 4096, 32768, },
  { 2, 4, 64, 128, 256, 4096, 32768, },
  { 1, 2, 4, 64, 128, 256, 4096, 32768, },
  { 8, 64, 128, 256, 4096, 32768, },
  { 1, 8, 64, 128, 256, 4096, 32768, },
  { 2, 8, 64, 128, 256, 4096, 32768, },
  { 1, 2, 8, 64, 128, 256, 4096, 32768, },
  { 4, 8, 64, 128, 256, 4096, 32768, },
  { 1, 4, 8, 64, 128, 256, 4096, 32768, },
  { 2, 4, 8, 64, 128, 256, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 4096, 32768, },
  { 16, 64, 128, 256, 4096, 32768, },
  { 1, 16, 64, 128, 256, 4096, 32768, },
  { 2, 16, 64, 128, 256, 4096, 32768, },
  { 1, 2, 16, 64, 128, 256, 4096, 32768, },
  { 4, 16, 64, 128, 256, 4096, 32768, },
  { 1, 4, 16, 64, 128, 256, 4096, 32768, },
  { 2, 4, 16, 64, 128, 256, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 4096, 32768, },
  { 8, 16, 64, 128, 256, 4096, 32768, },
  { 1, 8, 16, 64, 128, 256, 4096, 32768, },
  { 2, 8, 16, 64, 128, 256, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 4096, 32768, },
  { 4, 8, 16, 64, 128, 256, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 4096, 32768, },
  { 32, 64, 128, 256, 4096, 32768, },
  { 1, 32, 64, 128, 256, 4096, 32768, },
  { 2, 32, 64, 128, 256, 4096, 32768, },
  { 1, 2, 32, 64, 128, 256, 4096, 32768, },
  { 4, 32, 64, 128, 256, 4096, 32768, },
  { 1, 4, 32, 64, 128, 256, 4096, 32768, },
  { 2, 4, 32, 64, 128, 256, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 4096, 32768, },
  { 8, 32, 64, 128, 256, 4096, 32768, },
  { 1, 8, 32, 64, 128, 256, 4096, 32768, },
  { 2, 8, 32, 64, 128, 256, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 4096, 32768, },
  { 4, 8, 32, 64, 128, 256, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 4096, 32768, },
  { 16, 32, 64, 128, 256, 4096, 32768, },
  { 1, 16, 32, 64, 128, 256, 4096, 32768, },
  { 2, 16, 32, 64, 128, 256, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 4096, 32768, },
  { 4, 16, 32, 64, 128, 256, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 4096, 32768, },
  { 8, 16, 32, 64, 128, 256, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 4096, 32768, },
  { 512, 4096, 32768, },
  { 1, 512, 4096, 32768, },
  { 2, 512, 4096, 32768, },
  { 1, 2, 512, 4096, 32768, },
  { 4, 512, 4096, 32768, },
  { 1, 4, 512, 4096, 32768, },
  { 2, 4, 512, 4096, 32768, },
  { 1, 2, 4, 512, 4096, 32768, },
  { 8, 512, 4096, 32768, },
  { 1, 8, 512, 4096, 32768, },
  { 2, 8, 512, 4096, 32768, },
  { 1, 2, 8, 512, 4096, 32768, },
  { 4, 8, 512, 4096, 32768, },
  { 1, 4, 8, 512, 4096, 32768, },
  { 2, 4, 8, 512, 4096, 32768, },
  { 1, 2, 4, 8, 512, 4096, 32768, },
  { 16, 512, 4096, 32768, },
  { 1, 16, 512, 4096, 32768, },
  { 2, 16, 512, 4096, 32768, },
  { 1, 2, 16, 512, 4096, 32768, },
  { 4, 16, 512, 4096, 32768, },
  { 1, 4, 16, 512, 4096, 32768, },
  { 2, 4, 16, 512, 4096, 32768, },
  { 1, 2, 4, 16, 512, 4096, 32768, },
  { 8, 16, 512, 4096, 32768, },
  { 1, 8, 16, 512, 4096, 32768, },
  { 2, 8, 16, 512, 4096, 32768, },
  { 1, 2, 8, 16, 512, 4096, 32768, },
  { 4, 8, 16, 512, 4096, 32768, },
  { 1, 4, 8, 16, 512, 4096, 32768, },
  { 2, 4, 8, 16, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 512, 4096, 32768, },
  { 32, 512, 4096, 32768, },
  { 1, 32, 512, 4096, 32768, },
  { 2, 32, 512, 4096, 32768, },
  { 1, 2, 32, 512, 4096, 32768, },
  { 4, 32, 512, 4096, 32768, },
  { 1, 4, 32, 512, 4096, 32768, },
  { 2, 4, 32, 512, 4096, 32768, },
  { 1, 2, 4, 32, 512, 4096, 32768, },
  { 8, 32, 512, 4096, 32768, },
  { 1, 8, 32, 512, 4096, 32768, },
  { 2, 8, 32, 512, 4096, 32768, },
  { 1, 2, 8, 32, 512, 4096, 32768, },
  { 4, 8, 32, 512, 4096, 32768, },
  { 1, 4, 8, 32, 512, 4096, 32768, },
  { 2, 4, 8, 32, 512, 4096, 32768, },
  { 1, 2, 4, 8, 32, 512, 4096, 32768, },
  { 16, 32, 512, 4096, 32768, },
  { 1, 16, 32, 512, 4096, 32768, },
  { 2, 16, 32, 512, 4096, 32768, },
  { 1, 2, 16, 32, 512, 4096, 32768, },
  { 4, 16, 32, 512, 4096, 32768, },
  { 1, 4, 16, 32, 512, 4096, 32768, },
  { 2, 4, 16, 32, 512, 4096, 32768, },
  { 1, 2, 4, 16, 32, 512, 4096, 32768, },
  { 8, 16, 32, 512, 4096, 32768, },
  { 1, 8, 16, 32, 512, 4096, 32768, },
  { 2, 8, 16, 32, 512, 4096, 32768, },
  { 1, 2, 8, 16, 32, 512, 4096, 32768, },
  { 4, 8, 16, 32, 512, 4096, 32768, },
  { 1, 4, 8, 16, 32, 512, 4096, 32768, },
  { 2, 4, 8, 16, 32, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 4096, 32768, },
  { 64, 512, 4096, 32768, },
  { 1, 64, 512, 4096, 32768, },
  { 2, 64, 512, 4096, 32768, },
  { 1, 2, 64, 512, 4096, 32768, },
  { 4, 64, 512, 4096, 32768, },
  { 1, 4, 64, 512, 4096, 32768, },
  { 2, 4, 64, 512, 4096, 32768, },
  { 1, 2, 4, 64, 512, 4096, 32768, },
  { 8, 64, 512, 4096, 32768, },
  { 1, 8, 64, 512, 4096, 32768, },
  { 2, 8, 64, 512, 4096, 32768, },
  { 1, 2, 8, 64, 512, 4096, 32768, },
  { 4, 8, 64, 512, 4096, 32768, },
  { 1, 4, 8, 64, 512, 4096, 32768, },
  { 2, 4, 8, 64, 512, 4096, 32768, },
  { 1, 2, 4, 8, 64, 512, 4096, 32768, },
  { 16, 64, 512, 4096, 32768, },
  { 1, 16, 64, 512, 4096, 32768, },
  { 2, 16, 64, 512, 4096, 32768, },
  { 1, 2, 16, 64, 512, 4096, 32768, },
  { 4, 16, 64, 512, 4096, 32768, },
  { 1, 4, 16, 64, 512, 4096, 32768, },
  { 2, 4, 16, 64, 512, 4096, 32768, },
  { 1, 2, 4, 16, 64, 512, 4096, 32768, },
  { 8, 16, 64, 512, 4096, 32768, },
  { 1, 8, 16, 64, 512, 4096, 32768, },
  { 2, 8, 16, 64, 512, 4096, 32768, },
  { 1, 2, 8, 16, 64, 512, 4096, 32768, },
  { 4, 8, 16, 64, 512, 4096, 32768, },
  { 1, 4, 8, 16, 64, 512, 4096, 32768, },
  { 2, 4, 8, 16, 64, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 4096, 32768, },
  { 32, 64, 512, 4096, 32768, },
  { 1, 32, 64, 512, 4096, 32768, },
  { 2, 32, 64, 512, 4096, 32768, },
  { 1, 2, 32, 64, 512, 4096, 32768, },
  { 4, 32, 64, 512, 4096, 32768, },
  { 1, 4, 32, 64, 512, 4096, 32768, },
  { 2, 4, 32, 64, 512, 4096, 32768, },
  { 1, 2, 4, 32, 64, 512, 4096, 32768, },
  { 8, 32, 64, 512, 4096, 32768, },
  { 1, 8, 32, 64, 512, 4096, 32768, },
  { 2, 8, 32, 64, 512, 4096, 32768, },
  { 1, 2, 8, 32, 64, 512, 4096, 32768, },
  { 4, 8, 32, 64, 512, 4096, 32768, },
  { 1, 4, 8, 32, 64, 512, 4096, 32768, },
  { 2, 4, 8, 32, 64, 512, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 4096, 32768, },
  { 16, 32, 64, 512, 4096, 32768, },
  { 1, 16, 32, 64, 512, 4096, 32768, },
  { 2, 16, 32, 64, 512, 4096, 32768, },
  { 1, 2, 16, 32, 64, 512, 4096, 32768, },
  { 4, 16, 32, 64, 512, 4096, 32768, },
  { 1, 4, 16, 32, 64, 512, 4096, 32768, },
  { 2, 4, 16, 32, 64, 512, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 4096, 32768, },
  { 8, 16, 32, 64, 512, 4096, 32768, },
  { 1, 8, 16, 32, 64, 512, 4096, 32768, },
  { 2, 8, 16, 32, 64, 512, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 4096, 32768, },
  { 4, 8, 16, 32, 64, 512, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 4096, 32768, },
  { 128, 512, 4096, 32768, },
  { 1, 128, 512, 4096, 32768, },
  { 2, 128, 512, 4096, 32768, },
  { 1, 2, 128, 512, 4096, 32768, },
  { 4, 128, 512, 4096, 32768, },
  { 1, 4, 128, 512, 4096, 32768, },
  { 2, 4, 128, 512, 4096, 32768, },
  { 1, 2, 4, 128, 512, 4096, 32768, },
  { 8, 128, 512, 4096, 32768, },
  { 1, 8, 128, 512, 4096, 32768, },
  { 2, 8, 128, 512, 4096, 32768, },
  { 1, 2, 8, 128, 512, 4096, 32768, },
  { 4, 8, 128, 512, 4096, 32768, },
  { 1, 4, 8, 128, 512, 4096, 32768, },
  { 2, 4, 8, 128, 512, 4096, 32768, },
  { 1, 2, 4, 8, 128, 512, 4096, 32768, },
  { 16, 128, 512, 4096, 32768, },
  { 1, 16, 128, 512, 4096, 32768, },
  { 2, 16, 128, 512, 4096, 32768, },
  { 1, 2, 16, 128, 512, 4096, 32768, },
  { 4, 16, 128, 512, 4096, 32768, },
  { 1, 4, 16, 128, 512, 4096, 32768, },
  { 2, 4, 16, 128, 512, 4096, 32768, },
  { 1, 2, 4, 16, 128, 512, 4096, 32768, },
  { 8, 16, 128, 512, 4096, 32768, },
  { 1, 8, 16, 128, 512, 4096, 32768, },
  { 2, 8, 16, 128, 512, 4096, 32768, },
  { 1, 2, 8, 16, 128, 512, 4096, 32768, },
  { 4, 8, 16, 128, 512, 4096, 32768, },
  { 1, 4, 8, 16, 128, 512, 4096, 32768, },
  { 2, 4, 8, 16, 128, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 4096, 32768, },
  { 32, 128, 512, 4096, 32768, },
  { 1, 32, 128, 512, 4096, 32768, },
  { 2, 32, 128, 512, 4096, 32768, },
  { 1, 2, 32, 128, 512, 4096, 32768, },
  { 4, 32, 128, 512, 4096, 32768, },
  { 1, 4, 32, 128, 512, 4096, 32768, },
  { 2, 4, 32, 128, 512, 4096, 32768, },
  { 1, 2, 4, 32, 128, 512, 4096, 32768, },
  { 8, 32, 128, 512, 4096, 32768, },
  { 1, 8, 32, 128, 512, 4096, 32768, },
  { 2, 8, 32, 128, 512, 4096, 32768, },
  { 1, 2, 8, 32, 128, 512, 4096, 32768, },
  { 4, 8, 32, 128, 512, 4096, 32768, },
  { 1, 4, 8, 32, 128, 512, 4096, 32768, },
  { 2, 4, 8, 32, 128, 512, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 4096, 32768, },
  { 16, 32, 128, 512, 4096, 32768, },
  { 1, 16, 32, 128, 512, 4096, 32768, },
  { 2, 16, 32, 128, 512, 4096, 32768, },
  { 1, 2, 16, 32, 128, 512, 4096, 32768, },
  { 4, 16, 32, 128, 512, 4096, 32768, },
  { 1, 4, 16, 32, 128, 512, 4096, 32768, },
  { 2, 4, 16, 32, 128, 512, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 4096, 32768, },
  { 8, 16, 32, 128, 512, 4096, 32768, },
  { 1, 8, 16, 32, 128, 512, 4096, 32768, },
  { 2, 8, 16, 32, 128, 512, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 4096, 32768, },
  { 4, 8, 16, 32, 128, 512, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 4096, 32768, },
  { 64, 128, 512, 4096, 32768, },
  { 1, 64, 128, 512, 4096, 32768, },
  { 2, 64, 128, 512, 4096, 32768, },
  { 1, 2, 64, 128, 512, 4096, 32768, },
  { 4, 64, 128, 512, 4096, 32768, },
  { 1, 4, 64, 128, 512, 4096, 32768, },
  { 2, 4, 64, 128, 512, 4096, 32768, },
  { 1, 2, 4, 64, 128, 512, 4096, 32768, },
  { 8, 64, 128, 512, 4096, 32768, },
  { 1, 8, 64, 128, 512, 4096, 32768, },
  { 2, 8, 64, 128, 512, 4096, 32768, },
  { 1, 2, 8, 64, 128, 512, 4096, 32768, },
  { 4, 8, 64, 128, 512, 4096, 32768, },
  { 1, 4, 8, 64, 128, 512, 4096, 32768, },
  { 2, 4, 8, 64, 128, 512, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 4096, 32768, },
  { 16, 64, 128, 512, 4096, 32768, },
  { 1, 16, 64, 128, 512, 4096, 32768, },
  { 2, 16, 64, 128, 512, 4096, 32768, },
  { 1, 2, 16, 64, 128, 512, 4096, 32768, },
  { 4, 16, 64, 128, 512, 4096, 32768, },
  { 1, 4, 16, 64, 128, 512, 4096, 32768, },
  { 2, 4, 16, 64, 128, 512, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 4096, 32768, },
  { 8, 16, 64, 128, 512, 4096, 32768, },
  { 1, 8, 16, 64, 128, 512, 4096, 32768, },
  { 2, 8, 16, 64, 128, 512, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 4096, 32768, },
  { 4, 8, 16, 64, 128, 512, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 4096, 32768, },
  { 32, 64, 128, 512, 4096, 32768, },
  { 1, 32, 64, 128, 512, 4096, 32768, },
  { 2, 32, 64, 128, 512, 4096, 32768, },
  { 1, 2, 32, 64, 128, 512, 4096, 32768, },
  { 4, 32, 64, 128, 512, 4096, 32768, },
  { 1, 4, 32, 64, 128, 512, 4096, 32768, },
  { 2, 4, 32, 64, 128, 512, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 4096, 32768, },
  { 8, 32, 64, 128, 512, 4096, 32768, },
  { 1, 8, 32, 64, 128, 512, 4096, 32768, },
  { 2, 8, 32, 64, 128, 512, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 4096, 32768, },
  { 4, 8, 32, 64, 128, 512, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 4096, 32768, },
  { 16, 32, 64, 128, 512, 4096, 32768, },
  { 1, 16, 32, 64, 128, 512, 4096, 32768, },
  { 2, 16, 32, 64, 128, 512, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 4096, 32768, },
  { 4, 16, 32, 64, 128, 512, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 4096, 32768, },
  { 8, 16, 32, 64, 128, 512, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 4096, 32768, },
  { 256, 512, 4096, 32768, },
  { 1, 256, 512, 4096, 32768, },
  { 2, 256, 512, 4096, 32768, },
  { 1, 2, 256, 512, 4096, 32768, },
  { 4, 256, 512, 4096, 32768, },
  { 1, 4, 256, 512, 4096, 32768, },
  { 2, 4, 256, 512, 4096, 32768, },
  { 1, 2, 4, 256, 512, 4096, 32768, },
  { 8, 256, 512, 4096, 32768, },
  { 1, 8, 256, 512, 4096, 32768, },
  { 2, 8, 256, 512, 4096, 32768, },
  { 1, 2, 8, 256, 512, 4096, 32768, },
  { 4, 8, 256, 512, 4096, 32768, },
  { 1, 4, 8, 256, 512, 4096, 32768, },
  { 2, 4, 8, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 256, 512, 4096, 32768, },
  { 16, 256, 512, 4096, 32768, },
  { 1, 16, 256, 512, 4096, 32768, },
  { 2, 16, 256, 512, 4096, 32768, },
  { 1, 2, 16, 256, 512, 4096, 32768, },
  { 4, 16, 256, 512, 4096, 32768, },
  { 1, 4, 16, 256, 512, 4096, 32768, },
  { 2, 4, 16, 256, 512, 4096, 32768, },
  { 1, 2, 4, 16, 256, 512, 4096, 32768, },
  { 8, 16, 256, 512, 4096, 32768, },
  { 1, 8, 16, 256, 512, 4096, 32768, },
  { 2, 8, 16, 256, 512, 4096, 32768, },
  { 1, 2, 8, 16, 256, 512, 4096, 32768, },
  { 4, 8, 16, 256, 512, 4096, 32768, },
  { 1, 4, 8, 16, 256, 512, 4096, 32768, },
  { 2, 4, 8, 16, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 4096, 32768, },
  { 32, 256, 512, 4096, 32768, },
  { 1, 32, 256, 512, 4096, 32768, },
  { 2, 32, 256, 512, 4096, 32768, },
  { 1, 2, 32, 256, 512, 4096, 32768, },
  { 4, 32, 256, 512, 4096, 32768, },
  { 1, 4, 32, 256, 512, 4096, 32768, },
  { 2, 4, 32, 256, 512, 4096, 32768, },
  { 1, 2, 4, 32, 256, 512, 4096, 32768, },
  { 8, 32, 256, 512, 4096, 32768, },
  { 1, 8, 32, 256, 512, 4096, 32768, },
  { 2, 8, 32, 256, 512, 4096, 32768, },
  { 1, 2, 8, 32, 256, 512, 4096, 32768, },
  { 4, 8, 32, 256, 512, 4096, 32768, },
  { 1, 4, 8, 32, 256, 512, 4096, 32768, },
  { 2, 4, 8, 32, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 4096, 32768, },
  { 16, 32, 256, 512, 4096, 32768, },
  { 1, 16, 32, 256, 512, 4096, 32768, },
  { 2, 16, 32, 256, 512, 4096, 32768, },
  { 1, 2, 16, 32, 256, 512, 4096, 32768, },
  { 4, 16, 32, 256, 512, 4096, 32768, },
  { 1, 4, 16, 32, 256, 512, 4096, 32768, },
  { 2, 4, 16, 32, 256, 512, 4096, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 4096, 32768, },
  { 8, 16, 32, 256, 512, 4096, 32768, },
  { 1, 8, 16, 32, 256, 512, 4096, 32768, },
  { 2, 8, 16, 32, 256, 512, 4096, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 4096, 32768, },
  { 4, 8, 16, 32, 256, 512, 4096, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 4096, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 4096, 32768, },
  { 64, 256, 512, 4096, 32768, },
  { 1, 64, 256, 512, 4096, 32768, },
  { 2, 64, 256, 512, 4096, 32768, },
  { 1, 2, 64, 256, 512, 4096, 32768, },
  { 4, 64, 256, 512, 4096, 32768, },
  { 1, 4, 64, 256, 512, 4096, 32768, },
  { 2, 4, 64, 256, 512, 4096, 32768, },
  { 1, 2, 4, 64, 256, 512, 4096, 32768, },
  { 8, 64, 256, 512, 4096, 32768, },
  { 1, 8, 64, 256, 512, 4096, 32768, },
  { 2, 8, 64, 256, 512, 4096, 32768, },
  { 1, 2, 8, 64, 256, 512, 4096, 32768, },
  { 4, 8, 64, 256, 512, 4096, 32768, },
  { 1, 4, 8, 64, 256, 512, 4096, 32768, },
  { 2, 4, 8, 64, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 4096, 32768, },
  { 16, 64, 256, 512, 4096, 32768, },
  { 1, 16, 64, 256, 512, 4096, 32768, },
  { 2, 16, 64, 256, 512, 4096, 32768, },
  { 1, 2, 16, 64, 256, 512, 4096, 32768, },
  { 4, 16, 64, 256, 512, 4096, 32768, },
  { 1, 4, 16, 64, 256, 512, 4096, 32768, },
  { 2, 4, 16, 64, 256, 512, 4096, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 4096, 32768, },
  { 8, 16, 64, 256, 512, 4096, 32768, },
  { 1, 8, 16, 64, 256, 512, 4096, 32768, },
  { 2, 8, 16, 64, 256, 512, 4096, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 4096, 32768, },
  { 4, 8, 16, 64, 256, 512, 4096, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 4096, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 4096, 32768, },
  { 32, 64, 256, 512, 4096, 32768, },
  { 1, 32, 64, 256, 512, 4096, 32768, },
  { 2, 32, 64, 256, 512, 4096, 32768, },
  { 1, 2, 32, 64, 256, 512, 4096, 32768, },
  { 4, 32, 64, 256, 512, 4096, 32768, },
  { 1, 4, 32, 64, 256, 512, 4096, 32768, },
  { 2, 4, 32, 64, 256, 512, 4096, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 4096, 32768, },
  { 8, 32, 64, 256, 512, 4096, 32768, },
  { 1, 8, 32, 64, 256, 512, 4096, 32768, },
  { 2, 8, 32, 64, 256, 512, 4096, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 4096, 32768, },
  { 4, 8, 32, 64, 256, 512, 4096, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 4096, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 4096, 32768, },
  { 16, 32, 64, 256, 512, 4096, 32768, },
  { 1, 16, 32, 64, 256, 512, 4096, 32768, },
  { 2, 16, 32, 64, 256, 512, 4096, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 4096, 32768, },
  { 4, 16, 32, 64, 256, 512, 4096, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 4096, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 4096, 32768, },
  { 8, 16, 32, 64, 256, 512, 4096, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 4096, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 4096, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 4096, 32768, },
  { 128, 256, 512, 4096, 32768, },
  { 1, 128, 256, 512, 4096, 32768, },
  { 2, 128, 256, 512, 4096, 32768, },
  { 1, 2, 128, 256, 512, 4096, 32768, },
  { 4, 128, 256, 512, 4096, 32768, },
  { 1, 4, 128, 256, 512, 4096, 32768, },
  { 2, 4, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 128, 256, 512, 4096, 32768, },
  { 8, 128, 256, 512, 4096, 32768, },
  { 1, 8, 128, 256, 512, 4096, 32768, },
  { 2, 8, 128, 256, 512, 4096, 32768, },
  { 1, 2, 8, 128, 256, 512, 4096, 32768, },
  { 4, 8, 128, 256, 512, 4096, 32768, },
  { 1, 4, 8, 128, 256, 512, 4096, 32768, },
  { 2, 4, 8, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 4096, 32768, },
  { 16, 128, 256, 512, 4096, 32768, },
  { 1, 16, 128, 256, 512, 4096, 32768, },
  { 2, 16, 128, 256, 512, 4096, 32768, },
  { 1, 2, 16, 128, 256, 512, 4096, 32768, },
  { 4, 16, 128, 256, 512, 4096, 32768, },
  { 1, 4, 16, 128, 256, 512, 4096, 32768, },
  { 2, 4, 16, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 4096, 32768, },
  { 8, 16, 128, 256, 512, 4096, 32768, },
  { 1, 8, 16, 128, 256, 512, 4096, 32768, },
  { 2, 8, 16, 128, 256, 512, 4096, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 4096, 32768, },
  { 4, 8, 16, 128, 256, 512, 4096, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 4096, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 4096, 32768, },
  { 32, 128, 256, 512, 4096, 32768, },
  { 1, 32, 128, 256, 512, 4096, 32768, },
  { 2, 32, 128, 256, 512, 4096, 32768, },
  { 1, 2, 32, 128, 256, 512, 4096, 32768, },
  { 4, 32, 128, 256, 512, 4096, 32768, },
  { 1, 4, 32, 128, 256, 512, 4096, 32768, },
  { 2, 4, 32, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 4096, 32768, },
  { 8, 32, 128, 256, 512, 4096, 32768, },
  { 1, 8, 32, 128, 256, 512, 4096, 32768, },
  { 2, 8, 32, 128, 256, 512, 4096, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 4096, 32768, },
  { 4, 8, 32, 128, 256, 512, 4096, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 4096, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 4096, 32768, },
  { 16, 32, 128, 256, 512, 4096, 32768, },
  { 1, 16, 32, 128, 256, 512, 4096, 32768, },
  { 2, 16, 32, 128, 256, 512, 4096, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 4096, 32768, },
  { 4, 16, 32, 128, 256, 512, 4096, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 4096, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 4096, 32768, },
  { 8, 16, 32, 128, 256, 512, 4096, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 4096, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 4096, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 4096, 32768, },
  { 64, 128, 256, 512, 4096, 32768, },
  { 1, 64, 128, 256, 512, 4096, 32768, },
  { 2, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 64, 128, 256, 512, 4096, 32768, },
  { 4, 64, 128, 256, 512, 4096, 32768, },
  { 1, 4, 64, 128, 256, 512, 4096, 32768, },
  { 2, 4, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 4096, 32768, },
  { 8, 64, 128, 256, 512, 4096, 32768, },
  { 1, 8, 64, 128, 256, 512, 4096, 32768, },
  { 2, 8, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 4096, 32768, },
  { 4, 8, 64, 128, 256, 512, 4096, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 4096, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 4096, 32768, },
  { 16, 64, 128, 256, 512, 4096, 32768, },
  { 1, 16, 64, 128, 256, 512, 4096, 32768, },
  { 2, 16, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 4096, 32768, },
  { 4, 16, 64, 128, 256, 512, 4096, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 4096, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 4096, 32768, },
  { 8, 16, 64, 128, 256, 512, 4096, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 4096, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 4096, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 4096, 32768, },
  { 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 32, 64, 128, 256, 512, 4096, 32768, },
  { 2, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 4096, 32768, },
  { 4, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 4096, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 4096, 32768, },
  { 8, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 4096, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 4096, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 4096, 32768, },
  { 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 32768, },
  { 1024, 4096, 32768, },
  { 1, 1024, 4096, 32768, },
  { 2, 1024, 4096, 32768, },
  { 1, 2, 1024, 4096, 32768, },
  { 4, 1024, 4096, 32768, },
  { 1, 4, 1024, 4096, 32768, },
  { 2, 4, 1024, 4096, 32768, },
  { 1, 2, 4, 1024, 4096, 32768, },
  { 8, 1024, 4096, 32768, },
  { 1, 8, 1024, 4096, 32768, },
  { 2, 8, 1024, 4096, 32768, },
  { 1, 2, 8, 1024, 4096, 32768, },
  { 4, 8, 1024, 4096, 32768, },
  { 1, 4, 8, 1024, 4096, 32768, },
  { 2, 4, 8, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 1024, 4096, 32768, },
  { 16, 1024, 4096, 32768, },
  { 1, 16, 1024, 4096, 32768, },
  { 2, 16, 1024, 4096, 32768, },
  { 1, 2, 16, 1024, 4096, 32768, },
  { 4, 16, 1024, 4096, 32768, },
  { 1, 4, 16, 1024, 4096, 32768, },
  { 2, 4, 16, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 1024, 4096, 32768, },
  { 8, 16, 1024, 4096, 32768, },
  { 1, 8, 16, 1024, 4096, 32768, },
  { 2, 8, 16, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 1024, 4096, 32768, },
  { 4, 8, 16, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 1024, 4096, 32768, },
  { 32, 1024, 4096, 32768, },
  { 1, 32, 1024, 4096, 32768, },
  { 2, 32, 1024, 4096, 32768, },
  { 1, 2, 32, 1024, 4096, 32768, },
  { 4, 32, 1024, 4096, 32768, },
  { 1, 4, 32, 1024, 4096, 32768, },
  { 2, 4, 32, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 1024, 4096, 32768, },
  { 8, 32, 1024, 4096, 32768, },
  { 1, 8, 32, 1024, 4096, 32768, },
  { 2, 8, 32, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 1024, 4096, 32768, },
  { 4, 8, 32, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 1024, 4096, 32768, },
  { 16, 32, 1024, 4096, 32768, },
  { 1, 16, 32, 1024, 4096, 32768, },
  { 2, 16, 32, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 1024, 4096, 32768, },
  { 4, 16, 32, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 1024, 4096, 32768, },
  { 8, 16, 32, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 4096, 32768, },
  { 64, 1024, 4096, 32768, },
  { 1, 64, 1024, 4096, 32768, },
  { 2, 64, 1024, 4096, 32768, },
  { 1, 2, 64, 1024, 4096, 32768, },
  { 4, 64, 1024, 4096, 32768, },
  { 1, 4, 64, 1024, 4096, 32768, },
  { 2, 4, 64, 1024, 4096, 32768, },
  { 1, 2, 4, 64, 1024, 4096, 32768, },
  { 8, 64, 1024, 4096, 32768, },
  { 1, 8, 64, 1024, 4096, 32768, },
  { 2, 8, 64, 1024, 4096, 32768, },
  { 1, 2, 8, 64, 1024, 4096, 32768, },
  { 4, 8, 64, 1024, 4096, 32768, },
  { 1, 4, 8, 64, 1024, 4096, 32768, },
  { 2, 4, 8, 64, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 64, 1024, 4096, 32768, },
  { 16, 64, 1024, 4096, 32768, },
  { 1, 16, 64, 1024, 4096, 32768, },
  { 2, 16, 64, 1024, 4096, 32768, },
  { 1, 2, 16, 64, 1024, 4096, 32768, },
  { 4, 16, 64, 1024, 4096, 32768, },
  { 1, 4, 16, 64, 1024, 4096, 32768, },
  { 2, 4, 16, 64, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 64, 1024, 4096, 32768, },
  { 8, 16, 64, 1024, 4096, 32768, },
  { 1, 8, 16, 64, 1024, 4096, 32768, },
  { 2, 8, 16, 64, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 64, 1024, 4096, 32768, },
  { 4, 8, 16, 64, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 64, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 64, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 4096, 32768, },
  { 32, 64, 1024, 4096, 32768, },
  { 1, 32, 64, 1024, 4096, 32768, },
  { 2, 32, 64, 1024, 4096, 32768, },
  { 1, 2, 32, 64, 1024, 4096, 32768, },
  { 4, 32, 64, 1024, 4096, 32768, },
  { 1, 4, 32, 64, 1024, 4096, 32768, },
  { 2, 4, 32, 64, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 64, 1024, 4096, 32768, },
  { 8, 32, 64, 1024, 4096, 32768, },
  { 1, 8, 32, 64, 1024, 4096, 32768, },
  { 2, 8, 32, 64, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 64, 1024, 4096, 32768, },
  { 4, 8, 32, 64, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 64, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 64, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 4096, 32768, },
  { 16, 32, 64, 1024, 4096, 32768, },
  { 1, 16, 32, 64, 1024, 4096, 32768, },
  { 2, 16, 32, 64, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 64, 1024, 4096, 32768, },
  { 4, 16, 32, 64, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 64, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 64, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 4096, 32768, },
  { 8, 16, 32, 64, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 64, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 64, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 64, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 4096, 32768, },
  { 128, 1024, 4096, 32768, },
  { 1, 128, 1024, 4096, 32768, },
  { 2, 128, 1024, 4096, 32768, },
  { 1, 2, 128, 1024, 4096, 32768, },
  { 4, 128, 1024, 4096, 32768, },
  { 1, 4, 128, 1024, 4096, 32768, },
  { 2, 4, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 128, 1024, 4096, 32768, },
  { 8, 128, 1024, 4096, 32768, },
  { 1, 8, 128, 1024, 4096, 32768, },
  { 2, 8, 128, 1024, 4096, 32768, },
  { 1, 2, 8, 128, 1024, 4096, 32768, },
  { 4, 8, 128, 1024, 4096, 32768, },
  { 1, 4, 8, 128, 1024, 4096, 32768, },
  { 2, 4, 8, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 128, 1024, 4096, 32768, },
  { 16, 128, 1024, 4096, 32768, },
  { 1, 16, 128, 1024, 4096, 32768, },
  { 2, 16, 128, 1024, 4096, 32768, },
  { 1, 2, 16, 128, 1024, 4096, 32768, },
  { 4, 16, 128, 1024, 4096, 32768, },
  { 1, 4, 16, 128, 1024, 4096, 32768, },
  { 2, 4, 16, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 128, 1024, 4096, 32768, },
  { 8, 16, 128, 1024, 4096, 32768, },
  { 1, 8, 16, 128, 1024, 4096, 32768, },
  { 2, 8, 16, 128, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 128, 1024, 4096, 32768, },
  { 4, 8, 16, 128, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 128, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 4096, 32768, },
  { 32, 128, 1024, 4096, 32768, },
  { 1, 32, 128, 1024, 4096, 32768, },
  { 2, 32, 128, 1024, 4096, 32768, },
  { 1, 2, 32, 128, 1024, 4096, 32768, },
  { 4, 32, 128, 1024, 4096, 32768, },
  { 1, 4, 32, 128, 1024, 4096, 32768, },
  { 2, 4, 32, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 128, 1024, 4096, 32768, },
  { 8, 32, 128, 1024, 4096, 32768, },
  { 1, 8, 32, 128, 1024, 4096, 32768, },
  { 2, 8, 32, 128, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 128, 1024, 4096, 32768, },
  { 4, 8, 32, 128, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 128, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 4096, 32768, },
  { 16, 32, 128, 1024, 4096, 32768, },
  { 1, 16, 32, 128, 1024, 4096, 32768, },
  { 2, 16, 32, 128, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 128, 1024, 4096, 32768, },
  { 4, 16, 32, 128, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 128, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 4096, 32768, },
  { 8, 16, 32, 128, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 128, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 128, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 128, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 4096, 32768, },
  { 64, 128, 1024, 4096, 32768, },
  { 1, 64, 128, 1024, 4096, 32768, },
  { 2, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 64, 128, 1024, 4096, 32768, },
  { 4, 64, 128, 1024, 4096, 32768, },
  { 1, 4, 64, 128, 1024, 4096, 32768, },
  { 2, 4, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 64, 128, 1024, 4096, 32768, },
  { 8, 64, 128, 1024, 4096, 32768, },
  { 1, 8, 64, 128, 1024, 4096, 32768, },
  { 2, 8, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 8, 64, 128, 1024, 4096, 32768, },
  { 4, 8, 64, 128, 1024, 4096, 32768, },
  { 1, 4, 8, 64, 128, 1024, 4096, 32768, },
  { 2, 4, 8, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 4096, 32768, },
  { 16, 64, 128, 1024, 4096, 32768, },
  { 1, 16, 64, 128, 1024, 4096, 32768, },
  { 2, 16, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 16, 64, 128, 1024, 4096, 32768, },
  { 4, 16, 64, 128, 1024, 4096, 32768, },
  { 1, 4, 16, 64, 128, 1024, 4096, 32768, },
  { 2, 4, 16, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 4096, 32768, },
  { 8, 16, 64, 128, 1024, 4096, 32768, },
  { 1, 8, 16, 64, 128, 1024, 4096, 32768, },
  { 2, 8, 16, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 4096, 32768, },
  { 4, 8, 16, 64, 128, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 4096, 32768, },
  { 32, 64, 128, 1024, 4096, 32768, },
  { 1, 32, 64, 128, 1024, 4096, 32768, },
  { 2, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 32, 64, 128, 1024, 4096, 32768, },
  { 4, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 4, 32, 64, 128, 1024, 4096, 32768, },
  { 2, 4, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 4096, 32768, },
  { 8, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 8, 32, 64, 128, 1024, 4096, 32768, },
  { 2, 8, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 4096, 32768, },
  { 4, 8, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 4096, 32768, },
  { 16, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 2, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 4, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 8, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 32768, },
  { 256, 1024, 4096, 32768, },
  { 1, 256, 1024, 4096, 32768, },
  { 2, 256, 1024, 4096, 32768, },
  { 1, 2, 256, 1024, 4096, 32768, },
  { 4, 256, 1024, 4096, 32768, },
  { 1, 4, 256, 1024, 4096, 32768, },
  { 2, 4, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 256, 1024, 4096, 32768, },
  { 8, 256, 1024, 4096, 32768, },
  { 1, 8, 256, 1024, 4096, 32768, },
  { 2, 8, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 256, 1024, 4096, 32768, },
  { 4, 8, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 256, 1024, 4096, 32768, },
  { 16, 256, 1024, 4096, 32768, },
  { 1, 16, 256, 1024, 4096, 32768, },
  { 2, 16, 256, 1024, 4096, 32768, },
  { 1, 2, 16, 256, 1024, 4096, 32768, },
  { 4, 16, 256, 1024, 4096, 32768, },
  { 1, 4, 16, 256, 1024, 4096, 32768, },
  { 2, 4, 16, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 256, 1024, 4096, 32768, },
  { 8, 16, 256, 1024, 4096, 32768, },
  { 1, 8, 16, 256, 1024, 4096, 32768, },
  { 2, 8, 16, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 256, 1024, 4096, 32768, },
  { 4, 8, 16, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 4096, 32768, },
  { 32, 256, 1024, 4096, 32768, },
  { 1, 32, 256, 1024, 4096, 32768, },
  { 2, 32, 256, 1024, 4096, 32768, },
  { 1, 2, 32, 256, 1024, 4096, 32768, },
  { 4, 32, 256, 1024, 4096, 32768, },
  { 1, 4, 32, 256, 1024, 4096, 32768, },
  { 2, 4, 32, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 256, 1024, 4096, 32768, },
  { 8, 32, 256, 1024, 4096, 32768, },
  { 1, 8, 32, 256, 1024, 4096, 32768, },
  { 2, 8, 32, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 256, 1024, 4096, 32768, },
  { 4, 8, 32, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 4096, 32768, },
  { 16, 32, 256, 1024, 4096, 32768, },
  { 1, 16, 32, 256, 1024, 4096, 32768, },
  { 2, 16, 32, 256, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 256, 1024, 4096, 32768, },
  { 4, 16, 32, 256, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 256, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 4096, 32768, },
  { 8, 16, 32, 256, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 256, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 4096, 32768, },
  { 64, 256, 1024, 4096, 32768, },
  { 1, 64, 256, 1024, 4096, 32768, },
  { 2, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 64, 256, 1024, 4096, 32768, },
  { 4, 64, 256, 1024, 4096, 32768, },
  { 1, 4, 64, 256, 1024, 4096, 32768, },
  { 2, 4, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 64, 256, 1024, 4096, 32768, },
  { 8, 64, 256, 1024, 4096, 32768, },
  { 1, 8, 64, 256, 1024, 4096, 32768, },
  { 2, 8, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 64, 256, 1024, 4096, 32768, },
  { 4, 8, 64, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 64, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 4096, 32768, },
  { 16, 64, 256, 1024, 4096, 32768, },
  { 1, 16, 64, 256, 1024, 4096, 32768, },
  { 2, 16, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 16, 64, 256, 1024, 4096, 32768, },
  { 4, 16, 64, 256, 1024, 4096, 32768, },
  { 1, 4, 16, 64, 256, 1024, 4096, 32768, },
  { 2, 4, 16, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 4096, 32768, },
  { 8, 16, 64, 256, 1024, 4096, 32768, },
  { 1, 8, 16, 64, 256, 1024, 4096, 32768, },
  { 2, 8, 16, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 4096, 32768, },
  { 4, 8, 16, 64, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 4096, 32768, },
  { 32, 64, 256, 1024, 4096, 32768, },
  { 1, 32, 64, 256, 1024, 4096, 32768, },
  { 2, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 32, 64, 256, 1024, 4096, 32768, },
  { 4, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 4, 32, 64, 256, 1024, 4096, 32768, },
  { 2, 4, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 4096, 32768, },
  { 8, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 8, 32, 64, 256, 1024, 4096, 32768, },
  { 2, 8, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 4096, 32768, },
  { 4, 8, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 4096, 32768, },
  { 16, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 2, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 4, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 8, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 32768, },
  { 128, 256, 1024, 4096, 32768, },
  { 1, 128, 256, 1024, 4096, 32768, },
  { 2, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 128, 256, 1024, 4096, 32768, },
  { 4, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 128, 256, 1024, 4096, 32768, },
  { 8, 128, 256, 1024, 4096, 32768, },
  { 1, 8, 128, 256, 1024, 4096, 32768, },
  { 2, 8, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 128, 256, 1024, 4096, 32768, },
  { 4, 8, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 4096, 32768, },
  { 16, 128, 256, 1024, 4096, 32768, },
  { 1, 16, 128, 256, 1024, 4096, 32768, },
  { 2, 16, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 16, 128, 256, 1024, 4096, 32768, },
  { 4, 16, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 16, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 16, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 4096, 32768, },
  { 8, 16, 128, 256, 1024, 4096, 32768, },
  { 1, 8, 16, 128, 256, 1024, 4096, 32768, },
  { 2, 8, 16, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 4096, 32768, },
  { 4, 8, 16, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 4096, 32768, },
  { 32, 128, 256, 1024, 4096, 32768, },
  { 1, 32, 128, 256, 1024, 4096, 32768, },
  { 2, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 32, 128, 256, 1024, 4096, 32768, },
  { 4, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 32, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 4096, 32768, },
  { 8, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 8, 32, 128, 256, 1024, 4096, 32768, },
  { 2, 8, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 4096, 32768, },
  { 4, 8, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 4096, 32768, },
  { 16, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 2, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 4, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 8, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 32768, },
  { 64, 128, 256, 1024, 4096, 32768, },
  { 1, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 64, 128, 256, 1024, 4096, 32768, },
  { 4, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 4096, 32768, },
  { 8, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 8, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 8, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 4096, 32768, },
  { 4, 8, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 4096, 32768, },
  { 16, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 4, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 8, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 32768, },
  { 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 4, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 8, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 32768, },
  { 512, 1024, 4096, 32768, },
  { 1, 512, 1024, 4096, 32768, },
  { 2, 512, 1024, 4096, 32768, },
  { 1, 2, 512, 1024, 4096, 32768, },
  { 4, 512, 1024, 4096, 32768, },
  { 1, 4, 512, 1024, 4096, 32768, },
  { 2, 4, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 512, 1024, 4096, 32768, },
  { 8, 512, 1024, 4096, 32768, },
  { 1, 8, 512, 1024, 4096, 32768, },
  { 2, 8, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 512, 1024, 4096, 32768, },
  { 4, 8, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 512, 1024, 4096, 32768, },
  { 16, 512, 1024, 4096, 32768, },
  { 1, 16, 512, 1024, 4096, 32768, },
  { 2, 16, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 512, 1024, 4096, 32768, },
  { 4, 16, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 512, 1024, 4096, 32768, },
  { 8, 16, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 4096, 32768, },
  { 32, 512, 1024, 4096, 32768, },
  { 1, 32, 512, 1024, 4096, 32768, },
  { 2, 32, 512, 1024, 4096, 32768, },
  { 1, 2, 32, 512, 1024, 4096, 32768, },
  { 4, 32, 512, 1024, 4096, 32768, },
  { 1, 4, 32, 512, 1024, 4096, 32768, },
  { 2, 4, 32, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 512, 1024, 4096, 32768, },
  { 8, 32, 512, 1024, 4096, 32768, },
  { 1, 8, 32, 512, 1024, 4096, 32768, },
  { 2, 8, 32, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 512, 1024, 4096, 32768, },
  { 4, 8, 32, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 4096, 32768, },
  { 16, 32, 512, 1024, 4096, 32768, },
  { 1, 16, 32, 512, 1024, 4096, 32768, },
  { 2, 16, 32, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 512, 1024, 4096, 32768, },
  { 4, 16, 32, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 4096, 32768, },
  { 8, 16, 32, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 4096, 32768, },
  { 64, 512, 1024, 4096, 32768, },
  { 1, 64, 512, 1024, 4096, 32768, },
  { 2, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 64, 512, 1024, 4096, 32768, },
  { 4, 64, 512, 1024, 4096, 32768, },
  { 1, 4, 64, 512, 1024, 4096, 32768, },
  { 2, 4, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 64, 512, 1024, 4096, 32768, },
  { 8, 64, 512, 1024, 4096, 32768, },
  { 1, 8, 64, 512, 1024, 4096, 32768, },
  { 2, 8, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 64, 512, 1024, 4096, 32768, },
  { 4, 8, 64, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 64, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 4096, 32768, },
  { 16, 64, 512, 1024, 4096, 32768, },
  { 1, 16, 64, 512, 1024, 4096, 32768, },
  { 2, 16, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 64, 512, 1024, 4096, 32768, },
  { 4, 16, 64, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 64, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 4096, 32768, },
  { 8, 16, 64, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 64, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 64, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 4096, 32768, },
  { 32, 64, 512, 1024, 4096, 32768, },
  { 1, 32, 64, 512, 1024, 4096, 32768, },
  { 2, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 32, 64, 512, 1024, 4096, 32768, },
  { 4, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 4, 32, 64, 512, 1024, 4096, 32768, },
  { 2, 4, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 4096, 32768, },
  { 8, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 8, 32, 64, 512, 1024, 4096, 32768, },
  { 2, 8, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 4096, 32768, },
  { 4, 8, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 4096, 32768, },
  { 16, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 2, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 4, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 8, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 32768, },
  { 128, 512, 1024, 4096, 32768, },
  { 1, 128, 512, 1024, 4096, 32768, },
  { 2, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 128, 512, 1024, 4096, 32768, },
  { 4, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 128, 512, 1024, 4096, 32768, },
  { 8, 128, 512, 1024, 4096, 32768, },
  { 1, 8, 128, 512, 1024, 4096, 32768, },
  { 2, 8, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 128, 512, 1024, 4096, 32768, },
  { 4, 8, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 4096, 32768, },
  { 16, 128, 512, 1024, 4096, 32768, },
  { 1, 16, 128, 512, 1024, 4096, 32768, },
  { 2, 16, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 128, 512, 1024, 4096, 32768, },
  { 4, 16, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 4096, 32768, },
  { 8, 16, 128, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 128, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 4096, 32768, },
  { 32, 128, 512, 1024, 4096, 32768, },
  { 1, 32, 128, 512, 1024, 4096, 32768, },
  { 2, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 32, 128, 512, 1024, 4096, 32768, },
  { 4, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 32, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 4096, 32768, },
  { 8, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 8, 32, 128, 512, 1024, 4096, 32768, },
  { 2, 8, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 4096, 32768, },
  { 4, 8, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 4096, 32768, },
  { 16, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 2, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 4, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 8, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 32768, },
  { 64, 128, 512, 1024, 4096, 32768, },
  { 1, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 64, 128, 512, 1024, 4096, 32768, },
  { 4, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 4096, 32768, },
  { 8, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 8, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 8, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 4096, 32768, },
  { 4, 8, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 4096, 32768, },
  { 16, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 4, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 8, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 32768, },
  { 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 4, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 8, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 32768, },
  { 256, 512, 1024, 4096, 32768, },
  { 1, 256, 512, 1024, 4096, 32768, },
  { 2, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 256, 512, 1024, 4096, 32768, },
  { 4, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 256, 512, 1024, 4096, 32768, },
  { 8, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 4096, 32768, },
  { 16, 256, 512, 1024, 4096, 32768, },
  { 1, 16, 256, 512, 1024, 4096, 32768, },
  { 2, 16, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 256, 512, 1024, 4096, 32768, },
  { 4, 16, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 4096, 32768, },
  { 8, 16, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 4096, 32768, },
  { 32, 256, 512, 1024, 4096, 32768, },
  { 1, 32, 256, 512, 1024, 4096, 32768, },
  { 2, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 32, 256, 512, 1024, 4096, 32768, },
  { 4, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 32, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 4096, 32768, },
  { 8, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 32, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 4096, 32768, },
  { 16, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 2, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 4, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 8, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 32768, },
  { 64, 256, 512, 1024, 4096, 32768, },
  { 1, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 64, 256, 512, 1024, 4096, 32768, },
  { 4, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 4096, 32768, },
  { 8, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 4096, 32768, },
  { 16, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 4, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 8, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 32768, },
  { 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 4, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 8, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 32768, },
  { 128, 256, 512, 1024, 4096, 32768, },
  { 1, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 4096, 32768, },
  { 8, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 4096, 32768, },
  { 16, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 8, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 32768, },
  { 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 8, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 32768, },
  { 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 8, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 32768, },
  { 2048, 4096, 32768, },
  { 1, 2048, 4096, 32768, },
  { 2, 2048, 4096, 32768, },
  { 1, 2, 2048, 4096, 32768, },
  { 4, 2048, 4096, 32768, },
  { 1, 4, 2048, 4096, 32768, },
  { 2, 4, 2048, 4096, 32768, },
  { 1, 2, 4, 2048, 4096, 32768, },
  { 8, 2048, 4096, 32768, },
  { 1, 8, 2048, 4096, 32768, },
  { 2, 8, 2048, 4096, 32768, },
  { 1, 2, 8, 2048, 4096, 32768, },
  { 4, 8, 2048, 4096, 32768, },
  { 1, 4, 8, 2048, 4096, 32768, },
  { 2, 4, 8, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 2048, 4096, 32768, },
  { 16, 2048, 4096, 32768, },
  { 1, 16, 2048, 4096, 32768, },
  { 2, 16, 2048, 4096, 32768, },
  { 1, 2, 16, 2048, 4096, 32768, },
  { 4, 16, 2048, 4096, 32768, },
  { 1, 4, 16, 2048, 4096, 32768, },
  { 2, 4, 16, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 2048, 4096, 32768, },
  { 8, 16, 2048, 4096, 32768, },
  { 1, 8, 16, 2048, 4096, 32768, },
  { 2, 8, 16, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 2048, 4096, 32768, },
  { 4, 8, 16, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 2048, 4096, 32768, },
  { 32, 2048, 4096, 32768, },
  { 1, 32, 2048, 4096, 32768, },
  { 2, 32, 2048, 4096, 32768, },
  { 1, 2, 32, 2048, 4096, 32768, },
  { 4, 32, 2048, 4096, 32768, },
  { 1, 4, 32, 2048, 4096, 32768, },
  { 2, 4, 32, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 2048, 4096, 32768, },
  { 8, 32, 2048, 4096, 32768, },
  { 1, 8, 32, 2048, 4096, 32768, },
  { 2, 8, 32, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 2048, 4096, 32768, },
  { 4, 8, 32, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 2048, 4096, 32768, },
  { 16, 32, 2048, 4096, 32768, },
  { 1, 16, 32, 2048, 4096, 32768, },
  { 2, 16, 32, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 2048, 4096, 32768, },
  { 4, 16, 32, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 2048, 4096, 32768, },
  { 8, 16, 32, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 2048, 4096, 32768, },
  { 64, 2048, 4096, 32768, },
  { 1, 64, 2048, 4096, 32768, },
  { 2, 64, 2048, 4096, 32768, },
  { 1, 2, 64, 2048, 4096, 32768, },
  { 4, 64, 2048, 4096, 32768, },
  { 1, 4, 64, 2048, 4096, 32768, },
  { 2, 4, 64, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 2048, 4096, 32768, },
  { 8, 64, 2048, 4096, 32768, },
  { 1, 8, 64, 2048, 4096, 32768, },
  { 2, 8, 64, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 2048, 4096, 32768, },
  { 4, 8, 64, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 2048, 4096, 32768, },
  { 16, 64, 2048, 4096, 32768, },
  { 1, 16, 64, 2048, 4096, 32768, },
  { 2, 16, 64, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 2048, 4096, 32768, },
  { 4, 16, 64, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 2048, 4096, 32768, },
  { 8, 16, 64, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 2048, 4096, 32768, },
  { 32, 64, 2048, 4096, 32768, },
  { 1, 32, 64, 2048, 4096, 32768, },
  { 2, 32, 64, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 2048, 4096, 32768, },
  { 4, 32, 64, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 2048, 4096, 32768, },
  { 8, 32, 64, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 2048, 4096, 32768, },
  { 16, 32, 64, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 4096, 32768, },
  { 128, 2048, 4096, 32768, },
  { 1, 128, 2048, 4096, 32768, },
  { 2, 128, 2048, 4096, 32768, },
  { 1, 2, 128, 2048, 4096, 32768, },
  { 4, 128, 2048, 4096, 32768, },
  { 1, 4, 128, 2048, 4096, 32768, },
  { 2, 4, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 128, 2048, 4096, 32768, },
  { 8, 128, 2048, 4096, 32768, },
  { 1, 8, 128, 2048, 4096, 32768, },
  { 2, 8, 128, 2048, 4096, 32768, },
  { 1, 2, 8, 128, 2048, 4096, 32768, },
  { 4, 8, 128, 2048, 4096, 32768, },
  { 1, 4, 8, 128, 2048, 4096, 32768, },
  { 2, 4, 8, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 128, 2048, 4096, 32768, },
  { 16, 128, 2048, 4096, 32768, },
  { 1, 16, 128, 2048, 4096, 32768, },
  { 2, 16, 128, 2048, 4096, 32768, },
  { 1, 2, 16, 128, 2048, 4096, 32768, },
  { 4, 16, 128, 2048, 4096, 32768, },
  { 1, 4, 16, 128, 2048, 4096, 32768, },
  { 2, 4, 16, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 128, 2048, 4096, 32768, },
  { 8, 16, 128, 2048, 4096, 32768, },
  { 1, 8, 16, 128, 2048, 4096, 32768, },
  { 2, 8, 16, 128, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 128, 2048, 4096, 32768, },
  { 4, 8, 16, 128, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 128, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 2048, 4096, 32768, },
  { 32, 128, 2048, 4096, 32768, },
  { 1, 32, 128, 2048, 4096, 32768, },
  { 2, 32, 128, 2048, 4096, 32768, },
  { 1, 2, 32, 128, 2048, 4096, 32768, },
  { 4, 32, 128, 2048, 4096, 32768, },
  { 1, 4, 32, 128, 2048, 4096, 32768, },
  { 2, 4, 32, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 128, 2048, 4096, 32768, },
  { 8, 32, 128, 2048, 4096, 32768, },
  { 1, 8, 32, 128, 2048, 4096, 32768, },
  { 2, 8, 32, 128, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 128, 2048, 4096, 32768, },
  { 4, 8, 32, 128, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 128, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 2048, 4096, 32768, },
  { 16, 32, 128, 2048, 4096, 32768, },
  { 1, 16, 32, 128, 2048, 4096, 32768, },
  { 2, 16, 32, 128, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 128, 2048, 4096, 32768, },
  { 4, 16, 32, 128, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 128, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 2048, 4096, 32768, },
  { 8, 16, 32, 128, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 128, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 128, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 128, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 4096, 32768, },
  { 64, 128, 2048, 4096, 32768, },
  { 1, 64, 128, 2048, 4096, 32768, },
  { 2, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 64, 128, 2048, 4096, 32768, },
  { 4, 64, 128, 2048, 4096, 32768, },
  { 1, 4, 64, 128, 2048, 4096, 32768, },
  { 2, 4, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 128, 2048, 4096, 32768, },
  { 8, 64, 128, 2048, 4096, 32768, },
  { 1, 8, 64, 128, 2048, 4096, 32768, },
  { 2, 8, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 128, 2048, 4096, 32768, },
  { 4, 8, 64, 128, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 128, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 2048, 4096, 32768, },
  { 16, 64, 128, 2048, 4096, 32768, },
  { 1, 16, 64, 128, 2048, 4096, 32768, },
  { 2, 16, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 128, 2048, 4096, 32768, },
  { 4, 16, 64, 128, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 128, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 2048, 4096, 32768, },
  { 8, 16, 64, 128, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 128, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 128, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 4096, 32768, },
  { 32, 64, 128, 2048, 4096, 32768, },
  { 1, 32, 64, 128, 2048, 4096, 32768, },
  { 2, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 128, 2048, 4096, 32768, },
  { 4, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 128, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 2048, 4096, 32768, },
  { 8, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 128, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 4096, 32768, },
  { 16, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 32768, },
  { 256, 2048, 4096, 32768, },
  { 1, 256, 2048, 4096, 32768, },
  { 2, 256, 2048, 4096, 32768, },
  { 1, 2, 256, 2048, 4096, 32768, },
  { 4, 256, 2048, 4096, 32768, },
  { 1, 4, 256, 2048, 4096, 32768, },
  { 2, 4, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 256, 2048, 4096, 32768, },
  { 8, 256, 2048, 4096, 32768, },
  { 1, 8, 256, 2048, 4096, 32768, },
  { 2, 8, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 256, 2048, 4096, 32768, },
  { 4, 8, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 256, 2048, 4096, 32768, },
  { 16, 256, 2048, 4096, 32768, },
  { 1, 16, 256, 2048, 4096, 32768, },
  { 2, 16, 256, 2048, 4096, 32768, },
  { 1, 2, 16, 256, 2048, 4096, 32768, },
  { 4, 16, 256, 2048, 4096, 32768, },
  { 1, 4, 16, 256, 2048, 4096, 32768, },
  { 2, 4, 16, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 256, 2048, 4096, 32768, },
  { 8, 16, 256, 2048, 4096, 32768, },
  { 1, 8, 16, 256, 2048, 4096, 32768, },
  { 2, 8, 16, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 256, 2048, 4096, 32768, },
  { 4, 8, 16, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 256, 2048, 4096, 32768, },
  { 32, 256, 2048, 4096, 32768, },
  { 1, 32, 256, 2048, 4096, 32768, },
  { 2, 32, 256, 2048, 4096, 32768, },
  { 1, 2, 32, 256, 2048, 4096, 32768, },
  { 4, 32, 256, 2048, 4096, 32768, },
  { 1, 4, 32, 256, 2048, 4096, 32768, },
  { 2, 4, 32, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 256, 2048, 4096, 32768, },
  { 8, 32, 256, 2048, 4096, 32768, },
  { 1, 8, 32, 256, 2048, 4096, 32768, },
  { 2, 8, 32, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 256, 2048, 4096, 32768, },
  { 4, 8, 32, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 256, 2048, 4096, 32768, },
  { 16, 32, 256, 2048, 4096, 32768, },
  { 1, 16, 32, 256, 2048, 4096, 32768, },
  { 2, 16, 32, 256, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 256, 2048, 4096, 32768, },
  { 4, 16, 32, 256, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 256, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 256, 2048, 4096, 32768, },
  { 8, 16, 32, 256, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 256, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 256, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 4096, 32768, },
  { 64, 256, 2048, 4096, 32768, },
  { 1, 64, 256, 2048, 4096, 32768, },
  { 2, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 64, 256, 2048, 4096, 32768, },
  { 4, 64, 256, 2048, 4096, 32768, },
  { 1, 4, 64, 256, 2048, 4096, 32768, },
  { 2, 4, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 256, 2048, 4096, 32768, },
  { 8, 64, 256, 2048, 4096, 32768, },
  { 1, 8, 64, 256, 2048, 4096, 32768, },
  { 2, 8, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 256, 2048, 4096, 32768, },
  { 4, 8, 64, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 256, 2048, 4096, 32768, },
  { 16, 64, 256, 2048, 4096, 32768, },
  { 1, 16, 64, 256, 2048, 4096, 32768, },
  { 2, 16, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 256, 2048, 4096, 32768, },
  { 4, 16, 64, 256, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 256, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 256, 2048, 4096, 32768, },
  { 8, 16, 64, 256, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 256, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 256, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 4096, 32768, },
  { 32, 64, 256, 2048, 4096, 32768, },
  { 1, 32, 64, 256, 2048, 4096, 32768, },
  { 2, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 256, 2048, 4096, 32768, },
  { 4, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 256, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 256, 2048, 4096, 32768, },
  { 8, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 256, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 256, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 4096, 32768, },
  { 16, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 32768, },
  { 128, 256, 2048, 4096, 32768, },
  { 1, 128, 256, 2048, 4096, 32768, },
  { 2, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 128, 256, 2048, 4096, 32768, },
  { 4, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 128, 256, 2048, 4096, 32768, },
  { 8, 128, 256, 2048, 4096, 32768, },
  { 1, 8, 128, 256, 2048, 4096, 32768, },
  { 2, 8, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 128, 256, 2048, 4096, 32768, },
  { 4, 8, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 128, 256, 2048, 4096, 32768, },
  { 16, 128, 256, 2048, 4096, 32768, },
  { 1, 16, 128, 256, 2048, 4096, 32768, },
  { 2, 16, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 16, 128, 256, 2048, 4096, 32768, },
  { 4, 16, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 16, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 16, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 128, 256, 2048, 4096, 32768, },
  { 8, 16, 128, 256, 2048, 4096, 32768, },
  { 1, 8, 16, 128, 256, 2048, 4096, 32768, },
  { 2, 8, 16, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 128, 256, 2048, 4096, 32768, },
  { 4, 8, 16, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 4096, 32768, },
  { 32, 128, 256, 2048, 4096, 32768, },
  { 1, 32, 128, 256, 2048, 4096, 32768, },
  { 2, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 32, 128, 256, 2048, 4096, 32768, },
  { 4, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 32, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 128, 256, 2048, 4096, 32768, },
  { 8, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 8, 32, 128, 256, 2048, 4096, 32768, },
  { 2, 8, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 128, 256, 2048, 4096, 32768, },
  { 4, 8, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 4096, 32768, },
  { 16, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 2, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 4, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 8, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 32768, },
  { 64, 128, 256, 2048, 4096, 32768, },
  { 1, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 64, 128, 256, 2048, 4096, 32768, },
  { 4, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 128, 256, 2048, 4096, 32768, },
  { 8, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 8, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 8, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 128, 256, 2048, 4096, 32768, },
  { 4, 8, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 4096, 32768, },
  { 16, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 4, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 8, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 32768, },
  { 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 4, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 8, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 32768, },
  { 512, 2048, 4096, 32768, },
  { 1, 512, 2048, 4096, 32768, },
  { 2, 512, 2048, 4096, 32768, },
  { 1, 2, 512, 2048, 4096, 32768, },
  { 4, 512, 2048, 4096, 32768, },
  { 1, 4, 512, 2048, 4096, 32768, },
  { 2, 4, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 512, 2048, 4096, 32768, },
  { 8, 512, 2048, 4096, 32768, },
  { 1, 8, 512, 2048, 4096, 32768, },
  { 2, 8, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 512, 2048, 4096, 32768, },
  { 4, 8, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 512, 2048, 4096, 32768, },
  { 16, 512, 2048, 4096, 32768, },
  { 1, 16, 512, 2048, 4096, 32768, },
  { 2, 16, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 512, 2048, 4096, 32768, },
  { 4, 16, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 512, 2048, 4096, 32768, },
  { 8, 16, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 512, 2048, 4096, 32768, },
  { 32, 512, 2048, 4096, 32768, },
  { 1, 32, 512, 2048, 4096, 32768, },
  { 2, 32, 512, 2048, 4096, 32768, },
  { 1, 2, 32, 512, 2048, 4096, 32768, },
  { 4, 32, 512, 2048, 4096, 32768, },
  { 1, 4, 32, 512, 2048, 4096, 32768, },
  { 2, 4, 32, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 512, 2048, 4096, 32768, },
  { 8, 32, 512, 2048, 4096, 32768, },
  { 1, 8, 32, 512, 2048, 4096, 32768, },
  { 2, 8, 32, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 512, 2048, 4096, 32768, },
  { 4, 8, 32, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 512, 2048, 4096, 32768, },
  { 16, 32, 512, 2048, 4096, 32768, },
  { 1, 16, 32, 512, 2048, 4096, 32768, },
  { 2, 16, 32, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 512, 2048, 4096, 32768, },
  { 4, 16, 32, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 512, 2048, 4096, 32768, },
  { 8, 16, 32, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 4096, 32768, },
  { 64, 512, 2048, 4096, 32768, },
  { 1, 64, 512, 2048, 4096, 32768, },
  { 2, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 64, 512, 2048, 4096, 32768, },
  { 4, 64, 512, 2048, 4096, 32768, },
  { 1, 4, 64, 512, 2048, 4096, 32768, },
  { 2, 4, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 512, 2048, 4096, 32768, },
  { 8, 64, 512, 2048, 4096, 32768, },
  { 1, 8, 64, 512, 2048, 4096, 32768, },
  { 2, 8, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 512, 2048, 4096, 32768, },
  { 4, 8, 64, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 512, 2048, 4096, 32768, },
  { 16, 64, 512, 2048, 4096, 32768, },
  { 1, 16, 64, 512, 2048, 4096, 32768, },
  { 2, 16, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 512, 2048, 4096, 32768, },
  { 4, 16, 64, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 512, 2048, 4096, 32768, },
  { 8, 16, 64, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 4096, 32768, },
  { 32, 64, 512, 2048, 4096, 32768, },
  { 1, 32, 64, 512, 2048, 4096, 32768, },
  { 2, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 512, 2048, 4096, 32768, },
  { 4, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 512, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 512, 2048, 4096, 32768, },
  { 8, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 512, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 512, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 4096, 32768, },
  { 16, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 32768, },
  { 128, 512, 2048, 4096, 32768, },
  { 1, 128, 512, 2048, 4096, 32768, },
  { 2, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 128, 512, 2048, 4096, 32768, },
  { 4, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 128, 512, 2048, 4096, 32768, },
  { 8, 128, 512, 2048, 4096, 32768, },
  { 1, 8, 128, 512, 2048, 4096, 32768, },
  { 2, 8, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 128, 512, 2048, 4096, 32768, },
  { 4, 8, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 128, 512, 2048, 4096, 32768, },
  { 16, 128, 512, 2048, 4096, 32768, },
  { 1, 16, 128, 512, 2048, 4096, 32768, },
  { 2, 16, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 128, 512, 2048, 4096, 32768, },
  { 4, 16, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 128, 512, 2048, 4096, 32768, },
  { 8, 16, 128, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 128, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 128, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 4096, 32768, },
  { 32, 128, 512, 2048, 4096, 32768, },
  { 1, 32, 128, 512, 2048, 4096, 32768, },
  { 2, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 32, 128, 512, 2048, 4096, 32768, },
  { 4, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 32, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 128, 512, 2048, 4096, 32768, },
  { 8, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 8, 32, 128, 512, 2048, 4096, 32768, },
  { 2, 8, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 128, 512, 2048, 4096, 32768, },
  { 4, 8, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 4096, 32768, },
  { 16, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 2, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 4, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 8, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 32768, },
  { 64, 128, 512, 2048, 4096, 32768, },
  { 1, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 64, 128, 512, 2048, 4096, 32768, },
  { 4, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 128, 512, 2048, 4096, 32768, },
  { 8, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 8, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 8, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 128, 512, 2048, 4096, 32768, },
  { 4, 8, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 4096, 32768, },
  { 16, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 4, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 8, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 32768, },
  { 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 4, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 8, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 32768, },
  { 256, 512, 2048, 4096, 32768, },
  { 1, 256, 512, 2048, 4096, 32768, },
  { 2, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 256, 512, 2048, 4096, 32768, },
  { 4, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 256, 512, 2048, 4096, 32768, },
  { 8, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 256, 512, 2048, 4096, 32768, },
  { 16, 256, 512, 2048, 4096, 32768, },
  { 1, 16, 256, 512, 2048, 4096, 32768, },
  { 2, 16, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 256, 512, 2048, 4096, 32768, },
  { 4, 16, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 256, 512, 2048, 4096, 32768, },
  { 8, 16, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 4096, 32768, },
  { 32, 256, 512, 2048, 4096, 32768, },
  { 1, 32, 256, 512, 2048, 4096, 32768, },
  { 2, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 32, 256, 512, 2048, 4096, 32768, },
  { 4, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 32, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 256, 512, 2048, 4096, 32768, },
  { 8, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 32, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 4096, 32768, },
  { 16, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 2, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 4, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 8, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 32768, },
  { 64, 256, 512, 2048, 4096, 32768, },
  { 1, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 64, 256, 512, 2048, 4096, 32768, },
  { 4, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 256, 512, 2048, 4096, 32768, },
  { 8, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 4096, 32768, },
  { 16, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 4, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 8, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 32768, },
  { 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 4, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 8, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 32768, },
  { 128, 256, 512, 2048, 4096, 32768, },
  { 1, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 128, 256, 512, 2048, 4096, 32768, },
  { 8, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 4096, 32768, },
  { 16, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 8, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 32768, },
  { 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 8, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 8, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 32768, },
  { 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 8, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 8, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 8, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 32768, },
  { 1024, 2048, 4096, 32768, },
  { 1, 1024, 2048, 4096, 32768, },
  { 2, 1024, 2048, 4096, 32768, },
  { 1, 2, 1024, 2048, 4096, 32768, },
  { 4, 1024, 2048, 4096, 32768, },
  { 1, 4, 1024, 2048, 4096, 32768, },
  { 2, 4, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 1024, 2048, 4096, 32768, },
  { 8, 1024, 2048, 4096, 32768, },
  { 1, 8, 1024, 2048, 4096, 32768, },
  { 2, 8, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 1024, 2048, 4096, 32768, },
  { 4, 8, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 1024, 2048, 4096, 32768, },
  { 16, 1024, 2048, 4096, 32768, },
  { 1, 16, 1024, 2048, 4096, 32768, },
  { 2, 16, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 1024, 2048, 4096, 32768, },
  { 4, 16, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 1024, 2048, 4096, 32768, },
  { 8, 16, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 1024, 2048, 4096, 32768, },
  { 32, 1024, 2048, 4096, 32768, },
  { 1, 32, 1024, 2048, 4096, 32768, },
  { 2, 32, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 1024, 2048, 4096, 32768, },
  { 4, 32, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 1024, 2048, 4096, 32768, },
  { 8, 32, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 1024, 2048, 4096, 32768, },
  { 16, 32, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 4096, 32768, },
  { 64, 1024, 2048, 4096, 32768, },
  { 1, 64, 1024, 2048, 4096, 32768, },
  { 2, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 64, 1024, 2048, 4096, 32768, },
  { 4, 64, 1024, 2048, 4096, 32768, },
  { 1, 4, 64, 1024, 2048, 4096, 32768, },
  { 2, 4, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 1024, 2048, 4096, 32768, },
  { 8, 64, 1024, 2048, 4096, 32768, },
  { 1, 8, 64, 1024, 2048, 4096, 32768, },
  { 2, 8, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 1024, 2048, 4096, 32768, },
  { 4, 8, 64, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 1024, 2048, 4096, 32768, },
  { 16, 64, 1024, 2048, 4096, 32768, },
  { 1, 16, 64, 1024, 2048, 4096, 32768, },
  { 2, 16, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 1024, 2048, 4096, 32768, },
  { 4, 16, 64, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 1024, 2048, 4096, 32768, },
  { 8, 16, 64, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 4096, 32768, },
  { 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 32, 64, 1024, 2048, 4096, 32768, },
  { 2, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 1024, 2048, 4096, 32768, },
  { 4, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 1024, 2048, 4096, 32768, },
  { 8, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 4096, 32768, },
  { 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 32768, },
  { 128, 1024, 2048, 4096, 32768, },
  { 1, 128, 1024, 2048, 4096, 32768, },
  { 2, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 128, 1024, 2048, 4096, 32768, },
  { 4, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 128, 1024, 2048, 4096, 32768, },
  { 8, 128, 1024, 2048, 4096, 32768, },
  { 1, 8, 128, 1024, 2048, 4096, 32768, },
  { 2, 8, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 128, 1024, 2048, 4096, 32768, },
  { 4, 8, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 128, 1024, 2048, 4096, 32768, },
  { 16, 128, 1024, 2048, 4096, 32768, },
  { 1, 16, 128, 1024, 2048, 4096, 32768, },
  { 2, 16, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 128, 1024, 2048, 4096, 32768, },
  { 4, 16, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 128, 1024, 2048, 4096, 32768, },
  { 8, 16, 128, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 128, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 128, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 4096, 32768, },
  { 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 32, 128, 1024, 2048, 4096, 32768, },
  { 2, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 128, 1024, 2048, 4096, 32768, },
  { 4, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 128, 1024, 2048, 4096, 32768, },
  { 8, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 128, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 128, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 4096, 32768, },
  { 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 32768, },
  { 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 64, 128, 1024, 2048, 4096, 32768, },
  { 4, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 128, 1024, 2048, 4096, 32768, },
  { 8, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 8, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 8, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 128, 1024, 2048, 4096, 32768, },
  { 4, 8, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 4096, 32768, },
  { 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 4, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 8, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 32768, },
  { 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 4, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 8, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 32768, },
  { 256, 1024, 2048, 4096, 32768, },
  { 1, 256, 1024, 2048, 4096, 32768, },
  { 2, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 256, 1024, 2048, 4096, 32768, },
  { 4, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 256, 1024, 2048, 4096, 32768, },
  { 8, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 256, 1024, 2048, 4096, 32768, },
  { 16, 256, 1024, 2048, 4096, 32768, },
  { 1, 16, 256, 1024, 2048, 4096, 32768, },
  { 2, 16, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 256, 1024, 2048, 4096, 32768, },
  { 4, 16, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 256, 1024, 2048, 4096, 32768, },
  { 8, 16, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 4096, 32768, },
  { 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 32, 256, 1024, 2048, 4096, 32768, },
  { 2, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 256, 1024, 2048, 4096, 32768, },
  { 4, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 256, 1024, 2048, 4096, 32768, },
  { 8, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 4096, 32768, },
  { 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 32768, },
  { 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 64, 256, 1024, 2048, 4096, 32768, },
  { 4, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 256, 1024, 2048, 4096, 32768, },
  { 8, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 4096, 32768, },
  { 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 4, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 8, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 32768, },
  { 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 4, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 8, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 32768, },
  { 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 128, 256, 1024, 2048, 4096, 32768, },
  { 8, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 4096, 32768, },
  { 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 8, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 32768, },
  { 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 8, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 32768, },
  { 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 8, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 8, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 8, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 32768, },
  { 512, 1024, 2048, 4096, 32768, },
  { 1, 512, 1024, 2048, 4096, 32768, },
  { 2, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 512, 1024, 2048, 4096, 32768, },
  { 4, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 512, 1024, 2048, 4096, 32768, },
  { 8, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 512, 1024, 2048, 4096, 32768, },
  { 16, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 4096, 32768, },
  { 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 32, 512, 1024, 2048, 4096, 32768, },
  { 2, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 512, 1024, 2048, 4096, 32768, },
  { 4, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 512, 1024, 2048, 4096, 32768, },
  { 8, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 4096, 32768, },
  { 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 32768, },
  { 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 64, 512, 1024, 2048, 4096, 32768, },
  { 4, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 512, 1024, 2048, 4096, 32768, },
  { 8, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 4096, 32768, },
  { 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 32768, },
  { 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 4, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 8, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 32768, },
  { 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 128, 512, 1024, 2048, 4096, 32768, },
  { 8, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 4096, 32768, },
  { 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 32768, },
  { 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 8, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 32768, },
  { 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 8, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 8, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 32768, },
  { 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 4096, 32768, },
  { 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 32768, },
  { 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 32768, },
  { 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 32768, },
  { 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 32768, },
  { 8192, 32768, },
  { 1, 8192, 32768, },
  { 2, 8192, 32768, },
  { 1, 2, 8192, 32768, },
  { 4, 8192, 32768, },
  { 1, 4, 8192, 32768, },
  { 2, 4, 8192, 32768, },
  { 1, 2, 4, 8192, 32768, },
  { 8, 8192, 32768, },
  { 1, 8, 8192, 32768, },
  { 2, 8, 8192, 32768, },
  { 1, 2, 8, 8192, 32768, },
  { 4, 8, 8192, 32768, },
  { 1, 4, 8, 8192, 32768, },
  { 2, 4, 8, 8192, 32768, },
  { 1, 2, 4, 8, 8192, 32768, },
  { 16, 8192, 32768, },
  { 1, 16, 8192, 32768, },
  { 2, 16, 8192, 32768, },
  { 1, 2, 16, 8192, 32768, },
  { 4, 16, 8192, 32768, },
  { 1, 4, 16, 8192, 32768, },
  { 2, 4, 16, 8192, 32768, },
  { 1, 2, 4, 16, 8192, 32768, },
  { 8, 16, 8192, 32768, },
  { 1, 8, 16, 8192, 32768, },
  { 2, 8, 16, 8192, 32768, },
  { 1, 2, 8, 16, 8192, 32768, },
  { 4, 8, 16, 8192, 32768, },
  { 1, 4, 8, 16, 8192, 32768, },
  { 2, 4, 8, 16, 8192, 32768, },
  { 1, 2, 4, 8, 16, 8192, 32768, },
  { 32, 8192, 32768, },
  { 1, 32, 8192, 32768, },
  { 2, 32, 8192, 32768, },
  { 1, 2, 32, 8192, 32768, },
  { 4, 32, 8192, 32768, },
  { 1, 4, 32, 8192, 32768, },
  { 2, 4, 32, 8192, 32768, },
  { 1, 2, 4, 32, 8192, 32768, },
  { 8, 32, 8192, 32768, },
  { 1, 8, 32, 8192, 32768, },
  { 2, 8, 32, 8192, 32768, },
  { 1, 2, 8, 32, 8192, 32768, },
  { 4, 8, 32, 8192, 32768, },
  { 1, 4, 8, 32, 8192, 32768, },
  { 2, 4, 8, 32, 8192, 32768, },
  { 1, 2, 4, 8, 32, 8192, 32768, },
  { 16, 32, 8192, 32768, },
  { 1, 16, 32, 8192, 32768, },
  { 2, 16, 32, 8192, 32768, },
  { 1, 2, 16, 32, 8192, 32768, },
  { 4, 16, 32, 8192, 32768, },
  { 1, 4, 16, 32, 8192, 32768, },
  { 2, 4, 16, 32, 8192, 32768, },
  { 1, 2, 4, 16, 32, 8192, 32768, },
  { 8, 16, 32, 8192, 32768, },
  { 1, 8, 16, 32, 8192, 32768, },
  { 2, 8, 16, 32, 8192, 32768, },
  { 1, 2, 8, 16, 32, 8192, 32768, },
  { 4, 8, 16, 32, 8192, 32768, },
  { 1, 4, 8, 16, 32, 8192, 32768, },
  { 2, 4, 8, 16, 32, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 8192, 32768, },
  { 64, 8192, 32768, },
  { 1, 64, 8192, 32768, },
  { 2, 64, 8192, 32768, },
  { 1, 2, 64, 8192, 32768, },
  { 4, 64, 8192, 32768, },
  { 1, 4, 64, 8192, 32768, },
  { 2, 4, 64, 8192, 32768, },
  { 1, 2, 4, 64, 8192, 32768, },
  { 8, 64, 8192, 32768, },
  { 1, 8, 64, 8192, 32768, },
  { 2, 8, 64, 8192, 32768, },
  { 1, 2, 8, 64, 8192, 32768, },
  { 4, 8, 64, 8192, 32768, },
  { 1, 4, 8, 64, 8192, 32768, },
  { 2, 4, 8, 64, 8192, 32768, },
  { 1, 2, 4, 8, 64, 8192, 32768, },
  { 16, 64, 8192, 32768, },
  { 1, 16, 64, 8192, 32768, },
  { 2, 16, 64, 8192, 32768, },
  { 1, 2, 16, 64, 8192, 32768, },
  { 4, 16, 64, 8192, 32768, },
  { 1, 4, 16, 64, 8192, 32768, },
  { 2, 4, 16, 64, 8192, 32768, },
  { 1, 2, 4, 16, 64, 8192, 32768, },
  { 8, 16, 64, 8192, 32768, },
  { 1, 8, 16, 64, 8192, 32768, },
  { 2, 8, 16, 64, 8192, 32768, },
  { 1, 2, 8, 16, 64, 8192, 32768, },
  { 4, 8, 16, 64, 8192, 32768, },
  { 1, 4, 8, 16, 64, 8192, 32768, },
  { 2, 4, 8, 16, 64, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 8192, 32768, },
  { 32, 64, 8192, 32768, },
  { 1, 32, 64, 8192, 32768, },
  { 2, 32, 64, 8192, 32768, },
  { 1, 2, 32, 64, 8192, 32768, },
  { 4, 32, 64, 8192, 32768, },
  { 1, 4, 32, 64, 8192, 32768, },
  { 2, 4, 32, 64, 8192, 32768, },
  { 1, 2, 4, 32, 64, 8192, 32768, },
  { 8, 32, 64, 8192, 32768, },
  { 1, 8, 32, 64, 8192, 32768, },
  { 2, 8, 32, 64, 8192, 32768, },
  { 1, 2, 8, 32, 64, 8192, 32768, },
  { 4, 8, 32, 64, 8192, 32768, },
  { 1, 4, 8, 32, 64, 8192, 32768, },
  { 2, 4, 8, 32, 64, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 8192, 32768, },
  { 16, 32, 64, 8192, 32768, },
  { 1, 16, 32, 64, 8192, 32768, },
  { 2, 16, 32, 64, 8192, 32768, },
  { 1, 2, 16, 32, 64, 8192, 32768, },
  { 4, 16, 32, 64, 8192, 32768, },
  { 1, 4, 16, 32, 64, 8192, 32768, },
  { 2, 4, 16, 32, 64, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 8192, 32768, },
  { 8, 16, 32, 64, 8192, 32768, },
  { 1, 8, 16, 32, 64, 8192, 32768, },
  { 2, 8, 16, 32, 64, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 8192, 32768, },
  { 4, 8, 16, 32, 64, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 8192, 32768, },
  { 128, 8192, 32768, },
  { 1, 128, 8192, 32768, },
  { 2, 128, 8192, 32768, },
  { 1, 2, 128, 8192, 32768, },
  { 4, 128, 8192, 32768, },
  { 1, 4, 128, 8192, 32768, },
  { 2, 4, 128, 8192, 32768, },
  { 1, 2, 4, 128, 8192, 32768, },
  { 8, 128, 8192, 32768, },
  { 1, 8, 128, 8192, 32768, },
  { 2, 8, 128, 8192, 32768, },
  { 1, 2, 8, 128, 8192, 32768, },
  { 4, 8, 128, 8192, 32768, },
  { 1, 4, 8, 128, 8192, 32768, },
  { 2, 4, 8, 128, 8192, 32768, },
  { 1, 2, 4, 8, 128, 8192, 32768, },
  { 16, 128, 8192, 32768, },
  { 1, 16, 128, 8192, 32768, },
  { 2, 16, 128, 8192, 32768, },
  { 1, 2, 16, 128, 8192, 32768, },
  { 4, 16, 128, 8192, 32768, },
  { 1, 4, 16, 128, 8192, 32768, },
  { 2, 4, 16, 128, 8192, 32768, },
  { 1, 2, 4, 16, 128, 8192, 32768, },
  { 8, 16, 128, 8192, 32768, },
  { 1, 8, 16, 128, 8192, 32768, },
  { 2, 8, 16, 128, 8192, 32768, },
  { 1, 2, 8, 16, 128, 8192, 32768, },
  { 4, 8, 16, 128, 8192, 32768, },
  { 1, 4, 8, 16, 128, 8192, 32768, },
  { 2, 4, 8, 16, 128, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 8192, 32768, },
  { 32, 128, 8192, 32768, },
  { 1, 32, 128, 8192, 32768, },
  { 2, 32, 128, 8192, 32768, },
  { 1, 2, 32, 128, 8192, 32768, },
  { 4, 32, 128, 8192, 32768, },
  { 1, 4, 32, 128, 8192, 32768, },
  { 2, 4, 32, 128, 8192, 32768, },
  { 1, 2, 4, 32, 128, 8192, 32768, },
  { 8, 32, 128, 8192, 32768, },
  { 1, 8, 32, 128, 8192, 32768, },
  { 2, 8, 32, 128, 8192, 32768, },
  { 1, 2, 8, 32, 128, 8192, 32768, },
  { 4, 8, 32, 128, 8192, 32768, },
  { 1, 4, 8, 32, 128, 8192, 32768, },
  { 2, 4, 8, 32, 128, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 8192, 32768, },
  { 16, 32, 128, 8192, 32768, },
  { 1, 16, 32, 128, 8192, 32768, },
  { 2, 16, 32, 128, 8192, 32768, },
  { 1, 2, 16, 32, 128, 8192, 32768, },
  { 4, 16, 32, 128, 8192, 32768, },
  { 1, 4, 16, 32, 128, 8192, 32768, },
  { 2, 4, 16, 32, 128, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 8192, 32768, },
  { 8, 16, 32, 128, 8192, 32768, },
  { 1, 8, 16, 32, 128, 8192, 32768, },
  { 2, 8, 16, 32, 128, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 8192, 32768, },
  { 4, 8, 16, 32, 128, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 8192, 32768, },
  { 64, 128, 8192, 32768, },
  { 1, 64, 128, 8192, 32768, },
  { 2, 64, 128, 8192, 32768, },
  { 1, 2, 64, 128, 8192, 32768, },
  { 4, 64, 128, 8192, 32768, },
  { 1, 4, 64, 128, 8192, 32768, },
  { 2, 4, 64, 128, 8192, 32768, },
  { 1, 2, 4, 64, 128, 8192, 32768, },
  { 8, 64, 128, 8192, 32768, },
  { 1, 8, 64, 128, 8192, 32768, },
  { 2, 8, 64, 128, 8192, 32768, },
  { 1, 2, 8, 64, 128, 8192, 32768, },
  { 4, 8, 64, 128, 8192, 32768, },
  { 1, 4, 8, 64, 128, 8192, 32768, },
  { 2, 4, 8, 64, 128, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 8192, 32768, },
  { 16, 64, 128, 8192, 32768, },
  { 1, 16, 64, 128, 8192, 32768, },
  { 2, 16, 64, 128, 8192, 32768, },
  { 1, 2, 16, 64, 128, 8192, 32768, },
  { 4, 16, 64, 128, 8192, 32768, },
  { 1, 4, 16, 64, 128, 8192, 32768, },
  { 2, 4, 16, 64, 128, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 8192, 32768, },
  { 8, 16, 64, 128, 8192, 32768, },
  { 1, 8, 16, 64, 128, 8192, 32768, },
  { 2, 8, 16, 64, 128, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 8192, 32768, },
  { 4, 8, 16, 64, 128, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 8192, 32768, },
  { 32, 64, 128, 8192, 32768, },
  { 1, 32, 64, 128, 8192, 32768, },
  { 2, 32, 64, 128, 8192, 32768, },
  { 1, 2, 32, 64, 128, 8192, 32768, },
  { 4, 32, 64, 128, 8192, 32768, },
  { 1, 4, 32, 64, 128, 8192, 32768, },
  { 2, 4, 32, 64, 128, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 8192, 32768, },
  { 8, 32, 64, 128, 8192, 32768, },
  { 1, 8, 32, 64, 128, 8192, 32768, },
  { 2, 8, 32, 64, 128, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 8192, 32768, },
  { 4, 8, 32, 64, 128, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 8192, 32768, },
  { 16, 32, 64, 128, 8192, 32768, },
  { 1, 16, 32, 64, 128, 8192, 32768, },
  { 2, 16, 32, 64, 128, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 8192, 32768, },
  { 4, 16, 32, 64, 128, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 8192, 32768, },
  { 8, 16, 32, 64, 128, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 8192, 32768, },
  { 256, 8192, 32768, },
  { 1, 256, 8192, 32768, },
  { 2, 256, 8192, 32768, },
  { 1, 2, 256, 8192, 32768, },
  { 4, 256, 8192, 32768, },
  { 1, 4, 256, 8192, 32768, },
  { 2, 4, 256, 8192, 32768, },
  { 1, 2, 4, 256, 8192, 32768, },
  { 8, 256, 8192, 32768, },
  { 1, 8, 256, 8192, 32768, },
  { 2, 8, 256, 8192, 32768, },
  { 1, 2, 8, 256, 8192, 32768, },
  { 4, 8, 256, 8192, 32768, },
  { 1, 4, 8, 256, 8192, 32768, },
  { 2, 4, 8, 256, 8192, 32768, },
  { 1, 2, 4, 8, 256, 8192, 32768, },
  { 16, 256, 8192, 32768, },
  { 1, 16, 256, 8192, 32768, },
  { 2, 16, 256, 8192, 32768, },
  { 1, 2, 16, 256, 8192, 32768, },
  { 4, 16, 256, 8192, 32768, },
  { 1, 4, 16, 256, 8192, 32768, },
  { 2, 4, 16, 256, 8192, 32768, },
  { 1, 2, 4, 16, 256, 8192, 32768, },
  { 8, 16, 256, 8192, 32768, },
  { 1, 8, 16, 256, 8192, 32768, },
  { 2, 8, 16, 256, 8192, 32768, },
  { 1, 2, 8, 16, 256, 8192, 32768, },
  { 4, 8, 16, 256, 8192, 32768, },
  { 1, 4, 8, 16, 256, 8192, 32768, },
  { 2, 4, 8, 16, 256, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 8192, 32768, },
  { 32, 256, 8192, 32768, },
  { 1, 32, 256, 8192, 32768, },
  { 2, 32, 256, 8192, 32768, },
  { 1, 2, 32, 256, 8192, 32768, },
  { 4, 32, 256, 8192, 32768, },
  { 1, 4, 32, 256, 8192, 32768, },
  { 2, 4, 32, 256, 8192, 32768, },
  { 1, 2, 4, 32, 256, 8192, 32768, },
  { 8, 32, 256, 8192, 32768, },
  { 1, 8, 32, 256, 8192, 32768, },
  { 2, 8, 32, 256, 8192, 32768, },
  { 1, 2, 8, 32, 256, 8192, 32768, },
  { 4, 8, 32, 256, 8192, 32768, },
  { 1, 4, 8, 32, 256, 8192, 32768, },
  { 2, 4, 8, 32, 256, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 8192, 32768, },
  { 16, 32, 256, 8192, 32768, },
  { 1, 16, 32, 256, 8192, 32768, },
  { 2, 16, 32, 256, 8192, 32768, },
  { 1, 2, 16, 32, 256, 8192, 32768, },
  { 4, 16, 32, 256, 8192, 32768, },
  { 1, 4, 16, 32, 256, 8192, 32768, },
  { 2, 4, 16, 32, 256, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 8192, 32768, },
  { 8, 16, 32, 256, 8192, 32768, },
  { 1, 8, 16, 32, 256, 8192, 32768, },
  { 2, 8, 16, 32, 256, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 8192, 32768, },
  { 4, 8, 16, 32, 256, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 8192, 32768, },
  { 64, 256, 8192, 32768, },
  { 1, 64, 256, 8192, 32768, },
  { 2, 64, 256, 8192, 32768, },
  { 1, 2, 64, 256, 8192, 32768, },
  { 4, 64, 256, 8192, 32768, },
  { 1, 4, 64, 256, 8192, 32768, },
  { 2, 4, 64, 256, 8192, 32768, },
  { 1, 2, 4, 64, 256, 8192, 32768, },
  { 8, 64, 256, 8192, 32768, },
  { 1, 8, 64, 256, 8192, 32768, },
  { 2, 8, 64, 256, 8192, 32768, },
  { 1, 2, 8, 64, 256, 8192, 32768, },
  { 4, 8, 64, 256, 8192, 32768, },
  { 1, 4, 8, 64, 256, 8192, 32768, },
  { 2, 4, 8, 64, 256, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 8192, 32768, },
  { 16, 64, 256, 8192, 32768, },
  { 1, 16, 64, 256, 8192, 32768, },
  { 2, 16, 64, 256, 8192, 32768, },
  { 1, 2, 16, 64, 256, 8192, 32768, },
  { 4, 16, 64, 256, 8192, 32768, },
  { 1, 4, 16, 64, 256, 8192, 32768, },
  { 2, 4, 16, 64, 256, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 8192, 32768, },
  { 8, 16, 64, 256, 8192, 32768, },
  { 1, 8, 16, 64, 256, 8192, 32768, },
  { 2, 8, 16, 64, 256, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 8192, 32768, },
  { 4, 8, 16, 64, 256, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 8192, 32768, },
  { 32, 64, 256, 8192, 32768, },
  { 1, 32, 64, 256, 8192, 32768, },
  { 2, 32, 64, 256, 8192, 32768, },
  { 1, 2, 32, 64, 256, 8192, 32768, },
  { 4, 32, 64, 256, 8192, 32768, },
  { 1, 4, 32, 64, 256, 8192, 32768, },
  { 2, 4, 32, 64, 256, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 8192, 32768, },
  { 8, 32, 64, 256, 8192, 32768, },
  { 1, 8, 32, 64, 256, 8192, 32768, },
  { 2, 8, 32, 64, 256, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 8192, 32768, },
  { 4, 8, 32, 64, 256, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 8192, 32768, },
  { 16, 32, 64, 256, 8192, 32768, },
  { 1, 16, 32, 64, 256, 8192, 32768, },
  { 2, 16, 32, 64, 256, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 8192, 32768, },
  { 4, 16, 32, 64, 256, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 8192, 32768, },
  { 8, 16, 32, 64, 256, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 8192, 32768, },
  { 128, 256, 8192, 32768, },
  { 1, 128, 256, 8192, 32768, },
  { 2, 128, 256, 8192, 32768, },
  { 1, 2, 128, 256, 8192, 32768, },
  { 4, 128, 256, 8192, 32768, },
  { 1, 4, 128, 256, 8192, 32768, },
  { 2, 4, 128, 256, 8192, 32768, },
  { 1, 2, 4, 128, 256, 8192, 32768, },
  { 8, 128, 256, 8192, 32768, },
  { 1, 8, 128, 256, 8192, 32768, },
  { 2, 8, 128, 256, 8192, 32768, },
  { 1, 2, 8, 128, 256, 8192, 32768, },
  { 4, 8, 128, 256, 8192, 32768, },
  { 1, 4, 8, 128, 256, 8192, 32768, },
  { 2, 4, 8, 128, 256, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 8192, 32768, },
  { 16, 128, 256, 8192, 32768, },
  { 1, 16, 128, 256, 8192, 32768, },
  { 2, 16, 128, 256, 8192, 32768, },
  { 1, 2, 16, 128, 256, 8192, 32768, },
  { 4, 16, 128, 256, 8192, 32768, },
  { 1, 4, 16, 128, 256, 8192, 32768, },
  { 2, 4, 16, 128, 256, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 8192, 32768, },
  { 8, 16, 128, 256, 8192, 32768, },
  { 1, 8, 16, 128, 256, 8192, 32768, },
  { 2, 8, 16, 128, 256, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 8192, 32768, },
  { 4, 8, 16, 128, 256, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 8192, 32768, },
  { 32, 128, 256, 8192, 32768, },
  { 1, 32, 128, 256, 8192, 32768, },
  { 2, 32, 128, 256, 8192, 32768, },
  { 1, 2, 32, 128, 256, 8192, 32768, },
  { 4, 32, 128, 256, 8192, 32768, },
  { 1, 4, 32, 128, 256, 8192, 32768, },
  { 2, 4, 32, 128, 256, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 8192, 32768, },
  { 8, 32, 128, 256, 8192, 32768, },
  { 1, 8, 32, 128, 256, 8192, 32768, },
  { 2, 8, 32, 128, 256, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 8192, 32768, },
  { 4, 8, 32, 128, 256, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 8192, 32768, },
  { 16, 32, 128, 256, 8192, 32768, },
  { 1, 16, 32, 128, 256, 8192, 32768, },
  { 2, 16, 32, 128, 256, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 8192, 32768, },
  { 4, 16, 32, 128, 256, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 8192, 32768, },
  { 8, 16, 32, 128, 256, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 8192, 32768, },
  { 64, 128, 256, 8192, 32768, },
  { 1, 64, 128, 256, 8192, 32768, },
  { 2, 64, 128, 256, 8192, 32768, },
  { 1, 2, 64, 128, 256, 8192, 32768, },
  { 4, 64, 128, 256, 8192, 32768, },
  { 1, 4, 64, 128, 256, 8192, 32768, },
  { 2, 4, 64, 128, 256, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 8192, 32768, },
  { 8, 64, 128, 256, 8192, 32768, },
  { 1, 8, 64, 128, 256, 8192, 32768, },
  { 2, 8, 64, 128, 256, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 8192, 32768, },
  { 4, 8, 64, 128, 256, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 8192, 32768, },
  { 16, 64, 128, 256, 8192, 32768, },
  { 1, 16, 64, 128, 256, 8192, 32768, },
  { 2, 16, 64, 128, 256, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 8192, 32768, },
  { 4, 16, 64, 128, 256, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 8192, 32768, },
  { 8, 16, 64, 128, 256, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 8192, 32768, },
  { 32, 64, 128, 256, 8192, 32768, },
  { 1, 32, 64, 128, 256, 8192, 32768, },
  { 2, 32, 64, 128, 256, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 8192, 32768, },
  { 4, 32, 64, 128, 256, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 8192, 32768, },
  { 8, 32, 64, 128, 256, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 8192, 32768, },
  { 16, 32, 64, 128, 256, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 8192, 32768, },
  { 512, 8192, 32768, },
  { 1, 512, 8192, 32768, },
  { 2, 512, 8192, 32768, },
  { 1, 2, 512, 8192, 32768, },
  { 4, 512, 8192, 32768, },
  { 1, 4, 512, 8192, 32768, },
  { 2, 4, 512, 8192, 32768, },
  { 1, 2, 4, 512, 8192, 32768, },
  { 8, 512, 8192, 32768, },
  { 1, 8, 512, 8192, 32768, },
  { 2, 8, 512, 8192, 32768, },
  { 1, 2, 8, 512, 8192, 32768, },
  { 4, 8, 512, 8192, 32768, },
  { 1, 4, 8, 512, 8192, 32768, },
  { 2, 4, 8, 512, 8192, 32768, },
  { 1, 2, 4, 8, 512, 8192, 32768, },
  { 16, 512, 8192, 32768, },
  { 1, 16, 512, 8192, 32768, },
  { 2, 16, 512, 8192, 32768, },
  { 1, 2, 16, 512, 8192, 32768, },
  { 4, 16, 512, 8192, 32768, },
  { 1, 4, 16, 512, 8192, 32768, },
  { 2, 4, 16, 512, 8192, 32768, },
  { 1, 2, 4, 16, 512, 8192, 32768, },
  { 8, 16, 512, 8192, 32768, },
  { 1, 8, 16, 512, 8192, 32768, },
  { 2, 8, 16, 512, 8192, 32768, },
  { 1, 2, 8, 16, 512, 8192, 32768, },
  { 4, 8, 16, 512, 8192, 32768, },
  { 1, 4, 8, 16, 512, 8192, 32768, },
  { 2, 4, 8, 16, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 512, 8192, 32768, },
  { 32, 512, 8192, 32768, },
  { 1, 32, 512, 8192, 32768, },
  { 2, 32, 512, 8192, 32768, },
  { 1, 2, 32, 512, 8192, 32768, },
  { 4, 32, 512, 8192, 32768, },
  { 1, 4, 32, 512, 8192, 32768, },
  { 2, 4, 32, 512, 8192, 32768, },
  { 1, 2, 4, 32, 512, 8192, 32768, },
  { 8, 32, 512, 8192, 32768, },
  { 1, 8, 32, 512, 8192, 32768, },
  { 2, 8, 32, 512, 8192, 32768, },
  { 1, 2, 8, 32, 512, 8192, 32768, },
  { 4, 8, 32, 512, 8192, 32768, },
  { 1, 4, 8, 32, 512, 8192, 32768, },
  { 2, 4, 8, 32, 512, 8192, 32768, },
  { 1, 2, 4, 8, 32, 512, 8192, 32768, },
  { 16, 32, 512, 8192, 32768, },
  { 1, 16, 32, 512, 8192, 32768, },
  { 2, 16, 32, 512, 8192, 32768, },
  { 1, 2, 16, 32, 512, 8192, 32768, },
  { 4, 16, 32, 512, 8192, 32768, },
  { 1, 4, 16, 32, 512, 8192, 32768, },
  { 2, 4, 16, 32, 512, 8192, 32768, },
  { 1, 2, 4, 16, 32, 512, 8192, 32768, },
  { 8, 16, 32, 512, 8192, 32768, },
  { 1, 8, 16, 32, 512, 8192, 32768, },
  { 2, 8, 16, 32, 512, 8192, 32768, },
  { 1, 2, 8, 16, 32, 512, 8192, 32768, },
  { 4, 8, 16, 32, 512, 8192, 32768, },
  { 1, 4, 8, 16, 32, 512, 8192, 32768, },
  { 2, 4, 8, 16, 32, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 8192, 32768, },
  { 64, 512, 8192, 32768, },
  { 1, 64, 512, 8192, 32768, },
  { 2, 64, 512, 8192, 32768, },
  { 1, 2, 64, 512, 8192, 32768, },
  { 4, 64, 512, 8192, 32768, },
  { 1, 4, 64, 512, 8192, 32768, },
  { 2, 4, 64, 512, 8192, 32768, },
  { 1, 2, 4, 64, 512, 8192, 32768, },
  { 8, 64, 512, 8192, 32768, },
  { 1, 8, 64, 512, 8192, 32768, },
  { 2, 8, 64, 512, 8192, 32768, },
  { 1, 2, 8, 64, 512, 8192, 32768, },
  { 4, 8, 64, 512, 8192, 32768, },
  { 1, 4, 8, 64, 512, 8192, 32768, },
  { 2, 4, 8, 64, 512, 8192, 32768, },
  { 1, 2, 4, 8, 64, 512, 8192, 32768, },
  { 16, 64, 512, 8192, 32768, },
  { 1, 16, 64, 512, 8192, 32768, },
  { 2, 16, 64, 512, 8192, 32768, },
  { 1, 2, 16, 64, 512, 8192, 32768, },
  { 4, 16, 64, 512, 8192, 32768, },
  { 1, 4, 16, 64, 512, 8192, 32768, },
  { 2, 4, 16, 64, 512, 8192, 32768, },
  { 1, 2, 4, 16, 64, 512, 8192, 32768, },
  { 8, 16, 64, 512, 8192, 32768, },
  { 1, 8, 16, 64, 512, 8192, 32768, },
  { 2, 8, 16, 64, 512, 8192, 32768, },
  { 1, 2, 8, 16, 64, 512, 8192, 32768, },
  { 4, 8, 16, 64, 512, 8192, 32768, },
  { 1, 4, 8, 16, 64, 512, 8192, 32768, },
  { 2, 4, 8, 16, 64, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 8192, 32768, },
  { 32, 64, 512, 8192, 32768, },
  { 1, 32, 64, 512, 8192, 32768, },
  { 2, 32, 64, 512, 8192, 32768, },
  { 1, 2, 32, 64, 512, 8192, 32768, },
  { 4, 32, 64, 512, 8192, 32768, },
  { 1, 4, 32, 64, 512, 8192, 32768, },
  { 2, 4, 32, 64, 512, 8192, 32768, },
  { 1, 2, 4, 32, 64, 512, 8192, 32768, },
  { 8, 32, 64, 512, 8192, 32768, },
  { 1, 8, 32, 64, 512, 8192, 32768, },
  { 2, 8, 32, 64, 512, 8192, 32768, },
  { 1, 2, 8, 32, 64, 512, 8192, 32768, },
  { 4, 8, 32, 64, 512, 8192, 32768, },
  { 1, 4, 8, 32, 64, 512, 8192, 32768, },
  { 2, 4, 8, 32, 64, 512, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 8192, 32768, },
  { 16, 32, 64, 512, 8192, 32768, },
  { 1, 16, 32, 64, 512, 8192, 32768, },
  { 2, 16, 32, 64, 512, 8192, 32768, },
  { 1, 2, 16, 32, 64, 512, 8192, 32768, },
  { 4, 16, 32, 64, 512, 8192, 32768, },
  { 1, 4, 16, 32, 64, 512, 8192, 32768, },
  { 2, 4, 16, 32, 64, 512, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 8192, 32768, },
  { 8, 16, 32, 64, 512, 8192, 32768, },
  { 1, 8, 16, 32, 64, 512, 8192, 32768, },
  { 2, 8, 16, 32, 64, 512, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 8192, 32768, },
  { 4, 8, 16, 32, 64, 512, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 8192, 32768, },
  { 128, 512, 8192, 32768, },
  { 1, 128, 512, 8192, 32768, },
  { 2, 128, 512, 8192, 32768, },
  { 1, 2, 128, 512, 8192, 32768, },
  { 4, 128, 512, 8192, 32768, },
  { 1, 4, 128, 512, 8192, 32768, },
  { 2, 4, 128, 512, 8192, 32768, },
  { 1, 2, 4, 128, 512, 8192, 32768, },
  { 8, 128, 512, 8192, 32768, },
  { 1, 8, 128, 512, 8192, 32768, },
  { 2, 8, 128, 512, 8192, 32768, },
  { 1, 2, 8, 128, 512, 8192, 32768, },
  { 4, 8, 128, 512, 8192, 32768, },
  { 1, 4, 8, 128, 512, 8192, 32768, },
  { 2, 4, 8, 128, 512, 8192, 32768, },
  { 1, 2, 4, 8, 128, 512, 8192, 32768, },
  { 16, 128, 512, 8192, 32768, },
  { 1, 16, 128, 512, 8192, 32768, },
  { 2, 16, 128, 512, 8192, 32768, },
  { 1, 2, 16, 128, 512, 8192, 32768, },
  { 4, 16, 128, 512, 8192, 32768, },
  { 1, 4, 16, 128, 512, 8192, 32768, },
  { 2, 4, 16, 128, 512, 8192, 32768, },
  { 1, 2, 4, 16, 128, 512, 8192, 32768, },
  { 8, 16, 128, 512, 8192, 32768, },
  { 1, 8, 16, 128, 512, 8192, 32768, },
  { 2, 8, 16, 128, 512, 8192, 32768, },
  { 1, 2, 8, 16, 128, 512, 8192, 32768, },
  { 4, 8, 16, 128, 512, 8192, 32768, },
  { 1, 4, 8, 16, 128, 512, 8192, 32768, },
  { 2, 4, 8, 16, 128, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 8192, 32768, },
  { 32, 128, 512, 8192, 32768, },
  { 1, 32, 128, 512, 8192, 32768, },
  { 2, 32, 128, 512, 8192, 32768, },
  { 1, 2, 32, 128, 512, 8192, 32768, },
  { 4, 32, 128, 512, 8192, 32768, },
  { 1, 4, 32, 128, 512, 8192, 32768, },
  { 2, 4, 32, 128, 512, 8192, 32768, },
  { 1, 2, 4, 32, 128, 512, 8192, 32768, },
  { 8, 32, 128, 512, 8192, 32768, },
  { 1, 8, 32, 128, 512, 8192, 32768, },
  { 2, 8, 32, 128, 512, 8192, 32768, },
  { 1, 2, 8, 32, 128, 512, 8192, 32768, },
  { 4, 8, 32, 128, 512, 8192, 32768, },
  { 1, 4, 8, 32, 128, 512, 8192, 32768, },
  { 2, 4, 8, 32, 128, 512, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 8192, 32768, },
  { 16, 32, 128, 512, 8192, 32768, },
  { 1, 16, 32, 128, 512, 8192, 32768, },
  { 2, 16, 32, 128, 512, 8192, 32768, },
  { 1, 2, 16, 32, 128, 512, 8192, 32768, },
  { 4, 16, 32, 128, 512, 8192, 32768, },
  { 1, 4, 16, 32, 128, 512, 8192, 32768, },
  { 2, 4, 16, 32, 128, 512, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 8192, 32768, },
  { 8, 16, 32, 128, 512, 8192, 32768, },
  { 1, 8, 16, 32, 128, 512, 8192, 32768, },
  { 2, 8, 16, 32, 128, 512, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 8192, 32768, },
  { 4, 8, 16, 32, 128, 512, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 8192, 32768, },
  { 64, 128, 512, 8192, 32768, },
  { 1, 64, 128, 512, 8192, 32768, },
  { 2, 64, 128, 512, 8192, 32768, },
  { 1, 2, 64, 128, 512, 8192, 32768, },
  { 4, 64, 128, 512, 8192, 32768, },
  { 1, 4, 64, 128, 512, 8192, 32768, },
  { 2, 4, 64, 128, 512, 8192, 32768, },
  { 1, 2, 4, 64, 128, 512, 8192, 32768, },
  { 8, 64, 128, 512, 8192, 32768, },
  { 1, 8, 64, 128, 512, 8192, 32768, },
  { 2, 8, 64, 128, 512, 8192, 32768, },
  { 1, 2, 8, 64, 128, 512, 8192, 32768, },
  { 4, 8, 64, 128, 512, 8192, 32768, },
  { 1, 4, 8, 64, 128, 512, 8192, 32768, },
  { 2, 4, 8, 64, 128, 512, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 8192, 32768, },
  { 16, 64, 128, 512, 8192, 32768, },
  { 1, 16, 64, 128, 512, 8192, 32768, },
  { 2, 16, 64, 128, 512, 8192, 32768, },
  { 1, 2, 16, 64, 128, 512, 8192, 32768, },
  { 4, 16, 64, 128, 512, 8192, 32768, },
  { 1, 4, 16, 64, 128, 512, 8192, 32768, },
  { 2, 4, 16, 64, 128, 512, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 8192, 32768, },
  { 8, 16, 64, 128, 512, 8192, 32768, },
  { 1, 8, 16, 64, 128, 512, 8192, 32768, },
  { 2, 8, 16, 64, 128, 512, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 8192, 32768, },
  { 4, 8, 16, 64, 128, 512, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 8192, 32768, },
  { 32, 64, 128, 512, 8192, 32768, },
  { 1, 32, 64, 128, 512, 8192, 32768, },
  { 2, 32, 64, 128, 512, 8192, 32768, },
  { 1, 2, 32, 64, 128, 512, 8192, 32768, },
  { 4, 32, 64, 128, 512, 8192, 32768, },
  { 1, 4, 32, 64, 128, 512, 8192, 32768, },
  { 2, 4, 32, 64, 128, 512, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 8192, 32768, },
  { 8, 32, 64, 128, 512, 8192, 32768, },
  { 1, 8, 32, 64, 128, 512, 8192, 32768, },
  { 2, 8, 32, 64, 128, 512, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 8192, 32768, },
  { 4, 8, 32, 64, 128, 512, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 8192, 32768, },
  { 16, 32, 64, 128, 512, 8192, 32768, },
  { 1, 16, 32, 64, 128, 512, 8192, 32768, },
  { 2, 16, 32, 64, 128, 512, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 8192, 32768, },
  { 4, 16, 32, 64, 128, 512, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 8192, 32768, },
  { 8, 16, 32, 64, 128, 512, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 8192, 32768, },
  { 256, 512, 8192, 32768, },
  { 1, 256, 512, 8192, 32768, },
  { 2, 256, 512, 8192, 32768, },
  { 1, 2, 256, 512, 8192, 32768, },
  { 4, 256, 512, 8192, 32768, },
  { 1, 4, 256, 512, 8192, 32768, },
  { 2, 4, 256, 512, 8192, 32768, },
  { 1, 2, 4, 256, 512, 8192, 32768, },
  { 8, 256, 512, 8192, 32768, },
  { 1, 8, 256, 512, 8192, 32768, },
  { 2, 8, 256, 512, 8192, 32768, },
  { 1, 2, 8, 256, 512, 8192, 32768, },
  { 4, 8, 256, 512, 8192, 32768, },
  { 1, 4, 8, 256, 512, 8192, 32768, },
  { 2, 4, 8, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 256, 512, 8192, 32768, },
  { 16, 256, 512, 8192, 32768, },
  { 1, 16, 256, 512, 8192, 32768, },
  { 2, 16, 256, 512, 8192, 32768, },
  { 1, 2, 16, 256, 512, 8192, 32768, },
  { 4, 16, 256, 512, 8192, 32768, },
  { 1, 4, 16, 256, 512, 8192, 32768, },
  { 2, 4, 16, 256, 512, 8192, 32768, },
  { 1, 2, 4, 16, 256, 512, 8192, 32768, },
  { 8, 16, 256, 512, 8192, 32768, },
  { 1, 8, 16, 256, 512, 8192, 32768, },
  { 2, 8, 16, 256, 512, 8192, 32768, },
  { 1, 2, 8, 16, 256, 512, 8192, 32768, },
  { 4, 8, 16, 256, 512, 8192, 32768, },
  { 1, 4, 8, 16, 256, 512, 8192, 32768, },
  { 2, 4, 8, 16, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 8192, 32768, },
  { 32, 256, 512, 8192, 32768, },
  { 1, 32, 256, 512, 8192, 32768, },
  { 2, 32, 256, 512, 8192, 32768, },
  { 1, 2, 32, 256, 512, 8192, 32768, },
  { 4, 32, 256, 512, 8192, 32768, },
  { 1, 4, 32, 256, 512, 8192, 32768, },
  { 2, 4, 32, 256, 512, 8192, 32768, },
  { 1, 2, 4, 32, 256, 512, 8192, 32768, },
  { 8, 32, 256, 512, 8192, 32768, },
  { 1, 8, 32, 256, 512, 8192, 32768, },
  { 2, 8, 32, 256, 512, 8192, 32768, },
  { 1, 2, 8, 32, 256, 512, 8192, 32768, },
  { 4, 8, 32, 256, 512, 8192, 32768, },
  { 1, 4, 8, 32, 256, 512, 8192, 32768, },
  { 2, 4, 8, 32, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 8192, 32768, },
  { 16, 32, 256, 512, 8192, 32768, },
  { 1, 16, 32, 256, 512, 8192, 32768, },
  { 2, 16, 32, 256, 512, 8192, 32768, },
  { 1, 2, 16, 32, 256, 512, 8192, 32768, },
  { 4, 16, 32, 256, 512, 8192, 32768, },
  { 1, 4, 16, 32, 256, 512, 8192, 32768, },
  { 2, 4, 16, 32, 256, 512, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 8192, 32768, },
  { 8, 16, 32, 256, 512, 8192, 32768, },
  { 1, 8, 16, 32, 256, 512, 8192, 32768, },
  { 2, 8, 16, 32, 256, 512, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 8192, 32768, },
  { 4, 8, 16, 32, 256, 512, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 8192, 32768, },
  { 64, 256, 512, 8192, 32768, },
  { 1, 64, 256, 512, 8192, 32768, },
  { 2, 64, 256, 512, 8192, 32768, },
  { 1, 2, 64, 256, 512, 8192, 32768, },
  { 4, 64, 256, 512, 8192, 32768, },
  { 1, 4, 64, 256, 512, 8192, 32768, },
  { 2, 4, 64, 256, 512, 8192, 32768, },
  { 1, 2, 4, 64, 256, 512, 8192, 32768, },
  { 8, 64, 256, 512, 8192, 32768, },
  { 1, 8, 64, 256, 512, 8192, 32768, },
  { 2, 8, 64, 256, 512, 8192, 32768, },
  { 1, 2, 8, 64, 256, 512, 8192, 32768, },
  { 4, 8, 64, 256, 512, 8192, 32768, },
  { 1, 4, 8, 64, 256, 512, 8192, 32768, },
  { 2, 4, 8, 64, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 8192, 32768, },
  { 16, 64, 256, 512, 8192, 32768, },
  { 1, 16, 64, 256, 512, 8192, 32768, },
  { 2, 16, 64, 256, 512, 8192, 32768, },
  { 1, 2, 16, 64, 256, 512, 8192, 32768, },
  { 4, 16, 64, 256, 512, 8192, 32768, },
  { 1, 4, 16, 64, 256, 512, 8192, 32768, },
  { 2, 4, 16, 64, 256, 512, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 8192, 32768, },
  { 8, 16, 64, 256, 512, 8192, 32768, },
  { 1, 8, 16, 64, 256, 512, 8192, 32768, },
  { 2, 8, 16, 64, 256, 512, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 8192, 32768, },
  { 4, 8, 16, 64, 256, 512, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 8192, 32768, },
  { 32, 64, 256, 512, 8192, 32768, },
  { 1, 32, 64, 256, 512, 8192, 32768, },
  { 2, 32, 64, 256, 512, 8192, 32768, },
  { 1, 2, 32, 64, 256, 512, 8192, 32768, },
  { 4, 32, 64, 256, 512, 8192, 32768, },
  { 1, 4, 32, 64, 256, 512, 8192, 32768, },
  { 2, 4, 32, 64, 256, 512, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 8192, 32768, },
  { 8, 32, 64, 256, 512, 8192, 32768, },
  { 1, 8, 32, 64, 256, 512, 8192, 32768, },
  { 2, 8, 32, 64, 256, 512, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 8192, 32768, },
  { 4, 8, 32, 64, 256, 512, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 8192, 32768, },
  { 16, 32, 64, 256, 512, 8192, 32768, },
  { 1, 16, 32, 64, 256, 512, 8192, 32768, },
  { 2, 16, 32, 64, 256, 512, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 8192, 32768, },
  { 4, 16, 32, 64, 256, 512, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 8192, 32768, },
  { 8, 16, 32, 64, 256, 512, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 8192, 32768, },
  { 128, 256, 512, 8192, 32768, },
  { 1, 128, 256, 512, 8192, 32768, },
  { 2, 128, 256, 512, 8192, 32768, },
  { 1, 2, 128, 256, 512, 8192, 32768, },
  { 4, 128, 256, 512, 8192, 32768, },
  { 1, 4, 128, 256, 512, 8192, 32768, },
  { 2, 4, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 128, 256, 512, 8192, 32768, },
  { 8, 128, 256, 512, 8192, 32768, },
  { 1, 8, 128, 256, 512, 8192, 32768, },
  { 2, 8, 128, 256, 512, 8192, 32768, },
  { 1, 2, 8, 128, 256, 512, 8192, 32768, },
  { 4, 8, 128, 256, 512, 8192, 32768, },
  { 1, 4, 8, 128, 256, 512, 8192, 32768, },
  { 2, 4, 8, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 8192, 32768, },
  { 16, 128, 256, 512, 8192, 32768, },
  { 1, 16, 128, 256, 512, 8192, 32768, },
  { 2, 16, 128, 256, 512, 8192, 32768, },
  { 1, 2, 16, 128, 256, 512, 8192, 32768, },
  { 4, 16, 128, 256, 512, 8192, 32768, },
  { 1, 4, 16, 128, 256, 512, 8192, 32768, },
  { 2, 4, 16, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 8192, 32768, },
  { 8, 16, 128, 256, 512, 8192, 32768, },
  { 1, 8, 16, 128, 256, 512, 8192, 32768, },
  { 2, 8, 16, 128, 256, 512, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 8192, 32768, },
  { 4, 8, 16, 128, 256, 512, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 8192, 32768, },
  { 32, 128, 256, 512, 8192, 32768, },
  { 1, 32, 128, 256, 512, 8192, 32768, },
  { 2, 32, 128, 256, 512, 8192, 32768, },
  { 1, 2, 32, 128, 256, 512, 8192, 32768, },
  { 4, 32, 128, 256, 512, 8192, 32768, },
  { 1, 4, 32, 128, 256, 512, 8192, 32768, },
  { 2, 4, 32, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 8192, 32768, },
  { 8, 32, 128, 256, 512, 8192, 32768, },
  { 1, 8, 32, 128, 256, 512, 8192, 32768, },
  { 2, 8, 32, 128, 256, 512, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 8192, 32768, },
  { 4, 8, 32, 128, 256, 512, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 8192, 32768, },
  { 16, 32, 128, 256, 512, 8192, 32768, },
  { 1, 16, 32, 128, 256, 512, 8192, 32768, },
  { 2, 16, 32, 128, 256, 512, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 8192, 32768, },
  { 4, 16, 32, 128, 256, 512, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 8192, 32768, },
  { 8, 16, 32, 128, 256, 512, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 8192, 32768, },
  { 64, 128, 256, 512, 8192, 32768, },
  { 1, 64, 128, 256, 512, 8192, 32768, },
  { 2, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 64, 128, 256, 512, 8192, 32768, },
  { 4, 64, 128, 256, 512, 8192, 32768, },
  { 1, 4, 64, 128, 256, 512, 8192, 32768, },
  { 2, 4, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 8192, 32768, },
  { 8, 64, 128, 256, 512, 8192, 32768, },
  { 1, 8, 64, 128, 256, 512, 8192, 32768, },
  { 2, 8, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 8192, 32768, },
  { 4, 8, 64, 128, 256, 512, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 8192, 32768, },
  { 16, 64, 128, 256, 512, 8192, 32768, },
  { 1, 16, 64, 128, 256, 512, 8192, 32768, },
  { 2, 16, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 8192, 32768, },
  { 4, 16, 64, 128, 256, 512, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 8192, 32768, },
  { 8, 16, 64, 128, 256, 512, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 8192, 32768, },
  { 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 32, 64, 128, 256, 512, 8192, 32768, },
  { 2, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 8192, 32768, },
  { 4, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 8192, 32768, },
  { 8, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 8192, 32768, },
  { 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 32768, },
  { 1024, 8192, 32768, },
  { 1, 1024, 8192, 32768, },
  { 2, 1024, 8192, 32768, },
  { 1, 2, 1024, 8192, 32768, },
  { 4, 1024, 8192, 32768, },
  { 1, 4, 1024, 8192, 32768, },
  { 2, 4, 1024, 8192, 32768, },
  { 1, 2, 4, 1024, 8192, 32768, },
  { 8, 1024, 8192, 32768, },
  { 1, 8, 1024, 8192, 32768, },
  { 2, 8, 1024, 8192, 32768, },
  { 1, 2, 8, 1024, 8192, 32768, },
  { 4, 8, 1024, 8192, 32768, },
  { 1, 4, 8, 1024, 8192, 32768, },
  { 2, 4, 8, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 1024, 8192, 32768, },
  { 16, 1024, 8192, 32768, },
  { 1, 16, 1024, 8192, 32768, },
  { 2, 16, 1024, 8192, 32768, },
  { 1, 2, 16, 1024, 8192, 32768, },
  { 4, 16, 1024, 8192, 32768, },
  { 1, 4, 16, 1024, 8192, 32768, },
  { 2, 4, 16, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 1024, 8192, 32768, },
  { 8, 16, 1024, 8192, 32768, },
  { 1, 8, 16, 1024, 8192, 32768, },
  { 2, 8, 16, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 1024, 8192, 32768, },
  { 4, 8, 16, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 1024, 8192, 32768, },
  { 32, 1024, 8192, 32768, },
  { 1, 32, 1024, 8192, 32768, },
  { 2, 32, 1024, 8192, 32768, },
  { 1, 2, 32, 1024, 8192, 32768, },
  { 4, 32, 1024, 8192, 32768, },
  { 1, 4, 32, 1024, 8192, 32768, },
  { 2, 4, 32, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 1024, 8192, 32768, },
  { 8, 32, 1024, 8192, 32768, },
  { 1, 8, 32, 1024, 8192, 32768, },
  { 2, 8, 32, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 1024, 8192, 32768, },
  { 4, 8, 32, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 1024, 8192, 32768, },
  { 16, 32, 1024, 8192, 32768, },
  { 1, 16, 32, 1024, 8192, 32768, },
  { 2, 16, 32, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 1024, 8192, 32768, },
  { 4, 16, 32, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 1024, 8192, 32768, },
  { 8, 16, 32, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 8192, 32768, },
  { 64, 1024, 8192, 32768, },
  { 1, 64, 1024, 8192, 32768, },
  { 2, 64, 1024, 8192, 32768, },
  { 1, 2, 64, 1024, 8192, 32768, },
  { 4, 64, 1024, 8192, 32768, },
  { 1, 4, 64, 1024, 8192, 32768, },
  { 2, 4, 64, 1024, 8192, 32768, },
  { 1, 2, 4, 64, 1024, 8192, 32768, },
  { 8, 64, 1024, 8192, 32768, },
  { 1, 8, 64, 1024, 8192, 32768, },
  { 2, 8, 64, 1024, 8192, 32768, },
  { 1, 2, 8, 64, 1024, 8192, 32768, },
  { 4, 8, 64, 1024, 8192, 32768, },
  { 1, 4, 8, 64, 1024, 8192, 32768, },
  { 2, 4, 8, 64, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 64, 1024, 8192, 32768, },
  { 16, 64, 1024, 8192, 32768, },
  { 1, 16, 64, 1024, 8192, 32768, },
  { 2, 16, 64, 1024, 8192, 32768, },
  { 1, 2, 16, 64, 1024, 8192, 32768, },
  { 4, 16, 64, 1024, 8192, 32768, },
  { 1, 4, 16, 64, 1024, 8192, 32768, },
  { 2, 4, 16, 64, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 64, 1024, 8192, 32768, },
  { 8, 16, 64, 1024, 8192, 32768, },
  { 1, 8, 16, 64, 1024, 8192, 32768, },
  { 2, 8, 16, 64, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 64, 1024, 8192, 32768, },
  { 4, 8, 16, 64, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 64, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 64, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 8192, 32768, },
  { 32, 64, 1024, 8192, 32768, },
  { 1, 32, 64, 1024, 8192, 32768, },
  { 2, 32, 64, 1024, 8192, 32768, },
  { 1, 2, 32, 64, 1024, 8192, 32768, },
  { 4, 32, 64, 1024, 8192, 32768, },
  { 1, 4, 32, 64, 1024, 8192, 32768, },
  { 2, 4, 32, 64, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 64, 1024, 8192, 32768, },
  { 8, 32, 64, 1024, 8192, 32768, },
  { 1, 8, 32, 64, 1024, 8192, 32768, },
  { 2, 8, 32, 64, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 64, 1024, 8192, 32768, },
  { 4, 8, 32, 64, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 64, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 64, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 8192, 32768, },
  { 16, 32, 64, 1024, 8192, 32768, },
  { 1, 16, 32, 64, 1024, 8192, 32768, },
  { 2, 16, 32, 64, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 64, 1024, 8192, 32768, },
  { 4, 16, 32, 64, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 64, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 64, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 8192, 32768, },
  { 8, 16, 32, 64, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 64, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 64, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 64, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 8192, 32768, },
  { 128, 1024, 8192, 32768, },
  { 1, 128, 1024, 8192, 32768, },
  { 2, 128, 1024, 8192, 32768, },
  { 1, 2, 128, 1024, 8192, 32768, },
  { 4, 128, 1024, 8192, 32768, },
  { 1, 4, 128, 1024, 8192, 32768, },
  { 2, 4, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 128, 1024, 8192, 32768, },
  { 8, 128, 1024, 8192, 32768, },
  { 1, 8, 128, 1024, 8192, 32768, },
  { 2, 8, 128, 1024, 8192, 32768, },
  { 1, 2, 8, 128, 1024, 8192, 32768, },
  { 4, 8, 128, 1024, 8192, 32768, },
  { 1, 4, 8, 128, 1024, 8192, 32768, },
  { 2, 4, 8, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 128, 1024, 8192, 32768, },
  { 16, 128, 1024, 8192, 32768, },
  { 1, 16, 128, 1024, 8192, 32768, },
  { 2, 16, 128, 1024, 8192, 32768, },
  { 1, 2, 16, 128, 1024, 8192, 32768, },
  { 4, 16, 128, 1024, 8192, 32768, },
  { 1, 4, 16, 128, 1024, 8192, 32768, },
  { 2, 4, 16, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 128, 1024, 8192, 32768, },
  { 8, 16, 128, 1024, 8192, 32768, },
  { 1, 8, 16, 128, 1024, 8192, 32768, },
  { 2, 8, 16, 128, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 128, 1024, 8192, 32768, },
  { 4, 8, 16, 128, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 128, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 8192, 32768, },
  { 32, 128, 1024, 8192, 32768, },
  { 1, 32, 128, 1024, 8192, 32768, },
  { 2, 32, 128, 1024, 8192, 32768, },
  { 1, 2, 32, 128, 1024, 8192, 32768, },
  { 4, 32, 128, 1024, 8192, 32768, },
  { 1, 4, 32, 128, 1024, 8192, 32768, },
  { 2, 4, 32, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 128, 1024, 8192, 32768, },
  { 8, 32, 128, 1024, 8192, 32768, },
  { 1, 8, 32, 128, 1024, 8192, 32768, },
  { 2, 8, 32, 128, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 128, 1024, 8192, 32768, },
  { 4, 8, 32, 128, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 128, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 8192, 32768, },
  { 16, 32, 128, 1024, 8192, 32768, },
  { 1, 16, 32, 128, 1024, 8192, 32768, },
  { 2, 16, 32, 128, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 128, 1024, 8192, 32768, },
  { 4, 16, 32, 128, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 128, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 8192, 32768, },
  { 8, 16, 32, 128, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 128, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 128, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 128, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 8192, 32768, },
  { 64, 128, 1024, 8192, 32768, },
  { 1, 64, 128, 1024, 8192, 32768, },
  { 2, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 64, 128, 1024, 8192, 32768, },
  { 4, 64, 128, 1024, 8192, 32768, },
  { 1, 4, 64, 128, 1024, 8192, 32768, },
  { 2, 4, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 64, 128, 1024, 8192, 32768, },
  { 8, 64, 128, 1024, 8192, 32768, },
  { 1, 8, 64, 128, 1024, 8192, 32768, },
  { 2, 8, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 8, 64, 128, 1024, 8192, 32768, },
  { 4, 8, 64, 128, 1024, 8192, 32768, },
  { 1, 4, 8, 64, 128, 1024, 8192, 32768, },
  { 2, 4, 8, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 8192, 32768, },
  { 16, 64, 128, 1024, 8192, 32768, },
  { 1, 16, 64, 128, 1024, 8192, 32768, },
  { 2, 16, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 16, 64, 128, 1024, 8192, 32768, },
  { 4, 16, 64, 128, 1024, 8192, 32768, },
  { 1, 4, 16, 64, 128, 1024, 8192, 32768, },
  { 2, 4, 16, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 8192, 32768, },
  { 8, 16, 64, 128, 1024, 8192, 32768, },
  { 1, 8, 16, 64, 128, 1024, 8192, 32768, },
  { 2, 8, 16, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 8192, 32768, },
  { 4, 8, 16, 64, 128, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 8192, 32768, },
  { 32, 64, 128, 1024, 8192, 32768, },
  { 1, 32, 64, 128, 1024, 8192, 32768, },
  { 2, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 32, 64, 128, 1024, 8192, 32768, },
  { 4, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 4, 32, 64, 128, 1024, 8192, 32768, },
  { 2, 4, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 8192, 32768, },
  { 8, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 8, 32, 64, 128, 1024, 8192, 32768, },
  { 2, 8, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 8192, 32768, },
  { 4, 8, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 8192, 32768, },
  { 16, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 2, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 4, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 8, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 8192, 32768, },
  { 256, 1024, 8192, 32768, },
  { 1, 256, 1024, 8192, 32768, },
  { 2, 256, 1024, 8192, 32768, },
  { 1, 2, 256, 1024, 8192, 32768, },
  { 4, 256, 1024, 8192, 32768, },
  { 1, 4, 256, 1024, 8192, 32768, },
  { 2, 4, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 256, 1024, 8192, 32768, },
  { 8, 256, 1024, 8192, 32768, },
  { 1, 8, 256, 1024, 8192, 32768, },
  { 2, 8, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 256, 1024, 8192, 32768, },
  { 4, 8, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 256, 1024, 8192, 32768, },
  { 16, 256, 1024, 8192, 32768, },
  { 1, 16, 256, 1024, 8192, 32768, },
  { 2, 16, 256, 1024, 8192, 32768, },
  { 1, 2, 16, 256, 1024, 8192, 32768, },
  { 4, 16, 256, 1024, 8192, 32768, },
  { 1, 4, 16, 256, 1024, 8192, 32768, },
  { 2, 4, 16, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 256, 1024, 8192, 32768, },
  { 8, 16, 256, 1024, 8192, 32768, },
  { 1, 8, 16, 256, 1024, 8192, 32768, },
  { 2, 8, 16, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 256, 1024, 8192, 32768, },
  { 4, 8, 16, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 8192, 32768, },
  { 32, 256, 1024, 8192, 32768, },
  { 1, 32, 256, 1024, 8192, 32768, },
  { 2, 32, 256, 1024, 8192, 32768, },
  { 1, 2, 32, 256, 1024, 8192, 32768, },
  { 4, 32, 256, 1024, 8192, 32768, },
  { 1, 4, 32, 256, 1024, 8192, 32768, },
  { 2, 4, 32, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 256, 1024, 8192, 32768, },
  { 8, 32, 256, 1024, 8192, 32768, },
  { 1, 8, 32, 256, 1024, 8192, 32768, },
  { 2, 8, 32, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 256, 1024, 8192, 32768, },
  { 4, 8, 32, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 8192, 32768, },
  { 16, 32, 256, 1024, 8192, 32768, },
  { 1, 16, 32, 256, 1024, 8192, 32768, },
  { 2, 16, 32, 256, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 256, 1024, 8192, 32768, },
  { 4, 16, 32, 256, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 256, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 8192, 32768, },
  { 8, 16, 32, 256, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 256, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 8192, 32768, },
  { 64, 256, 1024, 8192, 32768, },
  { 1, 64, 256, 1024, 8192, 32768, },
  { 2, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 64, 256, 1024, 8192, 32768, },
  { 4, 64, 256, 1024, 8192, 32768, },
  { 1, 4, 64, 256, 1024, 8192, 32768, },
  { 2, 4, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 64, 256, 1024, 8192, 32768, },
  { 8, 64, 256, 1024, 8192, 32768, },
  { 1, 8, 64, 256, 1024, 8192, 32768, },
  { 2, 8, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 64, 256, 1024, 8192, 32768, },
  { 4, 8, 64, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 64, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 8192, 32768, },
  { 16, 64, 256, 1024, 8192, 32768, },
  { 1, 16, 64, 256, 1024, 8192, 32768, },
  { 2, 16, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 16, 64, 256, 1024, 8192, 32768, },
  { 4, 16, 64, 256, 1024, 8192, 32768, },
  { 1, 4, 16, 64, 256, 1024, 8192, 32768, },
  { 2, 4, 16, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 8192, 32768, },
  { 8, 16, 64, 256, 1024, 8192, 32768, },
  { 1, 8, 16, 64, 256, 1024, 8192, 32768, },
  { 2, 8, 16, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 8192, 32768, },
  { 4, 8, 16, 64, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 8192, 32768, },
  { 32, 64, 256, 1024, 8192, 32768, },
  { 1, 32, 64, 256, 1024, 8192, 32768, },
  { 2, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 32, 64, 256, 1024, 8192, 32768, },
  { 4, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 4, 32, 64, 256, 1024, 8192, 32768, },
  { 2, 4, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 8192, 32768, },
  { 8, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 8, 32, 64, 256, 1024, 8192, 32768, },
  { 2, 8, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 8192, 32768, },
  { 4, 8, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 8192, 32768, },
  { 16, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 2, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 4, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 8, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 8192, 32768, },
  { 128, 256, 1024, 8192, 32768, },
  { 1, 128, 256, 1024, 8192, 32768, },
  { 2, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 128, 256, 1024, 8192, 32768, },
  { 4, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 128, 256, 1024, 8192, 32768, },
  { 8, 128, 256, 1024, 8192, 32768, },
  { 1, 8, 128, 256, 1024, 8192, 32768, },
  { 2, 8, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 128, 256, 1024, 8192, 32768, },
  { 4, 8, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 8192, 32768, },
  { 16, 128, 256, 1024, 8192, 32768, },
  { 1, 16, 128, 256, 1024, 8192, 32768, },
  { 2, 16, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 16, 128, 256, 1024, 8192, 32768, },
  { 4, 16, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 16, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 16, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 8192, 32768, },
  { 8, 16, 128, 256, 1024, 8192, 32768, },
  { 1, 8, 16, 128, 256, 1024, 8192, 32768, },
  { 2, 8, 16, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 8192, 32768, },
  { 4, 8, 16, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 8192, 32768, },
  { 32, 128, 256, 1024, 8192, 32768, },
  { 1, 32, 128, 256, 1024, 8192, 32768, },
  { 2, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 32, 128, 256, 1024, 8192, 32768, },
  { 4, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 32, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 8192, 32768, },
  { 8, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 8, 32, 128, 256, 1024, 8192, 32768, },
  { 2, 8, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 8192, 32768, },
  { 4, 8, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 8192, 32768, },
  { 16, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 2, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 4, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 8, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 8192, 32768, },
  { 64, 128, 256, 1024, 8192, 32768, },
  { 1, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 64, 128, 256, 1024, 8192, 32768, },
  { 4, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 8192, 32768, },
  { 8, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 8, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 8, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 8192, 32768, },
  { 4, 8, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 8192, 32768, },
  { 16, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 4, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 8, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 8192, 32768, },
  { 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 4, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 8, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 32768, },
  { 512, 1024, 8192, 32768, },
  { 1, 512, 1024, 8192, 32768, },
  { 2, 512, 1024, 8192, 32768, },
  { 1, 2, 512, 1024, 8192, 32768, },
  { 4, 512, 1024, 8192, 32768, },
  { 1, 4, 512, 1024, 8192, 32768, },
  { 2, 4, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 512, 1024, 8192, 32768, },
  { 8, 512, 1024, 8192, 32768, },
  { 1, 8, 512, 1024, 8192, 32768, },
  { 2, 8, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 512, 1024, 8192, 32768, },
  { 4, 8, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 512, 1024, 8192, 32768, },
  { 16, 512, 1024, 8192, 32768, },
  { 1, 16, 512, 1024, 8192, 32768, },
  { 2, 16, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 512, 1024, 8192, 32768, },
  { 4, 16, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 512, 1024, 8192, 32768, },
  { 8, 16, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 8192, 32768, },
  { 32, 512, 1024, 8192, 32768, },
  { 1, 32, 512, 1024, 8192, 32768, },
  { 2, 32, 512, 1024, 8192, 32768, },
  { 1, 2, 32, 512, 1024, 8192, 32768, },
  { 4, 32, 512, 1024, 8192, 32768, },
  { 1, 4, 32, 512, 1024, 8192, 32768, },
  { 2, 4, 32, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 512, 1024, 8192, 32768, },
  { 8, 32, 512, 1024, 8192, 32768, },
  { 1, 8, 32, 512, 1024, 8192, 32768, },
  { 2, 8, 32, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 512, 1024, 8192, 32768, },
  { 4, 8, 32, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 8192, 32768, },
  { 16, 32, 512, 1024, 8192, 32768, },
  { 1, 16, 32, 512, 1024, 8192, 32768, },
  { 2, 16, 32, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 512, 1024, 8192, 32768, },
  { 4, 16, 32, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 8192, 32768, },
  { 8, 16, 32, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 8192, 32768, },
  { 64, 512, 1024, 8192, 32768, },
  { 1, 64, 512, 1024, 8192, 32768, },
  { 2, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 64, 512, 1024, 8192, 32768, },
  { 4, 64, 512, 1024, 8192, 32768, },
  { 1, 4, 64, 512, 1024, 8192, 32768, },
  { 2, 4, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 64, 512, 1024, 8192, 32768, },
  { 8, 64, 512, 1024, 8192, 32768, },
  { 1, 8, 64, 512, 1024, 8192, 32768, },
  { 2, 8, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 64, 512, 1024, 8192, 32768, },
  { 4, 8, 64, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 64, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 8192, 32768, },
  { 16, 64, 512, 1024, 8192, 32768, },
  { 1, 16, 64, 512, 1024, 8192, 32768, },
  { 2, 16, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 64, 512, 1024, 8192, 32768, },
  { 4, 16, 64, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 64, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 8192, 32768, },
  { 8, 16, 64, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 64, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 64, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 8192, 32768, },
  { 32, 64, 512, 1024, 8192, 32768, },
  { 1, 32, 64, 512, 1024, 8192, 32768, },
  { 2, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 32, 64, 512, 1024, 8192, 32768, },
  { 4, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 4, 32, 64, 512, 1024, 8192, 32768, },
  { 2, 4, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 8192, 32768, },
  { 8, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 8, 32, 64, 512, 1024, 8192, 32768, },
  { 2, 8, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 8192, 32768, },
  { 4, 8, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 8192, 32768, },
  { 16, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 2, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 4, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 8, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 8192, 32768, },
  { 128, 512, 1024, 8192, 32768, },
  { 1, 128, 512, 1024, 8192, 32768, },
  { 2, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 128, 512, 1024, 8192, 32768, },
  { 4, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 128, 512, 1024, 8192, 32768, },
  { 8, 128, 512, 1024, 8192, 32768, },
  { 1, 8, 128, 512, 1024, 8192, 32768, },
  { 2, 8, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 128, 512, 1024, 8192, 32768, },
  { 4, 8, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 8192, 32768, },
  { 16, 128, 512, 1024, 8192, 32768, },
  { 1, 16, 128, 512, 1024, 8192, 32768, },
  { 2, 16, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 128, 512, 1024, 8192, 32768, },
  { 4, 16, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 8192, 32768, },
  { 8, 16, 128, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 128, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 8192, 32768, },
  { 32, 128, 512, 1024, 8192, 32768, },
  { 1, 32, 128, 512, 1024, 8192, 32768, },
  { 2, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 32, 128, 512, 1024, 8192, 32768, },
  { 4, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 32, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 8192, 32768, },
  { 8, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 8, 32, 128, 512, 1024, 8192, 32768, },
  { 2, 8, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 8192, 32768, },
  { 4, 8, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 8192, 32768, },
  { 16, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 2, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 4, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 8, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 8192, 32768, },
  { 64, 128, 512, 1024, 8192, 32768, },
  { 1, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 64, 128, 512, 1024, 8192, 32768, },
  { 4, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 8192, 32768, },
  { 8, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 8, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 8, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 8192, 32768, },
  { 4, 8, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 8192, 32768, },
  { 16, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 4, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 8, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 8192, 32768, },
  { 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 4, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 8, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 32768, },
  { 256, 512, 1024, 8192, 32768, },
  { 1, 256, 512, 1024, 8192, 32768, },
  { 2, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 256, 512, 1024, 8192, 32768, },
  { 4, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 256, 512, 1024, 8192, 32768, },
  { 8, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 8192, 32768, },
  { 16, 256, 512, 1024, 8192, 32768, },
  { 1, 16, 256, 512, 1024, 8192, 32768, },
  { 2, 16, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 256, 512, 1024, 8192, 32768, },
  { 4, 16, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 8192, 32768, },
  { 8, 16, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 8192, 32768, },
  { 32, 256, 512, 1024, 8192, 32768, },
  { 1, 32, 256, 512, 1024, 8192, 32768, },
  { 2, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 32, 256, 512, 1024, 8192, 32768, },
  { 4, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 32, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 8192, 32768, },
  { 8, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 32, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 8192, 32768, },
  { 16, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 2, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 4, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 8, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 8192, 32768, },
  { 64, 256, 512, 1024, 8192, 32768, },
  { 1, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 64, 256, 512, 1024, 8192, 32768, },
  { 4, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 8192, 32768, },
  { 8, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 8192, 32768, },
  { 16, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 4, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 8, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 8192, 32768, },
  { 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 4, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 8, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 32768, },
  { 128, 256, 512, 1024, 8192, 32768, },
  { 1, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 8192, 32768, },
  { 8, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 8192, 32768, },
  { 16, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 8, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 8192, 32768, },
  { 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 8, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 32768, },
  { 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 8, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 32768, },
  { 2048, 8192, 32768, },
  { 1, 2048, 8192, 32768, },
  { 2, 2048, 8192, 32768, },
  { 1, 2, 2048, 8192, 32768, },
  { 4, 2048, 8192, 32768, },
  { 1, 4, 2048, 8192, 32768, },
  { 2, 4, 2048, 8192, 32768, },
  { 1, 2, 4, 2048, 8192, 32768, },
  { 8, 2048, 8192, 32768, },
  { 1, 8, 2048, 8192, 32768, },
  { 2, 8, 2048, 8192, 32768, },
  { 1, 2, 8, 2048, 8192, 32768, },
  { 4, 8, 2048, 8192, 32768, },
  { 1, 4, 8, 2048, 8192, 32768, },
  { 2, 4, 8, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 2048, 8192, 32768, },
  { 16, 2048, 8192, 32768, },
  { 1, 16, 2048, 8192, 32768, },
  { 2, 16, 2048, 8192, 32768, },
  { 1, 2, 16, 2048, 8192, 32768, },
  { 4, 16, 2048, 8192, 32768, },
  { 1, 4, 16, 2048, 8192, 32768, },
  { 2, 4, 16, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 2048, 8192, 32768, },
  { 8, 16, 2048, 8192, 32768, },
  { 1, 8, 16, 2048, 8192, 32768, },
  { 2, 8, 16, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 2048, 8192, 32768, },
  { 4, 8, 16, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 2048, 8192, 32768, },
  { 32, 2048, 8192, 32768, },
  { 1, 32, 2048, 8192, 32768, },
  { 2, 32, 2048, 8192, 32768, },
  { 1, 2, 32, 2048, 8192, 32768, },
  { 4, 32, 2048, 8192, 32768, },
  { 1, 4, 32, 2048, 8192, 32768, },
  { 2, 4, 32, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 2048, 8192, 32768, },
  { 8, 32, 2048, 8192, 32768, },
  { 1, 8, 32, 2048, 8192, 32768, },
  { 2, 8, 32, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 2048, 8192, 32768, },
  { 4, 8, 32, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 2048, 8192, 32768, },
  { 16, 32, 2048, 8192, 32768, },
  { 1, 16, 32, 2048, 8192, 32768, },
  { 2, 16, 32, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 2048, 8192, 32768, },
  { 4, 16, 32, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 2048, 8192, 32768, },
  { 8, 16, 32, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 2048, 8192, 32768, },
  { 64, 2048, 8192, 32768, },
  { 1, 64, 2048, 8192, 32768, },
  { 2, 64, 2048, 8192, 32768, },
  { 1, 2, 64, 2048, 8192, 32768, },
  { 4, 64, 2048, 8192, 32768, },
  { 1, 4, 64, 2048, 8192, 32768, },
  { 2, 4, 64, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 2048, 8192, 32768, },
  { 8, 64, 2048, 8192, 32768, },
  { 1, 8, 64, 2048, 8192, 32768, },
  { 2, 8, 64, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 2048, 8192, 32768, },
  { 4, 8, 64, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 2048, 8192, 32768, },
  { 16, 64, 2048, 8192, 32768, },
  { 1, 16, 64, 2048, 8192, 32768, },
  { 2, 16, 64, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 2048, 8192, 32768, },
  { 4, 16, 64, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 2048, 8192, 32768, },
  { 8, 16, 64, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 2048, 8192, 32768, },
  { 32, 64, 2048, 8192, 32768, },
  { 1, 32, 64, 2048, 8192, 32768, },
  { 2, 32, 64, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 2048, 8192, 32768, },
  { 4, 32, 64, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 2048, 8192, 32768, },
  { 8, 32, 64, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 2048, 8192, 32768, },
  { 16, 32, 64, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 8192, 32768, },
  { 128, 2048, 8192, 32768, },
  { 1, 128, 2048, 8192, 32768, },
  { 2, 128, 2048, 8192, 32768, },
  { 1, 2, 128, 2048, 8192, 32768, },
  { 4, 128, 2048, 8192, 32768, },
  { 1, 4, 128, 2048, 8192, 32768, },
  { 2, 4, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 128, 2048, 8192, 32768, },
  { 8, 128, 2048, 8192, 32768, },
  { 1, 8, 128, 2048, 8192, 32768, },
  { 2, 8, 128, 2048, 8192, 32768, },
  { 1, 2, 8, 128, 2048, 8192, 32768, },
  { 4, 8, 128, 2048, 8192, 32768, },
  { 1, 4, 8, 128, 2048, 8192, 32768, },
  { 2, 4, 8, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 128, 2048, 8192, 32768, },
  { 16, 128, 2048, 8192, 32768, },
  { 1, 16, 128, 2048, 8192, 32768, },
  { 2, 16, 128, 2048, 8192, 32768, },
  { 1, 2, 16, 128, 2048, 8192, 32768, },
  { 4, 16, 128, 2048, 8192, 32768, },
  { 1, 4, 16, 128, 2048, 8192, 32768, },
  { 2, 4, 16, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 128, 2048, 8192, 32768, },
  { 8, 16, 128, 2048, 8192, 32768, },
  { 1, 8, 16, 128, 2048, 8192, 32768, },
  { 2, 8, 16, 128, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 128, 2048, 8192, 32768, },
  { 4, 8, 16, 128, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 128, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 2048, 8192, 32768, },
  { 32, 128, 2048, 8192, 32768, },
  { 1, 32, 128, 2048, 8192, 32768, },
  { 2, 32, 128, 2048, 8192, 32768, },
  { 1, 2, 32, 128, 2048, 8192, 32768, },
  { 4, 32, 128, 2048, 8192, 32768, },
  { 1, 4, 32, 128, 2048, 8192, 32768, },
  { 2, 4, 32, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 128, 2048, 8192, 32768, },
  { 8, 32, 128, 2048, 8192, 32768, },
  { 1, 8, 32, 128, 2048, 8192, 32768, },
  { 2, 8, 32, 128, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 128, 2048, 8192, 32768, },
  { 4, 8, 32, 128, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 128, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 2048, 8192, 32768, },
  { 16, 32, 128, 2048, 8192, 32768, },
  { 1, 16, 32, 128, 2048, 8192, 32768, },
  { 2, 16, 32, 128, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 128, 2048, 8192, 32768, },
  { 4, 16, 32, 128, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 128, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 2048, 8192, 32768, },
  { 8, 16, 32, 128, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 128, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 128, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 128, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 8192, 32768, },
  { 64, 128, 2048, 8192, 32768, },
  { 1, 64, 128, 2048, 8192, 32768, },
  { 2, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 64, 128, 2048, 8192, 32768, },
  { 4, 64, 128, 2048, 8192, 32768, },
  { 1, 4, 64, 128, 2048, 8192, 32768, },
  { 2, 4, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 128, 2048, 8192, 32768, },
  { 8, 64, 128, 2048, 8192, 32768, },
  { 1, 8, 64, 128, 2048, 8192, 32768, },
  { 2, 8, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 128, 2048, 8192, 32768, },
  { 4, 8, 64, 128, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 128, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 2048, 8192, 32768, },
  { 16, 64, 128, 2048, 8192, 32768, },
  { 1, 16, 64, 128, 2048, 8192, 32768, },
  { 2, 16, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 128, 2048, 8192, 32768, },
  { 4, 16, 64, 128, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 128, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 2048, 8192, 32768, },
  { 8, 16, 64, 128, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 128, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 128, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 8192, 32768, },
  { 32, 64, 128, 2048, 8192, 32768, },
  { 1, 32, 64, 128, 2048, 8192, 32768, },
  { 2, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 128, 2048, 8192, 32768, },
  { 4, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 128, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 2048, 8192, 32768, },
  { 8, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 128, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 8192, 32768, },
  { 16, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 8192, 32768, },
  { 256, 2048, 8192, 32768, },
  { 1, 256, 2048, 8192, 32768, },
  { 2, 256, 2048, 8192, 32768, },
  { 1, 2, 256, 2048, 8192, 32768, },
  { 4, 256, 2048, 8192, 32768, },
  { 1, 4, 256, 2048, 8192, 32768, },
  { 2, 4, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 256, 2048, 8192, 32768, },
  { 8, 256, 2048, 8192, 32768, },
  { 1, 8, 256, 2048, 8192, 32768, },
  { 2, 8, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 256, 2048, 8192, 32768, },
  { 4, 8, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 256, 2048, 8192, 32768, },
  { 16, 256, 2048, 8192, 32768, },
  { 1, 16, 256, 2048, 8192, 32768, },
  { 2, 16, 256, 2048, 8192, 32768, },
  { 1, 2, 16, 256, 2048, 8192, 32768, },
  { 4, 16, 256, 2048, 8192, 32768, },
  { 1, 4, 16, 256, 2048, 8192, 32768, },
  { 2, 4, 16, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 256, 2048, 8192, 32768, },
  { 8, 16, 256, 2048, 8192, 32768, },
  { 1, 8, 16, 256, 2048, 8192, 32768, },
  { 2, 8, 16, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 256, 2048, 8192, 32768, },
  { 4, 8, 16, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 2048, 8192, 32768, },
  { 32, 256, 2048, 8192, 32768, },
  { 1, 32, 256, 2048, 8192, 32768, },
  { 2, 32, 256, 2048, 8192, 32768, },
  { 1, 2, 32, 256, 2048, 8192, 32768, },
  { 4, 32, 256, 2048, 8192, 32768, },
  { 1, 4, 32, 256, 2048, 8192, 32768, },
  { 2, 4, 32, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 256, 2048, 8192, 32768, },
  { 8, 32, 256, 2048, 8192, 32768, },
  { 1, 8, 32, 256, 2048, 8192, 32768, },
  { 2, 8, 32, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 256, 2048, 8192, 32768, },
  { 4, 8, 32, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 2048, 8192, 32768, },
  { 16, 32, 256, 2048, 8192, 32768, },
  { 1, 16, 32, 256, 2048, 8192, 32768, },
  { 2, 16, 32, 256, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 256, 2048, 8192, 32768, },
  { 4, 16, 32, 256, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 256, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 2048, 8192, 32768, },
  { 8, 16, 32, 256, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 256, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 8192, 32768, },
  { 64, 256, 2048, 8192, 32768, },
  { 1, 64, 256, 2048, 8192, 32768, },
  { 2, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 64, 256, 2048, 8192, 32768, },
  { 4, 64, 256, 2048, 8192, 32768, },
  { 1, 4, 64, 256, 2048, 8192, 32768, },
  { 2, 4, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 256, 2048, 8192, 32768, },
  { 8, 64, 256, 2048, 8192, 32768, },
  { 1, 8, 64, 256, 2048, 8192, 32768, },
  { 2, 8, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 256, 2048, 8192, 32768, },
  { 4, 8, 64, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 2048, 8192, 32768, },
  { 16, 64, 256, 2048, 8192, 32768, },
  { 1, 16, 64, 256, 2048, 8192, 32768, },
  { 2, 16, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 256, 2048, 8192, 32768, },
  { 4, 16, 64, 256, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 256, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 2048, 8192, 32768, },
  { 8, 16, 64, 256, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 256, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 8192, 32768, },
  { 32, 64, 256, 2048, 8192, 32768, },
  { 1, 32, 64, 256, 2048, 8192, 32768, },
  { 2, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 256, 2048, 8192, 32768, },
  { 4, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 256, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 2048, 8192, 32768, },
  { 8, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 256, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 8192, 32768, },
  { 16, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 8192, 32768, },
  { 128, 256, 2048, 8192, 32768, },
  { 1, 128, 256, 2048, 8192, 32768, },
  { 2, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 128, 256, 2048, 8192, 32768, },
  { 4, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 128, 256, 2048, 8192, 32768, },
  { 8, 128, 256, 2048, 8192, 32768, },
  { 1, 8, 128, 256, 2048, 8192, 32768, },
  { 2, 8, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 128, 256, 2048, 8192, 32768, },
  { 4, 8, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 2048, 8192, 32768, },
  { 16, 128, 256, 2048, 8192, 32768, },
  { 1, 16, 128, 256, 2048, 8192, 32768, },
  { 2, 16, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 16, 128, 256, 2048, 8192, 32768, },
  { 4, 16, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 16, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 16, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 2048, 8192, 32768, },
  { 8, 16, 128, 256, 2048, 8192, 32768, },
  { 1, 8, 16, 128, 256, 2048, 8192, 32768, },
  { 2, 8, 16, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 2048, 8192, 32768, },
  { 4, 8, 16, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 8192, 32768, },
  { 32, 128, 256, 2048, 8192, 32768, },
  { 1, 32, 128, 256, 2048, 8192, 32768, },
  { 2, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 32, 128, 256, 2048, 8192, 32768, },
  { 4, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 32, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 2048, 8192, 32768, },
  { 8, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 8, 32, 128, 256, 2048, 8192, 32768, },
  { 2, 8, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 2048, 8192, 32768, },
  { 4, 8, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 8192, 32768, },
  { 16, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 2, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 4, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 8, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 8192, 32768, },
  { 64, 128, 256, 2048, 8192, 32768, },
  { 1, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 64, 128, 256, 2048, 8192, 32768, },
  { 4, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 2048, 8192, 32768, },
  { 8, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 8, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 8, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 2048, 8192, 32768, },
  { 4, 8, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 8192, 32768, },
  { 16, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 4, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 8, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 8192, 32768, },
  { 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 4, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 8, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 32768, },
  { 512, 2048, 8192, 32768, },
  { 1, 512, 2048, 8192, 32768, },
  { 2, 512, 2048, 8192, 32768, },
  { 1, 2, 512, 2048, 8192, 32768, },
  { 4, 512, 2048, 8192, 32768, },
  { 1, 4, 512, 2048, 8192, 32768, },
  { 2, 4, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 512, 2048, 8192, 32768, },
  { 8, 512, 2048, 8192, 32768, },
  { 1, 8, 512, 2048, 8192, 32768, },
  { 2, 8, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 512, 2048, 8192, 32768, },
  { 4, 8, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 512, 2048, 8192, 32768, },
  { 16, 512, 2048, 8192, 32768, },
  { 1, 16, 512, 2048, 8192, 32768, },
  { 2, 16, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 512, 2048, 8192, 32768, },
  { 4, 16, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 512, 2048, 8192, 32768, },
  { 8, 16, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 512, 2048, 8192, 32768, },
  { 32, 512, 2048, 8192, 32768, },
  { 1, 32, 512, 2048, 8192, 32768, },
  { 2, 32, 512, 2048, 8192, 32768, },
  { 1, 2, 32, 512, 2048, 8192, 32768, },
  { 4, 32, 512, 2048, 8192, 32768, },
  { 1, 4, 32, 512, 2048, 8192, 32768, },
  { 2, 4, 32, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 512, 2048, 8192, 32768, },
  { 8, 32, 512, 2048, 8192, 32768, },
  { 1, 8, 32, 512, 2048, 8192, 32768, },
  { 2, 8, 32, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 512, 2048, 8192, 32768, },
  { 4, 8, 32, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 512, 2048, 8192, 32768, },
  { 16, 32, 512, 2048, 8192, 32768, },
  { 1, 16, 32, 512, 2048, 8192, 32768, },
  { 2, 16, 32, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 512, 2048, 8192, 32768, },
  { 4, 16, 32, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 512, 2048, 8192, 32768, },
  { 8, 16, 32, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 8192, 32768, },
  { 64, 512, 2048, 8192, 32768, },
  { 1, 64, 512, 2048, 8192, 32768, },
  { 2, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 64, 512, 2048, 8192, 32768, },
  { 4, 64, 512, 2048, 8192, 32768, },
  { 1, 4, 64, 512, 2048, 8192, 32768, },
  { 2, 4, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 512, 2048, 8192, 32768, },
  { 8, 64, 512, 2048, 8192, 32768, },
  { 1, 8, 64, 512, 2048, 8192, 32768, },
  { 2, 8, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 512, 2048, 8192, 32768, },
  { 4, 8, 64, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 512, 2048, 8192, 32768, },
  { 16, 64, 512, 2048, 8192, 32768, },
  { 1, 16, 64, 512, 2048, 8192, 32768, },
  { 2, 16, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 512, 2048, 8192, 32768, },
  { 4, 16, 64, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 512, 2048, 8192, 32768, },
  { 8, 16, 64, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 8192, 32768, },
  { 32, 64, 512, 2048, 8192, 32768, },
  { 1, 32, 64, 512, 2048, 8192, 32768, },
  { 2, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 512, 2048, 8192, 32768, },
  { 4, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 512, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 512, 2048, 8192, 32768, },
  { 8, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 512, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 512, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 8192, 32768, },
  { 16, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 8192, 32768, },
  { 128, 512, 2048, 8192, 32768, },
  { 1, 128, 512, 2048, 8192, 32768, },
  { 2, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 128, 512, 2048, 8192, 32768, },
  { 4, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 128, 512, 2048, 8192, 32768, },
  { 8, 128, 512, 2048, 8192, 32768, },
  { 1, 8, 128, 512, 2048, 8192, 32768, },
  { 2, 8, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 128, 512, 2048, 8192, 32768, },
  { 4, 8, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 128, 512, 2048, 8192, 32768, },
  { 16, 128, 512, 2048, 8192, 32768, },
  { 1, 16, 128, 512, 2048, 8192, 32768, },
  { 2, 16, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 128, 512, 2048, 8192, 32768, },
  { 4, 16, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 128, 512, 2048, 8192, 32768, },
  { 8, 16, 128, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 128, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 128, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 8192, 32768, },
  { 32, 128, 512, 2048, 8192, 32768, },
  { 1, 32, 128, 512, 2048, 8192, 32768, },
  { 2, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 32, 128, 512, 2048, 8192, 32768, },
  { 4, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 32, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 128, 512, 2048, 8192, 32768, },
  { 8, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 8, 32, 128, 512, 2048, 8192, 32768, },
  { 2, 8, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 128, 512, 2048, 8192, 32768, },
  { 4, 8, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 8192, 32768, },
  { 16, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 2, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 4, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 8, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 8192, 32768, },
  { 64, 128, 512, 2048, 8192, 32768, },
  { 1, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 64, 128, 512, 2048, 8192, 32768, },
  { 4, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 128, 512, 2048, 8192, 32768, },
  { 8, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 8, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 8, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 128, 512, 2048, 8192, 32768, },
  { 4, 8, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 8192, 32768, },
  { 16, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 4, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 8, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 8192, 32768, },
  { 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 4, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 8, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 32768, },
  { 256, 512, 2048, 8192, 32768, },
  { 1, 256, 512, 2048, 8192, 32768, },
  { 2, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 256, 512, 2048, 8192, 32768, },
  { 4, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 256, 512, 2048, 8192, 32768, },
  { 8, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 256, 512, 2048, 8192, 32768, },
  { 16, 256, 512, 2048, 8192, 32768, },
  { 1, 16, 256, 512, 2048, 8192, 32768, },
  { 2, 16, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 256, 512, 2048, 8192, 32768, },
  { 4, 16, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 256, 512, 2048, 8192, 32768, },
  { 8, 16, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 8192, 32768, },
  { 32, 256, 512, 2048, 8192, 32768, },
  { 1, 32, 256, 512, 2048, 8192, 32768, },
  { 2, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 32, 256, 512, 2048, 8192, 32768, },
  { 4, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 32, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 256, 512, 2048, 8192, 32768, },
  { 8, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 32, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 8192, 32768, },
  { 16, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 2, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 4, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 8, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 8192, 32768, },
  { 64, 256, 512, 2048, 8192, 32768, },
  { 1, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 64, 256, 512, 2048, 8192, 32768, },
  { 4, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 256, 512, 2048, 8192, 32768, },
  { 8, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 8192, 32768, },
  { 16, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 4, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 8, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 8192, 32768, },
  { 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 4, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 8, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 32768, },
  { 128, 256, 512, 2048, 8192, 32768, },
  { 1, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 128, 256, 512, 2048, 8192, 32768, },
  { 8, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 8192, 32768, },
  { 16, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 8, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 8192, 32768, },
  { 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 8, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 8, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 32768, },
  { 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 8, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 8, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 8, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 32768, },
  { 1024, 2048, 8192, 32768, },
  { 1, 1024, 2048, 8192, 32768, },
  { 2, 1024, 2048, 8192, 32768, },
  { 1, 2, 1024, 2048, 8192, 32768, },
  { 4, 1024, 2048, 8192, 32768, },
  { 1, 4, 1024, 2048, 8192, 32768, },
  { 2, 4, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 1024, 2048, 8192, 32768, },
  { 8, 1024, 2048, 8192, 32768, },
  { 1, 8, 1024, 2048, 8192, 32768, },
  { 2, 8, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 1024, 2048, 8192, 32768, },
  { 4, 8, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 1024, 2048, 8192, 32768, },
  { 16, 1024, 2048, 8192, 32768, },
  { 1, 16, 1024, 2048, 8192, 32768, },
  { 2, 16, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 1024, 2048, 8192, 32768, },
  { 4, 16, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 1024, 2048, 8192, 32768, },
  { 8, 16, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 1024, 2048, 8192, 32768, },
  { 32, 1024, 2048, 8192, 32768, },
  { 1, 32, 1024, 2048, 8192, 32768, },
  { 2, 32, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 1024, 2048, 8192, 32768, },
  { 4, 32, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 1024, 2048, 8192, 32768, },
  { 8, 32, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 1024, 2048, 8192, 32768, },
  { 16, 32, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 8192, 32768, },
  { 64, 1024, 2048, 8192, 32768, },
  { 1, 64, 1024, 2048, 8192, 32768, },
  { 2, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 64, 1024, 2048, 8192, 32768, },
  { 4, 64, 1024, 2048, 8192, 32768, },
  { 1, 4, 64, 1024, 2048, 8192, 32768, },
  { 2, 4, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 1024, 2048, 8192, 32768, },
  { 8, 64, 1024, 2048, 8192, 32768, },
  { 1, 8, 64, 1024, 2048, 8192, 32768, },
  { 2, 8, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 1024, 2048, 8192, 32768, },
  { 4, 8, 64, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 1024, 2048, 8192, 32768, },
  { 16, 64, 1024, 2048, 8192, 32768, },
  { 1, 16, 64, 1024, 2048, 8192, 32768, },
  { 2, 16, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 1024, 2048, 8192, 32768, },
  { 4, 16, 64, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 1024, 2048, 8192, 32768, },
  { 8, 16, 64, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 8192, 32768, },
  { 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 32, 64, 1024, 2048, 8192, 32768, },
  { 2, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 1024, 2048, 8192, 32768, },
  { 4, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 1024, 2048, 8192, 32768, },
  { 8, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 8192, 32768, },
  { 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 8192, 32768, },
  { 128, 1024, 2048, 8192, 32768, },
  { 1, 128, 1024, 2048, 8192, 32768, },
  { 2, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 128, 1024, 2048, 8192, 32768, },
  { 4, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 128, 1024, 2048, 8192, 32768, },
  { 8, 128, 1024, 2048, 8192, 32768, },
  { 1, 8, 128, 1024, 2048, 8192, 32768, },
  { 2, 8, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 128, 1024, 2048, 8192, 32768, },
  { 4, 8, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 128, 1024, 2048, 8192, 32768, },
  { 16, 128, 1024, 2048, 8192, 32768, },
  { 1, 16, 128, 1024, 2048, 8192, 32768, },
  { 2, 16, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 128, 1024, 2048, 8192, 32768, },
  { 4, 16, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 128, 1024, 2048, 8192, 32768, },
  { 8, 16, 128, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 128, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 128, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 8192, 32768, },
  { 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 32, 128, 1024, 2048, 8192, 32768, },
  { 2, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 128, 1024, 2048, 8192, 32768, },
  { 4, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 128, 1024, 2048, 8192, 32768, },
  { 8, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 128, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 128, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 8192, 32768, },
  { 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 8192, 32768, },
  { 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 64, 128, 1024, 2048, 8192, 32768, },
  { 4, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 128, 1024, 2048, 8192, 32768, },
  { 8, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 8, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 8, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 128, 1024, 2048, 8192, 32768, },
  { 4, 8, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 8192, 32768, },
  { 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 4, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 8, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 8192, 32768, },
  { 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 4, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 8, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 32768, },
  { 256, 1024, 2048, 8192, 32768, },
  { 1, 256, 1024, 2048, 8192, 32768, },
  { 2, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 256, 1024, 2048, 8192, 32768, },
  { 4, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 256, 1024, 2048, 8192, 32768, },
  { 8, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 256, 1024, 2048, 8192, 32768, },
  { 16, 256, 1024, 2048, 8192, 32768, },
  { 1, 16, 256, 1024, 2048, 8192, 32768, },
  { 2, 16, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 256, 1024, 2048, 8192, 32768, },
  { 4, 16, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 256, 1024, 2048, 8192, 32768, },
  { 8, 16, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 8192, 32768, },
  { 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 32, 256, 1024, 2048, 8192, 32768, },
  { 2, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 256, 1024, 2048, 8192, 32768, },
  { 4, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 256, 1024, 2048, 8192, 32768, },
  { 8, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 8192, 32768, },
  { 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 8192, 32768, },
  { 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 64, 256, 1024, 2048, 8192, 32768, },
  { 4, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 256, 1024, 2048, 8192, 32768, },
  { 8, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 8192, 32768, },
  { 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 4, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 8, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 8192, 32768, },
  { 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 4, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 8, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 32768, },
  { 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 128, 256, 1024, 2048, 8192, 32768, },
  { 8, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 8192, 32768, },
  { 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 8, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 8192, 32768, },
  { 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 8, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 32768, },
  { 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 8, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 8, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 8, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 32768, },
  { 512, 1024, 2048, 8192, 32768, },
  { 1, 512, 1024, 2048, 8192, 32768, },
  { 2, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 512, 1024, 2048, 8192, 32768, },
  { 4, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 512, 1024, 2048, 8192, 32768, },
  { 8, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 512, 1024, 2048, 8192, 32768, },
  { 16, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 8192, 32768, },
  { 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 32, 512, 1024, 2048, 8192, 32768, },
  { 2, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 512, 1024, 2048, 8192, 32768, },
  { 4, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 512, 1024, 2048, 8192, 32768, },
  { 8, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 8192, 32768, },
  { 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 8192, 32768, },
  { 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 64, 512, 1024, 2048, 8192, 32768, },
  { 4, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 512, 1024, 2048, 8192, 32768, },
  { 8, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 8192, 32768, },
  { 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 8192, 32768, },
  { 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 4, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 8, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 32768, },
  { 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 128, 512, 1024, 2048, 8192, 32768, },
  { 8, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 8192, 32768, },
  { 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 8192, 32768, },
  { 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 8, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 32768, },
  { 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 8, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 8, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 32768, },
  { 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 8192, 32768, },
  { 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 8192, 32768, },
  { 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 32768, },
  { 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 32768, },
  { 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 32768, },
  { 4096, 8192, 32768, },
  { 1, 4096, 8192, 32768, },
  { 2, 4096, 8192, 32768, },
  { 1, 2, 4096, 8192, 32768, },
  { 4, 4096, 8192, 32768, },
  { 1, 4, 4096, 8192, 32768, },
  { 2, 4, 4096, 8192, 32768, },
  { 1, 2, 4, 4096, 8192, 32768, },
  { 8, 4096, 8192, 32768, },
  { 1, 8, 4096, 8192, 32768, },
  { 2, 8, 4096, 8192, 32768, },
  { 1, 2, 8, 4096, 8192, 32768, },
  { 4, 8, 4096, 8192, 32768, },
  { 1, 4, 8, 4096, 8192, 32768, },
  { 2, 4, 8, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 4096, 8192, 32768, },
  { 16, 4096, 8192, 32768, },
  { 1, 16, 4096, 8192, 32768, },
  { 2, 16, 4096, 8192, 32768, },
  { 1, 2, 16, 4096, 8192, 32768, },
  { 4, 16, 4096, 8192, 32768, },
  { 1, 4, 16, 4096, 8192, 32768, },
  { 2, 4, 16, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 4096, 8192, 32768, },
  { 8, 16, 4096, 8192, 32768, },
  { 1, 8, 16, 4096, 8192, 32768, },
  { 2, 8, 16, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 4096, 8192, 32768, },
  { 4, 8, 16, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 4096, 8192, 32768, },
  { 32, 4096, 8192, 32768, },
  { 1, 32, 4096, 8192, 32768, },
  { 2, 32, 4096, 8192, 32768, },
  { 1, 2, 32, 4096, 8192, 32768, },
  { 4, 32, 4096, 8192, 32768, },
  { 1, 4, 32, 4096, 8192, 32768, },
  { 2, 4, 32, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 4096, 8192, 32768, },
  { 8, 32, 4096, 8192, 32768, },
  { 1, 8, 32, 4096, 8192, 32768, },
  { 2, 8, 32, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 4096, 8192, 32768, },
  { 4, 8, 32, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 4096, 8192, 32768, },
  { 16, 32, 4096, 8192, 32768, },
  { 1, 16, 32, 4096, 8192, 32768, },
  { 2, 16, 32, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 4096, 8192, 32768, },
  { 4, 16, 32, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 4096, 8192, 32768, },
  { 8, 16, 32, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 4096, 8192, 32768, },
  { 64, 4096, 8192, 32768, },
  { 1, 64, 4096, 8192, 32768, },
  { 2, 64, 4096, 8192, 32768, },
  { 1, 2, 64, 4096, 8192, 32768, },
  { 4, 64, 4096, 8192, 32768, },
  { 1, 4, 64, 4096, 8192, 32768, },
  { 2, 4, 64, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 4096, 8192, 32768, },
  { 8, 64, 4096, 8192, 32768, },
  { 1, 8, 64, 4096, 8192, 32768, },
  { 2, 8, 64, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 4096, 8192, 32768, },
  { 4, 8, 64, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 4096, 8192, 32768, },
  { 16, 64, 4096, 8192, 32768, },
  { 1, 16, 64, 4096, 8192, 32768, },
  { 2, 16, 64, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 4096, 8192, 32768, },
  { 4, 16, 64, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 4096, 8192, 32768, },
  { 8, 16, 64, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 4096, 8192, 32768, },
  { 32, 64, 4096, 8192, 32768, },
  { 1, 32, 64, 4096, 8192, 32768, },
  { 2, 32, 64, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 4096, 8192, 32768, },
  { 4, 32, 64, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 4096, 8192, 32768, },
  { 8, 32, 64, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 4096, 8192, 32768, },
  { 16, 32, 64, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 4096, 8192, 32768, },
  { 128, 4096, 8192, 32768, },
  { 1, 128, 4096, 8192, 32768, },
  { 2, 128, 4096, 8192, 32768, },
  { 1, 2, 128, 4096, 8192, 32768, },
  { 4, 128, 4096, 8192, 32768, },
  { 1, 4, 128, 4096, 8192, 32768, },
  { 2, 4, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 4096, 8192, 32768, },
  { 8, 128, 4096, 8192, 32768, },
  { 1, 8, 128, 4096, 8192, 32768, },
  { 2, 8, 128, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 4096, 8192, 32768, },
  { 4, 8, 128, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 4096, 8192, 32768, },
  { 16, 128, 4096, 8192, 32768, },
  { 1, 16, 128, 4096, 8192, 32768, },
  { 2, 16, 128, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 4096, 8192, 32768, },
  { 4, 16, 128, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 4096, 8192, 32768, },
  { 8, 16, 128, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 4096, 8192, 32768, },
  { 32, 128, 4096, 8192, 32768, },
  { 1, 32, 128, 4096, 8192, 32768, },
  { 2, 32, 128, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 4096, 8192, 32768, },
  { 4, 32, 128, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 4096, 8192, 32768, },
  { 8, 32, 128, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 4096, 8192, 32768, },
  { 16, 32, 128, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 4096, 8192, 32768, },
  { 64, 128, 4096, 8192, 32768, },
  { 1, 64, 128, 4096, 8192, 32768, },
  { 2, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 4096, 8192, 32768, },
  { 4, 64, 128, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 4096, 8192, 32768, },
  { 8, 64, 128, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 4096, 8192, 32768, },
  { 16, 64, 128, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 4096, 8192, 32768, },
  { 32, 64, 128, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 4096, 8192, 32768, },
  { 256, 4096, 8192, 32768, },
  { 1, 256, 4096, 8192, 32768, },
  { 2, 256, 4096, 8192, 32768, },
  { 1, 2, 256, 4096, 8192, 32768, },
  { 4, 256, 4096, 8192, 32768, },
  { 1, 4, 256, 4096, 8192, 32768, },
  { 2, 4, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 256, 4096, 8192, 32768, },
  { 8, 256, 4096, 8192, 32768, },
  { 1, 8, 256, 4096, 8192, 32768, },
  { 2, 8, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 256, 4096, 8192, 32768, },
  { 4, 8, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 256, 4096, 8192, 32768, },
  { 16, 256, 4096, 8192, 32768, },
  { 1, 16, 256, 4096, 8192, 32768, },
  { 2, 16, 256, 4096, 8192, 32768, },
  { 1, 2, 16, 256, 4096, 8192, 32768, },
  { 4, 16, 256, 4096, 8192, 32768, },
  { 1, 4, 16, 256, 4096, 8192, 32768, },
  { 2, 4, 16, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 256, 4096, 8192, 32768, },
  { 8, 16, 256, 4096, 8192, 32768, },
  { 1, 8, 16, 256, 4096, 8192, 32768, },
  { 2, 8, 16, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 256, 4096, 8192, 32768, },
  { 4, 8, 16, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 4096, 8192, 32768, },
  { 32, 256, 4096, 8192, 32768, },
  { 1, 32, 256, 4096, 8192, 32768, },
  { 2, 32, 256, 4096, 8192, 32768, },
  { 1, 2, 32, 256, 4096, 8192, 32768, },
  { 4, 32, 256, 4096, 8192, 32768, },
  { 1, 4, 32, 256, 4096, 8192, 32768, },
  { 2, 4, 32, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 256, 4096, 8192, 32768, },
  { 8, 32, 256, 4096, 8192, 32768, },
  { 1, 8, 32, 256, 4096, 8192, 32768, },
  { 2, 8, 32, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 256, 4096, 8192, 32768, },
  { 4, 8, 32, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 4096, 8192, 32768, },
  { 16, 32, 256, 4096, 8192, 32768, },
  { 1, 16, 32, 256, 4096, 8192, 32768, },
  { 2, 16, 32, 256, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 256, 4096, 8192, 32768, },
  { 4, 16, 32, 256, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 256, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 4096, 8192, 32768, },
  { 8, 16, 32, 256, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 256, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 4096, 8192, 32768, },
  { 64, 256, 4096, 8192, 32768, },
  { 1, 64, 256, 4096, 8192, 32768, },
  { 2, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 64, 256, 4096, 8192, 32768, },
  { 4, 64, 256, 4096, 8192, 32768, },
  { 1, 4, 64, 256, 4096, 8192, 32768, },
  { 2, 4, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 256, 4096, 8192, 32768, },
  { 8, 64, 256, 4096, 8192, 32768, },
  { 1, 8, 64, 256, 4096, 8192, 32768, },
  { 2, 8, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 256, 4096, 8192, 32768, },
  { 4, 8, 64, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 4096, 8192, 32768, },
  { 16, 64, 256, 4096, 8192, 32768, },
  { 1, 16, 64, 256, 4096, 8192, 32768, },
  { 2, 16, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 256, 4096, 8192, 32768, },
  { 4, 16, 64, 256, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 256, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 4096, 8192, 32768, },
  { 8, 16, 64, 256, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 256, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 4096, 8192, 32768, },
  { 32, 64, 256, 4096, 8192, 32768, },
  { 1, 32, 64, 256, 4096, 8192, 32768, },
  { 2, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 256, 4096, 8192, 32768, },
  { 4, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 256, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 4096, 8192, 32768, },
  { 8, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 256, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 4096, 8192, 32768, },
  { 16, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 4096, 8192, 32768, },
  { 128, 256, 4096, 8192, 32768, },
  { 1, 128, 256, 4096, 8192, 32768, },
  { 2, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 128, 256, 4096, 8192, 32768, },
  { 4, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 256, 4096, 8192, 32768, },
  { 8, 128, 256, 4096, 8192, 32768, },
  { 1, 8, 128, 256, 4096, 8192, 32768, },
  { 2, 8, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 256, 4096, 8192, 32768, },
  { 4, 8, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 4096, 8192, 32768, },
  { 16, 128, 256, 4096, 8192, 32768, },
  { 1, 16, 128, 256, 4096, 8192, 32768, },
  { 2, 16, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 256, 4096, 8192, 32768, },
  { 4, 16, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 4096, 8192, 32768, },
  { 8, 16, 128, 256, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 256, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 4096, 8192, 32768, },
  { 32, 128, 256, 4096, 8192, 32768, },
  { 1, 32, 128, 256, 4096, 8192, 32768, },
  { 2, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 256, 4096, 8192, 32768, },
  { 4, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 4096, 8192, 32768, },
  { 8, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 256, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 4096, 8192, 32768, },
  { 16, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 4096, 8192, 32768, },
  { 64, 128, 256, 4096, 8192, 32768, },
  { 1, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 256, 4096, 8192, 32768, },
  { 4, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 4096, 8192, 32768, },
  { 8, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 4096, 8192, 32768, },
  { 16, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 4096, 8192, 32768, },
  { 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 32768, },
  { 512, 4096, 8192, 32768, },
  { 1, 512, 4096, 8192, 32768, },
  { 2, 512, 4096, 8192, 32768, },
  { 1, 2, 512, 4096, 8192, 32768, },
  { 4, 512, 4096, 8192, 32768, },
  { 1, 4, 512, 4096, 8192, 32768, },
  { 2, 4, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 512, 4096, 8192, 32768, },
  { 8, 512, 4096, 8192, 32768, },
  { 1, 8, 512, 4096, 8192, 32768, },
  { 2, 8, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 512, 4096, 8192, 32768, },
  { 4, 8, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 512, 4096, 8192, 32768, },
  { 16, 512, 4096, 8192, 32768, },
  { 1, 16, 512, 4096, 8192, 32768, },
  { 2, 16, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 512, 4096, 8192, 32768, },
  { 4, 16, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 512, 4096, 8192, 32768, },
  { 8, 16, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 512, 4096, 8192, 32768, },
  { 32, 512, 4096, 8192, 32768, },
  { 1, 32, 512, 4096, 8192, 32768, },
  { 2, 32, 512, 4096, 8192, 32768, },
  { 1, 2, 32, 512, 4096, 8192, 32768, },
  { 4, 32, 512, 4096, 8192, 32768, },
  { 1, 4, 32, 512, 4096, 8192, 32768, },
  { 2, 4, 32, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 512, 4096, 8192, 32768, },
  { 8, 32, 512, 4096, 8192, 32768, },
  { 1, 8, 32, 512, 4096, 8192, 32768, },
  { 2, 8, 32, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 512, 4096, 8192, 32768, },
  { 4, 8, 32, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 512, 4096, 8192, 32768, },
  { 16, 32, 512, 4096, 8192, 32768, },
  { 1, 16, 32, 512, 4096, 8192, 32768, },
  { 2, 16, 32, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 512, 4096, 8192, 32768, },
  { 4, 16, 32, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 512, 4096, 8192, 32768, },
  { 8, 16, 32, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 4096, 8192, 32768, },
  { 64, 512, 4096, 8192, 32768, },
  { 1, 64, 512, 4096, 8192, 32768, },
  { 2, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 64, 512, 4096, 8192, 32768, },
  { 4, 64, 512, 4096, 8192, 32768, },
  { 1, 4, 64, 512, 4096, 8192, 32768, },
  { 2, 4, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 512, 4096, 8192, 32768, },
  { 8, 64, 512, 4096, 8192, 32768, },
  { 1, 8, 64, 512, 4096, 8192, 32768, },
  { 2, 8, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 512, 4096, 8192, 32768, },
  { 4, 8, 64, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 512, 4096, 8192, 32768, },
  { 16, 64, 512, 4096, 8192, 32768, },
  { 1, 16, 64, 512, 4096, 8192, 32768, },
  { 2, 16, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 512, 4096, 8192, 32768, },
  { 4, 16, 64, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 512, 4096, 8192, 32768, },
  { 8, 16, 64, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 4096, 8192, 32768, },
  { 32, 64, 512, 4096, 8192, 32768, },
  { 1, 32, 64, 512, 4096, 8192, 32768, },
  { 2, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 512, 4096, 8192, 32768, },
  { 4, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 512, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 512, 4096, 8192, 32768, },
  { 8, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 512, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 512, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 4096, 8192, 32768, },
  { 16, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 4096, 8192, 32768, },
  { 128, 512, 4096, 8192, 32768, },
  { 1, 128, 512, 4096, 8192, 32768, },
  { 2, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 128, 512, 4096, 8192, 32768, },
  { 4, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 512, 4096, 8192, 32768, },
  { 8, 128, 512, 4096, 8192, 32768, },
  { 1, 8, 128, 512, 4096, 8192, 32768, },
  { 2, 8, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 512, 4096, 8192, 32768, },
  { 4, 8, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 512, 4096, 8192, 32768, },
  { 16, 128, 512, 4096, 8192, 32768, },
  { 1, 16, 128, 512, 4096, 8192, 32768, },
  { 2, 16, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 512, 4096, 8192, 32768, },
  { 4, 16, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 512, 4096, 8192, 32768, },
  { 8, 16, 128, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 4096, 8192, 32768, },
  { 32, 128, 512, 4096, 8192, 32768, },
  { 1, 32, 128, 512, 4096, 8192, 32768, },
  { 2, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 512, 4096, 8192, 32768, },
  { 4, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 512, 4096, 8192, 32768, },
  { 8, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 512, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 512, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 4096, 8192, 32768, },
  { 16, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 4096, 8192, 32768, },
  { 64, 128, 512, 4096, 8192, 32768, },
  { 1, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 512, 4096, 8192, 32768, },
  { 4, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 512, 4096, 8192, 32768, },
  { 8, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 512, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 4096, 8192, 32768, },
  { 16, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 4096, 8192, 32768, },
  { 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 32768, },
  { 256, 512, 4096, 8192, 32768, },
  { 1, 256, 512, 4096, 8192, 32768, },
  { 2, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 256, 512, 4096, 8192, 32768, },
  { 4, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 256, 512, 4096, 8192, 32768, },
  { 8, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 256, 512, 4096, 8192, 32768, },
  { 16, 256, 512, 4096, 8192, 32768, },
  { 1, 16, 256, 512, 4096, 8192, 32768, },
  { 2, 16, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 256, 512, 4096, 8192, 32768, },
  { 4, 16, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 256, 512, 4096, 8192, 32768, },
  { 8, 16, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 4096, 8192, 32768, },
  { 32, 256, 512, 4096, 8192, 32768, },
  { 1, 32, 256, 512, 4096, 8192, 32768, },
  { 2, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 32, 256, 512, 4096, 8192, 32768, },
  { 4, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 32, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 256, 512, 4096, 8192, 32768, },
  { 8, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 32, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 4096, 8192, 32768, },
  { 16, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 2, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 4, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 8, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 4096, 8192, 32768, },
  { 64, 256, 512, 4096, 8192, 32768, },
  { 1, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 64, 256, 512, 4096, 8192, 32768, },
  { 4, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 256, 512, 4096, 8192, 32768, },
  { 8, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 4096, 8192, 32768, },
  { 16, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 4, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 8, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 4096, 8192, 32768, },
  { 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 4, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 8, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 32768, },
  { 128, 256, 512, 4096, 8192, 32768, },
  { 1, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 256, 512, 4096, 8192, 32768, },
  { 8, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 4096, 8192, 32768, },
  { 16, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 8, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 4096, 8192, 32768, },
  { 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 8, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 32768, },
  { 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 8, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 32768, },
  { 1024, 4096, 8192, 32768, },
  { 1, 1024, 4096, 8192, 32768, },
  { 2, 1024, 4096, 8192, 32768, },
  { 1, 2, 1024, 4096, 8192, 32768, },
  { 4, 1024, 4096, 8192, 32768, },
  { 1, 4, 1024, 4096, 8192, 32768, },
  { 2, 4, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 1024, 4096, 8192, 32768, },
  { 8, 1024, 4096, 8192, 32768, },
  { 1, 8, 1024, 4096, 8192, 32768, },
  { 2, 8, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 1024, 4096, 8192, 32768, },
  { 4, 8, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 1024, 4096, 8192, 32768, },
  { 16, 1024, 4096, 8192, 32768, },
  { 1, 16, 1024, 4096, 8192, 32768, },
  { 2, 16, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 1024, 4096, 8192, 32768, },
  { 4, 16, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 1024, 4096, 8192, 32768, },
  { 8, 16, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 1024, 4096, 8192, 32768, },
  { 32, 1024, 4096, 8192, 32768, },
  { 1, 32, 1024, 4096, 8192, 32768, },
  { 2, 32, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 1024, 4096, 8192, 32768, },
  { 4, 32, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 1024, 4096, 8192, 32768, },
  { 8, 32, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 1024, 4096, 8192, 32768, },
  { 16, 32, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 4096, 8192, 32768, },
  { 64, 1024, 4096, 8192, 32768, },
  { 1, 64, 1024, 4096, 8192, 32768, },
  { 2, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 64, 1024, 4096, 8192, 32768, },
  { 4, 64, 1024, 4096, 8192, 32768, },
  { 1, 4, 64, 1024, 4096, 8192, 32768, },
  { 2, 4, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 1024, 4096, 8192, 32768, },
  { 8, 64, 1024, 4096, 8192, 32768, },
  { 1, 8, 64, 1024, 4096, 8192, 32768, },
  { 2, 8, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 1024, 4096, 8192, 32768, },
  { 4, 8, 64, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 1024, 4096, 8192, 32768, },
  { 16, 64, 1024, 4096, 8192, 32768, },
  { 1, 16, 64, 1024, 4096, 8192, 32768, },
  { 2, 16, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 1024, 4096, 8192, 32768, },
  { 4, 16, 64, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 1024, 4096, 8192, 32768, },
  { 8, 16, 64, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 4096, 8192, 32768, },
  { 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 32, 64, 1024, 4096, 8192, 32768, },
  { 2, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 1024, 4096, 8192, 32768, },
  { 4, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 1024, 4096, 8192, 32768, },
  { 8, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 4096, 8192, 32768, },
  { 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 4096, 8192, 32768, },
  { 128, 1024, 4096, 8192, 32768, },
  { 1, 128, 1024, 4096, 8192, 32768, },
  { 2, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 128, 1024, 4096, 8192, 32768, },
  { 4, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 1024, 4096, 8192, 32768, },
  { 8, 128, 1024, 4096, 8192, 32768, },
  { 1, 8, 128, 1024, 4096, 8192, 32768, },
  { 2, 8, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 1024, 4096, 8192, 32768, },
  { 4, 8, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 1024, 4096, 8192, 32768, },
  { 16, 128, 1024, 4096, 8192, 32768, },
  { 1, 16, 128, 1024, 4096, 8192, 32768, },
  { 2, 16, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 1024, 4096, 8192, 32768, },
  { 4, 16, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 1024, 4096, 8192, 32768, },
  { 8, 16, 128, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 4096, 8192, 32768, },
  { 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 32, 128, 1024, 4096, 8192, 32768, },
  { 2, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 1024, 4096, 8192, 32768, },
  { 4, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 1024, 4096, 8192, 32768, },
  { 8, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 4096, 8192, 32768, },
  { 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 4096, 8192, 32768, },
  { 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 1024, 4096, 8192, 32768, },
  { 4, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 1024, 4096, 8192, 32768, },
  { 8, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 1024, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 4096, 8192, 32768, },
  { 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 4096, 8192, 32768, },
  { 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 32768, },
  { 256, 1024, 4096, 8192, 32768, },
  { 1, 256, 1024, 4096, 8192, 32768, },
  { 2, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 256, 1024, 4096, 8192, 32768, },
  { 4, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 256, 1024, 4096, 8192, 32768, },
  { 8, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 256, 1024, 4096, 8192, 32768, },
  { 16, 256, 1024, 4096, 8192, 32768, },
  { 1, 16, 256, 1024, 4096, 8192, 32768, },
  { 2, 16, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 256, 1024, 4096, 8192, 32768, },
  { 4, 16, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 256, 1024, 4096, 8192, 32768, },
  { 8, 16, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 4096, 8192, 32768, },
  { 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 32, 256, 1024, 4096, 8192, 32768, },
  { 2, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 256, 1024, 4096, 8192, 32768, },
  { 4, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 256, 1024, 4096, 8192, 32768, },
  { 8, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 4096, 8192, 32768, },
  { 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 4096, 8192, 32768, },
  { 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 64, 256, 1024, 4096, 8192, 32768, },
  { 4, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 256, 1024, 4096, 8192, 32768, },
  { 8, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 4096, 8192, 32768, },
  { 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 4, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 8, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 4096, 8192, 32768, },
  { 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 4, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 8, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 32768, },
  { 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 256, 1024, 4096, 8192, 32768, },
  { 8, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 4096, 8192, 32768, },
  { 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 8, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 4096, 8192, 32768, },
  { 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 8, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 32768, },
  { 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 8, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 32768, },
  { 512, 1024, 4096, 8192, 32768, },
  { 1, 512, 1024, 4096, 8192, 32768, },
  { 2, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 512, 1024, 4096, 8192, 32768, },
  { 4, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 512, 1024, 4096, 8192, 32768, },
  { 8, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 512, 1024, 4096, 8192, 32768, },
  { 16, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 4096, 8192, 32768, },
  { 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 32, 512, 1024, 4096, 8192, 32768, },
  { 2, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 512, 1024, 4096, 8192, 32768, },
  { 4, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 512, 1024, 4096, 8192, 32768, },
  { 8, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 4096, 8192, 32768, },
  { 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 4096, 8192, 32768, },
  { 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 64, 512, 1024, 4096, 8192, 32768, },
  { 4, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 512, 1024, 4096, 8192, 32768, },
  { 8, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 4096, 8192, 32768, },
  { 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 4096, 8192, 32768, },
  { 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 4, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 8, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 32768, },
  { 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 512, 1024, 4096, 8192, 32768, },
  { 8, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 4096, 8192, 32768, },
  { 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 4096, 8192, 32768, },
  { 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 8, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 32768, },
  { 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 8, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 32768, },
  { 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 4096, 8192, 32768, },
  { 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 4096, 8192, 32768, },
  { 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 32768, },
  { 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 32768, },
  { 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 32768, },
  { 2048, 4096, 8192, 32768, },
  { 1, 2048, 4096, 8192, 32768, },
  { 2, 2048, 4096, 8192, 32768, },
  { 1, 2, 2048, 4096, 8192, 32768, },
  { 4, 2048, 4096, 8192, 32768, },
  { 1, 4, 2048, 4096, 8192, 32768, },
  { 2, 4, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 2048, 4096, 8192, 32768, },
  { 8, 2048, 4096, 8192, 32768, },
  { 1, 8, 2048, 4096, 8192, 32768, },
  { 2, 8, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 2048, 4096, 8192, 32768, },
  { 4, 8, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 2048, 4096, 8192, 32768, },
  { 16, 2048, 4096, 8192, 32768, },
  { 1, 16, 2048, 4096, 8192, 32768, },
  { 2, 16, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 2048, 4096, 8192, 32768, },
  { 4, 16, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 2048, 4096, 8192, 32768, },
  { 8, 16, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 2048, 4096, 8192, 32768, },
  { 32, 2048, 4096, 8192, 32768, },
  { 1, 32, 2048, 4096, 8192, 32768, },
  { 2, 32, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 2048, 4096, 8192, 32768, },
  { 4, 32, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 2048, 4096, 8192, 32768, },
  { 8, 32, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 2048, 4096, 8192, 32768, },
  { 16, 32, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 2048, 4096, 8192, 32768, },
  { 64, 2048, 4096, 8192, 32768, },
  { 1, 64, 2048, 4096, 8192, 32768, },
  { 2, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 2048, 4096, 8192, 32768, },
  { 4, 64, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 2048, 4096, 8192, 32768, },
  { 8, 64, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 2048, 4096, 8192, 32768, },
  { 16, 64, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 2048, 4096, 8192, 32768, },
  { 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 4096, 8192, 32768, },
  { 128, 2048, 4096, 8192, 32768, },
  { 1, 128, 2048, 4096, 8192, 32768, },
  { 2, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 128, 2048, 4096, 8192, 32768, },
  { 4, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 2048, 4096, 8192, 32768, },
  { 8, 128, 2048, 4096, 8192, 32768, },
  { 1, 8, 128, 2048, 4096, 8192, 32768, },
  { 2, 8, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 2048, 4096, 8192, 32768, },
  { 4, 8, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 2048, 4096, 8192, 32768, },
  { 16, 128, 2048, 4096, 8192, 32768, },
  { 1, 16, 128, 2048, 4096, 8192, 32768, },
  { 2, 16, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 2048, 4096, 8192, 32768, },
  { 4, 16, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 2048, 4096, 8192, 32768, },
  { 8, 16, 128, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 2048, 4096, 8192, 32768, },
  { 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 32, 128, 2048, 4096, 8192, 32768, },
  { 2, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 2048, 4096, 8192, 32768, },
  { 4, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 2048, 4096, 8192, 32768, },
  { 8, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 2048, 4096, 8192, 32768, },
  { 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 4096, 8192, 32768, },
  { 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 2048, 4096, 8192, 32768, },
  { 4, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 2048, 4096, 8192, 32768, },
  { 8, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 2048, 4096, 8192, 32768, },
  { 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 4096, 8192, 32768, },
  { 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 32768, },
  { 256, 2048, 4096, 8192, 32768, },
  { 1, 256, 2048, 4096, 8192, 32768, },
  { 2, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 256, 2048, 4096, 8192, 32768, },
  { 4, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 256, 2048, 4096, 8192, 32768, },
  { 8, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 256, 2048, 4096, 8192, 32768, },
  { 16, 256, 2048, 4096, 8192, 32768, },
  { 1, 16, 256, 2048, 4096, 8192, 32768, },
  { 2, 16, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 256, 2048, 4096, 8192, 32768, },
  { 4, 16, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 256, 2048, 4096, 8192, 32768, },
  { 8, 16, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 2048, 4096, 8192, 32768, },
  { 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 32, 256, 2048, 4096, 8192, 32768, },
  { 2, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 256, 2048, 4096, 8192, 32768, },
  { 4, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 256, 2048, 4096, 8192, 32768, },
  { 8, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 2048, 4096, 8192, 32768, },
  { 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 4096, 8192, 32768, },
  { 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 256, 2048, 4096, 8192, 32768, },
  { 4, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 256, 2048, 4096, 8192, 32768, },
  { 8, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 2048, 4096, 8192, 32768, },
  { 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 4096, 8192, 32768, },
  { 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 32768, },
  { 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 256, 2048, 4096, 8192, 32768, },
  { 8, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 2048, 4096, 8192, 32768, },
  { 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 8, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 4096, 8192, 32768, },
  { 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 8, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 32768, },
  { 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 8, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 32768, },
  { 512, 2048, 4096, 8192, 32768, },
  { 1, 512, 2048, 4096, 8192, 32768, },
  { 2, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 512, 2048, 4096, 8192, 32768, },
  { 4, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 512, 2048, 4096, 8192, 32768, },
  { 8, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 512, 2048, 4096, 8192, 32768, },
  { 16, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 512, 2048, 4096, 8192, 32768, },
  { 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 32, 512, 2048, 4096, 8192, 32768, },
  { 2, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 512, 2048, 4096, 8192, 32768, },
  { 4, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 512, 2048, 4096, 8192, 32768, },
  { 8, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 512, 2048, 4096, 8192, 32768, },
  { 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 4096, 8192, 32768, },
  { 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 512, 2048, 4096, 8192, 32768, },
  { 4, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 512, 2048, 4096, 8192, 32768, },
  { 8, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 512, 2048, 4096, 8192, 32768, },
  { 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 4096, 8192, 32768, },
  { 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 32768, },
  { 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 512, 2048, 4096, 8192, 32768, },
  { 8, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 512, 2048, 4096, 8192, 32768, },
  { 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 4096, 8192, 32768, },
  { 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 8, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 32768, },
  { 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 8, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 32768, },
  { 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 256, 512, 2048, 4096, 8192, 32768, },
  { 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 4096, 8192, 32768, },
  { 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 32768, },
  { 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 32768, },
  { 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 32768, },
  { 1024, 2048, 4096, 8192, 32768, },
  { 1, 1024, 2048, 4096, 8192, 32768, },
  { 2, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 1024, 2048, 4096, 8192, 32768, },
  { 4, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 1024, 2048, 4096, 8192, 32768, },
  { 8, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 1024, 2048, 4096, 8192, 32768, },
  { 16, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 1024, 2048, 4096, 8192, 32768, },
  { 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 32768, },
  { 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 1024, 2048, 4096, 8192, 32768, },
  { 4, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 1024, 2048, 4096, 8192, 32768, },
  { 8, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 1024, 2048, 4096, 8192, 32768, },
  { 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 32768, },
  { 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 32768, },
  { 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 1024, 2048, 4096, 8192, 32768, },
  { 8, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 1024, 2048, 4096, 8192, 32768, },
  { 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 32768, },
  { 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 32768, },
  { 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 8, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 32768, },
  { 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 256, 1024, 2048, 4096, 8192, 32768, },
  { 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 32768, },
  { 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 32768, },
  { 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 32768, },
  { 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 32768, },
  { 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 32768, },
  { 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 32768, },
  { 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 32768, },
  { 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 32768, },
  { 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 32768, },
  { 16384, 32768, },
  { 1, 16384, 32768, },
  { 2, 16384, 32768, },
  { 1, 2, 16384, 32768, },
  { 4, 16384, 32768, },
  { 1, 4, 16384, 32768, },
  { 2, 4, 16384, 32768, },
  { 1, 2, 4, 16384, 32768, },
  { 8, 16384, 32768, },
  { 1, 8, 16384, 32768, },
  { 2, 8, 16384, 32768, },
  { 1, 2, 8, 16384, 32768, },
  { 4, 8, 16384, 32768, },
  { 1, 4, 8, 16384, 32768, },
  { 2, 4, 8, 16384, 32768, },
  { 1, 2, 4, 8, 16384, 32768, },
  { 16, 16384, 32768, },
  { 1, 16, 16384, 32768, },
  { 2, 16, 16384, 32768, },
  { 1, 2, 16, 16384, 32768, },
  { 4, 16, 16384, 32768, },
  { 1, 4, 16, 16384, 32768, },
  { 2, 4, 16, 16384, 32768, },
  { 1, 2, 4, 16, 16384, 32768, },
  { 8, 16, 16384, 32768, },
  { 1, 8, 16, 16384, 32768, },
  { 2, 8, 16, 16384, 32768, },
  { 1, 2, 8, 16, 16384, 32768, },
  { 4, 8, 16, 16384, 32768, },
  { 1, 4, 8, 16, 16384, 32768, },
  { 2, 4, 8, 16, 16384, 32768, },
  { 1, 2, 4, 8, 16, 16384, 32768, },
  { 32, 16384, 32768, },
  { 1, 32, 16384, 32768, },
  { 2, 32, 16384, 32768, },
  { 1, 2, 32, 16384, 32768, },
  { 4, 32, 16384, 32768, },
  { 1, 4, 32, 16384, 32768, },
  { 2, 4, 32, 16384, 32768, },
  { 1, 2, 4, 32, 16384, 32768, },
  { 8, 32, 16384, 32768, },
  { 1, 8, 32, 16384, 32768, },
  { 2, 8, 32, 16384, 32768, },
  { 1, 2, 8, 32, 16384, 32768, },
  { 4, 8, 32, 16384, 32768, },
  { 1, 4, 8, 32, 16384, 32768, },
  { 2, 4, 8, 32, 16384, 32768, },
  { 1, 2, 4, 8, 32, 16384, 32768, },
  { 16, 32, 16384, 32768, },
  { 1, 16, 32, 16384, 32768, },
  { 2, 16, 32, 16384, 32768, },
  { 1, 2, 16, 32, 16384, 32768, },
  { 4, 16, 32, 16384, 32768, },
  { 1, 4, 16, 32, 16384, 32768, },
  { 2, 4, 16, 32, 16384, 32768, },
  { 1, 2, 4, 16, 32, 16384, 32768, },
  { 8, 16, 32, 16384, 32768, },
  { 1, 8, 16, 32, 16384, 32768, },
  { 2, 8, 16, 32, 16384, 32768, },
  { 1, 2, 8, 16, 32, 16384, 32768, },
  { 4, 8, 16, 32, 16384, 32768, },
  { 1, 4, 8, 16, 32, 16384, 32768, },
  { 2, 4, 8, 16, 32, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 16384, 32768, },
  { 64, 16384, 32768, },
  { 1, 64, 16384, 32768, },
  { 2, 64, 16384, 32768, },
  { 1, 2, 64, 16384, 32768, },
  { 4, 64, 16384, 32768, },
  { 1, 4, 64, 16384, 32768, },
  { 2, 4, 64, 16384, 32768, },
  { 1, 2, 4, 64, 16384, 32768, },
  { 8, 64, 16384, 32768, },
  { 1, 8, 64, 16384, 32768, },
  { 2, 8, 64, 16384, 32768, },
  { 1, 2, 8, 64, 16384, 32768, },
  { 4, 8, 64, 16384, 32768, },
  { 1, 4, 8, 64, 16384, 32768, },
  { 2, 4, 8, 64, 16384, 32768, },
  { 1, 2, 4, 8, 64, 16384, 32768, },
  { 16, 64, 16384, 32768, },
  { 1, 16, 64, 16384, 32768, },
  { 2, 16, 64, 16384, 32768, },
  { 1, 2, 16, 64, 16384, 32768, },
  { 4, 16, 64, 16384, 32768, },
  { 1, 4, 16, 64, 16384, 32768, },
  { 2, 4, 16, 64, 16384, 32768, },
  { 1, 2, 4, 16, 64, 16384, 32768, },
  { 8, 16, 64, 16384, 32768, },
  { 1, 8, 16, 64, 16384, 32768, },
  { 2, 8, 16, 64, 16384, 32768, },
  { 1, 2, 8, 16, 64, 16384, 32768, },
  { 4, 8, 16, 64, 16384, 32768, },
  { 1, 4, 8, 16, 64, 16384, 32768, },
  { 2, 4, 8, 16, 64, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 16384, 32768, },
  { 32, 64, 16384, 32768, },
  { 1, 32, 64, 16384, 32768, },
  { 2, 32, 64, 16384, 32768, },
  { 1, 2, 32, 64, 16384, 32768, },
  { 4, 32, 64, 16384, 32768, },
  { 1, 4, 32, 64, 16384, 32768, },
  { 2, 4, 32, 64, 16384, 32768, },
  { 1, 2, 4, 32, 64, 16384, 32768, },
  { 8, 32, 64, 16384, 32768, },
  { 1, 8, 32, 64, 16384, 32768, },
  { 2, 8, 32, 64, 16384, 32768, },
  { 1, 2, 8, 32, 64, 16384, 32768, },
  { 4, 8, 32, 64, 16384, 32768, },
  { 1, 4, 8, 32, 64, 16384, 32768, },
  { 2, 4, 8, 32, 64, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 16384, 32768, },
  { 16, 32, 64, 16384, 32768, },
  { 1, 16, 32, 64, 16384, 32768, },
  { 2, 16, 32, 64, 16384, 32768, },
  { 1, 2, 16, 32, 64, 16384, 32768, },
  { 4, 16, 32, 64, 16384, 32768, },
  { 1, 4, 16, 32, 64, 16384, 32768, },
  { 2, 4, 16, 32, 64, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 16384, 32768, },
  { 8, 16, 32, 64, 16384, 32768, },
  { 1, 8, 16, 32, 64, 16384, 32768, },
  { 2, 8, 16, 32, 64, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 16384, 32768, },
  { 4, 8, 16, 32, 64, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 16384, 32768, },
  { 128, 16384, 32768, },
  { 1, 128, 16384, 32768, },
  { 2, 128, 16384, 32768, },
  { 1, 2, 128, 16384, 32768, },
  { 4, 128, 16384, 32768, },
  { 1, 4, 128, 16384, 32768, },
  { 2, 4, 128, 16384, 32768, },
  { 1, 2, 4, 128, 16384, 32768, },
  { 8, 128, 16384, 32768, },
  { 1, 8, 128, 16384, 32768, },
  { 2, 8, 128, 16384, 32768, },
  { 1, 2, 8, 128, 16384, 32768, },
  { 4, 8, 128, 16384, 32768, },
  { 1, 4, 8, 128, 16384, 32768, },
  { 2, 4, 8, 128, 16384, 32768, },
  { 1, 2, 4, 8, 128, 16384, 32768, },
  { 16, 128, 16384, 32768, },
  { 1, 16, 128, 16384, 32768, },
  { 2, 16, 128, 16384, 32768, },
  { 1, 2, 16, 128, 16384, 32768, },
  { 4, 16, 128, 16384, 32768, },
  { 1, 4, 16, 128, 16384, 32768, },
  { 2, 4, 16, 128, 16384, 32768, },
  { 1, 2, 4, 16, 128, 16384, 32768, },
  { 8, 16, 128, 16384, 32768, },
  { 1, 8, 16, 128, 16384, 32768, },
  { 2, 8, 16, 128, 16384, 32768, },
  { 1, 2, 8, 16, 128, 16384, 32768, },
  { 4, 8, 16, 128, 16384, 32768, },
  { 1, 4, 8, 16, 128, 16384, 32768, },
  { 2, 4, 8, 16, 128, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 16384, 32768, },
  { 32, 128, 16384, 32768, },
  { 1, 32, 128, 16384, 32768, },
  { 2, 32, 128, 16384, 32768, },
  { 1, 2, 32, 128, 16384, 32768, },
  { 4, 32, 128, 16384, 32768, },
  { 1, 4, 32, 128, 16384, 32768, },
  { 2, 4, 32, 128, 16384, 32768, },
  { 1, 2, 4, 32, 128, 16384, 32768, },
  { 8, 32, 128, 16384, 32768, },
  { 1, 8, 32, 128, 16384, 32768, },
  { 2, 8, 32, 128, 16384, 32768, },
  { 1, 2, 8, 32, 128, 16384, 32768, },
  { 4, 8, 32, 128, 16384, 32768, },
  { 1, 4, 8, 32, 128, 16384, 32768, },
  { 2, 4, 8, 32, 128, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 16384, 32768, },
  { 16, 32, 128, 16384, 32768, },
  { 1, 16, 32, 128, 16384, 32768, },
  { 2, 16, 32, 128, 16384, 32768, },
  { 1, 2, 16, 32, 128, 16384, 32768, },
  { 4, 16, 32, 128, 16384, 32768, },
  { 1, 4, 16, 32, 128, 16384, 32768, },
  { 2, 4, 16, 32, 128, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 16384, 32768, },
  { 8, 16, 32, 128, 16384, 32768, },
  { 1, 8, 16, 32, 128, 16384, 32768, },
  { 2, 8, 16, 32, 128, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 16384, 32768, },
  { 4, 8, 16, 32, 128, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 16384, 32768, },
  { 64, 128, 16384, 32768, },
  { 1, 64, 128, 16384, 32768, },
  { 2, 64, 128, 16384, 32768, },
  { 1, 2, 64, 128, 16384, 32768, },
  { 4, 64, 128, 16384, 32768, },
  { 1, 4, 64, 128, 16384, 32768, },
  { 2, 4, 64, 128, 16384, 32768, },
  { 1, 2, 4, 64, 128, 16384, 32768, },
  { 8, 64, 128, 16384, 32768, },
  { 1, 8, 64, 128, 16384, 32768, },
  { 2, 8, 64, 128, 16384, 32768, },
  { 1, 2, 8, 64, 128, 16384, 32768, },
  { 4, 8, 64, 128, 16384, 32768, },
  { 1, 4, 8, 64, 128, 16384, 32768, },
  { 2, 4, 8, 64, 128, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 16384, 32768, },
  { 16, 64, 128, 16384, 32768, },
  { 1, 16, 64, 128, 16384, 32768, },
  { 2, 16, 64, 128, 16384, 32768, },
  { 1, 2, 16, 64, 128, 16384, 32768, },
  { 4, 16, 64, 128, 16384, 32768, },
  { 1, 4, 16, 64, 128, 16384, 32768, },
  { 2, 4, 16, 64, 128, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 16384, 32768, },
  { 8, 16, 64, 128, 16384, 32768, },
  { 1, 8, 16, 64, 128, 16384, 32768, },
  { 2, 8, 16, 64, 128, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 16384, 32768, },
  { 4, 8, 16, 64, 128, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 16384, 32768, },
  { 32, 64, 128, 16384, 32768, },
  { 1, 32, 64, 128, 16384, 32768, },
  { 2, 32, 64, 128, 16384, 32768, },
  { 1, 2, 32, 64, 128, 16384, 32768, },
  { 4, 32, 64, 128, 16384, 32768, },
  { 1, 4, 32, 64, 128, 16384, 32768, },
  { 2, 4, 32, 64, 128, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 16384, 32768, },
  { 8, 32, 64, 128, 16384, 32768, },
  { 1, 8, 32, 64, 128, 16384, 32768, },
  { 2, 8, 32, 64, 128, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 16384, 32768, },
  { 4, 8, 32, 64, 128, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 16384, 32768, },
  { 16, 32, 64, 128, 16384, 32768, },
  { 1, 16, 32, 64, 128, 16384, 32768, },
  { 2, 16, 32, 64, 128, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 16384, 32768, },
  { 4, 16, 32, 64, 128, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 16384, 32768, },
  { 8, 16, 32, 64, 128, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 16384, 32768, },
  { 256, 16384, 32768, },
  { 1, 256, 16384, 32768, },
  { 2, 256, 16384, 32768, },
  { 1, 2, 256, 16384, 32768, },
  { 4, 256, 16384, 32768, },
  { 1, 4, 256, 16384, 32768, },
  { 2, 4, 256, 16384, 32768, },
  { 1, 2, 4, 256, 16384, 32768, },
  { 8, 256, 16384, 32768, },
  { 1, 8, 256, 16384, 32768, },
  { 2, 8, 256, 16384, 32768, },
  { 1, 2, 8, 256, 16384, 32768, },
  { 4, 8, 256, 16384, 32768, },
  { 1, 4, 8, 256, 16384, 32768, },
  { 2, 4, 8, 256, 16384, 32768, },
  { 1, 2, 4, 8, 256, 16384, 32768, },
  { 16, 256, 16384, 32768, },
  { 1, 16, 256, 16384, 32768, },
  { 2, 16, 256, 16384, 32768, },
  { 1, 2, 16, 256, 16384, 32768, },
  { 4, 16, 256, 16384, 32768, },
  { 1, 4, 16, 256, 16384, 32768, },
  { 2, 4, 16, 256, 16384, 32768, },
  { 1, 2, 4, 16, 256, 16384, 32768, },
  { 8, 16, 256, 16384, 32768, },
  { 1, 8, 16, 256, 16384, 32768, },
  { 2, 8, 16, 256, 16384, 32768, },
  { 1, 2, 8, 16, 256, 16384, 32768, },
  { 4, 8, 16, 256, 16384, 32768, },
  { 1, 4, 8, 16, 256, 16384, 32768, },
  { 2, 4, 8, 16, 256, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 16384, 32768, },
  { 32, 256, 16384, 32768, },
  { 1, 32, 256, 16384, 32768, },
  { 2, 32, 256, 16384, 32768, },
  { 1, 2, 32, 256, 16384, 32768, },
  { 4, 32, 256, 16384, 32768, },
  { 1, 4, 32, 256, 16384, 32768, },
  { 2, 4, 32, 256, 16384, 32768, },
  { 1, 2, 4, 32, 256, 16384, 32768, },
  { 8, 32, 256, 16384, 32768, },
  { 1, 8, 32, 256, 16384, 32768, },
  { 2, 8, 32, 256, 16384, 32768, },
  { 1, 2, 8, 32, 256, 16384, 32768, },
  { 4, 8, 32, 256, 16384, 32768, },
  { 1, 4, 8, 32, 256, 16384, 32768, },
  { 2, 4, 8, 32, 256, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 16384, 32768, },
  { 16, 32, 256, 16384, 32768, },
  { 1, 16, 32, 256, 16384, 32768, },
  { 2, 16, 32, 256, 16384, 32768, },
  { 1, 2, 16, 32, 256, 16384, 32768, },
  { 4, 16, 32, 256, 16384, 32768, },
  { 1, 4, 16, 32, 256, 16384, 32768, },
  { 2, 4, 16, 32, 256, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 16384, 32768, },
  { 8, 16, 32, 256, 16384, 32768, },
  { 1, 8, 16, 32, 256, 16384, 32768, },
  { 2, 8, 16, 32, 256, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 16384, 32768, },
  { 4, 8, 16, 32, 256, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 16384, 32768, },
  { 64, 256, 16384, 32768, },
  { 1, 64, 256, 16384, 32768, },
  { 2, 64, 256, 16384, 32768, },
  { 1, 2, 64, 256, 16384, 32768, },
  { 4, 64, 256, 16384, 32768, },
  { 1, 4, 64, 256, 16384, 32768, },
  { 2, 4, 64, 256, 16384, 32768, },
  { 1, 2, 4, 64, 256, 16384, 32768, },
  { 8, 64, 256, 16384, 32768, },
  { 1, 8, 64, 256, 16384, 32768, },
  { 2, 8, 64, 256, 16384, 32768, },
  { 1, 2, 8, 64, 256, 16384, 32768, },
  { 4, 8, 64, 256, 16384, 32768, },
  { 1, 4, 8, 64, 256, 16384, 32768, },
  { 2, 4, 8, 64, 256, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 16384, 32768, },
  { 16, 64, 256, 16384, 32768, },
  { 1, 16, 64, 256, 16384, 32768, },
  { 2, 16, 64, 256, 16384, 32768, },
  { 1, 2, 16, 64, 256, 16384, 32768, },
  { 4, 16, 64, 256, 16384, 32768, },
  { 1, 4, 16, 64, 256, 16384, 32768, },
  { 2, 4, 16, 64, 256, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 16384, 32768, },
  { 8, 16, 64, 256, 16384, 32768, },
  { 1, 8, 16, 64, 256, 16384, 32768, },
  { 2, 8, 16, 64, 256, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 16384, 32768, },
  { 4, 8, 16, 64, 256, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 16384, 32768, },
  { 32, 64, 256, 16384, 32768, },
  { 1, 32, 64, 256, 16384, 32768, },
  { 2, 32, 64, 256, 16384, 32768, },
  { 1, 2, 32, 64, 256, 16384, 32768, },
  { 4, 32, 64, 256, 16384, 32768, },
  { 1, 4, 32, 64, 256, 16384, 32768, },
  { 2, 4, 32, 64, 256, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 16384, 32768, },
  { 8, 32, 64, 256, 16384, 32768, },
  { 1, 8, 32, 64, 256, 16384, 32768, },
  { 2, 8, 32, 64, 256, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 16384, 32768, },
  { 4, 8, 32, 64, 256, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 16384, 32768, },
  { 16, 32, 64, 256, 16384, 32768, },
  { 1, 16, 32, 64, 256, 16384, 32768, },
  { 2, 16, 32, 64, 256, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 16384, 32768, },
  { 4, 16, 32, 64, 256, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 16384, 32768, },
  { 8, 16, 32, 64, 256, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 16384, 32768, },
  { 128, 256, 16384, 32768, },
  { 1, 128, 256, 16384, 32768, },
  { 2, 128, 256, 16384, 32768, },
  { 1, 2, 128, 256, 16384, 32768, },
  { 4, 128, 256, 16384, 32768, },
  { 1, 4, 128, 256, 16384, 32768, },
  { 2, 4, 128, 256, 16384, 32768, },
  { 1, 2, 4, 128, 256, 16384, 32768, },
  { 8, 128, 256, 16384, 32768, },
  { 1, 8, 128, 256, 16384, 32768, },
  { 2, 8, 128, 256, 16384, 32768, },
  { 1, 2, 8, 128, 256, 16384, 32768, },
  { 4, 8, 128, 256, 16384, 32768, },
  { 1, 4, 8, 128, 256, 16384, 32768, },
  { 2, 4, 8, 128, 256, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 16384, 32768, },
  { 16, 128, 256, 16384, 32768, },
  { 1, 16, 128, 256, 16384, 32768, },
  { 2, 16, 128, 256, 16384, 32768, },
  { 1, 2, 16, 128, 256, 16384, 32768, },
  { 4, 16, 128, 256, 16384, 32768, },
  { 1, 4, 16, 128, 256, 16384, 32768, },
  { 2, 4, 16, 128, 256, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 16384, 32768, },
  { 8, 16, 128, 256, 16384, 32768, },
  { 1, 8, 16, 128, 256, 16384, 32768, },
  { 2, 8, 16, 128, 256, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 16384, 32768, },
  { 4, 8, 16, 128, 256, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 16384, 32768, },
  { 32, 128, 256, 16384, 32768, },
  { 1, 32, 128, 256, 16384, 32768, },
  { 2, 32, 128, 256, 16384, 32768, },
  { 1, 2, 32, 128, 256, 16384, 32768, },
  { 4, 32, 128, 256, 16384, 32768, },
  { 1, 4, 32, 128, 256, 16384, 32768, },
  { 2, 4, 32, 128, 256, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 16384, 32768, },
  { 8, 32, 128, 256, 16384, 32768, },
  { 1, 8, 32, 128, 256, 16384, 32768, },
  { 2, 8, 32, 128, 256, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 16384, 32768, },
  { 4, 8, 32, 128, 256, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 16384, 32768, },
  { 16, 32, 128, 256, 16384, 32768, },
  { 1, 16, 32, 128, 256, 16384, 32768, },
  { 2, 16, 32, 128, 256, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 16384, 32768, },
  { 4, 16, 32, 128, 256, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 16384, 32768, },
  { 8, 16, 32, 128, 256, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 16384, 32768, },
  { 64, 128, 256, 16384, 32768, },
  { 1, 64, 128, 256, 16384, 32768, },
  { 2, 64, 128, 256, 16384, 32768, },
  { 1, 2, 64, 128, 256, 16384, 32768, },
  { 4, 64, 128, 256, 16384, 32768, },
  { 1, 4, 64, 128, 256, 16384, 32768, },
  { 2, 4, 64, 128, 256, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 16384, 32768, },
  { 8, 64, 128, 256, 16384, 32768, },
  { 1, 8, 64, 128, 256, 16384, 32768, },
  { 2, 8, 64, 128, 256, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 16384, 32768, },
  { 4, 8, 64, 128, 256, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 16384, 32768, },
  { 16, 64, 128, 256, 16384, 32768, },
  { 1, 16, 64, 128, 256, 16384, 32768, },
  { 2, 16, 64, 128, 256, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 16384, 32768, },
  { 4, 16, 64, 128, 256, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 16384, 32768, },
  { 8, 16, 64, 128, 256, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 16384, 32768, },
  { 32, 64, 128, 256, 16384, 32768, },
  { 1, 32, 64, 128, 256, 16384, 32768, },
  { 2, 32, 64, 128, 256, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 16384, 32768, },
  { 4, 32, 64, 128, 256, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 16384, 32768, },
  { 8, 32, 64, 128, 256, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 16384, 32768, },
  { 16, 32, 64, 128, 256, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 16384, 32768, },
  { 512, 16384, 32768, },
  { 1, 512, 16384, 32768, },
  { 2, 512, 16384, 32768, },
  { 1, 2, 512, 16384, 32768, },
  { 4, 512, 16384, 32768, },
  { 1, 4, 512, 16384, 32768, },
  { 2, 4, 512, 16384, 32768, },
  { 1, 2, 4, 512, 16384, 32768, },
  { 8, 512, 16384, 32768, },
  { 1, 8, 512, 16384, 32768, },
  { 2, 8, 512, 16384, 32768, },
  { 1, 2, 8, 512, 16384, 32768, },
  { 4, 8, 512, 16384, 32768, },
  { 1, 4, 8, 512, 16384, 32768, },
  { 2, 4, 8, 512, 16384, 32768, },
  { 1, 2, 4, 8, 512, 16384, 32768, },
  { 16, 512, 16384, 32768, },
  { 1, 16, 512, 16384, 32768, },
  { 2, 16, 512, 16384, 32768, },
  { 1, 2, 16, 512, 16384, 32768, },
  { 4, 16, 512, 16384, 32768, },
  { 1, 4, 16, 512, 16384, 32768, },
  { 2, 4, 16, 512, 16384, 32768, },
  { 1, 2, 4, 16, 512, 16384, 32768, },
  { 8, 16, 512, 16384, 32768, },
  { 1, 8, 16, 512, 16384, 32768, },
  { 2, 8, 16, 512, 16384, 32768, },
  { 1, 2, 8, 16, 512, 16384, 32768, },
  { 4, 8, 16, 512, 16384, 32768, },
  { 1, 4, 8, 16, 512, 16384, 32768, },
  { 2, 4, 8, 16, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 16384, 32768, },
  { 32, 512, 16384, 32768, },
  { 1, 32, 512, 16384, 32768, },
  { 2, 32, 512, 16384, 32768, },
  { 1, 2, 32, 512, 16384, 32768, },
  { 4, 32, 512, 16384, 32768, },
  { 1, 4, 32, 512, 16384, 32768, },
  { 2, 4, 32, 512, 16384, 32768, },
  { 1, 2, 4, 32, 512, 16384, 32768, },
  { 8, 32, 512, 16384, 32768, },
  { 1, 8, 32, 512, 16384, 32768, },
  { 2, 8, 32, 512, 16384, 32768, },
  { 1, 2, 8, 32, 512, 16384, 32768, },
  { 4, 8, 32, 512, 16384, 32768, },
  { 1, 4, 8, 32, 512, 16384, 32768, },
  { 2, 4, 8, 32, 512, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 16384, 32768, },
  { 16, 32, 512, 16384, 32768, },
  { 1, 16, 32, 512, 16384, 32768, },
  { 2, 16, 32, 512, 16384, 32768, },
  { 1, 2, 16, 32, 512, 16384, 32768, },
  { 4, 16, 32, 512, 16384, 32768, },
  { 1, 4, 16, 32, 512, 16384, 32768, },
  { 2, 4, 16, 32, 512, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 16384, 32768, },
  { 8, 16, 32, 512, 16384, 32768, },
  { 1, 8, 16, 32, 512, 16384, 32768, },
  { 2, 8, 16, 32, 512, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 16384, 32768, },
  { 4, 8, 16, 32, 512, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 16384, 32768, },
  { 64, 512, 16384, 32768, },
  { 1, 64, 512, 16384, 32768, },
  { 2, 64, 512, 16384, 32768, },
  { 1, 2, 64, 512, 16384, 32768, },
  { 4, 64, 512, 16384, 32768, },
  { 1, 4, 64, 512, 16384, 32768, },
  { 2, 4, 64, 512, 16384, 32768, },
  { 1, 2, 4, 64, 512, 16384, 32768, },
  { 8, 64, 512, 16384, 32768, },
  { 1, 8, 64, 512, 16384, 32768, },
  { 2, 8, 64, 512, 16384, 32768, },
  { 1, 2, 8, 64, 512, 16384, 32768, },
  { 4, 8, 64, 512, 16384, 32768, },
  { 1, 4, 8, 64, 512, 16384, 32768, },
  { 2, 4, 8, 64, 512, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 16384, 32768, },
  { 16, 64, 512, 16384, 32768, },
  { 1, 16, 64, 512, 16384, 32768, },
  { 2, 16, 64, 512, 16384, 32768, },
  { 1, 2, 16, 64, 512, 16384, 32768, },
  { 4, 16, 64, 512, 16384, 32768, },
  { 1, 4, 16, 64, 512, 16384, 32768, },
  { 2, 4, 16, 64, 512, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 16384, 32768, },
  { 8, 16, 64, 512, 16384, 32768, },
  { 1, 8, 16, 64, 512, 16384, 32768, },
  { 2, 8, 16, 64, 512, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 16384, 32768, },
  { 4, 8, 16, 64, 512, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 16384, 32768, },
  { 32, 64, 512, 16384, 32768, },
  { 1, 32, 64, 512, 16384, 32768, },
  { 2, 32, 64, 512, 16384, 32768, },
  { 1, 2, 32, 64, 512, 16384, 32768, },
  { 4, 32, 64, 512, 16384, 32768, },
  { 1, 4, 32, 64, 512, 16384, 32768, },
  { 2, 4, 32, 64, 512, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 16384, 32768, },
  { 8, 32, 64, 512, 16384, 32768, },
  { 1, 8, 32, 64, 512, 16384, 32768, },
  { 2, 8, 32, 64, 512, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 16384, 32768, },
  { 4, 8, 32, 64, 512, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 16384, 32768, },
  { 16, 32, 64, 512, 16384, 32768, },
  { 1, 16, 32, 64, 512, 16384, 32768, },
  { 2, 16, 32, 64, 512, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 16384, 32768, },
  { 4, 16, 32, 64, 512, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 16384, 32768, },
  { 8, 16, 32, 64, 512, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 16384, 32768, },
  { 128, 512, 16384, 32768, },
  { 1, 128, 512, 16384, 32768, },
  { 2, 128, 512, 16384, 32768, },
  { 1, 2, 128, 512, 16384, 32768, },
  { 4, 128, 512, 16384, 32768, },
  { 1, 4, 128, 512, 16384, 32768, },
  { 2, 4, 128, 512, 16384, 32768, },
  { 1, 2, 4, 128, 512, 16384, 32768, },
  { 8, 128, 512, 16384, 32768, },
  { 1, 8, 128, 512, 16384, 32768, },
  { 2, 8, 128, 512, 16384, 32768, },
  { 1, 2, 8, 128, 512, 16384, 32768, },
  { 4, 8, 128, 512, 16384, 32768, },
  { 1, 4, 8, 128, 512, 16384, 32768, },
  { 2, 4, 8, 128, 512, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 16384, 32768, },
  { 16, 128, 512, 16384, 32768, },
  { 1, 16, 128, 512, 16384, 32768, },
  { 2, 16, 128, 512, 16384, 32768, },
  { 1, 2, 16, 128, 512, 16384, 32768, },
  { 4, 16, 128, 512, 16384, 32768, },
  { 1, 4, 16, 128, 512, 16384, 32768, },
  { 2, 4, 16, 128, 512, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 16384, 32768, },
  { 8, 16, 128, 512, 16384, 32768, },
  { 1, 8, 16, 128, 512, 16384, 32768, },
  { 2, 8, 16, 128, 512, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 16384, 32768, },
  { 4, 8, 16, 128, 512, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 16384, 32768, },
  { 32, 128, 512, 16384, 32768, },
  { 1, 32, 128, 512, 16384, 32768, },
  { 2, 32, 128, 512, 16384, 32768, },
  { 1, 2, 32, 128, 512, 16384, 32768, },
  { 4, 32, 128, 512, 16384, 32768, },
  { 1, 4, 32, 128, 512, 16384, 32768, },
  { 2, 4, 32, 128, 512, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 16384, 32768, },
  { 8, 32, 128, 512, 16384, 32768, },
  { 1, 8, 32, 128, 512, 16384, 32768, },
  { 2, 8, 32, 128, 512, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 16384, 32768, },
  { 4, 8, 32, 128, 512, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 16384, 32768, },
  { 16, 32, 128, 512, 16384, 32768, },
  { 1, 16, 32, 128, 512, 16384, 32768, },
  { 2, 16, 32, 128, 512, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 16384, 32768, },
  { 4, 16, 32, 128, 512, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 16384, 32768, },
  { 8, 16, 32, 128, 512, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 16384, 32768, },
  { 64, 128, 512, 16384, 32768, },
  { 1, 64, 128, 512, 16384, 32768, },
  { 2, 64, 128, 512, 16384, 32768, },
  { 1, 2, 64, 128, 512, 16384, 32768, },
  { 4, 64, 128, 512, 16384, 32768, },
  { 1, 4, 64, 128, 512, 16384, 32768, },
  { 2, 4, 64, 128, 512, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 16384, 32768, },
  { 8, 64, 128, 512, 16384, 32768, },
  { 1, 8, 64, 128, 512, 16384, 32768, },
  { 2, 8, 64, 128, 512, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 16384, 32768, },
  { 4, 8, 64, 128, 512, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 16384, 32768, },
  { 16, 64, 128, 512, 16384, 32768, },
  { 1, 16, 64, 128, 512, 16384, 32768, },
  { 2, 16, 64, 128, 512, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 16384, 32768, },
  { 4, 16, 64, 128, 512, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 16384, 32768, },
  { 8, 16, 64, 128, 512, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 16384, 32768, },
  { 32, 64, 128, 512, 16384, 32768, },
  { 1, 32, 64, 128, 512, 16384, 32768, },
  { 2, 32, 64, 128, 512, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 16384, 32768, },
  { 4, 32, 64, 128, 512, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 16384, 32768, },
  { 8, 32, 64, 128, 512, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 16384, 32768, },
  { 16, 32, 64, 128, 512, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 16384, 32768, },
  { 256, 512, 16384, 32768, },
  { 1, 256, 512, 16384, 32768, },
  { 2, 256, 512, 16384, 32768, },
  { 1, 2, 256, 512, 16384, 32768, },
  { 4, 256, 512, 16384, 32768, },
  { 1, 4, 256, 512, 16384, 32768, },
  { 2, 4, 256, 512, 16384, 32768, },
  { 1, 2, 4, 256, 512, 16384, 32768, },
  { 8, 256, 512, 16384, 32768, },
  { 1, 8, 256, 512, 16384, 32768, },
  { 2, 8, 256, 512, 16384, 32768, },
  { 1, 2, 8, 256, 512, 16384, 32768, },
  { 4, 8, 256, 512, 16384, 32768, },
  { 1, 4, 8, 256, 512, 16384, 32768, },
  { 2, 4, 8, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 16384, 32768, },
  { 16, 256, 512, 16384, 32768, },
  { 1, 16, 256, 512, 16384, 32768, },
  { 2, 16, 256, 512, 16384, 32768, },
  { 1, 2, 16, 256, 512, 16384, 32768, },
  { 4, 16, 256, 512, 16384, 32768, },
  { 1, 4, 16, 256, 512, 16384, 32768, },
  { 2, 4, 16, 256, 512, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 16384, 32768, },
  { 8, 16, 256, 512, 16384, 32768, },
  { 1, 8, 16, 256, 512, 16384, 32768, },
  { 2, 8, 16, 256, 512, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 16384, 32768, },
  { 4, 8, 16, 256, 512, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 16384, 32768, },
  { 32, 256, 512, 16384, 32768, },
  { 1, 32, 256, 512, 16384, 32768, },
  { 2, 32, 256, 512, 16384, 32768, },
  { 1, 2, 32, 256, 512, 16384, 32768, },
  { 4, 32, 256, 512, 16384, 32768, },
  { 1, 4, 32, 256, 512, 16384, 32768, },
  { 2, 4, 32, 256, 512, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 16384, 32768, },
  { 8, 32, 256, 512, 16384, 32768, },
  { 1, 8, 32, 256, 512, 16384, 32768, },
  { 2, 8, 32, 256, 512, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 16384, 32768, },
  { 4, 8, 32, 256, 512, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 16384, 32768, },
  { 16, 32, 256, 512, 16384, 32768, },
  { 1, 16, 32, 256, 512, 16384, 32768, },
  { 2, 16, 32, 256, 512, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 16384, 32768, },
  { 4, 16, 32, 256, 512, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 16384, 32768, },
  { 8, 16, 32, 256, 512, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 16384, 32768, },
  { 64, 256, 512, 16384, 32768, },
  { 1, 64, 256, 512, 16384, 32768, },
  { 2, 64, 256, 512, 16384, 32768, },
  { 1, 2, 64, 256, 512, 16384, 32768, },
  { 4, 64, 256, 512, 16384, 32768, },
  { 1, 4, 64, 256, 512, 16384, 32768, },
  { 2, 4, 64, 256, 512, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 16384, 32768, },
  { 8, 64, 256, 512, 16384, 32768, },
  { 1, 8, 64, 256, 512, 16384, 32768, },
  { 2, 8, 64, 256, 512, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 16384, 32768, },
  { 4, 8, 64, 256, 512, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 16384, 32768, },
  { 16, 64, 256, 512, 16384, 32768, },
  { 1, 16, 64, 256, 512, 16384, 32768, },
  { 2, 16, 64, 256, 512, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 16384, 32768, },
  { 4, 16, 64, 256, 512, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 16384, 32768, },
  { 8, 16, 64, 256, 512, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 16384, 32768, },
  { 32, 64, 256, 512, 16384, 32768, },
  { 1, 32, 64, 256, 512, 16384, 32768, },
  { 2, 32, 64, 256, 512, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 16384, 32768, },
  { 4, 32, 64, 256, 512, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 16384, 32768, },
  { 8, 32, 64, 256, 512, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 16384, 32768, },
  { 16, 32, 64, 256, 512, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 16384, 32768, },
  { 128, 256, 512, 16384, 32768, },
  { 1, 128, 256, 512, 16384, 32768, },
  { 2, 128, 256, 512, 16384, 32768, },
  { 1, 2, 128, 256, 512, 16384, 32768, },
  { 4, 128, 256, 512, 16384, 32768, },
  { 1, 4, 128, 256, 512, 16384, 32768, },
  { 2, 4, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 16384, 32768, },
  { 8, 128, 256, 512, 16384, 32768, },
  { 1, 8, 128, 256, 512, 16384, 32768, },
  { 2, 8, 128, 256, 512, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 16384, 32768, },
  { 4, 8, 128, 256, 512, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 16384, 32768, },
  { 16, 128, 256, 512, 16384, 32768, },
  { 1, 16, 128, 256, 512, 16384, 32768, },
  { 2, 16, 128, 256, 512, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 16384, 32768, },
  { 4, 16, 128, 256, 512, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 16384, 32768, },
  { 8, 16, 128, 256, 512, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 16384, 32768, },
  { 32, 128, 256, 512, 16384, 32768, },
  { 1, 32, 128, 256, 512, 16384, 32768, },
  { 2, 32, 128, 256, 512, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 16384, 32768, },
  { 4, 32, 128, 256, 512, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 16384, 32768, },
  { 8, 32, 128, 256, 512, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 16384, 32768, },
  { 16, 32, 128, 256, 512, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 16384, 32768, },
  { 64, 128, 256, 512, 16384, 32768, },
  { 1, 64, 128, 256, 512, 16384, 32768, },
  { 2, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 16384, 32768, },
  { 4, 64, 128, 256, 512, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 16384, 32768, },
  { 8, 64, 128, 256, 512, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 16384, 32768, },
  { 16, 64, 128, 256, 512, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 16384, 32768, },
  { 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 16384, 32768, },
  { 1024, 16384, 32768, },
  { 1, 1024, 16384, 32768, },
  { 2, 1024, 16384, 32768, },
  { 1, 2, 1024, 16384, 32768, },
  { 4, 1024, 16384, 32768, },
  { 1, 4, 1024, 16384, 32768, },
  { 2, 4, 1024, 16384, 32768, },
  { 1, 2, 4, 1024, 16384, 32768, },
  { 8, 1024, 16384, 32768, },
  { 1, 8, 1024, 16384, 32768, },
  { 2, 8, 1024, 16384, 32768, },
  { 1, 2, 8, 1024, 16384, 32768, },
  { 4, 8, 1024, 16384, 32768, },
  { 1, 4, 8, 1024, 16384, 32768, },
  { 2, 4, 8, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 1024, 16384, 32768, },
  { 16, 1024, 16384, 32768, },
  { 1, 16, 1024, 16384, 32768, },
  { 2, 16, 1024, 16384, 32768, },
  { 1, 2, 16, 1024, 16384, 32768, },
  { 4, 16, 1024, 16384, 32768, },
  { 1, 4, 16, 1024, 16384, 32768, },
  { 2, 4, 16, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 1024, 16384, 32768, },
  { 8, 16, 1024, 16384, 32768, },
  { 1, 8, 16, 1024, 16384, 32768, },
  { 2, 8, 16, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 1024, 16384, 32768, },
  { 4, 8, 16, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 1024, 16384, 32768, },
  { 32, 1024, 16384, 32768, },
  { 1, 32, 1024, 16384, 32768, },
  { 2, 32, 1024, 16384, 32768, },
  { 1, 2, 32, 1024, 16384, 32768, },
  { 4, 32, 1024, 16384, 32768, },
  { 1, 4, 32, 1024, 16384, 32768, },
  { 2, 4, 32, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 1024, 16384, 32768, },
  { 8, 32, 1024, 16384, 32768, },
  { 1, 8, 32, 1024, 16384, 32768, },
  { 2, 8, 32, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 1024, 16384, 32768, },
  { 4, 8, 32, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 1024, 16384, 32768, },
  { 16, 32, 1024, 16384, 32768, },
  { 1, 16, 32, 1024, 16384, 32768, },
  { 2, 16, 32, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 1024, 16384, 32768, },
  { 4, 16, 32, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 1024, 16384, 32768, },
  { 8, 16, 32, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 16384, 32768, },
  { 64, 1024, 16384, 32768, },
  { 1, 64, 1024, 16384, 32768, },
  { 2, 64, 1024, 16384, 32768, },
  { 1, 2, 64, 1024, 16384, 32768, },
  { 4, 64, 1024, 16384, 32768, },
  { 1, 4, 64, 1024, 16384, 32768, },
  { 2, 4, 64, 1024, 16384, 32768, },
  { 1, 2, 4, 64, 1024, 16384, 32768, },
  { 8, 64, 1024, 16384, 32768, },
  { 1, 8, 64, 1024, 16384, 32768, },
  { 2, 8, 64, 1024, 16384, 32768, },
  { 1, 2, 8, 64, 1024, 16384, 32768, },
  { 4, 8, 64, 1024, 16384, 32768, },
  { 1, 4, 8, 64, 1024, 16384, 32768, },
  { 2, 4, 8, 64, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 64, 1024, 16384, 32768, },
  { 16, 64, 1024, 16384, 32768, },
  { 1, 16, 64, 1024, 16384, 32768, },
  { 2, 16, 64, 1024, 16384, 32768, },
  { 1, 2, 16, 64, 1024, 16384, 32768, },
  { 4, 16, 64, 1024, 16384, 32768, },
  { 1, 4, 16, 64, 1024, 16384, 32768, },
  { 2, 4, 16, 64, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 64, 1024, 16384, 32768, },
  { 8, 16, 64, 1024, 16384, 32768, },
  { 1, 8, 16, 64, 1024, 16384, 32768, },
  { 2, 8, 16, 64, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 64, 1024, 16384, 32768, },
  { 4, 8, 16, 64, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 64, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 64, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 16384, 32768, },
  { 32, 64, 1024, 16384, 32768, },
  { 1, 32, 64, 1024, 16384, 32768, },
  { 2, 32, 64, 1024, 16384, 32768, },
  { 1, 2, 32, 64, 1024, 16384, 32768, },
  { 4, 32, 64, 1024, 16384, 32768, },
  { 1, 4, 32, 64, 1024, 16384, 32768, },
  { 2, 4, 32, 64, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 64, 1024, 16384, 32768, },
  { 8, 32, 64, 1024, 16384, 32768, },
  { 1, 8, 32, 64, 1024, 16384, 32768, },
  { 2, 8, 32, 64, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 64, 1024, 16384, 32768, },
  { 4, 8, 32, 64, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 64, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 64, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 16384, 32768, },
  { 16, 32, 64, 1024, 16384, 32768, },
  { 1, 16, 32, 64, 1024, 16384, 32768, },
  { 2, 16, 32, 64, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 64, 1024, 16384, 32768, },
  { 4, 16, 32, 64, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 64, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 64, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 16384, 32768, },
  { 8, 16, 32, 64, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 64, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 64, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 64, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 16384, 32768, },
  { 128, 1024, 16384, 32768, },
  { 1, 128, 1024, 16384, 32768, },
  { 2, 128, 1024, 16384, 32768, },
  { 1, 2, 128, 1024, 16384, 32768, },
  { 4, 128, 1024, 16384, 32768, },
  { 1, 4, 128, 1024, 16384, 32768, },
  { 2, 4, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 128, 1024, 16384, 32768, },
  { 8, 128, 1024, 16384, 32768, },
  { 1, 8, 128, 1024, 16384, 32768, },
  { 2, 8, 128, 1024, 16384, 32768, },
  { 1, 2, 8, 128, 1024, 16384, 32768, },
  { 4, 8, 128, 1024, 16384, 32768, },
  { 1, 4, 8, 128, 1024, 16384, 32768, },
  { 2, 4, 8, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 128, 1024, 16384, 32768, },
  { 16, 128, 1024, 16384, 32768, },
  { 1, 16, 128, 1024, 16384, 32768, },
  { 2, 16, 128, 1024, 16384, 32768, },
  { 1, 2, 16, 128, 1024, 16384, 32768, },
  { 4, 16, 128, 1024, 16384, 32768, },
  { 1, 4, 16, 128, 1024, 16384, 32768, },
  { 2, 4, 16, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 128, 1024, 16384, 32768, },
  { 8, 16, 128, 1024, 16384, 32768, },
  { 1, 8, 16, 128, 1024, 16384, 32768, },
  { 2, 8, 16, 128, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 128, 1024, 16384, 32768, },
  { 4, 8, 16, 128, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 128, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 16384, 32768, },
  { 32, 128, 1024, 16384, 32768, },
  { 1, 32, 128, 1024, 16384, 32768, },
  { 2, 32, 128, 1024, 16384, 32768, },
  { 1, 2, 32, 128, 1024, 16384, 32768, },
  { 4, 32, 128, 1024, 16384, 32768, },
  { 1, 4, 32, 128, 1024, 16384, 32768, },
  { 2, 4, 32, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 128, 1024, 16384, 32768, },
  { 8, 32, 128, 1024, 16384, 32768, },
  { 1, 8, 32, 128, 1024, 16384, 32768, },
  { 2, 8, 32, 128, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 128, 1024, 16384, 32768, },
  { 4, 8, 32, 128, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 128, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 16384, 32768, },
  { 16, 32, 128, 1024, 16384, 32768, },
  { 1, 16, 32, 128, 1024, 16384, 32768, },
  { 2, 16, 32, 128, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 128, 1024, 16384, 32768, },
  { 4, 16, 32, 128, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 128, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 16384, 32768, },
  { 8, 16, 32, 128, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 128, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 128, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 128, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 16384, 32768, },
  { 64, 128, 1024, 16384, 32768, },
  { 1, 64, 128, 1024, 16384, 32768, },
  { 2, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 64, 128, 1024, 16384, 32768, },
  { 4, 64, 128, 1024, 16384, 32768, },
  { 1, 4, 64, 128, 1024, 16384, 32768, },
  { 2, 4, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 64, 128, 1024, 16384, 32768, },
  { 8, 64, 128, 1024, 16384, 32768, },
  { 1, 8, 64, 128, 1024, 16384, 32768, },
  { 2, 8, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 8, 64, 128, 1024, 16384, 32768, },
  { 4, 8, 64, 128, 1024, 16384, 32768, },
  { 1, 4, 8, 64, 128, 1024, 16384, 32768, },
  { 2, 4, 8, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 16384, 32768, },
  { 16, 64, 128, 1024, 16384, 32768, },
  { 1, 16, 64, 128, 1024, 16384, 32768, },
  { 2, 16, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 16, 64, 128, 1024, 16384, 32768, },
  { 4, 16, 64, 128, 1024, 16384, 32768, },
  { 1, 4, 16, 64, 128, 1024, 16384, 32768, },
  { 2, 4, 16, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 16384, 32768, },
  { 8, 16, 64, 128, 1024, 16384, 32768, },
  { 1, 8, 16, 64, 128, 1024, 16384, 32768, },
  { 2, 8, 16, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 16384, 32768, },
  { 4, 8, 16, 64, 128, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 16384, 32768, },
  { 32, 64, 128, 1024, 16384, 32768, },
  { 1, 32, 64, 128, 1024, 16384, 32768, },
  { 2, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 32, 64, 128, 1024, 16384, 32768, },
  { 4, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 4, 32, 64, 128, 1024, 16384, 32768, },
  { 2, 4, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 16384, 32768, },
  { 8, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 8, 32, 64, 128, 1024, 16384, 32768, },
  { 2, 8, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 16384, 32768, },
  { 4, 8, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 16384, 32768, },
  { 16, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 2, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 4, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 8, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 16384, 32768, },
  { 256, 1024, 16384, 32768, },
  { 1, 256, 1024, 16384, 32768, },
  { 2, 256, 1024, 16384, 32768, },
  { 1, 2, 256, 1024, 16384, 32768, },
  { 4, 256, 1024, 16384, 32768, },
  { 1, 4, 256, 1024, 16384, 32768, },
  { 2, 4, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 256, 1024, 16384, 32768, },
  { 8, 256, 1024, 16384, 32768, },
  { 1, 8, 256, 1024, 16384, 32768, },
  { 2, 8, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 256, 1024, 16384, 32768, },
  { 4, 8, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 256, 1024, 16384, 32768, },
  { 16, 256, 1024, 16384, 32768, },
  { 1, 16, 256, 1024, 16384, 32768, },
  { 2, 16, 256, 1024, 16384, 32768, },
  { 1, 2, 16, 256, 1024, 16384, 32768, },
  { 4, 16, 256, 1024, 16384, 32768, },
  { 1, 4, 16, 256, 1024, 16384, 32768, },
  { 2, 4, 16, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 256, 1024, 16384, 32768, },
  { 8, 16, 256, 1024, 16384, 32768, },
  { 1, 8, 16, 256, 1024, 16384, 32768, },
  { 2, 8, 16, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 256, 1024, 16384, 32768, },
  { 4, 8, 16, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 16384, 32768, },
  { 32, 256, 1024, 16384, 32768, },
  { 1, 32, 256, 1024, 16384, 32768, },
  { 2, 32, 256, 1024, 16384, 32768, },
  { 1, 2, 32, 256, 1024, 16384, 32768, },
  { 4, 32, 256, 1024, 16384, 32768, },
  { 1, 4, 32, 256, 1024, 16384, 32768, },
  { 2, 4, 32, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 256, 1024, 16384, 32768, },
  { 8, 32, 256, 1024, 16384, 32768, },
  { 1, 8, 32, 256, 1024, 16384, 32768, },
  { 2, 8, 32, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 256, 1024, 16384, 32768, },
  { 4, 8, 32, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 16384, 32768, },
  { 16, 32, 256, 1024, 16384, 32768, },
  { 1, 16, 32, 256, 1024, 16384, 32768, },
  { 2, 16, 32, 256, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 256, 1024, 16384, 32768, },
  { 4, 16, 32, 256, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 256, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 16384, 32768, },
  { 8, 16, 32, 256, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 256, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 16384, 32768, },
  { 64, 256, 1024, 16384, 32768, },
  { 1, 64, 256, 1024, 16384, 32768, },
  { 2, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 64, 256, 1024, 16384, 32768, },
  { 4, 64, 256, 1024, 16384, 32768, },
  { 1, 4, 64, 256, 1024, 16384, 32768, },
  { 2, 4, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 64, 256, 1024, 16384, 32768, },
  { 8, 64, 256, 1024, 16384, 32768, },
  { 1, 8, 64, 256, 1024, 16384, 32768, },
  { 2, 8, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 64, 256, 1024, 16384, 32768, },
  { 4, 8, 64, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 64, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 16384, 32768, },
  { 16, 64, 256, 1024, 16384, 32768, },
  { 1, 16, 64, 256, 1024, 16384, 32768, },
  { 2, 16, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 16, 64, 256, 1024, 16384, 32768, },
  { 4, 16, 64, 256, 1024, 16384, 32768, },
  { 1, 4, 16, 64, 256, 1024, 16384, 32768, },
  { 2, 4, 16, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 16384, 32768, },
  { 8, 16, 64, 256, 1024, 16384, 32768, },
  { 1, 8, 16, 64, 256, 1024, 16384, 32768, },
  { 2, 8, 16, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 16384, 32768, },
  { 4, 8, 16, 64, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 16384, 32768, },
  { 32, 64, 256, 1024, 16384, 32768, },
  { 1, 32, 64, 256, 1024, 16384, 32768, },
  { 2, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 32, 64, 256, 1024, 16384, 32768, },
  { 4, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 4, 32, 64, 256, 1024, 16384, 32768, },
  { 2, 4, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 16384, 32768, },
  { 8, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 8, 32, 64, 256, 1024, 16384, 32768, },
  { 2, 8, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 16384, 32768, },
  { 4, 8, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 16384, 32768, },
  { 16, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 2, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 4, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 8, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 16384, 32768, },
  { 128, 256, 1024, 16384, 32768, },
  { 1, 128, 256, 1024, 16384, 32768, },
  { 2, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 128, 256, 1024, 16384, 32768, },
  { 4, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 128, 256, 1024, 16384, 32768, },
  { 8, 128, 256, 1024, 16384, 32768, },
  { 1, 8, 128, 256, 1024, 16384, 32768, },
  { 2, 8, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 128, 256, 1024, 16384, 32768, },
  { 4, 8, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 16384, 32768, },
  { 16, 128, 256, 1024, 16384, 32768, },
  { 1, 16, 128, 256, 1024, 16384, 32768, },
  { 2, 16, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 16, 128, 256, 1024, 16384, 32768, },
  { 4, 16, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 16, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 16, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 16384, 32768, },
  { 8, 16, 128, 256, 1024, 16384, 32768, },
  { 1, 8, 16, 128, 256, 1024, 16384, 32768, },
  { 2, 8, 16, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 16384, 32768, },
  { 4, 8, 16, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 16384, 32768, },
  { 32, 128, 256, 1024, 16384, 32768, },
  { 1, 32, 128, 256, 1024, 16384, 32768, },
  { 2, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 32, 128, 256, 1024, 16384, 32768, },
  { 4, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 32, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 16384, 32768, },
  { 8, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 8, 32, 128, 256, 1024, 16384, 32768, },
  { 2, 8, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 16384, 32768, },
  { 4, 8, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 16384, 32768, },
  { 16, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 2, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 4, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 8, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 16384, 32768, },
  { 64, 128, 256, 1024, 16384, 32768, },
  { 1, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 64, 128, 256, 1024, 16384, 32768, },
  { 4, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 16384, 32768, },
  { 8, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 8, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 8, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 16384, 32768, },
  { 4, 8, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 16384, 32768, },
  { 16, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 4, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 8, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 16384, 32768, },
  { 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 4, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 8, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 16384, 32768, },
  { 512, 1024, 16384, 32768, },
  { 1, 512, 1024, 16384, 32768, },
  { 2, 512, 1024, 16384, 32768, },
  { 1, 2, 512, 1024, 16384, 32768, },
  { 4, 512, 1024, 16384, 32768, },
  { 1, 4, 512, 1024, 16384, 32768, },
  { 2, 4, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 512, 1024, 16384, 32768, },
  { 8, 512, 1024, 16384, 32768, },
  { 1, 8, 512, 1024, 16384, 32768, },
  { 2, 8, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 512, 1024, 16384, 32768, },
  { 4, 8, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 512, 1024, 16384, 32768, },
  { 16, 512, 1024, 16384, 32768, },
  { 1, 16, 512, 1024, 16384, 32768, },
  { 2, 16, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 512, 1024, 16384, 32768, },
  { 4, 16, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 512, 1024, 16384, 32768, },
  { 8, 16, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 16384, 32768, },
  { 32, 512, 1024, 16384, 32768, },
  { 1, 32, 512, 1024, 16384, 32768, },
  { 2, 32, 512, 1024, 16384, 32768, },
  { 1, 2, 32, 512, 1024, 16384, 32768, },
  { 4, 32, 512, 1024, 16384, 32768, },
  { 1, 4, 32, 512, 1024, 16384, 32768, },
  { 2, 4, 32, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 512, 1024, 16384, 32768, },
  { 8, 32, 512, 1024, 16384, 32768, },
  { 1, 8, 32, 512, 1024, 16384, 32768, },
  { 2, 8, 32, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 512, 1024, 16384, 32768, },
  { 4, 8, 32, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 16384, 32768, },
  { 16, 32, 512, 1024, 16384, 32768, },
  { 1, 16, 32, 512, 1024, 16384, 32768, },
  { 2, 16, 32, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 512, 1024, 16384, 32768, },
  { 4, 16, 32, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 16384, 32768, },
  { 8, 16, 32, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 16384, 32768, },
  { 64, 512, 1024, 16384, 32768, },
  { 1, 64, 512, 1024, 16384, 32768, },
  { 2, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 64, 512, 1024, 16384, 32768, },
  { 4, 64, 512, 1024, 16384, 32768, },
  { 1, 4, 64, 512, 1024, 16384, 32768, },
  { 2, 4, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 64, 512, 1024, 16384, 32768, },
  { 8, 64, 512, 1024, 16384, 32768, },
  { 1, 8, 64, 512, 1024, 16384, 32768, },
  { 2, 8, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 64, 512, 1024, 16384, 32768, },
  { 4, 8, 64, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 64, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 16384, 32768, },
  { 16, 64, 512, 1024, 16384, 32768, },
  { 1, 16, 64, 512, 1024, 16384, 32768, },
  { 2, 16, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 64, 512, 1024, 16384, 32768, },
  { 4, 16, 64, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 64, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 16384, 32768, },
  { 8, 16, 64, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 64, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 64, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 16384, 32768, },
  { 32, 64, 512, 1024, 16384, 32768, },
  { 1, 32, 64, 512, 1024, 16384, 32768, },
  { 2, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 32, 64, 512, 1024, 16384, 32768, },
  { 4, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 4, 32, 64, 512, 1024, 16384, 32768, },
  { 2, 4, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 16384, 32768, },
  { 8, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 8, 32, 64, 512, 1024, 16384, 32768, },
  { 2, 8, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 16384, 32768, },
  { 4, 8, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 16384, 32768, },
  { 16, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 2, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 4, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 8, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 16384, 32768, },
  { 128, 512, 1024, 16384, 32768, },
  { 1, 128, 512, 1024, 16384, 32768, },
  { 2, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 128, 512, 1024, 16384, 32768, },
  { 4, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 128, 512, 1024, 16384, 32768, },
  { 8, 128, 512, 1024, 16384, 32768, },
  { 1, 8, 128, 512, 1024, 16384, 32768, },
  { 2, 8, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 128, 512, 1024, 16384, 32768, },
  { 4, 8, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 16384, 32768, },
  { 16, 128, 512, 1024, 16384, 32768, },
  { 1, 16, 128, 512, 1024, 16384, 32768, },
  { 2, 16, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 128, 512, 1024, 16384, 32768, },
  { 4, 16, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 16384, 32768, },
  { 8, 16, 128, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 128, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 16384, 32768, },
  { 32, 128, 512, 1024, 16384, 32768, },
  { 1, 32, 128, 512, 1024, 16384, 32768, },
  { 2, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 32, 128, 512, 1024, 16384, 32768, },
  { 4, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 32, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 16384, 32768, },
  { 8, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 8, 32, 128, 512, 1024, 16384, 32768, },
  { 2, 8, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 16384, 32768, },
  { 4, 8, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 16384, 32768, },
  { 16, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 2, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 4, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 8, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 16384, 32768, },
  { 64, 128, 512, 1024, 16384, 32768, },
  { 1, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 64, 128, 512, 1024, 16384, 32768, },
  { 4, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 16384, 32768, },
  { 8, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 8, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 8, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 16384, 32768, },
  { 4, 8, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 16384, 32768, },
  { 16, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 4, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 8, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 16384, 32768, },
  { 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 4, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 8, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 16384, 32768, },
  { 256, 512, 1024, 16384, 32768, },
  { 1, 256, 512, 1024, 16384, 32768, },
  { 2, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 256, 512, 1024, 16384, 32768, },
  { 4, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 256, 512, 1024, 16384, 32768, },
  { 8, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 16384, 32768, },
  { 16, 256, 512, 1024, 16384, 32768, },
  { 1, 16, 256, 512, 1024, 16384, 32768, },
  { 2, 16, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 256, 512, 1024, 16384, 32768, },
  { 4, 16, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 16384, 32768, },
  { 8, 16, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 16384, 32768, },
  { 32, 256, 512, 1024, 16384, 32768, },
  { 1, 32, 256, 512, 1024, 16384, 32768, },
  { 2, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 32, 256, 512, 1024, 16384, 32768, },
  { 4, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 32, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 16384, 32768, },
  { 8, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 32, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 16384, 32768, },
  { 16, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 2, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 4, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 8, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 16384, 32768, },
  { 64, 256, 512, 1024, 16384, 32768, },
  { 1, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 64, 256, 512, 1024, 16384, 32768, },
  { 4, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 16384, 32768, },
  { 8, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 16384, 32768, },
  { 16, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 4, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 8, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 16384, 32768, },
  { 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 4, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 8, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 16384, 32768, },
  { 128, 256, 512, 1024, 16384, 32768, },
  { 1, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 16384, 32768, },
  { 8, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 16384, 32768, },
  { 16, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 8, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 16384, 32768, },
  { 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 8, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 16384, 32768, },
  { 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 8, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 16384, 32768, },
  { 2048, 16384, 32768, },
  { 1, 2048, 16384, 32768, },
  { 2, 2048, 16384, 32768, },
  { 1, 2, 2048, 16384, 32768, },
  { 4, 2048, 16384, 32768, },
  { 1, 4, 2048, 16384, 32768, },
  { 2, 4, 2048, 16384, 32768, },
  { 1, 2, 4, 2048, 16384, 32768, },
  { 8, 2048, 16384, 32768, },
  { 1, 8, 2048, 16384, 32768, },
  { 2, 8, 2048, 16384, 32768, },
  { 1, 2, 8, 2048, 16384, 32768, },
  { 4, 8, 2048, 16384, 32768, },
  { 1, 4, 8, 2048, 16384, 32768, },
  { 2, 4, 8, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 2048, 16384, 32768, },
  { 16, 2048, 16384, 32768, },
  { 1, 16, 2048, 16384, 32768, },
  { 2, 16, 2048, 16384, 32768, },
  { 1, 2, 16, 2048, 16384, 32768, },
  { 4, 16, 2048, 16384, 32768, },
  { 1, 4, 16, 2048, 16384, 32768, },
  { 2, 4, 16, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 2048, 16384, 32768, },
  { 8, 16, 2048, 16384, 32768, },
  { 1, 8, 16, 2048, 16384, 32768, },
  { 2, 8, 16, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 2048, 16384, 32768, },
  { 4, 8, 16, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 2048, 16384, 32768, },
  { 32, 2048, 16384, 32768, },
  { 1, 32, 2048, 16384, 32768, },
  { 2, 32, 2048, 16384, 32768, },
  { 1, 2, 32, 2048, 16384, 32768, },
  { 4, 32, 2048, 16384, 32768, },
  { 1, 4, 32, 2048, 16384, 32768, },
  { 2, 4, 32, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 2048, 16384, 32768, },
  { 8, 32, 2048, 16384, 32768, },
  { 1, 8, 32, 2048, 16384, 32768, },
  { 2, 8, 32, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 2048, 16384, 32768, },
  { 4, 8, 32, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 2048, 16384, 32768, },
  { 16, 32, 2048, 16384, 32768, },
  { 1, 16, 32, 2048, 16384, 32768, },
  { 2, 16, 32, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 2048, 16384, 32768, },
  { 4, 16, 32, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 2048, 16384, 32768, },
  { 8, 16, 32, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 2048, 16384, 32768, },
  { 64, 2048, 16384, 32768, },
  { 1, 64, 2048, 16384, 32768, },
  { 2, 64, 2048, 16384, 32768, },
  { 1, 2, 64, 2048, 16384, 32768, },
  { 4, 64, 2048, 16384, 32768, },
  { 1, 4, 64, 2048, 16384, 32768, },
  { 2, 4, 64, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 2048, 16384, 32768, },
  { 8, 64, 2048, 16384, 32768, },
  { 1, 8, 64, 2048, 16384, 32768, },
  { 2, 8, 64, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 2048, 16384, 32768, },
  { 4, 8, 64, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 2048, 16384, 32768, },
  { 16, 64, 2048, 16384, 32768, },
  { 1, 16, 64, 2048, 16384, 32768, },
  { 2, 16, 64, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 2048, 16384, 32768, },
  { 4, 16, 64, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 2048, 16384, 32768, },
  { 8, 16, 64, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 2048, 16384, 32768, },
  { 32, 64, 2048, 16384, 32768, },
  { 1, 32, 64, 2048, 16384, 32768, },
  { 2, 32, 64, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 2048, 16384, 32768, },
  { 4, 32, 64, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 2048, 16384, 32768, },
  { 8, 32, 64, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 2048, 16384, 32768, },
  { 16, 32, 64, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 16384, 32768, },
  { 128, 2048, 16384, 32768, },
  { 1, 128, 2048, 16384, 32768, },
  { 2, 128, 2048, 16384, 32768, },
  { 1, 2, 128, 2048, 16384, 32768, },
  { 4, 128, 2048, 16384, 32768, },
  { 1, 4, 128, 2048, 16384, 32768, },
  { 2, 4, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 128, 2048, 16384, 32768, },
  { 8, 128, 2048, 16384, 32768, },
  { 1, 8, 128, 2048, 16384, 32768, },
  { 2, 8, 128, 2048, 16384, 32768, },
  { 1, 2, 8, 128, 2048, 16384, 32768, },
  { 4, 8, 128, 2048, 16384, 32768, },
  { 1, 4, 8, 128, 2048, 16384, 32768, },
  { 2, 4, 8, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 128, 2048, 16384, 32768, },
  { 16, 128, 2048, 16384, 32768, },
  { 1, 16, 128, 2048, 16384, 32768, },
  { 2, 16, 128, 2048, 16384, 32768, },
  { 1, 2, 16, 128, 2048, 16384, 32768, },
  { 4, 16, 128, 2048, 16384, 32768, },
  { 1, 4, 16, 128, 2048, 16384, 32768, },
  { 2, 4, 16, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 128, 2048, 16384, 32768, },
  { 8, 16, 128, 2048, 16384, 32768, },
  { 1, 8, 16, 128, 2048, 16384, 32768, },
  { 2, 8, 16, 128, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 128, 2048, 16384, 32768, },
  { 4, 8, 16, 128, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 128, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 2048, 16384, 32768, },
  { 32, 128, 2048, 16384, 32768, },
  { 1, 32, 128, 2048, 16384, 32768, },
  { 2, 32, 128, 2048, 16384, 32768, },
  { 1, 2, 32, 128, 2048, 16384, 32768, },
  { 4, 32, 128, 2048, 16384, 32768, },
  { 1, 4, 32, 128, 2048, 16384, 32768, },
  { 2, 4, 32, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 128, 2048, 16384, 32768, },
  { 8, 32, 128, 2048, 16384, 32768, },
  { 1, 8, 32, 128, 2048, 16384, 32768, },
  { 2, 8, 32, 128, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 128, 2048, 16384, 32768, },
  { 4, 8, 32, 128, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 128, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 2048, 16384, 32768, },
  { 16, 32, 128, 2048, 16384, 32768, },
  { 1, 16, 32, 128, 2048, 16384, 32768, },
  { 2, 16, 32, 128, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 128, 2048, 16384, 32768, },
  { 4, 16, 32, 128, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 128, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 2048, 16384, 32768, },
  { 8, 16, 32, 128, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 128, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 128, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 128, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 16384, 32768, },
  { 64, 128, 2048, 16384, 32768, },
  { 1, 64, 128, 2048, 16384, 32768, },
  { 2, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 64, 128, 2048, 16384, 32768, },
  { 4, 64, 128, 2048, 16384, 32768, },
  { 1, 4, 64, 128, 2048, 16384, 32768, },
  { 2, 4, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 128, 2048, 16384, 32768, },
  { 8, 64, 128, 2048, 16384, 32768, },
  { 1, 8, 64, 128, 2048, 16384, 32768, },
  { 2, 8, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 128, 2048, 16384, 32768, },
  { 4, 8, 64, 128, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 128, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 2048, 16384, 32768, },
  { 16, 64, 128, 2048, 16384, 32768, },
  { 1, 16, 64, 128, 2048, 16384, 32768, },
  { 2, 16, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 128, 2048, 16384, 32768, },
  { 4, 16, 64, 128, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 128, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 2048, 16384, 32768, },
  { 8, 16, 64, 128, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 128, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 128, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 16384, 32768, },
  { 32, 64, 128, 2048, 16384, 32768, },
  { 1, 32, 64, 128, 2048, 16384, 32768, },
  { 2, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 128, 2048, 16384, 32768, },
  { 4, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 128, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 2048, 16384, 32768, },
  { 8, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 128, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 16384, 32768, },
  { 16, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 16384, 32768, },
  { 256, 2048, 16384, 32768, },
  { 1, 256, 2048, 16384, 32768, },
  { 2, 256, 2048, 16384, 32768, },
  { 1, 2, 256, 2048, 16384, 32768, },
  { 4, 256, 2048, 16384, 32768, },
  { 1, 4, 256, 2048, 16384, 32768, },
  { 2, 4, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 256, 2048, 16384, 32768, },
  { 8, 256, 2048, 16384, 32768, },
  { 1, 8, 256, 2048, 16384, 32768, },
  { 2, 8, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 256, 2048, 16384, 32768, },
  { 4, 8, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 256, 2048, 16384, 32768, },
  { 16, 256, 2048, 16384, 32768, },
  { 1, 16, 256, 2048, 16384, 32768, },
  { 2, 16, 256, 2048, 16384, 32768, },
  { 1, 2, 16, 256, 2048, 16384, 32768, },
  { 4, 16, 256, 2048, 16384, 32768, },
  { 1, 4, 16, 256, 2048, 16384, 32768, },
  { 2, 4, 16, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 256, 2048, 16384, 32768, },
  { 8, 16, 256, 2048, 16384, 32768, },
  { 1, 8, 16, 256, 2048, 16384, 32768, },
  { 2, 8, 16, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 256, 2048, 16384, 32768, },
  { 4, 8, 16, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 2048, 16384, 32768, },
  { 32, 256, 2048, 16384, 32768, },
  { 1, 32, 256, 2048, 16384, 32768, },
  { 2, 32, 256, 2048, 16384, 32768, },
  { 1, 2, 32, 256, 2048, 16384, 32768, },
  { 4, 32, 256, 2048, 16384, 32768, },
  { 1, 4, 32, 256, 2048, 16384, 32768, },
  { 2, 4, 32, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 256, 2048, 16384, 32768, },
  { 8, 32, 256, 2048, 16384, 32768, },
  { 1, 8, 32, 256, 2048, 16384, 32768, },
  { 2, 8, 32, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 256, 2048, 16384, 32768, },
  { 4, 8, 32, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 2048, 16384, 32768, },
  { 16, 32, 256, 2048, 16384, 32768, },
  { 1, 16, 32, 256, 2048, 16384, 32768, },
  { 2, 16, 32, 256, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 256, 2048, 16384, 32768, },
  { 4, 16, 32, 256, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 256, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 2048, 16384, 32768, },
  { 8, 16, 32, 256, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 256, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 16384, 32768, },
  { 64, 256, 2048, 16384, 32768, },
  { 1, 64, 256, 2048, 16384, 32768, },
  { 2, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 64, 256, 2048, 16384, 32768, },
  { 4, 64, 256, 2048, 16384, 32768, },
  { 1, 4, 64, 256, 2048, 16384, 32768, },
  { 2, 4, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 256, 2048, 16384, 32768, },
  { 8, 64, 256, 2048, 16384, 32768, },
  { 1, 8, 64, 256, 2048, 16384, 32768, },
  { 2, 8, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 256, 2048, 16384, 32768, },
  { 4, 8, 64, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 2048, 16384, 32768, },
  { 16, 64, 256, 2048, 16384, 32768, },
  { 1, 16, 64, 256, 2048, 16384, 32768, },
  { 2, 16, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 256, 2048, 16384, 32768, },
  { 4, 16, 64, 256, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 256, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 2048, 16384, 32768, },
  { 8, 16, 64, 256, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 256, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 16384, 32768, },
  { 32, 64, 256, 2048, 16384, 32768, },
  { 1, 32, 64, 256, 2048, 16384, 32768, },
  { 2, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 256, 2048, 16384, 32768, },
  { 4, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 256, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 2048, 16384, 32768, },
  { 8, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 256, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 16384, 32768, },
  { 16, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 16384, 32768, },
  { 128, 256, 2048, 16384, 32768, },
  { 1, 128, 256, 2048, 16384, 32768, },
  { 2, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 128, 256, 2048, 16384, 32768, },
  { 4, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 128, 256, 2048, 16384, 32768, },
  { 8, 128, 256, 2048, 16384, 32768, },
  { 1, 8, 128, 256, 2048, 16384, 32768, },
  { 2, 8, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 128, 256, 2048, 16384, 32768, },
  { 4, 8, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 2048, 16384, 32768, },
  { 16, 128, 256, 2048, 16384, 32768, },
  { 1, 16, 128, 256, 2048, 16384, 32768, },
  { 2, 16, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 16, 128, 256, 2048, 16384, 32768, },
  { 4, 16, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 16, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 16, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 2048, 16384, 32768, },
  { 8, 16, 128, 256, 2048, 16384, 32768, },
  { 1, 8, 16, 128, 256, 2048, 16384, 32768, },
  { 2, 8, 16, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 2048, 16384, 32768, },
  { 4, 8, 16, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 16384, 32768, },
  { 32, 128, 256, 2048, 16384, 32768, },
  { 1, 32, 128, 256, 2048, 16384, 32768, },
  { 2, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 32, 128, 256, 2048, 16384, 32768, },
  { 4, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 32, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 2048, 16384, 32768, },
  { 8, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 8, 32, 128, 256, 2048, 16384, 32768, },
  { 2, 8, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 2048, 16384, 32768, },
  { 4, 8, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 16384, 32768, },
  { 16, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 2, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 4, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 8, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 16384, 32768, },
  { 64, 128, 256, 2048, 16384, 32768, },
  { 1, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 64, 128, 256, 2048, 16384, 32768, },
  { 4, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 2048, 16384, 32768, },
  { 8, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 8, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 8, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 2048, 16384, 32768, },
  { 4, 8, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 16384, 32768, },
  { 16, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 4, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 8, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 16384, 32768, },
  { 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 4, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 8, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 16384, 32768, },
  { 512, 2048, 16384, 32768, },
  { 1, 512, 2048, 16384, 32768, },
  { 2, 512, 2048, 16384, 32768, },
  { 1, 2, 512, 2048, 16384, 32768, },
  { 4, 512, 2048, 16384, 32768, },
  { 1, 4, 512, 2048, 16384, 32768, },
  { 2, 4, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 512, 2048, 16384, 32768, },
  { 8, 512, 2048, 16384, 32768, },
  { 1, 8, 512, 2048, 16384, 32768, },
  { 2, 8, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 512, 2048, 16384, 32768, },
  { 4, 8, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 512, 2048, 16384, 32768, },
  { 16, 512, 2048, 16384, 32768, },
  { 1, 16, 512, 2048, 16384, 32768, },
  { 2, 16, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 512, 2048, 16384, 32768, },
  { 4, 16, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 512, 2048, 16384, 32768, },
  { 8, 16, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 2048, 16384, 32768, },
  { 32, 512, 2048, 16384, 32768, },
  { 1, 32, 512, 2048, 16384, 32768, },
  { 2, 32, 512, 2048, 16384, 32768, },
  { 1, 2, 32, 512, 2048, 16384, 32768, },
  { 4, 32, 512, 2048, 16384, 32768, },
  { 1, 4, 32, 512, 2048, 16384, 32768, },
  { 2, 4, 32, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 512, 2048, 16384, 32768, },
  { 8, 32, 512, 2048, 16384, 32768, },
  { 1, 8, 32, 512, 2048, 16384, 32768, },
  { 2, 8, 32, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 512, 2048, 16384, 32768, },
  { 4, 8, 32, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 2048, 16384, 32768, },
  { 16, 32, 512, 2048, 16384, 32768, },
  { 1, 16, 32, 512, 2048, 16384, 32768, },
  { 2, 16, 32, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 512, 2048, 16384, 32768, },
  { 4, 16, 32, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 2048, 16384, 32768, },
  { 8, 16, 32, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 16384, 32768, },
  { 64, 512, 2048, 16384, 32768, },
  { 1, 64, 512, 2048, 16384, 32768, },
  { 2, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 64, 512, 2048, 16384, 32768, },
  { 4, 64, 512, 2048, 16384, 32768, },
  { 1, 4, 64, 512, 2048, 16384, 32768, },
  { 2, 4, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 512, 2048, 16384, 32768, },
  { 8, 64, 512, 2048, 16384, 32768, },
  { 1, 8, 64, 512, 2048, 16384, 32768, },
  { 2, 8, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 512, 2048, 16384, 32768, },
  { 4, 8, 64, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 2048, 16384, 32768, },
  { 16, 64, 512, 2048, 16384, 32768, },
  { 1, 16, 64, 512, 2048, 16384, 32768, },
  { 2, 16, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 512, 2048, 16384, 32768, },
  { 4, 16, 64, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 2048, 16384, 32768, },
  { 8, 16, 64, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 16384, 32768, },
  { 32, 64, 512, 2048, 16384, 32768, },
  { 1, 32, 64, 512, 2048, 16384, 32768, },
  { 2, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 512, 2048, 16384, 32768, },
  { 4, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 512, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 2048, 16384, 32768, },
  { 8, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 512, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 16384, 32768, },
  { 16, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 16384, 32768, },
  { 128, 512, 2048, 16384, 32768, },
  { 1, 128, 512, 2048, 16384, 32768, },
  { 2, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 128, 512, 2048, 16384, 32768, },
  { 4, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 128, 512, 2048, 16384, 32768, },
  { 8, 128, 512, 2048, 16384, 32768, },
  { 1, 8, 128, 512, 2048, 16384, 32768, },
  { 2, 8, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 128, 512, 2048, 16384, 32768, },
  { 4, 8, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 2048, 16384, 32768, },
  { 16, 128, 512, 2048, 16384, 32768, },
  { 1, 16, 128, 512, 2048, 16384, 32768, },
  { 2, 16, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 128, 512, 2048, 16384, 32768, },
  { 4, 16, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 2048, 16384, 32768, },
  { 8, 16, 128, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 128, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 16384, 32768, },
  { 32, 128, 512, 2048, 16384, 32768, },
  { 1, 32, 128, 512, 2048, 16384, 32768, },
  { 2, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 32, 128, 512, 2048, 16384, 32768, },
  { 4, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 32, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 2048, 16384, 32768, },
  { 8, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 8, 32, 128, 512, 2048, 16384, 32768, },
  { 2, 8, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 2048, 16384, 32768, },
  { 4, 8, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 16384, 32768, },
  { 16, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 2, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 4, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 8, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 16384, 32768, },
  { 64, 128, 512, 2048, 16384, 32768, },
  { 1, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 64, 128, 512, 2048, 16384, 32768, },
  { 4, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 2048, 16384, 32768, },
  { 8, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 8, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 8, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 2048, 16384, 32768, },
  { 4, 8, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 16384, 32768, },
  { 16, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 4, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 8, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 16384, 32768, },
  { 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 4, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 8, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 16384, 32768, },
  { 256, 512, 2048, 16384, 32768, },
  { 1, 256, 512, 2048, 16384, 32768, },
  { 2, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 256, 512, 2048, 16384, 32768, },
  { 4, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 256, 512, 2048, 16384, 32768, },
  { 8, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 2048, 16384, 32768, },
  { 16, 256, 512, 2048, 16384, 32768, },
  { 1, 16, 256, 512, 2048, 16384, 32768, },
  { 2, 16, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 256, 512, 2048, 16384, 32768, },
  { 4, 16, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 2048, 16384, 32768, },
  { 8, 16, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 16384, 32768, },
  { 32, 256, 512, 2048, 16384, 32768, },
  { 1, 32, 256, 512, 2048, 16384, 32768, },
  { 2, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 32, 256, 512, 2048, 16384, 32768, },
  { 4, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 32, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 2048, 16384, 32768, },
  { 8, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 32, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 16384, 32768, },
  { 16, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 2, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 4, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 8, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 16384, 32768, },
  { 64, 256, 512, 2048, 16384, 32768, },
  { 1, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 64, 256, 512, 2048, 16384, 32768, },
  { 4, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 2048, 16384, 32768, },
  { 8, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 16384, 32768, },
  { 16, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 4, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 8, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 16384, 32768, },
  { 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 4, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 8, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 16384, 32768, },
  { 128, 256, 512, 2048, 16384, 32768, },
  { 1, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 2048, 16384, 32768, },
  { 8, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 16384, 32768, },
  { 16, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 8, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 16384, 32768, },
  { 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 8, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 16384, 32768, },
  { 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 8, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 16384, 32768, },
  { 1024, 2048, 16384, 32768, },
  { 1, 1024, 2048, 16384, 32768, },
  { 2, 1024, 2048, 16384, 32768, },
  { 1, 2, 1024, 2048, 16384, 32768, },
  { 4, 1024, 2048, 16384, 32768, },
  { 1, 4, 1024, 2048, 16384, 32768, },
  { 2, 4, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 1024, 2048, 16384, 32768, },
  { 8, 1024, 2048, 16384, 32768, },
  { 1, 8, 1024, 2048, 16384, 32768, },
  { 2, 8, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 1024, 2048, 16384, 32768, },
  { 4, 8, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 1024, 2048, 16384, 32768, },
  { 16, 1024, 2048, 16384, 32768, },
  { 1, 16, 1024, 2048, 16384, 32768, },
  { 2, 16, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 1024, 2048, 16384, 32768, },
  { 4, 16, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 1024, 2048, 16384, 32768, },
  { 8, 16, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 1024, 2048, 16384, 32768, },
  { 32, 1024, 2048, 16384, 32768, },
  { 1, 32, 1024, 2048, 16384, 32768, },
  { 2, 32, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 1024, 2048, 16384, 32768, },
  { 4, 32, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 1024, 2048, 16384, 32768, },
  { 8, 32, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 1024, 2048, 16384, 32768, },
  { 16, 32, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 16384, 32768, },
  { 64, 1024, 2048, 16384, 32768, },
  { 1, 64, 1024, 2048, 16384, 32768, },
  { 2, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 64, 1024, 2048, 16384, 32768, },
  { 4, 64, 1024, 2048, 16384, 32768, },
  { 1, 4, 64, 1024, 2048, 16384, 32768, },
  { 2, 4, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 1024, 2048, 16384, 32768, },
  { 8, 64, 1024, 2048, 16384, 32768, },
  { 1, 8, 64, 1024, 2048, 16384, 32768, },
  { 2, 8, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 1024, 2048, 16384, 32768, },
  { 4, 8, 64, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 1024, 2048, 16384, 32768, },
  { 16, 64, 1024, 2048, 16384, 32768, },
  { 1, 16, 64, 1024, 2048, 16384, 32768, },
  { 2, 16, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 1024, 2048, 16384, 32768, },
  { 4, 16, 64, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 1024, 2048, 16384, 32768, },
  { 8, 16, 64, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 16384, 32768, },
  { 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 32, 64, 1024, 2048, 16384, 32768, },
  { 2, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 1024, 2048, 16384, 32768, },
  { 4, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 1024, 2048, 16384, 32768, },
  { 8, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 16384, 32768, },
  { 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 16384, 32768, },
  { 128, 1024, 2048, 16384, 32768, },
  { 1, 128, 1024, 2048, 16384, 32768, },
  { 2, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 128, 1024, 2048, 16384, 32768, },
  { 4, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 128, 1024, 2048, 16384, 32768, },
  { 8, 128, 1024, 2048, 16384, 32768, },
  { 1, 8, 128, 1024, 2048, 16384, 32768, },
  { 2, 8, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 128, 1024, 2048, 16384, 32768, },
  { 4, 8, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 128, 1024, 2048, 16384, 32768, },
  { 16, 128, 1024, 2048, 16384, 32768, },
  { 1, 16, 128, 1024, 2048, 16384, 32768, },
  { 2, 16, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 128, 1024, 2048, 16384, 32768, },
  { 4, 16, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 128, 1024, 2048, 16384, 32768, },
  { 8, 16, 128, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 128, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 128, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 16384, 32768, },
  { 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 32, 128, 1024, 2048, 16384, 32768, },
  { 2, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 128, 1024, 2048, 16384, 32768, },
  { 4, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 128, 1024, 2048, 16384, 32768, },
  { 8, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 128, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 128, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 16384, 32768, },
  { 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 16384, 32768, },
  { 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 64, 128, 1024, 2048, 16384, 32768, },
  { 4, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 128, 1024, 2048, 16384, 32768, },
  { 8, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 8, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 8, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 128, 1024, 2048, 16384, 32768, },
  { 4, 8, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 16384, 32768, },
  { 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 4, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 8, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 16384, 32768, },
  { 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 4, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 8, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 16384, 32768, },
  { 256, 1024, 2048, 16384, 32768, },
  { 1, 256, 1024, 2048, 16384, 32768, },
  { 2, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 256, 1024, 2048, 16384, 32768, },
  { 4, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 256, 1024, 2048, 16384, 32768, },
  { 8, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 256, 1024, 2048, 16384, 32768, },
  { 16, 256, 1024, 2048, 16384, 32768, },
  { 1, 16, 256, 1024, 2048, 16384, 32768, },
  { 2, 16, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 256, 1024, 2048, 16384, 32768, },
  { 4, 16, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 256, 1024, 2048, 16384, 32768, },
  { 8, 16, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 16384, 32768, },
  { 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 32, 256, 1024, 2048, 16384, 32768, },
  { 2, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 256, 1024, 2048, 16384, 32768, },
  { 4, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 256, 1024, 2048, 16384, 32768, },
  { 8, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 16384, 32768, },
  { 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 16384, 32768, },
  { 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 64, 256, 1024, 2048, 16384, 32768, },
  { 4, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 256, 1024, 2048, 16384, 32768, },
  { 8, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 16384, 32768, },
  { 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 4, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 8, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 16384, 32768, },
  { 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 4, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 8, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 16384, 32768, },
  { 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 128, 256, 1024, 2048, 16384, 32768, },
  { 8, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 16384, 32768, },
  { 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 8, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 16384, 32768, },
  { 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 8, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 16384, 32768, },
  { 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 8, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 8, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 8, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 16384, 32768, },
  { 512, 1024, 2048, 16384, 32768, },
  { 1, 512, 1024, 2048, 16384, 32768, },
  { 2, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 512, 1024, 2048, 16384, 32768, },
  { 4, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 512, 1024, 2048, 16384, 32768, },
  { 8, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 512, 1024, 2048, 16384, 32768, },
  { 16, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 16384, 32768, },
  { 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 32, 512, 1024, 2048, 16384, 32768, },
  { 2, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 512, 1024, 2048, 16384, 32768, },
  { 4, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 512, 1024, 2048, 16384, 32768, },
  { 8, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 16384, 32768, },
  { 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 16384, 32768, },
  { 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 64, 512, 1024, 2048, 16384, 32768, },
  { 4, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 512, 1024, 2048, 16384, 32768, },
  { 8, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 16384, 32768, },
  { 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 16384, 32768, },
  { 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 4, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 8, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 16384, 32768, },
  { 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 128, 512, 1024, 2048, 16384, 32768, },
  { 8, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 16384, 32768, },
  { 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 16384, 32768, },
  { 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 8, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 16384, 32768, },
  { 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 8, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 8, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 16384, 32768, },
  { 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 16384, 32768, },
  { 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 16384, 32768, },
  { 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 16384, 32768, },
  { 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 16384, 32768, },
  { 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 16384, 32768, },
  { 4096, 16384, 32768, },
  { 1, 4096, 16384, 32768, },
  { 2, 4096, 16384, 32768, },
  { 1, 2, 4096, 16384, 32768, },
  { 4, 4096, 16384, 32768, },
  { 1, 4, 4096, 16384, 32768, },
  { 2, 4, 4096, 16384, 32768, },
  { 1, 2, 4, 4096, 16384, 32768, },
  { 8, 4096, 16384, 32768, },
  { 1, 8, 4096, 16384, 32768, },
  { 2, 8, 4096, 16384, 32768, },
  { 1, 2, 8, 4096, 16384, 32768, },
  { 4, 8, 4096, 16384, 32768, },
  { 1, 4, 8, 4096, 16384, 32768, },
  { 2, 4, 8, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 4096, 16384, 32768, },
  { 16, 4096, 16384, 32768, },
  { 1, 16, 4096, 16384, 32768, },
  { 2, 16, 4096, 16384, 32768, },
  { 1, 2, 16, 4096, 16384, 32768, },
  { 4, 16, 4096, 16384, 32768, },
  { 1, 4, 16, 4096, 16384, 32768, },
  { 2, 4, 16, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 4096, 16384, 32768, },
  { 8, 16, 4096, 16384, 32768, },
  { 1, 8, 16, 4096, 16384, 32768, },
  { 2, 8, 16, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 4096, 16384, 32768, },
  { 4, 8, 16, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 4096, 16384, 32768, },
  { 32, 4096, 16384, 32768, },
  { 1, 32, 4096, 16384, 32768, },
  { 2, 32, 4096, 16384, 32768, },
  { 1, 2, 32, 4096, 16384, 32768, },
  { 4, 32, 4096, 16384, 32768, },
  { 1, 4, 32, 4096, 16384, 32768, },
  { 2, 4, 32, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 4096, 16384, 32768, },
  { 8, 32, 4096, 16384, 32768, },
  { 1, 8, 32, 4096, 16384, 32768, },
  { 2, 8, 32, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 4096, 16384, 32768, },
  { 4, 8, 32, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 4096, 16384, 32768, },
  { 16, 32, 4096, 16384, 32768, },
  { 1, 16, 32, 4096, 16384, 32768, },
  { 2, 16, 32, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 4096, 16384, 32768, },
  { 4, 16, 32, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 4096, 16384, 32768, },
  { 8, 16, 32, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 4096, 16384, 32768, },
  { 64, 4096, 16384, 32768, },
  { 1, 64, 4096, 16384, 32768, },
  { 2, 64, 4096, 16384, 32768, },
  { 1, 2, 64, 4096, 16384, 32768, },
  { 4, 64, 4096, 16384, 32768, },
  { 1, 4, 64, 4096, 16384, 32768, },
  { 2, 4, 64, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 4096, 16384, 32768, },
  { 8, 64, 4096, 16384, 32768, },
  { 1, 8, 64, 4096, 16384, 32768, },
  { 2, 8, 64, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 4096, 16384, 32768, },
  { 4, 8, 64, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 4096, 16384, 32768, },
  { 16, 64, 4096, 16384, 32768, },
  { 1, 16, 64, 4096, 16384, 32768, },
  { 2, 16, 64, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 4096, 16384, 32768, },
  { 4, 16, 64, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 4096, 16384, 32768, },
  { 8, 16, 64, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 4096, 16384, 32768, },
  { 32, 64, 4096, 16384, 32768, },
  { 1, 32, 64, 4096, 16384, 32768, },
  { 2, 32, 64, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 4096, 16384, 32768, },
  { 4, 32, 64, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 4096, 16384, 32768, },
  { 8, 32, 64, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 4096, 16384, 32768, },
  { 16, 32, 64, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 4096, 16384, 32768, },
  { 128, 4096, 16384, 32768, },
  { 1, 128, 4096, 16384, 32768, },
  { 2, 128, 4096, 16384, 32768, },
  { 1, 2, 128, 4096, 16384, 32768, },
  { 4, 128, 4096, 16384, 32768, },
  { 1, 4, 128, 4096, 16384, 32768, },
  { 2, 4, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 4096, 16384, 32768, },
  { 8, 128, 4096, 16384, 32768, },
  { 1, 8, 128, 4096, 16384, 32768, },
  { 2, 8, 128, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 4096, 16384, 32768, },
  { 4, 8, 128, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 4096, 16384, 32768, },
  { 16, 128, 4096, 16384, 32768, },
  { 1, 16, 128, 4096, 16384, 32768, },
  { 2, 16, 128, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 4096, 16384, 32768, },
  { 4, 16, 128, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 4096, 16384, 32768, },
  { 8, 16, 128, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 4096, 16384, 32768, },
  { 32, 128, 4096, 16384, 32768, },
  { 1, 32, 128, 4096, 16384, 32768, },
  { 2, 32, 128, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 4096, 16384, 32768, },
  { 4, 32, 128, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 4096, 16384, 32768, },
  { 8, 32, 128, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 4096, 16384, 32768, },
  { 16, 32, 128, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 4096, 16384, 32768, },
  { 64, 128, 4096, 16384, 32768, },
  { 1, 64, 128, 4096, 16384, 32768, },
  { 2, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 4096, 16384, 32768, },
  { 4, 64, 128, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 4096, 16384, 32768, },
  { 8, 64, 128, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 4096, 16384, 32768, },
  { 16, 64, 128, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 4096, 16384, 32768, },
  { 32, 64, 128, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 4096, 16384, 32768, },
  { 256, 4096, 16384, 32768, },
  { 1, 256, 4096, 16384, 32768, },
  { 2, 256, 4096, 16384, 32768, },
  { 1, 2, 256, 4096, 16384, 32768, },
  { 4, 256, 4096, 16384, 32768, },
  { 1, 4, 256, 4096, 16384, 32768, },
  { 2, 4, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 256, 4096, 16384, 32768, },
  { 8, 256, 4096, 16384, 32768, },
  { 1, 8, 256, 4096, 16384, 32768, },
  { 2, 8, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 256, 4096, 16384, 32768, },
  { 4, 8, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 256, 4096, 16384, 32768, },
  { 16, 256, 4096, 16384, 32768, },
  { 1, 16, 256, 4096, 16384, 32768, },
  { 2, 16, 256, 4096, 16384, 32768, },
  { 1, 2, 16, 256, 4096, 16384, 32768, },
  { 4, 16, 256, 4096, 16384, 32768, },
  { 1, 4, 16, 256, 4096, 16384, 32768, },
  { 2, 4, 16, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 256, 4096, 16384, 32768, },
  { 8, 16, 256, 4096, 16384, 32768, },
  { 1, 8, 16, 256, 4096, 16384, 32768, },
  { 2, 8, 16, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 256, 4096, 16384, 32768, },
  { 4, 8, 16, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 4096, 16384, 32768, },
  { 32, 256, 4096, 16384, 32768, },
  { 1, 32, 256, 4096, 16384, 32768, },
  { 2, 32, 256, 4096, 16384, 32768, },
  { 1, 2, 32, 256, 4096, 16384, 32768, },
  { 4, 32, 256, 4096, 16384, 32768, },
  { 1, 4, 32, 256, 4096, 16384, 32768, },
  { 2, 4, 32, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 256, 4096, 16384, 32768, },
  { 8, 32, 256, 4096, 16384, 32768, },
  { 1, 8, 32, 256, 4096, 16384, 32768, },
  { 2, 8, 32, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 256, 4096, 16384, 32768, },
  { 4, 8, 32, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 4096, 16384, 32768, },
  { 16, 32, 256, 4096, 16384, 32768, },
  { 1, 16, 32, 256, 4096, 16384, 32768, },
  { 2, 16, 32, 256, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 256, 4096, 16384, 32768, },
  { 4, 16, 32, 256, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 256, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 4096, 16384, 32768, },
  { 8, 16, 32, 256, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 256, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 4096, 16384, 32768, },
  { 64, 256, 4096, 16384, 32768, },
  { 1, 64, 256, 4096, 16384, 32768, },
  { 2, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 64, 256, 4096, 16384, 32768, },
  { 4, 64, 256, 4096, 16384, 32768, },
  { 1, 4, 64, 256, 4096, 16384, 32768, },
  { 2, 4, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 256, 4096, 16384, 32768, },
  { 8, 64, 256, 4096, 16384, 32768, },
  { 1, 8, 64, 256, 4096, 16384, 32768, },
  { 2, 8, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 256, 4096, 16384, 32768, },
  { 4, 8, 64, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 4096, 16384, 32768, },
  { 16, 64, 256, 4096, 16384, 32768, },
  { 1, 16, 64, 256, 4096, 16384, 32768, },
  { 2, 16, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 256, 4096, 16384, 32768, },
  { 4, 16, 64, 256, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 256, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 4096, 16384, 32768, },
  { 8, 16, 64, 256, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 256, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 4096, 16384, 32768, },
  { 32, 64, 256, 4096, 16384, 32768, },
  { 1, 32, 64, 256, 4096, 16384, 32768, },
  { 2, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 256, 4096, 16384, 32768, },
  { 4, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 256, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 4096, 16384, 32768, },
  { 8, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 256, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 4096, 16384, 32768, },
  { 16, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 4096, 16384, 32768, },
  { 128, 256, 4096, 16384, 32768, },
  { 1, 128, 256, 4096, 16384, 32768, },
  { 2, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 128, 256, 4096, 16384, 32768, },
  { 4, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 256, 4096, 16384, 32768, },
  { 8, 128, 256, 4096, 16384, 32768, },
  { 1, 8, 128, 256, 4096, 16384, 32768, },
  { 2, 8, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 256, 4096, 16384, 32768, },
  { 4, 8, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 4096, 16384, 32768, },
  { 16, 128, 256, 4096, 16384, 32768, },
  { 1, 16, 128, 256, 4096, 16384, 32768, },
  { 2, 16, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 256, 4096, 16384, 32768, },
  { 4, 16, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 4096, 16384, 32768, },
  { 8, 16, 128, 256, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 256, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 4096, 16384, 32768, },
  { 32, 128, 256, 4096, 16384, 32768, },
  { 1, 32, 128, 256, 4096, 16384, 32768, },
  { 2, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 256, 4096, 16384, 32768, },
  { 4, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 4096, 16384, 32768, },
  { 8, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 256, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 4096, 16384, 32768, },
  { 16, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 4096, 16384, 32768, },
  { 64, 128, 256, 4096, 16384, 32768, },
  { 1, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 256, 4096, 16384, 32768, },
  { 4, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 4096, 16384, 32768, },
  { 8, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 4096, 16384, 32768, },
  { 16, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 4096, 16384, 32768, },
  { 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 4096, 16384, 32768, },
  { 512, 4096, 16384, 32768, },
  { 1, 512, 4096, 16384, 32768, },
  { 2, 512, 4096, 16384, 32768, },
  { 1, 2, 512, 4096, 16384, 32768, },
  { 4, 512, 4096, 16384, 32768, },
  { 1, 4, 512, 4096, 16384, 32768, },
  { 2, 4, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 512, 4096, 16384, 32768, },
  { 8, 512, 4096, 16384, 32768, },
  { 1, 8, 512, 4096, 16384, 32768, },
  { 2, 8, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 512, 4096, 16384, 32768, },
  { 4, 8, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 512, 4096, 16384, 32768, },
  { 16, 512, 4096, 16384, 32768, },
  { 1, 16, 512, 4096, 16384, 32768, },
  { 2, 16, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 512, 4096, 16384, 32768, },
  { 4, 16, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 512, 4096, 16384, 32768, },
  { 8, 16, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 4096, 16384, 32768, },
  { 32, 512, 4096, 16384, 32768, },
  { 1, 32, 512, 4096, 16384, 32768, },
  { 2, 32, 512, 4096, 16384, 32768, },
  { 1, 2, 32, 512, 4096, 16384, 32768, },
  { 4, 32, 512, 4096, 16384, 32768, },
  { 1, 4, 32, 512, 4096, 16384, 32768, },
  { 2, 4, 32, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 512, 4096, 16384, 32768, },
  { 8, 32, 512, 4096, 16384, 32768, },
  { 1, 8, 32, 512, 4096, 16384, 32768, },
  { 2, 8, 32, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 512, 4096, 16384, 32768, },
  { 4, 8, 32, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 4096, 16384, 32768, },
  { 16, 32, 512, 4096, 16384, 32768, },
  { 1, 16, 32, 512, 4096, 16384, 32768, },
  { 2, 16, 32, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 512, 4096, 16384, 32768, },
  { 4, 16, 32, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 4096, 16384, 32768, },
  { 8, 16, 32, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 4096, 16384, 32768, },
  { 64, 512, 4096, 16384, 32768, },
  { 1, 64, 512, 4096, 16384, 32768, },
  { 2, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 64, 512, 4096, 16384, 32768, },
  { 4, 64, 512, 4096, 16384, 32768, },
  { 1, 4, 64, 512, 4096, 16384, 32768, },
  { 2, 4, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 512, 4096, 16384, 32768, },
  { 8, 64, 512, 4096, 16384, 32768, },
  { 1, 8, 64, 512, 4096, 16384, 32768, },
  { 2, 8, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 512, 4096, 16384, 32768, },
  { 4, 8, 64, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 4096, 16384, 32768, },
  { 16, 64, 512, 4096, 16384, 32768, },
  { 1, 16, 64, 512, 4096, 16384, 32768, },
  { 2, 16, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 512, 4096, 16384, 32768, },
  { 4, 16, 64, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 4096, 16384, 32768, },
  { 8, 16, 64, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 4096, 16384, 32768, },
  { 32, 64, 512, 4096, 16384, 32768, },
  { 1, 32, 64, 512, 4096, 16384, 32768, },
  { 2, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 512, 4096, 16384, 32768, },
  { 4, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 512, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 4096, 16384, 32768, },
  { 8, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 512, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 4096, 16384, 32768, },
  { 16, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 4096, 16384, 32768, },
  { 128, 512, 4096, 16384, 32768, },
  { 1, 128, 512, 4096, 16384, 32768, },
  { 2, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 128, 512, 4096, 16384, 32768, },
  { 4, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 512, 4096, 16384, 32768, },
  { 8, 128, 512, 4096, 16384, 32768, },
  { 1, 8, 128, 512, 4096, 16384, 32768, },
  { 2, 8, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 512, 4096, 16384, 32768, },
  { 4, 8, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 4096, 16384, 32768, },
  { 16, 128, 512, 4096, 16384, 32768, },
  { 1, 16, 128, 512, 4096, 16384, 32768, },
  { 2, 16, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 512, 4096, 16384, 32768, },
  { 4, 16, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 4096, 16384, 32768, },
  { 8, 16, 128, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 4096, 16384, 32768, },
  { 32, 128, 512, 4096, 16384, 32768, },
  { 1, 32, 128, 512, 4096, 16384, 32768, },
  { 2, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 512, 4096, 16384, 32768, },
  { 4, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 4096, 16384, 32768, },
  { 8, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 512, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 4096, 16384, 32768, },
  { 16, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 4096, 16384, 32768, },
  { 64, 128, 512, 4096, 16384, 32768, },
  { 1, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 512, 4096, 16384, 32768, },
  { 4, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 4096, 16384, 32768, },
  { 8, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 4096, 16384, 32768, },
  { 16, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 4096, 16384, 32768, },
  { 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 4096, 16384, 32768, },
  { 256, 512, 4096, 16384, 32768, },
  { 1, 256, 512, 4096, 16384, 32768, },
  { 2, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 256, 512, 4096, 16384, 32768, },
  { 4, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 256, 512, 4096, 16384, 32768, },
  { 8, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 4096, 16384, 32768, },
  { 16, 256, 512, 4096, 16384, 32768, },
  { 1, 16, 256, 512, 4096, 16384, 32768, },
  { 2, 16, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 256, 512, 4096, 16384, 32768, },
  { 4, 16, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 4096, 16384, 32768, },
  { 8, 16, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 4096, 16384, 32768, },
  { 32, 256, 512, 4096, 16384, 32768, },
  { 1, 32, 256, 512, 4096, 16384, 32768, },
  { 2, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 32, 256, 512, 4096, 16384, 32768, },
  { 4, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 32, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 4096, 16384, 32768, },
  { 8, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 32, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 4096, 16384, 32768, },
  { 16, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 2, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 4, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 8, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 4096, 16384, 32768, },
  { 64, 256, 512, 4096, 16384, 32768, },
  { 1, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 64, 256, 512, 4096, 16384, 32768, },
  { 4, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 4096, 16384, 32768, },
  { 8, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 4096, 16384, 32768, },
  { 16, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 4, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 8, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 4096, 16384, 32768, },
  { 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 4, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 8, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 4096, 16384, 32768, },
  { 128, 256, 512, 4096, 16384, 32768, },
  { 1, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 4096, 16384, 32768, },
  { 8, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 4096, 16384, 32768, },
  { 16, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 8, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 4096, 16384, 32768, },
  { 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 8, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 4096, 16384, 32768, },
  { 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 8, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 16384, 32768, },
  { 1024, 4096, 16384, 32768, },
  { 1, 1024, 4096, 16384, 32768, },
  { 2, 1024, 4096, 16384, 32768, },
  { 1, 2, 1024, 4096, 16384, 32768, },
  { 4, 1024, 4096, 16384, 32768, },
  { 1, 4, 1024, 4096, 16384, 32768, },
  { 2, 4, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 1024, 4096, 16384, 32768, },
  { 8, 1024, 4096, 16384, 32768, },
  { 1, 8, 1024, 4096, 16384, 32768, },
  { 2, 8, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 1024, 4096, 16384, 32768, },
  { 4, 8, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 1024, 4096, 16384, 32768, },
  { 16, 1024, 4096, 16384, 32768, },
  { 1, 16, 1024, 4096, 16384, 32768, },
  { 2, 16, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 1024, 4096, 16384, 32768, },
  { 4, 16, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 1024, 4096, 16384, 32768, },
  { 8, 16, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 1024, 4096, 16384, 32768, },
  { 32, 1024, 4096, 16384, 32768, },
  { 1, 32, 1024, 4096, 16384, 32768, },
  { 2, 32, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 1024, 4096, 16384, 32768, },
  { 4, 32, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 1024, 4096, 16384, 32768, },
  { 8, 32, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 1024, 4096, 16384, 32768, },
  { 16, 32, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 4096, 16384, 32768, },
  { 64, 1024, 4096, 16384, 32768, },
  { 1, 64, 1024, 4096, 16384, 32768, },
  { 2, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 64, 1024, 4096, 16384, 32768, },
  { 4, 64, 1024, 4096, 16384, 32768, },
  { 1, 4, 64, 1024, 4096, 16384, 32768, },
  { 2, 4, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 1024, 4096, 16384, 32768, },
  { 8, 64, 1024, 4096, 16384, 32768, },
  { 1, 8, 64, 1024, 4096, 16384, 32768, },
  { 2, 8, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 1024, 4096, 16384, 32768, },
  { 4, 8, 64, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 1024, 4096, 16384, 32768, },
  { 16, 64, 1024, 4096, 16384, 32768, },
  { 1, 16, 64, 1024, 4096, 16384, 32768, },
  { 2, 16, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 1024, 4096, 16384, 32768, },
  { 4, 16, 64, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 1024, 4096, 16384, 32768, },
  { 8, 16, 64, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 4096, 16384, 32768, },
  { 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 32, 64, 1024, 4096, 16384, 32768, },
  { 2, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 1024, 4096, 16384, 32768, },
  { 4, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 1024, 4096, 16384, 32768, },
  { 8, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 4096, 16384, 32768, },
  { 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 4096, 16384, 32768, },
  { 128, 1024, 4096, 16384, 32768, },
  { 1, 128, 1024, 4096, 16384, 32768, },
  { 2, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 128, 1024, 4096, 16384, 32768, },
  { 4, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 1024, 4096, 16384, 32768, },
  { 8, 128, 1024, 4096, 16384, 32768, },
  { 1, 8, 128, 1024, 4096, 16384, 32768, },
  { 2, 8, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 1024, 4096, 16384, 32768, },
  { 4, 8, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 1024, 4096, 16384, 32768, },
  { 16, 128, 1024, 4096, 16384, 32768, },
  { 1, 16, 128, 1024, 4096, 16384, 32768, },
  { 2, 16, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 1024, 4096, 16384, 32768, },
  { 4, 16, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 1024, 4096, 16384, 32768, },
  { 8, 16, 128, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 4096, 16384, 32768, },
  { 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 32, 128, 1024, 4096, 16384, 32768, },
  { 2, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 1024, 4096, 16384, 32768, },
  { 4, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 1024, 4096, 16384, 32768, },
  { 8, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 4096, 16384, 32768, },
  { 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 4096, 16384, 32768, },
  { 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 1024, 4096, 16384, 32768, },
  { 4, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 1024, 4096, 16384, 32768, },
  { 8, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 1024, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 4096, 16384, 32768, },
  { 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 4096, 16384, 32768, },
  { 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 16384, 32768, },
  { 256, 1024, 4096, 16384, 32768, },
  { 1, 256, 1024, 4096, 16384, 32768, },
  { 2, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 256, 1024, 4096, 16384, 32768, },
  { 4, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 256, 1024, 4096, 16384, 32768, },
  { 8, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 256, 1024, 4096, 16384, 32768, },
  { 16, 256, 1024, 4096, 16384, 32768, },
  { 1, 16, 256, 1024, 4096, 16384, 32768, },
  { 2, 16, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 256, 1024, 4096, 16384, 32768, },
  { 4, 16, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 256, 1024, 4096, 16384, 32768, },
  { 8, 16, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 4096, 16384, 32768, },
  { 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 32, 256, 1024, 4096, 16384, 32768, },
  { 2, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 256, 1024, 4096, 16384, 32768, },
  { 4, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 256, 1024, 4096, 16384, 32768, },
  { 8, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 4096, 16384, 32768, },
  { 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 4096, 16384, 32768, },
  { 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 64, 256, 1024, 4096, 16384, 32768, },
  { 4, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 256, 1024, 4096, 16384, 32768, },
  { 8, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 4096, 16384, 32768, },
  { 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 4, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 8, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 4096, 16384, 32768, },
  { 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 4, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 8, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 16384, 32768, },
  { 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 256, 1024, 4096, 16384, 32768, },
  { 8, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 4096, 16384, 32768, },
  { 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 8, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 4096, 16384, 32768, },
  { 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 8, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 16384, 32768, },
  { 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 8, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 16384, 32768, },
  { 512, 1024, 4096, 16384, 32768, },
  { 1, 512, 1024, 4096, 16384, 32768, },
  { 2, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 512, 1024, 4096, 16384, 32768, },
  { 4, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 512, 1024, 4096, 16384, 32768, },
  { 8, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 512, 1024, 4096, 16384, 32768, },
  { 16, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 4096, 16384, 32768, },
  { 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 32, 512, 1024, 4096, 16384, 32768, },
  { 2, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 512, 1024, 4096, 16384, 32768, },
  { 4, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 512, 1024, 4096, 16384, 32768, },
  { 8, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 4096, 16384, 32768, },
  { 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 4096, 16384, 32768, },
  { 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 64, 512, 1024, 4096, 16384, 32768, },
  { 4, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 512, 1024, 4096, 16384, 32768, },
  { 8, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 4096, 16384, 32768, },
  { 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 4096, 16384, 32768, },
  { 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 4, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 8, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 16384, 32768, },
  { 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 512, 1024, 4096, 16384, 32768, },
  { 8, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 4096, 16384, 32768, },
  { 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 4096, 16384, 32768, },
  { 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 8, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 16384, 32768, },
  { 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 8, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 16384, 32768, },
  { 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 4096, 16384, 32768, },
  { 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 4096, 16384, 32768, },
  { 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 16384, 32768, },
  { 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 16384, 32768, },
  { 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 16384, 32768, },
  { 2048, 4096, 16384, 32768, },
  { 1, 2048, 4096, 16384, 32768, },
  { 2, 2048, 4096, 16384, 32768, },
  { 1, 2, 2048, 4096, 16384, 32768, },
  { 4, 2048, 4096, 16384, 32768, },
  { 1, 4, 2048, 4096, 16384, 32768, },
  { 2, 4, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 2048, 4096, 16384, 32768, },
  { 8, 2048, 4096, 16384, 32768, },
  { 1, 8, 2048, 4096, 16384, 32768, },
  { 2, 8, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 2048, 4096, 16384, 32768, },
  { 4, 8, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 2048, 4096, 16384, 32768, },
  { 16, 2048, 4096, 16384, 32768, },
  { 1, 16, 2048, 4096, 16384, 32768, },
  { 2, 16, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 2048, 4096, 16384, 32768, },
  { 4, 16, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 2048, 4096, 16384, 32768, },
  { 8, 16, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 2048, 4096, 16384, 32768, },
  { 32, 2048, 4096, 16384, 32768, },
  { 1, 32, 2048, 4096, 16384, 32768, },
  { 2, 32, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 2048, 4096, 16384, 32768, },
  { 4, 32, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 2048, 4096, 16384, 32768, },
  { 8, 32, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 2048, 4096, 16384, 32768, },
  { 16, 32, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 2048, 4096, 16384, 32768, },
  { 64, 2048, 4096, 16384, 32768, },
  { 1, 64, 2048, 4096, 16384, 32768, },
  { 2, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 2048, 4096, 16384, 32768, },
  { 4, 64, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 2048, 4096, 16384, 32768, },
  { 8, 64, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 2048, 4096, 16384, 32768, },
  { 16, 64, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 2048, 4096, 16384, 32768, },
  { 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 4096, 16384, 32768, },
  { 128, 2048, 4096, 16384, 32768, },
  { 1, 128, 2048, 4096, 16384, 32768, },
  { 2, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 128, 2048, 4096, 16384, 32768, },
  { 4, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 2048, 4096, 16384, 32768, },
  { 8, 128, 2048, 4096, 16384, 32768, },
  { 1, 8, 128, 2048, 4096, 16384, 32768, },
  { 2, 8, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 2048, 4096, 16384, 32768, },
  { 4, 8, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 2048, 4096, 16384, 32768, },
  { 16, 128, 2048, 4096, 16384, 32768, },
  { 1, 16, 128, 2048, 4096, 16384, 32768, },
  { 2, 16, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 2048, 4096, 16384, 32768, },
  { 4, 16, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 2048, 4096, 16384, 32768, },
  { 8, 16, 128, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 2048, 4096, 16384, 32768, },
  { 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 32, 128, 2048, 4096, 16384, 32768, },
  { 2, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 2048, 4096, 16384, 32768, },
  { 4, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 2048, 4096, 16384, 32768, },
  { 8, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 2048, 4096, 16384, 32768, },
  { 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 4096, 16384, 32768, },
  { 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 2048, 4096, 16384, 32768, },
  { 4, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 2048, 4096, 16384, 32768, },
  { 8, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 2048, 4096, 16384, 32768, },
  { 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 4096, 16384, 32768, },
  { 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 16384, 32768, },
  { 256, 2048, 4096, 16384, 32768, },
  { 1, 256, 2048, 4096, 16384, 32768, },
  { 2, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 256, 2048, 4096, 16384, 32768, },
  { 4, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 256, 2048, 4096, 16384, 32768, },
  { 8, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 256, 2048, 4096, 16384, 32768, },
  { 16, 256, 2048, 4096, 16384, 32768, },
  { 1, 16, 256, 2048, 4096, 16384, 32768, },
  { 2, 16, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 256, 2048, 4096, 16384, 32768, },
  { 4, 16, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 256, 2048, 4096, 16384, 32768, },
  { 8, 16, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 2048, 4096, 16384, 32768, },
  { 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 32, 256, 2048, 4096, 16384, 32768, },
  { 2, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 256, 2048, 4096, 16384, 32768, },
  { 4, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 256, 2048, 4096, 16384, 32768, },
  { 8, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 2048, 4096, 16384, 32768, },
  { 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 4096, 16384, 32768, },
  { 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 256, 2048, 4096, 16384, 32768, },
  { 4, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 256, 2048, 4096, 16384, 32768, },
  { 8, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 2048, 4096, 16384, 32768, },
  { 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 4096, 16384, 32768, },
  { 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 16384, 32768, },
  { 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 256, 2048, 4096, 16384, 32768, },
  { 8, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 2048, 4096, 16384, 32768, },
  { 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 8, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 4096, 16384, 32768, },
  { 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 8, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 16384, 32768, },
  { 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 8, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 16384, 32768, },
  { 512, 2048, 4096, 16384, 32768, },
  { 1, 512, 2048, 4096, 16384, 32768, },
  { 2, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 512, 2048, 4096, 16384, 32768, },
  { 4, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 512, 2048, 4096, 16384, 32768, },
  { 8, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 512, 2048, 4096, 16384, 32768, },
  { 16, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 2048, 4096, 16384, 32768, },
  { 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 32, 512, 2048, 4096, 16384, 32768, },
  { 2, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 512, 2048, 4096, 16384, 32768, },
  { 4, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 512, 2048, 4096, 16384, 32768, },
  { 8, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 2048, 4096, 16384, 32768, },
  { 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 4096, 16384, 32768, },
  { 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 512, 2048, 4096, 16384, 32768, },
  { 4, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 512, 2048, 4096, 16384, 32768, },
  { 8, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 2048, 4096, 16384, 32768, },
  { 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 4096, 16384, 32768, },
  { 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 16384, 32768, },
  { 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 512, 2048, 4096, 16384, 32768, },
  { 8, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 2048, 4096, 16384, 32768, },
  { 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 4096, 16384, 32768, },
  { 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 8, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 16384, 32768, },
  { 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 8, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 16384, 32768, },
  { 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 2048, 4096, 16384, 32768, },
  { 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 4096, 16384, 32768, },
  { 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 16384, 32768, },
  { 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 16384, 32768, },
  { 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 16384, 32768, },
  { 1024, 2048, 4096, 16384, 32768, },
  { 1, 1024, 2048, 4096, 16384, 32768, },
  { 2, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 1024, 2048, 4096, 16384, 32768, },
  { 4, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 1024, 2048, 4096, 16384, 32768, },
  { 8, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 1024, 2048, 4096, 16384, 32768, },
  { 16, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 1024, 2048, 4096, 16384, 32768, },
  { 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 4096, 16384, 32768, },
  { 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 1024, 2048, 4096, 16384, 32768, },
  { 4, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 1024, 2048, 4096, 16384, 32768, },
  { 8, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 1024, 2048, 4096, 16384, 32768, },
  { 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 4096, 16384, 32768, },
  { 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 16384, 32768, },
  { 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 1024, 2048, 4096, 16384, 32768, },
  { 8, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 1024, 2048, 4096, 16384, 32768, },
  { 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 4096, 16384, 32768, },
  { 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 16384, 32768, },
  { 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 8, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 16384, 32768, },
  { 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 256, 1024, 2048, 4096, 16384, 32768, },
  { 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 4096, 16384, 32768, },
  { 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 16384, 32768, },
  { 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 16384, 32768, },
  { 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 16384, 32768, },
  { 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 4096, 16384, 32768, },
  { 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 16384, 32768, },
  { 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 16384, 32768, },
  { 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 16384, 32768, },
  { 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 16384, 32768, },
  { 8192, 16384, 32768, },
  { 1, 8192, 16384, 32768, },
  { 2, 8192, 16384, 32768, },
  { 1, 2, 8192, 16384, 32768, },
  { 4, 8192, 16384, 32768, },
  { 1, 4, 8192, 16384, 32768, },
  { 2, 4, 8192, 16384, 32768, },
  { 1, 2, 4, 8192, 16384, 32768, },
  { 8, 8192, 16384, 32768, },
  { 1, 8, 8192, 16384, 32768, },
  { 2, 8, 8192, 16384, 32768, },
  { 1, 2, 8, 8192, 16384, 32768, },
  { 4, 8, 8192, 16384, 32768, },
  { 1, 4, 8, 8192, 16384, 32768, },
  { 2, 4, 8, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 8192, 16384, 32768, },
  { 16, 8192, 16384, 32768, },
  { 1, 16, 8192, 16384, 32768, },
  { 2, 16, 8192, 16384, 32768, },
  { 1, 2, 16, 8192, 16384, 32768, },
  { 4, 16, 8192, 16384, 32768, },
  { 1, 4, 16, 8192, 16384, 32768, },
  { 2, 4, 16, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 8192, 16384, 32768, },
  { 8, 16, 8192, 16384, 32768, },
  { 1, 8, 16, 8192, 16384, 32768, },
  { 2, 8, 16, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 8192, 16384, 32768, },
  { 4, 8, 16, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 8192, 16384, 32768, },
  { 32, 8192, 16384, 32768, },
  { 1, 32, 8192, 16384, 32768, },
  { 2, 32, 8192, 16384, 32768, },
  { 1, 2, 32, 8192, 16384, 32768, },
  { 4, 32, 8192, 16384, 32768, },
  { 1, 4, 32, 8192, 16384, 32768, },
  { 2, 4, 32, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 8192, 16384, 32768, },
  { 8, 32, 8192, 16384, 32768, },
  { 1, 8, 32, 8192, 16384, 32768, },
  { 2, 8, 32, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 8192, 16384, 32768, },
  { 4, 8, 32, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 8192, 16384, 32768, },
  { 16, 32, 8192, 16384, 32768, },
  { 1, 16, 32, 8192, 16384, 32768, },
  { 2, 16, 32, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 8192, 16384, 32768, },
  { 4, 16, 32, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 8192, 16384, 32768, },
  { 8, 16, 32, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 8192, 16384, 32768, },
  { 64, 8192, 16384, 32768, },
  { 1, 64, 8192, 16384, 32768, },
  { 2, 64, 8192, 16384, 32768, },
  { 1, 2, 64, 8192, 16384, 32768, },
  { 4, 64, 8192, 16384, 32768, },
  { 1, 4, 64, 8192, 16384, 32768, },
  { 2, 4, 64, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 8192, 16384, 32768, },
  { 8, 64, 8192, 16384, 32768, },
  { 1, 8, 64, 8192, 16384, 32768, },
  { 2, 8, 64, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 8192, 16384, 32768, },
  { 4, 8, 64, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 8192, 16384, 32768, },
  { 16, 64, 8192, 16384, 32768, },
  { 1, 16, 64, 8192, 16384, 32768, },
  { 2, 16, 64, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 8192, 16384, 32768, },
  { 4, 16, 64, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 8192, 16384, 32768, },
  { 8, 16, 64, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 8192, 16384, 32768, },
  { 32, 64, 8192, 16384, 32768, },
  { 1, 32, 64, 8192, 16384, 32768, },
  { 2, 32, 64, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 8192, 16384, 32768, },
  { 4, 32, 64, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 8192, 16384, 32768, },
  { 8, 32, 64, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 8192, 16384, 32768, },
  { 16, 32, 64, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 8192, 16384, 32768, },
  { 128, 8192, 16384, 32768, },
  { 1, 128, 8192, 16384, 32768, },
  { 2, 128, 8192, 16384, 32768, },
  { 1, 2, 128, 8192, 16384, 32768, },
  { 4, 128, 8192, 16384, 32768, },
  { 1, 4, 128, 8192, 16384, 32768, },
  { 2, 4, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 8192, 16384, 32768, },
  { 8, 128, 8192, 16384, 32768, },
  { 1, 8, 128, 8192, 16384, 32768, },
  { 2, 8, 128, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 8192, 16384, 32768, },
  { 4, 8, 128, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 8192, 16384, 32768, },
  { 16, 128, 8192, 16384, 32768, },
  { 1, 16, 128, 8192, 16384, 32768, },
  { 2, 16, 128, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 8192, 16384, 32768, },
  { 4, 16, 128, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 8192, 16384, 32768, },
  { 8, 16, 128, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 8192, 16384, 32768, },
  { 32, 128, 8192, 16384, 32768, },
  { 1, 32, 128, 8192, 16384, 32768, },
  { 2, 32, 128, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 8192, 16384, 32768, },
  { 4, 32, 128, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 8192, 16384, 32768, },
  { 8, 32, 128, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 8192, 16384, 32768, },
  { 16, 32, 128, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 8192, 16384, 32768, },
  { 64, 128, 8192, 16384, 32768, },
  { 1, 64, 128, 8192, 16384, 32768, },
  { 2, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 8192, 16384, 32768, },
  { 4, 64, 128, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 8192, 16384, 32768, },
  { 8, 64, 128, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 8192, 16384, 32768, },
  { 16, 64, 128, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 8192, 16384, 32768, },
  { 32, 64, 128, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 8192, 16384, 32768, },
  { 256, 8192, 16384, 32768, },
  { 1, 256, 8192, 16384, 32768, },
  { 2, 256, 8192, 16384, 32768, },
  { 1, 2, 256, 8192, 16384, 32768, },
  { 4, 256, 8192, 16384, 32768, },
  { 1, 4, 256, 8192, 16384, 32768, },
  { 2, 4, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 8192, 16384, 32768, },
  { 8, 256, 8192, 16384, 32768, },
  { 1, 8, 256, 8192, 16384, 32768, },
  { 2, 8, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 8192, 16384, 32768, },
  { 4, 8, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 8192, 16384, 32768, },
  { 16, 256, 8192, 16384, 32768, },
  { 1, 16, 256, 8192, 16384, 32768, },
  { 2, 16, 256, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 8192, 16384, 32768, },
  { 4, 16, 256, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 8192, 16384, 32768, },
  { 8, 16, 256, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 8192, 16384, 32768, },
  { 32, 256, 8192, 16384, 32768, },
  { 1, 32, 256, 8192, 16384, 32768, },
  { 2, 32, 256, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 8192, 16384, 32768, },
  { 4, 32, 256, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 8192, 16384, 32768, },
  { 8, 32, 256, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 8192, 16384, 32768, },
  { 16, 32, 256, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 8192, 16384, 32768, },
  { 64, 256, 8192, 16384, 32768, },
  { 1, 64, 256, 8192, 16384, 32768, },
  { 2, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 8192, 16384, 32768, },
  { 4, 64, 256, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 8192, 16384, 32768, },
  { 8, 64, 256, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 8192, 16384, 32768, },
  { 16, 64, 256, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 8192, 16384, 32768, },
  { 32, 64, 256, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 8192, 16384, 32768, },
  { 128, 256, 8192, 16384, 32768, },
  { 1, 128, 256, 8192, 16384, 32768, },
  { 2, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 8192, 16384, 32768, },
  { 4, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 8192, 16384, 32768, },
  { 8, 128, 256, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 8192, 16384, 32768, },
  { 16, 128, 256, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 8192, 16384, 32768, },
  { 32, 128, 256, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 8192, 16384, 32768, },
  { 64, 128, 256, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 8192, 16384, 32768, },
  { 512, 8192, 16384, 32768, },
  { 1, 512, 8192, 16384, 32768, },
  { 2, 512, 8192, 16384, 32768, },
  { 1, 2, 512, 8192, 16384, 32768, },
  { 4, 512, 8192, 16384, 32768, },
  { 1, 4, 512, 8192, 16384, 32768, },
  { 2, 4, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 512, 8192, 16384, 32768, },
  { 8, 512, 8192, 16384, 32768, },
  { 1, 8, 512, 8192, 16384, 32768, },
  { 2, 8, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 512, 8192, 16384, 32768, },
  { 4, 8, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 512, 8192, 16384, 32768, },
  { 16, 512, 8192, 16384, 32768, },
  { 1, 16, 512, 8192, 16384, 32768, },
  { 2, 16, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 512, 8192, 16384, 32768, },
  { 4, 16, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 512, 8192, 16384, 32768, },
  { 8, 16, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 8192, 16384, 32768, },
  { 32, 512, 8192, 16384, 32768, },
  { 1, 32, 512, 8192, 16384, 32768, },
  { 2, 32, 512, 8192, 16384, 32768, },
  { 1, 2, 32, 512, 8192, 16384, 32768, },
  { 4, 32, 512, 8192, 16384, 32768, },
  { 1, 4, 32, 512, 8192, 16384, 32768, },
  { 2, 4, 32, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 512, 8192, 16384, 32768, },
  { 8, 32, 512, 8192, 16384, 32768, },
  { 1, 8, 32, 512, 8192, 16384, 32768, },
  { 2, 8, 32, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 512, 8192, 16384, 32768, },
  { 4, 8, 32, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 8192, 16384, 32768, },
  { 16, 32, 512, 8192, 16384, 32768, },
  { 1, 16, 32, 512, 8192, 16384, 32768, },
  { 2, 16, 32, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 512, 8192, 16384, 32768, },
  { 4, 16, 32, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 8192, 16384, 32768, },
  { 8, 16, 32, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 8192, 16384, 32768, },
  { 64, 512, 8192, 16384, 32768, },
  { 1, 64, 512, 8192, 16384, 32768, },
  { 2, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 64, 512, 8192, 16384, 32768, },
  { 4, 64, 512, 8192, 16384, 32768, },
  { 1, 4, 64, 512, 8192, 16384, 32768, },
  { 2, 4, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 512, 8192, 16384, 32768, },
  { 8, 64, 512, 8192, 16384, 32768, },
  { 1, 8, 64, 512, 8192, 16384, 32768, },
  { 2, 8, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 512, 8192, 16384, 32768, },
  { 4, 8, 64, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 8192, 16384, 32768, },
  { 16, 64, 512, 8192, 16384, 32768, },
  { 1, 16, 64, 512, 8192, 16384, 32768, },
  { 2, 16, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 512, 8192, 16384, 32768, },
  { 4, 16, 64, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 8192, 16384, 32768, },
  { 8, 16, 64, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 8192, 16384, 32768, },
  { 32, 64, 512, 8192, 16384, 32768, },
  { 1, 32, 64, 512, 8192, 16384, 32768, },
  { 2, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 512, 8192, 16384, 32768, },
  { 4, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 512, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 8192, 16384, 32768, },
  { 8, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 512, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 8192, 16384, 32768, },
  { 16, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 8192, 16384, 32768, },
  { 128, 512, 8192, 16384, 32768, },
  { 1, 128, 512, 8192, 16384, 32768, },
  { 2, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 128, 512, 8192, 16384, 32768, },
  { 4, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 512, 8192, 16384, 32768, },
  { 8, 128, 512, 8192, 16384, 32768, },
  { 1, 8, 128, 512, 8192, 16384, 32768, },
  { 2, 8, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 512, 8192, 16384, 32768, },
  { 4, 8, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 8192, 16384, 32768, },
  { 16, 128, 512, 8192, 16384, 32768, },
  { 1, 16, 128, 512, 8192, 16384, 32768, },
  { 2, 16, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 512, 8192, 16384, 32768, },
  { 4, 16, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 8192, 16384, 32768, },
  { 8, 16, 128, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 8192, 16384, 32768, },
  { 32, 128, 512, 8192, 16384, 32768, },
  { 1, 32, 128, 512, 8192, 16384, 32768, },
  { 2, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 512, 8192, 16384, 32768, },
  { 4, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 8192, 16384, 32768, },
  { 8, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 512, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 8192, 16384, 32768, },
  { 16, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 8192, 16384, 32768, },
  { 64, 128, 512, 8192, 16384, 32768, },
  { 1, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 512, 8192, 16384, 32768, },
  { 4, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 8192, 16384, 32768, },
  { 8, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 8192, 16384, 32768, },
  { 16, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 8192, 16384, 32768, },
  { 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 8192, 16384, 32768, },
  { 256, 512, 8192, 16384, 32768, },
  { 1, 256, 512, 8192, 16384, 32768, },
  { 2, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 256, 512, 8192, 16384, 32768, },
  { 4, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 512, 8192, 16384, 32768, },
  { 8, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 8192, 16384, 32768, },
  { 16, 256, 512, 8192, 16384, 32768, },
  { 1, 16, 256, 512, 8192, 16384, 32768, },
  { 2, 16, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 512, 8192, 16384, 32768, },
  { 4, 16, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 8192, 16384, 32768, },
  { 8, 16, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 8192, 16384, 32768, },
  { 32, 256, 512, 8192, 16384, 32768, },
  { 1, 32, 256, 512, 8192, 16384, 32768, },
  { 2, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 512, 8192, 16384, 32768, },
  { 4, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 8192, 16384, 32768, },
  { 8, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 8192, 16384, 32768, },
  { 16, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 8192, 16384, 32768, },
  { 64, 256, 512, 8192, 16384, 32768, },
  { 1, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 512, 8192, 16384, 32768, },
  { 4, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 8192, 16384, 32768, },
  { 8, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 8192, 16384, 32768, },
  { 16, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 8192, 16384, 32768, },
  { 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 8192, 16384, 32768, },
  { 128, 256, 512, 8192, 16384, 32768, },
  { 1, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 8192, 16384, 32768, },
  { 8, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 8192, 16384, 32768, },
  { 16, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 8192, 16384, 32768, },
  { 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 8192, 16384, 32768, },
  { 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 8192, 16384, 32768, },
  { 1024, 8192, 16384, 32768, },
  { 1, 1024, 8192, 16384, 32768, },
  { 2, 1024, 8192, 16384, 32768, },
  { 1, 2, 1024, 8192, 16384, 32768, },
  { 4, 1024, 8192, 16384, 32768, },
  { 1, 4, 1024, 8192, 16384, 32768, },
  { 2, 4, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 1024, 8192, 16384, 32768, },
  { 8, 1024, 8192, 16384, 32768, },
  { 1, 8, 1024, 8192, 16384, 32768, },
  { 2, 8, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 1024, 8192, 16384, 32768, },
  { 4, 8, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 1024, 8192, 16384, 32768, },
  { 16, 1024, 8192, 16384, 32768, },
  { 1, 16, 1024, 8192, 16384, 32768, },
  { 2, 16, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 1024, 8192, 16384, 32768, },
  { 4, 16, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 1024, 8192, 16384, 32768, },
  { 8, 16, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 1024, 8192, 16384, 32768, },
  { 32, 1024, 8192, 16384, 32768, },
  { 1, 32, 1024, 8192, 16384, 32768, },
  { 2, 32, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 1024, 8192, 16384, 32768, },
  { 4, 32, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 1024, 8192, 16384, 32768, },
  { 8, 32, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 1024, 8192, 16384, 32768, },
  { 16, 32, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 8192, 16384, 32768, },
  { 64, 1024, 8192, 16384, 32768, },
  { 1, 64, 1024, 8192, 16384, 32768, },
  { 2, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 64, 1024, 8192, 16384, 32768, },
  { 4, 64, 1024, 8192, 16384, 32768, },
  { 1, 4, 64, 1024, 8192, 16384, 32768, },
  { 2, 4, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 1024, 8192, 16384, 32768, },
  { 8, 64, 1024, 8192, 16384, 32768, },
  { 1, 8, 64, 1024, 8192, 16384, 32768, },
  { 2, 8, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 1024, 8192, 16384, 32768, },
  { 4, 8, 64, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 1024, 8192, 16384, 32768, },
  { 16, 64, 1024, 8192, 16384, 32768, },
  { 1, 16, 64, 1024, 8192, 16384, 32768, },
  { 2, 16, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 1024, 8192, 16384, 32768, },
  { 4, 16, 64, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 1024, 8192, 16384, 32768, },
  { 8, 16, 64, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 8192, 16384, 32768, },
  { 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 32, 64, 1024, 8192, 16384, 32768, },
  { 2, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 1024, 8192, 16384, 32768, },
  { 4, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 1024, 8192, 16384, 32768, },
  { 8, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 8192, 16384, 32768, },
  { 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 8192, 16384, 32768, },
  { 128, 1024, 8192, 16384, 32768, },
  { 1, 128, 1024, 8192, 16384, 32768, },
  { 2, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 128, 1024, 8192, 16384, 32768, },
  { 4, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 1024, 8192, 16384, 32768, },
  { 8, 128, 1024, 8192, 16384, 32768, },
  { 1, 8, 128, 1024, 8192, 16384, 32768, },
  { 2, 8, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 1024, 8192, 16384, 32768, },
  { 4, 8, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 1024, 8192, 16384, 32768, },
  { 16, 128, 1024, 8192, 16384, 32768, },
  { 1, 16, 128, 1024, 8192, 16384, 32768, },
  { 2, 16, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 1024, 8192, 16384, 32768, },
  { 4, 16, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 1024, 8192, 16384, 32768, },
  { 8, 16, 128, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 8192, 16384, 32768, },
  { 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 32, 128, 1024, 8192, 16384, 32768, },
  { 2, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 1024, 8192, 16384, 32768, },
  { 4, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 1024, 8192, 16384, 32768, },
  { 8, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 8192, 16384, 32768, },
  { 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 8192, 16384, 32768, },
  { 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 1024, 8192, 16384, 32768, },
  { 4, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 1024, 8192, 16384, 32768, },
  { 8, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 1024, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 8192, 16384, 32768, },
  { 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 8192, 16384, 32768, },
  { 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 8192, 16384, 32768, },
  { 256, 1024, 8192, 16384, 32768, },
  { 1, 256, 1024, 8192, 16384, 32768, },
  { 2, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 256, 1024, 8192, 16384, 32768, },
  { 4, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 1024, 8192, 16384, 32768, },
  { 8, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 1024, 8192, 16384, 32768, },
  { 16, 256, 1024, 8192, 16384, 32768, },
  { 1, 16, 256, 1024, 8192, 16384, 32768, },
  { 2, 16, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 1024, 8192, 16384, 32768, },
  { 4, 16, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 1024, 8192, 16384, 32768, },
  { 8, 16, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 8192, 16384, 32768, },
  { 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 32, 256, 1024, 8192, 16384, 32768, },
  { 2, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 1024, 8192, 16384, 32768, },
  { 4, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 1024, 8192, 16384, 32768, },
  { 8, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 8192, 16384, 32768, },
  { 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 8192, 16384, 32768, },
  { 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 1024, 8192, 16384, 32768, },
  { 4, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 1024, 8192, 16384, 32768, },
  { 8, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 8192, 16384, 32768, },
  { 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 8192, 16384, 32768, },
  { 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 8192, 16384, 32768, },
  { 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 1024, 8192, 16384, 32768, },
  { 8, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 8192, 16384, 32768, },
  { 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 8192, 16384, 32768, },
  { 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 8192, 16384, 32768, },
  { 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 8192, 16384, 32768, },
  { 512, 1024, 8192, 16384, 32768, },
  { 1, 512, 1024, 8192, 16384, 32768, },
  { 2, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 512, 1024, 8192, 16384, 32768, },
  { 4, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 512, 1024, 8192, 16384, 32768, },
  { 8, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 512, 1024, 8192, 16384, 32768, },
  { 16, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 8192, 16384, 32768, },
  { 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 32, 512, 1024, 8192, 16384, 32768, },
  { 2, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 512, 1024, 8192, 16384, 32768, },
  { 4, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 512, 1024, 8192, 16384, 32768, },
  { 8, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 8192, 16384, 32768, },
  { 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 8192, 16384, 32768, },
  { 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 64, 512, 1024, 8192, 16384, 32768, },
  { 4, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 512, 1024, 8192, 16384, 32768, },
  { 8, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 8192, 16384, 32768, },
  { 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 8192, 16384, 32768, },
  { 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 4, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 8, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 8192, 16384, 32768, },
  { 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 512, 1024, 8192, 16384, 32768, },
  { 8, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 8192, 16384, 32768, },
  { 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 8192, 16384, 32768, },
  { 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 8, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 8192, 16384, 32768, },
  { 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 8, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 8192, 16384, 32768, },
  { 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 8192, 16384, 32768, },
  { 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 8192, 16384, 32768, },
  { 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 8192, 16384, 32768, },
  { 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 8192, 16384, 32768, },
  { 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 8192, 16384, 32768, },
  { 2048, 8192, 16384, 32768, },
  { 1, 2048, 8192, 16384, 32768, },
  { 2, 2048, 8192, 16384, 32768, },
  { 1, 2, 2048, 8192, 16384, 32768, },
  { 4, 2048, 8192, 16384, 32768, },
  { 1, 4, 2048, 8192, 16384, 32768, },
  { 2, 4, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 2048, 8192, 16384, 32768, },
  { 8, 2048, 8192, 16384, 32768, },
  { 1, 8, 2048, 8192, 16384, 32768, },
  { 2, 8, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 2048, 8192, 16384, 32768, },
  { 4, 8, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 2048, 8192, 16384, 32768, },
  { 16, 2048, 8192, 16384, 32768, },
  { 1, 16, 2048, 8192, 16384, 32768, },
  { 2, 16, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 2048, 8192, 16384, 32768, },
  { 4, 16, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 2048, 8192, 16384, 32768, },
  { 8, 16, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 2048, 8192, 16384, 32768, },
  { 32, 2048, 8192, 16384, 32768, },
  { 1, 32, 2048, 8192, 16384, 32768, },
  { 2, 32, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 2048, 8192, 16384, 32768, },
  { 4, 32, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 2048, 8192, 16384, 32768, },
  { 8, 32, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 2048, 8192, 16384, 32768, },
  { 16, 32, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 2048, 8192, 16384, 32768, },
  { 64, 2048, 8192, 16384, 32768, },
  { 1, 64, 2048, 8192, 16384, 32768, },
  { 2, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 2048, 8192, 16384, 32768, },
  { 4, 64, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 2048, 8192, 16384, 32768, },
  { 8, 64, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 2048, 8192, 16384, 32768, },
  { 16, 64, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 2048, 8192, 16384, 32768, },
  { 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 8192, 16384, 32768, },
  { 128, 2048, 8192, 16384, 32768, },
  { 1, 128, 2048, 8192, 16384, 32768, },
  { 2, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 128, 2048, 8192, 16384, 32768, },
  { 4, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 2048, 8192, 16384, 32768, },
  { 8, 128, 2048, 8192, 16384, 32768, },
  { 1, 8, 128, 2048, 8192, 16384, 32768, },
  { 2, 8, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 2048, 8192, 16384, 32768, },
  { 4, 8, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 2048, 8192, 16384, 32768, },
  { 16, 128, 2048, 8192, 16384, 32768, },
  { 1, 16, 128, 2048, 8192, 16384, 32768, },
  { 2, 16, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 2048, 8192, 16384, 32768, },
  { 4, 16, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 2048, 8192, 16384, 32768, },
  { 8, 16, 128, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 2048, 8192, 16384, 32768, },
  { 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 32, 128, 2048, 8192, 16384, 32768, },
  { 2, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 2048, 8192, 16384, 32768, },
  { 4, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 2048, 8192, 16384, 32768, },
  { 8, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 2048, 8192, 16384, 32768, },
  { 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 8192, 16384, 32768, },
  { 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 2048, 8192, 16384, 32768, },
  { 4, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 2048, 8192, 16384, 32768, },
  { 8, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 2048, 8192, 16384, 32768, },
  { 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 8192, 16384, 32768, },
  { 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 8192, 16384, 32768, },
  { 256, 2048, 8192, 16384, 32768, },
  { 1, 256, 2048, 8192, 16384, 32768, },
  { 2, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 256, 2048, 8192, 16384, 32768, },
  { 4, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 2048, 8192, 16384, 32768, },
  { 8, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 2048, 8192, 16384, 32768, },
  { 16, 256, 2048, 8192, 16384, 32768, },
  { 1, 16, 256, 2048, 8192, 16384, 32768, },
  { 2, 16, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 2048, 8192, 16384, 32768, },
  { 4, 16, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 2048, 8192, 16384, 32768, },
  { 8, 16, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 2048, 8192, 16384, 32768, },
  { 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 32, 256, 2048, 8192, 16384, 32768, },
  { 2, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 2048, 8192, 16384, 32768, },
  { 4, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 2048, 8192, 16384, 32768, },
  { 8, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 2048, 8192, 16384, 32768, },
  { 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 8192, 16384, 32768, },
  { 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 2048, 8192, 16384, 32768, },
  { 4, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 2048, 8192, 16384, 32768, },
  { 8, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 2048, 8192, 16384, 32768, },
  { 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 8192, 16384, 32768, },
  { 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 8192, 16384, 32768, },
  { 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 2048, 8192, 16384, 32768, },
  { 8, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 2048, 8192, 16384, 32768, },
  { 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 8192, 16384, 32768, },
  { 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 8192, 16384, 32768, },
  { 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 8192, 16384, 32768, },
  { 512, 2048, 8192, 16384, 32768, },
  { 1, 512, 2048, 8192, 16384, 32768, },
  { 2, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 512, 2048, 8192, 16384, 32768, },
  { 4, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 512, 2048, 8192, 16384, 32768, },
  { 8, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 512, 2048, 8192, 16384, 32768, },
  { 16, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 2048, 8192, 16384, 32768, },
  { 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 32, 512, 2048, 8192, 16384, 32768, },
  { 2, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 512, 2048, 8192, 16384, 32768, },
  { 4, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 512, 2048, 8192, 16384, 32768, },
  { 8, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 2048, 8192, 16384, 32768, },
  { 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 8192, 16384, 32768, },
  { 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 512, 2048, 8192, 16384, 32768, },
  { 4, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 512, 2048, 8192, 16384, 32768, },
  { 8, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 2048, 8192, 16384, 32768, },
  { 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 8192, 16384, 32768, },
  { 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 8192, 16384, 32768, },
  { 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 512, 2048, 8192, 16384, 32768, },
  { 8, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 2048, 8192, 16384, 32768, },
  { 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 8192, 16384, 32768, },
  { 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 8, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 8192, 16384, 32768, },
  { 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 8, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 8192, 16384, 32768, },
  { 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 2048, 8192, 16384, 32768, },
  { 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 8192, 16384, 32768, },
  { 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 8192, 16384, 32768, },
  { 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 8192, 16384, 32768, },
  { 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 8192, 16384, 32768, },
  { 1024, 2048, 8192, 16384, 32768, },
  { 1, 1024, 2048, 8192, 16384, 32768, },
  { 2, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 1024, 2048, 8192, 16384, 32768, },
  { 4, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 1024, 2048, 8192, 16384, 32768, },
  { 8, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 1024, 2048, 8192, 16384, 32768, },
  { 16, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 1024, 2048, 8192, 16384, 32768, },
  { 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 8192, 16384, 32768, },
  { 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 1024, 2048, 8192, 16384, 32768, },
  { 4, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 1024, 2048, 8192, 16384, 32768, },
  { 8, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 1024, 2048, 8192, 16384, 32768, },
  { 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 8192, 16384, 32768, },
  { 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 8192, 16384, 32768, },
  { 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 1024, 2048, 8192, 16384, 32768, },
  { 8, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 1024, 2048, 8192, 16384, 32768, },
  { 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 8192, 16384, 32768, },
  { 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 8192, 16384, 32768, },
  { 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 8, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 8192, 16384, 32768, },
  { 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 1024, 2048, 8192, 16384, 32768, },
  { 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 8192, 16384, 32768, },
  { 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 8192, 16384, 32768, },
  { 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 8192, 16384, 32768, },
  { 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 8192, 16384, 32768, },
  { 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 8192, 16384, 32768, },
  { 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 8192, 16384, 32768, },
  { 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 8192, 16384, 32768, },
  { 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 8192, 16384, 32768, },
  { 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 8192, 16384, 32768, },
  { 4096, 8192, 16384, 32768, },
  { 1, 4096, 8192, 16384, 32768, },
  { 2, 4096, 8192, 16384, 32768, },
  { 1, 2, 4096, 8192, 16384, 32768, },
  { 4, 4096, 8192, 16384, 32768, },
  { 1, 4, 4096, 8192, 16384, 32768, },
  { 2, 4, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 4096, 8192, 16384, 32768, },
  { 8, 4096, 8192, 16384, 32768, },
  { 1, 8, 4096, 8192, 16384, 32768, },
  { 2, 8, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 4096, 8192, 16384, 32768, },
  { 4, 8, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 4096, 8192, 16384, 32768, },
  { 16, 4096, 8192, 16384, 32768, },
  { 1, 16, 4096, 8192, 16384, 32768, },
  { 2, 16, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 4096, 8192, 16384, 32768, },
  { 4, 16, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 4096, 8192, 16384, 32768, },
  { 8, 16, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 4096, 8192, 16384, 32768, },
  { 32, 4096, 8192, 16384, 32768, },
  { 1, 32, 4096, 8192, 16384, 32768, },
  { 2, 32, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 4096, 8192, 16384, 32768, },
  { 4, 32, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 4096, 8192, 16384, 32768, },
  { 8, 32, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 4096, 8192, 16384, 32768, },
  { 16, 32, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 4096, 8192, 16384, 32768, },
  { 64, 4096, 8192, 16384, 32768, },
  { 1, 64, 4096, 8192, 16384, 32768, },
  { 2, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 4096, 8192, 16384, 32768, },
  { 4, 64, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 4096, 8192, 16384, 32768, },
  { 8, 64, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 4096, 8192, 16384, 32768, },
  { 16, 64, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 4096, 8192, 16384, 32768, },
  { 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 4096, 8192, 16384, 32768, },
  { 128, 4096, 8192, 16384, 32768, },
  { 1, 128, 4096, 8192, 16384, 32768, },
  { 2, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 4096, 8192, 16384, 32768, },
  { 4, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 4096, 8192, 16384, 32768, },
  { 8, 128, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 4096, 8192, 16384, 32768, },
  { 16, 128, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 4096, 8192, 16384, 32768, },
  { 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 4096, 8192, 16384, 32768, },
  { 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 4096, 8192, 16384, 32768, },
  { 256, 4096, 8192, 16384, 32768, },
  { 1, 256, 4096, 8192, 16384, 32768, },
  { 2, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 256, 4096, 8192, 16384, 32768, },
  { 4, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 4096, 8192, 16384, 32768, },
  { 8, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 4096, 8192, 16384, 32768, },
  { 16, 256, 4096, 8192, 16384, 32768, },
  { 1, 16, 256, 4096, 8192, 16384, 32768, },
  { 2, 16, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 4096, 8192, 16384, 32768, },
  { 4, 16, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 4096, 8192, 16384, 32768, },
  { 8, 16, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 4096, 8192, 16384, 32768, },
  { 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 32, 256, 4096, 8192, 16384, 32768, },
  { 2, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 4096, 8192, 16384, 32768, },
  { 4, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 4096, 8192, 16384, 32768, },
  { 8, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 4096, 8192, 16384, 32768, },
  { 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 4096, 8192, 16384, 32768, },
  { 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 4096, 8192, 16384, 32768, },
  { 4, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 4096, 8192, 16384, 32768, },
  { 8, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 4096, 8192, 16384, 32768, },
  { 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 4096, 8192, 16384, 32768, },
  { 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 4096, 8192, 16384, 32768, },
  { 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 4096, 8192, 16384, 32768, },
  { 8, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 4096, 8192, 16384, 32768, },
  { 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 4096, 8192, 16384, 32768, },
  { 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 4096, 8192, 16384, 32768, },
  { 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 4096, 8192, 16384, 32768, },
  { 512, 4096, 8192, 16384, 32768, },
  { 1, 512, 4096, 8192, 16384, 32768, },
  { 2, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 512, 4096, 8192, 16384, 32768, },
  { 4, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 512, 4096, 8192, 16384, 32768, },
  { 8, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 512, 4096, 8192, 16384, 32768, },
  { 16, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 4096, 8192, 16384, 32768, },
  { 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 32, 512, 4096, 8192, 16384, 32768, },
  { 2, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 512, 4096, 8192, 16384, 32768, },
  { 4, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 512, 4096, 8192, 16384, 32768, },
  { 8, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 4096, 8192, 16384, 32768, },
  { 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 4096, 8192, 16384, 32768, },
  { 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 512, 4096, 8192, 16384, 32768, },
  { 4, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 512, 4096, 8192, 16384, 32768, },
  { 8, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 4096, 8192, 16384, 32768, },
  { 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 4096, 8192, 16384, 32768, },
  { 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 4096, 8192, 16384, 32768, },
  { 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 512, 4096, 8192, 16384, 32768, },
  { 8, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 4096, 8192, 16384, 32768, },
  { 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 4096, 8192, 16384, 32768, },
  { 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 4096, 8192, 16384, 32768, },
  { 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 4096, 8192, 16384, 32768, },
  { 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 4096, 8192, 16384, 32768, },
  { 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 4096, 8192, 16384, 32768, },
  { 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 4096, 8192, 16384, 32768, },
  { 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 4096, 8192, 16384, 32768, },
  { 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 4096, 8192, 16384, 32768, },
  { 1024, 4096, 8192, 16384, 32768, },
  { 1, 1024, 4096, 8192, 16384, 32768, },
  { 2, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 1024, 4096, 8192, 16384, 32768, },
  { 4, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 1024, 4096, 8192, 16384, 32768, },
  { 8, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 1024, 4096, 8192, 16384, 32768, },
  { 16, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 1024, 4096, 8192, 16384, 32768, },
  { 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 4096, 8192, 16384, 32768, },
  { 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 1024, 4096, 8192, 16384, 32768, },
  { 4, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 1024, 4096, 8192, 16384, 32768, },
  { 8, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 1024, 4096, 8192, 16384, 32768, },
  { 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 4096, 8192, 16384, 32768, },
  { 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 4096, 8192, 16384, 32768, },
  { 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 1024, 4096, 8192, 16384, 32768, },
  { 8, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 1024, 4096, 8192, 16384, 32768, },
  { 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 4096, 8192, 16384, 32768, },
  { 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 4096, 8192, 16384, 32768, },
  { 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 4096, 8192, 16384, 32768, },
  { 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 1024, 4096, 8192, 16384, 32768, },
  { 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 4096, 8192, 16384, 32768, },
  { 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 4096, 8192, 16384, 32768, },
  { 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 4096, 8192, 16384, 32768, },
  { 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 4096, 8192, 16384, 32768, },
  { 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 4096, 8192, 16384, 32768, },
  { 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 4096, 8192, 16384, 32768, },
  { 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 4096, 8192, 16384, 32768, },
  { 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 4096, 8192, 16384, 32768, },
  { 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 4096, 8192, 16384, 32768, },
  { 2048, 4096, 8192, 16384, 32768, },
  { 1, 2048, 4096, 8192, 16384, 32768, },
  { 2, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 2048, 4096, 8192, 16384, 32768, },
  { 4, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 2048, 4096, 8192, 16384, 32768, },
  { 8, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 2048, 4096, 8192, 16384, 32768, },
  { 16, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 2048, 4096, 8192, 16384, 32768, },
  { 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 2048, 4096, 8192, 16384, 32768, },
  { 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 2048, 4096, 8192, 16384, 32768, },
  { 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 2048, 4096, 8192, 16384, 32768, },
  { 8, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 2048, 4096, 8192, 16384, 32768, },
  { 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 2048, 4096, 8192, 16384, 32768, },
  { 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 2048, 4096, 8192, 16384, 32768, },
  { 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 2048, 4096, 8192, 16384, 32768, },
  { 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 2048, 4096, 8192, 16384, 32768, },
  { 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 2048, 4096, 8192, 16384, 32768, },
  { 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 2048, 4096, 8192, 16384, 32768, },
  { 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 2048, 4096, 8192, 16384, 32768, },
  { 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 2048, 4096, 8192, 16384, 32768, },
  { 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 2048, 4096, 8192, 16384, 32768, },
  { 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 2048, 4096, 8192, 16384, 32768, },
  { 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 2048, 4096, 8192, 16384, 32768, },
  { 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 2048, 4096, 8192, 16384, 32768, },
  { 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 2048, 4096, 8192, 16384, 32768, },
  { 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 64, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 32, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 16, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 8, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 4, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 2, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 1, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
  { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, },
};

const size_t BIT_MASKS_16_IDX[65536] = {0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 
    3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
    3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
    4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 1, 2, 2, 3, 2, 
    3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
    3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
    4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 1, 2, 2, 3, 2, 
    3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
    3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
    13, 13, 14, 13, 14, 14, 15, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
    4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
    13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
    14, 14, 15, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
    13, 13, 14, 13, 14, 14, 15, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
    13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
    14, 14, 15, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
    13, 13, 14, 13, 14, 14, 15, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
    13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
    14, 14, 15, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 12, 
    13, 13, 14, 13, 14, 14, 15, 13, 14, 14, 15, 14, 15, 15, };

const size_t BITS_SET_16[65536][17] = {
    { },
  { 0, },
  { 1, },
  { 0, 1, },
  { 2, },
  { 0, 2, },
  { 1, 2, },
  { 0, 1, 2, },
  { 3, },
  { 0, 3, },
  { 1, 3, },
  { 0, 1, 3, },
  { 2, 3, },
  { 0, 2, 3, },
  { 1, 2, 3, },
  { 0, 1, 2, 3, },
  { 4, },
  { 0, 4, },
  { 1, 4, },
  { 0, 1, 4, },
  { 2, 4, },
  { 0, 2, 4, },
  { 1, 2, 4, },
  { 0, 1, 2, 4, },
  { 3, 4, },
  { 0, 3, 4, },
  { 1, 3, 4, },
  { 0, 1, 3, 4, },
  { 2, 3, 4, },
  { 0, 2, 3, 4, },
  { 1, 2, 3, 4, },
  { 0, 1, 2, 3, 4, },
  { 5, },
  { 0, 5, },
  { 1, 5, },
  { 0, 1, 5, },
  { 2, 5, },
  { 0, 2, 5, },
  { 1, 2, 5, },
  { 0, 1, 2, 5, },
  { 3, 5, },
  { 0, 3, 5, },
  { 1, 3, 5, },
  { 0, 1, 3, 5, },
  { 2, 3, 5, },
  { 0, 2, 3, 5, },
  { 1, 2, 3, 5, },
  { 0, 1, 2, 3, 5, },
  { 4, 5, },
  { 0, 4, 5, },
  { 1, 4, 5, },
  { 0, 1, 4, 5, },
  { 2, 4, 5, },
  { 0, 2, 4, 5, },
  { 1, 2, 4, 5, },
  { 0, 1, 2, 4, 5, },
  { 3, 4, 5, },
  { 0, 3, 4, 5, },
  { 1, 3, 4, 5, },
  { 0, 1, 3, 4, 5, },
  { 2, 3, 4, 5, },
  { 0, 2, 3, 4, 5, },
  { 1, 2, 3, 4, 5, },
  { 0, 1, 2, 3, 4, 5, },
  { 6, },
  { 0, 6, },
  { 1, 6, },
  { 0, 1, 6, },
  { 2, 6, },
  { 0, 2, 6, },
  { 1, 2, 6, },
  { 0, 1, 2, 6, },
  { 3, 6, },
  { 0, 3, 6, },
  { 1, 3, 6, },
  { 0, 1, 3, 6, },
  { 2, 3, 6, },
  { 0, 2, 3, 6, },
  { 1, 2, 3, 6, },
  { 0, 1, 2, 3, 6, },
  { 4, 6, },
  { 0, 4, 6, },
  { 1, 4, 6, },
  { 0, 1, 4, 6, },
  { 2, 4, 6, },
  { 0, 2, 4, 6, },
  { 1, 2, 4, 6, },
  { 0, 1, 2, 4, 6, },
  { 3, 4, 6, },
  { 0, 3, 4, 6, },
  { 1, 3, 4, 6, },
  { 0, 1, 3, 4, 6, },
  { 2, 3, 4, 6, },
  { 0, 2, 3, 4, 6, },
  { 1, 2, 3, 4, 6, },
  { 0, 1, 2, 3, 4, 6, },
  { 5, 6, },
  { 0, 5, 6, },
  { 1, 5, 6, },
  { 0, 1, 5, 6, },
  { 2, 5, 6, },
  { 0, 2, 5, 6, },
  { 1, 2, 5, 6, },
  { 0, 1, 2, 5, 6, },
  { 3, 5, 6, },
  { 0, 3, 5, 6, },
  { 1, 3, 5, 6, },
  { 0, 1, 3, 5, 6, },
  { 2, 3, 5, 6, },
  { 0, 2, 3, 5, 6, },
  { 1, 2, 3, 5, 6, },
  { 0, 1, 2, 3, 5, 6, },
  { 4, 5, 6, },
  { 0, 4, 5, 6, },
  { 1, 4, 5, 6, },
  { 0, 1, 4, 5, 6, },
  { 2, 4, 5, 6, },
  { 0, 2, 4, 5, 6, },
  { 1, 2, 4, 5, 6, },
  { 0, 1, 2, 4, 5, 6, },
  { 3, 4, 5, 6, },
  { 0, 3, 4, 5, 6, },
  { 1, 3, 4, 5, 6, },
  { 0, 1, 3, 4, 5, 6, },
  { 2, 3, 4, 5, 6, },
  { 0, 2, 3, 4, 5, 6, },
  { 1, 2, 3, 4, 5, 6, },
  { 0, 1, 2, 3, 4, 5, 6, },
  { 7, },
  { 0, 7, },
  { 1, 7, },
  { 0, 1, 7, },
  { 2, 7, },
  { 0, 2, 7, },
  { 1, 2, 7, },
  { 0, 1, 2, 7, },
  { 3, 7, },
  { 0, 3, 7, },
  { 1, 3, 7, },
  { 0, 1, 3, 7, },
  { 2, 3, 7, },
  { 0, 2, 3, 7, },
  { 1, 2, 3, 7, },
  { 0, 1, 2, 3, 7, },
  { 4, 7, },
  { 0, 4, 7, },
  { 1, 4, 7, },
  { 0, 1, 4, 7, },
  { 2, 4, 7, },
  { 0, 2, 4, 7, },
  { 1, 2, 4, 7, },
  { 0, 1, 2, 4, 7, },
  { 3, 4, 7, },
  { 0, 3, 4, 7, },
  { 1, 3, 4, 7, },
  { 0, 1, 3, 4, 7, },
  { 2, 3, 4, 7, },
  { 0, 2, 3, 4, 7, },
  { 1, 2, 3, 4, 7, },
  { 0, 1, 2, 3, 4, 7, },
  { 5, 7, },
  { 0, 5, 7, },
  { 1, 5, 7, },
  { 0, 1, 5, 7, },
  { 2, 5, 7, },
  { 0, 2, 5, 7, },
  { 1, 2, 5, 7, },
  { 0, 1, 2, 5, 7, },
  { 3, 5, 7, },
  { 0, 3, 5, 7, },
  { 1, 3, 5, 7, },
  { 0, 1, 3, 5, 7, },
  { 2, 3, 5, 7, },
  { 0, 2, 3, 5, 7, },
  { 1, 2, 3, 5, 7, },
  { 0, 1, 2, 3, 5, 7, },
  { 4, 5, 7, },
  { 0, 4, 5, 7, },
  { 1, 4, 5, 7, },
  { 0, 1, 4, 5, 7, },
  { 2, 4, 5, 7, },
  { 0, 2, 4, 5, 7, },
  { 1, 2, 4, 5, 7, },
  { 0, 1, 2, 4, 5, 7, },
  { 3, 4, 5, 7, },
  { 0, 3, 4, 5, 7, },
  { 1, 3, 4, 5, 7, },
  { 0, 1, 3, 4, 5, 7, },
  { 2, 3, 4, 5, 7, },
  { 0, 2, 3, 4, 5, 7, },
  { 1, 2, 3, 4, 5, 7, },
  { 0, 1, 2, 3, 4, 5, 7, },
  { 6, 7, },
  { 0, 6, 7, },
  { 1, 6, 7, },
  { 0, 1, 6, 7, },
  { 2, 6, 7, },
  { 0, 2, 6, 7, },
  { 1, 2, 6, 7, },
  { 0, 1, 2, 6, 7, },
  { 3, 6, 7, },
  { 0, 3, 6, 7, },
  { 1, 3, 6, 7, },
  { 0, 1, 3, 6, 7, },
  { 2, 3, 6, 7, },
  { 0, 2, 3, 6, 7, },
  { 1, 2, 3, 6, 7, },
  { 0, 1, 2, 3, 6, 7, },
  { 4, 6, 7, },
  { 0, 4, 6, 7, },
  { 1, 4, 6, 7, },
  { 0, 1, 4, 6, 7, },
  { 2, 4, 6, 7, },
  { 0, 2, 4, 6, 7, },
  { 1, 2, 4, 6, 7, },
  { 0, 1, 2, 4, 6, 7, },
  { 3, 4, 6, 7, },
  { 0, 3, 4, 6, 7, },
  { 1, 3, 4, 6, 7, },
  { 0, 1, 3, 4, 6, 7, },
  { 2, 3, 4, 6, 7, },
  { 0, 2, 3, 4, 6, 7, },
  { 1, 2, 3, 4, 6, 7, },
  { 0, 1, 2, 3, 4, 6, 7, },
  { 5, 6, 7, },
  { 0, 5, 6, 7, },
  { 1, 5, 6, 7, },
  { 0, 1, 5, 6, 7, },
  { 2, 5, 6, 7, },
  { 0, 2, 5, 6, 7, },
  { 1, 2, 5, 6, 7, },
  { 0, 1, 2, 5, 6, 7, },
  { 3, 5, 6, 7, },
  { 0, 3, 5, 6, 7, },
  { 1, 3, 5, 6, 7, },
  { 0, 1, 3, 5, 6, 7, },
  { 2, 3, 5, 6, 7, },
  { 0, 2, 3, 5, 6, 7, },
  { 1, 2, 3, 5, 6, 7, },
  { 0, 1, 2, 3, 5, 6, 7, },
  { 4, 5, 6, 7, },
  { 0, 4, 5, 6, 7, },
  { 1, 4, 5, 6, 7, },
  { 0, 1, 4, 5, 6, 7, },
  { 2, 4, 5, 6, 7, },
  { 0, 2, 4, 5, 6, 7, },
  { 1, 2, 4, 5, 6, 7, },
  { 0, 1, 2, 4, 5, 6, 7, },
  { 3, 4, 5, 6, 7, },
  { 0, 3, 4, 5, 6, 7, },
  { 1, 3, 4, 5, 6, 7, },
  { 0, 1, 3, 4, 5, 6, 7, },
  { 2, 3, 4, 5, 6, 7, },
  { 0, 2, 3, 4, 5, 6, 7, },
  { 1, 2, 3, 4, 5, 6, 7, },
  { 0, 1, 2, 3, 4, 5, 6, 7, },
  { 8, },
  { 0, 8, },
  { 1, 8, },
  { 0, 1, 8, },
  { 2, 8, },
  { 0, 2, 8, },
  { 1, 2, 8, },
  { 0, 1, 2, 8, },
  { 3, 8, },
  { 0, 3, 8, },
  { 1, 3, 8, },
  { 0, 1, 3, 8, },
  { 2, 3, 8, },
  { 0, 2, 3, 8, },
  { 1, 2, 3, 8, },
  { 0, 1, 2, 3, 8, },
  { 4, 8, },
  { 0, 4, 8, },
  { 1, 4, 8, },
  { 0, 1, 4, 8, },
  { 2, 4, 8, },
  { 0, 2, 4, 8, },
  { 1, 2, 4, 8, },
  { 0, 1, 2, 4, 8, },
  { 3, 4, 8, },
  { 0, 3, 4, 8, },
  { 1, 3, 4, 8, },
  { 0, 1, 3, 4, 8, },
  { 2, 3, 4, 8, },
  { 0, 2, 3, 4, 8, },
  { 1, 2, 3, 4, 8, },
  { 0, 1, 2, 3, 4, 8, },
  { 5, 8, },
  { 0, 5, 8, },
  { 1, 5, 8, },
  { 0, 1, 5, 8, },
  { 2, 5, 8, },
  { 0, 2, 5, 8, },
  { 1, 2, 5, 8, },
  { 0, 1, 2, 5, 8, },
  { 3, 5, 8, },
  { 0, 3, 5, 8, },
  { 1, 3, 5, 8, },
  { 0, 1, 3, 5, 8, },
  { 2, 3, 5, 8, },
  { 0, 2, 3, 5, 8, },
  { 1, 2, 3, 5, 8, },
  { 0, 1, 2, 3, 5, 8, },
  { 4, 5, 8, },
  { 0, 4, 5, 8, },
  { 1, 4, 5, 8, },
  { 0, 1, 4, 5, 8, },
  { 2, 4, 5, 8, },
  { 0, 2, 4, 5, 8, },
  { 1, 2, 4, 5, 8, },
  { 0, 1, 2, 4, 5, 8, },
  { 3, 4, 5, 8, },
  { 0, 3, 4, 5, 8, },
  { 1, 3, 4, 5, 8, },
  { 0, 1, 3, 4, 5, 8, },
  { 2, 3, 4, 5, 8, },
  { 0, 2, 3, 4, 5, 8, },
  { 1, 2, 3, 4, 5, 8, },
  { 0, 1, 2, 3, 4, 5, 8, },
  { 6, 8, },
  { 0, 6, 8, },
  { 1, 6, 8, },
  { 0, 1, 6, 8, },
  { 2, 6, 8, },
  { 0, 2, 6, 8, },
  { 1, 2, 6, 8, },
  { 0, 1, 2, 6, 8, },
  { 3, 6, 8, },
  { 0, 3, 6, 8, },
  { 1, 3, 6, 8, },
  { 0, 1, 3, 6, 8, },
  { 2, 3, 6, 8, },
  { 0, 2, 3, 6, 8, },
  { 1, 2, 3, 6, 8, },
  { 0, 1, 2, 3, 6, 8, },
  { 4, 6, 8, },
  { 0, 4, 6, 8, },
  { 1, 4, 6, 8, },
  { 0, 1, 4, 6, 8, },
  { 2, 4, 6, 8, },
  { 0, 2, 4, 6, 8, },
  { 1, 2, 4, 6, 8, },
  { 0, 1, 2, 4, 6, 8, },
  { 3, 4, 6, 8, },
  { 0, 3, 4, 6, 8, },
  { 1, 3, 4, 6, 8, },
  { 0, 1, 3, 4, 6, 8, },
  { 2, 3, 4, 6, 8, },
  { 0, 2, 3, 4, 6, 8, },
  { 1, 2, 3, 4, 6, 8, },
  { 0, 1, 2, 3, 4, 6, 8, },
  { 5, 6, 8, },
  { 0, 5, 6, 8, },
  { 1, 5, 6, 8, },
  { 0, 1, 5, 6, 8, },
  { 2, 5, 6, 8, },
  { 0, 2, 5, 6, 8, },
  { 1, 2, 5, 6, 8, },
  { 0, 1, 2, 5, 6, 8, },
  { 3, 5, 6, 8, },
  { 0, 3, 5, 6, 8, },
  { 1, 3, 5, 6, 8, },
  { 0, 1, 3, 5, 6, 8, },
  { 2, 3, 5, 6, 8, },
  { 0, 2, 3, 5, 6, 8, },
  { 1, 2, 3, 5, 6, 8, },
  { 0, 1, 2, 3, 5, 6, 8, },
  { 4, 5, 6, 8, },
  { 0, 4, 5, 6, 8, },
  { 1, 4, 5, 6, 8, },
  { 0, 1, 4, 5, 6, 8, },
  { 2, 4, 5, 6, 8, },
  { 0, 2, 4, 5, 6, 8, },
  { 1, 2, 4, 5, 6, 8, },
  { 0, 1, 2, 4, 5, 6, 8, },
  { 3, 4, 5, 6, 8, },
  { 0, 3, 4, 5, 6, 8, },
  { 1, 3, 4, 5, 6, 8, },
  { 0, 1, 3, 4, 5, 6, 8, },
  { 2, 3, 4, 5, 6, 8, },
  { 0, 2, 3, 4, 5, 6, 8, },
  { 1, 2, 3, 4, 5, 6, 8, },
  { 0, 1, 2, 3, 4, 5, 6, 8, },
  { 7, 8, },
  { 0, 7, 8, },
  { 1, 7, 8, },
  { 0, 1, 7, 8, },
  { 2, 7, 8, },
  { 0, 2, 7, 8, },
  { 1, 2, 7, 8, },
  { 0, 1, 2, 7, 8, },
  { 3, 7, 8, },
  { 0, 3, 7, 8, },
  { 1, 3, 7, 8, },
  { 0, 1, 3, 7, 8, },
  { 2, 3, 7, 8, },
  { 0, 2, 3, 7, 8, },
  { 1, 2, 3, 7, 8, },
  { 0, 1, 2, 3, 7, 8, },
  { 4, 7, 8, },
  { 0, 4, 7, 8, },
  { 1, 4, 7, 8, },
  { 0, 1, 4, 7, 8, },
  { 2, 4, 7, 8, },
  { 0, 2, 4, 7, 8, },
  { 1, 2, 4, 7, 8, },
  { 0, 1, 2, 4, 7, 8, },
  { 3, 4, 7, 8, },
  { 0, 3, 4, 7, 8, },
  { 1, 3, 4, 7, 8, },
  { 0, 1, 3, 4, 7, 8, },
  { 2, 3, 4, 7, 8, },
  { 0, 2, 3, 4, 7, 8, },
  { 1, 2, 3, 4, 7, 8, },
  { 0, 1, 2, 3, 4, 7, 8, },
  { 5, 7, 8, },
  { 0, 5, 7, 8, },
  { 1, 5, 7, 8, },
  { 0, 1, 5, 7, 8, },
  { 2, 5, 7, 8, },
  { 0, 2, 5, 7, 8, },
  { 1, 2, 5, 7, 8, },
  { 0, 1, 2, 5, 7, 8, },
  { 3, 5, 7, 8, },
  { 0, 3, 5, 7, 8, },
  { 1, 3, 5, 7, 8, },
  { 0, 1, 3, 5, 7, 8, },
  { 2, 3, 5, 7, 8, },
  { 0, 2, 3, 5, 7, 8, },
  { 1, 2, 3, 5, 7, 8, },
  { 0, 1, 2, 3, 5, 7, 8, },
  { 4, 5, 7, 8, },
  { 0, 4, 5, 7, 8, },
  { 1, 4, 5, 7, 8, },
  { 0, 1, 4, 5, 7, 8, },
  { 2, 4, 5, 7, 8, },
  { 0, 2, 4, 5, 7, 8, },
  { 1, 2, 4, 5, 7, 8, },
  { 0, 1, 2, 4, 5, 7, 8, },
  { 3, 4, 5, 7, 8, },
  { 0, 3, 4, 5, 7, 8, },
  { 1, 3, 4, 5, 7, 8, },
  { 0, 1, 3, 4, 5, 7, 8, },
  { 2, 3, 4, 5, 7, 8, },
  { 0, 2, 3, 4, 5, 7, 8, },
  { 1, 2, 3, 4, 5, 7, 8, },
  { 0, 1, 2, 3, 4, 5, 7, 8, },
  { 6, 7, 8, },
  { 0, 6, 7, 8, },
  { 1, 6, 7, 8, },
  { 0, 1, 6, 7, 8, },
  { 2, 6, 7, 8, },
  { 0, 2, 6, 7, 8, },
  { 1, 2, 6, 7, 8, },
  { 0, 1, 2, 6, 7, 8, },
  { 3, 6, 7, 8, },
  { 0, 3, 6, 7, 8, },
  { 1, 3, 6, 7, 8, },
  { 0, 1, 3, 6, 7, 8, },
  { 2, 3, 6, 7, 8, },
  { 0, 2, 3, 6, 7, 8, },
  { 1, 2, 3, 6, 7, 8, },
  { 0, 1, 2, 3, 6, 7, 8, },
  { 4, 6, 7, 8, },
  { 0, 4, 6, 7, 8, },
  { 1, 4, 6, 7, 8, },
  { 0, 1, 4, 6, 7, 8, },
  { 2, 4, 6, 7, 8, },
  { 0, 2, 4, 6, 7, 8, },
  { 1, 2, 4, 6, 7, 8, },
  { 0, 1, 2, 4, 6, 7, 8, },
  { 3, 4, 6, 7, 8, },
  { 0, 3, 4, 6, 7, 8, },
  { 1, 3, 4, 6, 7, 8, },
  { 0, 1, 3, 4, 6, 7, 8, },
  { 2, 3, 4, 6, 7, 8, },
  { 0, 2, 3, 4, 6, 7, 8, },
  { 1, 2, 3, 4, 6, 7, 8, },
  { 0, 1, 2, 3, 4, 6, 7, 8, },
  { 5, 6, 7, 8, },
  { 0, 5, 6, 7, 8, },
  { 1, 5, 6, 7, 8, },
  { 0, 1, 5, 6, 7, 8, },
  { 2, 5, 6, 7, 8, },
  { 0, 2, 5, 6, 7, 8, },
  { 1, 2, 5, 6, 7, 8, },
  { 0, 1, 2, 5, 6, 7, 8, },
  { 3, 5, 6, 7, 8, },
  { 0, 3, 5, 6, 7, 8, },
  { 1, 3, 5, 6, 7, 8, },
  { 0, 1, 3, 5, 6, 7, 8, },
  { 2, 3, 5, 6, 7, 8, },
  { 0, 2, 3, 5, 6, 7, 8, },
  { 1, 2, 3, 5, 6, 7, 8, },
  { 0, 1, 2, 3, 5, 6, 7, 8, },
  { 4, 5, 6, 7, 8, },
  { 0, 4, 5, 6, 7, 8, },
  { 1, 4, 5, 6, 7, 8, },
  { 0, 1, 4, 5, 6, 7, 8, },
  { 2, 4, 5, 6, 7, 8, },
  { 0, 2, 4, 5, 6, 7, 8, },
  { 1, 2, 4, 5, 6, 7, 8, },
  { 0, 1, 2, 4, 5, 6, 7, 8, },
  { 3, 4, 5, 6, 7, 8, },
  { 0, 3, 4, 5, 6, 7, 8, },
  { 1, 3, 4, 5, 6, 7, 8, },
  { 0, 1, 3, 4, 5, 6, 7, 8, },
  { 2, 3, 4, 5, 6, 7, 8, },
  { 0, 2, 3, 4, 5, 6, 7, 8, },
  { 1, 2, 3, 4, 5, 6, 7, 8, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, },
  { 9, },
  { 0, 9, },
  { 1, 9, },
  { 0, 1, 9, },
  { 2, 9, },
  { 0, 2, 9, },
  { 1, 2, 9, },
  { 0, 1, 2, 9, },
  { 3, 9, },
  { 0, 3, 9, },
  { 1, 3, 9, },
  { 0, 1, 3, 9, },
  { 2, 3, 9, },
  { 0, 2, 3, 9, },
  { 1, 2, 3, 9, },
  { 0, 1, 2, 3, 9, },
  { 4, 9, },
  { 0, 4, 9, },
  { 1, 4, 9, },
  { 0, 1, 4, 9, },
  { 2, 4, 9, },
  { 0, 2, 4, 9, },
  { 1, 2, 4, 9, },
  { 0, 1, 2, 4, 9, },
  { 3, 4, 9, },
  { 0, 3, 4, 9, },
  { 1, 3, 4, 9, },
  { 0, 1, 3, 4, 9, },
  { 2, 3, 4, 9, },
  { 0, 2, 3, 4, 9, },
  { 1, 2, 3, 4, 9, },
  { 0, 1, 2, 3, 4, 9, },
  { 5, 9, },
  { 0, 5, 9, },
  { 1, 5, 9, },
  { 0, 1, 5, 9, },
  { 2, 5, 9, },
  { 0, 2, 5, 9, },
  { 1, 2, 5, 9, },
  { 0, 1, 2, 5, 9, },
  { 3, 5, 9, },
  { 0, 3, 5, 9, },
  { 1, 3, 5, 9, },
  { 0, 1, 3, 5, 9, },
  { 2, 3, 5, 9, },
  { 0, 2, 3, 5, 9, },
  { 1, 2, 3, 5, 9, },
  { 0, 1, 2, 3, 5, 9, },
  { 4, 5, 9, },
  { 0, 4, 5, 9, },
  { 1, 4, 5, 9, },
  { 0, 1, 4, 5, 9, },
  { 2, 4, 5, 9, },
  { 0, 2, 4, 5, 9, },
  { 1, 2, 4, 5, 9, },
  { 0, 1, 2, 4, 5, 9, },
  { 3, 4, 5, 9, },
  { 0, 3, 4, 5, 9, },
  { 1, 3, 4, 5, 9, },
  { 0, 1, 3, 4, 5, 9, },
  { 2, 3, 4, 5, 9, },
  { 0, 2, 3, 4, 5, 9, },
  { 1, 2, 3, 4, 5, 9, },
  { 0, 1, 2, 3, 4, 5, 9, },
  { 6, 9, },
  { 0, 6, 9, },
  { 1, 6, 9, },
  { 0, 1, 6, 9, },
  { 2, 6, 9, },
  { 0, 2, 6, 9, },
  { 1, 2, 6, 9, },
  { 0, 1, 2, 6, 9, },
  { 3, 6, 9, },
  { 0, 3, 6, 9, },
  { 1, 3, 6, 9, },
  { 0, 1, 3, 6, 9, },
  { 2, 3, 6, 9, },
  { 0, 2, 3, 6, 9, },
  { 1, 2, 3, 6, 9, },
  { 0, 1, 2, 3, 6, 9, },
  { 4, 6, 9, },
  { 0, 4, 6, 9, },
  { 1, 4, 6, 9, },
  { 0, 1, 4, 6, 9, },
  { 2, 4, 6, 9, },
  { 0, 2, 4, 6, 9, },
  { 1, 2, 4, 6, 9, },
  { 0, 1, 2, 4, 6, 9, },
  { 3, 4, 6, 9, },
  { 0, 3, 4, 6, 9, },
  { 1, 3, 4, 6, 9, },
  { 0, 1, 3, 4, 6, 9, },
  { 2, 3, 4, 6, 9, },
  { 0, 2, 3, 4, 6, 9, },
  { 1, 2, 3, 4, 6, 9, },
  { 0, 1, 2, 3, 4, 6, 9, },
  { 5, 6, 9, },
  { 0, 5, 6, 9, },
  { 1, 5, 6, 9, },
  { 0, 1, 5, 6, 9, },
  { 2, 5, 6, 9, },
  { 0, 2, 5, 6, 9, },
  { 1, 2, 5, 6, 9, },
  { 0, 1, 2, 5, 6, 9, },
  { 3, 5, 6, 9, },
  { 0, 3, 5, 6, 9, },
  { 1, 3, 5, 6, 9, },
  { 0, 1, 3, 5, 6, 9, },
  { 2, 3, 5, 6, 9, },
  { 0, 2, 3, 5, 6, 9, },
  { 1, 2, 3, 5, 6, 9, },
  { 0, 1, 2, 3, 5, 6, 9, },
  { 4, 5, 6, 9, },
  { 0, 4, 5, 6, 9, },
  { 1, 4, 5, 6, 9, },
  { 0, 1, 4, 5, 6, 9, },
  { 2, 4, 5, 6, 9, },
  { 0, 2, 4, 5, 6, 9, },
  { 1, 2, 4, 5, 6, 9, },
  { 0, 1, 2, 4, 5, 6, 9, },
  { 3, 4, 5, 6, 9, },
  { 0, 3, 4, 5, 6, 9, },
  { 1, 3, 4, 5, 6, 9, },
  { 0, 1, 3, 4, 5, 6, 9, },
  { 2, 3, 4, 5, 6, 9, },
  { 0, 2, 3, 4, 5, 6, 9, },
  { 1, 2, 3, 4, 5, 6, 9, },
  { 0, 1, 2, 3, 4, 5, 6, 9, },
  { 7, 9, },
  { 0, 7, 9, },
  { 1, 7, 9, },
  { 0, 1, 7, 9, },
  { 2, 7, 9, },
  { 0, 2, 7, 9, },
  { 1, 2, 7, 9, },
  { 0, 1, 2, 7, 9, },
  { 3, 7, 9, },
  { 0, 3, 7, 9, },
  { 1, 3, 7, 9, },
  { 0, 1, 3, 7, 9, },
  { 2, 3, 7, 9, },
  { 0, 2, 3, 7, 9, },
  { 1, 2, 3, 7, 9, },
  { 0, 1, 2, 3, 7, 9, },
  { 4, 7, 9, },
  { 0, 4, 7, 9, },
  { 1, 4, 7, 9, },
  { 0, 1, 4, 7, 9, },
  { 2, 4, 7, 9, },
  { 0, 2, 4, 7, 9, },
  { 1, 2, 4, 7, 9, },
  { 0, 1, 2, 4, 7, 9, },
  { 3, 4, 7, 9, },
  { 0, 3, 4, 7, 9, },
  { 1, 3, 4, 7, 9, },
  { 0, 1, 3, 4, 7, 9, },
  { 2, 3, 4, 7, 9, },
  { 0, 2, 3, 4, 7, 9, },
  { 1, 2, 3, 4, 7, 9, },
  { 0, 1, 2, 3, 4, 7, 9, },
  { 5, 7, 9, },
  { 0, 5, 7, 9, },
  { 1, 5, 7, 9, },
  { 0, 1, 5, 7, 9, },
  { 2, 5, 7, 9, },
  { 0, 2, 5, 7, 9, },
  { 1, 2, 5, 7, 9, },
  { 0, 1, 2, 5, 7, 9, },
  { 3, 5, 7, 9, },
  { 0, 3, 5, 7, 9, },
  { 1, 3, 5, 7, 9, },
  { 0, 1, 3, 5, 7, 9, },
  { 2, 3, 5, 7, 9, },
  { 0, 2, 3, 5, 7, 9, },
  { 1, 2, 3, 5, 7, 9, },
  { 0, 1, 2, 3, 5, 7, 9, },
  { 4, 5, 7, 9, },
  { 0, 4, 5, 7, 9, },
  { 1, 4, 5, 7, 9, },
  { 0, 1, 4, 5, 7, 9, },
  { 2, 4, 5, 7, 9, },
  { 0, 2, 4, 5, 7, 9, },
  { 1, 2, 4, 5, 7, 9, },
  { 0, 1, 2, 4, 5, 7, 9, },
  { 3, 4, 5, 7, 9, },
  { 0, 3, 4, 5, 7, 9, },
  { 1, 3, 4, 5, 7, 9, },
  { 0, 1, 3, 4, 5, 7, 9, },
  { 2, 3, 4, 5, 7, 9, },
  { 0, 2, 3, 4, 5, 7, 9, },
  { 1, 2, 3, 4, 5, 7, 9, },
  { 0, 1, 2, 3, 4, 5, 7, 9, },
  { 6, 7, 9, },
  { 0, 6, 7, 9, },
  { 1, 6, 7, 9, },
  { 0, 1, 6, 7, 9, },
  { 2, 6, 7, 9, },
  { 0, 2, 6, 7, 9, },
  { 1, 2, 6, 7, 9, },
  { 0, 1, 2, 6, 7, 9, },
  { 3, 6, 7, 9, },
  { 0, 3, 6, 7, 9, },
  { 1, 3, 6, 7, 9, },
  { 0, 1, 3, 6, 7, 9, },
  { 2, 3, 6, 7, 9, },
  { 0, 2, 3, 6, 7, 9, },
  { 1, 2, 3, 6, 7, 9, },
  { 0, 1, 2, 3, 6, 7, 9, },
  { 4, 6, 7, 9, },
  { 0, 4, 6, 7, 9, },
  { 1, 4, 6, 7, 9, },
  { 0, 1, 4, 6, 7, 9, },
  { 2, 4, 6, 7, 9, },
  { 0, 2, 4, 6, 7, 9, },
  { 1, 2, 4, 6, 7, 9, },
  { 0, 1, 2, 4, 6, 7, 9, },
  { 3, 4, 6, 7, 9, },
  { 0, 3, 4, 6, 7, 9, },
  { 1, 3, 4, 6, 7, 9, },
  { 0, 1, 3, 4, 6, 7, 9, },
  { 2, 3, 4, 6, 7, 9, },
  { 0, 2, 3, 4, 6, 7, 9, },
  { 1, 2, 3, 4, 6, 7, 9, },
  { 0, 1, 2, 3, 4, 6, 7, 9, },
  { 5, 6, 7, 9, },
  { 0, 5, 6, 7, 9, },
  { 1, 5, 6, 7, 9, },
  { 0, 1, 5, 6, 7, 9, },
  { 2, 5, 6, 7, 9, },
  { 0, 2, 5, 6, 7, 9, },
  { 1, 2, 5, 6, 7, 9, },
  { 0, 1, 2, 5, 6, 7, 9, },
  { 3, 5, 6, 7, 9, },
  { 0, 3, 5, 6, 7, 9, },
  { 1, 3, 5, 6, 7, 9, },
  { 0, 1, 3, 5, 6, 7, 9, },
  { 2, 3, 5, 6, 7, 9, },
  { 0, 2, 3, 5, 6, 7, 9, },
  { 1, 2, 3, 5, 6, 7, 9, },
  { 0, 1, 2, 3, 5, 6, 7, 9, },
  { 4, 5, 6, 7, 9, },
  { 0, 4, 5, 6, 7, 9, },
  { 1, 4, 5, 6, 7, 9, },
  { 0, 1, 4, 5, 6, 7, 9, },
  { 2, 4, 5, 6, 7, 9, },
  { 0, 2, 4, 5, 6, 7, 9, },
  { 1, 2, 4, 5, 6, 7, 9, },
  { 0, 1, 2, 4, 5, 6, 7, 9, },
  { 3, 4, 5, 6, 7, 9, },
  { 0, 3, 4, 5, 6, 7, 9, },
  { 1, 3, 4, 5, 6, 7, 9, },
  { 0, 1, 3, 4, 5, 6, 7, 9, },
  { 2, 3, 4, 5, 6, 7, 9, },
  { 0, 2, 3, 4, 5, 6, 7, 9, },
  { 1, 2, 3, 4, 5, 6, 7, 9, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, },
  { 8, 9, },
  { 0, 8, 9, },
  { 1, 8, 9, },
  { 0, 1, 8, 9, },
  { 2, 8, 9, },
  { 0, 2, 8, 9, },
  { 1, 2, 8, 9, },
  { 0, 1, 2, 8, 9, },
  { 3, 8, 9, },
  { 0, 3, 8, 9, },
  { 1, 3, 8, 9, },
  { 0, 1, 3, 8, 9, },
  { 2, 3, 8, 9, },
  { 0, 2, 3, 8, 9, },
  { 1, 2, 3, 8, 9, },
  { 0, 1, 2, 3, 8, 9, },
  { 4, 8, 9, },
  { 0, 4, 8, 9, },
  { 1, 4, 8, 9, },
  { 0, 1, 4, 8, 9, },
  { 2, 4, 8, 9, },
  { 0, 2, 4, 8, 9, },
  { 1, 2, 4, 8, 9, },
  { 0, 1, 2, 4, 8, 9, },
  { 3, 4, 8, 9, },
  { 0, 3, 4, 8, 9, },
  { 1, 3, 4, 8, 9, },
  { 0, 1, 3, 4, 8, 9, },
  { 2, 3, 4, 8, 9, },
  { 0, 2, 3, 4, 8, 9, },
  { 1, 2, 3, 4, 8, 9, },
  { 0, 1, 2, 3, 4, 8, 9, },
  { 5, 8, 9, },
  { 0, 5, 8, 9, },
  { 1, 5, 8, 9, },
  { 0, 1, 5, 8, 9, },
  { 2, 5, 8, 9, },
  { 0, 2, 5, 8, 9, },
  { 1, 2, 5, 8, 9, },
  { 0, 1, 2, 5, 8, 9, },
  { 3, 5, 8, 9, },
  { 0, 3, 5, 8, 9, },
  { 1, 3, 5, 8, 9, },
  { 0, 1, 3, 5, 8, 9, },
  { 2, 3, 5, 8, 9, },
  { 0, 2, 3, 5, 8, 9, },
  { 1, 2, 3, 5, 8, 9, },
  { 0, 1, 2, 3, 5, 8, 9, },
  { 4, 5, 8, 9, },
  { 0, 4, 5, 8, 9, },
  { 1, 4, 5, 8, 9, },
  { 0, 1, 4, 5, 8, 9, },
  { 2, 4, 5, 8, 9, },
  { 0, 2, 4, 5, 8, 9, },
  { 1, 2, 4, 5, 8, 9, },
  { 0, 1, 2, 4, 5, 8, 9, },
  { 3, 4, 5, 8, 9, },
  { 0, 3, 4, 5, 8, 9, },
  { 1, 3, 4, 5, 8, 9, },
  { 0, 1, 3, 4, 5, 8, 9, },
  { 2, 3, 4, 5, 8, 9, },
  { 0, 2, 3, 4, 5, 8, 9, },
  { 1, 2, 3, 4, 5, 8, 9, },
  { 0, 1, 2, 3, 4, 5, 8, 9, },
  { 6, 8, 9, },
  { 0, 6, 8, 9, },
  { 1, 6, 8, 9, },
  { 0, 1, 6, 8, 9, },
  { 2, 6, 8, 9, },
  { 0, 2, 6, 8, 9, },
  { 1, 2, 6, 8, 9, },
  { 0, 1, 2, 6, 8, 9, },
  { 3, 6, 8, 9, },
  { 0, 3, 6, 8, 9, },
  { 1, 3, 6, 8, 9, },
  { 0, 1, 3, 6, 8, 9, },
  { 2, 3, 6, 8, 9, },
  { 0, 2, 3, 6, 8, 9, },
  { 1, 2, 3, 6, 8, 9, },
  { 0, 1, 2, 3, 6, 8, 9, },
  { 4, 6, 8, 9, },
  { 0, 4, 6, 8, 9, },
  { 1, 4, 6, 8, 9, },
  { 0, 1, 4, 6, 8, 9, },
  { 2, 4, 6, 8, 9, },
  { 0, 2, 4, 6, 8, 9, },
  { 1, 2, 4, 6, 8, 9, },
  { 0, 1, 2, 4, 6, 8, 9, },
  { 3, 4, 6, 8, 9, },
  { 0, 3, 4, 6, 8, 9, },
  { 1, 3, 4, 6, 8, 9, },
  { 0, 1, 3, 4, 6, 8, 9, },
  { 2, 3, 4, 6, 8, 9, },
  { 0, 2, 3, 4, 6, 8, 9, },
  { 1, 2, 3, 4, 6, 8, 9, },
  { 0, 1, 2, 3, 4, 6, 8, 9, },
  { 5, 6, 8, 9, },
  { 0, 5, 6, 8, 9, },
  { 1, 5, 6, 8, 9, },
  { 0, 1, 5, 6, 8, 9, },
  { 2, 5, 6, 8, 9, },
  { 0, 2, 5, 6, 8, 9, },
  { 1, 2, 5, 6, 8, 9, },
  { 0, 1, 2, 5, 6, 8, 9, },
  { 3, 5, 6, 8, 9, },
  { 0, 3, 5, 6, 8, 9, },
  { 1, 3, 5, 6, 8, 9, },
  { 0, 1, 3, 5, 6, 8, 9, },
  { 2, 3, 5, 6, 8, 9, },
  { 0, 2, 3, 5, 6, 8, 9, },
  { 1, 2, 3, 5, 6, 8, 9, },
  { 0, 1, 2, 3, 5, 6, 8, 9, },
  { 4, 5, 6, 8, 9, },
  { 0, 4, 5, 6, 8, 9, },
  { 1, 4, 5, 6, 8, 9, },
  { 0, 1, 4, 5, 6, 8, 9, },
  { 2, 4, 5, 6, 8, 9, },
  { 0, 2, 4, 5, 6, 8, 9, },
  { 1, 2, 4, 5, 6, 8, 9, },
  { 0, 1, 2, 4, 5, 6, 8, 9, },
  { 3, 4, 5, 6, 8, 9, },
  { 0, 3, 4, 5, 6, 8, 9, },
  { 1, 3, 4, 5, 6, 8, 9, },
  { 0, 1, 3, 4, 5, 6, 8, 9, },
  { 2, 3, 4, 5, 6, 8, 9, },
  { 0, 2, 3, 4, 5, 6, 8, 9, },
  { 1, 2, 3, 4, 5, 6, 8, 9, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, },
  { 7, 8, 9, },
  { 0, 7, 8, 9, },
  { 1, 7, 8, 9, },
  { 0, 1, 7, 8, 9, },
  { 2, 7, 8, 9, },
  { 0, 2, 7, 8, 9, },
  { 1, 2, 7, 8, 9, },
  { 0, 1, 2, 7, 8, 9, },
  { 3, 7, 8, 9, },
  { 0, 3, 7, 8, 9, },
  { 1, 3, 7, 8, 9, },
  { 0, 1, 3, 7, 8, 9, },
  { 2, 3, 7, 8, 9, },
  { 0, 2, 3, 7, 8, 9, },
  { 1, 2, 3, 7, 8, 9, },
  { 0, 1, 2, 3, 7, 8, 9, },
  { 4, 7, 8, 9, },
  { 0, 4, 7, 8, 9, },
  { 1, 4, 7, 8, 9, },
  { 0, 1, 4, 7, 8, 9, },
  { 2, 4, 7, 8, 9, },
  { 0, 2, 4, 7, 8, 9, },
  { 1, 2, 4, 7, 8, 9, },
  { 0, 1, 2, 4, 7, 8, 9, },
  { 3, 4, 7, 8, 9, },
  { 0, 3, 4, 7, 8, 9, },
  { 1, 3, 4, 7, 8, 9, },
  { 0, 1, 3, 4, 7, 8, 9, },
  { 2, 3, 4, 7, 8, 9, },
  { 0, 2, 3, 4, 7, 8, 9, },
  { 1, 2, 3, 4, 7, 8, 9, },
  { 0, 1, 2, 3, 4, 7, 8, 9, },
  { 5, 7, 8, 9, },
  { 0, 5, 7, 8, 9, },
  { 1, 5, 7, 8, 9, },
  { 0, 1, 5, 7, 8, 9, },
  { 2, 5, 7, 8, 9, },
  { 0, 2, 5, 7, 8, 9, },
  { 1, 2, 5, 7, 8, 9, },
  { 0, 1, 2, 5, 7, 8, 9, },
  { 3, 5, 7, 8, 9, },
  { 0, 3, 5, 7, 8, 9, },
  { 1, 3, 5, 7, 8, 9, },
  { 0, 1, 3, 5, 7, 8, 9, },
  { 2, 3, 5, 7, 8, 9, },
  { 0, 2, 3, 5, 7, 8, 9, },
  { 1, 2, 3, 5, 7, 8, 9, },
  { 0, 1, 2, 3, 5, 7, 8, 9, },
  { 4, 5, 7, 8, 9, },
  { 0, 4, 5, 7, 8, 9, },
  { 1, 4, 5, 7, 8, 9, },
  { 0, 1, 4, 5, 7, 8, 9, },
  { 2, 4, 5, 7, 8, 9, },
  { 0, 2, 4, 5, 7, 8, 9, },
  { 1, 2, 4, 5, 7, 8, 9, },
  { 0, 1, 2, 4, 5, 7, 8, 9, },
  { 3, 4, 5, 7, 8, 9, },
  { 0, 3, 4, 5, 7, 8, 9, },
  { 1, 3, 4, 5, 7, 8, 9, },
  { 0, 1, 3, 4, 5, 7, 8, 9, },
  { 2, 3, 4, 5, 7, 8, 9, },
  { 0, 2, 3, 4, 5, 7, 8, 9, },
  { 1, 2, 3, 4, 5, 7, 8, 9, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, },
  { 6, 7, 8, 9, },
  { 0, 6, 7, 8, 9, },
  { 1, 6, 7, 8, 9, },
  { 0, 1, 6, 7, 8, 9, },
  { 2, 6, 7, 8, 9, },
  { 0, 2, 6, 7, 8, 9, },
  { 1, 2, 6, 7, 8, 9, },
  { 0, 1, 2, 6, 7, 8, 9, },
  { 3, 6, 7, 8, 9, },
  { 0, 3, 6, 7, 8, 9, },
  { 1, 3, 6, 7, 8, 9, },
  { 0, 1, 3, 6, 7, 8, 9, },
  { 2, 3, 6, 7, 8, 9, },
  { 0, 2, 3, 6, 7, 8, 9, },
  { 1, 2, 3, 6, 7, 8, 9, },
  { 0, 1, 2, 3, 6, 7, 8, 9, },
  { 4, 6, 7, 8, 9, },
  { 0, 4, 6, 7, 8, 9, },
  { 1, 4, 6, 7, 8, 9, },
  { 0, 1, 4, 6, 7, 8, 9, },
  { 2, 4, 6, 7, 8, 9, },
  { 0, 2, 4, 6, 7, 8, 9, },
  { 1, 2, 4, 6, 7, 8, 9, },
  { 0, 1, 2, 4, 6, 7, 8, 9, },
  { 3, 4, 6, 7, 8, 9, },
  { 0, 3, 4, 6, 7, 8, 9, },
  { 1, 3, 4, 6, 7, 8, 9, },
  { 0, 1, 3, 4, 6, 7, 8, 9, },
  { 2, 3, 4, 6, 7, 8, 9, },
  { 0, 2, 3, 4, 6, 7, 8, 9, },
  { 1, 2, 3, 4, 6, 7, 8, 9, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, },
  { 5, 6, 7, 8, 9, },
  { 0, 5, 6, 7, 8, 9, },
  { 1, 5, 6, 7, 8, 9, },
  { 0, 1, 5, 6, 7, 8, 9, },
  { 2, 5, 6, 7, 8, 9, },
  { 0, 2, 5, 6, 7, 8, 9, },
  { 1, 2, 5, 6, 7, 8, 9, },
  { 0, 1, 2, 5, 6, 7, 8, 9, },
  { 3, 5, 6, 7, 8, 9, },
  { 0, 3, 5, 6, 7, 8, 9, },
  { 1, 3, 5, 6, 7, 8, 9, },
  { 0, 1, 3, 5, 6, 7, 8, 9, },
  { 2, 3, 5, 6, 7, 8, 9, },
  { 0, 2, 3, 5, 6, 7, 8, 9, },
  { 1, 2, 3, 5, 6, 7, 8, 9, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, },
  { 4, 5, 6, 7, 8, 9, },
  { 0, 4, 5, 6, 7, 8, 9, },
  { 1, 4, 5, 6, 7, 8, 9, },
  { 0, 1, 4, 5, 6, 7, 8, 9, },
  { 2, 4, 5, 6, 7, 8, 9, },
  { 0, 2, 4, 5, 6, 7, 8, 9, },
  { 1, 2, 4, 5, 6, 7, 8, 9, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, },
  { 3, 4, 5, 6, 7, 8, 9, },
  { 0, 3, 4, 5, 6, 7, 8, 9, },
  { 1, 3, 4, 5, 6, 7, 8, 9, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, },
  { 2, 3, 4, 5, 6, 7, 8, 9, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, },
  { 10, },
  { 0, 10, },
  { 1, 10, },
  { 0, 1, 10, },
  { 2, 10, },
  { 0, 2, 10, },
  { 1, 2, 10, },
  { 0, 1, 2, 10, },
  { 3, 10, },
  { 0, 3, 10, },
  { 1, 3, 10, },
  { 0, 1, 3, 10, },
  { 2, 3, 10, },
  { 0, 2, 3, 10, },
  { 1, 2, 3, 10, },
  { 0, 1, 2, 3, 10, },
  { 4, 10, },
  { 0, 4, 10, },
  { 1, 4, 10, },
  { 0, 1, 4, 10, },
  { 2, 4, 10, },
  { 0, 2, 4, 10, },
  { 1, 2, 4, 10, },
  { 0, 1, 2, 4, 10, },
  { 3, 4, 10, },
  { 0, 3, 4, 10, },
  { 1, 3, 4, 10, },
  { 0, 1, 3, 4, 10, },
  { 2, 3, 4, 10, },
  { 0, 2, 3, 4, 10, },
  { 1, 2, 3, 4, 10, },
  { 0, 1, 2, 3, 4, 10, },
  { 5, 10, },
  { 0, 5, 10, },
  { 1, 5, 10, },
  { 0, 1, 5, 10, },
  { 2, 5, 10, },
  { 0, 2, 5, 10, },
  { 1, 2, 5, 10, },
  { 0, 1, 2, 5, 10, },
  { 3, 5, 10, },
  { 0, 3, 5, 10, },
  { 1, 3, 5, 10, },
  { 0, 1, 3, 5, 10, },
  { 2, 3, 5, 10, },
  { 0, 2, 3, 5, 10, },
  { 1, 2, 3, 5, 10, },
  { 0, 1, 2, 3, 5, 10, },
  { 4, 5, 10, },
  { 0, 4, 5, 10, },
  { 1, 4, 5, 10, },
  { 0, 1, 4, 5, 10, },
  { 2, 4, 5, 10, },
  { 0, 2, 4, 5, 10, },
  { 1, 2, 4, 5, 10, },
  { 0, 1, 2, 4, 5, 10, },
  { 3, 4, 5, 10, },
  { 0, 3, 4, 5, 10, },
  { 1, 3, 4, 5, 10, },
  { 0, 1, 3, 4, 5, 10, },
  { 2, 3, 4, 5, 10, },
  { 0, 2, 3, 4, 5, 10, },
  { 1, 2, 3, 4, 5, 10, },
  { 0, 1, 2, 3, 4, 5, 10, },
  { 6, 10, },
  { 0, 6, 10, },
  { 1, 6, 10, },
  { 0, 1, 6, 10, },
  { 2, 6, 10, },
  { 0, 2, 6, 10, },
  { 1, 2, 6, 10, },
  { 0, 1, 2, 6, 10, },
  { 3, 6, 10, },
  { 0, 3, 6, 10, },
  { 1, 3, 6, 10, },
  { 0, 1, 3, 6, 10, },
  { 2, 3, 6, 10, },
  { 0, 2, 3, 6, 10, },
  { 1, 2, 3, 6, 10, },
  { 0, 1, 2, 3, 6, 10, },
  { 4, 6, 10, },
  { 0, 4, 6, 10, },
  { 1, 4, 6, 10, },
  { 0, 1, 4, 6, 10, },
  { 2, 4, 6, 10, },
  { 0, 2, 4, 6, 10, },
  { 1, 2, 4, 6, 10, },
  { 0, 1, 2, 4, 6, 10, },
  { 3, 4, 6, 10, },
  { 0, 3, 4, 6, 10, },
  { 1, 3, 4, 6, 10, },
  { 0, 1, 3, 4, 6, 10, },
  { 2, 3, 4, 6, 10, },
  { 0, 2, 3, 4, 6, 10, },
  { 1, 2, 3, 4, 6, 10, },
  { 0, 1, 2, 3, 4, 6, 10, },
  { 5, 6, 10, },
  { 0, 5, 6, 10, },
  { 1, 5, 6, 10, },
  { 0, 1, 5, 6, 10, },
  { 2, 5, 6, 10, },
  { 0, 2, 5, 6, 10, },
  { 1, 2, 5, 6, 10, },
  { 0, 1, 2, 5, 6, 10, },
  { 3, 5, 6, 10, },
  { 0, 3, 5, 6, 10, },
  { 1, 3, 5, 6, 10, },
  { 0, 1, 3, 5, 6, 10, },
  { 2, 3, 5, 6, 10, },
  { 0, 2, 3, 5, 6, 10, },
  { 1, 2, 3, 5, 6, 10, },
  { 0, 1, 2, 3, 5, 6, 10, },
  { 4, 5, 6, 10, },
  { 0, 4, 5, 6, 10, },
  { 1, 4, 5, 6, 10, },
  { 0, 1, 4, 5, 6, 10, },
  { 2, 4, 5, 6, 10, },
  { 0, 2, 4, 5, 6, 10, },
  { 1, 2, 4, 5, 6, 10, },
  { 0, 1, 2, 4, 5, 6, 10, },
  { 3, 4, 5, 6, 10, },
  { 0, 3, 4, 5, 6, 10, },
  { 1, 3, 4, 5, 6, 10, },
  { 0, 1, 3, 4, 5, 6, 10, },
  { 2, 3, 4, 5, 6, 10, },
  { 0, 2, 3, 4, 5, 6, 10, },
  { 1, 2, 3, 4, 5, 6, 10, },
  { 0, 1, 2, 3, 4, 5, 6, 10, },
  { 7, 10, },
  { 0, 7, 10, },
  { 1, 7, 10, },
  { 0, 1, 7, 10, },
  { 2, 7, 10, },
  { 0, 2, 7, 10, },
  { 1, 2, 7, 10, },
  { 0, 1, 2, 7, 10, },
  { 3, 7, 10, },
  { 0, 3, 7, 10, },
  { 1, 3, 7, 10, },
  { 0, 1, 3, 7, 10, },
  { 2, 3, 7, 10, },
  { 0, 2, 3, 7, 10, },
  { 1, 2, 3, 7, 10, },
  { 0, 1, 2, 3, 7, 10, },
  { 4, 7, 10, },
  { 0, 4, 7, 10, },
  { 1, 4, 7, 10, },
  { 0, 1, 4, 7, 10, },
  { 2, 4, 7, 10, },
  { 0, 2, 4, 7, 10, },
  { 1, 2, 4, 7, 10, },
  { 0, 1, 2, 4, 7, 10, },
  { 3, 4, 7, 10, },
  { 0, 3, 4, 7, 10, },
  { 1, 3, 4, 7, 10, },
  { 0, 1, 3, 4, 7, 10, },
  { 2, 3, 4, 7, 10, },
  { 0, 2, 3, 4, 7, 10, },
  { 1, 2, 3, 4, 7, 10, },
  { 0, 1, 2, 3, 4, 7, 10, },
  { 5, 7, 10, },
  { 0, 5, 7, 10, },
  { 1, 5, 7, 10, },
  { 0, 1, 5, 7, 10, },
  { 2, 5, 7, 10, },
  { 0, 2, 5, 7, 10, },
  { 1, 2, 5, 7, 10, },
  { 0, 1, 2, 5, 7, 10, },
  { 3, 5, 7, 10, },
  { 0, 3, 5, 7, 10, },
  { 1, 3, 5, 7, 10, },
  { 0, 1, 3, 5, 7, 10, },
  { 2, 3, 5, 7, 10, },
  { 0, 2, 3, 5, 7, 10, },
  { 1, 2, 3, 5, 7, 10, },
  { 0, 1, 2, 3, 5, 7, 10, },
  { 4, 5, 7, 10, },
  { 0, 4, 5, 7, 10, },
  { 1, 4, 5, 7, 10, },
  { 0, 1, 4, 5, 7, 10, },
  { 2, 4, 5, 7, 10, },
  { 0, 2, 4, 5, 7, 10, },
  { 1, 2, 4, 5, 7, 10, },
  { 0, 1, 2, 4, 5, 7, 10, },
  { 3, 4, 5, 7, 10, },
  { 0, 3, 4, 5, 7, 10, },
  { 1, 3, 4, 5, 7, 10, },
  { 0, 1, 3, 4, 5, 7, 10, },
  { 2, 3, 4, 5, 7, 10, },
  { 0, 2, 3, 4, 5, 7, 10, },
  { 1, 2, 3, 4, 5, 7, 10, },
  { 0, 1, 2, 3, 4, 5, 7, 10, },
  { 6, 7, 10, },
  { 0, 6, 7, 10, },
  { 1, 6, 7, 10, },
  { 0, 1, 6, 7, 10, },
  { 2, 6, 7, 10, },
  { 0, 2, 6, 7, 10, },
  { 1, 2, 6, 7, 10, },
  { 0, 1, 2, 6, 7, 10, },
  { 3, 6, 7, 10, },
  { 0, 3, 6, 7, 10, },
  { 1, 3, 6, 7, 10, },
  { 0, 1, 3, 6, 7, 10, },
  { 2, 3, 6, 7, 10, },
  { 0, 2, 3, 6, 7, 10, },
  { 1, 2, 3, 6, 7, 10, },
  { 0, 1, 2, 3, 6, 7, 10, },
  { 4, 6, 7, 10, },
  { 0, 4, 6, 7, 10, },
  { 1, 4, 6, 7, 10, },
  { 0, 1, 4, 6, 7, 10, },
  { 2, 4, 6, 7, 10, },
  { 0, 2, 4, 6, 7, 10, },
  { 1, 2, 4, 6, 7, 10, },
  { 0, 1, 2, 4, 6, 7, 10, },
  { 3, 4, 6, 7, 10, },
  { 0, 3, 4, 6, 7, 10, },
  { 1, 3, 4, 6, 7, 10, },
  { 0, 1, 3, 4, 6, 7, 10, },
  { 2, 3, 4, 6, 7, 10, },
  { 0, 2, 3, 4, 6, 7, 10, },
  { 1, 2, 3, 4, 6, 7, 10, },
  { 0, 1, 2, 3, 4, 6, 7, 10, },
  { 5, 6, 7, 10, },
  { 0, 5, 6, 7, 10, },
  { 1, 5, 6, 7, 10, },
  { 0, 1, 5, 6, 7, 10, },
  { 2, 5, 6, 7, 10, },
  { 0, 2, 5, 6, 7, 10, },
  { 1, 2, 5, 6, 7, 10, },
  { 0, 1, 2, 5, 6, 7, 10, },
  { 3, 5, 6, 7, 10, },
  { 0, 3, 5, 6, 7, 10, },
  { 1, 3, 5, 6, 7, 10, },
  { 0, 1, 3, 5, 6, 7, 10, },
  { 2, 3, 5, 6, 7, 10, },
  { 0, 2, 3, 5, 6, 7, 10, },
  { 1, 2, 3, 5, 6, 7, 10, },
  { 0, 1, 2, 3, 5, 6, 7, 10, },
  { 4, 5, 6, 7, 10, },
  { 0, 4, 5, 6, 7, 10, },
  { 1, 4, 5, 6, 7, 10, },
  { 0, 1, 4, 5, 6, 7, 10, },
  { 2, 4, 5, 6, 7, 10, },
  { 0, 2, 4, 5, 6, 7, 10, },
  { 1, 2, 4, 5, 6, 7, 10, },
  { 0, 1, 2, 4, 5, 6, 7, 10, },
  { 3, 4, 5, 6, 7, 10, },
  { 0, 3, 4, 5, 6, 7, 10, },
  { 1, 3, 4, 5, 6, 7, 10, },
  { 0, 1, 3, 4, 5, 6, 7, 10, },
  { 2, 3, 4, 5, 6, 7, 10, },
  { 0, 2, 3, 4, 5, 6, 7, 10, },
  { 1, 2, 3, 4, 5, 6, 7, 10, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, },
  { 8, 10, },
  { 0, 8, 10, },
  { 1, 8, 10, },
  { 0, 1, 8, 10, },
  { 2, 8, 10, },
  { 0, 2, 8, 10, },
  { 1, 2, 8, 10, },
  { 0, 1, 2, 8, 10, },
  { 3, 8, 10, },
  { 0, 3, 8, 10, },
  { 1, 3, 8, 10, },
  { 0, 1, 3, 8, 10, },
  { 2, 3, 8, 10, },
  { 0, 2, 3, 8, 10, },
  { 1, 2, 3, 8, 10, },
  { 0, 1, 2, 3, 8, 10, },
  { 4, 8, 10, },
  { 0, 4, 8, 10, },
  { 1, 4, 8, 10, },
  { 0, 1, 4, 8, 10, },
  { 2, 4, 8, 10, },
  { 0, 2, 4, 8, 10, },
  { 1, 2, 4, 8, 10, },
  { 0, 1, 2, 4, 8, 10, },
  { 3, 4, 8, 10, },
  { 0, 3, 4, 8, 10, },
  { 1, 3, 4, 8, 10, },
  { 0, 1, 3, 4, 8, 10, },
  { 2, 3, 4, 8, 10, },
  { 0, 2, 3, 4, 8, 10, },
  { 1, 2, 3, 4, 8, 10, },
  { 0, 1, 2, 3, 4, 8, 10, },
  { 5, 8, 10, },
  { 0, 5, 8, 10, },
  { 1, 5, 8, 10, },
  { 0, 1, 5, 8, 10, },
  { 2, 5, 8, 10, },
  { 0, 2, 5, 8, 10, },
  { 1, 2, 5, 8, 10, },
  { 0, 1, 2, 5, 8, 10, },
  { 3, 5, 8, 10, },
  { 0, 3, 5, 8, 10, },
  { 1, 3, 5, 8, 10, },
  { 0, 1, 3, 5, 8, 10, },
  { 2, 3, 5, 8, 10, },
  { 0, 2, 3, 5, 8, 10, },
  { 1, 2, 3, 5, 8, 10, },
  { 0, 1, 2, 3, 5, 8, 10, },
  { 4, 5, 8, 10, },
  { 0, 4, 5, 8, 10, },
  { 1, 4, 5, 8, 10, },
  { 0, 1, 4, 5, 8, 10, },
  { 2, 4, 5, 8, 10, },
  { 0, 2, 4, 5, 8, 10, },
  { 1, 2, 4, 5, 8, 10, },
  { 0, 1, 2, 4, 5, 8, 10, },
  { 3, 4, 5, 8, 10, },
  { 0, 3, 4, 5, 8, 10, },
  { 1, 3, 4, 5, 8, 10, },
  { 0, 1, 3, 4, 5, 8, 10, },
  { 2, 3, 4, 5, 8, 10, },
  { 0, 2, 3, 4, 5, 8, 10, },
  { 1, 2, 3, 4, 5, 8, 10, },
  { 0, 1, 2, 3, 4, 5, 8, 10, },
  { 6, 8, 10, },
  { 0, 6, 8, 10, },
  { 1, 6, 8, 10, },
  { 0, 1, 6, 8, 10, },
  { 2, 6, 8, 10, },
  { 0, 2, 6, 8, 10, },
  { 1, 2, 6, 8, 10, },
  { 0, 1, 2, 6, 8, 10, },
  { 3, 6, 8, 10, },
  { 0, 3, 6, 8, 10, },
  { 1, 3, 6, 8, 10, },
  { 0, 1, 3, 6, 8, 10, },
  { 2, 3, 6, 8, 10, },
  { 0, 2, 3, 6, 8, 10, },
  { 1, 2, 3, 6, 8, 10, },
  { 0, 1, 2, 3, 6, 8, 10, },
  { 4, 6, 8, 10, },
  { 0, 4, 6, 8, 10, },
  { 1, 4, 6, 8, 10, },
  { 0, 1, 4, 6, 8, 10, },
  { 2, 4, 6, 8, 10, },
  { 0, 2, 4, 6, 8, 10, },
  { 1, 2, 4, 6, 8, 10, },
  { 0, 1, 2, 4, 6, 8, 10, },
  { 3, 4, 6, 8, 10, },
  { 0, 3, 4, 6, 8, 10, },
  { 1, 3, 4, 6, 8, 10, },
  { 0, 1, 3, 4, 6, 8, 10, },
  { 2, 3, 4, 6, 8, 10, },
  { 0, 2, 3, 4, 6, 8, 10, },
  { 1, 2, 3, 4, 6, 8, 10, },
  { 0, 1, 2, 3, 4, 6, 8, 10, },
  { 5, 6, 8, 10, },
  { 0, 5, 6, 8, 10, },
  { 1, 5, 6, 8, 10, },
  { 0, 1, 5, 6, 8, 10, },
  { 2, 5, 6, 8, 10, },
  { 0, 2, 5, 6, 8, 10, },
  { 1, 2, 5, 6, 8, 10, },
  { 0, 1, 2, 5, 6, 8, 10, },
  { 3, 5, 6, 8, 10, },
  { 0, 3, 5, 6, 8, 10, },
  { 1, 3, 5, 6, 8, 10, },
  { 0, 1, 3, 5, 6, 8, 10, },
  { 2, 3, 5, 6, 8, 10, },
  { 0, 2, 3, 5, 6, 8, 10, },
  { 1, 2, 3, 5, 6, 8, 10, },
  { 0, 1, 2, 3, 5, 6, 8, 10, },
  { 4, 5, 6, 8, 10, },
  { 0, 4, 5, 6, 8, 10, },
  { 1, 4, 5, 6, 8, 10, },
  { 0, 1, 4, 5, 6, 8, 10, },
  { 2, 4, 5, 6, 8, 10, },
  { 0, 2, 4, 5, 6, 8, 10, },
  { 1, 2, 4, 5, 6, 8, 10, },
  { 0, 1, 2, 4, 5, 6, 8, 10, },
  { 3, 4, 5, 6, 8, 10, },
  { 0, 3, 4, 5, 6, 8, 10, },
  { 1, 3, 4, 5, 6, 8, 10, },
  { 0, 1, 3, 4, 5, 6, 8, 10, },
  { 2, 3, 4, 5, 6, 8, 10, },
  { 0, 2, 3, 4, 5, 6, 8, 10, },
  { 1, 2, 3, 4, 5, 6, 8, 10, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, },
  { 7, 8, 10, },
  { 0, 7, 8, 10, },
  { 1, 7, 8, 10, },
  { 0, 1, 7, 8, 10, },
  { 2, 7, 8, 10, },
  { 0, 2, 7, 8, 10, },
  { 1, 2, 7, 8, 10, },
  { 0, 1, 2, 7, 8, 10, },
  { 3, 7, 8, 10, },
  { 0, 3, 7, 8, 10, },
  { 1, 3, 7, 8, 10, },
  { 0, 1, 3, 7, 8, 10, },
  { 2, 3, 7, 8, 10, },
  { 0, 2, 3, 7, 8, 10, },
  { 1, 2, 3, 7, 8, 10, },
  { 0, 1, 2, 3, 7, 8, 10, },
  { 4, 7, 8, 10, },
  { 0, 4, 7, 8, 10, },
  { 1, 4, 7, 8, 10, },
  { 0, 1, 4, 7, 8, 10, },
  { 2, 4, 7, 8, 10, },
  { 0, 2, 4, 7, 8, 10, },
  { 1, 2, 4, 7, 8, 10, },
  { 0, 1, 2, 4, 7, 8, 10, },
  { 3, 4, 7, 8, 10, },
  { 0, 3, 4, 7, 8, 10, },
  { 1, 3, 4, 7, 8, 10, },
  { 0, 1, 3, 4, 7, 8, 10, },
  { 2, 3, 4, 7, 8, 10, },
  { 0, 2, 3, 4, 7, 8, 10, },
  { 1, 2, 3, 4, 7, 8, 10, },
  { 0, 1, 2, 3, 4, 7, 8, 10, },
  { 5, 7, 8, 10, },
  { 0, 5, 7, 8, 10, },
  { 1, 5, 7, 8, 10, },
  { 0, 1, 5, 7, 8, 10, },
  { 2, 5, 7, 8, 10, },
  { 0, 2, 5, 7, 8, 10, },
  { 1, 2, 5, 7, 8, 10, },
  { 0, 1, 2, 5, 7, 8, 10, },
  { 3, 5, 7, 8, 10, },
  { 0, 3, 5, 7, 8, 10, },
  { 1, 3, 5, 7, 8, 10, },
  { 0, 1, 3, 5, 7, 8, 10, },
  { 2, 3, 5, 7, 8, 10, },
  { 0, 2, 3, 5, 7, 8, 10, },
  { 1, 2, 3, 5, 7, 8, 10, },
  { 0, 1, 2, 3, 5, 7, 8, 10, },
  { 4, 5, 7, 8, 10, },
  { 0, 4, 5, 7, 8, 10, },
  { 1, 4, 5, 7, 8, 10, },
  { 0, 1, 4, 5, 7, 8, 10, },
  { 2, 4, 5, 7, 8, 10, },
  { 0, 2, 4, 5, 7, 8, 10, },
  { 1, 2, 4, 5, 7, 8, 10, },
  { 0, 1, 2, 4, 5, 7, 8, 10, },
  { 3, 4, 5, 7, 8, 10, },
  { 0, 3, 4, 5, 7, 8, 10, },
  { 1, 3, 4, 5, 7, 8, 10, },
  { 0, 1, 3, 4, 5, 7, 8, 10, },
  { 2, 3, 4, 5, 7, 8, 10, },
  { 0, 2, 3, 4, 5, 7, 8, 10, },
  { 1, 2, 3, 4, 5, 7, 8, 10, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, },
  { 6, 7, 8, 10, },
  { 0, 6, 7, 8, 10, },
  { 1, 6, 7, 8, 10, },
  { 0, 1, 6, 7, 8, 10, },
  { 2, 6, 7, 8, 10, },
  { 0, 2, 6, 7, 8, 10, },
  { 1, 2, 6, 7, 8, 10, },
  { 0, 1, 2, 6, 7, 8, 10, },
  { 3, 6, 7, 8, 10, },
  { 0, 3, 6, 7, 8, 10, },
  { 1, 3, 6, 7, 8, 10, },
  { 0, 1, 3, 6, 7, 8, 10, },
  { 2, 3, 6, 7, 8, 10, },
  { 0, 2, 3, 6, 7, 8, 10, },
  { 1, 2, 3, 6, 7, 8, 10, },
  { 0, 1, 2, 3, 6, 7, 8, 10, },
  { 4, 6, 7, 8, 10, },
  { 0, 4, 6, 7, 8, 10, },
  { 1, 4, 6, 7, 8, 10, },
  { 0, 1, 4, 6, 7, 8, 10, },
  { 2, 4, 6, 7, 8, 10, },
  { 0, 2, 4, 6, 7, 8, 10, },
  { 1, 2, 4, 6, 7, 8, 10, },
  { 0, 1, 2, 4, 6, 7, 8, 10, },
  { 3, 4, 6, 7, 8, 10, },
  { 0, 3, 4, 6, 7, 8, 10, },
  { 1, 3, 4, 6, 7, 8, 10, },
  { 0, 1, 3, 4, 6, 7, 8, 10, },
  { 2, 3, 4, 6, 7, 8, 10, },
  { 0, 2, 3, 4, 6, 7, 8, 10, },
  { 1, 2, 3, 4, 6, 7, 8, 10, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, },
  { 5, 6, 7, 8, 10, },
  { 0, 5, 6, 7, 8, 10, },
  { 1, 5, 6, 7, 8, 10, },
  { 0, 1, 5, 6, 7, 8, 10, },
  { 2, 5, 6, 7, 8, 10, },
  { 0, 2, 5, 6, 7, 8, 10, },
  { 1, 2, 5, 6, 7, 8, 10, },
  { 0, 1, 2, 5, 6, 7, 8, 10, },
  { 3, 5, 6, 7, 8, 10, },
  { 0, 3, 5, 6, 7, 8, 10, },
  { 1, 3, 5, 6, 7, 8, 10, },
  { 0, 1, 3, 5, 6, 7, 8, 10, },
  { 2, 3, 5, 6, 7, 8, 10, },
  { 0, 2, 3, 5, 6, 7, 8, 10, },
  { 1, 2, 3, 5, 6, 7, 8, 10, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, },
  { 4, 5, 6, 7, 8, 10, },
  { 0, 4, 5, 6, 7, 8, 10, },
  { 1, 4, 5, 6, 7, 8, 10, },
  { 0, 1, 4, 5, 6, 7, 8, 10, },
  { 2, 4, 5, 6, 7, 8, 10, },
  { 0, 2, 4, 5, 6, 7, 8, 10, },
  { 1, 2, 4, 5, 6, 7, 8, 10, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, },
  { 3, 4, 5, 6, 7, 8, 10, },
  { 0, 3, 4, 5, 6, 7, 8, 10, },
  { 1, 3, 4, 5, 6, 7, 8, 10, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, },
  { 2, 3, 4, 5, 6, 7, 8, 10, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, },
  { 9, 10, },
  { 0, 9, 10, },
  { 1, 9, 10, },
  { 0, 1, 9, 10, },
  { 2, 9, 10, },
  { 0, 2, 9, 10, },
  { 1, 2, 9, 10, },
  { 0, 1, 2, 9, 10, },
  { 3, 9, 10, },
  { 0, 3, 9, 10, },
  { 1, 3, 9, 10, },
  { 0, 1, 3, 9, 10, },
  { 2, 3, 9, 10, },
  { 0, 2, 3, 9, 10, },
  { 1, 2, 3, 9, 10, },
  { 0, 1, 2, 3, 9, 10, },
  { 4, 9, 10, },
  { 0, 4, 9, 10, },
  { 1, 4, 9, 10, },
  { 0, 1, 4, 9, 10, },
  { 2, 4, 9, 10, },
  { 0, 2, 4, 9, 10, },
  { 1, 2, 4, 9, 10, },
  { 0, 1, 2, 4, 9, 10, },
  { 3, 4, 9, 10, },
  { 0, 3, 4, 9, 10, },
  { 1, 3, 4, 9, 10, },
  { 0, 1, 3, 4, 9, 10, },
  { 2, 3, 4, 9, 10, },
  { 0, 2, 3, 4, 9, 10, },
  { 1, 2, 3, 4, 9, 10, },
  { 0, 1, 2, 3, 4, 9, 10, },
  { 5, 9, 10, },
  { 0, 5, 9, 10, },
  { 1, 5, 9, 10, },
  { 0, 1, 5, 9, 10, },
  { 2, 5, 9, 10, },
  { 0, 2, 5, 9, 10, },
  { 1, 2, 5, 9, 10, },
  { 0, 1, 2, 5, 9, 10, },
  { 3, 5, 9, 10, },
  { 0, 3, 5, 9, 10, },
  { 1, 3, 5, 9, 10, },
  { 0, 1, 3, 5, 9, 10, },
  { 2, 3, 5, 9, 10, },
  { 0, 2, 3, 5, 9, 10, },
  { 1, 2, 3, 5, 9, 10, },
  { 0, 1, 2, 3, 5, 9, 10, },
  { 4, 5, 9, 10, },
  { 0, 4, 5, 9, 10, },
  { 1, 4, 5, 9, 10, },
  { 0, 1, 4, 5, 9, 10, },
  { 2, 4, 5, 9, 10, },
  { 0, 2, 4, 5, 9, 10, },
  { 1, 2, 4, 5, 9, 10, },
  { 0, 1, 2, 4, 5, 9, 10, },
  { 3, 4, 5, 9, 10, },
  { 0, 3, 4, 5, 9, 10, },
  { 1, 3, 4, 5, 9, 10, },
  { 0, 1, 3, 4, 5, 9, 10, },
  { 2, 3, 4, 5, 9, 10, },
  { 0, 2, 3, 4, 5, 9, 10, },
  { 1, 2, 3, 4, 5, 9, 10, },
  { 0, 1, 2, 3, 4, 5, 9, 10, },
  { 6, 9, 10, },
  { 0, 6, 9, 10, },
  { 1, 6, 9, 10, },
  { 0, 1, 6, 9, 10, },
  { 2, 6, 9, 10, },
  { 0, 2, 6, 9, 10, },
  { 1, 2, 6, 9, 10, },
  { 0, 1, 2, 6, 9, 10, },
  { 3, 6, 9, 10, },
  { 0, 3, 6, 9, 10, },
  { 1, 3, 6, 9, 10, },
  { 0, 1, 3, 6, 9, 10, },
  { 2, 3, 6, 9, 10, },
  { 0, 2, 3, 6, 9, 10, },
  { 1, 2, 3, 6, 9, 10, },
  { 0, 1, 2, 3, 6, 9, 10, },
  { 4, 6, 9, 10, },
  { 0, 4, 6, 9, 10, },
  { 1, 4, 6, 9, 10, },
  { 0, 1, 4, 6, 9, 10, },
  { 2, 4, 6, 9, 10, },
  { 0, 2, 4, 6, 9, 10, },
  { 1, 2, 4, 6, 9, 10, },
  { 0, 1, 2, 4, 6, 9, 10, },
  { 3, 4, 6, 9, 10, },
  { 0, 3, 4, 6, 9, 10, },
  { 1, 3, 4, 6, 9, 10, },
  { 0, 1, 3, 4, 6, 9, 10, },
  { 2, 3, 4, 6, 9, 10, },
  { 0, 2, 3, 4, 6, 9, 10, },
  { 1, 2, 3, 4, 6, 9, 10, },
  { 0, 1, 2, 3, 4, 6, 9, 10, },
  { 5, 6, 9, 10, },
  { 0, 5, 6, 9, 10, },
  { 1, 5, 6, 9, 10, },
  { 0, 1, 5, 6, 9, 10, },
  { 2, 5, 6, 9, 10, },
  { 0, 2, 5, 6, 9, 10, },
  { 1, 2, 5, 6, 9, 10, },
  { 0, 1, 2, 5, 6, 9, 10, },
  { 3, 5, 6, 9, 10, },
  { 0, 3, 5, 6, 9, 10, },
  { 1, 3, 5, 6, 9, 10, },
  { 0, 1, 3, 5, 6, 9, 10, },
  { 2, 3, 5, 6, 9, 10, },
  { 0, 2, 3, 5, 6, 9, 10, },
  { 1, 2, 3, 5, 6, 9, 10, },
  { 0, 1, 2, 3, 5, 6, 9, 10, },
  { 4, 5, 6, 9, 10, },
  { 0, 4, 5, 6, 9, 10, },
  { 1, 4, 5, 6, 9, 10, },
  { 0, 1, 4, 5, 6, 9, 10, },
  { 2, 4, 5, 6, 9, 10, },
  { 0, 2, 4, 5, 6, 9, 10, },
  { 1, 2, 4, 5, 6, 9, 10, },
  { 0, 1, 2, 4, 5, 6, 9, 10, },
  { 3, 4, 5, 6, 9, 10, },
  { 0, 3, 4, 5, 6, 9, 10, },
  { 1, 3, 4, 5, 6, 9, 10, },
  { 0, 1, 3, 4, 5, 6, 9, 10, },
  { 2, 3, 4, 5, 6, 9, 10, },
  { 0, 2, 3, 4, 5, 6, 9, 10, },
  { 1, 2, 3, 4, 5, 6, 9, 10, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, },
  { 7, 9, 10, },
  { 0, 7, 9, 10, },
  { 1, 7, 9, 10, },
  { 0, 1, 7, 9, 10, },
  { 2, 7, 9, 10, },
  { 0, 2, 7, 9, 10, },
  { 1, 2, 7, 9, 10, },
  { 0, 1, 2, 7, 9, 10, },
  { 3, 7, 9, 10, },
  { 0, 3, 7, 9, 10, },
  { 1, 3, 7, 9, 10, },
  { 0, 1, 3, 7, 9, 10, },
  { 2, 3, 7, 9, 10, },
  { 0, 2, 3, 7, 9, 10, },
  { 1, 2, 3, 7, 9, 10, },
  { 0, 1, 2, 3, 7, 9, 10, },
  { 4, 7, 9, 10, },
  { 0, 4, 7, 9, 10, },
  { 1, 4, 7, 9, 10, },
  { 0, 1, 4, 7, 9, 10, },
  { 2, 4, 7, 9, 10, },
  { 0, 2, 4, 7, 9, 10, },
  { 1, 2, 4, 7, 9, 10, },
  { 0, 1, 2, 4, 7, 9, 10, },
  { 3, 4, 7, 9, 10, },
  { 0, 3, 4, 7, 9, 10, },
  { 1, 3, 4, 7, 9, 10, },
  { 0, 1, 3, 4, 7, 9, 10, },
  { 2, 3, 4, 7, 9, 10, },
  { 0, 2, 3, 4, 7, 9, 10, },
  { 1, 2, 3, 4, 7, 9, 10, },
  { 0, 1, 2, 3, 4, 7, 9, 10, },
  { 5, 7, 9, 10, },
  { 0, 5, 7, 9, 10, },
  { 1, 5, 7, 9, 10, },
  { 0, 1, 5, 7, 9, 10, },
  { 2, 5, 7, 9, 10, },
  { 0, 2, 5, 7, 9, 10, },
  { 1, 2, 5, 7, 9, 10, },
  { 0, 1, 2, 5, 7, 9, 10, },
  { 3, 5, 7, 9, 10, },
  { 0, 3, 5, 7, 9, 10, },
  { 1, 3, 5, 7, 9, 10, },
  { 0, 1, 3, 5, 7, 9, 10, },
  { 2, 3, 5, 7, 9, 10, },
  { 0, 2, 3, 5, 7, 9, 10, },
  { 1, 2, 3, 5, 7, 9, 10, },
  { 0, 1, 2, 3, 5, 7, 9, 10, },
  { 4, 5, 7, 9, 10, },
  { 0, 4, 5, 7, 9, 10, },
  { 1, 4, 5, 7, 9, 10, },
  { 0, 1, 4, 5, 7, 9, 10, },
  { 2, 4, 5, 7, 9, 10, },
  { 0, 2, 4, 5, 7, 9, 10, },
  { 1, 2, 4, 5, 7, 9, 10, },
  { 0, 1, 2, 4, 5, 7, 9, 10, },
  { 3, 4, 5, 7, 9, 10, },
  { 0, 3, 4, 5, 7, 9, 10, },
  { 1, 3, 4, 5, 7, 9, 10, },
  { 0, 1, 3, 4, 5, 7, 9, 10, },
  { 2, 3, 4, 5, 7, 9, 10, },
  { 0, 2, 3, 4, 5, 7, 9, 10, },
  { 1, 2, 3, 4, 5, 7, 9, 10, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, },
  { 6, 7, 9, 10, },
  { 0, 6, 7, 9, 10, },
  { 1, 6, 7, 9, 10, },
  { 0, 1, 6, 7, 9, 10, },
  { 2, 6, 7, 9, 10, },
  { 0, 2, 6, 7, 9, 10, },
  { 1, 2, 6, 7, 9, 10, },
  { 0, 1, 2, 6, 7, 9, 10, },
  { 3, 6, 7, 9, 10, },
  { 0, 3, 6, 7, 9, 10, },
  { 1, 3, 6, 7, 9, 10, },
  { 0, 1, 3, 6, 7, 9, 10, },
  { 2, 3, 6, 7, 9, 10, },
  { 0, 2, 3, 6, 7, 9, 10, },
  { 1, 2, 3, 6, 7, 9, 10, },
  { 0, 1, 2, 3, 6, 7, 9, 10, },
  { 4, 6, 7, 9, 10, },
  { 0, 4, 6, 7, 9, 10, },
  { 1, 4, 6, 7, 9, 10, },
  { 0, 1, 4, 6, 7, 9, 10, },
  { 2, 4, 6, 7, 9, 10, },
  { 0, 2, 4, 6, 7, 9, 10, },
  { 1, 2, 4, 6, 7, 9, 10, },
  { 0, 1, 2, 4, 6, 7, 9, 10, },
  { 3, 4, 6, 7, 9, 10, },
  { 0, 3, 4, 6, 7, 9, 10, },
  { 1, 3, 4, 6, 7, 9, 10, },
  { 0, 1, 3, 4, 6, 7, 9, 10, },
  { 2, 3, 4, 6, 7, 9, 10, },
  { 0, 2, 3, 4, 6, 7, 9, 10, },
  { 1, 2, 3, 4, 6, 7, 9, 10, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, },
  { 5, 6, 7, 9, 10, },
  { 0, 5, 6, 7, 9, 10, },
  { 1, 5, 6, 7, 9, 10, },
  { 0, 1, 5, 6, 7, 9, 10, },
  { 2, 5, 6, 7, 9, 10, },
  { 0, 2, 5, 6, 7, 9, 10, },
  { 1, 2, 5, 6, 7, 9, 10, },
  { 0, 1, 2, 5, 6, 7, 9, 10, },
  { 3, 5, 6, 7, 9, 10, },
  { 0, 3, 5, 6, 7, 9, 10, },
  { 1, 3, 5, 6, 7, 9, 10, },
  { 0, 1, 3, 5, 6, 7, 9, 10, },
  { 2, 3, 5, 6, 7, 9, 10, },
  { 0, 2, 3, 5, 6, 7, 9, 10, },
  { 1, 2, 3, 5, 6, 7, 9, 10, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, },
  { 4, 5, 6, 7, 9, 10, },
  { 0, 4, 5, 6, 7, 9, 10, },
  { 1, 4, 5, 6, 7, 9, 10, },
  { 0, 1, 4, 5, 6, 7, 9, 10, },
  { 2, 4, 5, 6, 7, 9, 10, },
  { 0, 2, 4, 5, 6, 7, 9, 10, },
  { 1, 2, 4, 5, 6, 7, 9, 10, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, },
  { 3, 4, 5, 6, 7, 9, 10, },
  { 0, 3, 4, 5, 6, 7, 9, 10, },
  { 1, 3, 4, 5, 6, 7, 9, 10, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, },
  { 2, 3, 4, 5, 6, 7, 9, 10, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, },
  { 8, 9, 10, },
  { 0, 8, 9, 10, },
  { 1, 8, 9, 10, },
  { 0, 1, 8, 9, 10, },
  { 2, 8, 9, 10, },
  { 0, 2, 8, 9, 10, },
  { 1, 2, 8, 9, 10, },
  { 0, 1, 2, 8, 9, 10, },
  { 3, 8, 9, 10, },
  { 0, 3, 8, 9, 10, },
  { 1, 3, 8, 9, 10, },
  { 0, 1, 3, 8, 9, 10, },
  { 2, 3, 8, 9, 10, },
  { 0, 2, 3, 8, 9, 10, },
  { 1, 2, 3, 8, 9, 10, },
  { 0, 1, 2, 3, 8, 9, 10, },
  { 4, 8, 9, 10, },
  { 0, 4, 8, 9, 10, },
  { 1, 4, 8, 9, 10, },
  { 0, 1, 4, 8, 9, 10, },
  { 2, 4, 8, 9, 10, },
  { 0, 2, 4, 8, 9, 10, },
  { 1, 2, 4, 8, 9, 10, },
  { 0, 1, 2, 4, 8, 9, 10, },
  { 3, 4, 8, 9, 10, },
  { 0, 3, 4, 8, 9, 10, },
  { 1, 3, 4, 8, 9, 10, },
  { 0, 1, 3, 4, 8, 9, 10, },
  { 2, 3, 4, 8, 9, 10, },
  { 0, 2, 3, 4, 8, 9, 10, },
  { 1, 2, 3, 4, 8, 9, 10, },
  { 0, 1, 2, 3, 4, 8, 9, 10, },
  { 5, 8, 9, 10, },
  { 0, 5, 8, 9, 10, },
  { 1, 5, 8, 9, 10, },
  { 0, 1, 5, 8, 9, 10, },
  { 2, 5, 8, 9, 10, },
  { 0, 2, 5, 8, 9, 10, },
  { 1, 2, 5, 8, 9, 10, },
  { 0, 1, 2, 5, 8, 9, 10, },
  { 3, 5, 8, 9, 10, },
  { 0, 3, 5, 8, 9, 10, },
  { 1, 3, 5, 8, 9, 10, },
  { 0, 1, 3, 5, 8, 9, 10, },
  { 2, 3, 5, 8, 9, 10, },
  { 0, 2, 3, 5, 8, 9, 10, },
  { 1, 2, 3, 5, 8, 9, 10, },
  { 0, 1, 2, 3, 5, 8, 9, 10, },
  { 4, 5, 8, 9, 10, },
  { 0, 4, 5, 8, 9, 10, },
  { 1, 4, 5, 8, 9, 10, },
  { 0, 1, 4, 5, 8, 9, 10, },
  { 2, 4, 5, 8, 9, 10, },
  { 0, 2, 4, 5, 8, 9, 10, },
  { 1, 2, 4, 5, 8, 9, 10, },
  { 0, 1, 2, 4, 5, 8, 9, 10, },
  { 3, 4, 5, 8, 9, 10, },
  { 0, 3, 4, 5, 8, 9, 10, },
  { 1, 3, 4, 5, 8, 9, 10, },
  { 0, 1, 3, 4, 5, 8, 9, 10, },
  { 2, 3, 4, 5, 8, 9, 10, },
  { 0, 2, 3, 4, 5, 8, 9, 10, },
  { 1, 2, 3, 4, 5, 8, 9, 10, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, },
  { 6, 8, 9, 10, },
  { 0, 6, 8, 9, 10, },
  { 1, 6, 8, 9, 10, },
  { 0, 1, 6, 8, 9, 10, },
  { 2, 6, 8, 9, 10, },
  { 0, 2, 6, 8, 9, 10, },
  { 1, 2, 6, 8, 9, 10, },
  { 0, 1, 2, 6, 8, 9, 10, },
  { 3, 6, 8, 9, 10, },
  { 0, 3, 6, 8, 9, 10, },
  { 1, 3, 6, 8, 9, 10, },
  { 0, 1, 3, 6, 8, 9, 10, },
  { 2, 3, 6, 8, 9, 10, },
  { 0, 2, 3, 6, 8, 9, 10, },
  { 1, 2, 3, 6, 8, 9, 10, },
  { 0, 1, 2, 3, 6, 8, 9, 10, },
  { 4, 6, 8, 9, 10, },
  { 0, 4, 6, 8, 9, 10, },
  { 1, 4, 6, 8, 9, 10, },
  { 0, 1, 4, 6, 8, 9, 10, },
  { 2, 4, 6, 8, 9, 10, },
  { 0, 2, 4, 6, 8, 9, 10, },
  { 1, 2, 4, 6, 8, 9, 10, },
  { 0, 1, 2, 4, 6, 8, 9, 10, },
  { 3, 4, 6, 8, 9, 10, },
  { 0, 3, 4, 6, 8, 9, 10, },
  { 1, 3, 4, 6, 8, 9, 10, },
  { 0, 1, 3, 4, 6, 8, 9, 10, },
  { 2, 3, 4, 6, 8, 9, 10, },
  { 0, 2, 3, 4, 6, 8, 9, 10, },
  { 1, 2, 3, 4, 6, 8, 9, 10, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, },
  { 5, 6, 8, 9, 10, },
  { 0, 5, 6, 8, 9, 10, },
  { 1, 5, 6, 8, 9, 10, },
  { 0, 1, 5, 6, 8, 9, 10, },
  { 2, 5, 6, 8, 9, 10, },
  { 0, 2, 5, 6, 8, 9, 10, },
  { 1, 2, 5, 6, 8, 9, 10, },
  { 0, 1, 2, 5, 6, 8, 9, 10, },
  { 3, 5, 6, 8, 9, 10, },
  { 0, 3, 5, 6, 8, 9, 10, },
  { 1, 3, 5, 6, 8, 9, 10, },
  { 0, 1, 3, 5, 6, 8, 9, 10, },
  { 2, 3, 5, 6, 8, 9, 10, },
  { 0, 2, 3, 5, 6, 8, 9, 10, },
  { 1, 2, 3, 5, 6, 8, 9, 10, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, },
  { 4, 5, 6, 8, 9, 10, },
  { 0, 4, 5, 6, 8, 9, 10, },
  { 1, 4, 5, 6, 8, 9, 10, },
  { 0, 1, 4, 5, 6, 8, 9, 10, },
  { 2, 4, 5, 6, 8, 9, 10, },
  { 0, 2, 4, 5, 6, 8, 9, 10, },
  { 1, 2, 4, 5, 6, 8, 9, 10, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, },
  { 3, 4, 5, 6, 8, 9, 10, },
  { 0, 3, 4, 5, 6, 8, 9, 10, },
  { 1, 3, 4, 5, 6, 8, 9, 10, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, },
  { 2, 3, 4, 5, 6, 8, 9, 10, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, },
  { 7, 8, 9, 10, },
  { 0, 7, 8, 9, 10, },
  { 1, 7, 8, 9, 10, },
  { 0, 1, 7, 8, 9, 10, },
  { 2, 7, 8, 9, 10, },
  { 0, 2, 7, 8, 9, 10, },
  { 1, 2, 7, 8, 9, 10, },
  { 0, 1, 2, 7, 8, 9, 10, },
  { 3, 7, 8, 9, 10, },
  { 0, 3, 7, 8, 9, 10, },
  { 1, 3, 7, 8, 9, 10, },
  { 0, 1, 3, 7, 8, 9, 10, },
  { 2, 3, 7, 8, 9, 10, },
  { 0, 2, 3, 7, 8, 9, 10, },
  { 1, 2, 3, 7, 8, 9, 10, },
  { 0, 1, 2, 3, 7, 8, 9, 10, },
  { 4, 7, 8, 9, 10, },
  { 0, 4, 7, 8, 9, 10, },
  { 1, 4, 7, 8, 9, 10, },
  { 0, 1, 4, 7, 8, 9, 10, },
  { 2, 4, 7, 8, 9, 10, },
  { 0, 2, 4, 7, 8, 9, 10, },
  { 1, 2, 4, 7, 8, 9, 10, },
  { 0, 1, 2, 4, 7, 8, 9, 10, },
  { 3, 4, 7, 8, 9, 10, },
  { 0, 3, 4, 7, 8, 9, 10, },
  { 1, 3, 4, 7, 8, 9, 10, },
  { 0, 1, 3, 4, 7, 8, 9, 10, },
  { 2, 3, 4, 7, 8, 9, 10, },
  { 0, 2, 3, 4, 7, 8, 9, 10, },
  { 1, 2, 3, 4, 7, 8, 9, 10, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, },
  { 5, 7, 8, 9, 10, },
  { 0, 5, 7, 8, 9, 10, },
  { 1, 5, 7, 8, 9, 10, },
  { 0, 1, 5, 7, 8, 9, 10, },
  { 2, 5, 7, 8, 9, 10, },
  { 0, 2, 5, 7, 8, 9, 10, },
  { 1, 2, 5, 7, 8, 9, 10, },
  { 0, 1, 2, 5, 7, 8, 9, 10, },
  { 3, 5, 7, 8, 9, 10, },
  { 0, 3, 5, 7, 8, 9, 10, },
  { 1, 3, 5, 7, 8, 9, 10, },
  { 0, 1, 3, 5, 7, 8, 9, 10, },
  { 2, 3, 5, 7, 8, 9, 10, },
  { 0, 2, 3, 5, 7, 8, 9, 10, },
  { 1, 2, 3, 5, 7, 8, 9, 10, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, },
  { 4, 5, 7, 8, 9, 10, },
  { 0, 4, 5, 7, 8, 9, 10, },
  { 1, 4, 5, 7, 8, 9, 10, },
  { 0, 1, 4, 5, 7, 8, 9, 10, },
  { 2, 4, 5, 7, 8, 9, 10, },
  { 0, 2, 4, 5, 7, 8, 9, 10, },
  { 1, 2, 4, 5, 7, 8, 9, 10, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, },
  { 3, 4, 5, 7, 8, 9, 10, },
  { 0, 3, 4, 5, 7, 8, 9, 10, },
  { 1, 3, 4, 5, 7, 8, 9, 10, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, },
  { 2, 3, 4, 5, 7, 8, 9, 10, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, },
  { 6, 7, 8, 9, 10, },
  { 0, 6, 7, 8, 9, 10, },
  { 1, 6, 7, 8, 9, 10, },
  { 0, 1, 6, 7, 8, 9, 10, },
  { 2, 6, 7, 8, 9, 10, },
  { 0, 2, 6, 7, 8, 9, 10, },
  { 1, 2, 6, 7, 8, 9, 10, },
  { 0, 1, 2, 6, 7, 8, 9, 10, },
  { 3, 6, 7, 8, 9, 10, },
  { 0, 3, 6, 7, 8, 9, 10, },
  { 1, 3, 6, 7, 8, 9, 10, },
  { 0, 1, 3, 6, 7, 8, 9, 10, },
  { 2, 3, 6, 7, 8, 9, 10, },
  { 0, 2, 3, 6, 7, 8, 9, 10, },
  { 1, 2, 3, 6, 7, 8, 9, 10, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, },
  { 4, 6, 7, 8, 9, 10, },
  { 0, 4, 6, 7, 8, 9, 10, },
  { 1, 4, 6, 7, 8, 9, 10, },
  { 0, 1, 4, 6, 7, 8, 9, 10, },
  { 2, 4, 6, 7, 8, 9, 10, },
  { 0, 2, 4, 6, 7, 8, 9, 10, },
  { 1, 2, 4, 6, 7, 8, 9, 10, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, },
  { 3, 4, 6, 7, 8, 9, 10, },
  { 0, 3, 4, 6, 7, 8, 9, 10, },
  { 1, 3, 4, 6, 7, 8, 9, 10, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, },
  { 2, 3, 4, 6, 7, 8, 9, 10, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, },
  { 5, 6, 7, 8, 9, 10, },
  { 0, 5, 6, 7, 8, 9, 10, },
  { 1, 5, 6, 7, 8, 9, 10, },
  { 0, 1, 5, 6, 7, 8, 9, 10, },
  { 2, 5, 6, 7, 8, 9, 10, },
  { 0, 2, 5, 6, 7, 8, 9, 10, },
  { 1, 2, 5, 6, 7, 8, 9, 10, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, },
  { 3, 5, 6, 7, 8, 9, 10, },
  { 0, 3, 5, 6, 7, 8, 9, 10, },
  { 1, 3, 5, 6, 7, 8, 9, 10, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, },
  { 2, 3, 5, 6, 7, 8, 9, 10, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, },
  { 4, 5, 6, 7, 8, 9, 10, },
  { 0, 4, 5, 6, 7, 8, 9, 10, },
  { 1, 4, 5, 6, 7, 8, 9, 10, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, },
  { 2, 4, 5, 6, 7, 8, 9, 10, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, },
  { 3, 4, 5, 6, 7, 8, 9, 10, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, },
  { 11, },
  { 0, 11, },
  { 1, 11, },
  { 0, 1, 11, },
  { 2, 11, },
  { 0, 2, 11, },
  { 1, 2, 11, },
  { 0, 1, 2, 11, },
  { 3, 11, },
  { 0, 3, 11, },
  { 1, 3, 11, },
  { 0, 1, 3, 11, },
  { 2, 3, 11, },
  { 0, 2, 3, 11, },
  { 1, 2, 3, 11, },
  { 0, 1, 2, 3, 11, },
  { 4, 11, },
  { 0, 4, 11, },
  { 1, 4, 11, },
  { 0, 1, 4, 11, },
  { 2, 4, 11, },
  { 0, 2, 4, 11, },
  { 1, 2, 4, 11, },
  { 0, 1, 2, 4, 11, },
  { 3, 4, 11, },
  { 0, 3, 4, 11, },
  { 1, 3, 4, 11, },
  { 0, 1, 3, 4, 11, },
  { 2, 3, 4, 11, },
  { 0, 2, 3, 4, 11, },
  { 1, 2, 3, 4, 11, },
  { 0, 1, 2, 3, 4, 11, },
  { 5, 11, },
  { 0, 5, 11, },
  { 1, 5, 11, },
  { 0, 1, 5, 11, },
  { 2, 5, 11, },
  { 0, 2, 5, 11, },
  { 1, 2, 5, 11, },
  { 0, 1, 2, 5, 11, },
  { 3, 5, 11, },
  { 0, 3, 5, 11, },
  { 1, 3, 5, 11, },
  { 0, 1, 3, 5, 11, },
  { 2, 3, 5, 11, },
  { 0, 2, 3, 5, 11, },
  { 1, 2, 3, 5, 11, },
  { 0, 1, 2, 3, 5, 11, },
  { 4, 5, 11, },
  { 0, 4, 5, 11, },
  { 1, 4, 5, 11, },
  { 0, 1, 4, 5, 11, },
  { 2, 4, 5, 11, },
  { 0, 2, 4, 5, 11, },
  { 1, 2, 4, 5, 11, },
  { 0, 1, 2, 4, 5, 11, },
  { 3, 4, 5, 11, },
  { 0, 3, 4, 5, 11, },
  { 1, 3, 4, 5, 11, },
  { 0, 1, 3, 4, 5, 11, },
  { 2, 3, 4, 5, 11, },
  { 0, 2, 3, 4, 5, 11, },
  { 1, 2, 3, 4, 5, 11, },
  { 0, 1, 2, 3, 4, 5, 11, },
  { 6, 11, },
  { 0, 6, 11, },
  { 1, 6, 11, },
  { 0, 1, 6, 11, },
  { 2, 6, 11, },
  { 0, 2, 6, 11, },
  { 1, 2, 6, 11, },
  { 0, 1, 2, 6, 11, },
  { 3, 6, 11, },
  { 0, 3, 6, 11, },
  { 1, 3, 6, 11, },
  { 0, 1, 3, 6, 11, },
  { 2, 3, 6, 11, },
  { 0, 2, 3, 6, 11, },
  { 1, 2, 3, 6, 11, },
  { 0, 1, 2, 3, 6, 11, },
  { 4, 6, 11, },
  { 0, 4, 6, 11, },
  { 1, 4, 6, 11, },
  { 0, 1, 4, 6, 11, },
  { 2, 4, 6, 11, },
  { 0, 2, 4, 6, 11, },
  { 1, 2, 4, 6, 11, },
  { 0, 1, 2, 4, 6, 11, },
  { 3, 4, 6, 11, },
  { 0, 3, 4, 6, 11, },
  { 1, 3, 4, 6, 11, },
  { 0, 1, 3, 4, 6, 11, },
  { 2, 3, 4, 6, 11, },
  { 0, 2, 3, 4, 6, 11, },
  { 1, 2, 3, 4, 6, 11, },
  { 0, 1, 2, 3, 4, 6, 11, },
  { 5, 6, 11, },
  { 0, 5, 6, 11, },
  { 1, 5, 6, 11, },
  { 0, 1, 5, 6, 11, },
  { 2, 5, 6, 11, },
  { 0, 2, 5, 6, 11, },
  { 1, 2, 5, 6, 11, },
  { 0, 1, 2, 5, 6, 11, },
  { 3, 5, 6, 11, },
  { 0, 3, 5, 6, 11, },
  { 1, 3, 5, 6, 11, },
  { 0, 1, 3, 5, 6, 11, },
  { 2, 3, 5, 6, 11, },
  { 0, 2, 3, 5, 6, 11, },
  { 1, 2, 3, 5, 6, 11, },
  { 0, 1, 2, 3, 5, 6, 11, },
  { 4, 5, 6, 11, },
  { 0, 4, 5, 6, 11, },
  { 1, 4, 5, 6, 11, },
  { 0, 1, 4, 5, 6, 11, },
  { 2, 4, 5, 6, 11, },
  { 0, 2, 4, 5, 6, 11, },
  { 1, 2, 4, 5, 6, 11, },
  { 0, 1, 2, 4, 5, 6, 11, },
  { 3, 4, 5, 6, 11, },
  { 0, 3, 4, 5, 6, 11, },
  { 1, 3, 4, 5, 6, 11, },
  { 0, 1, 3, 4, 5, 6, 11, },
  { 2, 3, 4, 5, 6, 11, },
  { 0, 2, 3, 4, 5, 6, 11, },
  { 1, 2, 3, 4, 5, 6, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 11, },
  { 7, 11, },
  { 0, 7, 11, },
  { 1, 7, 11, },
  { 0, 1, 7, 11, },
  { 2, 7, 11, },
  { 0, 2, 7, 11, },
  { 1, 2, 7, 11, },
  { 0, 1, 2, 7, 11, },
  { 3, 7, 11, },
  { 0, 3, 7, 11, },
  { 1, 3, 7, 11, },
  { 0, 1, 3, 7, 11, },
  { 2, 3, 7, 11, },
  { 0, 2, 3, 7, 11, },
  { 1, 2, 3, 7, 11, },
  { 0, 1, 2, 3, 7, 11, },
  { 4, 7, 11, },
  { 0, 4, 7, 11, },
  { 1, 4, 7, 11, },
  { 0, 1, 4, 7, 11, },
  { 2, 4, 7, 11, },
  { 0, 2, 4, 7, 11, },
  { 1, 2, 4, 7, 11, },
  { 0, 1, 2, 4, 7, 11, },
  { 3, 4, 7, 11, },
  { 0, 3, 4, 7, 11, },
  { 1, 3, 4, 7, 11, },
  { 0, 1, 3, 4, 7, 11, },
  { 2, 3, 4, 7, 11, },
  { 0, 2, 3, 4, 7, 11, },
  { 1, 2, 3, 4, 7, 11, },
  { 0, 1, 2, 3, 4, 7, 11, },
  { 5, 7, 11, },
  { 0, 5, 7, 11, },
  { 1, 5, 7, 11, },
  { 0, 1, 5, 7, 11, },
  { 2, 5, 7, 11, },
  { 0, 2, 5, 7, 11, },
  { 1, 2, 5, 7, 11, },
  { 0, 1, 2, 5, 7, 11, },
  { 3, 5, 7, 11, },
  { 0, 3, 5, 7, 11, },
  { 1, 3, 5, 7, 11, },
  { 0, 1, 3, 5, 7, 11, },
  { 2, 3, 5, 7, 11, },
  { 0, 2, 3, 5, 7, 11, },
  { 1, 2, 3, 5, 7, 11, },
  { 0, 1, 2, 3, 5, 7, 11, },
  { 4, 5, 7, 11, },
  { 0, 4, 5, 7, 11, },
  { 1, 4, 5, 7, 11, },
  { 0, 1, 4, 5, 7, 11, },
  { 2, 4, 5, 7, 11, },
  { 0, 2, 4, 5, 7, 11, },
  { 1, 2, 4, 5, 7, 11, },
  { 0, 1, 2, 4, 5, 7, 11, },
  { 3, 4, 5, 7, 11, },
  { 0, 3, 4, 5, 7, 11, },
  { 1, 3, 4, 5, 7, 11, },
  { 0, 1, 3, 4, 5, 7, 11, },
  { 2, 3, 4, 5, 7, 11, },
  { 0, 2, 3, 4, 5, 7, 11, },
  { 1, 2, 3, 4, 5, 7, 11, },
  { 0, 1, 2, 3, 4, 5, 7, 11, },
  { 6, 7, 11, },
  { 0, 6, 7, 11, },
  { 1, 6, 7, 11, },
  { 0, 1, 6, 7, 11, },
  { 2, 6, 7, 11, },
  { 0, 2, 6, 7, 11, },
  { 1, 2, 6, 7, 11, },
  { 0, 1, 2, 6, 7, 11, },
  { 3, 6, 7, 11, },
  { 0, 3, 6, 7, 11, },
  { 1, 3, 6, 7, 11, },
  { 0, 1, 3, 6, 7, 11, },
  { 2, 3, 6, 7, 11, },
  { 0, 2, 3, 6, 7, 11, },
  { 1, 2, 3, 6, 7, 11, },
  { 0, 1, 2, 3, 6, 7, 11, },
  { 4, 6, 7, 11, },
  { 0, 4, 6, 7, 11, },
  { 1, 4, 6, 7, 11, },
  { 0, 1, 4, 6, 7, 11, },
  { 2, 4, 6, 7, 11, },
  { 0, 2, 4, 6, 7, 11, },
  { 1, 2, 4, 6, 7, 11, },
  { 0, 1, 2, 4, 6, 7, 11, },
  { 3, 4, 6, 7, 11, },
  { 0, 3, 4, 6, 7, 11, },
  { 1, 3, 4, 6, 7, 11, },
  { 0, 1, 3, 4, 6, 7, 11, },
  { 2, 3, 4, 6, 7, 11, },
  { 0, 2, 3, 4, 6, 7, 11, },
  { 1, 2, 3, 4, 6, 7, 11, },
  { 0, 1, 2, 3, 4, 6, 7, 11, },
  { 5, 6, 7, 11, },
  { 0, 5, 6, 7, 11, },
  { 1, 5, 6, 7, 11, },
  { 0, 1, 5, 6, 7, 11, },
  { 2, 5, 6, 7, 11, },
  { 0, 2, 5, 6, 7, 11, },
  { 1, 2, 5, 6, 7, 11, },
  { 0, 1, 2, 5, 6, 7, 11, },
  { 3, 5, 6, 7, 11, },
  { 0, 3, 5, 6, 7, 11, },
  { 1, 3, 5, 6, 7, 11, },
  { 0, 1, 3, 5, 6, 7, 11, },
  { 2, 3, 5, 6, 7, 11, },
  { 0, 2, 3, 5, 6, 7, 11, },
  { 1, 2, 3, 5, 6, 7, 11, },
  { 0, 1, 2, 3, 5, 6, 7, 11, },
  { 4, 5, 6, 7, 11, },
  { 0, 4, 5, 6, 7, 11, },
  { 1, 4, 5, 6, 7, 11, },
  { 0, 1, 4, 5, 6, 7, 11, },
  { 2, 4, 5, 6, 7, 11, },
  { 0, 2, 4, 5, 6, 7, 11, },
  { 1, 2, 4, 5, 6, 7, 11, },
  { 0, 1, 2, 4, 5, 6, 7, 11, },
  { 3, 4, 5, 6, 7, 11, },
  { 0, 3, 4, 5, 6, 7, 11, },
  { 1, 3, 4, 5, 6, 7, 11, },
  { 0, 1, 3, 4, 5, 6, 7, 11, },
  { 2, 3, 4, 5, 6, 7, 11, },
  { 0, 2, 3, 4, 5, 6, 7, 11, },
  { 1, 2, 3, 4, 5, 6, 7, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, },
  { 8, 11, },
  { 0, 8, 11, },
  { 1, 8, 11, },
  { 0, 1, 8, 11, },
  { 2, 8, 11, },
  { 0, 2, 8, 11, },
  { 1, 2, 8, 11, },
  { 0, 1, 2, 8, 11, },
  { 3, 8, 11, },
  { 0, 3, 8, 11, },
  { 1, 3, 8, 11, },
  { 0, 1, 3, 8, 11, },
  { 2, 3, 8, 11, },
  { 0, 2, 3, 8, 11, },
  { 1, 2, 3, 8, 11, },
  { 0, 1, 2, 3, 8, 11, },
  { 4, 8, 11, },
  { 0, 4, 8, 11, },
  { 1, 4, 8, 11, },
  { 0, 1, 4, 8, 11, },
  { 2, 4, 8, 11, },
  { 0, 2, 4, 8, 11, },
  { 1, 2, 4, 8, 11, },
  { 0, 1, 2, 4, 8, 11, },
  { 3, 4, 8, 11, },
  { 0, 3, 4, 8, 11, },
  { 1, 3, 4, 8, 11, },
  { 0, 1, 3, 4, 8, 11, },
  { 2, 3, 4, 8, 11, },
  { 0, 2, 3, 4, 8, 11, },
  { 1, 2, 3, 4, 8, 11, },
  { 0, 1, 2, 3, 4, 8, 11, },
  { 5, 8, 11, },
  { 0, 5, 8, 11, },
  { 1, 5, 8, 11, },
  { 0, 1, 5, 8, 11, },
  { 2, 5, 8, 11, },
  { 0, 2, 5, 8, 11, },
  { 1, 2, 5, 8, 11, },
  { 0, 1, 2, 5, 8, 11, },
  { 3, 5, 8, 11, },
  { 0, 3, 5, 8, 11, },
  { 1, 3, 5, 8, 11, },
  { 0, 1, 3, 5, 8, 11, },
  { 2, 3, 5, 8, 11, },
  { 0, 2, 3, 5, 8, 11, },
  { 1, 2, 3, 5, 8, 11, },
  { 0, 1, 2, 3, 5, 8, 11, },
  { 4, 5, 8, 11, },
  { 0, 4, 5, 8, 11, },
  { 1, 4, 5, 8, 11, },
  { 0, 1, 4, 5, 8, 11, },
  { 2, 4, 5, 8, 11, },
  { 0, 2, 4, 5, 8, 11, },
  { 1, 2, 4, 5, 8, 11, },
  { 0, 1, 2, 4, 5, 8, 11, },
  { 3, 4, 5, 8, 11, },
  { 0, 3, 4, 5, 8, 11, },
  { 1, 3, 4, 5, 8, 11, },
  { 0, 1, 3, 4, 5, 8, 11, },
  { 2, 3, 4, 5, 8, 11, },
  { 0, 2, 3, 4, 5, 8, 11, },
  { 1, 2, 3, 4, 5, 8, 11, },
  { 0, 1, 2, 3, 4, 5, 8, 11, },
  { 6, 8, 11, },
  { 0, 6, 8, 11, },
  { 1, 6, 8, 11, },
  { 0, 1, 6, 8, 11, },
  { 2, 6, 8, 11, },
  { 0, 2, 6, 8, 11, },
  { 1, 2, 6, 8, 11, },
  { 0, 1, 2, 6, 8, 11, },
  { 3, 6, 8, 11, },
  { 0, 3, 6, 8, 11, },
  { 1, 3, 6, 8, 11, },
  { 0, 1, 3, 6, 8, 11, },
  { 2, 3, 6, 8, 11, },
  { 0, 2, 3, 6, 8, 11, },
  { 1, 2, 3, 6, 8, 11, },
  { 0, 1, 2, 3, 6, 8, 11, },
  { 4, 6, 8, 11, },
  { 0, 4, 6, 8, 11, },
  { 1, 4, 6, 8, 11, },
  { 0, 1, 4, 6, 8, 11, },
  { 2, 4, 6, 8, 11, },
  { 0, 2, 4, 6, 8, 11, },
  { 1, 2, 4, 6, 8, 11, },
  { 0, 1, 2, 4, 6, 8, 11, },
  { 3, 4, 6, 8, 11, },
  { 0, 3, 4, 6, 8, 11, },
  { 1, 3, 4, 6, 8, 11, },
  { 0, 1, 3, 4, 6, 8, 11, },
  { 2, 3, 4, 6, 8, 11, },
  { 0, 2, 3, 4, 6, 8, 11, },
  { 1, 2, 3, 4, 6, 8, 11, },
  { 0, 1, 2, 3, 4, 6, 8, 11, },
  { 5, 6, 8, 11, },
  { 0, 5, 6, 8, 11, },
  { 1, 5, 6, 8, 11, },
  { 0, 1, 5, 6, 8, 11, },
  { 2, 5, 6, 8, 11, },
  { 0, 2, 5, 6, 8, 11, },
  { 1, 2, 5, 6, 8, 11, },
  { 0, 1, 2, 5, 6, 8, 11, },
  { 3, 5, 6, 8, 11, },
  { 0, 3, 5, 6, 8, 11, },
  { 1, 3, 5, 6, 8, 11, },
  { 0, 1, 3, 5, 6, 8, 11, },
  { 2, 3, 5, 6, 8, 11, },
  { 0, 2, 3, 5, 6, 8, 11, },
  { 1, 2, 3, 5, 6, 8, 11, },
  { 0, 1, 2, 3, 5, 6, 8, 11, },
  { 4, 5, 6, 8, 11, },
  { 0, 4, 5, 6, 8, 11, },
  { 1, 4, 5, 6, 8, 11, },
  { 0, 1, 4, 5, 6, 8, 11, },
  { 2, 4, 5, 6, 8, 11, },
  { 0, 2, 4, 5, 6, 8, 11, },
  { 1, 2, 4, 5, 6, 8, 11, },
  { 0, 1, 2, 4, 5, 6, 8, 11, },
  { 3, 4, 5, 6, 8, 11, },
  { 0, 3, 4, 5, 6, 8, 11, },
  { 1, 3, 4, 5, 6, 8, 11, },
  { 0, 1, 3, 4, 5, 6, 8, 11, },
  { 2, 3, 4, 5, 6, 8, 11, },
  { 0, 2, 3, 4, 5, 6, 8, 11, },
  { 1, 2, 3, 4, 5, 6, 8, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, },
  { 7, 8, 11, },
  { 0, 7, 8, 11, },
  { 1, 7, 8, 11, },
  { 0, 1, 7, 8, 11, },
  { 2, 7, 8, 11, },
  { 0, 2, 7, 8, 11, },
  { 1, 2, 7, 8, 11, },
  { 0, 1, 2, 7, 8, 11, },
  { 3, 7, 8, 11, },
  { 0, 3, 7, 8, 11, },
  { 1, 3, 7, 8, 11, },
  { 0, 1, 3, 7, 8, 11, },
  { 2, 3, 7, 8, 11, },
  { 0, 2, 3, 7, 8, 11, },
  { 1, 2, 3, 7, 8, 11, },
  { 0, 1, 2, 3, 7, 8, 11, },
  { 4, 7, 8, 11, },
  { 0, 4, 7, 8, 11, },
  { 1, 4, 7, 8, 11, },
  { 0, 1, 4, 7, 8, 11, },
  { 2, 4, 7, 8, 11, },
  { 0, 2, 4, 7, 8, 11, },
  { 1, 2, 4, 7, 8, 11, },
  { 0, 1, 2, 4, 7, 8, 11, },
  { 3, 4, 7, 8, 11, },
  { 0, 3, 4, 7, 8, 11, },
  { 1, 3, 4, 7, 8, 11, },
  { 0, 1, 3, 4, 7, 8, 11, },
  { 2, 3, 4, 7, 8, 11, },
  { 0, 2, 3, 4, 7, 8, 11, },
  { 1, 2, 3, 4, 7, 8, 11, },
  { 0, 1, 2, 3, 4, 7, 8, 11, },
  { 5, 7, 8, 11, },
  { 0, 5, 7, 8, 11, },
  { 1, 5, 7, 8, 11, },
  { 0, 1, 5, 7, 8, 11, },
  { 2, 5, 7, 8, 11, },
  { 0, 2, 5, 7, 8, 11, },
  { 1, 2, 5, 7, 8, 11, },
  { 0, 1, 2, 5, 7, 8, 11, },
  { 3, 5, 7, 8, 11, },
  { 0, 3, 5, 7, 8, 11, },
  { 1, 3, 5, 7, 8, 11, },
  { 0, 1, 3, 5, 7, 8, 11, },
  { 2, 3, 5, 7, 8, 11, },
  { 0, 2, 3, 5, 7, 8, 11, },
  { 1, 2, 3, 5, 7, 8, 11, },
  { 0, 1, 2, 3, 5, 7, 8, 11, },
  { 4, 5, 7, 8, 11, },
  { 0, 4, 5, 7, 8, 11, },
  { 1, 4, 5, 7, 8, 11, },
  { 0, 1, 4, 5, 7, 8, 11, },
  { 2, 4, 5, 7, 8, 11, },
  { 0, 2, 4, 5, 7, 8, 11, },
  { 1, 2, 4, 5, 7, 8, 11, },
  { 0, 1, 2, 4, 5, 7, 8, 11, },
  { 3, 4, 5, 7, 8, 11, },
  { 0, 3, 4, 5, 7, 8, 11, },
  { 1, 3, 4, 5, 7, 8, 11, },
  { 0, 1, 3, 4, 5, 7, 8, 11, },
  { 2, 3, 4, 5, 7, 8, 11, },
  { 0, 2, 3, 4, 5, 7, 8, 11, },
  { 1, 2, 3, 4, 5, 7, 8, 11, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, },
  { 6, 7, 8, 11, },
  { 0, 6, 7, 8, 11, },
  { 1, 6, 7, 8, 11, },
  { 0, 1, 6, 7, 8, 11, },
  { 2, 6, 7, 8, 11, },
  { 0, 2, 6, 7, 8, 11, },
  { 1, 2, 6, 7, 8, 11, },
  { 0, 1, 2, 6, 7, 8, 11, },
  { 3, 6, 7, 8, 11, },
  { 0, 3, 6, 7, 8, 11, },
  { 1, 3, 6, 7, 8, 11, },
  { 0, 1, 3, 6, 7, 8, 11, },
  { 2, 3, 6, 7, 8, 11, },
  { 0, 2, 3, 6, 7, 8, 11, },
  { 1, 2, 3, 6, 7, 8, 11, },
  { 0, 1, 2, 3, 6, 7, 8, 11, },
  { 4, 6, 7, 8, 11, },
  { 0, 4, 6, 7, 8, 11, },
  { 1, 4, 6, 7, 8, 11, },
  { 0, 1, 4, 6, 7, 8, 11, },
  { 2, 4, 6, 7, 8, 11, },
  { 0, 2, 4, 6, 7, 8, 11, },
  { 1, 2, 4, 6, 7, 8, 11, },
  { 0, 1, 2, 4, 6, 7, 8, 11, },
  { 3, 4, 6, 7, 8, 11, },
  { 0, 3, 4, 6, 7, 8, 11, },
  { 1, 3, 4, 6, 7, 8, 11, },
  { 0, 1, 3, 4, 6, 7, 8, 11, },
  { 2, 3, 4, 6, 7, 8, 11, },
  { 0, 2, 3, 4, 6, 7, 8, 11, },
  { 1, 2, 3, 4, 6, 7, 8, 11, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, },
  { 5, 6, 7, 8, 11, },
  { 0, 5, 6, 7, 8, 11, },
  { 1, 5, 6, 7, 8, 11, },
  { 0, 1, 5, 6, 7, 8, 11, },
  { 2, 5, 6, 7, 8, 11, },
  { 0, 2, 5, 6, 7, 8, 11, },
  { 1, 2, 5, 6, 7, 8, 11, },
  { 0, 1, 2, 5, 6, 7, 8, 11, },
  { 3, 5, 6, 7, 8, 11, },
  { 0, 3, 5, 6, 7, 8, 11, },
  { 1, 3, 5, 6, 7, 8, 11, },
  { 0, 1, 3, 5, 6, 7, 8, 11, },
  { 2, 3, 5, 6, 7, 8, 11, },
  { 0, 2, 3, 5, 6, 7, 8, 11, },
  { 1, 2, 3, 5, 6, 7, 8, 11, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, },
  { 4, 5, 6, 7, 8, 11, },
  { 0, 4, 5, 6, 7, 8, 11, },
  { 1, 4, 5, 6, 7, 8, 11, },
  { 0, 1, 4, 5, 6, 7, 8, 11, },
  { 2, 4, 5, 6, 7, 8, 11, },
  { 0, 2, 4, 5, 6, 7, 8, 11, },
  { 1, 2, 4, 5, 6, 7, 8, 11, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, },
  { 3, 4, 5, 6, 7, 8, 11, },
  { 0, 3, 4, 5, 6, 7, 8, 11, },
  { 1, 3, 4, 5, 6, 7, 8, 11, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, },
  { 2, 3, 4, 5, 6, 7, 8, 11, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, },
  { 9, 11, },
  { 0, 9, 11, },
  { 1, 9, 11, },
  { 0, 1, 9, 11, },
  { 2, 9, 11, },
  { 0, 2, 9, 11, },
  { 1, 2, 9, 11, },
  { 0, 1, 2, 9, 11, },
  { 3, 9, 11, },
  { 0, 3, 9, 11, },
  { 1, 3, 9, 11, },
  { 0, 1, 3, 9, 11, },
  { 2, 3, 9, 11, },
  { 0, 2, 3, 9, 11, },
  { 1, 2, 3, 9, 11, },
  { 0, 1, 2, 3, 9, 11, },
  { 4, 9, 11, },
  { 0, 4, 9, 11, },
  { 1, 4, 9, 11, },
  { 0, 1, 4, 9, 11, },
  { 2, 4, 9, 11, },
  { 0, 2, 4, 9, 11, },
  { 1, 2, 4, 9, 11, },
  { 0, 1, 2, 4, 9, 11, },
  { 3, 4, 9, 11, },
  { 0, 3, 4, 9, 11, },
  { 1, 3, 4, 9, 11, },
  { 0, 1, 3, 4, 9, 11, },
  { 2, 3, 4, 9, 11, },
  { 0, 2, 3, 4, 9, 11, },
  { 1, 2, 3, 4, 9, 11, },
  { 0, 1, 2, 3, 4, 9, 11, },
  { 5, 9, 11, },
  { 0, 5, 9, 11, },
  { 1, 5, 9, 11, },
  { 0, 1, 5, 9, 11, },
  { 2, 5, 9, 11, },
  { 0, 2, 5, 9, 11, },
  { 1, 2, 5, 9, 11, },
  { 0, 1, 2, 5, 9, 11, },
  { 3, 5, 9, 11, },
  { 0, 3, 5, 9, 11, },
  { 1, 3, 5, 9, 11, },
  { 0, 1, 3, 5, 9, 11, },
  { 2, 3, 5, 9, 11, },
  { 0, 2, 3, 5, 9, 11, },
  { 1, 2, 3, 5, 9, 11, },
  { 0, 1, 2, 3, 5, 9, 11, },
  { 4, 5, 9, 11, },
  { 0, 4, 5, 9, 11, },
  { 1, 4, 5, 9, 11, },
  { 0, 1, 4, 5, 9, 11, },
  { 2, 4, 5, 9, 11, },
  { 0, 2, 4, 5, 9, 11, },
  { 1, 2, 4, 5, 9, 11, },
  { 0, 1, 2, 4, 5, 9, 11, },
  { 3, 4, 5, 9, 11, },
  { 0, 3, 4, 5, 9, 11, },
  { 1, 3, 4, 5, 9, 11, },
  { 0, 1, 3, 4, 5, 9, 11, },
  { 2, 3, 4, 5, 9, 11, },
  { 0, 2, 3, 4, 5, 9, 11, },
  { 1, 2, 3, 4, 5, 9, 11, },
  { 0, 1, 2, 3, 4, 5, 9, 11, },
  { 6, 9, 11, },
  { 0, 6, 9, 11, },
  { 1, 6, 9, 11, },
  { 0, 1, 6, 9, 11, },
  { 2, 6, 9, 11, },
  { 0, 2, 6, 9, 11, },
  { 1, 2, 6, 9, 11, },
  { 0, 1, 2, 6, 9, 11, },
  { 3, 6, 9, 11, },
  { 0, 3, 6, 9, 11, },
  { 1, 3, 6, 9, 11, },
  { 0, 1, 3, 6, 9, 11, },
  { 2, 3, 6, 9, 11, },
  { 0, 2, 3, 6, 9, 11, },
  { 1, 2, 3, 6, 9, 11, },
  { 0, 1, 2, 3, 6, 9, 11, },
  { 4, 6, 9, 11, },
  { 0, 4, 6, 9, 11, },
  { 1, 4, 6, 9, 11, },
  { 0, 1, 4, 6, 9, 11, },
  { 2, 4, 6, 9, 11, },
  { 0, 2, 4, 6, 9, 11, },
  { 1, 2, 4, 6, 9, 11, },
  { 0, 1, 2, 4, 6, 9, 11, },
  { 3, 4, 6, 9, 11, },
  { 0, 3, 4, 6, 9, 11, },
  { 1, 3, 4, 6, 9, 11, },
  { 0, 1, 3, 4, 6, 9, 11, },
  { 2, 3, 4, 6, 9, 11, },
  { 0, 2, 3, 4, 6, 9, 11, },
  { 1, 2, 3, 4, 6, 9, 11, },
  { 0, 1, 2, 3, 4, 6, 9, 11, },
  { 5, 6, 9, 11, },
  { 0, 5, 6, 9, 11, },
  { 1, 5, 6, 9, 11, },
  { 0, 1, 5, 6, 9, 11, },
  { 2, 5, 6, 9, 11, },
  { 0, 2, 5, 6, 9, 11, },
  { 1, 2, 5, 6, 9, 11, },
  { 0, 1, 2, 5, 6, 9, 11, },
  { 3, 5, 6, 9, 11, },
  { 0, 3, 5, 6, 9, 11, },
  { 1, 3, 5, 6, 9, 11, },
  { 0, 1, 3, 5, 6, 9, 11, },
  { 2, 3, 5, 6, 9, 11, },
  { 0, 2, 3, 5, 6, 9, 11, },
  { 1, 2, 3, 5, 6, 9, 11, },
  { 0, 1, 2, 3, 5, 6, 9, 11, },
  { 4, 5, 6, 9, 11, },
  { 0, 4, 5, 6, 9, 11, },
  { 1, 4, 5, 6, 9, 11, },
  { 0, 1, 4, 5, 6, 9, 11, },
  { 2, 4, 5, 6, 9, 11, },
  { 0, 2, 4, 5, 6, 9, 11, },
  { 1, 2, 4, 5, 6, 9, 11, },
  { 0, 1, 2, 4, 5, 6, 9, 11, },
  { 3, 4, 5, 6, 9, 11, },
  { 0, 3, 4, 5, 6, 9, 11, },
  { 1, 3, 4, 5, 6, 9, 11, },
  { 0, 1, 3, 4, 5, 6, 9, 11, },
  { 2, 3, 4, 5, 6, 9, 11, },
  { 0, 2, 3, 4, 5, 6, 9, 11, },
  { 1, 2, 3, 4, 5, 6, 9, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, },
  { 7, 9, 11, },
  { 0, 7, 9, 11, },
  { 1, 7, 9, 11, },
  { 0, 1, 7, 9, 11, },
  { 2, 7, 9, 11, },
  { 0, 2, 7, 9, 11, },
  { 1, 2, 7, 9, 11, },
  { 0, 1, 2, 7, 9, 11, },
  { 3, 7, 9, 11, },
  { 0, 3, 7, 9, 11, },
  { 1, 3, 7, 9, 11, },
  { 0, 1, 3, 7, 9, 11, },
  { 2, 3, 7, 9, 11, },
  { 0, 2, 3, 7, 9, 11, },
  { 1, 2, 3, 7, 9, 11, },
  { 0, 1, 2, 3, 7, 9, 11, },
  { 4, 7, 9, 11, },
  { 0, 4, 7, 9, 11, },
  { 1, 4, 7, 9, 11, },
  { 0, 1, 4, 7, 9, 11, },
  { 2, 4, 7, 9, 11, },
  { 0, 2, 4, 7, 9, 11, },
  { 1, 2, 4, 7, 9, 11, },
  { 0, 1, 2, 4, 7, 9, 11, },
  { 3, 4, 7, 9, 11, },
  { 0, 3, 4, 7, 9, 11, },
  { 1, 3, 4, 7, 9, 11, },
  { 0, 1, 3, 4, 7, 9, 11, },
  { 2, 3, 4, 7, 9, 11, },
  { 0, 2, 3, 4, 7, 9, 11, },
  { 1, 2, 3, 4, 7, 9, 11, },
  { 0, 1, 2, 3, 4, 7, 9, 11, },
  { 5, 7, 9, 11, },
  { 0, 5, 7, 9, 11, },
  { 1, 5, 7, 9, 11, },
  { 0, 1, 5, 7, 9, 11, },
  { 2, 5, 7, 9, 11, },
  { 0, 2, 5, 7, 9, 11, },
  { 1, 2, 5, 7, 9, 11, },
  { 0, 1, 2, 5, 7, 9, 11, },
  { 3, 5, 7, 9, 11, },
  { 0, 3, 5, 7, 9, 11, },
  { 1, 3, 5, 7, 9, 11, },
  { 0, 1, 3, 5, 7, 9, 11, },
  { 2, 3, 5, 7, 9, 11, },
  { 0, 2, 3, 5, 7, 9, 11, },
  { 1, 2, 3, 5, 7, 9, 11, },
  { 0, 1, 2, 3, 5, 7, 9, 11, },
  { 4, 5, 7, 9, 11, },
  { 0, 4, 5, 7, 9, 11, },
  { 1, 4, 5, 7, 9, 11, },
  { 0, 1, 4, 5, 7, 9, 11, },
  { 2, 4, 5, 7, 9, 11, },
  { 0, 2, 4, 5, 7, 9, 11, },
  { 1, 2, 4, 5, 7, 9, 11, },
  { 0, 1, 2, 4, 5, 7, 9, 11, },
  { 3, 4, 5, 7, 9, 11, },
  { 0, 3, 4, 5, 7, 9, 11, },
  { 1, 3, 4, 5, 7, 9, 11, },
  { 0, 1, 3, 4, 5, 7, 9, 11, },
  { 2, 3, 4, 5, 7, 9, 11, },
  { 0, 2, 3, 4, 5, 7, 9, 11, },
  { 1, 2, 3, 4, 5, 7, 9, 11, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, },
  { 6, 7, 9, 11, },
  { 0, 6, 7, 9, 11, },
  { 1, 6, 7, 9, 11, },
  { 0, 1, 6, 7, 9, 11, },
  { 2, 6, 7, 9, 11, },
  { 0, 2, 6, 7, 9, 11, },
  { 1, 2, 6, 7, 9, 11, },
  { 0, 1, 2, 6, 7, 9, 11, },
  { 3, 6, 7, 9, 11, },
  { 0, 3, 6, 7, 9, 11, },
  { 1, 3, 6, 7, 9, 11, },
  { 0, 1, 3, 6, 7, 9, 11, },
  { 2, 3, 6, 7, 9, 11, },
  { 0, 2, 3, 6, 7, 9, 11, },
  { 1, 2, 3, 6, 7, 9, 11, },
  { 0, 1, 2, 3, 6, 7, 9, 11, },
  { 4, 6, 7, 9, 11, },
  { 0, 4, 6, 7, 9, 11, },
  { 1, 4, 6, 7, 9, 11, },
  { 0, 1, 4, 6, 7, 9, 11, },
  { 2, 4, 6, 7, 9, 11, },
  { 0, 2, 4, 6, 7, 9, 11, },
  { 1, 2, 4, 6, 7, 9, 11, },
  { 0, 1, 2, 4, 6, 7, 9, 11, },
  { 3, 4, 6, 7, 9, 11, },
  { 0, 3, 4, 6, 7, 9, 11, },
  { 1, 3, 4, 6, 7, 9, 11, },
  { 0, 1, 3, 4, 6, 7, 9, 11, },
  { 2, 3, 4, 6, 7, 9, 11, },
  { 0, 2, 3, 4, 6, 7, 9, 11, },
  { 1, 2, 3, 4, 6, 7, 9, 11, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, },
  { 5, 6, 7, 9, 11, },
  { 0, 5, 6, 7, 9, 11, },
  { 1, 5, 6, 7, 9, 11, },
  { 0, 1, 5, 6, 7, 9, 11, },
  { 2, 5, 6, 7, 9, 11, },
  { 0, 2, 5, 6, 7, 9, 11, },
  { 1, 2, 5, 6, 7, 9, 11, },
  { 0, 1, 2, 5, 6, 7, 9, 11, },
  { 3, 5, 6, 7, 9, 11, },
  { 0, 3, 5, 6, 7, 9, 11, },
  { 1, 3, 5, 6, 7, 9, 11, },
  { 0, 1, 3, 5, 6, 7, 9, 11, },
  { 2, 3, 5, 6, 7, 9, 11, },
  { 0, 2, 3, 5, 6, 7, 9, 11, },
  { 1, 2, 3, 5, 6, 7, 9, 11, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, },
  { 4, 5, 6, 7, 9, 11, },
  { 0, 4, 5, 6, 7, 9, 11, },
  { 1, 4, 5, 6, 7, 9, 11, },
  { 0, 1, 4, 5, 6, 7, 9, 11, },
  { 2, 4, 5, 6, 7, 9, 11, },
  { 0, 2, 4, 5, 6, 7, 9, 11, },
  { 1, 2, 4, 5, 6, 7, 9, 11, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, },
  { 3, 4, 5, 6, 7, 9, 11, },
  { 0, 3, 4, 5, 6, 7, 9, 11, },
  { 1, 3, 4, 5, 6, 7, 9, 11, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, },
  { 2, 3, 4, 5, 6, 7, 9, 11, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, },
  { 8, 9, 11, },
  { 0, 8, 9, 11, },
  { 1, 8, 9, 11, },
  { 0, 1, 8, 9, 11, },
  { 2, 8, 9, 11, },
  { 0, 2, 8, 9, 11, },
  { 1, 2, 8, 9, 11, },
  { 0, 1, 2, 8, 9, 11, },
  { 3, 8, 9, 11, },
  { 0, 3, 8, 9, 11, },
  { 1, 3, 8, 9, 11, },
  { 0, 1, 3, 8, 9, 11, },
  { 2, 3, 8, 9, 11, },
  { 0, 2, 3, 8, 9, 11, },
  { 1, 2, 3, 8, 9, 11, },
  { 0, 1, 2, 3, 8, 9, 11, },
  { 4, 8, 9, 11, },
  { 0, 4, 8, 9, 11, },
  { 1, 4, 8, 9, 11, },
  { 0, 1, 4, 8, 9, 11, },
  { 2, 4, 8, 9, 11, },
  { 0, 2, 4, 8, 9, 11, },
  { 1, 2, 4, 8, 9, 11, },
  { 0, 1, 2, 4, 8, 9, 11, },
  { 3, 4, 8, 9, 11, },
  { 0, 3, 4, 8, 9, 11, },
  { 1, 3, 4, 8, 9, 11, },
  { 0, 1, 3, 4, 8, 9, 11, },
  { 2, 3, 4, 8, 9, 11, },
  { 0, 2, 3, 4, 8, 9, 11, },
  { 1, 2, 3, 4, 8, 9, 11, },
  { 0, 1, 2, 3, 4, 8, 9, 11, },
  { 5, 8, 9, 11, },
  { 0, 5, 8, 9, 11, },
  { 1, 5, 8, 9, 11, },
  { 0, 1, 5, 8, 9, 11, },
  { 2, 5, 8, 9, 11, },
  { 0, 2, 5, 8, 9, 11, },
  { 1, 2, 5, 8, 9, 11, },
  { 0, 1, 2, 5, 8, 9, 11, },
  { 3, 5, 8, 9, 11, },
  { 0, 3, 5, 8, 9, 11, },
  { 1, 3, 5, 8, 9, 11, },
  { 0, 1, 3, 5, 8, 9, 11, },
  { 2, 3, 5, 8, 9, 11, },
  { 0, 2, 3, 5, 8, 9, 11, },
  { 1, 2, 3, 5, 8, 9, 11, },
  { 0, 1, 2, 3, 5, 8, 9, 11, },
  { 4, 5, 8, 9, 11, },
  { 0, 4, 5, 8, 9, 11, },
  { 1, 4, 5, 8, 9, 11, },
  { 0, 1, 4, 5, 8, 9, 11, },
  { 2, 4, 5, 8, 9, 11, },
  { 0, 2, 4, 5, 8, 9, 11, },
  { 1, 2, 4, 5, 8, 9, 11, },
  { 0, 1, 2, 4, 5, 8, 9, 11, },
  { 3, 4, 5, 8, 9, 11, },
  { 0, 3, 4, 5, 8, 9, 11, },
  { 1, 3, 4, 5, 8, 9, 11, },
  { 0, 1, 3, 4, 5, 8, 9, 11, },
  { 2, 3, 4, 5, 8, 9, 11, },
  { 0, 2, 3, 4, 5, 8, 9, 11, },
  { 1, 2, 3, 4, 5, 8, 9, 11, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, },
  { 6, 8, 9, 11, },
  { 0, 6, 8, 9, 11, },
  { 1, 6, 8, 9, 11, },
  { 0, 1, 6, 8, 9, 11, },
  { 2, 6, 8, 9, 11, },
  { 0, 2, 6, 8, 9, 11, },
  { 1, 2, 6, 8, 9, 11, },
  { 0, 1, 2, 6, 8, 9, 11, },
  { 3, 6, 8, 9, 11, },
  { 0, 3, 6, 8, 9, 11, },
  { 1, 3, 6, 8, 9, 11, },
  { 0, 1, 3, 6, 8, 9, 11, },
  { 2, 3, 6, 8, 9, 11, },
  { 0, 2, 3, 6, 8, 9, 11, },
  { 1, 2, 3, 6, 8, 9, 11, },
  { 0, 1, 2, 3, 6, 8, 9, 11, },
  { 4, 6, 8, 9, 11, },
  { 0, 4, 6, 8, 9, 11, },
  { 1, 4, 6, 8, 9, 11, },
  { 0, 1, 4, 6, 8, 9, 11, },
  { 2, 4, 6, 8, 9, 11, },
  { 0, 2, 4, 6, 8, 9, 11, },
  { 1, 2, 4, 6, 8, 9, 11, },
  { 0, 1, 2, 4, 6, 8, 9, 11, },
  { 3, 4, 6, 8, 9, 11, },
  { 0, 3, 4, 6, 8, 9, 11, },
  { 1, 3, 4, 6, 8, 9, 11, },
  { 0, 1, 3, 4, 6, 8, 9, 11, },
  { 2, 3, 4, 6, 8, 9, 11, },
  { 0, 2, 3, 4, 6, 8, 9, 11, },
  { 1, 2, 3, 4, 6, 8, 9, 11, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, },
  { 5, 6, 8, 9, 11, },
  { 0, 5, 6, 8, 9, 11, },
  { 1, 5, 6, 8, 9, 11, },
  { 0, 1, 5, 6, 8, 9, 11, },
  { 2, 5, 6, 8, 9, 11, },
  { 0, 2, 5, 6, 8, 9, 11, },
  { 1, 2, 5, 6, 8, 9, 11, },
  { 0, 1, 2, 5, 6, 8, 9, 11, },
  { 3, 5, 6, 8, 9, 11, },
  { 0, 3, 5, 6, 8, 9, 11, },
  { 1, 3, 5, 6, 8, 9, 11, },
  { 0, 1, 3, 5, 6, 8, 9, 11, },
  { 2, 3, 5, 6, 8, 9, 11, },
  { 0, 2, 3, 5, 6, 8, 9, 11, },
  { 1, 2, 3, 5, 6, 8, 9, 11, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, },
  { 4, 5, 6, 8, 9, 11, },
  { 0, 4, 5, 6, 8, 9, 11, },
  { 1, 4, 5, 6, 8, 9, 11, },
  { 0, 1, 4, 5, 6, 8, 9, 11, },
  { 2, 4, 5, 6, 8, 9, 11, },
  { 0, 2, 4, 5, 6, 8, 9, 11, },
  { 1, 2, 4, 5, 6, 8, 9, 11, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, },
  { 3, 4, 5, 6, 8, 9, 11, },
  { 0, 3, 4, 5, 6, 8, 9, 11, },
  { 1, 3, 4, 5, 6, 8, 9, 11, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, },
  { 2, 3, 4, 5, 6, 8, 9, 11, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, },
  { 7, 8, 9, 11, },
  { 0, 7, 8, 9, 11, },
  { 1, 7, 8, 9, 11, },
  { 0, 1, 7, 8, 9, 11, },
  { 2, 7, 8, 9, 11, },
  { 0, 2, 7, 8, 9, 11, },
  { 1, 2, 7, 8, 9, 11, },
  { 0, 1, 2, 7, 8, 9, 11, },
  { 3, 7, 8, 9, 11, },
  { 0, 3, 7, 8, 9, 11, },
  { 1, 3, 7, 8, 9, 11, },
  { 0, 1, 3, 7, 8, 9, 11, },
  { 2, 3, 7, 8, 9, 11, },
  { 0, 2, 3, 7, 8, 9, 11, },
  { 1, 2, 3, 7, 8, 9, 11, },
  { 0, 1, 2, 3, 7, 8, 9, 11, },
  { 4, 7, 8, 9, 11, },
  { 0, 4, 7, 8, 9, 11, },
  { 1, 4, 7, 8, 9, 11, },
  { 0, 1, 4, 7, 8, 9, 11, },
  { 2, 4, 7, 8, 9, 11, },
  { 0, 2, 4, 7, 8, 9, 11, },
  { 1, 2, 4, 7, 8, 9, 11, },
  { 0, 1, 2, 4, 7, 8, 9, 11, },
  { 3, 4, 7, 8, 9, 11, },
  { 0, 3, 4, 7, 8, 9, 11, },
  { 1, 3, 4, 7, 8, 9, 11, },
  { 0, 1, 3, 4, 7, 8, 9, 11, },
  { 2, 3, 4, 7, 8, 9, 11, },
  { 0, 2, 3, 4, 7, 8, 9, 11, },
  { 1, 2, 3, 4, 7, 8, 9, 11, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, },
  { 5, 7, 8, 9, 11, },
  { 0, 5, 7, 8, 9, 11, },
  { 1, 5, 7, 8, 9, 11, },
  { 0, 1, 5, 7, 8, 9, 11, },
  { 2, 5, 7, 8, 9, 11, },
  { 0, 2, 5, 7, 8, 9, 11, },
  { 1, 2, 5, 7, 8, 9, 11, },
  { 0, 1, 2, 5, 7, 8, 9, 11, },
  { 3, 5, 7, 8, 9, 11, },
  { 0, 3, 5, 7, 8, 9, 11, },
  { 1, 3, 5, 7, 8, 9, 11, },
  { 0, 1, 3, 5, 7, 8, 9, 11, },
  { 2, 3, 5, 7, 8, 9, 11, },
  { 0, 2, 3, 5, 7, 8, 9, 11, },
  { 1, 2, 3, 5, 7, 8, 9, 11, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, },
  { 4, 5, 7, 8, 9, 11, },
  { 0, 4, 5, 7, 8, 9, 11, },
  { 1, 4, 5, 7, 8, 9, 11, },
  { 0, 1, 4, 5, 7, 8, 9, 11, },
  { 2, 4, 5, 7, 8, 9, 11, },
  { 0, 2, 4, 5, 7, 8, 9, 11, },
  { 1, 2, 4, 5, 7, 8, 9, 11, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, },
  { 3, 4, 5, 7, 8, 9, 11, },
  { 0, 3, 4, 5, 7, 8, 9, 11, },
  { 1, 3, 4, 5, 7, 8, 9, 11, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, },
  { 2, 3, 4, 5, 7, 8, 9, 11, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, },
  { 6, 7, 8, 9, 11, },
  { 0, 6, 7, 8, 9, 11, },
  { 1, 6, 7, 8, 9, 11, },
  { 0, 1, 6, 7, 8, 9, 11, },
  { 2, 6, 7, 8, 9, 11, },
  { 0, 2, 6, 7, 8, 9, 11, },
  { 1, 2, 6, 7, 8, 9, 11, },
  { 0, 1, 2, 6, 7, 8, 9, 11, },
  { 3, 6, 7, 8, 9, 11, },
  { 0, 3, 6, 7, 8, 9, 11, },
  { 1, 3, 6, 7, 8, 9, 11, },
  { 0, 1, 3, 6, 7, 8, 9, 11, },
  { 2, 3, 6, 7, 8, 9, 11, },
  { 0, 2, 3, 6, 7, 8, 9, 11, },
  { 1, 2, 3, 6, 7, 8, 9, 11, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, },
  { 4, 6, 7, 8, 9, 11, },
  { 0, 4, 6, 7, 8, 9, 11, },
  { 1, 4, 6, 7, 8, 9, 11, },
  { 0, 1, 4, 6, 7, 8, 9, 11, },
  { 2, 4, 6, 7, 8, 9, 11, },
  { 0, 2, 4, 6, 7, 8, 9, 11, },
  { 1, 2, 4, 6, 7, 8, 9, 11, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, },
  { 3, 4, 6, 7, 8, 9, 11, },
  { 0, 3, 4, 6, 7, 8, 9, 11, },
  { 1, 3, 4, 6, 7, 8, 9, 11, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, },
  { 2, 3, 4, 6, 7, 8, 9, 11, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, },
  { 5, 6, 7, 8, 9, 11, },
  { 0, 5, 6, 7, 8, 9, 11, },
  { 1, 5, 6, 7, 8, 9, 11, },
  { 0, 1, 5, 6, 7, 8, 9, 11, },
  { 2, 5, 6, 7, 8, 9, 11, },
  { 0, 2, 5, 6, 7, 8, 9, 11, },
  { 1, 2, 5, 6, 7, 8, 9, 11, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, },
  { 3, 5, 6, 7, 8, 9, 11, },
  { 0, 3, 5, 6, 7, 8, 9, 11, },
  { 1, 3, 5, 6, 7, 8, 9, 11, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, },
  { 2, 3, 5, 6, 7, 8, 9, 11, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, },
  { 4, 5, 6, 7, 8, 9, 11, },
  { 0, 4, 5, 6, 7, 8, 9, 11, },
  { 1, 4, 5, 6, 7, 8, 9, 11, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, },
  { 2, 4, 5, 6, 7, 8, 9, 11, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, },
  { 3, 4, 5, 6, 7, 8, 9, 11, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, },
  { 10, 11, },
  { 0, 10, 11, },
  { 1, 10, 11, },
  { 0, 1, 10, 11, },
  { 2, 10, 11, },
  { 0, 2, 10, 11, },
  { 1, 2, 10, 11, },
  { 0, 1, 2, 10, 11, },
  { 3, 10, 11, },
  { 0, 3, 10, 11, },
  { 1, 3, 10, 11, },
  { 0, 1, 3, 10, 11, },
  { 2, 3, 10, 11, },
  { 0, 2, 3, 10, 11, },
  { 1, 2, 3, 10, 11, },
  { 0, 1, 2, 3, 10, 11, },
  { 4, 10, 11, },
  { 0, 4, 10, 11, },
  { 1, 4, 10, 11, },
  { 0, 1, 4, 10, 11, },
  { 2, 4, 10, 11, },
  { 0, 2, 4, 10, 11, },
  { 1, 2, 4, 10, 11, },
  { 0, 1, 2, 4, 10, 11, },
  { 3, 4, 10, 11, },
  { 0, 3, 4, 10, 11, },
  { 1, 3, 4, 10, 11, },
  { 0, 1, 3, 4, 10, 11, },
  { 2, 3, 4, 10, 11, },
  { 0, 2, 3, 4, 10, 11, },
  { 1, 2, 3, 4, 10, 11, },
  { 0, 1, 2, 3, 4, 10, 11, },
  { 5, 10, 11, },
  { 0, 5, 10, 11, },
  { 1, 5, 10, 11, },
  { 0, 1, 5, 10, 11, },
  { 2, 5, 10, 11, },
  { 0, 2, 5, 10, 11, },
  { 1, 2, 5, 10, 11, },
  { 0, 1, 2, 5, 10, 11, },
  { 3, 5, 10, 11, },
  { 0, 3, 5, 10, 11, },
  { 1, 3, 5, 10, 11, },
  { 0, 1, 3, 5, 10, 11, },
  { 2, 3, 5, 10, 11, },
  { 0, 2, 3, 5, 10, 11, },
  { 1, 2, 3, 5, 10, 11, },
  { 0, 1, 2, 3, 5, 10, 11, },
  { 4, 5, 10, 11, },
  { 0, 4, 5, 10, 11, },
  { 1, 4, 5, 10, 11, },
  { 0, 1, 4, 5, 10, 11, },
  { 2, 4, 5, 10, 11, },
  { 0, 2, 4, 5, 10, 11, },
  { 1, 2, 4, 5, 10, 11, },
  { 0, 1, 2, 4, 5, 10, 11, },
  { 3, 4, 5, 10, 11, },
  { 0, 3, 4, 5, 10, 11, },
  { 1, 3, 4, 5, 10, 11, },
  { 0, 1, 3, 4, 5, 10, 11, },
  { 2, 3, 4, 5, 10, 11, },
  { 0, 2, 3, 4, 5, 10, 11, },
  { 1, 2, 3, 4, 5, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 10, 11, },
  { 6, 10, 11, },
  { 0, 6, 10, 11, },
  { 1, 6, 10, 11, },
  { 0, 1, 6, 10, 11, },
  { 2, 6, 10, 11, },
  { 0, 2, 6, 10, 11, },
  { 1, 2, 6, 10, 11, },
  { 0, 1, 2, 6, 10, 11, },
  { 3, 6, 10, 11, },
  { 0, 3, 6, 10, 11, },
  { 1, 3, 6, 10, 11, },
  { 0, 1, 3, 6, 10, 11, },
  { 2, 3, 6, 10, 11, },
  { 0, 2, 3, 6, 10, 11, },
  { 1, 2, 3, 6, 10, 11, },
  { 0, 1, 2, 3, 6, 10, 11, },
  { 4, 6, 10, 11, },
  { 0, 4, 6, 10, 11, },
  { 1, 4, 6, 10, 11, },
  { 0, 1, 4, 6, 10, 11, },
  { 2, 4, 6, 10, 11, },
  { 0, 2, 4, 6, 10, 11, },
  { 1, 2, 4, 6, 10, 11, },
  { 0, 1, 2, 4, 6, 10, 11, },
  { 3, 4, 6, 10, 11, },
  { 0, 3, 4, 6, 10, 11, },
  { 1, 3, 4, 6, 10, 11, },
  { 0, 1, 3, 4, 6, 10, 11, },
  { 2, 3, 4, 6, 10, 11, },
  { 0, 2, 3, 4, 6, 10, 11, },
  { 1, 2, 3, 4, 6, 10, 11, },
  { 0, 1, 2, 3, 4, 6, 10, 11, },
  { 5, 6, 10, 11, },
  { 0, 5, 6, 10, 11, },
  { 1, 5, 6, 10, 11, },
  { 0, 1, 5, 6, 10, 11, },
  { 2, 5, 6, 10, 11, },
  { 0, 2, 5, 6, 10, 11, },
  { 1, 2, 5, 6, 10, 11, },
  { 0, 1, 2, 5, 6, 10, 11, },
  { 3, 5, 6, 10, 11, },
  { 0, 3, 5, 6, 10, 11, },
  { 1, 3, 5, 6, 10, 11, },
  { 0, 1, 3, 5, 6, 10, 11, },
  { 2, 3, 5, 6, 10, 11, },
  { 0, 2, 3, 5, 6, 10, 11, },
  { 1, 2, 3, 5, 6, 10, 11, },
  { 0, 1, 2, 3, 5, 6, 10, 11, },
  { 4, 5, 6, 10, 11, },
  { 0, 4, 5, 6, 10, 11, },
  { 1, 4, 5, 6, 10, 11, },
  { 0, 1, 4, 5, 6, 10, 11, },
  { 2, 4, 5, 6, 10, 11, },
  { 0, 2, 4, 5, 6, 10, 11, },
  { 1, 2, 4, 5, 6, 10, 11, },
  { 0, 1, 2, 4, 5, 6, 10, 11, },
  { 3, 4, 5, 6, 10, 11, },
  { 0, 3, 4, 5, 6, 10, 11, },
  { 1, 3, 4, 5, 6, 10, 11, },
  { 0, 1, 3, 4, 5, 6, 10, 11, },
  { 2, 3, 4, 5, 6, 10, 11, },
  { 0, 2, 3, 4, 5, 6, 10, 11, },
  { 1, 2, 3, 4, 5, 6, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, },
  { 7, 10, 11, },
  { 0, 7, 10, 11, },
  { 1, 7, 10, 11, },
  { 0, 1, 7, 10, 11, },
  { 2, 7, 10, 11, },
  { 0, 2, 7, 10, 11, },
  { 1, 2, 7, 10, 11, },
  { 0, 1, 2, 7, 10, 11, },
  { 3, 7, 10, 11, },
  { 0, 3, 7, 10, 11, },
  { 1, 3, 7, 10, 11, },
  { 0, 1, 3, 7, 10, 11, },
  { 2, 3, 7, 10, 11, },
  { 0, 2, 3, 7, 10, 11, },
  { 1, 2, 3, 7, 10, 11, },
  { 0, 1, 2, 3, 7, 10, 11, },
  { 4, 7, 10, 11, },
  { 0, 4, 7, 10, 11, },
  { 1, 4, 7, 10, 11, },
  { 0, 1, 4, 7, 10, 11, },
  { 2, 4, 7, 10, 11, },
  { 0, 2, 4, 7, 10, 11, },
  { 1, 2, 4, 7, 10, 11, },
  { 0, 1, 2, 4, 7, 10, 11, },
  { 3, 4, 7, 10, 11, },
  { 0, 3, 4, 7, 10, 11, },
  { 1, 3, 4, 7, 10, 11, },
  { 0, 1, 3, 4, 7, 10, 11, },
  { 2, 3, 4, 7, 10, 11, },
  { 0, 2, 3, 4, 7, 10, 11, },
  { 1, 2, 3, 4, 7, 10, 11, },
  { 0, 1, 2, 3, 4, 7, 10, 11, },
  { 5, 7, 10, 11, },
  { 0, 5, 7, 10, 11, },
  { 1, 5, 7, 10, 11, },
  { 0, 1, 5, 7, 10, 11, },
  { 2, 5, 7, 10, 11, },
  { 0, 2, 5, 7, 10, 11, },
  { 1, 2, 5, 7, 10, 11, },
  { 0, 1, 2, 5, 7, 10, 11, },
  { 3, 5, 7, 10, 11, },
  { 0, 3, 5, 7, 10, 11, },
  { 1, 3, 5, 7, 10, 11, },
  { 0, 1, 3, 5, 7, 10, 11, },
  { 2, 3, 5, 7, 10, 11, },
  { 0, 2, 3, 5, 7, 10, 11, },
  { 1, 2, 3, 5, 7, 10, 11, },
  { 0, 1, 2, 3, 5, 7, 10, 11, },
  { 4, 5, 7, 10, 11, },
  { 0, 4, 5, 7, 10, 11, },
  { 1, 4, 5, 7, 10, 11, },
  { 0, 1, 4, 5, 7, 10, 11, },
  { 2, 4, 5, 7, 10, 11, },
  { 0, 2, 4, 5, 7, 10, 11, },
  { 1, 2, 4, 5, 7, 10, 11, },
  { 0, 1, 2, 4, 5, 7, 10, 11, },
  { 3, 4, 5, 7, 10, 11, },
  { 0, 3, 4, 5, 7, 10, 11, },
  { 1, 3, 4, 5, 7, 10, 11, },
  { 0, 1, 3, 4, 5, 7, 10, 11, },
  { 2, 3, 4, 5, 7, 10, 11, },
  { 0, 2, 3, 4, 5, 7, 10, 11, },
  { 1, 2, 3, 4, 5, 7, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, },
  { 6, 7, 10, 11, },
  { 0, 6, 7, 10, 11, },
  { 1, 6, 7, 10, 11, },
  { 0, 1, 6, 7, 10, 11, },
  { 2, 6, 7, 10, 11, },
  { 0, 2, 6, 7, 10, 11, },
  { 1, 2, 6, 7, 10, 11, },
  { 0, 1, 2, 6, 7, 10, 11, },
  { 3, 6, 7, 10, 11, },
  { 0, 3, 6, 7, 10, 11, },
  { 1, 3, 6, 7, 10, 11, },
  { 0, 1, 3, 6, 7, 10, 11, },
  { 2, 3, 6, 7, 10, 11, },
  { 0, 2, 3, 6, 7, 10, 11, },
  { 1, 2, 3, 6, 7, 10, 11, },
  { 0, 1, 2, 3, 6, 7, 10, 11, },
  { 4, 6, 7, 10, 11, },
  { 0, 4, 6, 7, 10, 11, },
  { 1, 4, 6, 7, 10, 11, },
  { 0, 1, 4, 6, 7, 10, 11, },
  { 2, 4, 6, 7, 10, 11, },
  { 0, 2, 4, 6, 7, 10, 11, },
  { 1, 2, 4, 6, 7, 10, 11, },
  { 0, 1, 2, 4, 6, 7, 10, 11, },
  { 3, 4, 6, 7, 10, 11, },
  { 0, 3, 4, 6, 7, 10, 11, },
  { 1, 3, 4, 6, 7, 10, 11, },
  { 0, 1, 3, 4, 6, 7, 10, 11, },
  { 2, 3, 4, 6, 7, 10, 11, },
  { 0, 2, 3, 4, 6, 7, 10, 11, },
  { 1, 2, 3, 4, 6, 7, 10, 11, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, },
  { 5, 6, 7, 10, 11, },
  { 0, 5, 6, 7, 10, 11, },
  { 1, 5, 6, 7, 10, 11, },
  { 0, 1, 5, 6, 7, 10, 11, },
  { 2, 5, 6, 7, 10, 11, },
  { 0, 2, 5, 6, 7, 10, 11, },
  { 1, 2, 5, 6, 7, 10, 11, },
  { 0, 1, 2, 5, 6, 7, 10, 11, },
  { 3, 5, 6, 7, 10, 11, },
  { 0, 3, 5, 6, 7, 10, 11, },
  { 1, 3, 5, 6, 7, 10, 11, },
  { 0, 1, 3, 5, 6, 7, 10, 11, },
  { 2, 3, 5, 6, 7, 10, 11, },
  { 0, 2, 3, 5, 6, 7, 10, 11, },
  { 1, 2, 3, 5, 6, 7, 10, 11, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, },
  { 4, 5, 6, 7, 10, 11, },
  { 0, 4, 5, 6, 7, 10, 11, },
  { 1, 4, 5, 6, 7, 10, 11, },
  { 0, 1, 4, 5, 6, 7, 10, 11, },
  { 2, 4, 5, 6, 7, 10, 11, },
  { 0, 2, 4, 5, 6, 7, 10, 11, },
  { 1, 2, 4, 5, 6, 7, 10, 11, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, },
  { 3, 4, 5, 6, 7, 10, 11, },
  { 0, 3, 4, 5, 6, 7, 10, 11, },
  { 1, 3, 4, 5, 6, 7, 10, 11, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, },
  { 2, 3, 4, 5, 6, 7, 10, 11, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, },
  { 8, 10, 11, },
  { 0, 8, 10, 11, },
  { 1, 8, 10, 11, },
  { 0, 1, 8, 10, 11, },
  { 2, 8, 10, 11, },
  { 0, 2, 8, 10, 11, },
  { 1, 2, 8, 10, 11, },
  { 0, 1, 2, 8, 10, 11, },
  { 3, 8, 10, 11, },
  { 0, 3, 8, 10, 11, },
  { 1, 3, 8, 10, 11, },
  { 0, 1, 3, 8, 10, 11, },
  { 2, 3, 8, 10, 11, },
  { 0, 2, 3, 8, 10, 11, },
  { 1, 2, 3, 8, 10, 11, },
  { 0, 1, 2, 3, 8, 10, 11, },
  { 4, 8, 10, 11, },
  { 0, 4, 8, 10, 11, },
  { 1, 4, 8, 10, 11, },
  { 0, 1, 4, 8, 10, 11, },
  { 2, 4, 8, 10, 11, },
  { 0, 2, 4, 8, 10, 11, },
  { 1, 2, 4, 8, 10, 11, },
  { 0, 1, 2, 4, 8, 10, 11, },
  { 3, 4, 8, 10, 11, },
  { 0, 3, 4, 8, 10, 11, },
  { 1, 3, 4, 8, 10, 11, },
  { 0, 1, 3, 4, 8, 10, 11, },
  { 2, 3, 4, 8, 10, 11, },
  { 0, 2, 3, 4, 8, 10, 11, },
  { 1, 2, 3, 4, 8, 10, 11, },
  { 0, 1, 2, 3, 4, 8, 10, 11, },
  { 5, 8, 10, 11, },
  { 0, 5, 8, 10, 11, },
  { 1, 5, 8, 10, 11, },
  { 0, 1, 5, 8, 10, 11, },
  { 2, 5, 8, 10, 11, },
  { 0, 2, 5, 8, 10, 11, },
  { 1, 2, 5, 8, 10, 11, },
  { 0, 1, 2, 5, 8, 10, 11, },
  { 3, 5, 8, 10, 11, },
  { 0, 3, 5, 8, 10, 11, },
  { 1, 3, 5, 8, 10, 11, },
  { 0, 1, 3, 5, 8, 10, 11, },
  { 2, 3, 5, 8, 10, 11, },
  { 0, 2, 3, 5, 8, 10, 11, },
  { 1, 2, 3, 5, 8, 10, 11, },
  { 0, 1, 2, 3, 5, 8, 10, 11, },
  { 4, 5, 8, 10, 11, },
  { 0, 4, 5, 8, 10, 11, },
  { 1, 4, 5, 8, 10, 11, },
  { 0, 1, 4, 5, 8, 10, 11, },
  { 2, 4, 5, 8, 10, 11, },
  { 0, 2, 4, 5, 8, 10, 11, },
  { 1, 2, 4, 5, 8, 10, 11, },
  { 0, 1, 2, 4, 5, 8, 10, 11, },
  { 3, 4, 5, 8, 10, 11, },
  { 0, 3, 4, 5, 8, 10, 11, },
  { 1, 3, 4, 5, 8, 10, 11, },
  { 0, 1, 3, 4, 5, 8, 10, 11, },
  { 2, 3, 4, 5, 8, 10, 11, },
  { 0, 2, 3, 4, 5, 8, 10, 11, },
  { 1, 2, 3, 4, 5, 8, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, },
  { 6, 8, 10, 11, },
  { 0, 6, 8, 10, 11, },
  { 1, 6, 8, 10, 11, },
  { 0, 1, 6, 8, 10, 11, },
  { 2, 6, 8, 10, 11, },
  { 0, 2, 6, 8, 10, 11, },
  { 1, 2, 6, 8, 10, 11, },
  { 0, 1, 2, 6, 8, 10, 11, },
  { 3, 6, 8, 10, 11, },
  { 0, 3, 6, 8, 10, 11, },
  { 1, 3, 6, 8, 10, 11, },
  { 0, 1, 3, 6, 8, 10, 11, },
  { 2, 3, 6, 8, 10, 11, },
  { 0, 2, 3, 6, 8, 10, 11, },
  { 1, 2, 3, 6, 8, 10, 11, },
  { 0, 1, 2, 3, 6, 8, 10, 11, },
  { 4, 6, 8, 10, 11, },
  { 0, 4, 6, 8, 10, 11, },
  { 1, 4, 6, 8, 10, 11, },
  { 0, 1, 4, 6, 8, 10, 11, },
  { 2, 4, 6, 8, 10, 11, },
  { 0, 2, 4, 6, 8, 10, 11, },
  { 1, 2, 4, 6, 8, 10, 11, },
  { 0, 1, 2, 4, 6, 8, 10, 11, },
  { 3, 4, 6, 8, 10, 11, },
  { 0, 3, 4, 6, 8, 10, 11, },
  { 1, 3, 4, 6, 8, 10, 11, },
  { 0, 1, 3, 4, 6, 8, 10, 11, },
  { 2, 3, 4, 6, 8, 10, 11, },
  { 0, 2, 3, 4, 6, 8, 10, 11, },
  { 1, 2, 3, 4, 6, 8, 10, 11, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, },
  { 5, 6, 8, 10, 11, },
  { 0, 5, 6, 8, 10, 11, },
  { 1, 5, 6, 8, 10, 11, },
  { 0, 1, 5, 6, 8, 10, 11, },
  { 2, 5, 6, 8, 10, 11, },
  { 0, 2, 5, 6, 8, 10, 11, },
  { 1, 2, 5, 6, 8, 10, 11, },
  { 0, 1, 2, 5, 6, 8, 10, 11, },
  { 3, 5, 6, 8, 10, 11, },
  { 0, 3, 5, 6, 8, 10, 11, },
  { 1, 3, 5, 6, 8, 10, 11, },
  { 0, 1, 3, 5, 6, 8, 10, 11, },
  { 2, 3, 5, 6, 8, 10, 11, },
  { 0, 2, 3, 5, 6, 8, 10, 11, },
  { 1, 2, 3, 5, 6, 8, 10, 11, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, },
  { 4, 5, 6, 8, 10, 11, },
  { 0, 4, 5, 6, 8, 10, 11, },
  { 1, 4, 5, 6, 8, 10, 11, },
  { 0, 1, 4, 5, 6, 8, 10, 11, },
  { 2, 4, 5, 6, 8, 10, 11, },
  { 0, 2, 4, 5, 6, 8, 10, 11, },
  { 1, 2, 4, 5, 6, 8, 10, 11, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, },
  { 3, 4, 5, 6, 8, 10, 11, },
  { 0, 3, 4, 5, 6, 8, 10, 11, },
  { 1, 3, 4, 5, 6, 8, 10, 11, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, },
  { 2, 3, 4, 5, 6, 8, 10, 11, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, },
  { 7, 8, 10, 11, },
  { 0, 7, 8, 10, 11, },
  { 1, 7, 8, 10, 11, },
  { 0, 1, 7, 8, 10, 11, },
  { 2, 7, 8, 10, 11, },
  { 0, 2, 7, 8, 10, 11, },
  { 1, 2, 7, 8, 10, 11, },
  { 0, 1, 2, 7, 8, 10, 11, },
  { 3, 7, 8, 10, 11, },
  { 0, 3, 7, 8, 10, 11, },
  { 1, 3, 7, 8, 10, 11, },
  { 0, 1, 3, 7, 8, 10, 11, },
  { 2, 3, 7, 8, 10, 11, },
  { 0, 2, 3, 7, 8, 10, 11, },
  { 1, 2, 3, 7, 8, 10, 11, },
  { 0, 1, 2, 3, 7, 8, 10, 11, },
  { 4, 7, 8, 10, 11, },
  { 0, 4, 7, 8, 10, 11, },
  { 1, 4, 7, 8, 10, 11, },
  { 0, 1, 4, 7, 8, 10, 11, },
  { 2, 4, 7, 8, 10, 11, },
  { 0, 2, 4, 7, 8, 10, 11, },
  { 1, 2, 4, 7, 8, 10, 11, },
  { 0, 1, 2, 4, 7, 8, 10, 11, },
  { 3, 4, 7, 8, 10, 11, },
  { 0, 3, 4, 7, 8, 10, 11, },
  { 1, 3, 4, 7, 8, 10, 11, },
  { 0, 1, 3, 4, 7, 8, 10, 11, },
  { 2, 3, 4, 7, 8, 10, 11, },
  { 0, 2, 3, 4, 7, 8, 10, 11, },
  { 1, 2, 3, 4, 7, 8, 10, 11, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, },
  { 5, 7, 8, 10, 11, },
  { 0, 5, 7, 8, 10, 11, },
  { 1, 5, 7, 8, 10, 11, },
  { 0, 1, 5, 7, 8, 10, 11, },
  { 2, 5, 7, 8, 10, 11, },
  { 0, 2, 5, 7, 8, 10, 11, },
  { 1, 2, 5, 7, 8, 10, 11, },
  { 0, 1, 2, 5, 7, 8, 10, 11, },
  { 3, 5, 7, 8, 10, 11, },
  { 0, 3, 5, 7, 8, 10, 11, },
  { 1, 3, 5, 7, 8, 10, 11, },
  { 0, 1, 3, 5, 7, 8, 10, 11, },
  { 2, 3, 5, 7, 8, 10, 11, },
  { 0, 2, 3, 5, 7, 8, 10, 11, },
  { 1, 2, 3, 5, 7, 8, 10, 11, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, },
  { 4, 5, 7, 8, 10, 11, },
  { 0, 4, 5, 7, 8, 10, 11, },
  { 1, 4, 5, 7, 8, 10, 11, },
  { 0, 1, 4, 5, 7, 8, 10, 11, },
  { 2, 4, 5, 7, 8, 10, 11, },
  { 0, 2, 4, 5, 7, 8, 10, 11, },
  { 1, 2, 4, 5, 7, 8, 10, 11, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, },
  { 3, 4, 5, 7, 8, 10, 11, },
  { 0, 3, 4, 5, 7, 8, 10, 11, },
  { 1, 3, 4, 5, 7, 8, 10, 11, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, },
  { 2, 3, 4, 5, 7, 8, 10, 11, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, },
  { 6, 7, 8, 10, 11, },
  { 0, 6, 7, 8, 10, 11, },
  { 1, 6, 7, 8, 10, 11, },
  { 0, 1, 6, 7, 8, 10, 11, },
  { 2, 6, 7, 8, 10, 11, },
  { 0, 2, 6, 7, 8, 10, 11, },
  { 1, 2, 6, 7, 8, 10, 11, },
  { 0, 1, 2, 6, 7, 8, 10, 11, },
  { 3, 6, 7, 8, 10, 11, },
  { 0, 3, 6, 7, 8, 10, 11, },
  { 1, 3, 6, 7, 8, 10, 11, },
  { 0, 1, 3, 6, 7, 8, 10, 11, },
  { 2, 3, 6, 7, 8, 10, 11, },
  { 0, 2, 3, 6, 7, 8, 10, 11, },
  { 1, 2, 3, 6, 7, 8, 10, 11, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, },
  { 4, 6, 7, 8, 10, 11, },
  { 0, 4, 6, 7, 8, 10, 11, },
  { 1, 4, 6, 7, 8, 10, 11, },
  { 0, 1, 4, 6, 7, 8, 10, 11, },
  { 2, 4, 6, 7, 8, 10, 11, },
  { 0, 2, 4, 6, 7, 8, 10, 11, },
  { 1, 2, 4, 6, 7, 8, 10, 11, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, },
  { 3, 4, 6, 7, 8, 10, 11, },
  { 0, 3, 4, 6, 7, 8, 10, 11, },
  { 1, 3, 4, 6, 7, 8, 10, 11, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, },
  { 2, 3, 4, 6, 7, 8, 10, 11, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, },
  { 5, 6, 7, 8, 10, 11, },
  { 0, 5, 6, 7, 8, 10, 11, },
  { 1, 5, 6, 7, 8, 10, 11, },
  { 0, 1, 5, 6, 7, 8, 10, 11, },
  { 2, 5, 6, 7, 8, 10, 11, },
  { 0, 2, 5, 6, 7, 8, 10, 11, },
  { 1, 2, 5, 6, 7, 8, 10, 11, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, },
  { 3, 5, 6, 7, 8, 10, 11, },
  { 0, 3, 5, 6, 7, 8, 10, 11, },
  { 1, 3, 5, 6, 7, 8, 10, 11, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, },
  { 2, 3, 5, 6, 7, 8, 10, 11, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, },
  { 4, 5, 6, 7, 8, 10, 11, },
  { 0, 4, 5, 6, 7, 8, 10, 11, },
  { 1, 4, 5, 6, 7, 8, 10, 11, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, },
  { 2, 4, 5, 6, 7, 8, 10, 11, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, },
  { 3, 4, 5, 6, 7, 8, 10, 11, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, },
  { 9, 10, 11, },
  { 0, 9, 10, 11, },
  { 1, 9, 10, 11, },
  { 0, 1, 9, 10, 11, },
  { 2, 9, 10, 11, },
  { 0, 2, 9, 10, 11, },
  { 1, 2, 9, 10, 11, },
  { 0, 1, 2, 9, 10, 11, },
  { 3, 9, 10, 11, },
  { 0, 3, 9, 10, 11, },
  { 1, 3, 9, 10, 11, },
  { 0, 1, 3, 9, 10, 11, },
  { 2, 3, 9, 10, 11, },
  { 0, 2, 3, 9, 10, 11, },
  { 1, 2, 3, 9, 10, 11, },
  { 0, 1, 2, 3, 9, 10, 11, },
  { 4, 9, 10, 11, },
  { 0, 4, 9, 10, 11, },
  { 1, 4, 9, 10, 11, },
  { 0, 1, 4, 9, 10, 11, },
  { 2, 4, 9, 10, 11, },
  { 0, 2, 4, 9, 10, 11, },
  { 1, 2, 4, 9, 10, 11, },
  { 0, 1, 2, 4, 9, 10, 11, },
  { 3, 4, 9, 10, 11, },
  { 0, 3, 4, 9, 10, 11, },
  { 1, 3, 4, 9, 10, 11, },
  { 0, 1, 3, 4, 9, 10, 11, },
  { 2, 3, 4, 9, 10, 11, },
  { 0, 2, 3, 4, 9, 10, 11, },
  { 1, 2, 3, 4, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 9, 10, 11, },
  { 5, 9, 10, 11, },
  { 0, 5, 9, 10, 11, },
  { 1, 5, 9, 10, 11, },
  { 0, 1, 5, 9, 10, 11, },
  { 2, 5, 9, 10, 11, },
  { 0, 2, 5, 9, 10, 11, },
  { 1, 2, 5, 9, 10, 11, },
  { 0, 1, 2, 5, 9, 10, 11, },
  { 3, 5, 9, 10, 11, },
  { 0, 3, 5, 9, 10, 11, },
  { 1, 3, 5, 9, 10, 11, },
  { 0, 1, 3, 5, 9, 10, 11, },
  { 2, 3, 5, 9, 10, 11, },
  { 0, 2, 3, 5, 9, 10, 11, },
  { 1, 2, 3, 5, 9, 10, 11, },
  { 0, 1, 2, 3, 5, 9, 10, 11, },
  { 4, 5, 9, 10, 11, },
  { 0, 4, 5, 9, 10, 11, },
  { 1, 4, 5, 9, 10, 11, },
  { 0, 1, 4, 5, 9, 10, 11, },
  { 2, 4, 5, 9, 10, 11, },
  { 0, 2, 4, 5, 9, 10, 11, },
  { 1, 2, 4, 5, 9, 10, 11, },
  { 0, 1, 2, 4, 5, 9, 10, 11, },
  { 3, 4, 5, 9, 10, 11, },
  { 0, 3, 4, 5, 9, 10, 11, },
  { 1, 3, 4, 5, 9, 10, 11, },
  { 0, 1, 3, 4, 5, 9, 10, 11, },
  { 2, 3, 4, 5, 9, 10, 11, },
  { 0, 2, 3, 4, 5, 9, 10, 11, },
  { 1, 2, 3, 4, 5, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, },
  { 6, 9, 10, 11, },
  { 0, 6, 9, 10, 11, },
  { 1, 6, 9, 10, 11, },
  { 0, 1, 6, 9, 10, 11, },
  { 2, 6, 9, 10, 11, },
  { 0, 2, 6, 9, 10, 11, },
  { 1, 2, 6, 9, 10, 11, },
  { 0, 1, 2, 6, 9, 10, 11, },
  { 3, 6, 9, 10, 11, },
  { 0, 3, 6, 9, 10, 11, },
  { 1, 3, 6, 9, 10, 11, },
  { 0, 1, 3, 6, 9, 10, 11, },
  { 2, 3, 6, 9, 10, 11, },
  { 0, 2, 3, 6, 9, 10, 11, },
  { 1, 2, 3, 6, 9, 10, 11, },
  { 0, 1, 2, 3, 6, 9, 10, 11, },
  { 4, 6, 9, 10, 11, },
  { 0, 4, 6, 9, 10, 11, },
  { 1, 4, 6, 9, 10, 11, },
  { 0, 1, 4, 6, 9, 10, 11, },
  { 2, 4, 6, 9, 10, 11, },
  { 0, 2, 4, 6, 9, 10, 11, },
  { 1, 2, 4, 6, 9, 10, 11, },
  { 0, 1, 2, 4, 6, 9, 10, 11, },
  { 3, 4, 6, 9, 10, 11, },
  { 0, 3, 4, 6, 9, 10, 11, },
  { 1, 3, 4, 6, 9, 10, 11, },
  { 0, 1, 3, 4, 6, 9, 10, 11, },
  { 2, 3, 4, 6, 9, 10, 11, },
  { 0, 2, 3, 4, 6, 9, 10, 11, },
  { 1, 2, 3, 4, 6, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, },
  { 5, 6, 9, 10, 11, },
  { 0, 5, 6, 9, 10, 11, },
  { 1, 5, 6, 9, 10, 11, },
  { 0, 1, 5, 6, 9, 10, 11, },
  { 2, 5, 6, 9, 10, 11, },
  { 0, 2, 5, 6, 9, 10, 11, },
  { 1, 2, 5, 6, 9, 10, 11, },
  { 0, 1, 2, 5, 6, 9, 10, 11, },
  { 3, 5, 6, 9, 10, 11, },
  { 0, 3, 5, 6, 9, 10, 11, },
  { 1, 3, 5, 6, 9, 10, 11, },
  { 0, 1, 3, 5, 6, 9, 10, 11, },
  { 2, 3, 5, 6, 9, 10, 11, },
  { 0, 2, 3, 5, 6, 9, 10, 11, },
  { 1, 2, 3, 5, 6, 9, 10, 11, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, },
  { 4, 5, 6, 9, 10, 11, },
  { 0, 4, 5, 6, 9, 10, 11, },
  { 1, 4, 5, 6, 9, 10, 11, },
  { 0, 1, 4, 5, 6, 9, 10, 11, },
  { 2, 4, 5, 6, 9, 10, 11, },
  { 0, 2, 4, 5, 6, 9, 10, 11, },
  { 1, 2, 4, 5, 6, 9, 10, 11, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, },
  { 3, 4, 5, 6, 9, 10, 11, },
  { 0, 3, 4, 5, 6, 9, 10, 11, },
  { 1, 3, 4, 5, 6, 9, 10, 11, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, },
  { 2, 3, 4, 5, 6, 9, 10, 11, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, },
  { 7, 9, 10, 11, },
  { 0, 7, 9, 10, 11, },
  { 1, 7, 9, 10, 11, },
  { 0, 1, 7, 9, 10, 11, },
  { 2, 7, 9, 10, 11, },
  { 0, 2, 7, 9, 10, 11, },
  { 1, 2, 7, 9, 10, 11, },
  { 0, 1, 2, 7, 9, 10, 11, },
  { 3, 7, 9, 10, 11, },
  { 0, 3, 7, 9, 10, 11, },
  { 1, 3, 7, 9, 10, 11, },
  { 0, 1, 3, 7, 9, 10, 11, },
  { 2, 3, 7, 9, 10, 11, },
  { 0, 2, 3, 7, 9, 10, 11, },
  { 1, 2, 3, 7, 9, 10, 11, },
  { 0, 1, 2, 3, 7, 9, 10, 11, },
  { 4, 7, 9, 10, 11, },
  { 0, 4, 7, 9, 10, 11, },
  { 1, 4, 7, 9, 10, 11, },
  { 0, 1, 4, 7, 9, 10, 11, },
  { 2, 4, 7, 9, 10, 11, },
  { 0, 2, 4, 7, 9, 10, 11, },
  { 1, 2, 4, 7, 9, 10, 11, },
  { 0, 1, 2, 4, 7, 9, 10, 11, },
  { 3, 4, 7, 9, 10, 11, },
  { 0, 3, 4, 7, 9, 10, 11, },
  { 1, 3, 4, 7, 9, 10, 11, },
  { 0, 1, 3, 4, 7, 9, 10, 11, },
  { 2, 3, 4, 7, 9, 10, 11, },
  { 0, 2, 3, 4, 7, 9, 10, 11, },
  { 1, 2, 3, 4, 7, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, },
  { 5, 7, 9, 10, 11, },
  { 0, 5, 7, 9, 10, 11, },
  { 1, 5, 7, 9, 10, 11, },
  { 0, 1, 5, 7, 9, 10, 11, },
  { 2, 5, 7, 9, 10, 11, },
  { 0, 2, 5, 7, 9, 10, 11, },
  { 1, 2, 5, 7, 9, 10, 11, },
  { 0, 1, 2, 5, 7, 9, 10, 11, },
  { 3, 5, 7, 9, 10, 11, },
  { 0, 3, 5, 7, 9, 10, 11, },
  { 1, 3, 5, 7, 9, 10, 11, },
  { 0, 1, 3, 5, 7, 9, 10, 11, },
  { 2, 3, 5, 7, 9, 10, 11, },
  { 0, 2, 3, 5, 7, 9, 10, 11, },
  { 1, 2, 3, 5, 7, 9, 10, 11, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, },
  { 4, 5, 7, 9, 10, 11, },
  { 0, 4, 5, 7, 9, 10, 11, },
  { 1, 4, 5, 7, 9, 10, 11, },
  { 0, 1, 4, 5, 7, 9, 10, 11, },
  { 2, 4, 5, 7, 9, 10, 11, },
  { 0, 2, 4, 5, 7, 9, 10, 11, },
  { 1, 2, 4, 5, 7, 9, 10, 11, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, },
  { 3, 4, 5, 7, 9, 10, 11, },
  { 0, 3, 4, 5, 7, 9, 10, 11, },
  { 1, 3, 4, 5, 7, 9, 10, 11, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, },
  { 2, 3, 4, 5, 7, 9, 10, 11, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, },
  { 6, 7, 9, 10, 11, },
  { 0, 6, 7, 9, 10, 11, },
  { 1, 6, 7, 9, 10, 11, },
  { 0, 1, 6, 7, 9, 10, 11, },
  { 2, 6, 7, 9, 10, 11, },
  { 0, 2, 6, 7, 9, 10, 11, },
  { 1, 2, 6, 7, 9, 10, 11, },
  { 0, 1, 2, 6, 7, 9, 10, 11, },
  { 3, 6, 7, 9, 10, 11, },
  { 0, 3, 6, 7, 9, 10, 11, },
  { 1, 3, 6, 7, 9, 10, 11, },
  { 0, 1, 3, 6, 7, 9, 10, 11, },
  { 2, 3, 6, 7, 9, 10, 11, },
  { 0, 2, 3, 6, 7, 9, 10, 11, },
  { 1, 2, 3, 6, 7, 9, 10, 11, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, },
  { 4, 6, 7, 9, 10, 11, },
  { 0, 4, 6, 7, 9, 10, 11, },
  { 1, 4, 6, 7, 9, 10, 11, },
  { 0, 1, 4, 6, 7, 9, 10, 11, },
  { 2, 4, 6, 7, 9, 10, 11, },
  { 0, 2, 4, 6, 7, 9, 10, 11, },
  { 1, 2, 4, 6, 7, 9, 10, 11, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, },
  { 3, 4, 6, 7, 9, 10, 11, },
  { 0, 3, 4, 6, 7, 9, 10, 11, },
  { 1, 3, 4, 6, 7, 9, 10, 11, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, },
  { 2, 3, 4, 6, 7, 9, 10, 11, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, },
  { 5, 6, 7, 9, 10, 11, },
  { 0, 5, 6, 7, 9, 10, 11, },
  { 1, 5, 6, 7, 9, 10, 11, },
  { 0, 1, 5, 6, 7, 9, 10, 11, },
  { 2, 5, 6, 7, 9, 10, 11, },
  { 0, 2, 5, 6, 7, 9, 10, 11, },
  { 1, 2, 5, 6, 7, 9, 10, 11, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, },
  { 3, 5, 6, 7, 9, 10, 11, },
  { 0, 3, 5, 6, 7, 9, 10, 11, },
  { 1, 3, 5, 6, 7, 9, 10, 11, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, },
  { 2, 3, 5, 6, 7, 9, 10, 11, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, },
  { 4, 5, 6, 7, 9, 10, 11, },
  { 0, 4, 5, 6, 7, 9, 10, 11, },
  { 1, 4, 5, 6, 7, 9, 10, 11, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, },
  { 2, 4, 5, 6, 7, 9, 10, 11, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, },
  { 3, 4, 5, 6, 7, 9, 10, 11, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, },
  { 8, 9, 10, 11, },
  { 0, 8, 9, 10, 11, },
  { 1, 8, 9, 10, 11, },
  { 0, 1, 8, 9, 10, 11, },
  { 2, 8, 9, 10, 11, },
  { 0, 2, 8, 9, 10, 11, },
  { 1, 2, 8, 9, 10, 11, },
  { 0, 1, 2, 8, 9, 10, 11, },
  { 3, 8, 9, 10, 11, },
  { 0, 3, 8, 9, 10, 11, },
  { 1, 3, 8, 9, 10, 11, },
  { 0, 1, 3, 8, 9, 10, 11, },
  { 2, 3, 8, 9, 10, 11, },
  { 0, 2, 3, 8, 9, 10, 11, },
  { 1, 2, 3, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 8, 9, 10, 11, },
  { 4, 8, 9, 10, 11, },
  { 0, 4, 8, 9, 10, 11, },
  { 1, 4, 8, 9, 10, 11, },
  { 0, 1, 4, 8, 9, 10, 11, },
  { 2, 4, 8, 9, 10, 11, },
  { 0, 2, 4, 8, 9, 10, 11, },
  { 1, 2, 4, 8, 9, 10, 11, },
  { 0, 1, 2, 4, 8, 9, 10, 11, },
  { 3, 4, 8, 9, 10, 11, },
  { 0, 3, 4, 8, 9, 10, 11, },
  { 1, 3, 4, 8, 9, 10, 11, },
  { 0, 1, 3, 4, 8, 9, 10, 11, },
  { 2, 3, 4, 8, 9, 10, 11, },
  { 0, 2, 3, 4, 8, 9, 10, 11, },
  { 1, 2, 3, 4, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, },
  { 5, 8, 9, 10, 11, },
  { 0, 5, 8, 9, 10, 11, },
  { 1, 5, 8, 9, 10, 11, },
  { 0, 1, 5, 8, 9, 10, 11, },
  { 2, 5, 8, 9, 10, 11, },
  { 0, 2, 5, 8, 9, 10, 11, },
  { 1, 2, 5, 8, 9, 10, 11, },
  { 0, 1, 2, 5, 8, 9, 10, 11, },
  { 3, 5, 8, 9, 10, 11, },
  { 0, 3, 5, 8, 9, 10, 11, },
  { 1, 3, 5, 8, 9, 10, 11, },
  { 0, 1, 3, 5, 8, 9, 10, 11, },
  { 2, 3, 5, 8, 9, 10, 11, },
  { 0, 2, 3, 5, 8, 9, 10, 11, },
  { 1, 2, 3, 5, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, },
  { 4, 5, 8, 9, 10, 11, },
  { 0, 4, 5, 8, 9, 10, 11, },
  { 1, 4, 5, 8, 9, 10, 11, },
  { 0, 1, 4, 5, 8, 9, 10, 11, },
  { 2, 4, 5, 8, 9, 10, 11, },
  { 0, 2, 4, 5, 8, 9, 10, 11, },
  { 1, 2, 4, 5, 8, 9, 10, 11, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, },
  { 3, 4, 5, 8, 9, 10, 11, },
  { 0, 3, 4, 5, 8, 9, 10, 11, },
  { 1, 3, 4, 5, 8, 9, 10, 11, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, },
  { 2, 3, 4, 5, 8, 9, 10, 11, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, },
  { 6, 8, 9, 10, 11, },
  { 0, 6, 8, 9, 10, 11, },
  { 1, 6, 8, 9, 10, 11, },
  { 0, 1, 6, 8, 9, 10, 11, },
  { 2, 6, 8, 9, 10, 11, },
  { 0, 2, 6, 8, 9, 10, 11, },
  { 1, 2, 6, 8, 9, 10, 11, },
  { 0, 1, 2, 6, 8, 9, 10, 11, },
  { 3, 6, 8, 9, 10, 11, },
  { 0, 3, 6, 8, 9, 10, 11, },
  { 1, 3, 6, 8, 9, 10, 11, },
  { 0, 1, 3, 6, 8, 9, 10, 11, },
  { 2, 3, 6, 8, 9, 10, 11, },
  { 0, 2, 3, 6, 8, 9, 10, 11, },
  { 1, 2, 3, 6, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, },
  { 4, 6, 8, 9, 10, 11, },
  { 0, 4, 6, 8, 9, 10, 11, },
  { 1, 4, 6, 8, 9, 10, 11, },
  { 0, 1, 4, 6, 8, 9, 10, 11, },
  { 2, 4, 6, 8, 9, 10, 11, },
  { 0, 2, 4, 6, 8, 9, 10, 11, },
  { 1, 2, 4, 6, 8, 9, 10, 11, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, },
  { 3, 4, 6, 8, 9, 10, 11, },
  { 0, 3, 4, 6, 8, 9, 10, 11, },
  { 1, 3, 4, 6, 8, 9, 10, 11, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, },
  { 2, 3, 4, 6, 8, 9, 10, 11, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, },
  { 5, 6, 8, 9, 10, 11, },
  { 0, 5, 6, 8, 9, 10, 11, },
  { 1, 5, 6, 8, 9, 10, 11, },
  { 0, 1, 5, 6, 8, 9, 10, 11, },
  { 2, 5, 6, 8, 9, 10, 11, },
  { 0, 2, 5, 6, 8, 9, 10, 11, },
  { 1, 2, 5, 6, 8, 9, 10, 11, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, },
  { 3, 5, 6, 8, 9, 10, 11, },
  { 0, 3, 5, 6, 8, 9, 10, 11, },
  { 1, 3, 5, 6, 8, 9, 10, 11, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, },
  { 2, 3, 5, 6, 8, 9, 10, 11, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, },
  { 4, 5, 6, 8, 9, 10, 11, },
  { 0, 4, 5, 6, 8, 9, 10, 11, },
  { 1, 4, 5, 6, 8, 9, 10, 11, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, },
  { 2, 4, 5, 6, 8, 9, 10, 11, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, },
  { 3, 4, 5, 6, 8, 9, 10, 11, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, },
  { 7, 8, 9, 10, 11, },
  { 0, 7, 8, 9, 10, 11, },
  { 1, 7, 8, 9, 10, 11, },
  { 0, 1, 7, 8, 9, 10, 11, },
  { 2, 7, 8, 9, 10, 11, },
  { 0, 2, 7, 8, 9, 10, 11, },
  { 1, 2, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 7, 8, 9, 10, 11, },
  { 3, 7, 8, 9, 10, 11, },
  { 0, 3, 7, 8, 9, 10, 11, },
  { 1, 3, 7, 8, 9, 10, 11, },
  { 0, 1, 3, 7, 8, 9, 10, 11, },
  { 2, 3, 7, 8, 9, 10, 11, },
  { 0, 2, 3, 7, 8, 9, 10, 11, },
  { 1, 2, 3, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, },
  { 4, 7, 8, 9, 10, 11, },
  { 0, 4, 7, 8, 9, 10, 11, },
  { 1, 4, 7, 8, 9, 10, 11, },
  { 0, 1, 4, 7, 8, 9, 10, 11, },
  { 2, 4, 7, 8, 9, 10, 11, },
  { 0, 2, 4, 7, 8, 9, 10, 11, },
  { 1, 2, 4, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, },
  { 3, 4, 7, 8, 9, 10, 11, },
  { 0, 3, 4, 7, 8, 9, 10, 11, },
  { 1, 3, 4, 7, 8, 9, 10, 11, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, },
  { 2, 3, 4, 7, 8, 9, 10, 11, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, },
  { 5, 7, 8, 9, 10, 11, },
  { 0, 5, 7, 8, 9, 10, 11, },
  { 1, 5, 7, 8, 9, 10, 11, },
  { 0, 1, 5, 7, 8, 9, 10, 11, },
  { 2, 5, 7, 8, 9, 10, 11, },
  { 0, 2, 5, 7, 8, 9, 10, 11, },
  { 1, 2, 5, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, },
  { 3, 5, 7, 8, 9, 10, 11, },
  { 0, 3, 5, 7, 8, 9, 10, 11, },
  { 1, 3, 5, 7, 8, 9, 10, 11, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, },
  { 2, 3, 5, 7, 8, 9, 10, 11, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, },
  { 4, 5, 7, 8, 9, 10, 11, },
  { 0, 4, 5, 7, 8, 9, 10, 11, },
  { 1, 4, 5, 7, 8, 9, 10, 11, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, },
  { 2, 4, 5, 7, 8, 9, 10, 11, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, },
  { 3, 4, 5, 7, 8, 9, 10, 11, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, },
  { 6, 7, 8, 9, 10, 11, },
  { 0, 6, 7, 8, 9, 10, 11, },
  { 1, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 6, 7, 8, 9, 10, 11, },
  { 2, 6, 7, 8, 9, 10, 11, },
  { 0, 2, 6, 7, 8, 9, 10, 11, },
  { 1, 2, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, },
  { 3, 6, 7, 8, 9, 10, 11, },
  { 0, 3, 6, 7, 8, 9, 10, 11, },
  { 1, 3, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, },
  { 2, 3, 6, 7, 8, 9, 10, 11, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, },
  { 4, 6, 7, 8, 9, 10, 11, },
  { 0, 4, 6, 7, 8, 9, 10, 11, },
  { 1, 4, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, },
  { 2, 4, 6, 7, 8, 9, 10, 11, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, },
  { 3, 4, 6, 7, 8, 9, 10, 11, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, },
  { 5, 6, 7, 8, 9, 10, 11, },
  { 0, 5, 6, 7, 8, 9, 10, 11, },
  { 1, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, },
  { 2, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, },
  { 3, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, },
  { 4, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, },
  { 12, },
  { 0, 12, },
  { 1, 12, },
  { 0, 1, 12, },
  { 2, 12, },
  { 0, 2, 12, },
  { 1, 2, 12, },
  { 0, 1, 2, 12, },
  { 3, 12, },
  { 0, 3, 12, },
  { 1, 3, 12, },
  { 0, 1, 3, 12, },
  { 2, 3, 12, },
  { 0, 2, 3, 12, },
  { 1, 2, 3, 12, },
  { 0, 1, 2, 3, 12, },
  { 4, 12, },
  { 0, 4, 12, },
  { 1, 4, 12, },
  { 0, 1, 4, 12, },
  { 2, 4, 12, },
  { 0, 2, 4, 12, },
  { 1, 2, 4, 12, },
  { 0, 1, 2, 4, 12, },
  { 3, 4, 12, },
  { 0, 3, 4, 12, },
  { 1, 3, 4, 12, },
  { 0, 1, 3, 4, 12, },
  { 2, 3, 4, 12, },
  { 0, 2, 3, 4, 12, },
  { 1, 2, 3, 4, 12, },
  { 0, 1, 2, 3, 4, 12, },
  { 5, 12, },
  { 0, 5, 12, },
  { 1, 5, 12, },
  { 0, 1, 5, 12, },
  { 2, 5, 12, },
  { 0, 2, 5, 12, },
  { 1, 2, 5, 12, },
  { 0, 1, 2, 5, 12, },
  { 3, 5, 12, },
  { 0, 3, 5, 12, },
  { 1, 3, 5, 12, },
  { 0, 1, 3, 5, 12, },
  { 2, 3, 5, 12, },
  { 0, 2, 3, 5, 12, },
  { 1, 2, 3, 5, 12, },
  { 0, 1, 2, 3, 5, 12, },
  { 4, 5, 12, },
  { 0, 4, 5, 12, },
  { 1, 4, 5, 12, },
  { 0, 1, 4, 5, 12, },
  { 2, 4, 5, 12, },
  { 0, 2, 4, 5, 12, },
  { 1, 2, 4, 5, 12, },
  { 0, 1, 2, 4, 5, 12, },
  { 3, 4, 5, 12, },
  { 0, 3, 4, 5, 12, },
  { 1, 3, 4, 5, 12, },
  { 0, 1, 3, 4, 5, 12, },
  { 2, 3, 4, 5, 12, },
  { 0, 2, 3, 4, 5, 12, },
  { 1, 2, 3, 4, 5, 12, },
  { 0, 1, 2, 3, 4, 5, 12, },
  { 6, 12, },
  { 0, 6, 12, },
  { 1, 6, 12, },
  { 0, 1, 6, 12, },
  { 2, 6, 12, },
  { 0, 2, 6, 12, },
  { 1, 2, 6, 12, },
  { 0, 1, 2, 6, 12, },
  { 3, 6, 12, },
  { 0, 3, 6, 12, },
  { 1, 3, 6, 12, },
  { 0, 1, 3, 6, 12, },
  { 2, 3, 6, 12, },
  { 0, 2, 3, 6, 12, },
  { 1, 2, 3, 6, 12, },
  { 0, 1, 2, 3, 6, 12, },
  { 4, 6, 12, },
  { 0, 4, 6, 12, },
  { 1, 4, 6, 12, },
  { 0, 1, 4, 6, 12, },
  { 2, 4, 6, 12, },
  { 0, 2, 4, 6, 12, },
  { 1, 2, 4, 6, 12, },
  { 0, 1, 2, 4, 6, 12, },
  { 3, 4, 6, 12, },
  { 0, 3, 4, 6, 12, },
  { 1, 3, 4, 6, 12, },
  { 0, 1, 3, 4, 6, 12, },
  { 2, 3, 4, 6, 12, },
  { 0, 2, 3, 4, 6, 12, },
  { 1, 2, 3, 4, 6, 12, },
  { 0, 1, 2, 3, 4, 6, 12, },
  { 5, 6, 12, },
  { 0, 5, 6, 12, },
  { 1, 5, 6, 12, },
  { 0, 1, 5, 6, 12, },
  { 2, 5, 6, 12, },
  { 0, 2, 5, 6, 12, },
  { 1, 2, 5, 6, 12, },
  { 0, 1, 2, 5, 6, 12, },
  { 3, 5, 6, 12, },
  { 0, 3, 5, 6, 12, },
  { 1, 3, 5, 6, 12, },
  { 0, 1, 3, 5, 6, 12, },
  { 2, 3, 5, 6, 12, },
  { 0, 2, 3, 5, 6, 12, },
  { 1, 2, 3, 5, 6, 12, },
  { 0, 1, 2, 3, 5, 6, 12, },
  { 4, 5, 6, 12, },
  { 0, 4, 5, 6, 12, },
  { 1, 4, 5, 6, 12, },
  { 0, 1, 4, 5, 6, 12, },
  { 2, 4, 5, 6, 12, },
  { 0, 2, 4, 5, 6, 12, },
  { 1, 2, 4, 5, 6, 12, },
  { 0, 1, 2, 4, 5, 6, 12, },
  { 3, 4, 5, 6, 12, },
  { 0, 3, 4, 5, 6, 12, },
  { 1, 3, 4, 5, 6, 12, },
  { 0, 1, 3, 4, 5, 6, 12, },
  { 2, 3, 4, 5, 6, 12, },
  { 0, 2, 3, 4, 5, 6, 12, },
  { 1, 2, 3, 4, 5, 6, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 12, },
  { 7, 12, },
  { 0, 7, 12, },
  { 1, 7, 12, },
  { 0, 1, 7, 12, },
  { 2, 7, 12, },
  { 0, 2, 7, 12, },
  { 1, 2, 7, 12, },
  { 0, 1, 2, 7, 12, },
  { 3, 7, 12, },
  { 0, 3, 7, 12, },
  { 1, 3, 7, 12, },
  { 0, 1, 3, 7, 12, },
  { 2, 3, 7, 12, },
  { 0, 2, 3, 7, 12, },
  { 1, 2, 3, 7, 12, },
  { 0, 1, 2, 3, 7, 12, },
  { 4, 7, 12, },
  { 0, 4, 7, 12, },
  { 1, 4, 7, 12, },
  { 0, 1, 4, 7, 12, },
  { 2, 4, 7, 12, },
  { 0, 2, 4, 7, 12, },
  { 1, 2, 4, 7, 12, },
  { 0, 1, 2, 4, 7, 12, },
  { 3, 4, 7, 12, },
  { 0, 3, 4, 7, 12, },
  { 1, 3, 4, 7, 12, },
  { 0, 1, 3, 4, 7, 12, },
  { 2, 3, 4, 7, 12, },
  { 0, 2, 3, 4, 7, 12, },
  { 1, 2, 3, 4, 7, 12, },
  { 0, 1, 2, 3, 4, 7, 12, },
  { 5, 7, 12, },
  { 0, 5, 7, 12, },
  { 1, 5, 7, 12, },
  { 0, 1, 5, 7, 12, },
  { 2, 5, 7, 12, },
  { 0, 2, 5, 7, 12, },
  { 1, 2, 5, 7, 12, },
  { 0, 1, 2, 5, 7, 12, },
  { 3, 5, 7, 12, },
  { 0, 3, 5, 7, 12, },
  { 1, 3, 5, 7, 12, },
  { 0, 1, 3, 5, 7, 12, },
  { 2, 3, 5, 7, 12, },
  { 0, 2, 3, 5, 7, 12, },
  { 1, 2, 3, 5, 7, 12, },
  { 0, 1, 2, 3, 5, 7, 12, },
  { 4, 5, 7, 12, },
  { 0, 4, 5, 7, 12, },
  { 1, 4, 5, 7, 12, },
  { 0, 1, 4, 5, 7, 12, },
  { 2, 4, 5, 7, 12, },
  { 0, 2, 4, 5, 7, 12, },
  { 1, 2, 4, 5, 7, 12, },
  { 0, 1, 2, 4, 5, 7, 12, },
  { 3, 4, 5, 7, 12, },
  { 0, 3, 4, 5, 7, 12, },
  { 1, 3, 4, 5, 7, 12, },
  { 0, 1, 3, 4, 5, 7, 12, },
  { 2, 3, 4, 5, 7, 12, },
  { 0, 2, 3, 4, 5, 7, 12, },
  { 1, 2, 3, 4, 5, 7, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 12, },
  { 6, 7, 12, },
  { 0, 6, 7, 12, },
  { 1, 6, 7, 12, },
  { 0, 1, 6, 7, 12, },
  { 2, 6, 7, 12, },
  { 0, 2, 6, 7, 12, },
  { 1, 2, 6, 7, 12, },
  { 0, 1, 2, 6, 7, 12, },
  { 3, 6, 7, 12, },
  { 0, 3, 6, 7, 12, },
  { 1, 3, 6, 7, 12, },
  { 0, 1, 3, 6, 7, 12, },
  { 2, 3, 6, 7, 12, },
  { 0, 2, 3, 6, 7, 12, },
  { 1, 2, 3, 6, 7, 12, },
  { 0, 1, 2, 3, 6, 7, 12, },
  { 4, 6, 7, 12, },
  { 0, 4, 6, 7, 12, },
  { 1, 4, 6, 7, 12, },
  { 0, 1, 4, 6, 7, 12, },
  { 2, 4, 6, 7, 12, },
  { 0, 2, 4, 6, 7, 12, },
  { 1, 2, 4, 6, 7, 12, },
  { 0, 1, 2, 4, 6, 7, 12, },
  { 3, 4, 6, 7, 12, },
  { 0, 3, 4, 6, 7, 12, },
  { 1, 3, 4, 6, 7, 12, },
  { 0, 1, 3, 4, 6, 7, 12, },
  { 2, 3, 4, 6, 7, 12, },
  { 0, 2, 3, 4, 6, 7, 12, },
  { 1, 2, 3, 4, 6, 7, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 12, },
  { 5, 6, 7, 12, },
  { 0, 5, 6, 7, 12, },
  { 1, 5, 6, 7, 12, },
  { 0, 1, 5, 6, 7, 12, },
  { 2, 5, 6, 7, 12, },
  { 0, 2, 5, 6, 7, 12, },
  { 1, 2, 5, 6, 7, 12, },
  { 0, 1, 2, 5, 6, 7, 12, },
  { 3, 5, 6, 7, 12, },
  { 0, 3, 5, 6, 7, 12, },
  { 1, 3, 5, 6, 7, 12, },
  { 0, 1, 3, 5, 6, 7, 12, },
  { 2, 3, 5, 6, 7, 12, },
  { 0, 2, 3, 5, 6, 7, 12, },
  { 1, 2, 3, 5, 6, 7, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 12, },
  { 4, 5, 6, 7, 12, },
  { 0, 4, 5, 6, 7, 12, },
  { 1, 4, 5, 6, 7, 12, },
  { 0, 1, 4, 5, 6, 7, 12, },
  { 2, 4, 5, 6, 7, 12, },
  { 0, 2, 4, 5, 6, 7, 12, },
  { 1, 2, 4, 5, 6, 7, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 12, },
  { 3, 4, 5, 6, 7, 12, },
  { 0, 3, 4, 5, 6, 7, 12, },
  { 1, 3, 4, 5, 6, 7, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 12, },
  { 2, 3, 4, 5, 6, 7, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 12, },
  { 8, 12, },
  { 0, 8, 12, },
  { 1, 8, 12, },
  { 0, 1, 8, 12, },
  { 2, 8, 12, },
  { 0, 2, 8, 12, },
  { 1, 2, 8, 12, },
  { 0, 1, 2, 8, 12, },
  { 3, 8, 12, },
  { 0, 3, 8, 12, },
  { 1, 3, 8, 12, },
  { 0, 1, 3, 8, 12, },
  { 2, 3, 8, 12, },
  { 0, 2, 3, 8, 12, },
  { 1, 2, 3, 8, 12, },
  { 0, 1, 2, 3, 8, 12, },
  { 4, 8, 12, },
  { 0, 4, 8, 12, },
  { 1, 4, 8, 12, },
  { 0, 1, 4, 8, 12, },
  { 2, 4, 8, 12, },
  { 0, 2, 4, 8, 12, },
  { 1, 2, 4, 8, 12, },
  { 0, 1, 2, 4, 8, 12, },
  { 3, 4, 8, 12, },
  { 0, 3, 4, 8, 12, },
  { 1, 3, 4, 8, 12, },
  { 0, 1, 3, 4, 8, 12, },
  { 2, 3, 4, 8, 12, },
  { 0, 2, 3, 4, 8, 12, },
  { 1, 2, 3, 4, 8, 12, },
  { 0, 1, 2, 3, 4, 8, 12, },
  { 5, 8, 12, },
  { 0, 5, 8, 12, },
  { 1, 5, 8, 12, },
  { 0, 1, 5, 8, 12, },
  { 2, 5, 8, 12, },
  { 0, 2, 5, 8, 12, },
  { 1, 2, 5, 8, 12, },
  { 0, 1, 2, 5, 8, 12, },
  { 3, 5, 8, 12, },
  { 0, 3, 5, 8, 12, },
  { 1, 3, 5, 8, 12, },
  { 0, 1, 3, 5, 8, 12, },
  { 2, 3, 5, 8, 12, },
  { 0, 2, 3, 5, 8, 12, },
  { 1, 2, 3, 5, 8, 12, },
  { 0, 1, 2, 3, 5, 8, 12, },
  { 4, 5, 8, 12, },
  { 0, 4, 5, 8, 12, },
  { 1, 4, 5, 8, 12, },
  { 0, 1, 4, 5, 8, 12, },
  { 2, 4, 5, 8, 12, },
  { 0, 2, 4, 5, 8, 12, },
  { 1, 2, 4, 5, 8, 12, },
  { 0, 1, 2, 4, 5, 8, 12, },
  { 3, 4, 5, 8, 12, },
  { 0, 3, 4, 5, 8, 12, },
  { 1, 3, 4, 5, 8, 12, },
  { 0, 1, 3, 4, 5, 8, 12, },
  { 2, 3, 4, 5, 8, 12, },
  { 0, 2, 3, 4, 5, 8, 12, },
  { 1, 2, 3, 4, 5, 8, 12, },
  { 0, 1, 2, 3, 4, 5, 8, 12, },
  { 6, 8, 12, },
  { 0, 6, 8, 12, },
  { 1, 6, 8, 12, },
  { 0, 1, 6, 8, 12, },
  { 2, 6, 8, 12, },
  { 0, 2, 6, 8, 12, },
  { 1, 2, 6, 8, 12, },
  { 0, 1, 2, 6, 8, 12, },
  { 3, 6, 8, 12, },
  { 0, 3, 6, 8, 12, },
  { 1, 3, 6, 8, 12, },
  { 0, 1, 3, 6, 8, 12, },
  { 2, 3, 6, 8, 12, },
  { 0, 2, 3, 6, 8, 12, },
  { 1, 2, 3, 6, 8, 12, },
  { 0, 1, 2, 3, 6, 8, 12, },
  { 4, 6, 8, 12, },
  { 0, 4, 6, 8, 12, },
  { 1, 4, 6, 8, 12, },
  { 0, 1, 4, 6, 8, 12, },
  { 2, 4, 6, 8, 12, },
  { 0, 2, 4, 6, 8, 12, },
  { 1, 2, 4, 6, 8, 12, },
  { 0, 1, 2, 4, 6, 8, 12, },
  { 3, 4, 6, 8, 12, },
  { 0, 3, 4, 6, 8, 12, },
  { 1, 3, 4, 6, 8, 12, },
  { 0, 1, 3, 4, 6, 8, 12, },
  { 2, 3, 4, 6, 8, 12, },
  { 0, 2, 3, 4, 6, 8, 12, },
  { 1, 2, 3, 4, 6, 8, 12, },
  { 0, 1, 2, 3, 4, 6, 8, 12, },
  { 5, 6, 8, 12, },
  { 0, 5, 6, 8, 12, },
  { 1, 5, 6, 8, 12, },
  { 0, 1, 5, 6, 8, 12, },
  { 2, 5, 6, 8, 12, },
  { 0, 2, 5, 6, 8, 12, },
  { 1, 2, 5, 6, 8, 12, },
  { 0, 1, 2, 5, 6, 8, 12, },
  { 3, 5, 6, 8, 12, },
  { 0, 3, 5, 6, 8, 12, },
  { 1, 3, 5, 6, 8, 12, },
  { 0, 1, 3, 5, 6, 8, 12, },
  { 2, 3, 5, 6, 8, 12, },
  { 0, 2, 3, 5, 6, 8, 12, },
  { 1, 2, 3, 5, 6, 8, 12, },
  { 0, 1, 2, 3, 5, 6, 8, 12, },
  { 4, 5, 6, 8, 12, },
  { 0, 4, 5, 6, 8, 12, },
  { 1, 4, 5, 6, 8, 12, },
  { 0, 1, 4, 5, 6, 8, 12, },
  { 2, 4, 5, 6, 8, 12, },
  { 0, 2, 4, 5, 6, 8, 12, },
  { 1, 2, 4, 5, 6, 8, 12, },
  { 0, 1, 2, 4, 5, 6, 8, 12, },
  { 3, 4, 5, 6, 8, 12, },
  { 0, 3, 4, 5, 6, 8, 12, },
  { 1, 3, 4, 5, 6, 8, 12, },
  { 0, 1, 3, 4, 5, 6, 8, 12, },
  { 2, 3, 4, 5, 6, 8, 12, },
  { 0, 2, 3, 4, 5, 6, 8, 12, },
  { 1, 2, 3, 4, 5, 6, 8, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 12, },
  { 7, 8, 12, },
  { 0, 7, 8, 12, },
  { 1, 7, 8, 12, },
  { 0, 1, 7, 8, 12, },
  { 2, 7, 8, 12, },
  { 0, 2, 7, 8, 12, },
  { 1, 2, 7, 8, 12, },
  { 0, 1, 2, 7, 8, 12, },
  { 3, 7, 8, 12, },
  { 0, 3, 7, 8, 12, },
  { 1, 3, 7, 8, 12, },
  { 0, 1, 3, 7, 8, 12, },
  { 2, 3, 7, 8, 12, },
  { 0, 2, 3, 7, 8, 12, },
  { 1, 2, 3, 7, 8, 12, },
  { 0, 1, 2, 3, 7, 8, 12, },
  { 4, 7, 8, 12, },
  { 0, 4, 7, 8, 12, },
  { 1, 4, 7, 8, 12, },
  { 0, 1, 4, 7, 8, 12, },
  { 2, 4, 7, 8, 12, },
  { 0, 2, 4, 7, 8, 12, },
  { 1, 2, 4, 7, 8, 12, },
  { 0, 1, 2, 4, 7, 8, 12, },
  { 3, 4, 7, 8, 12, },
  { 0, 3, 4, 7, 8, 12, },
  { 1, 3, 4, 7, 8, 12, },
  { 0, 1, 3, 4, 7, 8, 12, },
  { 2, 3, 4, 7, 8, 12, },
  { 0, 2, 3, 4, 7, 8, 12, },
  { 1, 2, 3, 4, 7, 8, 12, },
  { 0, 1, 2, 3, 4, 7, 8, 12, },
  { 5, 7, 8, 12, },
  { 0, 5, 7, 8, 12, },
  { 1, 5, 7, 8, 12, },
  { 0, 1, 5, 7, 8, 12, },
  { 2, 5, 7, 8, 12, },
  { 0, 2, 5, 7, 8, 12, },
  { 1, 2, 5, 7, 8, 12, },
  { 0, 1, 2, 5, 7, 8, 12, },
  { 3, 5, 7, 8, 12, },
  { 0, 3, 5, 7, 8, 12, },
  { 1, 3, 5, 7, 8, 12, },
  { 0, 1, 3, 5, 7, 8, 12, },
  { 2, 3, 5, 7, 8, 12, },
  { 0, 2, 3, 5, 7, 8, 12, },
  { 1, 2, 3, 5, 7, 8, 12, },
  { 0, 1, 2, 3, 5, 7, 8, 12, },
  { 4, 5, 7, 8, 12, },
  { 0, 4, 5, 7, 8, 12, },
  { 1, 4, 5, 7, 8, 12, },
  { 0, 1, 4, 5, 7, 8, 12, },
  { 2, 4, 5, 7, 8, 12, },
  { 0, 2, 4, 5, 7, 8, 12, },
  { 1, 2, 4, 5, 7, 8, 12, },
  { 0, 1, 2, 4, 5, 7, 8, 12, },
  { 3, 4, 5, 7, 8, 12, },
  { 0, 3, 4, 5, 7, 8, 12, },
  { 1, 3, 4, 5, 7, 8, 12, },
  { 0, 1, 3, 4, 5, 7, 8, 12, },
  { 2, 3, 4, 5, 7, 8, 12, },
  { 0, 2, 3, 4, 5, 7, 8, 12, },
  { 1, 2, 3, 4, 5, 7, 8, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 12, },
  { 6, 7, 8, 12, },
  { 0, 6, 7, 8, 12, },
  { 1, 6, 7, 8, 12, },
  { 0, 1, 6, 7, 8, 12, },
  { 2, 6, 7, 8, 12, },
  { 0, 2, 6, 7, 8, 12, },
  { 1, 2, 6, 7, 8, 12, },
  { 0, 1, 2, 6, 7, 8, 12, },
  { 3, 6, 7, 8, 12, },
  { 0, 3, 6, 7, 8, 12, },
  { 1, 3, 6, 7, 8, 12, },
  { 0, 1, 3, 6, 7, 8, 12, },
  { 2, 3, 6, 7, 8, 12, },
  { 0, 2, 3, 6, 7, 8, 12, },
  { 1, 2, 3, 6, 7, 8, 12, },
  { 0, 1, 2, 3, 6, 7, 8, 12, },
  { 4, 6, 7, 8, 12, },
  { 0, 4, 6, 7, 8, 12, },
  { 1, 4, 6, 7, 8, 12, },
  { 0, 1, 4, 6, 7, 8, 12, },
  { 2, 4, 6, 7, 8, 12, },
  { 0, 2, 4, 6, 7, 8, 12, },
  { 1, 2, 4, 6, 7, 8, 12, },
  { 0, 1, 2, 4, 6, 7, 8, 12, },
  { 3, 4, 6, 7, 8, 12, },
  { 0, 3, 4, 6, 7, 8, 12, },
  { 1, 3, 4, 6, 7, 8, 12, },
  { 0, 1, 3, 4, 6, 7, 8, 12, },
  { 2, 3, 4, 6, 7, 8, 12, },
  { 0, 2, 3, 4, 6, 7, 8, 12, },
  { 1, 2, 3, 4, 6, 7, 8, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 12, },
  { 5, 6, 7, 8, 12, },
  { 0, 5, 6, 7, 8, 12, },
  { 1, 5, 6, 7, 8, 12, },
  { 0, 1, 5, 6, 7, 8, 12, },
  { 2, 5, 6, 7, 8, 12, },
  { 0, 2, 5, 6, 7, 8, 12, },
  { 1, 2, 5, 6, 7, 8, 12, },
  { 0, 1, 2, 5, 6, 7, 8, 12, },
  { 3, 5, 6, 7, 8, 12, },
  { 0, 3, 5, 6, 7, 8, 12, },
  { 1, 3, 5, 6, 7, 8, 12, },
  { 0, 1, 3, 5, 6, 7, 8, 12, },
  { 2, 3, 5, 6, 7, 8, 12, },
  { 0, 2, 3, 5, 6, 7, 8, 12, },
  { 1, 2, 3, 5, 6, 7, 8, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 12, },
  { 4, 5, 6, 7, 8, 12, },
  { 0, 4, 5, 6, 7, 8, 12, },
  { 1, 4, 5, 6, 7, 8, 12, },
  { 0, 1, 4, 5, 6, 7, 8, 12, },
  { 2, 4, 5, 6, 7, 8, 12, },
  { 0, 2, 4, 5, 6, 7, 8, 12, },
  { 1, 2, 4, 5, 6, 7, 8, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 12, },
  { 3, 4, 5, 6, 7, 8, 12, },
  { 0, 3, 4, 5, 6, 7, 8, 12, },
  { 1, 3, 4, 5, 6, 7, 8, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 12, },
  { 2, 3, 4, 5, 6, 7, 8, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 12, },
  { 9, 12, },
  { 0, 9, 12, },
  { 1, 9, 12, },
  { 0, 1, 9, 12, },
  { 2, 9, 12, },
  { 0, 2, 9, 12, },
  { 1, 2, 9, 12, },
  { 0, 1, 2, 9, 12, },
  { 3, 9, 12, },
  { 0, 3, 9, 12, },
  { 1, 3, 9, 12, },
  { 0, 1, 3, 9, 12, },
  { 2, 3, 9, 12, },
  { 0, 2, 3, 9, 12, },
  { 1, 2, 3, 9, 12, },
  { 0, 1, 2, 3, 9, 12, },
  { 4, 9, 12, },
  { 0, 4, 9, 12, },
  { 1, 4, 9, 12, },
  { 0, 1, 4, 9, 12, },
  { 2, 4, 9, 12, },
  { 0, 2, 4, 9, 12, },
  { 1, 2, 4, 9, 12, },
  { 0, 1, 2, 4, 9, 12, },
  { 3, 4, 9, 12, },
  { 0, 3, 4, 9, 12, },
  { 1, 3, 4, 9, 12, },
  { 0, 1, 3, 4, 9, 12, },
  { 2, 3, 4, 9, 12, },
  { 0, 2, 3, 4, 9, 12, },
  { 1, 2, 3, 4, 9, 12, },
  { 0, 1, 2, 3, 4, 9, 12, },
  { 5, 9, 12, },
  { 0, 5, 9, 12, },
  { 1, 5, 9, 12, },
  { 0, 1, 5, 9, 12, },
  { 2, 5, 9, 12, },
  { 0, 2, 5, 9, 12, },
  { 1, 2, 5, 9, 12, },
  { 0, 1, 2, 5, 9, 12, },
  { 3, 5, 9, 12, },
  { 0, 3, 5, 9, 12, },
  { 1, 3, 5, 9, 12, },
  { 0, 1, 3, 5, 9, 12, },
  { 2, 3, 5, 9, 12, },
  { 0, 2, 3, 5, 9, 12, },
  { 1, 2, 3, 5, 9, 12, },
  { 0, 1, 2, 3, 5, 9, 12, },
  { 4, 5, 9, 12, },
  { 0, 4, 5, 9, 12, },
  { 1, 4, 5, 9, 12, },
  { 0, 1, 4, 5, 9, 12, },
  { 2, 4, 5, 9, 12, },
  { 0, 2, 4, 5, 9, 12, },
  { 1, 2, 4, 5, 9, 12, },
  { 0, 1, 2, 4, 5, 9, 12, },
  { 3, 4, 5, 9, 12, },
  { 0, 3, 4, 5, 9, 12, },
  { 1, 3, 4, 5, 9, 12, },
  { 0, 1, 3, 4, 5, 9, 12, },
  { 2, 3, 4, 5, 9, 12, },
  { 0, 2, 3, 4, 5, 9, 12, },
  { 1, 2, 3, 4, 5, 9, 12, },
  { 0, 1, 2, 3, 4, 5, 9, 12, },
  { 6, 9, 12, },
  { 0, 6, 9, 12, },
  { 1, 6, 9, 12, },
  { 0, 1, 6, 9, 12, },
  { 2, 6, 9, 12, },
  { 0, 2, 6, 9, 12, },
  { 1, 2, 6, 9, 12, },
  { 0, 1, 2, 6, 9, 12, },
  { 3, 6, 9, 12, },
  { 0, 3, 6, 9, 12, },
  { 1, 3, 6, 9, 12, },
  { 0, 1, 3, 6, 9, 12, },
  { 2, 3, 6, 9, 12, },
  { 0, 2, 3, 6, 9, 12, },
  { 1, 2, 3, 6, 9, 12, },
  { 0, 1, 2, 3, 6, 9, 12, },
  { 4, 6, 9, 12, },
  { 0, 4, 6, 9, 12, },
  { 1, 4, 6, 9, 12, },
  { 0, 1, 4, 6, 9, 12, },
  { 2, 4, 6, 9, 12, },
  { 0, 2, 4, 6, 9, 12, },
  { 1, 2, 4, 6, 9, 12, },
  { 0, 1, 2, 4, 6, 9, 12, },
  { 3, 4, 6, 9, 12, },
  { 0, 3, 4, 6, 9, 12, },
  { 1, 3, 4, 6, 9, 12, },
  { 0, 1, 3, 4, 6, 9, 12, },
  { 2, 3, 4, 6, 9, 12, },
  { 0, 2, 3, 4, 6, 9, 12, },
  { 1, 2, 3, 4, 6, 9, 12, },
  { 0, 1, 2, 3, 4, 6, 9, 12, },
  { 5, 6, 9, 12, },
  { 0, 5, 6, 9, 12, },
  { 1, 5, 6, 9, 12, },
  { 0, 1, 5, 6, 9, 12, },
  { 2, 5, 6, 9, 12, },
  { 0, 2, 5, 6, 9, 12, },
  { 1, 2, 5, 6, 9, 12, },
  { 0, 1, 2, 5, 6, 9, 12, },
  { 3, 5, 6, 9, 12, },
  { 0, 3, 5, 6, 9, 12, },
  { 1, 3, 5, 6, 9, 12, },
  { 0, 1, 3, 5, 6, 9, 12, },
  { 2, 3, 5, 6, 9, 12, },
  { 0, 2, 3, 5, 6, 9, 12, },
  { 1, 2, 3, 5, 6, 9, 12, },
  { 0, 1, 2, 3, 5, 6, 9, 12, },
  { 4, 5, 6, 9, 12, },
  { 0, 4, 5, 6, 9, 12, },
  { 1, 4, 5, 6, 9, 12, },
  { 0, 1, 4, 5, 6, 9, 12, },
  { 2, 4, 5, 6, 9, 12, },
  { 0, 2, 4, 5, 6, 9, 12, },
  { 1, 2, 4, 5, 6, 9, 12, },
  { 0, 1, 2, 4, 5, 6, 9, 12, },
  { 3, 4, 5, 6, 9, 12, },
  { 0, 3, 4, 5, 6, 9, 12, },
  { 1, 3, 4, 5, 6, 9, 12, },
  { 0, 1, 3, 4, 5, 6, 9, 12, },
  { 2, 3, 4, 5, 6, 9, 12, },
  { 0, 2, 3, 4, 5, 6, 9, 12, },
  { 1, 2, 3, 4, 5, 6, 9, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 12, },
  { 7, 9, 12, },
  { 0, 7, 9, 12, },
  { 1, 7, 9, 12, },
  { 0, 1, 7, 9, 12, },
  { 2, 7, 9, 12, },
  { 0, 2, 7, 9, 12, },
  { 1, 2, 7, 9, 12, },
  { 0, 1, 2, 7, 9, 12, },
  { 3, 7, 9, 12, },
  { 0, 3, 7, 9, 12, },
  { 1, 3, 7, 9, 12, },
  { 0, 1, 3, 7, 9, 12, },
  { 2, 3, 7, 9, 12, },
  { 0, 2, 3, 7, 9, 12, },
  { 1, 2, 3, 7, 9, 12, },
  { 0, 1, 2, 3, 7, 9, 12, },
  { 4, 7, 9, 12, },
  { 0, 4, 7, 9, 12, },
  { 1, 4, 7, 9, 12, },
  { 0, 1, 4, 7, 9, 12, },
  { 2, 4, 7, 9, 12, },
  { 0, 2, 4, 7, 9, 12, },
  { 1, 2, 4, 7, 9, 12, },
  { 0, 1, 2, 4, 7, 9, 12, },
  { 3, 4, 7, 9, 12, },
  { 0, 3, 4, 7, 9, 12, },
  { 1, 3, 4, 7, 9, 12, },
  { 0, 1, 3, 4, 7, 9, 12, },
  { 2, 3, 4, 7, 9, 12, },
  { 0, 2, 3, 4, 7, 9, 12, },
  { 1, 2, 3, 4, 7, 9, 12, },
  { 0, 1, 2, 3, 4, 7, 9, 12, },
  { 5, 7, 9, 12, },
  { 0, 5, 7, 9, 12, },
  { 1, 5, 7, 9, 12, },
  { 0, 1, 5, 7, 9, 12, },
  { 2, 5, 7, 9, 12, },
  { 0, 2, 5, 7, 9, 12, },
  { 1, 2, 5, 7, 9, 12, },
  { 0, 1, 2, 5, 7, 9, 12, },
  { 3, 5, 7, 9, 12, },
  { 0, 3, 5, 7, 9, 12, },
  { 1, 3, 5, 7, 9, 12, },
  { 0, 1, 3, 5, 7, 9, 12, },
  { 2, 3, 5, 7, 9, 12, },
  { 0, 2, 3, 5, 7, 9, 12, },
  { 1, 2, 3, 5, 7, 9, 12, },
  { 0, 1, 2, 3, 5, 7, 9, 12, },
  { 4, 5, 7, 9, 12, },
  { 0, 4, 5, 7, 9, 12, },
  { 1, 4, 5, 7, 9, 12, },
  { 0, 1, 4, 5, 7, 9, 12, },
  { 2, 4, 5, 7, 9, 12, },
  { 0, 2, 4, 5, 7, 9, 12, },
  { 1, 2, 4, 5, 7, 9, 12, },
  { 0, 1, 2, 4, 5, 7, 9, 12, },
  { 3, 4, 5, 7, 9, 12, },
  { 0, 3, 4, 5, 7, 9, 12, },
  { 1, 3, 4, 5, 7, 9, 12, },
  { 0, 1, 3, 4, 5, 7, 9, 12, },
  { 2, 3, 4, 5, 7, 9, 12, },
  { 0, 2, 3, 4, 5, 7, 9, 12, },
  { 1, 2, 3, 4, 5, 7, 9, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 12, },
  { 6, 7, 9, 12, },
  { 0, 6, 7, 9, 12, },
  { 1, 6, 7, 9, 12, },
  { 0, 1, 6, 7, 9, 12, },
  { 2, 6, 7, 9, 12, },
  { 0, 2, 6, 7, 9, 12, },
  { 1, 2, 6, 7, 9, 12, },
  { 0, 1, 2, 6, 7, 9, 12, },
  { 3, 6, 7, 9, 12, },
  { 0, 3, 6, 7, 9, 12, },
  { 1, 3, 6, 7, 9, 12, },
  { 0, 1, 3, 6, 7, 9, 12, },
  { 2, 3, 6, 7, 9, 12, },
  { 0, 2, 3, 6, 7, 9, 12, },
  { 1, 2, 3, 6, 7, 9, 12, },
  { 0, 1, 2, 3, 6, 7, 9, 12, },
  { 4, 6, 7, 9, 12, },
  { 0, 4, 6, 7, 9, 12, },
  { 1, 4, 6, 7, 9, 12, },
  { 0, 1, 4, 6, 7, 9, 12, },
  { 2, 4, 6, 7, 9, 12, },
  { 0, 2, 4, 6, 7, 9, 12, },
  { 1, 2, 4, 6, 7, 9, 12, },
  { 0, 1, 2, 4, 6, 7, 9, 12, },
  { 3, 4, 6, 7, 9, 12, },
  { 0, 3, 4, 6, 7, 9, 12, },
  { 1, 3, 4, 6, 7, 9, 12, },
  { 0, 1, 3, 4, 6, 7, 9, 12, },
  { 2, 3, 4, 6, 7, 9, 12, },
  { 0, 2, 3, 4, 6, 7, 9, 12, },
  { 1, 2, 3, 4, 6, 7, 9, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 12, },
  { 5, 6, 7, 9, 12, },
  { 0, 5, 6, 7, 9, 12, },
  { 1, 5, 6, 7, 9, 12, },
  { 0, 1, 5, 6, 7, 9, 12, },
  { 2, 5, 6, 7, 9, 12, },
  { 0, 2, 5, 6, 7, 9, 12, },
  { 1, 2, 5, 6, 7, 9, 12, },
  { 0, 1, 2, 5, 6, 7, 9, 12, },
  { 3, 5, 6, 7, 9, 12, },
  { 0, 3, 5, 6, 7, 9, 12, },
  { 1, 3, 5, 6, 7, 9, 12, },
  { 0, 1, 3, 5, 6, 7, 9, 12, },
  { 2, 3, 5, 6, 7, 9, 12, },
  { 0, 2, 3, 5, 6, 7, 9, 12, },
  { 1, 2, 3, 5, 6, 7, 9, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 12, },
  { 4, 5, 6, 7, 9, 12, },
  { 0, 4, 5, 6, 7, 9, 12, },
  { 1, 4, 5, 6, 7, 9, 12, },
  { 0, 1, 4, 5, 6, 7, 9, 12, },
  { 2, 4, 5, 6, 7, 9, 12, },
  { 0, 2, 4, 5, 6, 7, 9, 12, },
  { 1, 2, 4, 5, 6, 7, 9, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 12, },
  { 3, 4, 5, 6, 7, 9, 12, },
  { 0, 3, 4, 5, 6, 7, 9, 12, },
  { 1, 3, 4, 5, 6, 7, 9, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 12, },
  { 2, 3, 4, 5, 6, 7, 9, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 12, },
  { 8, 9, 12, },
  { 0, 8, 9, 12, },
  { 1, 8, 9, 12, },
  { 0, 1, 8, 9, 12, },
  { 2, 8, 9, 12, },
  { 0, 2, 8, 9, 12, },
  { 1, 2, 8, 9, 12, },
  { 0, 1, 2, 8, 9, 12, },
  { 3, 8, 9, 12, },
  { 0, 3, 8, 9, 12, },
  { 1, 3, 8, 9, 12, },
  { 0, 1, 3, 8, 9, 12, },
  { 2, 3, 8, 9, 12, },
  { 0, 2, 3, 8, 9, 12, },
  { 1, 2, 3, 8, 9, 12, },
  { 0, 1, 2, 3, 8, 9, 12, },
  { 4, 8, 9, 12, },
  { 0, 4, 8, 9, 12, },
  { 1, 4, 8, 9, 12, },
  { 0, 1, 4, 8, 9, 12, },
  { 2, 4, 8, 9, 12, },
  { 0, 2, 4, 8, 9, 12, },
  { 1, 2, 4, 8, 9, 12, },
  { 0, 1, 2, 4, 8, 9, 12, },
  { 3, 4, 8, 9, 12, },
  { 0, 3, 4, 8, 9, 12, },
  { 1, 3, 4, 8, 9, 12, },
  { 0, 1, 3, 4, 8, 9, 12, },
  { 2, 3, 4, 8, 9, 12, },
  { 0, 2, 3, 4, 8, 9, 12, },
  { 1, 2, 3, 4, 8, 9, 12, },
  { 0, 1, 2, 3, 4, 8, 9, 12, },
  { 5, 8, 9, 12, },
  { 0, 5, 8, 9, 12, },
  { 1, 5, 8, 9, 12, },
  { 0, 1, 5, 8, 9, 12, },
  { 2, 5, 8, 9, 12, },
  { 0, 2, 5, 8, 9, 12, },
  { 1, 2, 5, 8, 9, 12, },
  { 0, 1, 2, 5, 8, 9, 12, },
  { 3, 5, 8, 9, 12, },
  { 0, 3, 5, 8, 9, 12, },
  { 1, 3, 5, 8, 9, 12, },
  { 0, 1, 3, 5, 8, 9, 12, },
  { 2, 3, 5, 8, 9, 12, },
  { 0, 2, 3, 5, 8, 9, 12, },
  { 1, 2, 3, 5, 8, 9, 12, },
  { 0, 1, 2, 3, 5, 8, 9, 12, },
  { 4, 5, 8, 9, 12, },
  { 0, 4, 5, 8, 9, 12, },
  { 1, 4, 5, 8, 9, 12, },
  { 0, 1, 4, 5, 8, 9, 12, },
  { 2, 4, 5, 8, 9, 12, },
  { 0, 2, 4, 5, 8, 9, 12, },
  { 1, 2, 4, 5, 8, 9, 12, },
  { 0, 1, 2, 4, 5, 8, 9, 12, },
  { 3, 4, 5, 8, 9, 12, },
  { 0, 3, 4, 5, 8, 9, 12, },
  { 1, 3, 4, 5, 8, 9, 12, },
  { 0, 1, 3, 4, 5, 8, 9, 12, },
  { 2, 3, 4, 5, 8, 9, 12, },
  { 0, 2, 3, 4, 5, 8, 9, 12, },
  { 1, 2, 3, 4, 5, 8, 9, 12, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 12, },
  { 6, 8, 9, 12, },
  { 0, 6, 8, 9, 12, },
  { 1, 6, 8, 9, 12, },
  { 0, 1, 6, 8, 9, 12, },
  { 2, 6, 8, 9, 12, },
  { 0, 2, 6, 8, 9, 12, },
  { 1, 2, 6, 8, 9, 12, },
  { 0, 1, 2, 6, 8, 9, 12, },
  { 3, 6, 8, 9, 12, },
  { 0, 3, 6, 8, 9, 12, },
  { 1, 3, 6, 8, 9, 12, },
  { 0, 1, 3, 6, 8, 9, 12, },
  { 2, 3, 6, 8, 9, 12, },
  { 0, 2, 3, 6, 8, 9, 12, },
  { 1, 2, 3, 6, 8, 9, 12, },
  { 0, 1, 2, 3, 6, 8, 9, 12, },
  { 4, 6, 8, 9, 12, },
  { 0, 4, 6, 8, 9, 12, },
  { 1, 4, 6, 8, 9, 12, },
  { 0, 1, 4, 6, 8, 9, 12, },
  { 2, 4, 6, 8, 9, 12, },
  { 0, 2, 4, 6, 8, 9, 12, },
  { 1, 2, 4, 6, 8, 9, 12, },
  { 0, 1, 2, 4, 6, 8, 9, 12, },
  { 3, 4, 6, 8, 9, 12, },
  { 0, 3, 4, 6, 8, 9, 12, },
  { 1, 3, 4, 6, 8, 9, 12, },
  { 0, 1, 3, 4, 6, 8, 9, 12, },
  { 2, 3, 4, 6, 8, 9, 12, },
  { 0, 2, 3, 4, 6, 8, 9, 12, },
  { 1, 2, 3, 4, 6, 8, 9, 12, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 12, },
  { 5, 6, 8, 9, 12, },
  { 0, 5, 6, 8, 9, 12, },
  { 1, 5, 6, 8, 9, 12, },
  { 0, 1, 5, 6, 8, 9, 12, },
  { 2, 5, 6, 8, 9, 12, },
  { 0, 2, 5, 6, 8, 9, 12, },
  { 1, 2, 5, 6, 8, 9, 12, },
  { 0, 1, 2, 5, 6, 8, 9, 12, },
  { 3, 5, 6, 8, 9, 12, },
  { 0, 3, 5, 6, 8, 9, 12, },
  { 1, 3, 5, 6, 8, 9, 12, },
  { 0, 1, 3, 5, 6, 8, 9, 12, },
  { 2, 3, 5, 6, 8, 9, 12, },
  { 0, 2, 3, 5, 6, 8, 9, 12, },
  { 1, 2, 3, 5, 6, 8, 9, 12, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 12, },
  { 4, 5, 6, 8, 9, 12, },
  { 0, 4, 5, 6, 8, 9, 12, },
  { 1, 4, 5, 6, 8, 9, 12, },
  { 0, 1, 4, 5, 6, 8, 9, 12, },
  { 2, 4, 5, 6, 8, 9, 12, },
  { 0, 2, 4, 5, 6, 8, 9, 12, },
  { 1, 2, 4, 5, 6, 8, 9, 12, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 12, },
  { 3, 4, 5, 6, 8, 9, 12, },
  { 0, 3, 4, 5, 6, 8, 9, 12, },
  { 1, 3, 4, 5, 6, 8, 9, 12, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 12, },
  { 2, 3, 4, 5, 6, 8, 9, 12, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 12, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 12, },
  { 7, 8, 9, 12, },
  { 0, 7, 8, 9, 12, },
  { 1, 7, 8, 9, 12, },
  { 0, 1, 7, 8, 9, 12, },
  { 2, 7, 8, 9, 12, },
  { 0, 2, 7, 8, 9, 12, },
  { 1, 2, 7, 8, 9, 12, },
  { 0, 1, 2, 7, 8, 9, 12, },
  { 3, 7, 8, 9, 12, },
  { 0, 3, 7, 8, 9, 12, },
  { 1, 3, 7, 8, 9, 12, },
  { 0, 1, 3, 7, 8, 9, 12, },
  { 2, 3, 7, 8, 9, 12, },
  { 0, 2, 3, 7, 8, 9, 12, },
  { 1, 2, 3, 7, 8, 9, 12, },
  { 0, 1, 2, 3, 7, 8, 9, 12, },
  { 4, 7, 8, 9, 12, },
  { 0, 4, 7, 8, 9, 12, },
  { 1, 4, 7, 8, 9, 12, },
  { 0, 1, 4, 7, 8, 9, 12, },
  { 2, 4, 7, 8, 9, 12, },
  { 0, 2, 4, 7, 8, 9, 12, },
  { 1, 2, 4, 7, 8, 9, 12, },
  { 0, 1, 2, 4, 7, 8, 9, 12, },
  { 3, 4, 7, 8, 9, 12, },
  { 0, 3, 4, 7, 8, 9, 12, },
  { 1, 3, 4, 7, 8, 9, 12, },
  { 0, 1, 3, 4, 7, 8, 9, 12, },
  { 2, 3, 4, 7, 8, 9, 12, },
  { 0, 2, 3, 4, 7, 8, 9, 12, },
  { 1, 2, 3, 4, 7, 8, 9, 12, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 12, },
  { 5, 7, 8, 9, 12, },
  { 0, 5, 7, 8, 9, 12, },
  { 1, 5, 7, 8, 9, 12, },
  { 0, 1, 5, 7, 8, 9, 12, },
  { 2, 5, 7, 8, 9, 12, },
  { 0, 2, 5, 7, 8, 9, 12, },
  { 1, 2, 5, 7, 8, 9, 12, },
  { 0, 1, 2, 5, 7, 8, 9, 12, },
  { 3, 5, 7, 8, 9, 12, },
  { 0, 3, 5, 7, 8, 9, 12, },
  { 1, 3, 5, 7, 8, 9, 12, },
  { 0, 1, 3, 5, 7, 8, 9, 12, },
  { 2, 3, 5, 7, 8, 9, 12, },
  { 0, 2, 3, 5, 7, 8, 9, 12, },
  { 1, 2, 3, 5, 7, 8, 9, 12, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 12, },
  { 4, 5, 7, 8, 9, 12, },
  { 0, 4, 5, 7, 8, 9, 12, },
  { 1, 4, 5, 7, 8, 9, 12, },
  { 0, 1, 4, 5, 7, 8, 9, 12, },
  { 2, 4, 5, 7, 8, 9, 12, },
  { 0, 2, 4, 5, 7, 8, 9, 12, },
  { 1, 2, 4, 5, 7, 8, 9, 12, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 12, },
  { 3, 4, 5, 7, 8, 9, 12, },
  { 0, 3, 4, 5, 7, 8, 9, 12, },
  { 1, 3, 4, 5, 7, 8, 9, 12, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 12, },
  { 2, 3, 4, 5, 7, 8, 9, 12, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 12, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 12, },
  { 6, 7, 8, 9, 12, },
  { 0, 6, 7, 8, 9, 12, },
  { 1, 6, 7, 8, 9, 12, },
  { 0, 1, 6, 7, 8, 9, 12, },
  { 2, 6, 7, 8, 9, 12, },
  { 0, 2, 6, 7, 8, 9, 12, },
  { 1, 2, 6, 7, 8, 9, 12, },
  { 0, 1, 2, 6, 7, 8, 9, 12, },
  { 3, 6, 7, 8, 9, 12, },
  { 0, 3, 6, 7, 8, 9, 12, },
  { 1, 3, 6, 7, 8, 9, 12, },
  { 0, 1, 3, 6, 7, 8, 9, 12, },
  { 2, 3, 6, 7, 8, 9, 12, },
  { 0, 2, 3, 6, 7, 8, 9, 12, },
  { 1, 2, 3, 6, 7, 8, 9, 12, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 12, },
  { 4, 6, 7, 8, 9, 12, },
  { 0, 4, 6, 7, 8, 9, 12, },
  { 1, 4, 6, 7, 8, 9, 12, },
  { 0, 1, 4, 6, 7, 8, 9, 12, },
  { 2, 4, 6, 7, 8, 9, 12, },
  { 0, 2, 4, 6, 7, 8, 9, 12, },
  { 1, 2, 4, 6, 7, 8, 9, 12, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 12, },
  { 3, 4, 6, 7, 8, 9, 12, },
  { 0, 3, 4, 6, 7, 8, 9, 12, },
  { 1, 3, 4, 6, 7, 8, 9, 12, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 12, },
  { 2, 3, 4, 6, 7, 8, 9, 12, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 12, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 12, },
  { 5, 6, 7, 8, 9, 12, },
  { 0, 5, 6, 7, 8, 9, 12, },
  { 1, 5, 6, 7, 8, 9, 12, },
  { 0, 1, 5, 6, 7, 8, 9, 12, },
  { 2, 5, 6, 7, 8, 9, 12, },
  { 0, 2, 5, 6, 7, 8, 9, 12, },
  { 1, 2, 5, 6, 7, 8, 9, 12, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 12, },
  { 3, 5, 6, 7, 8, 9, 12, },
  { 0, 3, 5, 6, 7, 8, 9, 12, },
  { 1, 3, 5, 6, 7, 8, 9, 12, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 12, },
  { 2, 3, 5, 6, 7, 8, 9, 12, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 12, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 12, },
  { 4, 5, 6, 7, 8, 9, 12, },
  { 0, 4, 5, 6, 7, 8, 9, 12, },
  { 1, 4, 5, 6, 7, 8, 9, 12, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 12, },
  { 2, 4, 5, 6, 7, 8, 9, 12, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 12, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 12, },
  { 3, 4, 5, 6, 7, 8, 9, 12, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 12, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 12, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, },
  { 10, 12, },
  { 0, 10, 12, },
  { 1, 10, 12, },
  { 0, 1, 10, 12, },
  { 2, 10, 12, },
  { 0, 2, 10, 12, },
  { 1, 2, 10, 12, },
  { 0, 1, 2, 10, 12, },
  { 3, 10, 12, },
  { 0, 3, 10, 12, },
  { 1, 3, 10, 12, },
  { 0, 1, 3, 10, 12, },
  { 2, 3, 10, 12, },
  { 0, 2, 3, 10, 12, },
  { 1, 2, 3, 10, 12, },
  { 0, 1, 2, 3, 10, 12, },
  { 4, 10, 12, },
  { 0, 4, 10, 12, },
  { 1, 4, 10, 12, },
  { 0, 1, 4, 10, 12, },
  { 2, 4, 10, 12, },
  { 0, 2, 4, 10, 12, },
  { 1, 2, 4, 10, 12, },
  { 0, 1, 2, 4, 10, 12, },
  { 3, 4, 10, 12, },
  { 0, 3, 4, 10, 12, },
  { 1, 3, 4, 10, 12, },
  { 0, 1, 3, 4, 10, 12, },
  { 2, 3, 4, 10, 12, },
  { 0, 2, 3, 4, 10, 12, },
  { 1, 2, 3, 4, 10, 12, },
  { 0, 1, 2, 3, 4, 10, 12, },
  { 5, 10, 12, },
  { 0, 5, 10, 12, },
  { 1, 5, 10, 12, },
  { 0, 1, 5, 10, 12, },
  { 2, 5, 10, 12, },
  { 0, 2, 5, 10, 12, },
  { 1, 2, 5, 10, 12, },
  { 0, 1, 2, 5, 10, 12, },
  { 3, 5, 10, 12, },
  { 0, 3, 5, 10, 12, },
  { 1, 3, 5, 10, 12, },
  { 0, 1, 3, 5, 10, 12, },
  { 2, 3, 5, 10, 12, },
  { 0, 2, 3, 5, 10, 12, },
  { 1, 2, 3, 5, 10, 12, },
  { 0, 1, 2, 3, 5, 10, 12, },
  { 4, 5, 10, 12, },
  { 0, 4, 5, 10, 12, },
  { 1, 4, 5, 10, 12, },
  { 0, 1, 4, 5, 10, 12, },
  { 2, 4, 5, 10, 12, },
  { 0, 2, 4, 5, 10, 12, },
  { 1, 2, 4, 5, 10, 12, },
  { 0, 1, 2, 4, 5, 10, 12, },
  { 3, 4, 5, 10, 12, },
  { 0, 3, 4, 5, 10, 12, },
  { 1, 3, 4, 5, 10, 12, },
  { 0, 1, 3, 4, 5, 10, 12, },
  { 2, 3, 4, 5, 10, 12, },
  { 0, 2, 3, 4, 5, 10, 12, },
  { 1, 2, 3, 4, 5, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 10, 12, },
  { 6, 10, 12, },
  { 0, 6, 10, 12, },
  { 1, 6, 10, 12, },
  { 0, 1, 6, 10, 12, },
  { 2, 6, 10, 12, },
  { 0, 2, 6, 10, 12, },
  { 1, 2, 6, 10, 12, },
  { 0, 1, 2, 6, 10, 12, },
  { 3, 6, 10, 12, },
  { 0, 3, 6, 10, 12, },
  { 1, 3, 6, 10, 12, },
  { 0, 1, 3, 6, 10, 12, },
  { 2, 3, 6, 10, 12, },
  { 0, 2, 3, 6, 10, 12, },
  { 1, 2, 3, 6, 10, 12, },
  { 0, 1, 2, 3, 6, 10, 12, },
  { 4, 6, 10, 12, },
  { 0, 4, 6, 10, 12, },
  { 1, 4, 6, 10, 12, },
  { 0, 1, 4, 6, 10, 12, },
  { 2, 4, 6, 10, 12, },
  { 0, 2, 4, 6, 10, 12, },
  { 1, 2, 4, 6, 10, 12, },
  { 0, 1, 2, 4, 6, 10, 12, },
  { 3, 4, 6, 10, 12, },
  { 0, 3, 4, 6, 10, 12, },
  { 1, 3, 4, 6, 10, 12, },
  { 0, 1, 3, 4, 6, 10, 12, },
  { 2, 3, 4, 6, 10, 12, },
  { 0, 2, 3, 4, 6, 10, 12, },
  { 1, 2, 3, 4, 6, 10, 12, },
  { 0, 1, 2, 3, 4, 6, 10, 12, },
  { 5, 6, 10, 12, },
  { 0, 5, 6, 10, 12, },
  { 1, 5, 6, 10, 12, },
  { 0, 1, 5, 6, 10, 12, },
  { 2, 5, 6, 10, 12, },
  { 0, 2, 5, 6, 10, 12, },
  { 1, 2, 5, 6, 10, 12, },
  { 0, 1, 2, 5, 6, 10, 12, },
  { 3, 5, 6, 10, 12, },
  { 0, 3, 5, 6, 10, 12, },
  { 1, 3, 5, 6, 10, 12, },
  { 0, 1, 3, 5, 6, 10, 12, },
  { 2, 3, 5, 6, 10, 12, },
  { 0, 2, 3, 5, 6, 10, 12, },
  { 1, 2, 3, 5, 6, 10, 12, },
  { 0, 1, 2, 3, 5, 6, 10, 12, },
  { 4, 5, 6, 10, 12, },
  { 0, 4, 5, 6, 10, 12, },
  { 1, 4, 5, 6, 10, 12, },
  { 0, 1, 4, 5, 6, 10, 12, },
  { 2, 4, 5, 6, 10, 12, },
  { 0, 2, 4, 5, 6, 10, 12, },
  { 1, 2, 4, 5, 6, 10, 12, },
  { 0, 1, 2, 4, 5, 6, 10, 12, },
  { 3, 4, 5, 6, 10, 12, },
  { 0, 3, 4, 5, 6, 10, 12, },
  { 1, 3, 4, 5, 6, 10, 12, },
  { 0, 1, 3, 4, 5, 6, 10, 12, },
  { 2, 3, 4, 5, 6, 10, 12, },
  { 0, 2, 3, 4, 5, 6, 10, 12, },
  { 1, 2, 3, 4, 5, 6, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 12, },
  { 7, 10, 12, },
  { 0, 7, 10, 12, },
  { 1, 7, 10, 12, },
  { 0, 1, 7, 10, 12, },
  { 2, 7, 10, 12, },
  { 0, 2, 7, 10, 12, },
  { 1, 2, 7, 10, 12, },
  { 0, 1, 2, 7, 10, 12, },
  { 3, 7, 10, 12, },
  { 0, 3, 7, 10, 12, },
  { 1, 3, 7, 10, 12, },
  { 0, 1, 3, 7, 10, 12, },
  { 2, 3, 7, 10, 12, },
  { 0, 2, 3, 7, 10, 12, },
  { 1, 2, 3, 7, 10, 12, },
  { 0, 1, 2, 3, 7, 10, 12, },
  { 4, 7, 10, 12, },
  { 0, 4, 7, 10, 12, },
  { 1, 4, 7, 10, 12, },
  { 0, 1, 4, 7, 10, 12, },
  { 2, 4, 7, 10, 12, },
  { 0, 2, 4, 7, 10, 12, },
  { 1, 2, 4, 7, 10, 12, },
  { 0, 1, 2, 4, 7, 10, 12, },
  { 3, 4, 7, 10, 12, },
  { 0, 3, 4, 7, 10, 12, },
  { 1, 3, 4, 7, 10, 12, },
  { 0, 1, 3, 4, 7, 10, 12, },
  { 2, 3, 4, 7, 10, 12, },
  { 0, 2, 3, 4, 7, 10, 12, },
  { 1, 2, 3, 4, 7, 10, 12, },
  { 0, 1, 2, 3, 4, 7, 10, 12, },
  { 5, 7, 10, 12, },
  { 0, 5, 7, 10, 12, },
  { 1, 5, 7, 10, 12, },
  { 0, 1, 5, 7, 10, 12, },
  { 2, 5, 7, 10, 12, },
  { 0, 2, 5, 7, 10, 12, },
  { 1, 2, 5, 7, 10, 12, },
  { 0, 1, 2, 5, 7, 10, 12, },
  { 3, 5, 7, 10, 12, },
  { 0, 3, 5, 7, 10, 12, },
  { 1, 3, 5, 7, 10, 12, },
  { 0, 1, 3, 5, 7, 10, 12, },
  { 2, 3, 5, 7, 10, 12, },
  { 0, 2, 3, 5, 7, 10, 12, },
  { 1, 2, 3, 5, 7, 10, 12, },
  { 0, 1, 2, 3, 5, 7, 10, 12, },
  { 4, 5, 7, 10, 12, },
  { 0, 4, 5, 7, 10, 12, },
  { 1, 4, 5, 7, 10, 12, },
  { 0, 1, 4, 5, 7, 10, 12, },
  { 2, 4, 5, 7, 10, 12, },
  { 0, 2, 4, 5, 7, 10, 12, },
  { 1, 2, 4, 5, 7, 10, 12, },
  { 0, 1, 2, 4, 5, 7, 10, 12, },
  { 3, 4, 5, 7, 10, 12, },
  { 0, 3, 4, 5, 7, 10, 12, },
  { 1, 3, 4, 5, 7, 10, 12, },
  { 0, 1, 3, 4, 5, 7, 10, 12, },
  { 2, 3, 4, 5, 7, 10, 12, },
  { 0, 2, 3, 4, 5, 7, 10, 12, },
  { 1, 2, 3, 4, 5, 7, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 12, },
  { 6, 7, 10, 12, },
  { 0, 6, 7, 10, 12, },
  { 1, 6, 7, 10, 12, },
  { 0, 1, 6, 7, 10, 12, },
  { 2, 6, 7, 10, 12, },
  { 0, 2, 6, 7, 10, 12, },
  { 1, 2, 6, 7, 10, 12, },
  { 0, 1, 2, 6, 7, 10, 12, },
  { 3, 6, 7, 10, 12, },
  { 0, 3, 6, 7, 10, 12, },
  { 1, 3, 6, 7, 10, 12, },
  { 0, 1, 3, 6, 7, 10, 12, },
  { 2, 3, 6, 7, 10, 12, },
  { 0, 2, 3, 6, 7, 10, 12, },
  { 1, 2, 3, 6, 7, 10, 12, },
  { 0, 1, 2, 3, 6, 7, 10, 12, },
  { 4, 6, 7, 10, 12, },
  { 0, 4, 6, 7, 10, 12, },
  { 1, 4, 6, 7, 10, 12, },
  { 0, 1, 4, 6, 7, 10, 12, },
  { 2, 4, 6, 7, 10, 12, },
  { 0, 2, 4, 6, 7, 10, 12, },
  { 1, 2, 4, 6, 7, 10, 12, },
  { 0, 1, 2, 4, 6, 7, 10, 12, },
  { 3, 4, 6, 7, 10, 12, },
  { 0, 3, 4, 6, 7, 10, 12, },
  { 1, 3, 4, 6, 7, 10, 12, },
  { 0, 1, 3, 4, 6, 7, 10, 12, },
  { 2, 3, 4, 6, 7, 10, 12, },
  { 0, 2, 3, 4, 6, 7, 10, 12, },
  { 1, 2, 3, 4, 6, 7, 10, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 12, },
  { 5, 6, 7, 10, 12, },
  { 0, 5, 6, 7, 10, 12, },
  { 1, 5, 6, 7, 10, 12, },
  { 0, 1, 5, 6, 7, 10, 12, },
  { 2, 5, 6, 7, 10, 12, },
  { 0, 2, 5, 6, 7, 10, 12, },
  { 1, 2, 5, 6, 7, 10, 12, },
  { 0, 1, 2, 5, 6, 7, 10, 12, },
  { 3, 5, 6, 7, 10, 12, },
  { 0, 3, 5, 6, 7, 10, 12, },
  { 1, 3, 5, 6, 7, 10, 12, },
  { 0, 1, 3, 5, 6, 7, 10, 12, },
  { 2, 3, 5, 6, 7, 10, 12, },
  { 0, 2, 3, 5, 6, 7, 10, 12, },
  { 1, 2, 3, 5, 6, 7, 10, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 12, },
  { 4, 5, 6, 7, 10, 12, },
  { 0, 4, 5, 6, 7, 10, 12, },
  { 1, 4, 5, 6, 7, 10, 12, },
  { 0, 1, 4, 5, 6, 7, 10, 12, },
  { 2, 4, 5, 6, 7, 10, 12, },
  { 0, 2, 4, 5, 6, 7, 10, 12, },
  { 1, 2, 4, 5, 6, 7, 10, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 12, },
  { 3, 4, 5, 6, 7, 10, 12, },
  { 0, 3, 4, 5, 6, 7, 10, 12, },
  { 1, 3, 4, 5, 6, 7, 10, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 12, },
  { 2, 3, 4, 5, 6, 7, 10, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 12, },
  { 8, 10, 12, },
  { 0, 8, 10, 12, },
  { 1, 8, 10, 12, },
  { 0, 1, 8, 10, 12, },
  { 2, 8, 10, 12, },
  { 0, 2, 8, 10, 12, },
  { 1, 2, 8, 10, 12, },
  { 0, 1, 2, 8, 10, 12, },
  { 3, 8, 10, 12, },
  { 0, 3, 8, 10, 12, },
  { 1, 3, 8, 10, 12, },
  { 0, 1, 3, 8, 10, 12, },
  { 2, 3, 8, 10, 12, },
  { 0, 2, 3, 8, 10, 12, },
  { 1, 2, 3, 8, 10, 12, },
  { 0, 1, 2, 3, 8, 10, 12, },
  { 4, 8, 10, 12, },
  { 0, 4, 8, 10, 12, },
  { 1, 4, 8, 10, 12, },
  { 0, 1, 4, 8, 10, 12, },
  { 2, 4, 8, 10, 12, },
  { 0, 2, 4, 8, 10, 12, },
  { 1, 2, 4, 8, 10, 12, },
  { 0, 1, 2, 4, 8, 10, 12, },
  { 3, 4, 8, 10, 12, },
  { 0, 3, 4, 8, 10, 12, },
  { 1, 3, 4, 8, 10, 12, },
  { 0, 1, 3, 4, 8, 10, 12, },
  { 2, 3, 4, 8, 10, 12, },
  { 0, 2, 3, 4, 8, 10, 12, },
  { 1, 2, 3, 4, 8, 10, 12, },
  { 0, 1, 2, 3, 4, 8, 10, 12, },
  { 5, 8, 10, 12, },
  { 0, 5, 8, 10, 12, },
  { 1, 5, 8, 10, 12, },
  { 0, 1, 5, 8, 10, 12, },
  { 2, 5, 8, 10, 12, },
  { 0, 2, 5, 8, 10, 12, },
  { 1, 2, 5, 8, 10, 12, },
  { 0, 1, 2, 5, 8, 10, 12, },
  { 3, 5, 8, 10, 12, },
  { 0, 3, 5, 8, 10, 12, },
  { 1, 3, 5, 8, 10, 12, },
  { 0, 1, 3, 5, 8, 10, 12, },
  { 2, 3, 5, 8, 10, 12, },
  { 0, 2, 3, 5, 8, 10, 12, },
  { 1, 2, 3, 5, 8, 10, 12, },
  { 0, 1, 2, 3, 5, 8, 10, 12, },
  { 4, 5, 8, 10, 12, },
  { 0, 4, 5, 8, 10, 12, },
  { 1, 4, 5, 8, 10, 12, },
  { 0, 1, 4, 5, 8, 10, 12, },
  { 2, 4, 5, 8, 10, 12, },
  { 0, 2, 4, 5, 8, 10, 12, },
  { 1, 2, 4, 5, 8, 10, 12, },
  { 0, 1, 2, 4, 5, 8, 10, 12, },
  { 3, 4, 5, 8, 10, 12, },
  { 0, 3, 4, 5, 8, 10, 12, },
  { 1, 3, 4, 5, 8, 10, 12, },
  { 0, 1, 3, 4, 5, 8, 10, 12, },
  { 2, 3, 4, 5, 8, 10, 12, },
  { 0, 2, 3, 4, 5, 8, 10, 12, },
  { 1, 2, 3, 4, 5, 8, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 12, },
  { 6, 8, 10, 12, },
  { 0, 6, 8, 10, 12, },
  { 1, 6, 8, 10, 12, },
  { 0, 1, 6, 8, 10, 12, },
  { 2, 6, 8, 10, 12, },
  { 0, 2, 6, 8, 10, 12, },
  { 1, 2, 6, 8, 10, 12, },
  { 0, 1, 2, 6, 8, 10, 12, },
  { 3, 6, 8, 10, 12, },
  { 0, 3, 6, 8, 10, 12, },
  { 1, 3, 6, 8, 10, 12, },
  { 0, 1, 3, 6, 8, 10, 12, },
  { 2, 3, 6, 8, 10, 12, },
  { 0, 2, 3, 6, 8, 10, 12, },
  { 1, 2, 3, 6, 8, 10, 12, },
  { 0, 1, 2, 3, 6, 8, 10, 12, },
  { 4, 6, 8, 10, 12, },
  { 0, 4, 6, 8, 10, 12, },
  { 1, 4, 6, 8, 10, 12, },
  { 0, 1, 4, 6, 8, 10, 12, },
  { 2, 4, 6, 8, 10, 12, },
  { 0, 2, 4, 6, 8, 10, 12, },
  { 1, 2, 4, 6, 8, 10, 12, },
  { 0, 1, 2, 4, 6, 8, 10, 12, },
  { 3, 4, 6, 8, 10, 12, },
  { 0, 3, 4, 6, 8, 10, 12, },
  { 1, 3, 4, 6, 8, 10, 12, },
  { 0, 1, 3, 4, 6, 8, 10, 12, },
  { 2, 3, 4, 6, 8, 10, 12, },
  { 0, 2, 3, 4, 6, 8, 10, 12, },
  { 1, 2, 3, 4, 6, 8, 10, 12, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 12, },
  { 5, 6, 8, 10, 12, },
  { 0, 5, 6, 8, 10, 12, },
  { 1, 5, 6, 8, 10, 12, },
  { 0, 1, 5, 6, 8, 10, 12, },
  { 2, 5, 6, 8, 10, 12, },
  { 0, 2, 5, 6, 8, 10, 12, },
  { 1, 2, 5, 6, 8, 10, 12, },
  { 0, 1, 2, 5, 6, 8, 10, 12, },
  { 3, 5, 6, 8, 10, 12, },
  { 0, 3, 5, 6, 8, 10, 12, },
  { 1, 3, 5, 6, 8, 10, 12, },
  { 0, 1, 3, 5, 6, 8, 10, 12, },
  { 2, 3, 5, 6, 8, 10, 12, },
  { 0, 2, 3, 5, 6, 8, 10, 12, },
  { 1, 2, 3, 5, 6, 8, 10, 12, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 12, },
  { 4, 5, 6, 8, 10, 12, },
  { 0, 4, 5, 6, 8, 10, 12, },
  { 1, 4, 5, 6, 8, 10, 12, },
  { 0, 1, 4, 5, 6, 8, 10, 12, },
  { 2, 4, 5, 6, 8, 10, 12, },
  { 0, 2, 4, 5, 6, 8, 10, 12, },
  { 1, 2, 4, 5, 6, 8, 10, 12, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 12, },
  { 3, 4, 5, 6, 8, 10, 12, },
  { 0, 3, 4, 5, 6, 8, 10, 12, },
  { 1, 3, 4, 5, 6, 8, 10, 12, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 12, },
  { 2, 3, 4, 5, 6, 8, 10, 12, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 12, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 12, },
  { 7, 8, 10, 12, },
  { 0, 7, 8, 10, 12, },
  { 1, 7, 8, 10, 12, },
  { 0, 1, 7, 8, 10, 12, },
  { 2, 7, 8, 10, 12, },
  { 0, 2, 7, 8, 10, 12, },
  { 1, 2, 7, 8, 10, 12, },
  { 0, 1, 2, 7, 8, 10, 12, },
  { 3, 7, 8, 10, 12, },
  { 0, 3, 7, 8, 10, 12, },
  { 1, 3, 7, 8, 10, 12, },
  { 0, 1, 3, 7, 8, 10, 12, },
  { 2, 3, 7, 8, 10, 12, },
  { 0, 2, 3, 7, 8, 10, 12, },
  { 1, 2, 3, 7, 8, 10, 12, },
  { 0, 1, 2, 3, 7, 8, 10, 12, },
  { 4, 7, 8, 10, 12, },
  { 0, 4, 7, 8, 10, 12, },
  { 1, 4, 7, 8, 10, 12, },
  { 0, 1, 4, 7, 8, 10, 12, },
  { 2, 4, 7, 8, 10, 12, },
  { 0, 2, 4, 7, 8, 10, 12, },
  { 1, 2, 4, 7, 8, 10, 12, },
  { 0, 1, 2, 4, 7, 8, 10, 12, },
  { 3, 4, 7, 8, 10, 12, },
  { 0, 3, 4, 7, 8, 10, 12, },
  { 1, 3, 4, 7, 8, 10, 12, },
  { 0, 1, 3, 4, 7, 8, 10, 12, },
  { 2, 3, 4, 7, 8, 10, 12, },
  { 0, 2, 3, 4, 7, 8, 10, 12, },
  { 1, 2, 3, 4, 7, 8, 10, 12, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 12, },
  { 5, 7, 8, 10, 12, },
  { 0, 5, 7, 8, 10, 12, },
  { 1, 5, 7, 8, 10, 12, },
  { 0, 1, 5, 7, 8, 10, 12, },
  { 2, 5, 7, 8, 10, 12, },
  { 0, 2, 5, 7, 8, 10, 12, },
  { 1, 2, 5, 7, 8, 10, 12, },
  { 0, 1, 2, 5, 7, 8, 10, 12, },
  { 3, 5, 7, 8, 10, 12, },
  { 0, 3, 5, 7, 8, 10, 12, },
  { 1, 3, 5, 7, 8, 10, 12, },
  { 0, 1, 3, 5, 7, 8, 10, 12, },
  { 2, 3, 5, 7, 8, 10, 12, },
  { 0, 2, 3, 5, 7, 8, 10, 12, },
  { 1, 2, 3, 5, 7, 8, 10, 12, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 12, },
  { 4, 5, 7, 8, 10, 12, },
  { 0, 4, 5, 7, 8, 10, 12, },
  { 1, 4, 5, 7, 8, 10, 12, },
  { 0, 1, 4, 5, 7, 8, 10, 12, },
  { 2, 4, 5, 7, 8, 10, 12, },
  { 0, 2, 4, 5, 7, 8, 10, 12, },
  { 1, 2, 4, 5, 7, 8, 10, 12, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 12, },
  { 3, 4, 5, 7, 8, 10, 12, },
  { 0, 3, 4, 5, 7, 8, 10, 12, },
  { 1, 3, 4, 5, 7, 8, 10, 12, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 12, },
  { 2, 3, 4, 5, 7, 8, 10, 12, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 12, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, },
  { 6, 7, 8, 10, 12, },
  { 0, 6, 7, 8, 10, 12, },
  { 1, 6, 7, 8, 10, 12, },
  { 0, 1, 6, 7, 8, 10, 12, },
  { 2, 6, 7, 8, 10, 12, },
  { 0, 2, 6, 7, 8, 10, 12, },
  { 1, 2, 6, 7, 8, 10, 12, },
  { 0, 1, 2, 6, 7, 8, 10, 12, },
  { 3, 6, 7, 8, 10, 12, },
  { 0, 3, 6, 7, 8, 10, 12, },
  { 1, 3, 6, 7, 8, 10, 12, },
  { 0, 1, 3, 6, 7, 8, 10, 12, },
  { 2, 3, 6, 7, 8, 10, 12, },
  { 0, 2, 3, 6, 7, 8, 10, 12, },
  { 1, 2, 3, 6, 7, 8, 10, 12, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 12, },
  { 4, 6, 7, 8, 10, 12, },
  { 0, 4, 6, 7, 8, 10, 12, },
  { 1, 4, 6, 7, 8, 10, 12, },
  { 0, 1, 4, 6, 7, 8, 10, 12, },
  { 2, 4, 6, 7, 8, 10, 12, },
  { 0, 2, 4, 6, 7, 8, 10, 12, },
  { 1, 2, 4, 6, 7, 8, 10, 12, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 12, },
  { 3, 4, 6, 7, 8, 10, 12, },
  { 0, 3, 4, 6, 7, 8, 10, 12, },
  { 1, 3, 4, 6, 7, 8, 10, 12, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 12, },
  { 2, 3, 4, 6, 7, 8, 10, 12, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 12, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 12, },
  { 5, 6, 7, 8, 10, 12, },
  { 0, 5, 6, 7, 8, 10, 12, },
  { 1, 5, 6, 7, 8, 10, 12, },
  { 0, 1, 5, 6, 7, 8, 10, 12, },
  { 2, 5, 6, 7, 8, 10, 12, },
  { 0, 2, 5, 6, 7, 8, 10, 12, },
  { 1, 2, 5, 6, 7, 8, 10, 12, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 12, },
  { 3, 5, 6, 7, 8, 10, 12, },
  { 0, 3, 5, 6, 7, 8, 10, 12, },
  { 1, 3, 5, 6, 7, 8, 10, 12, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 12, },
  { 2, 3, 5, 6, 7, 8, 10, 12, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 12, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 12, },
  { 4, 5, 6, 7, 8, 10, 12, },
  { 0, 4, 5, 6, 7, 8, 10, 12, },
  { 1, 4, 5, 6, 7, 8, 10, 12, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 12, },
  { 2, 4, 5, 6, 7, 8, 10, 12, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 12, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 12, },
  { 3, 4, 5, 6, 7, 8, 10, 12, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 12, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 12, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, },
  { 9, 10, 12, },
  { 0, 9, 10, 12, },
  { 1, 9, 10, 12, },
  { 0, 1, 9, 10, 12, },
  { 2, 9, 10, 12, },
  { 0, 2, 9, 10, 12, },
  { 1, 2, 9, 10, 12, },
  { 0, 1, 2, 9, 10, 12, },
  { 3, 9, 10, 12, },
  { 0, 3, 9, 10, 12, },
  { 1, 3, 9, 10, 12, },
  { 0, 1, 3, 9, 10, 12, },
  { 2, 3, 9, 10, 12, },
  { 0, 2, 3, 9, 10, 12, },
  { 1, 2, 3, 9, 10, 12, },
  { 0, 1, 2, 3, 9, 10, 12, },
  { 4, 9, 10, 12, },
  { 0, 4, 9, 10, 12, },
  { 1, 4, 9, 10, 12, },
  { 0, 1, 4, 9, 10, 12, },
  { 2, 4, 9, 10, 12, },
  { 0, 2, 4, 9, 10, 12, },
  { 1, 2, 4, 9, 10, 12, },
  { 0, 1, 2, 4, 9, 10, 12, },
  { 3, 4, 9, 10, 12, },
  { 0, 3, 4, 9, 10, 12, },
  { 1, 3, 4, 9, 10, 12, },
  { 0, 1, 3, 4, 9, 10, 12, },
  { 2, 3, 4, 9, 10, 12, },
  { 0, 2, 3, 4, 9, 10, 12, },
  { 1, 2, 3, 4, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 9, 10, 12, },
  { 5, 9, 10, 12, },
  { 0, 5, 9, 10, 12, },
  { 1, 5, 9, 10, 12, },
  { 0, 1, 5, 9, 10, 12, },
  { 2, 5, 9, 10, 12, },
  { 0, 2, 5, 9, 10, 12, },
  { 1, 2, 5, 9, 10, 12, },
  { 0, 1, 2, 5, 9, 10, 12, },
  { 3, 5, 9, 10, 12, },
  { 0, 3, 5, 9, 10, 12, },
  { 1, 3, 5, 9, 10, 12, },
  { 0, 1, 3, 5, 9, 10, 12, },
  { 2, 3, 5, 9, 10, 12, },
  { 0, 2, 3, 5, 9, 10, 12, },
  { 1, 2, 3, 5, 9, 10, 12, },
  { 0, 1, 2, 3, 5, 9, 10, 12, },
  { 4, 5, 9, 10, 12, },
  { 0, 4, 5, 9, 10, 12, },
  { 1, 4, 5, 9, 10, 12, },
  { 0, 1, 4, 5, 9, 10, 12, },
  { 2, 4, 5, 9, 10, 12, },
  { 0, 2, 4, 5, 9, 10, 12, },
  { 1, 2, 4, 5, 9, 10, 12, },
  { 0, 1, 2, 4, 5, 9, 10, 12, },
  { 3, 4, 5, 9, 10, 12, },
  { 0, 3, 4, 5, 9, 10, 12, },
  { 1, 3, 4, 5, 9, 10, 12, },
  { 0, 1, 3, 4, 5, 9, 10, 12, },
  { 2, 3, 4, 5, 9, 10, 12, },
  { 0, 2, 3, 4, 5, 9, 10, 12, },
  { 1, 2, 3, 4, 5, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 12, },
  { 6, 9, 10, 12, },
  { 0, 6, 9, 10, 12, },
  { 1, 6, 9, 10, 12, },
  { 0, 1, 6, 9, 10, 12, },
  { 2, 6, 9, 10, 12, },
  { 0, 2, 6, 9, 10, 12, },
  { 1, 2, 6, 9, 10, 12, },
  { 0, 1, 2, 6, 9, 10, 12, },
  { 3, 6, 9, 10, 12, },
  { 0, 3, 6, 9, 10, 12, },
  { 1, 3, 6, 9, 10, 12, },
  { 0, 1, 3, 6, 9, 10, 12, },
  { 2, 3, 6, 9, 10, 12, },
  { 0, 2, 3, 6, 9, 10, 12, },
  { 1, 2, 3, 6, 9, 10, 12, },
  { 0, 1, 2, 3, 6, 9, 10, 12, },
  { 4, 6, 9, 10, 12, },
  { 0, 4, 6, 9, 10, 12, },
  { 1, 4, 6, 9, 10, 12, },
  { 0, 1, 4, 6, 9, 10, 12, },
  { 2, 4, 6, 9, 10, 12, },
  { 0, 2, 4, 6, 9, 10, 12, },
  { 1, 2, 4, 6, 9, 10, 12, },
  { 0, 1, 2, 4, 6, 9, 10, 12, },
  { 3, 4, 6, 9, 10, 12, },
  { 0, 3, 4, 6, 9, 10, 12, },
  { 1, 3, 4, 6, 9, 10, 12, },
  { 0, 1, 3, 4, 6, 9, 10, 12, },
  { 2, 3, 4, 6, 9, 10, 12, },
  { 0, 2, 3, 4, 6, 9, 10, 12, },
  { 1, 2, 3, 4, 6, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 12, },
  { 5, 6, 9, 10, 12, },
  { 0, 5, 6, 9, 10, 12, },
  { 1, 5, 6, 9, 10, 12, },
  { 0, 1, 5, 6, 9, 10, 12, },
  { 2, 5, 6, 9, 10, 12, },
  { 0, 2, 5, 6, 9, 10, 12, },
  { 1, 2, 5, 6, 9, 10, 12, },
  { 0, 1, 2, 5, 6, 9, 10, 12, },
  { 3, 5, 6, 9, 10, 12, },
  { 0, 3, 5, 6, 9, 10, 12, },
  { 1, 3, 5, 6, 9, 10, 12, },
  { 0, 1, 3, 5, 6, 9, 10, 12, },
  { 2, 3, 5, 6, 9, 10, 12, },
  { 0, 2, 3, 5, 6, 9, 10, 12, },
  { 1, 2, 3, 5, 6, 9, 10, 12, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 12, },
  { 4, 5, 6, 9, 10, 12, },
  { 0, 4, 5, 6, 9, 10, 12, },
  { 1, 4, 5, 6, 9, 10, 12, },
  { 0, 1, 4, 5, 6, 9, 10, 12, },
  { 2, 4, 5, 6, 9, 10, 12, },
  { 0, 2, 4, 5, 6, 9, 10, 12, },
  { 1, 2, 4, 5, 6, 9, 10, 12, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 12, },
  { 3, 4, 5, 6, 9, 10, 12, },
  { 0, 3, 4, 5, 6, 9, 10, 12, },
  { 1, 3, 4, 5, 6, 9, 10, 12, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 12, },
  { 2, 3, 4, 5, 6, 9, 10, 12, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 12, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 12, },
  { 7, 9, 10, 12, },
  { 0, 7, 9, 10, 12, },
  { 1, 7, 9, 10, 12, },
  { 0, 1, 7, 9, 10, 12, },
  { 2, 7, 9, 10, 12, },
  { 0, 2, 7, 9, 10, 12, },
  { 1, 2, 7, 9, 10, 12, },
  { 0, 1, 2, 7, 9, 10, 12, },
  { 3, 7, 9, 10, 12, },
  { 0, 3, 7, 9, 10, 12, },
  { 1, 3, 7, 9, 10, 12, },
  { 0, 1, 3, 7, 9, 10, 12, },
  { 2, 3, 7, 9, 10, 12, },
  { 0, 2, 3, 7, 9, 10, 12, },
  { 1, 2, 3, 7, 9, 10, 12, },
  { 0, 1, 2, 3, 7, 9, 10, 12, },
  { 4, 7, 9, 10, 12, },
  { 0, 4, 7, 9, 10, 12, },
  { 1, 4, 7, 9, 10, 12, },
  { 0, 1, 4, 7, 9, 10, 12, },
  { 2, 4, 7, 9, 10, 12, },
  { 0, 2, 4, 7, 9, 10, 12, },
  { 1, 2, 4, 7, 9, 10, 12, },
  { 0, 1, 2, 4, 7, 9, 10, 12, },
  { 3, 4, 7, 9, 10, 12, },
  { 0, 3, 4, 7, 9, 10, 12, },
  { 1, 3, 4, 7, 9, 10, 12, },
  { 0, 1, 3, 4, 7, 9, 10, 12, },
  { 2, 3, 4, 7, 9, 10, 12, },
  { 0, 2, 3, 4, 7, 9, 10, 12, },
  { 1, 2, 3, 4, 7, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 12, },
  { 5, 7, 9, 10, 12, },
  { 0, 5, 7, 9, 10, 12, },
  { 1, 5, 7, 9, 10, 12, },
  { 0, 1, 5, 7, 9, 10, 12, },
  { 2, 5, 7, 9, 10, 12, },
  { 0, 2, 5, 7, 9, 10, 12, },
  { 1, 2, 5, 7, 9, 10, 12, },
  { 0, 1, 2, 5, 7, 9, 10, 12, },
  { 3, 5, 7, 9, 10, 12, },
  { 0, 3, 5, 7, 9, 10, 12, },
  { 1, 3, 5, 7, 9, 10, 12, },
  { 0, 1, 3, 5, 7, 9, 10, 12, },
  { 2, 3, 5, 7, 9, 10, 12, },
  { 0, 2, 3, 5, 7, 9, 10, 12, },
  { 1, 2, 3, 5, 7, 9, 10, 12, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 12, },
  { 4, 5, 7, 9, 10, 12, },
  { 0, 4, 5, 7, 9, 10, 12, },
  { 1, 4, 5, 7, 9, 10, 12, },
  { 0, 1, 4, 5, 7, 9, 10, 12, },
  { 2, 4, 5, 7, 9, 10, 12, },
  { 0, 2, 4, 5, 7, 9, 10, 12, },
  { 1, 2, 4, 5, 7, 9, 10, 12, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 12, },
  { 3, 4, 5, 7, 9, 10, 12, },
  { 0, 3, 4, 5, 7, 9, 10, 12, },
  { 1, 3, 4, 5, 7, 9, 10, 12, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 12, },
  { 2, 3, 4, 5, 7, 9, 10, 12, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 12, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 12, },
  { 6, 7, 9, 10, 12, },
  { 0, 6, 7, 9, 10, 12, },
  { 1, 6, 7, 9, 10, 12, },
  { 0, 1, 6, 7, 9, 10, 12, },
  { 2, 6, 7, 9, 10, 12, },
  { 0, 2, 6, 7, 9, 10, 12, },
  { 1, 2, 6, 7, 9, 10, 12, },
  { 0, 1, 2, 6, 7, 9, 10, 12, },
  { 3, 6, 7, 9, 10, 12, },
  { 0, 3, 6, 7, 9, 10, 12, },
  { 1, 3, 6, 7, 9, 10, 12, },
  { 0, 1, 3, 6, 7, 9, 10, 12, },
  { 2, 3, 6, 7, 9, 10, 12, },
  { 0, 2, 3, 6, 7, 9, 10, 12, },
  { 1, 2, 3, 6, 7, 9, 10, 12, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 12, },
  { 4, 6, 7, 9, 10, 12, },
  { 0, 4, 6, 7, 9, 10, 12, },
  { 1, 4, 6, 7, 9, 10, 12, },
  { 0, 1, 4, 6, 7, 9, 10, 12, },
  { 2, 4, 6, 7, 9, 10, 12, },
  { 0, 2, 4, 6, 7, 9, 10, 12, },
  { 1, 2, 4, 6, 7, 9, 10, 12, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 12, },
  { 3, 4, 6, 7, 9, 10, 12, },
  { 0, 3, 4, 6, 7, 9, 10, 12, },
  { 1, 3, 4, 6, 7, 9, 10, 12, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 12, },
  { 2, 3, 4, 6, 7, 9, 10, 12, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 12, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 12, },
  { 5, 6, 7, 9, 10, 12, },
  { 0, 5, 6, 7, 9, 10, 12, },
  { 1, 5, 6, 7, 9, 10, 12, },
  { 0, 1, 5, 6, 7, 9, 10, 12, },
  { 2, 5, 6, 7, 9, 10, 12, },
  { 0, 2, 5, 6, 7, 9, 10, 12, },
  { 1, 2, 5, 6, 7, 9, 10, 12, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 12, },
  { 3, 5, 6, 7, 9, 10, 12, },
  { 0, 3, 5, 6, 7, 9, 10, 12, },
  { 1, 3, 5, 6, 7, 9, 10, 12, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 12, },
  { 2, 3, 5, 6, 7, 9, 10, 12, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 12, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 12, },
  { 4, 5, 6, 7, 9, 10, 12, },
  { 0, 4, 5, 6, 7, 9, 10, 12, },
  { 1, 4, 5, 6, 7, 9, 10, 12, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 12, },
  { 2, 4, 5, 6, 7, 9, 10, 12, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 12, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 12, },
  { 3, 4, 5, 6, 7, 9, 10, 12, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 12, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 12, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, },
  { 8, 9, 10, 12, },
  { 0, 8, 9, 10, 12, },
  { 1, 8, 9, 10, 12, },
  { 0, 1, 8, 9, 10, 12, },
  { 2, 8, 9, 10, 12, },
  { 0, 2, 8, 9, 10, 12, },
  { 1, 2, 8, 9, 10, 12, },
  { 0, 1, 2, 8, 9, 10, 12, },
  { 3, 8, 9, 10, 12, },
  { 0, 3, 8, 9, 10, 12, },
  { 1, 3, 8, 9, 10, 12, },
  { 0, 1, 3, 8, 9, 10, 12, },
  { 2, 3, 8, 9, 10, 12, },
  { 0, 2, 3, 8, 9, 10, 12, },
  { 1, 2, 3, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 8, 9, 10, 12, },
  { 4, 8, 9, 10, 12, },
  { 0, 4, 8, 9, 10, 12, },
  { 1, 4, 8, 9, 10, 12, },
  { 0, 1, 4, 8, 9, 10, 12, },
  { 2, 4, 8, 9, 10, 12, },
  { 0, 2, 4, 8, 9, 10, 12, },
  { 1, 2, 4, 8, 9, 10, 12, },
  { 0, 1, 2, 4, 8, 9, 10, 12, },
  { 3, 4, 8, 9, 10, 12, },
  { 0, 3, 4, 8, 9, 10, 12, },
  { 1, 3, 4, 8, 9, 10, 12, },
  { 0, 1, 3, 4, 8, 9, 10, 12, },
  { 2, 3, 4, 8, 9, 10, 12, },
  { 0, 2, 3, 4, 8, 9, 10, 12, },
  { 1, 2, 3, 4, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 12, },
  { 5, 8, 9, 10, 12, },
  { 0, 5, 8, 9, 10, 12, },
  { 1, 5, 8, 9, 10, 12, },
  { 0, 1, 5, 8, 9, 10, 12, },
  { 2, 5, 8, 9, 10, 12, },
  { 0, 2, 5, 8, 9, 10, 12, },
  { 1, 2, 5, 8, 9, 10, 12, },
  { 0, 1, 2, 5, 8, 9, 10, 12, },
  { 3, 5, 8, 9, 10, 12, },
  { 0, 3, 5, 8, 9, 10, 12, },
  { 1, 3, 5, 8, 9, 10, 12, },
  { 0, 1, 3, 5, 8, 9, 10, 12, },
  { 2, 3, 5, 8, 9, 10, 12, },
  { 0, 2, 3, 5, 8, 9, 10, 12, },
  { 1, 2, 3, 5, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 12, },
  { 4, 5, 8, 9, 10, 12, },
  { 0, 4, 5, 8, 9, 10, 12, },
  { 1, 4, 5, 8, 9, 10, 12, },
  { 0, 1, 4, 5, 8, 9, 10, 12, },
  { 2, 4, 5, 8, 9, 10, 12, },
  { 0, 2, 4, 5, 8, 9, 10, 12, },
  { 1, 2, 4, 5, 8, 9, 10, 12, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 12, },
  { 3, 4, 5, 8, 9, 10, 12, },
  { 0, 3, 4, 5, 8, 9, 10, 12, },
  { 1, 3, 4, 5, 8, 9, 10, 12, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 12, },
  { 2, 3, 4, 5, 8, 9, 10, 12, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 12, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 12, },
  { 6, 8, 9, 10, 12, },
  { 0, 6, 8, 9, 10, 12, },
  { 1, 6, 8, 9, 10, 12, },
  { 0, 1, 6, 8, 9, 10, 12, },
  { 2, 6, 8, 9, 10, 12, },
  { 0, 2, 6, 8, 9, 10, 12, },
  { 1, 2, 6, 8, 9, 10, 12, },
  { 0, 1, 2, 6, 8, 9, 10, 12, },
  { 3, 6, 8, 9, 10, 12, },
  { 0, 3, 6, 8, 9, 10, 12, },
  { 1, 3, 6, 8, 9, 10, 12, },
  { 0, 1, 3, 6, 8, 9, 10, 12, },
  { 2, 3, 6, 8, 9, 10, 12, },
  { 0, 2, 3, 6, 8, 9, 10, 12, },
  { 1, 2, 3, 6, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 12, },
  { 4, 6, 8, 9, 10, 12, },
  { 0, 4, 6, 8, 9, 10, 12, },
  { 1, 4, 6, 8, 9, 10, 12, },
  { 0, 1, 4, 6, 8, 9, 10, 12, },
  { 2, 4, 6, 8, 9, 10, 12, },
  { 0, 2, 4, 6, 8, 9, 10, 12, },
  { 1, 2, 4, 6, 8, 9, 10, 12, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 12, },
  { 3, 4, 6, 8, 9, 10, 12, },
  { 0, 3, 4, 6, 8, 9, 10, 12, },
  { 1, 3, 4, 6, 8, 9, 10, 12, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 12, },
  { 2, 3, 4, 6, 8, 9, 10, 12, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 12, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 12, },
  { 5, 6, 8, 9, 10, 12, },
  { 0, 5, 6, 8, 9, 10, 12, },
  { 1, 5, 6, 8, 9, 10, 12, },
  { 0, 1, 5, 6, 8, 9, 10, 12, },
  { 2, 5, 6, 8, 9, 10, 12, },
  { 0, 2, 5, 6, 8, 9, 10, 12, },
  { 1, 2, 5, 6, 8, 9, 10, 12, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 12, },
  { 3, 5, 6, 8, 9, 10, 12, },
  { 0, 3, 5, 6, 8, 9, 10, 12, },
  { 1, 3, 5, 6, 8, 9, 10, 12, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 12, },
  { 2, 3, 5, 6, 8, 9, 10, 12, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 12, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 12, },
  { 4, 5, 6, 8, 9, 10, 12, },
  { 0, 4, 5, 6, 8, 9, 10, 12, },
  { 1, 4, 5, 6, 8, 9, 10, 12, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 12, },
  { 2, 4, 5, 6, 8, 9, 10, 12, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 12, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 12, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, },
  { 3, 4, 5, 6, 8, 9, 10, 12, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 12, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 12, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 12, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 12, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 12, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, },
  { 7, 8, 9, 10, 12, },
  { 0, 7, 8, 9, 10, 12, },
  { 1, 7, 8, 9, 10, 12, },
  { 0, 1, 7, 8, 9, 10, 12, },
  { 2, 7, 8, 9, 10, 12, },
  { 0, 2, 7, 8, 9, 10, 12, },
  { 1, 2, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 7, 8, 9, 10, 12, },
  { 3, 7, 8, 9, 10, 12, },
  { 0, 3, 7, 8, 9, 10, 12, },
  { 1, 3, 7, 8, 9, 10, 12, },
  { 0, 1, 3, 7, 8, 9, 10, 12, },
  { 2, 3, 7, 8, 9, 10, 12, },
  { 0, 2, 3, 7, 8, 9, 10, 12, },
  { 1, 2, 3, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 12, },
  { 4, 7, 8, 9, 10, 12, },
  { 0, 4, 7, 8, 9, 10, 12, },
  { 1, 4, 7, 8, 9, 10, 12, },
  { 0, 1, 4, 7, 8, 9, 10, 12, },
  { 2, 4, 7, 8, 9, 10, 12, },
  { 0, 2, 4, 7, 8, 9, 10, 12, },
  { 1, 2, 4, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 12, },
  { 3, 4, 7, 8, 9, 10, 12, },
  { 0, 3, 4, 7, 8, 9, 10, 12, },
  { 1, 3, 4, 7, 8, 9, 10, 12, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 12, },
  { 2, 3, 4, 7, 8, 9, 10, 12, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 12, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 12, },
  { 5, 7, 8, 9, 10, 12, },
  { 0, 5, 7, 8, 9, 10, 12, },
  { 1, 5, 7, 8, 9, 10, 12, },
  { 0, 1, 5, 7, 8, 9, 10, 12, },
  { 2, 5, 7, 8, 9, 10, 12, },
  { 0, 2, 5, 7, 8, 9, 10, 12, },
  { 1, 2, 5, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 12, },
  { 3, 5, 7, 8, 9, 10, 12, },
  { 0, 3, 5, 7, 8, 9, 10, 12, },
  { 1, 3, 5, 7, 8, 9, 10, 12, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 12, },
  { 2, 3, 5, 7, 8, 9, 10, 12, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 12, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 12, },
  { 4, 5, 7, 8, 9, 10, 12, },
  { 0, 4, 5, 7, 8, 9, 10, 12, },
  { 1, 4, 5, 7, 8, 9, 10, 12, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 12, },
  { 2, 4, 5, 7, 8, 9, 10, 12, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 12, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 12, },
  { 3, 4, 5, 7, 8, 9, 10, 12, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 12, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 12, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 12, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 12, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 12, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, },
  { 6, 7, 8, 9, 10, 12, },
  { 0, 6, 7, 8, 9, 10, 12, },
  { 1, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 6, 7, 8, 9, 10, 12, },
  { 2, 6, 7, 8, 9, 10, 12, },
  { 0, 2, 6, 7, 8, 9, 10, 12, },
  { 1, 2, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 12, },
  { 3, 6, 7, 8, 9, 10, 12, },
  { 0, 3, 6, 7, 8, 9, 10, 12, },
  { 1, 3, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 12, },
  { 2, 3, 6, 7, 8, 9, 10, 12, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 12, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 12, },
  { 4, 6, 7, 8, 9, 10, 12, },
  { 0, 4, 6, 7, 8, 9, 10, 12, },
  { 1, 4, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 12, },
  { 2, 4, 6, 7, 8, 9, 10, 12, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 12, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 12, },
  { 3, 4, 6, 7, 8, 9, 10, 12, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 12, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 12, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 12, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 12, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, },
  { 5, 6, 7, 8, 9, 10, 12, },
  { 0, 5, 6, 7, 8, 9, 10, 12, },
  { 1, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 12, },
  { 2, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 12, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 12, },
  { 3, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 12, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 12, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 12, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, },
  { 4, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, },
  { 11, 12, },
  { 0, 11, 12, },
  { 1, 11, 12, },
  { 0, 1, 11, 12, },
  { 2, 11, 12, },
  { 0, 2, 11, 12, },
  { 1, 2, 11, 12, },
  { 0, 1, 2, 11, 12, },
  { 3, 11, 12, },
  { 0, 3, 11, 12, },
  { 1, 3, 11, 12, },
  { 0, 1, 3, 11, 12, },
  { 2, 3, 11, 12, },
  { 0, 2, 3, 11, 12, },
  { 1, 2, 3, 11, 12, },
  { 0, 1, 2, 3, 11, 12, },
  { 4, 11, 12, },
  { 0, 4, 11, 12, },
  { 1, 4, 11, 12, },
  { 0, 1, 4, 11, 12, },
  { 2, 4, 11, 12, },
  { 0, 2, 4, 11, 12, },
  { 1, 2, 4, 11, 12, },
  { 0, 1, 2, 4, 11, 12, },
  { 3, 4, 11, 12, },
  { 0, 3, 4, 11, 12, },
  { 1, 3, 4, 11, 12, },
  { 0, 1, 3, 4, 11, 12, },
  { 2, 3, 4, 11, 12, },
  { 0, 2, 3, 4, 11, 12, },
  { 1, 2, 3, 4, 11, 12, },
  { 0, 1, 2, 3, 4, 11, 12, },
  { 5, 11, 12, },
  { 0, 5, 11, 12, },
  { 1, 5, 11, 12, },
  { 0, 1, 5, 11, 12, },
  { 2, 5, 11, 12, },
  { 0, 2, 5, 11, 12, },
  { 1, 2, 5, 11, 12, },
  { 0, 1, 2, 5, 11, 12, },
  { 3, 5, 11, 12, },
  { 0, 3, 5, 11, 12, },
  { 1, 3, 5, 11, 12, },
  { 0, 1, 3, 5, 11, 12, },
  { 2, 3, 5, 11, 12, },
  { 0, 2, 3, 5, 11, 12, },
  { 1, 2, 3, 5, 11, 12, },
  { 0, 1, 2, 3, 5, 11, 12, },
  { 4, 5, 11, 12, },
  { 0, 4, 5, 11, 12, },
  { 1, 4, 5, 11, 12, },
  { 0, 1, 4, 5, 11, 12, },
  { 2, 4, 5, 11, 12, },
  { 0, 2, 4, 5, 11, 12, },
  { 1, 2, 4, 5, 11, 12, },
  { 0, 1, 2, 4, 5, 11, 12, },
  { 3, 4, 5, 11, 12, },
  { 0, 3, 4, 5, 11, 12, },
  { 1, 3, 4, 5, 11, 12, },
  { 0, 1, 3, 4, 5, 11, 12, },
  { 2, 3, 4, 5, 11, 12, },
  { 0, 2, 3, 4, 5, 11, 12, },
  { 1, 2, 3, 4, 5, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 11, 12, },
  { 6, 11, 12, },
  { 0, 6, 11, 12, },
  { 1, 6, 11, 12, },
  { 0, 1, 6, 11, 12, },
  { 2, 6, 11, 12, },
  { 0, 2, 6, 11, 12, },
  { 1, 2, 6, 11, 12, },
  { 0, 1, 2, 6, 11, 12, },
  { 3, 6, 11, 12, },
  { 0, 3, 6, 11, 12, },
  { 1, 3, 6, 11, 12, },
  { 0, 1, 3, 6, 11, 12, },
  { 2, 3, 6, 11, 12, },
  { 0, 2, 3, 6, 11, 12, },
  { 1, 2, 3, 6, 11, 12, },
  { 0, 1, 2, 3, 6, 11, 12, },
  { 4, 6, 11, 12, },
  { 0, 4, 6, 11, 12, },
  { 1, 4, 6, 11, 12, },
  { 0, 1, 4, 6, 11, 12, },
  { 2, 4, 6, 11, 12, },
  { 0, 2, 4, 6, 11, 12, },
  { 1, 2, 4, 6, 11, 12, },
  { 0, 1, 2, 4, 6, 11, 12, },
  { 3, 4, 6, 11, 12, },
  { 0, 3, 4, 6, 11, 12, },
  { 1, 3, 4, 6, 11, 12, },
  { 0, 1, 3, 4, 6, 11, 12, },
  { 2, 3, 4, 6, 11, 12, },
  { 0, 2, 3, 4, 6, 11, 12, },
  { 1, 2, 3, 4, 6, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 11, 12, },
  { 5, 6, 11, 12, },
  { 0, 5, 6, 11, 12, },
  { 1, 5, 6, 11, 12, },
  { 0, 1, 5, 6, 11, 12, },
  { 2, 5, 6, 11, 12, },
  { 0, 2, 5, 6, 11, 12, },
  { 1, 2, 5, 6, 11, 12, },
  { 0, 1, 2, 5, 6, 11, 12, },
  { 3, 5, 6, 11, 12, },
  { 0, 3, 5, 6, 11, 12, },
  { 1, 3, 5, 6, 11, 12, },
  { 0, 1, 3, 5, 6, 11, 12, },
  { 2, 3, 5, 6, 11, 12, },
  { 0, 2, 3, 5, 6, 11, 12, },
  { 1, 2, 3, 5, 6, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 11, 12, },
  { 4, 5, 6, 11, 12, },
  { 0, 4, 5, 6, 11, 12, },
  { 1, 4, 5, 6, 11, 12, },
  { 0, 1, 4, 5, 6, 11, 12, },
  { 2, 4, 5, 6, 11, 12, },
  { 0, 2, 4, 5, 6, 11, 12, },
  { 1, 2, 4, 5, 6, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 11, 12, },
  { 3, 4, 5, 6, 11, 12, },
  { 0, 3, 4, 5, 6, 11, 12, },
  { 1, 3, 4, 5, 6, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 11, 12, },
  { 2, 3, 4, 5, 6, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 12, },
  { 7, 11, 12, },
  { 0, 7, 11, 12, },
  { 1, 7, 11, 12, },
  { 0, 1, 7, 11, 12, },
  { 2, 7, 11, 12, },
  { 0, 2, 7, 11, 12, },
  { 1, 2, 7, 11, 12, },
  { 0, 1, 2, 7, 11, 12, },
  { 3, 7, 11, 12, },
  { 0, 3, 7, 11, 12, },
  { 1, 3, 7, 11, 12, },
  { 0, 1, 3, 7, 11, 12, },
  { 2, 3, 7, 11, 12, },
  { 0, 2, 3, 7, 11, 12, },
  { 1, 2, 3, 7, 11, 12, },
  { 0, 1, 2, 3, 7, 11, 12, },
  { 4, 7, 11, 12, },
  { 0, 4, 7, 11, 12, },
  { 1, 4, 7, 11, 12, },
  { 0, 1, 4, 7, 11, 12, },
  { 2, 4, 7, 11, 12, },
  { 0, 2, 4, 7, 11, 12, },
  { 1, 2, 4, 7, 11, 12, },
  { 0, 1, 2, 4, 7, 11, 12, },
  { 3, 4, 7, 11, 12, },
  { 0, 3, 4, 7, 11, 12, },
  { 1, 3, 4, 7, 11, 12, },
  { 0, 1, 3, 4, 7, 11, 12, },
  { 2, 3, 4, 7, 11, 12, },
  { 0, 2, 3, 4, 7, 11, 12, },
  { 1, 2, 3, 4, 7, 11, 12, },
  { 0, 1, 2, 3, 4, 7, 11, 12, },
  { 5, 7, 11, 12, },
  { 0, 5, 7, 11, 12, },
  { 1, 5, 7, 11, 12, },
  { 0, 1, 5, 7, 11, 12, },
  { 2, 5, 7, 11, 12, },
  { 0, 2, 5, 7, 11, 12, },
  { 1, 2, 5, 7, 11, 12, },
  { 0, 1, 2, 5, 7, 11, 12, },
  { 3, 5, 7, 11, 12, },
  { 0, 3, 5, 7, 11, 12, },
  { 1, 3, 5, 7, 11, 12, },
  { 0, 1, 3, 5, 7, 11, 12, },
  { 2, 3, 5, 7, 11, 12, },
  { 0, 2, 3, 5, 7, 11, 12, },
  { 1, 2, 3, 5, 7, 11, 12, },
  { 0, 1, 2, 3, 5, 7, 11, 12, },
  { 4, 5, 7, 11, 12, },
  { 0, 4, 5, 7, 11, 12, },
  { 1, 4, 5, 7, 11, 12, },
  { 0, 1, 4, 5, 7, 11, 12, },
  { 2, 4, 5, 7, 11, 12, },
  { 0, 2, 4, 5, 7, 11, 12, },
  { 1, 2, 4, 5, 7, 11, 12, },
  { 0, 1, 2, 4, 5, 7, 11, 12, },
  { 3, 4, 5, 7, 11, 12, },
  { 0, 3, 4, 5, 7, 11, 12, },
  { 1, 3, 4, 5, 7, 11, 12, },
  { 0, 1, 3, 4, 5, 7, 11, 12, },
  { 2, 3, 4, 5, 7, 11, 12, },
  { 0, 2, 3, 4, 5, 7, 11, 12, },
  { 1, 2, 3, 4, 5, 7, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 12, },
  { 6, 7, 11, 12, },
  { 0, 6, 7, 11, 12, },
  { 1, 6, 7, 11, 12, },
  { 0, 1, 6, 7, 11, 12, },
  { 2, 6, 7, 11, 12, },
  { 0, 2, 6, 7, 11, 12, },
  { 1, 2, 6, 7, 11, 12, },
  { 0, 1, 2, 6, 7, 11, 12, },
  { 3, 6, 7, 11, 12, },
  { 0, 3, 6, 7, 11, 12, },
  { 1, 3, 6, 7, 11, 12, },
  { 0, 1, 3, 6, 7, 11, 12, },
  { 2, 3, 6, 7, 11, 12, },
  { 0, 2, 3, 6, 7, 11, 12, },
  { 1, 2, 3, 6, 7, 11, 12, },
  { 0, 1, 2, 3, 6, 7, 11, 12, },
  { 4, 6, 7, 11, 12, },
  { 0, 4, 6, 7, 11, 12, },
  { 1, 4, 6, 7, 11, 12, },
  { 0, 1, 4, 6, 7, 11, 12, },
  { 2, 4, 6, 7, 11, 12, },
  { 0, 2, 4, 6, 7, 11, 12, },
  { 1, 2, 4, 6, 7, 11, 12, },
  { 0, 1, 2, 4, 6, 7, 11, 12, },
  { 3, 4, 6, 7, 11, 12, },
  { 0, 3, 4, 6, 7, 11, 12, },
  { 1, 3, 4, 6, 7, 11, 12, },
  { 0, 1, 3, 4, 6, 7, 11, 12, },
  { 2, 3, 4, 6, 7, 11, 12, },
  { 0, 2, 3, 4, 6, 7, 11, 12, },
  { 1, 2, 3, 4, 6, 7, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 12, },
  { 5, 6, 7, 11, 12, },
  { 0, 5, 6, 7, 11, 12, },
  { 1, 5, 6, 7, 11, 12, },
  { 0, 1, 5, 6, 7, 11, 12, },
  { 2, 5, 6, 7, 11, 12, },
  { 0, 2, 5, 6, 7, 11, 12, },
  { 1, 2, 5, 6, 7, 11, 12, },
  { 0, 1, 2, 5, 6, 7, 11, 12, },
  { 3, 5, 6, 7, 11, 12, },
  { 0, 3, 5, 6, 7, 11, 12, },
  { 1, 3, 5, 6, 7, 11, 12, },
  { 0, 1, 3, 5, 6, 7, 11, 12, },
  { 2, 3, 5, 6, 7, 11, 12, },
  { 0, 2, 3, 5, 6, 7, 11, 12, },
  { 1, 2, 3, 5, 6, 7, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 12, },
  { 4, 5, 6, 7, 11, 12, },
  { 0, 4, 5, 6, 7, 11, 12, },
  { 1, 4, 5, 6, 7, 11, 12, },
  { 0, 1, 4, 5, 6, 7, 11, 12, },
  { 2, 4, 5, 6, 7, 11, 12, },
  { 0, 2, 4, 5, 6, 7, 11, 12, },
  { 1, 2, 4, 5, 6, 7, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 12, },
  { 3, 4, 5, 6, 7, 11, 12, },
  { 0, 3, 4, 5, 6, 7, 11, 12, },
  { 1, 3, 4, 5, 6, 7, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 12, },
  { 2, 3, 4, 5, 6, 7, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 12, },
  { 8, 11, 12, },
  { 0, 8, 11, 12, },
  { 1, 8, 11, 12, },
  { 0, 1, 8, 11, 12, },
  { 2, 8, 11, 12, },
  { 0, 2, 8, 11, 12, },
  { 1, 2, 8, 11, 12, },
  { 0, 1, 2, 8, 11, 12, },
  { 3, 8, 11, 12, },
  { 0, 3, 8, 11, 12, },
  { 1, 3, 8, 11, 12, },
  { 0, 1, 3, 8, 11, 12, },
  { 2, 3, 8, 11, 12, },
  { 0, 2, 3, 8, 11, 12, },
  { 1, 2, 3, 8, 11, 12, },
  { 0, 1, 2, 3, 8, 11, 12, },
  { 4, 8, 11, 12, },
  { 0, 4, 8, 11, 12, },
  { 1, 4, 8, 11, 12, },
  { 0, 1, 4, 8, 11, 12, },
  { 2, 4, 8, 11, 12, },
  { 0, 2, 4, 8, 11, 12, },
  { 1, 2, 4, 8, 11, 12, },
  { 0, 1, 2, 4, 8, 11, 12, },
  { 3, 4, 8, 11, 12, },
  { 0, 3, 4, 8, 11, 12, },
  { 1, 3, 4, 8, 11, 12, },
  { 0, 1, 3, 4, 8, 11, 12, },
  { 2, 3, 4, 8, 11, 12, },
  { 0, 2, 3, 4, 8, 11, 12, },
  { 1, 2, 3, 4, 8, 11, 12, },
  { 0, 1, 2, 3, 4, 8, 11, 12, },
  { 5, 8, 11, 12, },
  { 0, 5, 8, 11, 12, },
  { 1, 5, 8, 11, 12, },
  { 0, 1, 5, 8, 11, 12, },
  { 2, 5, 8, 11, 12, },
  { 0, 2, 5, 8, 11, 12, },
  { 1, 2, 5, 8, 11, 12, },
  { 0, 1, 2, 5, 8, 11, 12, },
  { 3, 5, 8, 11, 12, },
  { 0, 3, 5, 8, 11, 12, },
  { 1, 3, 5, 8, 11, 12, },
  { 0, 1, 3, 5, 8, 11, 12, },
  { 2, 3, 5, 8, 11, 12, },
  { 0, 2, 3, 5, 8, 11, 12, },
  { 1, 2, 3, 5, 8, 11, 12, },
  { 0, 1, 2, 3, 5, 8, 11, 12, },
  { 4, 5, 8, 11, 12, },
  { 0, 4, 5, 8, 11, 12, },
  { 1, 4, 5, 8, 11, 12, },
  { 0, 1, 4, 5, 8, 11, 12, },
  { 2, 4, 5, 8, 11, 12, },
  { 0, 2, 4, 5, 8, 11, 12, },
  { 1, 2, 4, 5, 8, 11, 12, },
  { 0, 1, 2, 4, 5, 8, 11, 12, },
  { 3, 4, 5, 8, 11, 12, },
  { 0, 3, 4, 5, 8, 11, 12, },
  { 1, 3, 4, 5, 8, 11, 12, },
  { 0, 1, 3, 4, 5, 8, 11, 12, },
  { 2, 3, 4, 5, 8, 11, 12, },
  { 0, 2, 3, 4, 5, 8, 11, 12, },
  { 1, 2, 3, 4, 5, 8, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 12, },
  { 6, 8, 11, 12, },
  { 0, 6, 8, 11, 12, },
  { 1, 6, 8, 11, 12, },
  { 0, 1, 6, 8, 11, 12, },
  { 2, 6, 8, 11, 12, },
  { 0, 2, 6, 8, 11, 12, },
  { 1, 2, 6, 8, 11, 12, },
  { 0, 1, 2, 6, 8, 11, 12, },
  { 3, 6, 8, 11, 12, },
  { 0, 3, 6, 8, 11, 12, },
  { 1, 3, 6, 8, 11, 12, },
  { 0, 1, 3, 6, 8, 11, 12, },
  { 2, 3, 6, 8, 11, 12, },
  { 0, 2, 3, 6, 8, 11, 12, },
  { 1, 2, 3, 6, 8, 11, 12, },
  { 0, 1, 2, 3, 6, 8, 11, 12, },
  { 4, 6, 8, 11, 12, },
  { 0, 4, 6, 8, 11, 12, },
  { 1, 4, 6, 8, 11, 12, },
  { 0, 1, 4, 6, 8, 11, 12, },
  { 2, 4, 6, 8, 11, 12, },
  { 0, 2, 4, 6, 8, 11, 12, },
  { 1, 2, 4, 6, 8, 11, 12, },
  { 0, 1, 2, 4, 6, 8, 11, 12, },
  { 3, 4, 6, 8, 11, 12, },
  { 0, 3, 4, 6, 8, 11, 12, },
  { 1, 3, 4, 6, 8, 11, 12, },
  { 0, 1, 3, 4, 6, 8, 11, 12, },
  { 2, 3, 4, 6, 8, 11, 12, },
  { 0, 2, 3, 4, 6, 8, 11, 12, },
  { 1, 2, 3, 4, 6, 8, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 12, },
  { 5, 6, 8, 11, 12, },
  { 0, 5, 6, 8, 11, 12, },
  { 1, 5, 6, 8, 11, 12, },
  { 0, 1, 5, 6, 8, 11, 12, },
  { 2, 5, 6, 8, 11, 12, },
  { 0, 2, 5, 6, 8, 11, 12, },
  { 1, 2, 5, 6, 8, 11, 12, },
  { 0, 1, 2, 5, 6, 8, 11, 12, },
  { 3, 5, 6, 8, 11, 12, },
  { 0, 3, 5, 6, 8, 11, 12, },
  { 1, 3, 5, 6, 8, 11, 12, },
  { 0, 1, 3, 5, 6, 8, 11, 12, },
  { 2, 3, 5, 6, 8, 11, 12, },
  { 0, 2, 3, 5, 6, 8, 11, 12, },
  { 1, 2, 3, 5, 6, 8, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 12, },
  { 4, 5, 6, 8, 11, 12, },
  { 0, 4, 5, 6, 8, 11, 12, },
  { 1, 4, 5, 6, 8, 11, 12, },
  { 0, 1, 4, 5, 6, 8, 11, 12, },
  { 2, 4, 5, 6, 8, 11, 12, },
  { 0, 2, 4, 5, 6, 8, 11, 12, },
  { 1, 2, 4, 5, 6, 8, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 12, },
  { 3, 4, 5, 6, 8, 11, 12, },
  { 0, 3, 4, 5, 6, 8, 11, 12, },
  { 1, 3, 4, 5, 6, 8, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 12, },
  { 2, 3, 4, 5, 6, 8, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 12, },
  { 7, 8, 11, 12, },
  { 0, 7, 8, 11, 12, },
  { 1, 7, 8, 11, 12, },
  { 0, 1, 7, 8, 11, 12, },
  { 2, 7, 8, 11, 12, },
  { 0, 2, 7, 8, 11, 12, },
  { 1, 2, 7, 8, 11, 12, },
  { 0, 1, 2, 7, 8, 11, 12, },
  { 3, 7, 8, 11, 12, },
  { 0, 3, 7, 8, 11, 12, },
  { 1, 3, 7, 8, 11, 12, },
  { 0, 1, 3, 7, 8, 11, 12, },
  { 2, 3, 7, 8, 11, 12, },
  { 0, 2, 3, 7, 8, 11, 12, },
  { 1, 2, 3, 7, 8, 11, 12, },
  { 0, 1, 2, 3, 7, 8, 11, 12, },
  { 4, 7, 8, 11, 12, },
  { 0, 4, 7, 8, 11, 12, },
  { 1, 4, 7, 8, 11, 12, },
  { 0, 1, 4, 7, 8, 11, 12, },
  { 2, 4, 7, 8, 11, 12, },
  { 0, 2, 4, 7, 8, 11, 12, },
  { 1, 2, 4, 7, 8, 11, 12, },
  { 0, 1, 2, 4, 7, 8, 11, 12, },
  { 3, 4, 7, 8, 11, 12, },
  { 0, 3, 4, 7, 8, 11, 12, },
  { 1, 3, 4, 7, 8, 11, 12, },
  { 0, 1, 3, 4, 7, 8, 11, 12, },
  { 2, 3, 4, 7, 8, 11, 12, },
  { 0, 2, 3, 4, 7, 8, 11, 12, },
  { 1, 2, 3, 4, 7, 8, 11, 12, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 12, },
  { 5, 7, 8, 11, 12, },
  { 0, 5, 7, 8, 11, 12, },
  { 1, 5, 7, 8, 11, 12, },
  { 0, 1, 5, 7, 8, 11, 12, },
  { 2, 5, 7, 8, 11, 12, },
  { 0, 2, 5, 7, 8, 11, 12, },
  { 1, 2, 5, 7, 8, 11, 12, },
  { 0, 1, 2, 5, 7, 8, 11, 12, },
  { 3, 5, 7, 8, 11, 12, },
  { 0, 3, 5, 7, 8, 11, 12, },
  { 1, 3, 5, 7, 8, 11, 12, },
  { 0, 1, 3, 5, 7, 8, 11, 12, },
  { 2, 3, 5, 7, 8, 11, 12, },
  { 0, 2, 3, 5, 7, 8, 11, 12, },
  { 1, 2, 3, 5, 7, 8, 11, 12, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 12, },
  { 4, 5, 7, 8, 11, 12, },
  { 0, 4, 5, 7, 8, 11, 12, },
  { 1, 4, 5, 7, 8, 11, 12, },
  { 0, 1, 4, 5, 7, 8, 11, 12, },
  { 2, 4, 5, 7, 8, 11, 12, },
  { 0, 2, 4, 5, 7, 8, 11, 12, },
  { 1, 2, 4, 5, 7, 8, 11, 12, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 12, },
  { 3, 4, 5, 7, 8, 11, 12, },
  { 0, 3, 4, 5, 7, 8, 11, 12, },
  { 1, 3, 4, 5, 7, 8, 11, 12, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 12, },
  { 2, 3, 4, 5, 7, 8, 11, 12, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 12, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 12, },
  { 6, 7, 8, 11, 12, },
  { 0, 6, 7, 8, 11, 12, },
  { 1, 6, 7, 8, 11, 12, },
  { 0, 1, 6, 7, 8, 11, 12, },
  { 2, 6, 7, 8, 11, 12, },
  { 0, 2, 6, 7, 8, 11, 12, },
  { 1, 2, 6, 7, 8, 11, 12, },
  { 0, 1, 2, 6, 7, 8, 11, 12, },
  { 3, 6, 7, 8, 11, 12, },
  { 0, 3, 6, 7, 8, 11, 12, },
  { 1, 3, 6, 7, 8, 11, 12, },
  { 0, 1, 3, 6, 7, 8, 11, 12, },
  { 2, 3, 6, 7, 8, 11, 12, },
  { 0, 2, 3, 6, 7, 8, 11, 12, },
  { 1, 2, 3, 6, 7, 8, 11, 12, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 12, },
  { 4, 6, 7, 8, 11, 12, },
  { 0, 4, 6, 7, 8, 11, 12, },
  { 1, 4, 6, 7, 8, 11, 12, },
  { 0, 1, 4, 6, 7, 8, 11, 12, },
  { 2, 4, 6, 7, 8, 11, 12, },
  { 0, 2, 4, 6, 7, 8, 11, 12, },
  { 1, 2, 4, 6, 7, 8, 11, 12, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 12, },
  { 3, 4, 6, 7, 8, 11, 12, },
  { 0, 3, 4, 6, 7, 8, 11, 12, },
  { 1, 3, 4, 6, 7, 8, 11, 12, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 12, },
  { 2, 3, 4, 6, 7, 8, 11, 12, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 12, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 12, },
  { 5, 6, 7, 8, 11, 12, },
  { 0, 5, 6, 7, 8, 11, 12, },
  { 1, 5, 6, 7, 8, 11, 12, },
  { 0, 1, 5, 6, 7, 8, 11, 12, },
  { 2, 5, 6, 7, 8, 11, 12, },
  { 0, 2, 5, 6, 7, 8, 11, 12, },
  { 1, 2, 5, 6, 7, 8, 11, 12, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 12, },
  { 3, 5, 6, 7, 8, 11, 12, },
  { 0, 3, 5, 6, 7, 8, 11, 12, },
  { 1, 3, 5, 6, 7, 8, 11, 12, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 12, },
  { 2, 3, 5, 6, 7, 8, 11, 12, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 12, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 12, },
  { 4, 5, 6, 7, 8, 11, 12, },
  { 0, 4, 5, 6, 7, 8, 11, 12, },
  { 1, 4, 5, 6, 7, 8, 11, 12, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 12, },
  { 2, 4, 5, 6, 7, 8, 11, 12, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 12, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 12, },
  { 3, 4, 5, 6, 7, 8, 11, 12, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 12, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 12, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, },
  { 9, 11, 12, },
  { 0, 9, 11, 12, },
  { 1, 9, 11, 12, },
  { 0, 1, 9, 11, 12, },
  { 2, 9, 11, 12, },
  { 0, 2, 9, 11, 12, },
  { 1, 2, 9, 11, 12, },
  { 0, 1, 2, 9, 11, 12, },
  { 3, 9, 11, 12, },
  { 0, 3, 9, 11, 12, },
  { 1, 3, 9, 11, 12, },
  { 0, 1, 3, 9, 11, 12, },
  { 2, 3, 9, 11, 12, },
  { 0, 2, 3, 9, 11, 12, },
  { 1, 2, 3, 9, 11, 12, },
  { 0, 1, 2, 3, 9, 11, 12, },
  { 4, 9, 11, 12, },
  { 0, 4, 9, 11, 12, },
  { 1, 4, 9, 11, 12, },
  { 0, 1, 4, 9, 11, 12, },
  { 2, 4, 9, 11, 12, },
  { 0, 2, 4, 9, 11, 12, },
  { 1, 2, 4, 9, 11, 12, },
  { 0, 1, 2, 4, 9, 11, 12, },
  { 3, 4, 9, 11, 12, },
  { 0, 3, 4, 9, 11, 12, },
  { 1, 3, 4, 9, 11, 12, },
  { 0, 1, 3, 4, 9, 11, 12, },
  { 2, 3, 4, 9, 11, 12, },
  { 0, 2, 3, 4, 9, 11, 12, },
  { 1, 2, 3, 4, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 9, 11, 12, },
  { 5, 9, 11, 12, },
  { 0, 5, 9, 11, 12, },
  { 1, 5, 9, 11, 12, },
  { 0, 1, 5, 9, 11, 12, },
  { 2, 5, 9, 11, 12, },
  { 0, 2, 5, 9, 11, 12, },
  { 1, 2, 5, 9, 11, 12, },
  { 0, 1, 2, 5, 9, 11, 12, },
  { 3, 5, 9, 11, 12, },
  { 0, 3, 5, 9, 11, 12, },
  { 1, 3, 5, 9, 11, 12, },
  { 0, 1, 3, 5, 9, 11, 12, },
  { 2, 3, 5, 9, 11, 12, },
  { 0, 2, 3, 5, 9, 11, 12, },
  { 1, 2, 3, 5, 9, 11, 12, },
  { 0, 1, 2, 3, 5, 9, 11, 12, },
  { 4, 5, 9, 11, 12, },
  { 0, 4, 5, 9, 11, 12, },
  { 1, 4, 5, 9, 11, 12, },
  { 0, 1, 4, 5, 9, 11, 12, },
  { 2, 4, 5, 9, 11, 12, },
  { 0, 2, 4, 5, 9, 11, 12, },
  { 1, 2, 4, 5, 9, 11, 12, },
  { 0, 1, 2, 4, 5, 9, 11, 12, },
  { 3, 4, 5, 9, 11, 12, },
  { 0, 3, 4, 5, 9, 11, 12, },
  { 1, 3, 4, 5, 9, 11, 12, },
  { 0, 1, 3, 4, 5, 9, 11, 12, },
  { 2, 3, 4, 5, 9, 11, 12, },
  { 0, 2, 3, 4, 5, 9, 11, 12, },
  { 1, 2, 3, 4, 5, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 12, },
  { 6, 9, 11, 12, },
  { 0, 6, 9, 11, 12, },
  { 1, 6, 9, 11, 12, },
  { 0, 1, 6, 9, 11, 12, },
  { 2, 6, 9, 11, 12, },
  { 0, 2, 6, 9, 11, 12, },
  { 1, 2, 6, 9, 11, 12, },
  { 0, 1, 2, 6, 9, 11, 12, },
  { 3, 6, 9, 11, 12, },
  { 0, 3, 6, 9, 11, 12, },
  { 1, 3, 6, 9, 11, 12, },
  { 0, 1, 3, 6, 9, 11, 12, },
  { 2, 3, 6, 9, 11, 12, },
  { 0, 2, 3, 6, 9, 11, 12, },
  { 1, 2, 3, 6, 9, 11, 12, },
  { 0, 1, 2, 3, 6, 9, 11, 12, },
  { 4, 6, 9, 11, 12, },
  { 0, 4, 6, 9, 11, 12, },
  { 1, 4, 6, 9, 11, 12, },
  { 0, 1, 4, 6, 9, 11, 12, },
  { 2, 4, 6, 9, 11, 12, },
  { 0, 2, 4, 6, 9, 11, 12, },
  { 1, 2, 4, 6, 9, 11, 12, },
  { 0, 1, 2, 4, 6, 9, 11, 12, },
  { 3, 4, 6, 9, 11, 12, },
  { 0, 3, 4, 6, 9, 11, 12, },
  { 1, 3, 4, 6, 9, 11, 12, },
  { 0, 1, 3, 4, 6, 9, 11, 12, },
  { 2, 3, 4, 6, 9, 11, 12, },
  { 0, 2, 3, 4, 6, 9, 11, 12, },
  { 1, 2, 3, 4, 6, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 12, },
  { 5, 6, 9, 11, 12, },
  { 0, 5, 6, 9, 11, 12, },
  { 1, 5, 6, 9, 11, 12, },
  { 0, 1, 5, 6, 9, 11, 12, },
  { 2, 5, 6, 9, 11, 12, },
  { 0, 2, 5, 6, 9, 11, 12, },
  { 1, 2, 5, 6, 9, 11, 12, },
  { 0, 1, 2, 5, 6, 9, 11, 12, },
  { 3, 5, 6, 9, 11, 12, },
  { 0, 3, 5, 6, 9, 11, 12, },
  { 1, 3, 5, 6, 9, 11, 12, },
  { 0, 1, 3, 5, 6, 9, 11, 12, },
  { 2, 3, 5, 6, 9, 11, 12, },
  { 0, 2, 3, 5, 6, 9, 11, 12, },
  { 1, 2, 3, 5, 6, 9, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 12, },
  { 4, 5, 6, 9, 11, 12, },
  { 0, 4, 5, 6, 9, 11, 12, },
  { 1, 4, 5, 6, 9, 11, 12, },
  { 0, 1, 4, 5, 6, 9, 11, 12, },
  { 2, 4, 5, 6, 9, 11, 12, },
  { 0, 2, 4, 5, 6, 9, 11, 12, },
  { 1, 2, 4, 5, 6, 9, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 12, },
  { 3, 4, 5, 6, 9, 11, 12, },
  { 0, 3, 4, 5, 6, 9, 11, 12, },
  { 1, 3, 4, 5, 6, 9, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 12, },
  { 2, 3, 4, 5, 6, 9, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 12, },
  { 7, 9, 11, 12, },
  { 0, 7, 9, 11, 12, },
  { 1, 7, 9, 11, 12, },
  { 0, 1, 7, 9, 11, 12, },
  { 2, 7, 9, 11, 12, },
  { 0, 2, 7, 9, 11, 12, },
  { 1, 2, 7, 9, 11, 12, },
  { 0, 1, 2, 7, 9, 11, 12, },
  { 3, 7, 9, 11, 12, },
  { 0, 3, 7, 9, 11, 12, },
  { 1, 3, 7, 9, 11, 12, },
  { 0, 1, 3, 7, 9, 11, 12, },
  { 2, 3, 7, 9, 11, 12, },
  { 0, 2, 3, 7, 9, 11, 12, },
  { 1, 2, 3, 7, 9, 11, 12, },
  { 0, 1, 2, 3, 7, 9, 11, 12, },
  { 4, 7, 9, 11, 12, },
  { 0, 4, 7, 9, 11, 12, },
  { 1, 4, 7, 9, 11, 12, },
  { 0, 1, 4, 7, 9, 11, 12, },
  { 2, 4, 7, 9, 11, 12, },
  { 0, 2, 4, 7, 9, 11, 12, },
  { 1, 2, 4, 7, 9, 11, 12, },
  { 0, 1, 2, 4, 7, 9, 11, 12, },
  { 3, 4, 7, 9, 11, 12, },
  { 0, 3, 4, 7, 9, 11, 12, },
  { 1, 3, 4, 7, 9, 11, 12, },
  { 0, 1, 3, 4, 7, 9, 11, 12, },
  { 2, 3, 4, 7, 9, 11, 12, },
  { 0, 2, 3, 4, 7, 9, 11, 12, },
  { 1, 2, 3, 4, 7, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 12, },
  { 5, 7, 9, 11, 12, },
  { 0, 5, 7, 9, 11, 12, },
  { 1, 5, 7, 9, 11, 12, },
  { 0, 1, 5, 7, 9, 11, 12, },
  { 2, 5, 7, 9, 11, 12, },
  { 0, 2, 5, 7, 9, 11, 12, },
  { 1, 2, 5, 7, 9, 11, 12, },
  { 0, 1, 2, 5, 7, 9, 11, 12, },
  { 3, 5, 7, 9, 11, 12, },
  { 0, 3, 5, 7, 9, 11, 12, },
  { 1, 3, 5, 7, 9, 11, 12, },
  { 0, 1, 3, 5, 7, 9, 11, 12, },
  { 2, 3, 5, 7, 9, 11, 12, },
  { 0, 2, 3, 5, 7, 9, 11, 12, },
  { 1, 2, 3, 5, 7, 9, 11, 12, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 12, },
  { 4, 5, 7, 9, 11, 12, },
  { 0, 4, 5, 7, 9, 11, 12, },
  { 1, 4, 5, 7, 9, 11, 12, },
  { 0, 1, 4, 5, 7, 9, 11, 12, },
  { 2, 4, 5, 7, 9, 11, 12, },
  { 0, 2, 4, 5, 7, 9, 11, 12, },
  { 1, 2, 4, 5, 7, 9, 11, 12, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 12, },
  { 3, 4, 5, 7, 9, 11, 12, },
  { 0, 3, 4, 5, 7, 9, 11, 12, },
  { 1, 3, 4, 5, 7, 9, 11, 12, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 12, },
  { 2, 3, 4, 5, 7, 9, 11, 12, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 12, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 12, },
  { 6, 7, 9, 11, 12, },
  { 0, 6, 7, 9, 11, 12, },
  { 1, 6, 7, 9, 11, 12, },
  { 0, 1, 6, 7, 9, 11, 12, },
  { 2, 6, 7, 9, 11, 12, },
  { 0, 2, 6, 7, 9, 11, 12, },
  { 1, 2, 6, 7, 9, 11, 12, },
  { 0, 1, 2, 6, 7, 9, 11, 12, },
  { 3, 6, 7, 9, 11, 12, },
  { 0, 3, 6, 7, 9, 11, 12, },
  { 1, 3, 6, 7, 9, 11, 12, },
  { 0, 1, 3, 6, 7, 9, 11, 12, },
  { 2, 3, 6, 7, 9, 11, 12, },
  { 0, 2, 3, 6, 7, 9, 11, 12, },
  { 1, 2, 3, 6, 7, 9, 11, 12, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 12, },
  { 4, 6, 7, 9, 11, 12, },
  { 0, 4, 6, 7, 9, 11, 12, },
  { 1, 4, 6, 7, 9, 11, 12, },
  { 0, 1, 4, 6, 7, 9, 11, 12, },
  { 2, 4, 6, 7, 9, 11, 12, },
  { 0, 2, 4, 6, 7, 9, 11, 12, },
  { 1, 2, 4, 6, 7, 9, 11, 12, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 12, },
  { 3, 4, 6, 7, 9, 11, 12, },
  { 0, 3, 4, 6, 7, 9, 11, 12, },
  { 1, 3, 4, 6, 7, 9, 11, 12, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 12, },
  { 2, 3, 4, 6, 7, 9, 11, 12, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 12, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 12, },
  { 5, 6, 7, 9, 11, 12, },
  { 0, 5, 6, 7, 9, 11, 12, },
  { 1, 5, 6, 7, 9, 11, 12, },
  { 0, 1, 5, 6, 7, 9, 11, 12, },
  { 2, 5, 6, 7, 9, 11, 12, },
  { 0, 2, 5, 6, 7, 9, 11, 12, },
  { 1, 2, 5, 6, 7, 9, 11, 12, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 12, },
  { 3, 5, 6, 7, 9, 11, 12, },
  { 0, 3, 5, 6, 7, 9, 11, 12, },
  { 1, 3, 5, 6, 7, 9, 11, 12, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 12, },
  { 2, 3, 5, 6, 7, 9, 11, 12, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 12, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 12, },
  { 4, 5, 6, 7, 9, 11, 12, },
  { 0, 4, 5, 6, 7, 9, 11, 12, },
  { 1, 4, 5, 6, 7, 9, 11, 12, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 12, },
  { 2, 4, 5, 6, 7, 9, 11, 12, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 12, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 12, },
  { 3, 4, 5, 6, 7, 9, 11, 12, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 12, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 12, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, },
  { 8, 9, 11, 12, },
  { 0, 8, 9, 11, 12, },
  { 1, 8, 9, 11, 12, },
  { 0, 1, 8, 9, 11, 12, },
  { 2, 8, 9, 11, 12, },
  { 0, 2, 8, 9, 11, 12, },
  { 1, 2, 8, 9, 11, 12, },
  { 0, 1, 2, 8, 9, 11, 12, },
  { 3, 8, 9, 11, 12, },
  { 0, 3, 8, 9, 11, 12, },
  { 1, 3, 8, 9, 11, 12, },
  { 0, 1, 3, 8, 9, 11, 12, },
  { 2, 3, 8, 9, 11, 12, },
  { 0, 2, 3, 8, 9, 11, 12, },
  { 1, 2, 3, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 8, 9, 11, 12, },
  { 4, 8, 9, 11, 12, },
  { 0, 4, 8, 9, 11, 12, },
  { 1, 4, 8, 9, 11, 12, },
  { 0, 1, 4, 8, 9, 11, 12, },
  { 2, 4, 8, 9, 11, 12, },
  { 0, 2, 4, 8, 9, 11, 12, },
  { 1, 2, 4, 8, 9, 11, 12, },
  { 0, 1, 2, 4, 8, 9, 11, 12, },
  { 3, 4, 8, 9, 11, 12, },
  { 0, 3, 4, 8, 9, 11, 12, },
  { 1, 3, 4, 8, 9, 11, 12, },
  { 0, 1, 3, 4, 8, 9, 11, 12, },
  { 2, 3, 4, 8, 9, 11, 12, },
  { 0, 2, 3, 4, 8, 9, 11, 12, },
  { 1, 2, 3, 4, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 12, },
  { 5, 8, 9, 11, 12, },
  { 0, 5, 8, 9, 11, 12, },
  { 1, 5, 8, 9, 11, 12, },
  { 0, 1, 5, 8, 9, 11, 12, },
  { 2, 5, 8, 9, 11, 12, },
  { 0, 2, 5, 8, 9, 11, 12, },
  { 1, 2, 5, 8, 9, 11, 12, },
  { 0, 1, 2, 5, 8, 9, 11, 12, },
  { 3, 5, 8, 9, 11, 12, },
  { 0, 3, 5, 8, 9, 11, 12, },
  { 1, 3, 5, 8, 9, 11, 12, },
  { 0, 1, 3, 5, 8, 9, 11, 12, },
  { 2, 3, 5, 8, 9, 11, 12, },
  { 0, 2, 3, 5, 8, 9, 11, 12, },
  { 1, 2, 3, 5, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 12, },
  { 4, 5, 8, 9, 11, 12, },
  { 0, 4, 5, 8, 9, 11, 12, },
  { 1, 4, 5, 8, 9, 11, 12, },
  { 0, 1, 4, 5, 8, 9, 11, 12, },
  { 2, 4, 5, 8, 9, 11, 12, },
  { 0, 2, 4, 5, 8, 9, 11, 12, },
  { 1, 2, 4, 5, 8, 9, 11, 12, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 12, },
  { 3, 4, 5, 8, 9, 11, 12, },
  { 0, 3, 4, 5, 8, 9, 11, 12, },
  { 1, 3, 4, 5, 8, 9, 11, 12, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 12, },
  { 2, 3, 4, 5, 8, 9, 11, 12, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 12, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 12, },
  { 6, 8, 9, 11, 12, },
  { 0, 6, 8, 9, 11, 12, },
  { 1, 6, 8, 9, 11, 12, },
  { 0, 1, 6, 8, 9, 11, 12, },
  { 2, 6, 8, 9, 11, 12, },
  { 0, 2, 6, 8, 9, 11, 12, },
  { 1, 2, 6, 8, 9, 11, 12, },
  { 0, 1, 2, 6, 8, 9, 11, 12, },
  { 3, 6, 8, 9, 11, 12, },
  { 0, 3, 6, 8, 9, 11, 12, },
  { 1, 3, 6, 8, 9, 11, 12, },
  { 0, 1, 3, 6, 8, 9, 11, 12, },
  { 2, 3, 6, 8, 9, 11, 12, },
  { 0, 2, 3, 6, 8, 9, 11, 12, },
  { 1, 2, 3, 6, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 12, },
  { 4, 6, 8, 9, 11, 12, },
  { 0, 4, 6, 8, 9, 11, 12, },
  { 1, 4, 6, 8, 9, 11, 12, },
  { 0, 1, 4, 6, 8, 9, 11, 12, },
  { 2, 4, 6, 8, 9, 11, 12, },
  { 0, 2, 4, 6, 8, 9, 11, 12, },
  { 1, 2, 4, 6, 8, 9, 11, 12, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 12, },
  { 3, 4, 6, 8, 9, 11, 12, },
  { 0, 3, 4, 6, 8, 9, 11, 12, },
  { 1, 3, 4, 6, 8, 9, 11, 12, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 12, },
  { 2, 3, 4, 6, 8, 9, 11, 12, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 12, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 12, },
  { 5, 6, 8, 9, 11, 12, },
  { 0, 5, 6, 8, 9, 11, 12, },
  { 1, 5, 6, 8, 9, 11, 12, },
  { 0, 1, 5, 6, 8, 9, 11, 12, },
  { 2, 5, 6, 8, 9, 11, 12, },
  { 0, 2, 5, 6, 8, 9, 11, 12, },
  { 1, 2, 5, 6, 8, 9, 11, 12, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 12, },
  { 3, 5, 6, 8, 9, 11, 12, },
  { 0, 3, 5, 6, 8, 9, 11, 12, },
  { 1, 3, 5, 6, 8, 9, 11, 12, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 12, },
  { 2, 3, 5, 6, 8, 9, 11, 12, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 12, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 12, },
  { 4, 5, 6, 8, 9, 11, 12, },
  { 0, 4, 5, 6, 8, 9, 11, 12, },
  { 1, 4, 5, 6, 8, 9, 11, 12, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 12, },
  { 2, 4, 5, 6, 8, 9, 11, 12, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 12, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 12, },
  { 3, 4, 5, 6, 8, 9, 11, 12, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 12, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 12, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, },
  { 7, 8, 9, 11, 12, },
  { 0, 7, 8, 9, 11, 12, },
  { 1, 7, 8, 9, 11, 12, },
  { 0, 1, 7, 8, 9, 11, 12, },
  { 2, 7, 8, 9, 11, 12, },
  { 0, 2, 7, 8, 9, 11, 12, },
  { 1, 2, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 7, 8, 9, 11, 12, },
  { 3, 7, 8, 9, 11, 12, },
  { 0, 3, 7, 8, 9, 11, 12, },
  { 1, 3, 7, 8, 9, 11, 12, },
  { 0, 1, 3, 7, 8, 9, 11, 12, },
  { 2, 3, 7, 8, 9, 11, 12, },
  { 0, 2, 3, 7, 8, 9, 11, 12, },
  { 1, 2, 3, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 12, },
  { 4, 7, 8, 9, 11, 12, },
  { 0, 4, 7, 8, 9, 11, 12, },
  { 1, 4, 7, 8, 9, 11, 12, },
  { 0, 1, 4, 7, 8, 9, 11, 12, },
  { 2, 4, 7, 8, 9, 11, 12, },
  { 0, 2, 4, 7, 8, 9, 11, 12, },
  { 1, 2, 4, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 12, },
  { 3, 4, 7, 8, 9, 11, 12, },
  { 0, 3, 4, 7, 8, 9, 11, 12, },
  { 1, 3, 4, 7, 8, 9, 11, 12, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 12, },
  { 2, 3, 4, 7, 8, 9, 11, 12, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 12, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 12, },
  { 5, 7, 8, 9, 11, 12, },
  { 0, 5, 7, 8, 9, 11, 12, },
  { 1, 5, 7, 8, 9, 11, 12, },
  { 0, 1, 5, 7, 8, 9, 11, 12, },
  { 2, 5, 7, 8, 9, 11, 12, },
  { 0, 2, 5, 7, 8, 9, 11, 12, },
  { 1, 2, 5, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 12, },
  { 3, 5, 7, 8, 9, 11, 12, },
  { 0, 3, 5, 7, 8, 9, 11, 12, },
  { 1, 3, 5, 7, 8, 9, 11, 12, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 12, },
  { 2, 3, 5, 7, 8, 9, 11, 12, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 12, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 12, },
  { 4, 5, 7, 8, 9, 11, 12, },
  { 0, 4, 5, 7, 8, 9, 11, 12, },
  { 1, 4, 5, 7, 8, 9, 11, 12, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 12, },
  { 2, 4, 5, 7, 8, 9, 11, 12, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 12, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 12, },
  { 3, 4, 5, 7, 8, 9, 11, 12, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 12, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 12, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 12, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 12, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 12, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, },
  { 6, 7, 8, 9, 11, 12, },
  { 0, 6, 7, 8, 9, 11, 12, },
  { 1, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 6, 7, 8, 9, 11, 12, },
  { 2, 6, 7, 8, 9, 11, 12, },
  { 0, 2, 6, 7, 8, 9, 11, 12, },
  { 1, 2, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 12, },
  { 3, 6, 7, 8, 9, 11, 12, },
  { 0, 3, 6, 7, 8, 9, 11, 12, },
  { 1, 3, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 12, },
  { 2, 3, 6, 7, 8, 9, 11, 12, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 12, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 12, },
  { 4, 6, 7, 8, 9, 11, 12, },
  { 0, 4, 6, 7, 8, 9, 11, 12, },
  { 1, 4, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 12, },
  { 2, 4, 6, 7, 8, 9, 11, 12, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 12, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 12, },
  { 3, 4, 6, 7, 8, 9, 11, 12, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 12, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 12, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 12, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 12, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, },
  { 5, 6, 7, 8, 9, 11, 12, },
  { 0, 5, 6, 7, 8, 9, 11, 12, },
  { 1, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 12, },
  { 2, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 12, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 12, },
  { 3, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 12, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 12, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 12, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, },
  { 4, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, },
  { 10, 11, 12, },
  { 0, 10, 11, 12, },
  { 1, 10, 11, 12, },
  { 0, 1, 10, 11, 12, },
  { 2, 10, 11, 12, },
  { 0, 2, 10, 11, 12, },
  { 1, 2, 10, 11, 12, },
  { 0, 1, 2, 10, 11, 12, },
  { 3, 10, 11, 12, },
  { 0, 3, 10, 11, 12, },
  { 1, 3, 10, 11, 12, },
  { 0, 1, 3, 10, 11, 12, },
  { 2, 3, 10, 11, 12, },
  { 0, 2, 3, 10, 11, 12, },
  { 1, 2, 3, 10, 11, 12, },
  { 0, 1, 2, 3, 10, 11, 12, },
  { 4, 10, 11, 12, },
  { 0, 4, 10, 11, 12, },
  { 1, 4, 10, 11, 12, },
  { 0, 1, 4, 10, 11, 12, },
  { 2, 4, 10, 11, 12, },
  { 0, 2, 4, 10, 11, 12, },
  { 1, 2, 4, 10, 11, 12, },
  { 0, 1, 2, 4, 10, 11, 12, },
  { 3, 4, 10, 11, 12, },
  { 0, 3, 4, 10, 11, 12, },
  { 1, 3, 4, 10, 11, 12, },
  { 0, 1, 3, 4, 10, 11, 12, },
  { 2, 3, 4, 10, 11, 12, },
  { 0, 2, 3, 4, 10, 11, 12, },
  { 1, 2, 3, 4, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 10, 11, 12, },
  { 5, 10, 11, 12, },
  { 0, 5, 10, 11, 12, },
  { 1, 5, 10, 11, 12, },
  { 0, 1, 5, 10, 11, 12, },
  { 2, 5, 10, 11, 12, },
  { 0, 2, 5, 10, 11, 12, },
  { 1, 2, 5, 10, 11, 12, },
  { 0, 1, 2, 5, 10, 11, 12, },
  { 3, 5, 10, 11, 12, },
  { 0, 3, 5, 10, 11, 12, },
  { 1, 3, 5, 10, 11, 12, },
  { 0, 1, 3, 5, 10, 11, 12, },
  { 2, 3, 5, 10, 11, 12, },
  { 0, 2, 3, 5, 10, 11, 12, },
  { 1, 2, 3, 5, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 10, 11, 12, },
  { 4, 5, 10, 11, 12, },
  { 0, 4, 5, 10, 11, 12, },
  { 1, 4, 5, 10, 11, 12, },
  { 0, 1, 4, 5, 10, 11, 12, },
  { 2, 4, 5, 10, 11, 12, },
  { 0, 2, 4, 5, 10, 11, 12, },
  { 1, 2, 4, 5, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 10, 11, 12, },
  { 3, 4, 5, 10, 11, 12, },
  { 0, 3, 4, 5, 10, 11, 12, },
  { 1, 3, 4, 5, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 10, 11, 12, },
  { 2, 3, 4, 5, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 12, },
  { 6, 10, 11, 12, },
  { 0, 6, 10, 11, 12, },
  { 1, 6, 10, 11, 12, },
  { 0, 1, 6, 10, 11, 12, },
  { 2, 6, 10, 11, 12, },
  { 0, 2, 6, 10, 11, 12, },
  { 1, 2, 6, 10, 11, 12, },
  { 0, 1, 2, 6, 10, 11, 12, },
  { 3, 6, 10, 11, 12, },
  { 0, 3, 6, 10, 11, 12, },
  { 1, 3, 6, 10, 11, 12, },
  { 0, 1, 3, 6, 10, 11, 12, },
  { 2, 3, 6, 10, 11, 12, },
  { 0, 2, 3, 6, 10, 11, 12, },
  { 1, 2, 3, 6, 10, 11, 12, },
  { 0, 1, 2, 3, 6, 10, 11, 12, },
  { 4, 6, 10, 11, 12, },
  { 0, 4, 6, 10, 11, 12, },
  { 1, 4, 6, 10, 11, 12, },
  { 0, 1, 4, 6, 10, 11, 12, },
  { 2, 4, 6, 10, 11, 12, },
  { 0, 2, 4, 6, 10, 11, 12, },
  { 1, 2, 4, 6, 10, 11, 12, },
  { 0, 1, 2, 4, 6, 10, 11, 12, },
  { 3, 4, 6, 10, 11, 12, },
  { 0, 3, 4, 6, 10, 11, 12, },
  { 1, 3, 4, 6, 10, 11, 12, },
  { 0, 1, 3, 4, 6, 10, 11, 12, },
  { 2, 3, 4, 6, 10, 11, 12, },
  { 0, 2, 3, 4, 6, 10, 11, 12, },
  { 1, 2, 3, 4, 6, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 12, },
  { 5, 6, 10, 11, 12, },
  { 0, 5, 6, 10, 11, 12, },
  { 1, 5, 6, 10, 11, 12, },
  { 0, 1, 5, 6, 10, 11, 12, },
  { 2, 5, 6, 10, 11, 12, },
  { 0, 2, 5, 6, 10, 11, 12, },
  { 1, 2, 5, 6, 10, 11, 12, },
  { 0, 1, 2, 5, 6, 10, 11, 12, },
  { 3, 5, 6, 10, 11, 12, },
  { 0, 3, 5, 6, 10, 11, 12, },
  { 1, 3, 5, 6, 10, 11, 12, },
  { 0, 1, 3, 5, 6, 10, 11, 12, },
  { 2, 3, 5, 6, 10, 11, 12, },
  { 0, 2, 3, 5, 6, 10, 11, 12, },
  { 1, 2, 3, 5, 6, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 12, },
  { 4, 5, 6, 10, 11, 12, },
  { 0, 4, 5, 6, 10, 11, 12, },
  { 1, 4, 5, 6, 10, 11, 12, },
  { 0, 1, 4, 5, 6, 10, 11, 12, },
  { 2, 4, 5, 6, 10, 11, 12, },
  { 0, 2, 4, 5, 6, 10, 11, 12, },
  { 1, 2, 4, 5, 6, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 12, },
  { 3, 4, 5, 6, 10, 11, 12, },
  { 0, 3, 4, 5, 6, 10, 11, 12, },
  { 1, 3, 4, 5, 6, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 12, },
  { 2, 3, 4, 5, 6, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 12, },
  { 7, 10, 11, 12, },
  { 0, 7, 10, 11, 12, },
  { 1, 7, 10, 11, 12, },
  { 0, 1, 7, 10, 11, 12, },
  { 2, 7, 10, 11, 12, },
  { 0, 2, 7, 10, 11, 12, },
  { 1, 2, 7, 10, 11, 12, },
  { 0, 1, 2, 7, 10, 11, 12, },
  { 3, 7, 10, 11, 12, },
  { 0, 3, 7, 10, 11, 12, },
  { 1, 3, 7, 10, 11, 12, },
  { 0, 1, 3, 7, 10, 11, 12, },
  { 2, 3, 7, 10, 11, 12, },
  { 0, 2, 3, 7, 10, 11, 12, },
  { 1, 2, 3, 7, 10, 11, 12, },
  { 0, 1, 2, 3, 7, 10, 11, 12, },
  { 4, 7, 10, 11, 12, },
  { 0, 4, 7, 10, 11, 12, },
  { 1, 4, 7, 10, 11, 12, },
  { 0, 1, 4, 7, 10, 11, 12, },
  { 2, 4, 7, 10, 11, 12, },
  { 0, 2, 4, 7, 10, 11, 12, },
  { 1, 2, 4, 7, 10, 11, 12, },
  { 0, 1, 2, 4, 7, 10, 11, 12, },
  { 3, 4, 7, 10, 11, 12, },
  { 0, 3, 4, 7, 10, 11, 12, },
  { 1, 3, 4, 7, 10, 11, 12, },
  { 0, 1, 3, 4, 7, 10, 11, 12, },
  { 2, 3, 4, 7, 10, 11, 12, },
  { 0, 2, 3, 4, 7, 10, 11, 12, },
  { 1, 2, 3, 4, 7, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 12, },
  { 5, 7, 10, 11, 12, },
  { 0, 5, 7, 10, 11, 12, },
  { 1, 5, 7, 10, 11, 12, },
  { 0, 1, 5, 7, 10, 11, 12, },
  { 2, 5, 7, 10, 11, 12, },
  { 0, 2, 5, 7, 10, 11, 12, },
  { 1, 2, 5, 7, 10, 11, 12, },
  { 0, 1, 2, 5, 7, 10, 11, 12, },
  { 3, 5, 7, 10, 11, 12, },
  { 0, 3, 5, 7, 10, 11, 12, },
  { 1, 3, 5, 7, 10, 11, 12, },
  { 0, 1, 3, 5, 7, 10, 11, 12, },
  { 2, 3, 5, 7, 10, 11, 12, },
  { 0, 2, 3, 5, 7, 10, 11, 12, },
  { 1, 2, 3, 5, 7, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 12, },
  { 4, 5, 7, 10, 11, 12, },
  { 0, 4, 5, 7, 10, 11, 12, },
  { 1, 4, 5, 7, 10, 11, 12, },
  { 0, 1, 4, 5, 7, 10, 11, 12, },
  { 2, 4, 5, 7, 10, 11, 12, },
  { 0, 2, 4, 5, 7, 10, 11, 12, },
  { 1, 2, 4, 5, 7, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 12, },
  { 3, 4, 5, 7, 10, 11, 12, },
  { 0, 3, 4, 5, 7, 10, 11, 12, },
  { 1, 3, 4, 5, 7, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 12, },
  { 2, 3, 4, 5, 7, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 12, },
  { 6, 7, 10, 11, 12, },
  { 0, 6, 7, 10, 11, 12, },
  { 1, 6, 7, 10, 11, 12, },
  { 0, 1, 6, 7, 10, 11, 12, },
  { 2, 6, 7, 10, 11, 12, },
  { 0, 2, 6, 7, 10, 11, 12, },
  { 1, 2, 6, 7, 10, 11, 12, },
  { 0, 1, 2, 6, 7, 10, 11, 12, },
  { 3, 6, 7, 10, 11, 12, },
  { 0, 3, 6, 7, 10, 11, 12, },
  { 1, 3, 6, 7, 10, 11, 12, },
  { 0, 1, 3, 6, 7, 10, 11, 12, },
  { 2, 3, 6, 7, 10, 11, 12, },
  { 0, 2, 3, 6, 7, 10, 11, 12, },
  { 1, 2, 3, 6, 7, 10, 11, 12, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 12, },
  { 4, 6, 7, 10, 11, 12, },
  { 0, 4, 6, 7, 10, 11, 12, },
  { 1, 4, 6, 7, 10, 11, 12, },
  { 0, 1, 4, 6, 7, 10, 11, 12, },
  { 2, 4, 6, 7, 10, 11, 12, },
  { 0, 2, 4, 6, 7, 10, 11, 12, },
  { 1, 2, 4, 6, 7, 10, 11, 12, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 12, },
  { 3, 4, 6, 7, 10, 11, 12, },
  { 0, 3, 4, 6, 7, 10, 11, 12, },
  { 1, 3, 4, 6, 7, 10, 11, 12, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 12, },
  { 2, 3, 4, 6, 7, 10, 11, 12, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 12, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 12, },
  { 5, 6, 7, 10, 11, 12, },
  { 0, 5, 6, 7, 10, 11, 12, },
  { 1, 5, 6, 7, 10, 11, 12, },
  { 0, 1, 5, 6, 7, 10, 11, 12, },
  { 2, 5, 6, 7, 10, 11, 12, },
  { 0, 2, 5, 6, 7, 10, 11, 12, },
  { 1, 2, 5, 6, 7, 10, 11, 12, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 12, },
  { 3, 5, 6, 7, 10, 11, 12, },
  { 0, 3, 5, 6, 7, 10, 11, 12, },
  { 1, 3, 5, 6, 7, 10, 11, 12, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 12, },
  { 2, 3, 5, 6, 7, 10, 11, 12, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 12, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 12, },
  { 4, 5, 6, 7, 10, 11, 12, },
  { 0, 4, 5, 6, 7, 10, 11, 12, },
  { 1, 4, 5, 6, 7, 10, 11, 12, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 12, },
  { 2, 4, 5, 6, 7, 10, 11, 12, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 12, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 12, },
  { 3, 4, 5, 6, 7, 10, 11, 12, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 12, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 12, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, },
  { 8, 10, 11, 12, },
  { 0, 8, 10, 11, 12, },
  { 1, 8, 10, 11, 12, },
  { 0, 1, 8, 10, 11, 12, },
  { 2, 8, 10, 11, 12, },
  { 0, 2, 8, 10, 11, 12, },
  { 1, 2, 8, 10, 11, 12, },
  { 0, 1, 2, 8, 10, 11, 12, },
  { 3, 8, 10, 11, 12, },
  { 0, 3, 8, 10, 11, 12, },
  { 1, 3, 8, 10, 11, 12, },
  { 0, 1, 3, 8, 10, 11, 12, },
  { 2, 3, 8, 10, 11, 12, },
  { 0, 2, 3, 8, 10, 11, 12, },
  { 1, 2, 3, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 8, 10, 11, 12, },
  { 4, 8, 10, 11, 12, },
  { 0, 4, 8, 10, 11, 12, },
  { 1, 4, 8, 10, 11, 12, },
  { 0, 1, 4, 8, 10, 11, 12, },
  { 2, 4, 8, 10, 11, 12, },
  { 0, 2, 4, 8, 10, 11, 12, },
  { 1, 2, 4, 8, 10, 11, 12, },
  { 0, 1, 2, 4, 8, 10, 11, 12, },
  { 3, 4, 8, 10, 11, 12, },
  { 0, 3, 4, 8, 10, 11, 12, },
  { 1, 3, 4, 8, 10, 11, 12, },
  { 0, 1, 3, 4, 8, 10, 11, 12, },
  { 2, 3, 4, 8, 10, 11, 12, },
  { 0, 2, 3, 4, 8, 10, 11, 12, },
  { 1, 2, 3, 4, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 12, },
  { 5, 8, 10, 11, 12, },
  { 0, 5, 8, 10, 11, 12, },
  { 1, 5, 8, 10, 11, 12, },
  { 0, 1, 5, 8, 10, 11, 12, },
  { 2, 5, 8, 10, 11, 12, },
  { 0, 2, 5, 8, 10, 11, 12, },
  { 1, 2, 5, 8, 10, 11, 12, },
  { 0, 1, 2, 5, 8, 10, 11, 12, },
  { 3, 5, 8, 10, 11, 12, },
  { 0, 3, 5, 8, 10, 11, 12, },
  { 1, 3, 5, 8, 10, 11, 12, },
  { 0, 1, 3, 5, 8, 10, 11, 12, },
  { 2, 3, 5, 8, 10, 11, 12, },
  { 0, 2, 3, 5, 8, 10, 11, 12, },
  { 1, 2, 3, 5, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 12, },
  { 4, 5, 8, 10, 11, 12, },
  { 0, 4, 5, 8, 10, 11, 12, },
  { 1, 4, 5, 8, 10, 11, 12, },
  { 0, 1, 4, 5, 8, 10, 11, 12, },
  { 2, 4, 5, 8, 10, 11, 12, },
  { 0, 2, 4, 5, 8, 10, 11, 12, },
  { 1, 2, 4, 5, 8, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 12, },
  { 3, 4, 5, 8, 10, 11, 12, },
  { 0, 3, 4, 5, 8, 10, 11, 12, },
  { 1, 3, 4, 5, 8, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 12, },
  { 2, 3, 4, 5, 8, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 12, },
  { 6, 8, 10, 11, 12, },
  { 0, 6, 8, 10, 11, 12, },
  { 1, 6, 8, 10, 11, 12, },
  { 0, 1, 6, 8, 10, 11, 12, },
  { 2, 6, 8, 10, 11, 12, },
  { 0, 2, 6, 8, 10, 11, 12, },
  { 1, 2, 6, 8, 10, 11, 12, },
  { 0, 1, 2, 6, 8, 10, 11, 12, },
  { 3, 6, 8, 10, 11, 12, },
  { 0, 3, 6, 8, 10, 11, 12, },
  { 1, 3, 6, 8, 10, 11, 12, },
  { 0, 1, 3, 6, 8, 10, 11, 12, },
  { 2, 3, 6, 8, 10, 11, 12, },
  { 0, 2, 3, 6, 8, 10, 11, 12, },
  { 1, 2, 3, 6, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 12, },
  { 4, 6, 8, 10, 11, 12, },
  { 0, 4, 6, 8, 10, 11, 12, },
  { 1, 4, 6, 8, 10, 11, 12, },
  { 0, 1, 4, 6, 8, 10, 11, 12, },
  { 2, 4, 6, 8, 10, 11, 12, },
  { 0, 2, 4, 6, 8, 10, 11, 12, },
  { 1, 2, 4, 6, 8, 10, 11, 12, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 12, },
  { 3, 4, 6, 8, 10, 11, 12, },
  { 0, 3, 4, 6, 8, 10, 11, 12, },
  { 1, 3, 4, 6, 8, 10, 11, 12, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 12, },
  { 2, 3, 4, 6, 8, 10, 11, 12, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 12, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 12, },
  { 5, 6, 8, 10, 11, 12, },
  { 0, 5, 6, 8, 10, 11, 12, },
  { 1, 5, 6, 8, 10, 11, 12, },
  { 0, 1, 5, 6, 8, 10, 11, 12, },
  { 2, 5, 6, 8, 10, 11, 12, },
  { 0, 2, 5, 6, 8, 10, 11, 12, },
  { 1, 2, 5, 6, 8, 10, 11, 12, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 12, },
  { 3, 5, 6, 8, 10, 11, 12, },
  { 0, 3, 5, 6, 8, 10, 11, 12, },
  { 1, 3, 5, 6, 8, 10, 11, 12, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 12, },
  { 2, 3, 5, 6, 8, 10, 11, 12, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 12, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 12, },
  { 4, 5, 6, 8, 10, 11, 12, },
  { 0, 4, 5, 6, 8, 10, 11, 12, },
  { 1, 4, 5, 6, 8, 10, 11, 12, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 12, },
  { 2, 4, 5, 6, 8, 10, 11, 12, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 12, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 12, },
  { 3, 4, 5, 6, 8, 10, 11, 12, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 12, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 12, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, },
  { 7, 8, 10, 11, 12, },
  { 0, 7, 8, 10, 11, 12, },
  { 1, 7, 8, 10, 11, 12, },
  { 0, 1, 7, 8, 10, 11, 12, },
  { 2, 7, 8, 10, 11, 12, },
  { 0, 2, 7, 8, 10, 11, 12, },
  { 1, 2, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 7, 8, 10, 11, 12, },
  { 3, 7, 8, 10, 11, 12, },
  { 0, 3, 7, 8, 10, 11, 12, },
  { 1, 3, 7, 8, 10, 11, 12, },
  { 0, 1, 3, 7, 8, 10, 11, 12, },
  { 2, 3, 7, 8, 10, 11, 12, },
  { 0, 2, 3, 7, 8, 10, 11, 12, },
  { 1, 2, 3, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 12, },
  { 4, 7, 8, 10, 11, 12, },
  { 0, 4, 7, 8, 10, 11, 12, },
  { 1, 4, 7, 8, 10, 11, 12, },
  { 0, 1, 4, 7, 8, 10, 11, 12, },
  { 2, 4, 7, 8, 10, 11, 12, },
  { 0, 2, 4, 7, 8, 10, 11, 12, },
  { 1, 2, 4, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 12, },
  { 3, 4, 7, 8, 10, 11, 12, },
  { 0, 3, 4, 7, 8, 10, 11, 12, },
  { 1, 3, 4, 7, 8, 10, 11, 12, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 12, },
  { 2, 3, 4, 7, 8, 10, 11, 12, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 12, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 12, },
  { 5, 7, 8, 10, 11, 12, },
  { 0, 5, 7, 8, 10, 11, 12, },
  { 1, 5, 7, 8, 10, 11, 12, },
  { 0, 1, 5, 7, 8, 10, 11, 12, },
  { 2, 5, 7, 8, 10, 11, 12, },
  { 0, 2, 5, 7, 8, 10, 11, 12, },
  { 1, 2, 5, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 12, },
  { 3, 5, 7, 8, 10, 11, 12, },
  { 0, 3, 5, 7, 8, 10, 11, 12, },
  { 1, 3, 5, 7, 8, 10, 11, 12, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 12, },
  { 2, 3, 5, 7, 8, 10, 11, 12, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 12, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 12, },
  { 4, 5, 7, 8, 10, 11, 12, },
  { 0, 4, 5, 7, 8, 10, 11, 12, },
  { 1, 4, 5, 7, 8, 10, 11, 12, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 12, },
  { 2, 4, 5, 7, 8, 10, 11, 12, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 12, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, },
  { 3, 4, 5, 7, 8, 10, 11, 12, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 12, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 12, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, },
  { 6, 7, 8, 10, 11, 12, },
  { 0, 6, 7, 8, 10, 11, 12, },
  { 1, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 6, 7, 8, 10, 11, 12, },
  { 2, 6, 7, 8, 10, 11, 12, },
  { 0, 2, 6, 7, 8, 10, 11, 12, },
  { 1, 2, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 12, },
  { 3, 6, 7, 8, 10, 11, 12, },
  { 0, 3, 6, 7, 8, 10, 11, 12, },
  { 1, 3, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 12, },
  { 2, 3, 6, 7, 8, 10, 11, 12, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 12, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 12, },
  { 4, 6, 7, 8, 10, 11, 12, },
  { 0, 4, 6, 7, 8, 10, 11, 12, },
  { 1, 4, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 12, },
  { 2, 4, 6, 7, 8, 10, 11, 12, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 12, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 12, },
  { 3, 4, 6, 7, 8, 10, 11, 12, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 12, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 12, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 12, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 12, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, },
  { 5, 6, 7, 8, 10, 11, 12, },
  { 0, 5, 6, 7, 8, 10, 11, 12, },
  { 1, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 12, },
  { 2, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 12, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 12, },
  { 3, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 12, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 12, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 12, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, },
  { 4, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, },
  { 9, 10, 11, 12, },
  { 0, 9, 10, 11, 12, },
  { 1, 9, 10, 11, 12, },
  { 0, 1, 9, 10, 11, 12, },
  { 2, 9, 10, 11, 12, },
  { 0, 2, 9, 10, 11, 12, },
  { 1, 2, 9, 10, 11, 12, },
  { 0, 1, 2, 9, 10, 11, 12, },
  { 3, 9, 10, 11, 12, },
  { 0, 3, 9, 10, 11, 12, },
  { 1, 3, 9, 10, 11, 12, },
  { 0, 1, 3, 9, 10, 11, 12, },
  { 2, 3, 9, 10, 11, 12, },
  { 0, 2, 3, 9, 10, 11, 12, },
  { 1, 2, 3, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 9, 10, 11, 12, },
  { 4, 9, 10, 11, 12, },
  { 0, 4, 9, 10, 11, 12, },
  { 1, 4, 9, 10, 11, 12, },
  { 0, 1, 4, 9, 10, 11, 12, },
  { 2, 4, 9, 10, 11, 12, },
  { 0, 2, 4, 9, 10, 11, 12, },
  { 1, 2, 4, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 9, 10, 11, 12, },
  { 3, 4, 9, 10, 11, 12, },
  { 0, 3, 4, 9, 10, 11, 12, },
  { 1, 3, 4, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 9, 10, 11, 12, },
  { 2, 3, 4, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 12, },
  { 5, 9, 10, 11, 12, },
  { 0, 5, 9, 10, 11, 12, },
  { 1, 5, 9, 10, 11, 12, },
  { 0, 1, 5, 9, 10, 11, 12, },
  { 2, 5, 9, 10, 11, 12, },
  { 0, 2, 5, 9, 10, 11, 12, },
  { 1, 2, 5, 9, 10, 11, 12, },
  { 0, 1, 2, 5, 9, 10, 11, 12, },
  { 3, 5, 9, 10, 11, 12, },
  { 0, 3, 5, 9, 10, 11, 12, },
  { 1, 3, 5, 9, 10, 11, 12, },
  { 0, 1, 3, 5, 9, 10, 11, 12, },
  { 2, 3, 5, 9, 10, 11, 12, },
  { 0, 2, 3, 5, 9, 10, 11, 12, },
  { 1, 2, 3, 5, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 12, },
  { 4, 5, 9, 10, 11, 12, },
  { 0, 4, 5, 9, 10, 11, 12, },
  { 1, 4, 5, 9, 10, 11, 12, },
  { 0, 1, 4, 5, 9, 10, 11, 12, },
  { 2, 4, 5, 9, 10, 11, 12, },
  { 0, 2, 4, 5, 9, 10, 11, 12, },
  { 1, 2, 4, 5, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 12, },
  { 3, 4, 5, 9, 10, 11, 12, },
  { 0, 3, 4, 5, 9, 10, 11, 12, },
  { 1, 3, 4, 5, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 12, },
  { 2, 3, 4, 5, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, },
  { 6, 9, 10, 11, 12, },
  { 0, 6, 9, 10, 11, 12, },
  { 1, 6, 9, 10, 11, 12, },
  { 0, 1, 6, 9, 10, 11, 12, },
  { 2, 6, 9, 10, 11, 12, },
  { 0, 2, 6, 9, 10, 11, 12, },
  { 1, 2, 6, 9, 10, 11, 12, },
  { 0, 1, 2, 6, 9, 10, 11, 12, },
  { 3, 6, 9, 10, 11, 12, },
  { 0, 3, 6, 9, 10, 11, 12, },
  { 1, 3, 6, 9, 10, 11, 12, },
  { 0, 1, 3, 6, 9, 10, 11, 12, },
  { 2, 3, 6, 9, 10, 11, 12, },
  { 0, 2, 3, 6, 9, 10, 11, 12, },
  { 1, 2, 3, 6, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 12, },
  { 4, 6, 9, 10, 11, 12, },
  { 0, 4, 6, 9, 10, 11, 12, },
  { 1, 4, 6, 9, 10, 11, 12, },
  { 0, 1, 4, 6, 9, 10, 11, 12, },
  { 2, 4, 6, 9, 10, 11, 12, },
  { 0, 2, 4, 6, 9, 10, 11, 12, },
  { 1, 2, 4, 6, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 12, },
  { 3, 4, 6, 9, 10, 11, 12, },
  { 0, 3, 4, 6, 9, 10, 11, 12, },
  { 1, 3, 4, 6, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 12, },
  { 2, 3, 4, 6, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 12, },
  { 5, 6, 9, 10, 11, 12, },
  { 0, 5, 6, 9, 10, 11, 12, },
  { 1, 5, 6, 9, 10, 11, 12, },
  { 0, 1, 5, 6, 9, 10, 11, 12, },
  { 2, 5, 6, 9, 10, 11, 12, },
  { 0, 2, 5, 6, 9, 10, 11, 12, },
  { 1, 2, 5, 6, 9, 10, 11, 12, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 12, },
  { 3, 5, 6, 9, 10, 11, 12, },
  { 0, 3, 5, 6, 9, 10, 11, 12, },
  { 1, 3, 5, 6, 9, 10, 11, 12, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 12, },
  { 2, 3, 5, 6, 9, 10, 11, 12, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 12, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 12, },
  { 4, 5, 6, 9, 10, 11, 12, },
  { 0, 4, 5, 6, 9, 10, 11, 12, },
  { 1, 4, 5, 6, 9, 10, 11, 12, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 12, },
  { 2, 4, 5, 6, 9, 10, 11, 12, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 12, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 12, },
  { 3, 4, 5, 6, 9, 10, 11, 12, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 12, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 12, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, },
  { 7, 9, 10, 11, 12, },
  { 0, 7, 9, 10, 11, 12, },
  { 1, 7, 9, 10, 11, 12, },
  { 0, 1, 7, 9, 10, 11, 12, },
  { 2, 7, 9, 10, 11, 12, },
  { 0, 2, 7, 9, 10, 11, 12, },
  { 1, 2, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 7, 9, 10, 11, 12, },
  { 3, 7, 9, 10, 11, 12, },
  { 0, 3, 7, 9, 10, 11, 12, },
  { 1, 3, 7, 9, 10, 11, 12, },
  { 0, 1, 3, 7, 9, 10, 11, 12, },
  { 2, 3, 7, 9, 10, 11, 12, },
  { 0, 2, 3, 7, 9, 10, 11, 12, },
  { 1, 2, 3, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 12, },
  { 4, 7, 9, 10, 11, 12, },
  { 0, 4, 7, 9, 10, 11, 12, },
  { 1, 4, 7, 9, 10, 11, 12, },
  { 0, 1, 4, 7, 9, 10, 11, 12, },
  { 2, 4, 7, 9, 10, 11, 12, },
  { 0, 2, 4, 7, 9, 10, 11, 12, },
  { 1, 2, 4, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 12, },
  { 3, 4, 7, 9, 10, 11, 12, },
  { 0, 3, 4, 7, 9, 10, 11, 12, },
  { 1, 3, 4, 7, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 12, },
  { 2, 3, 4, 7, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 12, },
  { 5, 7, 9, 10, 11, 12, },
  { 0, 5, 7, 9, 10, 11, 12, },
  { 1, 5, 7, 9, 10, 11, 12, },
  { 0, 1, 5, 7, 9, 10, 11, 12, },
  { 2, 5, 7, 9, 10, 11, 12, },
  { 0, 2, 5, 7, 9, 10, 11, 12, },
  { 1, 2, 5, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 12, },
  { 3, 5, 7, 9, 10, 11, 12, },
  { 0, 3, 5, 7, 9, 10, 11, 12, },
  { 1, 3, 5, 7, 9, 10, 11, 12, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 12, },
  { 2, 3, 5, 7, 9, 10, 11, 12, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 12, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 12, },
  { 4, 5, 7, 9, 10, 11, 12, },
  { 0, 4, 5, 7, 9, 10, 11, 12, },
  { 1, 4, 5, 7, 9, 10, 11, 12, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 12, },
  { 2, 4, 5, 7, 9, 10, 11, 12, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 12, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 12, },
  { 3, 4, 5, 7, 9, 10, 11, 12, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 12, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 12, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, },
  { 6, 7, 9, 10, 11, 12, },
  { 0, 6, 7, 9, 10, 11, 12, },
  { 1, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 6, 7, 9, 10, 11, 12, },
  { 2, 6, 7, 9, 10, 11, 12, },
  { 0, 2, 6, 7, 9, 10, 11, 12, },
  { 1, 2, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 12, },
  { 3, 6, 7, 9, 10, 11, 12, },
  { 0, 3, 6, 7, 9, 10, 11, 12, },
  { 1, 3, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 12, },
  { 2, 3, 6, 7, 9, 10, 11, 12, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 12, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 12, },
  { 4, 6, 7, 9, 10, 11, 12, },
  { 0, 4, 6, 7, 9, 10, 11, 12, },
  { 1, 4, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 12, },
  { 2, 4, 6, 7, 9, 10, 11, 12, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 12, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 12, },
  { 3, 4, 6, 7, 9, 10, 11, 12, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 12, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 12, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, },
  { 5, 6, 7, 9, 10, 11, 12, },
  { 0, 5, 6, 7, 9, 10, 11, 12, },
  { 1, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 12, },
  { 2, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 12, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 12, },
  { 3, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 12, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 12, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 12, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, },
  { 4, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, },
  { 8, 9, 10, 11, 12, },
  { 0, 8, 9, 10, 11, 12, },
  { 1, 8, 9, 10, 11, 12, },
  { 0, 1, 8, 9, 10, 11, 12, },
  { 2, 8, 9, 10, 11, 12, },
  { 0, 2, 8, 9, 10, 11, 12, },
  { 1, 2, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 8, 9, 10, 11, 12, },
  { 3, 8, 9, 10, 11, 12, },
  { 0, 3, 8, 9, 10, 11, 12, },
  { 1, 3, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 8, 9, 10, 11, 12, },
  { 2, 3, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 12, },
  { 4, 8, 9, 10, 11, 12, },
  { 0, 4, 8, 9, 10, 11, 12, },
  { 1, 4, 8, 9, 10, 11, 12, },
  { 0, 1, 4, 8, 9, 10, 11, 12, },
  { 2, 4, 8, 9, 10, 11, 12, },
  { 0, 2, 4, 8, 9, 10, 11, 12, },
  { 1, 2, 4, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 12, },
  { 3, 4, 8, 9, 10, 11, 12, },
  { 0, 3, 4, 8, 9, 10, 11, 12, },
  { 1, 3, 4, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 12, },
  { 2, 3, 4, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 12, },
  { 5, 8, 9, 10, 11, 12, },
  { 0, 5, 8, 9, 10, 11, 12, },
  { 1, 5, 8, 9, 10, 11, 12, },
  { 0, 1, 5, 8, 9, 10, 11, 12, },
  { 2, 5, 8, 9, 10, 11, 12, },
  { 0, 2, 5, 8, 9, 10, 11, 12, },
  { 1, 2, 5, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 12, },
  { 3, 5, 8, 9, 10, 11, 12, },
  { 0, 3, 5, 8, 9, 10, 11, 12, },
  { 1, 3, 5, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 12, },
  { 2, 3, 5, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 12, },
  { 4, 5, 8, 9, 10, 11, 12, },
  { 0, 4, 5, 8, 9, 10, 11, 12, },
  { 1, 4, 5, 8, 9, 10, 11, 12, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 12, },
  { 2, 4, 5, 8, 9, 10, 11, 12, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 12, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 12, },
  { 3, 4, 5, 8, 9, 10, 11, 12, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 12, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 12, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, },
  { 6, 8, 9, 10, 11, 12, },
  { 0, 6, 8, 9, 10, 11, 12, },
  { 1, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 6, 8, 9, 10, 11, 12, },
  { 2, 6, 8, 9, 10, 11, 12, },
  { 0, 2, 6, 8, 9, 10, 11, 12, },
  { 1, 2, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 12, },
  { 3, 6, 8, 9, 10, 11, 12, },
  { 0, 3, 6, 8, 9, 10, 11, 12, },
  { 1, 3, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 12, },
  { 2, 3, 6, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 12, },
  { 4, 6, 8, 9, 10, 11, 12, },
  { 0, 4, 6, 8, 9, 10, 11, 12, },
  { 1, 4, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 12, },
  { 2, 4, 6, 8, 9, 10, 11, 12, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 12, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 12, },
  { 3, 4, 6, 8, 9, 10, 11, 12, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 12, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 12, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, },
  { 5, 6, 8, 9, 10, 11, 12, },
  { 0, 5, 6, 8, 9, 10, 11, 12, },
  { 1, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 12, },
  { 2, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 12, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 12, },
  { 3, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 12, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 12, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, },
  { 4, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, },
  { 7, 8, 9, 10, 11, 12, },
  { 0, 7, 8, 9, 10, 11, 12, },
  { 1, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 7, 8, 9, 10, 11, 12, },
  { 2, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 12, },
  { 3, 7, 8, 9, 10, 11, 12, },
  { 0, 3, 7, 8, 9, 10, 11, 12, },
  { 1, 3, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 12, },
  { 2, 3, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 12, },
  { 4, 7, 8, 9, 10, 11, 12, },
  { 0, 4, 7, 8, 9, 10, 11, 12, },
  { 1, 4, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 12, },
  { 2, 4, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 12, },
  { 3, 4, 7, 8, 9, 10, 11, 12, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 12, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 12, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, },
  { 5, 7, 8, 9, 10, 11, 12, },
  { 0, 5, 7, 8, 9, 10, 11, 12, },
  { 1, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 12, },
  { 2, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 12, },
  { 3, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 12, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 12, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, },
  { 4, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, },
  { 6, 7, 8, 9, 10, 11, 12, },
  { 0, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 12, },
  { 2, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 12, },
  { 3, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 12, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, },
  { 4, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, },
  { 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, },
  { 13, },
  { 0, 13, },
  { 1, 13, },
  { 0, 1, 13, },
  { 2, 13, },
  { 0, 2, 13, },
  { 1, 2, 13, },
  { 0, 1, 2, 13, },
  { 3, 13, },
  { 0, 3, 13, },
  { 1, 3, 13, },
  { 0, 1, 3, 13, },
  { 2, 3, 13, },
  { 0, 2, 3, 13, },
  { 1, 2, 3, 13, },
  { 0, 1, 2, 3, 13, },
  { 4, 13, },
  { 0, 4, 13, },
  { 1, 4, 13, },
  { 0, 1, 4, 13, },
  { 2, 4, 13, },
  { 0, 2, 4, 13, },
  { 1, 2, 4, 13, },
  { 0, 1, 2, 4, 13, },
  { 3, 4, 13, },
  { 0, 3, 4, 13, },
  { 1, 3, 4, 13, },
  { 0, 1, 3, 4, 13, },
  { 2, 3, 4, 13, },
  { 0, 2, 3, 4, 13, },
  { 1, 2, 3, 4, 13, },
  { 0, 1, 2, 3, 4, 13, },
  { 5, 13, },
  { 0, 5, 13, },
  { 1, 5, 13, },
  { 0, 1, 5, 13, },
  { 2, 5, 13, },
  { 0, 2, 5, 13, },
  { 1, 2, 5, 13, },
  { 0, 1, 2, 5, 13, },
  { 3, 5, 13, },
  { 0, 3, 5, 13, },
  { 1, 3, 5, 13, },
  { 0, 1, 3, 5, 13, },
  { 2, 3, 5, 13, },
  { 0, 2, 3, 5, 13, },
  { 1, 2, 3, 5, 13, },
  { 0, 1, 2, 3, 5, 13, },
  { 4, 5, 13, },
  { 0, 4, 5, 13, },
  { 1, 4, 5, 13, },
  { 0, 1, 4, 5, 13, },
  { 2, 4, 5, 13, },
  { 0, 2, 4, 5, 13, },
  { 1, 2, 4, 5, 13, },
  { 0, 1, 2, 4, 5, 13, },
  { 3, 4, 5, 13, },
  { 0, 3, 4, 5, 13, },
  { 1, 3, 4, 5, 13, },
  { 0, 1, 3, 4, 5, 13, },
  { 2, 3, 4, 5, 13, },
  { 0, 2, 3, 4, 5, 13, },
  { 1, 2, 3, 4, 5, 13, },
  { 0, 1, 2, 3, 4, 5, 13, },
  { 6, 13, },
  { 0, 6, 13, },
  { 1, 6, 13, },
  { 0, 1, 6, 13, },
  { 2, 6, 13, },
  { 0, 2, 6, 13, },
  { 1, 2, 6, 13, },
  { 0, 1, 2, 6, 13, },
  { 3, 6, 13, },
  { 0, 3, 6, 13, },
  { 1, 3, 6, 13, },
  { 0, 1, 3, 6, 13, },
  { 2, 3, 6, 13, },
  { 0, 2, 3, 6, 13, },
  { 1, 2, 3, 6, 13, },
  { 0, 1, 2, 3, 6, 13, },
  { 4, 6, 13, },
  { 0, 4, 6, 13, },
  { 1, 4, 6, 13, },
  { 0, 1, 4, 6, 13, },
  { 2, 4, 6, 13, },
  { 0, 2, 4, 6, 13, },
  { 1, 2, 4, 6, 13, },
  { 0, 1, 2, 4, 6, 13, },
  { 3, 4, 6, 13, },
  { 0, 3, 4, 6, 13, },
  { 1, 3, 4, 6, 13, },
  { 0, 1, 3, 4, 6, 13, },
  { 2, 3, 4, 6, 13, },
  { 0, 2, 3, 4, 6, 13, },
  { 1, 2, 3, 4, 6, 13, },
  { 0, 1, 2, 3, 4, 6, 13, },
  { 5, 6, 13, },
  { 0, 5, 6, 13, },
  { 1, 5, 6, 13, },
  { 0, 1, 5, 6, 13, },
  { 2, 5, 6, 13, },
  { 0, 2, 5, 6, 13, },
  { 1, 2, 5, 6, 13, },
  { 0, 1, 2, 5, 6, 13, },
  { 3, 5, 6, 13, },
  { 0, 3, 5, 6, 13, },
  { 1, 3, 5, 6, 13, },
  { 0, 1, 3, 5, 6, 13, },
  { 2, 3, 5, 6, 13, },
  { 0, 2, 3, 5, 6, 13, },
  { 1, 2, 3, 5, 6, 13, },
  { 0, 1, 2, 3, 5, 6, 13, },
  { 4, 5, 6, 13, },
  { 0, 4, 5, 6, 13, },
  { 1, 4, 5, 6, 13, },
  { 0, 1, 4, 5, 6, 13, },
  { 2, 4, 5, 6, 13, },
  { 0, 2, 4, 5, 6, 13, },
  { 1, 2, 4, 5, 6, 13, },
  { 0, 1, 2, 4, 5, 6, 13, },
  { 3, 4, 5, 6, 13, },
  { 0, 3, 4, 5, 6, 13, },
  { 1, 3, 4, 5, 6, 13, },
  { 0, 1, 3, 4, 5, 6, 13, },
  { 2, 3, 4, 5, 6, 13, },
  { 0, 2, 3, 4, 5, 6, 13, },
  { 1, 2, 3, 4, 5, 6, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 13, },
  { 7, 13, },
  { 0, 7, 13, },
  { 1, 7, 13, },
  { 0, 1, 7, 13, },
  { 2, 7, 13, },
  { 0, 2, 7, 13, },
  { 1, 2, 7, 13, },
  { 0, 1, 2, 7, 13, },
  { 3, 7, 13, },
  { 0, 3, 7, 13, },
  { 1, 3, 7, 13, },
  { 0, 1, 3, 7, 13, },
  { 2, 3, 7, 13, },
  { 0, 2, 3, 7, 13, },
  { 1, 2, 3, 7, 13, },
  { 0, 1, 2, 3, 7, 13, },
  { 4, 7, 13, },
  { 0, 4, 7, 13, },
  { 1, 4, 7, 13, },
  { 0, 1, 4, 7, 13, },
  { 2, 4, 7, 13, },
  { 0, 2, 4, 7, 13, },
  { 1, 2, 4, 7, 13, },
  { 0, 1, 2, 4, 7, 13, },
  { 3, 4, 7, 13, },
  { 0, 3, 4, 7, 13, },
  { 1, 3, 4, 7, 13, },
  { 0, 1, 3, 4, 7, 13, },
  { 2, 3, 4, 7, 13, },
  { 0, 2, 3, 4, 7, 13, },
  { 1, 2, 3, 4, 7, 13, },
  { 0, 1, 2, 3, 4, 7, 13, },
  { 5, 7, 13, },
  { 0, 5, 7, 13, },
  { 1, 5, 7, 13, },
  { 0, 1, 5, 7, 13, },
  { 2, 5, 7, 13, },
  { 0, 2, 5, 7, 13, },
  { 1, 2, 5, 7, 13, },
  { 0, 1, 2, 5, 7, 13, },
  { 3, 5, 7, 13, },
  { 0, 3, 5, 7, 13, },
  { 1, 3, 5, 7, 13, },
  { 0, 1, 3, 5, 7, 13, },
  { 2, 3, 5, 7, 13, },
  { 0, 2, 3, 5, 7, 13, },
  { 1, 2, 3, 5, 7, 13, },
  { 0, 1, 2, 3, 5, 7, 13, },
  { 4, 5, 7, 13, },
  { 0, 4, 5, 7, 13, },
  { 1, 4, 5, 7, 13, },
  { 0, 1, 4, 5, 7, 13, },
  { 2, 4, 5, 7, 13, },
  { 0, 2, 4, 5, 7, 13, },
  { 1, 2, 4, 5, 7, 13, },
  { 0, 1, 2, 4, 5, 7, 13, },
  { 3, 4, 5, 7, 13, },
  { 0, 3, 4, 5, 7, 13, },
  { 1, 3, 4, 5, 7, 13, },
  { 0, 1, 3, 4, 5, 7, 13, },
  { 2, 3, 4, 5, 7, 13, },
  { 0, 2, 3, 4, 5, 7, 13, },
  { 1, 2, 3, 4, 5, 7, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 13, },
  { 6, 7, 13, },
  { 0, 6, 7, 13, },
  { 1, 6, 7, 13, },
  { 0, 1, 6, 7, 13, },
  { 2, 6, 7, 13, },
  { 0, 2, 6, 7, 13, },
  { 1, 2, 6, 7, 13, },
  { 0, 1, 2, 6, 7, 13, },
  { 3, 6, 7, 13, },
  { 0, 3, 6, 7, 13, },
  { 1, 3, 6, 7, 13, },
  { 0, 1, 3, 6, 7, 13, },
  { 2, 3, 6, 7, 13, },
  { 0, 2, 3, 6, 7, 13, },
  { 1, 2, 3, 6, 7, 13, },
  { 0, 1, 2, 3, 6, 7, 13, },
  { 4, 6, 7, 13, },
  { 0, 4, 6, 7, 13, },
  { 1, 4, 6, 7, 13, },
  { 0, 1, 4, 6, 7, 13, },
  { 2, 4, 6, 7, 13, },
  { 0, 2, 4, 6, 7, 13, },
  { 1, 2, 4, 6, 7, 13, },
  { 0, 1, 2, 4, 6, 7, 13, },
  { 3, 4, 6, 7, 13, },
  { 0, 3, 4, 6, 7, 13, },
  { 1, 3, 4, 6, 7, 13, },
  { 0, 1, 3, 4, 6, 7, 13, },
  { 2, 3, 4, 6, 7, 13, },
  { 0, 2, 3, 4, 6, 7, 13, },
  { 1, 2, 3, 4, 6, 7, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 13, },
  { 5, 6, 7, 13, },
  { 0, 5, 6, 7, 13, },
  { 1, 5, 6, 7, 13, },
  { 0, 1, 5, 6, 7, 13, },
  { 2, 5, 6, 7, 13, },
  { 0, 2, 5, 6, 7, 13, },
  { 1, 2, 5, 6, 7, 13, },
  { 0, 1, 2, 5, 6, 7, 13, },
  { 3, 5, 6, 7, 13, },
  { 0, 3, 5, 6, 7, 13, },
  { 1, 3, 5, 6, 7, 13, },
  { 0, 1, 3, 5, 6, 7, 13, },
  { 2, 3, 5, 6, 7, 13, },
  { 0, 2, 3, 5, 6, 7, 13, },
  { 1, 2, 3, 5, 6, 7, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 13, },
  { 4, 5, 6, 7, 13, },
  { 0, 4, 5, 6, 7, 13, },
  { 1, 4, 5, 6, 7, 13, },
  { 0, 1, 4, 5, 6, 7, 13, },
  { 2, 4, 5, 6, 7, 13, },
  { 0, 2, 4, 5, 6, 7, 13, },
  { 1, 2, 4, 5, 6, 7, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 13, },
  { 3, 4, 5, 6, 7, 13, },
  { 0, 3, 4, 5, 6, 7, 13, },
  { 1, 3, 4, 5, 6, 7, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 13, },
  { 2, 3, 4, 5, 6, 7, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 13, },
  { 8, 13, },
  { 0, 8, 13, },
  { 1, 8, 13, },
  { 0, 1, 8, 13, },
  { 2, 8, 13, },
  { 0, 2, 8, 13, },
  { 1, 2, 8, 13, },
  { 0, 1, 2, 8, 13, },
  { 3, 8, 13, },
  { 0, 3, 8, 13, },
  { 1, 3, 8, 13, },
  { 0, 1, 3, 8, 13, },
  { 2, 3, 8, 13, },
  { 0, 2, 3, 8, 13, },
  { 1, 2, 3, 8, 13, },
  { 0, 1, 2, 3, 8, 13, },
  { 4, 8, 13, },
  { 0, 4, 8, 13, },
  { 1, 4, 8, 13, },
  { 0, 1, 4, 8, 13, },
  { 2, 4, 8, 13, },
  { 0, 2, 4, 8, 13, },
  { 1, 2, 4, 8, 13, },
  { 0, 1, 2, 4, 8, 13, },
  { 3, 4, 8, 13, },
  { 0, 3, 4, 8, 13, },
  { 1, 3, 4, 8, 13, },
  { 0, 1, 3, 4, 8, 13, },
  { 2, 3, 4, 8, 13, },
  { 0, 2, 3, 4, 8, 13, },
  { 1, 2, 3, 4, 8, 13, },
  { 0, 1, 2, 3, 4, 8, 13, },
  { 5, 8, 13, },
  { 0, 5, 8, 13, },
  { 1, 5, 8, 13, },
  { 0, 1, 5, 8, 13, },
  { 2, 5, 8, 13, },
  { 0, 2, 5, 8, 13, },
  { 1, 2, 5, 8, 13, },
  { 0, 1, 2, 5, 8, 13, },
  { 3, 5, 8, 13, },
  { 0, 3, 5, 8, 13, },
  { 1, 3, 5, 8, 13, },
  { 0, 1, 3, 5, 8, 13, },
  { 2, 3, 5, 8, 13, },
  { 0, 2, 3, 5, 8, 13, },
  { 1, 2, 3, 5, 8, 13, },
  { 0, 1, 2, 3, 5, 8, 13, },
  { 4, 5, 8, 13, },
  { 0, 4, 5, 8, 13, },
  { 1, 4, 5, 8, 13, },
  { 0, 1, 4, 5, 8, 13, },
  { 2, 4, 5, 8, 13, },
  { 0, 2, 4, 5, 8, 13, },
  { 1, 2, 4, 5, 8, 13, },
  { 0, 1, 2, 4, 5, 8, 13, },
  { 3, 4, 5, 8, 13, },
  { 0, 3, 4, 5, 8, 13, },
  { 1, 3, 4, 5, 8, 13, },
  { 0, 1, 3, 4, 5, 8, 13, },
  { 2, 3, 4, 5, 8, 13, },
  { 0, 2, 3, 4, 5, 8, 13, },
  { 1, 2, 3, 4, 5, 8, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 13, },
  { 6, 8, 13, },
  { 0, 6, 8, 13, },
  { 1, 6, 8, 13, },
  { 0, 1, 6, 8, 13, },
  { 2, 6, 8, 13, },
  { 0, 2, 6, 8, 13, },
  { 1, 2, 6, 8, 13, },
  { 0, 1, 2, 6, 8, 13, },
  { 3, 6, 8, 13, },
  { 0, 3, 6, 8, 13, },
  { 1, 3, 6, 8, 13, },
  { 0, 1, 3, 6, 8, 13, },
  { 2, 3, 6, 8, 13, },
  { 0, 2, 3, 6, 8, 13, },
  { 1, 2, 3, 6, 8, 13, },
  { 0, 1, 2, 3, 6, 8, 13, },
  { 4, 6, 8, 13, },
  { 0, 4, 6, 8, 13, },
  { 1, 4, 6, 8, 13, },
  { 0, 1, 4, 6, 8, 13, },
  { 2, 4, 6, 8, 13, },
  { 0, 2, 4, 6, 8, 13, },
  { 1, 2, 4, 6, 8, 13, },
  { 0, 1, 2, 4, 6, 8, 13, },
  { 3, 4, 6, 8, 13, },
  { 0, 3, 4, 6, 8, 13, },
  { 1, 3, 4, 6, 8, 13, },
  { 0, 1, 3, 4, 6, 8, 13, },
  { 2, 3, 4, 6, 8, 13, },
  { 0, 2, 3, 4, 6, 8, 13, },
  { 1, 2, 3, 4, 6, 8, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 13, },
  { 5, 6, 8, 13, },
  { 0, 5, 6, 8, 13, },
  { 1, 5, 6, 8, 13, },
  { 0, 1, 5, 6, 8, 13, },
  { 2, 5, 6, 8, 13, },
  { 0, 2, 5, 6, 8, 13, },
  { 1, 2, 5, 6, 8, 13, },
  { 0, 1, 2, 5, 6, 8, 13, },
  { 3, 5, 6, 8, 13, },
  { 0, 3, 5, 6, 8, 13, },
  { 1, 3, 5, 6, 8, 13, },
  { 0, 1, 3, 5, 6, 8, 13, },
  { 2, 3, 5, 6, 8, 13, },
  { 0, 2, 3, 5, 6, 8, 13, },
  { 1, 2, 3, 5, 6, 8, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 13, },
  { 4, 5, 6, 8, 13, },
  { 0, 4, 5, 6, 8, 13, },
  { 1, 4, 5, 6, 8, 13, },
  { 0, 1, 4, 5, 6, 8, 13, },
  { 2, 4, 5, 6, 8, 13, },
  { 0, 2, 4, 5, 6, 8, 13, },
  { 1, 2, 4, 5, 6, 8, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 13, },
  { 3, 4, 5, 6, 8, 13, },
  { 0, 3, 4, 5, 6, 8, 13, },
  { 1, 3, 4, 5, 6, 8, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 13, },
  { 2, 3, 4, 5, 6, 8, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 13, },
  { 7, 8, 13, },
  { 0, 7, 8, 13, },
  { 1, 7, 8, 13, },
  { 0, 1, 7, 8, 13, },
  { 2, 7, 8, 13, },
  { 0, 2, 7, 8, 13, },
  { 1, 2, 7, 8, 13, },
  { 0, 1, 2, 7, 8, 13, },
  { 3, 7, 8, 13, },
  { 0, 3, 7, 8, 13, },
  { 1, 3, 7, 8, 13, },
  { 0, 1, 3, 7, 8, 13, },
  { 2, 3, 7, 8, 13, },
  { 0, 2, 3, 7, 8, 13, },
  { 1, 2, 3, 7, 8, 13, },
  { 0, 1, 2, 3, 7, 8, 13, },
  { 4, 7, 8, 13, },
  { 0, 4, 7, 8, 13, },
  { 1, 4, 7, 8, 13, },
  { 0, 1, 4, 7, 8, 13, },
  { 2, 4, 7, 8, 13, },
  { 0, 2, 4, 7, 8, 13, },
  { 1, 2, 4, 7, 8, 13, },
  { 0, 1, 2, 4, 7, 8, 13, },
  { 3, 4, 7, 8, 13, },
  { 0, 3, 4, 7, 8, 13, },
  { 1, 3, 4, 7, 8, 13, },
  { 0, 1, 3, 4, 7, 8, 13, },
  { 2, 3, 4, 7, 8, 13, },
  { 0, 2, 3, 4, 7, 8, 13, },
  { 1, 2, 3, 4, 7, 8, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 13, },
  { 5, 7, 8, 13, },
  { 0, 5, 7, 8, 13, },
  { 1, 5, 7, 8, 13, },
  { 0, 1, 5, 7, 8, 13, },
  { 2, 5, 7, 8, 13, },
  { 0, 2, 5, 7, 8, 13, },
  { 1, 2, 5, 7, 8, 13, },
  { 0, 1, 2, 5, 7, 8, 13, },
  { 3, 5, 7, 8, 13, },
  { 0, 3, 5, 7, 8, 13, },
  { 1, 3, 5, 7, 8, 13, },
  { 0, 1, 3, 5, 7, 8, 13, },
  { 2, 3, 5, 7, 8, 13, },
  { 0, 2, 3, 5, 7, 8, 13, },
  { 1, 2, 3, 5, 7, 8, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 13, },
  { 4, 5, 7, 8, 13, },
  { 0, 4, 5, 7, 8, 13, },
  { 1, 4, 5, 7, 8, 13, },
  { 0, 1, 4, 5, 7, 8, 13, },
  { 2, 4, 5, 7, 8, 13, },
  { 0, 2, 4, 5, 7, 8, 13, },
  { 1, 2, 4, 5, 7, 8, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 13, },
  { 3, 4, 5, 7, 8, 13, },
  { 0, 3, 4, 5, 7, 8, 13, },
  { 1, 3, 4, 5, 7, 8, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 13, },
  { 2, 3, 4, 5, 7, 8, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 13, },
  { 6, 7, 8, 13, },
  { 0, 6, 7, 8, 13, },
  { 1, 6, 7, 8, 13, },
  { 0, 1, 6, 7, 8, 13, },
  { 2, 6, 7, 8, 13, },
  { 0, 2, 6, 7, 8, 13, },
  { 1, 2, 6, 7, 8, 13, },
  { 0, 1, 2, 6, 7, 8, 13, },
  { 3, 6, 7, 8, 13, },
  { 0, 3, 6, 7, 8, 13, },
  { 1, 3, 6, 7, 8, 13, },
  { 0, 1, 3, 6, 7, 8, 13, },
  { 2, 3, 6, 7, 8, 13, },
  { 0, 2, 3, 6, 7, 8, 13, },
  { 1, 2, 3, 6, 7, 8, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 13, },
  { 4, 6, 7, 8, 13, },
  { 0, 4, 6, 7, 8, 13, },
  { 1, 4, 6, 7, 8, 13, },
  { 0, 1, 4, 6, 7, 8, 13, },
  { 2, 4, 6, 7, 8, 13, },
  { 0, 2, 4, 6, 7, 8, 13, },
  { 1, 2, 4, 6, 7, 8, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 13, },
  { 3, 4, 6, 7, 8, 13, },
  { 0, 3, 4, 6, 7, 8, 13, },
  { 1, 3, 4, 6, 7, 8, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 13, },
  { 2, 3, 4, 6, 7, 8, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 13, },
  { 5, 6, 7, 8, 13, },
  { 0, 5, 6, 7, 8, 13, },
  { 1, 5, 6, 7, 8, 13, },
  { 0, 1, 5, 6, 7, 8, 13, },
  { 2, 5, 6, 7, 8, 13, },
  { 0, 2, 5, 6, 7, 8, 13, },
  { 1, 2, 5, 6, 7, 8, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 13, },
  { 3, 5, 6, 7, 8, 13, },
  { 0, 3, 5, 6, 7, 8, 13, },
  { 1, 3, 5, 6, 7, 8, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 13, },
  { 2, 3, 5, 6, 7, 8, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 13, },
  { 4, 5, 6, 7, 8, 13, },
  { 0, 4, 5, 6, 7, 8, 13, },
  { 1, 4, 5, 6, 7, 8, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 13, },
  { 2, 4, 5, 6, 7, 8, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 13, },
  { 3, 4, 5, 6, 7, 8, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 13, },
  { 9, 13, },
  { 0, 9, 13, },
  { 1, 9, 13, },
  { 0, 1, 9, 13, },
  { 2, 9, 13, },
  { 0, 2, 9, 13, },
  { 1, 2, 9, 13, },
  { 0, 1, 2, 9, 13, },
  { 3, 9, 13, },
  { 0, 3, 9, 13, },
  { 1, 3, 9, 13, },
  { 0, 1, 3, 9, 13, },
  { 2, 3, 9, 13, },
  { 0, 2, 3, 9, 13, },
  { 1, 2, 3, 9, 13, },
  { 0, 1, 2, 3, 9, 13, },
  { 4, 9, 13, },
  { 0, 4, 9, 13, },
  { 1, 4, 9, 13, },
  { 0, 1, 4, 9, 13, },
  { 2, 4, 9, 13, },
  { 0, 2, 4, 9, 13, },
  { 1, 2, 4, 9, 13, },
  { 0, 1, 2, 4, 9, 13, },
  { 3, 4, 9, 13, },
  { 0, 3, 4, 9, 13, },
  { 1, 3, 4, 9, 13, },
  { 0, 1, 3, 4, 9, 13, },
  { 2, 3, 4, 9, 13, },
  { 0, 2, 3, 4, 9, 13, },
  { 1, 2, 3, 4, 9, 13, },
  { 0, 1, 2, 3, 4, 9, 13, },
  { 5, 9, 13, },
  { 0, 5, 9, 13, },
  { 1, 5, 9, 13, },
  { 0, 1, 5, 9, 13, },
  { 2, 5, 9, 13, },
  { 0, 2, 5, 9, 13, },
  { 1, 2, 5, 9, 13, },
  { 0, 1, 2, 5, 9, 13, },
  { 3, 5, 9, 13, },
  { 0, 3, 5, 9, 13, },
  { 1, 3, 5, 9, 13, },
  { 0, 1, 3, 5, 9, 13, },
  { 2, 3, 5, 9, 13, },
  { 0, 2, 3, 5, 9, 13, },
  { 1, 2, 3, 5, 9, 13, },
  { 0, 1, 2, 3, 5, 9, 13, },
  { 4, 5, 9, 13, },
  { 0, 4, 5, 9, 13, },
  { 1, 4, 5, 9, 13, },
  { 0, 1, 4, 5, 9, 13, },
  { 2, 4, 5, 9, 13, },
  { 0, 2, 4, 5, 9, 13, },
  { 1, 2, 4, 5, 9, 13, },
  { 0, 1, 2, 4, 5, 9, 13, },
  { 3, 4, 5, 9, 13, },
  { 0, 3, 4, 5, 9, 13, },
  { 1, 3, 4, 5, 9, 13, },
  { 0, 1, 3, 4, 5, 9, 13, },
  { 2, 3, 4, 5, 9, 13, },
  { 0, 2, 3, 4, 5, 9, 13, },
  { 1, 2, 3, 4, 5, 9, 13, },
  { 0, 1, 2, 3, 4, 5, 9, 13, },
  { 6, 9, 13, },
  { 0, 6, 9, 13, },
  { 1, 6, 9, 13, },
  { 0, 1, 6, 9, 13, },
  { 2, 6, 9, 13, },
  { 0, 2, 6, 9, 13, },
  { 1, 2, 6, 9, 13, },
  { 0, 1, 2, 6, 9, 13, },
  { 3, 6, 9, 13, },
  { 0, 3, 6, 9, 13, },
  { 1, 3, 6, 9, 13, },
  { 0, 1, 3, 6, 9, 13, },
  { 2, 3, 6, 9, 13, },
  { 0, 2, 3, 6, 9, 13, },
  { 1, 2, 3, 6, 9, 13, },
  { 0, 1, 2, 3, 6, 9, 13, },
  { 4, 6, 9, 13, },
  { 0, 4, 6, 9, 13, },
  { 1, 4, 6, 9, 13, },
  { 0, 1, 4, 6, 9, 13, },
  { 2, 4, 6, 9, 13, },
  { 0, 2, 4, 6, 9, 13, },
  { 1, 2, 4, 6, 9, 13, },
  { 0, 1, 2, 4, 6, 9, 13, },
  { 3, 4, 6, 9, 13, },
  { 0, 3, 4, 6, 9, 13, },
  { 1, 3, 4, 6, 9, 13, },
  { 0, 1, 3, 4, 6, 9, 13, },
  { 2, 3, 4, 6, 9, 13, },
  { 0, 2, 3, 4, 6, 9, 13, },
  { 1, 2, 3, 4, 6, 9, 13, },
  { 0, 1, 2, 3, 4, 6, 9, 13, },
  { 5, 6, 9, 13, },
  { 0, 5, 6, 9, 13, },
  { 1, 5, 6, 9, 13, },
  { 0, 1, 5, 6, 9, 13, },
  { 2, 5, 6, 9, 13, },
  { 0, 2, 5, 6, 9, 13, },
  { 1, 2, 5, 6, 9, 13, },
  { 0, 1, 2, 5, 6, 9, 13, },
  { 3, 5, 6, 9, 13, },
  { 0, 3, 5, 6, 9, 13, },
  { 1, 3, 5, 6, 9, 13, },
  { 0, 1, 3, 5, 6, 9, 13, },
  { 2, 3, 5, 6, 9, 13, },
  { 0, 2, 3, 5, 6, 9, 13, },
  { 1, 2, 3, 5, 6, 9, 13, },
  { 0, 1, 2, 3, 5, 6, 9, 13, },
  { 4, 5, 6, 9, 13, },
  { 0, 4, 5, 6, 9, 13, },
  { 1, 4, 5, 6, 9, 13, },
  { 0, 1, 4, 5, 6, 9, 13, },
  { 2, 4, 5, 6, 9, 13, },
  { 0, 2, 4, 5, 6, 9, 13, },
  { 1, 2, 4, 5, 6, 9, 13, },
  { 0, 1, 2, 4, 5, 6, 9, 13, },
  { 3, 4, 5, 6, 9, 13, },
  { 0, 3, 4, 5, 6, 9, 13, },
  { 1, 3, 4, 5, 6, 9, 13, },
  { 0, 1, 3, 4, 5, 6, 9, 13, },
  { 2, 3, 4, 5, 6, 9, 13, },
  { 0, 2, 3, 4, 5, 6, 9, 13, },
  { 1, 2, 3, 4, 5, 6, 9, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 13, },
  { 7, 9, 13, },
  { 0, 7, 9, 13, },
  { 1, 7, 9, 13, },
  { 0, 1, 7, 9, 13, },
  { 2, 7, 9, 13, },
  { 0, 2, 7, 9, 13, },
  { 1, 2, 7, 9, 13, },
  { 0, 1, 2, 7, 9, 13, },
  { 3, 7, 9, 13, },
  { 0, 3, 7, 9, 13, },
  { 1, 3, 7, 9, 13, },
  { 0, 1, 3, 7, 9, 13, },
  { 2, 3, 7, 9, 13, },
  { 0, 2, 3, 7, 9, 13, },
  { 1, 2, 3, 7, 9, 13, },
  { 0, 1, 2, 3, 7, 9, 13, },
  { 4, 7, 9, 13, },
  { 0, 4, 7, 9, 13, },
  { 1, 4, 7, 9, 13, },
  { 0, 1, 4, 7, 9, 13, },
  { 2, 4, 7, 9, 13, },
  { 0, 2, 4, 7, 9, 13, },
  { 1, 2, 4, 7, 9, 13, },
  { 0, 1, 2, 4, 7, 9, 13, },
  { 3, 4, 7, 9, 13, },
  { 0, 3, 4, 7, 9, 13, },
  { 1, 3, 4, 7, 9, 13, },
  { 0, 1, 3, 4, 7, 9, 13, },
  { 2, 3, 4, 7, 9, 13, },
  { 0, 2, 3, 4, 7, 9, 13, },
  { 1, 2, 3, 4, 7, 9, 13, },
  { 0, 1, 2, 3, 4, 7, 9, 13, },
  { 5, 7, 9, 13, },
  { 0, 5, 7, 9, 13, },
  { 1, 5, 7, 9, 13, },
  { 0, 1, 5, 7, 9, 13, },
  { 2, 5, 7, 9, 13, },
  { 0, 2, 5, 7, 9, 13, },
  { 1, 2, 5, 7, 9, 13, },
  { 0, 1, 2, 5, 7, 9, 13, },
  { 3, 5, 7, 9, 13, },
  { 0, 3, 5, 7, 9, 13, },
  { 1, 3, 5, 7, 9, 13, },
  { 0, 1, 3, 5, 7, 9, 13, },
  { 2, 3, 5, 7, 9, 13, },
  { 0, 2, 3, 5, 7, 9, 13, },
  { 1, 2, 3, 5, 7, 9, 13, },
  { 0, 1, 2, 3, 5, 7, 9, 13, },
  { 4, 5, 7, 9, 13, },
  { 0, 4, 5, 7, 9, 13, },
  { 1, 4, 5, 7, 9, 13, },
  { 0, 1, 4, 5, 7, 9, 13, },
  { 2, 4, 5, 7, 9, 13, },
  { 0, 2, 4, 5, 7, 9, 13, },
  { 1, 2, 4, 5, 7, 9, 13, },
  { 0, 1, 2, 4, 5, 7, 9, 13, },
  { 3, 4, 5, 7, 9, 13, },
  { 0, 3, 4, 5, 7, 9, 13, },
  { 1, 3, 4, 5, 7, 9, 13, },
  { 0, 1, 3, 4, 5, 7, 9, 13, },
  { 2, 3, 4, 5, 7, 9, 13, },
  { 0, 2, 3, 4, 5, 7, 9, 13, },
  { 1, 2, 3, 4, 5, 7, 9, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 13, },
  { 6, 7, 9, 13, },
  { 0, 6, 7, 9, 13, },
  { 1, 6, 7, 9, 13, },
  { 0, 1, 6, 7, 9, 13, },
  { 2, 6, 7, 9, 13, },
  { 0, 2, 6, 7, 9, 13, },
  { 1, 2, 6, 7, 9, 13, },
  { 0, 1, 2, 6, 7, 9, 13, },
  { 3, 6, 7, 9, 13, },
  { 0, 3, 6, 7, 9, 13, },
  { 1, 3, 6, 7, 9, 13, },
  { 0, 1, 3, 6, 7, 9, 13, },
  { 2, 3, 6, 7, 9, 13, },
  { 0, 2, 3, 6, 7, 9, 13, },
  { 1, 2, 3, 6, 7, 9, 13, },
  { 0, 1, 2, 3, 6, 7, 9, 13, },
  { 4, 6, 7, 9, 13, },
  { 0, 4, 6, 7, 9, 13, },
  { 1, 4, 6, 7, 9, 13, },
  { 0, 1, 4, 6, 7, 9, 13, },
  { 2, 4, 6, 7, 9, 13, },
  { 0, 2, 4, 6, 7, 9, 13, },
  { 1, 2, 4, 6, 7, 9, 13, },
  { 0, 1, 2, 4, 6, 7, 9, 13, },
  { 3, 4, 6, 7, 9, 13, },
  { 0, 3, 4, 6, 7, 9, 13, },
  { 1, 3, 4, 6, 7, 9, 13, },
  { 0, 1, 3, 4, 6, 7, 9, 13, },
  { 2, 3, 4, 6, 7, 9, 13, },
  { 0, 2, 3, 4, 6, 7, 9, 13, },
  { 1, 2, 3, 4, 6, 7, 9, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 13, },
  { 5, 6, 7, 9, 13, },
  { 0, 5, 6, 7, 9, 13, },
  { 1, 5, 6, 7, 9, 13, },
  { 0, 1, 5, 6, 7, 9, 13, },
  { 2, 5, 6, 7, 9, 13, },
  { 0, 2, 5, 6, 7, 9, 13, },
  { 1, 2, 5, 6, 7, 9, 13, },
  { 0, 1, 2, 5, 6, 7, 9, 13, },
  { 3, 5, 6, 7, 9, 13, },
  { 0, 3, 5, 6, 7, 9, 13, },
  { 1, 3, 5, 6, 7, 9, 13, },
  { 0, 1, 3, 5, 6, 7, 9, 13, },
  { 2, 3, 5, 6, 7, 9, 13, },
  { 0, 2, 3, 5, 6, 7, 9, 13, },
  { 1, 2, 3, 5, 6, 7, 9, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 13, },
  { 4, 5, 6, 7, 9, 13, },
  { 0, 4, 5, 6, 7, 9, 13, },
  { 1, 4, 5, 6, 7, 9, 13, },
  { 0, 1, 4, 5, 6, 7, 9, 13, },
  { 2, 4, 5, 6, 7, 9, 13, },
  { 0, 2, 4, 5, 6, 7, 9, 13, },
  { 1, 2, 4, 5, 6, 7, 9, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 13, },
  { 3, 4, 5, 6, 7, 9, 13, },
  { 0, 3, 4, 5, 6, 7, 9, 13, },
  { 1, 3, 4, 5, 6, 7, 9, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 13, },
  { 2, 3, 4, 5, 6, 7, 9, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 13, },
  { 8, 9, 13, },
  { 0, 8, 9, 13, },
  { 1, 8, 9, 13, },
  { 0, 1, 8, 9, 13, },
  { 2, 8, 9, 13, },
  { 0, 2, 8, 9, 13, },
  { 1, 2, 8, 9, 13, },
  { 0, 1, 2, 8, 9, 13, },
  { 3, 8, 9, 13, },
  { 0, 3, 8, 9, 13, },
  { 1, 3, 8, 9, 13, },
  { 0, 1, 3, 8, 9, 13, },
  { 2, 3, 8, 9, 13, },
  { 0, 2, 3, 8, 9, 13, },
  { 1, 2, 3, 8, 9, 13, },
  { 0, 1, 2, 3, 8, 9, 13, },
  { 4, 8, 9, 13, },
  { 0, 4, 8, 9, 13, },
  { 1, 4, 8, 9, 13, },
  { 0, 1, 4, 8, 9, 13, },
  { 2, 4, 8, 9, 13, },
  { 0, 2, 4, 8, 9, 13, },
  { 1, 2, 4, 8, 9, 13, },
  { 0, 1, 2, 4, 8, 9, 13, },
  { 3, 4, 8, 9, 13, },
  { 0, 3, 4, 8, 9, 13, },
  { 1, 3, 4, 8, 9, 13, },
  { 0, 1, 3, 4, 8, 9, 13, },
  { 2, 3, 4, 8, 9, 13, },
  { 0, 2, 3, 4, 8, 9, 13, },
  { 1, 2, 3, 4, 8, 9, 13, },
  { 0, 1, 2, 3, 4, 8, 9, 13, },
  { 5, 8, 9, 13, },
  { 0, 5, 8, 9, 13, },
  { 1, 5, 8, 9, 13, },
  { 0, 1, 5, 8, 9, 13, },
  { 2, 5, 8, 9, 13, },
  { 0, 2, 5, 8, 9, 13, },
  { 1, 2, 5, 8, 9, 13, },
  { 0, 1, 2, 5, 8, 9, 13, },
  { 3, 5, 8, 9, 13, },
  { 0, 3, 5, 8, 9, 13, },
  { 1, 3, 5, 8, 9, 13, },
  { 0, 1, 3, 5, 8, 9, 13, },
  { 2, 3, 5, 8, 9, 13, },
  { 0, 2, 3, 5, 8, 9, 13, },
  { 1, 2, 3, 5, 8, 9, 13, },
  { 0, 1, 2, 3, 5, 8, 9, 13, },
  { 4, 5, 8, 9, 13, },
  { 0, 4, 5, 8, 9, 13, },
  { 1, 4, 5, 8, 9, 13, },
  { 0, 1, 4, 5, 8, 9, 13, },
  { 2, 4, 5, 8, 9, 13, },
  { 0, 2, 4, 5, 8, 9, 13, },
  { 1, 2, 4, 5, 8, 9, 13, },
  { 0, 1, 2, 4, 5, 8, 9, 13, },
  { 3, 4, 5, 8, 9, 13, },
  { 0, 3, 4, 5, 8, 9, 13, },
  { 1, 3, 4, 5, 8, 9, 13, },
  { 0, 1, 3, 4, 5, 8, 9, 13, },
  { 2, 3, 4, 5, 8, 9, 13, },
  { 0, 2, 3, 4, 5, 8, 9, 13, },
  { 1, 2, 3, 4, 5, 8, 9, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 13, },
  { 6, 8, 9, 13, },
  { 0, 6, 8, 9, 13, },
  { 1, 6, 8, 9, 13, },
  { 0, 1, 6, 8, 9, 13, },
  { 2, 6, 8, 9, 13, },
  { 0, 2, 6, 8, 9, 13, },
  { 1, 2, 6, 8, 9, 13, },
  { 0, 1, 2, 6, 8, 9, 13, },
  { 3, 6, 8, 9, 13, },
  { 0, 3, 6, 8, 9, 13, },
  { 1, 3, 6, 8, 9, 13, },
  { 0, 1, 3, 6, 8, 9, 13, },
  { 2, 3, 6, 8, 9, 13, },
  { 0, 2, 3, 6, 8, 9, 13, },
  { 1, 2, 3, 6, 8, 9, 13, },
  { 0, 1, 2, 3, 6, 8, 9, 13, },
  { 4, 6, 8, 9, 13, },
  { 0, 4, 6, 8, 9, 13, },
  { 1, 4, 6, 8, 9, 13, },
  { 0, 1, 4, 6, 8, 9, 13, },
  { 2, 4, 6, 8, 9, 13, },
  { 0, 2, 4, 6, 8, 9, 13, },
  { 1, 2, 4, 6, 8, 9, 13, },
  { 0, 1, 2, 4, 6, 8, 9, 13, },
  { 3, 4, 6, 8, 9, 13, },
  { 0, 3, 4, 6, 8, 9, 13, },
  { 1, 3, 4, 6, 8, 9, 13, },
  { 0, 1, 3, 4, 6, 8, 9, 13, },
  { 2, 3, 4, 6, 8, 9, 13, },
  { 0, 2, 3, 4, 6, 8, 9, 13, },
  { 1, 2, 3, 4, 6, 8, 9, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 13, },
  { 5, 6, 8, 9, 13, },
  { 0, 5, 6, 8, 9, 13, },
  { 1, 5, 6, 8, 9, 13, },
  { 0, 1, 5, 6, 8, 9, 13, },
  { 2, 5, 6, 8, 9, 13, },
  { 0, 2, 5, 6, 8, 9, 13, },
  { 1, 2, 5, 6, 8, 9, 13, },
  { 0, 1, 2, 5, 6, 8, 9, 13, },
  { 3, 5, 6, 8, 9, 13, },
  { 0, 3, 5, 6, 8, 9, 13, },
  { 1, 3, 5, 6, 8, 9, 13, },
  { 0, 1, 3, 5, 6, 8, 9, 13, },
  { 2, 3, 5, 6, 8, 9, 13, },
  { 0, 2, 3, 5, 6, 8, 9, 13, },
  { 1, 2, 3, 5, 6, 8, 9, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 13, },
  { 4, 5, 6, 8, 9, 13, },
  { 0, 4, 5, 6, 8, 9, 13, },
  { 1, 4, 5, 6, 8, 9, 13, },
  { 0, 1, 4, 5, 6, 8, 9, 13, },
  { 2, 4, 5, 6, 8, 9, 13, },
  { 0, 2, 4, 5, 6, 8, 9, 13, },
  { 1, 2, 4, 5, 6, 8, 9, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 13, },
  { 3, 4, 5, 6, 8, 9, 13, },
  { 0, 3, 4, 5, 6, 8, 9, 13, },
  { 1, 3, 4, 5, 6, 8, 9, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 13, },
  { 2, 3, 4, 5, 6, 8, 9, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 13, },
  { 7, 8, 9, 13, },
  { 0, 7, 8, 9, 13, },
  { 1, 7, 8, 9, 13, },
  { 0, 1, 7, 8, 9, 13, },
  { 2, 7, 8, 9, 13, },
  { 0, 2, 7, 8, 9, 13, },
  { 1, 2, 7, 8, 9, 13, },
  { 0, 1, 2, 7, 8, 9, 13, },
  { 3, 7, 8, 9, 13, },
  { 0, 3, 7, 8, 9, 13, },
  { 1, 3, 7, 8, 9, 13, },
  { 0, 1, 3, 7, 8, 9, 13, },
  { 2, 3, 7, 8, 9, 13, },
  { 0, 2, 3, 7, 8, 9, 13, },
  { 1, 2, 3, 7, 8, 9, 13, },
  { 0, 1, 2, 3, 7, 8, 9, 13, },
  { 4, 7, 8, 9, 13, },
  { 0, 4, 7, 8, 9, 13, },
  { 1, 4, 7, 8, 9, 13, },
  { 0, 1, 4, 7, 8, 9, 13, },
  { 2, 4, 7, 8, 9, 13, },
  { 0, 2, 4, 7, 8, 9, 13, },
  { 1, 2, 4, 7, 8, 9, 13, },
  { 0, 1, 2, 4, 7, 8, 9, 13, },
  { 3, 4, 7, 8, 9, 13, },
  { 0, 3, 4, 7, 8, 9, 13, },
  { 1, 3, 4, 7, 8, 9, 13, },
  { 0, 1, 3, 4, 7, 8, 9, 13, },
  { 2, 3, 4, 7, 8, 9, 13, },
  { 0, 2, 3, 4, 7, 8, 9, 13, },
  { 1, 2, 3, 4, 7, 8, 9, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 13, },
  { 5, 7, 8, 9, 13, },
  { 0, 5, 7, 8, 9, 13, },
  { 1, 5, 7, 8, 9, 13, },
  { 0, 1, 5, 7, 8, 9, 13, },
  { 2, 5, 7, 8, 9, 13, },
  { 0, 2, 5, 7, 8, 9, 13, },
  { 1, 2, 5, 7, 8, 9, 13, },
  { 0, 1, 2, 5, 7, 8, 9, 13, },
  { 3, 5, 7, 8, 9, 13, },
  { 0, 3, 5, 7, 8, 9, 13, },
  { 1, 3, 5, 7, 8, 9, 13, },
  { 0, 1, 3, 5, 7, 8, 9, 13, },
  { 2, 3, 5, 7, 8, 9, 13, },
  { 0, 2, 3, 5, 7, 8, 9, 13, },
  { 1, 2, 3, 5, 7, 8, 9, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 13, },
  { 4, 5, 7, 8, 9, 13, },
  { 0, 4, 5, 7, 8, 9, 13, },
  { 1, 4, 5, 7, 8, 9, 13, },
  { 0, 1, 4, 5, 7, 8, 9, 13, },
  { 2, 4, 5, 7, 8, 9, 13, },
  { 0, 2, 4, 5, 7, 8, 9, 13, },
  { 1, 2, 4, 5, 7, 8, 9, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 13, },
  { 3, 4, 5, 7, 8, 9, 13, },
  { 0, 3, 4, 5, 7, 8, 9, 13, },
  { 1, 3, 4, 5, 7, 8, 9, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 13, },
  { 2, 3, 4, 5, 7, 8, 9, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 13, },
  { 6, 7, 8, 9, 13, },
  { 0, 6, 7, 8, 9, 13, },
  { 1, 6, 7, 8, 9, 13, },
  { 0, 1, 6, 7, 8, 9, 13, },
  { 2, 6, 7, 8, 9, 13, },
  { 0, 2, 6, 7, 8, 9, 13, },
  { 1, 2, 6, 7, 8, 9, 13, },
  { 0, 1, 2, 6, 7, 8, 9, 13, },
  { 3, 6, 7, 8, 9, 13, },
  { 0, 3, 6, 7, 8, 9, 13, },
  { 1, 3, 6, 7, 8, 9, 13, },
  { 0, 1, 3, 6, 7, 8, 9, 13, },
  { 2, 3, 6, 7, 8, 9, 13, },
  { 0, 2, 3, 6, 7, 8, 9, 13, },
  { 1, 2, 3, 6, 7, 8, 9, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 13, },
  { 4, 6, 7, 8, 9, 13, },
  { 0, 4, 6, 7, 8, 9, 13, },
  { 1, 4, 6, 7, 8, 9, 13, },
  { 0, 1, 4, 6, 7, 8, 9, 13, },
  { 2, 4, 6, 7, 8, 9, 13, },
  { 0, 2, 4, 6, 7, 8, 9, 13, },
  { 1, 2, 4, 6, 7, 8, 9, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 13, },
  { 3, 4, 6, 7, 8, 9, 13, },
  { 0, 3, 4, 6, 7, 8, 9, 13, },
  { 1, 3, 4, 6, 7, 8, 9, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 13, },
  { 2, 3, 4, 6, 7, 8, 9, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 13, },
  { 5, 6, 7, 8, 9, 13, },
  { 0, 5, 6, 7, 8, 9, 13, },
  { 1, 5, 6, 7, 8, 9, 13, },
  { 0, 1, 5, 6, 7, 8, 9, 13, },
  { 2, 5, 6, 7, 8, 9, 13, },
  { 0, 2, 5, 6, 7, 8, 9, 13, },
  { 1, 2, 5, 6, 7, 8, 9, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 13, },
  { 3, 5, 6, 7, 8, 9, 13, },
  { 0, 3, 5, 6, 7, 8, 9, 13, },
  { 1, 3, 5, 6, 7, 8, 9, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 13, },
  { 2, 3, 5, 6, 7, 8, 9, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 13, },
  { 4, 5, 6, 7, 8, 9, 13, },
  { 0, 4, 5, 6, 7, 8, 9, 13, },
  { 1, 4, 5, 6, 7, 8, 9, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 13, },
  { 2, 4, 5, 6, 7, 8, 9, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 13, },
  { 3, 4, 5, 6, 7, 8, 9, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 13, },
  { 10, 13, },
  { 0, 10, 13, },
  { 1, 10, 13, },
  { 0, 1, 10, 13, },
  { 2, 10, 13, },
  { 0, 2, 10, 13, },
  { 1, 2, 10, 13, },
  { 0, 1, 2, 10, 13, },
  { 3, 10, 13, },
  { 0, 3, 10, 13, },
  { 1, 3, 10, 13, },
  { 0, 1, 3, 10, 13, },
  { 2, 3, 10, 13, },
  { 0, 2, 3, 10, 13, },
  { 1, 2, 3, 10, 13, },
  { 0, 1, 2, 3, 10, 13, },
  { 4, 10, 13, },
  { 0, 4, 10, 13, },
  { 1, 4, 10, 13, },
  { 0, 1, 4, 10, 13, },
  { 2, 4, 10, 13, },
  { 0, 2, 4, 10, 13, },
  { 1, 2, 4, 10, 13, },
  { 0, 1, 2, 4, 10, 13, },
  { 3, 4, 10, 13, },
  { 0, 3, 4, 10, 13, },
  { 1, 3, 4, 10, 13, },
  { 0, 1, 3, 4, 10, 13, },
  { 2, 3, 4, 10, 13, },
  { 0, 2, 3, 4, 10, 13, },
  { 1, 2, 3, 4, 10, 13, },
  { 0, 1, 2, 3, 4, 10, 13, },
  { 5, 10, 13, },
  { 0, 5, 10, 13, },
  { 1, 5, 10, 13, },
  { 0, 1, 5, 10, 13, },
  { 2, 5, 10, 13, },
  { 0, 2, 5, 10, 13, },
  { 1, 2, 5, 10, 13, },
  { 0, 1, 2, 5, 10, 13, },
  { 3, 5, 10, 13, },
  { 0, 3, 5, 10, 13, },
  { 1, 3, 5, 10, 13, },
  { 0, 1, 3, 5, 10, 13, },
  { 2, 3, 5, 10, 13, },
  { 0, 2, 3, 5, 10, 13, },
  { 1, 2, 3, 5, 10, 13, },
  { 0, 1, 2, 3, 5, 10, 13, },
  { 4, 5, 10, 13, },
  { 0, 4, 5, 10, 13, },
  { 1, 4, 5, 10, 13, },
  { 0, 1, 4, 5, 10, 13, },
  { 2, 4, 5, 10, 13, },
  { 0, 2, 4, 5, 10, 13, },
  { 1, 2, 4, 5, 10, 13, },
  { 0, 1, 2, 4, 5, 10, 13, },
  { 3, 4, 5, 10, 13, },
  { 0, 3, 4, 5, 10, 13, },
  { 1, 3, 4, 5, 10, 13, },
  { 0, 1, 3, 4, 5, 10, 13, },
  { 2, 3, 4, 5, 10, 13, },
  { 0, 2, 3, 4, 5, 10, 13, },
  { 1, 2, 3, 4, 5, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 10, 13, },
  { 6, 10, 13, },
  { 0, 6, 10, 13, },
  { 1, 6, 10, 13, },
  { 0, 1, 6, 10, 13, },
  { 2, 6, 10, 13, },
  { 0, 2, 6, 10, 13, },
  { 1, 2, 6, 10, 13, },
  { 0, 1, 2, 6, 10, 13, },
  { 3, 6, 10, 13, },
  { 0, 3, 6, 10, 13, },
  { 1, 3, 6, 10, 13, },
  { 0, 1, 3, 6, 10, 13, },
  { 2, 3, 6, 10, 13, },
  { 0, 2, 3, 6, 10, 13, },
  { 1, 2, 3, 6, 10, 13, },
  { 0, 1, 2, 3, 6, 10, 13, },
  { 4, 6, 10, 13, },
  { 0, 4, 6, 10, 13, },
  { 1, 4, 6, 10, 13, },
  { 0, 1, 4, 6, 10, 13, },
  { 2, 4, 6, 10, 13, },
  { 0, 2, 4, 6, 10, 13, },
  { 1, 2, 4, 6, 10, 13, },
  { 0, 1, 2, 4, 6, 10, 13, },
  { 3, 4, 6, 10, 13, },
  { 0, 3, 4, 6, 10, 13, },
  { 1, 3, 4, 6, 10, 13, },
  { 0, 1, 3, 4, 6, 10, 13, },
  { 2, 3, 4, 6, 10, 13, },
  { 0, 2, 3, 4, 6, 10, 13, },
  { 1, 2, 3, 4, 6, 10, 13, },
  { 0, 1, 2, 3, 4, 6, 10, 13, },
  { 5, 6, 10, 13, },
  { 0, 5, 6, 10, 13, },
  { 1, 5, 6, 10, 13, },
  { 0, 1, 5, 6, 10, 13, },
  { 2, 5, 6, 10, 13, },
  { 0, 2, 5, 6, 10, 13, },
  { 1, 2, 5, 6, 10, 13, },
  { 0, 1, 2, 5, 6, 10, 13, },
  { 3, 5, 6, 10, 13, },
  { 0, 3, 5, 6, 10, 13, },
  { 1, 3, 5, 6, 10, 13, },
  { 0, 1, 3, 5, 6, 10, 13, },
  { 2, 3, 5, 6, 10, 13, },
  { 0, 2, 3, 5, 6, 10, 13, },
  { 1, 2, 3, 5, 6, 10, 13, },
  { 0, 1, 2, 3, 5, 6, 10, 13, },
  { 4, 5, 6, 10, 13, },
  { 0, 4, 5, 6, 10, 13, },
  { 1, 4, 5, 6, 10, 13, },
  { 0, 1, 4, 5, 6, 10, 13, },
  { 2, 4, 5, 6, 10, 13, },
  { 0, 2, 4, 5, 6, 10, 13, },
  { 1, 2, 4, 5, 6, 10, 13, },
  { 0, 1, 2, 4, 5, 6, 10, 13, },
  { 3, 4, 5, 6, 10, 13, },
  { 0, 3, 4, 5, 6, 10, 13, },
  { 1, 3, 4, 5, 6, 10, 13, },
  { 0, 1, 3, 4, 5, 6, 10, 13, },
  { 2, 3, 4, 5, 6, 10, 13, },
  { 0, 2, 3, 4, 5, 6, 10, 13, },
  { 1, 2, 3, 4, 5, 6, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 13, },
  { 7, 10, 13, },
  { 0, 7, 10, 13, },
  { 1, 7, 10, 13, },
  { 0, 1, 7, 10, 13, },
  { 2, 7, 10, 13, },
  { 0, 2, 7, 10, 13, },
  { 1, 2, 7, 10, 13, },
  { 0, 1, 2, 7, 10, 13, },
  { 3, 7, 10, 13, },
  { 0, 3, 7, 10, 13, },
  { 1, 3, 7, 10, 13, },
  { 0, 1, 3, 7, 10, 13, },
  { 2, 3, 7, 10, 13, },
  { 0, 2, 3, 7, 10, 13, },
  { 1, 2, 3, 7, 10, 13, },
  { 0, 1, 2, 3, 7, 10, 13, },
  { 4, 7, 10, 13, },
  { 0, 4, 7, 10, 13, },
  { 1, 4, 7, 10, 13, },
  { 0, 1, 4, 7, 10, 13, },
  { 2, 4, 7, 10, 13, },
  { 0, 2, 4, 7, 10, 13, },
  { 1, 2, 4, 7, 10, 13, },
  { 0, 1, 2, 4, 7, 10, 13, },
  { 3, 4, 7, 10, 13, },
  { 0, 3, 4, 7, 10, 13, },
  { 1, 3, 4, 7, 10, 13, },
  { 0, 1, 3, 4, 7, 10, 13, },
  { 2, 3, 4, 7, 10, 13, },
  { 0, 2, 3, 4, 7, 10, 13, },
  { 1, 2, 3, 4, 7, 10, 13, },
  { 0, 1, 2, 3, 4, 7, 10, 13, },
  { 5, 7, 10, 13, },
  { 0, 5, 7, 10, 13, },
  { 1, 5, 7, 10, 13, },
  { 0, 1, 5, 7, 10, 13, },
  { 2, 5, 7, 10, 13, },
  { 0, 2, 5, 7, 10, 13, },
  { 1, 2, 5, 7, 10, 13, },
  { 0, 1, 2, 5, 7, 10, 13, },
  { 3, 5, 7, 10, 13, },
  { 0, 3, 5, 7, 10, 13, },
  { 1, 3, 5, 7, 10, 13, },
  { 0, 1, 3, 5, 7, 10, 13, },
  { 2, 3, 5, 7, 10, 13, },
  { 0, 2, 3, 5, 7, 10, 13, },
  { 1, 2, 3, 5, 7, 10, 13, },
  { 0, 1, 2, 3, 5, 7, 10, 13, },
  { 4, 5, 7, 10, 13, },
  { 0, 4, 5, 7, 10, 13, },
  { 1, 4, 5, 7, 10, 13, },
  { 0, 1, 4, 5, 7, 10, 13, },
  { 2, 4, 5, 7, 10, 13, },
  { 0, 2, 4, 5, 7, 10, 13, },
  { 1, 2, 4, 5, 7, 10, 13, },
  { 0, 1, 2, 4, 5, 7, 10, 13, },
  { 3, 4, 5, 7, 10, 13, },
  { 0, 3, 4, 5, 7, 10, 13, },
  { 1, 3, 4, 5, 7, 10, 13, },
  { 0, 1, 3, 4, 5, 7, 10, 13, },
  { 2, 3, 4, 5, 7, 10, 13, },
  { 0, 2, 3, 4, 5, 7, 10, 13, },
  { 1, 2, 3, 4, 5, 7, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 13, },
  { 6, 7, 10, 13, },
  { 0, 6, 7, 10, 13, },
  { 1, 6, 7, 10, 13, },
  { 0, 1, 6, 7, 10, 13, },
  { 2, 6, 7, 10, 13, },
  { 0, 2, 6, 7, 10, 13, },
  { 1, 2, 6, 7, 10, 13, },
  { 0, 1, 2, 6, 7, 10, 13, },
  { 3, 6, 7, 10, 13, },
  { 0, 3, 6, 7, 10, 13, },
  { 1, 3, 6, 7, 10, 13, },
  { 0, 1, 3, 6, 7, 10, 13, },
  { 2, 3, 6, 7, 10, 13, },
  { 0, 2, 3, 6, 7, 10, 13, },
  { 1, 2, 3, 6, 7, 10, 13, },
  { 0, 1, 2, 3, 6, 7, 10, 13, },
  { 4, 6, 7, 10, 13, },
  { 0, 4, 6, 7, 10, 13, },
  { 1, 4, 6, 7, 10, 13, },
  { 0, 1, 4, 6, 7, 10, 13, },
  { 2, 4, 6, 7, 10, 13, },
  { 0, 2, 4, 6, 7, 10, 13, },
  { 1, 2, 4, 6, 7, 10, 13, },
  { 0, 1, 2, 4, 6, 7, 10, 13, },
  { 3, 4, 6, 7, 10, 13, },
  { 0, 3, 4, 6, 7, 10, 13, },
  { 1, 3, 4, 6, 7, 10, 13, },
  { 0, 1, 3, 4, 6, 7, 10, 13, },
  { 2, 3, 4, 6, 7, 10, 13, },
  { 0, 2, 3, 4, 6, 7, 10, 13, },
  { 1, 2, 3, 4, 6, 7, 10, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 13, },
  { 5, 6, 7, 10, 13, },
  { 0, 5, 6, 7, 10, 13, },
  { 1, 5, 6, 7, 10, 13, },
  { 0, 1, 5, 6, 7, 10, 13, },
  { 2, 5, 6, 7, 10, 13, },
  { 0, 2, 5, 6, 7, 10, 13, },
  { 1, 2, 5, 6, 7, 10, 13, },
  { 0, 1, 2, 5, 6, 7, 10, 13, },
  { 3, 5, 6, 7, 10, 13, },
  { 0, 3, 5, 6, 7, 10, 13, },
  { 1, 3, 5, 6, 7, 10, 13, },
  { 0, 1, 3, 5, 6, 7, 10, 13, },
  { 2, 3, 5, 6, 7, 10, 13, },
  { 0, 2, 3, 5, 6, 7, 10, 13, },
  { 1, 2, 3, 5, 6, 7, 10, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 13, },
  { 4, 5, 6, 7, 10, 13, },
  { 0, 4, 5, 6, 7, 10, 13, },
  { 1, 4, 5, 6, 7, 10, 13, },
  { 0, 1, 4, 5, 6, 7, 10, 13, },
  { 2, 4, 5, 6, 7, 10, 13, },
  { 0, 2, 4, 5, 6, 7, 10, 13, },
  { 1, 2, 4, 5, 6, 7, 10, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 13, },
  { 3, 4, 5, 6, 7, 10, 13, },
  { 0, 3, 4, 5, 6, 7, 10, 13, },
  { 1, 3, 4, 5, 6, 7, 10, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 13, },
  { 2, 3, 4, 5, 6, 7, 10, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 13, },
  { 8, 10, 13, },
  { 0, 8, 10, 13, },
  { 1, 8, 10, 13, },
  { 0, 1, 8, 10, 13, },
  { 2, 8, 10, 13, },
  { 0, 2, 8, 10, 13, },
  { 1, 2, 8, 10, 13, },
  { 0, 1, 2, 8, 10, 13, },
  { 3, 8, 10, 13, },
  { 0, 3, 8, 10, 13, },
  { 1, 3, 8, 10, 13, },
  { 0, 1, 3, 8, 10, 13, },
  { 2, 3, 8, 10, 13, },
  { 0, 2, 3, 8, 10, 13, },
  { 1, 2, 3, 8, 10, 13, },
  { 0, 1, 2, 3, 8, 10, 13, },
  { 4, 8, 10, 13, },
  { 0, 4, 8, 10, 13, },
  { 1, 4, 8, 10, 13, },
  { 0, 1, 4, 8, 10, 13, },
  { 2, 4, 8, 10, 13, },
  { 0, 2, 4, 8, 10, 13, },
  { 1, 2, 4, 8, 10, 13, },
  { 0, 1, 2, 4, 8, 10, 13, },
  { 3, 4, 8, 10, 13, },
  { 0, 3, 4, 8, 10, 13, },
  { 1, 3, 4, 8, 10, 13, },
  { 0, 1, 3, 4, 8, 10, 13, },
  { 2, 3, 4, 8, 10, 13, },
  { 0, 2, 3, 4, 8, 10, 13, },
  { 1, 2, 3, 4, 8, 10, 13, },
  { 0, 1, 2, 3, 4, 8, 10, 13, },
  { 5, 8, 10, 13, },
  { 0, 5, 8, 10, 13, },
  { 1, 5, 8, 10, 13, },
  { 0, 1, 5, 8, 10, 13, },
  { 2, 5, 8, 10, 13, },
  { 0, 2, 5, 8, 10, 13, },
  { 1, 2, 5, 8, 10, 13, },
  { 0, 1, 2, 5, 8, 10, 13, },
  { 3, 5, 8, 10, 13, },
  { 0, 3, 5, 8, 10, 13, },
  { 1, 3, 5, 8, 10, 13, },
  { 0, 1, 3, 5, 8, 10, 13, },
  { 2, 3, 5, 8, 10, 13, },
  { 0, 2, 3, 5, 8, 10, 13, },
  { 1, 2, 3, 5, 8, 10, 13, },
  { 0, 1, 2, 3, 5, 8, 10, 13, },
  { 4, 5, 8, 10, 13, },
  { 0, 4, 5, 8, 10, 13, },
  { 1, 4, 5, 8, 10, 13, },
  { 0, 1, 4, 5, 8, 10, 13, },
  { 2, 4, 5, 8, 10, 13, },
  { 0, 2, 4, 5, 8, 10, 13, },
  { 1, 2, 4, 5, 8, 10, 13, },
  { 0, 1, 2, 4, 5, 8, 10, 13, },
  { 3, 4, 5, 8, 10, 13, },
  { 0, 3, 4, 5, 8, 10, 13, },
  { 1, 3, 4, 5, 8, 10, 13, },
  { 0, 1, 3, 4, 5, 8, 10, 13, },
  { 2, 3, 4, 5, 8, 10, 13, },
  { 0, 2, 3, 4, 5, 8, 10, 13, },
  { 1, 2, 3, 4, 5, 8, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 13, },
  { 6, 8, 10, 13, },
  { 0, 6, 8, 10, 13, },
  { 1, 6, 8, 10, 13, },
  { 0, 1, 6, 8, 10, 13, },
  { 2, 6, 8, 10, 13, },
  { 0, 2, 6, 8, 10, 13, },
  { 1, 2, 6, 8, 10, 13, },
  { 0, 1, 2, 6, 8, 10, 13, },
  { 3, 6, 8, 10, 13, },
  { 0, 3, 6, 8, 10, 13, },
  { 1, 3, 6, 8, 10, 13, },
  { 0, 1, 3, 6, 8, 10, 13, },
  { 2, 3, 6, 8, 10, 13, },
  { 0, 2, 3, 6, 8, 10, 13, },
  { 1, 2, 3, 6, 8, 10, 13, },
  { 0, 1, 2, 3, 6, 8, 10, 13, },
  { 4, 6, 8, 10, 13, },
  { 0, 4, 6, 8, 10, 13, },
  { 1, 4, 6, 8, 10, 13, },
  { 0, 1, 4, 6, 8, 10, 13, },
  { 2, 4, 6, 8, 10, 13, },
  { 0, 2, 4, 6, 8, 10, 13, },
  { 1, 2, 4, 6, 8, 10, 13, },
  { 0, 1, 2, 4, 6, 8, 10, 13, },
  { 3, 4, 6, 8, 10, 13, },
  { 0, 3, 4, 6, 8, 10, 13, },
  { 1, 3, 4, 6, 8, 10, 13, },
  { 0, 1, 3, 4, 6, 8, 10, 13, },
  { 2, 3, 4, 6, 8, 10, 13, },
  { 0, 2, 3, 4, 6, 8, 10, 13, },
  { 1, 2, 3, 4, 6, 8, 10, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 13, },
  { 5, 6, 8, 10, 13, },
  { 0, 5, 6, 8, 10, 13, },
  { 1, 5, 6, 8, 10, 13, },
  { 0, 1, 5, 6, 8, 10, 13, },
  { 2, 5, 6, 8, 10, 13, },
  { 0, 2, 5, 6, 8, 10, 13, },
  { 1, 2, 5, 6, 8, 10, 13, },
  { 0, 1, 2, 5, 6, 8, 10, 13, },
  { 3, 5, 6, 8, 10, 13, },
  { 0, 3, 5, 6, 8, 10, 13, },
  { 1, 3, 5, 6, 8, 10, 13, },
  { 0, 1, 3, 5, 6, 8, 10, 13, },
  { 2, 3, 5, 6, 8, 10, 13, },
  { 0, 2, 3, 5, 6, 8, 10, 13, },
  { 1, 2, 3, 5, 6, 8, 10, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 13, },
  { 4, 5, 6, 8, 10, 13, },
  { 0, 4, 5, 6, 8, 10, 13, },
  { 1, 4, 5, 6, 8, 10, 13, },
  { 0, 1, 4, 5, 6, 8, 10, 13, },
  { 2, 4, 5, 6, 8, 10, 13, },
  { 0, 2, 4, 5, 6, 8, 10, 13, },
  { 1, 2, 4, 5, 6, 8, 10, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 13, },
  { 3, 4, 5, 6, 8, 10, 13, },
  { 0, 3, 4, 5, 6, 8, 10, 13, },
  { 1, 3, 4, 5, 6, 8, 10, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 13, },
  { 2, 3, 4, 5, 6, 8, 10, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 13, },
  { 7, 8, 10, 13, },
  { 0, 7, 8, 10, 13, },
  { 1, 7, 8, 10, 13, },
  { 0, 1, 7, 8, 10, 13, },
  { 2, 7, 8, 10, 13, },
  { 0, 2, 7, 8, 10, 13, },
  { 1, 2, 7, 8, 10, 13, },
  { 0, 1, 2, 7, 8, 10, 13, },
  { 3, 7, 8, 10, 13, },
  { 0, 3, 7, 8, 10, 13, },
  { 1, 3, 7, 8, 10, 13, },
  { 0, 1, 3, 7, 8, 10, 13, },
  { 2, 3, 7, 8, 10, 13, },
  { 0, 2, 3, 7, 8, 10, 13, },
  { 1, 2, 3, 7, 8, 10, 13, },
  { 0, 1, 2, 3, 7, 8, 10, 13, },
  { 4, 7, 8, 10, 13, },
  { 0, 4, 7, 8, 10, 13, },
  { 1, 4, 7, 8, 10, 13, },
  { 0, 1, 4, 7, 8, 10, 13, },
  { 2, 4, 7, 8, 10, 13, },
  { 0, 2, 4, 7, 8, 10, 13, },
  { 1, 2, 4, 7, 8, 10, 13, },
  { 0, 1, 2, 4, 7, 8, 10, 13, },
  { 3, 4, 7, 8, 10, 13, },
  { 0, 3, 4, 7, 8, 10, 13, },
  { 1, 3, 4, 7, 8, 10, 13, },
  { 0, 1, 3, 4, 7, 8, 10, 13, },
  { 2, 3, 4, 7, 8, 10, 13, },
  { 0, 2, 3, 4, 7, 8, 10, 13, },
  { 1, 2, 3, 4, 7, 8, 10, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 13, },
  { 5, 7, 8, 10, 13, },
  { 0, 5, 7, 8, 10, 13, },
  { 1, 5, 7, 8, 10, 13, },
  { 0, 1, 5, 7, 8, 10, 13, },
  { 2, 5, 7, 8, 10, 13, },
  { 0, 2, 5, 7, 8, 10, 13, },
  { 1, 2, 5, 7, 8, 10, 13, },
  { 0, 1, 2, 5, 7, 8, 10, 13, },
  { 3, 5, 7, 8, 10, 13, },
  { 0, 3, 5, 7, 8, 10, 13, },
  { 1, 3, 5, 7, 8, 10, 13, },
  { 0, 1, 3, 5, 7, 8, 10, 13, },
  { 2, 3, 5, 7, 8, 10, 13, },
  { 0, 2, 3, 5, 7, 8, 10, 13, },
  { 1, 2, 3, 5, 7, 8, 10, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 13, },
  { 4, 5, 7, 8, 10, 13, },
  { 0, 4, 5, 7, 8, 10, 13, },
  { 1, 4, 5, 7, 8, 10, 13, },
  { 0, 1, 4, 5, 7, 8, 10, 13, },
  { 2, 4, 5, 7, 8, 10, 13, },
  { 0, 2, 4, 5, 7, 8, 10, 13, },
  { 1, 2, 4, 5, 7, 8, 10, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 13, },
  { 3, 4, 5, 7, 8, 10, 13, },
  { 0, 3, 4, 5, 7, 8, 10, 13, },
  { 1, 3, 4, 5, 7, 8, 10, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 13, },
  { 2, 3, 4, 5, 7, 8, 10, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 13, },
  { 6, 7, 8, 10, 13, },
  { 0, 6, 7, 8, 10, 13, },
  { 1, 6, 7, 8, 10, 13, },
  { 0, 1, 6, 7, 8, 10, 13, },
  { 2, 6, 7, 8, 10, 13, },
  { 0, 2, 6, 7, 8, 10, 13, },
  { 1, 2, 6, 7, 8, 10, 13, },
  { 0, 1, 2, 6, 7, 8, 10, 13, },
  { 3, 6, 7, 8, 10, 13, },
  { 0, 3, 6, 7, 8, 10, 13, },
  { 1, 3, 6, 7, 8, 10, 13, },
  { 0, 1, 3, 6, 7, 8, 10, 13, },
  { 2, 3, 6, 7, 8, 10, 13, },
  { 0, 2, 3, 6, 7, 8, 10, 13, },
  { 1, 2, 3, 6, 7, 8, 10, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 13, },
  { 4, 6, 7, 8, 10, 13, },
  { 0, 4, 6, 7, 8, 10, 13, },
  { 1, 4, 6, 7, 8, 10, 13, },
  { 0, 1, 4, 6, 7, 8, 10, 13, },
  { 2, 4, 6, 7, 8, 10, 13, },
  { 0, 2, 4, 6, 7, 8, 10, 13, },
  { 1, 2, 4, 6, 7, 8, 10, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 13, },
  { 3, 4, 6, 7, 8, 10, 13, },
  { 0, 3, 4, 6, 7, 8, 10, 13, },
  { 1, 3, 4, 6, 7, 8, 10, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 13, },
  { 2, 3, 4, 6, 7, 8, 10, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 13, },
  { 5, 6, 7, 8, 10, 13, },
  { 0, 5, 6, 7, 8, 10, 13, },
  { 1, 5, 6, 7, 8, 10, 13, },
  { 0, 1, 5, 6, 7, 8, 10, 13, },
  { 2, 5, 6, 7, 8, 10, 13, },
  { 0, 2, 5, 6, 7, 8, 10, 13, },
  { 1, 2, 5, 6, 7, 8, 10, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 13, },
  { 3, 5, 6, 7, 8, 10, 13, },
  { 0, 3, 5, 6, 7, 8, 10, 13, },
  { 1, 3, 5, 6, 7, 8, 10, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 13, },
  { 2, 3, 5, 6, 7, 8, 10, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 13, },
  { 4, 5, 6, 7, 8, 10, 13, },
  { 0, 4, 5, 6, 7, 8, 10, 13, },
  { 1, 4, 5, 6, 7, 8, 10, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 13, },
  { 2, 4, 5, 6, 7, 8, 10, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 13, },
  { 3, 4, 5, 6, 7, 8, 10, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, },
  { 9, 10, 13, },
  { 0, 9, 10, 13, },
  { 1, 9, 10, 13, },
  { 0, 1, 9, 10, 13, },
  { 2, 9, 10, 13, },
  { 0, 2, 9, 10, 13, },
  { 1, 2, 9, 10, 13, },
  { 0, 1, 2, 9, 10, 13, },
  { 3, 9, 10, 13, },
  { 0, 3, 9, 10, 13, },
  { 1, 3, 9, 10, 13, },
  { 0, 1, 3, 9, 10, 13, },
  { 2, 3, 9, 10, 13, },
  { 0, 2, 3, 9, 10, 13, },
  { 1, 2, 3, 9, 10, 13, },
  { 0, 1, 2, 3, 9, 10, 13, },
  { 4, 9, 10, 13, },
  { 0, 4, 9, 10, 13, },
  { 1, 4, 9, 10, 13, },
  { 0, 1, 4, 9, 10, 13, },
  { 2, 4, 9, 10, 13, },
  { 0, 2, 4, 9, 10, 13, },
  { 1, 2, 4, 9, 10, 13, },
  { 0, 1, 2, 4, 9, 10, 13, },
  { 3, 4, 9, 10, 13, },
  { 0, 3, 4, 9, 10, 13, },
  { 1, 3, 4, 9, 10, 13, },
  { 0, 1, 3, 4, 9, 10, 13, },
  { 2, 3, 4, 9, 10, 13, },
  { 0, 2, 3, 4, 9, 10, 13, },
  { 1, 2, 3, 4, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 9, 10, 13, },
  { 5, 9, 10, 13, },
  { 0, 5, 9, 10, 13, },
  { 1, 5, 9, 10, 13, },
  { 0, 1, 5, 9, 10, 13, },
  { 2, 5, 9, 10, 13, },
  { 0, 2, 5, 9, 10, 13, },
  { 1, 2, 5, 9, 10, 13, },
  { 0, 1, 2, 5, 9, 10, 13, },
  { 3, 5, 9, 10, 13, },
  { 0, 3, 5, 9, 10, 13, },
  { 1, 3, 5, 9, 10, 13, },
  { 0, 1, 3, 5, 9, 10, 13, },
  { 2, 3, 5, 9, 10, 13, },
  { 0, 2, 3, 5, 9, 10, 13, },
  { 1, 2, 3, 5, 9, 10, 13, },
  { 0, 1, 2, 3, 5, 9, 10, 13, },
  { 4, 5, 9, 10, 13, },
  { 0, 4, 5, 9, 10, 13, },
  { 1, 4, 5, 9, 10, 13, },
  { 0, 1, 4, 5, 9, 10, 13, },
  { 2, 4, 5, 9, 10, 13, },
  { 0, 2, 4, 5, 9, 10, 13, },
  { 1, 2, 4, 5, 9, 10, 13, },
  { 0, 1, 2, 4, 5, 9, 10, 13, },
  { 3, 4, 5, 9, 10, 13, },
  { 0, 3, 4, 5, 9, 10, 13, },
  { 1, 3, 4, 5, 9, 10, 13, },
  { 0, 1, 3, 4, 5, 9, 10, 13, },
  { 2, 3, 4, 5, 9, 10, 13, },
  { 0, 2, 3, 4, 5, 9, 10, 13, },
  { 1, 2, 3, 4, 5, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 13, },
  { 6, 9, 10, 13, },
  { 0, 6, 9, 10, 13, },
  { 1, 6, 9, 10, 13, },
  { 0, 1, 6, 9, 10, 13, },
  { 2, 6, 9, 10, 13, },
  { 0, 2, 6, 9, 10, 13, },
  { 1, 2, 6, 9, 10, 13, },
  { 0, 1, 2, 6, 9, 10, 13, },
  { 3, 6, 9, 10, 13, },
  { 0, 3, 6, 9, 10, 13, },
  { 1, 3, 6, 9, 10, 13, },
  { 0, 1, 3, 6, 9, 10, 13, },
  { 2, 3, 6, 9, 10, 13, },
  { 0, 2, 3, 6, 9, 10, 13, },
  { 1, 2, 3, 6, 9, 10, 13, },
  { 0, 1, 2, 3, 6, 9, 10, 13, },
  { 4, 6, 9, 10, 13, },
  { 0, 4, 6, 9, 10, 13, },
  { 1, 4, 6, 9, 10, 13, },
  { 0, 1, 4, 6, 9, 10, 13, },
  { 2, 4, 6, 9, 10, 13, },
  { 0, 2, 4, 6, 9, 10, 13, },
  { 1, 2, 4, 6, 9, 10, 13, },
  { 0, 1, 2, 4, 6, 9, 10, 13, },
  { 3, 4, 6, 9, 10, 13, },
  { 0, 3, 4, 6, 9, 10, 13, },
  { 1, 3, 4, 6, 9, 10, 13, },
  { 0, 1, 3, 4, 6, 9, 10, 13, },
  { 2, 3, 4, 6, 9, 10, 13, },
  { 0, 2, 3, 4, 6, 9, 10, 13, },
  { 1, 2, 3, 4, 6, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 13, },
  { 5, 6, 9, 10, 13, },
  { 0, 5, 6, 9, 10, 13, },
  { 1, 5, 6, 9, 10, 13, },
  { 0, 1, 5, 6, 9, 10, 13, },
  { 2, 5, 6, 9, 10, 13, },
  { 0, 2, 5, 6, 9, 10, 13, },
  { 1, 2, 5, 6, 9, 10, 13, },
  { 0, 1, 2, 5, 6, 9, 10, 13, },
  { 3, 5, 6, 9, 10, 13, },
  { 0, 3, 5, 6, 9, 10, 13, },
  { 1, 3, 5, 6, 9, 10, 13, },
  { 0, 1, 3, 5, 6, 9, 10, 13, },
  { 2, 3, 5, 6, 9, 10, 13, },
  { 0, 2, 3, 5, 6, 9, 10, 13, },
  { 1, 2, 3, 5, 6, 9, 10, 13, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 13, },
  { 4, 5, 6, 9, 10, 13, },
  { 0, 4, 5, 6, 9, 10, 13, },
  { 1, 4, 5, 6, 9, 10, 13, },
  { 0, 1, 4, 5, 6, 9, 10, 13, },
  { 2, 4, 5, 6, 9, 10, 13, },
  { 0, 2, 4, 5, 6, 9, 10, 13, },
  { 1, 2, 4, 5, 6, 9, 10, 13, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 13, },
  { 3, 4, 5, 6, 9, 10, 13, },
  { 0, 3, 4, 5, 6, 9, 10, 13, },
  { 1, 3, 4, 5, 6, 9, 10, 13, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 13, },
  { 2, 3, 4, 5, 6, 9, 10, 13, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 13, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 13, },
  { 7, 9, 10, 13, },
  { 0, 7, 9, 10, 13, },
  { 1, 7, 9, 10, 13, },
  { 0, 1, 7, 9, 10, 13, },
  { 2, 7, 9, 10, 13, },
  { 0, 2, 7, 9, 10, 13, },
  { 1, 2, 7, 9, 10, 13, },
  { 0, 1, 2, 7, 9, 10, 13, },
  { 3, 7, 9, 10, 13, },
  { 0, 3, 7, 9, 10, 13, },
  { 1, 3, 7, 9, 10, 13, },
  { 0, 1, 3, 7, 9, 10, 13, },
  { 2, 3, 7, 9, 10, 13, },
  { 0, 2, 3, 7, 9, 10, 13, },
  { 1, 2, 3, 7, 9, 10, 13, },
  { 0, 1, 2, 3, 7, 9, 10, 13, },
  { 4, 7, 9, 10, 13, },
  { 0, 4, 7, 9, 10, 13, },
  { 1, 4, 7, 9, 10, 13, },
  { 0, 1, 4, 7, 9, 10, 13, },
  { 2, 4, 7, 9, 10, 13, },
  { 0, 2, 4, 7, 9, 10, 13, },
  { 1, 2, 4, 7, 9, 10, 13, },
  { 0, 1, 2, 4, 7, 9, 10, 13, },
  { 3, 4, 7, 9, 10, 13, },
  { 0, 3, 4, 7, 9, 10, 13, },
  { 1, 3, 4, 7, 9, 10, 13, },
  { 0, 1, 3, 4, 7, 9, 10, 13, },
  { 2, 3, 4, 7, 9, 10, 13, },
  { 0, 2, 3, 4, 7, 9, 10, 13, },
  { 1, 2, 3, 4, 7, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 13, },
  { 5, 7, 9, 10, 13, },
  { 0, 5, 7, 9, 10, 13, },
  { 1, 5, 7, 9, 10, 13, },
  { 0, 1, 5, 7, 9, 10, 13, },
  { 2, 5, 7, 9, 10, 13, },
  { 0, 2, 5, 7, 9, 10, 13, },
  { 1, 2, 5, 7, 9, 10, 13, },
  { 0, 1, 2, 5, 7, 9, 10, 13, },
  { 3, 5, 7, 9, 10, 13, },
  { 0, 3, 5, 7, 9, 10, 13, },
  { 1, 3, 5, 7, 9, 10, 13, },
  { 0, 1, 3, 5, 7, 9, 10, 13, },
  { 2, 3, 5, 7, 9, 10, 13, },
  { 0, 2, 3, 5, 7, 9, 10, 13, },
  { 1, 2, 3, 5, 7, 9, 10, 13, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 13, },
  { 4, 5, 7, 9, 10, 13, },
  { 0, 4, 5, 7, 9, 10, 13, },
  { 1, 4, 5, 7, 9, 10, 13, },
  { 0, 1, 4, 5, 7, 9, 10, 13, },
  { 2, 4, 5, 7, 9, 10, 13, },
  { 0, 2, 4, 5, 7, 9, 10, 13, },
  { 1, 2, 4, 5, 7, 9, 10, 13, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 13, },
  { 3, 4, 5, 7, 9, 10, 13, },
  { 0, 3, 4, 5, 7, 9, 10, 13, },
  { 1, 3, 4, 5, 7, 9, 10, 13, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 13, },
  { 2, 3, 4, 5, 7, 9, 10, 13, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 13, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 13, },
  { 6, 7, 9, 10, 13, },
  { 0, 6, 7, 9, 10, 13, },
  { 1, 6, 7, 9, 10, 13, },
  { 0, 1, 6, 7, 9, 10, 13, },
  { 2, 6, 7, 9, 10, 13, },
  { 0, 2, 6, 7, 9, 10, 13, },
  { 1, 2, 6, 7, 9, 10, 13, },
  { 0, 1, 2, 6, 7, 9, 10, 13, },
  { 3, 6, 7, 9, 10, 13, },
  { 0, 3, 6, 7, 9, 10, 13, },
  { 1, 3, 6, 7, 9, 10, 13, },
  { 0, 1, 3, 6, 7, 9, 10, 13, },
  { 2, 3, 6, 7, 9, 10, 13, },
  { 0, 2, 3, 6, 7, 9, 10, 13, },
  { 1, 2, 3, 6, 7, 9, 10, 13, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 13, },
  { 4, 6, 7, 9, 10, 13, },
  { 0, 4, 6, 7, 9, 10, 13, },
  { 1, 4, 6, 7, 9, 10, 13, },
  { 0, 1, 4, 6, 7, 9, 10, 13, },
  { 2, 4, 6, 7, 9, 10, 13, },
  { 0, 2, 4, 6, 7, 9, 10, 13, },
  { 1, 2, 4, 6, 7, 9, 10, 13, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 13, },
  { 3, 4, 6, 7, 9, 10, 13, },
  { 0, 3, 4, 6, 7, 9, 10, 13, },
  { 1, 3, 4, 6, 7, 9, 10, 13, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 13, },
  { 2, 3, 4, 6, 7, 9, 10, 13, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 13, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 13, },
  { 5, 6, 7, 9, 10, 13, },
  { 0, 5, 6, 7, 9, 10, 13, },
  { 1, 5, 6, 7, 9, 10, 13, },
  { 0, 1, 5, 6, 7, 9, 10, 13, },
  { 2, 5, 6, 7, 9, 10, 13, },
  { 0, 2, 5, 6, 7, 9, 10, 13, },
  { 1, 2, 5, 6, 7, 9, 10, 13, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 13, },
  { 3, 5, 6, 7, 9, 10, 13, },
  { 0, 3, 5, 6, 7, 9, 10, 13, },
  { 1, 3, 5, 6, 7, 9, 10, 13, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 13, },
  { 2, 3, 5, 6, 7, 9, 10, 13, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 13, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 13, },
  { 4, 5, 6, 7, 9, 10, 13, },
  { 0, 4, 5, 6, 7, 9, 10, 13, },
  { 1, 4, 5, 6, 7, 9, 10, 13, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 13, },
  { 2, 4, 5, 6, 7, 9, 10, 13, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 13, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 13, },
  { 3, 4, 5, 6, 7, 9, 10, 13, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 13, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 13, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 13, },
  { 8, 9, 10, 13, },
  { 0, 8, 9, 10, 13, },
  { 1, 8, 9, 10, 13, },
  { 0, 1, 8, 9, 10, 13, },
  { 2, 8, 9, 10, 13, },
  { 0, 2, 8, 9, 10, 13, },
  { 1, 2, 8, 9, 10, 13, },
  { 0, 1, 2, 8, 9, 10, 13, },
  { 3, 8, 9, 10, 13, },
  { 0, 3, 8, 9, 10, 13, },
  { 1, 3, 8, 9, 10, 13, },
  { 0, 1, 3, 8, 9, 10, 13, },
  { 2, 3, 8, 9, 10, 13, },
  { 0, 2, 3, 8, 9, 10, 13, },
  { 1, 2, 3, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 8, 9, 10, 13, },
  { 4, 8, 9, 10, 13, },
  { 0, 4, 8, 9, 10, 13, },
  { 1, 4, 8, 9, 10, 13, },
  { 0, 1, 4, 8, 9, 10, 13, },
  { 2, 4, 8, 9, 10, 13, },
  { 0, 2, 4, 8, 9, 10, 13, },
  { 1, 2, 4, 8, 9, 10, 13, },
  { 0, 1, 2, 4, 8, 9, 10, 13, },
  { 3, 4, 8, 9, 10, 13, },
  { 0, 3, 4, 8, 9, 10, 13, },
  { 1, 3, 4, 8, 9, 10, 13, },
  { 0, 1, 3, 4, 8, 9, 10, 13, },
  { 2, 3, 4, 8, 9, 10, 13, },
  { 0, 2, 3, 4, 8, 9, 10, 13, },
  { 1, 2, 3, 4, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 13, },
  { 5, 8, 9, 10, 13, },
  { 0, 5, 8, 9, 10, 13, },
  { 1, 5, 8, 9, 10, 13, },
  { 0, 1, 5, 8, 9, 10, 13, },
  { 2, 5, 8, 9, 10, 13, },
  { 0, 2, 5, 8, 9, 10, 13, },
  { 1, 2, 5, 8, 9, 10, 13, },
  { 0, 1, 2, 5, 8, 9, 10, 13, },
  { 3, 5, 8, 9, 10, 13, },
  { 0, 3, 5, 8, 9, 10, 13, },
  { 1, 3, 5, 8, 9, 10, 13, },
  { 0, 1, 3, 5, 8, 9, 10, 13, },
  { 2, 3, 5, 8, 9, 10, 13, },
  { 0, 2, 3, 5, 8, 9, 10, 13, },
  { 1, 2, 3, 5, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 13, },
  { 4, 5, 8, 9, 10, 13, },
  { 0, 4, 5, 8, 9, 10, 13, },
  { 1, 4, 5, 8, 9, 10, 13, },
  { 0, 1, 4, 5, 8, 9, 10, 13, },
  { 2, 4, 5, 8, 9, 10, 13, },
  { 0, 2, 4, 5, 8, 9, 10, 13, },
  { 1, 2, 4, 5, 8, 9, 10, 13, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 13, },
  { 3, 4, 5, 8, 9, 10, 13, },
  { 0, 3, 4, 5, 8, 9, 10, 13, },
  { 1, 3, 4, 5, 8, 9, 10, 13, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 13, },
  { 2, 3, 4, 5, 8, 9, 10, 13, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 13, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 13, },
  { 6, 8, 9, 10, 13, },
  { 0, 6, 8, 9, 10, 13, },
  { 1, 6, 8, 9, 10, 13, },
  { 0, 1, 6, 8, 9, 10, 13, },
  { 2, 6, 8, 9, 10, 13, },
  { 0, 2, 6, 8, 9, 10, 13, },
  { 1, 2, 6, 8, 9, 10, 13, },
  { 0, 1, 2, 6, 8, 9, 10, 13, },
  { 3, 6, 8, 9, 10, 13, },
  { 0, 3, 6, 8, 9, 10, 13, },
  { 1, 3, 6, 8, 9, 10, 13, },
  { 0, 1, 3, 6, 8, 9, 10, 13, },
  { 2, 3, 6, 8, 9, 10, 13, },
  { 0, 2, 3, 6, 8, 9, 10, 13, },
  { 1, 2, 3, 6, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 13, },
  { 4, 6, 8, 9, 10, 13, },
  { 0, 4, 6, 8, 9, 10, 13, },
  { 1, 4, 6, 8, 9, 10, 13, },
  { 0, 1, 4, 6, 8, 9, 10, 13, },
  { 2, 4, 6, 8, 9, 10, 13, },
  { 0, 2, 4, 6, 8, 9, 10, 13, },
  { 1, 2, 4, 6, 8, 9, 10, 13, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 13, },
  { 3, 4, 6, 8, 9, 10, 13, },
  { 0, 3, 4, 6, 8, 9, 10, 13, },
  { 1, 3, 4, 6, 8, 9, 10, 13, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 13, },
  { 2, 3, 4, 6, 8, 9, 10, 13, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 13, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 13, },
  { 5, 6, 8, 9, 10, 13, },
  { 0, 5, 6, 8, 9, 10, 13, },
  { 1, 5, 6, 8, 9, 10, 13, },
  { 0, 1, 5, 6, 8, 9, 10, 13, },
  { 2, 5, 6, 8, 9, 10, 13, },
  { 0, 2, 5, 6, 8, 9, 10, 13, },
  { 1, 2, 5, 6, 8, 9, 10, 13, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 13, },
  { 3, 5, 6, 8, 9, 10, 13, },
  { 0, 3, 5, 6, 8, 9, 10, 13, },
  { 1, 3, 5, 6, 8, 9, 10, 13, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 13, },
  { 2, 3, 5, 6, 8, 9, 10, 13, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 13, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 13, },
  { 4, 5, 6, 8, 9, 10, 13, },
  { 0, 4, 5, 6, 8, 9, 10, 13, },
  { 1, 4, 5, 6, 8, 9, 10, 13, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 13, },
  { 2, 4, 5, 6, 8, 9, 10, 13, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 13, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 13, },
  { 3, 4, 5, 6, 8, 9, 10, 13, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 13, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 13, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 13, },
  { 7, 8, 9, 10, 13, },
  { 0, 7, 8, 9, 10, 13, },
  { 1, 7, 8, 9, 10, 13, },
  { 0, 1, 7, 8, 9, 10, 13, },
  { 2, 7, 8, 9, 10, 13, },
  { 0, 2, 7, 8, 9, 10, 13, },
  { 1, 2, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 7, 8, 9, 10, 13, },
  { 3, 7, 8, 9, 10, 13, },
  { 0, 3, 7, 8, 9, 10, 13, },
  { 1, 3, 7, 8, 9, 10, 13, },
  { 0, 1, 3, 7, 8, 9, 10, 13, },
  { 2, 3, 7, 8, 9, 10, 13, },
  { 0, 2, 3, 7, 8, 9, 10, 13, },
  { 1, 2, 3, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 13, },
  { 4, 7, 8, 9, 10, 13, },
  { 0, 4, 7, 8, 9, 10, 13, },
  { 1, 4, 7, 8, 9, 10, 13, },
  { 0, 1, 4, 7, 8, 9, 10, 13, },
  { 2, 4, 7, 8, 9, 10, 13, },
  { 0, 2, 4, 7, 8, 9, 10, 13, },
  { 1, 2, 4, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 13, },
  { 3, 4, 7, 8, 9, 10, 13, },
  { 0, 3, 4, 7, 8, 9, 10, 13, },
  { 1, 3, 4, 7, 8, 9, 10, 13, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 13, },
  { 2, 3, 4, 7, 8, 9, 10, 13, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 13, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 13, },
  { 5, 7, 8, 9, 10, 13, },
  { 0, 5, 7, 8, 9, 10, 13, },
  { 1, 5, 7, 8, 9, 10, 13, },
  { 0, 1, 5, 7, 8, 9, 10, 13, },
  { 2, 5, 7, 8, 9, 10, 13, },
  { 0, 2, 5, 7, 8, 9, 10, 13, },
  { 1, 2, 5, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 13, },
  { 3, 5, 7, 8, 9, 10, 13, },
  { 0, 3, 5, 7, 8, 9, 10, 13, },
  { 1, 3, 5, 7, 8, 9, 10, 13, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 13, },
  { 2, 3, 5, 7, 8, 9, 10, 13, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 13, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 13, },
  { 4, 5, 7, 8, 9, 10, 13, },
  { 0, 4, 5, 7, 8, 9, 10, 13, },
  { 1, 4, 5, 7, 8, 9, 10, 13, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 13, },
  { 2, 4, 5, 7, 8, 9, 10, 13, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 13, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 13, },
  { 3, 4, 5, 7, 8, 9, 10, 13, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 13, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 13, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 13, },
  { 6, 7, 8, 9, 10, 13, },
  { 0, 6, 7, 8, 9, 10, 13, },
  { 1, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 6, 7, 8, 9, 10, 13, },
  { 2, 6, 7, 8, 9, 10, 13, },
  { 0, 2, 6, 7, 8, 9, 10, 13, },
  { 1, 2, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 13, },
  { 3, 6, 7, 8, 9, 10, 13, },
  { 0, 3, 6, 7, 8, 9, 10, 13, },
  { 1, 3, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 13, },
  { 2, 3, 6, 7, 8, 9, 10, 13, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 13, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 13, },
  { 4, 6, 7, 8, 9, 10, 13, },
  { 0, 4, 6, 7, 8, 9, 10, 13, },
  { 1, 4, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 13, },
  { 2, 4, 6, 7, 8, 9, 10, 13, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 13, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 13, },
  { 3, 4, 6, 7, 8, 9, 10, 13, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 13, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 13, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 13, },
  { 5, 6, 7, 8, 9, 10, 13, },
  { 0, 5, 6, 7, 8, 9, 10, 13, },
  { 1, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 13, },
  { 2, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 13, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 13, },
  { 3, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 13, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 13, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 13, },
  { 4, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, },
  { 11, 13, },
  { 0, 11, 13, },
  { 1, 11, 13, },
  { 0, 1, 11, 13, },
  { 2, 11, 13, },
  { 0, 2, 11, 13, },
  { 1, 2, 11, 13, },
  { 0, 1, 2, 11, 13, },
  { 3, 11, 13, },
  { 0, 3, 11, 13, },
  { 1, 3, 11, 13, },
  { 0, 1, 3, 11, 13, },
  { 2, 3, 11, 13, },
  { 0, 2, 3, 11, 13, },
  { 1, 2, 3, 11, 13, },
  { 0, 1, 2, 3, 11, 13, },
  { 4, 11, 13, },
  { 0, 4, 11, 13, },
  { 1, 4, 11, 13, },
  { 0, 1, 4, 11, 13, },
  { 2, 4, 11, 13, },
  { 0, 2, 4, 11, 13, },
  { 1, 2, 4, 11, 13, },
  { 0, 1, 2, 4, 11, 13, },
  { 3, 4, 11, 13, },
  { 0, 3, 4, 11, 13, },
  { 1, 3, 4, 11, 13, },
  { 0, 1, 3, 4, 11, 13, },
  { 2, 3, 4, 11, 13, },
  { 0, 2, 3, 4, 11, 13, },
  { 1, 2, 3, 4, 11, 13, },
  { 0, 1, 2, 3, 4, 11, 13, },
  { 5, 11, 13, },
  { 0, 5, 11, 13, },
  { 1, 5, 11, 13, },
  { 0, 1, 5, 11, 13, },
  { 2, 5, 11, 13, },
  { 0, 2, 5, 11, 13, },
  { 1, 2, 5, 11, 13, },
  { 0, 1, 2, 5, 11, 13, },
  { 3, 5, 11, 13, },
  { 0, 3, 5, 11, 13, },
  { 1, 3, 5, 11, 13, },
  { 0, 1, 3, 5, 11, 13, },
  { 2, 3, 5, 11, 13, },
  { 0, 2, 3, 5, 11, 13, },
  { 1, 2, 3, 5, 11, 13, },
  { 0, 1, 2, 3, 5, 11, 13, },
  { 4, 5, 11, 13, },
  { 0, 4, 5, 11, 13, },
  { 1, 4, 5, 11, 13, },
  { 0, 1, 4, 5, 11, 13, },
  { 2, 4, 5, 11, 13, },
  { 0, 2, 4, 5, 11, 13, },
  { 1, 2, 4, 5, 11, 13, },
  { 0, 1, 2, 4, 5, 11, 13, },
  { 3, 4, 5, 11, 13, },
  { 0, 3, 4, 5, 11, 13, },
  { 1, 3, 4, 5, 11, 13, },
  { 0, 1, 3, 4, 5, 11, 13, },
  { 2, 3, 4, 5, 11, 13, },
  { 0, 2, 3, 4, 5, 11, 13, },
  { 1, 2, 3, 4, 5, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 11, 13, },
  { 6, 11, 13, },
  { 0, 6, 11, 13, },
  { 1, 6, 11, 13, },
  { 0, 1, 6, 11, 13, },
  { 2, 6, 11, 13, },
  { 0, 2, 6, 11, 13, },
  { 1, 2, 6, 11, 13, },
  { 0, 1, 2, 6, 11, 13, },
  { 3, 6, 11, 13, },
  { 0, 3, 6, 11, 13, },
  { 1, 3, 6, 11, 13, },
  { 0, 1, 3, 6, 11, 13, },
  { 2, 3, 6, 11, 13, },
  { 0, 2, 3, 6, 11, 13, },
  { 1, 2, 3, 6, 11, 13, },
  { 0, 1, 2, 3, 6, 11, 13, },
  { 4, 6, 11, 13, },
  { 0, 4, 6, 11, 13, },
  { 1, 4, 6, 11, 13, },
  { 0, 1, 4, 6, 11, 13, },
  { 2, 4, 6, 11, 13, },
  { 0, 2, 4, 6, 11, 13, },
  { 1, 2, 4, 6, 11, 13, },
  { 0, 1, 2, 4, 6, 11, 13, },
  { 3, 4, 6, 11, 13, },
  { 0, 3, 4, 6, 11, 13, },
  { 1, 3, 4, 6, 11, 13, },
  { 0, 1, 3, 4, 6, 11, 13, },
  { 2, 3, 4, 6, 11, 13, },
  { 0, 2, 3, 4, 6, 11, 13, },
  { 1, 2, 3, 4, 6, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 11, 13, },
  { 5, 6, 11, 13, },
  { 0, 5, 6, 11, 13, },
  { 1, 5, 6, 11, 13, },
  { 0, 1, 5, 6, 11, 13, },
  { 2, 5, 6, 11, 13, },
  { 0, 2, 5, 6, 11, 13, },
  { 1, 2, 5, 6, 11, 13, },
  { 0, 1, 2, 5, 6, 11, 13, },
  { 3, 5, 6, 11, 13, },
  { 0, 3, 5, 6, 11, 13, },
  { 1, 3, 5, 6, 11, 13, },
  { 0, 1, 3, 5, 6, 11, 13, },
  { 2, 3, 5, 6, 11, 13, },
  { 0, 2, 3, 5, 6, 11, 13, },
  { 1, 2, 3, 5, 6, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 11, 13, },
  { 4, 5, 6, 11, 13, },
  { 0, 4, 5, 6, 11, 13, },
  { 1, 4, 5, 6, 11, 13, },
  { 0, 1, 4, 5, 6, 11, 13, },
  { 2, 4, 5, 6, 11, 13, },
  { 0, 2, 4, 5, 6, 11, 13, },
  { 1, 2, 4, 5, 6, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 11, 13, },
  { 3, 4, 5, 6, 11, 13, },
  { 0, 3, 4, 5, 6, 11, 13, },
  { 1, 3, 4, 5, 6, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 11, 13, },
  { 2, 3, 4, 5, 6, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 13, },
  { 7, 11, 13, },
  { 0, 7, 11, 13, },
  { 1, 7, 11, 13, },
  { 0, 1, 7, 11, 13, },
  { 2, 7, 11, 13, },
  { 0, 2, 7, 11, 13, },
  { 1, 2, 7, 11, 13, },
  { 0, 1, 2, 7, 11, 13, },
  { 3, 7, 11, 13, },
  { 0, 3, 7, 11, 13, },
  { 1, 3, 7, 11, 13, },
  { 0, 1, 3, 7, 11, 13, },
  { 2, 3, 7, 11, 13, },
  { 0, 2, 3, 7, 11, 13, },
  { 1, 2, 3, 7, 11, 13, },
  { 0, 1, 2, 3, 7, 11, 13, },
  { 4, 7, 11, 13, },
  { 0, 4, 7, 11, 13, },
  { 1, 4, 7, 11, 13, },
  { 0, 1, 4, 7, 11, 13, },
  { 2, 4, 7, 11, 13, },
  { 0, 2, 4, 7, 11, 13, },
  { 1, 2, 4, 7, 11, 13, },
  { 0, 1, 2, 4, 7, 11, 13, },
  { 3, 4, 7, 11, 13, },
  { 0, 3, 4, 7, 11, 13, },
  { 1, 3, 4, 7, 11, 13, },
  { 0, 1, 3, 4, 7, 11, 13, },
  { 2, 3, 4, 7, 11, 13, },
  { 0, 2, 3, 4, 7, 11, 13, },
  { 1, 2, 3, 4, 7, 11, 13, },
  { 0, 1, 2, 3, 4, 7, 11, 13, },
  { 5, 7, 11, 13, },
  { 0, 5, 7, 11, 13, },
  { 1, 5, 7, 11, 13, },
  { 0, 1, 5, 7, 11, 13, },
  { 2, 5, 7, 11, 13, },
  { 0, 2, 5, 7, 11, 13, },
  { 1, 2, 5, 7, 11, 13, },
  { 0, 1, 2, 5, 7, 11, 13, },
  { 3, 5, 7, 11, 13, },
  { 0, 3, 5, 7, 11, 13, },
  { 1, 3, 5, 7, 11, 13, },
  { 0, 1, 3, 5, 7, 11, 13, },
  { 2, 3, 5, 7, 11, 13, },
  { 0, 2, 3, 5, 7, 11, 13, },
  { 1, 2, 3, 5, 7, 11, 13, },
  { 0, 1, 2, 3, 5, 7, 11, 13, },
  { 4, 5, 7, 11, 13, },
  { 0, 4, 5, 7, 11, 13, },
  { 1, 4, 5, 7, 11, 13, },
  { 0, 1, 4, 5, 7, 11, 13, },
  { 2, 4, 5, 7, 11, 13, },
  { 0, 2, 4, 5, 7, 11, 13, },
  { 1, 2, 4, 5, 7, 11, 13, },
  { 0, 1, 2, 4, 5, 7, 11, 13, },
  { 3, 4, 5, 7, 11, 13, },
  { 0, 3, 4, 5, 7, 11, 13, },
  { 1, 3, 4, 5, 7, 11, 13, },
  { 0, 1, 3, 4, 5, 7, 11, 13, },
  { 2, 3, 4, 5, 7, 11, 13, },
  { 0, 2, 3, 4, 5, 7, 11, 13, },
  { 1, 2, 3, 4, 5, 7, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 13, },
  { 6, 7, 11, 13, },
  { 0, 6, 7, 11, 13, },
  { 1, 6, 7, 11, 13, },
  { 0, 1, 6, 7, 11, 13, },
  { 2, 6, 7, 11, 13, },
  { 0, 2, 6, 7, 11, 13, },
  { 1, 2, 6, 7, 11, 13, },
  { 0, 1, 2, 6, 7, 11, 13, },
  { 3, 6, 7, 11, 13, },
  { 0, 3, 6, 7, 11, 13, },
  { 1, 3, 6, 7, 11, 13, },
  { 0, 1, 3, 6, 7, 11, 13, },
  { 2, 3, 6, 7, 11, 13, },
  { 0, 2, 3, 6, 7, 11, 13, },
  { 1, 2, 3, 6, 7, 11, 13, },
  { 0, 1, 2, 3, 6, 7, 11, 13, },
  { 4, 6, 7, 11, 13, },
  { 0, 4, 6, 7, 11, 13, },
  { 1, 4, 6, 7, 11, 13, },
  { 0, 1, 4, 6, 7, 11, 13, },
  { 2, 4, 6, 7, 11, 13, },
  { 0, 2, 4, 6, 7, 11, 13, },
  { 1, 2, 4, 6, 7, 11, 13, },
  { 0, 1, 2, 4, 6, 7, 11, 13, },
  { 3, 4, 6, 7, 11, 13, },
  { 0, 3, 4, 6, 7, 11, 13, },
  { 1, 3, 4, 6, 7, 11, 13, },
  { 0, 1, 3, 4, 6, 7, 11, 13, },
  { 2, 3, 4, 6, 7, 11, 13, },
  { 0, 2, 3, 4, 6, 7, 11, 13, },
  { 1, 2, 3, 4, 6, 7, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 13, },
  { 5, 6, 7, 11, 13, },
  { 0, 5, 6, 7, 11, 13, },
  { 1, 5, 6, 7, 11, 13, },
  { 0, 1, 5, 6, 7, 11, 13, },
  { 2, 5, 6, 7, 11, 13, },
  { 0, 2, 5, 6, 7, 11, 13, },
  { 1, 2, 5, 6, 7, 11, 13, },
  { 0, 1, 2, 5, 6, 7, 11, 13, },
  { 3, 5, 6, 7, 11, 13, },
  { 0, 3, 5, 6, 7, 11, 13, },
  { 1, 3, 5, 6, 7, 11, 13, },
  { 0, 1, 3, 5, 6, 7, 11, 13, },
  { 2, 3, 5, 6, 7, 11, 13, },
  { 0, 2, 3, 5, 6, 7, 11, 13, },
  { 1, 2, 3, 5, 6, 7, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 13, },
  { 4, 5, 6, 7, 11, 13, },
  { 0, 4, 5, 6, 7, 11, 13, },
  { 1, 4, 5, 6, 7, 11, 13, },
  { 0, 1, 4, 5, 6, 7, 11, 13, },
  { 2, 4, 5, 6, 7, 11, 13, },
  { 0, 2, 4, 5, 6, 7, 11, 13, },
  { 1, 2, 4, 5, 6, 7, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 13, },
  { 3, 4, 5, 6, 7, 11, 13, },
  { 0, 3, 4, 5, 6, 7, 11, 13, },
  { 1, 3, 4, 5, 6, 7, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 13, },
  { 2, 3, 4, 5, 6, 7, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 13, },
  { 8, 11, 13, },
  { 0, 8, 11, 13, },
  { 1, 8, 11, 13, },
  { 0, 1, 8, 11, 13, },
  { 2, 8, 11, 13, },
  { 0, 2, 8, 11, 13, },
  { 1, 2, 8, 11, 13, },
  { 0, 1, 2, 8, 11, 13, },
  { 3, 8, 11, 13, },
  { 0, 3, 8, 11, 13, },
  { 1, 3, 8, 11, 13, },
  { 0, 1, 3, 8, 11, 13, },
  { 2, 3, 8, 11, 13, },
  { 0, 2, 3, 8, 11, 13, },
  { 1, 2, 3, 8, 11, 13, },
  { 0, 1, 2, 3, 8, 11, 13, },
  { 4, 8, 11, 13, },
  { 0, 4, 8, 11, 13, },
  { 1, 4, 8, 11, 13, },
  { 0, 1, 4, 8, 11, 13, },
  { 2, 4, 8, 11, 13, },
  { 0, 2, 4, 8, 11, 13, },
  { 1, 2, 4, 8, 11, 13, },
  { 0, 1, 2, 4, 8, 11, 13, },
  { 3, 4, 8, 11, 13, },
  { 0, 3, 4, 8, 11, 13, },
  { 1, 3, 4, 8, 11, 13, },
  { 0, 1, 3, 4, 8, 11, 13, },
  { 2, 3, 4, 8, 11, 13, },
  { 0, 2, 3, 4, 8, 11, 13, },
  { 1, 2, 3, 4, 8, 11, 13, },
  { 0, 1, 2, 3, 4, 8, 11, 13, },
  { 5, 8, 11, 13, },
  { 0, 5, 8, 11, 13, },
  { 1, 5, 8, 11, 13, },
  { 0, 1, 5, 8, 11, 13, },
  { 2, 5, 8, 11, 13, },
  { 0, 2, 5, 8, 11, 13, },
  { 1, 2, 5, 8, 11, 13, },
  { 0, 1, 2, 5, 8, 11, 13, },
  { 3, 5, 8, 11, 13, },
  { 0, 3, 5, 8, 11, 13, },
  { 1, 3, 5, 8, 11, 13, },
  { 0, 1, 3, 5, 8, 11, 13, },
  { 2, 3, 5, 8, 11, 13, },
  { 0, 2, 3, 5, 8, 11, 13, },
  { 1, 2, 3, 5, 8, 11, 13, },
  { 0, 1, 2, 3, 5, 8, 11, 13, },
  { 4, 5, 8, 11, 13, },
  { 0, 4, 5, 8, 11, 13, },
  { 1, 4, 5, 8, 11, 13, },
  { 0, 1, 4, 5, 8, 11, 13, },
  { 2, 4, 5, 8, 11, 13, },
  { 0, 2, 4, 5, 8, 11, 13, },
  { 1, 2, 4, 5, 8, 11, 13, },
  { 0, 1, 2, 4, 5, 8, 11, 13, },
  { 3, 4, 5, 8, 11, 13, },
  { 0, 3, 4, 5, 8, 11, 13, },
  { 1, 3, 4, 5, 8, 11, 13, },
  { 0, 1, 3, 4, 5, 8, 11, 13, },
  { 2, 3, 4, 5, 8, 11, 13, },
  { 0, 2, 3, 4, 5, 8, 11, 13, },
  { 1, 2, 3, 4, 5, 8, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 13, },
  { 6, 8, 11, 13, },
  { 0, 6, 8, 11, 13, },
  { 1, 6, 8, 11, 13, },
  { 0, 1, 6, 8, 11, 13, },
  { 2, 6, 8, 11, 13, },
  { 0, 2, 6, 8, 11, 13, },
  { 1, 2, 6, 8, 11, 13, },
  { 0, 1, 2, 6, 8, 11, 13, },
  { 3, 6, 8, 11, 13, },
  { 0, 3, 6, 8, 11, 13, },
  { 1, 3, 6, 8, 11, 13, },
  { 0, 1, 3, 6, 8, 11, 13, },
  { 2, 3, 6, 8, 11, 13, },
  { 0, 2, 3, 6, 8, 11, 13, },
  { 1, 2, 3, 6, 8, 11, 13, },
  { 0, 1, 2, 3, 6, 8, 11, 13, },
  { 4, 6, 8, 11, 13, },
  { 0, 4, 6, 8, 11, 13, },
  { 1, 4, 6, 8, 11, 13, },
  { 0, 1, 4, 6, 8, 11, 13, },
  { 2, 4, 6, 8, 11, 13, },
  { 0, 2, 4, 6, 8, 11, 13, },
  { 1, 2, 4, 6, 8, 11, 13, },
  { 0, 1, 2, 4, 6, 8, 11, 13, },
  { 3, 4, 6, 8, 11, 13, },
  { 0, 3, 4, 6, 8, 11, 13, },
  { 1, 3, 4, 6, 8, 11, 13, },
  { 0, 1, 3, 4, 6, 8, 11, 13, },
  { 2, 3, 4, 6, 8, 11, 13, },
  { 0, 2, 3, 4, 6, 8, 11, 13, },
  { 1, 2, 3, 4, 6, 8, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 13, },
  { 5, 6, 8, 11, 13, },
  { 0, 5, 6, 8, 11, 13, },
  { 1, 5, 6, 8, 11, 13, },
  { 0, 1, 5, 6, 8, 11, 13, },
  { 2, 5, 6, 8, 11, 13, },
  { 0, 2, 5, 6, 8, 11, 13, },
  { 1, 2, 5, 6, 8, 11, 13, },
  { 0, 1, 2, 5, 6, 8, 11, 13, },
  { 3, 5, 6, 8, 11, 13, },
  { 0, 3, 5, 6, 8, 11, 13, },
  { 1, 3, 5, 6, 8, 11, 13, },
  { 0, 1, 3, 5, 6, 8, 11, 13, },
  { 2, 3, 5, 6, 8, 11, 13, },
  { 0, 2, 3, 5, 6, 8, 11, 13, },
  { 1, 2, 3, 5, 6, 8, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 13, },
  { 4, 5, 6, 8, 11, 13, },
  { 0, 4, 5, 6, 8, 11, 13, },
  { 1, 4, 5, 6, 8, 11, 13, },
  { 0, 1, 4, 5, 6, 8, 11, 13, },
  { 2, 4, 5, 6, 8, 11, 13, },
  { 0, 2, 4, 5, 6, 8, 11, 13, },
  { 1, 2, 4, 5, 6, 8, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 13, },
  { 3, 4, 5, 6, 8, 11, 13, },
  { 0, 3, 4, 5, 6, 8, 11, 13, },
  { 1, 3, 4, 5, 6, 8, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 13, },
  { 2, 3, 4, 5, 6, 8, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 13, },
  { 7, 8, 11, 13, },
  { 0, 7, 8, 11, 13, },
  { 1, 7, 8, 11, 13, },
  { 0, 1, 7, 8, 11, 13, },
  { 2, 7, 8, 11, 13, },
  { 0, 2, 7, 8, 11, 13, },
  { 1, 2, 7, 8, 11, 13, },
  { 0, 1, 2, 7, 8, 11, 13, },
  { 3, 7, 8, 11, 13, },
  { 0, 3, 7, 8, 11, 13, },
  { 1, 3, 7, 8, 11, 13, },
  { 0, 1, 3, 7, 8, 11, 13, },
  { 2, 3, 7, 8, 11, 13, },
  { 0, 2, 3, 7, 8, 11, 13, },
  { 1, 2, 3, 7, 8, 11, 13, },
  { 0, 1, 2, 3, 7, 8, 11, 13, },
  { 4, 7, 8, 11, 13, },
  { 0, 4, 7, 8, 11, 13, },
  { 1, 4, 7, 8, 11, 13, },
  { 0, 1, 4, 7, 8, 11, 13, },
  { 2, 4, 7, 8, 11, 13, },
  { 0, 2, 4, 7, 8, 11, 13, },
  { 1, 2, 4, 7, 8, 11, 13, },
  { 0, 1, 2, 4, 7, 8, 11, 13, },
  { 3, 4, 7, 8, 11, 13, },
  { 0, 3, 4, 7, 8, 11, 13, },
  { 1, 3, 4, 7, 8, 11, 13, },
  { 0, 1, 3, 4, 7, 8, 11, 13, },
  { 2, 3, 4, 7, 8, 11, 13, },
  { 0, 2, 3, 4, 7, 8, 11, 13, },
  { 1, 2, 3, 4, 7, 8, 11, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 13, },
  { 5, 7, 8, 11, 13, },
  { 0, 5, 7, 8, 11, 13, },
  { 1, 5, 7, 8, 11, 13, },
  { 0, 1, 5, 7, 8, 11, 13, },
  { 2, 5, 7, 8, 11, 13, },
  { 0, 2, 5, 7, 8, 11, 13, },
  { 1, 2, 5, 7, 8, 11, 13, },
  { 0, 1, 2, 5, 7, 8, 11, 13, },
  { 3, 5, 7, 8, 11, 13, },
  { 0, 3, 5, 7, 8, 11, 13, },
  { 1, 3, 5, 7, 8, 11, 13, },
  { 0, 1, 3, 5, 7, 8, 11, 13, },
  { 2, 3, 5, 7, 8, 11, 13, },
  { 0, 2, 3, 5, 7, 8, 11, 13, },
  { 1, 2, 3, 5, 7, 8, 11, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 13, },
  { 4, 5, 7, 8, 11, 13, },
  { 0, 4, 5, 7, 8, 11, 13, },
  { 1, 4, 5, 7, 8, 11, 13, },
  { 0, 1, 4, 5, 7, 8, 11, 13, },
  { 2, 4, 5, 7, 8, 11, 13, },
  { 0, 2, 4, 5, 7, 8, 11, 13, },
  { 1, 2, 4, 5, 7, 8, 11, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 13, },
  { 3, 4, 5, 7, 8, 11, 13, },
  { 0, 3, 4, 5, 7, 8, 11, 13, },
  { 1, 3, 4, 5, 7, 8, 11, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 13, },
  { 2, 3, 4, 5, 7, 8, 11, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 13, },
  { 6, 7, 8, 11, 13, },
  { 0, 6, 7, 8, 11, 13, },
  { 1, 6, 7, 8, 11, 13, },
  { 0, 1, 6, 7, 8, 11, 13, },
  { 2, 6, 7, 8, 11, 13, },
  { 0, 2, 6, 7, 8, 11, 13, },
  { 1, 2, 6, 7, 8, 11, 13, },
  { 0, 1, 2, 6, 7, 8, 11, 13, },
  { 3, 6, 7, 8, 11, 13, },
  { 0, 3, 6, 7, 8, 11, 13, },
  { 1, 3, 6, 7, 8, 11, 13, },
  { 0, 1, 3, 6, 7, 8, 11, 13, },
  { 2, 3, 6, 7, 8, 11, 13, },
  { 0, 2, 3, 6, 7, 8, 11, 13, },
  { 1, 2, 3, 6, 7, 8, 11, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 13, },
  { 4, 6, 7, 8, 11, 13, },
  { 0, 4, 6, 7, 8, 11, 13, },
  { 1, 4, 6, 7, 8, 11, 13, },
  { 0, 1, 4, 6, 7, 8, 11, 13, },
  { 2, 4, 6, 7, 8, 11, 13, },
  { 0, 2, 4, 6, 7, 8, 11, 13, },
  { 1, 2, 4, 6, 7, 8, 11, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 13, },
  { 3, 4, 6, 7, 8, 11, 13, },
  { 0, 3, 4, 6, 7, 8, 11, 13, },
  { 1, 3, 4, 6, 7, 8, 11, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 13, },
  { 2, 3, 4, 6, 7, 8, 11, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 13, },
  { 5, 6, 7, 8, 11, 13, },
  { 0, 5, 6, 7, 8, 11, 13, },
  { 1, 5, 6, 7, 8, 11, 13, },
  { 0, 1, 5, 6, 7, 8, 11, 13, },
  { 2, 5, 6, 7, 8, 11, 13, },
  { 0, 2, 5, 6, 7, 8, 11, 13, },
  { 1, 2, 5, 6, 7, 8, 11, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 13, },
  { 3, 5, 6, 7, 8, 11, 13, },
  { 0, 3, 5, 6, 7, 8, 11, 13, },
  { 1, 3, 5, 6, 7, 8, 11, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 13, },
  { 2, 3, 5, 6, 7, 8, 11, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 13, },
  { 4, 5, 6, 7, 8, 11, 13, },
  { 0, 4, 5, 6, 7, 8, 11, 13, },
  { 1, 4, 5, 6, 7, 8, 11, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 13, },
  { 2, 4, 5, 6, 7, 8, 11, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 13, },
  { 3, 4, 5, 6, 7, 8, 11, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, },
  { 9, 11, 13, },
  { 0, 9, 11, 13, },
  { 1, 9, 11, 13, },
  { 0, 1, 9, 11, 13, },
  { 2, 9, 11, 13, },
  { 0, 2, 9, 11, 13, },
  { 1, 2, 9, 11, 13, },
  { 0, 1, 2, 9, 11, 13, },
  { 3, 9, 11, 13, },
  { 0, 3, 9, 11, 13, },
  { 1, 3, 9, 11, 13, },
  { 0, 1, 3, 9, 11, 13, },
  { 2, 3, 9, 11, 13, },
  { 0, 2, 3, 9, 11, 13, },
  { 1, 2, 3, 9, 11, 13, },
  { 0, 1, 2, 3, 9, 11, 13, },
  { 4, 9, 11, 13, },
  { 0, 4, 9, 11, 13, },
  { 1, 4, 9, 11, 13, },
  { 0, 1, 4, 9, 11, 13, },
  { 2, 4, 9, 11, 13, },
  { 0, 2, 4, 9, 11, 13, },
  { 1, 2, 4, 9, 11, 13, },
  { 0, 1, 2, 4, 9, 11, 13, },
  { 3, 4, 9, 11, 13, },
  { 0, 3, 4, 9, 11, 13, },
  { 1, 3, 4, 9, 11, 13, },
  { 0, 1, 3, 4, 9, 11, 13, },
  { 2, 3, 4, 9, 11, 13, },
  { 0, 2, 3, 4, 9, 11, 13, },
  { 1, 2, 3, 4, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 9, 11, 13, },
  { 5, 9, 11, 13, },
  { 0, 5, 9, 11, 13, },
  { 1, 5, 9, 11, 13, },
  { 0, 1, 5, 9, 11, 13, },
  { 2, 5, 9, 11, 13, },
  { 0, 2, 5, 9, 11, 13, },
  { 1, 2, 5, 9, 11, 13, },
  { 0, 1, 2, 5, 9, 11, 13, },
  { 3, 5, 9, 11, 13, },
  { 0, 3, 5, 9, 11, 13, },
  { 1, 3, 5, 9, 11, 13, },
  { 0, 1, 3, 5, 9, 11, 13, },
  { 2, 3, 5, 9, 11, 13, },
  { 0, 2, 3, 5, 9, 11, 13, },
  { 1, 2, 3, 5, 9, 11, 13, },
  { 0, 1, 2, 3, 5, 9, 11, 13, },
  { 4, 5, 9, 11, 13, },
  { 0, 4, 5, 9, 11, 13, },
  { 1, 4, 5, 9, 11, 13, },
  { 0, 1, 4, 5, 9, 11, 13, },
  { 2, 4, 5, 9, 11, 13, },
  { 0, 2, 4, 5, 9, 11, 13, },
  { 1, 2, 4, 5, 9, 11, 13, },
  { 0, 1, 2, 4, 5, 9, 11, 13, },
  { 3, 4, 5, 9, 11, 13, },
  { 0, 3, 4, 5, 9, 11, 13, },
  { 1, 3, 4, 5, 9, 11, 13, },
  { 0, 1, 3, 4, 5, 9, 11, 13, },
  { 2, 3, 4, 5, 9, 11, 13, },
  { 0, 2, 3, 4, 5, 9, 11, 13, },
  { 1, 2, 3, 4, 5, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 13, },
  { 6, 9, 11, 13, },
  { 0, 6, 9, 11, 13, },
  { 1, 6, 9, 11, 13, },
  { 0, 1, 6, 9, 11, 13, },
  { 2, 6, 9, 11, 13, },
  { 0, 2, 6, 9, 11, 13, },
  { 1, 2, 6, 9, 11, 13, },
  { 0, 1, 2, 6, 9, 11, 13, },
  { 3, 6, 9, 11, 13, },
  { 0, 3, 6, 9, 11, 13, },
  { 1, 3, 6, 9, 11, 13, },
  { 0, 1, 3, 6, 9, 11, 13, },
  { 2, 3, 6, 9, 11, 13, },
  { 0, 2, 3, 6, 9, 11, 13, },
  { 1, 2, 3, 6, 9, 11, 13, },
  { 0, 1, 2, 3, 6, 9, 11, 13, },
  { 4, 6, 9, 11, 13, },
  { 0, 4, 6, 9, 11, 13, },
  { 1, 4, 6, 9, 11, 13, },
  { 0, 1, 4, 6, 9, 11, 13, },
  { 2, 4, 6, 9, 11, 13, },
  { 0, 2, 4, 6, 9, 11, 13, },
  { 1, 2, 4, 6, 9, 11, 13, },
  { 0, 1, 2, 4, 6, 9, 11, 13, },
  { 3, 4, 6, 9, 11, 13, },
  { 0, 3, 4, 6, 9, 11, 13, },
  { 1, 3, 4, 6, 9, 11, 13, },
  { 0, 1, 3, 4, 6, 9, 11, 13, },
  { 2, 3, 4, 6, 9, 11, 13, },
  { 0, 2, 3, 4, 6, 9, 11, 13, },
  { 1, 2, 3, 4, 6, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 13, },
  { 5, 6, 9, 11, 13, },
  { 0, 5, 6, 9, 11, 13, },
  { 1, 5, 6, 9, 11, 13, },
  { 0, 1, 5, 6, 9, 11, 13, },
  { 2, 5, 6, 9, 11, 13, },
  { 0, 2, 5, 6, 9, 11, 13, },
  { 1, 2, 5, 6, 9, 11, 13, },
  { 0, 1, 2, 5, 6, 9, 11, 13, },
  { 3, 5, 6, 9, 11, 13, },
  { 0, 3, 5, 6, 9, 11, 13, },
  { 1, 3, 5, 6, 9, 11, 13, },
  { 0, 1, 3, 5, 6, 9, 11, 13, },
  { 2, 3, 5, 6, 9, 11, 13, },
  { 0, 2, 3, 5, 6, 9, 11, 13, },
  { 1, 2, 3, 5, 6, 9, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 13, },
  { 4, 5, 6, 9, 11, 13, },
  { 0, 4, 5, 6, 9, 11, 13, },
  { 1, 4, 5, 6, 9, 11, 13, },
  { 0, 1, 4, 5, 6, 9, 11, 13, },
  { 2, 4, 5, 6, 9, 11, 13, },
  { 0, 2, 4, 5, 6, 9, 11, 13, },
  { 1, 2, 4, 5, 6, 9, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 13, },
  { 3, 4, 5, 6, 9, 11, 13, },
  { 0, 3, 4, 5, 6, 9, 11, 13, },
  { 1, 3, 4, 5, 6, 9, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 13, },
  { 2, 3, 4, 5, 6, 9, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 13, },
  { 7, 9, 11, 13, },
  { 0, 7, 9, 11, 13, },
  { 1, 7, 9, 11, 13, },
  { 0, 1, 7, 9, 11, 13, },
  { 2, 7, 9, 11, 13, },
  { 0, 2, 7, 9, 11, 13, },
  { 1, 2, 7, 9, 11, 13, },
  { 0, 1, 2, 7, 9, 11, 13, },
  { 3, 7, 9, 11, 13, },
  { 0, 3, 7, 9, 11, 13, },
  { 1, 3, 7, 9, 11, 13, },
  { 0, 1, 3, 7, 9, 11, 13, },
  { 2, 3, 7, 9, 11, 13, },
  { 0, 2, 3, 7, 9, 11, 13, },
  { 1, 2, 3, 7, 9, 11, 13, },
  { 0, 1, 2, 3, 7, 9, 11, 13, },
  { 4, 7, 9, 11, 13, },
  { 0, 4, 7, 9, 11, 13, },
  { 1, 4, 7, 9, 11, 13, },
  { 0, 1, 4, 7, 9, 11, 13, },
  { 2, 4, 7, 9, 11, 13, },
  { 0, 2, 4, 7, 9, 11, 13, },
  { 1, 2, 4, 7, 9, 11, 13, },
  { 0, 1, 2, 4, 7, 9, 11, 13, },
  { 3, 4, 7, 9, 11, 13, },
  { 0, 3, 4, 7, 9, 11, 13, },
  { 1, 3, 4, 7, 9, 11, 13, },
  { 0, 1, 3, 4, 7, 9, 11, 13, },
  { 2, 3, 4, 7, 9, 11, 13, },
  { 0, 2, 3, 4, 7, 9, 11, 13, },
  { 1, 2, 3, 4, 7, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 13, },
  { 5, 7, 9, 11, 13, },
  { 0, 5, 7, 9, 11, 13, },
  { 1, 5, 7, 9, 11, 13, },
  { 0, 1, 5, 7, 9, 11, 13, },
  { 2, 5, 7, 9, 11, 13, },
  { 0, 2, 5, 7, 9, 11, 13, },
  { 1, 2, 5, 7, 9, 11, 13, },
  { 0, 1, 2, 5, 7, 9, 11, 13, },
  { 3, 5, 7, 9, 11, 13, },
  { 0, 3, 5, 7, 9, 11, 13, },
  { 1, 3, 5, 7, 9, 11, 13, },
  { 0, 1, 3, 5, 7, 9, 11, 13, },
  { 2, 3, 5, 7, 9, 11, 13, },
  { 0, 2, 3, 5, 7, 9, 11, 13, },
  { 1, 2, 3, 5, 7, 9, 11, 13, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 13, },
  { 4, 5, 7, 9, 11, 13, },
  { 0, 4, 5, 7, 9, 11, 13, },
  { 1, 4, 5, 7, 9, 11, 13, },
  { 0, 1, 4, 5, 7, 9, 11, 13, },
  { 2, 4, 5, 7, 9, 11, 13, },
  { 0, 2, 4, 5, 7, 9, 11, 13, },
  { 1, 2, 4, 5, 7, 9, 11, 13, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 13, },
  { 3, 4, 5, 7, 9, 11, 13, },
  { 0, 3, 4, 5, 7, 9, 11, 13, },
  { 1, 3, 4, 5, 7, 9, 11, 13, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 13, },
  { 2, 3, 4, 5, 7, 9, 11, 13, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 13, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 13, },
  { 6, 7, 9, 11, 13, },
  { 0, 6, 7, 9, 11, 13, },
  { 1, 6, 7, 9, 11, 13, },
  { 0, 1, 6, 7, 9, 11, 13, },
  { 2, 6, 7, 9, 11, 13, },
  { 0, 2, 6, 7, 9, 11, 13, },
  { 1, 2, 6, 7, 9, 11, 13, },
  { 0, 1, 2, 6, 7, 9, 11, 13, },
  { 3, 6, 7, 9, 11, 13, },
  { 0, 3, 6, 7, 9, 11, 13, },
  { 1, 3, 6, 7, 9, 11, 13, },
  { 0, 1, 3, 6, 7, 9, 11, 13, },
  { 2, 3, 6, 7, 9, 11, 13, },
  { 0, 2, 3, 6, 7, 9, 11, 13, },
  { 1, 2, 3, 6, 7, 9, 11, 13, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 13, },
  { 4, 6, 7, 9, 11, 13, },
  { 0, 4, 6, 7, 9, 11, 13, },
  { 1, 4, 6, 7, 9, 11, 13, },
  { 0, 1, 4, 6, 7, 9, 11, 13, },
  { 2, 4, 6, 7, 9, 11, 13, },
  { 0, 2, 4, 6, 7, 9, 11, 13, },
  { 1, 2, 4, 6, 7, 9, 11, 13, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 13, },
  { 3, 4, 6, 7, 9, 11, 13, },
  { 0, 3, 4, 6, 7, 9, 11, 13, },
  { 1, 3, 4, 6, 7, 9, 11, 13, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 13, },
  { 2, 3, 4, 6, 7, 9, 11, 13, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 13, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 13, },
  { 5, 6, 7, 9, 11, 13, },
  { 0, 5, 6, 7, 9, 11, 13, },
  { 1, 5, 6, 7, 9, 11, 13, },
  { 0, 1, 5, 6, 7, 9, 11, 13, },
  { 2, 5, 6, 7, 9, 11, 13, },
  { 0, 2, 5, 6, 7, 9, 11, 13, },
  { 1, 2, 5, 6, 7, 9, 11, 13, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 13, },
  { 3, 5, 6, 7, 9, 11, 13, },
  { 0, 3, 5, 6, 7, 9, 11, 13, },
  { 1, 3, 5, 6, 7, 9, 11, 13, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 13, },
  { 2, 3, 5, 6, 7, 9, 11, 13, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 13, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, },
  { 4, 5, 6, 7, 9, 11, 13, },
  { 0, 4, 5, 6, 7, 9, 11, 13, },
  { 1, 4, 5, 6, 7, 9, 11, 13, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 13, },
  { 2, 4, 5, 6, 7, 9, 11, 13, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 13, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 13, },
  { 3, 4, 5, 6, 7, 9, 11, 13, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 13, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 13, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, },
  { 8, 9, 11, 13, },
  { 0, 8, 9, 11, 13, },
  { 1, 8, 9, 11, 13, },
  { 0, 1, 8, 9, 11, 13, },
  { 2, 8, 9, 11, 13, },
  { 0, 2, 8, 9, 11, 13, },
  { 1, 2, 8, 9, 11, 13, },
  { 0, 1, 2, 8, 9, 11, 13, },
  { 3, 8, 9, 11, 13, },
  { 0, 3, 8, 9, 11, 13, },
  { 1, 3, 8, 9, 11, 13, },
  { 0, 1, 3, 8, 9, 11, 13, },
  { 2, 3, 8, 9, 11, 13, },
  { 0, 2, 3, 8, 9, 11, 13, },
  { 1, 2, 3, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 8, 9, 11, 13, },
  { 4, 8, 9, 11, 13, },
  { 0, 4, 8, 9, 11, 13, },
  { 1, 4, 8, 9, 11, 13, },
  { 0, 1, 4, 8, 9, 11, 13, },
  { 2, 4, 8, 9, 11, 13, },
  { 0, 2, 4, 8, 9, 11, 13, },
  { 1, 2, 4, 8, 9, 11, 13, },
  { 0, 1, 2, 4, 8, 9, 11, 13, },
  { 3, 4, 8, 9, 11, 13, },
  { 0, 3, 4, 8, 9, 11, 13, },
  { 1, 3, 4, 8, 9, 11, 13, },
  { 0, 1, 3, 4, 8, 9, 11, 13, },
  { 2, 3, 4, 8, 9, 11, 13, },
  { 0, 2, 3, 4, 8, 9, 11, 13, },
  { 1, 2, 3, 4, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 13, },
  { 5, 8, 9, 11, 13, },
  { 0, 5, 8, 9, 11, 13, },
  { 1, 5, 8, 9, 11, 13, },
  { 0, 1, 5, 8, 9, 11, 13, },
  { 2, 5, 8, 9, 11, 13, },
  { 0, 2, 5, 8, 9, 11, 13, },
  { 1, 2, 5, 8, 9, 11, 13, },
  { 0, 1, 2, 5, 8, 9, 11, 13, },
  { 3, 5, 8, 9, 11, 13, },
  { 0, 3, 5, 8, 9, 11, 13, },
  { 1, 3, 5, 8, 9, 11, 13, },
  { 0, 1, 3, 5, 8, 9, 11, 13, },
  { 2, 3, 5, 8, 9, 11, 13, },
  { 0, 2, 3, 5, 8, 9, 11, 13, },
  { 1, 2, 3, 5, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 13, },
  { 4, 5, 8, 9, 11, 13, },
  { 0, 4, 5, 8, 9, 11, 13, },
  { 1, 4, 5, 8, 9, 11, 13, },
  { 0, 1, 4, 5, 8, 9, 11, 13, },
  { 2, 4, 5, 8, 9, 11, 13, },
  { 0, 2, 4, 5, 8, 9, 11, 13, },
  { 1, 2, 4, 5, 8, 9, 11, 13, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 13, },
  { 3, 4, 5, 8, 9, 11, 13, },
  { 0, 3, 4, 5, 8, 9, 11, 13, },
  { 1, 3, 4, 5, 8, 9, 11, 13, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 13, },
  { 2, 3, 4, 5, 8, 9, 11, 13, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 13, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 13, },
  { 6, 8, 9, 11, 13, },
  { 0, 6, 8, 9, 11, 13, },
  { 1, 6, 8, 9, 11, 13, },
  { 0, 1, 6, 8, 9, 11, 13, },
  { 2, 6, 8, 9, 11, 13, },
  { 0, 2, 6, 8, 9, 11, 13, },
  { 1, 2, 6, 8, 9, 11, 13, },
  { 0, 1, 2, 6, 8, 9, 11, 13, },
  { 3, 6, 8, 9, 11, 13, },
  { 0, 3, 6, 8, 9, 11, 13, },
  { 1, 3, 6, 8, 9, 11, 13, },
  { 0, 1, 3, 6, 8, 9, 11, 13, },
  { 2, 3, 6, 8, 9, 11, 13, },
  { 0, 2, 3, 6, 8, 9, 11, 13, },
  { 1, 2, 3, 6, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 13, },
  { 4, 6, 8, 9, 11, 13, },
  { 0, 4, 6, 8, 9, 11, 13, },
  { 1, 4, 6, 8, 9, 11, 13, },
  { 0, 1, 4, 6, 8, 9, 11, 13, },
  { 2, 4, 6, 8, 9, 11, 13, },
  { 0, 2, 4, 6, 8, 9, 11, 13, },
  { 1, 2, 4, 6, 8, 9, 11, 13, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 13, },
  { 3, 4, 6, 8, 9, 11, 13, },
  { 0, 3, 4, 6, 8, 9, 11, 13, },
  { 1, 3, 4, 6, 8, 9, 11, 13, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 13, },
  { 2, 3, 4, 6, 8, 9, 11, 13, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 13, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 13, },
  { 5, 6, 8, 9, 11, 13, },
  { 0, 5, 6, 8, 9, 11, 13, },
  { 1, 5, 6, 8, 9, 11, 13, },
  { 0, 1, 5, 6, 8, 9, 11, 13, },
  { 2, 5, 6, 8, 9, 11, 13, },
  { 0, 2, 5, 6, 8, 9, 11, 13, },
  { 1, 2, 5, 6, 8, 9, 11, 13, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 13, },
  { 3, 5, 6, 8, 9, 11, 13, },
  { 0, 3, 5, 6, 8, 9, 11, 13, },
  { 1, 3, 5, 6, 8, 9, 11, 13, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 13, },
  { 2, 3, 5, 6, 8, 9, 11, 13, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 13, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 13, },
  { 4, 5, 6, 8, 9, 11, 13, },
  { 0, 4, 5, 6, 8, 9, 11, 13, },
  { 1, 4, 5, 6, 8, 9, 11, 13, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 13, },
  { 2, 4, 5, 6, 8, 9, 11, 13, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 13, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 13, },
  { 3, 4, 5, 6, 8, 9, 11, 13, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 13, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 13, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 13, },
  { 7, 8, 9, 11, 13, },
  { 0, 7, 8, 9, 11, 13, },
  { 1, 7, 8, 9, 11, 13, },
  { 0, 1, 7, 8, 9, 11, 13, },
  { 2, 7, 8, 9, 11, 13, },
  { 0, 2, 7, 8, 9, 11, 13, },
  { 1, 2, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 7, 8, 9, 11, 13, },
  { 3, 7, 8, 9, 11, 13, },
  { 0, 3, 7, 8, 9, 11, 13, },
  { 1, 3, 7, 8, 9, 11, 13, },
  { 0, 1, 3, 7, 8, 9, 11, 13, },
  { 2, 3, 7, 8, 9, 11, 13, },
  { 0, 2, 3, 7, 8, 9, 11, 13, },
  { 1, 2, 3, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 13, },
  { 4, 7, 8, 9, 11, 13, },
  { 0, 4, 7, 8, 9, 11, 13, },
  { 1, 4, 7, 8, 9, 11, 13, },
  { 0, 1, 4, 7, 8, 9, 11, 13, },
  { 2, 4, 7, 8, 9, 11, 13, },
  { 0, 2, 4, 7, 8, 9, 11, 13, },
  { 1, 2, 4, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 13, },
  { 3, 4, 7, 8, 9, 11, 13, },
  { 0, 3, 4, 7, 8, 9, 11, 13, },
  { 1, 3, 4, 7, 8, 9, 11, 13, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 13, },
  { 2, 3, 4, 7, 8, 9, 11, 13, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 13, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 13, },
  { 5, 7, 8, 9, 11, 13, },
  { 0, 5, 7, 8, 9, 11, 13, },
  { 1, 5, 7, 8, 9, 11, 13, },
  { 0, 1, 5, 7, 8, 9, 11, 13, },
  { 2, 5, 7, 8, 9, 11, 13, },
  { 0, 2, 5, 7, 8, 9, 11, 13, },
  { 1, 2, 5, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 13, },
  { 3, 5, 7, 8, 9, 11, 13, },
  { 0, 3, 5, 7, 8, 9, 11, 13, },
  { 1, 3, 5, 7, 8, 9, 11, 13, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 13, },
  { 2, 3, 5, 7, 8, 9, 11, 13, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 13, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 13, },
  { 4, 5, 7, 8, 9, 11, 13, },
  { 0, 4, 5, 7, 8, 9, 11, 13, },
  { 1, 4, 5, 7, 8, 9, 11, 13, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 13, },
  { 2, 4, 5, 7, 8, 9, 11, 13, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 13, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 13, },
  { 3, 4, 5, 7, 8, 9, 11, 13, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 13, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 13, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 13, },
  { 6, 7, 8, 9, 11, 13, },
  { 0, 6, 7, 8, 9, 11, 13, },
  { 1, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 6, 7, 8, 9, 11, 13, },
  { 2, 6, 7, 8, 9, 11, 13, },
  { 0, 2, 6, 7, 8, 9, 11, 13, },
  { 1, 2, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 13, },
  { 3, 6, 7, 8, 9, 11, 13, },
  { 0, 3, 6, 7, 8, 9, 11, 13, },
  { 1, 3, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 13, },
  { 2, 3, 6, 7, 8, 9, 11, 13, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 13, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 13, },
  { 4, 6, 7, 8, 9, 11, 13, },
  { 0, 4, 6, 7, 8, 9, 11, 13, },
  { 1, 4, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 13, },
  { 2, 4, 6, 7, 8, 9, 11, 13, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 13, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 13, },
  { 3, 4, 6, 7, 8, 9, 11, 13, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 13, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 13, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 13, },
  { 5, 6, 7, 8, 9, 11, 13, },
  { 0, 5, 6, 7, 8, 9, 11, 13, },
  { 1, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 13, },
  { 2, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 13, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 13, },
  { 3, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 13, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 13, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 13, },
  { 4, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, },
  { 10, 11, 13, },
  { 0, 10, 11, 13, },
  { 1, 10, 11, 13, },
  { 0, 1, 10, 11, 13, },
  { 2, 10, 11, 13, },
  { 0, 2, 10, 11, 13, },
  { 1, 2, 10, 11, 13, },
  { 0, 1, 2, 10, 11, 13, },
  { 3, 10, 11, 13, },
  { 0, 3, 10, 11, 13, },
  { 1, 3, 10, 11, 13, },
  { 0, 1, 3, 10, 11, 13, },
  { 2, 3, 10, 11, 13, },
  { 0, 2, 3, 10, 11, 13, },
  { 1, 2, 3, 10, 11, 13, },
  { 0, 1, 2, 3, 10, 11, 13, },
  { 4, 10, 11, 13, },
  { 0, 4, 10, 11, 13, },
  { 1, 4, 10, 11, 13, },
  { 0, 1, 4, 10, 11, 13, },
  { 2, 4, 10, 11, 13, },
  { 0, 2, 4, 10, 11, 13, },
  { 1, 2, 4, 10, 11, 13, },
  { 0, 1, 2, 4, 10, 11, 13, },
  { 3, 4, 10, 11, 13, },
  { 0, 3, 4, 10, 11, 13, },
  { 1, 3, 4, 10, 11, 13, },
  { 0, 1, 3, 4, 10, 11, 13, },
  { 2, 3, 4, 10, 11, 13, },
  { 0, 2, 3, 4, 10, 11, 13, },
  { 1, 2, 3, 4, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 10, 11, 13, },
  { 5, 10, 11, 13, },
  { 0, 5, 10, 11, 13, },
  { 1, 5, 10, 11, 13, },
  { 0, 1, 5, 10, 11, 13, },
  { 2, 5, 10, 11, 13, },
  { 0, 2, 5, 10, 11, 13, },
  { 1, 2, 5, 10, 11, 13, },
  { 0, 1, 2, 5, 10, 11, 13, },
  { 3, 5, 10, 11, 13, },
  { 0, 3, 5, 10, 11, 13, },
  { 1, 3, 5, 10, 11, 13, },
  { 0, 1, 3, 5, 10, 11, 13, },
  { 2, 3, 5, 10, 11, 13, },
  { 0, 2, 3, 5, 10, 11, 13, },
  { 1, 2, 3, 5, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 10, 11, 13, },
  { 4, 5, 10, 11, 13, },
  { 0, 4, 5, 10, 11, 13, },
  { 1, 4, 5, 10, 11, 13, },
  { 0, 1, 4, 5, 10, 11, 13, },
  { 2, 4, 5, 10, 11, 13, },
  { 0, 2, 4, 5, 10, 11, 13, },
  { 1, 2, 4, 5, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 10, 11, 13, },
  { 3, 4, 5, 10, 11, 13, },
  { 0, 3, 4, 5, 10, 11, 13, },
  { 1, 3, 4, 5, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 10, 11, 13, },
  { 2, 3, 4, 5, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 13, },
  { 6, 10, 11, 13, },
  { 0, 6, 10, 11, 13, },
  { 1, 6, 10, 11, 13, },
  { 0, 1, 6, 10, 11, 13, },
  { 2, 6, 10, 11, 13, },
  { 0, 2, 6, 10, 11, 13, },
  { 1, 2, 6, 10, 11, 13, },
  { 0, 1, 2, 6, 10, 11, 13, },
  { 3, 6, 10, 11, 13, },
  { 0, 3, 6, 10, 11, 13, },
  { 1, 3, 6, 10, 11, 13, },
  { 0, 1, 3, 6, 10, 11, 13, },
  { 2, 3, 6, 10, 11, 13, },
  { 0, 2, 3, 6, 10, 11, 13, },
  { 1, 2, 3, 6, 10, 11, 13, },
  { 0, 1, 2, 3, 6, 10, 11, 13, },
  { 4, 6, 10, 11, 13, },
  { 0, 4, 6, 10, 11, 13, },
  { 1, 4, 6, 10, 11, 13, },
  { 0, 1, 4, 6, 10, 11, 13, },
  { 2, 4, 6, 10, 11, 13, },
  { 0, 2, 4, 6, 10, 11, 13, },
  { 1, 2, 4, 6, 10, 11, 13, },
  { 0, 1, 2, 4, 6, 10, 11, 13, },
  { 3, 4, 6, 10, 11, 13, },
  { 0, 3, 4, 6, 10, 11, 13, },
  { 1, 3, 4, 6, 10, 11, 13, },
  { 0, 1, 3, 4, 6, 10, 11, 13, },
  { 2, 3, 4, 6, 10, 11, 13, },
  { 0, 2, 3, 4, 6, 10, 11, 13, },
  { 1, 2, 3, 4, 6, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 13, },
  { 5, 6, 10, 11, 13, },
  { 0, 5, 6, 10, 11, 13, },
  { 1, 5, 6, 10, 11, 13, },
  { 0, 1, 5, 6, 10, 11, 13, },
  { 2, 5, 6, 10, 11, 13, },
  { 0, 2, 5, 6, 10, 11, 13, },
  { 1, 2, 5, 6, 10, 11, 13, },
  { 0, 1, 2, 5, 6, 10, 11, 13, },
  { 3, 5, 6, 10, 11, 13, },
  { 0, 3, 5, 6, 10, 11, 13, },
  { 1, 3, 5, 6, 10, 11, 13, },
  { 0, 1, 3, 5, 6, 10, 11, 13, },
  { 2, 3, 5, 6, 10, 11, 13, },
  { 0, 2, 3, 5, 6, 10, 11, 13, },
  { 1, 2, 3, 5, 6, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 13, },
  { 4, 5, 6, 10, 11, 13, },
  { 0, 4, 5, 6, 10, 11, 13, },
  { 1, 4, 5, 6, 10, 11, 13, },
  { 0, 1, 4, 5, 6, 10, 11, 13, },
  { 2, 4, 5, 6, 10, 11, 13, },
  { 0, 2, 4, 5, 6, 10, 11, 13, },
  { 1, 2, 4, 5, 6, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 13, },
  { 3, 4, 5, 6, 10, 11, 13, },
  { 0, 3, 4, 5, 6, 10, 11, 13, },
  { 1, 3, 4, 5, 6, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 13, },
  { 2, 3, 4, 5, 6, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 13, },
  { 7, 10, 11, 13, },
  { 0, 7, 10, 11, 13, },
  { 1, 7, 10, 11, 13, },
  { 0, 1, 7, 10, 11, 13, },
  { 2, 7, 10, 11, 13, },
  { 0, 2, 7, 10, 11, 13, },
  { 1, 2, 7, 10, 11, 13, },
  { 0, 1, 2, 7, 10, 11, 13, },
  { 3, 7, 10, 11, 13, },
  { 0, 3, 7, 10, 11, 13, },
  { 1, 3, 7, 10, 11, 13, },
  { 0, 1, 3, 7, 10, 11, 13, },
  { 2, 3, 7, 10, 11, 13, },
  { 0, 2, 3, 7, 10, 11, 13, },
  { 1, 2, 3, 7, 10, 11, 13, },
  { 0, 1, 2, 3, 7, 10, 11, 13, },
  { 4, 7, 10, 11, 13, },
  { 0, 4, 7, 10, 11, 13, },
  { 1, 4, 7, 10, 11, 13, },
  { 0, 1, 4, 7, 10, 11, 13, },
  { 2, 4, 7, 10, 11, 13, },
  { 0, 2, 4, 7, 10, 11, 13, },
  { 1, 2, 4, 7, 10, 11, 13, },
  { 0, 1, 2, 4, 7, 10, 11, 13, },
  { 3, 4, 7, 10, 11, 13, },
  { 0, 3, 4, 7, 10, 11, 13, },
  { 1, 3, 4, 7, 10, 11, 13, },
  { 0, 1, 3, 4, 7, 10, 11, 13, },
  { 2, 3, 4, 7, 10, 11, 13, },
  { 0, 2, 3, 4, 7, 10, 11, 13, },
  { 1, 2, 3, 4, 7, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 13, },
  { 5, 7, 10, 11, 13, },
  { 0, 5, 7, 10, 11, 13, },
  { 1, 5, 7, 10, 11, 13, },
  { 0, 1, 5, 7, 10, 11, 13, },
  { 2, 5, 7, 10, 11, 13, },
  { 0, 2, 5, 7, 10, 11, 13, },
  { 1, 2, 5, 7, 10, 11, 13, },
  { 0, 1, 2, 5, 7, 10, 11, 13, },
  { 3, 5, 7, 10, 11, 13, },
  { 0, 3, 5, 7, 10, 11, 13, },
  { 1, 3, 5, 7, 10, 11, 13, },
  { 0, 1, 3, 5, 7, 10, 11, 13, },
  { 2, 3, 5, 7, 10, 11, 13, },
  { 0, 2, 3, 5, 7, 10, 11, 13, },
  { 1, 2, 3, 5, 7, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 13, },
  { 4, 5, 7, 10, 11, 13, },
  { 0, 4, 5, 7, 10, 11, 13, },
  { 1, 4, 5, 7, 10, 11, 13, },
  { 0, 1, 4, 5, 7, 10, 11, 13, },
  { 2, 4, 5, 7, 10, 11, 13, },
  { 0, 2, 4, 5, 7, 10, 11, 13, },
  { 1, 2, 4, 5, 7, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 13, },
  { 3, 4, 5, 7, 10, 11, 13, },
  { 0, 3, 4, 5, 7, 10, 11, 13, },
  { 1, 3, 4, 5, 7, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 13, },
  { 2, 3, 4, 5, 7, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 13, },
  { 6, 7, 10, 11, 13, },
  { 0, 6, 7, 10, 11, 13, },
  { 1, 6, 7, 10, 11, 13, },
  { 0, 1, 6, 7, 10, 11, 13, },
  { 2, 6, 7, 10, 11, 13, },
  { 0, 2, 6, 7, 10, 11, 13, },
  { 1, 2, 6, 7, 10, 11, 13, },
  { 0, 1, 2, 6, 7, 10, 11, 13, },
  { 3, 6, 7, 10, 11, 13, },
  { 0, 3, 6, 7, 10, 11, 13, },
  { 1, 3, 6, 7, 10, 11, 13, },
  { 0, 1, 3, 6, 7, 10, 11, 13, },
  { 2, 3, 6, 7, 10, 11, 13, },
  { 0, 2, 3, 6, 7, 10, 11, 13, },
  { 1, 2, 3, 6, 7, 10, 11, 13, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 13, },
  { 4, 6, 7, 10, 11, 13, },
  { 0, 4, 6, 7, 10, 11, 13, },
  { 1, 4, 6, 7, 10, 11, 13, },
  { 0, 1, 4, 6, 7, 10, 11, 13, },
  { 2, 4, 6, 7, 10, 11, 13, },
  { 0, 2, 4, 6, 7, 10, 11, 13, },
  { 1, 2, 4, 6, 7, 10, 11, 13, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 13, },
  { 3, 4, 6, 7, 10, 11, 13, },
  { 0, 3, 4, 6, 7, 10, 11, 13, },
  { 1, 3, 4, 6, 7, 10, 11, 13, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 13, },
  { 2, 3, 4, 6, 7, 10, 11, 13, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 13, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 13, },
  { 5, 6, 7, 10, 11, 13, },
  { 0, 5, 6, 7, 10, 11, 13, },
  { 1, 5, 6, 7, 10, 11, 13, },
  { 0, 1, 5, 6, 7, 10, 11, 13, },
  { 2, 5, 6, 7, 10, 11, 13, },
  { 0, 2, 5, 6, 7, 10, 11, 13, },
  { 1, 2, 5, 6, 7, 10, 11, 13, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 13, },
  { 3, 5, 6, 7, 10, 11, 13, },
  { 0, 3, 5, 6, 7, 10, 11, 13, },
  { 1, 3, 5, 6, 7, 10, 11, 13, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 13, },
  { 2, 3, 5, 6, 7, 10, 11, 13, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 13, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 13, },
  { 4, 5, 6, 7, 10, 11, 13, },
  { 0, 4, 5, 6, 7, 10, 11, 13, },
  { 1, 4, 5, 6, 7, 10, 11, 13, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 13, },
  { 2, 4, 5, 6, 7, 10, 11, 13, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 13, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 13, },
  { 3, 4, 5, 6, 7, 10, 11, 13, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 13, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 13, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 13, },
  { 8, 10, 11, 13, },
  { 0, 8, 10, 11, 13, },
  { 1, 8, 10, 11, 13, },
  { 0, 1, 8, 10, 11, 13, },
  { 2, 8, 10, 11, 13, },
  { 0, 2, 8, 10, 11, 13, },
  { 1, 2, 8, 10, 11, 13, },
  { 0, 1, 2, 8, 10, 11, 13, },
  { 3, 8, 10, 11, 13, },
  { 0, 3, 8, 10, 11, 13, },
  { 1, 3, 8, 10, 11, 13, },
  { 0, 1, 3, 8, 10, 11, 13, },
  { 2, 3, 8, 10, 11, 13, },
  { 0, 2, 3, 8, 10, 11, 13, },
  { 1, 2, 3, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 8, 10, 11, 13, },
  { 4, 8, 10, 11, 13, },
  { 0, 4, 8, 10, 11, 13, },
  { 1, 4, 8, 10, 11, 13, },
  { 0, 1, 4, 8, 10, 11, 13, },
  { 2, 4, 8, 10, 11, 13, },
  { 0, 2, 4, 8, 10, 11, 13, },
  { 1, 2, 4, 8, 10, 11, 13, },
  { 0, 1, 2, 4, 8, 10, 11, 13, },
  { 3, 4, 8, 10, 11, 13, },
  { 0, 3, 4, 8, 10, 11, 13, },
  { 1, 3, 4, 8, 10, 11, 13, },
  { 0, 1, 3, 4, 8, 10, 11, 13, },
  { 2, 3, 4, 8, 10, 11, 13, },
  { 0, 2, 3, 4, 8, 10, 11, 13, },
  { 1, 2, 3, 4, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 13, },
  { 5, 8, 10, 11, 13, },
  { 0, 5, 8, 10, 11, 13, },
  { 1, 5, 8, 10, 11, 13, },
  { 0, 1, 5, 8, 10, 11, 13, },
  { 2, 5, 8, 10, 11, 13, },
  { 0, 2, 5, 8, 10, 11, 13, },
  { 1, 2, 5, 8, 10, 11, 13, },
  { 0, 1, 2, 5, 8, 10, 11, 13, },
  { 3, 5, 8, 10, 11, 13, },
  { 0, 3, 5, 8, 10, 11, 13, },
  { 1, 3, 5, 8, 10, 11, 13, },
  { 0, 1, 3, 5, 8, 10, 11, 13, },
  { 2, 3, 5, 8, 10, 11, 13, },
  { 0, 2, 3, 5, 8, 10, 11, 13, },
  { 1, 2, 3, 5, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 13, },
  { 4, 5, 8, 10, 11, 13, },
  { 0, 4, 5, 8, 10, 11, 13, },
  { 1, 4, 5, 8, 10, 11, 13, },
  { 0, 1, 4, 5, 8, 10, 11, 13, },
  { 2, 4, 5, 8, 10, 11, 13, },
  { 0, 2, 4, 5, 8, 10, 11, 13, },
  { 1, 2, 4, 5, 8, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 13, },
  { 3, 4, 5, 8, 10, 11, 13, },
  { 0, 3, 4, 5, 8, 10, 11, 13, },
  { 1, 3, 4, 5, 8, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 13, },
  { 2, 3, 4, 5, 8, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 13, },
  { 6, 8, 10, 11, 13, },
  { 0, 6, 8, 10, 11, 13, },
  { 1, 6, 8, 10, 11, 13, },
  { 0, 1, 6, 8, 10, 11, 13, },
  { 2, 6, 8, 10, 11, 13, },
  { 0, 2, 6, 8, 10, 11, 13, },
  { 1, 2, 6, 8, 10, 11, 13, },
  { 0, 1, 2, 6, 8, 10, 11, 13, },
  { 3, 6, 8, 10, 11, 13, },
  { 0, 3, 6, 8, 10, 11, 13, },
  { 1, 3, 6, 8, 10, 11, 13, },
  { 0, 1, 3, 6, 8, 10, 11, 13, },
  { 2, 3, 6, 8, 10, 11, 13, },
  { 0, 2, 3, 6, 8, 10, 11, 13, },
  { 1, 2, 3, 6, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 13, },
  { 4, 6, 8, 10, 11, 13, },
  { 0, 4, 6, 8, 10, 11, 13, },
  { 1, 4, 6, 8, 10, 11, 13, },
  { 0, 1, 4, 6, 8, 10, 11, 13, },
  { 2, 4, 6, 8, 10, 11, 13, },
  { 0, 2, 4, 6, 8, 10, 11, 13, },
  { 1, 2, 4, 6, 8, 10, 11, 13, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 13, },
  { 3, 4, 6, 8, 10, 11, 13, },
  { 0, 3, 4, 6, 8, 10, 11, 13, },
  { 1, 3, 4, 6, 8, 10, 11, 13, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 13, },
  { 2, 3, 4, 6, 8, 10, 11, 13, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 13, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 13, },
  { 5, 6, 8, 10, 11, 13, },
  { 0, 5, 6, 8, 10, 11, 13, },
  { 1, 5, 6, 8, 10, 11, 13, },
  { 0, 1, 5, 6, 8, 10, 11, 13, },
  { 2, 5, 6, 8, 10, 11, 13, },
  { 0, 2, 5, 6, 8, 10, 11, 13, },
  { 1, 2, 5, 6, 8, 10, 11, 13, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 13, },
  { 3, 5, 6, 8, 10, 11, 13, },
  { 0, 3, 5, 6, 8, 10, 11, 13, },
  { 1, 3, 5, 6, 8, 10, 11, 13, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 13, },
  { 2, 3, 5, 6, 8, 10, 11, 13, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 13, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 13, },
  { 4, 5, 6, 8, 10, 11, 13, },
  { 0, 4, 5, 6, 8, 10, 11, 13, },
  { 1, 4, 5, 6, 8, 10, 11, 13, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 13, },
  { 2, 4, 5, 6, 8, 10, 11, 13, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 13, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 13, },
  { 3, 4, 5, 6, 8, 10, 11, 13, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 13, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 13, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 13, },
  { 7, 8, 10, 11, 13, },
  { 0, 7, 8, 10, 11, 13, },
  { 1, 7, 8, 10, 11, 13, },
  { 0, 1, 7, 8, 10, 11, 13, },
  { 2, 7, 8, 10, 11, 13, },
  { 0, 2, 7, 8, 10, 11, 13, },
  { 1, 2, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 7, 8, 10, 11, 13, },
  { 3, 7, 8, 10, 11, 13, },
  { 0, 3, 7, 8, 10, 11, 13, },
  { 1, 3, 7, 8, 10, 11, 13, },
  { 0, 1, 3, 7, 8, 10, 11, 13, },
  { 2, 3, 7, 8, 10, 11, 13, },
  { 0, 2, 3, 7, 8, 10, 11, 13, },
  { 1, 2, 3, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 13, },
  { 4, 7, 8, 10, 11, 13, },
  { 0, 4, 7, 8, 10, 11, 13, },
  { 1, 4, 7, 8, 10, 11, 13, },
  { 0, 1, 4, 7, 8, 10, 11, 13, },
  { 2, 4, 7, 8, 10, 11, 13, },
  { 0, 2, 4, 7, 8, 10, 11, 13, },
  { 1, 2, 4, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 13, },
  { 3, 4, 7, 8, 10, 11, 13, },
  { 0, 3, 4, 7, 8, 10, 11, 13, },
  { 1, 3, 4, 7, 8, 10, 11, 13, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 13, },
  { 2, 3, 4, 7, 8, 10, 11, 13, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 13, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 13, },
  { 5, 7, 8, 10, 11, 13, },
  { 0, 5, 7, 8, 10, 11, 13, },
  { 1, 5, 7, 8, 10, 11, 13, },
  { 0, 1, 5, 7, 8, 10, 11, 13, },
  { 2, 5, 7, 8, 10, 11, 13, },
  { 0, 2, 5, 7, 8, 10, 11, 13, },
  { 1, 2, 5, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 13, },
  { 3, 5, 7, 8, 10, 11, 13, },
  { 0, 3, 5, 7, 8, 10, 11, 13, },
  { 1, 3, 5, 7, 8, 10, 11, 13, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 13, },
  { 2, 3, 5, 7, 8, 10, 11, 13, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 13, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 13, },
  { 4, 5, 7, 8, 10, 11, 13, },
  { 0, 4, 5, 7, 8, 10, 11, 13, },
  { 1, 4, 5, 7, 8, 10, 11, 13, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 13, },
  { 2, 4, 5, 7, 8, 10, 11, 13, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 13, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 13, },
  { 3, 4, 5, 7, 8, 10, 11, 13, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 13, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 13, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 13, },
  { 6, 7, 8, 10, 11, 13, },
  { 0, 6, 7, 8, 10, 11, 13, },
  { 1, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 6, 7, 8, 10, 11, 13, },
  { 2, 6, 7, 8, 10, 11, 13, },
  { 0, 2, 6, 7, 8, 10, 11, 13, },
  { 1, 2, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 13, },
  { 3, 6, 7, 8, 10, 11, 13, },
  { 0, 3, 6, 7, 8, 10, 11, 13, },
  { 1, 3, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 13, },
  { 2, 3, 6, 7, 8, 10, 11, 13, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 13, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 13, },
  { 4, 6, 7, 8, 10, 11, 13, },
  { 0, 4, 6, 7, 8, 10, 11, 13, },
  { 1, 4, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 13, },
  { 2, 4, 6, 7, 8, 10, 11, 13, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 13, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 13, },
  { 3, 4, 6, 7, 8, 10, 11, 13, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 13, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 13, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 13, },
  { 5, 6, 7, 8, 10, 11, 13, },
  { 0, 5, 6, 7, 8, 10, 11, 13, },
  { 1, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 13, },
  { 2, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 13, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 13, },
  { 3, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 13, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 13, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 13, },
  { 4, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, },
  { 9, 10, 11, 13, },
  { 0, 9, 10, 11, 13, },
  { 1, 9, 10, 11, 13, },
  { 0, 1, 9, 10, 11, 13, },
  { 2, 9, 10, 11, 13, },
  { 0, 2, 9, 10, 11, 13, },
  { 1, 2, 9, 10, 11, 13, },
  { 0, 1, 2, 9, 10, 11, 13, },
  { 3, 9, 10, 11, 13, },
  { 0, 3, 9, 10, 11, 13, },
  { 1, 3, 9, 10, 11, 13, },
  { 0, 1, 3, 9, 10, 11, 13, },
  { 2, 3, 9, 10, 11, 13, },
  { 0, 2, 3, 9, 10, 11, 13, },
  { 1, 2, 3, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 9, 10, 11, 13, },
  { 4, 9, 10, 11, 13, },
  { 0, 4, 9, 10, 11, 13, },
  { 1, 4, 9, 10, 11, 13, },
  { 0, 1, 4, 9, 10, 11, 13, },
  { 2, 4, 9, 10, 11, 13, },
  { 0, 2, 4, 9, 10, 11, 13, },
  { 1, 2, 4, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 9, 10, 11, 13, },
  { 3, 4, 9, 10, 11, 13, },
  { 0, 3, 4, 9, 10, 11, 13, },
  { 1, 3, 4, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 9, 10, 11, 13, },
  { 2, 3, 4, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 13, },
  { 5, 9, 10, 11, 13, },
  { 0, 5, 9, 10, 11, 13, },
  { 1, 5, 9, 10, 11, 13, },
  { 0, 1, 5, 9, 10, 11, 13, },
  { 2, 5, 9, 10, 11, 13, },
  { 0, 2, 5, 9, 10, 11, 13, },
  { 1, 2, 5, 9, 10, 11, 13, },
  { 0, 1, 2, 5, 9, 10, 11, 13, },
  { 3, 5, 9, 10, 11, 13, },
  { 0, 3, 5, 9, 10, 11, 13, },
  { 1, 3, 5, 9, 10, 11, 13, },
  { 0, 1, 3, 5, 9, 10, 11, 13, },
  { 2, 3, 5, 9, 10, 11, 13, },
  { 0, 2, 3, 5, 9, 10, 11, 13, },
  { 1, 2, 3, 5, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 13, },
  { 4, 5, 9, 10, 11, 13, },
  { 0, 4, 5, 9, 10, 11, 13, },
  { 1, 4, 5, 9, 10, 11, 13, },
  { 0, 1, 4, 5, 9, 10, 11, 13, },
  { 2, 4, 5, 9, 10, 11, 13, },
  { 0, 2, 4, 5, 9, 10, 11, 13, },
  { 1, 2, 4, 5, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 13, },
  { 3, 4, 5, 9, 10, 11, 13, },
  { 0, 3, 4, 5, 9, 10, 11, 13, },
  { 1, 3, 4, 5, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 13, },
  { 2, 3, 4, 5, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 13, },
  { 6, 9, 10, 11, 13, },
  { 0, 6, 9, 10, 11, 13, },
  { 1, 6, 9, 10, 11, 13, },
  { 0, 1, 6, 9, 10, 11, 13, },
  { 2, 6, 9, 10, 11, 13, },
  { 0, 2, 6, 9, 10, 11, 13, },
  { 1, 2, 6, 9, 10, 11, 13, },
  { 0, 1, 2, 6, 9, 10, 11, 13, },
  { 3, 6, 9, 10, 11, 13, },
  { 0, 3, 6, 9, 10, 11, 13, },
  { 1, 3, 6, 9, 10, 11, 13, },
  { 0, 1, 3, 6, 9, 10, 11, 13, },
  { 2, 3, 6, 9, 10, 11, 13, },
  { 0, 2, 3, 6, 9, 10, 11, 13, },
  { 1, 2, 3, 6, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 13, },
  { 4, 6, 9, 10, 11, 13, },
  { 0, 4, 6, 9, 10, 11, 13, },
  { 1, 4, 6, 9, 10, 11, 13, },
  { 0, 1, 4, 6, 9, 10, 11, 13, },
  { 2, 4, 6, 9, 10, 11, 13, },
  { 0, 2, 4, 6, 9, 10, 11, 13, },
  { 1, 2, 4, 6, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 13, },
  { 3, 4, 6, 9, 10, 11, 13, },
  { 0, 3, 4, 6, 9, 10, 11, 13, },
  { 1, 3, 4, 6, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 13, },
  { 2, 3, 4, 6, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 13, },
  { 5, 6, 9, 10, 11, 13, },
  { 0, 5, 6, 9, 10, 11, 13, },
  { 1, 5, 6, 9, 10, 11, 13, },
  { 0, 1, 5, 6, 9, 10, 11, 13, },
  { 2, 5, 6, 9, 10, 11, 13, },
  { 0, 2, 5, 6, 9, 10, 11, 13, },
  { 1, 2, 5, 6, 9, 10, 11, 13, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 13, },
  { 3, 5, 6, 9, 10, 11, 13, },
  { 0, 3, 5, 6, 9, 10, 11, 13, },
  { 1, 3, 5, 6, 9, 10, 11, 13, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 13, },
  { 2, 3, 5, 6, 9, 10, 11, 13, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 13, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 13, },
  { 4, 5, 6, 9, 10, 11, 13, },
  { 0, 4, 5, 6, 9, 10, 11, 13, },
  { 1, 4, 5, 6, 9, 10, 11, 13, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 13, },
  { 2, 4, 5, 6, 9, 10, 11, 13, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 13, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 13, },
  { 3, 4, 5, 6, 9, 10, 11, 13, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 13, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 13, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 13, },
  { 7, 9, 10, 11, 13, },
  { 0, 7, 9, 10, 11, 13, },
  { 1, 7, 9, 10, 11, 13, },
  { 0, 1, 7, 9, 10, 11, 13, },
  { 2, 7, 9, 10, 11, 13, },
  { 0, 2, 7, 9, 10, 11, 13, },
  { 1, 2, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 7, 9, 10, 11, 13, },
  { 3, 7, 9, 10, 11, 13, },
  { 0, 3, 7, 9, 10, 11, 13, },
  { 1, 3, 7, 9, 10, 11, 13, },
  { 0, 1, 3, 7, 9, 10, 11, 13, },
  { 2, 3, 7, 9, 10, 11, 13, },
  { 0, 2, 3, 7, 9, 10, 11, 13, },
  { 1, 2, 3, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 13, },
  { 4, 7, 9, 10, 11, 13, },
  { 0, 4, 7, 9, 10, 11, 13, },
  { 1, 4, 7, 9, 10, 11, 13, },
  { 0, 1, 4, 7, 9, 10, 11, 13, },
  { 2, 4, 7, 9, 10, 11, 13, },
  { 0, 2, 4, 7, 9, 10, 11, 13, },
  { 1, 2, 4, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 13, },
  { 3, 4, 7, 9, 10, 11, 13, },
  { 0, 3, 4, 7, 9, 10, 11, 13, },
  { 1, 3, 4, 7, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 13, },
  { 2, 3, 4, 7, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 13, },
  { 5, 7, 9, 10, 11, 13, },
  { 0, 5, 7, 9, 10, 11, 13, },
  { 1, 5, 7, 9, 10, 11, 13, },
  { 0, 1, 5, 7, 9, 10, 11, 13, },
  { 2, 5, 7, 9, 10, 11, 13, },
  { 0, 2, 5, 7, 9, 10, 11, 13, },
  { 1, 2, 5, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 13, },
  { 3, 5, 7, 9, 10, 11, 13, },
  { 0, 3, 5, 7, 9, 10, 11, 13, },
  { 1, 3, 5, 7, 9, 10, 11, 13, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 13, },
  { 2, 3, 5, 7, 9, 10, 11, 13, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 13, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 13, },
  { 4, 5, 7, 9, 10, 11, 13, },
  { 0, 4, 5, 7, 9, 10, 11, 13, },
  { 1, 4, 5, 7, 9, 10, 11, 13, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 13, },
  { 2, 4, 5, 7, 9, 10, 11, 13, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 13, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 13, },
  { 3, 4, 5, 7, 9, 10, 11, 13, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 13, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 13, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 13, },
  { 6, 7, 9, 10, 11, 13, },
  { 0, 6, 7, 9, 10, 11, 13, },
  { 1, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 6, 7, 9, 10, 11, 13, },
  { 2, 6, 7, 9, 10, 11, 13, },
  { 0, 2, 6, 7, 9, 10, 11, 13, },
  { 1, 2, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 13, },
  { 3, 6, 7, 9, 10, 11, 13, },
  { 0, 3, 6, 7, 9, 10, 11, 13, },
  { 1, 3, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 13, },
  { 2, 3, 6, 7, 9, 10, 11, 13, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 13, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 13, },
  { 4, 6, 7, 9, 10, 11, 13, },
  { 0, 4, 6, 7, 9, 10, 11, 13, },
  { 1, 4, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 13, },
  { 2, 4, 6, 7, 9, 10, 11, 13, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 13, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 13, },
  { 3, 4, 6, 7, 9, 10, 11, 13, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 13, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 13, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 13, },
  { 5, 6, 7, 9, 10, 11, 13, },
  { 0, 5, 6, 7, 9, 10, 11, 13, },
  { 1, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 13, },
  { 2, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 13, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 13, },
  { 3, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 13, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 13, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 13, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 13, },
  { 4, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, },
  { 8, 9, 10, 11, 13, },
  { 0, 8, 9, 10, 11, 13, },
  { 1, 8, 9, 10, 11, 13, },
  { 0, 1, 8, 9, 10, 11, 13, },
  { 2, 8, 9, 10, 11, 13, },
  { 0, 2, 8, 9, 10, 11, 13, },
  { 1, 2, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 8, 9, 10, 11, 13, },
  { 3, 8, 9, 10, 11, 13, },
  { 0, 3, 8, 9, 10, 11, 13, },
  { 1, 3, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 8, 9, 10, 11, 13, },
  { 2, 3, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 13, },
  { 4, 8, 9, 10, 11, 13, },
  { 0, 4, 8, 9, 10, 11, 13, },
  { 1, 4, 8, 9, 10, 11, 13, },
  { 0, 1, 4, 8, 9, 10, 11, 13, },
  { 2, 4, 8, 9, 10, 11, 13, },
  { 0, 2, 4, 8, 9, 10, 11, 13, },
  { 1, 2, 4, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 13, },
  { 3, 4, 8, 9, 10, 11, 13, },
  { 0, 3, 4, 8, 9, 10, 11, 13, },
  { 1, 3, 4, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 13, },
  { 2, 3, 4, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 13, },
  { 5, 8, 9, 10, 11, 13, },
  { 0, 5, 8, 9, 10, 11, 13, },
  { 1, 5, 8, 9, 10, 11, 13, },
  { 0, 1, 5, 8, 9, 10, 11, 13, },
  { 2, 5, 8, 9, 10, 11, 13, },
  { 0, 2, 5, 8, 9, 10, 11, 13, },
  { 1, 2, 5, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 13, },
  { 3, 5, 8, 9, 10, 11, 13, },
  { 0, 3, 5, 8, 9, 10, 11, 13, },
  { 1, 3, 5, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 13, },
  { 2, 3, 5, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 13, },
  { 4, 5, 8, 9, 10, 11, 13, },
  { 0, 4, 5, 8, 9, 10, 11, 13, },
  { 1, 4, 5, 8, 9, 10, 11, 13, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 13, },
  { 2, 4, 5, 8, 9, 10, 11, 13, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 13, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 13, },
  { 3, 4, 5, 8, 9, 10, 11, 13, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 13, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 13, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 13, },
  { 6, 8, 9, 10, 11, 13, },
  { 0, 6, 8, 9, 10, 11, 13, },
  { 1, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 6, 8, 9, 10, 11, 13, },
  { 2, 6, 8, 9, 10, 11, 13, },
  { 0, 2, 6, 8, 9, 10, 11, 13, },
  { 1, 2, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 13, },
  { 3, 6, 8, 9, 10, 11, 13, },
  { 0, 3, 6, 8, 9, 10, 11, 13, },
  { 1, 3, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 13, },
  { 2, 3, 6, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 13, },
  { 4, 6, 8, 9, 10, 11, 13, },
  { 0, 4, 6, 8, 9, 10, 11, 13, },
  { 1, 4, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 13, },
  { 2, 4, 6, 8, 9, 10, 11, 13, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 13, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 13, },
  { 3, 4, 6, 8, 9, 10, 11, 13, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 13, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 13, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 13, },
  { 5, 6, 8, 9, 10, 11, 13, },
  { 0, 5, 6, 8, 9, 10, 11, 13, },
  { 1, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 13, },
  { 2, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 13, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 13, },
  { 3, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 13, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 13, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 13, },
  { 4, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, },
  { 7, 8, 9, 10, 11, 13, },
  { 0, 7, 8, 9, 10, 11, 13, },
  { 1, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 7, 8, 9, 10, 11, 13, },
  { 2, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 13, },
  { 3, 7, 8, 9, 10, 11, 13, },
  { 0, 3, 7, 8, 9, 10, 11, 13, },
  { 1, 3, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 13, },
  { 2, 3, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 13, },
  { 4, 7, 8, 9, 10, 11, 13, },
  { 0, 4, 7, 8, 9, 10, 11, 13, },
  { 1, 4, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 13, },
  { 2, 4, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 13, },
  { 3, 4, 7, 8, 9, 10, 11, 13, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 13, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 13, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 13, },
  { 5, 7, 8, 9, 10, 11, 13, },
  { 0, 5, 7, 8, 9, 10, 11, 13, },
  { 1, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 13, },
  { 2, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 13, },
  { 3, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 13, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 13, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 13, },
  { 4, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, },
  { 6, 7, 8, 9, 10, 11, 13, },
  { 0, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 13, },
  { 2, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 13, },
  { 3, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 13, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 13, },
  { 4, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, },
  { 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, },
  { 12, 13, },
  { 0, 12, 13, },
  { 1, 12, 13, },
  { 0, 1, 12, 13, },
  { 2, 12, 13, },
  { 0, 2, 12, 13, },
  { 1, 2, 12, 13, },
  { 0, 1, 2, 12, 13, },
  { 3, 12, 13, },
  { 0, 3, 12, 13, },
  { 1, 3, 12, 13, },
  { 0, 1, 3, 12, 13, },
  { 2, 3, 12, 13, },
  { 0, 2, 3, 12, 13, },
  { 1, 2, 3, 12, 13, },
  { 0, 1, 2, 3, 12, 13, },
  { 4, 12, 13, },
  { 0, 4, 12, 13, },
  { 1, 4, 12, 13, },
  { 0, 1, 4, 12, 13, },
  { 2, 4, 12, 13, },
  { 0, 2, 4, 12, 13, },
  { 1, 2, 4, 12, 13, },
  { 0, 1, 2, 4, 12, 13, },
  { 3, 4, 12, 13, },
  { 0, 3, 4, 12, 13, },
  { 1, 3, 4, 12, 13, },
  { 0, 1, 3, 4, 12, 13, },
  { 2, 3, 4, 12, 13, },
  { 0, 2, 3, 4, 12, 13, },
  { 1, 2, 3, 4, 12, 13, },
  { 0, 1, 2, 3, 4, 12, 13, },
  { 5, 12, 13, },
  { 0, 5, 12, 13, },
  { 1, 5, 12, 13, },
  { 0, 1, 5, 12, 13, },
  { 2, 5, 12, 13, },
  { 0, 2, 5, 12, 13, },
  { 1, 2, 5, 12, 13, },
  { 0, 1, 2, 5, 12, 13, },
  { 3, 5, 12, 13, },
  { 0, 3, 5, 12, 13, },
  { 1, 3, 5, 12, 13, },
  { 0, 1, 3, 5, 12, 13, },
  { 2, 3, 5, 12, 13, },
  { 0, 2, 3, 5, 12, 13, },
  { 1, 2, 3, 5, 12, 13, },
  { 0, 1, 2, 3, 5, 12, 13, },
  { 4, 5, 12, 13, },
  { 0, 4, 5, 12, 13, },
  { 1, 4, 5, 12, 13, },
  { 0, 1, 4, 5, 12, 13, },
  { 2, 4, 5, 12, 13, },
  { 0, 2, 4, 5, 12, 13, },
  { 1, 2, 4, 5, 12, 13, },
  { 0, 1, 2, 4, 5, 12, 13, },
  { 3, 4, 5, 12, 13, },
  { 0, 3, 4, 5, 12, 13, },
  { 1, 3, 4, 5, 12, 13, },
  { 0, 1, 3, 4, 5, 12, 13, },
  { 2, 3, 4, 5, 12, 13, },
  { 0, 2, 3, 4, 5, 12, 13, },
  { 1, 2, 3, 4, 5, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 12, 13, },
  { 6, 12, 13, },
  { 0, 6, 12, 13, },
  { 1, 6, 12, 13, },
  { 0, 1, 6, 12, 13, },
  { 2, 6, 12, 13, },
  { 0, 2, 6, 12, 13, },
  { 1, 2, 6, 12, 13, },
  { 0, 1, 2, 6, 12, 13, },
  { 3, 6, 12, 13, },
  { 0, 3, 6, 12, 13, },
  { 1, 3, 6, 12, 13, },
  { 0, 1, 3, 6, 12, 13, },
  { 2, 3, 6, 12, 13, },
  { 0, 2, 3, 6, 12, 13, },
  { 1, 2, 3, 6, 12, 13, },
  { 0, 1, 2, 3, 6, 12, 13, },
  { 4, 6, 12, 13, },
  { 0, 4, 6, 12, 13, },
  { 1, 4, 6, 12, 13, },
  { 0, 1, 4, 6, 12, 13, },
  { 2, 4, 6, 12, 13, },
  { 0, 2, 4, 6, 12, 13, },
  { 1, 2, 4, 6, 12, 13, },
  { 0, 1, 2, 4, 6, 12, 13, },
  { 3, 4, 6, 12, 13, },
  { 0, 3, 4, 6, 12, 13, },
  { 1, 3, 4, 6, 12, 13, },
  { 0, 1, 3, 4, 6, 12, 13, },
  { 2, 3, 4, 6, 12, 13, },
  { 0, 2, 3, 4, 6, 12, 13, },
  { 1, 2, 3, 4, 6, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 12, 13, },
  { 5, 6, 12, 13, },
  { 0, 5, 6, 12, 13, },
  { 1, 5, 6, 12, 13, },
  { 0, 1, 5, 6, 12, 13, },
  { 2, 5, 6, 12, 13, },
  { 0, 2, 5, 6, 12, 13, },
  { 1, 2, 5, 6, 12, 13, },
  { 0, 1, 2, 5, 6, 12, 13, },
  { 3, 5, 6, 12, 13, },
  { 0, 3, 5, 6, 12, 13, },
  { 1, 3, 5, 6, 12, 13, },
  { 0, 1, 3, 5, 6, 12, 13, },
  { 2, 3, 5, 6, 12, 13, },
  { 0, 2, 3, 5, 6, 12, 13, },
  { 1, 2, 3, 5, 6, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 12, 13, },
  { 4, 5, 6, 12, 13, },
  { 0, 4, 5, 6, 12, 13, },
  { 1, 4, 5, 6, 12, 13, },
  { 0, 1, 4, 5, 6, 12, 13, },
  { 2, 4, 5, 6, 12, 13, },
  { 0, 2, 4, 5, 6, 12, 13, },
  { 1, 2, 4, 5, 6, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 12, 13, },
  { 3, 4, 5, 6, 12, 13, },
  { 0, 3, 4, 5, 6, 12, 13, },
  { 1, 3, 4, 5, 6, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 12, 13, },
  { 2, 3, 4, 5, 6, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 12, 13, },
  { 7, 12, 13, },
  { 0, 7, 12, 13, },
  { 1, 7, 12, 13, },
  { 0, 1, 7, 12, 13, },
  { 2, 7, 12, 13, },
  { 0, 2, 7, 12, 13, },
  { 1, 2, 7, 12, 13, },
  { 0, 1, 2, 7, 12, 13, },
  { 3, 7, 12, 13, },
  { 0, 3, 7, 12, 13, },
  { 1, 3, 7, 12, 13, },
  { 0, 1, 3, 7, 12, 13, },
  { 2, 3, 7, 12, 13, },
  { 0, 2, 3, 7, 12, 13, },
  { 1, 2, 3, 7, 12, 13, },
  { 0, 1, 2, 3, 7, 12, 13, },
  { 4, 7, 12, 13, },
  { 0, 4, 7, 12, 13, },
  { 1, 4, 7, 12, 13, },
  { 0, 1, 4, 7, 12, 13, },
  { 2, 4, 7, 12, 13, },
  { 0, 2, 4, 7, 12, 13, },
  { 1, 2, 4, 7, 12, 13, },
  { 0, 1, 2, 4, 7, 12, 13, },
  { 3, 4, 7, 12, 13, },
  { 0, 3, 4, 7, 12, 13, },
  { 1, 3, 4, 7, 12, 13, },
  { 0, 1, 3, 4, 7, 12, 13, },
  { 2, 3, 4, 7, 12, 13, },
  { 0, 2, 3, 4, 7, 12, 13, },
  { 1, 2, 3, 4, 7, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 12, 13, },
  { 5, 7, 12, 13, },
  { 0, 5, 7, 12, 13, },
  { 1, 5, 7, 12, 13, },
  { 0, 1, 5, 7, 12, 13, },
  { 2, 5, 7, 12, 13, },
  { 0, 2, 5, 7, 12, 13, },
  { 1, 2, 5, 7, 12, 13, },
  { 0, 1, 2, 5, 7, 12, 13, },
  { 3, 5, 7, 12, 13, },
  { 0, 3, 5, 7, 12, 13, },
  { 1, 3, 5, 7, 12, 13, },
  { 0, 1, 3, 5, 7, 12, 13, },
  { 2, 3, 5, 7, 12, 13, },
  { 0, 2, 3, 5, 7, 12, 13, },
  { 1, 2, 3, 5, 7, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 12, 13, },
  { 4, 5, 7, 12, 13, },
  { 0, 4, 5, 7, 12, 13, },
  { 1, 4, 5, 7, 12, 13, },
  { 0, 1, 4, 5, 7, 12, 13, },
  { 2, 4, 5, 7, 12, 13, },
  { 0, 2, 4, 5, 7, 12, 13, },
  { 1, 2, 4, 5, 7, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 12, 13, },
  { 3, 4, 5, 7, 12, 13, },
  { 0, 3, 4, 5, 7, 12, 13, },
  { 1, 3, 4, 5, 7, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 12, 13, },
  { 2, 3, 4, 5, 7, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 12, 13, },
  { 6, 7, 12, 13, },
  { 0, 6, 7, 12, 13, },
  { 1, 6, 7, 12, 13, },
  { 0, 1, 6, 7, 12, 13, },
  { 2, 6, 7, 12, 13, },
  { 0, 2, 6, 7, 12, 13, },
  { 1, 2, 6, 7, 12, 13, },
  { 0, 1, 2, 6, 7, 12, 13, },
  { 3, 6, 7, 12, 13, },
  { 0, 3, 6, 7, 12, 13, },
  { 1, 3, 6, 7, 12, 13, },
  { 0, 1, 3, 6, 7, 12, 13, },
  { 2, 3, 6, 7, 12, 13, },
  { 0, 2, 3, 6, 7, 12, 13, },
  { 1, 2, 3, 6, 7, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 12, 13, },
  { 4, 6, 7, 12, 13, },
  { 0, 4, 6, 7, 12, 13, },
  { 1, 4, 6, 7, 12, 13, },
  { 0, 1, 4, 6, 7, 12, 13, },
  { 2, 4, 6, 7, 12, 13, },
  { 0, 2, 4, 6, 7, 12, 13, },
  { 1, 2, 4, 6, 7, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 12, 13, },
  { 3, 4, 6, 7, 12, 13, },
  { 0, 3, 4, 6, 7, 12, 13, },
  { 1, 3, 4, 6, 7, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 12, 13, },
  { 2, 3, 4, 6, 7, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 12, 13, },
  { 5, 6, 7, 12, 13, },
  { 0, 5, 6, 7, 12, 13, },
  { 1, 5, 6, 7, 12, 13, },
  { 0, 1, 5, 6, 7, 12, 13, },
  { 2, 5, 6, 7, 12, 13, },
  { 0, 2, 5, 6, 7, 12, 13, },
  { 1, 2, 5, 6, 7, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 12, 13, },
  { 3, 5, 6, 7, 12, 13, },
  { 0, 3, 5, 6, 7, 12, 13, },
  { 1, 3, 5, 6, 7, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 12, 13, },
  { 2, 3, 5, 6, 7, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 12, 13, },
  { 4, 5, 6, 7, 12, 13, },
  { 0, 4, 5, 6, 7, 12, 13, },
  { 1, 4, 5, 6, 7, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 12, 13, },
  { 2, 4, 5, 6, 7, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 12, 13, },
  { 3, 4, 5, 6, 7, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 12, 13, },
  { 8, 12, 13, },
  { 0, 8, 12, 13, },
  { 1, 8, 12, 13, },
  { 0, 1, 8, 12, 13, },
  { 2, 8, 12, 13, },
  { 0, 2, 8, 12, 13, },
  { 1, 2, 8, 12, 13, },
  { 0, 1, 2, 8, 12, 13, },
  { 3, 8, 12, 13, },
  { 0, 3, 8, 12, 13, },
  { 1, 3, 8, 12, 13, },
  { 0, 1, 3, 8, 12, 13, },
  { 2, 3, 8, 12, 13, },
  { 0, 2, 3, 8, 12, 13, },
  { 1, 2, 3, 8, 12, 13, },
  { 0, 1, 2, 3, 8, 12, 13, },
  { 4, 8, 12, 13, },
  { 0, 4, 8, 12, 13, },
  { 1, 4, 8, 12, 13, },
  { 0, 1, 4, 8, 12, 13, },
  { 2, 4, 8, 12, 13, },
  { 0, 2, 4, 8, 12, 13, },
  { 1, 2, 4, 8, 12, 13, },
  { 0, 1, 2, 4, 8, 12, 13, },
  { 3, 4, 8, 12, 13, },
  { 0, 3, 4, 8, 12, 13, },
  { 1, 3, 4, 8, 12, 13, },
  { 0, 1, 3, 4, 8, 12, 13, },
  { 2, 3, 4, 8, 12, 13, },
  { 0, 2, 3, 4, 8, 12, 13, },
  { 1, 2, 3, 4, 8, 12, 13, },
  { 0, 1, 2, 3, 4, 8, 12, 13, },
  { 5, 8, 12, 13, },
  { 0, 5, 8, 12, 13, },
  { 1, 5, 8, 12, 13, },
  { 0, 1, 5, 8, 12, 13, },
  { 2, 5, 8, 12, 13, },
  { 0, 2, 5, 8, 12, 13, },
  { 1, 2, 5, 8, 12, 13, },
  { 0, 1, 2, 5, 8, 12, 13, },
  { 3, 5, 8, 12, 13, },
  { 0, 3, 5, 8, 12, 13, },
  { 1, 3, 5, 8, 12, 13, },
  { 0, 1, 3, 5, 8, 12, 13, },
  { 2, 3, 5, 8, 12, 13, },
  { 0, 2, 3, 5, 8, 12, 13, },
  { 1, 2, 3, 5, 8, 12, 13, },
  { 0, 1, 2, 3, 5, 8, 12, 13, },
  { 4, 5, 8, 12, 13, },
  { 0, 4, 5, 8, 12, 13, },
  { 1, 4, 5, 8, 12, 13, },
  { 0, 1, 4, 5, 8, 12, 13, },
  { 2, 4, 5, 8, 12, 13, },
  { 0, 2, 4, 5, 8, 12, 13, },
  { 1, 2, 4, 5, 8, 12, 13, },
  { 0, 1, 2, 4, 5, 8, 12, 13, },
  { 3, 4, 5, 8, 12, 13, },
  { 0, 3, 4, 5, 8, 12, 13, },
  { 1, 3, 4, 5, 8, 12, 13, },
  { 0, 1, 3, 4, 5, 8, 12, 13, },
  { 2, 3, 4, 5, 8, 12, 13, },
  { 0, 2, 3, 4, 5, 8, 12, 13, },
  { 1, 2, 3, 4, 5, 8, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 12, 13, },
  { 6, 8, 12, 13, },
  { 0, 6, 8, 12, 13, },
  { 1, 6, 8, 12, 13, },
  { 0, 1, 6, 8, 12, 13, },
  { 2, 6, 8, 12, 13, },
  { 0, 2, 6, 8, 12, 13, },
  { 1, 2, 6, 8, 12, 13, },
  { 0, 1, 2, 6, 8, 12, 13, },
  { 3, 6, 8, 12, 13, },
  { 0, 3, 6, 8, 12, 13, },
  { 1, 3, 6, 8, 12, 13, },
  { 0, 1, 3, 6, 8, 12, 13, },
  { 2, 3, 6, 8, 12, 13, },
  { 0, 2, 3, 6, 8, 12, 13, },
  { 1, 2, 3, 6, 8, 12, 13, },
  { 0, 1, 2, 3, 6, 8, 12, 13, },
  { 4, 6, 8, 12, 13, },
  { 0, 4, 6, 8, 12, 13, },
  { 1, 4, 6, 8, 12, 13, },
  { 0, 1, 4, 6, 8, 12, 13, },
  { 2, 4, 6, 8, 12, 13, },
  { 0, 2, 4, 6, 8, 12, 13, },
  { 1, 2, 4, 6, 8, 12, 13, },
  { 0, 1, 2, 4, 6, 8, 12, 13, },
  { 3, 4, 6, 8, 12, 13, },
  { 0, 3, 4, 6, 8, 12, 13, },
  { 1, 3, 4, 6, 8, 12, 13, },
  { 0, 1, 3, 4, 6, 8, 12, 13, },
  { 2, 3, 4, 6, 8, 12, 13, },
  { 0, 2, 3, 4, 6, 8, 12, 13, },
  { 1, 2, 3, 4, 6, 8, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 12, 13, },
  { 5, 6, 8, 12, 13, },
  { 0, 5, 6, 8, 12, 13, },
  { 1, 5, 6, 8, 12, 13, },
  { 0, 1, 5, 6, 8, 12, 13, },
  { 2, 5, 6, 8, 12, 13, },
  { 0, 2, 5, 6, 8, 12, 13, },
  { 1, 2, 5, 6, 8, 12, 13, },
  { 0, 1, 2, 5, 6, 8, 12, 13, },
  { 3, 5, 6, 8, 12, 13, },
  { 0, 3, 5, 6, 8, 12, 13, },
  { 1, 3, 5, 6, 8, 12, 13, },
  { 0, 1, 3, 5, 6, 8, 12, 13, },
  { 2, 3, 5, 6, 8, 12, 13, },
  { 0, 2, 3, 5, 6, 8, 12, 13, },
  { 1, 2, 3, 5, 6, 8, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 12, 13, },
  { 4, 5, 6, 8, 12, 13, },
  { 0, 4, 5, 6, 8, 12, 13, },
  { 1, 4, 5, 6, 8, 12, 13, },
  { 0, 1, 4, 5, 6, 8, 12, 13, },
  { 2, 4, 5, 6, 8, 12, 13, },
  { 0, 2, 4, 5, 6, 8, 12, 13, },
  { 1, 2, 4, 5, 6, 8, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 12, 13, },
  { 3, 4, 5, 6, 8, 12, 13, },
  { 0, 3, 4, 5, 6, 8, 12, 13, },
  { 1, 3, 4, 5, 6, 8, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 12, 13, },
  { 2, 3, 4, 5, 6, 8, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 12, 13, },
  { 7, 8, 12, 13, },
  { 0, 7, 8, 12, 13, },
  { 1, 7, 8, 12, 13, },
  { 0, 1, 7, 8, 12, 13, },
  { 2, 7, 8, 12, 13, },
  { 0, 2, 7, 8, 12, 13, },
  { 1, 2, 7, 8, 12, 13, },
  { 0, 1, 2, 7, 8, 12, 13, },
  { 3, 7, 8, 12, 13, },
  { 0, 3, 7, 8, 12, 13, },
  { 1, 3, 7, 8, 12, 13, },
  { 0, 1, 3, 7, 8, 12, 13, },
  { 2, 3, 7, 8, 12, 13, },
  { 0, 2, 3, 7, 8, 12, 13, },
  { 1, 2, 3, 7, 8, 12, 13, },
  { 0, 1, 2, 3, 7, 8, 12, 13, },
  { 4, 7, 8, 12, 13, },
  { 0, 4, 7, 8, 12, 13, },
  { 1, 4, 7, 8, 12, 13, },
  { 0, 1, 4, 7, 8, 12, 13, },
  { 2, 4, 7, 8, 12, 13, },
  { 0, 2, 4, 7, 8, 12, 13, },
  { 1, 2, 4, 7, 8, 12, 13, },
  { 0, 1, 2, 4, 7, 8, 12, 13, },
  { 3, 4, 7, 8, 12, 13, },
  { 0, 3, 4, 7, 8, 12, 13, },
  { 1, 3, 4, 7, 8, 12, 13, },
  { 0, 1, 3, 4, 7, 8, 12, 13, },
  { 2, 3, 4, 7, 8, 12, 13, },
  { 0, 2, 3, 4, 7, 8, 12, 13, },
  { 1, 2, 3, 4, 7, 8, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 12, 13, },
  { 5, 7, 8, 12, 13, },
  { 0, 5, 7, 8, 12, 13, },
  { 1, 5, 7, 8, 12, 13, },
  { 0, 1, 5, 7, 8, 12, 13, },
  { 2, 5, 7, 8, 12, 13, },
  { 0, 2, 5, 7, 8, 12, 13, },
  { 1, 2, 5, 7, 8, 12, 13, },
  { 0, 1, 2, 5, 7, 8, 12, 13, },
  { 3, 5, 7, 8, 12, 13, },
  { 0, 3, 5, 7, 8, 12, 13, },
  { 1, 3, 5, 7, 8, 12, 13, },
  { 0, 1, 3, 5, 7, 8, 12, 13, },
  { 2, 3, 5, 7, 8, 12, 13, },
  { 0, 2, 3, 5, 7, 8, 12, 13, },
  { 1, 2, 3, 5, 7, 8, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 12, 13, },
  { 4, 5, 7, 8, 12, 13, },
  { 0, 4, 5, 7, 8, 12, 13, },
  { 1, 4, 5, 7, 8, 12, 13, },
  { 0, 1, 4, 5, 7, 8, 12, 13, },
  { 2, 4, 5, 7, 8, 12, 13, },
  { 0, 2, 4, 5, 7, 8, 12, 13, },
  { 1, 2, 4, 5, 7, 8, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 12, 13, },
  { 3, 4, 5, 7, 8, 12, 13, },
  { 0, 3, 4, 5, 7, 8, 12, 13, },
  { 1, 3, 4, 5, 7, 8, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 12, 13, },
  { 2, 3, 4, 5, 7, 8, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 12, 13, },
  { 6, 7, 8, 12, 13, },
  { 0, 6, 7, 8, 12, 13, },
  { 1, 6, 7, 8, 12, 13, },
  { 0, 1, 6, 7, 8, 12, 13, },
  { 2, 6, 7, 8, 12, 13, },
  { 0, 2, 6, 7, 8, 12, 13, },
  { 1, 2, 6, 7, 8, 12, 13, },
  { 0, 1, 2, 6, 7, 8, 12, 13, },
  { 3, 6, 7, 8, 12, 13, },
  { 0, 3, 6, 7, 8, 12, 13, },
  { 1, 3, 6, 7, 8, 12, 13, },
  { 0, 1, 3, 6, 7, 8, 12, 13, },
  { 2, 3, 6, 7, 8, 12, 13, },
  { 0, 2, 3, 6, 7, 8, 12, 13, },
  { 1, 2, 3, 6, 7, 8, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 12, 13, },
  { 4, 6, 7, 8, 12, 13, },
  { 0, 4, 6, 7, 8, 12, 13, },
  { 1, 4, 6, 7, 8, 12, 13, },
  { 0, 1, 4, 6, 7, 8, 12, 13, },
  { 2, 4, 6, 7, 8, 12, 13, },
  { 0, 2, 4, 6, 7, 8, 12, 13, },
  { 1, 2, 4, 6, 7, 8, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 12, 13, },
  { 3, 4, 6, 7, 8, 12, 13, },
  { 0, 3, 4, 6, 7, 8, 12, 13, },
  { 1, 3, 4, 6, 7, 8, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 12, 13, },
  { 2, 3, 4, 6, 7, 8, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 12, 13, },
  { 5, 6, 7, 8, 12, 13, },
  { 0, 5, 6, 7, 8, 12, 13, },
  { 1, 5, 6, 7, 8, 12, 13, },
  { 0, 1, 5, 6, 7, 8, 12, 13, },
  { 2, 5, 6, 7, 8, 12, 13, },
  { 0, 2, 5, 6, 7, 8, 12, 13, },
  { 1, 2, 5, 6, 7, 8, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 12, 13, },
  { 3, 5, 6, 7, 8, 12, 13, },
  { 0, 3, 5, 6, 7, 8, 12, 13, },
  { 1, 3, 5, 6, 7, 8, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 12, 13, },
  { 2, 3, 5, 6, 7, 8, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 12, 13, },
  { 4, 5, 6, 7, 8, 12, 13, },
  { 0, 4, 5, 6, 7, 8, 12, 13, },
  { 1, 4, 5, 6, 7, 8, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 12, 13, },
  { 2, 4, 5, 6, 7, 8, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 12, 13, },
  { 3, 4, 5, 6, 7, 8, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 12, 13, },
  { 9, 12, 13, },
  { 0, 9, 12, 13, },
  { 1, 9, 12, 13, },
  { 0, 1, 9, 12, 13, },
  { 2, 9, 12, 13, },
  { 0, 2, 9, 12, 13, },
  { 1, 2, 9, 12, 13, },
  { 0, 1, 2, 9, 12, 13, },
  { 3, 9, 12, 13, },
  { 0, 3, 9, 12, 13, },
  { 1, 3, 9, 12, 13, },
  { 0, 1, 3, 9, 12, 13, },
  { 2, 3, 9, 12, 13, },
  { 0, 2, 3, 9, 12, 13, },
  { 1, 2, 3, 9, 12, 13, },
  { 0, 1, 2, 3, 9, 12, 13, },
  { 4, 9, 12, 13, },
  { 0, 4, 9, 12, 13, },
  { 1, 4, 9, 12, 13, },
  { 0, 1, 4, 9, 12, 13, },
  { 2, 4, 9, 12, 13, },
  { 0, 2, 4, 9, 12, 13, },
  { 1, 2, 4, 9, 12, 13, },
  { 0, 1, 2, 4, 9, 12, 13, },
  { 3, 4, 9, 12, 13, },
  { 0, 3, 4, 9, 12, 13, },
  { 1, 3, 4, 9, 12, 13, },
  { 0, 1, 3, 4, 9, 12, 13, },
  { 2, 3, 4, 9, 12, 13, },
  { 0, 2, 3, 4, 9, 12, 13, },
  { 1, 2, 3, 4, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 9, 12, 13, },
  { 5, 9, 12, 13, },
  { 0, 5, 9, 12, 13, },
  { 1, 5, 9, 12, 13, },
  { 0, 1, 5, 9, 12, 13, },
  { 2, 5, 9, 12, 13, },
  { 0, 2, 5, 9, 12, 13, },
  { 1, 2, 5, 9, 12, 13, },
  { 0, 1, 2, 5, 9, 12, 13, },
  { 3, 5, 9, 12, 13, },
  { 0, 3, 5, 9, 12, 13, },
  { 1, 3, 5, 9, 12, 13, },
  { 0, 1, 3, 5, 9, 12, 13, },
  { 2, 3, 5, 9, 12, 13, },
  { 0, 2, 3, 5, 9, 12, 13, },
  { 1, 2, 3, 5, 9, 12, 13, },
  { 0, 1, 2, 3, 5, 9, 12, 13, },
  { 4, 5, 9, 12, 13, },
  { 0, 4, 5, 9, 12, 13, },
  { 1, 4, 5, 9, 12, 13, },
  { 0, 1, 4, 5, 9, 12, 13, },
  { 2, 4, 5, 9, 12, 13, },
  { 0, 2, 4, 5, 9, 12, 13, },
  { 1, 2, 4, 5, 9, 12, 13, },
  { 0, 1, 2, 4, 5, 9, 12, 13, },
  { 3, 4, 5, 9, 12, 13, },
  { 0, 3, 4, 5, 9, 12, 13, },
  { 1, 3, 4, 5, 9, 12, 13, },
  { 0, 1, 3, 4, 5, 9, 12, 13, },
  { 2, 3, 4, 5, 9, 12, 13, },
  { 0, 2, 3, 4, 5, 9, 12, 13, },
  { 1, 2, 3, 4, 5, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 9, 12, 13, },
  { 6, 9, 12, 13, },
  { 0, 6, 9, 12, 13, },
  { 1, 6, 9, 12, 13, },
  { 0, 1, 6, 9, 12, 13, },
  { 2, 6, 9, 12, 13, },
  { 0, 2, 6, 9, 12, 13, },
  { 1, 2, 6, 9, 12, 13, },
  { 0, 1, 2, 6, 9, 12, 13, },
  { 3, 6, 9, 12, 13, },
  { 0, 3, 6, 9, 12, 13, },
  { 1, 3, 6, 9, 12, 13, },
  { 0, 1, 3, 6, 9, 12, 13, },
  { 2, 3, 6, 9, 12, 13, },
  { 0, 2, 3, 6, 9, 12, 13, },
  { 1, 2, 3, 6, 9, 12, 13, },
  { 0, 1, 2, 3, 6, 9, 12, 13, },
  { 4, 6, 9, 12, 13, },
  { 0, 4, 6, 9, 12, 13, },
  { 1, 4, 6, 9, 12, 13, },
  { 0, 1, 4, 6, 9, 12, 13, },
  { 2, 4, 6, 9, 12, 13, },
  { 0, 2, 4, 6, 9, 12, 13, },
  { 1, 2, 4, 6, 9, 12, 13, },
  { 0, 1, 2, 4, 6, 9, 12, 13, },
  { 3, 4, 6, 9, 12, 13, },
  { 0, 3, 4, 6, 9, 12, 13, },
  { 1, 3, 4, 6, 9, 12, 13, },
  { 0, 1, 3, 4, 6, 9, 12, 13, },
  { 2, 3, 4, 6, 9, 12, 13, },
  { 0, 2, 3, 4, 6, 9, 12, 13, },
  { 1, 2, 3, 4, 6, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 9, 12, 13, },
  { 5, 6, 9, 12, 13, },
  { 0, 5, 6, 9, 12, 13, },
  { 1, 5, 6, 9, 12, 13, },
  { 0, 1, 5, 6, 9, 12, 13, },
  { 2, 5, 6, 9, 12, 13, },
  { 0, 2, 5, 6, 9, 12, 13, },
  { 1, 2, 5, 6, 9, 12, 13, },
  { 0, 1, 2, 5, 6, 9, 12, 13, },
  { 3, 5, 6, 9, 12, 13, },
  { 0, 3, 5, 6, 9, 12, 13, },
  { 1, 3, 5, 6, 9, 12, 13, },
  { 0, 1, 3, 5, 6, 9, 12, 13, },
  { 2, 3, 5, 6, 9, 12, 13, },
  { 0, 2, 3, 5, 6, 9, 12, 13, },
  { 1, 2, 3, 5, 6, 9, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 9, 12, 13, },
  { 4, 5, 6, 9, 12, 13, },
  { 0, 4, 5, 6, 9, 12, 13, },
  { 1, 4, 5, 6, 9, 12, 13, },
  { 0, 1, 4, 5, 6, 9, 12, 13, },
  { 2, 4, 5, 6, 9, 12, 13, },
  { 0, 2, 4, 5, 6, 9, 12, 13, },
  { 1, 2, 4, 5, 6, 9, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 9, 12, 13, },
  { 3, 4, 5, 6, 9, 12, 13, },
  { 0, 3, 4, 5, 6, 9, 12, 13, },
  { 1, 3, 4, 5, 6, 9, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 9, 12, 13, },
  { 2, 3, 4, 5, 6, 9, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 9, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 12, 13, },
  { 7, 9, 12, 13, },
  { 0, 7, 9, 12, 13, },
  { 1, 7, 9, 12, 13, },
  { 0, 1, 7, 9, 12, 13, },
  { 2, 7, 9, 12, 13, },
  { 0, 2, 7, 9, 12, 13, },
  { 1, 2, 7, 9, 12, 13, },
  { 0, 1, 2, 7, 9, 12, 13, },
  { 3, 7, 9, 12, 13, },
  { 0, 3, 7, 9, 12, 13, },
  { 1, 3, 7, 9, 12, 13, },
  { 0, 1, 3, 7, 9, 12, 13, },
  { 2, 3, 7, 9, 12, 13, },
  { 0, 2, 3, 7, 9, 12, 13, },
  { 1, 2, 3, 7, 9, 12, 13, },
  { 0, 1, 2, 3, 7, 9, 12, 13, },
  { 4, 7, 9, 12, 13, },
  { 0, 4, 7, 9, 12, 13, },
  { 1, 4, 7, 9, 12, 13, },
  { 0, 1, 4, 7, 9, 12, 13, },
  { 2, 4, 7, 9, 12, 13, },
  { 0, 2, 4, 7, 9, 12, 13, },
  { 1, 2, 4, 7, 9, 12, 13, },
  { 0, 1, 2, 4, 7, 9, 12, 13, },
  { 3, 4, 7, 9, 12, 13, },
  { 0, 3, 4, 7, 9, 12, 13, },
  { 1, 3, 4, 7, 9, 12, 13, },
  { 0, 1, 3, 4, 7, 9, 12, 13, },
  { 2, 3, 4, 7, 9, 12, 13, },
  { 0, 2, 3, 4, 7, 9, 12, 13, },
  { 1, 2, 3, 4, 7, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 9, 12, 13, },
  { 5, 7, 9, 12, 13, },
  { 0, 5, 7, 9, 12, 13, },
  { 1, 5, 7, 9, 12, 13, },
  { 0, 1, 5, 7, 9, 12, 13, },
  { 2, 5, 7, 9, 12, 13, },
  { 0, 2, 5, 7, 9, 12, 13, },
  { 1, 2, 5, 7, 9, 12, 13, },
  { 0, 1, 2, 5, 7, 9, 12, 13, },
  { 3, 5, 7, 9, 12, 13, },
  { 0, 3, 5, 7, 9, 12, 13, },
  { 1, 3, 5, 7, 9, 12, 13, },
  { 0, 1, 3, 5, 7, 9, 12, 13, },
  { 2, 3, 5, 7, 9, 12, 13, },
  { 0, 2, 3, 5, 7, 9, 12, 13, },
  { 1, 2, 3, 5, 7, 9, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 9, 12, 13, },
  { 4, 5, 7, 9, 12, 13, },
  { 0, 4, 5, 7, 9, 12, 13, },
  { 1, 4, 5, 7, 9, 12, 13, },
  { 0, 1, 4, 5, 7, 9, 12, 13, },
  { 2, 4, 5, 7, 9, 12, 13, },
  { 0, 2, 4, 5, 7, 9, 12, 13, },
  { 1, 2, 4, 5, 7, 9, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 9, 12, 13, },
  { 3, 4, 5, 7, 9, 12, 13, },
  { 0, 3, 4, 5, 7, 9, 12, 13, },
  { 1, 3, 4, 5, 7, 9, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 9, 12, 13, },
  { 2, 3, 4, 5, 7, 9, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 9, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 12, 13, },
  { 6, 7, 9, 12, 13, },
  { 0, 6, 7, 9, 12, 13, },
  { 1, 6, 7, 9, 12, 13, },
  { 0, 1, 6, 7, 9, 12, 13, },
  { 2, 6, 7, 9, 12, 13, },
  { 0, 2, 6, 7, 9, 12, 13, },
  { 1, 2, 6, 7, 9, 12, 13, },
  { 0, 1, 2, 6, 7, 9, 12, 13, },
  { 3, 6, 7, 9, 12, 13, },
  { 0, 3, 6, 7, 9, 12, 13, },
  { 1, 3, 6, 7, 9, 12, 13, },
  { 0, 1, 3, 6, 7, 9, 12, 13, },
  { 2, 3, 6, 7, 9, 12, 13, },
  { 0, 2, 3, 6, 7, 9, 12, 13, },
  { 1, 2, 3, 6, 7, 9, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 9, 12, 13, },
  { 4, 6, 7, 9, 12, 13, },
  { 0, 4, 6, 7, 9, 12, 13, },
  { 1, 4, 6, 7, 9, 12, 13, },
  { 0, 1, 4, 6, 7, 9, 12, 13, },
  { 2, 4, 6, 7, 9, 12, 13, },
  { 0, 2, 4, 6, 7, 9, 12, 13, },
  { 1, 2, 4, 6, 7, 9, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 9, 12, 13, },
  { 3, 4, 6, 7, 9, 12, 13, },
  { 0, 3, 4, 6, 7, 9, 12, 13, },
  { 1, 3, 4, 6, 7, 9, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 9, 12, 13, },
  { 2, 3, 4, 6, 7, 9, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 9, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 12, 13, },
  { 5, 6, 7, 9, 12, 13, },
  { 0, 5, 6, 7, 9, 12, 13, },
  { 1, 5, 6, 7, 9, 12, 13, },
  { 0, 1, 5, 6, 7, 9, 12, 13, },
  { 2, 5, 6, 7, 9, 12, 13, },
  { 0, 2, 5, 6, 7, 9, 12, 13, },
  { 1, 2, 5, 6, 7, 9, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 9, 12, 13, },
  { 3, 5, 6, 7, 9, 12, 13, },
  { 0, 3, 5, 6, 7, 9, 12, 13, },
  { 1, 3, 5, 6, 7, 9, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 9, 12, 13, },
  { 2, 3, 5, 6, 7, 9, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 9, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 9, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 12, 13, },
  { 4, 5, 6, 7, 9, 12, 13, },
  { 0, 4, 5, 6, 7, 9, 12, 13, },
  { 1, 4, 5, 6, 7, 9, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 9, 12, 13, },
  { 2, 4, 5, 6, 7, 9, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 9, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 9, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 12, 13, },
  { 3, 4, 5, 6, 7, 9, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 9, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 9, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 9, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 12, 13, },
  { 8, 9, 12, 13, },
  { 0, 8, 9, 12, 13, },
  { 1, 8, 9, 12, 13, },
  { 0, 1, 8, 9, 12, 13, },
  { 2, 8, 9, 12, 13, },
  { 0, 2, 8, 9, 12, 13, },
  { 1, 2, 8, 9, 12, 13, },
  { 0, 1, 2, 8, 9, 12, 13, },
  { 3, 8, 9, 12, 13, },
  { 0, 3, 8, 9, 12, 13, },
  { 1, 3, 8, 9, 12, 13, },
  { 0, 1, 3, 8, 9, 12, 13, },
  { 2, 3, 8, 9, 12, 13, },
  { 0, 2, 3, 8, 9, 12, 13, },
  { 1, 2, 3, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 8, 9, 12, 13, },
  { 4, 8, 9, 12, 13, },
  { 0, 4, 8, 9, 12, 13, },
  { 1, 4, 8, 9, 12, 13, },
  { 0, 1, 4, 8, 9, 12, 13, },
  { 2, 4, 8, 9, 12, 13, },
  { 0, 2, 4, 8, 9, 12, 13, },
  { 1, 2, 4, 8, 9, 12, 13, },
  { 0, 1, 2, 4, 8, 9, 12, 13, },
  { 3, 4, 8, 9, 12, 13, },
  { 0, 3, 4, 8, 9, 12, 13, },
  { 1, 3, 4, 8, 9, 12, 13, },
  { 0, 1, 3, 4, 8, 9, 12, 13, },
  { 2, 3, 4, 8, 9, 12, 13, },
  { 0, 2, 3, 4, 8, 9, 12, 13, },
  { 1, 2, 3, 4, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 8, 9, 12, 13, },
  { 5, 8, 9, 12, 13, },
  { 0, 5, 8, 9, 12, 13, },
  { 1, 5, 8, 9, 12, 13, },
  { 0, 1, 5, 8, 9, 12, 13, },
  { 2, 5, 8, 9, 12, 13, },
  { 0, 2, 5, 8, 9, 12, 13, },
  { 1, 2, 5, 8, 9, 12, 13, },
  { 0, 1, 2, 5, 8, 9, 12, 13, },
  { 3, 5, 8, 9, 12, 13, },
  { 0, 3, 5, 8, 9, 12, 13, },
  { 1, 3, 5, 8, 9, 12, 13, },
  { 0, 1, 3, 5, 8, 9, 12, 13, },
  { 2, 3, 5, 8, 9, 12, 13, },
  { 0, 2, 3, 5, 8, 9, 12, 13, },
  { 1, 2, 3, 5, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 5, 8, 9, 12, 13, },
  { 4, 5, 8, 9, 12, 13, },
  { 0, 4, 5, 8, 9, 12, 13, },
  { 1, 4, 5, 8, 9, 12, 13, },
  { 0, 1, 4, 5, 8, 9, 12, 13, },
  { 2, 4, 5, 8, 9, 12, 13, },
  { 0, 2, 4, 5, 8, 9, 12, 13, },
  { 1, 2, 4, 5, 8, 9, 12, 13, },
  { 0, 1, 2, 4, 5, 8, 9, 12, 13, },
  { 3, 4, 5, 8, 9, 12, 13, },
  { 0, 3, 4, 5, 8, 9, 12, 13, },
  { 1, 3, 4, 5, 8, 9, 12, 13, },
  { 0, 1, 3, 4, 5, 8, 9, 12, 13, },
  { 2, 3, 4, 5, 8, 9, 12, 13, },
  { 0, 2, 3, 4, 5, 8, 9, 12, 13, },
  { 1, 2, 3, 4, 5, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 12, 13, },
  { 6, 8, 9, 12, 13, },
  { 0, 6, 8, 9, 12, 13, },
  { 1, 6, 8, 9, 12, 13, },
  { 0, 1, 6, 8, 9, 12, 13, },
  { 2, 6, 8, 9, 12, 13, },
  { 0, 2, 6, 8, 9, 12, 13, },
  { 1, 2, 6, 8, 9, 12, 13, },
  { 0, 1, 2, 6, 8, 9, 12, 13, },
  { 3, 6, 8, 9, 12, 13, },
  { 0, 3, 6, 8, 9, 12, 13, },
  { 1, 3, 6, 8, 9, 12, 13, },
  { 0, 1, 3, 6, 8, 9, 12, 13, },
  { 2, 3, 6, 8, 9, 12, 13, },
  { 0, 2, 3, 6, 8, 9, 12, 13, },
  { 1, 2, 3, 6, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 6, 8, 9, 12, 13, },
  { 4, 6, 8, 9, 12, 13, },
  { 0, 4, 6, 8, 9, 12, 13, },
  { 1, 4, 6, 8, 9, 12, 13, },
  { 0, 1, 4, 6, 8, 9, 12, 13, },
  { 2, 4, 6, 8, 9, 12, 13, },
  { 0, 2, 4, 6, 8, 9, 12, 13, },
  { 1, 2, 4, 6, 8, 9, 12, 13, },
  { 0, 1, 2, 4, 6, 8, 9, 12, 13, },
  { 3, 4, 6, 8, 9, 12, 13, },
  { 0, 3, 4, 6, 8, 9, 12, 13, },
  { 1, 3, 4, 6, 8, 9, 12, 13, },
  { 0, 1, 3, 4, 6, 8, 9, 12, 13, },
  { 2, 3, 4, 6, 8, 9, 12, 13, },
  { 0, 2, 3, 4, 6, 8, 9, 12, 13, },
  { 1, 2, 3, 4, 6, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 12, 13, },
  { 5, 6, 8, 9, 12, 13, },
  { 0, 5, 6, 8, 9, 12, 13, },
  { 1, 5, 6, 8, 9, 12, 13, },
  { 0, 1, 5, 6, 8, 9, 12, 13, },
  { 2, 5, 6, 8, 9, 12, 13, },
  { 0, 2, 5, 6, 8, 9, 12, 13, },
  { 1, 2, 5, 6, 8, 9, 12, 13, },
  { 0, 1, 2, 5, 6, 8, 9, 12, 13, },
  { 3, 5, 6, 8, 9, 12, 13, },
  { 0, 3, 5, 6, 8, 9, 12, 13, },
  { 1, 3, 5, 6, 8, 9, 12, 13, },
  { 0, 1, 3, 5, 6, 8, 9, 12, 13, },
  { 2, 3, 5, 6, 8, 9, 12, 13, },
  { 0, 2, 3, 5, 6, 8, 9, 12, 13, },
  { 1, 2, 3, 5, 6, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 12, 13, },
  { 4, 5, 6, 8, 9, 12, 13, },
  { 0, 4, 5, 6, 8, 9, 12, 13, },
  { 1, 4, 5, 6, 8, 9, 12, 13, },
  { 0, 1, 4, 5, 6, 8, 9, 12, 13, },
  { 2, 4, 5, 6, 8, 9, 12, 13, },
  { 0, 2, 4, 5, 6, 8, 9, 12, 13, },
  { 1, 2, 4, 5, 6, 8, 9, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 12, 13, },
  { 3, 4, 5, 6, 8, 9, 12, 13, },
  { 0, 3, 4, 5, 6, 8, 9, 12, 13, },
  { 1, 3, 4, 5, 6, 8, 9, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 12, 13, },
  { 2, 3, 4, 5, 6, 8, 9, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 12, 13, },
  { 7, 8, 9, 12, 13, },
  { 0, 7, 8, 9, 12, 13, },
  { 1, 7, 8, 9, 12, 13, },
  { 0, 1, 7, 8, 9, 12, 13, },
  { 2, 7, 8, 9, 12, 13, },
  { 0, 2, 7, 8, 9, 12, 13, },
  { 1, 2, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 7, 8, 9, 12, 13, },
  { 3, 7, 8, 9, 12, 13, },
  { 0, 3, 7, 8, 9, 12, 13, },
  { 1, 3, 7, 8, 9, 12, 13, },
  { 0, 1, 3, 7, 8, 9, 12, 13, },
  { 2, 3, 7, 8, 9, 12, 13, },
  { 0, 2, 3, 7, 8, 9, 12, 13, },
  { 1, 2, 3, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 7, 8, 9, 12, 13, },
  { 4, 7, 8, 9, 12, 13, },
  { 0, 4, 7, 8, 9, 12, 13, },
  { 1, 4, 7, 8, 9, 12, 13, },
  { 0, 1, 4, 7, 8, 9, 12, 13, },
  { 2, 4, 7, 8, 9, 12, 13, },
  { 0, 2, 4, 7, 8, 9, 12, 13, },
  { 1, 2, 4, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 4, 7, 8, 9, 12, 13, },
  { 3, 4, 7, 8, 9, 12, 13, },
  { 0, 3, 4, 7, 8, 9, 12, 13, },
  { 1, 3, 4, 7, 8, 9, 12, 13, },
  { 0, 1, 3, 4, 7, 8, 9, 12, 13, },
  { 2, 3, 4, 7, 8, 9, 12, 13, },
  { 0, 2, 3, 4, 7, 8, 9, 12, 13, },
  { 1, 2, 3, 4, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 12, 13, },
  { 5, 7, 8, 9, 12, 13, },
  { 0, 5, 7, 8, 9, 12, 13, },
  { 1, 5, 7, 8, 9, 12, 13, },
  { 0, 1, 5, 7, 8, 9, 12, 13, },
  { 2, 5, 7, 8, 9, 12, 13, },
  { 0, 2, 5, 7, 8, 9, 12, 13, },
  { 1, 2, 5, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 5, 7, 8, 9, 12, 13, },
  { 3, 5, 7, 8, 9, 12, 13, },
  { 0, 3, 5, 7, 8, 9, 12, 13, },
  { 1, 3, 5, 7, 8, 9, 12, 13, },
  { 0, 1, 3, 5, 7, 8, 9, 12, 13, },
  { 2, 3, 5, 7, 8, 9, 12, 13, },
  { 0, 2, 3, 5, 7, 8, 9, 12, 13, },
  { 1, 2, 3, 5, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 12, 13, },
  { 4, 5, 7, 8, 9, 12, 13, },
  { 0, 4, 5, 7, 8, 9, 12, 13, },
  { 1, 4, 5, 7, 8, 9, 12, 13, },
  { 0, 1, 4, 5, 7, 8, 9, 12, 13, },
  { 2, 4, 5, 7, 8, 9, 12, 13, },
  { 0, 2, 4, 5, 7, 8, 9, 12, 13, },
  { 1, 2, 4, 5, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 12, 13, },
  { 3, 4, 5, 7, 8, 9, 12, 13, },
  { 0, 3, 4, 5, 7, 8, 9, 12, 13, },
  { 1, 3, 4, 5, 7, 8, 9, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 12, 13, },
  { 2, 3, 4, 5, 7, 8, 9, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 12, 13, },
  { 6, 7, 8, 9, 12, 13, },
  { 0, 6, 7, 8, 9, 12, 13, },
  { 1, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 6, 7, 8, 9, 12, 13, },
  { 2, 6, 7, 8, 9, 12, 13, },
  { 0, 2, 6, 7, 8, 9, 12, 13, },
  { 1, 2, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 6, 7, 8, 9, 12, 13, },
  { 3, 6, 7, 8, 9, 12, 13, },
  { 0, 3, 6, 7, 8, 9, 12, 13, },
  { 1, 3, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 3, 6, 7, 8, 9, 12, 13, },
  { 2, 3, 6, 7, 8, 9, 12, 13, },
  { 0, 2, 3, 6, 7, 8, 9, 12, 13, },
  { 1, 2, 3, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 12, 13, },
  { 4, 6, 7, 8, 9, 12, 13, },
  { 0, 4, 6, 7, 8, 9, 12, 13, },
  { 1, 4, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 4, 6, 7, 8, 9, 12, 13, },
  { 2, 4, 6, 7, 8, 9, 12, 13, },
  { 0, 2, 4, 6, 7, 8, 9, 12, 13, },
  { 1, 2, 4, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 12, 13, },
  { 3, 4, 6, 7, 8, 9, 12, 13, },
  { 0, 3, 4, 6, 7, 8, 9, 12, 13, },
  { 1, 3, 4, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 12, 13, },
  { 2, 3, 4, 6, 7, 8, 9, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 12, 13, },
  { 5, 6, 7, 8, 9, 12, 13, },
  { 0, 5, 6, 7, 8, 9, 12, 13, },
  { 1, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 5, 6, 7, 8, 9, 12, 13, },
  { 2, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 2, 5, 6, 7, 8, 9, 12, 13, },
  { 1, 2, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 12, 13, },
  { 3, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 3, 5, 6, 7, 8, 9, 12, 13, },
  { 1, 3, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 12, 13, },
  { 2, 3, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 12, 13, },
  { 4, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 1, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 2, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 3, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, },
  { 10, 12, 13, },
  { 0, 10, 12, 13, },
  { 1, 10, 12, 13, },
  { 0, 1, 10, 12, 13, },
  { 2, 10, 12, 13, },
  { 0, 2, 10, 12, 13, },
  { 1, 2, 10, 12, 13, },
  { 0, 1, 2, 10, 12, 13, },
  { 3, 10, 12, 13, },
  { 0, 3, 10, 12, 13, },
  { 1, 3, 10, 12, 13, },
  { 0, 1, 3, 10, 12, 13, },
  { 2, 3, 10, 12, 13, },
  { 0, 2, 3, 10, 12, 13, },
  { 1, 2, 3, 10, 12, 13, },
  { 0, 1, 2, 3, 10, 12, 13, },
  { 4, 10, 12, 13, },
  { 0, 4, 10, 12, 13, },
  { 1, 4, 10, 12, 13, },
  { 0, 1, 4, 10, 12, 13, },
  { 2, 4, 10, 12, 13, },
  { 0, 2, 4, 10, 12, 13, },
  { 1, 2, 4, 10, 12, 13, },
  { 0, 1, 2, 4, 10, 12, 13, },
  { 3, 4, 10, 12, 13, },
  { 0, 3, 4, 10, 12, 13, },
  { 1, 3, 4, 10, 12, 13, },
  { 0, 1, 3, 4, 10, 12, 13, },
  { 2, 3, 4, 10, 12, 13, },
  { 0, 2, 3, 4, 10, 12, 13, },
  { 1, 2, 3, 4, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 10, 12, 13, },
  { 5, 10, 12, 13, },
  { 0, 5, 10, 12, 13, },
  { 1, 5, 10, 12, 13, },
  { 0, 1, 5, 10, 12, 13, },
  { 2, 5, 10, 12, 13, },
  { 0, 2, 5, 10, 12, 13, },
  { 1, 2, 5, 10, 12, 13, },
  { 0, 1, 2, 5, 10, 12, 13, },
  { 3, 5, 10, 12, 13, },
  { 0, 3, 5, 10, 12, 13, },
  { 1, 3, 5, 10, 12, 13, },
  { 0, 1, 3, 5, 10, 12, 13, },
  { 2, 3, 5, 10, 12, 13, },
  { 0, 2, 3, 5, 10, 12, 13, },
  { 1, 2, 3, 5, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 10, 12, 13, },
  { 4, 5, 10, 12, 13, },
  { 0, 4, 5, 10, 12, 13, },
  { 1, 4, 5, 10, 12, 13, },
  { 0, 1, 4, 5, 10, 12, 13, },
  { 2, 4, 5, 10, 12, 13, },
  { 0, 2, 4, 5, 10, 12, 13, },
  { 1, 2, 4, 5, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 10, 12, 13, },
  { 3, 4, 5, 10, 12, 13, },
  { 0, 3, 4, 5, 10, 12, 13, },
  { 1, 3, 4, 5, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 10, 12, 13, },
  { 2, 3, 4, 5, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 10, 12, 13, },
  { 6, 10, 12, 13, },
  { 0, 6, 10, 12, 13, },
  { 1, 6, 10, 12, 13, },
  { 0, 1, 6, 10, 12, 13, },
  { 2, 6, 10, 12, 13, },
  { 0, 2, 6, 10, 12, 13, },
  { 1, 2, 6, 10, 12, 13, },
  { 0, 1, 2, 6, 10, 12, 13, },
  { 3, 6, 10, 12, 13, },
  { 0, 3, 6, 10, 12, 13, },
  { 1, 3, 6, 10, 12, 13, },
  { 0, 1, 3, 6, 10, 12, 13, },
  { 2, 3, 6, 10, 12, 13, },
  { 0, 2, 3, 6, 10, 12, 13, },
  { 1, 2, 3, 6, 10, 12, 13, },
  { 0, 1, 2, 3, 6, 10, 12, 13, },
  { 4, 6, 10, 12, 13, },
  { 0, 4, 6, 10, 12, 13, },
  { 1, 4, 6, 10, 12, 13, },
  { 0, 1, 4, 6, 10, 12, 13, },
  { 2, 4, 6, 10, 12, 13, },
  { 0, 2, 4, 6, 10, 12, 13, },
  { 1, 2, 4, 6, 10, 12, 13, },
  { 0, 1, 2, 4, 6, 10, 12, 13, },
  { 3, 4, 6, 10, 12, 13, },
  { 0, 3, 4, 6, 10, 12, 13, },
  { 1, 3, 4, 6, 10, 12, 13, },
  { 0, 1, 3, 4, 6, 10, 12, 13, },
  { 2, 3, 4, 6, 10, 12, 13, },
  { 0, 2, 3, 4, 6, 10, 12, 13, },
  { 1, 2, 3, 4, 6, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 10, 12, 13, },
  { 5, 6, 10, 12, 13, },
  { 0, 5, 6, 10, 12, 13, },
  { 1, 5, 6, 10, 12, 13, },
  { 0, 1, 5, 6, 10, 12, 13, },
  { 2, 5, 6, 10, 12, 13, },
  { 0, 2, 5, 6, 10, 12, 13, },
  { 1, 2, 5, 6, 10, 12, 13, },
  { 0, 1, 2, 5, 6, 10, 12, 13, },
  { 3, 5, 6, 10, 12, 13, },
  { 0, 3, 5, 6, 10, 12, 13, },
  { 1, 3, 5, 6, 10, 12, 13, },
  { 0, 1, 3, 5, 6, 10, 12, 13, },
  { 2, 3, 5, 6, 10, 12, 13, },
  { 0, 2, 3, 5, 6, 10, 12, 13, },
  { 1, 2, 3, 5, 6, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 10, 12, 13, },
  { 4, 5, 6, 10, 12, 13, },
  { 0, 4, 5, 6, 10, 12, 13, },
  { 1, 4, 5, 6, 10, 12, 13, },
  { 0, 1, 4, 5, 6, 10, 12, 13, },
  { 2, 4, 5, 6, 10, 12, 13, },
  { 0, 2, 4, 5, 6, 10, 12, 13, },
  { 1, 2, 4, 5, 6, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 10, 12, 13, },
  { 3, 4, 5, 6, 10, 12, 13, },
  { 0, 3, 4, 5, 6, 10, 12, 13, },
  { 1, 3, 4, 5, 6, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 10, 12, 13, },
  { 2, 3, 4, 5, 6, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 12, 13, },
  { 7, 10, 12, 13, },
  { 0, 7, 10, 12, 13, },
  { 1, 7, 10, 12, 13, },
  { 0, 1, 7, 10, 12, 13, },
  { 2, 7, 10, 12, 13, },
  { 0, 2, 7, 10, 12, 13, },
  { 1, 2, 7, 10, 12, 13, },
  { 0, 1, 2, 7, 10, 12, 13, },
  { 3, 7, 10, 12, 13, },
  { 0, 3, 7, 10, 12, 13, },
  { 1, 3, 7, 10, 12, 13, },
  { 0, 1, 3, 7, 10, 12, 13, },
  { 2, 3, 7, 10, 12, 13, },
  { 0, 2, 3, 7, 10, 12, 13, },
  { 1, 2, 3, 7, 10, 12, 13, },
  { 0, 1, 2, 3, 7, 10, 12, 13, },
  { 4, 7, 10, 12, 13, },
  { 0, 4, 7, 10, 12, 13, },
  { 1, 4, 7, 10, 12, 13, },
  { 0, 1, 4, 7, 10, 12, 13, },
  { 2, 4, 7, 10, 12, 13, },
  { 0, 2, 4, 7, 10, 12, 13, },
  { 1, 2, 4, 7, 10, 12, 13, },
  { 0, 1, 2, 4, 7, 10, 12, 13, },
  { 3, 4, 7, 10, 12, 13, },
  { 0, 3, 4, 7, 10, 12, 13, },
  { 1, 3, 4, 7, 10, 12, 13, },
  { 0, 1, 3, 4, 7, 10, 12, 13, },
  { 2, 3, 4, 7, 10, 12, 13, },
  { 0, 2, 3, 4, 7, 10, 12, 13, },
  { 1, 2, 3, 4, 7, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 10, 12, 13, },
  { 5, 7, 10, 12, 13, },
  { 0, 5, 7, 10, 12, 13, },
  { 1, 5, 7, 10, 12, 13, },
  { 0, 1, 5, 7, 10, 12, 13, },
  { 2, 5, 7, 10, 12, 13, },
  { 0, 2, 5, 7, 10, 12, 13, },
  { 1, 2, 5, 7, 10, 12, 13, },
  { 0, 1, 2, 5, 7, 10, 12, 13, },
  { 3, 5, 7, 10, 12, 13, },
  { 0, 3, 5, 7, 10, 12, 13, },
  { 1, 3, 5, 7, 10, 12, 13, },
  { 0, 1, 3, 5, 7, 10, 12, 13, },
  { 2, 3, 5, 7, 10, 12, 13, },
  { 0, 2, 3, 5, 7, 10, 12, 13, },
  { 1, 2, 3, 5, 7, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 10, 12, 13, },
  { 4, 5, 7, 10, 12, 13, },
  { 0, 4, 5, 7, 10, 12, 13, },
  { 1, 4, 5, 7, 10, 12, 13, },
  { 0, 1, 4, 5, 7, 10, 12, 13, },
  { 2, 4, 5, 7, 10, 12, 13, },
  { 0, 2, 4, 5, 7, 10, 12, 13, },
  { 1, 2, 4, 5, 7, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 10, 12, 13, },
  { 3, 4, 5, 7, 10, 12, 13, },
  { 0, 3, 4, 5, 7, 10, 12, 13, },
  { 1, 3, 4, 5, 7, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 10, 12, 13, },
  { 2, 3, 4, 5, 7, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 12, 13, },
  { 6, 7, 10, 12, 13, },
  { 0, 6, 7, 10, 12, 13, },
  { 1, 6, 7, 10, 12, 13, },
  { 0, 1, 6, 7, 10, 12, 13, },
  { 2, 6, 7, 10, 12, 13, },
  { 0, 2, 6, 7, 10, 12, 13, },
  { 1, 2, 6, 7, 10, 12, 13, },
  { 0, 1, 2, 6, 7, 10, 12, 13, },
  { 3, 6, 7, 10, 12, 13, },
  { 0, 3, 6, 7, 10, 12, 13, },
  { 1, 3, 6, 7, 10, 12, 13, },
  { 0, 1, 3, 6, 7, 10, 12, 13, },
  { 2, 3, 6, 7, 10, 12, 13, },
  { 0, 2, 3, 6, 7, 10, 12, 13, },
  { 1, 2, 3, 6, 7, 10, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 10, 12, 13, },
  { 4, 6, 7, 10, 12, 13, },
  { 0, 4, 6, 7, 10, 12, 13, },
  { 1, 4, 6, 7, 10, 12, 13, },
  { 0, 1, 4, 6, 7, 10, 12, 13, },
  { 2, 4, 6, 7, 10, 12, 13, },
  { 0, 2, 4, 6, 7, 10, 12, 13, },
  { 1, 2, 4, 6, 7, 10, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 10, 12, 13, },
  { 3, 4, 6, 7, 10, 12, 13, },
  { 0, 3, 4, 6, 7, 10, 12, 13, },
  { 1, 3, 4, 6, 7, 10, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 10, 12, 13, },
  { 2, 3, 4, 6, 7, 10, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 10, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 12, 13, },
  { 5, 6, 7, 10, 12, 13, },
  { 0, 5, 6, 7, 10, 12, 13, },
  { 1, 5, 6, 7, 10, 12, 13, },
  { 0, 1, 5, 6, 7, 10, 12, 13, },
  { 2, 5, 6, 7, 10, 12, 13, },
  { 0, 2, 5, 6, 7, 10, 12, 13, },
  { 1, 2, 5, 6, 7, 10, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 10, 12, 13, },
  { 3, 5, 6, 7, 10, 12, 13, },
  { 0, 3, 5, 6, 7, 10, 12, 13, },
  { 1, 3, 5, 6, 7, 10, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 10, 12, 13, },
  { 2, 3, 5, 6, 7, 10, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 10, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 12, 13, },
  { 4, 5, 6, 7, 10, 12, 13, },
  { 0, 4, 5, 6, 7, 10, 12, 13, },
  { 1, 4, 5, 6, 7, 10, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 10, 12, 13, },
  { 2, 4, 5, 6, 7, 10, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 10, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 12, 13, },
  { 3, 4, 5, 6, 7, 10, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 10, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 12, 13, },
  { 8, 10, 12, 13, },
  { 0, 8, 10, 12, 13, },
  { 1, 8, 10, 12, 13, },
  { 0, 1, 8, 10, 12, 13, },
  { 2, 8, 10, 12, 13, },
  { 0, 2, 8, 10, 12, 13, },
  { 1, 2, 8, 10, 12, 13, },
  { 0, 1, 2, 8, 10, 12, 13, },
  { 3, 8, 10, 12, 13, },
  { 0, 3, 8, 10, 12, 13, },
  { 1, 3, 8, 10, 12, 13, },
  { 0, 1, 3, 8, 10, 12, 13, },
  { 2, 3, 8, 10, 12, 13, },
  { 0, 2, 3, 8, 10, 12, 13, },
  { 1, 2, 3, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 8, 10, 12, 13, },
  { 4, 8, 10, 12, 13, },
  { 0, 4, 8, 10, 12, 13, },
  { 1, 4, 8, 10, 12, 13, },
  { 0, 1, 4, 8, 10, 12, 13, },
  { 2, 4, 8, 10, 12, 13, },
  { 0, 2, 4, 8, 10, 12, 13, },
  { 1, 2, 4, 8, 10, 12, 13, },
  { 0, 1, 2, 4, 8, 10, 12, 13, },
  { 3, 4, 8, 10, 12, 13, },
  { 0, 3, 4, 8, 10, 12, 13, },
  { 1, 3, 4, 8, 10, 12, 13, },
  { 0, 1, 3, 4, 8, 10, 12, 13, },
  { 2, 3, 4, 8, 10, 12, 13, },
  { 0, 2, 3, 4, 8, 10, 12, 13, },
  { 1, 2, 3, 4, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 8, 10, 12, 13, },
  { 5, 8, 10, 12, 13, },
  { 0, 5, 8, 10, 12, 13, },
  { 1, 5, 8, 10, 12, 13, },
  { 0, 1, 5, 8, 10, 12, 13, },
  { 2, 5, 8, 10, 12, 13, },
  { 0, 2, 5, 8, 10, 12, 13, },
  { 1, 2, 5, 8, 10, 12, 13, },
  { 0, 1, 2, 5, 8, 10, 12, 13, },
  { 3, 5, 8, 10, 12, 13, },
  { 0, 3, 5, 8, 10, 12, 13, },
  { 1, 3, 5, 8, 10, 12, 13, },
  { 0, 1, 3, 5, 8, 10, 12, 13, },
  { 2, 3, 5, 8, 10, 12, 13, },
  { 0, 2, 3, 5, 8, 10, 12, 13, },
  { 1, 2, 3, 5, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 8, 10, 12, 13, },
  { 4, 5, 8, 10, 12, 13, },
  { 0, 4, 5, 8, 10, 12, 13, },
  { 1, 4, 5, 8, 10, 12, 13, },
  { 0, 1, 4, 5, 8, 10, 12, 13, },
  { 2, 4, 5, 8, 10, 12, 13, },
  { 0, 2, 4, 5, 8, 10, 12, 13, },
  { 1, 2, 4, 5, 8, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 8, 10, 12, 13, },
  { 3, 4, 5, 8, 10, 12, 13, },
  { 0, 3, 4, 5, 8, 10, 12, 13, },
  { 1, 3, 4, 5, 8, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 8, 10, 12, 13, },
  { 2, 3, 4, 5, 8, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 8, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 12, 13, },
  { 6, 8, 10, 12, 13, },
  { 0, 6, 8, 10, 12, 13, },
  { 1, 6, 8, 10, 12, 13, },
  { 0, 1, 6, 8, 10, 12, 13, },
  { 2, 6, 8, 10, 12, 13, },
  { 0, 2, 6, 8, 10, 12, 13, },
  { 1, 2, 6, 8, 10, 12, 13, },
  { 0, 1, 2, 6, 8, 10, 12, 13, },
  { 3, 6, 8, 10, 12, 13, },
  { 0, 3, 6, 8, 10, 12, 13, },
  { 1, 3, 6, 8, 10, 12, 13, },
  { 0, 1, 3, 6, 8, 10, 12, 13, },
  { 2, 3, 6, 8, 10, 12, 13, },
  { 0, 2, 3, 6, 8, 10, 12, 13, },
  { 1, 2, 3, 6, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 6, 8, 10, 12, 13, },
  { 4, 6, 8, 10, 12, 13, },
  { 0, 4, 6, 8, 10, 12, 13, },
  { 1, 4, 6, 8, 10, 12, 13, },
  { 0, 1, 4, 6, 8, 10, 12, 13, },
  { 2, 4, 6, 8, 10, 12, 13, },
  { 0, 2, 4, 6, 8, 10, 12, 13, },
  { 1, 2, 4, 6, 8, 10, 12, 13, },
  { 0, 1, 2, 4, 6, 8, 10, 12, 13, },
  { 3, 4, 6, 8, 10, 12, 13, },
  { 0, 3, 4, 6, 8, 10, 12, 13, },
  { 1, 3, 4, 6, 8, 10, 12, 13, },
  { 0, 1, 3, 4, 6, 8, 10, 12, 13, },
  { 2, 3, 4, 6, 8, 10, 12, 13, },
  { 0, 2, 3, 4, 6, 8, 10, 12, 13, },
  { 1, 2, 3, 4, 6, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 12, 13, },
  { 5, 6, 8, 10, 12, 13, },
  { 0, 5, 6, 8, 10, 12, 13, },
  { 1, 5, 6, 8, 10, 12, 13, },
  { 0, 1, 5, 6, 8, 10, 12, 13, },
  { 2, 5, 6, 8, 10, 12, 13, },
  { 0, 2, 5, 6, 8, 10, 12, 13, },
  { 1, 2, 5, 6, 8, 10, 12, 13, },
  { 0, 1, 2, 5, 6, 8, 10, 12, 13, },
  { 3, 5, 6, 8, 10, 12, 13, },
  { 0, 3, 5, 6, 8, 10, 12, 13, },
  { 1, 3, 5, 6, 8, 10, 12, 13, },
  { 0, 1, 3, 5, 6, 8, 10, 12, 13, },
  { 2, 3, 5, 6, 8, 10, 12, 13, },
  { 0, 2, 3, 5, 6, 8, 10, 12, 13, },
  { 1, 2, 3, 5, 6, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 12, 13, },
  { 4, 5, 6, 8, 10, 12, 13, },
  { 0, 4, 5, 6, 8, 10, 12, 13, },
  { 1, 4, 5, 6, 8, 10, 12, 13, },
  { 0, 1, 4, 5, 6, 8, 10, 12, 13, },
  { 2, 4, 5, 6, 8, 10, 12, 13, },
  { 0, 2, 4, 5, 6, 8, 10, 12, 13, },
  { 1, 2, 4, 5, 6, 8, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 12, 13, },
  { 3, 4, 5, 6, 8, 10, 12, 13, },
  { 0, 3, 4, 5, 6, 8, 10, 12, 13, },
  { 1, 3, 4, 5, 6, 8, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 12, 13, },
  { 2, 3, 4, 5, 6, 8, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 12, 13, },
  { 7, 8, 10, 12, 13, },
  { 0, 7, 8, 10, 12, 13, },
  { 1, 7, 8, 10, 12, 13, },
  { 0, 1, 7, 8, 10, 12, 13, },
  { 2, 7, 8, 10, 12, 13, },
  { 0, 2, 7, 8, 10, 12, 13, },
  { 1, 2, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 7, 8, 10, 12, 13, },
  { 3, 7, 8, 10, 12, 13, },
  { 0, 3, 7, 8, 10, 12, 13, },
  { 1, 3, 7, 8, 10, 12, 13, },
  { 0, 1, 3, 7, 8, 10, 12, 13, },
  { 2, 3, 7, 8, 10, 12, 13, },
  { 0, 2, 3, 7, 8, 10, 12, 13, },
  { 1, 2, 3, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 7, 8, 10, 12, 13, },
  { 4, 7, 8, 10, 12, 13, },
  { 0, 4, 7, 8, 10, 12, 13, },
  { 1, 4, 7, 8, 10, 12, 13, },
  { 0, 1, 4, 7, 8, 10, 12, 13, },
  { 2, 4, 7, 8, 10, 12, 13, },
  { 0, 2, 4, 7, 8, 10, 12, 13, },
  { 1, 2, 4, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 4, 7, 8, 10, 12, 13, },
  { 3, 4, 7, 8, 10, 12, 13, },
  { 0, 3, 4, 7, 8, 10, 12, 13, },
  { 1, 3, 4, 7, 8, 10, 12, 13, },
  { 0, 1, 3, 4, 7, 8, 10, 12, 13, },
  { 2, 3, 4, 7, 8, 10, 12, 13, },
  { 0, 2, 3, 4, 7, 8, 10, 12, 13, },
  { 1, 2, 3, 4, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 12, 13, },
  { 5, 7, 8, 10, 12, 13, },
  { 0, 5, 7, 8, 10, 12, 13, },
  { 1, 5, 7, 8, 10, 12, 13, },
  { 0, 1, 5, 7, 8, 10, 12, 13, },
  { 2, 5, 7, 8, 10, 12, 13, },
  { 0, 2, 5, 7, 8, 10, 12, 13, },
  { 1, 2, 5, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 5, 7, 8, 10, 12, 13, },
  { 3, 5, 7, 8, 10, 12, 13, },
  { 0, 3, 5, 7, 8, 10, 12, 13, },
  { 1, 3, 5, 7, 8, 10, 12, 13, },
  { 0, 1, 3, 5, 7, 8, 10, 12, 13, },
  { 2, 3, 5, 7, 8, 10, 12, 13, },
  { 0, 2, 3, 5, 7, 8, 10, 12, 13, },
  { 1, 2, 3, 5, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 12, 13, },
  { 4, 5, 7, 8, 10, 12, 13, },
  { 0, 4, 5, 7, 8, 10, 12, 13, },
  { 1, 4, 5, 7, 8, 10, 12, 13, },
  { 0, 1, 4, 5, 7, 8, 10, 12, 13, },
  { 2, 4, 5, 7, 8, 10, 12, 13, },
  { 0, 2, 4, 5, 7, 8, 10, 12, 13, },
  { 1, 2, 4, 5, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 12, 13, },
  { 3, 4, 5, 7, 8, 10, 12, 13, },
  { 0, 3, 4, 5, 7, 8, 10, 12, 13, },
  { 1, 3, 4, 5, 7, 8, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 12, 13, },
  { 2, 3, 4, 5, 7, 8, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 13, },
  { 6, 7, 8, 10, 12, 13, },
  { 0, 6, 7, 8, 10, 12, 13, },
  { 1, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 6, 7, 8, 10, 12, 13, },
  { 2, 6, 7, 8, 10, 12, 13, },
  { 0, 2, 6, 7, 8, 10, 12, 13, },
  { 1, 2, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 6, 7, 8, 10, 12, 13, },
  { 3, 6, 7, 8, 10, 12, 13, },
  { 0, 3, 6, 7, 8, 10, 12, 13, },
  { 1, 3, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 3, 6, 7, 8, 10, 12, 13, },
  { 2, 3, 6, 7, 8, 10, 12, 13, },
  { 0, 2, 3, 6, 7, 8, 10, 12, 13, },
  { 1, 2, 3, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 12, 13, },
  { 4, 6, 7, 8, 10, 12, 13, },
  { 0, 4, 6, 7, 8, 10, 12, 13, },
  { 1, 4, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 4, 6, 7, 8, 10, 12, 13, },
  { 2, 4, 6, 7, 8, 10, 12, 13, },
  { 0, 2, 4, 6, 7, 8, 10, 12, 13, },
  { 1, 2, 4, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 12, 13, },
  { 3, 4, 6, 7, 8, 10, 12, 13, },
  { 0, 3, 4, 6, 7, 8, 10, 12, 13, },
  { 1, 3, 4, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 12, 13, },
  { 2, 3, 4, 6, 7, 8, 10, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 12, 13, },
  { 5, 6, 7, 8, 10, 12, 13, },
  { 0, 5, 6, 7, 8, 10, 12, 13, },
  { 1, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 5, 6, 7, 8, 10, 12, 13, },
  { 2, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 2, 5, 6, 7, 8, 10, 12, 13, },
  { 1, 2, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 12, 13, },
  { 3, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 3, 5, 6, 7, 8, 10, 12, 13, },
  { 1, 3, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 12, 13, },
  { 2, 3, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 12, 13, },
  { 4, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 1, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 2, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 3, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, },
  { 9, 10, 12, 13, },
  { 0, 9, 10, 12, 13, },
  { 1, 9, 10, 12, 13, },
  { 0, 1, 9, 10, 12, 13, },
  { 2, 9, 10, 12, 13, },
  { 0, 2, 9, 10, 12, 13, },
  { 1, 2, 9, 10, 12, 13, },
  { 0, 1, 2, 9, 10, 12, 13, },
  { 3, 9, 10, 12, 13, },
  { 0, 3, 9, 10, 12, 13, },
  { 1, 3, 9, 10, 12, 13, },
  { 0, 1, 3, 9, 10, 12, 13, },
  { 2, 3, 9, 10, 12, 13, },
  { 0, 2, 3, 9, 10, 12, 13, },
  { 1, 2, 3, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 9, 10, 12, 13, },
  { 4, 9, 10, 12, 13, },
  { 0, 4, 9, 10, 12, 13, },
  { 1, 4, 9, 10, 12, 13, },
  { 0, 1, 4, 9, 10, 12, 13, },
  { 2, 4, 9, 10, 12, 13, },
  { 0, 2, 4, 9, 10, 12, 13, },
  { 1, 2, 4, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 9, 10, 12, 13, },
  { 3, 4, 9, 10, 12, 13, },
  { 0, 3, 4, 9, 10, 12, 13, },
  { 1, 3, 4, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 9, 10, 12, 13, },
  { 2, 3, 4, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 9, 10, 12, 13, },
  { 5, 9, 10, 12, 13, },
  { 0, 5, 9, 10, 12, 13, },
  { 1, 5, 9, 10, 12, 13, },
  { 0, 1, 5, 9, 10, 12, 13, },
  { 2, 5, 9, 10, 12, 13, },
  { 0, 2, 5, 9, 10, 12, 13, },
  { 1, 2, 5, 9, 10, 12, 13, },
  { 0, 1, 2, 5, 9, 10, 12, 13, },
  { 3, 5, 9, 10, 12, 13, },
  { 0, 3, 5, 9, 10, 12, 13, },
  { 1, 3, 5, 9, 10, 12, 13, },
  { 0, 1, 3, 5, 9, 10, 12, 13, },
  { 2, 3, 5, 9, 10, 12, 13, },
  { 0, 2, 3, 5, 9, 10, 12, 13, },
  { 1, 2, 3, 5, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 9, 10, 12, 13, },
  { 4, 5, 9, 10, 12, 13, },
  { 0, 4, 5, 9, 10, 12, 13, },
  { 1, 4, 5, 9, 10, 12, 13, },
  { 0, 1, 4, 5, 9, 10, 12, 13, },
  { 2, 4, 5, 9, 10, 12, 13, },
  { 0, 2, 4, 5, 9, 10, 12, 13, },
  { 1, 2, 4, 5, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 9, 10, 12, 13, },
  { 3, 4, 5, 9, 10, 12, 13, },
  { 0, 3, 4, 5, 9, 10, 12, 13, },
  { 1, 3, 4, 5, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 9, 10, 12, 13, },
  { 2, 3, 4, 5, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 12, 13, },
  { 6, 9, 10, 12, 13, },
  { 0, 6, 9, 10, 12, 13, },
  { 1, 6, 9, 10, 12, 13, },
  { 0, 1, 6, 9, 10, 12, 13, },
  { 2, 6, 9, 10, 12, 13, },
  { 0, 2, 6, 9, 10, 12, 13, },
  { 1, 2, 6, 9, 10, 12, 13, },
  { 0, 1, 2, 6, 9, 10, 12, 13, },
  { 3, 6, 9, 10, 12, 13, },
  { 0, 3, 6, 9, 10, 12, 13, },
  { 1, 3, 6, 9, 10, 12, 13, },
  { 0, 1, 3, 6, 9, 10, 12, 13, },
  { 2, 3, 6, 9, 10, 12, 13, },
  { 0, 2, 3, 6, 9, 10, 12, 13, },
  { 1, 2, 3, 6, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 6, 9, 10, 12, 13, },
  { 4, 6, 9, 10, 12, 13, },
  { 0, 4, 6, 9, 10, 12, 13, },
  { 1, 4, 6, 9, 10, 12, 13, },
  { 0, 1, 4, 6, 9, 10, 12, 13, },
  { 2, 4, 6, 9, 10, 12, 13, },
  { 0, 2, 4, 6, 9, 10, 12, 13, },
  { 1, 2, 4, 6, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 6, 9, 10, 12, 13, },
  { 3, 4, 6, 9, 10, 12, 13, },
  { 0, 3, 4, 6, 9, 10, 12, 13, },
  { 1, 3, 4, 6, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 6, 9, 10, 12, 13, },
  { 2, 3, 4, 6, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 6, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 6, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 12, 13, },
  { 5, 6, 9, 10, 12, 13, },
  { 0, 5, 6, 9, 10, 12, 13, },
  { 1, 5, 6, 9, 10, 12, 13, },
  { 0, 1, 5, 6, 9, 10, 12, 13, },
  { 2, 5, 6, 9, 10, 12, 13, },
  { 0, 2, 5, 6, 9, 10, 12, 13, },
  { 1, 2, 5, 6, 9, 10, 12, 13, },
  { 0, 1, 2, 5, 6, 9, 10, 12, 13, },
  { 3, 5, 6, 9, 10, 12, 13, },
  { 0, 3, 5, 6, 9, 10, 12, 13, },
  { 1, 3, 5, 6, 9, 10, 12, 13, },
  { 0, 1, 3, 5, 6, 9, 10, 12, 13, },
  { 2, 3, 5, 6, 9, 10, 12, 13, },
  { 0, 2, 3, 5, 6, 9, 10, 12, 13, },
  { 1, 2, 3, 5, 6, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 12, 13, },
  { 4, 5, 6, 9, 10, 12, 13, },
  { 0, 4, 5, 6, 9, 10, 12, 13, },
  { 1, 4, 5, 6, 9, 10, 12, 13, },
  { 0, 1, 4, 5, 6, 9, 10, 12, 13, },
  { 2, 4, 5, 6, 9, 10, 12, 13, },
  { 0, 2, 4, 5, 6, 9, 10, 12, 13, },
  { 1, 2, 4, 5, 6, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 12, 13, },
  { 3, 4, 5, 6, 9, 10, 12, 13, },
  { 0, 3, 4, 5, 6, 9, 10, 12, 13, },
  { 1, 3, 4, 5, 6, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 12, 13, },
  { 2, 3, 4, 5, 6, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 12, 13, },
  { 7, 9, 10, 12, 13, },
  { 0, 7, 9, 10, 12, 13, },
  { 1, 7, 9, 10, 12, 13, },
  { 0, 1, 7, 9, 10, 12, 13, },
  { 2, 7, 9, 10, 12, 13, },
  { 0, 2, 7, 9, 10, 12, 13, },
  { 1, 2, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 7, 9, 10, 12, 13, },
  { 3, 7, 9, 10, 12, 13, },
  { 0, 3, 7, 9, 10, 12, 13, },
  { 1, 3, 7, 9, 10, 12, 13, },
  { 0, 1, 3, 7, 9, 10, 12, 13, },
  { 2, 3, 7, 9, 10, 12, 13, },
  { 0, 2, 3, 7, 9, 10, 12, 13, },
  { 1, 2, 3, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 7, 9, 10, 12, 13, },
  { 4, 7, 9, 10, 12, 13, },
  { 0, 4, 7, 9, 10, 12, 13, },
  { 1, 4, 7, 9, 10, 12, 13, },
  { 0, 1, 4, 7, 9, 10, 12, 13, },
  { 2, 4, 7, 9, 10, 12, 13, },
  { 0, 2, 4, 7, 9, 10, 12, 13, },
  { 1, 2, 4, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 7, 9, 10, 12, 13, },
  { 3, 4, 7, 9, 10, 12, 13, },
  { 0, 3, 4, 7, 9, 10, 12, 13, },
  { 1, 3, 4, 7, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 7, 9, 10, 12, 13, },
  { 2, 3, 4, 7, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 7, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 12, 13, },
  { 5, 7, 9, 10, 12, 13, },
  { 0, 5, 7, 9, 10, 12, 13, },
  { 1, 5, 7, 9, 10, 12, 13, },
  { 0, 1, 5, 7, 9, 10, 12, 13, },
  { 2, 5, 7, 9, 10, 12, 13, },
  { 0, 2, 5, 7, 9, 10, 12, 13, },
  { 1, 2, 5, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 5, 7, 9, 10, 12, 13, },
  { 3, 5, 7, 9, 10, 12, 13, },
  { 0, 3, 5, 7, 9, 10, 12, 13, },
  { 1, 3, 5, 7, 9, 10, 12, 13, },
  { 0, 1, 3, 5, 7, 9, 10, 12, 13, },
  { 2, 3, 5, 7, 9, 10, 12, 13, },
  { 0, 2, 3, 5, 7, 9, 10, 12, 13, },
  { 1, 2, 3, 5, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 12, 13, },
  { 4, 5, 7, 9, 10, 12, 13, },
  { 0, 4, 5, 7, 9, 10, 12, 13, },
  { 1, 4, 5, 7, 9, 10, 12, 13, },
  { 0, 1, 4, 5, 7, 9, 10, 12, 13, },
  { 2, 4, 5, 7, 9, 10, 12, 13, },
  { 0, 2, 4, 5, 7, 9, 10, 12, 13, },
  { 1, 2, 4, 5, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 12, 13, },
  { 3, 4, 5, 7, 9, 10, 12, 13, },
  { 0, 3, 4, 5, 7, 9, 10, 12, 13, },
  { 1, 3, 4, 5, 7, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 12, 13, },
  { 2, 3, 4, 5, 7, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 12, 13, },
  { 6, 7, 9, 10, 12, 13, },
  { 0, 6, 7, 9, 10, 12, 13, },
  { 1, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 6, 7, 9, 10, 12, 13, },
  { 2, 6, 7, 9, 10, 12, 13, },
  { 0, 2, 6, 7, 9, 10, 12, 13, },
  { 1, 2, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 6, 7, 9, 10, 12, 13, },
  { 3, 6, 7, 9, 10, 12, 13, },
  { 0, 3, 6, 7, 9, 10, 12, 13, },
  { 1, 3, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 3, 6, 7, 9, 10, 12, 13, },
  { 2, 3, 6, 7, 9, 10, 12, 13, },
  { 0, 2, 3, 6, 7, 9, 10, 12, 13, },
  { 1, 2, 3, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 12, 13, },
  { 4, 6, 7, 9, 10, 12, 13, },
  { 0, 4, 6, 7, 9, 10, 12, 13, },
  { 1, 4, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 4, 6, 7, 9, 10, 12, 13, },
  { 2, 4, 6, 7, 9, 10, 12, 13, },
  { 0, 2, 4, 6, 7, 9, 10, 12, 13, },
  { 1, 2, 4, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 12, 13, },
  { 3, 4, 6, 7, 9, 10, 12, 13, },
  { 0, 3, 4, 6, 7, 9, 10, 12, 13, },
  { 1, 3, 4, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 12, 13, },
  { 2, 3, 4, 6, 7, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 12, 13, },
  { 5, 6, 7, 9, 10, 12, 13, },
  { 0, 5, 6, 7, 9, 10, 12, 13, },
  { 1, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 5, 6, 7, 9, 10, 12, 13, },
  { 2, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 2, 5, 6, 7, 9, 10, 12, 13, },
  { 1, 2, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 12, 13, },
  { 3, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 3, 5, 6, 7, 9, 10, 12, 13, },
  { 1, 3, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 12, 13, },
  { 2, 3, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 12, 13, },
  { 4, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 1, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 2, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 3, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, },
  { 8, 9, 10, 12, 13, },
  { 0, 8, 9, 10, 12, 13, },
  { 1, 8, 9, 10, 12, 13, },
  { 0, 1, 8, 9, 10, 12, 13, },
  { 2, 8, 9, 10, 12, 13, },
  { 0, 2, 8, 9, 10, 12, 13, },
  { 1, 2, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 8, 9, 10, 12, 13, },
  { 3, 8, 9, 10, 12, 13, },
  { 0, 3, 8, 9, 10, 12, 13, },
  { 1, 3, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 8, 9, 10, 12, 13, },
  { 2, 3, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 8, 9, 10, 12, 13, },
  { 4, 8, 9, 10, 12, 13, },
  { 0, 4, 8, 9, 10, 12, 13, },
  { 1, 4, 8, 9, 10, 12, 13, },
  { 0, 1, 4, 8, 9, 10, 12, 13, },
  { 2, 4, 8, 9, 10, 12, 13, },
  { 0, 2, 4, 8, 9, 10, 12, 13, },
  { 1, 2, 4, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 8, 9, 10, 12, 13, },
  { 3, 4, 8, 9, 10, 12, 13, },
  { 0, 3, 4, 8, 9, 10, 12, 13, },
  { 1, 3, 4, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 8, 9, 10, 12, 13, },
  { 2, 3, 4, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 12, 13, },
  { 5, 8, 9, 10, 12, 13, },
  { 0, 5, 8, 9, 10, 12, 13, },
  { 1, 5, 8, 9, 10, 12, 13, },
  { 0, 1, 5, 8, 9, 10, 12, 13, },
  { 2, 5, 8, 9, 10, 12, 13, },
  { 0, 2, 5, 8, 9, 10, 12, 13, },
  { 1, 2, 5, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 5, 8, 9, 10, 12, 13, },
  { 3, 5, 8, 9, 10, 12, 13, },
  { 0, 3, 5, 8, 9, 10, 12, 13, },
  { 1, 3, 5, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 5, 8, 9, 10, 12, 13, },
  { 2, 3, 5, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 5, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 5, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 12, 13, },
  { 4, 5, 8, 9, 10, 12, 13, },
  { 0, 4, 5, 8, 9, 10, 12, 13, },
  { 1, 4, 5, 8, 9, 10, 12, 13, },
  { 0, 1, 4, 5, 8, 9, 10, 12, 13, },
  { 2, 4, 5, 8, 9, 10, 12, 13, },
  { 0, 2, 4, 5, 8, 9, 10, 12, 13, },
  { 1, 2, 4, 5, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 12, 13, },
  { 3, 4, 5, 8, 9, 10, 12, 13, },
  { 0, 3, 4, 5, 8, 9, 10, 12, 13, },
  { 1, 3, 4, 5, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 12, 13, },
  { 2, 3, 4, 5, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 12, 13, },
  { 6, 8, 9, 10, 12, 13, },
  { 0, 6, 8, 9, 10, 12, 13, },
  { 1, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 6, 8, 9, 10, 12, 13, },
  { 2, 6, 8, 9, 10, 12, 13, },
  { 0, 2, 6, 8, 9, 10, 12, 13, },
  { 1, 2, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 6, 8, 9, 10, 12, 13, },
  { 3, 6, 8, 9, 10, 12, 13, },
  { 0, 3, 6, 8, 9, 10, 12, 13, },
  { 1, 3, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 6, 8, 9, 10, 12, 13, },
  { 2, 3, 6, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 6, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 12, 13, },
  { 4, 6, 8, 9, 10, 12, 13, },
  { 0, 4, 6, 8, 9, 10, 12, 13, },
  { 1, 4, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 4, 6, 8, 9, 10, 12, 13, },
  { 2, 4, 6, 8, 9, 10, 12, 13, },
  { 0, 2, 4, 6, 8, 9, 10, 12, 13, },
  { 1, 2, 4, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 12, 13, },
  { 3, 4, 6, 8, 9, 10, 12, 13, },
  { 0, 3, 4, 6, 8, 9, 10, 12, 13, },
  { 1, 3, 4, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 12, 13, },
  { 2, 3, 4, 6, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 12, 13, },
  { 5, 6, 8, 9, 10, 12, 13, },
  { 0, 5, 6, 8, 9, 10, 12, 13, },
  { 1, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 5, 6, 8, 9, 10, 12, 13, },
  { 2, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 2, 5, 6, 8, 9, 10, 12, 13, },
  { 1, 2, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 12, 13, },
  { 3, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 3, 5, 6, 8, 9, 10, 12, 13, },
  { 1, 3, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 12, 13, },
  { 2, 3, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 12, 13, },
  { 4, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 1, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 2, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 3, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, },
  { 7, 8, 9, 10, 12, 13, },
  { 0, 7, 8, 9, 10, 12, 13, },
  { 1, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 7, 8, 9, 10, 12, 13, },
  { 2, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 7, 8, 9, 10, 12, 13, },
  { 3, 7, 8, 9, 10, 12, 13, },
  { 0, 3, 7, 8, 9, 10, 12, 13, },
  { 1, 3, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 7, 8, 9, 10, 12, 13, },
  { 2, 3, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 12, 13, },
  { 4, 7, 8, 9, 10, 12, 13, },
  { 0, 4, 7, 8, 9, 10, 12, 13, },
  { 1, 4, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 4, 7, 8, 9, 10, 12, 13, },
  { 2, 4, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 4, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 4, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 12, 13, },
  { 3, 4, 7, 8, 9, 10, 12, 13, },
  { 0, 3, 4, 7, 8, 9, 10, 12, 13, },
  { 1, 3, 4, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 12, 13, },
  { 2, 3, 4, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 12, 13, },
  { 5, 7, 8, 9, 10, 12, 13, },
  { 0, 5, 7, 8, 9, 10, 12, 13, },
  { 1, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 5, 7, 8, 9, 10, 12, 13, },
  { 2, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 5, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 12, 13, },
  { 3, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 3, 5, 7, 8, 9, 10, 12, 13, },
  { 1, 3, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 12, 13, },
  { 2, 3, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 12, 13, },
  { 4, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 1, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 2, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 3, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, },
  { 6, 7, 8, 9, 10, 12, 13, },
  { 0, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 6, 7, 8, 9, 10, 12, 13, },
  { 2, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 12, 13, },
  { 3, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 3, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 3, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 12, 13, },
  { 2, 3, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 12, 13, },
  { 4, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 2, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 3, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, },
  { 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 2, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 3, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, },
  { 11, 12, 13, },
  { 0, 11, 12, 13, },
  { 1, 11, 12, 13, },
  { 0, 1, 11, 12, 13, },
  { 2, 11, 12, 13, },
  { 0, 2, 11, 12, 13, },
  { 1, 2, 11, 12, 13, },
  { 0, 1, 2, 11, 12, 13, },
  { 3, 11, 12, 13, },
  { 0, 3, 11, 12, 13, },
  { 1, 3, 11, 12, 13, },
  { 0, 1, 3, 11, 12, 13, },
  { 2, 3, 11, 12, 13, },
  { 0, 2, 3, 11, 12, 13, },
  { 1, 2, 3, 11, 12, 13, },
  { 0, 1, 2, 3, 11, 12, 13, },
  { 4, 11, 12, 13, },
  { 0, 4, 11, 12, 13, },
  { 1, 4, 11, 12, 13, },
  { 0, 1, 4, 11, 12, 13, },
  { 2, 4, 11, 12, 13, },
  { 0, 2, 4, 11, 12, 13, },
  { 1, 2, 4, 11, 12, 13, },
  { 0, 1, 2, 4, 11, 12, 13, },
  { 3, 4, 11, 12, 13, },
  { 0, 3, 4, 11, 12, 13, },
  { 1, 3, 4, 11, 12, 13, },
  { 0, 1, 3, 4, 11, 12, 13, },
  { 2, 3, 4, 11, 12, 13, },
  { 0, 2, 3, 4, 11, 12, 13, },
  { 1, 2, 3, 4, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 11, 12, 13, },
  { 5, 11, 12, 13, },
  { 0, 5, 11, 12, 13, },
  { 1, 5, 11, 12, 13, },
  { 0, 1, 5, 11, 12, 13, },
  { 2, 5, 11, 12, 13, },
  { 0, 2, 5, 11, 12, 13, },
  { 1, 2, 5, 11, 12, 13, },
  { 0, 1, 2, 5, 11, 12, 13, },
  { 3, 5, 11, 12, 13, },
  { 0, 3, 5, 11, 12, 13, },
  { 1, 3, 5, 11, 12, 13, },
  { 0, 1, 3, 5, 11, 12, 13, },
  { 2, 3, 5, 11, 12, 13, },
  { 0, 2, 3, 5, 11, 12, 13, },
  { 1, 2, 3, 5, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 11, 12, 13, },
  { 4, 5, 11, 12, 13, },
  { 0, 4, 5, 11, 12, 13, },
  { 1, 4, 5, 11, 12, 13, },
  { 0, 1, 4, 5, 11, 12, 13, },
  { 2, 4, 5, 11, 12, 13, },
  { 0, 2, 4, 5, 11, 12, 13, },
  { 1, 2, 4, 5, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 11, 12, 13, },
  { 3, 4, 5, 11, 12, 13, },
  { 0, 3, 4, 5, 11, 12, 13, },
  { 1, 3, 4, 5, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 11, 12, 13, },
  { 2, 3, 4, 5, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 11, 12, 13, },
  { 6, 11, 12, 13, },
  { 0, 6, 11, 12, 13, },
  { 1, 6, 11, 12, 13, },
  { 0, 1, 6, 11, 12, 13, },
  { 2, 6, 11, 12, 13, },
  { 0, 2, 6, 11, 12, 13, },
  { 1, 2, 6, 11, 12, 13, },
  { 0, 1, 2, 6, 11, 12, 13, },
  { 3, 6, 11, 12, 13, },
  { 0, 3, 6, 11, 12, 13, },
  { 1, 3, 6, 11, 12, 13, },
  { 0, 1, 3, 6, 11, 12, 13, },
  { 2, 3, 6, 11, 12, 13, },
  { 0, 2, 3, 6, 11, 12, 13, },
  { 1, 2, 3, 6, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 11, 12, 13, },
  { 4, 6, 11, 12, 13, },
  { 0, 4, 6, 11, 12, 13, },
  { 1, 4, 6, 11, 12, 13, },
  { 0, 1, 4, 6, 11, 12, 13, },
  { 2, 4, 6, 11, 12, 13, },
  { 0, 2, 4, 6, 11, 12, 13, },
  { 1, 2, 4, 6, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 11, 12, 13, },
  { 3, 4, 6, 11, 12, 13, },
  { 0, 3, 4, 6, 11, 12, 13, },
  { 1, 3, 4, 6, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 11, 12, 13, },
  { 2, 3, 4, 6, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 11, 12, 13, },
  { 5, 6, 11, 12, 13, },
  { 0, 5, 6, 11, 12, 13, },
  { 1, 5, 6, 11, 12, 13, },
  { 0, 1, 5, 6, 11, 12, 13, },
  { 2, 5, 6, 11, 12, 13, },
  { 0, 2, 5, 6, 11, 12, 13, },
  { 1, 2, 5, 6, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 11, 12, 13, },
  { 3, 5, 6, 11, 12, 13, },
  { 0, 3, 5, 6, 11, 12, 13, },
  { 1, 3, 5, 6, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 11, 12, 13, },
  { 2, 3, 5, 6, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 11, 12, 13, },
  { 4, 5, 6, 11, 12, 13, },
  { 0, 4, 5, 6, 11, 12, 13, },
  { 1, 4, 5, 6, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 11, 12, 13, },
  { 2, 4, 5, 6, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 11, 12, 13, },
  { 3, 4, 5, 6, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 12, 13, },
  { 7, 11, 12, 13, },
  { 0, 7, 11, 12, 13, },
  { 1, 7, 11, 12, 13, },
  { 0, 1, 7, 11, 12, 13, },
  { 2, 7, 11, 12, 13, },
  { 0, 2, 7, 11, 12, 13, },
  { 1, 2, 7, 11, 12, 13, },
  { 0, 1, 2, 7, 11, 12, 13, },
  { 3, 7, 11, 12, 13, },
  { 0, 3, 7, 11, 12, 13, },
  { 1, 3, 7, 11, 12, 13, },
  { 0, 1, 3, 7, 11, 12, 13, },
  { 2, 3, 7, 11, 12, 13, },
  { 0, 2, 3, 7, 11, 12, 13, },
  { 1, 2, 3, 7, 11, 12, 13, },
  { 0, 1, 2, 3, 7, 11, 12, 13, },
  { 4, 7, 11, 12, 13, },
  { 0, 4, 7, 11, 12, 13, },
  { 1, 4, 7, 11, 12, 13, },
  { 0, 1, 4, 7, 11, 12, 13, },
  { 2, 4, 7, 11, 12, 13, },
  { 0, 2, 4, 7, 11, 12, 13, },
  { 1, 2, 4, 7, 11, 12, 13, },
  { 0, 1, 2, 4, 7, 11, 12, 13, },
  { 3, 4, 7, 11, 12, 13, },
  { 0, 3, 4, 7, 11, 12, 13, },
  { 1, 3, 4, 7, 11, 12, 13, },
  { 0, 1, 3, 4, 7, 11, 12, 13, },
  { 2, 3, 4, 7, 11, 12, 13, },
  { 0, 2, 3, 4, 7, 11, 12, 13, },
  { 1, 2, 3, 4, 7, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 11, 12, 13, },
  { 5, 7, 11, 12, 13, },
  { 0, 5, 7, 11, 12, 13, },
  { 1, 5, 7, 11, 12, 13, },
  { 0, 1, 5, 7, 11, 12, 13, },
  { 2, 5, 7, 11, 12, 13, },
  { 0, 2, 5, 7, 11, 12, 13, },
  { 1, 2, 5, 7, 11, 12, 13, },
  { 0, 1, 2, 5, 7, 11, 12, 13, },
  { 3, 5, 7, 11, 12, 13, },
  { 0, 3, 5, 7, 11, 12, 13, },
  { 1, 3, 5, 7, 11, 12, 13, },
  { 0, 1, 3, 5, 7, 11, 12, 13, },
  { 2, 3, 5, 7, 11, 12, 13, },
  { 0, 2, 3, 5, 7, 11, 12, 13, },
  { 1, 2, 3, 5, 7, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 11, 12, 13, },
  { 4, 5, 7, 11, 12, 13, },
  { 0, 4, 5, 7, 11, 12, 13, },
  { 1, 4, 5, 7, 11, 12, 13, },
  { 0, 1, 4, 5, 7, 11, 12, 13, },
  { 2, 4, 5, 7, 11, 12, 13, },
  { 0, 2, 4, 5, 7, 11, 12, 13, },
  { 1, 2, 4, 5, 7, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 11, 12, 13, },
  { 3, 4, 5, 7, 11, 12, 13, },
  { 0, 3, 4, 5, 7, 11, 12, 13, },
  { 1, 3, 4, 5, 7, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 11, 12, 13, },
  { 2, 3, 4, 5, 7, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 12, 13, },
  { 6, 7, 11, 12, 13, },
  { 0, 6, 7, 11, 12, 13, },
  { 1, 6, 7, 11, 12, 13, },
  { 0, 1, 6, 7, 11, 12, 13, },
  { 2, 6, 7, 11, 12, 13, },
  { 0, 2, 6, 7, 11, 12, 13, },
  { 1, 2, 6, 7, 11, 12, 13, },
  { 0, 1, 2, 6, 7, 11, 12, 13, },
  { 3, 6, 7, 11, 12, 13, },
  { 0, 3, 6, 7, 11, 12, 13, },
  { 1, 3, 6, 7, 11, 12, 13, },
  { 0, 1, 3, 6, 7, 11, 12, 13, },
  { 2, 3, 6, 7, 11, 12, 13, },
  { 0, 2, 3, 6, 7, 11, 12, 13, },
  { 1, 2, 3, 6, 7, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 11, 12, 13, },
  { 4, 6, 7, 11, 12, 13, },
  { 0, 4, 6, 7, 11, 12, 13, },
  { 1, 4, 6, 7, 11, 12, 13, },
  { 0, 1, 4, 6, 7, 11, 12, 13, },
  { 2, 4, 6, 7, 11, 12, 13, },
  { 0, 2, 4, 6, 7, 11, 12, 13, },
  { 1, 2, 4, 6, 7, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 11, 12, 13, },
  { 3, 4, 6, 7, 11, 12, 13, },
  { 0, 3, 4, 6, 7, 11, 12, 13, },
  { 1, 3, 4, 6, 7, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 11, 12, 13, },
  { 2, 3, 4, 6, 7, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 12, 13, },
  { 5, 6, 7, 11, 12, 13, },
  { 0, 5, 6, 7, 11, 12, 13, },
  { 1, 5, 6, 7, 11, 12, 13, },
  { 0, 1, 5, 6, 7, 11, 12, 13, },
  { 2, 5, 6, 7, 11, 12, 13, },
  { 0, 2, 5, 6, 7, 11, 12, 13, },
  { 1, 2, 5, 6, 7, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 11, 12, 13, },
  { 3, 5, 6, 7, 11, 12, 13, },
  { 0, 3, 5, 6, 7, 11, 12, 13, },
  { 1, 3, 5, 6, 7, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 11, 12, 13, },
  { 2, 3, 5, 6, 7, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 12, 13, },
  { 4, 5, 6, 7, 11, 12, 13, },
  { 0, 4, 5, 6, 7, 11, 12, 13, },
  { 1, 4, 5, 6, 7, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 11, 12, 13, },
  { 2, 4, 5, 6, 7, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 12, 13, },
  { 3, 4, 5, 6, 7, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 12, 13, },
  { 8, 11, 12, 13, },
  { 0, 8, 11, 12, 13, },
  { 1, 8, 11, 12, 13, },
  { 0, 1, 8, 11, 12, 13, },
  { 2, 8, 11, 12, 13, },
  { 0, 2, 8, 11, 12, 13, },
  { 1, 2, 8, 11, 12, 13, },
  { 0, 1, 2, 8, 11, 12, 13, },
  { 3, 8, 11, 12, 13, },
  { 0, 3, 8, 11, 12, 13, },
  { 1, 3, 8, 11, 12, 13, },
  { 0, 1, 3, 8, 11, 12, 13, },
  { 2, 3, 8, 11, 12, 13, },
  { 0, 2, 3, 8, 11, 12, 13, },
  { 1, 2, 3, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 8, 11, 12, 13, },
  { 4, 8, 11, 12, 13, },
  { 0, 4, 8, 11, 12, 13, },
  { 1, 4, 8, 11, 12, 13, },
  { 0, 1, 4, 8, 11, 12, 13, },
  { 2, 4, 8, 11, 12, 13, },
  { 0, 2, 4, 8, 11, 12, 13, },
  { 1, 2, 4, 8, 11, 12, 13, },
  { 0, 1, 2, 4, 8, 11, 12, 13, },
  { 3, 4, 8, 11, 12, 13, },
  { 0, 3, 4, 8, 11, 12, 13, },
  { 1, 3, 4, 8, 11, 12, 13, },
  { 0, 1, 3, 4, 8, 11, 12, 13, },
  { 2, 3, 4, 8, 11, 12, 13, },
  { 0, 2, 3, 4, 8, 11, 12, 13, },
  { 1, 2, 3, 4, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 8, 11, 12, 13, },
  { 5, 8, 11, 12, 13, },
  { 0, 5, 8, 11, 12, 13, },
  { 1, 5, 8, 11, 12, 13, },
  { 0, 1, 5, 8, 11, 12, 13, },
  { 2, 5, 8, 11, 12, 13, },
  { 0, 2, 5, 8, 11, 12, 13, },
  { 1, 2, 5, 8, 11, 12, 13, },
  { 0, 1, 2, 5, 8, 11, 12, 13, },
  { 3, 5, 8, 11, 12, 13, },
  { 0, 3, 5, 8, 11, 12, 13, },
  { 1, 3, 5, 8, 11, 12, 13, },
  { 0, 1, 3, 5, 8, 11, 12, 13, },
  { 2, 3, 5, 8, 11, 12, 13, },
  { 0, 2, 3, 5, 8, 11, 12, 13, },
  { 1, 2, 3, 5, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 8, 11, 12, 13, },
  { 4, 5, 8, 11, 12, 13, },
  { 0, 4, 5, 8, 11, 12, 13, },
  { 1, 4, 5, 8, 11, 12, 13, },
  { 0, 1, 4, 5, 8, 11, 12, 13, },
  { 2, 4, 5, 8, 11, 12, 13, },
  { 0, 2, 4, 5, 8, 11, 12, 13, },
  { 1, 2, 4, 5, 8, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 8, 11, 12, 13, },
  { 3, 4, 5, 8, 11, 12, 13, },
  { 0, 3, 4, 5, 8, 11, 12, 13, },
  { 1, 3, 4, 5, 8, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 8, 11, 12, 13, },
  { 2, 3, 4, 5, 8, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 8, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 12, 13, },
  { 6, 8, 11, 12, 13, },
  { 0, 6, 8, 11, 12, 13, },
  { 1, 6, 8, 11, 12, 13, },
  { 0, 1, 6, 8, 11, 12, 13, },
  { 2, 6, 8, 11, 12, 13, },
  { 0, 2, 6, 8, 11, 12, 13, },
  { 1, 2, 6, 8, 11, 12, 13, },
  { 0, 1, 2, 6, 8, 11, 12, 13, },
  { 3, 6, 8, 11, 12, 13, },
  { 0, 3, 6, 8, 11, 12, 13, },
  { 1, 3, 6, 8, 11, 12, 13, },
  { 0, 1, 3, 6, 8, 11, 12, 13, },
  { 2, 3, 6, 8, 11, 12, 13, },
  { 0, 2, 3, 6, 8, 11, 12, 13, },
  { 1, 2, 3, 6, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 8, 11, 12, 13, },
  { 4, 6, 8, 11, 12, 13, },
  { 0, 4, 6, 8, 11, 12, 13, },
  { 1, 4, 6, 8, 11, 12, 13, },
  { 0, 1, 4, 6, 8, 11, 12, 13, },
  { 2, 4, 6, 8, 11, 12, 13, },
  { 0, 2, 4, 6, 8, 11, 12, 13, },
  { 1, 2, 4, 6, 8, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 8, 11, 12, 13, },
  { 3, 4, 6, 8, 11, 12, 13, },
  { 0, 3, 4, 6, 8, 11, 12, 13, },
  { 1, 3, 4, 6, 8, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 8, 11, 12, 13, },
  { 2, 3, 4, 6, 8, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 8, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 12, 13, },
  { 5, 6, 8, 11, 12, 13, },
  { 0, 5, 6, 8, 11, 12, 13, },
  { 1, 5, 6, 8, 11, 12, 13, },
  { 0, 1, 5, 6, 8, 11, 12, 13, },
  { 2, 5, 6, 8, 11, 12, 13, },
  { 0, 2, 5, 6, 8, 11, 12, 13, },
  { 1, 2, 5, 6, 8, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 8, 11, 12, 13, },
  { 3, 5, 6, 8, 11, 12, 13, },
  { 0, 3, 5, 6, 8, 11, 12, 13, },
  { 1, 3, 5, 6, 8, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 8, 11, 12, 13, },
  { 2, 3, 5, 6, 8, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 8, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 12, 13, },
  { 4, 5, 6, 8, 11, 12, 13, },
  { 0, 4, 5, 6, 8, 11, 12, 13, },
  { 1, 4, 5, 6, 8, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 8, 11, 12, 13, },
  { 2, 4, 5, 6, 8, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 8, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 8, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 12, 13, },
  { 3, 4, 5, 6, 8, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 8, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 8, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 8, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 12, 13, },
  { 7, 8, 11, 12, 13, },
  { 0, 7, 8, 11, 12, 13, },
  { 1, 7, 8, 11, 12, 13, },
  { 0, 1, 7, 8, 11, 12, 13, },
  { 2, 7, 8, 11, 12, 13, },
  { 0, 2, 7, 8, 11, 12, 13, },
  { 1, 2, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 7, 8, 11, 12, 13, },
  { 3, 7, 8, 11, 12, 13, },
  { 0, 3, 7, 8, 11, 12, 13, },
  { 1, 3, 7, 8, 11, 12, 13, },
  { 0, 1, 3, 7, 8, 11, 12, 13, },
  { 2, 3, 7, 8, 11, 12, 13, },
  { 0, 2, 3, 7, 8, 11, 12, 13, },
  { 1, 2, 3, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 7, 8, 11, 12, 13, },
  { 4, 7, 8, 11, 12, 13, },
  { 0, 4, 7, 8, 11, 12, 13, },
  { 1, 4, 7, 8, 11, 12, 13, },
  { 0, 1, 4, 7, 8, 11, 12, 13, },
  { 2, 4, 7, 8, 11, 12, 13, },
  { 0, 2, 4, 7, 8, 11, 12, 13, },
  { 1, 2, 4, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 4, 7, 8, 11, 12, 13, },
  { 3, 4, 7, 8, 11, 12, 13, },
  { 0, 3, 4, 7, 8, 11, 12, 13, },
  { 1, 3, 4, 7, 8, 11, 12, 13, },
  { 0, 1, 3, 4, 7, 8, 11, 12, 13, },
  { 2, 3, 4, 7, 8, 11, 12, 13, },
  { 0, 2, 3, 4, 7, 8, 11, 12, 13, },
  { 1, 2, 3, 4, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 12, 13, },
  { 5, 7, 8, 11, 12, 13, },
  { 0, 5, 7, 8, 11, 12, 13, },
  { 1, 5, 7, 8, 11, 12, 13, },
  { 0, 1, 5, 7, 8, 11, 12, 13, },
  { 2, 5, 7, 8, 11, 12, 13, },
  { 0, 2, 5, 7, 8, 11, 12, 13, },
  { 1, 2, 5, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 5, 7, 8, 11, 12, 13, },
  { 3, 5, 7, 8, 11, 12, 13, },
  { 0, 3, 5, 7, 8, 11, 12, 13, },
  { 1, 3, 5, 7, 8, 11, 12, 13, },
  { 0, 1, 3, 5, 7, 8, 11, 12, 13, },
  { 2, 3, 5, 7, 8, 11, 12, 13, },
  { 0, 2, 3, 5, 7, 8, 11, 12, 13, },
  { 1, 2, 3, 5, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 12, 13, },
  { 4, 5, 7, 8, 11, 12, 13, },
  { 0, 4, 5, 7, 8, 11, 12, 13, },
  { 1, 4, 5, 7, 8, 11, 12, 13, },
  { 0, 1, 4, 5, 7, 8, 11, 12, 13, },
  { 2, 4, 5, 7, 8, 11, 12, 13, },
  { 0, 2, 4, 5, 7, 8, 11, 12, 13, },
  { 1, 2, 4, 5, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 12, 13, },
  { 3, 4, 5, 7, 8, 11, 12, 13, },
  { 0, 3, 4, 5, 7, 8, 11, 12, 13, },
  { 1, 3, 4, 5, 7, 8, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 12, 13, },
  { 2, 3, 4, 5, 7, 8, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 12, 13, },
  { 6, 7, 8, 11, 12, 13, },
  { 0, 6, 7, 8, 11, 12, 13, },
  { 1, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 6, 7, 8, 11, 12, 13, },
  { 2, 6, 7, 8, 11, 12, 13, },
  { 0, 2, 6, 7, 8, 11, 12, 13, },
  { 1, 2, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 6, 7, 8, 11, 12, 13, },
  { 3, 6, 7, 8, 11, 12, 13, },
  { 0, 3, 6, 7, 8, 11, 12, 13, },
  { 1, 3, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 3, 6, 7, 8, 11, 12, 13, },
  { 2, 3, 6, 7, 8, 11, 12, 13, },
  { 0, 2, 3, 6, 7, 8, 11, 12, 13, },
  { 1, 2, 3, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 12, 13, },
  { 4, 6, 7, 8, 11, 12, 13, },
  { 0, 4, 6, 7, 8, 11, 12, 13, },
  { 1, 4, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 4, 6, 7, 8, 11, 12, 13, },
  { 2, 4, 6, 7, 8, 11, 12, 13, },
  { 0, 2, 4, 6, 7, 8, 11, 12, 13, },
  { 1, 2, 4, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 12, 13, },
  { 3, 4, 6, 7, 8, 11, 12, 13, },
  { 0, 3, 4, 6, 7, 8, 11, 12, 13, },
  { 1, 3, 4, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 12, 13, },
  { 2, 3, 4, 6, 7, 8, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 12, 13, },
  { 5, 6, 7, 8, 11, 12, 13, },
  { 0, 5, 6, 7, 8, 11, 12, 13, },
  { 1, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 5, 6, 7, 8, 11, 12, 13, },
  { 2, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 2, 5, 6, 7, 8, 11, 12, 13, },
  { 1, 2, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 12, 13, },
  { 3, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 3, 5, 6, 7, 8, 11, 12, 13, },
  { 1, 3, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 12, 13, },
  { 2, 3, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 12, 13, },
  { 4, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 1, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 2, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 3, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, },
  { 9, 11, 12, 13, },
  { 0, 9, 11, 12, 13, },
  { 1, 9, 11, 12, 13, },
  { 0, 1, 9, 11, 12, 13, },
  { 2, 9, 11, 12, 13, },
  { 0, 2, 9, 11, 12, 13, },
  { 1, 2, 9, 11, 12, 13, },
  { 0, 1, 2, 9, 11, 12, 13, },
  { 3, 9, 11, 12, 13, },
  { 0, 3, 9, 11, 12, 13, },
  { 1, 3, 9, 11, 12, 13, },
  { 0, 1, 3, 9, 11, 12, 13, },
  { 2, 3, 9, 11, 12, 13, },
  { 0, 2, 3, 9, 11, 12, 13, },
  { 1, 2, 3, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 9, 11, 12, 13, },
  { 4, 9, 11, 12, 13, },
  { 0, 4, 9, 11, 12, 13, },
  { 1, 4, 9, 11, 12, 13, },
  { 0, 1, 4, 9, 11, 12, 13, },
  { 2, 4, 9, 11, 12, 13, },
  { 0, 2, 4, 9, 11, 12, 13, },
  { 1, 2, 4, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 9, 11, 12, 13, },
  { 3, 4, 9, 11, 12, 13, },
  { 0, 3, 4, 9, 11, 12, 13, },
  { 1, 3, 4, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 9, 11, 12, 13, },
  { 2, 3, 4, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 9, 11, 12, 13, },
  { 5, 9, 11, 12, 13, },
  { 0, 5, 9, 11, 12, 13, },
  { 1, 5, 9, 11, 12, 13, },
  { 0, 1, 5, 9, 11, 12, 13, },
  { 2, 5, 9, 11, 12, 13, },
  { 0, 2, 5, 9, 11, 12, 13, },
  { 1, 2, 5, 9, 11, 12, 13, },
  { 0, 1, 2, 5, 9, 11, 12, 13, },
  { 3, 5, 9, 11, 12, 13, },
  { 0, 3, 5, 9, 11, 12, 13, },
  { 1, 3, 5, 9, 11, 12, 13, },
  { 0, 1, 3, 5, 9, 11, 12, 13, },
  { 2, 3, 5, 9, 11, 12, 13, },
  { 0, 2, 3, 5, 9, 11, 12, 13, },
  { 1, 2, 3, 5, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 9, 11, 12, 13, },
  { 4, 5, 9, 11, 12, 13, },
  { 0, 4, 5, 9, 11, 12, 13, },
  { 1, 4, 5, 9, 11, 12, 13, },
  { 0, 1, 4, 5, 9, 11, 12, 13, },
  { 2, 4, 5, 9, 11, 12, 13, },
  { 0, 2, 4, 5, 9, 11, 12, 13, },
  { 1, 2, 4, 5, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 9, 11, 12, 13, },
  { 3, 4, 5, 9, 11, 12, 13, },
  { 0, 3, 4, 5, 9, 11, 12, 13, },
  { 1, 3, 4, 5, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 9, 11, 12, 13, },
  { 2, 3, 4, 5, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 12, 13, },
  { 6, 9, 11, 12, 13, },
  { 0, 6, 9, 11, 12, 13, },
  { 1, 6, 9, 11, 12, 13, },
  { 0, 1, 6, 9, 11, 12, 13, },
  { 2, 6, 9, 11, 12, 13, },
  { 0, 2, 6, 9, 11, 12, 13, },
  { 1, 2, 6, 9, 11, 12, 13, },
  { 0, 1, 2, 6, 9, 11, 12, 13, },
  { 3, 6, 9, 11, 12, 13, },
  { 0, 3, 6, 9, 11, 12, 13, },
  { 1, 3, 6, 9, 11, 12, 13, },
  { 0, 1, 3, 6, 9, 11, 12, 13, },
  { 2, 3, 6, 9, 11, 12, 13, },
  { 0, 2, 3, 6, 9, 11, 12, 13, },
  { 1, 2, 3, 6, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 9, 11, 12, 13, },
  { 4, 6, 9, 11, 12, 13, },
  { 0, 4, 6, 9, 11, 12, 13, },
  { 1, 4, 6, 9, 11, 12, 13, },
  { 0, 1, 4, 6, 9, 11, 12, 13, },
  { 2, 4, 6, 9, 11, 12, 13, },
  { 0, 2, 4, 6, 9, 11, 12, 13, },
  { 1, 2, 4, 6, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 9, 11, 12, 13, },
  { 3, 4, 6, 9, 11, 12, 13, },
  { 0, 3, 4, 6, 9, 11, 12, 13, },
  { 1, 3, 4, 6, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 9, 11, 12, 13, },
  { 2, 3, 4, 6, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 12, 13, },
  { 5, 6, 9, 11, 12, 13, },
  { 0, 5, 6, 9, 11, 12, 13, },
  { 1, 5, 6, 9, 11, 12, 13, },
  { 0, 1, 5, 6, 9, 11, 12, 13, },
  { 2, 5, 6, 9, 11, 12, 13, },
  { 0, 2, 5, 6, 9, 11, 12, 13, },
  { 1, 2, 5, 6, 9, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 9, 11, 12, 13, },
  { 3, 5, 6, 9, 11, 12, 13, },
  { 0, 3, 5, 6, 9, 11, 12, 13, },
  { 1, 3, 5, 6, 9, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 9, 11, 12, 13, },
  { 2, 3, 5, 6, 9, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 9, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 12, 13, },
  { 4, 5, 6, 9, 11, 12, 13, },
  { 0, 4, 5, 6, 9, 11, 12, 13, },
  { 1, 4, 5, 6, 9, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 9, 11, 12, 13, },
  { 2, 4, 5, 6, 9, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 9, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 12, 13, },
  { 3, 4, 5, 6, 9, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 9, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 12, 13, },
  { 7, 9, 11, 12, 13, },
  { 0, 7, 9, 11, 12, 13, },
  { 1, 7, 9, 11, 12, 13, },
  { 0, 1, 7, 9, 11, 12, 13, },
  { 2, 7, 9, 11, 12, 13, },
  { 0, 2, 7, 9, 11, 12, 13, },
  { 1, 2, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 7, 9, 11, 12, 13, },
  { 3, 7, 9, 11, 12, 13, },
  { 0, 3, 7, 9, 11, 12, 13, },
  { 1, 3, 7, 9, 11, 12, 13, },
  { 0, 1, 3, 7, 9, 11, 12, 13, },
  { 2, 3, 7, 9, 11, 12, 13, },
  { 0, 2, 3, 7, 9, 11, 12, 13, },
  { 1, 2, 3, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 7, 9, 11, 12, 13, },
  { 4, 7, 9, 11, 12, 13, },
  { 0, 4, 7, 9, 11, 12, 13, },
  { 1, 4, 7, 9, 11, 12, 13, },
  { 0, 1, 4, 7, 9, 11, 12, 13, },
  { 2, 4, 7, 9, 11, 12, 13, },
  { 0, 2, 4, 7, 9, 11, 12, 13, },
  { 1, 2, 4, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 7, 9, 11, 12, 13, },
  { 3, 4, 7, 9, 11, 12, 13, },
  { 0, 3, 4, 7, 9, 11, 12, 13, },
  { 1, 3, 4, 7, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 7, 9, 11, 12, 13, },
  { 2, 3, 4, 7, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 7, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 12, 13, },
  { 5, 7, 9, 11, 12, 13, },
  { 0, 5, 7, 9, 11, 12, 13, },
  { 1, 5, 7, 9, 11, 12, 13, },
  { 0, 1, 5, 7, 9, 11, 12, 13, },
  { 2, 5, 7, 9, 11, 12, 13, },
  { 0, 2, 5, 7, 9, 11, 12, 13, },
  { 1, 2, 5, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 5, 7, 9, 11, 12, 13, },
  { 3, 5, 7, 9, 11, 12, 13, },
  { 0, 3, 5, 7, 9, 11, 12, 13, },
  { 1, 3, 5, 7, 9, 11, 12, 13, },
  { 0, 1, 3, 5, 7, 9, 11, 12, 13, },
  { 2, 3, 5, 7, 9, 11, 12, 13, },
  { 0, 2, 3, 5, 7, 9, 11, 12, 13, },
  { 1, 2, 3, 5, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 12, 13, },
  { 4, 5, 7, 9, 11, 12, 13, },
  { 0, 4, 5, 7, 9, 11, 12, 13, },
  { 1, 4, 5, 7, 9, 11, 12, 13, },
  { 0, 1, 4, 5, 7, 9, 11, 12, 13, },
  { 2, 4, 5, 7, 9, 11, 12, 13, },
  { 0, 2, 4, 5, 7, 9, 11, 12, 13, },
  { 1, 2, 4, 5, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 12, 13, },
  { 3, 4, 5, 7, 9, 11, 12, 13, },
  { 0, 3, 4, 5, 7, 9, 11, 12, 13, },
  { 1, 3, 4, 5, 7, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 12, 13, },
  { 2, 3, 4, 5, 7, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 12, 13, },
  { 6, 7, 9, 11, 12, 13, },
  { 0, 6, 7, 9, 11, 12, 13, },
  { 1, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 6, 7, 9, 11, 12, 13, },
  { 2, 6, 7, 9, 11, 12, 13, },
  { 0, 2, 6, 7, 9, 11, 12, 13, },
  { 1, 2, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 6, 7, 9, 11, 12, 13, },
  { 3, 6, 7, 9, 11, 12, 13, },
  { 0, 3, 6, 7, 9, 11, 12, 13, },
  { 1, 3, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 3, 6, 7, 9, 11, 12, 13, },
  { 2, 3, 6, 7, 9, 11, 12, 13, },
  { 0, 2, 3, 6, 7, 9, 11, 12, 13, },
  { 1, 2, 3, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 12, 13, },
  { 4, 6, 7, 9, 11, 12, 13, },
  { 0, 4, 6, 7, 9, 11, 12, 13, },
  { 1, 4, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 4, 6, 7, 9, 11, 12, 13, },
  { 2, 4, 6, 7, 9, 11, 12, 13, },
  { 0, 2, 4, 6, 7, 9, 11, 12, 13, },
  { 1, 2, 4, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 12, 13, },
  { 3, 4, 6, 7, 9, 11, 12, 13, },
  { 0, 3, 4, 6, 7, 9, 11, 12, 13, },
  { 1, 3, 4, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 12, 13, },
  { 2, 3, 4, 6, 7, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 12, 13, },
  { 5, 6, 7, 9, 11, 12, 13, },
  { 0, 5, 6, 7, 9, 11, 12, 13, },
  { 1, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 5, 6, 7, 9, 11, 12, 13, },
  { 2, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 2, 5, 6, 7, 9, 11, 12, 13, },
  { 1, 2, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 12, 13, },
  { 3, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 3, 5, 6, 7, 9, 11, 12, 13, },
  { 1, 3, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 12, 13, },
  { 2, 3, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 12, 13, },
  { 4, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 1, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 2, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 3, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, },
  { 8, 9, 11, 12, 13, },
  { 0, 8, 9, 11, 12, 13, },
  { 1, 8, 9, 11, 12, 13, },
  { 0, 1, 8, 9, 11, 12, 13, },
  { 2, 8, 9, 11, 12, 13, },
  { 0, 2, 8, 9, 11, 12, 13, },
  { 1, 2, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 8, 9, 11, 12, 13, },
  { 3, 8, 9, 11, 12, 13, },
  { 0, 3, 8, 9, 11, 12, 13, },
  { 1, 3, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 8, 9, 11, 12, 13, },
  { 2, 3, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 8, 9, 11, 12, 13, },
  { 4, 8, 9, 11, 12, 13, },
  { 0, 4, 8, 9, 11, 12, 13, },
  { 1, 4, 8, 9, 11, 12, 13, },
  { 0, 1, 4, 8, 9, 11, 12, 13, },
  { 2, 4, 8, 9, 11, 12, 13, },
  { 0, 2, 4, 8, 9, 11, 12, 13, },
  { 1, 2, 4, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 8, 9, 11, 12, 13, },
  { 3, 4, 8, 9, 11, 12, 13, },
  { 0, 3, 4, 8, 9, 11, 12, 13, },
  { 1, 3, 4, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 8, 9, 11, 12, 13, },
  { 2, 3, 4, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 12, 13, },
  { 5, 8, 9, 11, 12, 13, },
  { 0, 5, 8, 9, 11, 12, 13, },
  { 1, 5, 8, 9, 11, 12, 13, },
  { 0, 1, 5, 8, 9, 11, 12, 13, },
  { 2, 5, 8, 9, 11, 12, 13, },
  { 0, 2, 5, 8, 9, 11, 12, 13, },
  { 1, 2, 5, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 5, 8, 9, 11, 12, 13, },
  { 3, 5, 8, 9, 11, 12, 13, },
  { 0, 3, 5, 8, 9, 11, 12, 13, },
  { 1, 3, 5, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 5, 8, 9, 11, 12, 13, },
  { 2, 3, 5, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 5, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 5, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 12, 13, },
  { 4, 5, 8, 9, 11, 12, 13, },
  { 0, 4, 5, 8, 9, 11, 12, 13, },
  { 1, 4, 5, 8, 9, 11, 12, 13, },
  { 0, 1, 4, 5, 8, 9, 11, 12, 13, },
  { 2, 4, 5, 8, 9, 11, 12, 13, },
  { 0, 2, 4, 5, 8, 9, 11, 12, 13, },
  { 1, 2, 4, 5, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 12, 13, },
  { 3, 4, 5, 8, 9, 11, 12, 13, },
  { 0, 3, 4, 5, 8, 9, 11, 12, 13, },
  { 1, 3, 4, 5, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 12, 13, },
  { 2, 3, 4, 5, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 12, 13, },
  { 6, 8, 9, 11, 12, 13, },
  { 0, 6, 8, 9, 11, 12, 13, },
  { 1, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 6, 8, 9, 11, 12, 13, },
  { 2, 6, 8, 9, 11, 12, 13, },
  { 0, 2, 6, 8, 9, 11, 12, 13, },
  { 1, 2, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 6, 8, 9, 11, 12, 13, },
  { 3, 6, 8, 9, 11, 12, 13, },
  { 0, 3, 6, 8, 9, 11, 12, 13, },
  { 1, 3, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 6, 8, 9, 11, 12, 13, },
  { 2, 3, 6, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 6, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 12, 13, },
  { 4, 6, 8, 9, 11, 12, 13, },
  { 0, 4, 6, 8, 9, 11, 12, 13, },
  { 1, 4, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 4, 6, 8, 9, 11, 12, 13, },
  { 2, 4, 6, 8, 9, 11, 12, 13, },
  { 0, 2, 4, 6, 8, 9, 11, 12, 13, },
  { 1, 2, 4, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 12, 13, },
  { 3, 4, 6, 8, 9, 11, 12, 13, },
  { 0, 3, 4, 6, 8, 9, 11, 12, 13, },
  { 1, 3, 4, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 12, 13, },
  { 2, 3, 4, 6, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 12, 13, },
  { 5, 6, 8, 9, 11, 12, 13, },
  { 0, 5, 6, 8, 9, 11, 12, 13, },
  { 1, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 5, 6, 8, 9, 11, 12, 13, },
  { 2, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 2, 5, 6, 8, 9, 11, 12, 13, },
  { 1, 2, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 12, 13, },
  { 3, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 3, 5, 6, 8, 9, 11, 12, 13, },
  { 1, 3, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 12, 13, },
  { 2, 3, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 12, 13, },
  { 4, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 1, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 2, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 3, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, },
  { 7, 8, 9, 11, 12, 13, },
  { 0, 7, 8, 9, 11, 12, 13, },
  { 1, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 7, 8, 9, 11, 12, 13, },
  { 2, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 7, 8, 9, 11, 12, 13, },
  { 3, 7, 8, 9, 11, 12, 13, },
  { 0, 3, 7, 8, 9, 11, 12, 13, },
  { 1, 3, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 7, 8, 9, 11, 12, 13, },
  { 2, 3, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 12, 13, },
  { 4, 7, 8, 9, 11, 12, 13, },
  { 0, 4, 7, 8, 9, 11, 12, 13, },
  { 1, 4, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 4, 7, 8, 9, 11, 12, 13, },
  { 2, 4, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 4, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 4, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 12, 13, },
  { 3, 4, 7, 8, 9, 11, 12, 13, },
  { 0, 3, 4, 7, 8, 9, 11, 12, 13, },
  { 1, 3, 4, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 12, 13, },
  { 2, 3, 4, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 12, 13, },
  { 5, 7, 8, 9, 11, 12, 13, },
  { 0, 5, 7, 8, 9, 11, 12, 13, },
  { 1, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 5, 7, 8, 9, 11, 12, 13, },
  { 2, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 5, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 12, 13, },
  { 3, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 3, 5, 7, 8, 9, 11, 12, 13, },
  { 1, 3, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 12, 13, },
  { 2, 3, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 12, 13, },
  { 4, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 1, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 2, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 3, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, },
  { 6, 7, 8, 9, 11, 12, 13, },
  { 0, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 6, 7, 8, 9, 11, 12, 13, },
  { 2, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 12, 13, },
  { 3, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 3, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 3, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 12, 13, },
  { 2, 3, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 12, 13, },
  { 4, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 2, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 3, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, },
  { 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 2, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 3, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, },
  { 10, 11, 12, 13, },
  { 0, 10, 11, 12, 13, },
  { 1, 10, 11, 12, 13, },
  { 0, 1, 10, 11, 12, 13, },
  { 2, 10, 11, 12, 13, },
  { 0, 2, 10, 11, 12, 13, },
  { 1, 2, 10, 11, 12, 13, },
  { 0, 1, 2, 10, 11, 12, 13, },
  { 3, 10, 11, 12, 13, },
  { 0, 3, 10, 11, 12, 13, },
  { 1, 3, 10, 11, 12, 13, },
  { 0, 1, 3, 10, 11, 12, 13, },
  { 2, 3, 10, 11, 12, 13, },
  { 0, 2, 3, 10, 11, 12, 13, },
  { 1, 2, 3, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 10, 11, 12, 13, },
  { 4, 10, 11, 12, 13, },
  { 0, 4, 10, 11, 12, 13, },
  { 1, 4, 10, 11, 12, 13, },
  { 0, 1, 4, 10, 11, 12, 13, },
  { 2, 4, 10, 11, 12, 13, },
  { 0, 2, 4, 10, 11, 12, 13, },
  { 1, 2, 4, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 10, 11, 12, 13, },
  { 3, 4, 10, 11, 12, 13, },
  { 0, 3, 4, 10, 11, 12, 13, },
  { 1, 3, 4, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 10, 11, 12, 13, },
  { 2, 3, 4, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 10, 11, 12, 13, },
  { 5, 10, 11, 12, 13, },
  { 0, 5, 10, 11, 12, 13, },
  { 1, 5, 10, 11, 12, 13, },
  { 0, 1, 5, 10, 11, 12, 13, },
  { 2, 5, 10, 11, 12, 13, },
  { 0, 2, 5, 10, 11, 12, 13, },
  { 1, 2, 5, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 10, 11, 12, 13, },
  { 3, 5, 10, 11, 12, 13, },
  { 0, 3, 5, 10, 11, 12, 13, },
  { 1, 3, 5, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 10, 11, 12, 13, },
  { 2, 3, 5, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 10, 11, 12, 13, },
  { 4, 5, 10, 11, 12, 13, },
  { 0, 4, 5, 10, 11, 12, 13, },
  { 1, 4, 5, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 10, 11, 12, 13, },
  { 2, 4, 5, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 10, 11, 12, 13, },
  { 3, 4, 5, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 12, 13, },
  { 6, 10, 11, 12, 13, },
  { 0, 6, 10, 11, 12, 13, },
  { 1, 6, 10, 11, 12, 13, },
  { 0, 1, 6, 10, 11, 12, 13, },
  { 2, 6, 10, 11, 12, 13, },
  { 0, 2, 6, 10, 11, 12, 13, },
  { 1, 2, 6, 10, 11, 12, 13, },
  { 0, 1, 2, 6, 10, 11, 12, 13, },
  { 3, 6, 10, 11, 12, 13, },
  { 0, 3, 6, 10, 11, 12, 13, },
  { 1, 3, 6, 10, 11, 12, 13, },
  { 0, 1, 3, 6, 10, 11, 12, 13, },
  { 2, 3, 6, 10, 11, 12, 13, },
  { 0, 2, 3, 6, 10, 11, 12, 13, },
  { 1, 2, 3, 6, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 10, 11, 12, 13, },
  { 4, 6, 10, 11, 12, 13, },
  { 0, 4, 6, 10, 11, 12, 13, },
  { 1, 4, 6, 10, 11, 12, 13, },
  { 0, 1, 4, 6, 10, 11, 12, 13, },
  { 2, 4, 6, 10, 11, 12, 13, },
  { 0, 2, 4, 6, 10, 11, 12, 13, },
  { 1, 2, 4, 6, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 10, 11, 12, 13, },
  { 3, 4, 6, 10, 11, 12, 13, },
  { 0, 3, 4, 6, 10, 11, 12, 13, },
  { 1, 3, 4, 6, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 10, 11, 12, 13, },
  { 2, 3, 4, 6, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 12, 13, },
  { 5, 6, 10, 11, 12, 13, },
  { 0, 5, 6, 10, 11, 12, 13, },
  { 1, 5, 6, 10, 11, 12, 13, },
  { 0, 1, 5, 6, 10, 11, 12, 13, },
  { 2, 5, 6, 10, 11, 12, 13, },
  { 0, 2, 5, 6, 10, 11, 12, 13, },
  { 1, 2, 5, 6, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 10, 11, 12, 13, },
  { 3, 5, 6, 10, 11, 12, 13, },
  { 0, 3, 5, 6, 10, 11, 12, 13, },
  { 1, 3, 5, 6, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 10, 11, 12, 13, },
  { 2, 3, 5, 6, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 12, 13, },
  { 4, 5, 6, 10, 11, 12, 13, },
  { 0, 4, 5, 6, 10, 11, 12, 13, },
  { 1, 4, 5, 6, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 10, 11, 12, 13, },
  { 2, 4, 5, 6, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 12, 13, },
  { 3, 4, 5, 6, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, },
  { 7, 10, 11, 12, 13, },
  { 0, 7, 10, 11, 12, 13, },
  { 1, 7, 10, 11, 12, 13, },
  { 0, 1, 7, 10, 11, 12, 13, },
  { 2, 7, 10, 11, 12, 13, },
  { 0, 2, 7, 10, 11, 12, 13, },
  { 1, 2, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 7, 10, 11, 12, 13, },
  { 3, 7, 10, 11, 12, 13, },
  { 0, 3, 7, 10, 11, 12, 13, },
  { 1, 3, 7, 10, 11, 12, 13, },
  { 0, 1, 3, 7, 10, 11, 12, 13, },
  { 2, 3, 7, 10, 11, 12, 13, },
  { 0, 2, 3, 7, 10, 11, 12, 13, },
  { 1, 2, 3, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 7, 10, 11, 12, 13, },
  { 4, 7, 10, 11, 12, 13, },
  { 0, 4, 7, 10, 11, 12, 13, },
  { 1, 4, 7, 10, 11, 12, 13, },
  { 0, 1, 4, 7, 10, 11, 12, 13, },
  { 2, 4, 7, 10, 11, 12, 13, },
  { 0, 2, 4, 7, 10, 11, 12, 13, },
  { 1, 2, 4, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 7, 10, 11, 12, 13, },
  { 3, 4, 7, 10, 11, 12, 13, },
  { 0, 3, 4, 7, 10, 11, 12, 13, },
  { 1, 3, 4, 7, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 7, 10, 11, 12, 13, },
  { 2, 3, 4, 7, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 7, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 12, 13, },
  { 5, 7, 10, 11, 12, 13, },
  { 0, 5, 7, 10, 11, 12, 13, },
  { 1, 5, 7, 10, 11, 12, 13, },
  { 0, 1, 5, 7, 10, 11, 12, 13, },
  { 2, 5, 7, 10, 11, 12, 13, },
  { 0, 2, 5, 7, 10, 11, 12, 13, },
  { 1, 2, 5, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 7, 10, 11, 12, 13, },
  { 3, 5, 7, 10, 11, 12, 13, },
  { 0, 3, 5, 7, 10, 11, 12, 13, },
  { 1, 3, 5, 7, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 7, 10, 11, 12, 13, },
  { 2, 3, 5, 7, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 7, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 12, 13, },
  { 4, 5, 7, 10, 11, 12, 13, },
  { 0, 4, 5, 7, 10, 11, 12, 13, },
  { 1, 4, 5, 7, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 7, 10, 11, 12, 13, },
  { 2, 4, 5, 7, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 7, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 12, 13, },
  { 3, 4, 5, 7, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 7, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 7, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 7, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 12, 13, },
  { 6, 7, 10, 11, 12, 13, },
  { 0, 6, 7, 10, 11, 12, 13, },
  { 1, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 6, 7, 10, 11, 12, 13, },
  { 2, 6, 7, 10, 11, 12, 13, },
  { 0, 2, 6, 7, 10, 11, 12, 13, },
  { 1, 2, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 6, 7, 10, 11, 12, 13, },
  { 3, 6, 7, 10, 11, 12, 13, },
  { 0, 3, 6, 7, 10, 11, 12, 13, },
  { 1, 3, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 3, 6, 7, 10, 11, 12, 13, },
  { 2, 3, 6, 7, 10, 11, 12, 13, },
  { 0, 2, 3, 6, 7, 10, 11, 12, 13, },
  { 1, 2, 3, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 12, 13, },
  { 4, 6, 7, 10, 11, 12, 13, },
  { 0, 4, 6, 7, 10, 11, 12, 13, },
  { 1, 4, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 4, 6, 7, 10, 11, 12, 13, },
  { 2, 4, 6, 7, 10, 11, 12, 13, },
  { 0, 2, 4, 6, 7, 10, 11, 12, 13, },
  { 1, 2, 4, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 12, 13, },
  { 3, 4, 6, 7, 10, 11, 12, 13, },
  { 0, 3, 4, 6, 7, 10, 11, 12, 13, },
  { 1, 3, 4, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 12, 13, },
  { 2, 3, 4, 6, 7, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 12, 13, },
  { 5, 6, 7, 10, 11, 12, 13, },
  { 0, 5, 6, 7, 10, 11, 12, 13, },
  { 1, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 5, 6, 7, 10, 11, 12, 13, },
  { 2, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 2, 5, 6, 7, 10, 11, 12, 13, },
  { 1, 2, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 12, 13, },
  { 3, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 3, 5, 6, 7, 10, 11, 12, 13, },
  { 1, 3, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 12, 13, },
  { 2, 3, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 12, 13, },
  { 4, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 1, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 2, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 3, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, },
  { 8, 10, 11, 12, 13, },
  { 0, 8, 10, 11, 12, 13, },
  { 1, 8, 10, 11, 12, 13, },
  { 0, 1, 8, 10, 11, 12, 13, },
  { 2, 8, 10, 11, 12, 13, },
  { 0, 2, 8, 10, 11, 12, 13, },
  { 1, 2, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 8, 10, 11, 12, 13, },
  { 3, 8, 10, 11, 12, 13, },
  { 0, 3, 8, 10, 11, 12, 13, },
  { 1, 3, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 8, 10, 11, 12, 13, },
  { 2, 3, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 8, 10, 11, 12, 13, },
  { 4, 8, 10, 11, 12, 13, },
  { 0, 4, 8, 10, 11, 12, 13, },
  { 1, 4, 8, 10, 11, 12, 13, },
  { 0, 1, 4, 8, 10, 11, 12, 13, },
  { 2, 4, 8, 10, 11, 12, 13, },
  { 0, 2, 4, 8, 10, 11, 12, 13, },
  { 1, 2, 4, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 8, 10, 11, 12, 13, },
  { 3, 4, 8, 10, 11, 12, 13, },
  { 0, 3, 4, 8, 10, 11, 12, 13, },
  { 1, 3, 4, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 8, 10, 11, 12, 13, },
  { 2, 3, 4, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 12, 13, },
  { 5, 8, 10, 11, 12, 13, },
  { 0, 5, 8, 10, 11, 12, 13, },
  { 1, 5, 8, 10, 11, 12, 13, },
  { 0, 1, 5, 8, 10, 11, 12, 13, },
  { 2, 5, 8, 10, 11, 12, 13, },
  { 0, 2, 5, 8, 10, 11, 12, 13, },
  { 1, 2, 5, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 8, 10, 11, 12, 13, },
  { 3, 5, 8, 10, 11, 12, 13, },
  { 0, 3, 5, 8, 10, 11, 12, 13, },
  { 1, 3, 5, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 8, 10, 11, 12, 13, },
  { 2, 3, 5, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 12, 13, },
  { 4, 5, 8, 10, 11, 12, 13, },
  { 0, 4, 5, 8, 10, 11, 12, 13, },
  { 1, 4, 5, 8, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 8, 10, 11, 12, 13, },
  { 2, 4, 5, 8, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 8, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 12, 13, },
  { 3, 4, 5, 8, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 8, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 12, 13, },
  { 6, 8, 10, 11, 12, 13, },
  { 0, 6, 8, 10, 11, 12, 13, },
  { 1, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 6, 8, 10, 11, 12, 13, },
  { 2, 6, 8, 10, 11, 12, 13, },
  { 0, 2, 6, 8, 10, 11, 12, 13, },
  { 1, 2, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 6, 8, 10, 11, 12, 13, },
  { 3, 6, 8, 10, 11, 12, 13, },
  { 0, 3, 6, 8, 10, 11, 12, 13, },
  { 1, 3, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 6, 8, 10, 11, 12, 13, },
  { 2, 3, 6, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 6, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 12, 13, },
  { 4, 6, 8, 10, 11, 12, 13, },
  { 0, 4, 6, 8, 10, 11, 12, 13, },
  { 1, 4, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 4, 6, 8, 10, 11, 12, 13, },
  { 2, 4, 6, 8, 10, 11, 12, 13, },
  { 0, 2, 4, 6, 8, 10, 11, 12, 13, },
  { 1, 2, 4, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 12, 13, },
  { 3, 4, 6, 8, 10, 11, 12, 13, },
  { 0, 3, 4, 6, 8, 10, 11, 12, 13, },
  { 1, 3, 4, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 12, 13, },
  { 2, 3, 4, 6, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 12, 13, },
  { 5, 6, 8, 10, 11, 12, 13, },
  { 0, 5, 6, 8, 10, 11, 12, 13, },
  { 1, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 5, 6, 8, 10, 11, 12, 13, },
  { 2, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 2, 5, 6, 8, 10, 11, 12, 13, },
  { 1, 2, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 12, 13, },
  { 3, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 3, 5, 6, 8, 10, 11, 12, 13, },
  { 1, 3, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 12, 13, },
  { 2, 3, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 12, 13, },
  { 4, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 1, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 2, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 3, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, },
  { 7, 8, 10, 11, 12, 13, },
  { 0, 7, 8, 10, 11, 12, 13, },
  { 1, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 7, 8, 10, 11, 12, 13, },
  { 2, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 7, 8, 10, 11, 12, 13, },
  { 3, 7, 8, 10, 11, 12, 13, },
  { 0, 3, 7, 8, 10, 11, 12, 13, },
  { 1, 3, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 7, 8, 10, 11, 12, 13, },
  { 2, 3, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 12, 13, },
  { 4, 7, 8, 10, 11, 12, 13, },
  { 0, 4, 7, 8, 10, 11, 12, 13, },
  { 1, 4, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 4, 7, 8, 10, 11, 12, 13, },
  { 2, 4, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 4, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 4, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 12, 13, },
  { 3, 4, 7, 8, 10, 11, 12, 13, },
  { 0, 3, 4, 7, 8, 10, 11, 12, 13, },
  { 1, 3, 4, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 12, 13, },
  { 2, 3, 4, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 12, 13, },
  { 5, 7, 8, 10, 11, 12, 13, },
  { 0, 5, 7, 8, 10, 11, 12, 13, },
  { 1, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 5, 7, 8, 10, 11, 12, 13, },
  { 2, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 5, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 12, 13, },
  { 3, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 3, 5, 7, 8, 10, 11, 12, 13, },
  { 1, 3, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 12, 13, },
  { 2, 3, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 12, 13, },
  { 4, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 1, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 2, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 3, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, },
  { 6, 7, 8, 10, 11, 12, 13, },
  { 0, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 6, 7, 8, 10, 11, 12, 13, },
  { 2, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 12, 13, },
  { 3, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 3, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 3, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 12, 13, },
  { 2, 3, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 12, 13, },
  { 4, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 2, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 3, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, },
  { 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 2, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 3, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, },
  { 9, 10, 11, 12, 13, },
  { 0, 9, 10, 11, 12, 13, },
  { 1, 9, 10, 11, 12, 13, },
  { 0, 1, 9, 10, 11, 12, 13, },
  { 2, 9, 10, 11, 12, 13, },
  { 0, 2, 9, 10, 11, 12, 13, },
  { 1, 2, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 9, 10, 11, 12, 13, },
  { 3, 9, 10, 11, 12, 13, },
  { 0, 3, 9, 10, 11, 12, 13, },
  { 1, 3, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 9, 10, 11, 12, 13, },
  { 2, 3, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 9, 10, 11, 12, 13, },
  { 4, 9, 10, 11, 12, 13, },
  { 0, 4, 9, 10, 11, 12, 13, },
  { 1, 4, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 9, 10, 11, 12, 13, },
  { 2, 4, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 9, 10, 11, 12, 13, },
  { 3, 4, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 12, 13, },
  { 5, 9, 10, 11, 12, 13, },
  { 0, 5, 9, 10, 11, 12, 13, },
  { 1, 5, 9, 10, 11, 12, 13, },
  { 0, 1, 5, 9, 10, 11, 12, 13, },
  { 2, 5, 9, 10, 11, 12, 13, },
  { 0, 2, 5, 9, 10, 11, 12, 13, },
  { 1, 2, 5, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 9, 10, 11, 12, 13, },
  { 3, 5, 9, 10, 11, 12, 13, },
  { 0, 3, 5, 9, 10, 11, 12, 13, },
  { 1, 3, 5, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 9, 10, 11, 12, 13, },
  { 2, 3, 5, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 12, 13, },
  { 4, 5, 9, 10, 11, 12, 13, },
  { 0, 4, 5, 9, 10, 11, 12, 13, },
  { 1, 4, 5, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 9, 10, 11, 12, 13, },
  { 2, 4, 5, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 12, 13, },
  { 3, 4, 5, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, },
  { 6, 9, 10, 11, 12, 13, },
  { 0, 6, 9, 10, 11, 12, 13, },
  { 1, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 6, 9, 10, 11, 12, 13, },
  { 2, 6, 9, 10, 11, 12, 13, },
  { 0, 2, 6, 9, 10, 11, 12, 13, },
  { 1, 2, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 6, 9, 10, 11, 12, 13, },
  { 3, 6, 9, 10, 11, 12, 13, },
  { 0, 3, 6, 9, 10, 11, 12, 13, },
  { 1, 3, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 6, 9, 10, 11, 12, 13, },
  { 2, 3, 6, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 6, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 12, 13, },
  { 4, 6, 9, 10, 11, 12, 13, },
  { 0, 4, 6, 9, 10, 11, 12, 13, },
  { 1, 4, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 6, 9, 10, 11, 12, 13, },
  { 2, 4, 6, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 6, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 12, 13, },
  { 3, 4, 6, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 6, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 6, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 12, 13, },
  { 5, 6, 9, 10, 11, 12, 13, },
  { 0, 5, 6, 9, 10, 11, 12, 13, },
  { 1, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 5, 6, 9, 10, 11, 12, 13, },
  { 2, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 2, 5, 6, 9, 10, 11, 12, 13, },
  { 1, 2, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 12, 13, },
  { 3, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 3, 5, 6, 9, 10, 11, 12, 13, },
  { 1, 3, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 12, 13, },
  { 2, 3, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 12, 13, },
  { 4, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 1, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 2, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 3, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, },
  { 7, 9, 10, 11, 12, 13, },
  { 0, 7, 9, 10, 11, 12, 13, },
  { 1, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 7, 9, 10, 11, 12, 13, },
  { 2, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 7, 9, 10, 11, 12, 13, },
  { 3, 7, 9, 10, 11, 12, 13, },
  { 0, 3, 7, 9, 10, 11, 12, 13, },
  { 1, 3, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 7, 9, 10, 11, 12, 13, },
  { 2, 3, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 12, 13, },
  { 4, 7, 9, 10, 11, 12, 13, },
  { 0, 4, 7, 9, 10, 11, 12, 13, },
  { 1, 4, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 7, 9, 10, 11, 12, 13, },
  { 2, 4, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 12, 13, },
  { 3, 4, 7, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 7, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 12, 13, },
  { 5, 7, 9, 10, 11, 12, 13, },
  { 0, 5, 7, 9, 10, 11, 12, 13, },
  { 1, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 5, 7, 9, 10, 11, 12, 13, },
  { 2, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 5, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 12, 13, },
  { 3, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 3, 5, 7, 9, 10, 11, 12, 13, },
  { 1, 3, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 12, 13, },
  { 2, 3, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 12, 13, },
  { 4, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 1, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 2, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 3, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, },
  { 6, 7, 9, 10, 11, 12, 13, },
  { 0, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 6, 7, 9, 10, 11, 12, 13, },
  { 2, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 12, 13, },
  { 3, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 3, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 3, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 12, 13, },
  { 2, 3, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 12, 13, },
  { 4, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 2, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 3, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, },
  { 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 2, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 3, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, },
  { 8, 9, 10, 11, 12, 13, },
  { 0, 8, 9, 10, 11, 12, 13, },
  { 1, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 8, 9, 10, 11, 12, 13, },
  { 2, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 8, 9, 10, 11, 12, 13, },
  { 3, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 12, 13, },
  { 4, 8, 9, 10, 11, 12, 13, },
  { 0, 4, 8, 9, 10, 11, 12, 13, },
  { 1, 4, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 8, 9, 10, 11, 12, 13, },
  { 2, 4, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 12, 13, },
  { 3, 4, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 12, 13, },
  { 5, 8, 9, 10, 11, 12, 13, },
  { 0, 5, 8, 9, 10, 11, 12, 13, },
  { 1, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 5, 8, 9, 10, 11, 12, 13, },
  { 2, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 5, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 12, 13, },
  { 3, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 5, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 12, 13, },
  { 4, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 1, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 2, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 3, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, },
  { 6, 8, 9, 10, 11, 12, 13, },
  { 0, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 6, 8, 9, 10, 11, 12, 13, },
  { 2, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 12, 13, },
  { 3, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 12, 13, },
  { 4, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 2, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 3, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, },
  { 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 2, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 3, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, },
  { 7, 8, 9, 10, 11, 12, 13, },
  { 0, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 12, 13, },
  { 3, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, },
  { 4, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 3, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, },
  { 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 3, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, },
  { 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 3, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, },
  { 14, },
  { 0, 14, },
  { 1, 14, },
  { 0, 1, 14, },
  { 2, 14, },
  { 0, 2, 14, },
  { 1, 2, 14, },
  { 0, 1, 2, 14, },
  { 3, 14, },
  { 0, 3, 14, },
  { 1, 3, 14, },
  { 0, 1, 3, 14, },
  { 2, 3, 14, },
  { 0, 2, 3, 14, },
  { 1, 2, 3, 14, },
  { 0, 1, 2, 3, 14, },
  { 4, 14, },
  { 0, 4, 14, },
  { 1, 4, 14, },
  { 0, 1, 4, 14, },
  { 2, 4, 14, },
  { 0, 2, 4, 14, },
  { 1, 2, 4, 14, },
  { 0, 1, 2, 4, 14, },
  { 3, 4, 14, },
  { 0, 3, 4, 14, },
  { 1, 3, 4, 14, },
  { 0, 1, 3, 4, 14, },
  { 2, 3, 4, 14, },
  { 0, 2, 3, 4, 14, },
  { 1, 2, 3, 4, 14, },
  { 0, 1, 2, 3, 4, 14, },
  { 5, 14, },
  { 0, 5, 14, },
  { 1, 5, 14, },
  { 0, 1, 5, 14, },
  { 2, 5, 14, },
  { 0, 2, 5, 14, },
  { 1, 2, 5, 14, },
  { 0, 1, 2, 5, 14, },
  { 3, 5, 14, },
  { 0, 3, 5, 14, },
  { 1, 3, 5, 14, },
  { 0, 1, 3, 5, 14, },
  { 2, 3, 5, 14, },
  { 0, 2, 3, 5, 14, },
  { 1, 2, 3, 5, 14, },
  { 0, 1, 2, 3, 5, 14, },
  { 4, 5, 14, },
  { 0, 4, 5, 14, },
  { 1, 4, 5, 14, },
  { 0, 1, 4, 5, 14, },
  { 2, 4, 5, 14, },
  { 0, 2, 4, 5, 14, },
  { 1, 2, 4, 5, 14, },
  { 0, 1, 2, 4, 5, 14, },
  { 3, 4, 5, 14, },
  { 0, 3, 4, 5, 14, },
  { 1, 3, 4, 5, 14, },
  { 0, 1, 3, 4, 5, 14, },
  { 2, 3, 4, 5, 14, },
  { 0, 2, 3, 4, 5, 14, },
  { 1, 2, 3, 4, 5, 14, },
  { 0, 1, 2, 3, 4, 5, 14, },
  { 6, 14, },
  { 0, 6, 14, },
  { 1, 6, 14, },
  { 0, 1, 6, 14, },
  { 2, 6, 14, },
  { 0, 2, 6, 14, },
  { 1, 2, 6, 14, },
  { 0, 1, 2, 6, 14, },
  { 3, 6, 14, },
  { 0, 3, 6, 14, },
  { 1, 3, 6, 14, },
  { 0, 1, 3, 6, 14, },
  { 2, 3, 6, 14, },
  { 0, 2, 3, 6, 14, },
  { 1, 2, 3, 6, 14, },
  { 0, 1, 2, 3, 6, 14, },
  { 4, 6, 14, },
  { 0, 4, 6, 14, },
  { 1, 4, 6, 14, },
  { 0, 1, 4, 6, 14, },
  { 2, 4, 6, 14, },
  { 0, 2, 4, 6, 14, },
  { 1, 2, 4, 6, 14, },
  { 0, 1, 2, 4, 6, 14, },
  { 3, 4, 6, 14, },
  { 0, 3, 4, 6, 14, },
  { 1, 3, 4, 6, 14, },
  { 0, 1, 3, 4, 6, 14, },
  { 2, 3, 4, 6, 14, },
  { 0, 2, 3, 4, 6, 14, },
  { 1, 2, 3, 4, 6, 14, },
  { 0, 1, 2, 3, 4, 6, 14, },
  { 5, 6, 14, },
  { 0, 5, 6, 14, },
  { 1, 5, 6, 14, },
  { 0, 1, 5, 6, 14, },
  { 2, 5, 6, 14, },
  { 0, 2, 5, 6, 14, },
  { 1, 2, 5, 6, 14, },
  { 0, 1, 2, 5, 6, 14, },
  { 3, 5, 6, 14, },
  { 0, 3, 5, 6, 14, },
  { 1, 3, 5, 6, 14, },
  { 0, 1, 3, 5, 6, 14, },
  { 2, 3, 5, 6, 14, },
  { 0, 2, 3, 5, 6, 14, },
  { 1, 2, 3, 5, 6, 14, },
  { 0, 1, 2, 3, 5, 6, 14, },
  { 4, 5, 6, 14, },
  { 0, 4, 5, 6, 14, },
  { 1, 4, 5, 6, 14, },
  { 0, 1, 4, 5, 6, 14, },
  { 2, 4, 5, 6, 14, },
  { 0, 2, 4, 5, 6, 14, },
  { 1, 2, 4, 5, 6, 14, },
  { 0, 1, 2, 4, 5, 6, 14, },
  { 3, 4, 5, 6, 14, },
  { 0, 3, 4, 5, 6, 14, },
  { 1, 3, 4, 5, 6, 14, },
  { 0, 1, 3, 4, 5, 6, 14, },
  { 2, 3, 4, 5, 6, 14, },
  { 0, 2, 3, 4, 5, 6, 14, },
  { 1, 2, 3, 4, 5, 6, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 14, },
  { 7, 14, },
  { 0, 7, 14, },
  { 1, 7, 14, },
  { 0, 1, 7, 14, },
  { 2, 7, 14, },
  { 0, 2, 7, 14, },
  { 1, 2, 7, 14, },
  { 0, 1, 2, 7, 14, },
  { 3, 7, 14, },
  { 0, 3, 7, 14, },
  { 1, 3, 7, 14, },
  { 0, 1, 3, 7, 14, },
  { 2, 3, 7, 14, },
  { 0, 2, 3, 7, 14, },
  { 1, 2, 3, 7, 14, },
  { 0, 1, 2, 3, 7, 14, },
  { 4, 7, 14, },
  { 0, 4, 7, 14, },
  { 1, 4, 7, 14, },
  { 0, 1, 4, 7, 14, },
  { 2, 4, 7, 14, },
  { 0, 2, 4, 7, 14, },
  { 1, 2, 4, 7, 14, },
  { 0, 1, 2, 4, 7, 14, },
  { 3, 4, 7, 14, },
  { 0, 3, 4, 7, 14, },
  { 1, 3, 4, 7, 14, },
  { 0, 1, 3, 4, 7, 14, },
  { 2, 3, 4, 7, 14, },
  { 0, 2, 3, 4, 7, 14, },
  { 1, 2, 3, 4, 7, 14, },
  { 0, 1, 2, 3, 4, 7, 14, },
  { 5, 7, 14, },
  { 0, 5, 7, 14, },
  { 1, 5, 7, 14, },
  { 0, 1, 5, 7, 14, },
  { 2, 5, 7, 14, },
  { 0, 2, 5, 7, 14, },
  { 1, 2, 5, 7, 14, },
  { 0, 1, 2, 5, 7, 14, },
  { 3, 5, 7, 14, },
  { 0, 3, 5, 7, 14, },
  { 1, 3, 5, 7, 14, },
  { 0, 1, 3, 5, 7, 14, },
  { 2, 3, 5, 7, 14, },
  { 0, 2, 3, 5, 7, 14, },
  { 1, 2, 3, 5, 7, 14, },
  { 0, 1, 2, 3, 5, 7, 14, },
  { 4, 5, 7, 14, },
  { 0, 4, 5, 7, 14, },
  { 1, 4, 5, 7, 14, },
  { 0, 1, 4, 5, 7, 14, },
  { 2, 4, 5, 7, 14, },
  { 0, 2, 4, 5, 7, 14, },
  { 1, 2, 4, 5, 7, 14, },
  { 0, 1, 2, 4, 5, 7, 14, },
  { 3, 4, 5, 7, 14, },
  { 0, 3, 4, 5, 7, 14, },
  { 1, 3, 4, 5, 7, 14, },
  { 0, 1, 3, 4, 5, 7, 14, },
  { 2, 3, 4, 5, 7, 14, },
  { 0, 2, 3, 4, 5, 7, 14, },
  { 1, 2, 3, 4, 5, 7, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 14, },
  { 6, 7, 14, },
  { 0, 6, 7, 14, },
  { 1, 6, 7, 14, },
  { 0, 1, 6, 7, 14, },
  { 2, 6, 7, 14, },
  { 0, 2, 6, 7, 14, },
  { 1, 2, 6, 7, 14, },
  { 0, 1, 2, 6, 7, 14, },
  { 3, 6, 7, 14, },
  { 0, 3, 6, 7, 14, },
  { 1, 3, 6, 7, 14, },
  { 0, 1, 3, 6, 7, 14, },
  { 2, 3, 6, 7, 14, },
  { 0, 2, 3, 6, 7, 14, },
  { 1, 2, 3, 6, 7, 14, },
  { 0, 1, 2, 3, 6, 7, 14, },
  { 4, 6, 7, 14, },
  { 0, 4, 6, 7, 14, },
  { 1, 4, 6, 7, 14, },
  { 0, 1, 4, 6, 7, 14, },
  { 2, 4, 6, 7, 14, },
  { 0, 2, 4, 6, 7, 14, },
  { 1, 2, 4, 6, 7, 14, },
  { 0, 1, 2, 4, 6, 7, 14, },
  { 3, 4, 6, 7, 14, },
  { 0, 3, 4, 6, 7, 14, },
  { 1, 3, 4, 6, 7, 14, },
  { 0, 1, 3, 4, 6, 7, 14, },
  { 2, 3, 4, 6, 7, 14, },
  { 0, 2, 3, 4, 6, 7, 14, },
  { 1, 2, 3, 4, 6, 7, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 14, },
  { 5, 6, 7, 14, },
  { 0, 5, 6, 7, 14, },
  { 1, 5, 6, 7, 14, },
  { 0, 1, 5, 6, 7, 14, },
  { 2, 5, 6, 7, 14, },
  { 0, 2, 5, 6, 7, 14, },
  { 1, 2, 5, 6, 7, 14, },
  { 0, 1, 2, 5, 6, 7, 14, },
  { 3, 5, 6, 7, 14, },
  { 0, 3, 5, 6, 7, 14, },
  { 1, 3, 5, 6, 7, 14, },
  { 0, 1, 3, 5, 6, 7, 14, },
  { 2, 3, 5, 6, 7, 14, },
  { 0, 2, 3, 5, 6, 7, 14, },
  { 1, 2, 3, 5, 6, 7, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 14, },
  { 4, 5, 6, 7, 14, },
  { 0, 4, 5, 6, 7, 14, },
  { 1, 4, 5, 6, 7, 14, },
  { 0, 1, 4, 5, 6, 7, 14, },
  { 2, 4, 5, 6, 7, 14, },
  { 0, 2, 4, 5, 6, 7, 14, },
  { 1, 2, 4, 5, 6, 7, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 14, },
  { 3, 4, 5, 6, 7, 14, },
  { 0, 3, 4, 5, 6, 7, 14, },
  { 1, 3, 4, 5, 6, 7, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 14, },
  { 2, 3, 4, 5, 6, 7, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 14, },
  { 8, 14, },
  { 0, 8, 14, },
  { 1, 8, 14, },
  { 0, 1, 8, 14, },
  { 2, 8, 14, },
  { 0, 2, 8, 14, },
  { 1, 2, 8, 14, },
  { 0, 1, 2, 8, 14, },
  { 3, 8, 14, },
  { 0, 3, 8, 14, },
  { 1, 3, 8, 14, },
  { 0, 1, 3, 8, 14, },
  { 2, 3, 8, 14, },
  { 0, 2, 3, 8, 14, },
  { 1, 2, 3, 8, 14, },
  { 0, 1, 2, 3, 8, 14, },
  { 4, 8, 14, },
  { 0, 4, 8, 14, },
  { 1, 4, 8, 14, },
  { 0, 1, 4, 8, 14, },
  { 2, 4, 8, 14, },
  { 0, 2, 4, 8, 14, },
  { 1, 2, 4, 8, 14, },
  { 0, 1, 2, 4, 8, 14, },
  { 3, 4, 8, 14, },
  { 0, 3, 4, 8, 14, },
  { 1, 3, 4, 8, 14, },
  { 0, 1, 3, 4, 8, 14, },
  { 2, 3, 4, 8, 14, },
  { 0, 2, 3, 4, 8, 14, },
  { 1, 2, 3, 4, 8, 14, },
  { 0, 1, 2, 3, 4, 8, 14, },
  { 5, 8, 14, },
  { 0, 5, 8, 14, },
  { 1, 5, 8, 14, },
  { 0, 1, 5, 8, 14, },
  { 2, 5, 8, 14, },
  { 0, 2, 5, 8, 14, },
  { 1, 2, 5, 8, 14, },
  { 0, 1, 2, 5, 8, 14, },
  { 3, 5, 8, 14, },
  { 0, 3, 5, 8, 14, },
  { 1, 3, 5, 8, 14, },
  { 0, 1, 3, 5, 8, 14, },
  { 2, 3, 5, 8, 14, },
  { 0, 2, 3, 5, 8, 14, },
  { 1, 2, 3, 5, 8, 14, },
  { 0, 1, 2, 3, 5, 8, 14, },
  { 4, 5, 8, 14, },
  { 0, 4, 5, 8, 14, },
  { 1, 4, 5, 8, 14, },
  { 0, 1, 4, 5, 8, 14, },
  { 2, 4, 5, 8, 14, },
  { 0, 2, 4, 5, 8, 14, },
  { 1, 2, 4, 5, 8, 14, },
  { 0, 1, 2, 4, 5, 8, 14, },
  { 3, 4, 5, 8, 14, },
  { 0, 3, 4, 5, 8, 14, },
  { 1, 3, 4, 5, 8, 14, },
  { 0, 1, 3, 4, 5, 8, 14, },
  { 2, 3, 4, 5, 8, 14, },
  { 0, 2, 3, 4, 5, 8, 14, },
  { 1, 2, 3, 4, 5, 8, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 14, },
  { 6, 8, 14, },
  { 0, 6, 8, 14, },
  { 1, 6, 8, 14, },
  { 0, 1, 6, 8, 14, },
  { 2, 6, 8, 14, },
  { 0, 2, 6, 8, 14, },
  { 1, 2, 6, 8, 14, },
  { 0, 1, 2, 6, 8, 14, },
  { 3, 6, 8, 14, },
  { 0, 3, 6, 8, 14, },
  { 1, 3, 6, 8, 14, },
  { 0, 1, 3, 6, 8, 14, },
  { 2, 3, 6, 8, 14, },
  { 0, 2, 3, 6, 8, 14, },
  { 1, 2, 3, 6, 8, 14, },
  { 0, 1, 2, 3, 6, 8, 14, },
  { 4, 6, 8, 14, },
  { 0, 4, 6, 8, 14, },
  { 1, 4, 6, 8, 14, },
  { 0, 1, 4, 6, 8, 14, },
  { 2, 4, 6, 8, 14, },
  { 0, 2, 4, 6, 8, 14, },
  { 1, 2, 4, 6, 8, 14, },
  { 0, 1, 2, 4, 6, 8, 14, },
  { 3, 4, 6, 8, 14, },
  { 0, 3, 4, 6, 8, 14, },
  { 1, 3, 4, 6, 8, 14, },
  { 0, 1, 3, 4, 6, 8, 14, },
  { 2, 3, 4, 6, 8, 14, },
  { 0, 2, 3, 4, 6, 8, 14, },
  { 1, 2, 3, 4, 6, 8, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 14, },
  { 5, 6, 8, 14, },
  { 0, 5, 6, 8, 14, },
  { 1, 5, 6, 8, 14, },
  { 0, 1, 5, 6, 8, 14, },
  { 2, 5, 6, 8, 14, },
  { 0, 2, 5, 6, 8, 14, },
  { 1, 2, 5, 6, 8, 14, },
  { 0, 1, 2, 5, 6, 8, 14, },
  { 3, 5, 6, 8, 14, },
  { 0, 3, 5, 6, 8, 14, },
  { 1, 3, 5, 6, 8, 14, },
  { 0, 1, 3, 5, 6, 8, 14, },
  { 2, 3, 5, 6, 8, 14, },
  { 0, 2, 3, 5, 6, 8, 14, },
  { 1, 2, 3, 5, 6, 8, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 14, },
  { 4, 5, 6, 8, 14, },
  { 0, 4, 5, 6, 8, 14, },
  { 1, 4, 5, 6, 8, 14, },
  { 0, 1, 4, 5, 6, 8, 14, },
  { 2, 4, 5, 6, 8, 14, },
  { 0, 2, 4, 5, 6, 8, 14, },
  { 1, 2, 4, 5, 6, 8, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 14, },
  { 3, 4, 5, 6, 8, 14, },
  { 0, 3, 4, 5, 6, 8, 14, },
  { 1, 3, 4, 5, 6, 8, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 14, },
  { 2, 3, 4, 5, 6, 8, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 14, },
  { 7, 8, 14, },
  { 0, 7, 8, 14, },
  { 1, 7, 8, 14, },
  { 0, 1, 7, 8, 14, },
  { 2, 7, 8, 14, },
  { 0, 2, 7, 8, 14, },
  { 1, 2, 7, 8, 14, },
  { 0, 1, 2, 7, 8, 14, },
  { 3, 7, 8, 14, },
  { 0, 3, 7, 8, 14, },
  { 1, 3, 7, 8, 14, },
  { 0, 1, 3, 7, 8, 14, },
  { 2, 3, 7, 8, 14, },
  { 0, 2, 3, 7, 8, 14, },
  { 1, 2, 3, 7, 8, 14, },
  { 0, 1, 2, 3, 7, 8, 14, },
  { 4, 7, 8, 14, },
  { 0, 4, 7, 8, 14, },
  { 1, 4, 7, 8, 14, },
  { 0, 1, 4, 7, 8, 14, },
  { 2, 4, 7, 8, 14, },
  { 0, 2, 4, 7, 8, 14, },
  { 1, 2, 4, 7, 8, 14, },
  { 0, 1, 2, 4, 7, 8, 14, },
  { 3, 4, 7, 8, 14, },
  { 0, 3, 4, 7, 8, 14, },
  { 1, 3, 4, 7, 8, 14, },
  { 0, 1, 3, 4, 7, 8, 14, },
  { 2, 3, 4, 7, 8, 14, },
  { 0, 2, 3, 4, 7, 8, 14, },
  { 1, 2, 3, 4, 7, 8, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 14, },
  { 5, 7, 8, 14, },
  { 0, 5, 7, 8, 14, },
  { 1, 5, 7, 8, 14, },
  { 0, 1, 5, 7, 8, 14, },
  { 2, 5, 7, 8, 14, },
  { 0, 2, 5, 7, 8, 14, },
  { 1, 2, 5, 7, 8, 14, },
  { 0, 1, 2, 5, 7, 8, 14, },
  { 3, 5, 7, 8, 14, },
  { 0, 3, 5, 7, 8, 14, },
  { 1, 3, 5, 7, 8, 14, },
  { 0, 1, 3, 5, 7, 8, 14, },
  { 2, 3, 5, 7, 8, 14, },
  { 0, 2, 3, 5, 7, 8, 14, },
  { 1, 2, 3, 5, 7, 8, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 14, },
  { 4, 5, 7, 8, 14, },
  { 0, 4, 5, 7, 8, 14, },
  { 1, 4, 5, 7, 8, 14, },
  { 0, 1, 4, 5, 7, 8, 14, },
  { 2, 4, 5, 7, 8, 14, },
  { 0, 2, 4, 5, 7, 8, 14, },
  { 1, 2, 4, 5, 7, 8, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 14, },
  { 3, 4, 5, 7, 8, 14, },
  { 0, 3, 4, 5, 7, 8, 14, },
  { 1, 3, 4, 5, 7, 8, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 14, },
  { 2, 3, 4, 5, 7, 8, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 14, },
  { 6, 7, 8, 14, },
  { 0, 6, 7, 8, 14, },
  { 1, 6, 7, 8, 14, },
  { 0, 1, 6, 7, 8, 14, },
  { 2, 6, 7, 8, 14, },
  { 0, 2, 6, 7, 8, 14, },
  { 1, 2, 6, 7, 8, 14, },
  { 0, 1, 2, 6, 7, 8, 14, },
  { 3, 6, 7, 8, 14, },
  { 0, 3, 6, 7, 8, 14, },
  { 1, 3, 6, 7, 8, 14, },
  { 0, 1, 3, 6, 7, 8, 14, },
  { 2, 3, 6, 7, 8, 14, },
  { 0, 2, 3, 6, 7, 8, 14, },
  { 1, 2, 3, 6, 7, 8, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 14, },
  { 4, 6, 7, 8, 14, },
  { 0, 4, 6, 7, 8, 14, },
  { 1, 4, 6, 7, 8, 14, },
  { 0, 1, 4, 6, 7, 8, 14, },
  { 2, 4, 6, 7, 8, 14, },
  { 0, 2, 4, 6, 7, 8, 14, },
  { 1, 2, 4, 6, 7, 8, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 14, },
  { 3, 4, 6, 7, 8, 14, },
  { 0, 3, 4, 6, 7, 8, 14, },
  { 1, 3, 4, 6, 7, 8, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 14, },
  { 2, 3, 4, 6, 7, 8, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 14, },
  { 5, 6, 7, 8, 14, },
  { 0, 5, 6, 7, 8, 14, },
  { 1, 5, 6, 7, 8, 14, },
  { 0, 1, 5, 6, 7, 8, 14, },
  { 2, 5, 6, 7, 8, 14, },
  { 0, 2, 5, 6, 7, 8, 14, },
  { 1, 2, 5, 6, 7, 8, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 14, },
  { 3, 5, 6, 7, 8, 14, },
  { 0, 3, 5, 6, 7, 8, 14, },
  { 1, 3, 5, 6, 7, 8, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 14, },
  { 2, 3, 5, 6, 7, 8, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 14, },
  { 4, 5, 6, 7, 8, 14, },
  { 0, 4, 5, 6, 7, 8, 14, },
  { 1, 4, 5, 6, 7, 8, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 14, },
  { 2, 4, 5, 6, 7, 8, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 14, },
  { 3, 4, 5, 6, 7, 8, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 14, },
  { 9, 14, },
  { 0, 9, 14, },
  { 1, 9, 14, },
  { 0, 1, 9, 14, },
  { 2, 9, 14, },
  { 0, 2, 9, 14, },
  { 1, 2, 9, 14, },
  { 0, 1, 2, 9, 14, },
  { 3, 9, 14, },
  { 0, 3, 9, 14, },
  { 1, 3, 9, 14, },
  { 0, 1, 3, 9, 14, },
  { 2, 3, 9, 14, },
  { 0, 2, 3, 9, 14, },
  { 1, 2, 3, 9, 14, },
  { 0, 1, 2, 3, 9, 14, },
  { 4, 9, 14, },
  { 0, 4, 9, 14, },
  { 1, 4, 9, 14, },
  { 0, 1, 4, 9, 14, },
  { 2, 4, 9, 14, },
  { 0, 2, 4, 9, 14, },
  { 1, 2, 4, 9, 14, },
  { 0, 1, 2, 4, 9, 14, },
  { 3, 4, 9, 14, },
  { 0, 3, 4, 9, 14, },
  { 1, 3, 4, 9, 14, },
  { 0, 1, 3, 4, 9, 14, },
  { 2, 3, 4, 9, 14, },
  { 0, 2, 3, 4, 9, 14, },
  { 1, 2, 3, 4, 9, 14, },
  { 0, 1, 2, 3, 4, 9, 14, },
  { 5, 9, 14, },
  { 0, 5, 9, 14, },
  { 1, 5, 9, 14, },
  { 0, 1, 5, 9, 14, },
  { 2, 5, 9, 14, },
  { 0, 2, 5, 9, 14, },
  { 1, 2, 5, 9, 14, },
  { 0, 1, 2, 5, 9, 14, },
  { 3, 5, 9, 14, },
  { 0, 3, 5, 9, 14, },
  { 1, 3, 5, 9, 14, },
  { 0, 1, 3, 5, 9, 14, },
  { 2, 3, 5, 9, 14, },
  { 0, 2, 3, 5, 9, 14, },
  { 1, 2, 3, 5, 9, 14, },
  { 0, 1, 2, 3, 5, 9, 14, },
  { 4, 5, 9, 14, },
  { 0, 4, 5, 9, 14, },
  { 1, 4, 5, 9, 14, },
  { 0, 1, 4, 5, 9, 14, },
  { 2, 4, 5, 9, 14, },
  { 0, 2, 4, 5, 9, 14, },
  { 1, 2, 4, 5, 9, 14, },
  { 0, 1, 2, 4, 5, 9, 14, },
  { 3, 4, 5, 9, 14, },
  { 0, 3, 4, 5, 9, 14, },
  { 1, 3, 4, 5, 9, 14, },
  { 0, 1, 3, 4, 5, 9, 14, },
  { 2, 3, 4, 5, 9, 14, },
  { 0, 2, 3, 4, 5, 9, 14, },
  { 1, 2, 3, 4, 5, 9, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 14, },
  { 6, 9, 14, },
  { 0, 6, 9, 14, },
  { 1, 6, 9, 14, },
  { 0, 1, 6, 9, 14, },
  { 2, 6, 9, 14, },
  { 0, 2, 6, 9, 14, },
  { 1, 2, 6, 9, 14, },
  { 0, 1, 2, 6, 9, 14, },
  { 3, 6, 9, 14, },
  { 0, 3, 6, 9, 14, },
  { 1, 3, 6, 9, 14, },
  { 0, 1, 3, 6, 9, 14, },
  { 2, 3, 6, 9, 14, },
  { 0, 2, 3, 6, 9, 14, },
  { 1, 2, 3, 6, 9, 14, },
  { 0, 1, 2, 3, 6, 9, 14, },
  { 4, 6, 9, 14, },
  { 0, 4, 6, 9, 14, },
  { 1, 4, 6, 9, 14, },
  { 0, 1, 4, 6, 9, 14, },
  { 2, 4, 6, 9, 14, },
  { 0, 2, 4, 6, 9, 14, },
  { 1, 2, 4, 6, 9, 14, },
  { 0, 1, 2, 4, 6, 9, 14, },
  { 3, 4, 6, 9, 14, },
  { 0, 3, 4, 6, 9, 14, },
  { 1, 3, 4, 6, 9, 14, },
  { 0, 1, 3, 4, 6, 9, 14, },
  { 2, 3, 4, 6, 9, 14, },
  { 0, 2, 3, 4, 6, 9, 14, },
  { 1, 2, 3, 4, 6, 9, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 14, },
  { 5, 6, 9, 14, },
  { 0, 5, 6, 9, 14, },
  { 1, 5, 6, 9, 14, },
  { 0, 1, 5, 6, 9, 14, },
  { 2, 5, 6, 9, 14, },
  { 0, 2, 5, 6, 9, 14, },
  { 1, 2, 5, 6, 9, 14, },
  { 0, 1, 2, 5, 6, 9, 14, },
  { 3, 5, 6, 9, 14, },
  { 0, 3, 5, 6, 9, 14, },
  { 1, 3, 5, 6, 9, 14, },
  { 0, 1, 3, 5, 6, 9, 14, },
  { 2, 3, 5, 6, 9, 14, },
  { 0, 2, 3, 5, 6, 9, 14, },
  { 1, 2, 3, 5, 6, 9, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 14, },
  { 4, 5, 6, 9, 14, },
  { 0, 4, 5, 6, 9, 14, },
  { 1, 4, 5, 6, 9, 14, },
  { 0, 1, 4, 5, 6, 9, 14, },
  { 2, 4, 5, 6, 9, 14, },
  { 0, 2, 4, 5, 6, 9, 14, },
  { 1, 2, 4, 5, 6, 9, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 14, },
  { 3, 4, 5, 6, 9, 14, },
  { 0, 3, 4, 5, 6, 9, 14, },
  { 1, 3, 4, 5, 6, 9, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 14, },
  { 2, 3, 4, 5, 6, 9, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 14, },
  { 7, 9, 14, },
  { 0, 7, 9, 14, },
  { 1, 7, 9, 14, },
  { 0, 1, 7, 9, 14, },
  { 2, 7, 9, 14, },
  { 0, 2, 7, 9, 14, },
  { 1, 2, 7, 9, 14, },
  { 0, 1, 2, 7, 9, 14, },
  { 3, 7, 9, 14, },
  { 0, 3, 7, 9, 14, },
  { 1, 3, 7, 9, 14, },
  { 0, 1, 3, 7, 9, 14, },
  { 2, 3, 7, 9, 14, },
  { 0, 2, 3, 7, 9, 14, },
  { 1, 2, 3, 7, 9, 14, },
  { 0, 1, 2, 3, 7, 9, 14, },
  { 4, 7, 9, 14, },
  { 0, 4, 7, 9, 14, },
  { 1, 4, 7, 9, 14, },
  { 0, 1, 4, 7, 9, 14, },
  { 2, 4, 7, 9, 14, },
  { 0, 2, 4, 7, 9, 14, },
  { 1, 2, 4, 7, 9, 14, },
  { 0, 1, 2, 4, 7, 9, 14, },
  { 3, 4, 7, 9, 14, },
  { 0, 3, 4, 7, 9, 14, },
  { 1, 3, 4, 7, 9, 14, },
  { 0, 1, 3, 4, 7, 9, 14, },
  { 2, 3, 4, 7, 9, 14, },
  { 0, 2, 3, 4, 7, 9, 14, },
  { 1, 2, 3, 4, 7, 9, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 14, },
  { 5, 7, 9, 14, },
  { 0, 5, 7, 9, 14, },
  { 1, 5, 7, 9, 14, },
  { 0, 1, 5, 7, 9, 14, },
  { 2, 5, 7, 9, 14, },
  { 0, 2, 5, 7, 9, 14, },
  { 1, 2, 5, 7, 9, 14, },
  { 0, 1, 2, 5, 7, 9, 14, },
  { 3, 5, 7, 9, 14, },
  { 0, 3, 5, 7, 9, 14, },
  { 1, 3, 5, 7, 9, 14, },
  { 0, 1, 3, 5, 7, 9, 14, },
  { 2, 3, 5, 7, 9, 14, },
  { 0, 2, 3, 5, 7, 9, 14, },
  { 1, 2, 3, 5, 7, 9, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 14, },
  { 4, 5, 7, 9, 14, },
  { 0, 4, 5, 7, 9, 14, },
  { 1, 4, 5, 7, 9, 14, },
  { 0, 1, 4, 5, 7, 9, 14, },
  { 2, 4, 5, 7, 9, 14, },
  { 0, 2, 4, 5, 7, 9, 14, },
  { 1, 2, 4, 5, 7, 9, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 14, },
  { 3, 4, 5, 7, 9, 14, },
  { 0, 3, 4, 5, 7, 9, 14, },
  { 1, 3, 4, 5, 7, 9, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 14, },
  { 2, 3, 4, 5, 7, 9, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 14, },
  { 6, 7, 9, 14, },
  { 0, 6, 7, 9, 14, },
  { 1, 6, 7, 9, 14, },
  { 0, 1, 6, 7, 9, 14, },
  { 2, 6, 7, 9, 14, },
  { 0, 2, 6, 7, 9, 14, },
  { 1, 2, 6, 7, 9, 14, },
  { 0, 1, 2, 6, 7, 9, 14, },
  { 3, 6, 7, 9, 14, },
  { 0, 3, 6, 7, 9, 14, },
  { 1, 3, 6, 7, 9, 14, },
  { 0, 1, 3, 6, 7, 9, 14, },
  { 2, 3, 6, 7, 9, 14, },
  { 0, 2, 3, 6, 7, 9, 14, },
  { 1, 2, 3, 6, 7, 9, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 14, },
  { 4, 6, 7, 9, 14, },
  { 0, 4, 6, 7, 9, 14, },
  { 1, 4, 6, 7, 9, 14, },
  { 0, 1, 4, 6, 7, 9, 14, },
  { 2, 4, 6, 7, 9, 14, },
  { 0, 2, 4, 6, 7, 9, 14, },
  { 1, 2, 4, 6, 7, 9, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 14, },
  { 3, 4, 6, 7, 9, 14, },
  { 0, 3, 4, 6, 7, 9, 14, },
  { 1, 3, 4, 6, 7, 9, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 14, },
  { 2, 3, 4, 6, 7, 9, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 14, },
  { 5, 6, 7, 9, 14, },
  { 0, 5, 6, 7, 9, 14, },
  { 1, 5, 6, 7, 9, 14, },
  { 0, 1, 5, 6, 7, 9, 14, },
  { 2, 5, 6, 7, 9, 14, },
  { 0, 2, 5, 6, 7, 9, 14, },
  { 1, 2, 5, 6, 7, 9, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 14, },
  { 3, 5, 6, 7, 9, 14, },
  { 0, 3, 5, 6, 7, 9, 14, },
  { 1, 3, 5, 6, 7, 9, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 14, },
  { 2, 3, 5, 6, 7, 9, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 14, },
  { 4, 5, 6, 7, 9, 14, },
  { 0, 4, 5, 6, 7, 9, 14, },
  { 1, 4, 5, 6, 7, 9, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 14, },
  { 2, 4, 5, 6, 7, 9, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 14, },
  { 3, 4, 5, 6, 7, 9, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 14, },
  { 8, 9, 14, },
  { 0, 8, 9, 14, },
  { 1, 8, 9, 14, },
  { 0, 1, 8, 9, 14, },
  { 2, 8, 9, 14, },
  { 0, 2, 8, 9, 14, },
  { 1, 2, 8, 9, 14, },
  { 0, 1, 2, 8, 9, 14, },
  { 3, 8, 9, 14, },
  { 0, 3, 8, 9, 14, },
  { 1, 3, 8, 9, 14, },
  { 0, 1, 3, 8, 9, 14, },
  { 2, 3, 8, 9, 14, },
  { 0, 2, 3, 8, 9, 14, },
  { 1, 2, 3, 8, 9, 14, },
  { 0, 1, 2, 3, 8, 9, 14, },
  { 4, 8, 9, 14, },
  { 0, 4, 8, 9, 14, },
  { 1, 4, 8, 9, 14, },
  { 0, 1, 4, 8, 9, 14, },
  { 2, 4, 8, 9, 14, },
  { 0, 2, 4, 8, 9, 14, },
  { 1, 2, 4, 8, 9, 14, },
  { 0, 1, 2, 4, 8, 9, 14, },
  { 3, 4, 8, 9, 14, },
  { 0, 3, 4, 8, 9, 14, },
  { 1, 3, 4, 8, 9, 14, },
  { 0, 1, 3, 4, 8, 9, 14, },
  { 2, 3, 4, 8, 9, 14, },
  { 0, 2, 3, 4, 8, 9, 14, },
  { 1, 2, 3, 4, 8, 9, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 14, },
  { 5, 8, 9, 14, },
  { 0, 5, 8, 9, 14, },
  { 1, 5, 8, 9, 14, },
  { 0, 1, 5, 8, 9, 14, },
  { 2, 5, 8, 9, 14, },
  { 0, 2, 5, 8, 9, 14, },
  { 1, 2, 5, 8, 9, 14, },
  { 0, 1, 2, 5, 8, 9, 14, },
  { 3, 5, 8, 9, 14, },
  { 0, 3, 5, 8, 9, 14, },
  { 1, 3, 5, 8, 9, 14, },
  { 0, 1, 3, 5, 8, 9, 14, },
  { 2, 3, 5, 8, 9, 14, },
  { 0, 2, 3, 5, 8, 9, 14, },
  { 1, 2, 3, 5, 8, 9, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 14, },
  { 4, 5, 8, 9, 14, },
  { 0, 4, 5, 8, 9, 14, },
  { 1, 4, 5, 8, 9, 14, },
  { 0, 1, 4, 5, 8, 9, 14, },
  { 2, 4, 5, 8, 9, 14, },
  { 0, 2, 4, 5, 8, 9, 14, },
  { 1, 2, 4, 5, 8, 9, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 14, },
  { 3, 4, 5, 8, 9, 14, },
  { 0, 3, 4, 5, 8, 9, 14, },
  { 1, 3, 4, 5, 8, 9, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 14, },
  { 2, 3, 4, 5, 8, 9, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 14, },
  { 6, 8, 9, 14, },
  { 0, 6, 8, 9, 14, },
  { 1, 6, 8, 9, 14, },
  { 0, 1, 6, 8, 9, 14, },
  { 2, 6, 8, 9, 14, },
  { 0, 2, 6, 8, 9, 14, },
  { 1, 2, 6, 8, 9, 14, },
  { 0, 1, 2, 6, 8, 9, 14, },
  { 3, 6, 8, 9, 14, },
  { 0, 3, 6, 8, 9, 14, },
  { 1, 3, 6, 8, 9, 14, },
  { 0, 1, 3, 6, 8, 9, 14, },
  { 2, 3, 6, 8, 9, 14, },
  { 0, 2, 3, 6, 8, 9, 14, },
  { 1, 2, 3, 6, 8, 9, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 14, },
  { 4, 6, 8, 9, 14, },
  { 0, 4, 6, 8, 9, 14, },
  { 1, 4, 6, 8, 9, 14, },
  { 0, 1, 4, 6, 8, 9, 14, },
  { 2, 4, 6, 8, 9, 14, },
  { 0, 2, 4, 6, 8, 9, 14, },
  { 1, 2, 4, 6, 8, 9, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 14, },
  { 3, 4, 6, 8, 9, 14, },
  { 0, 3, 4, 6, 8, 9, 14, },
  { 1, 3, 4, 6, 8, 9, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 14, },
  { 2, 3, 4, 6, 8, 9, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 14, },
  { 5, 6, 8, 9, 14, },
  { 0, 5, 6, 8, 9, 14, },
  { 1, 5, 6, 8, 9, 14, },
  { 0, 1, 5, 6, 8, 9, 14, },
  { 2, 5, 6, 8, 9, 14, },
  { 0, 2, 5, 6, 8, 9, 14, },
  { 1, 2, 5, 6, 8, 9, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 14, },
  { 3, 5, 6, 8, 9, 14, },
  { 0, 3, 5, 6, 8, 9, 14, },
  { 1, 3, 5, 6, 8, 9, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 14, },
  { 2, 3, 5, 6, 8, 9, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 14, },
  { 4, 5, 6, 8, 9, 14, },
  { 0, 4, 5, 6, 8, 9, 14, },
  { 1, 4, 5, 6, 8, 9, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 14, },
  { 2, 4, 5, 6, 8, 9, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 14, },
  { 3, 4, 5, 6, 8, 9, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 14, },
  { 7, 8, 9, 14, },
  { 0, 7, 8, 9, 14, },
  { 1, 7, 8, 9, 14, },
  { 0, 1, 7, 8, 9, 14, },
  { 2, 7, 8, 9, 14, },
  { 0, 2, 7, 8, 9, 14, },
  { 1, 2, 7, 8, 9, 14, },
  { 0, 1, 2, 7, 8, 9, 14, },
  { 3, 7, 8, 9, 14, },
  { 0, 3, 7, 8, 9, 14, },
  { 1, 3, 7, 8, 9, 14, },
  { 0, 1, 3, 7, 8, 9, 14, },
  { 2, 3, 7, 8, 9, 14, },
  { 0, 2, 3, 7, 8, 9, 14, },
  { 1, 2, 3, 7, 8, 9, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 14, },
  { 4, 7, 8, 9, 14, },
  { 0, 4, 7, 8, 9, 14, },
  { 1, 4, 7, 8, 9, 14, },
  { 0, 1, 4, 7, 8, 9, 14, },
  { 2, 4, 7, 8, 9, 14, },
  { 0, 2, 4, 7, 8, 9, 14, },
  { 1, 2, 4, 7, 8, 9, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 14, },
  { 3, 4, 7, 8, 9, 14, },
  { 0, 3, 4, 7, 8, 9, 14, },
  { 1, 3, 4, 7, 8, 9, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 14, },
  { 2, 3, 4, 7, 8, 9, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 14, },
  { 5, 7, 8, 9, 14, },
  { 0, 5, 7, 8, 9, 14, },
  { 1, 5, 7, 8, 9, 14, },
  { 0, 1, 5, 7, 8, 9, 14, },
  { 2, 5, 7, 8, 9, 14, },
  { 0, 2, 5, 7, 8, 9, 14, },
  { 1, 2, 5, 7, 8, 9, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 14, },
  { 3, 5, 7, 8, 9, 14, },
  { 0, 3, 5, 7, 8, 9, 14, },
  { 1, 3, 5, 7, 8, 9, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 14, },
  { 2, 3, 5, 7, 8, 9, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 14, },
  { 4, 5, 7, 8, 9, 14, },
  { 0, 4, 5, 7, 8, 9, 14, },
  { 1, 4, 5, 7, 8, 9, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 14, },
  { 2, 4, 5, 7, 8, 9, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 14, },
  { 3, 4, 5, 7, 8, 9, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 14, },
  { 6, 7, 8, 9, 14, },
  { 0, 6, 7, 8, 9, 14, },
  { 1, 6, 7, 8, 9, 14, },
  { 0, 1, 6, 7, 8, 9, 14, },
  { 2, 6, 7, 8, 9, 14, },
  { 0, 2, 6, 7, 8, 9, 14, },
  { 1, 2, 6, 7, 8, 9, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 14, },
  { 3, 6, 7, 8, 9, 14, },
  { 0, 3, 6, 7, 8, 9, 14, },
  { 1, 3, 6, 7, 8, 9, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 14, },
  { 2, 3, 6, 7, 8, 9, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 14, },
  { 4, 6, 7, 8, 9, 14, },
  { 0, 4, 6, 7, 8, 9, 14, },
  { 1, 4, 6, 7, 8, 9, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 14, },
  { 2, 4, 6, 7, 8, 9, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 14, },
  { 3, 4, 6, 7, 8, 9, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 14, },
  { 5, 6, 7, 8, 9, 14, },
  { 0, 5, 6, 7, 8, 9, 14, },
  { 1, 5, 6, 7, 8, 9, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 14, },
  { 2, 5, 6, 7, 8, 9, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 14, },
  { 3, 5, 6, 7, 8, 9, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 14, },
  { 4, 5, 6, 7, 8, 9, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 14, },
  { 10, 14, },
  { 0, 10, 14, },
  { 1, 10, 14, },
  { 0, 1, 10, 14, },
  { 2, 10, 14, },
  { 0, 2, 10, 14, },
  { 1, 2, 10, 14, },
  { 0, 1, 2, 10, 14, },
  { 3, 10, 14, },
  { 0, 3, 10, 14, },
  { 1, 3, 10, 14, },
  { 0, 1, 3, 10, 14, },
  { 2, 3, 10, 14, },
  { 0, 2, 3, 10, 14, },
  { 1, 2, 3, 10, 14, },
  { 0, 1, 2, 3, 10, 14, },
  { 4, 10, 14, },
  { 0, 4, 10, 14, },
  { 1, 4, 10, 14, },
  { 0, 1, 4, 10, 14, },
  { 2, 4, 10, 14, },
  { 0, 2, 4, 10, 14, },
  { 1, 2, 4, 10, 14, },
  { 0, 1, 2, 4, 10, 14, },
  { 3, 4, 10, 14, },
  { 0, 3, 4, 10, 14, },
  { 1, 3, 4, 10, 14, },
  { 0, 1, 3, 4, 10, 14, },
  { 2, 3, 4, 10, 14, },
  { 0, 2, 3, 4, 10, 14, },
  { 1, 2, 3, 4, 10, 14, },
  { 0, 1, 2, 3, 4, 10, 14, },
  { 5, 10, 14, },
  { 0, 5, 10, 14, },
  { 1, 5, 10, 14, },
  { 0, 1, 5, 10, 14, },
  { 2, 5, 10, 14, },
  { 0, 2, 5, 10, 14, },
  { 1, 2, 5, 10, 14, },
  { 0, 1, 2, 5, 10, 14, },
  { 3, 5, 10, 14, },
  { 0, 3, 5, 10, 14, },
  { 1, 3, 5, 10, 14, },
  { 0, 1, 3, 5, 10, 14, },
  { 2, 3, 5, 10, 14, },
  { 0, 2, 3, 5, 10, 14, },
  { 1, 2, 3, 5, 10, 14, },
  { 0, 1, 2, 3, 5, 10, 14, },
  { 4, 5, 10, 14, },
  { 0, 4, 5, 10, 14, },
  { 1, 4, 5, 10, 14, },
  { 0, 1, 4, 5, 10, 14, },
  { 2, 4, 5, 10, 14, },
  { 0, 2, 4, 5, 10, 14, },
  { 1, 2, 4, 5, 10, 14, },
  { 0, 1, 2, 4, 5, 10, 14, },
  { 3, 4, 5, 10, 14, },
  { 0, 3, 4, 5, 10, 14, },
  { 1, 3, 4, 5, 10, 14, },
  { 0, 1, 3, 4, 5, 10, 14, },
  { 2, 3, 4, 5, 10, 14, },
  { 0, 2, 3, 4, 5, 10, 14, },
  { 1, 2, 3, 4, 5, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 10, 14, },
  { 6, 10, 14, },
  { 0, 6, 10, 14, },
  { 1, 6, 10, 14, },
  { 0, 1, 6, 10, 14, },
  { 2, 6, 10, 14, },
  { 0, 2, 6, 10, 14, },
  { 1, 2, 6, 10, 14, },
  { 0, 1, 2, 6, 10, 14, },
  { 3, 6, 10, 14, },
  { 0, 3, 6, 10, 14, },
  { 1, 3, 6, 10, 14, },
  { 0, 1, 3, 6, 10, 14, },
  { 2, 3, 6, 10, 14, },
  { 0, 2, 3, 6, 10, 14, },
  { 1, 2, 3, 6, 10, 14, },
  { 0, 1, 2, 3, 6, 10, 14, },
  { 4, 6, 10, 14, },
  { 0, 4, 6, 10, 14, },
  { 1, 4, 6, 10, 14, },
  { 0, 1, 4, 6, 10, 14, },
  { 2, 4, 6, 10, 14, },
  { 0, 2, 4, 6, 10, 14, },
  { 1, 2, 4, 6, 10, 14, },
  { 0, 1, 2, 4, 6, 10, 14, },
  { 3, 4, 6, 10, 14, },
  { 0, 3, 4, 6, 10, 14, },
  { 1, 3, 4, 6, 10, 14, },
  { 0, 1, 3, 4, 6, 10, 14, },
  { 2, 3, 4, 6, 10, 14, },
  { 0, 2, 3, 4, 6, 10, 14, },
  { 1, 2, 3, 4, 6, 10, 14, },
  { 0, 1, 2, 3, 4, 6, 10, 14, },
  { 5, 6, 10, 14, },
  { 0, 5, 6, 10, 14, },
  { 1, 5, 6, 10, 14, },
  { 0, 1, 5, 6, 10, 14, },
  { 2, 5, 6, 10, 14, },
  { 0, 2, 5, 6, 10, 14, },
  { 1, 2, 5, 6, 10, 14, },
  { 0, 1, 2, 5, 6, 10, 14, },
  { 3, 5, 6, 10, 14, },
  { 0, 3, 5, 6, 10, 14, },
  { 1, 3, 5, 6, 10, 14, },
  { 0, 1, 3, 5, 6, 10, 14, },
  { 2, 3, 5, 6, 10, 14, },
  { 0, 2, 3, 5, 6, 10, 14, },
  { 1, 2, 3, 5, 6, 10, 14, },
  { 0, 1, 2, 3, 5, 6, 10, 14, },
  { 4, 5, 6, 10, 14, },
  { 0, 4, 5, 6, 10, 14, },
  { 1, 4, 5, 6, 10, 14, },
  { 0, 1, 4, 5, 6, 10, 14, },
  { 2, 4, 5, 6, 10, 14, },
  { 0, 2, 4, 5, 6, 10, 14, },
  { 1, 2, 4, 5, 6, 10, 14, },
  { 0, 1, 2, 4, 5, 6, 10, 14, },
  { 3, 4, 5, 6, 10, 14, },
  { 0, 3, 4, 5, 6, 10, 14, },
  { 1, 3, 4, 5, 6, 10, 14, },
  { 0, 1, 3, 4, 5, 6, 10, 14, },
  { 2, 3, 4, 5, 6, 10, 14, },
  { 0, 2, 3, 4, 5, 6, 10, 14, },
  { 1, 2, 3, 4, 5, 6, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 14, },
  { 7, 10, 14, },
  { 0, 7, 10, 14, },
  { 1, 7, 10, 14, },
  { 0, 1, 7, 10, 14, },
  { 2, 7, 10, 14, },
  { 0, 2, 7, 10, 14, },
  { 1, 2, 7, 10, 14, },
  { 0, 1, 2, 7, 10, 14, },
  { 3, 7, 10, 14, },
  { 0, 3, 7, 10, 14, },
  { 1, 3, 7, 10, 14, },
  { 0, 1, 3, 7, 10, 14, },
  { 2, 3, 7, 10, 14, },
  { 0, 2, 3, 7, 10, 14, },
  { 1, 2, 3, 7, 10, 14, },
  { 0, 1, 2, 3, 7, 10, 14, },
  { 4, 7, 10, 14, },
  { 0, 4, 7, 10, 14, },
  { 1, 4, 7, 10, 14, },
  { 0, 1, 4, 7, 10, 14, },
  { 2, 4, 7, 10, 14, },
  { 0, 2, 4, 7, 10, 14, },
  { 1, 2, 4, 7, 10, 14, },
  { 0, 1, 2, 4, 7, 10, 14, },
  { 3, 4, 7, 10, 14, },
  { 0, 3, 4, 7, 10, 14, },
  { 1, 3, 4, 7, 10, 14, },
  { 0, 1, 3, 4, 7, 10, 14, },
  { 2, 3, 4, 7, 10, 14, },
  { 0, 2, 3, 4, 7, 10, 14, },
  { 1, 2, 3, 4, 7, 10, 14, },
  { 0, 1, 2, 3, 4, 7, 10, 14, },
  { 5, 7, 10, 14, },
  { 0, 5, 7, 10, 14, },
  { 1, 5, 7, 10, 14, },
  { 0, 1, 5, 7, 10, 14, },
  { 2, 5, 7, 10, 14, },
  { 0, 2, 5, 7, 10, 14, },
  { 1, 2, 5, 7, 10, 14, },
  { 0, 1, 2, 5, 7, 10, 14, },
  { 3, 5, 7, 10, 14, },
  { 0, 3, 5, 7, 10, 14, },
  { 1, 3, 5, 7, 10, 14, },
  { 0, 1, 3, 5, 7, 10, 14, },
  { 2, 3, 5, 7, 10, 14, },
  { 0, 2, 3, 5, 7, 10, 14, },
  { 1, 2, 3, 5, 7, 10, 14, },
  { 0, 1, 2, 3, 5, 7, 10, 14, },
  { 4, 5, 7, 10, 14, },
  { 0, 4, 5, 7, 10, 14, },
  { 1, 4, 5, 7, 10, 14, },
  { 0, 1, 4, 5, 7, 10, 14, },
  { 2, 4, 5, 7, 10, 14, },
  { 0, 2, 4, 5, 7, 10, 14, },
  { 1, 2, 4, 5, 7, 10, 14, },
  { 0, 1, 2, 4, 5, 7, 10, 14, },
  { 3, 4, 5, 7, 10, 14, },
  { 0, 3, 4, 5, 7, 10, 14, },
  { 1, 3, 4, 5, 7, 10, 14, },
  { 0, 1, 3, 4, 5, 7, 10, 14, },
  { 2, 3, 4, 5, 7, 10, 14, },
  { 0, 2, 3, 4, 5, 7, 10, 14, },
  { 1, 2, 3, 4, 5, 7, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 14, },
  { 6, 7, 10, 14, },
  { 0, 6, 7, 10, 14, },
  { 1, 6, 7, 10, 14, },
  { 0, 1, 6, 7, 10, 14, },
  { 2, 6, 7, 10, 14, },
  { 0, 2, 6, 7, 10, 14, },
  { 1, 2, 6, 7, 10, 14, },
  { 0, 1, 2, 6, 7, 10, 14, },
  { 3, 6, 7, 10, 14, },
  { 0, 3, 6, 7, 10, 14, },
  { 1, 3, 6, 7, 10, 14, },
  { 0, 1, 3, 6, 7, 10, 14, },
  { 2, 3, 6, 7, 10, 14, },
  { 0, 2, 3, 6, 7, 10, 14, },
  { 1, 2, 3, 6, 7, 10, 14, },
  { 0, 1, 2, 3, 6, 7, 10, 14, },
  { 4, 6, 7, 10, 14, },
  { 0, 4, 6, 7, 10, 14, },
  { 1, 4, 6, 7, 10, 14, },
  { 0, 1, 4, 6, 7, 10, 14, },
  { 2, 4, 6, 7, 10, 14, },
  { 0, 2, 4, 6, 7, 10, 14, },
  { 1, 2, 4, 6, 7, 10, 14, },
  { 0, 1, 2, 4, 6, 7, 10, 14, },
  { 3, 4, 6, 7, 10, 14, },
  { 0, 3, 4, 6, 7, 10, 14, },
  { 1, 3, 4, 6, 7, 10, 14, },
  { 0, 1, 3, 4, 6, 7, 10, 14, },
  { 2, 3, 4, 6, 7, 10, 14, },
  { 0, 2, 3, 4, 6, 7, 10, 14, },
  { 1, 2, 3, 4, 6, 7, 10, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 14, },
  { 5, 6, 7, 10, 14, },
  { 0, 5, 6, 7, 10, 14, },
  { 1, 5, 6, 7, 10, 14, },
  { 0, 1, 5, 6, 7, 10, 14, },
  { 2, 5, 6, 7, 10, 14, },
  { 0, 2, 5, 6, 7, 10, 14, },
  { 1, 2, 5, 6, 7, 10, 14, },
  { 0, 1, 2, 5, 6, 7, 10, 14, },
  { 3, 5, 6, 7, 10, 14, },
  { 0, 3, 5, 6, 7, 10, 14, },
  { 1, 3, 5, 6, 7, 10, 14, },
  { 0, 1, 3, 5, 6, 7, 10, 14, },
  { 2, 3, 5, 6, 7, 10, 14, },
  { 0, 2, 3, 5, 6, 7, 10, 14, },
  { 1, 2, 3, 5, 6, 7, 10, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 14, },
  { 4, 5, 6, 7, 10, 14, },
  { 0, 4, 5, 6, 7, 10, 14, },
  { 1, 4, 5, 6, 7, 10, 14, },
  { 0, 1, 4, 5, 6, 7, 10, 14, },
  { 2, 4, 5, 6, 7, 10, 14, },
  { 0, 2, 4, 5, 6, 7, 10, 14, },
  { 1, 2, 4, 5, 6, 7, 10, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 14, },
  { 3, 4, 5, 6, 7, 10, 14, },
  { 0, 3, 4, 5, 6, 7, 10, 14, },
  { 1, 3, 4, 5, 6, 7, 10, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 14, },
  { 2, 3, 4, 5, 6, 7, 10, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 14, },
  { 8, 10, 14, },
  { 0, 8, 10, 14, },
  { 1, 8, 10, 14, },
  { 0, 1, 8, 10, 14, },
  { 2, 8, 10, 14, },
  { 0, 2, 8, 10, 14, },
  { 1, 2, 8, 10, 14, },
  { 0, 1, 2, 8, 10, 14, },
  { 3, 8, 10, 14, },
  { 0, 3, 8, 10, 14, },
  { 1, 3, 8, 10, 14, },
  { 0, 1, 3, 8, 10, 14, },
  { 2, 3, 8, 10, 14, },
  { 0, 2, 3, 8, 10, 14, },
  { 1, 2, 3, 8, 10, 14, },
  { 0, 1, 2, 3, 8, 10, 14, },
  { 4, 8, 10, 14, },
  { 0, 4, 8, 10, 14, },
  { 1, 4, 8, 10, 14, },
  { 0, 1, 4, 8, 10, 14, },
  { 2, 4, 8, 10, 14, },
  { 0, 2, 4, 8, 10, 14, },
  { 1, 2, 4, 8, 10, 14, },
  { 0, 1, 2, 4, 8, 10, 14, },
  { 3, 4, 8, 10, 14, },
  { 0, 3, 4, 8, 10, 14, },
  { 1, 3, 4, 8, 10, 14, },
  { 0, 1, 3, 4, 8, 10, 14, },
  { 2, 3, 4, 8, 10, 14, },
  { 0, 2, 3, 4, 8, 10, 14, },
  { 1, 2, 3, 4, 8, 10, 14, },
  { 0, 1, 2, 3, 4, 8, 10, 14, },
  { 5, 8, 10, 14, },
  { 0, 5, 8, 10, 14, },
  { 1, 5, 8, 10, 14, },
  { 0, 1, 5, 8, 10, 14, },
  { 2, 5, 8, 10, 14, },
  { 0, 2, 5, 8, 10, 14, },
  { 1, 2, 5, 8, 10, 14, },
  { 0, 1, 2, 5, 8, 10, 14, },
  { 3, 5, 8, 10, 14, },
  { 0, 3, 5, 8, 10, 14, },
  { 1, 3, 5, 8, 10, 14, },
  { 0, 1, 3, 5, 8, 10, 14, },
  { 2, 3, 5, 8, 10, 14, },
  { 0, 2, 3, 5, 8, 10, 14, },
  { 1, 2, 3, 5, 8, 10, 14, },
  { 0, 1, 2, 3, 5, 8, 10, 14, },
  { 4, 5, 8, 10, 14, },
  { 0, 4, 5, 8, 10, 14, },
  { 1, 4, 5, 8, 10, 14, },
  { 0, 1, 4, 5, 8, 10, 14, },
  { 2, 4, 5, 8, 10, 14, },
  { 0, 2, 4, 5, 8, 10, 14, },
  { 1, 2, 4, 5, 8, 10, 14, },
  { 0, 1, 2, 4, 5, 8, 10, 14, },
  { 3, 4, 5, 8, 10, 14, },
  { 0, 3, 4, 5, 8, 10, 14, },
  { 1, 3, 4, 5, 8, 10, 14, },
  { 0, 1, 3, 4, 5, 8, 10, 14, },
  { 2, 3, 4, 5, 8, 10, 14, },
  { 0, 2, 3, 4, 5, 8, 10, 14, },
  { 1, 2, 3, 4, 5, 8, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 14, },
  { 6, 8, 10, 14, },
  { 0, 6, 8, 10, 14, },
  { 1, 6, 8, 10, 14, },
  { 0, 1, 6, 8, 10, 14, },
  { 2, 6, 8, 10, 14, },
  { 0, 2, 6, 8, 10, 14, },
  { 1, 2, 6, 8, 10, 14, },
  { 0, 1, 2, 6, 8, 10, 14, },
  { 3, 6, 8, 10, 14, },
  { 0, 3, 6, 8, 10, 14, },
  { 1, 3, 6, 8, 10, 14, },
  { 0, 1, 3, 6, 8, 10, 14, },
  { 2, 3, 6, 8, 10, 14, },
  { 0, 2, 3, 6, 8, 10, 14, },
  { 1, 2, 3, 6, 8, 10, 14, },
  { 0, 1, 2, 3, 6, 8, 10, 14, },
  { 4, 6, 8, 10, 14, },
  { 0, 4, 6, 8, 10, 14, },
  { 1, 4, 6, 8, 10, 14, },
  { 0, 1, 4, 6, 8, 10, 14, },
  { 2, 4, 6, 8, 10, 14, },
  { 0, 2, 4, 6, 8, 10, 14, },
  { 1, 2, 4, 6, 8, 10, 14, },
  { 0, 1, 2, 4, 6, 8, 10, 14, },
  { 3, 4, 6, 8, 10, 14, },
  { 0, 3, 4, 6, 8, 10, 14, },
  { 1, 3, 4, 6, 8, 10, 14, },
  { 0, 1, 3, 4, 6, 8, 10, 14, },
  { 2, 3, 4, 6, 8, 10, 14, },
  { 0, 2, 3, 4, 6, 8, 10, 14, },
  { 1, 2, 3, 4, 6, 8, 10, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 14, },
  { 5, 6, 8, 10, 14, },
  { 0, 5, 6, 8, 10, 14, },
  { 1, 5, 6, 8, 10, 14, },
  { 0, 1, 5, 6, 8, 10, 14, },
  { 2, 5, 6, 8, 10, 14, },
  { 0, 2, 5, 6, 8, 10, 14, },
  { 1, 2, 5, 6, 8, 10, 14, },
  { 0, 1, 2, 5, 6, 8, 10, 14, },
  { 3, 5, 6, 8, 10, 14, },
  { 0, 3, 5, 6, 8, 10, 14, },
  { 1, 3, 5, 6, 8, 10, 14, },
  { 0, 1, 3, 5, 6, 8, 10, 14, },
  { 2, 3, 5, 6, 8, 10, 14, },
  { 0, 2, 3, 5, 6, 8, 10, 14, },
  { 1, 2, 3, 5, 6, 8, 10, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 14, },
  { 4, 5, 6, 8, 10, 14, },
  { 0, 4, 5, 6, 8, 10, 14, },
  { 1, 4, 5, 6, 8, 10, 14, },
  { 0, 1, 4, 5, 6, 8, 10, 14, },
  { 2, 4, 5, 6, 8, 10, 14, },
  { 0, 2, 4, 5, 6, 8, 10, 14, },
  { 1, 2, 4, 5, 6, 8, 10, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 14, },
  { 3, 4, 5, 6, 8, 10, 14, },
  { 0, 3, 4, 5, 6, 8, 10, 14, },
  { 1, 3, 4, 5, 6, 8, 10, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 14, },
  { 2, 3, 4, 5, 6, 8, 10, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 14, },
  { 7, 8, 10, 14, },
  { 0, 7, 8, 10, 14, },
  { 1, 7, 8, 10, 14, },
  { 0, 1, 7, 8, 10, 14, },
  { 2, 7, 8, 10, 14, },
  { 0, 2, 7, 8, 10, 14, },
  { 1, 2, 7, 8, 10, 14, },
  { 0, 1, 2, 7, 8, 10, 14, },
  { 3, 7, 8, 10, 14, },
  { 0, 3, 7, 8, 10, 14, },
  { 1, 3, 7, 8, 10, 14, },
  { 0, 1, 3, 7, 8, 10, 14, },
  { 2, 3, 7, 8, 10, 14, },
  { 0, 2, 3, 7, 8, 10, 14, },
  { 1, 2, 3, 7, 8, 10, 14, },
  { 0, 1, 2, 3, 7, 8, 10, 14, },
  { 4, 7, 8, 10, 14, },
  { 0, 4, 7, 8, 10, 14, },
  { 1, 4, 7, 8, 10, 14, },
  { 0, 1, 4, 7, 8, 10, 14, },
  { 2, 4, 7, 8, 10, 14, },
  { 0, 2, 4, 7, 8, 10, 14, },
  { 1, 2, 4, 7, 8, 10, 14, },
  { 0, 1, 2, 4, 7, 8, 10, 14, },
  { 3, 4, 7, 8, 10, 14, },
  { 0, 3, 4, 7, 8, 10, 14, },
  { 1, 3, 4, 7, 8, 10, 14, },
  { 0, 1, 3, 4, 7, 8, 10, 14, },
  { 2, 3, 4, 7, 8, 10, 14, },
  { 0, 2, 3, 4, 7, 8, 10, 14, },
  { 1, 2, 3, 4, 7, 8, 10, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 14, },
  { 5, 7, 8, 10, 14, },
  { 0, 5, 7, 8, 10, 14, },
  { 1, 5, 7, 8, 10, 14, },
  { 0, 1, 5, 7, 8, 10, 14, },
  { 2, 5, 7, 8, 10, 14, },
  { 0, 2, 5, 7, 8, 10, 14, },
  { 1, 2, 5, 7, 8, 10, 14, },
  { 0, 1, 2, 5, 7, 8, 10, 14, },
  { 3, 5, 7, 8, 10, 14, },
  { 0, 3, 5, 7, 8, 10, 14, },
  { 1, 3, 5, 7, 8, 10, 14, },
  { 0, 1, 3, 5, 7, 8, 10, 14, },
  { 2, 3, 5, 7, 8, 10, 14, },
  { 0, 2, 3, 5, 7, 8, 10, 14, },
  { 1, 2, 3, 5, 7, 8, 10, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 14, },
  { 4, 5, 7, 8, 10, 14, },
  { 0, 4, 5, 7, 8, 10, 14, },
  { 1, 4, 5, 7, 8, 10, 14, },
  { 0, 1, 4, 5, 7, 8, 10, 14, },
  { 2, 4, 5, 7, 8, 10, 14, },
  { 0, 2, 4, 5, 7, 8, 10, 14, },
  { 1, 2, 4, 5, 7, 8, 10, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 14, },
  { 3, 4, 5, 7, 8, 10, 14, },
  { 0, 3, 4, 5, 7, 8, 10, 14, },
  { 1, 3, 4, 5, 7, 8, 10, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 14, },
  { 2, 3, 4, 5, 7, 8, 10, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 14, },
  { 6, 7, 8, 10, 14, },
  { 0, 6, 7, 8, 10, 14, },
  { 1, 6, 7, 8, 10, 14, },
  { 0, 1, 6, 7, 8, 10, 14, },
  { 2, 6, 7, 8, 10, 14, },
  { 0, 2, 6, 7, 8, 10, 14, },
  { 1, 2, 6, 7, 8, 10, 14, },
  { 0, 1, 2, 6, 7, 8, 10, 14, },
  { 3, 6, 7, 8, 10, 14, },
  { 0, 3, 6, 7, 8, 10, 14, },
  { 1, 3, 6, 7, 8, 10, 14, },
  { 0, 1, 3, 6, 7, 8, 10, 14, },
  { 2, 3, 6, 7, 8, 10, 14, },
  { 0, 2, 3, 6, 7, 8, 10, 14, },
  { 1, 2, 3, 6, 7, 8, 10, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 14, },
  { 4, 6, 7, 8, 10, 14, },
  { 0, 4, 6, 7, 8, 10, 14, },
  { 1, 4, 6, 7, 8, 10, 14, },
  { 0, 1, 4, 6, 7, 8, 10, 14, },
  { 2, 4, 6, 7, 8, 10, 14, },
  { 0, 2, 4, 6, 7, 8, 10, 14, },
  { 1, 2, 4, 6, 7, 8, 10, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 14, },
  { 3, 4, 6, 7, 8, 10, 14, },
  { 0, 3, 4, 6, 7, 8, 10, 14, },
  { 1, 3, 4, 6, 7, 8, 10, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 14, },
  { 2, 3, 4, 6, 7, 8, 10, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 14, },
  { 5, 6, 7, 8, 10, 14, },
  { 0, 5, 6, 7, 8, 10, 14, },
  { 1, 5, 6, 7, 8, 10, 14, },
  { 0, 1, 5, 6, 7, 8, 10, 14, },
  { 2, 5, 6, 7, 8, 10, 14, },
  { 0, 2, 5, 6, 7, 8, 10, 14, },
  { 1, 2, 5, 6, 7, 8, 10, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 14, },
  { 3, 5, 6, 7, 8, 10, 14, },
  { 0, 3, 5, 6, 7, 8, 10, 14, },
  { 1, 3, 5, 6, 7, 8, 10, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 14, },
  { 2, 3, 5, 6, 7, 8, 10, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 14, },
  { 4, 5, 6, 7, 8, 10, 14, },
  { 0, 4, 5, 6, 7, 8, 10, 14, },
  { 1, 4, 5, 6, 7, 8, 10, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 14, },
  { 2, 4, 5, 6, 7, 8, 10, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 14, },
  { 3, 4, 5, 6, 7, 8, 10, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 14, },
  { 9, 10, 14, },
  { 0, 9, 10, 14, },
  { 1, 9, 10, 14, },
  { 0, 1, 9, 10, 14, },
  { 2, 9, 10, 14, },
  { 0, 2, 9, 10, 14, },
  { 1, 2, 9, 10, 14, },
  { 0, 1, 2, 9, 10, 14, },
  { 3, 9, 10, 14, },
  { 0, 3, 9, 10, 14, },
  { 1, 3, 9, 10, 14, },
  { 0, 1, 3, 9, 10, 14, },
  { 2, 3, 9, 10, 14, },
  { 0, 2, 3, 9, 10, 14, },
  { 1, 2, 3, 9, 10, 14, },
  { 0, 1, 2, 3, 9, 10, 14, },
  { 4, 9, 10, 14, },
  { 0, 4, 9, 10, 14, },
  { 1, 4, 9, 10, 14, },
  { 0, 1, 4, 9, 10, 14, },
  { 2, 4, 9, 10, 14, },
  { 0, 2, 4, 9, 10, 14, },
  { 1, 2, 4, 9, 10, 14, },
  { 0, 1, 2, 4, 9, 10, 14, },
  { 3, 4, 9, 10, 14, },
  { 0, 3, 4, 9, 10, 14, },
  { 1, 3, 4, 9, 10, 14, },
  { 0, 1, 3, 4, 9, 10, 14, },
  { 2, 3, 4, 9, 10, 14, },
  { 0, 2, 3, 4, 9, 10, 14, },
  { 1, 2, 3, 4, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 9, 10, 14, },
  { 5, 9, 10, 14, },
  { 0, 5, 9, 10, 14, },
  { 1, 5, 9, 10, 14, },
  { 0, 1, 5, 9, 10, 14, },
  { 2, 5, 9, 10, 14, },
  { 0, 2, 5, 9, 10, 14, },
  { 1, 2, 5, 9, 10, 14, },
  { 0, 1, 2, 5, 9, 10, 14, },
  { 3, 5, 9, 10, 14, },
  { 0, 3, 5, 9, 10, 14, },
  { 1, 3, 5, 9, 10, 14, },
  { 0, 1, 3, 5, 9, 10, 14, },
  { 2, 3, 5, 9, 10, 14, },
  { 0, 2, 3, 5, 9, 10, 14, },
  { 1, 2, 3, 5, 9, 10, 14, },
  { 0, 1, 2, 3, 5, 9, 10, 14, },
  { 4, 5, 9, 10, 14, },
  { 0, 4, 5, 9, 10, 14, },
  { 1, 4, 5, 9, 10, 14, },
  { 0, 1, 4, 5, 9, 10, 14, },
  { 2, 4, 5, 9, 10, 14, },
  { 0, 2, 4, 5, 9, 10, 14, },
  { 1, 2, 4, 5, 9, 10, 14, },
  { 0, 1, 2, 4, 5, 9, 10, 14, },
  { 3, 4, 5, 9, 10, 14, },
  { 0, 3, 4, 5, 9, 10, 14, },
  { 1, 3, 4, 5, 9, 10, 14, },
  { 0, 1, 3, 4, 5, 9, 10, 14, },
  { 2, 3, 4, 5, 9, 10, 14, },
  { 0, 2, 3, 4, 5, 9, 10, 14, },
  { 1, 2, 3, 4, 5, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 14, },
  { 6, 9, 10, 14, },
  { 0, 6, 9, 10, 14, },
  { 1, 6, 9, 10, 14, },
  { 0, 1, 6, 9, 10, 14, },
  { 2, 6, 9, 10, 14, },
  { 0, 2, 6, 9, 10, 14, },
  { 1, 2, 6, 9, 10, 14, },
  { 0, 1, 2, 6, 9, 10, 14, },
  { 3, 6, 9, 10, 14, },
  { 0, 3, 6, 9, 10, 14, },
  { 1, 3, 6, 9, 10, 14, },
  { 0, 1, 3, 6, 9, 10, 14, },
  { 2, 3, 6, 9, 10, 14, },
  { 0, 2, 3, 6, 9, 10, 14, },
  { 1, 2, 3, 6, 9, 10, 14, },
  { 0, 1, 2, 3, 6, 9, 10, 14, },
  { 4, 6, 9, 10, 14, },
  { 0, 4, 6, 9, 10, 14, },
  { 1, 4, 6, 9, 10, 14, },
  { 0, 1, 4, 6, 9, 10, 14, },
  { 2, 4, 6, 9, 10, 14, },
  { 0, 2, 4, 6, 9, 10, 14, },
  { 1, 2, 4, 6, 9, 10, 14, },
  { 0, 1, 2, 4, 6, 9, 10, 14, },
  { 3, 4, 6, 9, 10, 14, },
  { 0, 3, 4, 6, 9, 10, 14, },
  { 1, 3, 4, 6, 9, 10, 14, },
  { 0, 1, 3, 4, 6, 9, 10, 14, },
  { 2, 3, 4, 6, 9, 10, 14, },
  { 0, 2, 3, 4, 6, 9, 10, 14, },
  { 1, 2, 3, 4, 6, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 14, },
  { 5, 6, 9, 10, 14, },
  { 0, 5, 6, 9, 10, 14, },
  { 1, 5, 6, 9, 10, 14, },
  { 0, 1, 5, 6, 9, 10, 14, },
  { 2, 5, 6, 9, 10, 14, },
  { 0, 2, 5, 6, 9, 10, 14, },
  { 1, 2, 5, 6, 9, 10, 14, },
  { 0, 1, 2, 5, 6, 9, 10, 14, },
  { 3, 5, 6, 9, 10, 14, },
  { 0, 3, 5, 6, 9, 10, 14, },
  { 1, 3, 5, 6, 9, 10, 14, },
  { 0, 1, 3, 5, 6, 9, 10, 14, },
  { 2, 3, 5, 6, 9, 10, 14, },
  { 0, 2, 3, 5, 6, 9, 10, 14, },
  { 1, 2, 3, 5, 6, 9, 10, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 14, },
  { 4, 5, 6, 9, 10, 14, },
  { 0, 4, 5, 6, 9, 10, 14, },
  { 1, 4, 5, 6, 9, 10, 14, },
  { 0, 1, 4, 5, 6, 9, 10, 14, },
  { 2, 4, 5, 6, 9, 10, 14, },
  { 0, 2, 4, 5, 6, 9, 10, 14, },
  { 1, 2, 4, 5, 6, 9, 10, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 14, },
  { 3, 4, 5, 6, 9, 10, 14, },
  { 0, 3, 4, 5, 6, 9, 10, 14, },
  { 1, 3, 4, 5, 6, 9, 10, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 14, },
  { 2, 3, 4, 5, 6, 9, 10, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 14, },
  { 7, 9, 10, 14, },
  { 0, 7, 9, 10, 14, },
  { 1, 7, 9, 10, 14, },
  { 0, 1, 7, 9, 10, 14, },
  { 2, 7, 9, 10, 14, },
  { 0, 2, 7, 9, 10, 14, },
  { 1, 2, 7, 9, 10, 14, },
  { 0, 1, 2, 7, 9, 10, 14, },
  { 3, 7, 9, 10, 14, },
  { 0, 3, 7, 9, 10, 14, },
  { 1, 3, 7, 9, 10, 14, },
  { 0, 1, 3, 7, 9, 10, 14, },
  { 2, 3, 7, 9, 10, 14, },
  { 0, 2, 3, 7, 9, 10, 14, },
  { 1, 2, 3, 7, 9, 10, 14, },
  { 0, 1, 2, 3, 7, 9, 10, 14, },
  { 4, 7, 9, 10, 14, },
  { 0, 4, 7, 9, 10, 14, },
  { 1, 4, 7, 9, 10, 14, },
  { 0, 1, 4, 7, 9, 10, 14, },
  { 2, 4, 7, 9, 10, 14, },
  { 0, 2, 4, 7, 9, 10, 14, },
  { 1, 2, 4, 7, 9, 10, 14, },
  { 0, 1, 2, 4, 7, 9, 10, 14, },
  { 3, 4, 7, 9, 10, 14, },
  { 0, 3, 4, 7, 9, 10, 14, },
  { 1, 3, 4, 7, 9, 10, 14, },
  { 0, 1, 3, 4, 7, 9, 10, 14, },
  { 2, 3, 4, 7, 9, 10, 14, },
  { 0, 2, 3, 4, 7, 9, 10, 14, },
  { 1, 2, 3, 4, 7, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 14, },
  { 5, 7, 9, 10, 14, },
  { 0, 5, 7, 9, 10, 14, },
  { 1, 5, 7, 9, 10, 14, },
  { 0, 1, 5, 7, 9, 10, 14, },
  { 2, 5, 7, 9, 10, 14, },
  { 0, 2, 5, 7, 9, 10, 14, },
  { 1, 2, 5, 7, 9, 10, 14, },
  { 0, 1, 2, 5, 7, 9, 10, 14, },
  { 3, 5, 7, 9, 10, 14, },
  { 0, 3, 5, 7, 9, 10, 14, },
  { 1, 3, 5, 7, 9, 10, 14, },
  { 0, 1, 3, 5, 7, 9, 10, 14, },
  { 2, 3, 5, 7, 9, 10, 14, },
  { 0, 2, 3, 5, 7, 9, 10, 14, },
  { 1, 2, 3, 5, 7, 9, 10, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 14, },
  { 4, 5, 7, 9, 10, 14, },
  { 0, 4, 5, 7, 9, 10, 14, },
  { 1, 4, 5, 7, 9, 10, 14, },
  { 0, 1, 4, 5, 7, 9, 10, 14, },
  { 2, 4, 5, 7, 9, 10, 14, },
  { 0, 2, 4, 5, 7, 9, 10, 14, },
  { 1, 2, 4, 5, 7, 9, 10, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 14, },
  { 3, 4, 5, 7, 9, 10, 14, },
  { 0, 3, 4, 5, 7, 9, 10, 14, },
  { 1, 3, 4, 5, 7, 9, 10, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 14, },
  { 2, 3, 4, 5, 7, 9, 10, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 14, },
  { 6, 7, 9, 10, 14, },
  { 0, 6, 7, 9, 10, 14, },
  { 1, 6, 7, 9, 10, 14, },
  { 0, 1, 6, 7, 9, 10, 14, },
  { 2, 6, 7, 9, 10, 14, },
  { 0, 2, 6, 7, 9, 10, 14, },
  { 1, 2, 6, 7, 9, 10, 14, },
  { 0, 1, 2, 6, 7, 9, 10, 14, },
  { 3, 6, 7, 9, 10, 14, },
  { 0, 3, 6, 7, 9, 10, 14, },
  { 1, 3, 6, 7, 9, 10, 14, },
  { 0, 1, 3, 6, 7, 9, 10, 14, },
  { 2, 3, 6, 7, 9, 10, 14, },
  { 0, 2, 3, 6, 7, 9, 10, 14, },
  { 1, 2, 3, 6, 7, 9, 10, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 14, },
  { 4, 6, 7, 9, 10, 14, },
  { 0, 4, 6, 7, 9, 10, 14, },
  { 1, 4, 6, 7, 9, 10, 14, },
  { 0, 1, 4, 6, 7, 9, 10, 14, },
  { 2, 4, 6, 7, 9, 10, 14, },
  { 0, 2, 4, 6, 7, 9, 10, 14, },
  { 1, 2, 4, 6, 7, 9, 10, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 14, },
  { 3, 4, 6, 7, 9, 10, 14, },
  { 0, 3, 4, 6, 7, 9, 10, 14, },
  { 1, 3, 4, 6, 7, 9, 10, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 14, },
  { 2, 3, 4, 6, 7, 9, 10, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 14, },
  { 5, 6, 7, 9, 10, 14, },
  { 0, 5, 6, 7, 9, 10, 14, },
  { 1, 5, 6, 7, 9, 10, 14, },
  { 0, 1, 5, 6, 7, 9, 10, 14, },
  { 2, 5, 6, 7, 9, 10, 14, },
  { 0, 2, 5, 6, 7, 9, 10, 14, },
  { 1, 2, 5, 6, 7, 9, 10, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 14, },
  { 3, 5, 6, 7, 9, 10, 14, },
  { 0, 3, 5, 6, 7, 9, 10, 14, },
  { 1, 3, 5, 6, 7, 9, 10, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 14, },
  { 2, 3, 5, 6, 7, 9, 10, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 14, },
  { 4, 5, 6, 7, 9, 10, 14, },
  { 0, 4, 5, 6, 7, 9, 10, 14, },
  { 1, 4, 5, 6, 7, 9, 10, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 14, },
  { 2, 4, 5, 6, 7, 9, 10, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 14, },
  { 3, 4, 5, 6, 7, 9, 10, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 14, },
  { 8, 9, 10, 14, },
  { 0, 8, 9, 10, 14, },
  { 1, 8, 9, 10, 14, },
  { 0, 1, 8, 9, 10, 14, },
  { 2, 8, 9, 10, 14, },
  { 0, 2, 8, 9, 10, 14, },
  { 1, 2, 8, 9, 10, 14, },
  { 0, 1, 2, 8, 9, 10, 14, },
  { 3, 8, 9, 10, 14, },
  { 0, 3, 8, 9, 10, 14, },
  { 1, 3, 8, 9, 10, 14, },
  { 0, 1, 3, 8, 9, 10, 14, },
  { 2, 3, 8, 9, 10, 14, },
  { 0, 2, 3, 8, 9, 10, 14, },
  { 1, 2, 3, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 8, 9, 10, 14, },
  { 4, 8, 9, 10, 14, },
  { 0, 4, 8, 9, 10, 14, },
  { 1, 4, 8, 9, 10, 14, },
  { 0, 1, 4, 8, 9, 10, 14, },
  { 2, 4, 8, 9, 10, 14, },
  { 0, 2, 4, 8, 9, 10, 14, },
  { 1, 2, 4, 8, 9, 10, 14, },
  { 0, 1, 2, 4, 8, 9, 10, 14, },
  { 3, 4, 8, 9, 10, 14, },
  { 0, 3, 4, 8, 9, 10, 14, },
  { 1, 3, 4, 8, 9, 10, 14, },
  { 0, 1, 3, 4, 8, 9, 10, 14, },
  { 2, 3, 4, 8, 9, 10, 14, },
  { 0, 2, 3, 4, 8, 9, 10, 14, },
  { 1, 2, 3, 4, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 14, },
  { 5, 8, 9, 10, 14, },
  { 0, 5, 8, 9, 10, 14, },
  { 1, 5, 8, 9, 10, 14, },
  { 0, 1, 5, 8, 9, 10, 14, },
  { 2, 5, 8, 9, 10, 14, },
  { 0, 2, 5, 8, 9, 10, 14, },
  { 1, 2, 5, 8, 9, 10, 14, },
  { 0, 1, 2, 5, 8, 9, 10, 14, },
  { 3, 5, 8, 9, 10, 14, },
  { 0, 3, 5, 8, 9, 10, 14, },
  { 1, 3, 5, 8, 9, 10, 14, },
  { 0, 1, 3, 5, 8, 9, 10, 14, },
  { 2, 3, 5, 8, 9, 10, 14, },
  { 0, 2, 3, 5, 8, 9, 10, 14, },
  { 1, 2, 3, 5, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 14, },
  { 4, 5, 8, 9, 10, 14, },
  { 0, 4, 5, 8, 9, 10, 14, },
  { 1, 4, 5, 8, 9, 10, 14, },
  { 0, 1, 4, 5, 8, 9, 10, 14, },
  { 2, 4, 5, 8, 9, 10, 14, },
  { 0, 2, 4, 5, 8, 9, 10, 14, },
  { 1, 2, 4, 5, 8, 9, 10, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 14, },
  { 3, 4, 5, 8, 9, 10, 14, },
  { 0, 3, 4, 5, 8, 9, 10, 14, },
  { 1, 3, 4, 5, 8, 9, 10, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 14, },
  { 2, 3, 4, 5, 8, 9, 10, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 14, },
  { 6, 8, 9, 10, 14, },
  { 0, 6, 8, 9, 10, 14, },
  { 1, 6, 8, 9, 10, 14, },
  { 0, 1, 6, 8, 9, 10, 14, },
  { 2, 6, 8, 9, 10, 14, },
  { 0, 2, 6, 8, 9, 10, 14, },
  { 1, 2, 6, 8, 9, 10, 14, },
  { 0, 1, 2, 6, 8, 9, 10, 14, },
  { 3, 6, 8, 9, 10, 14, },
  { 0, 3, 6, 8, 9, 10, 14, },
  { 1, 3, 6, 8, 9, 10, 14, },
  { 0, 1, 3, 6, 8, 9, 10, 14, },
  { 2, 3, 6, 8, 9, 10, 14, },
  { 0, 2, 3, 6, 8, 9, 10, 14, },
  { 1, 2, 3, 6, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 14, },
  { 4, 6, 8, 9, 10, 14, },
  { 0, 4, 6, 8, 9, 10, 14, },
  { 1, 4, 6, 8, 9, 10, 14, },
  { 0, 1, 4, 6, 8, 9, 10, 14, },
  { 2, 4, 6, 8, 9, 10, 14, },
  { 0, 2, 4, 6, 8, 9, 10, 14, },
  { 1, 2, 4, 6, 8, 9, 10, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 14, },
  { 3, 4, 6, 8, 9, 10, 14, },
  { 0, 3, 4, 6, 8, 9, 10, 14, },
  { 1, 3, 4, 6, 8, 9, 10, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 14, },
  { 2, 3, 4, 6, 8, 9, 10, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 14, },
  { 5, 6, 8, 9, 10, 14, },
  { 0, 5, 6, 8, 9, 10, 14, },
  { 1, 5, 6, 8, 9, 10, 14, },
  { 0, 1, 5, 6, 8, 9, 10, 14, },
  { 2, 5, 6, 8, 9, 10, 14, },
  { 0, 2, 5, 6, 8, 9, 10, 14, },
  { 1, 2, 5, 6, 8, 9, 10, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 14, },
  { 3, 5, 6, 8, 9, 10, 14, },
  { 0, 3, 5, 6, 8, 9, 10, 14, },
  { 1, 3, 5, 6, 8, 9, 10, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 14, },
  { 2, 3, 5, 6, 8, 9, 10, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 14, },
  { 4, 5, 6, 8, 9, 10, 14, },
  { 0, 4, 5, 6, 8, 9, 10, 14, },
  { 1, 4, 5, 6, 8, 9, 10, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 14, },
  { 2, 4, 5, 6, 8, 9, 10, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 14, },
  { 3, 4, 5, 6, 8, 9, 10, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 14, },
  { 7, 8, 9, 10, 14, },
  { 0, 7, 8, 9, 10, 14, },
  { 1, 7, 8, 9, 10, 14, },
  { 0, 1, 7, 8, 9, 10, 14, },
  { 2, 7, 8, 9, 10, 14, },
  { 0, 2, 7, 8, 9, 10, 14, },
  { 1, 2, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 7, 8, 9, 10, 14, },
  { 3, 7, 8, 9, 10, 14, },
  { 0, 3, 7, 8, 9, 10, 14, },
  { 1, 3, 7, 8, 9, 10, 14, },
  { 0, 1, 3, 7, 8, 9, 10, 14, },
  { 2, 3, 7, 8, 9, 10, 14, },
  { 0, 2, 3, 7, 8, 9, 10, 14, },
  { 1, 2, 3, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 14, },
  { 4, 7, 8, 9, 10, 14, },
  { 0, 4, 7, 8, 9, 10, 14, },
  { 1, 4, 7, 8, 9, 10, 14, },
  { 0, 1, 4, 7, 8, 9, 10, 14, },
  { 2, 4, 7, 8, 9, 10, 14, },
  { 0, 2, 4, 7, 8, 9, 10, 14, },
  { 1, 2, 4, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 14, },
  { 3, 4, 7, 8, 9, 10, 14, },
  { 0, 3, 4, 7, 8, 9, 10, 14, },
  { 1, 3, 4, 7, 8, 9, 10, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 14, },
  { 2, 3, 4, 7, 8, 9, 10, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 14, },
  { 5, 7, 8, 9, 10, 14, },
  { 0, 5, 7, 8, 9, 10, 14, },
  { 1, 5, 7, 8, 9, 10, 14, },
  { 0, 1, 5, 7, 8, 9, 10, 14, },
  { 2, 5, 7, 8, 9, 10, 14, },
  { 0, 2, 5, 7, 8, 9, 10, 14, },
  { 1, 2, 5, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 14, },
  { 3, 5, 7, 8, 9, 10, 14, },
  { 0, 3, 5, 7, 8, 9, 10, 14, },
  { 1, 3, 5, 7, 8, 9, 10, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 14, },
  { 2, 3, 5, 7, 8, 9, 10, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 14, },
  { 4, 5, 7, 8, 9, 10, 14, },
  { 0, 4, 5, 7, 8, 9, 10, 14, },
  { 1, 4, 5, 7, 8, 9, 10, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 14, },
  { 2, 4, 5, 7, 8, 9, 10, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 14, },
  { 3, 4, 5, 7, 8, 9, 10, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 14, },
  { 6, 7, 8, 9, 10, 14, },
  { 0, 6, 7, 8, 9, 10, 14, },
  { 1, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 6, 7, 8, 9, 10, 14, },
  { 2, 6, 7, 8, 9, 10, 14, },
  { 0, 2, 6, 7, 8, 9, 10, 14, },
  { 1, 2, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 14, },
  { 3, 6, 7, 8, 9, 10, 14, },
  { 0, 3, 6, 7, 8, 9, 10, 14, },
  { 1, 3, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 14, },
  { 2, 3, 6, 7, 8, 9, 10, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 14, },
  { 4, 6, 7, 8, 9, 10, 14, },
  { 0, 4, 6, 7, 8, 9, 10, 14, },
  { 1, 4, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 14, },
  { 2, 4, 6, 7, 8, 9, 10, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 14, },
  { 3, 4, 6, 7, 8, 9, 10, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 14, },
  { 5, 6, 7, 8, 9, 10, 14, },
  { 0, 5, 6, 7, 8, 9, 10, 14, },
  { 1, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 14, },
  { 2, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 14, },
  { 3, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 14, },
  { 4, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, },
  { 11, 14, },
  { 0, 11, 14, },
  { 1, 11, 14, },
  { 0, 1, 11, 14, },
  { 2, 11, 14, },
  { 0, 2, 11, 14, },
  { 1, 2, 11, 14, },
  { 0, 1, 2, 11, 14, },
  { 3, 11, 14, },
  { 0, 3, 11, 14, },
  { 1, 3, 11, 14, },
  { 0, 1, 3, 11, 14, },
  { 2, 3, 11, 14, },
  { 0, 2, 3, 11, 14, },
  { 1, 2, 3, 11, 14, },
  { 0, 1, 2, 3, 11, 14, },
  { 4, 11, 14, },
  { 0, 4, 11, 14, },
  { 1, 4, 11, 14, },
  { 0, 1, 4, 11, 14, },
  { 2, 4, 11, 14, },
  { 0, 2, 4, 11, 14, },
  { 1, 2, 4, 11, 14, },
  { 0, 1, 2, 4, 11, 14, },
  { 3, 4, 11, 14, },
  { 0, 3, 4, 11, 14, },
  { 1, 3, 4, 11, 14, },
  { 0, 1, 3, 4, 11, 14, },
  { 2, 3, 4, 11, 14, },
  { 0, 2, 3, 4, 11, 14, },
  { 1, 2, 3, 4, 11, 14, },
  { 0, 1, 2, 3, 4, 11, 14, },
  { 5, 11, 14, },
  { 0, 5, 11, 14, },
  { 1, 5, 11, 14, },
  { 0, 1, 5, 11, 14, },
  { 2, 5, 11, 14, },
  { 0, 2, 5, 11, 14, },
  { 1, 2, 5, 11, 14, },
  { 0, 1, 2, 5, 11, 14, },
  { 3, 5, 11, 14, },
  { 0, 3, 5, 11, 14, },
  { 1, 3, 5, 11, 14, },
  { 0, 1, 3, 5, 11, 14, },
  { 2, 3, 5, 11, 14, },
  { 0, 2, 3, 5, 11, 14, },
  { 1, 2, 3, 5, 11, 14, },
  { 0, 1, 2, 3, 5, 11, 14, },
  { 4, 5, 11, 14, },
  { 0, 4, 5, 11, 14, },
  { 1, 4, 5, 11, 14, },
  { 0, 1, 4, 5, 11, 14, },
  { 2, 4, 5, 11, 14, },
  { 0, 2, 4, 5, 11, 14, },
  { 1, 2, 4, 5, 11, 14, },
  { 0, 1, 2, 4, 5, 11, 14, },
  { 3, 4, 5, 11, 14, },
  { 0, 3, 4, 5, 11, 14, },
  { 1, 3, 4, 5, 11, 14, },
  { 0, 1, 3, 4, 5, 11, 14, },
  { 2, 3, 4, 5, 11, 14, },
  { 0, 2, 3, 4, 5, 11, 14, },
  { 1, 2, 3, 4, 5, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 11, 14, },
  { 6, 11, 14, },
  { 0, 6, 11, 14, },
  { 1, 6, 11, 14, },
  { 0, 1, 6, 11, 14, },
  { 2, 6, 11, 14, },
  { 0, 2, 6, 11, 14, },
  { 1, 2, 6, 11, 14, },
  { 0, 1, 2, 6, 11, 14, },
  { 3, 6, 11, 14, },
  { 0, 3, 6, 11, 14, },
  { 1, 3, 6, 11, 14, },
  { 0, 1, 3, 6, 11, 14, },
  { 2, 3, 6, 11, 14, },
  { 0, 2, 3, 6, 11, 14, },
  { 1, 2, 3, 6, 11, 14, },
  { 0, 1, 2, 3, 6, 11, 14, },
  { 4, 6, 11, 14, },
  { 0, 4, 6, 11, 14, },
  { 1, 4, 6, 11, 14, },
  { 0, 1, 4, 6, 11, 14, },
  { 2, 4, 6, 11, 14, },
  { 0, 2, 4, 6, 11, 14, },
  { 1, 2, 4, 6, 11, 14, },
  { 0, 1, 2, 4, 6, 11, 14, },
  { 3, 4, 6, 11, 14, },
  { 0, 3, 4, 6, 11, 14, },
  { 1, 3, 4, 6, 11, 14, },
  { 0, 1, 3, 4, 6, 11, 14, },
  { 2, 3, 4, 6, 11, 14, },
  { 0, 2, 3, 4, 6, 11, 14, },
  { 1, 2, 3, 4, 6, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 11, 14, },
  { 5, 6, 11, 14, },
  { 0, 5, 6, 11, 14, },
  { 1, 5, 6, 11, 14, },
  { 0, 1, 5, 6, 11, 14, },
  { 2, 5, 6, 11, 14, },
  { 0, 2, 5, 6, 11, 14, },
  { 1, 2, 5, 6, 11, 14, },
  { 0, 1, 2, 5, 6, 11, 14, },
  { 3, 5, 6, 11, 14, },
  { 0, 3, 5, 6, 11, 14, },
  { 1, 3, 5, 6, 11, 14, },
  { 0, 1, 3, 5, 6, 11, 14, },
  { 2, 3, 5, 6, 11, 14, },
  { 0, 2, 3, 5, 6, 11, 14, },
  { 1, 2, 3, 5, 6, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 11, 14, },
  { 4, 5, 6, 11, 14, },
  { 0, 4, 5, 6, 11, 14, },
  { 1, 4, 5, 6, 11, 14, },
  { 0, 1, 4, 5, 6, 11, 14, },
  { 2, 4, 5, 6, 11, 14, },
  { 0, 2, 4, 5, 6, 11, 14, },
  { 1, 2, 4, 5, 6, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 11, 14, },
  { 3, 4, 5, 6, 11, 14, },
  { 0, 3, 4, 5, 6, 11, 14, },
  { 1, 3, 4, 5, 6, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 11, 14, },
  { 2, 3, 4, 5, 6, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 14, },
  { 7, 11, 14, },
  { 0, 7, 11, 14, },
  { 1, 7, 11, 14, },
  { 0, 1, 7, 11, 14, },
  { 2, 7, 11, 14, },
  { 0, 2, 7, 11, 14, },
  { 1, 2, 7, 11, 14, },
  { 0, 1, 2, 7, 11, 14, },
  { 3, 7, 11, 14, },
  { 0, 3, 7, 11, 14, },
  { 1, 3, 7, 11, 14, },
  { 0, 1, 3, 7, 11, 14, },
  { 2, 3, 7, 11, 14, },
  { 0, 2, 3, 7, 11, 14, },
  { 1, 2, 3, 7, 11, 14, },
  { 0, 1, 2, 3, 7, 11, 14, },
  { 4, 7, 11, 14, },
  { 0, 4, 7, 11, 14, },
  { 1, 4, 7, 11, 14, },
  { 0, 1, 4, 7, 11, 14, },
  { 2, 4, 7, 11, 14, },
  { 0, 2, 4, 7, 11, 14, },
  { 1, 2, 4, 7, 11, 14, },
  { 0, 1, 2, 4, 7, 11, 14, },
  { 3, 4, 7, 11, 14, },
  { 0, 3, 4, 7, 11, 14, },
  { 1, 3, 4, 7, 11, 14, },
  { 0, 1, 3, 4, 7, 11, 14, },
  { 2, 3, 4, 7, 11, 14, },
  { 0, 2, 3, 4, 7, 11, 14, },
  { 1, 2, 3, 4, 7, 11, 14, },
  { 0, 1, 2, 3, 4, 7, 11, 14, },
  { 5, 7, 11, 14, },
  { 0, 5, 7, 11, 14, },
  { 1, 5, 7, 11, 14, },
  { 0, 1, 5, 7, 11, 14, },
  { 2, 5, 7, 11, 14, },
  { 0, 2, 5, 7, 11, 14, },
  { 1, 2, 5, 7, 11, 14, },
  { 0, 1, 2, 5, 7, 11, 14, },
  { 3, 5, 7, 11, 14, },
  { 0, 3, 5, 7, 11, 14, },
  { 1, 3, 5, 7, 11, 14, },
  { 0, 1, 3, 5, 7, 11, 14, },
  { 2, 3, 5, 7, 11, 14, },
  { 0, 2, 3, 5, 7, 11, 14, },
  { 1, 2, 3, 5, 7, 11, 14, },
  { 0, 1, 2, 3, 5, 7, 11, 14, },
  { 4, 5, 7, 11, 14, },
  { 0, 4, 5, 7, 11, 14, },
  { 1, 4, 5, 7, 11, 14, },
  { 0, 1, 4, 5, 7, 11, 14, },
  { 2, 4, 5, 7, 11, 14, },
  { 0, 2, 4, 5, 7, 11, 14, },
  { 1, 2, 4, 5, 7, 11, 14, },
  { 0, 1, 2, 4, 5, 7, 11, 14, },
  { 3, 4, 5, 7, 11, 14, },
  { 0, 3, 4, 5, 7, 11, 14, },
  { 1, 3, 4, 5, 7, 11, 14, },
  { 0, 1, 3, 4, 5, 7, 11, 14, },
  { 2, 3, 4, 5, 7, 11, 14, },
  { 0, 2, 3, 4, 5, 7, 11, 14, },
  { 1, 2, 3, 4, 5, 7, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 14, },
  { 6, 7, 11, 14, },
  { 0, 6, 7, 11, 14, },
  { 1, 6, 7, 11, 14, },
  { 0, 1, 6, 7, 11, 14, },
  { 2, 6, 7, 11, 14, },
  { 0, 2, 6, 7, 11, 14, },
  { 1, 2, 6, 7, 11, 14, },
  { 0, 1, 2, 6, 7, 11, 14, },
  { 3, 6, 7, 11, 14, },
  { 0, 3, 6, 7, 11, 14, },
  { 1, 3, 6, 7, 11, 14, },
  { 0, 1, 3, 6, 7, 11, 14, },
  { 2, 3, 6, 7, 11, 14, },
  { 0, 2, 3, 6, 7, 11, 14, },
  { 1, 2, 3, 6, 7, 11, 14, },
  { 0, 1, 2, 3, 6, 7, 11, 14, },
  { 4, 6, 7, 11, 14, },
  { 0, 4, 6, 7, 11, 14, },
  { 1, 4, 6, 7, 11, 14, },
  { 0, 1, 4, 6, 7, 11, 14, },
  { 2, 4, 6, 7, 11, 14, },
  { 0, 2, 4, 6, 7, 11, 14, },
  { 1, 2, 4, 6, 7, 11, 14, },
  { 0, 1, 2, 4, 6, 7, 11, 14, },
  { 3, 4, 6, 7, 11, 14, },
  { 0, 3, 4, 6, 7, 11, 14, },
  { 1, 3, 4, 6, 7, 11, 14, },
  { 0, 1, 3, 4, 6, 7, 11, 14, },
  { 2, 3, 4, 6, 7, 11, 14, },
  { 0, 2, 3, 4, 6, 7, 11, 14, },
  { 1, 2, 3, 4, 6, 7, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 14, },
  { 5, 6, 7, 11, 14, },
  { 0, 5, 6, 7, 11, 14, },
  { 1, 5, 6, 7, 11, 14, },
  { 0, 1, 5, 6, 7, 11, 14, },
  { 2, 5, 6, 7, 11, 14, },
  { 0, 2, 5, 6, 7, 11, 14, },
  { 1, 2, 5, 6, 7, 11, 14, },
  { 0, 1, 2, 5, 6, 7, 11, 14, },
  { 3, 5, 6, 7, 11, 14, },
  { 0, 3, 5, 6, 7, 11, 14, },
  { 1, 3, 5, 6, 7, 11, 14, },
  { 0, 1, 3, 5, 6, 7, 11, 14, },
  { 2, 3, 5, 6, 7, 11, 14, },
  { 0, 2, 3, 5, 6, 7, 11, 14, },
  { 1, 2, 3, 5, 6, 7, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 14, },
  { 4, 5, 6, 7, 11, 14, },
  { 0, 4, 5, 6, 7, 11, 14, },
  { 1, 4, 5, 6, 7, 11, 14, },
  { 0, 1, 4, 5, 6, 7, 11, 14, },
  { 2, 4, 5, 6, 7, 11, 14, },
  { 0, 2, 4, 5, 6, 7, 11, 14, },
  { 1, 2, 4, 5, 6, 7, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 14, },
  { 3, 4, 5, 6, 7, 11, 14, },
  { 0, 3, 4, 5, 6, 7, 11, 14, },
  { 1, 3, 4, 5, 6, 7, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 14, },
  { 2, 3, 4, 5, 6, 7, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 14, },
  { 8, 11, 14, },
  { 0, 8, 11, 14, },
  { 1, 8, 11, 14, },
  { 0, 1, 8, 11, 14, },
  { 2, 8, 11, 14, },
  { 0, 2, 8, 11, 14, },
  { 1, 2, 8, 11, 14, },
  { 0, 1, 2, 8, 11, 14, },
  { 3, 8, 11, 14, },
  { 0, 3, 8, 11, 14, },
  { 1, 3, 8, 11, 14, },
  { 0, 1, 3, 8, 11, 14, },
  { 2, 3, 8, 11, 14, },
  { 0, 2, 3, 8, 11, 14, },
  { 1, 2, 3, 8, 11, 14, },
  { 0, 1, 2, 3, 8, 11, 14, },
  { 4, 8, 11, 14, },
  { 0, 4, 8, 11, 14, },
  { 1, 4, 8, 11, 14, },
  { 0, 1, 4, 8, 11, 14, },
  { 2, 4, 8, 11, 14, },
  { 0, 2, 4, 8, 11, 14, },
  { 1, 2, 4, 8, 11, 14, },
  { 0, 1, 2, 4, 8, 11, 14, },
  { 3, 4, 8, 11, 14, },
  { 0, 3, 4, 8, 11, 14, },
  { 1, 3, 4, 8, 11, 14, },
  { 0, 1, 3, 4, 8, 11, 14, },
  { 2, 3, 4, 8, 11, 14, },
  { 0, 2, 3, 4, 8, 11, 14, },
  { 1, 2, 3, 4, 8, 11, 14, },
  { 0, 1, 2, 3, 4, 8, 11, 14, },
  { 5, 8, 11, 14, },
  { 0, 5, 8, 11, 14, },
  { 1, 5, 8, 11, 14, },
  { 0, 1, 5, 8, 11, 14, },
  { 2, 5, 8, 11, 14, },
  { 0, 2, 5, 8, 11, 14, },
  { 1, 2, 5, 8, 11, 14, },
  { 0, 1, 2, 5, 8, 11, 14, },
  { 3, 5, 8, 11, 14, },
  { 0, 3, 5, 8, 11, 14, },
  { 1, 3, 5, 8, 11, 14, },
  { 0, 1, 3, 5, 8, 11, 14, },
  { 2, 3, 5, 8, 11, 14, },
  { 0, 2, 3, 5, 8, 11, 14, },
  { 1, 2, 3, 5, 8, 11, 14, },
  { 0, 1, 2, 3, 5, 8, 11, 14, },
  { 4, 5, 8, 11, 14, },
  { 0, 4, 5, 8, 11, 14, },
  { 1, 4, 5, 8, 11, 14, },
  { 0, 1, 4, 5, 8, 11, 14, },
  { 2, 4, 5, 8, 11, 14, },
  { 0, 2, 4, 5, 8, 11, 14, },
  { 1, 2, 4, 5, 8, 11, 14, },
  { 0, 1, 2, 4, 5, 8, 11, 14, },
  { 3, 4, 5, 8, 11, 14, },
  { 0, 3, 4, 5, 8, 11, 14, },
  { 1, 3, 4, 5, 8, 11, 14, },
  { 0, 1, 3, 4, 5, 8, 11, 14, },
  { 2, 3, 4, 5, 8, 11, 14, },
  { 0, 2, 3, 4, 5, 8, 11, 14, },
  { 1, 2, 3, 4, 5, 8, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 14, },
  { 6, 8, 11, 14, },
  { 0, 6, 8, 11, 14, },
  { 1, 6, 8, 11, 14, },
  { 0, 1, 6, 8, 11, 14, },
  { 2, 6, 8, 11, 14, },
  { 0, 2, 6, 8, 11, 14, },
  { 1, 2, 6, 8, 11, 14, },
  { 0, 1, 2, 6, 8, 11, 14, },
  { 3, 6, 8, 11, 14, },
  { 0, 3, 6, 8, 11, 14, },
  { 1, 3, 6, 8, 11, 14, },
  { 0, 1, 3, 6, 8, 11, 14, },
  { 2, 3, 6, 8, 11, 14, },
  { 0, 2, 3, 6, 8, 11, 14, },
  { 1, 2, 3, 6, 8, 11, 14, },
  { 0, 1, 2, 3, 6, 8, 11, 14, },
  { 4, 6, 8, 11, 14, },
  { 0, 4, 6, 8, 11, 14, },
  { 1, 4, 6, 8, 11, 14, },
  { 0, 1, 4, 6, 8, 11, 14, },
  { 2, 4, 6, 8, 11, 14, },
  { 0, 2, 4, 6, 8, 11, 14, },
  { 1, 2, 4, 6, 8, 11, 14, },
  { 0, 1, 2, 4, 6, 8, 11, 14, },
  { 3, 4, 6, 8, 11, 14, },
  { 0, 3, 4, 6, 8, 11, 14, },
  { 1, 3, 4, 6, 8, 11, 14, },
  { 0, 1, 3, 4, 6, 8, 11, 14, },
  { 2, 3, 4, 6, 8, 11, 14, },
  { 0, 2, 3, 4, 6, 8, 11, 14, },
  { 1, 2, 3, 4, 6, 8, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 14, },
  { 5, 6, 8, 11, 14, },
  { 0, 5, 6, 8, 11, 14, },
  { 1, 5, 6, 8, 11, 14, },
  { 0, 1, 5, 6, 8, 11, 14, },
  { 2, 5, 6, 8, 11, 14, },
  { 0, 2, 5, 6, 8, 11, 14, },
  { 1, 2, 5, 6, 8, 11, 14, },
  { 0, 1, 2, 5, 6, 8, 11, 14, },
  { 3, 5, 6, 8, 11, 14, },
  { 0, 3, 5, 6, 8, 11, 14, },
  { 1, 3, 5, 6, 8, 11, 14, },
  { 0, 1, 3, 5, 6, 8, 11, 14, },
  { 2, 3, 5, 6, 8, 11, 14, },
  { 0, 2, 3, 5, 6, 8, 11, 14, },
  { 1, 2, 3, 5, 6, 8, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 14, },
  { 4, 5, 6, 8, 11, 14, },
  { 0, 4, 5, 6, 8, 11, 14, },
  { 1, 4, 5, 6, 8, 11, 14, },
  { 0, 1, 4, 5, 6, 8, 11, 14, },
  { 2, 4, 5, 6, 8, 11, 14, },
  { 0, 2, 4, 5, 6, 8, 11, 14, },
  { 1, 2, 4, 5, 6, 8, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 14, },
  { 3, 4, 5, 6, 8, 11, 14, },
  { 0, 3, 4, 5, 6, 8, 11, 14, },
  { 1, 3, 4, 5, 6, 8, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 14, },
  { 2, 3, 4, 5, 6, 8, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 14, },
  { 7, 8, 11, 14, },
  { 0, 7, 8, 11, 14, },
  { 1, 7, 8, 11, 14, },
  { 0, 1, 7, 8, 11, 14, },
  { 2, 7, 8, 11, 14, },
  { 0, 2, 7, 8, 11, 14, },
  { 1, 2, 7, 8, 11, 14, },
  { 0, 1, 2, 7, 8, 11, 14, },
  { 3, 7, 8, 11, 14, },
  { 0, 3, 7, 8, 11, 14, },
  { 1, 3, 7, 8, 11, 14, },
  { 0, 1, 3, 7, 8, 11, 14, },
  { 2, 3, 7, 8, 11, 14, },
  { 0, 2, 3, 7, 8, 11, 14, },
  { 1, 2, 3, 7, 8, 11, 14, },
  { 0, 1, 2, 3, 7, 8, 11, 14, },
  { 4, 7, 8, 11, 14, },
  { 0, 4, 7, 8, 11, 14, },
  { 1, 4, 7, 8, 11, 14, },
  { 0, 1, 4, 7, 8, 11, 14, },
  { 2, 4, 7, 8, 11, 14, },
  { 0, 2, 4, 7, 8, 11, 14, },
  { 1, 2, 4, 7, 8, 11, 14, },
  { 0, 1, 2, 4, 7, 8, 11, 14, },
  { 3, 4, 7, 8, 11, 14, },
  { 0, 3, 4, 7, 8, 11, 14, },
  { 1, 3, 4, 7, 8, 11, 14, },
  { 0, 1, 3, 4, 7, 8, 11, 14, },
  { 2, 3, 4, 7, 8, 11, 14, },
  { 0, 2, 3, 4, 7, 8, 11, 14, },
  { 1, 2, 3, 4, 7, 8, 11, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 14, },
  { 5, 7, 8, 11, 14, },
  { 0, 5, 7, 8, 11, 14, },
  { 1, 5, 7, 8, 11, 14, },
  { 0, 1, 5, 7, 8, 11, 14, },
  { 2, 5, 7, 8, 11, 14, },
  { 0, 2, 5, 7, 8, 11, 14, },
  { 1, 2, 5, 7, 8, 11, 14, },
  { 0, 1, 2, 5, 7, 8, 11, 14, },
  { 3, 5, 7, 8, 11, 14, },
  { 0, 3, 5, 7, 8, 11, 14, },
  { 1, 3, 5, 7, 8, 11, 14, },
  { 0, 1, 3, 5, 7, 8, 11, 14, },
  { 2, 3, 5, 7, 8, 11, 14, },
  { 0, 2, 3, 5, 7, 8, 11, 14, },
  { 1, 2, 3, 5, 7, 8, 11, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 14, },
  { 4, 5, 7, 8, 11, 14, },
  { 0, 4, 5, 7, 8, 11, 14, },
  { 1, 4, 5, 7, 8, 11, 14, },
  { 0, 1, 4, 5, 7, 8, 11, 14, },
  { 2, 4, 5, 7, 8, 11, 14, },
  { 0, 2, 4, 5, 7, 8, 11, 14, },
  { 1, 2, 4, 5, 7, 8, 11, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 14, },
  { 3, 4, 5, 7, 8, 11, 14, },
  { 0, 3, 4, 5, 7, 8, 11, 14, },
  { 1, 3, 4, 5, 7, 8, 11, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 14, },
  { 2, 3, 4, 5, 7, 8, 11, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 14, },
  { 6, 7, 8, 11, 14, },
  { 0, 6, 7, 8, 11, 14, },
  { 1, 6, 7, 8, 11, 14, },
  { 0, 1, 6, 7, 8, 11, 14, },
  { 2, 6, 7, 8, 11, 14, },
  { 0, 2, 6, 7, 8, 11, 14, },
  { 1, 2, 6, 7, 8, 11, 14, },
  { 0, 1, 2, 6, 7, 8, 11, 14, },
  { 3, 6, 7, 8, 11, 14, },
  { 0, 3, 6, 7, 8, 11, 14, },
  { 1, 3, 6, 7, 8, 11, 14, },
  { 0, 1, 3, 6, 7, 8, 11, 14, },
  { 2, 3, 6, 7, 8, 11, 14, },
  { 0, 2, 3, 6, 7, 8, 11, 14, },
  { 1, 2, 3, 6, 7, 8, 11, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 14, },
  { 4, 6, 7, 8, 11, 14, },
  { 0, 4, 6, 7, 8, 11, 14, },
  { 1, 4, 6, 7, 8, 11, 14, },
  { 0, 1, 4, 6, 7, 8, 11, 14, },
  { 2, 4, 6, 7, 8, 11, 14, },
  { 0, 2, 4, 6, 7, 8, 11, 14, },
  { 1, 2, 4, 6, 7, 8, 11, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 14, },
  { 3, 4, 6, 7, 8, 11, 14, },
  { 0, 3, 4, 6, 7, 8, 11, 14, },
  { 1, 3, 4, 6, 7, 8, 11, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 14, },
  { 2, 3, 4, 6, 7, 8, 11, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 14, },
  { 5, 6, 7, 8, 11, 14, },
  { 0, 5, 6, 7, 8, 11, 14, },
  { 1, 5, 6, 7, 8, 11, 14, },
  { 0, 1, 5, 6, 7, 8, 11, 14, },
  { 2, 5, 6, 7, 8, 11, 14, },
  { 0, 2, 5, 6, 7, 8, 11, 14, },
  { 1, 2, 5, 6, 7, 8, 11, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 14, },
  { 3, 5, 6, 7, 8, 11, 14, },
  { 0, 3, 5, 6, 7, 8, 11, 14, },
  { 1, 3, 5, 6, 7, 8, 11, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 14, },
  { 2, 3, 5, 6, 7, 8, 11, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 14, },
  { 4, 5, 6, 7, 8, 11, 14, },
  { 0, 4, 5, 6, 7, 8, 11, 14, },
  { 1, 4, 5, 6, 7, 8, 11, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 14, },
  { 2, 4, 5, 6, 7, 8, 11, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 14, },
  { 3, 4, 5, 6, 7, 8, 11, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 14, },
  { 9, 11, 14, },
  { 0, 9, 11, 14, },
  { 1, 9, 11, 14, },
  { 0, 1, 9, 11, 14, },
  { 2, 9, 11, 14, },
  { 0, 2, 9, 11, 14, },
  { 1, 2, 9, 11, 14, },
  { 0, 1, 2, 9, 11, 14, },
  { 3, 9, 11, 14, },
  { 0, 3, 9, 11, 14, },
  { 1, 3, 9, 11, 14, },
  { 0, 1, 3, 9, 11, 14, },
  { 2, 3, 9, 11, 14, },
  { 0, 2, 3, 9, 11, 14, },
  { 1, 2, 3, 9, 11, 14, },
  { 0, 1, 2, 3, 9, 11, 14, },
  { 4, 9, 11, 14, },
  { 0, 4, 9, 11, 14, },
  { 1, 4, 9, 11, 14, },
  { 0, 1, 4, 9, 11, 14, },
  { 2, 4, 9, 11, 14, },
  { 0, 2, 4, 9, 11, 14, },
  { 1, 2, 4, 9, 11, 14, },
  { 0, 1, 2, 4, 9, 11, 14, },
  { 3, 4, 9, 11, 14, },
  { 0, 3, 4, 9, 11, 14, },
  { 1, 3, 4, 9, 11, 14, },
  { 0, 1, 3, 4, 9, 11, 14, },
  { 2, 3, 4, 9, 11, 14, },
  { 0, 2, 3, 4, 9, 11, 14, },
  { 1, 2, 3, 4, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 9, 11, 14, },
  { 5, 9, 11, 14, },
  { 0, 5, 9, 11, 14, },
  { 1, 5, 9, 11, 14, },
  { 0, 1, 5, 9, 11, 14, },
  { 2, 5, 9, 11, 14, },
  { 0, 2, 5, 9, 11, 14, },
  { 1, 2, 5, 9, 11, 14, },
  { 0, 1, 2, 5, 9, 11, 14, },
  { 3, 5, 9, 11, 14, },
  { 0, 3, 5, 9, 11, 14, },
  { 1, 3, 5, 9, 11, 14, },
  { 0, 1, 3, 5, 9, 11, 14, },
  { 2, 3, 5, 9, 11, 14, },
  { 0, 2, 3, 5, 9, 11, 14, },
  { 1, 2, 3, 5, 9, 11, 14, },
  { 0, 1, 2, 3, 5, 9, 11, 14, },
  { 4, 5, 9, 11, 14, },
  { 0, 4, 5, 9, 11, 14, },
  { 1, 4, 5, 9, 11, 14, },
  { 0, 1, 4, 5, 9, 11, 14, },
  { 2, 4, 5, 9, 11, 14, },
  { 0, 2, 4, 5, 9, 11, 14, },
  { 1, 2, 4, 5, 9, 11, 14, },
  { 0, 1, 2, 4, 5, 9, 11, 14, },
  { 3, 4, 5, 9, 11, 14, },
  { 0, 3, 4, 5, 9, 11, 14, },
  { 1, 3, 4, 5, 9, 11, 14, },
  { 0, 1, 3, 4, 5, 9, 11, 14, },
  { 2, 3, 4, 5, 9, 11, 14, },
  { 0, 2, 3, 4, 5, 9, 11, 14, },
  { 1, 2, 3, 4, 5, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 14, },
  { 6, 9, 11, 14, },
  { 0, 6, 9, 11, 14, },
  { 1, 6, 9, 11, 14, },
  { 0, 1, 6, 9, 11, 14, },
  { 2, 6, 9, 11, 14, },
  { 0, 2, 6, 9, 11, 14, },
  { 1, 2, 6, 9, 11, 14, },
  { 0, 1, 2, 6, 9, 11, 14, },
  { 3, 6, 9, 11, 14, },
  { 0, 3, 6, 9, 11, 14, },
  { 1, 3, 6, 9, 11, 14, },
  { 0, 1, 3, 6, 9, 11, 14, },
  { 2, 3, 6, 9, 11, 14, },
  { 0, 2, 3, 6, 9, 11, 14, },
  { 1, 2, 3, 6, 9, 11, 14, },
  { 0, 1, 2, 3, 6, 9, 11, 14, },
  { 4, 6, 9, 11, 14, },
  { 0, 4, 6, 9, 11, 14, },
  { 1, 4, 6, 9, 11, 14, },
  { 0, 1, 4, 6, 9, 11, 14, },
  { 2, 4, 6, 9, 11, 14, },
  { 0, 2, 4, 6, 9, 11, 14, },
  { 1, 2, 4, 6, 9, 11, 14, },
  { 0, 1, 2, 4, 6, 9, 11, 14, },
  { 3, 4, 6, 9, 11, 14, },
  { 0, 3, 4, 6, 9, 11, 14, },
  { 1, 3, 4, 6, 9, 11, 14, },
  { 0, 1, 3, 4, 6, 9, 11, 14, },
  { 2, 3, 4, 6, 9, 11, 14, },
  { 0, 2, 3, 4, 6, 9, 11, 14, },
  { 1, 2, 3, 4, 6, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 14, },
  { 5, 6, 9, 11, 14, },
  { 0, 5, 6, 9, 11, 14, },
  { 1, 5, 6, 9, 11, 14, },
  { 0, 1, 5, 6, 9, 11, 14, },
  { 2, 5, 6, 9, 11, 14, },
  { 0, 2, 5, 6, 9, 11, 14, },
  { 1, 2, 5, 6, 9, 11, 14, },
  { 0, 1, 2, 5, 6, 9, 11, 14, },
  { 3, 5, 6, 9, 11, 14, },
  { 0, 3, 5, 6, 9, 11, 14, },
  { 1, 3, 5, 6, 9, 11, 14, },
  { 0, 1, 3, 5, 6, 9, 11, 14, },
  { 2, 3, 5, 6, 9, 11, 14, },
  { 0, 2, 3, 5, 6, 9, 11, 14, },
  { 1, 2, 3, 5, 6, 9, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 14, },
  { 4, 5, 6, 9, 11, 14, },
  { 0, 4, 5, 6, 9, 11, 14, },
  { 1, 4, 5, 6, 9, 11, 14, },
  { 0, 1, 4, 5, 6, 9, 11, 14, },
  { 2, 4, 5, 6, 9, 11, 14, },
  { 0, 2, 4, 5, 6, 9, 11, 14, },
  { 1, 2, 4, 5, 6, 9, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 14, },
  { 3, 4, 5, 6, 9, 11, 14, },
  { 0, 3, 4, 5, 6, 9, 11, 14, },
  { 1, 3, 4, 5, 6, 9, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 14, },
  { 2, 3, 4, 5, 6, 9, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 14, },
  { 7, 9, 11, 14, },
  { 0, 7, 9, 11, 14, },
  { 1, 7, 9, 11, 14, },
  { 0, 1, 7, 9, 11, 14, },
  { 2, 7, 9, 11, 14, },
  { 0, 2, 7, 9, 11, 14, },
  { 1, 2, 7, 9, 11, 14, },
  { 0, 1, 2, 7, 9, 11, 14, },
  { 3, 7, 9, 11, 14, },
  { 0, 3, 7, 9, 11, 14, },
  { 1, 3, 7, 9, 11, 14, },
  { 0, 1, 3, 7, 9, 11, 14, },
  { 2, 3, 7, 9, 11, 14, },
  { 0, 2, 3, 7, 9, 11, 14, },
  { 1, 2, 3, 7, 9, 11, 14, },
  { 0, 1, 2, 3, 7, 9, 11, 14, },
  { 4, 7, 9, 11, 14, },
  { 0, 4, 7, 9, 11, 14, },
  { 1, 4, 7, 9, 11, 14, },
  { 0, 1, 4, 7, 9, 11, 14, },
  { 2, 4, 7, 9, 11, 14, },
  { 0, 2, 4, 7, 9, 11, 14, },
  { 1, 2, 4, 7, 9, 11, 14, },
  { 0, 1, 2, 4, 7, 9, 11, 14, },
  { 3, 4, 7, 9, 11, 14, },
  { 0, 3, 4, 7, 9, 11, 14, },
  { 1, 3, 4, 7, 9, 11, 14, },
  { 0, 1, 3, 4, 7, 9, 11, 14, },
  { 2, 3, 4, 7, 9, 11, 14, },
  { 0, 2, 3, 4, 7, 9, 11, 14, },
  { 1, 2, 3, 4, 7, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 14, },
  { 5, 7, 9, 11, 14, },
  { 0, 5, 7, 9, 11, 14, },
  { 1, 5, 7, 9, 11, 14, },
  { 0, 1, 5, 7, 9, 11, 14, },
  { 2, 5, 7, 9, 11, 14, },
  { 0, 2, 5, 7, 9, 11, 14, },
  { 1, 2, 5, 7, 9, 11, 14, },
  { 0, 1, 2, 5, 7, 9, 11, 14, },
  { 3, 5, 7, 9, 11, 14, },
  { 0, 3, 5, 7, 9, 11, 14, },
  { 1, 3, 5, 7, 9, 11, 14, },
  { 0, 1, 3, 5, 7, 9, 11, 14, },
  { 2, 3, 5, 7, 9, 11, 14, },
  { 0, 2, 3, 5, 7, 9, 11, 14, },
  { 1, 2, 3, 5, 7, 9, 11, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 14, },
  { 4, 5, 7, 9, 11, 14, },
  { 0, 4, 5, 7, 9, 11, 14, },
  { 1, 4, 5, 7, 9, 11, 14, },
  { 0, 1, 4, 5, 7, 9, 11, 14, },
  { 2, 4, 5, 7, 9, 11, 14, },
  { 0, 2, 4, 5, 7, 9, 11, 14, },
  { 1, 2, 4, 5, 7, 9, 11, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 14, },
  { 3, 4, 5, 7, 9, 11, 14, },
  { 0, 3, 4, 5, 7, 9, 11, 14, },
  { 1, 3, 4, 5, 7, 9, 11, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 14, },
  { 2, 3, 4, 5, 7, 9, 11, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 14, },
  { 6, 7, 9, 11, 14, },
  { 0, 6, 7, 9, 11, 14, },
  { 1, 6, 7, 9, 11, 14, },
  { 0, 1, 6, 7, 9, 11, 14, },
  { 2, 6, 7, 9, 11, 14, },
  { 0, 2, 6, 7, 9, 11, 14, },
  { 1, 2, 6, 7, 9, 11, 14, },
  { 0, 1, 2, 6, 7, 9, 11, 14, },
  { 3, 6, 7, 9, 11, 14, },
  { 0, 3, 6, 7, 9, 11, 14, },
  { 1, 3, 6, 7, 9, 11, 14, },
  { 0, 1, 3, 6, 7, 9, 11, 14, },
  { 2, 3, 6, 7, 9, 11, 14, },
  { 0, 2, 3, 6, 7, 9, 11, 14, },
  { 1, 2, 3, 6, 7, 9, 11, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 14, },
  { 4, 6, 7, 9, 11, 14, },
  { 0, 4, 6, 7, 9, 11, 14, },
  { 1, 4, 6, 7, 9, 11, 14, },
  { 0, 1, 4, 6, 7, 9, 11, 14, },
  { 2, 4, 6, 7, 9, 11, 14, },
  { 0, 2, 4, 6, 7, 9, 11, 14, },
  { 1, 2, 4, 6, 7, 9, 11, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 14, },
  { 3, 4, 6, 7, 9, 11, 14, },
  { 0, 3, 4, 6, 7, 9, 11, 14, },
  { 1, 3, 4, 6, 7, 9, 11, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 14, },
  { 2, 3, 4, 6, 7, 9, 11, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 14, },
  { 5, 6, 7, 9, 11, 14, },
  { 0, 5, 6, 7, 9, 11, 14, },
  { 1, 5, 6, 7, 9, 11, 14, },
  { 0, 1, 5, 6, 7, 9, 11, 14, },
  { 2, 5, 6, 7, 9, 11, 14, },
  { 0, 2, 5, 6, 7, 9, 11, 14, },
  { 1, 2, 5, 6, 7, 9, 11, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 14, },
  { 3, 5, 6, 7, 9, 11, 14, },
  { 0, 3, 5, 6, 7, 9, 11, 14, },
  { 1, 3, 5, 6, 7, 9, 11, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 14, },
  { 2, 3, 5, 6, 7, 9, 11, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 14, },
  { 4, 5, 6, 7, 9, 11, 14, },
  { 0, 4, 5, 6, 7, 9, 11, 14, },
  { 1, 4, 5, 6, 7, 9, 11, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 14, },
  { 2, 4, 5, 6, 7, 9, 11, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 14, },
  { 3, 4, 5, 6, 7, 9, 11, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 14, },
  { 8, 9, 11, 14, },
  { 0, 8, 9, 11, 14, },
  { 1, 8, 9, 11, 14, },
  { 0, 1, 8, 9, 11, 14, },
  { 2, 8, 9, 11, 14, },
  { 0, 2, 8, 9, 11, 14, },
  { 1, 2, 8, 9, 11, 14, },
  { 0, 1, 2, 8, 9, 11, 14, },
  { 3, 8, 9, 11, 14, },
  { 0, 3, 8, 9, 11, 14, },
  { 1, 3, 8, 9, 11, 14, },
  { 0, 1, 3, 8, 9, 11, 14, },
  { 2, 3, 8, 9, 11, 14, },
  { 0, 2, 3, 8, 9, 11, 14, },
  { 1, 2, 3, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 8, 9, 11, 14, },
  { 4, 8, 9, 11, 14, },
  { 0, 4, 8, 9, 11, 14, },
  { 1, 4, 8, 9, 11, 14, },
  { 0, 1, 4, 8, 9, 11, 14, },
  { 2, 4, 8, 9, 11, 14, },
  { 0, 2, 4, 8, 9, 11, 14, },
  { 1, 2, 4, 8, 9, 11, 14, },
  { 0, 1, 2, 4, 8, 9, 11, 14, },
  { 3, 4, 8, 9, 11, 14, },
  { 0, 3, 4, 8, 9, 11, 14, },
  { 1, 3, 4, 8, 9, 11, 14, },
  { 0, 1, 3, 4, 8, 9, 11, 14, },
  { 2, 3, 4, 8, 9, 11, 14, },
  { 0, 2, 3, 4, 8, 9, 11, 14, },
  { 1, 2, 3, 4, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 14, },
  { 5, 8, 9, 11, 14, },
  { 0, 5, 8, 9, 11, 14, },
  { 1, 5, 8, 9, 11, 14, },
  { 0, 1, 5, 8, 9, 11, 14, },
  { 2, 5, 8, 9, 11, 14, },
  { 0, 2, 5, 8, 9, 11, 14, },
  { 1, 2, 5, 8, 9, 11, 14, },
  { 0, 1, 2, 5, 8, 9, 11, 14, },
  { 3, 5, 8, 9, 11, 14, },
  { 0, 3, 5, 8, 9, 11, 14, },
  { 1, 3, 5, 8, 9, 11, 14, },
  { 0, 1, 3, 5, 8, 9, 11, 14, },
  { 2, 3, 5, 8, 9, 11, 14, },
  { 0, 2, 3, 5, 8, 9, 11, 14, },
  { 1, 2, 3, 5, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 14, },
  { 4, 5, 8, 9, 11, 14, },
  { 0, 4, 5, 8, 9, 11, 14, },
  { 1, 4, 5, 8, 9, 11, 14, },
  { 0, 1, 4, 5, 8, 9, 11, 14, },
  { 2, 4, 5, 8, 9, 11, 14, },
  { 0, 2, 4, 5, 8, 9, 11, 14, },
  { 1, 2, 4, 5, 8, 9, 11, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 14, },
  { 3, 4, 5, 8, 9, 11, 14, },
  { 0, 3, 4, 5, 8, 9, 11, 14, },
  { 1, 3, 4, 5, 8, 9, 11, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 14, },
  { 2, 3, 4, 5, 8, 9, 11, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 14, },
  { 6, 8, 9, 11, 14, },
  { 0, 6, 8, 9, 11, 14, },
  { 1, 6, 8, 9, 11, 14, },
  { 0, 1, 6, 8, 9, 11, 14, },
  { 2, 6, 8, 9, 11, 14, },
  { 0, 2, 6, 8, 9, 11, 14, },
  { 1, 2, 6, 8, 9, 11, 14, },
  { 0, 1, 2, 6, 8, 9, 11, 14, },
  { 3, 6, 8, 9, 11, 14, },
  { 0, 3, 6, 8, 9, 11, 14, },
  { 1, 3, 6, 8, 9, 11, 14, },
  { 0, 1, 3, 6, 8, 9, 11, 14, },
  { 2, 3, 6, 8, 9, 11, 14, },
  { 0, 2, 3, 6, 8, 9, 11, 14, },
  { 1, 2, 3, 6, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 14, },
  { 4, 6, 8, 9, 11, 14, },
  { 0, 4, 6, 8, 9, 11, 14, },
  { 1, 4, 6, 8, 9, 11, 14, },
  { 0, 1, 4, 6, 8, 9, 11, 14, },
  { 2, 4, 6, 8, 9, 11, 14, },
  { 0, 2, 4, 6, 8, 9, 11, 14, },
  { 1, 2, 4, 6, 8, 9, 11, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 14, },
  { 3, 4, 6, 8, 9, 11, 14, },
  { 0, 3, 4, 6, 8, 9, 11, 14, },
  { 1, 3, 4, 6, 8, 9, 11, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 14, },
  { 2, 3, 4, 6, 8, 9, 11, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 14, },
  { 5, 6, 8, 9, 11, 14, },
  { 0, 5, 6, 8, 9, 11, 14, },
  { 1, 5, 6, 8, 9, 11, 14, },
  { 0, 1, 5, 6, 8, 9, 11, 14, },
  { 2, 5, 6, 8, 9, 11, 14, },
  { 0, 2, 5, 6, 8, 9, 11, 14, },
  { 1, 2, 5, 6, 8, 9, 11, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 14, },
  { 3, 5, 6, 8, 9, 11, 14, },
  { 0, 3, 5, 6, 8, 9, 11, 14, },
  { 1, 3, 5, 6, 8, 9, 11, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 14, },
  { 2, 3, 5, 6, 8, 9, 11, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 14, },
  { 4, 5, 6, 8, 9, 11, 14, },
  { 0, 4, 5, 6, 8, 9, 11, 14, },
  { 1, 4, 5, 6, 8, 9, 11, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 14, },
  { 2, 4, 5, 6, 8, 9, 11, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 14, },
  { 3, 4, 5, 6, 8, 9, 11, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 14, },
  { 7, 8, 9, 11, 14, },
  { 0, 7, 8, 9, 11, 14, },
  { 1, 7, 8, 9, 11, 14, },
  { 0, 1, 7, 8, 9, 11, 14, },
  { 2, 7, 8, 9, 11, 14, },
  { 0, 2, 7, 8, 9, 11, 14, },
  { 1, 2, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 7, 8, 9, 11, 14, },
  { 3, 7, 8, 9, 11, 14, },
  { 0, 3, 7, 8, 9, 11, 14, },
  { 1, 3, 7, 8, 9, 11, 14, },
  { 0, 1, 3, 7, 8, 9, 11, 14, },
  { 2, 3, 7, 8, 9, 11, 14, },
  { 0, 2, 3, 7, 8, 9, 11, 14, },
  { 1, 2, 3, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 14, },
  { 4, 7, 8, 9, 11, 14, },
  { 0, 4, 7, 8, 9, 11, 14, },
  { 1, 4, 7, 8, 9, 11, 14, },
  { 0, 1, 4, 7, 8, 9, 11, 14, },
  { 2, 4, 7, 8, 9, 11, 14, },
  { 0, 2, 4, 7, 8, 9, 11, 14, },
  { 1, 2, 4, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 14, },
  { 3, 4, 7, 8, 9, 11, 14, },
  { 0, 3, 4, 7, 8, 9, 11, 14, },
  { 1, 3, 4, 7, 8, 9, 11, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 14, },
  { 2, 3, 4, 7, 8, 9, 11, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 14, },
  { 5, 7, 8, 9, 11, 14, },
  { 0, 5, 7, 8, 9, 11, 14, },
  { 1, 5, 7, 8, 9, 11, 14, },
  { 0, 1, 5, 7, 8, 9, 11, 14, },
  { 2, 5, 7, 8, 9, 11, 14, },
  { 0, 2, 5, 7, 8, 9, 11, 14, },
  { 1, 2, 5, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 14, },
  { 3, 5, 7, 8, 9, 11, 14, },
  { 0, 3, 5, 7, 8, 9, 11, 14, },
  { 1, 3, 5, 7, 8, 9, 11, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 14, },
  { 2, 3, 5, 7, 8, 9, 11, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 14, },
  { 4, 5, 7, 8, 9, 11, 14, },
  { 0, 4, 5, 7, 8, 9, 11, 14, },
  { 1, 4, 5, 7, 8, 9, 11, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 14, },
  { 2, 4, 5, 7, 8, 9, 11, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 14, },
  { 3, 4, 5, 7, 8, 9, 11, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 14, },
  { 6, 7, 8, 9, 11, 14, },
  { 0, 6, 7, 8, 9, 11, 14, },
  { 1, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 6, 7, 8, 9, 11, 14, },
  { 2, 6, 7, 8, 9, 11, 14, },
  { 0, 2, 6, 7, 8, 9, 11, 14, },
  { 1, 2, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 14, },
  { 3, 6, 7, 8, 9, 11, 14, },
  { 0, 3, 6, 7, 8, 9, 11, 14, },
  { 1, 3, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 14, },
  { 2, 3, 6, 7, 8, 9, 11, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 14, },
  { 4, 6, 7, 8, 9, 11, 14, },
  { 0, 4, 6, 7, 8, 9, 11, 14, },
  { 1, 4, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 14, },
  { 2, 4, 6, 7, 8, 9, 11, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 14, },
  { 3, 4, 6, 7, 8, 9, 11, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 14, },
  { 5, 6, 7, 8, 9, 11, 14, },
  { 0, 5, 6, 7, 8, 9, 11, 14, },
  { 1, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 14, },
  { 2, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 14, },
  { 3, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 14, },
  { 4, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 14, },
  { 10, 11, 14, },
  { 0, 10, 11, 14, },
  { 1, 10, 11, 14, },
  { 0, 1, 10, 11, 14, },
  { 2, 10, 11, 14, },
  { 0, 2, 10, 11, 14, },
  { 1, 2, 10, 11, 14, },
  { 0, 1, 2, 10, 11, 14, },
  { 3, 10, 11, 14, },
  { 0, 3, 10, 11, 14, },
  { 1, 3, 10, 11, 14, },
  { 0, 1, 3, 10, 11, 14, },
  { 2, 3, 10, 11, 14, },
  { 0, 2, 3, 10, 11, 14, },
  { 1, 2, 3, 10, 11, 14, },
  { 0, 1, 2, 3, 10, 11, 14, },
  { 4, 10, 11, 14, },
  { 0, 4, 10, 11, 14, },
  { 1, 4, 10, 11, 14, },
  { 0, 1, 4, 10, 11, 14, },
  { 2, 4, 10, 11, 14, },
  { 0, 2, 4, 10, 11, 14, },
  { 1, 2, 4, 10, 11, 14, },
  { 0, 1, 2, 4, 10, 11, 14, },
  { 3, 4, 10, 11, 14, },
  { 0, 3, 4, 10, 11, 14, },
  { 1, 3, 4, 10, 11, 14, },
  { 0, 1, 3, 4, 10, 11, 14, },
  { 2, 3, 4, 10, 11, 14, },
  { 0, 2, 3, 4, 10, 11, 14, },
  { 1, 2, 3, 4, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 10, 11, 14, },
  { 5, 10, 11, 14, },
  { 0, 5, 10, 11, 14, },
  { 1, 5, 10, 11, 14, },
  { 0, 1, 5, 10, 11, 14, },
  { 2, 5, 10, 11, 14, },
  { 0, 2, 5, 10, 11, 14, },
  { 1, 2, 5, 10, 11, 14, },
  { 0, 1, 2, 5, 10, 11, 14, },
  { 3, 5, 10, 11, 14, },
  { 0, 3, 5, 10, 11, 14, },
  { 1, 3, 5, 10, 11, 14, },
  { 0, 1, 3, 5, 10, 11, 14, },
  { 2, 3, 5, 10, 11, 14, },
  { 0, 2, 3, 5, 10, 11, 14, },
  { 1, 2, 3, 5, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 10, 11, 14, },
  { 4, 5, 10, 11, 14, },
  { 0, 4, 5, 10, 11, 14, },
  { 1, 4, 5, 10, 11, 14, },
  { 0, 1, 4, 5, 10, 11, 14, },
  { 2, 4, 5, 10, 11, 14, },
  { 0, 2, 4, 5, 10, 11, 14, },
  { 1, 2, 4, 5, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 10, 11, 14, },
  { 3, 4, 5, 10, 11, 14, },
  { 0, 3, 4, 5, 10, 11, 14, },
  { 1, 3, 4, 5, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 10, 11, 14, },
  { 2, 3, 4, 5, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 14, },
  { 6, 10, 11, 14, },
  { 0, 6, 10, 11, 14, },
  { 1, 6, 10, 11, 14, },
  { 0, 1, 6, 10, 11, 14, },
  { 2, 6, 10, 11, 14, },
  { 0, 2, 6, 10, 11, 14, },
  { 1, 2, 6, 10, 11, 14, },
  { 0, 1, 2, 6, 10, 11, 14, },
  { 3, 6, 10, 11, 14, },
  { 0, 3, 6, 10, 11, 14, },
  { 1, 3, 6, 10, 11, 14, },
  { 0, 1, 3, 6, 10, 11, 14, },
  { 2, 3, 6, 10, 11, 14, },
  { 0, 2, 3, 6, 10, 11, 14, },
  { 1, 2, 3, 6, 10, 11, 14, },
  { 0, 1, 2, 3, 6, 10, 11, 14, },
  { 4, 6, 10, 11, 14, },
  { 0, 4, 6, 10, 11, 14, },
  { 1, 4, 6, 10, 11, 14, },
  { 0, 1, 4, 6, 10, 11, 14, },
  { 2, 4, 6, 10, 11, 14, },
  { 0, 2, 4, 6, 10, 11, 14, },
  { 1, 2, 4, 6, 10, 11, 14, },
  { 0, 1, 2, 4, 6, 10, 11, 14, },
  { 3, 4, 6, 10, 11, 14, },
  { 0, 3, 4, 6, 10, 11, 14, },
  { 1, 3, 4, 6, 10, 11, 14, },
  { 0, 1, 3, 4, 6, 10, 11, 14, },
  { 2, 3, 4, 6, 10, 11, 14, },
  { 0, 2, 3, 4, 6, 10, 11, 14, },
  { 1, 2, 3, 4, 6, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 14, },
  { 5, 6, 10, 11, 14, },
  { 0, 5, 6, 10, 11, 14, },
  { 1, 5, 6, 10, 11, 14, },
  { 0, 1, 5, 6, 10, 11, 14, },
  { 2, 5, 6, 10, 11, 14, },
  { 0, 2, 5, 6, 10, 11, 14, },
  { 1, 2, 5, 6, 10, 11, 14, },
  { 0, 1, 2, 5, 6, 10, 11, 14, },
  { 3, 5, 6, 10, 11, 14, },
  { 0, 3, 5, 6, 10, 11, 14, },
  { 1, 3, 5, 6, 10, 11, 14, },
  { 0, 1, 3, 5, 6, 10, 11, 14, },
  { 2, 3, 5, 6, 10, 11, 14, },
  { 0, 2, 3, 5, 6, 10, 11, 14, },
  { 1, 2, 3, 5, 6, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 14, },
  { 4, 5, 6, 10, 11, 14, },
  { 0, 4, 5, 6, 10, 11, 14, },
  { 1, 4, 5, 6, 10, 11, 14, },
  { 0, 1, 4, 5, 6, 10, 11, 14, },
  { 2, 4, 5, 6, 10, 11, 14, },
  { 0, 2, 4, 5, 6, 10, 11, 14, },
  { 1, 2, 4, 5, 6, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 14, },
  { 3, 4, 5, 6, 10, 11, 14, },
  { 0, 3, 4, 5, 6, 10, 11, 14, },
  { 1, 3, 4, 5, 6, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 14, },
  { 2, 3, 4, 5, 6, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 14, },
  { 7, 10, 11, 14, },
  { 0, 7, 10, 11, 14, },
  { 1, 7, 10, 11, 14, },
  { 0, 1, 7, 10, 11, 14, },
  { 2, 7, 10, 11, 14, },
  { 0, 2, 7, 10, 11, 14, },
  { 1, 2, 7, 10, 11, 14, },
  { 0, 1, 2, 7, 10, 11, 14, },
  { 3, 7, 10, 11, 14, },
  { 0, 3, 7, 10, 11, 14, },
  { 1, 3, 7, 10, 11, 14, },
  { 0, 1, 3, 7, 10, 11, 14, },
  { 2, 3, 7, 10, 11, 14, },
  { 0, 2, 3, 7, 10, 11, 14, },
  { 1, 2, 3, 7, 10, 11, 14, },
  { 0, 1, 2, 3, 7, 10, 11, 14, },
  { 4, 7, 10, 11, 14, },
  { 0, 4, 7, 10, 11, 14, },
  { 1, 4, 7, 10, 11, 14, },
  { 0, 1, 4, 7, 10, 11, 14, },
  { 2, 4, 7, 10, 11, 14, },
  { 0, 2, 4, 7, 10, 11, 14, },
  { 1, 2, 4, 7, 10, 11, 14, },
  { 0, 1, 2, 4, 7, 10, 11, 14, },
  { 3, 4, 7, 10, 11, 14, },
  { 0, 3, 4, 7, 10, 11, 14, },
  { 1, 3, 4, 7, 10, 11, 14, },
  { 0, 1, 3, 4, 7, 10, 11, 14, },
  { 2, 3, 4, 7, 10, 11, 14, },
  { 0, 2, 3, 4, 7, 10, 11, 14, },
  { 1, 2, 3, 4, 7, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 14, },
  { 5, 7, 10, 11, 14, },
  { 0, 5, 7, 10, 11, 14, },
  { 1, 5, 7, 10, 11, 14, },
  { 0, 1, 5, 7, 10, 11, 14, },
  { 2, 5, 7, 10, 11, 14, },
  { 0, 2, 5, 7, 10, 11, 14, },
  { 1, 2, 5, 7, 10, 11, 14, },
  { 0, 1, 2, 5, 7, 10, 11, 14, },
  { 3, 5, 7, 10, 11, 14, },
  { 0, 3, 5, 7, 10, 11, 14, },
  { 1, 3, 5, 7, 10, 11, 14, },
  { 0, 1, 3, 5, 7, 10, 11, 14, },
  { 2, 3, 5, 7, 10, 11, 14, },
  { 0, 2, 3, 5, 7, 10, 11, 14, },
  { 1, 2, 3, 5, 7, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 14, },
  { 4, 5, 7, 10, 11, 14, },
  { 0, 4, 5, 7, 10, 11, 14, },
  { 1, 4, 5, 7, 10, 11, 14, },
  { 0, 1, 4, 5, 7, 10, 11, 14, },
  { 2, 4, 5, 7, 10, 11, 14, },
  { 0, 2, 4, 5, 7, 10, 11, 14, },
  { 1, 2, 4, 5, 7, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 14, },
  { 3, 4, 5, 7, 10, 11, 14, },
  { 0, 3, 4, 5, 7, 10, 11, 14, },
  { 1, 3, 4, 5, 7, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 14, },
  { 2, 3, 4, 5, 7, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 14, },
  { 6, 7, 10, 11, 14, },
  { 0, 6, 7, 10, 11, 14, },
  { 1, 6, 7, 10, 11, 14, },
  { 0, 1, 6, 7, 10, 11, 14, },
  { 2, 6, 7, 10, 11, 14, },
  { 0, 2, 6, 7, 10, 11, 14, },
  { 1, 2, 6, 7, 10, 11, 14, },
  { 0, 1, 2, 6, 7, 10, 11, 14, },
  { 3, 6, 7, 10, 11, 14, },
  { 0, 3, 6, 7, 10, 11, 14, },
  { 1, 3, 6, 7, 10, 11, 14, },
  { 0, 1, 3, 6, 7, 10, 11, 14, },
  { 2, 3, 6, 7, 10, 11, 14, },
  { 0, 2, 3, 6, 7, 10, 11, 14, },
  { 1, 2, 3, 6, 7, 10, 11, 14, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 14, },
  { 4, 6, 7, 10, 11, 14, },
  { 0, 4, 6, 7, 10, 11, 14, },
  { 1, 4, 6, 7, 10, 11, 14, },
  { 0, 1, 4, 6, 7, 10, 11, 14, },
  { 2, 4, 6, 7, 10, 11, 14, },
  { 0, 2, 4, 6, 7, 10, 11, 14, },
  { 1, 2, 4, 6, 7, 10, 11, 14, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 14, },
  { 3, 4, 6, 7, 10, 11, 14, },
  { 0, 3, 4, 6, 7, 10, 11, 14, },
  { 1, 3, 4, 6, 7, 10, 11, 14, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 14, },
  { 2, 3, 4, 6, 7, 10, 11, 14, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 14, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 14, },
  { 5, 6, 7, 10, 11, 14, },
  { 0, 5, 6, 7, 10, 11, 14, },
  { 1, 5, 6, 7, 10, 11, 14, },
  { 0, 1, 5, 6, 7, 10, 11, 14, },
  { 2, 5, 6, 7, 10, 11, 14, },
  { 0, 2, 5, 6, 7, 10, 11, 14, },
  { 1, 2, 5, 6, 7, 10, 11, 14, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 14, },
  { 3, 5, 6, 7, 10, 11, 14, },
  { 0, 3, 5, 6, 7, 10, 11, 14, },
  { 1, 3, 5, 6, 7, 10, 11, 14, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 14, },
  { 2, 3, 5, 6, 7, 10, 11, 14, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 14, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 14, },
  { 4, 5, 6, 7, 10, 11, 14, },
  { 0, 4, 5, 6, 7, 10, 11, 14, },
  { 1, 4, 5, 6, 7, 10, 11, 14, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 14, },
  { 2, 4, 5, 6, 7, 10, 11, 14, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 14, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 14, },
  { 3, 4, 5, 6, 7, 10, 11, 14, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 14, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 14, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 14, },
  { 8, 10, 11, 14, },
  { 0, 8, 10, 11, 14, },
  { 1, 8, 10, 11, 14, },
  { 0, 1, 8, 10, 11, 14, },
  { 2, 8, 10, 11, 14, },
  { 0, 2, 8, 10, 11, 14, },
  { 1, 2, 8, 10, 11, 14, },
  { 0, 1, 2, 8, 10, 11, 14, },
  { 3, 8, 10, 11, 14, },
  { 0, 3, 8, 10, 11, 14, },
  { 1, 3, 8, 10, 11, 14, },
  { 0, 1, 3, 8, 10, 11, 14, },
  { 2, 3, 8, 10, 11, 14, },
  { 0, 2, 3, 8, 10, 11, 14, },
  { 1, 2, 3, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 8, 10, 11, 14, },
  { 4, 8, 10, 11, 14, },
  { 0, 4, 8, 10, 11, 14, },
  { 1, 4, 8, 10, 11, 14, },
  { 0, 1, 4, 8, 10, 11, 14, },
  { 2, 4, 8, 10, 11, 14, },
  { 0, 2, 4, 8, 10, 11, 14, },
  { 1, 2, 4, 8, 10, 11, 14, },
  { 0, 1, 2, 4, 8, 10, 11, 14, },
  { 3, 4, 8, 10, 11, 14, },
  { 0, 3, 4, 8, 10, 11, 14, },
  { 1, 3, 4, 8, 10, 11, 14, },
  { 0, 1, 3, 4, 8, 10, 11, 14, },
  { 2, 3, 4, 8, 10, 11, 14, },
  { 0, 2, 3, 4, 8, 10, 11, 14, },
  { 1, 2, 3, 4, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 14, },
  { 5, 8, 10, 11, 14, },
  { 0, 5, 8, 10, 11, 14, },
  { 1, 5, 8, 10, 11, 14, },
  { 0, 1, 5, 8, 10, 11, 14, },
  { 2, 5, 8, 10, 11, 14, },
  { 0, 2, 5, 8, 10, 11, 14, },
  { 1, 2, 5, 8, 10, 11, 14, },
  { 0, 1, 2, 5, 8, 10, 11, 14, },
  { 3, 5, 8, 10, 11, 14, },
  { 0, 3, 5, 8, 10, 11, 14, },
  { 1, 3, 5, 8, 10, 11, 14, },
  { 0, 1, 3, 5, 8, 10, 11, 14, },
  { 2, 3, 5, 8, 10, 11, 14, },
  { 0, 2, 3, 5, 8, 10, 11, 14, },
  { 1, 2, 3, 5, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 14, },
  { 4, 5, 8, 10, 11, 14, },
  { 0, 4, 5, 8, 10, 11, 14, },
  { 1, 4, 5, 8, 10, 11, 14, },
  { 0, 1, 4, 5, 8, 10, 11, 14, },
  { 2, 4, 5, 8, 10, 11, 14, },
  { 0, 2, 4, 5, 8, 10, 11, 14, },
  { 1, 2, 4, 5, 8, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 14, },
  { 3, 4, 5, 8, 10, 11, 14, },
  { 0, 3, 4, 5, 8, 10, 11, 14, },
  { 1, 3, 4, 5, 8, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 14, },
  { 2, 3, 4, 5, 8, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 14, },
  { 6, 8, 10, 11, 14, },
  { 0, 6, 8, 10, 11, 14, },
  { 1, 6, 8, 10, 11, 14, },
  { 0, 1, 6, 8, 10, 11, 14, },
  { 2, 6, 8, 10, 11, 14, },
  { 0, 2, 6, 8, 10, 11, 14, },
  { 1, 2, 6, 8, 10, 11, 14, },
  { 0, 1, 2, 6, 8, 10, 11, 14, },
  { 3, 6, 8, 10, 11, 14, },
  { 0, 3, 6, 8, 10, 11, 14, },
  { 1, 3, 6, 8, 10, 11, 14, },
  { 0, 1, 3, 6, 8, 10, 11, 14, },
  { 2, 3, 6, 8, 10, 11, 14, },
  { 0, 2, 3, 6, 8, 10, 11, 14, },
  { 1, 2, 3, 6, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 14, },
  { 4, 6, 8, 10, 11, 14, },
  { 0, 4, 6, 8, 10, 11, 14, },
  { 1, 4, 6, 8, 10, 11, 14, },
  { 0, 1, 4, 6, 8, 10, 11, 14, },
  { 2, 4, 6, 8, 10, 11, 14, },
  { 0, 2, 4, 6, 8, 10, 11, 14, },
  { 1, 2, 4, 6, 8, 10, 11, 14, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 14, },
  { 3, 4, 6, 8, 10, 11, 14, },
  { 0, 3, 4, 6, 8, 10, 11, 14, },
  { 1, 3, 4, 6, 8, 10, 11, 14, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 14, },
  { 2, 3, 4, 6, 8, 10, 11, 14, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 14, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 14, },
  { 5, 6, 8, 10, 11, 14, },
  { 0, 5, 6, 8, 10, 11, 14, },
  { 1, 5, 6, 8, 10, 11, 14, },
  { 0, 1, 5, 6, 8, 10, 11, 14, },
  { 2, 5, 6, 8, 10, 11, 14, },
  { 0, 2, 5, 6, 8, 10, 11, 14, },
  { 1, 2, 5, 6, 8, 10, 11, 14, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 14, },
  { 3, 5, 6, 8, 10, 11, 14, },
  { 0, 3, 5, 6, 8, 10, 11, 14, },
  { 1, 3, 5, 6, 8, 10, 11, 14, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 14, },
  { 2, 3, 5, 6, 8, 10, 11, 14, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 14, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 14, },
  { 4, 5, 6, 8, 10, 11, 14, },
  { 0, 4, 5, 6, 8, 10, 11, 14, },
  { 1, 4, 5, 6, 8, 10, 11, 14, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 14, },
  { 2, 4, 5, 6, 8, 10, 11, 14, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 14, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 14, },
  { 3, 4, 5, 6, 8, 10, 11, 14, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 14, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 14, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 14, },
  { 7, 8, 10, 11, 14, },
  { 0, 7, 8, 10, 11, 14, },
  { 1, 7, 8, 10, 11, 14, },
  { 0, 1, 7, 8, 10, 11, 14, },
  { 2, 7, 8, 10, 11, 14, },
  { 0, 2, 7, 8, 10, 11, 14, },
  { 1, 2, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 7, 8, 10, 11, 14, },
  { 3, 7, 8, 10, 11, 14, },
  { 0, 3, 7, 8, 10, 11, 14, },
  { 1, 3, 7, 8, 10, 11, 14, },
  { 0, 1, 3, 7, 8, 10, 11, 14, },
  { 2, 3, 7, 8, 10, 11, 14, },
  { 0, 2, 3, 7, 8, 10, 11, 14, },
  { 1, 2, 3, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 14, },
  { 4, 7, 8, 10, 11, 14, },
  { 0, 4, 7, 8, 10, 11, 14, },
  { 1, 4, 7, 8, 10, 11, 14, },
  { 0, 1, 4, 7, 8, 10, 11, 14, },
  { 2, 4, 7, 8, 10, 11, 14, },
  { 0, 2, 4, 7, 8, 10, 11, 14, },
  { 1, 2, 4, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 14, },
  { 3, 4, 7, 8, 10, 11, 14, },
  { 0, 3, 4, 7, 8, 10, 11, 14, },
  { 1, 3, 4, 7, 8, 10, 11, 14, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 14, },
  { 2, 3, 4, 7, 8, 10, 11, 14, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 14, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 14, },
  { 5, 7, 8, 10, 11, 14, },
  { 0, 5, 7, 8, 10, 11, 14, },
  { 1, 5, 7, 8, 10, 11, 14, },
  { 0, 1, 5, 7, 8, 10, 11, 14, },
  { 2, 5, 7, 8, 10, 11, 14, },
  { 0, 2, 5, 7, 8, 10, 11, 14, },
  { 1, 2, 5, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 14, },
  { 3, 5, 7, 8, 10, 11, 14, },
  { 0, 3, 5, 7, 8, 10, 11, 14, },
  { 1, 3, 5, 7, 8, 10, 11, 14, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 14, },
  { 2, 3, 5, 7, 8, 10, 11, 14, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 14, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 14, },
  { 4, 5, 7, 8, 10, 11, 14, },
  { 0, 4, 5, 7, 8, 10, 11, 14, },
  { 1, 4, 5, 7, 8, 10, 11, 14, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 14, },
  { 2, 4, 5, 7, 8, 10, 11, 14, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 14, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 14, },
  { 3, 4, 5, 7, 8, 10, 11, 14, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 14, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 14, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 14, },
  { 6, 7, 8, 10, 11, 14, },
  { 0, 6, 7, 8, 10, 11, 14, },
  { 1, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 6, 7, 8, 10, 11, 14, },
  { 2, 6, 7, 8, 10, 11, 14, },
  { 0, 2, 6, 7, 8, 10, 11, 14, },
  { 1, 2, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 14, },
  { 3, 6, 7, 8, 10, 11, 14, },
  { 0, 3, 6, 7, 8, 10, 11, 14, },
  { 1, 3, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 14, },
  { 2, 3, 6, 7, 8, 10, 11, 14, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 14, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 14, },
  { 4, 6, 7, 8, 10, 11, 14, },
  { 0, 4, 6, 7, 8, 10, 11, 14, },
  { 1, 4, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 14, },
  { 2, 4, 6, 7, 8, 10, 11, 14, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 14, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 14, },
  { 3, 4, 6, 7, 8, 10, 11, 14, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 14, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 14, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 14, },
  { 5, 6, 7, 8, 10, 11, 14, },
  { 0, 5, 6, 7, 8, 10, 11, 14, },
  { 1, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 14, },
  { 2, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 14, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 14, },
  { 3, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 14, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 14, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 14, },
  { 4, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 14, },
  { 9, 10, 11, 14, },
  { 0, 9, 10, 11, 14, },
  { 1, 9, 10, 11, 14, },
  { 0, 1, 9, 10, 11, 14, },
  { 2, 9, 10, 11, 14, },
  { 0, 2, 9, 10, 11, 14, },
  { 1, 2, 9, 10, 11, 14, },
  { 0, 1, 2, 9, 10, 11, 14, },
  { 3, 9, 10, 11, 14, },
  { 0, 3, 9, 10, 11, 14, },
  { 1, 3, 9, 10, 11, 14, },
  { 0, 1, 3, 9, 10, 11, 14, },
  { 2, 3, 9, 10, 11, 14, },
  { 0, 2, 3, 9, 10, 11, 14, },
  { 1, 2, 3, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 9, 10, 11, 14, },
  { 4, 9, 10, 11, 14, },
  { 0, 4, 9, 10, 11, 14, },
  { 1, 4, 9, 10, 11, 14, },
  { 0, 1, 4, 9, 10, 11, 14, },
  { 2, 4, 9, 10, 11, 14, },
  { 0, 2, 4, 9, 10, 11, 14, },
  { 1, 2, 4, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 9, 10, 11, 14, },
  { 3, 4, 9, 10, 11, 14, },
  { 0, 3, 4, 9, 10, 11, 14, },
  { 1, 3, 4, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 9, 10, 11, 14, },
  { 2, 3, 4, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 14, },
  { 5, 9, 10, 11, 14, },
  { 0, 5, 9, 10, 11, 14, },
  { 1, 5, 9, 10, 11, 14, },
  { 0, 1, 5, 9, 10, 11, 14, },
  { 2, 5, 9, 10, 11, 14, },
  { 0, 2, 5, 9, 10, 11, 14, },
  { 1, 2, 5, 9, 10, 11, 14, },
  { 0, 1, 2, 5, 9, 10, 11, 14, },
  { 3, 5, 9, 10, 11, 14, },
  { 0, 3, 5, 9, 10, 11, 14, },
  { 1, 3, 5, 9, 10, 11, 14, },
  { 0, 1, 3, 5, 9, 10, 11, 14, },
  { 2, 3, 5, 9, 10, 11, 14, },
  { 0, 2, 3, 5, 9, 10, 11, 14, },
  { 1, 2, 3, 5, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 14, },
  { 4, 5, 9, 10, 11, 14, },
  { 0, 4, 5, 9, 10, 11, 14, },
  { 1, 4, 5, 9, 10, 11, 14, },
  { 0, 1, 4, 5, 9, 10, 11, 14, },
  { 2, 4, 5, 9, 10, 11, 14, },
  { 0, 2, 4, 5, 9, 10, 11, 14, },
  { 1, 2, 4, 5, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 14, },
  { 3, 4, 5, 9, 10, 11, 14, },
  { 0, 3, 4, 5, 9, 10, 11, 14, },
  { 1, 3, 4, 5, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 14, },
  { 2, 3, 4, 5, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 14, },
  { 6, 9, 10, 11, 14, },
  { 0, 6, 9, 10, 11, 14, },
  { 1, 6, 9, 10, 11, 14, },
  { 0, 1, 6, 9, 10, 11, 14, },
  { 2, 6, 9, 10, 11, 14, },
  { 0, 2, 6, 9, 10, 11, 14, },
  { 1, 2, 6, 9, 10, 11, 14, },
  { 0, 1, 2, 6, 9, 10, 11, 14, },
  { 3, 6, 9, 10, 11, 14, },
  { 0, 3, 6, 9, 10, 11, 14, },
  { 1, 3, 6, 9, 10, 11, 14, },
  { 0, 1, 3, 6, 9, 10, 11, 14, },
  { 2, 3, 6, 9, 10, 11, 14, },
  { 0, 2, 3, 6, 9, 10, 11, 14, },
  { 1, 2, 3, 6, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 14, },
  { 4, 6, 9, 10, 11, 14, },
  { 0, 4, 6, 9, 10, 11, 14, },
  { 1, 4, 6, 9, 10, 11, 14, },
  { 0, 1, 4, 6, 9, 10, 11, 14, },
  { 2, 4, 6, 9, 10, 11, 14, },
  { 0, 2, 4, 6, 9, 10, 11, 14, },
  { 1, 2, 4, 6, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 14, },
  { 3, 4, 6, 9, 10, 11, 14, },
  { 0, 3, 4, 6, 9, 10, 11, 14, },
  { 1, 3, 4, 6, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 14, },
  { 2, 3, 4, 6, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 14, },
  { 5, 6, 9, 10, 11, 14, },
  { 0, 5, 6, 9, 10, 11, 14, },
  { 1, 5, 6, 9, 10, 11, 14, },
  { 0, 1, 5, 6, 9, 10, 11, 14, },
  { 2, 5, 6, 9, 10, 11, 14, },
  { 0, 2, 5, 6, 9, 10, 11, 14, },
  { 1, 2, 5, 6, 9, 10, 11, 14, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 14, },
  { 3, 5, 6, 9, 10, 11, 14, },
  { 0, 3, 5, 6, 9, 10, 11, 14, },
  { 1, 3, 5, 6, 9, 10, 11, 14, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 14, },
  { 2, 3, 5, 6, 9, 10, 11, 14, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 14, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 14, },
  { 4, 5, 6, 9, 10, 11, 14, },
  { 0, 4, 5, 6, 9, 10, 11, 14, },
  { 1, 4, 5, 6, 9, 10, 11, 14, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 14, },
  { 2, 4, 5, 6, 9, 10, 11, 14, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 14, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 14, },
  { 3, 4, 5, 6, 9, 10, 11, 14, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 14, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 14, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 14, },
  { 7, 9, 10, 11, 14, },
  { 0, 7, 9, 10, 11, 14, },
  { 1, 7, 9, 10, 11, 14, },
  { 0, 1, 7, 9, 10, 11, 14, },
  { 2, 7, 9, 10, 11, 14, },
  { 0, 2, 7, 9, 10, 11, 14, },
  { 1, 2, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 7, 9, 10, 11, 14, },
  { 3, 7, 9, 10, 11, 14, },
  { 0, 3, 7, 9, 10, 11, 14, },
  { 1, 3, 7, 9, 10, 11, 14, },
  { 0, 1, 3, 7, 9, 10, 11, 14, },
  { 2, 3, 7, 9, 10, 11, 14, },
  { 0, 2, 3, 7, 9, 10, 11, 14, },
  { 1, 2, 3, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 14, },
  { 4, 7, 9, 10, 11, 14, },
  { 0, 4, 7, 9, 10, 11, 14, },
  { 1, 4, 7, 9, 10, 11, 14, },
  { 0, 1, 4, 7, 9, 10, 11, 14, },
  { 2, 4, 7, 9, 10, 11, 14, },
  { 0, 2, 4, 7, 9, 10, 11, 14, },
  { 1, 2, 4, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 14, },
  { 3, 4, 7, 9, 10, 11, 14, },
  { 0, 3, 4, 7, 9, 10, 11, 14, },
  { 1, 3, 4, 7, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 14, },
  { 2, 3, 4, 7, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 14, },
  { 5, 7, 9, 10, 11, 14, },
  { 0, 5, 7, 9, 10, 11, 14, },
  { 1, 5, 7, 9, 10, 11, 14, },
  { 0, 1, 5, 7, 9, 10, 11, 14, },
  { 2, 5, 7, 9, 10, 11, 14, },
  { 0, 2, 5, 7, 9, 10, 11, 14, },
  { 1, 2, 5, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 14, },
  { 3, 5, 7, 9, 10, 11, 14, },
  { 0, 3, 5, 7, 9, 10, 11, 14, },
  { 1, 3, 5, 7, 9, 10, 11, 14, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 14, },
  { 2, 3, 5, 7, 9, 10, 11, 14, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 14, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 14, },
  { 4, 5, 7, 9, 10, 11, 14, },
  { 0, 4, 5, 7, 9, 10, 11, 14, },
  { 1, 4, 5, 7, 9, 10, 11, 14, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 14, },
  { 2, 4, 5, 7, 9, 10, 11, 14, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 14, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 14, },
  { 3, 4, 5, 7, 9, 10, 11, 14, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 14, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 14, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 14, },
  { 6, 7, 9, 10, 11, 14, },
  { 0, 6, 7, 9, 10, 11, 14, },
  { 1, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 6, 7, 9, 10, 11, 14, },
  { 2, 6, 7, 9, 10, 11, 14, },
  { 0, 2, 6, 7, 9, 10, 11, 14, },
  { 1, 2, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 14, },
  { 3, 6, 7, 9, 10, 11, 14, },
  { 0, 3, 6, 7, 9, 10, 11, 14, },
  { 1, 3, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 14, },
  { 2, 3, 6, 7, 9, 10, 11, 14, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 14, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 14, },
  { 4, 6, 7, 9, 10, 11, 14, },
  { 0, 4, 6, 7, 9, 10, 11, 14, },
  { 1, 4, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 14, },
  { 2, 4, 6, 7, 9, 10, 11, 14, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 14, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 14, },
  { 3, 4, 6, 7, 9, 10, 11, 14, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 14, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 14, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 14, },
  { 5, 6, 7, 9, 10, 11, 14, },
  { 0, 5, 6, 7, 9, 10, 11, 14, },
  { 1, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 14, },
  { 2, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 14, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 14, },
  { 3, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 14, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 14, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 14, },
  { 4, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 14, },
  { 8, 9, 10, 11, 14, },
  { 0, 8, 9, 10, 11, 14, },
  { 1, 8, 9, 10, 11, 14, },
  { 0, 1, 8, 9, 10, 11, 14, },
  { 2, 8, 9, 10, 11, 14, },
  { 0, 2, 8, 9, 10, 11, 14, },
  { 1, 2, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 8, 9, 10, 11, 14, },
  { 3, 8, 9, 10, 11, 14, },
  { 0, 3, 8, 9, 10, 11, 14, },
  { 1, 3, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 8, 9, 10, 11, 14, },
  { 2, 3, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 14, },
  { 4, 8, 9, 10, 11, 14, },
  { 0, 4, 8, 9, 10, 11, 14, },
  { 1, 4, 8, 9, 10, 11, 14, },
  { 0, 1, 4, 8, 9, 10, 11, 14, },
  { 2, 4, 8, 9, 10, 11, 14, },
  { 0, 2, 4, 8, 9, 10, 11, 14, },
  { 1, 2, 4, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 14, },
  { 3, 4, 8, 9, 10, 11, 14, },
  { 0, 3, 4, 8, 9, 10, 11, 14, },
  { 1, 3, 4, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 14, },
  { 2, 3, 4, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 14, },
  { 5, 8, 9, 10, 11, 14, },
  { 0, 5, 8, 9, 10, 11, 14, },
  { 1, 5, 8, 9, 10, 11, 14, },
  { 0, 1, 5, 8, 9, 10, 11, 14, },
  { 2, 5, 8, 9, 10, 11, 14, },
  { 0, 2, 5, 8, 9, 10, 11, 14, },
  { 1, 2, 5, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 14, },
  { 3, 5, 8, 9, 10, 11, 14, },
  { 0, 3, 5, 8, 9, 10, 11, 14, },
  { 1, 3, 5, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 14, },
  { 2, 3, 5, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 14, },
  { 4, 5, 8, 9, 10, 11, 14, },
  { 0, 4, 5, 8, 9, 10, 11, 14, },
  { 1, 4, 5, 8, 9, 10, 11, 14, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 14, },
  { 2, 4, 5, 8, 9, 10, 11, 14, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 14, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 14, },
  { 3, 4, 5, 8, 9, 10, 11, 14, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 14, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 14, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 14, },
  { 6, 8, 9, 10, 11, 14, },
  { 0, 6, 8, 9, 10, 11, 14, },
  { 1, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 6, 8, 9, 10, 11, 14, },
  { 2, 6, 8, 9, 10, 11, 14, },
  { 0, 2, 6, 8, 9, 10, 11, 14, },
  { 1, 2, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 14, },
  { 3, 6, 8, 9, 10, 11, 14, },
  { 0, 3, 6, 8, 9, 10, 11, 14, },
  { 1, 3, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 14, },
  { 2, 3, 6, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 14, },
  { 4, 6, 8, 9, 10, 11, 14, },
  { 0, 4, 6, 8, 9, 10, 11, 14, },
  { 1, 4, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 14, },
  { 2, 4, 6, 8, 9, 10, 11, 14, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 14, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 14, },
  { 3, 4, 6, 8, 9, 10, 11, 14, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 14, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 14, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 14, },
  { 5, 6, 8, 9, 10, 11, 14, },
  { 0, 5, 6, 8, 9, 10, 11, 14, },
  { 1, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 14, },
  { 2, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 14, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 14, },
  { 3, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 14, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 14, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 14, },
  { 4, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 14, },
  { 7, 8, 9, 10, 11, 14, },
  { 0, 7, 8, 9, 10, 11, 14, },
  { 1, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 7, 8, 9, 10, 11, 14, },
  { 2, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 14, },
  { 3, 7, 8, 9, 10, 11, 14, },
  { 0, 3, 7, 8, 9, 10, 11, 14, },
  { 1, 3, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 14, },
  { 2, 3, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 14, },
  { 4, 7, 8, 9, 10, 11, 14, },
  { 0, 4, 7, 8, 9, 10, 11, 14, },
  { 1, 4, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 14, },
  { 2, 4, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 14, },
  { 3, 4, 7, 8, 9, 10, 11, 14, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 14, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 14, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 14, },
  { 5, 7, 8, 9, 10, 11, 14, },
  { 0, 5, 7, 8, 9, 10, 11, 14, },
  { 1, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 14, },
  { 2, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 14, },
  { 3, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 14, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 14, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 14, },
  { 4, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 14, },
  { 6, 7, 8, 9, 10, 11, 14, },
  { 0, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 14, },
  { 2, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 14, },
  { 3, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 14, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 14, },
  { 4, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 14, },
  { 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, },
  { 12, 14, },
  { 0, 12, 14, },
  { 1, 12, 14, },
  { 0, 1, 12, 14, },
  { 2, 12, 14, },
  { 0, 2, 12, 14, },
  { 1, 2, 12, 14, },
  { 0, 1, 2, 12, 14, },
  { 3, 12, 14, },
  { 0, 3, 12, 14, },
  { 1, 3, 12, 14, },
  { 0, 1, 3, 12, 14, },
  { 2, 3, 12, 14, },
  { 0, 2, 3, 12, 14, },
  { 1, 2, 3, 12, 14, },
  { 0, 1, 2, 3, 12, 14, },
  { 4, 12, 14, },
  { 0, 4, 12, 14, },
  { 1, 4, 12, 14, },
  { 0, 1, 4, 12, 14, },
  { 2, 4, 12, 14, },
  { 0, 2, 4, 12, 14, },
  { 1, 2, 4, 12, 14, },
  { 0, 1, 2, 4, 12, 14, },
  { 3, 4, 12, 14, },
  { 0, 3, 4, 12, 14, },
  { 1, 3, 4, 12, 14, },
  { 0, 1, 3, 4, 12, 14, },
  { 2, 3, 4, 12, 14, },
  { 0, 2, 3, 4, 12, 14, },
  { 1, 2, 3, 4, 12, 14, },
  { 0, 1, 2, 3, 4, 12, 14, },
  { 5, 12, 14, },
  { 0, 5, 12, 14, },
  { 1, 5, 12, 14, },
  { 0, 1, 5, 12, 14, },
  { 2, 5, 12, 14, },
  { 0, 2, 5, 12, 14, },
  { 1, 2, 5, 12, 14, },
  { 0, 1, 2, 5, 12, 14, },
  { 3, 5, 12, 14, },
  { 0, 3, 5, 12, 14, },
  { 1, 3, 5, 12, 14, },
  { 0, 1, 3, 5, 12, 14, },
  { 2, 3, 5, 12, 14, },
  { 0, 2, 3, 5, 12, 14, },
  { 1, 2, 3, 5, 12, 14, },
  { 0, 1, 2, 3, 5, 12, 14, },
  { 4, 5, 12, 14, },
  { 0, 4, 5, 12, 14, },
  { 1, 4, 5, 12, 14, },
  { 0, 1, 4, 5, 12, 14, },
  { 2, 4, 5, 12, 14, },
  { 0, 2, 4, 5, 12, 14, },
  { 1, 2, 4, 5, 12, 14, },
  { 0, 1, 2, 4, 5, 12, 14, },
  { 3, 4, 5, 12, 14, },
  { 0, 3, 4, 5, 12, 14, },
  { 1, 3, 4, 5, 12, 14, },
  { 0, 1, 3, 4, 5, 12, 14, },
  { 2, 3, 4, 5, 12, 14, },
  { 0, 2, 3, 4, 5, 12, 14, },
  { 1, 2, 3, 4, 5, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 12, 14, },
  { 6, 12, 14, },
  { 0, 6, 12, 14, },
  { 1, 6, 12, 14, },
  { 0, 1, 6, 12, 14, },
  { 2, 6, 12, 14, },
  { 0, 2, 6, 12, 14, },
  { 1, 2, 6, 12, 14, },
  { 0, 1, 2, 6, 12, 14, },
  { 3, 6, 12, 14, },
  { 0, 3, 6, 12, 14, },
  { 1, 3, 6, 12, 14, },
  { 0, 1, 3, 6, 12, 14, },
  { 2, 3, 6, 12, 14, },
  { 0, 2, 3, 6, 12, 14, },
  { 1, 2, 3, 6, 12, 14, },
  { 0, 1, 2, 3, 6, 12, 14, },
  { 4, 6, 12, 14, },
  { 0, 4, 6, 12, 14, },
  { 1, 4, 6, 12, 14, },
  { 0, 1, 4, 6, 12, 14, },
  { 2, 4, 6, 12, 14, },
  { 0, 2, 4, 6, 12, 14, },
  { 1, 2, 4, 6, 12, 14, },
  { 0, 1, 2, 4, 6, 12, 14, },
  { 3, 4, 6, 12, 14, },
  { 0, 3, 4, 6, 12, 14, },
  { 1, 3, 4, 6, 12, 14, },
  { 0, 1, 3, 4, 6, 12, 14, },
  { 2, 3, 4, 6, 12, 14, },
  { 0, 2, 3, 4, 6, 12, 14, },
  { 1, 2, 3, 4, 6, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 12, 14, },
  { 5, 6, 12, 14, },
  { 0, 5, 6, 12, 14, },
  { 1, 5, 6, 12, 14, },
  { 0, 1, 5, 6, 12, 14, },
  { 2, 5, 6, 12, 14, },
  { 0, 2, 5, 6, 12, 14, },
  { 1, 2, 5, 6, 12, 14, },
  { 0, 1, 2, 5, 6, 12, 14, },
  { 3, 5, 6, 12, 14, },
  { 0, 3, 5, 6, 12, 14, },
  { 1, 3, 5, 6, 12, 14, },
  { 0, 1, 3, 5, 6, 12, 14, },
  { 2, 3, 5, 6, 12, 14, },
  { 0, 2, 3, 5, 6, 12, 14, },
  { 1, 2, 3, 5, 6, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 12, 14, },
  { 4, 5, 6, 12, 14, },
  { 0, 4, 5, 6, 12, 14, },
  { 1, 4, 5, 6, 12, 14, },
  { 0, 1, 4, 5, 6, 12, 14, },
  { 2, 4, 5, 6, 12, 14, },
  { 0, 2, 4, 5, 6, 12, 14, },
  { 1, 2, 4, 5, 6, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 12, 14, },
  { 3, 4, 5, 6, 12, 14, },
  { 0, 3, 4, 5, 6, 12, 14, },
  { 1, 3, 4, 5, 6, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 12, 14, },
  { 2, 3, 4, 5, 6, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 12, 14, },
  { 7, 12, 14, },
  { 0, 7, 12, 14, },
  { 1, 7, 12, 14, },
  { 0, 1, 7, 12, 14, },
  { 2, 7, 12, 14, },
  { 0, 2, 7, 12, 14, },
  { 1, 2, 7, 12, 14, },
  { 0, 1, 2, 7, 12, 14, },
  { 3, 7, 12, 14, },
  { 0, 3, 7, 12, 14, },
  { 1, 3, 7, 12, 14, },
  { 0, 1, 3, 7, 12, 14, },
  { 2, 3, 7, 12, 14, },
  { 0, 2, 3, 7, 12, 14, },
  { 1, 2, 3, 7, 12, 14, },
  { 0, 1, 2, 3, 7, 12, 14, },
  { 4, 7, 12, 14, },
  { 0, 4, 7, 12, 14, },
  { 1, 4, 7, 12, 14, },
  { 0, 1, 4, 7, 12, 14, },
  { 2, 4, 7, 12, 14, },
  { 0, 2, 4, 7, 12, 14, },
  { 1, 2, 4, 7, 12, 14, },
  { 0, 1, 2, 4, 7, 12, 14, },
  { 3, 4, 7, 12, 14, },
  { 0, 3, 4, 7, 12, 14, },
  { 1, 3, 4, 7, 12, 14, },
  { 0, 1, 3, 4, 7, 12, 14, },
  { 2, 3, 4, 7, 12, 14, },
  { 0, 2, 3, 4, 7, 12, 14, },
  { 1, 2, 3, 4, 7, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 12, 14, },
  { 5, 7, 12, 14, },
  { 0, 5, 7, 12, 14, },
  { 1, 5, 7, 12, 14, },
  { 0, 1, 5, 7, 12, 14, },
  { 2, 5, 7, 12, 14, },
  { 0, 2, 5, 7, 12, 14, },
  { 1, 2, 5, 7, 12, 14, },
  { 0, 1, 2, 5, 7, 12, 14, },
  { 3, 5, 7, 12, 14, },
  { 0, 3, 5, 7, 12, 14, },
  { 1, 3, 5, 7, 12, 14, },
  { 0, 1, 3, 5, 7, 12, 14, },
  { 2, 3, 5, 7, 12, 14, },
  { 0, 2, 3, 5, 7, 12, 14, },
  { 1, 2, 3, 5, 7, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 12, 14, },
  { 4, 5, 7, 12, 14, },
  { 0, 4, 5, 7, 12, 14, },
  { 1, 4, 5, 7, 12, 14, },
  { 0, 1, 4, 5, 7, 12, 14, },
  { 2, 4, 5, 7, 12, 14, },
  { 0, 2, 4, 5, 7, 12, 14, },
  { 1, 2, 4, 5, 7, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 12, 14, },
  { 3, 4, 5, 7, 12, 14, },
  { 0, 3, 4, 5, 7, 12, 14, },
  { 1, 3, 4, 5, 7, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 12, 14, },
  { 2, 3, 4, 5, 7, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 12, 14, },
  { 6, 7, 12, 14, },
  { 0, 6, 7, 12, 14, },
  { 1, 6, 7, 12, 14, },
  { 0, 1, 6, 7, 12, 14, },
  { 2, 6, 7, 12, 14, },
  { 0, 2, 6, 7, 12, 14, },
  { 1, 2, 6, 7, 12, 14, },
  { 0, 1, 2, 6, 7, 12, 14, },
  { 3, 6, 7, 12, 14, },
  { 0, 3, 6, 7, 12, 14, },
  { 1, 3, 6, 7, 12, 14, },
  { 0, 1, 3, 6, 7, 12, 14, },
  { 2, 3, 6, 7, 12, 14, },
  { 0, 2, 3, 6, 7, 12, 14, },
  { 1, 2, 3, 6, 7, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 12, 14, },
  { 4, 6, 7, 12, 14, },
  { 0, 4, 6, 7, 12, 14, },
  { 1, 4, 6, 7, 12, 14, },
  { 0, 1, 4, 6, 7, 12, 14, },
  { 2, 4, 6, 7, 12, 14, },
  { 0, 2, 4, 6, 7, 12, 14, },
  { 1, 2, 4, 6, 7, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 12, 14, },
  { 3, 4, 6, 7, 12, 14, },
  { 0, 3, 4, 6, 7, 12, 14, },
  { 1, 3, 4, 6, 7, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 12, 14, },
  { 2, 3, 4, 6, 7, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 12, 14, },
  { 5, 6, 7, 12, 14, },
  { 0, 5, 6, 7, 12, 14, },
  { 1, 5, 6, 7, 12, 14, },
  { 0, 1, 5, 6, 7, 12, 14, },
  { 2, 5, 6, 7, 12, 14, },
  { 0, 2, 5, 6, 7, 12, 14, },
  { 1, 2, 5, 6, 7, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 12, 14, },
  { 3, 5, 6, 7, 12, 14, },
  { 0, 3, 5, 6, 7, 12, 14, },
  { 1, 3, 5, 6, 7, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 12, 14, },
  { 2, 3, 5, 6, 7, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 12, 14, },
  { 4, 5, 6, 7, 12, 14, },
  { 0, 4, 5, 6, 7, 12, 14, },
  { 1, 4, 5, 6, 7, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 12, 14, },
  { 2, 4, 5, 6, 7, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 12, 14, },
  { 3, 4, 5, 6, 7, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 12, 14, },
  { 8, 12, 14, },
  { 0, 8, 12, 14, },
  { 1, 8, 12, 14, },
  { 0, 1, 8, 12, 14, },
  { 2, 8, 12, 14, },
  { 0, 2, 8, 12, 14, },
  { 1, 2, 8, 12, 14, },
  { 0, 1, 2, 8, 12, 14, },
  { 3, 8, 12, 14, },
  { 0, 3, 8, 12, 14, },
  { 1, 3, 8, 12, 14, },
  { 0, 1, 3, 8, 12, 14, },
  { 2, 3, 8, 12, 14, },
  { 0, 2, 3, 8, 12, 14, },
  { 1, 2, 3, 8, 12, 14, },
  { 0, 1, 2, 3, 8, 12, 14, },
  { 4, 8, 12, 14, },
  { 0, 4, 8, 12, 14, },
  { 1, 4, 8, 12, 14, },
  { 0, 1, 4, 8, 12, 14, },
  { 2, 4, 8, 12, 14, },
  { 0, 2, 4, 8, 12, 14, },
  { 1, 2, 4, 8, 12, 14, },
  { 0, 1, 2, 4, 8, 12, 14, },
  { 3, 4, 8, 12, 14, },
  { 0, 3, 4, 8, 12, 14, },
  { 1, 3, 4, 8, 12, 14, },
  { 0, 1, 3, 4, 8, 12, 14, },
  { 2, 3, 4, 8, 12, 14, },
  { 0, 2, 3, 4, 8, 12, 14, },
  { 1, 2, 3, 4, 8, 12, 14, },
  { 0, 1, 2, 3, 4, 8, 12, 14, },
  { 5, 8, 12, 14, },
  { 0, 5, 8, 12, 14, },
  { 1, 5, 8, 12, 14, },
  { 0, 1, 5, 8, 12, 14, },
  { 2, 5, 8, 12, 14, },
  { 0, 2, 5, 8, 12, 14, },
  { 1, 2, 5, 8, 12, 14, },
  { 0, 1, 2, 5, 8, 12, 14, },
  { 3, 5, 8, 12, 14, },
  { 0, 3, 5, 8, 12, 14, },
  { 1, 3, 5, 8, 12, 14, },
  { 0, 1, 3, 5, 8, 12, 14, },
  { 2, 3, 5, 8, 12, 14, },
  { 0, 2, 3, 5, 8, 12, 14, },
  { 1, 2, 3, 5, 8, 12, 14, },
  { 0, 1, 2, 3, 5, 8, 12, 14, },
  { 4, 5, 8, 12, 14, },
  { 0, 4, 5, 8, 12, 14, },
  { 1, 4, 5, 8, 12, 14, },
  { 0, 1, 4, 5, 8, 12, 14, },
  { 2, 4, 5, 8, 12, 14, },
  { 0, 2, 4, 5, 8, 12, 14, },
  { 1, 2, 4, 5, 8, 12, 14, },
  { 0, 1, 2, 4, 5, 8, 12, 14, },
  { 3, 4, 5, 8, 12, 14, },
  { 0, 3, 4, 5, 8, 12, 14, },
  { 1, 3, 4, 5, 8, 12, 14, },
  { 0, 1, 3, 4, 5, 8, 12, 14, },
  { 2, 3, 4, 5, 8, 12, 14, },
  { 0, 2, 3, 4, 5, 8, 12, 14, },
  { 1, 2, 3, 4, 5, 8, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 12, 14, },
  { 6, 8, 12, 14, },
  { 0, 6, 8, 12, 14, },
  { 1, 6, 8, 12, 14, },
  { 0, 1, 6, 8, 12, 14, },
  { 2, 6, 8, 12, 14, },
  { 0, 2, 6, 8, 12, 14, },
  { 1, 2, 6, 8, 12, 14, },
  { 0, 1, 2, 6, 8, 12, 14, },
  { 3, 6, 8, 12, 14, },
  { 0, 3, 6, 8, 12, 14, },
  { 1, 3, 6, 8, 12, 14, },
  { 0, 1, 3, 6, 8, 12, 14, },
  { 2, 3, 6, 8, 12, 14, },
  { 0, 2, 3, 6, 8, 12, 14, },
  { 1, 2, 3, 6, 8, 12, 14, },
  { 0, 1, 2, 3, 6, 8, 12, 14, },
  { 4, 6, 8, 12, 14, },
  { 0, 4, 6, 8, 12, 14, },
  { 1, 4, 6, 8, 12, 14, },
  { 0, 1, 4, 6, 8, 12, 14, },
  { 2, 4, 6, 8, 12, 14, },
  { 0, 2, 4, 6, 8, 12, 14, },
  { 1, 2, 4, 6, 8, 12, 14, },
  { 0, 1, 2, 4, 6, 8, 12, 14, },
  { 3, 4, 6, 8, 12, 14, },
  { 0, 3, 4, 6, 8, 12, 14, },
  { 1, 3, 4, 6, 8, 12, 14, },
  { 0, 1, 3, 4, 6, 8, 12, 14, },
  { 2, 3, 4, 6, 8, 12, 14, },
  { 0, 2, 3, 4, 6, 8, 12, 14, },
  { 1, 2, 3, 4, 6, 8, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 12, 14, },
  { 5, 6, 8, 12, 14, },
  { 0, 5, 6, 8, 12, 14, },
  { 1, 5, 6, 8, 12, 14, },
  { 0, 1, 5, 6, 8, 12, 14, },
  { 2, 5, 6, 8, 12, 14, },
  { 0, 2, 5, 6, 8, 12, 14, },
  { 1, 2, 5, 6, 8, 12, 14, },
  { 0, 1, 2, 5, 6, 8, 12, 14, },
  { 3, 5, 6, 8, 12, 14, },
  { 0, 3, 5, 6, 8, 12, 14, },
  { 1, 3, 5, 6, 8, 12, 14, },
  { 0, 1, 3, 5, 6, 8, 12, 14, },
  { 2, 3, 5, 6, 8, 12, 14, },
  { 0, 2, 3, 5, 6, 8, 12, 14, },
  { 1, 2, 3, 5, 6, 8, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 12, 14, },
  { 4, 5, 6, 8, 12, 14, },
  { 0, 4, 5, 6, 8, 12, 14, },
  { 1, 4, 5, 6, 8, 12, 14, },
  { 0, 1, 4, 5, 6, 8, 12, 14, },
  { 2, 4, 5, 6, 8, 12, 14, },
  { 0, 2, 4, 5, 6, 8, 12, 14, },
  { 1, 2, 4, 5, 6, 8, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 12, 14, },
  { 3, 4, 5, 6, 8, 12, 14, },
  { 0, 3, 4, 5, 6, 8, 12, 14, },
  { 1, 3, 4, 5, 6, 8, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 12, 14, },
  { 2, 3, 4, 5, 6, 8, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 12, 14, },
  { 7, 8, 12, 14, },
  { 0, 7, 8, 12, 14, },
  { 1, 7, 8, 12, 14, },
  { 0, 1, 7, 8, 12, 14, },
  { 2, 7, 8, 12, 14, },
  { 0, 2, 7, 8, 12, 14, },
  { 1, 2, 7, 8, 12, 14, },
  { 0, 1, 2, 7, 8, 12, 14, },
  { 3, 7, 8, 12, 14, },
  { 0, 3, 7, 8, 12, 14, },
  { 1, 3, 7, 8, 12, 14, },
  { 0, 1, 3, 7, 8, 12, 14, },
  { 2, 3, 7, 8, 12, 14, },
  { 0, 2, 3, 7, 8, 12, 14, },
  { 1, 2, 3, 7, 8, 12, 14, },
  { 0, 1, 2, 3, 7, 8, 12, 14, },
  { 4, 7, 8, 12, 14, },
  { 0, 4, 7, 8, 12, 14, },
  { 1, 4, 7, 8, 12, 14, },
  { 0, 1, 4, 7, 8, 12, 14, },
  { 2, 4, 7, 8, 12, 14, },
  { 0, 2, 4, 7, 8, 12, 14, },
  { 1, 2, 4, 7, 8, 12, 14, },
  { 0, 1, 2, 4, 7, 8, 12, 14, },
  { 3, 4, 7, 8, 12, 14, },
  { 0, 3, 4, 7, 8, 12, 14, },
  { 1, 3, 4, 7, 8, 12, 14, },
  { 0, 1, 3, 4, 7, 8, 12, 14, },
  { 2, 3, 4, 7, 8, 12, 14, },
  { 0, 2, 3, 4, 7, 8, 12, 14, },
  { 1, 2, 3, 4, 7, 8, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 12, 14, },
  { 5, 7, 8, 12, 14, },
  { 0, 5, 7, 8, 12, 14, },
  { 1, 5, 7, 8, 12, 14, },
  { 0, 1, 5, 7, 8, 12, 14, },
  { 2, 5, 7, 8, 12, 14, },
  { 0, 2, 5, 7, 8, 12, 14, },
  { 1, 2, 5, 7, 8, 12, 14, },
  { 0, 1, 2, 5, 7, 8, 12, 14, },
  { 3, 5, 7, 8, 12, 14, },
  { 0, 3, 5, 7, 8, 12, 14, },
  { 1, 3, 5, 7, 8, 12, 14, },
  { 0, 1, 3, 5, 7, 8, 12, 14, },
  { 2, 3, 5, 7, 8, 12, 14, },
  { 0, 2, 3, 5, 7, 8, 12, 14, },
  { 1, 2, 3, 5, 7, 8, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 12, 14, },
  { 4, 5, 7, 8, 12, 14, },
  { 0, 4, 5, 7, 8, 12, 14, },
  { 1, 4, 5, 7, 8, 12, 14, },
  { 0, 1, 4, 5, 7, 8, 12, 14, },
  { 2, 4, 5, 7, 8, 12, 14, },
  { 0, 2, 4, 5, 7, 8, 12, 14, },
  { 1, 2, 4, 5, 7, 8, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 12, 14, },
  { 3, 4, 5, 7, 8, 12, 14, },
  { 0, 3, 4, 5, 7, 8, 12, 14, },
  { 1, 3, 4, 5, 7, 8, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 12, 14, },
  { 2, 3, 4, 5, 7, 8, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 12, 14, },
  { 6, 7, 8, 12, 14, },
  { 0, 6, 7, 8, 12, 14, },
  { 1, 6, 7, 8, 12, 14, },
  { 0, 1, 6, 7, 8, 12, 14, },
  { 2, 6, 7, 8, 12, 14, },
  { 0, 2, 6, 7, 8, 12, 14, },
  { 1, 2, 6, 7, 8, 12, 14, },
  { 0, 1, 2, 6, 7, 8, 12, 14, },
  { 3, 6, 7, 8, 12, 14, },
  { 0, 3, 6, 7, 8, 12, 14, },
  { 1, 3, 6, 7, 8, 12, 14, },
  { 0, 1, 3, 6, 7, 8, 12, 14, },
  { 2, 3, 6, 7, 8, 12, 14, },
  { 0, 2, 3, 6, 7, 8, 12, 14, },
  { 1, 2, 3, 6, 7, 8, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 12, 14, },
  { 4, 6, 7, 8, 12, 14, },
  { 0, 4, 6, 7, 8, 12, 14, },
  { 1, 4, 6, 7, 8, 12, 14, },
  { 0, 1, 4, 6, 7, 8, 12, 14, },
  { 2, 4, 6, 7, 8, 12, 14, },
  { 0, 2, 4, 6, 7, 8, 12, 14, },
  { 1, 2, 4, 6, 7, 8, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 12, 14, },
  { 3, 4, 6, 7, 8, 12, 14, },
  { 0, 3, 4, 6, 7, 8, 12, 14, },
  { 1, 3, 4, 6, 7, 8, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 12, 14, },
  { 2, 3, 4, 6, 7, 8, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 12, 14, },
  { 5, 6, 7, 8, 12, 14, },
  { 0, 5, 6, 7, 8, 12, 14, },
  { 1, 5, 6, 7, 8, 12, 14, },
  { 0, 1, 5, 6, 7, 8, 12, 14, },
  { 2, 5, 6, 7, 8, 12, 14, },
  { 0, 2, 5, 6, 7, 8, 12, 14, },
  { 1, 2, 5, 6, 7, 8, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 12, 14, },
  { 3, 5, 6, 7, 8, 12, 14, },
  { 0, 3, 5, 6, 7, 8, 12, 14, },
  { 1, 3, 5, 6, 7, 8, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 12, 14, },
  { 2, 3, 5, 6, 7, 8, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 12, 14, },
  { 4, 5, 6, 7, 8, 12, 14, },
  { 0, 4, 5, 6, 7, 8, 12, 14, },
  { 1, 4, 5, 6, 7, 8, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 12, 14, },
  { 2, 4, 5, 6, 7, 8, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 12, 14, },
  { 3, 4, 5, 6, 7, 8, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 12, 14, },
  { 9, 12, 14, },
  { 0, 9, 12, 14, },
  { 1, 9, 12, 14, },
  { 0, 1, 9, 12, 14, },
  { 2, 9, 12, 14, },
  { 0, 2, 9, 12, 14, },
  { 1, 2, 9, 12, 14, },
  { 0, 1, 2, 9, 12, 14, },
  { 3, 9, 12, 14, },
  { 0, 3, 9, 12, 14, },
  { 1, 3, 9, 12, 14, },
  { 0, 1, 3, 9, 12, 14, },
  { 2, 3, 9, 12, 14, },
  { 0, 2, 3, 9, 12, 14, },
  { 1, 2, 3, 9, 12, 14, },
  { 0, 1, 2, 3, 9, 12, 14, },
  { 4, 9, 12, 14, },
  { 0, 4, 9, 12, 14, },
  { 1, 4, 9, 12, 14, },
  { 0, 1, 4, 9, 12, 14, },
  { 2, 4, 9, 12, 14, },
  { 0, 2, 4, 9, 12, 14, },
  { 1, 2, 4, 9, 12, 14, },
  { 0, 1, 2, 4, 9, 12, 14, },
  { 3, 4, 9, 12, 14, },
  { 0, 3, 4, 9, 12, 14, },
  { 1, 3, 4, 9, 12, 14, },
  { 0, 1, 3, 4, 9, 12, 14, },
  { 2, 3, 4, 9, 12, 14, },
  { 0, 2, 3, 4, 9, 12, 14, },
  { 1, 2, 3, 4, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 9, 12, 14, },
  { 5, 9, 12, 14, },
  { 0, 5, 9, 12, 14, },
  { 1, 5, 9, 12, 14, },
  { 0, 1, 5, 9, 12, 14, },
  { 2, 5, 9, 12, 14, },
  { 0, 2, 5, 9, 12, 14, },
  { 1, 2, 5, 9, 12, 14, },
  { 0, 1, 2, 5, 9, 12, 14, },
  { 3, 5, 9, 12, 14, },
  { 0, 3, 5, 9, 12, 14, },
  { 1, 3, 5, 9, 12, 14, },
  { 0, 1, 3, 5, 9, 12, 14, },
  { 2, 3, 5, 9, 12, 14, },
  { 0, 2, 3, 5, 9, 12, 14, },
  { 1, 2, 3, 5, 9, 12, 14, },
  { 0, 1, 2, 3, 5, 9, 12, 14, },
  { 4, 5, 9, 12, 14, },
  { 0, 4, 5, 9, 12, 14, },
  { 1, 4, 5, 9, 12, 14, },
  { 0, 1, 4, 5, 9, 12, 14, },
  { 2, 4, 5, 9, 12, 14, },
  { 0, 2, 4, 5, 9, 12, 14, },
  { 1, 2, 4, 5, 9, 12, 14, },
  { 0, 1, 2, 4, 5, 9, 12, 14, },
  { 3, 4, 5, 9, 12, 14, },
  { 0, 3, 4, 5, 9, 12, 14, },
  { 1, 3, 4, 5, 9, 12, 14, },
  { 0, 1, 3, 4, 5, 9, 12, 14, },
  { 2, 3, 4, 5, 9, 12, 14, },
  { 0, 2, 3, 4, 5, 9, 12, 14, },
  { 1, 2, 3, 4, 5, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 12, 14, },
  { 6, 9, 12, 14, },
  { 0, 6, 9, 12, 14, },
  { 1, 6, 9, 12, 14, },
  { 0, 1, 6, 9, 12, 14, },
  { 2, 6, 9, 12, 14, },
  { 0, 2, 6, 9, 12, 14, },
  { 1, 2, 6, 9, 12, 14, },
  { 0, 1, 2, 6, 9, 12, 14, },
  { 3, 6, 9, 12, 14, },
  { 0, 3, 6, 9, 12, 14, },
  { 1, 3, 6, 9, 12, 14, },
  { 0, 1, 3, 6, 9, 12, 14, },
  { 2, 3, 6, 9, 12, 14, },
  { 0, 2, 3, 6, 9, 12, 14, },
  { 1, 2, 3, 6, 9, 12, 14, },
  { 0, 1, 2, 3, 6, 9, 12, 14, },
  { 4, 6, 9, 12, 14, },
  { 0, 4, 6, 9, 12, 14, },
  { 1, 4, 6, 9, 12, 14, },
  { 0, 1, 4, 6, 9, 12, 14, },
  { 2, 4, 6, 9, 12, 14, },
  { 0, 2, 4, 6, 9, 12, 14, },
  { 1, 2, 4, 6, 9, 12, 14, },
  { 0, 1, 2, 4, 6, 9, 12, 14, },
  { 3, 4, 6, 9, 12, 14, },
  { 0, 3, 4, 6, 9, 12, 14, },
  { 1, 3, 4, 6, 9, 12, 14, },
  { 0, 1, 3, 4, 6, 9, 12, 14, },
  { 2, 3, 4, 6, 9, 12, 14, },
  { 0, 2, 3, 4, 6, 9, 12, 14, },
  { 1, 2, 3, 4, 6, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 12, 14, },
  { 5, 6, 9, 12, 14, },
  { 0, 5, 6, 9, 12, 14, },
  { 1, 5, 6, 9, 12, 14, },
  { 0, 1, 5, 6, 9, 12, 14, },
  { 2, 5, 6, 9, 12, 14, },
  { 0, 2, 5, 6, 9, 12, 14, },
  { 1, 2, 5, 6, 9, 12, 14, },
  { 0, 1, 2, 5, 6, 9, 12, 14, },
  { 3, 5, 6, 9, 12, 14, },
  { 0, 3, 5, 6, 9, 12, 14, },
  { 1, 3, 5, 6, 9, 12, 14, },
  { 0, 1, 3, 5, 6, 9, 12, 14, },
  { 2, 3, 5, 6, 9, 12, 14, },
  { 0, 2, 3, 5, 6, 9, 12, 14, },
  { 1, 2, 3, 5, 6, 9, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 12, 14, },
  { 4, 5, 6, 9, 12, 14, },
  { 0, 4, 5, 6, 9, 12, 14, },
  { 1, 4, 5, 6, 9, 12, 14, },
  { 0, 1, 4, 5, 6, 9, 12, 14, },
  { 2, 4, 5, 6, 9, 12, 14, },
  { 0, 2, 4, 5, 6, 9, 12, 14, },
  { 1, 2, 4, 5, 6, 9, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 12, 14, },
  { 3, 4, 5, 6, 9, 12, 14, },
  { 0, 3, 4, 5, 6, 9, 12, 14, },
  { 1, 3, 4, 5, 6, 9, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 12, 14, },
  { 2, 3, 4, 5, 6, 9, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 12, 14, },
  { 7, 9, 12, 14, },
  { 0, 7, 9, 12, 14, },
  { 1, 7, 9, 12, 14, },
  { 0, 1, 7, 9, 12, 14, },
  { 2, 7, 9, 12, 14, },
  { 0, 2, 7, 9, 12, 14, },
  { 1, 2, 7, 9, 12, 14, },
  { 0, 1, 2, 7, 9, 12, 14, },
  { 3, 7, 9, 12, 14, },
  { 0, 3, 7, 9, 12, 14, },
  { 1, 3, 7, 9, 12, 14, },
  { 0, 1, 3, 7, 9, 12, 14, },
  { 2, 3, 7, 9, 12, 14, },
  { 0, 2, 3, 7, 9, 12, 14, },
  { 1, 2, 3, 7, 9, 12, 14, },
  { 0, 1, 2, 3, 7, 9, 12, 14, },
  { 4, 7, 9, 12, 14, },
  { 0, 4, 7, 9, 12, 14, },
  { 1, 4, 7, 9, 12, 14, },
  { 0, 1, 4, 7, 9, 12, 14, },
  { 2, 4, 7, 9, 12, 14, },
  { 0, 2, 4, 7, 9, 12, 14, },
  { 1, 2, 4, 7, 9, 12, 14, },
  { 0, 1, 2, 4, 7, 9, 12, 14, },
  { 3, 4, 7, 9, 12, 14, },
  { 0, 3, 4, 7, 9, 12, 14, },
  { 1, 3, 4, 7, 9, 12, 14, },
  { 0, 1, 3, 4, 7, 9, 12, 14, },
  { 2, 3, 4, 7, 9, 12, 14, },
  { 0, 2, 3, 4, 7, 9, 12, 14, },
  { 1, 2, 3, 4, 7, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 12, 14, },
  { 5, 7, 9, 12, 14, },
  { 0, 5, 7, 9, 12, 14, },
  { 1, 5, 7, 9, 12, 14, },
  { 0, 1, 5, 7, 9, 12, 14, },
  { 2, 5, 7, 9, 12, 14, },
  { 0, 2, 5, 7, 9, 12, 14, },
  { 1, 2, 5, 7, 9, 12, 14, },
  { 0, 1, 2, 5, 7, 9, 12, 14, },
  { 3, 5, 7, 9, 12, 14, },
  { 0, 3, 5, 7, 9, 12, 14, },
  { 1, 3, 5, 7, 9, 12, 14, },
  { 0, 1, 3, 5, 7, 9, 12, 14, },
  { 2, 3, 5, 7, 9, 12, 14, },
  { 0, 2, 3, 5, 7, 9, 12, 14, },
  { 1, 2, 3, 5, 7, 9, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 12, 14, },
  { 4, 5, 7, 9, 12, 14, },
  { 0, 4, 5, 7, 9, 12, 14, },
  { 1, 4, 5, 7, 9, 12, 14, },
  { 0, 1, 4, 5, 7, 9, 12, 14, },
  { 2, 4, 5, 7, 9, 12, 14, },
  { 0, 2, 4, 5, 7, 9, 12, 14, },
  { 1, 2, 4, 5, 7, 9, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 12, 14, },
  { 3, 4, 5, 7, 9, 12, 14, },
  { 0, 3, 4, 5, 7, 9, 12, 14, },
  { 1, 3, 4, 5, 7, 9, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 12, 14, },
  { 2, 3, 4, 5, 7, 9, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 12, 14, },
  { 6, 7, 9, 12, 14, },
  { 0, 6, 7, 9, 12, 14, },
  { 1, 6, 7, 9, 12, 14, },
  { 0, 1, 6, 7, 9, 12, 14, },
  { 2, 6, 7, 9, 12, 14, },
  { 0, 2, 6, 7, 9, 12, 14, },
  { 1, 2, 6, 7, 9, 12, 14, },
  { 0, 1, 2, 6, 7, 9, 12, 14, },
  { 3, 6, 7, 9, 12, 14, },
  { 0, 3, 6, 7, 9, 12, 14, },
  { 1, 3, 6, 7, 9, 12, 14, },
  { 0, 1, 3, 6, 7, 9, 12, 14, },
  { 2, 3, 6, 7, 9, 12, 14, },
  { 0, 2, 3, 6, 7, 9, 12, 14, },
  { 1, 2, 3, 6, 7, 9, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 12, 14, },
  { 4, 6, 7, 9, 12, 14, },
  { 0, 4, 6, 7, 9, 12, 14, },
  { 1, 4, 6, 7, 9, 12, 14, },
  { 0, 1, 4, 6, 7, 9, 12, 14, },
  { 2, 4, 6, 7, 9, 12, 14, },
  { 0, 2, 4, 6, 7, 9, 12, 14, },
  { 1, 2, 4, 6, 7, 9, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 12, 14, },
  { 3, 4, 6, 7, 9, 12, 14, },
  { 0, 3, 4, 6, 7, 9, 12, 14, },
  { 1, 3, 4, 6, 7, 9, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 12, 14, },
  { 2, 3, 4, 6, 7, 9, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 12, 14, },
  { 5, 6, 7, 9, 12, 14, },
  { 0, 5, 6, 7, 9, 12, 14, },
  { 1, 5, 6, 7, 9, 12, 14, },
  { 0, 1, 5, 6, 7, 9, 12, 14, },
  { 2, 5, 6, 7, 9, 12, 14, },
  { 0, 2, 5, 6, 7, 9, 12, 14, },
  { 1, 2, 5, 6, 7, 9, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 12, 14, },
  { 3, 5, 6, 7, 9, 12, 14, },
  { 0, 3, 5, 6, 7, 9, 12, 14, },
  { 1, 3, 5, 6, 7, 9, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 12, 14, },
  { 2, 3, 5, 6, 7, 9, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 12, 14, },
  { 4, 5, 6, 7, 9, 12, 14, },
  { 0, 4, 5, 6, 7, 9, 12, 14, },
  { 1, 4, 5, 6, 7, 9, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 12, 14, },
  { 2, 4, 5, 6, 7, 9, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 12, 14, },
  { 3, 4, 5, 6, 7, 9, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 12, 14, },
  { 8, 9, 12, 14, },
  { 0, 8, 9, 12, 14, },
  { 1, 8, 9, 12, 14, },
  { 0, 1, 8, 9, 12, 14, },
  { 2, 8, 9, 12, 14, },
  { 0, 2, 8, 9, 12, 14, },
  { 1, 2, 8, 9, 12, 14, },
  { 0, 1, 2, 8, 9, 12, 14, },
  { 3, 8, 9, 12, 14, },
  { 0, 3, 8, 9, 12, 14, },
  { 1, 3, 8, 9, 12, 14, },
  { 0, 1, 3, 8, 9, 12, 14, },
  { 2, 3, 8, 9, 12, 14, },
  { 0, 2, 3, 8, 9, 12, 14, },
  { 1, 2, 3, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 8, 9, 12, 14, },
  { 4, 8, 9, 12, 14, },
  { 0, 4, 8, 9, 12, 14, },
  { 1, 4, 8, 9, 12, 14, },
  { 0, 1, 4, 8, 9, 12, 14, },
  { 2, 4, 8, 9, 12, 14, },
  { 0, 2, 4, 8, 9, 12, 14, },
  { 1, 2, 4, 8, 9, 12, 14, },
  { 0, 1, 2, 4, 8, 9, 12, 14, },
  { 3, 4, 8, 9, 12, 14, },
  { 0, 3, 4, 8, 9, 12, 14, },
  { 1, 3, 4, 8, 9, 12, 14, },
  { 0, 1, 3, 4, 8, 9, 12, 14, },
  { 2, 3, 4, 8, 9, 12, 14, },
  { 0, 2, 3, 4, 8, 9, 12, 14, },
  { 1, 2, 3, 4, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 12, 14, },
  { 5, 8, 9, 12, 14, },
  { 0, 5, 8, 9, 12, 14, },
  { 1, 5, 8, 9, 12, 14, },
  { 0, 1, 5, 8, 9, 12, 14, },
  { 2, 5, 8, 9, 12, 14, },
  { 0, 2, 5, 8, 9, 12, 14, },
  { 1, 2, 5, 8, 9, 12, 14, },
  { 0, 1, 2, 5, 8, 9, 12, 14, },
  { 3, 5, 8, 9, 12, 14, },
  { 0, 3, 5, 8, 9, 12, 14, },
  { 1, 3, 5, 8, 9, 12, 14, },
  { 0, 1, 3, 5, 8, 9, 12, 14, },
  { 2, 3, 5, 8, 9, 12, 14, },
  { 0, 2, 3, 5, 8, 9, 12, 14, },
  { 1, 2, 3, 5, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 12, 14, },
  { 4, 5, 8, 9, 12, 14, },
  { 0, 4, 5, 8, 9, 12, 14, },
  { 1, 4, 5, 8, 9, 12, 14, },
  { 0, 1, 4, 5, 8, 9, 12, 14, },
  { 2, 4, 5, 8, 9, 12, 14, },
  { 0, 2, 4, 5, 8, 9, 12, 14, },
  { 1, 2, 4, 5, 8, 9, 12, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 12, 14, },
  { 3, 4, 5, 8, 9, 12, 14, },
  { 0, 3, 4, 5, 8, 9, 12, 14, },
  { 1, 3, 4, 5, 8, 9, 12, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 12, 14, },
  { 2, 3, 4, 5, 8, 9, 12, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 12, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 12, 14, },
  { 6, 8, 9, 12, 14, },
  { 0, 6, 8, 9, 12, 14, },
  { 1, 6, 8, 9, 12, 14, },
  { 0, 1, 6, 8, 9, 12, 14, },
  { 2, 6, 8, 9, 12, 14, },
  { 0, 2, 6, 8, 9, 12, 14, },
  { 1, 2, 6, 8, 9, 12, 14, },
  { 0, 1, 2, 6, 8, 9, 12, 14, },
  { 3, 6, 8, 9, 12, 14, },
  { 0, 3, 6, 8, 9, 12, 14, },
  { 1, 3, 6, 8, 9, 12, 14, },
  { 0, 1, 3, 6, 8, 9, 12, 14, },
  { 2, 3, 6, 8, 9, 12, 14, },
  { 0, 2, 3, 6, 8, 9, 12, 14, },
  { 1, 2, 3, 6, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 12, 14, },
  { 4, 6, 8, 9, 12, 14, },
  { 0, 4, 6, 8, 9, 12, 14, },
  { 1, 4, 6, 8, 9, 12, 14, },
  { 0, 1, 4, 6, 8, 9, 12, 14, },
  { 2, 4, 6, 8, 9, 12, 14, },
  { 0, 2, 4, 6, 8, 9, 12, 14, },
  { 1, 2, 4, 6, 8, 9, 12, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 12, 14, },
  { 3, 4, 6, 8, 9, 12, 14, },
  { 0, 3, 4, 6, 8, 9, 12, 14, },
  { 1, 3, 4, 6, 8, 9, 12, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 12, 14, },
  { 2, 3, 4, 6, 8, 9, 12, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 12, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 12, 14, },
  { 5, 6, 8, 9, 12, 14, },
  { 0, 5, 6, 8, 9, 12, 14, },
  { 1, 5, 6, 8, 9, 12, 14, },
  { 0, 1, 5, 6, 8, 9, 12, 14, },
  { 2, 5, 6, 8, 9, 12, 14, },
  { 0, 2, 5, 6, 8, 9, 12, 14, },
  { 1, 2, 5, 6, 8, 9, 12, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 12, 14, },
  { 3, 5, 6, 8, 9, 12, 14, },
  { 0, 3, 5, 6, 8, 9, 12, 14, },
  { 1, 3, 5, 6, 8, 9, 12, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 12, 14, },
  { 2, 3, 5, 6, 8, 9, 12, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 12, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 12, 14, },
  { 4, 5, 6, 8, 9, 12, 14, },
  { 0, 4, 5, 6, 8, 9, 12, 14, },
  { 1, 4, 5, 6, 8, 9, 12, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 12, 14, },
  { 2, 4, 5, 6, 8, 9, 12, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 12, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 12, 14, },
  { 3, 4, 5, 6, 8, 9, 12, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 12, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 12, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 12, 14, },
  { 7, 8, 9, 12, 14, },
  { 0, 7, 8, 9, 12, 14, },
  { 1, 7, 8, 9, 12, 14, },
  { 0, 1, 7, 8, 9, 12, 14, },
  { 2, 7, 8, 9, 12, 14, },
  { 0, 2, 7, 8, 9, 12, 14, },
  { 1, 2, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 7, 8, 9, 12, 14, },
  { 3, 7, 8, 9, 12, 14, },
  { 0, 3, 7, 8, 9, 12, 14, },
  { 1, 3, 7, 8, 9, 12, 14, },
  { 0, 1, 3, 7, 8, 9, 12, 14, },
  { 2, 3, 7, 8, 9, 12, 14, },
  { 0, 2, 3, 7, 8, 9, 12, 14, },
  { 1, 2, 3, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 12, 14, },
  { 4, 7, 8, 9, 12, 14, },
  { 0, 4, 7, 8, 9, 12, 14, },
  { 1, 4, 7, 8, 9, 12, 14, },
  { 0, 1, 4, 7, 8, 9, 12, 14, },
  { 2, 4, 7, 8, 9, 12, 14, },
  { 0, 2, 4, 7, 8, 9, 12, 14, },
  { 1, 2, 4, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 12, 14, },
  { 3, 4, 7, 8, 9, 12, 14, },
  { 0, 3, 4, 7, 8, 9, 12, 14, },
  { 1, 3, 4, 7, 8, 9, 12, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 12, 14, },
  { 2, 3, 4, 7, 8, 9, 12, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 12, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 12, 14, },
  { 5, 7, 8, 9, 12, 14, },
  { 0, 5, 7, 8, 9, 12, 14, },
  { 1, 5, 7, 8, 9, 12, 14, },
  { 0, 1, 5, 7, 8, 9, 12, 14, },
  { 2, 5, 7, 8, 9, 12, 14, },
  { 0, 2, 5, 7, 8, 9, 12, 14, },
  { 1, 2, 5, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 12, 14, },
  { 3, 5, 7, 8, 9, 12, 14, },
  { 0, 3, 5, 7, 8, 9, 12, 14, },
  { 1, 3, 5, 7, 8, 9, 12, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 12, 14, },
  { 2, 3, 5, 7, 8, 9, 12, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 12, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 12, 14, },
  { 4, 5, 7, 8, 9, 12, 14, },
  { 0, 4, 5, 7, 8, 9, 12, 14, },
  { 1, 4, 5, 7, 8, 9, 12, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 12, 14, },
  { 2, 4, 5, 7, 8, 9, 12, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 12, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 12, 14, },
  { 3, 4, 5, 7, 8, 9, 12, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 12, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 12, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 12, 14, },
  { 6, 7, 8, 9, 12, 14, },
  { 0, 6, 7, 8, 9, 12, 14, },
  { 1, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 6, 7, 8, 9, 12, 14, },
  { 2, 6, 7, 8, 9, 12, 14, },
  { 0, 2, 6, 7, 8, 9, 12, 14, },
  { 1, 2, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 12, 14, },
  { 3, 6, 7, 8, 9, 12, 14, },
  { 0, 3, 6, 7, 8, 9, 12, 14, },
  { 1, 3, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 12, 14, },
  { 2, 3, 6, 7, 8, 9, 12, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 12, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 12, 14, },
  { 4, 6, 7, 8, 9, 12, 14, },
  { 0, 4, 6, 7, 8, 9, 12, 14, },
  { 1, 4, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 12, 14, },
  { 2, 4, 6, 7, 8, 9, 12, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 12, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 12, 14, },
  { 3, 4, 6, 7, 8, 9, 12, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 12, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 12, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 12, 14, },
  { 5, 6, 7, 8, 9, 12, 14, },
  { 0, 5, 6, 7, 8, 9, 12, 14, },
  { 1, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 12, 14, },
  { 2, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 12, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 12, 14, },
  { 3, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 12, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 12, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 12, 14, },
  { 4, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 14, },
  { 10, 12, 14, },
  { 0, 10, 12, 14, },
  { 1, 10, 12, 14, },
  { 0, 1, 10, 12, 14, },
  { 2, 10, 12, 14, },
  { 0, 2, 10, 12, 14, },
  { 1, 2, 10, 12, 14, },
  { 0, 1, 2, 10, 12, 14, },
  { 3, 10, 12, 14, },
  { 0, 3, 10, 12, 14, },
  { 1, 3, 10, 12, 14, },
  { 0, 1, 3, 10, 12, 14, },
  { 2, 3, 10, 12, 14, },
  { 0, 2, 3, 10, 12, 14, },
  { 1, 2, 3, 10, 12, 14, },
  { 0, 1, 2, 3, 10, 12, 14, },
  { 4, 10, 12, 14, },
  { 0, 4, 10, 12, 14, },
  { 1, 4, 10, 12, 14, },
  { 0, 1, 4, 10, 12, 14, },
  { 2, 4, 10, 12, 14, },
  { 0, 2, 4, 10, 12, 14, },
  { 1, 2, 4, 10, 12, 14, },
  { 0, 1, 2, 4, 10, 12, 14, },
  { 3, 4, 10, 12, 14, },
  { 0, 3, 4, 10, 12, 14, },
  { 1, 3, 4, 10, 12, 14, },
  { 0, 1, 3, 4, 10, 12, 14, },
  { 2, 3, 4, 10, 12, 14, },
  { 0, 2, 3, 4, 10, 12, 14, },
  { 1, 2, 3, 4, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 10, 12, 14, },
  { 5, 10, 12, 14, },
  { 0, 5, 10, 12, 14, },
  { 1, 5, 10, 12, 14, },
  { 0, 1, 5, 10, 12, 14, },
  { 2, 5, 10, 12, 14, },
  { 0, 2, 5, 10, 12, 14, },
  { 1, 2, 5, 10, 12, 14, },
  { 0, 1, 2, 5, 10, 12, 14, },
  { 3, 5, 10, 12, 14, },
  { 0, 3, 5, 10, 12, 14, },
  { 1, 3, 5, 10, 12, 14, },
  { 0, 1, 3, 5, 10, 12, 14, },
  { 2, 3, 5, 10, 12, 14, },
  { 0, 2, 3, 5, 10, 12, 14, },
  { 1, 2, 3, 5, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 10, 12, 14, },
  { 4, 5, 10, 12, 14, },
  { 0, 4, 5, 10, 12, 14, },
  { 1, 4, 5, 10, 12, 14, },
  { 0, 1, 4, 5, 10, 12, 14, },
  { 2, 4, 5, 10, 12, 14, },
  { 0, 2, 4, 5, 10, 12, 14, },
  { 1, 2, 4, 5, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 10, 12, 14, },
  { 3, 4, 5, 10, 12, 14, },
  { 0, 3, 4, 5, 10, 12, 14, },
  { 1, 3, 4, 5, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 10, 12, 14, },
  { 2, 3, 4, 5, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 10, 12, 14, },
  { 6, 10, 12, 14, },
  { 0, 6, 10, 12, 14, },
  { 1, 6, 10, 12, 14, },
  { 0, 1, 6, 10, 12, 14, },
  { 2, 6, 10, 12, 14, },
  { 0, 2, 6, 10, 12, 14, },
  { 1, 2, 6, 10, 12, 14, },
  { 0, 1, 2, 6, 10, 12, 14, },
  { 3, 6, 10, 12, 14, },
  { 0, 3, 6, 10, 12, 14, },
  { 1, 3, 6, 10, 12, 14, },
  { 0, 1, 3, 6, 10, 12, 14, },
  { 2, 3, 6, 10, 12, 14, },
  { 0, 2, 3, 6, 10, 12, 14, },
  { 1, 2, 3, 6, 10, 12, 14, },
  { 0, 1, 2, 3, 6, 10, 12, 14, },
  { 4, 6, 10, 12, 14, },
  { 0, 4, 6, 10, 12, 14, },
  { 1, 4, 6, 10, 12, 14, },
  { 0, 1, 4, 6, 10, 12, 14, },
  { 2, 4, 6, 10, 12, 14, },
  { 0, 2, 4, 6, 10, 12, 14, },
  { 1, 2, 4, 6, 10, 12, 14, },
  { 0, 1, 2, 4, 6, 10, 12, 14, },
  { 3, 4, 6, 10, 12, 14, },
  { 0, 3, 4, 6, 10, 12, 14, },
  { 1, 3, 4, 6, 10, 12, 14, },
  { 0, 1, 3, 4, 6, 10, 12, 14, },
  { 2, 3, 4, 6, 10, 12, 14, },
  { 0, 2, 3, 4, 6, 10, 12, 14, },
  { 1, 2, 3, 4, 6, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 10, 12, 14, },
  { 5, 6, 10, 12, 14, },
  { 0, 5, 6, 10, 12, 14, },
  { 1, 5, 6, 10, 12, 14, },
  { 0, 1, 5, 6, 10, 12, 14, },
  { 2, 5, 6, 10, 12, 14, },
  { 0, 2, 5, 6, 10, 12, 14, },
  { 1, 2, 5, 6, 10, 12, 14, },
  { 0, 1, 2, 5, 6, 10, 12, 14, },
  { 3, 5, 6, 10, 12, 14, },
  { 0, 3, 5, 6, 10, 12, 14, },
  { 1, 3, 5, 6, 10, 12, 14, },
  { 0, 1, 3, 5, 6, 10, 12, 14, },
  { 2, 3, 5, 6, 10, 12, 14, },
  { 0, 2, 3, 5, 6, 10, 12, 14, },
  { 1, 2, 3, 5, 6, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 10, 12, 14, },
  { 4, 5, 6, 10, 12, 14, },
  { 0, 4, 5, 6, 10, 12, 14, },
  { 1, 4, 5, 6, 10, 12, 14, },
  { 0, 1, 4, 5, 6, 10, 12, 14, },
  { 2, 4, 5, 6, 10, 12, 14, },
  { 0, 2, 4, 5, 6, 10, 12, 14, },
  { 1, 2, 4, 5, 6, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 10, 12, 14, },
  { 3, 4, 5, 6, 10, 12, 14, },
  { 0, 3, 4, 5, 6, 10, 12, 14, },
  { 1, 3, 4, 5, 6, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 10, 12, 14, },
  { 2, 3, 4, 5, 6, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 12, 14, },
  { 7, 10, 12, 14, },
  { 0, 7, 10, 12, 14, },
  { 1, 7, 10, 12, 14, },
  { 0, 1, 7, 10, 12, 14, },
  { 2, 7, 10, 12, 14, },
  { 0, 2, 7, 10, 12, 14, },
  { 1, 2, 7, 10, 12, 14, },
  { 0, 1, 2, 7, 10, 12, 14, },
  { 3, 7, 10, 12, 14, },
  { 0, 3, 7, 10, 12, 14, },
  { 1, 3, 7, 10, 12, 14, },
  { 0, 1, 3, 7, 10, 12, 14, },
  { 2, 3, 7, 10, 12, 14, },
  { 0, 2, 3, 7, 10, 12, 14, },
  { 1, 2, 3, 7, 10, 12, 14, },
  { 0, 1, 2, 3, 7, 10, 12, 14, },
  { 4, 7, 10, 12, 14, },
  { 0, 4, 7, 10, 12, 14, },
  { 1, 4, 7, 10, 12, 14, },
  { 0, 1, 4, 7, 10, 12, 14, },
  { 2, 4, 7, 10, 12, 14, },
  { 0, 2, 4, 7, 10, 12, 14, },
  { 1, 2, 4, 7, 10, 12, 14, },
  { 0, 1, 2, 4, 7, 10, 12, 14, },
  { 3, 4, 7, 10, 12, 14, },
  { 0, 3, 4, 7, 10, 12, 14, },
  { 1, 3, 4, 7, 10, 12, 14, },
  { 0, 1, 3, 4, 7, 10, 12, 14, },
  { 2, 3, 4, 7, 10, 12, 14, },
  { 0, 2, 3, 4, 7, 10, 12, 14, },
  { 1, 2, 3, 4, 7, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 10, 12, 14, },
  { 5, 7, 10, 12, 14, },
  { 0, 5, 7, 10, 12, 14, },
  { 1, 5, 7, 10, 12, 14, },
  { 0, 1, 5, 7, 10, 12, 14, },
  { 2, 5, 7, 10, 12, 14, },
  { 0, 2, 5, 7, 10, 12, 14, },
  { 1, 2, 5, 7, 10, 12, 14, },
  { 0, 1, 2, 5, 7, 10, 12, 14, },
  { 3, 5, 7, 10, 12, 14, },
  { 0, 3, 5, 7, 10, 12, 14, },
  { 1, 3, 5, 7, 10, 12, 14, },
  { 0, 1, 3, 5, 7, 10, 12, 14, },
  { 2, 3, 5, 7, 10, 12, 14, },
  { 0, 2, 3, 5, 7, 10, 12, 14, },
  { 1, 2, 3, 5, 7, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 10, 12, 14, },
  { 4, 5, 7, 10, 12, 14, },
  { 0, 4, 5, 7, 10, 12, 14, },
  { 1, 4, 5, 7, 10, 12, 14, },
  { 0, 1, 4, 5, 7, 10, 12, 14, },
  { 2, 4, 5, 7, 10, 12, 14, },
  { 0, 2, 4, 5, 7, 10, 12, 14, },
  { 1, 2, 4, 5, 7, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 10, 12, 14, },
  { 3, 4, 5, 7, 10, 12, 14, },
  { 0, 3, 4, 5, 7, 10, 12, 14, },
  { 1, 3, 4, 5, 7, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 10, 12, 14, },
  { 2, 3, 4, 5, 7, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 12, 14, },
  { 6, 7, 10, 12, 14, },
  { 0, 6, 7, 10, 12, 14, },
  { 1, 6, 7, 10, 12, 14, },
  { 0, 1, 6, 7, 10, 12, 14, },
  { 2, 6, 7, 10, 12, 14, },
  { 0, 2, 6, 7, 10, 12, 14, },
  { 1, 2, 6, 7, 10, 12, 14, },
  { 0, 1, 2, 6, 7, 10, 12, 14, },
  { 3, 6, 7, 10, 12, 14, },
  { 0, 3, 6, 7, 10, 12, 14, },
  { 1, 3, 6, 7, 10, 12, 14, },
  { 0, 1, 3, 6, 7, 10, 12, 14, },
  { 2, 3, 6, 7, 10, 12, 14, },
  { 0, 2, 3, 6, 7, 10, 12, 14, },
  { 1, 2, 3, 6, 7, 10, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 10, 12, 14, },
  { 4, 6, 7, 10, 12, 14, },
  { 0, 4, 6, 7, 10, 12, 14, },
  { 1, 4, 6, 7, 10, 12, 14, },
  { 0, 1, 4, 6, 7, 10, 12, 14, },
  { 2, 4, 6, 7, 10, 12, 14, },
  { 0, 2, 4, 6, 7, 10, 12, 14, },
  { 1, 2, 4, 6, 7, 10, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 10, 12, 14, },
  { 3, 4, 6, 7, 10, 12, 14, },
  { 0, 3, 4, 6, 7, 10, 12, 14, },
  { 1, 3, 4, 6, 7, 10, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 10, 12, 14, },
  { 2, 3, 4, 6, 7, 10, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 10, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 12, 14, },
  { 5, 6, 7, 10, 12, 14, },
  { 0, 5, 6, 7, 10, 12, 14, },
  { 1, 5, 6, 7, 10, 12, 14, },
  { 0, 1, 5, 6, 7, 10, 12, 14, },
  { 2, 5, 6, 7, 10, 12, 14, },
  { 0, 2, 5, 6, 7, 10, 12, 14, },
  { 1, 2, 5, 6, 7, 10, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 10, 12, 14, },
  { 3, 5, 6, 7, 10, 12, 14, },
  { 0, 3, 5, 6, 7, 10, 12, 14, },
  { 1, 3, 5, 6, 7, 10, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 10, 12, 14, },
  { 2, 3, 5, 6, 7, 10, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 10, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 12, 14, },
  { 4, 5, 6, 7, 10, 12, 14, },
  { 0, 4, 5, 6, 7, 10, 12, 14, },
  { 1, 4, 5, 6, 7, 10, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 10, 12, 14, },
  { 2, 4, 5, 6, 7, 10, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 10, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 12, 14, },
  { 3, 4, 5, 6, 7, 10, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 10, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 12, 14, },
  { 8, 10, 12, 14, },
  { 0, 8, 10, 12, 14, },
  { 1, 8, 10, 12, 14, },
  { 0, 1, 8, 10, 12, 14, },
  { 2, 8, 10, 12, 14, },
  { 0, 2, 8, 10, 12, 14, },
  { 1, 2, 8, 10, 12, 14, },
  { 0, 1, 2, 8, 10, 12, 14, },
  { 3, 8, 10, 12, 14, },
  { 0, 3, 8, 10, 12, 14, },
  { 1, 3, 8, 10, 12, 14, },
  { 0, 1, 3, 8, 10, 12, 14, },
  { 2, 3, 8, 10, 12, 14, },
  { 0, 2, 3, 8, 10, 12, 14, },
  { 1, 2, 3, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 8, 10, 12, 14, },
  { 4, 8, 10, 12, 14, },
  { 0, 4, 8, 10, 12, 14, },
  { 1, 4, 8, 10, 12, 14, },
  { 0, 1, 4, 8, 10, 12, 14, },
  { 2, 4, 8, 10, 12, 14, },
  { 0, 2, 4, 8, 10, 12, 14, },
  { 1, 2, 4, 8, 10, 12, 14, },
  { 0, 1, 2, 4, 8, 10, 12, 14, },
  { 3, 4, 8, 10, 12, 14, },
  { 0, 3, 4, 8, 10, 12, 14, },
  { 1, 3, 4, 8, 10, 12, 14, },
  { 0, 1, 3, 4, 8, 10, 12, 14, },
  { 2, 3, 4, 8, 10, 12, 14, },
  { 0, 2, 3, 4, 8, 10, 12, 14, },
  { 1, 2, 3, 4, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 8, 10, 12, 14, },
  { 5, 8, 10, 12, 14, },
  { 0, 5, 8, 10, 12, 14, },
  { 1, 5, 8, 10, 12, 14, },
  { 0, 1, 5, 8, 10, 12, 14, },
  { 2, 5, 8, 10, 12, 14, },
  { 0, 2, 5, 8, 10, 12, 14, },
  { 1, 2, 5, 8, 10, 12, 14, },
  { 0, 1, 2, 5, 8, 10, 12, 14, },
  { 3, 5, 8, 10, 12, 14, },
  { 0, 3, 5, 8, 10, 12, 14, },
  { 1, 3, 5, 8, 10, 12, 14, },
  { 0, 1, 3, 5, 8, 10, 12, 14, },
  { 2, 3, 5, 8, 10, 12, 14, },
  { 0, 2, 3, 5, 8, 10, 12, 14, },
  { 1, 2, 3, 5, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 8, 10, 12, 14, },
  { 4, 5, 8, 10, 12, 14, },
  { 0, 4, 5, 8, 10, 12, 14, },
  { 1, 4, 5, 8, 10, 12, 14, },
  { 0, 1, 4, 5, 8, 10, 12, 14, },
  { 2, 4, 5, 8, 10, 12, 14, },
  { 0, 2, 4, 5, 8, 10, 12, 14, },
  { 1, 2, 4, 5, 8, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 8, 10, 12, 14, },
  { 3, 4, 5, 8, 10, 12, 14, },
  { 0, 3, 4, 5, 8, 10, 12, 14, },
  { 1, 3, 4, 5, 8, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 8, 10, 12, 14, },
  { 2, 3, 4, 5, 8, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 8, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 12, 14, },
  { 6, 8, 10, 12, 14, },
  { 0, 6, 8, 10, 12, 14, },
  { 1, 6, 8, 10, 12, 14, },
  { 0, 1, 6, 8, 10, 12, 14, },
  { 2, 6, 8, 10, 12, 14, },
  { 0, 2, 6, 8, 10, 12, 14, },
  { 1, 2, 6, 8, 10, 12, 14, },
  { 0, 1, 2, 6, 8, 10, 12, 14, },
  { 3, 6, 8, 10, 12, 14, },
  { 0, 3, 6, 8, 10, 12, 14, },
  { 1, 3, 6, 8, 10, 12, 14, },
  { 0, 1, 3, 6, 8, 10, 12, 14, },
  { 2, 3, 6, 8, 10, 12, 14, },
  { 0, 2, 3, 6, 8, 10, 12, 14, },
  { 1, 2, 3, 6, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 6, 8, 10, 12, 14, },
  { 4, 6, 8, 10, 12, 14, },
  { 0, 4, 6, 8, 10, 12, 14, },
  { 1, 4, 6, 8, 10, 12, 14, },
  { 0, 1, 4, 6, 8, 10, 12, 14, },
  { 2, 4, 6, 8, 10, 12, 14, },
  { 0, 2, 4, 6, 8, 10, 12, 14, },
  { 1, 2, 4, 6, 8, 10, 12, 14, },
  { 0, 1, 2, 4, 6, 8, 10, 12, 14, },
  { 3, 4, 6, 8, 10, 12, 14, },
  { 0, 3, 4, 6, 8, 10, 12, 14, },
  { 1, 3, 4, 6, 8, 10, 12, 14, },
  { 0, 1, 3, 4, 6, 8, 10, 12, 14, },
  { 2, 3, 4, 6, 8, 10, 12, 14, },
  { 0, 2, 3, 4, 6, 8, 10, 12, 14, },
  { 1, 2, 3, 4, 6, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 12, 14, },
  { 5, 6, 8, 10, 12, 14, },
  { 0, 5, 6, 8, 10, 12, 14, },
  { 1, 5, 6, 8, 10, 12, 14, },
  { 0, 1, 5, 6, 8, 10, 12, 14, },
  { 2, 5, 6, 8, 10, 12, 14, },
  { 0, 2, 5, 6, 8, 10, 12, 14, },
  { 1, 2, 5, 6, 8, 10, 12, 14, },
  { 0, 1, 2, 5, 6, 8, 10, 12, 14, },
  { 3, 5, 6, 8, 10, 12, 14, },
  { 0, 3, 5, 6, 8, 10, 12, 14, },
  { 1, 3, 5, 6, 8, 10, 12, 14, },
  { 0, 1, 3, 5, 6, 8, 10, 12, 14, },
  { 2, 3, 5, 6, 8, 10, 12, 14, },
  { 0, 2, 3, 5, 6, 8, 10, 12, 14, },
  { 1, 2, 3, 5, 6, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 12, 14, },
  { 4, 5, 6, 8, 10, 12, 14, },
  { 0, 4, 5, 6, 8, 10, 12, 14, },
  { 1, 4, 5, 6, 8, 10, 12, 14, },
  { 0, 1, 4, 5, 6, 8, 10, 12, 14, },
  { 2, 4, 5, 6, 8, 10, 12, 14, },
  { 0, 2, 4, 5, 6, 8, 10, 12, 14, },
  { 1, 2, 4, 5, 6, 8, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 12, 14, },
  { 3, 4, 5, 6, 8, 10, 12, 14, },
  { 0, 3, 4, 5, 6, 8, 10, 12, 14, },
  { 1, 3, 4, 5, 6, 8, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 12, 14, },
  { 2, 3, 4, 5, 6, 8, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 12, 14, },
  { 7, 8, 10, 12, 14, },
  { 0, 7, 8, 10, 12, 14, },
  { 1, 7, 8, 10, 12, 14, },
  { 0, 1, 7, 8, 10, 12, 14, },
  { 2, 7, 8, 10, 12, 14, },
  { 0, 2, 7, 8, 10, 12, 14, },
  { 1, 2, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 7, 8, 10, 12, 14, },
  { 3, 7, 8, 10, 12, 14, },
  { 0, 3, 7, 8, 10, 12, 14, },
  { 1, 3, 7, 8, 10, 12, 14, },
  { 0, 1, 3, 7, 8, 10, 12, 14, },
  { 2, 3, 7, 8, 10, 12, 14, },
  { 0, 2, 3, 7, 8, 10, 12, 14, },
  { 1, 2, 3, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 7, 8, 10, 12, 14, },
  { 4, 7, 8, 10, 12, 14, },
  { 0, 4, 7, 8, 10, 12, 14, },
  { 1, 4, 7, 8, 10, 12, 14, },
  { 0, 1, 4, 7, 8, 10, 12, 14, },
  { 2, 4, 7, 8, 10, 12, 14, },
  { 0, 2, 4, 7, 8, 10, 12, 14, },
  { 1, 2, 4, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 4, 7, 8, 10, 12, 14, },
  { 3, 4, 7, 8, 10, 12, 14, },
  { 0, 3, 4, 7, 8, 10, 12, 14, },
  { 1, 3, 4, 7, 8, 10, 12, 14, },
  { 0, 1, 3, 4, 7, 8, 10, 12, 14, },
  { 2, 3, 4, 7, 8, 10, 12, 14, },
  { 0, 2, 3, 4, 7, 8, 10, 12, 14, },
  { 1, 2, 3, 4, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 12, 14, },
  { 5, 7, 8, 10, 12, 14, },
  { 0, 5, 7, 8, 10, 12, 14, },
  { 1, 5, 7, 8, 10, 12, 14, },
  { 0, 1, 5, 7, 8, 10, 12, 14, },
  { 2, 5, 7, 8, 10, 12, 14, },
  { 0, 2, 5, 7, 8, 10, 12, 14, },
  { 1, 2, 5, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 5, 7, 8, 10, 12, 14, },
  { 3, 5, 7, 8, 10, 12, 14, },
  { 0, 3, 5, 7, 8, 10, 12, 14, },
  { 1, 3, 5, 7, 8, 10, 12, 14, },
  { 0, 1, 3, 5, 7, 8, 10, 12, 14, },
  { 2, 3, 5, 7, 8, 10, 12, 14, },
  { 0, 2, 3, 5, 7, 8, 10, 12, 14, },
  { 1, 2, 3, 5, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 12, 14, },
  { 4, 5, 7, 8, 10, 12, 14, },
  { 0, 4, 5, 7, 8, 10, 12, 14, },
  { 1, 4, 5, 7, 8, 10, 12, 14, },
  { 0, 1, 4, 5, 7, 8, 10, 12, 14, },
  { 2, 4, 5, 7, 8, 10, 12, 14, },
  { 0, 2, 4, 5, 7, 8, 10, 12, 14, },
  { 1, 2, 4, 5, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 12, 14, },
  { 3, 4, 5, 7, 8, 10, 12, 14, },
  { 0, 3, 4, 5, 7, 8, 10, 12, 14, },
  { 1, 3, 4, 5, 7, 8, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 12, 14, },
  { 2, 3, 4, 5, 7, 8, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 14, },
  { 6, 7, 8, 10, 12, 14, },
  { 0, 6, 7, 8, 10, 12, 14, },
  { 1, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 6, 7, 8, 10, 12, 14, },
  { 2, 6, 7, 8, 10, 12, 14, },
  { 0, 2, 6, 7, 8, 10, 12, 14, },
  { 1, 2, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 6, 7, 8, 10, 12, 14, },
  { 3, 6, 7, 8, 10, 12, 14, },
  { 0, 3, 6, 7, 8, 10, 12, 14, },
  { 1, 3, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 3, 6, 7, 8, 10, 12, 14, },
  { 2, 3, 6, 7, 8, 10, 12, 14, },
  { 0, 2, 3, 6, 7, 8, 10, 12, 14, },
  { 1, 2, 3, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 12, 14, },
  { 4, 6, 7, 8, 10, 12, 14, },
  { 0, 4, 6, 7, 8, 10, 12, 14, },
  { 1, 4, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 4, 6, 7, 8, 10, 12, 14, },
  { 2, 4, 6, 7, 8, 10, 12, 14, },
  { 0, 2, 4, 6, 7, 8, 10, 12, 14, },
  { 1, 2, 4, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 12, 14, },
  { 3, 4, 6, 7, 8, 10, 12, 14, },
  { 0, 3, 4, 6, 7, 8, 10, 12, 14, },
  { 1, 3, 4, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 12, 14, },
  { 2, 3, 4, 6, 7, 8, 10, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 12, 14, },
  { 5, 6, 7, 8, 10, 12, 14, },
  { 0, 5, 6, 7, 8, 10, 12, 14, },
  { 1, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 5, 6, 7, 8, 10, 12, 14, },
  { 2, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 2, 5, 6, 7, 8, 10, 12, 14, },
  { 1, 2, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 12, 14, },
  { 3, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 3, 5, 6, 7, 8, 10, 12, 14, },
  { 1, 3, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 12, 14, },
  { 2, 3, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 12, 14, },
  { 4, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 1, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 2, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 3, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, },
  { 9, 10, 12, 14, },
  { 0, 9, 10, 12, 14, },
  { 1, 9, 10, 12, 14, },
  { 0, 1, 9, 10, 12, 14, },
  { 2, 9, 10, 12, 14, },
  { 0, 2, 9, 10, 12, 14, },
  { 1, 2, 9, 10, 12, 14, },
  { 0, 1, 2, 9, 10, 12, 14, },
  { 3, 9, 10, 12, 14, },
  { 0, 3, 9, 10, 12, 14, },
  { 1, 3, 9, 10, 12, 14, },
  { 0, 1, 3, 9, 10, 12, 14, },
  { 2, 3, 9, 10, 12, 14, },
  { 0, 2, 3, 9, 10, 12, 14, },
  { 1, 2, 3, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 9, 10, 12, 14, },
  { 4, 9, 10, 12, 14, },
  { 0, 4, 9, 10, 12, 14, },
  { 1, 4, 9, 10, 12, 14, },
  { 0, 1, 4, 9, 10, 12, 14, },
  { 2, 4, 9, 10, 12, 14, },
  { 0, 2, 4, 9, 10, 12, 14, },
  { 1, 2, 4, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 9, 10, 12, 14, },
  { 3, 4, 9, 10, 12, 14, },
  { 0, 3, 4, 9, 10, 12, 14, },
  { 1, 3, 4, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 9, 10, 12, 14, },
  { 2, 3, 4, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 9, 10, 12, 14, },
  { 5, 9, 10, 12, 14, },
  { 0, 5, 9, 10, 12, 14, },
  { 1, 5, 9, 10, 12, 14, },
  { 0, 1, 5, 9, 10, 12, 14, },
  { 2, 5, 9, 10, 12, 14, },
  { 0, 2, 5, 9, 10, 12, 14, },
  { 1, 2, 5, 9, 10, 12, 14, },
  { 0, 1, 2, 5, 9, 10, 12, 14, },
  { 3, 5, 9, 10, 12, 14, },
  { 0, 3, 5, 9, 10, 12, 14, },
  { 1, 3, 5, 9, 10, 12, 14, },
  { 0, 1, 3, 5, 9, 10, 12, 14, },
  { 2, 3, 5, 9, 10, 12, 14, },
  { 0, 2, 3, 5, 9, 10, 12, 14, },
  { 1, 2, 3, 5, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 9, 10, 12, 14, },
  { 4, 5, 9, 10, 12, 14, },
  { 0, 4, 5, 9, 10, 12, 14, },
  { 1, 4, 5, 9, 10, 12, 14, },
  { 0, 1, 4, 5, 9, 10, 12, 14, },
  { 2, 4, 5, 9, 10, 12, 14, },
  { 0, 2, 4, 5, 9, 10, 12, 14, },
  { 1, 2, 4, 5, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 9, 10, 12, 14, },
  { 3, 4, 5, 9, 10, 12, 14, },
  { 0, 3, 4, 5, 9, 10, 12, 14, },
  { 1, 3, 4, 5, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 9, 10, 12, 14, },
  { 2, 3, 4, 5, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 12, 14, },
  { 6, 9, 10, 12, 14, },
  { 0, 6, 9, 10, 12, 14, },
  { 1, 6, 9, 10, 12, 14, },
  { 0, 1, 6, 9, 10, 12, 14, },
  { 2, 6, 9, 10, 12, 14, },
  { 0, 2, 6, 9, 10, 12, 14, },
  { 1, 2, 6, 9, 10, 12, 14, },
  { 0, 1, 2, 6, 9, 10, 12, 14, },
  { 3, 6, 9, 10, 12, 14, },
  { 0, 3, 6, 9, 10, 12, 14, },
  { 1, 3, 6, 9, 10, 12, 14, },
  { 0, 1, 3, 6, 9, 10, 12, 14, },
  { 2, 3, 6, 9, 10, 12, 14, },
  { 0, 2, 3, 6, 9, 10, 12, 14, },
  { 1, 2, 3, 6, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 6, 9, 10, 12, 14, },
  { 4, 6, 9, 10, 12, 14, },
  { 0, 4, 6, 9, 10, 12, 14, },
  { 1, 4, 6, 9, 10, 12, 14, },
  { 0, 1, 4, 6, 9, 10, 12, 14, },
  { 2, 4, 6, 9, 10, 12, 14, },
  { 0, 2, 4, 6, 9, 10, 12, 14, },
  { 1, 2, 4, 6, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 6, 9, 10, 12, 14, },
  { 3, 4, 6, 9, 10, 12, 14, },
  { 0, 3, 4, 6, 9, 10, 12, 14, },
  { 1, 3, 4, 6, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 6, 9, 10, 12, 14, },
  { 2, 3, 4, 6, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 6, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 6, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 12, 14, },
  { 5, 6, 9, 10, 12, 14, },
  { 0, 5, 6, 9, 10, 12, 14, },
  { 1, 5, 6, 9, 10, 12, 14, },
  { 0, 1, 5, 6, 9, 10, 12, 14, },
  { 2, 5, 6, 9, 10, 12, 14, },
  { 0, 2, 5, 6, 9, 10, 12, 14, },
  { 1, 2, 5, 6, 9, 10, 12, 14, },
  { 0, 1, 2, 5, 6, 9, 10, 12, 14, },
  { 3, 5, 6, 9, 10, 12, 14, },
  { 0, 3, 5, 6, 9, 10, 12, 14, },
  { 1, 3, 5, 6, 9, 10, 12, 14, },
  { 0, 1, 3, 5, 6, 9, 10, 12, 14, },
  { 2, 3, 5, 6, 9, 10, 12, 14, },
  { 0, 2, 3, 5, 6, 9, 10, 12, 14, },
  { 1, 2, 3, 5, 6, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 12, 14, },
  { 4, 5, 6, 9, 10, 12, 14, },
  { 0, 4, 5, 6, 9, 10, 12, 14, },
  { 1, 4, 5, 6, 9, 10, 12, 14, },
  { 0, 1, 4, 5, 6, 9, 10, 12, 14, },
  { 2, 4, 5, 6, 9, 10, 12, 14, },
  { 0, 2, 4, 5, 6, 9, 10, 12, 14, },
  { 1, 2, 4, 5, 6, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 12, 14, },
  { 3, 4, 5, 6, 9, 10, 12, 14, },
  { 0, 3, 4, 5, 6, 9, 10, 12, 14, },
  { 1, 3, 4, 5, 6, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 12, 14, },
  { 2, 3, 4, 5, 6, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 12, 14, },
  { 7, 9, 10, 12, 14, },
  { 0, 7, 9, 10, 12, 14, },
  { 1, 7, 9, 10, 12, 14, },
  { 0, 1, 7, 9, 10, 12, 14, },
  { 2, 7, 9, 10, 12, 14, },
  { 0, 2, 7, 9, 10, 12, 14, },
  { 1, 2, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 7, 9, 10, 12, 14, },
  { 3, 7, 9, 10, 12, 14, },
  { 0, 3, 7, 9, 10, 12, 14, },
  { 1, 3, 7, 9, 10, 12, 14, },
  { 0, 1, 3, 7, 9, 10, 12, 14, },
  { 2, 3, 7, 9, 10, 12, 14, },
  { 0, 2, 3, 7, 9, 10, 12, 14, },
  { 1, 2, 3, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 7, 9, 10, 12, 14, },
  { 4, 7, 9, 10, 12, 14, },
  { 0, 4, 7, 9, 10, 12, 14, },
  { 1, 4, 7, 9, 10, 12, 14, },
  { 0, 1, 4, 7, 9, 10, 12, 14, },
  { 2, 4, 7, 9, 10, 12, 14, },
  { 0, 2, 4, 7, 9, 10, 12, 14, },
  { 1, 2, 4, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 7, 9, 10, 12, 14, },
  { 3, 4, 7, 9, 10, 12, 14, },
  { 0, 3, 4, 7, 9, 10, 12, 14, },
  { 1, 3, 4, 7, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 7, 9, 10, 12, 14, },
  { 2, 3, 4, 7, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 7, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 12, 14, },
  { 5, 7, 9, 10, 12, 14, },
  { 0, 5, 7, 9, 10, 12, 14, },
  { 1, 5, 7, 9, 10, 12, 14, },
  { 0, 1, 5, 7, 9, 10, 12, 14, },
  { 2, 5, 7, 9, 10, 12, 14, },
  { 0, 2, 5, 7, 9, 10, 12, 14, },
  { 1, 2, 5, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 5, 7, 9, 10, 12, 14, },
  { 3, 5, 7, 9, 10, 12, 14, },
  { 0, 3, 5, 7, 9, 10, 12, 14, },
  { 1, 3, 5, 7, 9, 10, 12, 14, },
  { 0, 1, 3, 5, 7, 9, 10, 12, 14, },
  { 2, 3, 5, 7, 9, 10, 12, 14, },
  { 0, 2, 3, 5, 7, 9, 10, 12, 14, },
  { 1, 2, 3, 5, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 12, 14, },
  { 4, 5, 7, 9, 10, 12, 14, },
  { 0, 4, 5, 7, 9, 10, 12, 14, },
  { 1, 4, 5, 7, 9, 10, 12, 14, },
  { 0, 1, 4, 5, 7, 9, 10, 12, 14, },
  { 2, 4, 5, 7, 9, 10, 12, 14, },
  { 0, 2, 4, 5, 7, 9, 10, 12, 14, },
  { 1, 2, 4, 5, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 12, 14, },
  { 3, 4, 5, 7, 9, 10, 12, 14, },
  { 0, 3, 4, 5, 7, 9, 10, 12, 14, },
  { 1, 3, 4, 5, 7, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 12, 14, },
  { 2, 3, 4, 5, 7, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 12, 14, },
  { 6, 7, 9, 10, 12, 14, },
  { 0, 6, 7, 9, 10, 12, 14, },
  { 1, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 6, 7, 9, 10, 12, 14, },
  { 2, 6, 7, 9, 10, 12, 14, },
  { 0, 2, 6, 7, 9, 10, 12, 14, },
  { 1, 2, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 6, 7, 9, 10, 12, 14, },
  { 3, 6, 7, 9, 10, 12, 14, },
  { 0, 3, 6, 7, 9, 10, 12, 14, },
  { 1, 3, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 3, 6, 7, 9, 10, 12, 14, },
  { 2, 3, 6, 7, 9, 10, 12, 14, },
  { 0, 2, 3, 6, 7, 9, 10, 12, 14, },
  { 1, 2, 3, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 12, 14, },
  { 4, 6, 7, 9, 10, 12, 14, },
  { 0, 4, 6, 7, 9, 10, 12, 14, },
  { 1, 4, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 4, 6, 7, 9, 10, 12, 14, },
  { 2, 4, 6, 7, 9, 10, 12, 14, },
  { 0, 2, 4, 6, 7, 9, 10, 12, 14, },
  { 1, 2, 4, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 12, 14, },
  { 3, 4, 6, 7, 9, 10, 12, 14, },
  { 0, 3, 4, 6, 7, 9, 10, 12, 14, },
  { 1, 3, 4, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 12, 14, },
  { 2, 3, 4, 6, 7, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 12, 14, },
  { 5, 6, 7, 9, 10, 12, 14, },
  { 0, 5, 6, 7, 9, 10, 12, 14, },
  { 1, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 5, 6, 7, 9, 10, 12, 14, },
  { 2, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 2, 5, 6, 7, 9, 10, 12, 14, },
  { 1, 2, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 12, 14, },
  { 3, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 3, 5, 6, 7, 9, 10, 12, 14, },
  { 1, 3, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 12, 14, },
  { 2, 3, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 12, 14, },
  { 4, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 1, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 2, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 3, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, },
  { 8, 9, 10, 12, 14, },
  { 0, 8, 9, 10, 12, 14, },
  { 1, 8, 9, 10, 12, 14, },
  { 0, 1, 8, 9, 10, 12, 14, },
  { 2, 8, 9, 10, 12, 14, },
  { 0, 2, 8, 9, 10, 12, 14, },
  { 1, 2, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 8, 9, 10, 12, 14, },
  { 3, 8, 9, 10, 12, 14, },
  { 0, 3, 8, 9, 10, 12, 14, },
  { 1, 3, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 8, 9, 10, 12, 14, },
  { 2, 3, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 8, 9, 10, 12, 14, },
  { 4, 8, 9, 10, 12, 14, },
  { 0, 4, 8, 9, 10, 12, 14, },
  { 1, 4, 8, 9, 10, 12, 14, },
  { 0, 1, 4, 8, 9, 10, 12, 14, },
  { 2, 4, 8, 9, 10, 12, 14, },
  { 0, 2, 4, 8, 9, 10, 12, 14, },
  { 1, 2, 4, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 8, 9, 10, 12, 14, },
  { 3, 4, 8, 9, 10, 12, 14, },
  { 0, 3, 4, 8, 9, 10, 12, 14, },
  { 1, 3, 4, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 8, 9, 10, 12, 14, },
  { 2, 3, 4, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 12, 14, },
  { 5, 8, 9, 10, 12, 14, },
  { 0, 5, 8, 9, 10, 12, 14, },
  { 1, 5, 8, 9, 10, 12, 14, },
  { 0, 1, 5, 8, 9, 10, 12, 14, },
  { 2, 5, 8, 9, 10, 12, 14, },
  { 0, 2, 5, 8, 9, 10, 12, 14, },
  { 1, 2, 5, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 5, 8, 9, 10, 12, 14, },
  { 3, 5, 8, 9, 10, 12, 14, },
  { 0, 3, 5, 8, 9, 10, 12, 14, },
  { 1, 3, 5, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 5, 8, 9, 10, 12, 14, },
  { 2, 3, 5, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 5, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 5, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 12, 14, },
  { 4, 5, 8, 9, 10, 12, 14, },
  { 0, 4, 5, 8, 9, 10, 12, 14, },
  { 1, 4, 5, 8, 9, 10, 12, 14, },
  { 0, 1, 4, 5, 8, 9, 10, 12, 14, },
  { 2, 4, 5, 8, 9, 10, 12, 14, },
  { 0, 2, 4, 5, 8, 9, 10, 12, 14, },
  { 1, 2, 4, 5, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 12, 14, },
  { 3, 4, 5, 8, 9, 10, 12, 14, },
  { 0, 3, 4, 5, 8, 9, 10, 12, 14, },
  { 1, 3, 4, 5, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 12, 14, },
  { 2, 3, 4, 5, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 12, 14, },
  { 6, 8, 9, 10, 12, 14, },
  { 0, 6, 8, 9, 10, 12, 14, },
  { 1, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 6, 8, 9, 10, 12, 14, },
  { 2, 6, 8, 9, 10, 12, 14, },
  { 0, 2, 6, 8, 9, 10, 12, 14, },
  { 1, 2, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 6, 8, 9, 10, 12, 14, },
  { 3, 6, 8, 9, 10, 12, 14, },
  { 0, 3, 6, 8, 9, 10, 12, 14, },
  { 1, 3, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 6, 8, 9, 10, 12, 14, },
  { 2, 3, 6, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 6, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 12, 14, },
  { 4, 6, 8, 9, 10, 12, 14, },
  { 0, 4, 6, 8, 9, 10, 12, 14, },
  { 1, 4, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 4, 6, 8, 9, 10, 12, 14, },
  { 2, 4, 6, 8, 9, 10, 12, 14, },
  { 0, 2, 4, 6, 8, 9, 10, 12, 14, },
  { 1, 2, 4, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 12, 14, },
  { 3, 4, 6, 8, 9, 10, 12, 14, },
  { 0, 3, 4, 6, 8, 9, 10, 12, 14, },
  { 1, 3, 4, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 12, 14, },
  { 2, 3, 4, 6, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 12, 14, },
  { 5, 6, 8, 9, 10, 12, 14, },
  { 0, 5, 6, 8, 9, 10, 12, 14, },
  { 1, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 5, 6, 8, 9, 10, 12, 14, },
  { 2, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 2, 5, 6, 8, 9, 10, 12, 14, },
  { 1, 2, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 12, 14, },
  { 3, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 3, 5, 6, 8, 9, 10, 12, 14, },
  { 1, 3, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 12, 14, },
  { 2, 3, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 12, 14, },
  { 4, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 1, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 2, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 3, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 14, },
  { 7, 8, 9, 10, 12, 14, },
  { 0, 7, 8, 9, 10, 12, 14, },
  { 1, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 7, 8, 9, 10, 12, 14, },
  { 2, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 7, 8, 9, 10, 12, 14, },
  { 3, 7, 8, 9, 10, 12, 14, },
  { 0, 3, 7, 8, 9, 10, 12, 14, },
  { 1, 3, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 7, 8, 9, 10, 12, 14, },
  { 2, 3, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 12, 14, },
  { 4, 7, 8, 9, 10, 12, 14, },
  { 0, 4, 7, 8, 9, 10, 12, 14, },
  { 1, 4, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 4, 7, 8, 9, 10, 12, 14, },
  { 2, 4, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 4, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 4, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 12, 14, },
  { 3, 4, 7, 8, 9, 10, 12, 14, },
  { 0, 3, 4, 7, 8, 9, 10, 12, 14, },
  { 1, 3, 4, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 12, 14, },
  { 2, 3, 4, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 12, 14, },
  { 5, 7, 8, 9, 10, 12, 14, },
  { 0, 5, 7, 8, 9, 10, 12, 14, },
  { 1, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 5, 7, 8, 9, 10, 12, 14, },
  { 2, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 5, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 12, 14, },
  { 3, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 3, 5, 7, 8, 9, 10, 12, 14, },
  { 1, 3, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 12, 14, },
  { 2, 3, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 12, 14, },
  { 4, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 1, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 2, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 3, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 14, },
  { 6, 7, 8, 9, 10, 12, 14, },
  { 0, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 6, 7, 8, 9, 10, 12, 14, },
  { 2, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 12, 14, },
  { 3, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 3, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 3, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 12, 14, },
  { 2, 3, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 12, 14, },
  { 4, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 2, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 3, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, },
  { 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 2, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 3, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, },
  { 11, 12, 14, },
  { 0, 11, 12, 14, },
  { 1, 11, 12, 14, },
  { 0, 1, 11, 12, 14, },
  { 2, 11, 12, 14, },
  { 0, 2, 11, 12, 14, },
  { 1, 2, 11, 12, 14, },
  { 0, 1, 2, 11, 12, 14, },
  { 3, 11, 12, 14, },
  { 0, 3, 11, 12, 14, },
  { 1, 3, 11, 12, 14, },
  { 0, 1, 3, 11, 12, 14, },
  { 2, 3, 11, 12, 14, },
  { 0, 2, 3, 11, 12, 14, },
  { 1, 2, 3, 11, 12, 14, },
  { 0, 1, 2, 3, 11, 12, 14, },
  { 4, 11, 12, 14, },
  { 0, 4, 11, 12, 14, },
  { 1, 4, 11, 12, 14, },
  { 0, 1, 4, 11, 12, 14, },
  { 2, 4, 11, 12, 14, },
  { 0, 2, 4, 11, 12, 14, },
  { 1, 2, 4, 11, 12, 14, },
  { 0, 1, 2, 4, 11, 12, 14, },
  { 3, 4, 11, 12, 14, },
  { 0, 3, 4, 11, 12, 14, },
  { 1, 3, 4, 11, 12, 14, },
  { 0, 1, 3, 4, 11, 12, 14, },
  { 2, 3, 4, 11, 12, 14, },
  { 0, 2, 3, 4, 11, 12, 14, },
  { 1, 2, 3, 4, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 11, 12, 14, },
  { 5, 11, 12, 14, },
  { 0, 5, 11, 12, 14, },
  { 1, 5, 11, 12, 14, },
  { 0, 1, 5, 11, 12, 14, },
  { 2, 5, 11, 12, 14, },
  { 0, 2, 5, 11, 12, 14, },
  { 1, 2, 5, 11, 12, 14, },
  { 0, 1, 2, 5, 11, 12, 14, },
  { 3, 5, 11, 12, 14, },
  { 0, 3, 5, 11, 12, 14, },
  { 1, 3, 5, 11, 12, 14, },
  { 0, 1, 3, 5, 11, 12, 14, },
  { 2, 3, 5, 11, 12, 14, },
  { 0, 2, 3, 5, 11, 12, 14, },
  { 1, 2, 3, 5, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 11, 12, 14, },
  { 4, 5, 11, 12, 14, },
  { 0, 4, 5, 11, 12, 14, },
  { 1, 4, 5, 11, 12, 14, },
  { 0, 1, 4, 5, 11, 12, 14, },
  { 2, 4, 5, 11, 12, 14, },
  { 0, 2, 4, 5, 11, 12, 14, },
  { 1, 2, 4, 5, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 11, 12, 14, },
  { 3, 4, 5, 11, 12, 14, },
  { 0, 3, 4, 5, 11, 12, 14, },
  { 1, 3, 4, 5, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 11, 12, 14, },
  { 2, 3, 4, 5, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 11, 12, 14, },
  { 6, 11, 12, 14, },
  { 0, 6, 11, 12, 14, },
  { 1, 6, 11, 12, 14, },
  { 0, 1, 6, 11, 12, 14, },
  { 2, 6, 11, 12, 14, },
  { 0, 2, 6, 11, 12, 14, },
  { 1, 2, 6, 11, 12, 14, },
  { 0, 1, 2, 6, 11, 12, 14, },
  { 3, 6, 11, 12, 14, },
  { 0, 3, 6, 11, 12, 14, },
  { 1, 3, 6, 11, 12, 14, },
  { 0, 1, 3, 6, 11, 12, 14, },
  { 2, 3, 6, 11, 12, 14, },
  { 0, 2, 3, 6, 11, 12, 14, },
  { 1, 2, 3, 6, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 11, 12, 14, },
  { 4, 6, 11, 12, 14, },
  { 0, 4, 6, 11, 12, 14, },
  { 1, 4, 6, 11, 12, 14, },
  { 0, 1, 4, 6, 11, 12, 14, },
  { 2, 4, 6, 11, 12, 14, },
  { 0, 2, 4, 6, 11, 12, 14, },
  { 1, 2, 4, 6, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 11, 12, 14, },
  { 3, 4, 6, 11, 12, 14, },
  { 0, 3, 4, 6, 11, 12, 14, },
  { 1, 3, 4, 6, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 11, 12, 14, },
  { 2, 3, 4, 6, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 11, 12, 14, },
  { 5, 6, 11, 12, 14, },
  { 0, 5, 6, 11, 12, 14, },
  { 1, 5, 6, 11, 12, 14, },
  { 0, 1, 5, 6, 11, 12, 14, },
  { 2, 5, 6, 11, 12, 14, },
  { 0, 2, 5, 6, 11, 12, 14, },
  { 1, 2, 5, 6, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 11, 12, 14, },
  { 3, 5, 6, 11, 12, 14, },
  { 0, 3, 5, 6, 11, 12, 14, },
  { 1, 3, 5, 6, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 11, 12, 14, },
  { 2, 3, 5, 6, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 11, 12, 14, },
  { 4, 5, 6, 11, 12, 14, },
  { 0, 4, 5, 6, 11, 12, 14, },
  { 1, 4, 5, 6, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 11, 12, 14, },
  { 2, 4, 5, 6, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 11, 12, 14, },
  { 3, 4, 5, 6, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 12, 14, },
  { 7, 11, 12, 14, },
  { 0, 7, 11, 12, 14, },
  { 1, 7, 11, 12, 14, },
  { 0, 1, 7, 11, 12, 14, },
  { 2, 7, 11, 12, 14, },
  { 0, 2, 7, 11, 12, 14, },
  { 1, 2, 7, 11, 12, 14, },
  { 0, 1, 2, 7, 11, 12, 14, },
  { 3, 7, 11, 12, 14, },
  { 0, 3, 7, 11, 12, 14, },
  { 1, 3, 7, 11, 12, 14, },
  { 0, 1, 3, 7, 11, 12, 14, },
  { 2, 3, 7, 11, 12, 14, },
  { 0, 2, 3, 7, 11, 12, 14, },
  { 1, 2, 3, 7, 11, 12, 14, },
  { 0, 1, 2, 3, 7, 11, 12, 14, },
  { 4, 7, 11, 12, 14, },
  { 0, 4, 7, 11, 12, 14, },
  { 1, 4, 7, 11, 12, 14, },
  { 0, 1, 4, 7, 11, 12, 14, },
  { 2, 4, 7, 11, 12, 14, },
  { 0, 2, 4, 7, 11, 12, 14, },
  { 1, 2, 4, 7, 11, 12, 14, },
  { 0, 1, 2, 4, 7, 11, 12, 14, },
  { 3, 4, 7, 11, 12, 14, },
  { 0, 3, 4, 7, 11, 12, 14, },
  { 1, 3, 4, 7, 11, 12, 14, },
  { 0, 1, 3, 4, 7, 11, 12, 14, },
  { 2, 3, 4, 7, 11, 12, 14, },
  { 0, 2, 3, 4, 7, 11, 12, 14, },
  { 1, 2, 3, 4, 7, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 11, 12, 14, },
  { 5, 7, 11, 12, 14, },
  { 0, 5, 7, 11, 12, 14, },
  { 1, 5, 7, 11, 12, 14, },
  { 0, 1, 5, 7, 11, 12, 14, },
  { 2, 5, 7, 11, 12, 14, },
  { 0, 2, 5, 7, 11, 12, 14, },
  { 1, 2, 5, 7, 11, 12, 14, },
  { 0, 1, 2, 5, 7, 11, 12, 14, },
  { 3, 5, 7, 11, 12, 14, },
  { 0, 3, 5, 7, 11, 12, 14, },
  { 1, 3, 5, 7, 11, 12, 14, },
  { 0, 1, 3, 5, 7, 11, 12, 14, },
  { 2, 3, 5, 7, 11, 12, 14, },
  { 0, 2, 3, 5, 7, 11, 12, 14, },
  { 1, 2, 3, 5, 7, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 11, 12, 14, },
  { 4, 5, 7, 11, 12, 14, },
  { 0, 4, 5, 7, 11, 12, 14, },
  { 1, 4, 5, 7, 11, 12, 14, },
  { 0, 1, 4, 5, 7, 11, 12, 14, },
  { 2, 4, 5, 7, 11, 12, 14, },
  { 0, 2, 4, 5, 7, 11, 12, 14, },
  { 1, 2, 4, 5, 7, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 11, 12, 14, },
  { 3, 4, 5, 7, 11, 12, 14, },
  { 0, 3, 4, 5, 7, 11, 12, 14, },
  { 1, 3, 4, 5, 7, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 11, 12, 14, },
  { 2, 3, 4, 5, 7, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 12, 14, },
  { 6, 7, 11, 12, 14, },
  { 0, 6, 7, 11, 12, 14, },
  { 1, 6, 7, 11, 12, 14, },
  { 0, 1, 6, 7, 11, 12, 14, },
  { 2, 6, 7, 11, 12, 14, },
  { 0, 2, 6, 7, 11, 12, 14, },
  { 1, 2, 6, 7, 11, 12, 14, },
  { 0, 1, 2, 6, 7, 11, 12, 14, },
  { 3, 6, 7, 11, 12, 14, },
  { 0, 3, 6, 7, 11, 12, 14, },
  { 1, 3, 6, 7, 11, 12, 14, },
  { 0, 1, 3, 6, 7, 11, 12, 14, },
  { 2, 3, 6, 7, 11, 12, 14, },
  { 0, 2, 3, 6, 7, 11, 12, 14, },
  { 1, 2, 3, 6, 7, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 11, 12, 14, },
  { 4, 6, 7, 11, 12, 14, },
  { 0, 4, 6, 7, 11, 12, 14, },
  { 1, 4, 6, 7, 11, 12, 14, },
  { 0, 1, 4, 6, 7, 11, 12, 14, },
  { 2, 4, 6, 7, 11, 12, 14, },
  { 0, 2, 4, 6, 7, 11, 12, 14, },
  { 1, 2, 4, 6, 7, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 11, 12, 14, },
  { 3, 4, 6, 7, 11, 12, 14, },
  { 0, 3, 4, 6, 7, 11, 12, 14, },
  { 1, 3, 4, 6, 7, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 11, 12, 14, },
  { 2, 3, 4, 6, 7, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 12, 14, },
  { 5, 6, 7, 11, 12, 14, },
  { 0, 5, 6, 7, 11, 12, 14, },
  { 1, 5, 6, 7, 11, 12, 14, },
  { 0, 1, 5, 6, 7, 11, 12, 14, },
  { 2, 5, 6, 7, 11, 12, 14, },
  { 0, 2, 5, 6, 7, 11, 12, 14, },
  { 1, 2, 5, 6, 7, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 11, 12, 14, },
  { 3, 5, 6, 7, 11, 12, 14, },
  { 0, 3, 5, 6, 7, 11, 12, 14, },
  { 1, 3, 5, 6, 7, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 11, 12, 14, },
  { 2, 3, 5, 6, 7, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 12, 14, },
  { 4, 5, 6, 7, 11, 12, 14, },
  { 0, 4, 5, 6, 7, 11, 12, 14, },
  { 1, 4, 5, 6, 7, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 11, 12, 14, },
  { 2, 4, 5, 6, 7, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 12, 14, },
  { 3, 4, 5, 6, 7, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 12, 14, },
  { 8, 11, 12, 14, },
  { 0, 8, 11, 12, 14, },
  { 1, 8, 11, 12, 14, },
  { 0, 1, 8, 11, 12, 14, },
  { 2, 8, 11, 12, 14, },
  { 0, 2, 8, 11, 12, 14, },
  { 1, 2, 8, 11, 12, 14, },
  { 0, 1, 2, 8, 11, 12, 14, },
  { 3, 8, 11, 12, 14, },
  { 0, 3, 8, 11, 12, 14, },
  { 1, 3, 8, 11, 12, 14, },
  { 0, 1, 3, 8, 11, 12, 14, },
  { 2, 3, 8, 11, 12, 14, },
  { 0, 2, 3, 8, 11, 12, 14, },
  { 1, 2, 3, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 8, 11, 12, 14, },
  { 4, 8, 11, 12, 14, },
  { 0, 4, 8, 11, 12, 14, },
  { 1, 4, 8, 11, 12, 14, },
  { 0, 1, 4, 8, 11, 12, 14, },
  { 2, 4, 8, 11, 12, 14, },
  { 0, 2, 4, 8, 11, 12, 14, },
  { 1, 2, 4, 8, 11, 12, 14, },
  { 0, 1, 2, 4, 8, 11, 12, 14, },
  { 3, 4, 8, 11, 12, 14, },
  { 0, 3, 4, 8, 11, 12, 14, },
  { 1, 3, 4, 8, 11, 12, 14, },
  { 0, 1, 3, 4, 8, 11, 12, 14, },
  { 2, 3, 4, 8, 11, 12, 14, },
  { 0, 2, 3, 4, 8, 11, 12, 14, },
  { 1, 2, 3, 4, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 8, 11, 12, 14, },
  { 5, 8, 11, 12, 14, },
  { 0, 5, 8, 11, 12, 14, },
  { 1, 5, 8, 11, 12, 14, },
  { 0, 1, 5, 8, 11, 12, 14, },
  { 2, 5, 8, 11, 12, 14, },
  { 0, 2, 5, 8, 11, 12, 14, },
  { 1, 2, 5, 8, 11, 12, 14, },
  { 0, 1, 2, 5, 8, 11, 12, 14, },
  { 3, 5, 8, 11, 12, 14, },
  { 0, 3, 5, 8, 11, 12, 14, },
  { 1, 3, 5, 8, 11, 12, 14, },
  { 0, 1, 3, 5, 8, 11, 12, 14, },
  { 2, 3, 5, 8, 11, 12, 14, },
  { 0, 2, 3, 5, 8, 11, 12, 14, },
  { 1, 2, 3, 5, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 8, 11, 12, 14, },
  { 4, 5, 8, 11, 12, 14, },
  { 0, 4, 5, 8, 11, 12, 14, },
  { 1, 4, 5, 8, 11, 12, 14, },
  { 0, 1, 4, 5, 8, 11, 12, 14, },
  { 2, 4, 5, 8, 11, 12, 14, },
  { 0, 2, 4, 5, 8, 11, 12, 14, },
  { 1, 2, 4, 5, 8, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 8, 11, 12, 14, },
  { 3, 4, 5, 8, 11, 12, 14, },
  { 0, 3, 4, 5, 8, 11, 12, 14, },
  { 1, 3, 4, 5, 8, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 8, 11, 12, 14, },
  { 2, 3, 4, 5, 8, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 8, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 12, 14, },
  { 6, 8, 11, 12, 14, },
  { 0, 6, 8, 11, 12, 14, },
  { 1, 6, 8, 11, 12, 14, },
  { 0, 1, 6, 8, 11, 12, 14, },
  { 2, 6, 8, 11, 12, 14, },
  { 0, 2, 6, 8, 11, 12, 14, },
  { 1, 2, 6, 8, 11, 12, 14, },
  { 0, 1, 2, 6, 8, 11, 12, 14, },
  { 3, 6, 8, 11, 12, 14, },
  { 0, 3, 6, 8, 11, 12, 14, },
  { 1, 3, 6, 8, 11, 12, 14, },
  { 0, 1, 3, 6, 8, 11, 12, 14, },
  { 2, 3, 6, 8, 11, 12, 14, },
  { 0, 2, 3, 6, 8, 11, 12, 14, },
  { 1, 2, 3, 6, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 8, 11, 12, 14, },
  { 4, 6, 8, 11, 12, 14, },
  { 0, 4, 6, 8, 11, 12, 14, },
  { 1, 4, 6, 8, 11, 12, 14, },
  { 0, 1, 4, 6, 8, 11, 12, 14, },
  { 2, 4, 6, 8, 11, 12, 14, },
  { 0, 2, 4, 6, 8, 11, 12, 14, },
  { 1, 2, 4, 6, 8, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 8, 11, 12, 14, },
  { 3, 4, 6, 8, 11, 12, 14, },
  { 0, 3, 4, 6, 8, 11, 12, 14, },
  { 1, 3, 4, 6, 8, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 8, 11, 12, 14, },
  { 2, 3, 4, 6, 8, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 8, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 12, 14, },
  { 5, 6, 8, 11, 12, 14, },
  { 0, 5, 6, 8, 11, 12, 14, },
  { 1, 5, 6, 8, 11, 12, 14, },
  { 0, 1, 5, 6, 8, 11, 12, 14, },
  { 2, 5, 6, 8, 11, 12, 14, },
  { 0, 2, 5, 6, 8, 11, 12, 14, },
  { 1, 2, 5, 6, 8, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 8, 11, 12, 14, },
  { 3, 5, 6, 8, 11, 12, 14, },
  { 0, 3, 5, 6, 8, 11, 12, 14, },
  { 1, 3, 5, 6, 8, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 8, 11, 12, 14, },
  { 2, 3, 5, 6, 8, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 8, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 12, 14, },
  { 4, 5, 6, 8, 11, 12, 14, },
  { 0, 4, 5, 6, 8, 11, 12, 14, },
  { 1, 4, 5, 6, 8, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 8, 11, 12, 14, },
  { 2, 4, 5, 6, 8, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 8, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 8, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 12, 14, },
  { 3, 4, 5, 6, 8, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 8, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 8, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 8, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 12, 14, },
  { 7, 8, 11, 12, 14, },
  { 0, 7, 8, 11, 12, 14, },
  { 1, 7, 8, 11, 12, 14, },
  { 0, 1, 7, 8, 11, 12, 14, },
  { 2, 7, 8, 11, 12, 14, },
  { 0, 2, 7, 8, 11, 12, 14, },
  { 1, 2, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 7, 8, 11, 12, 14, },
  { 3, 7, 8, 11, 12, 14, },
  { 0, 3, 7, 8, 11, 12, 14, },
  { 1, 3, 7, 8, 11, 12, 14, },
  { 0, 1, 3, 7, 8, 11, 12, 14, },
  { 2, 3, 7, 8, 11, 12, 14, },
  { 0, 2, 3, 7, 8, 11, 12, 14, },
  { 1, 2, 3, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 7, 8, 11, 12, 14, },
  { 4, 7, 8, 11, 12, 14, },
  { 0, 4, 7, 8, 11, 12, 14, },
  { 1, 4, 7, 8, 11, 12, 14, },
  { 0, 1, 4, 7, 8, 11, 12, 14, },
  { 2, 4, 7, 8, 11, 12, 14, },
  { 0, 2, 4, 7, 8, 11, 12, 14, },
  { 1, 2, 4, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 4, 7, 8, 11, 12, 14, },
  { 3, 4, 7, 8, 11, 12, 14, },
  { 0, 3, 4, 7, 8, 11, 12, 14, },
  { 1, 3, 4, 7, 8, 11, 12, 14, },
  { 0, 1, 3, 4, 7, 8, 11, 12, 14, },
  { 2, 3, 4, 7, 8, 11, 12, 14, },
  { 0, 2, 3, 4, 7, 8, 11, 12, 14, },
  { 1, 2, 3, 4, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 12, 14, },
  { 5, 7, 8, 11, 12, 14, },
  { 0, 5, 7, 8, 11, 12, 14, },
  { 1, 5, 7, 8, 11, 12, 14, },
  { 0, 1, 5, 7, 8, 11, 12, 14, },
  { 2, 5, 7, 8, 11, 12, 14, },
  { 0, 2, 5, 7, 8, 11, 12, 14, },
  { 1, 2, 5, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 5, 7, 8, 11, 12, 14, },
  { 3, 5, 7, 8, 11, 12, 14, },
  { 0, 3, 5, 7, 8, 11, 12, 14, },
  { 1, 3, 5, 7, 8, 11, 12, 14, },
  { 0, 1, 3, 5, 7, 8, 11, 12, 14, },
  { 2, 3, 5, 7, 8, 11, 12, 14, },
  { 0, 2, 3, 5, 7, 8, 11, 12, 14, },
  { 1, 2, 3, 5, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 12, 14, },
  { 4, 5, 7, 8, 11, 12, 14, },
  { 0, 4, 5, 7, 8, 11, 12, 14, },
  { 1, 4, 5, 7, 8, 11, 12, 14, },
  { 0, 1, 4, 5, 7, 8, 11, 12, 14, },
  { 2, 4, 5, 7, 8, 11, 12, 14, },
  { 0, 2, 4, 5, 7, 8, 11, 12, 14, },
  { 1, 2, 4, 5, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 12, 14, },
  { 3, 4, 5, 7, 8, 11, 12, 14, },
  { 0, 3, 4, 5, 7, 8, 11, 12, 14, },
  { 1, 3, 4, 5, 7, 8, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 12, 14, },
  { 2, 3, 4, 5, 7, 8, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 12, 14, },
  { 6, 7, 8, 11, 12, 14, },
  { 0, 6, 7, 8, 11, 12, 14, },
  { 1, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 6, 7, 8, 11, 12, 14, },
  { 2, 6, 7, 8, 11, 12, 14, },
  { 0, 2, 6, 7, 8, 11, 12, 14, },
  { 1, 2, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 6, 7, 8, 11, 12, 14, },
  { 3, 6, 7, 8, 11, 12, 14, },
  { 0, 3, 6, 7, 8, 11, 12, 14, },
  { 1, 3, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 3, 6, 7, 8, 11, 12, 14, },
  { 2, 3, 6, 7, 8, 11, 12, 14, },
  { 0, 2, 3, 6, 7, 8, 11, 12, 14, },
  { 1, 2, 3, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 12, 14, },
  { 4, 6, 7, 8, 11, 12, 14, },
  { 0, 4, 6, 7, 8, 11, 12, 14, },
  { 1, 4, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 4, 6, 7, 8, 11, 12, 14, },
  { 2, 4, 6, 7, 8, 11, 12, 14, },
  { 0, 2, 4, 6, 7, 8, 11, 12, 14, },
  { 1, 2, 4, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 12, 14, },
  { 3, 4, 6, 7, 8, 11, 12, 14, },
  { 0, 3, 4, 6, 7, 8, 11, 12, 14, },
  { 1, 3, 4, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 12, 14, },
  { 2, 3, 4, 6, 7, 8, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 12, 14, },
  { 5, 6, 7, 8, 11, 12, 14, },
  { 0, 5, 6, 7, 8, 11, 12, 14, },
  { 1, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 5, 6, 7, 8, 11, 12, 14, },
  { 2, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 2, 5, 6, 7, 8, 11, 12, 14, },
  { 1, 2, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 12, 14, },
  { 3, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 3, 5, 6, 7, 8, 11, 12, 14, },
  { 1, 3, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 12, 14, },
  { 2, 3, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 12, 14, },
  { 4, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 1, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 2, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 3, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, },
  { 9, 11, 12, 14, },
  { 0, 9, 11, 12, 14, },
  { 1, 9, 11, 12, 14, },
  { 0, 1, 9, 11, 12, 14, },
  { 2, 9, 11, 12, 14, },
  { 0, 2, 9, 11, 12, 14, },
  { 1, 2, 9, 11, 12, 14, },
  { 0, 1, 2, 9, 11, 12, 14, },
  { 3, 9, 11, 12, 14, },
  { 0, 3, 9, 11, 12, 14, },
  { 1, 3, 9, 11, 12, 14, },
  { 0, 1, 3, 9, 11, 12, 14, },
  { 2, 3, 9, 11, 12, 14, },
  { 0, 2, 3, 9, 11, 12, 14, },
  { 1, 2, 3, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 9, 11, 12, 14, },
  { 4, 9, 11, 12, 14, },
  { 0, 4, 9, 11, 12, 14, },
  { 1, 4, 9, 11, 12, 14, },
  { 0, 1, 4, 9, 11, 12, 14, },
  { 2, 4, 9, 11, 12, 14, },
  { 0, 2, 4, 9, 11, 12, 14, },
  { 1, 2, 4, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 9, 11, 12, 14, },
  { 3, 4, 9, 11, 12, 14, },
  { 0, 3, 4, 9, 11, 12, 14, },
  { 1, 3, 4, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 9, 11, 12, 14, },
  { 2, 3, 4, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 9, 11, 12, 14, },
  { 5, 9, 11, 12, 14, },
  { 0, 5, 9, 11, 12, 14, },
  { 1, 5, 9, 11, 12, 14, },
  { 0, 1, 5, 9, 11, 12, 14, },
  { 2, 5, 9, 11, 12, 14, },
  { 0, 2, 5, 9, 11, 12, 14, },
  { 1, 2, 5, 9, 11, 12, 14, },
  { 0, 1, 2, 5, 9, 11, 12, 14, },
  { 3, 5, 9, 11, 12, 14, },
  { 0, 3, 5, 9, 11, 12, 14, },
  { 1, 3, 5, 9, 11, 12, 14, },
  { 0, 1, 3, 5, 9, 11, 12, 14, },
  { 2, 3, 5, 9, 11, 12, 14, },
  { 0, 2, 3, 5, 9, 11, 12, 14, },
  { 1, 2, 3, 5, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 9, 11, 12, 14, },
  { 4, 5, 9, 11, 12, 14, },
  { 0, 4, 5, 9, 11, 12, 14, },
  { 1, 4, 5, 9, 11, 12, 14, },
  { 0, 1, 4, 5, 9, 11, 12, 14, },
  { 2, 4, 5, 9, 11, 12, 14, },
  { 0, 2, 4, 5, 9, 11, 12, 14, },
  { 1, 2, 4, 5, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 9, 11, 12, 14, },
  { 3, 4, 5, 9, 11, 12, 14, },
  { 0, 3, 4, 5, 9, 11, 12, 14, },
  { 1, 3, 4, 5, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 9, 11, 12, 14, },
  { 2, 3, 4, 5, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 12, 14, },
  { 6, 9, 11, 12, 14, },
  { 0, 6, 9, 11, 12, 14, },
  { 1, 6, 9, 11, 12, 14, },
  { 0, 1, 6, 9, 11, 12, 14, },
  { 2, 6, 9, 11, 12, 14, },
  { 0, 2, 6, 9, 11, 12, 14, },
  { 1, 2, 6, 9, 11, 12, 14, },
  { 0, 1, 2, 6, 9, 11, 12, 14, },
  { 3, 6, 9, 11, 12, 14, },
  { 0, 3, 6, 9, 11, 12, 14, },
  { 1, 3, 6, 9, 11, 12, 14, },
  { 0, 1, 3, 6, 9, 11, 12, 14, },
  { 2, 3, 6, 9, 11, 12, 14, },
  { 0, 2, 3, 6, 9, 11, 12, 14, },
  { 1, 2, 3, 6, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 9, 11, 12, 14, },
  { 4, 6, 9, 11, 12, 14, },
  { 0, 4, 6, 9, 11, 12, 14, },
  { 1, 4, 6, 9, 11, 12, 14, },
  { 0, 1, 4, 6, 9, 11, 12, 14, },
  { 2, 4, 6, 9, 11, 12, 14, },
  { 0, 2, 4, 6, 9, 11, 12, 14, },
  { 1, 2, 4, 6, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 9, 11, 12, 14, },
  { 3, 4, 6, 9, 11, 12, 14, },
  { 0, 3, 4, 6, 9, 11, 12, 14, },
  { 1, 3, 4, 6, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 9, 11, 12, 14, },
  { 2, 3, 4, 6, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 12, 14, },
  { 5, 6, 9, 11, 12, 14, },
  { 0, 5, 6, 9, 11, 12, 14, },
  { 1, 5, 6, 9, 11, 12, 14, },
  { 0, 1, 5, 6, 9, 11, 12, 14, },
  { 2, 5, 6, 9, 11, 12, 14, },
  { 0, 2, 5, 6, 9, 11, 12, 14, },
  { 1, 2, 5, 6, 9, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 9, 11, 12, 14, },
  { 3, 5, 6, 9, 11, 12, 14, },
  { 0, 3, 5, 6, 9, 11, 12, 14, },
  { 1, 3, 5, 6, 9, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 9, 11, 12, 14, },
  { 2, 3, 5, 6, 9, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 9, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 12, 14, },
  { 4, 5, 6, 9, 11, 12, 14, },
  { 0, 4, 5, 6, 9, 11, 12, 14, },
  { 1, 4, 5, 6, 9, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 9, 11, 12, 14, },
  { 2, 4, 5, 6, 9, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 9, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 12, 14, },
  { 3, 4, 5, 6, 9, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 9, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 12, 14, },
  { 7, 9, 11, 12, 14, },
  { 0, 7, 9, 11, 12, 14, },
  { 1, 7, 9, 11, 12, 14, },
  { 0, 1, 7, 9, 11, 12, 14, },
  { 2, 7, 9, 11, 12, 14, },
  { 0, 2, 7, 9, 11, 12, 14, },
  { 1, 2, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 7, 9, 11, 12, 14, },
  { 3, 7, 9, 11, 12, 14, },
  { 0, 3, 7, 9, 11, 12, 14, },
  { 1, 3, 7, 9, 11, 12, 14, },
  { 0, 1, 3, 7, 9, 11, 12, 14, },
  { 2, 3, 7, 9, 11, 12, 14, },
  { 0, 2, 3, 7, 9, 11, 12, 14, },
  { 1, 2, 3, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 7, 9, 11, 12, 14, },
  { 4, 7, 9, 11, 12, 14, },
  { 0, 4, 7, 9, 11, 12, 14, },
  { 1, 4, 7, 9, 11, 12, 14, },
  { 0, 1, 4, 7, 9, 11, 12, 14, },
  { 2, 4, 7, 9, 11, 12, 14, },
  { 0, 2, 4, 7, 9, 11, 12, 14, },
  { 1, 2, 4, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 7, 9, 11, 12, 14, },
  { 3, 4, 7, 9, 11, 12, 14, },
  { 0, 3, 4, 7, 9, 11, 12, 14, },
  { 1, 3, 4, 7, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 7, 9, 11, 12, 14, },
  { 2, 3, 4, 7, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 7, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 12, 14, },
  { 5, 7, 9, 11, 12, 14, },
  { 0, 5, 7, 9, 11, 12, 14, },
  { 1, 5, 7, 9, 11, 12, 14, },
  { 0, 1, 5, 7, 9, 11, 12, 14, },
  { 2, 5, 7, 9, 11, 12, 14, },
  { 0, 2, 5, 7, 9, 11, 12, 14, },
  { 1, 2, 5, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 5, 7, 9, 11, 12, 14, },
  { 3, 5, 7, 9, 11, 12, 14, },
  { 0, 3, 5, 7, 9, 11, 12, 14, },
  { 1, 3, 5, 7, 9, 11, 12, 14, },
  { 0, 1, 3, 5, 7, 9, 11, 12, 14, },
  { 2, 3, 5, 7, 9, 11, 12, 14, },
  { 0, 2, 3, 5, 7, 9, 11, 12, 14, },
  { 1, 2, 3, 5, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 12, 14, },
  { 4, 5, 7, 9, 11, 12, 14, },
  { 0, 4, 5, 7, 9, 11, 12, 14, },
  { 1, 4, 5, 7, 9, 11, 12, 14, },
  { 0, 1, 4, 5, 7, 9, 11, 12, 14, },
  { 2, 4, 5, 7, 9, 11, 12, 14, },
  { 0, 2, 4, 5, 7, 9, 11, 12, 14, },
  { 1, 2, 4, 5, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 12, 14, },
  { 3, 4, 5, 7, 9, 11, 12, 14, },
  { 0, 3, 4, 5, 7, 9, 11, 12, 14, },
  { 1, 3, 4, 5, 7, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 12, 14, },
  { 2, 3, 4, 5, 7, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 12, 14, },
  { 6, 7, 9, 11, 12, 14, },
  { 0, 6, 7, 9, 11, 12, 14, },
  { 1, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 6, 7, 9, 11, 12, 14, },
  { 2, 6, 7, 9, 11, 12, 14, },
  { 0, 2, 6, 7, 9, 11, 12, 14, },
  { 1, 2, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 6, 7, 9, 11, 12, 14, },
  { 3, 6, 7, 9, 11, 12, 14, },
  { 0, 3, 6, 7, 9, 11, 12, 14, },
  { 1, 3, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 3, 6, 7, 9, 11, 12, 14, },
  { 2, 3, 6, 7, 9, 11, 12, 14, },
  { 0, 2, 3, 6, 7, 9, 11, 12, 14, },
  { 1, 2, 3, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 12, 14, },
  { 4, 6, 7, 9, 11, 12, 14, },
  { 0, 4, 6, 7, 9, 11, 12, 14, },
  { 1, 4, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 4, 6, 7, 9, 11, 12, 14, },
  { 2, 4, 6, 7, 9, 11, 12, 14, },
  { 0, 2, 4, 6, 7, 9, 11, 12, 14, },
  { 1, 2, 4, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 12, 14, },
  { 3, 4, 6, 7, 9, 11, 12, 14, },
  { 0, 3, 4, 6, 7, 9, 11, 12, 14, },
  { 1, 3, 4, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 12, 14, },
  { 2, 3, 4, 6, 7, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 12, 14, },
  { 5, 6, 7, 9, 11, 12, 14, },
  { 0, 5, 6, 7, 9, 11, 12, 14, },
  { 1, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 5, 6, 7, 9, 11, 12, 14, },
  { 2, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 2, 5, 6, 7, 9, 11, 12, 14, },
  { 1, 2, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 12, 14, },
  { 3, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 3, 5, 6, 7, 9, 11, 12, 14, },
  { 1, 3, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 12, 14, },
  { 2, 3, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 12, 14, },
  { 4, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 1, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 2, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 3, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 14, },
  { 8, 9, 11, 12, 14, },
  { 0, 8, 9, 11, 12, 14, },
  { 1, 8, 9, 11, 12, 14, },
  { 0, 1, 8, 9, 11, 12, 14, },
  { 2, 8, 9, 11, 12, 14, },
  { 0, 2, 8, 9, 11, 12, 14, },
  { 1, 2, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 8, 9, 11, 12, 14, },
  { 3, 8, 9, 11, 12, 14, },
  { 0, 3, 8, 9, 11, 12, 14, },
  { 1, 3, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 8, 9, 11, 12, 14, },
  { 2, 3, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 8, 9, 11, 12, 14, },
  { 4, 8, 9, 11, 12, 14, },
  { 0, 4, 8, 9, 11, 12, 14, },
  { 1, 4, 8, 9, 11, 12, 14, },
  { 0, 1, 4, 8, 9, 11, 12, 14, },
  { 2, 4, 8, 9, 11, 12, 14, },
  { 0, 2, 4, 8, 9, 11, 12, 14, },
  { 1, 2, 4, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 8, 9, 11, 12, 14, },
  { 3, 4, 8, 9, 11, 12, 14, },
  { 0, 3, 4, 8, 9, 11, 12, 14, },
  { 1, 3, 4, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 8, 9, 11, 12, 14, },
  { 2, 3, 4, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 12, 14, },
  { 5, 8, 9, 11, 12, 14, },
  { 0, 5, 8, 9, 11, 12, 14, },
  { 1, 5, 8, 9, 11, 12, 14, },
  { 0, 1, 5, 8, 9, 11, 12, 14, },
  { 2, 5, 8, 9, 11, 12, 14, },
  { 0, 2, 5, 8, 9, 11, 12, 14, },
  { 1, 2, 5, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 5, 8, 9, 11, 12, 14, },
  { 3, 5, 8, 9, 11, 12, 14, },
  { 0, 3, 5, 8, 9, 11, 12, 14, },
  { 1, 3, 5, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 5, 8, 9, 11, 12, 14, },
  { 2, 3, 5, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 5, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 5, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 12, 14, },
  { 4, 5, 8, 9, 11, 12, 14, },
  { 0, 4, 5, 8, 9, 11, 12, 14, },
  { 1, 4, 5, 8, 9, 11, 12, 14, },
  { 0, 1, 4, 5, 8, 9, 11, 12, 14, },
  { 2, 4, 5, 8, 9, 11, 12, 14, },
  { 0, 2, 4, 5, 8, 9, 11, 12, 14, },
  { 1, 2, 4, 5, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 12, 14, },
  { 3, 4, 5, 8, 9, 11, 12, 14, },
  { 0, 3, 4, 5, 8, 9, 11, 12, 14, },
  { 1, 3, 4, 5, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 12, 14, },
  { 2, 3, 4, 5, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 12, 14, },
  { 6, 8, 9, 11, 12, 14, },
  { 0, 6, 8, 9, 11, 12, 14, },
  { 1, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 6, 8, 9, 11, 12, 14, },
  { 2, 6, 8, 9, 11, 12, 14, },
  { 0, 2, 6, 8, 9, 11, 12, 14, },
  { 1, 2, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 6, 8, 9, 11, 12, 14, },
  { 3, 6, 8, 9, 11, 12, 14, },
  { 0, 3, 6, 8, 9, 11, 12, 14, },
  { 1, 3, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 6, 8, 9, 11, 12, 14, },
  { 2, 3, 6, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 6, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 12, 14, },
  { 4, 6, 8, 9, 11, 12, 14, },
  { 0, 4, 6, 8, 9, 11, 12, 14, },
  { 1, 4, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 4, 6, 8, 9, 11, 12, 14, },
  { 2, 4, 6, 8, 9, 11, 12, 14, },
  { 0, 2, 4, 6, 8, 9, 11, 12, 14, },
  { 1, 2, 4, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 12, 14, },
  { 3, 4, 6, 8, 9, 11, 12, 14, },
  { 0, 3, 4, 6, 8, 9, 11, 12, 14, },
  { 1, 3, 4, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 12, 14, },
  { 2, 3, 4, 6, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 12, 14, },
  { 5, 6, 8, 9, 11, 12, 14, },
  { 0, 5, 6, 8, 9, 11, 12, 14, },
  { 1, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 5, 6, 8, 9, 11, 12, 14, },
  { 2, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 2, 5, 6, 8, 9, 11, 12, 14, },
  { 1, 2, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 12, 14, },
  { 3, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 3, 5, 6, 8, 9, 11, 12, 14, },
  { 1, 3, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 12, 14, },
  { 2, 3, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 12, 14, },
  { 4, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 1, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 2, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 3, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 14, },
  { 7, 8, 9, 11, 12, 14, },
  { 0, 7, 8, 9, 11, 12, 14, },
  { 1, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 7, 8, 9, 11, 12, 14, },
  { 2, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 7, 8, 9, 11, 12, 14, },
  { 3, 7, 8, 9, 11, 12, 14, },
  { 0, 3, 7, 8, 9, 11, 12, 14, },
  { 1, 3, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 7, 8, 9, 11, 12, 14, },
  { 2, 3, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 12, 14, },
  { 4, 7, 8, 9, 11, 12, 14, },
  { 0, 4, 7, 8, 9, 11, 12, 14, },
  { 1, 4, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 4, 7, 8, 9, 11, 12, 14, },
  { 2, 4, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 4, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 4, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 12, 14, },
  { 3, 4, 7, 8, 9, 11, 12, 14, },
  { 0, 3, 4, 7, 8, 9, 11, 12, 14, },
  { 1, 3, 4, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 12, 14, },
  { 2, 3, 4, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 12, 14, },
  { 5, 7, 8, 9, 11, 12, 14, },
  { 0, 5, 7, 8, 9, 11, 12, 14, },
  { 1, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 5, 7, 8, 9, 11, 12, 14, },
  { 2, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 5, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 12, 14, },
  { 3, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 3, 5, 7, 8, 9, 11, 12, 14, },
  { 1, 3, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 12, 14, },
  { 2, 3, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 12, 14, },
  { 4, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 1, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 2, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 3, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 14, },
  { 6, 7, 8, 9, 11, 12, 14, },
  { 0, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 6, 7, 8, 9, 11, 12, 14, },
  { 2, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 12, 14, },
  { 3, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 3, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 3, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 12, 14, },
  { 2, 3, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 12, 14, },
  { 4, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 2, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 3, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, },
  { 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 2, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 3, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, },
  { 10, 11, 12, 14, },
  { 0, 10, 11, 12, 14, },
  { 1, 10, 11, 12, 14, },
  { 0, 1, 10, 11, 12, 14, },
  { 2, 10, 11, 12, 14, },
  { 0, 2, 10, 11, 12, 14, },
  { 1, 2, 10, 11, 12, 14, },
  { 0, 1, 2, 10, 11, 12, 14, },
  { 3, 10, 11, 12, 14, },
  { 0, 3, 10, 11, 12, 14, },
  { 1, 3, 10, 11, 12, 14, },
  { 0, 1, 3, 10, 11, 12, 14, },
  { 2, 3, 10, 11, 12, 14, },
  { 0, 2, 3, 10, 11, 12, 14, },
  { 1, 2, 3, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 10, 11, 12, 14, },
  { 4, 10, 11, 12, 14, },
  { 0, 4, 10, 11, 12, 14, },
  { 1, 4, 10, 11, 12, 14, },
  { 0, 1, 4, 10, 11, 12, 14, },
  { 2, 4, 10, 11, 12, 14, },
  { 0, 2, 4, 10, 11, 12, 14, },
  { 1, 2, 4, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 10, 11, 12, 14, },
  { 3, 4, 10, 11, 12, 14, },
  { 0, 3, 4, 10, 11, 12, 14, },
  { 1, 3, 4, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 10, 11, 12, 14, },
  { 2, 3, 4, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 10, 11, 12, 14, },
  { 5, 10, 11, 12, 14, },
  { 0, 5, 10, 11, 12, 14, },
  { 1, 5, 10, 11, 12, 14, },
  { 0, 1, 5, 10, 11, 12, 14, },
  { 2, 5, 10, 11, 12, 14, },
  { 0, 2, 5, 10, 11, 12, 14, },
  { 1, 2, 5, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 10, 11, 12, 14, },
  { 3, 5, 10, 11, 12, 14, },
  { 0, 3, 5, 10, 11, 12, 14, },
  { 1, 3, 5, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 10, 11, 12, 14, },
  { 2, 3, 5, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 10, 11, 12, 14, },
  { 4, 5, 10, 11, 12, 14, },
  { 0, 4, 5, 10, 11, 12, 14, },
  { 1, 4, 5, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 10, 11, 12, 14, },
  { 2, 4, 5, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 10, 11, 12, 14, },
  { 3, 4, 5, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 12, 14, },
  { 6, 10, 11, 12, 14, },
  { 0, 6, 10, 11, 12, 14, },
  { 1, 6, 10, 11, 12, 14, },
  { 0, 1, 6, 10, 11, 12, 14, },
  { 2, 6, 10, 11, 12, 14, },
  { 0, 2, 6, 10, 11, 12, 14, },
  { 1, 2, 6, 10, 11, 12, 14, },
  { 0, 1, 2, 6, 10, 11, 12, 14, },
  { 3, 6, 10, 11, 12, 14, },
  { 0, 3, 6, 10, 11, 12, 14, },
  { 1, 3, 6, 10, 11, 12, 14, },
  { 0, 1, 3, 6, 10, 11, 12, 14, },
  { 2, 3, 6, 10, 11, 12, 14, },
  { 0, 2, 3, 6, 10, 11, 12, 14, },
  { 1, 2, 3, 6, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 10, 11, 12, 14, },
  { 4, 6, 10, 11, 12, 14, },
  { 0, 4, 6, 10, 11, 12, 14, },
  { 1, 4, 6, 10, 11, 12, 14, },
  { 0, 1, 4, 6, 10, 11, 12, 14, },
  { 2, 4, 6, 10, 11, 12, 14, },
  { 0, 2, 4, 6, 10, 11, 12, 14, },
  { 1, 2, 4, 6, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 10, 11, 12, 14, },
  { 3, 4, 6, 10, 11, 12, 14, },
  { 0, 3, 4, 6, 10, 11, 12, 14, },
  { 1, 3, 4, 6, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 10, 11, 12, 14, },
  { 2, 3, 4, 6, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 12, 14, },
  { 5, 6, 10, 11, 12, 14, },
  { 0, 5, 6, 10, 11, 12, 14, },
  { 1, 5, 6, 10, 11, 12, 14, },
  { 0, 1, 5, 6, 10, 11, 12, 14, },
  { 2, 5, 6, 10, 11, 12, 14, },
  { 0, 2, 5, 6, 10, 11, 12, 14, },
  { 1, 2, 5, 6, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 10, 11, 12, 14, },
  { 3, 5, 6, 10, 11, 12, 14, },
  { 0, 3, 5, 6, 10, 11, 12, 14, },
  { 1, 3, 5, 6, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 10, 11, 12, 14, },
  { 2, 3, 5, 6, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 12, 14, },
  { 4, 5, 6, 10, 11, 12, 14, },
  { 0, 4, 5, 6, 10, 11, 12, 14, },
  { 1, 4, 5, 6, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 10, 11, 12, 14, },
  { 2, 4, 5, 6, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 12, 14, },
  { 3, 4, 5, 6, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 14, },
  { 7, 10, 11, 12, 14, },
  { 0, 7, 10, 11, 12, 14, },
  { 1, 7, 10, 11, 12, 14, },
  { 0, 1, 7, 10, 11, 12, 14, },
  { 2, 7, 10, 11, 12, 14, },
  { 0, 2, 7, 10, 11, 12, 14, },
  { 1, 2, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 7, 10, 11, 12, 14, },
  { 3, 7, 10, 11, 12, 14, },
  { 0, 3, 7, 10, 11, 12, 14, },
  { 1, 3, 7, 10, 11, 12, 14, },
  { 0, 1, 3, 7, 10, 11, 12, 14, },
  { 2, 3, 7, 10, 11, 12, 14, },
  { 0, 2, 3, 7, 10, 11, 12, 14, },
  { 1, 2, 3, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 7, 10, 11, 12, 14, },
  { 4, 7, 10, 11, 12, 14, },
  { 0, 4, 7, 10, 11, 12, 14, },
  { 1, 4, 7, 10, 11, 12, 14, },
  { 0, 1, 4, 7, 10, 11, 12, 14, },
  { 2, 4, 7, 10, 11, 12, 14, },
  { 0, 2, 4, 7, 10, 11, 12, 14, },
  { 1, 2, 4, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 7, 10, 11, 12, 14, },
  { 3, 4, 7, 10, 11, 12, 14, },
  { 0, 3, 4, 7, 10, 11, 12, 14, },
  { 1, 3, 4, 7, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 7, 10, 11, 12, 14, },
  { 2, 3, 4, 7, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 7, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 12, 14, },
  { 5, 7, 10, 11, 12, 14, },
  { 0, 5, 7, 10, 11, 12, 14, },
  { 1, 5, 7, 10, 11, 12, 14, },
  { 0, 1, 5, 7, 10, 11, 12, 14, },
  { 2, 5, 7, 10, 11, 12, 14, },
  { 0, 2, 5, 7, 10, 11, 12, 14, },
  { 1, 2, 5, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 7, 10, 11, 12, 14, },
  { 3, 5, 7, 10, 11, 12, 14, },
  { 0, 3, 5, 7, 10, 11, 12, 14, },
  { 1, 3, 5, 7, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 7, 10, 11, 12, 14, },
  { 2, 3, 5, 7, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 7, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 12, 14, },
  { 4, 5, 7, 10, 11, 12, 14, },
  { 0, 4, 5, 7, 10, 11, 12, 14, },
  { 1, 4, 5, 7, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 7, 10, 11, 12, 14, },
  { 2, 4, 5, 7, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 7, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 12, 14, },
  { 3, 4, 5, 7, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 7, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 7, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 7, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 12, 14, },
  { 6, 7, 10, 11, 12, 14, },
  { 0, 6, 7, 10, 11, 12, 14, },
  { 1, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 6, 7, 10, 11, 12, 14, },
  { 2, 6, 7, 10, 11, 12, 14, },
  { 0, 2, 6, 7, 10, 11, 12, 14, },
  { 1, 2, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 6, 7, 10, 11, 12, 14, },
  { 3, 6, 7, 10, 11, 12, 14, },
  { 0, 3, 6, 7, 10, 11, 12, 14, },
  { 1, 3, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 3, 6, 7, 10, 11, 12, 14, },
  { 2, 3, 6, 7, 10, 11, 12, 14, },
  { 0, 2, 3, 6, 7, 10, 11, 12, 14, },
  { 1, 2, 3, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 12, 14, },
  { 4, 6, 7, 10, 11, 12, 14, },
  { 0, 4, 6, 7, 10, 11, 12, 14, },
  { 1, 4, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 4, 6, 7, 10, 11, 12, 14, },
  { 2, 4, 6, 7, 10, 11, 12, 14, },
  { 0, 2, 4, 6, 7, 10, 11, 12, 14, },
  { 1, 2, 4, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 12, 14, },
  { 3, 4, 6, 7, 10, 11, 12, 14, },
  { 0, 3, 4, 6, 7, 10, 11, 12, 14, },
  { 1, 3, 4, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 12, 14, },
  { 2, 3, 4, 6, 7, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 12, 14, },
  { 5, 6, 7, 10, 11, 12, 14, },
  { 0, 5, 6, 7, 10, 11, 12, 14, },
  { 1, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 5, 6, 7, 10, 11, 12, 14, },
  { 2, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 2, 5, 6, 7, 10, 11, 12, 14, },
  { 1, 2, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 12, 14, },
  { 3, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 3, 5, 6, 7, 10, 11, 12, 14, },
  { 1, 3, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 12, 14, },
  { 2, 3, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 12, 14, },
  { 4, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 1, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 2, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 3, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 14, },
  { 8, 10, 11, 12, 14, },
  { 0, 8, 10, 11, 12, 14, },
  { 1, 8, 10, 11, 12, 14, },
  { 0, 1, 8, 10, 11, 12, 14, },
  { 2, 8, 10, 11, 12, 14, },
  { 0, 2, 8, 10, 11, 12, 14, },
  { 1, 2, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 8, 10, 11, 12, 14, },
  { 3, 8, 10, 11, 12, 14, },
  { 0, 3, 8, 10, 11, 12, 14, },
  { 1, 3, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 8, 10, 11, 12, 14, },
  { 2, 3, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 8, 10, 11, 12, 14, },
  { 4, 8, 10, 11, 12, 14, },
  { 0, 4, 8, 10, 11, 12, 14, },
  { 1, 4, 8, 10, 11, 12, 14, },
  { 0, 1, 4, 8, 10, 11, 12, 14, },
  { 2, 4, 8, 10, 11, 12, 14, },
  { 0, 2, 4, 8, 10, 11, 12, 14, },
  { 1, 2, 4, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 8, 10, 11, 12, 14, },
  { 3, 4, 8, 10, 11, 12, 14, },
  { 0, 3, 4, 8, 10, 11, 12, 14, },
  { 1, 3, 4, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 8, 10, 11, 12, 14, },
  { 2, 3, 4, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 12, 14, },
  { 5, 8, 10, 11, 12, 14, },
  { 0, 5, 8, 10, 11, 12, 14, },
  { 1, 5, 8, 10, 11, 12, 14, },
  { 0, 1, 5, 8, 10, 11, 12, 14, },
  { 2, 5, 8, 10, 11, 12, 14, },
  { 0, 2, 5, 8, 10, 11, 12, 14, },
  { 1, 2, 5, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 8, 10, 11, 12, 14, },
  { 3, 5, 8, 10, 11, 12, 14, },
  { 0, 3, 5, 8, 10, 11, 12, 14, },
  { 1, 3, 5, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 8, 10, 11, 12, 14, },
  { 2, 3, 5, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 12, 14, },
  { 4, 5, 8, 10, 11, 12, 14, },
  { 0, 4, 5, 8, 10, 11, 12, 14, },
  { 1, 4, 5, 8, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 8, 10, 11, 12, 14, },
  { 2, 4, 5, 8, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 8, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 12, 14, },
  { 3, 4, 5, 8, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 8, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 12, 14, },
  { 6, 8, 10, 11, 12, 14, },
  { 0, 6, 8, 10, 11, 12, 14, },
  { 1, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 6, 8, 10, 11, 12, 14, },
  { 2, 6, 8, 10, 11, 12, 14, },
  { 0, 2, 6, 8, 10, 11, 12, 14, },
  { 1, 2, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 6, 8, 10, 11, 12, 14, },
  { 3, 6, 8, 10, 11, 12, 14, },
  { 0, 3, 6, 8, 10, 11, 12, 14, },
  { 1, 3, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 6, 8, 10, 11, 12, 14, },
  { 2, 3, 6, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 6, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 12, 14, },
  { 4, 6, 8, 10, 11, 12, 14, },
  { 0, 4, 6, 8, 10, 11, 12, 14, },
  { 1, 4, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 4, 6, 8, 10, 11, 12, 14, },
  { 2, 4, 6, 8, 10, 11, 12, 14, },
  { 0, 2, 4, 6, 8, 10, 11, 12, 14, },
  { 1, 2, 4, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 12, 14, },
  { 3, 4, 6, 8, 10, 11, 12, 14, },
  { 0, 3, 4, 6, 8, 10, 11, 12, 14, },
  { 1, 3, 4, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 12, 14, },
  { 2, 3, 4, 6, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 12, 14, },
  { 5, 6, 8, 10, 11, 12, 14, },
  { 0, 5, 6, 8, 10, 11, 12, 14, },
  { 1, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 5, 6, 8, 10, 11, 12, 14, },
  { 2, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 2, 5, 6, 8, 10, 11, 12, 14, },
  { 1, 2, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 12, 14, },
  { 3, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 3, 5, 6, 8, 10, 11, 12, 14, },
  { 1, 3, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 12, 14, },
  { 2, 3, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 12, 14, },
  { 4, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 1, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 2, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 3, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 14, },
  { 7, 8, 10, 11, 12, 14, },
  { 0, 7, 8, 10, 11, 12, 14, },
  { 1, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 7, 8, 10, 11, 12, 14, },
  { 2, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 7, 8, 10, 11, 12, 14, },
  { 3, 7, 8, 10, 11, 12, 14, },
  { 0, 3, 7, 8, 10, 11, 12, 14, },
  { 1, 3, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 7, 8, 10, 11, 12, 14, },
  { 2, 3, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 12, 14, },
  { 4, 7, 8, 10, 11, 12, 14, },
  { 0, 4, 7, 8, 10, 11, 12, 14, },
  { 1, 4, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 4, 7, 8, 10, 11, 12, 14, },
  { 2, 4, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 4, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 4, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 12, 14, },
  { 3, 4, 7, 8, 10, 11, 12, 14, },
  { 0, 3, 4, 7, 8, 10, 11, 12, 14, },
  { 1, 3, 4, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 12, 14, },
  { 2, 3, 4, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 12, 14, },
  { 5, 7, 8, 10, 11, 12, 14, },
  { 0, 5, 7, 8, 10, 11, 12, 14, },
  { 1, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 5, 7, 8, 10, 11, 12, 14, },
  { 2, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 5, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 12, 14, },
  { 3, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 3, 5, 7, 8, 10, 11, 12, 14, },
  { 1, 3, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 12, 14, },
  { 2, 3, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 12, 14, },
  { 4, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 1, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 2, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 3, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 14, },
  { 6, 7, 8, 10, 11, 12, 14, },
  { 0, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 6, 7, 8, 10, 11, 12, 14, },
  { 2, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 12, 14, },
  { 3, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 3, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 3, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 12, 14, },
  { 2, 3, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 12, 14, },
  { 4, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 2, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 3, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 14, },
  { 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 2, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 3, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, },
  { 9, 10, 11, 12, 14, },
  { 0, 9, 10, 11, 12, 14, },
  { 1, 9, 10, 11, 12, 14, },
  { 0, 1, 9, 10, 11, 12, 14, },
  { 2, 9, 10, 11, 12, 14, },
  { 0, 2, 9, 10, 11, 12, 14, },
  { 1, 2, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 9, 10, 11, 12, 14, },
  { 3, 9, 10, 11, 12, 14, },
  { 0, 3, 9, 10, 11, 12, 14, },
  { 1, 3, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 9, 10, 11, 12, 14, },
  { 2, 3, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 9, 10, 11, 12, 14, },
  { 4, 9, 10, 11, 12, 14, },
  { 0, 4, 9, 10, 11, 12, 14, },
  { 1, 4, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 9, 10, 11, 12, 14, },
  { 2, 4, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 9, 10, 11, 12, 14, },
  { 3, 4, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 12, 14, },
  { 5, 9, 10, 11, 12, 14, },
  { 0, 5, 9, 10, 11, 12, 14, },
  { 1, 5, 9, 10, 11, 12, 14, },
  { 0, 1, 5, 9, 10, 11, 12, 14, },
  { 2, 5, 9, 10, 11, 12, 14, },
  { 0, 2, 5, 9, 10, 11, 12, 14, },
  { 1, 2, 5, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 9, 10, 11, 12, 14, },
  { 3, 5, 9, 10, 11, 12, 14, },
  { 0, 3, 5, 9, 10, 11, 12, 14, },
  { 1, 3, 5, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 9, 10, 11, 12, 14, },
  { 2, 3, 5, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 12, 14, },
  { 4, 5, 9, 10, 11, 12, 14, },
  { 0, 4, 5, 9, 10, 11, 12, 14, },
  { 1, 4, 5, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 9, 10, 11, 12, 14, },
  { 2, 4, 5, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 12, 14, },
  { 3, 4, 5, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, 14, },
  { 6, 9, 10, 11, 12, 14, },
  { 0, 6, 9, 10, 11, 12, 14, },
  { 1, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 6, 9, 10, 11, 12, 14, },
  { 2, 6, 9, 10, 11, 12, 14, },
  { 0, 2, 6, 9, 10, 11, 12, 14, },
  { 1, 2, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 6, 9, 10, 11, 12, 14, },
  { 3, 6, 9, 10, 11, 12, 14, },
  { 0, 3, 6, 9, 10, 11, 12, 14, },
  { 1, 3, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 6, 9, 10, 11, 12, 14, },
  { 2, 3, 6, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 6, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 12, 14, },
  { 4, 6, 9, 10, 11, 12, 14, },
  { 0, 4, 6, 9, 10, 11, 12, 14, },
  { 1, 4, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 6, 9, 10, 11, 12, 14, },
  { 2, 4, 6, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 6, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 12, 14, },
  { 3, 4, 6, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 6, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 6, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 12, 14, },
  { 5, 6, 9, 10, 11, 12, 14, },
  { 0, 5, 6, 9, 10, 11, 12, 14, },
  { 1, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 5, 6, 9, 10, 11, 12, 14, },
  { 2, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 2, 5, 6, 9, 10, 11, 12, 14, },
  { 1, 2, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 12, 14, },
  { 3, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 3, 5, 6, 9, 10, 11, 12, 14, },
  { 1, 3, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 12, 14, },
  { 2, 3, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 12, 14, },
  { 4, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 1, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 2, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 3, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 14, },
  { 7, 9, 10, 11, 12, 14, },
  { 0, 7, 9, 10, 11, 12, 14, },
  { 1, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 7, 9, 10, 11, 12, 14, },
  { 2, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 7, 9, 10, 11, 12, 14, },
  { 3, 7, 9, 10, 11, 12, 14, },
  { 0, 3, 7, 9, 10, 11, 12, 14, },
  { 1, 3, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 7, 9, 10, 11, 12, 14, },
  { 2, 3, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 12, 14, },
  { 4, 7, 9, 10, 11, 12, 14, },
  { 0, 4, 7, 9, 10, 11, 12, 14, },
  { 1, 4, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 7, 9, 10, 11, 12, 14, },
  { 2, 4, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 12, 14, },
  { 3, 4, 7, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 7, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 12, 14, },
  { 5, 7, 9, 10, 11, 12, 14, },
  { 0, 5, 7, 9, 10, 11, 12, 14, },
  { 1, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 5, 7, 9, 10, 11, 12, 14, },
  { 2, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 5, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 12, 14, },
  { 3, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 3, 5, 7, 9, 10, 11, 12, 14, },
  { 1, 3, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 12, 14, },
  { 2, 3, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 12, 14, },
  { 4, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 1, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 2, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 3, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 14, },
  { 6, 7, 9, 10, 11, 12, 14, },
  { 0, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 6, 7, 9, 10, 11, 12, 14, },
  { 2, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 12, 14, },
  { 3, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 3, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 3, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 12, 14, },
  { 2, 3, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 12, 14, },
  { 4, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 2, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 3, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 14, },
  { 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 2, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 3, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, },
  { 8, 9, 10, 11, 12, 14, },
  { 0, 8, 9, 10, 11, 12, 14, },
  { 1, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 8, 9, 10, 11, 12, 14, },
  { 2, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 8, 9, 10, 11, 12, 14, },
  { 3, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 12, 14, },
  { 4, 8, 9, 10, 11, 12, 14, },
  { 0, 4, 8, 9, 10, 11, 12, 14, },
  { 1, 4, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 8, 9, 10, 11, 12, 14, },
  { 2, 4, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 12, 14, },
  { 3, 4, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 12, 14, },
  { 5, 8, 9, 10, 11, 12, 14, },
  { 0, 5, 8, 9, 10, 11, 12, 14, },
  { 1, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 5, 8, 9, 10, 11, 12, 14, },
  { 2, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 5, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 12, 14, },
  { 3, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 5, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 12, 14, },
  { 4, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 1, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 2, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 3, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 14, },
  { 6, 8, 9, 10, 11, 12, 14, },
  { 0, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 6, 8, 9, 10, 11, 12, 14, },
  { 2, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 12, 14, },
  { 3, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 12, 14, },
  { 4, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 2, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 3, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 14, },
  { 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 2, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 3, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, },
  { 7, 8, 9, 10, 11, 12, 14, },
  { 0, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 12, 14, },
  { 3, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 12, 14, },
  { 4, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 3, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 14, },
  { 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 3, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, },
  { 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 3, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, },
  { 13, 14, },
  { 0, 13, 14, },
  { 1, 13, 14, },
  { 0, 1, 13, 14, },
  { 2, 13, 14, },
  { 0, 2, 13, 14, },
  { 1, 2, 13, 14, },
  { 0, 1, 2, 13, 14, },
  { 3, 13, 14, },
  { 0, 3, 13, 14, },
  { 1, 3, 13, 14, },
  { 0, 1, 3, 13, 14, },
  { 2, 3, 13, 14, },
  { 0, 2, 3, 13, 14, },
  { 1, 2, 3, 13, 14, },
  { 0, 1, 2, 3, 13, 14, },
  { 4, 13, 14, },
  { 0, 4, 13, 14, },
  { 1, 4, 13, 14, },
  { 0, 1, 4, 13, 14, },
  { 2, 4, 13, 14, },
  { 0, 2, 4, 13, 14, },
  { 1, 2, 4, 13, 14, },
  { 0, 1, 2, 4, 13, 14, },
  { 3, 4, 13, 14, },
  { 0, 3, 4, 13, 14, },
  { 1, 3, 4, 13, 14, },
  { 0, 1, 3, 4, 13, 14, },
  { 2, 3, 4, 13, 14, },
  { 0, 2, 3, 4, 13, 14, },
  { 1, 2, 3, 4, 13, 14, },
  { 0, 1, 2, 3, 4, 13, 14, },
  { 5, 13, 14, },
  { 0, 5, 13, 14, },
  { 1, 5, 13, 14, },
  { 0, 1, 5, 13, 14, },
  { 2, 5, 13, 14, },
  { 0, 2, 5, 13, 14, },
  { 1, 2, 5, 13, 14, },
  { 0, 1, 2, 5, 13, 14, },
  { 3, 5, 13, 14, },
  { 0, 3, 5, 13, 14, },
  { 1, 3, 5, 13, 14, },
  { 0, 1, 3, 5, 13, 14, },
  { 2, 3, 5, 13, 14, },
  { 0, 2, 3, 5, 13, 14, },
  { 1, 2, 3, 5, 13, 14, },
  { 0, 1, 2, 3, 5, 13, 14, },
  { 4, 5, 13, 14, },
  { 0, 4, 5, 13, 14, },
  { 1, 4, 5, 13, 14, },
  { 0, 1, 4, 5, 13, 14, },
  { 2, 4, 5, 13, 14, },
  { 0, 2, 4, 5, 13, 14, },
  { 1, 2, 4, 5, 13, 14, },
  { 0, 1, 2, 4, 5, 13, 14, },
  { 3, 4, 5, 13, 14, },
  { 0, 3, 4, 5, 13, 14, },
  { 1, 3, 4, 5, 13, 14, },
  { 0, 1, 3, 4, 5, 13, 14, },
  { 2, 3, 4, 5, 13, 14, },
  { 0, 2, 3, 4, 5, 13, 14, },
  { 1, 2, 3, 4, 5, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 13, 14, },
  { 6, 13, 14, },
  { 0, 6, 13, 14, },
  { 1, 6, 13, 14, },
  { 0, 1, 6, 13, 14, },
  { 2, 6, 13, 14, },
  { 0, 2, 6, 13, 14, },
  { 1, 2, 6, 13, 14, },
  { 0, 1, 2, 6, 13, 14, },
  { 3, 6, 13, 14, },
  { 0, 3, 6, 13, 14, },
  { 1, 3, 6, 13, 14, },
  { 0, 1, 3, 6, 13, 14, },
  { 2, 3, 6, 13, 14, },
  { 0, 2, 3, 6, 13, 14, },
  { 1, 2, 3, 6, 13, 14, },
  { 0, 1, 2, 3, 6, 13, 14, },
  { 4, 6, 13, 14, },
  { 0, 4, 6, 13, 14, },
  { 1, 4, 6, 13, 14, },
  { 0, 1, 4, 6, 13, 14, },
  { 2, 4, 6, 13, 14, },
  { 0, 2, 4, 6, 13, 14, },
  { 1, 2, 4, 6, 13, 14, },
  { 0, 1, 2, 4, 6, 13, 14, },
  { 3, 4, 6, 13, 14, },
  { 0, 3, 4, 6, 13, 14, },
  { 1, 3, 4, 6, 13, 14, },
  { 0, 1, 3, 4, 6, 13, 14, },
  { 2, 3, 4, 6, 13, 14, },
  { 0, 2, 3, 4, 6, 13, 14, },
  { 1, 2, 3, 4, 6, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 13, 14, },
  { 5, 6, 13, 14, },
  { 0, 5, 6, 13, 14, },
  { 1, 5, 6, 13, 14, },
  { 0, 1, 5, 6, 13, 14, },
  { 2, 5, 6, 13, 14, },
  { 0, 2, 5, 6, 13, 14, },
  { 1, 2, 5, 6, 13, 14, },
  { 0, 1, 2, 5, 6, 13, 14, },
  { 3, 5, 6, 13, 14, },
  { 0, 3, 5, 6, 13, 14, },
  { 1, 3, 5, 6, 13, 14, },
  { 0, 1, 3, 5, 6, 13, 14, },
  { 2, 3, 5, 6, 13, 14, },
  { 0, 2, 3, 5, 6, 13, 14, },
  { 1, 2, 3, 5, 6, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 13, 14, },
  { 4, 5, 6, 13, 14, },
  { 0, 4, 5, 6, 13, 14, },
  { 1, 4, 5, 6, 13, 14, },
  { 0, 1, 4, 5, 6, 13, 14, },
  { 2, 4, 5, 6, 13, 14, },
  { 0, 2, 4, 5, 6, 13, 14, },
  { 1, 2, 4, 5, 6, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 13, 14, },
  { 3, 4, 5, 6, 13, 14, },
  { 0, 3, 4, 5, 6, 13, 14, },
  { 1, 3, 4, 5, 6, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 13, 14, },
  { 2, 3, 4, 5, 6, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 13, 14, },
  { 7, 13, 14, },
  { 0, 7, 13, 14, },
  { 1, 7, 13, 14, },
  { 0, 1, 7, 13, 14, },
  { 2, 7, 13, 14, },
  { 0, 2, 7, 13, 14, },
  { 1, 2, 7, 13, 14, },
  { 0, 1, 2, 7, 13, 14, },
  { 3, 7, 13, 14, },
  { 0, 3, 7, 13, 14, },
  { 1, 3, 7, 13, 14, },
  { 0, 1, 3, 7, 13, 14, },
  { 2, 3, 7, 13, 14, },
  { 0, 2, 3, 7, 13, 14, },
  { 1, 2, 3, 7, 13, 14, },
  { 0, 1, 2, 3, 7, 13, 14, },
  { 4, 7, 13, 14, },
  { 0, 4, 7, 13, 14, },
  { 1, 4, 7, 13, 14, },
  { 0, 1, 4, 7, 13, 14, },
  { 2, 4, 7, 13, 14, },
  { 0, 2, 4, 7, 13, 14, },
  { 1, 2, 4, 7, 13, 14, },
  { 0, 1, 2, 4, 7, 13, 14, },
  { 3, 4, 7, 13, 14, },
  { 0, 3, 4, 7, 13, 14, },
  { 1, 3, 4, 7, 13, 14, },
  { 0, 1, 3, 4, 7, 13, 14, },
  { 2, 3, 4, 7, 13, 14, },
  { 0, 2, 3, 4, 7, 13, 14, },
  { 1, 2, 3, 4, 7, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 13, 14, },
  { 5, 7, 13, 14, },
  { 0, 5, 7, 13, 14, },
  { 1, 5, 7, 13, 14, },
  { 0, 1, 5, 7, 13, 14, },
  { 2, 5, 7, 13, 14, },
  { 0, 2, 5, 7, 13, 14, },
  { 1, 2, 5, 7, 13, 14, },
  { 0, 1, 2, 5, 7, 13, 14, },
  { 3, 5, 7, 13, 14, },
  { 0, 3, 5, 7, 13, 14, },
  { 1, 3, 5, 7, 13, 14, },
  { 0, 1, 3, 5, 7, 13, 14, },
  { 2, 3, 5, 7, 13, 14, },
  { 0, 2, 3, 5, 7, 13, 14, },
  { 1, 2, 3, 5, 7, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 13, 14, },
  { 4, 5, 7, 13, 14, },
  { 0, 4, 5, 7, 13, 14, },
  { 1, 4, 5, 7, 13, 14, },
  { 0, 1, 4, 5, 7, 13, 14, },
  { 2, 4, 5, 7, 13, 14, },
  { 0, 2, 4, 5, 7, 13, 14, },
  { 1, 2, 4, 5, 7, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 13, 14, },
  { 3, 4, 5, 7, 13, 14, },
  { 0, 3, 4, 5, 7, 13, 14, },
  { 1, 3, 4, 5, 7, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 13, 14, },
  { 2, 3, 4, 5, 7, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 13, 14, },
  { 6, 7, 13, 14, },
  { 0, 6, 7, 13, 14, },
  { 1, 6, 7, 13, 14, },
  { 0, 1, 6, 7, 13, 14, },
  { 2, 6, 7, 13, 14, },
  { 0, 2, 6, 7, 13, 14, },
  { 1, 2, 6, 7, 13, 14, },
  { 0, 1, 2, 6, 7, 13, 14, },
  { 3, 6, 7, 13, 14, },
  { 0, 3, 6, 7, 13, 14, },
  { 1, 3, 6, 7, 13, 14, },
  { 0, 1, 3, 6, 7, 13, 14, },
  { 2, 3, 6, 7, 13, 14, },
  { 0, 2, 3, 6, 7, 13, 14, },
  { 1, 2, 3, 6, 7, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 13, 14, },
  { 4, 6, 7, 13, 14, },
  { 0, 4, 6, 7, 13, 14, },
  { 1, 4, 6, 7, 13, 14, },
  { 0, 1, 4, 6, 7, 13, 14, },
  { 2, 4, 6, 7, 13, 14, },
  { 0, 2, 4, 6, 7, 13, 14, },
  { 1, 2, 4, 6, 7, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 13, 14, },
  { 3, 4, 6, 7, 13, 14, },
  { 0, 3, 4, 6, 7, 13, 14, },
  { 1, 3, 4, 6, 7, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 13, 14, },
  { 2, 3, 4, 6, 7, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 13, 14, },
  { 5, 6, 7, 13, 14, },
  { 0, 5, 6, 7, 13, 14, },
  { 1, 5, 6, 7, 13, 14, },
  { 0, 1, 5, 6, 7, 13, 14, },
  { 2, 5, 6, 7, 13, 14, },
  { 0, 2, 5, 6, 7, 13, 14, },
  { 1, 2, 5, 6, 7, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 13, 14, },
  { 3, 5, 6, 7, 13, 14, },
  { 0, 3, 5, 6, 7, 13, 14, },
  { 1, 3, 5, 6, 7, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 13, 14, },
  { 2, 3, 5, 6, 7, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 13, 14, },
  { 4, 5, 6, 7, 13, 14, },
  { 0, 4, 5, 6, 7, 13, 14, },
  { 1, 4, 5, 6, 7, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 13, 14, },
  { 2, 4, 5, 6, 7, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 13, 14, },
  { 3, 4, 5, 6, 7, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 13, 14, },
  { 8, 13, 14, },
  { 0, 8, 13, 14, },
  { 1, 8, 13, 14, },
  { 0, 1, 8, 13, 14, },
  { 2, 8, 13, 14, },
  { 0, 2, 8, 13, 14, },
  { 1, 2, 8, 13, 14, },
  { 0, 1, 2, 8, 13, 14, },
  { 3, 8, 13, 14, },
  { 0, 3, 8, 13, 14, },
  { 1, 3, 8, 13, 14, },
  { 0, 1, 3, 8, 13, 14, },
  { 2, 3, 8, 13, 14, },
  { 0, 2, 3, 8, 13, 14, },
  { 1, 2, 3, 8, 13, 14, },
  { 0, 1, 2, 3, 8, 13, 14, },
  { 4, 8, 13, 14, },
  { 0, 4, 8, 13, 14, },
  { 1, 4, 8, 13, 14, },
  { 0, 1, 4, 8, 13, 14, },
  { 2, 4, 8, 13, 14, },
  { 0, 2, 4, 8, 13, 14, },
  { 1, 2, 4, 8, 13, 14, },
  { 0, 1, 2, 4, 8, 13, 14, },
  { 3, 4, 8, 13, 14, },
  { 0, 3, 4, 8, 13, 14, },
  { 1, 3, 4, 8, 13, 14, },
  { 0, 1, 3, 4, 8, 13, 14, },
  { 2, 3, 4, 8, 13, 14, },
  { 0, 2, 3, 4, 8, 13, 14, },
  { 1, 2, 3, 4, 8, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 13, 14, },
  { 5, 8, 13, 14, },
  { 0, 5, 8, 13, 14, },
  { 1, 5, 8, 13, 14, },
  { 0, 1, 5, 8, 13, 14, },
  { 2, 5, 8, 13, 14, },
  { 0, 2, 5, 8, 13, 14, },
  { 1, 2, 5, 8, 13, 14, },
  { 0, 1, 2, 5, 8, 13, 14, },
  { 3, 5, 8, 13, 14, },
  { 0, 3, 5, 8, 13, 14, },
  { 1, 3, 5, 8, 13, 14, },
  { 0, 1, 3, 5, 8, 13, 14, },
  { 2, 3, 5, 8, 13, 14, },
  { 0, 2, 3, 5, 8, 13, 14, },
  { 1, 2, 3, 5, 8, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 13, 14, },
  { 4, 5, 8, 13, 14, },
  { 0, 4, 5, 8, 13, 14, },
  { 1, 4, 5, 8, 13, 14, },
  { 0, 1, 4, 5, 8, 13, 14, },
  { 2, 4, 5, 8, 13, 14, },
  { 0, 2, 4, 5, 8, 13, 14, },
  { 1, 2, 4, 5, 8, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 13, 14, },
  { 3, 4, 5, 8, 13, 14, },
  { 0, 3, 4, 5, 8, 13, 14, },
  { 1, 3, 4, 5, 8, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 13, 14, },
  { 2, 3, 4, 5, 8, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 13, 14, },
  { 6, 8, 13, 14, },
  { 0, 6, 8, 13, 14, },
  { 1, 6, 8, 13, 14, },
  { 0, 1, 6, 8, 13, 14, },
  { 2, 6, 8, 13, 14, },
  { 0, 2, 6, 8, 13, 14, },
  { 1, 2, 6, 8, 13, 14, },
  { 0, 1, 2, 6, 8, 13, 14, },
  { 3, 6, 8, 13, 14, },
  { 0, 3, 6, 8, 13, 14, },
  { 1, 3, 6, 8, 13, 14, },
  { 0, 1, 3, 6, 8, 13, 14, },
  { 2, 3, 6, 8, 13, 14, },
  { 0, 2, 3, 6, 8, 13, 14, },
  { 1, 2, 3, 6, 8, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 13, 14, },
  { 4, 6, 8, 13, 14, },
  { 0, 4, 6, 8, 13, 14, },
  { 1, 4, 6, 8, 13, 14, },
  { 0, 1, 4, 6, 8, 13, 14, },
  { 2, 4, 6, 8, 13, 14, },
  { 0, 2, 4, 6, 8, 13, 14, },
  { 1, 2, 4, 6, 8, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 13, 14, },
  { 3, 4, 6, 8, 13, 14, },
  { 0, 3, 4, 6, 8, 13, 14, },
  { 1, 3, 4, 6, 8, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 13, 14, },
  { 2, 3, 4, 6, 8, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 13, 14, },
  { 5, 6, 8, 13, 14, },
  { 0, 5, 6, 8, 13, 14, },
  { 1, 5, 6, 8, 13, 14, },
  { 0, 1, 5, 6, 8, 13, 14, },
  { 2, 5, 6, 8, 13, 14, },
  { 0, 2, 5, 6, 8, 13, 14, },
  { 1, 2, 5, 6, 8, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 13, 14, },
  { 3, 5, 6, 8, 13, 14, },
  { 0, 3, 5, 6, 8, 13, 14, },
  { 1, 3, 5, 6, 8, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 13, 14, },
  { 2, 3, 5, 6, 8, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 13, 14, },
  { 4, 5, 6, 8, 13, 14, },
  { 0, 4, 5, 6, 8, 13, 14, },
  { 1, 4, 5, 6, 8, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 13, 14, },
  { 2, 4, 5, 6, 8, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 13, 14, },
  { 3, 4, 5, 6, 8, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 13, 14, },
  { 7, 8, 13, 14, },
  { 0, 7, 8, 13, 14, },
  { 1, 7, 8, 13, 14, },
  { 0, 1, 7, 8, 13, 14, },
  { 2, 7, 8, 13, 14, },
  { 0, 2, 7, 8, 13, 14, },
  { 1, 2, 7, 8, 13, 14, },
  { 0, 1, 2, 7, 8, 13, 14, },
  { 3, 7, 8, 13, 14, },
  { 0, 3, 7, 8, 13, 14, },
  { 1, 3, 7, 8, 13, 14, },
  { 0, 1, 3, 7, 8, 13, 14, },
  { 2, 3, 7, 8, 13, 14, },
  { 0, 2, 3, 7, 8, 13, 14, },
  { 1, 2, 3, 7, 8, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 13, 14, },
  { 4, 7, 8, 13, 14, },
  { 0, 4, 7, 8, 13, 14, },
  { 1, 4, 7, 8, 13, 14, },
  { 0, 1, 4, 7, 8, 13, 14, },
  { 2, 4, 7, 8, 13, 14, },
  { 0, 2, 4, 7, 8, 13, 14, },
  { 1, 2, 4, 7, 8, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 13, 14, },
  { 3, 4, 7, 8, 13, 14, },
  { 0, 3, 4, 7, 8, 13, 14, },
  { 1, 3, 4, 7, 8, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 13, 14, },
  { 2, 3, 4, 7, 8, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 13, 14, },
  { 5, 7, 8, 13, 14, },
  { 0, 5, 7, 8, 13, 14, },
  { 1, 5, 7, 8, 13, 14, },
  { 0, 1, 5, 7, 8, 13, 14, },
  { 2, 5, 7, 8, 13, 14, },
  { 0, 2, 5, 7, 8, 13, 14, },
  { 1, 2, 5, 7, 8, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 13, 14, },
  { 3, 5, 7, 8, 13, 14, },
  { 0, 3, 5, 7, 8, 13, 14, },
  { 1, 3, 5, 7, 8, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 13, 14, },
  { 2, 3, 5, 7, 8, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 13, 14, },
  { 4, 5, 7, 8, 13, 14, },
  { 0, 4, 5, 7, 8, 13, 14, },
  { 1, 4, 5, 7, 8, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 13, 14, },
  { 2, 4, 5, 7, 8, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 13, 14, },
  { 3, 4, 5, 7, 8, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 13, 14, },
  { 6, 7, 8, 13, 14, },
  { 0, 6, 7, 8, 13, 14, },
  { 1, 6, 7, 8, 13, 14, },
  { 0, 1, 6, 7, 8, 13, 14, },
  { 2, 6, 7, 8, 13, 14, },
  { 0, 2, 6, 7, 8, 13, 14, },
  { 1, 2, 6, 7, 8, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 13, 14, },
  { 3, 6, 7, 8, 13, 14, },
  { 0, 3, 6, 7, 8, 13, 14, },
  { 1, 3, 6, 7, 8, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 13, 14, },
  { 2, 3, 6, 7, 8, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 13, 14, },
  { 4, 6, 7, 8, 13, 14, },
  { 0, 4, 6, 7, 8, 13, 14, },
  { 1, 4, 6, 7, 8, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 13, 14, },
  { 2, 4, 6, 7, 8, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 13, 14, },
  { 3, 4, 6, 7, 8, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 13, 14, },
  { 5, 6, 7, 8, 13, 14, },
  { 0, 5, 6, 7, 8, 13, 14, },
  { 1, 5, 6, 7, 8, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 13, 14, },
  { 2, 5, 6, 7, 8, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 13, 14, },
  { 3, 5, 6, 7, 8, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 13, 14, },
  { 4, 5, 6, 7, 8, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 13, 14, },
  { 9, 13, 14, },
  { 0, 9, 13, 14, },
  { 1, 9, 13, 14, },
  { 0, 1, 9, 13, 14, },
  { 2, 9, 13, 14, },
  { 0, 2, 9, 13, 14, },
  { 1, 2, 9, 13, 14, },
  { 0, 1, 2, 9, 13, 14, },
  { 3, 9, 13, 14, },
  { 0, 3, 9, 13, 14, },
  { 1, 3, 9, 13, 14, },
  { 0, 1, 3, 9, 13, 14, },
  { 2, 3, 9, 13, 14, },
  { 0, 2, 3, 9, 13, 14, },
  { 1, 2, 3, 9, 13, 14, },
  { 0, 1, 2, 3, 9, 13, 14, },
  { 4, 9, 13, 14, },
  { 0, 4, 9, 13, 14, },
  { 1, 4, 9, 13, 14, },
  { 0, 1, 4, 9, 13, 14, },
  { 2, 4, 9, 13, 14, },
  { 0, 2, 4, 9, 13, 14, },
  { 1, 2, 4, 9, 13, 14, },
  { 0, 1, 2, 4, 9, 13, 14, },
  { 3, 4, 9, 13, 14, },
  { 0, 3, 4, 9, 13, 14, },
  { 1, 3, 4, 9, 13, 14, },
  { 0, 1, 3, 4, 9, 13, 14, },
  { 2, 3, 4, 9, 13, 14, },
  { 0, 2, 3, 4, 9, 13, 14, },
  { 1, 2, 3, 4, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 9, 13, 14, },
  { 5, 9, 13, 14, },
  { 0, 5, 9, 13, 14, },
  { 1, 5, 9, 13, 14, },
  { 0, 1, 5, 9, 13, 14, },
  { 2, 5, 9, 13, 14, },
  { 0, 2, 5, 9, 13, 14, },
  { 1, 2, 5, 9, 13, 14, },
  { 0, 1, 2, 5, 9, 13, 14, },
  { 3, 5, 9, 13, 14, },
  { 0, 3, 5, 9, 13, 14, },
  { 1, 3, 5, 9, 13, 14, },
  { 0, 1, 3, 5, 9, 13, 14, },
  { 2, 3, 5, 9, 13, 14, },
  { 0, 2, 3, 5, 9, 13, 14, },
  { 1, 2, 3, 5, 9, 13, 14, },
  { 0, 1, 2, 3, 5, 9, 13, 14, },
  { 4, 5, 9, 13, 14, },
  { 0, 4, 5, 9, 13, 14, },
  { 1, 4, 5, 9, 13, 14, },
  { 0, 1, 4, 5, 9, 13, 14, },
  { 2, 4, 5, 9, 13, 14, },
  { 0, 2, 4, 5, 9, 13, 14, },
  { 1, 2, 4, 5, 9, 13, 14, },
  { 0, 1, 2, 4, 5, 9, 13, 14, },
  { 3, 4, 5, 9, 13, 14, },
  { 0, 3, 4, 5, 9, 13, 14, },
  { 1, 3, 4, 5, 9, 13, 14, },
  { 0, 1, 3, 4, 5, 9, 13, 14, },
  { 2, 3, 4, 5, 9, 13, 14, },
  { 0, 2, 3, 4, 5, 9, 13, 14, },
  { 1, 2, 3, 4, 5, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 13, 14, },
  { 6, 9, 13, 14, },
  { 0, 6, 9, 13, 14, },
  { 1, 6, 9, 13, 14, },
  { 0, 1, 6, 9, 13, 14, },
  { 2, 6, 9, 13, 14, },
  { 0, 2, 6, 9, 13, 14, },
  { 1, 2, 6, 9, 13, 14, },
  { 0, 1, 2, 6, 9, 13, 14, },
  { 3, 6, 9, 13, 14, },
  { 0, 3, 6, 9, 13, 14, },
  { 1, 3, 6, 9, 13, 14, },
  { 0, 1, 3, 6, 9, 13, 14, },
  { 2, 3, 6, 9, 13, 14, },
  { 0, 2, 3, 6, 9, 13, 14, },
  { 1, 2, 3, 6, 9, 13, 14, },
  { 0, 1, 2, 3, 6, 9, 13, 14, },
  { 4, 6, 9, 13, 14, },
  { 0, 4, 6, 9, 13, 14, },
  { 1, 4, 6, 9, 13, 14, },
  { 0, 1, 4, 6, 9, 13, 14, },
  { 2, 4, 6, 9, 13, 14, },
  { 0, 2, 4, 6, 9, 13, 14, },
  { 1, 2, 4, 6, 9, 13, 14, },
  { 0, 1, 2, 4, 6, 9, 13, 14, },
  { 3, 4, 6, 9, 13, 14, },
  { 0, 3, 4, 6, 9, 13, 14, },
  { 1, 3, 4, 6, 9, 13, 14, },
  { 0, 1, 3, 4, 6, 9, 13, 14, },
  { 2, 3, 4, 6, 9, 13, 14, },
  { 0, 2, 3, 4, 6, 9, 13, 14, },
  { 1, 2, 3, 4, 6, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 13, 14, },
  { 5, 6, 9, 13, 14, },
  { 0, 5, 6, 9, 13, 14, },
  { 1, 5, 6, 9, 13, 14, },
  { 0, 1, 5, 6, 9, 13, 14, },
  { 2, 5, 6, 9, 13, 14, },
  { 0, 2, 5, 6, 9, 13, 14, },
  { 1, 2, 5, 6, 9, 13, 14, },
  { 0, 1, 2, 5, 6, 9, 13, 14, },
  { 3, 5, 6, 9, 13, 14, },
  { 0, 3, 5, 6, 9, 13, 14, },
  { 1, 3, 5, 6, 9, 13, 14, },
  { 0, 1, 3, 5, 6, 9, 13, 14, },
  { 2, 3, 5, 6, 9, 13, 14, },
  { 0, 2, 3, 5, 6, 9, 13, 14, },
  { 1, 2, 3, 5, 6, 9, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 13, 14, },
  { 4, 5, 6, 9, 13, 14, },
  { 0, 4, 5, 6, 9, 13, 14, },
  { 1, 4, 5, 6, 9, 13, 14, },
  { 0, 1, 4, 5, 6, 9, 13, 14, },
  { 2, 4, 5, 6, 9, 13, 14, },
  { 0, 2, 4, 5, 6, 9, 13, 14, },
  { 1, 2, 4, 5, 6, 9, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 13, 14, },
  { 3, 4, 5, 6, 9, 13, 14, },
  { 0, 3, 4, 5, 6, 9, 13, 14, },
  { 1, 3, 4, 5, 6, 9, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 13, 14, },
  { 2, 3, 4, 5, 6, 9, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 13, 14, },
  { 7, 9, 13, 14, },
  { 0, 7, 9, 13, 14, },
  { 1, 7, 9, 13, 14, },
  { 0, 1, 7, 9, 13, 14, },
  { 2, 7, 9, 13, 14, },
  { 0, 2, 7, 9, 13, 14, },
  { 1, 2, 7, 9, 13, 14, },
  { 0, 1, 2, 7, 9, 13, 14, },
  { 3, 7, 9, 13, 14, },
  { 0, 3, 7, 9, 13, 14, },
  { 1, 3, 7, 9, 13, 14, },
  { 0, 1, 3, 7, 9, 13, 14, },
  { 2, 3, 7, 9, 13, 14, },
  { 0, 2, 3, 7, 9, 13, 14, },
  { 1, 2, 3, 7, 9, 13, 14, },
  { 0, 1, 2, 3, 7, 9, 13, 14, },
  { 4, 7, 9, 13, 14, },
  { 0, 4, 7, 9, 13, 14, },
  { 1, 4, 7, 9, 13, 14, },
  { 0, 1, 4, 7, 9, 13, 14, },
  { 2, 4, 7, 9, 13, 14, },
  { 0, 2, 4, 7, 9, 13, 14, },
  { 1, 2, 4, 7, 9, 13, 14, },
  { 0, 1, 2, 4, 7, 9, 13, 14, },
  { 3, 4, 7, 9, 13, 14, },
  { 0, 3, 4, 7, 9, 13, 14, },
  { 1, 3, 4, 7, 9, 13, 14, },
  { 0, 1, 3, 4, 7, 9, 13, 14, },
  { 2, 3, 4, 7, 9, 13, 14, },
  { 0, 2, 3, 4, 7, 9, 13, 14, },
  { 1, 2, 3, 4, 7, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 13, 14, },
  { 5, 7, 9, 13, 14, },
  { 0, 5, 7, 9, 13, 14, },
  { 1, 5, 7, 9, 13, 14, },
  { 0, 1, 5, 7, 9, 13, 14, },
  { 2, 5, 7, 9, 13, 14, },
  { 0, 2, 5, 7, 9, 13, 14, },
  { 1, 2, 5, 7, 9, 13, 14, },
  { 0, 1, 2, 5, 7, 9, 13, 14, },
  { 3, 5, 7, 9, 13, 14, },
  { 0, 3, 5, 7, 9, 13, 14, },
  { 1, 3, 5, 7, 9, 13, 14, },
  { 0, 1, 3, 5, 7, 9, 13, 14, },
  { 2, 3, 5, 7, 9, 13, 14, },
  { 0, 2, 3, 5, 7, 9, 13, 14, },
  { 1, 2, 3, 5, 7, 9, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 13, 14, },
  { 4, 5, 7, 9, 13, 14, },
  { 0, 4, 5, 7, 9, 13, 14, },
  { 1, 4, 5, 7, 9, 13, 14, },
  { 0, 1, 4, 5, 7, 9, 13, 14, },
  { 2, 4, 5, 7, 9, 13, 14, },
  { 0, 2, 4, 5, 7, 9, 13, 14, },
  { 1, 2, 4, 5, 7, 9, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 13, 14, },
  { 3, 4, 5, 7, 9, 13, 14, },
  { 0, 3, 4, 5, 7, 9, 13, 14, },
  { 1, 3, 4, 5, 7, 9, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 13, 14, },
  { 2, 3, 4, 5, 7, 9, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 13, 14, },
  { 6, 7, 9, 13, 14, },
  { 0, 6, 7, 9, 13, 14, },
  { 1, 6, 7, 9, 13, 14, },
  { 0, 1, 6, 7, 9, 13, 14, },
  { 2, 6, 7, 9, 13, 14, },
  { 0, 2, 6, 7, 9, 13, 14, },
  { 1, 2, 6, 7, 9, 13, 14, },
  { 0, 1, 2, 6, 7, 9, 13, 14, },
  { 3, 6, 7, 9, 13, 14, },
  { 0, 3, 6, 7, 9, 13, 14, },
  { 1, 3, 6, 7, 9, 13, 14, },
  { 0, 1, 3, 6, 7, 9, 13, 14, },
  { 2, 3, 6, 7, 9, 13, 14, },
  { 0, 2, 3, 6, 7, 9, 13, 14, },
  { 1, 2, 3, 6, 7, 9, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 13, 14, },
  { 4, 6, 7, 9, 13, 14, },
  { 0, 4, 6, 7, 9, 13, 14, },
  { 1, 4, 6, 7, 9, 13, 14, },
  { 0, 1, 4, 6, 7, 9, 13, 14, },
  { 2, 4, 6, 7, 9, 13, 14, },
  { 0, 2, 4, 6, 7, 9, 13, 14, },
  { 1, 2, 4, 6, 7, 9, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 13, 14, },
  { 3, 4, 6, 7, 9, 13, 14, },
  { 0, 3, 4, 6, 7, 9, 13, 14, },
  { 1, 3, 4, 6, 7, 9, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 13, 14, },
  { 2, 3, 4, 6, 7, 9, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 13, 14, },
  { 5, 6, 7, 9, 13, 14, },
  { 0, 5, 6, 7, 9, 13, 14, },
  { 1, 5, 6, 7, 9, 13, 14, },
  { 0, 1, 5, 6, 7, 9, 13, 14, },
  { 2, 5, 6, 7, 9, 13, 14, },
  { 0, 2, 5, 6, 7, 9, 13, 14, },
  { 1, 2, 5, 6, 7, 9, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 13, 14, },
  { 3, 5, 6, 7, 9, 13, 14, },
  { 0, 3, 5, 6, 7, 9, 13, 14, },
  { 1, 3, 5, 6, 7, 9, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 13, 14, },
  { 2, 3, 5, 6, 7, 9, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 13, 14, },
  { 4, 5, 6, 7, 9, 13, 14, },
  { 0, 4, 5, 6, 7, 9, 13, 14, },
  { 1, 4, 5, 6, 7, 9, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 13, 14, },
  { 2, 4, 5, 6, 7, 9, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 13, 14, },
  { 3, 4, 5, 6, 7, 9, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 13, 14, },
  { 8, 9, 13, 14, },
  { 0, 8, 9, 13, 14, },
  { 1, 8, 9, 13, 14, },
  { 0, 1, 8, 9, 13, 14, },
  { 2, 8, 9, 13, 14, },
  { 0, 2, 8, 9, 13, 14, },
  { 1, 2, 8, 9, 13, 14, },
  { 0, 1, 2, 8, 9, 13, 14, },
  { 3, 8, 9, 13, 14, },
  { 0, 3, 8, 9, 13, 14, },
  { 1, 3, 8, 9, 13, 14, },
  { 0, 1, 3, 8, 9, 13, 14, },
  { 2, 3, 8, 9, 13, 14, },
  { 0, 2, 3, 8, 9, 13, 14, },
  { 1, 2, 3, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 8, 9, 13, 14, },
  { 4, 8, 9, 13, 14, },
  { 0, 4, 8, 9, 13, 14, },
  { 1, 4, 8, 9, 13, 14, },
  { 0, 1, 4, 8, 9, 13, 14, },
  { 2, 4, 8, 9, 13, 14, },
  { 0, 2, 4, 8, 9, 13, 14, },
  { 1, 2, 4, 8, 9, 13, 14, },
  { 0, 1, 2, 4, 8, 9, 13, 14, },
  { 3, 4, 8, 9, 13, 14, },
  { 0, 3, 4, 8, 9, 13, 14, },
  { 1, 3, 4, 8, 9, 13, 14, },
  { 0, 1, 3, 4, 8, 9, 13, 14, },
  { 2, 3, 4, 8, 9, 13, 14, },
  { 0, 2, 3, 4, 8, 9, 13, 14, },
  { 1, 2, 3, 4, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 13, 14, },
  { 5, 8, 9, 13, 14, },
  { 0, 5, 8, 9, 13, 14, },
  { 1, 5, 8, 9, 13, 14, },
  { 0, 1, 5, 8, 9, 13, 14, },
  { 2, 5, 8, 9, 13, 14, },
  { 0, 2, 5, 8, 9, 13, 14, },
  { 1, 2, 5, 8, 9, 13, 14, },
  { 0, 1, 2, 5, 8, 9, 13, 14, },
  { 3, 5, 8, 9, 13, 14, },
  { 0, 3, 5, 8, 9, 13, 14, },
  { 1, 3, 5, 8, 9, 13, 14, },
  { 0, 1, 3, 5, 8, 9, 13, 14, },
  { 2, 3, 5, 8, 9, 13, 14, },
  { 0, 2, 3, 5, 8, 9, 13, 14, },
  { 1, 2, 3, 5, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 13, 14, },
  { 4, 5, 8, 9, 13, 14, },
  { 0, 4, 5, 8, 9, 13, 14, },
  { 1, 4, 5, 8, 9, 13, 14, },
  { 0, 1, 4, 5, 8, 9, 13, 14, },
  { 2, 4, 5, 8, 9, 13, 14, },
  { 0, 2, 4, 5, 8, 9, 13, 14, },
  { 1, 2, 4, 5, 8, 9, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 13, 14, },
  { 3, 4, 5, 8, 9, 13, 14, },
  { 0, 3, 4, 5, 8, 9, 13, 14, },
  { 1, 3, 4, 5, 8, 9, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 13, 14, },
  { 2, 3, 4, 5, 8, 9, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 13, 14, },
  { 6, 8, 9, 13, 14, },
  { 0, 6, 8, 9, 13, 14, },
  { 1, 6, 8, 9, 13, 14, },
  { 0, 1, 6, 8, 9, 13, 14, },
  { 2, 6, 8, 9, 13, 14, },
  { 0, 2, 6, 8, 9, 13, 14, },
  { 1, 2, 6, 8, 9, 13, 14, },
  { 0, 1, 2, 6, 8, 9, 13, 14, },
  { 3, 6, 8, 9, 13, 14, },
  { 0, 3, 6, 8, 9, 13, 14, },
  { 1, 3, 6, 8, 9, 13, 14, },
  { 0, 1, 3, 6, 8, 9, 13, 14, },
  { 2, 3, 6, 8, 9, 13, 14, },
  { 0, 2, 3, 6, 8, 9, 13, 14, },
  { 1, 2, 3, 6, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 13, 14, },
  { 4, 6, 8, 9, 13, 14, },
  { 0, 4, 6, 8, 9, 13, 14, },
  { 1, 4, 6, 8, 9, 13, 14, },
  { 0, 1, 4, 6, 8, 9, 13, 14, },
  { 2, 4, 6, 8, 9, 13, 14, },
  { 0, 2, 4, 6, 8, 9, 13, 14, },
  { 1, 2, 4, 6, 8, 9, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 13, 14, },
  { 3, 4, 6, 8, 9, 13, 14, },
  { 0, 3, 4, 6, 8, 9, 13, 14, },
  { 1, 3, 4, 6, 8, 9, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 13, 14, },
  { 2, 3, 4, 6, 8, 9, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 13, 14, },
  { 5, 6, 8, 9, 13, 14, },
  { 0, 5, 6, 8, 9, 13, 14, },
  { 1, 5, 6, 8, 9, 13, 14, },
  { 0, 1, 5, 6, 8, 9, 13, 14, },
  { 2, 5, 6, 8, 9, 13, 14, },
  { 0, 2, 5, 6, 8, 9, 13, 14, },
  { 1, 2, 5, 6, 8, 9, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 13, 14, },
  { 3, 5, 6, 8, 9, 13, 14, },
  { 0, 3, 5, 6, 8, 9, 13, 14, },
  { 1, 3, 5, 6, 8, 9, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 13, 14, },
  { 2, 3, 5, 6, 8, 9, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 13, 14, },
  { 4, 5, 6, 8, 9, 13, 14, },
  { 0, 4, 5, 6, 8, 9, 13, 14, },
  { 1, 4, 5, 6, 8, 9, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 13, 14, },
  { 2, 4, 5, 6, 8, 9, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 13, 14, },
  { 3, 4, 5, 6, 8, 9, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 13, 14, },
  { 7, 8, 9, 13, 14, },
  { 0, 7, 8, 9, 13, 14, },
  { 1, 7, 8, 9, 13, 14, },
  { 0, 1, 7, 8, 9, 13, 14, },
  { 2, 7, 8, 9, 13, 14, },
  { 0, 2, 7, 8, 9, 13, 14, },
  { 1, 2, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 7, 8, 9, 13, 14, },
  { 3, 7, 8, 9, 13, 14, },
  { 0, 3, 7, 8, 9, 13, 14, },
  { 1, 3, 7, 8, 9, 13, 14, },
  { 0, 1, 3, 7, 8, 9, 13, 14, },
  { 2, 3, 7, 8, 9, 13, 14, },
  { 0, 2, 3, 7, 8, 9, 13, 14, },
  { 1, 2, 3, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 13, 14, },
  { 4, 7, 8, 9, 13, 14, },
  { 0, 4, 7, 8, 9, 13, 14, },
  { 1, 4, 7, 8, 9, 13, 14, },
  { 0, 1, 4, 7, 8, 9, 13, 14, },
  { 2, 4, 7, 8, 9, 13, 14, },
  { 0, 2, 4, 7, 8, 9, 13, 14, },
  { 1, 2, 4, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 13, 14, },
  { 3, 4, 7, 8, 9, 13, 14, },
  { 0, 3, 4, 7, 8, 9, 13, 14, },
  { 1, 3, 4, 7, 8, 9, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 13, 14, },
  { 2, 3, 4, 7, 8, 9, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 13, 14, },
  { 5, 7, 8, 9, 13, 14, },
  { 0, 5, 7, 8, 9, 13, 14, },
  { 1, 5, 7, 8, 9, 13, 14, },
  { 0, 1, 5, 7, 8, 9, 13, 14, },
  { 2, 5, 7, 8, 9, 13, 14, },
  { 0, 2, 5, 7, 8, 9, 13, 14, },
  { 1, 2, 5, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 13, 14, },
  { 3, 5, 7, 8, 9, 13, 14, },
  { 0, 3, 5, 7, 8, 9, 13, 14, },
  { 1, 3, 5, 7, 8, 9, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 13, 14, },
  { 2, 3, 5, 7, 8, 9, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 13, 14, },
  { 4, 5, 7, 8, 9, 13, 14, },
  { 0, 4, 5, 7, 8, 9, 13, 14, },
  { 1, 4, 5, 7, 8, 9, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 13, 14, },
  { 2, 4, 5, 7, 8, 9, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 13, 14, },
  { 3, 4, 5, 7, 8, 9, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 13, 14, },
  { 6, 7, 8, 9, 13, 14, },
  { 0, 6, 7, 8, 9, 13, 14, },
  { 1, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 6, 7, 8, 9, 13, 14, },
  { 2, 6, 7, 8, 9, 13, 14, },
  { 0, 2, 6, 7, 8, 9, 13, 14, },
  { 1, 2, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 13, 14, },
  { 3, 6, 7, 8, 9, 13, 14, },
  { 0, 3, 6, 7, 8, 9, 13, 14, },
  { 1, 3, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 13, 14, },
  { 2, 3, 6, 7, 8, 9, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 13, 14, },
  { 4, 6, 7, 8, 9, 13, 14, },
  { 0, 4, 6, 7, 8, 9, 13, 14, },
  { 1, 4, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 13, 14, },
  { 2, 4, 6, 7, 8, 9, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 13, 14, },
  { 3, 4, 6, 7, 8, 9, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 13, 14, },
  { 5, 6, 7, 8, 9, 13, 14, },
  { 0, 5, 6, 7, 8, 9, 13, 14, },
  { 1, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 13, 14, },
  { 2, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 13, 14, },
  { 3, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 13, 14, },
  { 4, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, },
  { 10, 13, 14, },
  { 0, 10, 13, 14, },
  { 1, 10, 13, 14, },
  { 0, 1, 10, 13, 14, },
  { 2, 10, 13, 14, },
  { 0, 2, 10, 13, 14, },
  { 1, 2, 10, 13, 14, },
  { 0, 1, 2, 10, 13, 14, },
  { 3, 10, 13, 14, },
  { 0, 3, 10, 13, 14, },
  { 1, 3, 10, 13, 14, },
  { 0, 1, 3, 10, 13, 14, },
  { 2, 3, 10, 13, 14, },
  { 0, 2, 3, 10, 13, 14, },
  { 1, 2, 3, 10, 13, 14, },
  { 0, 1, 2, 3, 10, 13, 14, },
  { 4, 10, 13, 14, },
  { 0, 4, 10, 13, 14, },
  { 1, 4, 10, 13, 14, },
  { 0, 1, 4, 10, 13, 14, },
  { 2, 4, 10, 13, 14, },
  { 0, 2, 4, 10, 13, 14, },
  { 1, 2, 4, 10, 13, 14, },
  { 0, 1, 2, 4, 10, 13, 14, },
  { 3, 4, 10, 13, 14, },
  { 0, 3, 4, 10, 13, 14, },
  { 1, 3, 4, 10, 13, 14, },
  { 0, 1, 3, 4, 10, 13, 14, },
  { 2, 3, 4, 10, 13, 14, },
  { 0, 2, 3, 4, 10, 13, 14, },
  { 1, 2, 3, 4, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 10, 13, 14, },
  { 5, 10, 13, 14, },
  { 0, 5, 10, 13, 14, },
  { 1, 5, 10, 13, 14, },
  { 0, 1, 5, 10, 13, 14, },
  { 2, 5, 10, 13, 14, },
  { 0, 2, 5, 10, 13, 14, },
  { 1, 2, 5, 10, 13, 14, },
  { 0, 1, 2, 5, 10, 13, 14, },
  { 3, 5, 10, 13, 14, },
  { 0, 3, 5, 10, 13, 14, },
  { 1, 3, 5, 10, 13, 14, },
  { 0, 1, 3, 5, 10, 13, 14, },
  { 2, 3, 5, 10, 13, 14, },
  { 0, 2, 3, 5, 10, 13, 14, },
  { 1, 2, 3, 5, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 10, 13, 14, },
  { 4, 5, 10, 13, 14, },
  { 0, 4, 5, 10, 13, 14, },
  { 1, 4, 5, 10, 13, 14, },
  { 0, 1, 4, 5, 10, 13, 14, },
  { 2, 4, 5, 10, 13, 14, },
  { 0, 2, 4, 5, 10, 13, 14, },
  { 1, 2, 4, 5, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 10, 13, 14, },
  { 3, 4, 5, 10, 13, 14, },
  { 0, 3, 4, 5, 10, 13, 14, },
  { 1, 3, 4, 5, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 10, 13, 14, },
  { 2, 3, 4, 5, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 10, 13, 14, },
  { 6, 10, 13, 14, },
  { 0, 6, 10, 13, 14, },
  { 1, 6, 10, 13, 14, },
  { 0, 1, 6, 10, 13, 14, },
  { 2, 6, 10, 13, 14, },
  { 0, 2, 6, 10, 13, 14, },
  { 1, 2, 6, 10, 13, 14, },
  { 0, 1, 2, 6, 10, 13, 14, },
  { 3, 6, 10, 13, 14, },
  { 0, 3, 6, 10, 13, 14, },
  { 1, 3, 6, 10, 13, 14, },
  { 0, 1, 3, 6, 10, 13, 14, },
  { 2, 3, 6, 10, 13, 14, },
  { 0, 2, 3, 6, 10, 13, 14, },
  { 1, 2, 3, 6, 10, 13, 14, },
  { 0, 1, 2, 3, 6, 10, 13, 14, },
  { 4, 6, 10, 13, 14, },
  { 0, 4, 6, 10, 13, 14, },
  { 1, 4, 6, 10, 13, 14, },
  { 0, 1, 4, 6, 10, 13, 14, },
  { 2, 4, 6, 10, 13, 14, },
  { 0, 2, 4, 6, 10, 13, 14, },
  { 1, 2, 4, 6, 10, 13, 14, },
  { 0, 1, 2, 4, 6, 10, 13, 14, },
  { 3, 4, 6, 10, 13, 14, },
  { 0, 3, 4, 6, 10, 13, 14, },
  { 1, 3, 4, 6, 10, 13, 14, },
  { 0, 1, 3, 4, 6, 10, 13, 14, },
  { 2, 3, 4, 6, 10, 13, 14, },
  { 0, 2, 3, 4, 6, 10, 13, 14, },
  { 1, 2, 3, 4, 6, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 10, 13, 14, },
  { 5, 6, 10, 13, 14, },
  { 0, 5, 6, 10, 13, 14, },
  { 1, 5, 6, 10, 13, 14, },
  { 0, 1, 5, 6, 10, 13, 14, },
  { 2, 5, 6, 10, 13, 14, },
  { 0, 2, 5, 6, 10, 13, 14, },
  { 1, 2, 5, 6, 10, 13, 14, },
  { 0, 1, 2, 5, 6, 10, 13, 14, },
  { 3, 5, 6, 10, 13, 14, },
  { 0, 3, 5, 6, 10, 13, 14, },
  { 1, 3, 5, 6, 10, 13, 14, },
  { 0, 1, 3, 5, 6, 10, 13, 14, },
  { 2, 3, 5, 6, 10, 13, 14, },
  { 0, 2, 3, 5, 6, 10, 13, 14, },
  { 1, 2, 3, 5, 6, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 10, 13, 14, },
  { 4, 5, 6, 10, 13, 14, },
  { 0, 4, 5, 6, 10, 13, 14, },
  { 1, 4, 5, 6, 10, 13, 14, },
  { 0, 1, 4, 5, 6, 10, 13, 14, },
  { 2, 4, 5, 6, 10, 13, 14, },
  { 0, 2, 4, 5, 6, 10, 13, 14, },
  { 1, 2, 4, 5, 6, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 10, 13, 14, },
  { 3, 4, 5, 6, 10, 13, 14, },
  { 0, 3, 4, 5, 6, 10, 13, 14, },
  { 1, 3, 4, 5, 6, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 10, 13, 14, },
  { 2, 3, 4, 5, 6, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 13, 14, },
  { 7, 10, 13, 14, },
  { 0, 7, 10, 13, 14, },
  { 1, 7, 10, 13, 14, },
  { 0, 1, 7, 10, 13, 14, },
  { 2, 7, 10, 13, 14, },
  { 0, 2, 7, 10, 13, 14, },
  { 1, 2, 7, 10, 13, 14, },
  { 0, 1, 2, 7, 10, 13, 14, },
  { 3, 7, 10, 13, 14, },
  { 0, 3, 7, 10, 13, 14, },
  { 1, 3, 7, 10, 13, 14, },
  { 0, 1, 3, 7, 10, 13, 14, },
  { 2, 3, 7, 10, 13, 14, },
  { 0, 2, 3, 7, 10, 13, 14, },
  { 1, 2, 3, 7, 10, 13, 14, },
  { 0, 1, 2, 3, 7, 10, 13, 14, },
  { 4, 7, 10, 13, 14, },
  { 0, 4, 7, 10, 13, 14, },
  { 1, 4, 7, 10, 13, 14, },
  { 0, 1, 4, 7, 10, 13, 14, },
  { 2, 4, 7, 10, 13, 14, },
  { 0, 2, 4, 7, 10, 13, 14, },
  { 1, 2, 4, 7, 10, 13, 14, },
  { 0, 1, 2, 4, 7, 10, 13, 14, },
  { 3, 4, 7, 10, 13, 14, },
  { 0, 3, 4, 7, 10, 13, 14, },
  { 1, 3, 4, 7, 10, 13, 14, },
  { 0, 1, 3, 4, 7, 10, 13, 14, },
  { 2, 3, 4, 7, 10, 13, 14, },
  { 0, 2, 3, 4, 7, 10, 13, 14, },
  { 1, 2, 3, 4, 7, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 10, 13, 14, },
  { 5, 7, 10, 13, 14, },
  { 0, 5, 7, 10, 13, 14, },
  { 1, 5, 7, 10, 13, 14, },
  { 0, 1, 5, 7, 10, 13, 14, },
  { 2, 5, 7, 10, 13, 14, },
  { 0, 2, 5, 7, 10, 13, 14, },
  { 1, 2, 5, 7, 10, 13, 14, },
  { 0, 1, 2, 5, 7, 10, 13, 14, },
  { 3, 5, 7, 10, 13, 14, },
  { 0, 3, 5, 7, 10, 13, 14, },
  { 1, 3, 5, 7, 10, 13, 14, },
  { 0, 1, 3, 5, 7, 10, 13, 14, },
  { 2, 3, 5, 7, 10, 13, 14, },
  { 0, 2, 3, 5, 7, 10, 13, 14, },
  { 1, 2, 3, 5, 7, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 10, 13, 14, },
  { 4, 5, 7, 10, 13, 14, },
  { 0, 4, 5, 7, 10, 13, 14, },
  { 1, 4, 5, 7, 10, 13, 14, },
  { 0, 1, 4, 5, 7, 10, 13, 14, },
  { 2, 4, 5, 7, 10, 13, 14, },
  { 0, 2, 4, 5, 7, 10, 13, 14, },
  { 1, 2, 4, 5, 7, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 10, 13, 14, },
  { 3, 4, 5, 7, 10, 13, 14, },
  { 0, 3, 4, 5, 7, 10, 13, 14, },
  { 1, 3, 4, 5, 7, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 10, 13, 14, },
  { 2, 3, 4, 5, 7, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 13, 14, },
  { 6, 7, 10, 13, 14, },
  { 0, 6, 7, 10, 13, 14, },
  { 1, 6, 7, 10, 13, 14, },
  { 0, 1, 6, 7, 10, 13, 14, },
  { 2, 6, 7, 10, 13, 14, },
  { 0, 2, 6, 7, 10, 13, 14, },
  { 1, 2, 6, 7, 10, 13, 14, },
  { 0, 1, 2, 6, 7, 10, 13, 14, },
  { 3, 6, 7, 10, 13, 14, },
  { 0, 3, 6, 7, 10, 13, 14, },
  { 1, 3, 6, 7, 10, 13, 14, },
  { 0, 1, 3, 6, 7, 10, 13, 14, },
  { 2, 3, 6, 7, 10, 13, 14, },
  { 0, 2, 3, 6, 7, 10, 13, 14, },
  { 1, 2, 3, 6, 7, 10, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 10, 13, 14, },
  { 4, 6, 7, 10, 13, 14, },
  { 0, 4, 6, 7, 10, 13, 14, },
  { 1, 4, 6, 7, 10, 13, 14, },
  { 0, 1, 4, 6, 7, 10, 13, 14, },
  { 2, 4, 6, 7, 10, 13, 14, },
  { 0, 2, 4, 6, 7, 10, 13, 14, },
  { 1, 2, 4, 6, 7, 10, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 10, 13, 14, },
  { 3, 4, 6, 7, 10, 13, 14, },
  { 0, 3, 4, 6, 7, 10, 13, 14, },
  { 1, 3, 4, 6, 7, 10, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 10, 13, 14, },
  { 2, 3, 4, 6, 7, 10, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 10, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 13, 14, },
  { 5, 6, 7, 10, 13, 14, },
  { 0, 5, 6, 7, 10, 13, 14, },
  { 1, 5, 6, 7, 10, 13, 14, },
  { 0, 1, 5, 6, 7, 10, 13, 14, },
  { 2, 5, 6, 7, 10, 13, 14, },
  { 0, 2, 5, 6, 7, 10, 13, 14, },
  { 1, 2, 5, 6, 7, 10, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 10, 13, 14, },
  { 3, 5, 6, 7, 10, 13, 14, },
  { 0, 3, 5, 6, 7, 10, 13, 14, },
  { 1, 3, 5, 6, 7, 10, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 10, 13, 14, },
  { 2, 3, 5, 6, 7, 10, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 10, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 13, 14, },
  { 4, 5, 6, 7, 10, 13, 14, },
  { 0, 4, 5, 6, 7, 10, 13, 14, },
  { 1, 4, 5, 6, 7, 10, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 10, 13, 14, },
  { 2, 4, 5, 6, 7, 10, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 10, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 13, 14, },
  { 3, 4, 5, 6, 7, 10, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 10, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 13, 14, },
  { 8, 10, 13, 14, },
  { 0, 8, 10, 13, 14, },
  { 1, 8, 10, 13, 14, },
  { 0, 1, 8, 10, 13, 14, },
  { 2, 8, 10, 13, 14, },
  { 0, 2, 8, 10, 13, 14, },
  { 1, 2, 8, 10, 13, 14, },
  { 0, 1, 2, 8, 10, 13, 14, },
  { 3, 8, 10, 13, 14, },
  { 0, 3, 8, 10, 13, 14, },
  { 1, 3, 8, 10, 13, 14, },
  { 0, 1, 3, 8, 10, 13, 14, },
  { 2, 3, 8, 10, 13, 14, },
  { 0, 2, 3, 8, 10, 13, 14, },
  { 1, 2, 3, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 8, 10, 13, 14, },
  { 4, 8, 10, 13, 14, },
  { 0, 4, 8, 10, 13, 14, },
  { 1, 4, 8, 10, 13, 14, },
  { 0, 1, 4, 8, 10, 13, 14, },
  { 2, 4, 8, 10, 13, 14, },
  { 0, 2, 4, 8, 10, 13, 14, },
  { 1, 2, 4, 8, 10, 13, 14, },
  { 0, 1, 2, 4, 8, 10, 13, 14, },
  { 3, 4, 8, 10, 13, 14, },
  { 0, 3, 4, 8, 10, 13, 14, },
  { 1, 3, 4, 8, 10, 13, 14, },
  { 0, 1, 3, 4, 8, 10, 13, 14, },
  { 2, 3, 4, 8, 10, 13, 14, },
  { 0, 2, 3, 4, 8, 10, 13, 14, },
  { 1, 2, 3, 4, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 10, 13, 14, },
  { 5, 8, 10, 13, 14, },
  { 0, 5, 8, 10, 13, 14, },
  { 1, 5, 8, 10, 13, 14, },
  { 0, 1, 5, 8, 10, 13, 14, },
  { 2, 5, 8, 10, 13, 14, },
  { 0, 2, 5, 8, 10, 13, 14, },
  { 1, 2, 5, 8, 10, 13, 14, },
  { 0, 1, 2, 5, 8, 10, 13, 14, },
  { 3, 5, 8, 10, 13, 14, },
  { 0, 3, 5, 8, 10, 13, 14, },
  { 1, 3, 5, 8, 10, 13, 14, },
  { 0, 1, 3, 5, 8, 10, 13, 14, },
  { 2, 3, 5, 8, 10, 13, 14, },
  { 0, 2, 3, 5, 8, 10, 13, 14, },
  { 1, 2, 3, 5, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 10, 13, 14, },
  { 4, 5, 8, 10, 13, 14, },
  { 0, 4, 5, 8, 10, 13, 14, },
  { 1, 4, 5, 8, 10, 13, 14, },
  { 0, 1, 4, 5, 8, 10, 13, 14, },
  { 2, 4, 5, 8, 10, 13, 14, },
  { 0, 2, 4, 5, 8, 10, 13, 14, },
  { 1, 2, 4, 5, 8, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 10, 13, 14, },
  { 3, 4, 5, 8, 10, 13, 14, },
  { 0, 3, 4, 5, 8, 10, 13, 14, },
  { 1, 3, 4, 5, 8, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 10, 13, 14, },
  { 2, 3, 4, 5, 8, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 13, 14, },
  { 6, 8, 10, 13, 14, },
  { 0, 6, 8, 10, 13, 14, },
  { 1, 6, 8, 10, 13, 14, },
  { 0, 1, 6, 8, 10, 13, 14, },
  { 2, 6, 8, 10, 13, 14, },
  { 0, 2, 6, 8, 10, 13, 14, },
  { 1, 2, 6, 8, 10, 13, 14, },
  { 0, 1, 2, 6, 8, 10, 13, 14, },
  { 3, 6, 8, 10, 13, 14, },
  { 0, 3, 6, 8, 10, 13, 14, },
  { 1, 3, 6, 8, 10, 13, 14, },
  { 0, 1, 3, 6, 8, 10, 13, 14, },
  { 2, 3, 6, 8, 10, 13, 14, },
  { 0, 2, 3, 6, 8, 10, 13, 14, },
  { 1, 2, 3, 6, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 10, 13, 14, },
  { 4, 6, 8, 10, 13, 14, },
  { 0, 4, 6, 8, 10, 13, 14, },
  { 1, 4, 6, 8, 10, 13, 14, },
  { 0, 1, 4, 6, 8, 10, 13, 14, },
  { 2, 4, 6, 8, 10, 13, 14, },
  { 0, 2, 4, 6, 8, 10, 13, 14, },
  { 1, 2, 4, 6, 8, 10, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 10, 13, 14, },
  { 3, 4, 6, 8, 10, 13, 14, },
  { 0, 3, 4, 6, 8, 10, 13, 14, },
  { 1, 3, 4, 6, 8, 10, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 10, 13, 14, },
  { 2, 3, 4, 6, 8, 10, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 10, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 13, 14, },
  { 5, 6, 8, 10, 13, 14, },
  { 0, 5, 6, 8, 10, 13, 14, },
  { 1, 5, 6, 8, 10, 13, 14, },
  { 0, 1, 5, 6, 8, 10, 13, 14, },
  { 2, 5, 6, 8, 10, 13, 14, },
  { 0, 2, 5, 6, 8, 10, 13, 14, },
  { 1, 2, 5, 6, 8, 10, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 10, 13, 14, },
  { 3, 5, 6, 8, 10, 13, 14, },
  { 0, 3, 5, 6, 8, 10, 13, 14, },
  { 1, 3, 5, 6, 8, 10, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 10, 13, 14, },
  { 2, 3, 5, 6, 8, 10, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 10, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 13, 14, },
  { 4, 5, 6, 8, 10, 13, 14, },
  { 0, 4, 5, 6, 8, 10, 13, 14, },
  { 1, 4, 5, 6, 8, 10, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 10, 13, 14, },
  { 2, 4, 5, 6, 8, 10, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 10, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 13, 14, },
  { 3, 4, 5, 6, 8, 10, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 10, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 13, 14, },
  { 7, 8, 10, 13, 14, },
  { 0, 7, 8, 10, 13, 14, },
  { 1, 7, 8, 10, 13, 14, },
  { 0, 1, 7, 8, 10, 13, 14, },
  { 2, 7, 8, 10, 13, 14, },
  { 0, 2, 7, 8, 10, 13, 14, },
  { 1, 2, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 7, 8, 10, 13, 14, },
  { 3, 7, 8, 10, 13, 14, },
  { 0, 3, 7, 8, 10, 13, 14, },
  { 1, 3, 7, 8, 10, 13, 14, },
  { 0, 1, 3, 7, 8, 10, 13, 14, },
  { 2, 3, 7, 8, 10, 13, 14, },
  { 0, 2, 3, 7, 8, 10, 13, 14, },
  { 1, 2, 3, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 10, 13, 14, },
  { 4, 7, 8, 10, 13, 14, },
  { 0, 4, 7, 8, 10, 13, 14, },
  { 1, 4, 7, 8, 10, 13, 14, },
  { 0, 1, 4, 7, 8, 10, 13, 14, },
  { 2, 4, 7, 8, 10, 13, 14, },
  { 0, 2, 4, 7, 8, 10, 13, 14, },
  { 1, 2, 4, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 10, 13, 14, },
  { 3, 4, 7, 8, 10, 13, 14, },
  { 0, 3, 4, 7, 8, 10, 13, 14, },
  { 1, 3, 4, 7, 8, 10, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 10, 13, 14, },
  { 2, 3, 4, 7, 8, 10, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 10, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 13, 14, },
  { 5, 7, 8, 10, 13, 14, },
  { 0, 5, 7, 8, 10, 13, 14, },
  { 1, 5, 7, 8, 10, 13, 14, },
  { 0, 1, 5, 7, 8, 10, 13, 14, },
  { 2, 5, 7, 8, 10, 13, 14, },
  { 0, 2, 5, 7, 8, 10, 13, 14, },
  { 1, 2, 5, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 10, 13, 14, },
  { 3, 5, 7, 8, 10, 13, 14, },
  { 0, 3, 5, 7, 8, 10, 13, 14, },
  { 1, 3, 5, 7, 8, 10, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 10, 13, 14, },
  { 2, 3, 5, 7, 8, 10, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 10, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 13, 14, },
  { 4, 5, 7, 8, 10, 13, 14, },
  { 0, 4, 5, 7, 8, 10, 13, 14, },
  { 1, 4, 5, 7, 8, 10, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 10, 13, 14, },
  { 2, 4, 5, 7, 8, 10, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 10, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 13, 14, },
  { 3, 4, 5, 7, 8, 10, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 10, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 13, 14, },
  { 6, 7, 8, 10, 13, 14, },
  { 0, 6, 7, 8, 10, 13, 14, },
  { 1, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 6, 7, 8, 10, 13, 14, },
  { 2, 6, 7, 8, 10, 13, 14, },
  { 0, 2, 6, 7, 8, 10, 13, 14, },
  { 1, 2, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 10, 13, 14, },
  { 3, 6, 7, 8, 10, 13, 14, },
  { 0, 3, 6, 7, 8, 10, 13, 14, },
  { 1, 3, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 10, 13, 14, },
  { 2, 3, 6, 7, 8, 10, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 10, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 13, 14, },
  { 4, 6, 7, 8, 10, 13, 14, },
  { 0, 4, 6, 7, 8, 10, 13, 14, },
  { 1, 4, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 10, 13, 14, },
  { 2, 4, 6, 7, 8, 10, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 10, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 13, 14, },
  { 3, 4, 6, 7, 8, 10, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 10, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 10, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 13, 14, },
  { 5, 6, 7, 8, 10, 13, 14, },
  { 0, 5, 6, 7, 8, 10, 13, 14, },
  { 1, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 10, 13, 14, },
  { 2, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 10, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 13, 14, },
  { 3, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 10, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 13, 14, },
  { 4, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, },
  { 9, 10, 13, 14, },
  { 0, 9, 10, 13, 14, },
  { 1, 9, 10, 13, 14, },
  { 0, 1, 9, 10, 13, 14, },
  { 2, 9, 10, 13, 14, },
  { 0, 2, 9, 10, 13, 14, },
  { 1, 2, 9, 10, 13, 14, },
  { 0, 1, 2, 9, 10, 13, 14, },
  { 3, 9, 10, 13, 14, },
  { 0, 3, 9, 10, 13, 14, },
  { 1, 3, 9, 10, 13, 14, },
  { 0, 1, 3, 9, 10, 13, 14, },
  { 2, 3, 9, 10, 13, 14, },
  { 0, 2, 3, 9, 10, 13, 14, },
  { 1, 2, 3, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 9, 10, 13, 14, },
  { 4, 9, 10, 13, 14, },
  { 0, 4, 9, 10, 13, 14, },
  { 1, 4, 9, 10, 13, 14, },
  { 0, 1, 4, 9, 10, 13, 14, },
  { 2, 4, 9, 10, 13, 14, },
  { 0, 2, 4, 9, 10, 13, 14, },
  { 1, 2, 4, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 9, 10, 13, 14, },
  { 3, 4, 9, 10, 13, 14, },
  { 0, 3, 4, 9, 10, 13, 14, },
  { 1, 3, 4, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 9, 10, 13, 14, },
  { 2, 3, 4, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 9, 10, 13, 14, },
  { 5, 9, 10, 13, 14, },
  { 0, 5, 9, 10, 13, 14, },
  { 1, 5, 9, 10, 13, 14, },
  { 0, 1, 5, 9, 10, 13, 14, },
  { 2, 5, 9, 10, 13, 14, },
  { 0, 2, 5, 9, 10, 13, 14, },
  { 1, 2, 5, 9, 10, 13, 14, },
  { 0, 1, 2, 5, 9, 10, 13, 14, },
  { 3, 5, 9, 10, 13, 14, },
  { 0, 3, 5, 9, 10, 13, 14, },
  { 1, 3, 5, 9, 10, 13, 14, },
  { 0, 1, 3, 5, 9, 10, 13, 14, },
  { 2, 3, 5, 9, 10, 13, 14, },
  { 0, 2, 3, 5, 9, 10, 13, 14, },
  { 1, 2, 3, 5, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 9, 10, 13, 14, },
  { 4, 5, 9, 10, 13, 14, },
  { 0, 4, 5, 9, 10, 13, 14, },
  { 1, 4, 5, 9, 10, 13, 14, },
  { 0, 1, 4, 5, 9, 10, 13, 14, },
  { 2, 4, 5, 9, 10, 13, 14, },
  { 0, 2, 4, 5, 9, 10, 13, 14, },
  { 1, 2, 4, 5, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 9, 10, 13, 14, },
  { 3, 4, 5, 9, 10, 13, 14, },
  { 0, 3, 4, 5, 9, 10, 13, 14, },
  { 1, 3, 4, 5, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 9, 10, 13, 14, },
  { 2, 3, 4, 5, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 13, 14, },
  { 6, 9, 10, 13, 14, },
  { 0, 6, 9, 10, 13, 14, },
  { 1, 6, 9, 10, 13, 14, },
  { 0, 1, 6, 9, 10, 13, 14, },
  { 2, 6, 9, 10, 13, 14, },
  { 0, 2, 6, 9, 10, 13, 14, },
  { 1, 2, 6, 9, 10, 13, 14, },
  { 0, 1, 2, 6, 9, 10, 13, 14, },
  { 3, 6, 9, 10, 13, 14, },
  { 0, 3, 6, 9, 10, 13, 14, },
  { 1, 3, 6, 9, 10, 13, 14, },
  { 0, 1, 3, 6, 9, 10, 13, 14, },
  { 2, 3, 6, 9, 10, 13, 14, },
  { 0, 2, 3, 6, 9, 10, 13, 14, },
  { 1, 2, 3, 6, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 6, 9, 10, 13, 14, },
  { 4, 6, 9, 10, 13, 14, },
  { 0, 4, 6, 9, 10, 13, 14, },
  { 1, 4, 6, 9, 10, 13, 14, },
  { 0, 1, 4, 6, 9, 10, 13, 14, },
  { 2, 4, 6, 9, 10, 13, 14, },
  { 0, 2, 4, 6, 9, 10, 13, 14, },
  { 1, 2, 4, 6, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 6, 9, 10, 13, 14, },
  { 3, 4, 6, 9, 10, 13, 14, },
  { 0, 3, 4, 6, 9, 10, 13, 14, },
  { 1, 3, 4, 6, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 6, 9, 10, 13, 14, },
  { 2, 3, 4, 6, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 6, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 6, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 13, 14, },
  { 5, 6, 9, 10, 13, 14, },
  { 0, 5, 6, 9, 10, 13, 14, },
  { 1, 5, 6, 9, 10, 13, 14, },
  { 0, 1, 5, 6, 9, 10, 13, 14, },
  { 2, 5, 6, 9, 10, 13, 14, },
  { 0, 2, 5, 6, 9, 10, 13, 14, },
  { 1, 2, 5, 6, 9, 10, 13, 14, },
  { 0, 1, 2, 5, 6, 9, 10, 13, 14, },
  { 3, 5, 6, 9, 10, 13, 14, },
  { 0, 3, 5, 6, 9, 10, 13, 14, },
  { 1, 3, 5, 6, 9, 10, 13, 14, },
  { 0, 1, 3, 5, 6, 9, 10, 13, 14, },
  { 2, 3, 5, 6, 9, 10, 13, 14, },
  { 0, 2, 3, 5, 6, 9, 10, 13, 14, },
  { 1, 2, 3, 5, 6, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 13, 14, },
  { 4, 5, 6, 9, 10, 13, 14, },
  { 0, 4, 5, 6, 9, 10, 13, 14, },
  { 1, 4, 5, 6, 9, 10, 13, 14, },
  { 0, 1, 4, 5, 6, 9, 10, 13, 14, },
  { 2, 4, 5, 6, 9, 10, 13, 14, },
  { 0, 2, 4, 5, 6, 9, 10, 13, 14, },
  { 1, 2, 4, 5, 6, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 13, 14, },
  { 3, 4, 5, 6, 9, 10, 13, 14, },
  { 0, 3, 4, 5, 6, 9, 10, 13, 14, },
  { 1, 3, 4, 5, 6, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 13, 14, },
  { 2, 3, 4, 5, 6, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 13, 14, },
  { 7, 9, 10, 13, 14, },
  { 0, 7, 9, 10, 13, 14, },
  { 1, 7, 9, 10, 13, 14, },
  { 0, 1, 7, 9, 10, 13, 14, },
  { 2, 7, 9, 10, 13, 14, },
  { 0, 2, 7, 9, 10, 13, 14, },
  { 1, 2, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 7, 9, 10, 13, 14, },
  { 3, 7, 9, 10, 13, 14, },
  { 0, 3, 7, 9, 10, 13, 14, },
  { 1, 3, 7, 9, 10, 13, 14, },
  { 0, 1, 3, 7, 9, 10, 13, 14, },
  { 2, 3, 7, 9, 10, 13, 14, },
  { 0, 2, 3, 7, 9, 10, 13, 14, },
  { 1, 2, 3, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 7, 9, 10, 13, 14, },
  { 4, 7, 9, 10, 13, 14, },
  { 0, 4, 7, 9, 10, 13, 14, },
  { 1, 4, 7, 9, 10, 13, 14, },
  { 0, 1, 4, 7, 9, 10, 13, 14, },
  { 2, 4, 7, 9, 10, 13, 14, },
  { 0, 2, 4, 7, 9, 10, 13, 14, },
  { 1, 2, 4, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 7, 9, 10, 13, 14, },
  { 3, 4, 7, 9, 10, 13, 14, },
  { 0, 3, 4, 7, 9, 10, 13, 14, },
  { 1, 3, 4, 7, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 7, 9, 10, 13, 14, },
  { 2, 3, 4, 7, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 7, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 13, 14, },
  { 5, 7, 9, 10, 13, 14, },
  { 0, 5, 7, 9, 10, 13, 14, },
  { 1, 5, 7, 9, 10, 13, 14, },
  { 0, 1, 5, 7, 9, 10, 13, 14, },
  { 2, 5, 7, 9, 10, 13, 14, },
  { 0, 2, 5, 7, 9, 10, 13, 14, },
  { 1, 2, 5, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 5, 7, 9, 10, 13, 14, },
  { 3, 5, 7, 9, 10, 13, 14, },
  { 0, 3, 5, 7, 9, 10, 13, 14, },
  { 1, 3, 5, 7, 9, 10, 13, 14, },
  { 0, 1, 3, 5, 7, 9, 10, 13, 14, },
  { 2, 3, 5, 7, 9, 10, 13, 14, },
  { 0, 2, 3, 5, 7, 9, 10, 13, 14, },
  { 1, 2, 3, 5, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 13, 14, },
  { 4, 5, 7, 9, 10, 13, 14, },
  { 0, 4, 5, 7, 9, 10, 13, 14, },
  { 1, 4, 5, 7, 9, 10, 13, 14, },
  { 0, 1, 4, 5, 7, 9, 10, 13, 14, },
  { 2, 4, 5, 7, 9, 10, 13, 14, },
  { 0, 2, 4, 5, 7, 9, 10, 13, 14, },
  { 1, 2, 4, 5, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 13, 14, },
  { 3, 4, 5, 7, 9, 10, 13, 14, },
  { 0, 3, 4, 5, 7, 9, 10, 13, 14, },
  { 1, 3, 4, 5, 7, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 13, 14, },
  { 2, 3, 4, 5, 7, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 13, 14, },
  { 6, 7, 9, 10, 13, 14, },
  { 0, 6, 7, 9, 10, 13, 14, },
  { 1, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 6, 7, 9, 10, 13, 14, },
  { 2, 6, 7, 9, 10, 13, 14, },
  { 0, 2, 6, 7, 9, 10, 13, 14, },
  { 1, 2, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 6, 7, 9, 10, 13, 14, },
  { 3, 6, 7, 9, 10, 13, 14, },
  { 0, 3, 6, 7, 9, 10, 13, 14, },
  { 1, 3, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 3, 6, 7, 9, 10, 13, 14, },
  { 2, 3, 6, 7, 9, 10, 13, 14, },
  { 0, 2, 3, 6, 7, 9, 10, 13, 14, },
  { 1, 2, 3, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 13, 14, },
  { 4, 6, 7, 9, 10, 13, 14, },
  { 0, 4, 6, 7, 9, 10, 13, 14, },
  { 1, 4, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 4, 6, 7, 9, 10, 13, 14, },
  { 2, 4, 6, 7, 9, 10, 13, 14, },
  { 0, 2, 4, 6, 7, 9, 10, 13, 14, },
  { 1, 2, 4, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 13, 14, },
  { 3, 4, 6, 7, 9, 10, 13, 14, },
  { 0, 3, 4, 6, 7, 9, 10, 13, 14, },
  { 1, 3, 4, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 13, 14, },
  { 2, 3, 4, 6, 7, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 13, 14, },
  { 5, 6, 7, 9, 10, 13, 14, },
  { 0, 5, 6, 7, 9, 10, 13, 14, },
  { 1, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 5, 6, 7, 9, 10, 13, 14, },
  { 2, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 2, 5, 6, 7, 9, 10, 13, 14, },
  { 1, 2, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 13, 14, },
  { 3, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 3, 5, 6, 7, 9, 10, 13, 14, },
  { 1, 3, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 13, 14, },
  { 2, 3, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 13, 14, },
  { 4, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 1, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 2, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 3, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 13, 14, },
  { 8, 9, 10, 13, 14, },
  { 0, 8, 9, 10, 13, 14, },
  { 1, 8, 9, 10, 13, 14, },
  { 0, 1, 8, 9, 10, 13, 14, },
  { 2, 8, 9, 10, 13, 14, },
  { 0, 2, 8, 9, 10, 13, 14, },
  { 1, 2, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 8, 9, 10, 13, 14, },
  { 3, 8, 9, 10, 13, 14, },
  { 0, 3, 8, 9, 10, 13, 14, },
  { 1, 3, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 8, 9, 10, 13, 14, },
  { 2, 3, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 8, 9, 10, 13, 14, },
  { 4, 8, 9, 10, 13, 14, },
  { 0, 4, 8, 9, 10, 13, 14, },
  { 1, 4, 8, 9, 10, 13, 14, },
  { 0, 1, 4, 8, 9, 10, 13, 14, },
  { 2, 4, 8, 9, 10, 13, 14, },
  { 0, 2, 4, 8, 9, 10, 13, 14, },
  { 1, 2, 4, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 8, 9, 10, 13, 14, },
  { 3, 4, 8, 9, 10, 13, 14, },
  { 0, 3, 4, 8, 9, 10, 13, 14, },
  { 1, 3, 4, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 8, 9, 10, 13, 14, },
  { 2, 3, 4, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 13, 14, },
  { 5, 8, 9, 10, 13, 14, },
  { 0, 5, 8, 9, 10, 13, 14, },
  { 1, 5, 8, 9, 10, 13, 14, },
  { 0, 1, 5, 8, 9, 10, 13, 14, },
  { 2, 5, 8, 9, 10, 13, 14, },
  { 0, 2, 5, 8, 9, 10, 13, 14, },
  { 1, 2, 5, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 5, 8, 9, 10, 13, 14, },
  { 3, 5, 8, 9, 10, 13, 14, },
  { 0, 3, 5, 8, 9, 10, 13, 14, },
  { 1, 3, 5, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 5, 8, 9, 10, 13, 14, },
  { 2, 3, 5, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 5, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 5, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 13, 14, },
  { 4, 5, 8, 9, 10, 13, 14, },
  { 0, 4, 5, 8, 9, 10, 13, 14, },
  { 1, 4, 5, 8, 9, 10, 13, 14, },
  { 0, 1, 4, 5, 8, 9, 10, 13, 14, },
  { 2, 4, 5, 8, 9, 10, 13, 14, },
  { 0, 2, 4, 5, 8, 9, 10, 13, 14, },
  { 1, 2, 4, 5, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 13, 14, },
  { 3, 4, 5, 8, 9, 10, 13, 14, },
  { 0, 3, 4, 5, 8, 9, 10, 13, 14, },
  { 1, 3, 4, 5, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 13, 14, },
  { 2, 3, 4, 5, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 13, 14, },
  { 6, 8, 9, 10, 13, 14, },
  { 0, 6, 8, 9, 10, 13, 14, },
  { 1, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 6, 8, 9, 10, 13, 14, },
  { 2, 6, 8, 9, 10, 13, 14, },
  { 0, 2, 6, 8, 9, 10, 13, 14, },
  { 1, 2, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 6, 8, 9, 10, 13, 14, },
  { 3, 6, 8, 9, 10, 13, 14, },
  { 0, 3, 6, 8, 9, 10, 13, 14, },
  { 1, 3, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 6, 8, 9, 10, 13, 14, },
  { 2, 3, 6, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 6, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 13, 14, },
  { 4, 6, 8, 9, 10, 13, 14, },
  { 0, 4, 6, 8, 9, 10, 13, 14, },
  { 1, 4, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 4, 6, 8, 9, 10, 13, 14, },
  { 2, 4, 6, 8, 9, 10, 13, 14, },
  { 0, 2, 4, 6, 8, 9, 10, 13, 14, },
  { 1, 2, 4, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 13, 14, },
  { 3, 4, 6, 8, 9, 10, 13, 14, },
  { 0, 3, 4, 6, 8, 9, 10, 13, 14, },
  { 1, 3, 4, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 13, 14, },
  { 2, 3, 4, 6, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 13, 14, },
  { 5, 6, 8, 9, 10, 13, 14, },
  { 0, 5, 6, 8, 9, 10, 13, 14, },
  { 1, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 5, 6, 8, 9, 10, 13, 14, },
  { 2, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 2, 5, 6, 8, 9, 10, 13, 14, },
  { 1, 2, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 13, 14, },
  { 3, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 3, 5, 6, 8, 9, 10, 13, 14, },
  { 1, 3, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 13, 14, },
  { 2, 3, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 13, 14, },
  { 4, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 1, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 2, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 3, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 13, 14, },
  { 7, 8, 9, 10, 13, 14, },
  { 0, 7, 8, 9, 10, 13, 14, },
  { 1, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 7, 8, 9, 10, 13, 14, },
  { 2, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 7, 8, 9, 10, 13, 14, },
  { 3, 7, 8, 9, 10, 13, 14, },
  { 0, 3, 7, 8, 9, 10, 13, 14, },
  { 1, 3, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 7, 8, 9, 10, 13, 14, },
  { 2, 3, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 13, 14, },
  { 4, 7, 8, 9, 10, 13, 14, },
  { 0, 4, 7, 8, 9, 10, 13, 14, },
  { 1, 4, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 4, 7, 8, 9, 10, 13, 14, },
  { 2, 4, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 4, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 4, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 13, 14, },
  { 3, 4, 7, 8, 9, 10, 13, 14, },
  { 0, 3, 4, 7, 8, 9, 10, 13, 14, },
  { 1, 3, 4, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 13, 14, },
  { 2, 3, 4, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 13, 14, },
  { 5, 7, 8, 9, 10, 13, 14, },
  { 0, 5, 7, 8, 9, 10, 13, 14, },
  { 1, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 5, 7, 8, 9, 10, 13, 14, },
  { 2, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 5, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 13, 14, },
  { 3, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 3, 5, 7, 8, 9, 10, 13, 14, },
  { 1, 3, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 13, 14, },
  { 2, 3, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 13, 14, },
  { 4, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 1, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 2, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 3, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 13, 14, },
  { 6, 7, 8, 9, 10, 13, 14, },
  { 0, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 6, 7, 8, 9, 10, 13, 14, },
  { 2, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 13, 14, },
  { 3, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 3, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 3, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 13, 14, },
  { 2, 3, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 13, 14, },
  { 4, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 2, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 3, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 13, 14, },
  { 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 2, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 3, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, },
  { 11, 13, 14, },
  { 0, 11, 13, 14, },
  { 1, 11, 13, 14, },
  { 0, 1, 11, 13, 14, },
  { 2, 11, 13, 14, },
  { 0, 2, 11, 13, 14, },
  { 1, 2, 11, 13, 14, },
  { 0, 1, 2, 11, 13, 14, },
  { 3, 11, 13, 14, },
  { 0, 3, 11, 13, 14, },
  { 1, 3, 11, 13, 14, },
  { 0, 1, 3, 11, 13, 14, },
  { 2, 3, 11, 13, 14, },
  { 0, 2, 3, 11, 13, 14, },
  { 1, 2, 3, 11, 13, 14, },
  { 0, 1, 2, 3, 11, 13, 14, },
  { 4, 11, 13, 14, },
  { 0, 4, 11, 13, 14, },
  { 1, 4, 11, 13, 14, },
  { 0, 1, 4, 11, 13, 14, },
  { 2, 4, 11, 13, 14, },
  { 0, 2, 4, 11, 13, 14, },
  { 1, 2, 4, 11, 13, 14, },
  { 0, 1, 2, 4, 11, 13, 14, },
  { 3, 4, 11, 13, 14, },
  { 0, 3, 4, 11, 13, 14, },
  { 1, 3, 4, 11, 13, 14, },
  { 0, 1, 3, 4, 11, 13, 14, },
  { 2, 3, 4, 11, 13, 14, },
  { 0, 2, 3, 4, 11, 13, 14, },
  { 1, 2, 3, 4, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 11, 13, 14, },
  { 5, 11, 13, 14, },
  { 0, 5, 11, 13, 14, },
  { 1, 5, 11, 13, 14, },
  { 0, 1, 5, 11, 13, 14, },
  { 2, 5, 11, 13, 14, },
  { 0, 2, 5, 11, 13, 14, },
  { 1, 2, 5, 11, 13, 14, },
  { 0, 1, 2, 5, 11, 13, 14, },
  { 3, 5, 11, 13, 14, },
  { 0, 3, 5, 11, 13, 14, },
  { 1, 3, 5, 11, 13, 14, },
  { 0, 1, 3, 5, 11, 13, 14, },
  { 2, 3, 5, 11, 13, 14, },
  { 0, 2, 3, 5, 11, 13, 14, },
  { 1, 2, 3, 5, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 11, 13, 14, },
  { 4, 5, 11, 13, 14, },
  { 0, 4, 5, 11, 13, 14, },
  { 1, 4, 5, 11, 13, 14, },
  { 0, 1, 4, 5, 11, 13, 14, },
  { 2, 4, 5, 11, 13, 14, },
  { 0, 2, 4, 5, 11, 13, 14, },
  { 1, 2, 4, 5, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 11, 13, 14, },
  { 3, 4, 5, 11, 13, 14, },
  { 0, 3, 4, 5, 11, 13, 14, },
  { 1, 3, 4, 5, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 11, 13, 14, },
  { 2, 3, 4, 5, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 11, 13, 14, },
  { 6, 11, 13, 14, },
  { 0, 6, 11, 13, 14, },
  { 1, 6, 11, 13, 14, },
  { 0, 1, 6, 11, 13, 14, },
  { 2, 6, 11, 13, 14, },
  { 0, 2, 6, 11, 13, 14, },
  { 1, 2, 6, 11, 13, 14, },
  { 0, 1, 2, 6, 11, 13, 14, },
  { 3, 6, 11, 13, 14, },
  { 0, 3, 6, 11, 13, 14, },
  { 1, 3, 6, 11, 13, 14, },
  { 0, 1, 3, 6, 11, 13, 14, },
  { 2, 3, 6, 11, 13, 14, },
  { 0, 2, 3, 6, 11, 13, 14, },
  { 1, 2, 3, 6, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 11, 13, 14, },
  { 4, 6, 11, 13, 14, },
  { 0, 4, 6, 11, 13, 14, },
  { 1, 4, 6, 11, 13, 14, },
  { 0, 1, 4, 6, 11, 13, 14, },
  { 2, 4, 6, 11, 13, 14, },
  { 0, 2, 4, 6, 11, 13, 14, },
  { 1, 2, 4, 6, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 11, 13, 14, },
  { 3, 4, 6, 11, 13, 14, },
  { 0, 3, 4, 6, 11, 13, 14, },
  { 1, 3, 4, 6, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 11, 13, 14, },
  { 2, 3, 4, 6, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 11, 13, 14, },
  { 5, 6, 11, 13, 14, },
  { 0, 5, 6, 11, 13, 14, },
  { 1, 5, 6, 11, 13, 14, },
  { 0, 1, 5, 6, 11, 13, 14, },
  { 2, 5, 6, 11, 13, 14, },
  { 0, 2, 5, 6, 11, 13, 14, },
  { 1, 2, 5, 6, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 11, 13, 14, },
  { 3, 5, 6, 11, 13, 14, },
  { 0, 3, 5, 6, 11, 13, 14, },
  { 1, 3, 5, 6, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 11, 13, 14, },
  { 2, 3, 5, 6, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 11, 13, 14, },
  { 4, 5, 6, 11, 13, 14, },
  { 0, 4, 5, 6, 11, 13, 14, },
  { 1, 4, 5, 6, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 11, 13, 14, },
  { 2, 4, 5, 6, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 11, 13, 14, },
  { 3, 4, 5, 6, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 13, 14, },
  { 7, 11, 13, 14, },
  { 0, 7, 11, 13, 14, },
  { 1, 7, 11, 13, 14, },
  { 0, 1, 7, 11, 13, 14, },
  { 2, 7, 11, 13, 14, },
  { 0, 2, 7, 11, 13, 14, },
  { 1, 2, 7, 11, 13, 14, },
  { 0, 1, 2, 7, 11, 13, 14, },
  { 3, 7, 11, 13, 14, },
  { 0, 3, 7, 11, 13, 14, },
  { 1, 3, 7, 11, 13, 14, },
  { 0, 1, 3, 7, 11, 13, 14, },
  { 2, 3, 7, 11, 13, 14, },
  { 0, 2, 3, 7, 11, 13, 14, },
  { 1, 2, 3, 7, 11, 13, 14, },
  { 0, 1, 2, 3, 7, 11, 13, 14, },
  { 4, 7, 11, 13, 14, },
  { 0, 4, 7, 11, 13, 14, },
  { 1, 4, 7, 11, 13, 14, },
  { 0, 1, 4, 7, 11, 13, 14, },
  { 2, 4, 7, 11, 13, 14, },
  { 0, 2, 4, 7, 11, 13, 14, },
  { 1, 2, 4, 7, 11, 13, 14, },
  { 0, 1, 2, 4, 7, 11, 13, 14, },
  { 3, 4, 7, 11, 13, 14, },
  { 0, 3, 4, 7, 11, 13, 14, },
  { 1, 3, 4, 7, 11, 13, 14, },
  { 0, 1, 3, 4, 7, 11, 13, 14, },
  { 2, 3, 4, 7, 11, 13, 14, },
  { 0, 2, 3, 4, 7, 11, 13, 14, },
  { 1, 2, 3, 4, 7, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 11, 13, 14, },
  { 5, 7, 11, 13, 14, },
  { 0, 5, 7, 11, 13, 14, },
  { 1, 5, 7, 11, 13, 14, },
  { 0, 1, 5, 7, 11, 13, 14, },
  { 2, 5, 7, 11, 13, 14, },
  { 0, 2, 5, 7, 11, 13, 14, },
  { 1, 2, 5, 7, 11, 13, 14, },
  { 0, 1, 2, 5, 7, 11, 13, 14, },
  { 3, 5, 7, 11, 13, 14, },
  { 0, 3, 5, 7, 11, 13, 14, },
  { 1, 3, 5, 7, 11, 13, 14, },
  { 0, 1, 3, 5, 7, 11, 13, 14, },
  { 2, 3, 5, 7, 11, 13, 14, },
  { 0, 2, 3, 5, 7, 11, 13, 14, },
  { 1, 2, 3, 5, 7, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 11, 13, 14, },
  { 4, 5, 7, 11, 13, 14, },
  { 0, 4, 5, 7, 11, 13, 14, },
  { 1, 4, 5, 7, 11, 13, 14, },
  { 0, 1, 4, 5, 7, 11, 13, 14, },
  { 2, 4, 5, 7, 11, 13, 14, },
  { 0, 2, 4, 5, 7, 11, 13, 14, },
  { 1, 2, 4, 5, 7, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 11, 13, 14, },
  { 3, 4, 5, 7, 11, 13, 14, },
  { 0, 3, 4, 5, 7, 11, 13, 14, },
  { 1, 3, 4, 5, 7, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 11, 13, 14, },
  { 2, 3, 4, 5, 7, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 13, 14, },
  { 6, 7, 11, 13, 14, },
  { 0, 6, 7, 11, 13, 14, },
  { 1, 6, 7, 11, 13, 14, },
  { 0, 1, 6, 7, 11, 13, 14, },
  { 2, 6, 7, 11, 13, 14, },
  { 0, 2, 6, 7, 11, 13, 14, },
  { 1, 2, 6, 7, 11, 13, 14, },
  { 0, 1, 2, 6, 7, 11, 13, 14, },
  { 3, 6, 7, 11, 13, 14, },
  { 0, 3, 6, 7, 11, 13, 14, },
  { 1, 3, 6, 7, 11, 13, 14, },
  { 0, 1, 3, 6, 7, 11, 13, 14, },
  { 2, 3, 6, 7, 11, 13, 14, },
  { 0, 2, 3, 6, 7, 11, 13, 14, },
  { 1, 2, 3, 6, 7, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 11, 13, 14, },
  { 4, 6, 7, 11, 13, 14, },
  { 0, 4, 6, 7, 11, 13, 14, },
  { 1, 4, 6, 7, 11, 13, 14, },
  { 0, 1, 4, 6, 7, 11, 13, 14, },
  { 2, 4, 6, 7, 11, 13, 14, },
  { 0, 2, 4, 6, 7, 11, 13, 14, },
  { 1, 2, 4, 6, 7, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 11, 13, 14, },
  { 3, 4, 6, 7, 11, 13, 14, },
  { 0, 3, 4, 6, 7, 11, 13, 14, },
  { 1, 3, 4, 6, 7, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 11, 13, 14, },
  { 2, 3, 4, 6, 7, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 13, 14, },
  { 5, 6, 7, 11, 13, 14, },
  { 0, 5, 6, 7, 11, 13, 14, },
  { 1, 5, 6, 7, 11, 13, 14, },
  { 0, 1, 5, 6, 7, 11, 13, 14, },
  { 2, 5, 6, 7, 11, 13, 14, },
  { 0, 2, 5, 6, 7, 11, 13, 14, },
  { 1, 2, 5, 6, 7, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 11, 13, 14, },
  { 3, 5, 6, 7, 11, 13, 14, },
  { 0, 3, 5, 6, 7, 11, 13, 14, },
  { 1, 3, 5, 6, 7, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 11, 13, 14, },
  { 2, 3, 5, 6, 7, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 13, 14, },
  { 4, 5, 6, 7, 11, 13, 14, },
  { 0, 4, 5, 6, 7, 11, 13, 14, },
  { 1, 4, 5, 6, 7, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 11, 13, 14, },
  { 2, 4, 5, 6, 7, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 13, 14, },
  { 3, 4, 5, 6, 7, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 13, 14, },
  { 8, 11, 13, 14, },
  { 0, 8, 11, 13, 14, },
  { 1, 8, 11, 13, 14, },
  { 0, 1, 8, 11, 13, 14, },
  { 2, 8, 11, 13, 14, },
  { 0, 2, 8, 11, 13, 14, },
  { 1, 2, 8, 11, 13, 14, },
  { 0, 1, 2, 8, 11, 13, 14, },
  { 3, 8, 11, 13, 14, },
  { 0, 3, 8, 11, 13, 14, },
  { 1, 3, 8, 11, 13, 14, },
  { 0, 1, 3, 8, 11, 13, 14, },
  { 2, 3, 8, 11, 13, 14, },
  { 0, 2, 3, 8, 11, 13, 14, },
  { 1, 2, 3, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 8, 11, 13, 14, },
  { 4, 8, 11, 13, 14, },
  { 0, 4, 8, 11, 13, 14, },
  { 1, 4, 8, 11, 13, 14, },
  { 0, 1, 4, 8, 11, 13, 14, },
  { 2, 4, 8, 11, 13, 14, },
  { 0, 2, 4, 8, 11, 13, 14, },
  { 1, 2, 4, 8, 11, 13, 14, },
  { 0, 1, 2, 4, 8, 11, 13, 14, },
  { 3, 4, 8, 11, 13, 14, },
  { 0, 3, 4, 8, 11, 13, 14, },
  { 1, 3, 4, 8, 11, 13, 14, },
  { 0, 1, 3, 4, 8, 11, 13, 14, },
  { 2, 3, 4, 8, 11, 13, 14, },
  { 0, 2, 3, 4, 8, 11, 13, 14, },
  { 1, 2, 3, 4, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 11, 13, 14, },
  { 5, 8, 11, 13, 14, },
  { 0, 5, 8, 11, 13, 14, },
  { 1, 5, 8, 11, 13, 14, },
  { 0, 1, 5, 8, 11, 13, 14, },
  { 2, 5, 8, 11, 13, 14, },
  { 0, 2, 5, 8, 11, 13, 14, },
  { 1, 2, 5, 8, 11, 13, 14, },
  { 0, 1, 2, 5, 8, 11, 13, 14, },
  { 3, 5, 8, 11, 13, 14, },
  { 0, 3, 5, 8, 11, 13, 14, },
  { 1, 3, 5, 8, 11, 13, 14, },
  { 0, 1, 3, 5, 8, 11, 13, 14, },
  { 2, 3, 5, 8, 11, 13, 14, },
  { 0, 2, 3, 5, 8, 11, 13, 14, },
  { 1, 2, 3, 5, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 11, 13, 14, },
  { 4, 5, 8, 11, 13, 14, },
  { 0, 4, 5, 8, 11, 13, 14, },
  { 1, 4, 5, 8, 11, 13, 14, },
  { 0, 1, 4, 5, 8, 11, 13, 14, },
  { 2, 4, 5, 8, 11, 13, 14, },
  { 0, 2, 4, 5, 8, 11, 13, 14, },
  { 1, 2, 4, 5, 8, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 11, 13, 14, },
  { 3, 4, 5, 8, 11, 13, 14, },
  { 0, 3, 4, 5, 8, 11, 13, 14, },
  { 1, 3, 4, 5, 8, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 11, 13, 14, },
  { 2, 3, 4, 5, 8, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 13, 14, },
  { 6, 8, 11, 13, 14, },
  { 0, 6, 8, 11, 13, 14, },
  { 1, 6, 8, 11, 13, 14, },
  { 0, 1, 6, 8, 11, 13, 14, },
  { 2, 6, 8, 11, 13, 14, },
  { 0, 2, 6, 8, 11, 13, 14, },
  { 1, 2, 6, 8, 11, 13, 14, },
  { 0, 1, 2, 6, 8, 11, 13, 14, },
  { 3, 6, 8, 11, 13, 14, },
  { 0, 3, 6, 8, 11, 13, 14, },
  { 1, 3, 6, 8, 11, 13, 14, },
  { 0, 1, 3, 6, 8, 11, 13, 14, },
  { 2, 3, 6, 8, 11, 13, 14, },
  { 0, 2, 3, 6, 8, 11, 13, 14, },
  { 1, 2, 3, 6, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 11, 13, 14, },
  { 4, 6, 8, 11, 13, 14, },
  { 0, 4, 6, 8, 11, 13, 14, },
  { 1, 4, 6, 8, 11, 13, 14, },
  { 0, 1, 4, 6, 8, 11, 13, 14, },
  { 2, 4, 6, 8, 11, 13, 14, },
  { 0, 2, 4, 6, 8, 11, 13, 14, },
  { 1, 2, 4, 6, 8, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 11, 13, 14, },
  { 3, 4, 6, 8, 11, 13, 14, },
  { 0, 3, 4, 6, 8, 11, 13, 14, },
  { 1, 3, 4, 6, 8, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 11, 13, 14, },
  { 2, 3, 4, 6, 8, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 13, 14, },
  { 5, 6, 8, 11, 13, 14, },
  { 0, 5, 6, 8, 11, 13, 14, },
  { 1, 5, 6, 8, 11, 13, 14, },
  { 0, 1, 5, 6, 8, 11, 13, 14, },
  { 2, 5, 6, 8, 11, 13, 14, },
  { 0, 2, 5, 6, 8, 11, 13, 14, },
  { 1, 2, 5, 6, 8, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 11, 13, 14, },
  { 3, 5, 6, 8, 11, 13, 14, },
  { 0, 3, 5, 6, 8, 11, 13, 14, },
  { 1, 3, 5, 6, 8, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 11, 13, 14, },
  { 2, 3, 5, 6, 8, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 13, 14, },
  { 4, 5, 6, 8, 11, 13, 14, },
  { 0, 4, 5, 6, 8, 11, 13, 14, },
  { 1, 4, 5, 6, 8, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 11, 13, 14, },
  { 2, 4, 5, 6, 8, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 13, 14, },
  { 3, 4, 5, 6, 8, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 13, 14, },
  { 7, 8, 11, 13, 14, },
  { 0, 7, 8, 11, 13, 14, },
  { 1, 7, 8, 11, 13, 14, },
  { 0, 1, 7, 8, 11, 13, 14, },
  { 2, 7, 8, 11, 13, 14, },
  { 0, 2, 7, 8, 11, 13, 14, },
  { 1, 2, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 7, 8, 11, 13, 14, },
  { 3, 7, 8, 11, 13, 14, },
  { 0, 3, 7, 8, 11, 13, 14, },
  { 1, 3, 7, 8, 11, 13, 14, },
  { 0, 1, 3, 7, 8, 11, 13, 14, },
  { 2, 3, 7, 8, 11, 13, 14, },
  { 0, 2, 3, 7, 8, 11, 13, 14, },
  { 1, 2, 3, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 11, 13, 14, },
  { 4, 7, 8, 11, 13, 14, },
  { 0, 4, 7, 8, 11, 13, 14, },
  { 1, 4, 7, 8, 11, 13, 14, },
  { 0, 1, 4, 7, 8, 11, 13, 14, },
  { 2, 4, 7, 8, 11, 13, 14, },
  { 0, 2, 4, 7, 8, 11, 13, 14, },
  { 1, 2, 4, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 11, 13, 14, },
  { 3, 4, 7, 8, 11, 13, 14, },
  { 0, 3, 4, 7, 8, 11, 13, 14, },
  { 1, 3, 4, 7, 8, 11, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 11, 13, 14, },
  { 2, 3, 4, 7, 8, 11, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 11, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 13, 14, },
  { 5, 7, 8, 11, 13, 14, },
  { 0, 5, 7, 8, 11, 13, 14, },
  { 1, 5, 7, 8, 11, 13, 14, },
  { 0, 1, 5, 7, 8, 11, 13, 14, },
  { 2, 5, 7, 8, 11, 13, 14, },
  { 0, 2, 5, 7, 8, 11, 13, 14, },
  { 1, 2, 5, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 11, 13, 14, },
  { 3, 5, 7, 8, 11, 13, 14, },
  { 0, 3, 5, 7, 8, 11, 13, 14, },
  { 1, 3, 5, 7, 8, 11, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 11, 13, 14, },
  { 2, 3, 5, 7, 8, 11, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 11, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 13, 14, },
  { 4, 5, 7, 8, 11, 13, 14, },
  { 0, 4, 5, 7, 8, 11, 13, 14, },
  { 1, 4, 5, 7, 8, 11, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 11, 13, 14, },
  { 2, 4, 5, 7, 8, 11, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 11, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 13, 14, },
  { 3, 4, 5, 7, 8, 11, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 11, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 13, 14, },
  { 6, 7, 8, 11, 13, 14, },
  { 0, 6, 7, 8, 11, 13, 14, },
  { 1, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 6, 7, 8, 11, 13, 14, },
  { 2, 6, 7, 8, 11, 13, 14, },
  { 0, 2, 6, 7, 8, 11, 13, 14, },
  { 1, 2, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 11, 13, 14, },
  { 3, 6, 7, 8, 11, 13, 14, },
  { 0, 3, 6, 7, 8, 11, 13, 14, },
  { 1, 3, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 11, 13, 14, },
  { 2, 3, 6, 7, 8, 11, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 11, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 13, 14, },
  { 4, 6, 7, 8, 11, 13, 14, },
  { 0, 4, 6, 7, 8, 11, 13, 14, },
  { 1, 4, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 11, 13, 14, },
  { 2, 4, 6, 7, 8, 11, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 11, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 13, 14, },
  { 3, 4, 6, 7, 8, 11, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 11, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 13, 14, },
  { 5, 6, 7, 8, 11, 13, 14, },
  { 0, 5, 6, 7, 8, 11, 13, 14, },
  { 1, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 11, 13, 14, },
  { 2, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 11, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 13, 14, },
  { 3, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 11, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 13, 14, },
  { 4, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, },
  { 9, 11, 13, 14, },
  { 0, 9, 11, 13, 14, },
  { 1, 9, 11, 13, 14, },
  { 0, 1, 9, 11, 13, 14, },
  { 2, 9, 11, 13, 14, },
  { 0, 2, 9, 11, 13, 14, },
  { 1, 2, 9, 11, 13, 14, },
  { 0, 1, 2, 9, 11, 13, 14, },
  { 3, 9, 11, 13, 14, },
  { 0, 3, 9, 11, 13, 14, },
  { 1, 3, 9, 11, 13, 14, },
  { 0, 1, 3, 9, 11, 13, 14, },
  { 2, 3, 9, 11, 13, 14, },
  { 0, 2, 3, 9, 11, 13, 14, },
  { 1, 2, 3, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 9, 11, 13, 14, },
  { 4, 9, 11, 13, 14, },
  { 0, 4, 9, 11, 13, 14, },
  { 1, 4, 9, 11, 13, 14, },
  { 0, 1, 4, 9, 11, 13, 14, },
  { 2, 4, 9, 11, 13, 14, },
  { 0, 2, 4, 9, 11, 13, 14, },
  { 1, 2, 4, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 9, 11, 13, 14, },
  { 3, 4, 9, 11, 13, 14, },
  { 0, 3, 4, 9, 11, 13, 14, },
  { 1, 3, 4, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 9, 11, 13, 14, },
  { 2, 3, 4, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 9, 11, 13, 14, },
  { 5, 9, 11, 13, 14, },
  { 0, 5, 9, 11, 13, 14, },
  { 1, 5, 9, 11, 13, 14, },
  { 0, 1, 5, 9, 11, 13, 14, },
  { 2, 5, 9, 11, 13, 14, },
  { 0, 2, 5, 9, 11, 13, 14, },
  { 1, 2, 5, 9, 11, 13, 14, },
  { 0, 1, 2, 5, 9, 11, 13, 14, },
  { 3, 5, 9, 11, 13, 14, },
  { 0, 3, 5, 9, 11, 13, 14, },
  { 1, 3, 5, 9, 11, 13, 14, },
  { 0, 1, 3, 5, 9, 11, 13, 14, },
  { 2, 3, 5, 9, 11, 13, 14, },
  { 0, 2, 3, 5, 9, 11, 13, 14, },
  { 1, 2, 3, 5, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 9, 11, 13, 14, },
  { 4, 5, 9, 11, 13, 14, },
  { 0, 4, 5, 9, 11, 13, 14, },
  { 1, 4, 5, 9, 11, 13, 14, },
  { 0, 1, 4, 5, 9, 11, 13, 14, },
  { 2, 4, 5, 9, 11, 13, 14, },
  { 0, 2, 4, 5, 9, 11, 13, 14, },
  { 1, 2, 4, 5, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 9, 11, 13, 14, },
  { 3, 4, 5, 9, 11, 13, 14, },
  { 0, 3, 4, 5, 9, 11, 13, 14, },
  { 1, 3, 4, 5, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 9, 11, 13, 14, },
  { 2, 3, 4, 5, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 13, 14, },
  { 6, 9, 11, 13, 14, },
  { 0, 6, 9, 11, 13, 14, },
  { 1, 6, 9, 11, 13, 14, },
  { 0, 1, 6, 9, 11, 13, 14, },
  { 2, 6, 9, 11, 13, 14, },
  { 0, 2, 6, 9, 11, 13, 14, },
  { 1, 2, 6, 9, 11, 13, 14, },
  { 0, 1, 2, 6, 9, 11, 13, 14, },
  { 3, 6, 9, 11, 13, 14, },
  { 0, 3, 6, 9, 11, 13, 14, },
  { 1, 3, 6, 9, 11, 13, 14, },
  { 0, 1, 3, 6, 9, 11, 13, 14, },
  { 2, 3, 6, 9, 11, 13, 14, },
  { 0, 2, 3, 6, 9, 11, 13, 14, },
  { 1, 2, 3, 6, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 9, 11, 13, 14, },
  { 4, 6, 9, 11, 13, 14, },
  { 0, 4, 6, 9, 11, 13, 14, },
  { 1, 4, 6, 9, 11, 13, 14, },
  { 0, 1, 4, 6, 9, 11, 13, 14, },
  { 2, 4, 6, 9, 11, 13, 14, },
  { 0, 2, 4, 6, 9, 11, 13, 14, },
  { 1, 2, 4, 6, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 9, 11, 13, 14, },
  { 3, 4, 6, 9, 11, 13, 14, },
  { 0, 3, 4, 6, 9, 11, 13, 14, },
  { 1, 3, 4, 6, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 9, 11, 13, 14, },
  { 2, 3, 4, 6, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 13, 14, },
  { 5, 6, 9, 11, 13, 14, },
  { 0, 5, 6, 9, 11, 13, 14, },
  { 1, 5, 6, 9, 11, 13, 14, },
  { 0, 1, 5, 6, 9, 11, 13, 14, },
  { 2, 5, 6, 9, 11, 13, 14, },
  { 0, 2, 5, 6, 9, 11, 13, 14, },
  { 1, 2, 5, 6, 9, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 9, 11, 13, 14, },
  { 3, 5, 6, 9, 11, 13, 14, },
  { 0, 3, 5, 6, 9, 11, 13, 14, },
  { 1, 3, 5, 6, 9, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 9, 11, 13, 14, },
  { 2, 3, 5, 6, 9, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 9, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 13, 14, },
  { 4, 5, 6, 9, 11, 13, 14, },
  { 0, 4, 5, 6, 9, 11, 13, 14, },
  { 1, 4, 5, 6, 9, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 9, 11, 13, 14, },
  { 2, 4, 5, 6, 9, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 9, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 13, 14, },
  { 3, 4, 5, 6, 9, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 9, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 13, 14, },
  { 7, 9, 11, 13, 14, },
  { 0, 7, 9, 11, 13, 14, },
  { 1, 7, 9, 11, 13, 14, },
  { 0, 1, 7, 9, 11, 13, 14, },
  { 2, 7, 9, 11, 13, 14, },
  { 0, 2, 7, 9, 11, 13, 14, },
  { 1, 2, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 7, 9, 11, 13, 14, },
  { 3, 7, 9, 11, 13, 14, },
  { 0, 3, 7, 9, 11, 13, 14, },
  { 1, 3, 7, 9, 11, 13, 14, },
  { 0, 1, 3, 7, 9, 11, 13, 14, },
  { 2, 3, 7, 9, 11, 13, 14, },
  { 0, 2, 3, 7, 9, 11, 13, 14, },
  { 1, 2, 3, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 7, 9, 11, 13, 14, },
  { 4, 7, 9, 11, 13, 14, },
  { 0, 4, 7, 9, 11, 13, 14, },
  { 1, 4, 7, 9, 11, 13, 14, },
  { 0, 1, 4, 7, 9, 11, 13, 14, },
  { 2, 4, 7, 9, 11, 13, 14, },
  { 0, 2, 4, 7, 9, 11, 13, 14, },
  { 1, 2, 4, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 7, 9, 11, 13, 14, },
  { 3, 4, 7, 9, 11, 13, 14, },
  { 0, 3, 4, 7, 9, 11, 13, 14, },
  { 1, 3, 4, 7, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 7, 9, 11, 13, 14, },
  { 2, 3, 4, 7, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 7, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 13, 14, },
  { 5, 7, 9, 11, 13, 14, },
  { 0, 5, 7, 9, 11, 13, 14, },
  { 1, 5, 7, 9, 11, 13, 14, },
  { 0, 1, 5, 7, 9, 11, 13, 14, },
  { 2, 5, 7, 9, 11, 13, 14, },
  { 0, 2, 5, 7, 9, 11, 13, 14, },
  { 1, 2, 5, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 5, 7, 9, 11, 13, 14, },
  { 3, 5, 7, 9, 11, 13, 14, },
  { 0, 3, 5, 7, 9, 11, 13, 14, },
  { 1, 3, 5, 7, 9, 11, 13, 14, },
  { 0, 1, 3, 5, 7, 9, 11, 13, 14, },
  { 2, 3, 5, 7, 9, 11, 13, 14, },
  { 0, 2, 3, 5, 7, 9, 11, 13, 14, },
  { 1, 2, 3, 5, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 13, 14, },
  { 4, 5, 7, 9, 11, 13, 14, },
  { 0, 4, 5, 7, 9, 11, 13, 14, },
  { 1, 4, 5, 7, 9, 11, 13, 14, },
  { 0, 1, 4, 5, 7, 9, 11, 13, 14, },
  { 2, 4, 5, 7, 9, 11, 13, 14, },
  { 0, 2, 4, 5, 7, 9, 11, 13, 14, },
  { 1, 2, 4, 5, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 13, 14, },
  { 3, 4, 5, 7, 9, 11, 13, 14, },
  { 0, 3, 4, 5, 7, 9, 11, 13, 14, },
  { 1, 3, 4, 5, 7, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 13, 14, },
  { 2, 3, 4, 5, 7, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 13, 14, },
  { 6, 7, 9, 11, 13, 14, },
  { 0, 6, 7, 9, 11, 13, 14, },
  { 1, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 6, 7, 9, 11, 13, 14, },
  { 2, 6, 7, 9, 11, 13, 14, },
  { 0, 2, 6, 7, 9, 11, 13, 14, },
  { 1, 2, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 6, 7, 9, 11, 13, 14, },
  { 3, 6, 7, 9, 11, 13, 14, },
  { 0, 3, 6, 7, 9, 11, 13, 14, },
  { 1, 3, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 3, 6, 7, 9, 11, 13, 14, },
  { 2, 3, 6, 7, 9, 11, 13, 14, },
  { 0, 2, 3, 6, 7, 9, 11, 13, 14, },
  { 1, 2, 3, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 13, 14, },
  { 4, 6, 7, 9, 11, 13, 14, },
  { 0, 4, 6, 7, 9, 11, 13, 14, },
  { 1, 4, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 4, 6, 7, 9, 11, 13, 14, },
  { 2, 4, 6, 7, 9, 11, 13, 14, },
  { 0, 2, 4, 6, 7, 9, 11, 13, 14, },
  { 1, 2, 4, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 13, 14, },
  { 3, 4, 6, 7, 9, 11, 13, 14, },
  { 0, 3, 4, 6, 7, 9, 11, 13, 14, },
  { 1, 3, 4, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 13, 14, },
  { 2, 3, 4, 6, 7, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 13, 14, },
  { 5, 6, 7, 9, 11, 13, 14, },
  { 0, 5, 6, 7, 9, 11, 13, 14, },
  { 1, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 5, 6, 7, 9, 11, 13, 14, },
  { 2, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 2, 5, 6, 7, 9, 11, 13, 14, },
  { 1, 2, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 13, 14, },
  { 3, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 3, 5, 6, 7, 9, 11, 13, 14, },
  { 1, 3, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 13, 14, },
  { 2, 3, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 14, },
  { 4, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 1, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 2, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 3, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 14, },
  { 8, 9, 11, 13, 14, },
  { 0, 8, 9, 11, 13, 14, },
  { 1, 8, 9, 11, 13, 14, },
  { 0, 1, 8, 9, 11, 13, 14, },
  { 2, 8, 9, 11, 13, 14, },
  { 0, 2, 8, 9, 11, 13, 14, },
  { 1, 2, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 8, 9, 11, 13, 14, },
  { 3, 8, 9, 11, 13, 14, },
  { 0, 3, 8, 9, 11, 13, 14, },
  { 1, 3, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 8, 9, 11, 13, 14, },
  { 2, 3, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 8, 9, 11, 13, 14, },
  { 4, 8, 9, 11, 13, 14, },
  { 0, 4, 8, 9, 11, 13, 14, },
  { 1, 4, 8, 9, 11, 13, 14, },
  { 0, 1, 4, 8, 9, 11, 13, 14, },
  { 2, 4, 8, 9, 11, 13, 14, },
  { 0, 2, 4, 8, 9, 11, 13, 14, },
  { 1, 2, 4, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 8, 9, 11, 13, 14, },
  { 3, 4, 8, 9, 11, 13, 14, },
  { 0, 3, 4, 8, 9, 11, 13, 14, },
  { 1, 3, 4, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 8, 9, 11, 13, 14, },
  { 2, 3, 4, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 13, 14, },
  { 5, 8, 9, 11, 13, 14, },
  { 0, 5, 8, 9, 11, 13, 14, },
  { 1, 5, 8, 9, 11, 13, 14, },
  { 0, 1, 5, 8, 9, 11, 13, 14, },
  { 2, 5, 8, 9, 11, 13, 14, },
  { 0, 2, 5, 8, 9, 11, 13, 14, },
  { 1, 2, 5, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 5, 8, 9, 11, 13, 14, },
  { 3, 5, 8, 9, 11, 13, 14, },
  { 0, 3, 5, 8, 9, 11, 13, 14, },
  { 1, 3, 5, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 5, 8, 9, 11, 13, 14, },
  { 2, 3, 5, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 5, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 5, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 13, 14, },
  { 4, 5, 8, 9, 11, 13, 14, },
  { 0, 4, 5, 8, 9, 11, 13, 14, },
  { 1, 4, 5, 8, 9, 11, 13, 14, },
  { 0, 1, 4, 5, 8, 9, 11, 13, 14, },
  { 2, 4, 5, 8, 9, 11, 13, 14, },
  { 0, 2, 4, 5, 8, 9, 11, 13, 14, },
  { 1, 2, 4, 5, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 13, 14, },
  { 3, 4, 5, 8, 9, 11, 13, 14, },
  { 0, 3, 4, 5, 8, 9, 11, 13, 14, },
  { 1, 3, 4, 5, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 13, 14, },
  { 2, 3, 4, 5, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 13, 14, },
  { 6, 8, 9, 11, 13, 14, },
  { 0, 6, 8, 9, 11, 13, 14, },
  { 1, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 6, 8, 9, 11, 13, 14, },
  { 2, 6, 8, 9, 11, 13, 14, },
  { 0, 2, 6, 8, 9, 11, 13, 14, },
  { 1, 2, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 6, 8, 9, 11, 13, 14, },
  { 3, 6, 8, 9, 11, 13, 14, },
  { 0, 3, 6, 8, 9, 11, 13, 14, },
  { 1, 3, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 6, 8, 9, 11, 13, 14, },
  { 2, 3, 6, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 6, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 13, 14, },
  { 4, 6, 8, 9, 11, 13, 14, },
  { 0, 4, 6, 8, 9, 11, 13, 14, },
  { 1, 4, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 4, 6, 8, 9, 11, 13, 14, },
  { 2, 4, 6, 8, 9, 11, 13, 14, },
  { 0, 2, 4, 6, 8, 9, 11, 13, 14, },
  { 1, 2, 4, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 13, 14, },
  { 3, 4, 6, 8, 9, 11, 13, 14, },
  { 0, 3, 4, 6, 8, 9, 11, 13, 14, },
  { 1, 3, 4, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 13, 14, },
  { 2, 3, 4, 6, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 13, 14, },
  { 5, 6, 8, 9, 11, 13, 14, },
  { 0, 5, 6, 8, 9, 11, 13, 14, },
  { 1, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 5, 6, 8, 9, 11, 13, 14, },
  { 2, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 2, 5, 6, 8, 9, 11, 13, 14, },
  { 1, 2, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 13, 14, },
  { 3, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 3, 5, 6, 8, 9, 11, 13, 14, },
  { 1, 3, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 13, 14, },
  { 2, 3, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 13, 14, },
  { 4, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 1, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 2, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 3, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, },
  { 7, 8, 9, 11, 13, 14, },
  { 0, 7, 8, 9, 11, 13, 14, },
  { 1, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 7, 8, 9, 11, 13, 14, },
  { 2, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 7, 8, 9, 11, 13, 14, },
  { 3, 7, 8, 9, 11, 13, 14, },
  { 0, 3, 7, 8, 9, 11, 13, 14, },
  { 1, 3, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 7, 8, 9, 11, 13, 14, },
  { 2, 3, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 13, 14, },
  { 4, 7, 8, 9, 11, 13, 14, },
  { 0, 4, 7, 8, 9, 11, 13, 14, },
  { 1, 4, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 4, 7, 8, 9, 11, 13, 14, },
  { 2, 4, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 4, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 4, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 13, 14, },
  { 3, 4, 7, 8, 9, 11, 13, 14, },
  { 0, 3, 4, 7, 8, 9, 11, 13, 14, },
  { 1, 3, 4, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 13, 14, },
  { 2, 3, 4, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 13, 14, },
  { 5, 7, 8, 9, 11, 13, 14, },
  { 0, 5, 7, 8, 9, 11, 13, 14, },
  { 1, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 5, 7, 8, 9, 11, 13, 14, },
  { 2, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 5, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 13, 14, },
  { 3, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 3, 5, 7, 8, 9, 11, 13, 14, },
  { 1, 3, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 13, 14, },
  { 2, 3, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 13, 14, },
  { 4, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 1, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 2, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 3, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 13, 14, },
  { 6, 7, 8, 9, 11, 13, 14, },
  { 0, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 6, 7, 8, 9, 11, 13, 14, },
  { 2, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 13, 14, },
  { 3, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 3, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 3, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 13, 14, },
  { 2, 3, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 13, 14, },
  { 4, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 2, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 3, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 13, 14, },
  { 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 2, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 3, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, },
  { 10, 11, 13, 14, },
  { 0, 10, 11, 13, 14, },
  { 1, 10, 11, 13, 14, },
  { 0, 1, 10, 11, 13, 14, },
  { 2, 10, 11, 13, 14, },
  { 0, 2, 10, 11, 13, 14, },
  { 1, 2, 10, 11, 13, 14, },
  { 0, 1, 2, 10, 11, 13, 14, },
  { 3, 10, 11, 13, 14, },
  { 0, 3, 10, 11, 13, 14, },
  { 1, 3, 10, 11, 13, 14, },
  { 0, 1, 3, 10, 11, 13, 14, },
  { 2, 3, 10, 11, 13, 14, },
  { 0, 2, 3, 10, 11, 13, 14, },
  { 1, 2, 3, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 10, 11, 13, 14, },
  { 4, 10, 11, 13, 14, },
  { 0, 4, 10, 11, 13, 14, },
  { 1, 4, 10, 11, 13, 14, },
  { 0, 1, 4, 10, 11, 13, 14, },
  { 2, 4, 10, 11, 13, 14, },
  { 0, 2, 4, 10, 11, 13, 14, },
  { 1, 2, 4, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 10, 11, 13, 14, },
  { 3, 4, 10, 11, 13, 14, },
  { 0, 3, 4, 10, 11, 13, 14, },
  { 1, 3, 4, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 10, 11, 13, 14, },
  { 2, 3, 4, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 10, 11, 13, 14, },
  { 5, 10, 11, 13, 14, },
  { 0, 5, 10, 11, 13, 14, },
  { 1, 5, 10, 11, 13, 14, },
  { 0, 1, 5, 10, 11, 13, 14, },
  { 2, 5, 10, 11, 13, 14, },
  { 0, 2, 5, 10, 11, 13, 14, },
  { 1, 2, 5, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 10, 11, 13, 14, },
  { 3, 5, 10, 11, 13, 14, },
  { 0, 3, 5, 10, 11, 13, 14, },
  { 1, 3, 5, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 10, 11, 13, 14, },
  { 2, 3, 5, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 10, 11, 13, 14, },
  { 4, 5, 10, 11, 13, 14, },
  { 0, 4, 5, 10, 11, 13, 14, },
  { 1, 4, 5, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 10, 11, 13, 14, },
  { 2, 4, 5, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 10, 11, 13, 14, },
  { 3, 4, 5, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 13, 14, },
  { 6, 10, 11, 13, 14, },
  { 0, 6, 10, 11, 13, 14, },
  { 1, 6, 10, 11, 13, 14, },
  { 0, 1, 6, 10, 11, 13, 14, },
  { 2, 6, 10, 11, 13, 14, },
  { 0, 2, 6, 10, 11, 13, 14, },
  { 1, 2, 6, 10, 11, 13, 14, },
  { 0, 1, 2, 6, 10, 11, 13, 14, },
  { 3, 6, 10, 11, 13, 14, },
  { 0, 3, 6, 10, 11, 13, 14, },
  { 1, 3, 6, 10, 11, 13, 14, },
  { 0, 1, 3, 6, 10, 11, 13, 14, },
  { 2, 3, 6, 10, 11, 13, 14, },
  { 0, 2, 3, 6, 10, 11, 13, 14, },
  { 1, 2, 3, 6, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 10, 11, 13, 14, },
  { 4, 6, 10, 11, 13, 14, },
  { 0, 4, 6, 10, 11, 13, 14, },
  { 1, 4, 6, 10, 11, 13, 14, },
  { 0, 1, 4, 6, 10, 11, 13, 14, },
  { 2, 4, 6, 10, 11, 13, 14, },
  { 0, 2, 4, 6, 10, 11, 13, 14, },
  { 1, 2, 4, 6, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 10, 11, 13, 14, },
  { 3, 4, 6, 10, 11, 13, 14, },
  { 0, 3, 4, 6, 10, 11, 13, 14, },
  { 1, 3, 4, 6, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 10, 11, 13, 14, },
  { 2, 3, 4, 6, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 13, 14, },
  { 5, 6, 10, 11, 13, 14, },
  { 0, 5, 6, 10, 11, 13, 14, },
  { 1, 5, 6, 10, 11, 13, 14, },
  { 0, 1, 5, 6, 10, 11, 13, 14, },
  { 2, 5, 6, 10, 11, 13, 14, },
  { 0, 2, 5, 6, 10, 11, 13, 14, },
  { 1, 2, 5, 6, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 10, 11, 13, 14, },
  { 3, 5, 6, 10, 11, 13, 14, },
  { 0, 3, 5, 6, 10, 11, 13, 14, },
  { 1, 3, 5, 6, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 10, 11, 13, 14, },
  { 2, 3, 5, 6, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 13, 14, },
  { 4, 5, 6, 10, 11, 13, 14, },
  { 0, 4, 5, 6, 10, 11, 13, 14, },
  { 1, 4, 5, 6, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 10, 11, 13, 14, },
  { 2, 4, 5, 6, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 13, 14, },
  { 3, 4, 5, 6, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 13, 14, },
  { 7, 10, 11, 13, 14, },
  { 0, 7, 10, 11, 13, 14, },
  { 1, 7, 10, 11, 13, 14, },
  { 0, 1, 7, 10, 11, 13, 14, },
  { 2, 7, 10, 11, 13, 14, },
  { 0, 2, 7, 10, 11, 13, 14, },
  { 1, 2, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 7, 10, 11, 13, 14, },
  { 3, 7, 10, 11, 13, 14, },
  { 0, 3, 7, 10, 11, 13, 14, },
  { 1, 3, 7, 10, 11, 13, 14, },
  { 0, 1, 3, 7, 10, 11, 13, 14, },
  { 2, 3, 7, 10, 11, 13, 14, },
  { 0, 2, 3, 7, 10, 11, 13, 14, },
  { 1, 2, 3, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 7, 10, 11, 13, 14, },
  { 4, 7, 10, 11, 13, 14, },
  { 0, 4, 7, 10, 11, 13, 14, },
  { 1, 4, 7, 10, 11, 13, 14, },
  { 0, 1, 4, 7, 10, 11, 13, 14, },
  { 2, 4, 7, 10, 11, 13, 14, },
  { 0, 2, 4, 7, 10, 11, 13, 14, },
  { 1, 2, 4, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 7, 10, 11, 13, 14, },
  { 3, 4, 7, 10, 11, 13, 14, },
  { 0, 3, 4, 7, 10, 11, 13, 14, },
  { 1, 3, 4, 7, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 7, 10, 11, 13, 14, },
  { 2, 3, 4, 7, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 7, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 13, 14, },
  { 5, 7, 10, 11, 13, 14, },
  { 0, 5, 7, 10, 11, 13, 14, },
  { 1, 5, 7, 10, 11, 13, 14, },
  { 0, 1, 5, 7, 10, 11, 13, 14, },
  { 2, 5, 7, 10, 11, 13, 14, },
  { 0, 2, 5, 7, 10, 11, 13, 14, },
  { 1, 2, 5, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 7, 10, 11, 13, 14, },
  { 3, 5, 7, 10, 11, 13, 14, },
  { 0, 3, 5, 7, 10, 11, 13, 14, },
  { 1, 3, 5, 7, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 7, 10, 11, 13, 14, },
  { 2, 3, 5, 7, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 7, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 13, 14, },
  { 4, 5, 7, 10, 11, 13, 14, },
  { 0, 4, 5, 7, 10, 11, 13, 14, },
  { 1, 4, 5, 7, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 7, 10, 11, 13, 14, },
  { 2, 4, 5, 7, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 7, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 13, 14, },
  { 3, 4, 5, 7, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 7, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 7, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 7, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 13, 14, },
  { 6, 7, 10, 11, 13, 14, },
  { 0, 6, 7, 10, 11, 13, 14, },
  { 1, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 6, 7, 10, 11, 13, 14, },
  { 2, 6, 7, 10, 11, 13, 14, },
  { 0, 2, 6, 7, 10, 11, 13, 14, },
  { 1, 2, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 6, 7, 10, 11, 13, 14, },
  { 3, 6, 7, 10, 11, 13, 14, },
  { 0, 3, 6, 7, 10, 11, 13, 14, },
  { 1, 3, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 3, 6, 7, 10, 11, 13, 14, },
  { 2, 3, 6, 7, 10, 11, 13, 14, },
  { 0, 2, 3, 6, 7, 10, 11, 13, 14, },
  { 1, 2, 3, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 13, 14, },
  { 4, 6, 7, 10, 11, 13, 14, },
  { 0, 4, 6, 7, 10, 11, 13, 14, },
  { 1, 4, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 4, 6, 7, 10, 11, 13, 14, },
  { 2, 4, 6, 7, 10, 11, 13, 14, },
  { 0, 2, 4, 6, 7, 10, 11, 13, 14, },
  { 1, 2, 4, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 13, 14, },
  { 3, 4, 6, 7, 10, 11, 13, 14, },
  { 0, 3, 4, 6, 7, 10, 11, 13, 14, },
  { 1, 3, 4, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 13, 14, },
  { 2, 3, 4, 6, 7, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 13, 14, },
  { 5, 6, 7, 10, 11, 13, 14, },
  { 0, 5, 6, 7, 10, 11, 13, 14, },
  { 1, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 5, 6, 7, 10, 11, 13, 14, },
  { 2, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 2, 5, 6, 7, 10, 11, 13, 14, },
  { 1, 2, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 13, 14, },
  { 3, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 3, 5, 6, 7, 10, 11, 13, 14, },
  { 1, 3, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 13, 14, },
  { 2, 3, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 13, 14, },
  { 4, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 1, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 2, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 3, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 13, 14, },
  { 8, 10, 11, 13, 14, },
  { 0, 8, 10, 11, 13, 14, },
  { 1, 8, 10, 11, 13, 14, },
  { 0, 1, 8, 10, 11, 13, 14, },
  { 2, 8, 10, 11, 13, 14, },
  { 0, 2, 8, 10, 11, 13, 14, },
  { 1, 2, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 8, 10, 11, 13, 14, },
  { 3, 8, 10, 11, 13, 14, },
  { 0, 3, 8, 10, 11, 13, 14, },
  { 1, 3, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 8, 10, 11, 13, 14, },
  { 2, 3, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 8, 10, 11, 13, 14, },
  { 4, 8, 10, 11, 13, 14, },
  { 0, 4, 8, 10, 11, 13, 14, },
  { 1, 4, 8, 10, 11, 13, 14, },
  { 0, 1, 4, 8, 10, 11, 13, 14, },
  { 2, 4, 8, 10, 11, 13, 14, },
  { 0, 2, 4, 8, 10, 11, 13, 14, },
  { 1, 2, 4, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 8, 10, 11, 13, 14, },
  { 3, 4, 8, 10, 11, 13, 14, },
  { 0, 3, 4, 8, 10, 11, 13, 14, },
  { 1, 3, 4, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 8, 10, 11, 13, 14, },
  { 2, 3, 4, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 13, 14, },
  { 5, 8, 10, 11, 13, 14, },
  { 0, 5, 8, 10, 11, 13, 14, },
  { 1, 5, 8, 10, 11, 13, 14, },
  { 0, 1, 5, 8, 10, 11, 13, 14, },
  { 2, 5, 8, 10, 11, 13, 14, },
  { 0, 2, 5, 8, 10, 11, 13, 14, },
  { 1, 2, 5, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 8, 10, 11, 13, 14, },
  { 3, 5, 8, 10, 11, 13, 14, },
  { 0, 3, 5, 8, 10, 11, 13, 14, },
  { 1, 3, 5, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 8, 10, 11, 13, 14, },
  { 2, 3, 5, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 13, 14, },
  { 4, 5, 8, 10, 11, 13, 14, },
  { 0, 4, 5, 8, 10, 11, 13, 14, },
  { 1, 4, 5, 8, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 8, 10, 11, 13, 14, },
  { 2, 4, 5, 8, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 8, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 13, 14, },
  { 3, 4, 5, 8, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 8, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 13, 14, },
  { 6, 8, 10, 11, 13, 14, },
  { 0, 6, 8, 10, 11, 13, 14, },
  { 1, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 6, 8, 10, 11, 13, 14, },
  { 2, 6, 8, 10, 11, 13, 14, },
  { 0, 2, 6, 8, 10, 11, 13, 14, },
  { 1, 2, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 6, 8, 10, 11, 13, 14, },
  { 3, 6, 8, 10, 11, 13, 14, },
  { 0, 3, 6, 8, 10, 11, 13, 14, },
  { 1, 3, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 6, 8, 10, 11, 13, 14, },
  { 2, 3, 6, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 6, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 13, 14, },
  { 4, 6, 8, 10, 11, 13, 14, },
  { 0, 4, 6, 8, 10, 11, 13, 14, },
  { 1, 4, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 4, 6, 8, 10, 11, 13, 14, },
  { 2, 4, 6, 8, 10, 11, 13, 14, },
  { 0, 2, 4, 6, 8, 10, 11, 13, 14, },
  { 1, 2, 4, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 13, 14, },
  { 3, 4, 6, 8, 10, 11, 13, 14, },
  { 0, 3, 4, 6, 8, 10, 11, 13, 14, },
  { 1, 3, 4, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 13, 14, },
  { 2, 3, 4, 6, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 13, 14, },
  { 5, 6, 8, 10, 11, 13, 14, },
  { 0, 5, 6, 8, 10, 11, 13, 14, },
  { 1, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 5, 6, 8, 10, 11, 13, 14, },
  { 2, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 2, 5, 6, 8, 10, 11, 13, 14, },
  { 1, 2, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 13, 14, },
  { 3, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 3, 5, 6, 8, 10, 11, 13, 14, },
  { 1, 3, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 13, 14, },
  { 2, 3, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 13, 14, },
  { 4, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 1, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 2, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 3, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 13, 14, },
  { 7, 8, 10, 11, 13, 14, },
  { 0, 7, 8, 10, 11, 13, 14, },
  { 1, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 7, 8, 10, 11, 13, 14, },
  { 2, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 7, 8, 10, 11, 13, 14, },
  { 3, 7, 8, 10, 11, 13, 14, },
  { 0, 3, 7, 8, 10, 11, 13, 14, },
  { 1, 3, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 7, 8, 10, 11, 13, 14, },
  { 2, 3, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 13, 14, },
  { 4, 7, 8, 10, 11, 13, 14, },
  { 0, 4, 7, 8, 10, 11, 13, 14, },
  { 1, 4, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 4, 7, 8, 10, 11, 13, 14, },
  { 2, 4, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 4, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 4, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 13, 14, },
  { 3, 4, 7, 8, 10, 11, 13, 14, },
  { 0, 3, 4, 7, 8, 10, 11, 13, 14, },
  { 1, 3, 4, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 13, 14, },
  { 2, 3, 4, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 13, 14, },
  { 5, 7, 8, 10, 11, 13, 14, },
  { 0, 5, 7, 8, 10, 11, 13, 14, },
  { 1, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 5, 7, 8, 10, 11, 13, 14, },
  { 2, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 5, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 13, 14, },
  { 3, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 3, 5, 7, 8, 10, 11, 13, 14, },
  { 1, 3, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 13, 14, },
  { 2, 3, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 13, 14, },
  { 4, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 1, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 2, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 3, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 13, 14, },
  { 6, 7, 8, 10, 11, 13, 14, },
  { 0, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 6, 7, 8, 10, 11, 13, 14, },
  { 2, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 13, 14, },
  { 3, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 3, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 3, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 13, 14, },
  { 2, 3, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 13, 14, },
  { 4, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 2, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 3, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 13, 14, },
  { 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 2, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 3, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, },
  { 9, 10, 11, 13, 14, },
  { 0, 9, 10, 11, 13, 14, },
  { 1, 9, 10, 11, 13, 14, },
  { 0, 1, 9, 10, 11, 13, 14, },
  { 2, 9, 10, 11, 13, 14, },
  { 0, 2, 9, 10, 11, 13, 14, },
  { 1, 2, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 9, 10, 11, 13, 14, },
  { 3, 9, 10, 11, 13, 14, },
  { 0, 3, 9, 10, 11, 13, 14, },
  { 1, 3, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 9, 10, 11, 13, 14, },
  { 2, 3, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 9, 10, 11, 13, 14, },
  { 4, 9, 10, 11, 13, 14, },
  { 0, 4, 9, 10, 11, 13, 14, },
  { 1, 4, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 9, 10, 11, 13, 14, },
  { 2, 4, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 9, 10, 11, 13, 14, },
  { 3, 4, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 13, 14, },
  { 5, 9, 10, 11, 13, 14, },
  { 0, 5, 9, 10, 11, 13, 14, },
  { 1, 5, 9, 10, 11, 13, 14, },
  { 0, 1, 5, 9, 10, 11, 13, 14, },
  { 2, 5, 9, 10, 11, 13, 14, },
  { 0, 2, 5, 9, 10, 11, 13, 14, },
  { 1, 2, 5, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 9, 10, 11, 13, 14, },
  { 3, 5, 9, 10, 11, 13, 14, },
  { 0, 3, 5, 9, 10, 11, 13, 14, },
  { 1, 3, 5, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 9, 10, 11, 13, 14, },
  { 2, 3, 5, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 13, 14, },
  { 4, 5, 9, 10, 11, 13, 14, },
  { 0, 4, 5, 9, 10, 11, 13, 14, },
  { 1, 4, 5, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 9, 10, 11, 13, 14, },
  { 2, 4, 5, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 13, 14, },
  { 3, 4, 5, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 13, 14, },
  { 6, 9, 10, 11, 13, 14, },
  { 0, 6, 9, 10, 11, 13, 14, },
  { 1, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 6, 9, 10, 11, 13, 14, },
  { 2, 6, 9, 10, 11, 13, 14, },
  { 0, 2, 6, 9, 10, 11, 13, 14, },
  { 1, 2, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 6, 9, 10, 11, 13, 14, },
  { 3, 6, 9, 10, 11, 13, 14, },
  { 0, 3, 6, 9, 10, 11, 13, 14, },
  { 1, 3, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 6, 9, 10, 11, 13, 14, },
  { 2, 3, 6, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 6, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 13, 14, },
  { 4, 6, 9, 10, 11, 13, 14, },
  { 0, 4, 6, 9, 10, 11, 13, 14, },
  { 1, 4, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 6, 9, 10, 11, 13, 14, },
  { 2, 4, 6, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 6, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 13, 14, },
  { 3, 4, 6, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 6, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 6, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 13, 14, },
  { 5, 6, 9, 10, 11, 13, 14, },
  { 0, 5, 6, 9, 10, 11, 13, 14, },
  { 1, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 5, 6, 9, 10, 11, 13, 14, },
  { 2, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 2, 5, 6, 9, 10, 11, 13, 14, },
  { 1, 2, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 13, 14, },
  { 3, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 3, 5, 6, 9, 10, 11, 13, 14, },
  { 1, 3, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 13, 14, },
  { 2, 3, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 13, 14, },
  { 4, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 1, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 2, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 3, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, },
  { 7, 9, 10, 11, 13, 14, },
  { 0, 7, 9, 10, 11, 13, 14, },
  { 1, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 7, 9, 10, 11, 13, 14, },
  { 2, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 7, 9, 10, 11, 13, 14, },
  { 3, 7, 9, 10, 11, 13, 14, },
  { 0, 3, 7, 9, 10, 11, 13, 14, },
  { 1, 3, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 7, 9, 10, 11, 13, 14, },
  { 2, 3, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 13, 14, },
  { 4, 7, 9, 10, 11, 13, 14, },
  { 0, 4, 7, 9, 10, 11, 13, 14, },
  { 1, 4, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 7, 9, 10, 11, 13, 14, },
  { 2, 4, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 13, 14, },
  { 3, 4, 7, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 7, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 13, 14, },
  { 5, 7, 9, 10, 11, 13, 14, },
  { 0, 5, 7, 9, 10, 11, 13, 14, },
  { 1, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 5, 7, 9, 10, 11, 13, 14, },
  { 2, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 5, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 13, 14, },
  { 3, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 3, 5, 7, 9, 10, 11, 13, 14, },
  { 1, 3, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 13, 14, },
  { 2, 3, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 13, 14, },
  { 4, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 1, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 2, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 3, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 13, 14, },
  { 6, 7, 9, 10, 11, 13, 14, },
  { 0, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 6, 7, 9, 10, 11, 13, 14, },
  { 2, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 13, 14, },
  { 3, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 3, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 3, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 13, 14, },
  { 2, 3, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 13, 14, },
  { 4, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 2, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 3, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 13, 14, },
  { 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 2, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 3, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, },
  { 8, 9, 10, 11, 13, 14, },
  { 0, 8, 9, 10, 11, 13, 14, },
  { 1, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 8, 9, 10, 11, 13, 14, },
  { 2, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 8, 9, 10, 11, 13, 14, },
  { 3, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 13, 14, },
  { 4, 8, 9, 10, 11, 13, 14, },
  { 0, 4, 8, 9, 10, 11, 13, 14, },
  { 1, 4, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 8, 9, 10, 11, 13, 14, },
  { 2, 4, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 13, 14, },
  { 3, 4, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 13, 14, },
  { 5, 8, 9, 10, 11, 13, 14, },
  { 0, 5, 8, 9, 10, 11, 13, 14, },
  { 1, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 5, 8, 9, 10, 11, 13, 14, },
  { 2, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 5, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 13, 14, },
  { 3, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 5, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 13, 14, },
  { 4, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 1, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 2, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 3, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 13, 14, },
  { 6, 8, 9, 10, 11, 13, 14, },
  { 0, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 6, 8, 9, 10, 11, 13, 14, },
  { 2, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 13, 14, },
  { 3, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 13, 14, },
  { 4, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 2, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 3, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 13, 14, },
  { 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 2, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 3, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, },
  { 7, 8, 9, 10, 11, 13, 14, },
  { 0, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 13, 14, },
  { 3, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 13, 14, },
  { 4, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 3, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 13, 14, },
  { 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 3, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, },
  { 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 3, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, },
  { 12, 13, 14, },
  { 0, 12, 13, 14, },
  { 1, 12, 13, 14, },
  { 0, 1, 12, 13, 14, },
  { 2, 12, 13, 14, },
  { 0, 2, 12, 13, 14, },
  { 1, 2, 12, 13, 14, },
  { 0, 1, 2, 12, 13, 14, },
  { 3, 12, 13, 14, },
  { 0, 3, 12, 13, 14, },
  { 1, 3, 12, 13, 14, },
  { 0, 1, 3, 12, 13, 14, },
  { 2, 3, 12, 13, 14, },
  { 0, 2, 3, 12, 13, 14, },
  { 1, 2, 3, 12, 13, 14, },
  { 0, 1, 2, 3, 12, 13, 14, },
  { 4, 12, 13, 14, },
  { 0, 4, 12, 13, 14, },
  { 1, 4, 12, 13, 14, },
  { 0, 1, 4, 12, 13, 14, },
  { 2, 4, 12, 13, 14, },
  { 0, 2, 4, 12, 13, 14, },
  { 1, 2, 4, 12, 13, 14, },
  { 0, 1, 2, 4, 12, 13, 14, },
  { 3, 4, 12, 13, 14, },
  { 0, 3, 4, 12, 13, 14, },
  { 1, 3, 4, 12, 13, 14, },
  { 0, 1, 3, 4, 12, 13, 14, },
  { 2, 3, 4, 12, 13, 14, },
  { 0, 2, 3, 4, 12, 13, 14, },
  { 1, 2, 3, 4, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 12, 13, 14, },
  { 5, 12, 13, 14, },
  { 0, 5, 12, 13, 14, },
  { 1, 5, 12, 13, 14, },
  { 0, 1, 5, 12, 13, 14, },
  { 2, 5, 12, 13, 14, },
  { 0, 2, 5, 12, 13, 14, },
  { 1, 2, 5, 12, 13, 14, },
  { 0, 1, 2, 5, 12, 13, 14, },
  { 3, 5, 12, 13, 14, },
  { 0, 3, 5, 12, 13, 14, },
  { 1, 3, 5, 12, 13, 14, },
  { 0, 1, 3, 5, 12, 13, 14, },
  { 2, 3, 5, 12, 13, 14, },
  { 0, 2, 3, 5, 12, 13, 14, },
  { 1, 2, 3, 5, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 12, 13, 14, },
  { 4, 5, 12, 13, 14, },
  { 0, 4, 5, 12, 13, 14, },
  { 1, 4, 5, 12, 13, 14, },
  { 0, 1, 4, 5, 12, 13, 14, },
  { 2, 4, 5, 12, 13, 14, },
  { 0, 2, 4, 5, 12, 13, 14, },
  { 1, 2, 4, 5, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 12, 13, 14, },
  { 3, 4, 5, 12, 13, 14, },
  { 0, 3, 4, 5, 12, 13, 14, },
  { 1, 3, 4, 5, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 12, 13, 14, },
  { 2, 3, 4, 5, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 12, 13, 14, },
  { 6, 12, 13, 14, },
  { 0, 6, 12, 13, 14, },
  { 1, 6, 12, 13, 14, },
  { 0, 1, 6, 12, 13, 14, },
  { 2, 6, 12, 13, 14, },
  { 0, 2, 6, 12, 13, 14, },
  { 1, 2, 6, 12, 13, 14, },
  { 0, 1, 2, 6, 12, 13, 14, },
  { 3, 6, 12, 13, 14, },
  { 0, 3, 6, 12, 13, 14, },
  { 1, 3, 6, 12, 13, 14, },
  { 0, 1, 3, 6, 12, 13, 14, },
  { 2, 3, 6, 12, 13, 14, },
  { 0, 2, 3, 6, 12, 13, 14, },
  { 1, 2, 3, 6, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 12, 13, 14, },
  { 4, 6, 12, 13, 14, },
  { 0, 4, 6, 12, 13, 14, },
  { 1, 4, 6, 12, 13, 14, },
  { 0, 1, 4, 6, 12, 13, 14, },
  { 2, 4, 6, 12, 13, 14, },
  { 0, 2, 4, 6, 12, 13, 14, },
  { 1, 2, 4, 6, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 12, 13, 14, },
  { 3, 4, 6, 12, 13, 14, },
  { 0, 3, 4, 6, 12, 13, 14, },
  { 1, 3, 4, 6, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 12, 13, 14, },
  { 2, 3, 4, 6, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 12, 13, 14, },
  { 5, 6, 12, 13, 14, },
  { 0, 5, 6, 12, 13, 14, },
  { 1, 5, 6, 12, 13, 14, },
  { 0, 1, 5, 6, 12, 13, 14, },
  { 2, 5, 6, 12, 13, 14, },
  { 0, 2, 5, 6, 12, 13, 14, },
  { 1, 2, 5, 6, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 12, 13, 14, },
  { 3, 5, 6, 12, 13, 14, },
  { 0, 3, 5, 6, 12, 13, 14, },
  { 1, 3, 5, 6, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 12, 13, 14, },
  { 2, 3, 5, 6, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 12, 13, 14, },
  { 4, 5, 6, 12, 13, 14, },
  { 0, 4, 5, 6, 12, 13, 14, },
  { 1, 4, 5, 6, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 12, 13, 14, },
  { 2, 4, 5, 6, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 12, 13, 14, },
  { 3, 4, 5, 6, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 12, 13, 14, },
  { 7, 12, 13, 14, },
  { 0, 7, 12, 13, 14, },
  { 1, 7, 12, 13, 14, },
  { 0, 1, 7, 12, 13, 14, },
  { 2, 7, 12, 13, 14, },
  { 0, 2, 7, 12, 13, 14, },
  { 1, 2, 7, 12, 13, 14, },
  { 0, 1, 2, 7, 12, 13, 14, },
  { 3, 7, 12, 13, 14, },
  { 0, 3, 7, 12, 13, 14, },
  { 1, 3, 7, 12, 13, 14, },
  { 0, 1, 3, 7, 12, 13, 14, },
  { 2, 3, 7, 12, 13, 14, },
  { 0, 2, 3, 7, 12, 13, 14, },
  { 1, 2, 3, 7, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 12, 13, 14, },
  { 4, 7, 12, 13, 14, },
  { 0, 4, 7, 12, 13, 14, },
  { 1, 4, 7, 12, 13, 14, },
  { 0, 1, 4, 7, 12, 13, 14, },
  { 2, 4, 7, 12, 13, 14, },
  { 0, 2, 4, 7, 12, 13, 14, },
  { 1, 2, 4, 7, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 12, 13, 14, },
  { 3, 4, 7, 12, 13, 14, },
  { 0, 3, 4, 7, 12, 13, 14, },
  { 1, 3, 4, 7, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 12, 13, 14, },
  { 2, 3, 4, 7, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 12, 13, 14, },
  { 5, 7, 12, 13, 14, },
  { 0, 5, 7, 12, 13, 14, },
  { 1, 5, 7, 12, 13, 14, },
  { 0, 1, 5, 7, 12, 13, 14, },
  { 2, 5, 7, 12, 13, 14, },
  { 0, 2, 5, 7, 12, 13, 14, },
  { 1, 2, 5, 7, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 12, 13, 14, },
  { 3, 5, 7, 12, 13, 14, },
  { 0, 3, 5, 7, 12, 13, 14, },
  { 1, 3, 5, 7, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 12, 13, 14, },
  { 2, 3, 5, 7, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 12, 13, 14, },
  { 4, 5, 7, 12, 13, 14, },
  { 0, 4, 5, 7, 12, 13, 14, },
  { 1, 4, 5, 7, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 12, 13, 14, },
  { 2, 4, 5, 7, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 12, 13, 14, },
  { 3, 4, 5, 7, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 12, 13, 14, },
  { 6, 7, 12, 13, 14, },
  { 0, 6, 7, 12, 13, 14, },
  { 1, 6, 7, 12, 13, 14, },
  { 0, 1, 6, 7, 12, 13, 14, },
  { 2, 6, 7, 12, 13, 14, },
  { 0, 2, 6, 7, 12, 13, 14, },
  { 1, 2, 6, 7, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 12, 13, 14, },
  { 3, 6, 7, 12, 13, 14, },
  { 0, 3, 6, 7, 12, 13, 14, },
  { 1, 3, 6, 7, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 12, 13, 14, },
  { 2, 3, 6, 7, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 12, 13, 14, },
  { 4, 6, 7, 12, 13, 14, },
  { 0, 4, 6, 7, 12, 13, 14, },
  { 1, 4, 6, 7, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 12, 13, 14, },
  { 2, 4, 6, 7, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 12, 13, 14, },
  { 3, 4, 6, 7, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 12, 13, 14, },
  { 5, 6, 7, 12, 13, 14, },
  { 0, 5, 6, 7, 12, 13, 14, },
  { 1, 5, 6, 7, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 12, 13, 14, },
  { 2, 5, 6, 7, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 12, 13, 14, },
  { 3, 5, 6, 7, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 12, 13, 14, },
  { 4, 5, 6, 7, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 12, 13, 14, },
  { 8, 12, 13, 14, },
  { 0, 8, 12, 13, 14, },
  { 1, 8, 12, 13, 14, },
  { 0, 1, 8, 12, 13, 14, },
  { 2, 8, 12, 13, 14, },
  { 0, 2, 8, 12, 13, 14, },
  { 1, 2, 8, 12, 13, 14, },
  { 0, 1, 2, 8, 12, 13, 14, },
  { 3, 8, 12, 13, 14, },
  { 0, 3, 8, 12, 13, 14, },
  { 1, 3, 8, 12, 13, 14, },
  { 0, 1, 3, 8, 12, 13, 14, },
  { 2, 3, 8, 12, 13, 14, },
  { 0, 2, 3, 8, 12, 13, 14, },
  { 1, 2, 3, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 8, 12, 13, 14, },
  { 4, 8, 12, 13, 14, },
  { 0, 4, 8, 12, 13, 14, },
  { 1, 4, 8, 12, 13, 14, },
  { 0, 1, 4, 8, 12, 13, 14, },
  { 2, 4, 8, 12, 13, 14, },
  { 0, 2, 4, 8, 12, 13, 14, },
  { 1, 2, 4, 8, 12, 13, 14, },
  { 0, 1, 2, 4, 8, 12, 13, 14, },
  { 3, 4, 8, 12, 13, 14, },
  { 0, 3, 4, 8, 12, 13, 14, },
  { 1, 3, 4, 8, 12, 13, 14, },
  { 0, 1, 3, 4, 8, 12, 13, 14, },
  { 2, 3, 4, 8, 12, 13, 14, },
  { 0, 2, 3, 4, 8, 12, 13, 14, },
  { 1, 2, 3, 4, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 12, 13, 14, },
  { 5, 8, 12, 13, 14, },
  { 0, 5, 8, 12, 13, 14, },
  { 1, 5, 8, 12, 13, 14, },
  { 0, 1, 5, 8, 12, 13, 14, },
  { 2, 5, 8, 12, 13, 14, },
  { 0, 2, 5, 8, 12, 13, 14, },
  { 1, 2, 5, 8, 12, 13, 14, },
  { 0, 1, 2, 5, 8, 12, 13, 14, },
  { 3, 5, 8, 12, 13, 14, },
  { 0, 3, 5, 8, 12, 13, 14, },
  { 1, 3, 5, 8, 12, 13, 14, },
  { 0, 1, 3, 5, 8, 12, 13, 14, },
  { 2, 3, 5, 8, 12, 13, 14, },
  { 0, 2, 3, 5, 8, 12, 13, 14, },
  { 1, 2, 3, 5, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 12, 13, 14, },
  { 4, 5, 8, 12, 13, 14, },
  { 0, 4, 5, 8, 12, 13, 14, },
  { 1, 4, 5, 8, 12, 13, 14, },
  { 0, 1, 4, 5, 8, 12, 13, 14, },
  { 2, 4, 5, 8, 12, 13, 14, },
  { 0, 2, 4, 5, 8, 12, 13, 14, },
  { 1, 2, 4, 5, 8, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 12, 13, 14, },
  { 3, 4, 5, 8, 12, 13, 14, },
  { 0, 3, 4, 5, 8, 12, 13, 14, },
  { 1, 3, 4, 5, 8, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 12, 13, 14, },
  { 2, 3, 4, 5, 8, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 12, 13, 14, },
  { 6, 8, 12, 13, 14, },
  { 0, 6, 8, 12, 13, 14, },
  { 1, 6, 8, 12, 13, 14, },
  { 0, 1, 6, 8, 12, 13, 14, },
  { 2, 6, 8, 12, 13, 14, },
  { 0, 2, 6, 8, 12, 13, 14, },
  { 1, 2, 6, 8, 12, 13, 14, },
  { 0, 1, 2, 6, 8, 12, 13, 14, },
  { 3, 6, 8, 12, 13, 14, },
  { 0, 3, 6, 8, 12, 13, 14, },
  { 1, 3, 6, 8, 12, 13, 14, },
  { 0, 1, 3, 6, 8, 12, 13, 14, },
  { 2, 3, 6, 8, 12, 13, 14, },
  { 0, 2, 3, 6, 8, 12, 13, 14, },
  { 1, 2, 3, 6, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 12, 13, 14, },
  { 4, 6, 8, 12, 13, 14, },
  { 0, 4, 6, 8, 12, 13, 14, },
  { 1, 4, 6, 8, 12, 13, 14, },
  { 0, 1, 4, 6, 8, 12, 13, 14, },
  { 2, 4, 6, 8, 12, 13, 14, },
  { 0, 2, 4, 6, 8, 12, 13, 14, },
  { 1, 2, 4, 6, 8, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 12, 13, 14, },
  { 3, 4, 6, 8, 12, 13, 14, },
  { 0, 3, 4, 6, 8, 12, 13, 14, },
  { 1, 3, 4, 6, 8, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 12, 13, 14, },
  { 2, 3, 4, 6, 8, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 12, 13, 14, },
  { 5, 6, 8, 12, 13, 14, },
  { 0, 5, 6, 8, 12, 13, 14, },
  { 1, 5, 6, 8, 12, 13, 14, },
  { 0, 1, 5, 6, 8, 12, 13, 14, },
  { 2, 5, 6, 8, 12, 13, 14, },
  { 0, 2, 5, 6, 8, 12, 13, 14, },
  { 1, 2, 5, 6, 8, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 12, 13, 14, },
  { 3, 5, 6, 8, 12, 13, 14, },
  { 0, 3, 5, 6, 8, 12, 13, 14, },
  { 1, 3, 5, 6, 8, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 12, 13, 14, },
  { 2, 3, 5, 6, 8, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 12, 13, 14, },
  { 4, 5, 6, 8, 12, 13, 14, },
  { 0, 4, 5, 6, 8, 12, 13, 14, },
  { 1, 4, 5, 6, 8, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 12, 13, 14, },
  { 2, 4, 5, 6, 8, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 12, 13, 14, },
  { 3, 4, 5, 6, 8, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 12, 13, 14, },
  { 7, 8, 12, 13, 14, },
  { 0, 7, 8, 12, 13, 14, },
  { 1, 7, 8, 12, 13, 14, },
  { 0, 1, 7, 8, 12, 13, 14, },
  { 2, 7, 8, 12, 13, 14, },
  { 0, 2, 7, 8, 12, 13, 14, },
  { 1, 2, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 7, 8, 12, 13, 14, },
  { 3, 7, 8, 12, 13, 14, },
  { 0, 3, 7, 8, 12, 13, 14, },
  { 1, 3, 7, 8, 12, 13, 14, },
  { 0, 1, 3, 7, 8, 12, 13, 14, },
  { 2, 3, 7, 8, 12, 13, 14, },
  { 0, 2, 3, 7, 8, 12, 13, 14, },
  { 1, 2, 3, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 12, 13, 14, },
  { 4, 7, 8, 12, 13, 14, },
  { 0, 4, 7, 8, 12, 13, 14, },
  { 1, 4, 7, 8, 12, 13, 14, },
  { 0, 1, 4, 7, 8, 12, 13, 14, },
  { 2, 4, 7, 8, 12, 13, 14, },
  { 0, 2, 4, 7, 8, 12, 13, 14, },
  { 1, 2, 4, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 12, 13, 14, },
  { 3, 4, 7, 8, 12, 13, 14, },
  { 0, 3, 4, 7, 8, 12, 13, 14, },
  { 1, 3, 4, 7, 8, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 12, 13, 14, },
  { 2, 3, 4, 7, 8, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 12, 13, 14, },
  { 5, 7, 8, 12, 13, 14, },
  { 0, 5, 7, 8, 12, 13, 14, },
  { 1, 5, 7, 8, 12, 13, 14, },
  { 0, 1, 5, 7, 8, 12, 13, 14, },
  { 2, 5, 7, 8, 12, 13, 14, },
  { 0, 2, 5, 7, 8, 12, 13, 14, },
  { 1, 2, 5, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 12, 13, 14, },
  { 3, 5, 7, 8, 12, 13, 14, },
  { 0, 3, 5, 7, 8, 12, 13, 14, },
  { 1, 3, 5, 7, 8, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 12, 13, 14, },
  { 2, 3, 5, 7, 8, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 12, 13, 14, },
  { 4, 5, 7, 8, 12, 13, 14, },
  { 0, 4, 5, 7, 8, 12, 13, 14, },
  { 1, 4, 5, 7, 8, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 12, 13, 14, },
  { 2, 4, 5, 7, 8, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 12, 13, 14, },
  { 3, 4, 5, 7, 8, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 12, 13, 14, },
  { 6, 7, 8, 12, 13, 14, },
  { 0, 6, 7, 8, 12, 13, 14, },
  { 1, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 6, 7, 8, 12, 13, 14, },
  { 2, 6, 7, 8, 12, 13, 14, },
  { 0, 2, 6, 7, 8, 12, 13, 14, },
  { 1, 2, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 12, 13, 14, },
  { 3, 6, 7, 8, 12, 13, 14, },
  { 0, 3, 6, 7, 8, 12, 13, 14, },
  { 1, 3, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 12, 13, 14, },
  { 2, 3, 6, 7, 8, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 12, 13, 14, },
  { 4, 6, 7, 8, 12, 13, 14, },
  { 0, 4, 6, 7, 8, 12, 13, 14, },
  { 1, 4, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 12, 13, 14, },
  { 2, 4, 6, 7, 8, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 12, 13, 14, },
  { 3, 4, 6, 7, 8, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 12, 13, 14, },
  { 5, 6, 7, 8, 12, 13, 14, },
  { 0, 5, 6, 7, 8, 12, 13, 14, },
  { 1, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 12, 13, 14, },
  { 2, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 12, 13, 14, },
  { 3, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 12, 13, 14, },
  { 4, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, },
  { 9, 12, 13, 14, },
  { 0, 9, 12, 13, 14, },
  { 1, 9, 12, 13, 14, },
  { 0, 1, 9, 12, 13, 14, },
  { 2, 9, 12, 13, 14, },
  { 0, 2, 9, 12, 13, 14, },
  { 1, 2, 9, 12, 13, 14, },
  { 0, 1, 2, 9, 12, 13, 14, },
  { 3, 9, 12, 13, 14, },
  { 0, 3, 9, 12, 13, 14, },
  { 1, 3, 9, 12, 13, 14, },
  { 0, 1, 3, 9, 12, 13, 14, },
  { 2, 3, 9, 12, 13, 14, },
  { 0, 2, 3, 9, 12, 13, 14, },
  { 1, 2, 3, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 9, 12, 13, 14, },
  { 4, 9, 12, 13, 14, },
  { 0, 4, 9, 12, 13, 14, },
  { 1, 4, 9, 12, 13, 14, },
  { 0, 1, 4, 9, 12, 13, 14, },
  { 2, 4, 9, 12, 13, 14, },
  { 0, 2, 4, 9, 12, 13, 14, },
  { 1, 2, 4, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 9, 12, 13, 14, },
  { 3, 4, 9, 12, 13, 14, },
  { 0, 3, 4, 9, 12, 13, 14, },
  { 1, 3, 4, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 9, 12, 13, 14, },
  { 2, 3, 4, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 9, 12, 13, 14, },
  { 5, 9, 12, 13, 14, },
  { 0, 5, 9, 12, 13, 14, },
  { 1, 5, 9, 12, 13, 14, },
  { 0, 1, 5, 9, 12, 13, 14, },
  { 2, 5, 9, 12, 13, 14, },
  { 0, 2, 5, 9, 12, 13, 14, },
  { 1, 2, 5, 9, 12, 13, 14, },
  { 0, 1, 2, 5, 9, 12, 13, 14, },
  { 3, 5, 9, 12, 13, 14, },
  { 0, 3, 5, 9, 12, 13, 14, },
  { 1, 3, 5, 9, 12, 13, 14, },
  { 0, 1, 3, 5, 9, 12, 13, 14, },
  { 2, 3, 5, 9, 12, 13, 14, },
  { 0, 2, 3, 5, 9, 12, 13, 14, },
  { 1, 2, 3, 5, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 9, 12, 13, 14, },
  { 4, 5, 9, 12, 13, 14, },
  { 0, 4, 5, 9, 12, 13, 14, },
  { 1, 4, 5, 9, 12, 13, 14, },
  { 0, 1, 4, 5, 9, 12, 13, 14, },
  { 2, 4, 5, 9, 12, 13, 14, },
  { 0, 2, 4, 5, 9, 12, 13, 14, },
  { 1, 2, 4, 5, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 9, 12, 13, 14, },
  { 3, 4, 5, 9, 12, 13, 14, },
  { 0, 3, 4, 5, 9, 12, 13, 14, },
  { 1, 3, 4, 5, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 9, 12, 13, 14, },
  { 2, 3, 4, 5, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 12, 13, 14, },
  { 6, 9, 12, 13, 14, },
  { 0, 6, 9, 12, 13, 14, },
  { 1, 6, 9, 12, 13, 14, },
  { 0, 1, 6, 9, 12, 13, 14, },
  { 2, 6, 9, 12, 13, 14, },
  { 0, 2, 6, 9, 12, 13, 14, },
  { 1, 2, 6, 9, 12, 13, 14, },
  { 0, 1, 2, 6, 9, 12, 13, 14, },
  { 3, 6, 9, 12, 13, 14, },
  { 0, 3, 6, 9, 12, 13, 14, },
  { 1, 3, 6, 9, 12, 13, 14, },
  { 0, 1, 3, 6, 9, 12, 13, 14, },
  { 2, 3, 6, 9, 12, 13, 14, },
  { 0, 2, 3, 6, 9, 12, 13, 14, },
  { 1, 2, 3, 6, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 9, 12, 13, 14, },
  { 4, 6, 9, 12, 13, 14, },
  { 0, 4, 6, 9, 12, 13, 14, },
  { 1, 4, 6, 9, 12, 13, 14, },
  { 0, 1, 4, 6, 9, 12, 13, 14, },
  { 2, 4, 6, 9, 12, 13, 14, },
  { 0, 2, 4, 6, 9, 12, 13, 14, },
  { 1, 2, 4, 6, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 9, 12, 13, 14, },
  { 3, 4, 6, 9, 12, 13, 14, },
  { 0, 3, 4, 6, 9, 12, 13, 14, },
  { 1, 3, 4, 6, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 9, 12, 13, 14, },
  { 2, 3, 4, 6, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 12, 13, 14, },
  { 5, 6, 9, 12, 13, 14, },
  { 0, 5, 6, 9, 12, 13, 14, },
  { 1, 5, 6, 9, 12, 13, 14, },
  { 0, 1, 5, 6, 9, 12, 13, 14, },
  { 2, 5, 6, 9, 12, 13, 14, },
  { 0, 2, 5, 6, 9, 12, 13, 14, },
  { 1, 2, 5, 6, 9, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 9, 12, 13, 14, },
  { 3, 5, 6, 9, 12, 13, 14, },
  { 0, 3, 5, 6, 9, 12, 13, 14, },
  { 1, 3, 5, 6, 9, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 9, 12, 13, 14, },
  { 2, 3, 5, 6, 9, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 9, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 12, 13, 14, },
  { 4, 5, 6, 9, 12, 13, 14, },
  { 0, 4, 5, 6, 9, 12, 13, 14, },
  { 1, 4, 5, 6, 9, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 9, 12, 13, 14, },
  { 2, 4, 5, 6, 9, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 9, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 12, 13, 14, },
  { 3, 4, 5, 6, 9, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 9, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 12, 13, 14, },
  { 7, 9, 12, 13, 14, },
  { 0, 7, 9, 12, 13, 14, },
  { 1, 7, 9, 12, 13, 14, },
  { 0, 1, 7, 9, 12, 13, 14, },
  { 2, 7, 9, 12, 13, 14, },
  { 0, 2, 7, 9, 12, 13, 14, },
  { 1, 2, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 7, 9, 12, 13, 14, },
  { 3, 7, 9, 12, 13, 14, },
  { 0, 3, 7, 9, 12, 13, 14, },
  { 1, 3, 7, 9, 12, 13, 14, },
  { 0, 1, 3, 7, 9, 12, 13, 14, },
  { 2, 3, 7, 9, 12, 13, 14, },
  { 0, 2, 3, 7, 9, 12, 13, 14, },
  { 1, 2, 3, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 9, 12, 13, 14, },
  { 4, 7, 9, 12, 13, 14, },
  { 0, 4, 7, 9, 12, 13, 14, },
  { 1, 4, 7, 9, 12, 13, 14, },
  { 0, 1, 4, 7, 9, 12, 13, 14, },
  { 2, 4, 7, 9, 12, 13, 14, },
  { 0, 2, 4, 7, 9, 12, 13, 14, },
  { 1, 2, 4, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 9, 12, 13, 14, },
  { 3, 4, 7, 9, 12, 13, 14, },
  { 0, 3, 4, 7, 9, 12, 13, 14, },
  { 1, 3, 4, 7, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 9, 12, 13, 14, },
  { 2, 3, 4, 7, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 12, 13, 14, },
  { 5, 7, 9, 12, 13, 14, },
  { 0, 5, 7, 9, 12, 13, 14, },
  { 1, 5, 7, 9, 12, 13, 14, },
  { 0, 1, 5, 7, 9, 12, 13, 14, },
  { 2, 5, 7, 9, 12, 13, 14, },
  { 0, 2, 5, 7, 9, 12, 13, 14, },
  { 1, 2, 5, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 9, 12, 13, 14, },
  { 3, 5, 7, 9, 12, 13, 14, },
  { 0, 3, 5, 7, 9, 12, 13, 14, },
  { 1, 3, 5, 7, 9, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 9, 12, 13, 14, },
  { 2, 3, 5, 7, 9, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 9, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 12, 13, 14, },
  { 4, 5, 7, 9, 12, 13, 14, },
  { 0, 4, 5, 7, 9, 12, 13, 14, },
  { 1, 4, 5, 7, 9, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 9, 12, 13, 14, },
  { 2, 4, 5, 7, 9, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 9, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 12, 13, 14, },
  { 3, 4, 5, 7, 9, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 9, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 12, 13, 14, },
  { 6, 7, 9, 12, 13, 14, },
  { 0, 6, 7, 9, 12, 13, 14, },
  { 1, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 6, 7, 9, 12, 13, 14, },
  { 2, 6, 7, 9, 12, 13, 14, },
  { 0, 2, 6, 7, 9, 12, 13, 14, },
  { 1, 2, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 9, 12, 13, 14, },
  { 3, 6, 7, 9, 12, 13, 14, },
  { 0, 3, 6, 7, 9, 12, 13, 14, },
  { 1, 3, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 9, 12, 13, 14, },
  { 2, 3, 6, 7, 9, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 9, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 12, 13, 14, },
  { 4, 6, 7, 9, 12, 13, 14, },
  { 0, 4, 6, 7, 9, 12, 13, 14, },
  { 1, 4, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 9, 12, 13, 14, },
  { 2, 4, 6, 7, 9, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 9, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 12, 13, 14, },
  { 3, 4, 6, 7, 9, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 9, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 12, 13, 14, },
  { 5, 6, 7, 9, 12, 13, 14, },
  { 0, 5, 6, 7, 9, 12, 13, 14, },
  { 1, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 9, 12, 13, 14, },
  { 2, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 9, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 12, 13, 14, },
  { 3, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 9, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 12, 13, 14, },
  { 4, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, },
  { 8, 9, 12, 13, 14, },
  { 0, 8, 9, 12, 13, 14, },
  { 1, 8, 9, 12, 13, 14, },
  { 0, 1, 8, 9, 12, 13, 14, },
  { 2, 8, 9, 12, 13, 14, },
  { 0, 2, 8, 9, 12, 13, 14, },
  { 1, 2, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 8, 9, 12, 13, 14, },
  { 3, 8, 9, 12, 13, 14, },
  { 0, 3, 8, 9, 12, 13, 14, },
  { 1, 3, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 8, 9, 12, 13, 14, },
  { 2, 3, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 8, 9, 12, 13, 14, },
  { 4, 8, 9, 12, 13, 14, },
  { 0, 4, 8, 9, 12, 13, 14, },
  { 1, 4, 8, 9, 12, 13, 14, },
  { 0, 1, 4, 8, 9, 12, 13, 14, },
  { 2, 4, 8, 9, 12, 13, 14, },
  { 0, 2, 4, 8, 9, 12, 13, 14, },
  { 1, 2, 4, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 8, 9, 12, 13, 14, },
  { 3, 4, 8, 9, 12, 13, 14, },
  { 0, 3, 4, 8, 9, 12, 13, 14, },
  { 1, 3, 4, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 8, 9, 12, 13, 14, },
  { 2, 3, 4, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 12, 13, 14, },
  { 5, 8, 9, 12, 13, 14, },
  { 0, 5, 8, 9, 12, 13, 14, },
  { 1, 5, 8, 9, 12, 13, 14, },
  { 0, 1, 5, 8, 9, 12, 13, 14, },
  { 2, 5, 8, 9, 12, 13, 14, },
  { 0, 2, 5, 8, 9, 12, 13, 14, },
  { 1, 2, 5, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 5, 8, 9, 12, 13, 14, },
  { 3, 5, 8, 9, 12, 13, 14, },
  { 0, 3, 5, 8, 9, 12, 13, 14, },
  { 1, 3, 5, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 5, 8, 9, 12, 13, 14, },
  { 2, 3, 5, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 5, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 5, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 12, 13, 14, },
  { 4, 5, 8, 9, 12, 13, 14, },
  { 0, 4, 5, 8, 9, 12, 13, 14, },
  { 1, 4, 5, 8, 9, 12, 13, 14, },
  { 0, 1, 4, 5, 8, 9, 12, 13, 14, },
  { 2, 4, 5, 8, 9, 12, 13, 14, },
  { 0, 2, 4, 5, 8, 9, 12, 13, 14, },
  { 1, 2, 4, 5, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 12, 13, 14, },
  { 3, 4, 5, 8, 9, 12, 13, 14, },
  { 0, 3, 4, 5, 8, 9, 12, 13, 14, },
  { 1, 3, 4, 5, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 12, 13, 14, },
  { 2, 3, 4, 5, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 12, 13, 14, },
  { 6, 8, 9, 12, 13, 14, },
  { 0, 6, 8, 9, 12, 13, 14, },
  { 1, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 6, 8, 9, 12, 13, 14, },
  { 2, 6, 8, 9, 12, 13, 14, },
  { 0, 2, 6, 8, 9, 12, 13, 14, },
  { 1, 2, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 6, 8, 9, 12, 13, 14, },
  { 3, 6, 8, 9, 12, 13, 14, },
  { 0, 3, 6, 8, 9, 12, 13, 14, },
  { 1, 3, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 6, 8, 9, 12, 13, 14, },
  { 2, 3, 6, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 6, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 12, 13, 14, },
  { 4, 6, 8, 9, 12, 13, 14, },
  { 0, 4, 6, 8, 9, 12, 13, 14, },
  { 1, 4, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 4, 6, 8, 9, 12, 13, 14, },
  { 2, 4, 6, 8, 9, 12, 13, 14, },
  { 0, 2, 4, 6, 8, 9, 12, 13, 14, },
  { 1, 2, 4, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 12, 13, 14, },
  { 3, 4, 6, 8, 9, 12, 13, 14, },
  { 0, 3, 4, 6, 8, 9, 12, 13, 14, },
  { 1, 3, 4, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 12, 13, 14, },
  { 2, 3, 4, 6, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 12, 13, 14, },
  { 5, 6, 8, 9, 12, 13, 14, },
  { 0, 5, 6, 8, 9, 12, 13, 14, },
  { 1, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 5, 6, 8, 9, 12, 13, 14, },
  { 2, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 2, 5, 6, 8, 9, 12, 13, 14, },
  { 1, 2, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 12, 13, 14, },
  { 3, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 3, 5, 6, 8, 9, 12, 13, 14, },
  { 1, 3, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 12, 13, 14, },
  { 2, 3, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 12, 13, 14, },
  { 4, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 1, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 2, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 3, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 12, 13, 14, },
  { 7, 8, 9, 12, 13, 14, },
  { 0, 7, 8, 9, 12, 13, 14, },
  { 1, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 7, 8, 9, 12, 13, 14, },
  { 2, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 7, 8, 9, 12, 13, 14, },
  { 3, 7, 8, 9, 12, 13, 14, },
  { 0, 3, 7, 8, 9, 12, 13, 14, },
  { 1, 3, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 7, 8, 9, 12, 13, 14, },
  { 2, 3, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 12, 13, 14, },
  { 4, 7, 8, 9, 12, 13, 14, },
  { 0, 4, 7, 8, 9, 12, 13, 14, },
  { 1, 4, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 4, 7, 8, 9, 12, 13, 14, },
  { 2, 4, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 4, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 4, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 12, 13, 14, },
  { 3, 4, 7, 8, 9, 12, 13, 14, },
  { 0, 3, 4, 7, 8, 9, 12, 13, 14, },
  { 1, 3, 4, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 12, 13, 14, },
  { 2, 3, 4, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 12, 13, 14, },
  { 5, 7, 8, 9, 12, 13, 14, },
  { 0, 5, 7, 8, 9, 12, 13, 14, },
  { 1, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 5, 7, 8, 9, 12, 13, 14, },
  { 2, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 5, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 12, 13, 14, },
  { 3, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 3, 5, 7, 8, 9, 12, 13, 14, },
  { 1, 3, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 12, 13, 14, },
  { 2, 3, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 12, 13, 14, },
  { 4, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 1, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 2, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 3, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 12, 13, 14, },
  { 6, 7, 8, 9, 12, 13, 14, },
  { 0, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 6, 7, 8, 9, 12, 13, 14, },
  { 2, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 12, 13, 14, },
  { 3, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 3, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 3, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 12, 13, 14, },
  { 2, 3, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 12, 13, 14, },
  { 4, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 2, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 3, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, },
  { 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 2, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 3, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, },
  { 10, 12, 13, 14, },
  { 0, 10, 12, 13, 14, },
  { 1, 10, 12, 13, 14, },
  { 0, 1, 10, 12, 13, 14, },
  { 2, 10, 12, 13, 14, },
  { 0, 2, 10, 12, 13, 14, },
  { 1, 2, 10, 12, 13, 14, },
  { 0, 1, 2, 10, 12, 13, 14, },
  { 3, 10, 12, 13, 14, },
  { 0, 3, 10, 12, 13, 14, },
  { 1, 3, 10, 12, 13, 14, },
  { 0, 1, 3, 10, 12, 13, 14, },
  { 2, 3, 10, 12, 13, 14, },
  { 0, 2, 3, 10, 12, 13, 14, },
  { 1, 2, 3, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 10, 12, 13, 14, },
  { 4, 10, 12, 13, 14, },
  { 0, 4, 10, 12, 13, 14, },
  { 1, 4, 10, 12, 13, 14, },
  { 0, 1, 4, 10, 12, 13, 14, },
  { 2, 4, 10, 12, 13, 14, },
  { 0, 2, 4, 10, 12, 13, 14, },
  { 1, 2, 4, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 10, 12, 13, 14, },
  { 3, 4, 10, 12, 13, 14, },
  { 0, 3, 4, 10, 12, 13, 14, },
  { 1, 3, 4, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 10, 12, 13, 14, },
  { 2, 3, 4, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 10, 12, 13, 14, },
  { 5, 10, 12, 13, 14, },
  { 0, 5, 10, 12, 13, 14, },
  { 1, 5, 10, 12, 13, 14, },
  { 0, 1, 5, 10, 12, 13, 14, },
  { 2, 5, 10, 12, 13, 14, },
  { 0, 2, 5, 10, 12, 13, 14, },
  { 1, 2, 5, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 10, 12, 13, 14, },
  { 3, 5, 10, 12, 13, 14, },
  { 0, 3, 5, 10, 12, 13, 14, },
  { 1, 3, 5, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 10, 12, 13, 14, },
  { 2, 3, 5, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 10, 12, 13, 14, },
  { 4, 5, 10, 12, 13, 14, },
  { 0, 4, 5, 10, 12, 13, 14, },
  { 1, 4, 5, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 10, 12, 13, 14, },
  { 2, 4, 5, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 10, 12, 13, 14, },
  { 3, 4, 5, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 10, 12, 13, 14, },
  { 6, 10, 12, 13, 14, },
  { 0, 6, 10, 12, 13, 14, },
  { 1, 6, 10, 12, 13, 14, },
  { 0, 1, 6, 10, 12, 13, 14, },
  { 2, 6, 10, 12, 13, 14, },
  { 0, 2, 6, 10, 12, 13, 14, },
  { 1, 2, 6, 10, 12, 13, 14, },
  { 0, 1, 2, 6, 10, 12, 13, 14, },
  { 3, 6, 10, 12, 13, 14, },
  { 0, 3, 6, 10, 12, 13, 14, },
  { 1, 3, 6, 10, 12, 13, 14, },
  { 0, 1, 3, 6, 10, 12, 13, 14, },
  { 2, 3, 6, 10, 12, 13, 14, },
  { 0, 2, 3, 6, 10, 12, 13, 14, },
  { 1, 2, 3, 6, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 10, 12, 13, 14, },
  { 4, 6, 10, 12, 13, 14, },
  { 0, 4, 6, 10, 12, 13, 14, },
  { 1, 4, 6, 10, 12, 13, 14, },
  { 0, 1, 4, 6, 10, 12, 13, 14, },
  { 2, 4, 6, 10, 12, 13, 14, },
  { 0, 2, 4, 6, 10, 12, 13, 14, },
  { 1, 2, 4, 6, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 10, 12, 13, 14, },
  { 3, 4, 6, 10, 12, 13, 14, },
  { 0, 3, 4, 6, 10, 12, 13, 14, },
  { 1, 3, 4, 6, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 10, 12, 13, 14, },
  { 2, 3, 4, 6, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 10, 12, 13, 14, },
  { 5, 6, 10, 12, 13, 14, },
  { 0, 5, 6, 10, 12, 13, 14, },
  { 1, 5, 6, 10, 12, 13, 14, },
  { 0, 1, 5, 6, 10, 12, 13, 14, },
  { 2, 5, 6, 10, 12, 13, 14, },
  { 0, 2, 5, 6, 10, 12, 13, 14, },
  { 1, 2, 5, 6, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 10, 12, 13, 14, },
  { 3, 5, 6, 10, 12, 13, 14, },
  { 0, 3, 5, 6, 10, 12, 13, 14, },
  { 1, 3, 5, 6, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 10, 12, 13, 14, },
  { 2, 3, 5, 6, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 10, 12, 13, 14, },
  { 4, 5, 6, 10, 12, 13, 14, },
  { 0, 4, 5, 6, 10, 12, 13, 14, },
  { 1, 4, 5, 6, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 10, 12, 13, 14, },
  { 2, 4, 5, 6, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 10, 12, 13, 14, },
  { 3, 4, 5, 6, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 12, 13, 14, },
  { 7, 10, 12, 13, 14, },
  { 0, 7, 10, 12, 13, 14, },
  { 1, 7, 10, 12, 13, 14, },
  { 0, 1, 7, 10, 12, 13, 14, },
  { 2, 7, 10, 12, 13, 14, },
  { 0, 2, 7, 10, 12, 13, 14, },
  { 1, 2, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 7, 10, 12, 13, 14, },
  { 3, 7, 10, 12, 13, 14, },
  { 0, 3, 7, 10, 12, 13, 14, },
  { 1, 3, 7, 10, 12, 13, 14, },
  { 0, 1, 3, 7, 10, 12, 13, 14, },
  { 2, 3, 7, 10, 12, 13, 14, },
  { 0, 2, 3, 7, 10, 12, 13, 14, },
  { 1, 2, 3, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 10, 12, 13, 14, },
  { 4, 7, 10, 12, 13, 14, },
  { 0, 4, 7, 10, 12, 13, 14, },
  { 1, 4, 7, 10, 12, 13, 14, },
  { 0, 1, 4, 7, 10, 12, 13, 14, },
  { 2, 4, 7, 10, 12, 13, 14, },
  { 0, 2, 4, 7, 10, 12, 13, 14, },
  { 1, 2, 4, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 10, 12, 13, 14, },
  { 3, 4, 7, 10, 12, 13, 14, },
  { 0, 3, 4, 7, 10, 12, 13, 14, },
  { 1, 3, 4, 7, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 10, 12, 13, 14, },
  { 2, 3, 4, 7, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 10, 12, 13, 14, },
  { 5, 7, 10, 12, 13, 14, },
  { 0, 5, 7, 10, 12, 13, 14, },
  { 1, 5, 7, 10, 12, 13, 14, },
  { 0, 1, 5, 7, 10, 12, 13, 14, },
  { 2, 5, 7, 10, 12, 13, 14, },
  { 0, 2, 5, 7, 10, 12, 13, 14, },
  { 1, 2, 5, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 10, 12, 13, 14, },
  { 3, 5, 7, 10, 12, 13, 14, },
  { 0, 3, 5, 7, 10, 12, 13, 14, },
  { 1, 3, 5, 7, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 10, 12, 13, 14, },
  { 2, 3, 5, 7, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 10, 12, 13, 14, },
  { 4, 5, 7, 10, 12, 13, 14, },
  { 0, 4, 5, 7, 10, 12, 13, 14, },
  { 1, 4, 5, 7, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 10, 12, 13, 14, },
  { 2, 4, 5, 7, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 10, 12, 13, 14, },
  { 3, 4, 5, 7, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 12, 13, 14, },
  { 6, 7, 10, 12, 13, 14, },
  { 0, 6, 7, 10, 12, 13, 14, },
  { 1, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 6, 7, 10, 12, 13, 14, },
  { 2, 6, 7, 10, 12, 13, 14, },
  { 0, 2, 6, 7, 10, 12, 13, 14, },
  { 1, 2, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 10, 12, 13, 14, },
  { 3, 6, 7, 10, 12, 13, 14, },
  { 0, 3, 6, 7, 10, 12, 13, 14, },
  { 1, 3, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 10, 12, 13, 14, },
  { 2, 3, 6, 7, 10, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 10, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 10, 12, 13, 14, },
  { 4, 6, 7, 10, 12, 13, 14, },
  { 0, 4, 6, 7, 10, 12, 13, 14, },
  { 1, 4, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 10, 12, 13, 14, },
  { 2, 4, 6, 7, 10, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 10, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 10, 12, 13, 14, },
  { 3, 4, 6, 7, 10, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 10, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 10, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 12, 13, 14, },
  { 5, 6, 7, 10, 12, 13, 14, },
  { 0, 5, 6, 7, 10, 12, 13, 14, },
  { 1, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 10, 12, 13, 14, },
  { 2, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 10, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 10, 12, 13, 14, },
  { 3, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 10, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 10, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 12, 13, 14, },
  { 4, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 12, 13, 14, },
  { 8, 10, 12, 13, 14, },
  { 0, 8, 10, 12, 13, 14, },
  { 1, 8, 10, 12, 13, 14, },
  { 0, 1, 8, 10, 12, 13, 14, },
  { 2, 8, 10, 12, 13, 14, },
  { 0, 2, 8, 10, 12, 13, 14, },
  { 1, 2, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 8, 10, 12, 13, 14, },
  { 3, 8, 10, 12, 13, 14, },
  { 0, 3, 8, 10, 12, 13, 14, },
  { 1, 3, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 8, 10, 12, 13, 14, },
  { 2, 3, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 8, 10, 12, 13, 14, },
  { 4, 8, 10, 12, 13, 14, },
  { 0, 4, 8, 10, 12, 13, 14, },
  { 1, 4, 8, 10, 12, 13, 14, },
  { 0, 1, 4, 8, 10, 12, 13, 14, },
  { 2, 4, 8, 10, 12, 13, 14, },
  { 0, 2, 4, 8, 10, 12, 13, 14, },
  { 1, 2, 4, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 8, 10, 12, 13, 14, },
  { 3, 4, 8, 10, 12, 13, 14, },
  { 0, 3, 4, 8, 10, 12, 13, 14, },
  { 1, 3, 4, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 8, 10, 12, 13, 14, },
  { 2, 3, 4, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 10, 12, 13, 14, },
  { 5, 8, 10, 12, 13, 14, },
  { 0, 5, 8, 10, 12, 13, 14, },
  { 1, 5, 8, 10, 12, 13, 14, },
  { 0, 1, 5, 8, 10, 12, 13, 14, },
  { 2, 5, 8, 10, 12, 13, 14, },
  { 0, 2, 5, 8, 10, 12, 13, 14, },
  { 1, 2, 5, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 8, 10, 12, 13, 14, },
  { 3, 5, 8, 10, 12, 13, 14, },
  { 0, 3, 5, 8, 10, 12, 13, 14, },
  { 1, 3, 5, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 8, 10, 12, 13, 14, },
  { 2, 3, 5, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 10, 12, 13, 14, },
  { 4, 5, 8, 10, 12, 13, 14, },
  { 0, 4, 5, 8, 10, 12, 13, 14, },
  { 1, 4, 5, 8, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 8, 10, 12, 13, 14, },
  { 2, 4, 5, 8, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 8, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 10, 12, 13, 14, },
  { 3, 4, 5, 8, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 8, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 12, 13, 14, },
  { 6, 8, 10, 12, 13, 14, },
  { 0, 6, 8, 10, 12, 13, 14, },
  { 1, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 6, 8, 10, 12, 13, 14, },
  { 2, 6, 8, 10, 12, 13, 14, },
  { 0, 2, 6, 8, 10, 12, 13, 14, },
  { 1, 2, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 6, 8, 10, 12, 13, 14, },
  { 3, 6, 8, 10, 12, 13, 14, },
  { 0, 3, 6, 8, 10, 12, 13, 14, },
  { 1, 3, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 6, 8, 10, 12, 13, 14, },
  { 2, 3, 6, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 6, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 10, 12, 13, 14, },
  { 4, 6, 8, 10, 12, 13, 14, },
  { 0, 4, 6, 8, 10, 12, 13, 14, },
  { 1, 4, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 4, 6, 8, 10, 12, 13, 14, },
  { 2, 4, 6, 8, 10, 12, 13, 14, },
  { 0, 2, 4, 6, 8, 10, 12, 13, 14, },
  { 1, 2, 4, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 10, 12, 13, 14, },
  { 3, 4, 6, 8, 10, 12, 13, 14, },
  { 0, 3, 4, 6, 8, 10, 12, 13, 14, },
  { 1, 3, 4, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 10, 12, 13, 14, },
  { 2, 3, 4, 6, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 12, 13, 14, },
  { 5, 6, 8, 10, 12, 13, 14, },
  { 0, 5, 6, 8, 10, 12, 13, 14, },
  { 1, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 5, 6, 8, 10, 12, 13, 14, },
  { 2, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 2, 5, 6, 8, 10, 12, 13, 14, },
  { 1, 2, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 10, 12, 13, 14, },
  { 3, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 3, 5, 6, 8, 10, 12, 13, 14, },
  { 1, 3, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 10, 12, 13, 14, },
  { 2, 3, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 12, 13, 14, },
  { 4, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 1, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 2, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 3, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 12, 13, 14, },
  { 7, 8, 10, 12, 13, 14, },
  { 0, 7, 8, 10, 12, 13, 14, },
  { 1, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 7, 8, 10, 12, 13, 14, },
  { 2, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 7, 8, 10, 12, 13, 14, },
  { 3, 7, 8, 10, 12, 13, 14, },
  { 0, 3, 7, 8, 10, 12, 13, 14, },
  { 1, 3, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 7, 8, 10, 12, 13, 14, },
  { 2, 3, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 10, 12, 13, 14, },
  { 4, 7, 8, 10, 12, 13, 14, },
  { 0, 4, 7, 8, 10, 12, 13, 14, },
  { 1, 4, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 4, 7, 8, 10, 12, 13, 14, },
  { 2, 4, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 4, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 4, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 10, 12, 13, 14, },
  { 3, 4, 7, 8, 10, 12, 13, 14, },
  { 0, 3, 4, 7, 8, 10, 12, 13, 14, },
  { 1, 3, 4, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 10, 12, 13, 14, },
  { 2, 3, 4, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 12, 13, 14, },
  { 5, 7, 8, 10, 12, 13, 14, },
  { 0, 5, 7, 8, 10, 12, 13, 14, },
  { 1, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 5, 7, 8, 10, 12, 13, 14, },
  { 2, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 5, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 10, 12, 13, 14, },
  { 3, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 3, 5, 7, 8, 10, 12, 13, 14, },
  { 1, 3, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 10, 12, 13, 14, },
  { 2, 3, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 12, 13, 14, },
  { 4, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 1, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 2, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 3, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 13, 14, },
  { 6, 7, 8, 10, 12, 13, 14, },
  { 0, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 6, 7, 8, 10, 12, 13, 14, },
  { 2, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 10, 12, 13, 14, },
  { 3, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 3, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 3, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 10, 12, 13, 14, },
  { 2, 3, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 12, 13, 14, },
  { 4, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 2, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 3, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 12, 13, 14, },
  { 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 2, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 3, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, },
  { 9, 10, 12, 13, 14, },
  { 0, 9, 10, 12, 13, 14, },
  { 1, 9, 10, 12, 13, 14, },
  { 0, 1, 9, 10, 12, 13, 14, },
  { 2, 9, 10, 12, 13, 14, },
  { 0, 2, 9, 10, 12, 13, 14, },
  { 1, 2, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 9, 10, 12, 13, 14, },
  { 3, 9, 10, 12, 13, 14, },
  { 0, 3, 9, 10, 12, 13, 14, },
  { 1, 3, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 9, 10, 12, 13, 14, },
  { 2, 3, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 9, 10, 12, 13, 14, },
  { 4, 9, 10, 12, 13, 14, },
  { 0, 4, 9, 10, 12, 13, 14, },
  { 1, 4, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 9, 10, 12, 13, 14, },
  { 2, 4, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 9, 10, 12, 13, 14, },
  { 3, 4, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 9, 10, 12, 13, 14, },
  { 5, 9, 10, 12, 13, 14, },
  { 0, 5, 9, 10, 12, 13, 14, },
  { 1, 5, 9, 10, 12, 13, 14, },
  { 0, 1, 5, 9, 10, 12, 13, 14, },
  { 2, 5, 9, 10, 12, 13, 14, },
  { 0, 2, 5, 9, 10, 12, 13, 14, },
  { 1, 2, 5, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 9, 10, 12, 13, 14, },
  { 3, 5, 9, 10, 12, 13, 14, },
  { 0, 3, 5, 9, 10, 12, 13, 14, },
  { 1, 3, 5, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 9, 10, 12, 13, 14, },
  { 2, 3, 5, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 9, 10, 12, 13, 14, },
  { 4, 5, 9, 10, 12, 13, 14, },
  { 0, 4, 5, 9, 10, 12, 13, 14, },
  { 1, 4, 5, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 9, 10, 12, 13, 14, },
  { 2, 4, 5, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 9, 10, 12, 13, 14, },
  { 3, 4, 5, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 12, 13, 14, },
  { 6, 9, 10, 12, 13, 14, },
  { 0, 6, 9, 10, 12, 13, 14, },
  { 1, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 6, 9, 10, 12, 13, 14, },
  { 2, 6, 9, 10, 12, 13, 14, },
  { 0, 2, 6, 9, 10, 12, 13, 14, },
  { 1, 2, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 6, 9, 10, 12, 13, 14, },
  { 3, 6, 9, 10, 12, 13, 14, },
  { 0, 3, 6, 9, 10, 12, 13, 14, },
  { 1, 3, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 6, 9, 10, 12, 13, 14, },
  { 2, 3, 6, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 6, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 9, 10, 12, 13, 14, },
  { 4, 6, 9, 10, 12, 13, 14, },
  { 0, 4, 6, 9, 10, 12, 13, 14, },
  { 1, 4, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 6, 9, 10, 12, 13, 14, },
  { 2, 4, 6, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 6, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 9, 10, 12, 13, 14, },
  { 3, 4, 6, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 6, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 6, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 12, 13, 14, },
  { 5, 6, 9, 10, 12, 13, 14, },
  { 0, 5, 6, 9, 10, 12, 13, 14, },
  { 1, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 5, 6, 9, 10, 12, 13, 14, },
  { 2, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 2, 5, 6, 9, 10, 12, 13, 14, },
  { 1, 2, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 9, 10, 12, 13, 14, },
  { 3, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 3, 5, 6, 9, 10, 12, 13, 14, },
  { 1, 3, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 9, 10, 12, 13, 14, },
  { 2, 3, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 12, 13, 14, },
  { 4, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 1, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 2, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 3, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 12, 13, 14, },
  { 7, 9, 10, 12, 13, 14, },
  { 0, 7, 9, 10, 12, 13, 14, },
  { 1, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 7, 9, 10, 12, 13, 14, },
  { 2, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 7, 9, 10, 12, 13, 14, },
  { 3, 7, 9, 10, 12, 13, 14, },
  { 0, 3, 7, 9, 10, 12, 13, 14, },
  { 1, 3, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 7, 9, 10, 12, 13, 14, },
  { 2, 3, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 9, 10, 12, 13, 14, },
  { 4, 7, 9, 10, 12, 13, 14, },
  { 0, 4, 7, 9, 10, 12, 13, 14, },
  { 1, 4, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 7, 9, 10, 12, 13, 14, },
  { 2, 4, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 9, 10, 12, 13, 14, },
  { 3, 4, 7, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 7, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 12, 13, 14, },
  { 5, 7, 9, 10, 12, 13, 14, },
  { 0, 5, 7, 9, 10, 12, 13, 14, },
  { 1, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 5, 7, 9, 10, 12, 13, 14, },
  { 2, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 5, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 9, 10, 12, 13, 14, },
  { 3, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 3, 5, 7, 9, 10, 12, 13, 14, },
  { 1, 3, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 9, 10, 12, 13, 14, },
  { 2, 3, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 12, 13, 14, },
  { 4, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 1, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 2, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 3, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 12, 13, 14, },
  { 6, 7, 9, 10, 12, 13, 14, },
  { 0, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 6, 7, 9, 10, 12, 13, 14, },
  { 2, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 9, 10, 12, 13, 14, },
  { 3, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 3, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 3, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 9, 10, 12, 13, 14, },
  { 2, 3, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 12, 13, 14, },
  { 4, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 2, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 3, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 12, 13, 14, },
  { 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 2, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 3, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, },
  { 8, 9, 10, 12, 13, 14, },
  { 0, 8, 9, 10, 12, 13, 14, },
  { 1, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 8, 9, 10, 12, 13, 14, },
  { 2, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 8, 9, 10, 12, 13, 14, },
  { 3, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 8, 9, 10, 12, 13, 14, },
  { 4, 8, 9, 10, 12, 13, 14, },
  { 0, 4, 8, 9, 10, 12, 13, 14, },
  { 1, 4, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 8, 9, 10, 12, 13, 14, },
  { 2, 4, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 8, 9, 10, 12, 13, 14, },
  { 3, 4, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 12, 13, 14, },
  { 5, 8, 9, 10, 12, 13, 14, },
  { 0, 5, 8, 9, 10, 12, 13, 14, },
  { 1, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 5, 8, 9, 10, 12, 13, 14, },
  { 2, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 5, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 8, 9, 10, 12, 13, 14, },
  { 3, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 5, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 12, 13, 14, },
  { 4, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 1, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 2, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 3, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 12, 13, 14, },
  { 6, 8, 9, 10, 12, 13, 14, },
  { 0, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 6, 8, 9, 10, 12, 13, 14, },
  { 2, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 6, 8, 9, 10, 12, 13, 14, },
  { 3, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 6, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 12, 13, 14, },
  { 4, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 2, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 3, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 12, 13, 14, },
  { 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 2, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 3, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, },
  { 7, 8, 9, 10, 12, 13, 14, },
  { 0, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 7, 8, 9, 10, 12, 13, 14, },
  { 3, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 12, 13, 14, },
  { 4, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 3, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 12, 13, 14, },
  { 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 3, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, },
  { 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 3, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, },
  { 11, 12, 13, 14, },
  { 0, 11, 12, 13, 14, },
  { 1, 11, 12, 13, 14, },
  { 0, 1, 11, 12, 13, 14, },
  { 2, 11, 12, 13, 14, },
  { 0, 2, 11, 12, 13, 14, },
  { 1, 2, 11, 12, 13, 14, },
  { 0, 1, 2, 11, 12, 13, 14, },
  { 3, 11, 12, 13, 14, },
  { 0, 3, 11, 12, 13, 14, },
  { 1, 3, 11, 12, 13, 14, },
  { 0, 1, 3, 11, 12, 13, 14, },
  { 2, 3, 11, 12, 13, 14, },
  { 0, 2, 3, 11, 12, 13, 14, },
  { 1, 2, 3, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 11, 12, 13, 14, },
  { 4, 11, 12, 13, 14, },
  { 0, 4, 11, 12, 13, 14, },
  { 1, 4, 11, 12, 13, 14, },
  { 0, 1, 4, 11, 12, 13, 14, },
  { 2, 4, 11, 12, 13, 14, },
  { 0, 2, 4, 11, 12, 13, 14, },
  { 1, 2, 4, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 11, 12, 13, 14, },
  { 3, 4, 11, 12, 13, 14, },
  { 0, 3, 4, 11, 12, 13, 14, },
  { 1, 3, 4, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 11, 12, 13, 14, },
  { 2, 3, 4, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 11, 12, 13, 14, },
  { 5, 11, 12, 13, 14, },
  { 0, 5, 11, 12, 13, 14, },
  { 1, 5, 11, 12, 13, 14, },
  { 0, 1, 5, 11, 12, 13, 14, },
  { 2, 5, 11, 12, 13, 14, },
  { 0, 2, 5, 11, 12, 13, 14, },
  { 1, 2, 5, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 11, 12, 13, 14, },
  { 3, 5, 11, 12, 13, 14, },
  { 0, 3, 5, 11, 12, 13, 14, },
  { 1, 3, 5, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 11, 12, 13, 14, },
  { 2, 3, 5, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 11, 12, 13, 14, },
  { 4, 5, 11, 12, 13, 14, },
  { 0, 4, 5, 11, 12, 13, 14, },
  { 1, 4, 5, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 11, 12, 13, 14, },
  { 2, 4, 5, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 11, 12, 13, 14, },
  { 3, 4, 5, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 11, 12, 13, 14, },
  { 6, 11, 12, 13, 14, },
  { 0, 6, 11, 12, 13, 14, },
  { 1, 6, 11, 12, 13, 14, },
  { 0, 1, 6, 11, 12, 13, 14, },
  { 2, 6, 11, 12, 13, 14, },
  { 0, 2, 6, 11, 12, 13, 14, },
  { 1, 2, 6, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 11, 12, 13, 14, },
  { 3, 6, 11, 12, 13, 14, },
  { 0, 3, 6, 11, 12, 13, 14, },
  { 1, 3, 6, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 11, 12, 13, 14, },
  { 2, 3, 6, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 11, 12, 13, 14, },
  { 4, 6, 11, 12, 13, 14, },
  { 0, 4, 6, 11, 12, 13, 14, },
  { 1, 4, 6, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 11, 12, 13, 14, },
  { 2, 4, 6, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 11, 12, 13, 14, },
  { 3, 4, 6, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 11, 12, 13, 14, },
  { 5, 6, 11, 12, 13, 14, },
  { 0, 5, 6, 11, 12, 13, 14, },
  { 1, 5, 6, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 11, 12, 13, 14, },
  { 2, 5, 6, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 11, 12, 13, 14, },
  { 3, 5, 6, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 11, 12, 13, 14, },
  { 4, 5, 6, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 12, 13, 14, },
  { 7, 11, 12, 13, 14, },
  { 0, 7, 11, 12, 13, 14, },
  { 1, 7, 11, 12, 13, 14, },
  { 0, 1, 7, 11, 12, 13, 14, },
  { 2, 7, 11, 12, 13, 14, },
  { 0, 2, 7, 11, 12, 13, 14, },
  { 1, 2, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 7, 11, 12, 13, 14, },
  { 3, 7, 11, 12, 13, 14, },
  { 0, 3, 7, 11, 12, 13, 14, },
  { 1, 3, 7, 11, 12, 13, 14, },
  { 0, 1, 3, 7, 11, 12, 13, 14, },
  { 2, 3, 7, 11, 12, 13, 14, },
  { 0, 2, 3, 7, 11, 12, 13, 14, },
  { 1, 2, 3, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 11, 12, 13, 14, },
  { 4, 7, 11, 12, 13, 14, },
  { 0, 4, 7, 11, 12, 13, 14, },
  { 1, 4, 7, 11, 12, 13, 14, },
  { 0, 1, 4, 7, 11, 12, 13, 14, },
  { 2, 4, 7, 11, 12, 13, 14, },
  { 0, 2, 4, 7, 11, 12, 13, 14, },
  { 1, 2, 4, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 11, 12, 13, 14, },
  { 3, 4, 7, 11, 12, 13, 14, },
  { 0, 3, 4, 7, 11, 12, 13, 14, },
  { 1, 3, 4, 7, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 11, 12, 13, 14, },
  { 2, 3, 4, 7, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 11, 12, 13, 14, },
  { 5, 7, 11, 12, 13, 14, },
  { 0, 5, 7, 11, 12, 13, 14, },
  { 1, 5, 7, 11, 12, 13, 14, },
  { 0, 1, 5, 7, 11, 12, 13, 14, },
  { 2, 5, 7, 11, 12, 13, 14, },
  { 0, 2, 5, 7, 11, 12, 13, 14, },
  { 1, 2, 5, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 11, 12, 13, 14, },
  { 3, 5, 7, 11, 12, 13, 14, },
  { 0, 3, 5, 7, 11, 12, 13, 14, },
  { 1, 3, 5, 7, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 11, 12, 13, 14, },
  { 2, 3, 5, 7, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 11, 12, 13, 14, },
  { 4, 5, 7, 11, 12, 13, 14, },
  { 0, 4, 5, 7, 11, 12, 13, 14, },
  { 1, 4, 5, 7, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 11, 12, 13, 14, },
  { 2, 4, 5, 7, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 11, 12, 13, 14, },
  { 3, 4, 5, 7, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 12, 13, 14, },
  { 6, 7, 11, 12, 13, 14, },
  { 0, 6, 7, 11, 12, 13, 14, },
  { 1, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 6, 7, 11, 12, 13, 14, },
  { 2, 6, 7, 11, 12, 13, 14, },
  { 0, 2, 6, 7, 11, 12, 13, 14, },
  { 1, 2, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 11, 12, 13, 14, },
  { 3, 6, 7, 11, 12, 13, 14, },
  { 0, 3, 6, 7, 11, 12, 13, 14, },
  { 1, 3, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 11, 12, 13, 14, },
  { 2, 3, 6, 7, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 11, 12, 13, 14, },
  { 4, 6, 7, 11, 12, 13, 14, },
  { 0, 4, 6, 7, 11, 12, 13, 14, },
  { 1, 4, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 11, 12, 13, 14, },
  { 2, 4, 6, 7, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 11, 12, 13, 14, },
  { 3, 4, 6, 7, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 12, 13, 14, },
  { 5, 6, 7, 11, 12, 13, 14, },
  { 0, 5, 6, 7, 11, 12, 13, 14, },
  { 1, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 11, 12, 13, 14, },
  { 2, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 11, 12, 13, 14, },
  { 3, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 12, 13, 14, },
  { 4, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 12, 13, 14, },
  { 8, 11, 12, 13, 14, },
  { 0, 8, 11, 12, 13, 14, },
  { 1, 8, 11, 12, 13, 14, },
  { 0, 1, 8, 11, 12, 13, 14, },
  { 2, 8, 11, 12, 13, 14, },
  { 0, 2, 8, 11, 12, 13, 14, },
  { 1, 2, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 8, 11, 12, 13, 14, },
  { 3, 8, 11, 12, 13, 14, },
  { 0, 3, 8, 11, 12, 13, 14, },
  { 1, 3, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 8, 11, 12, 13, 14, },
  { 2, 3, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 8, 11, 12, 13, 14, },
  { 4, 8, 11, 12, 13, 14, },
  { 0, 4, 8, 11, 12, 13, 14, },
  { 1, 4, 8, 11, 12, 13, 14, },
  { 0, 1, 4, 8, 11, 12, 13, 14, },
  { 2, 4, 8, 11, 12, 13, 14, },
  { 0, 2, 4, 8, 11, 12, 13, 14, },
  { 1, 2, 4, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 8, 11, 12, 13, 14, },
  { 3, 4, 8, 11, 12, 13, 14, },
  { 0, 3, 4, 8, 11, 12, 13, 14, },
  { 1, 3, 4, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 8, 11, 12, 13, 14, },
  { 2, 3, 4, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 11, 12, 13, 14, },
  { 5, 8, 11, 12, 13, 14, },
  { 0, 5, 8, 11, 12, 13, 14, },
  { 1, 5, 8, 11, 12, 13, 14, },
  { 0, 1, 5, 8, 11, 12, 13, 14, },
  { 2, 5, 8, 11, 12, 13, 14, },
  { 0, 2, 5, 8, 11, 12, 13, 14, },
  { 1, 2, 5, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 8, 11, 12, 13, 14, },
  { 3, 5, 8, 11, 12, 13, 14, },
  { 0, 3, 5, 8, 11, 12, 13, 14, },
  { 1, 3, 5, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 8, 11, 12, 13, 14, },
  { 2, 3, 5, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 11, 12, 13, 14, },
  { 4, 5, 8, 11, 12, 13, 14, },
  { 0, 4, 5, 8, 11, 12, 13, 14, },
  { 1, 4, 5, 8, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 8, 11, 12, 13, 14, },
  { 2, 4, 5, 8, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 8, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 11, 12, 13, 14, },
  { 3, 4, 5, 8, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 8, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 12, 13, 14, },
  { 6, 8, 11, 12, 13, 14, },
  { 0, 6, 8, 11, 12, 13, 14, },
  { 1, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 6, 8, 11, 12, 13, 14, },
  { 2, 6, 8, 11, 12, 13, 14, },
  { 0, 2, 6, 8, 11, 12, 13, 14, },
  { 1, 2, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 8, 11, 12, 13, 14, },
  { 3, 6, 8, 11, 12, 13, 14, },
  { 0, 3, 6, 8, 11, 12, 13, 14, },
  { 1, 3, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 8, 11, 12, 13, 14, },
  { 2, 3, 6, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 11, 12, 13, 14, },
  { 4, 6, 8, 11, 12, 13, 14, },
  { 0, 4, 6, 8, 11, 12, 13, 14, },
  { 1, 4, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 8, 11, 12, 13, 14, },
  { 2, 4, 6, 8, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 8, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 11, 12, 13, 14, },
  { 3, 4, 6, 8, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 8, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 12, 13, 14, },
  { 5, 6, 8, 11, 12, 13, 14, },
  { 0, 5, 6, 8, 11, 12, 13, 14, },
  { 1, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 8, 11, 12, 13, 14, },
  { 2, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 8, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 11, 12, 13, 14, },
  { 3, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 8, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 12, 13, 14, },
  { 4, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 12, 13, 14, },
  { 7, 8, 11, 12, 13, 14, },
  { 0, 7, 8, 11, 12, 13, 14, },
  { 1, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 7, 8, 11, 12, 13, 14, },
  { 2, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 7, 8, 11, 12, 13, 14, },
  { 3, 7, 8, 11, 12, 13, 14, },
  { 0, 3, 7, 8, 11, 12, 13, 14, },
  { 1, 3, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 7, 8, 11, 12, 13, 14, },
  { 2, 3, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 11, 12, 13, 14, },
  { 4, 7, 8, 11, 12, 13, 14, },
  { 0, 4, 7, 8, 11, 12, 13, 14, },
  { 1, 4, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 4, 7, 8, 11, 12, 13, 14, },
  { 2, 4, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 4, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 4, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 11, 12, 13, 14, },
  { 3, 4, 7, 8, 11, 12, 13, 14, },
  { 0, 3, 4, 7, 8, 11, 12, 13, 14, },
  { 1, 3, 4, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 11, 12, 13, 14, },
  { 2, 3, 4, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 12, 13, 14, },
  { 5, 7, 8, 11, 12, 13, 14, },
  { 0, 5, 7, 8, 11, 12, 13, 14, },
  { 1, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 5, 7, 8, 11, 12, 13, 14, },
  { 2, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 5, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 11, 12, 13, 14, },
  { 3, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 3, 5, 7, 8, 11, 12, 13, 14, },
  { 1, 3, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 11, 12, 13, 14, },
  { 2, 3, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 12, 13, 14, },
  { 4, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 1, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 2, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 3, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 12, 13, 14, },
  { 6, 7, 8, 11, 12, 13, 14, },
  { 0, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 6, 7, 8, 11, 12, 13, 14, },
  { 2, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 11, 12, 13, 14, },
  { 3, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 3, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 3, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 11, 12, 13, 14, },
  { 2, 3, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 12, 13, 14, },
  { 4, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 2, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 3, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 12, 13, 14, },
  { 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 2, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 3, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, },
  { 9, 11, 12, 13, 14, },
  { 0, 9, 11, 12, 13, 14, },
  { 1, 9, 11, 12, 13, 14, },
  { 0, 1, 9, 11, 12, 13, 14, },
  { 2, 9, 11, 12, 13, 14, },
  { 0, 2, 9, 11, 12, 13, 14, },
  { 1, 2, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 9, 11, 12, 13, 14, },
  { 3, 9, 11, 12, 13, 14, },
  { 0, 3, 9, 11, 12, 13, 14, },
  { 1, 3, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 9, 11, 12, 13, 14, },
  { 2, 3, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 9, 11, 12, 13, 14, },
  { 4, 9, 11, 12, 13, 14, },
  { 0, 4, 9, 11, 12, 13, 14, },
  { 1, 4, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 9, 11, 12, 13, 14, },
  { 2, 4, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 9, 11, 12, 13, 14, },
  { 3, 4, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 9, 11, 12, 13, 14, },
  { 5, 9, 11, 12, 13, 14, },
  { 0, 5, 9, 11, 12, 13, 14, },
  { 1, 5, 9, 11, 12, 13, 14, },
  { 0, 1, 5, 9, 11, 12, 13, 14, },
  { 2, 5, 9, 11, 12, 13, 14, },
  { 0, 2, 5, 9, 11, 12, 13, 14, },
  { 1, 2, 5, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 9, 11, 12, 13, 14, },
  { 3, 5, 9, 11, 12, 13, 14, },
  { 0, 3, 5, 9, 11, 12, 13, 14, },
  { 1, 3, 5, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 9, 11, 12, 13, 14, },
  { 2, 3, 5, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 9, 11, 12, 13, 14, },
  { 4, 5, 9, 11, 12, 13, 14, },
  { 0, 4, 5, 9, 11, 12, 13, 14, },
  { 1, 4, 5, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 9, 11, 12, 13, 14, },
  { 2, 4, 5, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 9, 11, 12, 13, 14, },
  { 3, 4, 5, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 12, 13, 14, },
  { 6, 9, 11, 12, 13, 14, },
  { 0, 6, 9, 11, 12, 13, 14, },
  { 1, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 6, 9, 11, 12, 13, 14, },
  { 2, 6, 9, 11, 12, 13, 14, },
  { 0, 2, 6, 9, 11, 12, 13, 14, },
  { 1, 2, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 9, 11, 12, 13, 14, },
  { 3, 6, 9, 11, 12, 13, 14, },
  { 0, 3, 6, 9, 11, 12, 13, 14, },
  { 1, 3, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 9, 11, 12, 13, 14, },
  { 2, 3, 6, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 9, 11, 12, 13, 14, },
  { 4, 6, 9, 11, 12, 13, 14, },
  { 0, 4, 6, 9, 11, 12, 13, 14, },
  { 1, 4, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 9, 11, 12, 13, 14, },
  { 2, 4, 6, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 9, 11, 12, 13, 14, },
  { 3, 4, 6, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 12, 13, 14, },
  { 5, 6, 9, 11, 12, 13, 14, },
  { 0, 5, 6, 9, 11, 12, 13, 14, },
  { 1, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 9, 11, 12, 13, 14, },
  { 2, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 9, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 9, 11, 12, 13, 14, },
  { 3, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 9, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 9, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 12, 13, 14, },
  { 4, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 12, 13, 14, },
  { 7, 9, 11, 12, 13, 14, },
  { 0, 7, 9, 11, 12, 13, 14, },
  { 1, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 7, 9, 11, 12, 13, 14, },
  { 2, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 7, 9, 11, 12, 13, 14, },
  { 3, 7, 9, 11, 12, 13, 14, },
  { 0, 3, 7, 9, 11, 12, 13, 14, },
  { 1, 3, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 7, 9, 11, 12, 13, 14, },
  { 2, 3, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 9, 11, 12, 13, 14, },
  { 4, 7, 9, 11, 12, 13, 14, },
  { 0, 4, 7, 9, 11, 12, 13, 14, },
  { 1, 4, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 7, 9, 11, 12, 13, 14, },
  { 2, 4, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 9, 11, 12, 13, 14, },
  { 3, 4, 7, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 7, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 12, 13, 14, },
  { 5, 7, 9, 11, 12, 13, 14, },
  { 0, 5, 7, 9, 11, 12, 13, 14, },
  { 1, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 5, 7, 9, 11, 12, 13, 14, },
  { 2, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 5, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 9, 11, 12, 13, 14, },
  { 3, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 3, 5, 7, 9, 11, 12, 13, 14, },
  { 1, 3, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 9, 11, 12, 13, 14, },
  { 2, 3, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 12, 13, 14, },
  { 4, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 1, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 2, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 3, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 12, 13, 14, },
  { 6, 7, 9, 11, 12, 13, 14, },
  { 0, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 6, 7, 9, 11, 12, 13, 14, },
  { 2, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 9, 11, 12, 13, 14, },
  { 3, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 3, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 3, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 9, 11, 12, 13, 14, },
  { 2, 3, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 12, 13, 14, },
  { 4, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 2, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 3, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 12, 13, 14, },
  { 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 2, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 3, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, },
  { 8, 9, 11, 12, 13, 14, },
  { 0, 8, 9, 11, 12, 13, 14, },
  { 1, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 8, 9, 11, 12, 13, 14, },
  { 2, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 8, 9, 11, 12, 13, 14, },
  { 3, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 8, 9, 11, 12, 13, 14, },
  { 4, 8, 9, 11, 12, 13, 14, },
  { 0, 4, 8, 9, 11, 12, 13, 14, },
  { 1, 4, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 8, 9, 11, 12, 13, 14, },
  { 2, 4, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 8, 9, 11, 12, 13, 14, },
  { 3, 4, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 12, 13, 14, },
  { 5, 8, 9, 11, 12, 13, 14, },
  { 0, 5, 8, 9, 11, 12, 13, 14, },
  { 1, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 5, 8, 9, 11, 12, 13, 14, },
  { 2, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 5, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 8, 9, 11, 12, 13, 14, },
  { 3, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 5, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 12, 13, 14, },
  { 4, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 1, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 2, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 3, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 12, 13, 14, },
  { 6, 8, 9, 11, 12, 13, 14, },
  { 0, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 6, 8, 9, 11, 12, 13, 14, },
  { 2, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 8, 9, 11, 12, 13, 14, },
  { 3, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 12, 13, 14, },
  { 4, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 2, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 3, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 12, 13, 14, },
  { 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 2, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 3, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, },
  { 7, 8, 9, 11, 12, 13, 14, },
  { 0, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 7, 8, 9, 11, 12, 13, 14, },
  { 3, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 12, 13, 14, },
  { 4, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 3, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 12, 13, 14, },
  { 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 3, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, },
  { 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 3, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, },
  { 10, 11, 12, 13, 14, },
  { 0, 10, 11, 12, 13, 14, },
  { 1, 10, 11, 12, 13, 14, },
  { 0, 1, 10, 11, 12, 13, 14, },
  { 2, 10, 11, 12, 13, 14, },
  { 0, 2, 10, 11, 12, 13, 14, },
  { 1, 2, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 10, 11, 12, 13, 14, },
  { 3, 10, 11, 12, 13, 14, },
  { 0, 3, 10, 11, 12, 13, 14, },
  { 1, 3, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 10, 11, 12, 13, 14, },
  { 2, 3, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 10, 11, 12, 13, 14, },
  { 4, 10, 11, 12, 13, 14, },
  { 0, 4, 10, 11, 12, 13, 14, },
  { 1, 4, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 10, 11, 12, 13, 14, },
  { 2, 4, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 10, 11, 12, 13, 14, },
  { 3, 4, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 10, 11, 12, 13, 14, },
  { 5, 10, 11, 12, 13, 14, },
  { 0, 5, 10, 11, 12, 13, 14, },
  { 1, 5, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 10, 11, 12, 13, 14, },
  { 2, 5, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 10, 11, 12, 13, 14, },
  { 3, 5, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 10, 11, 12, 13, 14, },
  { 4, 5, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 12, 13, 14, },
  { 6, 10, 11, 12, 13, 14, },
  { 0, 6, 10, 11, 12, 13, 14, },
  { 1, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 6, 10, 11, 12, 13, 14, },
  { 2, 6, 10, 11, 12, 13, 14, },
  { 0, 2, 6, 10, 11, 12, 13, 14, },
  { 1, 2, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 10, 11, 12, 13, 14, },
  { 3, 6, 10, 11, 12, 13, 14, },
  { 0, 3, 6, 10, 11, 12, 13, 14, },
  { 1, 3, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 10, 11, 12, 13, 14, },
  { 2, 3, 6, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 10, 11, 12, 13, 14, },
  { 4, 6, 10, 11, 12, 13, 14, },
  { 0, 4, 6, 10, 11, 12, 13, 14, },
  { 1, 4, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 10, 11, 12, 13, 14, },
  { 2, 4, 6, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 10, 11, 12, 13, 14, },
  { 3, 4, 6, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 12, 13, 14, },
  { 5, 6, 10, 11, 12, 13, 14, },
  { 0, 5, 6, 10, 11, 12, 13, 14, },
  { 1, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 10, 11, 12, 13, 14, },
  { 2, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 10, 11, 12, 13, 14, },
  { 3, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 12, 13, 14, },
  { 4, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, },
  { 7, 10, 11, 12, 13, 14, },
  { 0, 7, 10, 11, 12, 13, 14, },
  { 1, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 7, 10, 11, 12, 13, 14, },
  { 2, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 7, 10, 11, 12, 13, 14, },
  { 3, 7, 10, 11, 12, 13, 14, },
  { 0, 3, 7, 10, 11, 12, 13, 14, },
  { 1, 3, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 7, 10, 11, 12, 13, 14, },
  { 2, 3, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 10, 11, 12, 13, 14, },
  { 4, 7, 10, 11, 12, 13, 14, },
  { 0, 4, 7, 10, 11, 12, 13, 14, },
  { 1, 4, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 7, 10, 11, 12, 13, 14, },
  { 2, 4, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 10, 11, 12, 13, 14, },
  { 3, 4, 7, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 7, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 12, 13, 14, },
  { 5, 7, 10, 11, 12, 13, 14, },
  { 0, 5, 7, 10, 11, 12, 13, 14, },
  { 1, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 7, 10, 11, 12, 13, 14, },
  { 2, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 10, 11, 12, 13, 14, },
  { 3, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 7, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 12, 13, 14, },
  { 4, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 12, 13, 14, },
  { 6, 7, 10, 11, 12, 13, 14, },
  { 0, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 6, 7, 10, 11, 12, 13, 14, },
  { 2, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 10, 11, 12, 13, 14, },
  { 3, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 3, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 3, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 10, 11, 12, 13, 14, },
  { 2, 3, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 12, 13, 14, },
  { 4, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 2, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 3, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 12, 13, 14, },
  { 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 2, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 3, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, },
  { 8, 10, 11, 12, 13, 14, },
  { 0, 8, 10, 11, 12, 13, 14, },
  { 1, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 8, 10, 11, 12, 13, 14, },
  { 2, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 8, 10, 11, 12, 13, 14, },
  { 3, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 8, 10, 11, 12, 13, 14, },
  { 4, 8, 10, 11, 12, 13, 14, },
  { 0, 4, 8, 10, 11, 12, 13, 14, },
  { 1, 4, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 8, 10, 11, 12, 13, 14, },
  { 2, 4, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 8, 10, 11, 12, 13, 14, },
  { 3, 4, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 12, 13, 14, },
  { 5, 8, 10, 11, 12, 13, 14, },
  { 0, 5, 8, 10, 11, 12, 13, 14, },
  { 1, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 8, 10, 11, 12, 13, 14, },
  { 2, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 8, 10, 11, 12, 13, 14, },
  { 3, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 12, 13, 14, },
  { 4, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 12, 13, 14, },
  { 6, 8, 10, 11, 12, 13, 14, },
  { 0, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 6, 8, 10, 11, 12, 13, 14, },
  { 2, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 8, 10, 11, 12, 13, 14, },
  { 3, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 12, 13, 14, },
  { 4, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 2, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 3, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 12, 13, 14, },
  { 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 2, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 3, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, },
  { 7, 8, 10, 11, 12, 13, 14, },
  { 0, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 7, 8, 10, 11, 12, 13, 14, },
  { 3, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 12, 13, 14, },
  { 4, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 3, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 12, 13, 14, },
  { 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 3, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, },
  { 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 3, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, },
  { 9, 10, 11, 12, 13, 14, },
  { 0, 9, 10, 11, 12, 13, 14, },
  { 1, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 9, 10, 11, 12, 13, 14, },
  { 2, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 9, 10, 11, 12, 13, 14, },
  { 3, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 9, 10, 11, 12, 13, 14, },
  { 4, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 12, 13, 14, },
  { 5, 9, 10, 11, 12, 13, 14, },
  { 0, 5, 9, 10, 11, 12, 13, 14, },
  { 1, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 9, 10, 11, 12, 13, 14, },
  { 2, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 9, 10, 11, 12, 13, 14, },
  { 3, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 12, 13, 14, },
  { 4, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, 14, },
  { 6, 9, 10, 11, 12, 13, 14, },
  { 0, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 6, 9, 10, 11, 12, 13, 14, },
  { 2, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 9, 10, 11, 12, 13, 14, },
  { 3, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 12, 13, 14, },
  { 4, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 12, 13, 14, },
  { 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 2, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 3, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, },
  { 7, 9, 10, 11, 12, 13, 14, },
  { 0, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 7, 9, 10, 11, 12, 13, 14, },
  { 3, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 12, 13, 14, },
  { 4, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 12, 13, 14, },
  { 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 3, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, },
  { 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 3, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, },
  { 8, 9, 10, 11, 12, 13, 14, },
  { 0, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 12, 13, 14, },
  { 4, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 12, 13, 14, },
  { 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, },
  { 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, },
  { 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, },
  { 15, },
  { 0, 15, },
  { 1, 15, },
  { 0, 1, 15, },
  { 2, 15, },
  { 0, 2, 15, },
  { 1, 2, 15, },
  { 0, 1, 2, 15, },
  { 3, 15, },
  { 0, 3, 15, },
  { 1, 3, 15, },
  { 0, 1, 3, 15, },
  { 2, 3, 15, },
  { 0, 2, 3, 15, },
  { 1, 2, 3, 15, },
  { 0, 1, 2, 3, 15, },
  { 4, 15, },
  { 0, 4, 15, },
  { 1, 4, 15, },
  { 0, 1, 4, 15, },
  { 2, 4, 15, },
  { 0, 2, 4, 15, },
  { 1, 2, 4, 15, },
  { 0, 1, 2, 4, 15, },
  { 3, 4, 15, },
  { 0, 3, 4, 15, },
  { 1, 3, 4, 15, },
  { 0, 1, 3, 4, 15, },
  { 2, 3, 4, 15, },
  { 0, 2, 3, 4, 15, },
  { 1, 2, 3, 4, 15, },
  { 0, 1, 2, 3, 4, 15, },
  { 5, 15, },
  { 0, 5, 15, },
  { 1, 5, 15, },
  { 0, 1, 5, 15, },
  { 2, 5, 15, },
  { 0, 2, 5, 15, },
  { 1, 2, 5, 15, },
  { 0, 1, 2, 5, 15, },
  { 3, 5, 15, },
  { 0, 3, 5, 15, },
  { 1, 3, 5, 15, },
  { 0, 1, 3, 5, 15, },
  { 2, 3, 5, 15, },
  { 0, 2, 3, 5, 15, },
  { 1, 2, 3, 5, 15, },
  { 0, 1, 2, 3, 5, 15, },
  { 4, 5, 15, },
  { 0, 4, 5, 15, },
  { 1, 4, 5, 15, },
  { 0, 1, 4, 5, 15, },
  { 2, 4, 5, 15, },
  { 0, 2, 4, 5, 15, },
  { 1, 2, 4, 5, 15, },
  { 0, 1, 2, 4, 5, 15, },
  { 3, 4, 5, 15, },
  { 0, 3, 4, 5, 15, },
  { 1, 3, 4, 5, 15, },
  { 0, 1, 3, 4, 5, 15, },
  { 2, 3, 4, 5, 15, },
  { 0, 2, 3, 4, 5, 15, },
  { 1, 2, 3, 4, 5, 15, },
  { 0, 1, 2, 3, 4, 5, 15, },
  { 6, 15, },
  { 0, 6, 15, },
  { 1, 6, 15, },
  { 0, 1, 6, 15, },
  { 2, 6, 15, },
  { 0, 2, 6, 15, },
  { 1, 2, 6, 15, },
  { 0, 1, 2, 6, 15, },
  { 3, 6, 15, },
  { 0, 3, 6, 15, },
  { 1, 3, 6, 15, },
  { 0, 1, 3, 6, 15, },
  { 2, 3, 6, 15, },
  { 0, 2, 3, 6, 15, },
  { 1, 2, 3, 6, 15, },
  { 0, 1, 2, 3, 6, 15, },
  { 4, 6, 15, },
  { 0, 4, 6, 15, },
  { 1, 4, 6, 15, },
  { 0, 1, 4, 6, 15, },
  { 2, 4, 6, 15, },
  { 0, 2, 4, 6, 15, },
  { 1, 2, 4, 6, 15, },
  { 0, 1, 2, 4, 6, 15, },
  { 3, 4, 6, 15, },
  { 0, 3, 4, 6, 15, },
  { 1, 3, 4, 6, 15, },
  { 0, 1, 3, 4, 6, 15, },
  { 2, 3, 4, 6, 15, },
  { 0, 2, 3, 4, 6, 15, },
  { 1, 2, 3, 4, 6, 15, },
  { 0, 1, 2, 3, 4, 6, 15, },
  { 5, 6, 15, },
  { 0, 5, 6, 15, },
  { 1, 5, 6, 15, },
  { 0, 1, 5, 6, 15, },
  { 2, 5, 6, 15, },
  { 0, 2, 5, 6, 15, },
  { 1, 2, 5, 6, 15, },
  { 0, 1, 2, 5, 6, 15, },
  { 3, 5, 6, 15, },
  { 0, 3, 5, 6, 15, },
  { 1, 3, 5, 6, 15, },
  { 0, 1, 3, 5, 6, 15, },
  { 2, 3, 5, 6, 15, },
  { 0, 2, 3, 5, 6, 15, },
  { 1, 2, 3, 5, 6, 15, },
  { 0, 1, 2, 3, 5, 6, 15, },
  { 4, 5, 6, 15, },
  { 0, 4, 5, 6, 15, },
  { 1, 4, 5, 6, 15, },
  { 0, 1, 4, 5, 6, 15, },
  { 2, 4, 5, 6, 15, },
  { 0, 2, 4, 5, 6, 15, },
  { 1, 2, 4, 5, 6, 15, },
  { 0, 1, 2, 4, 5, 6, 15, },
  { 3, 4, 5, 6, 15, },
  { 0, 3, 4, 5, 6, 15, },
  { 1, 3, 4, 5, 6, 15, },
  { 0, 1, 3, 4, 5, 6, 15, },
  { 2, 3, 4, 5, 6, 15, },
  { 0, 2, 3, 4, 5, 6, 15, },
  { 1, 2, 3, 4, 5, 6, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 15, },
  { 7, 15, },
  { 0, 7, 15, },
  { 1, 7, 15, },
  { 0, 1, 7, 15, },
  { 2, 7, 15, },
  { 0, 2, 7, 15, },
  { 1, 2, 7, 15, },
  { 0, 1, 2, 7, 15, },
  { 3, 7, 15, },
  { 0, 3, 7, 15, },
  { 1, 3, 7, 15, },
  { 0, 1, 3, 7, 15, },
  { 2, 3, 7, 15, },
  { 0, 2, 3, 7, 15, },
  { 1, 2, 3, 7, 15, },
  { 0, 1, 2, 3, 7, 15, },
  { 4, 7, 15, },
  { 0, 4, 7, 15, },
  { 1, 4, 7, 15, },
  { 0, 1, 4, 7, 15, },
  { 2, 4, 7, 15, },
  { 0, 2, 4, 7, 15, },
  { 1, 2, 4, 7, 15, },
  { 0, 1, 2, 4, 7, 15, },
  { 3, 4, 7, 15, },
  { 0, 3, 4, 7, 15, },
  { 1, 3, 4, 7, 15, },
  { 0, 1, 3, 4, 7, 15, },
  { 2, 3, 4, 7, 15, },
  { 0, 2, 3, 4, 7, 15, },
  { 1, 2, 3, 4, 7, 15, },
  { 0, 1, 2, 3, 4, 7, 15, },
  { 5, 7, 15, },
  { 0, 5, 7, 15, },
  { 1, 5, 7, 15, },
  { 0, 1, 5, 7, 15, },
  { 2, 5, 7, 15, },
  { 0, 2, 5, 7, 15, },
  { 1, 2, 5, 7, 15, },
  { 0, 1, 2, 5, 7, 15, },
  { 3, 5, 7, 15, },
  { 0, 3, 5, 7, 15, },
  { 1, 3, 5, 7, 15, },
  { 0, 1, 3, 5, 7, 15, },
  { 2, 3, 5, 7, 15, },
  { 0, 2, 3, 5, 7, 15, },
  { 1, 2, 3, 5, 7, 15, },
  { 0, 1, 2, 3, 5, 7, 15, },
  { 4, 5, 7, 15, },
  { 0, 4, 5, 7, 15, },
  { 1, 4, 5, 7, 15, },
  { 0, 1, 4, 5, 7, 15, },
  { 2, 4, 5, 7, 15, },
  { 0, 2, 4, 5, 7, 15, },
  { 1, 2, 4, 5, 7, 15, },
  { 0, 1, 2, 4, 5, 7, 15, },
  { 3, 4, 5, 7, 15, },
  { 0, 3, 4, 5, 7, 15, },
  { 1, 3, 4, 5, 7, 15, },
  { 0, 1, 3, 4, 5, 7, 15, },
  { 2, 3, 4, 5, 7, 15, },
  { 0, 2, 3, 4, 5, 7, 15, },
  { 1, 2, 3, 4, 5, 7, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 15, },
  { 6, 7, 15, },
  { 0, 6, 7, 15, },
  { 1, 6, 7, 15, },
  { 0, 1, 6, 7, 15, },
  { 2, 6, 7, 15, },
  { 0, 2, 6, 7, 15, },
  { 1, 2, 6, 7, 15, },
  { 0, 1, 2, 6, 7, 15, },
  { 3, 6, 7, 15, },
  { 0, 3, 6, 7, 15, },
  { 1, 3, 6, 7, 15, },
  { 0, 1, 3, 6, 7, 15, },
  { 2, 3, 6, 7, 15, },
  { 0, 2, 3, 6, 7, 15, },
  { 1, 2, 3, 6, 7, 15, },
  { 0, 1, 2, 3, 6, 7, 15, },
  { 4, 6, 7, 15, },
  { 0, 4, 6, 7, 15, },
  { 1, 4, 6, 7, 15, },
  { 0, 1, 4, 6, 7, 15, },
  { 2, 4, 6, 7, 15, },
  { 0, 2, 4, 6, 7, 15, },
  { 1, 2, 4, 6, 7, 15, },
  { 0, 1, 2, 4, 6, 7, 15, },
  { 3, 4, 6, 7, 15, },
  { 0, 3, 4, 6, 7, 15, },
  { 1, 3, 4, 6, 7, 15, },
  { 0, 1, 3, 4, 6, 7, 15, },
  { 2, 3, 4, 6, 7, 15, },
  { 0, 2, 3, 4, 6, 7, 15, },
  { 1, 2, 3, 4, 6, 7, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 15, },
  { 5, 6, 7, 15, },
  { 0, 5, 6, 7, 15, },
  { 1, 5, 6, 7, 15, },
  { 0, 1, 5, 6, 7, 15, },
  { 2, 5, 6, 7, 15, },
  { 0, 2, 5, 6, 7, 15, },
  { 1, 2, 5, 6, 7, 15, },
  { 0, 1, 2, 5, 6, 7, 15, },
  { 3, 5, 6, 7, 15, },
  { 0, 3, 5, 6, 7, 15, },
  { 1, 3, 5, 6, 7, 15, },
  { 0, 1, 3, 5, 6, 7, 15, },
  { 2, 3, 5, 6, 7, 15, },
  { 0, 2, 3, 5, 6, 7, 15, },
  { 1, 2, 3, 5, 6, 7, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 15, },
  { 4, 5, 6, 7, 15, },
  { 0, 4, 5, 6, 7, 15, },
  { 1, 4, 5, 6, 7, 15, },
  { 0, 1, 4, 5, 6, 7, 15, },
  { 2, 4, 5, 6, 7, 15, },
  { 0, 2, 4, 5, 6, 7, 15, },
  { 1, 2, 4, 5, 6, 7, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 15, },
  { 3, 4, 5, 6, 7, 15, },
  { 0, 3, 4, 5, 6, 7, 15, },
  { 1, 3, 4, 5, 6, 7, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 15, },
  { 2, 3, 4, 5, 6, 7, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 15, },
  { 8, 15, },
  { 0, 8, 15, },
  { 1, 8, 15, },
  { 0, 1, 8, 15, },
  { 2, 8, 15, },
  { 0, 2, 8, 15, },
  { 1, 2, 8, 15, },
  { 0, 1, 2, 8, 15, },
  { 3, 8, 15, },
  { 0, 3, 8, 15, },
  { 1, 3, 8, 15, },
  { 0, 1, 3, 8, 15, },
  { 2, 3, 8, 15, },
  { 0, 2, 3, 8, 15, },
  { 1, 2, 3, 8, 15, },
  { 0, 1, 2, 3, 8, 15, },
  { 4, 8, 15, },
  { 0, 4, 8, 15, },
  { 1, 4, 8, 15, },
  { 0, 1, 4, 8, 15, },
  { 2, 4, 8, 15, },
  { 0, 2, 4, 8, 15, },
  { 1, 2, 4, 8, 15, },
  { 0, 1, 2, 4, 8, 15, },
  { 3, 4, 8, 15, },
  { 0, 3, 4, 8, 15, },
  { 1, 3, 4, 8, 15, },
  { 0, 1, 3, 4, 8, 15, },
  { 2, 3, 4, 8, 15, },
  { 0, 2, 3, 4, 8, 15, },
  { 1, 2, 3, 4, 8, 15, },
  { 0, 1, 2, 3, 4, 8, 15, },
  { 5, 8, 15, },
  { 0, 5, 8, 15, },
  { 1, 5, 8, 15, },
  { 0, 1, 5, 8, 15, },
  { 2, 5, 8, 15, },
  { 0, 2, 5, 8, 15, },
  { 1, 2, 5, 8, 15, },
  { 0, 1, 2, 5, 8, 15, },
  { 3, 5, 8, 15, },
  { 0, 3, 5, 8, 15, },
  { 1, 3, 5, 8, 15, },
  { 0, 1, 3, 5, 8, 15, },
  { 2, 3, 5, 8, 15, },
  { 0, 2, 3, 5, 8, 15, },
  { 1, 2, 3, 5, 8, 15, },
  { 0, 1, 2, 3, 5, 8, 15, },
  { 4, 5, 8, 15, },
  { 0, 4, 5, 8, 15, },
  { 1, 4, 5, 8, 15, },
  { 0, 1, 4, 5, 8, 15, },
  { 2, 4, 5, 8, 15, },
  { 0, 2, 4, 5, 8, 15, },
  { 1, 2, 4, 5, 8, 15, },
  { 0, 1, 2, 4, 5, 8, 15, },
  { 3, 4, 5, 8, 15, },
  { 0, 3, 4, 5, 8, 15, },
  { 1, 3, 4, 5, 8, 15, },
  { 0, 1, 3, 4, 5, 8, 15, },
  { 2, 3, 4, 5, 8, 15, },
  { 0, 2, 3, 4, 5, 8, 15, },
  { 1, 2, 3, 4, 5, 8, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 15, },
  { 6, 8, 15, },
  { 0, 6, 8, 15, },
  { 1, 6, 8, 15, },
  { 0, 1, 6, 8, 15, },
  { 2, 6, 8, 15, },
  { 0, 2, 6, 8, 15, },
  { 1, 2, 6, 8, 15, },
  { 0, 1, 2, 6, 8, 15, },
  { 3, 6, 8, 15, },
  { 0, 3, 6, 8, 15, },
  { 1, 3, 6, 8, 15, },
  { 0, 1, 3, 6, 8, 15, },
  { 2, 3, 6, 8, 15, },
  { 0, 2, 3, 6, 8, 15, },
  { 1, 2, 3, 6, 8, 15, },
  { 0, 1, 2, 3, 6, 8, 15, },
  { 4, 6, 8, 15, },
  { 0, 4, 6, 8, 15, },
  { 1, 4, 6, 8, 15, },
  { 0, 1, 4, 6, 8, 15, },
  { 2, 4, 6, 8, 15, },
  { 0, 2, 4, 6, 8, 15, },
  { 1, 2, 4, 6, 8, 15, },
  { 0, 1, 2, 4, 6, 8, 15, },
  { 3, 4, 6, 8, 15, },
  { 0, 3, 4, 6, 8, 15, },
  { 1, 3, 4, 6, 8, 15, },
  { 0, 1, 3, 4, 6, 8, 15, },
  { 2, 3, 4, 6, 8, 15, },
  { 0, 2, 3, 4, 6, 8, 15, },
  { 1, 2, 3, 4, 6, 8, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 15, },
  { 5, 6, 8, 15, },
  { 0, 5, 6, 8, 15, },
  { 1, 5, 6, 8, 15, },
  { 0, 1, 5, 6, 8, 15, },
  { 2, 5, 6, 8, 15, },
  { 0, 2, 5, 6, 8, 15, },
  { 1, 2, 5, 6, 8, 15, },
  { 0, 1, 2, 5, 6, 8, 15, },
  { 3, 5, 6, 8, 15, },
  { 0, 3, 5, 6, 8, 15, },
  { 1, 3, 5, 6, 8, 15, },
  { 0, 1, 3, 5, 6, 8, 15, },
  { 2, 3, 5, 6, 8, 15, },
  { 0, 2, 3, 5, 6, 8, 15, },
  { 1, 2, 3, 5, 6, 8, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 15, },
  { 4, 5, 6, 8, 15, },
  { 0, 4, 5, 6, 8, 15, },
  { 1, 4, 5, 6, 8, 15, },
  { 0, 1, 4, 5, 6, 8, 15, },
  { 2, 4, 5, 6, 8, 15, },
  { 0, 2, 4, 5, 6, 8, 15, },
  { 1, 2, 4, 5, 6, 8, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 15, },
  { 3, 4, 5, 6, 8, 15, },
  { 0, 3, 4, 5, 6, 8, 15, },
  { 1, 3, 4, 5, 6, 8, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 15, },
  { 2, 3, 4, 5, 6, 8, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 15, },
  { 7, 8, 15, },
  { 0, 7, 8, 15, },
  { 1, 7, 8, 15, },
  { 0, 1, 7, 8, 15, },
  { 2, 7, 8, 15, },
  { 0, 2, 7, 8, 15, },
  { 1, 2, 7, 8, 15, },
  { 0, 1, 2, 7, 8, 15, },
  { 3, 7, 8, 15, },
  { 0, 3, 7, 8, 15, },
  { 1, 3, 7, 8, 15, },
  { 0, 1, 3, 7, 8, 15, },
  { 2, 3, 7, 8, 15, },
  { 0, 2, 3, 7, 8, 15, },
  { 1, 2, 3, 7, 8, 15, },
  { 0, 1, 2, 3, 7, 8, 15, },
  { 4, 7, 8, 15, },
  { 0, 4, 7, 8, 15, },
  { 1, 4, 7, 8, 15, },
  { 0, 1, 4, 7, 8, 15, },
  { 2, 4, 7, 8, 15, },
  { 0, 2, 4, 7, 8, 15, },
  { 1, 2, 4, 7, 8, 15, },
  { 0, 1, 2, 4, 7, 8, 15, },
  { 3, 4, 7, 8, 15, },
  { 0, 3, 4, 7, 8, 15, },
  { 1, 3, 4, 7, 8, 15, },
  { 0, 1, 3, 4, 7, 8, 15, },
  { 2, 3, 4, 7, 8, 15, },
  { 0, 2, 3, 4, 7, 8, 15, },
  { 1, 2, 3, 4, 7, 8, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 15, },
  { 5, 7, 8, 15, },
  { 0, 5, 7, 8, 15, },
  { 1, 5, 7, 8, 15, },
  { 0, 1, 5, 7, 8, 15, },
  { 2, 5, 7, 8, 15, },
  { 0, 2, 5, 7, 8, 15, },
  { 1, 2, 5, 7, 8, 15, },
  { 0, 1, 2, 5, 7, 8, 15, },
  { 3, 5, 7, 8, 15, },
  { 0, 3, 5, 7, 8, 15, },
  { 1, 3, 5, 7, 8, 15, },
  { 0, 1, 3, 5, 7, 8, 15, },
  { 2, 3, 5, 7, 8, 15, },
  { 0, 2, 3, 5, 7, 8, 15, },
  { 1, 2, 3, 5, 7, 8, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 15, },
  { 4, 5, 7, 8, 15, },
  { 0, 4, 5, 7, 8, 15, },
  { 1, 4, 5, 7, 8, 15, },
  { 0, 1, 4, 5, 7, 8, 15, },
  { 2, 4, 5, 7, 8, 15, },
  { 0, 2, 4, 5, 7, 8, 15, },
  { 1, 2, 4, 5, 7, 8, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 15, },
  { 3, 4, 5, 7, 8, 15, },
  { 0, 3, 4, 5, 7, 8, 15, },
  { 1, 3, 4, 5, 7, 8, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 15, },
  { 2, 3, 4, 5, 7, 8, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 15, },
  { 6, 7, 8, 15, },
  { 0, 6, 7, 8, 15, },
  { 1, 6, 7, 8, 15, },
  { 0, 1, 6, 7, 8, 15, },
  { 2, 6, 7, 8, 15, },
  { 0, 2, 6, 7, 8, 15, },
  { 1, 2, 6, 7, 8, 15, },
  { 0, 1, 2, 6, 7, 8, 15, },
  { 3, 6, 7, 8, 15, },
  { 0, 3, 6, 7, 8, 15, },
  { 1, 3, 6, 7, 8, 15, },
  { 0, 1, 3, 6, 7, 8, 15, },
  { 2, 3, 6, 7, 8, 15, },
  { 0, 2, 3, 6, 7, 8, 15, },
  { 1, 2, 3, 6, 7, 8, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 15, },
  { 4, 6, 7, 8, 15, },
  { 0, 4, 6, 7, 8, 15, },
  { 1, 4, 6, 7, 8, 15, },
  { 0, 1, 4, 6, 7, 8, 15, },
  { 2, 4, 6, 7, 8, 15, },
  { 0, 2, 4, 6, 7, 8, 15, },
  { 1, 2, 4, 6, 7, 8, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 15, },
  { 3, 4, 6, 7, 8, 15, },
  { 0, 3, 4, 6, 7, 8, 15, },
  { 1, 3, 4, 6, 7, 8, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 15, },
  { 2, 3, 4, 6, 7, 8, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 15, },
  { 5, 6, 7, 8, 15, },
  { 0, 5, 6, 7, 8, 15, },
  { 1, 5, 6, 7, 8, 15, },
  { 0, 1, 5, 6, 7, 8, 15, },
  { 2, 5, 6, 7, 8, 15, },
  { 0, 2, 5, 6, 7, 8, 15, },
  { 1, 2, 5, 6, 7, 8, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 15, },
  { 3, 5, 6, 7, 8, 15, },
  { 0, 3, 5, 6, 7, 8, 15, },
  { 1, 3, 5, 6, 7, 8, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 15, },
  { 2, 3, 5, 6, 7, 8, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 15, },
  { 4, 5, 6, 7, 8, 15, },
  { 0, 4, 5, 6, 7, 8, 15, },
  { 1, 4, 5, 6, 7, 8, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 15, },
  { 2, 4, 5, 6, 7, 8, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 15, },
  { 3, 4, 5, 6, 7, 8, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 15, },
  { 9, 15, },
  { 0, 9, 15, },
  { 1, 9, 15, },
  { 0, 1, 9, 15, },
  { 2, 9, 15, },
  { 0, 2, 9, 15, },
  { 1, 2, 9, 15, },
  { 0, 1, 2, 9, 15, },
  { 3, 9, 15, },
  { 0, 3, 9, 15, },
  { 1, 3, 9, 15, },
  { 0, 1, 3, 9, 15, },
  { 2, 3, 9, 15, },
  { 0, 2, 3, 9, 15, },
  { 1, 2, 3, 9, 15, },
  { 0, 1, 2, 3, 9, 15, },
  { 4, 9, 15, },
  { 0, 4, 9, 15, },
  { 1, 4, 9, 15, },
  { 0, 1, 4, 9, 15, },
  { 2, 4, 9, 15, },
  { 0, 2, 4, 9, 15, },
  { 1, 2, 4, 9, 15, },
  { 0, 1, 2, 4, 9, 15, },
  { 3, 4, 9, 15, },
  { 0, 3, 4, 9, 15, },
  { 1, 3, 4, 9, 15, },
  { 0, 1, 3, 4, 9, 15, },
  { 2, 3, 4, 9, 15, },
  { 0, 2, 3, 4, 9, 15, },
  { 1, 2, 3, 4, 9, 15, },
  { 0, 1, 2, 3, 4, 9, 15, },
  { 5, 9, 15, },
  { 0, 5, 9, 15, },
  { 1, 5, 9, 15, },
  { 0, 1, 5, 9, 15, },
  { 2, 5, 9, 15, },
  { 0, 2, 5, 9, 15, },
  { 1, 2, 5, 9, 15, },
  { 0, 1, 2, 5, 9, 15, },
  { 3, 5, 9, 15, },
  { 0, 3, 5, 9, 15, },
  { 1, 3, 5, 9, 15, },
  { 0, 1, 3, 5, 9, 15, },
  { 2, 3, 5, 9, 15, },
  { 0, 2, 3, 5, 9, 15, },
  { 1, 2, 3, 5, 9, 15, },
  { 0, 1, 2, 3, 5, 9, 15, },
  { 4, 5, 9, 15, },
  { 0, 4, 5, 9, 15, },
  { 1, 4, 5, 9, 15, },
  { 0, 1, 4, 5, 9, 15, },
  { 2, 4, 5, 9, 15, },
  { 0, 2, 4, 5, 9, 15, },
  { 1, 2, 4, 5, 9, 15, },
  { 0, 1, 2, 4, 5, 9, 15, },
  { 3, 4, 5, 9, 15, },
  { 0, 3, 4, 5, 9, 15, },
  { 1, 3, 4, 5, 9, 15, },
  { 0, 1, 3, 4, 5, 9, 15, },
  { 2, 3, 4, 5, 9, 15, },
  { 0, 2, 3, 4, 5, 9, 15, },
  { 1, 2, 3, 4, 5, 9, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 15, },
  { 6, 9, 15, },
  { 0, 6, 9, 15, },
  { 1, 6, 9, 15, },
  { 0, 1, 6, 9, 15, },
  { 2, 6, 9, 15, },
  { 0, 2, 6, 9, 15, },
  { 1, 2, 6, 9, 15, },
  { 0, 1, 2, 6, 9, 15, },
  { 3, 6, 9, 15, },
  { 0, 3, 6, 9, 15, },
  { 1, 3, 6, 9, 15, },
  { 0, 1, 3, 6, 9, 15, },
  { 2, 3, 6, 9, 15, },
  { 0, 2, 3, 6, 9, 15, },
  { 1, 2, 3, 6, 9, 15, },
  { 0, 1, 2, 3, 6, 9, 15, },
  { 4, 6, 9, 15, },
  { 0, 4, 6, 9, 15, },
  { 1, 4, 6, 9, 15, },
  { 0, 1, 4, 6, 9, 15, },
  { 2, 4, 6, 9, 15, },
  { 0, 2, 4, 6, 9, 15, },
  { 1, 2, 4, 6, 9, 15, },
  { 0, 1, 2, 4, 6, 9, 15, },
  { 3, 4, 6, 9, 15, },
  { 0, 3, 4, 6, 9, 15, },
  { 1, 3, 4, 6, 9, 15, },
  { 0, 1, 3, 4, 6, 9, 15, },
  { 2, 3, 4, 6, 9, 15, },
  { 0, 2, 3, 4, 6, 9, 15, },
  { 1, 2, 3, 4, 6, 9, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 15, },
  { 5, 6, 9, 15, },
  { 0, 5, 6, 9, 15, },
  { 1, 5, 6, 9, 15, },
  { 0, 1, 5, 6, 9, 15, },
  { 2, 5, 6, 9, 15, },
  { 0, 2, 5, 6, 9, 15, },
  { 1, 2, 5, 6, 9, 15, },
  { 0, 1, 2, 5, 6, 9, 15, },
  { 3, 5, 6, 9, 15, },
  { 0, 3, 5, 6, 9, 15, },
  { 1, 3, 5, 6, 9, 15, },
  { 0, 1, 3, 5, 6, 9, 15, },
  { 2, 3, 5, 6, 9, 15, },
  { 0, 2, 3, 5, 6, 9, 15, },
  { 1, 2, 3, 5, 6, 9, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 15, },
  { 4, 5, 6, 9, 15, },
  { 0, 4, 5, 6, 9, 15, },
  { 1, 4, 5, 6, 9, 15, },
  { 0, 1, 4, 5, 6, 9, 15, },
  { 2, 4, 5, 6, 9, 15, },
  { 0, 2, 4, 5, 6, 9, 15, },
  { 1, 2, 4, 5, 6, 9, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 15, },
  { 3, 4, 5, 6, 9, 15, },
  { 0, 3, 4, 5, 6, 9, 15, },
  { 1, 3, 4, 5, 6, 9, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 15, },
  { 2, 3, 4, 5, 6, 9, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 15, },
  { 7, 9, 15, },
  { 0, 7, 9, 15, },
  { 1, 7, 9, 15, },
  { 0, 1, 7, 9, 15, },
  { 2, 7, 9, 15, },
  { 0, 2, 7, 9, 15, },
  { 1, 2, 7, 9, 15, },
  { 0, 1, 2, 7, 9, 15, },
  { 3, 7, 9, 15, },
  { 0, 3, 7, 9, 15, },
  { 1, 3, 7, 9, 15, },
  { 0, 1, 3, 7, 9, 15, },
  { 2, 3, 7, 9, 15, },
  { 0, 2, 3, 7, 9, 15, },
  { 1, 2, 3, 7, 9, 15, },
  { 0, 1, 2, 3, 7, 9, 15, },
  { 4, 7, 9, 15, },
  { 0, 4, 7, 9, 15, },
  { 1, 4, 7, 9, 15, },
  { 0, 1, 4, 7, 9, 15, },
  { 2, 4, 7, 9, 15, },
  { 0, 2, 4, 7, 9, 15, },
  { 1, 2, 4, 7, 9, 15, },
  { 0, 1, 2, 4, 7, 9, 15, },
  { 3, 4, 7, 9, 15, },
  { 0, 3, 4, 7, 9, 15, },
  { 1, 3, 4, 7, 9, 15, },
  { 0, 1, 3, 4, 7, 9, 15, },
  { 2, 3, 4, 7, 9, 15, },
  { 0, 2, 3, 4, 7, 9, 15, },
  { 1, 2, 3, 4, 7, 9, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 15, },
  { 5, 7, 9, 15, },
  { 0, 5, 7, 9, 15, },
  { 1, 5, 7, 9, 15, },
  { 0, 1, 5, 7, 9, 15, },
  { 2, 5, 7, 9, 15, },
  { 0, 2, 5, 7, 9, 15, },
  { 1, 2, 5, 7, 9, 15, },
  { 0, 1, 2, 5, 7, 9, 15, },
  { 3, 5, 7, 9, 15, },
  { 0, 3, 5, 7, 9, 15, },
  { 1, 3, 5, 7, 9, 15, },
  { 0, 1, 3, 5, 7, 9, 15, },
  { 2, 3, 5, 7, 9, 15, },
  { 0, 2, 3, 5, 7, 9, 15, },
  { 1, 2, 3, 5, 7, 9, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 15, },
  { 4, 5, 7, 9, 15, },
  { 0, 4, 5, 7, 9, 15, },
  { 1, 4, 5, 7, 9, 15, },
  { 0, 1, 4, 5, 7, 9, 15, },
  { 2, 4, 5, 7, 9, 15, },
  { 0, 2, 4, 5, 7, 9, 15, },
  { 1, 2, 4, 5, 7, 9, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 15, },
  { 3, 4, 5, 7, 9, 15, },
  { 0, 3, 4, 5, 7, 9, 15, },
  { 1, 3, 4, 5, 7, 9, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 15, },
  { 2, 3, 4, 5, 7, 9, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 15, },
  { 6, 7, 9, 15, },
  { 0, 6, 7, 9, 15, },
  { 1, 6, 7, 9, 15, },
  { 0, 1, 6, 7, 9, 15, },
  { 2, 6, 7, 9, 15, },
  { 0, 2, 6, 7, 9, 15, },
  { 1, 2, 6, 7, 9, 15, },
  { 0, 1, 2, 6, 7, 9, 15, },
  { 3, 6, 7, 9, 15, },
  { 0, 3, 6, 7, 9, 15, },
  { 1, 3, 6, 7, 9, 15, },
  { 0, 1, 3, 6, 7, 9, 15, },
  { 2, 3, 6, 7, 9, 15, },
  { 0, 2, 3, 6, 7, 9, 15, },
  { 1, 2, 3, 6, 7, 9, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 15, },
  { 4, 6, 7, 9, 15, },
  { 0, 4, 6, 7, 9, 15, },
  { 1, 4, 6, 7, 9, 15, },
  { 0, 1, 4, 6, 7, 9, 15, },
  { 2, 4, 6, 7, 9, 15, },
  { 0, 2, 4, 6, 7, 9, 15, },
  { 1, 2, 4, 6, 7, 9, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 15, },
  { 3, 4, 6, 7, 9, 15, },
  { 0, 3, 4, 6, 7, 9, 15, },
  { 1, 3, 4, 6, 7, 9, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 15, },
  { 2, 3, 4, 6, 7, 9, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 15, },
  { 5, 6, 7, 9, 15, },
  { 0, 5, 6, 7, 9, 15, },
  { 1, 5, 6, 7, 9, 15, },
  { 0, 1, 5, 6, 7, 9, 15, },
  { 2, 5, 6, 7, 9, 15, },
  { 0, 2, 5, 6, 7, 9, 15, },
  { 1, 2, 5, 6, 7, 9, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 15, },
  { 3, 5, 6, 7, 9, 15, },
  { 0, 3, 5, 6, 7, 9, 15, },
  { 1, 3, 5, 6, 7, 9, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 15, },
  { 2, 3, 5, 6, 7, 9, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 15, },
  { 4, 5, 6, 7, 9, 15, },
  { 0, 4, 5, 6, 7, 9, 15, },
  { 1, 4, 5, 6, 7, 9, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 15, },
  { 2, 4, 5, 6, 7, 9, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 15, },
  { 3, 4, 5, 6, 7, 9, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 15, },
  { 8, 9, 15, },
  { 0, 8, 9, 15, },
  { 1, 8, 9, 15, },
  { 0, 1, 8, 9, 15, },
  { 2, 8, 9, 15, },
  { 0, 2, 8, 9, 15, },
  { 1, 2, 8, 9, 15, },
  { 0, 1, 2, 8, 9, 15, },
  { 3, 8, 9, 15, },
  { 0, 3, 8, 9, 15, },
  { 1, 3, 8, 9, 15, },
  { 0, 1, 3, 8, 9, 15, },
  { 2, 3, 8, 9, 15, },
  { 0, 2, 3, 8, 9, 15, },
  { 1, 2, 3, 8, 9, 15, },
  { 0, 1, 2, 3, 8, 9, 15, },
  { 4, 8, 9, 15, },
  { 0, 4, 8, 9, 15, },
  { 1, 4, 8, 9, 15, },
  { 0, 1, 4, 8, 9, 15, },
  { 2, 4, 8, 9, 15, },
  { 0, 2, 4, 8, 9, 15, },
  { 1, 2, 4, 8, 9, 15, },
  { 0, 1, 2, 4, 8, 9, 15, },
  { 3, 4, 8, 9, 15, },
  { 0, 3, 4, 8, 9, 15, },
  { 1, 3, 4, 8, 9, 15, },
  { 0, 1, 3, 4, 8, 9, 15, },
  { 2, 3, 4, 8, 9, 15, },
  { 0, 2, 3, 4, 8, 9, 15, },
  { 1, 2, 3, 4, 8, 9, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 15, },
  { 5, 8, 9, 15, },
  { 0, 5, 8, 9, 15, },
  { 1, 5, 8, 9, 15, },
  { 0, 1, 5, 8, 9, 15, },
  { 2, 5, 8, 9, 15, },
  { 0, 2, 5, 8, 9, 15, },
  { 1, 2, 5, 8, 9, 15, },
  { 0, 1, 2, 5, 8, 9, 15, },
  { 3, 5, 8, 9, 15, },
  { 0, 3, 5, 8, 9, 15, },
  { 1, 3, 5, 8, 9, 15, },
  { 0, 1, 3, 5, 8, 9, 15, },
  { 2, 3, 5, 8, 9, 15, },
  { 0, 2, 3, 5, 8, 9, 15, },
  { 1, 2, 3, 5, 8, 9, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 15, },
  { 4, 5, 8, 9, 15, },
  { 0, 4, 5, 8, 9, 15, },
  { 1, 4, 5, 8, 9, 15, },
  { 0, 1, 4, 5, 8, 9, 15, },
  { 2, 4, 5, 8, 9, 15, },
  { 0, 2, 4, 5, 8, 9, 15, },
  { 1, 2, 4, 5, 8, 9, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 15, },
  { 3, 4, 5, 8, 9, 15, },
  { 0, 3, 4, 5, 8, 9, 15, },
  { 1, 3, 4, 5, 8, 9, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 15, },
  { 2, 3, 4, 5, 8, 9, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 15, },
  { 6, 8, 9, 15, },
  { 0, 6, 8, 9, 15, },
  { 1, 6, 8, 9, 15, },
  { 0, 1, 6, 8, 9, 15, },
  { 2, 6, 8, 9, 15, },
  { 0, 2, 6, 8, 9, 15, },
  { 1, 2, 6, 8, 9, 15, },
  { 0, 1, 2, 6, 8, 9, 15, },
  { 3, 6, 8, 9, 15, },
  { 0, 3, 6, 8, 9, 15, },
  { 1, 3, 6, 8, 9, 15, },
  { 0, 1, 3, 6, 8, 9, 15, },
  { 2, 3, 6, 8, 9, 15, },
  { 0, 2, 3, 6, 8, 9, 15, },
  { 1, 2, 3, 6, 8, 9, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 15, },
  { 4, 6, 8, 9, 15, },
  { 0, 4, 6, 8, 9, 15, },
  { 1, 4, 6, 8, 9, 15, },
  { 0, 1, 4, 6, 8, 9, 15, },
  { 2, 4, 6, 8, 9, 15, },
  { 0, 2, 4, 6, 8, 9, 15, },
  { 1, 2, 4, 6, 8, 9, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 15, },
  { 3, 4, 6, 8, 9, 15, },
  { 0, 3, 4, 6, 8, 9, 15, },
  { 1, 3, 4, 6, 8, 9, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 15, },
  { 2, 3, 4, 6, 8, 9, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 15, },
  { 5, 6, 8, 9, 15, },
  { 0, 5, 6, 8, 9, 15, },
  { 1, 5, 6, 8, 9, 15, },
  { 0, 1, 5, 6, 8, 9, 15, },
  { 2, 5, 6, 8, 9, 15, },
  { 0, 2, 5, 6, 8, 9, 15, },
  { 1, 2, 5, 6, 8, 9, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 15, },
  { 3, 5, 6, 8, 9, 15, },
  { 0, 3, 5, 6, 8, 9, 15, },
  { 1, 3, 5, 6, 8, 9, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 15, },
  { 2, 3, 5, 6, 8, 9, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 15, },
  { 4, 5, 6, 8, 9, 15, },
  { 0, 4, 5, 6, 8, 9, 15, },
  { 1, 4, 5, 6, 8, 9, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 15, },
  { 2, 4, 5, 6, 8, 9, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 15, },
  { 3, 4, 5, 6, 8, 9, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 15, },
  { 7, 8, 9, 15, },
  { 0, 7, 8, 9, 15, },
  { 1, 7, 8, 9, 15, },
  { 0, 1, 7, 8, 9, 15, },
  { 2, 7, 8, 9, 15, },
  { 0, 2, 7, 8, 9, 15, },
  { 1, 2, 7, 8, 9, 15, },
  { 0, 1, 2, 7, 8, 9, 15, },
  { 3, 7, 8, 9, 15, },
  { 0, 3, 7, 8, 9, 15, },
  { 1, 3, 7, 8, 9, 15, },
  { 0, 1, 3, 7, 8, 9, 15, },
  { 2, 3, 7, 8, 9, 15, },
  { 0, 2, 3, 7, 8, 9, 15, },
  { 1, 2, 3, 7, 8, 9, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 15, },
  { 4, 7, 8, 9, 15, },
  { 0, 4, 7, 8, 9, 15, },
  { 1, 4, 7, 8, 9, 15, },
  { 0, 1, 4, 7, 8, 9, 15, },
  { 2, 4, 7, 8, 9, 15, },
  { 0, 2, 4, 7, 8, 9, 15, },
  { 1, 2, 4, 7, 8, 9, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 15, },
  { 3, 4, 7, 8, 9, 15, },
  { 0, 3, 4, 7, 8, 9, 15, },
  { 1, 3, 4, 7, 8, 9, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 15, },
  { 2, 3, 4, 7, 8, 9, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 15, },
  { 5, 7, 8, 9, 15, },
  { 0, 5, 7, 8, 9, 15, },
  { 1, 5, 7, 8, 9, 15, },
  { 0, 1, 5, 7, 8, 9, 15, },
  { 2, 5, 7, 8, 9, 15, },
  { 0, 2, 5, 7, 8, 9, 15, },
  { 1, 2, 5, 7, 8, 9, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 15, },
  { 3, 5, 7, 8, 9, 15, },
  { 0, 3, 5, 7, 8, 9, 15, },
  { 1, 3, 5, 7, 8, 9, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 15, },
  { 2, 3, 5, 7, 8, 9, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 15, },
  { 4, 5, 7, 8, 9, 15, },
  { 0, 4, 5, 7, 8, 9, 15, },
  { 1, 4, 5, 7, 8, 9, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 15, },
  { 2, 4, 5, 7, 8, 9, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 15, },
  { 3, 4, 5, 7, 8, 9, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 15, },
  { 6, 7, 8, 9, 15, },
  { 0, 6, 7, 8, 9, 15, },
  { 1, 6, 7, 8, 9, 15, },
  { 0, 1, 6, 7, 8, 9, 15, },
  { 2, 6, 7, 8, 9, 15, },
  { 0, 2, 6, 7, 8, 9, 15, },
  { 1, 2, 6, 7, 8, 9, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 15, },
  { 3, 6, 7, 8, 9, 15, },
  { 0, 3, 6, 7, 8, 9, 15, },
  { 1, 3, 6, 7, 8, 9, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 15, },
  { 2, 3, 6, 7, 8, 9, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 15, },
  { 4, 6, 7, 8, 9, 15, },
  { 0, 4, 6, 7, 8, 9, 15, },
  { 1, 4, 6, 7, 8, 9, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 15, },
  { 2, 4, 6, 7, 8, 9, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 15, },
  { 3, 4, 6, 7, 8, 9, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 15, },
  { 5, 6, 7, 8, 9, 15, },
  { 0, 5, 6, 7, 8, 9, 15, },
  { 1, 5, 6, 7, 8, 9, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 15, },
  { 2, 5, 6, 7, 8, 9, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 15, },
  { 3, 5, 6, 7, 8, 9, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 15, },
  { 4, 5, 6, 7, 8, 9, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 15, },
  { 10, 15, },
  { 0, 10, 15, },
  { 1, 10, 15, },
  { 0, 1, 10, 15, },
  { 2, 10, 15, },
  { 0, 2, 10, 15, },
  { 1, 2, 10, 15, },
  { 0, 1, 2, 10, 15, },
  { 3, 10, 15, },
  { 0, 3, 10, 15, },
  { 1, 3, 10, 15, },
  { 0, 1, 3, 10, 15, },
  { 2, 3, 10, 15, },
  { 0, 2, 3, 10, 15, },
  { 1, 2, 3, 10, 15, },
  { 0, 1, 2, 3, 10, 15, },
  { 4, 10, 15, },
  { 0, 4, 10, 15, },
  { 1, 4, 10, 15, },
  { 0, 1, 4, 10, 15, },
  { 2, 4, 10, 15, },
  { 0, 2, 4, 10, 15, },
  { 1, 2, 4, 10, 15, },
  { 0, 1, 2, 4, 10, 15, },
  { 3, 4, 10, 15, },
  { 0, 3, 4, 10, 15, },
  { 1, 3, 4, 10, 15, },
  { 0, 1, 3, 4, 10, 15, },
  { 2, 3, 4, 10, 15, },
  { 0, 2, 3, 4, 10, 15, },
  { 1, 2, 3, 4, 10, 15, },
  { 0, 1, 2, 3, 4, 10, 15, },
  { 5, 10, 15, },
  { 0, 5, 10, 15, },
  { 1, 5, 10, 15, },
  { 0, 1, 5, 10, 15, },
  { 2, 5, 10, 15, },
  { 0, 2, 5, 10, 15, },
  { 1, 2, 5, 10, 15, },
  { 0, 1, 2, 5, 10, 15, },
  { 3, 5, 10, 15, },
  { 0, 3, 5, 10, 15, },
  { 1, 3, 5, 10, 15, },
  { 0, 1, 3, 5, 10, 15, },
  { 2, 3, 5, 10, 15, },
  { 0, 2, 3, 5, 10, 15, },
  { 1, 2, 3, 5, 10, 15, },
  { 0, 1, 2, 3, 5, 10, 15, },
  { 4, 5, 10, 15, },
  { 0, 4, 5, 10, 15, },
  { 1, 4, 5, 10, 15, },
  { 0, 1, 4, 5, 10, 15, },
  { 2, 4, 5, 10, 15, },
  { 0, 2, 4, 5, 10, 15, },
  { 1, 2, 4, 5, 10, 15, },
  { 0, 1, 2, 4, 5, 10, 15, },
  { 3, 4, 5, 10, 15, },
  { 0, 3, 4, 5, 10, 15, },
  { 1, 3, 4, 5, 10, 15, },
  { 0, 1, 3, 4, 5, 10, 15, },
  { 2, 3, 4, 5, 10, 15, },
  { 0, 2, 3, 4, 5, 10, 15, },
  { 1, 2, 3, 4, 5, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 15, },
  { 6, 10, 15, },
  { 0, 6, 10, 15, },
  { 1, 6, 10, 15, },
  { 0, 1, 6, 10, 15, },
  { 2, 6, 10, 15, },
  { 0, 2, 6, 10, 15, },
  { 1, 2, 6, 10, 15, },
  { 0, 1, 2, 6, 10, 15, },
  { 3, 6, 10, 15, },
  { 0, 3, 6, 10, 15, },
  { 1, 3, 6, 10, 15, },
  { 0, 1, 3, 6, 10, 15, },
  { 2, 3, 6, 10, 15, },
  { 0, 2, 3, 6, 10, 15, },
  { 1, 2, 3, 6, 10, 15, },
  { 0, 1, 2, 3, 6, 10, 15, },
  { 4, 6, 10, 15, },
  { 0, 4, 6, 10, 15, },
  { 1, 4, 6, 10, 15, },
  { 0, 1, 4, 6, 10, 15, },
  { 2, 4, 6, 10, 15, },
  { 0, 2, 4, 6, 10, 15, },
  { 1, 2, 4, 6, 10, 15, },
  { 0, 1, 2, 4, 6, 10, 15, },
  { 3, 4, 6, 10, 15, },
  { 0, 3, 4, 6, 10, 15, },
  { 1, 3, 4, 6, 10, 15, },
  { 0, 1, 3, 4, 6, 10, 15, },
  { 2, 3, 4, 6, 10, 15, },
  { 0, 2, 3, 4, 6, 10, 15, },
  { 1, 2, 3, 4, 6, 10, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 15, },
  { 5, 6, 10, 15, },
  { 0, 5, 6, 10, 15, },
  { 1, 5, 6, 10, 15, },
  { 0, 1, 5, 6, 10, 15, },
  { 2, 5, 6, 10, 15, },
  { 0, 2, 5, 6, 10, 15, },
  { 1, 2, 5, 6, 10, 15, },
  { 0, 1, 2, 5, 6, 10, 15, },
  { 3, 5, 6, 10, 15, },
  { 0, 3, 5, 6, 10, 15, },
  { 1, 3, 5, 6, 10, 15, },
  { 0, 1, 3, 5, 6, 10, 15, },
  { 2, 3, 5, 6, 10, 15, },
  { 0, 2, 3, 5, 6, 10, 15, },
  { 1, 2, 3, 5, 6, 10, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 15, },
  { 4, 5, 6, 10, 15, },
  { 0, 4, 5, 6, 10, 15, },
  { 1, 4, 5, 6, 10, 15, },
  { 0, 1, 4, 5, 6, 10, 15, },
  { 2, 4, 5, 6, 10, 15, },
  { 0, 2, 4, 5, 6, 10, 15, },
  { 1, 2, 4, 5, 6, 10, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 15, },
  { 3, 4, 5, 6, 10, 15, },
  { 0, 3, 4, 5, 6, 10, 15, },
  { 1, 3, 4, 5, 6, 10, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 15, },
  { 2, 3, 4, 5, 6, 10, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 15, },
  { 7, 10, 15, },
  { 0, 7, 10, 15, },
  { 1, 7, 10, 15, },
  { 0, 1, 7, 10, 15, },
  { 2, 7, 10, 15, },
  { 0, 2, 7, 10, 15, },
  { 1, 2, 7, 10, 15, },
  { 0, 1, 2, 7, 10, 15, },
  { 3, 7, 10, 15, },
  { 0, 3, 7, 10, 15, },
  { 1, 3, 7, 10, 15, },
  { 0, 1, 3, 7, 10, 15, },
  { 2, 3, 7, 10, 15, },
  { 0, 2, 3, 7, 10, 15, },
  { 1, 2, 3, 7, 10, 15, },
  { 0, 1, 2, 3, 7, 10, 15, },
  { 4, 7, 10, 15, },
  { 0, 4, 7, 10, 15, },
  { 1, 4, 7, 10, 15, },
  { 0, 1, 4, 7, 10, 15, },
  { 2, 4, 7, 10, 15, },
  { 0, 2, 4, 7, 10, 15, },
  { 1, 2, 4, 7, 10, 15, },
  { 0, 1, 2, 4, 7, 10, 15, },
  { 3, 4, 7, 10, 15, },
  { 0, 3, 4, 7, 10, 15, },
  { 1, 3, 4, 7, 10, 15, },
  { 0, 1, 3, 4, 7, 10, 15, },
  { 2, 3, 4, 7, 10, 15, },
  { 0, 2, 3, 4, 7, 10, 15, },
  { 1, 2, 3, 4, 7, 10, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 15, },
  { 5, 7, 10, 15, },
  { 0, 5, 7, 10, 15, },
  { 1, 5, 7, 10, 15, },
  { 0, 1, 5, 7, 10, 15, },
  { 2, 5, 7, 10, 15, },
  { 0, 2, 5, 7, 10, 15, },
  { 1, 2, 5, 7, 10, 15, },
  { 0, 1, 2, 5, 7, 10, 15, },
  { 3, 5, 7, 10, 15, },
  { 0, 3, 5, 7, 10, 15, },
  { 1, 3, 5, 7, 10, 15, },
  { 0, 1, 3, 5, 7, 10, 15, },
  { 2, 3, 5, 7, 10, 15, },
  { 0, 2, 3, 5, 7, 10, 15, },
  { 1, 2, 3, 5, 7, 10, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 15, },
  { 4, 5, 7, 10, 15, },
  { 0, 4, 5, 7, 10, 15, },
  { 1, 4, 5, 7, 10, 15, },
  { 0, 1, 4, 5, 7, 10, 15, },
  { 2, 4, 5, 7, 10, 15, },
  { 0, 2, 4, 5, 7, 10, 15, },
  { 1, 2, 4, 5, 7, 10, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 15, },
  { 3, 4, 5, 7, 10, 15, },
  { 0, 3, 4, 5, 7, 10, 15, },
  { 1, 3, 4, 5, 7, 10, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 15, },
  { 2, 3, 4, 5, 7, 10, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 15, },
  { 6, 7, 10, 15, },
  { 0, 6, 7, 10, 15, },
  { 1, 6, 7, 10, 15, },
  { 0, 1, 6, 7, 10, 15, },
  { 2, 6, 7, 10, 15, },
  { 0, 2, 6, 7, 10, 15, },
  { 1, 2, 6, 7, 10, 15, },
  { 0, 1, 2, 6, 7, 10, 15, },
  { 3, 6, 7, 10, 15, },
  { 0, 3, 6, 7, 10, 15, },
  { 1, 3, 6, 7, 10, 15, },
  { 0, 1, 3, 6, 7, 10, 15, },
  { 2, 3, 6, 7, 10, 15, },
  { 0, 2, 3, 6, 7, 10, 15, },
  { 1, 2, 3, 6, 7, 10, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 15, },
  { 4, 6, 7, 10, 15, },
  { 0, 4, 6, 7, 10, 15, },
  { 1, 4, 6, 7, 10, 15, },
  { 0, 1, 4, 6, 7, 10, 15, },
  { 2, 4, 6, 7, 10, 15, },
  { 0, 2, 4, 6, 7, 10, 15, },
  { 1, 2, 4, 6, 7, 10, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 15, },
  { 3, 4, 6, 7, 10, 15, },
  { 0, 3, 4, 6, 7, 10, 15, },
  { 1, 3, 4, 6, 7, 10, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 15, },
  { 2, 3, 4, 6, 7, 10, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 15, },
  { 5, 6, 7, 10, 15, },
  { 0, 5, 6, 7, 10, 15, },
  { 1, 5, 6, 7, 10, 15, },
  { 0, 1, 5, 6, 7, 10, 15, },
  { 2, 5, 6, 7, 10, 15, },
  { 0, 2, 5, 6, 7, 10, 15, },
  { 1, 2, 5, 6, 7, 10, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 15, },
  { 3, 5, 6, 7, 10, 15, },
  { 0, 3, 5, 6, 7, 10, 15, },
  { 1, 3, 5, 6, 7, 10, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 15, },
  { 2, 3, 5, 6, 7, 10, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 15, },
  { 4, 5, 6, 7, 10, 15, },
  { 0, 4, 5, 6, 7, 10, 15, },
  { 1, 4, 5, 6, 7, 10, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 15, },
  { 2, 4, 5, 6, 7, 10, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 15, },
  { 3, 4, 5, 6, 7, 10, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 15, },
  { 8, 10, 15, },
  { 0, 8, 10, 15, },
  { 1, 8, 10, 15, },
  { 0, 1, 8, 10, 15, },
  { 2, 8, 10, 15, },
  { 0, 2, 8, 10, 15, },
  { 1, 2, 8, 10, 15, },
  { 0, 1, 2, 8, 10, 15, },
  { 3, 8, 10, 15, },
  { 0, 3, 8, 10, 15, },
  { 1, 3, 8, 10, 15, },
  { 0, 1, 3, 8, 10, 15, },
  { 2, 3, 8, 10, 15, },
  { 0, 2, 3, 8, 10, 15, },
  { 1, 2, 3, 8, 10, 15, },
  { 0, 1, 2, 3, 8, 10, 15, },
  { 4, 8, 10, 15, },
  { 0, 4, 8, 10, 15, },
  { 1, 4, 8, 10, 15, },
  { 0, 1, 4, 8, 10, 15, },
  { 2, 4, 8, 10, 15, },
  { 0, 2, 4, 8, 10, 15, },
  { 1, 2, 4, 8, 10, 15, },
  { 0, 1, 2, 4, 8, 10, 15, },
  { 3, 4, 8, 10, 15, },
  { 0, 3, 4, 8, 10, 15, },
  { 1, 3, 4, 8, 10, 15, },
  { 0, 1, 3, 4, 8, 10, 15, },
  { 2, 3, 4, 8, 10, 15, },
  { 0, 2, 3, 4, 8, 10, 15, },
  { 1, 2, 3, 4, 8, 10, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 15, },
  { 5, 8, 10, 15, },
  { 0, 5, 8, 10, 15, },
  { 1, 5, 8, 10, 15, },
  { 0, 1, 5, 8, 10, 15, },
  { 2, 5, 8, 10, 15, },
  { 0, 2, 5, 8, 10, 15, },
  { 1, 2, 5, 8, 10, 15, },
  { 0, 1, 2, 5, 8, 10, 15, },
  { 3, 5, 8, 10, 15, },
  { 0, 3, 5, 8, 10, 15, },
  { 1, 3, 5, 8, 10, 15, },
  { 0, 1, 3, 5, 8, 10, 15, },
  { 2, 3, 5, 8, 10, 15, },
  { 0, 2, 3, 5, 8, 10, 15, },
  { 1, 2, 3, 5, 8, 10, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 15, },
  { 4, 5, 8, 10, 15, },
  { 0, 4, 5, 8, 10, 15, },
  { 1, 4, 5, 8, 10, 15, },
  { 0, 1, 4, 5, 8, 10, 15, },
  { 2, 4, 5, 8, 10, 15, },
  { 0, 2, 4, 5, 8, 10, 15, },
  { 1, 2, 4, 5, 8, 10, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 15, },
  { 3, 4, 5, 8, 10, 15, },
  { 0, 3, 4, 5, 8, 10, 15, },
  { 1, 3, 4, 5, 8, 10, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 15, },
  { 2, 3, 4, 5, 8, 10, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 15, },
  { 6, 8, 10, 15, },
  { 0, 6, 8, 10, 15, },
  { 1, 6, 8, 10, 15, },
  { 0, 1, 6, 8, 10, 15, },
  { 2, 6, 8, 10, 15, },
  { 0, 2, 6, 8, 10, 15, },
  { 1, 2, 6, 8, 10, 15, },
  { 0, 1, 2, 6, 8, 10, 15, },
  { 3, 6, 8, 10, 15, },
  { 0, 3, 6, 8, 10, 15, },
  { 1, 3, 6, 8, 10, 15, },
  { 0, 1, 3, 6, 8, 10, 15, },
  { 2, 3, 6, 8, 10, 15, },
  { 0, 2, 3, 6, 8, 10, 15, },
  { 1, 2, 3, 6, 8, 10, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 15, },
  { 4, 6, 8, 10, 15, },
  { 0, 4, 6, 8, 10, 15, },
  { 1, 4, 6, 8, 10, 15, },
  { 0, 1, 4, 6, 8, 10, 15, },
  { 2, 4, 6, 8, 10, 15, },
  { 0, 2, 4, 6, 8, 10, 15, },
  { 1, 2, 4, 6, 8, 10, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 15, },
  { 3, 4, 6, 8, 10, 15, },
  { 0, 3, 4, 6, 8, 10, 15, },
  { 1, 3, 4, 6, 8, 10, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 15, },
  { 2, 3, 4, 6, 8, 10, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 15, },
  { 5, 6, 8, 10, 15, },
  { 0, 5, 6, 8, 10, 15, },
  { 1, 5, 6, 8, 10, 15, },
  { 0, 1, 5, 6, 8, 10, 15, },
  { 2, 5, 6, 8, 10, 15, },
  { 0, 2, 5, 6, 8, 10, 15, },
  { 1, 2, 5, 6, 8, 10, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 15, },
  { 3, 5, 6, 8, 10, 15, },
  { 0, 3, 5, 6, 8, 10, 15, },
  { 1, 3, 5, 6, 8, 10, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 15, },
  { 2, 3, 5, 6, 8, 10, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 15, },
  { 4, 5, 6, 8, 10, 15, },
  { 0, 4, 5, 6, 8, 10, 15, },
  { 1, 4, 5, 6, 8, 10, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 15, },
  { 2, 4, 5, 6, 8, 10, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 15, },
  { 3, 4, 5, 6, 8, 10, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 15, },
  { 7, 8, 10, 15, },
  { 0, 7, 8, 10, 15, },
  { 1, 7, 8, 10, 15, },
  { 0, 1, 7, 8, 10, 15, },
  { 2, 7, 8, 10, 15, },
  { 0, 2, 7, 8, 10, 15, },
  { 1, 2, 7, 8, 10, 15, },
  { 0, 1, 2, 7, 8, 10, 15, },
  { 3, 7, 8, 10, 15, },
  { 0, 3, 7, 8, 10, 15, },
  { 1, 3, 7, 8, 10, 15, },
  { 0, 1, 3, 7, 8, 10, 15, },
  { 2, 3, 7, 8, 10, 15, },
  { 0, 2, 3, 7, 8, 10, 15, },
  { 1, 2, 3, 7, 8, 10, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 15, },
  { 4, 7, 8, 10, 15, },
  { 0, 4, 7, 8, 10, 15, },
  { 1, 4, 7, 8, 10, 15, },
  { 0, 1, 4, 7, 8, 10, 15, },
  { 2, 4, 7, 8, 10, 15, },
  { 0, 2, 4, 7, 8, 10, 15, },
  { 1, 2, 4, 7, 8, 10, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 15, },
  { 3, 4, 7, 8, 10, 15, },
  { 0, 3, 4, 7, 8, 10, 15, },
  { 1, 3, 4, 7, 8, 10, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 15, },
  { 2, 3, 4, 7, 8, 10, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 15, },
  { 5, 7, 8, 10, 15, },
  { 0, 5, 7, 8, 10, 15, },
  { 1, 5, 7, 8, 10, 15, },
  { 0, 1, 5, 7, 8, 10, 15, },
  { 2, 5, 7, 8, 10, 15, },
  { 0, 2, 5, 7, 8, 10, 15, },
  { 1, 2, 5, 7, 8, 10, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 15, },
  { 3, 5, 7, 8, 10, 15, },
  { 0, 3, 5, 7, 8, 10, 15, },
  { 1, 3, 5, 7, 8, 10, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 15, },
  { 2, 3, 5, 7, 8, 10, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 15, },
  { 4, 5, 7, 8, 10, 15, },
  { 0, 4, 5, 7, 8, 10, 15, },
  { 1, 4, 5, 7, 8, 10, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 15, },
  { 2, 4, 5, 7, 8, 10, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 15, },
  { 3, 4, 5, 7, 8, 10, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 15, },
  { 6, 7, 8, 10, 15, },
  { 0, 6, 7, 8, 10, 15, },
  { 1, 6, 7, 8, 10, 15, },
  { 0, 1, 6, 7, 8, 10, 15, },
  { 2, 6, 7, 8, 10, 15, },
  { 0, 2, 6, 7, 8, 10, 15, },
  { 1, 2, 6, 7, 8, 10, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 15, },
  { 3, 6, 7, 8, 10, 15, },
  { 0, 3, 6, 7, 8, 10, 15, },
  { 1, 3, 6, 7, 8, 10, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 15, },
  { 2, 3, 6, 7, 8, 10, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 15, },
  { 4, 6, 7, 8, 10, 15, },
  { 0, 4, 6, 7, 8, 10, 15, },
  { 1, 4, 6, 7, 8, 10, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 15, },
  { 2, 4, 6, 7, 8, 10, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 15, },
  { 3, 4, 6, 7, 8, 10, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 15, },
  { 5, 6, 7, 8, 10, 15, },
  { 0, 5, 6, 7, 8, 10, 15, },
  { 1, 5, 6, 7, 8, 10, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 15, },
  { 2, 5, 6, 7, 8, 10, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 15, },
  { 3, 5, 6, 7, 8, 10, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 15, },
  { 4, 5, 6, 7, 8, 10, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 15, },
  { 9, 10, 15, },
  { 0, 9, 10, 15, },
  { 1, 9, 10, 15, },
  { 0, 1, 9, 10, 15, },
  { 2, 9, 10, 15, },
  { 0, 2, 9, 10, 15, },
  { 1, 2, 9, 10, 15, },
  { 0, 1, 2, 9, 10, 15, },
  { 3, 9, 10, 15, },
  { 0, 3, 9, 10, 15, },
  { 1, 3, 9, 10, 15, },
  { 0, 1, 3, 9, 10, 15, },
  { 2, 3, 9, 10, 15, },
  { 0, 2, 3, 9, 10, 15, },
  { 1, 2, 3, 9, 10, 15, },
  { 0, 1, 2, 3, 9, 10, 15, },
  { 4, 9, 10, 15, },
  { 0, 4, 9, 10, 15, },
  { 1, 4, 9, 10, 15, },
  { 0, 1, 4, 9, 10, 15, },
  { 2, 4, 9, 10, 15, },
  { 0, 2, 4, 9, 10, 15, },
  { 1, 2, 4, 9, 10, 15, },
  { 0, 1, 2, 4, 9, 10, 15, },
  { 3, 4, 9, 10, 15, },
  { 0, 3, 4, 9, 10, 15, },
  { 1, 3, 4, 9, 10, 15, },
  { 0, 1, 3, 4, 9, 10, 15, },
  { 2, 3, 4, 9, 10, 15, },
  { 0, 2, 3, 4, 9, 10, 15, },
  { 1, 2, 3, 4, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 15, },
  { 5, 9, 10, 15, },
  { 0, 5, 9, 10, 15, },
  { 1, 5, 9, 10, 15, },
  { 0, 1, 5, 9, 10, 15, },
  { 2, 5, 9, 10, 15, },
  { 0, 2, 5, 9, 10, 15, },
  { 1, 2, 5, 9, 10, 15, },
  { 0, 1, 2, 5, 9, 10, 15, },
  { 3, 5, 9, 10, 15, },
  { 0, 3, 5, 9, 10, 15, },
  { 1, 3, 5, 9, 10, 15, },
  { 0, 1, 3, 5, 9, 10, 15, },
  { 2, 3, 5, 9, 10, 15, },
  { 0, 2, 3, 5, 9, 10, 15, },
  { 1, 2, 3, 5, 9, 10, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 15, },
  { 4, 5, 9, 10, 15, },
  { 0, 4, 5, 9, 10, 15, },
  { 1, 4, 5, 9, 10, 15, },
  { 0, 1, 4, 5, 9, 10, 15, },
  { 2, 4, 5, 9, 10, 15, },
  { 0, 2, 4, 5, 9, 10, 15, },
  { 1, 2, 4, 5, 9, 10, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 15, },
  { 3, 4, 5, 9, 10, 15, },
  { 0, 3, 4, 5, 9, 10, 15, },
  { 1, 3, 4, 5, 9, 10, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 15, },
  { 2, 3, 4, 5, 9, 10, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 15, },
  { 6, 9, 10, 15, },
  { 0, 6, 9, 10, 15, },
  { 1, 6, 9, 10, 15, },
  { 0, 1, 6, 9, 10, 15, },
  { 2, 6, 9, 10, 15, },
  { 0, 2, 6, 9, 10, 15, },
  { 1, 2, 6, 9, 10, 15, },
  { 0, 1, 2, 6, 9, 10, 15, },
  { 3, 6, 9, 10, 15, },
  { 0, 3, 6, 9, 10, 15, },
  { 1, 3, 6, 9, 10, 15, },
  { 0, 1, 3, 6, 9, 10, 15, },
  { 2, 3, 6, 9, 10, 15, },
  { 0, 2, 3, 6, 9, 10, 15, },
  { 1, 2, 3, 6, 9, 10, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 15, },
  { 4, 6, 9, 10, 15, },
  { 0, 4, 6, 9, 10, 15, },
  { 1, 4, 6, 9, 10, 15, },
  { 0, 1, 4, 6, 9, 10, 15, },
  { 2, 4, 6, 9, 10, 15, },
  { 0, 2, 4, 6, 9, 10, 15, },
  { 1, 2, 4, 6, 9, 10, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 15, },
  { 3, 4, 6, 9, 10, 15, },
  { 0, 3, 4, 6, 9, 10, 15, },
  { 1, 3, 4, 6, 9, 10, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 15, },
  { 2, 3, 4, 6, 9, 10, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 15, },
  { 5, 6, 9, 10, 15, },
  { 0, 5, 6, 9, 10, 15, },
  { 1, 5, 6, 9, 10, 15, },
  { 0, 1, 5, 6, 9, 10, 15, },
  { 2, 5, 6, 9, 10, 15, },
  { 0, 2, 5, 6, 9, 10, 15, },
  { 1, 2, 5, 6, 9, 10, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 15, },
  { 3, 5, 6, 9, 10, 15, },
  { 0, 3, 5, 6, 9, 10, 15, },
  { 1, 3, 5, 6, 9, 10, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 15, },
  { 2, 3, 5, 6, 9, 10, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 15, },
  { 4, 5, 6, 9, 10, 15, },
  { 0, 4, 5, 6, 9, 10, 15, },
  { 1, 4, 5, 6, 9, 10, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 15, },
  { 2, 4, 5, 6, 9, 10, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 15, },
  { 3, 4, 5, 6, 9, 10, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 15, },
  { 7, 9, 10, 15, },
  { 0, 7, 9, 10, 15, },
  { 1, 7, 9, 10, 15, },
  { 0, 1, 7, 9, 10, 15, },
  { 2, 7, 9, 10, 15, },
  { 0, 2, 7, 9, 10, 15, },
  { 1, 2, 7, 9, 10, 15, },
  { 0, 1, 2, 7, 9, 10, 15, },
  { 3, 7, 9, 10, 15, },
  { 0, 3, 7, 9, 10, 15, },
  { 1, 3, 7, 9, 10, 15, },
  { 0, 1, 3, 7, 9, 10, 15, },
  { 2, 3, 7, 9, 10, 15, },
  { 0, 2, 3, 7, 9, 10, 15, },
  { 1, 2, 3, 7, 9, 10, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 15, },
  { 4, 7, 9, 10, 15, },
  { 0, 4, 7, 9, 10, 15, },
  { 1, 4, 7, 9, 10, 15, },
  { 0, 1, 4, 7, 9, 10, 15, },
  { 2, 4, 7, 9, 10, 15, },
  { 0, 2, 4, 7, 9, 10, 15, },
  { 1, 2, 4, 7, 9, 10, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 15, },
  { 3, 4, 7, 9, 10, 15, },
  { 0, 3, 4, 7, 9, 10, 15, },
  { 1, 3, 4, 7, 9, 10, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 15, },
  { 2, 3, 4, 7, 9, 10, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 15, },
  { 5, 7, 9, 10, 15, },
  { 0, 5, 7, 9, 10, 15, },
  { 1, 5, 7, 9, 10, 15, },
  { 0, 1, 5, 7, 9, 10, 15, },
  { 2, 5, 7, 9, 10, 15, },
  { 0, 2, 5, 7, 9, 10, 15, },
  { 1, 2, 5, 7, 9, 10, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 15, },
  { 3, 5, 7, 9, 10, 15, },
  { 0, 3, 5, 7, 9, 10, 15, },
  { 1, 3, 5, 7, 9, 10, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 15, },
  { 2, 3, 5, 7, 9, 10, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 15, },
  { 4, 5, 7, 9, 10, 15, },
  { 0, 4, 5, 7, 9, 10, 15, },
  { 1, 4, 5, 7, 9, 10, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 15, },
  { 2, 4, 5, 7, 9, 10, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 15, },
  { 3, 4, 5, 7, 9, 10, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 15, },
  { 6, 7, 9, 10, 15, },
  { 0, 6, 7, 9, 10, 15, },
  { 1, 6, 7, 9, 10, 15, },
  { 0, 1, 6, 7, 9, 10, 15, },
  { 2, 6, 7, 9, 10, 15, },
  { 0, 2, 6, 7, 9, 10, 15, },
  { 1, 2, 6, 7, 9, 10, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 15, },
  { 3, 6, 7, 9, 10, 15, },
  { 0, 3, 6, 7, 9, 10, 15, },
  { 1, 3, 6, 7, 9, 10, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 15, },
  { 2, 3, 6, 7, 9, 10, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 15, },
  { 4, 6, 7, 9, 10, 15, },
  { 0, 4, 6, 7, 9, 10, 15, },
  { 1, 4, 6, 7, 9, 10, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 15, },
  { 2, 4, 6, 7, 9, 10, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 15, },
  { 3, 4, 6, 7, 9, 10, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 15, },
  { 5, 6, 7, 9, 10, 15, },
  { 0, 5, 6, 7, 9, 10, 15, },
  { 1, 5, 6, 7, 9, 10, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 15, },
  { 2, 5, 6, 7, 9, 10, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 15, },
  { 3, 5, 6, 7, 9, 10, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 15, },
  { 4, 5, 6, 7, 9, 10, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 15, },
  { 8, 9, 10, 15, },
  { 0, 8, 9, 10, 15, },
  { 1, 8, 9, 10, 15, },
  { 0, 1, 8, 9, 10, 15, },
  { 2, 8, 9, 10, 15, },
  { 0, 2, 8, 9, 10, 15, },
  { 1, 2, 8, 9, 10, 15, },
  { 0, 1, 2, 8, 9, 10, 15, },
  { 3, 8, 9, 10, 15, },
  { 0, 3, 8, 9, 10, 15, },
  { 1, 3, 8, 9, 10, 15, },
  { 0, 1, 3, 8, 9, 10, 15, },
  { 2, 3, 8, 9, 10, 15, },
  { 0, 2, 3, 8, 9, 10, 15, },
  { 1, 2, 3, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 15, },
  { 4, 8, 9, 10, 15, },
  { 0, 4, 8, 9, 10, 15, },
  { 1, 4, 8, 9, 10, 15, },
  { 0, 1, 4, 8, 9, 10, 15, },
  { 2, 4, 8, 9, 10, 15, },
  { 0, 2, 4, 8, 9, 10, 15, },
  { 1, 2, 4, 8, 9, 10, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 15, },
  { 3, 4, 8, 9, 10, 15, },
  { 0, 3, 4, 8, 9, 10, 15, },
  { 1, 3, 4, 8, 9, 10, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 15, },
  { 2, 3, 4, 8, 9, 10, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 15, },
  { 5, 8, 9, 10, 15, },
  { 0, 5, 8, 9, 10, 15, },
  { 1, 5, 8, 9, 10, 15, },
  { 0, 1, 5, 8, 9, 10, 15, },
  { 2, 5, 8, 9, 10, 15, },
  { 0, 2, 5, 8, 9, 10, 15, },
  { 1, 2, 5, 8, 9, 10, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 15, },
  { 3, 5, 8, 9, 10, 15, },
  { 0, 3, 5, 8, 9, 10, 15, },
  { 1, 3, 5, 8, 9, 10, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 15, },
  { 2, 3, 5, 8, 9, 10, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 15, },
  { 4, 5, 8, 9, 10, 15, },
  { 0, 4, 5, 8, 9, 10, 15, },
  { 1, 4, 5, 8, 9, 10, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 15, },
  { 2, 4, 5, 8, 9, 10, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 15, },
  { 3, 4, 5, 8, 9, 10, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 15, },
  { 6, 8, 9, 10, 15, },
  { 0, 6, 8, 9, 10, 15, },
  { 1, 6, 8, 9, 10, 15, },
  { 0, 1, 6, 8, 9, 10, 15, },
  { 2, 6, 8, 9, 10, 15, },
  { 0, 2, 6, 8, 9, 10, 15, },
  { 1, 2, 6, 8, 9, 10, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 15, },
  { 3, 6, 8, 9, 10, 15, },
  { 0, 3, 6, 8, 9, 10, 15, },
  { 1, 3, 6, 8, 9, 10, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 15, },
  { 2, 3, 6, 8, 9, 10, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 15, },
  { 4, 6, 8, 9, 10, 15, },
  { 0, 4, 6, 8, 9, 10, 15, },
  { 1, 4, 6, 8, 9, 10, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 15, },
  { 2, 4, 6, 8, 9, 10, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 15, },
  { 3, 4, 6, 8, 9, 10, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 15, },
  { 5, 6, 8, 9, 10, 15, },
  { 0, 5, 6, 8, 9, 10, 15, },
  { 1, 5, 6, 8, 9, 10, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 15, },
  { 2, 5, 6, 8, 9, 10, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 15, },
  { 3, 5, 6, 8, 9, 10, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 15, },
  { 4, 5, 6, 8, 9, 10, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 15, },
  { 7, 8, 9, 10, 15, },
  { 0, 7, 8, 9, 10, 15, },
  { 1, 7, 8, 9, 10, 15, },
  { 0, 1, 7, 8, 9, 10, 15, },
  { 2, 7, 8, 9, 10, 15, },
  { 0, 2, 7, 8, 9, 10, 15, },
  { 1, 2, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 15, },
  { 3, 7, 8, 9, 10, 15, },
  { 0, 3, 7, 8, 9, 10, 15, },
  { 1, 3, 7, 8, 9, 10, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 15, },
  { 2, 3, 7, 8, 9, 10, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 15, },
  { 4, 7, 8, 9, 10, 15, },
  { 0, 4, 7, 8, 9, 10, 15, },
  { 1, 4, 7, 8, 9, 10, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 15, },
  { 2, 4, 7, 8, 9, 10, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 15, },
  { 3, 4, 7, 8, 9, 10, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 15, },
  { 5, 7, 8, 9, 10, 15, },
  { 0, 5, 7, 8, 9, 10, 15, },
  { 1, 5, 7, 8, 9, 10, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 15, },
  { 2, 5, 7, 8, 9, 10, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 15, },
  { 3, 5, 7, 8, 9, 10, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 15, },
  { 4, 5, 7, 8, 9, 10, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 15, },
  { 6, 7, 8, 9, 10, 15, },
  { 0, 6, 7, 8, 9, 10, 15, },
  { 1, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 15, },
  { 2, 6, 7, 8, 9, 10, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 15, },
  { 3, 6, 7, 8, 9, 10, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 15, },
  { 4, 6, 7, 8, 9, 10, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 15, },
  { 5, 6, 7, 8, 9, 10, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, },
  { 11, 15, },
  { 0, 11, 15, },
  { 1, 11, 15, },
  { 0, 1, 11, 15, },
  { 2, 11, 15, },
  { 0, 2, 11, 15, },
  { 1, 2, 11, 15, },
  { 0, 1, 2, 11, 15, },
  { 3, 11, 15, },
  { 0, 3, 11, 15, },
  { 1, 3, 11, 15, },
  { 0, 1, 3, 11, 15, },
  { 2, 3, 11, 15, },
  { 0, 2, 3, 11, 15, },
  { 1, 2, 3, 11, 15, },
  { 0, 1, 2, 3, 11, 15, },
  { 4, 11, 15, },
  { 0, 4, 11, 15, },
  { 1, 4, 11, 15, },
  { 0, 1, 4, 11, 15, },
  { 2, 4, 11, 15, },
  { 0, 2, 4, 11, 15, },
  { 1, 2, 4, 11, 15, },
  { 0, 1, 2, 4, 11, 15, },
  { 3, 4, 11, 15, },
  { 0, 3, 4, 11, 15, },
  { 1, 3, 4, 11, 15, },
  { 0, 1, 3, 4, 11, 15, },
  { 2, 3, 4, 11, 15, },
  { 0, 2, 3, 4, 11, 15, },
  { 1, 2, 3, 4, 11, 15, },
  { 0, 1, 2, 3, 4, 11, 15, },
  { 5, 11, 15, },
  { 0, 5, 11, 15, },
  { 1, 5, 11, 15, },
  { 0, 1, 5, 11, 15, },
  { 2, 5, 11, 15, },
  { 0, 2, 5, 11, 15, },
  { 1, 2, 5, 11, 15, },
  { 0, 1, 2, 5, 11, 15, },
  { 3, 5, 11, 15, },
  { 0, 3, 5, 11, 15, },
  { 1, 3, 5, 11, 15, },
  { 0, 1, 3, 5, 11, 15, },
  { 2, 3, 5, 11, 15, },
  { 0, 2, 3, 5, 11, 15, },
  { 1, 2, 3, 5, 11, 15, },
  { 0, 1, 2, 3, 5, 11, 15, },
  { 4, 5, 11, 15, },
  { 0, 4, 5, 11, 15, },
  { 1, 4, 5, 11, 15, },
  { 0, 1, 4, 5, 11, 15, },
  { 2, 4, 5, 11, 15, },
  { 0, 2, 4, 5, 11, 15, },
  { 1, 2, 4, 5, 11, 15, },
  { 0, 1, 2, 4, 5, 11, 15, },
  { 3, 4, 5, 11, 15, },
  { 0, 3, 4, 5, 11, 15, },
  { 1, 3, 4, 5, 11, 15, },
  { 0, 1, 3, 4, 5, 11, 15, },
  { 2, 3, 4, 5, 11, 15, },
  { 0, 2, 3, 4, 5, 11, 15, },
  { 1, 2, 3, 4, 5, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 11, 15, },
  { 6, 11, 15, },
  { 0, 6, 11, 15, },
  { 1, 6, 11, 15, },
  { 0, 1, 6, 11, 15, },
  { 2, 6, 11, 15, },
  { 0, 2, 6, 11, 15, },
  { 1, 2, 6, 11, 15, },
  { 0, 1, 2, 6, 11, 15, },
  { 3, 6, 11, 15, },
  { 0, 3, 6, 11, 15, },
  { 1, 3, 6, 11, 15, },
  { 0, 1, 3, 6, 11, 15, },
  { 2, 3, 6, 11, 15, },
  { 0, 2, 3, 6, 11, 15, },
  { 1, 2, 3, 6, 11, 15, },
  { 0, 1, 2, 3, 6, 11, 15, },
  { 4, 6, 11, 15, },
  { 0, 4, 6, 11, 15, },
  { 1, 4, 6, 11, 15, },
  { 0, 1, 4, 6, 11, 15, },
  { 2, 4, 6, 11, 15, },
  { 0, 2, 4, 6, 11, 15, },
  { 1, 2, 4, 6, 11, 15, },
  { 0, 1, 2, 4, 6, 11, 15, },
  { 3, 4, 6, 11, 15, },
  { 0, 3, 4, 6, 11, 15, },
  { 1, 3, 4, 6, 11, 15, },
  { 0, 1, 3, 4, 6, 11, 15, },
  { 2, 3, 4, 6, 11, 15, },
  { 0, 2, 3, 4, 6, 11, 15, },
  { 1, 2, 3, 4, 6, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 11, 15, },
  { 5, 6, 11, 15, },
  { 0, 5, 6, 11, 15, },
  { 1, 5, 6, 11, 15, },
  { 0, 1, 5, 6, 11, 15, },
  { 2, 5, 6, 11, 15, },
  { 0, 2, 5, 6, 11, 15, },
  { 1, 2, 5, 6, 11, 15, },
  { 0, 1, 2, 5, 6, 11, 15, },
  { 3, 5, 6, 11, 15, },
  { 0, 3, 5, 6, 11, 15, },
  { 1, 3, 5, 6, 11, 15, },
  { 0, 1, 3, 5, 6, 11, 15, },
  { 2, 3, 5, 6, 11, 15, },
  { 0, 2, 3, 5, 6, 11, 15, },
  { 1, 2, 3, 5, 6, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 11, 15, },
  { 4, 5, 6, 11, 15, },
  { 0, 4, 5, 6, 11, 15, },
  { 1, 4, 5, 6, 11, 15, },
  { 0, 1, 4, 5, 6, 11, 15, },
  { 2, 4, 5, 6, 11, 15, },
  { 0, 2, 4, 5, 6, 11, 15, },
  { 1, 2, 4, 5, 6, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 11, 15, },
  { 3, 4, 5, 6, 11, 15, },
  { 0, 3, 4, 5, 6, 11, 15, },
  { 1, 3, 4, 5, 6, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 11, 15, },
  { 2, 3, 4, 5, 6, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 15, },
  { 7, 11, 15, },
  { 0, 7, 11, 15, },
  { 1, 7, 11, 15, },
  { 0, 1, 7, 11, 15, },
  { 2, 7, 11, 15, },
  { 0, 2, 7, 11, 15, },
  { 1, 2, 7, 11, 15, },
  { 0, 1, 2, 7, 11, 15, },
  { 3, 7, 11, 15, },
  { 0, 3, 7, 11, 15, },
  { 1, 3, 7, 11, 15, },
  { 0, 1, 3, 7, 11, 15, },
  { 2, 3, 7, 11, 15, },
  { 0, 2, 3, 7, 11, 15, },
  { 1, 2, 3, 7, 11, 15, },
  { 0, 1, 2, 3, 7, 11, 15, },
  { 4, 7, 11, 15, },
  { 0, 4, 7, 11, 15, },
  { 1, 4, 7, 11, 15, },
  { 0, 1, 4, 7, 11, 15, },
  { 2, 4, 7, 11, 15, },
  { 0, 2, 4, 7, 11, 15, },
  { 1, 2, 4, 7, 11, 15, },
  { 0, 1, 2, 4, 7, 11, 15, },
  { 3, 4, 7, 11, 15, },
  { 0, 3, 4, 7, 11, 15, },
  { 1, 3, 4, 7, 11, 15, },
  { 0, 1, 3, 4, 7, 11, 15, },
  { 2, 3, 4, 7, 11, 15, },
  { 0, 2, 3, 4, 7, 11, 15, },
  { 1, 2, 3, 4, 7, 11, 15, },
  { 0, 1, 2, 3, 4, 7, 11, 15, },
  { 5, 7, 11, 15, },
  { 0, 5, 7, 11, 15, },
  { 1, 5, 7, 11, 15, },
  { 0, 1, 5, 7, 11, 15, },
  { 2, 5, 7, 11, 15, },
  { 0, 2, 5, 7, 11, 15, },
  { 1, 2, 5, 7, 11, 15, },
  { 0, 1, 2, 5, 7, 11, 15, },
  { 3, 5, 7, 11, 15, },
  { 0, 3, 5, 7, 11, 15, },
  { 1, 3, 5, 7, 11, 15, },
  { 0, 1, 3, 5, 7, 11, 15, },
  { 2, 3, 5, 7, 11, 15, },
  { 0, 2, 3, 5, 7, 11, 15, },
  { 1, 2, 3, 5, 7, 11, 15, },
  { 0, 1, 2, 3, 5, 7, 11, 15, },
  { 4, 5, 7, 11, 15, },
  { 0, 4, 5, 7, 11, 15, },
  { 1, 4, 5, 7, 11, 15, },
  { 0, 1, 4, 5, 7, 11, 15, },
  { 2, 4, 5, 7, 11, 15, },
  { 0, 2, 4, 5, 7, 11, 15, },
  { 1, 2, 4, 5, 7, 11, 15, },
  { 0, 1, 2, 4, 5, 7, 11, 15, },
  { 3, 4, 5, 7, 11, 15, },
  { 0, 3, 4, 5, 7, 11, 15, },
  { 1, 3, 4, 5, 7, 11, 15, },
  { 0, 1, 3, 4, 5, 7, 11, 15, },
  { 2, 3, 4, 5, 7, 11, 15, },
  { 0, 2, 3, 4, 5, 7, 11, 15, },
  { 1, 2, 3, 4, 5, 7, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 15, },
  { 6, 7, 11, 15, },
  { 0, 6, 7, 11, 15, },
  { 1, 6, 7, 11, 15, },
  { 0, 1, 6, 7, 11, 15, },
  { 2, 6, 7, 11, 15, },
  { 0, 2, 6, 7, 11, 15, },
  { 1, 2, 6, 7, 11, 15, },
  { 0, 1, 2, 6, 7, 11, 15, },
  { 3, 6, 7, 11, 15, },
  { 0, 3, 6, 7, 11, 15, },
  { 1, 3, 6, 7, 11, 15, },
  { 0, 1, 3, 6, 7, 11, 15, },
  { 2, 3, 6, 7, 11, 15, },
  { 0, 2, 3, 6, 7, 11, 15, },
  { 1, 2, 3, 6, 7, 11, 15, },
  { 0, 1, 2, 3, 6, 7, 11, 15, },
  { 4, 6, 7, 11, 15, },
  { 0, 4, 6, 7, 11, 15, },
  { 1, 4, 6, 7, 11, 15, },
  { 0, 1, 4, 6, 7, 11, 15, },
  { 2, 4, 6, 7, 11, 15, },
  { 0, 2, 4, 6, 7, 11, 15, },
  { 1, 2, 4, 6, 7, 11, 15, },
  { 0, 1, 2, 4, 6, 7, 11, 15, },
  { 3, 4, 6, 7, 11, 15, },
  { 0, 3, 4, 6, 7, 11, 15, },
  { 1, 3, 4, 6, 7, 11, 15, },
  { 0, 1, 3, 4, 6, 7, 11, 15, },
  { 2, 3, 4, 6, 7, 11, 15, },
  { 0, 2, 3, 4, 6, 7, 11, 15, },
  { 1, 2, 3, 4, 6, 7, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 15, },
  { 5, 6, 7, 11, 15, },
  { 0, 5, 6, 7, 11, 15, },
  { 1, 5, 6, 7, 11, 15, },
  { 0, 1, 5, 6, 7, 11, 15, },
  { 2, 5, 6, 7, 11, 15, },
  { 0, 2, 5, 6, 7, 11, 15, },
  { 1, 2, 5, 6, 7, 11, 15, },
  { 0, 1, 2, 5, 6, 7, 11, 15, },
  { 3, 5, 6, 7, 11, 15, },
  { 0, 3, 5, 6, 7, 11, 15, },
  { 1, 3, 5, 6, 7, 11, 15, },
  { 0, 1, 3, 5, 6, 7, 11, 15, },
  { 2, 3, 5, 6, 7, 11, 15, },
  { 0, 2, 3, 5, 6, 7, 11, 15, },
  { 1, 2, 3, 5, 6, 7, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 15, },
  { 4, 5, 6, 7, 11, 15, },
  { 0, 4, 5, 6, 7, 11, 15, },
  { 1, 4, 5, 6, 7, 11, 15, },
  { 0, 1, 4, 5, 6, 7, 11, 15, },
  { 2, 4, 5, 6, 7, 11, 15, },
  { 0, 2, 4, 5, 6, 7, 11, 15, },
  { 1, 2, 4, 5, 6, 7, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 15, },
  { 3, 4, 5, 6, 7, 11, 15, },
  { 0, 3, 4, 5, 6, 7, 11, 15, },
  { 1, 3, 4, 5, 6, 7, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 15, },
  { 2, 3, 4, 5, 6, 7, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 15, },
  { 8, 11, 15, },
  { 0, 8, 11, 15, },
  { 1, 8, 11, 15, },
  { 0, 1, 8, 11, 15, },
  { 2, 8, 11, 15, },
  { 0, 2, 8, 11, 15, },
  { 1, 2, 8, 11, 15, },
  { 0, 1, 2, 8, 11, 15, },
  { 3, 8, 11, 15, },
  { 0, 3, 8, 11, 15, },
  { 1, 3, 8, 11, 15, },
  { 0, 1, 3, 8, 11, 15, },
  { 2, 3, 8, 11, 15, },
  { 0, 2, 3, 8, 11, 15, },
  { 1, 2, 3, 8, 11, 15, },
  { 0, 1, 2, 3, 8, 11, 15, },
  { 4, 8, 11, 15, },
  { 0, 4, 8, 11, 15, },
  { 1, 4, 8, 11, 15, },
  { 0, 1, 4, 8, 11, 15, },
  { 2, 4, 8, 11, 15, },
  { 0, 2, 4, 8, 11, 15, },
  { 1, 2, 4, 8, 11, 15, },
  { 0, 1, 2, 4, 8, 11, 15, },
  { 3, 4, 8, 11, 15, },
  { 0, 3, 4, 8, 11, 15, },
  { 1, 3, 4, 8, 11, 15, },
  { 0, 1, 3, 4, 8, 11, 15, },
  { 2, 3, 4, 8, 11, 15, },
  { 0, 2, 3, 4, 8, 11, 15, },
  { 1, 2, 3, 4, 8, 11, 15, },
  { 0, 1, 2, 3, 4, 8, 11, 15, },
  { 5, 8, 11, 15, },
  { 0, 5, 8, 11, 15, },
  { 1, 5, 8, 11, 15, },
  { 0, 1, 5, 8, 11, 15, },
  { 2, 5, 8, 11, 15, },
  { 0, 2, 5, 8, 11, 15, },
  { 1, 2, 5, 8, 11, 15, },
  { 0, 1, 2, 5, 8, 11, 15, },
  { 3, 5, 8, 11, 15, },
  { 0, 3, 5, 8, 11, 15, },
  { 1, 3, 5, 8, 11, 15, },
  { 0, 1, 3, 5, 8, 11, 15, },
  { 2, 3, 5, 8, 11, 15, },
  { 0, 2, 3, 5, 8, 11, 15, },
  { 1, 2, 3, 5, 8, 11, 15, },
  { 0, 1, 2, 3, 5, 8, 11, 15, },
  { 4, 5, 8, 11, 15, },
  { 0, 4, 5, 8, 11, 15, },
  { 1, 4, 5, 8, 11, 15, },
  { 0, 1, 4, 5, 8, 11, 15, },
  { 2, 4, 5, 8, 11, 15, },
  { 0, 2, 4, 5, 8, 11, 15, },
  { 1, 2, 4, 5, 8, 11, 15, },
  { 0, 1, 2, 4, 5, 8, 11, 15, },
  { 3, 4, 5, 8, 11, 15, },
  { 0, 3, 4, 5, 8, 11, 15, },
  { 1, 3, 4, 5, 8, 11, 15, },
  { 0, 1, 3, 4, 5, 8, 11, 15, },
  { 2, 3, 4, 5, 8, 11, 15, },
  { 0, 2, 3, 4, 5, 8, 11, 15, },
  { 1, 2, 3, 4, 5, 8, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 15, },
  { 6, 8, 11, 15, },
  { 0, 6, 8, 11, 15, },
  { 1, 6, 8, 11, 15, },
  { 0, 1, 6, 8, 11, 15, },
  { 2, 6, 8, 11, 15, },
  { 0, 2, 6, 8, 11, 15, },
  { 1, 2, 6, 8, 11, 15, },
  { 0, 1, 2, 6, 8, 11, 15, },
  { 3, 6, 8, 11, 15, },
  { 0, 3, 6, 8, 11, 15, },
  { 1, 3, 6, 8, 11, 15, },
  { 0, 1, 3, 6, 8, 11, 15, },
  { 2, 3, 6, 8, 11, 15, },
  { 0, 2, 3, 6, 8, 11, 15, },
  { 1, 2, 3, 6, 8, 11, 15, },
  { 0, 1, 2, 3, 6, 8, 11, 15, },
  { 4, 6, 8, 11, 15, },
  { 0, 4, 6, 8, 11, 15, },
  { 1, 4, 6, 8, 11, 15, },
  { 0, 1, 4, 6, 8, 11, 15, },
  { 2, 4, 6, 8, 11, 15, },
  { 0, 2, 4, 6, 8, 11, 15, },
  { 1, 2, 4, 6, 8, 11, 15, },
  { 0, 1, 2, 4, 6, 8, 11, 15, },
  { 3, 4, 6, 8, 11, 15, },
  { 0, 3, 4, 6, 8, 11, 15, },
  { 1, 3, 4, 6, 8, 11, 15, },
  { 0, 1, 3, 4, 6, 8, 11, 15, },
  { 2, 3, 4, 6, 8, 11, 15, },
  { 0, 2, 3, 4, 6, 8, 11, 15, },
  { 1, 2, 3, 4, 6, 8, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 15, },
  { 5, 6, 8, 11, 15, },
  { 0, 5, 6, 8, 11, 15, },
  { 1, 5, 6, 8, 11, 15, },
  { 0, 1, 5, 6, 8, 11, 15, },
  { 2, 5, 6, 8, 11, 15, },
  { 0, 2, 5, 6, 8, 11, 15, },
  { 1, 2, 5, 6, 8, 11, 15, },
  { 0, 1, 2, 5, 6, 8, 11, 15, },
  { 3, 5, 6, 8, 11, 15, },
  { 0, 3, 5, 6, 8, 11, 15, },
  { 1, 3, 5, 6, 8, 11, 15, },
  { 0, 1, 3, 5, 6, 8, 11, 15, },
  { 2, 3, 5, 6, 8, 11, 15, },
  { 0, 2, 3, 5, 6, 8, 11, 15, },
  { 1, 2, 3, 5, 6, 8, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 15, },
  { 4, 5, 6, 8, 11, 15, },
  { 0, 4, 5, 6, 8, 11, 15, },
  { 1, 4, 5, 6, 8, 11, 15, },
  { 0, 1, 4, 5, 6, 8, 11, 15, },
  { 2, 4, 5, 6, 8, 11, 15, },
  { 0, 2, 4, 5, 6, 8, 11, 15, },
  { 1, 2, 4, 5, 6, 8, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 15, },
  { 3, 4, 5, 6, 8, 11, 15, },
  { 0, 3, 4, 5, 6, 8, 11, 15, },
  { 1, 3, 4, 5, 6, 8, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 15, },
  { 2, 3, 4, 5, 6, 8, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 15, },
  { 7, 8, 11, 15, },
  { 0, 7, 8, 11, 15, },
  { 1, 7, 8, 11, 15, },
  { 0, 1, 7, 8, 11, 15, },
  { 2, 7, 8, 11, 15, },
  { 0, 2, 7, 8, 11, 15, },
  { 1, 2, 7, 8, 11, 15, },
  { 0, 1, 2, 7, 8, 11, 15, },
  { 3, 7, 8, 11, 15, },
  { 0, 3, 7, 8, 11, 15, },
  { 1, 3, 7, 8, 11, 15, },
  { 0, 1, 3, 7, 8, 11, 15, },
  { 2, 3, 7, 8, 11, 15, },
  { 0, 2, 3, 7, 8, 11, 15, },
  { 1, 2, 3, 7, 8, 11, 15, },
  { 0, 1, 2, 3, 7, 8, 11, 15, },
  { 4, 7, 8, 11, 15, },
  { 0, 4, 7, 8, 11, 15, },
  { 1, 4, 7, 8, 11, 15, },
  { 0, 1, 4, 7, 8, 11, 15, },
  { 2, 4, 7, 8, 11, 15, },
  { 0, 2, 4, 7, 8, 11, 15, },
  { 1, 2, 4, 7, 8, 11, 15, },
  { 0, 1, 2, 4, 7, 8, 11, 15, },
  { 3, 4, 7, 8, 11, 15, },
  { 0, 3, 4, 7, 8, 11, 15, },
  { 1, 3, 4, 7, 8, 11, 15, },
  { 0, 1, 3, 4, 7, 8, 11, 15, },
  { 2, 3, 4, 7, 8, 11, 15, },
  { 0, 2, 3, 4, 7, 8, 11, 15, },
  { 1, 2, 3, 4, 7, 8, 11, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 15, },
  { 5, 7, 8, 11, 15, },
  { 0, 5, 7, 8, 11, 15, },
  { 1, 5, 7, 8, 11, 15, },
  { 0, 1, 5, 7, 8, 11, 15, },
  { 2, 5, 7, 8, 11, 15, },
  { 0, 2, 5, 7, 8, 11, 15, },
  { 1, 2, 5, 7, 8, 11, 15, },
  { 0, 1, 2, 5, 7, 8, 11, 15, },
  { 3, 5, 7, 8, 11, 15, },
  { 0, 3, 5, 7, 8, 11, 15, },
  { 1, 3, 5, 7, 8, 11, 15, },
  { 0, 1, 3, 5, 7, 8, 11, 15, },
  { 2, 3, 5, 7, 8, 11, 15, },
  { 0, 2, 3, 5, 7, 8, 11, 15, },
  { 1, 2, 3, 5, 7, 8, 11, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 15, },
  { 4, 5, 7, 8, 11, 15, },
  { 0, 4, 5, 7, 8, 11, 15, },
  { 1, 4, 5, 7, 8, 11, 15, },
  { 0, 1, 4, 5, 7, 8, 11, 15, },
  { 2, 4, 5, 7, 8, 11, 15, },
  { 0, 2, 4, 5, 7, 8, 11, 15, },
  { 1, 2, 4, 5, 7, 8, 11, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 15, },
  { 3, 4, 5, 7, 8, 11, 15, },
  { 0, 3, 4, 5, 7, 8, 11, 15, },
  { 1, 3, 4, 5, 7, 8, 11, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 15, },
  { 2, 3, 4, 5, 7, 8, 11, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 15, },
  { 6, 7, 8, 11, 15, },
  { 0, 6, 7, 8, 11, 15, },
  { 1, 6, 7, 8, 11, 15, },
  { 0, 1, 6, 7, 8, 11, 15, },
  { 2, 6, 7, 8, 11, 15, },
  { 0, 2, 6, 7, 8, 11, 15, },
  { 1, 2, 6, 7, 8, 11, 15, },
  { 0, 1, 2, 6, 7, 8, 11, 15, },
  { 3, 6, 7, 8, 11, 15, },
  { 0, 3, 6, 7, 8, 11, 15, },
  { 1, 3, 6, 7, 8, 11, 15, },
  { 0, 1, 3, 6, 7, 8, 11, 15, },
  { 2, 3, 6, 7, 8, 11, 15, },
  { 0, 2, 3, 6, 7, 8, 11, 15, },
  { 1, 2, 3, 6, 7, 8, 11, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 15, },
  { 4, 6, 7, 8, 11, 15, },
  { 0, 4, 6, 7, 8, 11, 15, },
  { 1, 4, 6, 7, 8, 11, 15, },
  { 0, 1, 4, 6, 7, 8, 11, 15, },
  { 2, 4, 6, 7, 8, 11, 15, },
  { 0, 2, 4, 6, 7, 8, 11, 15, },
  { 1, 2, 4, 6, 7, 8, 11, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 15, },
  { 3, 4, 6, 7, 8, 11, 15, },
  { 0, 3, 4, 6, 7, 8, 11, 15, },
  { 1, 3, 4, 6, 7, 8, 11, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 15, },
  { 2, 3, 4, 6, 7, 8, 11, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 15, },
  { 5, 6, 7, 8, 11, 15, },
  { 0, 5, 6, 7, 8, 11, 15, },
  { 1, 5, 6, 7, 8, 11, 15, },
  { 0, 1, 5, 6, 7, 8, 11, 15, },
  { 2, 5, 6, 7, 8, 11, 15, },
  { 0, 2, 5, 6, 7, 8, 11, 15, },
  { 1, 2, 5, 6, 7, 8, 11, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 15, },
  { 3, 5, 6, 7, 8, 11, 15, },
  { 0, 3, 5, 6, 7, 8, 11, 15, },
  { 1, 3, 5, 6, 7, 8, 11, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 15, },
  { 2, 3, 5, 6, 7, 8, 11, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 15, },
  { 4, 5, 6, 7, 8, 11, 15, },
  { 0, 4, 5, 6, 7, 8, 11, 15, },
  { 1, 4, 5, 6, 7, 8, 11, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 15, },
  { 2, 4, 5, 6, 7, 8, 11, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 15, },
  { 3, 4, 5, 6, 7, 8, 11, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 15, },
  { 9, 11, 15, },
  { 0, 9, 11, 15, },
  { 1, 9, 11, 15, },
  { 0, 1, 9, 11, 15, },
  { 2, 9, 11, 15, },
  { 0, 2, 9, 11, 15, },
  { 1, 2, 9, 11, 15, },
  { 0, 1, 2, 9, 11, 15, },
  { 3, 9, 11, 15, },
  { 0, 3, 9, 11, 15, },
  { 1, 3, 9, 11, 15, },
  { 0, 1, 3, 9, 11, 15, },
  { 2, 3, 9, 11, 15, },
  { 0, 2, 3, 9, 11, 15, },
  { 1, 2, 3, 9, 11, 15, },
  { 0, 1, 2, 3, 9, 11, 15, },
  { 4, 9, 11, 15, },
  { 0, 4, 9, 11, 15, },
  { 1, 4, 9, 11, 15, },
  { 0, 1, 4, 9, 11, 15, },
  { 2, 4, 9, 11, 15, },
  { 0, 2, 4, 9, 11, 15, },
  { 1, 2, 4, 9, 11, 15, },
  { 0, 1, 2, 4, 9, 11, 15, },
  { 3, 4, 9, 11, 15, },
  { 0, 3, 4, 9, 11, 15, },
  { 1, 3, 4, 9, 11, 15, },
  { 0, 1, 3, 4, 9, 11, 15, },
  { 2, 3, 4, 9, 11, 15, },
  { 0, 2, 3, 4, 9, 11, 15, },
  { 1, 2, 3, 4, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 9, 11, 15, },
  { 5, 9, 11, 15, },
  { 0, 5, 9, 11, 15, },
  { 1, 5, 9, 11, 15, },
  { 0, 1, 5, 9, 11, 15, },
  { 2, 5, 9, 11, 15, },
  { 0, 2, 5, 9, 11, 15, },
  { 1, 2, 5, 9, 11, 15, },
  { 0, 1, 2, 5, 9, 11, 15, },
  { 3, 5, 9, 11, 15, },
  { 0, 3, 5, 9, 11, 15, },
  { 1, 3, 5, 9, 11, 15, },
  { 0, 1, 3, 5, 9, 11, 15, },
  { 2, 3, 5, 9, 11, 15, },
  { 0, 2, 3, 5, 9, 11, 15, },
  { 1, 2, 3, 5, 9, 11, 15, },
  { 0, 1, 2, 3, 5, 9, 11, 15, },
  { 4, 5, 9, 11, 15, },
  { 0, 4, 5, 9, 11, 15, },
  { 1, 4, 5, 9, 11, 15, },
  { 0, 1, 4, 5, 9, 11, 15, },
  { 2, 4, 5, 9, 11, 15, },
  { 0, 2, 4, 5, 9, 11, 15, },
  { 1, 2, 4, 5, 9, 11, 15, },
  { 0, 1, 2, 4, 5, 9, 11, 15, },
  { 3, 4, 5, 9, 11, 15, },
  { 0, 3, 4, 5, 9, 11, 15, },
  { 1, 3, 4, 5, 9, 11, 15, },
  { 0, 1, 3, 4, 5, 9, 11, 15, },
  { 2, 3, 4, 5, 9, 11, 15, },
  { 0, 2, 3, 4, 5, 9, 11, 15, },
  { 1, 2, 3, 4, 5, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 15, },
  { 6, 9, 11, 15, },
  { 0, 6, 9, 11, 15, },
  { 1, 6, 9, 11, 15, },
  { 0, 1, 6, 9, 11, 15, },
  { 2, 6, 9, 11, 15, },
  { 0, 2, 6, 9, 11, 15, },
  { 1, 2, 6, 9, 11, 15, },
  { 0, 1, 2, 6, 9, 11, 15, },
  { 3, 6, 9, 11, 15, },
  { 0, 3, 6, 9, 11, 15, },
  { 1, 3, 6, 9, 11, 15, },
  { 0, 1, 3, 6, 9, 11, 15, },
  { 2, 3, 6, 9, 11, 15, },
  { 0, 2, 3, 6, 9, 11, 15, },
  { 1, 2, 3, 6, 9, 11, 15, },
  { 0, 1, 2, 3, 6, 9, 11, 15, },
  { 4, 6, 9, 11, 15, },
  { 0, 4, 6, 9, 11, 15, },
  { 1, 4, 6, 9, 11, 15, },
  { 0, 1, 4, 6, 9, 11, 15, },
  { 2, 4, 6, 9, 11, 15, },
  { 0, 2, 4, 6, 9, 11, 15, },
  { 1, 2, 4, 6, 9, 11, 15, },
  { 0, 1, 2, 4, 6, 9, 11, 15, },
  { 3, 4, 6, 9, 11, 15, },
  { 0, 3, 4, 6, 9, 11, 15, },
  { 1, 3, 4, 6, 9, 11, 15, },
  { 0, 1, 3, 4, 6, 9, 11, 15, },
  { 2, 3, 4, 6, 9, 11, 15, },
  { 0, 2, 3, 4, 6, 9, 11, 15, },
  { 1, 2, 3, 4, 6, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 15, },
  { 5, 6, 9, 11, 15, },
  { 0, 5, 6, 9, 11, 15, },
  { 1, 5, 6, 9, 11, 15, },
  { 0, 1, 5, 6, 9, 11, 15, },
  { 2, 5, 6, 9, 11, 15, },
  { 0, 2, 5, 6, 9, 11, 15, },
  { 1, 2, 5, 6, 9, 11, 15, },
  { 0, 1, 2, 5, 6, 9, 11, 15, },
  { 3, 5, 6, 9, 11, 15, },
  { 0, 3, 5, 6, 9, 11, 15, },
  { 1, 3, 5, 6, 9, 11, 15, },
  { 0, 1, 3, 5, 6, 9, 11, 15, },
  { 2, 3, 5, 6, 9, 11, 15, },
  { 0, 2, 3, 5, 6, 9, 11, 15, },
  { 1, 2, 3, 5, 6, 9, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 15, },
  { 4, 5, 6, 9, 11, 15, },
  { 0, 4, 5, 6, 9, 11, 15, },
  { 1, 4, 5, 6, 9, 11, 15, },
  { 0, 1, 4, 5, 6, 9, 11, 15, },
  { 2, 4, 5, 6, 9, 11, 15, },
  { 0, 2, 4, 5, 6, 9, 11, 15, },
  { 1, 2, 4, 5, 6, 9, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 15, },
  { 3, 4, 5, 6, 9, 11, 15, },
  { 0, 3, 4, 5, 6, 9, 11, 15, },
  { 1, 3, 4, 5, 6, 9, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 15, },
  { 2, 3, 4, 5, 6, 9, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 15, },
  { 7, 9, 11, 15, },
  { 0, 7, 9, 11, 15, },
  { 1, 7, 9, 11, 15, },
  { 0, 1, 7, 9, 11, 15, },
  { 2, 7, 9, 11, 15, },
  { 0, 2, 7, 9, 11, 15, },
  { 1, 2, 7, 9, 11, 15, },
  { 0, 1, 2, 7, 9, 11, 15, },
  { 3, 7, 9, 11, 15, },
  { 0, 3, 7, 9, 11, 15, },
  { 1, 3, 7, 9, 11, 15, },
  { 0, 1, 3, 7, 9, 11, 15, },
  { 2, 3, 7, 9, 11, 15, },
  { 0, 2, 3, 7, 9, 11, 15, },
  { 1, 2, 3, 7, 9, 11, 15, },
  { 0, 1, 2, 3, 7, 9, 11, 15, },
  { 4, 7, 9, 11, 15, },
  { 0, 4, 7, 9, 11, 15, },
  { 1, 4, 7, 9, 11, 15, },
  { 0, 1, 4, 7, 9, 11, 15, },
  { 2, 4, 7, 9, 11, 15, },
  { 0, 2, 4, 7, 9, 11, 15, },
  { 1, 2, 4, 7, 9, 11, 15, },
  { 0, 1, 2, 4, 7, 9, 11, 15, },
  { 3, 4, 7, 9, 11, 15, },
  { 0, 3, 4, 7, 9, 11, 15, },
  { 1, 3, 4, 7, 9, 11, 15, },
  { 0, 1, 3, 4, 7, 9, 11, 15, },
  { 2, 3, 4, 7, 9, 11, 15, },
  { 0, 2, 3, 4, 7, 9, 11, 15, },
  { 1, 2, 3, 4, 7, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 15, },
  { 5, 7, 9, 11, 15, },
  { 0, 5, 7, 9, 11, 15, },
  { 1, 5, 7, 9, 11, 15, },
  { 0, 1, 5, 7, 9, 11, 15, },
  { 2, 5, 7, 9, 11, 15, },
  { 0, 2, 5, 7, 9, 11, 15, },
  { 1, 2, 5, 7, 9, 11, 15, },
  { 0, 1, 2, 5, 7, 9, 11, 15, },
  { 3, 5, 7, 9, 11, 15, },
  { 0, 3, 5, 7, 9, 11, 15, },
  { 1, 3, 5, 7, 9, 11, 15, },
  { 0, 1, 3, 5, 7, 9, 11, 15, },
  { 2, 3, 5, 7, 9, 11, 15, },
  { 0, 2, 3, 5, 7, 9, 11, 15, },
  { 1, 2, 3, 5, 7, 9, 11, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 15, },
  { 4, 5, 7, 9, 11, 15, },
  { 0, 4, 5, 7, 9, 11, 15, },
  { 1, 4, 5, 7, 9, 11, 15, },
  { 0, 1, 4, 5, 7, 9, 11, 15, },
  { 2, 4, 5, 7, 9, 11, 15, },
  { 0, 2, 4, 5, 7, 9, 11, 15, },
  { 1, 2, 4, 5, 7, 9, 11, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 15, },
  { 3, 4, 5, 7, 9, 11, 15, },
  { 0, 3, 4, 5, 7, 9, 11, 15, },
  { 1, 3, 4, 5, 7, 9, 11, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 15, },
  { 2, 3, 4, 5, 7, 9, 11, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 15, },
  { 6, 7, 9, 11, 15, },
  { 0, 6, 7, 9, 11, 15, },
  { 1, 6, 7, 9, 11, 15, },
  { 0, 1, 6, 7, 9, 11, 15, },
  { 2, 6, 7, 9, 11, 15, },
  { 0, 2, 6, 7, 9, 11, 15, },
  { 1, 2, 6, 7, 9, 11, 15, },
  { 0, 1, 2, 6, 7, 9, 11, 15, },
  { 3, 6, 7, 9, 11, 15, },
  { 0, 3, 6, 7, 9, 11, 15, },
  { 1, 3, 6, 7, 9, 11, 15, },
  { 0, 1, 3, 6, 7, 9, 11, 15, },
  { 2, 3, 6, 7, 9, 11, 15, },
  { 0, 2, 3, 6, 7, 9, 11, 15, },
  { 1, 2, 3, 6, 7, 9, 11, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 15, },
  { 4, 6, 7, 9, 11, 15, },
  { 0, 4, 6, 7, 9, 11, 15, },
  { 1, 4, 6, 7, 9, 11, 15, },
  { 0, 1, 4, 6, 7, 9, 11, 15, },
  { 2, 4, 6, 7, 9, 11, 15, },
  { 0, 2, 4, 6, 7, 9, 11, 15, },
  { 1, 2, 4, 6, 7, 9, 11, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 15, },
  { 3, 4, 6, 7, 9, 11, 15, },
  { 0, 3, 4, 6, 7, 9, 11, 15, },
  { 1, 3, 4, 6, 7, 9, 11, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 15, },
  { 2, 3, 4, 6, 7, 9, 11, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 15, },
  { 5, 6, 7, 9, 11, 15, },
  { 0, 5, 6, 7, 9, 11, 15, },
  { 1, 5, 6, 7, 9, 11, 15, },
  { 0, 1, 5, 6, 7, 9, 11, 15, },
  { 2, 5, 6, 7, 9, 11, 15, },
  { 0, 2, 5, 6, 7, 9, 11, 15, },
  { 1, 2, 5, 6, 7, 9, 11, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 15, },
  { 3, 5, 6, 7, 9, 11, 15, },
  { 0, 3, 5, 6, 7, 9, 11, 15, },
  { 1, 3, 5, 6, 7, 9, 11, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 15, },
  { 2, 3, 5, 6, 7, 9, 11, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 15, },
  { 4, 5, 6, 7, 9, 11, 15, },
  { 0, 4, 5, 6, 7, 9, 11, 15, },
  { 1, 4, 5, 6, 7, 9, 11, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 15, },
  { 2, 4, 5, 6, 7, 9, 11, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 15, },
  { 3, 4, 5, 6, 7, 9, 11, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 15, },
  { 8, 9, 11, 15, },
  { 0, 8, 9, 11, 15, },
  { 1, 8, 9, 11, 15, },
  { 0, 1, 8, 9, 11, 15, },
  { 2, 8, 9, 11, 15, },
  { 0, 2, 8, 9, 11, 15, },
  { 1, 2, 8, 9, 11, 15, },
  { 0, 1, 2, 8, 9, 11, 15, },
  { 3, 8, 9, 11, 15, },
  { 0, 3, 8, 9, 11, 15, },
  { 1, 3, 8, 9, 11, 15, },
  { 0, 1, 3, 8, 9, 11, 15, },
  { 2, 3, 8, 9, 11, 15, },
  { 0, 2, 3, 8, 9, 11, 15, },
  { 1, 2, 3, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 8, 9, 11, 15, },
  { 4, 8, 9, 11, 15, },
  { 0, 4, 8, 9, 11, 15, },
  { 1, 4, 8, 9, 11, 15, },
  { 0, 1, 4, 8, 9, 11, 15, },
  { 2, 4, 8, 9, 11, 15, },
  { 0, 2, 4, 8, 9, 11, 15, },
  { 1, 2, 4, 8, 9, 11, 15, },
  { 0, 1, 2, 4, 8, 9, 11, 15, },
  { 3, 4, 8, 9, 11, 15, },
  { 0, 3, 4, 8, 9, 11, 15, },
  { 1, 3, 4, 8, 9, 11, 15, },
  { 0, 1, 3, 4, 8, 9, 11, 15, },
  { 2, 3, 4, 8, 9, 11, 15, },
  { 0, 2, 3, 4, 8, 9, 11, 15, },
  { 1, 2, 3, 4, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 15, },
  { 5, 8, 9, 11, 15, },
  { 0, 5, 8, 9, 11, 15, },
  { 1, 5, 8, 9, 11, 15, },
  { 0, 1, 5, 8, 9, 11, 15, },
  { 2, 5, 8, 9, 11, 15, },
  { 0, 2, 5, 8, 9, 11, 15, },
  { 1, 2, 5, 8, 9, 11, 15, },
  { 0, 1, 2, 5, 8, 9, 11, 15, },
  { 3, 5, 8, 9, 11, 15, },
  { 0, 3, 5, 8, 9, 11, 15, },
  { 1, 3, 5, 8, 9, 11, 15, },
  { 0, 1, 3, 5, 8, 9, 11, 15, },
  { 2, 3, 5, 8, 9, 11, 15, },
  { 0, 2, 3, 5, 8, 9, 11, 15, },
  { 1, 2, 3, 5, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 15, },
  { 4, 5, 8, 9, 11, 15, },
  { 0, 4, 5, 8, 9, 11, 15, },
  { 1, 4, 5, 8, 9, 11, 15, },
  { 0, 1, 4, 5, 8, 9, 11, 15, },
  { 2, 4, 5, 8, 9, 11, 15, },
  { 0, 2, 4, 5, 8, 9, 11, 15, },
  { 1, 2, 4, 5, 8, 9, 11, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 15, },
  { 3, 4, 5, 8, 9, 11, 15, },
  { 0, 3, 4, 5, 8, 9, 11, 15, },
  { 1, 3, 4, 5, 8, 9, 11, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 15, },
  { 2, 3, 4, 5, 8, 9, 11, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 15, },
  { 6, 8, 9, 11, 15, },
  { 0, 6, 8, 9, 11, 15, },
  { 1, 6, 8, 9, 11, 15, },
  { 0, 1, 6, 8, 9, 11, 15, },
  { 2, 6, 8, 9, 11, 15, },
  { 0, 2, 6, 8, 9, 11, 15, },
  { 1, 2, 6, 8, 9, 11, 15, },
  { 0, 1, 2, 6, 8, 9, 11, 15, },
  { 3, 6, 8, 9, 11, 15, },
  { 0, 3, 6, 8, 9, 11, 15, },
  { 1, 3, 6, 8, 9, 11, 15, },
  { 0, 1, 3, 6, 8, 9, 11, 15, },
  { 2, 3, 6, 8, 9, 11, 15, },
  { 0, 2, 3, 6, 8, 9, 11, 15, },
  { 1, 2, 3, 6, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 15, },
  { 4, 6, 8, 9, 11, 15, },
  { 0, 4, 6, 8, 9, 11, 15, },
  { 1, 4, 6, 8, 9, 11, 15, },
  { 0, 1, 4, 6, 8, 9, 11, 15, },
  { 2, 4, 6, 8, 9, 11, 15, },
  { 0, 2, 4, 6, 8, 9, 11, 15, },
  { 1, 2, 4, 6, 8, 9, 11, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 15, },
  { 3, 4, 6, 8, 9, 11, 15, },
  { 0, 3, 4, 6, 8, 9, 11, 15, },
  { 1, 3, 4, 6, 8, 9, 11, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 15, },
  { 2, 3, 4, 6, 8, 9, 11, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 15, },
  { 5, 6, 8, 9, 11, 15, },
  { 0, 5, 6, 8, 9, 11, 15, },
  { 1, 5, 6, 8, 9, 11, 15, },
  { 0, 1, 5, 6, 8, 9, 11, 15, },
  { 2, 5, 6, 8, 9, 11, 15, },
  { 0, 2, 5, 6, 8, 9, 11, 15, },
  { 1, 2, 5, 6, 8, 9, 11, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 15, },
  { 3, 5, 6, 8, 9, 11, 15, },
  { 0, 3, 5, 6, 8, 9, 11, 15, },
  { 1, 3, 5, 6, 8, 9, 11, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 15, },
  { 2, 3, 5, 6, 8, 9, 11, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 15, },
  { 4, 5, 6, 8, 9, 11, 15, },
  { 0, 4, 5, 6, 8, 9, 11, 15, },
  { 1, 4, 5, 6, 8, 9, 11, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 15, },
  { 2, 4, 5, 6, 8, 9, 11, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 15, },
  { 3, 4, 5, 6, 8, 9, 11, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 15, },
  { 7, 8, 9, 11, 15, },
  { 0, 7, 8, 9, 11, 15, },
  { 1, 7, 8, 9, 11, 15, },
  { 0, 1, 7, 8, 9, 11, 15, },
  { 2, 7, 8, 9, 11, 15, },
  { 0, 2, 7, 8, 9, 11, 15, },
  { 1, 2, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 7, 8, 9, 11, 15, },
  { 3, 7, 8, 9, 11, 15, },
  { 0, 3, 7, 8, 9, 11, 15, },
  { 1, 3, 7, 8, 9, 11, 15, },
  { 0, 1, 3, 7, 8, 9, 11, 15, },
  { 2, 3, 7, 8, 9, 11, 15, },
  { 0, 2, 3, 7, 8, 9, 11, 15, },
  { 1, 2, 3, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 15, },
  { 4, 7, 8, 9, 11, 15, },
  { 0, 4, 7, 8, 9, 11, 15, },
  { 1, 4, 7, 8, 9, 11, 15, },
  { 0, 1, 4, 7, 8, 9, 11, 15, },
  { 2, 4, 7, 8, 9, 11, 15, },
  { 0, 2, 4, 7, 8, 9, 11, 15, },
  { 1, 2, 4, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 15, },
  { 3, 4, 7, 8, 9, 11, 15, },
  { 0, 3, 4, 7, 8, 9, 11, 15, },
  { 1, 3, 4, 7, 8, 9, 11, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 15, },
  { 2, 3, 4, 7, 8, 9, 11, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 15, },
  { 5, 7, 8, 9, 11, 15, },
  { 0, 5, 7, 8, 9, 11, 15, },
  { 1, 5, 7, 8, 9, 11, 15, },
  { 0, 1, 5, 7, 8, 9, 11, 15, },
  { 2, 5, 7, 8, 9, 11, 15, },
  { 0, 2, 5, 7, 8, 9, 11, 15, },
  { 1, 2, 5, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 15, },
  { 3, 5, 7, 8, 9, 11, 15, },
  { 0, 3, 5, 7, 8, 9, 11, 15, },
  { 1, 3, 5, 7, 8, 9, 11, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 15, },
  { 2, 3, 5, 7, 8, 9, 11, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 15, },
  { 4, 5, 7, 8, 9, 11, 15, },
  { 0, 4, 5, 7, 8, 9, 11, 15, },
  { 1, 4, 5, 7, 8, 9, 11, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 15, },
  { 2, 4, 5, 7, 8, 9, 11, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 15, },
  { 3, 4, 5, 7, 8, 9, 11, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 15, },
  { 6, 7, 8, 9, 11, 15, },
  { 0, 6, 7, 8, 9, 11, 15, },
  { 1, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 6, 7, 8, 9, 11, 15, },
  { 2, 6, 7, 8, 9, 11, 15, },
  { 0, 2, 6, 7, 8, 9, 11, 15, },
  { 1, 2, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 15, },
  { 3, 6, 7, 8, 9, 11, 15, },
  { 0, 3, 6, 7, 8, 9, 11, 15, },
  { 1, 3, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 15, },
  { 2, 3, 6, 7, 8, 9, 11, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 15, },
  { 4, 6, 7, 8, 9, 11, 15, },
  { 0, 4, 6, 7, 8, 9, 11, 15, },
  { 1, 4, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 15, },
  { 2, 4, 6, 7, 8, 9, 11, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 15, },
  { 3, 4, 6, 7, 8, 9, 11, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 15, },
  { 5, 6, 7, 8, 9, 11, 15, },
  { 0, 5, 6, 7, 8, 9, 11, 15, },
  { 1, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 15, },
  { 2, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 15, },
  { 3, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 15, },
  { 4, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 15, },
  { 10, 11, 15, },
  { 0, 10, 11, 15, },
  { 1, 10, 11, 15, },
  { 0, 1, 10, 11, 15, },
  { 2, 10, 11, 15, },
  { 0, 2, 10, 11, 15, },
  { 1, 2, 10, 11, 15, },
  { 0, 1, 2, 10, 11, 15, },
  { 3, 10, 11, 15, },
  { 0, 3, 10, 11, 15, },
  { 1, 3, 10, 11, 15, },
  { 0, 1, 3, 10, 11, 15, },
  { 2, 3, 10, 11, 15, },
  { 0, 2, 3, 10, 11, 15, },
  { 1, 2, 3, 10, 11, 15, },
  { 0, 1, 2, 3, 10, 11, 15, },
  { 4, 10, 11, 15, },
  { 0, 4, 10, 11, 15, },
  { 1, 4, 10, 11, 15, },
  { 0, 1, 4, 10, 11, 15, },
  { 2, 4, 10, 11, 15, },
  { 0, 2, 4, 10, 11, 15, },
  { 1, 2, 4, 10, 11, 15, },
  { 0, 1, 2, 4, 10, 11, 15, },
  { 3, 4, 10, 11, 15, },
  { 0, 3, 4, 10, 11, 15, },
  { 1, 3, 4, 10, 11, 15, },
  { 0, 1, 3, 4, 10, 11, 15, },
  { 2, 3, 4, 10, 11, 15, },
  { 0, 2, 3, 4, 10, 11, 15, },
  { 1, 2, 3, 4, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 10, 11, 15, },
  { 5, 10, 11, 15, },
  { 0, 5, 10, 11, 15, },
  { 1, 5, 10, 11, 15, },
  { 0, 1, 5, 10, 11, 15, },
  { 2, 5, 10, 11, 15, },
  { 0, 2, 5, 10, 11, 15, },
  { 1, 2, 5, 10, 11, 15, },
  { 0, 1, 2, 5, 10, 11, 15, },
  { 3, 5, 10, 11, 15, },
  { 0, 3, 5, 10, 11, 15, },
  { 1, 3, 5, 10, 11, 15, },
  { 0, 1, 3, 5, 10, 11, 15, },
  { 2, 3, 5, 10, 11, 15, },
  { 0, 2, 3, 5, 10, 11, 15, },
  { 1, 2, 3, 5, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 10, 11, 15, },
  { 4, 5, 10, 11, 15, },
  { 0, 4, 5, 10, 11, 15, },
  { 1, 4, 5, 10, 11, 15, },
  { 0, 1, 4, 5, 10, 11, 15, },
  { 2, 4, 5, 10, 11, 15, },
  { 0, 2, 4, 5, 10, 11, 15, },
  { 1, 2, 4, 5, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 10, 11, 15, },
  { 3, 4, 5, 10, 11, 15, },
  { 0, 3, 4, 5, 10, 11, 15, },
  { 1, 3, 4, 5, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 10, 11, 15, },
  { 2, 3, 4, 5, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 15, },
  { 6, 10, 11, 15, },
  { 0, 6, 10, 11, 15, },
  { 1, 6, 10, 11, 15, },
  { 0, 1, 6, 10, 11, 15, },
  { 2, 6, 10, 11, 15, },
  { 0, 2, 6, 10, 11, 15, },
  { 1, 2, 6, 10, 11, 15, },
  { 0, 1, 2, 6, 10, 11, 15, },
  { 3, 6, 10, 11, 15, },
  { 0, 3, 6, 10, 11, 15, },
  { 1, 3, 6, 10, 11, 15, },
  { 0, 1, 3, 6, 10, 11, 15, },
  { 2, 3, 6, 10, 11, 15, },
  { 0, 2, 3, 6, 10, 11, 15, },
  { 1, 2, 3, 6, 10, 11, 15, },
  { 0, 1, 2, 3, 6, 10, 11, 15, },
  { 4, 6, 10, 11, 15, },
  { 0, 4, 6, 10, 11, 15, },
  { 1, 4, 6, 10, 11, 15, },
  { 0, 1, 4, 6, 10, 11, 15, },
  { 2, 4, 6, 10, 11, 15, },
  { 0, 2, 4, 6, 10, 11, 15, },
  { 1, 2, 4, 6, 10, 11, 15, },
  { 0, 1, 2, 4, 6, 10, 11, 15, },
  { 3, 4, 6, 10, 11, 15, },
  { 0, 3, 4, 6, 10, 11, 15, },
  { 1, 3, 4, 6, 10, 11, 15, },
  { 0, 1, 3, 4, 6, 10, 11, 15, },
  { 2, 3, 4, 6, 10, 11, 15, },
  { 0, 2, 3, 4, 6, 10, 11, 15, },
  { 1, 2, 3, 4, 6, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 15, },
  { 5, 6, 10, 11, 15, },
  { 0, 5, 6, 10, 11, 15, },
  { 1, 5, 6, 10, 11, 15, },
  { 0, 1, 5, 6, 10, 11, 15, },
  { 2, 5, 6, 10, 11, 15, },
  { 0, 2, 5, 6, 10, 11, 15, },
  { 1, 2, 5, 6, 10, 11, 15, },
  { 0, 1, 2, 5, 6, 10, 11, 15, },
  { 3, 5, 6, 10, 11, 15, },
  { 0, 3, 5, 6, 10, 11, 15, },
  { 1, 3, 5, 6, 10, 11, 15, },
  { 0, 1, 3, 5, 6, 10, 11, 15, },
  { 2, 3, 5, 6, 10, 11, 15, },
  { 0, 2, 3, 5, 6, 10, 11, 15, },
  { 1, 2, 3, 5, 6, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 15, },
  { 4, 5, 6, 10, 11, 15, },
  { 0, 4, 5, 6, 10, 11, 15, },
  { 1, 4, 5, 6, 10, 11, 15, },
  { 0, 1, 4, 5, 6, 10, 11, 15, },
  { 2, 4, 5, 6, 10, 11, 15, },
  { 0, 2, 4, 5, 6, 10, 11, 15, },
  { 1, 2, 4, 5, 6, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 15, },
  { 3, 4, 5, 6, 10, 11, 15, },
  { 0, 3, 4, 5, 6, 10, 11, 15, },
  { 1, 3, 4, 5, 6, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 15, },
  { 2, 3, 4, 5, 6, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 15, },
  { 7, 10, 11, 15, },
  { 0, 7, 10, 11, 15, },
  { 1, 7, 10, 11, 15, },
  { 0, 1, 7, 10, 11, 15, },
  { 2, 7, 10, 11, 15, },
  { 0, 2, 7, 10, 11, 15, },
  { 1, 2, 7, 10, 11, 15, },
  { 0, 1, 2, 7, 10, 11, 15, },
  { 3, 7, 10, 11, 15, },
  { 0, 3, 7, 10, 11, 15, },
  { 1, 3, 7, 10, 11, 15, },
  { 0, 1, 3, 7, 10, 11, 15, },
  { 2, 3, 7, 10, 11, 15, },
  { 0, 2, 3, 7, 10, 11, 15, },
  { 1, 2, 3, 7, 10, 11, 15, },
  { 0, 1, 2, 3, 7, 10, 11, 15, },
  { 4, 7, 10, 11, 15, },
  { 0, 4, 7, 10, 11, 15, },
  { 1, 4, 7, 10, 11, 15, },
  { 0, 1, 4, 7, 10, 11, 15, },
  { 2, 4, 7, 10, 11, 15, },
  { 0, 2, 4, 7, 10, 11, 15, },
  { 1, 2, 4, 7, 10, 11, 15, },
  { 0, 1, 2, 4, 7, 10, 11, 15, },
  { 3, 4, 7, 10, 11, 15, },
  { 0, 3, 4, 7, 10, 11, 15, },
  { 1, 3, 4, 7, 10, 11, 15, },
  { 0, 1, 3, 4, 7, 10, 11, 15, },
  { 2, 3, 4, 7, 10, 11, 15, },
  { 0, 2, 3, 4, 7, 10, 11, 15, },
  { 1, 2, 3, 4, 7, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 15, },
  { 5, 7, 10, 11, 15, },
  { 0, 5, 7, 10, 11, 15, },
  { 1, 5, 7, 10, 11, 15, },
  { 0, 1, 5, 7, 10, 11, 15, },
  { 2, 5, 7, 10, 11, 15, },
  { 0, 2, 5, 7, 10, 11, 15, },
  { 1, 2, 5, 7, 10, 11, 15, },
  { 0, 1, 2, 5, 7, 10, 11, 15, },
  { 3, 5, 7, 10, 11, 15, },
  { 0, 3, 5, 7, 10, 11, 15, },
  { 1, 3, 5, 7, 10, 11, 15, },
  { 0, 1, 3, 5, 7, 10, 11, 15, },
  { 2, 3, 5, 7, 10, 11, 15, },
  { 0, 2, 3, 5, 7, 10, 11, 15, },
  { 1, 2, 3, 5, 7, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 15, },
  { 4, 5, 7, 10, 11, 15, },
  { 0, 4, 5, 7, 10, 11, 15, },
  { 1, 4, 5, 7, 10, 11, 15, },
  { 0, 1, 4, 5, 7, 10, 11, 15, },
  { 2, 4, 5, 7, 10, 11, 15, },
  { 0, 2, 4, 5, 7, 10, 11, 15, },
  { 1, 2, 4, 5, 7, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 15, },
  { 3, 4, 5, 7, 10, 11, 15, },
  { 0, 3, 4, 5, 7, 10, 11, 15, },
  { 1, 3, 4, 5, 7, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 15, },
  { 2, 3, 4, 5, 7, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 15, },
  { 6, 7, 10, 11, 15, },
  { 0, 6, 7, 10, 11, 15, },
  { 1, 6, 7, 10, 11, 15, },
  { 0, 1, 6, 7, 10, 11, 15, },
  { 2, 6, 7, 10, 11, 15, },
  { 0, 2, 6, 7, 10, 11, 15, },
  { 1, 2, 6, 7, 10, 11, 15, },
  { 0, 1, 2, 6, 7, 10, 11, 15, },
  { 3, 6, 7, 10, 11, 15, },
  { 0, 3, 6, 7, 10, 11, 15, },
  { 1, 3, 6, 7, 10, 11, 15, },
  { 0, 1, 3, 6, 7, 10, 11, 15, },
  { 2, 3, 6, 7, 10, 11, 15, },
  { 0, 2, 3, 6, 7, 10, 11, 15, },
  { 1, 2, 3, 6, 7, 10, 11, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 15, },
  { 4, 6, 7, 10, 11, 15, },
  { 0, 4, 6, 7, 10, 11, 15, },
  { 1, 4, 6, 7, 10, 11, 15, },
  { 0, 1, 4, 6, 7, 10, 11, 15, },
  { 2, 4, 6, 7, 10, 11, 15, },
  { 0, 2, 4, 6, 7, 10, 11, 15, },
  { 1, 2, 4, 6, 7, 10, 11, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 15, },
  { 3, 4, 6, 7, 10, 11, 15, },
  { 0, 3, 4, 6, 7, 10, 11, 15, },
  { 1, 3, 4, 6, 7, 10, 11, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 15, },
  { 2, 3, 4, 6, 7, 10, 11, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 15, },
  { 5, 6, 7, 10, 11, 15, },
  { 0, 5, 6, 7, 10, 11, 15, },
  { 1, 5, 6, 7, 10, 11, 15, },
  { 0, 1, 5, 6, 7, 10, 11, 15, },
  { 2, 5, 6, 7, 10, 11, 15, },
  { 0, 2, 5, 6, 7, 10, 11, 15, },
  { 1, 2, 5, 6, 7, 10, 11, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 15, },
  { 3, 5, 6, 7, 10, 11, 15, },
  { 0, 3, 5, 6, 7, 10, 11, 15, },
  { 1, 3, 5, 6, 7, 10, 11, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 15, },
  { 2, 3, 5, 6, 7, 10, 11, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 15, },
  { 4, 5, 6, 7, 10, 11, 15, },
  { 0, 4, 5, 6, 7, 10, 11, 15, },
  { 1, 4, 5, 6, 7, 10, 11, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 15, },
  { 2, 4, 5, 6, 7, 10, 11, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 15, },
  { 3, 4, 5, 6, 7, 10, 11, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 15, },
  { 8, 10, 11, 15, },
  { 0, 8, 10, 11, 15, },
  { 1, 8, 10, 11, 15, },
  { 0, 1, 8, 10, 11, 15, },
  { 2, 8, 10, 11, 15, },
  { 0, 2, 8, 10, 11, 15, },
  { 1, 2, 8, 10, 11, 15, },
  { 0, 1, 2, 8, 10, 11, 15, },
  { 3, 8, 10, 11, 15, },
  { 0, 3, 8, 10, 11, 15, },
  { 1, 3, 8, 10, 11, 15, },
  { 0, 1, 3, 8, 10, 11, 15, },
  { 2, 3, 8, 10, 11, 15, },
  { 0, 2, 3, 8, 10, 11, 15, },
  { 1, 2, 3, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 8, 10, 11, 15, },
  { 4, 8, 10, 11, 15, },
  { 0, 4, 8, 10, 11, 15, },
  { 1, 4, 8, 10, 11, 15, },
  { 0, 1, 4, 8, 10, 11, 15, },
  { 2, 4, 8, 10, 11, 15, },
  { 0, 2, 4, 8, 10, 11, 15, },
  { 1, 2, 4, 8, 10, 11, 15, },
  { 0, 1, 2, 4, 8, 10, 11, 15, },
  { 3, 4, 8, 10, 11, 15, },
  { 0, 3, 4, 8, 10, 11, 15, },
  { 1, 3, 4, 8, 10, 11, 15, },
  { 0, 1, 3, 4, 8, 10, 11, 15, },
  { 2, 3, 4, 8, 10, 11, 15, },
  { 0, 2, 3, 4, 8, 10, 11, 15, },
  { 1, 2, 3, 4, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 15, },
  { 5, 8, 10, 11, 15, },
  { 0, 5, 8, 10, 11, 15, },
  { 1, 5, 8, 10, 11, 15, },
  { 0, 1, 5, 8, 10, 11, 15, },
  { 2, 5, 8, 10, 11, 15, },
  { 0, 2, 5, 8, 10, 11, 15, },
  { 1, 2, 5, 8, 10, 11, 15, },
  { 0, 1, 2, 5, 8, 10, 11, 15, },
  { 3, 5, 8, 10, 11, 15, },
  { 0, 3, 5, 8, 10, 11, 15, },
  { 1, 3, 5, 8, 10, 11, 15, },
  { 0, 1, 3, 5, 8, 10, 11, 15, },
  { 2, 3, 5, 8, 10, 11, 15, },
  { 0, 2, 3, 5, 8, 10, 11, 15, },
  { 1, 2, 3, 5, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 15, },
  { 4, 5, 8, 10, 11, 15, },
  { 0, 4, 5, 8, 10, 11, 15, },
  { 1, 4, 5, 8, 10, 11, 15, },
  { 0, 1, 4, 5, 8, 10, 11, 15, },
  { 2, 4, 5, 8, 10, 11, 15, },
  { 0, 2, 4, 5, 8, 10, 11, 15, },
  { 1, 2, 4, 5, 8, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 15, },
  { 3, 4, 5, 8, 10, 11, 15, },
  { 0, 3, 4, 5, 8, 10, 11, 15, },
  { 1, 3, 4, 5, 8, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 15, },
  { 2, 3, 4, 5, 8, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 15, },
  { 6, 8, 10, 11, 15, },
  { 0, 6, 8, 10, 11, 15, },
  { 1, 6, 8, 10, 11, 15, },
  { 0, 1, 6, 8, 10, 11, 15, },
  { 2, 6, 8, 10, 11, 15, },
  { 0, 2, 6, 8, 10, 11, 15, },
  { 1, 2, 6, 8, 10, 11, 15, },
  { 0, 1, 2, 6, 8, 10, 11, 15, },
  { 3, 6, 8, 10, 11, 15, },
  { 0, 3, 6, 8, 10, 11, 15, },
  { 1, 3, 6, 8, 10, 11, 15, },
  { 0, 1, 3, 6, 8, 10, 11, 15, },
  { 2, 3, 6, 8, 10, 11, 15, },
  { 0, 2, 3, 6, 8, 10, 11, 15, },
  { 1, 2, 3, 6, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 15, },
  { 4, 6, 8, 10, 11, 15, },
  { 0, 4, 6, 8, 10, 11, 15, },
  { 1, 4, 6, 8, 10, 11, 15, },
  { 0, 1, 4, 6, 8, 10, 11, 15, },
  { 2, 4, 6, 8, 10, 11, 15, },
  { 0, 2, 4, 6, 8, 10, 11, 15, },
  { 1, 2, 4, 6, 8, 10, 11, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 15, },
  { 3, 4, 6, 8, 10, 11, 15, },
  { 0, 3, 4, 6, 8, 10, 11, 15, },
  { 1, 3, 4, 6, 8, 10, 11, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 15, },
  { 2, 3, 4, 6, 8, 10, 11, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 15, },
  { 5, 6, 8, 10, 11, 15, },
  { 0, 5, 6, 8, 10, 11, 15, },
  { 1, 5, 6, 8, 10, 11, 15, },
  { 0, 1, 5, 6, 8, 10, 11, 15, },
  { 2, 5, 6, 8, 10, 11, 15, },
  { 0, 2, 5, 6, 8, 10, 11, 15, },
  { 1, 2, 5, 6, 8, 10, 11, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 15, },
  { 3, 5, 6, 8, 10, 11, 15, },
  { 0, 3, 5, 6, 8, 10, 11, 15, },
  { 1, 3, 5, 6, 8, 10, 11, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 15, },
  { 2, 3, 5, 6, 8, 10, 11, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 15, },
  { 4, 5, 6, 8, 10, 11, 15, },
  { 0, 4, 5, 6, 8, 10, 11, 15, },
  { 1, 4, 5, 6, 8, 10, 11, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 15, },
  { 2, 4, 5, 6, 8, 10, 11, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 15, },
  { 3, 4, 5, 6, 8, 10, 11, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 15, },
  { 7, 8, 10, 11, 15, },
  { 0, 7, 8, 10, 11, 15, },
  { 1, 7, 8, 10, 11, 15, },
  { 0, 1, 7, 8, 10, 11, 15, },
  { 2, 7, 8, 10, 11, 15, },
  { 0, 2, 7, 8, 10, 11, 15, },
  { 1, 2, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 7, 8, 10, 11, 15, },
  { 3, 7, 8, 10, 11, 15, },
  { 0, 3, 7, 8, 10, 11, 15, },
  { 1, 3, 7, 8, 10, 11, 15, },
  { 0, 1, 3, 7, 8, 10, 11, 15, },
  { 2, 3, 7, 8, 10, 11, 15, },
  { 0, 2, 3, 7, 8, 10, 11, 15, },
  { 1, 2, 3, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 15, },
  { 4, 7, 8, 10, 11, 15, },
  { 0, 4, 7, 8, 10, 11, 15, },
  { 1, 4, 7, 8, 10, 11, 15, },
  { 0, 1, 4, 7, 8, 10, 11, 15, },
  { 2, 4, 7, 8, 10, 11, 15, },
  { 0, 2, 4, 7, 8, 10, 11, 15, },
  { 1, 2, 4, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 15, },
  { 3, 4, 7, 8, 10, 11, 15, },
  { 0, 3, 4, 7, 8, 10, 11, 15, },
  { 1, 3, 4, 7, 8, 10, 11, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 15, },
  { 2, 3, 4, 7, 8, 10, 11, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 15, },
  { 5, 7, 8, 10, 11, 15, },
  { 0, 5, 7, 8, 10, 11, 15, },
  { 1, 5, 7, 8, 10, 11, 15, },
  { 0, 1, 5, 7, 8, 10, 11, 15, },
  { 2, 5, 7, 8, 10, 11, 15, },
  { 0, 2, 5, 7, 8, 10, 11, 15, },
  { 1, 2, 5, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 15, },
  { 3, 5, 7, 8, 10, 11, 15, },
  { 0, 3, 5, 7, 8, 10, 11, 15, },
  { 1, 3, 5, 7, 8, 10, 11, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 15, },
  { 2, 3, 5, 7, 8, 10, 11, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 15, },
  { 4, 5, 7, 8, 10, 11, 15, },
  { 0, 4, 5, 7, 8, 10, 11, 15, },
  { 1, 4, 5, 7, 8, 10, 11, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 15, },
  { 2, 4, 5, 7, 8, 10, 11, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 15, },
  { 3, 4, 5, 7, 8, 10, 11, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 15, },
  { 6, 7, 8, 10, 11, 15, },
  { 0, 6, 7, 8, 10, 11, 15, },
  { 1, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 6, 7, 8, 10, 11, 15, },
  { 2, 6, 7, 8, 10, 11, 15, },
  { 0, 2, 6, 7, 8, 10, 11, 15, },
  { 1, 2, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 15, },
  { 3, 6, 7, 8, 10, 11, 15, },
  { 0, 3, 6, 7, 8, 10, 11, 15, },
  { 1, 3, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 15, },
  { 2, 3, 6, 7, 8, 10, 11, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 15, },
  { 4, 6, 7, 8, 10, 11, 15, },
  { 0, 4, 6, 7, 8, 10, 11, 15, },
  { 1, 4, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 15, },
  { 2, 4, 6, 7, 8, 10, 11, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 15, },
  { 3, 4, 6, 7, 8, 10, 11, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 15, },
  { 5, 6, 7, 8, 10, 11, 15, },
  { 0, 5, 6, 7, 8, 10, 11, 15, },
  { 1, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 15, },
  { 2, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 15, },
  { 3, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 15, },
  { 4, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 15, },
  { 9, 10, 11, 15, },
  { 0, 9, 10, 11, 15, },
  { 1, 9, 10, 11, 15, },
  { 0, 1, 9, 10, 11, 15, },
  { 2, 9, 10, 11, 15, },
  { 0, 2, 9, 10, 11, 15, },
  { 1, 2, 9, 10, 11, 15, },
  { 0, 1, 2, 9, 10, 11, 15, },
  { 3, 9, 10, 11, 15, },
  { 0, 3, 9, 10, 11, 15, },
  { 1, 3, 9, 10, 11, 15, },
  { 0, 1, 3, 9, 10, 11, 15, },
  { 2, 3, 9, 10, 11, 15, },
  { 0, 2, 3, 9, 10, 11, 15, },
  { 1, 2, 3, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 9, 10, 11, 15, },
  { 4, 9, 10, 11, 15, },
  { 0, 4, 9, 10, 11, 15, },
  { 1, 4, 9, 10, 11, 15, },
  { 0, 1, 4, 9, 10, 11, 15, },
  { 2, 4, 9, 10, 11, 15, },
  { 0, 2, 4, 9, 10, 11, 15, },
  { 1, 2, 4, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 9, 10, 11, 15, },
  { 3, 4, 9, 10, 11, 15, },
  { 0, 3, 4, 9, 10, 11, 15, },
  { 1, 3, 4, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 9, 10, 11, 15, },
  { 2, 3, 4, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 15, },
  { 5, 9, 10, 11, 15, },
  { 0, 5, 9, 10, 11, 15, },
  { 1, 5, 9, 10, 11, 15, },
  { 0, 1, 5, 9, 10, 11, 15, },
  { 2, 5, 9, 10, 11, 15, },
  { 0, 2, 5, 9, 10, 11, 15, },
  { 1, 2, 5, 9, 10, 11, 15, },
  { 0, 1, 2, 5, 9, 10, 11, 15, },
  { 3, 5, 9, 10, 11, 15, },
  { 0, 3, 5, 9, 10, 11, 15, },
  { 1, 3, 5, 9, 10, 11, 15, },
  { 0, 1, 3, 5, 9, 10, 11, 15, },
  { 2, 3, 5, 9, 10, 11, 15, },
  { 0, 2, 3, 5, 9, 10, 11, 15, },
  { 1, 2, 3, 5, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 15, },
  { 4, 5, 9, 10, 11, 15, },
  { 0, 4, 5, 9, 10, 11, 15, },
  { 1, 4, 5, 9, 10, 11, 15, },
  { 0, 1, 4, 5, 9, 10, 11, 15, },
  { 2, 4, 5, 9, 10, 11, 15, },
  { 0, 2, 4, 5, 9, 10, 11, 15, },
  { 1, 2, 4, 5, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 15, },
  { 3, 4, 5, 9, 10, 11, 15, },
  { 0, 3, 4, 5, 9, 10, 11, 15, },
  { 1, 3, 4, 5, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 15, },
  { 2, 3, 4, 5, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 15, },
  { 6, 9, 10, 11, 15, },
  { 0, 6, 9, 10, 11, 15, },
  { 1, 6, 9, 10, 11, 15, },
  { 0, 1, 6, 9, 10, 11, 15, },
  { 2, 6, 9, 10, 11, 15, },
  { 0, 2, 6, 9, 10, 11, 15, },
  { 1, 2, 6, 9, 10, 11, 15, },
  { 0, 1, 2, 6, 9, 10, 11, 15, },
  { 3, 6, 9, 10, 11, 15, },
  { 0, 3, 6, 9, 10, 11, 15, },
  { 1, 3, 6, 9, 10, 11, 15, },
  { 0, 1, 3, 6, 9, 10, 11, 15, },
  { 2, 3, 6, 9, 10, 11, 15, },
  { 0, 2, 3, 6, 9, 10, 11, 15, },
  { 1, 2, 3, 6, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 15, },
  { 4, 6, 9, 10, 11, 15, },
  { 0, 4, 6, 9, 10, 11, 15, },
  { 1, 4, 6, 9, 10, 11, 15, },
  { 0, 1, 4, 6, 9, 10, 11, 15, },
  { 2, 4, 6, 9, 10, 11, 15, },
  { 0, 2, 4, 6, 9, 10, 11, 15, },
  { 1, 2, 4, 6, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 15, },
  { 3, 4, 6, 9, 10, 11, 15, },
  { 0, 3, 4, 6, 9, 10, 11, 15, },
  { 1, 3, 4, 6, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 15, },
  { 2, 3, 4, 6, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 15, },
  { 5, 6, 9, 10, 11, 15, },
  { 0, 5, 6, 9, 10, 11, 15, },
  { 1, 5, 6, 9, 10, 11, 15, },
  { 0, 1, 5, 6, 9, 10, 11, 15, },
  { 2, 5, 6, 9, 10, 11, 15, },
  { 0, 2, 5, 6, 9, 10, 11, 15, },
  { 1, 2, 5, 6, 9, 10, 11, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 15, },
  { 3, 5, 6, 9, 10, 11, 15, },
  { 0, 3, 5, 6, 9, 10, 11, 15, },
  { 1, 3, 5, 6, 9, 10, 11, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 15, },
  { 2, 3, 5, 6, 9, 10, 11, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 15, },
  { 4, 5, 6, 9, 10, 11, 15, },
  { 0, 4, 5, 6, 9, 10, 11, 15, },
  { 1, 4, 5, 6, 9, 10, 11, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 15, },
  { 2, 4, 5, 6, 9, 10, 11, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 15, },
  { 3, 4, 5, 6, 9, 10, 11, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 15, },
  { 7, 9, 10, 11, 15, },
  { 0, 7, 9, 10, 11, 15, },
  { 1, 7, 9, 10, 11, 15, },
  { 0, 1, 7, 9, 10, 11, 15, },
  { 2, 7, 9, 10, 11, 15, },
  { 0, 2, 7, 9, 10, 11, 15, },
  { 1, 2, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 7, 9, 10, 11, 15, },
  { 3, 7, 9, 10, 11, 15, },
  { 0, 3, 7, 9, 10, 11, 15, },
  { 1, 3, 7, 9, 10, 11, 15, },
  { 0, 1, 3, 7, 9, 10, 11, 15, },
  { 2, 3, 7, 9, 10, 11, 15, },
  { 0, 2, 3, 7, 9, 10, 11, 15, },
  { 1, 2, 3, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 15, },
  { 4, 7, 9, 10, 11, 15, },
  { 0, 4, 7, 9, 10, 11, 15, },
  { 1, 4, 7, 9, 10, 11, 15, },
  { 0, 1, 4, 7, 9, 10, 11, 15, },
  { 2, 4, 7, 9, 10, 11, 15, },
  { 0, 2, 4, 7, 9, 10, 11, 15, },
  { 1, 2, 4, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 15, },
  { 3, 4, 7, 9, 10, 11, 15, },
  { 0, 3, 4, 7, 9, 10, 11, 15, },
  { 1, 3, 4, 7, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 15, },
  { 2, 3, 4, 7, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 15, },
  { 5, 7, 9, 10, 11, 15, },
  { 0, 5, 7, 9, 10, 11, 15, },
  { 1, 5, 7, 9, 10, 11, 15, },
  { 0, 1, 5, 7, 9, 10, 11, 15, },
  { 2, 5, 7, 9, 10, 11, 15, },
  { 0, 2, 5, 7, 9, 10, 11, 15, },
  { 1, 2, 5, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 15, },
  { 3, 5, 7, 9, 10, 11, 15, },
  { 0, 3, 5, 7, 9, 10, 11, 15, },
  { 1, 3, 5, 7, 9, 10, 11, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 15, },
  { 2, 3, 5, 7, 9, 10, 11, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 15, },
  { 4, 5, 7, 9, 10, 11, 15, },
  { 0, 4, 5, 7, 9, 10, 11, 15, },
  { 1, 4, 5, 7, 9, 10, 11, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 15, },
  { 2, 4, 5, 7, 9, 10, 11, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 15, },
  { 3, 4, 5, 7, 9, 10, 11, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 15, },
  { 6, 7, 9, 10, 11, 15, },
  { 0, 6, 7, 9, 10, 11, 15, },
  { 1, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 6, 7, 9, 10, 11, 15, },
  { 2, 6, 7, 9, 10, 11, 15, },
  { 0, 2, 6, 7, 9, 10, 11, 15, },
  { 1, 2, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 15, },
  { 3, 6, 7, 9, 10, 11, 15, },
  { 0, 3, 6, 7, 9, 10, 11, 15, },
  { 1, 3, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 15, },
  { 2, 3, 6, 7, 9, 10, 11, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 15, },
  { 4, 6, 7, 9, 10, 11, 15, },
  { 0, 4, 6, 7, 9, 10, 11, 15, },
  { 1, 4, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 15, },
  { 2, 4, 6, 7, 9, 10, 11, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 15, },
  { 3, 4, 6, 7, 9, 10, 11, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 15, },
  { 5, 6, 7, 9, 10, 11, 15, },
  { 0, 5, 6, 7, 9, 10, 11, 15, },
  { 1, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 15, },
  { 2, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 15, },
  { 3, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 15, },
  { 4, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 15, },
  { 8, 9, 10, 11, 15, },
  { 0, 8, 9, 10, 11, 15, },
  { 1, 8, 9, 10, 11, 15, },
  { 0, 1, 8, 9, 10, 11, 15, },
  { 2, 8, 9, 10, 11, 15, },
  { 0, 2, 8, 9, 10, 11, 15, },
  { 1, 2, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 8, 9, 10, 11, 15, },
  { 3, 8, 9, 10, 11, 15, },
  { 0, 3, 8, 9, 10, 11, 15, },
  { 1, 3, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 8, 9, 10, 11, 15, },
  { 2, 3, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 15, },
  { 4, 8, 9, 10, 11, 15, },
  { 0, 4, 8, 9, 10, 11, 15, },
  { 1, 4, 8, 9, 10, 11, 15, },
  { 0, 1, 4, 8, 9, 10, 11, 15, },
  { 2, 4, 8, 9, 10, 11, 15, },
  { 0, 2, 4, 8, 9, 10, 11, 15, },
  { 1, 2, 4, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 15, },
  { 3, 4, 8, 9, 10, 11, 15, },
  { 0, 3, 4, 8, 9, 10, 11, 15, },
  { 1, 3, 4, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 15, },
  { 2, 3, 4, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 15, },
  { 5, 8, 9, 10, 11, 15, },
  { 0, 5, 8, 9, 10, 11, 15, },
  { 1, 5, 8, 9, 10, 11, 15, },
  { 0, 1, 5, 8, 9, 10, 11, 15, },
  { 2, 5, 8, 9, 10, 11, 15, },
  { 0, 2, 5, 8, 9, 10, 11, 15, },
  { 1, 2, 5, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 15, },
  { 3, 5, 8, 9, 10, 11, 15, },
  { 0, 3, 5, 8, 9, 10, 11, 15, },
  { 1, 3, 5, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 15, },
  { 2, 3, 5, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 15, },
  { 4, 5, 8, 9, 10, 11, 15, },
  { 0, 4, 5, 8, 9, 10, 11, 15, },
  { 1, 4, 5, 8, 9, 10, 11, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 15, },
  { 2, 4, 5, 8, 9, 10, 11, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 15, },
  { 3, 4, 5, 8, 9, 10, 11, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 15, },
  { 6, 8, 9, 10, 11, 15, },
  { 0, 6, 8, 9, 10, 11, 15, },
  { 1, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 6, 8, 9, 10, 11, 15, },
  { 2, 6, 8, 9, 10, 11, 15, },
  { 0, 2, 6, 8, 9, 10, 11, 15, },
  { 1, 2, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 15, },
  { 3, 6, 8, 9, 10, 11, 15, },
  { 0, 3, 6, 8, 9, 10, 11, 15, },
  { 1, 3, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 15, },
  { 2, 3, 6, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 15, },
  { 4, 6, 8, 9, 10, 11, 15, },
  { 0, 4, 6, 8, 9, 10, 11, 15, },
  { 1, 4, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 15, },
  { 2, 4, 6, 8, 9, 10, 11, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 15, },
  { 3, 4, 6, 8, 9, 10, 11, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 15, },
  { 5, 6, 8, 9, 10, 11, 15, },
  { 0, 5, 6, 8, 9, 10, 11, 15, },
  { 1, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 15, },
  { 2, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 15, },
  { 3, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 15, },
  { 4, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 15, },
  { 7, 8, 9, 10, 11, 15, },
  { 0, 7, 8, 9, 10, 11, 15, },
  { 1, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 7, 8, 9, 10, 11, 15, },
  { 2, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 15, },
  { 3, 7, 8, 9, 10, 11, 15, },
  { 0, 3, 7, 8, 9, 10, 11, 15, },
  { 1, 3, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 15, },
  { 2, 3, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 15, },
  { 4, 7, 8, 9, 10, 11, 15, },
  { 0, 4, 7, 8, 9, 10, 11, 15, },
  { 1, 4, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 15, },
  { 2, 4, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 15, },
  { 3, 4, 7, 8, 9, 10, 11, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 15, },
  { 5, 7, 8, 9, 10, 11, 15, },
  { 0, 5, 7, 8, 9, 10, 11, 15, },
  { 1, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 15, },
  { 2, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 15, },
  { 3, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 15, },
  { 4, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 15, },
  { 6, 7, 8, 9, 10, 11, 15, },
  { 0, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 15, },
  { 2, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 15, },
  { 3, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 15, },
  { 4, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 15, },
  { 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, },
  { 12, 15, },
  { 0, 12, 15, },
  { 1, 12, 15, },
  { 0, 1, 12, 15, },
  { 2, 12, 15, },
  { 0, 2, 12, 15, },
  { 1, 2, 12, 15, },
  { 0, 1, 2, 12, 15, },
  { 3, 12, 15, },
  { 0, 3, 12, 15, },
  { 1, 3, 12, 15, },
  { 0, 1, 3, 12, 15, },
  { 2, 3, 12, 15, },
  { 0, 2, 3, 12, 15, },
  { 1, 2, 3, 12, 15, },
  { 0, 1, 2, 3, 12, 15, },
  { 4, 12, 15, },
  { 0, 4, 12, 15, },
  { 1, 4, 12, 15, },
  { 0, 1, 4, 12, 15, },
  { 2, 4, 12, 15, },
  { 0, 2, 4, 12, 15, },
  { 1, 2, 4, 12, 15, },
  { 0, 1, 2, 4, 12, 15, },
  { 3, 4, 12, 15, },
  { 0, 3, 4, 12, 15, },
  { 1, 3, 4, 12, 15, },
  { 0, 1, 3, 4, 12, 15, },
  { 2, 3, 4, 12, 15, },
  { 0, 2, 3, 4, 12, 15, },
  { 1, 2, 3, 4, 12, 15, },
  { 0, 1, 2, 3, 4, 12, 15, },
  { 5, 12, 15, },
  { 0, 5, 12, 15, },
  { 1, 5, 12, 15, },
  { 0, 1, 5, 12, 15, },
  { 2, 5, 12, 15, },
  { 0, 2, 5, 12, 15, },
  { 1, 2, 5, 12, 15, },
  { 0, 1, 2, 5, 12, 15, },
  { 3, 5, 12, 15, },
  { 0, 3, 5, 12, 15, },
  { 1, 3, 5, 12, 15, },
  { 0, 1, 3, 5, 12, 15, },
  { 2, 3, 5, 12, 15, },
  { 0, 2, 3, 5, 12, 15, },
  { 1, 2, 3, 5, 12, 15, },
  { 0, 1, 2, 3, 5, 12, 15, },
  { 4, 5, 12, 15, },
  { 0, 4, 5, 12, 15, },
  { 1, 4, 5, 12, 15, },
  { 0, 1, 4, 5, 12, 15, },
  { 2, 4, 5, 12, 15, },
  { 0, 2, 4, 5, 12, 15, },
  { 1, 2, 4, 5, 12, 15, },
  { 0, 1, 2, 4, 5, 12, 15, },
  { 3, 4, 5, 12, 15, },
  { 0, 3, 4, 5, 12, 15, },
  { 1, 3, 4, 5, 12, 15, },
  { 0, 1, 3, 4, 5, 12, 15, },
  { 2, 3, 4, 5, 12, 15, },
  { 0, 2, 3, 4, 5, 12, 15, },
  { 1, 2, 3, 4, 5, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 12, 15, },
  { 6, 12, 15, },
  { 0, 6, 12, 15, },
  { 1, 6, 12, 15, },
  { 0, 1, 6, 12, 15, },
  { 2, 6, 12, 15, },
  { 0, 2, 6, 12, 15, },
  { 1, 2, 6, 12, 15, },
  { 0, 1, 2, 6, 12, 15, },
  { 3, 6, 12, 15, },
  { 0, 3, 6, 12, 15, },
  { 1, 3, 6, 12, 15, },
  { 0, 1, 3, 6, 12, 15, },
  { 2, 3, 6, 12, 15, },
  { 0, 2, 3, 6, 12, 15, },
  { 1, 2, 3, 6, 12, 15, },
  { 0, 1, 2, 3, 6, 12, 15, },
  { 4, 6, 12, 15, },
  { 0, 4, 6, 12, 15, },
  { 1, 4, 6, 12, 15, },
  { 0, 1, 4, 6, 12, 15, },
  { 2, 4, 6, 12, 15, },
  { 0, 2, 4, 6, 12, 15, },
  { 1, 2, 4, 6, 12, 15, },
  { 0, 1, 2, 4, 6, 12, 15, },
  { 3, 4, 6, 12, 15, },
  { 0, 3, 4, 6, 12, 15, },
  { 1, 3, 4, 6, 12, 15, },
  { 0, 1, 3, 4, 6, 12, 15, },
  { 2, 3, 4, 6, 12, 15, },
  { 0, 2, 3, 4, 6, 12, 15, },
  { 1, 2, 3, 4, 6, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 12, 15, },
  { 5, 6, 12, 15, },
  { 0, 5, 6, 12, 15, },
  { 1, 5, 6, 12, 15, },
  { 0, 1, 5, 6, 12, 15, },
  { 2, 5, 6, 12, 15, },
  { 0, 2, 5, 6, 12, 15, },
  { 1, 2, 5, 6, 12, 15, },
  { 0, 1, 2, 5, 6, 12, 15, },
  { 3, 5, 6, 12, 15, },
  { 0, 3, 5, 6, 12, 15, },
  { 1, 3, 5, 6, 12, 15, },
  { 0, 1, 3, 5, 6, 12, 15, },
  { 2, 3, 5, 6, 12, 15, },
  { 0, 2, 3, 5, 6, 12, 15, },
  { 1, 2, 3, 5, 6, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 12, 15, },
  { 4, 5, 6, 12, 15, },
  { 0, 4, 5, 6, 12, 15, },
  { 1, 4, 5, 6, 12, 15, },
  { 0, 1, 4, 5, 6, 12, 15, },
  { 2, 4, 5, 6, 12, 15, },
  { 0, 2, 4, 5, 6, 12, 15, },
  { 1, 2, 4, 5, 6, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 12, 15, },
  { 3, 4, 5, 6, 12, 15, },
  { 0, 3, 4, 5, 6, 12, 15, },
  { 1, 3, 4, 5, 6, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 12, 15, },
  { 2, 3, 4, 5, 6, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 12, 15, },
  { 7, 12, 15, },
  { 0, 7, 12, 15, },
  { 1, 7, 12, 15, },
  { 0, 1, 7, 12, 15, },
  { 2, 7, 12, 15, },
  { 0, 2, 7, 12, 15, },
  { 1, 2, 7, 12, 15, },
  { 0, 1, 2, 7, 12, 15, },
  { 3, 7, 12, 15, },
  { 0, 3, 7, 12, 15, },
  { 1, 3, 7, 12, 15, },
  { 0, 1, 3, 7, 12, 15, },
  { 2, 3, 7, 12, 15, },
  { 0, 2, 3, 7, 12, 15, },
  { 1, 2, 3, 7, 12, 15, },
  { 0, 1, 2, 3, 7, 12, 15, },
  { 4, 7, 12, 15, },
  { 0, 4, 7, 12, 15, },
  { 1, 4, 7, 12, 15, },
  { 0, 1, 4, 7, 12, 15, },
  { 2, 4, 7, 12, 15, },
  { 0, 2, 4, 7, 12, 15, },
  { 1, 2, 4, 7, 12, 15, },
  { 0, 1, 2, 4, 7, 12, 15, },
  { 3, 4, 7, 12, 15, },
  { 0, 3, 4, 7, 12, 15, },
  { 1, 3, 4, 7, 12, 15, },
  { 0, 1, 3, 4, 7, 12, 15, },
  { 2, 3, 4, 7, 12, 15, },
  { 0, 2, 3, 4, 7, 12, 15, },
  { 1, 2, 3, 4, 7, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 12, 15, },
  { 5, 7, 12, 15, },
  { 0, 5, 7, 12, 15, },
  { 1, 5, 7, 12, 15, },
  { 0, 1, 5, 7, 12, 15, },
  { 2, 5, 7, 12, 15, },
  { 0, 2, 5, 7, 12, 15, },
  { 1, 2, 5, 7, 12, 15, },
  { 0, 1, 2, 5, 7, 12, 15, },
  { 3, 5, 7, 12, 15, },
  { 0, 3, 5, 7, 12, 15, },
  { 1, 3, 5, 7, 12, 15, },
  { 0, 1, 3, 5, 7, 12, 15, },
  { 2, 3, 5, 7, 12, 15, },
  { 0, 2, 3, 5, 7, 12, 15, },
  { 1, 2, 3, 5, 7, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 12, 15, },
  { 4, 5, 7, 12, 15, },
  { 0, 4, 5, 7, 12, 15, },
  { 1, 4, 5, 7, 12, 15, },
  { 0, 1, 4, 5, 7, 12, 15, },
  { 2, 4, 5, 7, 12, 15, },
  { 0, 2, 4, 5, 7, 12, 15, },
  { 1, 2, 4, 5, 7, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 12, 15, },
  { 3, 4, 5, 7, 12, 15, },
  { 0, 3, 4, 5, 7, 12, 15, },
  { 1, 3, 4, 5, 7, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 12, 15, },
  { 2, 3, 4, 5, 7, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 12, 15, },
  { 6, 7, 12, 15, },
  { 0, 6, 7, 12, 15, },
  { 1, 6, 7, 12, 15, },
  { 0, 1, 6, 7, 12, 15, },
  { 2, 6, 7, 12, 15, },
  { 0, 2, 6, 7, 12, 15, },
  { 1, 2, 6, 7, 12, 15, },
  { 0, 1, 2, 6, 7, 12, 15, },
  { 3, 6, 7, 12, 15, },
  { 0, 3, 6, 7, 12, 15, },
  { 1, 3, 6, 7, 12, 15, },
  { 0, 1, 3, 6, 7, 12, 15, },
  { 2, 3, 6, 7, 12, 15, },
  { 0, 2, 3, 6, 7, 12, 15, },
  { 1, 2, 3, 6, 7, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 12, 15, },
  { 4, 6, 7, 12, 15, },
  { 0, 4, 6, 7, 12, 15, },
  { 1, 4, 6, 7, 12, 15, },
  { 0, 1, 4, 6, 7, 12, 15, },
  { 2, 4, 6, 7, 12, 15, },
  { 0, 2, 4, 6, 7, 12, 15, },
  { 1, 2, 4, 6, 7, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 12, 15, },
  { 3, 4, 6, 7, 12, 15, },
  { 0, 3, 4, 6, 7, 12, 15, },
  { 1, 3, 4, 6, 7, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 12, 15, },
  { 2, 3, 4, 6, 7, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 12, 15, },
  { 5, 6, 7, 12, 15, },
  { 0, 5, 6, 7, 12, 15, },
  { 1, 5, 6, 7, 12, 15, },
  { 0, 1, 5, 6, 7, 12, 15, },
  { 2, 5, 6, 7, 12, 15, },
  { 0, 2, 5, 6, 7, 12, 15, },
  { 1, 2, 5, 6, 7, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 12, 15, },
  { 3, 5, 6, 7, 12, 15, },
  { 0, 3, 5, 6, 7, 12, 15, },
  { 1, 3, 5, 6, 7, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 12, 15, },
  { 2, 3, 5, 6, 7, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 12, 15, },
  { 4, 5, 6, 7, 12, 15, },
  { 0, 4, 5, 6, 7, 12, 15, },
  { 1, 4, 5, 6, 7, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 12, 15, },
  { 2, 4, 5, 6, 7, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 12, 15, },
  { 3, 4, 5, 6, 7, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 12, 15, },
  { 8, 12, 15, },
  { 0, 8, 12, 15, },
  { 1, 8, 12, 15, },
  { 0, 1, 8, 12, 15, },
  { 2, 8, 12, 15, },
  { 0, 2, 8, 12, 15, },
  { 1, 2, 8, 12, 15, },
  { 0, 1, 2, 8, 12, 15, },
  { 3, 8, 12, 15, },
  { 0, 3, 8, 12, 15, },
  { 1, 3, 8, 12, 15, },
  { 0, 1, 3, 8, 12, 15, },
  { 2, 3, 8, 12, 15, },
  { 0, 2, 3, 8, 12, 15, },
  { 1, 2, 3, 8, 12, 15, },
  { 0, 1, 2, 3, 8, 12, 15, },
  { 4, 8, 12, 15, },
  { 0, 4, 8, 12, 15, },
  { 1, 4, 8, 12, 15, },
  { 0, 1, 4, 8, 12, 15, },
  { 2, 4, 8, 12, 15, },
  { 0, 2, 4, 8, 12, 15, },
  { 1, 2, 4, 8, 12, 15, },
  { 0, 1, 2, 4, 8, 12, 15, },
  { 3, 4, 8, 12, 15, },
  { 0, 3, 4, 8, 12, 15, },
  { 1, 3, 4, 8, 12, 15, },
  { 0, 1, 3, 4, 8, 12, 15, },
  { 2, 3, 4, 8, 12, 15, },
  { 0, 2, 3, 4, 8, 12, 15, },
  { 1, 2, 3, 4, 8, 12, 15, },
  { 0, 1, 2, 3, 4, 8, 12, 15, },
  { 5, 8, 12, 15, },
  { 0, 5, 8, 12, 15, },
  { 1, 5, 8, 12, 15, },
  { 0, 1, 5, 8, 12, 15, },
  { 2, 5, 8, 12, 15, },
  { 0, 2, 5, 8, 12, 15, },
  { 1, 2, 5, 8, 12, 15, },
  { 0, 1, 2, 5, 8, 12, 15, },
  { 3, 5, 8, 12, 15, },
  { 0, 3, 5, 8, 12, 15, },
  { 1, 3, 5, 8, 12, 15, },
  { 0, 1, 3, 5, 8, 12, 15, },
  { 2, 3, 5, 8, 12, 15, },
  { 0, 2, 3, 5, 8, 12, 15, },
  { 1, 2, 3, 5, 8, 12, 15, },
  { 0, 1, 2, 3, 5, 8, 12, 15, },
  { 4, 5, 8, 12, 15, },
  { 0, 4, 5, 8, 12, 15, },
  { 1, 4, 5, 8, 12, 15, },
  { 0, 1, 4, 5, 8, 12, 15, },
  { 2, 4, 5, 8, 12, 15, },
  { 0, 2, 4, 5, 8, 12, 15, },
  { 1, 2, 4, 5, 8, 12, 15, },
  { 0, 1, 2, 4, 5, 8, 12, 15, },
  { 3, 4, 5, 8, 12, 15, },
  { 0, 3, 4, 5, 8, 12, 15, },
  { 1, 3, 4, 5, 8, 12, 15, },
  { 0, 1, 3, 4, 5, 8, 12, 15, },
  { 2, 3, 4, 5, 8, 12, 15, },
  { 0, 2, 3, 4, 5, 8, 12, 15, },
  { 1, 2, 3, 4, 5, 8, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 12, 15, },
  { 6, 8, 12, 15, },
  { 0, 6, 8, 12, 15, },
  { 1, 6, 8, 12, 15, },
  { 0, 1, 6, 8, 12, 15, },
  { 2, 6, 8, 12, 15, },
  { 0, 2, 6, 8, 12, 15, },
  { 1, 2, 6, 8, 12, 15, },
  { 0, 1, 2, 6, 8, 12, 15, },
  { 3, 6, 8, 12, 15, },
  { 0, 3, 6, 8, 12, 15, },
  { 1, 3, 6, 8, 12, 15, },
  { 0, 1, 3, 6, 8, 12, 15, },
  { 2, 3, 6, 8, 12, 15, },
  { 0, 2, 3, 6, 8, 12, 15, },
  { 1, 2, 3, 6, 8, 12, 15, },
  { 0, 1, 2, 3, 6, 8, 12, 15, },
  { 4, 6, 8, 12, 15, },
  { 0, 4, 6, 8, 12, 15, },
  { 1, 4, 6, 8, 12, 15, },
  { 0, 1, 4, 6, 8, 12, 15, },
  { 2, 4, 6, 8, 12, 15, },
  { 0, 2, 4, 6, 8, 12, 15, },
  { 1, 2, 4, 6, 8, 12, 15, },
  { 0, 1, 2, 4, 6, 8, 12, 15, },
  { 3, 4, 6, 8, 12, 15, },
  { 0, 3, 4, 6, 8, 12, 15, },
  { 1, 3, 4, 6, 8, 12, 15, },
  { 0, 1, 3, 4, 6, 8, 12, 15, },
  { 2, 3, 4, 6, 8, 12, 15, },
  { 0, 2, 3, 4, 6, 8, 12, 15, },
  { 1, 2, 3, 4, 6, 8, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 12, 15, },
  { 5, 6, 8, 12, 15, },
  { 0, 5, 6, 8, 12, 15, },
  { 1, 5, 6, 8, 12, 15, },
  { 0, 1, 5, 6, 8, 12, 15, },
  { 2, 5, 6, 8, 12, 15, },
  { 0, 2, 5, 6, 8, 12, 15, },
  { 1, 2, 5, 6, 8, 12, 15, },
  { 0, 1, 2, 5, 6, 8, 12, 15, },
  { 3, 5, 6, 8, 12, 15, },
  { 0, 3, 5, 6, 8, 12, 15, },
  { 1, 3, 5, 6, 8, 12, 15, },
  { 0, 1, 3, 5, 6, 8, 12, 15, },
  { 2, 3, 5, 6, 8, 12, 15, },
  { 0, 2, 3, 5, 6, 8, 12, 15, },
  { 1, 2, 3, 5, 6, 8, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 12, 15, },
  { 4, 5, 6, 8, 12, 15, },
  { 0, 4, 5, 6, 8, 12, 15, },
  { 1, 4, 5, 6, 8, 12, 15, },
  { 0, 1, 4, 5, 6, 8, 12, 15, },
  { 2, 4, 5, 6, 8, 12, 15, },
  { 0, 2, 4, 5, 6, 8, 12, 15, },
  { 1, 2, 4, 5, 6, 8, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 12, 15, },
  { 3, 4, 5, 6, 8, 12, 15, },
  { 0, 3, 4, 5, 6, 8, 12, 15, },
  { 1, 3, 4, 5, 6, 8, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 12, 15, },
  { 2, 3, 4, 5, 6, 8, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 12, 15, },
  { 7, 8, 12, 15, },
  { 0, 7, 8, 12, 15, },
  { 1, 7, 8, 12, 15, },
  { 0, 1, 7, 8, 12, 15, },
  { 2, 7, 8, 12, 15, },
  { 0, 2, 7, 8, 12, 15, },
  { 1, 2, 7, 8, 12, 15, },
  { 0, 1, 2, 7, 8, 12, 15, },
  { 3, 7, 8, 12, 15, },
  { 0, 3, 7, 8, 12, 15, },
  { 1, 3, 7, 8, 12, 15, },
  { 0, 1, 3, 7, 8, 12, 15, },
  { 2, 3, 7, 8, 12, 15, },
  { 0, 2, 3, 7, 8, 12, 15, },
  { 1, 2, 3, 7, 8, 12, 15, },
  { 0, 1, 2, 3, 7, 8, 12, 15, },
  { 4, 7, 8, 12, 15, },
  { 0, 4, 7, 8, 12, 15, },
  { 1, 4, 7, 8, 12, 15, },
  { 0, 1, 4, 7, 8, 12, 15, },
  { 2, 4, 7, 8, 12, 15, },
  { 0, 2, 4, 7, 8, 12, 15, },
  { 1, 2, 4, 7, 8, 12, 15, },
  { 0, 1, 2, 4, 7, 8, 12, 15, },
  { 3, 4, 7, 8, 12, 15, },
  { 0, 3, 4, 7, 8, 12, 15, },
  { 1, 3, 4, 7, 8, 12, 15, },
  { 0, 1, 3, 4, 7, 8, 12, 15, },
  { 2, 3, 4, 7, 8, 12, 15, },
  { 0, 2, 3, 4, 7, 8, 12, 15, },
  { 1, 2, 3, 4, 7, 8, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 12, 15, },
  { 5, 7, 8, 12, 15, },
  { 0, 5, 7, 8, 12, 15, },
  { 1, 5, 7, 8, 12, 15, },
  { 0, 1, 5, 7, 8, 12, 15, },
  { 2, 5, 7, 8, 12, 15, },
  { 0, 2, 5, 7, 8, 12, 15, },
  { 1, 2, 5, 7, 8, 12, 15, },
  { 0, 1, 2, 5, 7, 8, 12, 15, },
  { 3, 5, 7, 8, 12, 15, },
  { 0, 3, 5, 7, 8, 12, 15, },
  { 1, 3, 5, 7, 8, 12, 15, },
  { 0, 1, 3, 5, 7, 8, 12, 15, },
  { 2, 3, 5, 7, 8, 12, 15, },
  { 0, 2, 3, 5, 7, 8, 12, 15, },
  { 1, 2, 3, 5, 7, 8, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 12, 15, },
  { 4, 5, 7, 8, 12, 15, },
  { 0, 4, 5, 7, 8, 12, 15, },
  { 1, 4, 5, 7, 8, 12, 15, },
  { 0, 1, 4, 5, 7, 8, 12, 15, },
  { 2, 4, 5, 7, 8, 12, 15, },
  { 0, 2, 4, 5, 7, 8, 12, 15, },
  { 1, 2, 4, 5, 7, 8, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 12, 15, },
  { 3, 4, 5, 7, 8, 12, 15, },
  { 0, 3, 4, 5, 7, 8, 12, 15, },
  { 1, 3, 4, 5, 7, 8, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 12, 15, },
  { 2, 3, 4, 5, 7, 8, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 12, 15, },
  { 6, 7, 8, 12, 15, },
  { 0, 6, 7, 8, 12, 15, },
  { 1, 6, 7, 8, 12, 15, },
  { 0, 1, 6, 7, 8, 12, 15, },
  { 2, 6, 7, 8, 12, 15, },
  { 0, 2, 6, 7, 8, 12, 15, },
  { 1, 2, 6, 7, 8, 12, 15, },
  { 0, 1, 2, 6, 7, 8, 12, 15, },
  { 3, 6, 7, 8, 12, 15, },
  { 0, 3, 6, 7, 8, 12, 15, },
  { 1, 3, 6, 7, 8, 12, 15, },
  { 0, 1, 3, 6, 7, 8, 12, 15, },
  { 2, 3, 6, 7, 8, 12, 15, },
  { 0, 2, 3, 6, 7, 8, 12, 15, },
  { 1, 2, 3, 6, 7, 8, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 12, 15, },
  { 4, 6, 7, 8, 12, 15, },
  { 0, 4, 6, 7, 8, 12, 15, },
  { 1, 4, 6, 7, 8, 12, 15, },
  { 0, 1, 4, 6, 7, 8, 12, 15, },
  { 2, 4, 6, 7, 8, 12, 15, },
  { 0, 2, 4, 6, 7, 8, 12, 15, },
  { 1, 2, 4, 6, 7, 8, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 12, 15, },
  { 3, 4, 6, 7, 8, 12, 15, },
  { 0, 3, 4, 6, 7, 8, 12, 15, },
  { 1, 3, 4, 6, 7, 8, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 12, 15, },
  { 2, 3, 4, 6, 7, 8, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 12, 15, },
  { 5, 6, 7, 8, 12, 15, },
  { 0, 5, 6, 7, 8, 12, 15, },
  { 1, 5, 6, 7, 8, 12, 15, },
  { 0, 1, 5, 6, 7, 8, 12, 15, },
  { 2, 5, 6, 7, 8, 12, 15, },
  { 0, 2, 5, 6, 7, 8, 12, 15, },
  { 1, 2, 5, 6, 7, 8, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 12, 15, },
  { 3, 5, 6, 7, 8, 12, 15, },
  { 0, 3, 5, 6, 7, 8, 12, 15, },
  { 1, 3, 5, 6, 7, 8, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 12, 15, },
  { 2, 3, 5, 6, 7, 8, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 12, 15, },
  { 4, 5, 6, 7, 8, 12, 15, },
  { 0, 4, 5, 6, 7, 8, 12, 15, },
  { 1, 4, 5, 6, 7, 8, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 12, 15, },
  { 2, 4, 5, 6, 7, 8, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 12, 15, },
  { 3, 4, 5, 6, 7, 8, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 12, 15, },
  { 9, 12, 15, },
  { 0, 9, 12, 15, },
  { 1, 9, 12, 15, },
  { 0, 1, 9, 12, 15, },
  { 2, 9, 12, 15, },
  { 0, 2, 9, 12, 15, },
  { 1, 2, 9, 12, 15, },
  { 0, 1, 2, 9, 12, 15, },
  { 3, 9, 12, 15, },
  { 0, 3, 9, 12, 15, },
  { 1, 3, 9, 12, 15, },
  { 0, 1, 3, 9, 12, 15, },
  { 2, 3, 9, 12, 15, },
  { 0, 2, 3, 9, 12, 15, },
  { 1, 2, 3, 9, 12, 15, },
  { 0, 1, 2, 3, 9, 12, 15, },
  { 4, 9, 12, 15, },
  { 0, 4, 9, 12, 15, },
  { 1, 4, 9, 12, 15, },
  { 0, 1, 4, 9, 12, 15, },
  { 2, 4, 9, 12, 15, },
  { 0, 2, 4, 9, 12, 15, },
  { 1, 2, 4, 9, 12, 15, },
  { 0, 1, 2, 4, 9, 12, 15, },
  { 3, 4, 9, 12, 15, },
  { 0, 3, 4, 9, 12, 15, },
  { 1, 3, 4, 9, 12, 15, },
  { 0, 1, 3, 4, 9, 12, 15, },
  { 2, 3, 4, 9, 12, 15, },
  { 0, 2, 3, 4, 9, 12, 15, },
  { 1, 2, 3, 4, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 9, 12, 15, },
  { 5, 9, 12, 15, },
  { 0, 5, 9, 12, 15, },
  { 1, 5, 9, 12, 15, },
  { 0, 1, 5, 9, 12, 15, },
  { 2, 5, 9, 12, 15, },
  { 0, 2, 5, 9, 12, 15, },
  { 1, 2, 5, 9, 12, 15, },
  { 0, 1, 2, 5, 9, 12, 15, },
  { 3, 5, 9, 12, 15, },
  { 0, 3, 5, 9, 12, 15, },
  { 1, 3, 5, 9, 12, 15, },
  { 0, 1, 3, 5, 9, 12, 15, },
  { 2, 3, 5, 9, 12, 15, },
  { 0, 2, 3, 5, 9, 12, 15, },
  { 1, 2, 3, 5, 9, 12, 15, },
  { 0, 1, 2, 3, 5, 9, 12, 15, },
  { 4, 5, 9, 12, 15, },
  { 0, 4, 5, 9, 12, 15, },
  { 1, 4, 5, 9, 12, 15, },
  { 0, 1, 4, 5, 9, 12, 15, },
  { 2, 4, 5, 9, 12, 15, },
  { 0, 2, 4, 5, 9, 12, 15, },
  { 1, 2, 4, 5, 9, 12, 15, },
  { 0, 1, 2, 4, 5, 9, 12, 15, },
  { 3, 4, 5, 9, 12, 15, },
  { 0, 3, 4, 5, 9, 12, 15, },
  { 1, 3, 4, 5, 9, 12, 15, },
  { 0, 1, 3, 4, 5, 9, 12, 15, },
  { 2, 3, 4, 5, 9, 12, 15, },
  { 0, 2, 3, 4, 5, 9, 12, 15, },
  { 1, 2, 3, 4, 5, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 12, 15, },
  { 6, 9, 12, 15, },
  { 0, 6, 9, 12, 15, },
  { 1, 6, 9, 12, 15, },
  { 0, 1, 6, 9, 12, 15, },
  { 2, 6, 9, 12, 15, },
  { 0, 2, 6, 9, 12, 15, },
  { 1, 2, 6, 9, 12, 15, },
  { 0, 1, 2, 6, 9, 12, 15, },
  { 3, 6, 9, 12, 15, },
  { 0, 3, 6, 9, 12, 15, },
  { 1, 3, 6, 9, 12, 15, },
  { 0, 1, 3, 6, 9, 12, 15, },
  { 2, 3, 6, 9, 12, 15, },
  { 0, 2, 3, 6, 9, 12, 15, },
  { 1, 2, 3, 6, 9, 12, 15, },
  { 0, 1, 2, 3, 6, 9, 12, 15, },
  { 4, 6, 9, 12, 15, },
  { 0, 4, 6, 9, 12, 15, },
  { 1, 4, 6, 9, 12, 15, },
  { 0, 1, 4, 6, 9, 12, 15, },
  { 2, 4, 6, 9, 12, 15, },
  { 0, 2, 4, 6, 9, 12, 15, },
  { 1, 2, 4, 6, 9, 12, 15, },
  { 0, 1, 2, 4, 6, 9, 12, 15, },
  { 3, 4, 6, 9, 12, 15, },
  { 0, 3, 4, 6, 9, 12, 15, },
  { 1, 3, 4, 6, 9, 12, 15, },
  { 0, 1, 3, 4, 6, 9, 12, 15, },
  { 2, 3, 4, 6, 9, 12, 15, },
  { 0, 2, 3, 4, 6, 9, 12, 15, },
  { 1, 2, 3, 4, 6, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 12, 15, },
  { 5, 6, 9, 12, 15, },
  { 0, 5, 6, 9, 12, 15, },
  { 1, 5, 6, 9, 12, 15, },
  { 0, 1, 5, 6, 9, 12, 15, },
  { 2, 5, 6, 9, 12, 15, },
  { 0, 2, 5, 6, 9, 12, 15, },
  { 1, 2, 5, 6, 9, 12, 15, },
  { 0, 1, 2, 5, 6, 9, 12, 15, },
  { 3, 5, 6, 9, 12, 15, },
  { 0, 3, 5, 6, 9, 12, 15, },
  { 1, 3, 5, 6, 9, 12, 15, },
  { 0, 1, 3, 5, 6, 9, 12, 15, },
  { 2, 3, 5, 6, 9, 12, 15, },
  { 0, 2, 3, 5, 6, 9, 12, 15, },
  { 1, 2, 3, 5, 6, 9, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 12, 15, },
  { 4, 5, 6, 9, 12, 15, },
  { 0, 4, 5, 6, 9, 12, 15, },
  { 1, 4, 5, 6, 9, 12, 15, },
  { 0, 1, 4, 5, 6, 9, 12, 15, },
  { 2, 4, 5, 6, 9, 12, 15, },
  { 0, 2, 4, 5, 6, 9, 12, 15, },
  { 1, 2, 4, 5, 6, 9, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 12, 15, },
  { 3, 4, 5, 6, 9, 12, 15, },
  { 0, 3, 4, 5, 6, 9, 12, 15, },
  { 1, 3, 4, 5, 6, 9, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 12, 15, },
  { 2, 3, 4, 5, 6, 9, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 12, 15, },
  { 7, 9, 12, 15, },
  { 0, 7, 9, 12, 15, },
  { 1, 7, 9, 12, 15, },
  { 0, 1, 7, 9, 12, 15, },
  { 2, 7, 9, 12, 15, },
  { 0, 2, 7, 9, 12, 15, },
  { 1, 2, 7, 9, 12, 15, },
  { 0, 1, 2, 7, 9, 12, 15, },
  { 3, 7, 9, 12, 15, },
  { 0, 3, 7, 9, 12, 15, },
  { 1, 3, 7, 9, 12, 15, },
  { 0, 1, 3, 7, 9, 12, 15, },
  { 2, 3, 7, 9, 12, 15, },
  { 0, 2, 3, 7, 9, 12, 15, },
  { 1, 2, 3, 7, 9, 12, 15, },
  { 0, 1, 2, 3, 7, 9, 12, 15, },
  { 4, 7, 9, 12, 15, },
  { 0, 4, 7, 9, 12, 15, },
  { 1, 4, 7, 9, 12, 15, },
  { 0, 1, 4, 7, 9, 12, 15, },
  { 2, 4, 7, 9, 12, 15, },
  { 0, 2, 4, 7, 9, 12, 15, },
  { 1, 2, 4, 7, 9, 12, 15, },
  { 0, 1, 2, 4, 7, 9, 12, 15, },
  { 3, 4, 7, 9, 12, 15, },
  { 0, 3, 4, 7, 9, 12, 15, },
  { 1, 3, 4, 7, 9, 12, 15, },
  { 0, 1, 3, 4, 7, 9, 12, 15, },
  { 2, 3, 4, 7, 9, 12, 15, },
  { 0, 2, 3, 4, 7, 9, 12, 15, },
  { 1, 2, 3, 4, 7, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 12, 15, },
  { 5, 7, 9, 12, 15, },
  { 0, 5, 7, 9, 12, 15, },
  { 1, 5, 7, 9, 12, 15, },
  { 0, 1, 5, 7, 9, 12, 15, },
  { 2, 5, 7, 9, 12, 15, },
  { 0, 2, 5, 7, 9, 12, 15, },
  { 1, 2, 5, 7, 9, 12, 15, },
  { 0, 1, 2, 5, 7, 9, 12, 15, },
  { 3, 5, 7, 9, 12, 15, },
  { 0, 3, 5, 7, 9, 12, 15, },
  { 1, 3, 5, 7, 9, 12, 15, },
  { 0, 1, 3, 5, 7, 9, 12, 15, },
  { 2, 3, 5, 7, 9, 12, 15, },
  { 0, 2, 3, 5, 7, 9, 12, 15, },
  { 1, 2, 3, 5, 7, 9, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 12, 15, },
  { 4, 5, 7, 9, 12, 15, },
  { 0, 4, 5, 7, 9, 12, 15, },
  { 1, 4, 5, 7, 9, 12, 15, },
  { 0, 1, 4, 5, 7, 9, 12, 15, },
  { 2, 4, 5, 7, 9, 12, 15, },
  { 0, 2, 4, 5, 7, 9, 12, 15, },
  { 1, 2, 4, 5, 7, 9, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 12, 15, },
  { 3, 4, 5, 7, 9, 12, 15, },
  { 0, 3, 4, 5, 7, 9, 12, 15, },
  { 1, 3, 4, 5, 7, 9, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 12, 15, },
  { 2, 3, 4, 5, 7, 9, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 12, 15, },
  { 6, 7, 9, 12, 15, },
  { 0, 6, 7, 9, 12, 15, },
  { 1, 6, 7, 9, 12, 15, },
  { 0, 1, 6, 7, 9, 12, 15, },
  { 2, 6, 7, 9, 12, 15, },
  { 0, 2, 6, 7, 9, 12, 15, },
  { 1, 2, 6, 7, 9, 12, 15, },
  { 0, 1, 2, 6, 7, 9, 12, 15, },
  { 3, 6, 7, 9, 12, 15, },
  { 0, 3, 6, 7, 9, 12, 15, },
  { 1, 3, 6, 7, 9, 12, 15, },
  { 0, 1, 3, 6, 7, 9, 12, 15, },
  { 2, 3, 6, 7, 9, 12, 15, },
  { 0, 2, 3, 6, 7, 9, 12, 15, },
  { 1, 2, 3, 6, 7, 9, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 12, 15, },
  { 4, 6, 7, 9, 12, 15, },
  { 0, 4, 6, 7, 9, 12, 15, },
  { 1, 4, 6, 7, 9, 12, 15, },
  { 0, 1, 4, 6, 7, 9, 12, 15, },
  { 2, 4, 6, 7, 9, 12, 15, },
  { 0, 2, 4, 6, 7, 9, 12, 15, },
  { 1, 2, 4, 6, 7, 9, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 12, 15, },
  { 3, 4, 6, 7, 9, 12, 15, },
  { 0, 3, 4, 6, 7, 9, 12, 15, },
  { 1, 3, 4, 6, 7, 9, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 12, 15, },
  { 2, 3, 4, 6, 7, 9, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 12, 15, },
  { 5, 6, 7, 9, 12, 15, },
  { 0, 5, 6, 7, 9, 12, 15, },
  { 1, 5, 6, 7, 9, 12, 15, },
  { 0, 1, 5, 6, 7, 9, 12, 15, },
  { 2, 5, 6, 7, 9, 12, 15, },
  { 0, 2, 5, 6, 7, 9, 12, 15, },
  { 1, 2, 5, 6, 7, 9, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 12, 15, },
  { 3, 5, 6, 7, 9, 12, 15, },
  { 0, 3, 5, 6, 7, 9, 12, 15, },
  { 1, 3, 5, 6, 7, 9, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 12, 15, },
  { 2, 3, 5, 6, 7, 9, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 12, 15, },
  { 4, 5, 6, 7, 9, 12, 15, },
  { 0, 4, 5, 6, 7, 9, 12, 15, },
  { 1, 4, 5, 6, 7, 9, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 12, 15, },
  { 2, 4, 5, 6, 7, 9, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 12, 15, },
  { 3, 4, 5, 6, 7, 9, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 12, 15, },
  { 8, 9, 12, 15, },
  { 0, 8, 9, 12, 15, },
  { 1, 8, 9, 12, 15, },
  { 0, 1, 8, 9, 12, 15, },
  { 2, 8, 9, 12, 15, },
  { 0, 2, 8, 9, 12, 15, },
  { 1, 2, 8, 9, 12, 15, },
  { 0, 1, 2, 8, 9, 12, 15, },
  { 3, 8, 9, 12, 15, },
  { 0, 3, 8, 9, 12, 15, },
  { 1, 3, 8, 9, 12, 15, },
  { 0, 1, 3, 8, 9, 12, 15, },
  { 2, 3, 8, 9, 12, 15, },
  { 0, 2, 3, 8, 9, 12, 15, },
  { 1, 2, 3, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 8, 9, 12, 15, },
  { 4, 8, 9, 12, 15, },
  { 0, 4, 8, 9, 12, 15, },
  { 1, 4, 8, 9, 12, 15, },
  { 0, 1, 4, 8, 9, 12, 15, },
  { 2, 4, 8, 9, 12, 15, },
  { 0, 2, 4, 8, 9, 12, 15, },
  { 1, 2, 4, 8, 9, 12, 15, },
  { 0, 1, 2, 4, 8, 9, 12, 15, },
  { 3, 4, 8, 9, 12, 15, },
  { 0, 3, 4, 8, 9, 12, 15, },
  { 1, 3, 4, 8, 9, 12, 15, },
  { 0, 1, 3, 4, 8, 9, 12, 15, },
  { 2, 3, 4, 8, 9, 12, 15, },
  { 0, 2, 3, 4, 8, 9, 12, 15, },
  { 1, 2, 3, 4, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 12, 15, },
  { 5, 8, 9, 12, 15, },
  { 0, 5, 8, 9, 12, 15, },
  { 1, 5, 8, 9, 12, 15, },
  { 0, 1, 5, 8, 9, 12, 15, },
  { 2, 5, 8, 9, 12, 15, },
  { 0, 2, 5, 8, 9, 12, 15, },
  { 1, 2, 5, 8, 9, 12, 15, },
  { 0, 1, 2, 5, 8, 9, 12, 15, },
  { 3, 5, 8, 9, 12, 15, },
  { 0, 3, 5, 8, 9, 12, 15, },
  { 1, 3, 5, 8, 9, 12, 15, },
  { 0, 1, 3, 5, 8, 9, 12, 15, },
  { 2, 3, 5, 8, 9, 12, 15, },
  { 0, 2, 3, 5, 8, 9, 12, 15, },
  { 1, 2, 3, 5, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 12, 15, },
  { 4, 5, 8, 9, 12, 15, },
  { 0, 4, 5, 8, 9, 12, 15, },
  { 1, 4, 5, 8, 9, 12, 15, },
  { 0, 1, 4, 5, 8, 9, 12, 15, },
  { 2, 4, 5, 8, 9, 12, 15, },
  { 0, 2, 4, 5, 8, 9, 12, 15, },
  { 1, 2, 4, 5, 8, 9, 12, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 12, 15, },
  { 3, 4, 5, 8, 9, 12, 15, },
  { 0, 3, 4, 5, 8, 9, 12, 15, },
  { 1, 3, 4, 5, 8, 9, 12, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 12, 15, },
  { 2, 3, 4, 5, 8, 9, 12, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 12, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 12, 15, },
  { 6, 8, 9, 12, 15, },
  { 0, 6, 8, 9, 12, 15, },
  { 1, 6, 8, 9, 12, 15, },
  { 0, 1, 6, 8, 9, 12, 15, },
  { 2, 6, 8, 9, 12, 15, },
  { 0, 2, 6, 8, 9, 12, 15, },
  { 1, 2, 6, 8, 9, 12, 15, },
  { 0, 1, 2, 6, 8, 9, 12, 15, },
  { 3, 6, 8, 9, 12, 15, },
  { 0, 3, 6, 8, 9, 12, 15, },
  { 1, 3, 6, 8, 9, 12, 15, },
  { 0, 1, 3, 6, 8, 9, 12, 15, },
  { 2, 3, 6, 8, 9, 12, 15, },
  { 0, 2, 3, 6, 8, 9, 12, 15, },
  { 1, 2, 3, 6, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 12, 15, },
  { 4, 6, 8, 9, 12, 15, },
  { 0, 4, 6, 8, 9, 12, 15, },
  { 1, 4, 6, 8, 9, 12, 15, },
  { 0, 1, 4, 6, 8, 9, 12, 15, },
  { 2, 4, 6, 8, 9, 12, 15, },
  { 0, 2, 4, 6, 8, 9, 12, 15, },
  { 1, 2, 4, 6, 8, 9, 12, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 12, 15, },
  { 3, 4, 6, 8, 9, 12, 15, },
  { 0, 3, 4, 6, 8, 9, 12, 15, },
  { 1, 3, 4, 6, 8, 9, 12, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 12, 15, },
  { 2, 3, 4, 6, 8, 9, 12, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 12, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 12, 15, },
  { 5, 6, 8, 9, 12, 15, },
  { 0, 5, 6, 8, 9, 12, 15, },
  { 1, 5, 6, 8, 9, 12, 15, },
  { 0, 1, 5, 6, 8, 9, 12, 15, },
  { 2, 5, 6, 8, 9, 12, 15, },
  { 0, 2, 5, 6, 8, 9, 12, 15, },
  { 1, 2, 5, 6, 8, 9, 12, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 12, 15, },
  { 3, 5, 6, 8, 9, 12, 15, },
  { 0, 3, 5, 6, 8, 9, 12, 15, },
  { 1, 3, 5, 6, 8, 9, 12, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 12, 15, },
  { 2, 3, 5, 6, 8, 9, 12, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 12, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 12, 15, },
  { 4, 5, 6, 8, 9, 12, 15, },
  { 0, 4, 5, 6, 8, 9, 12, 15, },
  { 1, 4, 5, 6, 8, 9, 12, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 12, 15, },
  { 2, 4, 5, 6, 8, 9, 12, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 12, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 12, 15, },
  { 3, 4, 5, 6, 8, 9, 12, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 12, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 12, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 12, 15, },
  { 7, 8, 9, 12, 15, },
  { 0, 7, 8, 9, 12, 15, },
  { 1, 7, 8, 9, 12, 15, },
  { 0, 1, 7, 8, 9, 12, 15, },
  { 2, 7, 8, 9, 12, 15, },
  { 0, 2, 7, 8, 9, 12, 15, },
  { 1, 2, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 7, 8, 9, 12, 15, },
  { 3, 7, 8, 9, 12, 15, },
  { 0, 3, 7, 8, 9, 12, 15, },
  { 1, 3, 7, 8, 9, 12, 15, },
  { 0, 1, 3, 7, 8, 9, 12, 15, },
  { 2, 3, 7, 8, 9, 12, 15, },
  { 0, 2, 3, 7, 8, 9, 12, 15, },
  { 1, 2, 3, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 12, 15, },
  { 4, 7, 8, 9, 12, 15, },
  { 0, 4, 7, 8, 9, 12, 15, },
  { 1, 4, 7, 8, 9, 12, 15, },
  { 0, 1, 4, 7, 8, 9, 12, 15, },
  { 2, 4, 7, 8, 9, 12, 15, },
  { 0, 2, 4, 7, 8, 9, 12, 15, },
  { 1, 2, 4, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 12, 15, },
  { 3, 4, 7, 8, 9, 12, 15, },
  { 0, 3, 4, 7, 8, 9, 12, 15, },
  { 1, 3, 4, 7, 8, 9, 12, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 12, 15, },
  { 2, 3, 4, 7, 8, 9, 12, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 12, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 12, 15, },
  { 5, 7, 8, 9, 12, 15, },
  { 0, 5, 7, 8, 9, 12, 15, },
  { 1, 5, 7, 8, 9, 12, 15, },
  { 0, 1, 5, 7, 8, 9, 12, 15, },
  { 2, 5, 7, 8, 9, 12, 15, },
  { 0, 2, 5, 7, 8, 9, 12, 15, },
  { 1, 2, 5, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 12, 15, },
  { 3, 5, 7, 8, 9, 12, 15, },
  { 0, 3, 5, 7, 8, 9, 12, 15, },
  { 1, 3, 5, 7, 8, 9, 12, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 12, 15, },
  { 2, 3, 5, 7, 8, 9, 12, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 12, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 12, 15, },
  { 4, 5, 7, 8, 9, 12, 15, },
  { 0, 4, 5, 7, 8, 9, 12, 15, },
  { 1, 4, 5, 7, 8, 9, 12, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 12, 15, },
  { 2, 4, 5, 7, 8, 9, 12, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 12, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 12, 15, },
  { 3, 4, 5, 7, 8, 9, 12, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 12, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 12, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 12, 15, },
  { 6, 7, 8, 9, 12, 15, },
  { 0, 6, 7, 8, 9, 12, 15, },
  { 1, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 6, 7, 8, 9, 12, 15, },
  { 2, 6, 7, 8, 9, 12, 15, },
  { 0, 2, 6, 7, 8, 9, 12, 15, },
  { 1, 2, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 12, 15, },
  { 3, 6, 7, 8, 9, 12, 15, },
  { 0, 3, 6, 7, 8, 9, 12, 15, },
  { 1, 3, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 12, 15, },
  { 2, 3, 6, 7, 8, 9, 12, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 12, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 12, 15, },
  { 4, 6, 7, 8, 9, 12, 15, },
  { 0, 4, 6, 7, 8, 9, 12, 15, },
  { 1, 4, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 12, 15, },
  { 2, 4, 6, 7, 8, 9, 12, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 12, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 12, 15, },
  { 3, 4, 6, 7, 8, 9, 12, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 12, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 12, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 12, 15, },
  { 5, 6, 7, 8, 9, 12, 15, },
  { 0, 5, 6, 7, 8, 9, 12, 15, },
  { 1, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 12, 15, },
  { 2, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 12, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 12, 15, },
  { 3, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 12, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 12, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 12, 15, },
  { 4, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 15, },
  { 10, 12, 15, },
  { 0, 10, 12, 15, },
  { 1, 10, 12, 15, },
  { 0, 1, 10, 12, 15, },
  { 2, 10, 12, 15, },
  { 0, 2, 10, 12, 15, },
  { 1, 2, 10, 12, 15, },
  { 0, 1, 2, 10, 12, 15, },
  { 3, 10, 12, 15, },
  { 0, 3, 10, 12, 15, },
  { 1, 3, 10, 12, 15, },
  { 0, 1, 3, 10, 12, 15, },
  { 2, 3, 10, 12, 15, },
  { 0, 2, 3, 10, 12, 15, },
  { 1, 2, 3, 10, 12, 15, },
  { 0, 1, 2, 3, 10, 12, 15, },
  { 4, 10, 12, 15, },
  { 0, 4, 10, 12, 15, },
  { 1, 4, 10, 12, 15, },
  { 0, 1, 4, 10, 12, 15, },
  { 2, 4, 10, 12, 15, },
  { 0, 2, 4, 10, 12, 15, },
  { 1, 2, 4, 10, 12, 15, },
  { 0, 1, 2, 4, 10, 12, 15, },
  { 3, 4, 10, 12, 15, },
  { 0, 3, 4, 10, 12, 15, },
  { 1, 3, 4, 10, 12, 15, },
  { 0, 1, 3, 4, 10, 12, 15, },
  { 2, 3, 4, 10, 12, 15, },
  { 0, 2, 3, 4, 10, 12, 15, },
  { 1, 2, 3, 4, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 10, 12, 15, },
  { 5, 10, 12, 15, },
  { 0, 5, 10, 12, 15, },
  { 1, 5, 10, 12, 15, },
  { 0, 1, 5, 10, 12, 15, },
  { 2, 5, 10, 12, 15, },
  { 0, 2, 5, 10, 12, 15, },
  { 1, 2, 5, 10, 12, 15, },
  { 0, 1, 2, 5, 10, 12, 15, },
  { 3, 5, 10, 12, 15, },
  { 0, 3, 5, 10, 12, 15, },
  { 1, 3, 5, 10, 12, 15, },
  { 0, 1, 3, 5, 10, 12, 15, },
  { 2, 3, 5, 10, 12, 15, },
  { 0, 2, 3, 5, 10, 12, 15, },
  { 1, 2, 3, 5, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 10, 12, 15, },
  { 4, 5, 10, 12, 15, },
  { 0, 4, 5, 10, 12, 15, },
  { 1, 4, 5, 10, 12, 15, },
  { 0, 1, 4, 5, 10, 12, 15, },
  { 2, 4, 5, 10, 12, 15, },
  { 0, 2, 4, 5, 10, 12, 15, },
  { 1, 2, 4, 5, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 10, 12, 15, },
  { 3, 4, 5, 10, 12, 15, },
  { 0, 3, 4, 5, 10, 12, 15, },
  { 1, 3, 4, 5, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 10, 12, 15, },
  { 2, 3, 4, 5, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 12, 15, },
  { 6, 10, 12, 15, },
  { 0, 6, 10, 12, 15, },
  { 1, 6, 10, 12, 15, },
  { 0, 1, 6, 10, 12, 15, },
  { 2, 6, 10, 12, 15, },
  { 0, 2, 6, 10, 12, 15, },
  { 1, 2, 6, 10, 12, 15, },
  { 0, 1, 2, 6, 10, 12, 15, },
  { 3, 6, 10, 12, 15, },
  { 0, 3, 6, 10, 12, 15, },
  { 1, 3, 6, 10, 12, 15, },
  { 0, 1, 3, 6, 10, 12, 15, },
  { 2, 3, 6, 10, 12, 15, },
  { 0, 2, 3, 6, 10, 12, 15, },
  { 1, 2, 3, 6, 10, 12, 15, },
  { 0, 1, 2, 3, 6, 10, 12, 15, },
  { 4, 6, 10, 12, 15, },
  { 0, 4, 6, 10, 12, 15, },
  { 1, 4, 6, 10, 12, 15, },
  { 0, 1, 4, 6, 10, 12, 15, },
  { 2, 4, 6, 10, 12, 15, },
  { 0, 2, 4, 6, 10, 12, 15, },
  { 1, 2, 4, 6, 10, 12, 15, },
  { 0, 1, 2, 4, 6, 10, 12, 15, },
  { 3, 4, 6, 10, 12, 15, },
  { 0, 3, 4, 6, 10, 12, 15, },
  { 1, 3, 4, 6, 10, 12, 15, },
  { 0, 1, 3, 4, 6, 10, 12, 15, },
  { 2, 3, 4, 6, 10, 12, 15, },
  { 0, 2, 3, 4, 6, 10, 12, 15, },
  { 1, 2, 3, 4, 6, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 12, 15, },
  { 5, 6, 10, 12, 15, },
  { 0, 5, 6, 10, 12, 15, },
  { 1, 5, 6, 10, 12, 15, },
  { 0, 1, 5, 6, 10, 12, 15, },
  { 2, 5, 6, 10, 12, 15, },
  { 0, 2, 5, 6, 10, 12, 15, },
  { 1, 2, 5, 6, 10, 12, 15, },
  { 0, 1, 2, 5, 6, 10, 12, 15, },
  { 3, 5, 6, 10, 12, 15, },
  { 0, 3, 5, 6, 10, 12, 15, },
  { 1, 3, 5, 6, 10, 12, 15, },
  { 0, 1, 3, 5, 6, 10, 12, 15, },
  { 2, 3, 5, 6, 10, 12, 15, },
  { 0, 2, 3, 5, 6, 10, 12, 15, },
  { 1, 2, 3, 5, 6, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 12, 15, },
  { 4, 5, 6, 10, 12, 15, },
  { 0, 4, 5, 6, 10, 12, 15, },
  { 1, 4, 5, 6, 10, 12, 15, },
  { 0, 1, 4, 5, 6, 10, 12, 15, },
  { 2, 4, 5, 6, 10, 12, 15, },
  { 0, 2, 4, 5, 6, 10, 12, 15, },
  { 1, 2, 4, 5, 6, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 12, 15, },
  { 3, 4, 5, 6, 10, 12, 15, },
  { 0, 3, 4, 5, 6, 10, 12, 15, },
  { 1, 3, 4, 5, 6, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 12, 15, },
  { 2, 3, 4, 5, 6, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 12, 15, },
  { 7, 10, 12, 15, },
  { 0, 7, 10, 12, 15, },
  { 1, 7, 10, 12, 15, },
  { 0, 1, 7, 10, 12, 15, },
  { 2, 7, 10, 12, 15, },
  { 0, 2, 7, 10, 12, 15, },
  { 1, 2, 7, 10, 12, 15, },
  { 0, 1, 2, 7, 10, 12, 15, },
  { 3, 7, 10, 12, 15, },
  { 0, 3, 7, 10, 12, 15, },
  { 1, 3, 7, 10, 12, 15, },
  { 0, 1, 3, 7, 10, 12, 15, },
  { 2, 3, 7, 10, 12, 15, },
  { 0, 2, 3, 7, 10, 12, 15, },
  { 1, 2, 3, 7, 10, 12, 15, },
  { 0, 1, 2, 3, 7, 10, 12, 15, },
  { 4, 7, 10, 12, 15, },
  { 0, 4, 7, 10, 12, 15, },
  { 1, 4, 7, 10, 12, 15, },
  { 0, 1, 4, 7, 10, 12, 15, },
  { 2, 4, 7, 10, 12, 15, },
  { 0, 2, 4, 7, 10, 12, 15, },
  { 1, 2, 4, 7, 10, 12, 15, },
  { 0, 1, 2, 4, 7, 10, 12, 15, },
  { 3, 4, 7, 10, 12, 15, },
  { 0, 3, 4, 7, 10, 12, 15, },
  { 1, 3, 4, 7, 10, 12, 15, },
  { 0, 1, 3, 4, 7, 10, 12, 15, },
  { 2, 3, 4, 7, 10, 12, 15, },
  { 0, 2, 3, 4, 7, 10, 12, 15, },
  { 1, 2, 3, 4, 7, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 12, 15, },
  { 5, 7, 10, 12, 15, },
  { 0, 5, 7, 10, 12, 15, },
  { 1, 5, 7, 10, 12, 15, },
  { 0, 1, 5, 7, 10, 12, 15, },
  { 2, 5, 7, 10, 12, 15, },
  { 0, 2, 5, 7, 10, 12, 15, },
  { 1, 2, 5, 7, 10, 12, 15, },
  { 0, 1, 2, 5, 7, 10, 12, 15, },
  { 3, 5, 7, 10, 12, 15, },
  { 0, 3, 5, 7, 10, 12, 15, },
  { 1, 3, 5, 7, 10, 12, 15, },
  { 0, 1, 3, 5, 7, 10, 12, 15, },
  { 2, 3, 5, 7, 10, 12, 15, },
  { 0, 2, 3, 5, 7, 10, 12, 15, },
  { 1, 2, 3, 5, 7, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 12, 15, },
  { 4, 5, 7, 10, 12, 15, },
  { 0, 4, 5, 7, 10, 12, 15, },
  { 1, 4, 5, 7, 10, 12, 15, },
  { 0, 1, 4, 5, 7, 10, 12, 15, },
  { 2, 4, 5, 7, 10, 12, 15, },
  { 0, 2, 4, 5, 7, 10, 12, 15, },
  { 1, 2, 4, 5, 7, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 12, 15, },
  { 3, 4, 5, 7, 10, 12, 15, },
  { 0, 3, 4, 5, 7, 10, 12, 15, },
  { 1, 3, 4, 5, 7, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 12, 15, },
  { 2, 3, 4, 5, 7, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 12, 15, },
  { 6, 7, 10, 12, 15, },
  { 0, 6, 7, 10, 12, 15, },
  { 1, 6, 7, 10, 12, 15, },
  { 0, 1, 6, 7, 10, 12, 15, },
  { 2, 6, 7, 10, 12, 15, },
  { 0, 2, 6, 7, 10, 12, 15, },
  { 1, 2, 6, 7, 10, 12, 15, },
  { 0, 1, 2, 6, 7, 10, 12, 15, },
  { 3, 6, 7, 10, 12, 15, },
  { 0, 3, 6, 7, 10, 12, 15, },
  { 1, 3, 6, 7, 10, 12, 15, },
  { 0, 1, 3, 6, 7, 10, 12, 15, },
  { 2, 3, 6, 7, 10, 12, 15, },
  { 0, 2, 3, 6, 7, 10, 12, 15, },
  { 1, 2, 3, 6, 7, 10, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 12, 15, },
  { 4, 6, 7, 10, 12, 15, },
  { 0, 4, 6, 7, 10, 12, 15, },
  { 1, 4, 6, 7, 10, 12, 15, },
  { 0, 1, 4, 6, 7, 10, 12, 15, },
  { 2, 4, 6, 7, 10, 12, 15, },
  { 0, 2, 4, 6, 7, 10, 12, 15, },
  { 1, 2, 4, 6, 7, 10, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 12, 15, },
  { 3, 4, 6, 7, 10, 12, 15, },
  { 0, 3, 4, 6, 7, 10, 12, 15, },
  { 1, 3, 4, 6, 7, 10, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 12, 15, },
  { 2, 3, 4, 6, 7, 10, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 12, 15, },
  { 5, 6, 7, 10, 12, 15, },
  { 0, 5, 6, 7, 10, 12, 15, },
  { 1, 5, 6, 7, 10, 12, 15, },
  { 0, 1, 5, 6, 7, 10, 12, 15, },
  { 2, 5, 6, 7, 10, 12, 15, },
  { 0, 2, 5, 6, 7, 10, 12, 15, },
  { 1, 2, 5, 6, 7, 10, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 12, 15, },
  { 3, 5, 6, 7, 10, 12, 15, },
  { 0, 3, 5, 6, 7, 10, 12, 15, },
  { 1, 3, 5, 6, 7, 10, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 12, 15, },
  { 2, 3, 5, 6, 7, 10, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 12, 15, },
  { 4, 5, 6, 7, 10, 12, 15, },
  { 0, 4, 5, 6, 7, 10, 12, 15, },
  { 1, 4, 5, 6, 7, 10, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 12, 15, },
  { 2, 4, 5, 6, 7, 10, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 12, 15, },
  { 3, 4, 5, 6, 7, 10, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 12, 15, },
  { 8, 10, 12, 15, },
  { 0, 8, 10, 12, 15, },
  { 1, 8, 10, 12, 15, },
  { 0, 1, 8, 10, 12, 15, },
  { 2, 8, 10, 12, 15, },
  { 0, 2, 8, 10, 12, 15, },
  { 1, 2, 8, 10, 12, 15, },
  { 0, 1, 2, 8, 10, 12, 15, },
  { 3, 8, 10, 12, 15, },
  { 0, 3, 8, 10, 12, 15, },
  { 1, 3, 8, 10, 12, 15, },
  { 0, 1, 3, 8, 10, 12, 15, },
  { 2, 3, 8, 10, 12, 15, },
  { 0, 2, 3, 8, 10, 12, 15, },
  { 1, 2, 3, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 8, 10, 12, 15, },
  { 4, 8, 10, 12, 15, },
  { 0, 4, 8, 10, 12, 15, },
  { 1, 4, 8, 10, 12, 15, },
  { 0, 1, 4, 8, 10, 12, 15, },
  { 2, 4, 8, 10, 12, 15, },
  { 0, 2, 4, 8, 10, 12, 15, },
  { 1, 2, 4, 8, 10, 12, 15, },
  { 0, 1, 2, 4, 8, 10, 12, 15, },
  { 3, 4, 8, 10, 12, 15, },
  { 0, 3, 4, 8, 10, 12, 15, },
  { 1, 3, 4, 8, 10, 12, 15, },
  { 0, 1, 3, 4, 8, 10, 12, 15, },
  { 2, 3, 4, 8, 10, 12, 15, },
  { 0, 2, 3, 4, 8, 10, 12, 15, },
  { 1, 2, 3, 4, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 12, 15, },
  { 5, 8, 10, 12, 15, },
  { 0, 5, 8, 10, 12, 15, },
  { 1, 5, 8, 10, 12, 15, },
  { 0, 1, 5, 8, 10, 12, 15, },
  { 2, 5, 8, 10, 12, 15, },
  { 0, 2, 5, 8, 10, 12, 15, },
  { 1, 2, 5, 8, 10, 12, 15, },
  { 0, 1, 2, 5, 8, 10, 12, 15, },
  { 3, 5, 8, 10, 12, 15, },
  { 0, 3, 5, 8, 10, 12, 15, },
  { 1, 3, 5, 8, 10, 12, 15, },
  { 0, 1, 3, 5, 8, 10, 12, 15, },
  { 2, 3, 5, 8, 10, 12, 15, },
  { 0, 2, 3, 5, 8, 10, 12, 15, },
  { 1, 2, 3, 5, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 12, 15, },
  { 4, 5, 8, 10, 12, 15, },
  { 0, 4, 5, 8, 10, 12, 15, },
  { 1, 4, 5, 8, 10, 12, 15, },
  { 0, 1, 4, 5, 8, 10, 12, 15, },
  { 2, 4, 5, 8, 10, 12, 15, },
  { 0, 2, 4, 5, 8, 10, 12, 15, },
  { 1, 2, 4, 5, 8, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 12, 15, },
  { 3, 4, 5, 8, 10, 12, 15, },
  { 0, 3, 4, 5, 8, 10, 12, 15, },
  { 1, 3, 4, 5, 8, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 12, 15, },
  { 2, 3, 4, 5, 8, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 12, 15, },
  { 6, 8, 10, 12, 15, },
  { 0, 6, 8, 10, 12, 15, },
  { 1, 6, 8, 10, 12, 15, },
  { 0, 1, 6, 8, 10, 12, 15, },
  { 2, 6, 8, 10, 12, 15, },
  { 0, 2, 6, 8, 10, 12, 15, },
  { 1, 2, 6, 8, 10, 12, 15, },
  { 0, 1, 2, 6, 8, 10, 12, 15, },
  { 3, 6, 8, 10, 12, 15, },
  { 0, 3, 6, 8, 10, 12, 15, },
  { 1, 3, 6, 8, 10, 12, 15, },
  { 0, 1, 3, 6, 8, 10, 12, 15, },
  { 2, 3, 6, 8, 10, 12, 15, },
  { 0, 2, 3, 6, 8, 10, 12, 15, },
  { 1, 2, 3, 6, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 12, 15, },
  { 4, 6, 8, 10, 12, 15, },
  { 0, 4, 6, 8, 10, 12, 15, },
  { 1, 4, 6, 8, 10, 12, 15, },
  { 0, 1, 4, 6, 8, 10, 12, 15, },
  { 2, 4, 6, 8, 10, 12, 15, },
  { 0, 2, 4, 6, 8, 10, 12, 15, },
  { 1, 2, 4, 6, 8, 10, 12, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 12, 15, },
  { 3, 4, 6, 8, 10, 12, 15, },
  { 0, 3, 4, 6, 8, 10, 12, 15, },
  { 1, 3, 4, 6, 8, 10, 12, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 12, 15, },
  { 2, 3, 4, 6, 8, 10, 12, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 12, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 12, 15, },
  { 5, 6, 8, 10, 12, 15, },
  { 0, 5, 6, 8, 10, 12, 15, },
  { 1, 5, 6, 8, 10, 12, 15, },
  { 0, 1, 5, 6, 8, 10, 12, 15, },
  { 2, 5, 6, 8, 10, 12, 15, },
  { 0, 2, 5, 6, 8, 10, 12, 15, },
  { 1, 2, 5, 6, 8, 10, 12, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 12, 15, },
  { 3, 5, 6, 8, 10, 12, 15, },
  { 0, 3, 5, 6, 8, 10, 12, 15, },
  { 1, 3, 5, 6, 8, 10, 12, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 12, 15, },
  { 2, 3, 5, 6, 8, 10, 12, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 12, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 12, 15, },
  { 4, 5, 6, 8, 10, 12, 15, },
  { 0, 4, 5, 6, 8, 10, 12, 15, },
  { 1, 4, 5, 6, 8, 10, 12, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 12, 15, },
  { 2, 4, 5, 6, 8, 10, 12, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 12, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 12, 15, },
  { 3, 4, 5, 6, 8, 10, 12, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 12, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 12, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 12, 15, },
  { 7, 8, 10, 12, 15, },
  { 0, 7, 8, 10, 12, 15, },
  { 1, 7, 8, 10, 12, 15, },
  { 0, 1, 7, 8, 10, 12, 15, },
  { 2, 7, 8, 10, 12, 15, },
  { 0, 2, 7, 8, 10, 12, 15, },
  { 1, 2, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 7, 8, 10, 12, 15, },
  { 3, 7, 8, 10, 12, 15, },
  { 0, 3, 7, 8, 10, 12, 15, },
  { 1, 3, 7, 8, 10, 12, 15, },
  { 0, 1, 3, 7, 8, 10, 12, 15, },
  { 2, 3, 7, 8, 10, 12, 15, },
  { 0, 2, 3, 7, 8, 10, 12, 15, },
  { 1, 2, 3, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 12, 15, },
  { 4, 7, 8, 10, 12, 15, },
  { 0, 4, 7, 8, 10, 12, 15, },
  { 1, 4, 7, 8, 10, 12, 15, },
  { 0, 1, 4, 7, 8, 10, 12, 15, },
  { 2, 4, 7, 8, 10, 12, 15, },
  { 0, 2, 4, 7, 8, 10, 12, 15, },
  { 1, 2, 4, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 12, 15, },
  { 3, 4, 7, 8, 10, 12, 15, },
  { 0, 3, 4, 7, 8, 10, 12, 15, },
  { 1, 3, 4, 7, 8, 10, 12, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 12, 15, },
  { 2, 3, 4, 7, 8, 10, 12, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 12, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 12, 15, },
  { 5, 7, 8, 10, 12, 15, },
  { 0, 5, 7, 8, 10, 12, 15, },
  { 1, 5, 7, 8, 10, 12, 15, },
  { 0, 1, 5, 7, 8, 10, 12, 15, },
  { 2, 5, 7, 8, 10, 12, 15, },
  { 0, 2, 5, 7, 8, 10, 12, 15, },
  { 1, 2, 5, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 12, 15, },
  { 3, 5, 7, 8, 10, 12, 15, },
  { 0, 3, 5, 7, 8, 10, 12, 15, },
  { 1, 3, 5, 7, 8, 10, 12, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 12, 15, },
  { 2, 3, 5, 7, 8, 10, 12, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 12, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 12, 15, },
  { 4, 5, 7, 8, 10, 12, 15, },
  { 0, 4, 5, 7, 8, 10, 12, 15, },
  { 1, 4, 5, 7, 8, 10, 12, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 12, 15, },
  { 2, 4, 5, 7, 8, 10, 12, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 12, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 12, 15, },
  { 3, 4, 5, 7, 8, 10, 12, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 12, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 12, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 15, },
  { 6, 7, 8, 10, 12, 15, },
  { 0, 6, 7, 8, 10, 12, 15, },
  { 1, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 6, 7, 8, 10, 12, 15, },
  { 2, 6, 7, 8, 10, 12, 15, },
  { 0, 2, 6, 7, 8, 10, 12, 15, },
  { 1, 2, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 12, 15, },
  { 3, 6, 7, 8, 10, 12, 15, },
  { 0, 3, 6, 7, 8, 10, 12, 15, },
  { 1, 3, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 12, 15, },
  { 2, 3, 6, 7, 8, 10, 12, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 12, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 12, 15, },
  { 4, 6, 7, 8, 10, 12, 15, },
  { 0, 4, 6, 7, 8, 10, 12, 15, },
  { 1, 4, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 12, 15, },
  { 2, 4, 6, 7, 8, 10, 12, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 12, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 12, 15, },
  { 3, 4, 6, 7, 8, 10, 12, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 12, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 12, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 12, 15, },
  { 5, 6, 7, 8, 10, 12, 15, },
  { 0, 5, 6, 7, 8, 10, 12, 15, },
  { 1, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 12, 15, },
  { 2, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 12, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 12, 15, },
  { 3, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 12, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 12, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 12, 15, },
  { 4, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 15, },
  { 9, 10, 12, 15, },
  { 0, 9, 10, 12, 15, },
  { 1, 9, 10, 12, 15, },
  { 0, 1, 9, 10, 12, 15, },
  { 2, 9, 10, 12, 15, },
  { 0, 2, 9, 10, 12, 15, },
  { 1, 2, 9, 10, 12, 15, },
  { 0, 1, 2, 9, 10, 12, 15, },
  { 3, 9, 10, 12, 15, },
  { 0, 3, 9, 10, 12, 15, },
  { 1, 3, 9, 10, 12, 15, },
  { 0, 1, 3, 9, 10, 12, 15, },
  { 2, 3, 9, 10, 12, 15, },
  { 0, 2, 3, 9, 10, 12, 15, },
  { 1, 2, 3, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 9, 10, 12, 15, },
  { 4, 9, 10, 12, 15, },
  { 0, 4, 9, 10, 12, 15, },
  { 1, 4, 9, 10, 12, 15, },
  { 0, 1, 4, 9, 10, 12, 15, },
  { 2, 4, 9, 10, 12, 15, },
  { 0, 2, 4, 9, 10, 12, 15, },
  { 1, 2, 4, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 9, 10, 12, 15, },
  { 3, 4, 9, 10, 12, 15, },
  { 0, 3, 4, 9, 10, 12, 15, },
  { 1, 3, 4, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 9, 10, 12, 15, },
  { 2, 3, 4, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 12, 15, },
  { 5, 9, 10, 12, 15, },
  { 0, 5, 9, 10, 12, 15, },
  { 1, 5, 9, 10, 12, 15, },
  { 0, 1, 5, 9, 10, 12, 15, },
  { 2, 5, 9, 10, 12, 15, },
  { 0, 2, 5, 9, 10, 12, 15, },
  { 1, 2, 5, 9, 10, 12, 15, },
  { 0, 1, 2, 5, 9, 10, 12, 15, },
  { 3, 5, 9, 10, 12, 15, },
  { 0, 3, 5, 9, 10, 12, 15, },
  { 1, 3, 5, 9, 10, 12, 15, },
  { 0, 1, 3, 5, 9, 10, 12, 15, },
  { 2, 3, 5, 9, 10, 12, 15, },
  { 0, 2, 3, 5, 9, 10, 12, 15, },
  { 1, 2, 3, 5, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 12, 15, },
  { 4, 5, 9, 10, 12, 15, },
  { 0, 4, 5, 9, 10, 12, 15, },
  { 1, 4, 5, 9, 10, 12, 15, },
  { 0, 1, 4, 5, 9, 10, 12, 15, },
  { 2, 4, 5, 9, 10, 12, 15, },
  { 0, 2, 4, 5, 9, 10, 12, 15, },
  { 1, 2, 4, 5, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 12, 15, },
  { 3, 4, 5, 9, 10, 12, 15, },
  { 0, 3, 4, 5, 9, 10, 12, 15, },
  { 1, 3, 4, 5, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 12, 15, },
  { 2, 3, 4, 5, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 12, 15, },
  { 6, 9, 10, 12, 15, },
  { 0, 6, 9, 10, 12, 15, },
  { 1, 6, 9, 10, 12, 15, },
  { 0, 1, 6, 9, 10, 12, 15, },
  { 2, 6, 9, 10, 12, 15, },
  { 0, 2, 6, 9, 10, 12, 15, },
  { 1, 2, 6, 9, 10, 12, 15, },
  { 0, 1, 2, 6, 9, 10, 12, 15, },
  { 3, 6, 9, 10, 12, 15, },
  { 0, 3, 6, 9, 10, 12, 15, },
  { 1, 3, 6, 9, 10, 12, 15, },
  { 0, 1, 3, 6, 9, 10, 12, 15, },
  { 2, 3, 6, 9, 10, 12, 15, },
  { 0, 2, 3, 6, 9, 10, 12, 15, },
  { 1, 2, 3, 6, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 12, 15, },
  { 4, 6, 9, 10, 12, 15, },
  { 0, 4, 6, 9, 10, 12, 15, },
  { 1, 4, 6, 9, 10, 12, 15, },
  { 0, 1, 4, 6, 9, 10, 12, 15, },
  { 2, 4, 6, 9, 10, 12, 15, },
  { 0, 2, 4, 6, 9, 10, 12, 15, },
  { 1, 2, 4, 6, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 12, 15, },
  { 3, 4, 6, 9, 10, 12, 15, },
  { 0, 3, 4, 6, 9, 10, 12, 15, },
  { 1, 3, 4, 6, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 12, 15, },
  { 2, 3, 4, 6, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 12, 15, },
  { 5, 6, 9, 10, 12, 15, },
  { 0, 5, 6, 9, 10, 12, 15, },
  { 1, 5, 6, 9, 10, 12, 15, },
  { 0, 1, 5, 6, 9, 10, 12, 15, },
  { 2, 5, 6, 9, 10, 12, 15, },
  { 0, 2, 5, 6, 9, 10, 12, 15, },
  { 1, 2, 5, 6, 9, 10, 12, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 12, 15, },
  { 3, 5, 6, 9, 10, 12, 15, },
  { 0, 3, 5, 6, 9, 10, 12, 15, },
  { 1, 3, 5, 6, 9, 10, 12, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 12, 15, },
  { 2, 3, 5, 6, 9, 10, 12, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 12, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 12, 15, },
  { 4, 5, 6, 9, 10, 12, 15, },
  { 0, 4, 5, 6, 9, 10, 12, 15, },
  { 1, 4, 5, 6, 9, 10, 12, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 12, 15, },
  { 2, 4, 5, 6, 9, 10, 12, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 12, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 12, 15, },
  { 3, 4, 5, 6, 9, 10, 12, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 12, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 12, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 12, 15, },
  { 7, 9, 10, 12, 15, },
  { 0, 7, 9, 10, 12, 15, },
  { 1, 7, 9, 10, 12, 15, },
  { 0, 1, 7, 9, 10, 12, 15, },
  { 2, 7, 9, 10, 12, 15, },
  { 0, 2, 7, 9, 10, 12, 15, },
  { 1, 2, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 7, 9, 10, 12, 15, },
  { 3, 7, 9, 10, 12, 15, },
  { 0, 3, 7, 9, 10, 12, 15, },
  { 1, 3, 7, 9, 10, 12, 15, },
  { 0, 1, 3, 7, 9, 10, 12, 15, },
  { 2, 3, 7, 9, 10, 12, 15, },
  { 0, 2, 3, 7, 9, 10, 12, 15, },
  { 1, 2, 3, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 12, 15, },
  { 4, 7, 9, 10, 12, 15, },
  { 0, 4, 7, 9, 10, 12, 15, },
  { 1, 4, 7, 9, 10, 12, 15, },
  { 0, 1, 4, 7, 9, 10, 12, 15, },
  { 2, 4, 7, 9, 10, 12, 15, },
  { 0, 2, 4, 7, 9, 10, 12, 15, },
  { 1, 2, 4, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 12, 15, },
  { 3, 4, 7, 9, 10, 12, 15, },
  { 0, 3, 4, 7, 9, 10, 12, 15, },
  { 1, 3, 4, 7, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 12, 15, },
  { 2, 3, 4, 7, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 12, 15, },
  { 5, 7, 9, 10, 12, 15, },
  { 0, 5, 7, 9, 10, 12, 15, },
  { 1, 5, 7, 9, 10, 12, 15, },
  { 0, 1, 5, 7, 9, 10, 12, 15, },
  { 2, 5, 7, 9, 10, 12, 15, },
  { 0, 2, 5, 7, 9, 10, 12, 15, },
  { 1, 2, 5, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 12, 15, },
  { 3, 5, 7, 9, 10, 12, 15, },
  { 0, 3, 5, 7, 9, 10, 12, 15, },
  { 1, 3, 5, 7, 9, 10, 12, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 12, 15, },
  { 2, 3, 5, 7, 9, 10, 12, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 12, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 12, 15, },
  { 4, 5, 7, 9, 10, 12, 15, },
  { 0, 4, 5, 7, 9, 10, 12, 15, },
  { 1, 4, 5, 7, 9, 10, 12, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 12, 15, },
  { 2, 4, 5, 7, 9, 10, 12, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 12, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 12, 15, },
  { 3, 4, 5, 7, 9, 10, 12, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 12, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 12, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 12, 15, },
  { 6, 7, 9, 10, 12, 15, },
  { 0, 6, 7, 9, 10, 12, 15, },
  { 1, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 6, 7, 9, 10, 12, 15, },
  { 2, 6, 7, 9, 10, 12, 15, },
  { 0, 2, 6, 7, 9, 10, 12, 15, },
  { 1, 2, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 12, 15, },
  { 3, 6, 7, 9, 10, 12, 15, },
  { 0, 3, 6, 7, 9, 10, 12, 15, },
  { 1, 3, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 12, 15, },
  { 2, 3, 6, 7, 9, 10, 12, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 12, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 12, 15, },
  { 4, 6, 7, 9, 10, 12, 15, },
  { 0, 4, 6, 7, 9, 10, 12, 15, },
  { 1, 4, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 12, 15, },
  { 2, 4, 6, 7, 9, 10, 12, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 12, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 12, 15, },
  { 3, 4, 6, 7, 9, 10, 12, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 12, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 12, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 12, 15, },
  { 5, 6, 7, 9, 10, 12, 15, },
  { 0, 5, 6, 7, 9, 10, 12, 15, },
  { 1, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 12, 15, },
  { 2, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 12, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 12, 15, },
  { 3, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 12, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 12, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 12, 15, },
  { 4, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 15, },
  { 8, 9, 10, 12, 15, },
  { 0, 8, 9, 10, 12, 15, },
  { 1, 8, 9, 10, 12, 15, },
  { 0, 1, 8, 9, 10, 12, 15, },
  { 2, 8, 9, 10, 12, 15, },
  { 0, 2, 8, 9, 10, 12, 15, },
  { 1, 2, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 8, 9, 10, 12, 15, },
  { 3, 8, 9, 10, 12, 15, },
  { 0, 3, 8, 9, 10, 12, 15, },
  { 1, 3, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 8, 9, 10, 12, 15, },
  { 2, 3, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 12, 15, },
  { 4, 8, 9, 10, 12, 15, },
  { 0, 4, 8, 9, 10, 12, 15, },
  { 1, 4, 8, 9, 10, 12, 15, },
  { 0, 1, 4, 8, 9, 10, 12, 15, },
  { 2, 4, 8, 9, 10, 12, 15, },
  { 0, 2, 4, 8, 9, 10, 12, 15, },
  { 1, 2, 4, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 12, 15, },
  { 3, 4, 8, 9, 10, 12, 15, },
  { 0, 3, 4, 8, 9, 10, 12, 15, },
  { 1, 3, 4, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 12, 15, },
  { 2, 3, 4, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 12, 15, },
  { 5, 8, 9, 10, 12, 15, },
  { 0, 5, 8, 9, 10, 12, 15, },
  { 1, 5, 8, 9, 10, 12, 15, },
  { 0, 1, 5, 8, 9, 10, 12, 15, },
  { 2, 5, 8, 9, 10, 12, 15, },
  { 0, 2, 5, 8, 9, 10, 12, 15, },
  { 1, 2, 5, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 12, 15, },
  { 3, 5, 8, 9, 10, 12, 15, },
  { 0, 3, 5, 8, 9, 10, 12, 15, },
  { 1, 3, 5, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 12, 15, },
  { 2, 3, 5, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 12, 15, },
  { 4, 5, 8, 9, 10, 12, 15, },
  { 0, 4, 5, 8, 9, 10, 12, 15, },
  { 1, 4, 5, 8, 9, 10, 12, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 12, 15, },
  { 2, 4, 5, 8, 9, 10, 12, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 12, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 12, 15, },
  { 3, 4, 5, 8, 9, 10, 12, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 12, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 12, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 12, 15, },
  { 6, 8, 9, 10, 12, 15, },
  { 0, 6, 8, 9, 10, 12, 15, },
  { 1, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 6, 8, 9, 10, 12, 15, },
  { 2, 6, 8, 9, 10, 12, 15, },
  { 0, 2, 6, 8, 9, 10, 12, 15, },
  { 1, 2, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 12, 15, },
  { 3, 6, 8, 9, 10, 12, 15, },
  { 0, 3, 6, 8, 9, 10, 12, 15, },
  { 1, 3, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 12, 15, },
  { 2, 3, 6, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 12, 15, },
  { 4, 6, 8, 9, 10, 12, 15, },
  { 0, 4, 6, 8, 9, 10, 12, 15, },
  { 1, 4, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 12, 15, },
  { 2, 4, 6, 8, 9, 10, 12, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 12, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 12, 15, },
  { 3, 4, 6, 8, 9, 10, 12, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 12, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 12, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 12, 15, },
  { 5, 6, 8, 9, 10, 12, 15, },
  { 0, 5, 6, 8, 9, 10, 12, 15, },
  { 1, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 12, 15, },
  { 2, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 12, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 12, 15, },
  { 3, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 12, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 12, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 12, 15, },
  { 4, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 15, },
  { 7, 8, 9, 10, 12, 15, },
  { 0, 7, 8, 9, 10, 12, 15, },
  { 1, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 7, 8, 9, 10, 12, 15, },
  { 2, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 12, 15, },
  { 3, 7, 8, 9, 10, 12, 15, },
  { 0, 3, 7, 8, 9, 10, 12, 15, },
  { 1, 3, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 12, 15, },
  { 2, 3, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 12, 15, },
  { 4, 7, 8, 9, 10, 12, 15, },
  { 0, 4, 7, 8, 9, 10, 12, 15, },
  { 1, 4, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 12, 15, },
  { 2, 4, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 12, 15, },
  { 3, 4, 7, 8, 9, 10, 12, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 12, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 12, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 12, 15, },
  { 5, 7, 8, 9, 10, 12, 15, },
  { 0, 5, 7, 8, 9, 10, 12, 15, },
  { 1, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 12, 15, },
  { 2, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 12, 15, },
  { 3, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 12, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 12, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 12, 15, },
  { 4, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 15, },
  { 6, 7, 8, 9, 10, 12, 15, },
  { 0, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 12, 15, },
  { 2, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 12, 15, },
  { 3, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 12, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 12, 15, },
  { 4, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 15, },
  { 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, },
  { 11, 12, 15, },
  { 0, 11, 12, 15, },
  { 1, 11, 12, 15, },
  { 0, 1, 11, 12, 15, },
  { 2, 11, 12, 15, },
  { 0, 2, 11, 12, 15, },
  { 1, 2, 11, 12, 15, },
  { 0, 1, 2, 11, 12, 15, },
  { 3, 11, 12, 15, },
  { 0, 3, 11, 12, 15, },
  { 1, 3, 11, 12, 15, },
  { 0, 1, 3, 11, 12, 15, },
  { 2, 3, 11, 12, 15, },
  { 0, 2, 3, 11, 12, 15, },
  { 1, 2, 3, 11, 12, 15, },
  { 0, 1, 2, 3, 11, 12, 15, },
  { 4, 11, 12, 15, },
  { 0, 4, 11, 12, 15, },
  { 1, 4, 11, 12, 15, },
  { 0, 1, 4, 11, 12, 15, },
  { 2, 4, 11, 12, 15, },
  { 0, 2, 4, 11, 12, 15, },
  { 1, 2, 4, 11, 12, 15, },
  { 0, 1, 2, 4, 11, 12, 15, },
  { 3, 4, 11, 12, 15, },
  { 0, 3, 4, 11, 12, 15, },
  { 1, 3, 4, 11, 12, 15, },
  { 0, 1, 3, 4, 11, 12, 15, },
  { 2, 3, 4, 11, 12, 15, },
  { 0, 2, 3, 4, 11, 12, 15, },
  { 1, 2, 3, 4, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 11, 12, 15, },
  { 5, 11, 12, 15, },
  { 0, 5, 11, 12, 15, },
  { 1, 5, 11, 12, 15, },
  { 0, 1, 5, 11, 12, 15, },
  { 2, 5, 11, 12, 15, },
  { 0, 2, 5, 11, 12, 15, },
  { 1, 2, 5, 11, 12, 15, },
  { 0, 1, 2, 5, 11, 12, 15, },
  { 3, 5, 11, 12, 15, },
  { 0, 3, 5, 11, 12, 15, },
  { 1, 3, 5, 11, 12, 15, },
  { 0, 1, 3, 5, 11, 12, 15, },
  { 2, 3, 5, 11, 12, 15, },
  { 0, 2, 3, 5, 11, 12, 15, },
  { 1, 2, 3, 5, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 11, 12, 15, },
  { 4, 5, 11, 12, 15, },
  { 0, 4, 5, 11, 12, 15, },
  { 1, 4, 5, 11, 12, 15, },
  { 0, 1, 4, 5, 11, 12, 15, },
  { 2, 4, 5, 11, 12, 15, },
  { 0, 2, 4, 5, 11, 12, 15, },
  { 1, 2, 4, 5, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 11, 12, 15, },
  { 3, 4, 5, 11, 12, 15, },
  { 0, 3, 4, 5, 11, 12, 15, },
  { 1, 3, 4, 5, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 11, 12, 15, },
  { 2, 3, 4, 5, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 11, 12, 15, },
  { 6, 11, 12, 15, },
  { 0, 6, 11, 12, 15, },
  { 1, 6, 11, 12, 15, },
  { 0, 1, 6, 11, 12, 15, },
  { 2, 6, 11, 12, 15, },
  { 0, 2, 6, 11, 12, 15, },
  { 1, 2, 6, 11, 12, 15, },
  { 0, 1, 2, 6, 11, 12, 15, },
  { 3, 6, 11, 12, 15, },
  { 0, 3, 6, 11, 12, 15, },
  { 1, 3, 6, 11, 12, 15, },
  { 0, 1, 3, 6, 11, 12, 15, },
  { 2, 3, 6, 11, 12, 15, },
  { 0, 2, 3, 6, 11, 12, 15, },
  { 1, 2, 3, 6, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 11, 12, 15, },
  { 4, 6, 11, 12, 15, },
  { 0, 4, 6, 11, 12, 15, },
  { 1, 4, 6, 11, 12, 15, },
  { 0, 1, 4, 6, 11, 12, 15, },
  { 2, 4, 6, 11, 12, 15, },
  { 0, 2, 4, 6, 11, 12, 15, },
  { 1, 2, 4, 6, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 11, 12, 15, },
  { 3, 4, 6, 11, 12, 15, },
  { 0, 3, 4, 6, 11, 12, 15, },
  { 1, 3, 4, 6, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 11, 12, 15, },
  { 2, 3, 4, 6, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 11, 12, 15, },
  { 5, 6, 11, 12, 15, },
  { 0, 5, 6, 11, 12, 15, },
  { 1, 5, 6, 11, 12, 15, },
  { 0, 1, 5, 6, 11, 12, 15, },
  { 2, 5, 6, 11, 12, 15, },
  { 0, 2, 5, 6, 11, 12, 15, },
  { 1, 2, 5, 6, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 11, 12, 15, },
  { 3, 5, 6, 11, 12, 15, },
  { 0, 3, 5, 6, 11, 12, 15, },
  { 1, 3, 5, 6, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 11, 12, 15, },
  { 2, 3, 5, 6, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 11, 12, 15, },
  { 4, 5, 6, 11, 12, 15, },
  { 0, 4, 5, 6, 11, 12, 15, },
  { 1, 4, 5, 6, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 11, 12, 15, },
  { 2, 4, 5, 6, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 11, 12, 15, },
  { 3, 4, 5, 6, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 12, 15, },
  { 7, 11, 12, 15, },
  { 0, 7, 11, 12, 15, },
  { 1, 7, 11, 12, 15, },
  { 0, 1, 7, 11, 12, 15, },
  { 2, 7, 11, 12, 15, },
  { 0, 2, 7, 11, 12, 15, },
  { 1, 2, 7, 11, 12, 15, },
  { 0, 1, 2, 7, 11, 12, 15, },
  { 3, 7, 11, 12, 15, },
  { 0, 3, 7, 11, 12, 15, },
  { 1, 3, 7, 11, 12, 15, },
  { 0, 1, 3, 7, 11, 12, 15, },
  { 2, 3, 7, 11, 12, 15, },
  { 0, 2, 3, 7, 11, 12, 15, },
  { 1, 2, 3, 7, 11, 12, 15, },
  { 0, 1, 2, 3, 7, 11, 12, 15, },
  { 4, 7, 11, 12, 15, },
  { 0, 4, 7, 11, 12, 15, },
  { 1, 4, 7, 11, 12, 15, },
  { 0, 1, 4, 7, 11, 12, 15, },
  { 2, 4, 7, 11, 12, 15, },
  { 0, 2, 4, 7, 11, 12, 15, },
  { 1, 2, 4, 7, 11, 12, 15, },
  { 0, 1, 2, 4, 7, 11, 12, 15, },
  { 3, 4, 7, 11, 12, 15, },
  { 0, 3, 4, 7, 11, 12, 15, },
  { 1, 3, 4, 7, 11, 12, 15, },
  { 0, 1, 3, 4, 7, 11, 12, 15, },
  { 2, 3, 4, 7, 11, 12, 15, },
  { 0, 2, 3, 4, 7, 11, 12, 15, },
  { 1, 2, 3, 4, 7, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 11, 12, 15, },
  { 5, 7, 11, 12, 15, },
  { 0, 5, 7, 11, 12, 15, },
  { 1, 5, 7, 11, 12, 15, },
  { 0, 1, 5, 7, 11, 12, 15, },
  { 2, 5, 7, 11, 12, 15, },
  { 0, 2, 5, 7, 11, 12, 15, },
  { 1, 2, 5, 7, 11, 12, 15, },
  { 0, 1, 2, 5, 7, 11, 12, 15, },
  { 3, 5, 7, 11, 12, 15, },
  { 0, 3, 5, 7, 11, 12, 15, },
  { 1, 3, 5, 7, 11, 12, 15, },
  { 0, 1, 3, 5, 7, 11, 12, 15, },
  { 2, 3, 5, 7, 11, 12, 15, },
  { 0, 2, 3, 5, 7, 11, 12, 15, },
  { 1, 2, 3, 5, 7, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 11, 12, 15, },
  { 4, 5, 7, 11, 12, 15, },
  { 0, 4, 5, 7, 11, 12, 15, },
  { 1, 4, 5, 7, 11, 12, 15, },
  { 0, 1, 4, 5, 7, 11, 12, 15, },
  { 2, 4, 5, 7, 11, 12, 15, },
  { 0, 2, 4, 5, 7, 11, 12, 15, },
  { 1, 2, 4, 5, 7, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 11, 12, 15, },
  { 3, 4, 5, 7, 11, 12, 15, },
  { 0, 3, 4, 5, 7, 11, 12, 15, },
  { 1, 3, 4, 5, 7, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 11, 12, 15, },
  { 2, 3, 4, 5, 7, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 12, 15, },
  { 6, 7, 11, 12, 15, },
  { 0, 6, 7, 11, 12, 15, },
  { 1, 6, 7, 11, 12, 15, },
  { 0, 1, 6, 7, 11, 12, 15, },
  { 2, 6, 7, 11, 12, 15, },
  { 0, 2, 6, 7, 11, 12, 15, },
  { 1, 2, 6, 7, 11, 12, 15, },
  { 0, 1, 2, 6, 7, 11, 12, 15, },
  { 3, 6, 7, 11, 12, 15, },
  { 0, 3, 6, 7, 11, 12, 15, },
  { 1, 3, 6, 7, 11, 12, 15, },
  { 0, 1, 3, 6, 7, 11, 12, 15, },
  { 2, 3, 6, 7, 11, 12, 15, },
  { 0, 2, 3, 6, 7, 11, 12, 15, },
  { 1, 2, 3, 6, 7, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 11, 12, 15, },
  { 4, 6, 7, 11, 12, 15, },
  { 0, 4, 6, 7, 11, 12, 15, },
  { 1, 4, 6, 7, 11, 12, 15, },
  { 0, 1, 4, 6, 7, 11, 12, 15, },
  { 2, 4, 6, 7, 11, 12, 15, },
  { 0, 2, 4, 6, 7, 11, 12, 15, },
  { 1, 2, 4, 6, 7, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 11, 12, 15, },
  { 3, 4, 6, 7, 11, 12, 15, },
  { 0, 3, 4, 6, 7, 11, 12, 15, },
  { 1, 3, 4, 6, 7, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 11, 12, 15, },
  { 2, 3, 4, 6, 7, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 12, 15, },
  { 5, 6, 7, 11, 12, 15, },
  { 0, 5, 6, 7, 11, 12, 15, },
  { 1, 5, 6, 7, 11, 12, 15, },
  { 0, 1, 5, 6, 7, 11, 12, 15, },
  { 2, 5, 6, 7, 11, 12, 15, },
  { 0, 2, 5, 6, 7, 11, 12, 15, },
  { 1, 2, 5, 6, 7, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 11, 12, 15, },
  { 3, 5, 6, 7, 11, 12, 15, },
  { 0, 3, 5, 6, 7, 11, 12, 15, },
  { 1, 3, 5, 6, 7, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 11, 12, 15, },
  { 2, 3, 5, 6, 7, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 12, 15, },
  { 4, 5, 6, 7, 11, 12, 15, },
  { 0, 4, 5, 6, 7, 11, 12, 15, },
  { 1, 4, 5, 6, 7, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 11, 12, 15, },
  { 2, 4, 5, 6, 7, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 12, 15, },
  { 3, 4, 5, 6, 7, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 12, 15, },
  { 8, 11, 12, 15, },
  { 0, 8, 11, 12, 15, },
  { 1, 8, 11, 12, 15, },
  { 0, 1, 8, 11, 12, 15, },
  { 2, 8, 11, 12, 15, },
  { 0, 2, 8, 11, 12, 15, },
  { 1, 2, 8, 11, 12, 15, },
  { 0, 1, 2, 8, 11, 12, 15, },
  { 3, 8, 11, 12, 15, },
  { 0, 3, 8, 11, 12, 15, },
  { 1, 3, 8, 11, 12, 15, },
  { 0, 1, 3, 8, 11, 12, 15, },
  { 2, 3, 8, 11, 12, 15, },
  { 0, 2, 3, 8, 11, 12, 15, },
  { 1, 2, 3, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 8, 11, 12, 15, },
  { 4, 8, 11, 12, 15, },
  { 0, 4, 8, 11, 12, 15, },
  { 1, 4, 8, 11, 12, 15, },
  { 0, 1, 4, 8, 11, 12, 15, },
  { 2, 4, 8, 11, 12, 15, },
  { 0, 2, 4, 8, 11, 12, 15, },
  { 1, 2, 4, 8, 11, 12, 15, },
  { 0, 1, 2, 4, 8, 11, 12, 15, },
  { 3, 4, 8, 11, 12, 15, },
  { 0, 3, 4, 8, 11, 12, 15, },
  { 1, 3, 4, 8, 11, 12, 15, },
  { 0, 1, 3, 4, 8, 11, 12, 15, },
  { 2, 3, 4, 8, 11, 12, 15, },
  { 0, 2, 3, 4, 8, 11, 12, 15, },
  { 1, 2, 3, 4, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 8, 11, 12, 15, },
  { 5, 8, 11, 12, 15, },
  { 0, 5, 8, 11, 12, 15, },
  { 1, 5, 8, 11, 12, 15, },
  { 0, 1, 5, 8, 11, 12, 15, },
  { 2, 5, 8, 11, 12, 15, },
  { 0, 2, 5, 8, 11, 12, 15, },
  { 1, 2, 5, 8, 11, 12, 15, },
  { 0, 1, 2, 5, 8, 11, 12, 15, },
  { 3, 5, 8, 11, 12, 15, },
  { 0, 3, 5, 8, 11, 12, 15, },
  { 1, 3, 5, 8, 11, 12, 15, },
  { 0, 1, 3, 5, 8, 11, 12, 15, },
  { 2, 3, 5, 8, 11, 12, 15, },
  { 0, 2, 3, 5, 8, 11, 12, 15, },
  { 1, 2, 3, 5, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 8, 11, 12, 15, },
  { 4, 5, 8, 11, 12, 15, },
  { 0, 4, 5, 8, 11, 12, 15, },
  { 1, 4, 5, 8, 11, 12, 15, },
  { 0, 1, 4, 5, 8, 11, 12, 15, },
  { 2, 4, 5, 8, 11, 12, 15, },
  { 0, 2, 4, 5, 8, 11, 12, 15, },
  { 1, 2, 4, 5, 8, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 8, 11, 12, 15, },
  { 3, 4, 5, 8, 11, 12, 15, },
  { 0, 3, 4, 5, 8, 11, 12, 15, },
  { 1, 3, 4, 5, 8, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 8, 11, 12, 15, },
  { 2, 3, 4, 5, 8, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 8, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 12, 15, },
  { 6, 8, 11, 12, 15, },
  { 0, 6, 8, 11, 12, 15, },
  { 1, 6, 8, 11, 12, 15, },
  { 0, 1, 6, 8, 11, 12, 15, },
  { 2, 6, 8, 11, 12, 15, },
  { 0, 2, 6, 8, 11, 12, 15, },
  { 1, 2, 6, 8, 11, 12, 15, },
  { 0, 1, 2, 6, 8, 11, 12, 15, },
  { 3, 6, 8, 11, 12, 15, },
  { 0, 3, 6, 8, 11, 12, 15, },
  { 1, 3, 6, 8, 11, 12, 15, },
  { 0, 1, 3, 6, 8, 11, 12, 15, },
  { 2, 3, 6, 8, 11, 12, 15, },
  { 0, 2, 3, 6, 8, 11, 12, 15, },
  { 1, 2, 3, 6, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 8, 11, 12, 15, },
  { 4, 6, 8, 11, 12, 15, },
  { 0, 4, 6, 8, 11, 12, 15, },
  { 1, 4, 6, 8, 11, 12, 15, },
  { 0, 1, 4, 6, 8, 11, 12, 15, },
  { 2, 4, 6, 8, 11, 12, 15, },
  { 0, 2, 4, 6, 8, 11, 12, 15, },
  { 1, 2, 4, 6, 8, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 8, 11, 12, 15, },
  { 3, 4, 6, 8, 11, 12, 15, },
  { 0, 3, 4, 6, 8, 11, 12, 15, },
  { 1, 3, 4, 6, 8, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 8, 11, 12, 15, },
  { 2, 3, 4, 6, 8, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 8, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 12, 15, },
  { 5, 6, 8, 11, 12, 15, },
  { 0, 5, 6, 8, 11, 12, 15, },
  { 1, 5, 6, 8, 11, 12, 15, },
  { 0, 1, 5, 6, 8, 11, 12, 15, },
  { 2, 5, 6, 8, 11, 12, 15, },
  { 0, 2, 5, 6, 8, 11, 12, 15, },
  { 1, 2, 5, 6, 8, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 8, 11, 12, 15, },
  { 3, 5, 6, 8, 11, 12, 15, },
  { 0, 3, 5, 6, 8, 11, 12, 15, },
  { 1, 3, 5, 6, 8, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 8, 11, 12, 15, },
  { 2, 3, 5, 6, 8, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 8, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 12, 15, },
  { 4, 5, 6, 8, 11, 12, 15, },
  { 0, 4, 5, 6, 8, 11, 12, 15, },
  { 1, 4, 5, 6, 8, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 8, 11, 12, 15, },
  { 2, 4, 5, 6, 8, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 8, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 8, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 12, 15, },
  { 3, 4, 5, 6, 8, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 8, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 8, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 8, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 12, 15, },
  { 7, 8, 11, 12, 15, },
  { 0, 7, 8, 11, 12, 15, },
  { 1, 7, 8, 11, 12, 15, },
  { 0, 1, 7, 8, 11, 12, 15, },
  { 2, 7, 8, 11, 12, 15, },
  { 0, 2, 7, 8, 11, 12, 15, },
  { 1, 2, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 7, 8, 11, 12, 15, },
  { 3, 7, 8, 11, 12, 15, },
  { 0, 3, 7, 8, 11, 12, 15, },
  { 1, 3, 7, 8, 11, 12, 15, },
  { 0, 1, 3, 7, 8, 11, 12, 15, },
  { 2, 3, 7, 8, 11, 12, 15, },
  { 0, 2, 3, 7, 8, 11, 12, 15, },
  { 1, 2, 3, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 7, 8, 11, 12, 15, },
  { 4, 7, 8, 11, 12, 15, },
  { 0, 4, 7, 8, 11, 12, 15, },
  { 1, 4, 7, 8, 11, 12, 15, },
  { 0, 1, 4, 7, 8, 11, 12, 15, },
  { 2, 4, 7, 8, 11, 12, 15, },
  { 0, 2, 4, 7, 8, 11, 12, 15, },
  { 1, 2, 4, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 4, 7, 8, 11, 12, 15, },
  { 3, 4, 7, 8, 11, 12, 15, },
  { 0, 3, 4, 7, 8, 11, 12, 15, },
  { 1, 3, 4, 7, 8, 11, 12, 15, },
  { 0, 1, 3, 4, 7, 8, 11, 12, 15, },
  { 2, 3, 4, 7, 8, 11, 12, 15, },
  { 0, 2, 3, 4, 7, 8, 11, 12, 15, },
  { 1, 2, 3, 4, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 12, 15, },
  { 5, 7, 8, 11, 12, 15, },
  { 0, 5, 7, 8, 11, 12, 15, },
  { 1, 5, 7, 8, 11, 12, 15, },
  { 0, 1, 5, 7, 8, 11, 12, 15, },
  { 2, 5, 7, 8, 11, 12, 15, },
  { 0, 2, 5, 7, 8, 11, 12, 15, },
  { 1, 2, 5, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 5, 7, 8, 11, 12, 15, },
  { 3, 5, 7, 8, 11, 12, 15, },
  { 0, 3, 5, 7, 8, 11, 12, 15, },
  { 1, 3, 5, 7, 8, 11, 12, 15, },
  { 0, 1, 3, 5, 7, 8, 11, 12, 15, },
  { 2, 3, 5, 7, 8, 11, 12, 15, },
  { 0, 2, 3, 5, 7, 8, 11, 12, 15, },
  { 1, 2, 3, 5, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 12, 15, },
  { 4, 5, 7, 8, 11, 12, 15, },
  { 0, 4, 5, 7, 8, 11, 12, 15, },
  { 1, 4, 5, 7, 8, 11, 12, 15, },
  { 0, 1, 4, 5, 7, 8, 11, 12, 15, },
  { 2, 4, 5, 7, 8, 11, 12, 15, },
  { 0, 2, 4, 5, 7, 8, 11, 12, 15, },
  { 1, 2, 4, 5, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 12, 15, },
  { 3, 4, 5, 7, 8, 11, 12, 15, },
  { 0, 3, 4, 5, 7, 8, 11, 12, 15, },
  { 1, 3, 4, 5, 7, 8, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 12, 15, },
  { 2, 3, 4, 5, 7, 8, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 12, 15, },
  { 6, 7, 8, 11, 12, 15, },
  { 0, 6, 7, 8, 11, 12, 15, },
  { 1, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 6, 7, 8, 11, 12, 15, },
  { 2, 6, 7, 8, 11, 12, 15, },
  { 0, 2, 6, 7, 8, 11, 12, 15, },
  { 1, 2, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 6, 7, 8, 11, 12, 15, },
  { 3, 6, 7, 8, 11, 12, 15, },
  { 0, 3, 6, 7, 8, 11, 12, 15, },
  { 1, 3, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 3, 6, 7, 8, 11, 12, 15, },
  { 2, 3, 6, 7, 8, 11, 12, 15, },
  { 0, 2, 3, 6, 7, 8, 11, 12, 15, },
  { 1, 2, 3, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 12, 15, },
  { 4, 6, 7, 8, 11, 12, 15, },
  { 0, 4, 6, 7, 8, 11, 12, 15, },
  { 1, 4, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 4, 6, 7, 8, 11, 12, 15, },
  { 2, 4, 6, 7, 8, 11, 12, 15, },
  { 0, 2, 4, 6, 7, 8, 11, 12, 15, },
  { 1, 2, 4, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 12, 15, },
  { 3, 4, 6, 7, 8, 11, 12, 15, },
  { 0, 3, 4, 6, 7, 8, 11, 12, 15, },
  { 1, 3, 4, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 12, 15, },
  { 2, 3, 4, 6, 7, 8, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 12, 15, },
  { 5, 6, 7, 8, 11, 12, 15, },
  { 0, 5, 6, 7, 8, 11, 12, 15, },
  { 1, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 5, 6, 7, 8, 11, 12, 15, },
  { 2, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 2, 5, 6, 7, 8, 11, 12, 15, },
  { 1, 2, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 12, 15, },
  { 3, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 3, 5, 6, 7, 8, 11, 12, 15, },
  { 1, 3, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 12, 15, },
  { 2, 3, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 12, 15, },
  { 4, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 1, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 2, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 3, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 15, },
  { 9, 11, 12, 15, },
  { 0, 9, 11, 12, 15, },
  { 1, 9, 11, 12, 15, },
  { 0, 1, 9, 11, 12, 15, },
  { 2, 9, 11, 12, 15, },
  { 0, 2, 9, 11, 12, 15, },
  { 1, 2, 9, 11, 12, 15, },
  { 0, 1, 2, 9, 11, 12, 15, },
  { 3, 9, 11, 12, 15, },
  { 0, 3, 9, 11, 12, 15, },
  { 1, 3, 9, 11, 12, 15, },
  { 0, 1, 3, 9, 11, 12, 15, },
  { 2, 3, 9, 11, 12, 15, },
  { 0, 2, 3, 9, 11, 12, 15, },
  { 1, 2, 3, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 9, 11, 12, 15, },
  { 4, 9, 11, 12, 15, },
  { 0, 4, 9, 11, 12, 15, },
  { 1, 4, 9, 11, 12, 15, },
  { 0, 1, 4, 9, 11, 12, 15, },
  { 2, 4, 9, 11, 12, 15, },
  { 0, 2, 4, 9, 11, 12, 15, },
  { 1, 2, 4, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 9, 11, 12, 15, },
  { 3, 4, 9, 11, 12, 15, },
  { 0, 3, 4, 9, 11, 12, 15, },
  { 1, 3, 4, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 9, 11, 12, 15, },
  { 2, 3, 4, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 9, 11, 12, 15, },
  { 5, 9, 11, 12, 15, },
  { 0, 5, 9, 11, 12, 15, },
  { 1, 5, 9, 11, 12, 15, },
  { 0, 1, 5, 9, 11, 12, 15, },
  { 2, 5, 9, 11, 12, 15, },
  { 0, 2, 5, 9, 11, 12, 15, },
  { 1, 2, 5, 9, 11, 12, 15, },
  { 0, 1, 2, 5, 9, 11, 12, 15, },
  { 3, 5, 9, 11, 12, 15, },
  { 0, 3, 5, 9, 11, 12, 15, },
  { 1, 3, 5, 9, 11, 12, 15, },
  { 0, 1, 3, 5, 9, 11, 12, 15, },
  { 2, 3, 5, 9, 11, 12, 15, },
  { 0, 2, 3, 5, 9, 11, 12, 15, },
  { 1, 2, 3, 5, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 9, 11, 12, 15, },
  { 4, 5, 9, 11, 12, 15, },
  { 0, 4, 5, 9, 11, 12, 15, },
  { 1, 4, 5, 9, 11, 12, 15, },
  { 0, 1, 4, 5, 9, 11, 12, 15, },
  { 2, 4, 5, 9, 11, 12, 15, },
  { 0, 2, 4, 5, 9, 11, 12, 15, },
  { 1, 2, 4, 5, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 9, 11, 12, 15, },
  { 3, 4, 5, 9, 11, 12, 15, },
  { 0, 3, 4, 5, 9, 11, 12, 15, },
  { 1, 3, 4, 5, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 9, 11, 12, 15, },
  { 2, 3, 4, 5, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 12, 15, },
  { 6, 9, 11, 12, 15, },
  { 0, 6, 9, 11, 12, 15, },
  { 1, 6, 9, 11, 12, 15, },
  { 0, 1, 6, 9, 11, 12, 15, },
  { 2, 6, 9, 11, 12, 15, },
  { 0, 2, 6, 9, 11, 12, 15, },
  { 1, 2, 6, 9, 11, 12, 15, },
  { 0, 1, 2, 6, 9, 11, 12, 15, },
  { 3, 6, 9, 11, 12, 15, },
  { 0, 3, 6, 9, 11, 12, 15, },
  { 1, 3, 6, 9, 11, 12, 15, },
  { 0, 1, 3, 6, 9, 11, 12, 15, },
  { 2, 3, 6, 9, 11, 12, 15, },
  { 0, 2, 3, 6, 9, 11, 12, 15, },
  { 1, 2, 3, 6, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 9, 11, 12, 15, },
  { 4, 6, 9, 11, 12, 15, },
  { 0, 4, 6, 9, 11, 12, 15, },
  { 1, 4, 6, 9, 11, 12, 15, },
  { 0, 1, 4, 6, 9, 11, 12, 15, },
  { 2, 4, 6, 9, 11, 12, 15, },
  { 0, 2, 4, 6, 9, 11, 12, 15, },
  { 1, 2, 4, 6, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 9, 11, 12, 15, },
  { 3, 4, 6, 9, 11, 12, 15, },
  { 0, 3, 4, 6, 9, 11, 12, 15, },
  { 1, 3, 4, 6, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 9, 11, 12, 15, },
  { 2, 3, 4, 6, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 12, 15, },
  { 5, 6, 9, 11, 12, 15, },
  { 0, 5, 6, 9, 11, 12, 15, },
  { 1, 5, 6, 9, 11, 12, 15, },
  { 0, 1, 5, 6, 9, 11, 12, 15, },
  { 2, 5, 6, 9, 11, 12, 15, },
  { 0, 2, 5, 6, 9, 11, 12, 15, },
  { 1, 2, 5, 6, 9, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 9, 11, 12, 15, },
  { 3, 5, 6, 9, 11, 12, 15, },
  { 0, 3, 5, 6, 9, 11, 12, 15, },
  { 1, 3, 5, 6, 9, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 9, 11, 12, 15, },
  { 2, 3, 5, 6, 9, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 9, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 12, 15, },
  { 4, 5, 6, 9, 11, 12, 15, },
  { 0, 4, 5, 6, 9, 11, 12, 15, },
  { 1, 4, 5, 6, 9, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 9, 11, 12, 15, },
  { 2, 4, 5, 6, 9, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 9, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 12, 15, },
  { 3, 4, 5, 6, 9, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 9, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 12, 15, },
  { 7, 9, 11, 12, 15, },
  { 0, 7, 9, 11, 12, 15, },
  { 1, 7, 9, 11, 12, 15, },
  { 0, 1, 7, 9, 11, 12, 15, },
  { 2, 7, 9, 11, 12, 15, },
  { 0, 2, 7, 9, 11, 12, 15, },
  { 1, 2, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 7, 9, 11, 12, 15, },
  { 3, 7, 9, 11, 12, 15, },
  { 0, 3, 7, 9, 11, 12, 15, },
  { 1, 3, 7, 9, 11, 12, 15, },
  { 0, 1, 3, 7, 9, 11, 12, 15, },
  { 2, 3, 7, 9, 11, 12, 15, },
  { 0, 2, 3, 7, 9, 11, 12, 15, },
  { 1, 2, 3, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 7, 9, 11, 12, 15, },
  { 4, 7, 9, 11, 12, 15, },
  { 0, 4, 7, 9, 11, 12, 15, },
  { 1, 4, 7, 9, 11, 12, 15, },
  { 0, 1, 4, 7, 9, 11, 12, 15, },
  { 2, 4, 7, 9, 11, 12, 15, },
  { 0, 2, 4, 7, 9, 11, 12, 15, },
  { 1, 2, 4, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 7, 9, 11, 12, 15, },
  { 3, 4, 7, 9, 11, 12, 15, },
  { 0, 3, 4, 7, 9, 11, 12, 15, },
  { 1, 3, 4, 7, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 7, 9, 11, 12, 15, },
  { 2, 3, 4, 7, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 7, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 12, 15, },
  { 5, 7, 9, 11, 12, 15, },
  { 0, 5, 7, 9, 11, 12, 15, },
  { 1, 5, 7, 9, 11, 12, 15, },
  { 0, 1, 5, 7, 9, 11, 12, 15, },
  { 2, 5, 7, 9, 11, 12, 15, },
  { 0, 2, 5, 7, 9, 11, 12, 15, },
  { 1, 2, 5, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 5, 7, 9, 11, 12, 15, },
  { 3, 5, 7, 9, 11, 12, 15, },
  { 0, 3, 5, 7, 9, 11, 12, 15, },
  { 1, 3, 5, 7, 9, 11, 12, 15, },
  { 0, 1, 3, 5, 7, 9, 11, 12, 15, },
  { 2, 3, 5, 7, 9, 11, 12, 15, },
  { 0, 2, 3, 5, 7, 9, 11, 12, 15, },
  { 1, 2, 3, 5, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 12, 15, },
  { 4, 5, 7, 9, 11, 12, 15, },
  { 0, 4, 5, 7, 9, 11, 12, 15, },
  { 1, 4, 5, 7, 9, 11, 12, 15, },
  { 0, 1, 4, 5, 7, 9, 11, 12, 15, },
  { 2, 4, 5, 7, 9, 11, 12, 15, },
  { 0, 2, 4, 5, 7, 9, 11, 12, 15, },
  { 1, 2, 4, 5, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 12, 15, },
  { 3, 4, 5, 7, 9, 11, 12, 15, },
  { 0, 3, 4, 5, 7, 9, 11, 12, 15, },
  { 1, 3, 4, 5, 7, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 12, 15, },
  { 2, 3, 4, 5, 7, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 12, 15, },
  { 6, 7, 9, 11, 12, 15, },
  { 0, 6, 7, 9, 11, 12, 15, },
  { 1, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 6, 7, 9, 11, 12, 15, },
  { 2, 6, 7, 9, 11, 12, 15, },
  { 0, 2, 6, 7, 9, 11, 12, 15, },
  { 1, 2, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 6, 7, 9, 11, 12, 15, },
  { 3, 6, 7, 9, 11, 12, 15, },
  { 0, 3, 6, 7, 9, 11, 12, 15, },
  { 1, 3, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 3, 6, 7, 9, 11, 12, 15, },
  { 2, 3, 6, 7, 9, 11, 12, 15, },
  { 0, 2, 3, 6, 7, 9, 11, 12, 15, },
  { 1, 2, 3, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 12, 15, },
  { 4, 6, 7, 9, 11, 12, 15, },
  { 0, 4, 6, 7, 9, 11, 12, 15, },
  { 1, 4, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 4, 6, 7, 9, 11, 12, 15, },
  { 2, 4, 6, 7, 9, 11, 12, 15, },
  { 0, 2, 4, 6, 7, 9, 11, 12, 15, },
  { 1, 2, 4, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 12, 15, },
  { 3, 4, 6, 7, 9, 11, 12, 15, },
  { 0, 3, 4, 6, 7, 9, 11, 12, 15, },
  { 1, 3, 4, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 12, 15, },
  { 2, 3, 4, 6, 7, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 12, 15, },
  { 5, 6, 7, 9, 11, 12, 15, },
  { 0, 5, 6, 7, 9, 11, 12, 15, },
  { 1, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 5, 6, 7, 9, 11, 12, 15, },
  { 2, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 2, 5, 6, 7, 9, 11, 12, 15, },
  { 1, 2, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 12, 15, },
  { 3, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 3, 5, 6, 7, 9, 11, 12, 15, },
  { 1, 3, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 12, 15, },
  { 2, 3, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 12, 15, },
  { 4, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 1, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 2, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 3, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 15, },
  { 8, 9, 11, 12, 15, },
  { 0, 8, 9, 11, 12, 15, },
  { 1, 8, 9, 11, 12, 15, },
  { 0, 1, 8, 9, 11, 12, 15, },
  { 2, 8, 9, 11, 12, 15, },
  { 0, 2, 8, 9, 11, 12, 15, },
  { 1, 2, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 8, 9, 11, 12, 15, },
  { 3, 8, 9, 11, 12, 15, },
  { 0, 3, 8, 9, 11, 12, 15, },
  { 1, 3, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 8, 9, 11, 12, 15, },
  { 2, 3, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 8, 9, 11, 12, 15, },
  { 4, 8, 9, 11, 12, 15, },
  { 0, 4, 8, 9, 11, 12, 15, },
  { 1, 4, 8, 9, 11, 12, 15, },
  { 0, 1, 4, 8, 9, 11, 12, 15, },
  { 2, 4, 8, 9, 11, 12, 15, },
  { 0, 2, 4, 8, 9, 11, 12, 15, },
  { 1, 2, 4, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 8, 9, 11, 12, 15, },
  { 3, 4, 8, 9, 11, 12, 15, },
  { 0, 3, 4, 8, 9, 11, 12, 15, },
  { 1, 3, 4, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 8, 9, 11, 12, 15, },
  { 2, 3, 4, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 12, 15, },
  { 5, 8, 9, 11, 12, 15, },
  { 0, 5, 8, 9, 11, 12, 15, },
  { 1, 5, 8, 9, 11, 12, 15, },
  { 0, 1, 5, 8, 9, 11, 12, 15, },
  { 2, 5, 8, 9, 11, 12, 15, },
  { 0, 2, 5, 8, 9, 11, 12, 15, },
  { 1, 2, 5, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 5, 8, 9, 11, 12, 15, },
  { 3, 5, 8, 9, 11, 12, 15, },
  { 0, 3, 5, 8, 9, 11, 12, 15, },
  { 1, 3, 5, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 5, 8, 9, 11, 12, 15, },
  { 2, 3, 5, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 5, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 5, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 12, 15, },
  { 4, 5, 8, 9, 11, 12, 15, },
  { 0, 4, 5, 8, 9, 11, 12, 15, },
  { 1, 4, 5, 8, 9, 11, 12, 15, },
  { 0, 1, 4, 5, 8, 9, 11, 12, 15, },
  { 2, 4, 5, 8, 9, 11, 12, 15, },
  { 0, 2, 4, 5, 8, 9, 11, 12, 15, },
  { 1, 2, 4, 5, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 12, 15, },
  { 3, 4, 5, 8, 9, 11, 12, 15, },
  { 0, 3, 4, 5, 8, 9, 11, 12, 15, },
  { 1, 3, 4, 5, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 12, 15, },
  { 2, 3, 4, 5, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 12, 15, },
  { 6, 8, 9, 11, 12, 15, },
  { 0, 6, 8, 9, 11, 12, 15, },
  { 1, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 6, 8, 9, 11, 12, 15, },
  { 2, 6, 8, 9, 11, 12, 15, },
  { 0, 2, 6, 8, 9, 11, 12, 15, },
  { 1, 2, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 6, 8, 9, 11, 12, 15, },
  { 3, 6, 8, 9, 11, 12, 15, },
  { 0, 3, 6, 8, 9, 11, 12, 15, },
  { 1, 3, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 6, 8, 9, 11, 12, 15, },
  { 2, 3, 6, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 6, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 12, 15, },
  { 4, 6, 8, 9, 11, 12, 15, },
  { 0, 4, 6, 8, 9, 11, 12, 15, },
  { 1, 4, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 4, 6, 8, 9, 11, 12, 15, },
  { 2, 4, 6, 8, 9, 11, 12, 15, },
  { 0, 2, 4, 6, 8, 9, 11, 12, 15, },
  { 1, 2, 4, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 12, 15, },
  { 3, 4, 6, 8, 9, 11, 12, 15, },
  { 0, 3, 4, 6, 8, 9, 11, 12, 15, },
  { 1, 3, 4, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 12, 15, },
  { 2, 3, 4, 6, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 12, 15, },
  { 5, 6, 8, 9, 11, 12, 15, },
  { 0, 5, 6, 8, 9, 11, 12, 15, },
  { 1, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 5, 6, 8, 9, 11, 12, 15, },
  { 2, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 2, 5, 6, 8, 9, 11, 12, 15, },
  { 1, 2, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 12, 15, },
  { 3, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 3, 5, 6, 8, 9, 11, 12, 15, },
  { 1, 3, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 12, 15, },
  { 2, 3, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 12, 15, },
  { 4, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 1, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 2, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 3, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 15, },
  { 7, 8, 9, 11, 12, 15, },
  { 0, 7, 8, 9, 11, 12, 15, },
  { 1, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 7, 8, 9, 11, 12, 15, },
  { 2, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 7, 8, 9, 11, 12, 15, },
  { 3, 7, 8, 9, 11, 12, 15, },
  { 0, 3, 7, 8, 9, 11, 12, 15, },
  { 1, 3, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 7, 8, 9, 11, 12, 15, },
  { 2, 3, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 12, 15, },
  { 4, 7, 8, 9, 11, 12, 15, },
  { 0, 4, 7, 8, 9, 11, 12, 15, },
  { 1, 4, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 4, 7, 8, 9, 11, 12, 15, },
  { 2, 4, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 4, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 4, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 12, 15, },
  { 3, 4, 7, 8, 9, 11, 12, 15, },
  { 0, 3, 4, 7, 8, 9, 11, 12, 15, },
  { 1, 3, 4, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 12, 15, },
  { 2, 3, 4, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 12, 15, },
  { 5, 7, 8, 9, 11, 12, 15, },
  { 0, 5, 7, 8, 9, 11, 12, 15, },
  { 1, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 5, 7, 8, 9, 11, 12, 15, },
  { 2, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 5, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 12, 15, },
  { 3, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 3, 5, 7, 8, 9, 11, 12, 15, },
  { 1, 3, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 12, 15, },
  { 2, 3, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 12, 15, },
  { 4, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 1, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 2, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 3, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 15, },
  { 6, 7, 8, 9, 11, 12, 15, },
  { 0, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 6, 7, 8, 9, 11, 12, 15, },
  { 2, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 12, 15, },
  { 3, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 3, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 3, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 12, 15, },
  { 2, 3, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 12, 15, },
  { 4, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 2, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 3, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 15, },
  { 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 2, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 3, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 15, },
  { 10, 11, 12, 15, },
  { 0, 10, 11, 12, 15, },
  { 1, 10, 11, 12, 15, },
  { 0, 1, 10, 11, 12, 15, },
  { 2, 10, 11, 12, 15, },
  { 0, 2, 10, 11, 12, 15, },
  { 1, 2, 10, 11, 12, 15, },
  { 0, 1, 2, 10, 11, 12, 15, },
  { 3, 10, 11, 12, 15, },
  { 0, 3, 10, 11, 12, 15, },
  { 1, 3, 10, 11, 12, 15, },
  { 0, 1, 3, 10, 11, 12, 15, },
  { 2, 3, 10, 11, 12, 15, },
  { 0, 2, 3, 10, 11, 12, 15, },
  { 1, 2, 3, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 10, 11, 12, 15, },
  { 4, 10, 11, 12, 15, },
  { 0, 4, 10, 11, 12, 15, },
  { 1, 4, 10, 11, 12, 15, },
  { 0, 1, 4, 10, 11, 12, 15, },
  { 2, 4, 10, 11, 12, 15, },
  { 0, 2, 4, 10, 11, 12, 15, },
  { 1, 2, 4, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 10, 11, 12, 15, },
  { 3, 4, 10, 11, 12, 15, },
  { 0, 3, 4, 10, 11, 12, 15, },
  { 1, 3, 4, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 10, 11, 12, 15, },
  { 2, 3, 4, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 10, 11, 12, 15, },
  { 5, 10, 11, 12, 15, },
  { 0, 5, 10, 11, 12, 15, },
  { 1, 5, 10, 11, 12, 15, },
  { 0, 1, 5, 10, 11, 12, 15, },
  { 2, 5, 10, 11, 12, 15, },
  { 0, 2, 5, 10, 11, 12, 15, },
  { 1, 2, 5, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 10, 11, 12, 15, },
  { 3, 5, 10, 11, 12, 15, },
  { 0, 3, 5, 10, 11, 12, 15, },
  { 1, 3, 5, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 10, 11, 12, 15, },
  { 2, 3, 5, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 10, 11, 12, 15, },
  { 4, 5, 10, 11, 12, 15, },
  { 0, 4, 5, 10, 11, 12, 15, },
  { 1, 4, 5, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 10, 11, 12, 15, },
  { 2, 4, 5, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 10, 11, 12, 15, },
  { 3, 4, 5, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 12, 15, },
  { 6, 10, 11, 12, 15, },
  { 0, 6, 10, 11, 12, 15, },
  { 1, 6, 10, 11, 12, 15, },
  { 0, 1, 6, 10, 11, 12, 15, },
  { 2, 6, 10, 11, 12, 15, },
  { 0, 2, 6, 10, 11, 12, 15, },
  { 1, 2, 6, 10, 11, 12, 15, },
  { 0, 1, 2, 6, 10, 11, 12, 15, },
  { 3, 6, 10, 11, 12, 15, },
  { 0, 3, 6, 10, 11, 12, 15, },
  { 1, 3, 6, 10, 11, 12, 15, },
  { 0, 1, 3, 6, 10, 11, 12, 15, },
  { 2, 3, 6, 10, 11, 12, 15, },
  { 0, 2, 3, 6, 10, 11, 12, 15, },
  { 1, 2, 3, 6, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 10, 11, 12, 15, },
  { 4, 6, 10, 11, 12, 15, },
  { 0, 4, 6, 10, 11, 12, 15, },
  { 1, 4, 6, 10, 11, 12, 15, },
  { 0, 1, 4, 6, 10, 11, 12, 15, },
  { 2, 4, 6, 10, 11, 12, 15, },
  { 0, 2, 4, 6, 10, 11, 12, 15, },
  { 1, 2, 4, 6, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 10, 11, 12, 15, },
  { 3, 4, 6, 10, 11, 12, 15, },
  { 0, 3, 4, 6, 10, 11, 12, 15, },
  { 1, 3, 4, 6, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 10, 11, 12, 15, },
  { 2, 3, 4, 6, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 12, 15, },
  { 5, 6, 10, 11, 12, 15, },
  { 0, 5, 6, 10, 11, 12, 15, },
  { 1, 5, 6, 10, 11, 12, 15, },
  { 0, 1, 5, 6, 10, 11, 12, 15, },
  { 2, 5, 6, 10, 11, 12, 15, },
  { 0, 2, 5, 6, 10, 11, 12, 15, },
  { 1, 2, 5, 6, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 10, 11, 12, 15, },
  { 3, 5, 6, 10, 11, 12, 15, },
  { 0, 3, 5, 6, 10, 11, 12, 15, },
  { 1, 3, 5, 6, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 10, 11, 12, 15, },
  { 2, 3, 5, 6, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 12, 15, },
  { 4, 5, 6, 10, 11, 12, 15, },
  { 0, 4, 5, 6, 10, 11, 12, 15, },
  { 1, 4, 5, 6, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 10, 11, 12, 15, },
  { 2, 4, 5, 6, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 12, 15, },
  { 3, 4, 5, 6, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 15, },
  { 7, 10, 11, 12, 15, },
  { 0, 7, 10, 11, 12, 15, },
  { 1, 7, 10, 11, 12, 15, },
  { 0, 1, 7, 10, 11, 12, 15, },
  { 2, 7, 10, 11, 12, 15, },
  { 0, 2, 7, 10, 11, 12, 15, },
  { 1, 2, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 7, 10, 11, 12, 15, },
  { 3, 7, 10, 11, 12, 15, },
  { 0, 3, 7, 10, 11, 12, 15, },
  { 1, 3, 7, 10, 11, 12, 15, },
  { 0, 1, 3, 7, 10, 11, 12, 15, },
  { 2, 3, 7, 10, 11, 12, 15, },
  { 0, 2, 3, 7, 10, 11, 12, 15, },
  { 1, 2, 3, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 7, 10, 11, 12, 15, },
  { 4, 7, 10, 11, 12, 15, },
  { 0, 4, 7, 10, 11, 12, 15, },
  { 1, 4, 7, 10, 11, 12, 15, },
  { 0, 1, 4, 7, 10, 11, 12, 15, },
  { 2, 4, 7, 10, 11, 12, 15, },
  { 0, 2, 4, 7, 10, 11, 12, 15, },
  { 1, 2, 4, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 7, 10, 11, 12, 15, },
  { 3, 4, 7, 10, 11, 12, 15, },
  { 0, 3, 4, 7, 10, 11, 12, 15, },
  { 1, 3, 4, 7, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 7, 10, 11, 12, 15, },
  { 2, 3, 4, 7, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 7, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 12, 15, },
  { 5, 7, 10, 11, 12, 15, },
  { 0, 5, 7, 10, 11, 12, 15, },
  { 1, 5, 7, 10, 11, 12, 15, },
  { 0, 1, 5, 7, 10, 11, 12, 15, },
  { 2, 5, 7, 10, 11, 12, 15, },
  { 0, 2, 5, 7, 10, 11, 12, 15, },
  { 1, 2, 5, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 7, 10, 11, 12, 15, },
  { 3, 5, 7, 10, 11, 12, 15, },
  { 0, 3, 5, 7, 10, 11, 12, 15, },
  { 1, 3, 5, 7, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 7, 10, 11, 12, 15, },
  { 2, 3, 5, 7, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 7, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 12, 15, },
  { 4, 5, 7, 10, 11, 12, 15, },
  { 0, 4, 5, 7, 10, 11, 12, 15, },
  { 1, 4, 5, 7, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 7, 10, 11, 12, 15, },
  { 2, 4, 5, 7, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 7, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 12, 15, },
  { 3, 4, 5, 7, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 7, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 7, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 7, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 12, 15, },
  { 6, 7, 10, 11, 12, 15, },
  { 0, 6, 7, 10, 11, 12, 15, },
  { 1, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 6, 7, 10, 11, 12, 15, },
  { 2, 6, 7, 10, 11, 12, 15, },
  { 0, 2, 6, 7, 10, 11, 12, 15, },
  { 1, 2, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 6, 7, 10, 11, 12, 15, },
  { 3, 6, 7, 10, 11, 12, 15, },
  { 0, 3, 6, 7, 10, 11, 12, 15, },
  { 1, 3, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 3, 6, 7, 10, 11, 12, 15, },
  { 2, 3, 6, 7, 10, 11, 12, 15, },
  { 0, 2, 3, 6, 7, 10, 11, 12, 15, },
  { 1, 2, 3, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 12, 15, },
  { 4, 6, 7, 10, 11, 12, 15, },
  { 0, 4, 6, 7, 10, 11, 12, 15, },
  { 1, 4, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 4, 6, 7, 10, 11, 12, 15, },
  { 2, 4, 6, 7, 10, 11, 12, 15, },
  { 0, 2, 4, 6, 7, 10, 11, 12, 15, },
  { 1, 2, 4, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 12, 15, },
  { 3, 4, 6, 7, 10, 11, 12, 15, },
  { 0, 3, 4, 6, 7, 10, 11, 12, 15, },
  { 1, 3, 4, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 12, 15, },
  { 2, 3, 4, 6, 7, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 12, 15, },
  { 5, 6, 7, 10, 11, 12, 15, },
  { 0, 5, 6, 7, 10, 11, 12, 15, },
  { 1, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 5, 6, 7, 10, 11, 12, 15, },
  { 2, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 2, 5, 6, 7, 10, 11, 12, 15, },
  { 1, 2, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 12, 15, },
  { 3, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 3, 5, 6, 7, 10, 11, 12, 15, },
  { 1, 3, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 12, 15, },
  { 2, 3, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 12, 15, },
  { 4, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 1, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 2, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 3, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 15, },
  { 8, 10, 11, 12, 15, },
  { 0, 8, 10, 11, 12, 15, },
  { 1, 8, 10, 11, 12, 15, },
  { 0, 1, 8, 10, 11, 12, 15, },
  { 2, 8, 10, 11, 12, 15, },
  { 0, 2, 8, 10, 11, 12, 15, },
  { 1, 2, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 8, 10, 11, 12, 15, },
  { 3, 8, 10, 11, 12, 15, },
  { 0, 3, 8, 10, 11, 12, 15, },
  { 1, 3, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 8, 10, 11, 12, 15, },
  { 2, 3, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 8, 10, 11, 12, 15, },
  { 4, 8, 10, 11, 12, 15, },
  { 0, 4, 8, 10, 11, 12, 15, },
  { 1, 4, 8, 10, 11, 12, 15, },
  { 0, 1, 4, 8, 10, 11, 12, 15, },
  { 2, 4, 8, 10, 11, 12, 15, },
  { 0, 2, 4, 8, 10, 11, 12, 15, },
  { 1, 2, 4, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 8, 10, 11, 12, 15, },
  { 3, 4, 8, 10, 11, 12, 15, },
  { 0, 3, 4, 8, 10, 11, 12, 15, },
  { 1, 3, 4, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 8, 10, 11, 12, 15, },
  { 2, 3, 4, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 12, 15, },
  { 5, 8, 10, 11, 12, 15, },
  { 0, 5, 8, 10, 11, 12, 15, },
  { 1, 5, 8, 10, 11, 12, 15, },
  { 0, 1, 5, 8, 10, 11, 12, 15, },
  { 2, 5, 8, 10, 11, 12, 15, },
  { 0, 2, 5, 8, 10, 11, 12, 15, },
  { 1, 2, 5, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 8, 10, 11, 12, 15, },
  { 3, 5, 8, 10, 11, 12, 15, },
  { 0, 3, 5, 8, 10, 11, 12, 15, },
  { 1, 3, 5, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 8, 10, 11, 12, 15, },
  { 2, 3, 5, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 12, 15, },
  { 4, 5, 8, 10, 11, 12, 15, },
  { 0, 4, 5, 8, 10, 11, 12, 15, },
  { 1, 4, 5, 8, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 8, 10, 11, 12, 15, },
  { 2, 4, 5, 8, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 8, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 12, 15, },
  { 3, 4, 5, 8, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 8, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 12, 15, },
  { 6, 8, 10, 11, 12, 15, },
  { 0, 6, 8, 10, 11, 12, 15, },
  { 1, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 6, 8, 10, 11, 12, 15, },
  { 2, 6, 8, 10, 11, 12, 15, },
  { 0, 2, 6, 8, 10, 11, 12, 15, },
  { 1, 2, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 6, 8, 10, 11, 12, 15, },
  { 3, 6, 8, 10, 11, 12, 15, },
  { 0, 3, 6, 8, 10, 11, 12, 15, },
  { 1, 3, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 6, 8, 10, 11, 12, 15, },
  { 2, 3, 6, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 6, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 12, 15, },
  { 4, 6, 8, 10, 11, 12, 15, },
  { 0, 4, 6, 8, 10, 11, 12, 15, },
  { 1, 4, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 4, 6, 8, 10, 11, 12, 15, },
  { 2, 4, 6, 8, 10, 11, 12, 15, },
  { 0, 2, 4, 6, 8, 10, 11, 12, 15, },
  { 1, 2, 4, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 12, 15, },
  { 3, 4, 6, 8, 10, 11, 12, 15, },
  { 0, 3, 4, 6, 8, 10, 11, 12, 15, },
  { 1, 3, 4, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 12, 15, },
  { 2, 3, 4, 6, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 12, 15, },
  { 5, 6, 8, 10, 11, 12, 15, },
  { 0, 5, 6, 8, 10, 11, 12, 15, },
  { 1, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 5, 6, 8, 10, 11, 12, 15, },
  { 2, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 2, 5, 6, 8, 10, 11, 12, 15, },
  { 1, 2, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 12, 15, },
  { 3, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 3, 5, 6, 8, 10, 11, 12, 15, },
  { 1, 3, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 12, 15, },
  { 2, 3, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 12, 15, },
  { 4, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 1, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 2, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 3, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 15, },
  { 7, 8, 10, 11, 12, 15, },
  { 0, 7, 8, 10, 11, 12, 15, },
  { 1, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 7, 8, 10, 11, 12, 15, },
  { 2, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 7, 8, 10, 11, 12, 15, },
  { 3, 7, 8, 10, 11, 12, 15, },
  { 0, 3, 7, 8, 10, 11, 12, 15, },
  { 1, 3, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 7, 8, 10, 11, 12, 15, },
  { 2, 3, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 12, 15, },
  { 4, 7, 8, 10, 11, 12, 15, },
  { 0, 4, 7, 8, 10, 11, 12, 15, },
  { 1, 4, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 4, 7, 8, 10, 11, 12, 15, },
  { 2, 4, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 4, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 4, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 12, 15, },
  { 3, 4, 7, 8, 10, 11, 12, 15, },
  { 0, 3, 4, 7, 8, 10, 11, 12, 15, },
  { 1, 3, 4, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 12, 15, },
  { 2, 3, 4, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 12, 15, },
  { 5, 7, 8, 10, 11, 12, 15, },
  { 0, 5, 7, 8, 10, 11, 12, 15, },
  { 1, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 5, 7, 8, 10, 11, 12, 15, },
  { 2, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 5, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 12, 15, },
  { 3, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 3, 5, 7, 8, 10, 11, 12, 15, },
  { 1, 3, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 12, 15, },
  { 2, 3, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 12, 15, },
  { 4, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 1, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 2, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 3, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 15, },
  { 6, 7, 8, 10, 11, 12, 15, },
  { 0, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 6, 7, 8, 10, 11, 12, 15, },
  { 2, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 12, 15, },
  { 3, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 3, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 3, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 12, 15, },
  { 2, 3, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 12, 15, },
  { 4, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 2, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 3, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 15, },
  { 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 2, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 3, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 15, },
  { 9, 10, 11, 12, 15, },
  { 0, 9, 10, 11, 12, 15, },
  { 1, 9, 10, 11, 12, 15, },
  { 0, 1, 9, 10, 11, 12, 15, },
  { 2, 9, 10, 11, 12, 15, },
  { 0, 2, 9, 10, 11, 12, 15, },
  { 1, 2, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 9, 10, 11, 12, 15, },
  { 3, 9, 10, 11, 12, 15, },
  { 0, 3, 9, 10, 11, 12, 15, },
  { 1, 3, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 9, 10, 11, 12, 15, },
  { 2, 3, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 9, 10, 11, 12, 15, },
  { 4, 9, 10, 11, 12, 15, },
  { 0, 4, 9, 10, 11, 12, 15, },
  { 1, 4, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 9, 10, 11, 12, 15, },
  { 2, 4, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 9, 10, 11, 12, 15, },
  { 3, 4, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 12, 15, },
  { 5, 9, 10, 11, 12, 15, },
  { 0, 5, 9, 10, 11, 12, 15, },
  { 1, 5, 9, 10, 11, 12, 15, },
  { 0, 1, 5, 9, 10, 11, 12, 15, },
  { 2, 5, 9, 10, 11, 12, 15, },
  { 0, 2, 5, 9, 10, 11, 12, 15, },
  { 1, 2, 5, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 9, 10, 11, 12, 15, },
  { 3, 5, 9, 10, 11, 12, 15, },
  { 0, 3, 5, 9, 10, 11, 12, 15, },
  { 1, 3, 5, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 9, 10, 11, 12, 15, },
  { 2, 3, 5, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 12, 15, },
  { 4, 5, 9, 10, 11, 12, 15, },
  { 0, 4, 5, 9, 10, 11, 12, 15, },
  { 1, 4, 5, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 9, 10, 11, 12, 15, },
  { 2, 4, 5, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 12, 15, },
  { 3, 4, 5, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, 15, },
  { 6, 9, 10, 11, 12, 15, },
  { 0, 6, 9, 10, 11, 12, 15, },
  { 1, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 6, 9, 10, 11, 12, 15, },
  { 2, 6, 9, 10, 11, 12, 15, },
  { 0, 2, 6, 9, 10, 11, 12, 15, },
  { 1, 2, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 6, 9, 10, 11, 12, 15, },
  { 3, 6, 9, 10, 11, 12, 15, },
  { 0, 3, 6, 9, 10, 11, 12, 15, },
  { 1, 3, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 6, 9, 10, 11, 12, 15, },
  { 2, 3, 6, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 6, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 12, 15, },
  { 4, 6, 9, 10, 11, 12, 15, },
  { 0, 4, 6, 9, 10, 11, 12, 15, },
  { 1, 4, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 6, 9, 10, 11, 12, 15, },
  { 2, 4, 6, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 6, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 12, 15, },
  { 3, 4, 6, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 6, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 6, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 12, 15, },
  { 5, 6, 9, 10, 11, 12, 15, },
  { 0, 5, 6, 9, 10, 11, 12, 15, },
  { 1, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 5, 6, 9, 10, 11, 12, 15, },
  { 2, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 2, 5, 6, 9, 10, 11, 12, 15, },
  { 1, 2, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 12, 15, },
  { 3, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 3, 5, 6, 9, 10, 11, 12, 15, },
  { 1, 3, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 12, 15, },
  { 2, 3, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 12, 15, },
  { 4, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 1, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 2, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 3, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 15, },
  { 7, 9, 10, 11, 12, 15, },
  { 0, 7, 9, 10, 11, 12, 15, },
  { 1, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 7, 9, 10, 11, 12, 15, },
  { 2, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 7, 9, 10, 11, 12, 15, },
  { 3, 7, 9, 10, 11, 12, 15, },
  { 0, 3, 7, 9, 10, 11, 12, 15, },
  { 1, 3, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 7, 9, 10, 11, 12, 15, },
  { 2, 3, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 12, 15, },
  { 4, 7, 9, 10, 11, 12, 15, },
  { 0, 4, 7, 9, 10, 11, 12, 15, },
  { 1, 4, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 7, 9, 10, 11, 12, 15, },
  { 2, 4, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 12, 15, },
  { 3, 4, 7, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 7, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 12, 15, },
  { 5, 7, 9, 10, 11, 12, 15, },
  { 0, 5, 7, 9, 10, 11, 12, 15, },
  { 1, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 5, 7, 9, 10, 11, 12, 15, },
  { 2, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 5, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 12, 15, },
  { 3, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 3, 5, 7, 9, 10, 11, 12, 15, },
  { 1, 3, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 12, 15, },
  { 2, 3, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 12, 15, },
  { 4, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 1, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 2, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 3, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 15, },
  { 6, 7, 9, 10, 11, 12, 15, },
  { 0, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 6, 7, 9, 10, 11, 12, 15, },
  { 2, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 12, 15, },
  { 3, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 3, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 3, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 12, 15, },
  { 2, 3, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 12, 15, },
  { 4, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 2, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 3, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 15, },
  { 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 2, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 3, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 15, },
  { 8, 9, 10, 11, 12, 15, },
  { 0, 8, 9, 10, 11, 12, 15, },
  { 1, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 8, 9, 10, 11, 12, 15, },
  { 2, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 8, 9, 10, 11, 12, 15, },
  { 3, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 12, 15, },
  { 4, 8, 9, 10, 11, 12, 15, },
  { 0, 4, 8, 9, 10, 11, 12, 15, },
  { 1, 4, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 8, 9, 10, 11, 12, 15, },
  { 2, 4, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 12, 15, },
  { 3, 4, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 12, 15, },
  { 5, 8, 9, 10, 11, 12, 15, },
  { 0, 5, 8, 9, 10, 11, 12, 15, },
  { 1, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 5, 8, 9, 10, 11, 12, 15, },
  { 2, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 5, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 12, 15, },
  { 3, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 5, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 12, 15, },
  { 4, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 1, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 2, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 3, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 15, },
  { 6, 8, 9, 10, 11, 12, 15, },
  { 0, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 6, 8, 9, 10, 11, 12, 15, },
  { 2, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 12, 15, },
  { 3, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 12, 15, },
  { 4, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 2, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 3, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 15, },
  { 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 2, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 3, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, },
  { 7, 8, 9, 10, 11, 12, 15, },
  { 0, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 12, 15, },
  { 3, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 12, 15, },
  { 4, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 3, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 15, },
  { 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 3, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 15, },
  { 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 3, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, },
  { 13, 15, },
  { 0, 13, 15, },
  { 1, 13, 15, },
  { 0, 1, 13, 15, },
  { 2, 13, 15, },
  { 0, 2, 13, 15, },
  { 1, 2, 13, 15, },
  { 0, 1, 2, 13, 15, },
  { 3, 13, 15, },
  { 0, 3, 13, 15, },
  { 1, 3, 13, 15, },
  { 0, 1, 3, 13, 15, },
  { 2, 3, 13, 15, },
  { 0, 2, 3, 13, 15, },
  { 1, 2, 3, 13, 15, },
  { 0, 1, 2, 3, 13, 15, },
  { 4, 13, 15, },
  { 0, 4, 13, 15, },
  { 1, 4, 13, 15, },
  { 0, 1, 4, 13, 15, },
  { 2, 4, 13, 15, },
  { 0, 2, 4, 13, 15, },
  { 1, 2, 4, 13, 15, },
  { 0, 1, 2, 4, 13, 15, },
  { 3, 4, 13, 15, },
  { 0, 3, 4, 13, 15, },
  { 1, 3, 4, 13, 15, },
  { 0, 1, 3, 4, 13, 15, },
  { 2, 3, 4, 13, 15, },
  { 0, 2, 3, 4, 13, 15, },
  { 1, 2, 3, 4, 13, 15, },
  { 0, 1, 2, 3, 4, 13, 15, },
  { 5, 13, 15, },
  { 0, 5, 13, 15, },
  { 1, 5, 13, 15, },
  { 0, 1, 5, 13, 15, },
  { 2, 5, 13, 15, },
  { 0, 2, 5, 13, 15, },
  { 1, 2, 5, 13, 15, },
  { 0, 1, 2, 5, 13, 15, },
  { 3, 5, 13, 15, },
  { 0, 3, 5, 13, 15, },
  { 1, 3, 5, 13, 15, },
  { 0, 1, 3, 5, 13, 15, },
  { 2, 3, 5, 13, 15, },
  { 0, 2, 3, 5, 13, 15, },
  { 1, 2, 3, 5, 13, 15, },
  { 0, 1, 2, 3, 5, 13, 15, },
  { 4, 5, 13, 15, },
  { 0, 4, 5, 13, 15, },
  { 1, 4, 5, 13, 15, },
  { 0, 1, 4, 5, 13, 15, },
  { 2, 4, 5, 13, 15, },
  { 0, 2, 4, 5, 13, 15, },
  { 1, 2, 4, 5, 13, 15, },
  { 0, 1, 2, 4, 5, 13, 15, },
  { 3, 4, 5, 13, 15, },
  { 0, 3, 4, 5, 13, 15, },
  { 1, 3, 4, 5, 13, 15, },
  { 0, 1, 3, 4, 5, 13, 15, },
  { 2, 3, 4, 5, 13, 15, },
  { 0, 2, 3, 4, 5, 13, 15, },
  { 1, 2, 3, 4, 5, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 13, 15, },
  { 6, 13, 15, },
  { 0, 6, 13, 15, },
  { 1, 6, 13, 15, },
  { 0, 1, 6, 13, 15, },
  { 2, 6, 13, 15, },
  { 0, 2, 6, 13, 15, },
  { 1, 2, 6, 13, 15, },
  { 0, 1, 2, 6, 13, 15, },
  { 3, 6, 13, 15, },
  { 0, 3, 6, 13, 15, },
  { 1, 3, 6, 13, 15, },
  { 0, 1, 3, 6, 13, 15, },
  { 2, 3, 6, 13, 15, },
  { 0, 2, 3, 6, 13, 15, },
  { 1, 2, 3, 6, 13, 15, },
  { 0, 1, 2, 3, 6, 13, 15, },
  { 4, 6, 13, 15, },
  { 0, 4, 6, 13, 15, },
  { 1, 4, 6, 13, 15, },
  { 0, 1, 4, 6, 13, 15, },
  { 2, 4, 6, 13, 15, },
  { 0, 2, 4, 6, 13, 15, },
  { 1, 2, 4, 6, 13, 15, },
  { 0, 1, 2, 4, 6, 13, 15, },
  { 3, 4, 6, 13, 15, },
  { 0, 3, 4, 6, 13, 15, },
  { 1, 3, 4, 6, 13, 15, },
  { 0, 1, 3, 4, 6, 13, 15, },
  { 2, 3, 4, 6, 13, 15, },
  { 0, 2, 3, 4, 6, 13, 15, },
  { 1, 2, 3, 4, 6, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 13, 15, },
  { 5, 6, 13, 15, },
  { 0, 5, 6, 13, 15, },
  { 1, 5, 6, 13, 15, },
  { 0, 1, 5, 6, 13, 15, },
  { 2, 5, 6, 13, 15, },
  { 0, 2, 5, 6, 13, 15, },
  { 1, 2, 5, 6, 13, 15, },
  { 0, 1, 2, 5, 6, 13, 15, },
  { 3, 5, 6, 13, 15, },
  { 0, 3, 5, 6, 13, 15, },
  { 1, 3, 5, 6, 13, 15, },
  { 0, 1, 3, 5, 6, 13, 15, },
  { 2, 3, 5, 6, 13, 15, },
  { 0, 2, 3, 5, 6, 13, 15, },
  { 1, 2, 3, 5, 6, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 13, 15, },
  { 4, 5, 6, 13, 15, },
  { 0, 4, 5, 6, 13, 15, },
  { 1, 4, 5, 6, 13, 15, },
  { 0, 1, 4, 5, 6, 13, 15, },
  { 2, 4, 5, 6, 13, 15, },
  { 0, 2, 4, 5, 6, 13, 15, },
  { 1, 2, 4, 5, 6, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 13, 15, },
  { 3, 4, 5, 6, 13, 15, },
  { 0, 3, 4, 5, 6, 13, 15, },
  { 1, 3, 4, 5, 6, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 13, 15, },
  { 2, 3, 4, 5, 6, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 13, 15, },
  { 7, 13, 15, },
  { 0, 7, 13, 15, },
  { 1, 7, 13, 15, },
  { 0, 1, 7, 13, 15, },
  { 2, 7, 13, 15, },
  { 0, 2, 7, 13, 15, },
  { 1, 2, 7, 13, 15, },
  { 0, 1, 2, 7, 13, 15, },
  { 3, 7, 13, 15, },
  { 0, 3, 7, 13, 15, },
  { 1, 3, 7, 13, 15, },
  { 0, 1, 3, 7, 13, 15, },
  { 2, 3, 7, 13, 15, },
  { 0, 2, 3, 7, 13, 15, },
  { 1, 2, 3, 7, 13, 15, },
  { 0, 1, 2, 3, 7, 13, 15, },
  { 4, 7, 13, 15, },
  { 0, 4, 7, 13, 15, },
  { 1, 4, 7, 13, 15, },
  { 0, 1, 4, 7, 13, 15, },
  { 2, 4, 7, 13, 15, },
  { 0, 2, 4, 7, 13, 15, },
  { 1, 2, 4, 7, 13, 15, },
  { 0, 1, 2, 4, 7, 13, 15, },
  { 3, 4, 7, 13, 15, },
  { 0, 3, 4, 7, 13, 15, },
  { 1, 3, 4, 7, 13, 15, },
  { 0, 1, 3, 4, 7, 13, 15, },
  { 2, 3, 4, 7, 13, 15, },
  { 0, 2, 3, 4, 7, 13, 15, },
  { 1, 2, 3, 4, 7, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 13, 15, },
  { 5, 7, 13, 15, },
  { 0, 5, 7, 13, 15, },
  { 1, 5, 7, 13, 15, },
  { 0, 1, 5, 7, 13, 15, },
  { 2, 5, 7, 13, 15, },
  { 0, 2, 5, 7, 13, 15, },
  { 1, 2, 5, 7, 13, 15, },
  { 0, 1, 2, 5, 7, 13, 15, },
  { 3, 5, 7, 13, 15, },
  { 0, 3, 5, 7, 13, 15, },
  { 1, 3, 5, 7, 13, 15, },
  { 0, 1, 3, 5, 7, 13, 15, },
  { 2, 3, 5, 7, 13, 15, },
  { 0, 2, 3, 5, 7, 13, 15, },
  { 1, 2, 3, 5, 7, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 13, 15, },
  { 4, 5, 7, 13, 15, },
  { 0, 4, 5, 7, 13, 15, },
  { 1, 4, 5, 7, 13, 15, },
  { 0, 1, 4, 5, 7, 13, 15, },
  { 2, 4, 5, 7, 13, 15, },
  { 0, 2, 4, 5, 7, 13, 15, },
  { 1, 2, 4, 5, 7, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 13, 15, },
  { 3, 4, 5, 7, 13, 15, },
  { 0, 3, 4, 5, 7, 13, 15, },
  { 1, 3, 4, 5, 7, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 13, 15, },
  { 2, 3, 4, 5, 7, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 13, 15, },
  { 6, 7, 13, 15, },
  { 0, 6, 7, 13, 15, },
  { 1, 6, 7, 13, 15, },
  { 0, 1, 6, 7, 13, 15, },
  { 2, 6, 7, 13, 15, },
  { 0, 2, 6, 7, 13, 15, },
  { 1, 2, 6, 7, 13, 15, },
  { 0, 1, 2, 6, 7, 13, 15, },
  { 3, 6, 7, 13, 15, },
  { 0, 3, 6, 7, 13, 15, },
  { 1, 3, 6, 7, 13, 15, },
  { 0, 1, 3, 6, 7, 13, 15, },
  { 2, 3, 6, 7, 13, 15, },
  { 0, 2, 3, 6, 7, 13, 15, },
  { 1, 2, 3, 6, 7, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 13, 15, },
  { 4, 6, 7, 13, 15, },
  { 0, 4, 6, 7, 13, 15, },
  { 1, 4, 6, 7, 13, 15, },
  { 0, 1, 4, 6, 7, 13, 15, },
  { 2, 4, 6, 7, 13, 15, },
  { 0, 2, 4, 6, 7, 13, 15, },
  { 1, 2, 4, 6, 7, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 13, 15, },
  { 3, 4, 6, 7, 13, 15, },
  { 0, 3, 4, 6, 7, 13, 15, },
  { 1, 3, 4, 6, 7, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 13, 15, },
  { 2, 3, 4, 6, 7, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 13, 15, },
  { 5, 6, 7, 13, 15, },
  { 0, 5, 6, 7, 13, 15, },
  { 1, 5, 6, 7, 13, 15, },
  { 0, 1, 5, 6, 7, 13, 15, },
  { 2, 5, 6, 7, 13, 15, },
  { 0, 2, 5, 6, 7, 13, 15, },
  { 1, 2, 5, 6, 7, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 13, 15, },
  { 3, 5, 6, 7, 13, 15, },
  { 0, 3, 5, 6, 7, 13, 15, },
  { 1, 3, 5, 6, 7, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 13, 15, },
  { 2, 3, 5, 6, 7, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 13, 15, },
  { 4, 5, 6, 7, 13, 15, },
  { 0, 4, 5, 6, 7, 13, 15, },
  { 1, 4, 5, 6, 7, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 13, 15, },
  { 2, 4, 5, 6, 7, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 13, 15, },
  { 3, 4, 5, 6, 7, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 13, 15, },
  { 8, 13, 15, },
  { 0, 8, 13, 15, },
  { 1, 8, 13, 15, },
  { 0, 1, 8, 13, 15, },
  { 2, 8, 13, 15, },
  { 0, 2, 8, 13, 15, },
  { 1, 2, 8, 13, 15, },
  { 0, 1, 2, 8, 13, 15, },
  { 3, 8, 13, 15, },
  { 0, 3, 8, 13, 15, },
  { 1, 3, 8, 13, 15, },
  { 0, 1, 3, 8, 13, 15, },
  { 2, 3, 8, 13, 15, },
  { 0, 2, 3, 8, 13, 15, },
  { 1, 2, 3, 8, 13, 15, },
  { 0, 1, 2, 3, 8, 13, 15, },
  { 4, 8, 13, 15, },
  { 0, 4, 8, 13, 15, },
  { 1, 4, 8, 13, 15, },
  { 0, 1, 4, 8, 13, 15, },
  { 2, 4, 8, 13, 15, },
  { 0, 2, 4, 8, 13, 15, },
  { 1, 2, 4, 8, 13, 15, },
  { 0, 1, 2, 4, 8, 13, 15, },
  { 3, 4, 8, 13, 15, },
  { 0, 3, 4, 8, 13, 15, },
  { 1, 3, 4, 8, 13, 15, },
  { 0, 1, 3, 4, 8, 13, 15, },
  { 2, 3, 4, 8, 13, 15, },
  { 0, 2, 3, 4, 8, 13, 15, },
  { 1, 2, 3, 4, 8, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 13, 15, },
  { 5, 8, 13, 15, },
  { 0, 5, 8, 13, 15, },
  { 1, 5, 8, 13, 15, },
  { 0, 1, 5, 8, 13, 15, },
  { 2, 5, 8, 13, 15, },
  { 0, 2, 5, 8, 13, 15, },
  { 1, 2, 5, 8, 13, 15, },
  { 0, 1, 2, 5, 8, 13, 15, },
  { 3, 5, 8, 13, 15, },
  { 0, 3, 5, 8, 13, 15, },
  { 1, 3, 5, 8, 13, 15, },
  { 0, 1, 3, 5, 8, 13, 15, },
  { 2, 3, 5, 8, 13, 15, },
  { 0, 2, 3, 5, 8, 13, 15, },
  { 1, 2, 3, 5, 8, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 13, 15, },
  { 4, 5, 8, 13, 15, },
  { 0, 4, 5, 8, 13, 15, },
  { 1, 4, 5, 8, 13, 15, },
  { 0, 1, 4, 5, 8, 13, 15, },
  { 2, 4, 5, 8, 13, 15, },
  { 0, 2, 4, 5, 8, 13, 15, },
  { 1, 2, 4, 5, 8, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 13, 15, },
  { 3, 4, 5, 8, 13, 15, },
  { 0, 3, 4, 5, 8, 13, 15, },
  { 1, 3, 4, 5, 8, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 13, 15, },
  { 2, 3, 4, 5, 8, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 13, 15, },
  { 6, 8, 13, 15, },
  { 0, 6, 8, 13, 15, },
  { 1, 6, 8, 13, 15, },
  { 0, 1, 6, 8, 13, 15, },
  { 2, 6, 8, 13, 15, },
  { 0, 2, 6, 8, 13, 15, },
  { 1, 2, 6, 8, 13, 15, },
  { 0, 1, 2, 6, 8, 13, 15, },
  { 3, 6, 8, 13, 15, },
  { 0, 3, 6, 8, 13, 15, },
  { 1, 3, 6, 8, 13, 15, },
  { 0, 1, 3, 6, 8, 13, 15, },
  { 2, 3, 6, 8, 13, 15, },
  { 0, 2, 3, 6, 8, 13, 15, },
  { 1, 2, 3, 6, 8, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 13, 15, },
  { 4, 6, 8, 13, 15, },
  { 0, 4, 6, 8, 13, 15, },
  { 1, 4, 6, 8, 13, 15, },
  { 0, 1, 4, 6, 8, 13, 15, },
  { 2, 4, 6, 8, 13, 15, },
  { 0, 2, 4, 6, 8, 13, 15, },
  { 1, 2, 4, 6, 8, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 13, 15, },
  { 3, 4, 6, 8, 13, 15, },
  { 0, 3, 4, 6, 8, 13, 15, },
  { 1, 3, 4, 6, 8, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 13, 15, },
  { 2, 3, 4, 6, 8, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 13, 15, },
  { 5, 6, 8, 13, 15, },
  { 0, 5, 6, 8, 13, 15, },
  { 1, 5, 6, 8, 13, 15, },
  { 0, 1, 5, 6, 8, 13, 15, },
  { 2, 5, 6, 8, 13, 15, },
  { 0, 2, 5, 6, 8, 13, 15, },
  { 1, 2, 5, 6, 8, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 13, 15, },
  { 3, 5, 6, 8, 13, 15, },
  { 0, 3, 5, 6, 8, 13, 15, },
  { 1, 3, 5, 6, 8, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 13, 15, },
  { 2, 3, 5, 6, 8, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 13, 15, },
  { 4, 5, 6, 8, 13, 15, },
  { 0, 4, 5, 6, 8, 13, 15, },
  { 1, 4, 5, 6, 8, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 13, 15, },
  { 2, 4, 5, 6, 8, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 13, 15, },
  { 3, 4, 5, 6, 8, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 13, 15, },
  { 7, 8, 13, 15, },
  { 0, 7, 8, 13, 15, },
  { 1, 7, 8, 13, 15, },
  { 0, 1, 7, 8, 13, 15, },
  { 2, 7, 8, 13, 15, },
  { 0, 2, 7, 8, 13, 15, },
  { 1, 2, 7, 8, 13, 15, },
  { 0, 1, 2, 7, 8, 13, 15, },
  { 3, 7, 8, 13, 15, },
  { 0, 3, 7, 8, 13, 15, },
  { 1, 3, 7, 8, 13, 15, },
  { 0, 1, 3, 7, 8, 13, 15, },
  { 2, 3, 7, 8, 13, 15, },
  { 0, 2, 3, 7, 8, 13, 15, },
  { 1, 2, 3, 7, 8, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 13, 15, },
  { 4, 7, 8, 13, 15, },
  { 0, 4, 7, 8, 13, 15, },
  { 1, 4, 7, 8, 13, 15, },
  { 0, 1, 4, 7, 8, 13, 15, },
  { 2, 4, 7, 8, 13, 15, },
  { 0, 2, 4, 7, 8, 13, 15, },
  { 1, 2, 4, 7, 8, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 13, 15, },
  { 3, 4, 7, 8, 13, 15, },
  { 0, 3, 4, 7, 8, 13, 15, },
  { 1, 3, 4, 7, 8, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 13, 15, },
  { 2, 3, 4, 7, 8, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 13, 15, },
  { 5, 7, 8, 13, 15, },
  { 0, 5, 7, 8, 13, 15, },
  { 1, 5, 7, 8, 13, 15, },
  { 0, 1, 5, 7, 8, 13, 15, },
  { 2, 5, 7, 8, 13, 15, },
  { 0, 2, 5, 7, 8, 13, 15, },
  { 1, 2, 5, 7, 8, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 13, 15, },
  { 3, 5, 7, 8, 13, 15, },
  { 0, 3, 5, 7, 8, 13, 15, },
  { 1, 3, 5, 7, 8, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 13, 15, },
  { 2, 3, 5, 7, 8, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 13, 15, },
  { 4, 5, 7, 8, 13, 15, },
  { 0, 4, 5, 7, 8, 13, 15, },
  { 1, 4, 5, 7, 8, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 13, 15, },
  { 2, 4, 5, 7, 8, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 13, 15, },
  { 3, 4, 5, 7, 8, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 13, 15, },
  { 6, 7, 8, 13, 15, },
  { 0, 6, 7, 8, 13, 15, },
  { 1, 6, 7, 8, 13, 15, },
  { 0, 1, 6, 7, 8, 13, 15, },
  { 2, 6, 7, 8, 13, 15, },
  { 0, 2, 6, 7, 8, 13, 15, },
  { 1, 2, 6, 7, 8, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 13, 15, },
  { 3, 6, 7, 8, 13, 15, },
  { 0, 3, 6, 7, 8, 13, 15, },
  { 1, 3, 6, 7, 8, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 13, 15, },
  { 2, 3, 6, 7, 8, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 13, 15, },
  { 4, 6, 7, 8, 13, 15, },
  { 0, 4, 6, 7, 8, 13, 15, },
  { 1, 4, 6, 7, 8, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 13, 15, },
  { 2, 4, 6, 7, 8, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 13, 15, },
  { 3, 4, 6, 7, 8, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 13, 15, },
  { 5, 6, 7, 8, 13, 15, },
  { 0, 5, 6, 7, 8, 13, 15, },
  { 1, 5, 6, 7, 8, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 13, 15, },
  { 2, 5, 6, 7, 8, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 13, 15, },
  { 3, 5, 6, 7, 8, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 13, 15, },
  { 4, 5, 6, 7, 8, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 13, 15, },
  { 9, 13, 15, },
  { 0, 9, 13, 15, },
  { 1, 9, 13, 15, },
  { 0, 1, 9, 13, 15, },
  { 2, 9, 13, 15, },
  { 0, 2, 9, 13, 15, },
  { 1, 2, 9, 13, 15, },
  { 0, 1, 2, 9, 13, 15, },
  { 3, 9, 13, 15, },
  { 0, 3, 9, 13, 15, },
  { 1, 3, 9, 13, 15, },
  { 0, 1, 3, 9, 13, 15, },
  { 2, 3, 9, 13, 15, },
  { 0, 2, 3, 9, 13, 15, },
  { 1, 2, 3, 9, 13, 15, },
  { 0, 1, 2, 3, 9, 13, 15, },
  { 4, 9, 13, 15, },
  { 0, 4, 9, 13, 15, },
  { 1, 4, 9, 13, 15, },
  { 0, 1, 4, 9, 13, 15, },
  { 2, 4, 9, 13, 15, },
  { 0, 2, 4, 9, 13, 15, },
  { 1, 2, 4, 9, 13, 15, },
  { 0, 1, 2, 4, 9, 13, 15, },
  { 3, 4, 9, 13, 15, },
  { 0, 3, 4, 9, 13, 15, },
  { 1, 3, 4, 9, 13, 15, },
  { 0, 1, 3, 4, 9, 13, 15, },
  { 2, 3, 4, 9, 13, 15, },
  { 0, 2, 3, 4, 9, 13, 15, },
  { 1, 2, 3, 4, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 9, 13, 15, },
  { 5, 9, 13, 15, },
  { 0, 5, 9, 13, 15, },
  { 1, 5, 9, 13, 15, },
  { 0, 1, 5, 9, 13, 15, },
  { 2, 5, 9, 13, 15, },
  { 0, 2, 5, 9, 13, 15, },
  { 1, 2, 5, 9, 13, 15, },
  { 0, 1, 2, 5, 9, 13, 15, },
  { 3, 5, 9, 13, 15, },
  { 0, 3, 5, 9, 13, 15, },
  { 1, 3, 5, 9, 13, 15, },
  { 0, 1, 3, 5, 9, 13, 15, },
  { 2, 3, 5, 9, 13, 15, },
  { 0, 2, 3, 5, 9, 13, 15, },
  { 1, 2, 3, 5, 9, 13, 15, },
  { 0, 1, 2, 3, 5, 9, 13, 15, },
  { 4, 5, 9, 13, 15, },
  { 0, 4, 5, 9, 13, 15, },
  { 1, 4, 5, 9, 13, 15, },
  { 0, 1, 4, 5, 9, 13, 15, },
  { 2, 4, 5, 9, 13, 15, },
  { 0, 2, 4, 5, 9, 13, 15, },
  { 1, 2, 4, 5, 9, 13, 15, },
  { 0, 1, 2, 4, 5, 9, 13, 15, },
  { 3, 4, 5, 9, 13, 15, },
  { 0, 3, 4, 5, 9, 13, 15, },
  { 1, 3, 4, 5, 9, 13, 15, },
  { 0, 1, 3, 4, 5, 9, 13, 15, },
  { 2, 3, 4, 5, 9, 13, 15, },
  { 0, 2, 3, 4, 5, 9, 13, 15, },
  { 1, 2, 3, 4, 5, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 13, 15, },
  { 6, 9, 13, 15, },
  { 0, 6, 9, 13, 15, },
  { 1, 6, 9, 13, 15, },
  { 0, 1, 6, 9, 13, 15, },
  { 2, 6, 9, 13, 15, },
  { 0, 2, 6, 9, 13, 15, },
  { 1, 2, 6, 9, 13, 15, },
  { 0, 1, 2, 6, 9, 13, 15, },
  { 3, 6, 9, 13, 15, },
  { 0, 3, 6, 9, 13, 15, },
  { 1, 3, 6, 9, 13, 15, },
  { 0, 1, 3, 6, 9, 13, 15, },
  { 2, 3, 6, 9, 13, 15, },
  { 0, 2, 3, 6, 9, 13, 15, },
  { 1, 2, 3, 6, 9, 13, 15, },
  { 0, 1, 2, 3, 6, 9, 13, 15, },
  { 4, 6, 9, 13, 15, },
  { 0, 4, 6, 9, 13, 15, },
  { 1, 4, 6, 9, 13, 15, },
  { 0, 1, 4, 6, 9, 13, 15, },
  { 2, 4, 6, 9, 13, 15, },
  { 0, 2, 4, 6, 9, 13, 15, },
  { 1, 2, 4, 6, 9, 13, 15, },
  { 0, 1, 2, 4, 6, 9, 13, 15, },
  { 3, 4, 6, 9, 13, 15, },
  { 0, 3, 4, 6, 9, 13, 15, },
  { 1, 3, 4, 6, 9, 13, 15, },
  { 0, 1, 3, 4, 6, 9, 13, 15, },
  { 2, 3, 4, 6, 9, 13, 15, },
  { 0, 2, 3, 4, 6, 9, 13, 15, },
  { 1, 2, 3, 4, 6, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 13, 15, },
  { 5, 6, 9, 13, 15, },
  { 0, 5, 6, 9, 13, 15, },
  { 1, 5, 6, 9, 13, 15, },
  { 0, 1, 5, 6, 9, 13, 15, },
  { 2, 5, 6, 9, 13, 15, },
  { 0, 2, 5, 6, 9, 13, 15, },
  { 1, 2, 5, 6, 9, 13, 15, },
  { 0, 1, 2, 5, 6, 9, 13, 15, },
  { 3, 5, 6, 9, 13, 15, },
  { 0, 3, 5, 6, 9, 13, 15, },
  { 1, 3, 5, 6, 9, 13, 15, },
  { 0, 1, 3, 5, 6, 9, 13, 15, },
  { 2, 3, 5, 6, 9, 13, 15, },
  { 0, 2, 3, 5, 6, 9, 13, 15, },
  { 1, 2, 3, 5, 6, 9, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 13, 15, },
  { 4, 5, 6, 9, 13, 15, },
  { 0, 4, 5, 6, 9, 13, 15, },
  { 1, 4, 5, 6, 9, 13, 15, },
  { 0, 1, 4, 5, 6, 9, 13, 15, },
  { 2, 4, 5, 6, 9, 13, 15, },
  { 0, 2, 4, 5, 6, 9, 13, 15, },
  { 1, 2, 4, 5, 6, 9, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 13, 15, },
  { 3, 4, 5, 6, 9, 13, 15, },
  { 0, 3, 4, 5, 6, 9, 13, 15, },
  { 1, 3, 4, 5, 6, 9, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 13, 15, },
  { 2, 3, 4, 5, 6, 9, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 13, 15, },
  { 7, 9, 13, 15, },
  { 0, 7, 9, 13, 15, },
  { 1, 7, 9, 13, 15, },
  { 0, 1, 7, 9, 13, 15, },
  { 2, 7, 9, 13, 15, },
  { 0, 2, 7, 9, 13, 15, },
  { 1, 2, 7, 9, 13, 15, },
  { 0, 1, 2, 7, 9, 13, 15, },
  { 3, 7, 9, 13, 15, },
  { 0, 3, 7, 9, 13, 15, },
  { 1, 3, 7, 9, 13, 15, },
  { 0, 1, 3, 7, 9, 13, 15, },
  { 2, 3, 7, 9, 13, 15, },
  { 0, 2, 3, 7, 9, 13, 15, },
  { 1, 2, 3, 7, 9, 13, 15, },
  { 0, 1, 2, 3, 7, 9, 13, 15, },
  { 4, 7, 9, 13, 15, },
  { 0, 4, 7, 9, 13, 15, },
  { 1, 4, 7, 9, 13, 15, },
  { 0, 1, 4, 7, 9, 13, 15, },
  { 2, 4, 7, 9, 13, 15, },
  { 0, 2, 4, 7, 9, 13, 15, },
  { 1, 2, 4, 7, 9, 13, 15, },
  { 0, 1, 2, 4, 7, 9, 13, 15, },
  { 3, 4, 7, 9, 13, 15, },
  { 0, 3, 4, 7, 9, 13, 15, },
  { 1, 3, 4, 7, 9, 13, 15, },
  { 0, 1, 3, 4, 7, 9, 13, 15, },
  { 2, 3, 4, 7, 9, 13, 15, },
  { 0, 2, 3, 4, 7, 9, 13, 15, },
  { 1, 2, 3, 4, 7, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 13, 15, },
  { 5, 7, 9, 13, 15, },
  { 0, 5, 7, 9, 13, 15, },
  { 1, 5, 7, 9, 13, 15, },
  { 0, 1, 5, 7, 9, 13, 15, },
  { 2, 5, 7, 9, 13, 15, },
  { 0, 2, 5, 7, 9, 13, 15, },
  { 1, 2, 5, 7, 9, 13, 15, },
  { 0, 1, 2, 5, 7, 9, 13, 15, },
  { 3, 5, 7, 9, 13, 15, },
  { 0, 3, 5, 7, 9, 13, 15, },
  { 1, 3, 5, 7, 9, 13, 15, },
  { 0, 1, 3, 5, 7, 9, 13, 15, },
  { 2, 3, 5, 7, 9, 13, 15, },
  { 0, 2, 3, 5, 7, 9, 13, 15, },
  { 1, 2, 3, 5, 7, 9, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 13, 15, },
  { 4, 5, 7, 9, 13, 15, },
  { 0, 4, 5, 7, 9, 13, 15, },
  { 1, 4, 5, 7, 9, 13, 15, },
  { 0, 1, 4, 5, 7, 9, 13, 15, },
  { 2, 4, 5, 7, 9, 13, 15, },
  { 0, 2, 4, 5, 7, 9, 13, 15, },
  { 1, 2, 4, 5, 7, 9, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 13, 15, },
  { 3, 4, 5, 7, 9, 13, 15, },
  { 0, 3, 4, 5, 7, 9, 13, 15, },
  { 1, 3, 4, 5, 7, 9, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 13, 15, },
  { 2, 3, 4, 5, 7, 9, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 13, 15, },
  { 6, 7, 9, 13, 15, },
  { 0, 6, 7, 9, 13, 15, },
  { 1, 6, 7, 9, 13, 15, },
  { 0, 1, 6, 7, 9, 13, 15, },
  { 2, 6, 7, 9, 13, 15, },
  { 0, 2, 6, 7, 9, 13, 15, },
  { 1, 2, 6, 7, 9, 13, 15, },
  { 0, 1, 2, 6, 7, 9, 13, 15, },
  { 3, 6, 7, 9, 13, 15, },
  { 0, 3, 6, 7, 9, 13, 15, },
  { 1, 3, 6, 7, 9, 13, 15, },
  { 0, 1, 3, 6, 7, 9, 13, 15, },
  { 2, 3, 6, 7, 9, 13, 15, },
  { 0, 2, 3, 6, 7, 9, 13, 15, },
  { 1, 2, 3, 6, 7, 9, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 13, 15, },
  { 4, 6, 7, 9, 13, 15, },
  { 0, 4, 6, 7, 9, 13, 15, },
  { 1, 4, 6, 7, 9, 13, 15, },
  { 0, 1, 4, 6, 7, 9, 13, 15, },
  { 2, 4, 6, 7, 9, 13, 15, },
  { 0, 2, 4, 6, 7, 9, 13, 15, },
  { 1, 2, 4, 6, 7, 9, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 13, 15, },
  { 3, 4, 6, 7, 9, 13, 15, },
  { 0, 3, 4, 6, 7, 9, 13, 15, },
  { 1, 3, 4, 6, 7, 9, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 13, 15, },
  { 2, 3, 4, 6, 7, 9, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 13, 15, },
  { 5, 6, 7, 9, 13, 15, },
  { 0, 5, 6, 7, 9, 13, 15, },
  { 1, 5, 6, 7, 9, 13, 15, },
  { 0, 1, 5, 6, 7, 9, 13, 15, },
  { 2, 5, 6, 7, 9, 13, 15, },
  { 0, 2, 5, 6, 7, 9, 13, 15, },
  { 1, 2, 5, 6, 7, 9, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 13, 15, },
  { 3, 5, 6, 7, 9, 13, 15, },
  { 0, 3, 5, 6, 7, 9, 13, 15, },
  { 1, 3, 5, 6, 7, 9, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 13, 15, },
  { 2, 3, 5, 6, 7, 9, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 13, 15, },
  { 4, 5, 6, 7, 9, 13, 15, },
  { 0, 4, 5, 6, 7, 9, 13, 15, },
  { 1, 4, 5, 6, 7, 9, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 13, 15, },
  { 2, 4, 5, 6, 7, 9, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 13, 15, },
  { 3, 4, 5, 6, 7, 9, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 13, 15, },
  { 8, 9, 13, 15, },
  { 0, 8, 9, 13, 15, },
  { 1, 8, 9, 13, 15, },
  { 0, 1, 8, 9, 13, 15, },
  { 2, 8, 9, 13, 15, },
  { 0, 2, 8, 9, 13, 15, },
  { 1, 2, 8, 9, 13, 15, },
  { 0, 1, 2, 8, 9, 13, 15, },
  { 3, 8, 9, 13, 15, },
  { 0, 3, 8, 9, 13, 15, },
  { 1, 3, 8, 9, 13, 15, },
  { 0, 1, 3, 8, 9, 13, 15, },
  { 2, 3, 8, 9, 13, 15, },
  { 0, 2, 3, 8, 9, 13, 15, },
  { 1, 2, 3, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 8, 9, 13, 15, },
  { 4, 8, 9, 13, 15, },
  { 0, 4, 8, 9, 13, 15, },
  { 1, 4, 8, 9, 13, 15, },
  { 0, 1, 4, 8, 9, 13, 15, },
  { 2, 4, 8, 9, 13, 15, },
  { 0, 2, 4, 8, 9, 13, 15, },
  { 1, 2, 4, 8, 9, 13, 15, },
  { 0, 1, 2, 4, 8, 9, 13, 15, },
  { 3, 4, 8, 9, 13, 15, },
  { 0, 3, 4, 8, 9, 13, 15, },
  { 1, 3, 4, 8, 9, 13, 15, },
  { 0, 1, 3, 4, 8, 9, 13, 15, },
  { 2, 3, 4, 8, 9, 13, 15, },
  { 0, 2, 3, 4, 8, 9, 13, 15, },
  { 1, 2, 3, 4, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 13, 15, },
  { 5, 8, 9, 13, 15, },
  { 0, 5, 8, 9, 13, 15, },
  { 1, 5, 8, 9, 13, 15, },
  { 0, 1, 5, 8, 9, 13, 15, },
  { 2, 5, 8, 9, 13, 15, },
  { 0, 2, 5, 8, 9, 13, 15, },
  { 1, 2, 5, 8, 9, 13, 15, },
  { 0, 1, 2, 5, 8, 9, 13, 15, },
  { 3, 5, 8, 9, 13, 15, },
  { 0, 3, 5, 8, 9, 13, 15, },
  { 1, 3, 5, 8, 9, 13, 15, },
  { 0, 1, 3, 5, 8, 9, 13, 15, },
  { 2, 3, 5, 8, 9, 13, 15, },
  { 0, 2, 3, 5, 8, 9, 13, 15, },
  { 1, 2, 3, 5, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 13, 15, },
  { 4, 5, 8, 9, 13, 15, },
  { 0, 4, 5, 8, 9, 13, 15, },
  { 1, 4, 5, 8, 9, 13, 15, },
  { 0, 1, 4, 5, 8, 9, 13, 15, },
  { 2, 4, 5, 8, 9, 13, 15, },
  { 0, 2, 4, 5, 8, 9, 13, 15, },
  { 1, 2, 4, 5, 8, 9, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 13, 15, },
  { 3, 4, 5, 8, 9, 13, 15, },
  { 0, 3, 4, 5, 8, 9, 13, 15, },
  { 1, 3, 4, 5, 8, 9, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 13, 15, },
  { 2, 3, 4, 5, 8, 9, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 13, 15, },
  { 6, 8, 9, 13, 15, },
  { 0, 6, 8, 9, 13, 15, },
  { 1, 6, 8, 9, 13, 15, },
  { 0, 1, 6, 8, 9, 13, 15, },
  { 2, 6, 8, 9, 13, 15, },
  { 0, 2, 6, 8, 9, 13, 15, },
  { 1, 2, 6, 8, 9, 13, 15, },
  { 0, 1, 2, 6, 8, 9, 13, 15, },
  { 3, 6, 8, 9, 13, 15, },
  { 0, 3, 6, 8, 9, 13, 15, },
  { 1, 3, 6, 8, 9, 13, 15, },
  { 0, 1, 3, 6, 8, 9, 13, 15, },
  { 2, 3, 6, 8, 9, 13, 15, },
  { 0, 2, 3, 6, 8, 9, 13, 15, },
  { 1, 2, 3, 6, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 13, 15, },
  { 4, 6, 8, 9, 13, 15, },
  { 0, 4, 6, 8, 9, 13, 15, },
  { 1, 4, 6, 8, 9, 13, 15, },
  { 0, 1, 4, 6, 8, 9, 13, 15, },
  { 2, 4, 6, 8, 9, 13, 15, },
  { 0, 2, 4, 6, 8, 9, 13, 15, },
  { 1, 2, 4, 6, 8, 9, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 13, 15, },
  { 3, 4, 6, 8, 9, 13, 15, },
  { 0, 3, 4, 6, 8, 9, 13, 15, },
  { 1, 3, 4, 6, 8, 9, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 13, 15, },
  { 2, 3, 4, 6, 8, 9, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 13, 15, },
  { 5, 6, 8, 9, 13, 15, },
  { 0, 5, 6, 8, 9, 13, 15, },
  { 1, 5, 6, 8, 9, 13, 15, },
  { 0, 1, 5, 6, 8, 9, 13, 15, },
  { 2, 5, 6, 8, 9, 13, 15, },
  { 0, 2, 5, 6, 8, 9, 13, 15, },
  { 1, 2, 5, 6, 8, 9, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 13, 15, },
  { 3, 5, 6, 8, 9, 13, 15, },
  { 0, 3, 5, 6, 8, 9, 13, 15, },
  { 1, 3, 5, 6, 8, 9, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 13, 15, },
  { 2, 3, 5, 6, 8, 9, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 13, 15, },
  { 4, 5, 6, 8, 9, 13, 15, },
  { 0, 4, 5, 6, 8, 9, 13, 15, },
  { 1, 4, 5, 6, 8, 9, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 13, 15, },
  { 2, 4, 5, 6, 8, 9, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 13, 15, },
  { 3, 4, 5, 6, 8, 9, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 13, 15, },
  { 7, 8, 9, 13, 15, },
  { 0, 7, 8, 9, 13, 15, },
  { 1, 7, 8, 9, 13, 15, },
  { 0, 1, 7, 8, 9, 13, 15, },
  { 2, 7, 8, 9, 13, 15, },
  { 0, 2, 7, 8, 9, 13, 15, },
  { 1, 2, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 7, 8, 9, 13, 15, },
  { 3, 7, 8, 9, 13, 15, },
  { 0, 3, 7, 8, 9, 13, 15, },
  { 1, 3, 7, 8, 9, 13, 15, },
  { 0, 1, 3, 7, 8, 9, 13, 15, },
  { 2, 3, 7, 8, 9, 13, 15, },
  { 0, 2, 3, 7, 8, 9, 13, 15, },
  { 1, 2, 3, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 13, 15, },
  { 4, 7, 8, 9, 13, 15, },
  { 0, 4, 7, 8, 9, 13, 15, },
  { 1, 4, 7, 8, 9, 13, 15, },
  { 0, 1, 4, 7, 8, 9, 13, 15, },
  { 2, 4, 7, 8, 9, 13, 15, },
  { 0, 2, 4, 7, 8, 9, 13, 15, },
  { 1, 2, 4, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 13, 15, },
  { 3, 4, 7, 8, 9, 13, 15, },
  { 0, 3, 4, 7, 8, 9, 13, 15, },
  { 1, 3, 4, 7, 8, 9, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 13, 15, },
  { 2, 3, 4, 7, 8, 9, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 13, 15, },
  { 5, 7, 8, 9, 13, 15, },
  { 0, 5, 7, 8, 9, 13, 15, },
  { 1, 5, 7, 8, 9, 13, 15, },
  { 0, 1, 5, 7, 8, 9, 13, 15, },
  { 2, 5, 7, 8, 9, 13, 15, },
  { 0, 2, 5, 7, 8, 9, 13, 15, },
  { 1, 2, 5, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 13, 15, },
  { 3, 5, 7, 8, 9, 13, 15, },
  { 0, 3, 5, 7, 8, 9, 13, 15, },
  { 1, 3, 5, 7, 8, 9, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 13, 15, },
  { 2, 3, 5, 7, 8, 9, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 13, 15, },
  { 4, 5, 7, 8, 9, 13, 15, },
  { 0, 4, 5, 7, 8, 9, 13, 15, },
  { 1, 4, 5, 7, 8, 9, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 13, 15, },
  { 2, 4, 5, 7, 8, 9, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 13, 15, },
  { 3, 4, 5, 7, 8, 9, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 13, 15, },
  { 6, 7, 8, 9, 13, 15, },
  { 0, 6, 7, 8, 9, 13, 15, },
  { 1, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 6, 7, 8, 9, 13, 15, },
  { 2, 6, 7, 8, 9, 13, 15, },
  { 0, 2, 6, 7, 8, 9, 13, 15, },
  { 1, 2, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 13, 15, },
  { 3, 6, 7, 8, 9, 13, 15, },
  { 0, 3, 6, 7, 8, 9, 13, 15, },
  { 1, 3, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 13, 15, },
  { 2, 3, 6, 7, 8, 9, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 13, 15, },
  { 4, 6, 7, 8, 9, 13, 15, },
  { 0, 4, 6, 7, 8, 9, 13, 15, },
  { 1, 4, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 13, 15, },
  { 2, 4, 6, 7, 8, 9, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 13, 15, },
  { 3, 4, 6, 7, 8, 9, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 13, 15, },
  { 5, 6, 7, 8, 9, 13, 15, },
  { 0, 5, 6, 7, 8, 9, 13, 15, },
  { 1, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 13, 15, },
  { 2, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 13, 15, },
  { 3, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 13, 15, },
  { 4, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 13, 15, },
  { 10, 13, 15, },
  { 0, 10, 13, 15, },
  { 1, 10, 13, 15, },
  { 0, 1, 10, 13, 15, },
  { 2, 10, 13, 15, },
  { 0, 2, 10, 13, 15, },
  { 1, 2, 10, 13, 15, },
  { 0, 1, 2, 10, 13, 15, },
  { 3, 10, 13, 15, },
  { 0, 3, 10, 13, 15, },
  { 1, 3, 10, 13, 15, },
  { 0, 1, 3, 10, 13, 15, },
  { 2, 3, 10, 13, 15, },
  { 0, 2, 3, 10, 13, 15, },
  { 1, 2, 3, 10, 13, 15, },
  { 0, 1, 2, 3, 10, 13, 15, },
  { 4, 10, 13, 15, },
  { 0, 4, 10, 13, 15, },
  { 1, 4, 10, 13, 15, },
  { 0, 1, 4, 10, 13, 15, },
  { 2, 4, 10, 13, 15, },
  { 0, 2, 4, 10, 13, 15, },
  { 1, 2, 4, 10, 13, 15, },
  { 0, 1, 2, 4, 10, 13, 15, },
  { 3, 4, 10, 13, 15, },
  { 0, 3, 4, 10, 13, 15, },
  { 1, 3, 4, 10, 13, 15, },
  { 0, 1, 3, 4, 10, 13, 15, },
  { 2, 3, 4, 10, 13, 15, },
  { 0, 2, 3, 4, 10, 13, 15, },
  { 1, 2, 3, 4, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 10, 13, 15, },
  { 5, 10, 13, 15, },
  { 0, 5, 10, 13, 15, },
  { 1, 5, 10, 13, 15, },
  { 0, 1, 5, 10, 13, 15, },
  { 2, 5, 10, 13, 15, },
  { 0, 2, 5, 10, 13, 15, },
  { 1, 2, 5, 10, 13, 15, },
  { 0, 1, 2, 5, 10, 13, 15, },
  { 3, 5, 10, 13, 15, },
  { 0, 3, 5, 10, 13, 15, },
  { 1, 3, 5, 10, 13, 15, },
  { 0, 1, 3, 5, 10, 13, 15, },
  { 2, 3, 5, 10, 13, 15, },
  { 0, 2, 3, 5, 10, 13, 15, },
  { 1, 2, 3, 5, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 10, 13, 15, },
  { 4, 5, 10, 13, 15, },
  { 0, 4, 5, 10, 13, 15, },
  { 1, 4, 5, 10, 13, 15, },
  { 0, 1, 4, 5, 10, 13, 15, },
  { 2, 4, 5, 10, 13, 15, },
  { 0, 2, 4, 5, 10, 13, 15, },
  { 1, 2, 4, 5, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 10, 13, 15, },
  { 3, 4, 5, 10, 13, 15, },
  { 0, 3, 4, 5, 10, 13, 15, },
  { 1, 3, 4, 5, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 10, 13, 15, },
  { 2, 3, 4, 5, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 13, 15, },
  { 6, 10, 13, 15, },
  { 0, 6, 10, 13, 15, },
  { 1, 6, 10, 13, 15, },
  { 0, 1, 6, 10, 13, 15, },
  { 2, 6, 10, 13, 15, },
  { 0, 2, 6, 10, 13, 15, },
  { 1, 2, 6, 10, 13, 15, },
  { 0, 1, 2, 6, 10, 13, 15, },
  { 3, 6, 10, 13, 15, },
  { 0, 3, 6, 10, 13, 15, },
  { 1, 3, 6, 10, 13, 15, },
  { 0, 1, 3, 6, 10, 13, 15, },
  { 2, 3, 6, 10, 13, 15, },
  { 0, 2, 3, 6, 10, 13, 15, },
  { 1, 2, 3, 6, 10, 13, 15, },
  { 0, 1, 2, 3, 6, 10, 13, 15, },
  { 4, 6, 10, 13, 15, },
  { 0, 4, 6, 10, 13, 15, },
  { 1, 4, 6, 10, 13, 15, },
  { 0, 1, 4, 6, 10, 13, 15, },
  { 2, 4, 6, 10, 13, 15, },
  { 0, 2, 4, 6, 10, 13, 15, },
  { 1, 2, 4, 6, 10, 13, 15, },
  { 0, 1, 2, 4, 6, 10, 13, 15, },
  { 3, 4, 6, 10, 13, 15, },
  { 0, 3, 4, 6, 10, 13, 15, },
  { 1, 3, 4, 6, 10, 13, 15, },
  { 0, 1, 3, 4, 6, 10, 13, 15, },
  { 2, 3, 4, 6, 10, 13, 15, },
  { 0, 2, 3, 4, 6, 10, 13, 15, },
  { 1, 2, 3, 4, 6, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 13, 15, },
  { 5, 6, 10, 13, 15, },
  { 0, 5, 6, 10, 13, 15, },
  { 1, 5, 6, 10, 13, 15, },
  { 0, 1, 5, 6, 10, 13, 15, },
  { 2, 5, 6, 10, 13, 15, },
  { 0, 2, 5, 6, 10, 13, 15, },
  { 1, 2, 5, 6, 10, 13, 15, },
  { 0, 1, 2, 5, 6, 10, 13, 15, },
  { 3, 5, 6, 10, 13, 15, },
  { 0, 3, 5, 6, 10, 13, 15, },
  { 1, 3, 5, 6, 10, 13, 15, },
  { 0, 1, 3, 5, 6, 10, 13, 15, },
  { 2, 3, 5, 6, 10, 13, 15, },
  { 0, 2, 3, 5, 6, 10, 13, 15, },
  { 1, 2, 3, 5, 6, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 13, 15, },
  { 4, 5, 6, 10, 13, 15, },
  { 0, 4, 5, 6, 10, 13, 15, },
  { 1, 4, 5, 6, 10, 13, 15, },
  { 0, 1, 4, 5, 6, 10, 13, 15, },
  { 2, 4, 5, 6, 10, 13, 15, },
  { 0, 2, 4, 5, 6, 10, 13, 15, },
  { 1, 2, 4, 5, 6, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 13, 15, },
  { 3, 4, 5, 6, 10, 13, 15, },
  { 0, 3, 4, 5, 6, 10, 13, 15, },
  { 1, 3, 4, 5, 6, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 13, 15, },
  { 2, 3, 4, 5, 6, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 13, 15, },
  { 7, 10, 13, 15, },
  { 0, 7, 10, 13, 15, },
  { 1, 7, 10, 13, 15, },
  { 0, 1, 7, 10, 13, 15, },
  { 2, 7, 10, 13, 15, },
  { 0, 2, 7, 10, 13, 15, },
  { 1, 2, 7, 10, 13, 15, },
  { 0, 1, 2, 7, 10, 13, 15, },
  { 3, 7, 10, 13, 15, },
  { 0, 3, 7, 10, 13, 15, },
  { 1, 3, 7, 10, 13, 15, },
  { 0, 1, 3, 7, 10, 13, 15, },
  { 2, 3, 7, 10, 13, 15, },
  { 0, 2, 3, 7, 10, 13, 15, },
  { 1, 2, 3, 7, 10, 13, 15, },
  { 0, 1, 2, 3, 7, 10, 13, 15, },
  { 4, 7, 10, 13, 15, },
  { 0, 4, 7, 10, 13, 15, },
  { 1, 4, 7, 10, 13, 15, },
  { 0, 1, 4, 7, 10, 13, 15, },
  { 2, 4, 7, 10, 13, 15, },
  { 0, 2, 4, 7, 10, 13, 15, },
  { 1, 2, 4, 7, 10, 13, 15, },
  { 0, 1, 2, 4, 7, 10, 13, 15, },
  { 3, 4, 7, 10, 13, 15, },
  { 0, 3, 4, 7, 10, 13, 15, },
  { 1, 3, 4, 7, 10, 13, 15, },
  { 0, 1, 3, 4, 7, 10, 13, 15, },
  { 2, 3, 4, 7, 10, 13, 15, },
  { 0, 2, 3, 4, 7, 10, 13, 15, },
  { 1, 2, 3, 4, 7, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 13, 15, },
  { 5, 7, 10, 13, 15, },
  { 0, 5, 7, 10, 13, 15, },
  { 1, 5, 7, 10, 13, 15, },
  { 0, 1, 5, 7, 10, 13, 15, },
  { 2, 5, 7, 10, 13, 15, },
  { 0, 2, 5, 7, 10, 13, 15, },
  { 1, 2, 5, 7, 10, 13, 15, },
  { 0, 1, 2, 5, 7, 10, 13, 15, },
  { 3, 5, 7, 10, 13, 15, },
  { 0, 3, 5, 7, 10, 13, 15, },
  { 1, 3, 5, 7, 10, 13, 15, },
  { 0, 1, 3, 5, 7, 10, 13, 15, },
  { 2, 3, 5, 7, 10, 13, 15, },
  { 0, 2, 3, 5, 7, 10, 13, 15, },
  { 1, 2, 3, 5, 7, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 13, 15, },
  { 4, 5, 7, 10, 13, 15, },
  { 0, 4, 5, 7, 10, 13, 15, },
  { 1, 4, 5, 7, 10, 13, 15, },
  { 0, 1, 4, 5, 7, 10, 13, 15, },
  { 2, 4, 5, 7, 10, 13, 15, },
  { 0, 2, 4, 5, 7, 10, 13, 15, },
  { 1, 2, 4, 5, 7, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 13, 15, },
  { 3, 4, 5, 7, 10, 13, 15, },
  { 0, 3, 4, 5, 7, 10, 13, 15, },
  { 1, 3, 4, 5, 7, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 13, 15, },
  { 2, 3, 4, 5, 7, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 13, 15, },
  { 6, 7, 10, 13, 15, },
  { 0, 6, 7, 10, 13, 15, },
  { 1, 6, 7, 10, 13, 15, },
  { 0, 1, 6, 7, 10, 13, 15, },
  { 2, 6, 7, 10, 13, 15, },
  { 0, 2, 6, 7, 10, 13, 15, },
  { 1, 2, 6, 7, 10, 13, 15, },
  { 0, 1, 2, 6, 7, 10, 13, 15, },
  { 3, 6, 7, 10, 13, 15, },
  { 0, 3, 6, 7, 10, 13, 15, },
  { 1, 3, 6, 7, 10, 13, 15, },
  { 0, 1, 3, 6, 7, 10, 13, 15, },
  { 2, 3, 6, 7, 10, 13, 15, },
  { 0, 2, 3, 6, 7, 10, 13, 15, },
  { 1, 2, 3, 6, 7, 10, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 13, 15, },
  { 4, 6, 7, 10, 13, 15, },
  { 0, 4, 6, 7, 10, 13, 15, },
  { 1, 4, 6, 7, 10, 13, 15, },
  { 0, 1, 4, 6, 7, 10, 13, 15, },
  { 2, 4, 6, 7, 10, 13, 15, },
  { 0, 2, 4, 6, 7, 10, 13, 15, },
  { 1, 2, 4, 6, 7, 10, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 13, 15, },
  { 3, 4, 6, 7, 10, 13, 15, },
  { 0, 3, 4, 6, 7, 10, 13, 15, },
  { 1, 3, 4, 6, 7, 10, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 13, 15, },
  { 2, 3, 4, 6, 7, 10, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 13, 15, },
  { 5, 6, 7, 10, 13, 15, },
  { 0, 5, 6, 7, 10, 13, 15, },
  { 1, 5, 6, 7, 10, 13, 15, },
  { 0, 1, 5, 6, 7, 10, 13, 15, },
  { 2, 5, 6, 7, 10, 13, 15, },
  { 0, 2, 5, 6, 7, 10, 13, 15, },
  { 1, 2, 5, 6, 7, 10, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 13, 15, },
  { 3, 5, 6, 7, 10, 13, 15, },
  { 0, 3, 5, 6, 7, 10, 13, 15, },
  { 1, 3, 5, 6, 7, 10, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 13, 15, },
  { 2, 3, 5, 6, 7, 10, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 13, 15, },
  { 4, 5, 6, 7, 10, 13, 15, },
  { 0, 4, 5, 6, 7, 10, 13, 15, },
  { 1, 4, 5, 6, 7, 10, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 13, 15, },
  { 2, 4, 5, 6, 7, 10, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 13, 15, },
  { 3, 4, 5, 6, 7, 10, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 13, 15, },
  { 8, 10, 13, 15, },
  { 0, 8, 10, 13, 15, },
  { 1, 8, 10, 13, 15, },
  { 0, 1, 8, 10, 13, 15, },
  { 2, 8, 10, 13, 15, },
  { 0, 2, 8, 10, 13, 15, },
  { 1, 2, 8, 10, 13, 15, },
  { 0, 1, 2, 8, 10, 13, 15, },
  { 3, 8, 10, 13, 15, },
  { 0, 3, 8, 10, 13, 15, },
  { 1, 3, 8, 10, 13, 15, },
  { 0, 1, 3, 8, 10, 13, 15, },
  { 2, 3, 8, 10, 13, 15, },
  { 0, 2, 3, 8, 10, 13, 15, },
  { 1, 2, 3, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 8, 10, 13, 15, },
  { 4, 8, 10, 13, 15, },
  { 0, 4, 8, 10, 13, 15, },
  { 1, 4, 8, 10, 13, 15, },
  { 0, 1, 4, 8, 10, 13, 15, },
  { 2, 4, 8, 10, 13, 15, },
  { 0, 2, 4, 8, 10, 13, 15, },
  { 1, 2, 4, 8, 10, 13, 15, },
  { 0, 1, 2, 4, 8, 10, 13, 15, },
  { 3, 4, 8, 10, 13, 15, },
  { 0, 3, 4, 8, 10, 13, 15, },
  { 1, 3, 4, 8, 10, 13, 15, },
  { 0, 1, 3, 4, 8, 10, 13, 15, },
  { 2, 3, 4, 8, 10, 13, 15, },
  { 0, 2, 3, 4, 8, 10, 13, 15, },
  { 1, 2, 3, 4, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 13, 15, },
  { 5, 8, 10, 13, 15, },
  { 0, 5, 8, 10, 13, 15, },
  { 1, 5, 8, 10, 13, 15, },
  { 0, 1, 5, 8, 10, 13, 15, },
  { 2, 5, 8, 10, 13, 15, },
  { 0, 2, 5, 8, 10, 13, 15, },
  { 1, 2, 5, 8, 10, 13, 15, },
  { 0, 1, 2, 5, 8, 10, 13, 15, },
  { 3, 5, 8, 10, 13, 15, },
  { 0, 3, 5, 8, 10, 13, 15, },
  { 1, 3, 5, 8, 10, 13, 15, },
  { 0, 1, 3, 5, 8, 10, 13, 15, },
  { 2, 3, 5, 8, 10, 13, 15, },
  { 0, 2, 3, 5, 8, 10, 13, 15, },
  { 1, 2, 3, 5, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 13, 15, },
  { 4, 5, 8, 10, 13, 15, },
  { 0, 4, 5, 8, 10, 13, 15, },
  { 1, 4, 5, 8, 10, 13, 15, },
  { 0, 1, 4, 5, 8, 10, 13, 15, },
  { 2, 4, 5, 8, 10, 13, 15, },
  { 0, 2, 4, 5, 8, 10, 13, 15, },
  { 1, 2, 4, 5, 8, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 13, 15, },
  { 3, 4, 5, 8, 10, 13, 15, },
  { 0, 3, 4, 5, 8, 10, 13, 15, },
  { 1, 3, 4, 5, 8, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 13, 15, },
  { 2, 3, 4, 5, 8, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 13, 15, },
  { 6, 8, 10, 13, 15, },
  { 0, 6, 8, 10, 13, 15, },
  { 1, 6, 8, 10, 13, 15, },
  { 0, 1, 6, 8, 10, 13, 15, },
  { 2, 6, 8, 10, 13, 15, },
  { 0, 2, 6, 8, 10, 13, 15, },
  { 1, 2, 6, 8, 10, 13, 15, },
  { 0, 1, 2, 6, 8, 10, 13, 15, },
  { 3, 6, 8, 10, 13, 15, },
  { 0, 3, 6, 8, 10, 13, 15, },
  { 1, 3, 6, 8, 10, 13, 15, },
  { 0, 1, 3, 6, 8, 10, 13, 15, },
  { 2, 3, 6, 8, 10, 13, 15, },
  { 0, 2, 3, 6, 8, 10, 13, 15, },
  { 1, 2, 3, 6, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 13, 15, },
  { 4, 6, 8, 10, 13, 15, },
  { 0, 4, 6, 8, 10, 13, 15, },
  { 1, 4, 6, 8, 10, 13, 15, },
  { 0, 1, 4, 6, 8, 10, 13, 15, },
  { 2, 4, 6, 8, 10, 13, 15, },
  { 0, 2, 4, 6, 8, 10, 13, 15, },
  { 1, 2, 4, 6, 8, 10, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 13, 15, },
  { 3, 4, 6, 8, 10, 13, 15, },
  { 0, 3, 4, 6, 8, 10, 13, 15, },
  { 1, 3, 4, 6, 8, 10, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 13, 15, },
  { 2, 3, 4, 6, 8, 10, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 13, 15, },
  { 5, 6, 8, 10, 13, 15, },
  { 0, 5, 6, 8, 10, 13, 15, },
  { 1, 5, 6, 8, 10, 13, 15, },
  { 0, 1, 5, 6, 8, 10, 13, 15, },
  { 2, 5, 6, 8, 10, 13, 15, },
  { 0, 2, 5, 6, 8, 10, 13, 15, },
  { 1, 2, 5, 6, 8, 10, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 13, 15, },
  { 3, 5, 6, 8, 10, 13, 15, },
  { 0, 3, 5, 6, 8, 10, 13, 15, },
  { 1, 3, 5, 6, 8, 10, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 13, 15, },
  { 2, 3, 5, 6, 8, 10, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 13, 15, },
  { 4, 5, 6, 8, 10, 13, 15, },
  { 0, 4, 5, 6, 8, 10, 13, 15, },
  { 1, 4, 5, 6, 8, 10, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 13, 15, },
  { 2, 4, 5, 6, 8, 10, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 13, 15, },
  { 3, 4, 5, 6, 8, 10, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 13, 15, },
  { 7, 8, 10, 13, 15, },
  { 0, 7, 8, 10, 13, 15, },
  { 1, 7, 8, 10, 13, 15, },
  { 0, 1, 7, 8, 10, 13, 15, },
  { 2, 7, 8, 10, 13, 15, },
  { 0, 2, 7, 8, 10, 13, 15, },
  { 1, 2, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 7, 8, 10, 13, 15, },
  { 3, 7, 8, 10, 13, 15, },
  { 0, 3, 7, 8, 10, 13, 15, },
  { 1, 3, 7, 8, 10, 13, 15, },
  { 0, 1, 3, 7, 8, 10, 13, 15, },
  { 2, 3, 7, 8, 10, 13, 15, },
  { 0, 2, 3, 7, 8, 10, 13, 15, },
  { 1, 2, 3, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 13, 15, },
  { 4, 7, 8, 10, 13, 15, },
  { 0, 4, 7, 8, 10, 13, 15, },
  { 1, 4, 7, 8, 10, 13, 15, },
  { 0, 1, 4, 7, 8, 10, 13, 15, },
  { 2, 4, 7, 8, 10, 13, 15, },
  { 0, 2, 4, 7, 8, 10, 13, 15, },
  { 1, 2, 4, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 13, 15, },
  { 3, 4, 7, 8, 10, 13, 15, },
  { 0, 3, 4, 7, 8, 10, 13, 15, },
  { 1, 3, 4, 7, 8, 10, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 13, 15, },
  { 2, 3, 4, 7, 8, 10, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 13, 15, },
  { 5, 7, 8, 10, 13, 15, },
  { 0, 5, 7, 8, 10, 13, 15, },
  { 1, 5, 7, 8, 10, 13, 15, },
  { 0, 1, 5, 7, 8, 10, 13, 15, },
  { 2, 5, 7, 8, 10, 13, 15, },
  { 0, 2, 5, 7, 8, 10, 13, 15, },
  { 1, 2, 5, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 13, 15, },
  { 3, 5, 7, 8, 10, 13, 15, },
  { 0, 3, 5, 7, 8, 10, 13, 15, },
  { 1, 3, 5, 7, 8, 10, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 13, 15, },
  { 2, 3, 5, 7, 8, 10, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 13, 15, },
  { 4, 5, 7, 8, 10, 13, 15, },
  { 0, 4, 5, 7, 8, 10, 13, 15, },
  { 1, 4, 5, 7, 8, 10, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 13, 15, },
  { 2, 4, 5, 7, 8, 10, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 13, 15, },
  { 3, 4, 5, 7, 8, 10, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 13, 15, },
  { 6, 7, 8, 10, 13, 15, },
  { 0, 6, 7, 8, 10, 13, 15, },
  { 1, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 6, 7, 8, 10, 13, 15, },
  { 2, 6, 7, 8, 10, 13, 15, },
  { 0, 2, 6, 7, 8, 10, 13, 15, },
  { 1, 2, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 13, 15, },
  { 3, 6, 7, 8, 10, 13, 15, },
  { 0, 3, 6, 7, 8, 10, 13, 15, },
  { 1, 3, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 13, 15, },
  { 2, 3, 6, 7, 8, 10, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 13, 15, },
  { 4, 6, 7, 8, 10, 13, 15, },
  { 0, 4, 6, 7, 8, 10, 13, 15, },
  { 1, 4, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 13, 15, },
  { 2, 4, 6, 7, 8, 10, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 13, 15, },
  { 3, 4, 6, 7, 8, 10, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 13, 15, },
  { 5, 6, 7, 8, 10, 13, 15, },
  { 0, 5, 6, 7, 8, 10, 13, 15, },
  { 1, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 13, 15, },
  { 2, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 13, 15, },
  { 3, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 13, 15, },
  { 4, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, 15, },
  { 9, 10, 13, 15, },
  { 0, 9, 10, 13, 15, },
  { 1, 9, 10, 13, 15, },
  { 0, 1, 9, 10, 13, 15, },
  { 2, 9, 10, 13, 15, },
  { 0, 2, 9, 10, 13, 15, },
  { 1, 2, 9, 10, 13, 15, },
  { 0, 1, 2, 9, 10, 13, 15, },
  { 3, 9, 10, 13, 15, },
  { 0, 3, 9, 10, 13, 15, },
  { 1, 3, 9, 10, 13, 15, },
  { 0, 1, 3, 9, 10, 13, 15, },
  { 2, 3, 9, 10, 13, 15, },
  { 0, 2, 3, 9, 10, 13, 15, },
  { 1, 2, 3, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 9, 10, 13, 15, },
  { 4, 9, 10, 13, 15, },
  { 0, 4, 9, 10, 13, 15, },
  { 1, 4, 9, 10, 13, 15, },
  { 0, 1, 4, 9, 10, 13, 15, },
  { 2, 4, 9, 10, 13, 15, },
  { 0, 2, 4, 9, 10, 13, 15, },
  { 1, 2, 4, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 9, 10, 13, 15, },
  { 3, 4, 9, 10, 13, 15, },
  { 0, 3, 4, 9, 10, 13, 15, },
  { 1, 3, 4, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 9, 10, 13, 15, },
  { 2, 3, 4, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 13, 15, },
  { 5, 9, 10, 13, 15, },
  { 0, 5, 9, 10, 13, 15, },
  { 1, 5, 9, 10, 13, 15, },
  { 0, 1, 5, 9, 10, 13, 15, },
  { 2, 5, 9, 10, 13, 15, },
  { 0, 2, 5, 9, 10, 13, 15, },
  { 1, 2, 5, 9, 10, 13, 15, },
  { 0, 1, 2, 5, 9, 10, 13, 15, },
  { 3, 5, 9, 10, 13, 15, },
  { 0, 3, 5, 9, 10, 13, 15, },
  { 1, 3, 5, 9, 10, 13, 15, },
  { 0, 1, 3, 5, 9, 10, 13, 15, },
  { 2, 3, 5, 9, 10, 13, 15, },
  { 0, 2, 3, 5, 9, 10, 13, 15, },
  { 1, 2, 3, 5, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 13, 15, },
  { 4, 5, 9, 10, 13, 15, },
  { 0, 4, 5, 9, 10, 13, 15, },
  { 1, 4, 5, 9, 10, 13, 15, },
  { 0, 1, 4, 5, 9, 10, 13, 15, },
  { 2, 4, 5, 9, 10, 13, 15, },
  { 0, 2, 4, 5, 9, 10, 13, 15, },
  { 1, 2, 4, 5, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 13, 15, },
  { 3, 4, 5, 9, 10, 13, 15, },
  { 0, 3, 4, 5, 9, 10, 13, 15, },
  { 1, 3, 4, 5, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 13, 15, },
  { 2, 3, 4, 5, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 13, 15, },
  { 6, 9, 10, 13, 15, },
  { 0, 6, 9, 10, 13, 15, },
  { 1, 6, 9, 10, 13, 15, },
  { 0, 1, 6, 9, 10, 13, 15, },
  { 2, 6, 9, 10, 13, 15, },
  { 0, 2, 6, 9, 10, 13, 15, },
  { 1, 2, 6, 9, 10, 13, 15, },
  { 0, 1, 2, 6, 9, 10, 13, 15, },
  { 3, 6, 9, 10, 13, 15, },
  { 0, 3, 6, 9, 10, 13, 15, },
  { 1, 3, 6, 9, 10, 13, 15, },
  { 0, 1, 3, 6, 9, 10, 13, 15, },
  { 2, 3, 6, 9, 10, 13, 15, },
  { 0, 2, 3, 6, 9, 10, 13, 15, },
  { 1, 2, 3, 6, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 13, 15, },
  { 4, 6, 9, 10, 13, 15, },
  { 0, 4, 6, 9, 10, 13, 15, },
  { 1, 4, 6, 9, 10, 13, 15, },
  { 0, 1, 4, 6, 9, 10, 13, 15, },
  { 2, 4, 6, 9, 10, 13, 15, },
  { 0, 2, 4, 6, 9, 10, 13, 15, },
  { 1, 2, 4, 6, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 13, 15, },
  { 3, 4, 6, 9, 10, 13, 15, },
  { 0, 3, 4, 6, 9, 10, 13, 15, },
  { 1, 3, 4, 6, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 13, 15, },
  { 2, 3, 4, 6, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 13, 15, },
  { 5, 6, 9, 10, 13, 15, },
  { 0, 5, 6, 9, 10, 13, 15, },
  { 1, 5, 6, 9, 10, 13, 15, },
  { 0, 1, 5, 6, 9, 10, 13, 15, },
  { 2, 5, 6, 9, 10, 13, 15, },
  { 0, 2, 5, 6, 9, 10, 13, 15, },
  { 1, 2, 5, 6, 9, 10, 13, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 13, 15, },
  { 3, 5, 6, 9, 10, 13, 15, },
  { 0, 3, 5, 6, 9, 10, 13, 15, },
  { 1, 3, 5, 6, 9, 10, 13, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 13, 15, },
  { 2, 3, 5, 6, 9, 10, 13, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 13, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 13, 15, },
  { 4, 5, 6, 9, 10, 13, 15, },
  { 0, 4, 5, 6, 9, 10, 13, 15, },
  { 1, 4, 5, 6, 9, 10, 13, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 13, 15, },
  { 2, 4, 5, 6, 9, 10, 13, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 13, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 13, 15, },
  { 3, 4, 5, 6, 9, 10, 13, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 13, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 13, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 13, 15, },
  { 7, 9, 10, 13, 15, },
  { 0, 7, 9, 10, 13, 15, },
  { 1, 7, 9, 10, 13, 15, },
  { 0, 1, 7, 9, 10, 13, 15, },
  { 2, 7, 9, 10, 13, 15, },
  { 0, 2, 7, 9, 10, 13, 15, },
  { 1, 2, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 7, 9, 10, 13, 15, },
  { 3, 7, 9, 10, 13, 15, },
  { 0, 3, 7, 9, 10, 13, 15, },
  { 1, 3, 7, 9, 10, 13, 15, },
  { 0, 1, 3, 7, 9, 10, 13, 15, },
  { 2, 3, 7, 9, 10, 13, 15, },
  { 0, 2, 3, 7, 9, 10, 13, 15, },
  { 1, 2, 3, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 13, 15, },
  { 4, 7, 9, 10, 13, 15, },
  { 0, 4, 7, 9, 10, 13, 15, },
  { 1, 4, 7, 9, 10, 13, 15, },
  { 0, 1, 4, 7, 9, 10, 13, 15, },
  { 2, 4, 7, 9, 10, 13, 15, },
  { 0, 2, 4, 7, 9, 10, 13, 15, },
  { 1, 2, 4, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 13, 15, },
  { 3, 4, 7, 9, 10, 13, 15, },
  { 0, 3, 4, 7, 9, 10, 13, 15, },
  { 1, 3, 4, 7, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 13, 15, },
  { 2, 3, 4, 7, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 13, 15, },
  { 5, 7, 9, 10, 13, 15, },
  { 0, 5, 7, 9, 10, 13, 15, },
  { 1, 5, 7, 9, 10, 13, 15, },
  { 0, 1, 5, 7, 9, 10, 13, 15, },
  { 2, 5, 7, 9, 10, 13, 15, },
  { 0, 2, 5, 7, 9, 10, 13, 15, },
  { 1, 2, 5, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 13, 15, },
  { 3, 5, 7, 9, 10, 13, 15, },
  { 0, 3, 5, 7, 9, 10, 13, 15, },
  { 1, 3, 5, 7, 9, 10, 13, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 13, 15, },
  { 2, 3, 5, 7, 9, 10, 13, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 13, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 13, 15, },
  { 4, 5, 7, 9, 10, 13, 15, },
  { 0, 4, 5, 7, 9, 10, 13, 15, },
  { 1, 4, 5, 7, 9, 10, 13, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 13, 15, },
  { 2, 4, 5, 7, 9, 10, 13, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 13, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 13, 15, },
  { 3, 4, 5, 7, 9, 10, 13, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 13, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 13, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 13, 15, },
  { 6, 7, 9, 10, 13, 15, },
  { 0, 6, 7, 9, 10, 13, 15, },
  { 1, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 6, 7, 9, 10, 13, 15, },
  { 2, 6, 7, 9, 10, 13, 15, },
  { 0, 2, 6, 7, 9, 10, 13, 15, },
  { 1, 2, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 13, 15, },
  { 3, 6, 7, 9, 10, 13, 15, },
  { 0, 3, 6, 7, 9, 10, 13, 15, },
  { 1, 3, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 13, 15, },
  { 2, 3, 6, 7, 9, 10, 13, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 13, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 13, 15, },
  { 4, 6, 7, 9, 10, 13, 15, },
  { 0, 4, 6, 7, 9, 10, 13, 15, },
  { 1, 4, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 13, 15, },
  { 2, 4, 6, 7, 9, 10, 13, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 13, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 13, 15, },
  { 3, 4, 6, 7, 9, 10, 13, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 13, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 13, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 13, 15, },
  { 5, 6, 7, 9, 10, 13, 15, },
  { 0, 5, 6, 7, 9, 10, 13, 15, },
  { 1, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 13, 15, },
  { 2, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 13, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 13, 15, },
  { 3, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 13, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 13, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 13, 15, },
  { 4, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 13, 15, },
  { 8, 9, 10, 13, 15, },
  { 0, 8, 9, 10, 13, 15, },
  { 1, 8, 9, 10, 13, 15, },
  { 0, 1, 8, 9, 10, 13, 15, },
  { 2, 8, 9, 10, 13, 15, },
  { 0, 2, 8, 9, 10, 13, 15, },
  { 1, 2, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 8, 9, 10, 13, 15, },
  { 3, 8, 9, 10, 13, 15, },
  { 0, 3, 8, 9, 10, 13, 15, },
  { 1, 3, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 8, 9, 10, 13, 15, },
  { 2, 3, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 13, 15, },
  { 4, 8, 9, 10, 13, 15, },
  { 0, 4, 8, 9, 10, 13, 15, },
  { 1, 4, 8, 9, 10, 13, 15, },
  { 0, 1, 4, 8, 9, 10, 13, 15, },
  { 2, 4, 8, 9, 10, 13, 15, },
  { 0, 2, 4, 8, 9, 10, 13, 15, },
  { 1, 2, 4, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 13, 15, },
  { 3, 4, 8, 9, 10, 13, 15, },
  { 0, 3, 4, 8, 9, 10, 13, 15, },
  { 1, 3, 4, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 13, 15, },
  { 2, 3, 4, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 13, 15, },
  { 5, 8, 9, 10, 13, 15, },
  { 0, 5, 8, 9, 10, 13, 15, },
  { 1, 5, 8, 9, 10, 13, 15, },
  { 0, 1, 5, 8, 9, 10, 13, 15, },
  { 2, 5, 8, 9, 10, 13, 15, },
  { 0, 2, 5, 8, 9, 10, 13, 15, },
  { 1, 2, 5, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 13, 15, },
  { 3, 5, 8, 9, 10, 13, 15, },
  { 0, 3, 5, 8, 9, 10, 13, 15, },
  { 1, 3, 5, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 13, 15, },
  { 2, 3, 5, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 13, 15, },
  { 4, 5, 8, 9, 10, 13, 15, },
  { 0, 4, 5, 8, 9, 10, 13, 15, },
  { 1, 4, 5, 8, 9, 10, 13, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 13, 15, },
  { 2, 4, 5, 8, 9, 10, 13, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 13, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 13, 15, },
  { 3, 4, 5, 8, 9, 10, 13, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 13, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 13, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 13, 15, },
  { 6, 8, 9, 10, 13, 15, },
  { 0, 6, 8, 9, 10, 13, 15, },
  { 1, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 6, 8, 9, 10, 13, 15, },
  { 2, 6, 8, 9, 10, 13, 15, },
  { 0, 2, 6, 8, 9, 10, 13, 15, },
  { 1, 2, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 13, 15, },
  { 3, 6, 8, 9, 10, 13, 15, },
  { 0, 3, 6, 8, 9, 10, 13, 15, },
  { 1, 3, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 13, 15, },
  { 2, 3, 6, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 13, 15, },
  { 4, 6, 8, 9, 10, 13, 15, },
  { 0, 4, 6, 8, 9, 10, 13, 15, },
  { 1, 4, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 13, 15, },
  { 2, 4, 6, 8, 9, 10, 13, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 13, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 13, 15, },
  { 3, 4, 6, 8, 9, 10, 13, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 13, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 13, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 13, 15, },
  { 5, 6, 8, 9, 10, 13, 15, },
  { 0, 5, 6, 8, 9, 10, 13, 15, },
  { 1, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 13, 15, },
  { 2, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 13, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 13, 15, },
  { 3, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 13, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 13, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 13, 15, },
  { 4, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 13, 15, },
  { 7, 8, 9, 10, 13, 15, },
  { 0, 7, 8, 9, 10, 13, 15, },
  { 1, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 7, 8, 9, 10, 13, 15, },
  { 2, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 13, 15, },
  { 3, 7, 8, 9, 10, 13, 15, },
  { 0, 3, 7, 8, 9, 10, 13, 15, },
  { 1, 3, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 13, 15, },
  { 2, 3, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 13, 15, },
  { 4, 7, 8, 9, 10, 13, 15, },
  { 0, 4, 7, 8, 9, 10, 13, 15, },
  { 1, 4, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 13, 15, },
  { 2, 4, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 13, 15, },
  { 3, 4, 7, 8, 9, 10, 13, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 13, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 13, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 13, 15, },
  { 5, 7, 8, 9, 10, 13, 15, },
  { 0, 5, 7, 8, 9, 10, 13, 15, },
  { 1, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 13, 15, },
  { 2, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 13, 15, },
  { 3, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 13, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 13, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 13, 15, },
  { 4, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 13, 15, },
  { 6, 7, 8, 9, 10, 13, 15, },
  { 0, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 13, 15, },
  { 2, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 13, 15, },
  { 3, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 13, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 13, 15, },
  { 4, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 13, 15, },
  { 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 15, },
  { 11, 13, 15, },
  { 0, 11, 13, 15, },
  { 1, 11, 13, 15, },
  { 0, 1, 11, 13, 15, },
  { 2, 11, 13, 15, },
  { 0, 2, 11, 13, 15, },
  { 1, 2, 11, 13, 15, },
  { 0, 1, 2, 11, 13, 15, },
  { 3, 11, 13, 15, },
  { 0, 3, 11, 13, 15, },
  { 1, 3, 11, 13, 15, },
  { 0, 1, 3, 11, 13, 15, },
  { 2, 3, 11, 13, 15, },
  { 0, 2, 3, 11, 13, 15, },
  { 1, 2, 3, 11, 13, 15, },
  { 0, 1, 2, 3, 11, 13, 15, },
  { 4, 11, 13, 15, },
  { 0, 4, 11, 13, 15, },
  { 1, 4, 11, 13, 15, },
  { 0, 1, 4, 11, 13, 15, },
  { 2, 4, 11, 13, 15, },
  { 0, 2, 4, 11, 13, 15, },
  { 1, 2, 4, 11, 13, 15, },
  { 0, 1, 2, 4, 11, 13, 15, },
  { 3, 4, 11, 13, 15, },
  { 0, 3, 4, 11, 13, 15, },
  { 1, 3, 4, 11, 13, 15, },
  { 0, 1, 3, 4, 11, 13, 15, },
  { 2, 3, 4, 11, 13, 15, },
  { 0, 2, 3, 4, 11, 13, 15, },
  { 1, 2, 3, 4, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 11, 13, 15, },
  { 5, 11, 13, 15, },
  { 0, 5, 11, 13, 15, },
  { 1, 5, 11, 13, 15, },
  { 0, 1, 5, 11, 13, 15, },
  { 2, 5, 11, 13, 15, },
  { 0, 2, 5, 11, 13, 15, },
  { 1, 2, 5, 11, 13, 15, },
  { 0, 1, 2, 5, 11, 13, 15, },
  { 3, 5, 11, 13, 15, },
  { 0, 3, 5, 11, 13, 15, },
  { 1, 3, 5, 11, 13, 15, },
  { 0, 1, 3, 5, 11, 13, 15, },
  { 2, 3, 5, 11, 13, 15, },
  { 0, 2, 3, 5, 11, 13, 15, },
  { 1, 2, 3, 5, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 11, 13, 15, },
  { 4, 5, 11, 13, 15, },
  { 0, 4, 5, 11, 13, 15, },
  { 1, 4, 5, 11, 13, 15, },
  { 0, 1, 4, 5, 11, 13, 15, },
  { 2, 4, 5, 11, 13, 15, },
  { 0, 2, 4, 5, 11, 13, 15, },
  { 1, 2, 4, 5, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 11, 13, 15, },
  { 3, 4, 5, 11, 13, 15, },
  { 0, 3, 4, 5, 11, 13, 15, },
  { 1, 3, 4, 5, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 11, 13, 15, },
  { 2, 3, 4, 5, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 11, 13, 15, },
  { 6, 11, 13, 15, },
  { 0, 6, 11, 13, 15, },
  { 1, 6, 11, 13, 15, },
  { 0, 1, 6, 11, 13, 15, },
  { 2, 6, 11, 13, 15, },
  { 0, 2, 6, 11, 13, 15, },
  { 1, 2, 6, 11, 13, 15, },
  { 0, 1, 2, 6, 11, 13, 15, },
  { 3, 6, 11, 13, 15, },
  { 0, 3, 6, 11, 13, 15, },
  { 1, 3, 6, 11, 13, 15, },
  { 0, 1, 3, 6, 11, 13, 15, },
  { 2, 3, 6, 11, 13, 15, },
  { 0, 2, 3, 6, 11, 13, 15, },
  { 1, 2, 3, 6, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 11, 13, 15, },
  { 4, 6, 11, 13, 15, },
  { 0, 4, 6, 11, 13, 15, },
  { 1, 4, 6, 11, 13, 15, },
  { 0, 1, 4, 6, 11, 13, 15, },
  { 2, 4, 6, 11, 13, 15, },
  { 0, 2, 4, 6, 11, 13, 15, },
  { 1, 2, 4, 6, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 11, 13, 15, },
  { 3, 4, 6, 11, 13, 15, },
  { 0, 3, 4, 6, 11, 13, 15, },
  { 1, 3, 4, 6, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 11, 13, 15, },
  { 2, 3, 4, 6, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 11, 13, 15, },
  { 5, 6, 11, 13, 15, },
  { 0, 5, 6, 11, 13, 15, },
  { 1, 5, 6, 11, 13, 15, },
  { 0, 1, 5, 6, 11, 13, 15, },
  { 2, 5, 6, 11, 13, 15, },
  { 0, 2, 5, 6, 11, 13, 15, },
  { 1, 2, 5, 6, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 11, 13, 15, },
  { 3, 5, 6, 11, 13, 15, },
  { 0, 3, 5, 6, 11, 13, 15, },
  { 1, 3, 5, 6, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 11, 13, 15, },
  { 2, 3, 5, 6, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 11, 13, 15, },
  { 4, 5, 6, 11, 13, 15, },
  { 0, 4, 5, 6, 11, 13, 15, },
  { 1, 4, 5, 6, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 11, 13, 15, },
  { 2, 4, 5, 6, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 11, 13, 15, },
  { 3, 4, 5, 6, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 13, 15, },
  { 7, 11, 13, 15, },
  { 0, 7, 11, 13, 15, },
  { 1, 7, 11, 13, 15, },
  { 0, 1, 7, 11, 13, 15, },
  { 2, 7, 11, 13, 15, },
  { 0, 2, 7, 11, 13, 15, },
  { 1, 2, 7, 11, 13, 15, },
  { 0, 1, 2, 7, 11, 13, 15, },
  { 3, 7, 11, 13, 15, },
  { 0, 3, 7, 11, 13, 15, },
  { 1, 3, 7, 11, 13, 15, },
  { 0, 1, 3, 7, 11, 13, 15, },
  { 2, 3, 7, 11, 13, 15, },
  { 0, 2, 3, 7, 11, 13, 15, },
  { 1, 2, 3, 7, 11, 13, 15, },
  { 0, 1, 2, 3, 7, 11, 13, 15, },
  { 4, 7, 11, 13, 15, },
  { 0, 4, 7, 11, 13, 15, },
  { 1, 4, 7, 11, 13, 15, },
  { 0, 1, 4, 7, 11, 13, 15, },
  { 2, 4, 7, 11, 13, 15, },
  { 0, 2, 4, 7, 11, 13, 15, },
  { 1, 2, 4, 7, 11, 13, 15, },
  { 0, 1, 2, 4, 7, 11, 13, 15, },
  { 3, 4, 7, 11, 13, 15, },
  { 0, 3, 4, 7, 11, 13, 15, },
  { 1, 3, 4, 7, 11, 13, 15, },
  { 0, 1, 3, 4, 7, 11, 13, 15, },
  { 2, 3, 4, 7, 11, 13, 15, },
  { 0, 2, 3, 4, 7, 11, 13, 15, },
  { 1, 2, 3, 4, 7, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 11, 13, 15, },
  { 5, 7, 11, 13, 15, },
  { 0, 5, 7, 11, 13, 15, },
  { 1, 5, 7, 11, 13, 15, },
  { 0, 1, 5, 7, 11, 13, 15, },
  { 2, 5, 7, 11, 13, 15, },
  { 0, 2, 5, 7, 11, 13, 15, },
  { 1, 2, 5, 7, 11, 13, 15, },
  { 0, 1, 2, 5, 7, 11, 13, 15, },
  { 3, 5, 7, 11, 13, 15, },
  { 0, 3, 5, 7, 11, 13, 15, },
  { 1, 3, 5, 7, 11, 13, 15, },
  { 0, 1, 3, 5, 7, 11, 13, 15, },
  { 2, 3, 5, 7, 11, 13, 15, },
  { 0, 2, 3, 5, 7, 11, 13, 15, },
  { 1, 2, 3, 5, 7, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 11, 13, 15, },
  { 4, 5, 7, 11, 13, 15, },
  { 0, 4, 5, 7, 11, 13, 15, },
  { 1, 4, 5, 7, 11, 13, 15, },
  { 0, 1, 4, 5, 7, 11, 13, 15, },
  { 2, 4, 5, 7, 11, 13, 15, },
  { 0, 2, 4, 5, 7, 11, 13, 15, },
  { 1, 2, 4, 5, 7, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 11, 13, 15, },
  { 3, 4, 5, 7, 11, 13, 15, },
  { 0, 3, 4, 5, 7, 11, 13, 15, },
  { 1, 3, 4, 5, 7, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 11, 13, 15, },
  { 2, 3, 4, 5, 7, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 13, 15, },
  { 6, 7, 11, 13, 15, },
  { 0, 6, 7, 11, 13, 15, },
  { 1, 6, 7, 11, 13, 15, },
  { 0, 1, 6, 7, 11, 13, 15, },
  { 2, 6, 7, 11, 13, 15, },
  { 0, 2, 6, 7, 11, 13, 15, },
  { 1, 2, 6, 7, 11, 13, 15, },
  { 0, 1, 2, 6, 7, 11, 13, 15, },
  { 3, 6, 7, 11, 13, 15, },
  { 0, 3, 6, 7, 11, 13, 15, },
  { 1, 3, 6, 7, 11, 13, 15, },
  { 0, 1, 3, 6, 7, 11, 13, 15, },
  { 2, 3, 6, 7, 11, 13, 15, },
  { 0, 2, 3, 6, 7, 11, 13, 15, },
  { 1, 2, 3, 6, 7, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 11, 13, 15, },
  { 4, 6, 7, 11, 13, 15, },
  { 0, 4, 6, 7, 11, 13, 15, },
  { 1, 4, 6, 7, 11, 13, 15, },
  { 0, 1, 4, 6, 7, 11, 13, 15, },
  { 2, 4, 6, 7, 11, 13, 15, },
  { 0, 2, 4, 6, 7, 11, 13, 15, },
  { 1, 2, 4, 6, 7, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 11, 13, 15, },
  { 3, 4, 6, 7, 11, 13, 15, },
  { 0, 3, 4, 6, 7, 11, 13, 15, },
  { 1, 3, 4, 6, 7, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 11, 13, 15, },
  { 2, 3, 4, 6, 7, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 13, 15, },
  { 5, 6, 7, 11, 13, 15, },
  { 0, 5, 6, 7, 11, 13, 15, },
  { 1, 5, 6, 7, 11, 13, 15, },
  { 0, 1, 5, 6, 7, 11, 13, 15, },
  { 2, 5, 6, 7, 11, 13, 15, },
  { 0, 2, 5, 6, 7, 11, 13, 15, },
  { 1, 2, 5, 6, 7, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 11, 13, 15, },
  { 3, 5, 6, 7, 11, 13, 15, },
  { 0, 3, 5, 6, 7, 11, 13, 15, },
  { 1, 3, 5, 6, 7, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 11, 13, 15, },
  { 2, 3, 5, 6, 7, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 13, 15, },
  { 4, 5, 6, 7, 11, 13, 15, },
  { 0, 4, 5, 6, 7, 11, 13, 15, },
  { 1, 4, 5, 6, 7, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 11, 13, 15, },
  { 2, 4, 5, 6, 7, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 13, 15, },
  { 3, 4, 5, 6, 7, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 13, 15, },
  { 8, 11, 13, 15, },
  { 0, 8, 11, 13, 15, },
  { 1, 8, 11, 13, 15, },
  { 0, 1, 8, 11, 13, 15, },
  { 2, 8, 11, 13, 15, },
  { 0, 2, 8, 11, 13, 15, },
  { 1, 2, 8, 11, 13, 15, },
  { 0, 1, 2, 8, 11, 13, 15, },
  { 3, 8, 11, 13, 15, },
  { 0, 3, 8, 11, 13, 15, },
  { 1, 3, 8, 11, 13, 15, },
  { 0, 1, 3, 8, 11, 13, 15, },
  { 2, 3, 8, 11, 13, 15, },
  { 0, 2, 3, 8, 11, 13, 15, },
  { 1, 2, 3, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 8, 11, 13, 15, },
  { 4, 8, 11, 13, 15, },
  { 0, 4, 8, 11, 13, 15, },
  { 1, 4, 8, 11, 13, 15, },
  { 0, 1, 4, 8, 11, 13, 15, },
  { 2, 4, 8, 11, 13, 15, },
  { 0, 2, 4, 8, 11, 13, 15, },
  { 1, 2, 4, 8, 11, 13, 15, },
  { 0, 1, 2, 4, 8, 11, 13, 15, },
  { 3, 4, 8, 11, 13, 15, },
  { 0, 3, 4, 8, 11, 13, 15, },
  { 1, 3, 4, 8, 11, 13, 15, },
  { 0, 1, 3, 4, 8, 11, 13, 15, },
  { 2, 3, 4, 8, 11, 13, 15, },
  { 0, 2, 3, 4, 8, 11, 13, 15, },
  { 1, 2, 3, 4, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 11, 13, 15, },
  { 5, 8, 11, 13, 15, },
  { 0, 5, 8, 11, 13, 15, },
  { 1, 5, 8, 11, 13, 15, },
  { 0, 1, 5, 8, 11, 13, 15, },
  { 2, 5, 8, 11, 13, 15, },
  { 0, 2, 5, 8, 11, 13, 15, },
  { 1, 2, 5, 8, 11, 13, 15, },
  { 0, 1, 2, 5, 8, 11, 13, 15, },
  { 3, 5, 8, 11, 13, 15, },
  { 0, 3, 5, 8, 11, 13, 15, },
  { 1, 3, 5, 8, 11, 13, 15, },
  { 0, 1, 3, 5, 8, 11, 13, 15, },
  { 2, 3, 5, 8, 11, 13, 15, },
  { 0, 2, 3, 5, 8, 11, 13, 15, },
  { 1, 2, 3, 5, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 11, 13, 15, },
  { 4, 5, 8, 11, 13, 15, },
  { 0, 4, 5, 8, 11, 13, 15, },
  { 1, 4, 5, 8, 11, 13, 15, },
  { 0, 1, 4, 5, 8, 11, 13, 15, },
  { 2, 4, 5, 8, 11, 13, 15, },
  { 0, 2, 4, 5, 8, 11, 13, 15, },
  { 1, 2, 4, 5, 8, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 11, 13, 15, },
  { 3, 4, 5, 8, 11, 13, 15, },
  { 0, 3, 4, 5, 8, 11, 13, 15, },
  { 1, 3, 4, 5, 8, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 11, 13, 15, },
  { 2, 3, 4, 5, 8, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 13, 15, },
  { 6, 8, 11, 13, 15, },
  { 0, 6, 8, 11, 13, 15, },
  { 1, 6, 8, 11, 13, 15, },
  { 0, 1, 6, 8, 11, 13, 15, },
  { 2, 6, 8, 11, 13, 15, },
  { 0, 2, 6, 8, 11, 13, 15, },
  { 1, 2, 6, 8, 11, 13, 15, },
  { 0, 1, 2, 6, 8, 11, 13, 15, },
  { 3, 6, 8, 11, 13, 15, },
  { 0, 3, 6, 8, 11, 13, 15, },
  { 1, 3, 6, 8, 11, 13, 15, },
  { 0, 1, 3, 6, 8, 11, 13, 15, },
  { 2, 3, 6, 8, 11, 13, 15, },
  { 0, 2, 3, 6, 8, 11, 13, 15, },
  { 1, 2, 3, 6, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 11, 13, 15, },
  { 4, 6, 8, 11, 13, 15, },
  { 0, 4, 6, 8, 11, 13, 15, },
  { 1, 4, 6, 8, 11, 13, 15, },
  { 0, 1, 4, 6, 8, 11, 13, 15, },
  { 2, 4, 6, 8, 11, 13, 15, },
  { 0, 2, 4, 6, 8, 11, 13, 15, },
  { 1, 2, 4, 6, 8, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 11, 13, 15, },
  { 3, 4, 6, 8, 11, 13, 15, },
  { 0, 3, 4, 6, 8, 11, 13, 15, },
  { 1, 3, 4, 6, 8, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 11, 13, 15, },
  { 2, 3, 4, 6, 8, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 13, 15, },
  { 5, 6, 8, 11, 13, 15, },
  { 0, 5, 6, 8, 11, 13, 15, },
  { 1, 5, 6, 8, 11, 13, 15, },
  { 0, 1, 5, 6, 8, 11, 13, 15, },
  { 2, 5, 6, 8, 11, 13, 15, },
  { 0, 2, 5, 6, 8, 11, 13, 15, },
  { 1, 2, 5, 6, 8, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 11, 13, 15, },
  { 3, 5, 6, 8, 11, 13, 15, },
  { 0, 3, 5, 6, 8, 11, 13, 15, },
  { 1, 3, 5, 6, 8, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 11, 13, 15, },
  { 2, 3, 5, 6, 8, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 13, 15, },
  { 4, 5, 6, 8, 11, 13, 15, },
  { 0, 4, 5, 6, 8, 11, 13, 15, },
  { 1, 4, 5, 6, 8, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 11, 13, 15, },
  { 2, 4, 5, 6, 8, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 13, 15, },
  { 3, 4, 5, 6, 8, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 13, 15, },
  { 7, 8, 11, 13, 15, },
  { 0, 7, 8, 11, 13, 15, },
  { 1, 7, 8, 11, 13, 15, },
  { 0, 1, 7, 8, 11, 13, 15, },
  { 2, 7, 8, 11, 13, 15, },
  { 0, 2, 7, 8, 11, 13, 15, },
  { 1, 2, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 7, 8, 11, 13, 15, },
  { 3, 7, 8, 11, 13, 15, },
  { 0, 3, 7, 8, 11, 13, 15, },
  { 1, 3, 7, 8, 11, 13, 15, },
  { 0, 1, 3, 7, 8, 11, 13, 15, },
  { 2, 3, 7, 8, 11, 13, 15, },
  { 0, 2, 3, 7, 8, 11, 13, 15, },
  { 1, 2, 3, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 11, 13, 15, },
  { 4, 7, 8, 11, 13, 15, },
  { 0, 4, 7, 8, 11, 13, 15, },
  { 1, 4, 7, 8, 11, 13, 15, },
  { 0, 1, 4, 7, 8, 11, 13, 15, },
  { 2, 4, 7, 8, 11, 13, 15, },
  { 0, 2, 4, 7, 8, 11, 13, 15, },
  { 1, 2, 4, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 11, 13, 15, },
  { 3, 4, 7, 8, 11, 13, 15, },
  { 0, 3, 4, 7, 8, 11, 13, 15, },
  { 1, 3, 4, 7, 8, 11, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 11, 13, 15, },
  { 2, 3, 4, 7, 8, 11, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 11, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 13, 15, },
  { 5, 7, 8, 11, 13, 15, },
  { 0, 5, 7, 8, 11, 13, 15, },
  { 1, 5, 7, 8, 11, 13, 15, },
  { 0, 1, 5, 7, 8, 11, 13, 15, },
  { 2, 5, 7, 8, 11, 13, 15, },
  { 0, 2, 5, 7, 8, 11, 13, 15, },
  { 1, 2, 5, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 11, 13, 15, },
  { 3, 5, 7, 8, 11, 13, 15, },
  { 0, 3, 5, 7, 8, 11, 13, 15, },
  { 1, 3, 5, 7, 8, 11, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 11, 13, 15, },
  { 2, 3, 5, 7, 8, 11, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 11, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 13, 15, },
  { 4, 5, 7, 8, 11, 13, 15, },
  { 0, 4, 5, 7, 8, 11, 13, 15, },
  { 1, 4, 5, 7, 8, 11, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 11, 13, 15, },
  { 2, 4, 5, 7, 8, 11, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 11, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 13, 15, },
  { 3, 4, 5, 7, 8, 11, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 11, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 13, 15, },
  { 6, 7, 8, 11, 13, 15, },
  { 0, 6, 7, 8, 11, 13, 15, },
  { 1, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 6, 7, 8, 11, 13, 15, },
  { 2, 6, 7, 8, 11, 13, 15, },
  { 0, 2, 6, 7, 8, 11, 13, 15, },
  { 1, 2, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 11, 13, 15, },
  { 3, 6, 7, 8, 11, 13, 15, },
  { 0, 3, 6, 7, 8, 11, 13, 15, },
  { 1, 3, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 11, 13, 15, },
  { 2, 3, 6, 7, 8, 11, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 11, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 13, 15, },
  { 4, 6, 7, 8, 11, 13, 15, },
  { 0, 4, 6, 7, 8, 11, 13, 15, },
  { 1, 4, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 11, 13, 15, },
  { 2, 4, 6, 7, 8, 11, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 11, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 13, 15, },
  { 3, 4, 6, 7, 8, 11, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 11, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 13, 15, },
  { 5, 6, 7, 8, 11, 13, 15, },
  { 0, 5, 6, 7, 8, 11, 13, 15, },
  { 1, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 11, 13, 15, },
  { 2, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 11, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 13, 15, },
  { 3, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 11, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 13, 15, },
  { 4, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, 15, },
  { 9, 11, 13, 15, },
  { 0, 9, 11, 13, 15, },
  { 1, 9, 11, 13, 15, },
  { 0, 1, 9, 11, 13, 15, },
  { 2, 9, 11, 13, 15, },
  { 0, 2, 9, 11, 13, 15, },
  { 1, 2, 9, 11, 13, 15, },
  { 0, 1, 2, 9, 11, 13, 15, },
  { 3, 9, 11, 13, 15, },
  { 0, 3, 9, 11, 13, 15, },
  { 1, 3, 9, 11, 13, 15, },
  { 0, 1, 3, 9, 11, 13, 15, },
  { 2, 3, 9, 11, 13, 15, },
  { 0, 2, 3, 9, 11, 13, 15, },
  { 1, 2, 3, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 9, 11, 13, 15, },
  { 4, 9, 11, 13, 15, },
  { 0, 4, 9, 11, 13, 15, },
  { 1, 4, 9, 11, 13, 15, },
  { 0, 1, 4, 9, 11, 13, 15, },
  { 2, 4, 9, 11, 13, 15, },
  { 0, 2, 4, 9, 11, 13, 15, },
  { 1, 2, 4, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 9, 11, 13, 15, },
  { 3, 4, 9, 11, 13, 15, },
  { 0, 3, 4, 9, 11, 13, 15, },
  { 1, 3, 4, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 9, 11, 13, 15, },
  { 2, 3, 4, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 9, 11, 13, 15, },
  { 5, 9, 11, 13, 15, },
  { 0, 5, 9, 11, 13, 15, },
  { 1, 5, 9, 11, 13, 15, },
  { 0, 1, 5, 9, 11, 13, 15, },
  { 2, 5, 9, 11, 13, 15, },
  { 0, 2, 5, 9, 11, 13, 15, },
  { 1, 2, 5, 9, 11, 13, 15, },
  { 0, 1, 2, 5, 9, 11, 13, 15, },
  { 3, 5, 9, 11, 13, 15, },
  { 0, 3, 5, 9, 11, 13, 15, },
  { 1, 3, 5, 9, 11, 13, 15, },
  { 0, 1, 3, 5, 9, 11, 13, 15, },
  { 2, 3, 5, 9, 11, 13, 15, },
  { 0, 2, 3, 5, 9, 11, 13, 15, },
  { 1, 2, 3, 5, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 9, 11, 13, 15, },
  { 4, 5, 9, 11, 13, 15, },
  { 0, 4, 5, 9, 11, 13, 15, },
  { 1, 4, 5, 9, 11, 13, 15, },
  { 0, 1, 4, 5, 9, 11, 13, 15, },
  { 2, 4, 5, 9, 11, 13, 15, },
  { 0, 2, 4, 5, 9, 11, 13, 15, },
  { 1, 2, 4, 5, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 9, 11, 13, 15, },
  { 3, 4, 5, 9, 11, 13, 15, },
  { 0, 3, 4, 5, 9, 11, 13, 15, },
  { 1, 3, 4, 5, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 9, 11, 13, 15, },
  { 2, 3, 4, 5, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 13, 15, },
  { 6, 9, 11, 13, 15, },
  { 0, 6, 9, 11, 13, 15, },
  { 1, 6, 9, 11, 13, 15, },
  { 0, 1, 6, 9, 11, 13, 15, },
  { 2, 6, 9, 11, 13, 15, },
  { 0, 2, 6, 9, 11, 13, 15, },
  { 1, 2, 6, 9, 11, 13, 15, },
  { 0, 1, 2, 6, 9, 11, 13, 15, },
  { 3, 6, 9, 11, 13, 15, },
  { 0, 3, 6, 9, 11, 13, 15, },
  { 1, 3, 6, 9, 11, 13, 15, },
  { 0, 1, 3, 6, 9, 11, 13, 15, },
  { 2, 3, 6, 9, 11, 13, 15, },
  { 0, 2, 3, 6, 9, 11, 13, 15, },
  { 1, 2, 3, 6, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 9, 11, 13, 15, },
  { 4, 6, 9, 11, 13, 15, },
  { 0, 4, 6, 9, 11, 13, 15, },
  { 1, 4, 6, 9, 11, 13, 15, },
  { 0, 1, 4, 6, 9, 11, 13, 15, },
  { 2, 4, 6, 9, 11, 13, 15, },
  { 0, 2, 4, 6, 9, 11, 13, 15, },
  { 1, 2, 4, 6, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 9, 11, 13, 15, },
  { 3, 4, 6, 9, 11, 13, 15, },
  { 0, 3, 4, 6, 9, 11, 13, 15, },
  { 1, 3, 4, 6, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 9, 11, 13, 15, },
  { 2, 3, 4, 6, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 13, 15, },
  { 5, 6, 9, 11, 13, 15, },
  { 0, 5, 6, 9, 11, 13, 15, },
  { 1, 5, 6, 9, 11, 13, 15, },
  { 0, 1, 5, 6, 9, 11, 13, 15, },
  { 2, 5, 6, 9, 11, 13, 15, },
  { 0, 2, 5, 6, 9, 11, 13, 15, },
  { 1, 2, 5, 6, 9, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 9, 11, 13, 15, },
  { 3, 5, 6, 9, 11, 13, 15, },
  { 0, 3, 5, 6, 9, 11, 13, 15, },
  { 1, 3, 5, 6, 9, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 9, 11, 13, 15, },
  { 2, 3, 5, 6, 9, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 9, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 13, 15, },
  { 4, 5, 6, 9, 11, 13, 15, },
  { 0, 4, 5, 6, 9, 11, 13, 15, },
  { 1, 4, 5, 6, 9, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 9, 11, 13, 15, },
  { 2, 4, 5, 6, 9, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 9, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 13, 15, },
  { 3, 4, 5, 6, 9, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 9, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 13, 15, },
  { 7, 9, 11, 13, 15, },
  { 0, 7, 9, 11, 13, 15, },
  { 1, 7, 9, 11, 13, 15, },
  { 0, 1, 7, 9, 11, 13, 15, },
  { 2, 7, 9, 11, 13, 15, },
  { 0, 2, 7, 9, 11, 13, 15, },
  { 1, 2, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 7, 9, 11, 13, 15, },
  { 3, 7, 9, 11, 13, 15, },
  { 0, 3, 7, 9, 11, 13, 15, },
  { 1, 3, 7, 9, 11, 13, 15, },
  { 0, 1, 3, 7, 9, 11, 13, 15, },
  { 2, 3, 7, 9, 11, 13, 15, },
  { 0, 2, 3, 7, 9, 11, 13, 15, },
  { 1, 2, 3, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 7, 9, 11, 13, 15, },
  { 4, 7, 9, 11, 13, 15, },
  { 0, 4, 7, 9, 11, 13, 15, },
  { 1, 4, 7, 9, 11, 13, 15, },
  { 0, 1, 4, 7, 9, 11, 13, 15, },
  { 2, 4, 7, 9, 11, 13, 15, },
  { 0, 2, 4, 7, 9, 11, 13, 15, },
  { 1, 2, 4, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 7, 9, 11, 13, 15, },
  { 3, 4, 7, 9, 11, 13, 15, },
  { 0, 3, 4, 7, 9, 11, 13, 15, },
  { 1, 3, 4, 7, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 7, 9, 11, 13, 15, },
  { 2, 3, 4, 7, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 7, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 13, 15, },
  { 5, 7, 9, 11, 13, 15, },
  { 0, 5, 7, 9, 11, 13, 15, },
  { 1, 5, 7, 9, 11, 13, 15, },
  { 0, 1, 5, 7, 9, 11, 13, 15, },
  { 2, 5, 7, 9, 11, 13, 15, },
  { 0, 2, 5, 7, 9, 11, 13, 15, },
  { 1, 2, 5, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 5, 7, 9, 11, 13, 15, },
  { 3, 5, 7, 9, 11, 13, 15, },
  { 0, 3, 5, 7, 9, 11, 13, 15, },
  { 1, 3, 5, 7, 9, 11, 13, 15, },
  { 0, 1, 3, 5, 7, 9, 11, 13, 15, },
  { 2, 3, 5, 7, 9, 11, 13, 15, },
  { 0, 2, 3, 5, 7, 9, 11, 13, 15, },
  { 1, 2, 3, 5, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 13, 15, },
  { 4, 5, 7, 9, 11, 13, 15, },
  { 0, 4, 5, 7, 9, 11, 13, 15, },
  { 1, 4, 5, 7, 9, 11, 13, 15, },
  { 0, 1, 4, 5, 7, 9, 11, 13, 15, },
  { 2, 4, 5, 7, 9, 11, 13, 15, },
  { 0, 2, 4, 5, 7, 9, 11, 13, 15, },
  { 1, 2, 4, 5, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 13, 15, },
  { 3, 4, 5, 7, 9, 11, 13, 15, },
  { 0, 3, 4, 5, 7, 9, 11, 13, 15, },
  { 1, 3, 4, 5, 7, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 13, 15, },
  { 2, 3, 4, 5, 7, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 13, 15, },
  { 6, 7, 9, 11, 13, 15, },
  { 0, 6, 7, 9, 11, 13, 15, },
  { 1, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 6, 7, 9, 11, 13, 15, },
  { 2, 6, 7, 9, 11, 13, 15, },
  { 0, 2, 6, 7, 9, 11, 13, 15, },
  { 1, 2, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 6, 7, 9, 11, 13, 15, },
  { 3, 6, 7, 9, 11, 13, 15, },
  { 0, 3, 6, 7, 9, 11, 13, 15, },
  { 1, 3, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 3, 6, 7, 9, 11, 13, 15, },
  { 2, 3, 6, 7, 9, 11, 13, 15, },
  { 0, 2, 3, 6, 7, 9, 11, 13, 15, },
  { 1, 2, 3, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 13, 15, },
  { 4, 6, 7, 9, 11, 13, 15, },
  { 0, 4, 6, 7, 9, 11, 13, 15, },
  { 1, 4, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 4, 6, 7, 9, 11, 13, 15, },
  { 2, 4, 6, 7, 9, 11, 13, 15, },
  { 0, 2, 4, 6, 7, 9, 11, 13, 15, },
  { 1, 2, 4, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 13, 15, },
  { 3, 4, 6, 7, 9, 11, 13, 15, },
  { 0, 3, 4, 6, 7, 9, 11, 13, 15, },
  { 1, 3, 4, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 13, 15, },
  { 2, 3, 4, 6, 7, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 13, 15, },
  { 5, 6, 7, 9, 11, 13, 15, },
  { 0, 5, 6, 7, 9, 11, 13, 15, },
  { 1, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 5, 6, 7, 9, 11, 13, 15, },
  { 2, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 2, 5, 6, 7, 9, 11, 13, 15, },
  { 1, 2, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 13, 15, },
  { 3, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 3, 5, 6, 7, 9, 11, 13, 15, },
  { 1, 3, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 13, 15, },
  { 2, 3, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 15, },
  { 4, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 1, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 2, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 3, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 15, },
  { 8, 9, 11, 13, 15, },
  { 0, 8, 9, 11, 13, 15, },
  { 1, 8, 9, 11, 13, 15, },
  { 0, 1, 8, 9, 11, 13, 15, },
  { 2, 8, 9, 11, 13, 15, },
  { 0, 2, 8, 9, 11, 13, 15, },
  { 1, 2, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 8, 9, 11, 13, 15, },
  { 3, 8, 9, 11, 13, 15, },
  { 0, 3, 8, 9, 11, 13, 15, },
  { 1, 3, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 8, 9, 11, 13, 15, },
  { 2, 3, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 8, 9, 11, 13, 15, },
  { 4, 8, 9, 11, 13, 15, },
  { 0, 4, 8, 9, 11, 13, 15, },
  { 1, 4, 8, 9, 11, 13, 15, },
  { 0, 1, 4, 8, 9, 11, 13, 15, },
  { 2, 4, 8, 9, 11, 13, 15, },
  { 0, 2, 4, 8, 9, 11, 13, 15, },
  { 1, 2, 4, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 8, 9, 11, 13, 15, },
  { 3, 4, 8, 9, 11, 13, 15, },
  { 0, 3, 4, 8, 9, 11, 13, 15, },
  { 1, 3, 4, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 8, 9, 11, 13, 15, },
  { 2, 3, 4, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 13, 15, },
  { 5, 8, 9, 11, 13, 15, },
  { 0, 5, 8, 9, 11, 13, 15, },
  { 1, 5, 8, 9, 11, 13, 15, },
  { 0, 1, 5, 8, 9, 11, 13, 15, },
  { 2, 5, 8, 9, 11, 13, 15, },
  { 0, 2, 5, 8, 9, 11, 13, 15, },
  { 1, 2, 5, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 5, 8, 9, 11, 13, 15, },
  { 3, 5, 8, 9, 11, 13, 15, },
  { 0, 3, 5, 8, 9, 11, 13, 15, },
  { 1, 3, 5, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 5, 8, 9, 11, 13, 15, },
  { 2, 3, 5, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 5, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 5, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 13, 15, },
  { 4, 5, 8, 9, 11, 13, 15, },
  { 0, 4, 5, 8, 9, 11, 13, 15, },
  { 1, 4, 5, 8, 9, 11, 13, 15, },
  { 0, 1, 4, 5, 8, 9, 11, 13, 15, },
  { 2, 4, 5, 8, 9, 11, 13, 15, },
  { 0, 2, 4, 5, 8, 9, 11, 13, 15, },
  { 1, 2, 4, 5, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 13, 15, },
  { 3, 4, 5, 8, 9, 11, 13, 15, },
  { 0, 3, 4, 5, 8, 9, 11, 13, 15, },
  { 1, 3, 4, 5, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 13, 15, },
  { 2, 3, 4, 5, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 13, 15, },
  { 6, 8, 9, 11, 13, 15, },
  { 0, 6, 8, 9, 11, 13, 15, },
  { 1, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 6, 8, 9, 11, 13, 15, },
  { 2, 6, 8, 9, 11, 13, 15, },
  { 0, 2, 6, 8, 9, 11, 13, 15, },
  { 1, 2, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 6, 8, 9, 11, 13, 15, },
  { 3, 6, 8, 9, 11, 13, 15, },
  { 0, 3, 6, 8, 9, 11, 13, 15, },
  { 1, 3, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 6, 8, 9, 11, 13, 15, },
  { 2, 3, 6, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 6, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 13, 15, },
  { 4, 6, 8, 9, 11, 13, 15, },
  { 0, 4, 6, 8, 9, 11, 13, 15, },
  { 1, 4, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 4, 6, 8, 9, 11, 13, 15, },
  { 2, 4, 6, 8, 9, 11, 13, 15, },
  { 0, 2, 4, 6, 8, 9, 11, 13, 15, },
  { 1, 2, 4, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 13, 15, },
  { 3, 4, 6, 8, 9, 11, 13, 15, },
  { 0, 3, 4, 6, 8, 9, 11, 13, 15, },
  { 1, 3, 4, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 13, 15, },
  { 2, 3, 4, 6, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 13, 15, },
  { 5, 6, 8, 9, 11, 13, 15, },
  { 0, 5, 6, 8, 9, 11, 13, 15, },
  { 1, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 5, 6, 8, 9, 11, 13, 15, },
  { 2, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 2, 5, 6, 8, 9, 11, 13, 15, },
  { 1, 2, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 13, 15, },
  { 3, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 3, 5, 6, 8, 9, 11, 13, 15, },
  { 1, 3, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 13, 15, },
  { 2, 3, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 13, 15, },
  { 4, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 1, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 2, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 3, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 13, 15, },
  { 7, 8, 9, 11, 13, 15, },
  { 0, 7, 8, 9, 11, 13, 15, },
  { 1, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 7, 8, 9, 11, 13, 15, },
  { 2, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 7, 8, 9, 11, 13, 15, },
  { 3, 7, 8, 9, 11, 13, 15, },
  { 0, 3, 7, 8, 9, 11, 13, 15, },
  { 1, 3, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 7, 8, 9, 11, 13, 15, },
  { 2, 3, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 13, 15, },
  { 4, 7, 8, 9, 11, 13, 15, },
  { 0, 4, 7, 8, 9, 11, 13, 15, },
  { 1, 4, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 4, 7, 8, 9, 11, 13, 15, },
  { 2, 4, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 4, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 4, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 13, 15, },
  { 3, 4, 7, 8, 9, 11, 13, 15, },
  { 0, 3, 4, 7, 8, 9, 11, 13, 15, },
  { 1, 3, 4, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 13, 15, },
  { 2, 3, 4, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 13, 15, },
  { 5, 7, 8, 9, 11, 13, 15, },
  { 0, 5, 7, 8, 9, 11, 13, 15, },
  { 1, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 5, 7, 8, 9, 11, 13, 15, },
  { 2, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 5, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 13, 15, },
  { 3, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 3, 5, 7, 8, 9, 11, 13, 15, },
  { 1, 3, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 13, 15, },
  { 2, 3, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 13, 15, },
  { 4, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 1, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 2, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 3, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 13, 15, },
  { 6, 7, 8, 9, 11, 13, 15, },
  { 0, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 6, 7, 8, 9, 11, 13, 15, },
  { 2, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 13, 15, },
  { 3, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 3, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 3, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 13, 15, },
  { 2, 3, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 13, 15, },
  { 4, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 2, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 3, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 13, 15, },
  { 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 2, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 3, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 15, },
  { 10, 11, 13, 15, },
  { 0, 10, 11, 13, 15, },
  { 1, 10, 11, 13, 15, },
  { 0, 1, 10, 11, 13, 15, },
  { 2, 10, 11, 13, 15, },
  { 0, 2, 10, 11, 13, 15, },
  { 1, 2, 10, 11, 13, 15, },
  { 0, 1, 2, 10, 11, 13, 15, },
  { 3, 10, 11, 13, 15, },
  { 0, 3, 10, 11, 13, 15, },
  { 1, 3, 10, 11, 13, 15, },
  { 0, 1, 3, 10, 11, 13, 15, },
  { 2, 3, 10, 11, 13, 15, },
  { 0, 2, 3, 10, 11, 13, 15, },
  { 1, 2, 3, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 10, 11, 13, 15, },
  { 4, 10, 11, 13, 15, },
  { 0, 4, 10, 11, 13, 15, },
  { 1, 4, 10, 11, 13, 15, },
  { 0, 1, 4, 10, 11, 13, 15, },
  { 2, 4, 10, 11, 13, 15, },
  { 0, 2, 4, 10, 11, 13, 15, },
  { 1, 2, 4, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 10, 11, 13, 15, },
  { 3, 4, 10, 11, 13, 15, },
  { 0, 3, 4, 10, 11, 13, 15, },
  { 1, 3, 4, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 10, 11, 13, 15, },
  { 2, 3, 4, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 10, 11, 13, 15, },
  { 5, 10, 11, 13, 15, },
  { 0, 5, 10, 11, 13, 15, },
  { 1, 5, 10, 11, 13, 15, },
  { 0, 1, 5, 10, 11, 13, 15, },
  { 2, 5, 10, 11, 13, 15, },
  { 0, 2, 5, 10, 11, 13, 15, },
  { 1, 2, 5, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 10, 11, 13, 15, },
  { 3, 5, 10, 11, 13, 15, },
  { 0, 3, 5, 10, 11, 13, 15, },
  { 1, 3, 5, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 10, 11, 13, 15, },
  { 2, 3, 5, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 10, 11, 13, 15, },
  { 4, 5, 10, 11, 13, 15, },
  { 0, 4, 5, 10, 11, 13, 15, },
  { 1, 4, 5, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 10, 11, 13, 15, },
  { 2, 4, 5, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 10, 11, 13, 15, },
  { 3, 4, 5, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 13, 15, },
  { 6, 10, 11, 13, 15, },
  { 0, 6, 10, 11, 13, 15, },
  { 1, 6, 10, 11, 13, 15, },
  { 0, 1, 6, 10, 11, 13, 15, },
  { 2, 6, 10, 11, 13, 15, },
  { 0, 2, 6, 10, 11, 13, 15, },
  { 1, 2, 6, 10, 11, 13, 15, },
  { 0, 1, 2, 6, 10, 11, 13, 15, },
  { 3, 6, 10, 11, 13, 15, },
  { 0, 3, 6, 10, 11, 13, 15, },
  { 1, 3, 6, 10, 11, 13, 15, },
  { 0, 1, 3, 6, 10, 11, 13, 15, },
  { 2, 3, 6, 10, 11, 13, 15, },
  { 0, 2, 3, 6, 10, 11, 13, 15, },
  { 1, 2, 3, 6, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 10, 11, 13, 15, },
  { 4, 6, 10, 11, 13, 15, },
  { 0, 4, 6, 10, 11, 13, 15, },
  { 1, 4, 6, 10, 11, 13, 15, },
  { 0, 1, 4, 6, 10, 11, 13, 15, },
  { 2, 4, 6, 10, 11, 13, 15, },
  { 0, 2, 4, 6, 10, 11, 13, 15, },
  { 1, 2, 4, 6, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 10, 11, 13, 15, },
  { 3, 4, 6, 10, 11, 13, 15, },
  { 0, 3, 4, 6, 10, 11, 13, 15, },
  { 1, 3, 4, 6, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 10, 11, 13, 15, },
  { 2, 3, 4, 6, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 13, 15, },
  { 5, 6, 10, 11, 13, 15, },
  { 0, 5, 6, 10, 11, 13, 15, },
  { 1, 5, 6, 10, 11, 13, 15, },
  { 0, 1, 5, 6, 10, 11, 13, 15, },
  { 2, 5, 6, 10, 11, 13, 15, },
  { 0, 2, 5, 6, 10, 11, 13, 15, },
  { 1, 2, 5, 6, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 10, 11, 13, 15, },
  { 3, 5, 6, 10, 11, 13, 15, },
  { 0, 3, 5, 6, 10, 11, 13, 15, },
  { 1, 3, 5, 6, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 10, 11, 13, 15, },
  { 2, 3, 5, 6, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 13, 15, },
  { 4, 5, 6, 10, 11, 13, 15, },
  { 0, 4, 5, 6, 10, 11, 13, 15, },
  { 1, 4, 5, 6, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 10, 11, 13, 15, },
  { 2, 4, 5, 6, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 13, 15, },
  { 3, 4, 5, 6, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 13, 15, },
  { 7, 10, 11, 13, 15, },
  { 0, 7, 10, 11, 13, 15, },
  { 1, 7, 10, 11, 13, 15, },
  { 0, 1, 7, 10, 11, 13, 15, },
  { 2, 7, 10, 11, 13, 15, },
  { 0, 2, 7, 10, 11, 13, 15, },
  { 1, 2, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 7, 10, 11, 13, 15, },
  { 3, 7, 10, 11, 13, 15, },
  { 0, 3, 7, 10, 11, 13, 15, },
  { 1, 3, 7, 10, 11, 13, 15, },
  { 0, 1, 3, 7, 10, 11, 13, 15, },
  { 2, 3, 7, 10, 11, 13, 15, },
  { 0, 2, 3, 7, 10, 11, 13, 15, },
  { 1, 2, 3, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 7, 10, 11, 13, 15, },
  { 4, 7, 10, 11, 13, 15, },
  { 0, 4, 7, 10, 11, 13, 15, },
  { 1, 4, 7, 10, 11, 13, 15, },
  { 0, 1, 4, 7, 10, 11, 13, 15, },
  { 2, 4, 7, 10, 11, 13, 15, },
  { 0, 2, 4, 7, 10, 11, 13, 15, },
  { 1, 2, 4, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 7, 10, 11, 13, 15, },
  { 3, 4, 7, 10, 11, 13, 15, },
  { 0, 3, 4, 7, 10, 11, 13, 15, },
  { 1, 3, 4, 7, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 7, 10, 11, 13, 15, },
  { 2, 3, 4, 7, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 7, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 13, 15, },
  { 5, 7, 10, 11, 13, 15, },
  { 0, 5, 7, 10, 11, 13, 15, },
  { 1, 5, 7, 10, 11, 13, 15, },
  { 0, 1, 5, 7, 10, 11, 13, 15, },
  { 2, 5, 7, 10, 11, 13, 15, },
  { 0, 2, 5, 7, 10, 11, 13, 15, },
  { 1, 2, 5, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 7, 10, 11, 13, 15, },
  { 3, 5, 7, 10, 11, 13, 15, },
  { 0, 3, 5, 7, 10, 11, 13, 15, },
  { 1, 3, 5, 7, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 7, 10, 11, 13, 15, },
  { 2, 3, 5, 7, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 7, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 13, 15, },
  { 4, 5, 7, 10, 11, 13, 15, },
  { 0, 4, 5, 7, 10, 11, 13, 15, },
  { 1, 4, 5, 7, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 7, 10, 11, 13, 15, },
  { 2, 4, 5, 7, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 7, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 13, 15, },
  { 3, 4, 5, 7, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 7, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 7, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 7, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 13, 15, },
  { 6, 7, 10, 11, 13, 15, },
  { 0, 6, 7, 10, 11, 13, 15, },
  { 1, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 6, 7, 10, 11, 13, 15, },
  { 2, 6, 7, 10, 11, 13, 15, },
  { 0, 2, 6, 7, 10, 11, 13, 15, },
  { 1, 2, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 6, 7, 10, 11, 13, 15, },
  { 3, 6, 7, 10, 11, 13, 15, },
  { 0, 3, 6, 7, 10, 11, 13, 15, },
  { 1, 3, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 3, 6, 7, 10, 11, 13, 15, },
  { 2, 3, 6, 7, 10, 11, 13, 15, },
  { 0, 2, 3, 6, 7, 10, 11, 13, 15, },
  { 1, 2, 3, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 13, 15, },
  { 4, 6, 7, 10, 11, 13, 15, },
  { 0, 4, 6, 7, 10, 11, 13, 15, },
  { 1, 4, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 4, 6, 7, 10, 11, 13, 15, },
  { 2, 4, 6, 7, 10, 11, 13, 15, },
  { 0, 2, 4, 6, 7, 10, 11, 13, 15, },
  { 1, 2, 4, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 13, 15, },
  { 3, 4, 6, 7, 10, 11, 13, 15, },
  { 0, 3, 4, 6, 7, 10, 11, 13, 15, },
  { 1, 3, 4, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 13, 15, },
  { 2, 3, 4, 6, 7, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 13, 15, },
  { 5, 6, 7, 10, 11, 13, 15, },
  { 0, 5, 6, 7, 10, 11, 13, 15, },
  { 1, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 5, 6, 7, 10, 11, 13, 15, },
  { 2, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 2, 5, 6, 7, 10, 11, 13, 15, },
  { 1, 2, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 13, 15, },
  { 3, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 3, 5, 6, 7, 10, 11, 13, 15, },
  { 1, 3, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 13, 15, },
  { 2, 3, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 13, 15, },
  { 4, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 1, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 2, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 3, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 13, 15, },
  { 8, 10, 11, 13, 15, },
  { 0, 8, 10, 11, 13, 15, },
  { 1, 8, 10, 11, 13, 15, },
  { 0, 1, 8, 10, 11, 13, 15, },
  { 2, 8, 10, 11, 13, 15, },
  { 0, 2, 8, 10, 11, 13, 15, },
  { 1, 2, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 8, 10, 11, 13, 15, },
  { 3, 8, 10, 11, 13, 15, },
  { 0, 3, 8, 10, 11, 13, 15, },
  { 1, 3, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 8, 10, 11, 13, 15, },
  { 2, 3, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 8, 10, 11, 13, 15, },
  { 4, 8, 10, 11, 13, 15, },
  { 0, 4, 8, 10, 11, 13, 15, },
  { 1, 4, 8, 10, 11, 13, 15, },
  { 0, 1, 4, 8, 10, 11, 13, 15, },
  { 2, 4, 8, 10, 11, 13, 15, },
  { 0, 2, 4, 8, 10, 11, 13, 15, },
  { 1, 2, 4, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 8, 10, 11, 13, 15, },
  { 3, 4, 8, 10, 11, 13, 15, },
  { 0, 3, 4, 8, 10, 11, 13, 15, },
  { 1, 3, 4, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 8, 10, 11, 13, 15, },
  { 2, 3, 4, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 13, 15, },
  { 5, 8, 10, 11, 13, 15, },
  { 0, 5, 8, 10, 11, 13, 15, },
  { 1, 5, 8, 10, 11, 13, 15, },
  { 0, 1, 5, 8, 10, 11, 13, 15, },
  { 2, 5, 8, 10, 11, 13, 15, },
  { 0, 2, 5, 8, 10, 11, 13, 15, },
  { 1, 2, 5, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 8, 10, 11, 13, 15, },
  { 3, 5, 8, 10, 11, 13, 15, },
  { 0, 3, 5, 8, 10, 11, 13, 15, },
  { 1, 3, 5, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 8, 10, 11, 13, 15, },
  { 2, 3, 5, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 13, 15, },
  { 4, 5, 8, 10, 11, 13, 15, },
  { 0, 4, 5, 8, 10, 11, 13, 15, },
  { 1, 4, 5, 8, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 8, 10, 11, 13, 15, },
  { 2, 4, 5, 8, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 8, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 13, 15, },
  { 3, 4, 5, 8, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 8, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 13, 15, },
  { 6, 8, 10, 11, 13, 15, },
  { 0, 6, 8, 10, 11, 13, 15, },
  { 1, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 6, 8, 10, 11, 13, 15, },
  { 2, 6, 8, 10, 11, 13, 15, },
  { 0, 2, 6, 8, 10, 11, 13, 15, },
  { 1, 2, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 6, 8, 10, 11, 13, 15, },
  { 3, 6, 8, 10, 11, 13, 15, },
  { 0, 3, 6, 8, 10, 11, 13, 15, },
  { 1, 3, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 6, 8, 10, 11, 13, 15, },
  { 2, 3, 6, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 6, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 13, 15, },
  { 4, 6, 8, 10, 11, 13, 15, },
  { 0, 4, 6, 8, 10, 11, 13, 15, },
  { 1, 4, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 4, 6, 8, 10, 11, 13, 15, },
  { 2, 4, 6, 8, 10, 11, 13, 15, },
  { 0, 2, 4, 6, 8, 10, 11, 13, 15, },
  { 1, 2, 4, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 13, 15, },
  { 3, 4, 6, 8, 10, 11, 13, 15, },
  { 0, 3, 4, 6, 8, 10, 11, 13, 15, },
  { 1, 3, 4, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 13, 15, },
  { 2, 3, 4, 6, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 13, 15, },
  { 5, 6, 8, 10, 11, 13, 15, },
  { 0, 5, 6, 8, 10, 11, 13, 15, },
  { 1, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 5, 6, 8, 10, 11, 13, 15, },
  { 2, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 2, 5, 6, 8, 10, 11, 13, 15, },
  { 1, 2, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 13, 15, },
  { 3, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 3, 5, 6, 8, 10, 11, 13, 15, },
  { 1, 3, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 13, 15, },
  { 2, 3, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 13, 15, },
  { 4, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 1, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 2, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 3, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 13, 15, },
  { 7, 8, 10, 11, 13, 15, },
  { 0, 7, 8, 10, 11, 13, 15, },
  { 1, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 7, 8, 10, 11, 13, 15, },
  { 2, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 7, 8, 10, 11, 13, 15, },
  { 3, 7, 8, 10, 11, 13, 15, },
  { 0, 3, 7, 8, 10, 11, 13, 15, },
  { 1, 3, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 7, 8, 10, 11, 13, 15, },
  { 2, 3, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 13, 15, },
  { 4, 7, 8, 10, 11, 13, 15, },
  { 0, 4, 7, 8, 10, 11, 13, 15, },
  { 1, 4, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 4, 7, 8, 10, 11, 13, 15, },
  { 2, 4, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 4, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 4, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 13, 15, },
  { 3, 4, 7, 8, 10, 11, 13, 15, },
  { 0, 3, 4, 7, 8, 10, 11, 13, 15, },
  { 1, 3, 4, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 13, 15, },
  { 2, 3, 4, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 13, 15, },
  { 5, 7, 8, 10, 11, 13, 15, },
  { 0, 5, 7, 8, 10, 11, 13, 15, },
  { 1, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 5, 7, 8, 10, 11, 13, 15, },
  { 2, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 5, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 13, 15, },
  { 3, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 3, 5, 7, 8, 10, 11, 13, 15, },
  { 1, 3, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 13, 15, },
  { 2, 3, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 13, 15, },
  { 4, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 1, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 2, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 3, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 13, 15, },
  { 6, 7, 8, 10, 11, 13, 15, },
  { 0, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 6, 7, 8, 10, 11, 13, 15, },
  { 2, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 13, 15, },
  { 3, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 3, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 3, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 13, 15, },
  { 2, 3, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 13, 15, },
  { 4, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 2, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 3, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 13, 15, },
  { 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 2, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 3, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, },
  { 9, 10, 11, 13, 15, },
  { 0, 9, 10, 11, 13, 15, },
  { 1, 9, 10, 11, 13, 15, },
  { 0, 1, 9, 10, 11, 13, 15, },
  { 2, 9, 10, 11, 13, 15, },
  { 0, 2, 9, 10, 11, 13, 15, },
  { 1, 2, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 9, 10, 11, 13, 15, },
  { 3, 9, 10, 11, 13, 15, },
  { 0, 3, 9, 10, 11, 13, 15, },
  { 1, 3, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 9, 10, 11, 13, 15, },
  { 2, 3, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 9, 10, 11, 13, 15, },
  { 4, 9, 10, 11, 13, 15, },
  { 0, 4, 9, 10, 11, 13, 15, },
  { 1, 4, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 9, 10, 11, 13, 15, },
  { 2, 4, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 9, 10, 11, 13, 15, },
  { 3, 4, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 13, 15, },
  { 5, 9, 10, 11, 13, 15, },
  { 0, 5, 9, 10, 11, 13, 15, },
  { 1, 5, 9, 10, 11, 13, 15, },
  { 0, 1, 5, 9, 10, 11, 13, 15, },
  { 2, 5, 9, 10, 11, 13, 15, },
  { 0, 2, 5, 9, 10, 11, 13, 15, },
  { 1, 2, 5, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 9, 10, 11, 13, 15, },
  { 3, 5, 9, 10, 11, 13, 15, },
  { 0, 3, 5, 9, 10, 11, 13, 15, },
  { 1, 3, 5, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 9, 10, 11, 13, 15, },
  { 2, 3, 5, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 13, 15, },
  { 4, 5, 9, 10, 11, 13, 15, },
  { 0, 4, 5, 9, 10, 11, 13, 15, },
  { 1, 4, 5, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 9, 10, 11, 13, 15, },
  { 2, 4, 5, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 13, 15, },
  { 3, 4, 5, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 13, 15, },
  { 6, 9, 10, 11, 13, 15, },
  { 0, 6, 9, 10, 11, 13, 15, },
  { 1, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 6, 9, 10, 11, 13, 15, },
  { 2, 6, 9, 10, 11, 13, 15, },
  { 0, 2, 6, 9, 10, 11, 13, 15, },
  { 1, 2, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 6, 9, 10, 11, 13, 15, },
  { 3, 6, 9, 10, 11, 13, 15, },
  { 0, 3, 6, 9, 10, 11, 13, 15, },
  { 1, 3, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 6, 9, 10, 11, 13, 15, },
  { 2, 3, 6, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 6, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 13, 15, },
  { 4, 6, 9, 10, 11, 13, 15, },
  { 0, 4, 6, 9, 10, 11, 13, 15, },
  { 1, 4, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 6, 9, 10, 11, 13, 15, },
  { 2, 4, 6, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 6, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 13, 15, },
  { 3, 4, 6, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 6, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 6, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 13, 15, },
  { 5, 6, 9, 10, 11, 13, 15, },
  { 0, 5, 6, 9, 10, 11, 13, 15, },
  { 1, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 5, 6, 9, 10, 11, 13, 15, },
  { 2, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 2, 5, 6, 9, 10, 11, 13, 15, },
  { 1, 2, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 13, 15, },
  { 3, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 3, 5, 6, 9, 10, 11, 13, 15, },
  { 1, 3, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 13, 15, },
  { 2, 3, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 13, 15, },
  { 4, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 1, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 2, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 3, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 13, 15, },
  { 7, 9, 10, 11, 13, 15, },
  { 0, 7, 9, 10, 11, 13, 15, },
  { 1, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 7, 9, 10, 11, 13, 15, },
  { 2, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 7, 9, 10, 11, 13, 15, },
  { 3, 7, 9, 10, 11, 13, 15, },
  { 0, 3, 7, 9, 10, 11, 13, 15, },
  { 1, 3, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 7, 9, 10, 11, 13, 15, },
  { 2, 3, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 13, 15, },
  { 4, 7, 9, 10, 11, 13, 15, },
  { 0, 4, 7, 9, 10, 11, 13, 15, },
  { 1, 4, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 7, 9, 10, 11, 13, 15, },
  { 2, 4, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 13, 15, },
  { 3, 4, 7, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 7, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 13, 15, },
  { 5, 7, 9, 10, 11, 13, 15, },
  { 0, 5, 7, 9, 10, 11, 13, 15, },
  { 1, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 5, 7, 9, 10, 11, 13, 15, },
  { 2, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 5, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 13, 15, },
  { 3, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 3, 5, 7, 9, 10, 11, 13, 15, },
  { 1, 3, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 13, 15, },
  { 2, 3, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 13, 15, },
  { 4, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 1, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 2, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 3, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 13, 15, },
  { 6, 7, 9, 10, 11, 13, 15, },
  { 0, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 6, 7, 9, 10, 11, 13, 15, },
  { 2, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 13, 15, },
  { 3, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 3, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 3, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 13, 15, },
  { 2, 3, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 13, 15, },
  { 4, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 2, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 3, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 13, 15, },
  { 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 2, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 3, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, },
  { 8, 9, 10, 11, 13, 15, },
  { 0, 8, 9, 10, 11, 13, 15, },
  { 1, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 8, 9, 10, 11, 13, 15, },
  { 2, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 8, 9, 10, 11, 13, 15, },
  { 3, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 13, 15, },
  { 4, 8, 9, 10, 11, 13, 15, },
  { 0, 4, 8, 9, 10, 11, 13, 15, },
  { 1, 4, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 8, 9, 10, 11, 13, 15, },
  { 2, 4, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 13, 15, },
  { 3, 4, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 13, 15, },
  { 5, 8, 9, 10, 11, 13, 15, },
  { 0, 5, 8, 9, 10, 11, 13, 15, },
  { 1, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 5, 8, 9, 10, 11, 13, 15, },
  { 2, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 5, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 13, 15, },
  { 3, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 5, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 13, 15, },
  { 4, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 1, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 2, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 3, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 13, 15, },
  { 6, 8, 9, 10, 11, 13, 15, },
  { 0, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 6, 8, 9, 10, 11, 13, 15, },
  { 2, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 13, 15, },
  { 3, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 13, 15, },
  { 4, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 2, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 3, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 13, 15, },
  { 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 2, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 3, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 15, },
  { 7, 8, 9, 10, 11, 13, 15, },
  { 0, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 13, 15, },
  { 3, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 13, 15, },
  { 4, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 3, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 13, 15, },
  { 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 3, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 15, },
  { 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 3, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, },
  { 12, 13, 15, },
  { 0, 12, 13, 15, },
  { 1, 12, 13, 15, },
  { 0, 1, 12, 13, 15, },
  { 2, 12, 13, 15, },
  { 0, 2, 12, 13, 15, },
  { 1, 2, 12, 13, 15, },
  { 0, 1, 2, 12, 13, 15, },
  { 3, 12, 13, 15, },
  { 0, 3, 12, 13, 15, },
  { 1, 3, 12, 13, 15, },
  { 0, 1, 3, 12, 13, 15, },
  { 2, 3, 12, 13, 15, },
  { 0, 2, 3, 12, 13, 15, },
  { 1, 2, 3, 12, 13, 15, },
  { 0, 1, 2, 3, 12, 13, 15, },
  { 4, 12, 13, 15, },
  { 0, 4, 12, 13, 15, },
  { 1, 4, 12, 13, 15, },
  { 0, 1, 4, 12, 13, 15, },
  { 2, 4, 12, 13, 15, },
  { 0, 2, 4, 12, 13, 15, },
  { 1, 2, 4, 12, 13, 15, },
  { 0, 1, 2, 4, 12, 13, 15, },
  { 3, 4, 12, 13, 15, },
  { 0, 3, 4, 12, 13, 15, },
  { 1, 3, 4, 12, 13, 15, },
  { 0, 1, 3, 4, 12, 13, 15, },
  { 2, 3, 4, 12, 13, 15, },
  { 0, 2, 3, 4, 12, 13, 15, },
  { 1, 2, 3, 4, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 12, 13, 15, },
  { 5, 12, 13, 15, },
  { 0, 5, 12, 13, 15, },
  { 1, 5, 12, 13, 15, },
  { 0, 1, 5, 12, 13, 15, },
  { 2, 5, 12, 13, 15, },
  { 0, 2, 5, 12, 13, 15, },
  { 1, 2, 5, 12, 13, 15, },
  { 0, 1, 2, 5, 12, 13, 15, },
  { 3, 5, 12, 13, 15, },
  { 0, 3, 5, 12, 13, 15, },
  { 1, 3, 5, 12, 13, 15, },
  { 0, 1, 3, 5, 12, 13, 15, },
  { 2, 3, 5, 12, 13, 15, },
  { 0, 2, 3, 5, 12, 13, 15, },
  { 1, 2, 3, 5, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 12, 13, 15, },
  { 4, 5, 12, 13, 15, },
  { 0, 4, 5, 12, 13, 15, },
  { 1, 4, 5, 12, 13, 15, },
  { 0, 1, 4, 5, 12, 13, 15, },
  { 2, 4, 5, 12, 13, 15, },
  { 0, 2, 4, 5, 12, 13, 15, },
  { 1, 2, 4, 5, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 12, 13, 15, },
  { 3, 4, 5, 12, 13, 15, },
  { 0, 3, 4, 5, 12, 13, 15, },
  { 1, 3, 4, 5, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 12, 13, 15, },
  { 2, 3, 4, 5, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 12, 13, 15, },
  { 6, 12, 13, 15, },
  { 0, 6, 12, 13, 15, },
  { 1, 6, 12, 13, 15, },
  { 0, 1, 6, 12, 13, 15, },
  { 2, 6, 12, 13, 15, },
  { 0, 2, 6, 12, 13, 15, },
  { 1, 2, 6, 12, 13, 15, },
  { 0, 1, 2, 6, 12, 13, 15, },
  { 3, 6, 12, 13, 15, },
  { 0, 3, 6, 12, 13, 15, },
  { 1, 3, 6, 12, 13, 15, },
  { 0, 1, 3, 6, 12, 13, 15, },
  { 2, 3, 6, 12, 13, 15, },
  { 0, 2, 3, 6, 12, 13, 15, },
  { 1, 2, 3, 6, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 12, 13, 15, },
  { 4, 6, 12, 13, 15, },
  { 0, 4, 6, 12, 13, 15, },
  { 1, 4, 6, 12, 13, 15, },
  { 0, 1, 4, 6, 12, 13, 15, },
  { 2, 4, 6, 12, 13, 15, },
  { 0, 2, 4, 6, 12, 13, 15, },
  { 1, 2, 4, 6, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 12, 13, 15, },
  { 3, 4, 6, 12, 13, 15, },
  { 0, 3, 4, 6, 12, 13, 15, },
  { 1, 3, 4, 6, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 12, 13, 15, },
  { 2, 3, 4, 6, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 12, 13, 15, },
  { 5, 6, 12, 13, 15, },
  { 0, 5, 6, 12, 13, 15, },
  { 1, 5, 6, 12, 13, 15, },
  { 0, 1, 5, 6, 12, 13, 15, },
  { 2, 5, 6, 12, 13, 15, },
  { 0, 2, 5, 6, 12, 13, 15, },
  { 1, 2, 5, 6, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 12, 13, 15, },
  { 3, 5, 6, 12, 13, 15, },
  { 0, 3, 5, 6, 12, 13, 15, },
  { 1, 3, 5, 6, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 12, 13, 15, },
  { 2, 3, 5, 6, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 12, 13, 15, },
  { 4, 5, 6, 12, 13, 15, },
  { 0, 4, 5, 6, 12, 13, 15, },
  { 1, 4, 5, 6, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 12, 13, 15, },
  { 2, 4, 5, 6, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 12, 13, 15, },
  { 3, 4, 5, 6, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 12, 13, 15, },
  { 7, 12, 13, 15, },
  { 0, 7, 12, 13, 15, },
  { 1, 7, 12, 13, 15, },
  { 0, 1, 7, 12, 13, 15, },
  { 2, 7, 12, 13, 15, },
  { 0, 2, 7, 12, 13, 15, },
  { 1, 2, 7, 12, 13, 15, },
  { 0, 1, 2, 7, 12, 13, 15, },
  { 3, 7, 12, 13, 15, },
  { 0, 3, 7, 12, 13, 15, },
  { 1, 3, 7, 12, 13, 15, },
  { 0, 1, 3, 7, 12, 13, 15, },
  { 2, 3, 7, 12, 13, 15, },
  { 0, 2, 3, 7, 12, 13, 15, },
  { 1, 2, 3, 7, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 12, 13, 15, },
  { 4, 7, 12, 13, 15, },
  { 0, 4, 7, 12, 13, 15, },
  { 1, 4, 7, 12, 13, 15, },
  { 0, 1, 4, 7, 12, 13, 15, },
  { 2, 4, 7, 12, 13, 15, },
  { 0, 2, 4, 7, 12, 13, 15, },
  { 1, 2, 4, 7, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 12, 13, 15, },
  { 3, 4, 7, 12, 13, 15, },
  { 0, 3, 4, 7, 12, 13, 15, },
  { 1, 3, 4, 7, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 12, 13, 15, },
  { 2, 3, 4, 7, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 12, 13, 15, },
  { 5, 7, 12, 13, 15, },
  { 0, 5, 7, 12, 13, 15, },
  { 1, 5, 7, 12, 13, 15, },
  { 0, 1, 5, 7, 12, 13, 15, },
  { 2, 5, 7, 12, 13, 15, },
  { 0, 2, 5, 7, 12, 13, 15, },
  { 1, 2, 5, 7, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 12, 13, 15, },
  { 3, 5, 7, 12, 13, 15, },
  { 0, 3, 5, 7, 12, 13, 15, },
  { 1, 3, 5, 7, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 12, 13, 15, },
  { 2, 3, 5, 7, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 12, 13, 15, },
  { 4, 5, 7, 12, 13, 15, },
  { 0, 4, 5, 7, 12, 13, 15, },
  { 1, 4, 5, 7, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 12, 13, 15, },
  { 2, 4, 5, 7, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 12, 13, 15, },
  { 3, 4, 5, 7, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 12, 13, 15, },
  { 6, 7, 12, 13, 15, },
  { 0, 6, 7, 12, 13, 15, },
  { 1, 6, 7, 12, 13, 15, },
  { 0, 1, 6, 7, 12, 13, 15, },
  { 2, 6, 7, 12, 13, 15, },
  { 0, 2, 6, 7, 12, 13, 15, },
  { 1, 2, 6, 7, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 12, 13, 15, },
  { 3, 6, 7, 12, 13, 15, },
  { 0, 3, 6, 7, 12, 13, 15, },
  { 1, 3, 6, 7, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 12, 13, 15, },
  { 2, 3, 6, 7, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 12, 13, 15, },
  { 4, 6, 7, 12, 13, 15, },
  { 0, 4, 6, 7, 12, 13, 15, },
  { 1, 4, 6, 7, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 12, 13, 15, },
  { 2, 4, 6, 7, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 12, 13, 15, },
  { 3, 4, 6, 7, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 12, 13, 15, },
  { 5, 6, 7, 12, 13, 15, },
  { 0, 5, 6, 7, 12, 13, 15, },
  { 1, 5, 6, 7, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 12, 13, 15, },
  { 2, 5, 6, 7, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 12, 13, 15, },
  { 3, 5, 6, 7, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 12, 13, 15, },
  { 4, 5, 6, 7, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 12, 13, 15, },
  { 8, 12, 13, 15, },
  { 0, 8, 12, 13, 15, },
  { 1, 8, 12, 13, 15, },
  { 0, 1, 8, 12, 13, 15, },
  { 2, 8, 12, 13, 15, },
  { 0, 2, 8, 12, 13, 15, },
  { 1, 2, 8, 12, 13, 15, },
  { 0, 1, 2, 8, 12, 13, 15, },
  { 3, 8, 12, 13, 15, },
  { 0, 3, 8, 12, 13, 15, },
  { 1, 3, 8, 12, 13, 15, },
  { 0, 1, 3, 8, 12, 13, 15, },
  { 2, 3, 8, 12, 13, 15, },
  { 0, 2, 3, 8, 12, 13, 15, },
  { 1, 2, 3, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 8, 12, 13, 15, },
  { 4, 8, 12, 13, 15, },
  { 0, 4, 8, 12, 13, 15, },
  { 1, 4, 8, 12, 13, 15, },
  { 0, 1, 4, 8, 12, 13, 15, },
  { 2, 4, 8, 12, 13, 15, },
  { 0, 2, 4, 8, 12, 13, 15, },
  { 1, 2, 4, 8, 12, 13, 15, },
  { 0, 1, 2, 4, 8, 12, 13, 15, },
  { 3, 4, 8, 12, 13, 15, },
  { 0, 3, 4, 8, 12, 13, 15, },
  { 1, 3, 4, 8, 12, 13, 15, },
  { 0, 1, 3, 4, 8, 12, 13, 15, },
  { 2, 3, 4, 8, 12, 13, 15, },
  { 0, 2, 3, 4, 8, 12, 13, 15, },
  { 1, 2, 3, 4, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 12, 13, 15, },
  { 5, 8, 12, 13, 15, },
  { 0, 5, 8, 12, 13, 15, },
  { 1, 5, 8, 12, 13, 15, },
  { 0, 1, 5, 8, 12, 13, 15, },
  { 2, 5, 8, 12, 13, 15, },
  { 0, 2, 5, 8, 12, 13, 15, },
  { 1, 2, 5, 8, 12, 13, 15, },
  { 0, 1, 2, 5, 8, 12, 13, 15, },
  { 3, 5, 8, 12, 13, 15, },
  { 0, 3, 5, 8, 12, 13, 15, },
  { 1, 3, 5, 8, 12, 13, 15, },
  { 0, 1, 3, 5, 8, 12, 13, 15, },
  { 2, 3, 5, 8, 12, 13, 15, },
  { 0, 2, 3, 5, 8, 12, 13, 15, },
  { 1, 2, 3, 5, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 12, 13, 15, },
  { 4, 5, 8, 12, 13, 15, },
  { 0, 4, 5, 8, 12, 13, 15, },
  { 1, 4, 5, 8, 12, 13, 15, },
  { 0, 1, 4, 5, 8, 12, 13, 15, },
  { 2, 4, 5, 8, 12, 13, 15, },
  { 0, 2, 4, 5, 8, 12, 13, 15, },
  { 1, 2, 4, 5, 8, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 12, 13, 15, },
  { 3, 4, 5, 8, 12, 13, 15, },
  { 0, 3, 4, 5, 8, 12, 13, 15, },
  { 1, 3, 4, 5, 8, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 12, 13, 15, },
  { 2, 3, 4, 5, 8, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 12, 13, 15, },
  { 6, 8, 12, 13, 15, },
  { 0, 6, 8, 12, 13, 15, },
  { 1, 6, 8, 12, 13, 15, },
  { 0, 1, 6, 8, 12, 13, 15, },
  { 2, 6, 8, 12, 13, 15, },
  { 0, 2, 6, 8, 12, 13, 15, },
  { 1, 2, 6, 8, 12, 13, 15, },
  { 0, 1, 2, 6, 8, 12, 13, 15, },
  { 3, 6, 8, 12, 13, 15, },
  { 0, 3, 6, 8, 12, 13, 15, },
  { 1, 3, 6, 8, 12, 13, 15, },
  { 0, 1, 3, 6, 8, 12, 13, 15, },
  { 2, 3, 6, 8, 12, 13, 15, },
  { 0, 2, 3, 6, 8, 12, 13, 15, },
  { 1, 2, 3, 6, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 12, 13, 15, },
  { 4, 6, 8, 12, 13, 15, },
  { 0, 4, 6, 8, 12, 13, 15, },
  { 1, 4, 6, 8, 12, 13, 15, },
  { 0, 1, 4, 6, 8, 12, 13, 15, },
  { 2, 4, 6, 8, 12, 13, 15, },
  { 0, 2, 4, 6, 8, 12, 13, 15, },
  { 1, 2, 4, 6, 8, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 12, 13, 15, },
  { 3, 4, 6, 8, 12, 13, 15, },
  { 0, 3, 4, 6, 8, 12, 13, 15, },
  { 1, 3, 4, 6, 8, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 12, 13, 15, },
  { 2, 3, 4, 6, 8, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 12, 13, 15, },
  { 5, 6, 8, 12, 13, 15, },
  { 0, 5, 6, 8, 12, 13, 15, },
  { 1, 5, 6, 8, 12, 13, 15, },
  { 0, 1, 5, 6, 8, 12, 13, 15, },
  { 2, 5, 6, 8, 12, 13, 15, },
  { 0, 2, 5, 6, 8, 12, 13, 15, },
  { 1, 2, 5, 6, 8, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 12, 13, 15, },
  { 3, 5, 6, 8, 12, 13, 15, },
  { 0, 3, 5, 6, 8, 12, 13, 15, },
  { 1, 3, 5, 6, 8, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 12, 13, 15, },
  { 2, 3, 5, 6, 8, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 12, 13, 15, },
  { 4, 5, 6, 8, 12, 13, 15, },
  { 0, 4, 5, 6, 8, 12, 13, 15, },
  { 1, 4, 5, 6, 8, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 12, 13, 15, },
  { 2, 4, 5, 6, 8, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 12, 13, 15, },
  { 3, 4, 5, 6, 8, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 12, 13, 15, },
  { 7, 8, 12, 13, 15, },
  { 0, 7, 8, 12, 13, 15, },
  { 1, 7, 8, 12, 13, 15, },
  { 0, 1, 7, 8, 12, 13, 15, },
  { 2, 7, 8, 12, 13, 15, },
  { 0, 2, 7, 8, 12, 13, 15, },
  { 1, 2, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 7, 8, 12, 13, 15, },
  { 3, 7, 8, 12, 13, 15, },
  { 0, 3, 7, 8, 12, 13, 15, },
  { 1, 3, 7, 8, 12, 13, 15, },
  { 0, 1, 3, 7, 8, 12, 13, 15, },
  { 2, 3, 7, 8, 12, 13, 15, },
  { 0, 2, 3, 7, 8, 12, 13, 15, },
  { 1, 2, 3, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 12, 13, 15, },
  { 4, 7, 8, 12, 13, 15, },
  { 0, 4, 7, 8, 12, 13, 15, },
  { 1, 4, 7, 8, 12, 13, 15, },
  { 0, 1, 4, 7, 8, 12, 13, 15, },
  { 2, 4, 7, 8, 12, 13, 15, },
  { 0, 2, 4, 7, 8, 12, 13, 15, },
  { 1, 2, 4, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 12, 13, 15, },
  { 3, 4, 7, 8, 12, 13, 15, },
  { 0, 3, 4, 7, 8, 12, 13, 15, },
  { 1, 3, 4, 7, 8, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 12, 13, 15, },
  { 2, 3, 4, 7, 8, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 12, 13, 15, },
  { 5, 7, 8, 12, 13, 15, },
  { 0, 5, 7, 8, 12, 13, 15, },
  { 1, 5, 7, 8, 12, 13, 15, },
  { 0, 1, 5, 7, 8, 12, 13, 15, },
  { 2, 5, 7, 8, 12, 13, 15, },
  { 0, 2, 5, 7, 8, 12, 13, 15, },
  { 1, 2, 5, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 12, 13, 15, },
  { 3, 5, 7, 8, 12, 13, 15, },
  { 0, 3, 5, 7, 8, 12, 13, 15, },
  { 1, 3, 5, 7, 8, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 12, 13, 15, },
  { 2, 3, 5, 7, 8, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 12, 13, 15, },
  { 4, 5, 7, 8, 12, 13, 15, },
  { 0, 4, 5, 7, 8, 12, 13, 15, },
  { 1, 4, 5, 7, 8, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 12, 13, 15, },
  { 2, 4, 5, 7, 8, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 12, 13, 15, },
  { 3, 4, 5, 7, 8, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 12, 13, 15, },
  { 6, 7, 8, 12, 13, 15, },
  { 0, 6, 7, 8, 12, 13, 15, },
  { 1, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 6, 7, 8, 12, 13, 15, },
  { 2, 6, 7, 8, 12, 13, 15, },
  { 0, 2, 6, 7, 8, 12, 13, 15, },
  { 1, 2, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 12, 13, 15, },
  { 3, 6, 7, 8, 12, 13, 15, },
  { 0, 3, 6, 7, 8, 12, 13, 15, },
  { 1, 3, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 12, 13, 15, },
  { 2, 3, 6, 7, 8, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 12, 13, 15, },
  { 4, 6, 7, 8, 12, 13, 15, },
  { 0, 4, 6, 7, 8, 12, 13, 15, },
  { 1, 4, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 12, 13, 15, },
  { 2, 4, 6, 7, 8, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 12, 13, 15, },
  { 3, 4, 6, 7, 8, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 12, 13, 15, },
  { 5, 6, 7, 8, 12, 13, 15, },
  { 0, 5, 6, 7, 8, 12, 13, 15, },
  { 1, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 12, 13, 15, },
  { 2, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 12, 13, 15, },
  { 3, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 12, 13, 15, },
  { 4, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 12, 13, 15, },
  { 9, 12, 13, 15, },
  { 0, 9, 12, 13, 15, },
  { 1, 9, 12, 13, 15, },
  { 0, 1, 9, 12, 13, 15, },
  { 2, 9, 12, 13, 15, },
  { 0, 2, 9, 12, 13, 15, },
  { 1, 2, 9, 12, 13, 15, },
  { 0, 1, 2, 9, 12, 13, 15, },
  { 3, 9, 12, 13, 15, },
  { 0, 3, 9, 12, 13, 15, },
  { 1, 3, 9, 12, 13, 15, },
  { 0, 1, 3, 9, 12, 13, 15, },
  { 2, 3, 9, 12, 13, 15, },
  { 0, 2, 3, 9, 12, 13, 15, },
  { 1, 2, 3, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 9, 12, 13, 15, },
  { 4, 9, 12, 13, 15, },
  { 0, 4, 9, 12, 13, 15, },
  { 1, 4, 9, 12, 13, 15, },
  { 0, 1, 4, 9, 12, 13, 15, },
  { 2, 4, 9, 12, 13, 15, },
  { 0, 2, 4, 9, 12, 13, 15, },
  { 1, 2, 4, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 9, 12, 13, 15, },
  { 3, 4, 9, 12, 13, 15, },
  { 0, 3, 4, 9, 12, 13, 15, },
  { 1, 3, 4, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 9, 12, 13, 15, },
  { 2, 3, 4, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 9, 12, 13, 15, },
  { 5, 9, 12, 13, 15, },
  { 0, 5, 9, 12, 13, 15, },
  { 1, 5, 9, 12, 13, 15, },
  { 0, 1, 5, 9, 12, 13, 15, },
  { 2, 5, 9, 12, 13, 15, },
  { 0, 2, 5, 9, 12, 13, 15, },
  { 1, 2, 5, 9, 12, 13, 15, },
  { 0, 1, 2, 5, 9, 12, 13, 15, },
  { 3, 5, 9, 12, 13, 15, },
  { 0, 3, 5, 9, 12, 13, 15, },
  { 1, 3, 5, 9, 12, 13, 15, },
  { 0, 1, 3, 5, 9, 12, 13, 15, },
  { 2, 3, 5, 9, 12, 13, 15, },
  { 0, 2, 3, 5, 9, 12, 13, 15, },
  { 1, 2, 3, 5, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 9, 12, 13, 15, },
  { 4, 5, 9, 12, 13, 15, },
  { 0, 4, 5, 9, 12, 13, 15, },
  { 1, 4, 5, 9, 12, 13, 15, },
  { 0, 1, 4, 5, 9, 12, 13, 15, },
  { 2, 4, 5, 9, 12, 13, 15, },
  { 0, 2, 4, 5, 9, 12, 13, 15, },
  { 1, 2, 4, 5, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 9, 12, 13, 15, },
  { 3, 4, 5, 9, 12, 13, 15, },
  { 0, 3, 4, 5, 9, 12, 13, 15, },
  { 1, 3, 4, 5, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 9, 12, 13, 15, },
  { 2, 3, 4, 5, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 12, 13, 15, },
  { 6, 9, 12, 13, 15, },
  { 0, 6, 9, 12, 13, 15, },
  { 1, 6, 9, 12, 13, 15, },
  { 0, 1, 6, 9, 12, 13, 15, },
  { 2, 6, 9, 12, 13, 15, },
  { 0, 2, 6, 9, 12, 13, 15, },
  { 1, 2, 6, 9, 12, 13, 15, },
  { 0, 1, 2, 6, 9, 12, 13, 15, },
  { 3, 6, 9, 12, 13, 15, },
  { 0, 3, 6, 9, 12, 13, 15, },
  { 1, 3, 6, 9, 12, 13, 15, },
  { 0, 1, 3, 6, 9, 12, 13, 15, },
  { 2, 3, 6, 9, 12, 13, 15, },
  { 0, 2, 3, 6, 9, 12, 13, 15, },
  { 1, 2, 3, 6, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 9, 12, 13, 15, },
  { 4, 6, 9, 12, 13, 15, },
  { 0, 4, 6, 9, 12, 13, 15, },
  { 1, 4, 6, 9, 12, 13, 15, },
  { 0, 1, 4, 6, 9, 12, 13, 15, },
  { 2, 4, 6, 9, 12, 13, 15, },
  { 0, 2, 4, 6, 9, 12, 13, 15, },
  { 1, 2, 4, 6, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 9, 12, 13, 15, },
  { 3, 4, 6, 9, 12, 13, 15, },
  { 0, 3, 4, 6, 9, 12, 13, 15, },
  { 1, 3, 4, 6, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 9, 12, 13, 15, },
  { 2, 3, 4, 6, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 12, 13, 15, },
  { 5, 6, 9, 12, 13, 15, },
  { 0, 5, 6, 9, 12, 13, 15, },
  { 1, 5, 6, 9, 12, 13, 15, },
  { 0, 1, 5, 6, 9, 12, 13, 15, },
  { 2, 5, 6, 9, 12, 13, 15, },
  { 0, 2, 5, 6, 9, 12, 13, 15, },
  { 1, 2, 5, 6, 9, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 9, 12, 13, 15, },
  { 3, 5, 6, 9, 12, 13, 15, },
  { 0, 3, 5, 6, 9, 12, 13, 15, },
  { 1, 3, 5, 6, 9, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 9, 12, 13, 15, },
  { 2, 3, 5, 6, 9, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 9, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 12, 13, 15, },
  { 4, 5, 6, 9, 12, 13, 15, },
  { 0, 4, 5, 6, 9, 12, 13, 15, },
  { 1, 4, 5, 6, 9, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 9, 12, 13, 15, },
  { 2, 4, 5, 6, 9, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 9, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 12, 13, 15, },
  { 3, 4, 5, 6, 9, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 9, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 12, 13, 15, },
  { 7, 9, 12, 13, 15, },
  { 0, 7, 9, 12, 13, 15, },
  { 1, 7, 9, 12, 13, 15, },
  { 0, 1, 7, 9, 12, 13, 15, },
  { 2, 7, 9, 12, 13, 15, },
  { 0, 2, 7, 9, 12, 13, 15, },
  { 1, 2, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 7, 9, 12, 13, 15, },
  { 3, 7, 9, 12, 13, 15, },
  { 0, 3, 7, 9, 12, 13, 15, },
  { 1, 3, 7, 9, 12, 13, 15, },
  { 0, 1, 3, 7, 9, 12, 13, 15, },
  { 2, 3, 7, 9, 12, 13, 15, },
  { 0, 2, 3, 7, 9, 12, 13, 15, },
  { 1, 2, 3, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 9, 12, 13, 15, },
  { 4, 7, 9, 12, 13, 15, },
  { 0, 4, 7, 9, 12, 13, 15, },
  { 1, 4, 7, 9, 12, 13, 15, },
  { 0, 1, 4, 7, 9, 12, 13, 15, },
  { 2, 4, 7, 9, 12, 13, 15, },
  { 0, 2, 4, 7, 9, 12, 13, 15, },
  { 1, 2, 4, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 9, 12, 13, 15, },
  { 3, 4, 7, 9, 12, 13, 15, },
  { 0, 3, 4, 7, 9, 12, 13, 15, },
  { 1, 3, 4, 7, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 9, 12, 13, 15, },
  { 2, 3, 4, 7, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 12, 13, 15, },
  { 5, 7, 9, 12, 13, 15, },
  { 0, 5, 7, 9, 12, 13, 15, },
  { 1, 5, 7, 9, 12, 13, 15, },
  { 0, 1, 5, 7, 9, 12, 13, 15, },
  { 2, 5, 7, 9, 12, 13, 15, },
  { 0, 2, 5, 7, 9, 12, 13, 15, },
  { 1, 2, 5, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 9, 12, 13, 15, },
  { 3, 5, 7, 9, 12, 13, 15, },
  { 0, 3, 5, 7, 9, 12, 13, 15, },
  { 1, 3, 5, 7, 9, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 9, 12, 13, 15, },
  { 2, 3, 5, 7, 9, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 9, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 12, 13, 15, },
  { 4, 5, 7, 9, 12, 13, 15, },
  { 0, 4, 5, 7, 9, 12, 13, 15, },
  { 1, 4, 5, 7, 9, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 9, 12, 13, 15, },
  { 2, 4, 5, 7, 9, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 9, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 12, 13, 15, },
  { 3, 4, 5, 7, 9, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 9, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 12, 13, 15, },
  { 6, 7, 9, 12, 13, 15, },
  { 0, 6, 7, 9, 12, 13, 15, },
  { 1, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 6, 7, 9, 12, 13, 15, },
  { 2, 6, 7, 9, 12, 13, 15, },
  { 0, 2, 6, 7, 9, 12, 13, 15, },
  { 1, 2, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 9, 12, 13, 15, },
  { 3, 6, 7, 9, 12, 13, 15, },
  { 0, 3, 6, 7, 9, 12, 13, 15, },
  { 1, 3, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 9, 12, 13, 15, },
  { 2, 3, 6, 7, 9, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 9, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 12, 13, 15, },
  { 4, 6, 7, 9, 12, 13, 15, },
  { 0, 4, 6, 7, 9, 12, 13, 15, },
  { 1, 4, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 9, 12, 13, 15, },
  { 2, 4, 6, 7, 9, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 9, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 12, 13, 15, },
  { 3, 4, 6, 7, 9, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 9, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 12, 13, 15, },
  { 5, 6, 7, 9, 12, 13, 15, },
  { 0, 5, 6, 7, 9, 12, 13, 15, },
  { 1, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 9, 12, 13, 15, },
  { 2, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 9, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 12, 13, 15, },
  { 3, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 9, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 12, 13, 15, },
  { 4, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 12, 13, 15, },
  { 8, 9, 12, 13, 15, },
  { 0, 8, 9, 12, 13, 15, },
  { 1, 8, 9, 12, 13, 15, },
  { 0, 1, 8, 9, 12, 13, 15, },
  { 2, 8, 9, 12, 13, 15, },
  { 0, 2, 8, 9, 12, 13, 15, },
  { 1, 2, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 8, 9, 12, 13, 15, },
  { 3, 8, 9, 12, 13, 15, },
  { 0, 3, 8, 9, 12, 13, 15, },
  { 1, 3, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 8, 9, 12, 13, 15, },
  { 2, 3, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 8, 9, 12, 13, 15, },
  { 4, 8, 9, 12, 13, 15, },
  { 0, 4, 8, 9, 12, 13, 15, },
  { 1, 4, 8, 9, 12, 13, 15, },
  { 0, 1, 4, 8, 9, 12, 13, 15, },
  { 2, 4, 8, 9, 12, 13, 15, },
  { 0, 2, 4, 8, 9, 12, 13, 15, },
  { 1, 2, 4, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 8, 9, 12, 13, 15, },
  { 3, 4, 8, 9, 12, 13, 15, },
  { 0, 3, 4, 8, 9, 12, 13, 15, },
  { 1, 3, 4, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 8, 9, 12, 13, 15, },
  { 2, 3, 4, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 12, 13, 15, },
  { 5, 8, 9, 12, 13, 15, },
  { 0, 5, 8, 9, 12, 13, 15, },
  { 1, 5, 8, 9, 12, 13, 15, },
  { 0, 1, 5, 8, 9, 12, 13, 15, },
  { 2, 5, 8, 9, 12, 13, 15, },
  { 0, 2, 5, 8, 9, 12, 13, 15, },
  { 1, 2, 5, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 5, 8, 9, 12, 13, 15, },
  { 3, 5, 8, 9, 12, 13, 15, },
  { 0, 3, 5, 8, 9, 12, 13, 15, },
  { 1, 3, 5, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 5, 8, 9, 12, 13, 15, },
  { 2, 3, 5, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 5, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 5, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 12, 13, 15, },
  { 4, 5, 8, 9, 12, 13, 15, },
  { 0, 4, 5, 8, 9, 12, 13, 15, },
  { 1, 4, 5, 8, 9, 12, 13, 15, },
  { 0, 1, 4, 5, 8, 9, 12, 13, 15, },
  { 2, 4, 5, 8, 9, 12, 13, 15, },
  { 0, 2, 4, 5, 8, 9, 12, 13, 15, },
  { 1, 2, 4, 5, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 12, 13, 15, },
  { 3, 4, 5, 8, 9, 12, 13, 15, },
  { 0, 3, 4, 5, 8, 9, 12, 13, 15, },
  { 1, 3, 4, 5, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 12, 13, 15, },
  { 2, 3, 4, 5, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 12, 13, 15, },
  { 6, 8, 9, 12, 13, 15, },
  { 0, 6, 8, 9, 12, 13, 15, },
  { 1, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 6, 8, 9, 12, 13, 15, },
  { 2, 6, 8, 9, 12, 13, 15, },
  { 0, 2, 6, 8, 9, 12, 13, 15, },
  { 1, 2, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 6, 8, 9, 12, 13, 15, },
  { 3, 6, 8, 9, 12, 13, 15, },
  { 0, 3, 6, 8, 9, 12, 13, 15, },
  { 1, 3, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 6, 8, 9, 12, 13, 15, },
  { 2, 3, 6, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 6, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 12, 13, 15, },
  { 4, 6, 8, 9, 12, 13, 15, },
  { 0, 4, 6, 8, 9, 12, 13, 15, },
  { 1, 4, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 4, 6, 8, 9, 12, 13, 15, },
  { 2, 4, 6, 8, 9, 12, 13, 15, },
  { 0, 2, 4, 6, 8, 9, 12, 13, 15, },
  { 1, 2, 4, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 12, 13, 15, },
  { 3, 4, 6, 8, 9, 12, 13, 15, },
  { 0, 3, 4, 6, 8, 9, 12, 13, 15, },
  { 1, 3, 4, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 12, 13, 15, },
  { 2, 3, 4, 6, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 12, 13, 15, },
  { 5, 6, 8, 9, 12, 13, 15, },
  { 0, 5, 6, 8, 9, 12, 13, 15, },
  { 1, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 5, 6, 8, 9, 12, 13, 15, },
  { 2, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 2, 5, 6, 8, 9, 12, 13, 15, },
  { 1, 2, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 12, 13, 15, },
  { 3, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 3, 5, 6, 8, 9, 12, 13, 15, },
  { 1, 3, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 12, 13, 15, },
  { 2, 3, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 12, 13, 15, },
  { 4, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 1, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 2, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 3, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 12, 13, 15, },
  { 7, 8, 9, 12, 13, 15, },
  { 0, 7, 8, 9, 12, 13, 15, },
  { 1, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 7, 8, 9, 12, 13, 15, },
  { 2, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 7, 8, 9, 12, 13, 15, },
  { 3, 7, 8, 9, 12, 13, 15, },
  { 0, 3, 7, 8, 9, 12, 13, 15, },
  { 1, 3, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 7, 8, 9, 12, 13, 15, },
  { 2, 3, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 12, 13, 15, },
  { 4, 7, 8, 9, 12, 13, 15, },
  { 0, 4, 7, 8, 9, 12, 13, 15, },
  { 1, 4, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 4, 7, 8, 9, 12, 13, 15, },
  { 2, 4, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 4, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 4, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 12, 13, 15, },
  { 3, 4, 7, 8, 9, 12, 13, 15, },
  { 0, 3, 4, 7, 8, 9, 12, 13, 15, },
  { 1, 3, 4, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 12, 13, 15, },
  { 2, 3, 4, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 12, 13, 15, },
  { 5, 7, 8, 9, 12, 13, 15, },
  { 0, 5, 7, 8, 9, 12, 13, 15, },
  { 1, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 5, 7, 8, 9, 12, 13, 15, },
  { 2, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 5, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 12, 13, 15, },
  { 3, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 3, 5, 7, 8, 9, 12, 13, 15, },
  { 1, 3, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 12, 13, 15, },
  { 2, 3, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 12, 13, 15, },
  { 4, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 1, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 2, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 3, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 12, 13, 15, },
  { 6, 7, 8, 9, 12, 13, 15, },
  { 0, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 6, 7, 8, 9, 12, 13, 15, },
  { 2, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 12, 13, 15, },
  { 3, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 3, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 3, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 12, 13, 15, },
  { 2, 3, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 12, 13, 15, },
  { 4, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 2, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 3, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 12, 13, 15, },
  { 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 2, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 3, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, },
  { 10, 12, 13, 15, },
  { 0, 10, 12, 13, 15, },
  { 1, 10, 12, 13, 15, },
  { 0, 1, 10, 12, 13, 15, },
  { 2, 10, 12, 13, 15, },
  { 0, 2, 10, 12, 13, 15, },
  { 1, 2, 10, 12, 13, 15, },
  { 0, 1, 2, 10, 12, 13, 15, },
  { 3, 10, 12, 13, 15, },
  { 0, 3, 10, 12, 13, 15, },
  { 1, 3, 10, 12, 13, 15, },
  { 0, 1, 3, 10, 12, 13, 15, },
  { 2, 3, 10, 12, 13, 15, },
  { 0, 2, 3, 10, 12, 13, 15, },
  { 1, 2, 3, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 10, 12, 13, 15, },
  { 4, 10, 12, 13, 15, },
  { 0, 4, 10, 12, 13, 15, },
  { 1, 4, 10, 12, 13, 15, },
  { 0, 1, 4, 10, 12, 13, 15, },
  { 2, 4, 10, 12, 13, 15, },
  { 0, 2, 4, 10, 12, 13, 15, },
  { 1, 2, 4, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 10, 12, 13, 15, },
  { 3, 4, 10, 12, 13, 15, },
  { 0, 3, 4, 10, 12, 13, 15, },
  { 1, 3, 4, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 10, 12, 13, 15, },
  { 2, 3, 4, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 10, 12, 13, 15, },
  { 5, 10, 12, 13, 15, },
  { 0, 5, 10, 12, 13, 15, },
  { 1, 5, 10, 12, 13, 15, },
  { 0, 1, 5, 10, 12, 13, 15, },
  { 2, 5, 10, 12, 13, 15, },
  { 0, 2, 5, 10, 12, 13, 15, },
  { 1, 2, 5, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 10, 12, 13, 15, },
  { 3, 5, 10, 12, 13, 15, },
  { 0, 3, 5, 10, 12, 13, 15, },
  { 1, 3, 5, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 10, 12, 13, 15, },
  { 2, 3, 5, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 10, 12, 13, 15, },
  { 4, 5, 10, 12, 13, 15, },
  { 0, 4, 5, 10, 12, 13, 15, },
  { 1, 4, 5, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 10, 12, 13, 15, },
  { 2, 4, 5, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 10, 12, 13, 15, },
  { 3, 4, 5, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 12, 13, 15, },
  { 6, 10, 12, 13, 15, },
  { 0, 6, 10, 12, 13, 15, },
  { 1, 6, 10, 12, 13, 15, },
  { 0, 1, 6, 10, 12, 13, 15, },
  { 2, 6, 10, 12, 13, 15, },
  { 0, 2, 6, 10, 12, 13, 15, },
  { 1, 2, 6, 10, 12, 13, 15, },
  { 0, 1, 2, 6, 10, 12, 13, 15, },
  { 3, 6, 10, 12, 13, 15, },
  { 0, 3, 6, 10, 12, 13, 15, },
  { 1, 3, 6, 10, 12, 13, 15, },
  { 0, 1, 3, 6, 10, 12, 13, 15, },
  { 2, 3, 6, 10, 12, 13, 15, },
  { 0, 2, 3, 6, 10, 12, 13, 15, },
  { 1, 2, 3, 6, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 10, 12, 13, 15, },
  { 4, 6, 10, 12, 13, 15, },
  { 0, 4, 6, 10, 12, 13, 15, },
  { 1, 4, 6, 10, 12, 13, 15, },
  { 0, 1, 4, 6, 10, 12, 13, 15, },
  { 2, 4, 6, 10, 12, 13, 15, },
  { 0, 2, 4, 6, 10, 12, 13, 15, },
  { 1, 2, 4, 6, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 10, 12, 13, 15, },
  { 3, 4, 6, 10, 12, 13, 15, },
  { 0, 3, 4, 6, 10, 12, 13, 15, },
  { 1, 3, 4, 6, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 10, 12, 13, 15, },
  { 2, 3, 4, 6, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 12, 13, 15, },
  { 5, 6, 10, 12, 13, 15, },
  { 0, 5, 6, 10, 12, 13, 15, },
  { 1, 5, 6, 10, 12, 13, 15, },
  { 0, 1, 5, 6, 10, 12, 13, 15, },
  { 2, 5, 6, 10, 12, 13, 15, },
  { 0, 2, 5, 6, 10, 12, 13, 15, },
  { 1, 2, 5, 6, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 10, 12, 13, 15, },
  { 3, 5, 6, 10, 12, 13, 15, },
  { 0, 3, 5, 6, 10, 12, 13, 15, },
  { 1, 3, 5, 6, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 10, 12, 13, 15, },
  { 2, 3, 5, 6, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 12, 13, 15, },
  { 4, 5, 6, 10, 12, 13, 15, },
  { 0, 4, 5, 6, 10, 12, 13, 15, },
  { 1, 4, 5, 6, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 10, 12, 13, 15, },
  { 2, 4, 5, 6, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 12, 13, 15, },
  { 3, 4, 5, 6, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 12, 13, 15, },
  { 7, 10, 12, 13, 15, },
  { 0, 7, 10, 12, 13, 15, },
  { 1, 7, 10, 12, 13, 15, },
  { 0, 1, 7, 10, 12, 13, 15, },
  { 2, 7, 10, 12, 13, 15, },
  { 0, 2, 7, 10, 12, 13, 15, },
  { 1, 2, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 7, 10, 12, 13, 15, },
  { 3, 7, 10, 12, 13, 15, },
  { 0, 3, 7, 10, 12, 13, 15, },
  { 1, 3, 7, 10, 12, 13, 15, },
  { 0, 1, 3, 7, 10, 12, 13, 15, },
  { 2, 3, 7, 10, 12, 13, 15, },
  { 0, 2, 3, 7, 10, 12, 13, 15, },
  { 1, 2, 3, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 10, 12, 13, 15, },
  { 4, 7, 10, 12, 13, 15, },
  { 0, 4, 7, 10, 12, 13, 15, },
  { 1, 4, 7, 10, 12, 13, 15, },
  { 0, 1, 4, 7, 10, 12, 13, 15, },
  { 2, 4, 7, 10, 12, 13, 15, },
  { 0, 2, 4, 7, 10, 12, 13, 15, },
  { 1, 2, 4, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 10, 12, 13, 15, },
  { 3, 4, 7, 10, 12, 13, 15, },
  { 0, 3, 4, 7, 10, 12, 13, 15, },
  { 1, 3, 4, 7, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 10, 12, 13, 15, },
  { 2, 3, 4, 7, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 12, 13, 15, },
  { 5, 7, 10, 12, 13, 15, },
  { 0, 5, 7, 10, 12, 13, 15, },
  { 1, 5, 7, 10, 12, 13, 15, },
  { 0, 1, 5, 7, 10, 12, 13, 15, },
  { 2, 5, 7, 10, 12, 13, 15, },
  { 0, 2, 5, 7, 10, 12, 13, 15, },
  { 1, 2, 5, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 10, 12, 13, 15, },
  { 3, 5, 7, 10, 12, 13, 15, },
  { 0, 3, 5, 7, 10, 12, 13, 15, },
  { 1, 3, 5, 7, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 10, 12, 13, 15, },
  { 2, 3, 5, 7, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 12, 13, 15, },
  { 4, 5, 7, 10, 12, 13, 15, },
  { 0, 4, 5, 7, 10, 12, 13, 15, },
  { 1, 4, 5, 7, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 10, 12, 13, 15, },
  { 2, 4, 5, 7, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 12, 13, 15, },
  { 3, 4, 5, 7, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 12, 13, 15, },
  { 6, 7, 10, 12, 13, 15, },
  { 0, 6, 7, 10, 12, 13, 15, },
  { 1, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 6, 7, 10, 12, 13, 15, },
  { 2, 6, 7, 10, 12, 13, 15, },
  { 0, 2, 6, 7, 10, 12, 13, 15, },
  { 1, 2, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 10, 12, 13, 15, },
  { 3, 6, 7, 10, 12, 13, 15, },
  { 0, 3, 6, 7, 10, 12, 13, 15, },
  { 1, 3, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 10, 12, 13, 15, },
  { 2, 3, 6, 7, 10, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 10, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 12, 13, 15, },
  { 4, 6, 7, 10, 12, 13, 15, },
  { 0, 4, 6, 7, 10, 12, 13, 15, },
  { 1, 4, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 10, 12, 13, 15, },
  { 2, 4, 6, 7, 10, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 10, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 12, 13, 15, },
  { 3, 4, 6, 7, 10, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 10, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 12, 13, 15, },
  { 5, 6, 7, 10, 12, 13, 15, },
  { 0, 5, 6, 7, 10, 12, 13, 15, },
  { 1, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 10, 12, 13, 15, },
  { 2, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 10, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 12, 13, 15, },
  { 3, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 10, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 12, 13, 15, },
  { 4, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 12, 13, 15, },
  { 8, 10, 12, 13, 15, },
  { 0, 8, 10, 12, 13, 15, },
  { 1, 8, 10, 12, 13, 15, },
  { 0, 1, 8, 10, 12, 13, 15, },
  { 2, 8, 10, 12, 13, 15, },
  { 0, 2, 8, 10, 12, 13, 15, },
  { 1, 2, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 8, 10, 12, 13, 15, },
  { 3, 8, 10, 12, 13, 15, },
  { 0, 3, 8, 10, 12, 13, 15, },
  { 1, 3, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 8, 10, 12, 13, 15, },
  { 2, 3, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 8, 10, 12, 13, 15, },
  { 4, 8, 10, 12, 13, 15, },
  { 0, 4, 8, 10, 12, 13, 15, },
  { 1, 4, 8, 10, 12, 13, 15, },
  { 0, 1, 4, 8, 10, 12, 13, 15, },
  { 2, 4, 8, 10, 12, 13, 15, },
  { 0, 2, 4, 8, 10, 12, 13, 15, },
  { 1, 2, 4, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 8, 10, 12, 13, 15, },
  { 3, 4, 8, 10, 12, 13, 15, },
  { 0, 3, 4, 8, 10, 12, 13, 15, },
  { 1, 3, 4, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 8, 10, 12, 13, 15, },
  { 2, 3, 4, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 12, 13, 15, },
  { 5, 8, 10, 12, 13, 15, },
  { 0, 5, 8, 10, 12, 13, 15, },
  { 1, 5, 8, 10, 12, 13, 15, },
  { 0, 1, 5, 8, 10, 12, 13, 15, },
  { 2, 5, 8, 10, 12, 13, 15, },
  { 0, 2, 5, 8, 10, 12, 13, 15, },
  { 1, 2, 5, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 8, 10, 12, 13, 15, },
  { 3, 5, 8, 10, 12, 13, 15, },
  { 0, 3, 5, 8, 10, 12, 13, 15, },
  { 1, 3, 5, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 8, 10, 12, 13, 15, },
  { 2, 3, 5, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 12, 13, 15, },
  { 4, 5, 8, 10, 12, 13, 15, },
  { 0, 4, 5, 8, 10, 12, 13, 15, },
  { 1, 4, 5, 8, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 8, 10, 12, 13, 15, },
  { 2, 4, 5, 8, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 8, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 12, 13, 15, },
  { 3, 4, 5, 8, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 8, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 12, 13, 15, },
  { 6, 8, 10, 12, 13, 15, },
  { 0, 6, 8, 10, 12, 13, 15, },
  { 1, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 6, 8, 10, 12, 13, 15, },
  { 2, 6, 8, 10, 12, 13, 15, },
  { 0, 2, 6, 8, 10, 12, 13, 15, },
  { 1, 2, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 6, 8, 10, 12, 13, 15, },
  { 3, 6, 8, 10, 12, 13, 15, },
  { 0, 3, 6, 8, 10, 12, 13, 15, },
  { 1, 3, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 6, 8, 10, 12, 13, 15, },
  { 2, 3, 6, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 6, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 12, 13, 15, },
  { 4, 6, 8, 10, 12, 13, 15, },
  { 0, 4, 6, 8, 10, 12, 13, 15, },
  { 1, 4, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 4, 6, 8, 10, 12, 13, 15, },
  { 2, 4, 6, 8, 10, 12, 13, 15, },
  { 0, 2, 4, 6, 8, 10, 12, 13, 15, },
  { 1, 2, 4, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 12, 13, 15, },
  { 3, 4, 6, 8, 10, 12, 13, 15, },
  { 0, 3, 4, 6, 8, 10, 12, 13, 15, },
  { 1, 3, 4, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 12, 13, 15, },
  { 2, 3, 4, 6, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 12, 13, 15, },
  { 5, 6, 8, 10, 12, 13, 15, },
  { 0, 5, 6, 8, 10, 12, 13, 15, },
  { 1, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 5, 6, 8, 10, 12, 13, 15, },
  { 2, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 2, 5, 6, 8, 10, 12, 13, 15, },
  { 1, 2, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 12, 13, 15, },
  { 3, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 3, 5, 6, 8, 10, 12, 13, 15, },
  { 1, 3, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 12, 13, 15, },
  { 2, 3, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 12, 13, 15, },
  { 4, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 1, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 2, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 3, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 12, 13, 15, },
  { 7, 8, 10, 12, 13, 15, },
  { 0, 7, 8, 10, 12, 13, 15, },
  { 1, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 7, 8, 10, 12, 13, 15, },
  { 2, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 7, 8, 10, 12, 13, 15, },
  { 3, 7, 8, 10, 12, 13, 15, },
  { 0, 3, 7, 8, 10, 12, 13, 15, },
  { 1, 3, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 7, 8, 10, 12, 13, 15, },
  { 2, 3, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 12, 13, 15, },
  { 4, 7, 8, 10, 12, 13, 15, },
  { 0, 4, 7, 8, 10, 12, 13, 15, },
  { 1, 4, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 4, 7, 8, 10, 12, 13, 15, },
  { 2, 4, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 4, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 4, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 12, 13, 15, },
  { 3, 4, 7, 8, 10, 12, 13, 15, },
  { 0, 3, 4, 7, 8, 10, 12, 13, 15, },
  { 1, 3, 4, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 12, 13, 15, },
  { 2, 3, 4, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 12, 13, 15, },
  { 5, 7, 8, 10, 12, 13, 15, },
  { 0, 5, 7, 8, 10, 12, 13, 15, },
  { 1, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 5, 7, 8, 10, 12, 13, 15, },
  { 2, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 5, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 12, 13, 15, },
  { 3, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 3, 5, 7, 8, 10, 12, 13, 15, },
  { 1, 3, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 12, 13, 15, },
  { 2, 3, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 12, 13, 15, },
  { 4, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 1, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 2, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 3, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 13, 15, },
  { 6, 7, 8, 10, 12, 13, 15, },
  { 0, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 6, 7, 8, 10, 12, 13, 15, },
  { 2, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 12, 13, 15, },
  { 3, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 3, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 3, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 12, 13, 15, },
  { 2, 3, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 12, 13, 15, },
  { 4, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 2, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 3, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 12, 13, 15, },
  { 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 2, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 3, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, },
  { 9, 10, 12, 13, 15, },
  { 0, 9, 10, 12, 13, 15, },
  { 1, 9, 10, 12, 13, 15, },
  { 0, 1, 9, 10, 12, 13, 15, },
  { 2, 9, 10, 12, 13, 15, },
  { 0, 2, 9, 10, 12, 13, 15, },
  { 1, 2, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 9, 10, 12, 13, 15, },
  { 3, 9, 10, 12, 13, 15, },
  { 0, 3, 9, 10, 12, 13, 15, },
  { 1, 3, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 9, 10, 12, 13, 15, },
  { 2, 3, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 9, 10, 12, 13, 15, },
  { 4, 9, 10, 12, 13, 15, },
  { 0, 4, 9, 10, 12, 13, 15, },
  { 1, 4, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 9, 10, 12, 13, 15, },
  { 2, 4, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 9, 10, 12, 13, 15, },
  { 3, 4, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 12, 13, 15, },
  { 5, 9, 10, 12, 13, 15, },
  { 0, 5, 9, 10, 12, 13, 15, },
  { 1, 5, 9, 10, 12, 13, 15, },
  { 0, 1, 5, 9, 10, 12, 13, 15, },
  { 2, 5, 9, 10, 12, 13, 15, },
  { 0, 2, 5, 9, 10, 12, 13, 15, },
  { 1, 2, 5, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 9, 10, 12, 13, 15, },
  { 3, 5, 9, 10, 12, 13, 15, },
  { 0, 3, 5, 9, 10, 12, 13, 15, },
  { 1, 3, 5, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 9, 10, 12, 13, 15, },
  { 2, 3, 5, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 12, 13, 15, },
  { 4, 5, 9, 10, 12, 13, 15, },
  { 0, 4, 5, 9, 10, 12, 13, 15, },
  { 1, 4, 5, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 9, 10, 12, 13, 15, },
  { 2, 4, 5, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 12, 13, 15, },
  { 3, 4, 5, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 12, 13, 15, },
  { 6, 9, 10, 12, 13, 15, },
  { 0, 6, 9, 10, 12, 13, 15, },
  { 1, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 6, 9, 10, 12, 13, 15, },
  { 2, 6, 9, 10, 12, 13, 15, },
  { 0, 2, 6, 9, 10, 12, 13, 15, },
  { 1, 2, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 6, 9, 10, 12, 13, 15, },
  { 3, 6, 9, 10, 12, 13, 15, },
  { 0, 3, 6, 9, 10, 12, 13, 15, },
  { 1, 3, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 6, 9, 10, 12, 13, 15, },
  { 2, 3, 6, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 6, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 12, 13, 15, },
  { 4, 6, 9, 10, 12, 13, 15, },
  { 0, 4, 6, 9, 10, 12, 13, 15, },
  { 1, 4, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 6, 9, 10, 12, 13, 15, },
  { 2, 4, 6, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 6, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 12, 13, 15, },
  { 3, 4, 6, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 6, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 6, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 12, 13, 15, },
  { 5, 6, 9, 10, 12, 13, 15, },
  { 0, 5, 6, 9, 10, 12, 13, 15, },
  { 1, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 5, 6, 9, 10, 12, 13, 15, },
  { 2, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 2, 5, 6, 9, 10, 12, 13, 15, },
  { 1, 2, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 12, 13, 15, },
  { 3, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 3, 5, 6, 9, 10, 12, 13, 15, },
  { 1, 3, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 12, 13, 15, },
  { 2, 3, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 12, 13, 15, },
  { 4, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 1, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 2, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 3, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 12, 13, 15, },
  { 7, 9, 10, 12, 13, 15, },
  { 0, 7, 9, 10, 12, 13, 15, },
  { 1, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 7, 9, 10, 12, 13, 15, },
  { 2, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 7, 9, 10, 12, 13, 15, },
  { 3, 7, 9, 10, 12, 13, 15, },
  { 0, 3, 7, 9, 10, 12, 13, 15, },
  { 1, 3, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 7, 9, 10, 12, 13, 15, },
  { 2, 3, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 12, 13, 15, },
  { 4, 7, 9, 10, 12, 13, 15, },
  { 0, 4, 7, 9, 10, 12, 13, 15, },
  { 1, 4, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 7, 9, 10, 12, 13, 15, },
  { 2, 4, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 12, 13, 15, },
  { 3, 4, 7, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 7, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 12, 13, 15, },
  { 5, 7, 9, 10, 12, 13, 15, },
  { 0, 5, 7, 9, 10, 12, 13, 15, },
  { 1, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 5, 7, 9, 10, 12, 13, 15, },
  { 2, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 5, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 12, 13, 15, },
  { 3, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 3, 5, 7, 9, 10, 12, 13, 15, },
  { 1, 3, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 12, 13, 15, },
  { 2, 3, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 12, 13, 15, },
  { 4, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 1, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 2, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 3, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 12, 13, 15, },
  { 6, 7, 9, 10, 12, 13, 15, },
  { 0, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 6, 7, 9, 10, 12, 13, 15, },
  { 2, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 12, 13, 15, },
  { 3, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 3, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 3, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 12, 13, 15, },
  { 2, 3, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 12, 13, 15, },
  { 4, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 2, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 3, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 12, 13, 15, },
  { 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 2, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 3, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 15, },
  { 8, 9, 10, 12, 13, 15, },
  { 0, 8, 9, 10, 12, 13, 15, },
  { 1, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 8, 9, 10, 12, 13, 15, },
  { 2, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 8, 9, 10, 12, 13, 15, },
  { 3, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 12, 13, 15, },
  { 4, 8, 9, 10, 12, 13, 15, },
  { 0, 4, 8, 9, 10, 12, 13, 15, },
  { 1, 4, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 8, 9, 10, 12, 13, 15, },
  { 2, 4, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 12, 13, 15, },
  { 3, 4, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 12, 13, 15, },
  { 5, 8, 9, 10, 12, 13, 15, },
  { 0, 5, 8, 9, 10, 12, 13, 15, },
  { 1, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 5, 8, 9, 10, 12, 13, 15, },
  { 2, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 5, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 12, 13, 15, },
  { 3, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 5, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 12, 13, 15, },
  { 4, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 1, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 2, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 3, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 12, 13, 15, },
  { 6, 8, 9, 10, 12, 13, 15, },
  { 0, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 6, 8, 9, 10, 12, 13, 15, },
  { 2, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 12, 13, 15, },
  { 3, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 12, 13, 15, },
  { 4, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 2, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 3, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 12, 13, 15, },
  { 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 2, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 3, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, },
  { 7, 8, 9, 10, 12, 13, 15, },
  { 0, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 12, 13, 15, },
  { 3, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 12, 13, 15, },
  { 4, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 3, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 12, 13, 15, },
  { 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 3, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 15, },
  { 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 3, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, },
  { 11, 12, 13, 15, },
  { 0, 11, 12, 13, 15, },
  { 1, 11, 12, 13, 15, },
  { 0, 1, 11, 12, 13, 15, },
  { 2, 11, 12, 13, 15, },
  { 0, 2, 11, 12, 13, 15, },
  { 1, 2, 11, 12, 13, 15, },
  { 0, 1, 2, 11, 12, 13, 15, },
  { 3, 11, 12, 13, 15, },
  { 0, 3, 11, 12, 13, 15, },
  { 1, 3, 11, 12, 13, 15, },
  { 0, 1, 3, 11, 12, 13, 15, },
  { 2, 3, 11, 12, 13, 15, },
  { 0, 2, 3, 11, 12, 13, 15, },
  { 1, 2, 3, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 11, 12, 13, 15, },
  { 4, 11, 12, 13, 15, },
  { 0, 4, 11, 12, 13, 15, },
  { 1, 4, 11, 12, 13, 15, },
  { 0, 1, 4, 11, 12, 13, 15, },
  { 2, 4, 11, 12, 13, 15, },
  { 0, 2, 4, 11, 12, 13, 15, },
  { 1, 2, 4, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 11, 12, 13, 15, },
  { 3, 4, 11, 12, 13, 15, },
  { 0, 3, 4, 11, 12, 13, 15, },
  { 1, 3, 4, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 11, 12, 13, 15, },
  { 2, 3, 4, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 11, 12, 13, 15, },
  { 5, 11, 12, 13, 15, },
  { 0, 5, 11, 12, 13, 15, },
  { 1, 5, 11, 12, 13, 15, },
  { 0, 1, 5, 11, 12, 13, 15, },
  { 2, 5, 11, 12, 13, 15, },
  { 0, 2, 5, 11, 12, 13, 15, },
  { 1, 2, 5, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 11, 12, 13, 15, },
  { 3, 5, 11, 12, 13, 15, },
  { 0, 3, 5, 11, 12, 13, 15, },
  { 1, 3, 5, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 11, 12, 13, 15, },
  { 2, 3, 5, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 11, 12, 13, 15, },
  { 4, 5, 11, 12, 13, 15, },
  { 0, 4, 5, 11, 12, 13, 15, },
  { 1, 4, 5, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 11, 12, 13, 15, },
  { 2, 4, 5, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 11, 12, 13, 15, },
  { 3, 4, 5, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 11, 12, 13, 15, },
  { 6, 11, 12, 13, 15, },
  { 0, 6, 11, 12, 13, 15, },
  { 1, 6, 11, 12, 13, 15, },
  { 0, 1, 6, 11, 12, 13, 15, },
  { 2, 6, 11, 12, 13, 15, },
  { 0, 2, 6, 11, 12, 13, 15, },
  { 1, 2, 6, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 11, 12, 13, 15, },
  { 3, 6, 11, 12, 13, 15, },
  { 0, 3, 6, 11, 12, 13, 15, },
  { 1, 3, 6, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 11, 12, 13, 15, },
  { 2, 3, 6, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 11, 12, 13, 15, },
  { 4, 6, 11, 12, 13, 15, },
  { 0, 4, 6, 11, 12, 13, 15, },
  { 1, 4, 6, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 11, 12, 13, 15, },
  { 2, 4, 6, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 11, 12, 13, 15, },
  { 3, 4, 6, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 11, 12, 13, 15, },
  { 5, 6, 11, 12, 13, 15, },
  { 0, 5, 6, 11, 12, 13, 15, },
  { 1, 5, 6, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 11, 12, 13, 15, },
  { 2, 5, 6, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 11, 12, 13, 15, },
  { 3, 5, 6, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 11, 12, 13, 15, },
  { 4, 5, 6, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 12, 13, 15, },
  { 7, 11, 12, 13, 15, },
  { 0, 7, 11, 12, 13, 15, },
  { 1, 7, 11, 12, 13, 15, },
  { 0, 1, 7, 11, 12, 13, 15, },
  { 2, 7, 11, 12, 13, 15, },
  { 0, 2, 7, 11, 12, 13, 15, },
  { 1, 2, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 7, 11, 12, 13, 15, },
  { 3, 7, 11, 12, 13, 15, },
  { 0, 3, 7, 11, 12, 13, 15, },
  { 1, 3, 7, 11, 12, 13, 15, },
  { 0, 1, 3, 7, 11, 12, 13, 15, },
  { 2, 3, 7, 11, 12, 13, 15, },
  { 0, 2, 3, 7, 11, 12, 13, 15, },
  { 1, 2, 3, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 11, 12, 13, 15, },
  { 4, 7, 11, 12, 13, 15, },
  { 0, 4, 7, 11, 12, 13, 15, },
  { 1, 4, 7, 11, 12, 13, 15, },
  { 0, 1, 4, 7, 11, 12, 13, 15, },
  { 2, 4, 7, 11, 12, 13, 15, },
  { 0, 2, 4, 7, 11, 12, 13, 15, },
  { 1, 2, 4, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 11, 12, 13, 15, },
  { 3, 4, 7, 11, 12, 13, 15, },
  { 0, 3, 4, 7, 11, 12, 13, 15, },
  { 1, 3, 4, 7, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 11, 12, 13, 15, },
  { 2, 3, 4, 7, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 11, 12, 13, 15, },
  { 5, 7, 11, 12, 13, 15, },
  { 0, 5, 7, 11, 12, 13, 15, },
  { 1, 5, 7, 11, 12, 13, 15, },
  { 0, 1, 5, 7, 11, 12, 13, 15, },
  { 2, 5, 7, 11, 12, 13, 15, },
  { 0, 2, 5, 7, 11, 12, 13, 15, },
  { 1, 2, 5, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 11, 12, 13, 15, },
  { 3, 5, 7, 11, 12, 13, 15, },
  { 0, 3, 5, 7, 11, 12, 13, 15, },
  { 1, 3, 5, 7, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 11, 12, 13, 15, },
  { 2, 3, 5, 7, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 11, 12, 13, 15, },
  { 4, 5, 7, 11, 12, 13, 15, },
  { 0, 4, 5, 7, 11, 12, 13, 15, },
  { 1, 4, 5, 7, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 11, 12, 13, 15, },
  { 2, 4, 5, 7, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 11, 12, 13, 15, },
  { 3, 4, 5, 7, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 12, 13, 15, },
  { 6, 7, 11, 12, 13, 15, },
  { 0, 6, 7, 11, 12, 13, 15, },
  { 1, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 6, 7, 11, 12, 13, 15, },
  { 2, 6, 7, 11, 12, 13, 15, },
  { 0, 2, 6, 7, 11, 12, 13, 15, },
  { 1, 2, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 11, 12, 13, 15, },
  { 3, 6, 7, 11, 12, 13, 15, },
  { 0, 3, 6, 7, 11, 12, 13, 15, },
  { 1, 3, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 11, 12, 13, 15, },
  { 2, 3, 6, 7, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 11, 12, 13, 15, },
  { 4, 6, 7, 11, 12, 13, 15, },
  { 0, 4, 6, 7, 11, 12, 13, 15, },
  { 1, 4, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 11, 12, 13, 15, },
  { 2, 4, 6, 7, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 11, 12, 13, 15, },
  { 3, 4, 6, 7, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 12, 13, 15, },
  { 5, 6, 7, 11, 12, 13, 15, },
  { 0, 5, 6, 7, 11, 12, 13, 15, },
  { 1, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 11, 12, 13, 15, },
  { 2, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 11, 12, 13, 15, },
  { 3, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 12, 13, 15, },
  { 4, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 12, 13, 15, },
  { 8, 11, 12, 13, 15, },
  { 0, 8, 11, 12, 13, 15, },
  { 1, 8, 11, 12, 13, 15, },
  { 0, 1, 8, 11, 12, 13, 15, },
  { 2, 8, 11, 12, 13, 15, },
  { 0, 2, 8, 11, 12, 13, 15, },
  { 1, 2, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 8, 11, 12, 13, 15, },
  { 3, 8, 11, 12, 13, 15, },
  { 0, 3, 8, 11, 12, 13, 15, },
  { 1, 3, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 8, 11, 12, 13, 15, },
  { 2, 3, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 8, 11, 12, 13, 15, },
  { 4, 8, 11, 12, 13, 15, },
  { 0, 4, 8, 11, 12, 13, 15, },
  { 1, 4, 8, 11, 12, 13, 15, },
  { 0, 1, 4, 8, 11, 12, 13, 15, },
  { 2, 4, 8, 11, 12, 13, 15, },
  { 0, 2, 4, 8, 11, 12, 13, 15, },
  { 1, 2, 4, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 8, 11, 12, 13, 15, },
  { 3, 4, 8, 11, 12, 13, 15, },
  { 0, 3, 4, 8, 11, 12, 13, 15, },
  { 1, 3, 4, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 8, 11, 12, 13, 15, },
  { 2, 3, 4, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 11, 12, 13, 15, },
  { 5, 8, 11, 12, 13, 15, },
  { 0, 5, 8, 11, 12, 13, 15, },
  { 1, 5, 8, 11, 12, 13, 15, },
  { 0, 1, 5, 8, 11, 12, 13, 15, },
  { 2, 5, 8, 11, 12, 13, 15, },
  { 0, 2, 5, 8, 11, 12, 13, 15, },
  { 1, 2, 5, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 8, 11, 12, 13, 15, },
  { 3, 5, 8, 11, 12, 13, 15, },
  { 0, 3, 5, 8, 11, 12, 13, 15, },
  { 1, 3, 5, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 8, 11, 12, 13, 15, },
  { 2, 3, 5, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 11, 12, 13, 15, },
  { 4, 5, 8, 11, 12, 13, 15, },
  { 0, 4, 5, 8, 11, 12, 13, 15, },
  { 1, 4, 5, 8, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 8, 11, 12, 13, 15, },
  { 2, 4, 5, 8, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 8, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 11, 12, 13, 15, },
  { 3, 4, 5, 8, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 8, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 12, 13, 15, },
  { 6, 8, 11, 12, 13, 15, },
  { 0, 6, 8, 11, 12, 13, 15, },
  { 1, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 6, 8, 11, 12, 13, 15, },
  { 2, 6, 8, 11, 12, 13, 15, },
  { 0, 2, 6, 8, 11, 12, 13, 15, },
  { 1, 2, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 8, 11, 12, 13, 15, },
  { 3, 6, 8, 11, 12, 13, 15, },
  { 0, 3, 6, 8, 11, 12, 13, 15, },
  { 1, 3, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 8, 11, 12, 13, 15, },
  { 2, 3, 6, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 11, 12, 13, 15, },
  { 4, 6, 8, 11, 12, 13, 15, },
  { 0, 4, 6, 8, 11, 12, 13, 15, },
  { 1, 4, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 8, 11, 12, 13, 15, },
  { 2, 4, 6, 8, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 8, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 11, 12, 13, 15, },
  { 3, 4, 6, 8, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 8, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 12, 13, 15, },
  { 5, 6, 8, 11, 12, 13, 15, },
  { 0, 5, 6, 8, 11, 12, 13, 15, },
  { 1, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 8, 11, 12, 13, 15, },
  { 2, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 8, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 11, 12, 13, 15, },
  { 3, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 8, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 12, 13, 15, },
  { 4, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 12, 13, 15, },
  { 7, 8, 11, 12, 13, 15, },
  { 0, 7, 8, 11, 12, 13, 15, },
  { 1, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 7, 8, 11, 12, 13, 15, },
  { 2, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 7, 8, 11, 12, 13, 15, },
  { 3, 7, 8, 11, 12, 13, 15, },
  { 0, 3, 7, 8, 11, 12, 13, 15, },
  { 1, 3, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 7, 8, 11, 12, 13, 15, },
  { 2, 3, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 11, 12, 13, 15, },
  { 4, 7, 8, 11, 12, 13, 15, },
  { 0, 4, 7, 8, 11, 12, 13, 15, },
  { 1, 4, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 4, 7, 8, 11, 12, 13, 15, },
  { 2, 4, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 4, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 4, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 11, 12, 13, 15, },
  { 3, 4, 7, 8, 11, 12, 13, 15, },
  { 0, 3, 4, 7, 8, 11, 12, 13, 15, },
  { 1, 3, 4, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 11, 12, 13, 15, },
  { 2, 3, 4, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 12, 13, 15, },
  { 5, 7, 8, 11, 12, 13, 15, },
  { 0, 5, 7, 8, 11, 12, 13, 15, },
  { 1, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 5, 7, 8, 11, 12, 13, 15, },
  { 2, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 5, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 11, 12, 13, 15, },
  { 3, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 3, 5, 7, 8, 11, 12, 13, 15, },
  { 1, 3, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 11, 12, 13, 15, },
  { 2, 3, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 12, 13, 15, },
  { 4, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 1, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 2, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 3, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 12, 13, 15, },
  { 6, 7, 8, 11, 12, 13, 15, },
  { 0, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 6, 7, 8, 11, 12, 13, 15, },
  { 2, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 11, 12, 13, 15, },
  { 3, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 3, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 3, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 11, 12, 13, 15, },
  { 2, 3, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 12, 13, 15, },
  { 4, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 2, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 3, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 12, 13, 15, },
  { 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 2, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 3, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 15, },
  { 9, 11, 12, 13, 15, },
  { 0, 9, 11, 12, 13, 15, },
  { 1, 9, 11, 12, 13, 15, },
  { 0, 1, 9, 11, 12, 13, 15, },
  { 2, 9, 11, 12, 13, 15, },
  { 0, 2, 9, 11, 12, 13, 15, },
  { 1, 2, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 9, 11, 12, 13, 15, },
  { 3, 9, 11, 12, 13, 15, },
  { 0, 3, 9, 11, 12, 13, 15, },
  { 1, 3, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 9, 11, 12, 13, 15, },
  { 2, 3, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 9, 11, 12, 13, 15, },
  { 4, 9, 11, 12, 13, 15, },
  { 0, 4, 9, 11, 12, 13, 15, },
  { 1, 4, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 9, 11, 12, 13, 15, },
  { 2, 4, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 9, 11, 12, 13, 15, },
  { 3, 4, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 9, 11, 12, 13, 15, },
  { 5, 9, 11, 12, 13, 15, },
  { 0, 5, 9, 11, 12, 13, 15, },
  { 1, 5, 9, 11, 12, 13, 15, },
  { 0, 1, 5, 9, 11, 12, 13, 15, },
  { 2, 5, 9, 11, 12, 13, 15, },
  { 0, 2, 5, 9, 11, 12, 13, 15, },
  { 1, 2, 5, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 9, 11, 12, 13, 15, },
  { 3, 5, 9, 11, 12, 13, 15, },
  { 0, 3, 5, 9, 11, 12, 13, 15, },
  { 1, 3, 5, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 9, 11, 12, 13, 15, },
  { 2, 3, 5, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 9, 11, 12, 13, 15, },
  { 4, 5, 9, 11, 12, 13, 15, },
  { 0, 4, 5, 9, 11, 12, 13, 15, },
  { 1, 4, 5, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 9, 11, 12, 13, 15, },
  { 2, 4, 5, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 9, 11, 12, 13, 15, },
  { 3, 4, 5, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 12, 13, 15, },
  { 6, 9, 11, 12, 13, 15, },
  { 0, 6, 9, 11, 12, 13, 15, },
  { 1, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 6, 9, 11, 12, 13, 15, },
  { 2, 6, 9, 11, 12, 13, 15, },
  { 0, 2, 6, 9, 11, 12, 13, 15, },
  { 1, 2, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 9, 11, 12, 13, 15, },
  { 3, 6, 9, 11, 12, 13, 15, },
  { 0, 3, 6, 9, 11, 12, 13, 15, },
  { 1, 3, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 9, 11, 12, 13, 15, },
  { 2, 3, 6, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 9, 11, 12, 13, 15, },
  { 4, 6, 9, 11, 12, 13, 15, },
  { 0, 4, 6, 9, 11, 12, 13, 15, },
  { 1, 4, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 9, 11, 12, 13, 15, },
  { 2, 4, 6, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 9, 11, 12, 13, 15, },
  { 3, 4, 6, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 12, 13, 15, },
  { 5, 6, 9, 11, 12, 13, 15, },
  { 0, 5, 6, 9, 11, 12, 13, 15, },
  { 1, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 9, 11, 12, 13, 15, },
  { 2, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 9, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 9, 11, 12, 13, 15, },
  { 3, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 9, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 9, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 12, 13, 15, },
  { 4, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 12, 13, 15, },
  { 7, 9, 11, 12, 13, 15, },
  { 0, 7, 9, 11, 12, 13, 15, },
  { 1, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 7, 9, 11, 12, 13, 15, },
  { 2, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 7, 9, 11, 12, 13, 15, },
  { 3, 7, 9, 11, 12, 13, 15, },
  { 0, 3, 7, 9, 11, 12, 13, 15, },
  { 1, 3, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 7, 9, 11, 12, 13, 15, },
  { 2, 3, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 9, 11, 12, 13, 15, },
  { 4, 7, 9, 11, 12, 13, 15, },
  { 0, 4, 7, 9, 11, 12, 13, 15, },
  { 1, 4, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 7, 9, 11, 12, 13, 15, },
  { 2, 4, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 9, 11, 12, 13, 15, },
  { 3, 4, 7, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 7, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 12, 13, 15, },
  { 5, 7, 9, 11, 12, 13, 15, },
  { 0, 5, 7, 9, 11, 12, 13, 15, },
  { 1, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 5, 7, 9, 11, 12, 13, 15, },
  { 2, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 5, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 9, 11, 12, 13, 15, },
  { 3, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 3, 5, 7, 9, 11, 12, 13, 15, },
  { 1, 3, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 9, 11, 12, 13, 15, },
  { 2, 3, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 12, 13, 15, },
  { 4, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 1, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 2, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 3, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 12, 13, 15, },
  { 6, 7, 9, 11, 12, 13, 15, },
  { 0, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 6, 7, 9, 11, 12, 13, 15, },
  { 2, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 9, 11, 12, 13, 15, },
  { 3, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 3, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 3, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 9, 11, 12, 13, 15, },
  { 2, 3, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 12, 13, 15, },
  { 4, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 2, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 3, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 12, 13, 15, },
  { 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 2, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 3, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 15, },
  { 8, 9, 11, 12, 13, 15, },
  { 0, 8, 9, 11, 12, 13, 15, },
  { 1, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 8, 9, 11, 12, 13, 15, },
  { 2, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 8, 9, 11, 12, 13, 15, },
  { 3, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 8, 9, 11, 12, 13, 15, },
  { 4, 8, 9, 11, 12, 13, 15, },
  { 0, 4, 8, 9, 11, 12, 13, 15, },
  { 1, 4, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 8, 9, 11, 12, 13, 15, },
  { 2, 4, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 8, 9, 11, 12, 13, 15, },
  { 3, 4, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 12, 13, 15, },
  { 5, 8, 9, 11, 12, 13, 15, },
  { 0, 5, 8, 9, 11, 12, 13, 15, },
  { 1, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 5, 8, 9, 11, 12, 13, 15, },
  { 2, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 5, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 8, 9, 11, 12, 13, 15, },
  { 3, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 5, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 12, 13, 15, },
  { 4, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 1, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 2, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 3, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 12, 13, 15, },
  { 6, 8, 9, 11, 12, 13, 15, },
  { 0, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 6, 8, 9, 11, 12, 13, 15, },
  { 2, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 8, 9, 11, 12, 13, 15, },
  { 3, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 12, 13, 15, },
  { 4, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 2, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 3, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 12, 13, 15, },
  { 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 2, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 3, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 15, },
  { 7, 8, 9, 11, 12, 13, 15, },
  { 0, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 7, 8, 9, 11, 12, 13, 15, },
  { 3, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 12, 13, 15, },
  { 4, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 3, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 12, 13, 15, },
  { 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 3, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 3, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 15, },
  { 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 3, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 3, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 3, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 15, },
  { 10, 11, 12, 13, 15, },
  { 0, 10, 11, 12, 13, 15, },
  { 1, 10, 11, 12, 13, 15, },
  { 0, 1, 10, 11, 12, 13, 15, },
  { 2, 10, 11, 12, 13, 15, },
  { 0, 2, 10, 11, 12, 13, 15, },
  { 1, 2, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 10, 11, 12, 13, 15, },
  { 3, 10, 11, 12, 13, 15, },
  { 0, 3, 10, 11, 12, 13, 15, },
  { 1, 3, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 10, 11, 12, 13, 15, },
  { 2, 3, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 10, 11, 12, 13, 15, },
  { 4, 10, 11, 12, 13, 15, },
  { 0, 4, 10, 11, 12, 13, 15, },
  { 1, 4, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 10, 11, 12, 13, 15, },
  { 2, 4, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 10, 11, 12, 13, 15, },
  { 3, 4, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 10, 11, 12, 13, 15, },
  { 5, 10, 11, 12, 13, 15, },
  { 0, 5, 10, 11, 12, 13, 15, },
  { 1, 5, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 10, 11, 12, 13, 15, },
  { 2, 5, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 10, 11, 12, 13, 15, },
  { 3, 5, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 10, 11, 12, 13, 15, },
  { 4, 5, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 12, 13, 15, },
  { 6, 10, 11, 12, 13, 15, },
  { 0, 6, 10, 11, 12, 13, 15, },
  { 1, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 6, 10, 11, 12, 13, 15, },
  { 2, 6, 10, 11, 12, 13, 15, },
  { 0, 2, 6, 10, 11, 12, 13, 15, },
  { 1, 2, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 10, 11, 12, 13, 15, },
  { 3, 6, 10, 11, 12, 13, 15, },
  { 0, 3, 6, 10, 11, 12, 13, 15, },
  { 1, 3, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 10, 11, 12, 13, 15, },
  { 2, 3, 6, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 10, 11, 12, 13, 15, },
  { 4, 6, 10, 11, 12, 13, 15, },
  { 0, 4, 6, 10, 11, 12, 13, 15, },
  { 1, 4, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 10, 11, 12, 13, 15, },
  { 2, 4, 6, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 10, 11, 12, 13, 15, },
  { 3, 4, 6, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 12, 13, 15, },
  { 5, 6, 10, 11, 12, 13, 15, },
  { 0, 5, 6, 10, 11, 12, 13, 15, },
  { 1, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 10, 11, 12, 13, 15, },
  { 2, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 10, 11, 12, 13, 15, },
  { 3, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 12, 13, 15, },
  { 4, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, 15, },
  { 7, 10, 11, 12, 13, 15, },
  { 0, 7, 10, 11, 12, 13, 15, },
  { 1, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 7, 10, 11, 12, 13, 15, },
  { 2, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 7, 10, 11, 12, 13, 15, },
  { 3, 7, 10, 11, 12, 13, 15, },
  { 0, 3, 7, 10, 11, 12, 13, 15, },
  { 1, 3, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 7, 10, 11, 12, 13, 15, },
  { 2, 3, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 10, 11, 12, 13, 15, },
  { 4, 7, 10, 11, 12, 13, 15, },
  { 0, 4, 7, 10, 11, 12, 13, 15, },
  { 1, 4, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 7, 10, 11, 12, 13, 15, },
  { 2, 4, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 10, 11, 12, 13, 15, },
  { 3, 4, 7, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 7, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 12, 13, 15, },
  { 5, 7, 10, 11, 12, 13, 15, },
  { 0, 5, 7, 10, 11, 12, 13, 15, },
  { 1, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 7, 10, 11, 12, 13, 15, },
  { 2, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 10, 11, 12, 13, 15, },
  { 3, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 7, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 12, 13, 15, },
  { 4, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 12, 13, 15, },
  { 6, 7, 10, 11, 12, 13, 15, },
  { 0, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 6, 7, 10, 11, 12, 13, 15, },
  { 2, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 10, 11, 12, 13, 15, },
  { 3, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 3, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 3, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 10, 11, 12, 13, 15, },
  { 2, 3, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 12, 13, 15, },
  { 4, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 2, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 3, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 12, 13, 15, },
  { 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 2, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 3, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 15, },
  { 8, 10, 11, 12, 13, 15, },
  { 0, 8, 10, 11, 12, 13, 15, },
  { 1, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 8, 10, 11, 12, 13, 15, },
  { 2, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 8, 10, 11, 12, 13, 15, },
  { 3, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 8, 10, 11, 12, 13, 15, },
  { 4, 8, 10, 11, 12, 13, 15, },
  { 0, 4, 8, 10, 11, 12, 13, 15, },
  { 1, 4, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 8, 10, 11, 12, 13, 15, },
  { 2, 4, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 8, 10, 11, 12, 13, 15, },
  { 3, 4, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 12, 13, 15, },
  { 5, 8, 10, 11, 12, 13, 15, },
  { 0, 5, 8, 10, 11, 12, 13, 15, },
  { 1, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 8, 10, 11, 12, 13, 15, },
  { 2, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 8, 10, 11, 12, 13, 15, },
  { 3, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 12, 13, 15, },
  { 4, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 12, 13, 15, },
  { 6, 8, 10, 11, 12, 13, 15, },
  { 0, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 6, 8, 10, 11, 12, 13, 15, },
  { 2, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 8, 10, 11, 12, 13, 15, },
  { 3, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 12, 13, 15, },
  { 4, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 2, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 3, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 12, 13, 15, },
  { 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 2, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 3, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 15, },
  { 7, 8, 10, 11, 12, 13, 15, },
  { 0, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 7, 8, 10, 11, 12, 13, 15, },
  { 3, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 12, 13, 15, },
  { 4, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 3, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 12, 13, 15, },
  { 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 3, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 15, },
  { 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 3, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 3, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 15, },
  { 9, 10, 11, 12, 13, 15, },
  { 0, 9, 10, 11, 12, 13, 15, },
  { 1, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 9, 10, 11, 12, 13, 15, },
  { 2, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 9, 10, 11, 12, 13, 15, },
  { 3, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 9, 10, 11, 12, 13, 15, },
  { 4, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 12, 13, 15, },
  { 5, 9, 10, 11, 12, 13, 15, },
  { 0, 5, 9, 10, 11, 12, 13, 15, },
  { 1, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 9, 10, 11, 12, 13, 15, },
  { 2, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 9, 10, 11, 12, 13, 15, },
  { 3, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 12, 13, 15, },
  { 4, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, 15, },
  { 6, 9, 10, 11, 12, 13, 15, },
  { 0, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 6, 9, 10, 11, 12, 13, 15, },
  { 2, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 9, 10, 11, 12, 13, 15, },
  { 3, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 12, 13, 15, },
  { 4, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 12, 13, 15, },
  { 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 2, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 3, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 15, },
  { 7, 9, 10, 11, 12, 13, 15, },
  { 0, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 7, 9, 10, 11, 12, 13, 15, },
  { 3, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 12, 13, 15, },
  { 4, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 12, 13, 15, },
  { 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 3, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 15, },
  { 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 3, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 3, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 15, },
  { 8, 9, 10, 11, 12, 13, 15, },
  { 0, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 12, 13, 15, },
  { 4, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 12, 13, 15, },
  { 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 15, },
  { 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, },
  { 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, },
  { 14, 15, },
  { 0, 14, 15, },
  { 1, 14, 15, },
  { 0, 1, 14, 15, },
  { 2, 14, 15, },
  { 0, 2, 14, 15, },
  { 1, 2, 14, 15, },
  { 0, 1, 2, 14, 15, },
  { 3, 14, 15, },
  { 0, 3, 14, 15, },
  { 1, 3, 14, 15, },
  { 0, 1, 3, 14, 15, },
  { 2, 3, 14, 15, },
  { 0, 2, 3, 14, 15, },
  { 1, 2, 3, 14, 15, },
  { 0, 1, 2, 3, 14, 15, },
  { 4, 14, 15, },
  { 0, 4, 14, 15, },
  { 1, 4, 14, 15, },
  { 0, 1, 4, 14, 15, },
  { 2, 4, 14, 15, },
  { 0, 2, 4, 14, 15, },
  { 1, 2, 4, 14, 15, },
  { 0, 1, 2, 4, 14, 15, },
  { 3, 4, 14, 15, },
  { 0, 3, 4, 14, 15, },
  { 1, 3, 4, 14, 15, },
  { 0, 1, 3, 4, 14, 15, },
  { 2, 3, 4, 14, 15, },
  { 0, 2, 3, 4, 14, 15, },
  { 1, 2, 3, 4, 14, 15, },
  { 0, 1, 2, 3, 4, 14, 15, },
  { 5, 14, 15, },
  { 0, 5, 14, 15, },
  { 1, 5, 14, 15, },
  { 0, 1, 5, 14, 15, },
  { 2, 5, 14, 15, },
  { 0, 2, 5, 14, 15, },
  { 1, 2, 5, 14, 15, },
  { 0, 1, 2, 5, 14, 15, },
  { 3, 5, 14, 15, },
  { 0, 3, 5, 14, 15, },
  { 1, 3, 5, 14, 15, },
  { 0, 1, 3, 5, 14, 15, },
  { 2, 3, 5, 14, 15, },
  { 0, 2, 3, 5, 14, 15, },
  { 1, 2, 3, 5, 14, 15, },
  { 0, 1, 2, 3, 5, 14, 15, },
  { 4, 5, 14, 15, },
  { 0, 4, 5, 14, 15, },
  { 1, 4, 5, 14, 15, },
  { 0, 1, 4, 5, 14, 15, },
  { 2, 4, 5, 14, 15, },
  { 0, 2, 4, 5, 14, 15, },
  { 1, 2, 4, 5, 14, 15, },
  { 0, 1, 2, 4, 5, 14, 15, },
  { 3, 4, 5, 14, 15, },
  { 0, 3, 4, 5, 14, 15, },
  { 1, 3, 4, 5, 14, 15, },
  { 0, 1, 3, 4, 5, 14, 15, },
  { 2, 3, 4, 5, 14, 15, },
  { 0, 2, 3, 4, 5, 14, 15, },
  { 1, 2, 3, 4, 5, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 14, 15, },
  { 6, 14, 15, },
  { 0, 6, 14, 15, },
  { 1, 6, 14, 15, },
  { 0, 1, 6, 14, 15, },
  { 2, 6, 14, 15, },
  { 0, 2, 6, 14, 15, },
  { 1, 2, 6, 14, 15, },
  { 0, 1, 2, 6, 14, 15, },
  { 3, 6, 14, 15, },
  { 0, 3, 6, 14, 15, },
  { 1, 3, 6, 14, 15, },
  { 0, 1, 3, 6, 14, 15, },
  { 2, 3, 6, 14, 15, },
  { 0, 2, 3, 6, 14, 15, },
  { 1, 2, 3, 6, 14, 15, },
  { 0, 1, 2, 3, 6, 14, 15, },
  { 4, 6, 14, 15, },
  { 0, 4, 6, 14, 15, },
  { 1, 4, 6, 14, 15, },
  { 0, 1, 4, 6, 14, 15, },
  { 2, 4, 6, 14, 15, },
  { 0, 2, 4, 6, 14, 15, },
  { 1, 2, 4, 6, 14, 15, },
  { 0, 1, 2, 4, 6, 14, 15, },
  { 3, 4, 6, 14, 15, },
  { 0, 3, 4, 6, 14, 15, },
  { 1, 3, 4, 6, 14, 15, },
  { 0, 1, 3, 4, 6, 14, 15, },
  { 2, 3, 4, 6, 14, 15, },
  { 0, 2, 3, 4, 6, 14, 15, },
  { 1, 2, 3, 4, 6, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 14, 15, },
  { 5, 6, 14, 15, },
  { 0, 5, 6, 14, 15, },
  { 1, 5, 6, 14, 15, },
  { 0, 1, 5, 6, 14, 15, },
  { 2, 5, 6, 14, 15, },
  { 0, 2, 5, 6, 14, 15, },
  { 1, 2, 5, 6, 14, 15, },
  { 0, 1, 2, 5, 6, 14, 15, },
  { 3, 5, 6, 14, 15, },
  { 0, 3, 5, 6, 14, 15, },
  { 1, 3, 5, 6, 14, 15, },
  { 0, 1, 3, 5, 6, 14, 15, },
  { 2, 3, 5, 6, 14, 15, },
  { 0, 2, 3, 5, 6, 14, 15, },
  { 1, 2, 3, 5, 6, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 14, 15, },
  { 4, 5, 6, 14, 15, },
  { 0, 4, 5, 6, 14, 15, },
  { 1, 4, 5, 6, 14, 15, },
  { 0, 1, 4, 5, 6, 14, 15, },
  { 2, 4, 5, 6, 14, 15, },
  { 0, 2, 4, 5, 6, 14, 15, },
  { 1, 2, 4, 5, 6, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 14, 15, },
  { 3, 4, 5, 6, 14, 15, },
  { 0, 3, 4, 5, 6, 14, 15, },
  { 1, 3, 4, 5, 6, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 14, 15, },
  { 2, 3, 4, 5, 6, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 14, 15, },
  { 7, 14, 15, },
  { 0, 7, 14, 15, },
  { 1, 7, 14, 15, },
  { 0, 1, 7, 14, 15, },
  { 2, 7, 14, 15, },
  { 0, 2, 7, 14, 15, },
  { 1, 2, 7, 14, 15, },
  { 0, 1, 2, 7, 14, 15, },
  { 3, 7, 14, 15, },
  { 0, 3, 7, 14, 15, },
  { 1, 3, 7, 14, 15, },
  { 0, 1, 3, 7, 14, 15, },
  { 2, 3, 7, 14, 15, },
  { 0, 2, 3, 7, 14, 15, },
  { 1, 2, 3, 7, 14, 15, },
  { 0, 1, 2, 3, 7, 14, 15, },
  { 4, 7, 14, 15, },
  { 0, 4, 7, 14, 15, },
  { 1, 4, 7, 14, 15, },
  { 0, 1, 4, 7, 14, 15, },
  { 2, 4, 7, 14, 15, },
  { 0, 2, 4, 7, 14, 15, },
  { 1, 2, 4, 7, 14, 15, },
  { 0, 1, 2, 4, 7, 14, 15, },
  { 3, 4, 7, 14, 15, },
  { 0, 3, 4, 7, 14, 15, },
  { 1, 3, 4, 7, 14, 15, },
  { 0, 1, 3, 4, 7, 14, 15, },
  { 2, 3, 4, 7, 14, 15, },
  { 0, 2, 3, 4, 7, 14, 15, },
  { 1, 2, 3, 4, 7, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 14, 15, },
  { 5, 7, 14, 15, },
  { 0, 5, 7, 14, 15, },
  { 1, 5, 7, 14, 15, },
  { 0, 1, 5, 7, 14, 15, },
  { 2, 5, 7, 14, 15, },
  { 0, 2, 5, 7, 14, 15, },
  { 1, 2, 5, 7, 14, 15, },
  { 0, 1, 2, 5, 7, 14, 15, },
  { 3, 5, 7, 14, 15, },
  { 0, 3, 5, 7, 14, 15, },
  { 1, 3, 5, 7, 14, 15, },
  { 0, 1, 3, 5, 7, 14, 15, },
  { 2, 3, 5, 7, 14, 15, },
  { 0, 2, 3, 5, 7, 14, 15, },
  { 1, 2, 3, 5, 7, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 14, 15, },
  { 4, 5, 7, 14, 15, },
  { 0, 4, 5, 7, 14, 15, },
  { 1, 4, 5, 7, 14, 15, },
  { 0, 1, 4, 5, 7, 14, 15, },
  { 2, 4, 5, 7, 14, 15, },
  { 0, 2, 4, 5, 7, 14, 15, },
  { 1, 2, 4, 5, 7, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 14, 15, },
  { 3, 4, 5, 7, 14, 15, },
  { 0, 3, 4, 5, 7, 14, 15, },
  { 1, 3, 4, 5, 7, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 14, 15, },
  { 2, 3, 4, 5, 7, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 14, 15, },
  { 6, 7, 14, 15, },
  { 0, 6, 7, 14, 15, },
  { 1, 6, 7, 14, 15, },
  { 0, 1, 6, 7, 14, 15, },
  { 2, 6, 7, 14, 15, },
  { 0, 2, 6, 7, 14, 15, },
  { 1, 2, 6, 7, 14, 15, },
  { 0, 1, 2, 6, 7, 14, 15, },
  { 3, 6, 7, 14, 15, },
  { 0, 3, 6, 7, 14, 15, },
  { 1, 3, 6, 7, 14, 15, },
  { 0, 1, 3, 6, 7, 14, 15, },
  { 2, 3, 6, 7, 14, 15, },
  { 0, 2, 3, 6, 7, 14, 15, },
  { 1, 2, 3, 6, 7, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 14, 15, },
  { 4, 6, 7, 14, 15, },
  { 0, 4, 6, 7, 14, 15, },
  { 1, 4, 6, 7, 14, 15, },
  { 0, 1, 4, 6, 7, 14, 15, },
  { 2, 4, 6, 7, 14, 15, },
  { 0, 2, 4, 6, 7, 14, 15, },
  { 1, 2, 4, 6, 7, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 14, 15, },
  { 3, 4, 6, 7, 14, 15, },
  { 0, 3, 4, 6, 7, 14, 15, },
  { 1, 3, 4, 6, 7, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 14, 15, },
  { 2, 3, 4, 6, 7, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 14, 15, },
  { 5, 6, 7, 14, 15, },
  { 0, 5, 6, 7, 14, 15, },
  { 1, 5, 6, 7, 14, 15, },
  { 0, 1, 5, 6, 7, 14, 15, },
  { 2, 5, 6, 7, 14, 15, },
  { 0, 2, 5, 6, 7, 14, 15, },
  { 1, 2, 5, 6, 7, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 14, 15, },
  { 3, 5, 6, 7, 14, 15, },
  { 0, 3, 5, 6, 7, 14, 15, },
  { 1, 3, 5, 6, 7, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 14, 15, },
  { 2, 3, 5, 6, 7, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 14, 15, },
  { 4, 5, 6, 7, 14, 15, },
  { 0, 4, 5, 6, 7, 14, 15, },
  { 1, 4, 5, 6, 7, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 14, 15, },
  { 2, 4, 5, 6, 7, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 14, 15, },
  { 3, 4, 5, 6, 7, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 14, 15, },
  { 8, 14, 15, },
  { 0, 8, 14, 15, },
  { 1, 8, 14, 15, },
  { 0, 1, 8, 14, 15, },
  { 2, 8, 14, 15, },
  { 0, 2, 8, 14, 15, },
  { 1, 2, 8, 14, 15, },
  { 0, 1, 2, 8, 14, 15, },
  { 3, 8, 14, 15, },
  { 0, 3, 8, 14, 15, },
  { 1, 3, 8, 14, 15, },
  { 0, 1, 3, 8, 14, 15, },
  { 2, 3, 8, 14, 15, },
  { 0, 2, 3, 8, 14, 15, },
  { 1, 2, 3, 8, 14, 15, },
  { 0, 1, 2, 3, 8, 14, 15, },
  { 4, 8, 14, 15, },
  { 0, 4, 8, 14, 15, },
  { 1, 4, 8, 14, 15, },
  { 0, 1, 4, 8, 14, 15, },
  { 2, 4, 8, 14, 15, },
  { 0, 2, 4, 8, 14, 15, },
  { 1, 2, 4, 8, 14, 15, },
  { 0, 1, 2, 4, 8, 14, 15, },
  { 3, 4, 8, 14, 15, },
  { 0, 3, 4, 8, 14, 15, },
  { 1, 3, 4, 8, 14, 15, },
  { 0, 1, 3, 4, 8, 14, 15, },
  { 2, 3, 4, 8, 14, 15, },
  { 0, 2, 3, 4, 8, 14, 15, },
  { 1, 2, 3, 4, 8, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 14, 15, },
  { 5, 8, 14, 15, },
  { 0, 5, 8, 14, 15, },
  { 1, 5, 8, 14, 15, },
  { 0, 1, 5, 8, 14, 15, },
  { 2, 5, 8, 14, 15, },
  { 0, 2, 5, 8, 14, 15, },
  { 1, 2, 5, 8, 14, 15, },
  { 0, 1, 2, 5, 8, 14, 15, },
  { 3, 5, 8, 14, 15, },
  { 0, 3, 5, 8, 14, 15, },
  { 1, 3, 5, 8, 14, 15, },
  { 0, 1, 3, 5, 8, 14, 15, },
  { 2, 3, 5, 8, 14, 15, },
  { 0, 2, 3, 5, 8, 14, 15, },
  { 1, 2, 3, 5, 8, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 14, 15, },
  { 4, 5, 8, 14, 15, },
  { 0, 4, 5, 8, 14, 15, },
  { 1, 4, 5, 8, 14, 15, },
  { 0, 1, 4, 5, 8, 14, 15, },
  { 2, 4, 5, 8, 14, 15, },
  { 0, 2, 4, 5, 8, 14, 15, },
  { 1, 2, 4, 5, 8, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 14, 15, },
  { 3, 4, 5, 8, 14, 15, },
  { 0, 3, 4, 5, 8, 14, 15, },
  { 1, 3, 4, 5, 8, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 14, 15, },
  { 2, 3, 4, 5, 8, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 14, 15, },
  { 6, 8, 14, 15, },
  { 0, 6, 8, 14, 15, },
  { 1, 6, 8, 14, 15, },
  { 0, 1, 6, 8, 14, 15, },
  { 2, 6, 8, 14, 15, },
  { 0, 2, 6, 8, 14, 15, },
  { 1, 2, 6, 8, 14, 15, },
  { 0, 1, 2, 6, 8, 14, 15, },
  { 3, 6, 8, 14, 15, },
  { 0, 3, 6, 8, 14, 15, },
  { 1, 3, 6, 8, 14, 15, },
  { 0, 1, 3, 6, 8, 14, 15, },
  { 2, 3, 6, 8, 14, 15, },
  { 0, 2, 3, 6, 8, 14, 15, },
  { 1, 2, 3, 6, 8, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 14, 15, },
  { 4, 6, 8, 14, 15, },
  { 0, 4, 6, 8, 14, 15, },
  { 1, 4, 6, 8, 14, 15, },
  { 0, 1, 4, 6, 8, 14, 15, },
  { 2, 4, 6, 8, 14, 15, },
  { 0, 2, 4, 6, 8, 14, 15, },
  { 1, 2, 4, 6, 8, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 14, 15, },
  { 3, 4, 6, 8, 14, 15, },
  { 0, 3, 4, 6, 8, 14, 15, },
  { 1, 3, 4, 6, 8, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 14, 15, },
  { 2, 3, 4, 6, 8, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 14, 15, },
  { 5, 6, 8, 14, 15, },
  { 0, 5, 6, 8, 14, 15, },
  { 1, 5, 6, 8, 14, 15, },
  { 0, 1, 5, 6, 8, 14, 15, },
  { 2, 5, 6, 8, 14, 15, },
  { 0, 2, 5, 6, 8, 14, 15, },
  { 1, 2, 5, 6, 8, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 14, 15, },
  { 3, 5, 6, 8, 14, 15, },
  { 0, 3, 5, 6, 8, 14, 15, },
  { 1, 3, 5, 6, 8, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 14, 15, },
  { 2, 3, 5, 6, 8, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 14, 15, },
  { 4, 5, 6, 8, 14, 15, },
  { 0, 4, 5, 6, 8, 14, 15, },
  { 1, 4, 5, 6, 8, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 14, 15, },
  { 2, 4, 5, 6, 8, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 14, 15, },
  { 3, 4, 5, 6, 8, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 14, 15, },
  { 7, 8, 14, 15, },
  { 0, 7, 8, 14, 15, },
  { 1, 7, 8, 14, 15, },
  { 0, 1, 7, 8, 14, 15, },
  { 2, 7, 8, 14, 15, },
  { 0, 2, 7, 8, 14, 15, },
  { 1, 2, 7, 8, 14, 15, },
  { 0, 1, 2, 7, 8, 14, 15, },
  { 3, 7, 8, 14, 15, },
  { 0, 3, 7, 8, 14, 15, },
  { 1, 3, 7, 8, 14, 15, },
  { 0, 1, 3, 7, 8, 14, 15, },
  { 2, 3, 7, 8, 14, 15, },
  { 0, 2, 3, 7, 8, 14, 15, },
  { 1, 2, 3, 7, 8, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 14, 15, },
  { 4, 7, 8, 14, 15, },
  { 0, 4, 7, 8, 14, 15, },
  { 1, 4, 7, 8, 14, 15, },
  { 0, 1, 4, 7, 8, 14, 15, },
  { 2, 4, 7, 8, 14, 15, },
  { 0, 2, 4, 7, 8, 14, 15, },
  { 1, 2, 4, 7, 8, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 14, 15, },
  { 3, 4, 7, 8, 14, 15, },
  { 0, 3, 4, 7, 8, 14, 15, },
  { 1, 3, 4, 7, 8, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 14, 15, },
  { 2, 3, 4, 7, 8, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 14, 15, },
  { 5, 7, 8, 14, 15, },
  { 0, 5, 7, 8, 14, 15, },
  { 1, 5, 7, 8, 14, 15, },
  { 0, 1, 5, 7, 8, 14, 15, },
  { 2, 5, 7, 8, 14, 15, },
  { 0, 2, 5, 7, 8, 14, 15, },
  { 1, 2, 5, 7, 8, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 14, 15, },
  { 3, 5, 7, 8, 14, 15, },
  { 0, 3, 5, 7, 8, 14, 15, },
  { 1, 3, 5, 7, 8, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 14, 15, },
  { 2, 3, 5, 7, 8, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 14, 15, },
  { 4, 5, 7, 8, 14, 15, },
  { 0, 4, 5, 7, 8, 14, 15, },
  { 1, 4, 5, 7, 8, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 14, 15, },
  { 2, 4, 5, 7, 8, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 14, 15, },
  { 3, 4, 5, 7, 8, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 14, 15, },
  { 6, 7, 8, 14, 15, },
  { 0, 6, 7, 8, 14, 15, },
  { 1, 6, 7, 8, 14, 15, },
  { 0, 1, 6, 7, 8, 14, 15, },
  { 2, 6, 7, 8, 14, 15, },
  { 0, 2, 6, 7, 8, 14, 15, },
  { 1, 2, 6, 7, 8, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 14, 15, },
  { 3, 6, 7, 8, 14, 15, },
  { 0, 3, 6, 7, 8, 14, 15, },
  { 1, 3, 6, 7, 8, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 14, 15, },
  { 2, 3, 6, 7, 8, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 14, 15, },
  { 4, 6, 7, 8, 14, 15, },
  { 0, 4, 6, 7, 8, 14, 15, },
  { 1, 4, 6, 7, 8, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 14, 15, },
  { 2, 4, 6, 7, 8, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 14, 15, },
  { 3, 4, 6, 7, 8, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 14, 15, },
  { 5, 6, 7, 8, 14, 15, },
  { 0, 5, 6, 7, 8, 14, 15, },
  { 1, 5, 6, 7, 8, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 14, 15, },
  { 2, 5, 6, 7, 8, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 14, 15, },
  { 3, 5, 6, 7, 8, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 14, 15, },
  { 4, 5, 6, 7, 8, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 14, 15, },
  { 9, 14, 15, },
  { 0, 9, 14, 15, },
  { 1, 9, 14, 15, },
  { 0, 1, 9, 14, 15, },
  { 2, 9, 14, 15, },
  { 0, 2, 9, 14, 15, },
  { 1, 2, 9, 14, 15, },
  { 0, 1, 2, 9, 14, 15, },
  { 3, 9, 14, 15, },
  { 0, 3, 9, 14, 15, },
  { 1, 3, 9, 14, 15, },
  { 0, 1, 3, 9, 14, 15, },
  { 2, 3, 9, 14, 15, },
  { 0, 2, 3, 9, 14, 15, },
  { 1, 2, 3, 9, 14, 15, },
  { 0, 1, 2, 3, 9, 14, 15, },
  { 4, 9, 14, 15, },
  { 0, 4, 9, 14, 15, },
  { 1, 4, 9, 14, 15, },
  { 0, 1, 4, 9, 14, 15, },
  { 2, 4, 9, 14, 15, },
  { 0, 2, 4, 9, 14, 15, },
  { 1, 2, 4, 9, 14, 15, },
  { 0, 1, 2, 4, 9, 14, 15, },
  { 3, 4, 9, 14, 15, },
  { 0, 3, 4, 9, 14, 15, },
  { 1, 3, 4, 9, 14, 15, },
  { 0, 1, 3, 4, 9, 14, 15, },
  { 2, 3, 4, 9, 14, 15, },
  { 0, 2, 3, 4, 9, 14, 15, },
  { 1, 2, 3, 4, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 14, 15, },
  { 5, 9, 14, 15, },
  { 0, 5, 9, 14, 15, },
  { 1, 5, 9, 14, 15, },
  { 0, 1, 5, 9, 14, 15, },
  { 2, 5, 9, 14, 15, },
  { 0, 2, 5, 9, 14, 15, },
  { 1, 2, 5, 9, 14, 15, },
  { 0, 1, 2, 5, 9, 14, 15, },
  { 3, 5, 9, 14, 15, },
  { 0, 3, 5, 9, 14, 15, },
  { 1, 3, 5, 9, 14, 15, },
  { 0, 1, 3, 5, 9, 14, 15, },
  { 2, 3, 5, 9, 14, 15, },
  { 0, 2, 3, 5, 9, 14, 15, },
  { 1, 2, 3, 5, 9, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 14, 15, },
  { 4, 5, 9, 14, 15, },
  { 0, 4, 5, 9, 14, 15, },
  { 1, 4, 5, 9, 14, 15, },
  { 0, 1, 4, 5, 9, 14, 15, },
  { 2, 4, 5, 9, 14, 15, },
  { 0, 2, 4, 5, 9, 14, 15, },
  { 1, 2, 4, 5, 9, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 14, 15, },
  { 3, 4, 5, 9, 14, 15, },
  { 0, 3, 4, 5, 9, 14, 15, },
  { 1, 3, 4, 5, 9, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 14, 15, },
  { 2, 3, 4, 5, 9, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 14, 15, },
  { 6, 9, 14, 15, },
  { 0, 6, 9, 14, 15, },
  { 1, 6, 9, 14, 15, },
  { 0, 1, 6, 9, 14, 15, },
  { 2, 6, 9, 14, 15, },
  { 0, 2, 6, 9, 14, 15, },
  { 1, 2, 6, 9, 14, 15, },
  { 0, 1, 2, 6, 9, 14, 15, },
  { 3, 6, 9, 14, 15, },
  { 0, 3, 6, 9, 14, 15, },
  { 1, 3, 6, 9, 14, 15, },
  { 0, 1, 3, 6, 9, 14, 15, },
  { 2, 3, 6, 9, 14, 15, },
  { 0, 2, 3, 6, 9, 14, 15, },
  { 1, 2, 3, 6, 9, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 14, 15, },
  { 4, 6, 9, 14, 15, },
  { 0, 4, 6, 9, 14, 15, },
  { 1, 4, 6, 9, 14, 15, },
  { 0, 1, 4, 6, 9, 14, 15, },
  { 2, 4, 6, 9, 14, 15, },
  { 0, 2, 4, 6, 9, 14, 15, },
  { 1, 2, 4, 6, 9, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 14, 15, },
  { 3, 4, 6, 9, 14, 15, },
  { 0, 3, 4, 6, 9, 14, 15, },
  { 1, 3, 4, 6, 9, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 14, 15, },
  { 2, 3, 4, 6, 9, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 14, 15, },
  { 5, 6, 9, 14, 15, },
  { 0, 5, 6, 9, 14, 15, },
  { 1, 5, 6, 9, 14, 15, },
  { 0, 1, 5, 6, 9, 14, 15, },
  { 2, 5, 6, 9, 14, 15, },
  { 0, 2, 5, 6, 9, 14, 15, },
  { 1, 2, 5, 6, 9, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 14, 15, },
  { 3, 5, 6, 9, 14, 15, },
  { 0, 3, 5, 6, 9, 14, 15, },
  { 1, 3, 5, 6, 9, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 14, 15, },
  { 2, 3, 5, 6, 9, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 14, 15, },
  { 4, 5, 6, 9, 14, 15, },
  { 0, 4, 5, 6, 9, 14, 15, },
  { 1, 4, 5, 6, 9, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 14, 15, },
  { 2, 4, 5, 6, 9, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 14, 15, },
  { 3, 4, 5, 6, 9, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 14, 15, },
  { 7, 9, 14, 15, },
  { 0, 7, 9, 14, 15, },
  { 1, 7, 9, 14, 15, },
  { 0, 1, 7, 9, 14, 15, },
  { 2, 7, 9, 14, 15, },
  { 0, 2, 7, 9, 14, 15, },
  { 1, 2, 7, 9, 14, 15, },
  { 0, 1, 2, 7, 9, 14, 15, },
  { 3, 7, 9, 14, 15, },
  { 0, 3, 7, 9, 14, 15, },
  { 1, 3, 7, 9, 14, 15, },
  { 0, 1, 3, 7, 9, 14, 15, },
  { 2, 3, 7, 9, 14, 15, },
  { 0, 2, 3, 7, 9, 14, 15, },
  { 1, 2, 3, 7, 9, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 14, 15, },
  { 4, 7, 9, 14, 15, },
  { 0, 4, 7, 9, 14, 15, },
  { 1, 4, 7, 9, 14, 15, },
  { 0, 1, 4, 7, 9, 14, 15, },
  { 2, 4, 7, 9, 14, 15, },
  { 0, 2, 4, 7, 9, 14, 15, },
  { 1, 2, 4, 7, 9, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 14, 15, },
  { 3, 4, 7, 9, 14, 15, },
  { 0, 3, 4, 7, 9, 14, 15, },
  { 1, 3, 4, 7, 9, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 14, 15, },
  { 2, 3, 4, 7, 9, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 14, 15, },
  { 5, 7, 9, 14, 15, },
  { 0, 5, 7, 9, 14, 15, },
  { 1, 5, 7, 9, 14, 15, },
  { 0, 1, 5, 7, 9, 14, 15, },
  { 2, 5, 7, 9, 14, 15, },
  { 0, 2, 5, 7, 9, 14, 15, },
  { 1, 2, 5, 7, 9, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 14, 15, },
  { 3, 5, 7, 9, 14, 15, },
  { 0, 3, 5, 7, 9, 14, 15, },
  { 1, 3, 5, 7, 9, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 14, 15, },
  { 2, 3, 5, 7, 9, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 14, 15, },
  { 4, 5, 7, 9, 14, 15, },
  { 0, 4, 5, 7, 9, 14, 15, },
  { 1, 4, 5, 7, 9, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 14, 15, },
  { 2, 4, 5, 7, 9, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 14, 15, },
  { 3, 4, 5, 7, 9, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 14, 15, },
  { 6, 7, 9, 14, 15, },
  { 0, 6, 7, 9, 14, 15, },
  { 1, 6, 7, 9, 14, 15, },
  { 0, 1, 6, 7, 9, 14, 15, },
  { 2, 6, 7, 9, 14, 15, },
  { 0, 2, 6, 7, 9, 14, 15, },
  { 1, 2, 6, 7, 9, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 14, 15, },
  { 3, 6, 7, 9, 14, 15, },
  { 0, 3, 6, 7, 9, 14, 15, },
  { 1, 3, 6, 7, 9, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 14, 15, },
  { 2, 3, 6, 7, 9, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 14, 15, },
  { 4, 6, 7, 9, 14, 15, },
  { 0, 4, 6, 7, 9, 14, 15, },
  { 1, 4, 6, 7, 9, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 14, 15, },
  { 2, 4, 6, 7, 9, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 14, 15, },
  { 3, 4, 6, 7, 9, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 14, 15, },
  { 5, 6, 7, 9, 14, 15, },
  { 0, 5, 6, 7, 9, 14, 15, },
  { 1, 5, 6, 7, 9, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 14, 15, },
  { 2, 5, 6, 7, 9, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 14, 15, },
  { 3, 5, 6, 7, 9, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 14, 15, },
  { 4, 5, 6, 7, 9, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 14, 15, },
  { 8, 9, 14, 15, },
  { 0, 8, 9, 14, 15, },
  { 1, 8, 9, 14, 15, },
  { 0, 1, 8, 9, 14, 15, },
  { 2, 8, 9, 14, 15, },
  { 0, 2, 8, 9, 14, 15, },
  { 1, 2, 8, 9, 14, 15, },
  { 0, 1, 2, 8, 9, 14, 15, },
  { 3, 8, 9, 14, 15, },
  { 0, 3, 8, 9, 14, 15, },
  { 1, 3, 8, 9, 14, 15, },
  { 0, 1, 3, 8, 9, 14, 15, },
  { 2, 3, 8, 9, 14, 15, },
  { 0, 2, 3, 8, 9, 14, 15, },
  { 1, 2, 3, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 14, 15, },
  { 4, 8, 9, 14, 15, },
  { 0, 4, 8, 9, 14, 15, },
  { 1, 4, 8, 9, 14, 15, },
  { 0, 1, 4, 8, 9, 14, 15, },
  { 2, 4, 8, 9, 14, 15, },
  { 0, 2, 4, 8, 9, 14, 15, },
  { 1, 2, 4, 8, 9, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 14, 15, },
  { 3, 4, 8, 9, 14, 15, },
  { 0, 3, 4, 8, 9, 14, 15, },
  { 1, 3, 4, 8, 9, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 14, 15, },
  { 2, 3, 4, 8, 9, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 14, 15, },
  { 5, 8, 9, 14, 15, },
  { 0, 5, 8, 9, 14, 15, },
  { 1, 5, 8, 9, 14, 15, },
  { 0, 1, 5, 8, 9, 14, 15, },
  { 2, 5, 8, 9, 14, 15, },
  { 0, 2, 5, 8, 9, 14, 15, },
  { 1, 2, 5, 8, 9, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 14, 15, },
  { 3, 5, 8, 9, 14, 15, },
  { 0, 3, 5, 8, 9, 14, 15, },
  { 1, 3, 5, 8, 9, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 14, 15, },
  { 2, 3, 5, 8, 9, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 14, 15, },
  { 4, 5, 8, 9, 14, 15, },
  { 0, 4, 5, 8, 9, 14, 15, },
  { 1, 4, 5, 8, 9, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 14, 15, },
  { 2, 4, 5, 8, 9, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 14, 15, },
  { 3, 4, 5, 8, 9, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 14, 15, },
  { 6, 8, 9, 14, 15, },
  { 0, 6, 8, 9, 14, 15, },
  { 1, 6, 8, 9, 14, 15, },
  { 0, 1, 6, 8, 9, 14, 15, },
  { 2, 6, 8, 9, 14, 15, },
  { 0, 2, 6, 8, 9, 14, 15, },
  { 1, 2, 6, 8, 9, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 14, 15, },
  { 3, 6, 8, 9, 14, 15, },
  { 0, 3, 6, 8, 9, 14, 15, },
  { 1, 3, 6, 8, 9, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 14, 15, },
  { 2, 3, 6, 8, 9, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 14, 15, },
  { 4, 6, 8, 9, 14, 15, },
  { 0, 4, 6, 8, 9, 14, 15, },
  { 1, 4, 6, 8, 9, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 14, 15, },
  { 2, 4, 6, 8, 9, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 14, 15, },
  { 3, 4, 6, 8, 9, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 14, 15, },
  { 5, 6, 8, 9, 14, 15, },
  { 0, 5, 6, 8, 9, 14, 15, },
  { 1, 5, 6, 8, 9, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 14, 15, },
  { 2, 5, 6, 8, 9, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 14, 15, },
  { 3, 5, 6, 8, 9, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 14, 15, },
  { 4, 5, 6, 8, 9, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 14, 15, },
  { 7, 8, 9, 14, 15, },
  { 0, 7, 8, 9, 14, 15, },
  { 1, 7, 8, 9, 14, 15, },
  { 0, 1, 7, 8, 9, 14, 15, },
  { 2, 7, 8, 9, 14, 15, },
  { 0, 2, 7, 8, 9, 14, 15, },
  { 1, 2, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 14, 15, },
  { 3, 7, 8, 9, 14, 15, },
  { 0, 3, 7, 8, 9, 14, 15, },
  { 1, 3, 7, 8, 9, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 14, 15, },
  { 2, 3, 7, 8, 9, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 14, 15, },
  { 4, 7, 8, 9, 14, 15, },
  { 0, 4, 7, 8, 9, 14, 15, },
  { 1, 4, 7, 8, 9, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 14, 15, },
  { 2, 4, 7, 8, 9, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 14, 15, },
  { 3, 4, 7, 8, 9, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 14, 15, },
  { 5, 7, 8, 9, 14, 15, },
  { 0, 5, 7, 8, 9, 14, 15, },
  { 1, 5, 7, 8, 9, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 14, 15, },
  { 2, 5, 7, 8, 9, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 14, 15, },
  { 3, 5, 7, 8, 9, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 14, 15, },
  { 4, 5, 7, 8, 9, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 14, 15, },
  { 6, 7, 8, 9, 14, 15, },
  { 0, 6, 7, 8, 9, 14, 15, },
  { 1, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 14, 15, },
  { 2, 6, 7, 8, 9, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 14, 15, },
  { 3, 6, 7, 8, 9, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 14, 15, },
  { 4, 6, 7, 8, 9, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 14, 15, },
  { 5, 6, 7, 8, 9, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 14, 15, },
  { 10, 14, 15, },
  { 0, 10, 14, 15, },
  { 1, 10, 14, 15, },
  { 0, 1, 10, 14, 15, },
  { 2, 10, 14, 15, },
  { 0, 2, 10, 14, 15, },
  { 1, 2, 10, 14, 15, },
  { 0, 1, 2, 10, 14, 15, },
  { 3, 10, 14, 15, },
  { 0, 3, 10, 14, 15, },
  { 1, 3, 10, 14, 15, },
  { 0, 1, 3, 10, 14, 15, },
  { 2, 3, 10, 14, 15, },
  { 0, 2, 3, 10, 14, 15, },
  { 1, 2, 3, 10, 14, 15, },
  { 0, 1, 2, 3, 10, 14, 15, },
  { 4, 10, 14, 15, },
  { 0, 4, 10, 14, 15, },
  { 1, 4, 10, 14, 15, },
  { 0, 1, 4, 10, 14, 15, },
  { 2, 4, 10, 14, 15, },
  { 0, 2, 4, 10, 14, 15, },
  { 1, 2, 4, 10, 14, 15, },
  { 0, 1, 2, 4, 10, 14, 15, },
  { 3, 4, 10, 14, 15, },
  { 0, 3, 4, 10, 14, 15, },
  { 1, 3, 4, 10, 14, 15, },
  { 0, 1, 3, 4, 10, 14, 15, },
  { 2, 3, 4, 10, 14, 15, },
  { 0, 2, 3, 4, 10, 14, 15, },
  { 1, 2, 3, 4, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 10, 14, 15, },
  { 5, 10, 14, 15, },
  { 0, 5, 10, 14, 15, },
  { 1, 5, 10, 14, 15, },
  { 0, 1, 5, 10, 14, 15, },
  { 2, 5, 10, 14, 15, },
  { 0, 2, 5, 10, 14, 15, },
  { 1, 2, 5, 10, 14, 15, },
  { 0, 1, 2, 5, 10, 14, 15, },
  { 3, 5, 10, 14, 15, },
  { 0, 3, 5, 10, 14, 15, },
  { 1, 3, 5, 10, 14, 15, },
  { 0, 1, 3, 5, 10, 14, 15, },
  { 2, 3, 5, 10, 14, 15, },
  { 0, 2, 3, 5, 10, 14, 15, },
  { 1, 2, 3, 5, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 10, 14, 15, },
  { 4, 5, 10, 14, 15, },
  { 0, 4, 5, 10, 14, 15, },
  { 1, 4, 5, 10, 14, 15, },
  { 0, 1, 4, 5, 10, 14, 15, },
  { 2, 4, 5, 10, 14, 15, },
  { 0, 2, 4, 5, 10, 14, 15, },
  { 1, 2, 4, 5, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 10, 14, 15, },
  { 3, 4, 5, 10, 14, 15, },
  { 0, 3, 4, 5, 10, 14, 15, },
  { 1, 3, 4, 5, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 10, 14, 15, },
  { 2, 3, 4, 5, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 14, 15, },
  { 6, 10, 14, 15, },
  { 0, 6, 10, 14, 15, },
  { 1, 6, 10, 14, 15, },
  { 0, 1, 6, 10, 14, 15, },
  { 2, 6, 10, 14, 15, },
  { 0, 2, 6, 10, 14, 15, },
  { 1, 2, 6, 10, 14, 15, },
  { 0, 1, 2, 6, 10, 14, 15, },
  { 3, 6, 10, 14, 15, },
  { 0, 3, 6, 10, 14, 15, },
  { 1, 3, 6, 10, 14, 15, },
  { 0, 1, 3, 6, 10, 14, 15, },
  { 2, 3, 6, 10, 14, 15, },
  { 0, 2, 3, 6, 10, 14, 15, },
  { 1, 2, 3, 6, 10, 14, 15, },
  { 0, 1, 2, 3, 6, 10, 14, 15, },
  { 4, 6, 10, 14, 15, },
  { 0, 4, 6, 10, 14, 15, },
  { 1, 4, 6, 10, 14, 15, },
  { 0, 1, 4, 6, 10, 14, 15, },
  { 2, 4, 6, 10, 14, 15, },
  { 0, 2, 4, 6, 10, 14, 15, },
  { 1, 2, 4, 6, 10, 14, 15, },
  { 0, 1, 2, 4, 6, 10, 14, 15, },
  { 3, 4, 6, 10, 14, 15, },
  { 0, 3, 4, 6, 10, 14, 15, },
  { 1, 3, 4, 6, 10, 14, 15, },
  { 0, 1, 3, 4, 6, 10, 14, 15, },
  { 2, 3, 4, 6, 10, 14, 15, },
  { 0, 2, 3, 4, 6, 10, 14, 15, },
  { 1, 2, 3, 4, 6, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 14, 15, },
  { 5, 6, 10, 14, 15, },
  { 0, 5, 6, 10, 14, 15, },
  { 1, 5, 6, 10, 14, 15, },
  { 0, 1, 5, 6, 10, 14, 15, },
  { 2, 5, 6, 10, 14, 15, },
  { 0, 2, 5, 6, 10, 14, 15, },
  { 1, 2, 5, 6, 10, 14, 15, },
  { 0, 1, 2, 5, 6, 10, 14, 15, },
  { 3, 5, 6, 10, 14, 15, },
  { 0, 3, 5, 6, 10, 14, 15, },
  { 1, 3, 5, 6, 10, 14, 15, },
  { 0, 1, 3, 5, 6, 10, 14, 15, },
  { 2, 3, 5, 6, 10, 14, 15, },
  { 0, 2, 3, 5, 6, 10, 14, 15, },
  { 1, 2, 3, 5, 6, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 14, 15, },
  { 4, 5, 6, 10, 14, 15, },
  { 0, 4, 5, 6, 10, 14, 15, },
  { 1, 4, 5, 6, 10, 14, 15, },
  { 0, 1, 4, 5, 6, 10, 14, 15, },
  { 2, 4, 5, 6, 10, 14, 15, },
  { 0, 2, 4, 5, 6, 10, 14, 15, },
  { 1, 2, 4, 5, 6, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 14, 15, },
  { 3, 4, 5, 6, 10, 14, 15, },
  { 0, 3, 4, 5, 6, 10, 14, 15, },
  { 1, 3, 4, 5, 6, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 14, 15, },
  { 2, 3, 4, 5, 6, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 14, 15, },
  { 7, 10, 14, 15, },
  { 0, 7, 10, 14, 15, },
  { 1, 7, 10, 14, 15, },
  { 0, 1, 7, 10, 14, 15, },
  { 2, 7, 10, 14, 15, },
  { 0, 2, 7, 10, 14, 15, },
  { 1, 2, 7, 10, 14, 15, },
  { 0, 1, 2, 7, 10, 14, 15, },
  { 3, 7, 10, 14, 15, },
  { 0, 3, 7, 10, 14, 15, },
  { 1, 3, 7, 10, 14, 15, },
  { 0, 1, 3, 7, 10, 14, 15, },
  { 2, 3, 7, 10, 14, 15, },
  { 0, 2, 3, 7, 10, 14, 15, },
  { 1, 2, 3, 7, 10, 14, 15, },
  { 0, 1, 2, 3, 7, 10, 14, 15, },
  { 4, 7, 10, 14, 15, },
  { 0, 4, 7, 10, 14, 15, },
  { 1, 4, 7, 10, 14, 15, },
  { 0, 1, 4, 7, 10, 14, 15, },
  { 2, 4, 7, 10, 14, 15, },
  { 0, 2, 4, 7, 10, 14, 15, },
  { 1, 2, 4, 7, 10, 14, 15, },
  { 0, 1, 2, 4, 7, 10, 14, 15, },
  { 3, 4, 7, 10, 14, 15, },
  { 0, 3, 4, 7, 10, 14, 15, },
  { 1, 3, 4, 7, 10, 14, 15, },
  { 0, 1, 3, 4, 7, 10, 14, 15, },
  { 2, 3, 4, 7, 10, 14, 15, },
  { 0, 2, 3, 4, 7, 10, 14, 15, },
  { 1, 2, 3, 4, 7, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 14, 15, },
  { 5, 7, 10, 14, 15, },
  { 0, 5, 7, 10, 14, 15, },
  { 1, 5, 7, 10, 14, 15, },
  { 0, 1, 5, 7, 10, 14, 15, },
  { 2, 5, 7, 10, 14, 15, },
  { 0, 2, 5, 7, 10, 14, 15, },
  { 1, 2, 5, 7, 10, 14, 15, },
  { 0, 1, 2, 5, 7, 10, 14, 15, },
  { 3, 5, 7, 10, 14, 15, },
  { 0, 3, 5, 7, 10, 14, 15, },
  { 1, 3, 5, 7, 10, 14, 15, },
  { 0, 1, 3, 5, 7, 10, 14, 15, },
  { 2, 3, 5, 7, 10, 14, 15, },
  { 0, 2, 3, 5, 7, 10, 14, 15, },
  { 1, 2, 3, 5, 7, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 14, 15, },
  { 4, 5, 7, 10, 14, 15, },
  { 0, 4, 5, 7, 10, 14, 15, },
  { 1, 4, 5, 7, 10, 14, 15, },
  { 0, 1, 4, 5, 7, 10, 14, 15, },
  { 2, 4, 5, 7, 10, 14, 15, },
  { 0, 2, 4, 5, 7, 10, 14, 15, },
  { 1, 2, 4, 5, 7, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 14, 15, },
  { 3, 4, 5, 7, 10, 14, 15, },
  { 0, 3, 4, 5, 7, 10, 14, 15, },
  { 1, 3, 4, 5, 7, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 14, 15, },
  { 2, 3, 4, 5, 7, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 14, 15, },
  { 6, 7, 10, 14, 15, },
  { 0, 6, 7, 10, 14, 15, },
  { 1, 6, 7, 10, 14, 15, },
  { 0, 1, 6, 7, 10, 14, 15, },
  { 2, 6, 7, 10, 14, 15, },
  { 0, 2, 6, 7, 10, 14, 15, },
  { 1, 2, 6, 7, 10, 14, 15, },
  { 0, 1, 2, 6, 7, 10, 14, 15, },
  { 3, 6, 7, 10, 14, 15, },
  { 0, 3, 6, 7, 10, 14, 15, },
  { 1, 3, 6, 7, 10, 14, 15, },
  { 0, 1, 3, 6, 7, 10, 14, 15, },
  { 2, 3, 6, 7, 10, 14, 15, },
  { 0, 2, 3, 6, 7, 10, 14, 15, },
  { 1, 2, 3, 6, 7, 10, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 14, 15, },
  { 4, 6, 7, 10, 14, 15, },
  { 0, 4, 6, 7, 10, 14, 15, },
  { 1, 4, 6, 7, 10, 14, 15, },
  { 0, 1, 4, 6, 7, 10, 14, 15, },
  { 2, 4, 6, 7, 10, 14, 15, },
  { 0, 2, 4, 6, 7, 10, 14, 15, },
  { 1, 2, 4, 6, 7, 10, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 14, 15, },
  { 3, 4, 6, 7, 10, 14, 15, },
  { 0, 3, 4, 6, 7, 10, 14, 15, },
  { 1, 3, 4, 6, 7, 10, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 14, 15, },
  { 2, 3, 4, 6, 7, 10, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 14, 15, },
  { 5, 6, 7, 10, 14, 15, },
  { 0, 5, 6, 7, 10, 14, 15, },
  { 1, 5, 6, 7, 10, 14, 15, },
  { 0, 1, 5, 6, 7, 10, 14, 15, },
  { 2, 5, 6, 7, 10, 14, 15, },
  { 0, 2, 5, 6, 7, 10, 14, 15, },
  { 1, 2, 5, 6, 7, 10, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 14, 15, },
  { 3, 5, 6, 7, 10, 14, 15, },
  { 0, 3, 5, 6, 7, 10, 14, 15, },
  { 1, 3, 5, 6, 7, 10, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 14, 15, },
  { 2, 3, 5, 6, 7, 10, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 14, 15, },
  { 4, 5, 6, 7, 10, 14, 15, },
  { 0, 4, 5, 6, 7, 10, 14, 15, },
  { 1, 4, 5, 6, 7, 10, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 14, 15, },
  { 2, 4, 5, 6, 7, 10, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 14, 15, },
  { 3, 4, 5, 6, 7, 10, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 14, 15, },
  { 8, 10, 14, 15, },
  { 0, 8, 10, 14, 15, },
  { 1, 8, 10, 14, 15, },
  { 0, 1, 8, 10, 14, 15, },
  { 2, 8, 10, 14, 15, },
  { 0, 2, 8, 10, 14, 15, },
  { 1, 2, 8, 10, 14, 15, },
  { 0, 1, 2, 8, 10, 14, 15, },
  { 3, 8, 10, 14, 15, },
  { 0, 3, 8, 10, 14, 15, },
  { 1, 3, 8, 10, 14, 15, },
  { 0, 1, 3, 8, 10, 14, 15, },
  { 2, 3, 8, 10, 14, 15, },
  { 0, 2, 3, 8, 10, 14, 15, },
  { 1, 2, 3, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 8, 10, 14, 15, },
  { 4, 8, 10, 14, 15, },
  { 0, 4, 8, 10, 14, 15, },
  { 1, 4, 8, 10, 14, 15, },
  { 0, 1, 4, 8, 10, 14, 15, },
  { 2, 4, 8, 10, 14, 15, },
  { 0, 2, 4, 8, 10, 14, 15, },
  { 1, 2, 4, 8, 10, 14, 15, },
  { 0, 1, 2, 4, 8, 10, 14, 15, },
  { 3, 4, 8, 10, 14, 15, },
  { 0, 3, 4, 8, 10, 14, 15, },
  { 1, 3, 4, 8, 10, 14, 15, },
  { 0, 1, 3, 4, 8, 10, 14, 15, },
  { 2, 3, 4, 8, 10, 14, 15, },
  { 0, 2, 3, 4, 8, 10, 14, 15, },
  { 1, 2, 3, 4, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 14, 15, },
  { 5, 8, 10, 14, 15, },
  { 0, 5, 8, 10, 14, 15, },
  { 1, 5, 8, 10, 14, 15, },
  { 0, 1, 5, 8, 10, 14, 15, },
  { 2, 5, 8, 10, 14, 15, },
  { 0, 2, 5, 8, 10, 14, 15, },
  { 1, 2, 5, 8, 10, 14, 15, },
  { 0, 1, 2, 5, 8, 10, 14, 15, },
  { 3, 5, 8, 10, 14, 15, },
  { 0, 3, 5, 8, 10, 14, 15, },
  { 1, 3, 5, 8, 10, 14, 15, },
  { 0, 1, 3, 5, 8, 10, 14, 15, },
  { 2, 3, 5, 8, 10, 14, 15, },
  { 0, 2, 3, 5, 8, 10, 14, 15, },
  { 1, 2, 3, 5, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 14, 15, },
  { 4, 5, 8, 10, 14, 15, },
  { 0, 4, 5, 8, 10, 14, 15, },
  { 1, 4, 5, 8, 10, 14, 15, },
  { 0, 1, 4, 5, 8, 10, 14, 15, },
  { 2, 4, 5, 8, 10, 14, 15, },
  { 0, 2, 4, 5, 8, 10, 14, 15, },
  { 1, 2, 4, 5, 8, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 14, 15, },
  { 3, 4, 5, 8, 10, 14, 15, },
  { 0, 3, 4, 5, 8, 10, 14, 15, },
  { 1, 3, 4, 5, 8, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 14, 15, },
  { 2, 3, 4, 5, 8, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 14, 15, },
  { 6, 8, 10, 14, 15, },
  { 0, 6, 8, 10, 14, 15, },
  { 1, 6, 8, 10, 14, 15, },
  { 0, 1, 6, 8, 10, 14, 15, },
  { 2, 6, 8, 10, 14, 15, },
  { 0, 2, 6, 8, 10, 14, 15, },
  { 1, 2, 6, 8, 10, 14, 15, },
  { 0, 1, 2, 6, 8, 10, 14, 15, },
  { 3, 6, 8, 10, 14, 15, },
  { 0, 3, 6, 8, 10, 14, 15, },
  { 1, 3, 6, 8, 10, 14, 15, },
  { 0, 1, 3, 6, 8, 10, 14, 15, },
  { 2, 3, 6, 8, 10, 14, 15, },
  { 0, 2, 3, 6, 8, 10, 14, 15, },
  { 1, 2, 3, 6, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 14, 15, },
  { 4, 6, 8, 10, 14, 15, },
  { 0, 4, 6, 8, 10, 14, 15, },
  { 1, 4, 6, 8, 10, 14, 15, },
  { 0, 1, 4, 6, 8, 10, 14, 15, },
  { 2, 4, 6, 8, 10, 14, 15, },
  { 0, 2, 4, 6, 8, 10, 14, 15, },
  { 1, 2, 4, 6, 8, 10, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 14, 15, },
  { 3, 4, 6, 8, 10, 14, 15, },
  { 0, 3, 4, 6, 8, 10, 14, 15, },
  { 1, 3, 4, 6, 8, 10, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 14, 15, },
  { 2, 3, 4, 6, 8, 10, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 14, 15, },
  { 5, 6, 8, 10, 14, 15, },
  { 0, 5, 6, 8, 10, 14, 15, },
  { 1, 5, 6, 8, 10, 14, 15, },
  { 0, 1, 5, 6, 8, 10, 14, 15, },
  { 2, 5, 6, 8, 10, 14, 15, },
  { 0, 2, 5, 6, 8, 10, 14, 15, },
  { 1, 2, 5, 6, 8, 10, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 14, 15, },
  { 3, 5, 6, 8, 10, 14, 15, },
  { 0, 3, 5, 6, 8, 10, 14, 15, },
  { 1, 3, 5, 6, 8, 10, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 14, 15, },
  { 2, 3, 5, 6, 8, 10, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 14, 15, },
  { 4, 5, 6, 8, 10, 14, 15, },
  { 0, 4, 5, 6, 8, 10, 14, 15, },
  { 1, 4, 5, 6, 8, 10, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 14, 15, },
  { 2, 4, 5, 6, 8, 10, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 14, 15, },
  { 3, 4, 5, 6, 8, 10, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 14, 15, },
  { 7, 8, 10, 14, 15, },
  { 0, 7, 8, 10, 14, 15, },
  { 1, 7, 8, 10, 14, 15, },
  { 0, 1, 7, 8, 10, 14, 15, },
  { 2, 7, 8, 10, 14, 15, },
  { 0, 2, 7, 8, 10, 14, 15, },
  { 1, 2, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 7, 8, 10, 14, 15, },
  { 3, 7, 8, 10, 14, 15, },
  { 0, 3, 7, 8, 10, 14, 15, },
  { 1, 3, 7, 8, 10, 14, 15, },
  { 0, 1, 3, 7, 8, 10, 14, 15, },
  { 2, 3, 7, 8, 10, 14, 15, },
  { 0, 2, 3, 7, 8, 10, 14, 15, },
  { 1, 2, 3, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 14, 15, },
  { 4, 7, 8, 10, 14, 15, },
  { 0, 4, 7, 8, 10, 14, 15, },
  { 1, 4, 7, 8, 10, 14, 15, },
  { 0, 1, 4, 7, 8, 10, 14, 15, },
  { 2, 4, 7, 8, 10, 14, 15, },
  { 0, 2, 4, 7, 8, 10, 14, 15, },
  { 1, 2, 4, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 14, 15, },
  { 3, 4, 7, 8, 10, 14, 15, },
  { 0, 3, 4, 7, 8, 10, 14, 15, },
  { 1, 3, 4, 7, 8, 10, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 14, 15, },
  { 2, 3, 4, 7, 8, 10, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 14, 15, },
  { 5, 7, 8, 10, 14, 15, },
  { 0, 5, 7, 8, 10, 14, 15, },
  { 1, 5, 7, 8, 10, 14, 15, },
  { 0, 1, 5, 7, 8, 10, 14, 15, },
  { 2, 5, 7, 8, 10, 14, 15, },
  { 0, 2, 5, 7, 8, 10, 14, 15, },
  { 1, 2, 5, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 14, 15, },
  { 3, 5, 7, 8, 10, 14, 15, },
  { 0, 3, 5, 7, 8, 10, 14, 15, },
  { 1, 3, 5, 7, 8, 10, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 14, 15, },
  { 2, 3, 5, 7, 8, 10, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 14, 15, },
  { 4, 5, 7, 8, 10, 14, 15, },
  { 0, 4, 5, 7, 8, 10, 14, 15, },
  { 1, 4, 5, 7, 8, 10, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 14, 15, },
  { 2, 4, 5, 7, 8, 10, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 14, 15, },
  { 3, 4, 5, 7, 8, 10, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 14, 15, },
  { 6, 7, 8, 10, 14, 15, },
  { 0, 6, 7, 8, 10, 14, 15, },
  { 1, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 6, 7, 8, 10, 14, 15, },
  { 2, 6, 7, 8, 10, 14, 15, },
  { 0, 2, 6, 7, 8, 10, 14, 15, },
  { 1, 2, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 14, 15, },
  { 3, 6, 7, 8, 10, 14, 15, },
  { 0, 3, 6, 7, 8, 10, 14, 15, },
  { 1, 3, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 14, 15, },
  { 2, 3, 6, 7, 8, 10, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 14, 15, },
  { 4, 6, 7, 8, 10, 14, 15, },
  { 0, 4, 6, 7, 8, 10, 14, 15, },
  { 1, 4, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 14, 15, },
  { 2, 4, 6, 7, 8, 10, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 14, 15, },
  { 3, 4, 6, 7, 8, 10, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 14, 15, },
  { 5, 6, 7, 8, 10, 14, 15, },
  { 0, 5, 6, 7, 8, 10, 14, 15, },
  { 1, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 14, 15, },
  { 2, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 14, 15, },
  { 3, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 14, 15, },
  { 4, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 14, 15, },
  { 9, 10, 14, 15, },
  { 0, 9, 10, 14, 15, },
  { 1, 9, 10, 14, 15, },
  { 0, 1, 9, 10, 14, 15, },
  { 2, 9, 10, 14, 15, },
  { 0, 2, 9, 10, 14, 15, },
  { 1, 2, 9, 10, 14, 15, },
  { 0, 1, 2, 9, 10, 14, 15, },
  { 3, 9, 10, 14, 15, },
  { 0, 3, 9, 10, 14, 15, },
  { 1, 3, 9, 10, 14, 15, },
  { 0, 1, 3, 9, 10, 14, 15, },
  { 2, 3, 9, 10, 14, 15, },
  { 0, 2, 3, 9, 10, 14, 15, },
  { 1, 2, 3, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 9, 10, 14, 15, },
  { 4, 9, 10, 14, 15, },
  { 0, 4, 9, 10, 14, 15, },
  { 1, 4, 9, 10, 14, 15, },
  { 0, 1, 4, 9, 10, 14, 15, },
  { 2, 4, 9, 10, 14, 15, },
  { 0, 2, 4, 9, 10, 14, 15, },
  { 1, 2, 4, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 9, 10, 14, 15, },
  { 3, 4, 9, 10, 14, 15, },
  { 0, 3, 4, 9, 10, 14, 15, },
  { 1, 3, 4, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 9, 10, 14, 15, },
  { 2, 3, 4, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 14, 15, },
  { 5, 9, 10, 14, 15, },
  { 0, 5, 9, 10, 14, 15, },
  { 1, 5, 9, 10, 14, 15, },
  { 0, 1, 5, 9, 10, 14, 15, },
  { 2, 5, 9, 10, 14, 15, },
  { 0, 2, 5, 9, 10, 14, 15, },
  { 1, 2, 5, 9, 10, 14, 15, },
  { 0, 1, 2, 5, 9, 10, 14, 15, },
  { 3, 5, 9, 10, 14, 15, },
  { 0, 3, 5, 9, 10, 14, 15, },
  { 1, 3, 5, 9, 10, 14, 15, },
  { 0, 1, 3, 5, 9, 10, 14, 15, },
  { 2, 3, 5, 9, 10, 14, 15, },
  { 0, 2, 3, 5, 9, 10, 14, 15, },
  { 1, 2, 3, 5, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 14, 15, },
  { 4, 5, 9, 10, 14, 15, },
  { 0, 4, 5, 9, 10, 14, 15, },
  { 1, 4, 5, 9, 10, 14, 15, },
  { 0, 1, 4, 5, 9, 10, 14, 15, },
  { 2, 4, 5, 9, 10, 14, 15, },
  { 0, 2, 4, 5, 9, 10, 14, 15, },
  { 1, 2, 4, 5, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 14, 15, },
  { 3, 4, 5, 9, 10, 14, 15, },
  { 0, 3, 4, 5, 9, 10, 14, 15, },
  { 1, 3, 4, 5, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 14, 15, },
  { 2, 3, 4, 5, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 14, 15, },
  { 6, 9, 10, 14, 15, },
  { 0, 6, 9, 10, 14, 15, },
  { 1, 6, 9, 10, 14, 15, },
  { 0, 1, 6, 9, 10, 14, 15, },
  { 2, 6, 9, 10, 14, 15, },
  { 0, 2, 6, 9, 10, 14, 15, },
  { 1, 2, 6, 9, 10, 14, 15, },
  { 0, 1, 2, 6, 9, 10, 14, 15, },
  { 3, 6, 9, 10, 14, 15, },
  { 0, 3, 6, 9, 10, 14, 15, },
  { 1, 3, 6, 9, 10, 14, 15, },
  { 0, 1, 3, 6, 9, 10, 14, 15, },
  { 2, 3, 6, 9, 10, 14, 15, },
  { 0, 2, 3, 6, 9, 10, 14, 15, },
  { 1, 2, 3, 6, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 14, 15, },
  { 4, 6, 9, 10, 14, 15, },
  { 0, 4, 6, 9, 10, 14, 15, },
  { 1, 4, 6, 9, 10, 14, 15, },
  { 0, 1, 4, 6, 9, 10, 14, 15, },
  { 2, 4, 6, 9, 10, 14, 15, },
  { 0, 2, 4, 6, 9, 10, 14, 15, },
  { 1, 2, 4, 6, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 14, 15, },
  { 3, 4, 6, 9, 10, 14, 15, },
  { 0, 3, 4, 6, 9, 10, 14, 15, },
  { 1, 3, 4, 6, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 14, 15, },
  { 2, 3, 4, 6, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 14, 15, },
  { 5, 6, 9, 10, 14, 15, },
  { 0, 5, 6, 9, 10, 14, 15, },
  { 1, 5, 6, 9, 10, 14, 15, },
  { 0, 1, 5, 6, 9, 10, 14, 15, },
  { 2, 5, 6, 9, 10, 14, 15, },
  { 0, 2, 5, 6, 9, 10, 14, 15, },
  { 1, 2, 5, 6, 9, 10, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 14, 15, },
  { 3, 5, 6, 9, 10, 14, 15, },
  { 0, 3, 5, 6, 9, 10, 14, 15, },
  { 1, 3, 5, 6, 9, 10, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 14, 15, },
  { 2, 3, 5, 6, 9, 10, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 14, 15, },
  { 4, 5, 6, 9, 10, 14, 15, },
  { 0, 4, 5, 6, 9, 10, 14, 15, },
  { 1, 4, 5, 6, 9, 10, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 14, 15, },
  { 2, 4, 5, 6, 9, 10, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 14, 15, },
  { 3, 4, 5, 6, 9, 10, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 14, 15, },
  { 7, 9, 10, 14, 15, },
  { 0, 7, 9, 10, 14, 15, },
  { 1, 7, 9, 10, 14, 15, },
  { 0, 1, 7, 9, 10, 14, 15, },
  { 2, 7, 9, 10, 14, 15, },
  { 0, 2, 7, 9, 10, 14, 15, },
  { 1, 2, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 7, 9, 10, 14, 15, },
  { 3, 7, 9, 10, 14, 15, },
  { 0, 3, 7, 9, 10, 14, 15, },
  { 1, 3, 7, 9, 10, 14, 15, },
  { 0, 1, 3, 7, 9, 10, 14, 15, },
  { 2, 3, 7, 9, 10, 14, 15, },
  { 0, 2, 3, 7, 9, 10, 14, 15, },
  { 1, 2, 3, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 14, 15, },
  { 4, 7, 9, 10, 14, 15, },
  { 0, 4, 7, 9, 10, 14, 15, },
  { 1, 4, 7, 9, 10, 14, 15, },
  { 0, 1, 4, 7, 9, 10, 14, 15, },
  { 2, 4, 7, 9, 10, 14, 15, },
  { 0, 2, 4, 7, 9, 10, 14, 15, },
  { 1, 2, 4, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 14, 15, },
  { 3, 4, 7, 9, 10, 14, 15, },
  { 0, 3, 4, 7, 9, 10, 14, 15, },
  { 1, 3, 4, 7, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 14, 15, },
  { 2, 3, 4, 7, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 14, 15, },
  { 5, 7, 9, 10, 14, 15, },
  { 0, 5, 7, 9, 10, 14, 15, },
  { 1, 5, 7, 9, 10, 14, 15, },
  { 0, 1, 5, 7, 9, 10, 14, 15, },
  { 2, 5, 7, 9, 10, 14, 15, },
  { 0, 2, 5, 7, 9, 10, 14, 15, },
  { 1, 2, 5, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 14, 15, },
  { 3, 5, 7, 9, 10, 14, 15, },
  { 0, 3, 5, 7, 9, 10, 14, 15, },
  { 1, 3, 5, 7, 9, 10, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 14, 15, },
  { 2, 3, 5, 7, 9, 10, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 14, 15, },
  { 4, 5, 7, 9, 10, 14, 15, },
  { 0, 4, 5, 7, 9, 10, 14, 15, },
  { 1, 4, 5, 7, 9, 10, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 14, 15, },
  { 2, 4, 5, 7, 9, 10, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 14, 15, },
  { 3, 4, 5, 7, 9, 10, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 14, 15, },
  { 6, 7, 9, 10, 14, 15, },
  { 0, 6, 7, 9, 10, 14, 15, },
  { 1, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 6, 7, 9, 10, 14, 15, },
  { 2, 6, 7, 9, 10, 14, 15, },
  { 0, 2, 6, 7, 9, 10, 14, 15, },
  { 1, 2, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 14, 15, },
  { 3, 6, 7, 9, 10, 14, 15, },
  { 0, 3, 6, 7, 9, 10, 14, 15, },
  { 1, 3, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 14, 15, },
  { 2, 3, 6, 7, 9, 10, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 14, 15, },
  { 4, 6, 7, 9, 10, 14, 15, },
  { 0, 4, 6, 7, 9, 10, 14, 15, },
  { 1, 4, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 14, 15, },
  { 2, 4, 6, 7, 9, 10, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 14, 15, },
  { 3, 4, 6, 7, 9, 10, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 14, 15, },
  { 5, 6, 7, 9, 10, 14, 15, },
  { 0, 5, 6, 7, 9, 10, 14, 15, },
  { 1, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 14, 15, },
  { 2, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 14, 15, },
  { 3, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 14, 15, },
  { 4, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 14, 15, },
  { 8, 9, 10, 14, 15, },
  { 0, 8, 9, 10, 14, 15, },
  { 1, 8, 9, 10, 14, 15, },
  { 0, 1, 8, 9, 10, 14, 15, },
  { 2, 8, 9, 10, 14, 15, },
  { 0, 2, 8, 9, 10, 14, 15, },
  { 1, 2, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 8, 9, 10, 14, 15, },
  { 3, 8, 9, 10, 14, 15, },
  { 0, 3, 8, 9, 10, 14, 15, },
  { 1, 3, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 8, 9, 10, 14, 15, },
  { 2, 3, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 14, 15, },
  { 4, 8, 9, 10, 14, 15, },
  { 0, 4, 8, 9, 10, 14, 15, },
  { 1, 4, 8, 9, 10, 14, 15, },
  { 0, 1, 4, 8, 9, 10, 14, 15, },
  { 2, 4, 8, 9, 10, 14, 15, },
  { 0, 2, 4, 8, 9, 10, 14, 15, },
  { 1, 2, 4, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 14, 15, },
  { 3, 4, 8, 9, 10, 14, 15, },
  { 0, 3, 4, 8, 9, 10, 14, 15, },
  { 1, 3, 4, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 14, 15, },
  { 2, 3, 4, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 14, 15, },
  { 5, 8, 9, 10, 14, 15, },
  { 0, 5, 8, 9, 10, 14, 15, },
  { 1, 5, 8, 9, 10, 14, 15, },
  { 0, 1, 5, 8, 9, 10, 14, 15, },
  { 2, 5, 8, 9, 10, 14, 15, },
  { 0, 2, 5, 8, 9, 10, 14, 15, },
  { 1, 2, 5, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 14, 15, },
  { 3, 5, 8, 9, 10, 14, 15, },
  { 0, 3, 5, 8, 9, 10, 14, 15, },
  { 1, 3, 5, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 14, 15, },
  { 2, 3, 5, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 14, 15, },
  { 4, 5, 8, 9, 10, 14, 15, },
  { 0, 4, 5, 8, 9, 10, 14, 15, },
  { 1, 4, 5, 8, 9, 10, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 14, 15, },
  { 2, 4, 5, 8, 9, 10, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 14, 15, },
  { 3, 4, 5, 8, 9, 10, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 14, 15, },
  { 6, 8, 9, 10, 14, 15, },
  { 0, 6, 8, 9, 10, 14, 15, },
  { 1, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 6, 8, 9, 10, 14, 15, },
  { 2, 6, 8, 9, 10, 14, 15, },
  { 0, 2, 6, 8, 9, 10, 14, 15, },
  { 1, 2, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 14, 15, },
  { 3, 6, 8, 9, 10, 14, 15, },
  { 0, 3, 6, 8, 9, 10, 14, 15, },
  { 1, 3, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 14, 15, },
  { 2, 3, 6, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 14, 15, },
  { 4, 6, 8, 9, 10, 14, 15, },
  { 0, 4, 6, 8, 9, 10, 14, 15, },
  { 1, 4, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 14, 15, },
  { 2, 4, 6, 8, 9, 10, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 14, 15, },
  { 3, 4, 6, 8, 9, 10, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 14, 15, },
  { 5, 6, 8, 9, 10, 14, 15, },
  { 0, 5, 6, 8, 9, 10, 14, 15, },
  { 1, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 14, 15, },
  { 2, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 14, 15, },
  { 3, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 14, 15, },
  { 4, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 14, 15, },
  { 7, 8, 9, 10, 14, 15, },
  { 0, 7, 8, 9, 10, 14, 15, },
  { 1, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 7, 8, 9, 10, 14, 15, },
  { 2, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 14, 15, },
  { 3, 7, 8, 9, 10, 14, 15, },
  { 0, 3, 7, 8, 9, 10, 14, 15, },
  { 1, 3, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 14, 15, },
  { 2, 3, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 14, 15, },
  { 4, 7, 8, 9, 10, 14, 15, },
  { 0, 4, 7, 8, 9, 10, 14, 15, },
  { 1, 4, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 14, 15, },
  { 2, 4, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 14, 15, },
  { 3, 4, 7, 8, 9, 10, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 14, 15, },
  { 5, 7, 8, 9, 10, 14, 15, },
  { 0, 5, 7, 8, 9, 10, 14, 15, },
  { 1, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 14, 15, },
  { 2, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 14, 15, },
  { 3, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 14, 15, },
  { 4, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 14, 15, },
  { 6, 7, 8, 9, 10, 14, 15, },
  { 0, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 14, 15, },
  { 2, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 14, 15, },
  { 3, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 14, 15, },
  { 4, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 14, 15, },
  { 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, 15, },
  { 11, 14, 15, },
  { 0, 11, 14, 15, },
  { 1, 11, 14, 15, },
  { 0, 1, 11, 14, 15, },
  { 2, 11, 14, 15, },
  { 0, 2, 11, 14, 15, },
  { 1, 2, 11, 14, 15, },
  { 0, 1, 2, 11, 14, 15, },
  { 3, 11, 14, 15, },
  { 0, 3, 11, 14, 15, },
  { 1, 3, 11, 14, 15, },
  { 0, 1, 3, 11, 14, 15, },
  { 2, 3, 11, 14, 15, },
  { 0, 2, 3, 11, 14, 15, },
  { 1, 2, 3, 11, 14, 15, },
  { 0, 1, 2, 3, 11, 14, 15, },
  { 4, 11, 14, 15, },
  { 0, 4, 11, 14, 15, },
  { 1, 4, 11, 14, 15, },
  { 0, 1, 4, 11, 14, 15, },
  { 2, 4, 11, 14, 15, },
  { 0, 2, 4, 11, 14, 15, },
  { 1, 2, 4, 11, 14, 15, },
  { 0, 1, 2, 4, 11, 14, 15, },
  { 3, 4, 11, 14, 15, },
  { 0, 3, 4, 11, 14, 15, },
  { 1, 3, 4, 11, 14, 15, },
  { 0, 1, 3, 4, 11, 14, 15, },
  { 2, 3, 4, 11, 14, 15, },
  { 0, 2, 3, 4, 11, 14, 15, },
  { 1, 2, 3, 4, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 11, 14, 15, },
  { 5, 11, 14, 15, },
  { 0, 5, 11, 14, 15, },
  { 1, 5, 11, 14, 15, },
  { 0, 1, 5, 11, 14, 15, },
  { 2, 5, 11, 14, 15, },
  { 0, 2, 5, 11, 14, 15, },
  { 1, 2, 5, 11, 14, 15, },
  { 0, 1, 2, 5, 11, 14, 15, },
  { 3, 5, 11, 14, 15, },
  { 0, 3, 5, 11, 14, 15, },
  { 1, 3, 5, 11, 14, 15, },
  { 0, 1, 3, 5, 11, 14, 15, },
  { 2, 3, 5, 11, 14, 15, },
  { 0, 2, 3, 5, 11, 14, 15, },
  { 1, 2, 3, 5, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 11, 14, 15, },
  { 4, 5, 11, 14, 15, },
  { 0, 4, 5, 11, 14, 15, },
  { 1, 4, 5, 11, 14, 15, },
  { 0, 1, 4, 5, 11, 14, 15, },
  { 2, 4, 5, 11, 14, 15, },
  { 0, 2, 4, 5, 11, 14, 15, },
  { 1, 2, 4, 5, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 11, 14, 15, },
  { 3, 4, 5, 11, 14, 15, },
  { 0, 3, 4, 5, 11, 14, 15, },
  { 1, 3, 4, 5, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 11, 14, 15, },
  { 2, 3, 4, 5, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 11, 14, 15, },
  { 6, 11, 14, 15, },
  { 0, 6, 11, 14, 15, },
  { 1, 6, 11, 14, 15, },
  { 0, 1, 6, 11, 14, 15, },
  { 2, 6, 11, 14, 15, },
  { 0, 2, 6, 11, 14, 15, },
  { 1, 2, 6, 11, 14, 15, },
  { 0, 1, 2, 6, 11, 14, 15, },
  { 3, 6, 11, 14, 15, },
  { 0, 3, 6, 11, 14, 15, },
  { 1, 3, 6, 11, 14, 15, },
  { 0, 1, 3, 6, 11, 14, 15, },
  { 2, 3, 6, 11, 14, 15, },
  { 0, 2, 3, 6, 11, 14, 15, },
  { 1, 2, 3, 6, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 11, 14, 15, },
  { 4, 6, 11, 14, 15, },
  { 0, 4, 6, 11, 14, 15, },
  { 1, 4, 6, 11, 14, 15, },
  { 0, 1, 4, 6, 11, 14, 15, },
  { 2, 4, 6, 11, 14, 15, },
  { 0, 2, 4, 6, 11, 14, 15, },
  { 1, 2, 4, 6, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 11, 14, 15, },
  { 3, 4, 6, 11, 14, 15, },
  { 0, 3, 4, 6, 11, 14, 15, },
  { 1, 3, 4, 6, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 11, 14, 15, },
  { 2, 3, 4, 6, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 11, 14, 15, },
  { 5, 6, 11, 14, 15, },
  { 0, 5, 6, 11, 14, 15, },
  { 1, 5, 6, 11, 14, 15, },
  { 0, 1, 5, 6, 11, 14, 15, },
  { 2, 5, 6, 11, 14, 15, },
  { 0, 2, 5, 6, 11, 14, 15, },
  { 1, 2, 5, 6, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 11, 14, 15, },
  { 3, 5, 6, 11, 14, 15, },
  { 0, 3, 5, 6, 11, 14, 15, },
  { 1, 3, 5, 6, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 11, 14, 15, },
  { 2, 3, 5, 6, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 11, 14, 15, },
  { 4, 5, 6, 11, 14, 15, },
  { 0, 4, 5, 6, 11, 14, 15, },
  { 1, 4, 5, 6, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 11, 14, 15, },
  { 2, 4, 5, 6, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 11, 14, 15, },
  { 3, 4, 5, 6, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 14, 15, },
  { 7, 11, 14, 15, },
  { 0, 7, 11, 14, 15, },
  { 1, 7, 11, 14, 15, },
  { 0, 1, 7, 11, 14, 15, },
  { 2, 7, 11, 14, 15, },
  { 0, 2, 7, 11, 14, 15, },
  { 1, 2, 7, 11, 14, 15, },
  { 0, 1, 2, 7, 11, 14, 15, },
  { 3, 7, 11, 14, 15, },
  { 0, 3, 7, 11, 14, 15, },
  { 1, 3, 7, 11, 14, 15, },
  { 0, 1, 3, 7, 11, 14, 15, },
  { 2, 3, 7, 11, 14, 15, },
  { 0, 2, 3, 7, 11, 14, 15, },
  { 1, 2, 3, 7, 11, 14, 15, },
  { 0, 1, 2, 3, 7, 11, 14, 15, },
  { 4, 7, 11, 14, 15, },
  { 0, 4, 7, 11, 14, 15, },
  { 1, 4, 7, 11, 14, 15, },
  { 0, 1, 4, 7, 11, 14, 15, },
  { 2, 4, 7, 11, 14, 15, },
  { 0, 2, 4, 7, 11, 14, 15, },
  { 1, 2, 4, 7, 11, 14, 15, },
  { 0, 1, 2, 4, 7, 11, 14, 15, },
  { 3, 4, 7, 11, 14, 15, },
  { 0, 3, 4, 7, 11, 14, 15, },
  { 1, 3, 4, 7, 11, 14, 15, },
  { 0, 1, 3, 4, 7, 11, 14, 15, },
  { 2, 3, 4, 7, 11, 14, 15, },
  { 0, 2, 3, 4, 7, 11, 14, 15, },
  { 1, 2, 3, 4, 7, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 11, 14, 15, },
  { 5, 7, 11, 14, 15, },
  { 0, 5, 7, 11, 14, 15, },
  { 1, 5, 7, 11, 14, 15, },
  { 0, 1, 5, 7, 11, 14, 15, },
  { 2, 5, 7, 11, 14, 15, },
  { 0, 2, 5, 7, 11, 14, 15, },
  { 1, 2, 5, 7, 11, 14, 15, },
  { 0, 1, 2, 5, 7, 11, 14, 15, },
  { 3, 5, 7, 11, 14, 15, },
  { 0, 3, 5, 7, 11, 14, 15, },
  { 1, 3, 5, 7, 11, 14, 15, },
  { 0, 1, 3, 5, 7, 11, 14, 15, },
  { 2, 3, 5, 7, 11, 14, 15, },
  { 0, 2, 3, 5, 7, 11, 14, 15, },
  { 1, 2, 3, 5, 7, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 11, 14, 15, },
  { 4, 5, 7, 11, 14, 15, },
  { 0, 4, 5, 7, 11, 14, 15, },
  { 1, 4, 5, 7, 11, 14, 15, },
  { 0, 1, 4, 5, 7, 11, 14, 15, },
  { 2, 4, 5, 7, 11, 14, 15, },
  { 0, 2, 4, 5, 7, 11, 14, 15, },
  { 1, 2, 4, 5, 7, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 11, 14, 15, },
  { 3, 4, 5, 7, 11, 14, 15, },
  { 0, 3, 4, 5, 7, 11, 14, 15, },
  { 1, 3, 4, 5, 7, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 11, 14, 15, },
  { 2, 3, 4, 5, 7, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 14, 15, },
  { 6, 7, 11, 14, 15, },
  { 0, 6, 7, 11, 14, 15, },
  { 1, 6, 7, 11, 14, 15, },
  { 0, 1, 6, 7, 11, 14, 15, },
  { 2, 6, 7, 11, 14, 15, },
  { 0, 2, 6, 7, 11, 14, 15, },
  { 1, 2, 6, 7, 11, 14, 15, },
  { 0, 1, 2, 6, 7, 11, 14, 15, },
  { 3, 6, 7, 11, 14, 15, },
  { 0, 3, 6, 7, 11, 14, 15, },
  { 1, 3, 6, 7, 11, 14, 15, },
  { 0, 1, 3, 6, 7, 11, 14, 15, },
  { 2, 3, 6, 7, 11, 14, 15, },
  { 0, 2, 3, 6, 7, 11, 14, 15, },
  { 1, 2, 3, 6, 7, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 11, 14, 15, },
  { 4, 6, 7, 11, 14, 15, },
  { 0, 4, 6, 7, 11, 14, 15, },
  { 1, 4, 6, 7, 11, 14, 15, },
  { 0, 1, 4, 6, 7, 11, 14, 15, },
  { 2, 4, 6, 7, 11, 14, 15, },
  { 0, 2, 4, 6, 7, 11, 14, 15, },
  { 1, 2, 4, 6, 7, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 11, 14, 15, },
  { 3, 4, 6, 7, 11, 14, 15, },
  { 0, 3, 4, 6, 7, 11, 14, 15, },
  { 1, 3, 4, 6, 7, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 11, 14, 15, },
  { 2, 3, 4, 6, 7, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 14, 15, },
  { 5, 6, 7, 11, 14, 15, },
  { 0, 5, 6, 7, 11, 14, 15, },
  { 1, 5, 6, 7, 11, 14, 15, },
  { 0, 1, 5, 6, 7, 11, 14, 15, },
  { 2, 5, 6, 7, 11, 14, 15, },
  { 0, 2, 5, 6, 7, 11, 14, 15, },
  { 1, 2, 5, 6, 7, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 11, 14, 15, },
  { 3, 5, 6, 7, 11, 14, 15, },
  { 0, 3, 5, 6, 7, 11, 14, 15, },
  { 1, 3, 5, 6, 7, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 11, 14, 15, },
  { 2, 3, 5, 6, 7, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 14, 15, },
  { 4, 5, 6, 7, 11, 14, 15, },
  { 0, 4, 5, 6, 7, 11, 14, 15, },
  { 1, 4, 5, 6, 7, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 11, 14, 15, },
  { 2, 4, 5, 6, 7, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 14, 15, },
  { 3, 4, 5, 6, 7, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 14, 15, },
  { 8, 11, 14, 15, },
  { 0, 8, 11, 14, 15, },
  { 1, 8, 11, 14, 15, },
  { 0, 1, 8, 11, 14, 15, },
  { 2, 8, 11, 14, 15, },
  { 0, 2, 8, 11, 14, 15, },
  { 1, 2, 8, 11, 14, 15, },
  { 0, 1, 2, 8, 11, 14, 15, },
  { 3, 8, 11, 14, 15, },
  { 0, 3, 8, 11, 14, 15, },
  { 1, 3, 8, 11, 14, 15, },
  { 0, 1, 3, 8, 11, 14, 15, },
  { 2, 3, 8, 11, 14, 15, },
  { 0, 2, 3, 8, 11, 14, 15, },
  { 1, 2, 3, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 8, 11, 14, 15, },
  { 4, 8, 11, 14, 15, },
  { 0, 4, 8, 11, 14, 15, },
  { 1, 4, 8, 11, 14, 15, },
  { 0, 1, 4, 8, 11, 14, 15, },
  { 2, 4, 8, 11, 14, 15, },
  { 0, 2, 4, 8, 11, 14, 15, },
  { 1, 2, 4, 8, 11, 14, 15, },
  { 0, 1, 2, 4, 8, 11, 14, 15, },
  { 3, 4, 8, 11, 14, 15, },
  { 0, 3, 4, 8, 11, 14, 15, },
  { 1, 3, 4, 8, 11, 14, 15, },
  { 0, 1, 3, 4, 8, 11, 14, 15, },
  { 2, 3, 4, 8, 11, 14, 15, },
  { 0, 2, 3, 4, 8, 11, 14, 15, },
  { 1, 2, 3, 4, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 11, 14, 15, },
  { 5, 8, 11, 14, 15, },
  { 0, 5, 8, 11, 14, 15, },
  { 1, 5, 8, 11, 14, 15, },
  { 0, 1, 5, 8, 11, 14, 15, },
  { 2, 5, 8, 11, 14, 15, },
  { 0, 2, 5, 8, 11, 14, 15, },
  { 1, 2, 5, 8, 11, 14, 15, },
  { 0, 1, 2, 5, 8, 11, 14, 15, },
  { 3, 5, 8, 11, 14, 15, },
  { 0, 3, 5, 8, 11, 14, 15, },
  { 1, 3, 5, 8, 11, 14, 15, },
  { 0, 1, 3, 5, 8, 11, 14, 15, },
  { 2, 3, 5, 8, 11, 14, 15, },
  { 0, 2, 3, 5, 8, 11, 14, 15, },
  { 1, 2, 3, 5, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 11, 14, 15, },
  { 4, 5, 8, 11, 14, 15, },
  { 0, 4, 5, 8, 11, 14, 15, },
  { 1, 4, 5, 8, 11, 14, 15, },
  { 0, 1, 4, 5, 8, 11, 14, 15, },
  { 2, 4, 5, 8, 11, 14, 15, },
  { 0, 2, 4, 5, 8, 11, 14, 15, },
  { 1, 2, 4, 5, 8, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 11, 14, 15, },
  { 3, 4, 5, 8, 11, 14, 15, },
  { 0, 3, 4, 5, 8, 11, 14, 15, },
  { 1, 3, 4, 5, 8, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 11, 14, 15, },
  { 2, 3, 4, 5, 8, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 14, 15, },
  { 6, 8, 11, 14, 15, },
  { 0, 6, 8, 11, 14, 15, },
  { 1, 6, 8, 11, 14, 15, },
  { 0, 1, 6, 8, 11, 14, 15, },
  { 2, 6, 8, 11, 14, 15, },
  { 0, 2, 6, 8, 11, 14, 15, },
  { 1, 2, 6, 8, 11, 14, 15, },
  { 0, 1, 2, 6, 8, 11, 14, 15, },
  { 3, 6, 8, 11, 14, 15, },
  { 0, 3, 6, 8, 11, 14, 15, },
  { 1, 3, 6, 8, 11, 14, 15, },
  { 0, 1, 3, 6, 8, 11, 14, 15, },
  { 2, 3, 6, 8, 11, 14, 15, },
  { 0, 2, 3, 6, 8, 11, 14, 15, },
  { 1, 2, 3, 6, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 11, 14, 15, },
  { 4, 6, 8, 11, 14, 15, },
  { 0, 4, 6, 8, 11, 14, 15, },
  { 1, 4, 6, 8, 11, 14, 15, },
  { 0, 1, 4, 6, 8, 11, 14, 15, },
  { 2, 4, 6, 8, 11, 14, 15, },
  { 0, 2, 4, 6, 8, 11, 14, 15, },
  { 1, 2, 4, 6, 8, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 11, 14, 15, },
  { 3, 4, 6, 8, 11, 14, 15, },
  { 0, 3, 4, 6, 8, 11, 14, 15, },
  { 1, 3, 4, 6, 8, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 11, 14, 15, },
  { 2, 3, 4, 6, 8, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 14, 15, },
  { 5, 6, 8, 11, 14, 15, },
  { 0, 5, 6, 8, 11, 14, 15, },
  { 1, 5, 6, 8, 11, 14, 15, },
  { 0, 1, 5, 6, 8, 11, 14, 15, },
  { 2, 5, 6, 8, 11, 14, 15, },
  { 0, 2, 5, 6, 8, 11, 14, 15, },
  { 1, 2, 5, 6, 8, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 11, 14, 15, },
  { 3, 5, 6, 8, 11, 14, 15, },
  { 0, 3, 5, 6, 8, 11, 14, 15, },
  { 1, 3, 5, 6, 8, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 11, 14, 15, },
  { 2, 3, 5, 6, 8, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 14, 15, },
  { 4, 5, 6, 8, 11, 14, 15, },
  { 0, 4, 5, 6, 8, 11, 14, 15, },
  { 1, 4, 5, 6, 8, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 11, 14, 15, },
  { 2, 4, 5, 6, 8, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 14, 15, },
  { 3, 4, 5, 6, 8, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 14, 15, },
  { 7, 8, 11, 14, 15, },
  { 0, 7, 8, 11, 14, 15, },
  { 1, 7, 8, 11, 14, 15, },
  { 0, 1, 7, 8, 11, 14, 15, },
  { 2, 7, 8, 11, 14, 15, },
  { 0, 2, 7, 8, 11, 14, 15, },
  { 1, 2, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 7, 8, 11, 14, 15, },
  { 3, 7, 8, 11, 14, 15, },
  { 0, 3, 7, 8, 11, 14, 15, },
  { 1, 3, 7, 8, 11, 14, 15, },
  { 0, 1, 3, 7, 8, 11, 14, 15, },
  { 2, 3, 7, 8, 11, 14, 15, },
  { 0, 2, 3, 7, 8, 11, 14, 15, },
  { 1, 2, 3, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 11, 14, 15, },
  { 4, 7, 8, 11, 14, 15, },
  { 0, 4, 7, 8, 11, 14, 15, },
  { 1, 4, 7, 8, 11, 14, 15, },
  { 0, 1, 4, 7, 8, 11, 14, 15, },
  { 2, 4, 7, 8, 11, 14, 15, },
  { 0, 2, 4, 7, 8, 11, 14, 15, },
  { 1, 2, 4, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 11, 14, 15, },
  { 3, 4, 7, 8, 11, 14, 15, },
  { 0, 3, 4, 7, 8, 11, 14, 15, },
  { 1, 3, 4, 7, 8, 11, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 11, 14, 15, },
  { 2, 3, 4, 7, 8, 11, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 11, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 14, 15, },
  { 5, 7, 8, 11, 14, 15, },
  { 0, 5, 7, 8, 11, 14, 15, },
  { 1, 5, 7, 8, 11, 14, 15, },
  { 0, 1, 5, 7, 8, 11, 14, 15, },
  { 2, 5, 7, 8, 11, 14, 15, },
  { 0, 2, 5, 7, 8, 11, 14, 15, },
  { 1, 2, 5, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 11, 14, 15, },
  { 3, 5, 7, 8, 11, 14, 15, },
  { 0, 3, 5, 7, 8, 11, 14, 15, },
  { 1, 3, 5, 7, 8, 11, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 11, 14, 15, },
  { 2, 3, 5, 7, 8, 11, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 11, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 14, 15, },
  { 4, 5, 7, 8, 11, 14, 15, },
  { 0, 4, 5, 7, 8, 11, 14, 15, },
  { 1, 4, 5, 7, 8, 11, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 11, 14, 15, },
  { 2, 4, 5, 7, 8, 11, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 11, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 14, 15, },
  { 3, 4, 5, 7, 8, 11, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 11, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 14, 15, },
  { 6, 7, 8, 11, 14, 15, },
  { 0, 6, 7, 8, 11, 14, 15, },
  { 1, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 6, 7, 8, 11, 14, 15, },
  { 2, 6, 7, 8, 11, 14, 15, },
  { 0, 2, 6, 7, 8, 11, 14, 15, },
  { 1, 2, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 11, 14, 15, },
  { 3, 6, 7, 8, 11, 14, 15, },
  { 0, 3, 6, 7, 8, 11, 14, 15, },
  { 1, 3, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 11, 14, 15, },
  { 2, 3, 6, 7, 8, 11, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 11, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 14, 15, },
  { 4, 6, 7, 8, 11, 14, 15, },
  { 0, 4, 6, 7, 8, 11, 14, 15, },
  { 1, 4, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 11, 14, 15, },
  { 2, 4, 6, 7, 8, 11, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 11, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 14, 15, },
  { 3, 4, 6, 7, 8, 11, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 11, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 14, 15, },
  { 5, 6, 7, 8, 11, 14, 15, },
  { 0, 5, 6, 7, 8, 11, 14, 15, },
  { 1, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 11, 14, 15, },
  { 2, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 11, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 14, 15, },
  { 3, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 11, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 14, 15, },
  { 4, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 14, 15, },
  { 9, 11, 14, 15, },
  { 0, 9, 11, 14, 15, },
  { 1, 9, 11, 14, 15, },
  { 0, 1, 9, 11, 14, 15, },
  { 2, 9, 11, 14, 15, },
  { 0, 2, 9, 11, 14, 15, },
  { 1, 2, 9, 11, 14, 15, },
  { 0, 1, 2, 9, 11, 14, 15, },
  { 3, 9, 11, 14, 15, },
  { 0, 3, 9, 11, 14, 15, },
  { 1, 3, 9, 11, 14, 15, },
  { 0, 1, 3, 9, 11, 14, 15, },
  { 2, 3, 9, 11, 14, 15, },
  { 0, 2, 3, 9, 11, 14, 15, },
  { 1, 2, 3, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 9, 11, 14, 15, },
  { 4, 9, 11, 14, 15, },
  { 0, 4, 9, 11, 14, 15, },
  { 1, 4, 9, 11, 14, 15, },
  { 0, 1, 4, 9, 11, 14, 15, },
  { 2, 4, 9, 11, 14, 15, },
  { 0, 2, 4, 9, 11, 14, 15, },
  { 1, 2, 4, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 9, 11, 14, 15, },
  { 3, 4, 9, 11, 14, 15, },
  { 0, 3, 4, 9, 11, 14, 15, },
  { 1, 3, 4, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 9, 11, 14, 15, },
  { 2, 3, 4, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 11, 14, 15, },
  { 5, 9, 11, 14, 15, },
  { 0, 5, 9, 11, 14, 15, },
  { 1, 5, 9, 11, 14, 15, },
  { 0, 1, 5, 9, 11, 14, 15, },
  { 2, 5, 9, 11, 14, 15, },
  { 0, 2, 5, 9, 11, 14, 15, },
  { 1, 2, 5, 9, 11, 14, 15, },
  { 0, 1, 2, 5, 9, 11, 14, 15, },
  { 3, 5, 9, 11, 14, 15, },
  { 0, 3, 5, 9, 11, 14, 15, },
  { 1, 3, 5, 9, 11, 14, 15, },
  { 0, 1, 3, 5, 9, 11, 14, 15, },
  { 2, 3, 5, 9, 11, 14, 15, },
  { 0, 2, 3, 5, 9, 11, 14, 15, },
  { 1, 2, 3, 5, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 11, 14, 15, },
  { 4, 5, 9, 11, 14, 15, },
  { 0, 4, 5, 9, 11, 14, 15, },
  { 1, 4, 5, 9, 11, 14, 15, },
  { 0, 1, 4, 5, 9, 11, 14, 15, },
  { 2, 4, 5, 9, 11, 14, 15, },
  { 0, 2, 4, 5, 9, 11, 14, 15, },
  { 1, 2, 4, 5, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 11, 14, 15, },
  { 3, 4, 5, 9, 11, 14, 15, },
  { 0, 3, 4, 5, 9, 11, 14, 15, },
  { 1, 3, 4, 5, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 11, 14, 15, },
  { 2, 3, 4, 5, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 14, 15, },
  { 6, 9, 11, 14, 15, },
  { 0, 6, 9, 11, 14, 15, },
  { 1, 6, 9, 11, 14, 15, },
  { 0, 1, 6, 9, 11, 14, 15, },
  { 2, 6, 9, 11, 14, 15, },
  { 0, 2, 6, 9, 11, 14, 15, },
  { 1, 2, 6, 9, 11, 14, 15, },
  { 0, 1, 2, 6, 9, 11, 14, 15, },
  { 3, 6, 9, 11, 14, 15, },
  { 0, 3, 6, 9, 11, 14, 15, },
  { 1, 3, 6, 9, 11, 14, 15, },
  { 0, 1, 3, 6, 9, 11, 14, 15, },
  { 2, 3, 6, 9, 11, 14, 15, },
  { 0, 2, 3, 6, 9, 11, 14, 15, },
  { 1, 2, 3, 6, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 11, 14, 15, },
  { 4, 6, 9, 11, 14, 15, },
  { 0, 4, 6, 9, 11, 14, 15, },
  { 1, 4, 6, 9, 11, 14, 15, },
  { 0, 1, 4, 6, 9, 11, 14, 15, },
  { 2, 4, 6, 9, 11, 14, 15, },
  { 0, 2, 4, 6, 9, 11, 14, 15, },
  { 1, 2, 4, 6, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 11, 14, 15, },
  { 3, 4, 6, 9, 11, 14, 15, },
  { 0, 3, 4, 6, 9, 11, 14, 15, },
  { 1, 3, 4, 6, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 11, 14, 15, },
  { 2, 3, 4, 6, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 14, 15, },
  { 5, 6, 9, 11, 14, 15, },
  { 0, 5, 6, 9, 11, 14, 15, },
  { 1, 5, 6, 9, 11, 14, 15, },
  { 0, 1, 5, 6, 9, 11, 14, 15, },
  { 2, 5, 6, 9, 11, 14, 15, },
  { 0, 2, 5, 6, 9, 11, 14, 15, },
  { 1, 2, 5, 6, 9, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 11, 14, 15, },
  { 3, 5, 6, 9, 11, 14, 15, },
  { 0, 3, 5, 6, 9, 11, 14, 15, },
  { 1, 3, 5, 6, 9, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 11, 14, 15, },
  { 2, 3, 5, 6, 9, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 14, 15, },
  { 4, 5, 6, 9, 11, 14, 15, },
  { 0, 4, 5, 6, 9, 11, 14, 15, },
  { 1, 4, 5, 6, 9, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 11, 14, 15, },
  { 2, 4, 5, 6, 9, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 14, 15, },
  { 3, 4, 5, 6, 9, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 14, 15, },
  { 7, 9, 11, 14, 15, },
  { 0, 7, 9, 11, 14, 15, },
  { 1, 7, 9, 11, 14, 15, },
  { 0, 1, 7, 9, 11, 14, 15, },
  { 2, 7, 9, 11, 14, 15, },
  { 0, 2, 7, 9, 11, 14, 15, },
  { 1, 2, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 7, 9, 11, 14, 15, },
  { 3, 7, 9, 11, 14, 15, },
  { 0, 3, 7, 9, 11, 14, 15, },
  { 1, 3, 7, 9, 11, 14, 15, },
  { 0, 1, 3, 7, 9, 11, 14, 15, },
  { 2, 3, 7, 9, 11, 14, 15, },
  { 0, 2, 3, 7, 9, 11, 14, 15, },
  { 1, 2, 3, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 11, 14, 15, },
  { 4, 7, 9, 11, 14, 15, },
  { 0, 4, 7, 9, 11, 14, 15, },
  { 1, 4, 7, 9, 11, 14, 15, },
  { 0, 1, 4, 7, 9, 11, 14, 15, },
  { 2, 4, 7, 9, 11, 14, 15, },
  { 0, 2, 4, 7, 9, 11, 14, 15, },
  { 1, 2, 4, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 11, 14, 15, },
  { 3, 4, 7, 9, 11, 14, 15, },
  { 0, 3, 4, 7, 9, 11, 14, 15, },
  { 1, 3, 4, 7, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 11, 14, 15, },
  { 2, 3, 4, 7, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 14, 15, },
  { 5, 7, 9, 11, 14, 15, },
  { 0, 5, 7, 9, 11, 14, 15, },
  { 1, 5, 7, 9, 11, 14, 15, },
  { 0, 1, 5, 7, 9, 11, 14, 15, },
  { 2, 5, 7, 9, 11, 14, 15, },
  { 0, 2, 5, 7, 9, 11, 14, 15, },
  { 1, 2, 5, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 11, 14, 15, },
  { 3, 5, 7, 9, 11, 14, 15, },
  { 0, 3, 5, 7, 9, 11, 14, 15, },
  { 1, 3, 5, 7, 9, 11, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 11, 14, 15, },
  { 2, 3, 5, 7, 9, 11, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 11, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 14, 15, },
  { 4, 5, 7, 9, 11, 14, 15, },
  { 0, 4, 5, 7, 9, 11, 14, 15, },
  { 1, 4, 5, 7, 9, 11, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 11, 14, 15, },
  { 2, 4, 5, 7, 9, 11, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 11, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 14, 15, },
  { 3, 4, 5, 7, 9, 11, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 11, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 14, 15, },
  { 6, 7, 9, 11, 14, 15, },
  { 0, 6, 7, 9, 11, 14, 15, },
  { 1, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 6, 7, 9, 11, 14, 15, },
  { 2, 6, 7, 9, 11, 14, 15, },
  { 0, 2, 6, 7, 9, 11, 14, 15, },
  { 1, 2, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 11, 14, 15, },
  { 3, 6, 7, 9, 11, 14, 15, },
  { 0, 3, 6, 7, 9, 11, 14, 15, },
  { 1, 3, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 11, 14, 15, },
  { 2, 3, 6, 7, 9, 11, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 11, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 14, 15, },
  { 4, 6, 7, 9, 11, 14, 15, },
  { 0, 4, 6, 7, 9, 11, 14, 15, },
  { 1, 4, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 11, 14, 15, },
  { 2, 4, 6, 7, 9, 11, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 11, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 14, 15, },
  { 3, 4, 6, 7, 9, 11, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 11, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 14, 15, },
  { 5, 6, 7, 9, 11, 14, 15, },
  { 0, 5, 6, 7, 9, 11, 14, 15, },
  { 1, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 11, 14, 15, },
  { 2, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 11, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 14, 15, },
  { 3, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 11, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 14, 15, },
  { 4, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 14, 15, },
  { 8, 9, 11, 14, 15, },
  { 0, 8, 9, 11, 14, 15, },
  { 1, 8, 9, 11, 14, 15, },
  { 0, 1, 8, 9, 11, 14, 15, },
  { 2, 8, 9, 11, 14, 15, },
  { 0, 2, 8, 9, 11, 14, 15, },
  { 1, 2, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 8, 9, 11, 14, 15, },
  { 3, 8, 9, 11, 14, 15, },
  { 0, 3, 8, 9, 11, 14, 15, },
  { 1, 3, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 8, 9, 11, 14, 15, },
  { 2, 3, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 11, 14, 15, },
  { 4, 8, 9, 11, 14, 15, },
  { 0, 4, 8, 9, 11, 14, 15, },
  { 1, 4, 8, 9, 11, 14, 15, },
  { 0, 1, 4, 8, 9, 11, 14, 15, },
  { 2, 4, 8, 9, 11, 14, 15, },
  { 0, 2, 4, 8, 9, 11, 14, 15, },
  { 1, 2, 4, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 11, 14, 15, },
  { 3, 4, 8, 9, 11, 14, 15, },
  { 0, 3, 4, 8, 9, 11, 14, 15, },
  { 1, 3, 4, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 11, 14, 15, },
  { 2, 3, 4, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 14, 15, },
  { 5, 8, 9, 11, 14, 15, },
  { 0, 5, 8, 9, 11, 14, 15, },
  { 1, 5, 8, 9, 11, 14, 15, },
  { 0, 1, 5, 8, 9, 11, 14, 15, },
  { 2, 5, 8, 9, 11, 14, 15, },
  { 0, 2, 5, 8, 9, 11, 14, 15, },
  { 1, 2, 5, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 11, 14, 15, },
  { 3, 5, 8, 9, 11, 14, 15, },
  { 0, 3, 5, 8, 9, 11, 14, 15, },
  { 1, 3, 5, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 11, 14, 15, },
  { 2, 3, 5, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 14, 15, },
  { 4, 5, 8, 9, 11, 14, 15, },
  { 0, 4, 5, 8, 9, 11, 14, 15, },
  { 1, 4, 5, 8, 9, 11, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 11, 14, 15, },
  { 2, 4, 5, 8, 9, 11, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 11, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 14, 15, },
  { 3, 4, 5, 8, 9, 11, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 11, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 14, 15, },
  { 6, 8, 9, 11, 14, 15, },
  { 0, 6, 8, 9, 11, 14, 15, },
  { 1, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 6, 8, 9, 11, 14, 15, },
  { 2, 6, 8, 9, 11, 14, 15, },
  { 0, 2, 6, 8, 9, 11, 14, 15, },
  { 1, 2, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 11, 14, 15, },
  { 3, 6, 8, 9, 11, 14, 15, },
  { 0, 3, 6, 8, 9, 11, 14, 15, },
  { 1, 3, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 11, 14, 15, },
  { 2, 3, 6, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 14, 15, },
  { 4, 6, 8, 9, 11, 14, 15, },
  { 0, 4, 6, 8, 9, 11, 14, 15, },
  { 1, 4, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 11, 14, 15, },
  { 2, 4, 6, 8, 9, 11, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 11, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 14, 15, },
  { 3, 4, 6, 8, 9, 11, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 11, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 14, 15, },
  { 5, 6, 8, 9, 11, 14, 15, },
  { 0, 5, 6, 8, 9, 11, 14, 15, },
  { 1, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 11, 14, 15, },
  { 2, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 11, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 14, 15, },
  { 3, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 11, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 14, 15, },
  { 4, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 14, 15, },
  { 7, 8, 9, 11, 14, 15, },
  { 0, 7, 8, 9, 11, 14, 15, },
  { 1, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 7, 8, 9, 11, 14, 15, },
  { 2, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 11, 14, 15, },
  { 3, 7, 8, 9, 11, 14, 15, },
  { 0, 3, 7, 8, 9, 11, 14, 15, },
  { 1, 3, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 11, 14, 15, },
  { 2, 3, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 14, 15, },
  { 4, 7, 8, 9, 11, 14, 15, },
  { 0, 4, 7, 8, 9, 11, 14, 15, },
  { 1, 4, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 11, 14, 15, },
  { 2, 4, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 14, 15, },
  { 3, 4, 7, 8, 9, 11, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 11, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 14, 15, },
  { 5, 7, 8, 9, 11, 14, 15, },
  { 0, 5, 7, 8, 9, 11, 14, 15, },
  { 1, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 11, 14, 15, },
  { 2, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 14, 15, },
  { 3, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 11, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 14, 15, },
  { 4, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 14, 15, },
  { 6, 7, 8, 9, 11, 14, 15, },
  { 0, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 11, 14, 15, },
  { 2, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 14, 15, },
  { 3, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 14, 15, },
  { 4, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 14, 15, },
  { 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 14, 15, },
  { 10, 11, 14, 15, },
  { 0, 10, 11, 14, 15, },
  { 1, 10, 11, 14, 15, },
  { 0, 1, 10, 11, 14, 15, },
  { 2, 10, 11, 14, 15, },
  { 0, 2, 10, 11, 14, 15, },
  { 1, 2, 10, 11, 14, 15, },
  { 0, 1, 2, 10, 11, 14, 15, },
  { 3, 10, 11, 14, 15, },
  { 0, 3, 10, 11, 14, 15, },
  { 1, 3, 10, 11, 14, 15, },
  { 0, 1, 3, 10, 11, 14, 15, },
  { 2, 3, 10, 11, 14, 15, },
  { 0, 2, 3, 10, 11, 14, 15, },
  { 1, 2, 3, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 10, 11, 14, 15, },
  { 4, 10, 11, 14, 15, },
  { 0, 4, 10, 11, 14, 15, },
  { 1, 4, 10, 11, 14, 15, },
  { 0, 1, 4, 10, 11, 14, 15, },
  { 2, 4, 10, 11, 14, 15, },
  { 0, 2, 4, 10, 11, 14, 15, },
  { 1, 2, 4, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 10, 11, 14, 15, },
  { 3, 4, 10, 11, 14, 15, },
  { 0, 3, 4, 10, 11, 14, 15, },
  { 1, 3, 4, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 10, 11, 14, 15, },
  { 2, 3, 4, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 10, 11, 14, 15, },
  { 5, 10, 11, 14, 15, },
  { 0, 5, 10, 11, 14, 15, },
  { 1, 5, 10, 11, 14, 15, },
  { 0, 1, 5, 10, 11, 14, 15, },
  { 2, 5, 10, 11, 14, 15, },
  { 0, 2, 5, 10, 11, 14, 15, },
  { 1, 2, 5, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 10, 11, 14, 15, },
  { 3, 5, 10, 11, 14, 15, },
  { 0, 3, 5, 10, 11, 14, 15, },
  { 1, 3, 5, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 10, 11, 14, 15, },
  { 2, 3, 5, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 10, 11, 14, 15, },
  { 4, 5, 10, 11, 14, 15, },
  { 0, 4, 5, 10, 11, 14, 15, },
  { 1, 4, 5, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 10, 11, 14, 15, },
  { 2, 4, 5, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 10, 11, 14, 15, },
  { 3, 4, 5, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 14, 15, },
  { 6, 10, 11, 14, 15, },
  { 0, 6, 10, 11, 14, 15, },
  { 1, 6, 10, 11, 14, 15, },
  { 0, 1, 6, 10, 11, 14, 15, },
  { 2, 6, 10, 11, 14, 15, },
  { 0, 2, 6, 10, 11, 14, 15, },
  { 1, 2, 6, 10, 11, 14, 15, },
  { 0, 1, 2, 6, 10, 11, 14, 15, },
  { 3, 6, 10, 11, 14, 15, },
  { 0, 3, 6, 10, 11, 14, 15, },
  { 1, 3, 6, 10, 11, 14, 15, },
  { 0, 1, 3, 6, 10, 11, 14, 15, },
  { 2, 3, 6, 10, 11, 14, 15, },
  { 0, 2, 3, 6, 10, 11, 14, 15, },
  { 1, 2, 3, 6, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 10, 11, 14, 15, },
  { 4, 6, 10, 11, 14, 15, },
  { 0, 4, 6, 10, 11, 14, 15, },
  { 1, 4, 6, 10, 11, 14, 15, },
  { 0, 1, 4, 6, 10, 11, 14, 15, },
  { 2, 4, 6, 10, 11, 14, 15, },
  { 0, 2, 4, 6, 10, 11, 14, 15, },
  { 1, 2, 4, 6, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 10, 11, 14, 15, },
  { 3, 4, 6, 10, 11, 14, 15, },
  { 0, 3, 4, 6, 10, 11, 14, 15, },
  { 1, 3, 4, 6, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 10, 11, 14, 15, },
  { 2, 3, 4, 6, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 14, 15, },
  { 5, 6, 10, 11, 14, 15, },
  { 0, 5, 6, 10, 11, 14, 15, },
  { 1, 5, 6, 10, 11, 14, 15, },
  { 0, 1, 5, 6, 10, 11, 14, 15, },
  { 2, 5, 6, 10, 11, 14, 15, },
  { 0, 2, 5, 6, 10, 11, 14, 15, },
  { 1, 2, 5, 6, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 10, 11, 14, 15, },
  { 3, 5, 6, 10, 11, 14, 15, },
  { 0, 3, 5, 6, 10, 11, 14, 15, },
  { 1, 3, 5, 6, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 10, 11, 14, 15, },
  { 2, 3, 5, 6, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 14, 15, },
  { 4, 5, 6, 10, 11, 14, 15, },
  { 0, 4, 5, 6, 10, 11, 14, 15, },
  { 1, 4, 5, 6, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 10, 11, 14, 15, },
  { 2, 4, 5, 6, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 14, 15, },
  { 3, 4, 5, 6, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 14, 15, },
  { 7, 10, 11, 14, 15, },
  { 0, 7, 10, 11, 14, 15, },
  { 1, 7, 10, 11, 14, 15, },
  { 0, 1, 7, 10, 11, 14, 15, },
  { 2, 7, 10, 11, 14, 15, },
  { 0, 2, 7, 10, 11, 14, 15, },
  { 1, 2, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 7, 10, 11, 14, 15, },
  { 3, 7, 10, 11, 14, 15, },
  { 0, 3, 7, 10, 11, 14, 15, },
  { 1, 3, 7, 10, 11, 14, 15, },
  { 0, 1, 3, 7, 10, 11, 14, 15, },
  { 2, 3, 7, 10, 11, 14, 15, },
  { 0, 2, 3, 7, 10, 11, 14, 15, },
  { 1, 2, 3, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 7, 10, 11, 14, 15, },
  { 4, 7, 10, 11, 14, 15, },
  { 0, 4, 7, 10, 11, 14, 15, },
  { 1, 4, 7, 10, 11, 14, 15, },
  { 0, 1, 4, 7, 10, 11, 14, 15, },
  { 2, 4, 7, 10, 11, 14, 15, },
  { 0, 2, 4, 7, 10, 11, 14, 15, },
  { 1, 2, 4, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 7, 10, 11, 14, 15, },
  { 3, 4, 7, 10, 11, 14, 15, },
  { 0, 3, 4, 7, 10, 11, 14, 15, },
  { 1, 3, 4, 7, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 7, 10, 11, 14, 15, },
  { 2, 3, 4, 7, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 7, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 14, 15, },
  { 5, 7, 10, 11, 14, 15, },
  { 0, 5, 7, 10, 11, 14, 15, },
  { 1, 5, 7, 10, 11, 14, 15, },
  { 0, 1, 5, 7, 10, 11, 14, 15, },
  { 2, 5, 7, 10, 11, 14, 15, },
  { 0, 2, 5, 7, 10, 11, 14, 15, },
  { 1, 2, 5, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 7, 10, 11, 14, 15, },
  { 3, 5, 7, 10, 11, 14, 15, },
  { 0, 3, 5, 7, 10, 11, 14, 15, },
  { 1, 3, 5, 7, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 7, 10, 11, 14, 15, },
  { 2, 3, 5, 7, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 7, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 14, 15, },
  { 4, 5, 7, 10, 11, 14, 15, },
  { 0, 4, 5, 7, 10, 11, 14, 15, },
  { 1, 4, 5, 7, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 7, 10, 11, 14, 15, },
  { 2, 4, 5, 7, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 7, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 14, 15, },
  { 3, 4, 5, 7, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 7, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 7, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 7, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 14, 15, },
  { 6, 7, 10, 11, 14, 15, },
  { 0, 6, 7, 10, 11, 14, 15, },
  { 1, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 6, 7, 10, 11, 14, 15, },
  { 2, 6, 7, 10, 11, 14, 15, },
  { 0, 2, 6, 7, 10, 11, 14, 15, },
  { 1, 2, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 6, 7, 10, 11, 14, 15, },
  { 3, 6, 7, 10, 11, 14, 15, },
  { 0, 3, 6, 7, 10, 11, 14, 15, },
  { 1, 3, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 3, 6, 7, 10, 11, 14, 15, },
  { 2, 3, 6, 7, 10, 11, 14, 15, },
  { 0, 2, 3, 6, 7, 10, 11, 14, 15, },
  { 1, 2, 3, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 14, 15, },
  { 4, 6, 7, 10, 11, 14, 15, },
  { 0, 4, 6, 7, 10, 11, 14, 15, },
  { 1, 4, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 4, 6, 7, 10, 11, 14, 15, },
  { 2, 4, 6, 7, 10, 11, 14, 15, },
  { 0, 2, 4, 6, 7, 10, 11, 14, 15, },
  { 1, 2, 4, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 14, 15, },
  { 3, 4, 6, 7, 10, 11, 14, 15, },
  { 0, 3, 4, 6, 7, 10, 11, 14, 15, },
  { 1, 3, 4, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 14, 15, },
  { 2, 3, 4, 6, 7, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 14, 15, },
  { 5, 6, 7, 10, 11, 14, 15, },
  { 0, 5, 6, 7, 10, 11, 14, 15, },
  { 1, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 5, 6, 7, 10, 11, 14, 15, },
  { 2, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 2, 5, 6, 7, 10, 11, 14, 15, },
  { 1, 2, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 14, 15, },
  { 3, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 3, 5, 6, 7, 10, 11, 14, 15, },
  { 1, 3, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 14, 15, },
  { 2, 3, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 14, 15, },
  { 4, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 1, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 2, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 3, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 14, 15, },
  { 8, 10, 11, 14, 15, },
  { 0, 8, 10, 11, 14, 15, },
  { 1, 8, 10, 11, 14, 15, },
  { 0, 1, 8, 10, 11, 14, 15, },
  { 2, 8, 10, 11, 14, 15, },
  { 0, 2, 8, 10, 11, 14, 15, },
  { 1, 2, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 8, 10, 11, 14, 15, },
  { 3, 8, 10, 11, 14, 15, },
  { 0, 3, 8, 10, 11, 14, 15, },
  { 1, 3, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 8, 10, 11, 14, 15, },
  { 2, 3, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 8, 10, 11, 14, 15, },
  { 4, 8, 10, 11, 14, 15, },
  { 0, 4, 8, 10, 11, 14, 15, },
  { 1, 4, 8, 10, 11, 14, 15, },
  { 0, 1, 4, 8, 10, 11, 14, 15, },
  { 2, 4, 8, 10, 11, 14, 15, },
  { 0, 2, 4, 8, 10, 11, 14, 15, },
  { 1, 2, 4, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 8, 10, 11, 14, 15, },
  { 3, 4, 8, 10, 11, 14, 15, },
  { 0, 3, 4, 8, 10, 11, 14, 15, },
  { 1, 3, 4, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 8, 10, 11, 14, 15, },
  { 2, 3, 4, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 14, 15, },
  { 5, 8, 10, 11, 14, 15, },
  { 0, 5, 8, 10, 11, 14, 15, },
  { 1, 5, 8, 10, 11, 14, 15, },
  { 0, 1, 5, 8, 10, 11, 14, 15, },
  { 2, 5, 8, 10, 11, 14, 15, },
  { 0, 2, 5, 8, 10, 11, 14, 15, },
  { 1, 2, 5, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 8, 10, 11, 14, 15, },
  { 3, 5, 8, 10, 11, 14, 15, },
  { 0, 3, 5, 8, 10, 11, 14, 15, },
  { 1, 3, 5, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 8, 10, 11, 14, 15, },
  { 2, 3, 5, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 14, 15, },
  { 4, 5, 8, 10, 11, 14, 15, },
  { 0, 4, 5, 8, 10, 11, 14, 15, },
  { 1, 4, 5, 8, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 8, 10, 11, 14, 15, },
  { 2, 4, 5, 8, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 8, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 14, 15, },
  { 3, 4, 5, 8, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 8, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 14, 15, },
  { 6, 8, 10, 11, 14, 15, },
  { 0, 6, 8, 10, 11, 14, 15, },
  { 1, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 6, 8, 10, 11, 14, 15, },
  { 2, 6, 8, 10, 11, 14, 15, },
  { 0, 2, 6, 8, 10, 11, 14, 15, },
  { 1, 2, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 6, 8, 10, 11, 14, 15, },
  { 3, 6, 8, 10, 11, 14, 15, },
  { 0, 3, 6, 8, 10, 11, 14, 15, },
  { 1, 3, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 6, 8, 10, 11, 14, 15, },
  { 2, 3, 6, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 6, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 14, 15, },
  { 4, 6, 8, 10, 11, 14, 15, },
  { 0, 4, 6, 8, 10, 11, 14, 15, },
  { 1, 4, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 4, 6, 8, 10, 11, 14, 15, },
  { 2, 4, 6, 8, 10, 11, 14, 15, },
  { 0, 2, 4, 6, 8, 10, 11, 14, 15, },
  { 1, 2, 4, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 14, 15, },
  { 3, 4, 6, 8, 10, 11, 14, 15, },
  { 0, 3, 4, 6, 8, 10, 11, 14, 15, },
  { 1, 3, 4, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 14, 15, },
  { 2, 3, 4, 6, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 14, 15, },
  { 5, 6, 8, 10, 11, 14, 15, },
  { 0, 5, 6, 8, 10, 11, 14, 15, },
  { 1, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 5, 6, 8, 10, 11, 14, 15, },
  { 2, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 2, 5, 6, 8, 10, 11, 14, 15, },
  { 1, 2, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 14, 15, },
  { 3, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 3, 5, 6, 8, 10, 11, 14, 15, },
  { 1, 3, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 14, 15, },
  { 2, 3, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 14, 15, },
  { 4, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 1, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 2, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 3, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 14, 15, },
  { 7, 8, 10, 11, 14, 15, },
  { 0, 7, 8, 10, 11, 14, 15, },
  { 1, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 7, 8, 10, 11, 14, 15, },
  { 2, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 7, 8, 10, 11, 14, 15, },
  { 3, 7, 8, 10, 11, 14, 15, },
  { 0, 3, 7, 8, 10, 11, 14, 15, },
  { 1, 3, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 7, 8, 10, 11, 14, 15, },
  { 2, 3, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 14, 15, },
  { 4, 7, 8, 10, 11, 14, 15, },
  { 0, 4, 7, 8, 10, 11, 14, 15, },
  { 1, 4, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 4, 7, 8, 10, 11, 14, 15, },
  { 2, 4, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 4, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 4, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 14, 15, },
  { 3, 4, 7, 8, 10, 11, 14, 15, },
  { 0, 3, 4, 7, 8, 10, 11, 14, 15, },
  { 1, 3, 4, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 14, 15, },
  { 2, 3, 4, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 14, 15, },
  { 5, 7, 8, 10, 11, 14, 15, },
  { 0, 5, 7, 8, 10, 11, 14, 15, },
  { 1, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 5, 7, 8, 10, 11, 14, 15, },
  { 2, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 5, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 14, 15, },
  { 3, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 3, 5, 7, 8, 10, 11, 14, 15, },
  { 1, 3, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 14, 15, },
  { 2, 3, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 14, 15, },
  { 4, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 1, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 2, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 3, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 14, 15, },
  { 6, 7, 8, 10, 11, 14, 15, },
  { 0, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 6, 7, 8, 10, 11, 14, 15, },
  { 2, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 14, 15, },
  { 3, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 3, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 3, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 14, 15, },
  { 2, 3, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 14, 15, },
  { 4, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 2, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 3, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 14, 15, },
  { 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 2, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 3, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 14, 15, },
  { 9, 10, 11, 14, 15, },
  { 0, 9, 10, 11, 14, 15, },
  { 1, 9, 10, 11, 14, 15, },
  { 0, 1, 9, 10, 11, 14, 15, },
  { 2, 9, 10, 11, 14, 15, },
  { 0, 2, 9, 10, 11, 14, 15, },
  { 1, 2, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 9, 10, 11, 14, 15, },
  { 3, 9, 10, 11, 14, 15, },
  { 0, 3, 9, 10, 11, 14, 15, },
  { 1, 3, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 9, 10, 11, 14, 15, },
  { 2, 3, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 9, 10, 11, 14, 15, },
  { 4, 9, 10, 11, 14, 15, },
  { 0, 4, 9, 10, 11, 14, 15, },
  { 1, 4, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 9, 10, 11, 14, 15, },
  { 2, 4, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 9, 10, 11, 14, 15, },
  { 3, 4, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 14, 15, },
  { 5, 9, 10, 11, 14, 15, },
  { 0, 5, 9, 10, 11, 14, 15, },
  { 1, 5, 9, 10, 11, 14, 15, },
  { 0, 1, 5, 9, 10, 11, 14, 15, },
  { 2, 5, 9, 10, 11, 14, 15, },
  { 0, 2, 5, 9, 10, 11, 14, 15, },
  { 1, 2, 5, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 9, 10, 11, 14, 15, },
  { 3, 5, 9, 10, 11, 14, 15, },
  { 0, 3, 5, 9, 10, 11, 14, 15, },
  { 1, 3, 5, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 9, 10, 11, 14, 15, },
  { 2, 3, 5, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 14, 15, },
  { 4, 5, 9, 10, 11, 14, 15, },
  { 0, 4, 5, 9, 10, 11, 14, 15, },
  { 1, 4, 5, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 9, 10, 11, 14, 15, },
  { 2, 4, 5, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 14, 15, },
  { 3, 4, 5, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 14, 15, },
  { 6, 9, 10, 11, 14, 15, },
  { 0, 6, 9, 10, 11, 14, 15, },
  { 1, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 6, 9, 10, 11, 14, 15, },
  { 2, 6, 9, 10, 11, 14, 15, },
  { 0, 2, 6, 9, 10, 11, 14, 15, },
  { 1, 2, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 6, 9, 10, 11, 14, 15, },
  { 3, 6, 9, 10, 11, 14, 15, },
  { 0, 3, 6, 9, 10, 11, 14, 15, },
  { 1, 3, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 6, 9, 10, 11, 14, 15, },
  { 2, 3, 6, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 6, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 14, 15, },
  { 4, 6, 9, 10, 11, 14, 15, },
  { 0, 4, 6, 9, 10, 11, 14, 15, },
  { 1, 4, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 6, 9, 10, 11, 14, 15, },
  { 2, 4, 6, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 6, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 14, 15, },
  { 3, 4, 6, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 6, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 6, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 14, 15, },
  { 5, 6, 9, 10, 11, 14, 15, },
  { 0, 5, 6, 9, 10, 11, 14, 15, },
  { 1, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 5, 6, 9, 10, 11, 14, 15, },
  { 2, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 2, 5, 6, 9, 10, 11, 14, 15, },
  { 1, 2, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 14, 15, },
  { 3, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 3, 5, 6, 9, 10, 11, 14, 15, },
  { 1, 3, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 14, 15, },
  { 2, 3, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 14, 15, },
  { 4, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 1, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 2, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 3, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 14, 15, },
  { 7, 9, 10, 11, 14, 15, },
  { 0, 7, 9, 10, 11, 14, 15, },
  { 1, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 7, 9, 10, 11, 14, 15, },
  { 2, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 7, 9, 10, 11, 14, 15, },
  { 3, 7, 9, 10, 11, 14, 15, },
  { 0, 3, 7, 9, 10, 11, 14, 15, },
  { 1, 3, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 7, 9, 10, 11, 14, 15, },
  { 2, 3, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 14, 15, },
  { 4, 7, 9, 10, 11, 14, 15, },
  { 0, 4, 7, 9, 10, 11, 14, 15, },
  { 1, 4, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 7, 9, 10, 11, 14, 15, },
  { 2, 4, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 14, 15, },
  { 3, 4, 7, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 7, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 14, 15, },
  { 5, 7, 9, 10, 11, 14, 15, },
  { 0, 5, 7, 9, 10, 11, 14, 15, },
  { 1, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 5, 7, 9, 10, 11, 14, 15, },
  { 2, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 5, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 14, 15, },
  { 3, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 3, 5, 7, 9, 10, 11, 14, 15, },
  { 1, 3, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 14, 15, },
  { 2, 3, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 14, 15, },
  { 4, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 1, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 2, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 3, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 14, 15, },
  { 6, 7, 9, 10, 11, 14, 15, },
  { 0, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 6, 7, 9, 10, 11, 14, 15, },
  { 2, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 14, 15, },
  { 3, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 3, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 3, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 14, 15, },
  { 2, 3, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 14, 15, },
  { 4, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 2, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 3, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 14, 15, },
  { 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 2, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 3, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 14, 15, },
  { 8, 9, 10, 11, 14, 15, },
  { 0, 8, 9, 10, 11, 14, 15, },
  { 1, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 8, 9, 10, 11, 14, 15, },
  { 2, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 8, 9, 10, 11, 14, 15, },
  { 3, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 14, 15, },
  { 4, 8, 9, 10, 11, 14, 15, },
  { 0, 4, 8, 9, 10, 11, 14, 15, },
  { 1, 4, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 8, 9, 10, 11, 14, 15, },
  { 2, 4, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 14, 15, },
  { 3, 4, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 14, 15, },
  { 5, 8, 9, 10, 11, 14, 15, },
  { 0, 5, 8, 9, 10, 11, 14, 15, },
  { 1, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 5, 8, 9, 10, 11, 14, 15, },
  { 2, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 5, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 14, 15, },
  { 3, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 5, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 14, 15, },
  { 4, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 1, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 2, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 3, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 14, 15, },
  { 6, 8, 9, 10, 11, 14, 15, },
  { 0, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 6, 8, 9, 10, 11, 14, 15, },
  { 2, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 14, 15, },
  { 3, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 14, 15, },
  { 4, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 2, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 3, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 14, 15, },
  { 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 2, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 3, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 14, 15, },
  { 7, 8, 9, 10, 11, 14, 15, },
  { 0, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 14, 15, },
  { 3, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 14, 15, },
  { 4, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 3, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 14, 15, },
  { 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 3, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, },
  { 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 3, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15, },
  { 12, 14, 15, },
  { 0, 12, 14, 15, },
  { 1, 12, 14, 15, },
  { 0, 1, 12, 14, 15, },
  { 2, 12, 14, 15, },
  { 0, 2, 12, 14, 15, },
  { 1, 2, 12, 14, 15, },
  { 0, 1, 2, 12, 14, 15, },
  { 3, 12, 14, 15, },
  { 0, 3, 12, 14, 15, },
  { 1, 3, 12, 14, 15, },
  { 0, 1, 3, 12, 14, 15, },
  { 2, 3, 12, 14, 15, },
  { 0, 2, 3, 12, 14, 15, },
  { 1, 2, 3, 12, 14, 15, },
  { 0, 1, 2, 3, 12, 14, 15, },
  { 4, 12, 14, 15, },
  { 0, 4, 12, 14, 15, },
  { 1, 4, 12, 14, 15, },
  { 0, 1, 4, 12, 14, 15, },
  { 2, 4, 12, 14, 15, },
  { 0, 2, 4, 12, 14, 15, },
  { 1, 2, 4, 12, 14, 15, },
  { 0, 1, 2, 4, 12, 14, 15, },
  { 3, 4, 12, 14, 15, },
  { 0, 3, 4, 12, 14, 15, },
  { 1, 3, 4, 12, 14, 15, },
  { 0, 1, 3, 4, 12, 14, 15, },
  { 2, 3, 4, 12, 14, 15, },
  { 0, 2, 3, 4, 12, 14, 15, },
  { 1, 2, 3, 4, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 12, 14, 15, },
  { 5, 12, 14, 15, },
  { 0, 5, 12, 14, 15, },
  { 1, 5, 12, 14, 15, },
  { 0, 1, 5, 12, 14, 15, },
  { 2, 5, 12, 14, 15, },
  { 0, 2, 5, 12, 14, 15, },
  { 1, 2, 5, 12, 14, 15, },
  { 0, 1, 2, 5, 12, 14, 15, },
  { 3, 5, 12, 14, 15, },
  { 0, 3, 5, 12, 14, 15, },
  { 1, 3, 5, 12, 14, 15, },
  { 0, 1, 3, 5, 12, 14, 15, },
  { 2, 3, 5, 12, 14, 15, },
  { 0, 2, 3, 5, 12, 14, 15, },
  { 1, 2, 3, 5, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 12, 14, 15, },
  { 4, 5, 12, 14, 15, },
  { 0, 4, 5, 12, 14, 15, },
  { 1, 4, 5, 12, 14, 15, },
  { 0, 1, 4, 5, 12, 14, 15, },
  { 2, 4, 5, 12, 14, 15, },
  { 0, 2, 4, 5, 12, 14, 15, },
  { 1, 2, 4, 5, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 12, 14, 15, },
  { 3, 4, 5, 12, 14, 15, },
  { 0, 3, 4, 5, 12, 14, 15, },
  { 1, 3, 4, 5, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 12, 14, 15, },
  { 2, 3, 4, 5, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 12, 14, 15, },
  { 6, 12, 14, 15, },
  { 0, 6, 12, 14, 15, },
  { 1, 6, 12, 14, 15, },
  { 0, 1, 6, 12, 14, 15, },
  { 2, 6, 12, 14, 15, },
  { 0, 2, 6, 12, 14, 15, },
  { 1, 2, 6, 12, 14, 15, },
  { 0, 1, 2, 6, 12, 14, 15, },
  { 3, 6, 12, 14, 15, },
  { 0, 3, 6, 12, 14, 15, },
  { 1, 3, 6, 12, 14, 15, },
  { 0, 1, 3, 6, 12, 14, 15, },
  { 2, 3, 6, 12, 14, 15, },
  { 0, 2, 3, 6, 12, 14, 15, },
  { 1, 2, 3, 6, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 12, 14, 15, },
  { 4, 6, 12, 14, 15, },
  { 0, 4, 6, 12, 14, 15, },
  { 1, 4, 6, 12, 14, 15, },
  { 0, 1, 4, 6, 12, 14, 15, },
  { 2, 4, 6, 12, 14, 15, },
  { 0, 2, 4, 6, 12, 14, 15, },
  { 1, 2, 4, 6, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 12, 14, 15, },
  { 3, 4, 6, 12, 14, 15, },
  { 0, 3, 4, 6, 12, 14, 15, },
  { 1, 3, 4, 6, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 12, 14, 15, },
  { 2, 3, 4, 6, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 12, 14, 15, },
  { 5, 6, 12, 14, 15, },
  { 0, 5, 6, 12, 14, 15, },
  { 1, 5, 6, 12, 14, 15, },
  { 0, 1, 5, 6, 12, 14, 15, },
  { 2, 5, 6, 12, 14, 15, },
  { 0, 2, 5, 6, 12, 14, 15, },
  { 1, 2, 5, 6, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 12, 14, 15, },
  { 3, 5, 6, 12, 14, 15, },
  { 0, 3, 5, 6, 12, 14, 15, },
  { 1, 3, 5, 6, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 12, 14, 15, },
  { 2, 3, 5, 6, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 12, 14, 15, },
  { 4, 5, 6, 12, 14, 15, },
  { 0, 4, 5, 6, 12, 14, 15, },
  { 1, 4, 5, 6, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 12, 14, 15, },
  { 2, 4, 5, 6, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 12, 14, 15, },
  { 3, 4, 5, 6, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 12, 14, 15, },
  { 7, 12, 14, 15, },
  { 0, 7, 12, 14, 15, },
  { 1, 7, 12, 14, 15, },
  { 0, 1, 7, 12, 14, 15, },
  { 2, 7, 12, 14, 15, },
  { 0, 2, 7, 12, 14, 15, },
  { 1, 2, 7, 12, 14, 15, },
  { 0, 1, 2, 7, 12, 14, 15, },
  { 3, 7, 12, 14, 15, },
  { 0, 3, 7, 12, 14, 15, },
  { 1, 3, 7, 12, 14, 15, },
  { 0, 1, 3, 7, 12, 14, 15, },
  { 2, 3, 7, 12, 14, 15, },
  { 0, 2, 3, 7, 12, 14, 15, },
  { 1, 2, 3, 7, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 12, 14, 15, },
  { 4, 7, 12, 14, 15, },
  { 0, 4, 7, 12, 14, 15, },
  { 1, 4, 7, 12, 14, 15, },
  { 0, 1, 4, 7, 12, 14, 15, },
  { 2, 4, 7, 12, 14, 15, },
  { 0, 2, 4, 7, 12, 14, 15, },
  { 1, 2, 4, 7, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 12, 14, 15, },
  { 3, 4, 7, 12, 14, 15, },
  { 0, 3, 4, 7, 12, 14, 15, },
  { 1, 3, 4, 7, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 12, 14, 15, },
  { 2, 3, 4, 7, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 12, 14, 15, },
  { 5, 7, 12, 14, 15, },
  { 0, 5, 7, 12, 14, 15, },
  { 1, 5, 7, 12, 14, 15, },
  { 0, 1, 5, 7, 12, 14, 15, },
  { 2, 5, 7, 12, 14, 15, },
  { 0, 2, 5, 7, 12, 14, 15, },
  { 1, 2, 5, 7, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 12, 14, 15, },
  { 3, 5, 7, 12, 14, 15, },
  { 0, 3, 5, 7, 12, 14, 15, },
  { 1, 3, 5, 7, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 12, 14, 15, },
  { 2, 3, 5, 7, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 12, 14, 15, },
  { 4, 5, 7, 12, 14, 15, },
  { 0, 4, 5, 7, 12, 14, 15, },
  { 1, 4, 5, 7, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 12, 14, 15, },
  { 2, 4, 5, 7, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 12, 14, 15, },
  { 3, 4, 5, 7, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 12, 14, 15, },
  { 6, 7, 12, 14, 15, },
  { 0, 6, 7, 12, 14, 15, },
  { 1, 6, 7, 12, 14, 15, },
  { 0, 1, 6, 7, 12, 14, 15, },
  { 2, 6, 7, 12, 14, 15, },
  { 0, 2, 6, 7, 12, 14, 15, },
  { 1, 2, 6, 7, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 12, 14, 15, },
  { 3, 6, 7, 12, 14, 15, },
  { 0, 3, 6, 7, 12, 14, 15, },
  { 1, 3, 6, 7, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 12, 14, 15, },
  { 2, 3, 6, 7, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 12, 14, 15, },
  { 4, 6, 7, 12, 14, 15, },
  { 0, 4, 6, 7, 12, 14, 15, },
  { 1, 4, 6, 7, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 12, 14, 15, },
  { 2, 4, 6, 7, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 12, 14, 15, },
  { 3, 4, 6, 7, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 12, 14, 15, },
  { 5, 6, 7, 12, 14, 15, },
  { 0, 5, 6, 7, 12, 14, 15, },
  { 1, 5, 6, 7, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 12, 14, 15, },
  { 2, 5, 6, 7, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 12, 14, 15, },
  { 3, 5, 6, 7, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 12, 14, 15, },
  { 4, 5, 6, 7, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 12, 14, 15, },
  { 8, 12, 14, 15, },
  { 0, 8, 12, 14, 15, },
  { 1, 8, 12, 14, 15, },
  { 0, 1, 8, 12, 14, 15, },
  { 2, 8, 12, 14, 15, },
  { 0, 2, 8, 12, 14, 15, },
  { 1, 2, 8, 12, 14, 15, },
  { 0, 1, 2, 8, 12, 14, 15, },
  { 3, 8, 12, 14, 15, },
  { 0, 3, 8, 12, 14, 15, },
  { 1, 3, 8, 12, 14, 15, },
  { 0, 1, 3, 8, 12, 14, 15, },
  { 2, 3, 8, 12, 14, 15, },
  { 0, 2, 3, 8, 12, 14, 15, },
  { 1, 2, 3, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 8, 12, 14, 15, },
  { 4, 8, 12, 14, 15, },
  { 0, 4, 8, 12, 14, 15, },
  { 1, 4, 8, 12, 14, 15, },
  { 0, 1, 4, 8, 12, 14, 15, },
  { 2, 4, 8, 12, 14, 15, },
  { 0, 2, 4, 8, 12, 14, 15, },
  { 1, 2, 4, 8, 12, 14, 15, },
  { 0, 1, 2, 4, 8, 12, 14, 15, },
  { 3, 4, 8, 12, 14, 15, },
  { 0, 3, 4, 8, 12, 14, 15, },
  { 1, 3, 4, 8, 12, 14, 15, },
  { 0, 1, 3, 4, 8, 12, 14, 15, },
  { 2, 3, 4, 8, 12, 14, 15, },
  { 0, 2, 3, 4, 8, 12, 14, 15, },
  { 1, 2, 3, 4, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 12, 14, 15, },
  { 5, 8, 12, 14, 15, },
  { 0, 5, 8, 12, 14, 15, },
  { 1, 5, 8, 12, 14, 15, },
  { 0, 1, 5, 8, 12, 14, 15, },
  { 2, 5, 8, 12, 14, 15, },
  { 0, 2, 5, 8, 12, 14, 15, },
  { 1, 2, 5, 8, 12, 14, 15, },
  { 0, 1, 2, 5, 8, 12, 14, 15, },
  { 3, 5, 8, 12, 14, 15, },
  { 0, 3, 5, 8, 12, 14, 15, },
  { 1, 3, 5, 8, 12, 14, 15, },
  { 0, 1, 3, 5, 8, 12, 14, 15, },
  { 2, 3, 5, 8, 12, 14, 15, },
  { 0, 2, 3, 5, 8, 12, 14, 15, },
  { 1, 2, 3, 5, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 12, 14, 15, },
  { 4, 5, 8, 12, 14, 15, },
  { 0, 4, 5, 8, 12, 14, 15, },
  { 1, 4, 5, 8, 12, 14, 15, },
  { 0, 1, 4, 5, 8, 12, 14, 15, },
  { 2, 4, 5, 8, 12, 14, 15, },
  { 0, 2, 4, 5, 8, 12, 14, 15, },
  { 1, 2, 4, 5, 8, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 12, 14, 15, },
  { 3, 4, 5, 8, 12, 14, 15, },
  { 0, 3, 4, 5, 8, 12, 14, 15, },
  { 1, 3, 4, 5, 8, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 12, 14, 15, },
  { 2, 3, 4, 5, 8, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 12, 14, 15, },
  { 6, 8, 12, 14, 15, },
  { 0, 6, 8, 12, 14, 15, },
  { 1, 6, 8, 12, 14, 15, },
  { 0, 1, 6, 8, 12, 14, 15, },
  { 2, 6, 8, 12, 14, 15, },
  { 0, 2, 6, 8, 12, 14, 15, },
  { 1, 2, 6, 8, 12, 14, 15, },
  { 0, 1, 2, 6, 8, 12, 14, 15, },
  { 3, 6, 8, 12, 14, 15, },
  { 0, 3, 6, 8, 12, 14, 15, },
  { 1, 3, 6, 8, 12, 14, 15, },
  { 0, 1, 3, 6, 8, 12, 14, 15, },
  { 2, 3, 6, 8, 12, 14, 15, },
  { 0, 2, 3, 6, 8, 12, 14, 15, },
  { 1, 2, 3, 6, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 12, 14, 15, },
  { 4, 6, 8, 12, 14, 15, },
  { 0, 4, 6, 8, 12, 14, 15, },
  { 1, 4, 6, 8, 12, 14, 15, },
  { 0, 1, 4, 6, 8, 12, 14, 15, },
  { 2, 4, 6, 8, 12, 14, 15, },
  { 0, 2, 4, 6, 8, 12, 14, 15, },
  { 1, 2, 4, 6, 8, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 12, 14, 15, },
  { 3, 4, 6, 8, 12, 14, 15, },
  { 0, 3, 4, 6, 8, 12, 14, 15, },
  { 1, 3, 4, 6, 8, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 12, 14, 15, },
  { 2, 3, 4, 6, 8, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 12, 14, 15, },
  { 5, 6, 8, 12, 14, 15, },
  { 0, 5, 6, 8, 12, 14, 15, },
  { 1, 5, 6, 8, 12, 14, 15, },
  { 0, 1, 5, 6, 8, 12, 14, 15, },
  { 2, 5, 6, 8, 12, 14, 15, },
  { 0, 2, 5, 6, 8, 12, 14, 15, },
  { 1, 2, 5, 6, 8, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 12, 14, 15, },
  { 3, 5, 6, 8, 12, 14, 15, },
  { 0, 3, 5, 6, 8, 12, 14, 15, },
  { 1, 3, 5, 6, 8, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 12, 14, 15, },
  { 2, 3, 5, 6, 8, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 12, 14, 15, },
  { 4, 5, 6, 8, 12, 14, 15, },
  { 0, 4, 5, 6, 8, 12, 14, 15, },
  { 1, 4, 5, 6, 8, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 12, 14, 15, },
  { 2, 4, 5, 6, 8, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 12, 14, 15, },
  { 3, 4, 5, 6, 8, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 12, 14, 15, },
  { 7, 8, 12, 14, 15, },
  { 0, 7, 8, 12, 14, 15, },
  { 1, 7, 8, 12, 14, 15, },
  { 0, 1, 7, 8, 12, 14, 15, },
  { 2, 7, 8, 12, 14, 15, },
  { 0, 2, 7, 8, 12, 14, 15, },
  { 1, 2, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 7, 8, 12, 14, 15, },
  { 3, 7, 8, 12, 14, 15, },
  { 0, 3, 7, 8, 12, 14, 15, },
  { 1, 3, 7, 8, 12, 14, 15, },
  { 0, 1, 3, 7, 8, 12, 14, 15, },
  { 2, 3, 7, 8, 12, 14, 15, },
  { 0, 2, 3, 7, 8, 12, 14, 15, },
  { 1, 2, 3, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 12, 14, 15, },
  { 4, 7, 8, 12, 14, 15, },
  { 0, 4, 7, 8, 12, 14, 15, },
  { 1, 4, 7, 8, 12, 14, 15, },
  { 0, 1, 4, 7, 8, 12, 14, 15, },
  { 2, 4, 7, 8, 12, 14, 15, },
  { 0, 2, 4, 7, 8, 12, 14, 15, },
  { 1, 2, 4, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 12, 14, 15, },
  { 3, 4, 7, 8, 12, 14, 15, },
  { 0, 3, 4, 7, 8, 12, 14, 15, },
  { 1, 3, 4, 7, 8, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 12, 14, 15, },
  { 2, 3, 4, 7, 8, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 12, 14, 15, },
  { 5, 7, 8, 12, 14, 15, },
  { 0, 5, 7, 8, 12, 14, 15, },
  { 1, 5, 7, 8, 12, 14, 15, },
  { 0, 1, 5, 7, 8, 12, 14, 15, },
  { 2, 5, 7, 8, 12, 14, 15, },
  { 0, 2, 5, 7, 8, 12, 14, 15, },
  { 1, 2, 5, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 12, 14, 15, },
  { 3, 5, 7, 8, 12, 14, 15, },
  { 0, 3, 5, 7, 8, 12, 14, 15, },
  { 1, 3, 5, 7, 8, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 12, 14, 15, },
  { 2, 3, 5, 7, 8, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 12, 14, 15, },
  { 4, 5, 7, 8, 12, 14, 15, },
  { 0, 4, 5, 7, 8, 12, 14, 15, },
  { 1, 4, 5, 7, 8, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 12, 14, 15, },
  { 2, 4, 5, 7, 8, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 12, 14, 15, },
  { 3, 4, 5, 7, 8, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 12, 14, 15, },
  { 6, 7, 8, 12, 14, 15, },
  { 0, 6, 7, 8, 12, 14, 15, },
  { 1, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 6, 7, 8, 12, 14, 15, },
  { 2, 6, 7, 8, 12, 14, 15, },
  { 0, 2, 6, 7, 8, 12, 14, 15, },
  { 1, 2, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 12, 14, 15, },
  { 3, 6, 7, 8, 12, 14, 15, },
  { 0, 3, 6, 7, 8, 12, 14, 15, },
  { 1, 3, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 12, 14, 15, },
  { 2, 3, 6, 7, 8, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 12, 14, 15, },
  { 4, 6, 7, 8, 12, 14, 15, },
  { 0, 4, 6, 7, 8, 12, 14, 15, },
  { 1, 4, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 12, 14, 15, },
  { 2, 4, 6, 7, 8, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 12, 14, 15, },
  { 3, 4, 6, 7, 8, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 12, 14, 15, },
  { 5, 6, 7, 8, 12, 14, 15, },
  { 0, 5, 6, 7, 8, 12, 14, 15, },
  { 1, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 12, 14, 15, },
  { 2, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 12, 14, 15, },
  { 3, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 12, 14, 15, },
  { 4, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 12, 14, 15, },
  { 9, 12, 14, 15, },
  { 0, 9, 12, 14, 15, },
  { 1, 9, 12, 14, 15, },
  { 0, 1, 9, 12, 14, 15, },
  { 2, 9, 12, 14, 15, },
  { 0, 2, 9, 12, 14, 15, },
  { 1, 2, 9, 12, 14, 15, },
  { 0, 1, 2, 9, 12, 14, 15, },
  { 3, 9, 12, 14, 15, },
  { 0, 3, 9, 12, 14, 15, },
  { 1, 3, 9, 12, 14, 15, },
  { 0, 1, 3, 9, 12, 14, 15, },
  { 2, 3, 9, 12, 14, 15, },
  { 0, 2, 3, 9, 12, 14, 15, },
  { 1, 2, 3, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 9, 12, 14, 15, },
  { 4, 9, 12, 14, 15, },
  { 0, 4, 9, 12, 14, 15, },
  { 1, 4, 9, 12, 14, 15, },
  { 0, 1, 4, 9, 12, 14, 15, },
  { 2, 4, 9, 12, 14, 15, },
  { 0, 2, 4, 9, 12, 14, 15, },
  { 1, 2, 4, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 9, 12, 14, 15, },
  { 3, 4, 9, 12, 14, 15, },
  { 0, 3, 4, 9, 12, 14, 15, },
  { 1, 3, 4, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 9, 12, 14, 15, },
  { 2, 3, 4, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 12, 14, 15, },
  { 5, 9, 12, 14, 15, },
  { 0, 5, 9, 12, 14, 15, },
  { 1, 5, 9, 12, 14, 15, },
  { 0, 1, 5, 9, 12, 14, 15, },
  { 2, 5, 9, 12, 14, 15, },
  { 0, 2, 5, 9, 12, 14, 15, },
  { 1, 2, 5, 9, 12, 14, 15, },
  { 0, 1, 2, 5, 9, 12, 14, 15, },
  { 3, 5, 9, 12, 14, 15, },
  { 0, 3, 5, 9, 12, 14, 15, },
  { 1, 3, 5, 9, 12, 14, 15, },
  { 0, 1, 3, 5, 9, 12, 14, 15, },
  { 2, 3, 5, 9, 12, 14, 15, },
  { 0, 2, 3, 5, 9, 12, 14, 15, },
  { 1, 2, 3, 5, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 12, 14, 15, },
  { 4, 5, 9, 12, 14, 15, },
  { 0, 4, 5, 9, 12, 14, 15, },
  { 1, 4, 5, 9, 12, 14, 15, },
  { 0, 1, 4, 5, 9, 12, 14, 15, },
  { 2, 4, 5, 9, 12, 14, 15, },
  { 0, 2, 4, 5, 9, 12, 14, 15, },
  { 1, 2, 4, 5, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 12, 14, 15, },
  { 3, 4, 5, 9, 12, 14, 15, },
  { 0, 3, 4, 5, 9, 12, 14, 15, },
  { 1, 3, 4, 5, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 12, 14, 15, },
  { 2, 3, 4, 5, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 12, 14, 15, },
  { 6, 9, 12, 14, 15, },
  { 0, 6, 9, 12, 14, 15, },
  { 1, 6, 9, 12, 14, 15, },
  { 0, 1, 6, 9, 12, 14, 15, },
  { 2, 6, 9, 12, 14, 15, },
  { 0, 2, 6, 9, 12, 14, 15, },
  { 1, 2, 6, 9, 12, 14, 15, },
  { 0, 1, 2, 6, 9, 12, 14, 15, },
  { 3, 6, 9, 12, 14, 15, },
  { 0, 3, 6, 9, 12, 14, 15, },
  { 1, 3, 6, 9, 12, 14, 15, },
  { 0, 1, 3, 6, 9, 12, 14, 15, },
  { 2, 3, 6, 9, 12, 14, 15, },
  { 0, 2, 3, 6, 9, 12, 14, 15, },
  { 1, 2, 3, 6, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 12, 14, 15, },
  { 4, 6, 9, 12, 14, 15, },
  { 0, 4, 6, 9, 12, 14, 15, },
  { 1, 4, 6, 9, 12, 14, 15, },
  { 0, 1, 4, 6, 9, 12, 14, 15, },
  { 2, 4, 6, 9, 12, 14, 15, },
  { 0, 2, 4, 6, 9, 12, 14, 15, },
  { 1, 2, 4, 6, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 12, 14, 15, },
  { 3, 4, 6, 9, 12, 14, 15, },
  { 0, 3, 4, 6, 9, 12, 14, 15, },
  { 1, 3, 4, 6, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 12, 14, 15, },
  { 2, 3, 4, 6, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 12, 14, 15, },
  { 5, 6, 9, 12, 14, 15, },
  { 0, 5, 6, 9, 12, 14, 15, },
  { 1, 5, 6, 9, 12, 14, 15, },
  { 0, 1, 5, 6, 9, 12, 14, 15, },
  { 2, 5, 6, 9, 12, 14, 15, },
  { 0, 2, 5, 6, 9, 12, 14, 15, },
  { 1, 2, 5, 6, 9, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 12, 14, 15, },
  { 3, 5, 6, 9, 12, 14, 15, },
  { 0, 3, 5, 6, 9, 12, 14, 15, },
  { 1, 3, 5, 6, 9, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 12, 14, 15, },
  { 2, 3, 5, 6, 9, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 12, 14, 15, },
  { 4, 5, 6, 9, 12, 14, 15, },
  { 0, 4, 5, 6, 9, 12, 14, 15, },
  { 1, 4, 5, 6, 9, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 12, 14, 15, },
  { 2, 4, 5, 6, 9, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 12, 14, 15, },
  { 3, 4, 5, 6, 9, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 12, 14, 15, },
  { 7, 9, 12, 14, 15, },
  { 0, 7, 9, 12, 14, 15, },
  { 1, 7, 9, 12, 14, 15, },
  { 0, 1, 7, 9, 12, 14, 15, },
  { 2, 7, 9, 12, 14, 15, },
  { 0, 2, 7, 9, 12, 14, 15, },
  { 1, 2, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 7, 9, 12, 14, 15, },
  { 3, 7, 9, 12, 14, 15, },
  { 0, 3, 7, 9, 12, 14, 15, },
  { 1, 3, 7, 9, 12, 14, 15, },
  { 0, 1, 3, 7, 9, 12, 14, 15, },
  { 2, 3, 7, 9, 12, 14, 15, },
  { 0, 2, 3, 7, 9, 12, 14, 15, },
  { 1, 2, 3, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 12, 14, 15, },
  { 4, 7, 9, 12, 14, 15, },
  { 0, 4, 7, 9, 12, 14, 15, },
  { 1, 4, 7, 9, 12, 14, 15, },
  { 0, 1, 4, 7, 9, 12, 14, 15, },
  { 2, 4, 7, 9, 12, 14, 15, },
  { 0, 2, 4, 7, 9, 12, 14, 15, },
  { 1, 2, 4, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 12, 14, 15, },
  { 3, 4, 7, 9, 12, 14, 15, },
  { 0, 3, 4, 7, 9, 12, 14, 15, },
  { 1, 3, 4, 7, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 12, 14, 15, },
  { 2, 3, 4, 7, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 12, 14, 15, },
  { 5, 7, 9, 12, 14, 15, },
  { 0, 5, 7, 9, 12, 14, 15, },
  { 1, 5, 7, 9, 12, 14, 15, },
  { 0, 1, 5, 7, 9, 12, 14, 15, },
  { 2, 5, 7, 9, 12, 14, 15, },
  { 0, 2, 5, 7, 9, 12, 14, 15, },
  { 1, 2, 5, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 12, 14, 15, },
  { 3, 5, 7, 9, 12, 14, 15, },
  { 0, 3, 5, 7, 9, 12, 14, 15, },
  { 1, 3, 5, 7, 9, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 12, 14, 15, },
  { 2, 3, 5, 7, 9, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 12, 14, 15, },
  { 4, 5, 7, 9, 12, 14, 15, },
  { 0, 4, 5, 7, 9, 12, 14, 15, },
  { 1, 4, 5, 7, 9, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 12, 14, 15, },
  { 2, 4, 5, 7, 9, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 12, 14, 15, },
  { 3, 4, 5, 7, 9, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 12, 14, 15, },
  { 6, 7, 9, 12, 14, 15, },
  { 0, 6, 7, 9, 12, 14, 15, },
  { 1, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 6, 7, 9, 12, 14, 15, },
  { 2, 6, 7, 9, 12, 14, 15, },
  { 0, 2, 6, 7, 9, 12, 14, 15, },
  { 1, 2, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 12, 14, 15, },
  { 3, 6, 7, 9, 12, 14, 15, },
  { 0, 3, 6, 7, 9, 12, 14, 15, },
  { 1, 3, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 12, 14, 15, },
  { 2, 3, 6, 7, 9, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 12, 14, 15, },
  { 4, 6, 7, 9, 12, 14, 15, },
  { 0, 4, 6, 7, 9, 12, 14, 15, },
  { 1, 4, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 12, 14, 15, },
  { 2, 4, 6, 7, 9, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 12, 14, 15, },
  { 3, 4, 6, 7, 9, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 12, 14, 15, },
  { 5, 6, 7, 9, 12, 14, 15, },
  { 0, 5, 6, 7, 9, 12, 14, 15, },
  { 1, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 12, 14, 15, },
  { 2, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 12, 14, 15, },
  { 3, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 12, 14, 15, },
  { 4, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 12, 14, 15, },
  { 8, 9, 12, 14, 15, },
  { 0, 8, 9, 12, 14, 15, },
  { 1, 8, 9, 12, 14, 15, },
  { 0, 1, 8, 9, 12, 14, 15, },
  { 2, 8, 9, 12, 14, 15, },
  { 0, 2, 8, 9, 12, 14, 15, },
  { 1, 2, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 8, 9, 12, 14, 15, },
  { 3, 8, 9, 12, 14, 15, },
  { 0, 3, 8, 9, 12, 14, 15, },
  { 1, 3, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 8, 9, 12, 14, 15, },
  { 2, 3, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 12, 14, 15, },
  { 4, 8, 9, 12, 14, 15, },
  { 0, 4, 8, 9, 12, 14, 15, },
  { 1, 4, 8, 9, 12, 14, 15, },
  { 0, 1, 4, 8, 9, 12, 14, 15, },
  { 2, 4, 8, 9, 12, 14, 15, },
  { 0, 2, 4, 8, 9, 12, 14, 15, },
  { 1, 2, 4, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 12, 14, 15, },
  { 3, 4, 8, 9, 12, 14, 15, },
  { 0, 3, 4, 8, 9, 12, 14, 15, },
  { 1, 3, 4, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 12, 14, 15, },
  { 2, 3, 4, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 12, 14, 15, },
  { 5, 8, 9, 12, 14, 15, },
  { 0, 5, 8, 9, 12, 14, 15, },
  { 1, 5, 8, 9, 12, 14, 15, },
  { 0, 1, 5, 8, 9, 12, 14, 15, },
  { 2, 5, 8, 9, 12, 14, 15, },
  { 0, 2, 5, 8, 9, 12, 14, 15, },
  { 1, 2, 5, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 12, 14, 15, },
  { 3, 5, 8, 9, 12, 14, 15, },
  { 0, 3, 5, 8, 9, 12, 14, 15, },
  { 1, 3, 5, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 12, 14, 15, },
  { 2, 3, 5, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 12, 14, 15, },
  { 4, 5, 8, 9, 12, 14, 15, },
  { 0, 4, 5, 8, 9, 12, 14, 15, },
  { 1, 4, 5, 8, 9, 12, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 12, 14, 15, },
  { 2, 4, 5, 8, 9, 12, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 12, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 12, 14, 15, },
  { 3, 4, 5, 8, 9, 12, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 12, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 12, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 12, 14, 15, },
  { 6, 8, 9, 12, 14, 15, },
  { 0, 6, 8, 9, 12, 14, 15, },
  { 1, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 6, 8, 9, 12, 14, 15, },
  { 2, 6, 8, 9, 12, 14, 15, },
  { 0, 2, 6, 8, 9, 12, 14, 15, },
  { 1, 2, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 12, 14, 15, },
  { 3, 6, 8, 9, 12, 14, 15, },
  { 0, 3, 6, 8, 9, 12, 14, 15, },
  { 1, 3, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 12, 14, 15, },
  { 2, 3, 6, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 12, 14, 15, },
  { 4, 6, 8, 9, 12, 14, 15, },
  { 0, 4, 6, 8, 9, 12, 14, 15, },
  { 1, 4, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 12, 14, 15, },
  { 2, 4, 6, 8, 9, 12, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 12, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 12, 14, 15, },
  { 3, 4, 6, 8, 9, 12, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 12, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 12, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 12, 14, 15, },
  { 5, 6, 8, 9, 12, 14, 15, },
  { 0, 5, 6, 8, 9, 12, 14, 15, },
  { 1, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 12, 14, 15, },
  { 2, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 12, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 12, 14, 15, },
  { 3, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 12, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 12, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 12, 14, 15, },
  { 4, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 12, 14, 15, },
  { 7, 8, 9, 12, 14, 15, },
  { 0, 7, 8, 9, 12, 14, 15, },
  { 1, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 7, 8, 9, 12, 14, 15, },
  { 2, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 12, 14, 15, },
  { 3, 7, 8, 9, 12, 14, 15, },
  { 0, 3, 7, 8, 9, 12, 14, 15, },
  { 1, 3, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 12, 14, 15, },
  { 2, 3, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 12, 14, 15, },
  { 4, 7, 8, 9, 12, 14, 15, },
  { 0, 4, 7, 8, 9, 12, 14, 15, },
  { 1, 4, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 12, 14, 15, },
  { 2, 4, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 12, 14, 15, },
  { 3, 4, 7, 8, 9, 12, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 12, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 12, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 12, 14, 15, },
  { 5, 7, 8, 9, 12, 14, 15, },
  { 0, 5, 7, 8, 9, 12, 14, 15, },
  { 1, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 12, 14, 15, },
  { 2, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 12, 14, 15, },
  { 3, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 12, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 12, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 12, 14, 15, },
  { 4, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 12, 14, 15, },
  { 6, 7, 8, 9, 12, 14, 15, },
  { 0, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 12, 14, 15, },
  { 2, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 12, 14, 15, },
  { 3, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 12, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 12, 14, 15, },
  { 4, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 12, 14, 15, },
  { 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 14, 15, },
  { 10, 12, 14, 15, },
  { 0, 10, 12, 14, 15, },
  { 1, 10, 12, 14, 15, },
  { 0, 1, 10, 12, 14, 15, },
  { 2, 10, 12, 14, 15, },
  { 0, 2, 10, 12, 14, 15, },
  { 1, 2, 10, 12, 14, 15, },
  { 0, 1, 2, 10, 12, 14, 15, },
  { 3, 10, 12, 14, 15, },
  { 0, 3, 10, 12, 14, 15, },
  { 1, 3, 10, 12, 14, 15, },
  { 0, 1, 3, 10, 12, 14, 15, },
  { 2, 3, 10, 12, 14, 15, },
  { 0, 2, 3, 10, 12, 14, 15, },
  { 1, 2, 3, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 10, 12, 14, 15, },
  { 4, 10, 12, 14, 15, },
  { 0, 4, 10, 12, 14, 15, },
  { 1, 4, 10, 12, 14, 15, },
  { 0, 1, 4, 10, 12, 14, 15, },
  { 2, 4, 10, 12, 14, 15, },
  { 0, 2, 4, 10, 12, 14, 15, },
  { 1, 2, 4, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 10, 12, 14, 15, },
  { 3, 4, 10, 12, 14, 15, },
  { 0, 3, 4, 10, 12, 14, 15, },
  { 1, 3, 4, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 10, 12, 14, 15, },
  { 2, 3, 4, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 10, 12, 14, 15, },
  { 5, 10, 12, 14, 15, },
  { 0, 5, 10, 12, 14, 15, },
  { 1, 5, 10, 12, 14, 15, },
  { 0, 1, 5, 10, 12, 14, 15, },
  { 2, 5, 10, 12, 14, 15, },
  { 0, 2, 5, 10, 12, 14, 15, },
  { 1, 2, 5, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 10, 12, 14, 15, },
  { 3, 5, 10, 12, 14, 15, },
  { 0, 3, 5, 10, 12, 14, 15, },
  { 1, 3, 5, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 10, 12, 14, 15, },
  { 2, 3, 5, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 10, 12, 14, 15, },
  { 4, 5, 10, 12, 14, 15, },
  { 0, 4, 5, 10, 12, 14, 15, },
  { 1, 4, 5, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 10, 12, 14, 15, },
  { 2, 4, 5, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 10, 12, 14, 15, },
  { 3, 4, 5, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 12, 14, 15, },
  { 6, 10, 12, 14, 15, },
  { 0, 6, 10, 12, 14, 15, },
  { 1, 6, 10, 12, 14, 15, },
  { 0, 1, 6, 10, 12, 14, 15, },
  { 2, 6, 10, 12, 14, 15, },
  { 0, 2, 6, 10, 12, 14, 15, },
  { 1, 2, 6, 10, 12, 14, 15, },
  { 0, 1, 2, 6, 10, 12, 14, 15, },
  { 3, 6, 10, 12, 14, 15, },
  { 0, 3, 6, 10, 12, 14, 15, },
  { 1, 3, 6, 10, 12, 14, 15, },
  { 0, 1, 3, 6, 10, 12, 14, 15, },
  { 2, 3, 6, 10, 12, 14, 15, },
  { 0, 2, 3, 6, 10, 12, 14, 15, },
  { 1, 2, 3, 6, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 10, 12, 14, 15, },
  { 4, 6, 10, 12, 14, 15, },
  { 0, 4, 6, 10, 12, 14, 15, },
  { 1, 4, 6, 10, 12, 14, 15, },
  { 0, 1, 4, 6, 10, 12, 14, 15, },
  { 2, 4, 6, 10, 12, 14, 15, },
  { 0, 2, 4, 6, 10, 12, 14, 15, },
  { 1, 2, 4, 6, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 10, 12, 14, 15, },
  { 3, 4, 6, 10, 12, 14, 15, },
  { 0, 3, 4, 6, 10, 12, 14, 15, },
  { 1, 3, 4, 6, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 10, 12, 14, 15, },
  { 2, 3, 4, 6, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 12, 14, 15, },
  { 5, 6, 10, 12, 14, 15, },
  { 0, 5, 6, 10, 12, 14, 15, },
  { 1, 5, 6, 10, 12, 14, 15, },
  { 0, 1, 5, 6, 10, 12, 14, 15, },
  { 2, 5, 6, 10, 12, 14, 15, },
  { 0, 2, 5, 6, 10, 12, 14, 15, },
  { 1, 2, 5, 6, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 10, 12, 14, 15, },
  { 3, 5, 6, 10, 12, 14, 15, },
  { 0, 3, 5, 6, 10, 12, 14, 15, },
  { 1, 3, 5, 6, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 10, 12, 14, 15, },
  { 2, 3, 5, 6, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 12, 14, 15, },
  { 4, 5, 6, 10, 12, 14, 15, },
  { 0, 4, 5, 6, 10, 12, 14, 15, },
  { 1, 4, 5, 6, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 10, 12, 14, 15, },
  { 2, 4, 5, 6, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 12, 14, 15, },
  { 3, 4, 5, 6, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 12, 14, 15, },
  { 7, 10, 12, 14, 15, },
  { 0, 7, 10, 12, 14, 15, },
  { 1, 7, 10, 12, 14, 15, },
  { 0, 1, 7, 10, 12, 14, 15, },
  { 2, 7, 10, 12, 14, 15, },
  { 0, 2, 7, 10, 12, 14, 15, },
  { 1, 2, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 7, 10, 12, 14, 15, },
  { 3, 7, 10, 12, 14, 15, },
  { 0, 3, 7, 10, 12, 14, 15, },
  { 1, 3, 7, 10, 12, 14, 15, },
  { 0, 1, 3, 7, 10, 12, 14, 15, },
  { 2, 3, 7, 10, 12, 14, 15, },
  { 0, 2, 3, 7, 10, 12, 14, 15, },
  { 1, 2, 3, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 10, 12, 14, 15, },
  { 4, 7, 10, 12, 14, 15, },
  { 0, 4, 7, 10, 12, 14, 15, },
  { 1, 4, 7, 10, 12, 14, 15, },
  { 0, 1, 4, 7, 10, 12, 14, 15, },
  { 2, 4, 7, 10, 12, 14, 15, },
  { 0, 2, 4, 7, 10, 12, 14, 15, },
  { 1, 2, 4, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 10, 12, 14, 15, },
  { 3, 4, 7, 10, 12, 14, 15, },
  { 0, 3, 4, 7, 10, 12, 14, 15, },
  { 1, 3, 4, 7, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 10, 12, 14, 15, },
  { 2, 3, 4, 7, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 12, 14, 15, },
  { 5, 7, 10, 12, 14, 15, },
  { 0, 5, 7, 10, 12, 14, 15, },
  { 1, 5, 7, 10, 12, 14, 15, },
  { 0, 1, 5, 7, 10, 12, 14, 15, },
  { 2, 5, 7, 10, 12, 14, 15, },
  { 0, 2, 5, 7, 10, 12, 14, 15, },
  { 1, 2, 5, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 10, 12, 14, 15, },
  { 3, 5, 7, 10, 12, 14, 15, },
  { 0, 3, 5, 7, 10, 12, 14, 15, },
  { 1, 3, 5, 7, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 10, 12, 14, 15, },
  { 2, 3, 5, 7, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 12, 14, 15, },
  { 4, 5, 7, 10, 12, 14, 15, },
  { 0, 4, 5, 7, 10, 12, 14, 15, },
  { 1, 4, 5, 7, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 10, 12, 14, 15, },
  { 2, 4, 5, 7, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 12, 14, 15, },
  { 3, 4, 5, 7, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 12, 14, 15, },
  { 6, 7, 10, 12, 14, 15, },
  { 0, 6, 7, 10, 12, 14, 15, },
  { 1, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 6, 7, 10, 12, 14, 15, },
  { 2, 6, 7, 10, 12, 14, 15, },
  { 0, 2, 6, 7, 10, 12, 14, 15, },
  { 1, 2, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 10, 12, 14, 15, },
  { 3, 6, 7, 10, 12, 14, 15, },
  { 0, 3, 6, 7, 10, 12, 14, 15, },
  { 1, 3, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 10, 12, 14, 15, },
  { 2, 3, 6, 7, 10, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 10, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 12, 14, 15, },
  { 4, 6, 7, 10, 12, 14, 15, },
  { 0, 4, 6, 7, 10, 12, 14, 15, },
  { 1, 4, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 10, 12, 14, 15, },
  { 2, 4, 6, 7, 10, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 10, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 12, 14, 15, },
  { 3, 4, 6, 7, 10, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 10, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 12, 14, 15, },
  { 5, 6, 7, 10, 12, 14, 15, },
  { 0, 5, 6, 7, 10, 12, 14, 15, },
  { 1, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 10, 12, 14, 15, },
  { 2, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 10, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 12, 14, 15, },
  { 3, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 10, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 12, 14, 15, },
  { 4, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 12, 14, 15, },
  { 8, 10, 12, 14, 15, },
  { 0, 8, 10, 12, 14, 15, },
  { 1, 8, 10, 12, 14, 15, },
  { 0, 1, 8, 10, 12, 14, 15, },
  { 2, 8, 10, 12, 14, 15, },
  { 0, 2, 8, 10, 12, 14, 15, },
  { 1, 2, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 8, 10, 12, 14, 15, },
  { 3, 8, 10, 12, 14, 15, },
  { 0, 3, 8, 10, 12, 14, 15, },
  { 1, 3, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 8, 10, 12, 14, 15, },
  { 2, 3, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 8, 10, 12, 14, 15, },
  { 4, 8, 10, 12, 14, 15, },
  { 0, 4, 8, 10, 12, 14, 15, },
  { 1, 4, 8, 10, 12, 14, 15, },
  { 0, 1, 4, 8, 10, 12, 14, 15, },
  { 2, 4, 8, 10, 12, 14, 15, },
  { 0, 2, 4, 8, 10, 12, 14, 15, },
  { 1, 2, 4, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 8, 10, 12, 14, 15, },
  { 3, 4, 8, 10, 12, 14, 15, },
  { 0, 3, 4, 8, 10, 12, 14, 15, },
  { 1, 3, 4, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 8, 10, 12, 14, 15, },
  { 2, 3, 4, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 12, 14, 15, },
  { 5, 8, 10, 12, 14, 15, },
  { 0, 5, 8, 10, 12, 14, 15, },
  { 1, 5, 8, 10, 12, 14, 15, },
  { 0, 1, 5, 8, 10, 12, 14, 15, },
  { 2, 5, 8, 10, 12, 14, 15, },
  { 0, 2, 5, 8, 10, 12, 14, 15, },
  { 1, 2, 5, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 8, 10, 12, 14, 15, },
  { 3, 5, 8, 10, 12, 14, 15, },
  { 0, 3, 5, 8, 10, 12, 14, 15, },
  { 1, 3, 5, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 8, 10, 12, 14, 15, },
  { 2, 3, 5, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 12, 14, 15, },
  { 4, 5, 8, 10, 12, 14, 15, },
  { 0, 4, 5, 8, 10, 12, 14, 15, },
  { 1, 4, 5, 8, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 8, 10, 12, 14, 15, },
  { 2, 4, 5, 8, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 8, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 12, 14, 15, },
  { 3, 4, 5, 8, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 8, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 12, 14, 15, },
  { 6, 8, 10, 12, 14, 15, },
  { 0, 6, 8, 10, 12, 14, 15, },
  { 1, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 6, 8, 10, 12, 14, 15, },
  { 2, 6, 8, 10, 12, 14, 15, },
  { 0, 2, 6, 8, 10, 12, 14, 15, },
  { 1, 2, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 6, 8, 10, 12, 14, 15, },
  { 3, 6, 8, 10, 12, 14, 15, },
  { 0, 3, 6, 8, 10, 12, 14, 15, },
  { 1, 3, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 6, 8, 10, 12, 14, 15, },
  { 2, 3, 6, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 6, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 12, 14, 15, },
  { 4, 6, 8, 10, 12, 14, 15, },
  { 0, 4, 6, 8, 10, 12, 14, 15, },
  { 1, 4, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 4, 6, 8, 10, 12, 14, 15, },
  { 2, 4, 6, 8, 10, 12, 14, 15, },
  { 0, 2, 4, 6, 8, 10, 12, 14, 15, },
  { 1, 2, 4, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 12, 14, 15, },
  { 3, 4, 6, 8, 10, 12, 14, 15, },
  { 0, 3, 4, 6, 8, 10, 12, 14, 15, },
  { 1, 3, 4, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 12, 14, 15, },
  { 2, 3, 4, 6, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 12, 14, 15, },
  { 5, 6, 8, 10, 12, 14, 15, },
  { 0, 5, 6, 8, 10, 12, 14, 15, },
  { 1, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 5, 6, 8, 10, 12, 14, 15, },
  { 2, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 2, 5, 6, 8, 10, 12, 14, 15, },
  { 1, 2, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 12, 14, 15, },
  { 3, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 3, 5, 6, 8, 10, 12, 14, 15, },
  { 1, 3, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 12, 14, 15, },
  { 2, 3, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 12, 14, 15, },
  { 4, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 1, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 2, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 3, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 12, 14, 15, },
  { 7, 8, 10, 12, 14, 15, },
  { 0, 7, 8, 10, 12, 14, 15, },
  { 1, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 7, 8, 10, 12, 14, 15, },
  { 2, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 7, 8, 10, 12, 14, 15, },
  { 3, 7, 8, 10, 12, 14, 15, },
  { 0, 3, 7, 8, 10, 12, 14, 15, },
  { 1, 3, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 7, 8, 10, 12, 14, 15, },
  { 2, 3, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 12, 14, 15, },
  { 4, 7, 8, 10, 12, 14, 15, },
  { 0, 4, 7, 8, 10, 12, 14, 15, },
  { 1, 4, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 4, 7, 8, 10, 12, 14, 15, },
  { 2, 4, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 4, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 4, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 12, 14, 15, },
  { 3, 4, 7, 8, 10, 12, 14, 15, },
  { 0, 3, 4, 7, 8, 10, 12, 14, 15, },
  { 1, 3, 4, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 12, 14, 15, },
  { 2, 3, 4, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 12, 14, 15, },
  { 5, 7, 8, 10, 12, 14, 15, },
  { 0, 5, 7, 8, 10, 12, 14, 15, },
  { 1, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 5, 7, 8, 10, 12, 14, 15, },
  { 2, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 5, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 12, 14, 15, },
  { 3, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 3, 5, 7, 8, 10, 12, 14, 15, },
  { 1, 3, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 12, 14, 15, },
  { 2, 3, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 12, 14, 15, },
  { 4, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 1, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 2, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 3, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 14, 15, },
  { 6, 7, 8, 10, 12, 14, 15, },
  { 0, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 6, 7, 8, 10, 12, 14, 15, },
  { 2, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 12, 14, 15, },
  { 3, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 3, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 3, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 12, 14, 15, },
  { 2, 3, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 12, 14, 15, },
  { 4, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 2, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 3, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 12, 14, 15, },
  { 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 2, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 3, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, },
  { 9, 10, 12, 14, 15, },
  { 0, 9, 10, 12, 14, 15, },
  { 1, 9, 10, 12, 14, 15, },
  { 0, 1, 9, 10, 12, 14, 15, },
  { 2, 9, 10, 12, 14, 15, },
  { 0, 2, 9, 10, 12, 14, 15, },
  { 1, 2, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 9, 10, 12, 14, 15, },
  { 3, 9, 10, 12, 14, 15, },
  { 0, 3, 9, 10, 12, 14, 15, },
  { 1, 3, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 9, 10, 12, 14, 15, },
  { 2, 3, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 9, 10, 12, 14, 15, },
  { 4, 9, 10, 12, 14, 15, },
  { 0, 4, 9, 10, 12, 14, 15, },
  { 1, 4, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 9, 10, 12, 14, 15, },
  { 2, 4, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 9, 10, 12, 14, 15, },
  { 3, 4, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 12, 14, 15, },
  { 5, 9, 10, 12, 14, 15, },
  { 0, 5, 9, 10, 12, 14, 15, },
  { 1, 5, 9, 10, 12, 14, 15, },
  { 0, 1, 5, 9, 10, 12, 14, 15, },
  { 2, 5, 9, 10, 12, 14, 15, },
  { 0, 2, 5, 9, 10, 12, 14, 15, },
  { 1, 2, 5, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 9, 10, 12, 14, 15, },
  { 3, 5, 9, 10, 12, 14, 15, },
  { 0, 3, 5, 9, 10, 12, 14, 15, },
  { 1, 3, 5, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 9, 10, 12, 14, 15, },
  { 2, 3, 5, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 12, 14, 15, },
  { 4, 5, 9, 10, 12, 14, 15, },
  { 0, 4, 5, 9, 10, 12, 14, 15, },
  { 1, 4, 5, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 9, 10, 12, 14, 15, },
  { 2, 4, 5, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 12, 14, 15, },
  { 3, 4, 5, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 12, 14, 15, },
  { 6, 9, 10, 12, 14, 15, },
  { 0, 6, 9, 10, 12, 14, 15, },
  { 1, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 6, 9, 10, 12, 14, 15, },
  { 2, 6, 9, 10, 12, 14, 15, },
  { 0, 2, 6, 9, 10, 12, 14, 15, },
  { 1, 2, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 6, 9, 10, 12, 14, 15, },
  { 3, 6, 9, 10, 12, 14, 15, },
  { 0, 3, 6, 9, 10, 12, 14, 15, },
  { 1, 3, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 6, 9, 10, 12, 14, 15, },
  { 2, 3, 6, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 6, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 12, 14, 15, },
  { 4, 6, 9, 10, 12, 14, 15, },
  { 0, 4, 6, 9, 10, 12, 14, 15, },
  { 1, 4, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 6, 9, 10, 12, 14, 15, },
  { 2, 4, 6, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 6, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 12, 14, 15, },
  { 3, 4, 6, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 6, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 6, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 12, 14, 15, },
  { 5, 6, 9, 10, 12, 14, 15, },
  { 0, 5, 6, 9, 10, 12, 14, 15, },
  { 1, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 5, 6, 9, 10, 12, 14, 15, },
  { 2, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 2, 5, 6, 9, 10, 12, 14, 15, },
  { 1, 2, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 12, 14, 15, },
  { 3, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 3, 5, 6, 9, 10, 12, 14, 15, },
  { 1, 3, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 12, 14, 15, },
  { 2, 3, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 12, 14, 15, },
  { 4, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 1, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 2, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 3, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 12, 14, 15, },
  { 7, 9, 10, 12, 14, 15, },
  { 0, 7, 9, 10, 12, 14, 15, },
  { 1, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 7, 9, 10, 12, 14, 15, },
  { 2, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 7, 9, 10, 12, 14, 15, },
  { 3, 7, 9, 10, 12, 14, 15, },
  { 0, 3, 7, 9, 10, 12, 14, 15, },
  { 1, 3, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 7, 9, 10, 12, 14, 15, },
  { 2, 3, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 12, 14, 15, },
  { 4, 7, 9, 10, 12, 14, 15, },
  { 0, 4, 7, 9, 10, 12, 14, 15, },
  { 1, 4, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 7, 9, 10, 12, 14, 15, },
  { 2, 4, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 12, 14, 15, },
  { 3, 4, 7, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 7, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 12, 14, 15, },
  { 5, 7, 9, 10, 12, 14, 15, },
  { 0, 5, 7, 9, 10, 12, 14, 15, },
  { 1, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 5, 7, 9, 10, 12, 14, 15, },
  { 2, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 5, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 12, 14, 15, },
  { 3, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 3, 5, 7, 9, 10, 12, 14, 15, },
  { 1, 3, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 12, 14, 15, },
  { 2, 3, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 12, 14, 15, },
  { 4, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 1, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 2, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 3, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 12, 14, 15, },
  { 6, 7, 9, 10, 12, 14, 15, },
  { 0, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 6, 7, 9, 10, 12, 14, 15, },
  { 2, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 12, 14, 15, },
  { 3, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 3, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 3, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 12, 14, 15, },
  { 2, 3, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 12, 14, 15, },
  { 4, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 2, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 3, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 12, 14, 15, },
  { 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 2, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 3, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, 15, },
  { 8, 9, 10, 12, 14, 15, },
  { 0, 8, 9, 10, 12, 14, 15, },
  { 1, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 8, 9, 10, 12, 14, 15, },
  { 2, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 8, 9, 10, 12, 14, 15, },
  { 3, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 12, 14, 15, },
  { 4, 8, 9, 10, 12, 14, 15, },
  { 0, 4, 8, 9, 10, 12, 14, 15, },
  { 1, 4, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 8, 9, 10, 12, 14, 15, },
  { 2, 4, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 12, 14, 15, },
  { 3, 4, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 12, 14, 15, },
  { 5, 8, 9, 10, 12, 14, 15, },
  { 0, 5, 8, 9, 10, 12, 14, 15, },
  { 1, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 5, 8, 9, 10, 12, 14, 15, },
  { 2, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 5, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 12, 14, 15, },
  { 3, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 5, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 12, 14, 15, },
  { 4, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 1, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 2, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 3, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 12, 14, 15, },
  { 6, 8, 9, 10, 12, 14, 15, },
  { 0, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 6, 8, 9, 10, 12, 14, 15, },
  { 2, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 12, 14, 15, },
  { 3, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 12, 14, 15, },
  { 4, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 2, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 3, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 12, 14, 15, },
  { 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 2, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 3, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 14, 15, },
  { 7, 8, 9, 10, 12, 14, 15, },
  { 0, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 12, 14, 15, },
  { 3, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 12, 14, 15, },
  { 4, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 3, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 12, 14, 15, },
  { 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 3, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 14, 15, },
  { 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 3, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 15, },
  { 11, 12, 14, 15, },
  { 0, 11, 12, 14, 15, },
  { 1, 11, 12, 14, 15, },
  { 0, 1, 11, 12, 14, 15, },
  { 2, 11, 12, 14, 15, },
  { 0, 2, 11, 12, 14, 15, },
  { 1, 2, 11, 12, 14, 15, },
  { 0, 1, 2, 11, 12, 14, 15, },
  { 3, 11, 12, 14, 15, },
  { 0, 3, 11, 12, 14, 15, },
  { 1, 3, 11, 12, 14, 15, },
  { 0, 1, 3, 11, 12, 14, 15, },
  { 2, 3, 11, 12, 14, 15, },
  { 0, 2, 3, 11, 12, 14, 15, },
  { 1, 2, 3, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 11, 12, 14, 15, },
  { 4, 11, 12, 14, 15, },
  { 0, 4, 11, 12, 14, 15, },
  { 1, 4, 11, 12, 14, 15, },
  { 0, 1, 4, 11, 12, 14, 15, },
  { 2, 4, 11, 12, 14, 15, },
  { 0, 2, 4, 11, 12, 14, 15, },
  { 1, 2, 4, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 11, 12, 14, 15, },
  { 3, 4, 11, 12, 14, 15, },
  { 0, 3, 4, 11, 12, 14, 15, },
  { 1, 3, 4, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 11, 12, 14, 15, },
  { 2, 3, 4, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 11, 12, 14, 15, },
  { 5, 11, 12, 14, 15, },
  { 0, 5, 11, 12, 14, 15, },
  { 1, 5, 11, 12, 14, 15, },
  { 0, 1, 5, 11, 12, 14, 15, },
  { 2, 5, 11, 12, 14, 15, },
  { 0, 2, 5, 11, 12, 14, 15, },
  { 1, 2, 5, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 11, 12, 14, 15, },
  { 3, 5, 11, 12, 14, 15, },
  { 0, 3, 5, 11, 12, 14, 15, },
  { 1, 3, 5, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 11, 12, 14, 15, },
  { 2, 3, 5, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 11, 12, 14, 15, },
  { 4, 5, 11, 12, 14, 15, },
  { 0, 4, 5, 11, 12, 14, 15, },
  { 1, 4, 5, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 11, 12, 14, 15, },
  { 2, 4, 5, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 11, 12, 14, 15, },
  { 3, 4, 5, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 11, 12, 14, 15, },
  { 6, 11, 12, 14, 15, },
  { 0, 6, 11, 12, 14, 15, },
  { 1, 6, 11, 12, 14, 15, },
  { 0, 1, 6, 11, 12, 14, 15, },
  { 2, 6, 11, 12, 14, 15, },
  { 0, 2, 6, 11, 12, 14, 15, },
  { 1, 2, 6, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 11, 12, 14, 15, },
  { 3, 6, 11, 12, 14, 15, },
  { 0, 3, 6, 11, 12, 14, 15, },
  { 1, 3, 6, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 11, 12, 14, 15, },
  { 2, 3, 6, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 11, 12, 14, 15, },
  { 4, 6, 11, 12, 14, 15, },
  { 0, 4, 6, 11, 12, 14, 15, },
  { 1, 4, 6, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 11, 12, 14, 15, },
  { 2, 4, 6, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 11, 12, 14, 15, },
  { 3, 4, 6, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 11, 12, 14, 15, },
  { 5, 6, 11, 12, 14, 15, },
  { 0, 5, 6, 11, 12, 14, 15, },
  { 1, 5, 6, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 11, 12, 14, 15, },
  { 2, 5, 6, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 11, 12, 14, 15, },
  { 3, 5, 6, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 11, 12, 14, 15, },
  { 4, 5, 6, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 12, 14, 15, },
  { 7, 11, 12, 14, 15, },
  { 0, 7, 11, 12, 14, 15, },
  { 1, 7, 11, 12, 14, 15, },
  { 0, 1, 7, 11, 12, 14, 15, },
  { 2, 7, 11, 12, 14, 15, },
  { 0, 2, 7, 11, 12, 14, 15, },
  { 1, 2, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 7, 11, 12, 14, 15, },
  { 3, 7, 11, 12, 14, 15, },
  { 0, 3, 7, 11, 12, 14, 15, },
  { 1, 3, 7, 11, 12, 14, 15, },
  { 0, 1, 3, 7, 11, 12, 14, 15, },
  { 2, 3, 7, 11, 12, 14, 15, },
  { 0, 2, 3, 7, 11, 12, 14, 15, },
  { 1, 2, 3, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 11, 12, 14, 15, },
  { 4, 7, 11, 12, 14, 15, },
  { 0, 4, 7, 11, 12, 14, 15, },
  { 1, 4, 7, 11, 12, 14, 15, },
  { 0, 1, 4, 7, 11, 12, 14, 15, },
  { 2, 4, 7, 11, 12, 14, 15, },
  { 0, 2, 4, 7, 11, 12, 14, 15, },
  { 1, 2, 4, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 11, 12, 14, 15, },
  { 3, 4, 7, 11, 12, 14, 15, },
  { 0, 3, 4, 7, 11, 12, 14, 15, },
  { 1, 3, 4, 7, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 11, 12, 14, 15, },
  { 2, 3, 4, 7, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 11, 12, 14, 15, },
  { 5, 7, 11, 12, 14, 15, },
  { 0, 5, 7, 11, 12, 14, 15, },
  { 1, 5, 7, 11, 12, 14, 15, },
  { 0, 1, 5, 7, 11, 12, 14, 15, },
  { 2, 5, 7, 11, 12, 14, 15, },
  { 0, 2, 5, 7, 11, 12, 14, 15, },
  { 1, 2, 5, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 11, 12, 14, 15, },
  { 3, 5, 7, 11, 12, 14, 15, },
  { 0, 3, 5, 7, 11, 12, 14, 15, },
  { 1, 3, 5, 7, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 11, 12, 14, 15, },
  { 2, 3, 5, 7, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 11, 12, 14, 15, },
  { 4, 5, 7, 11, 12, 14, 15, },
  { 0, 4, 5, 7, 11, 12, 14, 15, },
  { 1, 4, 5, 7, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 11, 12, 14, 15, },
  { 2, 4, 5, 7, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 11, 12, 14, 15, },
  { 3, 4, 5, 7, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 12, 14, 15, },
  { 6, 7, 11, 12, 14, 15, },
  { 0, 6, 7, 11, 12, 14, 15, },
  { 1, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 6, 7, 11, 12, 14, 15, },
  { 2, 6, 7, 11, 12, 14, 15, },
  { 0, 2, 6, 7, 11, 12, 14, 15, },
  { 1, 2, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 11, 12, 14, 15, },
  { 3, 6, 7, 11, 12, 14, 15, },
  { 0, 3, 6, 7, 11, 12, 14, 15, },
  { 1, 3, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 11, 12, 14, 15, },
  { 2, 3, 6, 7, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 11, 12, 14, 15, },
  { 4, 6, 7, 11, 12, 14, 15, },
  { 0, 4, 6, 7, 11, 12, 14, 15, },
  { 1, 4, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 11, 12, 14, 15, },
  { 2, 4, 6, 7, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 11, 12, 14, 15, },
  { 3, 4, 6, 7, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 12, 14, 15, },
  { 5, 6, 7, 11, 12, 14, 15, },
  { 0, 5, 6, 7, 11, 12, 14, 15, },
  { 1, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 11, 12, 14, 15, },
  { 2, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 11, 12, 14, 15, },
  { 3, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 12, 14, 15, },
  { 4, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 12, 14, 15, },
  { 8, 11, 12, 14, 15, },
  { 0, 8, 11, 12, 14, 15, },
  { 1, 8, 11, 12, 14, 15, },
  { 0, 1, 8, 11, 12, 14, 15, },
  { 2, 8, 11, 12, 14, 15, },
  { 0, 2, 8, 11, 12, 14, 15, },
  { 1, 2, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 8, 11, 12, 14, 15, },
  { 3, 8, 11, 12, 14, 15, },
  { 0, 3, 8, 11, 12, 14, 15, },
  { 1, 3, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 8, 11, 12, 14, 15, },
  { 2, 3, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 8, 11, 12, 14, 15, },
  { 4, 8, 11, 12, 14, 15, },
  { 0, 4, 8, 11, 12, 14, 15, },
  { 1, 4, 8, 11, 12, 14, 15, },
  { 0, 1, 4, 8, 11, 12, 14, 15, },
  { 2, 4, 8, 11, 12, 14, 15, },
  { 0, 2, 4, 8, 11, 12, 14, 15, },
  { 1, 2, 4, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 8, 11, 12, 14, 15, },
  { 3, 4, 8, 11, 12, 14, 15, },
  { 0, 3, 4, 8, 11, 12, 14, 15, },
  { 1, 3, 4, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 8, 11, 12, 14, 15, },
  { 2, 3, 4, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 11, 12, 14, 15, },
  { 5, 8, 11, 12, 14, 15, },
  { 0, 5, 8, 11, 12, 14, 15, },
  { 1, 5, 8, 11, 12, 14, 15, },
  { 0, 1, 5, 8, 11, 12, 14, 15, },
  { 2, 5, 8, 11, 12, 14, 15, },
  { 0, 2, 5, 8, 11, 12, 14, 15, },
  { 1, 2, 5, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 8, 11, 12, 14, 15, },
  { 3, 5, 8, 11, 12, 14, 15, },
  { 0, 3, 5, 8, 11, 12, 14, 15, },
  { 1, 3, 5, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 8, 11, 12, 14, 15, },
  { 2, 3, 5, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 11, 12, 14, 15, },
  { 4, 5, 8, 11, 12, 14, 15, },
  { 0, 4, 5, 8, 11, 12, 14, 15, },
  { 1, 4, 5, 8, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 8, 11, 12, 14, 15, },
  { 2, 4, 5, 8, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 8, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 11, 12, 14, 15, },
  { 3, 4, 5, 8, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 8, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 12, 14, 15, },
  { 6, 8, 11, 12, 14, 15, },
  { 0, 6, 8, 11, 12, 14, 15, },
  { 1, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 6, 8, 11, 12, 14, 15, },
  { 2, 6, 8, 11, 12, 14, 15, },
  { 0, 2, 6, 8, 11, 12, 14, 15, },
  { 1, 2, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 8, 11, 12, 14, 15, },
  { 3, 6, 8, 11, 12, 14, 15, },
  { 0, 3, 6, 8, 11, 12, 14, 15, },
  { 1, 3, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 8, 11, 12, 14, 15, },
  { 2, 3, 6, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 11, 12, 14, 15, },
  { 4, 6, 8, 11, 12, 14, 15, },
  { 0, 4, 6, 8, 11, 12, 14, 15, },
  { 1, 4, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 8, 11, 12, 14, 15, },
  { 2, 4, 6, 8, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 8, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 11, 12, 14, 15, },
  { 3, 4, 6, 8, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 8, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 12, 14, 15, },
  { 5, 6, 8, 11, 12, 14, 15, },
  { 0, 5, 6, 8, 11, 12, 14, 15, },
  { 1, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 8, 11, 12, 14, 15, },
  { 2, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 8, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 11, 12, 14, 15, },
  { 3, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 8, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 12, 14, 15, },
  { 4, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 12, 14, 15, },
  { 7, 8, 11, 12, 14, 15, },
  { 0, 7, 8, 11, 12, 14, 15, },
  { 1, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 7, 8, 11, 12, 14, 15, },
  { 2, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 7, 8, 11, 12, 14, 15, },
  { 3, 7, 8, 11, 12, 14, 15, },
  { 0, 3, 7, 8, 11, 12, 14, 15, },
  { 1, 3, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 7, 8, 11, 12, 14, 15, },
  { 2, 3, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 11, 12, 14, 15, },
  { 4, 7, 8, 11, 12, 14, 15, },
  { 0, 4, 7, 8, 11, 12, 14, 15, },
  { 1, 4, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 4, 7, 8, 11, 12, 14, 15, },
  { 2, 4, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 4, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 4, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 11, 12, 14, 15, },
  { 3, 4, 7, 8, 11, 12, 14, 15, },
  { 0, 3, 4, 7, 8, 11, 12, 14, 15, },
  { 1, 3, 4, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 11, 12, 14, 15, },
  { 2, 3, 4, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 12, 14, 15, },
  { 5, 7, 8, 11, 12, 14, 15, },
  { 0, 5, 7, 8, 11, 12, 14, 15, },
  { 1, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 5, 7, 8, 11, 12, 14, 15, },
  { 2, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 5, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 11, 12, 14, 15, },
  { 3, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 3, 5, 7, 8, 11, 12, 14, 15, },
  { 1, 3, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 11, 12, 14, 15, },
  { 2, 3, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 12, 14, 15, },
  { 4, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 1, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 2, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 3, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 12, 14, 15, },
  { 6, 7, 8, 11, 12, 14, 15, },
  { 0, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 6, 7, 8, 11, 12, 14, 15, },
  { 2, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 11, 12, 14, 15, },
  { 3, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 3, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 3, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 11, 12, 14, 15, },
  { 2, 3, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 12, 14, 15, },
  { 4, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 2, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 3, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 12, 14, 15, },
  { 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 2, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 3, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 14, 15, },
  { 9, 11, 12, 14, 15, },
  { 0, 9, 11, 12, 14, 15, },
  { 1, 9, 11, 12, 14, 15, },
  { 0, 1, 9, 11, 12, 14, 15, },
  { 2, 9, 11, 12, 14, 15, },
  { 0, 2, 9, 11, 12, 14, 15, },
  { 1, 2, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 9, 11, 12, 14, 15, },
  { 3, 9, 11, 12, 14, 15, },
  { 0, 3, 9, 11, 12, 14, 15, },
  { 1, 3, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 9, 11, 12, 14, 15, },
  { 2, 3, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 9, 11, 12, 14, 15, },
  { 4, 9, 11, 12, 14, 15, },
  { 0, 4, 9, 11, 12, 14, 15, },
  { 1, 4, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 9, 11, 12, 14, 15, },
  { 2, 4, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 9, 11, 12, 14, 15, },
  { 3, 4, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 11, 12, 14, 15, },
  { 5, 9, 11, 12, 14, 15, },
  { 0, 5, 9, 11, 12, 14, 15, },
  { 1, 5, 9, 11, 12, 14, 15, },
  { 0, 1, 5, 9, 11, 12, 14, 15, },
  { 2, 5, 9, 11, 12, 14, 15, },
  { 0, 2, 5, 9, 11, 12, 14, 15, },
  { 1, 2, 5, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 9, 11, 12, 14, 15, },
  { 3, 5, 9, 11, 12, 14, 15, },
  { 0, 3, 5, 9, 11, 12, 14, 15, },
  { 1, 3, 5, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 9, 11, 12, 14, 15, },
  { 2, 3, 5, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 11, 12, 14, 15, },
  { 4, 5, 9, 11, 12, 14, 15, },
  { 0, 4, 5, 9, 11, 12, 14, 15, },
  { 1, 4, 5, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 9, 11, 12, 14, 15, },
  { 2, 4, 5, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 11, 12, 14, 15, },
  { 3, 4, 5, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 12, 14, 15, },
  { 6, 9, 11, 12, 14, 15, },
  { 0, 6, 9, 11, 12, 14, 15, },
  { 1, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 6, 9, 11, 12, 14, 15, },
  { 2, 6, 9, 11, 12, 14, 15, },
  { 0, 2, 6, 9, 11, 12, 14, 15, },
  { 1, 2, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 9, 11, 12, 14, 15, },
  { 3, 6, 9, 11, 12, 14, 15, },
  { 0, 3, 6, 9, 11, 12, 14, 15, },
  { 1, 3, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 9, 11, 12, 14, 15, },
  { 2, 3, 6, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 11, 12, 14, 15, },
  { 4, 6, 9, 11, 12, 14, 15, },
  { 0, 4, 6, 9, 11, 12, 14, 15, },
  { 1, 4, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 9, 11, 12, 14, 15, },
  { 2, 4, 6, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 11, 12, 14, 15, },
  { 3, 4, 6, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 12, 14, 15, },
  { 5, 6, 9, 11, 12, 14, 15, },
  { 0, 5, 6, 9, 11, 12, 14, 15, },
  { 1, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 9, 11, 12, 14, 15, },
  { 2, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 9, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 11, 12, 14, 15, },
  { 3, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 9, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 12, 14, 15, },
  { 4, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 12, 14, 15, },
  { 7, 9, 11, 12, 14, 15, },
  { 0, 7, 9, 11, 12, 14, 15, },
  { 1, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 7, 9, 11, 12, 14, 15, },
  { 2, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 7, 9, 11, 12, 14, 15, },
  { 3, 7, 9, 11, 12, 14, 15, },
  { 0, 3, 7, 9, 11, 12, 14, 15, },
  { 1, 3, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 7, 9, 11, 12, 14, 15, },
  { 2, 3, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 11, 12, 14, 15, },
  { 4, 7, 9, 11, 12, 14, 15, },
  { 0, 4, 7, 9, 11, 12, 14, 15, },
  { 1, 4, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 7, 9, 11, 12, 14, 15, },
  { 2, 4, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 11, 12, 14, 15, },
  { 3, 4, 7, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 7, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 12, 14, 15, },
  { 5, 7, 9, 11, 12, 14, 15, },
  { 0, 5, 7, 9, 11, 12, 14, 15, },
  { 1, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 5, 7, 9, 11, 12, 14, 15, },
  { 2, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 5, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 11, 12, 14, 15, },
  { 3, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 3, 5, 7, 9, 11, 12, 14, 15, },
  { 1, 3, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 11, 12, 14, 15, },
  { 2, 3, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 12, 14, 15, },
  { 4, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 1, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 2, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 3, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 12, 14, 15, },
  { 6, 7, 9, 11, 12, 14, 15, },
  { 0, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 6, 7, 9, 11, 12, 14, 15, },
  { 2, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 11, 12, 14, 15, },
  { 3, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 3, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 3, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 11, 12, 14, 15, },
  { 2, 3, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 12, 14, 15, },
  { 4, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 2, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 3, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 12, 14, 15, },
  { 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 2, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 3, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 14, 15, },
  { 8, 9, 11, 12, 14, 15, },
  { 0, 8, 9, 11, 12, 14, 15, },
  { 1, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 8, 9, 11, 12, 14, 15, },
  { 2, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 8, 9, 11, 12, 14, 15, },
  { 3, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 11, 12, 14, 15, },
  { 4, 8, 9, 11, 12, 14, 15, },
  { 0, 4, 8, 9, 11, 12, 14, 15, },
  { 1, 4, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 8, 9, 11, 12, 14, 15, },
  { 2, 4, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 11, 12, 14, 15, },
  { 3, 4, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 12, 14, 15, },
  { 5, 8, 9, 11, 12, 14, 15, },
  { 0, 5, 8, 9, 11, 12, 14, 15, },
  { 1, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 5, 8, 9, 11, 12, 14, 15, },
  { 2, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 5, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 11, 12, 14, 15, },
  { 3, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 5, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 12, 14, 15, },
  { 4, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 1, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 2, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 3, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 12, 14, 15, },
  { 6, 8, 9, 11, 12, 14, 15, },
  { 0, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 6, 8, 9, 11, 12, 14, 15, },
  { 2, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 11, 12, 14, 15, },
  { 3, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 12, 14, 15, },
  { 4, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 2, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 3, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 12, 14, 15, },
  { 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 2, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 3, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 14, 15, },
  { 7, 8, 9, 11, 12, 14, 15, },
  { 0, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 11, 12, 14, 15, },
  { 3, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 12, 14, 15, },
  { 4, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 3, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 12, 14, 15, },
  { 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 3, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 14, 15, },
  { 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 3, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, },
  { 10, 11, 12, 14, 15, },
  { 0, 10, 11, 12, 14, 15, },
  { 1, 10, 11, 12, 14, 15, },
  { 0, 1, 10, 11, 12, 14, 15, },
  { 2, 10, 11, 12, 14, 15, },
  { 0, 2, 10, 11, 12, 14, 15, },
  { 1, 2, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 10, 11, 12, 14, 15, },
  { 3, 10, 11, 12, 14, 15, },
  { 0, 3, 10, 11, 12, 14, 15, },
  { 1, 3, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 10, 11, 12, 14, 15, },
  { 2, 3, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 10, 11, 12, 14, 15, },
  { 4, 10, 11, 12, 14, 15, },
  { 0, 4, 10, 11, 12, 14, 15, },
  { 1, 4, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 10, 11, 12, 14, 15, },
  { 2, 4, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 10, 11, 12, 14, 15, },
  { 3, 4, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 10, 11, 12, 14, 15, },
  { 5, 10, 11, 12, 14, 15, },
  { 0, 5, 10, 11, 12, 14, 15, },
  { 1, 5, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 10, 11, 12, 14, 15, },
  { 2, 5, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 10, 11, 12, 14, 15, },
  { 3, 5, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 10, 11, 12, 14, 15, },
  { 4, 5, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 12, 14, 15, },
  { 6, 10, 11, 12, 14, 15, },
  { 0, 6, 10, 11, 12, 14, 15, },
  { 1, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 6, 10, 11, 12, 14, 15, },
  { 2, 6, 10, 11, 12, 14, 15, },
  { 0, 2, 6, 10, 11, 12, 14, 15, },
  { 1, 2, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 10, 11, 12, 14, 15, },
  { 3, 6, 10, 11, 12, 14, 15, },
  { 0, 3, 6, 10, 11, 12, 14, 15, },
  { 1, 3, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 10, 11, 12, 14, 15, },
  { 2, 3, 6, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 10, 11, 12, 14, 15, },
  { 4, 6, 10, 11, 12, 14, 15, },
  { 0, 4, 6, 10, 11, 12, 14, 15, },
  { 1, 4, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 10, 11, 12, 14, 15, },
  { 2, 4, 6, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 10, 11, 12, 14, 15, },
  { 3, 4, 6, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 12, 14, 15, },
  { 5, 6, 10, 11, 12, 14, 15, },
  { 0, 5, 6, 10, 11, 12, 14, 15, },
  { 1, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 10, 11, 12, 14, 15, },
  { 2, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 10, 11, 12, 14, 15, },
  { 3, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 12, 14, 15, },
  { 4, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 14, 15, },
  { 7, 10, 11, 12, 14, 15, },
  { 0, 7, 10, 11, 12, 14, 15, },
  { 1, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 7, 10, 11, 12, 14, 15, },
  { 2, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 7, 10, 11, 12, 14, 15, },
  { 3, 7, 10, 11, 12, 14, 15, },
  { 0, 3, 7, 10, 11, 12, 14, 15, },
  { 1, 3, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 7, 10, 11, 12, 14, 15, },
  { 2, 3, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 10, 11, 12, 14, 15, },
  { 4, 7, 10, 11, 12, 14, 15, },
  { 0, 4, 7, 10, 11, 12, 14, 15, },
  { 1, 4, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 7, 10, 11, 12, 14, 15, },
  { 2, 4, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 10, 11, 12, 14, 15, },
  { 3, 4, 7, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 7, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 12, 14, 15, },
  { 5, 7, 10, 11, 12, 14, 15, },
  { 0, 5, 7, 10, 11, 12, 14, 15, },
  { 1, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 7, 10, 11, 12, 14, 15, },
  { 2, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 10, 11, 12, 14, 15, },
  { 3, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 7, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 12, 14, 15, },
  { 4, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 12, 14, 15, },
  { 6, 7, 10, 11, 12, 14, 15, },
  { 0, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 6, 7, 10, 11, 12, 14, 15, },
  { 2, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 10, 11, 12, 14, 15, },
  { 3, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 3, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 3, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 10, 11, 12, 14, 15, },
  { 2, 3, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 12, 14, 15, },
  { 4, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 2, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 3, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 12, 14, 15, },
  { 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 2, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 3, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 14, 15, },
  { 8, 10, 11, 12, 14, 15, },
  { 0, 8, 10, 11, 12, 14, 15, },
  { 1, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 8, 10, 11, 12, 14, 15, },
  { 2, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 8, 10, 11, 12, 14, 15, },
  { 3, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 8, 10, 11, 12, 14, 15, },
  { 4, 8, 10, 11, 12, 14, 15, },
  { 0, 4, 8, 10, 11, 12, 14, 15, },
  { 1, 4, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 8, 10, 11, 12, 14, 15, },
  { 2, 4, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 8, 10, 11, 12, 14, 15, },
  { 3, 4, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 12, 14, 15, },
  { 5, 8, 10, 11, 12, 14, 15, },
  { 0, 5, 8, 10, 11, 12, 14, 15, },
  { 1, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 8, 10, 11, 12, 14, 15, },
  { 2, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 8, 10, 11, 12, 14, 15, },
  { 3, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 12, 14, 15, },
  { 4, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 12, 14, 15, },
  { 6, 8, 10, 11, 12, 14, 15, },
  { 0, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 6, 8, 10, 11, 12, 14, 15, },
  { 2, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 8, 10, 11, 12, 14, 15, },
  { 3, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 12, 14, 15, },
  { 4, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 2, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 3, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 12, 14, 15, },
  { 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 2, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 3, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 14, 15, },
  { 7, 8, 10, 11, 12, 14, 15, },
  { 0, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 7, 8, 10, 11, 12, 14, 15, },
  { 3, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 12, 14, 15, },
  { 4, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 3, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 12, 14, 15, },
  { 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 3, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, },
  { 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 3, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 3, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 15, },
  { 9, 10, 11, 12, 14, 15, },
  { 0, 9, 10, 11, 12, 14, 15, },
  { 1, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 9, 10, 11, 12, 14, 15, },
  { 2, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 9, 10, 11, 12, 14, 15, },
  { 3, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 9, 10, 11, 12, 14, 15, },
  { 4, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 12, 14, 15, },
  { 5, 9, 10, 11, 12, 14, 15, },
  { 0, 5, 9, 10, 11, 12, 14, 15, },
  { 1, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 9, 10, 11, 12, 14, 15, },
  { 2, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 9, 10, 11, 12, 14, 15, },
  { 3, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 12, 14, 15, },
  { 4, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, 14, 15, },
  { 6, 9, 10, 11, 12, 14, 15, },
  { 0, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 6, 9, 10, 11, 12, 14, 15, },
  { 2, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 9, 10, 11, 12, 14, 15, },
  { 3, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 12, 14, 15, },
  { 4, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 12, 14, 15, },
  { 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 2, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 3, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 14, 15, },
  { 7, 9, 10, 11, 12, 14, 15, },
  { 0, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 7, 9, 10, 11, 12, 14, 15, },
  { 3, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 12, 14, 15, },
  { 4, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 12, 14, 15, },
  { 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 3, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 14, 15, },
  { 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 3, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 3, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, },
  { 8, 9, 10, 11, 12, 14, 15, },
  { 0, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 12, 14, 15, },
  { 4, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 12, 14, 15, },
  { 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 14, 15, },
  { 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 14, 15, },
  { 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, },
  { 13, 14, 15, },
  { 0, 13, 14, 15, },
  { 1, 13, 14, 15, },
  { 0, 1, 13, 14, 15, },
  { 2, 13, 14, 15, },
  { 0, 2, 13, 14, 15, },
  { 1, 2, 13, 14, 15, },
  { 0, 1, 2, 13, 14, 15, },
  { 3, 13, 14, 15, },
  { 0, 3, 13, 14, 15, },
  { 1, 3, 13, 14, 15, },
  { 0, 1, 3, 13, 14, 15, },
  { 2, 3, 13, 14, 15, },
  { 0, 2, 3, 13, 14, 15, },
  { 1, 2, 3, 13, 14, 15, },
  { 0, 1, 2, 3, 13, 14, 15, },
  { 4, 13, 14, 15, },
  { 0, 4, 13, 14, 15, },
  { 1, 4, 13, 14, 15, },
  { 0, 1, 4, 13, 14, 15, },
  { 2, 4, 13, 14, 15, },
  { 0, 2, 4, 13, 14, 15, },
  { 1, 2, 4, 13, 14, 15, },
  { 0, 1, 2, 4, 13, 14, 15, },
  { 3, 4, 13, 14, 15, },
  { 0, 3, 4, 13, 14, 15, },
  { 1, 3, 4, 13, 14, 15, },
  { 0, 1, 3, 4, 13, 14, 15, },
  { 2, 3, 4, 13, 14, 15, },
  { 0, 2, 3, 4, 13, 14, 15, },
  { 1, 2, 3, 4, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 13, 14, 15, },
  { 5, 13, 14, 15, },
  { 0, 5, 13, 14, 15, },
  { 1, 5, 13, 14, 15, },
  { 0, 1, 5, 13, 14, 15, },
  { 2, 5, 13, 14, 15, },
  { 0, 2, 5, 13, 14, 15, },
  { 1, 2, 5, 13, 14, 15, },
  { 0, 1, 2, 5, 13, 14, 15, },
  { 3, 5, 13, 14, 15, },
  { 0, 3, 5, 13, 14, 15, },
  { 1, 3, 5, 13, 14, 15, },
  { 0, 1, 3, 5, 13, 14, 15, },
  { 2, 3, 5, 13, 14, 15, },
  { 0, 2, 3, 5, 13, 14, 15, },
  { 1, 2, 3, 5, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 13, 14, 15, },
  { 4, 5, 13, 14, 15, },
  { 0, 4, 5, 13, 14, 15, },
  { 1, 4, 5, 13, 14, 15, },
  { 0, 1, 4, 5, 13, 14, 15, },
  { 2, 4, 5, 13, 14, 15, },
  { 0, 2, 4, 5, 13, 14, 15, },
  { 1, 2, 4, 5, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 13, 14, 15, },
  { 3, 4, 5, 13, 14, 15, },
  { 0, 3, 4, 5, 13, 14, 15, },
  { 1, 3, 4, 5, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 13, 14, 15, },
  { 2, 3, 4, 5, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 13, 14, 15, },
  { 6, 13, 14, 15, },
  { 0, 6, 13, 14, 15, },
  { 1, 6, 13, 14, 15, },
  { 0, 1, 6, 13, 14, 15, },
  { 2, 6, 13, 14, 15, },
  { 0, 2, 6, 13, 14, 15, },
  { 1, 2, 6, 13, 14, 15, },
  { 0, 1, 2, 6, 13, 14, 15, },
  { 3, 6, 13, 14, 15, },
  { 0, 3, 6, 13, 14, 15, },
  { 1, 3, 6, 13, 14, 15, },
  { 0, 1, 3, 6, 13, 14, 15, },
  { 2, 3, 6, 13, 14, 15, },
  { 0, 2, 3, 6, 13, 14, 15, },
  { 1, 2, 3, 6, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 13, 14, 15, },
  { 4, 6, 13, 14, 15, },
  { 0, 4, 6, 13, 14, 15, },
  { 1, 4, 6, 13, 14, 15, },
  { 0, 1, 4, 6, 13, 14, 15, },
  { 2, 4, 6, 13, 14, 15, },
  { 0, 2, 4, 6, 13, 14, 15, },
  { 1, 2, 4, 6, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 13, 14, 15, },
  { 3, 4, 6, 13, 14, 15, },
  { 0, 3, 4, 6, 13, 14, 15, },
  { 1, 3, 4, 6, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 13, 14, 15, },
  { 2, 3, 4, 6, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 13, 14, 15, },
  { 5, 6, 13, 14, 15, },
  { 0, 5, 6, 13, 14, 15, },
  { 1, 5, 6, 13, 14, 15, },
  { 0, 1, 5, 6, 13, 14, 15, },
  { 2, 5, 6, 13, 14, 15, },
  { 0, 2, 5, 6, 13, 14, 15, },
  { 1, 2, 5, 6, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 13, 14, 15, },
  { 3, 5, 6, 13, 14, 15, },
  { 0, 3, 5, 6, 13, 14, 15, },
  { 1, 3, 5, 6, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 13, 14, 15, },
  { 2, 3, 5, 6, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 13, 14, 15, },
  { 4, 5, 6, 13, 14, 15, },
  { 0, 4, 5, 6, 13, 14, 15, },
  { 1, 4, 5, 6, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 13, 14, 15, },
  { 2, 4, 5, 6, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 13, 14, 15, },
  { 3, 4, 5, 6, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 13, 14, 15, },
  { 7, 13, 14, 15, },
  { 0, 7, 13, 14, 15, },
  { 1, 7, 13, 14, 15, },
  { 0, 1, 7, 13, 14, 15, },
  { 2, 7, 13, 14, 15, },
  { 0, 2, 7, 13, 14, 15, },
  { 1, 2, 7, 13, 14, 15, },
  { 0, 1, 2, 7, 13, 14, 15, },
  { 3, 7, 13, 14, 15, },
  { 0, 3, 7, 13, 14, 15, },
  { 1, 3, 7, 13, 14, 15, },
  { 0, 1, 3, 7, 13, 14, 15, },
  { 2, 3, 7, 13, 14, 15, },
  { 0, 2, 3, 7, 13, 14, 15, },
  { 1, 2, 3, 7, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 13, 14, 15, },
  { 4, 7, 13, 14, 15, },
  { 0, 4, 7, 13, 14, 15, },
  { 1, 4, 7, 13, 14, 15, },
  { 0, 1, 4, 7, 13, 14, 15, },
  { 2, 4, 7, 13, 14, 15, },
  { 0, 2, 4, 7, 13, 14, 15, },
  { 1, 2, 4, 7, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 13, 14, 15, },
  { 3, 4, 7, 13, 14, 15, },
  { 0, 3, 4, 7, 13, 14, 15, },
  { 1, 3, 4, 7, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 13, 14, 15, },
  { 2, 3, 4, 7, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 13, 14, 15, },
  { 5, 7, 13, 14, 15, },
  { 0, 5, 7, 13, 14, 15, },
  { 1, 5, 7, 13, 14, 15, },
  { 0, 1, 5, 7, 13, 14, 15, },
  { 2, 5, 7, 13, 14, 15, },
  { 0, 2, 5, 7, 13, 14, 15, },
  { 1, 2, 5, 7, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 13, 14, 15, },
  { 3, 5, 7, 13, 14, 15, },
  { 0, 3, 5, 7, 13, 14, 15, },
  { 1, 3, 5, 7, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 13, 14, 15, },
  { 2, 3, 5, 7, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 13, 14, 15, },
  { 4, 5, 7, 13, 14, 15, },
  { 0, 4, 5, 7, 13, 14, 15, },
  { 1, 4, 5, 7, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 13, 14, 15, },
  { 2, 4, 5, 7, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 13, 14, 15, },
  { 3, 4, 5, 7, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 13, 14, 15, },
  { 6, 7, 13, 14, 15, },
  { 0, 6, 7, 13, 14, 15, },
  { 1, 6, 7, 13, 14, 15, },
  { 0, 1, 6, 7, 13, 14, 15, },
  { 2, 6, 7, 13, 14, 15, },
  { 0, 2, 6, 7, 13, 14, 15, },
  { 1, 2, 6, 7, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 13, 14, 15, },
  { 3, 6, 7, 13, 14, 15, },
  { 0, 3, 6, 7, 13, 14, 15, },
  { 1, 3, 6, 7, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 13, 14, 15, },
  { 2, 3, 6, 7, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 13, 14, 15, },
  { 4, 6, 7, 13, 14, 15, },
  { 0, 4, 6, 7, 13, 14, 15, },
  { 1, 4, 6, 7, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 13, 14, 15, },
  { 2, 4, 6, 7, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 13, 14, 15, },
  { 3, 4, 6, 7, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 13, 14, 15, },
  { 5, 6, 7, 13, 14, 15, },
  { 0, 5, 6, 7, 13, 14, 15, },
  { 1, 5, 6, 7, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 13, 14, 15, },
  { 2, 5, 6, 7, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 13, 14, 15, },
  { 3, 5, 6, 7, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 13, 14, 15, },
  { 4, 5, 6, 7, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 13, 14, 15, },
  { 8, 13, 14, 15, },
  { 0, 8, 13, 14, 15, },
  { 1, 8, 13, 14, 15, },
  { 0, 1, 8, 13, 14, 15, },
  { 2, 8, 13, 14, 15, },
  { 0, 2, 8, 13, 14, 15, },
  { 1, 2, 8, 13, 14, 15, },
  { 0, 1, 2, 8, 13, 14, 15, },
  { 3, 8, 13, 14, 15, },
  { 0, 3, 8, 13, 14, 15, },
  { 1, 3, 8, 13, 14, 15, },
  { 0, 1, 3, 8, 13, 14, 15, },
  { 2, 3, 8, 13, 14, 15, },
  { 0, 2, 3, 8, 13, 14, 15, },
  { 1, 2, 3, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 13, 14, 15, },
  { 4, 8, 13, 14, 15, },
  { 0, 4, 8, 13, 14, 15, },
  { 1, 4, 8, 13, 14, 15, },
  { 0, 1, 4, 8, 13, 14, 15, },
  { 2, 4, 8, 13, 14, 15, },
  { 0, 2, 4, 8, 13, 14, 15, },
  { 1, 2, 4, 8, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 13, 14, 15, },
  { 3, 4, 8, 13, 14, 15, },
  { 0, 3, 4, 8, 13, 14, 15, },
  { 1, 3, 4, 8, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 13, 14, 15, },
  { 2, 3, 4, 8, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 13, 14, 15, },
  { 5, 8, 13, 14, 15, },
  { 0, 5, 8, 13, 14, 15, },
  { 1, 5, 8, 13, 14, 15, },
  { 0, 1, 5, 8, 13, 14, 15, },
  { 2, 5, 8, 13, 14, 15, },
  { 0, 2, 5, 8, 13, 14, 15, },
  { 1, 2, 5, 8, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 13, 14, 15, },
  { 3, 5, 8, 13, 14, 15, },
  { 0, 3, 5, 8, 13, 14, 15, },
  { 1, 3, 5, 8, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 13, 14, 15, },
  { 2, 3, 5, 8, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 13, 14, 15, },
  { 4, 5, 8, 13, 14, 15, },
  { 0, 4, 5, 8, 13, 14, 15, },
  { 1, 4, 5, 8, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 13, 14, 15, },
  { 2, 4, 5, 8, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 13, 14, 15, },
  { 3, 4, 5, 8, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 13, 14, 15, },
  { 6, 8, 13, 14, 15, },
  { 0, 6, 8, 13, 14, 15, },
  { 1, 6, 8, 13, 14, 15, },
  { 0, 1, 6, 8, 13, 14, 15, },
  { 2, 6, 8, 13, 14, 15, },
  { 0, 2, 6, 8, 13, 14, 15, },
  { 1, 2, 6, 8, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 13, 14, 15, },
  { 3, 6, 8, 13, 14, 15, },
  { 0, 3, 6, 8, 13, 14, 15, },
  { 1, 3, 6, 8, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 13, 14, 15, },
  { 2, 3, 6, 8, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 13, 14, 15, },
  { 4, 6, 8, 13, 14, 15, },
  { 0, 4, 6, 8, 13, 14, 15, },
  { 1, 4, 6, 8, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 13, 14, 15, },
  { 2, 4, 6, 8, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 13, 14, 15, },
  { 3, 4, 6, 8, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 13, 14, 15, },
  { 5, 6, 8, 13, 14, 15, },
  { 0, 5, 6, 8, 13, 14, 15, },
  { 1, 5, 6, 8, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 13, 14, 15, },
  { 2, 5, 6, 8, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 13, 14, 15, },
  { 3, 5, 6, 8, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 13, 14, 15, },
  { 4, 5, 6, 8, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 13, 14, 15, },
  { 7, 8, 13, 14, 15, },
  { 0, 7, 8, 13, 14, 15, },
  { 1, 7, 8, 13, 14, 15, },
  { 0, 1, 7, 8, 13, 14, 15, },
  { 2, 7, 8, 13, 14, 15, },
  { 0, 2, 7, 8, 13, 14, 15, },
  { 1, 2, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 13, 14, 15, },
  { 3, 7, 8, 13, 14, 15, },
  { 0, 3, 7, 8, 13, 14, 15, },
  { 1, 3, 7, 8, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 13, 14, 15, },
  { 2, 3, 7, 8, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 13, 14, 15, },
  { 4, 7, 8, 13, 14, 15, },
  { 0, 4, 7, 8, 13, 14, 15, },
  { 1, 4, 7, 8, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 13, 14, 15, },
  { 2, 4, 7, 8, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 13, 14, 15, },
  { 3, 4, 7, 8, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 13, 14, 15, },
  { 5, 7, 8, 13, 14, 15, },
  { 0, 5, 7, 8, 13, 14, 15, },
  { 1, 5, 7, 8, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 13, 14, 15, },
  { 2, 5, 7, 8, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 13, 14, 15, },
  { 3, 5, 7, 8, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 13, 14, 15, },
  { 4, 5, 7, 8, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 13, 14, 15, },
  { 6, 7, 8, 13, 14, 15, },
  { 0, 6, 7, 8, 13, 14, 15, },
  { 1, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 13, 14, 15, },
  { 2, 6, 7, 8, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 13, 14, 15, },
  { 3, 6, 7, 8, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 13, 14, 15, },
  { 4, 6, 7, 8, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 13, 14, 15, },
  { 5, 6, 7, 8, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 13, 14, 15, },
  { 9, 13, 14, 15, },
  { 0, 9, 13, 14, 15, },
  { 1, 9, 13, 14, 15, },
  { 0, 1, 9, 13, 14, 15, },
  { 2, 9, 13, 14, 15, },
  { 0, 2, 9, 13, 14, 15, },
  { 1, 2, 9, 13, 14, 15, },
  { 0, 1, 2, 9, 13, 14, 15, },
  { 3, 9, 13, 14, 15, },
  { 0, 3, 9, 13, 14, 15, },
  { 1, 3, 9, 13, 14, 15, },
  { 0, 1, 3, 9, 13, 14, 15, },
  { 2, 3, 9, 13, 14, 15, },
  { 0, 2, 3, 9, 13, 14, 15, },
  { 1, 2, 3, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 9, 13, 14, 15, },
  { 4, 9, 13, 14, 15, },
  { 0, 4, 9, 13, 14, 15, },
  { 1, 4, 9, 13, 14, 15, },
  { 0, 1, 4, 9, 13, 14, 15, },
  { 2, 4, 9, 13, 14, 15, },
  { 0, 2, 4, 9, 13, 14, 15, },
  { 1, 2, 4, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 9, 13, 14, 15, },
  { 3, 4, 9, 13, 14, 15, },
  { 0, 3, 4, 9, 13, 14, 15, },
  { 1, 3, 4, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 9, 13, 14, 15, },
  { 2, 3, 4, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 13, 14, 15, },
  { 5, 9, 13, 14, 15, },
  { 0, 5, 9, 13, 14, 15, },
  { 1, 5, 9, 13, 14, 15, },
  { 0, 1, 5, 9, 13, 14, 15, },
  { 2, 5, 9, 13, 14, 15, },
  { 0, 2, 5, 9, 13, 14, 15, },
  { 1, 2, 5, 9, 13, 14, 15, },
  { 0, 1, 2, 5, 9, 13, 14, 15, },
  { 3, 5, 9, 13, 14, 15, },
  { 0, 3, 5, 9, 13, 14, 15, },
  { 1, 3, 5, 9, 13, 14, 15, },
  { 0, 1, 3, 5, 9, 13, 14, 15, },
  { 2, 3, 5, 9, 13, 14, 15, },
  { 0, 2, 3, 5, 9, 13, 14, 15, },
  { 1, 2, 3, 5, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 13, 14, 15, },
  { 4, 5, 9, 13, 14, 15, },
  { 0, 4, 5, 9, 13, 14, 15, },
  { 1, 4, 5, 9, 13, 14, 15, },
  { 0, 1, 4, 5, 9, 13, 14, 15, },
  { 2, 4, 5, 9, 13, 14, 15, },
  { 0, 2, 4, 5, 9, 13, 14, 15, },
  { 1, 2, 4, 5, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 13, 14, 15, },
  { 3, 4, 5, 9, 13, 14, 15, },
  { 0, 3, 4, 5, 9, 13, 14, 15, },
  { 1, 3, 4, 5, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 13, 14, 15, },
  { 2, 3, 4, 5, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 13, 14, 15, },
  { 6, 9, 13, 14, 15, },
  { 0, 6, 9, 13, 14, 15, },
  { 1, 6, 9, 13, 14, 15, },
  { 0, 1, 6, 9, 13, 14, 15, },
  { 2, 6, 9, 13, 14, 15, },
  { 0, 2, 6, 9, 13, 14, 15, },
  { 1, 2, 6, 9, 13, 14, 15, },
  { 0, 1, 2, 6, 9, 13, 14, 15, },
  { 3, 6, 9, 13, 14, 15, },
  { 0, 3, 6, 9, 13, 14, 15, },
  { 1, 3, 6, 9, 13, 14, 15, },
  { 0, 1, 3, 6, 9, 13, 14, 15, },
  { 2, 3, 6, 9, 13, 14, 15, },
  { 0, 2, 3, 6, 9, 13, 14, 15, },
  { 1, 2, 3, 6, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 13, 14, 15, },
  { 4, 6, 9, 13, 14, 15, },
  { 0, 4, 6, 9, 13, 14, 15, },
  { 1, 4, 6, 9, 13, 14, 15, },
  { 0, 1, 4, 6, 9, 13, 14, 15, },
  { 2, 4, 6, 9, 13, 14, 15, },
  { 0, 2, 4, 6, 9, 13, 14, 15, },
  { 1, 2, 4, 6, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 13, 14, 15, },
  { 3, 4, 6, 9, 13, 14, 15, },
  { 0, 3, 4, 6, 9, 13, 14, 15, },
  { 1, 3, 4, 6, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 13, 14, 15, },
  { 2, 3, 4, 6, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 13, 14, 15, },
  { 5, 6, 9, 13, 14, 15, },
  { 0, 5, 6, 9, 13, 14, 15, },
  { 1, 5, 6, 9, 13, 14, 15, },
  { 0, 1, 5, 6, 9, 13, 14, 15, },
  { 2, 5, 6, 9, 13, 14, 15, },
  { 0, 2, 5, 6, 9, 13, 14, 15, },
  { 1, 2, 5, 6, 9, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 13, 14, 15, },
  { 3, 5, 6, 9, 13, 14, 15, },
  { 0, 3, 5, 6, 9, 13, 14, 15, },
  { 1, 3, 5, 6, 9, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 13, 14, 15, },
  { 2, 3, 5, 6, 9, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 13, 14, 15, },
  { 4, 5, 6, 9, 13, 14, 15, },
  { 0, 4, 5, 6, 9, 13, 14, 15, },
  { 1, 4, 5, 6, 9, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 13, 14, 15, },
  { 2, 4, 5, 6, 9, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 13, 14, 15, },
  { 3, 4, 5, 6, 9, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 13, 14, 15, },
  { 7, 9, 13, 14, 15, },
  { 0, 7, 9, 13, 14, 15, },
  { 1, 7, 9, 13, 14, 15, },
  { 0, 1, 7, 9, 13, 14, 15, },
  { 2, 7, 9, 13, 14, 15, },
  { 0, 2, 7, 9, 13, 14, 15, },
  { 1, 2, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 7, 9, 13, 14, 15, },
  { 3, 7, 9, 13, 14, 15, },
  { 0, 3, 7, 9, 13, 14, 15, },
  { 1, 3, 7, 9, 13, 14, 15, },
  { 0, 1, 3, 7, 9, 13, 14, 15, },
  { 2, 3, 7, 9, 13, 14, 15, },
  { 0, 2, 3, 7, 9, 13, 14, 15, },
  { 1, 2, 3, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 13, 14, 15, },
  { 4, 7, 9, 13, 14, 15, },
  { 0, 4, 7, 9, 13, 14, 15, },
  { 1, 4, 7, 9, 13, 14, 15, },
  { 0, 1, 4, 7, 9, 13, 14, 15, },
  { 2, 4, 7, 9, 13, 14, 15, },
  { 0, 2, 4, 7, 9, 13, 14, 15, },
  { 1, 2, 4, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 13, 14, 15, },
  { 3, 4, 7, 9, 13, 14, 15, },
  { 0, 3, 4, 7, 9, 13, 14, 15, },
  { 1, 3, 4, 7, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 13, 14, 15, },
  { 2, 3, 4, 7, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 13, 14, 15, },
  { 5, 7, 9, 13, 14, 15, },
  { 0, 5, 7, 9, 13, 14, 15, },
  { 1, 5, 7, 9, 13, 14, 15, },
  { 0, 1, 5, 7, 9, 13, 14, 15, },
  { 2, 5, 7, 9, 13, 14, 15, },
  { 0, 2, 5, 7, 9, 13, 14, 15, },
  { 1, 2, 5, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 13, 14, 15, },
  { 3, 5, 7, 9, 13, 14, 15, },
  { 0, 3, 5, 7, 9, 13, 14, 15, },
  { 1, 3, 5, 7, 9, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 13, 14, 15, },
  { 2, 3, 5, 7, 9, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 13, 14, 15, },
  { 4, 5, 7, 9, 13, 14, 15, },
  { 0, 4, 5, 7, 9, 13, 14, 15, },
  { 1, 4, 5, 7, 9, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 13, 14, 15, },
  { 2, 4, 5, 7, 9, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 13, 14, 15, },
  { 3, 4, 5, 7, 9, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 13, 14, 15, },
  { 6, 7, 9, 13, 14, 15, },
  { 0, 6, 7, 9, 13, 14, 15, },
  { 1, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 6, 7, 9, 13, 14, 15, },
  { 2, 6, 7, 9, 13, 14, 15, },
  { 0, 2, 6, 7, 9, 13, 14, 15, },
  { 1, 2, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 13, 14, 15, },
  { 3, 6, 7, 9, 13, 14, 15, },
  { 0, 3, 6, 7, 9, 13, 14, 15, },
  { 1, 3, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 13, 14, 15, },
  { 2, 3, 6, 7, 9, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 13, 14, 15, },
  { 4, 6, 7, 9, 13, 14, 15, },
  { 0, 4, 6, 7, 9, 13, 14, 15, },
  { 1, 4, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 13, 14, 15, },
  { 2, 4, 6, 7, 9, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 13, 14, 15, },
  { 3, 4, 6, 7, 9, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 13, 14, 15, },
  { 5, 6, 7, 9, 13, 14, 15, },
  { 0, 5, 6, 7, 9, 13, 14, 15, },
  { 1, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 13, 14, 15, },
  { 2, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 13, 14, 15, },
  { 3, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 13, 14, 15, },
  { 4, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 13, 14, 15, },
  { 8, 9, 13, 14, 15, },
  { 0, 8, 9, 13, 14, 15, },
  { 1, 8, 9, 13, 14, 15, },
  { 0, 1, 8, 9, 13, 14, 15, },
  { 2, 8, 9, 13, 14, 15, },
  { 0, 2, 8, 9, 13, 14, 15, },
  { 1, 2, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 8, 9, 13, 14, 15, },
  { 3, 8, 9, 13, 14, 15, },
  { 0, 3, 8, 9, 13, 14, 15, },
  { 1, 3, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 8, 9, 13, 14, 15, },
  { 2, 3, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 13, 14, 15, },
  { 4, 8, 9, 13, 14, 15, },
  { 0, 4, 8, 9, 13, 14, 15, },
  { 1, 4, 8, 9, 13, 14, 15, },
  { 0, 1, 4, 8, 9, 13, 14, 15, },
  { 2, 4, 8, 9, 13, 14, 15, },
  { 0, 2, 4, 8, 9, 13, 14, 15, },
  { 1, 2, 4, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 13, 14, 15, },
  { 3, 4, 8, 9, 13, 14, 15, },
  { 0, 3, 4, 8, 9, 13, 14, 15, },
  { 1, 3, 4, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 13, 14, 15, },
  { 2, 3, 4, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 13, 14, 15, },
  { 5, 8, 9, 13, 14, 15, },
  { 0, 5, 8, 9, 13, 14, 15, },
  { 1, 5, 8, 9, 13, 14, 15, },
  { 0, 1, 5, 8, 9, 13, 14, 15, },
  { 2, 5, 8, 9, 13, 14, 15, },
  { 0, 2, 5, 8, 9, 13, 14, 15, },
  { 1, 2, 5, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 13, 14, 15, },
  { 3, 5, 8, 9, 13, 14, 15, },
  { 0, 3, 5, 8, 9, 13, 14, 15, },
  { 1, 3, 5, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 13, 14, 15, },
  { 2, 3, 5, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 13, 14, 15, },
  { 4, 5, 8, 9, 13, 14, 15, },
  { 0, 4, 5, 8, 9, 13, 14, 15, },
  { 1, 4, 5, 8, 9, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 13, 14, 15, },
  { 2, 4, 5, 8, 9, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 13, 14, 15, },
  { 3, 4, 5, 8, 9, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 13, 14, 15, },
  { 6, 8, 9, 13, 14, 15, },
  { 0, 6, 8, 9, 13, 14, 15, },
  { 1, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 6, 8, 9, 13, 14, 15, },
  { 2, 6, 8, 9, 13, 14, 15, },
  { 0, 2, 6, 8, 9, 13, 14, 15, },
  { 1, 2, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 13, 14, 15, },
  { 3, 6, 8, 9, 13, 14, 15, },
  { 0, 3, 6, 8, 9, 13, 14, 15, },
  { 1, 3, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 13, 14, 15, },
  { 2, 3, 6, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 13, 14, 15, },
  { 4, 6, 8, 9, 13, 14, 15, },
  { 0, 4, 6, 8, 9, 13, 14, 15, },
  { 1, 4, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 13, 14, 15, },
  { 2, 4, 6, 8, 9, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 13, 14, 15, },
  { 3, 4, 6, 8, 9, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 13, 14, 15, },
  { 5, 6, 8, 9, 13, 14, 15, },
  { 0, 5, 6, 8, 9, 13, 14, 15, },
  { 1, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 13, 14, 15, },
  { 2, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 13, 14, 15, },
  { 3, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 13, 14, 15, },
  { 4, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 13, 14, 15, },
  { 7, 8, 9, 13, 14, 15, },
  { 0, 7, 8, 9, 13, 14, 15, },
  { 1, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 7, 8, 9, 13, 14, 15, },
  { 2, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 13, 14, 15, },
  { 3, 7, 8, 9, 13, 14, 15, },
  { 0, 3, 7, 8, 9, 13, 14, 15, },
  { 1, 3, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 13, 14, 15, },
  { 2, 3, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 13, 14, 15, },
  { 4, 7, 8, 9, 13, 14, 15, },
  { 0, 4, 7, 8, 9, 13, 14, 15, },
  { 1, 4, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 13, 14, 15, },
  { 2, 4, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 13, 14, 15, },
  { 3, 4, 7, 8, 9, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 13, 14, 15, },
  { 5, 7, 8, 9, 13, 14, 15, },
  { 0, 5, 7, 8, 9, 13, 14, 15, },
  { 1, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 13, 14, 15, },
  { 2, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 13, 14, 15, },
  { 3, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 13, 14, 15, },
  { 4, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 13, 14, 15, },
  { 6, 7, 8, 9, 13, 14, 15, },
  { 0, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 13, 14, 15, },
  { 2, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 13, 14, 15, },
  { 3, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 13, 14, 15, },
  { 4, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 13, 14, 15, },
  { 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 13, 14, 15, },
  { 10, 13, 14, 15, },
  { 0, 10, 13, 14, 15, },
  { 1, 10, 13, 14, 15, },
  { 0, 1, 10, 13, 14, 15, },
  { 2, 10, 13, 14, 15, },
  { 0, 2, 10, 13, 14, 15, },
  { 1, 2, 10, 13, 14, 15, },
  { 0, 1, 2, 10, 13, 14, 15, },
  { 3, 10, 13, 14, 15, },
  { 0, 3, 10, 13, 14, 15, },
  { 1, 3, 10, 13, 14, 15, },
  { 0, 1, 3, 10, 13, 14, 15, },
  { 2, 3, 10, 13, 14, 15, },
  { 0, 2, 3, 10, 13, 14, 15, },
  { 1, 2, 3, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 10, 13, 14, 15, },
  { 4, 10, 13, 14, 15, },
  { 0, 4, 10, 13, 14, 15, },
  { 1, 4, 10, 13, 14, 15, },
  { 0, 1, 4, 10, 13, 14, 15, },
  { 2, 4, 10, 13, 14, 15, },
  { 0, 2, 4, 10, 13, 14, 15, },
  { 1, 2, 4, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 10, 13, 14, 15, },
  { 3, 4, 10, 13, 14, 15, },
  { 0, 3, 4, 10, 13, 14, 15, },
  { 1, 3, 4, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 10, 13, 14, 15, },
  { 2, 3, 4, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 10, 13, 14, 15, },
  { 5, 10, 13, 14, 15, },
  { 0, 5, 10, 13, 14, 15, },
  { 1, 5, 10, 13, 14, 15, },
  { 0, 1, 5, 10, 13, 14, 15, },
  { 2, 5, 10, 13, 14, 15, },
  { 0, 2, 5, 10, 13, 14, 15, },
  { 1, 2, 5, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 10, 13, 14, 15, },
  { 3, 5, 10, 13, 14, 15, },
  { 0, 3, 5, 10, 13, 14, 15, },
  { 1, 3, 5, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 10, 13, 14, 15, },
  { 2, 3, 5, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 10, 13, 14, 15, },
  { 4, 5, 10, 13, 14, 15, },
  { 0, 4, 5, 10, 13, 14, 15, },
  { 1, 4, 5, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 10, 13, 14, 15, },
  { 2, 4, 5, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 10, 13, 14, 15, },
  { 3, 4, 5, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 13, 14, 15, },
  { 6, 10, 13, 14, 15, },
  { 0, 6, 10, 13, 14, 15, },
  { 1, 6, 10, 13, 14, 15, },
  { 0, 1, 6, 10, 13, 14, 15, },
  { 2, 6, 10, 13, 14, 15, },
  { 0, 2, 6, 10, 13, 14, 15, },
  { 1, 2, 6, 10, 13, 14, 15, },
  { 0, 1, 2, 6, 10, 13, 14, 15, },
  { 3, 6, 10, 13, 14, 15, },
  { 0, 3, 6, 10, 13, 14, 15, },
  { 1, 3, 6, 10, 13, 14, 15, },
  { 0, 1, 3, 6, 10, 13, 14, 15, },
  { 2, 3, 6, 10, 13, 14, 15, },
  { 0, 2, 3, 6, 10, 13, 14, 15, },
  { 1, 2, 3, 6, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 10, 13, 14, 15, },
  { 4, 6, 10, 13, 14, 15, },
  { 0, 4, 6, 10, 13, 14, 15, },
  { 1, 4, 6, 10, 13, 14, 15, },
  { 0, 1, 4, 6, 10, 13, 14, 15, },
  { 2, 4, 6, 10, 13, 14, 15, },
  { 0, 2, 4, 6, 10, 13, 14, 15, },
  { 1, 2, 4, 6, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 10, 13, 14, 15, },
  { 3, 4, 6, 10, 13, 14, 15, },
  { 0, 3, 4, 6, 10, 13, 14, 15, },
  { 1, 3, 4, 6, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 10, 13, 14, 15, },
  { 2, 3, 4, 6, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 13, 14, 15, },
  { 5, 6, 10, 13, 14, 15, },
  { 0, 5, 6, 10, 13, 14, 15, },
  { 1, 5, 6, 10, 13, 14, 15, },
  { 0, 1, 5, 6, 10, 13, 14, 15, },
  { 2, 5, 6, 10, 13, 14, 15, },
  { 0, 2, 5, 6, 10, 13, 14, 15, },
  { 1, 2, 5, 6, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 10, 13, 14, 15, },
  { 3, 5, 6, 10, 13, 14, 15, },
  { 0, 3, 5, 6, 10, 13, 14, 15, },
  { 1, 3, 5, 6, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 10, 13, 14, 15, },
  { 2, 3, 5, 6, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 13, 14, 15, },
  { 4, 5, 6, 10, 13, 14, 15, },
  { 0, 4, 5, 6, 10, 13, 14, 15, },
  { 1, 4, 5, 6, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 10, 13, 14, 15, },
  { 2, 4, 5, 6, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 13, 14, 15, },
  { 3, 4, 5, 6, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 13, 14, 15, },
  { 7, 10, 13, 14, 15, },
  { 0, 7, 10, 13, 14, 15, },
  { 1, 7, 10, 13, 14, 15, },
  { 0, 1, 7, 10, 13, 14, 15, },
  { 2, 7, 10, 13, 14, 15, },
  { 0, 2, 7, 10, 13, 14, 15, },
  { 1, 2, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 7, 10, 13, 14, 15, },
  { 3, 7, 10, 13, 14, 15, },
  { 0, 3, 7, 10, 13, 14, 15, },
  { 1, 3, 7, 10, 13, 14, 15, },
  { 0, 1, 3, 7, 10, 13, 14, 15, },
  { 2, 3, 7, 10, 13, 14, 15, },
  { 0, 2, 3, 7, 10, 13, 14, 15, },
  { 1, 2, 3, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 10, 13, 14, 15, },
  { 4, 7, 10, 13, 14, 15, },
  { 0, 4, 7, 10, 13, 14, 15, },
  { 1, 4, 7, 10, 13, 14, 15, },
  { 0, 1, 4, 7, 10, 13, 14, 15, },
  { 2, 4, 7, 10, 13, 14, 15, },
  { 0, 2, 4, 7, 10, 13, 14, 15, },
  { 1, 2, 4, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 10, 13, 14, 15, },
  { 3, 4, 7, 10, 13, 14, 15, },
  { 0, 3, 4, 7, 10, 13, 14, 15, },
  { 1, 3, 4, 7, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 10, 13, 14, 15, },
  { 2, 3, 4, 7, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 13, 14, 15, },
  { 5, 7, 10, 13, 14, 15, },
  { 0, 5, 7, 10, 13, 14, 15, },
  { 1, 5, 7, 10, 13, 14, 15, },
  { 0, 1, 5, 7, 10, 13, 14, 15, },
  { 2, 5, 7, 10, 13, 14, 15, },
  { 0, 2, 5, 7, 10, 13, 14, 15, },
  { 1, 2, 5, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 10, 13, 14, 15, },
  { 3, 5, 7, 10, 13, 14, 15, },
  { 0, 3, 5, 7, 10, 13, 14, 15, },
  { 1, 3, 5, 7, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 10, 13, 14, 15, },
  { 2, 3, 5, 7, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 13, 14, 15, },
  { 4, 5, 7, 10, 13, 14, 15, },
  { 0, 4, 5, 7, 10, 13, 14, 15, },
  { 1, 4, 5, 7, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 10, 13, 14, 15, },
  { 2, 4, 5, 7, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 13, 14, 15, },
  { 3, 4, 5, 7, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 13, 14, 15, },
  { 6, 7, 10, 13, 14, 15, },
  { 0, 6, 7, 10, 13, 14, 15, },
  { 1, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 6, 7, 10, 13, 14, 15, },
  { 2, 6, 7, 10, 13, 14, 15, },
  { 0, 2, 6, 7, 10, 13, 14, 15, },
  { 1, 2, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 10, 13, 14, 15, },
  { 3, 6, 7, 10, 13, 14, 15, },
  { 0, 3, 6, 7, 10, 13, 14, 15, },
  { 1, 3, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 10, 13, 14, 15, },
  { 2, 3, 6, 7, 10, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 10, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 13, 14, 15, },
  { 4, 6, 7, 10, 13, 14, 15, },
  { 0, 4, 6, 7, 10, 13, 14, 15, },
  { 1, 4, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 10, 13, 14, 15, },
  { 2, 4, 6, 7, 10, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 10, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 13, 14, 15, },
  { 3, 4, 6, 7, 10, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 10, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 13, 14, 15, },
  { 5, 6, 7, 10, 13, 14, 15, },
  { 0, 5, 6, 7, 10, 13, 14, 15, },
  { 1, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 10, 13, 14, 15, },
  { 2, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 10, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 13, 14, 15, },
  { 3, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 10, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 13, 14, 15, },
  { 4, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 13, 14, 15, },
  { 8, 10, 13, 14, 15, },
  { 0, 8, 10, 13, 14, 15, },
  { 1, 8, 10, 13, 14, 15, },
  { 0, 1, 8, 10, 13, 14, 15, },
  { 2, 8, 10, 13, 14, 15, },
  { 0, 2, 8, 10, 13, 14, 15, },
  { 1, 2, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 8, 10, 13, 14, 15, },
  { 3, 8, 10, 13, 14, 15, },
  { 0, 3, 8, 10, 13, 14, 15, },
  { 1, 3, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 8, 10, 13, 14, 15, },
  { 2, 3, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 10, 13, 14, 15, },
  { 4, 8, 10, 13, 14, 15, },
  { 0, 4, 8, 10, 13, 14, 15, },
  { 1, 4, 8, 10, 13, 14, 15, },
  { 0, 1, 4, 8, 10, 13, 14, 15, },
  { 2, 4, 8, 10, 13, 14, 15, },
  { 0, 2, 4, 8, 10, 13, 14, 15, },
  { 1, 2, 4, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 10, 13, 14, 15, },
  { 3, 4, 8, 10, 13, 14, 15, },
  { 0, 3, 4, 8, 10, 13, 14, 15, },
  { 1, 3, 4, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 10, 13, 14, 15, },
  { 2, 3, 4, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 13, 14, 15, },
  { 5, 8, 10, 13, 14, 15, },
  { 0, 5, 8, 10, 13, 14, 15, },
  { 1, 5, 8, 10, 13, 14, 15, },
  { 0, 1, 5, 8, 10, 13, 14, 15, },
  { 2, 5, 8, 10, 13, 14, 15, },
  { 0, 2, 5, 8, 10, 13, 14, 15, },
  { 1, 2, 5, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 10, 13, 14, 15, },
  { 3, 5, 8, 10, 13, 14, 15, },
  { 0, 3, 5, 8, 10, 13, 14, 15, },
  { 1, 3, 5, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 10, 13, 14, 15, },
  { 2, 3, 5, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 13, 14, 15, },
  { 4, 5, 8, 10, 13, 14, 15, },
  { 0, 4, 5, 8, 10, 13, 14, 15, },
  { 1, 4, 5, 8, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 10, 13, 14, 15, },
  { 2, 4, 5, 8, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 13, 14, 15, },
  { 3, 4, 5, 8, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 13, 14, 15, },
  { 6, 8, 10, 13, 14, 15, },
  { 0, 6, 8, 10, 13, 14, 15, },
  { 1, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 6, 8, 10, 13, 14, 15, },
  { 2, 6, 8, 10, 13, 14, 15, },
  { 0, 2, 6, 8, 10, 13, 14, 15, },
  { 1, 2, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 10, 13, 14, 15, },
  { 3, 6, 8, 10, 13, 14, 15, },
  { 0, 3, 6, 8, 10, 13, 14, 15, },
  { 1, 3, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 10, 13, 14, 15, },
  { 2, 3, 6, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 13, 14, 15, },
  { 4, 6, 8, 10, 13, 14, 15, },
  { 0, 4, 6, 8, 10, 13, 14, 15, },
  { 1, 4, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 10, 13, 14, 15, },
  { 2, 4, 6, 8, 10, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 10, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 13, 14, 15, },
  { 3, 4, 6, 8, 10, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 10, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 13, 14, 15, },
  { 5, 6, 8, 10, 13, 14, 15, },
  { 0, 5, 6, 8, 10, 13, 14, 15, },
  { 1, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 10, 13, 14, 15, },
  { 2, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 10, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 13, 14, 15, },
  { 3, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 10, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 13, 14, 15, },
  { 4, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 13, 14, 15, },
  { 7, 8, 10, 13, 14, 15, },
  { 0, 7, 8, 10, 13, 14, 15, },
  { 1, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 7, 8, 10, 13, 14, 15, },
  { 2, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 10, 13, 14, 15, },
  { 3, 7, 8, 10, 13, 14, 15, },
  { 0, 3, 7, 8, 10, 13, 14, 15, },
  { 1, 3, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 10, 13, 14, 15, },
  { 2, 3, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 13, 14, 15, },
  { 4, 7, 8, 10, 13, 14, 15, },
  { 0, 4, 7, 8, 10, 13, 14, 15, },
  { 1, 4, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 10, 13, 14, 15, },
  { 2, 4, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 13, 14, 15, },
  { 3, 4, 7, 8, 10, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 10, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 13, 14, 15, },
  { 5, 7, 8, 10, 13, 14, 15, },
  { 0, 5, 7, 8, 10, 13, 14, 15, },
  { 1, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 10, 13, 14, 15, },
  { 2, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 13, 14, 15, },
  { 3, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 10, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 13, 14, 15, },
  { 4, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 13, 14, 15, },
  { 6, 7, 8, 10, 13, 14, 15, },
  { 0, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 10, 13, 14, 15, },
  { 2, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 13, 14, 15, },
  { 3, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 13, 14, 15, },
  { 4, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 13, 14, 15, },
  { 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, 15, },
  { 9, 10, 13, 14, 15, },
  { 0, 9, 10, 13, 14, 15, },
  { 1, 9, 10, 13, 14, 15, },
  { 0, 1, 9, 10, 13, 14, 15, },
  { 2, 9, 10, 13, 14, 15, },
  { 0, 2, 9, 10, 13, 14, 15, },
  { 1, 2, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 9, 10, 13, 14, 15, },
  { 3, 9, 10, 13, 14, 15, },
  { 0, 3, 9, 10, 13, 14, 15, },
  { 1, 3, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 9, 10, 13, 14, 15, },
  { 2, 3, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 9, 10, 13, 14, 15, },
  { 4, 9, 10, 13, 14, 15, },
  { 0, 4, 9, 10, 13, 14, 15, },
  { 1, 4, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 9, 10, 13, 14, 15, },
  { 2, 4, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 9, 10, 13, 14, 15, },
  { 3, 4, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 13, 14, 15, },
  { 5, 9, 10, 13, 14, 15, },
  { 0, 5, 9, 10, 13, 14, 15, },
  { 1, 5, 9, 10, 13, 14, 15, },
  { 0, 1, 5, 9, 10, 13, 14, 15, },
  { 2, 5, 9, 10, 13, 14, 15, },
  { 0, 2, 5, 9, 10, 13, 14, 15, },
  { 1, 2, 5, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 9, 10, 13, 14, 15, },
  { 3, 5, 9, 10, 13, 14, 15, },
  { 0, 3, 5, 9, 10, 13, 14, 15, },
  { 1, 3, 5, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 9, 10, 13, 14, 15, },
  { 2, 3, 5, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 13, 14, 15, },
  { 4, 5, 9, 10, 13, 14, 15, },
  { 0, 4, 5, 9, 10, 13, 14, 15, },
  { 1, 4, 5, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 9, 10, 13, 14, 15, },
  { 2, 4, 5, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 13, 14, 15, },
  { 3, 4, 5, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 13, 14, 15, },
  { 6, 9, 10, 13, 14, 15, },
  { 0, 6, 9, 10, 13, 14, 15, },
  { 1, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 6, 9, 10, 13, 14, 15, },
  { 2, 6, 9, 10, 13, 14, 15, },
  { 0, 2, 6, 9, 10, 13, 14, 15, },
  { 1, 2, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 6, 9, 10, 13, 14, 15, },
  { 3, 6, 9, 10, 13, 14, 15, },
  { 0, 3, 6, 9, 10, 13, 14, 15, },
  { 1, 3, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 6, 9, 10, 13, 14, 15, },
  { 2, 3, 6, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 6, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 13, 14, 15, },
  { 4, 6, 9, 10, 13, 14, 15, },
  { 0, 4, 6, 9, 10, 13, 14, 15, },
  { 1, 4, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 6, 9, 10, 13, 14, 15, },
  { 2, 4, 6, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 6, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 13, 14, 15, },
  { 3, 4, 6, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 6, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 6, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 13, 14, 15, },
  { 5, 6, 9, 10, 13, 14, 15, },
  { 0, 5, 6, 9, 10, 13, 14, 15, },
  { 1, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 5, 6, 9, 10, 13, 14, 15, },
  { 2, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 2, 5, 6, 9, 10, 13, 14, 15, },
  { 1, 2, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 13, 14, 15, },
  { 3, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 3, 5, 6, 9, 10, 13, 14, 15, },
  { 1, 3, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 13, 14, 15, },
  { 2, 3, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 13, 14, 15, },
  { 4, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 1, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 2, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 3, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 13, 14, 15, },
  { 7, 9, 10, 13, 14, 15, },
  { 0, 7, 9, 10, 13, 14, 15, },
  { 1, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 7, 9, 10, 13, 14, 15, },
  { 2, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 7, 9, 10, 13, 14, 15, },
  { 3, 7, 9, 10, 13, 14, 15, },
  { 0, 3, 7, 9, 10, 13, 14, 15, },
  { 1, 3, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 7, 9, 10, 13, 14, 15, },
  { 2, 3, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 13, 14, 15, },
  { 4, 7, 9, 10, 13, 14, 15, },
  { 0, 4, 7, 9, 10, 13, 14, 15, },
  { 1, 4, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 7, 9, 10, 13, 14, 15, },
  { 2, 4, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 13, 14, 15, },
  { 3, 4, 7, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 7, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 13, 14, 15, },
  { 5, 7, 9, 10, 13, 14, 15, },
  { 0, 5, 7, 9, 10, 13, 14, 15, },
  { 1, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 5, 7, 9, 10, 13, 14, 15, },
  { 2, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 5, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 13, 14, 15, },
  { 3, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 3, 5, 7, 9, 10, 13, 14, 15, },
  { 1, 3, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 13, 14, 15, },
  { 2, 3, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 13, 14, 15, },
  { 4, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 1, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 2, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 3, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 13, 14, 15, },
  { 6, 7, 9, 10, 13, 14, 15, },
  { 0, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 6, 7, 9, 10, 13, 14, 15, },
  { 2, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 13, 14, 15, },
  { 3, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 3, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 3, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 13, 14, 15, },
  { 2, 3, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 13, 14, 15, },
  { 4, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 2, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 3, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 13, 14, 15, },
  { 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 2, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 3, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 13, 14, 15, },
  { 8, 9, 10, 13, 14, 15, },
  { 0, 8, 9, 10, 13, 14, 15, },
  { 1, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 8, 9, 10, 13, 14, 15, },
  { 2, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 8, 9, 10, 13, 14, 15, },
  { 3, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 13, 14, 15, },
  { 4, 8, 9, 10, 13, 14, 15, },
  { 0, 4, 8, 9, 10, 13, 14, 15, },
  { 1, 4, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 8, 9, 10, 13, 14, 15, },
  { 2, 4, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 13, 14, 15, },
  { 3, 4, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 13, 14, 15, },
  { 5, 8, 9, 10, 13, 14, 15, },
  { 0, 5, 8, 9, 10, 13, 14, 15, },
  { 1, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 5, 8, 9, 10, 13, 14, 15, },
  { 2, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 5, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 13, 14, 15, },
  { 3, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 5, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 13, 14, 15, },
  { 4, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 1, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 2, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 3, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 13, 14, 15, },
  { 6, 8, 9, 10, 13, 14, 15, },
  { 0, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 6, 8, 9, 10, 13, 14, 15, },
  { 2, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 13, 14, 15, },
  { 3, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 13, 14, 15, },
  { 4, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 2, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 3, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 13, 14, 15, },
  { 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 2, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 3, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 13, 14, 15, },
  { 7, 8, 9, 10, 13, 14, 15, },
  { 0, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 13, 14, 15, },
  { 3, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 13, 14, 15, },
  { 4, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 3, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 13, 14, 15, },
  { 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 3, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 13, 14, 15, },
  { 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 3, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 13, 14, 15, },
  { 11, 13, 14, 15, },
  { 0, 11, 13, 14, 15, },
  { 1, 11, 13, 14, 15, },
  { 0, 1, 11, 13, 14, 15, },
  { 2, 11, 13, 14, 15, },
  { 0, 2, 11, 13, 14, 15, },
  { 1, 2, 11, 13, 14, 15, },
  { 0, 1, 2, 11, 13, 14, 15, },
  { 3, 11, 13, 14, 15, },
  { 0, 3, 11, 13, 14, 15, },
  { 1, 3, 11, 13, 14, 15, },
  { 0, 1, 3, 11, 13, 14, 15, },
  { 2, 3, 11, 13, 14, 15, },
  { 0, 2, 3, 11, 13, 14, 15, },
  { 1, 2, 3, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 11, 13, 14, 15, },
  { 4, 11, 13, 14, 15, },
  { 0, 4, 11, 13, 14, 15, },
  { 1, 4, 11, 13, 14, 15, },
  { 0, 1, 4, 11, 13, 14, 15, },
  { 2, 4, 11, 13, 14, 15, },
  { 0, 2, 4, 11, 13, 14, 15, },
  { 1, 2, 4, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 11, 13, 14, 15, },
  { 3, 4, 11, 13, 14, 15, },
  { 0, 3, 4, 11, 13, 14, 15, },
  { 1, 3, 4, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 11, 13, 14, 15, },
  { 2, 3, 4, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 11, 13, 14, 15, },
  { 5, 11, 13, 14, 15, },
  { 0, 5, 11, 13, 14, 15, },
  { 1, 5, 11, 13, 14, 15, },
  { 0, 1, 5, 11, 13, 14, 15, },
  { 2, 5, 11, 13, 14, 15, },
  { 0, 2, 5, 11, 13, 14, 15, },
  { 1, 2, 5, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 11, 13, 14, 15, },
  { 3, 5, 11, 13, 14, 15, },
  { 0, 3, 5, 11, 13, 14, 15, },
  { 1, 3, 5, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 11, 13, 14, 15, },
  { 2, 3, 5, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 11, 13, 14, 15, },
  { 4, 5, 11, 13, 14, 15, },
  { 0, 4, 5, 11, 13, 14, 15, },
  { 1, 4, 5, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 11, 13, 14, 15, },
  { 2, 4, 5, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 11, 13, 14, 15, },
  { 3, 4, 5, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 11, 13, 14, 15, },
  { 6, 11, 13, 14, 15, },
  { 0, 6, 11, 13, 14, 15, },
  { 1, 6, 11, 13, 14, 15, },
  { 0, 1, 6, 11, 13, 14, 15, },
  { 2, 6, 11, 13, 14, 15, },
  { 0, 2, 6, 11, 13, 14, 15, },
  { 1, 2, 6, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 11, 13, 14, 15, },
  { 3, 6, 11, 13, 14, 15, },
  { 0, 3, 6, 11, 13, 14, 15, },
  { 1, 3, 6, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 11, 13, 14, 15, },
  { 2, 3, 6, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 11, 13, 14, 15, },
  { 4, 6, 11, 13, 14, 15, },
  { 0, 4, 6, 11, 13, 14, 15, },
  { 1, 4, 6, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 11, 13, 14, 15, },
  { 2, 4, 6, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 11, 13, 14, 15, },
  { 3, 4, 6, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 11, 13, 14, 15, },
  { 5, 6, 11, 13, 14, 15, },
  { 0, 5, 6, 11, 13, 14, 15, },
  { 1, 5, 6, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 11, 13, 14, 15, },
  { 2, 5, 6, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 11, 13, 14, 15, },
  { 3, 5, 6, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 11, 13, 14, 15, },
  { 4, 5, 6, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 13, 14, 15, },
  { 7, 11, 13, 14, 15, },
  { 0, 7, 11, 13, 14, 15, },
  { 1, 7, 11, 13, 14, 15, },
  { 0, 1, 7, 11, 13, 14, 15, },
  { 2, 7, 11, 13, 14, 15, },
  { 0, 2, 7, 11, 13, 14, 15, },
  { 1, 2, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 7, 11, 13, 14, 15, },
  { 3, 7, 11, 13, 14, 15, },
  { 0, 3, 7, 11, 13, 14, 15, },
  { 1, 3, 7, 11, 13, 14, 15, },
  { 0, 1, 3, 7, 11, 13, 14, 15, },
  { 2, 3, 7, 11, 13, 14, 15, },
  { 0, 2, 3, 7, 11, 13, 14, 15, },
  { 1, 2, 3, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 11, 13, 14, 15, },
  { 4, 7, 11, 13, 14, 15, },
  { 0, 4, 7, 11, 13, 14, 15, },
  { 1, 4, 7, 11, 13, 14, 15, },
  { 0, 1, 4, 7, 11, 13, 14, 15, },
  { 2, 4, 7, 11, 13, 14, 15, },
  { 0, 2, 4, 7, 11, 13, 14, 15, },
  { 1, 2, 4, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 11, 13, 14, 15, },
  { 3, 4, 7, 11, 13, 14, 15, },
  { 0, 3, 4, 7, 11, 13, 14, 15, },
  { 1, 3, 4, 7, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 11, 13, 14, 15, },
  { 2, 3, 4, 7, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 11, 13, 14, 15, },
  { 5, 7, 11, 13, 14, 15, },
  { 0, 5, 7, 11, 13, 14, 15, },
  { 1, 5, 7, 11, 13, 14, 15, },
  { 0, 1, 5, 7, 11, 13, 14, 15, },
  { 2, 5, 7, 11, 13, 14, 15, },
  { 0, 2, 5, 7, 11, 13, 14, 15, },
  { 1, 2, 5, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 11, 13, 14, 15, },
  { 3, 5, 7, 11, 13, 14, 15, },
  { 0, 3, 5, 7, 11, 13, 14, 15, },
  { 1, 3, 5, 7, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 11, 13, 14, 15, },
  { 2, 3, 5, 7, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 11, 13, 14, 15, },
  { 4, 5, 7, 11, 13, 14, 15, },
  { 0, 4, 5, 7, 11, 13, 14, 15, },
  { 1, 4, 5, 7, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 11, 13, 14, 15, },
  { 2, 4, 5, 7, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 11, 13, 14, 15, },
  { 3, 4, 5, 7, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 13, 14, 15, },
  { 6, 7, 11, 13, 14, 15, },
  { 0, 6, 7, 11, 13, 14, 15, },
  { 1, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 6, 7, 11, 13, 14, 15, },
  { 2, 6, 7, 11, 13, 14, 15, },
  { 0, 2, 6, 7, 11, 13, 14, 15, },
  { 1, 2, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 11, 13, 14, 15, },
  { 3, 6, 7, 11, 13, 14, 15, },
  { 0, 3, 6, 7, 11, 13, 14, 15, },
  { 1, 3, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 11, 13, 14, 15, },
  { 2, 3, 6, 7, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 11, 13, 14, 15, },
  { 4, 6, 7, 11, 13, 14, 15, },
  { 0, 4, 6, 7, 11, 13, 14, 15, },
  { 1, 4, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 11, 13, 14, 15, },
  { 2, 4, 6, 7, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 11, 13, 14, 15, },
  { 3, 4, 6, 7, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 13, 14, 15, },
  { 5, 6, 7, 11, 13, 14, 15, },
  { 0, 5, 6, 7, 11, 13, 14, 15, },
  { 1, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 11, 13, 14, 15, },
  { 2, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 11, 13, 14, 15, },
  { 3, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 13, 14, 15, },
  { 4, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 13, 14, 15, },
  { 8, 11, 13, 14, 15, },
  { 0, 8, 11, 13, 14, 15, },
  { 1, 8, 11, 13, 14, 15, },
  { 0, 1, 8, 11, 13, 14, 15, },
  { 2, 8, 11, 13, 14, 15, },
  { 0, 2, 8, 11, 13, 14, 15, },
  { 1, 2, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 8, 11, 13, 14, 15, },
  { 3, 8, 11, 13, 14, 15, },
  { 0, 3, 8, 11, 13, 14, 15, },
  { 1, 3, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 8, 11, 13, 14, 15, },
  { 2, 3, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 11, 13, 14, 15, },
  { 4, 8, 11, 13, 14, 15, },
  { 0, 4, 8, 11, 13, 14, 15, },
  { 1, 4, 8, 11, 13, 14, 15, },
  { 0, 1, 4, 8, 11, 13, 14, 15, },
  { 2, 4, 8, 11, 13, 14, 15, },
  { 0, 2, 4, 8, 11, 13, 14, 15, },
  { 1, 2, 4, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 11, 13, 14, 15, },
  { 3, 4, 8, 11, 13, 14, 15, },
  { 0, 3, 4, 8, 11, 13, 14, 15, },
  { 1, 3, 4, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 11, 13, 14, 15, },
  { 2, 3, 4, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 11, 13, 14, 15, },
  { 5, 8, 11, 13, 14, 15, },
  { 0, 5, 8, 11, 13, 14, 15, },
  { 1, 5, 8, 11, 13, 14, 15, },
  { 0, 1, 5, 8, 11, 13, 14, 15, },
  { 2, 5, 8, 11, 13, 14, 15, },
  { 0, 2, 5, 8, 11, 13, 14, 15, },
  { 1, 2, 5, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 11, 13, 14, 15, },
  { 3, 5, 8, 11, 13, 14, 15, },
  { 0, 3, 5, 8, 11, 13, 14, 15, },
  { 1, 3, 5, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 11, 13, 14, 15, },
  { 2, 3, 5, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 11, 13, 14, 15, },
  { 4, 5, 8, 11, 13, 14, 15, },
  { 0, 4, 5, 8, 11, 13, 14, 15, },
  { 1, 4, 5, 8, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 11, 13, 14, 15, },
  { 2, 4, 5, 8, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 11, 13, 14, 15, },
  { 3, 4, 5, 8, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 13, 14, 15, },
  { 6, 8, 11, 13, 14, 15, },
  { 0, 6, 8, 11, 13, 14, 15, },
  { 1, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 6, 8, 11, 13, 14, 15, },
  { 2, 6, 8, 11, 13, 14, 15, },
  { 0, 2, 6, 8, 11, 13, 14, 15, },
  { 1, 2, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 11, 13, 14, 15, },
  { 3, 6, 8, 11, 13, 14, 15, },
  { 0, 3, 6, 8, 11, 13, 14, 15, },
  { 1, 3, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 11, 13, 14, 15, },
  { 2, 3, 6, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 11, 13, 14, 15, },
  { 4, 6, 8, 11, 13, 14, 15, },
  { 0, 4, 6, 8, 11, 13, 14, 15, },
  { 1, 4, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 11, 13, 14, 15, },
  { 2, 4, 6, 8, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 11, 13, 14, 15, },
  { 3, 4, 6, 8, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 13, 14, 15, },
  { 5, 6, 8, 11, 13, 14, 15, },
  { 0, 5, 6, 8, 11, 13, 14, 15, },
  { 1, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 11, 13, 14, 15, },
  { 2, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 11, 13, 14, 15, },
  { 3, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 13, 14, 15, },
  { 4, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 13, 14, 15, },
  { 7, 8, 11, 13, 14, 15, },
  { 0, 7, 8, 11, 13, 14, 15, },
  { 1, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 7, 8, 11, 13, 14, 15, },
  { 2, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 11, 13, 14, 15, },
  { 3, 7, 8, 11, 13, 14, 15, },
  { 0, 3, 7, 8, 11, 13, 14, 15, },
  { 1, 3, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 11, 13, 14, 15, },
  { 2, 3, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 11, 13, 14, 15, },
  { 4, 7, 8, 11, 13, 14, 15, },
  { 0, 4, 7, 8, 11, 13, 14, 15, },
  { 1, 4, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 11, 13, 14, 15, },
  { 2, 4, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 11, 13, 14, 15, },
  { 3, 4, 7, 8, 11, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 11, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 11, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 13, 14, 15, },
  { 5, 7, 8, 11, 13, 14, 15, },
  { 0, 5, 7, 8, 11, 13, 14, 15, },
  { 1, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 11, 13, 14, 15, },
  { 2, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 11, 13, 14, 15, },
  { 3, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 11, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 11, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 13, 14, 15, },
  { 4, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 13, 14, 15, },
  { 6, 7, 8, 11, 13, 14, 15, },
  { 0, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 11, 13, 14, 15, },
  { 2, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 11, 13, 14, 15, },
  { 3, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 11, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 13, 14, 15, },
  { 4, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 13, 14, 15, },
  { 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, },
  { 9, 11, 13, 14, 15, },
  { 0, 9, 11, 13, 14, 15, },
  { 1, 9, 11, 13, 14, 15, },
  { 0, 1, 9, 11, 13, 14, 15, },
  { 2, 9, 11, 13, 14, 15, },
  { 0, 2, 9, 11, 13, 14, 15, },
  { 1, 2, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 9, 11, 13, 14, 15, },
  { 3, 9, 11, 13, 14, 15, },
  { 0, 3, 9, 11, 13, 14, 15, },
  { 1, 3, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 9, 11, 13, 14, 15, },
  { 2, 3, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 9, 11, 13, 14, 15, },
  { 4, 9, 11, 13, 14, 15, },
  { 0, 4, 9, 11, 13, 14, 15, },
  { 1, 4, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 9, 11, 13, 14, 15, },
  { 2, 4, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 9, 11, 13, 14, 15, },
  { 3, 4, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 11, 13, 14, 15, },
  { 5, 9, 11, 13, 14, 15, },
  { 0, 5, 9, 11, 13, 14, 15, },
  { 1, 5, 9, 11, 13, 14, 15, },
  { 0, 1, 5, 9, 11, 13, 14, 15, },
  { 2, 5, 9, 11, 13, 14, 15, },
  { 0, 2, 5, 9, 11, 13, 14, 15, },
  { 1, 2, 5, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 9, 11, 13, 14, 15, },
  { 3, 5, 9, 11, 13, 14, 15, },
  { 0, 3, 5, 9, 11, 13, 14, 15, },
  { 1, 3, 5, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 9, 11, 13, 14, 15, },
  { 2, 3, 5, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 11, 13, 14, 15, },
  { 4, 5, 9, 11, 13, 14, 15, },
  { 0, 4, 5, 9, 11, 13, 14, 15, },
  { 1, 4, 5, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 9, 11, 13, 14, 15, },
  { 2, 4, 5, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 11, 13, 14, 15, },
  { 3, 4, 5, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 13, 14, 15, },
  { 6, 9, 11, 13, 14, 15, },
  { 0, 6, 9, 11, 13, 14, 15, },
  { 1, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 6, 9, 11, 13, 14, 15, },
  { 2, 6, 9, 11, 13, 14, 15, },
  { 0, 2, 6, 9, 11, 13, 14, 15, },
  { 1, 2, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 9, 11, 13, 14, 15, },
  { 3, 6, 9, 11, 13, 14, 15, },
  { 0, 3, 6, 9, 11, 13, 14, 15, },
  { 1, 3, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 9, 11, 13, 14, 15, },
  { 2, 3, 6, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 11, 13, 14, 15, },
  { 4, 6, 9, 11, 13, 14, 15, },
  { 0, 4, 6, 9, 11, 13, 14, 15, },
  { 1, 4, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 9, 11, 13, 14, 15, },
  { 2, 4, 6, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 11, 13, 14, 15, },
  { 3, 4, 6, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 13, 14, 15, },
  { 5, 6, 9, 11, 13, 14, 15, },
  { 0, 5, 6, 9, 11, 13, 14, 15, },
  { 1, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 9, 11, 13, 14, 15, },
  { 2, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 9, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 11, 13, 14, 15, },
  { 3, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 9, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 13, 14, 15, },
  { 4, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 13, 14, 15, },
  { 7, 9, 11, 13, 14, 15, },
  { 0, 7, 9, 11, 13, 14, 15, },
  { 1, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 7, 9, 11, 13, 14, 15, },
  { 2, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 7, 9, 11, 13, 14, 15, },
  { 3, 7, 9, 11, 13, 14, 15, },
  { 0, 3, 7, 9, 11, 13, 14, 15, },
  { 1, 3, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 7, 9, 11, 13, 14, 15, },
  { 2, 3, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 11, 13, 14, 15, },
  { 4, 7, 9, 11, 13, 14, 15, },
  { 0, 4, 7, 9, 11, 13, 14, 15, },
  { 1, 4, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 7, 9, 11, 13, 14, 15, },
  { 2, 4, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 11, 13, 14, 15, },
  { 3, 4, 7, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 7, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 13, 14, 15, },
  { 5, 7, 9, 11, 13, 14, 15, },
  { 0, 5, 7, 9, 11, 13, 14, 15, },
  { 1, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 5, 7, 9, 11, 13, 14, 15, },
  { 2, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 5, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 11, 13, 14, 15, },
  { 3, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 3, 5, 7, 9, 11, 13, 14, 15, },
  { 1, 3, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 11, 13, 14, 15, },
  { 2, 3, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 13, 14, 15, },
  { 4, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 1, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 2, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 3, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 13, 14, 15, },
  { 6, 7, 9, 11, 13, 14, 15, },
  { 0, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 6, 7, 9, 11, 13, 14, 15, },
  { 2, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 11, 13, 14, 15, },
  { 3, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 3, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 3, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 11, 13, 14, 15, },
  { 2, 3, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 13, 14, 15, },
  { 4, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 2, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 3, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 13, 14, 15, },
  { 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 2, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 3, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 14, 15, },
  { 8, 9, 11, 13, 14, 15, },
  { 0, 8, 9, 11, 13, 14, 15, },
  { 1, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 8, 9, 11, 13, 14, 15, },
  { 2, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 8, 9, 11, 13, 14, 15, },
  { 3, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 11, 13, 14, 15, },
  { 4, 8, 9, 11, 13, 14, 15, },
  { 0, 4, 8, 9, 11, 13, 14, 15, },
  { 1, 4, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 8, 9, 11, 13, 14, 15, },
  { 2, 4, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 11, 13, 14, 15, },
  { 3, 4, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 13, 14, 15, },
  { 5, 8, 9, 11, 13, 14, 15, },
  { 0, 5, 8, 9, 11, 13, 14, 15, },
  { 1, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 5, 8, 9, 11, 13, 14, 15, },
  { 2, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 5, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 11, 13, 14, 15, },
  { 3, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 5, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 13, 14, 15, },
  { 4, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 1, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 2, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 3, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 13, 14, 15, },
  { 6, 8, 9, 11, 13, 14, 15, },
  { 0, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 6, 8, 9, 11, 13, 14, 15, },
  { 2, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 11, 13, 14, 15, },
  { 3, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 13, 14, 15, },
  { 4, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 2, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 3, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 13, 14, 15, },
  { 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 2, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 3, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15, },
  { 7, 8, 9, 11, 13, 14, 15, },
  { 0, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 11, 13, 14, 15, },
  { 3, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 13, 14, 15, },
  { 4, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 3, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 13, 14, 15, },
  { 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 3, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 13, 14, 15, },
  { 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 3, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 15, },
  { 10, 11, 13, 14, 15, },
  { 0, 10, 11, 13, 14, 15, },
  { 1, 10, 11, 13, 14, 15, },
  { 0, 1, 10, 11, 13, 14, 15, },
  { 2, 10, 11, 13, 14, 15, },
  { 0, 2, 10, 11, 13, 14, 15, },
  { 1, 2, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 10, 11, 13, 14, 15, },
  { 3, 10, 11, 13, 14, 15, },
  { 0, 3, 10, 11, 13, 14, 15, },
  { 1, 3, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 10, 11, 13, 14, 15, },
  { 2, 3, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 10, 11, 13, 14, 15, },
  { 4, 10, 11, 13, 14, 15, },
  { 0, 4, 10, 11, 13, 14, 15, },
  { 1, 4, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 10, 11, 13, 14, 15, },
  { 2, 4, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 10, 11, 13, 14, 15, },
  { 3, 4, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 10, 11, 13, 14, 15, },
  { 5, 10, 11, 13, 14, 15, },
  { 0, 5, 10, 11, 13, 14, 15, },
  { 1, 5, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 10, 11, 13, 14, 15, },
  { 2, 5, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 10, 11, 13, 14, 15, },
  { 3, 5, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 10, 11, 13, 14, 15, },
  { 4, 5, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 13, 14, 15, },
  { 6, 10, 11, 13, 14, 15, },
  { 0, 6, 10, 11, 13, 14, 15, },
  { 1, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 6, 10, 11, 13, 14, 15, },
  { 2, 6, 10, 11, 13, 14, 15, },
  { 0, 2, 6, 10, 11, 13, 14, 15, },
  { 1, 2, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 10, 11, 13, 14, 15, },
  { 3, 6, 10, 11, 13, 14, 15, },
  { 0, 3, 6, 10, 11, 13, 14, 15, },
  { 1, 3, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 10, 11, 13, 14, 15, },
  { 2, 3, 6, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 10, 11, 13, 14, 15, },
  { 4, 6, 10, 11, 13, 14, 15, },
  { 0, 4, 6, 10, 11, 13, 14, 15, },
  { 1, 4, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 10, 11, 13, 14, 15, },
  { 2, 4, 6, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 10, 11, 13, 14, 15, },
  { 3, 4, 6, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 13, 14, 15, },
  { 5, 6, 10, 11, 13, 14, 15, },
  { 0, 5, 6, 10, 11, 13, 14, 15, },
  { 1, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 10, 11, 13, 14, 15, },
  { 2, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 10, 11, 13, 14, 15, },
  { 3, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 13, 14, 15, },
  { 4, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 13, 14, 15, },
  { 7, 10, 11, 13, 14, 15, },
  { 0, 7, 10, 11, 13, 14, 15, },
  { 1, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 7, 10, 11, 13, 14, 15, },
  { 2, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 7, 10, 11, 13, 14, 15, },
  { 3, 7, 10, 11, 13, 14, 15, },
  { 0, 3, 7, 10, 11, 13, 14, 15, },
  { 1, 3, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 7, 10, 11, 13, 14, 15, },
  { 2, 3, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 10, 11, 13, 14, 15, },
  { 4, 7, 10, 11, 13, 14, 15, },
  { 0, 4, 7, 10, 11, 13, 14, 15, },
  { 1, 4, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 7, 10, 11, 13, 14, 15, },
  { 2, 4, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 10, 11, 13, 14, 15, },
  { 3, 4, 7, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 7, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 13, 14, 15, },
  { 5, 7, 10, 11, 13, 14, 15, },
  { 0, 5, 7, 10, 11, 13, 14, 15, },
  { 1, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 7, 10, 11, 13, 14, 15, },
  { 2, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 10, 11, 13, 14, 15, },
  { 3, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 7, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 13, 14, 15, },
  { 4, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 13, 14, 15, },
  { 6, 7, 10, 11, 13, 14, 15, },
  { 0, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 6, 7, 10, 11, 13, 14, 15, },
  { 2, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 10, 11, 13, 14, 15, },
  { 3, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 3, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 3, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 10, 11, 13, 14, 15, },
  { 2, 3, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 13, 14, 15, },
  { 4, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 2, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 3, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 13, 14, 15, },
  { 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 2, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 3, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 13, 14, 15, },
  { 8, 10, 11, 13, 14, 15, },
  { 0, 8, 10, 11, 13, 14, 15, },
  { 1, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 8, 10, 11, 13, 14, 15, },
  { 2, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 8, 10, 11, 13, 14, 15, },
  { 3, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 10, 11, 13, 14, 15, },
  { 4, 8, 10, 11, 13, 14, 15, },
  { 0, 4, 8, 10, 11, 13, 14, 15, },
  { 1, 4, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 8, 10, 11, 13, 14, 15, },
  { 2, 4, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 10, 11, 13, 14, 15, },
  { 3, 4, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 13, 14, 15, },
  { 5, 8, 10, 11, 13, 14, 15, },
  { 0, 5, 8, 10, 11, 13, 14, 15, },
  { 1, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 8, 10, 11, 13, 14, 15, },
  { 2, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 10, 11, 13, 14, 15, },
  { 3, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 13, 14, 15, },
  { 4, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 13, 14, 15, },
  { 6, 8, 10, 11, 13, 14, 15, },
  { 0, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 6, 8, 10, 11, 13, 14, 15, },
  { 2, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 10, 11, 13, 14, 15, },
  { 3, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 13, 14, 15, },
  { 4, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 2, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 3, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 13, 14, 15, },
  { 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 2, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 3, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 13, 14, 15, },
  { 7, 8, 10, 11, 13, 14, 15, },
  { 0, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 10, 11, 13, 14, 15, },
  { 3, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 13, 14, 15, },
  { 4, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 3, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 13, 14, 15, },
  { 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 3, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 13, 14, 15, },
  { 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 3, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 15, },
  { 9, 10, 11, 13, 14, 15, },
  { 0, 9, 10, 11, 13, 14, 15, },
  { 1, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 9, 10, 11, 13, 14, 15, },
  { 2, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 9, 10, 11, 13, 14, 15, },
  { 3, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 9, 10, 11, 13, 14, 15, },
  { 4, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 13, 14, 15, },
  { 5, 9, 10, 11, 13, 14, 15, },
  { 0, 5, 9, 10, 11, 13, 14, 15, },
  { 1, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 9, 10, 11, 13, 14, 15, },
  { 2, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 9, 10, 11, 13, 14, 15, },
  { 3, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 13, 14, 15, },
  { 4, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 13, 14, 15, },
  { 6, 9, 10, 11, 13, 14, 15, },
  { 0, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 6, 9, 10, 11, 13, 14, 15, },
  { 2, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 9, 10, 11, 13, 14, 15, },
  { 3, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 13, 14, 15, },
  { 4, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 13, 14, 15, },
  { 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 2, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 3, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 13, 14, 15, },
  { 7, 9, 10, 11, 13, 14, 15, },
  { 0, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 7, 9, 10, 11, 13, 14, 15, },
  { 3, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 13, 14, 15, },
  { 4, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 13, 14, 15, },
  { 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 3, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 13, 14, 15, },
  { 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 3, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 3, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 15, },
  { 8, 9, 10, 11, 13, 14, 15, },
  { 0, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 13, 14, 15, },
  { 4, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 13, 14, 15, },
  { 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 13, 14, 15, },
  { 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 13, 14, 15, },
  { 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, },
  { 12, 13, 14, 15, },
  { 0, 12, 13, 14, 15, },
  { 1, 12, 13, 14, 15, },
  { 0, 1, 12, 13, 14, 15, },
  { 2, 12, 13, 14, 15, },
  { 0, 2, 12, 13, 14, 15, },
  { 1, 2, 12, 13, 14, 15, },
  { 0, 1, 2, 12, 13, 14, 15, },
  { 3, 12, 13, 14, 15, },
  { 0, 3, 12, 13, 14, 15, },
  { 1, 3, 12, 13, 14, 15, },
  { 0, 1, 3, 12, 13, 14, 15, },
  { 2, 3, 12, 13, 14, 15, },
  { 0, 2, 3, 12, 13, 14, 15, },
  { 1, 2, 3, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 12, 13, 14, 15, },
  { 4, 12, 13, 14, 15, },
  { 0, 4, 12, 13, 14, 15, },
  { 1, 4, 12, 13, 14, 15, },
  { 0, 1, 4, 12, 13, 14, 15, },
  { 2, 4, 12, 13, 14, 15, },
  { 0, 2, 4, 12, 13, 14, 15, },
  { 1, 2, 4, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 12, 13, 14, 15, },
  { 3, 4, 12, 13, 14, 15, },
  { 0, 3, 4, 12, 13, 14, 15, },
  { 1, 3, 4, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 12, 13, 14, 15, },
  { 2, 3, 4, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 12, 13, 14, 15, },
  { 5, 12, 13, 14, 15, },
  { 0, 5, 12, 13, 14, 15, },
  { 1, 5, 12, 13, 14, 15, },
  { 0, 1, 5, 12, 13, 14, 15, },
  { 2, 5, 12, 13, 14, 15, },
  { 0, 2, 5, 12, 13, 14, 15, },
  { 1, 2, 5, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 12, 13, 14, 15, },
  { 3, 5, 12, 13, 14, 15, },
  { 0, 3, 5, 12, 13, 14, 15, },
  { 1, 3, 5, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 12, 13, 14, 15, },
  { 2, 3, 5, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 12, 13, 14, 15, },
  { 4, 5, 12, 13, 14, 15, },
  { 0, 4, 5, 12, 13, 14, 15, },
  { 1, 4, 5, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 12, 13, 14, 15, },
  { 2, 4, 5, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 12, 13, 14, 15, },
  { 3, 4, 5, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 12, 13, 14, 15, },
  { 6, 12, 13, 14, 15, },
  { 0, 6, 12, 13, 14, 15, },
  { 1, 6, 12, 13, 14, 15, },
  { 0, 1, 6, 12, 13, 14, 15, },
  { 2, 6, 12, 13, 14, 15, },
  { 0, 2, 6, 12, 13, 14, 15, },
  { 1, 2, 6, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 12, 13, 14, 15, },
  { 3, 6, 12, 13, 14, 15, },
  { 0, 3, 6, 12, 13, 14, 15, },
  { 1, 3, 6, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 12, 13, 14, 15, },
  { 2, 3, 6, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 12, 13, 14, 15, },
  { 4, 6, 12, 13, 14, 15, },
  { 0, 4, 6, 12, 13, 14, 15, },
  { 1, 4, 6, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 12, 13, 14, 15, },
  { 2, 4, 6, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 12, 13, 14, 15, },
  { 3, 4, 6, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 12, 13, 14, 15, },
  { 5, 6, 12, 13, 14, 15, },
  { 0, 5, 6, 12, 13, 14, 15, },
  { 1, 5, 6, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 12, 13, 14, 15, },
  { 2, 5, 6, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 12, 13, 14, 15, },
  { 3, 5, 6, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 12, 13, 14, 15, },
  { 4, 5, 6, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 12, 13, 14, 15, },
  { 7, 12, 13, 14, 15, },
  { 0, 7, 12, 13, 14, 15, },
  { 1, 7, 12, 13, 14, 15, },
  { 0, 1, 7, 12, 13, 14, 15, },
  { 2, 7, 12, 13, 14, 15, },
  { 0, 2, 7, 12, 13, 14, 15, },
  { 1, 2, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 12, 13, 14, 15, },
  { 3, 7, 12, 13, 14, 15, },
  { 0, 3, 7, 12, 13, 14, 15, },
  { 1, 3, 7, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 12, 13, 14, 15, },
  { 2, 3, 7, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 12, 13, 14, 15, },
  { 4, 7, 12, 13, 14, 15, },
  { 0, 4, 7, 12, 13, 14, 15, },
  { 1, 4, 7, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 12, 13, 14, 15, },
  { 2, 4, 7, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 12, 13, 14, 15, },
  { 3, 4, 7, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 12, 13, 14, 15, },
  { 5, 7, 12, 13, 14, 15, },
  { 0, 5, 7, 12, 13, 14, 15, },
  { 1, 5, 7, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 12, 13, 14, 15, },
  { 2, 5, 7, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 12, 13, 14, 15, },
  { 3, 5, 7, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 12, 13, 14, 15, },
  { 4, 5, 7, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 12, 13, 14, 15, },
  { 6, 7, 12, 13, 14, 15, },
  { 0, 6, 7, 12, 13, 14, 15, },
  { 1, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 12, 13, 14, 15, },
  { 2, 6, 7, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 12, 13, 14, 15, },
  { 3, 6, 7, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 12, 13, 14, 15, },
  { 4, 6, 7, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 12, 13, 14, 15, },
  { 5, 6, 7, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 12, 13, 14, 15, },
  { 8, 12, 13, 14, 15, },
  { 0, 8, 12, 13, 14, 15, },
  { 1, 8, 12, 13, 14, 15, },
  { 0, 1, 8, 12, 13, 14, 15, },
  { 2, 8, 12, 13, 14, 15, },
  { 0, 2, 8, 12, 13, 14, 15, },
  { 1, 2, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 8, 12, 13, 14, 15, },
  { 3, 8, 12, 13, 14, 15, },
  { 0, 3, 8, 12, 13, 14, 15, },
  { 1, 3, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 8, 12, 13, 14, 15, },
  { 2, 3, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 12, 13, 14, 15, },
  { 4, 8, 12, 13, 14, 15, },
  { 0, 4, 8, 12, 13, 14, 15, },
  { 1, 4, 8, 12, 13, 14, 15, },
  { 0, 1, 4, 8, 12, 13, 14, 15, },
  { 2, 4, 8, 12, 13, 14, 15, },
  { 0, 2, 4, 8, 12, 13, 14, 15, },
  { 1, 2, 4, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 12, 13, 14, 15, },
  { 3, 4, 8, 12, 13, 14, 15, },
  { 0, 3, 4, 8, 12, 13, 14, 15, },
  { 1, 3, 4, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 12, 13, 14, 15, },
  { 2, 3, 4, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 12, 13, 14, 15, },
  { 5, 8, 12, 13, 14, 15, },
  { 0, 5, 8, 12, 13, 14, 15, },
  { 1, 5, 8, 12, 13, 14, 15, },
  { 0, 1, 5, 8, 12, 13, 14, 15, },
  { 2, 5, 8, 12, 13, 14, 15, },
  { 0, 2, 5, 8, 12, 13, 14, 15, },
  { 1, 2, 5, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 12, 13, 14, 15, },
  { 3, 5, 8, 12, 13, 14, 15, },
  { 0, 3, 5, 8, 12, 13, 14, 15, },
  { 1, 3, 5, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 12, 13, 14, 15, },
  { 2, 3, 5, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 12, 13, 14, 15, },
  { 4, 5, 8, 12, 13, 14, 15, },
  { 0, 4, 5, 8, 12, 13, 14, 15, },
  { 1, 4, 5, 8, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 12, 13, 14, 15, },
  { 2, 4, 5, 8, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 12, 13, 14, 15, },
  { 3, 4, 5, 8, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 12, 13, 14, 15, },
  { 6, 8, 12, 13, 14, 15, },
  { 0, 6, 8, 12, 13, 14, 15, },
  { 1, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 6, 8, 12, 13, 14, 15, },
  { 2, 6, 8, 12, 13, 14, 15, },
  { 0, 2, 6, 8, 12, 13, 14, 15, },
  { 1, 2, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 12, 13, 14, 15, },
  { 3, 6, 8, 12, 13, 14, 15, },
  { 0, 3, 6, 8, 12, 13, 14, 15, },
  { 1, 3, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 12, 13, 14, 15, },
  { 2, 3, 6, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 12, 13, 14, 15, },
  { 4, 6, 8, 12, 13, 14, 15, },
  { 0, 4, 6, 8, 12, 13, 14, 15, },
  { 1, 4, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 12, 13, 14, 15, },
  { 2, 4, 6, 8, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 12, 13, 14, 15, },
  { 3, 4, 6, 8, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 12, 13, 14, 15, },
  { 5, 6, 8, 12, 13, 14, 15, },
  { 0, 5, 6, 8, 12, 13, 14, 15, },
  { 1, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 12, 13, 14, 15, },
  { 2, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 12, 13, 14, 15, },
  { 3, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 12, 13, 14, 15, },
  { 4, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 12, 13, 14, 15, },
  { 7, 8, 12, 13, 14, 15, },
  { 0, 7, 8, 12, 13, 14, 15, },
  { 1, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 7, 8, 12, 13, 14, 15, },
  { 2, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 12, 13, 14, 15, },
  { 3, 7, 8, 12, 13, 14, 15, },
  { 0, 3, 7, 8, 12, 13, 14, 15, },
  { 1, 3, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 12, 13, 14, 15, },
  { 2, 3, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 12, 13, 14, 15, },
  { 4, 7, 8, 12, 13, 14, 15, },
  { 0, 4, 7, 8, 12, 13, 14, 15, },
  { 1, 4, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 12, 13, 14, 15, },
  { 2, 4, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 12, 13, 14, 15, },
  { 3, 4, 7, 8, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 12, 13, 14, 15, },
  { 5, 7, 8, 12, 13, 14, 15, },
  { 0, 5, 7, 8, 12, 13, 14, 15, },
  { 1, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 12, 13, 14, 15, },
  { 2, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 12, 13, 14, 15, },
  { 3, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 12, 13, 14, 15, },
  { 4, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 12, 13, 14, 15, },
  { 6, 7, 8, 12, 13, 14, 15, },
  { 0, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 12, 13, 14, 15, },
  { 2, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 12, 13, 14, 15, },
  { 3, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 12, 13, 14, 15, },
  { 4, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 12, 13, 14, 15, },
  { 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, 15, },
  { 9, 12, 13, 14, 15, },
  { 0, 9, 12, 13, 14, 15, },
  { 1, 9, 12, 13, 14, 15, },
  { 0, 1, 9, 12, 13, 14, 15, },
  { 2, 9, 12, 13, 14, 15, },
  { 0, 2, 9, 12, 13, 14, 15, },
  { 1, 2, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 9, 12, 13, 14, 15, },
  { 3, 9, 12, 13, 14, 15, },
  { 0, 3, 9, 12, 13, 14, 15, },
  { 1, 3, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 9, 12, 13, 14, 15, },
  { 2, 3, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 9, 12, 13, 14, 15, },
  { 4, 9, 12, 13, 14, 15, },
  { 0, 4, 9, 12, 13, 14, 15, },
  { 1, 4, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 9, 12, 13, 14, 15, },
  { 2, 4, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 9, 12, 13, 14, 15, },
  { 3, 4, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 12, 13, 14, 15, },
  { 5, 9, 12, 13, 14, 15, },
  { 0, 5, 9, 12, 13, 14, 15, },
  { 1, 5, 9, 12, 13, 14, 15, },
  { 0, 1, 5, 9, 12, 13, 14, 15, },
  { 2, 5, 9, 12, 13, 14, 15, },
  { 0, 2, 5, 9, 12, 13, 14, 15, },
  { 1, 2, 5, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 9, 12, 13, 14, 15, },
  { 3, 5, 9, 12, 13, 14, 15, },
  { 0, 3, 5, 9, 12, 13, 14, 15, },
  { 1, 3, 5, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 9, 12, 13, 14, 15, },
  { 2, 3, 5, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 12, 13, 14, 15, },
  { 4, 5, 9, 12, 13, 14, 15, },
  { 0, 4, 5, 9, 12, 13, 14, 15, },
  { 1, 4, 5, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 9, 12, 13, 14, 15, },
  { 2, 4, 5, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 12, 13, 14, 15, },
  { 3, 4, 5, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 12, 13, 14, 15, },
  { 6, 9, 12, 13, 14, 15, },
  { 0, 6, 9, 12, 13, 14, 15, },
  { 1, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 6, 9, 12, 13, 14, 15, },
  { 2, 6, 9, 12, 13, 14, 15, },
  { 0, 2, 6, 9, 12, 13, 14, 15, },
  { 1, 2, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 9, 12, 13, 14, 15, },
  { 3, 6, 9, 12, 13, 14, 15, },
  { 0, 3, 6, 9, 12, 13, 14, 15, },
  { 1, 3, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 9, 12, 13, 14, 15, },
  { 2, 3, 6, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 12, 13, 14, 15, },
  { 4, 6, 9, 12, 13, 14, 15, },
  { 0, 4, 6, 9, 12, 13, 14, 15, },
  { 1, 4, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 9, 12, 13, 14, 15, },
  { 2, 4, 6, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 12, 13, 14, 15, },
  { 3, 4, 6, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 12, 13, 14, 15, },
  { 5, 6, 9, 12, 13, 14, 15, },
  { 0, 5, 6, 9, 12, 13, 14, 15, },
  { 1, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 9, 12, 13, 14, 15, },
  { 2, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 9, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 12, 13, 14, 15, },
  { 3, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 9, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 12, 13, 14, 15, },
  { 4, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 12, 13, 14, 15, },
  { 7, 9, 12, 13, 14, 15, },
  { 0, 7, 9, 12, 13, 14, 15, },
  { 1, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 7, 9, 12, 13, 14, 15, },
  { 2, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 9, 12, 13, 14, 15, },
  { 3, 7, 9, 12, 13, 14, 15, },
  { 0, 3, 7, 9, 12, 13, 14, 15, },
  { 1, 3, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 9, 12, 13, 14, 15, },
  { 2, 3, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 12, 13, 14, 15, },
  { 4, 7, 9, 12, 13, 14, 15, },
  { 0, 4, 7, 9, 12, 13, 14, 15, },
  { 1, 4, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 9, 12, 13, 14, 15, },
  { 2, 4, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 12, 13, 14, 15, },
  { 3, 4, 7, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 12, 13, 14, 15, },
  { 5, 7, 9, 12, 13, 14, 15, },
  { 0, 5, 7, 9, 12, 13, 14, 15, },
  { 1, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 9, 12, 13, 14, 15, },
  { 2, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 12, 13, 14, 15, },
  { 3, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 9, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 12, 13, 14, 15, },
  { 4, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 12, 13, 14, 15, },
  { 6, 7, 9, 12, 13, 14, 15, },
  { 0, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 9, 12, 13, 14, 15, },
  { 2, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 12, 13, 14, 15, },
  { 3, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 12, 13, 14, 15, },
  { 4, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 12, 13, 14, 15, },
  { 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, 15, },
  { 8, 9, 12, 13, 14, 15, },
  { 0, 8, 9, 12, 13, 14, 15, },
  { 1, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 8, 9, 12, 13, 14, 15, },
  { 2, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 8, 9, 12, 13, 14, 15, },
  { 3, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 12, 13, 14, 15, },
  { 4, 8, 9, 12, 13, 14, 15, },
  { 0, 4, 8, 9, 12, 13, 14, 15, },
  { 1, 4, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 8, 9, 12, 13, 14, 15, },
  { 2, 4, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 12, 13, 14, 15, },
  { 3, 4, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 12, 13, 14, 15, },
  { 5, 8, 9, 12, 13, 14, 15, },
  { 0, 5, 8, 9, 12, 13, 14, 15, },
  { 1, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 5, 8, 9, 12, 13, 14, 15, },
  { 2, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 5, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 12, 13, 14, 15, },
  { 3, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 5, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 12, 13, 14, 15, },
  { 4, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 1, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 2, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 3, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 12, 13, 14, 15, },
  { 6, 8, 9, 12, 13, 14, 15, },
  { 0, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 6, 8, 9, 12, 13, 14, 15, },
  { 2, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 12, 13, 14, 15, },
  { 3, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 12, 13, 14, 15, },
  { 4, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 2, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 3, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 12, 13, 14, 15, },
  { 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 2, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 3, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 12, 13, 14, 15, },
  { 7, 8, 9, 12, 13, 14, 15, },
  { 0, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 12, 13, 14, 15, },
  { 3, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 12, 13, 14, 15, },
  { 4, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 3, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 12, 13, 14, 15, },
  { 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 3, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 12, 13, 14, 15, },
  { 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 3, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 12, 13, 14, 15, },
  { 10, 12, 13, 14, 15, },
  { 0, 10, 12, 13, 14, 15, },
  { 1, 10, 12, 13, 14, 15, },
  { 0, 1, 10, 12, 13, 14, 15, },
  { 2, 10, 12, 13, 14, 15, },
  { 0, 2, 10, 12, 13, 14, 15, },
  { 1, 2, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 10, 12, 13, 14, 15, },
  { 3, 10, 12, 13, 14, 15, },
  { 0, 3, 10, 12, 13, 14, 15, },
  { 1, 3, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 10, 12, 13, 14, 15, },
  { 2, 3, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 10, 12, 13, 14, 15, },
  { 4, 10, 12, 13, 14, 15, },
  { 0, 4, 10, 12, 13, 14, 15, },
  { 1, 4, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 10, 12, 13, 14, 15, },
  { 2, 4, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 10, 12, 13, 14, 15, },
  { 3, 4, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 10, 12, 13, 14, 15, },
  { 5, 10, 12, 13, 14, 15, },
  { 0, 5, 10, 12, 13, 14, 15, },
  { 1, 5, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 10, 12, 13, 14, 15, },
  { 2, 5, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 10, 12, 13, 14, 15, },
  { 3, 5, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 10, 12, 13, 14, 15, },
  { 4, 5, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 12, 13, 14, 15, },
  { 6, 10, 12, 13, 14, 15, },
  { 0, 6, 10, 12, 13, 14, 15, },
  { 1, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 6, 10, 12, 13, 14, 15, },
  { 2, 6, 10, 12, 13, 14, 15, },
  { 0, 2, 6, 10, 12, 13, 14, 15, },
  { 1, 2, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 10, 12, 13, 14, 15, },
  { 3, 6, 10, 12, 13, 14, 15, },
  { 0, 3, 6, 10, 12, 13, 14, 15, },
  { 1, 3, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 10, 12, 13, 14, 15, },
  { 2, 3, 6, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 10, 12, 13, 14, 15, },
  { 4, 6, 10, 12, 13, 14, 15, },
  { 0, 4, 6, 10, 12, 13, 14, 15, },
  { 1, 4, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 10, 12, 13, 14, 15, },
  { 2, 4, 6, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 10, 12, 13, 14, 15, },
  { 3, 4, 6, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 12, 13, 14, 15, },
  { 5, 6, 10, 12, 13, 14, 15, },
  { 0, 5, 6, 10, 12, 13, 14, 15, },
  { 1, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 10, 12, 13, 14, 15, },
  { 2, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 10, 12, 13, 14, 15, },
  { 3, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 12, 13, 14, 15, },
  { 4, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 12, 13, 14, 15, },
  { 7, 10, 12, 13, 14, 15, },
  { 0, 7, 10, 12, 13, 14, 15, },
  { 1, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 7, 10, 12, 13, 14, 15, },
  { 2, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 10, 12, 13, 14, 15, },
  { 3, 7, 10, 12, 13, 14, 15, },
  { 0, 3, 7, 10, 12, 13, 14, 15, },
  { 1, 3, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 10, 12, 13, 14, 15, },
  { 2, 3, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 10, 12, 13, 14, 15, },
  { 4, 7, 10, 12, 13, 14, 15, },
  { 0, 4, 7, 10, 12, 13, 14, 15, },
  { 1, 4, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 10, 12, 13, 14, 15, },
  { 2, 4, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 10, 12, 13, 14, 15, },
  { 3, 4, 7, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 12, 13, 14, 15, },
  { 5, 7, 10, 12, 13, 14, 15, },
  { 0, 5, 7, 10, 12, 13, 14, 15, },
  { 1, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 10, 12, 13, 14, 15, },
  { 2, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 10, 12, 13, 14, 15, },
  { 3, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 12, 13, 14, 15, },
  { 4, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 12, 13, 14, 15, },
  { 6, 7, 10, 12, 13, 14, 15, },
  { 0, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 10, 12, 13, 14, 15, },
  { 2, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 10, 12, 13, 14, 15, },
  { 3, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 10, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 12, 13, 14, 15, },
  { 4, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 12, 13, 14, 15, },
  { 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 12, 13, 14, 15, },
  { 8, 10, 12, 13, 14, 15, },
  { 0, 8, 10, 12, 13, 14, 15, },
  { 1, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 8, 10, 12, 13, 14, 15, },
  { 2, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 8, 10, 12, 13, 14, 15, },
  { 3, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 10, 12, 13, 14, 15, },
  { 4, 8, 10, 12, 13, 14, 15, },
  { 0, 4, 8, 10, 12, 13, 14, 15, },
  { 1, 4, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 8, 10, 12, 13, 14, 15, },
  { 2, 4, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 10, 12, 13, 14, 15, },
  { 3, 4, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 12, 13, 14, 15, },
  { 5, 8, 10, 12, 13, 14, 15, },
  { 0, 5, 8, 10, 12, 13, 14, 15, },
  { 1, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 8, 10, 12, 13, 14, 15, },
  { 2, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 10, 12, 13, 14, 15, },
  { 3, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 12, 13, 14, 15, },
  { 4, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 12, 13, 14, 15, },
  { 6, 8, 10, 12, 13, 14, 15, },
  { 0, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 6, 8, 10, 12, 13, 14, 15, },
  { 2, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 10, 12, 13, 14, 15, },
  { 3, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 12, 13, 14, 15, },
  { 4, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 2, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 3, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 12, 13, 14, 15, },
  { 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 2, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 3, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 12, 13, 14, 15, },
  { 7, 8, 10, 12, 13, 14, 15, },
  { 0, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 10, 12, 13, 14, 15, },
  { 3, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 12, 13, 14, 15, },
  { 4, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 3, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 12, 13, 14, 15, },
  { 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 3, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 13, 14, 15, },
  { 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 3, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 14, 15, },
  { 9, 10, 12, 13, 14, 15, },
  { 0, 9, 10, 12, 13, 14, 15, },
  { 1, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 9, 10, 12, 13, 14, 15, },
  { 2, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 9, 10, 12, 13, 14, 15, },
  { 3, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 9, 10, 12, 13, 14, 15, },
  { 4, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 12, 13, 14, 15, },
  { 5, 9, 10, 12, 13, 14, 15, },
  { 0, 5, 9, 10, 12, 13, 14, 15, },
  { 1, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 9, 10, 12, 13, 14, 15, },
  { 2, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 9, 10, 12, 13, 14, 15, },
  { 3, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 12, 13, 14, 15, },
  { 4, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 12, 13, 14, 15, },
  { 6, 9, 10, 12, 13, 14, 15, },
  { 0, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 6, 9, 10, 12, 13, 14, 15, },
  { 2, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 9, 10, 12, 13, 14, 15, },
  { 3, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 12, 13, 14, 15, },
  { 4, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 12, 13, 14, 15, },
  { 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 2, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 3, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 12, 13, 14, 15, },
  { 7, 9, 10, 12, 13, 14, 15, },
  { 0, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 9, 10, 12, 13, 14, 15, },
  { 3, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 12, 13, 14, 15, },
  { 4, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 12, 13, 14, 15, },
  { 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 3, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 12, 13, 14, 15, },
  { 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 3, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14, 15, },
  { 8, 9, 10, 12, 13, 14, 15, },
  { 0, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 12, 13, 14, 15, },
  { 4, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 12, 13, 14, 15, },
  { 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 12, 13, 14, 15, },
  { 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 14, 15, },
  { 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, },
  { 11, 12, 13, 14, 15, },
  { 0, 11, 12, 13, 14, 15, },
  { 1, 11, 12, 13, 14, 15, },
  { 0, 1, 11, 12, 13, 14, 15, },
  { 2, 11, 12, 13, 14, 15, },
  { 0, 2, 11, 12, 13, 14, 15, },
  { 1, 2, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 11, 12, 13, 14, 15, },
  { 3, 11, 12, 13, 14, 15, },
  { 0, 3, 11, 12, 13, 14, 15, },
  { 1, 3, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 11, 12, 13, 14, 15, },
  { 2, 3, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 11, 12, 13, 14, 15, },
  { 4, 11, 12, 13, 14, 15, },
  { 0, 4, 11, 12, 13, 14, 15, },
  { 1, 4, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 11, 12, 13, 14, 15, },
  { 2, 4, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 11, 12, 13, 14, 15, },
  { 3, 4, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 11, 12, 13, 14, 15, },
  { 5, 11, 12, 13, 14, 15, },
  { 0, 5, 11, 12, 13, 14, 15, },
  { 1, 5, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 11, 12, 13, 14, 15, },
  { 2, 5, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 11, 12, 13, 14, 15, },
  { 3, 5, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 11, 12, 13, 14, 15, },
  { 4, 5, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 11, 12, 13, 14, 15, },
  { 6, 11, 12, 13, 14, 15, },
  { 0, 6, 11, 12, 13, 14, 15, },
  { 1, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 11, 12, 13, 14, 15, },
  { 2, 6, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 11, 12, 13, 14, 15, },
  { 3, 6, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 11, 12, 13, 14, 15, },
  { 4, 6, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 11, 12, 13, 14, 15, },
  { 5, 6, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 11, 12, 13, 14, 15, },
  { 7, 11, 12, 13, 14, 15, },
  { 0, 7, 11, 12, 13, 14, 15, },
  { 1, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 7, 11, 12, 13, 14, 15, },
  { 2, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 11, 12, 13, 14, 15, },
  { 3, 7, 11, 12, 13, 14, 15, },
  { 0, 3, 7, 11, 12, 13, 14, 15, },
  { 1, 3, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 11, 12, 13, 14, 15, },
  { 2, 3, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 11, 12, 13, 14, 15, },
  { 4, 7, 11, 12, 13, 14, 15, },
  { 0, 4, 7, 11, 12, 13, 14, 15, },
  { 1, 4, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 11, 12, 13, 14, 15, },
  { 2, 4, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 11, 12, 13, 14, 15, },
  { 3, 4, 7, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 11, 12, 13, 14, 15, },
  { 5, 7, 11, 12, 13, 14, 15, },
  { 0, 5, 7, 11, 12, 13, 14, 15, },
  { 1, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 11, 12, 13, 14, 15, },
  { 2, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 11, 12, 13, 14, 15, },
  { 3, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 11, 12, 13, 14, 15, },
  { 4, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 11, 12, 13, 14, 15, },
  { 6, 7, 11, 12, 13, 14, 15, },
  { 0, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 11, 12, 13, 14, 15, },
  { 2, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 11, 12, 13, 14, 15, },
  { 3, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 11, 12, 13, 14, 15, },
  { 4, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 11, 12, 13, 14, 15, },
  { 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 11, 12, 13, 14, 15, },
  { 8, 11, 12, 13, 14, 15, },
  { 0, 8, 11, 12, 13, 14, 15, },
  { 1, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 8, 11, 12, 13, 14, 15, },
  { 2, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 8, 11, 12, 13, 14, 15, },
  { 3, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 11, 12, 13, 14, 15, },
  { 4, 8, 11, 12, 13, 14, 15, },
  { 0, 4, 8, 11, 12, 13, 14, 15, },
  { 1, 4, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 8, 11, 12, 13, 14, 15, },
  { 2, 4, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 11, 12, 13, 14, 15, },
  { 3, 4, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 11, 12, 13, 14, 15, },
  { 5, 8, 11, 12, 13, 14, 15, },
  { 0, 5, 8, 11, 12, 13, 14, 15, },
  { 1, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 8, 11, 12, 13, 14, 15, },
  { 2, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 11, 12, 13, 14, 15, },
  { 3, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 11, 12, 13, 14, 15, },
  { 4, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 11, 12, 13, 14, 15, },
  { 6, 8, 11, 12, 13, 14, 15, },
  { 0, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 8, 11, 12, 13, 14, 15, },
  { 2, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 11, 12, 13, 14, 15, },
  { 3, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 11, 12, 13, 14, 15, },
  { 4, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 11, 12, 13, 14, 15, },
  { 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 11, 12, 13, 14, 15, },
  { 7, 8, 11, 12, 13, 14, 15, },
  { 0, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 11, 12, 13, 14, 15, },
  { 3, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 11, 12, 13, 14, 15, },
  { 4, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 3, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 11, 12, 13, 14, 15, },
  { 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 3, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 11, 12, 13, 14, 15, },
  { 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 3, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 12, 13, 14, 15, },
  { 9, 11, 12, 13, 14, 15, },
  { 0, 9, 11, 12, 13, 14, 15, },
  { 1, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 9, 11, 12, 13, 14, 15, },
  { 2, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 9, 11, 12, 13, 14, 15, },
  { 3, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 9, 11, 12, 13, 14, 15, },
  { 4, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 11, 12, 13, 14, 15, },
  { 5, 9, 11, 12, 13, 14, 15, },
  { 0, 5, 9, 11, 12, 13, 14, 15, },
  { 1, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 9, 11, 12, 13, 14, 15, },
  { 2, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 9, 11, 12, 13, 14, 15, },
  { 3, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 11, 12, 13, 14, 15, },
  { 4, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 11, 12, 13, 14, 15, },
  { 6, 9, 11, 12, 13, 14, 15, },
  { 0, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 9, 11, 12, 13, 14, 15, },
  { 2, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 9, 11, 12, 13, 14, 15, },
  { 3, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 11, 12, 13, 14, 15, },
  { 4, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 11, 12, 13, 14, 15, },
  { 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 11, 12, 13, 14, 15, },
  { 7, 9, 11, 12, 13, 14, 15, },
  { 0, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 9, 11, 12, 13, 14, 15, },
  { 3, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 11, 12, 13, 14, 15, },
  { 4, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 11, 12, 13, 14, 15, },
  { 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 3, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 11, 12, 13, 14, 15, },
  { 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 3, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 12, 13, 14, 15, },
  { 8, 9, 11, 12, 13, 14, 15, },
  { 0, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 11, 12, 13, 14, 15, },
  { 4, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 11, 12, 13, 14, 15, },
  { 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 11, 12, 13, 14, 15, },
  { 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 11, 12, 13, 14, 15, },
  { 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, },
  { 10, 11, 12, 13, 14, 15, },
  { 0, 10, 11, 12, 13, 14, 15, },
  { 1, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 10, 11, 12, 13, 14, 15, },
  { 2, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 10, 11, 12, 13, 14, 15, },
  { 3, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 10, 11, 12, 13, 14, 15, },
  { 4, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 10, 11, 12, 13, 14, 15, },
  { 5, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 10, 11, 12, 13, 14, 15, },
  { 6, 10, 11, 12, 13, 14, 15, },
  { 0, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 10, 11, 12, 13, 14, 15, },
  { 2, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 10, 11, 12, 13, 14, 15, },
  { 3, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 10, 11, 12, 13, 14, 15, },
  { 4, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 10, 11, 12, 13, 14, 15, },
  { 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, },
  { 7, 10, 11, 12, 13, 14, 15, },
  { 0, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 10, 11, 12, 13, 14, 15, },
  { 3, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 10, 11, 12, 13, 14, 15, },
  { 4, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 10, 11, 12, 13, 14, 15, },
  { 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 10, 11, 12, 13, 14, 15, },
  { 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 3, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, },
  { 8, 10, 11, 12, 13, 14, 15, },
  { 0, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 10, 11, 12, 13, 14, 15, },
  { 4, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 10, 11, 12, 13, 14, 15, },
  { 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 10, 11, 12, 13, 14, 15, },
  { 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 10, 11, 12, 13, 14, 15, },
  { 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, },
  { 9, 10, 11, 12, 13, 14, 15, },
  { 0, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 9, 10, 11, 12, 13, 14, 15, },
  { 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, 14, 15, },
  { 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, },
  { 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15, },
  { 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
  { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, },
};

const size_t BITS_SET_16_IDX[65536] = {0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 
    3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
    3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
    4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 1, 2, 2, 3, 2, 
    3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
    3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
    4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 1, 2, 2, 3, 2, 
    3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 1, 2, 2, 3, 2, 3, 3, 4, 2, 
    3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 
    5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
    13, 13, 14, 13, 14, 14, 15, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 
    4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 2, 3, 3, 4, 3, 
    4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 2, 
    3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 
    5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
    13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 2, 3, 3, 4, 3, 4, 4, 5, 3, 
    4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 3, 4, 4, 5, 4, 
    5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 3, 
    4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 
    6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
    14, 14, 15, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 4, 5, 5, 6, 5, 
    6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 
    7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 4, 5, 5, 6, 5, 6, 6, 7, 5, 
    6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 5, 
    6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 4, 
    5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 5, 6, 6, 7, 6, 
    7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 5, 6, 6, 7, 6, 7, 7, 8, 6, 
    7, 7, 8, 7, 8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 8, 8, 9, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 
    9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
    13, 13, 14, 13, 14, 14, 15, 5, 6, 6, 7, 6, 7, 7, 8, 6, 7, 7, 8, 7, 
    8, 8, 9, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 6, 7, 7, 8, 7, 
    8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 6, 7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 6, 
    7, 7, 8, 7, 8, 8, 9, 7, 8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 
    9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
    13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 6, 7, 7, 8, 7, 8, 8, 9, 7, 
    8, 8, 9, 8, 9, 9, 10, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 7, 8, 8, 9, 8, 
    9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 7, 
    8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 
    10, 10, 11, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
    14, 14, 15, 7, 8, 8, 9, 8, 9, 9, 10, 8, 9, 9, 10, 9, 10, 10, 11, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 8, 9, 9, 10, 9, 
    10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 8, 9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 
    11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 
    10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 8, 9, 9, 10, 9, 10, 10, 11, 9, 
    10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 8, 
    9, 9, 10, 9, 10, 10, 11, 9, 10, 10, 11, 10, 11, 11, 12, 9, 10, 10, 11, 10, 
    11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 9, 10, 10, 11, 10, 11, 11, 12, 10, 
    11, 11, 12, 11, 12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 
    13, 13, 14, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12, 12, 13, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 11, 11, 12, 11, 
    12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 
    13, 13, 14, 13, 14, 14, 15, 9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 
    12, 12, 13, 10, 11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 10, 
    11, 11, 12, 11, 12, 12, 13, 11, 12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 
    13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 10, 11, 11, 12, 11, 12, 12, 13, 11, 
    12, 12, 13, 12, 13, 13, 14, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 
    14, 14, 15, 11, 12, 12, 13, 12, 13, 13, 14, 12, 13, 13, 14, 13, 14, 14, 15, 12, 
    13, 13, 14, 13, 14, 14, 15, 13, 14, 14, 15, 14, 15, 15, };
